;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                       ;;
;;;                Centre for Speech Technology Research                  ;;
;;;                     University of Edinburgh, UK                       ;;
;;;                         Copyright (c) 1998                            ;;
;;;                        All Rights Reserved.                           ;;
;;;                                                                       ;;
;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
;;;  this software and its documentation without restriction, including   ;;
;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
;;;  permit persons to whom this work is furnished to do so, subject to   ;;
;;;  the following conditions:                                            ;;
;;;   1. The code must retain the above copyright notice, this list of    ;;
;;;      conditions and the following disclaimer.                         ;;
;;;   2. Any modifications must be clearly marked as such.                ;;
;;;   3. Original authors' names are not deleted.                         ;;
;;;   4. The authors' names are not used to endorse or promote products   ;;
;;;      derived from this software without specific prior written        ;;
;;;      permission.                                                      ;;
;;;                                                                       ;;
;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
;;;  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ;;
;;;  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE     ;;
;;;  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ;;
;;;  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ;;
;;;  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ;;
;;;  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ;;
;;;  THIS SOFTWARE.                                                       ;;
;;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  
;;;  Extra items that have sufficient frequency and are pronounced
;;;  wrongly that are to be added to the compiled version of 
;;;  the CMULEX lexicon
;;;
;;;  This list we derived from various text databases and actually
;;;  listening to the LTS models

("dont" v (d ow1 n t))
("doesnt" v (d ah1 z n t))
("youre" nil (y ao1 r))
("wasnt" v (w aa1 z n t))
("wouldnt" v (w uh1 d n t))
("couldnt" v (k uh1 d n t))
("shouldnt" v (sh uh1 d n t))
("youve" nil (y uw1 v))
("werent" nil (w er1 n t))
("Lieut" nil (l uw t eh1 n ax n t))
("hed" v (hh iy1 d))
("youd" v (y uw1 d))
("Ayatullah" n (ay1 ax t uh1 l ax))
("intifadeh" n (ih1 n t ax f aa1 d ey))
("Gaddafi" n (g ax d ae1 f iy))
("Muammar" n (m uw ae1 m m aa1 r) )
("prizewinning" n (p r ay1 z w ih1 n ax ng) )
("Yevgeni" n (y eh v g eh1 n iy) )
("HIV" n (ey1 ch ay1 v iy) )
("Lemonick" b (l eh m ah1 n ax k) )
("sures" n (sh uw1 r z) )
("putsch" n (p uh1 t ch) )
("oclock" n (ax k l aa1 k) )
("Peterzell" n (p iy1 t er z eh l) )
("yds" n (y aa1 r d z) )
("mujahedin" n (m uw1 hh ax hh eh d iy n) )
("Zedong" n (z ey1 d ao ng) )
("Ungeheuer" n (ah n g ax hh uw er) )
("antiapartheid" n (ae1 n t iy ax p aa1 r t hh ay t ) )
("auteur" n (ow t uw1 r) )
("surefire" n (sh uw r f ay1 er) )
("moviegoer" n (m uw1 v iy g ow er) )
("OHara" n (ow hh aa1 r ax) )
("NIH" n (eh1 n ay ey1 ch) )
("Sotheby" n (s ao1 th ax b iy) )
("Dak" n (d aa1 k) )
("itll" n (ih1 t ax l) )
("Kryuchkov" n (k r uw1 ch k ax v) )
("FDIC" n (eh1 f d ax k) )
("Caltech" n (k ae1 l t eh1 k) )
("VIP" n (v iy1 ay p iy1) )
("whove" n (hh uw1 v) )
("Seiichi" n (s ey1 iy ch iy) )
("Kanise" n (k ae1 n iy z) )
("Gup" n (g uh1 p) )
("prizewinner" n (p r ay1 z w ih1 n er) )
("genocidal" n (jh eh1 n ax s ay1 d ax l) )
("eponymous" n (ax p aa1 n ax m ax s) )
("Tallinn" n (t aa1 l iy n) )
("Lothar" n (l ao th aa1 r) )
("Kampuchea" n (k ae m p ax ch iy1 ax) )
("Bessmertnykh" n (b eh s m eh1 r t n ax k) )
("yd" n (y aa1 r d) )
("atavistic" n (ae1 t ax v ih1 s t ax k ) )
("Tadzhikistan" n (t ae1 d zh ax k ax s t aa1 n) )
("sadomasochistic" n (s ey1 d ax m ae s ax k ih1 s t ax k) )
("impudent" n (ih1 m p y ax d ax n t) )
("antinuclear" n (ae1 n t ax n uw1 k l iy er) )
("Komsomolskaya" n (k ax m s aa1 m ax l s k ay1 ax ) )
("proj" n (p r aa1 jh eh k t) )
("ROTC" n (aa1 r ow t iy s iy1) )
("Jamil" n (jh ax m iy1 l) )
("Agha" n (aa1 g ax) )
("skewered" n (s k y uw1 er d) )
("celibate" n (s eh1 l ax b ax t) )
("Bubiyan" n (b uw1 b ax y aa n) )
("Bakatin" n (b ae1 k ax t ax n) )
("whod" n (hh uw1 d) )
("reverent" n (r eh1 v er ax n t) )
("occupiers" n (aa1 k y ax p ay1 r z) )
("meanspirited" n (m iy1 n s p ih1 r ax t ax d) )
("bluesy" n (b l uw1 z iy) )
("aboveground" n (ax b ah1 v g r aw n d) )
("Yanayev" n (y ae1 n ay eh1 v) )
("VOL" n (v ao1 l) )
("Teutonic" n (t uw t aa1 n ax k) )
("Serrill" n (s eh1 r ax l) )
("Branegan" n (b r ae1 n ax g ax n) )
("Bjerklie" n (b eh1 r k l iy) )
("unerring" n (uh n eh1 r ax ng) )
("sugarcane" n (sh uh1 g er k ey n) )
("simpleminded" n (s ih1 m p ax l m ay1 n d ax d) )
("menaced" n (m eh1 n ax s d) )
("kidnapers" n (k ih1 d n ae p er z) )
("detonator" n (d eh1 t ax n ey1 t er) )
("dachas" n (d aa1 ch ax s) )
("cheerily" n (ch iy1 r ax l iy) )
("chanteuse" n (sh ae1 n t ow z) )
("acidly" n (ae1 s ax d l iy) )
("Kirghizia" n (k ax r g ih1 t s iy ax) )
("Isozaki" n (iy1 s ax z aa1 k iy) )
("Ilyich" n (iy1 l y ax ch) )
("Eurocentric" n (y uw1 r ax s eh1 n t r ax k) )
("worshiping" n (w er1 sh ax p ax ng) )
("villainy" n (v ih1 l ax n iy) )
("souffle" n (s uw f l ey1) )
("redolent" n (r eh1 d ax l ax n t) )
("probusiness" n (p r ow b ih1 z n ax s) )
("postgraduate" n (p ow1 s t g r ae1 jh ax w ax t) )
("obstinate" n (aa1 b s t ax n ax t) )
("hallucination" n (hh ae l uw1 s ax n ey sh ax n) )
("goddam" n (g aa1 d ae m) )
("exorcism" n (eh1 k s ax r s ih1 z ax m ) )
("daringly" n (d ey1 r ax ng l iy) )
("backstabbing" n (b ae1 k s t ae1 b ax ng) )
("Wenceslas" n (w eh1 n s ax s l ax s) )
("Uzis" n (uw1 z iy z) )
("Trabant" n (t r ax b aa1 n t) )
("Sinead" n (sh ih1 n ey d) )
("Nazarbayev" n (n ae1 z er b ay1 ax v) )
("Nadia" n (n aa1 d iy ax) )
("unquenchable" n (uh n k w eh1 n sh ax b ax l) )
("typecast" n (t ay1 p k ae1 s t) )
("therell" n (dh eh1 r ax l) )
("syrupy" n (s ih1 r ax p iy) )
("superagent" n (s uw1 p er ey1 jh eh n t) )
("sourly" n (s aw1 er l iy) )
("shuddered" n (sh ah1 d er ax d) )
("seceding" n (s eh s iy1 d ax ng) )
("preternatural" n (p r iy t er n ae1 t ch er ax l) )
("postseason" n (p ow1 s t s iy1 s ax n) )
("policymaker" n (p aa1 l ax s iy m ey1 k er) )
("nosy" n (n ow1 z iy) )
("maleness" n (m ey1 l n ax s) )
("luscious" n (l ah1 sh ax s) )
("gigolo" n (jh ih1 g ax l ow) )
("genially" n (jh iy1 n iy ax l iy) )
("epiphanies" n (eh p ih1 f ae n iy z) )
("dungeon" n (d ah1 n jh ax n) )
("dogmas" n (d ao1 g m ax s) )
("discomfited" n (d ax s k aa1 m f ax t ax d) )
("deadlier" n (d eh1 d l iy er) )
("autoworkers" n (ao1 t ow w er k er z) )
("Yanqui" n (y ae1 n k iy) )
("Wehrmacht" n (v er m aa1 k t) )
("Trabants" n (t r ax b aa1 n t z) )
("Swahili" n (s w ax hh iy1 l iy) )
("Sodom" n (s aa1 d ax m) )
("Seurat" n (s uw1 r ax t) )
("NAACP" n (eh1 n d ah1 b ax l ey1 s iy p iy1) )
("Manichaean" n (m ae1 n ax k iy ax n) )
("Khaled" n (k aa1 l eh d) )
("Farouk" n (f aa r uw1 k) )
("Bendjedid" n (b eh1 n jh ax d ax d) )
("Beetlejuice" n (b iy1 t ax l jh uw s) )
("whir" n (w ih1 r) )
("tragicomedy" n (t r ae1 jh ax k aa1 m ax d iy) )
("touchingly" n (t ah1 ch ax ng l iy) )
("thered" n (dh eh1 r ax d) )
("tangy" n (t ae1 ng iy) )
("synonyms" n (s ih1 n ax n ax m z) )
("subspecies" n (s ah1 b s p iy sh iy z) )
("subjugated" n (s ax b jh ax g ey1 t ax d) )
("stunner" n (s t ah1 n er) )
("springy" n (s p r ih1 ng iy) )
("sorrows" n (s aa1 r ow z) )
("sequins" n (s iy1 k w ax n z) )
("schoolgirl" n (s k uw1 l g er l) )
("schoolchild" n (s k uw1 l ch ay l d) )
("scavenging" n (s k ae1 v ax n jh ax ng) )
("rampaging" n (r ae1 m p ey jh ax ng ) )
("radiance" n (r ey1 d y ax n s) )
("pussycat" n (p uh1 s iy k ae t) )
("paparazzi" n (p ae1 p er ae t s iy) )
("ovaries" n (ow1 v er iy z) )
("languorous" n (l ae1 ng ao r ax s) )
("incorruptible" n (ih1 n k er ah1 p t ax b ax l) )
("hypnotically" n (hh ih1 p n aa1 t ax k ax l iy) )
("grouchy" n (g r aw1 ch iy) )
("genitalia" n (jh eh1 n ax t ey1 l y ax) )
("fearlessly" n (f iy1 r l ax s l iy) )
("divinely" n (d ih1 v ay n l iy) )
("bluefish" n (b l uw1 f ax sh) )
("bayonets" n (b ey1 y ax n eh t s) )
("autoworker" n (ao1 t ow w er k er) )
("antihero" n (ae1 n t ax hh iy1 r ow) )
("Yel" n (y eh1 l) )
("Talmudic" n (t ae1 l m ax d ax k) )
("Staci" n (s t aa1 z iy) )
("Sese" n (s eh1 s iy1 ) )
("Senegalese" n (s ax n ax g ao l iy1 z) )
("Quezon" n (k w eh1 z ax n) )
("Pesci" n (p eh1 s k iy) )
("Mengele" n (m eh1 n g ax l ey) )
("Lagerfeld" n (l ae1 g er f eh l d) )
("Kerouac" n (k eh1 r ow ae k) )
("Kamisar" n (k ae1 m ax s aa r) )
("Janata" n (jh ax n ae1 t ax) )
("Hequet" n (hh eh1 k ax t) )
("Friedan" n (f r iy1 d ax n) )
("Dubrovnik" n (d ax b r ah1 v n ax k) )
("Candide" n (k ae n d iy1 d) )
("Bubba" n (b ah1 b ax) )
("AWOL" n (ey1 w ao l) )
("wouldve" n (w uh1 d ax v) )
("wisecrack" n (w ay1 z k r ae1 k) )
("voce" n (v ao1 k ax) )
("uproarious" n (ax p r ao1 r iy ax s) )
("unearthing" n (ax n er1 th ax ng) )
("undoes" n (ax n d ah1 z) )
("underappreciated" n (ah1 n d er ax p r iy1 sh iy ey1 t ax d) )
("tousled" n (t ao1 s ax l d) )
("thatll" n (dh ae t ax l) )
("teasingly" n (t iy1 z ax ng l iy) )
("strafed" n (s t r ey1 f d) )
("stateless" n (s t ey1 t l ax s) )
("stagings" n (s t ey1 jh ax ng z) )
("reveres" n (r eh1 v ax r iy z) )
("retells" n (r iy t eh1 l z) )
("provocateur" n (p r ax v ao k ax t uh1 r) )
("prenuptial" n (p r iy n ah1 p sh ax l) )
("pinup" n (p ih1 n ah1 p) )
("paterfamilias" n (p ae1 t er f ax m iy1 l iy ax z) )
("paroxysm" n (p ax r aa1 k s ax z ax m) )
("parodying" n (p ae1 r ax d iy ax ng) )
("merriment" n (m eh1 r ax m ax n t) )
("menstruation" n (m ax n s t r ey1 sh ax n) )
("indiscretion" n (ih1 n d ax s k r eh1 sh ax n) )
("guesthouse" n (g eh1 s t hh aw s) )
("fratricidal" n (f r ae t r ax s ay1 d ax l) )
("fiendishly" n (f iy1 n d ax sh l iy) )
("enthuses" n (eh n th uw1 z ax z) )
("encapsulates" n (eh n k ae1 p s ax l ey1 t s) )
("discomfiting" n (d ax s k aa1 m f ax t ax ng) )
("cul" n (k uh1 l) )
("cheekbones" n (ch iy1 k b ow n z) )
("bizarrely" n (b ax z aa1 r l iy) )
("bestiality" n (b iy1 s t iy ae1 l ax t iy) )
("audiotapes" n (aa1 d iy ow t ey1 p s) )
("acolyte" n (ae1 k ax l ay t) )
("achingly" n (ey1 k ax ng l iy) )
("Wayans" n (w ey1 y ax n z) )
("Urquart" n (ae1 r k er t) )
("Ural" n (y uw1 r ax l) )
("Sindayen" n (s ih1 n d ax y ax n) )
("Seuss" n (s uw1 s) )
("Plexiglas" n (p l eh1 k s iy g l ae s) )
("Overbye" n (ow1 v er b ay) )
("Nostradamus" n (n aa1 s t r ax d ae1 m ax s) )
("Navajos" n (n ae1 v ax hh ow z) )
("Massoud" n (m ax s uw1 d) )
("Kales" n (k ey1 l z) )
("Gio" n (jh iy1 ow) )
("Gianfranco" n (jh iy1 ax n f r ae n k ow) )
("Gaultier" n (g ao1 l t iy ey) )
("Foucault" n (f ow1 k ow l t) )
("FSLIC" n (eh1 f s l ax k) )
("Disneyesque" n (d ih1 z n ey eh1 s k) )
("Corleone" n (k aa1 r l iy ow1 n iy) )
("Arnaz" n (aa1 r n ax z) )
("Almash" n (ae1 l m ax sh) )
("yous" n (y uw1 z) )
("whooshing" n (w uw1 sh ax ng) )
("volunteerism" n (v ao1 l ax n t iy1 r ax z ax m) )
("upended" n (ah1 p eh1 n d ax d) )
("unacknowledged" n (ah1 n ax k n aa1 l ax jh d) )
("thrillingly" n (th r ih1 l ax g l iy) )
("tackiness" n (t ae1 k ax n ax s) )
("supernal" n (s uw p eh1 r n ax l) )
("superheroes" n (s uw1 p er hh iy1 r ow z) )
("superego" n (s uw1 p er iy1 g ow) )
("straitlaced" n (s t r ey1 t l ey s d) )
("stateroom" n (s t ey1 t r uw m) )
("sorcerer" n (s ao1 r s er er ) )
("skewering" n (s k y uw1 er ax ng) )
("semiprivate" n (s eh1 m ay p r ay1 v ax t) )
("saviors" n (s ey1 v y er z) )
("roguish" n (r ow1 g ax sh) )
("rereleased" n (r iy1 r ax l iy1 s t) )
("repressing" n (r ax p r eh1 s ax ng) )
("puppeteers" n (p ah1 p ax t iy1 r z) )
("provocateurs" n (p r ax v ao k ax t uh1 r z) )
("prodemocracy" n (p r ow1 d ax m aa1 k r ax s iy) )
("prizewinners" n (p r ay1 z w ih1 n er z) )
("presummit" n (p r iy1 s ah1 m ax t) )
("pregame" n (p r iy1 g ey1 m) )
("preadolescent" n (p r iy1 ae1 d ax l eh s ax n t) )
("postindustrial" n (p ow1 s t ax n d ah1 s t r y ax l) )
("playgoers" n (p l ey1 g ow1 er z) )
("pastiches" n (p ax s t iy1 ch ax z) )
("paramour" n (p ae1 r ax m ao r) )
("outdoorsman" n (aw1 t d ao r z m ax n) )
("ordeals" n (ao r d iy1 l z) )
("nuisances" n (n uw1 s ax n s ax z) )
("nonmusical" n (n aa1 n m y uh1 z ax k ax l) )
("noncombatant" n (n aa1 n k aa1 m b ax t ax n t) )
("neurosurgery" n (n y uw1 r ax s er1 jh er iy) )
("narcotraficantes" n (n ae1 r k ax t r ae1 f ax k ae1 n t iy z) )
("mischievously" n (m ax s ch iy1 v y ax s l iy) )
("menaces" n (m eh1 n ax s ax z) )
("manifestos" n (m ae1 n ax f eh1 s t ow s) )
("magnanimity" n (m ae1 g n ax n ih1 m ax t iy) )
("limply" n (l ih1 m p l iy) )
("knifing" n (n ay1 f ax ng) )
("inveighed" n (ax n v ey1 d) )
("inelegant" n (ax n eh1 l ax g ax n t) )
("highflyers" n (hh ay1 f l ay1 er z) )
("groped" n (g r ow1 p d) )
("filmgoers" n (f ih1 l m g ow1 er z) )
("eunuchs" n (y uh1 n ax k s) )
("ennobling" n (eh n ow1 b ax l ax ng) )
("doyenne" n (d ao y eh1 n) )
("docudramas" n (d ao k y ax d r ae1 m ax z) )
("detours" n (d iy t uh1 r z) )
("desecrated" n (d eh1 s ax k r ey1 t ax d) )
("decoded" n (d ax k ow1 d ax d) )
("dOrsay" n (d ao1 r s iy) )
("cynosure" n (s ay1 n ax sh uh r) )
("curmudgeonly" n (k er m uh1 d jh ax n l iy) )
("cmon" n (k ax m aa1 n) )
("cellblock" n (s eh1 l b l aa k) )
("carousing" n (k ax r aw1 z ax ng))
("blitzing" n (b l ih1 t z ax ng) )
("bejeweled" n (b ax jh uw1 ax l d) )
("bedevils" n (b ax d eh1 v ax l z) )
("bareknuckle" n (b eh1 r n ah1 k ax l) )
("awestruck" n (ao1 s t r ah k) )
("automatons" n (ao t aa1 m ax t ax n z) )
("anteroom" n (ae1 n t ax r uw m) )
("anchorwoman" n (ae1 ng k aa r w uh1 m ax n) )
("Yahweh" n (y ae1 w ey) )
("Vartoogian" n (v er t ow1 g iy ax n) )
("Uttar" n (uw1 t er) )
("Tsufura" n (t s uw f uw1 r ax) )
("Sousuke" n (s ow s uh1 k ey) )
("Segelken" n (s eh1 g ax l k ax n) )
("Scissorhands" n (s ih1 s er hh ae1 n d z) )
("Presbyterians" n (p r eh s b ax t iy1 r iy ax n z) )
("Pharaoh" n (f ey1 r ow) )
("Nachtwey" n (n ae1 k t w ey) )
("Medecins" n (m eh d ax s ae1 n z) )
("Manhattanites" n (m ae n hh ae1 t ax n ay1 t s) )
("Madama" n (m ax d ae1 m ax) )
("Ljubljana" n (l uh1 b l y ae1 n ae) )
("Knute" n (k ax n uw1 t) )
("Hangin" n (hh ae ng ax n) )
("Halaki" n (hh ax l ae1 k iy) )
("Gordievsky" n (g ao r d ax eh1 v s k iy) )
("Geremek" n (g eh1 r ax m eh k) )
("Ehud" n (ey1 hh uw1 d) )
("Dieu" n (d y uw1) )
("Dachau" n (d ae1 k aw) )
("Chaim" n (ch ey1 m) )
("Bnai" n (b ax n ay1) )
("Antichrist" n (ae1 n t iy k r ay1 s t) )
("Afrocentric" n (ae1 f r ax s eh1 n t r ax k) )
("wombs" n (w uw1 m z) )
("windblown" n (w ih1 n d b l ow n) )
("vibrates" v (v ay1 b r ey t s) )
("vertebrae" n (v er1 t ax b r ey) )
("underpants" n (ah1 n d er p ae1 n t s) )
("tuxedoes" n (t ax k s iy1 d ow z) )
("tumbleweed" n (t ah1 m b ax l w iy d) )
("tryin" n (t r ay1 ax n) )
("torturous" n (t ao1 r t y ax s) )
("superrich" n (s uw1 p er r ih1 ch) )
("subliminally" n (s ax b l ih1 m ax n ax l iy) )
("subjugate" n (s ax b jh ax g ey1 t) )
("stupefying" n (s t uw1 p ax f ay ax ng) )
("stomachache" n (s t ah1 m ax k ey1 k) )
("sophistry" n (s ow1 f ax s t r iy) )
("sociopath" n (s ow1 sh y ow p ae1 th) )
("snugly" n (s n ah1 g l iy) )
("serenades" n (s eh1 r ax n ey1 d z) )
("sepulchral" n (s ax p uh1 l k r ax l) )
("satirize" n (s ae1 t ax r ay z) )
("satiny" n (s ae1 t ax n y ) )
("runaways" n (r ah1 n ax w ey z) )
("rivulets" n (r ih1 v ax l ax t s) )
("ricocheting" n (r ih1 k ax sh ey ax ng) )
("reimagine" n (r iy ax m ae1 jh ax n) )
("refusenik" n (r ax f y uw1 z n ax k) )
("ravings" n (r ey1 v ax ng z) )
("radicchio" n (r ax d ih1 k iy ow) )
("preseason" n (p r iy1 s iy1 s ax n) )
("premed" n (p r iy1 m eh1 d) )
("preinvasion" n (p r iy1 ax n v ey1 sh ax n) )
("precancerous" n (p r iy k ae1 n s er ax s) )
("postmenopausal" n (p ow1 s t m eh n ax p ao1 s ax l) )
("pinups" n (p ih1 n ah1 p s) )
("passerby" n (p ae1 s er b ay) )
("parroted" n (p ae1 r ax t ax d) )
("paranormal" n (p ae1 r ax n ao1 r m ax l) )
("owlish" n (aw1 l ax sh) )
("overhyped" n (ow1 v er hh ay1 p t) )
("noire" n (n ax w ae1 r) )
("motorboats" n (m ow1 t er b ow t s) )
("miscreant" n (m ih1 s k r iy ax n t) )
("megalomaniacal" n (m eh1 g l ax m ey1 n iy ax k ax l) )
("marquees" n (m aa1 r k iy z) )
("lapels" n (l ax p eh1 l z) )
("kowtowing" n (k aw1 t aw ax ng) )
("jeremiads" n (jh eh1 r ax m ay1 ax d z) )
("interweaves" n (ih1 n t er w iy1 v z) )
("insuperable" n (ax n s uw1 p er ax b ax l) )
("insanely" n (ax n s ey1 n l iy) )
("inflames" n (ax n f l ey1 m z) )
("infest" n (ax n f eh1 s t) )
("inebriated" n (ax n iy1 b r ax ey1 t ax d) )
("incompletely" n (ih1 n k ax m p l iy1 t l iy) )
("impersonate" n (ax m p eh1 r s ax n ey t) )
("hysterectomies" n (hh ih1 s t er eh k t ax m iy z) )
("hypothalamus" n (hh ay1 p ax th ae1 l ax m ax s) )
("hucksterism" n (hh ah1 k s t er ax z ax m) )
("hotheads" n (hh aa1 t hh eh1 d z) )
("hotheaded" n (hh aa1 t hh eh1 d ax d) )
("heavenward" n (hh eh1 v ax n w er d) )
("hauteur" n (hh ow t uh1 r) )
("gunplay" n (g ah1 n p l ey ) )
("griddle" n (g r ih1 d ax l) )
("gewgaws" n (g iy1 g ao z) )
("gaiety" n (g ey1 ax t iy) )
("fuchsia" n (f y uw1 sh ax) )
("fixers" n (f ih1 k s  er z) )
("fidgety" n (f ih1 d jh ax t iy) )
("farewells" n (f eh1 r w eh1 l z) )
("exoticism" n (eh g z aa1 t ax k ax z ax m) )
("enshrines" n (ax n sh r ay1 n z) )
("dyslexia" n (d ax s l eh1 k s y ax) )
("dynamiting" n (d ay1 n ax m ay t ax ng) )
("drumbeats" n (d r ah1 m b iy t s) )
("dopers" n (d ow1 p er z) )
("doin" n (d uw1 ax n) )
("distantly" n (d ih1 s t ax n t l iy) )
("dissolute" n (d ih1 s ax l uw t) )
("detests" n (d ax t eh1 s t s) )
("despots" n (d eh1 s p ax t s) )
("denizen" n (d eh1 n ax z ax n) )
("decaf" n (d iy1 k ae1 f) )
("dIsere" n (d iy s eh1 r iy) )
("clanged" n (k l ae1 ng d ) )
("bewitching" n (b ax w ih1 ch ax ng) )
("barest" n (b eh1 r ax s t ) )
("banjoist" n (b ae1 n jh ow ax s t) )
("balsamic" n (b ax l s ae1 m ax k) )
("auteurs" n (ow t uw1 r z) )
("aswarm" n (ae z w er1 m) )
("arrivistes" n (ae1 r ax v iy1 s t ax z) )
("apparat" n (ae1 p ax r ae t) )
("amorality" n (ey1 m er ae1 l ax t iy) )

;;;  So what about UK words ?
("cheque" n (ch eh1 k) )
("cheques" n (ch eh1 k s) )
("chequing" n (ch eh1 k ax ng) )
("honour" n (aa1 n er) )
("honouring" n (aa1 n er ax ng) )
("honours" n (aa1 n er z) )
("honourable" n (aa1 n er ax b ax l) )
("honoured" n (aa1 n er d) )
("favour" n (f ey1 v er) )
("favours" n (f ey1 v er z) )
("favoured" n (f ey1 v er d) )
("favourite" n (f ey1 v er ih t) )
("favourites" n (f ey1 v er ih t s) )
("favouring" n (f ey1 v er ax ng) )
("favourable" n (f ey1 v er ax b ax l) )
("favourably" n (f ey1 v er ax b l iy) )
("labour" n (l ey1 b er) )
("labours" n (l ey1 b er z) )
("laboured" n (l ey1 b er d) )
("labouring" n (l ey1 b er ax ng) )
("colour" n (k ah1 l er) )
("colouring" n (k ah1 l er ax ng) )
("colours" n (k ah1 l er z) )
("colourful" n (k ah1 l er f ax l) )
("colourfully" n (k ah1 l er f ax l iy) )

;; Added July 2004 
("tanja" n (t ae1 n y ah))
("waibel" n (w ay1 b ax l))
("eskenazi" n (eh s k ax n aa1 z iy))
("carbonell" nil (k aa1 r b ow n eh1 l))
("metze" nil (m eh1 t z ax))
("toth" nil (t ow1 th))
("adele" nil (ax d eh1 l))
("weitz" nil (w ay1 t s))
("shamos" nil (sh ey1 m ao s))
("ziad" nil (z iy1 ax d))
("cmu" nil (s iy1 eh1 m y uw1))
("hua" nil (w aa1))
("kornel" nil (k ao r n eh1 l))
("lavie" nil (l ax v iy1))
("raux" nil (r ow1))
("maeda" nil (m ay1 d ax))
("portune" nil (p ao r t uw1 n))
("stefanie" nil (s t eh1 f ae n iy))
("woszczyna" nil (w aa zh iy1 n ax0))
("waveform" nil (w ey1 v f ao1 r m))
("waveforms" nil (w ey1 v f ao1 r m z))
("dataset" nil (d ey1 t ah0 s eh1 t))
("datasets" nil (d ey1 t ah0 s eh1 t s))

;; Added July 2008
("hermione" n (hh er m ay1 ax n iy))
("tidied" v (t ay1 d ax d))
("tidies" v (t ay1 d ax z))
("untidy" v (ax n t ay1 d iy))
("tidying" v (t ay1 d iy ih ng))
("putting" v (p uh1 t ih ng))
("iphone" n (ay1 f ow n))
("ipod" n (ay1 p aa d))
("untie" v (ax n t ay1))
("kilohertz" n (k ih1 l ax hh er1 t s))
("linux" n (l ih1 n ah k s))
("miso" n (m iy1 s ow))
("gaijin" n (g ay1 jh ih n))
("uk" n (y uw1 k ey1))
("katie" n (k ey1 t iy))
("waistcoat" n (w ey1 s t k ow t))
("tarpaulin" n (t aa r p ao1 l ih n))
("trestle" n (t r eh1 s ax l))
("trestles" n (t r eh1 s ax l z))
("clunits" n (s iy1 eh1 l y uw1 n ih t s))
("clustergen" n (k l ah1 s t er jh eh1 n))
("libravox" n (l iy1 b r ax v aa1 k s))
("peeped" n (p iy1 p t))
("goodbyes" n (g uh1 d b ay1 z))
("spearmint" j (s p iy1 r m ih0 n t))
("surfed" v (s er1 f t))
("unpacked" v (ax0 n p ae1 k t))
("cookstove" v (k uh1 k s t ow1 v))
("woodstove" v (w uh1 d s t ow1 v))
("watertight" j (w ao1 t er0 t ay0 t))
("youtube" n (y uw1 t uw1 b))
("schlep" v (sh l eh1 p))
("drawknife" n (d r ao1 n ay0 f))
("wifi" n (w ay1 f ay1))
("waterproof" j (w ao1 t er0 p r uw0 f))
("understair" j (ah1 n d er0 s t ey1 r))
("wikipedia" n (w ih0 k ih0 p iy1 d iy0 ax0))
("smelt" v (s m eh1 l t))


