;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                                                                       ;;
;;;                Centre for Speech Technology Research                  ;;
;;;                     University of Edinburgh, UK                       ;;
;;;                       Copyright (c) 1996,1997                         ;;
;;;                        All Rights Reserved.                           ;;
;;;                                                                       ;;
;;;  Permission is hereby granted, free of charge, to use and distribute  ;;
;;;  this software and its documentation without restriction, including   ;;
;;;  without limitation the rights to use, copy, modify, merge, publish,  ;;
;;;  distribute, sublicense, and/or sell copies of this work, and to      ;;
;;;  permit persons to whom this work is furnished to do so, subject to   ;;
;;;  the following conditions:                                            ;;
;;;   1. The code must retain the above copyright notice, this list of    ;;
;;;      conditions and the following disclaimer.                         ;;
;;;   2. Any modifications must be clearly marked as such.                ;;
;;;   3. Original authors' names are not deleted.                         ;;
;;;   4. The authors' names are not used to endorse or promote products   ;;
;;;      derived from this software without specific prior written        ;;
;;;      permission.                                                      ;;
;;;                                                                       ;;
;;;  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ;;
;;;  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ;;
;;;  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   ;;
;;;  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE     ;;
;;;  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    ;;
;;;  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   ;;
;;;  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          ;;
;;;  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       ;;
;;;  THIS SOFTWARE.                                                       ;;
;;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;   Phone duration info for Klatt rules, for mrpa phone set

(set! duration_klatt_params
'(
(a	230.0	80.0)
(aa	240.0	100.0)
(@	120.0	60.0)
(@@	180.0	80.0)
(ai	250.0	150.0)
(au	240.0	100.0)
(b	85.0	60.0)
(ch	70.0	50.0)
(d	75.0	50.0)
(dh	50.0	30.0)
(e	150.0	70.0)
(e@	270.0	130.0)
(ei	180.0	100.0)
(f	100.0	80.0)
(g	80.0	60.0)
(h	80.0	20.0)
(i	135.0	40.0)
(i@	230.0	100.0)
(ii	155.0	55)
(jh	70.0	50.0)
(k	80.0	60.0)
(l	80.0	40.0)
(m	70.0	60.0)
(n	60.0	50.0)
(ng	95.0	60.0)
(o	240.0	130.0)
(oi	280.0	150.0)
(oo	240.0	130.0)
(ou	220.0	80.0)
(p	90.0	50.0)
(r	80.0	30.0)
(s	105.0	60.0)
(sh	105.0	80.0)
(t	75.0	50.0)
(th	90.0	60.0)
(u	210.0	70.0)
(u@	230.0	110.0)
(uh	160.0	60.0)
(uu	230.0	150.0)
(v	60.0	40.0)
(w	80.0	60.0)
(y	80.0	40.0)
(z	75.0	40.0)
(zh	70.0	40.0)
(#	100.0	100.0)
))

(provide 'klatt_durs)
