
;; CMUDICT-0.4 Converted to Festival lexicon format
;; ## date:  11-8-95
;; ##
;; ## the carnegie mellon pronouncing dictionary [cmudict.0.4] is copyright 1995
;; ## by carnegie mellon university. use of this dictionary, for any research or
;; ## commercial purpose, is completely unrestricted.  if you make use of or
;; ## redistribute this material, we would appreciate acknowlegement of its
;; ## origin.
;; ##
;; ## cmudict.0.4 is the fourth release of cmudict, first released as cmudict.0.1
;; ## in september of 1993.
;; ##
;; ## see the readme in this directory before you use this dictionary.
;; ##
;; ## i would like to thank bill huggins at bbn; bill fisher at nist; alex
;; ## hauptmann, alex rudnicky, jack mostow, lin chase, roni rosenfeld, and
;; ## matthew siegler at cmu; doug paul at mit lincoln labs; ben serridge at
;; ## mit sls; murray spiegel at bellcore; tony robinson at cambridge uk;
;; ## david bowness of cae electronics ltd. and crim; stephen hocking; jerry
;; ## quinn at bnr canada, and marshal midden for bringing to my attention
;; ## problems and inadequacies with the first three releases. i hope i have
;; ## addressed most of the problems they uncovered.
;; ##
;; ## i welcome input from users and will continue to acknowledge such input
;; ## in subsequent releases. if i failed to acknowledge your input in this
;; ## release, please remind me and i will update these comments. if i failed to
;; ## fix things that you brought to my attention, please remind me and have
;; ## patience. if i actually fixed things that you brought to my attention and
;; ## you appreciate it, i wouldn't mind a pat on the back.
;; ##
;; ## this version differs from previous releases of cmudict most significantly
;; ## in the addition of new words from the common arpa tasks for 1995.
;; ##
;; ## there are undoubtedly still errors and inconsistencies in this dictionary
;; ## so keep your eyes open for problems and mail them to me.
;; ##
;; ## i hope this dictionary is an improvement over cmudict.0.3.
;; ## 
;; ## bob weide (weide@cs.cmu.edu)
;; ##
("a" dt (ax))
("a" n (ey1))
("aaa" nil (t r ih1 p ax l ey1))
("aaberg" nil (aa1 b er0 g))
("aachen" nil (aa1 k ax n))
("aaker" nil (aa1 k er0))
("aalseth" nil (aa1 l s eh0 th))
("aamodt" nil (aa1 m ax t))
("aancor" nil (aa1 n k ao1 r))
("aardema" nil (aa0 r d eh1 m ax))
("aardvark" nil (aa1 r d v aa1 r k))
("aaron" nil (eh1 r ax n))
("aarons" nil (eh1 r ax n z))
("aaronson" nil (aa1 r ax n s ax n))
("aase" nil (aa1 s))
("aasen" nil (aa1 s ax n))
("ababa" nil (ax b aa1 b ax))
("abacha" nil (ae1 b ax k ax))
("aback" nil (ax b ae1 k))
("abaco" nil (ae1 b ax k ow1))
("abacus" nil (ae1 b ax k ax s))
("abad" nil (ax b aa1 d))
("abadie" nil (ax b ae1 d iy0))
("abair" nil (ax b eh1 r))
("abalkin" nil (ax b aa1 l k ih0 n))
("abalone" nil (ae1 b ax l ow1 n iy0))
("abalos" nil (aa0 b aa1 l ow0 z))
("abandon" nil (ax b ae1 n d ax n))
("abandoned" nil (ax b ae1 n d ax n d))
("abandoning" nil (ax b ae1 n d ax n ih0 ng))
("abandonment" nil (ax b ae1 n d ax n m ax n t))
("abandonments" nil (ax b ae1 n d ax n m ax n t s))
("abandons" nil (ax b ae1 n d ax n z))
("abanto" nil (ax b ae1 n t ow0))
("abarca" nil (ax b aa1 r k ax))
("abare" nil (aa0 b aa1 r iy0))
("abascal" nil (ae1 b ax s k ax l))
("abashed" nil (ax b ae1 sh t))
("abate" nil (ax b ey1 t))
("abated" nil (ax b ey1 t ih0 d))
("abatement" nil (ax b ey1 t m ax n t))
("abatements" nil (ax b ey1 t m ax n t s))
("abates" nil (ax b ey1 t s))
("abating" nil (ax b ey1 t ih0 ng))
("abba" nil (ae1 b ax))
("abbado" nil (ax b aa1 d ow0))
("abbas" nil (ax b aa1 s))
("abbasi" nil (aa0 b aa1 s iy0))
("abbate" nil (aa1 b ey0 t))
("abbatiello" nil (aa0 b aa0 t iy0 eh1 l ow0))
("abbe" nil (ae1 b iy0))
("abbenhaus" nil (ae1 b ax n hh aw1 s))
("abbett" nil (ax b eh1 t))
("abbey" nil (ae1 b iy0))
("abbie" nil (ae1 b iy0))
("abbitt" nil (ae1 b ih0 t))
("abbot" nil (ae1 b ax t))
("abbott" nil (ae1 b ax t))
("abboud" nil (ax b uw1 d))
("abbreviate" nil (ax b r iy1 v iy0 ey1 t))
("abbreviated" nil (ax b r iy1 v iy0 ey1 t ax d))
("abbreviation" nil (ax b r iy1 v iy0 ey1 sh ax n))
("abbreviations" nil (ax b r iy1 v iy0 ey1 sh ax n z))
("abbruzzese" nil (aa0 b r uw0 t s ey1 z iy0))
("abbs" nil (ae1 b z))
("abby" nil (ae1 b iy0))
("abco" nil (ae1 b k ow0))
("abcotek" nil (ae1 b k ow0 t eh1 k))
("abdalla" nil (ae1 b d ae1 l ax))
("abdallah" nil (ae1 b d ae1 l ax))
("abdel" nil (ae1 b d eh1 l))
("abdella" nil (ae1 b d eh1 l ax))
("abdicate" nil (ae1 b d ax k ey1 t))
("abdicated" nil (ae1 b d ax k ey1 t ax d))
("abdicating" nil (ae1 b d ih0 k ey1 t ih0 ng))
("abdication" nil (ae1 b d ih0 k ey1 sh ax n))
("abdnor" nil (ae1 b d n er0))
("abdo" nil (ae1 b d ow0))
("abdollah" nil (ae1 b d aa1 l ax))
("abdomen" nil (ae0 b d ow1 m ax n))
("abdominal" nil (ae0 b d aa1 m ax n ax l))
("abducted" nil (ae0 b d ah1 k t ih0 d))
("abducting" nil (ax b d ah1 k t ih0 ng))
("abduction" nil (ae0 b d ah1 k sh ax n))
("abductions" nil (ae0 b d ah1 k sh ax n z))
("abductor" nil (ax b d ah1 k t er0))
("abductors" nil (ax b d ah1 k t er0 z))
("abdul" nil (ae0 b d uw1 l))
("abdulaziz" nil (ae0 b d uw1 l ax z iy1 z))
("abdulla" nil (aa0 b d uw1 l ax))
("abdullah" nil (ae1 b d ah1 l ax))
("abe" nil (ey1 b))
("abed" nil (ax b eh1 d))
("abedi" nil (ax b eh1 d iy0))
("abee" nil (ax b iy1))
("abel" nil (ey1 b ax l))
("abela" nil (aa0 b eh1 l ax))
("abelard" nil (ae1 b ih0 l er0 d))
("abele" nil (ax b iy1 l))
("abeles" nil (ax b iy1 l z))
("abell" nil (ey1 b ax l))
("abella" nil (ax b eh1 l ax))
("abeln" nil (ae1 b ih0 l n))
("abelow" nil (ae1 b ax l ow0))
("abels" nil (ey1 b ax l z))
("abelson" nil (ae1 b ih0 l s ax n))
("abend" nil (ae1 b eh0 n d))
("abendroth" nil (ae1 b ih0 n d r ao0 th))
("aber" nil (ey1 b er0))
("abercrombie" nil (ae1 b er0 k r aa1 m b iy0))
("aberdeen" nil (ae1 b er0 d iy1 n))
("aberford" nil (ey1 b er0 f er0 d))
("aberg" nil (ae1 b er0 g))
("aberle" nil (ae1 b er0 ax l))
("abermin" nil (ae1 b er0 m ih0 n))
("abernathy" nil (ae1 b er0 n ae1 th iy0))
("abernethy" nil (ae1 b er0 n eh1 th iy0))
("aberrant" nil (ae0 b eh1 r ax n t))
("aberration" nil (ae1 b er0 ey1 sh ax n))
("aberrations" nil (ae1 b er0 ey1 sh ax n z))
("abert" nil (ae1 b er0 t))
("abet" nil (ax b eh1 t))
("abetted" nil (ax b eh1 t ih0 d))
("abetting" nil (ax b eh1 t ih0 ng))
("abex" nil (ey1 b eh0 k s))
("abeyance" nil (ax b ey1 ax n s))
("abeyta" nil (aa0 b ey1 t ax))
("abhor" nil (ae0 b hh ao1 r))
("abhorred" nil (ax b hh ao1 r d))
("abhorrence" nil (ax b hh ao1 r ax n s))
("abhorrent" nil (ae0 b hh ao1 r ax n t))
("abhors" nil (ax b hh ao1 r z))
("abid" nil (ey1 b ih0 d))
("abide" nil (ax b ay1 d))
("abided" nil (ax b ay1 d ih0 d))
("abides" nil (ax b ay1 d z))
("abiding" nil (ax b ay1 d ih0 ng))
("abie" nil (ae1 b iy0))
("abigail" nil (ae1 b ax g ey1 l))
("abila" nil (aa0 b iy1 l ax))
("abilene" nil (ae1 b ih0 l iy1 n))
("abilities" nil (ax b ih1 l ax t iy0 z))
("ability" nil (ax b ih1 l ax t iy0))
("abington" nil (ae1 b ih0 ng t ax n))
("abio" nil (aa1 b iy0 ow0))
("abiola" nil (aa1 b iy0 ow1 l ax))
("abiomed" nil (ey0 b iy1 ax m eh0 d))
("abitibi" nil (ae1 b ih0 t iy1 b iy0))
("abitz" nil (ae1 b ih0 t s))
("abject" v (ae0 b jh eh1 k t))
("abject" n (ae1 b jh eh0 k t))
("ablaze" nil (ax b l ey1 z))
("able" nil (ey1 b ax l))
("abler" nil (ey1 b ax l er0))
("ables" nil (ey1 b ax l z))
("ablest" nil (ey1 b ax l s t))
("abloom" nil (ax b l uw1 m))
("ably" nil (ey1 b l iy0))
("abner" nil (ae1 b n er0))
("abney" nil (ae1 b n iy0))
("abnormal" nil (ae0 b n ao1 r m ax l))
("abnormalities" nil (ae1 b n ao0 r m ae1 l ax t iy0 z))
("abnormality" nil (ae1 b n ao0 r m ae1 l ax t iy0))
("abnormally" nil (ae0 b n ao1 r m ax l iy0))
("abo" nil (aa1 b ow0))
("aboard" nil (ax b ao1 r d))
("abode" nil (ax b ow1 d))
("abolish" nil (ax b aa1 l ih0 sh))
("abolished" nil (ax b aa1 l ih0 sh t))
("abolishes" nil (ax b aa1 l ih0 sh ih0 z))
("abolishing" nil (ax b aa1 l ih0 sh ih0 ng))
("abolition" nil (ae1 b ax l ih1 sh ax n))
("abolitionism" nil (ae1 b ax l ih1 sh ax n ih1 z ax m))
("abolitionist" nil (ae1 b ax l ih1 sh ax n ax s t))
("abolitionists" nil (ae1 b ax l ih1 sh ax n ax s t s))
("abominable" nil (ax b aa1 m ax n ax b ax l))
("abomination" nil (ax b aa1 m ax n ey1 sh ax n))
("abood" nil (ax b uw1 d))
("aboodi" nil (ax b uw1 d iy0))
("aboriginal" nil (ae1 b er0 ih1 jh ax n ax l))
("aborigine" nil (ae1 b er0 ih1 jh ax n iy0))
("aborigines" nil (ae1 b er0 ih1 jh ax n iy0 z))
("abort" nil (ax b ao1 r t))
("aborted" nil (ax b ao1 r t ax d))
("abortifacients" nil (ax b ao1 r t ax f ey1 sh ax n t s))
("aborting" nil (ax b ao1 r t ih0 ng))
("abortion" nil (ax b ao1 r sh ax n))
("abortionist" nil (ax b ao1 r sh ax n ih0 s t))
("abortionists" nil (ax b ao1 r sh ax n ih0 s t s))
("abortions" nil (ax b ao1 r sh ax n z))
("abortive" nil (ax b ao1 r t ih0 v))
("abott" nil (ax b aa1 t))
("aboud" nil (aa0 b uw1 d))
("abouhalima" nil (aa1 b uw0 hh aa0 l iy1 m ax))
("abound" nil (ax b aw1 n d))
("abounded" nil (ax b aw1 n d ax d))
("abounding" nil (ax b aw1 n d ih0 ng))
("abounds" nil (ax b aw1 n d z))
("about" nil (ax b aw1 t))
("above" nil (ax b ah1 v))
("aboveboard" nil (ax b ah1 v b ao1 r d))
("abplanalp" nil (ae1 b p l ax n ae0 l p))
("abra" nil (aa1 b r ax))
("abracadabra" nil (ae1 b r ax k ax d ae1 b r ax))
("abraham" nil (ey1 b r ax hh ae1 m))
("abrahamian" nil (ae1 b r ax hh ey1 m iy0 ax n))
("abrahams" nil (ey1 b r ax hh ae1 m z))
("abrahamsen" nil (ae0 b r ax hh ae1 m s ax n))
("abrahamson" nil (ax b r ae1 hh ax m s ax n))
("abram" nil (ax b r ae1 m))
("abramczyk" nil (aa1 b r ax m ch ih0 k))
("abramo" nil (aa0 b r aa1 m ow0))
("abramovitz" nil (ax b r aa1 m ax v ih0 t s))
("abramowicz" nil (ax b r aa1 m ax v ih0 ch))
("abramowitz" nil (ax b r aa1 m ax w ih0 t s))
("abrams" nil (ey1 b r ax m z))
("abramson" nil (ey1 b r ax m s ax n))
("abrasion" nil (ax b r ey1 zh ax n))
("abrasive" nil (ax b r ey1 s ih0 v))
("abrasives" nil (ax b r ey1 s ih0 v z))
("abreast" nil (ax b r eh1 s t))
("abrego" nil (aa0 b r eh1 g ow0))
("abreu" nil (ax b r uw1))
("abridged" nil (ax b r ih1 jh d))
("abril" nil (ax b r ih1 l))
("abroad" nil (ax b r ao1 d))
("abrogate" nil (ae1 b r ax g ey1 t))
("abrogated" nil (ae1 b r ax g ey1 t ih0 d))
("abrogating" nil (ae1 b r ax g ey1 t ih0 ng))
("abrogation" nil (ae1 b r ax g ey1 sh ax n))
("abron" nil (ax b r aa1 n))
("abrupt" nil (ax b r ah1 p t))
("abruptly" nil (ax b r ah1 p t l iy0))
("abruptness" nil (ax b r ah1 p t n ax s))
("abrutyn" nil (ey1 b r uw0 t ih0 n))
("abruzzese" nil (aa0 b r uw0 t s ey1 z iy0))
("abruzzo" nil (aa0 b r uw1 z ow0))
("absalom" nil (ae1 b s ax l ax m))
("abscam" nil (ae1 b s k ae0 m))
("abscess" nil (ae1 b s eh1 s))
("absence" nil (ae1 b s ax n s))
("absences" nil (ae1 b s ax n s ax z))
("absent" j (ae1 b s ax n t))
("absent" v (ax0 b s eh1 n t))
("absentee" nil (ae1 b s ax n t iy1))
("absenteeism" nil (ae1 b s ax n t iy1 ih0 z ax m))
("absentees" nil (ae1 b s ax n t iy1 z))
("absentia" nil (ae0 b s eh1 n sh ax))
("absher" nil (ae1 b sh er0))
("abshier" nil (ae1 b sh iy0 er0))
("abshire" nil (ae1 b sh ay1 r))
("absolom" nil (ae1 b s ax l ax m))
("absolut" nil (ae1 b s ax l uw1 t))
("absolute" nil (ae1 b s ax l uw1 t))
("absolutely" nil (ae1 b s ax l uw1 t l iy0))
("absoluteness" nil (ae1 b s ax l uw1 t n ax s))
("absolution" nil (ae1 b s ax l uw1 sh ax n))
("absolutism" nil (ae1 b s ax l uw1 t ih1 z ax m))
("absolve" nil (ax b z aa1 l v))
("absolved" nil (ax b z aa1 l v d))
("absolves" nil (ax b z aa1 l v z))
("absolving" nil (ax b z aa1 l v ih0 ng))
("absorb" nil (ax b z ao1 r b))
("absorbed" nil (ax b z ao1 r b d))
("absorbency" nil (ax b z ao1 r b ax n s iy0))
("absorbent" nil (ax b z ao1 r b ax n t))
("absorber" nil (ax b z ao1 r b er0))
("absorbers" nil (ax b z ao1 r b er0 z))
("absorbing" nil (ax b z ao1 r b ih0 ng))
("absorbs" nil (ax b z ao1 r b z))
("absorption" nil (ax b z ao1 r p sh ax n))
("abstain" nil (ax b s t ey1 n))
("abstained" nil (ax b s t ey1 n d))
("abstaining" nil (ax b s t ey1 n ih0 ng))
("abstention" nil (ax b s t eh1 n ch ax n))
("abstentions" nil (ax b s t eh1 n ch ax n z))
("abstinence" nil (ae1 b s t ax n ax n s))
("abstinent" nil (ae1 b s t ax n ax n t))
("abston" nil (ae1 b s t ax n))
("abstract" v (ae0 b s t r ae1 k t))
("abstract" n (ae1 b s t r ae0 k t))
("abstracted" nil (ae1 b s t r ae1 k t ih0 d))
("abstraction" nil (ae0 b s t r ae1 k sh ax n))
("abstractions" nil (ae0 b s t r ae1 k sh ax n z))
("abstracts" n (ae1 b s t r ae0 k t s))
("abstracts" v (ax0 b s t r ae1 k t s))
("abstruse" nil (ax b s t r uw1 s))
("absurd" nil (ax b s er1 d))
("absurdist" nil (ax b s er1 d ih0 s t))
("absurdities" nil (ax b s er1 d ax t iy0 z))
("absurdity" nil (ax b s er1 d ax t iy0))
("absurdly" nil (ax b s er1 d l iy0))
("abt" nil (ae1 b t))
("abts" nil (ae1 b t s))
("abu" nil (ae1 b uw0))
("abuladze" nil (ae1 b y uw0 l ae1 d z iy0))
("abundance" nil (ax b ah1 n d ax n s))
("abundant" nil (ax b ah1 n d ax n t))
("abundantly" nil (ax b ah1 n d ax n t l iy0))
("aburto" nil (ax b uh1 r t ow1))
("abuse" n (ax b y uw1 s))
("abuse" v (ax b y uw1 z))
("abused" nil (ax b y uw1 z d))
("abuser" nil (ax b y uw1 z er0))
("abusers" nil (ax b y uw1 z er0 z))
("abuses" n (ax b y uw1 s ax z))
("abuses" v (ax b y uw1 z ih0 z))
("abusing" nil (ax b y uw1 z ih0 ng))
("abusive" nil (ax b y uw1 s ih0 v))
("abut" nil (ax b ah1 t))
("abuts" nil (ax b ah1 t s))
("abutted" nil (ax b ah1 t ax d))
("abutting" nil (ax b ah1 t ih0 ng))
("abuzz" nil (ax b ah1 z))
("abysmal" nil (ax b ih1 z m ax l))
("abysmally" nil (ax b ih1 z m ax l iy0))
("abyss" nil (ax b ih1 s))
("ac" nil (ey1 s iy1))
("aca" nil (ae1 k ax))
("acacia" nil (ax k ey1 sh ax))
("academe" nil (ae1 k ax d iy1 m))
("academia" nil (ae1 k ax d iy1 m iy0 ax))
("academic" nil (ae1 k ax d eh1 m ih0 k))
("academically" nil (ae1 k ax d eh1 m ih0 k l iy0))
("academician" nil (ae1 k ax d ax m ih1 sh ax n))
("academicians" nil (ae1 k ax d ax m ih1 sh ax n z))
("academics" nil (ae1 k ax d eh1 m ih0 k s))
("academies" nil (ax k ae1 d ax m iy0 z))
("academy" nil (ax k ae1 d ax m iy0))
("acadia" nil (ax k ey1 d iy0 ax))
("acampora" nil (ax k ae1 m p er0 ax))
("acantha" nil (aa0 k aa1 n dh ax))
("acapulco" nil (ae1 k ax p uh1 l k ow0))
("accardi" nil (aa0 k aa1 r d iy0))
("accardo" nil (aa0 k aa1 r d ow0))
("accede" nil (ae0 k s iy1 d))
("acceded" nil (ae0 k s iy1 d ih0 d))
("accedes" nil (ae0 k s iy1 d z))
("acceding" nil (ae0 k s iy1 d ih0 ng))
("accel" nil (ax k s eh1 l))
("accelerate" nil (ae0 k s eh1 l er0 ey1 t))
("accelerated" nil (ae0 k s eh1 l er0 ey1 t ax d))
("accelerates" nil (ae0 k s eh1 l er0 ey1 t s))
("accelerating" nil (ae0 k s eh1 l er0 ey1 t ih0 ng))
("acceleration" nil (ae1 k s eh1 l er0 ey1 sh ax n))
("accelerator" nil (ae0 k s eh1 l er0 ey1 t er0))
("accelerometer" nil (ae0 k s eh1 l er0 aa1 m ax t er0))
("accelerometers" nil (ae0 k s eh1 l er0 aa1 m ax t er0 z))
("accent" v (ae0 k s eh1 n t))
("accent" n (ae1 k s eh0 n t))
("accented" nil (ae1 k s eh0 n t ih0 d))
("accents" n (ae1 k s eh0 n t s))
("accents" v (ae0 k s eh1 n t s))
("accentuate" nil (ax k s eh1 n ch uw0 ey0 t))
("accentuated" nil (ae0 k s eh1 n ch ax w ey1 t ih0 d))
("accentuating" nil (ae0 k s eh1 n ch ax w ey1 t ih0 ng))
("accept" nil (ae0 k s eh1 p t))
("acceptability" nil (ax k s eh1 p t ax b ih1 l ax t iy0))
("acceptable" nil (ae0 k s eh1 p t ax b ax l))
("acceptance" nil (ae0 k s eh1 p t ax n s))
("acceptances" nil (ae0 k s eh1 p t ax n s ih0 z))
("accepted" nil (ae0 k s eh1 p t ih0 d))
("accepting" nil (ae0 k s eh1 p t ih0 ng))
("accepts" nil (ae0 k s eh1 p t s))
("access" nil (ae1 k s eh1 s))
("accessed" nil (ae1 k s eh1 s t))
("accessibility" nil (ae1 k s eh0 s ax b ih1 l ih0 t iy0))
("accessible" nil (ae0 k s eh1 s ax b ax l))
("accessing" nil (ae1 k s eh1 s ih0 ng))
("accession" nil (ax k s eh1 sh ax n))
("accessories" nil (ae0 k s eh1 s er0 iy0 z))
("accessory" nil (ae0 k s eh1 s er0 iy0))
("accetta" nil (aa0 ch eh1 t ax))
("accident" nil (ae1 k s ax d ax n t))
("accidental" nil (ae1 k s ax d eh1 n t ax l))
("accidentally" nil (ae1 k s ax d eh1 n t ax l iy0))
("accidently" nil (ae1 k s ax d ax n t l iy0))
("accidents" nil (ae1 k s ax d ax n t s))
("accion" nil (ae1 ch iy0 ax n))
("accival" nil (ae1 s ih0 v aa1 l))
("acclaim" nil (ax k l ey1 m))
("acclaimed" nil (ax k l ey1 m d))
("acclimate" nil (ae1 k l ax m ey1 t))
("acclimated" nil (ae1 k l ax m ey1 t ih0 d))
("acclimation" nil (ae1 k l ax m ey1 sh ax n))
("acco" nil (ae1 k ow0))
("accola" nil (aa0 k ow1 l ax))
("accolade" nil (ae1 k ax l ey1 d))
("accolades" nil (ae1 k ax l ey1 d z))
("accomando" nil (aa0 k ow0 m aa1 n d ow0))
("accommodate" nil (ax k aa1 m ax d ey1 t))
("accommodated" nil (ax k aa1 m ax d ey1 t ax d))
("accommodates" nil (ax k aa1 m ax d ey1 t s))
("accommodating" nil (ax k aa1 m ax d ey1 t ih0 ng))
("accommodation" nil (ax k aa1 m ax d ey1 sh ax n))
("accommodations" nil (ax k aa1 m ax d ey1 sh ax n z))
("accommodative" nil (ax k aa1 m ax d ey1 t ih0 v))
("accompanied" nil (ax k ah1 m p ax n iy0 d))
("accompanies" nil (ax k ah1 m p ax n iy0 z))
("accompaniment" nil (ax k ah1 m p n ih0 m ax n t))
("accompaniments" nil (ax k ah1 m p n ih0 m ax n t s))
("accompanist" nil (ax k ah1 m p ax n ax s t))
("accompany" nil (ax k ah1 m p ax n iy0))
("accompanying" nil (ax k ah1 m p ax n iy0 ih0 ng))
("accompli" nil (aa1 k aa1 m p l iy0))
("accomplice" nil (ax k aa1 m p l ax s))
("accomplices" nil (ax k aa1 m p l ax s ax z))
("accomplish" nil (ax k aa1 m p l ih0 sh))
("accomplished" nil (ax k aa1 m p l ih0 sh t))
("accomplishes" nil (ax k aa1 m p l ih0 sh ih0 z))
("accomplishing" nil (ax k aa1 m p l ih0 sh ih0 ng))
("accomplishment" nil (ax k aa1 m p l ih0 sh m ax n t))
("accomplishments" nil (ax k aa1 m p l ih0 sh m ax n t s))
("accor" nil (ae1 k ao1 r))
("accord" nil (ax k ao1 r d))
("accordance" nil (ax k ao1 r d ax n s))
("accorded" nil (ax k ao1 r d ih0 d))
("according" nil (ax k ao1 r d ih0 ng))
("accordingly" nil (ax k ao1 r d ih0 ng l iy0))
("accordion" nil (ax k ao1 r d iy0 ax n))
("accords" nil (ax k ao1 r d z))
("accost" nil (ax k ao1 s t))
("accosted" nil (ax k aa1 s t ax d))
("account" nil (ax k aw1 n t))
("accountability" nil (ax k aw1 n t ax b ih1 l ih0 t iy0))
("accountable" nil (ax k aw1 n t ax b ax l))
("accountancy" nil (ax k aw1 n t ax n s iy0))
("accountant" nil (ax k aw1 n t ax n t))
("accountants" nil (ax k aw1 n t ax n t s))
("accounted" nil (ax k aw1 n t ax d))
("accountemps" nil (ax k aw1 n t eh1 m p s))
("accounting" nil (ax k aw1 n t ih0 ng))
("accounts" nil (ax k aw1 n t s))
("accouterments" nil (ax k uw1 t er0 m ax n t s))
("accreditation" nil (ax k r eh1 d ax t ey1 sh ax n))
("accreditations" nil (ax k r eh1 d ax d ey1 sh ax n z))
("accredited" nil (ax k r eh1 d ih0 t ih0 d))
("accrediting" nil (ax k r eh1 d ax t ih0 ng))
("accretion" nil (ax k r iy1 sh ax n))
("accrual" nil (ax k r uw1 ax l))
("accruals" nil (ax k r uw1 ax l z))
("accrue" nil (ax k r uw1))
("accrued" nil (ax k r uw1 d))
("accrues" nil (ax k r uw1 z))
("accruing" nil (ax k r uw1 ih0 ng))
("accumulate" nil (ax k y uw1 m y ax l ey1 t))
("accumulated" nil (ax k y uw1 m y ax l ey1 t ax d))
("accumulates" nil (ax k y uw1 m y ax l ey1 t s))
("accumulating" nil (ax k y uw1 m y ax l ey1 t ih0 ng))
("accumulation" nil (ax k y uw1 m y ax l ey1 sh ax n))
("accumulations" nil (ax k y uw1 m y ax l ey1 sh ax n z))
("accumulative" nil (ax k y uw1 m y ax l ey1 t ih0 v))
("accumulators" nil (ax k y uw1 m y ax l ey1 t er0 z))
("accuracies" nil (ae1 k y er0 ax s iy0 z))
("accuracy" nil (ae1 k y er0 ax s iy0))
("accurate" nil (ae1 k y er0 ax t))
("accurately" nil (ae1 k y er0 ax t l iy0))
("accuray" nil (ae1 k y er0 ey1))
("accuride" nil (ae1 k y er0 ay1 d))
("accurso" nil (aa0 k uh1 r s ow0))
("accusation" nil (ae1 k y ax z ey1 sh ax n))
("accusations" nil (ae1 k y ax z ey1 sh ax n z))
("accusative" nil (ax k y uw1 z ax t ih0 v))
("accusatory" nil (ax k y uw1 z ax t ao1 r iy0))
("accuse" nil (ax k y uw1 z))
("accused" nil (ax k y uw1 z d))
("accuser" nil (ax k y uw1 z er0))
("accusers" nil (ax k y uw1 z er0 z))
("accuses" nil (ax k y uw1 z ih0 z))
("accusing" nil (ax k y uw1 z ih0 ng))
("accusingly" nil (ax k y uw1 z ih0 ng l iy0))
("accustom" nil (ax k ah1 s t ax m))
("accustomed" nil (ax k ah1 s t ax m d))
("accutane" nil (ae1 k y uw0 t ey1 n))
("ace" nil (ey1 s))
("aced" nil (ey1 s t))
("acer" nil (ey1 s er0))
("acerbic" nil (ax s eh1 r b ih0 k))
("acero" nil (ax s eh1 r ow0))
("acerra" nil (ax s eh1 r ax))
("aces" nil (ey1 s ih0 z))
("acetaminophen" nil (ax s iy1 t ax m ih1 n ax f ax n))
("acetate" nil (ae1 s ax t ey1 t))
("acetic" nil (ax s eh1 t ih0 k))
("aceto" nil (aa0 s eh1 t ow0))
("acetone" nil (ae1 s ax t ow1 n))
("acetylcholine" nil (ax s eh1 t ax l k ow1 l iy0 n))
("acetylene" nil (ax s eh1 t ax l iy1 n))
("acevedo" nil (ae0 s ax v ey1 d ow0))
("aceves" nil (aa0 s ey1 v eh0 s))
("acey" nil (ey1 s iy0))
("achatz" nil (ae1 k ax t s))
("ache" nil (ey1 k))
("achebe" nil (aa0 ch ey1 b iy0))
("achee" nil (ax ch iy1))
("achenbach" nil (ae1 k ih0 n b aa0 k))
("achenbaum" nil (ae1 k ax n b aw1 m))
("aches" nil (ey1 k s))
("acheson" nil (ae1 ch ax s ax n))
("achey" nil (ae1 ch iy0))
("achievable" nil (ax ch iy1 v ax b ax l))
("achieve" nil (ax ch iy1 v))
("achieved" nil (ax ch iy1 v d))
("achievement" nil (ax ch iy1 v m ax n t))
("achievements" nil (ax ch iy1 v m ax n t s))
("achiever" nil (ax ch iy1 v er0))
("achievers" nil (ax ch iy1 v er0 z))
("achieves" nil (ax ch iy1 v z))
("achieving" nil (ax ch iy1 v ih0 ng))
("achille" nil (ax k ih1 l iy0))
("achilles" nil (ax k ih1 l iy0 z))
("aching" nil (ey1 k ih0 ng))
("achoa" nil (ax ch ow1 ax))
("achor" nil (ey1 k er0))
("achord" nil (ae1 k ao0 r d))
("achorn" nil (ae1 k er0 n))
("achterberg" nil (ae1 k t er0 b er0 g))
("achy" nil (ey1 k iy0))
("acid" nil (ae1 s ax d))
("acidic" nil (ax s ih1 d ih0 k))
("acidification" nil (ax s ih1 d ax f ax k ey1 sh ax n))
("acidified" nil (ax s ih1 d ax f ay1 d))
("acidifies" nil (ax s ih1 d ax f ay1 z))
("acidify" nil (ax s ih1 d ax f ay1))
("acidity" nil (ax s ih1 d ax t iy0))
("acidosis" nil (ae1 s ax d ow1 s ax s))
("acids" nil (ae1 s ax d z))
("aciduria" nil (ae1 s ax d uh1 r iy0 ax))
("acierno" nil (aa0 s ih1 r n ow0))
("ack" nil (ae1 k))
("acker" nil (ae1 k er0))
("ackerley" nil (ae1 k er0 l iy0))
("ackerly" nil (ae1 k er0 l iy0))
("ackerman" nil (ae1 k er0 m ax n))
("ackermann" nil (ae1 k er0 m ax n))
("ackerson" nil (ae1 k er0 s ax n))
("ackert" nil (ae1 k er0 t))
("ackland" nil (ae1 k l ax n d))
("ackles" nil (ae1 k ax l z))
("ackley" nil (ae1 k l iy0))
("acklin" nil (ae1 k l ih0 n))
("ackman" nil (ae1 k m ax n))
("acknowledge" nil (ae0 k n aa1 l ih0 jh))
("acknowledgeable" nil (ae0 k n aa1 l ih0 jh ax b ax l))
("acknowledged" nil (ae0 k n aa1 l ih0 jh d))
("acknowledgement" nil (ae0 k n aa1 l ih0 jh m ax n t))
("acknowledges" nil (ae0 k n aa1 l ih0 jh ih0 z))
("acknowledging" nil (ae0 k n aa1 l ih0 jh ih0 ng))
("acknowledgment" nil (ae0 k n aa1 l ih0 jh m ax n t))
("acknowleged" nil (ae0 k n aa1 l ih0 jh d))
("ackroyd" nil (ae1 k r oy1 d))
("acmat" nil (ae1 k m ae0 t))
("acme" nil (ae1 k m iy0))
("acne" nil (ae1 k n iy0))
("acocella" nil (aa0 k ow0 ch eh1 l ax))
("acoff" nil (ae1 k ao0 f))
("acolytes" nil (ae1 k ax l ay1 t s))
("acord" nil (ax k ao1 r d))
("acorn" nil (ey1 k ao0 r n))
("acorns" nil (ey1 k ao0 r n z))
("acosta" nil (ax k ao1 s t ax))
("acoustic" nil (ax k uw1 s t ih0 k))
("acoustical" nil (ax k uw1 s t ih0 k ax l))
("acoustically" nil (ax k uw1 s t ih0 k l iy0))
("acoustics" nil (ax k uw1 s t ih0 k s))
("acquaint" nil (ax k w ey1 n t))
("acquaintance" nil (ax k w ey1 n t ax n s))
("acquaintances" nil (ax k w ey1 n t ax n s ih0 z))
("acquainted" nil (ax k w ey1 n t ax d))
("acquaviva" nil (aa0 k w aa0 v iy1 v ax))
("acquiesce" nil (ae1 k w iy0 eh1 s))
("acquiesced" nil (ae1 k w iy0 eh1 s t))
("acquiescence" nil (ae1 k w iy0 eh1 s ax n s))
("acquiescing" nil (ae1 k w iy0 eh1 s ih0 ng))
("acquire" nil (ax k w ay1 er0))
("acquired" nil (ax k w ay1 er0 d))
("acquirer" nil (ax k w ay1 er0 er0))
("acquirers" nil (ax k w ay1 er0 er0 z))
("acquires" nil (ax k w ay1 er0 z))
("acquiring" nil (ax k w ay1 r ih0 ng))
("acquisition" nil (ae1 k w ax z ih1 sh ax n))
("acquisitions" nil (ae1 k w ih0 z ih1 sh ax n z))
("acquisitive" nil (ax k w ih1 z ax t ih0 v))
("acquistion" nil (ax k w ih1 s ch ax n))
("acquistions" nil (ax k w ih1 s ch ax n z))
("acquit" nil (ax k w ih1 t))
("acquitaine" nil (ae1 k w ih0 t ey1 n))
("acquittal" nil (ax k w ih1 t ax l))
("acquitted" nil (ax k w ih1 t ax d))
("acquitting" nil (ax k w ih1 t ih0 ng))
("acre" nil (ey1 k er0))
("acreage" nil (ey1 k er0 ih0 jh))
("acree" nil (ax k r iy1))
("acres" nil (ey1 k er0 z))
("acrey" nil (ae1 k r iy0))
("acri" nil (aa1 k r iy0))
("acrimonious" nil (ae1 k r ax m ow1 n iy0 ax s))
("acrimony" nil (ae1 k r ih0 m ow1 n iy0))
("acrobat" nil (ae1 k r ax b ae1 t))
("acrobatic" nil (ae1 k r ax b ae1 t ih0 k))
("acrobatics" nil (ae1 k r ax b ae1 t ih0 k s))
("acrobats" nil (ae1 k r ax b ae1 t s))
("acronym" nil (ae1 k r ax n ih0 m))
("acronyms" nil (ae1 k r ax n ih0 m z))
("acropolis" nil (ax k r aa1 p ax l ax s))
("across" nil (ax k r ao1 s))
("acrylic" nil (ax k r ih1 l ih0 k))
("acrylics" nil (ax k r ih1 l ih0 k s))
("act" nil (ae1 k t))
("actava" nil (ae1 k t aa1 v ax))
("acted" nil (ae1 k t ax d))
("actigall" nil (ae1 k t ih0 g ao0 l))
("actin" nil (ae1 k t ax n))
("acting" nil (ae1 k t ih0 ng))
("actinide" nil (ae1 k t ih0 n ay1 d))
("actinidia" nil (ae1 k t ih0 n ih1 d iy0 ax))
("action" nil (ae1 k sh ax n))
("actionable" nil (ae1 k sh ax n ax b ax l))
("actions" nil (ae1 k sh ax n z))
("activase" nil (ae1 k t ih0 v ey1 z))
("activate" nil (ae1 k t ax v ey1 t))
("activated" nil (ae1 k t ax v ey1 t ax d))
("activates" nil (ae1 k t ax v ey1 t s))
("activating" nil (ae1 k t ax v ey1 t ih0 ng))
("activation" nil (ae1 k t ax v ey1 sh ax n))
("activator" nil (ae1 k t ax v ey1 t er0))
("active" nil (ae1 k t ih0 v))
("actively" nil (ae1 k t ih0 v l iy0))
("actives" nil (ae1 k t ih0 v z))
("activision" nil (ae1 k t ih0 v ih1 zh ax n))
("activism" nil (ae1 k t ih0 v ih1 z ax m))
("activist" nil (ae1 k t ax v ax s t))
("activists" nil (ae1 k t ax v ax s t s))
("activities" nil (ae0 k t ih1 v ax t iy0 z))
("activity" nil (ae0 k t ih1 v ax t iy0))
("actmedia" nil (ae0 k t m iy1 d iy0 ax))
("acton" nil (ae1 k t ax n))
("actor" nil (ae1 k t er0))
("actors" nil (ae1 k t er0 z))
("actress" nil (ae1 k t r ax s))
("actresses" nil (ae1 k t r ax s ih0 z))
("acts" nil (ae1 k t s))
("actual" nil (ae1 k ch ax w ax l))
("actuality" nil (ae1 k ch ax w ae1 l ax t iy0))
("actualize" nil (ae1 k ch ax w ax l ay1 z))
("actually" nil (ae1 k ch ax w ax l iy0))
("actuarial" nil (ae1 k ch ax w eh1 r iy0 ax l))
("actuaries" nil (ae1 k ch ax w eh1 r iy0 z))
("actuary" nil (ae1 k ch ax w eh1 r iy0))
("actus" nil (ae1 k t ax s))
("acuff" nil (ax k ah1 f))
("acuity" nil (ax k y uw1 ax t iy0))
("acumen" nil (ax k y uw1 m ax n))
("acuna" nil (aa0 k uw1 n ax))
("acupuncture" nil (ae1 k y uw0 p ah1 ng k ch er0))
("acura" nil (ae1 k y er0 ax))
("acuras" nil (ae1 k y er0 ax z))
("acuson" nil (ae1 k y uw1 s ax n))
("acustar" nil (ae1 k y uw0 s t aa1 r))
("acusyst" nil (ae1 k y uw0 s ih0 s t))
("acute" nil (ax k y uw1 t))
("acutely" nil (ax k y uw1 t l iy0))
("acuteness" nil (ax k y uw1 t n ax s))
("acyclovir" nil (ax s ih1 k l ow0 v ih1 r))
("ad" nil (ae1 d))
("ada" nil (ey1 d ax))
("adabel" nil (ae1 d ax b eh0 l))
("adabelle" nil (ae1 d ax b ax l))
("adachi" nil (aa0 d aa1 k iy0))
("adage" nil (ae1 d ax jh))
("adagio" nil (ax d aa1 zh iy0 ow1))
("adah" nil (ae1 d aa0))
("adair" nil (ax d eh1 r))
("adaire" nil (aa0 d eh1 r))
("adalia" nil (aa0 d aa1 l iy0 ax))
("adam" nil (ae1 d ax m))
("adamant" nil (ae1 d ax m ax n t))
("adamantly" nil (ae1 d ax m ax n t l iy0))
("adamcik" nil (aa1 d ax m ch ih0 k))
("adamczak" nil (aa1 d ax m ch ae0 k))
("adamczyk" nil (aa1 d ax m ch ih0 k))
("adame" nil (aa0 d aa1 m iy0))
("adamec" nil (ax d aa1 m ih0 k))
("adamek" nil (ax d aa1 m eh0 k))
("adames" nil (ax d ey1 m z))
("adami" nil (aa0 d aa1 m iy0))
("adamik" nil (ax d aa1 m ih0 k))
("adamina" nil (aa0 d aa0 m iy1 n ax))
("adamo" nil (aa0 d aa1 m ow0))
("adamowicz" nil (ax d aa1 m ax v ih0 ch))
("adams" nil (ae1 d ax m z))
("adamski" nil (ax d ae1 m s k iy1))
("adamson" nil (ae1 d ax m s ax n))
("adamstown" nil (ae1 d ax m s t aw1 n))
("adan" nil (ey1 d ax n))
("adapso" nil (ax d ae1 p s ow0))
("adapt" nil (ax d ae1 p t))
("adaptability" nil (ax d ae1 p t ax b ih1 l ax t iy0))
("adaptable" nil (ax d ae1 p t ax b ax l))
("adaptaplex" nil (ax d ae1 p t ax p l eh1 k s))
("adaptation" nil (ae1 d ax p t ey1 sh ax n))
("adaptations" nil (ae1 d ae0 p t ey1 sh ax n z))
("adaptec" nil (ax d ae1 p t eh1 k))
("adapted" nil (ax d ae1 p t ax d))
("adapter" nil (ax d ae1 p t er0))
("adapters" nil (ax d ae1 p t er0 z))
("adapting" nil (ax d ae1 p t ih0 ng))
("adaptive" nil (ax d ae1 p t ih0 v))
("adaptor" nil (ax d ae1 p t er0))
("adapts" nil (ax d ae1 p t s))
("adar" nil (ax d aa1 r))
("aday" nil (ax d ey1))
("adaza" nil (ax d aa1 z ax))
("adcock" nil (ax d k aa1 k))
("adcox" nil (ax d k aa1 k s))
("add" nil (ae1 d))
("addair" nil (ax d eh1 r))
("addams" nil (ae1 d ax m z))
("added" nil (ae1 d ax d))
("addeo" nil (aa1 d iy0 ow0))
("adder" nil (ae1 d er0))
("adderley" nil (ax d er1 l iy0))
("addicks" nil (ae1 d ih0 k s))
("addict" v (ax d ih1 k t))
("addict" n (ah1 d ih0 k t))
("addicted" nil (ax d ih1 k t ax d))
("addicting" nil (ax d ih1 k t ih0 ng))
("addiction" nil (ax d ih1 k sh ax n))
("addictions" nil (ax d ih1 k sh ax n z))
("addictive" nil (ax d ih1 k t ih0 v))
("addicts" v (ax d ih1 k t s))
("addicts" n (ah1 d ih0 k t s))
("addidas" nil (ax d iy1 d ax s))
("addie" nil (ae1 d iy0))
("adding" nil (ae1 d ih0 ng))
("addington" nil (ae1 d ih0 ng t ax n))
("addis" nil (aa1 d ih0 s))
("addison" nil (ae1 d ax s ax n))
("addition" nil (ax d ih1 sh ax n))
("additional" nil (ax d ih1 sh ax n ax l))
("additionally" nil (ax d ih1 sh ax n ax l iy0))
("additions" nil (ax d ih1 sh ax n z))
("additive" nil (ae1 d ax t ih0 v))
("additives" nil (ae1 d ax t ih0 v z))
("addled" nil (ae1 d ax l d))
("addleman" nil (ae1 d ax l m ax n))
("address" v (ae1 d r eh1 s))
("address" n (ax d r eh1 s))
("addressable" nil (ax d r eh1 s ax b ax l))
("addressed" nil (ax d r eh1 s t))
("addressee" nil (ae1 d r eh0 s iy1))
("addresses" nil (ae1 d r eh1 s ih0 z))
("addressing" nil (ax d r eh1 s ih0 ng))
("adds" nil (ae1 d z))
("adduci" nil (aa0 d uw1 ch iy0))
("addwest" nil (ae1 d w eh1 s t))
("addy" nil (ae1 d iy0))
("ade" nil (ey1 d))
("adee" nil (ax d iy1))
("adel" nil (ax d eh1 l))
("adela" nil (ax d eh1 l ax))
("adelaar" nil (ae1 d ax l aa1 r))
("adelaide" nil (ae1 d ax l ey1 d))
("adelbert" nil (ax d eh1 l b er0 t))
("adele" nil (ax d eh1 l ey0))
("adeline" nil (ae1 d ax l ay1 n))
("adelizzi" nil (ae1 d ax l iy1 z iy0))
("adell" nil (ax d eh1 l))
("adelle" nil (ax d eh1 l))
("adelman" nil (ae1 d ax l m ax n))
("adelmann" nil (ae1 d ax l m ax n))
("adelpha" nil (ax d eh1 l f ax))
("adelphia" nil (ax d eh1 l f iy0 ax))
("adelsberger" nil (ae1 d ih0 l s b er0 g er0))
("adelson" nil (ae1 d ax l s ax n))
("adelstein" nil (ae1 d ax l s t ay0 n))
("aden" nil (ey1 d ax n))
("adena" nil (ae1 d ih0 n ax))
("adenauer" nil (ey1 d ax n aw1 r))
("adenine" nil (ae1 d ax n iy1 n))
("adenoids" nil (ae1 d ax n oy1 d z))
("adept" nil (ax d eh1 p t))
("adequacy" nil (ae1 d ax k w ax s iy0))
("adequate" nil (ae1 d ax k w ax t))
("adequately" nil (ae1 d ax k w ax t l iy0))
("ader" nil (ey1 d er0))
("aderhold" nil (ae1 d er0 hh ow0 l d))
("aderholt" nil (ae1 d er0 hh ow0 l t))
("aderman" nil (ae1 d er0 m ax n))
("ades" nil (ey1 d z))
("adey" nil (ey1 d iy0))
("adger" nil (ae1 jh er0))
("adhere" nil (ax d hh ih1 r))
("adhered" nil (ae0 d hh ih1 r d))
("adherence" nil (ax d hh ih1 r ax n s))
("adherent" nil (ax d hh ih1 r ax n t))
("adherents" nil (ae0 d hh ih1 r ax n t s))
("adheres" nil (ax d hh ih1 r z))
("adhering" nil (ax d hh ih1 r ih0 ng))
("adhesive" nil (ae0 d hh iy1 s ih0 v))
("adhesives" nil (ae0 d hh iy1 s ih0 v z))
("adia" nil (aa1 d iy0 ax))
("adidas" nil (ax d iy1 d ax s))
("adieu" nil (ax d uw1))
("adin" nil (ax d ih1 n))
("adina" nil (aa0 d iy1 n ax))
("adine" nil (aa0 d iy1 n iy0))
("adinolfi" nil (aa0 d iy0 n ow1 l f iy0))
("adios" nil (aa1 d iy0 ow1 s))
("adipose" nil (ae1 d ax p ow1 s))
("adirondack" nil (ae1 d er0 aa1 n d ae0 k))
("adjacent" nil (ax jh ey1 s ax n t))
("adjective" nil (ae1 jh ih0 k t ih0 v))
("adjectives" nil (ae1 jh ax k t ih0 v z))
("adjoin" nil (ax jh oy1 n))
("adjoining" nil (ax jh oy1 n ih0 ng))
("adjoins" nil (ax jh oy1 n z))
("adjourn" nil (ax jh er1 n))
("adjourned" nil (ax jh er1 n d))
("adjourning" nil (ax jh er1 n ih0 ng))
("adjournment" nil (ax jh er1 n m ax n t))
("adjourns" nil (ax jh er1 n z))
("adjudged" nil (ax jh ah1 jh d))
("adjudicate" nil (ax jh uw1 d ih0 k ey1 t))
("adjudicated" nil (ax jh uw1 d ax k ey1 t ih0 d))
("adjudicating" nil (ax jh uw1 d ih0 k ey1 t ih0 ng))
("adjudication" nil (ax jh uw1 d ax k ey1 sh ax n))
("adjunct" nil (ae1 jh ah1 ng k t))
("adjuncts" nil (ae1 jh ah1 ng k t s))
("adjust" nil (ax jh ah1 s t))
("adjustable" nil (ax jh ah1 s t ax b ax l))
("adjustables" nil (ax jh ah1 s t ax b ax l z))
("adjusted" nil (ax jh ah1 s t ax d))
("adjuster" nil (ax jh ah1 s t er0))
("adjusters" nil (ax jh ah1 s t er0 z))
("adjusting" nil (ax jh ah1 s t ih0 ng))
("adjustment" nil (ax jh ah1 s t m ax n t))
("adjustments" nil (ax jh ah1 s t m ax n t s))
("adjusts" nil (ax jh ah1 s t s))
("adjutant" nil (ae1 jh ax t ax n t))
("adkins" nil (ae1 d k ih0 n z))
("adkinson" nil (ae1 d k ih0 n s ax n))
("adkison" nil (ae1 d k ih0 s ax n))
("adkisson" nil (ae1 d k ih0 s ax n))
("adlai" nil (aa0 d l aa1 iy0))
("adler" nil (ae1 d l er0))
("adley" nil (ae1 d l iy0))
("adlon" nil (ae1 d l aa0 n))
("adman" nil (ae1 d m ax n))
("admen" nil (ae1 d m ax n))
("administer" nil (ax d m ih1 n ax s t er0))
("administered" nil (ax d m ih1 n ax s t er0 d))
("administering" nil (ae0 d m ih1 n ih0 s t er0 ih0 ng))
("administers" nil (ae0 d m ih1 n ih0 s t er0 z))
("administrate" nil (ae0 d m ih1 n ih0 s t r ey1 t))
("administrating" nil (ax d m ih1 n ax s t r ey1 t ih0 ng))
("administration" nil (ae0 d m ih1 n ih0 s t r ey1 sh ax n))
("administrations" nil (ae0 d m ih1 n ih0 s t r ey1 sh ax n z))
("administrative" nil (ax d m ih1 n ax s t r ey1 t ih0 v))
("administratively" nil (ae0 d m ih1 n ax s t r ey1 t ih0 v l iy0))
("administrator" nil (ax d m ih1 n ax s t r ey1 t er0))
("administrators" nil (ae0 d m ih1 n ih0 s t r ey1 t er0 z))
("adminstration" nil (ae1 d m ih0 n s t r ey1 sh ax n))
("adminstrative" nil (ae0 d m ih1 n s t r ax t ih0 v))
("admira" nil (ae0 d m ay1 r ax))
("admirable" nil (ae1 d m er0 ax b ax l))
("admirably" nil (ae1 d m er0 ax b l iy0))
("admiral" nil (ae1 d m er0 ax l))
("admirals" nil (ae1 d m er0 ax l z))
("admiralty" nil (ae1 d m er0 ax l t iy0))
("admiration" nil (ae1 d m er0 ey1 sh ax n))
("admire" nil (ae0 d m ay1 r))
("admired" nil (ax d m ay1 er0 d))
("admirer" nil (ae0 d m ay1 r er0))
("admirers" nil (ax d m ay1 r er0 z))
("admires" nil (ae0 d m ay1 r z))
("admiring" nil (ae0 d m ay1 r ih0 ng))
("admiringly" nil (ae0 d m ay1 r ih0 ng l iy0))
("admissibility" nil (ax d m ih1 s ax b ih1 l ax t iy0))
("admissible" nil (ax d m ih1 s ax b ax l))
("admission" nil (ae0 d m ih1 sh ax n))
("admissions" nil (ae0 d m ih1 sh ax n z))
("admit" nil (ax d m ih1 t))
("admits" nil (ax d m ih1 t s))
("admittance" nil (ax d m ih1 t ax n s))
("admitted" nil (ax d m ih1 t ax d))
("admittedly" nil (ae0 d m ih1 t ih0 d l iy0))
("admitting" nil (ae0 d m ih1 t ih0 ng))
("admonish" nil (ae0 d m aa1 n ih0 sh))
("admonished" nil (ax d m aa1 n ih0 sh t))
("admonishes" nil (ae0 d m aa1 n ih0 sh ih0 z))
("admonishing" nil (ae0 d m aa1 n ih0 sh ih0 ng))
("admonition" nil (ae1 d m ax n ih1 sh ax n))
("admonitions" nil (ae1 d m ax n ih1 sh ax n z))
("adnan" nil (ae1 d n ax n))
("adney" nil (ae1 d n iy0))
("ado" nil (ax d uw1))
("adobe" nil (ax d ow1 b iy0))
("adolescence" nil (ae1 d ax l eh1 s ax n s))
("adolescent" nil (ae1 d ax l eh1 s ax n t))
("adolescents" nil (ae1 d ax l eh1 s ax n t s))
("adolf" nil (ey1 d aa0 l f))
("adolfo" nil (ax d aa1 l f ow1))
("adolph" nil (ey1 d ao0 l f))
("adolpha" nil (aa0 d ow1 l f ax))
("adolphson" nil (ae1 d ow0 l f s ax n))
("adon" nil (aa0 d ao1 n))
("adonia" nil (aa0 d ow1 n iy0 ax))
("adopt" nil (ax d aa1 p t))
("adoptable" nil (ax d aa1 p t ax b ax l))
("adopted" nil (ax d aa1 p t ax d))
("adoptees" nil (ax d aa1 p t iy1 z))
("adopting" nil (ax d aa1 p t ih0 ng))
("adoption" nil (ax d aa1 p sh ax n))
("adoptions" nil (ax d aa1 p sh ax n z))
("adoptive" nil (ax d aa1 p t ih0 v))
("adopts" nil (ax d aa1 p t s))
("adora" nil (aa0 d ao1 r ax))
("adorabelle" nil (ae1 d er0 ax b ax l))
("adorable" nil (ax d ao1 r ax b ax l))
("adoration" nil (ae1 d er0 ey1 sh ax n))
("adore" nil (ax d ao1 r))
("adored" nil (ax d ao1 r d))
("adoree" nil (ae0 d er0 iy1))
("adores" nil (ax d ao1 r z))
("adoring" nil (ax d ao1 r ih0 ng))
("adorn" nil (ax d ao1 r n))
("adorna" nil (aa0 d ao1 r n ax))
("adorned" nil (ax d ao1 r n d))
("adornment" nil (ax d ao1 r n m ax n t))
("adorno" nil (aa0 d ao1 r n ow0))
("adorns" nil (ax d ao1 r n z))
("adra" nil (ey1 d r ax))
("adragna" nil (aa0 d r aa1 g n ax))
("adrda" nil (ey1 d er0 d ax))
("adrea" nil (aa1 d r iy0 ax))
("adrenal" nil (ax d r iy1 n ax l))
("adrenalin" nil (ax d r eh1 n ax l ih0 n))
("adrenaline" nil (ax d r eh1 n ax l ax n))
("adria" nil (aa1 d r iy0 ax))
("adrian" nil (ey1 d r iy0 ax n))
("adriana" nil (ey1 d r iy0 ae1 n ax))
("adriance" nil (aa0 d r iy1 ax n s))
("adriano" nil (aa0 d r iy0 aa1 n ow0))
("adriatic" nil (ey1 d r iy0 ae1 t ih0 k))
("adriel" nil (ax d r iy1 l))
("adrienne" nil (aa0 d r iy0 eh1 n))
("adrift" nil (ax d r ih1 f t))
("adroit" nil (ax d r oy1 t))
("adroitly" nil (ax d r oy1 t l iy0))
("ads" nil (ae1 d z))
("adsit" nil (ae1 d s ih0 t))
("adss" nil (ae1 d s))
("adtec" nil (ae1 d t eh1 k))
("adulation" nil (ae1 jh ax l ey1 sh ax n))
("adult" nil (ax d ah1 l t))
("adulterated" nil (ax d ah1 l t er0 ey1 t ih0 d))
("adulterer" nil (ax d ah1 l t er0 er0))
("adulterers" nil (ax d ah1 l t er0 er0 z))
("adulterous" nil (ax d ah1 l t er0 ax s))
("adultery" nil (ax d ah1 l t er0 iy0))
("adulthood" nil (ax d ah1 l t hh uh1 d))
("adults" nil (ax d ah1 l t s))
("advance" nil (ax d v ae1 n s))
("advanced" nil (ax d v ae1 n s t))
("advancement" nil (ax d v ae1 n s m ax n t))
("advancements" nil (ax d v ae1 n s m ax n t s))
("advancers" nil (ax d v ae1 n s er0 z))
("advances" nil (ax d v ae1 n s ax z))
("advancing" nil (ax d v ae1 n s ih0 ng))
("advanta" nil (ae0 d v ae1 n t ax))
("advantage" nil (ae0 d v ae1 n t ih0 jh))
("advantaged" nil (ae0 d v ae1 n t ih0 jh d))
("advantageous" nil (ae1 d v ax n t ey1 jh ax s))
("advantages" nil (ae0 d v ae1 n t ih0 jh ih0 z))
("advantest" nil (ae0 d v ae1 n t ax s t))
("advection" nil (ae0 d v eh1 k sh ax n))
("advent" nil (ae1 d v eh1 n t))
("adventist" nil (ae1 d v eh1 n t ih0 s t))
("adventists" nil (ae1 d v eh1 n t ih0 s t s))
("adventure" nil (ae0 d v eh1 n ch er0))
("adventurer" nil (ae0 d v eh1 n ch er0 er0))
("adventurers" nil (ae0 d v eh1 n ch er0 er0 z))
("adventures" nil (ae0 d v eh1 n ch er0 z))
("adventuresome" nil (ae0 d v eh1 n ch er0 s ax m))
("adventurism" nil (ae0 d v eh1 n ch er0 ih1 z ax m))
("adventurous" nil (ae0 d v eh1 n ch er0 ax s))
("adverbial" nil (ae0 d v er1 b iy0 ax l))
("adverbs" nil (ae1 d v er0 b z))
("adversarial" nil (ae1 d v er0 s eh1 r iy0 ax l))
("adversaries" nil (ae1 d v er0 s eh1 r iy0 z))
("adversary" nil (ae1 d v er0 s eh1 r iy0))
("adverse" nil (ae0 d v er1 s))
("adversely" nil (ae0 d v er1 s l iy0))
("adversity" nil (ae0 d v er1 s ih0 t iy0))
("advertise" nil (ae1 d v er0 t ay1 z))
("advertised" nil (ae1 d v er0 t ay1 z d))
("advertisement" nil (ax d v er1 t ax z m ax n t))
("advertisements" nil (ae1 d v er0 t ay1 z m ax n t s))
("advertiser" nil (ae1 d v er0 t ay1 z er0))
("advertisers" nil (ae1 d v er0 t ay1 z er0 z))
("advertises" nil (ae1 d v er0 t ay1 z ih0 z))
("advertising" nil (ae1 d v er0 t ay1 z ih0 ng))
("advertorial" nil (ae1 d v er0 t ao1 r iy0 ax l))
("advertorials" nil (ae1 d v er0 t ao1 r iy0 ax l z))
("advest" nil (ae1 d v eh0 s t))
("advice" nil (ae0 d v ay1 s))
("advil" nil (ae1 d v ih1 l))
("advisability" nil (ae1 d v ay1 z ax b ih1 l ih0 t iy0))
("advisable" nil (ax d v ay1 z ax b ax l))
("advise" nil (ae0 d v ay1 z))
("advised" nil (ae0 d v ay1 z d))
("advisedly" nil (ae0 d v ay1 z ax d l iy0))
("advisement" nil (ax d v ay1 z m ax n t))
("adviser" nil (ae0 d v ay1 z er0))
("advisers" nil (ae0 d v ay1 z er0 z))
("advises" nil (ae0 d v ay1 z ih0 z))
("advising" nil (ae0 d v ay1 z ih0 ng))
("advisor" nil (ae0 d v ay1 z er0))
("advisories" nil (ax d v ay1 z er0 iy0 z))
("advisors" nil (ae0 d v ay1 z er0 z))
("advisory" nil (ae0 d v ay1 z er0 iy0))
("advo" nil (ae1 d v ow0))
("advocacy" nil (ae1 d v ax k ax s iy0))
("advocate" n (ae1 d v ax k ax t))
("advocate" v (ae1 d v ax k ey1 t))
("advocated" nil (ae1 d v ax k ey1 t ax d))
("advocates" n (ae1 d v ax k ax t s))
("advocates" v (ae1 d v ax k ey1 t s))
("advocating" nil (ae1 d v ax k ey1 t ih0 ng))
("advocation" nil (ae1 d v ax k ey1 sh ax n))
("adweek" nil (ae1 d w iy0 k))
("adwell" nil (ax d w eh1 l))
("ady" nil (ey1 d iy0))
("adz" nil (ae1 d z))
("ae" nil (ey1))
("aegean" nil (ih0 jh iy1 ax n))
("aegerter" nil (eh1 g er0 t er0))
("aegis" nil (iy1 jh ax s))
("aegon" nil (ey1 g aa0 n))
("aeltus" nil (ae1 l t ax s))
("aeneas" nil (ae1 n iy0 ax s))
("aequitron" nil (ey1 k w ih0 t r aa0 n))
("aer" nil (eh1 r))
("aerial" nil (eh1 r iy0 ax l))
("aeriens" nil (eh1 r iy0 ax n z))
("aeritalia" nil (eh1 r ih0 t ae1 l y ax))
("aero" nil (eh1 r ow0))
("aerobatics" nil (eh1 r ax b ae1 t ih0 k s))
("aerobic" nil (eh0 r ow1 b ih0 k))
("aerobically" nil (eh0 r ow1 b ih0 k l iy0))
("aerobics" nil (er0 ow1 b ih0 k s))
("aerodrome" nil (eh1 r ax d r ow1 m))
("aerodynamic" nil (eh1 r ow0 d ay0 n ae1 m ih0 k))
("aerodynamically" nil (eh1 r ow0 d ay0 n ae1 m ih0 k l iy0))
("aerodynamics" nil (eh1 r ow0 d ay0 n ae1 m ih0 k s))
("aerodyne" nil (eh1 r ax d ay1 n))
("aeroflot" nil (eh1 r ow0 f l aa1 t))
("aerojet" nil (eh1 r ow0 jh eh1 t))
("aerolift" nil (eh1 r ow0 l ih1 f t))
("aerolineas" nil (eh1 r ow0 l ih1 n iy0 ax s))
("aeromexico" nil (eh1 r ow0 m eh1 k s ih0 k ow1))
("aeronautical" nil (eh1 r ax n aa1 t ax k ax l))
("aeronauticas" nil (eh1 r ow0 n ao1 t ih0 k ax s))
("aeronautics" nil (eh1 r ax n ao1 t ih0 k s))
("aeroquip" nil (eh1 r ax k w ih1 p))
("aerosmith" nil (eh1 r ow0 s m ih1 th))
("aerosol" nil (eh1 r ax s aa1 l))
("aerospace" nil (eh1 r ow0 s p ey1 s))
("aerospatiale" nil (eh1 r ow0 s p aa1 s iy0 aa1 l))
("aerostar" nil (eh1 r ow0 s t aa1 r))
("aerostars" nil (eh1 r ow0 s t aa1 r z))
("aerotech" nil (eh1 r ow0 t eh1 k))
("aerts" nil (eh1 r t s))
("aeschliman" nil (eh1 sh l iy0 m ax n))
("aesthete" nil (eh1 s th iy0 t))
("aesthetic" nil (eh0 s th eh1 t ih0 k))
("aesthetically" nil (eh0 s th eh1 t ih0 k l iy0))
("aesthetics" nil (eh0 s th eh1 t ih0 k s))
("aetna" nil (eh1 t n ax))
("afanasyev" nil (ae1 f ax n ey1 s iy0 eh0 v))
("afar" nil (ax f aa1 r))
("affable" nil (ae1 f ax b ax l))
("affair" nil (ax f eh1 r))
("affairs" nil (ax f eh1 r z))
("affect" nil (ax f eh1 k t))
("affectation" nil (ae1 f eh0 k t ey1 sh ax n))
("affected" nil (ax f eh1 k t ax d))
("affecting" nil (ax f eh1 k t ih0 ng))
("affection" nil (ax f eh1 k sh ax n))
("affectionate" nil (ax f eh1 k sh ax n ax t))
("affectionately" nil (ax f eh1 k sh ax n ax t l iy0))
("affections" nil (ax f eh1 k sh ax n z))
("affects" nil (ax f eh1 k t s))
("affeldt" nil (ae1 f ih0 l t))
("afferent" nil (ae1 f er0 ax n t))
("affidavit" nil (ae1 f ax d ey1 v ax t))
("affidavits" nil (ae1 f ih0 d ey1 v ih0 t s))
("affiliate" nil (ax f ih1 l iy0 ey1 t))
("affiliated" nil (ax f ih1 l iy0 ey1 t ax d))
("affiliates" nil (ax f ih1 l iy0 ax t s))
("affiliating" nil (ax f ih1 l iy0 ey1 t ih0 ng))
("affiliation" nil (ax f ih1 l iy0 ey1 sh ax n))
("affiliations" nil (ax f ih1 l iy0 ey1 sh ax n z))
("affinities" nil (ax f ih1 n ax t iy0 z))
("affinity" nil (ax f ih1 n ax t iy0))
("affirm" nil (ax f er1 m))
("affirmation" nil (ae1 f er0 m ey1 sh ax n))
("affirmations" nil (ae1 f er0 m ey1 sh ax n z))
("affirmative" nil (ax f er1 m ax t ih0 v))
("affirmatively" nil (ax f er1 m ax t ih0 v l iy0))
("affirmed" nil (ax f er1 m d))
("affirming" nil (ax f er1 m ih0 ng))
("affirms" nil (ax f er1 m z))
("affix" n (ae1 f ih0 k s))
("affix" v (ae0 f ih1 k s))
("affixed" nil (ax f ih1 k s t))
("affixes" vl (ax f ih1 k s ih0 z))
("affixes" n (ah1 f ih0 k s ih0 z))
("affixing" nil (ax f ih1 k s ih0 ng))
("affleck" nil (ae1 f l ih0 k))
("afflerbach" nil (ae1 f l er0 b aa1 k))
("afflict" nil (ax f l ih1 k t))
("afflicted" nil (ax f l ih1 k t ax d))
("afflicting" nil (ax f l ih1 k t ih0 ng))
("affliction" nil (ax f l ih1 k sh ax n))
("afflictions" nil (ax f l ih1 k sh ax n z))
("afflicts" nil (ax f l ih1 k t s))
("affluence" nil (ae1 f l uw0 ax n s))
("affluent" nil (ae1 f l uw0 ax n t))
("affolter" nil (ae1 f ow0 l t er0))
("afford" nil (ax f ao1 r d))
("affordability" nil (ax f ao1 r d ax b ih1 l ax t iy0))
("affordable" nil (ax f ao1 r d ax b ax l))
("afforded" nil (ax f ao1 r d ax d))
("affording" nil (ax f ao1 r d ih0 ng))
("affords" nil (ax f ao1 r d z))
("affricates" nil (ae1 f r ax k ax t s))
("affrication" nil (ae1 f r ax k ey1 sh ax n))
("affront" nil (ax f r ah1 n t))
("affronted" nil (ax f r ah1 n t ih0 d))
("affronts" nil (ax f r ah1 n t s))
("affymax" nil (ae1 f iy0 m ae1 k s))
("afghan" nil (ae1 f g ae1 n))
("afghani" nil (ae0 f g aa1 n iy0))
("afghanis" nil (ae0 f g ae1 n iy0 z))
("afghanistan" nil (ae0 f g ae1 n ax s t ae1 n))
("afghans" nil (ae1 f g ae1 n z))
("afheldt" nil (ae1 f eh1 l t))
("aficionado" nil (ax f iy1 sh y ax n aa1 d ow1))
("aficionados" nil (ax f ih1 sh ax n aa1 d ow0 z))
("afield" nil (ax f iy1 l d))
("afire" nil (ax f ay1 r))
("aflame" nil (ax f l ey1 m))
("aflatoxin" nil (ae1 f l ax t aa1 k s ih0 n))
("afloat" nil (ax f l ow1 t))
("aflutter" nil (ax f l ah1 t er0))
("afonso" nil (ax f aa1 n s ow0))
("afoot" nil (ax f uh1 t))
("aforementioned" nil (ax f ao1 r m eh1 n sh ax n d))
("aforesaid" nil (ax f ao1 r s eh1 d))
("afoul" nil (ax f aw1 l))
("afraid" nil (ax f r ey1 d))
("afresh" nil (ax f r eh1 sh))
("africa" nil (ae1 f r ax k ax))
("african" nil (ae1 f r ax k ax n))
("africanized" nil (ae1 f r ax k ax n ay1 z d))
("africans" nil (ae1 f r ax k ax n z))
("afrikaans" nil (ae1 f r ax k aa1 n z))
("afrikaner" nil (ae1 f r ax k aa1 n er0))
("afrikanerdom" nil (ae1 f r ax k aa1 n er0 d ax m))
("afrikaners" nil (ae1 f r ih0 k aa1 n er0 z))
("afro" nil (ae1 f r ow0))
("afsane" nil (aa0 f s aa1 n ey1))
("afshar" nil (ae1 f sh er0))
("aft" nil (ae1 f t))
("after" nil (ae1 f t er0))
("afterburners" nil (ae1 f t er0 b er1 n er0 z))
("aftereffect" nil (ae1 f t er0 ax f eh1 k t))
("aftereffects" nil (ae1 f t er0 ax f eh1 k t s))
("afterglow" nil (ae1 f t er0 g l ow1))
("afterimage" nil (ae1 f t er0 ih1 m ih0 jh))
("afterlife" nil (ae1 f t er0 l ay1 f))
("aftermarket" nil (ae1 f t er0 m aa1 r k ih0 t))
("aftermath" nil (ae1 f t er0 m ae1 th))
("afternoon" nil (ae1 f t er0 n uw1 n))
("afternoons" nil (ae1 f t er0 n uw1 n z))
("aftershock" nil (ae1 f t er0 sh aa1 k))
("aftershocks" nil (ae1 f t er0 sh aa1 k s))
("aftertaste" nil (ae1 f t er0 t ey1 s t))
("aftertax" nil (ae1 f t er0 t ae1 k s))
("afterthought" nil (ae1 f t er0 th aa1 t))
("afterward" nil (ae1 f t er0 w er0 d))
("afterwards" nil (ae1 f t er0 w er0 d z))
("afula" nil (ax f uw1 l ax))
("ag" nil (ae1 g))
("aga" nil (aa1 g ax))
("agache" nil (ae1 g ae0 ch))
("again" nil (ax g eh1 n))
("against" nil (ax g eh1 n s t))
("agan" nil (ey1 g ax n))
("aganbegyan" nil (ae1 g ax n b eh1 g y ax n))
("agans" nil (aa1 g aa0 n z))
("agape" nil (ax g ey1 p))
("agar" nil (ey1 g er0))
("agard" nil (ae1 g er0 d))
("agarwal" nil (aa1 g aa0 r w aa0 l))
("agassi" nil (ae1 g ax s iy0))
("agassiz" nil (ax g ae1 s ih0 z))
("agate" nil (ae1 g ax t))
("agates" nil (ae1 g ax t s))
("agatha" nil (ae1 g ax th ax))
("agco" nil (ae1 g k ow1))
("age" nil (ey1 jh))
("aged" v (ey1 jh d))
("aged" j (ey1 jh ih0 d))
("agee" nil (ey1 jh iy1))
("ageless" nil (ey1 jh l ax s))
("agence" nil (ae1 jh ax n s))
("agencies" nil (ey1 jh ax n s iy0 z))
("agency" nil (ey1 jh ax n s iy0))
("agenda" nil (ax jh eh1 n d ax))
("agendas" nil (ax jh eh1 n d ax z))
("agent" nil (ey1 jh ax n t))
("agents" nil (ey1 jh ax n t s))
("ager" nil (ey1 jh er0))
("ageratum" nil (ax jh eh1 r ax t ax m))
("ageratums" nil (ax jh eh1 r ax t ax m z))
("agers" nil (ey1 jh er0 z))
("agerton" nil (ey1 g er0 t ax n))
("ages" nil (ey1 jh ax z))
("agfa" nil (ae1 g f ax))
("aggarwal" nil (ax g aa1 r w ax l))
("aggie" nil (ae1 g iy0))
("aggies" nil (ae1 g iy0 z))
("agglomeration" nil (ax g l aa1 m er0 ey1 sh ax n))
("aggrandizement" nil (ae1 g r ax n d ay1 z m ax n t))
("aggrandizing" nil (ae1 g r ax n d ay1 z ih0 ng))
("aggravate" nil (ae1 g r ax v ey1 t))
("aggravated" nil (ae1 g r ax v ey1 t ax d))
("aggravates" nil (ae1 g r ax v ey1 t s))
("aggravating" nil (ae1 g r ax v ey1 t ih0 ng))
("aggravation" nil (ae1 g r ax v ey1 sh ax n))
("aggregate" v (ae1 g r ax g ax t))
("aggregate" n (ae1 g r ax g ih0 t))
("aggregated" nil (ae1 g r ax g ey1 t ax d))
("aggregates" n (ae1 g r ax g ih0 t s))
("aggregates" v (ae1 g r ax g ax t s))
("aggression" nil (ax g r eh1 sh ax n))
("aggressions" nil (ax g r eh1 sh ax n z))
("aggressive" nil (ax g r eh1 s ih0 v))
("aggressively" nil (ax g r eh1 s ih0 v l iy0))
("aggressiveness" nil (ax g r eh1 s ih0 v n ax s))
("aggressor" nil (ax g r eh1 s er0))
("aggressors" nil (ax g r eh1 s er0 z))
("aggrieved" nil (ax g r iy1 v d))
("aggy" nil (ae1 g iy0))
("aghast" nil (ax g ae1 s t))
("aghazadeh" nil (ae1 g ax z aa1 d eh1))
("agie" nil (ae1 g iy0))
("agildo" nil (ax g ih1 l d ow0))
("agile" nil (ae1 jh ax l))
("agility" nil (ax jh ih1 l ax t iy0))
("agin" nil (aa0 jh iy1 n))
("agincourt" nil (ae1 jh ax n k ao1 r t))
("aging" nil (ey1 jh ih0 ng))
("agins" nil (ey1 g ih0 n z))
("agip" nil (ey1 g ih0 p))
("agitate" nil (ae1 jh ax t ey1 t))
("agitated" nil (ae1 jh ax t ey1 t ax d))
("agitating" nil (ae1 jh ax t ey1 t ih0 ng))
("agitation" nil (ae1 jh ax t ey1 sh ax n))
("agitator" nil (ae1 jh ax t ey1 t er0))
("agitators" nil (ae1 jh ih0 t ey1 t er0 z))
("agius" nil (ey1 jh iy0 ih0 s))
("agleam" nil (ax g l iy1 m))
("agler" nil (ae1 g l er0))
("aglitter" nil (ax g l ih1 t er0))
("aglow" nil (ax g l ow1))
("agna" nil (ae1 g n ax))
("agne" nil (ey1 n y))
("agnella" nil (ae1 g n eh1 l ax))
("agnelli" nil (ae1 g n eh1 l iy0))
("agnello" nil (ae1 g n eh1 l ow0))
("agner" nil (ae1 g n er0))
("agnes" nil (ae1 g n ih0 s))
("agneta" nil (aa0 g n eh1 t ax))
("agnew" nil (ae1 g n uw0))
("agnico" nil (ae1 g n ih0 k ow1))
("agnor" nil (ae1 g n er0))
("agnos" nil (ae1 g n ow0 s))
("agnostic" nil (ae0 g n aa1 s t ih0 k))
("ago" nil (ax g ow1))
("agog" nil (ax g aa1 g))
("agoglia" nil (ax g aa1 g l iy0 ax))
("agonies" nil (ae1 g ax n iy0 z))
("agonists" nil (ae1 g ax n ih0 s t s))
("agonize" nil (ae1 g ax n ay1 z))
("agonized" nil (ae1 g ax n ay1 z d))
("agonizes" nil (ae1 g ax n ay1 z ih0 z))
("agonizing" nil (ae1 g ax n ay0 z ih0 ng))
("agonizingly" nil (ae1 g ax n ay0 z ih0 ng l iy0))
("agony" nil (ae1 g ax n iy0))
("agora" nil (ae1 g er0 ax))
("agosta" nil (aa0 g ow1 s t ax))
("agostinelli" nil (aa0 g ow0 s t iy0 n eh1 l iy0))
("agostini" nil (aa0 g ow0 s t iy1 n iy0))
("agostino" nil (aa0 g ao0 s t iy1 n ow0))
("agosto" nil (aa0 g ow1 s t ow0))
("agoura" nil (ax g uw1 r ax))
("agouron" nil (ax g uw1 r aa0 n))
("agra" nil (ae1 g r ax))
("agrarian" nil (ax g r eh1 r iy0 ax n))
("agrarianism" nil (ax g r eh1 r iy0 ax n ih1 z ax m))
("agrawal" nil (ax g r ae1 w ax l))
("agree" nil (ax g r iy1))
("agreeable" nil (ax g r iy1 ax b ax l))
("agreed" nil (ax g r iy1 d))
("agreeement" nil (ax g r iy1 m ax n t))
("agreeing" nil (ax g r iy1 ih0 ng))
("agreement" nil (ax g r iy1 m ax n t))
("agreements" nil (ax g r iy1 m ax n t s))
("agrees" nil (ax g r iy1 z))
("agresource" nil (aa1 g r eh0 s ao1 r s))
("agressive" nil (ax g r eh1 s ih0 v))
("agresta" nil (aa0 g r eh1 s t ax))
("agresti" nil (aa0 g r eh1 s t iy0))
("agri" nil (ae1 g r iy0))
("agribusiness" nil (ae1 g r ax b ih1 z n ax s))
("agrico" nil (ax g r iy1 k ow0))
("agricola" nil (ae1 g r ih0 k ow1 l ax))
("agricole" nil (ae1 g r ih0 k ow1 l))
("agricultural" nil (ae1 g r ax k ah1 l ch er0 ax l))
("agriculturalist" nil (ae1 g r ax k ah1 l ch er0 ax l ax s t))
("agriculturally" nil (ae1 g r ih0 k ah1 l ch er0 ax l iy0))
("agriculture" nil (ae1 g r ih0 k ah1 l ch er0))
("agrifuels" nil (ae1 g r ax f y uw1 l z))
("agrippa" nil (ax g r ih1 p ax))
("agrivisor" nil (ae1 g r ax v ay1 z er0))
("agro" nil (ae1 g r ow0))
("agrochemical" nil (ae1 g r ow0 k eh1 m ax k ax l))
("agrochemicals" nil (ae1 g r ow0 k eh1 m ih0 k ax l z))
("agrokomerc" nil (ae1 g r ax k ow0 m er1 k))
("agronomist" nil (ax g r aa1 n ax m ih0 s t))
("agronomists" nil (ax g r aa1 n ax m ih0 s t s))
("aground" nil (ax g r aw1 n d))
("agrusa" nil (aa0 g r uw1 s ax))
("aguacate" nil (ae1 g w ax k ey1 t))
("aguado" nil (aa0 g w aa1 d ow0))
("aguayo" nil (aa0 g w ey1 ow0))
("agudelo" nil (aa0 g uw0 d ey1 l ow0))
("aguero" nil (aa0 g eh1 r ow0))
("aguiar" nil (aa1 g w y er0))
("aguila" nil (aa0 g w iy1 l ax))
("aguilar" nil (ae1 g ax l aa0 r))
("aguilera" nil (aa0 g w iy0 l eh1 r ax))
("aguillard" nil (ae1 g ih0 l er0 d))
("aguillon" nil (aa0 g w iy0 l ao1 n))
("aguinaga" nil (aa0 g uw0 iy0 n aa1 g ax))
("aguirra" nil (ax g w ih1 r ax))
("aguirre" nil (aa0 g w ih1 r ey0))
("agustin" nil (ax g ao1 s t ih0 n))
("ah" nil (aa1))
("aha" nil (aa1 hh aa1))
("ahab" nil (ey1 hh ae1 b))
("ahah" nil (aa1 hh aa0))
("aharon" nil (ae1 hh er0 aa0 n))
("ahart" nil (ax hh aa1 r t))
("ahead" nil (ax hh eh1 d))
("ahearn" nil (ax hh er1 n))
("aherin" nil (aa1 er0 ih0 n))
("ahern" nil (ax hh er1 n))
("aherne" nil (ax hh er1 n))
("ahh" nil (aa1))
("ahl" nil (aa1 l))
("ahlberg" nil (aa1 l b er0 g))
("ahlborn" nil (aa1 l b er0 n))
("ahlen" nil (ax l ey1 n))
("ahlers" nil (aa1 l er0 z))
("ahles" nil (ey1 ax l z))
("ahlf" nil (aa1 l f))
("ahlgren" nil (aa1 l g r eh0 n))
("ahlgrim" nil (aa1 l g r ih0 m))
("ahlin" nil (aa1 l ih0 n))
("ahlman" nil (aa1 l m ax n))
("ahlquist" nil (aa1 l k w ih1 s t))
("ahlstrand" nil (aa1 l s t r ax n d))
("ahlstrom" nil (aa1 l s t r ax m))
("ahmad" nil (aa1 m aa0 d))
("ahmadi" nil (aa0 m aa1 d iy0))
("ahmann" nil (aa1 m ax n))
("ahmanson" nil (aa1 m ax n s ax n))
("ahmed" nil (aa1 m ax d))
("ahn" nil (ae1 n))
("ahner" nil (aa1 n er0))
("aho" nil (aa1 hh ow0))
("ahola" nil (ae1 hh ax l ax))
("ahold" nil (ax hh ow1 l d))
("ahonen" nil (ax hh ow1 n ax n))
("ahr" nil (aa1 r))
("ahren" nil (aa1 r ax n))
("ahrendt" nil (aa1 r ih0 n t))
("ahrens" nil (aa1 r ih0 n z))
("ahs" nil (aa1 z))
("ahuja" nil (aa0 hh uw1 y ax))
("ahumada" nil (aa0 y uw0 m aa1 d ax))
("ai" nil (ay1))
("aichele" nil (ay1 k ax l))
("aicher" nil (ay1 k er0))
("aichi" nil (aa0 iy1 ch iy0))
("aickin" nil (ey1 k ih0 n))
("aid" nil (ey1 d))
("aida" nil (ay0 iy1 d ax))
("aidan" nil (aa0 iy1 d aa0 n))
("aide" nil (ey1 d))
("aided" nil (ey1 d ax d))
("aides" nil (ey1 d z))
("aidid" nil (ay1 d iy1 d))
("aiding" nil (ey1 d ih0 ng))
("aids" nil (ey1 d z))
("aiello" nil (ay1 eh1 l ow0))
("aigner" nil (ey1 n er0))
("aiguebelle" nil (ay1 g ax b eh1 l))
("aiken" nil (ey1 k ih0 n))
("aikens" nil (ey1 k ih0 n z))
("aikey" nil (ey1 k iy0))
("aikin" nil (aa0 iy1 k iy0 n))
("aikins" nil (ay1 k ih0 n z))
("aikman" nil (ey1 k m ax n))
("ailee" nil (ey1 l iy1))
("aileen" nil (ay0 l iy1 n))
("ailerons" nil (ey1 l er0 aa1 n z))
("ailes" nil (ay1 l z))
("ailey" nil (ey1 l iy0))
("ailing" nil (ey1 l ih0 ng))
("ailment" nil (ey1 l m ax n t))
("ailments" nil (ey1 l m ax n t s))
("ailor" nil (ey1 l er0))
("ails" nil (ey1 l z))
("ailsa" nil (ey1 l s ax))
("aim" nil (ey1 m))
("aime" nil (ey1 m))
("aimed" nil (ey1 m d))
("aimee" nil (ey1 m iy1))
("aiming" nil (ey1 m ih0 ng))
("aimless" nil (ey1 m l ax s))
("aimlessly" nil (ey1 m l ax s l iy0))
("aimone" nil (ey1 m ow1 n))
("aims" nil (ey1 m z))
("aine" nil (ey1 n))
("ainge" nil (ey1 ng))
("ainley" nil (ey1 n l iy0))
("ainsberg" nil (ey1 n z b er0 g))
("ainsley" nil (ey1 n s l iy0))
("ainslie" nil (ey1 n z l iy0))
("ainsworth" nil (ey1 n s w er0 th))
("aipac" nil (ay1 p ae1 k))
("air" nil (eh1 r))
("airbag" nil (eh1 r b ae1 g))
("airbags" nil (eh1 r b ae1 g z))
("airborne" nil (eh1 r b ao1 r n))
("airbus" nil (eh1 r b ax s))
("aircal" nil (eh1 r k aa0 l))
("airco" nil (eh1 r k ow0))
("aircoa" nil (eh1 r k ow1 ax))
("aircraft" nil (eh1 r k r ae1 f t))
("aircrafts" nil (eh1 r k r ae1 f t s))
("aird" nil (eh1 r d))
("airdrop" nil (eh1 r d r aa0 p))
("aired" nil (eh1 r d))
("airedale" nil (eh1 r d ey1 l))
("aires" nil (eh1 r iy0 z))
("airey" nil (eh1 r iy0))
("airfare" nil (eh1 r f eh1 r))
("airfares" nil (eh1 r f eh1 r z))
("airfield" nil (eh1 r f iy1 l d))
("airfields" nil (eh1 r f iy1 l d z))
("airflow" nil (eh1 r f l ow0))
("airfoil" nil (eh1 r f oy1 l))
("airfoils" nil (eh1 r f oy1 l z))
("airfone" nil (eh1 r f ow1 n))
("airframe" nil (eh1 r f r ey1 m))
("airfreight" nil (eh1 r f r ey1 t))
("airgas" nil (eh1 r g ae1 s))
("airglow" nil (eh1 r g l ow1))
("airhart" nil (eh1 r hh aa0 r t))
("airhead" nil (eh1 r hh eh1 d))
("airing" nil (eh1 r ih0 ng))
("airington" nil (eh1 r ih0 ng t ax n))
("airless" nil (eh1 r l ax s))
("airlia" nil (eh1 r l iy0 ax))
("airlie" nil (eh1 r l iy0))
("airlift" nil (eh1 r l ih1 f t))
("airlifted" nil (eh1 r l ih1 f t ih0 d))
("airlifting" nil (eh1 r l ih1 f t ih0 ng))
("airline" nil (eh1 r l ay1 n))
("airliner" nil (eh1 r l ay1 n er0))
("airliners" nil (eh1 r l ay1 n er0 z))
("airlines" nil (eh1 r l ay1 n z))
("airlink" nil (eh1 r l ih1 ng k))
("airmail" nil (eh1 r m ey1 l))
("airman" nil (eh1 r m ax n))
("airmen" nil (eh1 r m eh1 n))
("airmotive" nil (eh1 r m ow1 t ih0 v))
("airplane" nil (eh1 r p l ey1 n))
("airplanes" nil (eh1 r p l ey0 n z))
("airport" nil (eh1 r p ao1 r t))
("airports" nil (eh1 r p ao1 r t s))
("airpower" nil (eh1 r p aw1 er0))
("airs" nil (eh1 r z))
("airship" nil (eh1 r sh ih1 p))
("airships" nil (eh1 r sh ih1 p s))
("airspace" nil (eh1 r s p ey1 s))
("airspeed" nil (eh1 r s p iy1 d))
("airstrike" nil (eh1 r s t r ay1 k))
("airstrikes" nil (eh1 r s t r ay1 k s))
("airstrip" nil (eh1 r s t r ih1 p))
("airstrips" nil (eh1 r s t r ih1 p s))
("airtight" nil (eh1 r t ay1 t))
("airtime" nil (eh1 r t ay1 m))
("airtouch" nil (eh1 r t ah1 ch))
("airtran" nil (eh1 r t r ae1 n))
("airwave" nil (eh1 r w ey1 v))
("airwaves" nil (eh1 r w ey1 v z))
("airway" nil (eh1 r w ey1))
("airways" nil (eh1 r w ey1 z))
("airworthiness" nil (eh1 r w er1 dh iy0 n ax s))
("airworthy" nil (eh1 r w er1 dh iy0))
("airy" nil (eh1 r iy0))
("ais" nil (ay1 z))
("aisle" nil (ay1 l))
("aisles" nil (ay1 l z))
("aislinn" nil (ey1 s l ih0 n))
("aitchison" nil (ey1 ch ih0 s ax n))
("aithne" nil (ey1 th n))
("aitken" nil (ay1 t k ax n))
("aiton" nil (aa0 iy1 t ow0 n))
("aiwa" nil (ay1 w ax))
("aja" nil (ay1 ax))
("ajaj" nil (ax jh aa1 jh))
("ajami" nil (ey1 jh aa1 m iy0))
("ajar" nil (ax jh aa1 r))
("ajax" nil (ey1 jh ae1 k s))
("ajinomoto" nil (ax jh ih1 n ax m ow1 t ow0))
("akai" nil (ax k ay1))
("akali" nil (ax k aa1 l iy0))
("akamine" nil (ae1 k ax m ay1 n))
("akana" nil (aa0 k aa1 n ax))
("akard" nil (ae1 k er0 d))
("akashi" nil (ax k aa1 sh iy0))
("akbar" nil (ae1 k b er0))
("akc" nil (ae1 k))
("ake" nil (ey1 k))
("akel" nil (ax k eh1 l))
("akella" nil (ax k eh1 l ax))
("akens" nil (ey1 k ax n z))
("aker" nil (ae1 k er0))
("akerley" nil (ax k er1 l iy0))
("akerman" nil (ae1 k er0 m ax n))
("akers" nil (ey1 k er0 z))
("akerson" nil (ae1 k er0 s ax n))
("akey" nil (ax k iy1))
("akhtar" nil (ae1 k t er0))
("aki" nil (aa1 k iy0))
("akiba" nil (ax k iy1 b ax))
("akihito" nil (aa1 k iy0 hh iy1 t ow1))
("akimbo" nil (ax k ih1 m b ow1))
("akin" nil (ax k ih1 n))
("akins" nil (ax k ih1 n z))
("akio" nil (aa1 k iy0 ow0))
("akira" nil (ax k iy1 r ax))
("akita" nil (ax k iy1 t ax))
("akiyama" nil (aa0 k iy0 y aa1 m ax))
("akkadian" nil (ax k ey1 d iy0 ax n))
("akkerman" nil (ae1 k er0 m ax n))
("akre" nil (ae1 k er0))
("akridge" nil (ax k r ih1 jh))
("akron" nil (ae1 k r ax n))
("aksamit" nil (ae1 k s ax m ih0 t))
("aksler" nil (ae1 k s l er0))
("akst" nil (ae1 k s t))
("akzo" nil (ae1 k z ow0))
("al" nil (ae1 l))
("ala" nil (ey1 l ax))
("alabama" nil (ae1 l ax b ae1 m ax))
("alabaster" nil (ae1 l ax b ae1 s t er0))
("alachlor" nil (ax l ae1 k l er0))
("alacrity" nil (ax l ae1 k r ax t iy0))
("aladdin" nil (ax l ae1 d ih0 n))
("alafi" nil (ax l aa1 f iy0))
("alagem" nil (ae1 l ax jh eh0 m))
("alagna" nil (aa0 l aa1 g n ax))
("alagoas" nil (ae1 l ax g ow1 z))
("alai" nil (ax l ay1))
("alaimo" nil (ax l ey1 m ow0))
("alain" nil (ax l ey1 n))
("alaine" nil (ax l ey1 n))
("alair" nil (ax l eh1 r))
("alam" nil (ax l ae1 m))
("alamco" nil (ax l ae1 m k ow0))
("alameda" nil (ae1 l ax m iy1 d ax))
("alamein" nil (ae1 l ax m ay1 n))
("alamillo" nil (ae1 l ax m ih1 l ow0))
("alamito" nil (ae1 l ax m iy1 t ow0))
("alamitos" nil (ae1 l ax m iy1 t ow0 s))
("alamo" nil (ae1 l ax m ow0))
("alamos" nil (ae1 l ax m ow0 z))
("alan" nil (ae1 l ax n))
("alana" nil (aa0 l ae1 n ax))
("aland" nil (ae1 l ax n d))
("alane" nil (ax l ey1 n))
("alanis" nil (aa0 l aa1 n ih0 s))
("alaniz" nil (ae1 l ax n ih0 z))
("alann" nil (ae1 l ae0 n))
("alanna" nil (aa0 l aa1 n ax))
("alar" nil (ey1 l aa1 r))
("alarcon" nil (ax l aa1 r k aa1 n))
("alard" nil (ax l aa1 r d))
("alaric" nil (ae1 l er0 ih0 k))
("alarica" nil (aa0 l aa0 r iy1 k ax))
("alarice" nil (aa0 l aa1 r ih0 s))
("alarid" nil (ax l eh1 r ih0 d))
("alarie" nil (ax l eh1 r iy0))
("alarm" nil (ax l aa1 r m))
("alarmed" nil (ax l aa1 r m d))
("alarming" nil (ax l aa1 r m ih0 ng))
("alarmingly" nil (ax l aa1 r m ih0 ng l iy0))
("alarmist" nil (ax l aa1 r m ax s t))
("alarms" nil (ax l aa1 r m z))
("alas" nil (ax l ae1 s))
("alaska" nil (ax l ae1 s k ax))
("alaskamen" nil (ax l ae1 s k ax m ax n))
("alaskan" nil (ax l ae1 s k ax n))
("alaskans" nil (ax l ae1 s k ax n z))
("alastair" nil (ae1 l ax s t eh1 r))
("alaster" nil (ae1 l ax s t er0))
("alatorre" nil (aa0 l aa0 t ao1 r iy0))
("alayne" nil (ax l ey1 n))
("alba" nil (ae1 l b ax))
("albach" nil (ae1 l b aa0 k))
("albacore" nil (ae1 l b ax k ao1 r))
("alban" nil (aa1 l b ax n))
("albanese" nil (aa0 l b aa0 n ey1 z iy0))
("albani" nil (ae0 l b aa1 n iy0))
("albania" nil (ae0 l b ey1 n iy0 ax))
("albanian" nil (ae0 l b ey1 n iy0 ax n))
("albanians" nil (ae0 l b ey1 n iy0 ax n z))
("albano" nil (aa0 l b aa1 n ow0))
("albans" nil (ae1 l b ae0 n z))
("albany" nil (ao1 l b ax n iy0))
("albarado" nil (aa0 l b aa0 r aa1 d ow0))
("albarran" nil (ae1 l b ae1 r ax n))
("albatross" nil (ae1 l b ax t r aa1 s))
("albaugh" nil (ax l b ao1))
("albea" nil (ae1 l b iy0 ax))
("albee" nil (ax l b iy1))
("albeit" nil (ao0 l b iy1 ih0 t))
("albemarle" nil (ae1 l b ax m aa1 r l))
("alben" nil (ao1 l b ax n))
("alber" nil (ae1 l b er0))
("alberding" nil (ae1 l b er0 d ih0 ng))
("alberg" nil (ae1 l b er0 g))
("albergo" nil (aa0 l b eh1 r g ow0))
("alberico" nil (aa0 l b er0 iy1 k ow0))
("albern" nil (ae1 l b er0 n))
("albers" nil (ao1 l b er0 z))
("alberson" nil (ae1 l b er0 s ax n))
("albert" nil (ae1 l b er0 t))
("alberta" nil (ae0 l b er1 t ax))
("alberthal" nil (ae1 l b er0 t ao1 l))
("alberti" nil (aa0 l b eh1 r t iy0))
("albertina" nil (aa0 l b er0 t iy1 n ax))
("albertine" nil (ae1 l b er0 t iy1 n))
("albertini" nil (aa0 l b er0 t iy1 n iy0))
("alberto" nil (ae0 l b er1 t ow0))
("alberts" nil (ae1 l b er0 t s))
("albertsen" nil (ae1 l b er0 t s ax n))
("albertson" nil (ae1 l b er0 t s ax n))
("albertus" nil (ae0 l b er1 t ax s))
("albertville" nil (ae1 l b er0 t v ih1 l))
("alberty" nil (ax l b er1 t iy0))
("albie" nil (ao1 l b iy0))
("albin" nil (ae1 l b ih0 n))
("albina" nil (aa0 l b iy1 n ax))
("albini" nil (aa0 l b iy1 n iy0))
("albinia" nil (aa0 l b iy1 n iy0 ax))
("albino" nil (ae0 l b ay1 n ow1))
("albion" nil (ae1 l b iy0 ax n))
("albo" nil (ae1 l b ow0))
("albrecht" nil (ao1 l b r eh1 k t))
("albright" nil (ao1 l b r ay1 t))
("albritton" nil (ae1 l b r ih0 t aa0 n))
("albro" nil (ae1 l b r ow0))
("album" nil (ae1 l b ax m))
("albumin" nil (ae0 l b y uw1 m ax n))
("albums" nil (ae1 l b ax m z))
("albunex" nil (ae1 l b y uw1 n eh0 k s))
("albuquerque" nil (ae1 l b ax k er0 k iy0))
("albury" nil (ae1 l b eh0 r iy0))
("albus" nil (ae1 l b ax s))
("albuterol" nil (ae1 l b y uw1 t er0 ao0 l))
("alcala" nil (aa0 l k aa1 l ax))
("alcan" nil (ae1 l k ae1 n))
("alcantar" nil (ae1 l k ae1 n t er0))
("alcantara" nil (aa0 l k aa0 n t aa1 r ax))
("alcaraz" nil (aa0 l k aa1 r aa0 z))
("alcasa" nil (ae0 l k aa1 s ax))
("alcatel" nil (ae1 l k ax t eh1 l))
("alcee" nil (ae1 l s iy0))
("alchemically" nil (ae0 l k eh1 m ax k l iy0))
("alchemist" nil (ae1 l ch ax m ih0 s t))
("alchemy" nil (ae1 l k ax m iy0))
("alcide" nil (ae1 l s ay1 d))
("alcids" nil (ae1 l s ih0 d z))
("alcina" nil (aa0 l ch iy1 n ax))
("alco" nil (ae1 l k ow0))
("alcoa" nil (ae1 l k ow0 ax))
("alcocer" nil (ax l k ow1 s er0))
("alcock" nil (ax l k aa1 k))
("alcohol" nil (ae1 l k ax hh aa1 l))
("alcoholic" nil (ae1 l k ax hh aa1 l ih0 k))
("alcoholics" nil (ae1 l k ax hh aa1 l ih0 k s))
("alcoholism" nil (ae1 l k ax hh ao1 l ih1 z ax m))
("alcon" nil (ax l k aa1 n))
("alcorta" nil (aa0 l k ao1 r t ax))
("alcott" nil (ae1 l k aa1 t))
("alcoves" nil (ae1 l k ow1 v z))
("alda" nil (aa1 l d ax))
("aldaco" nil (aa0 l d aa1 k ow0))
("aldama" nil (aa0 l d aa1 m ax))
("aldana" nil (aa0 l d ae1 n ax))
("aldape" nil (aa0 l d aa1 p ey0))
("alday" nil (ae1 l d ey0))
("aldebaran" nil (ae0 l d eh1 b er0 ax n))
("alden" nil (aa1 l d ax n))
("alder" nil (ao1 l d er0))
("aldercy" nil (ax l d er1 k iy0))
("alderete" nil (ae1 l d er0 iy0 t))
("alderfer" nil (ae1 l d er0 f er0))
("alderman" nil (ao1 l d er0 m ax n))
("aldermen" nil (ao1 l d er0 m ih0 n))
("alderson" nil (ao1 l d er0 s ax n))
("alderton" nil (ao1 l d er0 t ax n))
("aldi" nil (aa1 l d iy0))
("aldicarb" nil (ao1 l d ih0 k aa1 r b))
("aldila" nil (ae1 l d ih1 l ax))
("aldin" nil (aa0 l d iy1 n))
("aldinger" nil (ao1 l d ih0 ng er0))
("aldis" nil (aa1 l d ih0 s))
("aldo" nil (aa1 l d ow0))
("aldora" nil (aa0 l d ao1 r ax))
("aldous" nil (aa1 l d ax s))
("aldred" nil (ae1 l d er0 d))
("aldredge" nil (ao1 l d r eh0 jh))
("aldrete" nil (ao1 l d r iy0 t))
("aldric" nil (ae1 l d r ih0 k))
("aldrich" nil (ao1 l d r ih0 ch))
("aldridge" nil (ao1 l d r ih0 jh))
("aldrin" nil (ao1 l d r ih0 n))
("aldus" nil (aa1 l d ih0 s))
("aldwin" nil (ao1 l d w ih0 n))
("aldwyn" nil (ao1 l d w ih0 n))
("aldys" nil (aa1 l d y s))
("ale" nil (ey1 l))
("aleatory" nil (ey1 l iy0 ax t ao1 r iy0))
("alec" nil (ae1 l ih0 k))
("aleda" nil (aa0 l ey1 d ax))
("aleen" nil (ax l iy1 n))
("alegre" nil (aa0 l eh1 g r iy0))
("alegrett" nil (ae1 l ax g r ax t))
("alegria" nil (ax l eh1 g r iy0 ax))
("alehouse" nil (ey1 l hh aw1 s))
("aleichem" nil (ax l ay1 k ax m))
("alejandre" nil (aa0 l ey0 y aa1 n d r ey0))
("alejandro" nil (aa0 l ey0 y aa1 n d r ow0))
("alejo" nil (aa0 l ey1 y ow0))
("alejos" nil (aa0 l ey1 y ow0 z))
("aleksandr" nil (ae1 l ax g z ae1 n d er0))
("aleman" nil (ey1 l m ax n))
("alena" nil (aa0 l ey1 n ax))
("alene" nil (ax l iy1 n))
("alenia" nil (ax l iy1 n iy0 ax))
("aleo" nil (aa1 l iy0 ow0))
("aleph" nil (aa1 l ax f))
("aleria" nil (ax l iy1 r iy0 ax))
("aleron" nil (aa0 l eh0 r ao1 n))
("alert" nil (ax l er1 t))
("alerted" nil (ax l er1 t ih0 d))
("alerting" nil (ax l er1 t ih0 ng))
("alertness" nil (ax l er1 t n ax s))
("alerts" nil (ax l er1 t s))
("ales" nil (ey1 l z))
("aleshire" nil (aa0 l ey0 sh ih1 r ey0))
("alesi" nil (aa0 l eh1 s iy0))
("alessandra" nil (ae1 l eh0 s ae1 n d r ax))
("alessandrini" nil (aa0 l eh0 s aa0 n d r iy1 n iy0))
("alessandro" nil (aa0 l ey0 z aa1 n d r ow0))
("alessi" nil (ax l eh1 s iy0))
("alessio" nil (ax l eh1 s iy0 ow0))
("aleta" nil (aa0 l eh1 t ax))
("alethea" nil (ae1 l ax th iy1 ax))
("aletti" nil (ax l eh1 t iy0))
("aleutian" nil (ax l uw1 sh ax n))
("aleutians" nil (ax l uw1 sh ax n z))
("aleve" nil (ax l iy1 v))
("alewife" nil (ey1 l w ay1 f))
("alewine" nil (ey1 l w ay1 n))
("alewives" nil (ey1 l w ay1 v z))
("alex" nil (ae1 l ax k s))
("alexa" nil (ax l eh1 k s ax))
("alexander" nil (ae1 l ax g z ae1 n d er0))
("alexanders" nil (ae1 l ih0 g z ae1 n d er0 z))
("alexandra" nil (ae1 l eh0 g z ae1 n d r ax))
("alexandre" nil (ae0 l ih0 k s aa1 n d er0))
("alexandria" nil (ae1 l ax g z ae1 n d r iy0 ax))
("alexandrine" nil (ae1 l ax g z ae1 n d r iy0 n))
("alexandrines" nil (ae1 l ax g z ae1 n d r iy0 n z))
("alexei" nil (ax l eh1 k s ey1))
("alexi" nil (ax l eh1 k s iy0))
("alexia" nil (ax l eh1 k s iy0 ax))
("alexine" nil (ax l eh1 k s ay0 n))
("alexis" nil (ax l eh1 k s ih0 s))
("alexopoulos" nil (ae0 l ih0 g z aa1 p ax l ih0 s))
("alexy" nil (ax l iy1 k s iy0))
("aley" nil (ey1 l iy0))
("alf" nil (ae1 l f))
("alfa" nil (ae1 l f ax))
("alfalfa" nil (ae1 l f ae1 l f ax))
("alfano" nil (aa0 l f aa1 n ow0))
("alfaro" nil (aa0 l f aa1 r ow0))
("alfie" nil (ae1 l f iy0))
("alfieri" nil (aa0 l f ih1 r iy0))
("alfiero" nil (ae1 l f iy0 eh1 r ow0))
("alfin" nil (ae1 l f ih0 n))
("alfons" nil (aa1 l f ow0 n z))
("alfonse" nil (ae1 l f aa0 n s))
("alfonsin" nil (ae1 l f aa1 n s ih0 n))
("alfonsine" nil (aa0 l f ow0 n s iy1 n iy0))
("alfonso" nil (ae1 l f aa1 n s ow0))
("alford" nil (ae1 l f er0 d))
("alfred" nil (ae1 l f r ax d))
("alfredo" nil (ae1 l f r ey1 d ow0))
("alfredson" nil (ae1 l f r ih0 d s ax n))
("alfrey" nil (ae1 l f r iy0))
("algae" nil (ae1 l jh iy0))
("algal" nil (ae1 l g ax l))
("algar" nil (aa0 l g aa1 r))
("algarin" nil (ae1 l g er0 ih0 n))
("algebra" nil (ae1 l jh ax b r ax))
("algebraic" nil (ae1 l jh ax b r ey1 ih0 k))
("algebraically" nil (ae1 l jh ax b r ey1 ih0 k l iy0))
("algemene" nil (ae1 l g ax m iy1 n))
("algeo" nil (ae1 l jh iy0 ow0))
("alger" nil (ae1 l jh er0))
("algeria" nil (ae0 l jh ih1 r iy0 ax))
("algerian" nil (ae0 l jh ih1 r iy0 ax n))
("algerians" nil (ae0 l jh iy1 r iy0 ax n z))
("algernon" nil (ae1 l jh er0 n aa0 n))
("algie" nil (ao1 l g iy0))
("algiers" nil (ae0 l jh ih1 r z))
("algol" nil (ae1 l g aa0 l))
("algom" nil (ae1 l g ax m))
("algoma" nil (ae0 l g ow1 m ax))
("algonquian" nil (ae0 l g aa1 ng k iy0 ax n))
("algonquin" nil (ae0 l g aa1 ng k w ih0 n))
("algorithm" nil (ae1 l g er0 ih1 dh ax m))
("algorithms" nil (ae1 l g er0 ih1 dh ax m z))
("alguire" nil (aa0 l g w ih1 r ey0))
("algy" nil (ae1 l jh iy0))
("alhadeff" nil (ae1 l ax d eh0 f))
("alhambra" nil (ae0 l hh ae1 m b r ax))
("ali" nil (aa1 l iy0))
("aliano" nil (aa0 l iy0 aa1 n ow0))
("alianza" nil (ae1 l iy0 ae1 n z ax))
("alias" nil (ey1 l iy0 ax s))
("aliases" nil (ey1 l iy0 ax s ih0 z))
("aliberti" nil (aa0 l iy0 b eh1 r t iy0))
("alibi" nil (ae1 l ax b ay1))
("alibis" nil (ae1 l ax b ay1 z))
("alibrandi" nil (ae1 l ih0 b r ae1 n d iy0))
("alice" nil (ae1 l ax s))
("alicea" nil (ax l ih1 s iy0 ax))
("alicia" nil (ax l ih1 sh ax))
("alida" nil (aa0 l iy1 d ax))
("alie" nil (ae1 l iy0))
("alien" nil (ey1 l iy0 ax n))
("alienate" nil (ey1 l y ax n ey1 t))
("alienated" nil (ey1 l iy0 ax n ey1 t ax d))
("alienates" nil (ey1 l iy0 ax n ey1 t s))
("alienating" nil (ey1 l iy0 ax n ey1 t ih0 ng))
("alienation" nil (ey1 l iy0 ax n ey1 sh ax n))
("aliens" nil (ey1 l iy0 ax n z))
("aliff" nil (ae1 l ih0 f))
("alig" nil (ae1 l ih0 g))
("alight" nil (ax l ay1 t))
("align" nil (ax l ay1 n))
("aligned" nil (ax l ay1 n d))
("aligning" nil (ax l ay1 n ih0 ng))
("alignment" nil (ax l ay1 n m ax n t))
("alignments" nil (ax l ay1 n m ax n t s))
("aligns" nil (ax l ay1 n z))
("alija" nil (ax l ay1 jh ax))
("alike" nil (ax l ay1 k))
("alikes" nil (ax l ay1 k s))
("alima" nil (aa0 l iy1 m ax))
("alimenies" nil (ae1 l ih0 m eh1 n iy0 z))
("alimentary" nil (ae1 l ax m eh1 n t er0 iy0))
("alimony" nil (ae1 l ax m ow1 n iy0))
("alina" nil (ax l iy1 n ax))
("aline" nil (ax l ay1 n))
("alines" nil (ax l ay1 n z))
("alioto" nil (aa0 l iy0 ow1 t ow0))
("aliquippa" nil (ae1 l ax k w ih1 p ax))
("aliquippas" nil (ae1 l ax k w ih1 p ax z))
("alire" nil (aa0 l ih1 r ey0))
("alisky" nil (ax l ih1 s k iy0))
("alison" nil (ae1 l ih0 s ax n))
("alissa" nil (ax l ih1 s ax))
("alistair" nil (ae1 l ih0 s t eh1 r))
("alister" nil (ae1 l ih0 s t er0))
("alita" nil (aa0 l iy1 t ax))
("alitalia" nil (ae1 l ih0 t ey1 l iy0 ax))
("alithia" nil (ax l ih1 th iy0 ax))
("alito" nil (ax l iy1 t ow0))
("alive" nil (ax l ay1 v))
("alix" nil (ae1 l ih0 k s))
("alizac" nil (ae1 l ih0 z ae0 k))
("alka" nil (ae1 l k ax))
("alkahest" nil (ae1 l k ax hh eh1 s t))
("alkali" nil (ae1 l k ax l ay1))
("alkalies" nil (ae1 l k ax l ay1 z))
("alkaline" nil (ae1 l k ax l ay1 n))
("alkalinity" nil (ae1 l k ax l ih1 n ax t iy0))
("alkaloid" nil (ae1 l k ax l oy1 d))
("alkaloidal" nil (ae0 l k ax l oy1 d ax l))
("alkaloids" nil (ae1 l k ax l oy1 d z))
("alkanes" nil (ae1 l k ey1 n z))
("alkema" nil (ae1 l k ih0 m ax))
("alkene" nil (ae1 l k iy1 n))
("alkenes" nil (ae1 l k iy1 n z))
("alkire" nil (ax l k ay1 r))
("all" nil (ao1 l))
("alla" nil (aa1 l ax))
("allah" nil (aa1 l ax))
("allain" nil (ax l ey1 n))
("allaire" nil (aa0 l eh1 r))
("allais" nil (ax l ey1))
("allan" nil (ae1 l ax n))
("allante" nil (ae1 l aa1 n t ey0))
("allar" nil (ax l aa1 r))
("allard" nil (ae1 l er0 d))
("allay" nil (ax l ey1))
("allayed" nil (ax l ey1 d))
("allaying" nil (ax l ey1 ih0 ng))
("allays" nil (ax l ey1 z))
("allbaugh" nil (ax l b ao1))
("allbee" nil (ao1 l b iy1))
("allbright" nil (ao1 l b r ay1 t))
("allbritten" nil (ae1 l b r ih0 t ax n))
("allbritton" nil (ae1 l b r ih0 t aa0 n))
("allcock" nil (ao1 l k aa1 k))
("allcorn" nil (ax l k ao1 r n))
("allday" nil (ao1 l d ey1))
("allderdice" nil (aa1 l d er0 d ay1 s))
("alldredge" nil (ax l d r eh1 jh))
("allebach" nil (ae1 l ih0 b aa0 k))
("alleco" nil (ae1 l eh1 k ow0))
("allee" nil (ax l iy1))
("allegation" nil (ae1 l ax g ey1 sh ax n))
("allegations" nil (ae1 l ax g ey1 sh ax n z))
("allege" nil (ax l eh1 jh))
("alleged" nil (ax l eh1 jh d))
("allegedly" nil (ax l eh1 jh ax d l iy0))
("alleges" nil (ax l eh1 jh ax z))
("alleghany" nil (ae1 l ax g ey1 n iy0))
("allegheny" nil (ae1 l ax g ey1 n iy0))
("allegiance" nil (ax l iy1 jh ax n s))
("allegiances" nil (ae1 l iy1 jh iy0 ae1 n s ih0 z))
("alleging" nil (ax l eh1 jh ih0 ng))
("allegis" nil (ae1 l ey1 jh ih0 s))
("allegorical" nil (ae1 l ax g ao1 r ax k ax l))
("allegories" nil (ae1 l ax g ao1 r iy0 z))
("allegory" nil (ae1 l ax g ao1 r iy0))
("allegra" nil (aa0 l eh1 g r ax))
("allegretti" nil (aa0 l eh0 g r eh1 t iy0))
("allegro" nil (ax l eh1 g r ow1))
("alleman" nil (ey1 l m ax n))
("allemand" nil (ae1 l ih0 m ax n d))
("allen" nil (ae1 l ax n))
("allenbaugh" nil (ax l eh1 n b ao0))
("allendale" nil (ae1 l ax n d ey1 l))
("allende" nil (aa1 y eh1 n d ey0))
("allender" nil (aa0 l y eh1 n d ey0 er0))
("allendorf" nil (ae1 l ih0 n d ao0 r f))
("allene" nil (ae1 l iy1 n))
("allenhurst" nil (ae1 l ax n hh er1 s t))
("allensbach" nil (ae1 l ax n z b aa1 k))
("allensworth" nil (ax l eh1 n s w er0 th))
("allentown" nil (ae1 l ax n t aw1 n))
("allenwood" nil (ae1 l ax n w uh1 d))
("aller" nil (ao1 l er0))
("allergan" nil (ae1 l er0 jh ax n))
("allergic" nil (ax l er1 jh ih0 k))
("allergies" nil (ae1 l er0 jh iy0 z))
("allergist" nil (ae1 l er0 jh ax s t))
("allergy" nil (ae1 l er0 jh iy0))
("allers" nil (ao1 l er0 z))
("allert" nil (ae1 l er0 t))
("allerton" nil (ae1 l er0 t ax n))
("alles" nil (ey1 l z))
("alleva" nil (aa0 l ey1 v ax))
("alleviate" nil (ax l iy1 v iy0 ey1 t))
("alleviated" nil (ax l iy1 v iy0 ey1 t ax d))
("alleviates" nil (ax l iy1 v iy0 ey0 t s))
("alleviating" nil (ax l iy1 v iy0 ey1 t ih0 ng))
("alley" nil (ae1 l iy0))
("alleyne" nil (ae1 l ey1 n))
("alleys" nil (ae1 l iy0 z))
("alleyway" nil (ae1 l iy0 w ey1))
("alleyways" nil (ae1 l iy0 w ey1 z))
("allgaier" nil (ae1 l g ay0 er0))
("allgeier" nil (ae1 l g ay0 er0))
("allgemeine" nil (ao1 l g ax m ay1 n))
("allgeyer" nil (ae1 l g iy0 er0))
("allgood" nil (ao1 l g uh1 d))
("alli" nil (ae1 l iy0))
("alliance" nil (ax l ay1 ax n s))
("alliances" nil (ax l ay1 ax n s ax z))
("alliant" nil (ax l ay1 ax n t))
("allianz" nil (ae1 l iy0 ax n z))
("allick" nil (ae1 l ih0 k))
("allie" nil (ae1 l iy0))
("allied" nil (ax l ay1 d))
("alliedsignal" nil (ae1 l ay1 d s ih1 g n ax l))
("allies" n (ae1 l ay0 z))
("allies" v (ax l ay1 z))
("alligator" nil (ae1 l ax g ey1 t er0))
("alligators" nil (ae1 l ax g ey1 t er0 z))
("alligood" nil (ae1 l ih0 g uh1 d))
("allin" nil (ax l ih1 n))
("allinder" nil (ae1 l ih0 n d er0))
("alling" nil (ao1 l ih0 ng))
("allinger" nil (ao1 l ih0 ng er0))
("allingham" nil (ao1 l ih0 ng hh ae1 m))
("allington" nil (ao1 l ih0 ng t ax n))
("allinson" nil (ae1 l ih0 n s ax n))
("allis" nil (ae1 l ih0 s))
("allison" nil (ae1 l ax s ax n))
("allister" nil (ao1 l ih0 s t er0))
("alliston" nil (ae1 l ih0 s t aa0 n))
("alliterative" nil (ax l ih1 t er0 ey1 t ih0 v))
("allman" nil (ao1 l m ax n))
("allmendinger" nil (ae1 l m eh0 n d ih0 ng er0))
("allmon" nil (ao1 l m ax n))
("allmond" nil (ax l m aa1 n d))
("allnet" nil (ao1 l n eh1 t))
("allnutt" nil (ax l n ah1 t))
("allocate" nil (ae1 l ax k ey1 t))
("allocated" nil (ae1 l ax k ey1 t ih0 d))
("allocates" nil (ae1 l ax k ey1 t s))
("allocating" nil (ae1 l ax k ey1 t ih0 ng))
("allocation" nil (ae1 l ax k ey1 sh ax n))
("allocations" nil (ae1 l ax k ey1 sh ax n z))
("allocator" nil (ae1 l ax k ey1 t er0))
("allocators" nil (ae1 l ax k ey1 t er0 z))
("allocca" nil (ae1 l aa1 k ax))
("allocco" nil (ae1 l aa1 k ow0))
("allomorphs" nil (ae1 l ax m ao0 r f s))
("allophones" nil (ae1 l ax f ow1 n z))
("allophonic" nil (ae1 l ax f aa1 n ih0 k))
("allor" nil (ao1 l er0))
("allot" nil (ax l aa1 t))
("alloted" nil (ax l aa1 t ih0 d))
("allotment" nil (ax l aa1 t m ax n t))
("allotments" nil (ax l aa1 t m ax n t s))
("allotropes" nil (ae1 l ax t r ow1 p s))
("allots" nil (ax l aa1 t s))
("allotted" nil (ax l aa1 t ih0 d))
("allotting" nil (ax l aa1 t ih0 ng))
("allow" nil (ax l aw1))
("allowable" nil (ax l aw1 ax b ax l))
("allowance" nil (ax l aw1 ax n s))
("allowances" nil (ax l aw1 ax n s ih0 z))
("alloway" nil (ae1 l ow0 w ey1))
("allowed" nil (ax l aw1 d))
("allowing" nil (ax l aw1 ih0 ng))
("allows" nil (ax l aw1 z))
("alloy" nil (ae1 l oy1))
("alloys" nil (ae1 l oy1 z))
("allphin" nil (ax l f ih1 n))
("allport" nil (ao1 l p ao1 r t))
("allred" nil (ae1 l r d))
("alls" nil (ao1 l z))
("allsbrook" nil (ao1 l z b r uh1 k))
("allshouse" nil (ao1 l z hh aw1 s))
("allsop" nil (ao1 l s aa1 p))
("allsopp" nil (ax l s aa1 p))
("allspice" nil (ao1 l s p ay1 s))
("allstate" nil (ao1 l s t ey1 t))
("allston" nil (ao1 l s t ax n))
("allsup" nil (ae1 l s ax p))
("alltel" nil (ao1 l t eh1 l))
("alltime" nil (ao1 l t ay1 m))
("allton" nil (ao1 l t ax n))
("alltop" nil (ao1 l t aa1 p))
("allude" nil (ax l uw1 d))
("alluded" nil (ax l uw1 d ax d))
("alludes" nil (ax l uw1 d z))
("alluding" nil (ax l uw1 d ih0 ng))
("allum" nil (ae1 l ax m))
("allums" nil (ae1 l ax m z))
("allure" nil (ax l uh1 r))
("alluring" nil (ax l uh1 r ih0 ng))
("allusion" nil (ax l uw1 zh ax n))
("allusions" nil (ax l uw1 zh ax n z))
("allusive" nil (ax l uw1 s ih0 v))
("alluvial" nil (ae1 l uw1 v iy0 ax l))
("alluvium" nil (ax l uw1 v iy0 ax m))
("allwaste" nil (ao1 l w ey1 s t))
("allways" nil (ao1 l w ey1 z))
("allweiss" nil (aa1 l w iy1 s))
("ally" n (ae1 l ay0))
("ally" v (ax l ay1))
("allying" nil (ae1 l ay0 ih0 ng))
("allyn" nil (ae1 l ih0 n))
("allys" nil (ae1 l ay0 z))
("alm" nil (aa1 m))
("alma" nil (ae1 l m ax))
("almada" nil (aa0 l m aa1 d ax))
("almadani" nil (ao1 l m ax d aa1 n iy0))
("almaden" nil (ae1 l m ax d ax n))
("almaguer" nil (aa0 l m aa0 g w eh1 r))
("alman" nil (ae1 l m ax n))
("almanac" nil (ao1 l m ax n ae1 k))
("almand" nil (ae1 l m ax n d))
("almanza" nil (aa0 l m aa1 n z ax))
("almanzar" nil (aa0 l m aa0 n z aa1 r))
("almaraz" nil (aa0 l m aa1 r aa0 z))
("almas" nil (ae1 l m ax z))
("almasy" nil (ae1 l m ax s iy0))
("almazan" nil (aa0 l m aa0 z aa1 n))
("almeda" nil (aa0 l m ey1 d ax))
("almeida" nil (aa0 l m iy1 d ax))
("almendarez" nil (aa0 l m ey0 n d aa1 r eh0 z))
("almer" nil (ao1 l m er0))
("almgren" nil (ae1 l m g r eh0 n))
("almighty" nil (ao0 l m ay1 t iy0))
("almira" nil (aa0 l m ih1 r ax))
("almo" nil (aa1 l m ow0))
("almodovar" nil (aa0 l m ow0 d ow0 v aa1 r))
("almond" nil (aa1 m ax n d))
("almonds" nil (aa1 l m ax n d z))
("almonte" nil (aa0 l m ow1 n t ey0))
("almost" nil (ao1 l m ow1 s t))
("almquist" nil (ae1 l m k w ih0 s t))
("alms" nil (aa1 l m z))
("almy" nil (ao1 m iy0))
("almys" nil (ae1 l m iy0 z))
("alodie" nil (ax l aa1 d iy0))
("aloe" nil (ae1 l ow1))
("aloft" nil (ax l ao1 f t))
("aloha" nil (ax l ow1 hh aa0))
("aloi" nil (aa1 l oy0))
("aloia" nil (aa0 l ow1 y ax))
("alois" nil (aa0 l oy1 s))
("aloisa" nil (aa0 l oy1 s ax))
("aloisi" nil (aa0 l oy1 s iy0))
("aloisia" nil (aa0 l oy1 s iy0 ax))
("aloisio" nil (aa0 l oy1 s iy0 ow0))
("aloka" nil (ax l ow1 k ax))
("alon" nil (ax l aa1 n))
("alone" nil (ax l ow1 n))
("along" nil (ax l ao1 ng))
("alonge" nil (ae1 l ax n jh))
("alongi" nil (aa0 l ow1 ng g iy0))
("alongside" nil (ax l ao1 ng s ay1 d))
("alonso" nil (ax l aa1 n s ow0))
("alonza" nil (ax l aa1 n z ax))
("alonzo" nil (ax l aa1 n z ow0))
("aloof" nil (ax l uw1 f))
("aloofness" nil (ax l uw1 f n ax s))
("alot" nil (ax l aa1 t))
("aloud" nil (ax l aw1 d))
("aloys" nil (ax l oy1 z))
("aloyse" nil (ax l oy1 s))
("aloysia" nil (aa0 l oy1 s iy0 ax))
("aloysius" nil (ae1 l ow0 ih1 sh ih0 s))
("alpa" nil (ae1 l p ax))
("alpaca" nil (ae0 l p ae1 k ax))
("alpaugh" nil (ax l p ao1))
("alper" nil (ae1 l p er0))
("alperin" nil (ae1 l p er0 ih1 n))
("alpern" nil (ax l p er1 n))
("alpers" nil (ae1 l p er0 z))
("alpert" nil (ae1 l p er0 t))
("alpex" nil (ae1 l p eh0 k s))
("alpha" nil (ae1 l f ax))
("alphabet" nil (ae1 l f ax b eh1 t))
("alphabetic" nil (ae1 l f ax b eh1 t ih0 k))
("alphabetical" nil (ae1 l f ax b eh1 t ih0 k ax l))
("alphabetically" nil (ae1 l f ax b eh1 t ih0 k l iy0))
("alphabetization" nil (ae1 l f ax b eh1 t ax z ey1 sh ax n))
("alphabetize" nil (ae1 l f ax b ax t ay1 z))
("alphametrics" nil (ae1 l f ax m eh1 t r ih0 k s))
("alphandery" nil (ae1 l f ae1 d er0 iy0))
("alphanumeric" nil (ae1 l f ax n uw0 m eh1 r ih0 k))
("alpharel" nil (ae1 l f er0 eh1 l))
("alpharetta" nil (ae1 l f er0 eh1 t ax))
("alphin" nil (ae1 l f ih0 n))
("alphonse" nil (ae0 l f aa1 n z))
("alphonsine" nil (ax l f aa1 n s iy0 n))
("alphonso" nil (ae0 l f aa1 n s ow0))
("alpin" nil (ax l p ih1 n))
("alpine" nil (ae1 l p ay1 n))
("alpo" nil (ae1 l p ow0))
("alps" nil (ae1 l p s))
("alquist" nil (ae1 l k w ih0 s t))
("already" nil (ao0 l r eh1 d iy0))
("alred" nil (ao1 l r d))
("alright" nil (ao1 l r ay1 t))
("alroy" nil (ax l r oy1))
("alsace" nil (ae0 l s aa1 s))
("alsatian" nil (ae0 l s ey1 sh ax n))
("alsbrooks" nil (ae1 l s b r uh0 k s))
("alsbury" nil (ao1 l s b eh0 r iy0))
("alsdorf" nil (ae1 l s d ao0 r f))
("alsip" nil (ae1 l s ih0 p))
("also" nil (ao1 l s ow0))
("alsobrook" nil (ae1 l s ax b r uh0 k))
("alsobrooks" nil (ae1 l s ax b r uh0 k s))
("alsop" nil (ae1 l s aa0 p))
("alspach" nil (ae1 l s p ax k))
("alspaugh" nil (ax l s p ao1))
("alsthom" nil (ae1 l s th ax m))
("alston" nil (ao1 l s t ax n))
("alsup" nil (ae1 l s ax p))
("alt" nil (aa1 l t))
("alta" nil (aa1 l t ax))
("altai" nil (ae0 l t ay1))
("altaic" nil (ae0 l t ey1 ih0 k))
("altamirano" nil (aa0 l t aa0 m ih0 r aa1 n ow0))
("altamuro" nil (ao1 l t ax m uh1 r ow0))
("altar" nil (ao1 l t er0))
("altarpiece" nil (ao1 l t er0 p iy1 s))
("altavista" nil (ao1 l t ax v ih1 s t ax))
("altay" nil (ao1 l t ay0))
("altemose" nil (ae1 l t ih0 m ow0 s))
("altemus" nil (ae1 l t ih0 m ih0 s))
("altenburg" nil (ao1 l t ax n b er0 g))
("altenhofen" nil (ae1 l t ih0 n hh ax f ax n))
("alter" nil (ao1 l t er0))
("altera" nil (ao1 l t eh1 r ax))
("alteration" nil (ao1 l t er0 ey1 sh ax n))
("alterations" nil (ao1 l t er0 ey1 sh ax n z))
("altercation" nil (ae1 l t er0 k ey1 sh ax n))
("altered" nil (ao1 l t er0 d))
("altergott" nil (ae1 l t er0 g ax t))
("altering" nil (ao1 l t er0 ih0 ng))
("alterman" nil (ao1 l t er0 m ax n))
("alternacare" nil (ao0 l t er1 n ax k eh1 r))
("alternate" j (ao1 l t er0 n ax t))
("alternate" v (ao1 l t er0 n ey1 t))
("alternated" nil (ao1 l t er0 n ey1 t ax d))
("alternately" nil (ao1 l t er0 n ax t l iy0))
("alternates" nil (ao1 l t er0 n ey1 t s))
("alternating" nil (ao1 l t er0 n ey1 t ih0 ng))
("alternative" nil (ao0 l t er1 n ax t ih0 v))
("alternatively" nil (ao0 l t er1 n ax t ih0 v l iy0))
("alternatives" nil (ao0 l t er1 n ax t ih0 v z))
("alternator" nil (ao1 l t er0 n ey1 t er0))
("alters" nil (ao1 l t er0 z))
("altfest" nil (aa1 l t f eh1 s t))
("althaus" nil (ae1 l t hh aw0 s))
("althaver" nil (ae1 l th ey1 v er0))
("althea" nil (ae0 l th iy1 ax))
("althoff" nil (ae1 l t hh ao0 f))
("although" nil (ao1 l dh ow1))
("althouse" nil (ao1 l t hh aw1 s))
("altice" nil (aa1 l t ih0 s))
("altier" nil (ao1 l t iy0 er0))
("altieri" nil (aa0 l t ih1 r iy0))
("altima" nil (aa1 l t ih1 m ax))
("altimeter" nil (ae0 l t ih1 m ax t er0))
("altimorano" nil (aa1 l t iy1 m ao1 r aa1 n ow0))
("altiplano" nil (ae1 l t ax p l aa1 n ow1))
("altitude" nil (ae1 l t ax t uw1 d))
("altitudes" nil (ae1 l t ih0 t uw1 d z))
("altizer" nil (ae1 l t ay0 z er0))
("altland" nil (ae1 l t l ax n d))
("altman" nil (ao1 l t m ax n))
("altmann" nil (ao1 l t m ax n))
("altmeyer" nil (ae1 l t m ay0 er0))
("alto" nil (ae1 l t ow0))
("altobelli" nil (aa0 l t ow0 b eh1 l iy0))
("altogether" nil (ao1 l t ax g eh1 dh er0))
("altom" nil (ax l t aa1 m))
("altomare" nil (aa0 l t ow0 m aa1 r iy0))
("alton" nil (ao1 l t ax n))
("altos" nil (ae1 l t ow0 z))
("altra" nil (aa1 l t r ax))
("altron" nil (ao1 l t r aa0 n))
("altruism" nil (ae1 l t r uw0 ih1 z ax m))
("altruistic" nil (ao1 l t r uw0 ih1 s t ih0 k))
("altschiller" nil (ao1 l ch ih0 l er0))
("altschul" nil (ae1 l ch ax l))
("altschuler" nil (ae1 l ch y uw0 l er0))
("altshuler" nil (ae1 l ch y uw0 l er0))
("altucher" nil (ae0 l t ah1 k er0))
("altus" nil (ae1 l t ax s))
("altvater" nil (ae1 l t v ax t er0))
("alu" nil (aa1 l uw0))
("alum" nil (ae1 l ax m))
("alumax" nil (ae1 l uw0 m ae0 k s))
("alumbaugh" nil (ax l ah1 m b ao0))
("alumina" nil (ax l uw1 m ax n ax))
("aluminio" nil (ae1 l uw0 m iy1 n iy0 ow0))
("aluminium" nil (ae1 l y ax m ih1 n iy0 ax m))
("aluminized" nil (ax l uw1 m ax n ay1 z d))
("aluminosilicate" nil (ax l uw1 m ax n ow0 s ih1 l ax k ey1 t))
("aluminum" nil (ax l uw1 m ax n ax m))
("alumni" nil (ax l ah1 m n ay1))
("alumnus" nil (ax l ah1 m n ax s))
("alums" nil (ae1 l ax m z))
("alun" nil (ey1 l ax n))
("alura" nil (aa0 l uh1 r ax))
("alusuisse" nil (ae1 l y uw0 s w ih1 s))
("alva" nil (ae1 l v ax))
("alvah" nil (ae0 l v aa1))
("alvan" nil (ae1 l v ax n))
("alvarado" nil (ae1 l v er0 aa1 d ow0))
("alvardo" nil (aa0 l v aa1 r d ow0))
("alvarenga" nil (aa0 l v aa0 r eh1 ng g ax))
("alvares" nil (aa0 l v aa1 r eh0 s))
("alvarez" nil (ae1 l v er0 eh1 z))
("alvaro" nil (ax l v aa1 r ow0))
("alvear" nil (aa0 l v ih1 r))
("alveolar" nil (ae0 l v iy1 ax l er0))
("alveoli" nil (ae0 l v iy1 ax l ay1))
("alverez" nil (aa0 l v eh1 r eh0 z))
("alvero" nil (ae0 l v eh1 r ow0))
("alverson" nil (aa0 l v eh1 r s ax n))
("alves" nil (aa1 l v eh0 s))
("alvey" nil (ae1 l v iy0))
("alvidrez" nil (aa0 l v iy1 d r eh0 z))
("alvin" nil (ae1 l v ih0 n))
("alvina" nil (ae0 l v ay1 n ax))
("alvino" nil (aa0 l v iy1 n ow0))
("alvis" nil (aa1 l v ih0 s))
("alvita" nil (aa0 l v iy1 t ax))
("alvite" nil (ae1 l v ay1 t))
("alvord" nil (ae0 l v ao1 r d))
("alwaleed" nil (ae1 l w ax l iy1 d))
("alward" nil (ae0 l w er1 d))
("always" nil (ao1 l w ey1 z))
("alwin" nil (ae1 l w ih0 n))
("alwine" nil (ae1 l w ay1 n))
("alwood" nil (ae1 l w uh0 d))
("alwyn" nil (ae1 l w ih0 n))
("aly" nil (ey1 l iy0))
("alyce" nil (ae1 l ih0 s))
("alyea" nil (ae0 l iy1 ax))
("alyeska" nil (ae0 l iy0 eh1 s k ax))
("alys" nil (ae1 l iy0 z))
("alysheba" nil (ae1 l ih0 sh iy1 b ax))
("alysia" nil (ax l ih1 s iy0 ax))
("alyssa" nil (ax l ih1 s ax))
("alza" nil (ae1 l z ax))
("alzena" nil (aa0 l z eh1 n ax))
("alzheimer" nil (ae1 l z hh ay1 m er0))
("am" nil (ae1 m))
("amabel" nil (ae1 m ax b eh1 l))
("amabelle" nil (ae1 m ax b ax l))
("amabile" nil (aa0 m aa1 b ax l))
("amacher" nil (ae1 m ax k er0))
("amacker" nil (ae1 m ax k er0))
("amadea" nil (aa0 m aa1 d iy0 ax))
("amadeo" nil (aa0 m aa1 d iy0 ow0))
("amadeus" nil (ae1 m ax d ey1 ax s))
("amado" nil (aa0 m aa1 d ow0))
("amadon" nil (aa0 m aa0 d ao1 n))
("amador" nil (ae1 m ax d ao1 r))
("amadou" nil (ae1 m ax d uw1))
("amaker" nil (ae1 m ey0 k er0))
("amakudari" nil (ae1 m ax k y uw0 d aa1 r iy0))
("amal" nil (ax m aa1 l))
("amalea" nil (ae1 m ax l iy1 ax))
("amalfitano" nil (aa0 m aa0 l f iy0 t aa1 n ow0))
("amalgam" nil (ax m ae1 l g ax m))
("amalgamated" nil (ax m ae1 l g ax m ey1 t ih0 d))
("amalgamation" nil (ax m ae1 l g ax m ey1 sh ax n))
("amalgams" nil (ax m ae1 l g ax m z))
("amalia" nil (ax m aa1 l y ax))
("amalie" nil (ae1 m ax l iy0))
("aman" nil (aa1 m ax n))
("amana" nil (ax m ae1 n ax))
("amanda" nil (ax m ae1 n d ax))
("amann" nil (ae1 m ax n))
("amano" nil (ax m aa1 n ow0))
("amanpour" nil (aa1 m aa1 n p uh1 r))
("amante" nil (aa0 m aa1 n t iy0))
("amar" nil (ax m aa1 r))
("amara" nil (aa0 m aa1 r ax))
("amaral" nil (aa0 m aa0 r ae1 l))
("amarante" nil (aa0 m aa0 r aa1 n t iy0))
("amaranth" nil (ae1 m er0 ae1 n th))
("amari" nil (aa0 m aa1 r iy0))
("amarillo" nil (ae1 m er0 ih1 l ow0))
("amarin" nil (ae1 m er0 ih0 n))
("amaris" nil (ae1 m er0 ih0 s))
("amaro" nil (aa0 m aa1 r ow0))
("amaryllis" nil (ae1 m er0 ih1 l ax s))
("amaryllises" nil (ae1 m er0 ih1 l ax s ax z))
("amasa" nil (ae1 m ax s ax))
("amason" nil (ae1 m ax s ax n))
("amass" nil (ax m ae1 s))
("amassed" nil (ax m ae1 s t))
("amassing" nil (ax m ae1 s ih0 ng))
("amateur" nil (ae1 m ax t er1))
("amateurish" nil (ae1 m ax ch er0 ih0 sh))
("amateurism" nil (ae1 m ax ch er0 ih0 z ax m))
("amateurs" nil (ae1 m ax t er1 z))
("amatil" nil (ae1 m ax t ih0 l))
("amato" nil (aa0 m aa1 t ow0))
("amauligak" nil (ax m aw1 l ih0 g ae0 k))
("amax" nil (ey1 m ae0 k s))
("amaya" nil (aa0 m aa1 y ax))
("amaze" nil (ax m ey1 z))
("amazed" nil (ax m ey1 z d))
("amazement" nil (ax m ey1 z m ax n t))
("amazes" nil (ax m ey1 z ih0 z))
("amazing" nil (ax m ey1 z ih0 ng))
("amazingly" nil (ax m ey1 z ih0 ng l iy0))
("amazon" nil (ae1 m ax z aa1 n))
("amazonia" nil (ae1 m ax z ow1 n iy0 ax))
("amazonian" nil (ae1 m ax z ow1 n iy0 ax n))
("amazonians" nil (ae1 m ax z ow1 n iy0 ax n z))
("amazons" nil (ae1 m ax z aa1 n z))
("ambac" nil (ae1 m b ae0 k))
("ambase" nil (ae1 m b ey1 s))
("ambassador" nil (ae0 m b ae1 s ax d er0))
("ambassadorial" nil (ae0 m b ae1 s ax d ao1 r iy0 ax l))
("ambassadors" nil (ae0 m b ae1 s ax d er0 z))
("ambassadorship" nil (ae0 m b ae1 s ax d er0 sh ih1 p))
("ambassadorships" nil (ae0 m b ae1 s ax d er0 sh ih1 p s))
("ambassadress" nil (ae0 m b ae1 s ax d r ax s))
("amber" nil (ae1 m b er0))
("amberg" nil (ae1 m b er0 g))
("amberger" nil (ae1 m b er0 g er0))
("ambergris" nil (ae1 m b er0 g r ih0 s))
("ambers" nil (ae1 m b er0 z))
("amberson" nil (ae1 m b er0 s ax n))
("ambiance" nil (ae1 m b iy0 ax n s))
("ambidextrous" nil (ae1 m b ax d eh1 k s t r ax s))
("ambience" nil (ae1 m b iy0 ax n s))
("ambient" nil (ae1 m b iy0 ax n t))
("ambiguities" nil (ae0 m b ax g y uw1 ax t iy0 z))
("ambiguity" nil (ae1 m b ih0 g y uw1 ax t iy0))
("ambiguous" nil (ae0 m b ih1 g y uw0 ax s))
("ambition" nil (ae0 m b ih1 sh ax n))
("ambitions" nil (ae0 m b ih1 sh ax n z))
("ambitious" nil (ae0 m b ih1 sh ax s))
("ambitiously" nil (ae0 m b ih1 sh ax s l iy0))
("ambivalence" nil (ae0 m b ih1 v ax l ax n s))
("ambivalent" nil (ae0 m b ih1 v ax l ax n t))
("amble" nil (ae1 m b ax l))
("ambled" nil (ae1 m b ax l d))
("ambler" nil (ae1 m b l er0))
("ambles" nil (ae1 m b ax l z))
("amblin" nil (ae1 m b l ih0 n))
("ambling" nil (ae1 m b ax l ih0 ng))
("amborn" nil (ax m b ao1 r n))
("ambra" nil (ae1 m b r ax))
("ambriano" nil (ae1 m b r iy0 aa1 n ow0))
("ambrit" nil (ae1 m b r ih0 t))
("ambriz" nil (ae1 m b r ih0 z))
("ambrogio" nil (ae1 m b r ow1 jh iy0 ow0))
("ambrose" nil (ae1 m b r ow1 z))
("ambrosia" nil (ae0 m b r ow1 zh ax))
("ambrosial" nil (ae0 m b r ow1 zh ax l))
("ambrosian" nil (ae0 m b r ow1 z ax n))
("ambrosiano" nil (ae0 m b r ow1 s iy0 aa1 n ow0))
("ambrosine" nil (aa0 m b r ow0 s iy1 n iy0))
("ambrosini" nil (aa0 m b r ow0 s iy1 n iy0))
("ambrosino" nil (aa0 m b r ow0 s iy1 n ow0))
("ambrosio" nil (ae1 m b r ow1 s iy0 ow0))
("ambrosius" nil (ae1 m b r ax s iy0 ih0 s))
("ambs" nil (ae1 m z))
("ambuehl" nil (ae1 m b uh0 l))
("ambulance" nil (ae1 m b y ax l ax n s))
("ambulances" nil (ae1 m b y ax l ax n s ax z))
("ambulatory" nil (ae1 m b y ax l ax t ao1 r iy0))
("amburgey" nil (ae1 m b er0 g iy0))
("amburn" nil (ax m b er1 n))
("ambush" nil (ae1 m b uh1 sh))
("ambushed" nil (ae1 m b uh1 sh t))
("ambushes" nil (ae1 m b uh0 sh ih0 z))
("ambushing" nil (ae1 m b uh1 sh ih0 ng))
("amc" nil (ae1 m k))
("amca" nil (ae1 m k ax))
("amcast" nil (ae1 m k ae1 s t))
("amcole" nil (ae1 m k ow1 l))
("amcor" nil (ae1 m k ao1 r))
("amcore" nil (ae1 m k ao1 r))
("amdahl" nil (ae1 m d aa1 l))
("amdec" nil (ae1 m d eh1 k))
("amdek" nil (ae1 m d eh0 k))
("amdur" nil (ae1 m d er0))
("amdura" nil (ae0 m d uh1 r ax))
("ame" nil (ey1 m))
("amedco" nil (ax m eh1 d k ow0))
("amedee" nil (ae1 m ih0 d iy0))
("ameen" nil (ax m iy1 n))
("amelia" nil (ax m iy1 l y ax))
("amelinda" nil (aa0 m eh0 l iy1 n d ax))
("ameline" nil (aa0 m eh0 l iy1 n iy0))
("amelio" nil (ax m iy1 l iy0 ow0))
("ameliorate" nil (ax m iy1 l y er0 ey1 t))
("ameliorated" nil (ax m iy1 l iy0 er0 ey1 t ih0 d))
("amelioration" nil (ax m iy1 l y er0 ey1 sh ax n))
("amelita" nil (aa0 m eh0 l iy1 t ax))
("amell" nil (aa0 m ey1 l))
("amen" nil (ey1 m eh1 n))
("amenable" nil (ax m eh1 n ax b ax l))
("amend" nil (ax m eh1 n d))
("amendable" nil (ax m eh1 n d ax b ax l))
("amended" nil (ax m eh1 n d ax d))
("amending" nil (ax m eh1 n d ih0 ng))
("amendment" nil (ax m eh1 n d m ax n t))
("amendments" nil (ax m eh1 n d m ax n t s))
("amendola" nil (aa0 m eh0 n d ow1 l ax))
("amends" nil (ax m eh1 n d z))
("amenities" nil (ax m eh1 n ax t iy0 z))
("amenity" nil (ax m eh1 n ax t iy0))
("ament" nil (ae1 m ih0 n t))
("amenta" nil (ax m eh1 n t ax))
("amer" nil (ey1 m er0))
("amerada" nil (ae1 m eh0 r aa1 d ax))
("amerco" nil (ax m eh1 r k ow0))
("amerford" nil (ey1 m er0 f er0 d))
("ameri" nil (ax m eh1 r iy0))
("ameribanc" nil (ax m eh1 r ih0 b ae1 ng k))
("america" nil (ax m eh1 r ax k ax))
("american" nil (ax m eh1 r ax k ax n))
("americana" nil (ax m eh1 r ax k ae1 n ax))
("americanism" nil (ax m eh1 r ih0 k ax n ih1 z ax m))
("americanization" nil (ax m eh1 r ax k ax n ax z ey1 sh ax n))
("americanize" nil (ax m eh1 r ax k ax n ay1 z))
("americanized" nil (ax m eh1 r ih0 k ax n ay1 z d))
("americano" nil (ax m eh1 r ih0 k aa1 n ow0))
("americans" nil (ax m eh1 r ax k ax n z))
("americar" nil (ax m eh1 r ih0 k aa1 r))
("americare" nil (ax m eh1 r ih0 k eh1 r))
("americares" nil (ax m eh1 r ih0 k eh1 r z))
("americas" nil (ax m eh1 r ax k ax z))
("americold" nil (ax m eh1 r ih0 k ow1 l d))
("americorps" nil (ax m eh1 r ih0 k ao1 r))
("americus" nil (ax m eh1 r ih0 k ax s))
("amerifirst" nil (ax m eh1 r ih0 f er0 s t))
("amerigas" nil (ax m eh1 r ih0 g ae1 s))
("amerigo" nil (aa0 m er0 iy1 g ow0))
("amerika" nil (ax m eh1 r ih0 k ax))
("amerindian" nil (ae1 m er0 ih1 n d iy0 ax n))
("amerine" nil (aa0 m er0 iy1 n iy0))
("ameritech" nil (ax m eh1 r ih0 t eh1 k))
("ameritrust" nil (ax m eh1 r ih0 t r ah1 s t))
("amerman" nil (ae1 m er0 m ax n))
("amero" nil (aa0 m eh1 r ow0))
("ameron" nil (ae1 m er0 aa0 n))
("amerongen" nil (ae1 m er0 ao1 n jh ax n))
("amersham" nil (ae1 m er0 sh ae1 m))
("amerson" nil (ae1 m er0 s ax n))
("amery" nil (ae1 m er0 iy0))
("ames" nil (ey1 m z))
("ametek" nil (ae1 m ax t eh1 k))
("amethyst" nil (ae1 m ih0 th ih0 s t))
("amev" nil (ae1 m eh0 v))
("amex" nil (ae1 m eh0 k s))
("amexco" nil (ax m eh1 k s k ow0))
("amey" nil (ey1 m iy0))
("amezcua" nil (ax m eh1 z k y uw0 ax))
("amezquita" nil (aa0 m eh0 z k w iy1 t ax))
("amfac" nil (ae1 m f ae0 k))
("amfesco" nil (ae0 m f eh1 s k ow0))
("amgen" nil (ae1 m jh eh0 n))
("amherst" nil (ae1 m er0 s t))
("amhoist" nil (ae0 m hh oy1 s t))
("ami" nil (aa1 m iy0))
("amiability" nil (ey1 m iy0 ax b ih1 l ax t iy0))
("amiable" nil (ey1 m iy0 ax b ax l))
("amiably" nil (ey1 m iy0 ax b l iy0))
("amicable" nil (ae1 m ih0 k ax b ax l))
("amicably" nil (ae1 m ih0 k ax b l iy0))
("amick" nil (ae1 m ih0 k))
("amico" nil (aa0 m iy1 k ow0))
("amicone" nil (ae1 m ih0 k ow1 n))
("amicus" nil (ax m iy1 k ax s))
("amid" nil (ax m ih1 d))
("amidi" nil (aa0 m iy1 d iy0))
("amidon" nil (ae1 m ih0 d aa0 n))
("amidships" nil (ax m ih1 d sh ih0 p s))
("amidst" nil (ax m ih1 d s t))
("amie" nil (ae1 m iy0))
("amiga" nil (ax m iy1 g ax))
("amigo" nil (ax m iy1 g ow1))
("amigos" nil (ax m iy1 g ow1 z))
("amilia" nil (aa0 m iy1 l iy0 ax))
("amin" nil (aa0 m iy1 n))
("amino" nil (ax m iy1 n ow0))
("aminta" nil (ax m ih1 n t ax))
("amiot" nil (ey1 m iy0 ax t))
("amiprilose" nil (ax m ih1 p r ax l ow1 s))
("amir" nil (ax m ih1 r))
("amiram" nil (ae1 m er0 ae0 m))
("amiran" nil (ae1 m ih0 r ax n))
("amirault" nil (ae1 m ay0 r aw0 l t))
("amirav" nil (ae1 m ih0 r ae1 v))
("amis" nil (ae1 m ih0 s))
("amish" nil (aa1 m ih0 sh))
("amison" nil (ae1 m ih0 s ax n))
("amiss" nil (ax m ih1 s))
("amitai" nil (ae1 m ih0 t ay1))
("amity" nil (ae1 m ih0 t iy0))
("amityville" nil (ae1 m ih0 t iy0 v ih1 l))
("amman" nil (ae1 m ax n))
("ammann" nil (ae1 m ax n))
("ammeen" nil (ax m iy1 n))
("ammerman" nil (ae1 m er0 m ax n))
("ammeters" nil (ae1 m iy1 t er0 z))
("ammirati" nil (aa0 m ih0 r aa1 t iy0))
("ammo" nil (ae1 m ow1))
("ammon" nil (ae1 m ax n))
("ammonia" nil (ax m ow1 n y ax))
("ammonite" nil (ae1 m ax n ay1 t))
("ammonites" nil (ae1 m ax n ay1 t s))
("ammonium" nil (ax m ow1 n iy0 ax m))
("ammons" nil (ae1 m ax n z))
("ammunition" nil (ae1 m y ax n ih1 sh ax n))
("amnesia" nil (ae0 m n iy1 zh ax))
("amnesiac" nil (ae0 m n iy1 z iy0 ae1 k))
("amnesties" nil (ae1 m n ax s t iy0 z))
("amnesty" nil (ae1 m n ax s t iy0))
("amniocentesis" nil (ae1 m n iy0 ow0 s eh1 n t iy1 s ih0 s))
("amniotic" nil (ae1 m n iy0 ao0 t ih0 k))
("amo" nil (aa1 m ow0))
("amoco" nil (ae1 m ax k ow0))
("amoeba" nil (ax m iy1 b ax))
("amoebas" nil (ax m iy1 b ax z))
("amoebic" nil (ax m iy1 b ih0 k))
("amok" nil (ax m ah1 k))
("amon" nil (aa0 m ao1 n))
("among" nil (ax m ah1 ng))
("amongst" nil (ax m ah1 ng s t))
("amor" nil (ae1 m er0))
("amoral" nil (ey0 m ao1 r ax l))
("amore" nil (aa1 m ao0 r))
("amorette" nil (ae1 m er0 eh1 t))
("amorist" nil (ae1 m er0 ax s t))
("amorita" nil (aa0 m ao0 r iy1 t ax))
("amorosi" nil (aa0 m ao0 r ow1 s iy0))
("amoroso" nil (aa0 m ao0 r ow1 s ow0))
("amorous" nil (ae1 m er0 ax s))
("amorphous" nil (ax m ao1 r f ax s))
("amortization" nil (ae1 m er0 t ih0 z ey1 sh ax n))
("amortize" nil (ae1 m er0 t ay1 z))
("amortized" nil (ae1 m er0 t ay1 z d))
("amortizing" nil (ae1 m er0 t ay1 z ih0 ng))
("amoruso" nil (ae1 m er0 uw1 s ow0))
("amory" nil (ey1 m er0 iy0))
("amos" nil (ey1 m ax s))
("amoskeag" nil (ae1 m ax s k ey1 g))
("amoss" nil (ax m ao1 s))
("amount" nil (ax m aw1 n t))
("amounted" nil (ax m aw1 n t ih0 d))
("amounting" nil (ax m aw1 n t ih0 ng))
("amounts" nil (ax m aw1 n t s))
("ampad" nil (ae1 m p ae0 d))
("ampal" nil (ae1 m p ax l))
("amparan" nil (ae1 m p er0 ax n))
("ampco" nil (ae1 m p k ow0))
("amperage" nil (ae1 m p er0 ih0 jh))
("ampersand" nil (ae1 m p er0 s ae1 n d))
("ampex" nil (ae1 m p eh1 k s))
("amphenol" nil (ae1 m f ax n ao0 l))
("amphetamine" nil (ae0 m f eh1 t ax m iy1 n))
("amphetamines" nil (ae0 m f eh1 t ax m iy1 n z))
("amphibian" nil (ae0 m f ih1 b iy0 ax n))
("amphibians" nil (ae0 m f ih1 b iy0 ax n z))
("amphibious" nil (ae0 m f ih1 b iy0 ax s))
("amphibole" nil (ae1 m f ax b ow1 l))
("amphitheater" nil (ae1 m f ax th iy1 ax t er0))
("amphitheaters" nil (ae1 m f ax th iy1 ax t er0 z))
("amphitheatre" nil (ae1 m p ax th iy1 ax t er0))
("amphora" nil (ae1 m f er0 ax))
("amphorae" nil (ae1 m f er0 iy1))
("ample" nil (ae1 m p ax l))
("amplification" nil (ae1 m p l ax f ax k ey1 sh ax n))
("amplifications" nil (ae1 m p l ax f ax k ey1 sh ax n z))
("amplified" nil (ae1 m p l ax f ay1 d))
("amplifier" nil (ae1 m p l ax f ay1 er0))
("amplifiers" nil (ae1 m p l ax f ay1 er0 z))
("amplifies" nil (ae1 m p l ax f ay1 z))
("amplify" nil (ae1 m p l ax f ay1))
("amplifying" nil (ae1 m p l ax f ay1 ih0 ng))
("ampligen" nil (ae1 m p l ih0 jh eh0 n))
("amplitude" nil (ae1 m p l ax t uw1 d))
("amply" nil (ae1 m p l iy0))
("ampol" nil (ae1 m p ao0 l))
("amputate" nil (ae1 m p y ax t ey1 t))
("amputated" nil (ae1 m p y ax t ey1 t ih0 d))
("amputation" nil (ae1 m p y ax t ey1 sh ax n))
("amputations" nil (ae1 m p y uw0 t ey1 sh ax n z))
("amputee" nil (ae1 m p y ax t iy1))
("amraam" nil (ae0 m r aa1 m))
("amre" nil (ae1 m r ax))
("amrein" nil (ae1 m r ay0 n))
("amrep" nil (ae1 m r eh0 p))
("amrhein" nil (ae1 m er0 hh ay1 n))
("amrine" nil (ae1 m r iy0 n))
("amritsar" nil (ae0 m r ih1 t s er0))
("amro" nil (ae1 m r ow0))
("ams" nil (ae1 m z))
("amsbaugh" nil (ae1 m z b ao1))
("amsco" nil (ae1 m s k ow0))
("amsden" nil (ae1 m z d ax n))
("amsler" nil (ae1 m z l er0))
("amsouth" nil (ae1 m s aw1 th))
("amspacher" nil (ae1 m s p ax k er0))
("amstar" nil (ae1 m s t aa1 r))
("amster" nil (ae1 m s t er0))
("amsterdam" nil (ae1 m s t er0 d ae1 m))
("amstrad" nil (ae1 m s t r ae1 d))
("amstutz" nil (ae1 m s t ax t s))
("amtech" nil (ae1 m t eh1 k))
("amtrack" nil (ae1 m t r ae1 k))
("amtrak" nil (ae1 m t r ae0 k))
("amuck" nil (ax m ah1 k))
("amulet" nil (ae1 m y ax l ax t))
("amulets" nil (ae1 m y ax l ax t s))
("amundsen" nil (ey1 m ax n d s ax n))
("amundson" nil (ae1 m ax n d s ax n))
("amuse" nil (ax m y uw1 z))
("amused" nil (ax m y uw1 z d))
("amusement" nil (ax m y uw1 z m ax n t))
("amusements" nil (ax m y uw1 z m ax n t s))
("amuses" nil (ax m y uw1 z ih0 z))
("amusing" nil (ax m y uw1 z ih0 ng))
("amusingly" nil (ax m y uw1 z ih0 ng l iy0))
("amvest" nil (ae1 m v eh0 s t))
("amvestors" nil (ae1 m v eh1 s t er0 z))
("amway" nil (ae1 m w ey1))
("amy" nil (ey1 m iy0))
("amylin" nil (ae1 m ih0 l ih1 n))
("amyloid" nil (ae1 m ih0 l oy1 d))
("amyotrophic" nil (ae1 m ay0 ax t r ow1 f ih0 k))
("amythyst" nil (ae1 m ih0 th ih0 s t))
("amyx" nil (ae1 m ih0 k s))
("an" nil (ae1 n))
("ana" nil (aa1 n ax))
("anabaptist" nil (ae1 n ax b ae1 p t ax s t))
("anable" nil (ey1 n ax b ax l))
("anabolic" nil (ae1 n ax b aa1 l ih0 k))
("anac" nil (ae1 n ae0 k))
("anachronism" nil (ax n ae1 k r ax n ih1 z ax m))
("anachronisms" nil (ax n ae1 k r ax n ih1 z ax m z))
("anachronistic" nil (ax n ae1 k r ax n ih1 s t ih0 k))
("anacin" nil (ae1 n ax s ih0 n))
("anacker" nil (ae1 n ax k er0))
("anacomp" nil (ae1 n ax k aa1 m p))
("anaconda" nil (ae1 n ax k aa1 n d ax))
("anacostia" nil (ae1 n ax k aa1 s t iy0 ax))
("anadarko" nil (ae1 n ax d aa1 r k ow0))
("anaerobes" nil (ae1 n er0 ow1 b z))
("anaerobic" nil (ae1 n er0 ow1 b ih0 k))
("anaesthesia" nil (ae1 n ax s th iy1 zh ax))
("anafranil" nil (ax n ae1 f r ax n ih1 l))
("anagnos" nil (aa0 n aa1 g n ow0 z))
("anagram" nil (ae1 n ax g r ae1 m))
("anaheim" nil (ae1 n ax hh ay1 m))
("anal" nil (ey1 n ax l))
("analgesic" nil (ae1 n ax l jh iy1 s ih0 k))
("analgesics" nil (ae1 n ax l jh iy1 z ih0 k s))
("analog" nil (ae1 n ax l ao1 g))
("analogic" nil (ae1 n ax l aa1 jh ih0 k))
("analogies" nil (ax n ae1 l ax jh iy0 z))
("analogous" nil (ax n ae1 l ax g ax s))
("analogue" nil (ae1 n ax l ao1 g))
("analogy" nil (ax n ae1 l ax jh iy0))
("analyses" n (ax n ae1 l ax s iy0 z))
("analyses" v (ah1 n ae0 l ay s ih0 z))
("analysis" nil (ax n ae1 l ax s ax s))
("analyst" nil (ae1 n ax l ax s t))
("analysts" nil (ae1 n ax l ax s t s))
("analytic" nil (ae1 n ax l ih1 t ih0 k))
("analytical" nil (ae1 n ax l ih1 t ih0 k ax l))
("analytically" nil (ae1 n ax l ih1 t ih0 k l iy0))
("analyticity" nil (ae1 n ax l ax t ih1 s ax t iy0))
("analytics" nil (ae1 n ax l ih1 t ih0 k s))
("analyzable" nil (ae1 n ax l ay1 z ax b ax l))
("analyze" nil (ae1 n ax l ay1 z))
("analyzed" nil (ae1 n ax l ay1 z d))
("analyzer" nil (ae1 n ax l ay1 z er0))
("analyzers" nil (ae1 n ax l ay1 z er0 z))
("analyzes" nil (ae1 n ax l ay1 z ih0 z))
("analyzing" nil (ae1 n ax l ay1 z ih0 ng))
("anand" nil (ae1 n ax n d))
("anandeep" nil (aa1 n aa0 n d iy1 p))
("anania" nil (aa0 n aa1 n iy0 ax))
("anantha" nil (ax n ae1 n th ax))
("anaphora" nil (ax n ae1 f er0 ax))
("anarchic" nil (ae0 n aa1 r k ih0 k))
("anarchical" nil (ae0 n aa1 r k ax k ax l))
("anarchist" nil (ae1 n er0 k ax s t))
("anarchy" nil (ae1 n er0 k iy0))
("anaren" nil (ae1 n er0 ax n))
("anast" nil (aa1 n aa0 s t))
("anastas" nil (ae1 n ax s t ax z))
("anastasi" nil (aa0 n aa0 s t aa1 s iy0))
("anastasia" nil (ae0 n ax s t ey1 zh ax))
("anastasio" nil (aa0 n aa0 s t aa1 s iy0 ow0))
("anastos" nil (ae1 n ax s t ow0 z))
("anathema" nil (ax n ae1 th ax m ax))
("anatola" nil (aa0 n aa0 t ow1 l ax))
("anatole" nil (ae1 n ax t ow1 l))
("anatoli" nil (ae1 n ax t ow1 l iy0))
("anatolia" nil (ae1 n ax t ow1 l iy0 ax))
("anatolian" nil (ae1 n ax t ow1 l iy0 ax n))
("anatoly" nil (ae1 n ax t ow1 l iy0))
("anatomical" nil (ae1 n ax t aa1 m ax k ax l))
("anatomically" nil (ae1 n ax t aa1 m ax k l iy0))
("anatomist" nil (ax n ae1 t ax m ax s t))
("anatomy" nil (ax n ae1 t ax m iy0))
("anaya" nil (aa0 n aa1 y ax))
("anaylsts" nil (ax n ey1 l s t s))
("ancel" nil (ax n s eh1 l))
("ancell" nil (aa0 n s ey1 l))
("ancestor" nil (ae1 n s eh1 s t er0))
("ancestors" nil (ae1 n s eh1 s t er0 z))
("ancestral" nil (ae0 n s eh1 s t r ax l))
("ancestry" nil (ae1 n s eh0 s t r iy0))
("ancheta" nil (aa0 n k eh1 t ax))
("anchondo" nil (aa0 n k ow1 n d ow0))
("anchor" nil (ae1 ng k er0))
("anchorage" nil (ae1 ng k er0 ax jh))
("anchored" nil (ae1 ng k er0 d))
("anchoring" nil (ae1 ng k er0 ih0 ng))
("anchorman" nil (ae1 ng k er0 m ae1 n))
("anchormen" nil (ae1 ng k er0 m eh1 n))
("anchors" nil (ae1 ng k er0 z))
("anchovies" nil (ae0 n ch ow1 v iy0 z))
("anchovy" nil (ae0 n ch ow1 v iy0))
("ancient" nil (ey1 n ch ax n t))
("ancients" nil (ey1 n ch ax n t s))
("ancillary" nil (ae1 n s ax l eh1 r iy0))
("ancira" nil (aa0 n ch ih1 r ax))
("ancona" nil (aa0 n k ow1 n ax))
("ancrum" nil (ax n k r ah1 m))
("anctil" nil (ae1 ng k t ih0 l))
("and" nil (ae1 n d))
("andal" nil (ae1 n d ax l))
("andalusia" nil (ae1 n d ax l uw1 zh ax))
("andalusian" nil (ae1 n d ax l uw1 sh ax n))
("andante" nil (aa0 n d aa1 n t ey0))
("andantino" nil (aa1 n d aa1 n t iy1 n ow0))
("andean" nil (ae1 n d iy0 ax n))
("andel" nil (ae1 n d ax l))
("ander" nil (ae1 n d er0))
("anderberg" nil (ae1 n d er0 b er0 g))
("anderegg" nil (ae1 n d er0 ih0 g))
("anderle" nil (ae1 n d er0 ax l))
("anderlini" nil (ae1 n d er0 l iy1 n iy0))
("anderman" nil (ae1 n d er0 m ax n))
("anders" nil (ae1 n d er0 z))
("andersen" nil (ae1 n d er0 s ax n))
("anderson" nil (ae1 n d er0 s ax n))
("andersons" nil (ae1 n d er0 s ax n z))
("andersson" nil (ae1 n d er0 s ax n))
("andert" nil (ae1 n d er0 t))
("anderton" nil (ae1 n d er0 t ax n))
("andes" nil (ae1 n d iy0 z))
("andesite" nil (ae1 n d ih0 s ay1 t))
("andie" nil (ae1 n d iy0))
("anding" nil (ae1 n d ih0 ng))
("andino" nil (aa0 n d iy1 n ow0))
("andiron" nil (ae1 n d ay1 er0 n))
("andis" nil (ae1 n d ih0 s))
("andler" nil (ae1 n d l er0))
("ando" nil (ae1 n d ow0))
("andolina" nil (aa0 n d ow0 l iy1 n ax))
("andonian" nil (ae1 n d ow1 n iy0 ax n))
("andorra" nil (ae0 n d ao1 r ax))
("andover" nil (ae1 n d ow0 v er0))
("andrada" nil (aa0 n d r aa1 d ax))
("andrade" nil (ae1 n d r ey1 d))
("andrae" nil (aa1 n d r ay0))
("andras" nil (aa1 n d r ax s))
("andre" nil (aa1 n d r ey1))
("andrea" nil (ae1 n d r iy0 ax))
("andreae" nil (aa0 n d r ey1 aa0))
("andreana" nil (ae1 n d r iy1 n ax))
("andreano" nil (ae1 n d r iy1 n ow0))
("andreas" nil (aa0 n d r ey1 ax s))
("andreasen" nil (ae1 n d r iy0 s ax n))
("andreason" nil (ae1 n d r iy1 s ax n))
("andreassen" nil (ae1 n d r ax s ax n))
("andree" nil (ax n d r iy1))
("andreen" nil (ax n d r iy1 n))
("andrei" nil (ae1 n d r ey1))
("andreini" nil (aa0 n d r ey0 iy1 n iy0))
("andrej" nil (aa1 n d r ey0))
("andren" nil (ae1 n d er0 ax n))
("andreoli" nil (aa0 n d r ey0 ow1 l iy0))
("andreoni" nil (aa0 n d r ey0 ow1 n iy0))
("andreotti" nil (aa0 n d r iy0 aa1 t iy0))
("andreozzi" nil (aa0 n d r iy0 aa1 t s iy0))
("andrepont" nil (aa0 n d r ey1 p ow0 n t))
("andres" nil (aa1 n d r ey0 z))
("andresen" nil (ae0 n d r iy1 s ax n))
("andreski" nil (ae0 n d r eh1 s k iy0))
("andress" nil (aa1 n d r eh0 s))
("andretti" nil (ae1 n d r eh1 t iy0))
("andreu" nil (ae1 n d r uw0))
("andrew" nil (ae1 n d r uw0))
("andrews" nil (ae1 n d r uw1 z))
("andrey" nil (ae1 n d r iy0))
("andrezak" nil (ae1 n d r ax z ae0 k))
("andria" nil (ae1 n d r iy0 ax))
("andriana" nil (aa0 n d r iy0 ae1 n ax))
("andriano" nil (aa0 n d r iy0 aa1 n ow0))
("andrich" nil (ae1 n d r ih0 k))
("andrick" nil (ae1 n d r ih0 k))
("andries" nil (ax n d r iy1 z))
("andriessen" nil (ae1 n d r iy1 s ax n))
("andringa" nil (aa0 n d r iy1 ng g ax))
("andriola" nil (aa0 n d r iy0 ow1 l ax))
("andrist" nil (ae1 n d r ih0 s t))
("androgynous" nil (ae0 n d r ao1 jh ax n ax s))
("android" nil (ae1 n d r oy1 d))
("andromeda" nil (ae0 n d r aa1 m ax d ax))
("andropov" nil (ae0 n d r aa1 p aa1 v))
("andros" nil (ae1 n d r aa0 s))
("andrus" nil (ae1 n d r ax s))
("andruskevich" nil (ae1 n d r ax s eh1 v ih0 ch))
("andry" nil (ae1 n d r iy0))
("andrzej" nil (aa1 n d r ey1))
("andrzejewski" nil (ax n jh ey0 eh1 f s k iy0))
("ands" nil (ae1 n d z))
("andujar" nil (aa0 n d uw0 y aa1 r))
("andy" nil (ae1 n d iy0))
("anecdotal" nil (ae1 n ax k d ow1 t ax l))
("anecdote" nil (ae1 n ax k d ow1 t))
("anecdotes" nil (ae1 n ax k d ow1 t s))
("anello" nil (ax n eh1 l ow0))
("anemia" nil (ax n iy1 m iy0 ax))
("anemias" nil (ax n iy1 m iy0 ax z))
("anemic" nil (ax n iy1 m ih0 k))
("anemometer" nil (ae1 n ax m aa1 m ax t er0))
("anemone" nil (ae1 n ih0 m ow1 n))
("anencephalic" nil (ae0 n eh1 n s ax f ae1 l ih0 k))
("anesthesia" nil (ae1 n ih0 s th iy1 zh ax))
("anesthesiologist" nil (ae1 n ax s th iy1 z iy0 aa1 l ax jh ax s t))
("anesthesiologists" nil (ae1 n ax s th iy1 z iy0 aa1 l ax jh ax s t s))
("anesthesiology" nil (ae1 n ax s th iy0 z iy0 aa1 l ax jh iy0))
("anesthetic" nil (ae1 n ax s th eh1 t ih0 k))
("anesthetist" nil (ax n eh1 s th ax t ax s t))
("anette" nil (ax n eh1 t))
("anew" nil (ax n uw1))
("anfinson" nil (ae1 n f ih0 n s ax n))
("ang" nil (ae1 ng))
("ange" nil (ey1 n jh))
("angel" nil (ey1 n jh ax l))
("angela" nil (ae1 n jh ax l ax))
("angelenos" nil (ae1 n jh ax l iy1 n ow0 z))
("angeles" nil (ae1 n jh ax l ih0 s))
("angeletti" nil (aa0 ng g eh0 l eh1 t iy0))
("angelfish" nil (ey1 n jh ax l f ih1 sh))
("angeli" nil (aa0 ng g eh1 l iy0))
("angelic" nil (ae1 n jh eh1 l ih0 k))
("angelica" nil (ae0 n jh eh1 l ih0 k ax))
("angelically" nil (ae0 n jh eh1 l ih0 k l iy0))
("angelicas" nil (ae0 n jh eh1 l ih0 k ax z))
("angelico" nil (aa0 ng g eh0 l iy1 k ow0))
("angelika" nil (ae0 n jh eh1 l ih1 k ax))
("angelillo" nil (aa0 ng g eh0 l ih1 l ow0))
("angeline" nil (ey1 ng g ih0 l ay0 n))
("angelini" nil (aa0 ng g eh0 l iy1 n iy0))
("angelino" nil (ae1 n jh ax l iy1 n ow0))
("angelita" nil (aa0 ng g eh0 l iy1 t ax))
("angell" nil (ey1 n jh ax l))
("angelle" nil (ey0 ng g eh1 l))
("angello" nil (ae1 n jh eh1 l ow0))
("angelo" nil (ae1 n jh ax l ow1))
("angeloff" nil (ey1 ng g ih0 l ao0 f))
("angelone" nil (aa0 ng g eh0 l ow1 n iy0))
("angeloni" nil (aa0 ng g eh0 l ow1 n iy0))
("angelos" nil (ae1 n jh ax l ow1 z))
("angeloz" nil (ae1 n jh ax l ow1 z))
("angels" nil (ey1 n jh ax l z))
("angelucci" nil (aa0 ng g eh0 l uw1 ch iy0))
("anger" nil (ae1 ng g er0))
("angered" nil (ae1 ng g er0 d))
("angerer" nil (ae1 ng g er0 er0))
("angering" nil (ae1 ng g er0 ih0 ng))
("angerman" nil (ae1 ng er0 m ax n))
("angermeier" nil (ey1 ng g er0 m ay0 er0))
("angers" nil (ae1 ng g er0 z))
("angert" nil (ey1 ng g er0 t))
("angevine" nil (ey1 ng g ih0 v ay0 n))
("angi" nil (ae1 n jh iy0))
("angie" nil (ae1 n jh iy0))
("angier" nil (ae1 n jh iy0 er0))
("angina" nil (ae0 n jh ay1 n ax))
("angio" nil (ae1 n jh iy0 ow0))
("angiomedics" nil (ae1 n jh iy0 ow0 m eh1 d ih0 k s))
("angioplasty" nil (ae1 n jh iy0 ax p l ae1 s t iy0))
("angiotensin" nil (ae1 n jh iy0 ow0 t eh1 n s ih0 n))
("angle" nil (ae1 ng g ax l))
("angled" nil (ae1 ng g ax l d))
("anglemyer" nil (ae1 ng g l ih0 m iy0 er0))
("anglen" nil (ae1 ng g ax l ax n))
("angler" nil (ae1 ng g l er0))
("anglers" nil (ae1 ng g l er0 z))
("angles" nil (ae1 ng g ax l z))
("angleton" nil (ae1 ng g ax l t ax n))
("angley" nil (ae1 ng g l iy0))
("anglia" nil (ae1 ng g l iy0 ax))
("anglican" nil (ae1 ng g l ax k ax n))
("anglim" nil (ae1 ng g l ih0 m))
("anglin" nil (ae1 ng g l ih0 n))
("angling" nil (ae1 ng g l ih0 ng))
("anglo" nil (ae1 ng g l ow0))
("anglos" nil (ae1 ng g l ow0 z))
("angola" nil (ae0 ng g ow1 l ax))
("angolan" nil (ae1 ng g ow0 l ax n))
("angolans" nil (ae1 ng g ow0 l ax n z))
("angora" nil (ae0 ng g ao1 r ax))
("angoras" nil (ae0 ng g ao1 r ax z))
("angotti" nil (aa0 ng g ow1 t iy0))
("angove" nil (aa0 ng g ow1 v iy0))
("angrier" nil (ae1 ng g r iy0 er0))
("angrily" nil (ae1 ng g r ax l iy0))
("angry" nil (ae1 ng g r iy0))
("angst" nil (aa1 ng k s t))
("angstadt" nil (ae1 ng sh t ae0 t))
("angstrom" nil (ae1 ng s t r ax m))
("angstroms" nil (ae1 ng s t r ax m z))
("anguiano" nil (aa0 n g iy0 aa1 n ow0))
("anguilla" nil (ae1 ng w ih1 l ax))
("anguish" nil (ae1 ng g w ih0 sh))
("anguished" nil (ae1 ng g w ih0 sh t))
("anguishing" nil (ae1 ng g w ih0 sh ih0 ng))
("angular" nil (ae1 ng g y ax l er0))
("angulo" nil (aa0 ng g uw1 l ow0))
("angus" nil (ae1 ng g ax s))
("angy" nil (ae1 n jh iy0))
("anhalt" nil (ae1 n hh ax l t))
("anheuser" nil (ae1 n hh ay1 z er0))
("anhydride" nil (ae0 n hh ay1 d r ih0 d))
("anibal" nil (ae1 n ih0 b ax l))
("anikst" nil (ae1 n ih0 k s t))
("aniline" nil (ae1 n ax l iy1 n))
("animal" nil (ae1 n ax m ax l))
("animals" nil (ae1 n ax m ax l z))
("animate" j (ae1 n ax m ax t))
("animate" v (ae1 n ax m ey1 t))
("animated" nil (ae1 n ax m ey1 t ax d))
("animates" nil (ae1 n ax m ax t s))
("animation" nil (ae1 n ax m ey1 sh ax n))
("animator" nil (ae1 n ax m ey1 t er0))
("animators" nil (ae1 n ax m ey1 t er0 z))
("animism" nil (ae1 n ax m ih1 z ax m))
("animists" nil (ae1 n ax m ax s t s))
("animosities" nil (ae1 n ax m aa1 s ax t iy0 z))
("animosity" nil (ae1 n ax m aa1 s ax t iy0))
("animus" nil (ae1 n ih0 m ax s))
("aninat" nil (ae1 n ih0 n ae0 t))
("anise" nil (ae1 n ax s))
("aniseed" nil (ae1 n ax s iy1 d))
("anisette" nil (ae1 n ax s eh1 t))
("anita" nil (ax n iy1 t ax))
("anitec" nil (ae1 n ih0 t eh1 k))
("anixter" nil (ae1 n ih0 k s t er0))
("anja" nil (aa1 n jh ax))
("ankara" nil (ae1 ng k er0 ax))
("ankeney" nil (ax ng k eh1 n iy0))
("ankeny" nil (ax ng k iy1 n iy0))
("anker" nil (ae1 ng k er0))
("anklam" nil (ae1 ng k l ax m))
("ankle" nil (ae1 ng k ax l))
("anklebone" nil (ae1 ng k ax l b ow1 n))
("ankles" nil (ae1 ng k ax l z))
("anklet" nil (ae1 ng k l ih0 t))
("anklets" nil (ae1 ng k l ax t s))
("ankney" nil (ae1 ng k n iy0))
("ankrom" nil (ae1 ng k r ax m))
("ankrum" nil (ae1 ng k r ax m))
("ann" nil (ae1 n))
("anna" nil (ae1 n ax))
("annabel" nil (ae1 n ax b eh1 l))
("annabella" nil (ae1 n ax b eh1 l ax))
("annabelle" nil (ae1 n ax b ax l))
("annable" nil (ae1 n ax b ax l))
("annals" nil (ae1 n ax l z))
("annamese" nil (ae1 n ax m iy1 z))
("annan" nil (ae1 n ax n))
("annandale" nil (ae1 n ax n d ey1 l))
("annapolis" nil (ax n ae1 p ax l ih0 s))
("annas" nil (ae1 n ax z))
("anne" nil (ae1 n))
("annealing" nil (ax n iy1 l ih0 ng))
("annelid" nil (ae1 n ax l ih0 d))
("annelids" nil (ae1 n ax l ih0 d z))
("annen" nil (ae1 n ax n))
("annenberg" nil (ae1 n ax n b er0 g))
("annese" nil (aa0 n ey1 z iy0))
("annett" nil (ax n eh1 t))
("annette" nil (ax n eh1 t))
("annex" n (ae1 n eh0 k s))
("annex" v (ax n eh1 k s))
("annexation" nil (ae1 n eh0 k s ey1 sh ax n))
("annexed" nil (ae1 n eh0 k s t))
("annexes" n (ae1 n eh0 k s ih0 z))
("annexes" v (ax n eh1 k s ih0 z))
("annexing" nil (ax n eh1 k s ih0 ng))
("annick" nil (ae1 n ih0 k))
("annie" nil (ae1 n iy0))
("annihilate" nil (ax n ay1 ax l ey1 t))
("annihilated" nil (ax n ay1 ax l ey1 t ih0 d))
("annihilation" nil (ax n ay1 ax l ey1 sh ax n))
("annis" nil (ae1 n iy0 z))
("anniversaries" nil (ae1 n ax v er1 s er0 iy0 z))
("anniversary" nil (ae1 n ax v er1 s er0 iy0))
("annotate" nil (ae1 n ax t ey1 t))
("annotated" nil (ae1 n ax t ey1 t ih0 d))
("announce" nil (ax n aw1 n s))
("announced" nil (ax n aw1 n s t))
("announcement" nil (ax n aw1 n s m ax n t))
("announcements" nil (ax n aw1 n s m ax n t s))
("announcer" nil (ax n aw1 n s er0))
("announcers" nil (ax n aw1 n s er0 z))
("announces" nil (ax n aw1 n s ih0 z))
("announcing" nil (ax n aw1 n s ih0 ng))
("announcment" nil (ax n aw1 n s m ax n t))
("annoy" nil (ax n oy1))
("annoyance" nil (ax n oy1 ax n s))
("annoyances" nil (ax n oy1 ax n s ih0 z))
("annoyed" nil (ax n oy1 d))
("annoying" nil (ax n oy1 ih0 ng))
("annoys" nil (ax n oy1 z))
("annual" nil (ae1 n y uw0 ax l))
("annualized" nil (ae1 n y uw0 w ax l ay1 z d))
("annually" nil (ae1 n y uw0 ax l iy0))
("annuals" nil (ae1 n uw0 ax l z))
("annuities" nil (ax n uw1 ih0 t iy0 z))
("annuity" nil (ax n uw1 ax t iy0))
("annul" nil (ae1 n ax l))
("annular" nil (ae1 n y ax l er0))
("annulled" nil (ae1 n ax l d))
("annulment" nil (ae1 n ax l m ax n t))
("annum" nil (ae1 n ax m))
("annunciata" nil (aa0 n uw0 n ch aa1 t ax))
("annunziata" nil (aa0 n uw0 n z iy0 aa1 t ax))
("annunziato" nil (aa0 n uw0 n z iy0 aa1 t ow0))
("annunzio" nil (ax n ah1 n z iy0 ow0))
("anny" nil (ae1 n iy0))
("anode" nil (ae1 n ow1 d))
("anodes" nil (ae1 n ow1 d z))
("anointed" nil (ax n oy1 n t ax d))
("anomalies" nil (ax n aa1 m ax l iy0 z))
("anomalous" nil (ax n aa1 m ax l ax s))
("anomaly" nil (ax n aa1 m ax l iy0))
("anomie" nil (ae1 n ax m iy0))
("anona" nil (aa0 n ow1 n ax))
("anonymity" nil (ae1 n ax n ih1 m ih0 t iy0))
("anonymous" nil (ax n aa1 n ax m ax s))
("anonymously" nil (ax n aa1 n ax m ax s l iy0))
("anora" nil (aa0 n ao1 r ax))
("anorak" nil (ae1 n er0 ae1 k))
("anorexia" nil (ae1 n er0 eh1 k s iy0 ax))
("anorexic" nil (ae1 n er0 eh1 k s ih0 k))
("anorthite" nil (ae0 n ao1 r th ay1 t))
("another" nil (ax n ah1 dh er0))
("anreder" nil (ae1 n r eh1 d er0))
("anrig" nil (ae1 n r ih0 g))
("ansa" nil (ae1 n s ax))
("ansaid" nil (ae1 n s ey1 d))
("ansaldo" nil (ae0 n s aa1 l d ow0))
("ansari" nil (aa0 n s aa1 r iy0))
("ansbacher" nil (ae1 n z b aa1 k er0))
("anschel" nil (ae1 n sh ax l))
("anschluss" nil (ae1 n sh l ax s))
("anschutz" nil (ae1 n sh ax t s))
("anscom" nil (ax n s k aa1 m))
("anscomb" nil (ax n s k aa1 m))
("anse" nil (ae1 n s))
("ansel" nil (ax n s eh1 l))
("ansell" nil (ae1 n s eh1 l))
("anselm" nil (ae1 n s eh0 l m))
("anselma" nil (aa0 n s eh1 l m ax))
("anselmi" nil (aa0 n s eh1 l m iy0))
("anselmo" nil (aa0 n s eh1 l m ow0))
("ansett" nil (ae1 n s eh1 t))
("anshan" nil (ae1 n sh ax n))
("ansin" nil (ae1 n s ih0 n))
("ansley" nil (ae1 n s l iy0))
("anson" nil (ae1 n s ax n))
("ansonia" nil (ae0 n s ow1 n iy0 ax))
("anspach" nil (ae1 n s p aa1 k))
("anspaugh" nil (ax n s p ao1))
("anstead" nil (ae1 n s t eh1 d))
("anstett" nil (ax n s t eh1 t))
("anstey" nil (ae1 n s t iy0))
("anstice" nil (aa1 n s t ih0 s))
("anstine" nil (aa0 n s t iy1 n iy0))
("anstiss" nil (ae1 n s t ih0 s))
("answer" nil (ae1 n s er0))
("answerable" nil (ae1 n s er0 ax b ax l))
("answered" nil (ae1 n s er0 d))
("answering" nil (ae1 n s er0 ih0 ng))
("answers" nil (ae1 n s er0 z))
("ant" nil (ae1 n t))
("antacid" nil (ae0 n t ae1 s ax d))
("antacids" nil (ae0 n t ae1 s ax d z))
("antagonism" nil (ae0 n t ae1 g ax n ih1 z ax m))
("antagonisms" nil (ae0 n t ae1 g ax n ih1 z ax m z))
("antagonist" nil (ae0 n t ae1 g ax n ax s t))
("antagonistic" nil (ae0 n t ae1 g ax n ih1 s t ih0 k))
("antagonists" nil (ae0 n t ae1 g ax n ax s t s))
("antagonize" nil (ae0 n t ae1 g ax n ay1 z))
("antagonized" nil (ae0 n t ae1 g ax n ay1 z d))
("antagonizing" nil (ae0 n t ae1 g ax n ay1 z ih0 ng))
("antal" nil (ae1 n t ax l))
("antar" nil (ae1 n t er0))
("antarctic" nil (ae0 n t aa1 r k t ih0 k))
("antarctica" nil (ae1 n t aa1 r k t ih0 k ax))
("antares" nil (ae0 n t eh1 r iy0 z))
("antaya" nil (aa0 n t ey1 ax))
("antczak" nil (ae1 n t ch ae0 k))
("ante" nil (ae1 n t iy0))
("anteaters" nil (ae1 n t iy1 t er0 z))
("antebellum" nil (ae1 n t ih0 b eh1 l ax m))
("antec" nil (ae1 n t eh1 k))
("antecedents" nil (ae1 n t ax s iy1 d ax n t s))
("anted" nil (ae1 n t ih0 d))
("antell" nil (ae0 n t eh1 l))
("antelope" nil (ae1 n t ax l ow1 p))
("antenna" nil (ae0 n t eh1 n ax))
("antennae" nil (ae0 n t eh1 n iy0))
("antennas" nil (ae0 n t eh1 n ax z))
("antenne" nil (ae0 n t eh1 n))
("antenucci" nil (ae0 n t ih0 n uw1 ch iy0))
("anterior" nil (ae0 n t ih1 r iy0 er0))
("anteriormost" nil (ae0 n t ih1 r iy0 er0 m ow1 s t))
("antes" nil (ae1 n t iy0 z))
("anthea" nil (ae1 n th iy0 ax))
("anthem" nil (ae1 n th ax m))
("anthems" nil (ae1 n th ax m z))
("anthes" nil (ae1 n dh z))
("anthiel" nil (ae1 n th iy0 ax l))
("anthills" nil (ae1 n t hh ih1 l z))
("anthis" nil (ae1 n th ih0 s))
("anthologies" nil (ae0 n th aa1 l ax jh iy0 z))
("anthology" nil (ae0 n th aa1 l ax jh iy0))
("anthon" nil (ae1 n th ax n))
("anthony" nil (ae1 n th ax n iy0))
("anthracite" nil (ae1 n th r ax s ay1 t))
("anthrax" nil (ae1 n th r ae0 k s))
("anthropocentric" nil (ae1 n th r ax p ax s eh1 n t r ih0 k))
("anthropological" nil (ae1 n th r ax p ax l aa1 jh ax k ax l))
("anthropologist" nil (ae1 n th r ax p aa1 l ax jh ax s t))
("anthropologists" nil (ae1 n th r ax p aa1 l ax jh ax s t s))
("anthropology" nil (ae1 n th r ax p aa1 l ax jh iy0))
("anthropomorphic" nil (ae1 n th r ax p ax m ao1 r f ih0 k))
("anti" nil (ae1 n t iy0))
("antiabortion" nil (ae1 n t iy0 ax b ao1 r sh ax n))
("antiaircraft" nil (ae1 n t ay0 eh1 r k r ae1 f t))
("antibacterial" nil (ae1 n t iy0 b ae0 k t ih1 r iy0 ax l))
("antiballistic" nil (ae1 n t iy0 b ax l ih1 s t ih0 k))
("antibiotic" nil (ae1 n t iy0 b iy0 aa1 t ih0 k))
("antibioticos" nil (ae1 n t iy0 b iy0 aa1 t ih0 k ow0 s))
("antibiotics" nil (ae1 n t iy0 b iy0 aa1 t ih0 k s))
("antibodies" nil (ae1 n t ih0 b aa1 d iy0 z))
("antibody" nil (ae1 n t ih0 b aa1 d iy0))
("antic" nil (ae1 n t ih0 k))
("anticancer" nil (ae1 n t ay1 k ae1 n s er0))
("anticipate" nil (ae0 n t ih1 s ax p ey1 t))
("anticipated" nil (ae0 n t ih1 s ax p ey1 t ax d))
("anticipates" nil (ae0 n t ih1 s ax p ey1 t s))
("anticipating" nil (ae0 n t ih1 s ax p ey1 t ih0 ng))
("anticipation" nil (ae0 n t ih1 s ax p ey1 sh ax n))
("anticipatory" nil (ae0 n t ih1 s ax p ax t ao1 r iy0))
("anticlimactic" nil (ae1 n t iy0 k l ay0 m ae1 k t ih0 k))
("anticline" nil (ae1 n t ih0 k l ay1 n))
("antico" nil (aa0 n t iy1 k ow0))
("anticommunist" nil (ae1 n t iy0 k aa1 m y ax n ih0 s t))
("anticompetitive" nil (ae1 n t ih0 k ax m p eh1 t ih0 t ih0 v))
("anticorruption" nil (ae1 n t ay1 k er0 ah1 p sh ax n))
("anticrime" nil (ae1 n t iy0 k r ay1 m))
("antics" nil (ae1 n t ih0 k s))
("antidepressant" nil (ae1 n t iy0 d ih0 p r eh1 s ax n t))
("antidepressants" nil (ae1 n t ay1 d ih0 p r eh1 s ax n t s))
("antidiscrimination" nil (ae1 n t iy0 d ih0 s k r ih1 m ax n ey1 sh ax n))
("antidote" nil (ae1 n t ih0 d ow1 t))
("antidrug" nil (ae1 n t ih0 d r ah1 g))
("antidumping" nil (ae1 n t iy0 d ah1 m p ih0 ng))
("antifraud" nil (ae1 n t iy0 f r aa1 d))
("antifreeze" nil (ae1 n t iy0 f r iy1 z))
("antifungal" nil (ae1 n t ay1 f ah1 ng g ax l))
("antigay" nil (ae1 n t ay1 g ey1))
("antigen" nil (ae1 n t ax jh ax n))
("antigens" nil (ae1 n t ih0 jh ax n z))
("antigone" nil (ae0 t ih1 g ax n iy0))
("antigones" nil (ae0 n t ih1 g ax n iy1 z))
("antigovernment" nil (ae1 n t iy0 g ah1 v er0 m ax n t))
("antigua" nil (ae0 n t iy1 g w ax))
("antihistamine" nil (ae1 n t iy0 hh ih1 s t ax m ax n))
("antihistamines" nil (ae1 n t iy0 hh ih1 s t ax m ax n z))
("antiknock" nil (ae1 n t iy0 n aa1 k))
("antill" nil (ae0 n t ih1 l))
("antilla" nil (aa0 n t ih1 l ax))
("antilles" nil (ae0 n t ih1 l iy0 z))
("antilock" nil (ae1 n t iy0 l aa1 k))
("antimissile" nil (ae1 n t ay1 m ih1 s ax l))
("antioch" nil (ae1 n t iy0 aa1 k))
("antioxidant" nil (ae1 n t iy0 aa1 k s ax d ax n t))
("antioxidants" nil (ae1 n t iy0 aa1 k s ax d ax n t s))
("antipathies" nil (ae0 n t ih1 p ax th iy0 z))
("antipathy" nil (ae0 n t ih1 p ax th iy0))
("antiphons" nil (ae1 n t ax f aa1 n z))
("antipodal" nil (ae0 n t ih1 p ax d ax l))
("antipoverty" nil (ae1 n t ay0 p aa1 v er0 t iy0))
("antiquated" nil (ae1 n t ax k w ey1 t ax d))
("antique" nil (ae0 n t iy1 k))
("antiques" nil (ae0 n t iy1 k s))
("antiquities" nil (ae0 n t ih1 k w ax t iy0 z))
("antiquity" nil (ae0 n t ih1 k w ax t iy0))
("antireformer" nil (ae1 n t iy0 r ih0 f ao1 r m er0))
("antireformers" nil (ae1 n t iy0 r ih0 f ao1 r m er0 z))
("antis" nil (ae1 n t ay0 z))
("antisense" nil (ae1 n t iy0 s eh1 n s))
("antiseptic" nil (ae1 n t ax s eh1 p t ih0 k))
("antismoking" nil (ae1 n t iy0 s m ow1 k ih0 ng))
("antisocial" nil (ae1 n t ih0 s ow1 sh ax l))
("antisubmarine" nil (ae1 n t ih0 s ah1 b m er0 iy1 n))
("antitakeover" nil (ae1 n t iy0 t ey1 k ow1 v er0))
("antitank" nil (ae1 n t iy0 t ae1 ng k))
("antitax" nil (ae1 n t ay1 t ae1 k s))
("antitheft" nil (ae1 n t ay1 th eh1 f t))
("antithesis" nil (ae0 n t ih1 th ax s ax s))
("antithetical" nil (ae1 n t ax th eh1 t ih0 k ax l))
("antitoxins" nil (ae1 n t iy0 t aa1 k s ax n z))
("antitrust" nil (ae1 n t ay0 t r ah1 s t))
("antiviral" nil (ae1 n t iy0 v ay1 r ax l))
("antiwar" nil (ae1 n t ay0 w ao1 r))
("antkowiak" nil (ax n t k aw1 iy0 ae0 k))
("antle" nil (ae1 n t ax l))
("antlered" nil (ae1 n t l er0 d))
("antlers" nil (ae1 n t l er0 z))
("antley" nil (ae1 n t l iy0))
("antoine" nil (aa0 n t w aa1 n))
("antoinette" nil (ae1 n t w ax n eh1 t))
("antol" nil (aa0 n t ao1 l))
("antolik" nil (ae1 n t ax l ih0 k))
("antolini" nil (ae1 n t ow0 l iy1 n iy0))
("anton" nil (ae1 n t ao1 n))
("antonacci" nil (aa0 n t ow0 n aa1 ch iy0))
("antone" nil (aa0 n t ow1 n iy0))
("antonelli" nil (aa0 n t ow0 n eh1 l iy0))
("antonellis" nil (ae0 n t ax n eh1 l ih0 s))
("antonetti" nil (aa0 n t ow0 n eh1 t iy0))
("antoni" nil (aa0 n t ow1 n iy0))
("antonia" nil (ae0 n t ow1 n iy0 ax))
("antonin" nil (ae1 n t ax n ih0 n))
("antonini" nil (aa0 n t ow0 n iy1 n iy0))
("antonio" nil (ae0 n t ow1 n iy0 ow0))
("antoniou" nil (aa0 n t ow0 n iy1 uw0))
("antoniu" nil (ae1 n t ow1 n iy0 uw0))
("antonius" nil (ae0 n t ow1 n iy0 ax s))
("antonopoulos" nil (ae0 n t ax n aa1 p ax l ih0 s))
("antonovich" nil (ae1 n t aa1 n ax v ih0 ch))
("antonsen" nil (ax n t aa1 n s ax n))
("antonson" nil (ae1 n t ax n s ax n))
("antonucci" nil (aa0 n t ow0 n uw1 ch iy0))
("antony" nil (ae1 n t ax n iy0))
("antoon" nil (ae1 n t uw1 n))
("antos" nil (aa1 n t ow0 z))
("antosh" nil (ax n t aa1 sh))
("antril" nil (ae1 n t r ih0 l))
("antrim" nil (ae1 n t r ih0 m))
("antrobus" nil (ae1 n t r ax b ih0 s))
("ants" nil (ae1 n t s))
("antsy" nil (ae1 n t s iy0))
("anttila" nil (aa0 n t iy1 l ax))
("antuna" nil (aa0 n t uw1 n ax))
("antunes" nil (aa0 n t uw1 n eh0 s))
("antunez" nil (aa0 n t uw1 n eh0 z))
("antwerp" nil (ae1 n t w er0 p))
("antwine" nil (ae1 n t w ay1 n))
("anvil" nil (ae1 n v ax l))
("anwar" nil (ae1 n w aa0 r))
("anway" nil (ax n w ey1))
("anwell" nil (ax n w eh1 l))
("anwyl" nil (ae1 n w ih0 l))
("anwyll" nil (ae1 n w ih0 l))
("anxieties" nil (ae0 ng z ay1 ax t iy0 z))
("anxiety" nil (ae0 ng z ay1 ax t iy0))
("anxious" nil (ae1 ng k sh ax s))
("anxiously" nil (ae1 ng k sh ax s l iy0))
("any" nil (eh1 n iy0))
("anybody" nil (eh1 n iy0 b ax d iy0))
("anyhow" nil (eh1 n iy0 hh aw1))
("anymore" nil (eh1 n iy0 m ao1 r))
("anyon" nil (eh1 n iy0 aa0 n))
("anyone" nil (eh1 n iy0 w ah1 n))
("anyplace" nil (eh1 n iy0 p l ey1 s))
("anything" nil (eh1 n iy0 th ih1 ng))
("anytime" nil (eh1 n iy0 t ay1 m))
("anyway" nil (eh1 n iy0 w ey1))
("anyways" nil (eh1 n iy0 w ey1 z))
("anywhere" nil (eh1 n iy0 w eh1 r))
("anza" nil (ae1 n z ax))
("anzaldua" nil (aa0 n z aa0 l d uw1 ax))
("anzalone" nil (ae1 n z ax l ow1 n))
("anzelmo" nil (aa0 n z eh1 l m ow0))
("anzus" nil (ae1 n z ax s))
("aoi" nil (aw1 iy0))
("aoki" nil (ey0 ow1 k iy0))
("aon" nil (ey1 ow0 n))
("aorta" nil (ey0 ao1 r t ax))
("aoshima" nil (aw1 sh iy1 m ax))
("aoun" nil (aw1 ax n))
("aouzou" nil (aw1 y uw1 z uw0))
("aoyama" nil (aw1 y aa1 m ax))
("apace" nil (ax p ey1 s))
("apache" nil (ax p ae1 ch iy0))
("apaches" nil (ax p ae1 ch iy0 z))
("apalachicola" nil (ae1 p ax l ae1 ch ax k ow1 l ax))
("aparicio" nil (ae1 p er0 ih1 s iy0 ow0))
("apart" nil (ax p aa1 r t))
("apartheid" nil (ax p aa1 r t ay1 t))
("apartment" nil (ax p aa1 r t m ax n t))
("apartments" nil (ax p aa1 r t m ax n t s))
("apathetic" nil (ae1 p ax th eh1 t ih0 k))
("apathy" nil (ae1 p ax th iy0))
("apatite" nil (ae1 p ax t ay1 t))
("apatites" nil (ae1 p ax t ay1 t s))
("apcar" nil (ae1 p k aa0 r))
("ape" nil (ey1 p))
("apec" nil (ey1 p eh1 k))
("apel" nil (aa0 p eh1 l))
("apelike" nil (ey1 p l ay1 k))
("apennine" nil (ae1 p ax n iy1 n))
("aperture" nil (ae1 p er0 ch er0))
("apes" nil (ey1 p s))
("apex" nil (ey1 p eh1 k s))
("apfel" nil (ae1 p f ax l))
("apfelbaum" nil (ae1 p f ax l b aw1 m))
("apgar" nil (ae1 p g er0))
("aphasia" nil (ax f ey1 zh ax))
("aphids" nil (ae1 f ih0 d z))
("aphorism" nil (ae1 f er0 ih1 z ax m))
("aphorisms" nil (ae1 f er0 ih1 z ax m z))
("aphrodisiac" nil (ae1 f r ow0 d iy1 z iy0 ae0 k))
("aphrodites" nil (ae1 f r ax d ay1 t iy0 z))
("apicella" nil (ae1 p ih0 s eh1 l ax))
("apiece" nil (ax p iy1 s))
("aping" nil (ey1 p ih0 ng))
("aplenty" nil (ax p l eh1 n t iy0))
("aplin" nil (ae1 p l ih0 n))
("aplomb" nil (ax p l aa1 m))
("apnea" nil (ae1 p n iy0 ax))
("apocalypse" nil (ax p aa1 k ax l ih1 p s))
("apocalyptic" nil (ax p aa1 k ax l ih1 p t ih0 k))
("apocryphal" nil (ax p aa1 k r ax f ax l))
("apodaca" nil (aa0 p ow0 d aa1 k ax))
("apogee" nil (ae1 p ax jh iy1))
("apolitical" nil (ey1 p ax l ih1 t ih0 k ax l))
("apolline" nil (ae1 p ax l ay1 n))
("apollinian" nil (ae1 p ax l ih1 n iy0 ax n))
("apollo" nil (ax p aa1 l ow0))
("apollonian" nil (ae1 p ax l ow1 n iy0 ax n))
("apologetic" nil (ax p aa1 l ax jh eh1 t ih0 k))
("apologetically" nil (ax p aa1 l ax jh eh1 t ih0 k l iy0))
("apologies" nil (ax p aa1 l ax jh iy0 z))
("apologist" nil (ax p aa1 l ax jh ax s t))
("apologists" nil (ax p aa1 l ax jh ax s t s))
("apologize" nil (ax p aa1 l ax jh ay1 z))
("apologized" nil (ax p aa1 l ax jh ay1 z d))
("apologizes" nil (ax p aa1 l ax jh ay1 z ih0 z))
("apologizing" nil (ax p aa1 l ax jh ay1 z ih0 ng))
("apology" nil (ax p aa1 l ax jh iy0))
("aponte" nil (ax p aa1 n t iy0))
("apopa" nil (ax p ow1 p ax))
("apoplectic" nil (ae1 p ax p l eh1 k t ih0 k))
("apoplexy" nil (ae1 p ax p l eh1 k s iy0))
("apostle" nil (ax p aa1 s ax l))
("apostles" nil (ax p aa1 s ax l z))
("apostol" nil (ae1 p ax s t ax l))
("apostolopoul" nil (ax p aa1 s t ow0 l ow0 p uw1 l))
("apostrophe" nil (ax p aa1 s t r ax f iy0))
("apothecary" nil (ax p aa1 th ax k eh1 r iy0))
("apotheosis" nil (ax p aa1 th iy0 ow1 s ax s))
("app" nil (ae1 p))
("appalachia" nil (ae1 p ax l ae1 ch iy0 ax))
("appalachian" nil (ae1 p ax l ey1 ch ax n))
("appalachians" nil (ae1 p ax l ey1 ch ax n z))
("appalled" nil (ax p ao1 l d))
("appalling" nil (ax p ao1 l ih0 ng))
("appallingly" nil (ax p ao1 l ih0 ng l iy0))
("appalls" nil (ax p ao1 l z))
("apparatchiks" nil (ae1 p er0 ae1 ch ih0 k s))
("apparatus" nil (ae1 p er0 ae1 t ax s))
("apparatuses" nil (ae1 p er0 ae1 t ax s ax z))
("apparel" nil (ax p ae1 r ax l))
("apparent" nil (ax p eh1 r ax n t))
("apparently" nil (ax p eh1 r ax n t l iy0))
("apparition" nil (ae1 p er0 ih1 sh ax n))
("appeal" nil (ax p iy1 l))
("appealed" nil (ax p iy1 l d))
("appealing" nil (ax p iy1 l ih0 ng))
("appealingly" nil (ax p iy1 l ih0 ng l iy0))
("appeals" nil (ax p iy1 l z))
("appear" nil (ax p ih1 r))
("appearance" nil (ax p ih1 r ax n s))
("appearances" nil (ax p ih1 r ax n s ax z))
("appeared" nil (ax p ih1 r d))
("appearing" nil (ax p ih1 r ih0 ng))
("appears" nil (ax p ih1 r z))
("appease" nil (ax p iy1 z))
("appeased" nil (ax p iy1 z d))
("appeasement" nil (ax p iy1 z m ax n t))
("appeasing" nil (ax p iy1 z ih0 ng))
("appel" nil (ae1 p ax l))
("appelbaum" nil (ae1 p ax l b aw1 m))
("appelhans" nil (ae1 p ih0 l hh ax n z))
("appell" nil (ae1 p ax l))
("appellate" nil (ax p eh1 l ih0 t))
("appellation" nil (ae1 p ax l ey1 sh ax n))
("appellations" nil (ae1 p ax l ey1 sh ax n z))
("appelman" nil (ae1 p ax l m ax n))
("appelt" nil (ae1 p ih0 l t))
("append" nil (ax p eh1 n d))
("appendage" nil (ax p eh1 n d ih0 jh))
("appendages" nil (ax p eh1 n d ih0 jh ih0 z))
("appendectomies" nil (ae1 p ax n d eh1 k t ax m iy0 z))
("appendectomy" nil (ae1 p ih0 n d eh1 k t ax m iy0))
("appended" nil (ax p eh1 n d ih0 d))
("appendix" nil (ax p eh1 n d ih0 k s))
("appendixes" nil (ax p eh1 n d ih0 k s ih0 z))
("appenzeller" nil (ae1 p ih0 n z ax l er0))
("apperson" nil (ae1 p er0 s ax n))
("appert" nil (ae1 p er0 t))
("appetite" nil (ae1 p ax t ay1 t))
("appetites" nil (ae1 p ax t ay1 t s))
("appetito" nil (ae1 p ax t iy1 t ow1))
("appetizer" nil (ae1 p ax t ay1 z er0))
("appetizers" nil (ae1 p ax t ay1 z er0 z))
("appetizing" nil (ae1 p ih0 t ay1 z ih0 ng))
("appia" nil (ae1 p iy0 ax))
("appian" nil (ae1 p iy0 ax n))
("applaud" nil (ax p l ao1 d))
("applauded" nil (ax p l ao1 d ax d))
("applauding" nil (ax p l ao1 d ih0 ng))
("applauds" nil (ax p l ao1 d z))
("applause" nil (ax p l ao1 z))
("apple" nil (ae1 p ax l))
("applebaum" nil (ae1 p ax l b aw1 m))
("applebee" nil (ae1 p ax l b iy1))
("appleberry" nil (ae1 p ax l b eh1 r iy0))
("appleby" nil (ae1 p l ih0 b iy0))
("applegarth" nil (ae1 p l ih0 g aa0 r th))
("applegate" nil (ae1 p ax l g ey1 t))
("appleman" nil (ae1 p ax l m ax n))
("applequist" nil (ae1 p l ih0 k w ih0 s t))
("apples" nil (ae1 p ax l z))
("applesauce" nil (ae1 p ax l s ao1 s))
("appleseed" nil (ae1 p ax l s iy1 d))
("appleshare" nil (ae1 p ax l sh eh1 r))
("appleton" nil (ae1 p ax l t ax n))
("applewhite" nil (ae1 p ax l w ay1 t))
("appleyard" nil (ae1 p ax l y aa1 r d))
("appliance" nil (ax p l ay1 ax n s))
("appliances" nil (ax p l ay1 ax n s ax z))
("applicability" nil (ae1 p l ax k ax b ih1 l ax t iy0))
("applicable" nil (ae1 p l ax k ax b ax l))
("applicant" nil (ae1 p l ih0 k ax n t))
("applicants" nil (ae1 p l ih0 k ax n t s))
("application" nil (ae1 p l ax k ey1 sh ax n))
("applications" nil (ae1 p l ax k ey1 sh ax n z))
("applicator" nil (ae1 p l ax k ey1 t er0))
("applicators" nil (ae1 p l ax k ey1 t er0 z))
("applied" nil (ax p l ay1 d))
("applies" nil (ax p l ay1 z))
("applin" nil (ae1 p l ih0 n))
("appling" nil (ae1 p ax l ih0 ng))
("apply" nil (ax p l ay1))
("applying" nil (ax p l ay1 ih0 ng))
("appoint" nil (ax p oy1 n t))
("appointed" nil (ax p oy1 n t ax d))
("appointee" nil (ax p oy0 n t iy1))
("appointees" nil (ax p oy0 n t iy1 z))
("appointing" nil (ax p oy1 n t ih0 ng))
("appointive" nil (ax p oy1 n t ih0 v))
("appointment" nil (ax p oy1 n t m ax n t))
("appointments" nil (ax p oy1 n t m ax n t s))
("appoints" nil (ax p oy1 n t s))
("apportion" nil (ax p ao1 r sh ax n))
("apportioned" nil (ax p ao1 r sh ax n d))
("apportioning" nil (ax p ao1 r sh ax n ih0 ng))
("apportionment" nil (ax p ao1 r sh ax n m ax n t))
("appraisal" nil (ax p r ey1 z ax l))
("appraisals" nil (ax p r ey1 z ax l z))
("appraise" nil (ax p r ey1 z))
("appraised" nil (ax p r ey1 z d))
("appraiser" nil (ax p r ey1 z er0))
("appraisers" nil (ax p r ey1 z er0 z))
("appraises" nil (ax p r ey1 z ih0 z))
("appraising" nil (ax p r ey1 z ih0 ng))
("appreciable" nil (ax p r iy1 sh ax b ax l))
("appreciably" nil (ax p r iy1 sh ax b l iy0))
("appreciate" nil (ax p r iy1 sh iy0 ey1 t))
("appreciated" nil (ax p r iy1 sh iy0 ey1 t ih0 d))
("appreciates" nil (ax p r iy1 sh iy0 ey1 t s))
("appreciating" nil (ax p r iy1 sh iy0 ey1 t ih0 ng))
("appreciation" nil (ax p r iy1 sh iy0 ey1 sh ax n))
("appreciative" nil (ax p r iy1 sh iy0 ey1 t ih0 v))
("appreciatively" nil (ax p r iy1 sh iy0 ey1 t ih0 v l iy0))
("apprehend" nil (ae1 p r ih0 hh eh1 n d))
("apprehended" nil (ae1 p r ax hh eh1 n d ax d))
("apprehension" nil (ae1 p r ih0 hh eh1 n sh ax n))
("apprehensions" nil (ae1 p r ih0 hh eh1 n sh ax n z))
("apprehensive" nil (ae1 p r ih0 hh eh1 n s ih0 v))
("apprentice" nil (ax p r eh1 n t ax s))
("apprenticed" nil (ax p r eh1 n t ax s t))
("apprentices" nil (ax p r eh1 n t ax s ih0 z))
("apprenticeship" nil (ax p r eh1 n t ax s sh ih1 p))
("apprised" nil (ax p r ay1 z d))
("approach" nil (ax p r ow1 ch))
("approachable" nil (ax p r ow1 ch ax b ax l))
("approached" nil (ax p r ow1 ch t))
("appropriate" j (ax p r ow1 p r iy0 ax t))
("appropriate" v (ax p r ow1 p r iy0 ey0 t))
("approaching" nil (ax p r ow1 ch ih0 ng))
("approbation" nil (ae1 p r ax b ey1 sh ax n))
("appropriated" nil (ax p r ow1 p r iy0 ey1 t ih0 d))
("appropriately" nil (ax p r ow1 p r iy0 ih0 t l iy0))
("appropriateness" nil (ax p r ow1 p r iy0 ax t n ax s))
("appropriates" nil (ax p r ow1 p r iy0 ey1 t s))
("appropriating" nil (ax p r ow1 p r iy0 ey1 t ih0 ng))
("appropriation" nil (ax p r ow1 p r iy0 ey1 sh ax n))
("appropriations" nil (ax p r ow1 p r iy0 ey1 sh ax n z))
("appropriators" nil (ax p r ow1 p r iy0 ey1 t er0 z))
("approvable" nil (ax p r uw1 v ax b ax l))
("approval" nil (ax p r uw1 v ax l))
("approvals" nil (ax p r uw1 v ax l z))
("approve" nil (ax p r uw1 v))
("approved" nil (ax p r uw1 v d))
("approves" nil (ax p r uw1 v z))
("approving" nil (ax p r uw1 v ih0 ng))
("approvingly" nil (ax p r uw1 v ih0 ng l iy0))
("approximate" j (ax p r aa1 k s ax m ax t))
("approximate" v (ax p r aa1 k s ax m ey0 t))
("approximated" nil (ax p r aa1 k s ax m ey1 t ax d))
("approximately" nil (ax p r aa1 k s ax m ax t l iy0))
("approximates" nil (ax p r aa1 k s ax m ax t s))
("approximating" nil (ax p r aa1 k s ih0 m ey1 t ih0 ng))
("approximation" nil (ax p r aa1 k s ax m ey1 sh ax n))
("apps" nil (ae1 p s))
("apra" nil (ey1 p r ax))
("aprea" nil (aa1 p r iy0 ax))
("apricot" nil (ey1 p r ax k aa1 t))
("apricots" nil (ae1 p r ax k aa1 t s))
("april" nil (ey1 p r ax l))
("aprile" nil (aa1 p r ax l))
("apron" nil (ey1 p r ax n))
("aprons" nil (ey1 p r ax n z))
("apropos" nil (ae1 p r ax p ow1))
("apsey" nil (ae1 p s iy0))
("apstar" nil (ae1 p s t aa1 r))
("apt" nil (ae1 p t))
("apter" nil (ae1 p t er0))
("aptitude" nil (ae1 p t ax t uw1 d))
("aptitudes" nil (ae1 p t ax t uw1 d z))
("aptiva" nil (ae1 p t iy1 v ax))
("aptly" nil (ae1 p t l iy0))
("apuzzo" nil (aa0 p uw1 z ow0))
("aqaba" nil (ax k aa1 b ax))
("aqazadeh" nil (ae1 k ax z aa1 d eh1))
("aqcess" nil (ae1 k s eh0 s))
("aqua" nil (ae1 k w ax))
("aquaculture" nil (ae1 k w ax k ah1 l ch er0))
("aqualung" nil (ae1 k w ax l ax ng))
("aquamarine" nil (ae1 k w ax m er0 iy1 n))
("aquarists" nil (ax k w eh1 r ax s t s))
("aquarium" nil (ax k w eh1 r iy0 ax m))
("aquariums" nil (ax k w eh1 r iy0 ax m z))
("aquarius" nil (ax k w eh1 r iy0 ax s))
("aquatic" nil (ax k w aa1 t ih0 k))
("aqueduct" nil (ae1 k w ax d ah1 k t))
("aqueducts" nil (ae1 k w ax d ah1 k t s))
("aqueous" nil (ey1 k w iy0 ax s))
("aquifer" nil (ae1 k w ax f er0))
("aquifers" nil (ae1 k w ax f er0 z))
("aquila" nil (ae1 k w ax l ax))
("aquilar" nil (aa0 k w iy0 l aa1 r))
("aquilina" nil (aa0 k w iy0 l iy1 n ax))
("aquilino" nil (aa0 k w iy0 l iy1 n ow0))
("aquinas" nil (ax k w ay1 n ax s))
("aquino" nil (ax k iy1 n ow0))
("aquirre" nil (ae1 k w er0))
("aquitaine" nil (ae1 k w ih0 t ey1 n))
("ar" nil (aa1 r))
("ara" nil (ae1 r ax))
("arab" nil (ae1 r ax b))
("arabe" nil (aa0 r aa1 b ey0))
("arabella" nil (ae1 r ax b eh1 l ax))
("arabia" nil (er0 ey1 b iy0 ax))
("arabian" nil (er0 ey1 b iy0 ax n))
("arabians" nil (er0 ey1 b iy0 ax n z))
("arabic" nil (ae1 r ax b ih0 k))
("arabica" nil (er0 ae1 b ih0 k ax))
("arabie" nil (er0 ae1 b iy0))
("arable" nil (eh1 r ax b ax l))
("arabs" nil (ae1 r ax b z))
("arachnids" nil (er0 ae1 k n ax d z))
("arafat" nil (eh1 r ax f ae0 t))
("aragon" nil (eh1 r ax g aa1 n))
("aragona" nil (aa0 r aa0 g ow1 n ax))
("arai" nil (aa0 r aa1 iy0))
("araiza" nil (er0 ey1 z ax))
("arakaki" nil (aa0 r aa0 k aa1 k iy0))
("arakawa" nil (aa1 r aa0 k aa1 w ax))
("arakelian" nil (ae0 r ax k eh1 l y ax n))
("araki" nil (aa0 r aa1 k iy0))
("aral" nil (aa1 r ax l))
("arambula" nil (aa0 r aa0 m b uw1 l ax))
("aramco" nil (er0 ae1 m k ow0))
("aramid" nil (eh1 r ax m ih0 d))
("araminta" nil (ae1 r ax m ih1 n t ax))
("aramis" nil (eh1 r ax m ih0 s))
("aramony" nil (aa1 r ax m ow1 n iy0))
("aran" nil (eh1 r ax n))
("arana" nil (aa0 r ae1 n ax))
("aranda" nil (er0 ae1 n d ax))
("arango" nil (aa0 r aa1 ng g ow0))
("aransas" nil (er0 ae1 n s ax s))
("arant" nil (aa1 r ax n t))
("aranyos" nil (er0 aa1 n y ow1 s))
("arapaho" nil (er0 ae1 p ax hh ow1))
("araskog" nil (er0 ae1 s aa0 g))
("arata" nil (aa0 r aa1 t ax))
("arau" nil (ax r aw1))
("araujo" nil (aa0 r aw1 y ow0))
("aravind" nil (aa1 r v ax n d))
("araya" nil (er0 ay1 ax))
("arb" nil (aa1 r b))
("arba" nil (aa1 r b ax))
("arbat" nil (aa1 r b ae0 t))
("arbaugh" nil (aa1 r b ao1))
("arbed" nil (aa1 r b ih0 d))
("arbeiter" nil (aa1 r b ay0 t er0))
("arbel" nil (aa1 r b ax l))
("arbella" nil (aa1 r b eh1 l ax))
("arbiter" nil (aa1 r b ih0 t er0))
("arbiters" nil (aa1 r b ax t er0 z))
("arbitrage" nil (aa1 r b ih0 t r aa1 zh))
("arbitrager" nil (aa1 r b ih0 t r aa1 zh er0))
("arbitragers" nil (aa1 r b ih0 t r aa1 zh er0 z))
("arbitrageur" nil (aa1 r b ih0 t r aa1 zh er0))
("arbitrageurs" nil (aa1 r b ax t r aa1 zh er1 z))
("arbitraging" nil (aa1 r b ax t r aa1 jh ih0 ng))
("arbitrarily" nil (aa1 r b ih0 t r eh1 r ax l iy0))
("arbitrariness" nil (aa1 r b ax t r eh1 r iy0 n ax s))
("arbitrary" nil (aa1 r b ax t r eh1 r iy0))
("arbitrate" nil (aa1 r b ax t r ey1 t))
("arbitrated" nil (aa1 r b ax t r ey1 t ih0 d))
("arbitrates" nil (aa1 r b ih0 t r ey1 t s))
("arbitrating" nil (aa1 r b ax t r ey1 t ih0 ng))
("arbitration" nil (aa1 r b ih0 t r ey1 sh ax n))
("arbitrations" nil (aa1 r b ih0 t r ey1 sh ax n z))
("arbitrator" nil (aa1 r b ih0 t r ey1 t er0))
("arbitrators" nil (aa1 r b ih0 t r ey1 t er0 z))
("arbitron" nil (aa1 r b ax t r aa1 n))
("arbo" nil (aa1 r b ow0))
("arbogast" nil (aa0 r b ow1 g aa0 s t))
("arboleda" nil (aa0 r b ow0 l eh1 d ax))
("arbor" nil (aa1 r b er0))
("arboreal" nil (aa0 r b ao1 r iy0 ax l))
("arboretum" nil (aa1 r b er0 iy1 t ax m))
("arbour" nil (aa1 r b er0))
("arbs" nil (aa1 r b z))
("arbuckle" nil (aa1 r b ax k ax l))
("arbuckles" nil (aa1 r b ax k ax l z))
("arbuthnot" nil (aa0 r b ah1 th n ax t))
("arc" nil (aa1 r k))
("arcade" nil (aa0 r k ey1 d))
("arcades" nil (aa0 r k ey1 d z))
("arcadia" nil (aa0 r k ey1 d iy0 ax))
("arcadian" nil (aa1 r k ey1 d iy0 ax n))
("arcadians" nil (aa1 r k ey1 d iy0 ax n z))
("arcana" nil (aa0 r k ae1 n ax))
("arcand" nil (aa1 r k ax n d))
("arcane" nil (aa1 r k ey1 n))
("arcanum" nil (aa1 r k ax n ax m))
("arcaro" nil (aa0 r k aa1 r ow0))
("arcata" nil (aa0 r k aa1 t ax))
("arce" nil (aa1 r s))
("arcement" nil (aa0 r s ey1 m ax n t))
("arceneaux" nil (aa1 r s ih0 n ow0))
("arceo" nil (aa1 r s iy0 ow0))
("arch" nil (aa1 r ch))
("archaeological" nil (aa1 r k iy0 ax l aa1 jh ih0 k ax l))
("archaeologist" nil (aa1 r k iy0 aa1 l ax jh ih0 s t))
("archaeologists" nil (aa1 r k iy0 aa1 l ax jh ih0 s t s))
("archaeology" nil (aa1 r k iy0 aa1 l ax jh iy0))
("archaic" nil (aa0 r k ey1 ih0 k))
("archambault" nil (aa1 r sh ax m b ow0))
("archambeau" nil (aa1 r sh ax m b ow1))
("archambeault" nil (aa1 r sh ax m b ow0))
("archard" nil (aa0 r sh aa1 r d))
("archbishop" nil (aa1 r ch b ih1 sh ax p))
("archbold" nil (aa1 r ch b ow1 l d))
("archdiocese" nil (aa1 r ch d ay1 ax s ax s))
("archduke" nil (aa1 r ch d uw1 k))
("arched" nil (aa1 r ch t))
("archenemy" nil (aa1 r ch eh1 n ax m iy0))
("archeological" nil (aa1 r k iy0 ax l aa1 jh ih0 k ax l))
("archeology" nil (aa1 r k iy0 aa1 l ax jh iy0))
("archer" nil (aa1 r ch er0))
("archerd" nil (aa1 r k er0 d))
("archery" nil (aa1 r ch er0 iy0))
("arches" nil (aa1 r ch ax z))
("archetypal" nil (aa1 r k t ay1 p ax l))
("archetype" nil (aa1 r k ih0 t ay1 p))
("archetypes" nil (aa1 r k ih0 t ay1 p s))
("archey" nil (aa1 r k iy0))
("archibald" nil (aa1 r ch ax b ao1 l d))
("archibeque" nil (aa1 r sh ih0 b iy0 k))
("archie" nil (aa1 r ch iy0))
("archimedes" nil (aa1 r k ax m iy1 d iy0 z))
("arching" nil (aa1 r ch ih0 ng))
("archipelago" nil (aa1 r k ax p eh1 l ax g ow1))
("architect" nil (aa1 r k ax t eh1 k t))
("architects" nil (aa1 r k ax t eh1 k t s))
("architectural" nil (aa1 r k ax t eh1 k ch er0 ax l))
("architecturally" nil (aa1 r k ih0 t eh1 k ch er0 ax l iy0))
("architecture" nil (aa1 r k ax t eh1 k ch er0))
("architectures" nil (aa1 r k ax t eh1 k ch er0 z))
("architraves" nil (aa1 r k ax t r ey1 v z))
("archival" nil (aa0 r k ay1 v ax l))
("archive" nil (aa1 r k ay1 v))
("archives" nil (aa1 r k ay1 v z))
("archivist" nil (aa1 r k ax v ih0 s t))
("archivists" nil (aa1 r k ax v ih0 s t s))
("archly" nil (aa1 r ch l iy0))
("archrival" nil (aa1 r ch r ay1 v ax l))
("archrivals" nil (aa1 r ch r ay1 v ax l z))
("archuleta" nil (aa0 r k uw0 l eh1 t ax))
("archuletta" nil (aa0 r k uw0 l eh1 t ax))
("archway" nil (aa1 r ch w ey1))
("arcidiacono" nil (aa0 r ch iy0 d iy0 aa0 k ow1 n ow0))
("arcieri" nil (aa0 r ch ih1 r iy0))
("arciniega" nil (aa0 r ch iy0 n iy1 g ax))
("arco" nil (aa1 r k ow0))
("arcola" nil (aa0 r k ow1 l ax))
("arcos" nil (aa1 r k ow0 z))
("arcosanti" nil (aa1 r k ax s ae1 n t iy0))
("arcs" nil (aa1 r k s))
("arctic" nil (aa1 r k t ih0 k))
("arcturus" nil (aa1 r k t uh1 r ax s))
("arcuri" nil (aa0 r k uh1 r iy0))
("ard" nil (aa1 r d))
("arda" nil (aa1 r d ax))
("ardath" nil (aa1 r d ax th))
("ardeen" nil (aa0 r d iy1 n))
("ardelia" nil (aa0 r d eh1 l iy0 ax))
("ardella" nil (aa1 r d eh1 l ax))
("ardelle" nil (aa0 r d eh1 l))
("arden" nil (aa1 r d ax n))
("ardene" nil (aa1 r d iy1 n))
("ardent" nil (aa1 r d ax n t))
("ardently" nil (aa1 r d ax n t l iy0))
("ardin" nil (aa1 r d ih0 n))
("ardine" nil (aa0 r d iy1 n iy0))
("ardis" nil (aa1 r d ih0 s))
("ardith" nil (aa1 r d ih0 th))
("ardito" nil (aa0 r d iy1 t ow0))
("ardley" nil (aa1 r d l iy0))
("ardmore" nil (aa1 r d m ao1 r))
("ardoin" nil (aa0 r d oy1 n))
("ardolino" nil (aa0 r d ow0 l iy1 n ow0))
("ardolph" nil (aa1 r d ow1 l f))
("ardor" nil (aa1 r d er0))
("ardra" nil (aa1 r d r ax))
("ardrey" nil (aa1 r d r iy0))
("ardshiel" nil (aa1 r d sh iy1 l))
("ardsley" nil (aa1 r d z l iy0))
("arduini" nil (aa0 r d uw0 iy1 n iy0))
("arduous" nil (aa1 r jh uw0 ax s))
("are" nil (aa1 r))
("area" nil (eh1 r iy0 ax))
("areas" nil (eh1 r iy0 ax z))
("arebalo" nil (aa0 r eh0 b aa1 l ow0))
("arechiga" nil (aa0 r eh0 k iy1 g ax))
("arehart" nil (ae1 r ih0 hh aa0 r t))
("arel" nil (aa0 r eh1 l))
("arellanes" nil (ae1 r ih0 l ey1 n z))
("arellano" nil (aa0 r eh0 l aa1 n ow0))
("arena" nil (er0 iy1 n ax))
("arenaball" nil (er0 iy1 n ax b ao1 l))
("arenas" nil (er0 iy1 n ax z))
("arencibia" nil (aa0 r eh0 n ch iy1 b iy0 ax))
("arend" nil (er0 eh1 n d))
("arends" nil (er0 eh1 n d z))
("arendsee" nil (er0 eh1 n d s iy1))
("arendt" nil (eh1 r ih0 n t))
("arenella" nil (aa1 r ax n eh1 l ax))
("arens" nil (eh1 r ax n z))
("arenson" nil (ae1 r ih0 n s ax n))
("arent" nil (aa1 r ax n t))
("arentz" nil (ae1 r ih0 n t s))
("arenz" nil (aa1 r ey0 n z))
("ares" nil (aa1 r z))
("areta" nil (aa0 r eh1 t ax))
("aretha" nil (er0 iy1 th ax))
("aretina" nil (aa0 r eh0 t iy1 n ax))
("aretsky" nil (eh0 r eh1 t s k iy1))
("aretta" nil (aa0 r eh1 t ax))
("arevalo" nil (aa0 r ey0 v aa1 l ow0))
("arey" nil (ae1 r iy0))
("argabright" nil (aa1 r g ax b r ay1 t))
("argall" nil (aa1 r g ax l))
("arganbright" nil (aa0 r g ae1 n b r ay0 t))
("argenbright" nil (aa1 r g ih0 n b r ay0 t))
("argent" nil (aa1 r jh ih0 n t))
("argenta" nil (aa1 r jh eh1 n t ax))
("argentieri" nil (aa0 r jh eh0 n t ih1 r iy0))
("argentina" nil (aa1 r jh ax n t iy1 n ax))
("argentinas" nil (aa1 r jh ax n t iy1 n ax z))
("argentine" nil (aa1 r jh ax n t iy1 n))
("argentines" nil (aa1 r jh ax n t iy1 n z))
("argentinian" nil (aa1 r jh ax n t ih1 n iy0 ax n))
("argentinians" nil (aa1 r jh ax n t ih1 n iy0 ax n z))
("argento" nil (aa1 r jh eh1 n t ow0))
("argiro" nil (aa0 r jh ih1 r ow0))
("argo" nil (aa1 r g ow0))
("argon" nil (aa1 r g aa1 n))
("argonaut" nil (aa1 r g ax n ao1 t))
("argonauts" nil (aa1 r g ax n ao1 t s))
("argonne" nil (aa1 r g aa1 n))
("argosy" nil (aa1 r g ax s iy0))
("argosystems" nil (aa1 r g ow0 s ih1 s t ax m z))
("argot" nil (aa1 r g ax t))
("arguable" nil (aa1 r g y uw0 ax b ax l))
("arguably" nil (aa1 r g y uw0 ax b l iy0))
("argue" nil (aa1 r g y uw0))
("argued" nil (aa1 r g y uw0 d))
("arguelles" nil (aa0 r g ey1 l eh0 s))
("arguello" nil (aa0 r g eh1 l ow0))
("argues" nil (aa1 r g y uw0 z))
("argueta" nil (aa0 r g ey1 t ax))
("arguijo" nil (aa0 r g w iy1 y ow0))
("arguing" nil (aa1 r g y uw0 ih0 ng))
("argument" nil (aa1 r g y ax m ax n t))
("argumentative" nil (aa1 r g y ax m eh1 n t ax t ih0 v))
("arguments" nil (aa1 r g y ax m ax n t s))
("argus" nil (aa1 r g ax s))
("argyle" nil (aa1 r g ay1 l))
("argyll" nil (aa0 r g ay1 l))
("argyropoulos" nil (aa1 r jh ih0 r aa1 p ow0 l ax s))
("ari" nil (aa1 r iy0))
("aria" nil (aa1 r iy0 ax))
("ariadne" nil (eh1 r iy0 ae1 d n iy0))
("ariana" nil (aa0 r iy0 ae1 n ax))
("ariane" nil (eh1 r iy0 ae1 n))
("arianespace" nil (eh1 r iy0 ae1 n s p ey1 s))
("arianism" nil (ae1 r iy0 ax n ih1 z ax m))
("arianna" nil (aa0 r iy0 ae1 n ax))
("arias" nil (aa1 r iy0 ax z))
("aric" nil (ae1 r ih0 k))
("arico" nil (aa0 r iy1 k ow0))
("arid" nil (ae1 r ax d))
("arida" nil (ax r iy1 d ax))
("arie" nil (eh1 r iy0))
("ariel" nil (eh1 r iy0 ax l))
("ariella" nil (aa0 r iy0 eh1 l ax))
("aries" nil (eh1 r iy0 z))
("arington" nil (aa1 r ih0 ng t ax n))
("ariola" nil (aa0 r iy0 ow1 l ax))
("aris" nil (er0 iy1 s))
("arise" nil (er0 ay1 z))
("arisen" nil (er0 ih1 z ax n))
("arises" nil (er0 ay1 z ax z))
("arising" nil (er0 ay1 z ih0 ng))
("arison" nil (eh1 r ih0 s ax n))
("arispe" nil (aa0 r iy1 s p iy0))
("arista" nil (er0 ih1 s t ax))
("aristech" nil (eh1 r ih0 s t eh1 k))
("aristede" nil (aa1 r ih0 s t iy1 d))
("aristide" nil (aa1 r ih0 s t iy1 d))
("aristocracy" nil (eh1 r ax s t aa1 k r ax s iy0))
("aristocrat" nil (er0 ih1 s t ax k r ae1 t))
("aristocratic" nil (er0 ih1 s t ax k r ae1 t ih0 k))
("aristocrats" nil (er0 ih1 s t ax k r ae1 t s))
("aristotelian" nil (er0 ih1 s t ax t iy1 l iy0 ax n))
("aristotle" nil (eh1 r ax s t aa1 t ax l))
("arithmetic" nil (eh1 r ih0 th m eh1 t ih0 k))
("ariza" nil (aa0 r iy1 z ax))
("arizmendi" nil (aa0 r iy0 z m eh1 n d iy0))
("arizona" nil (eh1 r ih0 z ow1 n ax))
("arizonans" nil (er0 ih1 z ax n ax n z))
("arjo" nil (aa1 r jh ow0))
("ark" nil (aa1 r k))
("arkansans" nil (aa1 r k ae1 n z ax n z))
("arkansas" nil (aa1 r k ax n s aa1))
("arkin" nil (aa1 r k ih0 n))
("arkla" nil (aa1 r k l ax))
("arko" nil (aa1 r k ow0))
("arkose" nil (aa0 r k ow1 s))
("arkwright" nil (aa1 r k r ay1 t))
("arlana" nil (aa0 r l ae1 n ax))
("arlauskas" nil (aa0 r l ao1 s k ax s))
("arledge" nil (aa1 r l ih0 jh))
("arleen" nil (aa0 r l iy1 n))
("arlen" nil (aa1 r l ax n))
("arlena" nil (aa1 r l ih0 n ax))
("arlene" nil (aa1 r l iy1 n))
("arleta" nil (aa1 r l ih0 t ax))
("arlette" nil (aa1 r l eh1 t))
("arley" nil (aa1 r l iy0))
("arlie" nil (aa1 r l iy0))
("arlin" nil (aa1 r l ih0 n))
("arlinda" nil (aa1 r l ih1 n d ax))
("arline" nil (aa1 r l ay1 n))
("arlington" nil (aa1 r l ih0 ng t ax n))
("arlt" nil (aa1 r l t))
("arlyne" nil (aa1 r l ay1 n))
("arm" nil (aa1 r m))
("armacost" nil (aa1 r m ax k ao1 s t))
("armada" nil (aa0 r m aa1 d ax))
("armadillo" nil (aa1 r m ax d ih1 l ow0))
("armadillos" nil (aa1 r m ax d ih1 l ow0 z))
("armageddon" nil (aa1 r m ax g eh1 d ax n))
("armagost" nil (aa0 r m aa1 g ow0 s t))
("armament" nil (aa1 r m ax m ax n t))
("armaments" nil (aa1 r m ax m ax n t s))
("arman" nil (aa1 r m ax n))
("armand" nil (aa1 r m ax n d))
("armando" nil (aa0 r m aa1 n d ow0))
("armandt" nil (aa1 r m ae0 n t))
("armani" nil (aa1 r m aa1 n iy0))
("armantrout" nil (aa1 r m ax n t r aw1 t))
("armas" nil (aa1 r m ax z))
("armato" nil (aa0 r m aa1 t ow0))
("armbands" nil (aa1 r m b ae1 n d z))
("armbrister" nil (aa1 r m b er0 ih0 s t er0))
("armbrust" nil (aa1 r m b r ax s t))
("armbruster" nil (aa1 r m b r ax s t er0))
("armchair" nil (aa1 r m ch eh1 r))
("armchairs" nil (aa1 r m ch eh1 r z))
("armco" nil (aa1 r m k ow1))
("armed" nil (aa1 r m d))
("armel" nil (aa0 r m eh1 l))
("armellino" nil (aa1 r m eh0 l iy1 n ow0))
("armen" nil (aa1 r m ax n))
("armendarez" nil (aa0 r m ey0 n d aa1 r eh0 z))
("armendariz" nil (aa0 r m ey0 n d aa1 r iy0 z))
("armenia" nil (aa0 r m iy1 n iy0 ax))
("armenian" nil (aa0 r m iy1 n iy0 ax n))
("armenians" nil (aa0 r m iy1 n iy0 ax n z))
("arment" nil (aa1 r m ax n t))
("armenta" nil (aa0 r m eh1 n t ax))
("armentor" nil (aa0 r m ax n t ao1 r))
("armentrout" nil (aa1 r m ax n t r aw1 t))
("armer" nil (aa1 r m er0))
("armes" nil (aa1 r m z))
("armetta" nil (aa0 r m eh1 t ax))
("armey" nil (aa1 r m iy0))
("armfield" nil (aa1 r m f iy1 l d))
("armida" nil (aa0 r m iy1 d ax))
("armies" nil (aa1 r m iy0 z))
("armiger" nil (aa1 r m ih0 g er0))
("armijo" nil (aa0 r m iy1 y ow0))
("armilla" nil (aa0 r m ih1 l ax))
("armin" nil (aa1 r m ax n))
("armina" nil (aa0 r m iy1 n ax))
("armine" nil (aa0 r m iy1 n iy0))
("arming" nil (aa1 r m ih0 ng))
("armington" nil (aa1 r m ih0 ng t ax n))
("arminie" nil (aa1 r m ih0 n iy0))
("armistead" nil (aa1 r m ih0 s t eh1 d))
("armistice" nil (aa1 r m ax s t ax s))
("armitage" nil (aa1 r m ih0 t ih0 jh))
("armold" nil (aa1 r m ow1 l d))
("armon" nil (aa1 r m ax n))
("armond" nil (aa1 r m ax n d))
("armonk" nil (aa1 r m ax ng k))
("armor" nil (aa1 r m er0))
("armored" nil (aa1 r m er0 d))
("armory" nil (aa1 r m er0 iy0))
("armour" nil (aa1 r m er0))
("armpit" nil (aa1 r m p ih1 t))
("armpits" nil (aa1 r m p ih1 t s))
("armrest" nil (aa1 r m r eh1 s t))
("armrests" nil (aa1 r m r eh1 s t s))
("arms" nil (aa1 r m z))
("armstead" nil (aa1 r m s t eh1 d))
("armstrad" nil (aa1 r m s t r ae1 d))
("armstrong" nil (aa1 r m s t r aa1 ng))
("armtec" nil (aa1 r m t eh1 k))
("armtek" nil (aa1 r m t eh1 k))
("armwood" nil (aa1 r m w uh1 d))
("army" nil (aa1 r m iy0))
("arn" nil (aa1 r n))
("arnalda" nil (aa0 r n aa1 l d ax))
("arnaldo" nil (aa0 r n aa1 l d ow0))
("arnall" nil (aa1 r n ax l))
("arnatt" nil (aa0 r n ae1 t))
("arnaud" nil (aa0 r n ow1))
("arnault" nil (aa1 r n ow1))
("arndt" nil (aa1 r n t))
("arne" nil (aa1 r n))
("arnell" nil (aa1 r n ax l))
("arnelle" nil (aa1 r n eh1 l))
("arner" nil (aa1 r n er0))
("arnesen" nil (aa0 r n iy1 z ax n))
("arneson" nil (aa1 r n ih0 s ax n))
("arnett" nil (aa0 r n eh1 t))
("arnette" nil (aa0 r n eh1 t))
("arney" nil (aa1 r n iy0))
("arnhold" nil (aa1 r n hh ow1 l d))
("arnholt" nil (aa1 r n hh ow1 l t))
("arnie" nil (aa1 r n iy0))
("arnn" nil (aa1 r n))
("arno" nil (aa1 r n ow0))
("arnold" nil (aa1 r n ax l d))
("arnoldi" nil (aa0 r n ow1 l d iy0))
("arnoldy" nil (aa1 r n ow0 l d iy0))
("arnolphe" nil (aa0 r n aa1 l f iy0))
("arnone" nil (aa0 r n ow1 n iy0))
("arnot" nil (aa1 r n ax t))
("arnott" nil (aa1 r n aa0 t))
("arnotts" nil (aa1 r n aa0 t s))
("arnow" nil (aa1 r n ow0))
("arnox" nil (aa1 r n aa0 k s))
("arns" nil (aa1 r n z))
("arnst" nil (aa1 r n s t))
("arnstein" nil (aa1 r n s t ay0 n))
("arntson" nil (aa1 r n t s ax n))
("arntz" nil (aa1 r n t s))
("arntzen" nil (aa1 r n t z ax n))
("arnulfo" nil (aa0 r n ah1 l f ow0))
("arnwine" nil (aa1 r n w ay1 n))
("arnzen" nil (aa1 r n z ax n))
("aro" nil (aa1 r ow0))
("arocha" nil (aa0 r ow1 k ax))
("arocho" nil (aa0 r ow1 k ow0))
("aroma" nil (er0 ow1 m ax))
("aromas" nil (er0 ow1 m ax z))
("aromatic" nil (eh1 r ax m ae1 t ih0 k))
("aron" nil (eh1 r ax n))
("arone" nil (er0 ow1 n))
("aronhalt" nil (ae1 r ax n hh ax l t))
("aronoff" nil (eh1 r ax n ao0 f))
("aronow" nil (er0 ow1 n ow0))
("aronowitz" nil (er0 aa1 n ax w ih0 t s))
("arons" nil (aa1 r ow0 n z))
("aronson" nil (eh1 r ax n s ax n))
("aronstein" nil (eh1 r ax n s t iy1 n))
("arora" nil (aa0 r ao1 r ax))
("aros" nil (aa1 r ow0 z))
("arose" nil (er0 ow1 z))
("around" nil (er0 aw1 n d))
("arousal" nil (er0 aw1 z ax l))
("arouse" nil (er0 aw1 z))
("aroused" nil (er0 aw1 z d))
("arouses" nil (er0 aw1 z ih0 z))
("arousing" nil (er0 aw1 z ih0 ng))
("arp" nil (aa1 r p))
("arpa" nil (aa1 r p ax))
("arpanet" nil (aa1 r p ax n eh1 t))
("arpin" nil (aa1 r p ih0 n))
("arpino" nil (aa0 r p iy1 n ow0))
("arps" nil (aa1 r p s))
("arquette" nil (aa0 r k eh1 t))
("arquilla" nil (aa1 r k w ih1 l ax))
("arraigned" nil (er0 ey1 n d))
("arraignment" nil (er0 ey1 n m ax n t))
("arrange" nil (er0 ey1 n jh))
("arranged" nil (er0 ey1 n jh d))
("arrangement" nil (er0 ey1 n jh m ax n t))
("arrangements" nil (er0 ey1 n jh m ax n t s))
("arranger" nil (er0 ey1 n jh er0))
("arranges" nil (er0 ey1 n jh ih0 z))
("arranging" nil (er0 ey1 n jh ih0 ng))
("arrant" nil (ae1 r ax n t))
("arrants" nil (ae1 r ax n t s))
("arrasmith" nil (eh1 r ax s m ih1 th))
("array" nil (er0 ey1))
("arrayed" nil (er0 ey1 d))
("arrays" nil (er0 ey1 z))
("arrearage" nil (er0 iy1 r ih0 jh))
("arrearages" nil (er0 ih1 r ih0 jh ih0 z))
("arrears" nil (er0 ih1 r z))
("arredondo" nil (aa0 r eh0 d ow1 n d ow0))
("arreguin" nil (aa0 r ey0 g w iy1 n))
("arrendondo" nil (aa0 r eh0 n d ow1 n d ow0))
("arreola" nil (aa0 r iy0 aa1 l ax))
("arrest" nil (er0 eh1 s t))
("arrested" nil (er0 eh1 s t ax d))
("arresting" nil (er0 eh1 s t ih0 ng))
("arrests" nil (er0 eh1 s t s))
("arrhythmias" nil (er0 ih1 dh m iy0 ax z))
("arriaga" nil (aa1 r iy0 aa1 g ax))
("arrick" nil (ae1 r ih0 k))
("arrieta" nil (aa0 r iy1 t ax))
("arrigo" nil (aa1 r iy1 g ow0))
("arrigoni" nil (aa0 r iy0 g ow1 n iy0))
("arrington" nil (eh1 r ih0 ng t ax n))
("arriola" nil (aa1 r iy0 ow1 l ax))
("arriortua" nil (aa1 r iy0 ao0 r t uw1 ax))
("arrival" nil (er0 ay1 v ax l))
("arrivals" nil (er0 ay1 v ax l z))
("arrive" nil (er0 ay1 v))
("arrived" nil (er0 ay1 v d))
("arrives" nil (er0 ay1 v z))
("arriving" nil (er0 ay1 v ih0 ng))
("arrogance" nil (eh1 r ax g ax n s))
("arrogant" nil (eh1 r ax g ax n t))
("arrogantly" nil (eh1 r ax g ax n t l iy0))
("arrogate" nil (ae1 r ow0 g ey1 t))
("arrow" nil (ae1 r ow0))
("arrowhead" nil (ae1 r ow0 hh eh1 d))
("arrowheads" nil (ae1 r ow0 hh eh1 d z))
("arrowood" nil (ae1 r ow0 w uh1 d))
("arrowroot" nil (eh1 r ow0 r uw1 t))
("arrows" nil (ae1 r ow0 z))
("arrowsmith" nil (ae1 r ow0 s m ih1 th))
("arroyo" nil (er0 oy1 ow0))
("arruda" nil (aa0 r uw1 d ax))
("arrupe" nil (ae1 r uw1 p))
("arsdale" nil (aa1 r z d ey1 l))
("arsenal" nil (aa1 r s ax n ax l))
("arsenals" nil (aa1 r s ax n ax l z))
("arsenault" nil (aa1 r s ih0 n ow0))
("arseneau" nil (aa1 r s ih0 n ow0))
("arseneault" nil (aa1 r s ih0 n ow0))
("arsenic" nil (aa1 r s ax n ih0 k))
("arsenide" nil (aa1 r s ax n ay1 d))
("arsenio" nil (aa1 r s iy1 n iy0 ow0))
("arslanian" nil (aa1 r s l ey1 n iy0 ax n))
("arson" nil (aa1 r s ax n))
("arsonist" nil (aa1 r s ax n ax s t))
("arsonists" nil (aa1 r s ax n ax s t s))
("art" nil (aa1 r t))
("artale" nil (aa0 r t aa1 l iy0))
("artcarved" nil (aa1 r t k aa1 r v d))
("arteaga" nil (aa1 r t iy1 g ax))
("artemas" nil (aa1 r t ih0 m ax z))
("artemus" nil (aa1 r t ax m ax s))
("arter" nil (aa1 r t er0))
("arterberry" nil (aa1 r t er0 b eh1 r iy0))
("arterburn" nil (aa1 r t er0 b er1 n))
("arterial" nil (aa0 r t ih1 r iy0 ax l))
("arteries" nil (aa1 r t er0 iy0 z))
("arteriosclerosis" nil (aa0 r t ih1 r iy0 ow0 s k l er0 ow1 s ax s))
("artery" nil (aa1 r t er0 iy0))
("artful" nil (aa1 r t f ax l))
("artfully" nil (aa1 r t f ax l iy0))
("arth" nil (aa1 r th))
("arthritic" nil (aa0 r th r ih1 t ih0 k))
("arthritis" nil (aa0 r th r ay1 t ax s))
("arthropods" nil (aa1 r th r ax p aa1 d z))
("arthur" nil (aa1 r th er0))
("arthurian" nil (aa0 r th uh1 r iy0 ax n))
("arthurs" nil (aa1 r th er0 z))
("artichoke" nil (aa1 r t ax ch ow1 k))
("artichokes" nil (aa1 r t ih0 ch ow1 k s))
("article" nil (aa1 r t ax k ax l))
("articles" nil (aa1 r t ax k ax l z))
("articulate" v (aa0 r t ih1 k y ax l ey1 t))
("articulate" j (aa0 r t ih1 k y ax l ax t))
("articulated" nil (aa0 r t ih1 k y ax l ey1 t ax d))
("articulates" nil (aa0 r t ih1 k y ax l ax t s))
("articulating" nil (aa0 r t ih1 k y ax l ey1 t ih0 ng))
("articulation" nil (aa1 r t ih0 k y ax l ey1 sh ax n))
("articulatory" nil (aa0 r t ih1 k y ax l ax t ao1 r iy0))
("artie" nil (aa1 r t iy0))
("artifact" nil (aa1 r t ax f ae1 k t))
("artifacts" nil (aa1 r t ax f ae1 k t s))
("artifically" nil (aa0 r t ax f ih1 sh ax l iy0))
("artifice" nil (aa1 r t ax f ih0 s))
("artificial" nil (aa1 r t ax f ih1 sh ax l))
("artificially" nil (aa1 r t ax f ih1 sh ax l iy0))
("artillery" nil (aa0 r t ih1 l er0 iy0))
("artino" nil (aa0 r t iy1 n ow0))
("artiodactyls" nil (aa1 r t iy0 ow0 d ae1 k t ax l z))
("artis" nil (aa1 r t ih0 s))
("artisan" nil (aa1 r t ax z ax n))
("artisans" nil (aa1 r t ax z ax n z))
("artist" nil (aa1 r t ax s t))
("artistic" nil (aa0 r t ih1 s t ih0 k))
("artistically" nil (aa0 r t ih1 s t ih0 k l iy0))
("artistry" nil (aa1 r t ih0 s t r iy0))
("artists" nil (aa1 r t ax s t s))
("artley" nil (aa1 r t l iy0))
("artman" nil (aa1 r t m ax n))
("artois" nil (aa0 r t w aa1))
("arton" nil (aa1 r t aa1 n))
("artra" nil (aa1 r t r ax))
("artrip" nil (aa1 r t r ih0 p))
("arts" nil (aa1 r t s))
("artsy" nil (aa1 r t s iy1))
("arturo" nil (aa0 r t uh1 r ow0))
("artus" nil (aa1 r t ax s))
("artwork" nil (aa1 r t w er1 k))
("artworks" nil (aa1 r t w er1 k s))
("arty" nil (aa1 r t iy0))
("artz" nil (aa1 r t s))
("artzt" nil (aa1 r t s t))
("aruba" nil (er0 uw1 b ax))
("arum" nil (eh1 r ax m))
("arundel" nil (eh1 r ax n d ax l))
("arup" nil (er0 uw1 p))
("arva" nil (aa1 r v ax))
("arvad" nil (aa0 r v ae1 d))
("arval" nil (aa1 r v ax l))
("arvanitis" nil (aa0 r v ax n ay1 t ih0 s))
("arvay" nil (aa1 r v ey0))
("arve" nil (aa1 r v))
("arvel" nil (aa0 r v eh1 l))
("arvey" nil (aa1 r v iy0))
("arvid" nil (aa1 r v ih0 d))
("arvida" nil (aa0 r v iy1 d ax))
("arvidson" nil (aa1 r v ih0 d s ax n))
("arvin" nil (aa1 r v ih0 n))
("arvind" nil (aa1 r v ih0 n d))
("arvizu" nil (aa0 r v iy1 z uw0))
("arwood" nil (aa1 r w uh1 d))
("arx" nil (aa1 r k s))
("ary" nil (eh1 r iy0))
("aryan" nil (aa1 r iy0 ax n))
("arzate" nil (aa1 r z ey1 t))
("arzola" nil (aa0 r z ow1 l ax))
("arzt" nil (aa1 r z t))
("as" nil (ae1 z))
("asa" nil (aa1 s ax))
("asaf" nil (aa0 s aa1 f))
("asahan" nil (ae1 s ax hh ae0 n))
("asahi" nil (ax s aa1 hh iy0))
("asamera" nil (ae1 s ax m eh1 r ax))
("asarco" nil (ax s aa1 r k ow0))
("asaro" nil (aa0 s aa1 r ow0))
("asat" nil (ae1 z ae0 t))
("asato" nil (aa0 s aa1 t ow0))
("asay" nil (ax s ey1))
("asbell" nil (ae1 s b eh0 l))
("asberry" nil (ae1 s b eh0 r iy0))
("asbestec" nil (ae1 s b eh1 s t eh0 k))
("asbestos" nil (ae0 s b eh1 s t ax s))
("asbestosis" nil (ae1 s b eh1 s t ow1 s ax s))
("asbill" nil (ax s b ih1 l))
("asbridge" nil (ax s b r ih1 jh))
("asbury" nil (ae1 z b eh1 r iy0))
("asby" nil (ae1 s b iy0))
("ascap" nil (ae1 s k ae1 p))
("ascencio" nil (aa0 s ch eh1 n ch iy0 ow0))
("ascend" nil (ax s eh1 n d))
("ascendancy" nil (ax s eh1 n d ax n s iy0))
("ascendant" nil (ax s eh1 n d ax n t))
("ascended" nil (ax s eh1 n d ax d))
("ascendency" nil (ax s eh1 n d ax n s iy0))
("ascending" nil (ax s eh1 n d ih0 ng))
("ascends" nil (ax s eh1 n d z))
("ascension" nil (ax s eh1 n sh ax n))
("ascent" nil (ax s eh1 n t))
("ascertain" nil (ae1 s er0 t ey1 n))
("ascertained" nil (ae1 s er0 t ey1 n d))
("ascetic" nil (ax s eh1 t ih0 k))
("asch" nil (ae1 sh))
("asche" nil (ae1 sh))
("aschenbach" nil (ae1 sh ih0 n b aa0 k))
("aschenbrenner" nil (ae1 sh ih0 n b r ih0 n er0))
("ascher" nil (ae1 sh er0))
("aschoff" nil (ae1 s k hh ao0 f))
("ascii" nil (ae1 s k iy0))
("ascorbic" nil (ax s k ao1 r b ih0 k))
("ascot" nil (ae1 s k aa1 t))
("ascott" nil (ax s k aa1 t))
("ascribe" nil (ax s k r ay1 b))
("ascribed" nil (ax s k r ay1 b d))
("ascribes" nil (ax s k r ay1 b z))
("asda" nil (ae1 s d ax))
("asea" nil (ax z iy1 ax))
("asean" nil (ax z iy1 ax n))
("aselma" nil (ax s eh1 l m ax))
("aseltine" nil (aa0 s eh0 l t iy1 n iy0))
("asencio" nil (ax s eh1 n s iy0 ow0))
("aseritis" nil (ax s eh1 r ih0 t ih0 s))
("ash" nil (ae1 sh))
("ashamed" nil (ax sh ey1 m d))
("ashare" nil (ae1 sh eh1 r))
("ashbaugh" nil (ax sh b ao1))
("ashbrook" nil (ae1 sh b r uh1 k))
("ashburn" nil (ae1 sh b er0 n))
("ashby" nil (ae1 sh b iy0))
("ashcraft" nil (ae1 sh k r ae1 f t))
("ashcreek" nil (ae1 sh k r iy1 k))
("ashcroft" nil (ae1 sh k r ao1 f t))
("ashdown" nil (ae1 sh d aw1 n))
("ashe" nil (ae1 sh))
("ashen" nil (ae1 sh ax n))
("ashenberg" nil (ae1 sh ax n b er0 g))
("ashenfelter" nil (ae1 sh ax n f eh1 l t er0))
("asher" nil (ae1 sh er0))
("ashes" nil (ae1 sh ax z))
("asheville" nil (ae1 sh v ih1 l))
("ashford" nil (ae1 sh f er0 d))
("ashland" nil (ae1 sh l ax n d))
("ashley" nil (ae1 sh l iy0))
("ashlin" nil (ae1 sh l ih0 n))
("ashline" nil (ae1 sh l ay1 n))
("ashlock" nil (ae1 sh l aa1 k))
("ashman" nil (ae1 sh m ax n))
("ashmead" nil (ae1 sh m iy1 d))
("ashmore" nil (ae1 sh m ao0 r))
("ashok" nil (ae1 sh aa0 k))
("ashore" nil (ax sh ao1 r))
("ashton" nil (ae1 sh t ax n))
("ashtray" nil (ae1 sh t r ey1))
("ashtrays" nil (ae1 sh t r ey1 z))
("ashur" nil (ae1 sh er0))
("ashurst" nil (ae1 sh er0 s t))
("ashwell" nil (ae1 sh w eh1 l))
("ashwood" nil (ae1 sh w uh1 d))
("ashworth" nil (ae1 sh w er1 th))
("ashy" nil (ae1 sh iy0))
("asia" nil (ey1 zh ax))
("asiain" nil (ey1 z iy0 ey1 n))
("asiamerica" nil (ey1 s iy0 ax m eh1 r ih0 k ax))
("asian" nil (ey1 zh ax n))
("asians" nil (ey1 zh ax n z))
("asiatic" nil (ey1 zh iy0 ae1 t ih0 k))
("asiaweek" nil (ey1 zh ax w iy1 k))
("asic" nil (ae1 z ih0 k))
("asics" nil (ae1 z ih0 k s))
("aside" nil (ax s ay1 d))
("asides" nil (ax s ay1 d z))
("asiel" nil (ae1 z iy0 ax l))
("asimov" nil (ae1 s ih0 m aa0 v))
("asimow" nil (ae1 s ih0 m ow0))
("asinine" nil (ae1 s ax n ay1 n))
("ask" nil (ae1 s k))
("aska" nil (ae1 s k ax))
("askance" nil (ax s k ae1 n s))
("asked" nil (ae1 s k t))
("asker" nil (ae1 s k er0))
("askers" nil (ae1 s k er0 z))
("askew" nil (ax s k y uw1))
("askey" nil (ax s k iy1))
("askin" nil (ax s k ih1 n))
("asking" nil (ae1 s k ih0 ng))
("askington" nil (ae1 s k ih0 ng t ax n))
("askins" nil (ax s k ih1 n z))
("asko" nil (ae1 s k ow0))
("askoldov" nil (ax s k ow1 l d aa0 v))
("askren" nil (ae1 s k er0 ax n))
("asks" nil (ae1 s k s))
("aslanian" nil (ax s l ey1 n iy0 ax n))
("asleep" nil (ax s l iy1 p))
("asleson" nil (ae1 s l ih0 s ax n))
("aslin" nil (ax s l ih1 n))
("asman" nil (ae1 s m ax n))
("asmara" nil (ae1 z m aa1 r ax))
("asmus" nil (ax z m uw1 s))
("asmussen" nil (ax z m ah1 s ax n))
("asner" nil (ae1 s n er0))
("asp" nil (ae1 s p))
("asparagus" nil (ax s p eh1 r ax g ax s))
("aspartame" nil (ae1 s p er0 t ey1 m))
("aspect" nil (ae1 s p eh1 k t))
("aspects" nil (ae1 s p eh1 k t s))
("aspen" nil (ae1 s p ax n))
("aspens" nil (ae1 s p ax n z))
("asper" nil (ae1 s p er0))
("aspersions" nil (ax s p er1 zh ax n z))
("asphalt" nil (ae1 s f ao1 l t))
("aspin" nil (ae1 s p ih0 n))
("aspinall" nil (ae1 s p ih0 n ao0 l))
("aspirant" nil (ae1 s p er0 ax n t))
("aspirants" nil (ae1 s p er0 ax n t s))
("aspirate" n (ae1 s p er0 ax t))
("aspirate" v (ae1 s p er0 ey1 t))
("aspiration" nil (ae1 s p er0 ey1 sh ax n))
("aspirations" nil (ae1 s p er0 ey1 sh ax n z))
("aspire" nil (ax s p ay1 r))
("aspired" nil (ax s p ay1 r d))
("aspires" nil (ax s p ay1 er0 z))
("aspirin" nil (ae1 s p r ih0 n))
("aspiring" nil (ax s p ay1 r ih0 ng))
("asplund" nil (ae1 s p l ax n d))
("asquith" nil (ae1 s k w ih0 th))
("ass" nil (ae1 s))
("assad" nil (ax s aa1 d))
("assaf" nil (ax s ae1 f))
("assail" nil (ax s ey1 l))
("assailant" nil (ax s ey1 l ax n t))
("assailants" nil (ax s ey1 l ax n t s))
("assailed" nil (ax s ey1 l d))
("assailing" nil (ax s ey1 l ih0 ng))
("assails" nil (ax s ey1 l z))
("assante" nil (aa0 s aa1 n t iy0))
("assassin" nil (ax s ae1 s ax n))
("assassinate" nil (ax s ae1 s ax n ey1 t))
("assassinated" nil (ax s ae1 s ax n ey1 t ax d))
("assassination" nil (ax s ae1 s ax n ey1 sh ax n))
("assassinations" nil (ax s ae1 s ax n ey1 sh ax n z))
("assassins" nil (ax s ae1 s ax n z))
("assault" nil (ax s ao1 l t))
("assaulted" nil (ax s ao1 l t ih0 d))
("assaulting" nil (ax s ao1 l t ih0 ng))
("assaults" nil (ax s ao1 l t s))
("assay" nil (ae1 s iy0))
("assayer" nil (ae0 s ey1 er0))
("assed" nil (ae1 s t))
("asselin" nil (ae1 s ih0 l ih0 n))
("asselstine" nil (ae1 s ax l s t ay1 n))
("assemblage" nil (ax s eh1 m b l ax jh))
("assemble" nil (ax s eh1 m b ax l))
("assembled" nil (ax s eh1 m b ax l d))
("assembler" nil (ax s eh1 m b l er0))
("assemblers" nil (ax s eh1 m b l er0 z))
("assembles" nil (ax s eh1 m b ax l z))
("assemblies" nil (ax s eh1 m b l iy0 z))
("assembling" nil (ax s eh1 m b ax l ih0 ng))
("assembly" nil (ax s eh1 m b l iy0))
("assemblyman" nil (ax s eh1 m b l iy0 m ae1 n))
("assemblymen" nil (ax s eh1 m b l iy0 m ih0 n))
("assent" nil (ax s eh1 n t))
("assert" nil (ax s er1 t))
("asserted" nil (ax s er1 t ax d))
("assertedly" nil (ax s er1 t ih0 d l iy0))
("asserting" nil (ax s er1 t ih0 ng))
("assertion" nil (ax s er1 sh ax n))
("assertions" nil (ax s er1 sh ax n z))
("assertive" nil (ax s er1 t ih0 v))
("assertiveness" nil (ax s er1 t ih0 v n ax s))
("asserts" nil (ax s er1 t s))
("asses" nil (ae1 s ax z))
("assess" nil (ax s eh1 s))
("assessed" nil (ax s eh1 s t))
("assesses" nil (ax s eh1 s ih0 z))
("assessing" nil (ax s eh1 s ih0 ng))
("assessment" nil (ax s eh1 s m ax n t))
("assessments" nil (ax s eh1 s m ax n t s))
("assessor" nil (ax s eh1 s er0))
("assessors" nil (ax s eh1 s er0 z))
("asset" nil (ae1 s eh1 t))
("assets" nil (ae1 s eh1 t s))
("assicurazioni" nil (ax s iy1 k er0 ae1 z iy0 ow1 n iy0))
("assiduous" nil (ax s ih1 d w ax s))
("assiduously" nil (ax s ih1 d w ax s l iy0))
("assign" nil (ax s ay1 n))
("assigned" nil (ax s ay1 n d))
("assigning" nil (ax s ay1 n ih0 ng))
("assignment" nil (ax s ay1 n m ax n t))
("assignments" nil (ax s ay1 n m ax n t s))
("assigns" nil (ax s ay1 n z))
("assimilate" nil (ax s ih1 m ax l ey1 t))
("assimilated" nil (ax s ih1 m ax l ey1 t ih0 d))
("assimilating" nil (ax s ih1 m ax l ey1 t ih0 ng))
("assimilation" nil (ax s ih1 m ax l ey1 sh ax n))
("assisi" nil (ax s iy1 s iy0))
("assist" nil (ax s ih1 s t))
("assistance" nil (ax s ih1 s t ax n s))
("assistant" nil (ax s ih1 s t ax n t))
("assistants" nil (ax s ih1 s t ax n t s))
("assisted" nil (ax s ih1 s t ax d))
("assisting" nil (ax s ih1 s t ih0 ng))
("assists" nil (ax s ih1 s t s))
("assocation" nil (ae1 s ax k ey1 sh ax n))
("associate" n (ax s ow1 s iy0 ax t))
("associate" v (ax s ow1 s iy0 ey0 t))
("associated" nil (ax s ow1 s iy0 ey1 t ax d))
("associates" n (ax s ow1 s iy0 ih0 t s))
("associates" v (ax s ow1 s iy0 ey0 t s))
("associating" nil (ax s ow1 s iy0 ey1 t ih0 ng))
("association" nil (ax s ow1 s iy0 ey1 sh ax n))
("associations" nil (ax s ow1 s iy0 ey1 sh ax n z))
("associes" nil (ae1 s ax s iy0 z))
("assorted" nil (ax s ao1 r t ih0 d))
("assortment" nil (ax s ao1 r t m ax n t))
("assuage" nil (ax s w ey1 jh))
("assuaged" nil (ax s w ey1 jh d))
("assubel" nil (ae1 s ax b eh1 l))
("assume" nil (ax s uw1 m))
("assumed" nil (ax s uw1 m d))
("assumes" nil (ax s uw1 m z))
("assuming" nil (ax s uw1 m ih0 ng))
("assumption" nil (ax s ah1 m p sh ax n))
("assumptions" nil (ax s ah1 m p sh ax n z))
("assurance" nil (ax sh uh1 r ax n s))
("assurances" nil (ax sh uh1 r ax n s ih0 z))
("assure" nil (ax sh uh1 r))
("assured" nil (ax sh uh1 r d))
("assuredly" nil (ax sh uh1 r ax d l iy0))
("assures" nil (ax sh uh1 r z))
("assuring" nil (ax sh uh1 r ih0 ng))
("assyria" nil (ax s ih1 r iy0 ax))
("assyrian" nil (ax s ih1 r iy0 ax n))
("assyrians" nil (ax s ih1 r iy0 ax n z))
("ast" nil (ae1 s t))
("asta" nil (aa1 s t ax))
("astaire" nil (ax s t eh1 r))
("aster" nil (ae1 s t er0))
("asterisk" nil (ae1 s t er0 ih0 s k))
("asteroid" nil (ae1 s t er0 oy1 d))
("asteroids" nil (ae1 s t er0 oy1 d z))
("asters" nil (ae1 s t er0 z))
("asthma" nil (ae1 z m ax))
("asthmatic" nil (ae0 z m ae1 t ih0 k))
("asthmatics" nil (eh0 s th m eh1 t ih0 k s))
("astigmatism" nil (ax s t ih1 g m ax t ih1 z ax m))
("astin" nil (ax s t ih1 n))
("astle" nil (ae1 s ax l))
("astley" nil (ae1 s t l iy0))
("aston" nil (ae1 s t ax n))
("astonished" nil (ax s t aa1 n ih0 sh t))
("astonishing" nil (ax s t aa1 n ih0 sh ih0 ng))
("astonishingly" nil (ax s t aa1 n ih0 sh ih0 ng l iy0))
("astonishment" nil (ax s t aa1 n ih0 sh m ax n t))
("astor" nil (ae1 s t er0))
("astorga" nil (aa0 s t ao1 r g ax))
("astoria" nil (ae1 s t ao1 r iy0 ax))
("astorino" nil (aa0 s t ao0 r iy1 n ow0))
("astound" nil (ax s t aw1 n d))
("astounded" nil (ax s t aw1 n d ih0 d))
("astounding" nil (ax s t aw1 n d ih0 ng))
("astoundingly" nil (ax s t aw1 n d ih0 ng l iy0))
("astounds" nil (ax s t aw1 n d z))
("astra" nil (ae1 s t r ax))
("astraddle" nil (ax s t r ae1 d ax l))
("astral" nil (ae1 s t r ax l))
("astray" nil (ax s t r ey1))
("astred" nil (ae1 s t er0 d))
("astrid" nil (ae1 s t r ih0 d))
("astride" nil (ax s t r ay1 d))
("astringent" nil (ax s t r ih1 n jh ax n t))
("astringents" nil (ax s t r ih1 n jh ax n t s))
("astro" nil (ae1 s t r ow0))
("astrodome" nil (ae1 s t r ax d ow1 m))
("astrogeology" nil (ae1 s t r ax jh iy1 aa1 l ax jh iy0))
("astrologer" nil (ax s t r aa1 l ax jh er0))
("astrologers" nil (ax s t r aa1 l ax jh er0 z))
("astrological" nil (ae1 s t r ax l aa1 jh ih0 k ax l))
("astrology" nil (ax s t r aa1 l ax jh iy0))
("astronaut" nil (ae1 s t r ax n aa1 t))
("astronautical" nil (ae1 s t r ax n aa1 t ih0 k ax l))
("astronautics" nil (ae1 s t r ax n aa1 t ih0 k s))
("astronauts" nil (ae1 s t r ax n ao1 t s))
("astronomer" nil (ax s t r aa1 n ax m er0))
("astronomers" nil (ax s t r aa1 n ax m er0 z))
("astronomical" nil (ae1 s t r ax n aa1 m ih0 k ax l))
("astronomically" nil (ae1 s t r ax n aa1 m ih0 k l iy0))
("astronomy" nil (ax s t r aa1 n ax m iy0))
("astrophotography" nil (ae1 s t r ow0 f ax t aa1 g r ax f iy0))
("astrophysicist" nil (ae1 s t r ow0 f ih1 s ih0 s ih0 s t))
("astrophysics" nil (ae1 s t r ow0 f ih1 z ih0 k s))
("astros" nil (ae1 s t r ow0 s))
("astrotech" nil (ae1 s t r ow0 t eh1 k))
("astroturf" nil (ae1 s t r ow0 t er1 f))
("astute" nil (ax s t uw1 t))
("astutely" nil (ax s t uw1 t l iy0))
("astuteness" nil (ax s t uw1 t n ax s))
("asuncion" nil (ax s ah1 n sh ax n))
("asunder" nil (ax s ah1 n d er0))
("aswin" nil (ax s w ih1 n))
("asylum" nil (ax s ay1 l ax m))
("asymmetrical" nil (ey1 s ax m eh1 t r ih0 k ax l))
("asymmetries" nil (ey1 s ih1 m ax t r iy0 z))
("asymmetry" nil (ey1 s ih1 m ax t r iy0))
("asymptotes" nil (ae1 s ax m t ow1 t s))
("asynchronous" nil (ey1 s ih1 ng k r ax n ax s))
("at" nil (ae1 t))
("ata" nil (aa1 t ax))
("atalanta" nil (ae1 t ax l ae1 n t ax))
("atalaya" nil (aa0 t aa0 l ey1 ax))
("atalia" nil (aa0 t aa1 l iy0 ax))
("atalie" nil (ae1 t ax l iy0))
("atamian" nil (ax t ey1 m iy0 ax n))
("atari" nil (ax t aa1 r iy0))
("ataturk" nil (ae1 t ax t er1 k))
("ataxia" nil (ey1 t ae1 k s iy0 ax))
("atcheson" nil (ae1 ch ih0 s ax n))
("atchinson" nil (ae1 ch ih0 n s ax n))
("atchison" nil (ae1 ch ih0 s ax n))
("atchley" nil (ae1 ch l iy0))
("atco" nil (ae1 t k ow0))
("atcor" nil (ae1 t k ao0 r))
("ate" nil (ey1 t))
("atek" nil (ey1 t eh1 k))
("aten" nil (ey1 t ax n))
("atencio" nil (ax t eh1 n s iy0 ow0))
("ater" nil (ey1 t er0))
("ates" nil (ey1 t s))
("atha" nil (ae1 th ax))
("athalia" nil (ey0 th ae1 l iy0 ax))
("athanas" nil (ey0 th ae1 n ax z))
("athanassiou" nil (aa1 th ax n ax s iy1 uw0))
("athans" nil (ae1 th ax n z))
("athas" nil (ey1 dh ax z))
("athearn" nil (ey1 th er0 n))
("atheism" nil (ax th ay1 s ax m))
("atheist" nil (ey1 th iy0 ax s t))
("atheistic" nil (ey1 th iy0 ih1 s t ih0 k))
("atheists" nil (ey1 th iy0 ax s t s))
("athena" nil (ax th iy1 n ax))
("athenaeum" nil (ae1 th ax n iy1 ax m))
("athenaios" nil (ae1 th ax n ay1 ow0 s))
("atheneum" nil (ae1 th ax n uw0 m))
("athenian" nil (ax th iy1 n iy0 ax n))
("athenians" nil (ax th iy1 n iy0 ax n z))
("athens" nil (ae1 th ax n z))
("atherosclerosis" nil (ae1 th er0 ow0 s k l er0 ow1 s ih0 s))
("atherton" nil (ae1 th er0 t ax n))
("athey" nil (ae1 th iy0))
("athie" nil (ey1 th iy0))
("athlete" nil (ae1 th l iy1 t))
("athletes" nil (ae1 th l iy1 t s))
("athletic" nil (ae0 th l eh1 t ih0 k))
("athletically" nil (ae0 th l eh1 t ih0 k l iy0))
("athletics" nil (ae0 th l eh1 t ih0 k s))
("athlone" nil (ae0 th l ow1 n))
("athwart" nil (ax th w ao1 r t))
("atico" nil (ae1 t ih0 k ow1))
("atienza" nil (aa0 t iy1 n z ax))
("atilano" nil (aa0 t iy0 l aa1 n ow0))
("atkerson" nil (ae1 t k er0 s ax n))
("atkin" nil (ax t k ih1 n))
("atkins" nil (ae1 t k ih0 n z))
("atkinson" nil (ae1 t k ax n s ax n))
("atkison" nil (ae1 t k ih0 s ax n))
("atkisson" nil (ae1 t k ih0 s ax n))
("atla" nil (ae1 t l ax))
("atlanta" nil (ae0 t l ae1 n t ax))
("atlantans" nil (ae1 t l ae1 n t ax n z))
("atlantic" nil (ax t l ae1 n t ih0 k))
("atlantica" nil (ae1 t l ae1 n t ih0 k ax))
("atlantico" nil (ae1 t l ae1 n t ih0 k ow0))
("atlantis" nil (ae0 t l ae1 n t ih0 s))
("atlas" nil (ae1 t l ax s))
("atlases" nil (ae0 t l ey1 s ih0 z))
("atley" nil (ae1 t l iy0))
("atmel" nil (ae1 t m ax l))
("atmosphere" nil (ae1 t m ax s f ih1 r))
("atmospheric" nil (ae1 t m ax s f eh1 r ih0 k))
("atnip" nil (ae1 t n ih0 p))
("atocha" nil (ax t aa1 ch ax))
("atoll" nil (ae1 t aa0 l))
("atom" nil (ae1 t ax m))
("atomic" nil (ax t aa1 m ih0 k))
("atomizer" nil (ae1 t ax m ay1 z er0))
("atoms" nil (ae1 t ax m z))
("aton" nil (ax t aa1 n))
("atonal" nil (ey0 t ow1 n ax l))
("atone" nil (ax t ow1 n))
("atonement" nil (ax t ow1 n m ax n t))
("atop" nil (ax t aa1 p))
("ator" nil (eh1 t er0))
("atorino" nil (ae1 t er0 iy1 n ow0))
("atra" nil (ey1 t r ax))
("atria" nil (ey1 t r iy0 ax))
("atrium" nil (ey1 t r iy0 ax m))
("atrocious" nil (ax t r ow1 sh ax s))
("atrocities" nil (ax t r aa1 s ax t iy0 z))
("atrocity" nil (ax t r aa1 s ax t iy0))
("atrophied" nil (ae1 t r ax f iy0 d))
("atrophy" nil (ae1 t r ax f iy0))
("atsushi" nil (aa0 s s uw0 sh iy0))
("atta" nil (ae1 t ax))
("attach" nil (ax t ae1 ch))
("attache" nil (ae1 t ax sh ey1))
("attached" nil (ax t ae1 ch t))
("attaches" nil (ax t ae1 ch ih0 z))
("attaching" nil (ax t ae1 ch ih0 ng))
("attachment" nil (ax t ae1 ch m ax n t))
("attachments" nil (ax t ae1 ch m ax n t s))
("attack" nil (ax t ae1 k))
("attacked" nil (ax t ae1 k t))
("attacker" nil (ax t ae1 k er0))
("attackers" nil (ax t ae1 k er0 z))
("attacking" nil (ax t ae1 k ih0 ng))
("attacks" nil (ax t ae1 k s))
("attain" nil (ax t ey1 n))
("attainable" nil (ax t ey1 n ax b ax l))
("attainder" nil (ax t ey1 n d er0))
("attained" nil (ax t ey1 n d))
("attaining" nil (ax t ey1 n ih0 ng))
("attainment" nil (ax t ey1 n m ax n t))
("attains" nil (ax t ey1 n z))
("attali" nil (ax t aa1 l iy0))
("attalla" nil (ax t ae1 l ax))
("attanasio" nil (aa0 t aa0 n aa1 s iy0 ow0))
("attar" nil (ae1 t er0))
("attard" nil (ae1 t er0 d))
("attardo" nil (aa0 t aa1 r d ow0))
("attaway" nil (ae1 t ax w ey0))
("atteberry" nil (ae1 t b eh0 r iy0))
("attebery" nil (ax t eh1 b er0 iy0))
("attebury" nil (ae1 t b eh0 r iy0))
("attempt" nil (ax t eh1 m p t))
("attempted" nil (ax t eh1 m p t ax d))
("attempting" nil (ax t eh1 m p t ih0 ng))
("attempts" nil (ax t eh1 m p t s))
("attenborough" nil (ae1 t ah1 n b er0 ow0))
("attend" nil (ax t eh1 n d))
("attendance" nil (ax t eh1 n d ax n s))
("attendant" nil (ax t eh1 n d ax n t))
("attendants" nil (ax t eh1 n d ax n t s))
("attended" nil (ax t eh1 n d ax d))
("attendee" nil (ax t eh1 n d iy1))
("attendees" nil (ax t eh1 n d iy1 z))
("attending" nil (ax t eh1 n d ih0 ng))
("attends" nil (ax t eh1 n d z))
("attention" nil (ax t eh1 n sh ax n))
("attentions" nil (ax t eh1 n sh ax n z))
("attentive" nil (ax t eh1 n t ih0 v))
("attentively" nil (ax t eh1 n t ih0 v l iy0))
("attentiveness" nil (ax t eh1 n t ih0 v n ax s))
("attenuated" nil (ax t eh1 n y uw0 ey1 t ih0 d))
("atterberry" nil (ae1 t er0 b eh0 r iy0))
("atterbury" nil (ae1 t er0 b eh1 r iy0))
("attermann" nil (ae1 t er0 m ax n))
("attest" nil (ax t eh1 s t))
("attested" nil (ax t eh1 s t ih0 d))
("attesting" nil (ax t eh1 s t ih0 ng))
("attests" nil (ax t eh1 s t s))
("attic" nil (ae1 t ih0 k))
("attica" nil (ae1 t ih0 k ax))
("attics" nil (ae1 t ih0 k s))
("attie" nil (ae1 t iy0))
("attila" nil (ax t ih1 l ax))
("attire" nil (ax t ay1 er0))
("attired" nil (ax t ay1 r d))
("attitude" nil (ae1 t ax t uw1 d))
("attitudes" nil (ae1 t ax t uw1 d z))
("attitudinal" nil (ae1 t ax t uw1 d ax n ax l))
("attkisson" nil (ae1 t k ih0 s ax n))
("attleboro" nil (ae1 t ax l b er0 ow0))
("attlee" nil (ae1 t l iy0))
("attorney" nil (ax t er1 n iy0))
("attorneys" nil (ax t er1 n iy0 z))
("attract" nil (ax t r ae1 k t))
("attracted" nil (ax t r ae1 k t ax d))
("attracting" nil (ax t r ae1 k t ih0 ng))
("attraction" nil (ax t r ae1 k sh ax n))
("attractions" nil (ax t r ae1 k sh ax n z))
("attractive" nil (ax t r ae1 k t ih0 v))
("attractively" nil (ax t r ae1 k t ih0 v l iy0))
("attractiveness" nil (ax t r ae1 k t ih0 v n ax s))
("attracts" nil (ax t r ae1 k t s))
("attributable" nil (ax t r ih1 b y ax t ax b ax l))
("attribute" n (ae1 t r ax b y uw1 t))
("attribute" v (ax t r ih1 b y uw1 t))
("attributed" nil (ax t r ih1 b y ax t ax d))
("attributes" n (ae1 t r ax b y uw1 t s))
("attributes" v (ax t r ih1 b y uw1 t s))
("attributing" nil (ax t r ih1 b y ax t ih0 ng))
("attribution" nil (ae1 t r ih0 b y uw1 sh ax n))
("attridge" nil (ax t r ih1 jh))
("attrition" nil (ax t r ih1 sh ax n))
("attuned" nil (ax t uw1 n d))
("attwood" nil (ae1 t w uh1 d))
("attwoods" nil (ae1 t w uh1 d z))
("atwater" nil (ae1 t w ao0 t er0))
("atwell" nil (ax t w eh1 l))
("atwood" nil (ae1 t w uh1 d))
("atworth" nil (ae1 t w er0 th))
("atypical" nil (ey0 t ih1 p ih0 k ax l))
("atz" nil (ae1 t s))
("au" nil (ow1))
("aube" nil (ao1 b))
("aubel" nil (aw1 b ax l))
("auber" nil (ao1 b er0))
("auberry" nil (ao1 b eh1 r iy0))
("aubert" nil (ao1 b er0 t))
("aubin" nil (ao1 b ih0 n))
("auble" nil (ao1 b ax l))
("aubrey" nil (ao1 b r iy0))
("aubry" nil (ao1 b r iy0))
("aubuchon" nil (aw1 b ax k ax n))
("auburn" nil (aa1 b er0 n))
("auch" nil (ao1 ch))
("auchter" nil (aw1 k t er0))
("auckland" nil (aa1 k l ax n d))
("auclair" nil (ow0 k l eh1 r))
("aucoin" nil (ow0 k oy1 n))
("aucott" nil (ao1 k aa1 t))
("auction" nil (aa1 k sh ax n))
("auctioned" nil (ao1 k sh ax n d))
("auctioneer" nil (aa1 k sh ax n ih1 r))
("auctioneering" nil (ao1 k sh ax n ih1 r ih0 ng))
("auctioneers" nil (ao1 k sh ax n ih1 r z))
("auctioning" nil (ao1 k sh ax n ih0 ng))
("auctions" nil (aa1 k sh ax n z))
("aud" nil (ao1 d))
("audacious" nil (aa0 d ey1 sh ax s))
("audacity" nil (aa0 d ae1 s ax t iy0))
("audas" nil (ow0 d aa1 z))
("auden" nil (ao1 d ax n))
("audet" nil (ow0 d eh1 t))
("audette" nil (ow0 d eh1 t))
("audi" nil (ao1 d iy0))
("audia" nil (ao1 d iy0 ax))
("audible" nil (aa1 d ax b ax l))
("audibly" nil (aa1 d ax b l iy0))
("audience" nil (aa1 d iy0 ax n s))
("audiences" nil (aa1 d iy0 ax n s ax z))
("audino" nil (ao1 d iy1 n ow0))
("audio" nil (aa1 d iy0 ow1))
("audiophiles" nil (ao1 d iy1 ow0 f iy1 l z))
("audiotape" nil (ao1 d iy0 ow0 t ey1 p))
("audiotex" nil (ao1 d iy0 ow0 t eh1 k s))
("audiotronics" nil (ao1 d iy0 ow0 t r aa1 n ih0 k s))
("audiovisual" nil (ao1 d iy0 ow0 v ih1 zh uw0 ax l))
("audiovox" nil (ao1 d iy0 ow0 v aa1 k s))
("audis" nil (ao1 d ih0 s))
("audit" nil (ao1 d ih0 t))
("audited" nil (ao1 d ih0 t ih0 d))
("auditing" nil (aa1 d ax t ih0 ng))
("audition" nil (aa0 d ih1 sh ax n))
("auditioned" nil (ao1 d ih1 sh ax n d))
("auditioning" nil (aa0 d ih1 sh ax n ih0 ng))
("auditions" nil (ao1 d ih1 sh ax n z))
("auditor" nil (ao1 d ih0 t er0))
("auditorium" nil (ao1 d ax t ao1 r iy0 ax m))
("auditoriums" nil (ao1 d ih0 t ao1 r iy0 ax m z))
("auditors" nil (aa1 d ax t er0 z))
("auditory" nil (ao1 d ih0 t ao1 r iy0))
("audits" nil (aa1 d ax t s))
("audran" nil (ao1 d r ax n))
("audrey" nil (ao1 d r iy0))
("audric" nil (ao1 d r ih0 k))
("audrie" nil (ao1 d er0 iy0))
("audris" nil (ow0 d r iy1 s))
("audry" nil (ao1 d r iy0))
("audubon" nil (ao1 d ax b aa0 n))
("audwin" nil (ao1 d w ih0 n))
("aue" nil (aw1))
("auel" nil (aw1 ax l))
("auen" nil (aw1 ax n))
("auer" nil (aw1 er0))
("auerbach" nil (aw1 er0 b aa0 k))
("auerback" nil (aw1 r b ae1 k))
("aufderheide" nil (aw1 f d er0 hh ay0 d))
("aufhauser" nil (ao1 f hh aw1 z er0))
("aufiero" nil (ow0 f ih1 r ow0))
("aug" nil (ao1 g ax s t))
("augat" nil (ao1 g ae0 t))
("auge" nil (ao1 jh))
("augello" nil (ao0 jh eh1 l ow0))
("augenstein" nil (aw1 g ax n s t ay0 n))
("auger" nil (ao1 g er0))
("aughenbaugh" nil (ao0 eh1 n b ao0))
("augment" nil (ao0 g m eh1 n t))
("augmentable" nil (aa0 g m eh1 n t ax b ax l))
("augmented" nil (aa0 g m eh1 n t ax d))
("augmenting" nil (ao1 g m eh1 n t ih0 ng))
("augsburg" nil (ao1 g z b er0 g))
("augsburger" nil (aw1 g z b er0 g er0))
("augspurger" nil (aw1 g s p er0 g er0))
("augur" nil (ao1 g er0))
("auguring" nil (ao1 g er0 ih0 ng))
("augurs" nil (ao1 g er0 z))
("august" nil (aa1 g ax s t))
("augusta" nil (ax g ah1 s t ax))
("auguste" nil (ao1 g ax s t))
("augustin" nil (aw0 g uw0 s t iy1 n))
("augustine" nil (aa1 g ax s t iy1 n))
("augustinians" nil (aa1 g ax s t ih1 n iy0 ax n z))
("augusto" nil (ao1 g ah1 s t ow0))
("augustson" nil (ao1 g ax s t s ax n))
("augustus" nil (aa0 g ah1 s t ax s))
("augustyn" nil (aw0 g ah1 s t ih0 n))
("augustyniak" nil (aw0 g ax s t ih1 n iy0 ae0 k))
("auker" nil (ao1 k er0))
("aukerman" nil (aw1 k er0 m ax n))
("aul" nil (ao1 l))
("aulana" nil (aw1 l aa1 n ax))
("auld" nil (ao1 l d))
("aulds" nil (ao1 l d z))
("auletta" nil (ao0 l eh1 t ax))
("aull" nil (ao1 l))
("ault" nil (ao1 l t))
("aultman" nil (ao1 l t m ax n))
("auman" nil (ao1 m ax n))
("aumann" nil (ao1 m ax n))
("aument" nil (aw1 m ax n t))
("aumiller" nil (aw1 m ax l er0))
("aune" nil (ao1 n))
("aung" nil (ao1 ng))
("aungst" nil (aw1 ng g s t))
("aunt" nil (ae1 n t))
("aunts" nil (ae1 n t s))
("aupperle" nil (aw1 p er0 l iy0))
("aura" nil (ao1 r ax))
("aural" nil (ao1 r ax l))
("aurand" nil (ao1 r ax n d))
("aurea" nil (aw0 r ey1 aa0))
("aurelio" nil (aw0 r ey1 l iy0 ow0))
("auria" nil (ao1 r iy0 ax))
("aurich" nil (aw1 r ih0 k))
("auriemma" nil (ao0 r iy1 m ax))
("aurora" nil (er0 ao1 r ax))
("auroral" nil (er0 ao1 r ax l))
("aurum" nil (ao1 r ax m))
("aus" nil (aw1 z))
("ausbrooks" nil (aw1 s b r uh0 k s))
("ausburn" nil (aw1 s b er0 n))
("auschwitz" nil (ao1 sh w ih0 t s))
("ausherman" nil (aw1 sh er0 m ax n))
("ausimont" nil (aw1 s ih0 m aa1 n t))
("auslander" nil (aw1 s l ax n d er0))
("ausley" nil (ao1 s l iy0))
("ausman" nil (aw1 s m ax n))
("ausmus" nil (aw1 s m ih0 s))
("auspices" nil (ao1 s p ih0 s ih0 z))
("auspicious" nil (aa0 s p ih1 sh ax s))
("aussie" nil (ao1 s iy0))
("aust" nil (ao1 s t))
("austad" nil (aw1 s t ax d))
("austell" nil (aw1 s t ax l))
("austen" nil (ao1 s t ax n))
("auster" nil (ao1 s t er0))
("austere" nil (ao0 s t ih1 r))
("austerely" nil (ao1 s t ih1 r l iy0))
("austerity" nil (ao1 s t eh1 r ih0 t iy0))
("austill" nil (aw1 s t ax l))
("austin" nil (ao1 s t ax n))
("austine" nil (ao1 s t iy0 n))
("auston" nil (ao1 s t ax n))
("austral" nil (ao1 s t r ax l))
("australasia" nil (ao1 s t r ax l ey1 zh ax))
("australia" nil (ao0 s t r ey1 l y ax))
("australian" nil (ao0 s t r ey1 l y ax n))
("australians" nil (ao0 s t r ey1 l y ax n z))
("australopithecine" nil (ao0 s t r ey1 l ow0 p ih1 th ax s ay1 n))
("australs" nil (ao1 s t r ax l z))
("austrasia" nil (ao0 s t r ey1 zh ax))
("austria" nil (ao1 s t r iy0 ax))
("austrian" nil (ao1 s t r iy0 ax n))
("austrians" nil (ao1 s t r iy0 ax n z))
("austro" nil (ao1 s t r ow0))
("austroasiatic" nil (ao1 s t r ow1 ey1 zh iy0 ae1 t ih0 k))
("austron" nil (ao1 s t r ax n))
("auten" nil (aw1 t ax n))
("autenrieth" nil (aw1 t ih0 n r iy0 ih0 th))
("autery" nil (ow1 t er0 iy0))
("auth" nil (ao1 th))
("authement" nil (aw1 th m ax n t))
("authentic" nil (ax th eh1 n t ih0 k))
("authentically" nil (ao1 th eh1 n t ih0 k l iy0))
("authenticate" nil (ao0 th eh1 n t ax k ey1 t))
("authentication" nil (ao0 th eh1 n t ax k ey1 sh ax n))
("authentications" nil (ao0 th eh1 n t ax k ey1 sh ax n z))
("authenticity" nil (ao1 th ax n t ih1 s ih0 t iy0))
("authier" nil (aw1 th iy0 er0))
("author" nil (ao1 th er0))
("authored" nil (ao1 th er0 d))
("authoritarian" nil (ax th ao1 r ax t eh1 r iy0 ax n))
("authoritarianism" nil (ax th ao1 r ax t eh1 r iy0 ax n ih1 z ax m))
("authoritative" nil (ax th ao1 r ax t ey1 t ih0 v))
("authorities" nil (ax th ao1 r ax t iy0 z))
("authority" nil (ax th ao1 r ax t iy0))
("authorization" nil (ao1 th er0 ax z ey1 sh ax n))
("authorizations" nil (ao1 th er0 ax z ey1 sh ax n z))
("authorize" nil (ao1 th er0 ay1 z))
("authorized" nil (ao1 th er0 ay1 z d))
("authorizes" nil (ao1 th er0 ay1 z ih0 z))
("authorizing" nil (ao1 th er0 ay1 z ih0 ng))
("authors" nil (ao1 th er0 z))
("authorship" nil (ao1 th er0 sh ih1 p))
("autism" nil (ao1 t ih1 z ax m))
("autistic" nil (ao0 t ih1 s t ih0 k))
("auto" nil (ao1 t ow0))
("autobahn" nil (ao1 t ow0 b aa1 n))
("autobiographer" nil (ao1 t ax b ay0 aa1 g r ax f er0))
("autobiographical" nil (ao1 t ax b ay1 ax g r ae1 f ih0 k ax l))
("autobiographies" nil (ao1 t ax b ay0 aa1 g r ax f iy0 z))
("autobiography" nil (ao1 t ax b ay0 aa1 g r ax f iy0))
("autoclave" nil (ao1 t ow0 k l ey1 v))
("autocracy" nil (ao0 t aa1 k r ax s iy0))
("autocrat" nil (ao1 t ax k r ae1 t))
("autocratic" nil (ao1 t ax k r ae1 t ih0 k))
("autodesk" nil (ao1 t ow0 d eh1 s k))
("autodie" nil (ao1 t ow0 d ay1))
("autofacts" nil (ao1 t ow0 f ae1 k t s))
("autograph" nil (ao1 t ax g r ae1 f))
("autographed" nil (ao1 t ax g r ae1 f t))
("autographs" nil (ao1 t ax g r ae1 f s))
("autoimmune" nil (ao1 t oy1 m y uw0 n))
("autolatina" nil (ao1 t ow1 l ax t iy1 n ax))
("automaker" nil (ao1 t ow0 m ey1 k er0))
("automakers" nil (ao1 t ow0 m ey1 k er0 z))
("automap" nil (ao1 t ow0 m ae1 p))
("automata" nil (ao0 t aa1 m ax t ax))
("automate" nil (ao1 t ax m ey1 t))
("automated" nil (ao1 t ax m ey1 t ih0 d))
("automates" nil (ao1 t ax m ey1 t s))
("automatic" nil (ao1 t ax m ae1 t ih0 k))
("automatically" nil (ao1 t ax m ae1 t ih0 k l iy0))
("automatics" nil (ao1 t ax m ae1 t ih0 k s))
("automating" nil (ao1 t ax m ey1 t ih0 ng))
("automation" nil (ao0 t ax m ey1 sh ax n))
("automatix" nil (ao1 t ow0 m ey1 t ih0 k s))
("automaton" nil (ao0 t aa1 m ax t aa1 n))
("automobile" nil (ao1 t ax m ow0 b iy1 l))
("automobiles" nil (ao1 t ax m ow0 b iy1 l z))
("automobili" nil (ao1 t ow0 m ow0 b iy1 l iy0))
("automotive" nil (ao1 t ax m ow1 t ih0 v))
("automoviles" nil (ao1 t ow0 m ow0 v iy1 l eh0 z))
("auton" nil (ao1 t ax n))
("autonomic" nil (ao1 t ax n aa1 m ih0 k))
("autonomous" nil (ao0 t aa1 n ax m ax s))
("autonomously" nil (ao1 t aa1 n ow0 m ax s l iy0))
("autonomy" nil (ao0 t aa1 n ax m iy0))
("autopacific" nil (ao1 t ow0 p ax s ih1 f ih0 k))
("autoparts" nil (ao1 t ow0 p aa1 r t s))
("autopilot" nil (ao1 t ow0 p ay1 l ax t))
("autopsies" nil (ao1 t aa1 p s iy0 z))
("autopsy" nil (ao1 t aa1 p s iy0))
("autos" nil (ao1 t ow0 z))
("autospa" nil (ao1 t ow1 s p ax))
("autotote" nil (ao1 t ow0 t ow1 t))
("autotrophic" nil (ao1 t ax t r ow1 f ih0 k))
("autrey" nil (ao1 t r iy0))
("autry" nil (ao1 t r iy0))
("autumn" nil (ao1 t ax m))
("autumnal" nil (ao0 t ah1 m n ax l))
("auvil" nil (ow0 v ih1 l))
("aux" nil (ow1))
("auxier" nil (ao1 k s iy0 er0))
("auxiliary" nil (aa0 g z ih1 l y er0 iy0))
("auxton" nil (ao1 k s t ax n))
("auyeung" nil (aw1 y uw1 ng))
("auzenne" nil (ao1 z ih0 n))
("ava" nil (ey1 v ax))
("avail" nil (ax v ey1 l))
("availabilities" nil (ax v ey1 l ax b ih1 l ih0 t iy0 z))
("availability" nil (ax v ey1 l ax b ih1 l ax t iy0))
("available" nil (ax v ey1 l ax b ax l))
("availed" nil (ax v ey1 l d))
("avakian" nil (ax v ey1 k iy0 ax n))
("avalanche" nil (ae1 v ax l ae1 n ch))
("avallone" nil (ae1 v ax l ow1 n))
("avalon" nil (ae1 v ax l aa1 n))
("avalos" nil (aa0 v aa1 l ow0 z))
("avana" nil (ax v ae1 n ax))
("avant" nil (ax v aa1 n t))
("avantek" nil (ax v aa1 n t eh0 k))
("avanti" nil (ax v aa1 n t iy0))
("avants" nil (aa0 v ao1 n t s))
("avarice" nil (ae1 v er0 ax s))
("avasso" nil (ax v aa1 s ow1))
("avco" nil (ae1 v k ow0))
("avdel" nil (ae1 v d eh1 l))
("ave" nil (ae1 v ax n uw0))
("avedisian" nil (ae0 v ih0 d ih1 zh ih0 n))
("avelar" nil (aa0 v ey0 l aa1 r))
("aveline" nil (aa0 v eh0 l iy1 n iy0))
("avella" nil (ax v eh1 l ax))
("avellino" nil (ae1 v ax l iy1 n ow0))
("avemco" nil (ax v eh1 m k ow0))
("aven" nil (aa0 v ey1 ax n))
("avena" nil (aa0 v ey1 n ax))
("avenall" nil (aa0 v ey0 n aa1 l))
("avendano" nil (aa0 v eh0 n d aa1 n ow0))
("avenel" nil (aa0 v ey0 n eh1 l))
("avenell" nil (aa0 v ey0 n ey1 l))
("avenge" nil (ax v eh1 n jh))
("avenged" nil (ax v eh1 n jh d))
("avenger" nil (ax v eh1 n jh er0))
("avenging" nil (ax v eh1 n jh ih0 ng))
("aveni" nil (aa0 v eh1 n iy0))
("avent" nil (aa1 v ax n t))
("avenue" nil (ae1 v ax n uw1))
("avenues" nil (ae1 v ax n uw1 z))
("aver" nil (ey1 v er0))
("avera" nil (aa0 v eh1 r ax))
("average" nil (ae1 v er0 ih0 jh))
("averaged" nil (ae1 v r ax jh d))
("averages" nil (ae1 v r ih0 jh ih0 z))
("averaging" nil (ae1 v r ih0 jh ih0 ng))
("averbeck" nil (ey1 v er0 b eh1 k))
("averell" nil (aa0 v eh0 r ey1 l))
("averett" nil (ae1 v er0 eh0 t))
("averette" nil (ae1 v er0 eh1 t))
("averil" nil (ey1 v er0 ih0 l))
("averill" nil (aa0 v eh0 r iy1 l))
("averitt" nil (ae1 v er0 ih0 t))
("averof" nil (ae1 v er0 aa1 f))
("avers" nil (ey1 v er0 z))
("aversa" nil (aa0 v eh1 r s ax))
("aversano" nil (aa0 v er0 s aa1 n ow0))
("averse" nil (ax v er1 s))
("aversion" nil (ax v er1 zh ax n))
("avert" nil (ax v er1 t))
("averted" nil (ax v er1 t ax d))
("averting" nil (ax v er1 t ih0 ng))
("averts" nil (ax v er1 t s))
("avery" nil (ey1 v er0 iy0))
("averyl" nil (ae1 v er0 ih0 l))
("avey" nil (ey1 v iy0))
("avi" nil (aa1 v iy0))
("avia" nil (aa1 v iy0 ax))
("aviacion" nil (ey1 v iy0 ey1 sh ax n))
("aviall" nil (ey1 v iy0 ax l))
("aviano" nil (aa1 v iy0 aa1 n ow0))
("aviaries" nil (ey1 v iy0 eh1 r iy0 z))
("aviary" nil (ey1 v iy0 eh1 r iy0))
("aviation" nil (ey1 v iy0 ey1 sh ax n))
("aviator" nil (ey1 v iy0 ey1 t er0))
("aviators" nil (ey1 v iy0 ey0 t er0 z))
("aviazione" nil (ae1 v iy0 ae1 z iy0 ow1 n iy0))
("avice" nil (aa1 v ay0 s))
("avid" nil (ae1 v ax d))
("avidly" nil (ae1 v ax d l iy0))
("avila" nil (ax v ih1 l ax))
("aviles" nil (aa0 v iy1 l eh0 s))
("avilez" nil (aa0 v iy1 l eh0 z))
("avilla" nil (ax v ih1 l ax))
("avina" nil (aa0 v iy1 n ax))
("avinger" nil (ey1 v ih0 ng er0))
("avionic" nil (ey1 v iy0 aa1 n ih0 k))
("avionics" nil (ey1 v iy0 aa1 n ih0 k s))
("avions" nil (ey1 v iy0 ax n z))
("avis" nil (ey1 v ih0 s))
("avitabile" nil (aa0 v iy0 t aa1 b ax l))
("avitia" nil (aa0 v iy1 sh ax))
("aviv" nil (aa0 v iy1 v))
("avmark" nil (ae1 v m aa1 r k))
("avner" nil (ae1 v n er0))
("avnet" nil (ae1 v n eh1 t))
("avocado" nil (ae1 v ax k aa1 d ow0))
("avocados" nil (ae1 v ax k aa1 d ow0 z))
("avocation" nil (ae1 v ax k ey1 sh ax n))
("avocet" nil (ae1 v ax s eh1 t))
("avoid" nil (ax v oy1 d))
("avoidable" nil (ax v oy1 d ax b ax l))
("avoidance" nil (ax v oy1 d ax n s))
("avoided" nil (ax v oy1 d ax d))
("avoiding" nil (ax v oy1 d ih0 ng))
("avoids" nil (ax v oy1 d z))
("avolio" nil (ax v ow1 l iy0 ow0))
("avon" nil (ey1 v aa0 n))
("avondale" nil (ae1 v ax n d ey1 l))
("avowed" nil (ax v aw1 d))
("avowedly" nil (ax v aw1 ax d l iy0))
("avrett" nil (ae1 v r eh0 t))
("avril" nil (ae1 v r ih0 l))
("avrim" nil (aa0 v r iy1 m))
("avtex" nil (ae1 v t eh1 k s))
("avtovaz" nil (ae1 v t ow1 v ae1 z))
("avuncular" nil (ax v ah1 ng k y ax l er0))
("aw" nil (ao1))
("awacs" nil (ey1 w ae1 k s))
("awad" nil (ax w aa1 d))
("await" nil (ax w ey1 t))
("awaited" nil (ax w ey1 t ax d))
("awaiting" nil (ax w ey1 t ih0 ng))
("awaits" nil (ax w ey1 t s))
("awake" nil (ax w ey1 k))
("awaken" nil (ax w ey1 k ax n))
("awakened" nil (ax w ey1 k ax n d))
("awakening" nil (ax w ey1 k ax n ih0 ng))
("awakens" nil (ax w ey1 k ax n z))
("awalt" nil (aa1 v ax l t))
("awan" nil (ey1 w ax n))
("award" nil (ax w ao1 r d))
("awarded" nil (ax w ao1 r d ax d))
("awarding" nil (ax w ao1 r d ih0 ng))
("awards" nil (ax w ao1 r d z))
("aware" nil (ax w eh1 r))
("awareness" nil (ax w eh1 r n ax s))
("awash" nil (ax w aa1 sh))
("away" nil (ax w ey1))
("aways" nil (ey1 w ey1 z))
("awbrey" nil (ao1 b r iy0))
("awe" nil (aa1))
("awed" nil (ao1 d))
("aweida" nil (ax w ey1 d ax))
("awesome" nil (aa1 s ax m))
("awful" nil (aa1 f ax l))
("awfully" nil (aa1 f l iy0))
("awfulness" nil (ao1 f ax l n ax s))
("awhile" nil (ax w ay1 l))
("awkward" nil (aa1 k w er0 d))
("awkwardly" nil (ao1 k w er0 d l iy0))
("awkwardness" nil (ao1 k w er0 d n ax s))
("awoke" nil (ax w ow1 k))
("awry" nil (er0 ay1))
("awtrey" nil (ao1 t r iy0))
("ax" nil (ae1 k s))
("axa" nil (ae1 k s ax))
("axe" nil (ae1 k s))
("axed" nil (ae1 k s t))
("axel" nil (ae1 k s ax l))
("axelrad" nil (ae0 k s eh1 l r ax d))
("axelrod" nil (ae1 k s ax l r aa1 d))
("axelsen" nil (ae0 k s eh1 l s ax n))
("axelson" nil (ae1 k s ih0 l s ax n))
("axes" v (ae1 k s ih0 z))
("axes" n (ae1 k s iy0 z))
("axford" nil (ae0 k s f ao1 r d))
("axid" nil (ae1 k s ih0 d))
("axilrod" nil (ae1 k s ih0 l r aa1 d))
("axiom" nil (ae1 k s iy0 ax m))
("axiomatic" nil (ae1 k s iy0 ax m ae1 t ih0 k))
("axioms" nil (ae1 k s iy0 ax m z))
("axis" nil (ae1 k s ax s))
("axle" nil (ae1 k s ax l))
("axles" nil (ae1 k s ax l z))
("axley" nil (ae1 k s l iy0))
("axline" nil (ae1 k s l ay1 n))
("axlon" nil (ae1 k s l aa0 n))
("axon" nil (ae1 k s aa1 n))
("axons" nil (ae1 k s aa1 n z))
("axsom" nil (ae1 k s ax m))
("axt" nil (ae1 k s t))
("axtell" nil (ae0 k s t eh1 l))
("axtman" nil (ae1 k s t m ax n))
("axton" nil (ae1 k s t ax n))
("ay" nil (ey1))
("ayacucho" nil (ay1 ax k ah1 ch ow0))
("ayala" nil (ax y aa1 l ax))
("ayars" nil (ey1 er0 z))
("ayatollah" nil (ay1 ax t ow1 l ax))
("ayatollahs" nil (ay1 ax t aa1 l aa0 z))
("aybar" nil (ey1 b er0))
("aycock" nil (ey1 k ax k))
("aydelott" nil (ey1 d ih0 l aa0 t))
("aydelotte" nil (ey1 d ax l aa1 t))
("aydin" nil (ey1 d ih0 n))
("aydt" nil (ey1 t))
("aye" nil (ay1))
("ayer" nil (ay1 er0))
("ayers" nil (ay1 er0 z))
("ayerst" nil (ey1 er0 s t))
("ayes" nil (ay1 z))
("aykroyd" nil (ey1 k r oy1 d))
("ayler" nil (ey1 l er0))
("ayles" nil (ey1 l z))
("aylesworth" nil (ey1 l ih0 s w er0 th))
("ayling" nil (ey1 l ih0 ng))
("aylmer" nil (ey1 l m er0))
("aylor" nil (ey1 l er0))
("aylsworth" nil (ey1 l s w er0 th))
("aylward" nil (ey1 l w er0 d))
("aylwin" nil (ey1 l w ih0 n))
("aylworth" nil (ey1 l w er0 th))
("aymond" nil (ey1 m ax n d))
("ayn" nil (ey1 n))
("aynes" nil (ey1 n z))
("ayo" nil (ey1 ow0))
("ayodhya" nil (ay1 aa1 d y ax))
("ayon" nil (ey0 ao1 n))
("ayotte" nil (ey0 ao1 t))
("ayoub" nil (ey1 aw0 b))
("ayr" nil (ey1 r))
("ayre" nil (eh1 r))
("ayres" nil (eh1 r z))
("ayscue" nil (ey1 s k y uw0))
("aytes" nil (ey1 t s))
("ayuso" nil (ay0 uw1 s ow0))
("ayyad" nil (ay1 aa1 d))
("azactam" nil (ax z ae1 k t ae0 m))
("azalea" nil (ax z ey1 l y ax))
("azaleas" nil (ax z ey1 l y ax z))
("azalia" nil (aa0 z aa1 l iy0 ax))
("azar" nil (aa0 z aa1 r))
("azarow" nil (ae1 z er0 ow0))
("azbell" nil (ae1 z b eh0 l))
("azbill" nil (aa0 z b iy1 l))
("azcarraga" nil (ae0 z k aa1 r aa1 g ax))
("azcona" nil (ae0 z k ow1 n ax))
("azcuenaga" nil (ae0 z k w ey1 n aa1 g ax))
("azelia" nil (aa0 z eh1 l iy0 ax))
("azerbaijan" nil (aa1 z er0 b ay0 jh aa1 n))
("azerbaijani" nil (aa1 z er0 b ay0 jh aa1 n iy0))
("azerbaijanis" nil (aa1 z er0 b ay0 jh aa1 n iy0 z))
("azeri" nil (ax z eh1 r iy0))
("azevedo" nil (aa0 z ey0 v ey1 d ow0))
("azhar" nil (ax z aa1 r))
("azides" nil (ae1 z ay1 d z))
("azidothymidine" nil (ax z ih1 d ax th ay1 m ax d iy1 n))
("azinger" nil (ey1 z ih0 ng er0))
("aziz" nil (ax z iy1 z))
("azoff" nil (ae1 z ao0 f))
("azores" nil (ax z ao1 r z))
("azpurua" nil (ae0 z p y uh1 r uw0 ax))
("aztar" nil (ae1 z t er0))
("aztec" nil (ae1 z t eh1 k))
("azteca" nil (ae1 z t eh1 k ax))
("aztecs" nil (ae1 z t eh1 k s))
("azura" nil (ax z uh1 r ax))
("azure" nil (ae1 zh er0))
("azzara" nil (aa0 t s aa1 r ax))
("azzarello" nil (aa0 t s aa0 r eh1 l ow0))
("azzaro" nil (aa0 t s aa1 r ow0))
("azzato" nil (ax z aa1 t ow0))
("azzopardi" nil (aa0 t s ow0 p aa1 r d iy0))
("b" nil (b iy1))
("baab" nil (b aa1 b))
("baack" nil (b aa1 k))
("baade" nil (b aa1 d))
("baar" nil (b aa1 r))
("baars" nil (b aa1 r z))
("baas" nil (b aa1 z))
("baasch" nil (b aa1 sh))
("baatz" nil (b aa1 t s))
("bab" nil (b ae1 b))
("baba" nil (b aa1 b ax))
("babb" nil (b ae1 b))
("babbio" nil (b ae1 b iy0 ow0))
("babbit" nil (b ae1 b ih0 t))
("babbitt" nil (b ae1 b ih0 t))
("babbitts" nil (b ae1 b ax t s))
("babble" nil (b ae1 b ax l))
("babbled" nil (b ae1 b ax l d))
("babblers" nil (b ae1 b l er0 z))
("babbling" nil (b ae1 b ax l ih0 ng))
("babbs" nil (b ae1 b z))
("babcock" nil (b ae1 b k ao0 k))
("babe" nil (b ey1 b))
("babel" nil (b ae1 b ax l))
("baber" nil (b ey1 b er0))
("babers" nil (b ey1 b er0 z))
("babes" nil (b ey1 b z))
("babette" nil (b ax b eh1 t))
("babiak" nil (b aa1 b iy0 ae0 k))
("babiarz" nil (b ax b iy1 er0 z))
("babic" nil (b aa1 b ih0 k))
("babich" nil (b ae1 b ih0 ch))
("babies" nil (b ey1 b iy0 z))
("babik" nil (b aa1 b ih0 k))
("babin" nil (b ae1 b ih0 n))
("babineau" nil (b ae1 b ih0 n ow1))
("babineaux" nil (b ae1 b ih0 n ow1))
("babington" nil (b ae1 b ih0 ng t ax n))
("babino" nil (b aa0 b iy1 n ow0))
("babinski" nil (b ax b ih1 n s k iy0))
("babish" nil (b ae1 b ih0 sh))
("babita" nil (b aa0 b iy1 t ax))
("babka" nil (b ae1 b k ax))
("babler" nil (b ey1 b ax l er0))
("baboon" nil (b ax b uw1 n))
("baboons" nil (b ae0 b uw1 n z))
("babs" nil (b ae1 b z))
("babson" nil (b ae1 b s ax n))
("babu" nil (b aa0 b uw1))
("babula" nil (b aa0 b uw1 l ax))
("baby" nil (b ey1 b iy0))
("babyak" nil (b ae1 b iy0 ae0 k))
("babyhood" nil (b ey1 b iy0 hh uh1 d))
("babyish" nil (b ey1 b iy0 ih0 sh))
("babylon" nil (b ae1 b ax l aa1 n))
("babylonian" nil (b ae1 b ax l ow1 n iy0 ax n))
("babylonians" nil (b ae1 b ax l ow1 n iy0 ax n z))
("babysat" nil (b ey1 b iy0 s ae1 t))
("babysitter" nil (b ey1 b iy0 s ih1 t er0))
("babysitters" nil (b ey1 b iy0 s ih1 t er0 z))
("babysitting" nil (b ey1 b iy0 s ih1 t ih0 ng))
("baca" nil (b ae1 k ax))
("bacall" nil (b ax k ao1 l))
("bacardi" nil (b ax k aa1 r d iy0))
("baccalaureate" nil (b ae1 k ax l ao1 r iy0 ax t))
("baccarat" nil (b aa1 k er0 aa1))
("baccari" nil (b aa0 k aa1 r iy0))
("bacchanalia" nil (b ae1 k ax n ey1 l y ax))
("bacchi" nil (b ae1 k iy0))
("bacchus" nil (b ae1 k ih0 s))
("bacci" nil (b aa1 ch iy0))
("baccus" nil (b ae1 k ax s))
("bach" nil (b aa1 k))
("bacha" nil (b ae1 ch ax))
("bachand" nil (b ae1 ch ax n d))
("bachar" nil (b aa1 k er0))
("bacharach" nil (b ae1 k er0 ae0 k))
("bache" nil (b ae1 ch))
("bachelder" nil (b aa1 k ih0 l d er0))
("bacheller" nil (b aa1 k ax l er0))
("bachelor" nil (b ae1 ch ax l er0))
("bachelors" nil (b ae1 ch l er0 z))
("bacher" nil (b aa1 k er0))
("bachert" nil (b ae1 ch er0 t))
("bachler" nil (b ae1 k l er0))
("bachman" nil (b aa1 k m ax n))
("bachmann" nil (b aa1 k m ax n))
("bachmeier" nil (b aa1 k m ay0 er0))
("bachner" nil (b aa1 k n er0))
("bachrach" nil (b aa1 k r ax k))
("bachtel" nil (b ae1 k t ax l))
("bachtell" nil (b ae1 k t ax l))
("bachus" nil (b ae1 ch ax s))
("bacigalupi" nil (b aa0 ch iy0 g aa0 l uw1 p iy0))
("bacigalupo" nil (b aa0 ch iy0 g aa0 l uw1 p ow0))
("bacik" nil (b aa1 ch ih0 k))
("bacillus" nil (b ax s ih1 l ax s))
("bacino" nil (b aa0 ch iy1 n ow0))
("back" nil (b ae1 k))
("backache" nil (b ae1 k ey1 k))
("backaches" nil (b ae1 k ey1 k s))
("backbite" nil (b ae1 k b ay1 t))
("backbiting" nil (b ae1 k b ay1 t ih0 ng))
("backbone" nil (b ae1 k b ow1 n))
("backdated" nil (b ae1 k d ey1 t ih0 d))
("backdoor" nil (b ae1 k d ao1 r))
("backdrop" nil (b ae1 k d r aa1 p))
("backdrops" nil (b ae1 k d r aa1 p s))
("backe" nil (b ae1 k))
("backed" nil (b ae1 k t))
("backer" nil (b ae1 k er0))
("backers" nil (b ae1 k er0 z))
("backes" nil (b ae1 k s))
("backfield" nil (b ae1 k f iy1 l d))
("backfire" nil (b ae1 k f ay1 r))
("backfired" nil (b ae1 k f ay1 er0 d))
("backfires" nil (b ae1 k f ay1 er0 z))
("backfiring" nil (b ae1 k f ay1 er0 ih0 ng))
("backgammon" nil (b ae1 k g ae1 m ax n))
("background" nil (b ae1 k g r aw1 n d))
("backgrounds" nil (b ae1 k g r aw1 n d z))
("backhand" nil (b ae1 k hh ae1 n d))
("backhanded" nil (b ae1 k hh ae1 n d ax d))
("backhaus" nil (b ae1 k hh aw1 s))
("backhoe" nil (b ae1 k hh ow1))
("backing" nil (b ae1 k ih0 ng))
("backlash" nil (b ae1 k l ae1 sh))
("backless" nil (b ae1 k l ax s))
("backlights" nil (b ae1 k l ay1 t s))
("backlist" nil (b ae1 k l ih1 s t))
("backlog" nil (b ae1 k l aa1 g))
("backlogged" nil (b ae1 k l aa1 g d))
("backlogs" nil (b ae1 k l ao1 g z))
("backlund" nil (b ae1 k l ax n d))
("backman" nil (b ae1 k m ax n))
("backpack" nil (b ae1 k p ae1 k))
("backpacks" nil (b ae1 k p ae1 k s))
("backpedaled" nil (b ae1 k p eh1 d ax l d))
("backpedaling" nil (b ae1 k p eh1 d ax l ih0 ng))
("backrest" nil (b ae1 k r eh1 s t))
("backroom" nil (b ae1 k r uw1 m))
("backs" nil (b ae1 k s))
("backsaw" nil (b ae1 k s aa1))
("backset" nil (b ae1 k s eh1 t))
("backside" nil (b ae1 k s ay1 d))
("backslapping" nil (b ae1 k s l ae1 p ih0 ng))
("backslash" nil (b ae1 k s l ae1 sh))
("backslide" nil (b ae1 k s l ay1 d))
("backsliding" nil (b ae1 k s l ay1 d ih0 ng))
("backspin" nil (b ae1 k s p ih0 n))
("backstage" nil (b ae1 k s t ey1 jh))
("backstairs" nil (b ae1 k s t eh1 r z))
("backstitch" nil (b ae1 k s t ih0 ch))
("backstop" nil (b ae1 k s t aa1 p))
("backstroke" nil (b ae1 k s t r ow1 k))
("backstrom" nil (b ae1 k s t r ax m))
("backtrack" nil (b ae1 k t r ae1 k))
("backtracked" nil (b ae1 k t r ae1 k t))
("backtracking" nil (b ae1 k t r ae1 k ih0 ng))
("backup" nil (b ae1 k ah1 p))
("backups" nil (b ae1 k ah1 p s))
("backus" nil (b ae1 k ax s))
("backward" nil (b ae1 k w er0 d))
("backwardation" nil (b ae1 k w er0 d ey1 sh ax n))
("backwardness" nil (b ae1 k w er0 d n ax s))
("backwards" nil (b ae1 k w er0 d z))
("backwater" nil (b ae1 k w ao1 t er0))
("backwaters" nil (b ae1 k w ao1 t er0 z))
("backwoods" nil (b ae1 k w uh1 d z))
("backwoodsman" nil (b ae1 k w uh1 d z m ax n))
("backyard" nil (b ae1 k y aa1 r d))
("backyards" nil (b ae1 k y aa1 r d z))
("bacon" nil (b ey1 k ax n))
("bacorn" nil (b ae1 k er0 n))
("bacot" nil (b ae1 k ax t))
("bacote" nil (b ax k ow1 t))
("bacteria" nil (b ae0 k t ih1 r iy0 ax))
("bacterial" nil (b ae0 k t ih1 r iy0 ax l))
("bacteriology" nil (b ae1 k t ih0 r iy1 aa1 l ax jh iy0))
("bacterium" nil (b ae0 k t ih1 r iy0 ax m))
("bacus" nil (b ae1 k ih0 s))
("bad" nil (b ae1 d))
("badal" nil (b ey1 d ax l))
("badalamenti" nil (b aa0 d aa0 l aa0 m eh1 n t iy0))
("badami" nil (b aa0 d aa1 m iy0))
("badders" nil (b ae1 d er0 z))
("bade" nil (b ey1 d))
("badeau" nil (b ax d ow1))
("badeaux" nil (b ax d ow1))
("baden" nil (b ey1 d ax n))
("bader" nil (b ae1 d er0))
("badertscher" nil (b ae1 d er0 ch er0))
("badge" nil (b ae1 jh))
("badger" nil (b ae1 jh er0))
("badgered" nil (b ae1 jh er0 d))
("badgering" nil (b ae1 jh er0 ih0 ng))
("badgers" nil (b ae1 jh er0 z))
("badges" nil (b ae1 jh ih0 z))
("badgett" nil (b ae1 jh ih0 t))
("badgley" nil (b ae1 jh l iy0))
("badia" nil (b aa1 d iy0 ax))
("badillo" nil (b ax d ih1 l ow0))
("badlands" nil (b ae1 d l ae1 n d z))
("badley" nil (b ae1 d l iy0))
("badly" nil (b ae1 d l iy0))
("badman" nil (b ae1 d m ax n))
("badminton" nil (b ae1 d m ih0 n t ax n))
("badmintons" nil (b ae1 d m ih1 n t ax n z))
("badness" nil (b ae1 d n ax s))
("badolato" nil (b aa0 d ow0 l aa1 t ow0))
("badour" nil (b ax d uh1 r))
("badura" nil (b aa0 d uh1 r ax))
("bady" nil (b ey1 d iy0))
("bae" nil (b ay1))
("baechle" nil (b eh1 k ax l))
("baecker" nil (b eh1 k er0))
("baedeker" nil (b ey1 d ax k er0))
("baehr" nil (b iy1 r))
("baek" nil (b iy1 k))
("baer" nil (b eh1 r))
("baerwald" nil (b eh1 r w ao0 l d))
("baeten" nil (b eh1 t ax n))
("baetjer" nil (b eh1 t jh er0))
("baetz" nil (b iy1 t s))
("baez" nil (b ay0 eh1 z))
("baeza" nil (b iy1 z ax))
("baffa" nil (b ae1 f ax))
("baffle" nil (b ae1 f ax l))
("baffled" nil (b ae1 f ax l d))
("baffles" nil (b ae1 f ax l z))
("baffling" nil (b ae1 f ax l ih0 ng))
("bafokeng" nil (b ae1 f ow0 k eh1 ng))
("bag" nil (b ae1 g))
("bagan" nil (b ey1 g ax n))
("bagby" nil (b ae1 g b iy0))
("bagdad" nil (b ae1 g d ae0 d))
("bagdasarian" nil (b ae1 g d ax s eh1 r iy0 ax n))
("bagdikian" nil (b ae1 g d ih1 k iy0 ax n))
("bagdon" nil (b ae1 g d ax n))
("bagdonas" nil (b ae1 g d ax n ax z))
("bagel" nil (b ey1 g ax l))
("bagels" nil (b ey1 g ax l z))
("bagent" nil (b ey1 jh ax n t))
("bagful" nil (b ae1 g f uh1 l))
("bagg" nil (b ae1 g))
("baggage" nil (b ae1 g ax jh))
("bagge" nil (b ae1 g))
("bagged" nil (b ae1 g d))
("baggerly" nil (b ae1 g er0 l iy0))
("baggett" nil (b ae1 g ih0 t))
("bagginess" nil (b ae1 g iy0 n ax s))
("bagging" nil (b ae1 g ih0 ng))
("baggio" nil (b aa1 jh iy0 ow0))
("baggott" nil (b ae1 g ax t))
("baggs" nil (b ae1 g z))
("baggy" nil (b ae1 g iy0))
("baghdad" nil (b ae1 g d ae0 d))
("baghli" nil (b ae1 g l iy0))
("baginski" nil (b ax g ih1 n s k iy0))
("bagley" nil (b ae1 g l iy0))
("baglio" nil (b ae1 g l iy0 ow0))
("bagman" nil (b ae1 g m ax n))
("bagnall" nil (b ae1 g n ax l))
("bagnato" nil (b aa0 g n aa1 t ow0))
("bagnell" nil (b ae1 g n ax l))
("bagpipe" nil (b ae1 g p ay1 p))
("bagpiper" nil (b ae1 g p ay1 p er0))
("bagpipes" nil (b ae1 g p ay1 p s))
("bags" nil (b ae1 g z))
("bagsby" nil (b ae1 g z b iy0))
("bagshaw" nil (b ae1 g sh ao1))
("bagwell" nil (b ae1 g w eh1 l))
("bah" nil (b aa1))
("baham" nil (b aa1 ax m))
("bahama" nil (b ax hh aa1 m ax))
("bahamas" nil (b ax hh aa1 m ax z))
("bahamian" nil (b ax hh ey1 m iy0 ax n))
("bahan" nil (b ae1 hh ax n))
("bahe" nil (b ey1 hh))
("bahena" nil (b ae1 hh ih0 n ax))
("bahia" nil (b aa1 hh iy0 ax))
("bahl" nil (b aa1 l))
("bahler" nil (b aa1 l er0))
("bahls" nil (b aa1 l z))
("bahm" nil (b ae1 m))
("bahman" nil (b aa1 m ax n))
("bahn" nil (b ae1 n))
("bahner" nil (b aa1 n er0))
("bahnsen" nil (b aa1 n s ax n))
("bahr" nil (b eh1 r))
("bahrain" nil (b aa0 r ey1 n))
("bahraini" nil (b er0 ey1 n iy0))
("bahrainian" nil (b er0 ey1 n iy0 ax n))
("bahrainis" nil (b er0 ey1 n iy0 z))
("bahre" nil (b aa1 r))
("bahrenburg" nil (b aa1 r ax n b er0 g))
("baht" nil (b aa1 t))
("baidoa" nil (b ay1 d ow1 ax))
("baidoan" nil (b ay1 d ow1 ax n))
("baidoans" nil (b ay1 d ow1 ax n z))
("baidoas" nil (b ay1 d ow1 ax z))
("baie" nil (b ay1 iy0))
("baier" nil (b ey1 er0))
("baig" nil (b ey1 g))
("bail" nil (b ey1 l))
("bailard" nil (b ey1 l er0 d))
("bailby" nil (b ey1 l b iy0))
("baile" nil (b ey1 l))
("bailed" nil (b ey1 l d))
("bailer" nil (b ey1 l er0))
("bailes" nil (b ey1 l z))
("bailey" nil (b ey1 l iy0))
("baileys" nil (b ey1 l iy0 z))
("bailie" nil (b ey1 l iy0))
("bailiff" nil (b ey1 l ax f))
("bailiffs" nil (b ey1 l ax f s))
("bailin" nil (b eh0 l ae1 n))
("bailing" nil (b ey1 l ih0 ng))
("bailiwick" nil (b ey1 l ih0 w ih1 k))
("baillargeon" nil (b ax l aa1 r jh ih0 n))
("baillie" nil (b ey1 l iy0))
("bailly" nil (b ey1 l iy0))
("bailon" nil (b ey1 l ax n))
("bailor" nil (b ey1 l er0))
("bailout" nil (b ey1 l aw1 t))
("bailouts" nil (b ey1 l aw1 t s))
("bails" nil (b ey1 l z))
("baily" nil (b ey1 l iy0))
("bailyn" nil (b ey1 l ih0 n))
("baim" nil (b ey1 m))
("bain" nil (b ey1 n))
("bainbridge" nil (b ey1 n b r ih0 jh))
("baine" nil (b ey1 n))
("bainerd" nil (b ey1 n er0 d))
("baines" nil (b ey1 n z))
("bains" nil (b ey1 n z))
("bainter" nil (b ey1 n t er0))
("baio" nil (b aa1 iy0 ow0))
("bair" nil (b eh1 r))
("baird" nil (b eh1 r d))
("bairnco" nil (b eh1 r n k ow0))
("baisch" nil (b ay1 sh))
("baisden" nil (b ey1 s d ax n))
("baise" nil (b ey1 z))
("baisley" nil (b ey1 z l iy0))
("bait" nil (b ey1 t))
("baited" nil (b ey1 t ih0 d))
("baiting" nil (b ey1 t ih0 ng))
("baits" nil (b ey1 t s))
("baity" nil (b ey1 t iy0))
("baiul" nil (b ay1 uw1 l))
("baize" nil (b ey1 z))
("baja" nil (b aa1 hh aa1))
("bajarin" nil (b ae1 jh er0 ih0 n))
("bajema" nil (b aa0 y ey1 m ax))
("bajorek" nil (b ae1 jh er0 eh0 k))
("bak" nil (b ae1 k))
("baka" nil (b aa1 k ax))
("bakalar" nil (b ae1 k ax l er0))
("bake" nil (b ey1 k))
("baked" nil (b ey1 k t))
("bakeman" nil (b ey1 k m ax n))
("baker" nil (b ey1 k er0))
("bakeries" nil (b ey1 k er0 iy0 z))
("bakers" nil (b ey1 k er0 z))
("bakersfield" nil (b ey1 k er0 z f iy1 l d))
("bakery" nil (b ey1 k er0 iy0))
("bakes" nil (b ey1 k s))
("bakeware" nil (b ey1 k w eh1 r))
("bakewell" nil (b ey1 k w eh1 l))
("bakey" nil (b ey1 k iy0))
("baking" nil (b ey1 k ih0 ng))
("bakke" nil (b ae1 k))
("bakken" nil (b ae1 k ax n))
("bakker" nil (b ae1 k er0))
("bakkers" nil (b ae1 k er0 z))
("bakley" nil (b ae1 k l iy0))
("bakos" nil (b aa1 k ow0 z))
("bakowski" nil (b ax k ao1 f s k iy0))
("bakrie" nil (b ae1 k r ay0))
("bakshi" nil (b ae1 k sh iy0))
("bakst" nil (b ae1 k s t))
("baku" nil (b aa0 k uw1))
("bakula" nil (b ax k uw1 l ax))
("bal" nil (b ae1 l))
("bala" nil (b aa1 l ax))
("balaban" nil (b aa0 l aa0 b aa1 n))
("balaguer" nil (b ae1 l ax g er0))
("balakumar" nil (b aa1 l ax k uw0 m aa1 r))
("balan" nil (b ey1 l ax n))
("balance" nil (b ae1 l ax n s))
("balanced" nil (b ae1 l ax n s t))
("balances" nil (b ae1 l ax n s ax z))
("balanchine" nil (b ae1 l ax n ch iy1 n))
("balancing" nil (b ae1 l ax n s ih0 ng))
("balas" nil (b ae1 l ax s))
("balash" nil (b ae1 l ax sh))
("balata" nil (b ae1 l ax t ax))
("balazs" nil (b ao1 l ax sh))
("balbach" nil (b ae1 l b aa0 k))
("balbi" nil (b ae1 l b iy0))
("balbina" nil (b aa0 l b iy1 n ax))
("balbo" nil (b ae1 l b ow0))
("balboa" nil (b aa0 l b ow1 ax))
("balboni" nil (b aa0 l b ow1 n iy0))
("balbriggan" nil (b ae0 l b r ih1 g ax n))
("balbriggans" nil (b ae0 l b r ih1 g ax n z))
("balcer" nil (b ey1 l s er0))
("balcerowicz" nil (b ae0 l s eh1 r ax w ih0 t s))
("balcerzak" nil (b ax l ch er1 z ax k))
("balch" nil (b ae1 l ch))
("balchunas" nil (b ae1 l k uw0 n ax z))
("balcom" nil (b ae1 l k ax m))
("balconies" nil (b ae1 l k ax n iy0 z))
("balcony" nil (b ae1 l k ax n iy0))
("balcor" nil (b ae1 l k ao0 r))
("bald" nil (b ao1 l d))
("baldassare" nil (b aa0 l d aa0 s aa1 r iy0))
("baldassari" nil (b aa0 l d aa0 s aa1 r iy0))
("baldassarre" nil (b ao0 l d ax s aa1 r iy0))
("baldauf" nil (b ae1 l d aw0 f))
("baldelli" nil (b aa0 l d eh1 l iy0))
("baldemar" nil (b aa0 l d ey0 m aa1 r))
("balder" nil (b ao1 l d er0))
("balderas" nil (b ae1 l d er0 ax z))
("balderdash" nil (b ao1 l d er0 d ae1 sh))
("balderrama" nil (b aa0 l d eh0 r aa1 m ax))
("balderson" nil (b ae1 l d er0 s ax n))
("balderston" nil (b ae1 l d er0 s t ax n))
("baldi" nil (b aa1 l d iy0))
("balding" nil (b ao1 l d ih0 ng))
("baldinger" nil (b ao1 l d ih0 ng er0))
("baldini" nil (b aa0 l d iy1 n iy0))
("baldino" nil (b aa0 l d iy1 n ow0))
("baldly" nil (b ao1 l d l iy0))
("baldner" nil (b ae1 l d n er0))
("baldness" nil (b ao1 l d n ax s))
("baldo" nil (b aa1 l d ow0))
("baldock" nil (b ae1 l d ax k))
("baldonado" nil (b aa0 l d ow0 n aa1 d ow0))
("baldoni" nil (b aa0 l d ow1 n iy0))
("baldor" nil (b ao1 l d er0))
("baldree" nil (b ax l d r iy1))
("baldric" nil (b ae1 l d r ih0 k))
("baldridge" nil (b ae1 l d r ih0 jh))
("baldrige" nil (b ao1 l d r ih0 jh))
("baldry" nil (b ao1 l d r iy0))
("balducci" nil (b aa0 l d uw1 ch iy0))
("baldur" nil (b aa0 l d uh1 r))
("baldus" nil (b aa1 l d ih0 s))
("baldwin" nil (b ao1 l d w ax n))
("baldyga" nil (b aa1 l d iy0 g ax))
("bale" nil (b ey1 l))
("baleen" nil (b ax l iy1 n))
("baleful" nil (b ey1 l f ax l))
("balentine" nil (b aa0 l eh0 n t iy1 n iy0))
("bales" nil (b ey1 l z))
("balestra" nil (b ax l eh1 s t r ax))
("balestrieri" nil (b aa0 l eh0 s t r ih1 r iy0))
("baley" nil (b ey1 l iy0))
("balfanz" nil (b aa1 l f aa0 n z))
("balfe" nil (b ey1 l f))
("balfour" nil (b ae1 l f ao0 r))
("bali" nil (b aa1 l iy0))
("balian" nil (b ey1 l iy0 ax n))
("balicki" nil (b ax l ih1 k iy0))
("balik" nil (b aa1 l ih0 k))
("baliles" nil (b ax l iy1 l eh0 z))
("balin" nil (b ae1 l ih0 n))
("baling" nil (b ey1 l ih0 ng))
("balinski" nil (b ax l ih1 n s k iy0))
("balint" nil (b aa1 l iy0 n t))
("balis" nil (b aa1 l iy0 z))
("balistreri" nil (b aa0 l iy0 s t r eh1 r iy0))
("balk" nil (b ao1 k))
("balkan" nil (b ao1 l k ax n))
("balkanized" nil (b ao1 l k ax n ay1 z d))
("balkans" nil (b ao1 l k ax n z))
("balkcom" nil (b ae1 l k ax m))
("balke" nil (b ey1 l k))
("balked" nil (b ao1 k t))
("balkin" nil (b ae1 l k ih0 n))
("balking" nil (b ao1 k ih0 ng))
("balko" nil (b ae1 l k ow0))
("balks" nil (b ao1 k s))
("balky" nil (b ao1 k iy0))
("ball" nil (b ao1 l))
("balla" nil (b ae1 l ax))
("ballad" nil (b ae1 l ax d))
("ballade" nil (b ax l aa1 d))
("ballads" nil (b ae1 l ax d z))
("balladur" nil (b ae1 l ax d er0))
("ballance" nil (b aa1 l ax n s))
("ballantine" nil (b ae1 l ax n t ay1 n))
("ballantrae" nil (b ae1 l ax n t r ey1))
("ballantyne" nil (b ax l ae1 n t ay0 n))
("ballard" nil (b ae1 l er0 d))
("ballas" nil (b ae1 l ax z))
("ballast" nil (b ae1 l ax s t))
("ballasts" nil (b ae1 l ax s t s))
("ballcock" nil (b ao1 l k aa1 k))
("ballcocks" nil (b ao1 l k aa1 k s))
("balle" nil (b ey1 l))
("balled" nil (b ao1 l d))
("ballengee" nil (b ax l eh1 ng g iy0))
("ballenger" nil (b ae1 l ih0 n jh er0))
("ballentine" nil (b aa0 l eh0 n t iy1 n iy0))
("baller" nil (b ao1 l er0))
("ballerina" nil (b ae1 l er0 iy1 n ax))
("ballerinas" nil (b ae1 l er0 iy1 n ax z))
("balles" nil (b ey1 l z))
("ballester" nil (b ae1 l ih0 s t er0))
("ballesteros" nil (b ae1 l ih0 s t er0 ow0 z))
("ballet" nil (b ae0 l ey1))
("ballets" nil (b ae0 l ey1 z))
("balletto" nil (b ae1 l eh1 t ow0))
("ballew" nil (b ae1 l uw0))
("ballgame" nil (b ao0 l g ey1 m))
("balli" nil (b ae1 l iy0))
("balliet" nil (b ae1 l iy0 ih0 t))
("balliett" nil (b ae1 l iy1 t))
("ballin" nil (b ae1 l ih0 n))
("balling" nil (b ao1 l ih0 ng))
("ballinger" nil (b ao1 l ih1 n jh er0))
("ballistic" nil (b ax l ih1 s t ih0 k))
("ballistics" nil (b ax l ih1 s t ih0 k s))
("ballman" nil (b ao1 l m ax n))
("ballmer" nil (b ae1 l m er0))
("ballo" nil (b ae1 l ow0))
("ballon" nil (b ae1 l ax n))
("balloon" nil (b ax l uw1 n))
("ballooned" nil (b ax l uw1 n d))
("ballooning" nil (b ax l uw1 n ih0 ng))
("balloons" nil (b ax l uw1 n z))
("ballor" nil (b ao1 l er0))
("ballot" nil (b ae1 l ax t))
("balloting" nil (b ae1 l ax t ih0 ng))
("ballots" nil (b ae1 l ax t s))
("ballou" nil (b ax l uw1))
("ballow" nil (b ae1 l ow0))
("ballowe" nil (b ae1 l aw0))
("ballpark" nil (b ao1 l p aa1 r k))
("ballparks" nil (b ao1 l p aa1 r k s))
("ballplayer" nil (b ao1 l p l ey1 er0))
("ballplayers" nil (b ao0 l p l ey1 er0 z))
("ballroom" nil (b ao1 l r uw1 m))
("ballrooms" nil (b ao1 l r uw1 m z))
("balls" nil (b ao1 l z))
("ballweg" nil (b ae1 l w ih0 g))
("bally" nil (b ae1 l iy0))
("ballyhoo" nil (b ae1 l iy0 hh uw1))
("ballyhooed" nil (b ae1 l iy0 hh uw1 d))
("balm" nil (b aa1 m))
("balmer" nil (b aa1 m er0))
("balmes" nil (b aa1 l m eh0 s))
("balmoral" nil (b ae0 l m ao1 r ax l))
("balmorals" nil (b ae0 l m ao1 r ax l z))
("balms" nil (b aa1 m z))
("balmy" nil (b aa1 m iy0))
("balog" nil (b ae1 l ao0 g))
("baloga" nil (b aa0 l ow1 g ax))
("balogh" nil (b ae1 l ow0))
("balon" nil (b aa0 l ao1 n))
("baloney" nil (b ax l ow1 n iy0))
("balow" nil (b ae1 l ow0))
("balsa" nil (b ao1 l s ax))
("balsam" nil (b ao1 l s ax m))
("balsamo" nil (b aa0 l s aa1 m ow0))
("balsbaugh" nil (b ao1 l z b ao1))
("balser" nil (b ey1 l s er0))
("balsiger" nil (b ae1 l s ih0 g er0))
("balsley" nil (b ae1 l s l iy0))
("balster" nil (b ae1 l s t er0))
("baltazar" nil (b aa0 l t aa0 z aa1 r))
("balter" nil (b ao1 l t er0))
("baltes" nil (b ey1 l t s))
("balthasar" nil (b ae1 l th ax s er0))
("balthaser" nil (b ae1 l th ax s er0))
("balthazar" nil (b ae1 l th ax z er0))
("balthazor" nil (b aa0 l th aa0 z ao1 r))
("balthrop" nil (b ae1 l th r ax p))
("baltic" nil (b ao1 l t ih0 k))
("baltica" nil (b ae1 l t ih0 k ax))
("baltics" nil (b ao1 l t ih0 k s))
("baltierra" nil (b aa0 l t ih1 r ax))
("baltimore" nil (b ao1 l t ax m ao1 r))
("baltodano" nil (b ao1 l t ow0 d aa1 n ow0))
("baltsa" nil (b ao1 l t s ax))
("baltz" nil (b ae1 l t s))
("baltzell" nil (b ae1 l t z ax l))
("baltzer" nil (b ae1 l t z er0))
("baluchi" nil (b ax l uw1 ch iy0))
("baluja" nil (b ax l uw1 jh ax))
("balukas" nil (b ax l uw1 k ax z))
("balyeat" nil (b ae1 l iy0 ae1 t))
("balz" nil (b ao1 l z))
("balzano" nil (b aa0 l z aa1 n ow0))
("balzarini" nil (b aa0 l z aa0 r iy1 n iy0))
("balzer" nil (b ey1 l z er0))
("bam" nil (b ae1 m))
("bamba" nil (b ae1 m b ax))
("bambach" nil (b ae1 m b aa1 k))
("bambenek" nil (b ae0 m b eh1 n eh0 k))
("bamber" nil (b ae1 m b er0))
("bamberg" nil (b ae1 m b er0 g))
("bamberger" nil (b ae1 m b er0 g er0))
("bambi" nil (b ae1 m b iy0))
("bambino" nil (b ae1 m b iy1 n ow0))
("bambinos" nil (b ae1 m b iy1 n ow0 z))
("bamboo" nil (b ae0 m b uw1))
("bambrick" nil (b ae1 m b r ih0 k))
("bamburg" nil (b ae1 m b er0 g))
("bame" nil (b ey1 m))
("bamford" nil (b ae1 m f er0 d))
("ban" nil (b ae1 n))
("banacci" nil (b ax n aa1 ch iy0))
("banach" nil (b ae1 n ax k))
("banal" nil (b ax n aa1 l))
("banalities" nil (b ax n ae1 l ih0 t iy0 z))
("banality" nil (b ax n ae1 l ih0 t iy0))
("banamex" nil (b ae1 n ax m eh1 k s))
("banana" nil (b ax n ae1 n ax))
("bananas" nil (b ax n ae1 n ax z))
("banas" nil (b ae1 n ax z))
("banasiak" nil (b ax n aa1 s iy0 ae0 k))
("banaszak" nil (b ax n aa1 sh ax k))
("banbury" nil (b ae1 n b eh1 r iy0))
("banc" nil (b ae1 ng k))
("banca" nil (b ae1 ng k ax))
("bancaire" nil (b ae0 n k eh1 r))
("bancario" nil (b ae0 n k eh1 r iy0 ow0))
("banco" nil (b ae1 ng k ow0))
("bancoklahoma" nil (b ae0 ng k aa1 k l ax hh ow1 m ax))
("bancomer" nil (b ae1 ng k ax m er0))
("bancor" nil (b ae1 n k ao1 r))
("bancorp" nil (b ae1 ng k ao0 r p))
("bancorporation" nil (b ae1 n k ao1 r p er0 ey0 sh ax n))
("bancroft" nil (b ae1 ng k r ao0 f t))
("bancserve" nil (b ae1 n k s er0 v))
("bancshares" nil (b ae1 ng k sh eh0 r z))
("banctec" nil (b ae1 ng k t eh1 k))
("banctexas" nil (b ae0 ng k t eh1 k s ax s))
("band" nil (b ae1 n d))
("banda" nil (b ae1 n d ax))
("bandag" nil (b ae1 n d ae1 g))
("bandage" nil (b ae1 n d ih0 jh))
("bandaged" nil (b ae1 n d ax jh d))
("bandages" nil (b ae1 n d ax jh ax z))
("bandai" nil (b ae1 n d ay1))
("bandar" nil (b ae1 n d aa0 r))
("banded" nil (b ae1 n d ih0 d))
("bandel" nil (b ae1 n d ax l))
("bandemer" nil (b ae1 n d iy0 m er0))
("bander" nil (b ae1 n d er0))
("bandicoots" nil (b ae1 n d ih0 k uw1 t s))
("bandied" nil (b ae1 n d iy0 d))
("banding" nil (b ae1 n d ih0 ng))
("bandit" nil (b ae1 n d ax t))
("banditry" nil (b ae1 n d ax t r iy0))
("bandits" nil (b ae1 n d ax t s))
("bandolier" nil (b ae1 n d ax l ih1 r))
("bandoliers" nil (b ae1 n d ax l ih1 r z))
("bandow" nil (b ae1 n d aw1))
("bands" nil (b ae1 n d z))
("bandshell" nil (b ae0 n d sh eh1 l))
("bandstand" nil (b ae1 n d s t ae1 n d))
("bandt" nil (b ae1 n t))
("banducci" nil (b aa0 n d uw1 ch iy0))
("bandwagon" nil (b ae1 n d w ae1 g ax n))
("bandwidth" nil (b ae1 n d w ih0 d th))
("bandy" nil (b ae1 n d iy0))
("bane" nil (b ey1 n))
("banegas" nil (b ae1 n ih0 g ax z))
("banerjee" nil (b ax n er1 jh iy0))
("banes" nil (b ey1 n z))
("banesto" nil (b ax n eh1 s t ow0))
("banet" nil (b ae1 n ih0 t))
("baney" nil (b ey1 n iy0))
("banff" nil (b ae1 n f))
("banfield" nil (b ae1 n f iy1 l d))
("banford" nil (b ae1 n f er0 d))
("bang" nil (b ae1 ng))
("bangalore" nil (b ae1 ng g ax l ao1 r))
("bange" nil (b ae1 n jh))
("banged" nil (b ae1 ng d))
("bangemann" nil (b ae1 ng g ax m ax n))
("bangert" nil (b ey1 ng g er0 t))
("bangerter" nil (b ey1 ng g er0 t er0))
("banghart" nil (b ae1 ng hh aa1 r t))
("banging" nil (b ae1 ng ih0 ng))
("bangish" nil (b ae1 ng ih0 sh))
("bangkok" nil (b ae0 ng k aa1 k))
("bangladesh" nil (b ae1 ng l ax d eh1 sh))
("bangladeshi" nil (b ae1 ng l ax d eh1 sh iy0))
("bangladeshis" nil (b ae1 ng l ax d eh1 sh iy0 z))
("bangle" nil (b ae1 ng g ax l))
("bango" nil (b aa1 ng g ow0))
("bangor" nil (b ae1 ng g er0))
("bangs" nil (b ae1 ng z))
("banh" nil (b ae1 n))
("bania" nil (b aa1 n iy0 ax))
("banick" nil (b ae1 n ih0 k))
("banik" nil (b ae1 n ih0 k))
("banis" nil (b ae1 n ih0 s))
("banish" nil (b ae1 n ih0 sh))
("banished" nil (b ae1 n ih0 sh t))
("banishing" nil (b ae1 n ih0 sh ih0 ng))
("banishment" nil (b ae1 n ih0 sh m ax n t))
("banister" nil (b ae1 n ih0 s t er0))
("banja" nil (b aa1 n y ax))
("banjo" nil (b ae1 n jh ow1))
("bank" nil (b ae1 ng k))
("bankable" nil (b ae1 ng k ax b ax l))
("bankamerica" nil (b ae1 ng k ax m eh1 r ih0 k ax))
("bankatlantic" nil (b ae1 ng k ax t l ae1 n t ih0 k))
("bankcard" nil (b ae1 ng k aa1 r d))
("bankcorp" nil (b ae1 ng k ao1 r p))
("banke" nil (b ae1 ng k))
("bankeast" nil (b ae1 ng k iy1 s t))
("banked" nil (b ae1 ng k t))
("banken" nil (b ae1 ng k ax n))
("banker" nil (b ae1 ng k er0))
("bankers" nil (b ae1 ng k er0 z))
("bankert" nil (b ae1 ng k er0 t))
("bankes" nil (b ae1 ng k s))
("bankey" nil (b ae1 n k iy1))
("bankhead" nil (b ae1 ng k hh eh1 d))
("bankholding" nil (b ae1 ng k hh ow1 l d ih0 ng))
("banking" nil (b ae1 ng k ih0 ng))
("banknote" nil (b ae1 ng k n ow1 t))
("banknotes" nil (b ae1 ng k n ow1 t s))
("banko" nil (b ae1 ng k ow0))
("bankowski" nil (b ax ng k ao1 f s k iy0))
("bankroll" nil (b ae1 ng k r ow1 l))
("bankrolled" nil (b ae1 ng k r ow1 l d))
("bankrolling" nil (b ae1 ng k r ow1 l ih0 ng))
("bankrolls" nil (b ae1 ng k r ow1 l z))
("bankrupcty" nil (b ae1 ng k r ax p t s iy0))
("bankrupt" nil (b ae1 ng k r ax p t))
("bankruptcies" nil (b ae1 ng k r ax p t s iy0 z))
("bankruptcy" nil (b ae1 ng k r ax p s iy0))
("bankrupted" nil (b ae1 ng k r ax p t ih0 d))
("bankrupting" nil (b ae1 ng k r ah1 p t ih0 ng))
("bankrutpcy" nil (b ae1 ng k r ax t p s iy0))
("banks" nil (b ae1 ng k s))
("bankshares" nil (b ae1 ng k sh eh1 r z))
("bankson" nil (b ae1 ng k s ax n))
("bankston" nil (b ae1 ng k s t ax n))
("bankverein" nil (b ae1 ng k v er0 ay1 n))
("bankvermont" nil (b ae1 ng k v er0 m aa1 n t))
("bankworcester" nil (b ae1 ng k w er1 ch eh1 s t er0))
("bann" nil (b ae1 n))
("bannan" nil (b ae1 n ax n))
("banned" nil (b ae1 n d))
("banner" nil (b ae1 n er0))
("bannerman" nil (b ae1 n er0 m ax n))
("banners" nil (b ae1 n er0 z))
("bannick" nil (b ae1 n ih0 k))
("banning" nil (b ae1 n ih0 ng))
("bannings" nil (b ae1 n ih0 ng z))
("bannister" nil (b ae1 n ax s t er0))
("bannon" nil (b ae1 n ax n))
("banos" nil (b aa1 n ow0 z))
("banoun" nil (b ax n uw1 n))
("banpais" nil (b ae1 n p ey1))
("banponce" nil (b ae1 n p aa0 n s))
("banque" nil (b ae1 ng k))
("banquet" nil (b ae1 ng k w ax t))
("banquets" nil (b ae1 ng k w ax t s))
("bans" nil (b ae1 n z))
("banshee" nil (b ae0 n sh iy1))
("banta" nil (b ae1 n t ax))
("bantam" nil (b ae1 n t ax m))
("bantams" nil (b ae1 n t ax m z))
("banter" nil (b ae1 n t er0))
("bantle" nil (b ae1 n t ax l))
("banton" nil (b ae1 n t ax n))
("bantu" nil (b ae1 n t uw0))
("bantz" nil (b ae1 n t s))
("banuelos" nil (b aa0 n w eh1 l ow0 z))
("banville" nil (b aa1 n v ih0 l))
("banwart" nil (b ae1 n w ao1 r t))
("banxquote" nil (b ae0 ng k s k w ow1 t))
("banya" nil (b aa1 n y ax))
("banyan" nil (b ae1 n y ax n))
("banyas" nil (b aa1 n y ax z))
("banyu" nil (b aa1 n uw0))
("banzhaf" nil (b ae1 n z hh ax f))
("bao" nil (b aw1))
("baoguang" nil (b ey1 ow0 g w ae1 ng))
("bapna" nil (b aa1 p n ax))
("baptism" nil (b ae1 p t ih0 z ax m))
("baptismal" nil (b ae0 p t ih1 z m ax l))
("baptisms" nil (b ae1 p t ih1 z ax m z))
("baptist" nil (b ae1 p t ax s t))
("baptista" nil (b ae1 p t ih1 s t ax))
("baptiste" nil (b ax p t ih1 s t))
("baptistery" nil (b ae1 p t ax s t r iy0))
("baptists" nil (b ae1 p t ax s t s))
("baptized" nil (b ae0 p t ay1 z d))
("bar" nil (b aa1 r))
("bara" nil (b aa1 r ax))
("barabar" nil (b eh1 r ax b aa0 r))
("barach" nil (b ax r aa1 k))
("baraff" nil (b aa0 r aa1 f))
("barahona" nil (b ae1 r ax hh ow1 n ax))
("barajas" nil (b aa0 r aa1 y aa0 z))
("barak" nil (b aa1 r ax k))
("barakat" nil (b aa1 r ax k aa1 t))
("baram" nil (b eh1 r ae0 m))
("baran" nil (b aa0 r aa1 n))
("baranek" nil (b ae1 r ax n ih0 k))
("baranoski" nil (b er0 ax n aw1 s k iy0))
("baranowski" nil (b er0 ax n ao1 f s k iy0))
("baranski" nil (b er0 ae1 n s k iy0))
("barany" nil (b er0 ao1 n iy0))
("barasch" nil (b ae1 r ax sh))
("barash" nil (b ae1 r ax sh))
("barath" nil (b ae1 r ax th))
("baratta" nil (b aa0 r aa1 t ax))
("barb" nil (b aa1 r b))
("barba" nil (b aa1 r b ax))
("barbados" nil (b aa0 r b ey1 d ow0 s))
("barbagallo" nil (b aa1 r b ax g ae1 l ow0))
("barbakow" nil (b aa1 r b ax k aw1))
("barbanel" nil (b aa1 r b ax n ax l))
("barbano" nil (b aa0 r b aa1 n ow0))
("barbara" nil (b aa1 r b er0 ax))
("barbaree" nil (b aa1 r b er0 iy1))
("barbarian" nil (b aa0 r b eh1 r iy0 ax n))
("barbarians" nil (b aa0 r b eh1 r iy0 ax n z))
("barbaric" nil (b aa0 r b ae1 r ih0 k))
("barbarino" nil (b aa0 r b aa0 r iy1 n ow0))
("barbarism" nil (b aa1 r b er0 ih1 z ax m))
("barbaro" nil (b aa0 r b aa1 r ow0))
("barbarous" nil (b aa1 r b er0 ax s))
("barbary" nil (b aa1 r b er0 iy0))
("barbash" nil (b aa1 r b ae1 sh))
("barbato" nil (b aa0 r b aa1 t ow0))
("barbe" nil (b aa1 r b))
("barbeau" nil (b aa0 r b ow1))
("barbecue" nil (b aa1 r b ih0 k y uw1))
("barbecued" nil (b aa1 r b ih0 k y uw1 d))
("barbecueing" nil (b aa1 r b ih0 k y uw1 ih0 ng))
("barbecues" nil (b aa1 r b ih0 k y uw1 z))
("barbed" nil (b aa1 r b d))
("barbee" nil (b aa1 r b iy1))
("barbella" nil (b aa1 r b eh1 l ax))
("barbells" nil (b aa1 r b eh1 l z))
("barbels" nil (b aa1 r b ax l z))
("barbeque" nil (b aa1 r b ih0 k y uw1))
("barbequed" nil (b aa1 r b ih0 k y uw1 d))
("barbequeing" nil (b aa1 r b ih0 k y uw1 ih0 ng))
("barbeques" nil (b aa1 r b ih0 k y uw1 z))
("barber" nil (b aa1 r b er0))
("barbera" nil (b aa0 r b eh1 r ax))
("barberi" nil (b aa0 r b eh1 r iy0))
("barberio" nil (b aa1 r b iy1 r iy0 ow0))
("barberis" nil (b aa1 r b er0 ih0 s))
("barbero" nil (b aa0 r b eh1 r ow0))
("barbers" nil (b aa1 r b er0 z))
("barbershop" nil (b aa1 r b er0 sh aa1 p))
("barbette" nil (b aa0 r b eh1 t))
("barbian" nil (b aa1 r b iy0 ax n))
("barbie" nil (b aa1 r b iy0))
("barbier" nil (b aa1 r b iy0 er0))
("barbieri" nil (b aa0 r b ih1 r iy0))
("barbin" nil (b aa1 r b ih0 n))
("barbiturates" nil (b aa0 r b ih1 ch er0 ax t s))
("barbo" nil (b aa1 r b ow0))
("barbone" nil (b aa1 r b ow1 n))
("barbosa" nil (b aa0 r b ow1 s ax))
("barbour" nil (b aa1 r b er0))
("barboza" nil (b aa0 r b ow1 z ax))
("barbra" nil (b aa1 r b r ax))
("barbre" nil (b aa1 r b er0))
("barbs" nil (b aa1 r b z))
("barby" nil (b aa1 r b iy0))
("barca" nil (b aa1 r k ax))
("barcella" nil (b aa1 r s eh1 l ax))
("barcellos" nil (b aa0 r s eh1 l ow0 z))
("barcelo" nil (b aa0 r ch eh1 l ow0))
("barcelona" nil (b aa1 r s ih0 l ow1 n ax))
("barcenas" nil (b aa1 r s ih0 n ax z))
("barch" nil (b aa1 r k))
("barcia" nil (b aa1 r ch ax))
("barclay" nil (b aa1 r k l ey1))
("barclays" nil (b aa1 r k l iy0 z))
("barclift" nil (b aa1 r k l ih0 f t))
("barco" nil (b aa1 r k ow0))
("barcomb" nil (b aa1 r k ax m))
("barcroft" nil (b aa1 r k r ao1 f t))
("barcus" nil (b aa1 r k ax s))
("barczak" nil (b aa1 r ch ae0 k))
("bard" nil (b aa1 r d))
("bardell" nil (b aa0 r d eh1 l))
("barden" nil (b aa1 r d ax n))
("bardin" nil (b aa1 r d ih0 n))
("bardo" nil (b aa1 r d ow0))
("bardolf" nil (b aa1 r d ow1 l f))
("bardolph" nil (b aa1 r d aa0 l f))
("bardon" nil (b aa0 r d ao1 n))
("bardrick" nil (b aa1 r d r ih0 k))
("bardsley" nil (b aa1 r d s l iy0))
("bardulf" nil (b aa1 r d ax l f))
("bardulph" nil (b aa1 r d ax l f))
("bardwell" nil (b aa1 r d w eh1 l))
("bare" nil (b eh1 r))
("bared" nil (b eh1 r d))
("barefield" nil (b ae1 r ih0 f iy0 l d))
("barefoot" nil (b eh1 r f uh1 t))
("bareis" nil (b ae1 r ay0 z))
("barela" nil (b aa0 r eh1 l ax))
("barely" nil (b eh1 r l iy0))
("barenboim" nil (b eh1 r ax n b oy1 m))
("barens" nil (b eh1 r ax n z))
("barentine" nil (b aa0 r eh0 n t iy1 n iy0))
("barents" nil (b eh1 r ax n t s))
("bares" nil (b eh1 r z))
("barfield" nil (b aa1 r f iy1 l d))
("barfknecht" nil (b aa1 r f k ax n eh0 k t))
("barfoot" nil (b aa1 r f uh1 t))
("barfuss" nil (b aa1 r f ah1 s))
("barg" nil (b aa1 r g))
("barga" nil (b aa1 r g ax))
("bargain" nil (b aa1 r g ax n))
("bargained" nil (b aa1 r g ax n d))
("bargainer" nil (b aa1 r g ih0 n er0))
("bargainers" nil (b aa1 r g ih0 n er0 z))
("bargaining" nil (b aa1 r g ih0 n ih0 ng))
("bargains" nil (b aa1 r g ax n z))
("barganier" nil (b aa1 r g ax n iy0 er0))
("bargar" nil (b aa0 r g aa1 r))
("bargas" nil (b aa1 r g ax z))
("barge" nil (b aa1 r jh))
("barged" nil (b aa1 r jh d))
("barger" nil (b aa1 r jh er0))
("bargeron" nil (b aa1 r g er0 ax n))
("barges" nil (b aa1 r jh ax z))
("bargman" nil (b aa1 r g m ax n))
("bargmann" nil (b aa1 r g m ax n))
("bargo" nil (b aa1 r g ow1))
("barhorst" nil (b aa1 r hh ao0 r s t))
("bari" nil (b aa1 r iy0))
("baribeau" nil (b ae1 r ih0 b ow0))
("barich" nil (b ae1 r ih0 k))
("baril" nil (b eh1 r ax l))
("barile" nil (b aa1 r ax l))
("barillari" nil (b eh1 r ih0 l eh1 r iy0))
("barillo" nil (b er0 ih1 l ow0))
("barinco" nil (b er0 ih1 ng k ow1))
("baring" nil (b eh1 r ih0 ng))
("baringer" nil (b eh1 r ih0 ng er0))
("barings" nil (b eh1 r ih0 ng z))
("baris" nil (b aa1 r iy0 z))
("barish" nil (b eh1 r ih0 sh))
("barite" nil (b eh1 r ay0 t))
("barites" nil (b eh1 r ay0 t s))
("baritone" nil (b eh1 r ax t ow1 n))
("barium" nil (b eh1 r iy0 ax m))
("bark" nil (b aa1 r k))
("barkai" nil (b aa0 r k ay1))
("barkalow" nil (b aa1 r k ax l ow1))
("barkan" nil (b aa1 r k ax n))
("barkdoll" nil (b aa1 r k d ax l))
("barkdull" nil (b aa1 r k d ax l))
("barke" nil (b aa1 r k))
("barked" nil (b aa1 r k t))
("barkeley" nil (b aa1 r k l iy0))
("barker" nil (b aa1 r k er0))
("barkers" nil (b aa1 r k er0 z))
("barkes" nil (b aa1 r k s))
("barkett" nil (b aa1 r k ih0 t))
("barkey" nil (b aa1 r k iy1))
("barkhurst" nil (b aa1 r k hh er0 s t))
("barkin" nil (b aa1 r k ih0 n))
("barking" nil (b aa1 r k ih0 ng))
("barkley" nil (b aa1 r k l iy0))
("barklow" nil (b aa1 r k l ow1))
("barkman" nil (b aa1 r k m ax n))
("barko" nil (b aa1 r k ow0))
("barkocy" nil (b aa1 r k ax s iy0))
("barkow" nil (b aa1 r k ow0))
("barks" nil (b aa1 r k s))
("barksdale" nil (b aa1 r k s d ey1 l))
("barkshire" nil (b aa1 r k sh ay1 r))
("barkus" nil (b aa1 r k ax s))
("barlage" nil (b aa1 r l ih0 jh))
("barlett" nil (b aa1 r l ih0 t))
("barletta" nil (b aa0 r l eh1 t ax))
("barlettesville" nil (b aa1 r l ax t s v ih1 l))
("barley" nil (b aa1 r l iy0))
("barlow" nil (b aa1 r l ow1))
("barlowe" nil (b aa1 r l ow1))
("barman" nil (b aa1 r m ax n))
("barmore" nil (b aa1 r m ao0 r))
("barn" nil (b aa1 r n))
("barna" nil (b aa1 r n ax))
("barnabas" nil (b aa1 r n ax b ax s))
("barnaby" nil (b aa1 r n ax b iy0))
("barnacle" nil (b aa1 r n ax k ax l))
("barnacles" nil (b aa1 r n ax k ax l z))
("barnard" nil (b aa1 r n er0 r d))
("barnards" nil (b aa1 r n er0 d z))
("barnas" nil (b aa1 r n ax z))
("barndt" nil (b aa1 r n t))
("barnell" nil (b aa1 r n ax l))
("barner" nil (b aa1 r n er0))
("barnes" nil (b aa1 r n z))
("barnet" nil (b aa1 r n ih0 t))
("barnett" nil (b aa0 r n eh1 t))
("barnette" nil (b aa1 r n eh1 t))
("barnevik" nil (b aa0 r n eh1 v ih0 k))
("barney" nil (b aa1 r n iy0))
("barneys" nil (b aa1 r n iy0 z))
("barnfield" nil (b aa1 r n f iy1 l d))
("barnhard" nil (b aa1 r n hh aa1 r d))
("barnhardt" nil (b aa1 r n hh aa1 r t))
("barnhart" nil (b aa1 r n hh aa1 r t))
("barnhill" nil (b aa1 r n hh ih1 l))
("barnhouse" nil (b aa1 r n hh aw1 s))
("barnick" nil (b aa1 r n ih0 k))
("barnicle" nil (b aa1 r n ih0 k ax l))
("barnish" nil (b aa1 r n ih0 sh))
("barno" nil (b aa1 r n ow0))
("barnowski" nil (b aa0 r n aw1 s k iy0))
("barns" nil (b aa1 r n z))
("barnstorming" nil (b aa1 r n s t ao1 r m ih0 ng))
("barnum" nil (b aa1 r n ax m))
("barnwell" nil (b aa1 r n w eh1 l))
("barny" nil (b aa1 r n iy0))
("barnyard" nil (b aa1 r n y aa1 r d))
("baro" nil (b aa1 r ow0))
("baroid" nil (b er0 oy1 d))
("barometer" nil (b er0 aa1 m ih0 t er0))
("barometers" nil (b er0 aa1 m ih0 t er0 z))
("barometric" nil (b ae1 r ax m eh1 t r ih0 k))
("baron" nil (b ae1 r ax n))
("barone" nil (b er0 ow1 n))
("baronet" nil (b eh1 r ax n ax t))
("baronets" nil (b eh1 r ax n ax t s))
("baroni" nil (b aa0 r ow1 n iy0))
("barons" nil (b ae1 r ax n z))
("baroody" nil (b er0 uw1 d iy0))
("baroque" nil (b er0 ow1 k))
("baros" nil (b aa1 r ow0 z))
("barovic" nil (b eh1 r ax v ih0 k))
("barovsky" nil (b er0 aa1 v s k iy0))
("barr" nil (b aa1 r))
("barra" nil (b aa1 r ax))
("barrack" nil (b ae1 r ax k))
("barracks" nil (b ae1 r ax k s))
("barraclough" nil (b ae1 r ax k l aw0))
("barraco" nil (b aa0 r aa1 k ow0))
("barracuda" nil (b eh1 r ax k uw1 d ax))
("barragan" nil (b eh1 r ax g ax n))
("barrage" nil (b er0 aa1 zh))
("barraged" nil (b er0 aa1 zh d))
("barrages" nil (b er0 aa1 zh ih0 z))
("barranco" nil (b aa0 r aa1 n k ow0))
("barras" nil (b ae1 r ax z))
("barrasso" nil (b aa1 r aa1 s ow0))
("barratt" nil (b ae1 r ax t))
("barraza" nil (b aa1 r aa1 z ax))
("barre" nil (b eh1 r iy0))
("barreca" nil (b aa1 r eh1 k ax))
("barred" nil (b aa1 r d))
("barreda" nil (b aa0 r ey1 d ax))
("barreiro" nil (b aa0 r eh1 r ow0))
("barrel" nil (b ae1 r ax l))
("barreled" nil (b ae1 r ax l d))
("barreling" nil (b ae1 r ax l ih0 ng))
("barrell" nil (b aa0 r ey1 l))
("barrels" nil (b ae1 r ax l z))
("barren" nil (b ae1 r ax n))
("barrens" nil (b ae1 r ax n z))
("barrentine" nil (b ae1 r ax n t ay1 n))
("barrera" nil (b aa1 r eh1 r ax))
("barreras" nil (b aa0 r eh1 r aa0 z))
("barres" nil (b aa1 r z))
("barresi" nil (b aa1 r eh1 s iy0))
("barret" nil (b ae1 r ih0 t))
("barreto" nil (b aa0 r eh1 t ow0))
("barrett" nil (b ae1 r ih0 t))
("barretta" nil (b aa0 r eh1 t ax))
("barrette" nil (b er0 eh1 t))
("barretto" nil (b aa0 r eh1 t ow0))
("barricada" nil (b aa1 r ih0 k aa1 d ax))
("barricade" nil (b ae1 r ax k ey1 d))
("barricaded" nil (b ae1 r ax k ey1 d ih0 d))
("barricades" nil (b ae1 r ax k ey1 d z))
("barrick" nil (b ae1 r ih0 k))
("barricklow" nil (b ae1 r ih0 k l aw0))
("barrickman" nil (b ae1 r ih0 k m ax n))
("barrie" nil (b ae1 r iy0))
("barrientes" nil (b er0 iy1 n t s))
("barrientez" nil (b aa0 r iy1 n t eh0 z))
("barrientos" nil (b aa0 r iy1 n t ow0 z))
("barrier" nil (b ae1 r iy0 er0))
("barriere" nil (b aa0 r ih1 r iy0))
("barriers" nil (b ae1 r iy0 er0 z))
("barriga" nil (b aa1 r iy1 g ax))
("barriger" nil (b ae1 r ih0 g er0))
("barrilleaux" nil (b ae1 r ih0 l ow1))
("barrineau" nil (b ae1 r ih0 n ow1))
("barring" nil (b aa1 r ih0 ng))
("barringer" nil (b ae1 r ih0 ng er0))
("barrington" nil (b ae1 r ih0 ng t ax n))
("barrio" nil (b aa1 r iy0 ow0))
("barrios" nil (b aa0 r iy1 ow0 z))
("barris" nil (b ae1 r ih0 s))
("barrish" nil (b ae1 r ih0 sh))
("barrister" nil (b ae1 r ih0 s t er0))
("barristers" nil (b ae1 r ih0 s t er0 z))
("barritt" nil (b er0 ih1 t))
("barro" nil (b aa1 r ow0))
("barron" nil (b ae1 r ax n))
("barrone" nil (b er0 ow1 n))
("barroom" nil (b aa1 r r uw1 m))
("barros" nil (b aa1 r ow0 z))
("barroso" nil (b aa1 r ow1 s ow0))
("barrow" nil (b ae1 r ow0))
("barrowman" nil (b ae1 r ow0 m ax n))
("barrows" nil (b ae1 r ow0 z))
("barrs" nil (b aa1 r z))
("barrus" nil (b ae1 r ax s))
("barry" nil (b ae1 r iy0))
("barrymore" nil (b ae1 r iy0 m ao1 r))
("bars" nil (b aa1 r z))
("barsamian" nil (b aa1 r s ey1 m iy0 ax n))
("barsanti" nil (b aa0 r s aa1 n t iy0))
("barsch" nil (b aa1 r sh))
("barschel" nil (b aa1 r sh ax l))
("barsh" nil (b aa1 r sh))
("barshefsky" nil (b aa1 r sh eh1 f s k iy0))
("barsky" nil (b aa1 r s k iy0))
("barsness" nil (b aa1 r z n ax s))
("barson" nil (b aa1 r s ax n))
("barsotti" nil (b aa0 r s ow1 t iy0))
("barstad" nil (b aa1 r s t ax d))
("barstow" nil (b aa1 r s t ow1))
("barszcz" nil (b aa1 r sh ch))
("bart" nil (b aa1 r t))
("barta" nil (b aa1 r t ax))
("bartak" nil (b aa1 r t ax k))
("bartee" nil (b aa1 r t iy1))
("bartek" nil (b aa1 r t eh1 k))
("bartel" nil (b aa1 r t ax l))
("bartell" nil (b aa0 r t ey1 l))
("bartels" nil (b aa1 r t ax l z))
("bartelson" nil (b aa1 r t ih0 l s ax n))
("bartelt" nil (b aa1 r t ih0 l t))
("barten" nil (b aa1 r t ax n))
("bartender" nil (b aa1 r t eh1 n d er0))
("bartenders" nil (b aa1 r t eh1 n d er0 z))
("barter" nil (b aa1 r t er0))
("bartered" nil (b aa1 r t er0 d))
("bartering" nil (b aa1 r t er0 ih0 ng))
("barth" nil (b aa1 r th))
("bartha" nil (b aa1 r th ax))
("barthel" nil (b aa1 r th ax l))
("barthelemy" nil (b aa1 r th eh1 l ax m iy0))
("barthelme" nil (b aa1 r th ax l m))
("barthes" nil (b aa1 r th s))
("barthold" nil (b aa1 r th ow1 l d))
("bartholomay" nil (b aa0 r th ax l aa1 m ey0))
("bartholomew" nil (b aa0 r th aa1 l ax m y uw1))
("bartholow" nil (b aa1 r th ax l ow0))
("barthram" nil (b aa1 r th r ae1 m))
("bartko" nil (b aa1 r t k ow0))
("bartkowiak" nil (b er0 t k aw1 iy0 ae0 k))
("bartkowski" nil (b er0 t k ao1 f s k iy0))
("bartkus" nil (b aa1 r t k ih0 s))
("bartl" nil (b aa1 r t ax l))
("bartle" nil (b aa1 r t ax l))
("bartlebaugh" nil (b aa1 r t l ih0 b ao0))
("bartles" nil (b aa1 r t ax l z))
("bartleson" nil (b aa1 r t l ih0 s ax n))
("bartlesville" nil (b aa1 r t ax l z v ih1 l))
("bartlett" nil (b aa1 r t l ih0 t))
("bartley" nil (b aa1 r t l iy0))
("bartling" nil (b aa1 r t l ih0 ng))
("bartlow" nil (b aa1 r t l ow0))
("bartman" nil (b aa1 r t m ax n))
("bartmess" nil (b aa1 r t m ih0 s))
("bartnick" nil (b aa1 r t n ih0 k))
("bartnicki" nil (b er0 t n ih1 t s k iy0))
("bartnik" nil (b aa1 r t n ih0 k))
("barto" nil (b aa1 r t ow0))
("bartok" nil (b aa1 r t ax k))
("bartol" nil (b aa1 r t ax l))
("bartoletti" nil (b aa0 r t ow0 l eh1 t iy0))
("bartoli" nil (b aa0 r t ow1 l iy0))
("bartolini" nil (b aa0 r t ow0 l iy1 n iy0))
("bartolo" nil (b aa0 r t ow1 l ow0))
("bartolomei" nil (b aa0 r t ow0 l ow0 m ey1 iy0))
("bartolomeo" nil (b aa0 r t ao0 l ax m ey1 ow0))
("bartolucci" nil (b aa0 r t ow0 l uw1 ch iy0))
("barton" nil (b aa1 r t ax n))
("bartone" nil (b aa0 r t ow1 n iy0))
("bartoo" nil (b aa1 r ch uw0))
("bartos" nil (b aa1 r t ow0 z))
("bartosch" nil (b aa1 r t ao0 sh))
("bartosh" nil (b aa1 r t ax sh))
("bartosiewicz" nil (b er0 t aa1 s ax v ih0 ch))
("bartosik" nil (b er0 t ow1 s ih0 k))
("bartoszek" nil (b er0 t aa1 sh eh0 k))
("bartow" nil (b aa1 r t ow1))
("bartram" nil (b aa1 r t r ae1 m))
("barts" nil (b aa1 r t s))
("bartsch" nil (b aa1 r ch))
("bartunek" nil (b aa1 r t ax n ih0 k))
("bartus" nil (b aa1 r t ax s))
("bartz" nil (b aa1 r t s))
("baruch" nil (b aa1 r uw1 k))
("barwick" nil (b aa1 r w ih1 k))
("baryshnikov" nil (b er0 ih1 sh n ih0 k aa1 v))
("barz" nil (b aa1 r z))
("basa" nil (b aa1 s ax))
("basal" nil (b ey1 s ax l))
("basaldua" nil (b aa0 s aa0 l d uw1 ax))
("basalt" nil (b ax s ao1 l t))
("basalts" nil (b ax s ao1 l t s))
("basara" nil (b ax s eh1 r ax))
("basch" nil (b ae1 sh))
("basco" nil (b aa1 s k ow0))
("bascom" nil (b ae1 s k ax m))
("basden" nil (b ae1 s d ax n))
("base" nil (b ey1 s))
("baseball" nil (b ey1 s b ao1 l))
("baseballs" nil (b ey1 s b ao1 l z))
("baseboard" nil (b ey1 s b ao1 r d))
("baseboards" nil (b ey1 s b ao1 r d z))
("based" nil (b ey1 s t))
("basehore" nil (b ae1 s ih0 hh ao0 r))
("basel" nil (b aa1 z ax l))
("baseless" nil (b ey1 s l ax s))
("baseline" nil (b ey1 s l ay1 n))
("baseman" nil (b ey1 s m ax n))
("basement" nil (b ey1 s m ax n t))
("basements" nil (b ey1 s m ax n t s))
("basenji" nil (b ax s eh1 n jh iy0))
("bases" nil (b ey1 s ax z))
("basey" nil (b ey1 s iy0))
("basford" nil (b ae1 s f er0 d))
("basgall" nil (b ae1 s g ax l))
("bash" nil (b ae1 sh))
("basham" nil (b ae1 sh ax m))
("bashaw" nil (b ax sh ao1))
("bashed" nil (b ae1 sh t))
("basher" nil (b ae1 sh er0))
("bashers" nil (b ae1 sh er0 z))
("bashes" nil (b ae1 sh ih0 z))
("bashford" nil (b ae1 sh f er0 d))
("bashful" nil (b ae1 sh f ax l))
("bashing" nil (b ae1 sh ih0 ng))
("bashir" nil (b ax sh ih1 r))
("bashor" nil (b ae1 sh er0))
("bashore" nil (b ae1 sh er0))
("basic" nil (b ey1 s ih0 k))
("basically" nil (b ey1 s ih0 k l iy0))
("basich" nil (b aa1 s ih0 hh))
("basics" nil (b ey1 s ih0 k s))
("basie" nil (b ey1 z iy0))
("basil" nil (b ae1 z ax l))
("basile" nil (b aa1 s ax l))
("basilia" nil (b aa0 s iy1 l iy0 ax))
("basilica" nil (b ax s ih1 l ih0 k ax))
("basilio" nil (b aa0 s iy1 l iy0 ow0))
("basim" nil (b aa0 s iy1 m))
("basin" nil (b ey1 s ax n))
("basing" nil (b ey1 s ih0 ng))
("basinger" nil (b ey1 s ih0 ng er0))
("basins" nil (b ey1 s ax n z))
("basinski" nil (b ax s ih1 n s k iy0))
("basis" nil (b ey1 s ax s))
("basista" nil (b ax s ih1 s t ax))
("basix" nil (b ey1 s ih0 k s))
("bask" nil (b ae1 s k))
("basked" nil (b ae1 s k t))
("baskerville" nil (b ae1 s k er0 v ih1 l))
("basket" nil (b ae1 s k ax t))
("basketball" nil (b ae1 s k ax t b ao1 l))
("basketballs" nil (b ae1 s k ax t b ao1 l z))
("basketmaking" nil (b ae1 s k ax t m ey1 k ih0 ng))
("basketry" nil (b ae1 s k ax t r iy0))
("baskets" nil (b ae1 s k ax t s))
("baskett" nil (b ae1 s k ih0 t))
("baskette" nil (b ax s k eh1 t))
("baskin" nil (b ae1 s k ih0 n))
("basking" nil (b ae1 s k ih0 ng))
("baskins" nil (b ae1 s k ih0 n z))
("basle" nil (b ae1 s ax l))
("basler" nil (b ey1 z ax l er0))
("basner" nil (b ae1 s n er0))
("basnett" nil (b ae1 s n ih0 t))
("basnight" nil (b ae1 z n ay1 t))
("bason" nil (b ae1 s ax n))
("basore" nil (b aa0 s ao1 r iy0))
("basque" nil (b ae1 s k))
("basques" nil (b ae1 s k s))
("basquez" nil (b aa0 s k w eh1 z))
("basra" nil (b aa1 s r aa0))
("bass" nil (b ae1 s))
("bassa" nil (b ae1 s ax))
("basse" nil (b ae1 s))
("basses" nil (b ae1 s ih0 z))
("basset" nil (b ae1 s ih0 t))
("bassett" nil (b ae1 s ih0 t))
("bassette" nil (b ax s eh1 t))
("bassetti" nil (b aa0 s eh1 t iy0))
("bassford" nil (b ae1 s f er0 d))
("bassham" nil (b ae1 sh ax m))
("bassi" nil (b ae1 s iy0))
("bassin" nil (b ae1 s ih0 n))
("bassinger" nil (b ae1 s ih0 n jh er0))
("bassir" nil (b ax s ih1 r))
("bassist" nil (b ey1 s ih0 s t))
("bassists" nil (b ey1 s ih0 s t s))
("bassler" nil (b ae1 s l er0))
("bassman" nil (b ae1 s m ax n))
("basso" nil (b ae1 s ow0))
("bassoon" nil (b ax s uw1 n))
("bast" nil (b ae1 s t))
("basta" nil (b ae1 s t ax))
("bastarache" nil (b aa0 s t aa1 r ey0 k))
("bastard" nil (b ae1 s t er0 d))
("bastards" nil (b ae1 s t er0 d z))
("bastedo" nil (b aa0 s t ey1 d ow0))
("basten" nil (b ey1 s t ax n))
("bastian" nil (b ae1 s ch ih0 n))
("bastien" nil (b ae1 s t ax n))
("bastille" nil (b ae1 s t ih0 l))
("bastin" nil (b ae1 s t ih0 n))
("bastion" nil (b ae1 s ch ax n))
("bastions" nil (b ae1 s ch ax n z))
("bastogne" nil (b ae1 s t ow1 n))
("baston" nil (b ae1 s t ax n))
("bastone" nil (b aa1 s t ow0 n))
("bastow" nil (b ae1 s t ow0))
("bastyr" nil (b ey1 s t ih0 r))
("basu" nil (b aa1 s uw0))
("basulto" nil (b ax s ah1 l t ow0))
("basurto" nil (b aa0 s uh1 r t ow0))
("baswell" nil (b ae1 s w eh0 l))
("basye" nil (b ae1 s ay0))
("bat" nil (b ae1 t))
("bataan" nil (b ax t aa1 n))
("batan" nil (b ey1 t ax n))
("batavia" nil (b ax t ey1 v iy0 ax))
("batch" nil (b ae1 ch))
("batchelder" nil (b ae1 ch eh1 l d er0))
("batcheller" nil (b ae1 ch ax l er0))
("batchelor" nil (b ae1 ch ax l er0))
("batches" nil (b ae1 ch ax z))
("batdorf" nil (b ae1 t d ao0 r f))
("bate" nil (b ey1 t))
("bated" nil (b ey1 t ih0 d))
("bateman" nil (b ey1 t m ax n))
("bates" nil (b ey1 t s))
("bateson" nil (b ae1 t ih0 s ax n))
("batesville" nil (b ey1 t s v ih1 l))
("batey" nil (b ey1 t iy0))
("bath" nil (b ae1 th))
("bathe" nil (b ey1 dh))
("bathed" nil (b ey1 dh d))
("bathers" nil (b ae1 dh er0 z))
("bathes" nil (b ey1 dh z))
("bathgate" nil (b ae1 th g ey1 t))
("bathing" nil (b ey1 dh ih0 ng))
("bathke" nil (b ae1 th k))
("batholiths" nil (b ae1 th ax l ih0 th s))
("bathrick" nil (b ae1 th r ih0 k))
("bathrobe" nil (b ae1 th r ow1 b))
("bathrobes" nil (b ae1 th r ow1 b z))
("bathroom" nil (b ae1 th r uw1 m))
("bathrooms" nil (b ae1 th r uw1 m z))
("baths" v (b ae1 th z))
("baths" n (b ae1 th s))
("bathsheba" nil (b ae0 th sh iy1 b ax))
("bathtub" nil (b ae1 th t ax b))
("bathtubs" nil (b ae1 th t ah1 b z))
("bathurst" nil (b ae1 th er0 s t))
("batibot" nil (b ae1 t ih0 b aa0 t))
("batie" nil (b ey1 t iy0))
("batignolles" nil (b ae1 t ih0 n y ow1 l ax s))
("batik" nil (b ax t iy1 k))
("batiks" nil (b ae1 t ih0 k s))
("batista" nil (b ax t ih1 s t ax))
("batley" nil (b ae1 t l iy0))
("batman" nil (b ae1 t m ae1 n))
("batogowski" nil (b ae1 t ow0 g aw1 s k iy0))
("baton" nil (b ax t aa1 n))
("batons" nil (b ae0 t aa1 n z))
("bator" nil (b aa0 t ao1 r))
("batra" nil (b ae1 t r ax))
("batres" nil (b ey1 t er0 z))
("bats" nil (b ae1 t s))
("batson" nil (b ae1 t s ax n))
("batt" nil (b ae1 t))
("battaglia" nil (b aa0 t aa1 g l iy0 ax))
("battaglini" nil (b aa0 t aa0 g l iy1 n iy0))
("battalion" nil (b ax t ae1 l y ax n))
("battalions" nil (b ax t ae1 l y ax n z))
("batte" nil (b ae1 t))
("batted" nil (b ae1 t ih0 d))
("battelle" nil (b ax t eh1 l))
("batten" nil (b ae1 t ax n))
("battenberg" nil (b ae1 t ax n b er1 g))
("battenfield" nil (b ae1 t ax n f iy1 l d))
("battening" nil (b ae1 t ax n ih0 ng))
("batter" nil (b ae1 t er0))
("battered" nil (b ae1 t er0 d))
("batterer" nil (b ae1 t er0 er0))
("batterers" nil (b ae1 t er0 er0 z))
("batteries" nil (b ae1 t er0 iy0 z))
("battering" nil (b ae1 t er0 ih0 ng))
("batterman" nil (b ae1 t er0 m ax n))
("batters" nil (b ae1 t er0 z))
("battersby" nil (b ae1 t er0 s b iy0))
("batterson" nil (b ae1 t er0 s ax n))
("battery" nil (b ae1 t er0 iy0))
("batterymarch" nil (b ax t eh1 r ih0 m aa0 r ch))
("battey" nil (b ae1 t iy0))
("battiato" nil (b aa0 t iy0 aa1 t ow0))
("batticaloa" nil (b ae1 t ih0 k ax l ow1 ax))
("battin" nil (b ae1 t ih0 n))
("batting" nil (b ae1 t ih0 ng))
("battipaglia" nil (b ae1 t ih0 p ae1 g l iy0 ax))
("battis" nil (b ae1 t ih0 s))
("battison" nil (b ae1 t ih0 s ax n))
("battista" nil (b ax t iy1 s t ax))
("battiste" nil (b aa0 t iy1 s t iy0))
("battisti" nil (b aa0 t iy1 s t iy0))
("battistoni" nil (b aa0 t iy0 s t ow1 n iy0))
("battle" nil (b ae1 t ax l))
("battled" nil (b ae1 t ax l d))
("battlefield" nil (b ae1 t ax l f iy1 l d))
("battlefields" nil (b ae1 t ax l f iy1 l d z))
("battlefront" nil (b ae1 t ax l f r ah1 n t))
("battleground" nil (b ae1 t ax l g r aw1 n d))
("battlegrounds" nil (b ae1 t ax l g r aw1 n d z))
("battlement" nil (b ae1 t ax l m ax n t))
("battlements" nil (b ae1 t ax l m ax n t s))
("battles" nil (b ae1 t ax l z))
("battleship" nil (b ae1 t ax l sh ih1 p))
("battleships" nil (b ae1 t ax l sh ih1 p s))
("battling" nil (b ae1 t ax l ih0 ng))
("batton" nil (b ae1 t ax n))
("batts" nil (b ae1 t s))
("batty" nil (b ae1 t iy0))
("batuigas" nil (b ae1 ch uw0 iy1 g ax s))
("batus" nil (b ae1 t ax s))
("baty" nil (b ey1 t iy0))
("batz" nil (b ae1 t s))
("batzel" nil (b ae1 t z ax l))
("batzer" nil (b ey1 t z er0))
("baubles" nil (b ao1 b ax l z))
("baublitz" nil (b aw1 b l ih0 t s))
("bauch" nil (b ao1 ch))
("baucom" nil (b ow0 k aa1 m))
("baucum" nil (b ow0 k ah1 m))
("baucus" nil (b aa1 k ax s))
("baud" nil (b ao1 d))
("baudendistel" nil (b aw1 d ih0 n d ih0 s t ax l))
("bauder" nil (b aw1 d er0))
("baudoin" nil (b ow0 d w aa1 ey0 n))
("baudouin" nil (b ao0 d uw1 ih0 n))
("bauer" nil (b aw1 er0))
("bauerle" nil (b aw1 er0 ax l))
("bauerlein" nil (b aw1 er0 l ay0 n))
("bauermeister" nil (b aw1 er0 m ay0 s t er0))
("bauernfeind" nil (b aw1 er0 n f ay0 n d))
("bauers" nil (b aw1 er0 z))
("baugh" nil (b ao1))
("baughan" nil (b ao1 ax n))
("baugher" nil (b ao1 er0))
("baughman" nil (b ao1 m ax n))
("baughn" nil (b ao1 n))
("bauguess" nil (b aw1 g ih0 s))
("baugus" nil (b ao1 g ax s))
("bauknecht" nil (b aw1 k n ih0 k t))
("bauknight" nil (b ao1 k n ay1 t))
("baum" nil (b ao1 m))
("bauman" nil (b aw1 m ax n))
("baumann" nil (b aw1 m ax n))
("baumbach" nil (b aw1 m b aa1 k))
("baumberger" nil (b aw1 m b er0 g er0))
("baumeister" nil (b aw1 m ay1 s t er0))
("baumel" nil (b aw1 m ax l))
("baumer" nil (b aw1 m er0))
("baumert" nil (b aw1 m er0 t))
("baumgard" nil (b aw1 m g er0 d))
("baumgardner" nil (b aw1 m g aa1 r d n er0))
("baumgardt" nil (b aw1 m g aa1 r t))
("baumgarner" nil (b aw1 m g aa1 r n er0))
("baumgart" nil (b aw1 m g aa1 r t))
("baumgarten" nil (b aw1 m g aa0 r t ax n))
("baumgartner" nil (b aw1 m g aa1 r t n er0))
("baumhardt" nil (b aw1 m hh aa1 r t))
("baumler" nil (b aw1 m ax l er0))
("baumstark" nil (b aw1 m s t aa1 r k))
("baun" nil (b ao1 n))
("baur" nil (b ao1 r))
("baus" nil (b ao1 z))
("bausch" nil (b aw1 sh))
("bauserman" nil (b aw1 s er0 m ax n))
("bausman" nil (b aw1 s m ax n))
("bautch" nil (b aw1 ch))
("bautista" nil (b ao0 t iy1 s t ax))
("bautz" nil (b ao1 t s))
("bautzer" nil (b aw1 t s er0))
("bauwens" nil (b aw1 w ax n z))
("bauxite" nil (b ao1 k s ay0 t))
("bauza" nil (b aw1 z ax))
("bava" nil (b aa1 v ax))
("bavadra" nil (b ax v ae1 d r ax))
("bavaria" nil (b ax v eh1 r iy0 ax))
("bavarian" nil (b ax v eh1 r iy0 ax n))
("bavaro" nil (b aa0 v aa1 r ow0))
("baver" nil (b ey1 v er0))
("bavis" nil (b ey1 v ih0 s))
("bawd" nil (b ao1 d))
("bawden" nil (b ao1 d ax n))
("bawdy" nil (b ao1 d iy0))
("bawer" nil (b ao1 er0))
("bawled" nil (b ao1 l d))
("bax" nil (b ae1 k s))
("baxa" nil (b ae1 k s ax))
("baxendale" nil (b ae1 k s ax n d ey0 l))
("baxley" nil (b ae1 k s l iy0))
("baxt" nil (b ae1 k s t))
("baxter" nil (b ae1 k s t er0))
("bay" nil (b ey1))
("bayard" nil (b ey1 er0 d))
("baybank" nil (b ey1 b ae1 ng k))
("baybanks" nil (b ey1 b ae1 ng k s))
("baye" nil (b ey1))
("bayed" nil (b ey1 d))
("bayer" nil (b ey1 er0))
("bayerische" nil (b ey1 er0 iy1 sh))
("bayers" nil (b ey1 er0 z))
("bayes" nil (b ey1 z))
("bayh" nil (b ey1))
("baying" nil (b ey1 ih0 ng))
("bayle" nil (b ey1 l))
("baylen" nil (b ey1 l ax n))
("bayles" nil (b ey1 l z))
("bayless" nil (b ey1 l ih0 s))
("bayley" nil (b ey1 l iy0))
("bayliff" nil (b ey1 l ih0 f))
("bayliner" nil (b ey1 l ay1 n er0))
("baylis" nil (b ey1 l ih0 s))
("bayliss" nil (b ey1 l ih0 s))
("baylor" nil (b ey1 l er0))
("bayly" nil (b ey1 l iy0))
("bayman" nil (b ey1 m ax n))
("baynard" nil (b ey1 n er0 d))
("bayne" nil (b ey1 n))
("baynes" nil (b ey1 n z))
("baynham" nil (b ey1 n hh ax m))
("bayog" nil (b ey1 aa1 g))
("bayonet" nil (b ey1 ax n ax t))
("bayonne" nil (b ey1 y ow1 n))
("bayou" nil (b ay1 uw0))
("bayous" nil (b ay1 uw0 z))
("bayreuth" nil (b ey1 r oy1 th))
("bays" nil (b ey1 z))
("bayse" nil (b ey1 z))
("bayside" nil (b ey1 s ay1 d))
("baysinger" nil (b ey1 s ih0 n jh er0))
("baytown" nil (b ey1 t aw1 n))
("baytree" nil (b ey1 t r iy1))
("bayuk" nil (b ey1 ax k))
("bayus" nil (b ey1 ax s))
("baz" nil (b ae1 z))
("bazaar" nil (b ax z aa1 r))
("bazaars" nil (b ax z aa1 r z))
("bazan" nil (b ey1 z ax n))
("bazar" nil (b ax z aa1 r))
("baze" nil (b ey1 z))
("bazemore" nil (b aa1 z m ao0 r))
("bazile" nil (b aa1 z ay0 l))
("bazinet" nil (b ae1 z ih0 n eh0 t))
("bazzano" nil (b aa0 t s aa1 n ow0))
("bazzell" nil (b ae1 z ax l))
("bazzle" nil (b ae1 z ax l))
("bbc" nil (b iy1 b iy0 s iy1))
("bbq" nil (b iy1 b iy0 k y uw1))
("be" nil (b iy1))
("bea" nil (b iy1))
("beaber" nil (b iy1 b er0))
("beabout" nil (b ax b uw1 t))
("beach" nil (b iy1 ch))
("beacham" nil (b ax sh ae1 m))
("beachcomber" nil (b iy1 ch k ow1 m er0))
("beached" nil (b iy1 ch t))
("beachem" nil (b iy1 k ih0 m))
("beacher" nil (b iy1 ch er0))
("beaches" nil (b iy1 ch ax z))
("beachfront" nil (b iy1 ch f r aa0 n t))
("beachhead" nil (b iy1 ch hh eh1 d))
("beachheads" nil (b iy1 ch hh eh1 d z))
("beaching" nil (b iy1 ch ih0 ng))
("beachler" nil (b iy1 k ax l er0))
("beachum" nil (b iy1 k ax m))
("beachy" nil (b iy1 ch iy0))
("beacom" nil (b iy1 k ax m))
("beacon" nil (b iy1 k ax n))
("beacons" nil (b iy1 k ax n z))
("bead" nil (b iy1 d))
("beaded" nil (b iy1 d ih0 d))
("beadle" nil (b iy1 d ax l))
("beadles" nil (b iy1 d ax l z))
("beadling" nil (b iy1 d l ih0 ng))
("beads" nil (b iy1 d z))
("beadwork" nil (b iy1 d w er1 k))
("beady" nil (b iy1 d iy0))
("beagan" nil (b ax g ae1 n))
("beagen" nil (b iy1 g ax n))
("beagle" nil (b iy1 g ax l))
("beagles" nil (b iy1 g ax l z))
("beagley" nil (b iy1 g l iy0))
("beahan" nil (b iy1 hh ax n))
("beahm" nil (b iy1 m))
("beaird" nil (b ih1 r d))
("beak" nil (b iy1 k))
("beaker" nil (b iy1 k er0))
("beakley" nil (b iy1 k l iy0))
("beaklike" nil (b iy1 k l ay1 k))
("beal" nil (b iy1 l))
("beale" nil (b iy1 l))
("bealer" nil (b iy1 l er0))
("beales" nil (b iy1 l z))
("beall" nil (b iy1 l))
("beals" nil (b iy1 l z))
("beam" nil (b iy1 m))
("beaman" nil (b iy1 m ax n))
("beamed" nil (b iy1 m d))
("beamer" nil (b iy1 m er0))
("beaming" nil (b iy1 m ih0 ng))
("beamish" nil (b iy1 m ih0 sh))
("beamon" nil (b iy1 m ax n))
("beams" nil (b iy1 m z))
("bean" nil (b iy1 n))
("beanblossom" nil (b iy1 n b l ax s ax m))
("beane" nil (b iy1 n))
("beans" nil (b iy1 n z))
("beantown" nil (b iy1 n t aw1 n))
("bear" nil (b eh1 r))
("bearable" nil (b eh1 r ax b ax l))
("bearce" nil (b er1 s))
("beard" nil (b ih1 r d))
("bearded" nil (b ih1 r d ax d))
("bearden" nil (b ih1 r d ax n))
("beardmore" nil (b ih1 r d m ao1 r))
("beards" nil (b ih1 r d z))
("beardslee" nil (b ih1 r d z l iy0))
("beardsley" nil (b ih1 r d z l iy0))
("beardstown" nil (b ih1 r d s t aw1 n))
("beare" nil (b ih1 r))
("bearer" nil (b eh1 r er0))
("bearers" nil (b eh1 r er0 z))
("bearfield" nil (b er1 f iy0 l d))
("bearing" nil (b eh1 r ih0 ng))
("bearings" nil (b eh1 r ih0 ng z))
("bearish" nil (b eh1 r ih0 sh))
("bearishly" nil (b eh1 r ih0 sh l iy0))
("bearishness" nil (b eh1 r ih0 sh n ax s))
("bearman" nil (b eh1 r m ax n))
("bears" nil (b eh1 r z))
("bearse" nil (b er1 s))
("bearss" nil (b er1 s))
("bearup" nil (b eh1 r ax p))
("beary" nil (b ih1 r iy0))
("beas" nil (b iy1 z))
("beasley" nil (b iy1 z l iy0))
("beason" nil (b iy1 z ax n))
("beast" nil (b iy1 s t))
("beastie" nil (b iy1 s t iy0))
("beaston" nil (b iy1 s t ax n))
("beasts" nil (b iy1 s t s))
("beat" nil (b iy1 t))
("beata" nil (b iy0 ae1 t ax))
("beaten" nil (b iy1 t ax n))
("beater" nil (b iy1 t er0))
("beaters" nil (b iy1 t er0 z))
("beatie" nil (b iy1 t iy0))
("beating" nil (b iy1 t ih0 ng))
("beatings" nil (b iy1 t ih0 ng z))
("beatle" nil (b iy1 t ax l))
("beatles" nil (b iy1 t ax l z))
("beatniks" nil (b iy1 t n ih0 k s))
("beato" nil (b iy0 ae1 t ow0))
("beaton" nil (b iy1 t ax n))
("beatrice" nil (b iy1 ax t r ax s))
("beatrix" nil (b iy1 t r ih0 k s))
("beats" nil (b iy1 t s))
("beatson" nil (b iy1 t s ax n))
("beattie" nil (b iy1 t iy0))
("beatty" nil (b iy1 t iy0))
("beaty" nil (b iy1 t iy0))
("beau" nil (b ow1))
("beaubien" nil (b ow0 b iy1 n))
("beaucage" nil (b ow1 k ey1 jh))
("beauce" nil (b ow1 s iy1))
("beauchaine" nil (b ow1 sh ey1 n))
("beauchamp" nil (b ow1 sh aa1 m p))
("beauchemin" nil (b ow1 sh ih0 m ae0 n))
("beauchene" nil (b ow1 sh iy1 n))
("beauchesne" nil (b ow0 sh eh1 n))
("beaudet" nil (b ow0 d eh1 t))
("beaudette" nil (b ow1 d eh1 t))
("beaudin" nil (b ow0 d ae1 n))
("beaudoin" nil (b ow0 d oy1 n))
("beaudreau" nil (b ow1 d r ow1))
("beaudry" nil (b ow0 d r iy1))
("beauford" nil (b ow0 f ao1 r d))
("beaufort" nil (b ow1 f er0 t))
("beaufrere" nil (b ow1 f r eh1 r))
("beaujolais" nil (b ow1 jh ow0 l ey1))
("beaulac" nil (b ow0 l ae1 k))
("beaumier" nil (b y uw1 m iy0 er0))
("beaumont" nil (b ow1 m ao0 n t))
("beaupre" nil (b ow1 p er0))
("beauregard" nil (b ow1 r ih0 g aa1 r d))
("beausoleil" nil (b ow1 s ax l ey0))
("beautician" nil (b ow1 t ih1 sh ax n))
("beauties" nil (b y uw1 t iy0 z))
("beautiful" nil (b y uw1 t ax f ax l))
("beautifully" nil (b y uw1 t ax f l iy0))
("beautify" nil (b y uw1 t ih0 f ay1))
("beauty" nil (b y uw1 t iy0))
("beauvais" nil (b ow0 v ey1))
("beaux" nil (b ow1))
("beavan" nil (b ax v ae1 n))
("beaven" nil (b iy1 v ax n))
("beaver" nil (b iy1 v er0))
("beavers" nil (b iy1 v er0 z))
("beaverson" nil (b iy1 v er0 s ax n))
("beaverton" nil (b iy1 v er0 t ax n))
("beavin" nil (b iy1 v ih0 n))
("beavis" nil (b iy1 v ax s))
("beazer" nil (b iy1 z er0))
("beazley" nil (b iy1 z l iy0))
("bebb" nil (b eh1 b))
("bebber" nil (b eh1 b er0))
("bebe" nil (b iy1 b iy0))
("bebear" nil (b ax b ih1 r))
("bebeau" nil (b ih0 b ow1))
("bebee" nil (b eh1 b iy0))
("beber" nil (b iy1 b er0))
("bebop" nil (b iy1 b aa1 p))
("bebout" nil (b ih0 b uw1 t))
("becalmed" nil (b ih0 k aa1 m d))
("became" nil (b ih0 k ey1 m))
("because" nil (b ih0 k ao1 z))
("becerra" nil (b eh0 ch eh1 r ax))
("becerril" nil (b ey0 s eh0 r iy1 l))
("bechard" nil (b eh1 k er0 d))
("becher" nil (b eh1 k er0))
("becherer" nil (b eh1 k er0 er0))
("bechler" nil (b eh1 k l er0))
("becht" nil (b eh1 k t))
("bechtel" nil (b eh1 k t eh1 l))
("bechthold" nil (b eh1 k th ow0 l d))
("bechtle" nil (b eh1 ch t ax l))
("bechtol" nil (b eh1 k t ax l))
("bechtold" nil (b eh1 k t ow1 l d))
("beck" nil (b eh1 k))
("becka" nil (b eh1 s k ax))
("beckel" nil (b eh1 k eh0 l))
("becker" nil (b eh1 k er0))
("beckerman" nil (b eh1 k er0 m ax n))
("beckers" nil (b eh1 k er0 z))
("beckert" nil (b eh1 k er0 t))
("becket" nil (b eh1 k ax t))
("beckett" nil (b eh1 k ih0 t))
("beckford" nil (b eh1 k f ao0 r d))
("beckham" nil (b eh1 k hh ae1 m))
("beckie" nil (b eh1 k iy0))
("beckius" nil (b eh1 k iy0 ih0 s))
("beckler" nil (b eh1 k l er0))
("beckles" nil (b eh1 k ax l z))
("beckley" nil (b eh1 k l iy0))
("becklund" nil (b eh1 k l ax n d))
("beckman" nil (b eh1 k m ax n))
("beckmann" nil (b eh1 k m ax n))
("beckmeyer" nil (b eh1 k m ay0 er0))
("becknell" nil (b eh1 k n eh1 l))
("beckner" nil (b eh1 k n er0))
("beckom" nil (b eh1 k aa0 m))
("beckon" nil (b eh1 k ax n))
("beckoned" nil (b eh1 k ax n d))
("beckoning" nil (b eh1 k ax n ih0 ng))
("beckons" nil (b eh1 k ax n z))
("becks" nil (b eh1 k s))
("beckstead" nil (b eh1 k s t eh1 d))
("beckstrand" nil (b eh1 k s t r ae1 n d))
("beckstrom" nil (b eh1 k s t r aa0 m))
("beckum" nil (b eh1 k ax m))
("beckwith" nil (b eh1 k w ih1 th))
("beckwitt" nil (b eh1 k w ih1 t))
("beckworth" nil (b eh1 k w er1 th))
("becky" nil (b eh1 k iy0))
("becnel" nil (b eh1 k n ax l))
("beco" nil (b iy1 k ow0))
("become" nil (b ih0 k ah1 m))
("becomes" nil (b ih0 k ah1 m z))
("becoming" nil (b ih0 k ah1 m ih0 ng))
("becor" nil (b eh1 k ao1 r))
("becraft" nil (b iy1 k r ae0 f t))
("becton" nil (b eh1 k t ax n))
("becvar" nil (b eh1 k v er0))
("bed" nil (b eh1 d))
("beda" nil (b ey1 d ax))
("bedard" nil (b ih0 d aa1 r d))
("bedbugs" nil (b eh1 d b ax g z))
("bedchamber" nil (b eh1 d ch ey1 m b er0))
("bedding" nil (b eh1 d ih0 ng))
("beddingfield" nil (b eh1 d ih0 ng f iy1 l d))
("beddoe" nil (b eh1 d ow0))
("beddors" nil (b eh1 d er0 z))
("beddow" nil (b eh1 d ow0))
("bedecked" nil (b ih0 d eh1 k t))
("bedel" nil (b ih0 d eh1 l))
("bedell" nil (b eh1 d ax l))
("bedenbaugh" nil (b eh1 d ih0 n b aw0))
("bedevil" nil (b ih0 d eh1 v ax l))
("bedeviled" nil (b ih0 d eh1 v ax l d))
("bedfellows" nil (b eh1 d f eh1 l ow0 z))
("bedford" nil (b eh1 d f er0 d))
("bedgood" nil (b eh1 d g uh1 d))
("bedient" nil (b iy1 d y ih0 n t))
("bedinger" nil (b eh1 d ih0 ng er0))
("bedingfield" nil (b eh1 d ih0 ng f iy1 l d))
("bedke" nil (b eh1 d k iy0))
("bedlam" nil (b eh1 d l ax m))
("bednar" nil (b eh1 d n er0))
("bednarczyk" nil (b eh1 d n er0 ch ih0 k))
("bednarek" nil (b eh1 d n aa0 r eh0 k))
("bednarik" nil (b eh1 d n aa0 r ih0 k))
("bednarski" nil (b eh1 d n aa0 r s k iy0))
("bednarz" nil (b eh1 d n aa0 r z))
("bedner" nil (b eh1 d n er0))
("bednorz" nil (b eh1 d n ao0 r z))
("bedolla" nil (b eh0 d ow1 l ax))
("bedore" nil (b eh1 d ao1 r))
("bedouin" nil (b eh1 d ow0 ax n))
("bedouins" nil (b eh1 d ow0 ax n z))
("bedoya" nil (b ey0 d oy1 ax))
("bedpans" nil (b eh1 d p ae1 n z))
("bedraggled" nil (b ih0 d r ae1 g ax l d))
("bedridden" nil (b eh1 d r ih1 d ax n))
("bedrock" nil (b eh1 d r aa1 k))
("bedroom" nil (b eh1 d r uw1 m))
("bedrooms" nil (b eh1 d r uw1 m z))
("bedrosian" nil (b eh0 d r aa1 zh ih0 n))
("beds" nil (b eh1 d z))
("bedsaul" nil (b eh1 d s ao0 l))
("bedside" nil (b eh1 d s ay1 d))
("bedsole" nil (b eh1 d s ow1 l))
("bedspread" nil (b eh1 d s p r eh1 d))
("bedstraw" nil (b eh1 d s t r ao1))
("bedtime" nil (b eh1 d t ay1 m))
("bedwell" nil (b eh1 d w eh1 l))
("bee" nil (b iy1))
("beebe" nil (b iy1 b iy1))
("beebes" nil (b iy1 b iy1 z))
("beebower" nil (b iy1 b ow1 er0))
("beeby" nil (b iy1 b iy0))
("beech" nil (b iy1 ch))
("beecham" nil (b iy1 ch ax m))
("beechcraft" nil (b iy1 ch k r ae1 f t))
("beecher" nil (b iy1 ch er0))
("beechler" nil (b iy1 ch l er0))
("beechwood" nil (b iy1 ch w uh1 d))
("beechy" nil (b iy1 ch iy0))
("beeck" nil (b iy1 k))
("beecroft" nil (b iy1 k r ao1 f t))
("beede" nil (b iy1 d))
("beedie" nil (b iy1 d iy0))
("beedle" nil (b iy1 d ax l))
("beedy" nil (b iy1 d iy0))
("beef" nil (b iy1 f))
("beefeater" nil (b iy1 f iy1 t er0))
("beefed" nil (b iy1 f t))
("beefing" nil (b iy1 f ih0 ng))
("beefs" nil (b iy1 f s))
("beefsteak" nil (b iy1 f s t ey1 k))
("beefy" nil (b iy1 f iy0))
("beeghly" nil (b iy1 g l iy0))
("beegle" nil (b iy1 g ax l))
("beehive" nil (b iy1 hh ay1 v))
("beehler" nil (b iy1 l er0))
("beek" nil (b iy1 k))
("beekeepers" nil (b iy1 k iy1 p er0 z))
("beeker" nil (b iy1 k er0))
("beekman" nil (b iy1 k m ax n))
("beeks" nil (b iy1 k s))
("beel" nil (b iy1 l))
("beeler" nil (b iy1 l er0))
("beem" nil (b iy1 m))
("beeman" nil (b iy1 m ax n))
("beemer" nil (b iy1 m er0))
("been" nil (b ih1 n))
("beene" nil (b iy1 n))
("beeney" nil (b iy1 n iy0))
("beenken" nil (b iy1 ng k ax n))
("beens" nil (b ih1 n z))
("beep" nil (b iy1 p))
("beeper" nil (b iy1 p er0))
("beepers" nil (b iy1 p er0 z))
("beer" nil (b ih1 r))
("beerbower" nil (b ih1 r b ow0 er0))
("beerman" nil (b ih1 r m ax n))
("beers" nil (b ih1 r z))
("beery" nil (b ih1 r iy0))
("bees" nil (b iy1 z))
("beese" nil (b iy1 z))
("beesley" nil (b iy1 z l iy0))
("beeson" nil (b iy1 z ax n))
("beet" nil (b iy1 t))
("beethoven" nil (b ey1 t ow0 v ax n))
("beetle" nil (b iy1 t ax l))
("beetles" nil (b iy1 t ax l z))
("beeton" nil (b iy1 t ax n))
("beets" nil (b iy1 t s))
("beever" nil (b iy1 v er0))
("beevers" nil (b iy1 v er0 z))
("beezley" nil (b iy1 z l iy0))
("befall" nil (b ih0 f ao1 l))
("befallen" nil (b ax f ae1 l ax n))
("befalls" nil (b ih0 f ao1 l z))
("befell" nil (b ih0 f eh1 l))
("beffa" nil (b ih1 f aa1))
("befits" nil (b ih0 f ih1 t s))
("befitting" nil (b ih0 f ih1 t ih0 ng))
("before" nil (b ih0 f ao1 r))
("beforehand" nil (b ih0 f ao1 r hh ae1 n d))
("befort" nil (b eh1 f er0 t))
("befriend" nil (b ih0 f r eh1 n d))
("befriended" nil (b ih0 f r eh1 n d ih0 d))
("befriends" nil (b ih0 f r eh1 n d z))
("befuddled" nil (b ih0 f ah1 d ax l d))
("befuddles" nil (b ih0 f ah1 d ax l z))
("beg" nil (b eh1 g))
("begajah" nil (b eh0 g aa1 jh ax))
("begala" nil (b eh0 g aa1 l ax))
("began" nil (b ih0 g ae1 n))
("begat" nil (b ih0 g ae1 t))
("begay" nil (b eh1 g ey0))
("begel" nil (b ey1 g ax l))
("begelman" nil (b ey1 g ax l m ax n))
("begeman" nil (b iy1 g m ax n))
("beger" nil (b eh1 g er0))
("beget" nil (b ih0 g eh1 t))
("begets" nil (b ih0 g eh1 t s))
("begetting" nil (b ih0 g eh1 t ih0 ng))
("begg" nil (b eh1 g))
("beggar" nil (b eh1 g er0))
("beggars" nil (b eh1 g er0 z))
("begged" nil (b eh1 g d))
("begging" nil (b eh1 g ih0 ng))
("beggs" nil (b eh1 g z))
("beghin" nil (b eh1 g ih0 n))
("begin" nil (b ih0 g ih1 n))
("beginner" nil (b ih0 g ih1 n er0))
("beginners" nil (b ih0 g ih1 n er0 z))
("beginning" nil (b ih0 g ih1 n ih0 ng))
("beginnings" nil (b ih0 g ih1 n ih0 ng z))
("begins" nil (b ih0 g ih1 n z))
("begleiter" nil (b eh1 g l ay1 t er0))
("begley" nil (b eh1 g l iy0))
("begnaud" nil (b ih0 g n ow1))
("begnoche" nil (b eh0 g n ow1 k iy0))
("begonia" nil (b ih0 g ow1 n y ax))
("begonias" nil (b ih0 g ow1 n y ax z))
("begrudge" nil (b ih0 g r ah1 jh))
("begs" nil (b eh1 g z))
("begue" nil (b ey1 g))
("beguelin" nil (b ey1 g ax l ih0 n))
("beguiled" nil (b ih0 g ay1 l d))
("beguiling" nil (b ih0 g ay1 l ih0 ng))
("begun" nil (b ih0 g ah1 n))
("behalf" nil (b ih0 hh ae1 f))
("behan" nil (b eh1 hh ax n))
("behar" nil (b eh1 hh er0))
("behave" nil (b ih0 hh ey1 v))
("behaved" nil (b ih0 hh ey1 v d))
("behaves" nil (b ih0 hh ey1 v z))
("behaving" nil (b ih0 hh ey1 v ih0 ng))
("behavior" nil (b ih0 hh ey1 v y er0))
("behavioral" nil (b ih0 hh ey1 v y er0 ax l))
("behaviorists" nil (b ih0 hh ey1 v y er0 ih0 s t s))
("behaviors" nil (b ih0 hh ey1 v y er0 z))
("beheaded" nil (b ih0 hh eh1 d ih0 d))
("beheading" nil (b ih0 hh eh1 d ih0 ng))
("beheler" nil (b eh1 hh ax l er0))
("behemoth" nil (b ax hh iy1 m ax th))
("behemoths" nil (b ih0 hh iy1 m ax th s))
("behest" nil (b ih0 hh eh1 s t))
("behind" nil (b ih0 hh ay1 n d))
("behinds" nil (b ih0 hh ay1 n d z))
("behl" nil (b eh1 l))
("behle" nil (b eh1 hh ax l))
("behler" nil (b eh1 l er0))
("behling" nil (b eh1 l ih0 ng))
("behlke" nil (b eh1 l k))
("behm" nil (b eh1 m))
("behmer" nil (b eh1 m er0))
("behn" nil (b eh1 n))
("behne" nil (b eh1 n))
("behner" nil (b eh1 n er0))
("behney" nil (b eh1 n iy0))
("behning" nil (b eh1 n ih0 ng))
("behnke" nil (b eh1 ng k))
("behnken" nil (b eh1 ng k ax n))
("behof" nil (b iy1 hh aa0 f))
("behold" nil (b ih0 hh ow1 l d))
("beholden" nil (b ih0 hh ow1 l d ax n))
("beholder" nil (b iy0 hh ow1 l d er0))
("beholding" nil (b ih0 hh ow1 l d ih0 ng))
("behoove" nil (b ih0 hh uw1 v))
("behr" nil (b eh1 r))
("behrend" nil (b eh1 r eh0 n d))
("behrends" nil (b eh1 r eh0 n d z))
("behrendt" nil (b eh1 r ih0 n t))
("behrens" nil (b eh1 r ax n z))
("behring" nil (b eh1 r ih0 ng))
("behringer" nil (b eh1 r ih0 ng er0))
("behringwerke" nil (b eh1 r ih1 ng w er0 k))
("behrle" nil (b eh1 r ax l))
("behrman" nil (b eh1 r m ax n))
("behrmann" nil (b eh1 r m ax n))
("behrns" nil (b eh1 r n z))
("behunin" nil (b eh1 hh ax n ih0 n))
("behymer" nil (b eh1 hh ay1 m er0))
("beidaihe" nil (b ay1 d ey1 hh iy0))
("beidleman" nil (b ay1 d ax l m ax n))
("beidler" nil (b ay1 d ax l er0))
("beier" nil (b ay1 er0))
("beierle" nil (b ay1 er0 ax l))
("beiersdorf" nil (b ay1 r z d ao1 r f))
("beige" nil (b ey1 zh))
("beigel" nil (b ay1 g ax l))
("beiges" nil (b ey1 zh ax z))
("beighley" nil (b ey1 g l iy0))
("beightol" nil (b ey1 t ax l))
("beijer" nil (b ay1 r))
("beijing" nil (b ey1 zh ih1 ng))
("beil" nil (b iy1 l))
("beilenson" nil (b ay1 l ax n s ax n))
("beiler" nil (b ay1 l er0))
("beilfuss" nil (b ay1 l f ax s))
("beilke" nil (b iy1 l k))
("beilman" nil (b ay1 l m ax n))
("beim" nil (b ay1 m))
("bein" nil (b iy1 n))
("beine" nil (b iy1 n))
("being" nil (b iy1 ih0 ng))
("beings" nil (b iy1 ih0 ng z))
("beiny" nil (b ay1 n iy0))
("beira" nil (b ey1 r ax))
("beirne" nil (b ih1 r n))
("beirut" nil (b ey0 r uw1 t))
("beisel" nil (b ay1 s ax l))
("beiser" nil (b ay1 s er0))
("beisner" nil (b ay1 s n er0))
("beissel" nil (b ay1 s ax l))
("beiswenger" nil (b ay1 s w ih0 n jh er0))
("beit" nil (b ey1 t))
("beitel" nil (b ay1 t ax l))
("beiter" nil (b ay1 t er0))
("beitler" nil (b ay1 t ax l er0))
("beitz" nil (b iy1 t s))
("beitzel" nil (b ay1 t z ax l))
("bejar" nil (b ey0 y aa1 r))
("bejarano" nil (b ey0 y aa0 r aa1 n ow0))
("bekaa" nil (b eh0 k aa1))
("bekaert" nil (b ax k eh1 r t))
("beker" nil (b eh1 k er0))
("bekins" nil (b iy1 k ih0 n z))
("bekker" nil (b eh1 k er0))
("bekki" nil (b eh1 k iy0))
("bel" nil (b eh1 l))
("bela" nil (b eh1 l ax))
("belabor" nil (b ih0 l ey1 b er0))
("belabored" nil (b ih0 l ey1 b er0 d))
("belabores" nil (b ih0 l ey1 b er0 z))
("belaboring" nil (b ih0 l ey1 b er0 ih0 ng))
("beladur" nil (b eh1 l ax d uh1 r))
("belafonte" nil (b eh1 l ax f aa1 n t iy0))
("belair" nil (b ih0 l eh1 r))
("belaire" nil (b ax l eh1 r))
("belak" nil (b eh1 l ax k))
("beland" nil (b eh1 l ax n d))
("belanger" nil (b eh1 l ax ng er0))
("belanoff" nil (b eh1 l ax n ao0 f))
("belarus" nil (b eh0 l aa1 r ax s))
("belasco" nil (b eh0 l aa1 s k ow0))
("belated" nil (b ih0 l ey1 t ih0 d))
("belatedly" nil (b ih0 l ey1 t ax d l iy0))
("belcastro" nil (b eh0 l k ae1 s t r ow0))
("belch" nil (b eh1 l ch))
("belched" nil (b eh1 l ch t))
("belcher" nil (b eh1 l ch er0))
("belching" nil (b eh1 l ch ih0 ng))
("belcourt" nil (b eh1 l k ao1 r t))
("belden" nil (b eh1 l d ax n))
("beldin" nil (b eh1 l d ih0 n))
("belding" nil (b eh1 l d ih0 ng))
("beldock" nil (b eh1 l d aa1 k))
("beldon" nil (b eh1 l d ax n))
("beleaguered" nil (b ih0 l iy1 g er0 d))
("belen" nil (b eh1 l ax n))
("belew" nil (b eh1 l uw0))
("belfast" nil (b eh1 l f ae1 s t))
("belfer" nil (b eh1 l f er0))
("belfield" nil (b eh1 l f iy1 l d))
("belfiore" nil (b eh0 l f iy0 ao1 r iy0))
("belflower" nil (b eh1 l f l aw1 er0))
("belford" nil (b eh1 l f er0 d))
("belfry" nil (b eh1 l f r iy0))
("belgacom" nil (b eh1 l g ax k aa0 m))
("belgard" nil (b ih0 l g aa1 r d))
("belgarde" nil (b eh0 l g aa1 r d iy0))
("belge" nil (b eh1 l jh iy0))
("belger" nil (b eh1 l g er0))
("belgian" nil (b eh1 l jh ax n))
("belgians" nil (b eh1 l jh ax n z))
("belgique" nil (b eh0 l zh iy1 k))
("belgium" nil (b eh1 l jh ax m))
("belgo" nil (b eh1 l g ow1))
("belgrade" nil (b eh1 l g r ey0 d))
("belgrave" nil (b eh1 l g r ey1 v))
("belich" nil (b eh1 l ih0 k))
("belie" nil (b ih0 l ay1))
("belied" nil (b ih0 l ay1 d))
("belief" nil (b ih0 l iy1 f))
("beliefs" nil (b ih0 l iy1 f s))
("belier" nil (b eh1 l y er0))
("belies" nil (b ih0 l ay1 z))
("believability" nil (b ax l iy1 v ax b ih1 l ih0 t iy0))
("believable" nil (b ax l iy1 v ax b ax l))
("believe" nil (b ih0 l iy1 v))
("believed" nil (b ih0 l iy1 v d))
("believer" nil (b ax l iy1 v er0))
("believers" nil (b ax l iy1 v er0 z))
("believes" nil (b ih0 l iy1 v z))
("believing" nil (b ih0 l iy1 v ih0 ng))
("belin" nil (b eh1 l ih0 n))
("belinda" nil (b ax l ih1 n d ax))
("belinsky" nil (b ih0 l ih1 n s k iy0))
("belisle" nil (b eh1 l ay0 ax l))
("belittle" nil (b ih0 l ih1 t ax l))
("belittled" nil (b ih0 l ih1 t ax l d))
("belittles" nil (b ih0 l ih1 t ax l z))
("belittling" nil (b ih0 l ih1 t ax l ih0 ng))
("belitz" nil (b eh1 l ih0 t s))
("beliveau" nil (b eh1 l ih0 v ow1))
("belize" nil (b eh0 l iy1 z))
("belk" nil (b eh1 l k))
("belka" nil (b eh1 l k ax))
("belkacem" nil (b eh1 l k ax s ax m))
("belke" nil (b eh1 l k))
("belkin" nil (b eh1 l k ih0 n))
("belknap" nil (b eh1 l n ae0 p))
("belko" nil (b eh1 l k ow0))
("bell" nil (b eh1 l))
("bella" nil (b eh1 l ax))
("bellah" nil (b eh1 l ax))
("bellamy" nil (b eh1 l ax m iy0))
("bellanca" nil (b eh0 l aa1 n k ax))
("belland" nil (b eh1 l ax n d))
("bellanger" nil (b eh1 l ae1 ng g er0))
("bellante" nil (b eh0 l aa1 n t iy0))
("bellantoni" nil (b eh0 l aa0 n t ow1 n iy0))
("bellar" nil (b eh1 l er0))
("bellard" nil (b ih0 l aa1 r d))
("bellas" nil (b eh1 l ax z))
("bellavance" nil (b eh0 l aa1 v ax n s))
("bellavia" nil (b eh0 l aa1 v iy0 ax))
("bellavista" nil (b eh1 l ax v ih1 s t ax))
("bellcore" nil (b eh1 l k ao1 r))
("belle" nil (b eh1 l))
("belleau" nil (b ih0 l ow1))
("bellefeuille" nil (b eh1 l ax f iy0 uw1 l))
("bellemare" nil (b eh1 l ax m eh1 r))
("beller" nil (b eh1 l er0))
("bellerose" nil (b eh1 l er0 ax z))
("belles" nil (b eh1 l z))
("belleville" nil (b ax l v ih1 l))
("bellevue" nil (b eh1 l v y uw1))
("bellew" nil (b ih0 l uw1))
("bellflower" nil (b eh1 l f l aw1 er0))
("bellflowers" nil (b eh1 l f l aw1 er0 z))
("bellhops" nil (b eh1 l hh aa1 p s))
("belli" nil (b eh1 l iy0))
("bellicose" nil (b eh1 l ax k ow1 s))
("bellied" nil (b eh1 l iy0 d))
("bellies" nil (b eh1 l iy0 z))
("belligerence" nil (b ax l ih1 jh er0 ax n s))
("belligerent" nil (b ax l ih1 jh er0 ax n t))
("belligerents" nil (b ax l ih1 jh er0 ax n t s))
("bellin" nil (b eh1 l ih0 n))
("bellina" nil (b eh0 l iy1 n ax))
("belling" nil (b eh1 l ih0 ng))
("bellinger" nil (b eh1 l ih0 ng er0))
("bellingham" nil (b eh1 l ih0 ng hh ae1 m))
("bellinghausen" nil (b eh1 l ih0 ng hh aw1 z ax n))
("bellini" nil (b eh0 l iy1 n iy0))
("bellino" nil (b eh0 l iy1 n ow0))
("bellis" nil (b eh1 l ih0 s))
("bellissimo" nil (b eh0 l iy0 s iy1 m ow0))
("belliveau" nil (b eh1 l ih0 v ow1))
("bellizzi" nil (b eh0 l iy1 t s iy0))
("bellm" nil (b eh1 l m))
("bellman" nil (b eh1 l m ax n))
("bellmon" nil (b eh1 l m ax n))
("bellmore" nil (b eh1 l m ao0 r))
("bello" nil (b eh1 l ow0))
("bellomo" nil (b eh0 l ow1 m ow0))
("bellomy" nil (b eh1 l ax m iy0))
("bellon" nil (b eh1 l ax n))
("bellone" nil (b eh0 l ow1 n iy0))
("bellotti" nil (b eh0 l ow1 t iy0))
("bellow" nil (b eh1 l ow0))
("bellowed" nil (b eh1 l ow0 d))
("bellowing" nil (b eh1 l ow0 ih0 ng))
("bellows" nil (b eh1 l ow0 z))
("bellrose" nil (b eh1 l r ow1 z))
("bells" nil (b eh1 l z))
("bellsouth" nil (b eh1 l s aw1 th))
("bellucci" nil (b eh0 l uw1 ch iy0))
("belluomini" nil (b eh1 l w ow0 m iy1 n iy0))
("bellville" nil (b eh1 l v ih1 l))
("bellwether" nil (b eh1 l w eh1 dh er0))
("bellwethers" nil (b eh1 l w eh1 dh er0 z))
("bellwood" nil (b eh1 l w uh1 d))
("belly" nil (b eh1 l iy0))
("bellyache" nil (b eh1 l iy0 ey1 k))
("belman" nil (b eh1 l m ax n))
("belmont" nil (b eh1 l m aa1 n t))
("belmonte" nil (b eh0 l m ow1 n t iy0))
("belmore" nil (b eh1 l m ao0 r))
("belnap" nil (b eh1 l n ae1 p))
("belnick" nil (b eh1 l n ih0 k))
("belo" nil (b eh1 l ow0))
("beloff" nil (b eh1 l ao1 f))
("beloit" nil (b ih0 l oy1 t))
("belong" nil (b ih0 l ao1 ng))
("belonged" nil (b ih0 l ao1 ng d))
("belongia" nil (b eh0 l ow1 n jh ax))
("belonging" nil (b ih0 l ao1 ng ih0 ng))
("belongings" nil (b ih0 l ao1 ng ih0 ng z))
("belongs" nil (b ih0 l ao1 ng z))
("belote" nil (b eh0 l ow1 t iy0))
("belous" nil (b eh1 l ax s))
("belov" nil (b eh1 l aa0 v))
("belove" nil (b ih0 l ah1 v))
("beloved" nil (b ih0 l ah1 v d))
("below" nil (b ih0 l ow1))
("bels" nil (b eh1 l z))
("belser" nil (b eh1 l s er0))
("belshaw" nil (b eh1 l sh ao1))
("belshe" nil (b eh1 l sh))
("belsito" nil (b eh0 l s iy1 t ow0))
("belsky" nil (b eh1 l s k iy0))
("belson" nil (b eh1 l s ax n))
("belt" nil (b eh1 l t))
("belted" nil (b eh1 l t ax d))
("belter" nil (b eh1 l t er0))
("belth" nil (b eh1 l th))
("belting" nil (b eh1 l t ih0 ng))
("belton" nil (b eh1 l t ax n))
("beltram" nil (b eh1 l t r ae1 m))
("beltran" nil (b eh1 l t r ax n))
("belts" nil (b eh1 l t s))
("beltsville" nil (b eh1 l t s v ih1 l))
("beltway" nil (b eh1 l t w ey1))
("beltz" nil (b eh1 l t s))
("belue" nil (b eh1 l y uw0))
("beluga" nil (b ih0 l uw1 g ax))
("belushi" nil (b eh0 l uw1 sh iy0))
("belva" nil (b ey1 l v ax))
("belveal" nil (b eh1 l v ax l))
("belvedere" nil (b eh1 l v ax d ih1 r))
("belvia" nil (b eh1 l v iy0 ax))
("belvidere" nil (b eh1 l v ih0 d ih1 r))
("belville" nil (b eh1 l v ih1 l))
("belvin" nil (b eh1 l v ih0 n))
("belyea" nil (b eh0 l iy1 ax))
("belyeu" nil (b eh1 l iy0 uw1))
("belying" nil (b ih0 l ay1 ih0 ng))
("belz" nil (b eh1 l z))
("belzberg" nil (b eh1 l t s b er0 g))
("belzbergs" nil (b eh1 l t s b er0 g z))
("belzer" nil (b eh1 l z er0))
("bem" nil (b eh1 m))
("beman" nil (b iy1 m ax n))
("bembenek" nil (b eh1 m b ih0 n ax k))
("bembry" nil (b eh1 m b r iy0))
("bement" nil (b iy1 m ax n t))
("bemis" nil (b iy1 m ax s))
("bemiss" nil (b eh1 m ih0 s))
("bemoan" nil (b ih0 m ow1 n))
("bemoaned" nil (b ih0 m ow1 n d))
("bemoaning" nil (b ih0 m ow1 n ih0 ng))
("bemoans" nil (b ih0 m ow1 n z))
("bemused" nil (b ih0 m y uw1 z d))
("bemusement" nil (b eh1 m y uw0 s m ax n t))
("ben" nil (b eh1 n))
("bena" nil (b eh1 n ax))
("benackova" nil (b eh1 n ax k ow1 v ax))
("benak" nil (b eh1 n ax k))
("benanti" nil (b eh0 n aa1 n t iy0))
("benard" nil (b ih0 n aa1 r d))
("benassi" nil (b eh0 n aa1 s iy0))
("benasuli" nil (b eh1 n ax s uw1 l iy0))
("benatar" nil (b eh1 n ax t er0))
("benavente" nil (b eh0 n aa0 v eh1 n t iy0))
("benavides" nil (b ey0 n aa0 v iy1 d eh0 s))
("benavidez" nil (b ey0 n aa0 v iy1 d eh0 z))
("benazir" nil (b eh1 n ax z ih1 r))
("benbow" nil (b eh1 n b ow0))
("benbrook" nil (b eh1 n b r uh1 k))
("bence" nil (b eh1 n s))
("bench" nil (b eh1 n ch))
("benches" nil (b eh1 n ch ih0 z))
("benchley" nil (b eh1 n ch l iy0))
("benchmark" nil (b eh1 n ch m aa1 r k))
("benchmarks" nil (b eh1 n ch m aa1 r k s))
("bencivenga" nil (b eh0 n ch iy0 v eh1 ng g ax))
("bencomo" nil (b eh0 n k ow1 m ow0))
("bencsik" nil (b eh1 ng k s ih0 k))
("bend" nil (b eh1 n d))
("benda" nil (b eh1 n d ax))
("bendall" nil (b eh1 n d ax l))
("bendectin" nil (b eh0 n d eh1 k t ih0 n))
("bended" nil (b eh1 n d ih0 d))
("bendel" nil (b eh1 n d ax l))
("bendele" nil (b eh1 n d ax l))
("bender" nil (b eh1 n d er0))
("benders" nil (b eh1 n d er0 z))
("bendick" nil (b eh1 n d ih0 k))
("bendickson" nil (b eh1 n d ih0 k s ax n))
("bendig" nil (b eh1 n d ih0 g))
("bending" nil (b eh1 n d ih0 ng))
("bendix" nil (b eh1 n d ih0 k s))
("bendixen" nil (b ih0 n d ih1 k s ax n))
("bendler" nil (b eh1 n d l er0))
("bendorf" nil (b eh1 n d ao0 r f))
("bends" nil (b eh1 n d z))
("bendt" nil (b eh1 n t))
("bendure" nil (b ey0 n d uh1 r ey0))
("bene" nil (b eh1 n ax))
("beneath" nil (b ih0 n iy1 th))
("benecke" nil (b eh1 n ax k))
("benedek" nil (b eh1 n ax d ih0 k))
("benedetti" nil (b eh1 n ax d eh1 t iy0))
("benedetto" nil (b ih0 n ax d eh1 t ow0))
("benedick" nil (b eh1 n ax d ih0 k))
("benedict" nil (b eh1 n ax d ih1 k t))
("benedicta" nil (b eh1 n ax d iy0 k t ax))
("benedictine" nil (b eh1 n ax d ih1 k t iy0 n))
("benedictines" nil (b eh1 n ax d ih1 k t iy0 n z))
("benedikt" nil (b eh1 n ax d ih0 k t))
("benedix" nil (b eh1 n ax d ih0 k s))
("benefactor" nil (b eh1 n ax f ae1 k t er0))
("benefactors" nil (b eh1 n ax f ae1 k t er0 z))
("beneficence" nil (b ax n eh1 f ax s ax n s))
("beneficent" nil (b eh1 n ax f ih1 sh ax n t))
("beneficial" nil (b eh1 n ax f ih1 sh ax l))
("beneficially" nil (b eh1 n ax f ih1 sh ax l iy0))
("beneficiaries" nil (b eh1 n ax f ih1 sh iy0 eh1 r iy0 z))
("beneficiary" nil (b eh1 n ax f ih1 sh iy0 eh1 r iy0))
("benefiel" nil (b eh1 n ax f iy0 l))
("benefield" nil (b eh1 n ax f iy0 l d))
("benefit" nil (b eh1 n ax f ih0 t))
("benefited" nil (b eh1 n ax f ih1 t ih0 d))
("benefiting" nil (b eh1 n ax f ih0 t ih0 ng))
("benefits" nil (b eh1 n ax f ih0 t s))
("benefitted" nil (b eh1 n ax f ih0 t ih0 d))
("benefitting" nil (b eh1 n ax f ih0 t ih0 ng))
("beneke" nil (b eh1 n ax k))
("benel" nil (b eh1 n ax l))
("benelux" nil (b eh1 n ax l ax k s))
("benenati" nil (b ih0 n ax n aa1 t iy0))
("benequity" nil (b eh1 n eh1 k w ax t iy0))
("benes" nil (b eh1 n iy0 s))
("benesch" nil (b eh1 n ax sh))
("benesh" nil (b eh1 n ax sh))
("benetti" nil (b eh0 n eh1 t iy0))
("benetton" nil (b eh1 n ax t ax n))
("benevento" nil (b eh1 n ax v ey0 n t ow0))
("benevides" nil (b eh1 n ax v iy0 d eh0 s))
("benevolence" nil (b ax n eh1 v ax l ax n s))
("benevolent" nil (b ax n eh1 v ax l ax n t))
("benezra" nil (b eh1 n ax z r ax))
("benfer" nil (b eh1 n f er0))
("benfield" nil (b eh1 n f iy0 l d))
("benford" nil (b eh1 n f er0 d))
("bengal" nil (b eh0 ng g ao1 l))
("bengalis" nil (b eh0 ng g ao1 l iy0 z))
("bengals" nil (b eh1 ng g ax l z))
("benge" nil (b eh1 n jh))
("bengel" nil (b eh1 ng g ax l))
("bengoechea" nil (b eh1 ng g ow0 eh0 ch iy1 ax))
("bengston" nil (b eh1 ng g s t ax n))
("bengt" nil (b eh1 ng k t))
("bengtson" nil (b eh1 ng t s ax n))
("benguet" nil (b eh1 ng g ax t))
("benham" nil (b eh1 n hh ax m))
("benhamou" nil (b eh1 n hh ax m uw1))
("beni" nil (b eh1 n iy0))
("benighted" nil (b ih0 n ay1 t ih0 d))
("benign" nil (b ih0 n ay1 n))
("benigna" nil (b eh0 n iy1 g n ax))
("benignly" nil (b ax n ay1 n l iy0))
("benigno" nil (b eh1 n iy1 n y ow0))
("benihana" nil (b eh1 n ih0 hh aa1 n ax))
("benin" nil (b iy1 n ih0 n))
("beninati" nil (b eh0 n iy0 n aa1 t iy0))
("benincasa" nil (b eh0 n iy0 n k aa1 s ax))
("benish" nil (b eh1 n ih0 sh))
("benita" nil (b ax n iy1 t ax))
("benites" nil (b eh1 n ay0 t s))
("benitez" nil (b ey0 n iy1 t eh0 z))
("benito" nil (b eh0 n iy1 t ow0))
("benjamin" nil (b eh1 n jh ax m ax n))
("benjimen" nil (b eh1 n jh ax m ih0 n))
("benjy" nil (b eh1 n jh iy0))
("benke" nil (b eh1 ng k))
("benker" nil (b eh1 ng k er0))
("benkert" nil (b eh1 ng k er0 t))
("benko" nil (b eh1 ng k ow0))
("benlate" nil (b eh1 n l ey1 t))
("benlox" nil (b eh1 n l aa0 k s))
("benn" nil (b eh1 n))
("bennardo" nil (b ax n aa1 r d ow0))
("benne" nil (b eh1 n))
("bennefield" nil (b eh1 n ih0 f iy0 l d))
("benner" nil (b eh1 n er0))
("bennet" nil (b eh1 n ih0 t))
("bennett" nil (b eh1 n ax t))
("bennette" nil (b ih0 n eh1 t))
("bennetts" nil (b eh1 n ih0 t s))
("benney" nil (b eh1 n iy0))
("bennick" nil (b eh1 n ih0 k))
("bennie" nil (b eh1 n iy0))
("benning" nil (b eh1 n ih0 ng))
("benninger" nil (b eh1 n ih0 ng er0))
("benningfield" nil (b eh1 n ih0 ng f iy0 l d))
("benninghoff" nil (b eh1 n ih0 ng hh ao1 f))
("bennington" nil (b eh1 n ih0 ng t ax n))
("bennink" nil (b eh1 n ih0 ng k))
("bennion" nil (b eh1 n y ax n))
("bennis" nil (b eh1 n ih0 s))
("bennison" nil (b eh1 n ih0 s ax n))
("bennitt" nil (b eh1 n ih0 t))
("benno" nil (b eh1 n ow0))
("benny" nil (b eh1 n iy0))
("beno" nil (b ey1 n ow0))
("benoist" nil (b iy1 n ow0 ih0 s t))
("benoit" nil (b ax n oy1 t))
("benoni" nil (b eh0 n ow1 n iy0))
("benowitz" nil (b eh1 n ax w ih0 t s))
("benoy" nil (b eh1 n oy0))
("bens" nil (b eh1 n z))
("bensalem" nil (b eh1 n s ey1 l ax m))
("bensch" nil (b eh1 n sh))
("benscoter" nil (b eh1 n s k ax t er0))
("bensel" nil (b eh1 n s ax l))
("bensen" nil (b eh1 n s ax n))
("benshoof" nil (b eh1 n sh uh0 f))
("bensing" nil (b eh1 n s ih0 ng))
("bensinger" nil (b eh1 n s ih0 n jh er0))
("benskin" nil (b eh1 n s k ih0 n))
("bensley" nil (b eh1 n s l iy0))
("bensman" nil (b eh1 n s m ax n))
("benson" nil (b eh1 n s ax n))
("bensonhurst" nil (b eh1 n s ax n hh er0 s t))
("benstock" nil (b eh1 n s t aa1 k))
("bent" nil (b eh1 n t))
("bente" nil (b eh1 n t))
("benter" nil (b eh1 n t er0))
("benthall" nil (b eh1 n th ax l))
("bentivegna" nil (b eh0 n t iy0 v eh1 g n ax))
("bentler" nil (b eh1 n t l er0))
("bentley" nil (b eh1 n t l iy0))
("bently" nil (b eh1 n t l iy0))
("bento" nil (b eh1 n t ow0))
("benton" nil (b eh1 n t ax n))
("bentonite" nil (b eh1 n t ax n ay1 t))
("bentonville" nil (b eh1 n t ax n v ih1 l))
("bentsen" nil (b eh1 n t s ax n))
("bentson" nil (b eh1 n t s ax n))
("bentz" nil (b eh1 n t s))
("bentzel" nil (b eh1 n t z ax l))
("bentzen" nil (b eh1 n t z ax n))
("benveniste" nil (b eh0 n v eh0 n iy1 s t iy0))
("benvenuti" nil (b eh0 n v eh0 n uw1 t iy0))
("benvenuto" nil (b eh0 n v eh0 n uw1 t ow0))
("benware" nil (b eh1 n w eh0 r))
("benway" nil (b eh1 n w ey1))
("benyo" nil (b ey1 n y ow0))
("benz" nil (b eh1 n z))
("benzel" nil (b eh1 n z ax l))
("benzene" nil (b eh0 n z iy1 n))
("benzes" nil (b eh1 n z ih0 z))
("benzie" nil (b eh1 n z iy0))
("benziger" nil (b eh1 n z ih0 g er0))
("benzing" nil (b eh1 n z ih0 ng))
("benzinger" nil (b eh1 n z ih0 ng er0))
("beougher" nil (b aw1 f er0))
("beowulf" nil (b ey1 ax w uh1 l f))
("bequeath" nil (b ih0 k w iy1 th))
("bequeathed" nil (b ax k w iy1 th t))
("bequest" nil (b ih0 k w eh1 s t))
("bequests" nil (b ih0 k w eh1 s t s))
("bequette" nil (b ih0 k eh1 t))
("bera" nil (b eh1 r ax))
("beran" nil (b eh1 r ax n))
("beranek" nil (b eh1 r ax n ih0 k))
("berard" nil (b er0 aa1 r d))
("berardi" nil (b er0 aa1 r d iy0))
("berardinelli" nil (b er0 aa0 r d iy0 n eh1 l iy0))
("berardino" nil (b er0 aa0 r d iy1 n ow0))
("berardo" nil (b er0 aa1 r d ow0))
("berarducci" nil (b er0 aa0 r d uw1 ch iy0))
("berate" nil (b ih0 r ey1 t))
("berated" nil (b ih0 r ey1 t ih0 d))
("berating" nil (b ih0 r ey1 t ih0 ng))
("berber" nil (b er1 b er0))
("berberian" nil (b er0 b ih1 r iy0 ax n))
("berberich" nil (b er1 b er0 ih0 k))
("berbick" nil (b er1 b ih0 k))
("bercaw" nil (b er1 k ao0))
("berch" nil (b er1 k))
("berchtold" nil (b er1 k t ow0 l d))
("bercier" nil (b er1 k iy0 er0))
("bercor" nil (b er1 k ao1 r))
("berdahl" nil (b er1 d aa0 l))
("berdan" nil (b er1 d ax n))
("berdine" nil (b er0 d iy1 n iy0))
("bere" nil (b ih1 r))
("berea" nil (b er0 iy1 ax))
("bereaved" nil (b er0 iy1 v d))
("bereavement" nil (b er0 iy1 v m ax n t))
("bereft" nil (b er0 eh1 f t))
("beregovoy" nil (b er0 eh1 g ax v oy1))
("berend" nil (b eh1 r eh0 n d))
("berends" nil (b eh1 r eh0 n d z))
("berendt" nil (b eh1 r ih0 n t))
("berens" nil (b ih1 r ax n z))
("berenson" nil (b eh1 r ih0 n s ax n))
("berent" nil (b eh1 r ax n t))
("beres" nil (b iy1 r z))
("beresford" nil (b ih1 r z f er0 d))
("beret" nil (b eh1 r ax t))
("berets" nil (b eh1 r ax t s))
("beretta" nil (b er0 eh1 t ax))
("berettas" nil (b er0 eh1 t ax s))
("berg" nil (b er1 g))
("bergamini" nil (b er0 g aa0 m iy1 n iy0))
("bergamo" nil (b er0 g aa1 m ow0))
("bergan" nil (b er1 g ax n))
("bergdahl" nil (b er1 g d aa0 l))
("bergdoll" nil (b er1 g d aa1 l))
("bergdorf" nil (b er1 g d ao1 r f))
("berge" nil (b er1 jh))
("bergeman" nil (b er1 g m ax n))
("bergemann" nil (b er1 g m ax n))
("bergen" nil (b er1 g ax n))
("bergendahl" nil (b er1 g eh0 n d aa0 l))
("bergenfield" nil (b er1 g ax n f iy1 l d))
("berger" nil (b er1 g er0))
("bergeron" nil (b er1 g er0 ao0 n))
("bergerson" nil (b er1 g er0 s ax n))
("berges" nil (b er1 jh ih0 z))
("bergeson" nil (b er1 g ih0 s ax n))
("berget" nil (b er1 g eh0 t))
("bergevin" nil (b er1 g eh0 v ih0 n))
("bergey" nil (b er1 jh iy0))
("bergfeld" nil (b er1 g f eh0 l d))
("berggren" nil (b er1 g r eh0 n))
("bergh" nil (b er1 g))
("berghoff" nil (b er1 g hh ao0 f))
("berghuis" nil (b er1 g hh uw0 ih0 z))
("bergin" nil (b er1 g ih1 n))
("bergland" nil (b er1 g l ae1 n d))
("bergling" nil (b er1 g l ih0 ng))
("berglund" nil (b er1 g l ax n d))
("bergman" nil (b er1 g m ax n))
("bergmann" nil (b er1 g m ax n))
("bergner" nil (b er1 g n er0))
("bergquist" nil (b er1 g k w ih0 s t))
("bergren" nil (b er1 g r eh0 n))
("bergs" nil (b er1 g z))
("bergschneider" nil (b er1 g sh n ay0 d er0))
("bergsma" nil (b eh1 r g z m ax))
("bergstedt" nil (b er1 g s t eh0 t))
("bergstein" nil (b er1 g s t ay1 n))
("bergsten" nil (b er1 g s ax n))
("bergstrand" nil (b er1 g s t r ae1 n d))
("bergstresser" nil (b er1 g s t r eh1 s er0))
("bergstrom" nil (b er1 g s t r aa0 m))
("bergthold" nil (b er1 g th ow1 l d))
("bergum" nil (b er1 g ax m))
("berhow" nil (b er1 hh ow0))
("bering" nil (b eh1 r ih0 ng))
("beringer" nil (b eh1 r ih0 ng er0))
("berisford" nil (b eh1 r ih0 s f er0 d))
("berish" nil (b er1 ih0 sh))
("berjaya" nil (b er0 jh ay1 ax))
("berk" nil (b er1 k))
("berka" nil (b er1 k ax))
("berke" nil (b er1 k))
("berkebile" nil (b er1 k ih0 b ax l))
("berkel" nil (b er1 k ax l))
("berkeley" nil (b er1 k l iy0))
("berkelman" nil (b er1 k ax l m ax n))
("berkemeier" nil (b er1 k ih0 m ay0 er0))
("berken" nil (b er1 k ax n))
("berkery" nil (b er1 k er0 iy0))
("berkes" nil (b er1 k s))
("berkey" nil (b er1 k iy0))
("berkheimer" nil (b er1 k hh ay0 m er0))
("berkland" nil (b er1 k l ax n d))
("berklee" nil (b er1 k l iy0))
("berkley" nil (b er1 k l iy0))
("berkline" nil (b er1 k l ay1 n))
("berkman" nil (b er1 k m ax n))
("berko" nil (b er1 k ow0))
("berkoff" nil (b er1 k ao0 f))
("berkovitz" nil (b er1 k ax v ih0 t s))
("berkowitz" nil (b er1 k ax w ih0 t s))
("berkshire" nil (b er1 k sh er0))
("berkshires" nil (b er1 k sh ih1 r z))
("berkson" nil (b er1 k s ax n))
("berkstresser" nil (b er1 k s t r ih0 s er0))
("berlack" nil (b er1 l ae0 k))
("berland" nil (b er1 l ax n d))
("berlanga" nil (b er0 l aa1 ng g ax))
("berlascone" nil (b eh1 r l ax s k ow1 n iy0))
("berle" nil (b er1 l))
("berlex" nil (b er1 l eh1 k s))
("berlin" nil (b er0 l ih1 n))
("berliner" nil (b er0 l ih1 n er0))
("berliners" nil (b er0 l ih1 n er0 z))
("berling" nil (b er1 l ih0 ng))
("berlinger" nil (b er1 l ih0 ng er0))
("berlitz" nil (b er0 l ih1 t s))
("berlottes" nil (b er0 l aa1 t s))
("berls" nil (b er1 l z))
("berlusconi" nil (b er1 l ax s k ow1 n iy0))
("berm" nil (b er1 m))
("berman" nil (b er1 m ax n))
("bermans" nil (b er1 m ax n z))
("bermea" nil (b eh1 r m iy0 ax))
("bermel" nil (b er1 m ax l))
("bermingham" nil (b er1 m ih0 ng hh ae0 m))
("bermuda" nil (b er0 m y uw1 d ax))
("bermudez" nil (b er0 m y uw1 d eh1 z))
("bern" nil (b er1 n))
("berna" nil (b eh1 r n ax))
("bernabe" nil (b er1 n ax b))
("bernabei" nil (b er1 n ax b ay0))
("bernacki" nil (b er0 n aa1 t s k iy0))
("bernadene" nil (b er1 n ax d iy0 n))
("bernadette" nil (b er0 n ax d eh1 t))
("bernadine" nil (b er0 n ax d iy1 n))
("bernadino" nil (b er1 n ax d iy1 n ow0))
("bernal" nil (b er1 n ax l))
("bernama" nil (b er0 n aa1 m ax))
("bernard" nil (b er0 n aa1 r d))
("bernardi" nil (b er0 n aa1 r d iy0))
("bernardin" nil (b er1 n aa0 r d iy0 n))
("bernardini" nil (b er0 n aa0 r d iy1 n iy0))
("bernardino" nil (b er0 n ax d iy1 n ow0))
("bernardo" nil (b er0 n aa1 r d ow0))
("bernards" nil (b er0 n aa1 r d z))
("bernardy" nil (b er0 n aa1 r d iy0))
("bernas" nil (b er1 n ax z))
("bernasconi" nil (b er0 n aa0 s k ow1 n iy0))
("bernat" nil (b er1 n ax t))
("bernath" nil (b er1 n ax th))
("bernauer" nil (b er1 n aw0 er0))
("bernbach" nil (b er1 n b aa1 k))
("bernd" nil (b er1 n t))
("berndt" nil (b er1 n t))
("berne" nil (b er1 n))
("berneice" nil (b er0 n ay1 s))
("berner" nil (b er1 n er0))
("bernet" nil (b er0 n eh1 t))
("bernett" nil (b er1 n ih0 t))
("berney" nil (b er1 n iy0))
("bernhagen" nil (b er1 n hh ax g ax n))
("bernhard" nil (b er1 n hh aa0 r d))
("bernhardt" nil (b er1 n hh aa0 r t))
("bernhart" nil (b er1 n hh aa0 r t))
("bernheim" nil (b er1 n hh ay0 m))
("berni" nil (b eh1 r n iy0))
("bernia" nil (b eh1 r n iy0 ax))
("bernice" nil (b er0 n iy1 s))
("bernick" nil (b er1 n ih0 k))
("bernie" nil (b er1 n iy0))
("bernier" nil (b er1 n iy0 er0))
("berning" nil (b er1 n ih0 ng))
("berninger" nil (b er1 n ih0 ng er0))
("bernita" nil (b er0 n iy1 t ax))
("bernoulli" nil (b er0 n uw1 l iy0))
("berns" nil (b er1 n z))
("bernsen" nil (b er1 n s ax n))
("bernson" nil (b er1 n s ax n))
("bernstein" nil (b er1 n s t ay0 n))
("bernsteins" nil (b er1 n s t ay0 n z))
("bernt" nil (b er1 n t))
("berntsen" nil (b er1 n t s ax n))
("berntson" nil (b er1 n t s ax n))
("berny" nil (b er1 n iy0))
("bero" nil (b eh1 r ow0))
("beron" nil (b eh1 r ax n))
("berquist" nil (b er1 k w ih0 s t))
("berra" nil (b eh1 r ax))
("berrard" nil (b eh1 r aa1 r d))
("berres" nil (b eh1 r z))
("berreth" nil (b eh1 r ih0 th))
("berrett" nil (b eh1 r ih0 t))
("berrey" nil (b eh1 r iy0))
("berri" nil (b eh1 r iy0))
("berrian" nil (b eh1 r iy0 ax n))
("berridge" nil (b eh1 r ih0 jh))
("berrie" nil (b eh1 r iy0))
("berrien" nil (b eh1 r iy0 ax n))
("berrier" nil (b eh1 r iy0 er0))
("berries" nil (b eh1 r iy0 z))
("berrigan" nil (b eh1 r ih0 g ax n))
("berrill" nil (b eh1 r ax l))
("berringer" nil (b eh1 r ih0 ng er0))
("berrios" nil (b eh0 r iy1 ow0 z))
("berrong" nil (b eh1 r ao0 ng))
("berry" nil (b eh1 r iy0))
("berryhill" nil (b eh1 r iy0 hh ih1 l))
("berrylike" nil (b eh1 r iy0 l ay1 k))
("berryman" nil (b eh1 r iy0 m ax n))
("bersch" nil (b er1 sh))
("berserk" nil (b er0 s er1 k))
("bershad" nil (b er0 sh aa1 d))
("berson" nil (b er1 s ax n))
("berst" nil (b er1 s t))
("bert" nil (b er1 t))
("berta" nil (b er1 t ax))
("bertch" nil (b er1 ch))
("berte" nil (b er1 t))
("bertelli" nil (b er0 t eh1 l iy0))
("bertels" nil (b er1 t ax l z))
("bertelsen" nil (b er1 t ih0 l s ax n))
("bertelsmann" nil (b er1 t ax l z m ax n))
("bertelson" nil (b er1 t ih0 l s ax n))
("berth" nil (b er1 th))
("bertha" nil (b er1 th ax))
("berthelot" nil (b er1 th ax l aa0 t))
("berthelsen" nil (b er1 th ax l s ax n))
("berthiaume" nil (b er1 th iy0 uw1 m iy0))
("berthold" nil (b er1 th ow1 l d))
("bertholf" nil (b er1 th ow0 l f))
("berths" nil (b er1 th s))
("berti" nil (b eh1 r t iy0))
("bertie" nil (b er1 t iy0))
("bertil" nil (b er1 t ih0 l))
("bertilde" nil (b er1 t ih0 l d))
("bertin" nil (b er1 t ih0 n))
("bertini" nil (b er0 t iy1 n iy0))
("bertino" nil (b er0 t iy1 n ow0))
("bertke" nil (b er1 t k iy0))
("bertling" nil (b er1 t ax l ih0 ng))
("bertold" nil (b er1 t ow0 l d))
("bertoldi" nil (b er0 t ow1 l d iy0))
("bertoli" nil (b er0 t ow1 l iy0))
("bertolini" nil (b er0 t ow0 l iy1 n iy0))
("bertolino" nil (b er0 t ow0 l iy1 n ow0))
("bertolotti" nil (b er0 t ow0 l ow1 t iy0))
("bertolucci" nil (b er1 t ow0 l uw1 ch iy0))
("berton" nil (b er1 t ax n))
("bertone" nil (b er0 t ow1 n iy0))
("bertoni" nil (b er0 t ow1 n iy0))
("bertrade" nil (b er1 t r ax d))
("bertram" nil (b er1 t r ax m))
("bertran" nil (b er1 t r ax n))
("bertrand" nil (b er1 t r ax n d))
("bertsch" nil (b er1 ch))
("bertsche" nil (b er1 ch))
("bertucci" nil (b er0 t uw1 ch iy0))
("berty" nil (b er1 t iy0))
("bertz" nil (b er1 t s))
("berube" nil (b eh1 r uw0 b))
("berumen" nil (b ih1 r uw0 m eh0 n))
("berwald" nil (b er1 w ao0 l d))
("berwanger" nil (b er1 w ao0 ng er0))
("berwick" nil (b er1 w ih0 k))
("berwyn" nil (b er1 w ih0 n))
("beryl" nil (b eh1 r ax l))
("beryllium" nil (b er0 ih1 l iy0 ax m))
("berzins" nil (b er1 z ih0 n z))
("besancon" nil (b ih0 s ae1 n k ax n))
("besant" nil (b eh1 z ax n t))
("besaw" nil (b iy1 s ao0))
("besch" nil (b eh1 sh))
("beschloss" nil (b eh1 sh l ao1 s))
("besecker" nil (b eh1 s ih0 k er0))
("beseiged" nil (b ih0 s iy1 jh d))
("besemer" nil (b eh1 s iy0 m er0))
("beset" nil (b ih0 s eh1 t))
("besetting" nil (b ih0 s eh1 t ih0 ng))
("beshara" nil (b ih0 sh aa1 r ax))
("besharov" nil (b eh1 sh er0 aa0 v))
("beshears" nil (b eh1 sh ih0 r z))
("beshore" nil (b eh1 sh er0))
("beside" nil (b ih0 s ay1 d))
("besides" nil (b ih0 s ay1 d z))
("besiege" nil (b ih0 s iy1 jh))
("besieged" nil (b ih0 s iy1 jh d))
("beske" nil (b eh1 s k))
("besler" nil (b eh1 s ax l er0))
("besner" nil (b r eh1 s n er0))
("bespeak" nil (b ih0 s p iy1 k))
("bespeaks" nil (b ih0 s p iy1 k s))
("bespectacled" nil (b ih0 s p eh1 k t ax k ax l d))
("bess" nil (b eh1 s))
("besse" nil (b eh1 s))
("bessemer" nil (b eh1 s ax m er0))
("bessent" nil (b eh1 s ax n t))
("besser" nil (b eh1 s er0))
("bessert" nil (b eh1 s er0 t))
("bessette" nil (b ih0 s eh1 t))
("bessey" nil (b eh1 s iy0))
("bessie" nil (b eh1 s iy0))
("bessinger" nil (b eh1 s ih0 n jh er0))
("bessire" nil (b eh0 s ih1 r iy0))
("bessler" nil (b eh1 s l er0))
("besso" nil (b eh1 s ow0))
("besson" nil (b eh1 s ax n))
("bessy" nil (b eh1 s iy0))
("best" nil (b eh1 s t))
("beste" nil (b eh1 s t))
("bested" nil (b eh1 s t ih0 d))
("bester" nil (b eh1 s t er0))
("bestial" nil (b eh1 s ch ax l))
("besting" nil (b eh1 s t ih0 ng))
("bestow" nil (b ih0 s t ow1))
("bestowed" nil (b ih0 s t ow1 d))
("bestowing" nil (b ih0 s t ow1 ih0 ng))
("bestows" nil (b ih0 s t ow1 z))
("bestrode" nil (b ih0 s t r ow1 d))
("bestseller" nil (b eh1 s t s eh1 l er0))
("bestsellers" nil (b eh1 s t s eh1 l er0 z))
("bestselling" nil (b eh1 s t s eh1 l ih0 ng))
("bestul" nil (b eh1 s t ax l))
("bestwick" nil (b eh1 s t w ih1 k))
("beswick" nil (b eh1 s w ih0 k))
("bet" nil (b eh1 t))
("beta" nil (b ey1 t ax))
("betamax" nil (b ey1 t ax m ae0 k s))
("betancourt" nil (b eh1 t ax n k ao0 r t))
("betancur" nil (b ax t ae1 ng k er0))
("betas" nil (b ey1 t ax z))
("betaseron" nil (b ey1 t ax s eh1 r aa1 n))
("betavon" nil (b eh1 t ax v aa0 n))
("betcher" nil (b eh1 ch er0))
("betel" nil (b eh1 t ax l))
("beteta" nil (b eh0 t ey1 t ax))
("beth" nil (b eh1 th))
("bethany" nil (b eh1 th ax n iy0))
("bethard" nil (b eh1 th er0 d))
("bethards" nil (b eh1 th er0 d z))
("bethea" nil (b eh1 dh iy0 ax))
("bethel" nil (b eh1 th ax l))
("bethell" nil (b eh1 th ax l))
("bethesda" nil (b ax th eh1 z d ax))
("bethke" nil (b eh1 th k))
("bethlehem" nil (b eh1 th l ih0 hh eh1 m))
("bethpage" nil (b eh1 th p ey1 jh))
("bethseda" nil (b eh0 th s eh1 d ax))
("bethune" nil (b ax th y uw1 n))
("betide" nil (b ih0 t ay1 d))
("betke" nil (b eh1 t k iy0))
("betker" nil (b eh1 t k er0))
("betley" nil (b eh1 t l iy0))
("betray" nil (b ih0 t r ey1))
("betrayal" nil (b ih0 t r ey1 ax l))
("betrayals" nil (b ih0 t r ey1 ax l z))
("betrayed" nil (b ih0 t r ey1 d))
("betraying" nil (b ih0 t r ey1 ih0 ng))
("betrays" nil (b ih0 t r ey1 z))
("bets" nil (b eh1 t s))
("betsch" nil (b eh1 ch))
("betschart" nil (b eh1 t s hh aa1 r t))
("betsey" nil (b eh1 t s iy0))
("betsill" nil (b eh1 t s ax l))
("betsy" nil (b eh1 t s iy0))
("bett" nil (b eh1 t))
("betta" nil (b eh1 t ax))
("bettcher" nil (b eh1 t ch er0))
("bette" nil (b eh1 t iy0))
("betten" nil (b eh1 t ax n))
("bettenberg" nil (b eh1 t ax n b er0 g))
("bettencourt" nil (b eh1 t ih0 n k ao0 r t))
("bettendorf" nil (b eh1 t ih0 n d ao0 r f))
("bettenhausen" nil (b eh1 t ih0 n hh aw0 z ax n))
("better" nil (b eh1 t er0))
("bettered" nil (b eh1 t er0 d))
("betteridge" nil (b eh1 t er0 ih0 jh))
("bettering" nil (b eh1 t er0 ih0 ng))
("betterment" nil (b eh1 t er0 m ax n t))
("betters" nil (b eh1 t er0 z))
("betterton" nil (b eh1 t er0 t ax n))
("bettes" nil (b eh1 t iy0 z))
("betti" nil (b eh1 t iy0))
("bettin" nil (b eh1 t ih0 n))
("bettina" nil (b ax t iy1 n ax))
("betting" nil (b eh1 t ih0 ng))
("bettinger" nil (b eh1 t ih0 ng er0))
("bettini" nil (b eh0 t iy1 n iy0))
("bettino" nil (b eh0 t iy1 n ow0))
("bettis" nil (b eh1 t ih0 s))
("bettman" nil (b eh1 t m ax n))
("bettner" nil (b eh1 t n er0))
("betton" nil (b eh1 t ax n))
("bettor" nil (b eh1 t er0))
("bettors" nil (b eh1 t er0 z))
("betts" nil (b eh1 t s))
("betty" nil (b eh1 t iy0))
("between" nil (b ih0 t w iy1 n))
("betweens" nil (b ax t w iy1 n z))
("betz" nil (b eh1 t s))
("betzer" nil (b eh1 t z er0))
("betzler" nil (b eh1 t s l er0))
("betzold" nil (b eh1 t z ow0 l d))
("beu" nil (b uw1))
("beucler" nil (b oy1 k ax l er0))
("beukema" nil (b uw0 k iy1 m ax))
("beula" nil (b uw1 l ax))
("beulah" nil (b y uw1 l ax))
("beumer" nil (b iy1 ax m er0))
("beutel" nil (b y uw0 t eh1 l))
("beuthin" nil (b y uw1 th ih0 n))
("beutler" nil (b oy1 t ax l er0))
("beuys" nil (b y uw1 iy0 z))
("bev" nil (b eh1 v))
("bevacqua" nil (b eh0 v aa1 k w ax))
("bevan" nil (b eh1 v ax n))
("bevans" nil (b eh1 v ax n z))
("bevard" nil (b ih0 v aa1 r d))
("bevel" nil (b eh1 v ax l))
("bevelled" nil (b eh1 v ax l d))
("beven" nil (b eh1 v ax n))
("bevens" nil (b iy1 v ax n z))
("bever" nil (b eh1 v er0))
("beverage" nil (b eh1 v er0 ih0 jh))
("beverages" nil (b eh1 v r ih0 jh ih0 z))
("beveridge" nil (b eh1 v er0 ih0 jh))
("beverley" nil (b eh1 v er0 l iy0))
("beverlin" nil (b eh1 v er0 l ih0 n))
("beverly" nil (b eh1 v er0 l iy0))
("bevers" nil (b eh1 v er0 z))
("bevier" nil (b eh1 v iy0 er0))
("bevil" nil (b eh1 v ax l))
("bevilacqua" nil (b eh0 v iy0 l aa1 k w ax))
("bevill" nil (b eh1 v ax l))
("beville" nil (b iy1 v ih0 l))
("bevin" nil (b eh1 v ih0 n))
("bevington" nil (b eh1 v ih0 ng t ax n))
("bevins" nil (b eh1 v ih0 n z))
("bevis" nil (b eh1 v ih0 s))
("bevmark" nil (b eh1 v m aa1 r k))
("bevy" nil (b eh1 v iy0))
("beware" nil (b ih0 w eh1 r))
("bewildered" nil (b ih0 w ih1 l d er0 d))
("bewildering" nil (b ih0 w ih1 l d er0 ih0 ng))
("bewilderment" nil (b ih0 w ih1 l d er0 m ax n t))
("bewilders" nil (b ih0 w ih1 l d er0 z))
("bewitched" nil (b ih0 w ih1 ch t))
("bewley" nil (b y uw1 l iy0))
("bexley" nil (b eh1 k s l iy0))
("bey" nil (b ey1))
("beyer" nil (b ey1 er0))
("beyerle" nil (b ay1 r ax l))
("beyerlein" nil (b ay1 r l ay0 n))
("beyers" nil (b ey1 er0 z))
("beyersdorf" nil (b ay1 r s d ao0 r f))
("beyl" nil (b ey1 l))
("beymer" nil (b ey1 m er0))
("beynon" nil (b ey1 n ax n))
("beyond" nil (b ih0 aa1 n d))
("beytout" nil (b ey1 t aw1 t))
("bezaire" nil (b ax z eh1 r))
("bezanson" nil (b eh1 z ax n s ax n))
("bezdek" nil (b eh1 z d ih0 k))
("bezek" nil (b eh1 z eh0 k))
("bezner" nil (b eh1 z n er0))
("bezold" nil (b eh1 z ow0 l d))
("bhakta" nil (b aa1 k t ax))
("bhatia" nil (b aa1 sh ax))
("bhatt" nil (b ae1 t))
("bhattacharjy" nil (b aa1 t ax ch aa1 r jh iy0))
("bhatti" nil (b aa1 t iy0))
("bhirud" nil (b ih1 r ax d))
("bhopal" nil (b ow0 p aa1 l))
("bhutan" nil (b uw1 t ae1 n))
("bhutto" nil (b uw1 t ow0))
("bi" nil (b ay1))
("biaggi" nil (b iy0 ae1 jh iy0))
("biaggini" nil (b iy1 ax g iy1 n iy0))
("biagi" nil (b iy0 aa1 jh iy0))
("biagini" nil (b iy0 ax jh iy1 n iy0))
("biagioni" nil (b iy0 aa1 jh iy0 ow1 n iy0))
("bialas" nil (b iy0 aa1 l ax s))
("bialecki" nil (b iy0 ax l eh1 k iy0))
("bialek" nil (b iy0 aa1 l eh0 k))
("bialik" nil (b iy0 aa1 l ih0 k))
("bialkin" nil (b iy0 aa1 l k ax n))
("bialkowski" nil (b iy0 ax l k aw1 s k iy0))
("bialy" nil (b iy0 aa1 l iy0))
("biamby" nil (b iy0 aa1 m b iy0))
("biamonte" nil (b iy0 ax m ao1 n t iy0))
("bianca" nil (b iy0 aa1 ng k ax))
("bianchi" nil (b iy0 aa1 n ch iy0))
("bianchini" nil (b iy0 aa0 n ch iy1 n iy0))
("bianco" nil (b iy0 aa1 ng k ow0))
("bianconi" nil (b iy0 aa0 ng k ow1 n iy0))
("bianculli" nil (b iy0 aa0 ng k uw1 l iy0))
("biannual" nil (b ay0 ae1 n uw0 ax l))
("bias" nil (b ay1 ax s))
("biased" nil (b ay1 ax s t))
("biases" nil (b ay1 ax s ih0 z))
("biasi" nil (b iy0 aa1 s iy0))
("biaxial" nil (b ay0 ae1 k s iy0 ax l))
("bib" nil (b ih1 b))
("biba" nil (b iy1 b ax))
("bibb" nil (b ih1 b))
("bibbee" nil (b ih1 b iy1))
("bibbins" nil (b ih1 b ih0 n z))
("bibbo" nil (b iy1 b ow0))
("bibbs" nil (b ih1 b z))
("bibby" nil (b ih1 b iy0))
("bibeau" nil (b ih0 b ow1))
("bibeault" nil (b ih0 b ow1))
("bibee" nil (b ih0 b iy1))
("biber" nil (b ay1 b er0))
("bible" nil (b ay1 b ax l))
("bibler" nil (b ay1 b ax l er0))
("bibles" nil (b ay1 b ax l z))
("biblical" nil (b ih1 b l ax k ax l))
("bibliographies" nil (b ih1 b l iy0 aa1 g r ax f iy0 z))
("bibliography" nil (b ih1 b l iy0 aa1 g r ax f iy0))
("bibs" nil (b ih1 b z))
("biby" nil (b ay1 b iy0))
("bic" nil (b ih1 k))
("bicarbonate" nil (b ay0 k aa1 r b ax n ax t))
("bice" nil (b ay1 s))
("bicentennial" nil (b ay1 s eh0 n t eh1 n iy0 ax l))
("bichler" nil (b ih1 k ax l er0))
("bichsel" nil (b ih1 k s ax l))
("bick" nil (b ih1 k))
("bickel" nil (b ih1 k ax l))
("bickell" nil (b ih1 k ax l))
("bicker" nil (b ih1 k er0))
("bickered" nil (b ih1 k er0 d))
("bickering" nil (b ih1 k er0 ih0 ng))
("bickers" nil (b ih1 k er0 z))
("bickerstaff" nil (b ih1 k er0 s t ae1 f))
("bickert" nil (b ih1 k er0 t))
("bickerton" nil (b ih1 k er0 t ax n))
("bickett" nil (b ih1 k ih0 t))
("bickford" nil (b ih1 k f er0 d))
("bickham" nil (b ih1 k hh ax m))
("bickhart" nil (b ih1 k hh aa1 r t))
("bicking" nil (b ih1 k ih0 ng))
("bickle" nil (b ih1 k ax l))
("bickler" nil (b ih1 k l er0))
("bickley" nil (b ih1 k l iy0))
("bickmore" nil (b ih1 k m ao0 r))
("bicknell" nil (b ih1 k n ax l))
("bickner" nil (b ih1 k n er0))
("bicksler" nil (b ih1 k s l er0))
("bickwit" nil (b ih1 k w ih0 t))
("bicoastal" nil (b ih0 k ow1 s t ax l))
("bicuspids" nil (b ay0 k ah1 s p ax d z))
("bicycle" nil (b ay1 s ih0 k ax l))
("bicycled" nil (b ay1 s ih0 k ax l d))
("bicycles" nil (b ay1 s ih0 k ax l z))
("bicycling" nil (b ay1 s ih1 k ax l ih0 ng))
("bid" nil (b ih1 d))
("bidco" nil (b ih1 d k ow0))
("bidcos" nil (b ih1 d k ow0 s))
("bidder" nil (b ih1 d er0))
("bidders" nil (b ih1 d er0 z))
("biddie" nil (b ih1 d iy0))
("bidding" nil (b ih1 d ih0 ng))
("biddinger" nil (b ih1 d ih0 ng er0))
("biddison" nil (b ih1 d ih0 s ax n))
("biddix" nil (b ih1 d ih0 k s))
("biddle" nil (b ih1 d ax l))
("biddy" nil (b ih1 d iy0))
("bide" nil (b ay1 d))
("biden" nil (b ay1 d ax n))
("biderman" nil (b ay1 d er0 m ax n))
("bidgood" nil (b ih1 d g uh1 d))
("biding" nil (b ay1 d ih0 ng))
("bidinger" nil (b ay1 d ih0 ng er0))
("bidlack" nil (b ih1 d l ae1 k))
("bids" nil (b ih1 d z))
("bidwell" nil (b ih1 d w eh1 l))
("bidwill" nil (b ih1 d w ih1 l))
("biebel" nil (b iy1 b ax l))
("bieber" nil (b iy1 b er0))
("bieda" nil (b iy1 d ax))
("biederman" nil (b iy1 d er0 m ax n))
("biedermann" nil (b ay1 d er0 m ax n))
("biedermeier" nil (b iy1 d er0 m ay1 r))
("biedrzycki" nil (b ih0 jh ih1 t s k iy0))
("biegel" nil (b iy1 g ax l))
("biegler" nil (b iy1 g ax l er0))
("biehl" nil (b iy1 l))
("biehle" nil (b iy1 hh ax l))
("biehler" nil (b iy1 l er0))
("biehn" nil (b iy1 n))
("bieker" nil (b iy1 k er0))
("biel" nil (b iy1 l))
("biela" nil (b iy1 l ax))
("bielak" nil (b iy1 l ax k))
("bielanski" nil (b iy0 l ae1 n s k iy0))
("bielat" nil (b iy1 l ax t))
("bielawski" nil (b iy0 l aa1 f s k iy0))
("bielby" nil (b iy1 l b iy0))
("bielecki" nil (b iy0 l eh1 t s k iy0))
("bielefeld" nil (b iy1 l ih0 f eh0 l d))
("bielefeldt" nil (b iy1 l ih0 f ih0 l t))
("bielen" nil (b iy1 l ax n))
("bielenberg" nil (b iy1 l ax n b er0 g))
("bieler" nil (b iy1 l er0))
("bielicki" nil (b ih0 l ih1 t s k iy0))
("bielinski" nil (b ih0 l ih1 n s k iy0))
("bielke" nil (b iy1 l k))
("bielski" nil (b iy1 l s k iy0))
("bien" nil (b iy1 n))
("bieniek" nil (b ih1 n iy0 eh0 k))
("bienkowski" nil (b ih0 ng k ao1 f s k iy0))
("biennale" nil (b iy0 eh1 n ey1 l))
("biennial" nil (b ay0 eh1 n iy0 ax l))
("bienstock" nil (b iy1 n s t aa1 k))
("bienvenu" nil (b ax n v eh1 n uw0))
("bienvenue" nil (b ax n v ey1 n w eh0))
("bier" nil (b iy1 r))
("bierbauer" nil (b ih1 r b aw0 er0))
("bierbaum" nil (b ih1 r b aw0 m))
("bierbusse" nil (b ih1 r b ax s))
("bierce" nil (b ih1 r s))
("bierer" nil (b ih1 r er0))
("bieri" nil (b ih1 r iy0))
("bierlein" nil (b ih1 r l ay0 n))
("bierley" nil (b ih1 r l iy0))
("bierly" nil (b ih1 r l iy0))
("bierma" nil (b ih1 r m ax))
("bierman" nil (b ih1 r m ax n))
("biermann" nil (b ih1 r m ax n))
("biernacki" nil (b ih0 r n aa1 t s k iy0))
("biernat" nil (b ih0 r n ae1 t))
("biers" nil (b iy1 r z))
("bierwirth" nil (b ih1 r w er0 th))
("biery" nil (b ih1 r iy0))
("bies" nil (b ay1 z))
("biesecker" nil (b iy1 s ih0 k er0))
("bieser" nil (b iy1 s er0))
("bietz" nil (b iy1 t s))
("biever" nil (b iy1 v er0))
("bifano" nil (b ih0 f aa1 n ow0))
("biff" nil (b ih1 f))
("biffle" nil (b ih1 f ax l))
("bifocals" nil (b ay1 f ow0 k ax l z))
("bifulco" nil (b ih0 f uw1 l k ow0))
("bifurcated" nil (b ih1 f er0 k ey1 t ih0 d))
("bifurcation" nil (b ih1 f er0 k ey1 sh ax n))
("big" nil (b ih1 g))
("bigbee" nil (b ih1 g b iy1))
("bigbie" nil (b ih1 g b iy0))
("bigby" nil (b ih1 g b iy0))
("bigelow" nil (b ih1 g ax l ow1))
("bigeyes" nil (b ih1 g ay1 z))
("bigfoot" nil (b ih1 g f uh1 t))
("bigford" nil (b ih1 g f er0 d))
("biggar" nil (b ih1 g er0))
("bigger" nil (b ih1 g er0))
("biggers" nil (b ih1 g er0 z))
("biggerstaff" nil (b ih1 g er0 s t ae1 f))
("biggest" nil (b ih1 g ax s t))
("biggie" nil (b ih1 g iy0))
("biggies" nil (b ih1 g iy0 z))
("biggins" nil (b ih1 g ih0 n z))
("biggio" nil (b iy1 jh iy0 ow0))
("biggs" nil (b ih1 g z))
("bigham" nil (b ay1 g hh ax m))
("bighorn" nil (b ih1 g hh ao1 r n))
("bighorns" nil (b ih1 g hh ao1 r n z))
("bigler" nil (b ay1 g ax l er0))
("bigley" nil (b ih1 g l iy0))
("biglin" nil (b ih1 g l ih0 n))
("biglow" nil (b ih1 g l ow0))
("bignell" nil (b ih0 g n eh1 l))
("bigness" nil (b ih1 g n ax s))
("bigos" nil (b iy1 g ow0 z))
("bigot" nil (b ih1 g ax t))
("bigoted" nil (b ih1 g ax t ih0 d))
("bigotry" nil (b ih1 g ax t r iy0))
("bigots" nil (b ih1 g ax t s))
("bigs" nil (b ih1 g z))
("bigsby" nil (b ih1 g z b iy0))
("bigwig" nil (b ih1 g w ih1 g))
("bigwigs" nil (b ih1 g w ih1 g z))
("bigwood" nil (b ih1 g w uh1 d))
("bihac" nil (b iy1 hh aa1 ch))
("bihari" nil (b ih0 hh aa1 r iy0))
("bihl" nil (b ih1 l))
("bihm" nil (b ih1 m))
("bihn" nil (b ih1 n))
("bijan" nil (b ih1 jh ax n))
("bijur" nil (b iy0 zh uh1 r))
("bike" nil (b ay1 k))
("biked" nil (b ay1 k t))
("biker" nil (b ay1 k er0))
("bikers" nil (b ay1 k er0 z))
("bikes" nil (b ay1 k s))
("bikin" nil (b ih1 k ih0 n))
("biking" nil (b ay1 k ih0 ng))
("bikini" nil (b ih0 k iy1 n iy0))
("bikinis" nil (b ax k iy1 n iy0 z))
("biko" nil (b iy1 k ow0))
("bil" nil (b ih1 l))
("bila" nil (b iy1 l ax))
("bilateral" nil (b ay0 l ae1 t er0 ax l))
("bilaterally" nil (b ay0 l ae1 t er0 ax l iy0))
("bilbao" nil (b ih0 l b aw1))
("bilbo" nil (b ih1 l b ow1))
("bilbrey" nil (b ih1 l b r iy0))
("bilbro" nil (b iy1 l b r ow0))
("bilby" nil (b ih1 l b iy0))
("bild" nil (b ih1 l d))
("bilderback" nil (b ay1 l d er0 b ae0 k))
("bildner" nil (b ih1 l d n er0))
("bile" nil (b ay1 l))
("bilek" nil (b ih1 l eh0 k))
("bilello" nil (b ih0 l eh1 l ow0))
("biles" nil (b ay1 l z))
("bilger" nil (b ih1 l g er0))
("bilicki" nil (b ih0 l ih1 t s k iy0))
("bilings" nil (b ay1 l ih0 ng z))
("bilingual" nil (b ay0 l ih1 ng g w ax l))
("bilingualism" nil (b ay0 l ih1 ng g w ax l ih1 z ax m))
("bilinski" nil (b ih0 l ih1 n s k iy0))
("bilious" nil (b ih1 l iy0 ax s))
("bilirakis" nil (b ih0 l ih1 r ax k ih0 s))
("bilk" nil (b ih1 l k))
("bilka" nil (b ih1 l k ax))
("bilked" nil (b ih1 l k t))
("bilking" nil (b ih1 l k ih0 ng))
("bill" nil (b ih1 l))
("billable" nil (b ih1 l ax b ax l))
("billancourt" nil (b ih1 l ax n k ao1 r t))
("billard" nil (b ih0 l aa1 r d))
("billboard" nil (b ih1 l b ao1 r d))
("billboards" nil (b ih1 l b ao1 r d z))
("bille" nil (b ay1 l))
("billed" nil (b ih1 l d))
("biller" nil (b ih1 l er0))
("billerica" nil (b ih1 l er0 iy1 k ax))
("billes" nil (b ih1 l z))
("billet" nil (b ih1 l ax t))
("billeter" nil (b ih1 l iy0 t er0))
("billets" nil (b ih1 l ax t s))
("billett" nil (b ih1 l ih0 t))
("billfold" nil (b ih1 l f ow1 l d))
("billiard" nil (b ih1 l y er0 d))
("billiards" nil (b ih1 l y er0 d z))
("billick" nil (b ih1 l ih0 k))
("billie" nil (b ih1 l iy0))
("billig" nil (b ih1 l ih0 g))
("billing" nil (b ih1 l ih0 ng))
("billinger" nil (b ih1 l ih0 ng er0))
("billingham" nil (b ih1 l ih0 ng hh ae1 m))
("billings" nil (b ih1 l ih0 ng z))
("billingslea" nil (b ih1 l ih0 ng z l iy0))
("billingsley" nil (b ih1 l ih0 ng z l iy0))
("billingsly" nil (b ih1 l ih0 ng z l iy0))
("billington" nil (b ih1 l ih0 ng t ax n))
("billion" nil (b ih1 l y ax n))
("billionaire" nil (b ih1 l y ax n eh1 r))
("billionaires" nil (b ih1 l y ax n eh1 r z))
("billions" nil (b ih1 l y ax n z))
("billionth" nil (b ih1 l y ax n th))
("billionths" nil (b ih1 l y ax n th s))
("billiot" nil (b ih1 l iy0 aa0 t))
("billips" nil (b ih1 l ih0 p s))
("billiter" nil (b ih1 l iy0 t er0))
("billiton" nil (b ih1 l ih0 t ax n))
("billman" nil (b ih1 l m ax n))
("billmeyer" nil (b ih1 l m ay0 er0))
("billon" nil (b ih1 l ax n))
("billow" nil (b ih1 l ow0))
("billowed" nil (b ih1 l ow0 d))
("billowing" nil (b ih1 l ow0 ih0 ng))
("billows" nil (b ih1 l ow0 z))
("bills" nil (b ih1 l z))
("billups" nil (b ih1 l ax p s))
("billy" nil (b ih1 l iy0))
("bilodeau" nil (b ih1 l ax d ow0))
("bilotta" nil (b ih0 l ow1 t ax))
("bilotti" nil (b ih0 l aa1 t iy0))
("bilow" nil (b ih1 l ow0))
("biloxi" nil (b ax l ah1 k s iy0))
("bilski" nil (b ih1 l s k iy0))
("bilsky" nil (b ih1 l s k iy0))
("bilson" nil (b ih1 l s ax n))
("biltmore" nil (b ih1 l t m ao1 r))
("bilton" nil (b ih1 l t ax n))
("biltz" nil (b ih1 l t s))
("bily" nil (b ih1 l iy0))
("bilyeu" nil (b ih1 l iy0 uw0))
("bilyk" nil (b ih1 l ih0 k))
("bilzerian" nil (b ih0 l z eh1 r iy0 ax n))
("bima" nil (b iy1 m ax))
("bimbo" nil (b ih1 m b ow0))
("bimbos" nil (b ih1 m b ow0 s))
("bimini" nil (b ih1 m ax n iy0))
("bimonthly" nil (b ay0 m ah1 n th l iy0))
("bin" nil (b ih1 n))
("bina" nil (b iy1 n ax))
("binary" nil (b ay1 n er0 iy0))
("binational" nil (b ay0 n ae1 sh ax n ax l))
("bind" nil (b ay1 n d))
("bindel" nil (b ih1 n d ax l))
("binder" nil (b ay1 n d er0))
("binders" nil (b ay1 n d er0 z))
("binding" nil (b ay1 n d ih0 ng))
("bindley" nil (b ih1 n d l iy0))
("binds" nil (b ay1 n d z))
("binegar" nil (b ih1 n ih0 g er0))
("bines" nil (b ay1 n z))
("binette" nil (b ih0 n eh1 t))
("binetti" nil (b ih0 n eh1 t iy0))
("binfield" nil (b ih1 n f iy1 l d))
("binford" nil (b ih1 n f er0 d))
("bing" nil (b ih1 ng))
("binga" nil (b iy1 ng g ax))
("bingaman" nil (b ih1 ng ax m ax n))
("binge" nil (b ih1 n jh))
("bingel" nil (b ih1 ng g ax l))
("bingenheimer" nil (b ih1 ng g ih0 n hh ay0 m er0))
("binger" nil (b ih1 ng er0))
("binges" nil (b ih1 n jh ih0 z))
("bingham" nil (b ih1 ng ax m))
("binghamton" nil (b ih1 ng ax m t ax n))
("bingle" nil (b ih1 ng g ax l))
("bingley" nil (b ih1 ng l iy0))
("bingman" nil (b ih1 ng m ax n))
("bingo" nil (b ih1 ng g ow0))
("binion" nil (b ih1 n y ax n))
("bink" nil (b ih1 ng k))
("binkley" nil (b ih1 ng k l iy0))
("binkowski" nil (b ih0 ng k ao1 f s k iy0))
("binn" nil (b ih1 n))
("binner" nil (b ih1 n er0))
("binney" nil (b ih1 n iy0))
("binnie" nil (b ih1 n iy0))
("binning" nil (b ih1 n ih0 ng))
("binns" nil (b ih1 n z))
("binocular" nil (b ax n aa1 k y ax l er0))
("binoculars" nil (b ax n aa1 k y ax l er0 z))
("binomial" nil (b ay0 n ow1 m iy0 ax l))
("bins" nil (b ih1 n z))
("binstock" nil (b ih1 n s t aa1 k))
("bintz" nil (b ih1 n t s))
("binz" nil (b ih1 n z))
("bio" nil (b ay1 ow1))
("biochem" nil (b ay1 ax ch ax m))
("biochemical" nil (b ay1 ow0 k eh1 m ax k ax l))
("biochemist" nil (b ay1 ow0 k eh1 m ax s t))
("biochemistry" nil (b ay1 ow0 k eh1 m ax s t r iy0))
("biocine" nil (b ay1 ax s iy1 n))
("biocontrol" nil (b ay1 ax k aa1 n t r aa0 l))
("biocraft" nil (b ay1 ow0 k r ae1 f t))
("biodegradable" nil (b ay1 ow0 d ax g r ey1 d ax b ax l))
("biodiverse" nil (b ay1 ow0 d ay0 v er1 s))
("biodiversity" nil (b ay1 ow0 d ay0 v er1 s ax t iy0))
("bioengineered" nil (b ay1 ow0 eh1 n jh ax n ih1 r d))
("bioengineering" nil (b ay1 ow0 eh1 n jh ax n ih1 r ih0 ng))
("biofeedback" nil (b ay0 ow0 f iy1 d b ae1 k))
("biogen" nil (b ay1 ow0 jh eh1 n))
("biographer" nil (b ay0 aa1 g r ax f er0))
("biographers" nil (b ay0 aa1 g r ax f er0 z))
("biographical" nil (b ay1 ax g r ae1 f ih0 k ax l))
("biographies" nil (b ay0 aa1 g r ax f iy0 z))
("biography" nil (b ay0 aa1 g r ax f iy0))
("biological" nil (b ay1 ax l aa1 jh ih0 k ax l))
("biologically" nil (b ay0 ax l aa1 jh ih0 k l iy0))
("biologicals" nil (b ay0 ax l aa1 jh ih0 k ax l z))
("biologics" nil (b ay0 ax l aa1 jh ih0 k s))
("biologist" nil (b ay0 aa1 l ax jh ax s t))
("biologists" nil (b ay0 aa1 l ax jh ax s t s))
("biology" nil (b ay0 aa1 l ax jh iy0))
("biomass" nil (b ay1 ax m ae0 s))
("biomaterials" nil (b ay1 ow0 m ax t ih1 r iy0 ax l z))
("biome" nil (b ay1 ow1 m))
("biomed" nil (b ay1 ow0 m eh1 d))
("biomedical" nil (b ay1 ow0 m eh1 d ih0 k ax l))
("biomedicals" nil (b ay1 ow0 m eh1 d ih0 k ax l z))
("biomes" nil (b ay1 ow1 m z))
("biomet" nil (b ay1 ow0 m eh0 t))
("biondi" nil (b iy0 aa1 n d iy0))
("biondo" nil (b iy0 ow1 n d ow0))
("biondolillo" nil (b iy0 ow0 n d ow0 l ih1 l ow0))
("bionetics" nil (b ay1 ow0 n eh1 t ih0 k s))
("biopharm" nil (b ay1 ax f aa0 r m))
("biopharmaceutical" nil (b ay1 ow0 f aa1 r m ax s uw1 t ih0 k ax l))
("biophysics" nil (b ay1 ow0 f ih1 s ih0 k s))
("biopsy" nil (b ay1 aa0 p s iy0))
("bios" nil (b ay1 ow0 s))
("biosafety" nil (b ay1 ow0 s ey1 f t iy0))
("bioscience" nil (b ay1 ow0 s iy1 ax n s))
("biosciences" nil (b ay0 ao1 s iy0 eh1 n s ih0 z))
("biosensor" nil (b ay1 ow0 s eh1 n s er0))
("biosensors" nil (b ay1 ow0 s eh1 n s er0 z))
("biosis" nil (b iy0 ow1 z ih0 s))
("biosphere" nil (b ay1 ow0 s f ih1 r))
("biospheres" nil (b ay1 ow0 s f ih1 r z))
("biosys" nil (b ay1 ow0 s ih0 s))
("biosystems" nil (b ay1 ow0 s ih1 s t ax m z))
("biotech" nil (b ay1 ow0 t eh1 k))
("biotechnica" nil (b ay1 ow0 t eh1 k n ih0 k ax))
("biotechnological" nil (b ay1 ow0 t eh1 k n ax l aa1 jh ih0 k ax l))
("biotechnologies" nil (b ay1 ow0 t eh1 k n aa1 l ax jh iy0 z))
("biotechnology" nil (b ay1 ow0 t eh1 k n aa1 l ax jh iy0))
("biotherapeutics" nil (b ay1 ow0 th eh1 r ax p y uw1 t ih0 k s))
("biotin" nil (b ay1 ax t ax n))
("biotite" nil (b ay1 ax t ay1 t))
("biovest" nil (b ay1 ow0 v ax s t))
("bipartisan" nil (b ay0 p aa1 r t ih0 z ax n))
("bipartisanship" nil (b ay0 p aa1 r t ax z ax n sh ih1 p))
("biphenyls" nil (b ih1 f ax n ax l z))
("biplane" nil (b ay1 p l ey1 n))
("bipolar" nil (b ay0 p ow1 l er0))
("bippus" nil (b ih1 p ax s))
("bir" nil (b er1))
("biracial" nil (b ay0 r ey1 sh ax l))
("birch" nil (b er1 ch))
("birchall" nil (b er1 k ax l))
("birchard" nil (b er1 k er0 d))
("bircher" nil (b er1 ch er0))
("birchett" nil (b er1 ch ih0 t))
("birchfield" nil (b er1 ch f iy1 l d))
("birchler" nil (b er1 k ax l er0))
("birchmeier" nil (b er1 k m ay0 er0))
("birckhead" nil (b er1 k hh eh0 d))
("bird" nil (b er1 d))
("birden" nil (b er1 d ax n))
("birdfeather" nil (b er1 d f eh1 dh er0))
("birdfeeder" nil (b er1 d f iy1 d er0))
("birdfeeders" nil (b er1 d f iy1 d er0 z))
("birdfinder" nil (b er1 d f ay1 n d er0))
("birdie" nil (b er1 d iy0))
("birdied" nil (b er1 d iy0 d))
("birdlife" nil (b er1 d l ay1 f))
("birds" nil (b er1 d z))
("birdsall" nil (b er1 d z ao1 l))
("birdsell" nil (b er1 d s ax l))
("birdsong" nil (b er1 d s ao1 ng))
("birdwell" nil (b er1 d w eh1 l))
("bireme" nil (b ay1 r iy1 m))
("biremes" nil (b ay1 r iy1 m z))
("birenbaum" nil (b ay1 r ax n b aw0 m))
("bires" nil (b ay1 r z))
("birge" nil (b er1 jh))
("birinyi" nil (b ih0 r iy1 n y iy0))
("birk" nil (b er1 k))
("birkedal" nil (b er1 k ax d aa1 l))
("birkel" nil (b er1 k ax l))
("birkeland" nil (b er1 k l ax n d))
("birkenau" nil (b er1 k ax n aw0))
("birkes" nil (b er1 k s))
("birkett" nil (b er1 k ih0 t))
("birkey" nil (b er1 k iy0))
("birkhead" nil (b er1 k hh eh0 d))
("birkhimer" nil (b er1 k hh ih0 m er0))
("birkholz" nil (b er1 k hh ow0 l z))
("birkland" nil (b er1 k l ax n d))
("birkner" nil (b er1 k n er0))
("birks" nil (b er1 k s))
("birky" nil (b er1 k iy0))
("birle" nil (b er1 l))
("birley" nil (b er1 l iy0))
("birman" nil (b er1 m ax n))
("birmid" nil (b er1 m ih0 d))
("birmingham" nil (b er1 m ih0 ng hh ae1 m))
("birnbaum" nil (b er1 n b aw0 m))
("birney" nil (b er1 n iy0))
("birnie" nil (b er1 n iy0))
("biro" nil (b ih1 r ow0))
("biron" nil (b ay1 r ax n))
("biros" nil (b ay1 r ow0 z))
("birr" nil (b er1))
("birren" nil (b er1 ax n))
("birt" nil (b er1 t))
("birtcher" nil (b er1 ch er0))
("birth" nil (b er1 th))
("birthday" nil (b er1 th d ey1))
("birthdays" nil (b er1 th d ey1 z))
("birthing" nil (b er1 th ih0 ng))
("birthmarks" nil (b er1 th m aa1 r k s))
("birthplace" nil (b er1 th p l ey1 s))
("birthrate" nil (b er1 th r ey1 t))
("birthrates" nil (b er1 th r ey1 t s))
("birthright" nil (b er1 th r ay1 t))
("births" nil (b er1 th s))
("birtle" nil (b er1 t ax l))
("birtley" nil (b er1 t l iy0))
("bis" nil (b ih1 s))
("bisaillon" nil (b ay1 s ax l aa0 n))
("bisbee" nil (b ih1 s b iy0))
("bisbing" nil (b ih1 s b ih0 ng))
("biscardi" nil (b ih0 s k aa1 r d iy0))
("biscayne" nil (b ih0 s k ey1 n))
("bisceglia" nil (b ih0 s ch eh1 g l iy0 ax))
("bisch" nil (b ih1 sh))
("bischel" nil (b ih1 sh ax l))
("bischof" nil (b ih1 sh ax f))
("bischofberger" nil (b ih1 sh ao0 f b er0 g er0))
("bischoff" nil (b ih1 s k hh ao0 f))
("biscoe" nil (b ih0 s k ow1))
("biscuit" nil (b ih1 s k ax t))
("biscuits" nil (b ih1 s k ax t s))
("bise" nil (b ay1 z))
("bisek" nil (b ih1 s ih0 k))
("bisel" nil (b ih1 s ax l))
("biser" nil (b ay1 z er0))
("bisesi" nil (b ih0 s eh1 s iy0))
("bisexual" nil (b ay1 s eh1 k sh uw0 ax l))
("bish" nil (b ih1 sh))
("bisher" nil (b ih1 sh er0))
("bishoff" nil (b ih1 s hh ao0 f))
("bishop" nil (b ih1 sh ax p))
("bishoprics" nil (b ih1 sh ax p r ih0 k s))
("bishops" nil (b ih1 sh ax p s))
("bishopsgate" nil (b ih1 sh ax p s g ey1 t))
("bisiani" nil (b ih0 s iy0 aa1 n iy0))
("bisig" nil (b ih1 s ih0 g))
("bisignano" nil (b ih0 s iy0 g n aa1 n ow0))
("biskup" nil (b ih1 s k ax p))
("bismarck" nil (b ih1 z m aa1 r k))
("bismark" nil (b ih1 z m aa1 r k))
("bismuth" nil (b ih1 z m ax th))
("bison" nil (b ay1 s ax n))
("bisping" nil (b ih1 s p ih0 ng))
("bisque" nil (b ih1 s k))
("biss" nil (b ih1 s))
("bissell" nil (b ih1 s ax l))
("bissen" nil (b ih1 s ax n))
("bisset" nil (b ih1 s ih0 t))
("bissett" nil (b ih1 s ih0 t))
("bissette" nil (b ih0 s eh1 t))
("bissey" nil (b ih1 s iy0))
("bissinger" nil (b ih1 s ih0 n jh er0))
("bisso" nil (b iy1 s ow0))
("bisson" nil (b ih1 s ax n))
("bissonette" nil (b ih1 s ax n eh0 t))
("bissonnette" nil (b ih1 s ax n eh1 t))
("bistline" nil (b ih1 s t l ay1 n))
("bistodeau" nil (b ih1 s t ax d ow0))
("bistro" nil (b ih1 s t r ow0))
("bisuteki" nil (b iy1 s uw0 t ey1 k iy0))
("bit" nil (b ih1 t))
("bitar" nil (b ih1 t er0))
("bitch" nil (b ih1 ch))
("bitchy" nil (b ih1 ch iy0))
("bite" nil (b ay1 t))
("bites" nil (b ay1 t s))
("bitesize" nil (b ih1 t s ay1 z))
("bither" nil (b ih1 dh er0))
("biting" nil (b ay1 t ih0 ng))
("bitler" nil (b ay1 t ax l er0))
("bitner" nil (b ih1 t n er0))
("bitney" nil (b ih1 t n iy0))
("bits" nil (b ih1 t s))
("bittel" nil (b ih1 t ax l))
("bitten" nil (b ih1 t ax n))
("bittenbender" nil (b ih1 t ih0 n b eh0 n d er0))
("bitter" nil (b ih1 t er0))
("bitterest" nil (b ih1 t er0 ax s t))
("bitterly" nil (b ih1 t er0 l iy0))
("bitterman" nil (b ih1 t er0 m ax n))
("bittermann" nil (b ih1 t er0 m ax n))
("bitterness" nil (b ih1 t er0 n ax s))
("bitterroot" nil (b ih1 t er0 r uw1 t))
("bitters" nil (b ih1 t er0 z))
("bittersweet" nil (b ih1 t er0 s w iy1 t))
("bittick" nil (b ih1 t ih0 k))
("bitting" nil (b ih1 t ih0 ng))
("bittinger" nil (b ih1 t ih0 ng er0))
("bittle" nil (b ih1 t ax l))
("bittman" nil (b ih1 t m ax n))
("bittner" nil (b ih1 t n er0))
("bitton" nil (b ih1 t ax n))
("bitumen" nil (b ih1 t uw1 m ax n))
("bituminous" nil (b ih0 t uw1 m ax n ax s))
("bitz" nil (b ih1 t s))
("bitzer" nil (b ih1 t z er0))
("bivalve" nil (b ay1 v ae1 l v))
("bivalves" nil (b ay1 v ae1 l v z))
("biven" nil (b ay1 v ax n))
("bivens" nil (b ay1 v ax n z))
("biviano" nil (b iy1 v iy0 aa1 n ow0))
("bivin" nil (b ih1 v ih0 n))
("bivins" nil (b ih1 v ih0 n z))
("bivona" nil (b ih0 v ow1 n ax))
("bivouac" nil (b ih1 v w ae0 k))
("biweekly" nil (b ay0 w iy1 k l iy0))
("bix" nil (b ih1 k s))
("bixby" nil (b ih1 k s b iy0))
("bixel" nil (b ih1 k s ax l))
("bixler" nil (b ih1 k s l er0))
("biz" nil (b ih1 z))
("bizango" nil (b ih0 z ae1 ng g ow0))
("bizarre" nil (b ax z aa1 r))
("bizmart" nil (b ih1 z m aa1 r t))
("bizub" nil (b ih1 z ax b))
("bizzell" nil (b ih1 z ax l))
("bjelland" nil (b y eh1 l ax n d))
("bjerke" nil (b y er1 k))
("bjoern" nil (b y ao1 r n))
("bjorge" nil (b y ao1 r g))
("bjork" nil (b y ao1 r k))
("bjorklund" nil (b y ao1 r k l ax n d))
("bjorkman" nil (b y ao1 r k m ax n))
("bjorn" nil (b y ao1 r n))
("bjornstad" nil (b y ao1 r n s t ax d))
("blachly" nil (b l aa1 ch l iy0))
("black" nil (b l ae1 k))
("blackard" nil (b l ae1 k er0 d))
("blackball" nil (b l ae1 k b ao1 l))
("blackberries" nil (b l ae1 k b eh1 r iy0 z))
("blackberry" nil (b l ae1 k b eh1 r iy0))
("blackboard" nil (b l ae1 k b ao1 r d))
("blackburn" nil (b l ae1 k b er0 n))
("blacked" nil (b l ae1 k t))
("blackened" nil (b l ae1 k ax n d))
("blackening" nil (b l ae1 k ax n ih0 ng))
("blackens" nil (b l ae1 k ax n z))
("blacker" nil (b l ae1 k er0))
("blackerby" nil (b l ae1 k er0 b iy0))
("blackest" nil (b l ae1 k ax s t))
("blacketer" nil (b l ae1 k iy0 t er0))
("blackett" nil (b l ae1 k ih0 t))
("blackfoot" nil (b l ae1 k f uh1 t))
("blackford" nil (b l ae1 k f er0 d))
("blackham" nil (b l ae1 k hh ax m))
("blackhawk" nil (b l ae1 k hh ao1 k))
("blackhawks" nil (b l ae1 k hh ao1 k s))
("blackheath" nil (b l ae1 k hh iy1 th))
("blackhurst" nil (b l ae1 k hh er0 s t))
("blackjack" nil (b l ae1 k jh ae1 k))
("blackledge" nil (b l ae1 k l eh1 jh))
("blackley" nil (b l ae1 k l iy0))
("blacklist" nil (b l ae1 k l ih1 s t))
("blacklisted" nil (b l ae1 k l ih1 s t ih0 d))
("blacklisting" nil (b l ae1 k l ih1 s t ih0 ng))
("blacklock" nil (b l ae1 k l aa1 k))
("blackmail" nil (b l ae1 k m ey1 l))
("blackmailed" nil (b l ae1 k m ey1 l d))
("blackmailing" nil (b l ae1 k m ey1 l ih0 ng))
("blackman" nil (b l ae1 k m ax n))
("blackmer" nil (b l ae1 k m er0))
("blackmon" nil (b l ae1 k m ax n))
("blackmore" nil (b l ae1 k m ao0 r))
("blackmun" nil (b l ae1 k m ax n))
("blackness" nil (b l ae1 k n ax s))
("blackout" nil (b l ae1 k aw1 t))
("blackouts" nil (b l ae1 k aw1 t s))
("blackpool" nil (b l ae1 k p uw1 l))
("blackrock" nil (b l ae1 k r aa1 k))
("blacks" nil (b l ae1 k s))
("blacksburg" nil (b l ae1 k s b er0 g))
("blackshear" nil (b l ae1 k sh ih0 r))
("blacksher" nil (b l ae1 k sh er0))
("blackshire" nil (b l ae1 k sh ay1 r))
("blacksmith" nil (b l ae1 k s m ih1 th))
("blackson" nil (b l ae1 k s ax n))
("blackstock" nil (b l ae1 k s t aa1 k))
("blackston" nil (b l ae1 k s t ax n))
("blackstone" nil (b l ae1 k s t ow1 n))
("blacktop" nil (b l ae1 k t aa1 p))
("blackwelder" nil (b l ae1 k w eh1 l d er0))
("blackwell" nil (b l ae1 k w eh1 l))
("blackwood" nil (b l ae1 k w uh1 d))
("bladder" nil (b l ae1 d er0))
("blade" nil (b l ey1 d))
("bladen" nil (b l ey1 d ax n))
("blades" nil (b l ey1 d z))
("bladow" nil (b l ae1 d ow0))
("blaese" nil (b l ey1 z))
("blaesing" nil (b l eh1 s ih0 ng))
("blagden" nil (b l ae1 g d ax n))
("blagg" nil (b l ae1 g))
("blah" nil (b l aa1))
("blaha" nil (b l aa1 hh ax))
("blahnik" nil (b l aa1 n ih0 k))
("blahut" nil (b l ae1 hh ax t))
("blaich" nil (b l ey1 ch))
("blaikie" nil (b l ey1 k iy0))
("blain" nil (b l ey1 n))
("blaine" nil (b l ey1 n))
("blair" nil (b l eh1 r))
("blais" nil (b l eh1 z))
("blaisdell" nil (b l ey1 s d ax l))
("blaise" nil (b l ey1 z))
("blaize" nil (b l ey1 z))
("blake" nil (b l ey1 k))
("blakeley" nil (b l ey1 k l iy0))
("blakely" nil (b l ey1 k l iy0))
("blakeman" nil (b l ey1 k m ax n))
("blakemore" nil (b l ey1 k m ao0 r))
("blakeney" nil (b l ey1 k n iy0))
("blakenham" nil (b l ey1 k ax n hh ae1 m))
("blakenship" nil (b l ey1 k ax n sh ih0 p))
("blaker" nil (b l ey1 k er0))
("blakes" nil (b l ey1 k s))
("blakeslee" nil (b l ey1 k s l iy0))
("blakesley" nil (b l ey1 k s l iy0))
("blakey" nil (b l ey1 k iy0))
("blakley" nil (b l ae1 k l iy0))
("blakney" nil (b l ae1 k n iy0))
("blalack" nil (b l ae1 l ax k))
("blalock" nil (b l ae1 l aa0 k))
("blame" nil (b l ey1 m))
("blamed" nil (b l ey1 m d))
("blameless" nil (b l ey1 m l ax s))
("blames" nil (b l ey1 m z))
("blaming" nil (b l ey1 m ih0 ng))
("blampied" nil (b l ae1 m p iy0 d))
("blan" nil (b l ae1 n))
("blanc" nil (b l ae1 ng k))
("blanca" nil (b l aa1 ng k ax))
("blancett" nil (b l ae1 n s ih0 t))
("blanch" nil (b l ae1 n ch))
("blanchard" nil (b l ae1 n ch er0 d))
("blanchards" nil (b l ae1 n ch er0 d z))
("blanche" nil (b l ae1 n ch))
("blanchet" nil (b l ae1 n k ih0 t))
("blanchett" nil (b l ae1 n ch ih0 t))
("blanchette" nil (b l ax n sh eh1 t))
("blanchfield" nil (b l ae1 n ch f iy1 l d))
("blanck" nil (b l ae1 ng k))
("blanco" nil (b l ae1 ng k ow0))
("bland" nil (b l ae1 n d))
("blanda" nil (b l ae1 n d ax))
("blander" nil (b l ae1 n d er0))
("blandford" nil (b l ae1 n d f ao0 r d))
("blandin" nil (b l ae1 n d ih0 n))
("blanding" nil (b l ae1 n d ih0 ng))
("blandino" nil (b l aa0 n d iy1 n ow0))
("blandishments" nil (b l ae1 n d ih0 sh m ax n t s))
("blandly" nil (b l ae1 n d l iy0))
("blandness" nil (b l ae1 n d n ax s))
("blando" nil (b l ae1 n d ow0))
("blandon" nil (b l ae1 n d ih0 n))
("blane" nil (b l ey1 n))
("blaney" nil (b l ey1 n iy0))
("blanford" nil (b l ae1 n f er0 d))
("blank" nil (b l ae1 ng k))
("blanke" nil (b l ae1 ng k))
("blanken" nil (b l ae1 ng k ax n))
("blankenbaker" nil (b l ae1 ng k ax n b ey1 k er0))
("blankenbeckler" nil (b l ae1 ng k ax n b eh0 k ax l er0))
("blankenburg" nil (b l ae1 ng k ax n b er0 g))
("blankenhorn" nil (b l ae1 ng k ih0 n hh er0 n))
("blankenship" nil (b l ae1 ng k ax n sh ih1 p))
("blanket" nil (b l ae1 ng k ax t))
("blanketed" nil (b l ae1 ng k ax t ih0 d))
("blanketing" nil (b l ae1 ng k ax t ih0 ng))
("blankets" nil (b l ae1 ng k ax t s))
("blanking" nil (b l ae1 ng k ih0 ng))
("blankinship" nil (b l ae1 ng k ih0 n sh ih0 p))
("blankley" nil (b l ae1 ng k l iy0))
("blankly" nil (b l ae1 ng k l iy0))
("blanks" nil (b l ae1 ng k s))
("blann" nil (b l ae1 n))
("blansett" nil (b l ae1 n s ih0 t))
("blanton" nil (b l ae1 n t ax n))
("blare" nil (b l eh1 r))
("blared" nil (b l eh1 r d))
("blares" nil (b l eh1 r z))
("blaring" nil (b l eh1 r ih0 ng))
("blas" nil (b l aa1 s))
("blaschke" nil (b l ae1 sh k))
("blasco" nil (b l aa1 s k ow0))
("blasdel" nil (b l ae1 s d ax l))
("blasdell" nil (b l ae1 s d ax l))
("blase" nil (b l ey1 z))
("blaser" nil (b l ey1 z er0))
("blasi" nil (b l ey1 z iy0))
("blasia" nil (b l aa1 s iy0 ax))
("blasing" nil (b l ey1 z ih0 ng))
("blasingame" nil (b l aa0 s ih0 ng g aa1 m iy0))
("blasius" nil (b l ey1 s iy0 ih0 s))
("blaske" nil (b l ey1 s k))
("blasko" nil (b l aa1 s k ow0))
("blasphemous" nil (b l ae1 s f ax m ax s))
("blasphemy" nil (b l ae1 s f ax m iy0))
("blass" nil (b l ae1 s))
("blassingame" nil (b l aa0 s ih0 ng g aa1 m iy0))
("blast" nil (b l ae1 s t))
("blastdown" nil (b l ae1 s t d aw1 n))
("blasted" nil (b l ae1 s t ax d))
("blaster" nil (b l ae1 s t er0))
("blasting" nil (b l ae1 s t ih0 ng))
("blastoff" nil (b l ae1 s t ao1 f))
("blasts" nil (b l ae1 s t s))
("blaszak" nil (b l aa1 sh ax k))
("blaszczyk" nil (b l aa1 sh ch ih0 k))
("blatant" nil (b l ey1 t ax n t))
("blatantly" nil (b l ey1 t ax n t l iy0))
("blatchford" nil (b l ae1 ch f er0 d))
("blatchley" nil (b l ae1 ch l iy0))
("blather" nil (b l ae1 dh er0))
("blatnik" nil (b l ae1 t n ih0 k))
("blatt" nil (b l ae1 t))
("blatter" nil (b l ae1 t er0))
("blattner" nil (b l ae1 t n er0))
("blatz" nil (b l ae1 t s))
("blau" nil (b l aw1))
("blauch" nil (b l ao1 ch))
("blauer" nil (b l aw1 r))
("blaum" nil (b l ao1 m))
("blauser" nil (b l aw1 s er0))
("blaustein" nil (b l aw1 s t ay0 n))
("blauvelt" nil (b l aw1 v ih0 l t))
("blay" nil (b l ey1))
("blaydes" nil (b l ey1 d z))
("blaydon" nil (b l ey1 d ax n))
("blaylock" nil (b l ey1 l ax k))
("blayne" nil (b l ey1 n))
("blayney" nil (b l ey1 n iy0))
("blayze" nil (b l ey1 z))
("blaze" nil (b l ey1 z))
("blazed" nil (b l ey1 z d))
("blazejewski" nil (b l ax z ey0 eh1 f s k iy0))
("blazek" nil (b l aa1 z eh0 k))
("blazer" nil (b l ey1 z er0))
("blazers" nil (b l ey1 z er0 z))
("blazes" nil (b l ey1 z ih0 z))
("blazier" nil (b l ey1 z iy0 er0))
("blazina" nil (b l aa0 z iy1 n ax))
("blazing" nil (b l ey1 z ih0 ng))
("blea" nil (b l iy1))
("bleach" nil (b l iy1 ch))
("bleached" nil (b l iy1 ch t))
("bleacher" nil (b l iy1 ch er0))
("bleachers" nil (b l iy1 ch er0 z))
("bleaching" nil (b l iy1 ch ih0 ng))
("bleak" nil (b l iy1 k))
("bleaker" nil (b l iy1 k er0))
("bleakest" nil (b l iy1 k ax s t))
("bleakley" nil (b l iy1 k l iy0))
("bleakness" nil (b l iy1 k n ax s))
("bleakney" nil (b l iy1 k n iy0))
("bleam" nil (b l iy1 m))
("bleary" nil (b l ih1 r iy0))
("bleau" nil (b l ow1))
("blech" nil (b l eh1 k))
("blecha" nil (b l eh1 ch ax))
("blecher" nil (b l eh1 k er0))
("blechman" nil (b l eh1 k m ax n))
("bleck" nil (b l eh1 k))
("blecker" nil (b l eh1 k er0))
("bled" nil (b l eh1 d))
("bledsoe" nil (b l eh1 d s ow0))
("bleecker" nil (b l iy1 k er0))
("bleed" nil (b l iy1 d))
("bleeding" nil (b l iy1 d ih0 ng))
("bleeds" nil (b l iy1 d z))
("bleeker" nil (b l iy1 k er0))
("bleep" nil (b l iy1 p))
("bleeping" nil (b l iy1 p ih0 ng))
("blegen" nil (b l eh1 g ax n))
("blehm" nil (b l eh1 m))
("bleich" nil (b l ay1 k))
("bleicher" nil (b l ay1 k er0))
("bleier" nil (b l ay1 er0))
("bleil" nil (b l iy1 l))
("bleiler" nil (b l ay1 l er0))
("blelloch" nil (b l eh1 l aa0 k))
("blemish" nil (b l eh1 m ih0 sh))
("blemished" nil (b l eh1 m ih0 sh t))
("blemishes" nil (b l eh1 m ih0 sh ih0 z))
("blend" nil (b l eh1 n d))
("blenda" nil (b l eh1 n d ax))
("blendax" nil (b l eh1 n d ae1 k s))
("blended" nil (b l eh1 n d ax d))
("blender" nil (b l eh1 n d er0))
("blenders" nil (b l eh1 n d er0 z))
("blending" nil (b l eh1 n d ih0 ng))
("blends" nil (b l eh1 n d z))
("blenheim" nil (b l eh1 n hh ay1 m))
("blepharisma" nil (b l eh1 f er0 ih1 z m ax))
("bleser" nil (b l iy1 z er0))
("bless" nil (b l eh1 s))
("blessed" v (b l eh1 s t))
("blessed" j (b l eh1 s ih d))
("blesses" nil (b l eh1 s ih0 z))
("blessing" nil (b l eh1 s ih0 ng))
("blessinger" nil (b l eh1 s ih0 ng er0))
("blessings" nil (b l eh1 s ih0 ng z))
("blessington" nil (b l eh1 s ih0 ng t ax n))
("blest" nil (b l eh1 s t))
("blethen" nil (b l eh1 th ax n))
("bleu" nil (b l uw1))
("blevens" nil (b l iy1 v ax n z))
("blevins" nil (b l eh1 v ih0 n z))
("blew" nil (b l uw1))
("blewett" nil (b l uw1 ih0 t))
("blewitt" nil (b l uw1 ih0 t))
("bley" nil (b l ey1))
("blick" nil (b l ih1 k))
("blickenstaff" nil (b l ih1 k ih0 n s t ax f))
("bligh" nil (b l ay1))
("blight" nil (b l ay1 t))
("blighted" nil (b l ay1 t ih0 d))
("bliley" nil (b l ay1 l iy0))
("blimp" nil (b l ih1 m p))
("blimps" nil (b l ih1 m p s))
("blincoe" nil (b l ih0 n k ow1))
("blind" nil (b l ay1 n d))
("blinded" nil (b l ay1 n d ih0 d))
("blinder" nil (b l ay1 n d er0))
("blinders" nil (b l ay1 n d er0 z))
("blindfold" nil (b l ay1 n d f ow1 l d))
("blindfolded" nil (b l ay1 n d f ow1 l d ih0 d))
("blinding" nil (b l ay1 n d ih0 ng))
("blindly" nil (b l ay1 n d l iy0))
("blindness" nil (b l ay1 n d n ax s))
("blinds" nil (b l ay1 n d z))
("blindsided" nil (b l ay1 n d s ay1 d ih0 d))
("blink" nil (b l ih1 ng k))
("blinked" nil (b l ih1 ng k t))
("blinking" nil (b l ih1 ng k ih0 ng))
("blinks" nil (b l ih1 ng k s))
("blinn" nil (b l ih1 n))
("blip" nil (b l ih1 p))
("blips" nil (b l ih1 p s))
("blish" nil (b l ih1 sh))
("bliss" nil (b l ih1 s))
("blissett" nil (b l ih1 s ih0 t))
("blissful" nil (b l ih1 s f ax l))
("blissfully" nil (b l ih1 s f ax l iy0))
("blister" nil (b l ih1 s t er0))
("blistered" nil (b l ih1 s t er0 d))
("blistering" nil (b l ih1 s t er0 ih0 ng))
("blisters" nil (b l ih1 s t er0 z))
("blitch" nil (b l ih1 ch))
("blithe" nil (b l ay1 dh))
("blithely" nil (b l ay1 th l iy0))
("blitz" nil (b l ih1 t s))
("blitzed" nil (b l ih1 t s t))
("blitzer" nil (b l ih1 t z er0))
("blitzes" nil (b l ih1 t s ih0 z))
("blitzkrieg" nil (b l ih1 t s k r iy1 g))
("bliven" nil (b l ay1 v ax n))
("blixt" nil (b l ih1 k s t))
("blizard" nil (b l ih1 z er0 d))
("blizzard" nil (b l ih1 z er0 d))
("blizzards" nil (b l ih1 z er0 d z))
("bloat" nil (b l ow1 t))
("bloated" nil (b l ow1 t ih0 d))
("blob" nil (b l aa1 b))
("blobby" nil (b l aa1 b iy0))
("bloc" nil (b l aa1 k))
("bloch" nil (b l aa1 k))
("blocher" nil (b l aa1 k er0))
("block" nil (b l aa1 k))
("blockade" nil (b l aa1 k ey1 d))
("blockaded" nil (b l aa1 k ey1 d ih0 d))
("blockades" nil (b l aa1 k ey1 d z))
("blockading" nil (b l aa1 k ey1 d ih0 ng))
("blockage" nil (b l aa1 k ih0 jh))
("blockages" nil (b l aa1 k ih0 jh ih0 z))
("blockbuster" nil (b l aa1 k b ah1 s t er0))
("blockbusters" nil (b l aa1 k b ah1 s t er0 z))
("blocked" nil (b l aa1 k t))
("blocker" nil (b l aa1 k er0))
("blockers" nil (b l aa1 k er0 z))
("blocking" nil (b l aa1 k ih0 ng))
("blocks" nil (b l aa1 k s))
("blocs" nil (b l aa1 k s))
("blodgett" nil (b l aa1 jh ih0 t))
("bloedel" nil (b l ow1 d ax l))
("bloedorn" nil (b l ow1 d er0 n))
("bloem" nil (b l ow1 m))
("bloemer" nil (b l ow1 m er0))
("bloemker" nil (b l ow1 m k er0))
("blohm" nil (b l ow1 m))
("blok" nil (b l aa1 k))
("blokes" nil (b l ow1 k s))
("blom" nil (b l aa1 m))
("blomberg" nil (b l aa1 m b er0 g))
("blome" nil (b l ow1 m))
("blomgren" nil (b l aa1 m g r eh0 n))
("blomquist" nil (b l aa1 m k w ih1 s t))
("blomstrom" nil (b l aa1 m s t r ax m))
("blond" nil (b l aa1 n d))
("blonde" nil (b l aa1 n d))
("blondell" nil (b l aa1 n d ax l))
("blonder" nil (b l aa1 n d er0))
("blondes" nil (b l aa1 n d z))
("blondie" nil (b l aa1 n d iy0))
("blondin" nil (b l aa1 n d ih0 n))
("blonds" nil (b l aa1 n d z))
("blood" nil (b l ah1 d))
("bloodbath" nil (b l ah1 d b ae1 th))
("blooded" nil (b l ah1 d ih0 d))
("bloodgood" nil (b l ah1 d g uh1 d))
("bloodied" nil (b l ah1 d iy0 d))
("bloodier" nil (b l ah1 d iy0 er0))
("bloodiest" nil (b l ah1 d iy0 ax s t))
("bloodless" nil (b l ah1 d l ax s))
("bloodletting" nil (b l ah1 d l eh1 t ih0 ng))
("bloodlines" nil (b l ah1 d l ay1 n z))
("bloods" nil (b l ah1 d z))
("bloodshed" nil (b l ah1 d sh eh1 d))
("bloodstain" nil (b l ah1 d s t ey1 n))
("bloodstained" nil (b l ah1 d s t ey1 n d))
("bloodstains" nil (b l ah1 d s t ey1 n z))
("bloodstone" nil (b l ah1 d s t ow1 n))
("bloodstream" nil (b l ah1 d s t r iy1 m))
("bloodsucking" nil (b l ah1 d s ah1 k ih0 ng))
("bloodsworth" nil (b l ah1 d z w er1 th))
("bloodthirsty" nil (b l ah1 d th er1 s t iy0))
("bloodworth" nil (b l ah1 d w er1 th))
("bloody" nil (b l ah1 d iy0))
("bloom" nil (b l uw1 m))
("bloomberg" nil (b l uw1 m b er0 g))
("bloomed" nil (b l uw1 m d))
("bloomer" nil (b l uw1 m er0))
("bloomers" nil (b l uw1 m er0 z))
("bloomfield" nil (b l uw1 m f iy1 l d))
("blooming" nil (b l uw1 m ih0 ng))
("bloomingdale" nil (b l uw1 m ih0 ng d ey1 l))
("bloomington" nil (b l uw1 m ih0 ng t ax n))
("bloomquist" nil (b l uw1 m k w ih1 s t))
("blooms" nil (b l uw1 m z))
("blooper" nil (b l uw1 p er0))
("bloopers" nil (b l uw1 p er0 z))
("bloor" nil (b l uh1 r))
("blose" nil (b l ow1 z))
("bloss" nil (b l ao1 s))
("blosser" nil (b l ao1 s er0))
("blossom" nil (b l aa1 s ax m))
("blossomed" nil (b l aa1 s ax m d))
("blossoming" nil (b l aa1 s ax m ih0 ng))
("blossoms" nil (b l aa1 s ax m z))
("blot" nil (b l aa1 t))
("blotnick" nil (b l aa1 t n ih0 k))
("blots" nil (b l aa1 t s))
("blotter" nil (b l aa1 t er0))
("blouch" nil (b l aw1 ch))
("blough" nil (b l aw1))
("blouin" nil (b l w iy1 n))
("blount" nil (b l aw1 n t))
("blouse" nil (b l aw1 s))
("blouses" nil (b l aw1 s ax z))
("blow" nil (b l ow1))
("blowe" nil (b l ow1))
("blower" nil (b l ow1 er0))
("blowers" nil (b l ow1 er0 z))
("blowing" nil (b l ow1 ih0 ng))
("blown" nil (b l ow1 n))
("blowout" nil (b l ow1 aw1 t))
("blowouts" nil (b l ow1 aw1 t s))
("blows" nil (b l ow1 z))
("blowtorch" nil (b l ow1 t ao1 r ch))
("blowup" nil (b l ow1 ah1 p))
("blowy" nil (b l ow1 iy0))
("bloxham" nil (b l aa1 k s ax m))
("bloxom" nil (b l aa1 k s ax m))
("bloxsom" nil (b l aa1 k s ax m))
("bloyd" nil (b l oy1 d))
("bloyer" nil (b l oy1 er0))
("blubaugh" nil (b l ah1 b ao0))
("blubber" nil (b l ah1 b er0))
("bludgeon" nil (b l ah1 jh ax n))
("bludgeoned" nil (b l ah1 jh ax n d))
("blue" nil (b l uw1))
("blueberries" nil (b l uw1 b eh1 r iy0 z))
("blueberry" nil (b l uw1 b eh1 r iy0))
("bluebird" nil (b l uw1 b er1 d))
("bluebonnets" nil (b l uw1 b aa1 n ax t s))
("bluechip" nil (b l uw1 ch ih1 p))
("bluefield" nil (b l uw1 f iy1 l d))
("bluegrass" nil (b l uw1 g r ae1 s))
("bluejay" nil (b l uw1 jh ey1))
("bluemel" nil (b l uh1 m ax l))
("blueprint" nil (b l uw1 p r ih1 n t))
("blueprints" nil (b l uw1 p r ih1 n t s))
("blues" nil (b l uw1 z))
("bluest" nil (b l uw1 ax s t))
("bluestein" nil (b l uh1 s t ay0 n))
("bluestine" nil (b l uw1 s t ay1 n))
("bluestone" nil (b l uw1 s t ow1 n))
("bluett" nil (b l uw1 t))
("bluey" nil (b l uw1 iy0))
("bluff" nil (b l ah1 f))
("bluffed" nil (b l ah1 f t))
("bluffing" nil (b l ah1 f ih0 ng))
("bluffs" nil (b l ah1 f s))
("bluford" nil (b l uw1 f er0 d))
("bluhdorn" nil (b l ah1 d ao1 r n))
("bluhm" nil (b l ah1 m))
("bluing" nil (b l uw1 ih0 ng))
("bluish" nil (b l uw1 ih0 sh))
("bluitt" nil (b l uw1 t))
("blum" nil (b l uw1 m))
("blumberg" nil (b l ah1 m b er0 g))
("blume" nil (b l uw1 m))
("blumenberg" nil (b l uw1 m eh0 n b er0 g))
("blumenfeld" nil (b l ah1 m ih0 n f eh0 l d))
("blumenschein" nil (b l ah1 m ih0 n sh ay0 n))
("blumenshine" nil (b l ah1 m ih0 n sh ay0 n))
("blumenstein" nil (b l uw1 m eh0 n s t ay0 n))
("blumenstock" nil (b l uw1 m eh0 n s t aa0 k))
("blumenthal" nil (b l uw1 m ax n th ao1 l))
("blumer" nil (b l uw1 m er0))
("blumstein" nil (b l ah1 m s t ay0 n))
("blunck" nil (b l ah1 ng k))
("blundall" nil (b l ah1 n d ax l))
("blundell" nil (b l ah1 n d ax l))
("blunder" nil (b l ah1 n d er0))
("blundered" nil (b l ah1 n d er0 d))
("blundering" nil (b l ah1 n d er0 ih0 ng))
("blunders" nil (b l ah1 n d er0 z))
("blunk" nil (b l ah1 ng k))
("blunt" nil (b l ah1 n t))
("blunted" nil (b l ah1 n t ax d))
("blunter" nil (b l ah1 n t er0))
("bluntest" nil (b l ah1 n t ax s t))
("blunting" nil (b l ah1 n t ih0 ng))
("bluntly" nil (b l ah1 n t l iy0))
("bluntness" nil (b l ah1 n t n ax s))
("blunts" nil (b l ah1 n t s))
("blur" nil (b l er1))
("blurb" nil (b l er1 b))
("blurbs" nil (b l er1 b z))
("blurred" nil (b l er1 d))
("blurring" nil (b l er1 ih0 ng))
("blurry" nil (b l er1 iy0))
("blurs" nil (b l er1 z))
("blurt" nil (b l er1 t))
("blurted" nil (b l er1 t ih0 d))
("blurton" nil (b l er1 t ax n))
("blurts" nil (b l er1 t s))
("blush" nil (b l ah1 sh))
("blushed" nil (b l ah1 sh t))
("blushes" nil (b l ah1 sh ax z))
("blushing" nil (b l ah1 sh ih0 ng))
("blust" nil (b l ah1 s t))
("bluster" nil (b l ah1 s t er0))
("blustering" nil (b l ah1 s t er0 ih0 ng))
("blustery" nil (b l ah1 s t er0 iy0))
("bluth" nil (b l uw1 th))
("bly" nil (b l ay1))
("blye" nil (b l ay1))
("blyler" nil (b l ay1 l er0))
("blystone" nil (b l ay1 s t ow1 n))
("blyth" nil (b l ih1 th))
("blythe" nil (b l ay1 dh))
("bo" nil (b ow1))
("boa" nil (b ow1 ax))
("boak" nil (b ow1 k))
("boake" nil (b ow1 k))
("boal" nil (b ow1 l))
("boals" nil (b ow1 l z))
("boan" nil (b ow1 n))
("boar" nil (b ao1 r))
("board" nil (b ao1 r d))
("boardbent" nil (b ao1 r d b eh1 n t))
("boarded" nil (b ao1 r d ax d))
("boarding" nil (b ao1 r d ih0 ng))
("boardinghouses" nil (b ao1 r d ih0 ng hh aw1 s ih0 z))
("boardings" nil (b ao1 r d ih0 ng z))
("boardman" nil (b ao1 r d m ax n))
("boardroom" nil (b ao1 r d r uw1 m))
("boardrooms" nil (b ao1 r d r uw1 m z))
("boards" nil (b ao1 r d z))
("boardwalk" nil (b ao1 r d w ao1 k))
("boarman" nil (b ao1 r m ax n))
("boart" nil (b ao1 r t))
("boas" nil (b ow1 ax z))
("boase" nil (b ow1 z))
("boast" nil (b ow1 s t))
("boasted" nil (b ow1 s t ax d))
("boastful" nil (b ow1 s t f ax l))
("boasting" nil (b ow1 s t ih0 ng))
("boasts" nil (b ow1 s t s))
("boat" nil (b ow1 t))
("boated" nil (b ow1 t ih0 d))
("boaters" nil (b ow1 t er0 z))
("boathouse" nil (b ow1 t hh aw1 s))
("boating" nil (b ow1 t ih0 ng))
("boatlift" nil (b ow1 t l ih1 f t))
("boatlifts" nil (b ow1 t l ih1 f t s))
("boatload" nil (b ow1 t l ow1 d))
("boatloads" nil (b ow1 t l ow1 d z))
("boatman" nil (b ow1 t m ax n))
("boatner" nil (b ow1 t n er0))
("boatright" nil (b ow1 t r ay1 t))
("boats" nil (b ow1 t s))
("boatwright" nil (b ow1 t r ay1 t))
("boatyard" nil (b ow1 t y aa1 r d))
("boaz" nil (b ow1 ae0 z))
("bob" nil (b aa1 b))
("boback" nil (b ow1 b ae1 k))
("bobadilla" nil (b ow0 b aa0 d ih1 l ax))
("bobak" nil (b ow1 b ax k))
("bobb" nil (b aa1 b))
("bobber" nil (b aa1 b er0))
("bobbett" nil (b aa1 b ih0 t))
("bobbette" nil (b aa1 b eh1 t))
("bobbie" nil (b aa1 b iy0))
("bobbin" nil (b aa1 b ax n))
("bobbing" nil (b aa1 b ih0 ng))
("bobbinger" nil (b aa1 b ih0 ng er0))
("bobbitt" nil (b aa1 b ih0 t))
("bobbitts" nil (b aa1 b ih0 t s))
("bobbo" nil (b aa1 b ow0))
("bobby" nil (b aa1 b iy0))
("bobcat" nil (b aa1 b k ae1 t))
("bobe" nil (b ow1 b))
("bobeck" nil (b ow1 b eh1 k))
("bobek" nil (b ow1 b ih0 k))
("bober" nil (b aa1 b er0))
("boberg" nil (b ow1 b er0 g))
("bobick" nil (b aa1 b ih0 k))
("bobier" nil (b ow1 b iy0 er0))
("bobinski" nil (b ax b ih1 n s k iy0))
("boblitt" nil (b ax b l ih1 t))
("bobo" nil (b ow1 b ow0))
("bobolas" nil (b ow1 b ow0 l ax s))
("bobrow" nil (b aa1 b r ow1))
("bobrowski" nil (b ax b r ao1 f s k iy0))
("bobsled" nil (b aa1 b s l eh1 d))
("bobst" nil (b aa1 b s t))
("bobzien" nil (b aa1 b z iy0 n))
("boca" nil (b ow1 k ax))
("bocanegra" nil (b aa1 k ax n eh1 g r ax))
("bocchino" nil (b ow1 k iy1 n ow0))
("boccia" nil (b ow1 ch ax))
("boccio" nil (b ow1 ch iy0 ow0))
("boccuzzi" nil (b ow0 k uw1 t s iy0))
("bocek" nil (b ow1 ch eh1 k))
("boch" nil (b aa1 k))
("bocharov" nil (b aa1 ch er0 aa0 v))
("bochco" nil (b aa1 ch k ow0))
("boche" nil (b aa1 ch))
("bochenek" nil (b aa1 k ih0 n ih0 k))
("boches" nil (b aa1 ch ih0 z))
("bochicchio" nil (b ow0 k iy1 k iy0 ow0))
("bochner" nil (b aa1 k n er0))
("bocian" nil (b ow1 sh ax n))
("bock" nil (b aa1 k))
("bockelman" nil (b aa1 k ax l m ax n))
("bockius" nil (b aa1 k iy0 ax s))
("bockman" nil (b aa1 k m ax n))
("bockus" nil (b aa1 k ax s))
("bocock" nil (b aa1 k ax k))
("bocook" nil (b aa1 k uh0 k))
("boda" nil (b ow1 d ax))
("bodamer" nil (b aa1 d ax m er0))
("bodanis" nil (b ow0 d aa1 n ih0 s))
("bodden" nil (b aa1 d ax n))
("boddie" nil (b aa1 d iy0))
("boddington" nil (b aa1 d ih0 ng t ax n))
("boddy" nil (b aa1 d iy0))
("bode" nil (b ow1 d))
("bodegas" nil (b ow0 d ey1 g ax s))
("bodell" nil (b ax d eh1 l))
("boden" nil (b ow1 d ax n))
("bodenhamer" nil (b aa1 d ih0 n hh ax m er0))
("bodenheimer" nil (b aa1 d ih0 n hh ay0 m er0))
("bodenstein" nil (b ow1 d ax n s t ay1 n))
("bodensteiner" nil (b ow1 d ax n s t ay1 n er0))
("bodes" nil (b ow1 d z))
("bodey" nil (b ow1 d iy0))
("bodi" nil (b ow1 d iy0))
("bodie" nil (b ow1 d iy0))
("bodied" nil (b aa1 d iy0 d))
("bodies" nil (b aa1 d iy0 z))
("bodily" nil (b aa1 d ax l iy0))
("bodin" nil (b ow1 d ih0 n))
("bodine" nil (b ow0 d iy1 n iy0))
("bodkin" nil (b aa1 d k ih0 n))
("bodkins" nil (b aa1 d k ih0 n z))
("bodle" nil (b ow1 d ax l))
("bodley" nil (b aa1 d l iy0))
("bodman" nil (b aa1 d m ax n))
("bodmer" nil (b aa1 d m er0))
("bodnar" nil (b ax d n aa1 r))
("bodner" nil (b aa1 d n er0))
("bodo" nil (b ow1 d ow0))
("bodwell" nil (b aa1 d w eh1 l))
("body" nil (b aa1 d iy0))
("bodyguard" nil (b aa1 d iy0 g aa1 r d))
("bodyguards" nil (b aa1 d iy0 g aa1 r d z))
("boe" nil (b ow1))
("boeck" nil (b ow1 k))
("boeckel" nil (b ow1 k ax l))
("boecker" nil (b ow1 k er0))
("boeckman" nil (b ow1 k m ax n))
("boeckmann" nil (b ow1 k m ax n))
("boedecker" nil (b ow1 d ih0 k er0))
("boedeker" nil (b ow1 d ih0 k er0))
("boeder" nil (b ow1 d er0))
("boeding" nil (b ow1 d ih0 ng))
("boege" nil (b ow1 jh))
("boeger" nil (b ow1 g er0))
("boeh" nil (b ow1))
("boehl" nil (b ow1 l))
("boehle" nil (b ow1 hh ax l))
("boehler" nil (b ow1 l er0))
("boehlert" nil (b ow1 l er0 t))
("boehlke" nil (b ow1 l k))
("boehm" nil (b ow1 m))
("boehme" nil (b ow1 m))
("boehmer" nil (b ow1 m er0))
("boehmke" nil (b ow1 m k iy0))
("boehne" nil (b ow1 n))
("boehner" nil (b ow1 n er0))
("boehning" nil (b ow1 n ih0 ng))
("boehnke" nil (b ow1 ng k))
("boehnlein" nil (b ow1 n l ay1 n))
("boehringer" nil (b ao1 r ih0 ng er0))
("boeing" nil (b ow1 ih0 ng))
("boeke" nil (b ow1 k))
("boeker" nil (b ow1 k er0))
("boelens" nil (b ow1 l ax n z))
("boelkow" nil (b ow1 l k aw0))
("boelman" nil (b ow1 l m ax n))
("boelter" nil (b ow1 l t er0))
("boen" nil (b ow1 n))
("boenig" nil (b ow1 n ih0 g))
("boening" nil (b aa1 ax n ih0 ng))
("boepple" nil (b ow1 p ax l))
("boer" nil (b ao1 r))
("boerger" nil (b ao1 r g er0))
("boerman" nil (b ao1 r m ax n))
("boerner" nil (b ao1 r n er0))
("boers" nil (b ao1 r z))
("boersma" nil (b ao1 r s m ax))
("boes" nil (b ow1 z))
("boesak" nil (b ow1 z ae0 k))
("boesch" nil (b ow1 sh))
("boeschenstein" nil (b ow1 sh ax n s t iy1 n))
("boese" nil (b ow1 s))
("boesel" nil (b ow1 s ax l))
("boesen" nil (b ow1 s ax n))
("boesky" nil (b ow1 s k iy0))
("boeskys" nil (b ow1 s k iy0 z))
("boetcher" nil (b ow1 ch er0))
("boettcher" nil (b ow1 ch er0))
("boettger" nil (b ow1 t g er0))
("boettner" nil (b ow1 t n er0))
("boeve" nil (b ow1 v))
("boff" nil (b ao1 f))
("boffa" nil (b aa1 f ax))
("boffo" nil (b ao1 f ow0))
("bofors" nil (b ow1 f er0 z))
("bog" nil (b aa1 g))
("bogacki" nil (b ax g aa1 t s k iy0))
("bogacz" nil (b aa1 g ax ch))
("bogan" nil (b ow1 g ax n))
("bogar" nil (b ax g aa1 r))
("bogard" nil (b aa1 g er0 d))
("bogardus" nil (b aa1 g aa0 r d ih0 s))
("bogart" nil (b ow1 g aa1 r t))
("bogdan" nil (b aa1 g d ax n))
("bogdanoff" nil (b aa1 g d ax n ao0 f))
("bogdanski" nil (b ax g d ae1 n s k iy0))
("bogden" nil (b aa1 g d ax n))
("bogdon" nil (b aa1 g d ax n))
("boge" nil (b ow1 jh))
("bogen" nil (b aa1 g ax n))
("bogenschutz" nil (b aa1 g ih0 n sh ax t s))
("boger" nil (b ow1 g er0))
("bogert" nil (b aa1 g er0 t))
("bogey" nil (b ow1 g iy0))
("bogeyed" nil (b ow1 g iy0 d))
("bogeyman" nil (b ow1 g iy0 m ae1 n))
("bogeymen" nil (b ow1 g iy0 m ih1 n))
("boggan" nil (b aa1 g ax n))
("bogged" nil (b aa1 g d))
("boggess" nil (b aa1 g ih0 s))
("bogging" nil (b ao1 g ih0 ng))
("boggio" nil (b ow1 jh iy0 ow0))
("boggled" nil (b aa1 g ax l d))
("boggles" nil (b aa1 g ax l z))
("boggling" nil (b aa1 g ax l ih0 ng))
("boggs" nil (b ao1 g z))
("boggus" nil (b aa1 g ax s))
("boghosian" nil (b ax g hh aa1 zh ih0 n))
("bogie" nil (b uh1 g iy0))
("bogin" nil (b ow1 jh ih0 n))
("bogle" nil (b ow1 g ax l))
("bognar" nil (b ax g n aa1 r))
("bogner" nil (b aa1 g n er0))
("bogor" nil (b ow1 g ao1 r))
("bogosian" nil (b ax g aa1 zh ih0 n))
("bogota" nil (b ow1 g ax t aa1))
("bogs" nil (b aa1 g z))
("bogucki" nil (b ax g ah1 t s k iy0))
("bogue" nil (b ow1 g))
("bogus" nil (b ow1 g ax s))
("boguslavskaya" nil (b ow0 g ah1 s l ax v s k ay1 ax))
("bogusz" nil (b aa1 g ax sh))
("bohac" nil (b ow1 hh ax k))
("bohall" nil (b aa1 hh ax l))
("bohan" nil (b ow1 hh ax n))
("bohanan" nil (b aa1 hh ax n ae0 n))
("bohannan" nil (b ax hh ae1 n ax n))
("bohannon" nil (b aa1 hh ax n aa0 n))
("bohanon" nil (b aa1 hh ax n aa0 n))
("bohart" nil (b aa1 hh aa0 r t))
("boheme" nil (b ow0 hh ey1 m ey0))
("bohemia" nil (b ow0 hh iy1 m iy0 ax))
("bohemian" nil (b ow0 hh iy1 m iy0 ax n))
("bohen" nil (b ow1 ax n))
("bohl" nil (b ow1 l))
("bohland" nil (b ow1 l ax n d))
("bohlander" nil (b ow1 l ax n d er0))
("bohle" nil (b ow1 hh ax l))
("bohlen" nil (b ow1 l ax n))
("bohler" nil (b ow1 l er0))
("bohlin" nil (b ow1 l ih0 n))
("bohling" nil (b ow1 l ih0 ng))
("bohlinger" nil (b ow1 l ih0 ng er0))
("bohlke" nil (b aa1 k))
("bohlken" nil (b ow1 l k ax n))
("bohlman" nil (b ow1 l m ax n))
("bohlmann" nil (b ow1 l m ax n))
("bohm" nil (b aa1 m))
("bohman" nil (b ow1 m ax n))
("bohmer" nil (b ow1 m er0))
("bohn" nil (b ow1 n))
("bohne" nil (b ow1 n))
("bohnen" nil (b ow1 n ax n))
("bohnenkamp" nil (b ow1 n ih0 n k ae0 m p))
("bohner" nil (b ow1 n er0))
("bohnert" nil (b ow1 n er0 t))
("bohnet" nil (b aa1 n ih0 t))
("bohnhoff" nil (b ow1 n hh ao1 f))
("bohning" nil (b aa1 n ih0 ng))
("bohnsack" nil (b ow1 n s ax k))
("bohon" nil (b ow1 hh ax n))
("bohr" nil (b ao1 r))
("bohren" nil (b ao1 r ax n))
("bohrer" nil (b ao1 r er0))
("boice" nil (b oy1 s))
("boies" nil (b oy1 z))
("boik" nil (b oy1 k))
("boike" nil (b oy1 k))
("boil" nil (b oy1 l))
("boilard" nil (b oy0 l aa1 r d))
("boileau" nil (b oy1 l ow1))
("boiled" nil (b oy1 l d))
("boiler" nil (b oy1 l er0))
("boilermaker" nil (b oy1 l er0 m ey1 k er0))
("boilerplate" nil (b oy1 l er0 p l ey1 t))
("boilers" nil (b oy1 l er0 z))
("boiling" nil (b oy1 l ih0 ng))
("boils" nil (b oy1 l z))
("bois" nil (b w aa1))
("boisclair" nil (b w aa0 k l eh1 r))
("boise" nil (b oy1 z iy0))
("boisen" nil (b oy1 s ax n))
("boisi" nil (b oy1 s iy0))
("boisjoly" nil (b oy1 s jh ax l iy0))
("boisseau" nil (b w aa1 s ow1))
("boissonneault" nil (b w aa1 s ax n ow0))
("boisterous" nil (b oy1 s t er0 ax s))
("boisvert" nil (b w aa0 v er1 t))
("boitano" nil (b oy1 t aa1 n ow0))
("boitnott" nil (b oy0 t n aa1 t))
("boivin" nil (b oy0 v ae1 n))
("bojanowski" nil (b ax y ax n ao1 f s k iy0))
("bojarski" nil (b ax y aa1 r s k iy0))
("bojorquez" nil (b ow0 y ao1 r k w eh0 z))
("bok" nil (b aa1 k))
("bokassa" nil (b ax k aa1 s ax))
("boker" nil (b ow1 k er0))
("bokor" nil (b ow1 k er0))
("boksen" nil (b aa1 k s ax n))
("bol" nil (b aa1 l))
("bolam" nil (b ow1 l ax m))
("bolan" nil (b ow1 l ax n))
("boland" nil (b ow1 l ax n d))
("bolander" nil (b aa1 l ax n d er0))
("bolanos" nil (b ow0 l aa1 n ow0 z))
("bolar" nil (b ow1 l er0))
("bolch" nil (b ow1 l ch))
("bolcom" nil (b ow1 l k ax m))
("bold" nil (b ow1 l d))
("bolda" nil (b ow1 l d ax))
("bolden" nil (b ow1 l d ax n))
("bolder" nil (b ow1 l d er0))
("boldest" nil (b ow1 l d ih0 s t))
("boldface" nil (b ow1 l d f ey1 s))
("boldin" nil (b ow1 l d ih0 n))
("bolding" nil (b ow1 l d ih0 ng))
("boldly" nil (b ow1 l d l iy0))
("boldman" nil (b ow1 l d m ax n))
("boldness" nil (b ow1 l d n ax s))
("boldon" nil (b ow1 l d ax n))
("bolds" nil (b ow1 l d z))
("boldt" nil (b ow1 l t))
("bolduc" nil (b ow1 l d ax k))
("bole" nil (b ow1 l))
("bolek" nil (b ow1 l ih0 k))
("bolen" nil (b ow1 l ax n))
("bolenbaugh" nil (b ax l eh1 n b ao0))
("bolender" nil (b aa1 l eh0 n d er0))
("boler" nil (b ow1 l er0))
("bolerjack" nil (b aa1 l er0 jh ax k))
("bolero" nil (b ow0 l eh1 r ow0))
("boles" nil (b ow1 l z))
("boleware" nil (b ow1 l w eh0 r))
("boley" nil (b ow1 l iy0))
("bolf" nil (b ow1 l f))
("bolger" nil (b ow1 l jh er0))
("bolich" nil (b aa1 l ih0 k))
("bolick" nil (b aa1 l ih0 k))
("boliden" nil (b ow1 l ih0 d ax n))
("bolin" nil (b ow1 l ih0 n))
("boline" nil (b aa1 l ay0 n))
("boling" nil (b ow1 l ih0 ng))
("bolinger" nil (b ow1 l ih0 ng er0))
("bolio" nil (b ow1 l iy0 ow0))
("bolitho" nil (b ow0 l ih1 th ax))
("bolivar" nil (b aa1 l ax v er0))
("bolivars" nil (b aa1 l ax v er0 z))
("bolivia" nil (b ax l ih1 v iy0 ax))
("bolivian" nil (b ow0 l ih1 v iy0 ax n))
("bolker" nil (b ow1 l k er0))
("bolkow" nil (b ow1 l k aw0))
("boll" nil (b ow1 l))
("bolla" nil (b aa1 l ax))
("bolland" nil (b aa1 l ax n d))
("bollapragada" nil (b ow0 l aa1 p r ax g aa1 d ax))
("bollard" nil (b aa1 l er0 d))
("bolle" nil (b aa1 l))
("bollen" nil (b aa1 l ax n))
("bollenbach" nil (b aa1 l ih0 n b aa0 k))
("bollenbacher" nil (b aa1 l ih0 n b aa0 k er0))
("boller" nil (b aa1 l er0))
("bolles" nil (b ow1 l z))
("bollettieri" nil (b ax l eh1 t iy0 eh1 r iy0))
("bollier" nil (b aa1 l iy0 er0))
("bollig" nil (b aa1 l ih0 g))
("bolliger" nil (b aa1 l ih0 g er0))
("bollin" nil (b aa1 l ih0 n))
("bolling" nil (b ow1 l ih0 ng))
("bollinger" nil (b aa1 l ih0 ng er0))
("bollman" nil (b aa1 l m ax n))
("bollmann" nil (b aa1 l m ax n))
("bollore" nil (b aa1 l ao0 r))
("bolls" nil (b ow1 l z))
("bologna" nil (b ax l ow1 n iy0))
("bolognese" nil (b ow0 l ow0 g n ey1 z iy0))
("bolognesi" nil (b aa1 l aa0 g n eh1 s iy0))
("bolon" nil (b ow1 l ax n))
("bolotin" nil (b aa1 l ax t ih0 n))
("bolsa" nil (b ow1 l s ax))
("bolser" nil (b ow1 l s er0))
("bolshevik" nil (b ow1 l sh ax v ih1 k))
("bolsheviks" nil (b ow1 l sh ax v ih1 k s))
("bolshevism" nil (b ow1 l sh ax v ih1 z ax m))
("bolshevist" nil (b ow1 l sh ih0 v ih1 s t))
("bolshoi" nil (b ow0 l sh oy1))
("bolson" nil (b ow1 l s ax n))
("bolstad" nil (b ow1 l s t ax d))
("bolster" nil (b ow1 l s t er0))
("bolstered" nil (b ow1 l s t er0 d))
("bolstering" nil (b ow1 l s t er0 ih0 ng))
("bolsters" nil (b ow1 l s t er0 z))
("bolt" nil (b ow1 l t))
("bolte" nil (b ow1 l t))
("bolted" nil (b ow1 l t ax d))
("bolten" nil (b ow1 l t ax n))
("bolter" nil (b ow1 l t er0))
("bolthouse" nil (b ow1 l t hh aw1 s))
("bolting" nil (b ow1 l t ih0 ng))
("bolton" nil (b ow1 l t ax n))
("bolts" nil (b ow1 l t s))
("boltz" nil (b ow1 l t s))
("boltzmanns" nil (b ow1 l t s m ax n z))
("bolus" nil (b ow1 l ax s))
("bolyard" nil (b ax l y aa1 r d))
("bolz" nil (b ow1 l z))
("bom" nil (b aa1 m))
("bomag" nil (b ow1 m ae0 g))
("boman" nil (b ow1 m ax n))
("bomar" nil (b ax m aa1 r))
("bomb" nil (b aa1 m))
("bomba" nil (b aa1 m b ax))
("bombard" nil (b aa0 m b aa1 r d))
("bombarded" nil (b aa0 m b aa1 r d ih0 d))
("bombardier" nil (b aa1 m b er0 d ih1 r))
("bombarding" nil (b aa0 m b aa1 r d ih0 ng))
("bombardment" nil (b aa0 m b aa1 r d m ax n t))
("bombardments" nil (b aa0 m b aa1 r d m ax n t s))
("bombast" nil (b aa1 m b ae0 s t))
("bombastic" nil (b aa0 m b ae1 s t ih0 k))
("bombay" nil (b aa0 m b ey1))
("bombeck" nil (b aa1 m b eh1 k))
("bombed" nil (b aa1 m d))
("bomber" nil (b aa1 m er0))
("bomberg" nil (b aa1 m b er0 g))
("bomberger" nil (b aa1 m b er0 g er0))
("bombers" nil (b aa1 m er0 z))
("bombing" nil (b aa1 m ih0 ng))
("bombings" nil (b aa1 m ih0 ng z))
("bombs" nil (b aa1 m z))
("bombshell" nil (b aa1 m sh eh1 l))
("bombshells" nil (b aa1 m sh eh1 l z))
("bomer" nil (b ow1 m er0))
("bomgardner" nil (b aa1 m g aa1 r d n er0))
("bommarito" nil (b aa1 m er0 iy1 t ow0))
("bommer" nil (b aa1 m er0))
("bon" nil (b aa1 n))
("bona" nil (b ow1 n ax))
("bonacci" nil (b ow0 n aa1 ch iy0))
("bonadio" nil (b ow0 n aa1 d iy0 ow0))
("bonadonna" nil (b aa1 n ax d aa1 n ax))
("bonafede" nil (b aa1 n ax f iy1 d))
("bonanni" nil (b ow0 n aa1 n iy0))
("bonanno" nil (b ow0 n aa1 n ow0))
("bonanza" nil (b ax n ae1 n z ax))
("bonaparte" nil (b ow1 n ax p aa1 r t))
("bonar" nil (b ow1 n er0))
("bonasera" nil (b ow0 n aa0 s eh1 r ax))
("bonaventura" nil (b ow0 n aa0 v eh0 n t uh1 r ax))
("bonaventure" nil (b aa1 n ax v eh1 n ch er0))
("bonavia" nil (b aa1 n ey1 v iy0 ax))
("bonavita" nil (b ow0 n aa0 v iy1 t ax))
("bonawitz" nil (b aa1 n ax w ih0 t s))
("bonczek" nil (b aa1 n ch eh0 k))
("bond" nil (b aa1 n d))
("bondage" nil (b aa1 n d ih0 jh))
("bondar" nil (b ax n d aa1 r))
("bonde" nil (b aa1 n d))
("bonded" nil (b aa1 n d ax d))
("bonderman" nil (b aa1 n d er0 m ax n))
("bondholder" nil (b aa1 n d hh ow1 l d er0))
("bondholders" nil (b aa1 n d hh ow1 l d er0 z))
("bondi" nil (b aa1 n d iy0))
("bonding" nil (b aa1 n d ih0 ng))
("bonds" nil (b aa1 n d z))
("bondt" nil (b aa1 n t))
("bondurant" nil (b ow0 n d uh1 r ax n t))
("bondy" nil (b aa1 n d iy0))
("bone" nil (b ow1 n))
("bonebrake" nil (b ow1 n b r ey1 k))
("bonecrusher" nil (b ow1 n k r ah1 sh er0))
("bonecutter" nil (b ow1 n k ah1 t er0))
("boned" nil (b ow1 n d))
("boneless" nil (b ow1 n l ax s))
("bonelli" nil (b ow0 n eh1 l iy0))
("bonello" nil (b ow0 n eh1 l ow0))
("bonenberger" nil (b ow1 n ax n b er0 g er0))
("bonenfant" nil (b ow1 n f ax n t))
("boner" nil (b ow1 n er0))
("bones" nil (b ow1 n z))
("bonesteel" nil (b ow1 n s t iy1 l))
("bonet" nil (b ow1 n t))
("bonetti" nil (b ow0 n eh1 t iy0))
("boney" nil (b ow1 n iy0))
("bonfanti" nil (b ow0 n f aa1 n t iy0))
("bonfield" nil (b aa1 n f iy1 l d))
("bonfiglio" nil (b aa1 n f ih1 g l iy0 ow0))
("bonfire" nil (b aa1 n f ay1 er0))
("bonfires" nil (b aa1 n f ay1 er0 z))
("bong" nil (b aa1 ng))
("bongard" nil (b ax ng g aa1 r d))
("bongarten" nil (b aa1 n g aa1 r t ax n))
("bongers" nil (b ao1 ng er0 z))
("bongiorno" nil (b ow0 n jh ao1 r n ow0))
("bongiovanni" nil (b ow0 n jh ow0 v aa1 n iy0))
("bongos" nil (b aa1 ng g ow1 z))
("bonham" nil (b aa1 n hh ax m))
("boni" nil (b ow1 n iy0))
("bonica" nil (b aa1 n ih0 k ax))
("bonifacio" nil (b aa1 n ih0 f ey1 s iy0 ow0))
("bonifas" nil (b aa1 n ih0 f ax z))
("bonifay" nil (b aa1 n ih0 f ey1))
("bonifield" nil (b aa1 n ih0 f iy1 l d))
("bonilla" nil (b ax n ih1 l ax))
("bonin" nil (b ow1 n ih0 n))
("bonine" nil (b ow0 n iy1 n iy0))
("bonini" nil (b ow0 n iy1 n iy0))
("bonino" nil (b ow0 n iy1 n ow0))
("bonior" nil (b ow1 n iy0 er0))
("bonita" nil (b ow0 n iy1 t ax))
("bonito" nil (b ax n iy1 t ow0))
("bonjour" nil (b ax n zh uh1 r))
("bonk" nil (b aa1 ng k))
("bonker" nil (b ao1 ng k er0))
("bonkers" nil (b aa1 ng k er0 z))
("bonkowski" nil (b ax ng k ao1 f s k iy0))
("bonn" nil (b aa1 n))
("bonne" nil (b aa1 n))
("bonneau" nil (b ax n ow1))
("bonnell" nil (b aa1 n ax l))
("bonnema" nil (b aa1 n ih0 m ax))
("bonner" nil (b ao1 n er0))
("bonnes" nil (b ao1 n z))
("bonnet" nil (b aa1 n ax t))
("bonnett" nil (b aa1 n ih0 t))
("bonnette" nil (b ax n eh1 t))
("bonneville" nil (b aa1 n ax v ih1 l))
("bonnevilles" nil (b aa1 n ax v ih1 l z))
("bonney" nil (b aa1 n iy0))
("bonnibel" nil (b aa1 n ih0 b ax l))
("bonnibelle" nil (b aa1 n ih0 b ax l))
("bonnie" nil (b aa1 n iy0))
("bonnin" nil (b aa1 n ih0 n))
("bonnor" nil (b aa1 n er0))
("bonny" nil (b aa1 n iy0))
("bono" nil (b ow1 n ow0))
("bonomo" nil (b aa1 n ax m ow0))
("bonsai" nil (b aa0 n s ay1))
("bonsall" nil (b ow0 n s aa1 l))
("bonser" nil (b aa1 n s er0))
("bonsignore" nil (b ow0 n s iy0 g n ao1 r iy0))
("bonta" nil (b aa1 n t ax))
("bonte" nil (b aa1 n t))
("bontempo" nil (b aa1 n t eh1 m p ow0))
("bontrager" nil (b aa1 n t r ey1 g er0))
("bonum" nil (b aa1 n ax m))
("bonura" nil (b ow0 n uh1 r ax))
("bonus" nil (b ow1 n ax s))
("bonuses" nil (b ow1 n ax s ih0 z))
("bonvillain" nil (b aa1 n v ih0 l ey1 n))
("bonville" nil (b ow1 n v ih0 l))
("bonwell" nil (b aa1 n w eh1 l))
("bonwit" nil (b aa1 n w ih0 t))
("bony" nil (b ow1 n iy0))
("bonzo" nil (b aa1 n z ow0))
("boo" nil (b uw1))
("booby" nil (b uw1 b iy0))
("boock" nil (b uw1 k))
("boodles" nil (b uw1 d ax l z))
("boody" nil (b uw1 d iy0))
("booe" nil (b aa1 ow0))
("booed" nil (b uw1 d))
("boogie" nil (b uw1 g iy0))
("booher" nil (b uw1 er0))
("booing" nil (b uw1 ih0 ng))
("book" nil (b uh1 k))
("bookbinder" nil (b uh1 k b ay1 n d er0))
("bookcase" nil (b uh1 k k ey1 s))
("bookcases" nil (b uh1 k k ey1 s ih0 z))
("bookcellar" nil (b uh1 k s eh1 l er0))
("booked" nil (b uh1 k t))
("bookends" nil (b uh1 k eh1 n d z))
("booker" nil (b uh1 k er0))
("bookers" nil (b uh1 k er0 z))
("bookies" nil (b uh1 k iy0 z))
("bookin" nil (b uh1 k ih0 n))
("booking" nil (b uh1 k ih0 ng))
("bookings" nil (b uh1 k ih0 ng z))
("bookish" nil (b uh1 k ih0 sh))
("bookkeeper" nil (b uh1 k k iy1 p er0))
("bookkeepers" nil (b uh1 k k iy1 p er0 z))
("bookkeeping" nil (b uh1 k k iy1 p ih0 ng))
("booklet" nil (b uh1 k l ih0 t))
("booklets" nil (b uh1 k l ax t s))
("bookmaking" nil (b uh1 k m ey1 k ih0 ng))
("bookman" nil (b uh1 k m ax n))
("bookmobile" nil (b uh1 k m ow0 b iy1 l))
("bookout" nil (b uh1 k aw1 t))
("books" nil (b uh1 k s))
("bookseller" nil (b uh1 k s eh1 l er0))
("booksellers" nil (b uh1 k s eh1 l er0 z))
("bookshelf" nil (b uh1 k sh eh1 l f))
("bookshelves" nil (b uh1 k sh eh1 l v z))
("bookshop" nil (b uh1 k sh aa1 p))
("bookshops" nil (b uh1 k sh aa1 p s))
("booksmith" nil (b uh1 k s m ih1 th))
("bookstop" nil (b uh1 k s t aa1 p))
("bookstore" nil (b uh1 k s t ao1 r))
("bookstores" nil (b uh1 k s t ao1 r z))
("bookwalter" nil (b uh1 k w ax l t er0))
("bookworm" nil (b uh1 k w er1 m))
("boole" nil (b uw1 l))
("boom" nil (b uw1 m))
("boomed" nil (b uw1 m d))
("boomer" nil (b uw1 m er0))
("boomerang" nil (b uw1 m er0 ae1 ng))
("boomeranged" nil (b uw1 m er0 ae1 ng d))
("boomers" nil (b uw1 m er0 z))
("boomershine" nil (b uw1 m er0 sh ay1 n))
("boomhower" nil (b uw1 m hh ow0 er0))
("booming" nil (b uw1 m ih0 ng))
("boomlet" nil (b uw1 m l ax t))
("booms" nil (b uw1 m z))
("boomsma" nil (b uw1 m z m ax))
("boomtown" nil (b uw1 m t aw1 n))
("boon" nil (b uw1 n))
("boondocks" nil (b uw1 n d aa1 k s))
("boondoggle" nil (b uw1 n d aa1 g ax l))
("boondoggles" nil (b uw1 n d aa1 g ax l z))
("boone" nil (b uw1 n))
("boonstra" nil (b uw1 n s t r ax))
("boonton" nil (b uw1 n t ax n))
("boonville" nil (b uw1 n v ih0 l))
("boop" nil (b uw1 p))
("boor" nil (b uh1 r))
("booras" nil (b uh1 r ax z))
("boord" nil (b uh1 r d))
("boorda" nil (b uh1 r d ax))
("boorish" nil (b uh1 r ih0 sh))
("boorishness" nil (b uh1 r ih0 sh n ax s))
("boorman" nil (b uh1 r m ax n))
("boors" nil (b uh1 r z))
("boorstin" nil (b uw1 r s t ih0 n))
("boos" nil (b uw1 z))
("boose" nil (b uw1 s))
("boost" nil (b uw1 s t))
("boosted" nil (b uw1 s t ih0 d))
("booster" nil (b uw1 s t er0))
("boosterism" nil (b uw1 s t er0 ih1 z ax m))
("boosters" nil (b uw1 s t er0 z))
("boosting" nil (b uw1 s t ih0 ng))
("boosts" nil (b uw1 s t s))
("boot" nil (b uw1 t))
("boote" nil (b uw1 t))
("booted" nil (b uw1 t ih0 d))
("booten" nil (b uw1 t ax n))
("booth" nil (b uw1 th))
("boothby" nil (b uw1 th b iy0))
("boothe" nil (b uw1 dh))
("boothman" nil (b uw1 th m ax n))
("boothroyd" nil (b uw1 th r oy1 d))
("booths" nil (b uw1 th s))
("booties" nil (b uw1 t iy0 z))
("booting" nil (b uw1 t ih0 ng))
("bootleg" nil (b uw1 t l eh1 g))
("bootlegger" nil (b uw1 t l eh1 g er0))
("bootleggers" nil (b uw1 t l eh1 g er0 z))
("bootlegging" nil (b uw1 t l eh1 g ih0 ng))
("bootlegs" nil (b uw1 t l eh1 g z))
("booton" nil (b uw1 t ax n))
("boots" nil (b uw1 t s))
("bootstrap" nil (b uw1 t s t r ae1 p))
("bootstraps" nil (b uw1 t s t r ae1 p s))
("booty" nil (b uw1 t iy0))
("bootz" nil (b uw1 t s))
("booz" nil (b uw1 z))
("booze" nil (b uw1 z))
("boozer" nil (b uw1 z er0))
("boozier" nil (b uw1 z iy0 er0))
("boozy" nil (b uw1 z iy0))
("bop" nil (b aa1 p))
("bopeep" nil (b ow0 p iy1 p))
("bopera" nil (b ow0 p eh1 r ax))
("bophuthatswana" nil (b ow1 f uw0 th aa0 t s w aa1 n ax))
("bopp" nil (b aa1 p))
("bopper" nil (b aa1 p er0))
("boppers" nil (b aa1 p er0 z))
("boquist" nil (b aa1 k w ih0 s t))
("bora" nil (b ao1 r ax))
("borah" nil (b ao1 r ax))
("borak" nil (b ao1 r ax k))
("boral" nil (b ao1 r ax l))
("borawski" nil (b er0 aa1 f s k iy0))
("borax" nil (b ao1 r ae1 k s))
("borba" nil (b ao1 r b ax))
("borchard" nil (b er0 sh aa1 r d))
("borchardt" nil (b er0 sh aa1 r d t))
("borchelt" nil (b ao1 r k ih0 l t))
("borcherding" nil (b ao1 r k er0 d ih0 ng))
("borchers" nil (b ao1 r k er0 z))
("borchert" nil (b ao1 r k er0 t))
("borck" nil (b ao1 r k))
("borda" nil (b ao1 r d ax))
("bordallo" nil (b ao0 r d ae1 l ow0))
("bordas" nil (b ao1 r d ax z))
("bordeau" nil (b er0 d ow1))
("bordeaux" nil (b ao0 r d ow1))
("bordello" nil (b ao0 r d eh1 l ow1))
("bordelon" nil (b ao1 r d ih0 l aa0 n))
("borden" nil (b ao1 r d ax n))
("bordenave" nil (b ao0 r d eh0 n aa1 v iy0))
("border" nil (b ao1 r d er0))
("bordered" nil (b ao1 r d er0 d))
("bordering" nil (b ao1 r d er0 ih0 ng))
("borderline" nil (b ao1 r d er0 l ay1 n))
("borders" nil (b ao1 r d er0 z))
("bordes" nil (b ao1 r d z))
("bordner" nil (b ao1 r d n er0))
("bordonaro" nil (b ao0 r d ow0 n aa1 r ow0))
("bordwell" nil (b ao1 r d w eh0 l))
("bore" nil (b ao1 r))
("borealis" nil (b ao1 r iy0 ae1 l ax s))
("bored" nil (b ao1 r d))
("boredom" nil (b ao1 r d ax m))
("borek" nil (b ao1 r ih0 k))
("borel" nil (b ao1 r ax l))
("borell" nil (b ao1 r ax l))
("borella" nil (b ao0 r eh1 l ax))
("borelli" nil (b ao0 r eh1 l iy0))
("borello" nil (b ao0 r eh1 l ow0))
("boren" nil (b ao1 r ax n))
("borenstein" nil (b ao1 r ax n s t ay1 n))
("borer" nil (b ao1 r er0))
("bores" nil (b ao1 r z))
("borg" nil (b ao1 r g))
("borge" nil (b ao1 r jh))
("borgelt" nil (b ao1 r g ih0 l t))
("borgen" nil (b ao1 r g ax n))
("borger" nil (b ao1 r g er0))
("borgerding" nil (b ao1 r g er0 d ih0 ng))
("borgert" nil (b ao1 r g er0 t))
("borges" nil (b ao1 r g ey0 s))
("borgeson" nil (b ao1 r g ih0 s ax n))
("borghi" nil (b ao1 r g iy0))
("borgia" nil (b ao1 r jh ax))
("borgman" nil (b ao1 r g m ax n))
("borgmann" nil (b ao1 r g m ax n))
("borgmeyer" nil (b ao1 r g m ay0 er0))
("borgstrom" nil (b ao1 r g s t r ax m))
("borgwardt" nil (b ao1 r g w ao1 r t))
("boric" nil (b ao1 r ih0 k))
("borin" nil (b ao1 r ih0 n))
("boring" nil (b ao1 r ih0 ng))
("boris" nil (b ao1 r ih0 s))
("borja" nil (b ao1 r y ax))
("borjas" nil (b ao1 r y ax z))
("bork" nil (b ao1 r k))
("borkenhagen" nil (b ao1 r k ih0 n hh aa1 g ax n))
("borkowski" nil (b er0 k ao1 f s k iy0))
("borland" nil (b ao1 r l ax n d))
("borman" nil (b ao1 r m ax n))
("bormann" nil (b ao1 r m ax n))
("born" nil (b ao1 r n))
("borne" nil (b ao1 r n))
("borneman" nil (b ao1 r n m ax n))
("bornemann" nil (b ao1 r n m ax n))
("borneo" nil (b ao1 r n iy0 ow1))
("borner" nil (b ao1 r n er0))
("bornholdt" nil (b ao1 r n hh ow0 l t))
("bornhorst" nil (b ao1 r n hh ao0 r s t))
("bornman" nil (b ao1 r n m ax n))
("borns" nil (b ao1 r n z))
("bornstein" nil (b ao1 r n s t ay1 n))
("borntrager" nil (b ao1 r n t r ey0 g er0))
("boro" nil (b er1 ow0))
("boroff" nil (b ao1 r ao0 f))
("boroian" nil (b ao0 r oy1 ax n))
("boron" nil (b ao1 r aa1 n))
("boros" nil (b er1 ow0 z))
("borosage" nil (b ao1 r ax s ih0 jh))
("boroski" nil (b er0 aw1 s k iy0))
("borough" nil (b er1 ow1))
("boroughs" nil (b er1 ow1 z))
("borowiak" nil (b er0 aw1 iy0 ae0 k))
("borowicz" nil (b er1 ow0 v ih0 ch))
("borowiec" nil (b er0 aw1 iy0 k))
("borowski" nil (b er0 ao1 f s k iy0))
("borowsky" nil (b er0 aw1 s k iy0))
("borowy" nil (b er0 aw1 iy0))
("borquez" nil (b ao0 r k w eh1 z))
("borras" nil (b ao1 r ax z))
("borre" nil (b ao1 r))
("borrego" nil (b ao0 r eh1 g ow0))
("borrell" nil (b ao0 r ey1 l))
("borrelli" nil (b ao0 r eh1 l iy0))
("borrello" nil (b ao1 r eh1 l ow0))
("borrero" nil (b ao0 r eh1 r ow0))
("borris" nil (b ao1 r ih0 s))
("borror" nil (b ao1 er0 r))
("borrow" nil (b aa1 r ow1))
("borrowed" nil (b aa1 r ow1 d))
("borrower" nil (b aa1 r ow0 er0))
("borrowers" nil (b aa1 r ow0 er0 z))
("borrowing" nil (b aa1 r ow0 ih0 ng))
("borrowings" nil (b aa1 r ow0 ih0 ng z))
("borrowman" nil (b aa1 r ow0 m ax n))
("borrows" nil (b aa1 r ow0 z))
("borruso" nil (b ao1 r uw1 s ow0))
("bors" nil (b ao1 r z))
("borsch" nil (b ao1 r sh))
("borscht" nil (b ao1 r sh t))
("borse" nil (b ao1 r s))
("borseth" nil (b ao1 r s ih0 th))
("borski" nil (b ao1 r s k iy0))
("borst" nil (b ao1 r s t))
("borsuk" nil (b ao1 r s ax k))
("bort" nil (b ao1 r t))
("bortel" nil (b ao1 r t eh1 l))
("borten" nil (b ao1 r t ax n))
("borth" nil (b ao1 r th))
("borthwick" nil (b ao1 r th w ih0 k))
("bortle" nil (b ao1 r t ax l))
("bortner" nil (b ao1 r t n er0))
("bortnick" nil (b ao1 r t n ih0 k))
("borton" nil (b ao1 r t ax n))
("bortz" nil (b ao1 r t s))
("boruch" nil (b ao1 r ax k))
("borucki" nil (b er0 ah1 t s k iy0))
("boruff" nil (b ao1 r ax f))
("borum" nil (b ao1 r ax m))
("borunda" nil (b ao0 r uw1 n d ax))
("borup" nil (b ao1 r ax p))
("borwn" nil (b ao1 r w n))
("borys" nil (b ao1 r iy0 z))
("bos" nil (b aa1 s))
("bosak" nil (b ow1 s ax k))
("bosarge" nil (b aa1 s aa0 r g))
("boscarino" nil (b ow0 s k aa0 r iy1 n ow0))
("bosch" nil (b ao1 sh))
("boschee" nil (b ao1 sh iy0))
("boschen" nil (b ao1 sh ax n))
("boschert" nil (b ao1 sh er0 t))
("boschwitz" nil (b ao1 sh w ih0 t s))
("boscia" nil (b ow1 s ch ax))
("bosco" nil (b ao1 s k ow0))
("bose" nil (b ow1 z))
("boseman" nil (b ow1 s m ax n))
("bosendorfer" nil (b ow1 z ax n d ao1 r f er0))
("boser" nil (b ow1 z er0))
("bosh" nil (b aa1 sh))
("boshart" nil (b aa1 sh aa0 r t))
("boshears" nil (b aa1 sh ih0 r z))
("boshell" nil (b aa1 sh ax l))
("bosher" nil (b aa1 sh er0))
("boshers" nil (b aa1 sh er0 z))
("boskin" nil (b ao1 s k ih0 n))
("bosko" nil (b ow1 s k ow0))
("boskovich" nil (b aa1 s k ax v ih0 ch))
("boslego" nil (b ao1 s l ey1 g ow0))
("bosler" nil (b aa1 s ax l er0))
("bosley" nil (b aa1 z l iy0))
("bosma" nil (b ow1 s m ax))
("bosman" nil (b aa1 s m ax n))
("bosnia" nil (b aa1 z n iy0 ax))
("bosnian" nil (b aa1 z n iy0 ax n))
("bosnians" nil (b aa1 z n iy0 ax n z))
("bosnias" nil (b aa1 z n iy0 ax z))
("boso" nil (b ow1 s ow0))
("bosom" nil (b uh1 z ax m))
("bosquez" nil (b ow0 s k w eh1 z))
("boss" nil (b aa1 s))
("bossard" nil (b ax s aa1 r d))
("bossart" nil (b ax s aa1 r t))
("bosse" nil (b aa1 s))
("bosserman" nil (b ao1 s er0 m ax n))
("bossert" nil (b aa1 s er0 t))
("bosses" nil (b ao1 s ih0 z))
("bosshardt" nil (b ao1 s hh aa1 r t))
("bosshart" nil (b ao1 s hh aa1 r t))
("bossi" nil (b ow1 s iy0))
("bossidy" nil (b ao1 s ih0 d iy0))
("bossie" nil (b ao1 s iy0))
("bossier" nil (b ao1 s iy0 er0))
("bossler" nil (b aa1 s ax l er0))
("bossman" nil (b ao1 s m ax n))
("bosso" nil (b ow1 s ow0))
("bossy" nil (b ao1 s iy0))
("bost" nil (b aa1 s t))
("bostelman" nil (b aa1 s t ax l m ax n))
("boster" nil (b aa1 s t er0))
("bostian" nil (b aa1 s ch ih0 n))
("bostic" nil (b aa1 s t ih0 k))
("bostick" nil (b ow1 s t ih0 k))
("bostock" nil (b ow1 s t aa1 k))
("boston" nil (b aa1 s t ax n))
("bostonian" nil (b ao1 s t ow1 n iy0 ax n))
("bostonians" nil (b aa1 s t ow0 n iy0 ax n z))
("bostra" nil (b aa1 s t r ax))
("bostrom" nil (b aa1 s t r ax m))
("bostwick" nil (b aa1 s t w ih0 k))
("boswell" nil (b aa1 z w eh0 l))
("bosworth" nil (b ao1 z w er0 th))
("botanic" nil (b ax t ae1 n ih0 k))
("botanical" nil (b ax t ae1 n ih0 k ax l))
("botanically" nil (b ax t ae1 n ax k ax l iy0))
("botanist" nil (b aa1 t ax n ax s t))
("botany" nil (b aa1 t ax n iy0))
("botched" nil (b aa1 ch t))
("boteler" nil (b aa1 t ax l er0))
("botelho" nil (b ow0 t eh1 l ow0))
("botello" nil (b ow0 t eh1 l ow0))
("botero" nil (b ow0 t eh1 r ow0))
("botfly" nil (b aa1 t f l ay1))
("both" nil (b ow1 th))
("botha" nil (b aa1 th ax))
("botham" nil (b aa1 th ax m))
("bothe" nil (b ow1 dh))
("bothell" nil (b aa1 th ax l))
("bother" nil (b aa1 dh er0))
("bothered" nil (b aa1 dh er0 d))
("bothering" nil (b aa1 dh er0 ih0 ng))
("bothers" nil (b aa1 dh er0 z))
("bothersome" nil (b aa1 dh er0 s ax m))
("bothun" nil (b aa1 th ax n))
("bothwell" nil (b aa1 th w eh1 l))
("botin" nil (b aa1 t ih0 n))
("botkin" nil (b aa1 t k ih0 n))
("botkins" nil (b aa1 t k ih0 n z))
("botner" nil (b aa1 t n er0))
("botolf" nil (b aa1 t ow0 l f))
("botos" nil (b ow1 t ow0 z))
("botrytis" nil (b ax t r ih1 t ih0 s))
("botsford" nil (b aa1 t s f er0 d))
("botshabelo" nil (b aa1 ch ax b eh1 l ow0))
("botswana" nil (b aa0 t s w aa1 n ax))
("bott" nil (b aa1 t))
("botta" nil (b aa1 t ax))
("bottari" nil (b ow0 t aa1 r iy0))
("bottcher" nil (b aa1 t ch er0))
("bottel" nil (b aa1 t ax l))
("botten" nil (b aa1 t ax n))
("bottenfield" nil (b ax t eh1 n f iy0 l d))
("bottger" nil (b aa1 t g er0))
("botting" nil (b aa1 t ih0 ng))
("bottini" nil (b ow0 t iy1 n iy0))
("bottino" nil (b ow0 t iy1 n ow0))
("bottle" nil (b aa1 t ax l))
("bottled" nil (b aa1 t ax l d))
("bottleneck" nil (b aa1 t ax l n eh1 k))
("bottlenecks" nil (b aa1 t ax l n eh1 k s))
("bottler" nil (b aa1 t l er0))
("bottlers" nil (b aa1 t l er0 z))
("bottles" nil (b aa1 t ax l z))
("bottling" nil (b aa1 t ax l ih0 ng))
("bottom" nil (b aa1 t ax m))
("bottomed" nil (b aa1 t ax m d))
("bottomfish" nil (b aa1 t ax m f ih1 sh))
("bottoming" nil (b aa1 t ax m ih0 ng))
("bottomless" nil (b aa1 t ax m l ax s))
("bottomley" nil (b aa1 t ax m l iy0))
("bottoms" nil (b aa1 t ax m z))
("bottone" nil (b ow0 t ow1 n iy0))
("bottorf" nil (b aa1 t er0 f))
("bottorff" nil (b aa1 t er0 f))
("bottrell" nil (b aa1 t r ax l))
("botts" nil (b aa1 t s))
("botulism" nil (b aa1 ch uw0 l ih1 z ax m))
("botz" nil (b aa1 t s))
("bouch" nil (b aw1 ch))
("bouchard" nil (b uw0 sh aa1 r d))
("bouche" nil (b aw1 ch))
("boucher" nil (b uw1 sh ax))
("bouchey" nil (b uw0 sh iy1))
("bouchie" nil (b aw1 ch iy0))
("bouchillon" nil (b aw1 ch ih0 l aa0 n))
("bouck" nil (b ow1 k))
("boudin" nil (b uw1 d ih0 n))
("boudoin" nil (b uw0 d oy1 n))
("boudoir" nil (b uw1 d oy1 r))
("boudreau" nil (b uw1 d r ow1))
("boudreaux" nil (b uw1 d r ow1))
("bouffard" nil (b uw0 f aa1 r d))
("bougainville" nil (b uw1 g ih0 n v ih1 l))
("bougainvillea" nil (b uw1 g ey1 n v ih1 l iy0 ax))
("bough" nil (b aw1))
("boughan" nil (b aw1 ax n))
("bougher" nil (b aw1 er0))
("boughman" nil (b aw1 m ax n))
("boughner" nil (b aw1 n er0))
("boughs" nil (b aw1 z))
("bought" nil (b aa1 t))
("boughten" nil (b ao1 t ax n))
("boughter" nil (b ao1 t er0))
("boughton" nil (b aw1 t ax n))
("bougie" nil (b uw1 zh iy1))
("bouie" nil (b uw0 iy1))
("bouillon" nil (b uw1 w ih1 l ax n))
("bouknight" nil (b aw1 k n ay0 t))
("boulais" nil (b uw0 l ey1))
("boulalas" nil (b uw1 l ax l ax s))
("boulanger" nil (b aw1 l ax ng er0))
("boulangerie" nil (b uw1 l ae1 ng g er0 iy0))
("boulay" nil (b uw0 l ey1))
("boulden" nil (b uh1 d ax n))
("boulder" nil (b ow1 l d er0))
("boulders" nil (b ow1 l d er0 z))
("boule" nil (b uw1 l))
("bouler" nil (b aw1 l er0))
("boulet" nil (b uw0 l eh1 t))
("boulette" nil (b uw1 l eh1 t))
("boulevard" nil (b uh1 l ax v aa1 r d))
("boulevards" nil (b uh1 l ax v aa1 r d z))
("bouley" nil (b uw0 l iy1))
("boulez" nil (b uw1 l eh0 z))
("boulier" nil (b uw1 l iy0 er0))
("boullion" nil (b uw1 l y ax n))
("boulos" nil (b uw0 l ow1 z))
("boulter" nil (b ow1 l t er0))
("boultinghouse" nil (b aw1 l t ih0 ng hh aw1 s))
("boulton" nil (b aw1 l t ax n))
("boulware" nil (b aw1 l w eh0 r))
("bouma" nil (b ow1 m ax))
("bouman" nil (b uw0 m ae1 n))
("bounce" nil (b aw1 n s))
("bounced" nil (b aw1 n s t))
("bouncer" nil (b aw1 n s er0))
("bouncers" nil (b aw1 n s er0 z))
("bounces" nil (b aw1 n s ih0 z))
("bouncing" nil (b aw1 n s ih0 ng))
("bouncy" nil (b aw1 n s iy0))
("bound" nil (b aw1 n d))
("boundaries" nil (b aw1 n d er0 iy0 z))
("boundary" nil (b aw1 n d er0 iy0))
("bounded" nil (b aw1 n d ax d))
("bounding" nil (b aw1 n d ih0 ng))
("boundless" nil (b aw1 n d l ax s))
("bounds" nil (b aw1 n d z))
("boundy" nil (b aw1 n d iy0))
("bounties" nil (b aw1 n t iy0 z))
("bountiful" nil (b aw1 n t ih0 f ax l))
("bounty" nil (b aw1 n t iy0))
("bouquet" nil (b uw0 k ey1))
("bouquets" nil (b ow0 k ey1 z))
("bour" nil (b aw1 r))
("bourassa" nil (b uh0 r aa1 s ax))
("bourbeau" nil (b uh0 r b ow1))
("bourbon" nil (b er1 b ax n))
("bourbons" nil (b er1 b ax n z))
("bourcier" nil (b aw1 r k iy0 er0))
("bourdeau" nil (b uh0 r d ow1))
("bourg" nil (b ao1 r g))
("bourgault" nil (b uh0 r g ow1))
("bourgeois" nil (b uh0 r zh w aa1))
("bourgeoisie" nil (b uh1 r zh w aa1 z iy1))
("bourget" nil (b uh0 r zh eh1 t))
("bourgoin" nil (b uh0 r g oy1 n))
("bourguiba" nil (b ao0 r g w iy1 b ax))
("bourguignon" nil (b uh1 r g iy0 n y ow1 n))
("bourke" nil (b er1 k))
("bourland" nil (b uh0 r l ae1 n d))
("bourn" nil (b ao1 r n))
("bourne" nil (b ao1 r n))
("bournewood" nil (b ao1 r n w uh1 d))
("bournias" nil (b ao1 r n iy0 ax s))
("bournonville" nil (b ao1 r n ax n v ih1 l))
("bourque" nil (b uh1 r k))
("bourquin" nil (b uh0 r k w ae1 n))
("bourret" nil (b uh0 r eh1 t))
("bourse" nil (b ao1 r s))
("bourses" nil (b ao1 r s ih0 z))
("bouse" nil (b aw1 s))
("bouska" nil (b uw1 s k ax))
("bousman" nil (b ax s m ae1 n))
("bousquet" nil (b uw0 s k eh1 t))
("boussac" nil (b uw1 s ae0 k))
("bout" nil (b aw1 t))
("boutell" nil (b uw0 t eh1 l))
("boutelle" nil (b uw1 t eh1 l))
("bouthillier" nil (b aw1 th ax l iy0 er0))
("boutilier" nil (b aw1 t ax l iy0 er0))
("boutin" nil (b uw0 t ae1 n))
("boutique" nil (b uw0 t iy1 k))
("boutiques" nil (b uw0 t iy1 k s))
("bouton" nil (b aw1 t ax n))
("boutros" nil (b uw1 t r ow1 s))
("bouts" nil (b aw1 t s))
("boutte" nil (b uw1 t))
("boutwell" nil (b aw1 t w eh1 l))
("bouvier" nil (b uw1 v iy0 ey1))
("bouwens" nil (b aw1 ax n z))
("bouwkamp" nil (b aw1 k ae1 m p))
("bouwman" nil (b aw1 m ax n))
("bouwsma" nil (b uw1 z m ax))
("bouyer" nil (b oy1 er0))
("bouygues" nil (b oy1 zh ey1))
("bouza" nil (b uw1 z ax))
("bova" nil (b ow1 v ax))
("bovard" nil (b ax v aa1 r d))
("bove" nil (b ow1 v))
("bovee" nil (b ah1 v iy0))
("boven" nil (b ah1 v ax n))
("bovenzi" nil (b ow0 v eh1 n z iy0))
("boveri" nil (b ow0 v eh1 r iy0))
("bovespa" nil (b ow1 v s p aa0))
("bovey" nil (b ow1 v iy0))
("bovin" nil (b ow1 v ih0 n))
("bovine" nil (b ow1 v ay1 n))
("bovino" nil (b ow0 v iy1 n ow0))
("bow" nil (b aw1))
("bowar" nil (b ow1 er0))
("boward" nil (b ow1 er0 d))
("bowater" nil (b aw1 ah1 t er0))
("bowcan" nil (b ow1 k ax n))
("bowden" nil (b ow1 d ax n))
("bowdish" nil (b ow1 d ih0 sh))
("bowditch" nil (b ow1 d ih0 ch))
("bowdle" nil (b ow1 d ax l))
("bowdoin" nil (b ow0 d oy1 n))
("bowe" nil (b ow1))
("bowed" nil (b aw1 d))
("bowel" nil (b aw1 ax l))
("bowell" nil (b aa1 w eh0 l))
("bowels" nil (b aw1 ax l z))
("bowen" nil (b ow1 ax n))
("bowens" nil (b ow1 ax n z))
("bower" nil (b aw1 er0))
("bowerman" nil (b ow1 er0 m ax n))
("bowermaster" nil (b ow1 er0 m ae0 s t er0))
("bowers" nil (b aw1 er0 z))
("bowersock" nil (b ow0 er1 s ax k))
("bowersox" nil (b ow0 er1 s aa0 k s))
("bowery" nil (b aw1 er0 iy0))
("bowes" nil (b ow1 z))
("bowick" nil (b ow1 ih0 k))
("bowie" nil (b ow1 iy0))
("bowing" nil (b ow1 ih0 ng))
("bowker" nil (b ow1 k er0))
("bowl" nil (b ow1 l))
("bowland" nil (b ow1 l ax n d))
("bowlby" nil (b ow1 l b iy0))
("bowlds" nil (b ow1 l d z))
("bowled" nil (b ow1 l d))
("bowlen" nil (b ow1 l ax n))
("bowler" nil (b ow1 l er0))
("bowlers" nil (b ow1 l er0 z))
("bowles" nil (b ow1 l z))
("bowley" nil (b ow1 l iy0))
("bowlin" nil (b ow1 l ih0 n))
("bowling" nil (b ow1 l ih0 ng))
("bowls" nil (b ow1 l z))
("bowlus" nil (b ow1 l ax s))
("bowman" nil (b ow1 m ax n))
("bowmen" nil (b ow1 m ax n))
("bowmer" nil (b ow1 m er0))
("bown" nil (b ow1 n))
("bownds" nil (b ow1 n d z))
("bowne" nil (b ow1 n))
("bowring" nil (b ow1 r ih0 ng))
("bowron" nil (b ow1 r ax n))
("bows" nil (b aw1 z))
("bowse" nil (b ow1 s))
("bowsed" nil (b aw1 z d))
("bowser" nil (b ow1 z er0))
("bowses" nil (b aw1 z ih0 z))
("bowsher" nil (b ow1 sh er0))
("bowsing" nil (b aw1 z ih0 ng))
("bowyer" nil (b ow1 y er0))
("box" nil (b aa1 k s))
("boxberger" nil (b aa1 k s b er0 g er0))
("boxcar" nil (b aa1 k s k aa1 r))
("boxcars" nil (b aa1 k s k aa1 r z))
("boxed" nil (b aa1 k s t))
("boxell" nil (b aa1 k s ax l))
("boxer" nil (b aa1 k s er0))
("boxers" nil (b aa1 k s er0 z))
("boxes" nil (b aa1 k s ax z))
("boxing" nil (b aa1 k s ih0 ng))
("boxley" nil (b aa1 k s l iy0))
("boxwell" nil (b aa1 k s w eh1 l))
("boxwood" nil (b aa1 k s w uh1 d))
("boxx" nil (b aa1 k s k s))
("boxy" nil (b aa1 k s iy0))
("boy" nil (b oy1))
("boyack" nil (b oy1 ax k))
("boyajian" nil (b oy0 ae1 jh iy0 ax n))
("boyan" nil (b oy1 aa0 n))
("boyar" nil (b ow0 y aa1 r))
("boyce" nil (b oy1 s))
("boycott" nil (b oy1 k aa1 t))
("boycotted" nil (b oy1 k aa1 t ih0 d))
("boycotting" nil (b oy1 k aa1 t ih0 ng))
("boycotts" nil (b oy1 k aa1 t s))
("boyd" nil (b oy1 d))
("boyde" nil (b oy1 d))
("boyden" nil (b oy1 d ax n))
("boydston" nil (b oy1 d s t ax n))
("boydstun" nil (b oy1 d s t ax n))
("boye" nil (b oy1))
("boyea" nil (b oy1 iy0 ax))
("boyer" nil (b oy1 er0))
("boyers" nil (b oy1 er0 z))
("boyes" nil (b oy1 z))
("boyett" nil (b oy1 ih0 t))
("boyette" nil (b oy1 eh1 t))
("boyfriend" nil (b oy1 f r eh1 n d))
("boyfriends" nil (b oy1 f r eh1 n d z))
("boyhood" nil (b oy1 hh uh1 d))
("boyington" nil (b oy1 ih0 ng t ax n))
("boyish" nil (b oy1 ih0 sh))
("boykin" nil (b oy1 k ih0 n))
("boykins" nil (b oy1 k ih0 n z))
("boyko" nil (b oy1 k ow0))
("boylan" nil (b oy1 l ax n))
("boyland" nil (b oy1 l ax n d))
("boyle" nil (b oy1 l))
("boylen" nil (b oy1 l ax n))
("boyles" nil (b oy1 l z))
("boylls" nil (b oy1 l z))
("boylston" nil (b oy1 l s t ax n))
("boyne" nil (b oy1 n))
("boynton" nil (b oy1 n t ax n))
("boys" nil (b oy1 z))
("boysel" nil (b oy1 s ax l))
("boysen" nil (b oy1 s ax n))
("boyson" nil (b oy1 z ax n))
("boyt" nil (b oy1 t))
("boyte" nil (b oy1 t))
("boyter" nil (b oy1 t er0))
("boyum" nil (b oy0 ah1 m))
("boyz" nil (b oy1 z))
("boza" nil (b ow1 z ax))
("bozard" nil (b ax z aa1 r d))
("bozarth" nil (b aa1 z er0 th))
("boze" nil (b ow1 z))
("bozek" nil (b ow1 z eh0 k))
("bozell" nil (b ow0 z eh1 l))
("bozeman" nil (b ow1 z m ax n))
("bozian" nil (b ow1 z iy0 ax n))
("bozic" nil (b aa1 z ih0 k))
("bozich" nil (b aa1 z ih0 hh))
("bozman" nil (b aa1 z m ax n))
("bozo" nil (b ow1 z ow1))
("bozza" nil (b aa1 z ax))
("bozzi" nil (b aa1 z iy0))
("bozzo" nil (b aa1 z ow0))
("bra" nil (b r aa1))
("braaksma" nil (b r aa1 k s m ax))
("braasch" nil (b r aa1 sh))
("braaten" nil (b r aa1 ey0 t ax n))
("braatz" nil (b r aa1 t s))
("brabant" nil (b r aa1 b ax n t))
("brabec" nil (b r aa1 b ih0 k))
("brabender" nil (b r ae1 b eh0 n d er0))
("brabham" nil (b r ae1 b hh ax m))
("brabson" nil (b r ae1 b s ax n))
("brac" nil (b r ae1 k))
("bracamonte" nil (b r ae1 k ax m aa1 n t iy0))
("bracamontes" nil (b r ae1 k ax m aa1 n t iy0 z))
("bracci" nil (b r aa1 ch iy0))
("bracco" nil (b r ae1 k ow0))
("brace" nil (b r ey1 s))
("braced" nil (b r ey1 s t))
("bracelet" nil (b r ey1 s l ax t))
("bracelets" nil (b r ey1 s l ih0 t s))
("bracer" nil (b r ey1 s er0))
("bracero" nil (b r aa0 ch eh1 r ow0))
("braces" nil (b r ey1 s ax z))
("bracewell" nil (b r ey1 s w eh1 l))
("bracey" nil (b r ey1 s iy0))
("brach" nil (b r ae1 ch))
("bracher" nil (b r ae1 k er0))
("brachfeld" nil (b r aa1 k f eh1 l d))
("brachiopods" nil (b r ey1 k iy0 ax p aa1 d z))
("bracht" nil (b r ae1 k t))
("bracing" nil (b r ey1 s ih0 ng))
("brack" nil (b r ae1 k))
("brackbill" nil (b r ae1 k b ax l))
("brackeen" nil (b r ax k iy1 n))
("bracken" nil (b r ae1 k ax n))
("brackenbury" nil (b r ae1 k ax n b eh1 r iy0))
("brackens" nil (b r ae1 k ax n z))
("bracker" nil (b r ae1 k er0))
("bracket" nil (b r ae1 k ih0 t))
("brackets" nil (b r ae1 k ax t s))
("brackett" nil (b r ae1 k ih0 t))
("brackin" nil (b r ae1 k ih0 n))
("brackins" nil (b r ae1 k ih0 n z))
("brackish" nil (b r ae1 k ih0 sh))
("brackman" nil (b r ae1 k m ax n))
("bracknell" nil (b r ae0 k n eh1 l))
("brackney" nil (b r ae1 k n iy0))
("bracy" nil (b r ey1 s iy0))
("brad" nil (b r ae1 d))
("bradberry" nil (b r ae1 d b eh1 r iy0))
("bradburn" nil (b r ae1 d b er1 n))
("bradbury" nil (b r ae1 d b eh1 r iy0))
("bradco" nil (b r ae1 d k ow0))
("braddock" nil (b r ae1 d ax k))
("braddy" nil (b r ae1 d iy0))
("bradeen" nil (b r ax d iy1 n))
("brademas" nil (b r ax d ey1 m ax s))
("braden" nil (b r ey1 d ax n))
("bradenton" nil (b r ae1 d ax n t ax n))
("brader" nil (b r ae1 d er0))
("bradfield" nil (b r ae1 d f iy0 l d))
("bradford" nil (b r ae1 d f er0 d))
("bradfords" nil (b r ae1 d f er0 d z))
("bradham" nil (b r ae1 d hh ax m))
("bradish" nil (b r ae1 d ih0 sh))
("bradlee" nil (b r ae1 d l iy1))
("bradlees" nil (b r ae1 d l iy1 z))
("bradley" nil (b r ae1 d l iy0))
("bradner" nil (b r ae1 d n er0))
("bradney" nil (b r ae1 d n iy0))
("bradshaw" nil (b r ae1 d sh ao1))
("bradsher" nil (b r ae1 d sh er0))
("bradstreet" nil (b r ae1 d s t r iy1 t))
("bradt" nil (b r ae1 d t))
("bradtke" nil (b r ae1 d k iy0))
("bradtmiller" nil (b r ae1 t m ih1 l er0))
("bradway" nil (b r ae1 d w ey1))
("bradwell" nil (b r ae1 d w eh1 l))
("brady" nil (b r ey1 d iy0))
("bradycardia" nil (b r ae1 d ax k aa1 r d iy0 ax))
("bradykinin" nil (b r ax d ih1 k ih0 n ih0 n))
("brae" nil (b r ey1))
("braendstroem" nil (b r ae1 n d s t r ow0 m))
("braff" nil (b r ae1 f))
("brafford" nil (b r ae1 f er0 d))
("brag" nil (b r ae1 g))
("braga" nil (b r aa1 g ax))
("bragan" nil (b r ey1 g ax n))
("bragdon" nil (b r ae1 g d ax n))
("brager" nil (b r ey1 g er0))
("bragg" nil (b r ae1 g))
("braggadocio" nil (b r ae1 g ax d ow1 sh iy0 ow1))
("bragged" nil (b r ae1 g d))
("bragging" nil (b r ae1 g ih0 ng))
("braggiotti" nil (b r ae1 z iy0 aa1 t iy0))
("braggs" nil (b r ae1 g z))
("brags" nil (b r ae1 g z))
("braham" nil (b r ae1 hh ax m))
("brahm" nil (b r aa1 m))
("brahmin" nil (b r aa1 m ih0 n))
("brahms" nil (b r aa1 m z))
("braid" nil (b r ey1 d))
("braided" nil (b r ey1 d ih0 d))
("braiding" nil (b r ey1 d ih0 ng))
("braids" nil (b r ey1 d z))
("braidwood" nil (b r ey1 d w uh1 d))
("brailey" nil (b r ey1 l iy0))
("braille" nil (b r ey1 l))
("brailles" nil (b r ey1 l z))
("brailsford" nil (b r ey1 l s f er0 d))
("brain" nil (b r ey1 n))
("brainard" nil (b r ey1 n er0 d))
("brainchild" nil (b r ey1 n ch ay1 l d))
("brainer" nil (b r ey1 n er0))
("brainerd" nil (b r ey1 n er0 d))
("brainpower" nil (b r ey1 n p aw1 er0))
("brains" nil (b r ey1 n z))
("brainstorm" nil (b r ey1 n s t ao1 r m))
("brainstorming" nil (b r ey1 n s t ao1 r m ih0 ng))
("braintree" nil (b r ey1 n t r iy1))
("brainwash" nil (b r ey1 n w aa1 sh))
("brainwashed" nil (b r ey1 n w aa1 sh t))
("brainwashing" nil (b r ey1 n w aa1 sh ih0 ng))
("brainy" nil (b r ey1 n iy0))
("braithwaite" nil (b r ey1 th w ey1 t))
("brajdas" nil (b r ay1 d ax s))
("brajovic" nil (b r aa1 jh ow0 v ih0 ch))
("brake" nil (b r ey1 k))
("brakebill" nil (b r ey1 k b ih1 l))
("braked" nil (b r ey1 k t))
("brakefield" nil (b r ey1 k f iy1 l d))
("brakeman" nil (b r ey1 k m ax n))
("brakemen" nil (b r ey1 k m eh0 n))
("braker" nil (b r ey1 k er0))
("brakes" nil (b r ey1 k s))
("braking" nil (b r ey1 k ih0 ng))
("brakke" nil (b r ae1 k))
("braley" nil (b r ae1 l iy0))
("bralley" nil (b r ae1 l iy0))
("brallier" nil (b r ae1 l iy0 er0))
("bralorne" nil (b r ae1 l ao0 r n))
("braly" nil (b r aa1 l iy0))
("bram" nil (b r ae1 m))
("bramah" nil (b r aa1 m ax))
("bramalea" nil (b r ae1 m ax l iy1 ax))
("braman" nil (b r ey1 m ax n))
("brambila" nil (b r aa0 m b iy1 l ax))
("bramble" nil (b r ae1 m b ax l))
("brambles" nil (b r ae1 m b ax l z))
("bramblett" nil (b r ae1 m b l ih0 t))
("brame" nil (b r ey1 m))
("bramel" nil (b r ae1 m ax l))
("bramer" nil (b r ey1 m er0))
("bramhall" nil (b r ae1 m hh ax l))
("bramlage" nil (b r ae1 m l ih0 jh))
("bramlet" nil (b r ae1 m l ih0 t))
("bramlett" nil (b r ae1 m l ih0 t))
("bramlette" nil (b r ae1 m l eh1 t))
("bramley" nil (b r ae1 m l iy0))
("brammeier" nil (b r ae1 m ay0 er0))
("brammer" nil (b r ae1 m er0))
("brampton" nil (b r ae1 m p t ax n))
("bramson" nil (b r ae1 m s ax n))
("bramwell" nil (b r ae1 m w eh1 l))
("bran" nil (b r ae1 n))
("branagan" nil (b r ae1 n ax g ae0 n))
("branam" nil (b r ae1 n ax m))
("branaman" nil (b r ae1 n ax m ax n))
("branan" nil (b r ey1 n ax n))
("branca" nil (b r ae1 ng k ax))
("brancaccio" nil (b r aa0 n k aa1 ch iy0 ow0))
("brancato" nil (b r aa0 n k aa1 t ow0))
("branch" nil (b r ae1 n ch))
("branche" nil (b r ae1 n ch))
("brancheau" nil (b r ax n sh ow1))
("branched" nil (b r ae1 n ch t))
("branches" nil (b r ae1 n ch ax z))
("branching" nil (b r ae1 n ch ih0 ng))
("branchlets" nil (b r ae1 n ch l ax t s))
("branco" nil (b r ae1 ng k ow0))
("brand" nil (b r ae1 n d))
("branda" nil (b r ae1 n d ax))
("brandau" nil (b r ae1 n d aw0))
("brande" nil (b r ae1 n d))
("brandeberry" nil (b r ae1 n d b eh0 r iy0))
("branded" nil (b r ae1 n d ih0 d))
("brandeis" nil (b r ae1 n d ay0 s))
("brandel" nil (b r ae1 n d ax l))
("branden" nil (b r ae1 n d ax n))
("brandenberg" nil (b r ae1 n d ax n b er0 g))
("brandenberger" nil (b r ae1 n d ax n b er0 g er0))
("brandenburg" nil (b r ae1 n d ax n b er0 g))
("brandenburger" nil (b r ae1 n d ax n b er0 g er0))
("brander" nil (b r ae1 n d er0))
("brandes" nil (b r ae1 n d z))
("brandford" nil (b r ae1 n d f er0 d))
("brandhorst" nil (b r ae1 n d hh ao0 r s t))
("brandi" nil (b r ae1 n d iy0))
("branding" nil (b r ae1 n d ih0 ng))
("brandis" nil (b r ae1 n d ih0 s))
("brandish" nil (b r ae1 n d ih0 sh))
("brandished" nil (b r ae1 n d ih0 sh t))
("brandishes" nil (b r ae1 n d ih0 sh ih0 z))
("brandishing" nil (b r ae1 n d ih0 sh ih0 ng))
("brandl" nil (b r ae1 n d ax l))
("brandle" nil (b r ae1 n d ax l))
("brandname" nil (b r ae1 n d n ey1 m))
("brandner" nil (b r ae1 n d n er0))
("brando" nil (b r ae1 n d ow0))
("brandon" nil (b r ae1 n d ax n))
("brandow" nil (b r ae1 n d aw1))
("brands" nil (b r ae1 n d z))
("brandstetter" nil (b r ae1 n d s t ih0 t er0))
("brandt" nil (b r ae1 n t))
("brandvold" nil (b r ae1 n d v ow1 l d))
("brandwein" nil (b r ae1 n d w ay1 n))
("brandy" nil (b r ae1 n d iy0))
("brandywine" nil (b r ae1 n d iy0 w ay1 n))
("braner" nil (b r ey1 n er0))
("branford" nil (b r ae1 n f er0 d))
("branham" nil (b r ae1 n hh ax m))
("braniff" nil (b r ae1 n ih0 f))
("branigan" nil (b r ae1 n ih0 g ax n))
("branin" nil (b r ae1 n ih0 n))
("branitzky" nil (b r ax n ih1 t s k iy1))
("brank" nil (b r ae1 ng k))
("branko" nil (b r ae1 ng k ow0))
("brann" nil (b r ae1 n))
("branna" nil (b r ae1 n ax))
("brannam" nil (b r ae1 n ax m))
("brannan" nil (b r ae1 n ax n))
("brannen" nil (b r ae1 n ax n))
("branner" nil (b r ae1 n er0))
("brannick" nil (b r ae1 n ih0 k))
("brannigan" nil (b r ae1 n ih0 g ax n))
("branning" nil (b r ae1 n ih0 ng))
("brannock" nil (b r ae1 n ax k))
("brannon" nil (b r ae1 n ax n))
("brannum" nil (b r ae1 n ax m))
("branon" nil (b r ae1 n ax n))
("branscom" nil (b r ae1 n s k ax m))
("branscomb" nil (b r ae1 n s k ax m))
("branscome" nil (b r ae1 n z k ah1 m))
("branscum" nil (b r ae1 n s k ax m))
("bransfield" nil (b r ae1 n z f iy1 l d))
("bransford" nil (b r ae1 n s f er0 d))
("bransom" nil (b r ae1 n s ax m))
("branson" nil (b r ae1 n s ax n))
("branstad" nil (b r ae1 n s t ae1 d))
("branstetter" nil (b r ae1 n s t ih0 t er0))
("brant" nil (b r ae1 n t))
("brantley" nil (b r ae1 n t l iy0))
("brantly" nil (b r ae1 n t l iy0))
("brantner" nil (b r ae1 n t n er0))
("branton" nil (b r ae1 n t ax n))
("branum" nil (b r ae1 n ax m))
("branyon" nil (b r ae1 n y ax n))
("branz" nil (b r ae1 n z))
("braque" nil (b r ae1 k))
("bras" nil (b r ae1 s))
("brascade" nil (b r ax s k ey1 d))
("brascan" nil (b r ae1 s k ax n))
("brasch" nil (b r ae1 sh))
("brase" nil (b r ey1 z))
("brasel" nil (b r ae1 s ax l))
("braselton" nil (b r ax s eh1 l t ax n))
("brasfield" nil (b r ae1 s f iy0 l d))
("brash" nil (b r ae1 sh))
("brashear" nil (b r ae1 sh ih0 r))
("brashears" nil (b r ae1 sh ih0 r z))
("brasher" nil (b r ae1 sh er0))
("brashers" nil (b r ae1 sh er0 z))
("brashier" nil (b r ae1 sh iy0 er0))
("brashness" nil (b r ae1 sh n ax s))
("brasil" nil (b r ae1 s ax l))
("brasilia" nil (b r ax z ih1 l y ax))
("brasington" nil (b r ae1 s ih0 ng t ax n))
("brass" nil (b r ae1 s))
("brassard" nil (b r ae1 s er0 d))
("brasseaux" nil (b r ax s ow1))
("brassell" nil (b r ae1 s ax l))
("brasserie" nil (b r ae1 s er0 iy0))
("brasseur" nil (b r ae1 s er0))
("brassfield" nil (b r ae1 s f iy1 l d))
("brassy" nil (b r ae1 s iy0))
("braswell" nil (b r ae1 s w eh0 l))
("brat" nil (b r ae1 t))
("bratcher" nil (b r ae1 ch er0))
("brats" nil (b r ae1 t s))
("bratsch" nil (b r ae1 ch))
("bratt" nil (b r ae1 t))
("brattain" nil (b r ax t ey1 n))
("brattaslava" nil (b r aa1 t ax s l aa1 v ax))
("bratten" nil (b r ae1 t ax n))
("brattin" nil (b r ae1 t ih0 n))
("brattle" nil (b r ae1 t ax l))
("bratton" nil (b r ae1 t ax n))
("bratwurst" nil (b r ae1 t w er0 s t))
("bratz" nil (b r ae1 t s))
("brau" nil (b r aw1))
("brauch" nil (b r ao1 ch))
("braucher" nil (b r ao1 ch er0))
("brauchli" nil (b r ao1 ch l iy0))
("braud" nil (b r ao1 d))
("braude" nil (b r ao1 d))
("brauer" nil (b r aw1 er0))
("braughton" nil (b r ao1 t ax n))
("brault" nil (b r ao1 l t))
("braun" nil (b r ao1 n))
("braund" nil (b r ao1 n d))
("braune" nil (b r ao1 n))
("brauner" nil (b r ao1 n er0))
("brauns" nil (b r ao1 n z))
("braunschweig" nil (b r aw1 n sh w ay0 g))
("braunstein" nil (b r ao1 n s t iy1 n))
("braunwald" nil (b r ao1 n w ao1 l d))
("brause" nil (b r ao1 z))
("brautigam" nil (b r ow1 t ih0 g ax m))
("brautigams" nil (b r ao1 t ih0 g ae0 m z))
("bravado" nil (b r ax v aa1 d ow0))
("brave" nil (b r ey1 v))
("braved" nil (b r ey1 v d))
("bravely" nil (b r ey1 v l iy0))
("braver" nil (b r ey1 v er0))
("braverman" nil (b r ey1 v er0 m ax n))
("bravery" nil (b r ey1 v er0 iy0))
("braves" nil (b r ey1 v z))
("bravest" nil (b r ey1 v ax s t))
("braving" nil (b r ey1 v ih0 ng))
("bravo" nil (b r aa1 v ow0))
("bravura" nil (b r ax v y uh1 r ax))
("brawer" nil (b r ao1 er0))
("brawl" nil (b r ao1 l))
("brawley" nil (b r ao1 l iy0))
("brawling" nil (b r ao1 l ih0 ng))
("brawls" nil (b r ao1 l z))
("brawn" nil (b r ao1 n))
("brawner" nil (b r ao1 n er0))
("brawny" nil (b r ao1 n iy0))
("braxton" nil (b r ae1 k s t ax n))
("bray" nil (b r ey1))
("brayboy" nil (b r ey1 b oy1))
("brayer" nil (b r ey1 er0))
("brayfield" nil (b r ey1 f iy1 l d))
("brayman" nil (b r ey1 m ax n))
("brayton" nil (b r ey1 t ax n))
("braz" nil (b r ae1 z))
("brazda" nil (b r ae1 z d ax))
("brazeal" nil (b r ax z iy1 l))
("brazeau" nil (b r ax z ow1))
("brazee" nil (b r ae1 z iy0))
("brazel" nil (b r ae1 z ax l))
("brazell" nil (b r ae1 z ax l))
("brazelton" nil (b r ax z eh1 l t ax n))
("brazen" nil (b r ey1 z ax n))
("brazenly" nil (b r ey1 z ax n l iy0))
("braziel" nil (b r ax z iy1 l))
("brazier" nil (b r ey1 z iy0 er0))
("braziers" nil (b r ey1 zh er0 z))
("brazil" nil (b r ax z ih1 l))
("brazile" nil (b r aa1 z ay0 l))
("brazilian" nil (b r ax z ih1 l y ax n))
("brazilians" nil (b r ax z ih1 l y ax n z))
("brazill" nil (b r ae1 z ax l))
("brazos" nil (b r aa1 z ow0 s))
("brazzaville" nil (b r ae1 z ax v ih0 l))
("brazzel" nil (b r ae1 z ax l))
("brazzell" nil (b r ae1 z ax l))
("brcko" nil (b er1 ch k ow0))
("brea" nil (b r iy1))
("breach" nil (b r iy1 ch))
("breached" nil (b r iy1 ch t))
("breaches" nil (b r iy1 ch ih0 z))
("breaching" nil (b r iy1 ch ih0 ng))
("bread" nil (b r eh1 d))
("breadbasket" nil (b r eh1 d b ae1 s k ax t))
("breadbox" nil (b r eh1 d b aa1 k s))
("breaded" nil (b r eh1 d ih0 d))
("breadfruit" nil (b r eh1 d f r uw1 t))
("breading" nil (b r eh1 d ih0 ng))
("breads" nil (b r eh1 d z))
("breadth" nil (b r eh1 d th))
("breadwinner" nil (b r eh1 d w ih1 n er0))
("breadwinners" nil (b r eh1 d w ih1 n er0 z))
("bready" nil (b r eh1 d iy0))
("break" nil (b r ey1 k))
("breakable" nil (b r ey1 k ax b ax l))
("breakage" nil (b r ey1 k ih0 jh))
("breakaway" nil (b r ey1 k ax w ey1))
("breakdown" nil (b r ey1 k d aw1 n))
("breakdowns" nil (b r ey1 k d aw1 n z))
("breaker" nil (b r ey1 k er0))
("breakers" nil (b r ey1 k er0 z))
("breakeven" nil (b r ey1 k iy1 v ax n))
("breakey" nil (b r iy1 k iy0))
("breakfast" nil (b r eh1 k f ax s t))
("breakfasts" nil (b r eh1 k f ax s t s))
("breakfield" nil (b r ey1 k f iy1 l d))
("breaking" nil (b r ey1 k ih0 ng))
("breakmate" nil (b r ey1 k m ey1 t))
("breakneck" nil (b r ey1 k n eh1 k))
("breakout" nil (b r ey1 k aw1 t))
("breakouts" nil (b r ey1 k aw1 t s))
("breaks" nil (b r ey1 k s))
("breakthrough" nil (b r ey1 k th r uw1))
("breakthroughs" nil (b r ey1 k th r uw1 z))
("breakup" nil (b r ey1 k ah1 p))
("breakups" nil (b r iy1 k ax p s))
("breakwater" nil (b r ey1 k w ao1 t er0))
("bream" nil (b r iy1 m))
("brean" nil (b r iy1 n))
("brearley" nil (b r er1 l iy0))
("breast" nil (b r eh1 s t))
("breasted" nil (b r eh1 s t ax d))
("breasting" nil (b r eh1 s t ih0 ng))
("breastplate" nil (b r eh1 s t p l ey1 t))
("breasts" nil (b r eh1 s t s))
("breath" nil (b r eh1 th))
("breathe" nil (b r iy1 dh))
("breathed" nil (b r iy1 dh d))
("breather" nil (b r iy1 dh er0))
("breathes" nil (b r iy1 dh z))
("breathing" nil (b r iy1 dh ih0 ng))
("breathless" nil (b r eh1 th l ax s))
("breathlessly" nil (b r eh1 th l ax s l iy0))
("breaths" nil (b r eh1 th s))
("breathtaking" nil (b r eh1 th t ey1 k ih0 ng))
("breathtakingly" nil (b r eh1 th t ey1 k ih0 ng l iy0))
("breathy" nil (b r eh1 th iy0))
("breau" nil (b r ow1))
("breault" nil (b r ow1))
("breaux" nil (b r ow1))
("breazeale" nil (b r iy1 z iy1 l))
("brebach" nil (b r eh1 b ax k))
("brechbill" nil (b r eh1 k b ax l))
("brecheen" nil (b r eh1 k iy0 n))
("brecheisen" nil (b r eh1 k ay0 s ax n))
("brecher" nil (b r eh1 k er0))
("brecht" nil (b r eh1 k t))
("brechtel" nil (b r eh1 k t ax l))
("breck" nil (b r eh1 k))
("breckenridge" nil (b r eh1 k ax n r ih1 jh))
("brecker" nil (b r eh1 k er0))
("breco" nil (b r eh1 k ow0))
("bred" nil (b r eh1 d))
("breda" nil (b r ey1 d ax))
("brede" nil (b r iy1 d))
("breden" nil (b r iy1 d ax n))
("bredeson" nil (b r eh1 d ih0 s ax n))
("bree" nil (b r iy1))
("breece" nil (b r iy1 s))
("breech" nil (b r iy1 ch))
("breed" nil (b r iy1 d))
("breeden" nil (b r iy1 d ax n))
("breeder" nil (b r iy1 d er0))
("breeders" nil (b r iy1 d er0 z))
("breeding" nil (b r iy1 d ih0 ng))
("breedlove" nil (b r iy1 d l ah1 v))
("breeds" nil (b r iy1 d z))
("breeland" nil (b r iy1 l ax n d))
("breen" nil (b r iy1 n))
("brees" nil (b r iy1 z))
("breese" nil (b r iy1 z))
("breeze" nil (b r iy1 z))
("breezed" nil (b r iy1 z d))
("breezes" nil (b r iy1 z ih0 z))
("breezeway" nil (b r iy1 z w ey1))
("breezy" nil (b r iy1 z iy0))
("brege" nil (b r iy1 jh))
("breger" nil (b r iy1 g er0))
("breglio" nil (b r eh1 g l iy0 ow0))
("bregman" nil (b r eh1 g m ax n))
("breguet" nil (b r iy1 g ax t))
("brehm" nil (b r eh1 m))
("brehmer" nil (b r eh1 m er0))
("breidenbach" nil (b r ay1 d ih0 n b aa0 k))
("breidenstein" nil (b r ay1 d ax n s t ay0 n))
("breier" nil (b r ay1 er0))
("breighner" nil (b r ey1 n er0))
("breiner" nil (b r ay1 n er0))
("breining" nil (b r ay1 n ih0 ng))
("breininger" nil (b r ay1 n ih0 ng er0))
("breisch" nil (b r ay1 sh))
("breit" nil (b r iy1 t))
("breitbach" nil (b r ay1 t b aa1 k))
("breitbarth" nil (b r ay1 t b aa1 r th))
("breitenbach" nil (b r ay1 t ih0 n b aa0 k))
("breitenstein" nil (b r ay1 t ax n s t ay0 n))
("breithaupt" nil (b r ay1 th aw0 p t))
("breitkreutz" nil (b r ay1 t k r oy1 t s))
("breitling" nil (b r ay1 t ax l ih0 ng))
("breitman" nil (b r ay1 t m ax n))
("breitschwerdt" nil (b r ay1 ch w er0 t))
("breitweiser" nil (b r ay1 t w ay1 s er0))
("breitzman" nil (b r ay1 t s m ax n))
("brekke" nil (b r eh1 k))
("breland" nil (b r eh1 l ax n d))
("brelsford" nil (b r eh1 l s f er0 d))
("brem" nil (b r eh1 m))
("bremen" nil (b r eh1 m ax n))
("bremer" nil (b r iy1 m er0))
("bremmer" nil (b r eh1 m er0))
("bremner" nil (b r eh1 m n er0))
("brems" nil (b r eh1 m z))
("bren" nil (b r eh1 n))
("brenda" nil (b r eh1 n d ax))
("brendan" nil (b r eh1 n d ax n))
("brendel" nil (b r eh1 n d ax l))
("brenden" nil (b r eh1 n d ax n))
("brender" nil (b r eh1 n d er0))
("brendle" nil (b r eh1 n d ax l))
("brendlinger" nil (b r eh1 n d ax l ih0 ng er0))
("brendsel" nil (b r eh1 n d s ax l))
("breneman" nil (b r iy1 n m ax n))
("brener" nil (b r iy1 n er0))
("brenes" nil (b r iy1 n z))
("brengle" nil (b r ih1 ng ax l))
("brenizer" nil (b r eh1 n ay0 z er0))
("brenn" nil (b r eh1 n))
("brenna" nil (b r eh1 n ax))
("brennan" nil (b r eh1 n ax n))
("brennans" nil (b r eh1 n ax n z))
("brennecke" nil (b r eh1 n ih0 k))
("brenneke" nil (b r eh1 n ax k iy0))
("brenneman" nil (b r eh1 n m ax n))
("brennen" nil (b r eh1 n ax n))
("brenner" nil (b r eh1 n er0))
("brenning" nil (b r eh1 n ih0 ng))
("brensinger" nil (b r eh1 n s ih0 ng er0))
("brent" nil (b r eh1 n t))
("brentano" nil (b r eh1 n t aa1 n ow0))
("brentanos" nil (b r eh1 n t aa1 n ow0 z))
("brentlinger" nil (b r eh1 n t ax l ih0 ng er0))
("brenton" nil (b r eh1 n t ax n))
("brents" nil (b r eh1 n t s))
("brentwood" nil (b r eh1 n t w uh1 d))
("breon" nil (b r iy1 ax n))
("brescia" nil (b r eh1 s ch ax))
("bresee" nil (b r ih0 s iy1))
("bresette" nil (b r ih0 s eh1 t))
("breshears" nil (b r eh1 sh ih0 r z))
("bresler" nil (b r eh1 s l er0))
("breslin" nil (b r eh1 s l ih0 n))
("breslow" nil (b r eh1 s l ow0))
("bresnahan" nil (b r eh1 s n ax hh ae0 n))
("bresnan" nil (b r eh1 s n ax n))
("bresnick" nil (b r eh1 s n ih0 k))
("bress" nil (b r eh1 s))
("bresse" nil (b r eh1 s))
("bresser" nil (b r eh1 s er0))
("bressette" nil (b r eh1 s eh1 t))
("bressler" nil (b r eh1 s l er0))
("bressman" nil (b r eh1 s m ax n))
("bresson" nil (b r eh1 s ax n))
("brest" nil (b r eh1 s t))
("brester" nil (b r eh1 s t er0))
("bret" nil (b r eh1 t))
("breth" nil (b r eh1 th))
("brethauer" nil (b r eh1 th aw0 er0))
("brethen" nil (b r iy1 th ax n))
("brethren" nil (b r eh1 dh r ax n))
("breton" nil (b r eh1 t ax n))
("bretschneider" nil (b r eh1 ch n ay0 d er0))
("brett" nil (b r eh1 t))
("bretthauer" nil (b r eh1 th aw0 er0))
("bretton" nil (b r eh1 t ax n))
("bretts" nil (b r eh1 t s))
("brettschneider" nil (b r eh1 ch n ay1 d er0))
("bretz" nil (b r eh1 t s))
("breuer" nil (b r uw1 er0))
("breunig" nil (b r uw1 n ih0 g))
("breuninger" nil (b r uw1 n ih0 ng er0))
("brevard" nil (b r eh1 v er0 d))
("brevets" nil (b r ax v eh1 t s))
("brevig" nil (b r eh1 v ih0 g))
("brevik" nil (b r eh1 v ih0 k))
("brevity" nil (b r eh1 v ax t iy0))
("brew" nil (b r uw1))
("brewbaker" nil (b r uw1 b ey1 k er0))
("brewed" nil (b r uw1 d))
("brewer" nil (b r uw1 er0))
("breweries" nil (b r uw1 er0 iy0 z))
("brewers" nil (b r uw1 er0 z))
("brewery" nil (b r uw1 er0 iy0))
("brewing" nil (b r uw1 ih0 ng))
("brewington" nil (b r uw1 ih0 ng t ax n))
("brews" nil (b r uw1 z))
("brewster" nil (b r uw1 s t er0))
("brewton" nil (b r uw1 t ax n))
("brey" nil (b r ey1))
("breyer" nil (b r ey1 er0))
("breyfogle" nil (b r ey1 f ow1 g ax l))
("breza" nil (b r eh1 z ax))
("brezhnev" nil (b r eh1 z n ax v))
("brezina" nil (b r eh0 z iy1 n ax))
("brezinski" nil (b r ih0 z ih1 n s k iy0))
("bria" nil (b r iy1 ax))
("brian" nil (b r ay1 ax n))
("briana" nil (b r iy0 ae1 n ax))
("briand" nil (b r ay1 ax n d))
("brianna" nil (b r iy0 ae1 n ax))
("briant" nil (b r ay1 ax n t))
("briar" nil (b r ay1 er0))
("briarcliff" nil (b r ay1 r k l ih1 f))
("bribe" nil (b r ay1 b))
("bribed" nil (b r ay1 b d))
("bribery" nil (b r ay1 b er0 iy0))
("bribes" nil (b r ay1 b z))
("bribing" nil (b r ay1 b ih0 ng))
("bric" nil (b r ih1 k))
("briccetti" nil (b r ih0 ch eh1 t iy0))
("bricco" nil (b r ih1 k ow0))
("brice" nil (b r ay1 s))
("briceno" nil (b r iy0 ch eh1 n ow0))
("brick" nil (b r ih1 k))
("brickbats" nil (b r ih1 k b ae1 t s))
("brickel" nil (b r ih1 k ax l))
("brickell" nil (b r ih1 k ax l))
("bricker" nil (b r ih1 k er0))
("brickey" nil (b r ih1 k iy0))
("brickhouse" nil (b r ih1 k hh aw1 s))
("bricklayers" nil (b r ih1 k l ey1 er0 z))
("brickle" nil (b r ih1 k ax l))
("brickler" nil (b r ih1 k l er0))
("brickley" nil (b r ih1 k l iy0))
("bricklin" nil (b r ih1 k l ih0 n))
("brickman" nil (b r ih1 k m ax n))
("brickner" nil (b r ih1 k n er0))
("bricks" nil (b r ih1 k s))
("brickyard" nil (b r ih1 k y aa1 r d))
("bricom" nil (b r ih1 k ax m))
("bridal" nil (b r ay1 d ax l))
("bride" nil (b r ay1 d))
("brideau" nil (b r ih0 d ow1))
("bridegroom" nil (b r ay1 d g r uw1 m))
("briden" nil (b r ay1 d ax n))
("bridenbaugh" nil (b r ih1 d ih0 n b aw0))
("bridenstine" nil (b r ih1 d ih0 n s t iy0 n))
("brides" nil (b r ay1 d z))
("bridesmaid" nil (b r ay1 d z m ey1 d))
("bridesmaids" nil (b r ay1 d z m ey1 d z))
("bridge" nil (b r ih1 jh))
("bridged" nil (b r ih1 jh d))
("bridgeford" nil (b r ih1 jh ih0 f ao0 r d))
("bridgeforth" nil (b r ih1 jh f ao1 r th))
("bridgehead" nil (b r ih1 jh hh eh1 d))
("bridgeman" nil (b r ih1 jh m ax n))
("bridgeport" nil (b r ih1 jh p ao1 r t))
("bridger" nil (b r ih1 jh er0))
("bridgers" nil (b r ih1 jh er0 z))
("bridges" nil (b r ih1 jh ax z))
("bridgestone" nil (b r ih1 jh s t ow1 n))
("bridget" nil (b r ih1 jh ax t))
("bridgeton" nil (b r ih1 jh t ax n))
("bridgett" nil (b r ih1 jh ih0 t))
("bridgewater" nil (b r ih1 jh w ao1 t er0))
("bridgham" nil (b r ih1 jh hh ax m))
("bridging" nil (b r ih1 jh ih0 ng))
("bridgman" nil (b r ih1 jh m ax n))
("bridie" nil (b r ih1 d iy0))
("bridle" nil (b r ay1 d ax l))
("bridled" nil (b r ay1 d ax l d))
("bridwell" nil (b r ih1 d w eh1 l))
("brie" nil (b r iy1))
("brieant" nil (b r iy1 ax n t))
("brief" nil (b r iy1 f))
("briefcase" nil (b r iy1 f k ey1 s))
("briefcases" nil (b r iy1 f k ey1 s ih0 z))
("briefed" nil (b r iy1 f t))
("briefer" nil (b r iy1 f er0))
("briefing" nil (b r iy1 f ih0 ng))
("briefings" nil (b r iy1 f ih0 ng z))
("briefly" nil (b r iy1 f l iy0))
("briefs" nil (b r iy1 f s))
("briegel" nil (b r iy1 g ax l))
("brieger" nil (b r iy1 g er0))
("briel" nil (b r iy1 l))
("brien" nil (b r ay1 ih0 n))
("brienza" nil (b r iy1 n z ax))
("brier" nil (b r ay1 er0))
("briere" nil (b r ih1 r))
("brierley" nil (b r ay1 er0 l iy0))
("brierly" nil (b r ay1 er0 l iy0))
("briese" nil (b r iy1 z))
("brietzke" nil (b r iy1 t z k))
("brig" nil (b r ih1 g))
("brigade" nil (b r ax g ey1 d))
("brigadeer" nil (b r ih1 g ax d ih1 r))
("brigades" nil (b r ih0 g ey1 d z))
("brigadier" nil (b r ih1 g ax d ih1 r))
("brigance" nil (b r ih1 g ax n s))
("brigandi" nil (b r ih0 g ae1 n d iy0))
("brigante" nil (b r iy0 g aa1 n t iy0))
("briganti" nil (b r ih0 g ae1 n t iy0))
("briggs" nil (b r ih1 g z))
("brigham" nil (b r ih1 g ax m))
("brighams" nil (b r ih1 g ax m z))
("bright" nil (b r ay1 t))
("brightbill" nil (b r ay1 t b ih1 l))
("brighten" nil (b r ay1 t ax n))
("brightened" nil (b r ay1 t ax n d))
("brightening" nil (b r ay1 t ax n ih0 ng))
("brightens" nil (b r ay1 t ax n z))
("brighter" nil (b r ay1 t er0))
("brightest" nil (b r ay1 t ax s t))
("brightly" nil (b r ay1 t l iy0))
("brightman" nil (b r ay1 t m ax n))
("brightness" nil (b r ay1 t n ax s))
("brighton" nil (b r ay1 t ax n))
("brightwell" nil (b r ay1 t w eh1 l))
("brigitte" nil (b r ih1 jh ih0 t))
("brigman" nil (b r ih1 g m ax n))
("brignac" nil (b r ih1 g n ax k))
("brigner" nil (b r ay1 g n er0))
("brigode" nil (b r ih0 g ow1 d ax))
("briguglio" nil (b r ih0 g ah1 g l iy0 ow0))
("briles" nil (b r ay1 l z))
("briley" nil (b r ih1 l iy0))
("brill" nil (b r ih1 l))
("brillhart" nil (b r ih1 l hh aa1 r t))
("brilliance" nil (b r ih1 l y ax n s))
("brilliant" nil (b r ih1 l y ax n t))
("brilliantly" nil (b r ih1 l y ax n t l iy0))
("brillstein" nil (b r ih1 l s t iy1 n))
("brim" nil (b r ih1 m))
("brimberry" nil (b r ih1 m b eh1 r iy0))
("brimelow" nil (b r ih1 m ax l ow0))
("brimer" nil (b r ay1 m er0))
("brimhall" nil (b r ih1 m hh ao1 l))
("brimm" nil (b r ih1 m))
("brimmed" nil (b r ih1 m d))
("brimmer" nil (b r ih1 m er0))
("brimming" nil (b r ih1 m ih0 ng))
("brin" nil (b r ih1 n))
("brindel" nil (b r ih1 n d eh1 l))
("brindle" nil (b r ih1 n d ax l))
("brindley" nil (b r ih1 n d l iy0))
("brine" nil (b r ay1 n))
("brinegar" nil (b r ih1 n ih0 g er0))
("briner" nil (b r ay1 n er0))
("brines" nil (b r ay1 n z))
("briney" nil (b r ih1 n iy0))
("bring" nil (b r ih1 ng))
("bringhurst" nil (b r ih1 ng hh er0 s t))
("bringing" nil (b r ih1 ng ih0 ng))
("bringle" nil (b r ih1 ng g ax l))
("bringman" nil (b r ih1 ng m ax n))
("brings" nil (b r ih1 ng z))
("brining" nil (b r ay1 n ih0 ng))
("brink" nil (b r ih1 ng k))
("brinker" nil (b r ih1 ng k er0))
("brinkerhoff" nil (b r ih1 ng k er0 hh ao1 f))
("brinkley" nil (b r ih1 ng k l iy0))
("brinkman" nil (b r ih1 ng k m ax n))
("brinkmann" nil (b r ih1 ng k m ax n))
("brinkmanship" nil (b r ih1 ng k m ax n sh ih1 p))
("brinkmeier" nil (b r ih1 ng k m ay0 er0))
("brinkmeyer" nil (b r ih1 ng k m ay0 er0))
("brinks" nil (b r ih1 ng k s))
("brinksmanship" nil (b r ih1 ng k s m ax n sh ih1 p))
("brinlee" nil (b r ih1 n l iy0))
("brinley" nil (b r ih1 n l iy0))
("brinn" nil (b r ih1 n))
("brinner" nil (b r ih1 n er0))
("brinser" nil (b r ih1 n s er0))
("brinsfield" nil (b r ih1 n s f iy0 l d))
("brinson" nil (b r ih1 n s ax n))
("brintec" nil (b r ih1 n t eh1 k))
("brinton" nil (b r ih1 n t ax n))
("brio" nil (b r ay1 ow0))
("brioche" nil (b r iy1 ow1 sh))
("briody" nil (b r ay1 ax d iy0))
("brion" nil (b r ay1 ax n))
("briones" nil (b r iy0 ow1 n eh0 s))
("brisbane" nil (b r ih1 z b ey1 n))
("brisbin" nil (b r ih1 z b ih0 n))
("brisbois" nil (b r ih0 z b w aa1))
("brisbon" nil (b r ih1 z b ax n))
("brisco" nil (b r iy1 s k ow0))
("briscoe" nil (b r ih1 s k ow0))
("brisendine" nil (b r ih1 s ih0 n d ay1 n))
("briseno" nil (b r iy0 s eh1 n ow0))
("brisk" nil (b r ih1 s k))
("brisker" nil (b r ih1 s k er0))
("brisket" nil (b r ih1 s k ax t))
("briskey" nil (b r ih1 s k iy0))
("briski" nil (b r ih1 s k iy0))
("briskin" nil (b r ih1 s k ih0 n))
("briskly" nil (b r ih1 s k l iy0))
("brisky" nil (b r ih1 s k iy0))
("brislin" nil (b r ih1 s l ih0 n))
("brison" nil (b r ih1 s ax n))
("brissette" nil (b r ih0 s eh1 t))
("brissey" nil (b r ih1 s iy0))
("brisson" nil (b r ih1 s ax n))
("brister" nil (b r ih1 s t er0))
("bristle" nil (b r ih1 s ax l))
("bristled" nil (b r ih1 s ax l d))
("bristles" nil (b r ih1 s ax l z))
("bristling" nil (b r ih1 s ax l ih0 ng))
("bristol" nil (b r ih1 s t ax l))
("bristow" nil (b r ih1 s t ow0))
("brit" nil (b r ih1 t))
("brita" nil (b r iy1 t ax))
("britain" nil (b r ih1 t ax n))
("britannia" nil (b r ih0 t ae1 n iy0 ax))
("britannica" nil (b r ih0 t ae1 n ih0 k ax))
("britcher" nil (b r ih1 ch er0))
("britches" nil (b r ih1 ch ax z))
("brite" nil (b r ay1 t))
("britian" nil (b r ih1 sh ax n))
("british" nil (b r ih1 t ih0 sh))
("britnell" nil (b r ih1 t n ax l))
("brito" nil (b r iy1 t ow0))
("britoil" nil (b r ih0 t oy1 l))
("briton" nil (b r ih1 t ax n))
("britons" nil (b r ih1 t ax n z))
("brits" nil (b r ih1 t s))
("britsch" nil (b r ih1 ch))
("britt" nil (b r ih1 t))
("brittain" nil (b r ih1 t ax n))
("brittan" nil (b r ih1 t ax n))
("brittany" nil (b r ih1 t ax n iy0))
("britten" nil (b r ih1 t ax n))
("brittenham" nil (b r ih1 t ih0 n hh ax m))
("brittian" nil (b r ih1 t iy0 ax n))
("brittin" nil (b r ih1 t ih0 n))
("brittingham" nil (b r ih1 t ih0 ng hh ae0 m))
("brittle" nil (b r ih1 t ax l))
("britton" nil (b r ih1 t ax n))
("britts" nil (b r ih1 t s))
("britz" nil (b r ih1 t s))
("brix" nil (b r ih1 k s))
("brixey" nil (b r ih1 k s iy0))
("brixius" nil (b r ay1 k s iy0 ih0 s))
("brizendine" nil (b r iy0 z eh0 n d iy1 n iy0))
("brizola" nil (b r ih0 z ow1 l ax))
("brizzi" nil (b r ih1 z iy0))
("brizzolara" nil (b r iy0 t s ow0 l aa1 r ax))
("bro" nil (b r ow1))
("broach" nil (b r ow1 ch))
("broached" nil (b r ow1 ch t))
("broaching" nil (b r ow1 ch ih0 ng))
("broad" nil (b r ao1 d))
("broadaway" nil (b r ao1 d ax w ey1))
("broadband" nil (b r ao1 d b ae1 n d))
("broadbased" nil (b r ao1 d b ey1 s t))
("broadbeach" nil (b r ao1 d b iy1 ch))
("broadbent" nil (b r ao1 d b eh1 n t))
("broadcast" nil (b r ao1 d k ae1 s t))
("broadcaster" nil (b r ao1 d k ae1 s t er0))
("broadcasters" nil (b r ao1 d k ae1 s t er0 z))
("broadcasting" nil (b r ao1 d k ae1 s t ih0 ng))
("broadcasts" nil (b r ao1 d k ae1 s t s))
("broaddus" nil (b r ao1 d ih0 s))
("broaden" nil (b r ao1 d ax n))
("broadened" nil (b r ao1 d ax n d))
("broadening" nil (b r ao1 d ax n ih0 ng))
("broadens" nil (b r ao1 d ax n z))
("broader" nil (b r ao1 d er0))
("broadest" nil (b r ao1 d ih0 s t))
("broadhead" nil (b r ao1 d hh eh1 d))
("broadhurst" nil (b r ao1 d hh er0 s t))
("broadie" nil (b r ao1 d iy0))
("broadley" nil (b r ao1 d l iy0))
("broadly" nil (b r ao1 d l iy0))
("broadnax" nil (b r ao1 d n ae0 k s))
("broadpfoot" nil (b r ao1 d f uh0 t))
("broadrick" nil (b r ao1 d r ih0 k))
("broadside" nil (b r ao1 d s ay1 d))
("broadstreet" nil (b r ao1 d s t r iy1 t))
("broadsword" nil (b r ao1 d s ao1 r d))
("broadus" nil (b r ow1 d ax s))
("broadview" nil (b r ao1 d v y uw1))
("broadway" nil (b r ao1 d w ey1))
("broadwell" nil (b r ao1 d w eh1 l))
("broady" nil (b r ao1 d iy0))
("brobdingnagian" nil (b r ao1 b d ih0 g n ae1 g iy0 ax n))
("brobdingnagians" nil (b r ao1 b d ih0 g n ae1 g iy0 ax n z))
("brobeck" nil (b r ow1 b eh1 k))
("broberg" nil (b r ow1 b er0 g))
("brobst" nil (b r aa1 b s t))
("broc" nil (b r aa1 k))
("brocade" nil (b r ow0 k ey1 d))
("brocades" nil (b r ow0 k ey1 d z))
("brocato" nil (b r ow0 k aa1 t ow0))
("brocco" nil (b r aa1 k ow0))
("broccoli" nil (b r aa1 k ax l iy0))
("broce" nil (b r ow1 s))
("brochu" nil (b r ow1 k uw0))
("brochure" nil (b r ow0 sh uh1 r))
("brochures" nil (b r ow0 sh uh1 r z))
("brocious" nil (b r ax sh iy1 s))
("brock" nil (b r aa1 k))
("brockbank" nil (b r aa1 k b ax ng k))
("brockel" nil (b r aa1 k ax l))
("brocker" nil (b r aa1 k er0))
("brockert" nil (b r aa1 k er0 t))
("brockett" nil (b r aa1 k ih0 t))
("brockhaus" nil (b r aa1 k hh aw1 s))
("brockhoff" nil (b r aa1 k hh ao1 f))
("brockhouse" nil (b r aa1 k hh aw1 s))
("brockie" nil (b r aa1 k iy0))
("brockington" nil (b r aa1 k ih0 ng t ax n))
("brocklehurst" nil (b r aa1 k l ih0 hh er0 s t))
("brockley" nil (b r aa1 k l iy0))
("brockman" nil (b r aa1 k m ax n))
("brockmann" nil (b r aa1 k m ax n))
("brockmeier" nil (b r aa1 k m ay0 er0))
("brockmeyer" nil (b r aa1 k m ay0 er0))
("brockner" nil (b r aa1 k n er0))
("brocksmith" nil (b r aa1 k s m ih1 th))
("brockton" nil (b r aa1 k t ax n))
("brockway" nil (b r aa1 k w ey1))
("brockwell" nil (b r aa1 k w eh1 l))
("brod" nil (b r aa1 d))
("broda" nil (b r ow1 d ax))
("brodbeck" nil (b r aa1 d b eh1 k))
("brode" nil (b r ow1 d))
("broden" nil (b r ow1 d ax n))
("broder" nil (b r ow1 d er0))
("broderbund" nil (b r ow1 t er0 b ax n d))
("broderick" nil (b r aa1 d er0 ih0 k))
("brodersen" nil (b r aa1 d er0 s ax n))
("brodersohn" nil (b r ow1 t er0 s ax n))
("broderson" nil (b r aa1 d er0 s ax n))
("brodeur" nil (b r aa1 d er0))
("brodhead" nil (b r aa1 d hh eh1 d))
("brodie" nil (b r ow1 t iy0))
("brodin" nil (b r ow1 d ih0 n))
("brodkin" nil (b r aa1 d k ih0 n))
("brodman" nil (b r aa1 d m ax n))
("brodnax" nil (b r aa1 d n ae0 k s))
("brodowski" nil (b r ax d ao1 f s k iy0))
("brodrick" nil (b r aa1 d r ih0 k))
("brodsky" nil (b r aa1 d s k iy0))
("brodt" nil (b r aa1 d t))
("brody" nil (b r ow1 d iy0))
("brodzinski" nil (b r ax jh ih1 n s k iy0))
("broe" nil (b r ow1))
("broecker" nil (b r ow1 k er0))
("broeker" nil (b r ow1 k er0))
("broer" nil (b r ow1 er0))
("broerman" nil (b r ow1 er0 m ax n))
("broers" nil (b r ow1 er0 z))
("brogan" nil (b r ow1 g ax n))
("brogden" nil (b r aa1 g d ax n))
("brogdon" nil (b r aa1 g d ax n))
("brogna" nil (b r ow1 g n ax))
("broich" nil (b r oy1 ch))
("broil" nil (b r oy1 l))
("broiled" nil (b r oy1 l d))
("broiler" nil (b r oy1 l er0))
("broilers" nil (b r oy1 l er0 z))
("broiling" nil (b r oy1 l ih0 ng))
("brok" nil (b r aa1 k))
("brokaw" nil (b r ow1 k ao0))
("broke" nil (b r ow1 k))
("broken" nil (b r ow1 k ax n))
("broker" nil (b r ow1 k er0))
("brokerage" nil (b r ow1 k er0 ih0 jh))
("brokerages" nil (b r ow1 k er0 ih0 jh ih0 z))
("brokered" nil (b r ow1 k er0 d))
("brokering" nil (b r ow1 k er0 ih0 ng))
("brokers" nil (b r ow1 k er0 z))
("broking" nil (b r ow1 k ih0 ng))
("brolin" nil (b r ow1 l ih0 n))
("brolly" nil (b r aa1 l iy0))
("brom" nil (b r aa1 m))
("broman" nil (b r ow1 m ax n))
("bromberg" nil (b r aa1 m b er0 g))
("bromfield" nil (b r aa1 m f iy1 l d))
("bromides" nil (b r ow1 m ay1 d z))
("bromine" nil (b r ow1 m iy1 n))
("bromley" nil (b r aa1 m l iy0))
("bromm" nil (b r aa1 m))
("brommer" nil (b r aa1 m er0))
("bromont" nil (b r ow1 m aa1 n t))
("brompheril" nil (b r aa1 m f eh0 r ih0 l))
("bromwell" nil (b r aa1 m w eh1 l))
("bromwich" nil (b r aa1 m w ih0 ch))
("bronaugh" nil (b r aa1 n ao0))
("bronc" nil (b r aa1 ng k))
("bronchial" nil (b r aa1 n ch iy0 ax l))
("bronchitis" nil (b r aa0 ng k ay1 t ax s))
("bronco" nil (b r aa1 ng k ow0))
("broncos" nil (b r aa1 ng k ow0 z))
("bronder" nil (b r aa1 n d er0))
("broner" nil (b r ow1 n er0))
("bronfman" nil (b r aa1 n f m ax n))
("bronfmans" nil (b r aa1 n f m ax n z))
("bronk" nil (b r aa1 ng k))
("bronkema" nil (b r ax ng k iy1 m ax))
("bronn" nil (b r aa1 n))
("bronner" nil (b r aa1 n er0))
("brons" nil (b r aa1 n z))
("bronson" nil (b r aa1 n s ax n))
("bronstein" nil (b r aa1 n s t iy1 n))
("bronston" nil (b r aa1 n s t ax n))
("bronte" nil (b r aa1 n t iy0))
("bronwen" nil (b r ao1 n w ih0 n))
("bronx" nil (b r aa1 ng k s))
("bronze" nil (b r aa1 n z))
("bronzed" nil (b r aa1 n z d))
("bronzen" nil (b r aa1 n z ax n))
("bronzes" nil (b r aa1 n z ax z))
("brooch" nil (b r uw1 ch))
("brood" nil (b r uw1 d))
("brooded" nil (b r uw1 d ih0 d))
("brooding" nil (b r uw1 d ih0 ng))
("broody" nil (b r uw1 d iy0))
("brook" nil (b r uh1 k))
("brookbank" nil (b r uh1 k b ae1 ng k))
("brooke" nil (b r uh1 k))
("brookehill" nil (b r uh1 k hh ih1 l))
("brookens" nil (b r uh1 k ax n z))
("brooker" nil (b r uh1 k er0))
("brookes" nil (b r uh1 k s))
("brookfield" nil (b r uh1 k f iy1 l d))
("brookhart" nil (b r uw1 k hh aa0 r t))
("brookhaven" nil (b r uh1 k hh ey1 v ax n))
("brookhurst" nil (b r uh1 k hh er1 s t))
("brooking" nil (b r uh1 k ih0 ng))
("brookings" nil (b r uh1 k ih0 ng z))
("brookins" nil (b r uw1 k ih0 n z))
("brookline" nil (b r uh1 k l ay1 n))
("brooklyn" nil (b r uh1 k l ax n))
("brookman" nil (b r uh1 k m ax n))
("brookner" nil (b r uh1 k n er0))
("brookover" nil (b r uh1 k ow1 v er0))
("brooks" nil (b r uh1 k s))
("brookshier" nil (b r uw1 k sh iy0 er0))
("brookshire" nil (b r uw1 k sh ay0 r))
("brookside" nil (b r uh1 k s ay1 d))
("brookstone" nil (b r uh1 k s t ow1 n))
("brooksville" nil (b r uh1 k s v ih0 l))
("broom" nil (b r uw1 m))
("broomall" nil (b r uw1 m ax l))
("broome" nil (b r uw1 m))
("broomfield" nil (b r uw1 m f iy1 l d))
("brooms" nil (b r uw1 m z))
("broomstick" nil (b r uw1 m s t ih1 k))
("brophy" nil (b r ow1 f iy0))
("brosch" nil (b r ao1 sh))
("brose" nil (b r ow1 z))
("brosh" nil (b r aa1 sh))
("broshears" nil (b r aa1 sh ih0 r z))
("brosious" nil (b r ow1 z iy0 ax s))
("brosius" nil (b r ow1 s iy0 ih0 s))
("broski" nil (b r aw1 s k iy0))
("brosky" nil (b r aa1 s k iy0))
("brosnahan" nil (b r aa1 s n ax hh ae0 n))
("brosnan" nil (b r aa1 s n ax n))
("bross" nil (b r ao1 s))
("brossard" nil (b r ax s aa1 r d))
("brossart" nil (b r aa1 s aa0 r t))
("brosseau" nil (b r ax s ow1))
("brossette" nil (b r ax s eh1 t))
("brossman" nil (b r ao1 s m ax n))
("brost" nil (b r aa1 s t))
("brostrom" nil (b r aa1 s t r ax m))
("brosz" nil (b r aa1 sh))
("broten" nil (b r ow1 t ax n))
("broth" nil (b r ao1 th))
("brothel" nil (b r aa1 th ax l))
("brothels" nil (b r aa1 th ax l z))
("brother" nil (b r ah1 dh er0))
("brotherhood" nil (b r ah1 dh er0 hh uh1 d))
("brotherly" nil (b r ah1 dh er0 l iy0))
("brothers" nil (b r ah1 dh er0 z))
("brotherson" nil (b r ah1 dh er0 s ax n))
("brotherton" nil (b r ah1 dh er0 t ax n))
("broths" nil (b r ao1 th s))
("brotman" nil (b r aa1 t m ax n))
("brott" nil (b r aa1 t))
("brotzman" nil (b r aa1 t s m ax n))
("broudy" nil (b r aw1 d iy0))
("brough" nil (b r aw1))
("brougham" nil (b r uw1 g ax m))
("brougher" nil (b r aw1 er0))
("broughman" nil (b r aw1 m ax n))
("brought" nil (b r ao1 t))
("broughton" nil (b r ao1 t ax n))
("brouhaha" nil (b r uw1 hh aa0 hh aa0))
("brouhard" nil (b r aa1 uw0 er0 d))
("brouillard" nil (b r w iy0 l aa1 r d))
("brouillet" nil (b r w iy0 l eh1 t))
("brouillette" nil (b r w iy0 l eh1 t))
("broun" nil (b r uw1 n))
("brountas" nil (b r aw1 n t ax s))
("brouse" nil (b r aw1 s))
("broussard" nil (b r uw0 s aa1 r d))
("brousseau" nil (b r uw1 s ow1))
("brouwer" nil (b r aw1 w er0))
("brow" nil (b r aw1))
("broward" nil (b r aw1 er0 d))
("browbeat" nil (b r aw1 b iy1 t))
("browder" nil (b r aw1 d er0))
("browe" nil (b r ow1))
("browed" nil (b r aw1 d))
("brower" nil (b r aw1 er0))
("brown" nil (b r aw1 n))
("browne" nil (b r aw1 n))
("browned" nil (b r aw1 n d))
("brownell" nil (b r aw0 n eh1 l))
("browner" nil (b r aw1 n er0))
("brownfield" nil (b r aw1 n f iy1 l d))
("brownie" nil (b r aw1 n iy0))
("brownies" nil (b r aw1 n iy0 z))
("browning" nil (b r aw1 n ih0 ng))
("brownish" nil (b r aw1 n ih0 sh))
("brownlee" nil (b r aw1 n l iy0))
("brownley" nil (b r aw1 n l iy0))
("brownlie" nil (b r aw1 n l iy0))
("brownlow" nil (b r aw1 n l ow1))
("brownmiller" nil (b r aw1 n m ih1 l er0))
("brownout" nil (b r aw1 n aw1 t))
("brownouts" nil (b r aw1 n aw1 t s))
("brownrigg" nil (b r aw1 n r ih0 g))
("browns" nil (b r aw1 n z))
("brownson" nil (b r aw1 n s ax n))
("brownstein" nil (b r aw1 n s t ay1 n))
("brownstone" nil (b r aw1 n s t ow1 n))
("brownsville" nil (b r aw1 n z v ih0 l))
("brows" nil (b r aw1 z))
("browse" nil (b r aw1 z))
("browsed" nil (b r aw1 z d))
("browser" nil (b r aw1 z er0))
("browsers" nil (b r aw1 z er0 z))
("browsing" nil (b r aw1 z ih0 ng))
("brox" nil (b r aa1 k s))
("broxson" nil (b r aa1 k s ax n))
("broxterman" nil (b r aa1 k s t er0 m ax n))
("broxton" nil (b r aa1 k s t ax n))
("broy" nil (b r oy1))
("broyhill" nil (b r oy1 hh ih1 l))
("broyles" nil (b r oy1 l z))
("broz" nil (b r aa1 z))
("brozek" nil (b r ow1 z eh0 k))
("brozman" nil (b r aa1 z m ax n))
("brozovich" nil (b r aa1 z ax v ih0 ch))
("brozowski" nil (b r ax z ao1 f s k iy0))
("brubaker" nil (b r ah1 b ax k er0))
("brubeck" nil (b r uw1 b eh1 k))
("brucato" nil (b r uw0 k aa1 t ow0))
("bruce" nil (b r uw1 s))
("bruch" nil (b r ah1 ch))
("bruchhausen" nil (b r uw1 k hh aw1 z ax n))
("brucie" nil (b r ah1 k iy0))
("brucite" nil (b r uw1 s ay1 t))
("bruck" nil (b r ah1 k))
("brucker" nil (b r ah1 k er0))
("bruckman" nil (b r ah1 k m ax n))
("bruckner" nil (b r ah1 k n er0))
("brucks" nil (b r ah1 k s))
("bruder" nil (b r uw1 d er0))
("brue" nil (b r uw1))
("bruecher" nil (b r uw1 ch er0))
("brueck" nil (b r uw1 k))
("brueckner" nil (b r uh1 k n er0))
("bruegge" nil (b r uw1 g ax))
("brueggeman" nil (b r uw1 g m ax n))
("brueggemann" nil (b r uw1 g m ax n))
("brueggen" nil (b r uw1 g ax n))
("bruegger" nil (b r uw1 g er0))
("bruehl" nil (b r uw1 l))
("bruella" nil (b r uw1 eh1 l ax))
("bruemmer" nil (b r uw1 m er0))
("bruen" nil (b r uw1 n))
("bruening" nil (b r uw1 n ih0 ng))
("bruer" nil (b r uw1 er0))
("bruff" nil (b r ah1 f))
("bruford" nil (b r uw1 f er0 d))
("bruges" nil (b r uw1 jh ih0 z))
("bruggeman" nil (b r ah1 g m ax n))
("brugger" nil (b r ah1 g er0))
("brugh" nil (b r ah1))
("brugman" nil (b r ah1 g m ax n))
("bruha" nil (b r uw1 hh ax))
("bruhl" nil (b r ah1 l))
("bruhn" nil (b r ah1 n))
("bruin" nil (b r uw1 ih0 n))
("bruington" nil (b r uw1 ih0 ng t ax n))
("bruins" nil (b r uw1 ih0 n z))
("bruinsma" nil (b r uw0 ih1 n s m ax))
("bruise" nil (b r uw1 z))
("bruised" nil (b r uw1 z d))
("bruises" nil (b r uw1 z ax z))
("bruising" nil (b r uw1 z ih0 ng))
("brule" nil (b r uw1 l))
("bruley" nil (b r uw1 l iy0))
("brum" nil (b r ah1 m))
("brumbach" nil (b r ah1 m b aa1 k))
("brumback" nil (b r ah1 m b ae1 k))
("brumbaugh" nil (b r ah1 m b ao1))
("brumbelow" nil (b r ah1 m b ih0 l ow0))
("brumer" nil (b r uw1 m er0))
("brumett" nil (b r ah1 m ih0 t))
("brumfield" nil (b r ah1 m f iy1 l d))
("brumit" nil (b r uw1 m ih0 t))
("brumitt" nil (b r uw1 m ih0 t))
("brumley" nil (b r ah1 m l iy0))
("brumlow" nil (b r ah1 m l ow0))
("brumm" nil (b r ah1 m))
("brummel" nil (b r ah1 m ax l))
("brummell" nil (b r ah1 m ax l))
("brummer" nil (b r ah1 m er0))
("brummet" nil (b r ah1 m ih0 t))
("brummett" nil (b r ah1 m ih0 t))
("brummitt" nil (b r ah1 m ih0 t))
("brummond" nil (b r ah1 m ax n d))
("brun" nil (b r ah1 n))
("bruna" nil (b r uw1 n ax))
("brunch" nil (b r ah1 n ch))
("brunches" nil (b r ah1 n ch ih0 z))
("bruncor" nil (b r ah1 n k ao1 r))
("brundage" nil (b r ah1 n d ih0 jh))
("brundidge" nil (b r ah1 n d ih0 jh))
("brundige" nil (b r ah1 n d ih0 g))
("brundtland" nil (b r ah1 n t l ax n d))
("brune" nil (b r uw1 n))
("bruneau" nil (b r ax n ow1))
("brunei" nil (b r uw0 n ay1))
("brunell" nil (b r ah1 n ax l))
("brunella" nil (b r uw1 n eh1 l ax))
("brunelle" nil (b r ax n eh1 l))
("brunelli" nil (b r uw0 n eh1 l iy0))
("bruner" nil (b r uw1 n er0))
("brunet" nil (b r uw0 n eh1 t))
("brunett" nil (b r ah1 n ih0 t))
("brunetta" nil (b r uw0 n eh1 t ax))
("brunette" nil (b r uw0 n eh1 t))
("brunetti" nil (b r uw0 n eh1 t iy0))
("brunetto" nil (b r uw0 n eh1 t ow0))
("bruney" nil (b r uw1 n iy0))
("brungard" nil (b r ah1 ng g er0 d))
("brungardt" nil (b r ah1 ng g aa0 r t))
("brunger" nil (b r ah1 ng er0))
("brunhilda" nil (b r uw0 n hh iy1 l d ax))
("bruni" nil (b r uw1 n iy0))
("brunick" nil (b r uw1 n ih0 k))
("bruning" nil (b r uw1 n ih0 ng))
("brunjes" nil (b r ax n zh iy1 z))
("brunk" nil (b r ah1 ng k))
("brunke" nil (b r ah1 ng k))
("brunken" nil (b r ah1 ng k ax n))
("brunker" nil (b r ah1 ng k er0))
("brunkhorst" nil (b r ah1 ng k hh ao0 r s t))
("brunkow" nil (b r ah1 ng k ow0))
("brunn" nil (b r ah1 n))
("brunner" nil (b r ah1 n er0))
("brunnhilde" nil (b r ah1 n hh ih0 l d))
("bruno" nil (b r uw1 n ow0))
("bruns" nil (b r ah1 n z))
("brunskill" nil (b r ah1 n s k ih1 l))
("brunsman" nil (b r ah1 n s m ax n))
("brunson" nil (b r ah1 n s ax n))
("brunsvold" nil (b r ah1 n s v ow1 l d))
("brunswick" nil (b r ah1 n z w ih0 k))
("brunswig" nil (b r ah1 n s w ih0 g))
("brunt" nil (b r ah1 n t))
("bruntjen" nil (b r ah1 n t jh eh1 n))
("brunton" nil (b r ah1 n t ax n))
("brunty" nil (b r ah1 n t iy0))
("bruntz" nil (b r ah1 n t s))
("brus" nil (b r ah1 s))
("brusca" nil (b r ah1 s k ax))
("bruschi" nil (b r uw1 s k iy0))
("brusco" nil (b r uw1 s k ow0))
("bruse" nil (b r uw1 z))
("brush" nil (b r ah1 sh))
("brushaber" nil (b r ah1 sh ax b er0))
("brushed" nil (b r ah1 sh t))
("brushes" nil (b r ah1 sh ih0 z))
("brushing" nil (b r ah1 sh ih0 ng))
("brushwork" nil (b r ah1 sh w er1 k))
("brushy" nil (b r ah1 sh iy0))
("bruske" nil (b r ah1 s k))
("bruski" nil (b r ah1 s k iy0))
("brusky" nil (b r ah1 s k iy0))
("bruso" nil (b r uw1 s ow0))
("brusque" nil (b r ah1 s k))
("brusquely" nil (b r ah1 s k l iy0))
("bruss" nil (b r ah1 s))
("brusseau" nil (b r ax s ow1))
("brusselmans" nil (b r ah1 s ax l m ax n z))
("brussels" nil (b r ah1 s ax l z))
("brust" nil (b r ah1 s t))
("bruster" nil (b r ah1 s t er0))
("brustoloni" nil (b r uw1 s t ow0 l ow1 n iy0))
("brut" nil (b r uw1 t))
("brutal" nil (b r uw1 t ax l))
("brutalities" nil (b r uw0 t ae1 l ax t iy0 z))
("brutality" nil (b r uw0 t ae1 l ax t iy0))
("brutalized" nil (b r uw1 t ax l ay1 z d))
("brutalizes" nil (b r uw1 t ax l ay1 z ih0 z))
("brutalizing" nil (b r uw1 t ax l ay1 z ih0 ng))
("brutally" nil (b r uw1 t ax l iy0))
("brute" nil (b r uw1 t))
("brutish" nil (b r uw1 t ih0 sh))
("bruton" nil (b r uw1 t ax n))
("brutsche" nil (b r ah1 ch iy0))
("brutus" nil (b r uw1 t ax s))
("bruun" nil (b r uw1 n))
("bruxelles" nil (b r ax k s eh1 l ax s))
("bruyette" nil (b r uw0 eh1 t))
("bruynes" nil (b r uw1 ih0 n z))
("bruzzese" nil (b r uw0 t s ey1 z iy0))
("bryan" nil (b r ay1 ax n))
("bryans" nil (b r ay1 ax n z))
("bryant" nil (b r ay1 ax n t))
("bryars" nil (b r ay1 er0 z))
("bryce" nil (b r ay1 s))
("bryden" nil (b r ay1 d ax n))
("brydges" nil (b r ih1 jh ih0 z))
("brydie" nil (b r ih1 d iy0))
("brydon" nil (b r ih1 d ax n))
("brye" nil (b r ay1))
("bryen" nil (b r ay1 ax n))
("bryer" nil (b r ay1 er0))
("bryk" nil (b r ih1 k))
("brymer" nil (b r ay1 m er0))
("bryn" nil (b r ih1 n))
("bryna" nil (b r ih1 n ax))
("bryne" nil (b r ay1 n))
("bryner" nil (b r ay1 n er0))
("bryngelson" nil (b r ih1 ng g ih0 l s ax n))
("bryon" nil (b r ay1 ax n))
("brys" nil (b r ih1 s))
("bryson" nil (b r ay1 s ax n))
("brzezinski" nil (b r ih0 z ih1 n s k iy0))
("brzoska" nil (b r ow1 s k ax))
("brzozowski" nil (b r ax z ao1 f s k iy0))
("brzycki" nil (b r ih1 t s k iy0))
("bt" nil (b iy1 t iy1))
("bta" nil (b iy1 t iy1 ey1))
("bua" nil (b y uw1 ax))
("bub" nil (b ah1 b))
("bubar" nil (b uw1 b er0))
("bubb" nil (b ah1 b))
("bubble" nil (b ah1 b ax l))
("bubbled" nil (b ah1 b ax l d))
("bubbles" nil (b ah1 b ax l z))
("bubbling" nil (b ah1 b ax l ih0 ng))
("bubbly" nil (b ah1 b l iy0))
("bubeck" nil (b uw1 b eh0 k))
("bubel" nil (b uw1 b ax l))
("bubier" nil (b y uw1 b iy0 er0))
("bubka" nil (b ah1 b k ax))
("bublitz" nil (b ah1 b l ih0 t s))
("buboltz" nil (b y uw1 b ow1 l t s))
("bubolz" nil (b y uw1 b ow1 l z))
("bubonic" nil (b y uw0 b aa1 n ih0 k))
("buc" nil (b ah1 k))
("bucaro" nil (b uw0 k aa1 r ow0))
("buccaneers" nil (b ah1 k ax n iy1 r z))
("buccellato" nil (b uw0 ch eh0 l aa1 t ow0))
("buccheri" nil (b uw0 k eh1 r iy0))
("bucci" nil (b uw1 ch iy0))
("bucciarelli" nil (b uw0 ch er0 eh1 l iy0))
("buccieri" nil (b uw0 ch ih1 r iy0))
("buccino" nil (b uw0 ch iy1 n ow0))
("buccola" nil (b uw0 k ow1 l ax))
("bucek" nil (b uw1 ch eh0 k))
("bucey" nil (b y uw1 s iy0))
("buch" nil (b ah1 ch))
("buchalter" nil (b ah1 k ax l t er0))
("buchan" nil (b ah1 k ax n))
("buchanan" nil (b y uw0 k ae1 n ax n))
("buchannan" nil (b y uw0 k ae1 n ax n))
("buchanon" nil (b y uw0 k ae1 n ax n))
("bucharest" nil (b y uw1 k er0 eh1 s t))
("buchberger" nil (b ah1 k b er0 g er0))
("buchbinder" nil (b ah1 k b ih1 n d er0))
("buche" nil (b ah1 ch))
("buchen" nil (b ah1 k ax n))
("bucher" nil (b ah1 k er0))
("buchert" nil (b ah1 ch er0 t))
("buchheit" nil (b ah1 k hh ay0 t))
("buchholtz" nil (b ah1 k hh ow0 l t s))
("buchholz" nil (b ah1 k hh ow0 l z))
("buchi" nil (b ah1 ch iy0))
("buchinger" nil (b ah1 k ih0 n jh er0))
("buchko" nil (b ah1 ch k ow0))
("buchler" nil (b ah1 k ax l er0))
("buchman" nil (b ah1 k m ax n))
("buchmann" nil (b ah1 k m ax n))
("buchmiller" nil (b ah1 k m ax l er0))
("buchner" nil (b ah1 k n er0))
("bucholtz" nil (b ah1 k ow0 l t s))
("bucholz" nil (b ah1 k ow0 l z))
("buchs" nil (b ah1 k s))
("buchsbaum" nil (b ah1 k s b aw0 m))
("buchta" nil (b ah1 ch t ax))
("buchter" nil (b ah1 k t er0))
("buchwald" nil (b ah1 k w ao0 l d))
("buck" nil (b ah1 k))
("buckalew" nil (b ah1 k ax l uw0))
("buckbee" nil (b ah1 k b iy1))
("bucked" nil (b ah1 k t))
("buckel" nil (b ah1 k ax l))
("buckelew" nil (b ah1 k ih0 l uw0))
("buckels" nil (b ah1 k ax l z))
("bucket" nil (b ah1 k ax t))
("buckets" nil (b ah1 k ax t s))
("buckey" nil (b ah1 k iy0))
("buckeye" nil (b ah1 k ay1))
("buckhantz" nil (b ah1 k hh ae1 n t s))
("buckholtz" nil (b ah1 k hh ow1 l t s))
("buckholz" nil (b ah1 k hh ow0 l z))
("buckhorn" nil (b ah1 k hh ao1 r n))
("bucking" nil (b ah1 k ih0 ng))
("buckingham" nil (b ah1 k ih0 ng hh ae1 m))
("buckland" nil (b ah1 k l ax n d))
("buckle" nil (b ah1 k ax l))
("buckled" nil (b ah1 k ax l d))
("buckler" nil (b ah1 k ax l er0))
("buckles" nil (b ah1 k ax l z))
("bucklew" nil (b ah1 k l uw0))
("buckley" nil (b ah1 k l iy0))
("bucklin" nil (b ah1 k l ih0 n))
("buckling" nil (b ah1 k l ih0 ng))
("buckman" nil (b ah1 k m ax n))
("buckmaster" nil (b ah1 k m ae1 s t er0))
("buckminster" nil (b ah1 k m ih1 n s t er0))
("bucknam" nil (b ah1 k n ax m))
("bucknell" nil (b ah1 k n eh1 l))
("buckner" nil (b ah1 k n er0))
("bucko" nil (b ah1 k ow0))
("bucks" nil (b ah1 k s))
("buckshot" nil (b ah1 k sh aa1 t))
("buckskin" nil (b ah1 k s k ih1 n))
("buckstein" nil (b ah1 k s t iy1 n))
("buckthorn" nil (b ah1 k th ao1 r n))
("buckwalter" nil (b ah1 k w ax l t er0))
("buckwheat" nil (b ah1 k w iy1 t))
("bucolic" nil (b y uw0 k aa1 l ih0 k))
("bucy" nil (b y uw1 s iy0))
("buczek" nil (b ah1 ch eh0 k))
("buczkowski" nil (b ax ch k ao1 f s k iy0))
("buczynski" nil (b ax ch ih1 n s k iy0))
("bud" nil (b ah1 d))
("budai" nil (b uw0 d aa1 iy0))
("budapest" nil (b uw1 d ax p eh1 s t))
("buday" nil (b uw1 d ey0))
("budd" nil (b ah1 d))
("budde" nil (b ah1 d))
("budden" nil (b ah1 d ax n))
("buddenbrooks" nil (b ah1 d ax n b r uh1 k s))
("buddenhagen" nil (b ah1 d ih0 n hh ax g ax n))
("buddha" nil (b uw1 d ax))
("buddhism" nil (b uw1 d ih0 z ax m))
("buddhist" nil (b uw1 d ax s t))
("buddhists" nil (b uw1 d ax s t s))
("buddie" nil (b ah1 d iy0))
("buddies" nil (b ah1 d iy0 z))
("buddin" nil (b ah1 d ih0 n))
("budding" nil (b ah1 d ih0 ng))
("buddy" nil (b ah1 d iy0))
("buder" nil (b y uw1 d er0))
("budge" nil (b ah1 jh))
("budged" nil (b ah1 jh d))
("budget" nil (b ah1 jh ax t))
("budgetary" nil (b ah1 jh ax t eh1 r iy0))
("budgeted" nil (b ah1 jh ax t ax d))
("budgeteers" nil (b ah1 jh ax t ih1 r z))
("budgeting" nil (b ah1 jh ax t ih0 ng))
("budgets" nil (b ah1 jh ax t s))
("budging" nil (b ah1 jh ih0 ng))
("budiman" nil (b y uw1 t ih0 m ax n))
("budin" nil (b uw1 d ih0 n))
("budinger" nil (b y uw1 d ih0 ng er0))
("budka" nil (b ah1 d k ax))
("budke" nil (b ah1 d k iy0))
("budlong" nil (b ah1 d l ao1 ng))
("budner" nil (b ah1 d n er0))
("budney" nil (b ah1 d n iy0))
("budnick" nil (b ah1 d n ih0 k))
("budnik" nil (b ah1 d n ih0 k))
("budny" nil (b ah1 d n iy0))
("budreau" nil (b ax d r ow1))
("budrow" nil (b ah1 d r ow1))
("buds" nil (b ah1 d z))
("budvar" nil (b ah1 d v aa1 r))
("budweiser" nil (b ah1 d w ay0 z er0))
("budz" nil (b ah1 d z))
("budzinski" nil (b ax jh ih1 n s k iy0))
("budzynski" nil (b ax jh ih1 n s k iy0))
("bue" nil (b w eh1))
("bueche" nil (b uw1 ch))
("buechel" nil (b y uw1 k ax l))
("buechele" nil (b y uw1 k ax l))
("buechler" nil (b y uw1 k ax l er0))
("buechner" nil (b y uw1 k n er0))
("buege" nil (b uw1 jh))
("buegler" nil (b y uw1 g l er0))
("buehl" nil (b y uw1 l))
("buehler" nil (b y uw1 l er0))
("buehner" nil (b y uw1 n er0))
("buehrer" nil (b y uw1 hh er0))
("buehring" nil (b y uw1 r ih0 ng))
("buehrle" nil (b y uw1 r ax l))
("bueker" nil (b y uw1 k er0))
("buel" nil (b y uw1 l))
("buell" nil (b y uw1 ax l))
("buelow" nil (b uw1 l ow0))
("buena" nil (b uw1 n ax))
("buendia" nil (b uw1 n d iy0 ax))
("buenger" nil (b y uw1 n jh er0))
("buening" nil (b w eh1 n ih0 ng))
("bueno" nil (b w ey1 n ow0))
("buenos" nil (b w ey1 n ow0 s))
("buenrostro" nil (b w eh0 n r ow1 s t r ow0))
("buentello" nil (b uw0 n t eh1 l ow0))
("buer" nil (b uw1 er0))
("buerge" nil (b y uw1 r jh))
("buerger" nil (b y uw1 r g er0))
("buerkle" nil (b y uw1 r k ax l))
("buescher" nil (b y uw1 sh er0))
("buesing" nil (b y uw1 s ih0 ng))
("bueter" nil (b y uw1 t er0))
("buetow" nil (b uw1 t ow0))
("buettner" nil (b y uw1 t n er0))
("bufano" nil (b uw0 f aa1 n ow0))
("bufe" nil (b y uw1 f))
("bufete" nil (b y uw1 f iy1 t))
("buff" nil (b ah1 f))
("buffa" nil (b ah1 f ax))
("buffalo" nil (b ah1 f ax l ow1))
("buffaloe" nil (b ah1 f ax l ow0))
("buffalos" nil (b ah1 f ax l ow1 z))
("buffer" nil (b ah1 f er0))
("buffered" nil (b ah1 f er0 d))
("bufferin" nil (b ah1 f er0 ih0 n))
("buffers" nil (b ah1 f er0 z))
("buffet" nil (b ah1 f ax t))
("buffeted" nil (b ax f ey1 d))
("buffeting" nil (b ax f ey1 ih0 ng))
("buffets" nil (b ah1 f ax t s))
("buffett" nil (b ah1 f ih0 t))
("buffin" nil (b ah1 f ih0 n))
("buffington" nil (b ah1 f ih0 ng t ax n))
("buffkin" nil (b ah1 f k ih0 n))
("buffo" nil (b uw1 f ow0))
("buffone" nil (b uw0 f ow1 n iy0))
("buffoon" nil (b ax f uw1 n))
("bufford" nil (b ah1 f er0 d))
("buffs" nil (b ah1 f s))
("buffton" nil (b ah1 f t ax n))
("buffum" nil (b ah1 f ax m))
("bufkin" nil (b ah1 f k ih0 n))
("buford" nil (b y uw1 f er0 d))
("bug" nil (b ah1 g))
("bugaboo" nil (b ah1 g ax b uw1))
("bugaj" nil (b uw1 g ax jh))
("bugarin" nil (b y uw1 g er0 ih0 n))
("bugay" nil (b y uw1 g ey0))
("bugbee" nil (b ah1 g b iy1))
("bugeyed" nil (b ah1 g ay1 d))
("bugg" nil (b ah1 g))
("bugge" nil (b ah1 g))
("bugged" nil (b ah1 g d))
("buggers" nil (b ah1 g er0 z))
("buggies" nil (b ah1 g iy0 z))
("bugging" nil (b ah1 g ih0 ng))
("buggs" nil (b ah1 g z))
("buggy" nil (b ah1 g iy0))
("bugh" nil (b y uw1 g))
("bugher" nil (b y uw1 g er0))
("bugle" nil (b y uw1 g ax l))
("bugles" nil (b y uw1 g ax l z))
("bugling" nil (b y uw1 g ax l ih0 ng))
("bugliosi" nil (b uw0 g l iy0 ow1 s iy0))
("bugs" nil (b ah1 g z))
("buhl" nil (b y uw1 l))
("buhler" nil (b uw1 l er0))
("buhman" nil (b ah1 m ax n))
("buhr" nil (b y uh1 r))
("buhrman" nil (b uh1 r m ax n))
("buhrow" nil (b uh1 r ow0))
("bui" nil (b ih1))
("buice" nil (b ih1 s))
("buick" nil (b y uw1 ih0 k))
("buicks" nil (b y uw1 ih0 k s))
("buie" nil (b uw0 iy1))
("buikema" nil (b ih0 k ey1 m ax))
("build" nil (b ih1 l d))
("buildable" nil (b ih1 l d ax b ax l))
("builder" nil (b ih1 l d er0))
("builders" nil (b ih1 l d er0 z))
("building" nil (b ih1 l d ih0 ng))
("buildings" nil (b ih1 l d ih0 ng z))
("builds" nil (b ih1 l d z))
("buildup" nil (b ih1 l d ah1 p))
("buildups" nil (b ih1 l d ah1 p s))
("built" nil (b ih1 l t))
("buis" nil (b ih1 z))
("buisson" nil (b w iy1 s aa1 n))
("buist" nil (b uw1 ih0 s t))
("buitoni" nil (b y uw0 t ow1 n iy0))
("buitrago" nil (b ih0 t r aa1 g ow0))
("buitron" nil (b ih1 t r ax n))
("bujak" nil (b uw1 y ax k))
("bukavu" nil (b uw0 k aa1 v uw1))
("buker" nil (b y uw1 k er0))
("bukharin" nil (b ah1 k hh er0 ih0 n))
("bukovsky" nil (b uw0 k aa1 v s k iy0))
("bukowski" nil (b y uw0 k ao1 f s k iy0))
("bula" nil (b y uw1 l ax))
("bulat" nil (b uw1 l ax t))
("bulb" nil (b ah1 l b))
("bulbous" nil (b ah1 l b ax s))
("bulbs" nil (b ah1 l b z))
("bulen" nil (b ah1 l ax n))
("bulent" nil (b y uw1 l ax n t))
("buley" nil (b y uw1 l iy0))
("bulfinch" nil (b uh1 l f ih1 n ch))
("bulgaria" nil (b ax l g eh1 r iy0 ax))
("bulgarian" nil (b ax l g eh1 r iy0 ax n))
("bulgarians" nil (b ax l g eh1 r iy0 ax n z))
("bulge" nil (b ah1 l jh))
("bulged" nil (b ah1 l jh d))
("bulger" nil (b ah1 l g er0))
("bulges" nil (b ah1 l jh ih0 z))
("bulging" nil (b ah1 l jh ih0 ng))
("bulgrin" nil (b uh1 l g r ih0 n))
("bulimia" nil (b y uw0 l ih1 m iy0 ax))
("bulin" nil (b y uw1 l ih0 n))
("bulk" nil (b ah1 l k))
("bulkeley" nil (b uh1 l k ax l iy0))
("bulkhead" nil (b ah1 l k hh eh1 d))
("bulkheads" nil (b ah1 l k hh eh1 d z))
("bulkier" nil (b ah1 l k iy0 er0))
("bulkley" nil (b ah1 l k l iy0))
("bulky" nil (b ah1 l k iy0))
("bull" nil (b uh1 l))
("bulla" nil (b uh1 l ax))
("bullard" nil (b uh1 l er0 d))
("bulldog" nil (b uh1 l d ao1 g))
("bulldoze" nil (b uh1 l d ow1 z))
("bulldozed" nil (b uh1 l d ow1 z d))
("bulldozer" nil (b uh1 l d ow1 z er0))
("bulldozers" nil (b uh1 l d ow1 z er0 z))
("bulldozing" nil (b uh1 l d ow1 z ih0 ng))
("bullen" nil (b uh1 l ax n))
("buller" nil (b uh1 l er0))
("bullet" nil (b uh1 l ax t))
("bulletin" nil (b uh1 l ih0 t ax n))
("bulletins" nil (b uh1 l ax t ax n z))
("bulletproof" nil (b uh1 l ax t p r uw1 f))
("bullets" nil (b uh1 l ax t s))
("bullfighter" nil (b uh1 l f ay1 t er0))
("bullheads" nil (b uh1 l hh eh1 d z))
("bullhorn" nil (b uh1 l hh ao1 r n))
("bullhorns" nil (b uh1 l hh ao1 r n z))
("bullied" nil (b uh1 l iy0 d))
("bullies" nil (b uh1 l iy0 z))
("bullinger" nil (b uh1 l ih0 ng er0))
("bullington" nil (b uh1 l ih0 ng t ax n))
("bullins" nil (b uh1 l ih0 n z))
("bullion" nil (b uh1 l y ax n))
("bullis" nil (b uh1 l ih0 s))
("bullish" nil (b uh1 l ih0 sh))
("bullishly" nil (b uh1 l ih0 sh l iy0))
("bullishness" nil (b uh1 l ih0 sh n ax s))
("bullman" nil (b uh1 l m ax n))
("bulloch" nil (b uh1 l ax k))
("bullock" nil (b uh1 l ax k))
("bullocks" nil (b uh1 l ax k s))
("bullpen" nil (b uh1 l p eh1 n))
("bulls" nil (b uh1 l z))
("bulluck" nil (b uh1 l ax k))
("bullwinkle" nil (b uh1 l w ih1 ng k ax l))
("bully" nil (b uh1 l iy0))
("bullying" nil (b uh1 l iy0 ih0 ng))
("bulman" nil (b ah1 l m ax n))
("bulmer" nil (b ah1 l m er0))
("bulova" nil (b y uw0 l ow1 v ax))
("bulow" nil (b y uw1 l ow0))
("bulrush" nil (b uh1 l r ax sh))
("bulson" nil (b uh1 l s ax n))
("bult" nil (b ah1 l t))
("bultema" nil (b uw0 l t eh1 m ax))
("bulthuis" nil (b ah1 l dh uw0 ih0 z))
("bultman" nil (b uh1 l t m ax n))
("bulwark" nil (b uh1 l w er0 k))
("bum" nil (b ah1 m))
("bumann" nil (b y uw1 m ax n))
("bumbalough" nil (b ah1 m b ax l aw0))
("bumbarger" nil (b ah1 m b aa1 r g er0))
("bumbaugh" nil (b ah1 m b ao1))
("bumble" nil (b ah1 m b ax l))
("bumbling" nil (b ah1 m b ax l ih0 ng))
("bumgardner" nil (b ah1 m g aa1 r d n er0))
("bumgarner" nil (b ah1 m g aa1 r n er0))
("bumiputra" nil (b uw1 m iy0 p y uw1 t r ax))
("bummed" nil (b ah1 m d))
("bummer" nil (b ah1 m er0))
("bump" nil (b ah1 m p))
("bumpas" nil (b ah1 m p ax z))
("bumpass" nil (b ah1 m p ae1 s))
("bumped" nil (b ah1 m p t))
("bumper" nil (b ah1 m p er0))
("bumpers" nil (b ah1 m p er0 z))
("bumping" nil (b ah1 m p ih0 ng))
("bumpings" nil (b ah1 m p ih0 ng z))
("bumpkin" nil (b ah1 m p k ih0 n))
("bumps" nil (b ah1 m p s))
("bumpus" nil (b ah1 m p ax s))
("bumpy" nil (b ah1 m p iy0))
("bums" nil (b ah1 m z))
("bumstead" nil (b ah1 m s t eh1 d))
("bun" nil (b ah1 n))
("bunce" nil (b ah1 n s))
("bunch" nil (b ah1 n ch))
("bunched" nil (b ah1 n ch t))
("bunches" nil (b ah1 n ch ax z))
("bunching" nil (b ah1 n ch ih0 ng))
("bunchy" nil (b ah1 n ch iy0))
("bund" nil (b ah1 n d))
("bunda" nil (b ah1 n d ax))
("bunde" nil (b ah1 n d))
("bundesbank" nil (b uh1 n d ih0 s b ae1 ng k))
("bundespost" nil (b uh1 n d ih0 s p ow1 s t))
("bundesrat" nil (b uh1 n d ih0 s r ae1 t))
("bundestag" nil (b ah1 n d ax s t ae1 g))
("bundeswehr" nil (b ah1 n d ax s w eh1 r))
("bundick" nil (b ah1 n d ih0 k))
("bundle" nil (b ah1 n d ax l))
("bundled" nil (b ah1 n d ax l d))
("bundles" nil (b ah1 n d ax l z))
("bundling" nil (b ah1 n d ax l ih0 ng))
("bundren" nil (b ah1 n d er0 ax n))
("bundrick" nil (b ah1 n d r ih0 k))
("bunds" nil (b ah1 n d z))
("bundschuh" nil (b ah1 n d sh uw0))
("bundy" nil (b ah1 n d iy0))
("bundys" nil (b ah1 n d iy0 z))
("bung" nil (b ah1 ng))
("bungalow" nil (b ah1 ng g ax l ow1))
("bungalows" nil (b ah1 ng g ax l ow1 z))
("bungard" nil (b ah1 ng g er0 d))
("bunge" nil (b ah1 n jh))
("bungee" nil (b ah1 n jh iy0))
("bunger" nil (b ah1 ng er0))
("bungert" nil (b ah1 ng g er0 t))
("bungey" nil (b ah1 n jh iy0))
("bungled" nil (b ah1 ng g ax l d))
("bungling" nil (b ah1 ng g ax l ih0 ng))
("bunk" nil (b ah1 ng k))
("bunke" nil (b ah1 ng k))
("bunker" nil (b ah1 ng k er0))
("bunkers" nil (b ah1 ng k er0 z))
("bunkley" nil (b ah1 ng k l iy0))
("bunks" nil (b ah1 ng k s))
("bunn" nil (b ah1 n))
("bunnell" nil (b ah1 n ax l))
("bunner" nil (b ah1 n er0))
("bunney" nil (b ah1 n iy0))
("bunni" nil (b uw1 n iy0))
("bunnie" nil (b ah1 n iy0))
("bunnies" nil (b ah1 n iy0 z))
("bunning" nil (b ah1 n ih0 ng))
("bunny" nil (b ah1 n iy0))
("buns" nil (b ah1 n z))
("bunt" nil (b ah1 n t))
("buntain" nil (b ax n t ey1 n))
("bunte" nil (b ah1 n t))
("bunten" nil (b ah1 n t ax n))
("buntin" nil (b ah1 n t ih0 n))
("bunting" nil (b ah1 n t ih0 ng))
("bunton" nil (b ah1 n t ax n))
("buntrock" nil (b ah1 n t r aa1 k))
("buntstrock" nil (b ah1 n t s t r aa1 k))
("buntyn" nil (b ah1 n t ih0 n))
("buntz" nil (b ah1 n t s))
("bunyan" nil (b ah1 n y ax n))
("bunyard" nil (b ax n y aa1 r d))
("bunzl" nil (b ah1 n z ax l))
("buol" nil (b ow1 l))
("buonanno" nil (b w ow0 n aa1 n ow0))
("buonicontis" nil (b w aa1 n ih0 k aa1 n t iy0 z))
("buono" nil (b w ow1 n ow0))
("buonocore" nil (b w ow0 n ow0 k ao1 r iy0))
("buonomo" nil (b w ow0 n ow1 m ow0))
("buoy" nil (b uw1 iy0))
("buoyancy" nil (b oy1 ax n s iy0))
("buoyant" nil (b oy1 ax n t))
("buoyed" nil (b uw1 iy0 d))
("buoying" nil (b oy1 ih0 ng))
("buoys" nil (b uw1 iy0 z))
("bupp" nil (b ah1 p))
("buprenorphine" nil (b y uw1 p r ax n er0 f iy1 n))
("bur" nil (b er1))
("burack" nil (b y uw1 r ax k))
("burak" nil (b y uw1 r ax k))
("buran" nil (b y uw1 r ax n))
("burandt" nil (b y uw1 r ax n t))
("buras" nil (b y uw1 r ax z))
("burba" nil (b er1 b ax))
("burbach" nil (b er1 b aa0 k))
("burback" nil (b er1 b ae0 k))
("burbage" nil (b er1 b ih0 jh))
("burbank" nil (b er1 b ae1 ng k))
("burbidge" nil (b er1 b ih0 jh))
("burbridge" nil (b er1 b r ih0 jh))
("burby" nil (b er1 b iy0))
("burch" nil (b er1 ch))
("burcham" nil (b er0 sh ae1 m))
("burchard" nil (b er0 sh aa1 r d))
("burchell" nil (b er1 k ax l))
("burcher" nil (b er1 ch er0))
("burchett" nil (b er1 ch ih0 t))
("burchette" nil (b er0 sh eh1 t))
("burchfield" nil (b er1 ch f iy0 l d))
("burchill" nil (b er1 k hh ih0 l))
("burciaga" nil (b uh0 r s iy0 aa1 g ax))
("burck" nil (b er1 k))
("burckhard" nil (b er1 k hh er0 d))
("burckhardt" nil (b er1 k hh aa0 r t))
("burd" nil (b er1 d))
("burda" nil (b uh1 r d ax))
("burdell" nil (b er0 d eh1 l))
("burden" nil (b er1 d ax n))
("burdened" nil (b er1 d ax n d))
("burdening" nil (b er1 d ax n ih0 ng))
("burdens" nil (b er1 d ax n z))
("burdensome" nil (b er1 d ax n s ax m))
("burdett" nil (b er1 d ih0 t))
("burdette" nil (b er0 d eh1 t))
("burdge" nil (b er1 jh))
("burdi" nil (b uh1 r d iy0))
("burdick" nil (b er1 d ih0 k))
("burdin" nil (b er1 d ih0 n))
("burdine" nil (b uh0 r d iy1 n iy0))
("burdines" nil (b er0 d iy1 n eh0 z))
("burditt" nil (b er0 d ih1 t))
("burdo" nil (b uh1 r d ow0))
("burdon" nil (b er1 d ax n))
("burdsall" nil (b er1 d s ax l))
("bureau" nil (b y uh1 r ow0))
("bureaucracies" nil (b y uh0 r aa1 k r ax s iy0 z))
("bureaucracy" nil (b y uh0 r aa1 k r ax s iy0))
("bureaucrat" nil (b y uh1 r ax k r ae1 t))
("bureaucratic" nil (b y uh1 r ax k r ae1 t ih0 k))
("bureaucrats" nil (b y uh1 r ax k r ae1 t s))
("bureaus" nil (b y uh1 r ow0 z))
("burek" nil (b eh1 r ih0 k))
("burel" nil (b eh1 r ax l))
("burell" nil (b eh1 r ax l))
("buren" nil (b y uh1 r ax n))
("bures" nil (b eh1 r z))
("buresh" nil (b eh1 r ih0 sh))
("burfeind" nil (b er1 f ay0 n d))
("burfield" nil (b er1 f iy0 l d))
("burford" nil (b er1 f er0 d))
("burg" nil (b er1 g))
("burgamy" nil (b er1 g ax m iy0))
("burgan" nil (b er1 g ae0 n))
("burgard" nil (b er1 g er0 d))
("burgdorf" nil (b er1 g d ao0 r f))
("burge" nil (b er1 g))
("burgee" nil (b er1 g iy0))
("burgener" nil (b er1 g iy0 n er0))
("burgeoned" nil (b er1 jh ax n d))
("burgeoning" nil (b er1 jh ax n ih0 ng))
("burger" nil (b er1 g er0))
("burgers" nil (b er1 g er0 z))
("burgert" nil (b er1 g er0 t))
("burges" nil (b er1 g z))
("burgeson" nil (b er1 g ih0 s ax n))
("burgess" nil (b er1 jh ax s))
("burget" nil (b er1 g eh0 t))
("burgett" nil (b er1 jh ih0 t))
("burggraf" nil (b er1 g r ax f))
("burghardt" nil (b er1 g aa0 r t))
("burghart" nil (b er1 g hh aa0 r t))
("burgher" nil (b er1 g er0))
("burgin" nil (b er1 g ih0 n))
("burgio" nil (b er1 g iy0 ow0))
("burglar" nil (b er1 g l er0))
("burglaries" nil (b er1 g l er0 iy0 z))
("burglarize" nil (b er1 g l er0 ay1 z))
("burglars" nil (b er1 g l er0 z))
("burglary" nil (b er1 g l er0 iy0))
("burgling" nil (b er1 g l ih0 ng))
("burgman" nil (b er1 g m ax n))
("burgmaster" nil (b er1 g m ae1 s t er0))
("burgner" nil (b er1 g n er0))
("burgo" nil (b er1 g ow0))
("burgomasters" nil (b er1 g ax m ae1 s t er0 z))
("burgoon" nil (b er1 g uw0 n))
("burgos" nil (b er1 g ow0 z))
("burgoyne" nil (b er0 g oy1 n))
("burgundians" nil (b er0 g ah1 n d iy0 ax n z))
("burgundies" nil (b er1 g ax n d iy0 z))
("burgundy" nil (b er1 g ax n d iy0))
("burgy" nil (b er1 jh iy0))
("burham" nil (b er1 hh ax m))
("burhans" nil (b er1 hh ax n z))
("buri" nil (b uh1 r iy0))
("burial" nil (b eh1 r iy0 ax l))
("burials" nil (b eh1 r iy0 ax l z))
("burian" nil (b y uh1 r iy0 ax n))
("burich" nil (b eh1 r ih0 k))
("buried" nil (b eh1 r iy0 d))
("buries" nil (b eh1 r iy0 z))
("burk" nil (b er1 k))
("burkard" nil (b er1 k er0 d))
("burkart" nil (b er1 k aa0 r t))
("burke" nil (b er1 k))
("burkeen" nil (b er0 k iy1 n))
("burkel" nil (b er1 k ax l))
("burkemper" nil (b er1 k ih0 m p er0))
("burkert" nil (b er1 k er0 t))
("burkes" nil (b er1 k s))
("burket" nil (b er1 k ih0 t))
("burkett" nil (b er1 k ih0 t))
("burkey" nil (b er1 k iy0))
("burkhalter" nil (b er1 k hh ax l t er0))
("burkhammer" nil (b er1 k hh ax m er0))
("burkhard" nil (b er1 k hh er0 d))
("burkhardt" nil (b er1 k hh aa0 r t))
("burkhart" nil (b er1 k hh aa0 r t))
("burkhead" nil (b er1 k hh eh0 d))
("burkholder" nil (b er1 k hh ow0 l d er0))
("burkina" nil (b er0 k iy1 n ax))
("burkins" nil (b er1 k ih0 n z))
("burkitt" nil (b er1 k ih0 t))
("burkland" nil (b er1 k l ax n d))
("burkle" nil (b er1 k ax l))
("burkley" nil (b er1 k l iy0))
("burklow" nil (b er1 k l aw0))
("burklund" nil (b er1 k l ax n d))
("burkman" nil (b er1 k m ax n))
("burks" nil (b er1 k s))
("burl" nil (b er1 l))
("burland" nil (b er1 l ax n d))
("burlap" nil (b er1 l ae1 p))
("burlapped" nil (b er1 l ae1 p t))
("burlatsky" nil (b er0 l ae1 t s k iy0))
("burleigh" nil (b er1 l ax))
("burleson" nil (b er1 l ih0 s ax n))
("burlesque" nil (b er0 l eh1 s k))
("burlew" nil (b er1 l uw0))
("burley" nil (b er1 l iy0))
("burling" nil (b er1 l ih0 ng))
("burlingame" nil (b er1 l ih0 ng g ey1 m))
("burlingham" nil (b er1 l ih0 ng hh ae1 m))
("burlington" nil (b er1 l ih0 ng t ax n))
("burlison" nil (b er1 l ih0 s ax n))
("burly" nil (b er1 l iy0))
("burma" nil (b er1 m ax))
("burmah" nil (b er1 m ax))
("burman" nil (b er1 m ax n))
("burmans" nil (b er1 m ax n z))
("burmaster" nil (b er1 m ae0 s t er0))
("burmeister" nil (b er1 m ay0 s t er0))
("burmese" nil (b er0 m iy1 z))
("burmester" nil (b er1 m ih0 s t er0))
("burn" nil (b er1 n))
("burnable" nil (b er1 n ax b ax l))
("burnaby" nil (b er1 n ax b iy0))
("burnam" nil (b er1 n ax m))
("burnap" nil (b er1 n ax p))
("burnard" nil (b er0 n aa1 r d))
("burndy" nil (b er1 n d iy0))
("burne" nil (b er1 n))
("burned" nil (b er1 n d))
("burnell" nil (b er1 n ax l))
("burner" nil (b er1 n er0))
("burners" nil (b er1 n er0 z))
("burnes" nil (b er1 n z))
("burness" nil (b er1 n ax s))
("burnet" nil (b er1 n ih0 t))
("burnett" nil (b er0 n eh1 t))
("burnette" nil (b er1 n eh1 t))
("burney" nil (b er1 n iy0))
("burnham" nil (b er1 n ax m))
("burning" nil (b er1 n ih0 ng))
("burningham" nil (b er1 n ih0 ng hh ae1 m))
("burnings" nil (b er1 n ih0 ng z))
("burnish" nil (b er1 n ih0 sh))
("burnished" nil (b er1 n ih0 sh t))
("burnley" nil (b er1 n l iy0))
("burnout" nil (b er1 n aw1 t))
("burns" nil (b er1 n z))
("burnsed" nil (b er1 n z d))
("burnside" nil (b er1 n s ay1 d))
("burnstein" nil (b er1 n s t ay1 n))
("burnsworth" nil (b er1 n z w er1 th))
("burnt" nil (b er1 n t))
("burnup" nil (b er1 n ah1 p))
("burnworth" nil (b er1 n w er1 th))
("buroker" nil (b y uw1 r ax k er0))
("burow" nil (b y uh1 r ow0))
("burp" nil (b er1 p))
("burpee" nil (b er1 p iy0))
("burping" nil (b er1 p ih0 ng))
("burpo" nil (b uh1 r p ow0))
("burr" nil (b er1))
("burrage" nil (b er1 ih0 jh))
("burrell" nil (b er0 eh1 l))
("burrer" nil (b er1 er0))
("burres" nil (b er1 z))
("burress" nil (b er1 ax s))
("burri" nil (b uh1 r iy0))
("burridge" nil (b ao1 r ih0 jh))
("burrier" nil (b er1 iy0 er0))
("burright" nil (b ao1 r ay0 t))
("burrill" nil (b ao1 r ax l))
("burrington" nil (b er1 ih0 ng t ax n))
("burris" nil (b er1 ih0 s))
("burriss" nil (b er1 ih0 s ih0 z))
("burrito" nil (b er0 iy1 t ow0))
("burritos" nil (b er0 iy1 t ow0 s))
("burritt" nil (b er1 ih0 t))
("burro" nil (b er1 ow0))
("burrola" nil (b uh0 r ow1 l ax))
("burros" nil (b er1 ow0 z))
("burrough" nil (b er1 ow0))
("burroughs" nil (b ah1 r ow0 z))
("burrous" nil (b er1 ax s))
("burrow" nil (b er1 ow0))
("burrowers" nil (b er1 ow0 er0 z))
("burrowes" nil (b er1 ow1 z))
("burrowing" nil (b er1 ow0 ih0 ng))
("burrows" nil (b er1 ow0 z))
("burrus" nil (b uh1 r ax s))
("burruss" nil (b uh1 r ax s))
("burry" nil (b er1 iy0))
("bursch" nil (b er1 sh))
("burse" nil (b er1 s))
("bursey" nil (b er1 s iy0))
("bursley" nil (b er1 s l iy0))
("burson" nil (b er1 s ax n))
("burst" nil (b er1 s t))
("burstein" nil (b er1 s t ay0 n))
("bursting" nil (b er1 s t ih0 ng))
("burston" nil (b er1 s t ax n))
("bursts" nil (b er1 s t s))
("burt" nil (b er1 t))
("burtch" nil (b er1 ch))
("burtis" nil (b er1 t ih0 s))
("burtner" nil (b er1 t n er0))
("burtness" nil (b er1 t n ih0 s))
("burtnett" nil (b er1 t n ih0 t))
("burton" nil (b er1 t ax n))
("burts" nil (b er1 t s))
("burtt" nil (b er1 t))
("burundi" nil (b er0 ah1 n d iy0))
("burwell" nil (b er1 w eh0 l))
("burwick" nil (b er1 w ih0 k))
("bury" nil (b eh1 r iy0))
("burying" nil (b eh1 r iy0 ih0 ng))
("burzynski" nil (b er0 z ih1 n s k iy0))
("bus" nil (b ah1 s))
("busa" nil (b y uw1 s ax))
("busalacchi" nil (b uw0 s aa0 l aa1 k iy0))
("busam" nil (b ih1 s ax m))
("busbee" nil (b ah1 s b iy1))
("busbey" nil (b ah1 s b iy0))
("busbin" nil (b ah1 s b ih0 n))
("busboom" nil (b ah1 s b uw1 m))
("busboy" nil (b ah1 s b oy1))
("busboys" nil (b ah1 s b oy1 z))
("busby" nil (b ah1 z b iy0))
("buscaglia" nil (b uw0 s k aa1 g l iy0 ax))
("buscemi" nil (b uw0 s ch eh1 m iy0))
("busch" nil (b uh1 sh))
("buschbaum" nil (b uh1 sh b aw1 m))
("busche" nil (b ah1 sh))
("buscher" nil (b uw1 sh er0))
("buschman" nil (b ah1 sh m ax n))
("buschmann" nil (b ah1 sh m ax n))
("buse" nil (b y uw1 z))
("bused" nil (b ah1 s t))
("buseman" nil (b ih1 s m ax n))
("busenbark" nil (b ih1 s ih0 n b aa0 r k))
("buser" nil (b ih1 s er0))
("buses" nil (b ah1 s ih0 z))
("busey" nil (b y uw1 z iy0))
("bush" nil (b uh1 sh))
("busha" nil (b ah1 sh ax))
("bushard" nil (b uh1 sh er0 d))
("bushart" nil (b uh1 sh aa0 r t))
("bushaw" nil (b uh1 sh ao0))
("bushby" nil (b uh1 sh b iy0))
("bushee" nil (b uh1 sh iy1))
("bushel" nil (b uh1 sh ax l))
("bushell" nil (b uh1 sh ax l))
("bushels" nil (b uh1 sh ax l z))
("busher" nil (b uh1 sh er0))
("bushes" nil (b uh1 sh ax z))
("bushey" nil (b uh1 sh iy0))
("bushings" nil (b uh1 sh ih0 ng z))
("bushkin" nil (b uh1 sh k ih1 n))
("bushman" nil (b uh1 sh m ax n))
("bushnell" nil (b uh1 sh n ax l))
("bushong" nil (b uh1 sh ao0 ng))
("bushway" nil (b uh1 sh w ey1))
("bushy" nil (b uh1 sh iy0))
("busic" nil (b ah1 s ih0 k))
("busick" nil (b ih1 s ih0 k))
("busied" nil (b ih1 z iy0 d))
("busier" nil (b ih1 z iy0 er0))
("busiest" nil (b ih1 z iy0 ax s t))
("busily" nil (b ih1 z ax l iy0))
("business" nil (b ih1 z n ax s))
("businesses" nil (b ih1 z n ax s ax z))
("businessland" nil (b ih1 z n ih0 s l ae1 n d))
("businesslike" nil (b ih1 z n ih0 s l ay1 k))
("businessman" nil (b ih1 z n ax s m ae1 n))
("businessmen" nil (b ih1 z n ih0 s m eh1 n))
("businesspeople" nil (b y uw1 s ih0 n ax s p iy1 p ax l))
("businessphones" nil (b y uw0 s ih1 n ax s f ow1 n z))
("businesswoman" nil (b ih1 z n ih0 s w uh1 m ax n))
("businesswomen" nil (b y uw1 s ih0 n ax s w ow1 m ax n))
("busing" nil (b ah1 s ih0 ng))
("busk" nil (b ah1 s k))
("buske" nil (b ah1 s k))
("busker" nil (b ah1 s k er0))
("buskey" nil (b ah1 s k iy1))
("buskirk" nil (b ah1 s k er0 k))
("buslease" nil (b ah1 s l iy1 s))
("busler" nil (b ah1 s ax l er0))
("busload" nil (b ax s l ow1 d))
("busloads" nil (b ax s l ow1 d z))
("buspar" nil (b ah1 s p er0))
("buss" nil (b ah1 s))
("bussa" nil (b uw1 s ax))
("bussard" nil (b ah1 s er0 d))
("busse" nil (b ah1 s))
("bussed" nil (b ah1 s t))
("bussell" nil (b ah1 s ax l))
("bussen" nil (b ah1 s ax n))
("busser" nil (b ah1 s er0))
("bussert" nil (b ah1 s er0 t))
("busses" nil (b ah1 s ax z))
("bussey" nil (b ah1 s iy0))
("bussi" nil (b ah1 s iy0))
("bussie" nil (b ah1 s iy0))
("bussiere" nil (b ah1 s iy0 eh0 r))
("bussing" nil (b ah1 s ih0 ng))
("bussinger" nil (b ah1 s ih0 n jh er0))
("bussman" nil (b ah1 s m ax n))
("bust" nil (b ah1 s t))
("busta" nil (b ah1 s t ax))
("bustamante" nil (b uw1 s t ax m aa1 n t iy0))
("bustard" nil (b ah1 s t er0 d))
("busted" nil (b ah1 s t ih0 d))
("buster" nil (b ah1 s t er0))
("busters" nil (b ah1 s t er0 z))
("busti" nil (b ah1 s t iy0))
("bustillo" nil (b uw0 s t ih1 l ow0))
("bustillos" nil (b ah1 s t ay0 l ow0 z))
("bustin" nil (b ah1 s t ih0 n))
("busting" nil (b ah1 s t ih0 ng))
("bustle" nil (b ah1 s ax l))
("bustling" nil (b ah1 s ax l ih0 ng))
("busto" nil (b ah1 s t ow0))
("bustos" nil (b ah1 s t ow0 z))
("busts" nil (b ah1 s t s))
("buswell" nil (b ah1 s w eh1 l))
("busy" nil (b ih1 z iy0))
("but" nil (b ah1 t))
("butala" nil (b uw0 t aa1 l ax))
("butane" nil (b y uw0 t ey1 n))
("butare" nil (b uw0 t aa1 r iy0))
("butch" nil (b uh1 ch))
("butchart" nil (b ah1 ch er0 t))
("butcher" nil (b uh1 ch er0))
("butchered" nil (b uh1 ch er0 d))
("butchering" nil (b uh1 ch er0 ih0 ng))
("butchers" nil (b uh1 ch er0 z))
("butchery" nil (b uh1 ch er0 iy0))
("butchko" nil (b ah1 ch k ow0))
("bute" nil (b y uw1 t))
("buteau" nil (b y uw0 t ow1))
("buteco" nil (b uw1 t eh1 k ow0))
("butenhoff" nil (b y uw1 t ih0 n hh ao0 f))
("butera" nil (b uw0 t eh1 r ax))
("buterbaugh" nil (b y uw1 t er0 b aw0))
("buth" nil (b uw1 th))
("buthelezi" nil (b uw1 t ax l ey1 z iy0))
("butka" nil (b ah1 t k ax))
("butkiewicz" nil (b ah1 t k ax v ih0 ch))
("butkovich" nil (b ah1 t k ax v ih0 ch))
("butkus" nil (b ah1 t k ih0 s))
("butler" nil (b ah1 t l er0))
("butman" nil (b ah1 t m ax n))
("butner" nil (b ah1 t n er0))
("butorac" nil (b y uw0 t ao1 r ax k))
("butrick" nil (b ah1 t r ih0 k))
("butros" nil (b uw1 t r ow1 s))
("buts" nil (b ah1 t s))
("butsch" nil (b ah1 ch))
("butson" nil (b ah1 t s ax n))
("butt" nil (b ah1 t))
("buttacavoli" nil (b uw0 t aa0 k aa0 v ow1 l iy0))
("buttafuoco" nil (b uw0 t ax f w ow1 k ow0))
("buttars" nil (b ah1 t er0 z))
("butte" nil (b y uw1 t))
("butter" nil (b ah1 t er0))
("butterball" nil (b ah1 t er0 b ao1 l))
("butterbaugh" nil (b ah1 t er0 b aw0))
("buttercup" nil (b ah1 t er0 k ah1 p))
("buttercups" nil (b ah1 t er0 k ah1 p s))
("buttered" nil (b ah1 t er0 d))
("butterfat" nil (b ah1 t er0 f ae1 t))
("butterfield" nil (b ah1 t er0 f iy0 l d))
("butterflies" nil (b ah1 t er0 f l ay1 z))
("butterfly" nil (b ah1 t er0 f l ay1))
("buttermilk" nil (b ah1 t er0 m ih1 l k))
("buttermore" nil (b ah1 t er0 m ao0 r))
("butters" nil (b ah1 t er0 z))
("butterscotch" nil (b ah1 t er0 s k aa1 ch))
("butterworth" nil (b ah1 t er0 w er1 th))
("buttery" nil (b ah1 t er0 iy0))
("butting" nil (b ah1 t ih0 ng))
("buttitta" nil (b uw0 t iy1 t ax))
("buttke" nil (b ah1 t k iy0))
("buttler" nil (b ah1 t l er0))
("buttner" nil (b ah1 t n er0))
("buttocks" nil (b ah1 t ax k s))
("button" nil (b ah1 t ax n))
("buttoned" nil (b ah1 t ax n d))
("buttonholed" nil (b ah1 t ax n hh ow1 l d))
("buttonholes" nil (b ah1 t ax n hh ow1 l z))
("buttons" nil (b ah1 t ax n z))
("buttonville" nil (b ah1 t ax n v ih1 l))
("buttram" nil (b ah1 t r ae1 m))
("buttress" nil (b ah1 t r ax s))
("buttressed" nil (b ah1 t r ax s t))
("buttresses" nil (b ah1 t r ax s ax z))
("buttressing" nil (b ah1 t r ax s ih0 ng))
("buttrey" nil (b ah1 t r iy0))
("buttrick" nil (b ah1 t r ih0 k))
("buttrum" nil (b ah1 t r ax m))
("buttry" nil (b ah1 t r iy0))
("butts" nil (b ah1 t s))
("butulesi" nil (b uw1 t ax l ey1 z iy0))
("butyl" nil (b y uw1 t ax l))
("butz" nil (b ah1 t s))
("butzberger" nil (b ah1 t s b er0 g er0))
("butzen" nil (b ah1 t s ax n))
("butzer" nil (b ah1 t s er0))
("butzin" nil (b ah1 t s ax n))
("buus" nil (b uw1 z))
("buxbaum" nil (b ah1 k s b aw0 m))
("buxom" nil (b ah1 k s ax m))
("buxton" nil (b ah1 k s t ax n))
("buy" nil (b ay1))
("buyback" nil (b ay1 b ae1 k))
("buybacks" nil (b ay1 b ae1 k s))
("buyer" nil (b ay1 er0))
("buyers" nil (b ay1 er0 z))
("buying" nil (b ay1 ih0 ng))
("buyout" nil (b ay1 aw1 t))
("buyouts" nil (b ay1 aw1 t s))
("buys" nil (b ay1 z))
("buysse" nil (b ay1 s))
("buza" nil (b y uw1 z ax))
("buzan" nil (b y uw1 z ax n))
("buzard" nil (b y uw0 z aa1 r d))
("buzbee" nil (b ah1 z b iy1))
("buzby" nil (b ah1 z b iy0))
("buzek" nil (b uw1 z eh0 k))
("buzz" nil (b ah1 z))
("buzzard" nil (b ah1 z er0 d))
("buzzards" nil (b ah1 z er0 d z))
("buzzed" nil (b ah1 z d))
("buzzell" nil (b ax z eh1 l))
("buzzelli" nil (b uw0 t s eh1 l iy0))
("buzzer" nil (b ah1 z er0))
("buzzetta" nil (b y uw0 z eh1 t ax))
("buzzing" nil (b ah1 z ih0 ng))
("buzzword" nil (b ah1 z w er0 d))
("buzzwords" nil (b ah1 z w er0 d z))
("buzzy" nil (b ah1 z iy0))
("by" nil (b ay1))
("byam" nil (b ay1 ax m))
("byard" nil (b y aa1 r d))
("byars" nil (b ay1 er0 z))
("byas" nil (b ay1 ax s))
("byassee" nil (b iy0 aa1 s iy0))
("bybee" nil (b ay1 b iy1))
("bye" nil (b ay1))
("byelorussia" nil (b ay1 ax l ow0 r ah1 sh ax))
("byer" nil (b ay1 er0))
("byerlein" nil (b ay1 r l ay1 n))
("byerley" nil (b ay1 er0 l iy0))
("byerly" nil (b ay1 er0 l iy0))
("byers" nil (b ay1 er0 z))
("byfield" nil (b ay1 f iy1 l d))
("byford" nil (b ih1 f er0 d))
("bygone" nil (b ay1 g ao1 n))
("bygones" nil (b ay1 g ao1 n z))
("byington" nil (b ay1 ih0 ng t ax n))
("byker" nil (b ay1 k er0))
("bykowski" nil (b ih0 k ao1 f s k iy0))
("byland" nil (b ay1 l ax n d))
("bylaw" nil (b ay1 l ao1))
("bylaws" nil (b ay1 l ao1 z))
("byler" nil (b ay1 l er0))
("byles" nil (b ay1 l z))
("byline" nil (b ay1 l ay1 n))
("bylines" nil (b ay1 l iy1 n z))
("bylsma" nil (b ih1 l s m ax))
("bylund" nil (b ih1 l ax n d))
("byner" nil (b ay1 n er0))
("bynes" nil (b ay1 n z))
("bynoe" nil (b ih1 n ow0))
("bynum" nil (b ih1 n ax m))
("bypass" nil (b ay1 p ae1 s))
("bypassed" nil (b ay1 p ae1 s t))
("bypasses" nil (b ay1 p ae1 s ih0 z))
("bypassing" nil (b ay1 p ae1 s ih0 ng))
("byproduct" nil (b ay1 p r aa0 d ax k t))
("byproducts" nil (b ay1 p r ao1 d ax k t s))
("byram" nil (b ih1 r ax m))
("byrd" nil (b er1 d))
("byrer" nil (b ay1 r er0))
("byrge" nil (b ay1 r jh))
("byrle" nil (b ay1 r ax l))
("byrn" nil (b ih1 r n))
("byrne" nil (b er1 n))
("byrnes" nil (b er1 n z))
("byrns" nil (b ih1 r n z))
("byrom" nil (b ay1 r ax m))
("byron" nil (b ay1 r ax n))
("byrum" nil (b ih1 r ax m))
("bystander" nil (b ay1 s t ae1 n d er0))
("bystanders" nil (b ay1 s t ae1 n d er0 z))
("bystrom" nil (b ih1 s t r ax m))
("byte" nil (b ay1 t))
("bytes" nil (b ay1 t s))
("byun" nil (b y ah1 n))
("byus" nil (b ay1 ax s))
("bywater" nil (b ay1 w ao1 t er0))
("byways" nil (b ay1 w ey1 z))
("byword" nil (b ay1 w er1 d))
("byzantine" nil (b ih1 z ax n t ay1 n))
("byzantium" nil (b ax z ae1 n t iy0 ax m))
("c" nil (s iy1))
("cab" nil (k ae1 b))
("caba" nil (k aa1 b ax))
("cabal" nil (k ax b aa1 l))
("caballero" nil (k ae1 b ax y eh1 r ow0))
("caban" nil (k ey1 b ax n))
("cabana" nil (k ax b ae1 n ax))
("cabanas" nil (k ax b ae1 n ax z))
("cabaniss" nil (k ae1 b ax n ih0 s))
("cabaret" nil (k ae1 b er0 ey1))
("cabbage" nil (k ae1 b ax jh))
("cabbages" nil (k ae1 b ih0 jh ih0 z))
("cabbies" nil (k ae1 b iy0 z))
("cabby" nil (k ae1 b iy0))
("cabdriver" nil (k ae1 b d r ay1 v er0))
("cabdrivers" nil (k ae1 b d r ay1 v er0 z))
("cabe" nil (k ey1 b))
("cabell" nil (k aa0 b eh1 l))
("cabello" nil (k ax b eh1 l ow0))
("cabernet" nil (k ae1 b er0 n ey1))
("cabernets" nil (k ae1 b er0 n eh1 t s))
("cabezas" nil (k ax b ey1 z ax z))
("cabin" nil (k ae1 b ax n))
("cabinda" nil (k ax b ih1 n d ax))
("cabiness" nil (k ey1 b iy0 n ih0 s))
("cabinet" nil (k ae1 b ax n ax t))
("cabinetry" nil (k ae1 b n ax t r iy0))
("cabinets" nil (k ae1 b ax n ax t s))
("cabins" nil (k ae1 b ax n z))
("cable" nil (k ey1 b ax l))
("cablec" nil (k ae1 b l ax k))
("cablecomms" nil (k ey1 b ax l k aa1 m z))
("cabled" nil (k ey1 b ax l d))
("cablegram" nil (k ey1 b ax l g r ae1 m))
("cableone" nil (k ey1 b ax l w ah1 n))
("cabler" nil (k ey1 b ax l er0))
("cables" nil (k ey1 b ax l z))
("cablesystems" nil (k ey1 b ax l s ih1 s t ax m z))
("cabletron" nil (k ey1 b ax l t r ao1 n))
("cablevision" nil (k ey1 b ax l v ih1 zh ax n))
("caboodle" nil (k ax b uw1 d ax l))
("caboose" nil (k ax b uw1 s))
("cabooses" nil (k ax b uw1 s ih0 z))
("cabot" nil (k ae1 b ax t))
("cabotage" nil (k ae1 b ax t ih0 jh))
("cabral" nil (k ae1 b r ax l))
("cabrales" nil (k aa0 b r aa1 l eh0 s))
("cabrall" nil (k ax b r aa1 l))
("cabranes" nil (k ax b r ey1 n z))
("cabrera" nil (k aa0 b r eh1 r ax))
("cabrini" nil (k ax b r iy1 n iy0))
("cabriolet" nil (k ae1 b r iy0 ow0 l ey1))
("cabs" nil (k ae1 b z))
("cac" nil (k ae1 k))
("cacace" nil (k ae1 k ax s))
("cacaci" nil (k ax k aa1 s iy0))
("cacao" nil (k ax k ey1 ow0))
("caccamise" nil (k aa0 k aa1 m ih0 s))
("caccamo" nil (k aa0 k aa1 m ow0))
("caccavale" nil (k aa0 k ax v aa1 l iy0))
("caccia" nil (k aa1 ch ax))
("cacciatore" nil (k aa0 ch ax t ao1 r iy0))
("cacciola" nil (k aa0 k ch ow1 l ax))
("caceres" nil (k aa0 s eh1 r eh0 s))
("cache" nil (k ae1 sh))
("caches" nil (k ae1 sh ih0 z))
("cachet" nil (k ae1 sh ey0))
("cacho" nil (k ae1 ch ow0))
("cacioppo" nil (k aa0 ch ow1 p ow0))
("cacique" nil (k ax s iy1 k))
("cackle" nil (k ae1 k ax l))
("cackling" nil (k ae1 k ax l ih0 ng))
("cacophony" nil (k ae0 k aa1 f ax n iy0))
("cacti" nil (k ae1 k t ay0))
("cactus" nil (k ae1 k t ax s))
("cad" nil (k ae1 d))
("cada" nil (k aa1 d ax))
("cadam" nil (k ae1 d ax m))
("cadaver" nil (k ax d ae1 v er0))
("cadavers" nil (k ax d ae1 v er0 z))
("cadbury" nil (k ae1 d b eh1 r iy0))
("cadby" nil (k ae1 d b iy0))
("caddell" nil (k ae1 d ax l))
("cadden" nil (k ae1 d ax n))
("caddick" nil (k ae1 d ih0 k))
("caddock" nil (k ae1 d ax k))
("caddy" nil (k ae1 d iy0))
("caddyshack" nil (k ae1 d ih0 sh ae1 k))
("cade" nil (k ey1 d))
("cadell" nil (k aa0 d ey1 l))
("cadena" nil (k ax d iy1 n ax))
("cadence" nil (k ey1 d ax n s))
("cadences" nil (k ey1 d ax n s ih0 z))
("cadenhead" nil (k ey1 d ax n hh eh1 d))
("cadet" nil (k ax d eh1 t))
("cadets" nil (k ax d eh1 t s))
("cadieux" nil (k ae1 d iy0 ow0))
("cadillac" nil (k ae1 d ax l ae1 k))
("cadillacs" nil (k ae1 d ax l ae1 k s))
("cadiz" nil (k ax d iy1 z))
("cadle" nil (k ey1 d ax l))
("cadman" nil (k ae1 d m ax n))
("cadmium" nil (k ae1 d m iy0 ax m))
("cadmus" nil (k ae1 d m ax s))
("cadnetix" nil (k ae1 d n eh1 t ih0 k s))
("cadogan" nil (k aa0 d ow0 g aa1 n))
("cadorette" nil (k ae1 d er0 eh0 t))
("cadotte" nil (k ax d ao1 t))
("cadre" nil (k ae1 d r iy0))
("cadres" nil (k ae1 d r iy0 z))
("cadwalader" nil (k ae1 d w ao1 l ax d er0))
("cadwallader" nil (k ae1 d w ao1 l ax d er0))
("cadwell" nil (k ae1 d w eh1 l))
("cady" nil (k ey1 d iy0))
("caen" nil (k ae1 n))
("caesar" nil (s iy1 z er0))
("caesarea" nil (k ey1 s er0 iy1 ax))
("caesarean" nil (k ey1 s er0 iy1 n))
("caesareans" nil (k ey1 s er0 iy1 n z))
("caesars" nil (s iy1 z er0 z))
("caetano" nil (k ax t aa1 n ow0))
("cafarella" nil (k aa0 f aa0 r eh1 l ax))
("cafarelli" nil (k aa0 f aa0 r eh1 l iy0))
("cafaro" nil (k aa0 f aa1 r ow0))
("cafe" nil (k ax f ey1))
("cafes" nil (k ae1 f ey1 z))
("cafeteria" nil (k ae1 f ax t ih1 r iy0 ax))
("cafeterias" nil (k ae1 f ax t ih1 r iy0 ax z))
("cafetizer" nil (k ae1 f ih0 t ay1 z er0))
("caffee" nil (k ae1 f iy0))
("caffeine" nil (k ae0 f iy1 n))
("cafferty" nil (k ae1 f er0 t iy0))
("caffery" nil (k ae1 f er0 iy0))
("caffey" nil (k ae1 f iy0))
("caffrey" nil (k ae1 f r iy0))
("cafiero" nil (k ae1 f iy0 eh1 r ow0))
("cagan" nil (k ey1 g ax n))
("cage" nil (k ey1 jh))
("caged" nil (k ey1 jh d))
("cages" nil (k ey1 jh ih0 z))
("cagey" nil (k ey1 jh iy0))
("caggiano" nil (k aa0 jh iy0 aa1 n ow0))
("cagle" nil (k ey1 g ax l))
("cagley" nil (k ae1 g l iy0))
("cagliari" nil (k ae1 g l iy0 aa1 r iy0))
("cagney" nil (k ae1 g n iy0))
("caguas" nil (k aa1 g w aa1 s))
("cahalan" nil (k ae1 hh ax l ax n))
("cahall" nil (k ae1 hh ax l))
("cahan" nil (k ae1 hh ax n))
("cahasa" nil (k ax hh aa1 s ax))
("cahill" nil (k ey1 hh ih1 l))
("cahn" nil (k ae1 n))
("cahners" nil (k aa1 n er0 z))
("cahoon" nil (k ax hh uw1 n))
("cahoots" nil (k ax hh uw1 t s))
("cahora" nil (k ax hh ao1 r ax))
("cahouet" nil (k ax hh uw1 t))
("cai" nil (k ay1))
("caiazzo" nil (k aa0 y aa1 z ow0))
("caicedo" nil (k ax ch ey1 d ow0))
("cail" nil (k ey1 l))
("caillier" nil (k ey1 l iy0 er0))
("caillouet" nil (k ey1 l uw1 t))
("cain" nil (k ey1 n))
("caine" nil (k ey1 n))
("caines" nil (k ey1 n z))
("caiola" nil (k ay1 ow0 l ax))
("caire" nil (k eh1 r))
("cairnes" nil (k eh1 r n z))
("cairns" nil (k eh1 r n z))
("cairo" nil (k ay1 r ow0))
("caison" nil (k ey1 z ax n))
("caisse" nil (k ey1 s))
("caissie" nil (k ey1 s iy0))
("caito" nil (k ey1 t ow0))
("cajole" nil (k ax jh ow1 l))
("cajoled" nil (k ax jh ow1 l d))
("cajoling" nil (k ax jh ow1 l ih0 ng))
("cajun" nil (k ey1 jh ax n))
("cajuns" nil (k ey1 jh ax n z))
("cake" nil (k ey1 k))
("cakes" nil (k ey1 k s))
("cal" nil (k ae1 l))
("cala" nil (k aa1 l ax))
("calabasas" nil (k ae1 l ax b aa1 s ax s))
("calabrese" nil (k aa0 l aa0 b r ey1 z iy0))
("calabria" nil (k ax l ae1 b r iy0 ax))
("calabro" nil (k ax l ae1 b r ow0))
("caladiums" nil (k ax l ey1 d iy0 ax m z))
("calaf" nil (k ae1 l ax f))
("calahan" nil (k ae1 l ax hh ae0 n))
("calais" nil (k ax l ey1))
("calamari" nil (k aa0 l aa0 m aa1 r iy0))
("calame" nil (k aa0 l aa1 m iy0))
("calamia" nil (k ax l ey1 m iy0 ax))
("calamine" nil (k ae1 l ax m ay1 n))
("calamities" nil (k ax l ae1 m ax t iy0 z))
("calamitous" nil (k ax l ae1 m ax t ax s))
("calamity" nil (k ax l ae1 m ax t iy0))
("calan" nil (k ey1 l ax n))
("calandra" nil (k ax l ae1 n d r ax))
("calandro" nil (k ax l ae1 n d r ow0))
("calantha" nil (k ax l ae1 n th ax))
("calarco" nil (k ax l aa1 r k ow0))
("calaway" nil (k aa1 l ax w ey0))
("calbert" nil (k ae1 l b er0 t))
("calcagni" nil (k aa0 l k aa1 g n iy0))
("calcagno" nil (k aa0 l k aa1 g n ow0))
("calcaterra" nil (k ae1 l k ax t eh1 r ax))
("calcified" nil (k ae1 l s ax f ay1 d))
("calcite" nil (k ae1 l s ay1 t))
("calcium" nil (k ae1 l s iy0 ax m))
("calcomp" nil (k ao1 l k aa1 m p))
("calcote" nil (k aa0 l k ow1 t iy0))
("calculate" nil (k ae1 l k y ax l ey1 t))
("calculated" nil (k ae1 l k y ax l ey1 t ax d))
("calculates" nil (k ae1 l k y ax l ey1 t s))
("calculating" nil (k ae1 l k y ax l ey1 t ih0 ng))
("calculation" nil (k ae1 l k y ax l ey1 sh ax n))
("calculations" nil (k ae1 l k y ax l ey1 sh ax n z))
("calculator" nil (k ae1 l k y ax l ey1 t er0))
("calculators" nil (k ae1 l k y ax l ey1 t er0 z))
("calculus" nil (k ae1 l k y ax l ax s))
("calcutt" nil (k ae1 l k ax t))
("calcutta" nil (k ae1 l k ah1 t ax))
("caldeira" nil (k aa0 l d eh1 r ax))
("calder" nil (k ao1 l d er0))
("caldera" nil (k ae1 l d eh1 r ax))
("calderaro" nil (k aa0 l d er0 aa1 r ow0))
("calderon" nil (k ao1 l d er0 aa0 n))
("calderone" nil (k aa0 l d er0 ow1 n iy0))
("calderwood" nil (k ao1 l d er0 w uh0 d))
("caldor" nil (k ae1 l d ao0 r))
("caldrello" nil (k aa1 l d r eh1 l ow0))
("caldwell" nil (k ao1 l d w eh0 l))
("cale" nil (k ey1 l))
("caleb" nil (k ey1 l ax b))
("caleca" nil (k aa0 l eh1 k ax))
("caledonia" nil (k ae1 l ax d ow1 n iy0 ax))
("caledonian" nil (k ae1 l ih0 d ow1 n iy0 ax n))
("calef" nil (k ae1 l ih0 f))
("calendar" nil (k ae1 l ax n d er0))
("calendars" nil (k ae1 l ax n d er0 z))
("caler" nil (k ey1 l er0))
("calero" nil (k ax l eh1 r ow0))
("cales" nil (k ey1 l z))
("caley" nil (k ey1 l iy0))
("calf" nil (k ae1 f))
("calfed" nil (k ae1 f t))
("calfee" nil (k ae1 l f iy0))
("calgary" nil (k ae1 l g er0 iy0))
("calgene" nil (k ae1 l jh iy1 n))
("calgon" nil (k ae1 l g ax n))
("calgroup" nil (k ae1 l g r uw1 p))
("calhoon" nil (k ae0 l hh uw1 n))
("calhoun" nil (k ae0 l hh uw1 n))
("cali" nil (k aa1 l iy0))
("calia" nil (k aa1 l iy0 ax))
("caliber" nil (k ae1 l ax b er0))
("calibrate" nil (k ae1 l ax b r ey1 t))
("calibrated" nil (k ae1 l ax b r ey1 t ax d))
("calibration" nil (k ae1 l ax b r ey1 sh ax n))
("calico" nil (k ae1 l ax k ow1))
("calida" nil (k aa0 l iy1 d ax))
("caliendo" nil (k aa0 l iy0 eh1 n d ow0))
("calif" nil (k ae1 l ax f ao1 r n y ax))
("califano" nil (k aa0 l iy0 f aa1 n ow0))
("califf" nil (k ae1 l ih0 f))
("california" nil (k ae1 l ax f ao1 r n y ax))
("californian" nil (k ae1 l ih0 f ao1 r n y ax n))
("californians" nil (k ae1 l ih0 f ao1 r n y ax n z))
("caligiuri" nil (k ae1 l ax jh ih1 r iy0))
("calipers" nil (k ae1 l ax p er0 z))
("caliph" nil (k ae1 l ax f))
("caliri" nil (k aa0 l ih1 r iy0))
("calise" nil (k aa1 l ay0 z))
("calista" nil (k ax l ih1 s t ax))
("calisthenics" nil (k ae1 l ax s th eh1 n ih0 k s))
("calite" nil (k ae1 l ay1 t))
("calix" nil (k ae1 l ih0 k s))
("calk" nil (k ao1 k))
("calkin" nil (k ae1 l k ih0 n))
("calkins" nil (k ae1 l k ih0 n z))
("call" nil (k ao1 l))
("calla" nil (k ae1 l ax))
("callable" nil (k ao1 l ax b ax l))
("callaghan" nil (k ae1 l ax hh ae0 n))
("callaham" nil (k ae1 l ax hh ae1 m))
("callahan" nil (k ae1 l ax hh ae1 n))
("callais" nil (k ax l ey1))
("callan" nil (k ae1 l ax n))
("callanan" nil (k ae1 l ax n ax n))
("calland" nil (k ae1 l ax n d))
("callander" nil (k ae1 l ax n d er0))
("callari" nil (k aa0 l aa1 r iy0))
("callas" nil (k ae1 l ax z))
("callaway" nil (k ae1 l ax w ey0))
("calle" nil (k ey1 l))
("callebs" nil (k ey1 l eh1 b z))
("called" nil (k ao1 l d))
("callegari" nil (k aa0 l eh0 g aa1 r iy0))
("calleja" nil (k aa0 l ey1 y ax))
("callejas" nil (k aa0 l ey1 y aa0 z))
("callen" nil (k ae1 l ax n))
("callender" nil (k ax l eh1 n d er0))
("callens" nil (k ao1 l ax n z))
("caller" nil (k ao1 l er0))
("callers" nil (k ao1 l er0 z))
("callery" nil (k ae1 l er0 iy0))
("calles" nil (k ey1 l z))
("calley" nil (k ae1 l iy0))
("callicoat" nil (k ae1 l ih0 k ow1 t))
("callicott" nil (k ae1 l ih0 k aa0 t))
("callicutt" nil (k ae1 l ih0 k ax t))
("callie" nil (k ao1 l iy0))
("callier" nil (k ae1 l iy0 er0))
("callies" nil (k ae1 l iy0 z))
("calligan" nil (k ae1 l ih0 g ax n))
("calligraphic" nil (k ae1 l ax g r ae1 f ih0 k))
("calligraphy" nil (k ax l ih1 g r ax f iy0))
("callihan" nil (k ae1 l ih0 hh ae0 n))
("callinan" nil (k ae1 l ih0 n ax n))
("calling" nil (k ao1 l ih0 ng))
("calliope" nil (k ax l ay1 ax p iy1))
("calliopes" nil (k ax l ay1 ax p iy1 z))
("callis" nil (k ae1 l ih0 s))
("callison" nil (k ae1 l ih0 s ax n))
("callister" nil (k ao1 l ih0 s t er0))
("callous" nil (k ae1 l ax s))
("calloused" nil (k ae1 l ax s t))
("callously" nil (k ae1 l ax s l iy0))
("callousness" nil (k ae1 l ax s n ax s))
("callow" nil (k ae1 l ow0))
("calloway" nil (k ae1 l ow0 w ey1))
("calls" nil (k ao1 l z))
("callula" nil (k ae1 l uw0 l ax))
("callum" nil (k ae1 l ax m))
("callus" nil (k ae1 l ax s))
("cally" nil (k ae1 l iy0))
("calm" nil (k aa1 m))
("calma" nil (k aa1 l m ax))
("calmaquip" nil (k ae1 l m ax k w ih1 p))
("calmar" nil (k ae1 l m aa0 r))
("calmark" nil (k aa1 l m aa1 r k))
("calmart" nil (k aa1 l m aa1 r t))
("calmat" nil (k ae1 l m ae0 t))
("calmed" nil (k aa1 m d))
("calmer" nil (k aa1 m er0))
("calmes" nil (k aa1 l m eh0 s))
("calming" nil (k aa1 m ih0 ng))
("calmly" nil (k aa1 m l iy0))
("calmness" nil (k aa1 m n ax s))
("calms" nil (k aa1 m z))
("calnan" nil (k ae1 l n ax n))
("calny" nil (k ae1 l n iy0))
("calo" nil (k aa1 l ow0))
("calogero" nil (k aa0 l ow0 jh eh1 r ow0))
("calor" nil (k ae1 l er0))
("caloric" nil (k ax l ao1 r ih0 k))
("calorie" nil (k ae1 l er0 iy0))
("calories" nil (k ae1 l er0 iy0 z))
("caloway" nil (k ae1 l ow0 w ey1))
("calpers" nil (k ae1 l p er0 z))
("calpis" nil (k ae1 l p ih0 s))
("caltabiano" nil (k aa0 l t aa0 b iy0 aa1 n ow0))
("caltagirone" nil (k aa0 l t aa0 jh ih0 r ow1 n iy0))
("caltex" nil (k ah1 l t eh1 k s))
("caltha" nil (k ae1 l dh ax))
("calton" nil (k ae1 l t ax n))
("caltrans" nil (k ae1 l t r ae1 n z))
("calumet" nil (k ae1 l y ax m eh1 t))
("calvani" nil (k ao1 l v aa1 n iy0))
("calvano" nil (k aa0 l v aa1 n ow0))
("calvaries" nil (k ae1 l v er0 iy0 z))
("calvary" nil (k ae1 l v er0 iy0))
("calverley" nil (k ae1 l v er0 l iy0))
("calvert" nil (k ae1 l v er0 t))
("calvery" nil (k ae1 l v er0 iy0))
("calves" nil (k ae1 v z))
("calvet" nil (k ae1 l v ax t))
("calvey" nil (k ae0 l v ey1))
("calvi" nil (k aa1 l v iy0))
("calvillo" nil (k aa0 l v ih1 l ow0))
("calvin" nil (k ae1 l v ax n))
("calvina" nil (k aa0 l v iy1 n ax))
("calving" nil (k ae1 v ih0 ng))
("calvinist" nil (k ae1 l v ax n ax s t))
("calvino" nil (k ao1 l v iy1 n ow0))
("calvo" nil (k aa1 l v ow0))
("calypso" nil (k ax l ih1 p s ow1))
("calypsos" nil (k ax l ih1 p s ow1 z))
("calzada" nil (k aa0 l z aa1 d ax))
("cam" nil (k ae1 m))
("camacho" nil (k ax m aa1 ch ow0))
("camara" nil (k aa0 m aa1 r ax))
("camaraderie" nil (k aa1 m er0 aa1 d er0 iy0))
("camarata" nil (k aa0 m aa0 r aa1 t ax))
("camarena" nil (k aa0 m aa0 r eh1 n ax))
("camargo" nil (k aa0 m aa1 r g ow0))
("camarillo" nil (k aa0 m aa0 r ih1 l ow0))
("camaro" nil (k ax m eh1 r ow0))
("camaros" nil (k ax m aa1 r ow0 s))
("cambell" nil (k ax m b eh1 l))
("camber" nil (k ae1 m b er0))
("cambex" nil (k ae1 m b ax k s))
("cambior" nil (k ae1 m b iy0 er0))
("cambodia" nil (k ae1 m b ow1 d iy0 ax))
("cambodian" nil (k ae1 m b ow1 d iy0 ax n))
("cambodians" nil (k ae1 m b ow1 d iy0 ax n z))
("cambra" nil (k ae1 m b r ax))
("cambre" nil (k ae1 m b er0))
("cambria" nil (k ae1 m b r iy0 ax))
("cambrian" nil (k ae1 m b r iy0 ax n))
("cambridge" nil (k ey1 m b r ih0 jh))
("cambridgeport" nil (k ey1 m b r ih1 jh p ao1 r t))
("cambridgeside" nil (k ey1 m b r ih1 jh s ay1 d))
("cambron" nil (k ae1 m b r ax n))
("camburn" nil (k ae1 m b er1 n))
("camco" nil (k ae1 m k ow0))
("camcorder" nil (k ae1 m k ao1 r d er0))
("camcorders" nil (k ae1 m k ao1 r d er0 z))
("camden" nil (k ae1 m d ax n))
("camdessus" nil (k ae0 m d eh1 s ax s))
("came" nil (k ey1 m))
("cameco" nil (k ae1 m ih0 k ow0))
("camejo" nil (k aa0 m ey1 y ow0))
("camel" nil (k ae1 m ax l))
("camelot" nil (k ae1 m ax l aa1 t))
("camels" nil (k ae1 m ax l z))
("cameo" nil (k ae1 m iy0 ow1))
("cameos" nil (k ae1 m iy0 ow1 z))
("camera" nil (k ae1 m er0 ax))
("cameraman" nil (k ae1 m er0 ax m ax n))
("cameramen" nil (k ae1 m er0 ax m eh0 n))
("cameras" nil (k ae1 m er0 ax z))
("camerer" nil (k ae1 m er0 er0))
("camero" nil (k ax m eh1 r ow0))
("cameron" nil (k ae1 m er0 ax n))
("cameroon" nil (k ae1 m er0 uw1 n))
("camfield" nil (k ae1 m f iy1 l d))
("camhi" nil (k ae1 m hh iy0))
("camilla" nil (k ax m ih1 l ax))
("camille" nil (k ax m iy1 l))
("camilleri" nil (k aa0 m iy0 l eh1 r iy0))
("camilli" nil (k ax m ih1 l iy0))
("camillo" nil (k ax m ih1 l ow0))
("caminiti" nil (k aa0 m iy0 n iy1 t iy0))
("camino" nil (k ax m iy1 n ow0))
("camire" nil (k aa0 m ih1 r iy0))
("camm" nil (k ae1 m))
("cammack" nil (k ae1 m ax k))
("cammarano" nil (k aa0 m aa0 r aa1 n ow0))
("cammarata" nil (k aa0 m aa0 r aa1 t ax))
("cammarota" nil (k aa0 m aa0 r ow1 t ax))
("cammer" nil (k ae1 m er0))
("cammie" nil (k ae1 m iy0))
("cammisa" nil (k ax m iy1 s ax))
("cammon" nil (k ae1 m ax n))
("cammy" nil (k ae1 m iy0))
("camorra" nil (k ax m ao1 r ax))
("camouflage" nil (k ae1 m ax f l aa1 zh))
("camouflaged" nil (k ae1 m ax f l aa1 zh d))
("camouflaging" nil (k ae1 m ax f l aa1 zh ih0 ng))
("camp" nil (k ae1 m p))
("campa" nil (k ae1 m p ax))
("campaign" nil (k ae0 m p ey1 n))
("campaigned" nil (k ae0 m p ey1 n d))
("campaigner" nil (k ae0 m p ey1 n er0))
("campaigners" nil (k ae0 m p ey1 n er0 z))
("campaigning" nil (k ae0 m p ey1 n ih0 ng))
("campaigns" nil (k ae0 m p ey1 n z))
("campana" nil (k aa0 m p ae1 n ax))
("campanale" nil (k aa0 m p aa0 n aa1 l iy0))
("campanaro" nil (k aa0 m p aa0 n aa1 r ow0))
("campanella" nil (k ae1 m p ax n eh1 l ax))
("campanelli" nil (k aa0 m p aa0 n eh1 l iy0))
("campanis" nil (k ae1 m p aa1 n ih0 s))
("campau" nil (k aa1 m p aw0))
("campbell" nil (k ae1 m b ax l))
("campbells" nil (k ae1 m b ax l z))
("campeau" nil (k ae0 m p ow1))
("camped" nil (k ae1 m p t))
("campen" nil (k ae1 m p ax n))
("camper" nil (k ae1 m p er0))
("campers" nil (k ae1 m p er0 z))
("campesinos" nil (k ae1 m p eh0 s iy1 n ow0 s))
("campfield" nil (k ae1 m p f iy1 l d))
("campfire" nil (k ae1 m p f ay1 er0))
("campfires" nil (k ae1 m p f ay1 er0 z))
("campground" nil (k ae1 m p g r aw1 n d))
("campgrounds" nil (k ae1 m p g r aw1 n d z))
("campi" nil (k ae1 m p iy0))
("camping" nil (k ae1 m p ih0 ng))
("campion" nil (k ae1 m p iy0 ax n))
("campione" nil (k aa0 m p iy0 ow1 n iy0))
("campise" nil (k aa1 m p ay0 z))
("campisi" nil (k aa0 m p iy1 s iy0))
("campo" nil (k ae1 m p ow0))
("campobasso" nil (k aa0 m p ow0 b aa1 s ow0))
("campofrio" nil (k ae1 m p ao1 f r iy0 ow0))
("campoli" nil (k aa0 m p ow1 l iy0))
("campopiano" nil (k aa0 m p ow0 p iy0 aa1 n ow0))
("campos" nil (k ae1 m p ow0 z))
("camps" nil (k ae1 m p s))
("campsite" nil (k ae1 m p s ay1 t))
("campsites" nil (k ae1 m p s ay1 t s))
("campton" nil (k ae1 m p t ax n))
("campus" nil (k ae1 m p ax s))
("campuses" nil (k ae1 m p ax s ax z))
("campuzano" nil (k aa0 m p uw0 z aa1 n ow0))
("campy" nil (k ae1 m p iy0))
("camro" nil (k ae1 m r ow0))
("camry" nil (k ae1 m r iy0))
("camrys" nil (k ae1 m r iy0 z))
("camshaft" nil (k ae1 m sh ae1 f t))
("camshafts" nil (k ae1 m sh ae1 f t s))
("camus" nil (k ae1 m ih0 s))
("camuso" nil (k aa0 m uw1 s ow0))
("can" nil (k ae1 n))
("canaan" nil (k ey1 n ax n))
("canaanite" nil (k ey1 n ax n ay1 t))
("canace" nil (k aa0 n aa1 ch iy0))
("canada" nil (k ae1 n ax d ax))
("canadair" nil (k ae1 n ax d eh1 r))
("canaday" nil (k ae1 n ax d ey1))
("canadian" nil (k ax n ey1 d iy0 ax n))
("canadians" nil (k ax n ey1 d iy0 ax n z))
("canadienne" nil (k ax n ae1 d iy0 eh1 n))
("canady" nil (k ax n aa1 d iy0))
("canal" nil (k ax n ae1 l))
("canale" nil (k aa0 n aa1 l iy0))
("canales" nil (k ae1 n ax l z))
("canals" nil (k ax n ae1 l z))
("canam" nil (k ae1 n ax m))
("canan" nil (k ey1 n ax n))
("canandaigua" nil (k ae1 n ax n d ey1 g w ax))
("cananea" nil (k ae1 n ax n iy1 ax))
("canard" nil (k ax n aa1 r d))
("canaries" nil (k ax n eh1 r iy0 z))
("canary" nil (k ax n eh1 r iy0))
("canas" nil (k ae1 n ax z))
("canasta" nil (k ax n ae1 s t ax))
("canavan" nil (k ae1 n ax v ae1 n))
("canaveral" nil (k ax n ae1 v er0 ax l))
("canberra" nil (k ae1 n b eh1 r ax))
("canby" nil (k ae1 n b iy0))
("cancan" nil (k ae1 n k ae1 n))
("cancel" nil (k ae1 n s ax l))
("canceled" nil (k ae1 n s ax l d))
("canceling" nil (k ae1 n s ax l ih0 ng))
("cancellation" nil (k ae1 n s ax l ey1 sh ax n))
("cancellations" nil (k ae1 n s ax l ey1 sh ax n z))
("cancelled" nil (k ae1 n s ax l d))
("cancelling" nil (k ae1 n s ax l ih0 ng))
("cancels" nil (k ae1 n s ax l z))
("cancer" nil (k ae1 n s er0))
("cancerous" nil (k ae1 n s er0 ax s))
("cancerphobia" nil (k ae1 n s er0 f ow1 b iy0 ax))
("cancers" nil (k ae1 n s er0 z))
("canchola" nil (k aa0 n k ow1 l ax))
("cancienne" nil (k aa0 n ch iy1 eh0 n))
("cancilla" nil (k aa0 n ch ih1 l ax))
("cancino" nil (k aa0 n ch iy1 n ow0))
("cancio" nil (k ae1 n s iy0 ow0))
("cancom" nil (k ae1 ng k ax m))
("cancro" nil (k aa1 n k r ow0))
("cancun" nil (k ae1 ng k ax n))
("candace" nil (k ae1 n d ax s))
("candee" nil (k ae1 n d iy1))
("candela" nil (k ae1 n d eh1 l ax))
("candelabra" nil (k ae1 n d ax l aa1 b r ax))
("candelaria" nil (k aa0 n d eh0 l aa1 r iy0 ax))
("candelario" nil (k aa0 n d eh0 l aa1 r iy0 ow0))
("candella" nil (k aa0 n d eh1 l ax))
("candia" nil (k ae1 n d iy0 ax))
("candice" nil (k ae1 n d ih0 s))
("candid" nil (k ae1 n d ax d))
("candida" nil (k ae1 n d ih0 d ax))
("candidacies" nil (k ae1 n d ax d ax s iy0 z))
("candidacy" nil (k ae1 n d ih0 d ax s iy0))
("candidate" nil (k ae1 n d ax d ey0 t))
("candidates" nil (k ae1 n d ax d ey0 t s))
("candidly" nil (k ae1 n d ih0 d l iy0))
("candido" nil (k ae0 n d iy1 d ow0))
("candie" nil (k ae1 n d iy0))
("candied" nil (k ae1 n d iy0 d))
("candies" nil (k ae1 n d iy0 z))
("candiotti" nil (k ae1 n d iy0 aa1 t iy0))
("candle" nil (k ae1 n d ax l))
("candlelight" nil (k ae1 n d ax l l ay1 t))
("candlemaker" nil (k ae1 n d ax l m ey1 k er0))
("candler" nil (k ae1 n d ax l er0))
("candles" nil (k ae1 n d ax l z))
("candlestick" nil (k ae1 n d ax l s t ih1 k))
("candlesticks" nil (k ae1 n d ax l s t ih1 k s))
("candlish" nil (k ae1 n d l ih0 sh))
("candor" nil (k ae1 n d er0))
("candy" nil (k ae1 n d iy0))
("cane" nil (k ey1 n))
("caned" nil (k ey1 n d))
("canedo" nil (k aa0 n ey1 d ow0))
("canedy" nil (k ax n iy1 d iy0))
("canelo" nil (k ax n eh1 l ow0))
("canepa" nil (k aa0 n eh1 p ax))
("canes" nil (k ey1 n z))
("canevari" nil (k aa0 n eh0 v aa1 r iy0))
("canez" nil (k aa0 n eh1 z))
("canfield" nil (k ae1 n f iy1 l d))
("canfor" nil (k ae1 n f er0))
("cangelosi" nil (k aa0 ng g eh0 l ow1 s iy0))
("cangemi" nil (k aa0 ng g eh1 m iy0))
("cangialosi" nil (k aa0 n jh ax l ow1 s iy0))
("cangiano" nil (k aa0 ng g iy0 aa1 n ow0))
("canham" nil (k ae1 n hh ax m))
("canida" nil (k aa0 n iy1 d ax))
("caniglia" nil (k ax n ih1 g l iy0 ax))
("canilles" nil (k ax n ih1 l iy0 z))
("canin" nil (k ey1 n ih0 n))
("canine" nil (k ey1 n ay1 n))
("canines" nil (k ey1 n ay1 n z))
("caning" nil (k ey1 n ih0 ng))
("canings" nil (k ey1 n ih0 ng z))
("canino" nil (k aa0 n iy1 n ow0))
("canion" nil (k ae1 n y ax n))
("canipe" nil (k aa0 n iy1 p iy0))
("canister" nil (k ae1 n ax s t er0))
("canisters" nil (k ae1 n ax s t er0 z))
("canker" nil (k ae1 ng k er0))
("cankers" nil (k ae1 ng k er0 z))
("cann" nil (k ae1 n))
("cannabis" nil (k ae1 n ax b ax s))
("cannaday" nil (k ae1 n ax d ey1))
("cannady" nil (k ae1 n ax d iy0))
("cannan" nil (k ae1 n ax n))
("cannata" nil (k aa0 n aa1 t ax))
("cannavino" nil (k ae1 n ax v iy1 n ow0))
("cannavo" nil (k aa0 n aa1 v ow0))
("canned" nil (k ae1 n d))
("cannedy" nil (k ae1 n ih0 d iy0))
("cannell" nil (k ae1 n ax l))
("cannella" nil (k aa0 n eh1 l ax))
("cannelton" nil (k ae1 n ax l t ax n))
("canner" nil (k ae1 n er0))
("canneries" nil (k ae1 n er0 iy0 z))
("cannery" nil (k ae1 n er0 iy0))
("cannes" nil (k ae1 n z))
("canney" nil (k ae1 n iy0))
("cannibal" nil (k ae1 n ax b ax l))
("cannibalism" nil (k ae1 n ax b ax l ih1 z ax m))
("cannibalization" nil (k ae1 n ax b ax l ih0 z ey1 sh ax n))
("cannibalize" nil (k ae1 n ax b ax l ay1 z))
("cannibalizing" nil (k ae1 n ax b ax l ay1 z ih0 ng))
("canniff" nil (k ae1 n ih0 f))
("canning" nil (k ae1 n ih0 ng))
("cannister" nil (k ae1 n ih0 s t er0))
("cannisters" nil (k ae1 n ih0 s t er0 z))
("cannistraro" nil (k ae1 n ih0 s t r aa1 r ow0))
("cannizzaro" nil (k aa0 n iy0 t s aa1 r ow0))
("cannizzo" nil (k aa0 n iy1 z ow0))
("cannon" nil (k ae1 n ax n))
("cannone" nil (k aa0 n ow1 n iy0))
("cannons" nil (k ae1 n ax n z))
("cannot" nil (k ae0 n aa1 t))
("canny" nil (k ae1 n iy0))
("cano" nil (k aa1 n ow0))
("canoe" nil (k ax n uw1))
("canoed" nil (k ax n uw1 d))
("canoeing" nil (k ax n uw1 ih0 ng))
("canoeist" nil (k ax n uw1 ax s t))
("canoes" nil (k ax n uw1 z))
("canoga" nil (k ax n ow1 g ax))
("canola" nil (k ax n ow1 l ax))
("canon" nil (k ae1 n ax n))
("canonico" nil (k aa0 n ow0 n iy1 k ow0))
("canonie" nil (k ae1 n ax n iy0))
("canons" nil (k ae1 n ax n z))
("canopy" nil (k ae1 n ax p iy0))
("canosa" nil (k ax n ow1 s ax))
("canova" nil (k aa0 n ow1 v ax))
("canoy" nil (k ae1 n oy0))
("canrad" nil (k ae1 n r ae0 d))
("canron" nil (k ae1 n r ax n))
("cans" nil (k ae1 n z))
("canseco" nil (k ae1 n s eh1 k ow0))
("cansler" nil (k ae1 n s ax l er0))
("canso" nil (k ae1 n s ow0))
("canstar" nil (k ae1 n s t aa1 r))
("cant" nil (k ae1 n t))
("cantaloupe" nil (k ae1 n t ax l ow1 p))
("cantaloupes" nil (k ae1 n t ax l ow1 p s))
("cantalupo" nil (k ae1 n t ax l uw1 p ow0))
("cantankerous" nil (k ae0 n t ae1 ng k er0 ax s))
("cantara" nil (k aa0 n t aa1 r ax))
("cantata" nil (k ae1 n t aa1 t ax))
("canteen" nil (k ae0 n t iy1 n))
("cantel" nil (k ae1 n t eh1 l))
("canter" nil (k ae1 n t er0))
("canterbury" nil (k ae1 n t er0 b eh1 r iy0))
("cantero" nil (k aa0 n t eh1 r ow0))
("canterra" nil (k aa1 n t eh1 r ax))
("cantey" nil (k ae1 n t iy0))
("cantin" nil (k aa0 n t iy1 n))
("cantina" nil (k ae1 n t iy1 n ax))
("cantle" nil (k ae1 n t ax l))
("cantley" nil (k ae1 n t l iy0))
("cantlin" nil (k ae1 n t l ih0 n))
("cantlon" nil (k ae1 n t l ax n))
("canto" nil (k ae1 n t ow0))
("canton" nil (k ae1 n t ax n))
("cantonal" nil (k ae1 n t ax n ax l))
("cantone" nil (k aa0 n t ow1 n iy0))
("cantonese" nil (k ae1 n t ax n iy1 z))
("cantons" nil (k ae1 n t ax n z))
("cantor" nil (k ae1 n t er0))
("cantore" nil (k ae1 n t ao1 r))
("cantrall" nil (k ae1 n t r ax l))
("cantrell" nil (k ae0 n t r eh1 l))
("cantrelle" nil (k ax n t r eh1 l))
("cantu" nil (k ae1 n t uw0))
("cantv" nil (k ae1 n t iy1 v iy1))
("cantwell" nil (k ae1 n t w eh1 l))
("canty" nil (k ae1 n t iy0))
("canup" nil (k ae1 n ah1 p))
("canupp" nil (k ae1 n ax p))
("canvas" nil (k ae1 n v ax s))
("canvases" nil (k ae1 n v ax s ih0 z))
("canvass" nil (k ae1 n v ax s))
("canvassed" nil (k ae1 n v ax s t))
("canvassers" nil (k ae1 n v ax s er0 z))
("canvassing" nil (k ae1 n v ax s ih0 ng))
("canwest" nil (k ae1 n w eh1 s t))
("canyon" nil (k ae1 n y ax n))
("canyons" nil (k ae1 n y ax n z))
("canzano" nil (k aa0 n z aa1 n ow0))
("canzoneri" nil (k aa0 n z ow0 n eh1 r iy0))
("cao" nil (k aw1))
("caouette" nil (k ey1 uw1 t))
("cap" nil (k ae1 p))
("capabilities" nil (k ey1 p ax b ih1 l ax t iy0 z))
("capability" nil (k ey1 p ax b ih1 l ax t iy0))
("capable" nil (k ey1 p ax b ax l))
("capacious" nil (k ax p ey1 sh ax s))
("capacitance" nil (k ax p ae1 s ax t ax n s))
("capacities" nil (k ax p ae1 s ax t iy0 z))
("capacitor" nil (k ax p ae1 s ax t er0))
("capacitors" nil (k ax p ae1 s ax t er0 z))
("capacity" nil (k ax p ae1 s ax t iy0))
("capalbo" nil (k ax p ae1 l b ow0))
("capaldi" nil (k aa0 p aa1 l d iy0))
("capaldo" nil (k aa0 p aa1 l d ow0))
("capano" nil (k aa0 p aa1 n ow0))
("capasso" nil (k aa0 p aa1 s ow0))
("capcom" nil (k ae1 p k ax m))
("cape" nil (k ey1 p))
("capece" nil (k ax p iy1 s))
("capehart" nil (k ey1 p hh aa1 r t))
("capek" nil (k ae1 p ih0 k))
("capel" nil (k ey1 p ax l))
("capell" nil (k aa0 p ey1 l))
("capella" nil (k ax p eh1 l ax))
("capelle" nil (k ax p eh1 l))
("capelli" nil (k ax p eh1 l iy0))
("capello" nil (k ax p eh1 l ow0))
("capen" nil (k ey1 p ax n))
("caper" nil (k ey1 p er0))
("capers" nil (k ey1 p er0 z))
("caperton" nil (k ey1 p er0 t ax n))
("capes" nil (k ey1 p s))
("capetillo" nil (k aa0 p eh0 t ih1 l ow0))
("capetown" nil (k ey1 p t aw1 n))
("capillaries" nil (k ae1 p ax l eh1 r iy0 z))
("capillary" nil (k ae1 p ax l eh1 r iy0))
("capistrano" nil (k ae1 p ih0 s t r aa1 n ow0))
("capita" nil (k ae1 p ih0 t ax))
("capital" nil (k ae1 p ax t ax l))
("capitalism" nil (k ae1 p ih0 t ax l ih1 z ax m))
("capitalist" nil (k ae1 p ax t ax l ax s t))
("capitalistic" nil (k ae1 p ih0 t ax l ih1 s t ih0 k))
("capitalists" nil (k ae1 p ax t ax l ax s t s))
("capitalization" nil (k ae1 p ih0 t ax l ih0 z ey1 sh ax n))
("capitalizations" nil (k ae1 p ih0 t ax l ih0 z ey1 sh ax n z))
("capitalize" nil (k ae1 p ax t ax l ay1 z))
("capitalized" nil (k ae1 p ih0 t ax l ay1 z d))
("capitalizes" nil (k ae1 p ax t ax l ay1 z ih0 z))
("capitalizing" nil (k ae1 p ih0 t ax l ay1 z ih0 ng))
("capitals" nil (k ae1 p ax t ax l z))
("capitan" nil (k ae1 p ih0 t ax n))
("capitano" nil (k aa0 p iy0 t aa1 n ow0))
("capito" nil (k aa0 p iy1 t ow0))
("capitol" nil (k ae1 p ih0 t ax l))
("capitoline" nil (k ae1 p ih0 t ow1 l ay1 n))
("capitulate" nil (k ax p ih1 ch uw0 l ih0 t))
("capitulated" nil (k ax p ih1 ch ax l ey1 t ih0 d))
("capitulation" nil (k ax p ih1 ch ax l ey1 sh ax n))
("capizzi" nil (k aa0 p iy1 t s iy0))
("caplan" nil (k ae1 p l ax n))
("caple" nil (k ey1 p ax l))
("caples" nil (k ey1 p ax l z))
("capley" nil (k ae1 p l iy0))
("caplin" nil (k ae1 p l ih0 n))
("caplinger" nil (k ey1 p ax l ih0 ng er0))
("capo" nil (k aa1 p ow0))
("capobianco" nil (k aa0 p ow0 b iy0 aa1 n k ow0))
("capon" nil (k ey1 p aa1 n))
("capone" nil (k ax p ow1 n))
("caponi" nil (k aa0 p ow1 n iy0))
("caponigro" nil (k aa0 p ow0 n iy1 g r ow0))
("caporale" nil (k aa0 p ao0 r aa1 l iy0))
("caporaso" nil (k aa0 p ao0 r aa1 s ow0))
("capote" nil (k ax p ow1 t))
("capoten" nil (k ax p ow1 t ax n))
("capozzi" nil (k ax p aa1 z iy0))
("capozzoli" nil (k aa0 p ow0 t s ow1 l iy0))
("capp" nil (k ae1 p))
("cappa" nil (k ae1 p ax))
("cappadona" nil (k aa0 p aa0 d ow1 n ax))
("capparelli" nil (k aa0 p aa0 r eh1 l iy0))
("capped" nil (k ae1 p t))
("cappel" nil (k ae1 p ax l))
("cappella" nil (k aa0 p eh1 l ax))
("cappelletti" nil (k aa0 p eh0 l eh1 t iy0))
("cappelli" nil (k aa0 p eh1 l iy0))
("cappello" nil (k aa0 p eh1 l ow0))
("capper" nil (k ae1 p er0))
("cappetta" nil (k aa0 p eh1 t ax))
("cappiello" nil (k aa0 p iy0 eh1 l ow0))
("capping" nil (k ae1 p ih0 ng))
("cappo" nil (k ae1 p ow0))
("capps" nil (k ae1 p s))
("cappuccino" nil (k ae1 p uw1 ch iy1 n ow0))
("cappuccio" nil (k aa0 p uw1 ch iy0 ow0))
("cappy" nil (k ae1 p iy0))
("capra" nil (k ae1 p r ax))
("capraro" nil (k aa0 p r aa1 r ow0))
("capri" nil (k ae1 p r iy0))
("capriati" nil (k ae1 p r iy0 aa1 t iy0))
("caprice" nil (k ax p r iy1 s))
("capricious" nil (k ax p r ih1 sh ax s))
("capriciously" nil (k ax p r ih1 sh ih0 s l iy0))
("capricorn" nil (k ae1 p r ax k ao0 r n))
("caprio" nil (k ae1 p r iy0 ow0))
("capriotti" nil (k aa0 p r iy0 ow1 t iy0))
("capron" nil (k ae1 p r ax n))
("caps" nil (k ae1 p s))
("capshaw" nil (k ae1 p sh ao1))
("capsize" nil (k ae1 p s ay1 z))
("capsized" nil (k ae1 p s ay1 z d))
("capstan" nil (k ae1 p s t ax n))
("capstick" nil (k ae1 p s t ih1 k))
("capsule" nil (k ae1 p s ax l))
("capsules" nil (k ae1 p s ax l z))
("captain" nil (k ae1 p t ax n))
("captains" nil (k ae1 p t ax n z))
("caption" nil (k ae1 p sh ax n))
("captioning" nil (k ae1 p sh ax n ih0 ng))
("captions" nil (k ae1 p sh ax n z))
("captivated" nil (k ae1 p t ih0 v ey1 t ih0 d))
("captivating" nil (k ae1 p t ih0 v ey1 t ih0 ng))
("captive" nil (k ae1 p t ih0 v))
("captives" nil (k ae1 p t ih0 v z))
("captivity" nil (k ae0 p t ih1 v ax t iy0))
("captors" nil (k ae1 p t er0 z))
("capture" nil (k ae1 p ch er0))
("captured" nil (k ae1 p ch er0 d))
("captures" nil (k ae1 p ch er0 z))
("capturing" nil (k ae1 p ch er0 ih0 ng))
("capua" nil (k ae1 p y uw0 ax))
("capuano" nil (k aa0 p uw0 aa1 n ow0))
("caputi" nil (k aa0 p uw1 t iy0))
("caputo" nil (k aa0 p uw1 t ow0))
("capwell" nil (k ae1 p w eh1 l))
("caq" nil (k ae1 k))
("car" nil (k aa1 r))
("cara" nil (k eh1 r ax))
("carabajal" nil (k er0 ae1 b ax jh ax l))
("caraballo" nil (k ae1 r ax b ae1 l ow0))
("carabello" nil (k ae1 r ax b eh1 l ow0))
("caracara" nil (k aa1 r ax k aa1 r ax))
("caracas" nil (k er0 aa1 k ax s))
("caracci" nil (k aa0 r aa1 ch iy0))
("caracciolo" nil (k aa0 r aa0 k ch ow1 l ow0))
("caradine" nil (k aa0 r aa0 d iy1 n iy0))
("caradonna" nil (k aa0 r aa0 d ow1 n ax))
("carafate" nil (k eh1 r ax f ey1 t))
("caraher" nil (k ae1 r ax hh er0))
("caraker" nil (k ae1 r ax k er0))
("caramel" nil (k eh1 r ax m ax l))
("carangelo" nil (k aa0 r aa0 ng g eh1 l ow0))
("carano" nil (k aa0 r aa1 n ow0))
("caras" nil (k aa1 r ax z))
("carat" nil (k eh1 r ax t))
("carats" nil (k eh1 r ax t s))
("caravan" nil (k ae1 r ax v ae1 n))
("caravans" nil (k ae1 r ax v ae1 n z))
("caravel" nil (k eh1 r ax v eh1 l))
("caravella" nil (k ae1 r ax v eh1 l ax))
("caravelle" nil (k eh1 r ax v eh1 l))
("caravello" nil (k ae1 r ax v eh1 l ow0))
("caraveo" nil (k aa0 r aa1 v iy0 ow0))
("carawan" nil (k ae1 r ax w ae0 n))
("caraway" nil (k ae1 r ax w ey1))
("carbajal" nil (k aa0 r b aa0 y ae1 l))
("carballo" nil (k aa0 r b aa1 l ow0))
("carbary" nil (k aa1 r b eh0 r iy0))
("carbaugh" nil (k aa1 r b ao1))
("carberry" nil (k aa1 r b eh1 r iy0))
("carbide" nil (k aa1 r b ay1 d))
("carbin" nil (k aa1 r b ih0 n))
("carbine" nil (k aa1 r b ay1 n))
("carbo" nil (k aa1 r b ow0))
("carbohydrate" nil (k aa1 r b ow0 hh ay1 d r ey1 t))
("carbohydrates" nil (k aa1 r b ow0 hh ay1 d r ey0 t s))
("carbon" nil (k aa1 r b ax n))
("carbonara" nil (k aa0 r b ow0 n aa1 r ax))
("carbonaro" nil (k aa0 r b ow0 n aa1 r ow0))
("carbonate" nil (k aa1 r b ax n ey1 t))
("carbonated" nil (k aa1 r b ax n ey1 t ax d))
("carbonates" nil (k aa1 r b ax n ey1 t s))
("carbondale" nil (k aa1 r b ax n d ey1 l))
("carbone" nil (k aa1 r b ow1 n))
("carbonetto" nil (k aa1 r b ow0 n eh1 t ow0))
("carboni" nil (k aa0 r b ow1 n iy0))
("carboniferous" nil (k aa1 r b ax n ih1 f er0 ax s))
("carbonized" nil (k aa1 r b ax n ay1 z d))
("carbonneau" nil (k aa1 r b ax n ow1))
("carbons" nil (k aa1 r b ax n z))
("carborundum" nil (k aa1 r b er0 ah1 n d ax m))
("carboy" nil (k aa1 r b oy1))
("carbureted" nil (k aa1 r b y er0 eh1 t ih0 d))
("carburetion" nil (k aa1 r b y er0 iy1 sh ax n))
("carburetor" nil (k aa1 r b er0 ey1 t er0))
("carburetors" nil (k aa1 r b er0 ey1 t er0 z))
("carby" nil (k aa1 r b iy0))
("carcass" nil (k aa1 r k ax s))
("carcasses" nil (k aa1 r k ax s ih0 z))
("carcinogen" nil (k aa0 r s ih1 n ax jh ax n))
("carcinogenic" nil (k aa1 r s ax n ow0 jh eh1 n ih0 k))
("carcinogens" nil (k aa0 r s ih1 n ax jh ax n z))
("carcinoma" nil (k aa1 r s ax n ow1 m ax))
("carcione" nil (k aa0 r ch ow1 n iy0))
("card" nil (k aa1 r d))
("carda" nil (k aa1 r d ax))
("cardarelli" nil (k aa0 r d aa0 r eh1 l iy0))
("cardboard" nil (k aa1 r d b ao1 r d))
("cardboards" nil (k aa1 r d b ao1 r d z))
("carded" nil (k aa1 r d ax d))
("cardell" nil (k aa0 r d ey1 l))
("cardella" nil (k aa1 r d eh1 l ax))
("cardello" nil (k aa1 r d eh1 l ow0))
("carden" nil (k aa1 r d ax n))
("cardenas" nil (k aa0 r d iy1 n ax s))
("carder" nil (k aa1 r d er0))
("cardholder" nil (k aa1 r d hh ow1 l d er0))
("cardholders" nil (k aa1 r d hh ow1 l d er0 z))
("cardia" nil (k aa1 r d iy0 ax))
("cardiac" nil (k aa1 r d iy0 ae1 k))
("cardiel" nil (k aa1 r d iy1 l))
("cardiff" nil (k aa1 r d ih0 f))
("cardigan" nil (k aa1 r d ih0 g ax n))
("cardigans" nil (k aa1 r d ih0 g ax n z))
("cardiges" nil (k aa1 r d ih0 jh ih0 z))
("cardillo" nil (k aa1 r d ih1 l ow0))
("cardin" nil (k aa1 r d ih0 n))
("cardinal" nil (k aa1 r d ax n ax l))
("cardinale" nil (k aa0 r d iy0 n aa1 l iy0))
("cardinali" nil (k aa0 r d iy0 n aa1 l iy0))
("cardinals" nil (k aa1 r d ax n ax l z))
("cardio" nil (k aa1 r d iy0 ow0))
("cardiologist" nil (k aa1 r d iy0 aa1 l ax jh ih0 s t))
("cardiologists" nil (k aa1 r d iy0 aa1 l ax jh ih0 s t s))
("cardiology" nil (k aa1 r d iy0 aa1 l ax jh iy0))
("cardiopulmonary" nil (k aa1 r d iy0 ow0 p uh1 l m ax n eh1 r iy0))
("cardiovascular" nil (k aa1 r d iy0 ow0 v ae1 s k y ax l er0))
("cardis" nil (k aa1 r d ih0 s))
("cardiss" nil (k aa1 r d ih0 s))
("cardizem" nil (k aa1 r d ih0 z eh1 m))
("cardoen" nil (k aa1 r d ow1 ax n))
("cardon" nil (k aa0 r d ao1 n))
("cardona" nil (k aa0 r d ow1 n ax))
("cardone" nil (k aa0 r d ow1 n iy0))
("cardoni" nil (k aa0 r d ow1 n iy0))
("cardoon" nil (k aa0 r d uw1 n))
("cardosa" nil (k aa0 r d ow1 s ax))
("cardosi" nil (k aa0 r d ow1 s iy0))
("cardoso" nil (k aa0 r d ow1 s ow0))
("cardoza" nil (k aa0 r d ow1 z ax))
("cardozo" nil (k aa0 r d ow1 z ow0))
("cards" nil (k aa1 r d z))
("carducci" nil (k aa0 r d uw1 ch iy0))
("cardwell" nil (k aa1 r d w eh1 l))
("cardy" nil (k aa1 r d iy0))
("care" nil (k eh1 r))
("cared" nil (k eh1 r d))
("careen" nil (k er0 iy1 n))
("careened" nil (k er0 iy1 n d))
("careening" nil (k er0 iy1 n ih0 ng))
("careens" nil (k er0 iy1 n z))
("career" nil (k er0 ih1 r))
("careercom" nil (k er0 ih1 r k aa0 m))
("careerist" nil (k er0 ih1 r ih0 s t))
("careerists" nil (k er0 ih1 r ih0 s t s))
("careers" nil (k er0 ih1 r z))
("carefree" nil (k eh1 r f r iy1))
("careful" nil (k eh1 r f ax l))
("carefully" nil (k eh1 r f ax l iy0))
("caregivers" nil (k eh1 r g ih1 v er0 z))
("carel" nil (k aa0 r eh1 l))
("careless" nil (k eh1 r l ax s))
("carelessly" nil (k eh1 r l ax s l iy0))
("carelessness" nil (k eh1 r l ax s n ax s))
("carella" nil (k aa0 r eh1 l ax))
("carelli" nil (k aa0 r eh1 l iy0))
("carello" nil (k eh1 r eh1 l ow0))
("carelock" nil (k ae1 r ih0 l aa0 k))
("caremark" nil (k eh1 r m aa1 r k))
("carena" nil (k eh1 r iy1 n ax))
("careplus" nil (k eh1 r p l ah1 s))
("carer" nil (k eh1 r er0))
("cares" nil (k eh1 r z))
("caress" nil (k er0 eh1 s))
("caresse" nil (k aa1 r eh0 s))
("caressing" nil (k er0 eh1 s ih0 ng))
("caretaker" nil (k eh1 r t ey1 k er0))
("caretakers" nil (k eh1 r t ey1 k er0 z))
("careunit" nil (k eh1 r y uw1 n ih0 t))
("carew" nil (k ae1 r uw0))
("carey" nil (k eh1 r iy0))
("carfagno" nil (k aa0 r f aa1 g n ow0))
("carfora" nil (k aa0 r f ao1 r ax))
("cargal" nil (k aa0 r g ae1 l))
("cargile" nil (k aa1 r jh ax l))
("cargill" nil (k aa1 r g ih1 l))
("cargo" nil (k aa1 r g ow1))
("cargoes" nil (k aa1 r g ow1 z))
("cargos" nil (k aa1 r g ow1 z))
("carhart" nil (k aa1 r hh aa1 r t))
("caribbean" nil (k er0 ih1 b iy0 ax n))
("caribe" nil (k eh1 r iy1 b))
("caribou" nil (k eh1 r ih0 b uw1))
("caricature" nil (k eh1 r ax k ax ch er0))
("caricatured" nil (k er0 ih1 k ax ch er0 d))
("caricatures" nil (k er0 ih1 k ax ch er0 z))
("caricaturist" nil (k eh1 r ax k ax ch er0 ax s t))
("carico" nil (k aa0 r iy1 k ow0))
("caricom" nil (k eh1 r ih0 k aa1 m))
("caridi" nil (k aa0 r iy1 d iy0))
("carie" nil (k eh1 r iy0))
("cariello" nil (k aa0 r iy0 eh1 l ow0))
("carignan" nil (k ae1 r ih0 g n ax n))
("cariker" nil (k ae1 r ih0 k er0))
("carilla" nil (k aa0 r ih1 l ax))
("carillo" nil (k aa0 r ih1 l ow0))
("carillon" nil (k eh1 r ax l aa1 n))
("carin" nil (k ae1 r ih0 n))
("carine" nil (k aa0 r iy1 n iy0))
("caring" nil (k eh1 r ih0 ng))
("carini" nil (k aa0 r iy1 n iy0))
("carinii" nil (k eh1 r iy1 n iy0))
("carino" nil (k aa0 r iy1 n ow0))
("carioca" nil (k eh1 r iy0 ow1 k ax))
("caris" nil (k ae1 r ih0 s))
("carisbrook" nil (k eh1 r ih0 s b r uh1 k))
("carissa" nil (k aa0 r iy1 s ax))
("carita" nil (k aa0 r iy1 t ax))
("carithers" nil (k ae1 r ih0 dh er0 z))
("carjack" nil (k aa1 r jh ae1 k))
("carjacked" nil (k aa1 r jh ae1 k t))
("carjacker" nil (k aa1 r jh ae1 k er0))
("carjackers" nil (k aa1 r jh ae1 k er0 z))
("carjacking" nil (k aa1 r jh ae1 k ih0 ng))
("carjackings" nil (k aa1 r jh ae1 k ih0 ng z))
("carjacks" nil (k aa1 r jh ae1 k s))
("carkhuff" nil (k aa1 r k hh ax f))
("carl" nil (k aa1 r l))
("carla" nil (k aa1 r l ax))
("carlan" nil (k aa1 r l ax n))
("carland" nil (k aa1 r l ax n d))
("carlberg" nil (k aa1 r l b er0 g))
("carle" nil (k aa1 r ax l))
("carleen" nil (k aa0 r l iy1 n))
("carlen" nil (k aa1 r l ax n))
("carlene" nil (k aa1 r l iy1 n))
("carles" nil (k aa1 r l z))
("carleton" nil (k aa1 r l t ax n))
("carley" nil (k aa1 r l iy0))
("carli" nil (k aa1 r l iy0))
("carlile" nil (k aa1 r l ay0 l))
("carlin" nil (k aa1 r l ih0 n))
("carliner" nil (k aa1 r l ay1 n er0))
("carling" nil (k aa1 r l ih0 ng))
("carlini" nil (k aa0 r l iy1 n iy0))
("carlino" nil (k aa0 r l iy1 n ow0))
("carlisi" nil (k aa0 r l iy1 s iy0))
("carlisle" nil (k aa1 r l ay1 l))
("carlita" nil (k aa0 r l iy1 t ax))
("carll" nil (k aa1 r l))
("carlo" nil (k aa1 r l ow0))
("carload" nil (k aa1 r l ow1 d))
("carloadings" nil (k aa1 r l ow1 d ih0 ng z))
("carlock" nil (k aa1 r l aa1 k))
("carlon" nil (k aa1 r l ax n))
("carlone" nil (k aa0 r l ow1 n iy0))
("carloni" nil (k aa0 r l ow1 n iy0))
("carlos" nil (k aa1 r l ow0 s))
("carlotta" nil (k aa0 r l ao1 t ax))
("carlough" nil (k aa1 r l ow0))
("carlow" nil (k aa1 r l ow1))
("carls" nil (k aa1 r l z))
("carlsbad" nil (k aa1 r l s b ae0 d))
("carlsberg" nil (k aa1 r l z b er0 g))
("carlsen" nil (k aa1 r l s ax n))
("carlson" nil (k aa1 r l s ax n))
("carlsson" nil (k aa1 r l s ax n))
("carlstadt" nil (k aa1 r l s t ae1 t))
("carlston" nil (k aa1 r l s t ax n))
("carlstrom" nil (k aa1 r l s t r ax m))
("carlton" nil (k aa1 r l t ax n))
("carlucci" nil (k aa0 r l uw1 ch iy0))
("carly" nil (k aa1 r l iy0))
("carlyle" nil (k aa1 r l ay1 l))
("carlyon" nil (k aa1 r l iy0 aa0 n))
("carlzon" nil (k aa1 r l z aa0 n))
("carma" nil (k aa1 r m ax))
("carmack" nil (k aa1 r m ax k))
("carmaker" nil (k aa1 r m ey1 k er0))
("carmakers" nil (k aa1 r m ey1 k er0 z))
("carman" nil (k aa1 r m ax n))
("carmany" nil (k aa1 r m ax n iy0))
("carmean" nil (k aa1 r m ax n))
("carmel" nil (k aa0 r m eh1 l))
("carmela" nil (k aa0 r m eh1 l ax))
("carmelite" nil (k aa1 r m ax l ay1 t))
("carmelo" nil (k aa0 r m eh1 l ow0))
("carmen" nil (k aa1 r m ax n))
("carmer" nil (k aa1 r m er0))
("carmical" nil (k aa1 r m ih0 k ax l))
("carmichael" nil (k aa1 r m ay1 k ax l))
("carmickle" nil (k aa1 r m ih0 k ax l))
("carmie" nil (k aa1 r m iy0))
("carmike" nil (k aa1 r m ay1 k))
("carmine" nil (k aa1 r m ax n))
("carmita" nil (k aa0 r m iy1 t ax))
("carmody" nil (k aa1 r m ax d iy0))
("carmon" nil (k aa1 r m ax n))
("carmona" nil (k aa0 r m ow1 n ax))
("carmony" nil (k aa1 r m ow0 n iy0))
("carmouche" nil (k aa0 r m uw1 sh))
("carmoy" nil (k aa1 r m oy0))
("carmy" nil (k aa1 r m iy0))
("carn" nil (k aa1 r n))
("carnage" nil (k aa1 r n ih0 jh))
("carnahan" nil (k aa1 r n ax hh ae0 n))
("carnal" nil (k aa1 r n ax l))
("carnathan" nil (k aa1 r n ax th ae0 n))
("carnation" nil (k aa0 r n ey1 sh ax n))
("carnations" nil (k aa0 r n ey1 sh ax n z))
("carnauba" nil (k aa0 r n ao1 b ax))
("carnaud" nil (k aa0 r n ao1 b))
("carne" nil (k aa1 r n))
("carneal" nil (k aa1 r n ax l))
("carnegie" nil (k aa1 r n ax g iy0))
("carnegies" nil (k aa1 r n ax g iy0 z))
("carneiro" nil (k aa0 r n eh1 r ow0))
("carnell" nil (k aa1 r n ax l))
("carner" nil (k aa1 r n er0))
("carnes" nil (k aa1 r n z))
("carnett" nil (k aa1 r n ax t))
("carnevale" nil (k aa0 r n ey0 v aa1 l ey0))
("carney" nil (k aa1 r n iy0))
("carnicero" nil (k aa1 r n ih0 s eh1 r ow0))
("carnine" nil (k aa0 r n iy1 n iy0))
("carnival" nil (k aa1 r n ax v ax l))
("carnivals" nil (k aa1 r n ax v ax l z))
("carnivore" nil (k aa1 r n ih0 v ao1 r))
("carnivores" nil (k aa1 r n ax v ao1 r z))
("carnivorous" nil (k aa0 r n ih1 v er0 ax s))
("carnley" nil (k aa1 r n l iy0))
("carns" nil (k aa1 r n z))
("caro" nil (k aa1 r ow0))
("carob" nil (k eh1 r ax b))
("caroche" nil (k er0 ow1 ch))
("carol" nil (k ae1 r ax l))
("carolan" nil (k eh1 r ax l ae0 n))
("carolco" nil (k er0 aa1 l k ow0))
("carole" nil (k ae1 r ax l))
("carolina" nil (k eh1 r ax l ay1 n ax))
("carolinas" nil (k eh1 r ax l ay1 n ax z))
("caroline" nil (k eh1 r ax l ay1 n))
("carolingian" nil (k eh1 r ax l ih1 n jh iy0 ax n))
("carolinians" nil (k eh1 r ax l ih1 n iy0 ax n z))
("carollan" nil (k er0 aa1 l ax n))
("carollo" nil (k er0 aa1 l ow0))
("carols" nil (k eh1 r ax l z))
("carolus" nil (k eh1 r ax l ax s))
("carolyn" nil (k eh1 r ax l ih0 n))
("carolyne" nil (k eh1 r ax l ih0 n))
("caron" nil (k aa0 r ao1 n))
("carone" nil (k er0 ow1 n))
("caronna" nil (k er0 aa1 n ax))
("carosella" nil (k aa0 r ow0 s eh1 l ax))
("caroselli" nil (k aa0 r ow0 s eh1 l iy0))
("carosi" nil (k er0 ow1 s iy0))
("carota" nil (k er0 ow1 t ax))
("carotene" nil (k eh1 r ax t iy1 n))
("carotenuto" nil (k aa0 r ow0 t eh0 n uw1 t ow0))
("carothers" nil (k ae1 r ax dh er0 z))
("carousel" nil (k eh1 r ax s eh1 l))
("carow" nil (k ae1 r ow0))
("carozza" nil (k er0 aa1 z ax))
("carp" nil (k aa1 r p))
("carpal" nil (k aa1 r p ax l))
("carpencic" nil (k aa0 r p eh1 n ch ih0 k))
("carpenito" nil (k aa0 r p eh0 n iy1 t ow0))
("carpenter" nil (k aa1 r p ax n t er0))
("carpenters" nil (k aa1 r p ax n t er0 z))
("carpentier" nil (k aa1 r p ax n t iy0 er0))
("carpentieri" nil (k aa0 r p eh0 n t ih1 r iy0))
("carpentry" nil (k aa1 r p ax n t r iy0))
("carper" nil (k aa1 r p er0))
("carpet" nil (k aa1 r p ax t))
("carpetbagger" nil (k aa1 r p ax t b ae1 g er0))
("carpetbaggers" nil (k aa1 r p ax t b ae1 g er0 z))
("carpeted" nil (k aa1 r p ax t ih0 d))
("carpeting" nil (k aa1 r p ax t ih0 ng))
("carpets" nil (k aa1 r p ax t s))
("carpinelli" nil (k aa0 r p iy0 n eh1 l iy0))
("carping" nil (k aa1 r p ih0 ng))
("carpino" nil (k aa0 r p iy1 n ow0))
("carpio" nil (k aa1 r p iy0 ow0))
("carport" nil (k aa1 r p ao1 r t))
("carports" nil (k aa1 r p ao1 r t s))
("carr" nil (k aa1 r))
("carra" nil (k aa1 r ax))
("carragher" nil (k ae1 r ax g hh er0))
("carraher" nil (k ae1 r ax hh er0))
("carranco" nil (k aa0 r aa1 n k ow0))
("carrano" nil (k aa1 r aa1 n ow0))
("carranza" nil (k aa0 r aa1 n z ax))
("carrara" nil (k aa1 r aa1 r ax))
("carras" nil (k ae1 r ax z))
("carrasco" nil (k aa0 r aa1 s k ow0))
("carrasquel" nil (k eh1 r ax s k eh1 l))
("carrasquillo" nil (k eh1 r ax s k ih1 l ow0))
("carraway" nil (k ae1 r ax w ey1))
("carre" nil (k aa1 r))
("carreira" nil (k aa0 r eh1 r ax))
("carreiro" nil (k aa0 r eh1 r ow0))
("carreker" nil (k eh1 r ih0 k er0))
("carrel" nil (k ae1 r ax l))
("carrell" nil (k aa0 r ey1 l))
("carreno" nil (k aa0 r eh1 n ow0))
("carreon" nil (k aa0 r ey0 ao1 n))
("carrera" nil (k aa0 r eh1 r ax))
("carreras" nil (k aa0 r eh1 r aa0 z))
("carrere" nil (k aa0 r eh1 r ey0))
("carrero" nil (k aa1 r eh1 r ow0))
("carretta" nil (k aa0 r eh1 t ax))
("carrey" nil (k eh1 r iy0))
("carriage" nil (k ae1 r ih0 jh))
("carriages" nil (k ae1 r ih0 jh ih0 z))
("carrian" nil (k eh1 r iy0 ax n))
("carrick" nil (k eh1 r ih0 k))
("carrico" nil (k aa0 r iy1 k ow0))
("carrie" nil (k eh1 r iy0))
("carried" nil (k ae1 r iy0 d))
("carrier" nil (k ae1 r iy0 er0))
("carriere" nil (k aa0 r ih1 r iy0))
("carriero" nil (k aa0 r ih1 r ow0))
("carriers" nil (k ae1 r iy0 er0 z))
("carries" nil (k ae1 r iy0 z))
("carrig" nil (k ae1 r ih0 g))
("carrigan" nil (k ae1 r ih0 g ax n))
("carriger" nil (k ae1 r ih0 jh er0))
("carrigg" nil (k ae1 r ih0 g))
("carriker" nil (k ae1 r ih0 k er0))
("carrillo" nil (k er0 ih1 l ow0))
("carringer" nil (k ae1 r ih0 ng er0))
("carrington" nil (k eh1 r ih0 ng t ax n))
("carrino" nil (k aa1 r iy1 n ow0))
("carrion" nil (k eh1 r iy0 ax n))
("carris" nil (k ae1 r ih0 s))
("carrisyn" nil (k ae1 r ih0 s ih0 n))
("carrithers" nil (k ae1 r ih0 dh er0 z))
("carriveau" nil (k ae1 r ih0 v ow1))
("carrizales" nil (k aa0 r iy0 z aa1 l eh0 s))
("carro" nil (k aa1 r ow0))
("carrol" nil (k ae1 r ax l))
("carroll" nil (k ae1 r ax l))
("carrollton" nil (k eh1 r ax l t ax n))
("carron" nil (k ae1 r ax n))
("carrot" nil (k ae1 r ax t))
("carrothers" nil (k ae1 r ax dh er0 z))
("carrots" nil (k ae1 r ax t s))
("carrousel" nil (k eh1 r ax s eh1 l))
("carrow" nil (k ae1 r ow0))
("carrozza" nil (k aa0 r ow1 z ax))
("carrubba" nil (k aa1 r uw1 b ax))
("carruth" nil (k ae1 r uw0 th))
("carruthers" nil (k er0 ah1 dh er0 z))
("carry" nil (k ae1 r iy0))
("carryforwards" nil (k eh1 r iy0 f ao1 r w er0 d z))
("carrying" nil (k ae1 r iy0 ih0 ng))
("carryover" nil (k eh1 r y ow1 v er0))
("cars" nil (k aa1 r z))
("carse" nil (k aa1 r s))
("carsey" nil (k aa1 r s iy0))
("carsick" nil (k aa1 r s ih0 k))
("carson" nil (k aa1 r s ax n))
("carstarphen" nil (k aa0 r s t aa1 r f ax n))
("carsten" nil (k aa1 r s ax n))
("carstens" nil (k aa1 r s ax n z))
("carstensen" nil (k aa0 r s t eh1 n s ax n))
("carswell" nil (k aa1 r s w eh1 l))
("cart" nil (k aa1 r t))
("carta" nil (k aa1 r t ax))
("cartagena" nil (k aa1 r t ax jh iy1 n ax))
("cartaya" nil (k aa1 r t ay1 ax))
("carte" nil (k aa1 r t))
("carted" nil (k aa1 r t ih0 d))
("cartee" nil (k aa1 r t iy1))
("cartel" nil (k aa0 r t eh1 l))
("cartels" nil (k aa0 r t eh1 l z))
("carter" nil (k aa1 r t er0))
("cartera" nil (k aa1 r t eh1 r ax))
("carteret" nil (k aa1 r t er0 ih0 t))
("carters" nil (k aa1 r t er0 z))
("cartersville" nil (k aa1 r t er0 z v ih1 l))
("carthage" nil (k aa1 r th ax jh))
("carthaginian" nil (k aa1 r th ax jh ih1 n iy0 ax n))
("cartier" nil (k aa1 r t iy0 er0))
("cartilage" nil (k aa1 r t ax l ax jh))
("carting" nil (k aa1 r t ih0 ng))
("cartland" nil (k aa1 r t l ax n d))
("cartledge" nil (k aa1 r t l eh1 jh))
("cartlidge" nil (k aa1 r t l ih0 jh))
("cartmell" nil (k aa0 r t m ey1 l))
("cartmill" nil (k aa1 r t m ih1 l))
("cartner" nil (k aa1 r t n er0))
("carton" nil (k aa1 r t ax n))
("cartoneros" nil (k aa1 r t ow1 n eh1 r ow0 s))
("cartons" nil (k aa1 r t ax n z))
("cartoon" nil (k aa0 r t uw1 n))
("cartooning" nil (k aa0 r t uw1 n ih0 ng))
("cartoonist" nil (k aa0 r t uw1 n ax s t))
("cartoonists" nil (k aa0 r t uw1 n ih0 s t s))
("cartoons" nil (k aa0 r t uw1 n z))
("cartrette" nil (k aa1 r t r eh1 t))
("cartridge" nil (k aa1 r t r ax jh))
("cartridges" nil (k aa1 r t r ax jh ax z))
("carts" nil (k aa1 r t s))
("cartusciello" nil (k aa0 r t uw1 s iy0 eh1 l ow0))
("cartwheels" nil (k aa1 r t w iy1 l z))
("cartwright" nil (k aa1 r t r ay1 t))
("carty" nil (k aa1 r t iy0))
("caruana" nil (k aa0 r uw0 ae1 n ax))
("carucci" nil (k aa0 r uw1 ch iy0))
("caruso" nil (k er0 uw1 s ow0))
("carusone" nil (k aa0 r uw0 s ow1 n iy0))
("caruth" nil (k ae1 r uw0 th))
("caruthers" nil (k er0 ah1 dh er0 z))
("carvajal" nil (k aa0 r v aa0 y ae1 l))
("carvalho" nil (k aa0 r v aa1 l hh ow0))
("carve" nil (k aa1 r v))
("carved" nil (k aa1 r v d))
("carvell" nil (k aa0 r v ey1 l))
("carver" nil (k aa1 r v er0))
("carvers" nil (k aa1 r v er0 z))
("carves" nil (k aa1 r v z))
("carvey" nil (k aa1 r v iy0))
("carville" nil (k aa1 r v ih1 l))
("carvin" nil (k aa1 r v ih0 n))
("carving" nil (k aa1 r v ih0 ng))
("carvings" nil (k aa1 r v ih0 ng z))
("carwell" nil (k aa1 r w eh1 l))
("carwile" nil (k aa1 r w ay1 l))
("cary" nil (k eh1 r iy0))
("caryl" nil (k eh1 r ax l))
("caryn" nil (k aa1 r ih0 n))
("casa" nil (k aa1 s ax))
("casablanca" nil (k ae1 s ax b l ae1 ng k ax))
("casad" nil (k ae1 s ax d))
("casada" nil (k aa0 s aa1 d ax))
("casado" nil (k aa0 s aa1 d ow0))
("casados" nil (k aa0 s aa1 d ow0 z))
("casady" nil (k ax s aa1 d iy0))
("casagrande" nil (k aa0 s aa1 g r ae0 n d iy0))
("casale" nil (k aa0 s aa1 l iy0))
("casali" nil (k aa0 s aa1 l iy0))
("casalino" nil (k aa0 s aa0 l iy1 n ow0))
("casamento" nil (k ae1 s ax m eh1 n t ow0))
("casanova" nil (k ae1 s ax n ow1 v ax))
("casares" nil (k aa0 s aa1 r eh0 s))
("casarez" nil (k aa0 s aa1 r eh0 z))
("casas" nil (k aa1 s ax z))
("casassa" nil (k aa0 s aa1 s ax))
("casaus" nil (k ae1 s ao0 z))
("casavant" nil (k ae1 s ax v ax n t))
("casazza" nil (k aa0 s aa1 t s ax))
("casbah" nil (k ae1 s b ax))
("cascade" nil (k ae0 s k ey1 d))
("cascaded" nil (k ae0 s k ey1 d ih0 d))
("cascades" nil (k ae0 s k ey1 d z))
("cascading" nil (k ae0 s k ey1 d ih0 ng))
("casciano" nil (k aa0 s ch iy0 aa1 n ow0))
("casciato" nil (k aa0 s ch aa1 t ow0))
("cascio" nil (k ae1 s iy0 ow0))
("casco" nil (k ae1 s k ow0))
("cascone" nil (k aa0 s k ow1 n iy0))
("casdorph" nil (k ae1 s d er0 f))
("case" nil (k ey1 s))
("casebeer" nil (k ae0 s ih0 b ih1 r))
("casebier" nil (k ae1 s ih0 b iy0 er0))
("casebolt" nil (k ey1 s b ow1 l t))
("casein" nil (k ey0 s iy1 n))
("casella" nil (k ax s eh1 l ax))
("caselli" nil (k ax s eh1 l iy0))
("caseload" nil (k ey1 s l ow1 d))
("caseloads" nil (k ae1 s ax l ow1 d z))
("casera" nil (k ax s eh1 r ax))
("caserta" nil (k aa0 s eh1 r t ax))
("cases" nil (k ey1 s ax z))
("casesa" nil (k ax s eh1 s ax))
("caseworker" nil (k ey1 s w er1 k er0))
("caseworkers" nil (k ey1 s w er1 k er0 z))
("casey" nil (k ey1 s iy0))
("casgrain" nil (k ae1 s g r ey1 n))
("cash" nil (k ae1 sh))
("cashatt" nil (k ae1 sh ax t))
("cashdollar" nil (k ae1 sh d aa1 l er0))
("cashed" nil (k ae1 sh t))
("cashen" nil (k ae1 sh ax n))
("casher" nil (k ae1 sh er0))
("cashes" nil (k ae1 sh ih0 z))
("cashew" nil (k ae1 sh uw1))
("cashews" nil (k ae1 sh uw1 z))
("cashflow" nil (k ae1 sh f l ow1))
("cashier" nil (k ae0 sh ih1 r))
("cashiered" nil (k ae0 sh ih1 r d))
("cashiers" nil (k ae1 sh ih1 r z))
("cashin" nil (k ae1 sh ih0 n))
("cashing" nil (k ae1 sh ih0 ng))
("cashion" nil (k aa0 sh iy1 n))
("cashless" nil (k ae1 sh l ax s))
("cashman" nil (k ae1 sh m ax n))
("cashmere" nil (k ae1 zh m ih0 r))
("cashmore" nil (k ae1 sh m ao0 r))
("cashon" nil (k ae1 sh ax n))
("cashways" nil (k ae1 sh w ey1 z))
("cashwell" nil (k ae1 sh w eh1 l))
("casiano" nil (k aa0 s iy0 aa1 n ow0))
("casias" nil (k aa0 s iy1 ax z))
("casillas" nil (k aa0 s ih1 l ax z))
("casillo" nil (k ax s ih1 l ow0))
("casimir" nil (k ae1 s ih0 m iy1 r))
("casimiro" nil (k aa0 s iy0 m ih1 r ow0))
("casing" nil (k ey1 s ih0 ng))
("casings" nil (k ey1 s ih0 ng z))
("casino" nil (k ax s iy1 n ow0))
("casinos" nil (k ax s iy1 n ow0 z))
("casio" nil (k ae1 s iy0 ow0))
("cask" nil (k ae1 s k))
("casket" nil (k ae1 s k ax t))
("caskets" nil (k ae1 s k ax t s))
("caskey" nil (k ae1 s k iy0))
("casks" nil (k ae1 s k s))
("casler" nil (k ae1 s ax l er0))
("caslin" nil (k ae1 s l ih0 n))
("casner" nil (k ae1 s n er0))
("caso" nil (k aa1 s ow0))
("casola" nil (k aa0 s ow1 l ax))
("cason" nil (k ae1 s ax n))
("caspar" nil (k ae1 s p er0))
("casper" nil (k ae1 s p er0))
("caspers" nil (k ae1 s p er0 z))
("caspersen" nil (k ae1 s p er0 s ax n))
("casperson" nil (k ae1 s p er0 s ax n))
("caspian" nil (k ae1 s p iy0 ax n))
("cass" nil (k ae1 s))
("cassada" nil (k aa0 s aa1 d ax))
("cassaday" nil (k ae1 s ax d ey1))
("cassady" nil (k ae1 s ax d iy0))
("cassandra" nil (k ax s ae1 n d r ax))
("cassandras" nil (k ax s ae1 n d r ax s))
("cassani" nil (k aa0 s aa1 n iy0))
("cassano" nil (k aa0 s aa1 n ow0))
("cassar" nil (k ae1 s er0))
("cassara" nil (k aa0 s aa1 r ax))
("cassarino" nil (k aa0 s aa0 r iy1 n ow0))
("cassaro" nil (k aa0 s aa1 r ow0))
("cassata" nil (k aa0 s aa1 t ax))
("cassatt" nil (k ae1 s ax t))
("casseb" nil (k ae1 s ax b))
("cassedy" nil (k ae1 s ih0 d iy0))
("cassel" nil (k ae1 s ax l))
("casselberry" nil (k ae1 s ax l b eh1 r iy0))
("cassell" nil (k ae1 s ax l))
("cassella" nil (k aa0 s eh1 l ax))
("cassells" nil (k ae1 s ax l z))
("casselman" nil (k ae1 s ax l m ax n))
("cassels" nil (k ae1 s ax l z))
("cassens" nil (k ae1 s ax n z))
("casserly" nil (k ae1 s er0 l iy0))
("casserole" nil (k ae1 s er0 ow1 l))
("cassese" nil (k aa0 s ey1 z iy0))
("cassetta" nil (k aa0 s eh1 t ax))
("cassette" nil (k ax s eh1 t))
("cassettes" nil (k ax s eh1 t s))
("cassetty" nil (k ae1 s ih0 t iy0))
("cassia" nil (k ae1 sh iy0 ax))
("cassiday" nil (k ae1 s iy0 d ey0))
("cassidy" nil (k ae1 s ih0 d iy0))
("cassie" nil (k ae1 s iy0))
("cassin" nil (k ae1 s ih0 n))
("cassinelli" nil (k aa0 s iy0 n eh1 l iy0))
("cassini" nil (k ax s iy1 n iy0))
("cassino" nil (k ax s iy1 n ow0))
("cassis" nil (k ae1 s ih0 s))
("cassity" nil (k ae1 s ih0 t iy0))
("casso" nil (k ae1 s ow0))
("cassock" nil (k ae1 s ax k))
("casson" nil (k ae1 s ax n))
("cassone" nil (k aa0 s ow1 n iy0))
("cassoni" nil (k ax s ow1 n iy0))
("cassowary" nil (k ae1 s ax w eh1 r iy0))
("casstevens" nil (k aa0 s t ey0 v ey1 ax n z))
("cast" nil (k ae1 s t))
("casta" nil (k ae1 s t ax))
("castagna" nil (k aa0 s t aa1 g n ax))
("castagno" nil (k aa0 s t aa1 g n ow0))
("castagnola" nil (k aa0 s t aa0 g n ow1 l ax))
("castaldi" nil (k aa0 s t aa1 l d iy0))
("castaldo" nil (k aa0 s t aa1 l d ow0))
("castaneda" nil (k aa0 s t aa0 n eh1 d ax))
("castano" nil (k aa0 s t aa1 n ow0))
("castanon" nil (k ae1 s t ax n aa0 n))
("castaways" nil (k ae1 s t ax w ey1 z))
("caste" nil (k ae1 s t))
("casteel" nil (k ey1 s t iy0 l))
("casteen" nil (k ax s t iy1 n))
("castel" nil (k ey1 s t ax l))
("castell" nil (k ey1 s t ax l))
("castellana" nil (k aa0 s t eh0 l ae1 n ax))
("castellani" nil (k aa0 s t eh0 l aa1 n iy0))
("castellano" nil (k aa0 s t eh0 l aa1 n ow0))
("castellanos" nil (k aa0 s t ey0 l aa1 n ow0 z))
("castellaw" nil (k ey1 s t ih0 l ao0))
("castelli" nil (k aa0 s t eh1 l iy0))
("castello" nil (k ax s t eh1 l ow0))
("castellon" nil (k ey1 s t ih0 l aa0 n))
("castellucci" nil (k aa0 s t eh0 l uw1 ch iy0))
("castelluccio" nil (k aa0 s t eh0 l uw1 ch iy0 ow0))
("castellvi" nil (k ax s t eh1 l v iy0))
("castelo" nil (k aa0 s t eh1 l ow0))
("casten" nil (k ae1 s t ax n))
("caster" nil (k ae1 s t er0))
("casterline" nil (k ae1 s t er0 l ay1 n))
("casters" nil (k ae1 s t er0 z))
("castigate" nil (k ae1 s t ax g ey1 t))
("castigated" nil (k ae1 s t ax g ey1 t ih0 d))
("castigating" nil (k ae1 s t ax g ey1 t ih0 ng))
("castiglia" nil (k aa0 s t iy1 g l iy0 ax))
("castiglione" nil (k aa0 s t iy0 g l iy0 ow1 n iy0))
("castilla" nil (k aa0 s t ih1 l ax))
("castille" nil (k ae1 s t ih0 l))
("castilleja" nil (k aa0 s t iy0 l ey1 y ax))
("castillo" nil (k ae1 s t iy1 ow0))
("castillon" nil (k ae1 s t ih0 l aa0 n))
("castine" nil (k aa0 s t iy1 n iy0))
("casting" nil (k ae1 s t ih0 ng))
("castings" nil (k ae1 s t ih0 ng z))
("castle" nil (k ae1 s ax l))
("castleberry" nil (k ae1 s ax l b eh1 r iy0))
("castleman" nil (k ae1 s ax l m ax n))
("castles" nil (k ae1 s ax l z))
("castner" nil (k ae1 s t n er0))
("casto" nil (k ae1 s t ow0))
("castoffs" nil (k ae1 s t ao1 f s))
("caston" nil (k ae1 s t ax n))
("castonguay" nil (k ax s t aa1 n g ey0))
("castor" nil (k ae1 s t er0))
("castorena" nil (k aa1 s t ao0 r n ax))
("castoro" nil (k aa0 s t ao1 r ow0))
("castparts" nil (k ae1 s t p aa1 r t s))
("castrate" nil (k ae1 s t r ey1 t))
("castrated" nil (k ae1 s t r ey1 t ih0 d))
("castrates" nil (k ae1 s t r ey1 t s))
("castrating" nil (k ae1 s t r ey1 t ih0 ng))
("castration" nil (k ae1 s t r ey1 sh ax n))
("castrations" nil (k ae1 s t r ey1 sh ax n z))
("castrejon" nil (k ae1 s t r ih0 jh aa0 n))
("castro" nil (k ae1 s t r ow0))
("castrol" nil (k ae1 s t r aa0 l))
("castronova" nil (k ae0 s t r ow0 n ow1 v ax))
("castronovo" nil (k ae0 s t r ow0 n ow1 v ow0))
("castrucci" nil (k ax s t r uw1 ch iy0))
("castruita" nil (k aa0 s t r uw1 t ax))
("casts" nil (k ae1 s t s))
("casual" nil (k ae1 zh ax w ax l))
("casually" nil (k ae1 zh ax w ax l iy0))
("casuals" nil (k ae1 zh ax w ax l z))
("casualties" nil (k ae1 zh ax w ax l t iy0 z))
("casualty" nil (k ae1 zh ax w ax l t iy0))
("caswell" nil (k ae1 z w eh1 l))
("cat" nil (k ae1 t))
("cataclysm" nil (k ae1 t ax k l ih1 s ax m))
("cataclysmic" nil (k ae1 t ax k l ih1 z m ih0 k))
("catacombs" nil (k ae1 t ax k ow1 m z))
("catacosinos" nil (k ax t ae1 k ax s iy1 n ow0 s))
("catain" nil (k ae1 t ih0 n))
("catalan" nil (k ae1 t ax l ax n))
("catalano" nil (k aa0 t aa0 l aa1 n ow0))
("catalanotto" nil (k aa0 t aa0 l aa0 n ow1 t ow0))
("cataldi" nil (k aa0 t aa1 l d iy0))
("cataldo" nil (k aa0 t aa1 l d ow0))
("catalfamo" nil (k aa0 t aa0 l f aa1 m ow0))
("catalina" nil (k ae1 t ax l iy1 n ax))
("catalog" nil (k ae1 t ax l ao0 g))
("cataloged" nil (k ae1 t ax l ao0 g d))
("catalogers" nil (k ae1 t ax l ao1 g er0 z))
("cataloging" nil (k ae1 t ax l aa0 g ih0 ng))
("catalogs" nil (k ae1 t ax l aa0 g z))
("catalogue" nil (k ae1 t ax l ao1 g))
("catalogued" nil (k ae1 t ax l ao1 g d))
("catalogues" nil (k ae1 t ax l ao1 g z))
("cataloguing" nil (k ae1 t ax l ao1 g ih0 ng))
("catalonia" nil (k ae1 t ax l ow1 n y ax))
("catalyst" nil (k ae1 t ax l ax s t))
("catalysts" nil (k ae1 t ax l ax s t s))
("catalytic" nil (k ae1 t ax l ih1 t ih0 k))
("catalyzed" nil (k ae1 t ax l ay1 z d))
("catamaran" nil (k ae1 t ax m er0 ae1 n))
("catamarans" nil (k ax t ae1 m er0 ax n z))
("catamount" nil (k ae1 t ax m aw1 n t))
("catanese" nil (k aa0 t aa0 n ey1 z iy0))
("catania" nil (k ax t ey1 n y ax))
("catano" nil (k aa0 t aa1 n ow0))
("catanzarite" nil (k ax t ae1 n z er0 ay1 t))
("catanzaro" nil (k aa0 t aa0 n z aa1 r ow0))
("catapano" nil (k aa0 t aa0 p aa1 n ow0))
("catapult" nil (k ae1 t ax p ah1 l t))
("catapulted" nil (k ae1 t ax p ah1 l t ih0 d))
("catapulting" nil (k ae1 t ax p ah1 l t ih0 ng))
("catapults" nil (k ae1 t ax p ax l t s))
("cataract" nil (k ae1 t er0 ae0 k t))
("cataracts" nil (k ae1 t er0 ae1 k t s))
("catastrophe" nil (k ax t ae1 s t r ax f iy0))
("catastrophes" nil (k ax t ae1 s t r ax f iy0 z))
("catastrophic" nil (k ae1 t ax s t r aa1 f ih0 k))
("catatonic" nil (k ae1 t ax t aa1 n ih0 k))
("catawba" nil (k ax t ao1 b ax))
("catbird" nil (k ae1 t b er1 d))
("catcalls" nil (k ae1 t k ao1 l z))
("catch" nil (k ae1 ch))
("catchall" nil (k ae1 ch ao1 l))
("catcher" nil (k ae1 ch er0))
("catchers" nil (k ae1 ch er0 z))
("catches" nil (k ae1 ch ax z))
("catching" nil (k ae1 ch ih0 ng))
("catchings" nil (k ae1 ch ih0 ng z))
("catchup" nil (k ae1 ch ax p))
("catchword" nil (k ae1 ch w er1 d))
("catchy" nil (k ae1 ch iy0))
("cate" nil (k ey1 t))
("catechism" nil (k ae1 t ax k ih1 z ax m))
("categorical" nil (k ae1 t ax g aa1 r ih0 k ax l))
("categorically" nil (k ae1 t ax g aa1 r ih0 k l iy0))
("categories" nil (k ae1 t ax g ao1 r iy0 z))
("categorization" nil (k ae1 t ax g er0 ih0 z ey1 sh ax n))
("categorize" nil (k ae1 t ax g er0 ay1 z))
("categorized" nil (k ae1 t ax g er0 ay1 z d))
("categorizes" nil (k ae1 t ax g er0 ay1 z ih0 z))
("categorizing" nil (k ae1 t ax g er0 ay1 z ih0 ng))
("category" nil (k ae1 t ax g ao1 r iy0))
("catena" nil (k ax t iy1 n ax))
("cater" nil (k ey1 t er0))
("catered" nil (k ey1 t er0 d))
("caterer" nil (k ey1 t er0 er0))
("caterers" nil (k ey1 t er0 er0 z))
("catering" nil (k ey1 t er0 ih0 ng))
("caterino" nil (k aa0 t er0 iy1 n ow0))
("caterpillar" nil (k ae1 t ax p ih1 l er0))
("caterpillars" nil (k ae1 t ax p ih1 l er0 z))
("caterpiller" nil (k ae1 t ax p ih1 l er0))
("caters" nil (k ey1 t er0 z))
("cates" nil (k ey1 t s))
("catfish" nil (k ae1 t f ih1 sh))
("catharine" nil (k ae1 th r ih0 n))
("catharines" nil (k ae1 th r ih0 n z))
("catharsis" nil (k ax th aa1 r s ax s))
("cathartic" nil (k ax th aa1 r t ih0 k))
("cathay" nil (k ae0 th ey1))
("cathcart" nil (k ae1 th k aa0 r t))
("cathedral" nil (k ax th iy1 d r ax l))
("cathedrals" nil (k ax th iy1 d r ax l z))
("cathell" nil (k ae1 th ax l))
("cather" nil (k ae1 dh er0))
("catherina" nil (k aa0 th er0 iy1 n ax))
("catherine" nil (k ae1 th er0 ax n))
("catherman" nil (k ae1 dh er0 m ax n))
("cathers" nil (k ae1 dh er0 z))
("catherwood" nil (k ae1 dh er0 w uh1 d))
("catheter" nil (k ae1 th ax t er0))
("catheters" nil (k ae1 th ax t er0 z))
("cathey" nil (k ae1 dh iy0))
("cathie" nil (k ae1 th iy0))
("cathleen" nil (k ae1 th l iy1 n))
("cathmor" nil (k ae1 th ax m er0))
("cathode" nil (k ae1 th ow1 d))
("cathodes" nil (k ae1 th ow1 d z))
("catholic" nil (k ae1 th l ih0 k))
("catholicism" nil (k ax th ao1 l ax s ih1 z ax m))
("catholics" nil (k ae1 th l ih0 k s))
("cathy" nil (k ae1 th iy0))
("catino" nil (k aa0 t iy1 n ow0))
("catkins" nil (k ae1 t k ax n z))
("catledge" nil (k ae1 t l ih0 jh))
("catlett" nil (k ae1 t l ih0 t))
("catlike" nil (k ae1 t l ay1 k))
("catlin" nil (k ae1 t l ih0 n))
("catnap" nil (k ae1 t n ae1 p))
("catnip" nil (k ae1 t n ih0 p))
("cato" nil (k ey1 t ow0))
("catoe" nil (k ae1 t ow0))
("catolica" nil (k ax t ow1 l ih0 k ax))
("caton" nil (k ae1 t ax n))
("catone" nil (k ax t ow1 n))
("catrambone" nil (k ae1 t r ae1 m b ow1 n))
("catrett" nil (k ae1 t r ih0 t))
("catron" nil (k ae1 t r ax n))
("cats" nil (k ae1 t s))
("catskill" nil (k ae1 t s k ih1 l))
("catskills" nil (k ae1 t s k ih1 l z))
("catsup" nil (k eh1 ch ax p))
("catt" nil (k ae1 t))
("cattanach" nil (k ae1 t ax n ae0 ch))
("cattaneo" nil (k aa0 t aa1 n iy0 ow0))
("cattani" nil (k aa0 t aa1 n iy0))
("cattell" nil (k ax t eh1 l))
("catterall" nil (k ae1 t er0 ao1 l))
("catterson" nil (k ae1 t er0 s ax n))
("catterton" nil (k ae1 t er0 t ax n))
("cattle" nil (k ae1 t ax l))
("cattlemen" nil (k ae1 t ax l m ax n))
("catto" nil (k ae1 t ow0))
("cattolica" nil (k ax t ow1 l ih0 k ax))
("catton" nil (k ae1 t ax n))
("catty" nil (k ae1 t iy0))
("catwalk" nil (k ae1 t w aa1 k))
("cauble" nil (k ao1 b ax l))
("caucasian" nil (k ao0 k ey1 zh ax n))
("caucasus" nil (k ao1 k ax s ax s))
("caucus" nil (k ao1 k ax s))
("caucuses" nil (k ao1 k ax s ih0 z))
("caudal" nil (k aa1 d ax l))
("caudell" nil (k ow0 d eh1 l))
("caudill" nil (k ao1 d ax l))
("caudillo" nil (k ao1 d ih1 l ow0))
("caudle" nil (k ao1 d ax l))
("cauffman" nil (k ao1 f m ax n))
("caufield" nil (k ow1 f iy0 l d))
("caughey" nil (k ao1 iy0))
("caughlin" nil (k ao1 l ih0 n))
("caughman" nil (k ao1 m ax n))
("caughron" nil (k ao1 r ax n))
("caught" nil (k aa1 t))
("caul" nil (k aa1 l))
("caulder" nil (k ao1 l d er0))
("cauldron" nil (k aa1 l d r ax n))
("cauley" nil (k ao1 l iy0))
("caulfield" nil (k ao1 l f iy1 l d))
("cauliflower" nil (k aa1 l ax f l aw1 er0))
("caulk" nil (k aa1 k))
("caulking" nil (k ao1 k ih0 ng))
("caulkins" nil (k ao1 l k ih0 n z))
("causal" nil (k ao1 z ax l))
("causation" nil (k ao1 z ey1 sh ax n))
("causative" nil (k aa1 z ax t ih0 v))
("causby" nil (k ao1 z b iy0))
("cause" nil (k aa1 z))
("caused" nil (k aa1 z d))
("causer" nil (k ao1 z er0))
("causes" nil (k aa1 z ax z))
("causeway" nil (k aa1 z w ey1))
("causeways" nil (k ao1 z w ey1 z))
("causey" nil (k ao1 z iy0))
("causing" nil (k aa1 z ih0 ng))
("caustic" nil (k aa1 s t ih0 k))
("cauthen" nil (k ao1 th ax n))
("cauthon" nil (k ao1 th ax n))
("cauthorn" nil (k ao1 th er0 n))
("caution" nil (k aa1 sh ax n))
("cautionary" nil (k ao1 sh ax n eh1 r iy0))
("cautioned" nil (k aa1 sh ax n d))
("cautioning" nil (k ao1 sh ax n ih0 ng))
("cautions" nil (k ao1 sh ax n z))
("cautious" nil (k ao1 sh ax s))
("cautiously" nil (k ao1 sh ax s l iy0))
("cautiousness" nil (k ao1 sh ax s n ax s))
("cava" nil (k aa1 v ax))
("cavaco" nil (k ae1 v ax k ow0))
("cavagnaro" nil (k aa0 v aa0 g n aa1 r ow0))
("cavalcade" nil (k ae1 v ax l k ey1 d))
("cavalier" nil (k ae1 v ax l ih1 r))
("cavaliere" nil (k aa0 v aa0 l ih1 r iy0))
("cavalieri" nil (k aa0 v aa0 l ih1 r iy0))
("cavalierly" nil (k ae1 v ax l ih1 r l iy0))
("cavaliers" nil (k ae1 v ax l ih1 r z))
("cavallaro" nil (k aa0 v aa0 l aa1 r ow0))
("cavallero" nil (k aa0 v aa0 l eh1 r ow0))
("cavalli" nil (k ax v ae1 l iy0))
("cavallo" nil (k ax v ae1 l ow0))
("cavalry" nil (k ae1 v ax l r iy0))
("cavan" nil (k ey1 v ax n))
("cavanagh" nil (k ae1 v ax n ae0 g))
("cavanah" nil (k ae1 v ax n ax))
("cavanaugh" nil (k ae1 v ax n ao0))
("cavaness" nil (k aa1 v ax n ih0 s))
("cavataio" nil (k aa0 v aa0 t aa1 iy0 ow0))
("cavazos" nil (k aa0 v aa1 z ow0 z))
("cave" nil (k ey1 v))
("caveat" nil (k ey1 v iy0 ae1 t))
("caveats" nil (k ey1 v iy0 ae1 t s))
("caved" nil (k ey1 v d))
("caveman" nil (k ey1 v m ae1 n))
("caven" nil (k ey1 v ax n))
("cavenaugh" nil (k ae1 v ih0 n ao0))
("cavender" nil (k ax v eh1 n d er0))
("cavendish" nil (k ae1 v ax n d ih0 sh))
("caver" nil (k ey1 v er0))
("caverly" nil (k ey1 v er0 l iy0))
("cavern" nil (k ae1 v er0 n))
("cavernous" nil (k ae1 v er0 n ax s))
("caverns" nil (k ae1 v er0 n z))
("cavers" nil (k ey1 v er0 z))
("caves" nil (k ey1 v z))
("cavett" nil (k ae1 v ih0 t))
("cavey" nil (k ey1 v iy0))
("caviar" nil (k ae1 v iy0 aa1 r))
("cavin" nil (k ae1 v ih0 n))
("caviness" nil (k ey1 v iy0 n ih0 s))
("caving" nil (k ey1 v ih0 ng))
("cavins" nil (k ae1 v ih0 n z))
("cavities" nil (k ae1 v ih0 t iy0 z))
("cavitt" nil (k ae1 v ih0 t))
("cavity" nil (k ae1 v ax t iy0))
("cavness" nil (k ae1 v n ih0 s))
("cavort" nil (k ax v ao1 r t))
("cavorting" nil (k ax v ao1 r t ih0 ng))
("caw" nil (k ao1))
("cawley" nil (k ao1 l iy0))
("cawood" nil (k aa1 w uh0 d))
("cawsl" nil (k ao1 s ax l))
("cawthon" nil (k ao1 th ax n))
("cawthorn" nil (k ao1 th er0 n))
("cawthorne" nil (k ao1 th er0 n))
("caxton" nil (k ae1 k s t ax n))
("cay" nil (k ey1))
("caya" nil (k ey1 ax))
("cayce" nil (k ey1 s))
("cayenne" nil (k ay1 eh1 n))
("cayer" nil (k ey1 er0))
("caylor" nil (k ey1 l er0))
("cayman" nil (k ey1 m ax n))
("caymans" nil (k ey1 m ax n z))
("cayne" nil (k ey1 n))
("cayson" nil (k ey1 s ax n))
("cayton" nil (k ey1 t ax n))
("cayuses" nil (k ay1 uw1 s ax z))
("caywood" nil (k ey1 w uh1 d))
("cazares" nil (k aa0 z aa1 r eh0 s))
("cazenove" nil (k ae1 z ax n ow1 v))
("cazier" nil (k ey1 z iy0 er0))
("cd" nil (s iy1 d iy1))
("cdebaca" nil (s iy0 d ih0 b aa1 k ax))
("cea" nil (s iy1))
("ceara" nil (s iy1 r ax))
("cearley" nil (s er1 l iy0))
("ceasar" nil (s ax s aa1 r))
("cease" nil (s iy1 s))
("ceased" nil (s iy1 s t))
("ceasefire" nil (s iy1 s f ay1 er0))
("ceaseless" nil (s iy1 s l ih0 s))
("ceaselessly" nil (s iy1 z l ax s l iy0))
("ceaser" nil (s iy1 s er0))
("ceases" nil (s iy1 s ih0 z))
("ceasing" nil (s iy1 s ih0 ng))
("ceausescu" nil (ch aw0 ch eh1 s k y uw0))
("ceballos" nil (s ey0 b aa1 l ow0 z))
("cebu" nil (s iy0 b uw1))
("cebula" nil (ch eh0 b uw1 l ax))
("cebulski" nil (ch ih0 b ah1 l s k iy0))
("cecala" nil (ch eh0 k aa1 l ax))
("ceccarelli" nil (ch eh0 k er0 eh1 l iy0))
("cecchi" nil (s eh1 k iy0))
("cecchini" nil (ch eh0 k iy1 n iy0))
("cecconi" nil (ch eh0 k ow1 n iy0))
("cece" nil (s iy1 s))
("cecelia" nil (s ih0 s iy1 l y ax))
("cecere" nil (ch eh0 ch eh1 r iy0))
("cech" nil (s eh1 k))
("ceci" nil (s eh1 s iy0))
("cecil" nil (s iy1 s ax l))
("cecile" nil (s ih0 s iy1 l))
("cecilia" nil (s ih0 s iy1 l y ax))
("cecin" nil (s eh1 s ih0 n))
("ceco" nil (s iy1 k ow0))
("cecola" nil (s eh0 k ow1 l ax))
("cecos" nil (s iy1 k ow0 s))
("cedar" nil (s iy1 d er0))
("cedars" nil (s iy1 d er0 z))
("cede" nil (s iy1 d))
("ceded" nil (s iy1 d ax d))
("cedeno" nil (ch eh0 d eh1 n ow0))
("ceder" nil (s iy1 d er0))
("cederberg" nil (s iy1 d er0 b er0 g))
("cedergren" nil (s iy1 d er0 g r eh0 n))
("cederholm" nil (s iy1 d er0 hh ow0 m))
("cederquist" nil (s eh1 d er0 k w ih0 s t))
("cedillo" nil (ch eh0 d ih1 l ow0))
("ceding" nil (s iy1 d ih0 ng))
("cedras" nil (s ey1 d r aa1 s))
("cedric" nil (s eh1 d r ih0 k))
("cedrone" nil (s ey0 d r ow1 n ey0))
("cees" nil (s iy1 z))
("cefalo" nil (ch eh0 f aa1 l ow0))
("cefalu" nil (ch eh0 f aa1 l uw0))
("cegielski" nil (ch ih0 g iy1 l s k iy0))
("ceiling" nil (s iy1 l ih0 ng))
("ceilings" nil (s iy1 l ih0 ng z))
("ceja" nil (s ey1 y ax))
("cejka" nil (ch ey1 k ax))
("cela" nil (s eh1 l ax))
("celadon" nil (s eh1 l ax d aa1 n))
("celandine" nil (s eh1 l ax n d ay1 n))
("celanese" nil (s eh1 l ax n iy1 z))
("celani" nil (ch eh0 l aa1 n iy0))
("celano" nil (ch eh0 l aa1 n ow0))
("celaya" nil (s ey0 l ey1 ax))
("cele" nil (s iy1 l))
("celebrants" nil (s eh1 l ax b r ax n t s))
("celebrate" nil (s eh1 l ax b r ey1 t))
("celebrated" nil (s eh1 l ax b r ey1 t ax d))
("celebrates" nil (s eh1 l ax b r ey1 t s))
("celebrating" nil (s eh1 l ax b r ey1 t ih0 ng))
("celebration" nil (s eh1 l ax b r ey1 sh ax n))
("celebrations" nil (s eh1 l ax b r ey1 sh ax n z))
("celebratory" nil (s ax l eh1 b r ax t ao1 r iy0))
("celebre" nil (s eh1 l ax b r ax))
("celebrities" nil (s ax l eh1 b r ih0 t iy0 z))
("celebrity" nil (s ax l eh1 b r ih0 t iy0))
("celena" nil (ch eh0 l eh1 n ax))
("celene" nil (ch eh1 l iy0 n))
("celentano" nil (ch eh0 l eh0 n t aa1 n ow0))
("celeron" nil (s eh1 l er0 aa0 n))
("celery" nil (s eh1 l er0 iy0))
("celesta" nil (s ih0 l eh1 s t ax))
("celeste" nil (s ax l eh1 s t))
("celestial" nil (s ax l eh1 s ch ax l))
("celestin" nil (s eh1 l ih0 s t ih0 n))
("celestina" nil (ch eh0 l eh0 s t iy1 n ax))
("celestine" nil (ch eh0 l eh0 s t iy1 n iy0))
("celestino" nil (ch eh0 l eh0 s t iy1 n ow0))
("celia" nil (s iy1 l y ax))
("celibacy" nil (s eh1 l ax b ax s iy0))
("celica" nil (s eh1 l ih0 k ax))
("celie" nil (s eh1 l iy0))
("celimene" nil (s eh1 l ih0 m iy1 n))
("celina" nil (s ax l iy1 n ax))
("celinda" nil (ch eh0 l iy1 n d ax))
("celine" nil (s ax l iy1 n))
("celio" nil (s iy1 l iy0 ow0))
("celis" nil (s eh1 l ih0 s))
("cell" nil (s eh1 l))
("cella" nil (s eh1 l ax))
("cellar" nil (s eh1 l er0))
("cellars" nil (s eh1 l er0 z))
("celled" nil (s eh1 l d))
("celli" nil (ch eh1 l iy0))
("cellini" nil (ch eh0 l iy1 n iy0))
("cellist" nil (ch eh1 l ax s t))
("cellmark" nil (s eh1 l m aa1 r k))
("cellnet" nil (s eh1 l n eh1 t))
("cello" nil (ch eh1 l ow0))
("cellophane" nil (s eh1 l ax f ey1 n))
("cellpro" nil (s eh1 l p r ow0))
("cells" nil (s eh1 l z))
("cellstar" nil (s eh1 l s t aa1 r))
("cellucci" nil (ch eh0 l uw1 ch iy0))
("cellular" nil (s eh1 l y ax l er0))
("celluloid" nil (s eh1 l ax l oy1 d))
("cellulosa" nil (s eh1 l uw0 l ow1 s ax))
("cellulose" nil (s eh1 l y ax l ow1 s))
("celmer" nil (s eh1 l m er0))
("celnik" nil (s eh1 l n ih0 k))
("celona" nil (ch eh0 l ow1 n ax))
("celosia" nil (ch eh0 l ow1 s iy0 ax))
("celsius" nil (s eh1 l s iy0 ax s))
("celso" nil (s eh1 l s ow0))
("celt" nil (s eh1 l t))
("celtic" nil (s eh1 l t ih0 k))
("celtics" nil (s eh1 l t ih0 k s))
("celts" nil (s eh1 l t s))
("cement" nil (s ax m eh1 n t))
("cemented" nil (s ax m eh1 n t ax d))
("cementing" nil (s ih0 m eh1 n t ih0 ng))
("cementos" nil (s eh0 m eh1 n t ow0 s))
("cemeteries" nil (s eh1 m ax t eh1 r iy0 z))
("cemetery" nil (s eh1 m ax t eh1 r iy0))
("cemex" nil (k eh1 m eh1 k s))
("cemp" nil (s eh1 m p))
("cencall" nil (s eh1 n s eh1 l))
("cenci" nil (ch eh1 n ch iy0))
("cencor" nil (s eh1 n k ao1 r))
("cendejas" nil (s ey0 n d ey1 y aa0 z))
("cenergy" nil (s eh1 n er0 jh iy0))
("ceniceros" nil (s ey0 n iy0 s eh1 r ow0 z))
("cenith" nil (s eh1 n ih0 th))
("cenozoic" nil (s iy1 n ax z ow1 ih0 k))
("censer" nil (s eh1 n s er0))
("censor" nil (s eh1 n s er0))
("censored" nil (s eh1 n s er0 d))
("censoring" nil (s eh1 n s er0 ih0 ng))
("censors" nil (s eh1 n s er0 z))
("censorship" nil (s eh1 n s er0 sh ih1 p))
("censure" nil (s eh1 n sh er0))
("censured" nil (s eh1 n sh er0 d))
("census" nil (s eh1 n s ax s))
("censuses" nil (s eh1 n s ax s ih0 z))
("cent" nil (s eh1 n t))
("centanni" nil (ch eh0 n t aa1 n iy0))
("centaur" nil (s eh1 n t ao1 r))
("centaurs" nil (s eh1 n t ao1 r z))
("centavos" nil (s eh0 n t aa1 v ow1 s))
("centel" nil (s eh1 n t eh1 l))
("centenarians" nil (s eh1 n t ax n eh1 r iy0 ax n z))
("centenary" nil (s eh1 n t ax n eh1 r iy0))
("centennial" nil (s eh0 n t eh1 n iy0 ax l))
("centeno" nil (ch eh0 n t eh1 n ow0))
("center" nil (s eh1 n t er0))
("centerbanc" nil (s eh1 n t er0 b ae0 ng k))
("centerbank" nil (s eh1 n t er0 b ae1 ng k))
("centered" nil (s eh1 n t er0 d))
("centerfielder" nil (s eh1 n t er0 f iy1 l d er0))
("centerfold" nil (s eh1 n t er0 f ow1 l d))
("centering" nil (s eh1 n t er0 ih0 ng))
("centerior" nil (s eh1 n t ih1 r iy0 er0))
("centerpiece" nil (s eh1 n t er0 p iy1 s))
("centerre" nil (s eh1 n t er0))
("centers" nil (s eh1 n t er0 z))
("centex" nil (s eh1 n t eh1 k s))
("centigrade" nil (s eh1 n t ax g r ey1 d))
("centigram" nil (s eh1 n t ax g r ae1 m))
("centime" nil (s eh1 n t ay1 m))
("centimes" nil (s eh1 n t ay1 m z))
("centimeter" nil (s eh1 n t ax m iy1 t er0))
("centimeters" nil (s eh1 n t ax m iy1 t er0 z))
("centipede" nil (s eh1 n t ih0 p iy1 d))
("centner" nil (s eh1 n t n er0))
("centocor" nil (s eh1 n t ax k ao1 r))
("centofanti" nil (ch eh0 n t ow0 f aa1 n t iy0))
("centola" nil (ch eh0 n t ow1 l ax))
("centoxin" nil (s eh1 n t aa1 k s ih0 n))
("central" nil (s eh1 n t r ax l))
("centrale" nil (s eh0 n t r aa1 l))
("centralia" nil (s eh0 n t r ey1 l iy0 ax))
("centralism" nil (s eh1 n t r ax l ih1 z ax m))
("centralists" nil (s eh1 n t r ax l ih0 s t s))
("centralization" nil (s eh1 n t r ax l ih0 z ey1 sh ax n))
("centralize" nil (s eh1 n t r ax l ay1 z))
("centralized" nil (s eh1 n t r ax l ay1 z d))
("centralizing" nil (s eh1 n t r ax l ay1 z ih0 ng))
("centrally" nil (s eh1 n t r ax l iy0))
("centram" nil (s eh1 n t r ae1 m))
("centre" nil (s eh1 n t er0))
("centrella" nil (s eh1 n t r eh1 l ax))
("centres" nil (s eh1 n t er0 z))
("centrex" nil (s eh1 n t r ax k s))
("centrifugal" nil (s eh0 n t r ih1 f ih0 g ax l))
("centrifuge" nil (s eh1 n t r ax f y uw1 jh))
("centrifuges" nil (s eh1 n t r ax f y uw1 jh ih0 z))
("centrist" nil (s eh1 n t r ih0 s t))
("centrists" nil (s eh1 n t r ih0 s t s))
("centro" nil (s eh1 n t r ow0))
("centromin" nil (s eh1 n t r ax m ih0 n))
("centronics" nil (s eh1 n t r aa1 n ih0 k s))
("centrust" nil (s eh1 n t r ah1 s t))
("cents" nil (s eh1 n t s))
("centum" nil (k eh1 n t ax m))
("centuri" nil (s eh0 n t uh1 r iy0))
("centuries" nil (s eh1 n ch er0 iy0 z))
("centurion" nil (s eh0 n t uh1 r iy0 ax n))
("century" nil (s eh1 n ch er0 iy0))
("cenvill" nil (s eh1 n v ih0 l))
("cep" nil (s eh1 p))
("cepeda" nil (s ey0 p ey1 d ax))
("cepero" nil (s ey0 p eh1 r ow0))
("cephalon" nil (s eh1 f ax l aa1 n))
("cephalopod" nil (s eh1 f ax l ax p aa1 d))
("cephalosporin" nil (s eh1 f ax l ao1 s p er0 ih0 n))
("cephas" nil (s eh1 f ax z))
("cephus" nil (s eh1 f ax s))
("cera" nil (s eh1 r ax))
("ceraceous" nil (s er0 ey1 sh ax s))
("ceradyne" nil (s eh1 r ax d ay1 n))
("cerami" nil (ch er0 aa1 m iy0))
("ceramic" nil (s er0 ae1 m ih0 k))
("ceramics" nil (s er0 ae1 m ih0 k s))
("cerankosky" nil (s eh1 r ax ng k ao1 s k iy0))
("cerasoli" nil (ch er0 aa0 s ow1 l iy0))
("ceravolo" nil (ch er0 aa0 v ow1 l ow0))
("cerbone" nil (ch er1 b ow0 n))
("cercone" nil (ch er0 k ow1 n iy0))
("cerda" nil (ch eh1 r d ax))
("cereal" nil (s ih1 r iy0 ax l))
("cereals" nil (s ih1 r iy0 ax l z))
("cerebral" nil (s eh1 r ax b r ax l))
("cerebrally" nil (s er0 iy1 b r ax l iy0))
("ceredase" nil (s eh1 r ax d ey1 z))
("cereghino" nil (ch er0 eh0 g iy1 n ow0))
("cerelia" nil (ch er0 eh1 l iy0 ax))
("ceremonial" nil (s eh1 r ax m ow1 n iy0 ax l))
("ceremonies" nil (s eh1 r ax m ow1 n iy0 z))
("ceremony" nil (s eh1 r ax m ow1 n iy0))
("ceres" nil (s ih1 r iy0 z))
("cerezo" nil (s eh1 r ey1 z ow0))
("cerf" nil (s er1 f))
("ceridian" nil (s er0 ih1 d iy0 ax n))
("cerino" nil (ch er0 iy1 n ow0))
("cerio" nil (ch eh1 r iy0 ow0))
("cerise" nil (s er0 iy1 s))
("cermak" nil (ch er1 m ax k))
("cern" nil (s er1 n))
("cerna" nil (ch eh1 r n ax))
("cerney" nil (s er1 n iy0))
("cerniglia" nil (ch er0 n iy1 g l iy0 ax))
("cernuda" nil (s er0 n uw1 d ax))
("cerny" nil (s er1 n iy0))
("cerone" nil (ch er0 ow1 n iy0))
("cerra" nil (s eh1 r ax))
("cerrato" nil (ch er0 aa1 t ow0))
("cerreta" nil (ch er0 eh1 t ax))
("cerrito" nil (ch er0 iy1 t ow0))
("cerritos" nil (s eh0 r iy1 t ow0 s))
("cerro" nil (s eh1 r ow0))
("cerrone" nil (ch er0 ow1 n iy0))
("cerruti" nil (ch er0 uw1 t iy0))
("certain" nil (s er1 t ax n))
("certainly" nil (s er1 t ax n l iy0))
("certainteed" nil (s er1 t ax n t iy1 d))
("certainties" nil (s er1 t ax n t iy0 z))
("certainty" nil (s er1 t ax n t iy0))
("certificate" n (s er1 t ah1 f ax k ax t))
("certificate" v (s er1 t ah1 f ax k ey0 t))
("certificates" nil (s er0 t ih1 f ih0 k ax t s))
("certification" nil (s er1 t ax f ax k ey1 sh ax n))
("certifications" nil (s er1 t ax f ax k ey1 sh ax n z))
("certified" nil (s er1 t ax f ay1 d))
("certifies" nil (s er1 t ax f ay1 z))
("certify" nil (s er1 t ax f ay1))
("certifying" nil (s er1 t ax f ay1 ih0 ng))
("certitude" nil (s er1 t ax t uw1 d))
("certo" nil (ch eh1 r t ow0))
("certron" nil (s er1 t r aa0 n))
("cerulli" nil (ch er0 uw1 l iy0))
("cerullo" nil (ch er0 uw1 l ow0))
("cerus" nil (s eh1 r ax s))
("cerutti" nil (ch er0 uw1 t iy0))
("cervantez" nil (s eh0 r v aa1 n t eh0 z))
("cerveceria" nil (s er1 v ax s ih1 r iy0 ax))
("cervenka" nil (s eh0 r v ey1 ng k ax))
("cerveny" nil (ch er0 v iy1 n iy0))
("cervera" nil (ch er0 v eh1 r ax))
("cervesato" nil (s eh1 r v eh0 s aa1 t ow1))
("cerveza" nil (s er1 v ey1 z ax))
("cervi" nil (ch eh1 r v iy0))
("cervical" nil (s er1 v ax k ax l))
("cervini" nil (ch er0 v iy1 n iy0))
("cervix" nil (s er1 v ih0 k s))
("cervone" nil (ch er0 v ow1 n iy0))
("ceryl" nil (s eh1 r ax l))
("cesar" nil (s iy1 z er0))
("cesare" nil (ch ey0 z aa1 r ey0))
("cesario" nil (ch eh0 s aa1 r iy0 ow0))
("cesaro" nil (ch eh0 s aa1 r ow0))
("cesarz" nil (s ey1 s aa0 r z))
("cesena" nil (ch eh0 s eh1 n ax))
("cesium" nil (s iy1 z iy0 ax m))
("cespedes" nil (s ey0 s p ey1 d eh0 s))
("cessation" nil (s eh1 s ey1 sh ax n))
("cessna" nil (s eh1 s n ax))
("cesspool" nil (s eh1 s p uw1 l))
("cestaro" nil (ch eh0 s t aa1 r ow0))
("cetacean" nil (s ih0 t ey1 sh ax n))
("cetec" nil (s iy1 t eh1 k))
("cetera" nil (s eh1 t er0 ax))
("cetus" nil (s iy1 t ax s))
("cevallos" nil (s ey0 v aa1 l ow0 z))
("cevaxs" nil (s eh1 v ae0 k s ih0 z))
("ceylon" nil (s ih0 l aa1 n))
("cezanne" nil (s eh1 z ae1 n))
("cha" nil (ch aa1))
("chablis" nil (sh ax b l iy1))
("chabon" nil (ch ey1 b ax n))
("chabot" nil (sh ax b ow1))
("chacabuco" nil (ch ae1 k ax b y uw1 k ow0))
("chace" nil (ch ey1 s))
("chachere" nil (sh ax sh ih1 r))
("chacin" nil (sh ey1 s ih0 n))
("chacko" nil (ch ae1 k ow0))
("chacon" nil (ch ae1 k ax n))
("chad" nil (ch ae1 d))
("chadbourne" nil (sh ax d b uh1 r n))
("chadd" nil (ch ae1 d))
("chadderdon" nil (ch ax d er1 d ax n))
("chaddock" nil (ch ae1 d ax k))
("chadian" nil (ch ey1 d iy0 ax n))
("chadick" nil (ch ae1 d ih0 k))
("chadli" nil (ch ae1 d l iy0))
("chadron" nil (ch ae1 d r ax n))
("chadwell" nil (ch ae1 d w eh1 l))
("chadwick" nil (ch ae1 d w ih0 k))
("chae" nil (ch ay1))
("chaebol" nil (ch ey1 b ax l))
("chafe" nil (ch ey1 f))
("chafed" nil (ch ey1 f t))
("chafee" nil (ch ae1 f iy0))
("chafes" nil (ch ey1 f s))
("chaff" nil (ch ae1 f))
("chaffee" nil (ch ae1 f iy0))
("chaffin" nil (ch ae1 f ih0 n))
("chaffins" nil (ch ae1 f ih0 n z))
("chaffy" nil (ch ae1 f iy0))
("chafin" nil (ch ae1 f ih0 n))
("chafing" nil (ch ey1 f ih0 ng))
("chagall" nil (ch ae1 g ax l))
("chagnon" nil (ch ae1 g n ax n))
("chagrin" nil (sh ax g r ih1 n))
("chagrined" nil (sh ax g r ih1 n d))
("chai" nil (ch ay1))
("chaidez" nil (ch aa0 iy1 d eh0 z))
("chaiken" nil (ch ey1 k ax n))
("chaikin" nil (ch ey1 k ih0 n))
("chain" nil (ch ey1 n))
("chained" nil (ch ey1 n d))
("chaining" nil (ch ey1 n ih0 ng))
("chains" nil (ch ey1 n z))
("chainsaw" nil (ch ey1 n s ao1))
("chainsaws" nil (ch ey1 n s ao1 z))
("chair" nil (ch eh1 r))
("chaired" nil (ch eh1 r d))
("chaires" nil (sh eh1 r z))
("chairez" nil (ch aa0 ih1 r eh0 z))
("chairing" nil (ch eh1 r ih0 ng))
("chairman" nil (ch eh1 r m ax n))
("chairmanship" nil (ch eh1 r m ax n sh ih1 p))
("chairmanships" nil (ch eh1 r m ax n sh ih1 p s))
("chairmen" nil (ch eh1 r m ax n))
("chairmen" nil (ch eh1 r m ih0 n))
("chairs" nil (ch eh1 r z))
("chairwoman" nil (ch eh1 r w uh1 m ax n))
("chaise" nil (sh ey1 z))
("chaisson" nil (ch ey1 s ax n))
("chait" nil (ch ey1 t))
("chajet" nil (ch ae1 jh ax t))
("chalabi" nil (ch ax l aa1 b iy0))
("chalasani" nil (ch ae1 l ax s ae1 n iy0))
("chalet" nil (sh ae1 l ey1))
("chalets" nil (sh ax l ey1 z))
("chalfant" nil (ch ae1 f ax n t))
("chalfin" nil (ch ae1 l f ih0 n))
("chalifoux" nil (sh ae1 l ih0 f uw0))
("chalk" nil (ch aa1 k))
("chalked" nil (ch ao1 k t))
("chalker" nil (ch ao1 k er0))
("chalking" nil (ch ao1 k ih0 ng))
("chalkley" nil (ch ae1 l k l iy0))
("chalks" nil (ch ao1 k s))
("challender" nil (ch ax l eh1 n d er0))
("challenge" nil (ch ae1 l ax n jh))
("challenged" nil (ch ae1 l ax jh d))
("challenger" nil (ch ae1 l ax n jh er0))
("challengers" nil (ch ae1 l ax n jh er0 z))
("challenges" nil (ch ae1 l ax n jh ih0 z))
("challenging" nil (ch ae1 l ax n jh ih0 ng))
("challis" nil (ch ae1 l ih0 s))
("chalmers" nil (ch aa1 m er0 z))
("chalmette" nil (sh ae0 l m eh1 t))
("chaloupka" nil (ch ax l uw1 p k ax))
("chaloux" nil (sh ax l uw1))
("chalsty" nil (ch ae1 l s t iy0))
("chalupa" nil (ch ax l uw1 p ax))
("cham" nil (ch ae1 m))
("chamber" nil (ch ey1 m b er0))
("chambered" nil (ch ey1 m b er0 d))
("chamberlain" nil (ch ey1 m b er0 l ax n))
("chamberland" nil (ch ae1 m b er0 l ax n d))
("chamberlayne" nil (ch ey1 m b er0 l ey1 n))
("chamberlin" nil (ch ey1 m b er0 l ih0 n))
("chambers" nil (ch ey1 m b er0 z))
("chamblee" nil (ch ae1 m b l iy0))
("chambless" nil (sh ax m b l iy1 s))
("chamblin" nil (ch ae1 m b l ih0 n))
("chambliss" nil (ch ae1 m b l ih0 s))
("chambon" nil (ch ae1 m b ax n))
("chameleon" nil (ch ax m eh1 l iy0 ax n))
("chamlee" nil (ch ae1 m l iy0))
("chamness" nil (ch ae1 m n ih0 s))
("chamorro" nil (ch ax m ao1 r ow0))
("champ" nil (ch ae1 m p))
("champa" nil (k aa1 m p ax))
("champagne" nil (sh ae0 m p ey1 n))
("champagnes" nil (sh ae0 m p ey1 n z))
("champaign" nil (ch ae0 m p ey1 n))
("champeau" nil (sh ae0 m p ow1))
("champine" nil (ch ae1 m p ay1 n))
("champion" nil (ch ae1 m p iy0 ax n))
("championed" nil (ch ae1 m p iy0 ax n d))
("championing" nil (ch ae1 m p iy0 ax n ih0 ng))
("champions" nil (ch ae1 m p iy0 ax n z))
("championship" nil (ch ae1 m p iy0 ax n sh ih1 p))
("championships" nil (ch ae1 m p iy0 ax n sh ih1 p s))
("champlain" nil (sh ae0 m p l ey1 n))
("champlin" nil (ch ae1 m p l ih0 n))
("champney" nil (ch ae1 m p n iy0))
("champoux" nil (sh ae0 m p uw1))
("champs" nil (ch ae1 m p s))
("chan" nil (ch ae1 n))
("chana" nil (ch ae1 n ax))
("chance" nil (ch ae1 n s))
("chancellor" nil (ch ae1 n s ax l er0))
("chancellors" nil (ch ae1 n s ax l er0 z))
("chancery" nil (ch ae1 n s er0 iy0))
("chances" nil (ch ae1 n s ax z))
("chancey" nil (ch ae1 n s iy0))
("chancy" nil (ch ae1 n s iy0))
("chand" nil (ch ae1 n d))
("chanda" nil (ch ae1 n d ax))
("chandelier" nil (sh ae0 n d ax l ih1 r))
("chandeliers" nil (sh ae1 n d ax l ih1 z))
("chandler" nil (ch ae1 n d l er0))
("chandley" nil (ch ae1 n d l iy0))
("chandon" nil (ch ae1 n d ih0 n))
("chandra" nil (ch ae1 n d r ax))
("chandross" nil (ch ae0 n d r ao1 s))
("chanel" nil (sh ax n eh1 l))
("chaney" nil (ch ey1 n iy0))
("chang" nil (ch ae1 ng))
("changchun" nil (ch aa1 ng ch uh1 n))
("change" nil (ch ey1 n jh))
("changeable" nil (ch ey1 n jh ax b ax l))
("changed" nil (ch ey1 n jh d))
("changeover" nil (ch ey1 n jh ow1 v er0))
("changeovers" nil (ch ey1 n jh ow1 v er0 z))
("changers" nil (ch ey1 n jh er0 z))
("changes" nil (ch ey1 n jh ax z))
("changing" nil (ch ey1 n jh ih0 ng))
("changsho" nil (ch ae1 ng sh ow1))
("chanin" nil (ch ae1 n ih0 n))
("chanley" nil (ch ae1 n l iy0))
("channel" nil (ch ae1 n ax l))
("channeled" nil (ch ae1 n ax l d))
("channeling" nil (ch ae1 n ax l ih0 ng))
("channell" nil (ch ae1 n ax l))
("channels" nil (ch ae1 n ax l z))
("channing" nil (ch ae1 n ih0 ng))
("channon" nil (ch ae1 n ax n))
("chanos" nil (ch aa1 n ow0 s))
("chant" nil (ch ae1 n t))
("chantal" nil (ch ae1 n t ax l))
("chanted" nil (ch ae1 n t ih0 d))
("chantilly" nil (sh ae1 n t ih1 l iy0))
("chanting" nil (ch ae1 n t ih0 ng))
("chants" nil (ch ae1 n t s))
("chanukah" nil (hh aa1 n ax k ax))
("chany" nil (ch ey1 n iy0))
("chao" nil (ch aw1))
("chaos" nil (k ey1 aa0 s))
("chaotic" nil (k ey0 aa1 t ih0 k))
("chap" nil (ch ae1 p))
("chapa" nil (ch aa1 p ax))
("chaparral" nil (sh ae1 p er0 ae1 l))
("chaparro" nil (k aa0 p aa1 r ow0))
("chapas" nil (ch aa1 p ax s))
("chapdelaine" nil (sh ae1 p d ih0 l ey0 n))
("chapek" nil (ch ae1 p ih0 k))
("chapel" nil (ch ae1 p ax l))
("chapell" nil (sh ax p eh1 l))
("chapelle" nil (sh ax p eh1 l))
("chaperon" nil (sh ae1 p er0 ow1 n))
("chaperone" nil (sh ae1 p er0 ow1 n))
("chapin" nil (sh ax p ae1 n))
("chaplain" nil (ch ae1 p l ax n))
("chaplains" nil (ch ae1 p l ax n z))
("chaplin" nil (ch ae1 p l ax n))
("chapman" nil (ch ae1 p m ax n))
("chapnick" nil (ch ae1 p n ih0 k))
("chapoton" nil (ch ae1 p ow0 aa1 n))
("chapp" nil (ch ae1 p))
("chappaquiddick" nil (ch ae1 p ax k w ih1 d ih0 k))
("chappel" nil (ch ae1 p ax l))
("chappelear" nil (sh ae1 p ih0 l er0))
("chappell" nil (ch ae1 p ax l))
("chappelle" nil (sh ax p eh1 l))
("chapple" nil (ch ae1 p ax l))
("chappuis" nil (sh ae1 p uw0 ih0 z))
("chaps" nil (ch ae1 p s))
("chapter" nil (ch ae1 p t er0))
("chapters" nil (ch ae1 p t er0 z))
("chaput" nil (ch ae1 p ax t))
("char" nil (ch aa1 r))
("character" nil (k eh1 r ih0 k t er0))
("characteristic" nil (k eh1 r ax k t er0 ih1 s t ih0 k))
("characteristically" nil (k eh1 r ax k t er0 ih1 s t ih0 k l iy0))
("characteristics" nil (k eh1 r ax k t er0 ih1 s t ih0 k s))
("characterization" nil (k eh1 r ax k t er0 ax z ey1 sh ax n))
("characterizations" nil (ch eh1 r ax k t er0 ih0 z ey1 sh ax n z))
("characterize" nil (k eh1 r ax k t er0 ay1 z))
("characterized" nil (k eh1 r ax k t er0 ay1 z d))
("characterizes" nil (k eh1 r ax k t er0 ay1 z ax z))
("characterizing" nil (k eh1 r ih0 k t er0 ay1 z ih0 ng))
("characters" nil (k ae1 r ih0 k t er0 z))
("charade" nil (sh er0 ey1 d))
("charades" nil (sh er0 ey1 d z))
("charalambos" nil (ch aa1 r ax l aa1 m b ow0 s))
("charasse" nil (ch eh1 r ae0 s))
("charboneau" nil (sh aa1 r b ax n ow0))
("charbonneau" nil (sh aa1 r b ax n ow1))
("charbonnet" nil (sh aa1 r b ax n ih0 t))
("charcoal" nil (ch aa1 r k ow1 l))
("charcoals" nil (ch aa1 r k ow1 l z))
("chard" nil (ch aa1 r d))
("chardonnay" nil (ch aa0 r d aa1 n ey0))
("chardonnays" nil (ch aa0 r d aa1 n ih0 z))
("charen" nil (ch aa1 r ax n))
("charest" nil (ch aa0 r ey1 ih0 s t))
("charette" nil (sh er0 eh1 t))
("charge" nil (ch aa1 r jh))
("chargeable" nil (ch aa1 r jh ax b ax l))
("charged" nil (ch aa1 r jh d))
("charger" nil (ch aa1 r jh er0))
("chargers" nil (ch aa1 r jh er0 z))
("charges" nil (ch aa1 r jh ax z))
("chargeurs" nil (ch aa0 r g y uw1 r z))
("charging" nil (ch aa1 r jh ih0 ng))
("chargit" nil (ch aa1 r jh ih0 t))
("chargois" nil (sh aa0 r g w aa1))
("chariot" nil (ch eh1 r iy0 ax t))
("chariots" nil (ch eh1 r iy0 ax t s))
("charisma" nil (k er0 ih1 z m ax))
("charismatic" nil (k eh0 r ax z m ae1 t ih0 k))
("charismatics" nil (k eh1 r ih0 z m ae1 t ih0 k s))
("charissa" nil (k aa0 r iy1 s ax))
("charita" nil (k aa0 r iy1 t ax))
("charitable" nil (ch ae1 r ax t ax b ax l))
("charitably" nil (ch eh1 r ih0 t ax b l iy0))
("charities" nil (ch eh1 r ax t iy0 z))
("charity" nil (ch eh1 r ih0 t iy0))
("charla" nil (ch aa1 r l ax))
("charland" nil (ch aa1 r l ax n d))
("charlatans" nil (sh aa1 r l ax t ax n z))
("charlayne" nil (sh aa0 r l ey1 n))
("charlebois" nil (sh aa1 r l ih0 b w aa0))
("charleen" nil (ch aa0 r l iy1 n))
("charlemagne" nil (sh aa1 r l ax m ey1 n))
("charlene" nil (sh aa0 r l iy1 n))
("charles" nil (ch aa1 r l z))
("charleston" nil (ch aa1 r l s t ax n))
("charlestown" nil (ch aa1 r l z t aw1 n))
("charlesworth" nil (ch aa1 r l ih0 s w er0 th))
("charlet" nil (ch aa1 r l ih0 t))
("charley" nil (ch aa1 r l iy0))
("charlie" nil (ch aa1 r l iy0))
("charlier" nil (ch aa1 r l iy0 er0))
("charline" nil (sh aa0 r l iy1 n))
("charlot" nil (ch aa1 r l ax t))
("charlotte" nil (sh aa1 r l ax t))
("charlottesville" nil (sh aa1 r l ax t s v ih1 l))
("charlottetown" nil (sh aa1 r l ax t t aw1 n))
("charls" nil (ch aa1 r l z))
("charlson" nil (ch aa1 r l s ax n))
("charlton" nil (ch aa1 r l t ax n))
("charm" nil (ch aa1 r m))
("charmain" nil (sh aa0 r m ey1 n))
("charmaine" nil (sh aa0 r m ey1 n))
("charmed" nil (ch aa1 r m d))
("charmer" nil (ch aa1 r m er0))
("charmers" nil (ch aa1 r m er0 z))
("charmian" nil (ch aa1 r m iy0 ax n))
("charming" nil (ch aa1 r m ih0 ng))
("charmingly" nil (ch aa1 r m ih0 ng l iy0))
("charmion" nil (ch aa1 r m iy0 ax n))
("charms" nil (ch aa1 r m z))
("charney" nil (ch aa1 r n iy0))
("charnley" nil (ch aa1 r n l iy0))
("charnock" nil (ch aa1 r n ax k))
("charon" nil (k eh1 r ax n))
("charpentier" nil (sh aa1 r p ax n t ih1 r))
("charpie" nil (ch aa1 r p iy0))
("charred" nil (ch aa1 r d))
("charren" nil (ch eh1 r ax n))
("charrier" nil (ch ae1 r iy0 er0))
("charron" nil (ch eh1 r ax n))
("charry" nil (ch ae1 r iy0))
("chart" nil (ch aa1 r t))
("charted" nil (ch aa1 r t ax d))
("charter" nil (ch aa1 r t er0))
("chartered" nil (ch aa1 r t er0 d))
("charterhouse" nil (ch aa1 r t er0 hh aw1 s))
("chartering" nil (ch aa1 r t er0 ih0 ng))
("charters" nil (ch aa1 r t er0 z))
("chartier" nil (ch aa1 r t iy0 er0))
("charting" nil (ch aa1 r t ih0 ng))
("chartist" nil (ch aa1 r t ih0 s t))
("chartists" nil (ch aa1 r t ih0 s t s))
("chartrand" nil (ch aa1 r t r ax n d))
("chartres" nil (ch aa1 r t r iy0 z))
("charts" nil (ch aa1 r t s))
("chartwell" nil (ch aa1 r t w eh1 l))
("charvat" nil (ch aa1 r v ax t))
("chary" nil (ch aa1 r iy0))
("charyl" nil (ch ae1 r ax l))
("chas" nil (ch aa1 r l z))
("chase" nil (ch ey1 s))
("chased" nil (ch ey1 s t))
("chasen" nil (ch ey1 s ax n))
("chaser" nil (ch ey1 s er0))
("chasers" nil (ch ey1 s er0 z))
("chases" nil (ch ey1 s ax z))
("chasin" nil (ch ae1 s ih0 n))
("chasing" nil (ch ey1 s ih0 ng))
("chaska" nil (ch ae1 s k ax))
("chasm" nil (k ae1 z ax m))
("chason" nil (ch ae1 s ax n))
("chasse" nil (ch ae1 s))
("chassis" nil (ch ae1 s iy0))
("chastain" nil (sh ax s t ey1 n))
("chaste" nil (ch ey1 s t))
("chasteen" nil (ch ax s t iy1 n))
("chastened" nil (ch ey1 s ax n d))
("chastise" nil (ch ae0 s t ay1 z))
("chastised" nil (ch ae0 s t ay1 z d))
("chastises" nil (ch ae0 s t ay1 z ih0 z))
("chastising" nil (ch ae0 s t ay1 z ih0 ng))
("chastity" nil (ch ae1 s t ax t iy0))
("chat" nil (ch ae1 t))
("chateau" nil (sh ae0 t ow1))
("chateaux" nil (sh ax t ow1))
("chatfield" nil (ch ae1 t f iy1 l d))
("chatham" nil (ch ae1 t ax m))
("chatichai" nil (ch ae1 t ih0 ch ay1))
("chatihachi" nil (ch ae1 t ih0 hh aa1 ch iy0))
("chatman" nil (ch ae1 t m ax n))
("chatmon" nil (ch ae1 t m ax n))
("chats" nil (ch ae1 t s))
("chatswood" nil (ch ae1 t s w uh1 d))
("chatsworth" nil (ch ae1 t s w er1 th))
("chattahoochee" nil (ch ae1 t ax hh uw1 ch iy0))
("chattanooga" nil (ch ae1 t ax n uw1 g ax))
("chatted" nil (ch ae1 t ax d))
("chattel" nil (ch ae1 t ax l))
("chatter" nil (ch ae1 t er0))
("chattering" nil (ch ae1 t er0 ih0 ng))
("chatterjee" nil (ch ax t er1 jh iy0))
("chatterton" nil (ch ae1 t er0 t ax n))
("chattin" nil (ch ae1 t ih0 n))
("chatting" nil (ch ae1 t ih0 ng))
("chatty" nil (ch ae1 t iy0))
("chatwal" nil (ch ae1 t w ao1 l))
("chatwin" nil (ch ae1 t w ih0 n))
("chatz" nil (ch ae1 t s))
("chau" nil (sh ow1))
("chaucer" nil (ch ao1 s er0))
("chaudhry" nil (ch ao1 d r iy0))
("chaudoin" nil (sh ow0 d oy1 n))
("chauffeur" nil (sh ow0 f er1))
("chauffeured" nil (sh ow0 f er1 d))
("chauffeurs" nil (sh ow0 f er1 z))
("chaumet" nil (ch ao1 m ih0 t))
("chaunce" nil (ch ao1 n s))
("chauncey" nil (ch ao1 n s iy0))
("chauncy" nil (ch ao1 n s iy0))
("chaus" nil (ch aw1 s))
("chausse" nil (ch aw1 s))
("chaussee" nil (ch aw1 s iy0))
("chautauqua" nil (sh ax t ao1 k w ax))
("chautauquans" nil (sh ax t ao1 k w ax n z))
("chauvin" nil (sh ow0 v ae1 n))
("chauvinism" nil (sh ow1 v ax n ih1 z ax m))
("chauvinist" nil (sh ow1 v ax n ax s t))
("chauvinistic" nil (ch ow1 v ax n ih1 s t ih0 k))
("chavarria" nil (ch ax v ae1 r iy0 ax))
("chavers" nil (ch ey1 v er0 z))
("chaves" nil (ch ey1 v z))
("chavez" nil (sh aa1 v eh0 z))
("chavin" nil (ch ey1 v ih0 n))
("chavira" nil (k aa0 v ih1 r ax))
("chavis" nil (ch ae1 v ih0 s))
("chavous" nil (sh ax v ao1 s))
("chaw" nil (ch ao1))
("chawla" nil (ch ao1 l ax))
("chazanoff" nil (ch ae1 z ax n ao0 f))
("chazen" nil (ch ey1 z ax n))
("chazov" nil (ch ae1 z aa0 v))
("che" nil (ch ey1))
("chea" nil (ch iy1))
("cheadle" nil (ch iy1 d ax l))
("cheane" nil (ch iy1 n))
("cheaney" nil (ch iy1 n iy0))
("cheap" nil (ch iy1 p))
("cheapen" nil (ch iy1 p ax n))
("cheapened" nil (ch iy1 p ax n d))
("cheapening" nil (ch iy1 p ax n ih0 ng))
("cheaper" nil (ch iy1 p er0))
("cheapest" nil (ch iy1 p ax s t))
("cheaply" nil (ch iy1 p l iy0))
("cheapness" nil (ch iy1 p n ax s))
("cheapo" nil (ch iy1 p ow1))
("cheapskate" nil (ch iy1 p s k ey1 t))
("cheat" nil (ch iy1 t))
("cheated" nil (ch iy1 t ax d))
("cheaters" nil (ch iy1 t er0 z))
("cheatham" nil (ch iy1 t ax m))
("cheating" nil (ch iy1 t ih0 ng))
("cheats" nil (ch iy1 t s))
("cheatum" nil (ch iy1 t ax m))
("cheatwood" nil (ch iy1 t w uh1 d))
("chebrikov" nil (ch eh1 b r ih0 k aa1 v))
("checchi" nil (ch eh1 k iy0))
("chechen" nil (ch eh1 ch ih0 n))
("chechens" nil (ch eh1 ch ih0 n z))
("chechnya" nil (ch eh1 ch n iy0 ax))
("chechnyan" nil (ch eh1 ch n iy0 ax n))
("check" nil (ch eh1 k))
("checkbook" nil (ch eh1 k b uh1 k))
("checkbooks" nil (ch eh1 k b uh1 k s))
("checked" nil (ch eh1 k t))
("checker" nil (ch eh1 k er0))
("checkerboard" nil (ch eh1 k er0 b ao1 r d))
("checkerboarding" nil (ch eh1 k er0 b ao1 r d ih0 ng))
("checkerboards" nil (ch eh1 k er0 b ao1 r d z))
("checkered" nil (ch eh1 k er0 d))
("checkers" nil (ch eh1 k er0 z))
("checkerspot" nil (ch eh1 k er0 s p aa1 t))
("checketts" nil (ch eh1 k ih0 t s))
("checking" nil (ch eh1 k ih0 ng))
("checklist" nil (ch eh1 k l ih1 s t))
("checkmate" nil (ch eh1 k m ey1 t))
("checkoff" nil (ch eh1 k ao1 f))
("checkout" nil (ch eh1 k aw1 t))
("checkouts" nil (ch eh1 k aw1 t s))
("checkpoint" nil (ch eh1 k p oy1 n t))
("checkpoints" nil (ch eh1 k p oy1 n t s))
("checks" nil (ch eh1 k s))
("checkup" nil (ch eh1 k ah1 p))
("checkups" nil (ch eh1 k ah1 p s))
("cheddar" nil (ch eh1 d er0))
("cheddars" nil (ch eh1 d er0 z))
("chedester" nil (ch eh1 d ih0 s t er0))
("chee" nil (ch iy1))
("cheech" nil (ch iy1 ch))
("cheek" nil (ch iy1 k))
("cheeked" nil (ch iy1 k t))
("cheeks" nil (ch iy1 k s))
("cheeky" nil (ch iy1 k iy0))
("cheely" nil (ch iy1 l iy0))
("cheer" nil (ch ih1 r))
("cheered" nil (ch ih1 r d))
("cheerful" nil (ch ih1 r f ax l))
("cheerfully" nil (ch ih1 r f ax l iy0))
("cheering" nil (ch ih1 r ih0 ng))
("cheerios" nil (ch ih1 r iy0 ow0 s))
("cheerleader" nil (ch ih1 r l iy1 d er0))
("cheerleaders" nil (ch ih1 r l iy1 d er0 z))
("cheerleading" nil (ch ih1 r l iy1 d ih0 ng))
("cheers" nil (ch ih1 r z))
("cheery" nil (ch ih1 r iy0))
("cheese" nil (ch iy1 z))
("cheeseburger" nil (ch iy1 z b er0 g er0))
("cheeseburgers" nil (ch iy1 z b er0 g er0 z))
("cheesecake" nil (ch iy1 z k ey1 k))
("cheeseman" nil (ch iy1 z m ax n))
("cheeses" nil (ch iy1 z ih0 z))
("cheesier" nil (ch iy1 z iy0 er0))
("cheesiest" nil (ch iy1 z iy0 ih0 s t))
("cheesman" nil (ch iy1 z m ax n))
("cheesy" nil (ch iy1 z iy0))
("cheetah" nil (ch iy1 t ax))
("cheetal" nil (ch iy1 t ax l))
("cheetham" nil (ch iy1 th ax m))
("cheever" nil (ch iy1 v er0))
("chef" nil (sh eh1 f))
("chefitz" nil (ch eh1 f ih0 t s))
("chefs" nil (sh eh1 f s))
("cheil" nil (ch ay1 l))
("chek" nil (ch eh1 k))
("chekhov" nil (ch eh1 k aa0 v))
("chelan" nil (ch eh1 l ax n))
("chelette" nil (sh ih0 l eh1 t))
("chelf" nil (ch eh1 l f))
("chell" nil (ch eh1 l))
("chellis" nil (ch eh1 l ih0 s))
("chelmsford" nil (ch eh1 l m s f er0 d))
("chelsea" nil (ch eh1 l s iy0))
("chem" nil (k eh1 m))
("chema" nil (ch eh1 m aa0))
("chemclear" nil (ch eh1 m k l ih1 r))
("chemdesign" nil (ch eh1 m d ax z ay1 n))
("chemed" nil (k eh1 m eh0 d))
("chemetron" nil (ch eh1 m ax t r aa0 n))
("chemfix" nil (ch eh1 m f ih0 k s))
("chemical" nil (k eh1 m ax k ax l))
("chemically" nil (k eh1 m ax k l iy0))
("chemicals" nil (k eh1 m ih0 k ax l z))
("chemie" nil (ch eh1 m iy0))
("chemins" nil (ch eh1 m ih0 n z))
("chemise" nil (sh ax m iy1 z))
("chemist" nil (k eh1 m ih0 s t))
("chemistry" nil (k eh1 m ax s t r iy0))
("chemists" nil (k eh1 m ax s t s))
("chemlawn" nil (k eh1 m l ao1 n))
("chemotherapy" nil (k iy1 m ow0 th eh1 r ax p iy0))
("chen" nil (ch eh1 n))
("chenard" nil (ch eh1 n er0 d))
("chenault" nil (sh ih0 n ow1))
("chenette" nil (sh ih0 n eh1 t))
("chenevert" nil (ch eh1 n ih0 v er0 t))
("cheney" nil (ch ey1 n iy0))
("cheng" nil (ch eh1 ng))
("chengdu" nil (ch eh0 ng d uw1))
("chengxiang" nil (ch eh1 ng sh aa1 ng))
("chenier" nil (ch iy1 n iy0 er0))
("chennault" nil (ch eh1 n ao0 l t))
("chenoweth" nil (ch eh1 n aw0 eh0 th))
("cheong" nil (ch iy1 aa0 ng))
("cher" nil (sh eh1 r))
("cheramie" nil (ch eh1 r ax m iy0))
("cheri" nil (k eh1 r iy0))
("cherico" nil (k er0 iy1 k ow0))
("cherie" nil (ch ih1 r iy0))
("cherish" nil (ch eh1 r ih0 sh))
("cherished" nil (ch eh1 r ih0 sh t))
("cherishes" nil (ch eh1 r ih0 sh ih0 z))
("chermak" nil (ch er1 m ax k))
("chern" nil (ch er1 n))
("cherne" nil (ch er1 n))
("chernenko" nil (ch er0 n eh1 ng k ow0))
("cherney" nil (ch er1 n iy0))
("chernick" nil (ch er1 n ih0 k))
("chernin" nil (ch er1 n ih0 n))
("chernobyl" nil (ch er0 n ow1 b ax l))
("chernoff" nil (k er1 n ao0 f))
("chernomyrdin" nil (ch eh1 r n ow0 m ih1 r d ax n))
("chernow" nil (ch er1 n ow0))
("cherny" nil (ch er1 n iy0))
("cherokee" nil (ch eh1 r ax k iy1))
("cherokees" nil (ch eh1 r ax k iy1 z))
("cherrapunji" nil (ch eh1 r ax p ah1 n jh iy0))
("cherrier" nil (ch eh1 r iy0 er0))
("cherries" nil (ch eh1 r iy0 z))
("cherrington" nil (ch eh1 r ih0 ng t ax n))
("cherry" nil (ch eh1 r iy0))
("cherrystones" nil (ch eh1 r iy0 s t ow1 n z))
("chert" nil (ch er1 t))
("chertkow" nil (ch er1 t k aw0))
("chertoff" nil (ch er1 t aa0 f))
("cherts" nil (ch er1 t s))
("cherub" nil (ch eh1 r ax b))
("cherubic" nil (ch er0 uw1 b ih0 k))
("cherubini" nil (k er0 uw0 b iy1 n iy0))
("cherubs" nil (ch eh1 r ax b z))
("cherumirdan" nil (ch eh1 r ax m ih1 r d ax n))
("chervenak" nil (ch er1 v ih0 n ae0 k))
("chery" nil (ch eh1 r iy0))
("cheryl" nil (sh eh1 r ax l))
("ches" nil (ch eh1 s))
("chesapeake" nil (ch eh1 s ax p iy1 k))
("chesbro" nil (k eh1 s b r ow0))
("chesebro" nil (k eh0 s eh1 b r ow0))
("chesebrough" nil (ch eh1 s ax b r uw0))
("chesher" nil (ch eh1 sh er0))
("cheshier" nil (ch eh1 sh iy0 er0))
("cheshire" nil (ch eh1 sh er0))
("chesler" nil (ch eh1 s ax l er0))
("chesley" nil (ch eh1 s l iy0))
("cheslock" nil (ch eh1 s l ax k))
("chesney" nil (ch eh1 s n iy0))
("chesnut" nil (ch eh1 s n ah1 t))
("chesnutt" nil (sh ih0 s n ah1 t))
("chess" nil (ch eh1 s))
("chessboard" nil (ch eh1 s b ao1 r d))
("chesser" nil (ch eh1 s er0))
("chessher" nil (ch eh1 sh er0))
("chesshir" nil (sh ih0 sh er1))
("chessman" nil (ch eh1 s m ax n))
("chesson" nil (ch eh1 s ax n))
("chest" nil (ch eh1 s t))
("chestang" nil (ch eh1 s t ax ng))
("chested" nil (ch eh1 s t ax d))
("chester" nil (ch eh1 s t er0))
("chesterfield" nil (ch eh1 s t er0 f iy1 l d))
("chesterman" nil (ch eh1 s t er0 m ax n))
("chesters" nil (ch eh1 s t er0 z))
("chesterton" nil (ch eh1 s t er0 t ax n))
("chestman" nil (ch eh1 s t m ax n))
("chestnut" nil (ch eh1 s n ah1 t))
("chestnuts" nil (ch eh1 s n ax t s))
("chestnutt" nil (ch eh1 s t n ax t))
("cheston" nil (ch eh1 s t ax n))
("chests" nil (ch eh1 s t s))
("chet" nil (ch eh1 t))
("chetwin" nil (ch eh1 t w ih0 n))
("cheung" nil (ch y uw1 ng))
("cheuse" nil (ch uw1 s))
("cheuvront" nil (sh uw0 v r aa1 n t))
("chevalier" nil (sh eh1 v ax l ih1 r))
("cheverly" nil (ch eh1 v er0 l iy0))
("cheves" nil (ch iy1 v z))
("chevette" nil (sh ax v eh1 t))
("chevies" nil (ch eh1 v iy0 z))
("cheviot" nil (sh eh1 v iy0 ax t))
("cheviots" nil (ch iy1 v iy0 ax t s))
("chevis" nil (ch eh1 v ih0 s))
("chevrette" nil (sh ih0 v r eh1 t))
("chevrier" nil (ch eh1 v er0 iy0 er0))
("chevrolet" nil (sh eh1 v r ax l ey1))
("chevrolets" nil (sh eh1 v r ax l ey1 z))
("chevron" nil (sh eh1 v r ax n))
("chevy" nil (ch eh1 v iy0))
("chevys" nil (ch eh1 v ih0 s))
("chew" nil (ch uw1))
("chewed" nil (ch uw1 d))
("chewing" nil (ch uw1 ih0 ng))
("chewning" nil (ch uw1 n ih0 ng))
("chews" nil (ch uw1 z))
("chewy" nil (ch uw1 iy0))
("cheyenne" nil (sh ay0 ae1 n))
("cheyne" nil (ch ey1 n))
("cheyney" nil (ch ey1 n iy0))
("chez" nil (ch eh1 z))
("chi" nil (k ay1))
("chia" nil (ch iy1 ax))
("chianese" nil (k iy0 aa0 n ey1 z iy0))
("chiang" nil (ch ae1 ng))
("chianti" nil (ch iy0 ae1 n t iy0))
("chiapas" nil (ch iy0 aa1 p ax s))
("chiapparone" nil (ch iy0 ae1 p er0 ow1 n))
("chiappetta" nil (k iy0 aa0 p eh1 t ax))
("chiappone" nil (k iy0 aa0 p ow1 n iy0))
("chiara" nil (k y aa1 r ax))
("chiaramonte" nil (k iy0 aa0 r aa0 m ow1 n t iy0))
("chiarella" nil (k iy0 aa0 r eh1 l ax))
("chiarelli" nil (k iy0 aa0 r eh1 l iy0))
("chiarello" nil (k iy0 aa0 r eh1 l ow0))
("chiarenza" nil (k iy0 aa0 r eh1 n z ax))
("chiaro" nil (k iy0 aa1 r ow0))
("chiasson" nil (ch iy0 aa1 s ax n))
("chiat" nil (ch iy1 ae0 t))
("chiavetta" nil (k iy0 aa0 v eh1 t ax))
("chiba" nil (ch iy1 b ax))
("chic" nil (sh iy1 k))
("chicago" nil (sh ax k aa1 g ow1))
("chicagoan" nil (ch ih1 k ax g ow1 n))
("chicagoans" nil (sh ax k aa1 g ow1 ax n z))
("chicanery" nil (sh ih0 k ey1 n er0 iy0))
("chicano" nil (ch ih0 k aa1 n ow0))
("chichauha" nil (ch iy1 ch aw1 ax))
("chichester" nil (ch ih1 ch eh0 s t er0))
("chick" nil (ch ih1 k))
("chickasaw" nil (ch ih1 k ax s ao1))
("chickasaws" nil (ch ih1 k ax s ao1 z))
("chicken" nil (ch ih1 k ax n))
("chickened" nil (ch ih1 k ax n d))
("chickens" nil (ch ih1 k ax n z))
("chickering" nil (ch ih1 k er0 ih0 ng))
("chicks" nil (ch ih1 k s))
("chico" nil (ch iy1 k ow1))
("chicoine" nil (ch ih0 k oy1 n))
("chicots" nil (ch ih1 k ax t s))
("chide" nil (ch ay1 d))
("chided" nil (ch ay1 d ih0 d))
("chides" nil (ch ay1 d z))
("chidester" nil (ch ih1 d ih0 s t er0))
("chiding" nil (ch ay1 d ih0 ng))
("chidsey" nil (ch ih1 d z iy0))
("chief" nil (ch iy1 f))
("chiefdom" nil (ch iy1 f d ax m))
("chieffo" nil (k iy1 f ow0))
("chiefly" nil (ch iy1 f l iy0))
("chiefs" nil (ch iy1 f s))
("chieftain" nil (ch iy1 f t ax n))
("chieftains" nil (ch iy1 f t ax n z))
("chien" nil (ch eh1 n))
("chiengmai" nil (ch eh1 ng m ay1))
("chiesa" nil (k iy1 s ax))
("chiggers" nil (ch ih1 g er0 z))
("chihuahua" nil (ch ax w aa1 w aa1))
("chikane" nil (ch ih0 k aa1 n ey0))
("chikos" nil (ch iy1 k ow0 s))
("chilcoat" nil (ch ih1 l k ow1 t))
("chilcote" nil (ch ih1 l k ow1 t))
("chilcott" nil (ch ih1 l k ax t))
("chilcutt" nil (ch ih1 l k ax t))
("child" nil (ch ay1 l d))
("childbearing" nil (ch ay1 l d b eh1 r ih0 ng))
("childbirth" nil (ch ay1 l d b er1 th))
("childcare" nil (ch ay1 l d k eh1 r))
("childcraft" nil (ch ay1 l d k r ae1 f t))
("childe" nil (ch ih1 l d))
("childers" nil (ch ih1 l d er0 z))
("childhood" nil (ch ay1 l d hh uh1 d))
("childish" nil (ch ay1 l d ih0 sh))
("childless" nil (ch ay1 l d l ax s))
("childlike" nil (ch ay1 l d l ay1 k))
("childree" nil (ch ay0 l d r iy1))
("children" nil (ch ih1 l d r ax n))
("childrens" nil (ch ih1 l d r ax n z))
("childres" nil (ch ay1 l d er0 z))
("childress" nil (ch ih1 l d r ih0 s))
("childrey" nil (ch ih1 l d r iy0))
("childs" nil (ch ay1 l d z))
("chile" nil (ch ih1 l iy0))
("chilean" nil (ch ih1 l iy0 ax n))
("chileans" nil (ch ih1 l iy0 ax n z))
("chiles" nil (ch ay1 l z))
("chili" nil (ch ih1 l iy0))
("chilies" nil (ch ih1 l iy0 z))
("chill" nil (ch ih1 l))
("chilled" nil (ch ih1 l d))
("chillemi" nil (k iy0 l eh1 m iy0))
("chillier" nil (ch ih1 l iy0 er0))
("chilling" nil (ch ih1 l ih0 ng))
("chillingly" nil (ch ih1 l ih0 ng l iy0))
("chills" nil (ch ih1 l z))
("chilly" nil (ch ih1 l iy0))
("chilmark" nil (ch ih1 l m aa1 r k))
("chilson" nil (ch ih1 l s ax n))
("chilton" nil (ch ih1 l t ax n))
("chime" nil (ch ay1 m))
("chimed" nil (ch ay1 m d))
("chimenti" nil (ch ih0 m eh1 n t iy0))
("chimento" nil (ch ih0 m eh1 n t ow0))
("chimera" nil (ch ih0 m eh1 r ax))
("chimerine" nil (ch ih1 m er0 iy1 n))
("chimes" nil (ch ay1 m z))
("chimicles" nil (ch ih1 m ih0 k ax l z))
("chimie" nil (ch ih1 m iy0))
("chimney" nil (ch ih1 m n iy0))
("chimneys" nil (ch ih1 m n iy0 z))
("chimp" nil (ch ih1 m p))
("chimpanzee" nil (ch ih0 m p ae1 n z iy0))
("chimpanzees" nil (ch ih0 m p ae1 n z iy0 z))
("chimps" nil (ch ih1 m p s))
("chin" nil (ch ih1 n))
("china" nil (ch ay1 n ax))
("chinatown" nil (ch ay1 n ax t aw1 n))
("chinchilla" nil (ch ih0 n ch ih1 l ax))
("chinen" nil (ch ih1 n ax n))
("chinese" nil (ch ay0 n iy1 z))
("ching" nil (ch ih1 ng))
("chinh" nil (ch ih1 n))
("chink" nil (ch ih1 ng k))
("chinks" nil (ch ih1 ng k s))
("chinn" nil (ch ih1 n))
("chinnici" nil (k iy0 n iy1 ch iy0))
("chinnock" nil (ch ih1 n ax k))
("chino" nil (ch iy1 n ow0))
("chinook" nil (sh ih0 n uh1 k))
("chinooks" nil (ch ih0 n uh1 k s))
("chinoy" nil (ch ih1 n oy1))
("chintung" nil (ch ih1 n t ax ng))
("chintzy" nil (ch ih1 n t s iy0))
("chiodo" nil (k iy0 ow1 d ow0))
("chip" nil (ch ih1 p))
("chipboard" nil (ch ih1 p b ao1 r d))
("chipcom" nil (ch ih1 p k aa1 m))
("chipello" nil (ch ih0 p eh1 l ow0))
("chipley" nil (ch ih1 p l iy0))
("chipmaker" nil (ch ih1 p m ey1 k er0))
("chipmakers" nil (ch ih1 p m ey1 k er0 z))
("chipman" nil (ch ih1 p m ax n))
("chipote" nil (ch ih0 p ow1 t))
("chipped" nil (ch ih1 p t))
("chipper" nil (ch ih1 p er0))
("chippewa" nil (ch ih1 p ax w aa1))
("chipping" nil (ch ih1 p ih0 ng))
("chipps" nil (ch ih1 p s))
("chips" nil (ch ih1 p s))
("chipsoft" nil (ch ih1 p s ao1 f t))
("chiquita" nil (k ih0 k w iy1 t ax))
("chirac" nil (sh ih0 r ae1 k))
("chirco" nil (k ih1 r k ow0))
("chirico" nil (k ih0 r iy1 k ow0))
("chiron" nil (ch ay1 r ax n))
("chiropractic" nil (k ay1 r ow0 p r ae1 k t ih0 k))
("chiropractor" nil (k ay1 r ax p r ae1 k t er0))
("chiropractors" nil (k ay1 r ax p r ae1 k t er0 z))
("chirp" nil (ch er1 p))
("chirping" nil (ch er1 p ih0 ng))
("chirps" nil (ch er1 p s))
("chirpy" nil (ch er1 p iy0))
("chisam" nil (ch ih1 s ax m))
("chisel" nil (ch ih1 z ax l))
("chiseled" nil (ch ih1 z ax l d))
("chisels" nil (ch ih1 z ax l z))
("chisenhall" nil (ch ih0 s eh1 n hh ax l))
("chisholm" nil (ch ih1 z ax m))
("chisler" nil (ch ih1 s ax l er0))
("chism" nil (ch ih1 z ax m))
("chisman" nil (ch ih1 s m ax n))
("chismar" nil (ch ih1 z m er0))
("chisolm" nil (ch ih1 s ow0 m))
("chisom" nil (ch ih1 s ax m))
("chissano" nil (ch ih0 s aa1 n ow0))
("chisum" nil (ch ih1 z ax m))
("chit" nil (ch ih1 t))
("chitchat" nil (ch ih1 t ch ae1 t))
("chitinous" nil (k ay1 t ax n ax s))
("chitra" nil (ch ih1 t r ax))
("chitrao" nil (ch ih1 t r aw0))
("chittenden" nil (ch ih1 t ax n d ax n))
("chittick" nil (ch ih1 t ih0 k))
("chittum" nil (ch ih1 t ax m))
("chitty" nil (ch ih1 t iy0))
("chitwood" nil (ch ih1 t w uh1 d))
("chiu" nil (ch uw1))
("chiusano" nil (k iy0 uw0 s aa1 n ow0))
("chivalry" nil (sh ih1 v ax l r iy0))
("chivas" nil (ch iy1 v ax s))
("chivers" nil (ch ay1 v er0 z))
("chives" nil (ch ay1 v z))
("chiyoda" nil (ch ih0 y ow1 d ax))
("chizek" nil (ch ih1 z eh0 k))
("chizmar" nil (ch ih1 z m er0))
("chlebowski" nil (ch l ih0 b ao1 f s k iy0))
("chloe" nil (k l ow1 iy0))
("chlorate" nil (k l ao1 r ey0 t))
("chlordane" nil (k l ao1 r d ey1 n))
("chloride" nil (k l ao1 r ay0 d))
("chlorinated" nil (k l ao1 r ax n ey1 t ax d))
("chlorinating" nil (k l ao1 r ax n ey1 t ih0 ng))
("chlorine" nil (k l ao1 r iy0 n))
("chloris" nil (k l ao1 r ih0 s))
("chlorofluorocarbon" nil (k l ao0 r ow0 f l ao1 r ow0 k aa1 r b aa0 n))
("chlorofluorocarbons" nil (k l ao1 r ow0 f l ao1 r ow0 k aa1 r b aa0 n z))
("chloroform" nil (k l ao1 r ax f ao1 r m))
("chlorophyll" nil (k l ao1 r ax f ih0 l))
("chloroplasts" nil (k l ao1 r ax p l ae1 s t s))
("chloroprene" nil (k l ao1 r ax p r iy1 n))
("chmiel" nil (ch ax m iy1 l))
("chmielewski" nil (ch ax m ax l eh1 f s k iy0))
("chmura" nil (ch ax m uh1 r ax))
("cho" nil (ch ow1))
("choat" nil (ch ow1 t))
("choate" nil (ch ow1 t))
("chock" nil (ch aa1 k))
("chocolat" nil (ch aa1 k l ax t))
("chocolate" nil (ch ao1 k l ax t))
("chocolates" nil (ch ao1 k l ax t s))
("chodorow" nil (ch ow1 d er0 ow0))
("choe" nil (ch ow1))
("choi" nil (ch oy1))
("choice" nil (ch oy1 s))
("choices" nil (ch oy1 s ax z))
("choicest" nil (ch oy1 s ax s t))
("choiniere" nil (sh oy1 n iy0 eh0 r))
("choinski" nil (ch oy1 n s k iy0))
("choir" nil (k w ay1 er0))
("choirs" nil (k w ay1 r z))
("chojnacki" nil (ch ax y n aa1 t s k iy0))
("chojnowski" nil (ch ax y n ao1 f s k iy0))
("chok" nil (ch aa1 k))
("choke" nil (ch ow1 k))
("choked" nil (ch ow1 k t))
("chokes" nil (ch ow1 k s))
("choking" nil (ch ow1 k ih0 ng))
("cholera" nil (k aa1 l er0 ax))
("choleric" nil (k aa1 l er0 ih0 k))
("cholesterol" nil (k ax l eh1 s t er0 ao1 l))
("cholestyramine" nil (ch ow0 l eh1 s t ih0 r ax m ay1 n))
("cholet" nil (ch ow1 l ax t))
("cholewa" nil (ch ax l uw1 ax))
("cholla" nil (ch aa1 l ax))
("choma" nil (ch ow1 m ax))
("chomp" nil (ch aa1 m p))
("chomping" nil (ch aa1 m p ih0 ng))
("chon" nil (ch aa1 n))
("chong" nil (ch ao1 ng))
("chongqing" nil (ch ao1 ng k ih1 ng))
("chonko" nil (ch aa1 ng k ow0))
("chontales" nil (sh aa1 n t ey1 l z))
("choo" nil (ch uw1))
("choong" nil (ch uw1 ng))
("choose" nil (ch uw1 z))
("chooses" nil (ch uw1 z ax z))
("choosing" nil (ch uw1 z ih0 ng))
("choosy" nil (ch uw1 z iy0))
("chop" nil (ch aa1 p))
("choper" nil (ch ow1 p er0))
("chopin" nil (sh ow1 p ae0 n))
("choplin" nil (ch aa1 p l ih0 n))
("chopp" nil (ch aa1 p))
("chopped" nil (ch aa1 p t))
("chopper" nil (ch aa1 p er0))
("choppers" nil (ch aa1 p er0 z))
("chopping" nil (ch aa1 p ih0 ng))
("choppy" nil (ch aa1 p iy0))
("chopra" nil (ch aa1 p r ax))
("chops" nil (ch aa1 p s))
("chopsticks" nil (ch aa1 p s t ih1 k s))
("chopsuey" nil (ch aa1 p s uw1 iy0))
("choquette" nil (sh ax k eh1 t))
("choral" nil (k ao1 r ax l))
("chorals" nil (k ao1 r ax l z))
("chorba" nil (k ao1 r b ax))
("chord" nil (k ao1 r d))
("chordates" nil (k ao1 r d ey1 t s))
("chords" nil (k ao1 r d z))
("chore" nil (ch ao1 r))
("choreograph" nil (k ao1 r iy0 ax g r ae1 f))
("choreographed" nil (k ao1 r iy0 ax g r ae1 f t))
("choreographer" nil (k ao1 r iy0 aa1 g r ax f er0))
("choreographers" nil (k ao1 r iy0 aa1 g r ax f er0 z))
("choreographic" nil (k ao1 r iy0 ax g r ae1 f ih0 k))
("choreography" nil (k ao1 r iy0 aa1 g r ax f iy0))
("chores" nil (ch ao1 r z))
("choric" nil (k ao1 r ih0 k))
("chorney" nil (k ao1 r n iy0))
("chortled" nil (ch ao1 r t ax l d))
("chortles" nil (ch ao1 r t ax l z))
("chortling" nil (ch ao1 r t ax l ih0 ng))
("chorus" nil (k ao1 r ax s))
("choruses" nil (k ao1 r ax s ih0 z))
("chose" nil (ch ow1 z))
("chosen" nil (ch ow1 z ax n))
("chosun" nil (ch ow1 z ax n))
("chou" nil (ch uw1))
("chouinard" nil (sh w iy0 n aa1 r d))
("chovan" nil (ch ow1 v ax n))
("chovanec" nil (ch ax v ae1 n ih0 k))
("chow" nil (ch aw1))
("chowder" nil (ch aw1 d er0))
("chowdhury" nil (ch aw1 d hh y uw0 r iy0))
("chowning" nil (ch aw1 n ih0 ng))
("chows" nil (ch aw1 z))
("choy" nil (ch oy1))
("choyce" nil (ch oy1 s))
("chrest" nil (k r eh1 s t))
("chrestman" nil (k r eh1 s t m ax n))
("chretien" nil (sh r ih0 t iy1 n))
("chriboniko" nil (ch r iy1 b ow0 n iy1 k ow0))
("chris" nil (k r ih1 s))
("chrisco" nil (k r iy1 s k ow0))
("chriscoe" nil (k r ih1 s k ow0))
("chrisman" nil (k r ih1 s m ax n))
("chrismer" nil (k er1 ih0 z ax m er0))
("chrismon" nil (k r ih1 z m ax n))
("chrisp" nil (k r ih1 s p))
("chriss" nil (k r ih1 s))
("chrissie" nil (k r ih1 s iy0))
("chrissy" nil (k r ih1 s iy0))
("christ" nil (k r ay1 s t))
("christa" nil (k r ih1 s t ax))
("christabelle" nil (sh r ih1 s t ax b ax l))
("christakos" nil (k r ih1 s t ax k ow0 z))
("christal" nil (k r ih1 s t ax l))
("christel" nil (k r ih1 s t ax l))
("christen" nil (k r ih1 s ax n))
("christenberry" nil (k r ih1 s ax n b eh1 r iy0))
("christenbury" nil (k r ih1 s ax n b eh1 r iy0))
("christendom" nil (k r ih1 s ax n d ax m))
("christened" nil (k r ih1 s ax n d))
("christening" nil (k r ih1 s ax n ih0 ng))
("christensen" nil (k r ih1 s t ax n s ax n))
("christenson" nil (k r ih1 s t ih0 n s ax n))
("christeson" nil (k r ih1 s t ih0 s ax n))
("christi" nil (k r ih1 s t iy0))
("christiaan" nil (k r ih1 s t iy0 aa1 n))
("christian" nil (k r ih1 s ch ax n))
("christiana" nil (k r ih1 s t iy0 ae1 n ax))
("christiane" nil (k r ih0 s t iy0 aa1 n))
("christianity" nil (k r ih1 s ch iy0 ae1 n ih0 t iy0))
("christianization" nil (k r ih1 s ch ax n ax z ey1 sh ax n))
("christianized" nil (k r ih1 s ch ax n ay1 z d))
("christiano" nil (k r iy0 s t iy0 aa1 n ow0))
("christians" nil (k r ih1 s ch ax n z))
("christiansen" nil (k r ih1 s ch ax n s ax n))
("christianson" nil (k r ih1 s ch ax n s ax n))
("christic" nil (k r ih1 s t ih0 k))
("christie" nil (k r ih1 s t iy0))
("christies" nil (k r ih1 s t iy0 z))
("christina" nil (k r ih0 s t iy1 n ax))
("christine" nil (k r ih0 s t iy1 n))
("christison" nil (k r ih1 s t ih0 s ax n))
("christlieb" nil (k r ih1 s t l iy1 b))
("christman" nil (k r ih1 s t m ax n))
("christmann" nil (k r ih1 s t m ax n))
("christmas" nil (k r ih1 s m ax s))
("christmastime" nil (k r ih1 s t m ax s t ay1 m))
("christner" nil (k r ih1 s t n er0))
("christo" nil (k r ih1 s t ow0))
("christoff" nil (k r ih1 s t ao0 f))
("christoffel" nil (k r ih1 s t ax f eh0 l))
("christoffersen" nil (k r ih0 s t ax f er1 s ax n))
("christofferson" nil (k r ih0 s t aa1 f er0 s ax n))
("christon" nil (k r ih1 s t ax n))
("christoph" nil (k r ih1 s t ao0 f))
("christophe" nil (k r ih0 s t ao1 f))
("christophel" nil (k r ih1 s t ax f eh0 l))
("christopher" nil (k r ih1 s t ax f er0))
("christophersen" nil (k r ih0 s t ax f er1 s ax n))
("christopherson" nil (k r ih0 s t aa1 f er0 s ax n))
("christopoulos" nil (k r ih0 s t aa1 p ax l ih0 s))
("christy" nil (k r ih1 s t iy0))
("chriswell" nil (k r ih1 s w eh1 l))
("chrobak" nil (k r ow1 b ax k))
("chromakalim" nil (ch r ow1 m ax k aa1 l iy1 m))
("chromalloy" nil (k r ow0 m ae1 l oy0))
("chrome" nil (k r ow1 m))
("chrominance" nil (k r ow1 m ax n ax n s))
("chromium" nil (k r ow1 m iy0 ax m))
("chromosome" nil (k r ow1 m ax s ow1 m))
("chromosomes" nil (k r ow1 m ax z ow1 m z))
("chronar" nil (k r aa1 n er0))
("chronic" nil (k r aa1 n ih0 k))
("chronically" nil (k r aa1 n ih0 k ax l iy0))
("chronicle" nil (k r aa1 n ax k ax l))
("chronicled" nil (k r aa1 n ih0 k ax l d))
("chronicler" nil (k r aa1 n ih0 k l er0))
("chroniclers" nil (k r aa1 n ax k l er0 z))
("chronicles" nil (k r aa1 n ax k ax l z))
("chronicling" nil (k r aa1 n ih0 k l ih0 ng))
("chronis" nil (k r ow1 n ih0 s))
("chronister" nil (k r aa1 n ih0 s t er0))
("chronological" nil (k r aa1 n ax l aa1 jh ih0 k ax l))
("chronologically" nil (k r aa1 n ax l aa1 jh ih0 k l iy0))
("chronologies" nil (k r ax n aa1 l ax jh iy0 z))
("chronology" nil (k r ax n aa1 l ax jh iy0))
("chronowitz" nil (k r aa1 n ax w ih0 t s))
("chrostowski" nil (k r ax s t ao1 f s k iy0))
("chrusciel" nil (k r ah1 s iy1 l))
("chrysanthemum" nil (k r ih0 s ae1 n th ax m ax m))
("chrysanthemums" nil (k r ih0 s ae1 n th ax m ax m z))
("chryseis" nil (k r ih1 s ax z))
("chrysler" nil (k r ay1 s l er0))
("chryslers" nil (k r ay1 s l er0 z))
("chryst" nil (ch r ih1 s t))
("chrystal" nil (k r ih1 s t ax l))
("chrzan" nil (k er1 z ae1 n))
("chrzanowski" nil (k er1 z ax n ao1 f s k iy0))
("chseing" nil (ch ey1 ng))
("chu" nil (ch uw1))
("chua" nil (k uw1 ax))
("chuah" nil (ch uw1 aa0))
("chuan" nil (ch uw1 aa1 n))
("chuang" nil (ch ae1 ng))
("chuba" nil (ch uw1 b ax))
("chubais" nil (ch uw1 b ay1))
("chubb" nil (ch ah1 b))
("chubbuck" nil (ch ah1 b ax k))
("chubby" nil (ch ah1 b iy0))
("chubu" nil (ch uw1 b uw0))
("chuck" nil (ch ah1 k))
("chucked" nil (ch ah1 k t))
("chuckie" nil (ch ah1 k iy0))
("chucking" nil (ch ah1 k ih0 ng))
("chuckle" nil (ch ah1 k ax l))
("chuckled" nil (ch ah1 k ax l d))
("chuckles" nil (ch ah1 k ax l z))
("chuckling" nil (ch ah1 k l ih0 ng))
("chudler" nil (ch ah1 d l er0))
("chudy" nil (ch uw1 d iy0))
("chudzik" nil (ch ah1 d z ih0 k))
("chudzinski" nil (ch ax jh ih1 n s k iy0))
("chug" nil (ch ah1 g))
("chugai" nil (ch uw0 g ay1))
("chugged" nil (ch ah1 g d))
("chugging" nil (ch ah1 g ih0 ng))
("chugoku" nil (ch uw0 g ow1 k uw1))
("chui" nil (k uw1 ih0))
("chujitsuya" nil (ch uw1 jh iy0 t s uw0 y ax))
("chukchi" nil (ch uw1 k ch iy0))
("chul" nil (ch ah1 l))
("chula" nil (ch uw1 l ax))
("chum" nil (ch ah1 m))
("chumbley" nil (ch ah1 m b l iy0))
("chumley" nil (ch ah1 m l iy0))
("chummy" nil (ch ah1 m iy0))
("chumney" nil (ch ah1 m n iy0))
("chump" nil (ch ah1 m p))
("chums" nil (ch ah1 m z))
("chun" nil (ch ah1 n))
("chung" nil (ch ah1 ng))
("chunk" nil (ch ah1 ng k))
("chunks" nil (ch ah1 ng k s))
("chunky" nil (ch ah1 ng k iy0))
("chunn" nil (ch ah1 n))
("chunnel" nil (ch ah1 n ax l))
("chupp" nil (ch ah1 p))
("chura" nil (ch uh1 r ax))
("church" nil (ch er1 ch))
("churches" nil (ch er1 ch ax z))
("churchgoers" nil (ch er1 ch g ow1 er0 z))
("churchill" nil (ch er1 ch ih0 l))
("churchman" nil (ch er1 ch m ax n))
("churchmen" nil (ch er1 ch m ax n))
("churchwell" nil (ch er1 ch w eh1 l))
("churchyard" nil (ch er1 ch y aa1 r d))
("churilla" nil (ch er0 ih1 l ax))
("churkin" nil (ch er1 k ax n))
("churlish" nil (ch er1 l ih0 sh))
("churn" nil (ch er1 n))
("churned" nil (ch er1 n d))
("churning" nil (ch er1 n ih0 ng))
("churns" nil (ch er1 n z))
("chusmir" nil (ch uh0 s m ih1 r))
("chustz" nil (ch ah1 s t s))
("chute" nil (sh uw1 t))
("chutes" nil (sh uw1 t s))
("chutney" nil (ch ah1 t n iy0))
("chutzpah" nil (ch ah1 t s p aa1))
("chynoweth" nil (ch ih1 n aw0 eh0 th))
("chyron" nil (ch ay1 r ax n))
("cia" nil (s iy1 ax))
("ciaccia" nil (ch iy0 ax ch iy1 ax))
("ciaccio" nil (ch ao1 ch iy0 ow0))
("ciampa" nil (ch ao1 m p ax))
("ciampi" nil (ch ao1 m p iy0))
("cian" nil (sh iy1 n))
("cianci" nil (ch ao1 n ch iy0))
("ciancio" nil (ch ao1 n ch iy0 ow0))
("cianciola" nil (ch ao1 n ch ow0 l ax))
("cianciolo" nil (ch ao1 n ch ow0 l ow0))
("cianciulli" nil (ch ao1 n ch uw0 l iy0))
("ciani" nil (ch ao1 n iy0))
("ciano" nil (ch iy0 aa1 n ow0))
("ciaobella" nil (ch aw1 b eh1 l ax))
("ciaramella" nil (ch er0 aa0 m eh1 l ax))
("ciaramitaro" nil (ch er1 aa0 m iy0 t aa0 r ow0))
("ciaravino" nil (ch er0 aa0 v iy1 n ow0))
("ciardi" nil (ch er1 d iy0))
("ciarlo" nil (ch er1 l ow0))
("ciavarella" nil (ch ax v aa0 r eh1 l ax))
("ciba" nil (s iy1 b ax))
("ciborowski" nil (ch ih0 b er0 ao1 f s k iy0))
("cibro" nil (s ih1 b r ow0))
("cibula" nil (ch iy0 b uw1 l ax))
("cicada" nil (s ax k ey1 d ax))
("cicadas" nil (s ih0 k ey1 d ax z))
("cicala" nil (s ih0 k aa1 l ax))
("cicalese" nil (ch iy0 k aa0 l ey1 z iy0))
("ciccarelli" nil (ch iy0 k er0 eh1 l iy0))
("ciccarello" nil (ch iy0 k er0 eh1 l ow0))
("ciccarone" nil (s ih1 k er0 ow1 n))
("cicco" nil (s ih1 k ow0))
("ciccone" nil (ch iy0 k ow1 n iy0))
("cicely" nil (s ih1 s ax l iy0))
("cicero" nil (s ih1 s er0 ow1))
("cicerone" nil (s ih1 s er0 ow1 n))
("cichocki" nil (s ih0 ch aa1 k iy0))
("cichon" nil (s ih1 ch ax n))
("cichowski" nil (ch ih0 hh ao1 f s k iy0))
("cichy" nil (s ih1 ch iy0))
("cicily" nil (ch ih1 ch ax l iy0))
("cicio" nil (s ih1 s iy0 ow0))
("cicippio" nil (s ih0 s ih1 p iy0 ow0))
("cid" nil (s ih1 d))
("cider" nil (s ay1 d er0))
("cie" nil (s iy1))
("ciel" nil (s iy1 l))
("ciera" nil (s iy1 r ax))
("cieri" nil (s iy1 r iy0))
("ciesielski" nil (ch eh0 s iy1 l s k iy0))
("ciesla" nil (ch eh1 s l ax))
("cieslak" nil (ch eh1 s l ax k))
("cieslewicz" nil (ch eh1 s l ih0 v ih0 ch))
("cieslik" nil (ch eh1 s l ih0 k))
("cieslinski" nil (ch eh0 s l ih1 n s k iy0))
("cifelli" nil (s ih0 f eh1 l iy0))
("cifra" nil (s ih1 f r ax))
("cifuentes" nil (s iy0 f w eh1 n t eh0 s))
("ciga" nil (s iy1 g ax))
("cigar" nil (s ih0 g aa1 r))
("cigarette" nil (s ih1 g er0 eh1 t))
("cigarettes" nil (s ih1 g er0 eh1 t s))
("cigars" nil (s ih0 g aa1 r z))
("cigna" nil (s ih1 g n ax))
("cihak" nil (s ih1 hh ax k))
("cihlar" nil (s ih1 l er0))
("cilag" nil (s ih1 l ae0 g))
("cilcorp" nil (s ih1 l k ao0 r p))
("cilento" nil (s ih0 l eh1 n t ow0))
("ciliates" nil (s ih1 l iy0 ax t s))
("ciliberto" nil (ch iy0 l iy0 b eh1 r t ow0))
("cilicia" nil (s ih0 l ih1 sh ax))
("ciller" nil (s ih1 l er0))
("cilley" nil (s ih1 l iy0))
("cillo" nil (s ih1 l ow0))
("cilluffo" nil (s ih0 l uw1 f ow0))
("cilva" nil (s ih1 l v ax))
("cima" nil (ch iy1 m ax))
("cimaglia" nil (s ih0 m ae1 g l iy0 ax))
("cimarron" nil (s ih1 m er0 aa1 n))
("ciments" nil (s ih0 m eh1 n t s))
("ciminero" nil (s iy1 m ih0 n eh1 r ow0))
("cimini" nil (ch iy0 m iy1 n iy0))
("cimino" nil (ch iy0 m iy1 n ow0))
("cimmino" nil (ch iy0 m iy1 n ow0))
("cimo" nil (ch iy1 m ow0))
("cimorelli" nil (ch iy0 m ao0 r eh1 l iy0))
("cina" nil (ch iy1 n ax))
("cinch" nil (s ih1 n ch))
("cinched" nil (s ih1 n ch t))
("cincinnati" nil (s ih1 n s ax n ae1 t iy0))
("cinco" nil (s ih1 ng k ow0))
("cincotta" nil (ch iy0 n k ow1 t ax))
("cinder" nil (s ih1 n d er0))
("cinderella" nil (s ih1 n d er0 eh1 l ax))
("cinders" nil (s ih1 n d er0 z))
("cindie" nil (s ay1 n d iy0))
("cindric" nil (s ih1 n d r ih0 k))
("cindy" nil (s ih1 n d iy0))
("cinelli" nil (s ih0 n eh1 l iy0))
("cinema" nil (s ih1 n ax m ax))
("cinemark" nil (s ih1 n ax m aa1 k))
("cinemas" nil (s ih1 n ax m ax z))
("cinematic" nil (s ih1 n ax m ae1 t ih0 k))
("cinematographer" nil (s ih1 n ih0 m ax t aa1 g r ax f er0))
("cinematography" nil (s ih1 n ih0 m ax t aa1 g r ax f iy0))
("cinemax" nil (s ih1 n ax m ae0 k s))
("cineplex" nil (s ih1 n ax p l eh1 k s))
("cinergy" nil (s ih1 n er0 jh iy0))
("cini" nil (ch iy1 n iy0))
("cinnabar" nil (s ih1 n ax b aa1 r))
("cinnaminson" nil (s ih1 n ax m ih0 n s ax n))
("cinnamon" nil (s ih1 n ax m ax n))
("cino" nil (ch iy1 n ow0))
("cinq" nil (s ih1 ng k))
("cinque" nil (s ih1 ng k))
("cinquemani" nil (ch iy0 n k w eh0 m aa1 n iy0))
("cinram" nil (s ih1 n r ae0 m))
("cinthie" nil (s ih1 n th iy0))
("cintron" nil (s ih1 n t r ax n))
("ciocca" nil (ch ow1 k ax))
("cioffi" nil (ch iy0 ow1 f iy0))
("ciolek" nil (ch iy0 ow1 l eh0 k))
("ciolino" nil (ch ow0 l iy1 n ow0))
("ciotti" nil (ch ow1 t iy0))
("cipher" nil (s ay1 f er0))
("cipolla" nil (s ih0 p aa1 l ax))
("cipollone" nil (s ih1 p ax l ow1 n))
("cipriani" nil (ch iy0 p r iy0 aa1 n iy0))
("cipriano" nil (ch iy0 p r iy0 aa1 n ow0))
("cira" nil (s er1 ax))
("ciraulo" nil (s er0 ao1 l ow0))
("circa" nil (s er1 k ax))
("circadian" nil (s er0 k ey1 d iy0 ax n))
("circle" nil (s er1 k ax l))
("circled" nil (s er1 k ax l d))
("circles" nil (s er1 k ax l z))
("circling" nil (s er1 k ax l ih0 ng))
("circon" nil (s er1 k aa0 n))
("circuit" nil (s er1 k ax t))
("circuited" nil (s er1 k ax t ih0 d))
("circuitous" nil (s er0 k y uw1 ih0 t ax s))
("circuitry" nil (s er1 k ax t r iy0))
("circuits" nil (s er1 k ax t s))
("circular" nil (s er1 k y ax l er0))
("circularly" nil (s er1 k y ax l er0 l iy0))
("circulars" nil (s er1 k y ax l er0 z))
("circulate" nil (s er1 k y ax l ey1 t))
("circulated" nil (s er1 k y ax l ey1 t ax d))
("circulates" nil (s er1 k y ax l ey1 t s))
("circulating" nil (s er1 k y ax l ey1 t ih0 ng))
("circulation" nil (s er1 k y ax l ey1 sh ax n))
("circulations" nil (s er1 k y ax l ey1 sh ax n z))
("circulatory" nil (s er1 k y ax l ax t ao1 r iy0))
("circumcision" nil (s er1 k ax m s ih1 zh ax n))
("circumference" nil (s er0 k ah1 m f r ax n s))
("circumscribed" nil (s er1 k ax m s k r ay1 b d))
("circumspect" nil (s er1 k ax m s p eh1 k t))
("circumspection" nil (s er1 k ax m s p eh1 k sh ax n))
("circumstance" nil (s er1 k ax m s t ae1 n s))
("circumstances" nil (s er1 k ax m s t ae1 n s ax z))
("circumstantial" nil (s er1 k ax m s t ae1 n ch ax l))
("circumvent" nil (s er1 k ax m v eh1 n t))
("circumvented" nil (s er1 k ax m v eh1 n t ih0 d))
("circumventing" nil (s er1 k ax m v eh1 n t ih0 ng))
("circumvention" nil (s er1 k ax m v eh1 n ch ax n))
("circumvents" nil (s er1 k ax m v eh1 n t s))
("circus" nil (s er1 k ax s))
("circuses" nil (s er1 k ax s ax z))
("cirelli" nil (s ih0 r eh1 l iy0))
("ciresi" nil (s er0 eh1 s iy0))
("ciriaco" nil (s ih1 r iy0 aa1 k ow0))
("ciriello" nil (s er0 iy0 eh1 l ow0))
("cirigliano" nil (s er0 iy0 g l iy0 aa1 n ow0))
("cirillo" nil (s ih0 r ih1 l ow0))
("cirincione" nil (s er0 iy0 n ch ow1 n iy0))
("cirino" nil (s er0 iy1 n ow0))
("cirone" nil (s ih0 r ow1 n))
("ciros" nil (s ih1 r ow1 z))
("cirque" nil (s er1 k))
("cirrhosis" nil (s er0 ow1 s ax s))
("cirrincione" nil (s er0 r iy0 n ch ow1 n iy0))
("cirrus" nil (s ih1 r ax s))
("cisar" nil (s ih0 s aa1 r))
("cisco" nil (s ih1 s k ow0))
("cisek" nil (ch ih1 s eh0 k))
("cisewski" nil (ch ih0 s eh1 f s k iy0))
("ciskei" nil (s ih0 s k ey1))
("cisler" nil (s ih1 s ax l er0))
("cislo" nil (ch iy1 s l ow0))
("cisneros" nil (s ih0 s n eh1 r ow0 s))
("cisney" nil (s ih1 z n iy0))
("cissell" nil (s ih1 s ax l))
("cissie" nil (s ih1 s iy0))
("cissna" nil (s ih1 s n ax))
("cist" nil (s ih1 s t))
("cistercian" nil (s ih0 s t er1 sh ax n))
("cisterns" nil (s ih1 s t er0 n z))
("ciszek" nil (ch ih1 sh eh0 k))
("ciszewski" nil (ch ih0 sh eh1 f s k iy0))
("citadel" nil (s ih1 t ax d eh1 l))
("citation" nil (s ay0 t ey1 sh ax n))
("citations" nil (s ay0 t ey1 sh ax n z))
("cite" nil (s ay1 t))
("cited" nil (s ay1 t ax d))
("cites" nil (s ay1 t s))
("citgo" nil (s ih1 t g ow0))
("citi" nil (s ih1 t iy0))
("citibank" nil (s ih1 t iy0 b ae1 ng k))
("citic" nil (s ih1 t ih0 k))
("citicorp" nil (s ih1 t iy0 k ao1 r p))
("citicorps" nil (s ih1 t iy0 k ao1 r p s))
("cities" nil (s ih1 t iy0 z))
("citing" nil (s ay1 t ih0 ng))
("citisteel" nil (s ih1 t iy0 s t iy1 l))
("citizen" nil (s ih1 t ax z ax n))
("citizenry" nil (s ih1 t ih0 z ax n r iy0))
("citizens" nil (s ih1 t ax z ax n z))
("citizenship" nil (s ih1 t ih0 z ax n sh ih1 p))
("citrano" nil (ch iy0 t r aa1 n ow0))
("citric" nil (s ih1 t r ih0 k))
("citrin" nil (s ih1 t r ih0 n))
("citro" nil (s ih1 t r ow0))
("citroen" nil (s ih1 t r ow0 n))
("citron" nil (s ih1 t r ax n))
("citrosuco" nil (s ih1 t r ax s uw1 k ow0))
("citrucel" nil (s ih1 t r ax s eh1 l))
("citrus" nil (s ih1 t r ax s))
("cittadino" nil (ch iy0 t aa0 d iy1 n ow0))
("city" nil (s ih1 t iy0))
("cityfed" nil (s ih1 t iy0 f eh1 d))
("cityplace" nil (s ih1 t iy0 p l ey1 s))
("cityside" nil (s ih1 t iy0 s ay1 d))
("citytrust" nil (s ih1 t iy0 t r ah1 s t))
("citywide" nil (s ih1 t iy0 w ay1 d))
("ciucci" nil (ch uw1 ch iy0))
("ciudad" nil (s iy1 uw0 d ae1 d))
("ciulla" nil (ch uw1 l ax))
("ciullo" nil (ch uw1 l ow0))
("civet" nil (s ih1 v ax t))
("civic" nil (s ih1 v ih0 k))
("civics" nil (s ih1 v ih0 k s))
("civil" nil (s ih1 v ax l))
("civiletti" nil (s iy1 v ih0 l eh1 t iy0))
("civilian" nil (s ax v ih1 l y ax n))
("civilians" nil (s ax v ih1 l y ax n z))
("civility" nil (s ax v ih1 l ax t iy0))
("civilization" nil (s ih1 v ax l ih0 z ey1 sh ax n))
("civilizations" nil (s ih1 v ax l ih0 z ey1 sh ax n z))
("civilize" nil (s ih1 v ax l ay1 z))
("civilized" nil (s ih1 v ax l ay1 z d))
("civilly" nil (s ih1 v ih0 l iy0))
("civitello" nil (ch iy0 v iy0 t eh1 l ow0))
("cizek" nil (ch ih1 z eh0 k))
("cizik" nil (s iy1 z ih0 k))
("cizneros" nil (s ih1 z n eh1 r ow0 s))
("claar" nil (k l aa1 r))
("claassen" nil (k l aa1 s ax n))
("clabaugh" nil (k l ae1 b ao0))
("clabir" nil (k l ae1 b ih0 r))
("clabo" nil (k l aa1 b ow0))
("claborn" nil (k l ae1 b er0 n))
("clabough" nil (k l ae1 b aw0))
("clack" nil (k l ae1 k))
("clad" nil (k l ae1 d))
("claddagh" nil (k l ae1 d ah1))
("claes" nil (k l ey1 z))
("claeys" nil (k l ey1 z))
("claffey" nil (k l ae1 f iy0))
("claflin" nil (k l ae1 f l ih0 n))
("clagett" nil (k l ae1 jh ih0 t))
("clagg" nil (k l ae1 g))
("claggett" nil (k l ae1 g ih0 t))
("clague" nil (k l aa1 g))
("claiborn" nil (k l ey1 b er0 n))
("claiborne" nil (k l ey1 b er0 n))
("claim" nil (k l ey1 m))
("claimant" nil (k l ey1 m ax n t))
("claimants" nil (k l ey1 m ax n t s))
("claimed" nil (k l ey1 m d))
("claiming" nil (k l ey1 m ih0 ng))
("claims" nil (k l ey1 m z))
("clair" nil (k l eh1 r))
("claire" nil (k l eh1 r))
("clairmont" nil (k l eh1 r m ax n t))
("clairol" nil (k l eh1 r aa0 l))
("clairson" nil (k l eh1 r s ax n))
("clairvoyance" nil (k l eh0 r v oy1 ax n s))
("clam" nil (k l ae1 m))
("clamber" nil (k l ae1 m b er0))
("clambered" nil (k l ae1 m b er0 d))
("clamen" nil (k l ey1 m ax n))
("clammy" nil (k l ae1 m iy0))
("clamor" nil (k l ae1 m er0))
("clamored" nil (k l ae1 m er0 d))
("clamoring" nil (k l ae1 m er0 ih0 ng))
("clamp" nil (k l ae1 m p))
("clampdown" nil (k l ae1 m p d aw1 n))
("clamped" nil (k l ae1 m p t))
("clamping" nil (k l ae1 m p ih0 ng))
("clampitt" nil (k l ax m p ih1 t))
("clamps" nil (k l ae1 m p s))
("clams" nil (k l ae1 m z))
("clamshell" nil (k l ae1 m sh eh1 l))
("clan" nil (k l ae1 n))
("clancey" nil (k l ae1 n s iy0))
("clancy" nil (k l ae1 n s iy0))
("clandestine" nil (k l ae0 n d eh1 s t ih0 n))
("clandestinely" nil (k l ae0 n d eh1 s t ax n l iy0))
("clang" nil (k l ae1 ng))
("clanging" nil (k l ae1 ng ih0 ng))
("clanin" nil (k l ae1 n ih0 n))
("clanking" nil (k l ae1 ng k ih0 ng))
("clannish" nil (k l ae1 n ih0 sh))
("clans" nil (k l ae1 n z))
("clanton" nil (k l ae1 n t ax n))
("clap" nil (k l ae1 p))
("clapboard" nil (k l ae1 p b ao1 r d))
("clapboards" nil (k l ae1 p b ao1 r d z))
("clapham" nil (k l ae1 f ax m))
("clapman" nil (k l ae1 p m ax n))
("clapp" nil (k l ae1 p))
("clapped" nil (k l ae1 p t))
("clapper" nil (k l ae1 p er0))
("clapping" nil (k l ae1 p ih0 ng))
("claps" nil (k l ae1 p s))
("clapsaddle" nil (k l ae1 p s ae1 d ax l))
("clapton" nil (k l ae1 p t ax n))
("clar" nil (k l aa1 r))
("clara" nil (k l ae1 r ax))
("clarabelle" nil (k l ae1 r ax b ax l))
("claramae" nil (k l aa0 r aa1 m ay0))
("clarcor" nil (k l aa1 r k ao1 r))
("clardy" nil (k l aa1 r d iy0))
("clare" nil (k l eh1 r))
("claremont" nil (k l eh1 r m aa1 n t))
("claren" nil (k l ae1 r ax n))
("clarence" nil (k l eh1 r ax n s))
("clarendon" nil (k l eh1 r ax n d ax n))
("claresta" nil (k l aa0 r eh1 s t ax))
("clarette" nil (k l er0 eh1 t))
("clarey" nil (k l ae1 r iy0))
("claribel" nil (k l eh1 r ax b eh1 l))
("clarice" nil (k l er0 iy1 s))
("clarida" nil (k l aa0 r iy1 d ax))
("claridge" nil (k l eh1 r ih0 jh))
("clarification" nil (k l eh1 r ax f ax k ey1 sh ax n))
("clarifications" nil (k l eh1 r ih0 f ih0 k ey1 sh ax n z))
("clarified" nil (k l eh1 r ax f ay1 d))
("clarifies" nil (k l eh1 r ax f ay1 z))
("clarify" nil (k l eh1 r ax f ay1))
("clarifying" nil (k l eh1 r ax f ay1 ih0 ng))
("clarimond" nil (k l ae1 r ih0 m ax n d))
("clarinda" nil (k l er0 ih1 n d ax))
("clarine" nil (k l eh1 r iy0 n))
("clarinet" nil (k l eh1 r ax n eh1 t))
("clarinetist" nil (k l eh1 r ax n eh1 t ih0 s t))
("clarinetists" nil (k l eh1 r ax n eh1 t ih0 s t s))
("clarins" nil (k l eh1 r ih0 n z))
("clarion" nil (k l eh1 r iy0 ax n))
("claris" nil (k l eh1 r ih0 s))
("clarissa" nil (k l er0 ih1 s ax))
("clarisse" nil (k l ae1 r ih0 s))
("clarita" nil (k l aa0 r iy1 t ax))
("claritin" nil (k l eh1 r ih0 t ih0 n))
("clarity" nil (k l eh1 r ax t iy0))
("clark" nil (k l aa1 r k))
("clarke" nil (k l aa1 r k))
("clarkin" nil (k l aa1 r k ih0 n))
("clarks" nil (k l aa1 r k s))
("clarksburg" nil (k l aa1 r k s b er0 g))
("clarkson" nil (k l aa1 r k s ax n))
("clarkston" nil (k l aa1 r k s t ax n))
("clarksville" nil (k l aa1 r k s v ih1 l))
("claro" nil (k l aa1 r ow0))
("clarostat" nil (k l eh1 r ax s t ae1 t))
("clarridge" nil (k l ae1 r ih0 jh))
("clarrisse" nil (k l ae1 r ih0 s))
("clary" nil (k l eh1 r iy0))
("clasby" nil (k l ae1 s b iy0))
("clasen" nil (k l ey1 s ax n))
("clash" nil (k l ae1 sh))
("clashed" nil (k l ae1 sh t))
("clashes" nil (k l ae1 sh ih0 z))
("clashing" nil (k l ae1 sh ih0 ng))
("clason" nil (k l ae1 s ax n))
("clasp" nil (k l ae1 s p))
("clasped" nil (k l ae1 s p t))
("class" nil (k l ae1 s))
("classaction" nil (k l ae1 s ae1 k sh ax n))
("classed" nil (k l ae1 s t))
("classen" nil (k l ae1 s ax n))
("classes" nil (k l ae1 s ax z))
("classic" nil (k l ae1 s ih0 k))
("classical" nil (k l ae1 s ih0 k ax l))
("classically" nil (k l ae1 s ih0 k l iy0))
("classicism" nil (k l ae1 s ih0 s ih1 z ax m))
("classicist" nil (k l ae1 s ax s ax s t))
("classics" nil (k l ae1 s ih0 k s))
("classier" nil (k l ae1 s iy0 er0))
("classifiable" nil (k l ae1 s ax f ay1 ax b ax l))
("classification" nil (k l ae1 s ax f ax k ey1 sh ax n))
("classifications" nil (k l ae1 s ax f ax k ey1 sh ax n z))
("classified" nil (k l ae1 s ax f ay1 d))
("classifies" nil (k l ae1 s ax f ay1 z))
("classify" nil (k l ae1 s ax f ay1))
("classifying" nil (k l ae1 s ax f ay1 ih0 ng))
("classing" nil (k l ae1 s ih0 ng))
("classless" nil (k l ae1 s l ax s))
("classmate" nil (k l ae1 s m ey1 t))
("classmates" nil (k l ae1 s m ey1 t s))
("classon" nil (k l ae1 s ax n))
("classroom" nil (k l ae1 s r uw1 m))
("classrooms" nil (k l ae1 s r uw1 m z))
("classy" nil (k l ae1 s iy0))
("clatter" nil (k l ae1 t er0))
("claud" nil (k l ao1 d))
("claude" nil (k l ao1 d))
("claudette" nil (k l ow0 d eh1 t))
("claudia" nil (k l ao1 d iy0 ax))
("claudian" nil (k l ao1 d iy0 ax n))
("claudie" nil (k l ao1 d iy0))
("claudina" nil (k l ao1 d ih0 n ax))
("claudine" nil (k l ao0 d iy1 n))
("claudio" nil (k l ao1 d iy0 ow1))
("claudius" nil (k l ao1 d iy0 ax s))
("claudson" nil (k l ao1 d s ax n))
("claunch" nil (k l ao1 n ch))
("claus" nil (k l ao1 z))
("clause" nil (k l ao1 z))
("clausell" nil (k l ao1 z ax l))
("clausen" nil (k l aw1 s ax n))
("clauser" nil (k l ao1 z er0))
("clauses" nil (k l ao1 z ax z))
("clausing" nil (k l ao1 z ih0 ng))
("clauson" nil (k l ao1 z ax n))
("clauss" nil (k l ao1 s))
("claussen" nil (k l ao1 z s ax n))
("claustrophobia" nil (k l ao1 s t r ax f ow1 b iy0 ax))
("claustrophobic" nil (k l ao1 s t r ax f ow1 b ih0 k))
("clavette" nil (k l ax v eh1 t))
("clavichord" nil (k l ae1 v ax k ao1 r d))
("clavicle" nil (k l ae1 v ax k ax l))
("clavin" nil (k l ae1 v ih0 n))
("claw" nil (k l ao1))
("clawed" nil (k l ao1 d))
("clawing" nil (k l ao1 ih0 ng))
("claws" nil (k l ao1 z))
("clawson" nil (k l ao1 s ax n))
("claxon" nil (k l ae1 k s ax n))
("claxton" nil (k l ae1 k s t ax n))
("clay" nil (k l ey1))
("claybaugh" nil (k l ey1 b ao1))
("clayborn" nil (k l ey1 b er0 n))
("clayborne" nil (k l ey1 b er0 n))
("claybourne" nil (k l ey1 b er0 n))
("claybrook" nil (k l ey1 b r uh1 k))
("claybrooks" nil (k l ey1 b r uh1 k s))
("clayburn" nil (k l ey1 b er1 n))
("claycomb" nil (k l ey1 k ax m))
("clayey" nil (k l ey1 iy0))
("clayman" nil (k l ey1 m ax n))
("claymation" nil (k l ey1 m ey1 sh ax n))
("claymont" nil (k l ey1 m aa1 n t))
("claymore" nil (k l ey1 m ao1 r))
("claypool" nil (k l ey1 p uw1 l))
("claypoole" nil (k l ey1 p uw1 l))
("clays" nil (k l ey1 z))
("clayson" nil (k l ey1 z ax n))
("clayton" nil (k l ey1 t ax n))
("claytor" nil (k l ey1 t er0))
("claywell" nil (k l ey1 w eh1 l))
("cleah" nil (k iy1 ax))
("clean" nil (k l iy1 n))
("cleaned" nil (k l iy1 n d))
("cleaner" nil (k l iy1 n er0))
("cleaners" nil (k l iy1 n er0 z))
("cleanest" nil (k l iy1 n ax s t))
("cleaning" nil (k l iy1 n ih0 ng))
("cleanliness" nil (k l eh1 n l iy0 n ih0 s))
("cleanly" nil (k l iy1 n l iy0))
("cleans" nil (k l iy1 n z))
("cleanse" nil (k l eh1 n z))
("cleansed" nil (k l eh1 n z d))
("cleanser" nil (k l eh1 n z er0))
("cleansers" nil (k l eh1 n z er0 z))
("cleansing" nil (k l eh1 n z ih0 ng))
("cleantha" nil (k l iy1 n th ax))
("cleanup" nil (k l iy1 n ah1 p))
("cleanups" nil (k l iy1 n ah1 p s))
("clear" nil (k l ih1 r))
("clearance" nil (k l ih1 r ax n s))
("clearances" nil (k l ih1 r ax n s ih0 z))
("clearcut" nil (k l ih1 r k ah1 t))
("clearcuts" nil (k l ih1 r k ah1 t s))
("clearcutting" nil (k l ih1 r k ah1 t ih0 ng))
("cleared" nil (k l ih1 r d))
("clearer" nil (k l ih1 r er0))
("clearest" nil (k l ih1 r ih0 s t))
("clearing" nil (k l ih1 r ih0 ng))
("clearinghouse" nil (k l ih1 r ih0 ng hh aw1 s))
("clearinghouses" nil (k l iy1 r ih0 ng hh aw1 s ih0 z))
("clearly" nil (k l ih1 r l iy0))
("clearman" nil (k l ih1 r m ax n))
("clears" nil (k l ih1 r z))
("clearwater" nil (k l ih1 r w ao1 t er0))
("cleary" nil (k l ih1 r iy0))
("cleat" nil (k l iy1 t))
("cleats" nil (k l iy1 t s))
("cleavage" nil (k l iy1 v ax jh))
("cleaveland" nil (k l iy1 v l ax n d))
("cleavenger" nil (k l iy1 v ih0 n jh er0))
("cleaver" nil (k l iy1 v er0))
("cleaves" nil (k l iy1 v z))
("cleckler" nil (k l eh1 k l er0))
("cleckley" nil (k l eh1 k l iy0))
("cleckner" nil (k l eh1 k n er0))
("cleek" nil (k l iy1 k))
("cleere" nil (k l ih1 r))
("cleese" nil (k l iy1 s))
("cleeton" nil (k l iy1 t ax n))
("clegg" nil (k l eh1 g))
("cleghorn" nil (k l eh1 g hh er0 n))
("cleland" nil (k l eh1 l ax n d))
("clell" nil (k l eh1 l))
("clelland" nil (k l eh1 l ax n d))
("clem" nil (k l eh1 m))
("clemans" nil (k l eh1 m ax n z))
("clematis" nil (k l eh1 m ax t ih0 s))
("clemen" nil (k l eh1 m ax n))
("clemence" nil (k l eh1 m ax n s))
("clemency" nil (k l eh1 m ax n s iy0))
("clemens" nil (k l eh1 m ax n z))
("clemensen" nil (k l eh1 m ax n s ax n))
("clemenson" nil (k l eh1 m ih0 n s ax n))
("clement" nil (k l eh1 m ax n t))
("clemente" nil (k l ax m eh1 n t ey0))
("clementi" nil (k l ey0 m ey1 n t iy0))
("clementia" nil (k l ey0 m ey1 n sh ax))
("clementine" nil (k l eh1 m ax n t ay1 n))
("clements" nil (k l eh1 m ax n t s))
("clementson" nil (k l eh1 m ih0 n t s ax n))
("clementz" nil (k l eh1 m ih0 n t s))
("clemmer" nil (k l eh1 m er0))
("clemmie" nil (k l eh1 m iy0))
("clemmons" nil (k l eh1 m ax n z))
("clemmy" nil (k l eh1 m iy0))
("clemo" nil (k l ey1 m ow0))
("clemons" nil (k l eh1 m ax n z))
("clemson" nil (k l eh1 m s ax n))
("clenched" nil (k l eh1 n ch t))
("clendaniel" nil (k l eh1 n d ax n iy1 l))
("clendenen" nil (k l eh1 n d ax n ax n))
("clendenin" nil (k l eh1 n d ih0 n ih0 n))
("clendening" nil (k l eh1 n d ax n ih0 ng))
("clendenning" nil (k l eh1 n d eh1 n ih0 ng))
("clenney" nil (k l eh1 n iy0))
("cleo" nil (k l iy1 ow0))
("cleopatra" nil (k l iy1 ax p ae1 t r ax))
("clephane" nil (k l eh1 f ey1 n))
("clepper" nil (k l eh1 p er0))
("clerc" nil (k l er1 k))
("clercq" nil (k l er1 k))
("clergy" nil (k l er1 jh iy0))
("clergyman" nil (k l er1 jh iy0 m ax n))
("clergymen" nil (k l er1 jh iy0 m ih0 n))
("cleric" nil (k l eh1 r ih0 k))
("clerical" nil (k l eh1 r ax k ax l))
("clerics" nil (k l eh1 r ih0 k s))
("clerissa" nil (k l er0 iy1 s ax))
("clerk" nil (k l er1 k))
("clerkin" nil (k l er1 k ih0 n))
("clerks" nil (k l er1 k s))
("clermont" nil (k l eh1 r m aa1 n t))
("cleva" nil (k l iy1 v ax))
("cleve" nil (k l iy1 v))
("cleveland" nil (k l iy1 v l ax n d))
("clevelanders" nil (k l iy1 v l ax n d er0 z))
("cleven" nil (k l iy1 v ax n))
("clevenger" nil (k l eh1 v ih0 n jh er0))
("clever" nil (k l eh1 v er0))
("cleverly" nil (k l eh1 v er0 l iy0))
("cleverness" nil (k l eh1 v er0 n ax s))
("clevetrust" nil (k l iy1 v t r ah1 s t))
("clevie" nil (k l iy1 v iy0))
("clevinger" nil (k l eh1 v ih0 ng er0))
("clevite" nil (k l ax v ay1 t))
("clewell" nil (k l eh1 w eh0 l))
("clewis" nil (k l uw1 ih0 s))
("clews" nil (k l uw1 z))
("cliantha" nil (k l iy0 ae1 n th ax))
("cliburn" nil (k l ih1 b er0 n))
("cliche" nil (k l iy0 sh ey1))
("cliched" nil (k l iy1 sh ey1 d))
("cliches" nil (k l iy0 sh ey1 z))
("click" nil (k l ih1 k))
("clicked" nil (k l ih1 k t))
("clicking" nil (k l ih1 k ih0 ng))
("clickner" nil (k l ih1 k n er0))
("clicks" nil (k l ih1 k s))
("client" nil (k l ay1 ax n t))
("clientele" nil (k l ay1 ax n t eh1 l))
("clients" nil (k l ay1 ax n t s))
("cliett" nil (k l iy1 t))
("clif" nil (k l ih1 f))
("cliff" nil (k l ih1 f))
("cliffe" nil (k l ih1 f))
("cliffhanger" nil (k l ih1 f hh ae1 ng er0))
("cliffhangers" nil (k l ih1 f hh ae1 ng er0 z))
("clifford" nil (k l ih1 f er0 d))
("cliffs" nil (k l ih1 f s))
("clift" nil (k l ih1 f t))
("clifton" nil (k l ih1 f t ax n))
("cliggott" nil (k l ih1 g ax t))
("clim" nil (k l ih1 m))
("climaco" nil (k l ih1 m ax k ow0))
("climactic" nil (k l ay0 m ae1 k t ih0 k))
("climate" nil (k l ay1 m ax t))
("climates" nil (k l ay1 m ax t s))
("climatic" nil (k l ay0 m ae1 t ih0 k))
("climatologist" nil (k l ih1 m ax t aa1 l ax jh ih0 s t))
("climax" nil (k l ay1 m ae1 k s))
("climaxed" nil (k l ay1 m ae1 k s t))
("climaxes" nil (k l ih1 m ae0 k s ih0 z))
("climb" nil (k l ay1 m))
("climbed" nil (k l ay1 m d))
("climber" nil (k l ay1 m er0))
("climbers" nil (k l ay1 m er0 z))
("climbing" nil (k l ay1 m ih0 ng))
("climbs" nil (k l ay1 m z))
("climer" nil (k l ay1 m er0))
("climes" nil (k l ay1 m z))
("clinard" nil (k l ih1 n er0 d))
("clinch" nil (k l ih1 n ch))
("clinched" nil (k l ih1 n ch t))
("clincher" nil (k l ih1 n ch er0))
("clinches" nil (k l ih1 n ch ax z))
("clinching" nil (k l ih1 n ch ih0 ng))
("cline" nil (k l ay1 n))
("clines" nil (k l ay1 n z))
("cling" nil (k l ih1 ng))
("clingan" nil (k l ih1 ng g ax n))
("clingenpeel" nil (k l ih0 ng g eh1 n p iy0 l))
("clinger" nil (k l ih1 ng er0))
("clingerman" nil (k l ih1 ng er0 m ax n))
("clinging" nil (k l ih1 ng ih0 ng))
("clingman" nil (k l ih1 ng m ax n))
("clings" nil (k l ih1 ng z))
("clini" nil (k l iy1 n iy0))
("clinic" nil (k l ih1 n ih0 k))
("clinical" nil (k l ih1 n ax k ax l))
("clinically" nil (k l ih1 n ih0 k ax l iy0))
("clinician" nil (k l ih0 n ih1 sh ax n))
("clinicians" nil (k l ih0 n ih1 sh ax n z))
("clinics" nil (k l ih1 n ih0 k s))
("clink" nil (k l ih1 ng k))
("clinkenbeard" nil (k l ih0 ng k eh1 n b er0 d))
("clinkscale" nil (k l ih1 ng k s k ey1 l))
("clinkscales" nil (k l ih1 ng k s k ey1 l z))
("clint" nil (k l ih1 n t))
("clinton" nil (k l ih1 n t ax n))
("clintonites" nil (k l ih1 n t ax n ay1 t s))
("clintonomics" nil (k l ih1 n t ax n aa1 m ih0 k s))
("clintons" nil (k l ih1 n t ax n z))
("clio" nil (k l iy1 ow0))
("clip" nil (k l ih1 p))
("clipboard" nil (k l ih1 p b ao1 r d))
("clippard" nil (k l ih1 p er0 d))
("clipped" nil (k l ih1 p t))
("clipper" nil (k l ih1 p er0))
("clippers" nil (k l ih1 p er0 z))
("clipping" nil (k l ih1 p ih0 ng))
("clippinger" nil (k l ih1 p ih0 ng er0))
("clippings" nil (k l ih1 p ih0 ng z))
("clips" nil (k l ih1 p s))
("clique" nil (k l iy1 k))
("cliques" nil (k l ih1 k s))
("clites" nil (k l ay1 t s))
("clive" nil (k l ay1 v))
("cliver" nil (k l ay1 v er0))
("cloak" nil (k l ow1 k))
("cloaked" nil (k l ow1 k t))
("cloaking" nil (k l ow1 k ih0 ng))
("clobber" nil (k l aa1 b er0))
("clobbered" nil (k l aa1 b er0 d))
("clobbering" nil (k l aa1 b er0 ih0 ng))
("clock" nil (k l aa1 k))
("clocked" nil (k l aa1 k t))
("clocks" nil (k l aa1 k s))
("clockwork" nil (k l aa1 k w er1 k))
("clodfelter" nil (k l aa1 d f eh1 l t er0))
("clodoveo" nil (k l ow0 d ow1 v iy0 ow0))
("cloe" nil (k l ow1))
("cloer" nil (k l ow1 er0))
("cloey" nil (k l ow1 iy0))
("clog" nil (k l aa1 g))
("clogged" nil (k l ao1 g d))
("clogging" nil (k l aa1 g ih0 ng))
("clogs" nil (k l aa1 g z))
("clogston" nil (k l aa1 g s t ax n))
("cloherty" nil (k l aa1 hh er0 t iy0))
("clohessy" nil (k l aa1 hh ih0 s iy0))
("cloistered" nil (k l oy1 s t er0 d))
("cloisters" nil (k l oy1 s t er0 z))
("cloke" nil (k l ow1 k))
("clomipramine" nil (k l ow1 m ih0 p r ax m ay1 n))
("clonch" nil (k l aa1 n ch))
("clone" nil (k l ow1 n))
("cloned" nil (k l ow1 n d))
("clones" nil (k l ow1 n z))
("clonidine" nil (k l aa1 n ih0 d iy1 n))
("cloning" nil (k l ow1 n ih0 ng))
("cloninger" nil (k l ow1 n ih0 ng er0))
("clonts" nil (k l aa1 n t s))
("clontz" nil (k l aa1 n t s))
("cloonan" nil (k l uw1 n ax n))
("clootie" nil (k l uw1 t iy0))
("clopper" nil (k l aa1 p er0))
("clopton" nil (k l aa1 p t ax n))
("clore" nil (k l ao1 r))
("clorinda" nil (k l ao0 r iy1 n d ax))
("clorox" nil (k l ao1 r aa0 k s))
("clos" nil (k l aa1 s))
("close" n (k l ow1 s))
("close" v (k l ow1 z))
("closed" nil (k l ow1 z d))
("closedown" nil (k l ow1 z d aw1 n))
("closedowns" nil (k l ow1 z d aw1 n z))
("closely" nil (k l ow1 s l iy0))
("closeness" nil (k l ow1 s n ih0 s))
("closer" nil (k l ow1 s er0))
("closes" nil (k l ow1 z ih0 z))
("closest" nil (k l ow1 s ax s t))
("closet" nil (k l aa1 z ax t))
("closeted" nil (k l aa1 z ax t ih0 d))
("closets" nil (k l aa1 z ax t s))
("closeup" nil (k l ow1 s ah1 p))
("closeups" nil (k l ow1 s ah1 p s))
("closing" nil (k l ow1 z ih0 ng))
("closings" nil (k l ow1 z ih0 ng z))
("closs" nil (k l ao1 s))
("closser" nil (k l ao1 s er0))
("closson" nil (k l aa1 s ax n))
("closure" nil (k l ow1 zh er0))
("closures" nil (k l ow1 zh er0 z))
("clot" nil (k l aa1 t))
("clotfelter" nil (k l aa1 t f eh1 l t er0))
("cloth" nil (k l ao1 th))
("clothe" nil (k l ow1 dh))
("clothed" nil (k l ow1 dh d))
("clothes" nil (k l ow1 dh z))
("clotheshorse" nil (k l ow1 z hh ao1 r s))
("clothestime" nil (k l ow1 dh z t ay1 m))
("clothier" nil (k l ow1 dh y er0))
("clothiers" nil (k l ow1 dh y er0 z))
("clothilda" nil (k l ax th ih1 l d ax))
("clothilde" nil (k l aa1 th ih0 l d))
("clothing" nil (k l ow1 dh ih0 ng))
("cloths" nil (k l ao1 th s))
("clotilda" nil (k l ax t ih1 l d ax))
("clots" nil (k l aa1 t s))
("clott" nil (k l aa1 t))
("clotted" nil (k l aa1 t ax d))
("clotting" nil (k l aa1 t ih0 ng))
("cloture" nil (k l ow1 ch er0))
("clouatre" nil (k l aw1 ax t r))
("cloud" nil (k l aw1 d))
("cloudburst" nil (k l aw1 d b er1 s t))
("clouded" nil (k l aw1 d ax d))
("clouding" nil (k l aw1 d ih0 ng))
("cloudless" nil (k l aw1 d l ax s))
("clouds" nil (k l aw1 d z))
("cloudy" nil (k l aw1 d iy0))
("clough" nil (k l aw1))
("clougherty" nil (k l aw1 er0 t iy0))
("clouse" nil (k l aw1 s))
("clouser" nil (k l aw1 s er0))
("clout" nil (k l aw1 t))
("clouthier" nil (k l aw1 th iy0 er0))
("cloutier" nil (k l aw1 t iy0 er0))
("clover" nil (k l ow1 v er0))
("cloverleaf" nil (k l ow1 v er0 l iy1 f))
("cloves" nil (k l ow1 v z))
("clovis" nil (k l ow1 v ih0 s))
("clow" nil (k l ow1))
("cloward" nil (k l ow1 w er0 d))
("clowdus" nil (k l aw1 d ih0 s))
("clower" nil (k l aw1 er0))
("clowers" nil (k l aw1 er0 z))
("clowes" nil (k l aw1 z))
("clown" nil (k l aw1 n))
("clowney" nil (k l aw1 n iy0))
("clowning" nil (k l aw1 n ih0 ng))
("clowns" nil (k l aw1 n z))
("cloyd" nil (k l oy1 d))
("cloying" nil (k l oy1 ih0 ng))
("clozapine" nil (k l ow1 z ax p ay1 n))
("club" nil (k l ah1 b))
("clubb" nil (k l ah1 b))
("clubbed" nil (k l ah1 b d))
("clubbing" nil (k l ah1 b ih0 ng))
("clubby" nil (k l ah1 b iy0))
("clubhouse" nil (k l ah1 b hh aw1 s))
("clubhouses" nil (k l ah1 b hh aw1 s ih0 z))
("clubs" nil (k l ah1 b z))
("clucas" nil (k l uw1 k ax z))
("cluck" nil (k l ah1 k))
("cluckey" nil (k l ah1 k iy0))
("clucking" nil (k l ah1 k ih0 ng))
("clucks" nil (k l ah1 k s))
("clue" nil (k l uw1))
("clued" nil (k l uw1 d))
("clueless" nil (k l uw1 l ax s))
("clues" nil (k l uw1 z))
("cluett" nil (k l uw1 ih0 t))
("cluff" nil (k l ah1 f))
("clugston" nil (k l ah1 g s t ax n))
("clukey" nil (k l uw1 k iy0))
("clum" nil (k l ah1 m))
("clump" nil (k l ah1 m p))
("clumps" nil (k l ah1 m p s))
("clumpy" nil (k l ah1 m p iy0))
("clumsily" nil (k l ah1 m s ax l iy0))
("clumsiness" nil (k l ah1 m z iy0 n ax s))
("clumsy" nil (k l ah1 m z iy0))
("clune" nil (k l uw1 n))
("clung" nil (k l ah1 ng))
("clunky" nil (k l ah1 ng k iy0))
("cluny" nil (k l uw1 n iy0))
("cluster" nil (k l ah1 s t er0))
("clustered" nil (k l ah1 s t er0 d))
("clustering" nil (k l ah1 s t er0 ih0 ng))
("clusters" nil (k l ah1 s t er0 z))
("clutch" nil (k l ah1 ch))
("clutched" nil (k l ah1 ch t))
("clutches" nil (k l ah1 ch ax z))
("clutching" nil (k l ah1 ch ih0 ng))
("clute" nil (k l uw1 t))
("clutter" nil (k l ah1 t er0))
("cluttered" nil (k l ah1 t er0 d))
("clutts" nil (k l ah1 t s))
("clyatt" nil (k l ay1 ax t))
("clyburn" nil (k l ih1 b er0 n))
("clyde" nil (k l ay1 d))
("clydesdale" nil (k l ay1 d z d ey1 l))
("clymene" nil (k l ih0 m iy1 n))
("clymer" nil (k l ay1 m er0))
("clyne" nil (k l ay1 n))
("clyte" nil (k l ay1 t))
("clytie" nil (k l ih1 t iy0))
("clyve" nil (k l ay1 v))
("cmos" nil (s iy1 m ow0 s))
("cmx" nil (k ax m eh1 k s))
("co" nil (k ow1))
("coach" nil (k ow1 ch))
("coached" nil (k ow1 ch t))
("coaches" nil (k ow1 ch ih0 z))
("coaching" nil (k ow1 ch ih0 ng))
("coachman" nil (k ow1 ch m ax n))
("coad" nil (k ow1 d))
("coady" nil (k ow1 d iy0))
("coagulating" nil (k ow0 ae1 g y ax l ey1 t ih0 ng))
("coagulation" nil (k ow0 ae1 g y ax l ey1 sh ax n))
("coakley" nil (k ow1 k l iy0))
("coal" nil (k ow1 l))
("coale" nil (k ow1 l))
("coalesce" nil (k ow1 ax l eh1 s))
("coalesced" nil (k ow1 ax l eh1 s t))
("coalescing" nil (k ow1 ax l eh1 s ih0 ng))
("coalition" nil (k ow1 ax l ih1 sh ax n))
("coalitions" nil (k ow1 ax l ih1 sh ax n z))
("coals" nil (k ow1 l z))
("coalson" nil (k ow1 l s ax n))
("coan" nil (k ow1 n))
("coar" nil (k ao1 r))
("coarse" nil (k ao1 r s))
("coarser" nil (k ao1 r s er0))
("coast" nil (k ow1 s t))
("coastal" nil (k ow1 s t ax l))
("coastamerica" nil (k ow1 s t ax m eh1 r ih0 k ax))
("coasted" nil (k ow1 s t ih0 d))
("coaster" nil (k ow1 s t er0))
("coasters" nil (k ow1 s t er0 z))
("coasting" nil (k ow1 s t ih0 ng))
("coastline" nil (k ow1 s t l ay1 n))
("coastlines" nil (k ow1 s t l ay1 n z))
("coasts" nil (k ow1 s t s))
("coat" nil (k ow1 t))
("coate" nil (k ow1 ey1 t))
("coated" nil (k ow1 t ax d))
("coates" nil (k ow1 ey1 t s))
("coatesville" nil (k ow1 t s v ih1 l))
("coating" nil (k ow1 t ih0 ng))
("coatings" nil (k ow1 t ih0 ng z))
("coatney" nil (k ow1 t n iy0))
("coats" nil (k ow1 t s))
("coattails" nil (k ow1 t t ey1 l z))
("coauthors" nil (k ow1 aa1 th er0 z))
("coax" nil (k ow1 k s))
("coaxed" nil (k ow1 k s t))
("coaxes" nil (k ow1 k s ih0 z))
("coaxial" nil (k ow1 ae1 k s iy0 ax l))
("coaxing" nil (k ow1 k s ih0 ng))
("coaxum" nil (k ow1 k s ax m))
("cob" nil (k aa1 b))
("cobain" nil (k ow1 b ey1 n))
("cobalt" nil (k ow1 b ao1 l t))
("cobaugh" nil (k aa1 b ao0))
("cobb" nil (k aa1 b))
("cobbett" nil (k aa1 b ih0 t))
("cobbins" nil (k aa1 b ih0 n z))
("cobble" nil (k aa1 b ax l))
("cobbled" nil (k aa1 b ax l d))
("cobbler" nil (k aa1 b l er0))
("cobblers" nil (k aa1 b l er0 z))
("cobblestones" nil (k aa1 b ax l s t ow1 n z))
("cobbs" nil (k aa1 b z))
("cobe" nil (k ow1 b))
("coben" nil (k ow1 b ax n))
("cobepa" nil (k ow0 b ey1 p ax))
("coberly" nil (k ow1 b er0 l iy0))
("cobern" nil (k aa1 b er0 n))
("cobert" nil (k aa1 b er0 t))
("cobey" nil (k ow1 b iy0))
("cobia" nil (k ow1 b iy0 ax))
("cobian" nil (k ow1 b iy0 ax n))
("cobin" nil (k ow1 b ih0 n))
("coble" nil (k ow1 b ax l))
("cobleigh" nil (k aa1 b l ax))
("coblentz" nil (k aa1 b l ih0 n t s))
("cobler" nil (k ow1 b ax l er0))
("cobo" nil (k ow1 b ow0))
("cobos" nil (k ow1 b ow0 z))
("cobra" nil (k ow1 b r ax))
("cobras" nil (k ow1 b r ax z))
("cobre" nil (k aa1 b r ax))
("cobrin" nil (k aa1 b r ih0 n))
("cobs" nil (k aa1 b z))
("coburn" nil (k ow1 b er0 n))
("cobwebs" nil (k aa1 b w eh1 b z))
("coby" nil (k ow1 b iy0))
("coca" nil (k ow1 k ax))
("cocaine" nil (k ow0 k ey1 n))
("cocanougher" nil (k aa1 k ax n ah1 f er0))
("cocca" nil (k ow1 k ax))
("coccaro" nil (k ow0 k aa1 r ow0))
("cocchi" nil (k ow1 k iy0))
("coccia" nil (k ow1 ch ax))
("cocco" nil (k ow1 k ow0))
("coccus" nil (k aa1 k ax s))
("cochairman" nil (k ow1 ch eh1 r m ax n))
("cochenour" nil (k aa1 sh ih0 n uh0 r))
("cochin" nil (k ow1 ch ih0 n))
("cochlea" nil (k aa1 k l iy0 ax))
("cochlear" nil (k aa1 k l iy0 er0))
("cochran" nil (k aa1 k r ax n))
("cochrane" nil (k aa1 k r ax n))
("cock" nil (k aa1 k))
("cockatoo" nil (k aa1 k ax t uw1))
("cockatoos" nil (k aa1 k ax t uw1 z))
("cockburn" nil (k aa1 k b er1 n))
("cocke" nil (k ow1 k))
("cocked" nil (k aa1 k t))
("cocker" nil (k aa1 k er0))
("cockerell" nil (k aa1 k er0 eh1 l))
("cockerham" nil (k aa1 k er0 hh ae1 m))
("cockerill" nil (k aa1 k er0 ih1 l))
("cockey" nil (k aa1 k iy0))
("cockeyed" nil (k aa1 k ay1 d))
("cockfield" nil (k aa1 k f iy1 l d))
("cockiness" nil (k aa1 k ih0 n ax s))
("cocking" nil (k aa1 k ih0 ng))
("cocklin" nil (k aa1 k l ih0 n))
("cockman" nil (k aa1 k m ax n))
("cockney" nil (k aa1 k n iy0))
("cockpit" nil (k aa1 k p ih1 t))
("cockpits" nil (k aa1 k p ih1 t s))
("cockran" nil (k aa1 k r ax n))
("cockrell" nil (k aa1 k r ax l))
("cockrill" nil (k aa1 k r ax l))
("cockroach" nil (k aa1 k r ow1 ch))
("cockroaches" nil (k aa1 k r ow1 ch ih0 z))
("cockroft" nil (k aa1 k r ax f t))
("cockrum" nil (k aa1 k r ax m))
("cocks" nil (k aa1 k s))
("cocktail" nil (k aa1 k t ey1 l))
("cocktails" nil (k aa1 k t ey1 l z))
("cockwell" nil (k aa1 k w eh1 l))
("cocky" nil (k aa1 k iy0))
("coco" nil (k ow1 k ow1))
("cocoa" nil (k ow1 k ow0))
("cocoanuts" nil (k ow1 k ow0 n ah1 t s))
("cocom" nil (k ow1 k aa1 m))
("coconut" nil (k ow1 k ax n ah1 t))
("coconuts" nil (k ow1 k ax n ah1 t s))
("cocoon" nil (k ax k uw1 n))
("cocoons" nil (k ax k uw1 n z))
("cocos" nil (k ow1 k ow1 z))
("cocozza" nil (k ow0 k ow1 t s ax))
("cocuzza" nil (k ow0 k uw1 t s ax))
("cod" nil (k aa1 d))
("coda" nil (k ow1 d ax))
("codag" nil (k ow1 d ae1 g))
("coday" nil (k ow1 d ey1))
("codd" nil (k aa1 d))
("codding" nil (k aa1 d ih0 ng))
("coddington" nil (k aa1 d ih0 ng t ax n))
("coddle" nil (k aa1 d ax l))
("coddled" nil (k aa1 d ax l d))
("coddling" nil (k aa1 d ax l ih0 ng))
("code" nil (k ow1 d))
("coded" nil (k ow1 d ih0 d))
("codelco" nil (k ow0 d eh1 l k ow0))
("coder" nil (k ow1 d er0))
("coderre" nil (k ax d eh1 r))
("codes" nil (k ow1 d z))
("codification" nil (k aa1 d ax f ax k ey1 sh ax n))
("codified" nil (k aa1 d ax f ay1 d))
("codifies" nil (k ow1 d ax f ay1 z))
("codify" nil (k ow1 d ax f ay1))
("codifying" nil (k ow1 d ax f ay1 ih0 ng))
("coding" nil (k ow1 d ih0 ng))
("codispoti" nil (k ow0 d iy0 s p ow1 t iy0))
("codlin" nil (k aa1 d l ih0 n))
("codner" nil (k aa1 d n er0))
("codrescu" nil (k ax d r eh1 s k y uw1))
("cody" nil (k ow1 d iy0))
("coe" nil (k ow1))
("coeburn" nil (k ow1 b er0 n))
("coed" nil (k ow1 eh1 d))
("coeds" nil (k ow1 eh1 d z))
("coeducational" nil (k ow1 eh1 jh ax k ey1 sh ax n ax l))
("coefficient" nil (k ow1 ax f ih1 sh ax n t))
("coefficients" nil (k ow1 ax f ih1 sh ax n t s))
("coehlo" nil (k ow1 l ow0))
("coelacanth" nil (s iy1 l ax k ae1 n th))
("coelho" nil (k ow1 eh1 l ow0))
("coello" nil (k ow1 eh1 l ow0))
("coen" nil (k ow1 ih0 n))
("coenen" nil (k ow0 iy1 n ax n))
("coenzyme" nil (k ow0 eh1 n z ay0 m))
("coequal" nil (k ow0 iy1 k w ax l))
("coerce" nil (k ow0 er1 s))
("coerced" nil (k ow0 er1 s t))
("coercing" nil (k ow0 er1 s ih0 ng))
("coercion" nil (k ow0 er1 sh ax n))
("coercive" nil (k ow0 er1 s ih0 v))
("coeur" nil (k uw1 r))
("coexist" nil (k ow1 ax g z ih1 s t))
("coexisted" nil (k ow1 ax g z ih1 s t ax d))
("coexistence" nil (k ow1 ih0 g z ih1 s t ax n s))
("coexisting" nil (k ow1 ih0 g z ih1 s t ih0 ng))
("coey" nil (k ow1 iy0))
("cofer" nil (k ow1 f er1))
("coffaro" nil (k ow0 f aa1 r ow0))
("coffee" nil (k aa1 f iy0))
("coffeen" nil (k ax f iy1 n))
("coffees" nil (k ao1 f iy0 z))
("coffel" nil (k aa1 f ax l))
("coffelt" nil (k aa1 f ih0 l t))
("coffer" nil (k ao1 f er0))
("coffers" nil (k aa1 f er0 z))
("coffey" nil (k aa1 f iy0))
("coffield" nil (k aa1 f iy0 l d))
("coffin" nil (k ao1 f ih0 n))
("coffing" nil (k ao1 f ih0 ng))
("coffins" nil (k ao1 f ih0 n z))
("coffman" nil (k ao1 f m ax n))
("cofide" nil (k ow1 f ay1 d))
("cofield" nil (k ow1 f iy1 l d))
("cofounder" nil (k ow1 f aw1 n d er0))
("cog" nil (k ao1 g))
("cogan" nil (k ow1 g ax n))
("cogar" nil (k ow1 g er0))
("cogbill" nil (k aa1 g b ih1 l))
("cogburn" nil (k aa1 g b er1 n))
("cogdell" nil (k aa1 g d ax l))
("cogdill" nil (k aa1 g d ax l))
("cogeco" nil (k ow1 jh eh1 k ow0))
("cogema" nil (k ow1 g m aa0))
("cogency" nil (k ow1 jh ax n s iy0))
("cogenerated" nil (k ow1 jh eh1 n er0 ey1 t ih0 d))
("cogeneration" nil (k ow1 jh eh1 n er0 ey1 sh ax n))
("cogenerators" nil (k ow0 jh eh1 n er0 ey1 t er0 z))
("cogent" nil (k ow1 jh ax n t))
("coger" nil (k ow1 jh er0))
("coggeshall" nil (k aa1 g ih0 sh ao0 l))
("coggin" nil (k aa1 g ih0 n))
("coggins" nil (k aa1 g ih0 n z))
("coghill" nil (k aa1 g hh ih1 l))
("coghlan" nil (k aa1 g l ax n))
("cogley" nil (k aa1 g l iy0))
("coglianese" nil (k ow0 g l iy0 ax n ey1 z iy0))
("cogliano" nil (k ow0 g l iy0 aa1 n ow0))
("cognac" nil (k ow1 n y ae1 k))
("cognetics" nil (k aa1 g n eh1 t ih0 k s))
("cognex" nil (k aa1 g n eh0 k s))
("cognition" nil (k aa0 g n ih1 sh ax n))
("cognitive" nil (k aa1 g n ih0 t ih0 v))
("cognizance" nil (k aa1 g n ax z ax n s))
("cognizant" nil (k aa1 g n ax z ax n t))
("cognoscenti" nil (k aa1 g n ao1 sh eh1 n t iy0))
("cogswell" nil (k aa1 g s w eh1 l))
("cohabitation" nil (k ow0 hh ae1 b ax t ey1 sh ax n))
("cohabiting" nil (k ow0 hh ae1 b ih0 t ih0 ng))
("cohan" nil (k ow1 hh ax n))
("cohasset" nil (k ow0 hh ae1 s ax t))
("cohea" nil (k aa1 hh iy0 ax))
("cohee" nil (k aa1 hh iy0))
("cohen" nil (k ow1 ax n))
("cohenour" nil (k ax hh eh1 n er0))
("coherence" nil (k ow0 hh ih1 r ax n s))
("coherent" nil (k ow0 hh ih1 r ax n t))
("coherently" nil (k ow0 hh iy1 r ax n t l iy0))
("cohesion" nil (k ow0 hh iy1 zh ax n))
("cohesive" nil (k ow0 hh iy1 s ih0 v))
("cohesively" nil (k ow0 hh iy1 s ih0 v l iy0))
("cohesiveness" nil (k ow0 hh iy1 s ih0 v n ax s))
("cohick" nil (k aa1 hh ih0 k))
("cohill" nil (k ow1 hh ih1 l))
("cohn" nil (k ow1 n))
("cohoon" nil (k ax hh uw1 n))
("cohort" nil (k ow1 hh ao0 r t))
("cohorts" nil (k ow1 hh ao0 r t s))
("cohron" nil (k aa1 r ax n))
("cohrs" nil (k aa1 r z))
("coia" nil (k ow1 y ax))
("coiffed" nil (k oy1 f t))
("coil" nil (k oy1 l))
("coile" nil (k oy1 l))
("coiled" nil (k oy1 l d))
("coils" nil (k oy1 l z))
("coin" nil (k oy1 n))
("coinage" nil (k oy1 n ih0 jh))
("coincide" nil (k ow1 ih0 n s ay1 d))
("coincided" nil (k ow1 ax n s ay1 d ax d))
("coincidence" nil (k ow0 ih1 n s ih0 d ax n s))
("coincidences" nil (k ow0 ih1 n s ax d ax n s ih0 z))
("coincident" nil (k ow0 ih1 n s ax d ax n t))
("coincidental" nil (k ow0 ih1 n s ax d eh1 n t ax l))
("coincidentally" nil (k ow0 ih1 n s ih0 d eh1 n t ax l iy0))
("coincides" nil (k ow1 ih0 n s ay1 d z))
("coinciding" nil (k ow1 ax n s ay1 d ih0 ng))
("coined" nil (k oy1 n d))
("coiner" nil (k oy1 n er0))
("coining" nil (k oy1 n ih0 ng))
("coins" nil (k oy1 n z))
("coinsurance" nil (k ow1 ih0 n sh er1 ax n s))
("cointreau" nil (k ao1 n t r ow1))
("coipa" nil (k oy1 p ax))
("coiro" nil (k oy1 r ow0))
("coit" nil (k oy1 t))
("cojimar" nil (k ow1 jh ih0 m aa1 r))
("cojuangco" nil (k ow0 w aa1 ng k ow0))
("coke" nil (k ow1 k))
("coker" nil (k ow1 k er0))
("cokes" nil (k ow1 k s))
("cokie" nil (k ow1 k iy0))
("coking" nil (k ow1 k ih0 ng))
("cokley" nil (k aa1 k l iy0))
("cola" nil (k ow1 l ax))
("colab" nil (k ow1 l ae1 b))
("colabella" nil (k ow1 l ax b eh1 l ax))
("colaianni" nil (k ow0 l aa0 y aa1 n iy0))
("colaizzi" nil (k ow1 l ey1 z iy0))
("colaluca" nil (k ow1 l ax l uw1 k ax))
("colan" nil (k ow1 l ax n))
("colangelo" nil (k ow0 l aa0 ng g eh1 l ow0))
("colantonio" nil (k ow0 l aa0 n t ow1 n iy0 ow0))
("colantuono" nil (k ow0 l aa0 n t w ow1 n ow0))
("colao" nil (k ow1 l aw0))
("colarusso" nil (k ow0 l aa0 r uw1 s ow0))
("colas" nil (k ow1 l ax s))
("colasanti" nil (k ow1 l ax s ae1 n t iy0))
("colasurdo" nil (k ow0 l aa0 s uh1 r d ow0))
("colavito" nil (k ow0 l aa0 v iy1 t ow0))
("colaw" nil (k ow1 l ao1))
("colbath" nil (k ow1 l b ax th))
("colbaugh" nil (k ow1 l b ao1))
("colbeck" nil (k aa1 l b eh0 k))
("colberg" nil (k aa1 l b er0 g))
("colbert" nil (k ow1 l b er0 t))
("colborn" nil (k ow1 l b er0 n))
("colburn" nil (k ow1 l b er0 n))
("colby" nil (k ow1 l b iy0))
("colbys" nil (k ow1 l b iy0 z))
("colclasure" nil (k ow0 l k l aa1 zh er0))
("colclough" nil (k ow1 l k l aw0))
("colcord" nil (k ow1 l k er0 d))
("cold" nil (k ow1 l d))
("colden" nil (k ow1 l d ax n))
("colder" nil (k ow1 l d er0))
("coldest" nil (k ow1 l d ax s t))
("coldiron" nil (k ow1 l d er0 aa0 n))
("coldly" nil (k ow1 l d l iy0))
("coldness" nil (k ow1 l d n ax s))
("coldren" nil (k ow1 l d er0 ax n))
("colds" nil (k ow1 l d z))
("coldwater" nil (k ow1 l d w ao1 t er0))
("coldwell" nil (k ow1 l d w eh1 l))
("cole" nil (k ow1 l))
("colebank" nil (k ow1 l b ae1 ng k))
("colebrook" nil (k ow1 l b r uh1 k))
("coleco" nil (k ow1 l eh1 k ow0))
("colee" nil (k ow1 l iy1))
("colegrove" nil (k ow1 l g r ow1 v))
("colella" nil (k ow1 l eh1 l ax))
("colello" nil (k ow1 l eh1 l ow0))
("coleman" nil (k ow1 l m ax n))
("colen" nil (k ow1 l ax n))
("coler" nil (k ow1 l er0))
("coleridge" nil (k ow1 l r ih0 jh))
("coles" nil (k ow1 l z))
("coleslaw" nil (k ow1 l s l aa1))
("coleson" nil (k aa1 l ih0 s ax n))
("colestipol" nil (k ow1 l s t ih1 p aa1 l))
("colestock" nil (k ow1 l s t aa1 k))
("coletta" nil (k ow0 l eh1 t ax))
("colette" nil (k ow1 l eh1 t))
("coletti" nil (k ow0 l eh1 t iy0))
("coleus" nil (k ow1 l iy0 ax s))
("coley" nil (k ow1 l iy0))
("colfer" nil (k ow1 l f er0))
("colford" nil (k ow1 l f er0 d))
("colgan" nil (k ow1 l g ax n))
("colgate" nil (k ow1 l g ey0 t))
("colgin" nil (k ow1 l jh ih0 n))
("colglazier" nil (k ow1 l g l ax z iy0 er0))
("colgrove" nil (k ow1 l g r ax v))
("coli" nil (k ow1 l iy0))
("colicchio" nil (k ow1 l ih1 k iy0 ow0))
("colier" nil (k ow1 l iy0 er0))
("colin" nil (k ow1 l ih0 n))
("colina" nil (k ow0 l iy1 n ax))
("colinas" nil (k ow0 l iy1 n ax s))
("coline" nil (k ow0 l iy1 n iy0))
("colino" nil (k ow0 l iy1 n ow0))
("colis" nil (k ow1 l ih0 s))
("coliseum" nil (k aa1 l ax s iy1 ax m))
("coll" nil (k aa1 l))
("colla" nil (k ow1 l ax))
("collaborate" nil (k ax l ae1 b er0 ey1 t))
("collaborated" nil (k ax l ae1 b er0 ey1 t ax d))
("collaborating" nil (k ax l ae1 b er0 ey1 t ih0 ng))
("collaboration" nil (k ax l ae1 b er0 ey1 sh ax n))
("collaborations" nil (k aa1 l ax b er0 ey1 sh ax n z))
("collaborative" nil (k ax l ae1 b er0 ey1 t ih0 v))
("collaborator" nil (k ax l ae1 b er0 ey1 t er0))
("collaborators" nil (k ax l ae1 b er0 ey1 t er0 z))
("collado" nil (k ow0 l aa1 d ow0))
("collage" nil (k ax l aa1 zh))
("collagen" nil (k aa1 l ax g ax n))
("collages" nil (k ax l aa1 zh ih0 z))
("collamore" nil (k ow0 l aa1 m ao0 r))
("collapse" nil (k ax l ae1 p s))
("collapsed" nil (k ax l ae1 p s t))
("collapses" nil (k ax l ae1 p s ih0 z))
("collapsible" nil (k ax l ae1 p s ax b ax l))
("collapsing" nil (k ax l ae1 p s ih0 ng))
("collar" nil (k aa1 l er0))
("collarbone" nil (k aa1 l er0 b ow1 n))
("collard" nil (k aa1 l er0 d))
("collards" nil (k aa1 l er0 d z))
("collared" nil (k aa1 l er0 d))
("collars" nil (k aa1 l er0 z))
("collate" nil (k ax l ey1 t))
("collateral" nil (k ax l ae1 t er0 ax l))
("collateralize" nil (k ax l ae1 t er0 ax l ay1 z))
("collateralized" nil (k ax l ae1 t er0 ax l ay1 z d))
("collazo" nil (k ow0 l aa1 z ow0))
("colle" nil (k ow1 l))
("colleague" nil (k aa1 l iy0 g))
("colleagues" nil (k aa1 l iy0 g z))
("collect" n (k ah1 l eh k t))
("collect" v (k ax l eh1 k t))
("collected" nil (k ax l eh1 k t ax d))
("collectibility" nil (k ax l eh1 k t ih0 b ih1 l ih0 t iy0))
("collectible" nil (k ax l eh1 k t ax b ax l))
("collectibles" nil (k ax l eh1 k t ax b ax l z))
("collecting" nil (k ax l eh1 k t ih0 ng))
("collection" nil (k ax l eh1 k sh ax n))
("collections" nil (k ax l eh1 k sh ax n z))
("collective" nil (k ax l eh1 k t ih0 v))
("collectively" nil (k ax l eh1 k t ih0 v l iy0))
("collectives" nil (k ax l eh1 k t ih0 v z))
("collectivism" nil (k ax l eh1 k t ih0 v ih1 z ax m))
("collectivist" nil (k ax l eh1 k t ih0 v ih0 s t))
("collectivization" nil (k ax l eh1 k t ih0 v ih0 z ey1 sh ax n))
("collectivized" nil (k ax l eh1 k t ih0 v ay1 z d))
("collector" nil (k ax l eh1 k t er0))
("collectors" nil (k ax l eh1 k t er0 z))
("collects" nil (k ax l eh1 k t s))
("colledge" nil (k aa1 l ih0 jh))
("colleen" nil (k aa1 l iy1 n))
("college" nil (k aa1 l ih0 jh))
("colleges" nil (k aa1 l ih0 jh ih0 z))
("collegeville" nil (k aa1 l ax jh v ih0 l))
("collegial" nil (k ax l iy1 jh iy0 ax l))
("collegiality" nil (k ax l iy1 jh iy0 ae1 l ih0 t iy0))
("collegians" nil (k ax l iy1 jh ax n z))
("collegiate" nil (k ax l iy1 jh ih0 t))
("collen" nil (k aa1 l ax n))
("coller" nil (k aa1 l er0))
("colleran" nil (k aa1 l er0 ae0 n))
("collet" nil (k aa1 l ih0 t))
("collett" nil (k aa1 l ih0 t))
("colletta" nil (k ow0 l eh1 t ax))
("collette" nil (k ax l eh1 t))
("colletti" nil (k ow0 l eh1 t iy0))
("colleville" nil (k ow1 l v ih0 l))
("colley" nil (k aa1 l iy0))
("colli" nil (k ow1 l iy0))
("collick" nil (k aa1 l ih0 k))
("collide" nil (k ax l ay1 d))
("collided" nil (k ax l ay1 d ih0 d))
("collider" nil (k ax l ay1 d er0))
("colliding" nil (k ax l ay1 d ih0 ng))
("collie" nil (k aa1 l iy0))
("collier" nil (k aa1 l y er0))
("colliers" nil (k aa1 l y er0 z))
("collies" nil (k aa1 l iy0 z))
("colligan" nil (k aa1 l ih0 g ae0 n))
("collignon" nil (k ax l ih1 g n ax n))
("collin" nil (k aa1 l ih0 n))
("colling" nil (k aa1 l ih0 ng))
("collinge" nil (k aa1 l ih0 n jh))
("collings" nil (k aa1 l ih0 ng z))
("collingsworth" nil (k ax l ih1 ng z w er0 th))
("collingwood" nil (k aa1 l ih0 ng w uh1 d))
("collins" nil (k aa1 l ih0 n z))
("collinson" nil (k aa1 l ih0 n s ax n))
("collinsworth" nil (k ax l ih1 n s w er0 th))
("collis" nil (k aa1 l ih0 s))
("collision" nil (k ax l ih1 zh ax n))
("collisional" nil (k ax l ih1 zh ax n ax l))
("collisions" nil (k ax l ih1 zh ax n z))
("collison" nil (k aa1 l ih0 s ax n))
("collister" nil (k aa1 l ih0 s t er0))
("colliver" nil (k aa1 l ih0 v er0))
("collman" nil (k aa1 l m ax n))
("collodion" nil (k ax l ow1 d iy0 ax n))
("colloid" nil (k aa1 l oy0 d))
("colloidal" nil (k ax l oy1 d ax l))
("collom" nil (k aa1 l ax m))
("collomb" nil (k aa1 l aa0 m))
("collopy" nil (k ax l ow1 p iy0))
("colloquial" nil (k ax l ow1 k w iy0 ax l))
("colloquium" nil (k ax l ow1 k w iy0 ax m))
("colloquy" nil (k aa1 l ax k w iy0))
("collor" nil (k aa1 l er0))
("collosio" nil (k ax l ow1 s iy0 ow0))
("collude" nil (k ax l uw1 d))
("colluded" nil (k ax l uw1 d ih0 d))
("colluding" nil (k ax l uw1 d ih0 ng))
("collum" nil (k aa1 l ax m))
("collums" nil (k aa1 l ax m z))
("collura" nil (k aa1 l uh0 r ax))
("collusion" nil (k ax l uw1 zh ax n))
("collusive" nil (k ax l uw1 s ih0 v))
("collver" nil (k aa1 l v er0))
("colly" nil (k aa1 l iy0))
("collyer" nil (k aa1 l iy0 er0))
("colman" nil (k ow1 l m ax n))
("colmenero" nil (k ow0 l m ey0 n eh1 r ow0))
("colmer" nil (k ow1 m er0))
("colo" nil (k ow1 l ow0))
("colodny" nil (k ax l aa1 d n iy0))
("cologne" nil (k ax l ow1 n))
("coloma" nil (k ow1 l ow1 m ax))
("colomb" nil (k aa1 l ax m))
("colombari" nil (k aa1 l aa0 m b aa1 r iy0))
("colombe" nil (k ow0 l ow1 m b iy0))
("colombia" nil (k ax l ah1 m b iy0 ax))
("colombian" nil (k ax l ah1 m b iy0 ax n))
("colombians" nil (k ax l ah1 m b iy0 ax n z))
("colombo" nil (k ax l ah1 m b ow0))
("colon" nil (k ow1 l ax n))
("colonel" nil (k er1 n ax l))
("colonels" nil (k er1 n ax l z))
("colonia" nil (k ax l ow1 n iy0 ax))
("colonial" nil (k ax l ow1 n iy0 ax l))
("colonialism" nil (k ax l ow1 n iy0 ax l ih1 z ax m))
("colonialists" nil (k ax l ow1 n iy0 ax l ih0 s t s))
("colonies" nil (k aa1 l ax n iy0 z))
("colonists" nil (k aa1 l ax n ax s t s))
("colonization" nil (k aa1 l ax n ih0 z ey1 sh ax n))
("colonize" nil (k aa1 l ax n ay1 z))
("colonized" nil (k aa1 l ax n ay1 z d))
("colonizers" nil (k aa1 l ax n ay1 z er0 z))
("colonna" nil (k ow0 l ow1 n ax))
("colonnade" nil (k aa1 l ax n ey1 d))
("colonus" nil (k ax l ow1 n ax s))
("colony" nil (k aa1 l ax n iy0))
("colopy" nil (k ax l ow1 p iy0))
("color" nil (k ah1 l er0))
("colorado" nil (k aa1 l er0 aa1 d ow0))
("coloration" nil (k ah1 l er0 ey1 sh ax n))
("coloratura" nil (k ax l er0 ax t uh1 r ax))
("colorblind" nil (k ah1 l er0 b l ay1 n d))
("colorcraft" nil (k ah1 l er0 k r ae1 f t))
("colored" nil (k ah1 l er0 d))
("coloreds" nil (k aa1 l er0 ax d z))
("colorfast" nil (k ah1 l er0 f ae1 s t))
("colorful" nil (k ah1 l er0 f ax l))
("colorfully" nil (k ah1 l er0 f l iy0))
("coloring" nil (k ah1 l er0 ih0 ng))
("coloristic" nil (k ah1 l er0 ih1 s t ih0 k))
("colorization" nil (k ah1 l er0 ax z ey1 sh ax n))
("colorize" nil (k ah1 l er0 ay1 z))
("colorized" nil (k ah1 l er0 ay1 z d))
("colorizing" nil (k ah1 l er0 ay1 z ih0 ng))
("colorless" nil (k ah1 l er0 l ax s))
("colorocs" nil (k ah1 l er0 aa1 k s))
("coloroll" nil (k ah1 l er0 ow1 l))
("colors" nil (k ah1 l er0 z))
("colorwatch" nil (k ah1 l er0 w aa1 ch))
("colosi" nil (k ax l ow1 s iy0))
("colosimo" nil (k ow0 l ow0 s iy1 m ow0))
("colosio" nil (k ax l ow1 s iy0 ow0))
("colossal" nil (k ax l aa1 s ax l))
("colosseum" nil (k aa1 l ax s iy1 ax m))
("colossus" nil (k ax l aa1 s ax s))
("coloured" nil (k ah1 l er0 d))
("colpitts" nil (k ow1 l p ih0 t s))
("colquitt" nil (k ow1 l k w ih0 t))
("colson" nil (k ow1 l s ax n))
("colston" nil (k ow1 l s t ax n))
("colstrip" nil (k ow1 l s t r ih0 p))
("colt" nil (k ow1 l t))
("coltec" nil (k ow1 l t eh1 k))
("colter" nil (k ow1 l t er0))
("coltharp" nil (k ow1 l th aa0 r p))
("colton" nil (k ow1 l t ax n))
("coltrain" nil (k ow1 l t r ey1 n))
("coltrane" nil (k ow1 l t r ax n))
("coltrin" nil (k ow1 l t r ih0 n))
("colts" nil (k ow1 l t s))
("coltsfoot" nil (k ow1 l t s f uh1 t))
("colucci" nil (k ow0 l uw1 ch iy0))
("coluccio" nil (k ow0 l uw1 ch iy0 ow0))
("columba" nil (k ow1 l ah1 m b ax))
("columbia" nil (k ax l ah1 m b iy0 ax))
("columbian" nil (k ow1 l ah1 m b iy0 ax n))
("columbine" nil (k aa1 l ax m b ay1 n))
("columbines" nil (k aa1 l ax m b ay1 n z))
("columbo" nil (k ow1 l ah1 m b ow0))
("columbus" nil (k ax l ah1 m b ax s))
("column" nil (k aa1 l ax m))
("columned" nil (k aa1 l ax m d))
("columnist" nil (k aa1 l ax m n ax s t))
("columnists" nil (k aa1 l ax m n ax s t s))
("columns" nil (k aa1 l ax m z))
("colunga" nil (k ow0 l uw1 ng g ax))
("colussy" nil (k ax l uw1 s iy0))
("colvard" nil (k aa1 l v er0 d))
("colver" nil (k ow1 l v er0))
("colvert" nil (k aa1 l v er0 t))
("colville" nil (k aa1 l v ih0 l))
("colvin" nil (k ow1 l v ih0 n))
("colwell" nil (k ow1 l w eh1 l))
("colyer" nil (k ow1 l iy0 er0))
("com" nil (k aa1 m))
("coma" nil (k ow1 m ax))
("comair" nil (k aa1 m eh1 r))
("coman" nil (k ow1 m ax n))
("comanche" nil (k ax m ae1 n ch iy0))
("comanches" nil (k ax m ae1 n ch iy0 z))
("comandante" nil (k ow1 m ax n d aa1 n t ey0))
("comandantes" nil (k ow1 m ax n d aa1 n t eh0 z))
("comarco" nil (k ow0 m aa1 r k ow0))
("comas" nil (k ow1 m ax z))
("comatose" nil (k ow1 m ax t ow1 s))
("comb" nil (k ow1 m))
("combat" nil (k aa1 m b ae0 t))
("combatant" nil (k ax m b ae1 t ax n t))
("combatants" nil (k ax m b ae1 t ax n t s))
("combating" nil (k ax m b ae1 t ih0 ng))
("combative" nil (k ax m b ae1 t ih0 v))
("combativeness" nil (k ax m b ae1 t ih0 v n ax s))
("combats" nil (k ax m b ae1 t s))
("combatting" nil (k ax m b ae1 t ih0 ng))
("combe" nil (k ow1 m))
("combed" nil (k ow1 m d))
("combee" nil (k aa1 m b iy1))
("comber" nil (k ow1 m er0))
("combes" nil (k ow1 m z))
("combest" nil (k ow1 m ih0 s t))
("combinability" nil (k ax m b ay1 n ax b ih1 l ax t iy0))
("combinable" nil (k ax m b ay1 n ax b ax l))
("combination" nil (k aa1 m b ax n ey1 sh ax n))
("combinations" nil (k aa1 m b ax n ey1 sh ax n z))
("combine" n (k aa1 m b ay0 n))
("combine" v (k ax m b ay1 n))
("combined" nil (k ax m b ay1 n d))
("combines" n (k ah1 m b ay0 n z))
("combines" v (k ax m b ay1 n z))
("combing" nil (k ow1 m ih0 ng))
("combining" nil (k ax m b ay1 n ih0 ng))
("combo" nil (k aa1 m b ow1))
("combs" nil (k ow1 m z))
("combustable" nil (k ax m b ah1 s t ax b ax l))
("combustion" nil (k ax m b ah1 s ch ax n))
("comcast" nil (k aa1 m k ae1 s t))
("comdata" nil (k aa1 m d ae1 d ax))
("comdex" nil (k aa1 m d ax k s))
("comdisco" nil (k aa1 m d ih1 s k ow0))
("come" nil (k ah1 m))
("comeau" nil (k ax m ow1))
("comeaux" nil (k ax m ow1))
("comeback" nil (k ah1 m b ae1 k))
("comebacks" nil (k ah1 m b ae1 k s))
("comecon" nil (k aa1 m ax k aa1 n))
("comedian" nil (k ax m iy1 d iy0 ax n))
("comedians" nil (k ax m iy1 d iy0 ax n z))
("comedic" nil (k ax m iy1 d ih0 k))
("comedienne" nil (k ax m iy1 d iy0 eh1 n))
("comedies" nil (k aa1 m ax d iy0 z))
("comedown" nil (k ah1 m d aw1 n))
("comedy" nil (k aa1 m ax d iy0))
("comegys" nil (k aa1 m ih0 jh iy0 z))
("comella" nil (k ow0 m eh1 l ax))
("comely" nil (k ah1 m l iy0))
("comer" nil (k ah1 m er0))
("comerford" nil (k ax m er1 f er0 d))
("comerica" nil (k ax m eh1 r ih0 k ax))
("comers" nil (k ah1 m er0 z))
("comes" nil (k ah1 m z))
("comet" nil (k aa1 m ax t))
("cometary" nil (k aa1 m ax t eh1 r iy0))
("cometra" nil (k ow0 m eh1 t r ax))
("comets" nil (k aa1 m ax t s))
("comeuppance" nil (k ah1 m ah1 p ax n s))
("comex" nil (k aa1 m eh1 k s))
("comfed" nil (k aa1 m f eh1 d))
("comfinance" nil (k aa1 m f ih0 n ax n s))
("comfort" nil (k ah1 m f er0 t))
("comfortable" nil (k ah1 m f er0 t ax b ax l))
("comfortably" nil (k ah1 m f er0 t ax b l iy0))
("comforted" nil (k ah1 m f er0 t ih0 d))
("comforter" nil (k ah1 m f er0 t er0))
("comforters" nil (k ah1 m f er0 t er0 z))
("comforting" nil (k ah1 m f er0 t ih0 ng))
("comforts" nil (k ah1 m f er0 t s))
("comfrey" nil (k ah1 m f r iy0))
("comfy" nil (k ah1 m f iy0))
("comic" nil (k aa1 m ih0 k))
("comical" nil (k aa1 m ih0 k ax l))
("comically" nil (k aa1 m ih0 k ax l iy0))
("comicopia" nil (k aa1 m ih0 k ow1 p iy0 ax))
("comics" nil (k aa1 m ih0 k s))
("cominco" nil (k ow0 m ih1 ng k ow0))
("coming" nil (k ah1 m ih0 ng))
("comings" nil (k ah1 m ih0 ng z))
("comins" nil (k ow1 m ih0 n z))
("cominsky" nil (k ax m ih1 n s k iy0))
("comiskey" nil (k ow1 m ih0 s k iy1))
("comito" nil (k ow0 m iy1 t ow0))
("comity" nil (k ow1 m ih0 t iy0))
("comley" nil (k aa1 m l iy0))
("comly" nil (k aa1 m l iy0))
("comma" nil (k aa1 m ax))
("commack" nil (k aa1 m ax k))
("command" nil (k ax m ae1 n d))
("commandant" nil (k aa1 m ax n d aa1 n t))
("commanded" nil (k ax m ae1 n d ax d))
("commandeered" nil (k aa1 m ax n d ih1 r d))
("commander" nil (k ax m ae1 n d er0))
("commanders" nil (k ax m ae1 n d er0 z))
("commanding" nil (k ax m ae1 n d ih0 ng))
("commandment" nil (k ax m ae1 n d m ax n t))
("commandments" nil (k ax m ae1 n d m ax n t s))
("commando" nil (k ax m ae1 n d ow1))
("commandos" nil (k ax m ae1 n d ow1 z))
("commands" nil (k ax m ae1 n d z))
("commas" nil (k aa1 m ax z))
("commemorate" nil (k ax m eh1 m er0 ey1 t))
("commemorated" nil (k ax m eh1 m er0 ey1 t ih0 d))
("commemorates" nil (k ax m eh1 m er0 ey1 t s))
("commemorating" nil (k ax m eh1 m er0 ey1 t ih0 ng))
("commemoration" nil (k ax m eh1 m er0 ey1 sh ax n))
("commemorations" nil (k ax m eh1 m er0 ey1 sh ax n z))
("commemorative" nil (k ax m eh1 m r ax t ih0 v))
("commence" nil (k ax m eh1 n s))
("commenced" nil (k ax m eh1 n s t))
("commencement" nil (k ax m eh1 n s m ax n t))
("commences" nil (k ax m eh1 n s ax z))
("commencing" nil (k ax m eh1 n s ih0 ng))
("commend" nil (k ax m eh1 n d))
("commendable" nil (k ax m eh1 n d ax b ax l))
("commendation" nil (k aa1 m ax n d ey1 sh ax n))
("commended" nil (k ax m eh1 n d ih0 d))
("commensurate" nil (k ax m eh1 n s er0 ax t))
("comment" nil (k aa1 m eh0 n t))
("commentaries" nil (k aa1 m ax n t eh1 r iy0 z))
("commentary" nil (k aa1 m ax n t eh1 r iy0))
("commentator" nil (k aa1 m ax n t ey1 t er0))
("commentators" nil (k aa1 m ax n t ey1 t er0 z))
("commented" nil (k aa1 m eh0 n t ax d))
("commenting" nil (k aa1 m eh0 n t ih0 ng))
("comments" nil (k aa1 m eh0 n t s))
("commerce" nil (k aa1 m er0 s))
("commercebancorp" nil (k aa1 m er0 s b ae1 n k ao1 r p))
("commercial" nil (k ax m er1 sh ax l))
("commerciale" nil (k ax m er1 s iy0 ae1 l))
("commercialism" nil (k ax m er1 sh ax l ih1 z ax m))
("commercialization" nil (k ax m er1 sh ax l ih0 z ey1 sh ax n))
("commercialize" nil (k ax m er1 sh ax l ay1 z))
("commercialized" nil (k ax m er1 sh ax l ay1 z d))
("commercializing" nil (k ax m er1 sh ax l ay1 z ih0 ng))
("commercially" nil (k ax m er1 sh ax l iy0))
("commercials" nil (k ax m er1 sh ax l z))
("commerford" nil (k ax m er1 f er0 d))
("commerical" nil (k ax m er1 sh ax l))
("commerzbank" nil (k aa1 m er0 z b ae1 ng k))
("commie" nil (k aa1 m iy0))
("commingle" nil (k ax m ih1 ng g ax l))
("commingled" nil (k aa0 m ih1 ng g ax l d))
("commingling" nil (k aa0 m ih1 ng g ax l ih0 ng))
("commins" nil (k aa1 m ih0 n z))
("commiserate" nil (k ax m ih1 s er0 ey1 t))
("commision" nil (k ax m ih1 z ax n))
("commissar" nil (k aa1 m ax s aa1 r))
("commissaries" nil (k aa1 m ax s eh1 r iy0 z))
("commissars" nil (k ax m ih1 s er0 z))
("commissary" nil (k aa1 m ax s eh1 r iy0))
("commission" nil (k ax m ih1 sh ax n))
("commissioned" nil (k ax m ih1 sh ax n d))
("commissioner" nil (k ax m ih1 sh ax n er0))
("commissioners" nil (k ax m ih1 sh ax n er0 z))
("commissioning" nil (k ax m ih1 sh ax n ih0 ng))
("commissions" nil (k ax m ih1 sh ax n z))
("commisso" nil (k ow0 m iy1 s ow0))
("commit" nil (k ax m ih1 t))
("commitee" nil (k aa1 m ih0 t iy0))
("commitment" nil (k ax m ih1 t m ax n t))
("commitments" nil (k ax m ih1 t m ax n t s))
("commits" nil (k ax m ih1 t s))
("committal" nil (k ax m ih1 t ax l))
("committed" nil (k ax m ih1 t ax d))
("committee" nil (k ax m ih1 t iy0))
("committeeman" nil (k ax m ih1 t iy0 m ax n))
("committees" nil (k ax m ih1 t iy0 z))
("committing" nil (k ax m ih1 t ih0 ng))
("commmon" nil (k aa1 m ax n))
("commode" nil (k ax m ow1 d))
("commodious" nil (k ax m ow1 d iy0 ax s))
("commodities" nil (k ax m aa1 d ax t iy0 z))
("commodity" nil (k ax m aa1 d ax t iy0))
("commodore" nil (k aa1 m ax d ao1 r))
("common" nil (k aa1 m ax n))
("commonality" nil (k aa1 m ax n ae1 l ax t iy0))
("commoner" nil (k aa1 m ax n er0))
("commoners" nil (k aa1 m ax n er0 z))
("commonly" nil (k aa1 m ax n l iy0))
("commonplace" nil (k aa1 m ax n p l ey1 s))
("commons" nil (k aa1 m ax n z))
("commonwealth" nil (k aa1 m ax n w eh1 l th))
("commotion" nil (k ax m ow1 sh ax n))
("communal" nil (k ax m y uw1 n ax l))
("commune" nil (k aa1 m y uw0 n))
("communes" nil (k ax m y uw1 n z))
("communicable" nil (k ax m y uw1 n ax k ax b ax l))
("communicate" nil (k ax m y uw1 n ax k ey1 t))
("communicated" nil (k ax m y uw1 n ax k ey1 t ih0 d))
("communicates" nil (k ax m y uw1 n ih0 k ey1 t s))
("communicating" nil (k ax m y uw1 n ax k ey1 t ih0 ng))
("communication" nil (k ax m y uw1 n ax k ey1 sh ax n))
("communications" nil (k ax m y uw1 n ax k ey1 sh ax n z))
("communicative" nil (k ax m y uw1 n ax k ax t ih0 v))
("communicator" nil (k ax m y uw1 n ax k ey1 t er0))
("communicators" nil (k ax m y uw1 n ax k ey0 t er0 z))
("communion" nil (k ax m y uw1 n y ax n))
("communique" nil (k ax m y uw1 n ax k ey1))
("communiques" nil (k ax m y uw1 n ih0 k ey1 z))
("communism" nil (k aa1 m y ax n ih1 z ax m))
("communist" nil (k aa1 m y ax n ax s t))
("communists" nil (k aa1 m y ax n ax s t s))
("communities" nil (k ax m y uw1 n ax t iy0 z))
("community" nil (k ax m y uw1 n ax t iy0))
("communitywide" nil (k ax m y uw1 n ih0 t iy0 w ay1 d))
("communization" nil (k aa1 m y ax n ax z ey1 sh ax n))
("communize" nil (k aa1 m y ax n ay1 z))
("commute" nil (k ax m y uw1 t))
("commuted" nil (k ax m y uw1 t ih0 d))
("commuter" nil (k ax m y uw1 t er0))
("commuters" nil (k ax m y uw1 t er0 z))
("commutes" nil (k ax m y uw1 t s))
("commuting" nil (k ax m y uw1 t ih0 ng))
("como" nil (k ow1 m ow0))
("comp" nil (k aa1 m p))
("compact" n (k aa1 m p ae0 k t))
("compact" v (k ax m p ae1 k t))
("compacted" nil (k ax m p ae1 k t ih0 d))
("compactness" nil (k ax m p ae1 k t n ax s))
("compactor" nil (k ax m p ae1 k t er0))
("compacts" n (k ah1 m p ae0 k t s))
("compacts" v (k ax m p ae1 k t s))
("compagnie" nil (k ah1 m p ax n iy0))
("companhia" nil (k ax m p ae1 n hh iy0 ax))
("compania" nil (k ax m p ey1 n iy0 ax))
("companies" nil (k ah1 m p ax n iy0 z))
("companion" nil (k ax m p ae1 n y ax n))
("companions" nil (k ax m p ae1 n y ax n z))
("companionship" nil (k ax m p ae1 n y ax n sh ih1 p))
("companionway" nil (k ax m p ae1 n y ax n w ey1))
("company" nil (k ah1 m p ax n iy0))
("companywide" nil (k ah1 m p ax n iy0 w ay1 d))
("compaq" nil (k aa1 m p ae1 k))
("comparability" nil (k aa1 m p er0 ax b ih1 l ih0 t iy0))
("comparable" nil (k aa1 m p er0 ax b ax l))
("comparably" nil (k aa1 m p er0 ax b l iy0))
("comparative" nil (k ax m p eh1 r ax t ih0 v))
("comparatively" nil (k ax m p eh1 r ax t ih0 v l iy0))
("compare" nil (k ax m p eh1 r))
("compared" nil (k ax m p eh1 r d))
("compares" nil (k ax m p eh1 r z))
("comparing" nil (k ax m p eh1 r ih0 ng))
("comparison" nil (k ax m p eh1 r ax s ax n))
("comparisons" nil (k ax m p eh1 r ax s ax n z))
("compartment" nil (k ax m p aa1 r t m ax n t))
("compartmental" nil (k ax m p aa1 r t m eh1 n t ax l))
("compartmentalize" nil (k ax m p aa1 r t m eh1 n t ax l ay1 z))
("compartmentalized" nil (k aa1 m p aa0 r t m eh1 n t ax l ay0 z d))
("compartments" nil (k ax m p aa1 r t m ax n t s))
("compas" nil (k aa1 m p ax s))
("compass" nil (k ah1 m p ax s))
("compassion" nil (k ax m p ae1 sh ax n))
("compassionate" nil (k ax m p ae1 sh ax n ax t))
("compatibility" nil (k ax m p ae1 t ax b ih1 l ax t iy0))
("compatible" nil (k ax m p ae1 t ax b ax l))
("compatibles" nil (k ax m p ae1 t ih0 b ax l z))
("compatriots" nil (k ax m p ey1 t r iy0 ax t s))
("compean" nil (k aa1 m p ax n))
("compeau" nil (k ax m p ow1))
("compel" nil (k ax m p eh1 l))
("compelled" nil (k ax m p eh1 l d))
("compelling" nil (k ax m p eh1 l ih0 ng))
("compellingly" nil (k ax m p eh1 l ih0 ng l iy0))
("compels" nil (k ax m p eh1 l z))
("compendium" nil (k ax m p eh1 n d iy0 ax m))
("compensable" nil (k ax m p eh1 n s ax b ax l))
("compensate" nil (k aa1 m p ax n s ey1 t))
("compensated" nil (k aa1 m p ax n s ey1 t ax d))
("compensates" nil (k aa1 m p ax n s ey1 t s))
("compensating" nil (k aa1 m p ax n s ey1 t ih0 ng))
("compensation" nil (k aa1 m p ax n s ey1 sh ax n))
("compensations" nil (k aa1 m p ax n s ey1 sh ax n z))
("compensatory" nil (k ax m p eh1 n s ax t ao1 r iy0))
("compete" nil (k ax m p iy1 t))
("competed" nil (k ax m p iy1 t ih0 d))
("competence" nil (k aa1 m p ax t ax n s))
("competency" nil (k aa1 m p ax t ax n s iy0))
("competent" nil (k aa1 m p ax t ax n t))
("competently" nil (k aa1 m p ax t ax n t l iy0))
("competes" nil (k ax m p iy1 t s))
("competing" nil (k ax m p iy1 t ih0 ng))
("competition" nil (k aa1 m p ax t ih1 sh ax n))
("competitions" nil (k aa1 m p ax t ih1 sh ax n z))
("competitive" nil (k ax m p eh1 t ax t ih0 v))
("competitively" nil (k ax m p eh1 t ih0 t ih0 v l iy0))
("competitiveness" nil (k ax m p eh1 t ih0 t ih0 v n ih0 s))
("competitor" nil (k ax m p eh1 t ax t er0))
("competitors" nil (k ax m p eh1 t ax t er0 z))
("competitve" nil (k aa1 m p ax t ih1 t v iy0))
("compher" nil (k aa1 m f er0))
("compilation" nil (k aa1 m p ax l ey1 sh ax n))
("compilations" nil (k aa1 m p ax l ey1 sh ax n z))
("compile" nil (k ax m p ay1 l))
("compiled" nil (k ax m p ay1 l d))
("compilers" nil (k ax m p ay1 l er0 z))
("compiles" nil (k ax m p ay1 l z))
("compiling" nil (k ax m p ay1 l ih0 ng))
("complacency" nil (k ax m p l ey1 s ax n s iy0))
("complacent" nil (k ax m p l ey1 s ax n t))
("complacently" nil (k ax m p l ey1 s ax n t l iy0))
("complain" nil (k ax m p l ey1 n))
("complainants" nil (k ax m p l ey1 n ax n t s))
("complained" nil (k ax m p l ey1 n d))
("complainer" nil (k ax m p l ey1 n er0))
("complainers" nil (k aa1 m p l ey1 n er0 z))
("complaining" nil (k ax m p l ey1 n ih0 ng))
("complains" nil (k ax m p l ey1 n z))
("complaint" nil (k ax m p l ey1 n t))
("complaints" nil (k ax m p l ey1 n t s))
("complaisant" nil (k ax m p l ey1 s ax n t))
("complement" nil (k aa1 m p l ax m ax n t))
("complementary" nil (k aa1 m p l ax m eh1 n t r iy0))
("complemented" nil (k aa1 m p l ax m eh1 n t ih0 d))
("complementing" nil (k aa1 m p l ax m eh1 n t ih0 ng))
("complements" nil (k aa1 m p l ax m ax n t s))
("complete" nil (k ax m p l iy1 t))
("completed" nil (k ax m p l iy1 t ax d))
("completely" nil (k ax m p l iy1 t l iy0))
("completeness" nil (k ax m p l iy1 t n ax s))
("completes" nil (k ax m p l iy1 t s))
("completing" nil (k ax m p l iy1 t ih0 ng))
("completion" nil (k ax m p l iy1 sh ax n))
("completions" nil (k ax m p l iy1 sh ax n z))
("complex" nil (k aa1 m p l eh0 k s))
("complexes" nil (k aa1 m p l eh0 k s ax z))
("complexion" nil (k ax m p l eh1 k sh ax n))
("complexities" nil (k ax m p l eh1 k s ih0 t iy0 z))
("complexity" nil (k ax m p l eh1 k s ax t iy0))
("compliance" nil (k ax m p l ay1 ax n s))
("compliant" nil (k ax m p l ay1 ax n t))
("complicate" nil (k aa1 m p l ax k ey1 t))
("complicated" nil (k aa1 m p l ax k ey1 t ax d))
("complicates" nil (k aa1 m p l ih0 k ey1 t s))
("complicating" nil (k aa1 m p l ax k ey1 t ih0 ng))
("complication" nil (k aa1 m p l ax k ey1 sh ax n))
("complications" nil (k aa1 m p l ax k ey1 sh ax n z))
("complicity" nil (k ax m p l ih1 s ax t iy0))
("complied" nil (k ax m p l ay1 d))
("complies" nil (k ax m p l ay1 z))
("compliment" n (k aa1 m p l ax m ax n t))
("compliment" v (k aa1 m p l ax m eh0 n t))
("complimentary" nil (k aa1 m p l ax m eh1 n t er0 iy0))
("compliments" n (k aa1 m p l ax m ax n t s))
("compliments" v (k aa1 m p l ax m eh0 n t s))
("comply" nil (k ax m p l ay1))
("complying" nil (k ax m p l ay1 ih0 ng))
("compo" nil (k aa1 m p ow1))
("component" nil (k ax m p ow1 n ax n t))
("components" nil (k ax m p ow1 n ax n t s))
("comportment" nil (k ax m p ao1 r t m ax n t))
("compose" nil (k ax m p ow1 z))
("composed" nil (k ax m p ow1 z d))
("composer" nil (k ax m p ow1 z er0))
("composers" nil (k ax m p ow1 z er0 z))
("composes" nil (k ax m p ow1 z ih0 z))
("composing" nil (k ax m p ow1 z ih0 ng))
("composite" nil (k ax m p aa1 z ax t))
("composites" nil (k ax m p aa1 z ax t s))
("composition" nil (k aa1 m p ax z ih1 sh ax n))
("compositional" nil (k aa1 m p ax z ih1 sh ax n ax l))
("compositions" nil (k aa1 m p ax z ih1 sh ax n z))
("compost" nil (k aa1 m p ow0 s t))
("composting" nil (k aa1 m p ow1 s t ih0 ng))
("composure" nil (k ax m p ow1 zh er0))
("compote" nil (k aa1 m p ow0 t))
("compound" n (k aa1 m p aw0 n d))
("compound" v (k ax m p aw1 n d))
("compounded" nil (k ax m p aw1 n d ax d))
("compounding" nil (k ax m p aw1 n d ih0 ng))
("compounds" n (k aa1 m p aw0 n d z))
("compounds" v (k ax m p aw1 n d z))
("comprehend" nil (k aa1 m p r iy0 hh eh1 n d))
("comprehended" nil (k aa1 m p r ih0 hh eh1 n d ih0 d))
("comprehending" nil (k aa1 m p r ih0 hh eh1 n d ih0 ng))
("comprehensibility" nil (k aa1 m p r iy0 hh eh1 n s ax b ih1 l ax t iy0))
("comprehensible" nil (k aa1 m p r iy0 hh eh1 n s ax b ax l))
("comprehension" nil (k aa1 m p r iy0 hh eh1 n sh ax n))
("comprehensive" nil (k aa1 m p r iy0 hh eh1 n s ih0 v))
("comprehensively" nil (k aa1 m p r ih0 hh eh1 n s ih0 v l iy0))
("compress" n (k aa1 m p r eh0 s))
("compress" v (k ax m p r eh1 s))
("compressed" nil (k ax m p r eh1 s t))
("compresses" n (k aa1 m p r eh0 s ax z))
("compresses" v (k ax m p r eh1 s ax z))
("compressing" nil (k ax m p r eh1 s ih0 ng))
("compression" nil (k ax m p r eh1 sh ax n))
("compressor" nil (k ax m p r eh1 s er0))
("compressors" nil (k ax m p r eh1 s er0 z))
("comprint" nil (k aa1 m p r ih1 n t))
("comprise" nil (k ax m p r ay1 z))
("comprised" nil (k ax m p r ay1 z d))
("comprises" nil (k ax m p r ay1 z ax z))
("comprising" nil (k ax m p r ay1 z ih0 ng))
("compromise" nil (k aa1 m p r ax m ay1 z))
("compromised" nil (k aa1 m p r ax m ay1 z d))
("compromises" nil (k aa1 m p r ax m ay1 z ih0 z))
("compromising" nil (k aa1 m p r ax m ay1 z ih0 ng))
("compston" nil (k aa1 m p s t ax n))
("comptek" nil (k aa1 m p t eh1 k))
("compton" nil (k aa1 m p t ax n))
("comptroller" nil (k ax n t r ow1 l er0))
("comptronix" nil (k aa1 m p t r aa1 n ih1 k s))
("compuadd" nil (k aa1 m p y uw0 ae1 d))
("compuchem" nil (k aa1 m p y uw0 k eh1 m))
("compudyne" nil (k aa1 m p y uw0 d ay1 n))
("compufund" nil (k aa1 m p y uw0 f ah1 n d))
("compugraphic" nil (k aa1 m p y uw0 g r ae1 f ih0 k))
("compulsion" nil (k ax m p ah1 l sh ax n))
("compulsions" nil (k ax m p uh1 l sh ax n z))
("compulsive" nil (k ax m p ah1 l s ih0 v))
("compulsively" nil (k ax m p ah1 l s ih0 v l iy0))
("compulsory" nil (k ax m p ah1 l s er0 iy0))
("compumat" nil (k aa1 m p y uw0 m ae1 t))
("compunction" nil (k ax m p ah1 ng k sh ax n))
("compuserve" nil (k aa1 m p y uw0 s er0 v))
("computalog" nil (k aa1 m p y uw0 t ae1 l ao0 g))
("computation" nil (k aa1 m p y ax t ey1 sh ax n))
("computational" nil (k aa1 m p y uw0 t ey1 sh ax n ax l))
("computations" nil (k aa1 m p y uw0 t ey1 sh ax n z))
("compute" nil (k ax m p y uw1 t))
("computed" nil (k ax m p y uw1 t ax d))
("computer" nil (k ax m p y uw1 t er0))
("computercraft" nil (k ax m p y uw1 t er0 k r ae1 f t))
("computerization" nil (k ax m p y uw1 t er0 ih0 z ey1 sh ax n))
("computerize" nil (k ax m p y uw1 t er0 ay1 z))
("computerized" nil (k ax m p y uw1 t er0 ay1 z d))
("computerizing" nil (k ax m p y uw1 t er0 ay1 z ih0 ng))
("computerland" nil (k ax m p y uw1 t er0 l ae1 n d))
("computerlike" nil (k ax m p y uw1 t er0 l ay1 k))
("computers" nil (k ax m p y uw1 t er0 z))
("computervision" nil (k ax m p y uw1 t er0 v ih1 zh ax n))
("computerworld" nil (k ax m p y uw1 t er0 w er1 l d))
("computes" nil (k ax m p y uw1 t s))
("computing" nil (k ax m p y uw1 t ih0 ng))
("computrac" nil (k aa1 m p y uw0 t r ae1 k))
("compuware" nil (k aa1 m p y uw0 w eh1 r))
("comrade" nil (k aa1 m r ae1 d))
("comrades" nil (k aa1 m r ae1 d z))
("comrie" nil (k aa1 m er0 iy0))
("comsat" nil (k aa1 m s ae0 t))
("comstock" nil (k aa1 m s t aa1 k))
("comtois" nil (k ax m t w aa1))
("comtrex" nil (k aa1 m t r eh1 k s))
("comunale" nil (k ow0 m uw0 n aa1 l iy0))
("con" nil (k aa1 n))
("cona" nil (k ow1 n ax))
("conable" nil (k ow1 n ax b ax l))
("conagra" nil (k aa1 n ae1 g r ax))
("conahan" nil (k aa1 n ax hh ae0 n))
("conair" nil (k aa1 n eh1 r))
("conal" nil (k ow1 n ax l))
("conan" nil (k ow1 n ax n))
("conant" nil (k ow1 n ax n t))
("conard" nil (k aa1 n er0 d))
("conasupo" nil (k aa1 n ax s uw1 p ow0))
("conatser" nil (k aa1 n ax t s er0))
("conaty" nil (k aa1 n ax t iy0))
("conaway" nil (k ow1 n ax w ey0))
("conboy" nil (k aa1 n b oy0))
("conca" nil (k ow1 n k ax))
("concannon" nil (k aa1 n k ax n aa0 n))
("concatenate" nil (k ax n k ae1 t ax n ey1 t))
("concatenation" nil (k ax n k ae1 t ax n ey1 sh ax n))
("concave" nil (k aa0 n k ey1 v))
("conceal" nil (k ax n s iy1 l))
("concealed" nil (k ax n s iy1 l d))
("concealing" nil (k ax n s iy1 l ih0 ng))
("concealment" nil (k ax n s iy1 l m ax n t))
("conceals" nil (k ax n s iy1 l z))
("concede" nil (k ax n s iy1 d))
("conceded" nil (k ax n s iy1 d ih0 d))
("concedes" nil (k ax n s iy1 d z))
("conceding" nil (k ax n s iy1 d ih0 ng))
("conceit" nil (k ax n s iy1 t))
("conceited" nil (k ax n s iy1 t ax d))
("conceivable" nil (k ax n s iy1 v ax b ax l))
("conceivably" nil (k ax n s iy1 v ax b l iy0))
("conceive" nil (k ax n s iy1 v))
("conceived" nil (k ax n s iy1 v d))
("conceiving" nil (k ax n s iy1 v ih0 ng))
("concentrate" nil (k aa1 n s ax n t r ey1 t))
("concentrated" nil (k aa1 n s ax n t r ey1 t ax d))
("concentrates" nil (k aa1 n s ax n t r ey1 t s))
("concentrating" nil (k aa1 n s ax n t r ey1 t ih0 ng))
("concentration" nil (k aa1 n s ax n t r ey1 sh ax n))
("concentrations" nil (k aa1 n s ax n t r ey1 sh ax n z))
("concentric" nil (k ax n s eh1 n t r ih0 k))
("concepcion" nil (k ax n s eh1 p s iy0 ow1 n))
("concept" nil (k aa1 n s eh0 p t))
("conception" nil (k ax n s eh1 p sh ax n))
("conceptions" nil (k ax n s eh1 p sh ax n z))
("concepts" nil (k aa1 n s eh0 p t s))
("conceptual" nil (k ax n s eh1 p ch uw0 ax l))
("conceptually" nil (k ax n s eh1 p ch uw0 ax l iy0))
("concern" nil (k ax n s er1 n))
("concerned" nil (k ax n s er1 n d))
("concerning" nil (k ax n s er1 n ih0 ng))
("concerns" nil (k ax n s er1 n z))
("concert" n (k aa1 n s er0 t))
("concert" v (k aa0 n s er1 t))
("concerted" nil (k ax n s er1 t ax d))
("concerti" nil (k ax n ch eh1 r t iy0))
("concertmaster" nil (k aa1 n s er0 t m ae1 s t er0))
("concerto" nil (k ax n ch eh1 r t ow0))
("concertos" nil (k ax n ch eh1 r t ow0 z))
("concerts" n (k aa1 n s er0 t s))
("concerts" v (k ax n s er1 t s))
("concession" nil (k ax n s eh1 sh ax n))
("concessionaire" nil (k ax n s eh1 sh ax n eh1 r))
("concessional" nil (k ax n s eh1 sh ax n ax l))
("concessionary" nil (k ax n s eh1 sh ax n eh1 r iy0))
("concessions" nil (k ax n s eh1 sh ax n z))
("conch" nil (k aa1 n ch))
("concha" nil (k aa1 n ch ax))
("conchemco" nil (k aa1 n ch eh1 m k ow0))
("conchita" nil (k ow0 n k iy1 t ax))
("concierge" nil (k aa1 n s iy0 eh1 r zh))
("conciliation" nil (k ax n s ih1 l iy0 ey1 sh ax n))
("conciliator" nil (k ax n s iy1 l iy0 ey1 t er0))
("conciliatory" nil (k ax n s ih1 l iy1 ax t ao1 r iy0))
("concise" nil (k ax n s ay1 s))
("concisely" nil (k ax n s ay1 s l iy0))
("conclave" nil (k aa1 n k l ey1 v))
("conclude" nil (k ax n k l uw1 d))
("concluded" nil (k ax n k l uw1 d ax d))
("concludes" nil (k ax n k l uw1 d z))
("concluding" nil (k ax n k l uw1 d ih0 ng))
("conclusion" nil (k ax n k l uw1 zh ax n))
("conclusions" nil (k ax n k l uw1 zh ax n z))
("conclusive" nil (k ax n k l uw1 s ih0 v))
("conclusively" nil (k ax n k l uw1 s ih0 v l iy0))
("concoct" nil (k ax n k aa1 k t))
("concocted" nil (k ax n k aa1 k t ax d))
("concocting" nil (k ax n k aa1 k t ih0 ng))
("concoction" nil (k ax n k aa1 k sh ax n))
("concoctions" nil (k ax n k aa1 k sh ax n z))
("concomitant" nil (k aa1 n k aa1 m ax t ax n t))
("concomitantly" nil (k aa1 n k aa1 m ax t ax n t l iy0))
("concord" nil (k aa1 n k ao1 r d))
("concorde" nil (k aa1 n k ao1 r d))
("concourse" nil (k aa1 n k ao1 r s))
("concourses" nil (k aa1 n k ao1 r s ih0 z))
("concrete" nil (k ax n k r iy1 t))
("concretely" nil (k aa1 n k r iy1 t l iy0))
("concubinage" nil (k aa0 n k y uw1 b ax n ax jh))
("concubines" nil (k aa1 n k y ax b ay1 n z))
("concur" nil (k ax n k er1))
("concurred" nil (k ax n k er1 d))
("concurrence" nil (k ax n k er1 ax n s))
("concurrent" nil (k ax n k er1 ax n t))
("concurrently" nil (k ax n k er1 ax n t l iy0))
("concurring" nil (k ax n k er1 ih0 ng))
("concurs" nil (k ax n k er1 z))
("concussion" nil (k ax n k ah1 sh ax n))
("concussions" nil (k ax n k ah1 sh ax n z))
("conde" nil (k aa1 n d))
("condello" nil (k ow0 n d eh1 l ow0))
("condemn" nil (k ax n d eh1 m))
("condemnation" nil (k aa1 n d ax m n ey1 sh ax n))
("condemnations" nil (k aa1 n d ax m n ey1 sh ax n z))
("condemned" nil (k ax n d eh1 m d))
("condemning" nil (k ax n d eh1 m ih0 ng))
("condemns" nil (k ax n d eh1 m z))
("condensate" nil (k aa1 n d ax n s ey1 t))
("condensates" nil (k aa1 n d ax n s ey1 t s))
("condensation" nil (k aa1 n d ax n s ey1 sh ax n))
("condense" nil (k ax n d eh1 n s))
("condensed" nil (k ax n d eh1 n s t))
("condenser" nil (k ax n d eh1 n s er0))
("conder" nil (k aa1 n d er0))
("condescending" nil (k aa1 n d ih0 s eh1 n d ih0 ng))
("condescension" nil (k aa1 n d ax s eh1 n sh ax n))
("condie" nil (k aa1 n d iy0))
("condiment" nil (k aa1 n d ax m ax n t))
("condiments" nil (k aa1 n d ax m ax n t s))
("condit" nil (k aa1 n d ih0 t))
("condition" nil (k ax n d ih1 sh ax n))
("conditional" nil (k ax n d ih1 sh ax n ax l))
("conditionality" nil (k ax n d ih1 sh ax n ae1 l ih0 t iy0))
("conditionally" nil (k ax n d ih1 sh ax n ax l iy0))
("conditioned" nil (k ax n d ih1 sh ax n d))
("conditioner" nil (k ax n d ih1 sh ax n er0))
("conditioners" nil (k ax n d ih1 sh ax n er0 z))
("conditioning" nil (k ax n d ih1 sh ax n ih0 ng))
("conditions" nil (k ax n d ih1 sh ax n z))
("conditt" nil (k aa1 n d ih0 t))
("condo" nil (k aa1 n d ow0))
("condolences" nil (k ax n d ow1 l ax n s ax z))
("condom" nil (k aa1 n d ax m))
("condominium" nil (k aa1 n d ax m ih1 n iy0 ax m))
("condominiums" nil (k aa1 n d ax m ih1 n iy0 ax m z))
("condoms" nil (k aa1 n d ax m z))
("condon" nil (k aa1 n d ax n))
("condone" nil (k ax n d ow1 n))
("condoned" nil (k ax n d ow1 n d))
("condones" nil (k ax n d ow1 n z))
("condoning" nil (k ax n d ow1 n ih0 ng))
("condor" nil (k aa1 n d er0))
("condos" nil (k aa1 n d ow0 z))
("condra" nil (k ow1 n d r ax))
("condracky" nil (k aa1 n d r ae1 k iy0))
("condray" nil (k aa1 n d r ey0))
("condren" nil (k aa1 n d er0 ax n))
("condrey" nil (k aa1 n d r iy0))
("condron" nil (k aa1 n d r ax n))
("condry" nil (k aa1 n d er0 iy0))
("conducive" nil (k ax n d uw1 s ih0 v))
("conduct" v (k ax n d ah1 k t))
("conduct" n (k ah1 n d ax k t))
("conducted" nil (k ax n d ah1 k t ax d))
("conducting" nil (k ax n d ah1 k t ih0 ng))
("conduction" nil (k ax n d ah1 k sh ax n))
("conductive" nil (k ax n d ah1 k t ih0 v))
("conductivity" nil (k aa1 n d ah1 k t ih1 v ax t iy0))
("conductor" nil (k ax n d ah1 k t er0))
("conductors" nil (k ax n d ah1 k t er0 z))
("conducts" nil (k ax n d ah1 k t s))
("conduit" nil (k aa1 n d uw0 ih0 t))
("conduits" nil (k aa1 n d uw0 ax t s))
("cone" nil (k ow1 n))
("coneflower" nil (k ow1 n f l aw1 er0))
("conely" nil (k ow1 n l iy0))
("conerly" nil (k ow1 n er0 l iy0))
("conery" nil (k ow1 n er0 iy0))
("cones" nil (k ow1 n z))
("conestoga" nil (k aa1 n ax s t ow1 g ax))
("coney" nil (k ow1 n iy0))
("confabulation" nil (k ax n f ae1 b y ax l ey1 sh ax n))
("confair" nil (k aa1 n f eh1 r))
("confection" nil (k ax n f eh1 k sh ax n))
("confectionary" nil (k ax n f eh1 k sh ax n eh1 r iy0))
("confectioner" nil (k ax n f eh1 k sh ax n er0))
("confectioners" nil (k ax n f eh1 k sh ax n er0 z))
("confectionery" nil (k ax n f eh1 k sh ax n eh1 r iy0))
("confections" nil (k ax n f eh1 k sh ax n z))
("confederacy" nil (k ax n f eh1 d er0 ax s iy0))
("confederate" nil (k ax n f eh1 d er0 ax t))
("confederation" nil (k ax n f eh1 d er0 ey1 sh ax n))
("confer" nil (k ax n f er1))
("conferees" nil (k aa1 n f er0 iy1 z))
("conference" nil (k aa1 n f er0 ax n s))
("conferences" nil (k aa1 n f er0 ax n s ax z))
("conferencing" nil (k aa1 n f r ax n s ih0 ng))
("conferred" nil (k ax n f er1 d))
("conferring" nil (k ax n f er1 ih0 ng))
("confers" nil (k ax n f er1 z))
("confess" nil (k ax n f eh1 s))
("confessed" nil (k ax n f eh1 s t))
("confesses" nil (k ax n f eh1 s ih0 z))
("confessing" nil (k ax n f eh1 s ih0 ng))
("confession" nil (k ax n f eh1 sh ax n))
("confessional" nil (k ax n f eh1 sh ax n ax l))
("confessionals" nil (k ax n f eh1 sh ax n ax l z))
("confessions" nil (k ax n f eh1 sh ax n z))
("confetti" nil (k ax n f eh1 t iy0))
("confidant" nil (k aa1 n f ax d aa1 n t))
("confidante" nil (k aa1 n f ax d ae1 n t))
("confidants" nil (k aa1 n f ax d ae1 n t s))
("confide" nil (k ax n f ay1 d))
("confided" nil (k ax n f ay1 d ax d))
("confidence" nil (k aa1 n f ax d ax n s))
("confidences" nil (k aa1 n f ax d ax n s ih0 z))
("confident" nil (k aa1 n f ax d ax n t))
("confidential" nil (k aa1 n f ax d eh1 n sh ax l))
("confidentiality" nil (k aa1 n f ax d eh1 n sh iy0 ae1 l ax t iy0))
("confidentially" nil (k aa1 n f ax d eh1 n sh ax l iy0))
("confidently" nil (k aa1 n f ax d ax n t l iy0))
("confides" nil (k ax n f ay1 d z))
("confiding" nil (k ax n f ay1 d ih0 ng))
("configuration" nil (k ax n f ih1 g y er0 ey1 sh ax n))
("configurations" nil (k ax n f ih1 g y er0 ey1 sh ax n z))
("configured" nil (k ax n f ih1 g y er0 d))
("confindustria" nil (k aa1 n f ih0 n d ah1 s t r iy0 ax))
("confine" v (k ax n f ay1 n))
("confine" n (k aa1 n f ay0 n))
("confined" nil (k ax n f ay1 n d))
("confinement" nil (k ax n f ay1 n m ax n t))
("confines" n (k aa1 n f ay1 n z))
("confines" v (k ax n f ay1 n z))
("confining" nil (k ax n f ay1 n ih0 ng))
("confirm" nil (k ax n f er1 m))
("confirmable" nil (k ax n f er1 m ax b ax l))
("confirmation" nil (k aa1 n f er0 m ey1 sh ax n))
("confirmations" nil (k aa1 n f er0 m ey1 sh ax n z))
("confirmatory" nil (k ax n f er1 m ax t ao1 r iy0))
("confirmed" nil (k ax n f er1 m d))
("confirming" nil (k ax n f er1 m ih0 ng))
("confirms" nil (k ax n f er1 m z))
("confiscate" nil (k aa1 n f ax s k ey1 t))
("confiscated" nil (k aa1 n f ax s k ey1 t ax d))
("confiscating" nil (k aa1 n f ax s k ey1 t ih0 ng))
("confiscation" nil (k aa1 n f ax s k ey1 sh ax n))
("confiscatory" nil (k ax n f ih1 s k ax t ao1 r iy0))
("confiterias" nil (k aa1 n f ih0 t ih1 r iy0 ax z))
("conflagration" nil (k aa1 n f l ax g r ey1 sh ax n))
("conflict" n (k aa1 n f l ih0 k t))
("conflict" v (k ax n f l ih1 k t))
("conflicted" nil (k ax n f l ih1 k t ih0 d))
("conflicting" nil (k ax n f l ih1 k t ih0 ng))
("conflicts" v (k ax n f l ih1 k t s))
("conflicts" n (k aa1 n f l ih0 k t s))
("confluence" nil (k aa1 n f l uw0 ax n s))
("conform" nil (k ax n f ao1 r m))
("conformance" nil (k ax n f ao1 r m ax n s))
("conformational" nil (k aa1 n f er0 m ey1 sh ax n ax l))
("conformed" nil (k ax n f ao1 r m d))
("conforming" nil (k ax n f ao1 r m ih0 ng))
("conformist" nil (k ax n f ao1 r m ih0 s t))
("conformists" nil (k ax n f ao1 r m ax s t s))
("conformity" nil (k ax n f ao1 r m ax t iy0))
("conforms" nil (k ax n f ao1 r m z))
("conforti" nil (k ow0 n f ao1 r t iy0))
("confound" nil (k aa0 n f aw1 n d))
("confounded" nil (k ax n f aw1 n d ih0 d))
("confounding" nil (k ax n f aw1 n d ih0 ng))
("confounds" nil (k ax n f aw1 n d z))
("confront" nil (k ax n f r ah1 n t))
("confrontation" nil (k aa1 n f r ax n t ey1 sh ax n))
("confrontational" nil (k aa1 n f r ax n t ey1 sh ax n ax l))
("confrontations" nil (k aa1 n f r ax n t ey1 sh ax n z))
("confronted" nil (k ax n f r ah1 n t ax d))
("confronting" nil (k ax n f r ah1 n t ih0 ng))
("confronts" nil (k ax n f r ah1 n t s))
("confucian" nil (k ax n f y uw1 sh ax n))
("confucianism" nil (k ax n f y uw1 sh ax n ih1 z ax m))
("confucius" nil (k ax n f y uw1 sh ax s))
("confuse" nil (k ax n f y uw1 z))
("confused" nil (k ax n f y uw1 z d))
("confuses" nil (k ax n f y uw1 z ih0 z))
("confusing" nil (k ax n f y uw1 z ih0 ng))
("confusingly" nil (k ax n f y uw1 z ih0 ng l iy0))
("confusion" nil (k ax n f y uw1 zh ax n))
("confusions" nil (k ax n f y uw1 zh ax n z))
("cong" nil (k ao1 ng))
("conga" nil (k ao1 ng g ax))
("congdon" nil (k aa1 ng d ax n))
("congeal" nil (k ax n jh iy1 l))
("congealed" nil (k ax n jh iy1 l d))
("congenial" nil (k ax n jh iy1 n y ax l))
("congeniality" nil (k ax n jh iy1 n iy0 ae1 l ax t iy0))
("congenital" nil (k ax n jh eh1 n ax t ax l))
("conger" nil (k ao1 ng er0))
("congested" nil (k ax n jh eh1 s t ax d))
("congestion" nil (k ax n jh eh1 s ch ax n))
("congestive" nil (k ax n jh eh1 s t ih0 v))
("congleton" nil (k aa1 ng g l ih0 t aa0 n))
("conglomerate" nil (k ax n g l aa1 m er0 ax t))
("conglomerates" nil (k ax n g l aa1 m er0 ax t s))
("conglomeration" nil (k ax n g l aa1 m er0 ey1 sh ax n))
("congo" nil (k aa1 ng g ow0))
("congratulate" nil (k ax n g r ae1 ch ax l ey1 t))
("congratulated" nil (k ax n g r ae1 ch ax l ey1 t ih0 d))
("congratulating" nil (k ax n g r ae1 ch ax l ey1 t ih0 ng))
("congratulation" nil (k ax n g r ae1 ch ax l ey1 sh ax n))
("congratulations" nil (k ax n g r ae1 ch ax l ey1 sh ax n z))
("congratulatory" nil (k ax n g r ae1 ch ax l ax t ao1 r iy0))
("congregate" nil (k aa1 ng g r ax g ey1 t))
("congregated" nil (k aa1 ng g r ih0 g ey1 t ih0 d))
("congregation" nil (k aa1 ng g r ax g ey1 sh ax n))
("congregational" nil (k aa1 ng g r ax g ey1 sh ax n ax l))
("congregations" nil (k aa1 ng g r ax g ey1 sh ax n z))
("congress" nil (k aa1 ng g r ax s))
("congresses" nil (k aa1 ng g r ax s ih0 z))
("congressional" nil (k ax n g r eh1 sh ax n ax l))
("congressionally" nil (k ax n g r eh1 sh ax n ax l iy0))
("congressman" nil (k aa1 ng g r ax s m ax n))
("congressmen" nil (k aa1 ng g r ax s m ih0 n))
("congresspersons" nil (k aa1 ng g r ax s p er1 s ax n z))
("congresswoman" nil (k aa1 ng g r ax s w uh1 m ax n))
("congrove" nil (k aa1 ng g r ax v))
("congruence" nil (k ao1 n g r uw0 ax n s))
("congruity" nil (k ax n g r uw1 ax t iy0))
("conical" nil (k aa1 n ih0 k ax l))
("conics" nil (k aa1 n ih0 k s))
("conifer" nil (k aa1 n ax f er0))
("coniferous" nil (k ax n ih1 f er0 ax s))
("conifers" nil (k aa1 n ax f er0 z))
("conigliaro" nil (k ax n ih1 g l iy0 aa1 r ow0))
("coniglio" nil (k ax n ih1 g l iy0 ow0))
("coniston" nil (k aa1 n ax s t ax n))
("conjecture" nil (k ax n jh eh1 k ch er0))
("conjectures" nil (k ax n jh eh1 k ch er0 z))
("conjunction" nil (k ax n jh ah1 ng k sh ax n))
("conjunctions" nil (k ax n jh ah1 ng k sh ax n z))
("conjunctiva" nil (k aa1 n jh ax ng k t ay1 v ax))
("conjure" nil (k aa1 n jh er0))
("conjured" nil (k aa1 n jh er0 d))
("conjures" nil (k aa1 n jh er0 z))
("conjuring" nil (k aa1 n jh er0 ih0 ng))
("conjuror" nil (k aa1 n jh er0 er0))
("conk" nil (k aa1 ng k))
("conkel" nil (k aa1 ng k ax l))
("conkey" nil (k aa1 n k iy0))
("conkin" nil (k aa1 ng k ih0 n))
("conkle" nil (k aa1 ng k ax l))
("conklin" nil (k aa1 ng k l ih0 n))
("conkling" nil (k aa1 ng k l ih0 ng))
("conkright" nil (k aa1 ng k r ay1 t))
("conlan" nil (k aa1 n l ax n))
("conlee" nil (k aa1 n l iy0))
("conley" nil (k aa1 n l iy0))
("conlin" nil (k aa1 n l ih0 n))
("conlon" nil (k aa1 n l ax n))
("conly" nil (k aa1 n l iy0))
("conn" nil (k aa1 n))
("connally" nil (k aa1 n ax l iy0))
("connaught" nil (k aa1 n ao0 t))
("connaughton" nil (k aa1 n ao0 t aa0 n))
("connaway" nil (k aa1 n ax w ey1))
("connect" nil (k ax n eh1 k t))
("connected" nil (k ax n eh1 k t ax d))
("connecter" nil (k ax n eh1 k t er0))
("connecticut" nil (k ax n eh1 t ax k ax t))
("connecting" nil (k ax n eh1 k t ih0 ng))
("connection" nil (k ax n eh1 k sh ax n))
("connections" nil (k ax n eh1 k sh ax n z))
("connective" nil (k ax n eh1 k t ih0 v))
("connectivity" nil (k ax n eh0 k t ih1 v ih0 t iy0))
("connector" nil (k ax n eh1 k t er0))
("connectors" nil (k ax n eh1 k t er0 z))
("connects" nil (k ax n eh1 k t s))
("conned" nil (k aa1 n d))
("conneely" nil (k ax n iy1 l iy0))
("connell" nil (k aa1 n ax l))
("connelley" nil (k aa1 n ih0 l iy0))
("connelly" nil (k aa1 n ax l iy0))
("connely" nil (k aa1 n l iy0))
("conner" nil (k aa1 n er0))
("conners" nil (k aa1 n er0 z))
("connery" nil (k aa1 n er0 iy0))
("connett" nil (k aa1 n ih0 t))
("connick" nil (k aa1 n ih0 k))
("connie" nil (k ao1 n iy0))
("conniff" nil (k aa1 n ih0 f))
("conning" nil (k aa1 n ih0 ng))
("conniption" nil (k ax n ih1 p sh ax n))
("connivance" nil (k ax n ay1 v ax n s))
("connive" nil (k ax n ay1 v))
("conniving" nil (k ax n ay1 v ih0 ng))
("connoisseur" nil (k aa1 n ax s er1))
("connoisseurs" nil (k aa1 n ax s er1 z))
("connole" nil (k aa1 n ax l))
("connolly" nil (k ao1 n ax l iy0))
("connon" nil (k aa1 n ax n))
("connor" nil (k aa1 n er0))
("connors" nil (k aa1 n er0 z))
("connotation" nil (k aa1 n ax t ey1 sh ax n))
("connotational" nil (k aa1 n ax t ey1 sh ax n ax l))
("connotations" nil (k aa1 n ax t ey1 sh ax n z))
("connote" nil (k ax n ow1 t))
("connotes" nil (k ax n ow1 t s))
("conny" nil (k aa1 n iy0))
("conoco" nil (k aa1 n ax k ow0))
("conolly" nil (k aa1 n ow0 l iy0))
("conoly" nil (k ow1 n ow0 l iy0))
("conover" nil (k aa1 n ax v er0))
("conquer" nil (k aa1 ng k er0))
("conquered" nil (k aa1 ng k er0 d))
("conquering" nil (k aa1 ng k er0 ih0 ng))
("conqueror" nil (k aa1 ng k er0 er0))
("conquerors" nil (k aa1 ng k er0 er0 z))
("conquers" nil (k aa1 ng k er0 z))
("conquest" nil (k aa1 ng k w eh0 s t))
("conquests" nil (k aa1 n k w eh1 s t s))
("conrac" nil (k aa1 n r ae0 k))
("conrad" nil (k aa1 n r ae0 d))
("conrades" nil (k ax n r ey1 d z))
("conradi" nil (k ow0 n r aa1 d iy0))
("conradine" nil (k aa1 n r ax d ay1 n))
("conrads" nil (k aa1 n r ae0 d z))
("conradt" nil (k aa1 n r ae0 t))
("conrady" nil (k ax n r aa1 d iy0))
("conrail" nil (k aa1 n r ey1 l))
("conran" nil (k aa1 n r ax n))
("conrath" nil (k aa1 n r ax th))
("conrey" nil (k aa1 n r iy0))
("conroe" nil (k aa1 n r ow0))
("conrow" nil (k aa1 n r ow0))
("conroy" nil (k ao1 n r oy0))
("conry" nil (k aa1 n r iy0))
("cons" nil (k aa1 n z))
("consalvo" nil (k ow0 n s aa1 l v ow0))
("conscience" nil (k aa1 n sh ax n s))
("consciences" nil (k aa1 n ch ih0 n s ih0 z))
("conscientious" nil (k aa1 n sh iy0 eh1 n sh ax s))
("conscientiously" nil (k aa1 n ch iy0 eh1 n ch ax s l iy0))
("conscious" nil (k aa1 n sh ax s))
("consciously" nil (k aa1 n sh ax s l iy0))
("consciousness" nil (k aa1 n sh ax s n ax s))
("conscript" n (k aa1 n s k r ih0 p t))
("conscript" v (k ax n s k r ih1 p t))
("conscripted" nil (k ax n s k r ih1 p t ih0 d))
("conscription" nil (k ax n s k r ih1 p sh ax n))
("conscripts" n (k aa1 n s k r ih0 p t s))
("conscripts" v (k ax n s k r ih1 p t s))
("conseco" nil (k aa0 n s ey1 k ow0))
("consecrated" nil (k aa1 n s ax k r ey1 t ax d))
("consecutive" nil (k ax n s eh1 k y ax t ih0 v))
("consensual" nil (k ax n s eh1 n s uw0 ax l))
("consensus" nil (k ax n s eh1 n s ax s))
("consent" nil (k ax n s eh1 n t))
("consented" nil (k ax n s eh1 n t ih0 d))
("consenting" nil (k ax n s eh1 n t ih0 ng))
("consentino" nil (k ow0 n s eh0 n t iy1 n ow0))
("consents" nil (k ax n s eh1 n t s))
("consequence" nil (k aa1 n s ax k w ax n s))
("consequences" nil (k aa1 n s ax k w eh1 n s ax z))
("consequent" nil (k aa1 n s ax k w ax n t))
("consequential" nil (k aa1 n s ax k w eh1 n ch ax l))
("consequently" nil (k aa1 n s ax k w ax n t l iy0))
("conser" nil (k aa1 n s er0))
("conservancy" nil (k ax n s er1 v ax n s iy0))
("conservation" nil (k aa1 n s er0 v ey1 sh ax n))
("conservationist" nil (k aa1 n s er0 v ey1 sh ax n ax s t))
("conservationists" nil (k aa1 n s er0 v ey1 sh ax n ax s t s))
("conservatism" nil (k ax n s er1 v ax t ih1 z ax m))
("conservative" nil (k ax n s er1 v ax t ih0 v))
("conservatively" nil (k ax n s er1 v ax t ih0 v l iy0))
("conservatives" nil (k ax n s er1 v ax t ih0 v z))
("conservator" nil (k ax n s er1 v ax t er0))
("conservatories" nil (k ax n s er1 v ax t ao1 r iy0 z))
("conservators" nil (k ax n s er1 v ax t er0 z))
("conservatorship" nil (k ax n s er1 v ax t er0 sh ih1 p))
("conservatory" nil (k ax n s er1 v ax t ao0 r iy0))
("conserve" nil (k ax n s er1 v))
("conserved" nil (k ax n s er1 v d))
("conserving" nil (k ax n s er1 v ih0 ng))
("conshohocken" nil (k aa1 n sh ax hh aa1 k ax n))
("consider" nil (k ax n s ih1 d er0))
("considerable" nil (k ax n s ih1 d er0 ax b ax l))
("considerably" nil (k ax n s ih1 d er0 ax b l iy0))
("considerate" nil (k ax n s ih1 d er0 ax t))
("consideration" nil (k ax n s ih1 d er0 ey1 sh ax n))
("considerations" nil (k ax n s ih1 d er0 ey1 sh ax n z))
("considered" nil (k ax n s ih1 d er0 d))
("considering" nil (k ax n s ih1 d er0 ih0 ng))
("considers" nil (k ax n s ih1 d er0 z))
("considine" nil (k aa1 n s ih0 d ay1 n))
("consiglio" nil (k ow0 n s iy1 g l iy0 ow0))
("consign" nil (k ax n s ay1 n))
("consigned" nil (k ax n s ay1 n d))
("consignment" nil (k ax n s ay1 n m ax n t))
("consist" nil (k ax n s ih1 s t))
("consisted" nil (k ax n s ih1 s t ax d))
("consistence" nil (k ax n s ih1 s t ax n s))
("consistency" nil (k ax n s ih1 s t ax n s iy0))
("consistent" nil (k ax n s ih1 s t ax n t))
("consistently" nil (k ax n s ih1 s t ax n t l iy0))
("consisting" nil (k ax n s ih1 s t ih0 ng))
("consists" nil (k ax n s ih1 s t s))
("consob" nil (k aa1 n s aa0 b))
("consol" nil (k aa1 n s aa0 l))
("consolata" nil (k ow0 n s ow0 l aa1 t ax))
("consolation" nil (k aa1 n s ax l ey1 sh ax n))
("consolations" nil (k aa1 n s ax l ey1 sh ax n z))
("console" n (k aa1 n s ow0 l))
("console" v (k ax n s ow1 l))
("consoled" nil (k ax n s ow1 l d))
("consoles" v (k ax n s ow1 l z))
("consoles" n (k aa1 n s ow0 l z))
("consoli" nil (k ow0 n s ow1 l iy0))
("consolidate" nil (k ax n s aa1 l ih0 d ey1 t))
("consolidated" nil (k ax n s aa1 l ax d ey1 t ax d))
("consolidates" nil (k ax n s aa1 l ih0 d ey1 t s))
("consolidating" nil (k ax n s aa1 l ax d ey1 t ih0 ng))
("consolidation" nil (k ax n s aa1 l ax d ey1 sh ax n))
("consolidations" nil (k ax n s aa1 l ih0 d ey1 sh ax n z))
("consolidators" nil (k ax n s aa1 l ih0 d ey1 t er0 z))
("consoling" nil (k ax n s ow1 l ih0 ng))
("consolo" nil (k ow0 n s ow1 l ow0))
("consonant" nil (k aa1 n s ax n ax n t))
("consonantal" nil (k ae1 n s ax n ae1 n t ax l))
("consonants" nil (k aa1 n s ax n ax n t s))
("consort" v (k ax n s ao1 r t))
("consort" n (k aa1 n s ao0 r t))
("consortia" nil (k ax n s ao1 r sh ax))
("consorting" nil (k ax n s ao1 r t ih0 ng))
("consortium" nil (k ax n s ao1 r sh iy0 ax m))
("consortiums" nil (k ax n s ao1 r sh ax m z))
("consorts" v (k ax n s ao1 r t s))
("consorts" n (k aa1 n s ao0 r t s))
("conspicuous" nil (k ax n s p ih1 k y uw0 ax s))
("conspicuously" nil (k ax n s p ih1 k y uw0 ax s l iy0))
("conspiracies" nil (k ax n s p ih1 r ax s iy0 z))
("conspiracy" nil (k ax n s p ih1 r ax s iy0))
("conspirator" nil (k ax n s p ih1 r ax t er0))
("conspiratorial" nil (k ax n s p ih1 r ax t ao1 r iy0 ax l))
("conspirators" nil (k ax n s p ih1 r ax t er0 z))
("conspire" nil (k ax n s p ay1 er0))
("conspired" nil (k ax n s p ay1 er0 d))
("conspiring" nil (k ax n s p ay1 r ih0 ng))
("constable" nil (k aa1 n s t ax b ax l))
("constables" nil (k aa1 n s t ax b ax l z))
("constabulary" nil (k ax n s t ae1 b y ax l eh1 r iy0))
("constance" nil (k aa1 n s t ax n s))
("constancy" nil (k aa1 n s t ax n s iy0))
("constant" nil (k aa1 n s t ax n t))
("constanta" nil (k ow0 n s t aa1 n t ax))
("constantin" nil (k ax n s t ae1 n t ih0 n))
("constantina" nil (k ow0 n s t aa0 n t iy1 n ax))
("constantine" nil (k aa1 n s t ax n t iy1 n))
("constantino" nil (k ow0 n s t aa0 n t iy1 n ow0))
("constantinople" nil (k aa1 n s t ae0 n t ax n ow1 p ax l))
("constantly" nil (k aa1 n s t ax n t l iy0))
("constants" nil (k aa1 n s t ax n t s))
("constar" nil (k aa1 n s t aa1 r))
("constellation" nil (k aa1 n s t ax l ey1 sh ax n))
("constellations" nil (k aa1 n s t ax l ey1 sh ax n z))
("consternation" nil (k aa1 n s t er0 n ey1 sh ax n))
("constipated" nil (k aa1 n s t ax p ey1 t ax d))
("constipation" nil (k aa1 n s t ax p ey1 sh ax n))
("constituencies" nil (k ax n s t ih1 ch uw0 ax n s iy0 z))
("constituency" nil (k ax n s t ih1 ch uw0 ax n s iy0))
("constituent" nil (k ax n s t ih1 ch uw0 ax n t))
("constituents" nil (k ax n s t ih1 ch uw0 ax n t s))
("constitute" nil (k aa1 n s t ax t uw1 t))
("constituted" nil (k aa1 n s t ax t uw1 t ax d))
("constitutes" nil (k aa1 n s t ax t uw1 t s))
("constituting" nil (k aa1 n s t ax t uw1 t ih0 ng))
("constitution" nil (k aa1 n s t ax t uw1 sh ax n))
("constitutional" nil (k aa1 n s t ax t uw1 sh ax n ax l))
("constitutionality" nil (k aa1 n s t ih0 t uw1 sh ax n ae1 l ih0 t iy0))
("constitutionally" nil (k aa1 n s t ax t uw1 sh ax n ax l iy0))
("constitutions" nil (k aa1 n s t ih0 t uw1 sh ax n z))
("constrain" nil (k ax n s t r ey1 n))
("constrained" nil (k ax n s t r ey1 n d))
("constraining" nil (k ax n s t r ey1 n ih0 ng))
("constrains" nil (k ax n s t r ey1 n z))
("constraint" nil (k ax n s t r ey1 n t))
("constraints" nil (k ax n s t r ey1 n t s))
("constrict" nil (k ax n s t r ih1 k t))
("constricted" nil (k ax n s t r ih1 k t ax d))
("constricting" nil (k ax n s t r ih1 k t ih0 ng))
("constriction" nil (k ax n s t r ih1 k sh ax n))
("constrictions" nil (k ax n s t r ih1 k sh ax n z))
("constrictor" nil (k ax n s t r ih1 k t er0))
("constrictors" nil (k ax n s t r ih1 k t er0 z))
("construcciones" nil (k ax n s t r uw1 ch iy0 ow1 n ey0 z))
("construct" nil (k ax n s t r ah1 k t))
("constructed" nil (k ax n s t r ah1 k t ax d))
("constructing" nil (k ax n s t r ah1 k t ih0 ng))
("construction" nil (k ax n s t r ah1 k sh ax n))
("constructionist" nil (k ax n s t r ah1 k sh ax n ih0 s t))
("constructions" nil (k ax n s t r ah1 k sh ax n z))
("constructive" nil (k ax n s t r ah1 k t ih0 v))
("constructively" nil (k ax n s t r ah1 k t ih0 v l iy0))
("constructors" nil (k ax n s t r ah1 k t er0 z))
("constructs" nil (k ax n s t r ah1 k t s))
("construe" nil (k ax n s t r uw1))
("construed" nil (k ax n s t r uw1 d))
("consuela" nil (k ow0 n s uw1 l ax))
("consul" nil (k aa1 n s ax l))
("consular" nil (k aa1 n s ax l er0))
("consulate" nil (k aa1 n s ax l ax t))
("consulates" nil (k aa1 n s ax l ax t s))
("consulship" nil (k aa1 n s ax l sh ih1 p))
("consult" nil (k ax n s ah1 l t))
("consultancy" nil (k ax n s ah1 l t ax n s iy0))
("consultant" nil (k ax n s ah1 l t ax n t))
("consultants" nil (k ax n s ah1 l t ax n t s))
("consultation" nil (k aa1 n s ax l t ey1 sh ax n))
("consultations" nil (k aa1 n s ax l t ey1 sh ax n z))
("consultative" nil (k ax n s ah1 l t ax t ih0 v))
("consulted" nil (k ax n s ah1 l t ax d))
("consulting" nil (k ax n s ah1 l t ih0 ng))
("consults" nil (k ax n s ah1 l t s))
("consumable" nil (k ax n s uw1 m ax b ax l))
("consume" nil (k ax n s uw1 m))
("consumed" nil (k ax n s uw1 m d))
("consumer" nil (k ax n s uw1 m er0))
("consumerism" nil (k ax n s uw1 m er0 ih1 z ax m))
("consumerist" nil (k ax n s uw1 m er0 ih0 s t))
("consumerists" nil (k ax n s uw1 m er0 ih0 s t s))
("consumers" nil (k ax n s uw1 m er0 z))
("consumes" nil (k ax n s uw1 m z))
("consuming" nil (k ax n s uw1 m ih0 ng))
("consummate" nil (k aa1 n s ax m ax t))
("consummated" nil (k aa1 n s ax m ey1 t ax d))
("consummating" nil (k aa1 n s ax m ey1 t ih0 ng))
("consummation" nil (k aa1 n s ax m ey1 sh ax n))
("consumption" nil (k ax n s ah1 m p sh ax n))
("contac" nil (k aa1 n t ae0 k))
("contact" nil (k aa1 n t ae1 k t))
("contacted" nil (k aa1 n t ae1 k t ih0 d))
("contacting" nil (k aa1 n t ae1 k t ih0 ng))
("contacts" nil (k aa1 n t ae1 k t s))
("contadora" nil (k aa1 n t ax d ao1 r ax))
("contagion" nil (k ax n t ey1 jh ax n))
("contagious" nil (k ax n t ey1 jh ax s))
("contagiousness" nil (k ax n t ey1 jh ax s n ax s))
("contain" nil (k ax n t ey1 n))
("contained" nil (k ax n t ey1 n d))
("container" nil (k ax n t ey1 n er0))
("containerboard" nil (k ax n t ey1 n er0 b ao1 r d))
("containerized" nil (k ax n t ey1 n er0 ay1 z d))
("containers" nil (k ax n t ey1 n er0 z))
("containing" nil (k ax n t ey1 n ih0 ng))
("containment" nil (k ax n t ey1 n m ax n t))
("contains" nil (k ax n t ey1 n z))
("contaminant" nil (k ax n t ae1 m ax n ax n t))
("contaminants" nil (k ax n t ae1 m ax n ax n t s))
("contaminate" nil (k ax n t ae1 m ax n ey1 t))
("contaminated" nil (k ax n t ae1 m ax n ey1 t ax d))
("contaminating" nil (k ax n t ae1 m ax n ey1 t ih0 ng))
("contamination" nil (k ax n t ae1 m ax n ey1 sh ax n))
("contant" nil (k aa1 n t ax n t))
("conte" nil (k ow1 n t))
("contel" nil (k aa1 n t eh1 l))
("contemplate" nil (k aa1 n t ax m p l ey1 t))
("contemplated" nil (k aa1 n t ax m p l ey1 t ih0 d))
("contemplates" nil (k aa1 n t ax m p l ey1 t s))
("contemplating" nil (k aa1 n t ax m p l ey1 t ih0 ng))
("contemplation" nil (k aa1 n t ax m p l ey1 sh ax n))
("contemplative" nil (k ax n t eh1 m p l ax t ih0 v))
("contempo" nil (k aa1 n t eh1 m p ow0))
("contemporaneous" nil (k ax n t eh1 m p er0 ae1 n iy0 ax s))
("contemporaries" nil (k ax n t eh1 m p er0 eh1 r iy0 z))
("contemporary" nil (k ax n t eh1 m p er0 eh1 r iy0))
("contempt" nil (k ax n t eh1 m p t))
("contemptible" nil (k ax n t eh1 m p t ax b ax l))
("contemptuous" nil (k ax n t eh1 m p ch uw0 ax s))
("contemptuously" nil (k ax n t eh1 m p ch w ax s l iy0))
("contend" nil (k ax n t eh1 n d))
("contended" nil (k ax n t eh1 n d ih0 d))
("contender" nil (k ax n t eh1 n d er0))
("contenders" nil (k ax n t eh1 n d er0 z))
("contending" nil (k ax n t eh1 n d ih0 ng))
("contends" nil (k ax n t eh1 n d z))
("content" n (k aa1 n t eh0 n t))
("content" v (k ax n t eh1 n t))
("contented" nil (k ax n t eh1 n t ax d))
("contentedly" nil (k ax n t eh1 n t ax d l iy0))
("contention" nil (k ax n t eh1 n sh ax n))
("contentions" nil (k ax n t eh1 n sh ax n z))
("contentious" nil (k ax n t eh1 n sh ax s))
("contentiousness" nil (k ax n t eh1 n sh ax s n ax s))
("contentment" nil (k ax n t eh1 n t m ax n t))
("contento" nil (k ow0 n t eh1 n t ow0))
("contents" n (k aa1 n t eh0 n t s))
("contents" v (k ax n t eh1 n t s))
("contest" n (k aa1 n t eh0 s t))
("contest" v (k ax n t eh1 s t))
("contestable" nil (k ax n t eh1 s t ax b ax l))
("contestant" nil (k ax n t eh1 s t ax n t))
("contestants" nil (k ax n t eh1 s t ax n t s))
("contested" nil (k ax n t eh1 s t ax d))
("contesting" nil (k ax n t eh1 s t ih0 ng))
("contests" n (k aa1 n t eh0 s t s))
("contests" v (k ax n t eh1 s t s))
("context" nil (k aa1 n t eh0 k s t))
("contexts" nil (k aa1 n t eh1 k s t s))
("conti" nil (k ow1 n t iy0))
("contibel" nil (k aa1 n t ih0 b ax l))
("conticommodity" nil (k aa1 n t ih0 k ax m aa1 d ih0 t iy0))
("contiguous" nil (k ax n t ih1 g y uw0 ax s))
("continent" nil (k aa1 n t ax n ax n t))
("continental" nil (k aa1 n t ax n eh1 n t ax l))
("continentally" nil (k aa1 n t ax n eh1 n t ax l iy0))
("continentals" nil (k aa1 n t ax n eh1 n t ax l z))
("continents" nil (k aa1 n t ax n ax n t s))
("contingencies" nil (k ax n t ih1 n jh ax n s iy0 z))
("contingency" nil (k ax n t ih1 n jh ax n s iy0))
("contingent" nil (k ax n t ih1 n jh ax n t))
("contingents" nil (k ax n t ih1 n jh ax n t s))
("contini" nil (k ow0 n t iy1 n iy0))
("contino" nil (k ow0 n t iy1 n ow0))
("continual" nil (k ax n t ih1 n y uw0 ax l))
("continually" nil (k ax n t ih1 n y uw0 ax l iy0))
("continuance" nil (k ax n t ih1 n y uw0 ax n s))
("continuances" nil (k ax n t ih1 n y uw0 ax n s ih0 z))
("continuation" nil (k ax n t ih1 n y uw0 ey1 sh ax n))
("continue" nil (k ax n t ih1 n y uw0))
("continued" nil (k ax n t ih1 n y uw0 d))
("continues" nil (k ax n t ih1 n y uw0 z))
("continuing" nil (k ax n t ih1 n y uw0 ih0 ng))
("continuity" nil (k aa1 n t ax n uw1 ax t iy0))
("continuous" nil (k ax n t ih1 n y uw0 ax s))
("continuously" nil (k ax n t ih1 n y uw0 ax s l iy0))
("continuum" nil (k ax n t ih1 n y uw0 ax m))
("contois" nil (k ax n t w aa1))
("contort" nil (k ax n t ao1 r t))
("contorted" nil (k ax n t ao1 r t ax d))
("contortion" nil (k ax n t ao1 r sh ax n))
("contortionist" nil (k ax n t ao1 r sh ax n ax s t))
("contortions" nil (k ax n t ao1 r sh ax n z))
("contorts" nil (k ax n t ao1 r t s))
("contos" nil (k ow1 n t ow0 z))
("contour" nil (k aa1 n t uh1 r))
("contoured" nil (k aa1 n t uh1 r d))
("contours" nil (k aa1 n t uh1 r z))
("contra" nil (k aa1 n t r ax))
("contraband" nil (k aa1 n t r ax b ae1 n d))
("contrabassoon" nil (k aa1 n t r ax b ae0 s uw1 n))
("contraception" nil (k aa1 n t r ax s eh1 p sh ax n))
("contraceptive" nil (k aa1 n t r ax s eh1 p t ih0 v))
("contraceptives" nil (k aa1 n t r ax s eh1 p t ih0 v z))
("contract" j (k aa1 n t r ae0 k t))
("contract" n (k aa1 n t r ae1 k t))
("contract" v (k ax n t r ae1 k t))
("contracted" nil (k aa1 n t r ae0 k t ax d))
("contracting" nil (k aa1 n t r ae0 k t ih0 ng))
("contraction" nil (k ax n t r ae1 k sh ax n))
("contractions" nil (k ax n t r ae1 k sh ax n z))
("contractor" nil (k aa1 n t r ae1 k t er0))
("contractors" nil (k aa1 n t r ae1 k t er0 z))
("contracts" n (k aa1 n t r ae1 k t s))
("contracts" v (k ax n t r ae1 k t s))
("contractual" nil (k ax n t r ae1 k ch uw0 ax l))
("contractually" nil (k ax n t r ae1 k ch uw0 ax l iy0))
("contradict" nil (k aa1 n t r ax d ih1 k t))
("contradicted" nil (k aa1 n t r ax d ih1 k t ax d))
("contradicting" nil (k aa1 n t r ax d ih1 k t ih0 ng))
("contradiction" nil (k aa1 n t r ax d ih1 k sh ax n))
("contradictions" nil (k aa1 n t r ax d ih1 k sh ax n z))
("contradictorily" nil (k aa1 n t r ax d ih1 k t er0 ax l iy0))
("contradictory" nil (k aa1 n t r ax d ih1 k t er0 iy0))
("contradicts" nil (k aa1 n t r ax d ih1 k t s))
("contran" nil (k aa1 n t r ae1 n))
("contraption" nil (k ax n t r ae1 p sh ax n))
("contraptions" nil (k ax n t r ae1 p sh ax n z))
("contrarian" nil (k aa1 n t r eh1 r iy0 ax n))
("contrarians" nil (k ax n t r eh1 r iy0 ax n z))
("contrariness" nil (k aa1 n t r eh0 r iy0 n ax s))
("contrary" n (k aa1 n t r eh0 r iy0))
("contrary" j (k aa1 n t r eh1 r iy0))
("contras" nil (k aa1 n t r ax z))
("contrast" n (k aa1 n t r ae0 s t))
("contrast" v (k ax n t r ae1 s t))
("contrasted" nil (k ax n t r ae1 s t ax d))
("contrasting" nil (k ax n t r ae1 s t ih0 ng))
("contrasts" n (k aa1 n t r ae0 s t s))
("contrasts" v (k ax n t r ae1 s t s))
("contravene" nil (k aa1 n t r ax v iy1 n))
("contravention" nil (k aa1 n t r ax v eh1 n ch ax n))
("contreras" nil (k ow0 n t r eh1 r aa0 z))
("contretemps" nil (k aa1 n t r ax t eh1 m p s))
("contribute" nil (k ax n t r ih1 b y uw0 t))
("contributed" nil (k ax n t r ih1 b y uw0 t ih0 d))
("contributes" nil (k ax n t r ih1 b y uw0 t s))
("contributing" nil (k ax n t r ih1 b y uw0 t ih0 ng))
("contribution" nil (k aa1 n t r ax b y uw1 sh ax n))
("contributions" nil (k aa1 n t r ax b y uw1 sh ax n z))
("contributor" nil (k ax n t r ih1 b y ax t er0))
("contributors" nil (k ax n t r ih1 b y ax t er0 z))
("contributory" nil (k ax n t r ih1 b y ax t ao1 r iy0))
("contrite" nil (k ax n t r ay1 t))
("contrition" nil (k ax n t r ih1 sh ax n))
("contrivance" nil (k ax n t r ay1 v ax n s))
("contrivances" nil (k ax n t r ay1 v ax n s ih0 z))
("contrive" nil (k ax n t r ay1 v))
("contrived" nil (k ax n t r ay1 v d))
("control" nil (k ax n t r ow1 l))
("controladora" nil (k ax n t r ow1 l ax d ao1 r ax))
("controllable" nil (k ax n t r ow1 l ax b ax l))
("controlled" nil (k ax n t r ow1 l d))
("controller" nil (k ax n t r ow1 l er0))
("controllers" nil (k ax n t r ow1 l er0 z))
("controlling" nil (k ax n t r ow1 l ih0 ng))
("controls" nil (k ax n t r ow1 l z))
("controversial" nil (k aa1 n t r ax v er1 sh ax l))
("controversies" nil (k aa1 n t r ax v er1 s iy0 z))
("controversy" nil (k aa1 n t r ax v er1 s iy0))
("contruction" nil (k ax n t r ah1 k sh ax n))
("contura" nil (k aa1 n t uh1 r ax))
("contusion" nil (k ax n t uw1 zh ax n))
("conundrum" nil (k ax n ah1 n d r ax m))
("conus" nil (k ow1 n ax s))
("convair" nil (k aa0 n v eh1 r))
("convalesce" nil (k aa1 n v ax l eh1 s))
("convalescence" nil (k aa1 n v ax l eh1 s ax n s))
("convalescent" nil (k aa1 n v ax l eh1 s ax n t))
("convene" nil (k ax n v iy1 n))
("convened" nil (k ax n v iy1 n d))
("convenes" nil (k ax n v iy1 n z))
("convenience" nil (k ax n v iy1 n y ax n s))
("conveniences" nil (k ax n v iy1 n y ax n s ih0 z))
("convenient" nil (k ax n v iy1 n y ax n t))
("conveniently" nil (k ax n v iy1 n y ax n t l iy0))
("convening" nil (k ax n v iy1 n ih0 ng))
("convent" nil (k aa1 n v ax n t))
("convention" nil (k ax n v eh1 n sh ax n))
("conventional" nil (k ax n v eh1 n sh ax n ax l))
("conventionally" nil (k ax n v eh1 n sh ax n ax l iy0))
("conventioneer" nil (k ax n v eh1 n sh ax n ih1 r))
("conventioneers" nil (k ax n v eh1 n sh ax n ih1 r z))
("conventions" nil (k ax n v eh1 n sh ax n z))
("converge" nil (k ax n v er1 jh))
("converged" nil (k ax n v er1 jh d))
("convergence" nil (k ax n v er1 jh ax n s))
("convergent" nil (k ax n v er1 jh ax n t))
("converging" nil (k ax n v er1 jh ih0 ng))
("conversant" nil (k ax n v er1 s ax n t))
("conversation" nil (k aa1 n v er0 s ey1 sh ax n))
("conversational" nil (k aa1 n v er0 s ey1 sh ax n ax l))
("conversationalist" nil (k aa1 n v er0 s ey1 sh ax n ax l ax s t))
("conversations" nil (k aa1 n v er0 s ey1 sh ax n z))
("converse" n (k aa1 n v er0 s))
("converse" v (k ax n v er1 s))
("conversed" nil (k ax n v er1 s t))
("conversely" nil (k aa1 n v er0 s l iy0))
("conversing" nil (k ax n v er1 s ih0 ng))
("conversion" nil (k ax n v er1 zh ax n))
("conversions" nil (k ax n v er1 zh ax n z))
("convert" n (k aa1 n v er0 t))
("convert" v (k ax n v er1 t))
("converted" nil (k ax n v er1 t ih0 d))
("converter" nil (k ax n v er1 t er0))
("converters" nil (k ax n v er1 t er0 z))
("convertibility" nil (k aa1 n v er0 t ih0 b ih1 l ih0 t iy0))
("convertible" nil (k ax n v er1 t ax b ax l))
("convertibles" nil (k ax n v er1 t ax b ax l z))
("converting" nil (k ax n v er1 t ih0 ng))
("converts" n (k aa1 n v er0 t s))
("converts" v (k ax n v er1 t s))
("convery" nil (k aa1 n v er0 iy0))
("convex" nil (k ax n v eh1 k s))
("convey" nil (k ax n v ey1))
("conveyance" nil (k ax n v ey1 ax n s))
("conveyed" nil (k ax n v ey1 d))
("conveyer" nil (k ax n v ey1 er0))
("conveying" nil (k ax n v ey1 ih0 ng))
("conveyor" nil (k ax n v ey1 er0))
("conveys" nil (k ax n v ey1 z))
("convict" v (k ax n v ih1 k t))
("convict" n (k aa1 n v ih0 k t))
("convicted" nil (k ax n v ih1 k t ax d))
("convicting" nil (k ax n v ih1 k t ih0 ng))
("conviction" nil (k ax n v ih1 k sh ax n))
("convictions" nil (k ax n v ih1 k sh ax n z))
("convicts" n (k aa1 n v ih0 k t s))
("convicts" v (k ax n v ih1 k t s))
("conville" nil (k ow1 n v ih0 l))
("convince" nil (k ax n v ih1 n s))
("convinced" nil (k ax n v ih1 n s t))
("convinces" nil (k ax n v ih1 n s ih0 z))
("convincing" nil (k ax n v ih1 n s ih0 ng))
("convincingly" nil (k ax n v ih1 n s ih0 ng l iy0))
("convivial" nil (k ax n v ih1 v iy0 ax l))
("convocation" nil (k aa1 n v ax k ey1 sh ax n))
("convoluted" nil (k aa1 n v ax l uw1 t ax d))
("convoy" nil (k aa1 n v oy1))
("convoys" nil (k aa1 n v oy1 z))
("convulsion" nil (k ax n v ah1 l sh ax n))
("convulsions" nil (k ax n v ah1 l sh ax n z))
("convulsive" nil (k ax n v ah1 l s ih0 v))
("conway" nil (k aa1 n w ey1))
("conwell" nil (k aa1 n w eh1 l))
("conyer" nil (k aa1 n y er0))
("conyers" nil (k aa1 n y er0 z))
("coody" nil (k uw1 d iy0))
("coogan" nil (k uw1 g ax n))
("coogle" nil (k uw1 g ax l))
("coogler" nil (k uw1 g ax l er0))
("cook" nil (k uh1 k))
("cookbook" nil (k uh1 k b uh1 k))
("cookbooks" nil (k uh1 k b uh1 k s))
("cooke" nil (k uh1 k))
("cooked" nil (k uh1 k t))
("cooker" nil (k uh1 k er0))
("cookers" nil (k uh1 k er0 z))
("cookie" nil (k uh1 k iy0))
("cookies" nil (k uh1 k iy0 z))
("cooking" nil (k uh1 k ih0 ng))
("cookingham" nil (k uw1 k ih0 ng hh ae0 m))
("cookman" nil (k uh1 k m ax n))
("cookout" nil (k uh1 k aw1 t))
("cookouts" nil (k uh1 k aw1 t s))
("cooks" nil (k uh1 k s))
("cooksey" nil (k uw1 k s iy0))
("cookson" nil (k uw1 k s ax n))
("cookston" nil (k uh1 k s t ax n))
("cookware" nil (k uh1 k w eh1 r))
("cool" nil (k uw1 l))
("coolant" nil (k uw1 l ax n t))
("coolants" nil (k uw1 l ax n t s))
("coolbaugh" nil (k uw1 l b ao1))
("coole" nil (k uw1 l))
("cooled" nil (k uw1 l d))
("cooler" nil (k uw1 l er0))
("coolers" nil (k uw1 l er0 z))
("coolest" nil (k uw1 l ax s t))
("cooley" nil (k uw1 l iy0))
("coolidge" nil (k uw1 l ih0 jh))
("cooling" nil (k uw1 l ih0 ng))
("coolly" nil (k uw1 l iy0))
("coolman" nil (k uw1 l m ax n))
("coolness" nil (k uw1 l n ax s))
("cools" nil (k uw1 l z))
("coombe" nil (k uw1 m b))
("coombes" nil (k uw1 m b z))
("coombs" nil (k uw1 m z))
("coomer" nil (k uw1 m er0))
("coomes" nil (k uw1 m z))
("coon" nil (k uw1 n))
("coonan" nil (k uw1 n ax n))
("coonce" nil (k uw1 n s))
("coone" nil (k uw1 n))
("cooner" nil (k uw1 n er0))
("coones" nil (k uw1 n z))
("cooney" nil (k uw1 n iy0))
("coonradt" nil (k uw1 n r ae0 t))
("coonrod" nil (k uw1 n r aa1 d))
("coons" nil (k uw1 n z))
("coontz" nil (k uw1 n t s))
("coop" nil (k uw1 p))
("cooped" nil (k uw1 p t))
("cooper" nil (k uw1 p er0))
("cooperage" nil (k uw1 p er0 ax jh))
("cooperate" nil (k ow0 aa1 p er0 ey1 t))
("cooperated" nil (k ow0 aa1 p er0 ey1 t ax d))
("cooperates" nil (k ow0 aa1 p er0 ey1 t s))
("cooperating" nil (k ow0 aa1 p er0 ey1 t ih0 ng))
("cooperation" nil (k ow0 aa1 p er0 ey1 sh ax n))
("cooperations" nil (k ow0 aa1 p er0 ey1 sh ax n z))
("cooperative" nil (k ow0 aa1 p er0 ey1 t ih0 v))
("cooperatively" nil (k ow0 aa1 p r ax t ih0 v l iy0))
("cooperatives" nil (k ow0 aa1 p r ax t ih0 v z))
("cooperider" nil (k uw1 p er0 ay0 d er0))
("cooperman" nil (k uw1 p er0 m ae1 n))
("cooperrider" nil (k uw1 p er0 ay0 d er0))
("coopers" nil (k uw1 p er0 z))
("coopersmith" nil (k uw1 p er0 s m ih1 th))
("cooperstein" nil (k uw1 p er0 s t ay1 n))
("cooperstown" nil (k uw1 p er0 z t aw1 n))
("coopervision" nil (k uw1 p er0 v ih1 zh ax n))
("coopman" nil (k uw1 p m ax n))
("cooprider" nil (k uw1 p r ay1 d er0))
("coordinate" n (k ow0 ao1 r d ax n ax t))
("coordinate" v (k ow0 ao1 r d ax n ey1 t))
("coordinated" nil (k ow0 ao1 r d ax n ey0 t ih0 d))
("coordinates" n (k ow0 ao1 r d ax n ax t s))
("coordinates" v (k ow0 ao1 r d ax n ey1 t s))
("coordinating" nil (k ow0 ao1 r d ax n ey1 t ih0 ng))
("coordination" nil (k ow0 ao1 r d ax n ey1 sh ax n))
("coordinator" nil (k ow0 ao1 r d ax n ey1 t er0))
("coordinators" nil (k ow0 ao1 r d ax n ey1 t er0 z))
("coors" nil (k uw1 r z))
("coos" nil (k uw1 s))
("coosa" nil (k uw1 s ax))
("coot" nil (k uw1 t))
("coote" nil (k uw1 t))
("cooter" nil (k uw1 t er0))
("coots" nil (k uw1 t s))
("coover" nil (k uw1 v er0))
("cop" nil (k aa1 p))
("copacabana" nil (k ow1 p ax k ax b ae1 n ax))
("copartners" nil (k ow1 p aa1 r t n er0 z))
("copas" nil (k ow1 p ax z))
("copayments" nil (k ow1 p ey1 m ax n t s))
("copco" nil (k aa1 p k ow0))
("cope" nil (k ow1 p))
("coped" nil (k ow1 p t))
("copelan" nil (k aa1 p ih0 l ae0 n))
("copeland" nil (k ow1 p l ax n d))
("copelco" nil (k ow1 p eh1 l k ow0))
("copelin" nil (k aa1 p ih0 l ih0 n))
("copeman" nil (k ow1 p m ax n))
("copen" nil (k ow1 p ax n))
("copenhagen" nil (k ow1 p ax n hh ey1 g ax n))
("copenhaver" nil (k ax p eh1 n hh ax v er0))
("copernican" nil (k ax p er1 n ax k ax n))
("copernicus" nil (k ax p er1 n ax k ax s))
("copes" nil (k ow1 p s))
("copetas" nil (k ax p ey1 t ax s))
("copher" nil (k aa1 f er0))
("copiague" nil (k ow1 p iy0 ae0 g))
("copied" nil (k aa1 p iy0 d))
("copier" nil (k aa1 p iy0 er0))
("copiers" nil (k aa1 p iy0 er0 z))
("copies" nil (k aa1 p iy0 z))
("coping" nil (k ow1 p ih0 ng))
("copious" nil (k ow1 p iy0 ax s))
("coplan" nil (k aa1 p l ax n))
("copland" nil (k aa1 p l ax n d))
("coplen" nil (k aa1 p ax l ax n))
("copley" nil (k aa1 p l iy0))
("copleys" nil (k aa1 p l iy0 z))
("coplin" nil (k aa1 p l ih0 n))
("copp" nil (k aa1 p))
("coppa" nil (k ow1 p ax))
("coppage" nil (k aa1 p ih0 jh))
("copped" nil (k aa1 p t))
("coppedge" nil (k aa1 p ih0 jh))
("coppee" nil (k aa1 p iy1))
("coppens" nil (k aa1 p ax n z))
("copper" nil (k aa1 p er0))
("copperfield" nil (k aa1 p er0 f iy1 l d))
("copperfields" nil (k aa1 p er0 f iy1 l d z))
("copperhead" nil (k aa1 p er0 hh eh1 d))
("copperheads" nil (k aa1 p er0 hh eh1 d z))
("copperman" nil (k aa1 p er0 m ax n))
("coppernoll" nil (k ax p er1 n ow0 l))
("coppers" nil (k aa1 p er0 z))
("coppersmith" nil (k aa1 p er0 s m ih1 th))
("copperweld" nil (k aa1 p er0 w eh1 l d))
("coppery" nil (k aa1 p er0 iy0))
("coppess" nil (k aa1 p ax s))
("coppin" nil (k aa1 p ih0 n))
("copping" nil (k aa1 p ih0 ng))
("coppinger" nil (k aa1 p ih0 ng er0))
("coppins" nil (k aa1 p ih0 n z))
("copple" nil (k aa1 p ax l))
("coppock" nil (k aa1 p ax k))
("coppola" nil (k ow0 p ow1 l ax))
("coppolino" nil (k ow0 p ow0 l iy1 n ow0))
("copps" nil (k aa1 p s))
("copra" nil (k ow1 p r ax))
("coprocessor" nil (k ow1 p r aa1 s eh0 s er0))
("coprocessors" nil (k ow1 p r aa1 s eh0 s er0 z))
("cops" nil (k aa1 p s))
("copsey" nil (k aa1 p s iy0))
("coptic" nil (k aa1 p t ih0 k))
("copulate" nil (k aa1 p y ax l ey1 t))
("copus" nil (k ow1 p ax s))
("copy" nil (k aa1 p iy0))
("copycat" nil (k aa1 p iy0 k ae1 t))
("copycats" nil (k aa1 p iy0 k ae1 t s))
("copycode" nil (k aa1 p iy0 k ow1 d))
("copying" nil (k aa1 p iy0 ih0 ng))
("copyreaders" nil (k aa1 p iy0 r iy1 d er0 z))
("copyright" nil (k aa1 p iy0 r ay1 t))
("copyrightable" nil (k aa1 p iy0 r ay1 t ax b ax l))
("copyrighted" nil (k aa1 p iy0 r ay1 t ih0 d))
("copyrights" nil (k aa1 p iy0 r ay1 t s))
("copytele" nil (k aa1 p iy0 t eh1 l ax))
("copywrited" nil (k aa1 p iy0 r ay1 t ih0 d))
("copywriter" nil (k aa1 p iy0 r ay1 t er0))
("copywriters" nil (k aa1 p iy0 r ay1 t er0 z))
("cor" nil (k ao1 r))
("cora" nil (k ao1 r ax))
("corabelle" nil (k ao1 r ax b ax l))
("coracle" nil (k ao1 r ax k ax l))
("coral" nil (k ao1 r ax l))
("coralie" nil (k ao1 r ax l iy0))
("coraline" nil (k ao1 r ax l ay1 n))
("corallo" nil (k ao1 r ae1 l ow0))
("corals" nil (k ao1 r ax l z))
("coram" nil (k ao1 r ae1 m))
("corazon" nil (k ao1 r ax z ow1 n))
("corban" nil (k ao1 r b ax n))
("corbeil" nil (k ao1 r b ay1 l))
("corbel" nil (k ao1 r b ax l))
("corbell" nil (k ao1 r b eh0 l))
("corbelled" nil (k ao1 r b ax l d))
("corbello" nil (k er0 b eh1 l ow0))
("corbet" nil (k ao1 r b ih0 t))
("corbett" nil (k ao1 r b ih0 t))
("corbin" nil (k ao1 r b ih0 n))
("corbit" nil (k ao1 r b ih0 t))
("corbitt" nil (k ao1 r b ih0 t))
("corbo" nil (k ao1 r b ow0))
("corbridge" nil (k ao1 r b r ih0 jh))
("corbu" nil (k ao1 r b uw1))
("corbusier" nil (k ao1 r b y uw1 s y er0))
("corby" nil (k ao1 r b iy0))
("corcap" nil (k ao1 r k ae1 p))
("corcoran" nil (k ao1 r k er0 ax n))
("cord" nil (k ao1 r d))
("corda" nil (k ao1 r d ax))
("cordaro" nil (k er0 d aa1 r ow0))
("cordasco" nil (k er0 d aa1 s k ow0))
("corday" nil (k ao1 r d ey1))
("corded" nil (k ao1 r d ax d))
("cordeiro" nil (k er0 d eh1 r ow0))
("cordell" nil (k ao1 r d eh1 l))
("corder" nil (k ao1 r d er0))
("cordero" nil (k er0 d eh1 r ow0))
("cordes" nil (k ao1 r d z))
("cordesman" nil (k ao1 r d ax s m ax n))
("cordial" nil (k ao1 r jh ax l))
("cordially" nil (k ao1 r jh ax l iy0))
("cordiant" nil (k ao1 r d iy0 ax n t))
("cordie" nil (k ao1 r d iy0))
("cordier" nil (k ao1 r d iy0 er0))
("cordill" nil (k ao1 r d ax l))
("cording" nil (k ao1 r d ih0 ng))
("cordis" nil (k ao1 r d ih0 s))
("cordle" nil (k ao1 r d ax l))
("cordless" nil (k ao1 r d l ax s))
("cordner" nil (k ao1 r d n er0))
("cordoba" nil (k ao1 r d ow1 b ax))
("cordobas" nil (k ao1 r d ax b ax z))
("cordon" nil (k ao1 r d ax n))
("cordone" nil (k ao1 r d ax n))
("cordoned" nil (k ao1 r d ax n d))
("cordova" nil (k ao1 r d ow1 v ax))
("cordover" nil (k ao1 r d ow1 v er0))
("cordovez" nil (k ao1 r d ow1 v eh0 z))
("cordray" nil (k ao1 r d r ey1))
("cordrey" nil (k ao1 r d r iy0))
("cordry" nil (k ao1 r d er0 iy0))
("cords" nil (k ao1 r d z))
("cordts" nil (k ao1 r d t s))
("corduroy" nil (k ao1 r d er0 oy1))
("cordy" nil (k ao1 r d iy0))
("core" nil (k ao1 r))
("corea" nil (k ao0 r iy1 ax))
("corel" nil (k ao1 r eh1 l))
("corell" nil (k ao1 r eh1 l))
("corella" nil (k er0 eh1 l ax))
("coren" nil (k ao1 r ax n))
("cores" nil (k ao1 r z))
("corespondent" nil (k ao1 r ax s p aa1 n d ax n t))
("corestates" nil (k ao1 r s t ey1 t s))
("coretech" nil (k ao1 r t eh1 k))
("coretta" nil (k ao0 r eh1 t ax))
("corette" nil (k ao1 r eh1 t))
("corexit" nil (k ao0 r eh1 k s ih0 t))
("corey" nil (k ao1 r iy0))
("corgi" nil (k ao1 r g iy0))
("coria" nil (k ao1 r iy0 ax))
("coriander" nil (k ao1 r iy0 ae1 n d er0))
("coriell" nil (k ao1 r iy0 l))
("corigliano" nil (k er0 iy0 g l iy0 aa1 n ow0))
("corinna" nil (k ao1 r ih1 n ax))
("corinne" nil (k er0 iy1 n))
("corinth" nil (k ao1 r ax n th))
("corinthian" nil (k er0 ih1 n th iy0 ax n))
("corinto" nil (k ao1 r ih1 n t ow0))
("corio" nil (k ao1 r iy0 ow0))
("corisa" nil (k er0 iy1 s ax))
("corissa" nil (k ao1 r ih1 s ax))
("cork" nil (k ao1 r k))
("corken" nil (k ao1 r k ax n))
("corker" nil (k ao1 r k er0))
("corkern" nil (k ao1 r k er0 n))
("corkery" nil (k ao1 r k er0 iy0))
("corkill" nil (k ao1 r k ih1 l))
("corking" nil (k ao1 r k ih0 ng))
("corkins" nil (k ao1 r k ih0 n z))
("corkran" nil (k ao1 r k r ax n))
("corks" nil (k ao1 r k s))
("corkscrew" nil (k ao1 r k s k r uw1))
("corkum" nil (k ao1 r k ax m))
("corkwood" nil (k ao1 r k w uh1 d))
("corky" nil (k ao1 r k iy0))
("corl" nil (k ao1 r l))
("corle" nil (k ao1 r ax l))
("corless" nil (k ao1 r l ih0 s))
("corlett" nil (k ao1 r l ih0 t))
("corlew" nil (k ao1 r l uw0))
("corley" nil (k ao1 r l iy0))
("corliss" nil (k ao1 r l ih0 s))
("cormac" nil (k ao1 r m ax k))
("cormack" nil (k ao1 r m ax k))
("corman" nil (k ao1 r m ax n))
("cormany" nil (k ao1 r m ax n iy0))
("cormican" nil (k ao1 r m ih0 k ax n))
("cormick" nil (k ao1 r m ih0 k))
("cormier" nil (k ao1 r m iy0 er0))
("cormorant" nil (k ao1 r m er0 ax n t))
("corn" nil (k ao1 r n))
("cornacchia" nil (k er0 n aa1 k iy0 ax))
("cornall" nil (k ao1 r n ax l))
("corncob" nil (k ao1 r n k aa1 b))
("corncrib" nil (k ao1 r n k r ih1 b))
("corne" nil (k ao1 r n))
("cornea" nil (k ao1 r n iy0 ax))
("corneal" nil (k ao1 r n iy1 l))
("corneas" nil (k ao1 r n iy0 ax z))
("cornejo" nil (k er0 n ey1 y ow0))
("cornel" nil (k ao1 r n ax l))
("cornela" nil (k er0 n eh1 l ax))
("cornelia" nil (k ao0 r n iy1 l y ax))
("cornelio" nil (k er0 n eh1 l iy0 ow0))
("cornelious" nil (k ao1 r n ih0 l iy0 ih0 s))
("cornelison" nil (k er0 n eh1 l ih0 s ax n))
("cornelius" nil (k ao0 r n iy1 l y ax s))
("corneliuson" nil (k ao1 r n ih0 l iy0 uw1 s ax n))
("cornell" nil (k ao0 r n eh1 l))
("cornelle" nil (k er0 n eh1 l))
("corner" nil (k ao1 r n er0))
("cornerback" nil (k ao1 r n er0 b ae1 k))
("cornered" nil (k ao1 r n er0 d))
("cornering" nil (k ao1 r n er0 ih0 ng))
("corners" nil (k ao1 r n er0 z))
("cornerstone" nil (k ao1 r n er0 s t ow1 n))
("cornerstones" nil (k ao1 r n er0 s t ow1 n z))
("cornet" nil (k ao0 r n eh1 t))
("cornett" nil (k ao1 r n ih0 t))
("cornette" nil (k er0 n eh1 t))
("cornettist" nil (k ao0 r n eh1 t ax s t))
("corney" nil (k ao1 r n iy0))
("cornfeld" nil (k ao1 r n f eh1 l d))
("cornfield" nil (k ao1 r n f iy1 l d))
("cornfields" nil (k ao1 r n f iy1 l d z))
("cornflowers" nil (k ao1 r n f l aw1 er0 z))
("cornforth" nil (k ao1 r n f er0 th))
("cornhusking" nil (k ao1 r n hh ah1 s k ih0 ng))
("cornia" nil (k ao1 r n iy0 ax))
("corniche" nil (k ao1 r n ih0 sh))
("cornick" nil (k ao1 r n ih0 k))
("cornie" nil (k ao1 r n iy0))
("corniel" nil (k ao1 r n iy1 l))
("corning" nil (k ao1 r n ih0 ng))
("cornish" nil (k ao1 r n ih0 sh))
("cornman" nil (k ao1 r n m ax n))
("cornmeal" nil (k ao1 r n m iy1 l))
("corns" nil (k ao1 r n z))
("cornstalks" nil (k ao1 r n s t ao1 k s))
("cornstarch" nil (k ao1 r n s t aa1 r ch))
("cornucopia" nil (k ao1 r n ax k ow1 p iy0 ax))
("cornwall" nil (k ao1 r n w ao0 l))
("cornwallis" nil (k ao0 r n w ao1 l ax s))
("cornwell" nil (k ao1 r n w eh1 l))
("corny" nil (k ao1 r n iy0))
("corolla" nil (k er0 aa1 l ax))
("corollaries" nil (k ao1 r ax l eh1 r iy0 z))
("corollary" nil (k ao1 r ax l eh1 r iy0))
("corona" nil (k er0 ow1 n ax))
("coronado" nil (k er0 ow0 n aa1 d ow0))
("coronary" nil (k ao1 r ax n eh1 r iy0))
("coronation" nil (k ao1 r ax n ey1 sh ax n))
("coroner" nil (k ao1 r ax n er0))
("coronet" nil (k ao1 r ax n eh1 t))
("coronets" nil (k ao1 r ax n eh1 t s))
("corp" nil (k ao1 r p))
("corpening" nil (k ao1 r p ax n ih0 ng))
("corporacion" nil (k ao1 r p er0 aa1 s iy0 ow1 n))
("corporal" nil (k ao1 r p er0 ax l))
("corporate" nil (k ao1 r p er0 ax t))
("corporates" nil (k ao1 r p er0 ax t s))
("corporatewatch" nil (k ao1 r p r ax t w aa1 ch))
("corporatewide" nil (k ao1 r p er0 ih0 t w ay1 d))
("corporation" nil (k ao1 r p er0 ey1 sh ax n))
("corporations" nil (k ao1 r p er0 ey1 sh ax n z))
("corporatism" nil (k ao1 r p er0 ax t ih1 z ax m))
("corporatist" nil (k ao1 r p er0 ax t ih0 s t))
("corps" nil (k ao1 r))
("corpse" nil (k ao1 r p s))
("corpses" nil (k ao1 r p s ax z))
("corpsman" nil (k ao1 r m ax n))
("corpulant" nil (k ao1 r p y ax l ax n t))
("corpus" nil (k ao1 r p ax s))
("corpuz" nil (k ao1 r p uw0 z))
("corr" nil (k ao1 r))
("corradi" nil (k er0 aa1 d iy0))
("corradino" nil (k er0 aa0 d iy1 n ow0))
("corrado" nil (k er0 aa1 d ow0))
("corral" nil (k er0 ae1 l))
("corrales" nil (k er0 aa1 l eh0 s))
("corralled" nil (k er0 ae1 l d))
("corralling" nil (k er0 ae1 l ih0 ng))
("corrao" nil (k ao1 r aw0))
("correa" nil (k ao1 r iy0 ax))
("correale" nil (k ao1 r ax l))
("correct" nil (k er0 eh1 k t))
("corrected" nil (k er0 eh1 k t ax d))
("correcting" nil (k er0 eh1 k t ih0 ng))
("correction" nil (k er0 eh1 k sh ax n))
("correctional" nil (k er0 eh1 k sh ax n ax l))
("corrections" nil (k er0 eh1 k sh ax n z))
("corrective" nil (k er0 eh1 k t ih0 v))
("correctly" nil (k er0 eh1 k t l iy0))
("correctness" nil (k er0 eh1 k t n ax s))
("corrects" nil (k er0 eh1 k t s))
("correia" nil (k er0 ey1 iy0 ax))
("correira" nil (k er0 eh1 r ax))
("correlate" nil (k ao1 r ax l ey1 t))
("correlated" nil (k ao1 r ax l ey1 t ax d))
("correlation" nil (k ao1 r ax l ey1 sh ax n))
("correlations" nil (k ao1 r ax l ey1 sh ax n z))
("correll" nil (k er0 ey1 l))
("correne" nil (k ao1 r iy0 n))
("corrente" nil (k er0 eh1 n t iy0))
("correnti" nil (k er0 eh1 n t iy0))
("correspond" nil (k ao1 r ax s p aa1 n d))
("corresponded" nil (k ao1 r ax s p aa1 n d ih0 d))
("correspondence" nil (k ao1 r ax s p aa1 n d ax n s))
("correspondent" nil (k ao1 r ax s p aa1 n d ax n t))
("correspondents" nil (k ao1 r ax s p aa1 n d ax n t s))
("corresponding" nil (k ao1 r ax s p aa1 n d ih0 ng))
("correspondingly" nil (k ao1 r ax s p aa1 n d ih0 ng l iy0))
("corresponds" nil (k ao1 r ax s p aa1 n d z))
("corrib" nil (k ao1 r ih0 b))
("corrick" nil (k ao1 r ih0 k))
("corridor" nil (k ao1 r ax d er0))
("corridors" nil (k ao1 r ih0 d er0 z))
("corrie" nil (k ao1 r iy0))
("corriere" nil (k ao1 r y eh1 r))
("corrigan" nil (k ao1 r ih0 g ax n))
("corriher" nil (k ao1 r ih0 hh er0))
("corrin" nil (k ao1 r ih0 n))
("corrina" nil (k er0 iy1 n ax))
("corrington" nil (k ao1 r ih0 ng t ax n))
("corriveau" nil (k ao1 r ih0 v ow0))
("corroborate" nil (k er0 aa1 b er0 ey1 t))
("corroborated" nil (k er0 aa1 b er0 ey1 t ih0 d))
("corroborates" nil (k er0 ao1 b er0 ey1 t s))
("corroborating" nil (k er0 aa1 b er0 ey1 t ih0 ng))
("corroboration" nil (k er0 ao1 b er0 ey1 sh ax n))
("corrode" nil (k er0 ow1 d))
("corroded" nil (k er0 ow1 d ih0 d))
("corrodes" nil (k er0 ow1 d z))
("corron" nil (k er0 ao1 n))
("corroon" nil (k ao0 r uw1 n))
("corrosion" nil (k er0 ow1 zh ax n))
("corrosive" nil (k er0 ow1 s ih0 v))
("corrow" nil (k ao1 r ow0))
("corrugated" nil (k ao1 r ax g ey1 t ax d))
("corrupt" nil (k er0 ah1 p t))
("corrupted" nil (k er0 ah1 p t ih0 d))
("corrupting" nil (k er0 ah1 p t ih0 ng))
("corruption" nil (k er0 ah1 p sh ax n))
("corruptions" nil (k er0 ah1 p sh ax n z))
("corruptive" nil (k er0 ah1 p t ih0 v))
("corrupts" nil (k er0 ah1 p t s))
("corry" nil (k ao1 r iy0))
("corsa" nil (k ao1 s ax))
("corsage" nil (k ao0 r s aa1 zh))
("corsair" nil (k ao1 r s eh0 r))
("corsaro" nil (k er0 s aa1 r ow0))
("corse" nil (k ao1 r s))
("corsello" nil (k er0 s eh1 l ow0))
("corsentino" nil (k er0 s eh0 n t iy1 n ow0))
("corser" nil (k ao1 r s er0))
("corset" nil (k ao1 r s ax t))
("corsetti" nil (k er0 s eh1 t iy0))
("corsi" nil (k ao1 r s iy0))
("corsica" nil (k ao1 r s ih0 k ax))
("corsican" nil (k ao1 r s ax k ax n))
("corsicas" nil (k ao1 r s ih0 k ax z))
("corsiglia" nil (k er0 s iy1 g l iy0 ax))
("corsini" nil (k er0 s iy1 n iy0))
("corso" nil (k ao1 r s ow0))
("corson" nil (k ao1 r s ax n))
("cort" nil (k ao1 r t))
("corte" nil (k ao1 r t))
("cortelyou" nil (k ao1 r t eh0 l y uw1))
("corter" nil (k ao1 r t er0))
("cortes" nil (k ao1 r t s))
("cortese" nil (k er0 t ey1 z iy0))
("cortesi" nil (k er0 t eh1 s iy0))
("cortex" nil (k ao1 r t eh0 k s))
("cortez" nil (k ao0 r t eh1 z))
("corti" nil (k ao1 r t iy0))
("corticosteroids" nil (k ao1 r t ih0 k ow0 s t ih1 r oy1 d z))
("cortina" nil (k er0 t iy1 n ax))
("cortinas" nil (k ao1 r t ih0 n ax z))
("cortisone" nil (k ao1 r t ax z ow1 n))
("cortland" nil (k ao1 r t l ax n d))
("cortner" nil (k ao1 r t n er0))
("cortopassi" nil (k er0 t ow0 p aa1 s iy0))
("cortright" nil (k ao1 r t r ay0 t))
("corum" nil (k ao1 r ax m))
("corundum" nil (k er0 ah1 n d ax m))
("corvallis" nil (k ao1 r v ae1 l ih0 s))
("corvette" nil (k ao0 r v eh1 t))
("corvettes" nil (k ao1 r v eh1 t s))
("corvin" nil (k ao1 r v ih0 n))
("corvino" nil (k er0 v iy1 n ow0))
("corvo" nil (k ao1 r v ow0))
("corvus" nil (k ao1 r v ax s))
("corwin" nil (k ao1 r w ih0 n))
("cory" nil (k ao1 r iy0))
("corzine" nil (k er0 z iy1 n iy0))
("corzo" nil (k ao1 r z ow0))
("cos" nil (k ao1 s))
("cosa" nil (k ow1 s ax))
("cosatu" nil (k ax s aa1 t uw1))
("cosby" nil (k ao1 z b iy0))
("coscia" nil (k ow1 s ch ax))
("cosens" nil (k ow1 s ax n z))
("cosentino" nil (k ow1 s eh0 n t iy1 n ow0))
("cosenza" nil (k ow1 s eh1 n z ax))
("cosette" nil (k ax s eh1 t))
("cosey" nil (k ow1 z iy0))
("cosgriff" nil (k aa1 s g r ih0 f))
("cosgrove" nil (k aa1 s g r ax v))
("cosi" nil (k ow1 s iy0))
("cosima" nil (k ow0 s iy1 m ax))
("cosio" nil (k ow1 s iy0 ow0))
("coslett" nil (k aa1 s l ih0 t))
("cosma" nil (k ow1 z m ax))
("cosmair" nil (k ao1 z m eh1 r))
("cosman" nil (k aa1 s m ax n))
("cosme" nil (k ow1 z m))
("cosmetic" nil (k aa0 z m eh1 t ih0 k))
("cosmetically" nil (k ao1 z m eh1 t ih0 k l iy0))
("cosmetics" nil (k aa0 z m eh1 t ih0 k s))
("cosmetology" nil (k aa1 z m ax t aa1 l ax jh iy0))
("cosmic" nil (k aa1 z m ih0 k))
("cosmo" nil (k ao1 z m ow0))
("cosmonaut" nil (k ao1 z m ax n ao1 t))
("cosmonauts" nil (k ao1 z m ax n ao1 t s))
("cosmopolitan" nil (k aa1 z m ax p aa1 l ax t ax n))
("cosmopulos" nil (k ao1 z m ax p y uw1 l ow0 s))
("cosmos" nil (k aa1 z m ow0 s))
("cosner" nil (k aa1 s n er0))
("cosper" nil (k aa1 s p er0))
("coss" nil (k ao1 s))
("cossa" nil (k ao1 s ax))
("cossack" nil (k ao1 s ax k))
("cossacks" nil (k aa1 s ae0 k s))
("cossairt" nil (k aa1 s er0 t))
("cosset" nil (k aa1 s ax t))
("cossette" nil (k ax s eh1 t))
("cossey" nil (k aa1 s iy0))
("cossiga" nil (k ao1 s iy1 g ax))
("cossin" nil (k aa1 s ih0 n))
("cossman" nil (k ao1 s m ax n))
("cost" nil (k aa1 s t))
("costa" nil (k ao1 s t ax))
("costabile" nil (k ao1 s t ax b ay1 l))
("costain" nil (k ow1 s t ey1 n))
("costales" nil (k aa1 s t ax l z))
("costantini" nil (k ow0 s t aa0 n t iy1 n iy0))
("costantino" nil (k ow0 s t aa0 n t iy1 n ow0))
("costanza" nil (k ow0 s t aa1 n z ax))
("costanzo" nil (k ow0 s t aa1 n z ow0))
("costas" nil (k ao1 s t ax z))
("costco" nil (k ao1 s t k ow0))
("costcutting" nil (k ao1 s t k ah1 t ih0 ng))
("coste" nil (k ow1 s t))
("costella" nil (k ow0 s t eh1 l ax))
("costello" nil (k ao1 s t eh1 l ow0))
("costen" nil (k ao1 s t ax n))
("coster" nil (k ao1 s t er0))
("costigan" nil (k aa1 s t ih0 g ae0 n))
("costilla" nil (k ow0 s t ih1 l ax))
("costilow" nil (k aa1 s t ih0 l ow0))
("costin" nil (k aa1 s t ih0 n))
("costing" nil (k ao1 s t ih0 ng))
("costley" nil (k aa1 s t l iy0))
("costlier" nil (k ao1 s t l iy0 er0))
("costliest" nil (k ao1 s t l iy0 ih0 s t))
("costlow" nil (k ao1 s t l ow1))
("costly" nil (k aa1 s t l iy0))
("costner" nil (k aa1 s t n er0))
("coston" nil (k aa1 s t ax n))
("costs" nil (k aa1 s t s))
("costume" nil (k aa0 s t uw1 m))
("costumed" nil (k ao1 s t uw1 m d))
("costumes" nil (k aa0 s t uw1 m z))
("cot" nil (k aa1 t))
("cota" nil (k ow1 t ax))
("cote" nil (k ow1 t))
("cotelle" nil (k ow1 t eh1 l))
("coterie" nil (k ow1 t er0 iy0))
("cotes" nil (k ow1 t s))
("cotham" nil (k aa1 th ax m))
("cothern" nil (k ah1 dh er0 n))
("cothran" nil (k aa1 th r ax n))
("cothren" nil (k aa1 th er0 ax n))
("cothron" nil (k aa1 th r ax n))
("cotler" nil (k ow1 t ax l er0))
("cotman" nil (k aa1 t m ax n))
("cotner" nil (k aa1 t n er0))
("cotney" nil (k aa1 t n iy0))
("cotnoir" nil (k ax t n w aa1 r))
("coto" nil (k ow1 t ow0))
("cotret" nil (k aa1 t r ax t))
("cotrone" nil (k ow0 t r ow1 n iy0))
("cotroneo" nil (k ow0 t r ow1 n iy0 ow0))
("cots" nil (k aa1 t s))
("cott" nil (k aa1 t))
("cotta" nil (k aa1 t ax))
("cottage" nil (k aa1 t ax jh))
("cottages" nil (k aa1 t ih0 jh ih0 z))
("cottam" nil (k aa1 t ax m))
("cotten" nil (k aa1 t ax n))
("cotter" nil (k aa1 t er0))
("cotterill" nil (k aa1 t er0 ih0 l))
("cotterman" nil (k aa1 t er0 m ax n))
("cottier" nil (k aa1 t iy0 er0))
("cottingham" nil (k aa1 t ih0 ng hh ae1 m))
("cottle" nil (k aa1 t ax l))
("cottman" nil (k aa1 t m ax n))
("cotto" nil (k ow1 t ow0))
("cottom" nil (k aa1 t ax m))
("cotton" nil (k aa1 t ax n))
("cottone" nil (k ow0 t ow1 n iy0))
("cottoned" nil (k aa1 t ax n d))
("cottongim" nil (k ax t aa1 ng g ih0 m))
("cottonmouth" nil (k aa1 t ax n m aw1 th))
("cottons" nil (k aa1 t ax n z))
("cottonseed" nil (k aa1 t ax n s iy1 d))
("cottonwood" nil (k aa1 t ax n w uh1 d))
("cottonwoods" nil (k aa1 t ax n w uh1 d z))
("cottrell" nil (k aa1 t r eh1 l))
("cottrill" nil (k aa1 t r ax l))
("cotty" nil (k aa1 t iy0))
("cotugno" nil (k ow0 t uw1 g n ow0))
("coty" nil (k ow1 t iy0))
("couch" nil (k aw1 ch))
("couched" nil (k aw1 ch t))
("couches" nil (k aw1 ch ih0 z))
("couchman" nil (k uw0 sh m ae1 n))
("coudersport" nil (k aw1 d er0 z p ao1 r t))
("coudert" nil (k uw1 d er0 t))
("couey" nil (k uw0 iy1))
("coufal" nil (k uw0 f ae1 l))
("cougar" nil (k uw1 g er0))
("cougars" nil (k uw1 g er0 z))
("cough" nil (k aa1 f))
("coughed" nil (k ao1 f t))
("coughenour" nil (k ao0 f eh1 n er0))
("coughing" nil (k aa1 f ih0 ng))
("coughlan" nil (k ao1 g l ax n))
("coughlin" nil (k ao1 g l ih0 n))
("coughran" nil (k ao1 g r ax n))
("coughs" nil (k ao1 f s))
("couillard" nil (k w iy0 l aa1 r d))
("could" nil (k uh1 d))
("coull" nil (k aw1 l))
("coulombe" nil (k aw0 l ow1 m b iy0))
("coulon" nil (k aw1 l ax n))
("coulson" nil (k aw1 l s ax n))
("coulston" nil (k aw1 l s t ax n))
("coultas" nil (k uw0 l t aa1 z))
("coulter" nil (k ow1 l t er0))
("coulthard" nil (k uw0 l th aa1 r d))
("counce" nil (k aw1 n s))
("council" nil (k aw1 n s ax l))
("councilman" nil (k aw1 n s ax l m ax n))
("councilmen" nil (k aw1 n s ax l m eh1 n))
("councilor" nil (k aw1 n s ax l er0))
("councilors" nil (k aw1 n s ax l er0 z))
("councils" nil (k aw1 n s ax l z))
("councilwoman" nil (k aw1 n s ax l w uh1 m ax n))
("counihan" nil (k aw1 n ih0 hh ae0 n))
("counsel" nil (k aw1 n s ax l))
("counseled" nil (k aw1 n s ax l d))
("counseling" nil (k aw1 n s ax l ih0 ng))
("counsell" nil (k aw1 n s ax l))
("counsellors" nil (k aw1 n s ax l er0 z))
("counselman" nil (k aw1 n s ax l m ax n))
("counselor" nil (k aw1 n s ax l er0))
("counselors" nil (k aw1 n s ax l er0 z))
("counsels" nil (k aw1 n s ax l z))
("count" nil (k aw1 n t))
("countable" nil (k aw1 n t ax b ax l))
("countdown" nil (k aw1 n t d aw1 n))
("countdowns" nil (k aw1 n t d aw1 n z))
("counted" nil (k aw1 n t ax d))
("countenance" nil (k aw1 n t ax n ax n s))
("countenanced" nil (k aw1 n t ax n ax n s t))
("countenances" nil (k aw1 n t ax n ax n s ih0 z))
("counter" nil (k aw1 n t er0))
("counteract" nil (k aw1 n t er0 ae1 k t))
("counteracted" nil (k aw1 n t er0 ae1 k t ih0 d))
("counteracting" nil (k aw1 n t er0 ae1 k t ih0 ng))
("counterattack" nil (k aw1 n t er0 ax t ae1 k))
("counterattacked" nil (k aw1 n t er0 ax t ae1 k t))
("counterattacks" nil (k aw1 n t er0 ax t ae1 k s))
("counterbalance" nil (k aw1 n t er0 b ae1 l ax n s))
("counterbalanced" nil (k aw1 n t er0 b ae1 l ax n s t))
("counterbid" nil (k aw1 n t er0 b ih1 d))
("counterchallenge" nil (k aw1 n t er0 ch ae1 l ax n jh))
("countercharges" nil (k aw1 n t er0 ch aa1 r jh ih0 z))
("counterclaim" nil (k aw1 n t er0 k l ey1 m))
("counterclaims" nil (k aw1 n t er0 k l ey1 m z))
("countercultural" nil (k aw1 n t er0 k ah1 l ch er0 ax l))
("counterculture" nil (k aw1 n t er0 k ah1 l ch er0))
("countered" nil (k aw1 n t er0 d))
("counterespionage" nil (k aw1 n t er0 eh1 s p iy0 ax n aa0 jh))
("counterfeit" nil (k aw1 n t er0 f ih1 t))
("counterfeited" nil (k aw1 n t er0 f ih1 t ih0 d))
("counterfeiters" nil (k aw1 n t er0 f ih1 t er0 z))
("counterfeiting" nil (k aw1 n t er0 f ih1 t ih0 ng))
("counterforce" nil (k aw1 n t er0 f ao1 r s))
("countering" nil (k aw1 n t er0 ih0 ng))
("counterinsurgency" nil (k aw1 n t er0 ih0 n s er1 jh ax n s iy0))
("counterintelligence" nil (k aw1 n t er0 ih0 n t eh1 l ih0 jh ax n s))
("counterlife" nil (k aw1 n t er0 l ay1 f))
("counterman" nil (k aw1 n t er0 m ae1 n))
("countermeasure" nil (k aw1 n t er0 m eh1 zh er0))
("countermeasures" nil (k aw1 n t er0 m eh1 zh er0 z))
("countermove" nil (k aw1 n t er0 m uw1 v))
("countermoves" nil (k aw1 n t er0 m uw1 v z))
("counteroffensive" nil (k aw1 n t er0 ao0 f eh1 n s ih0 v))
("counteroffer" nil (k aw1 n t er0 ao1 f er0))
("counteroffers" nil (k aw1 n t er0 ao1 f er0 z))
("counterpart" nil (k aw1 n t er0 p aa1 r t))
("counterparts" nil (k aw1 n t er0 p aa1 r t s))
("counterparty" nil (k aw1 n t er0 p aa1 r t iy0))
("counterpoint" nil (k aw1 n t er0 p oy1 n t))
("counterproductive" nil (k aw1 n t er0 p r ax d ah1 k t ih0 v))
("counterproposal" nil (k aw1 n t er0 p r ax p ow1 z ax l))
("counterproposals" nil (k aw1 n t er0 p r ax p ow1 z ax l z))
("counterpunch" nil (k aw1 n t er0 p ah1 n ch))
("counterrevolution" nil (k aw1 n t er0 r eh0 v ax l uw1 sh ax n))
("counterrevolutionary" nil (k aw1 n t er0 r eh0 v ax l uw1 sh ax n eh1 r iy0))
("counters" nil (k aw1 n t er0 z))
("countersued" nil (k aw1 n t er0 s uw1 d))
("countersuit" nil (k aw1 n t er0 s uw1 t))
("countertenor" nil (k aw1 n t er0 t eh1 n er0))
("counterterrorism" nil (k aw1 n t er0 t eh1 r er0 ih1 z ax m))
("counterterrorist" nil (k aw1 n t er0 t eh1 r er0 ih0 s t))
("countertop" nil (k aw1 n t er0 t aa1 p))
("countertrade" nil (k aw1 n t er0 t r ey1 d))
("countervailing" nil (k aw1 n t er0 v ey1 l ih0 ng))
("counterweight" nil (k aw1 n t er0 w ey1 t))
("countess" nil (k aw1 n t ax s))
("counties" nil (k aw1 n t iy0 z))
("counting" nil (k aw1 n t ih0 ng))
("countless" nil (k aw1 n t l ax s))
("countries" nil (k ah1 n t r iy0 z))
("country" nil (k ah1 n t r iy0))
("countryfolk" nil (k ah1 n t r iy0 f ow1 k))
("countryman" nil (k ah1 n t r iy0 m ax n))
("countrymen" nil (k ah1 n t r iy0 m ih0 n))
("countryside" nil (k ah1 n t r iy0 s ay1 d))
("countrywide" nil (k ah1 n t r iy0 w ay1 d))
("counts" nil (k aw1 n t s))
("county" nil (k aw1 n t iy0))
("coup" nil (k uw1))
("coupe" nil (k uw1 p))
("couper" nil (k uw1 er0))
("coupes" nil (k uw1 p s))
("coupland" nil (k uw1 p l ax n d))
("couple" nil (k ah1 p ax l))
("coupled" nil (k ah1 p ax l d))
("couplers" nil (k ah1 p l er0 z))
("couples" nil (k ah1 p ax l z))
("coupling" nil (k ah1 p l ih0 ng))
("couplings" nil (k ah1 p l ih0 ng z))
("coupon" nil (k uw1 p ao1 n))
("couponing" nil (k uw1 p aa0 n ih0 ng))
("coupons" nil (k uw1 p ao1 n z))
("coups" nil (k uw1 z))
("courage" nil (k er1 ax jh))
("courageous" nil (k er0 ey1 jh ax s))
("courageously" nil (k er0 ey1 jh ax s l iy0))
("courant" nil (k uh1 r ax n t))
("courchaine" nil (k uh0 r sh ey1 n))
("courchesne" nil (k uh0 r sh eh1 n))
("courier" nil (k er1 iy0 er0))
("couriers" nil (k er1 iy0 er0 z))
("courington" nil (k ao1 r ih0 ng t ax n))
("cournoyer" nil (k ao1 r n oy0 er0))
("course" nil (k ao1 r s))
("coursen" nil (k ao1 r s ax n))
("courser" nil (k ao1 r s er0))
("courses" nil (k ao1 r s ax z))
("coursey" nil (k ao1 r s iy0))
("coursing" nil (k ao1 r s ih0 ng))
("courson" nil (k ao1 r s ax n))
("court" nil (k ao1 r t))
("courtade" nil (k ao1 r t ey0 d))
("courtaulds" nil (k ao1 r t ao1 l d z))
("courteau" nil (k er1 t ow0))
("courted" nil (k ao1 r t ih0 d))
("courtemanche" nil (k ao1 r t m ae1 n sh))
("courtenay" nil (k er1 t ax n ey0))
("courteous" nil (k er1 t iy0 ax s))
("courter" nil (k ao1 r t er0))
("courtesies" nil (k er1 t ax s iy0 z))
("courtesy" nil (k er1 t ax s iy0))
("courthouse" nil (k ao1 r t hh aw1 s))
("courthouses" nil (k ao1 r t hh aw1 s ih0 z))
("courtier" nil (k ao1 r t iy0 er0))
("courtiers" nil (k ao1 r t iy0 er0 z))
("courting" nil (k ao1 r t ih0 ng))
("courtland" nil (k ao1 r t l ae0 n d))
("courtly" nil (k ao1 r t l iy0))
("courtney" nil (k ao1 r t n iy0))
("courtois" nil (k ao1 r t w aa0))
("courtright" nil (k ao1 r t r ay1 t))
("courtroom" nil (k ao1 r t r uw1 m))
("courtrooms" nil (k ao1 r t r uw1 m z))
("courts" nil (k ao1 r t s))
("courtship" nil (k ao1 r ch ih1 p))
("courtwright" nil (k ao1 r t r ay1 t))
("courtyard" nil (k ao1 r t y aa1 r d))
("courtyards" nil (k ao1 r t y aa1 r d z))
("courville" nil (k uh0 r v ih1 l))
("coury" nil (k ao1 r iy0))
("cousar" nil (k uw0 s aa1 r))
("couse" nil (k aw1 s))
("cousens" nil (k uw1 s ax n z))
("couser" nil (k aw1 s er0))
("cousin" nil (k ah1 z ax n))
("cousineau" nil (k uw1 s ih0 n ow1))
("cousino" nil (k aw0 s iy1 n ow0))
("cousins" nil (k ah1 z ax n z))
("cousy" nil (k uw1 z iy0))
("coutant" nil (k uw0 t ao1 n t))
("coutee" nil (k uw0 t iy1))
("couto" nil (k aw1 t ow0))
("couts" nil (k aw1 t s))
("coutts" nil (k aw1 t s))
("coutu" nil (k uw0 ch uw1))
("couture" nil (k uw0 t uh1 r))
("couturier" nil (k uw0 t uh1 r iy0 er0))
("couvillion" nil (k uw0 v iy0 l y ao1 n))
("couvillon" nil (k uw0 v iy0 l ao1 n))
("couzens" nil (k uw1 z ax n z))
("covalt" nil (k ow1 v aa0 l t))
("covarrubias" nil (k ow0 v aa0 r uw0 b iy1 ax z))
("covas" nil (k ow1 v ax s))
("covatta" nil (k ow0 v aa1 t ax))
("covault" nil (k ow1 v ao1 l t))
("cove" nil (k ow1 v))
("covel" nil (k ow1 v ax l))
("covell" nil (k aa1 v ax l))
("covelli" nil (k ow1 v eh1 l iy0))
("covello" nil (k ow1 v eh1 l ow0))
("coven" nil (k ah1 v ax n))
("covenant" nil (k ah1 v ax n ax n t))
("covenanter" nil (k ah1 v ax n ax n t er0))
("covenanters" nil (k ah1 v ax n ax n t er0 z))
("covenants" nil (k ah1 v ax n ax n t s))
("coveney" nil (k aa1 v ih0 n iy0))
("covent" nil (k ah1 v ax n t))
("coventry" nil (k ah1 v ax n t r iy0))
("cover" nil (k ah1 v er0))
("coverage" nil (k ah1 v er0 ax jh))
("coverages" nil (k ah1 v er0 ey1 jh ih0 z))
("coveralls" nil (k ah1 v er0 ao1 l z))
("coverdale" nil (k ah1 v er0 d ey1 l))
("coverdell" nil (k ah1 v er0 d eh1 l))
("covered" nil (k ah1 v er0 d))
("covering" nil (k ah1 v er0 ih0 ng))
("coverings" nil (k ah1 v er0 ih0 ng z))
("covers" nil (k ah1 v er0 z))
("coverstone" nil (k ah1 v er0 s t ow1 n))
("covert" nil (k ow1 v er0 t))
("covertly" nil (k ah1 v er0 t l iy0))
("coverup" nil (k ah1 v er0 ah1 p))
("coves" nil (k ow1 v z))
("covet" nil (k ah1 v ax t))
("coveted" nil (k ah1 v ax t ih0 d))
("covets" nil (k ah1 v ax t s))
("covey" nil (k ah1 v iy0))
("covia" nil (k ow1 v iy0 ax))
("coviello" nil (k ow1 v iy0 eh1 l ow0))
("covill" nil (k aa1 v ax l))
("coville" nil (k ow1 v ih1 l))
("covin" nil (k ow1 v ih0 n))
("covina" nil (k ow0 v iy1 n ax))
("covington" nil (k ah1 v ih0 ng t ax n))
("covino" nil (k ow0 v iy1 n ow0))
("covitz" nil (k ow1 v ih0 t s))
("cow" nil (k aw1))
("cowan" nil (k aw1 ax n))
("cowans" nil (k aw1 ax n z))
("coward" nil (k aw1 er0 d))
("cowardice" nil (k aw1 er0 d ax s))
("cowardly" nil (k aw1 er0 d l iy0))
("cowards" nil (k aw1 er0 d z))
("cowart" nil (k aw1 aa0 r t))
("cowbells" nil (k aw1 b eh1 l z))
("cowboy" nil (k aw1 b oy1))
("cowboys" nil (k aw1 b oy1 z))
("cowden" nil (k aw1 d ax n))
("cowdery" nil (k aw1 d er0 iy0))
("cowdrey" nil (k aw1 d r iy0))
("cowed" nil (k aw1 d))
("cowell" nil (k aa1 w eh0 l))
("cowen" nil (k aw1 ax n))
("cowens" nil (k aw1 ax n z))
("cower" nil (k aw1 er0))
("cowering" nil (k aw1 er0 ih0 ng))
("cowger" nil (k aw1 jh er0))
("cowgill" nil (k aw1 g ih1 l))
("cowgirl" nil (k aw1 g er1 l))
("cowgirls" nil (k aw1 g er1 l z))
("cowher" nil (k aa1 w er0))
("cowherd" nil (k aw1 hh er1 d))
("cowie" nil (k aw1 iy0))
("cowin" nil (k aw1 ih0 n))
("cowing" nil (k aw1 ih0 ng))
("cowles" nil (k aw1 ax l z))
("cowley" nil (k aw1 l iy0))
("cowling" nil (k aw1 l ih0 ng))
("cowlings" nil (k aw1 l ih0 ng z))
("cowman" nil (k aw1 m ax n))
("coworker" nil (k ow1 w er1 k er0))
("coworkers" nil (k ow1 w er1 k er0 z))
("cowper" nil (k aw1 p er0))
("cowperthwaite" nil (k aw1 p er0 th w ey1 t))
("cowries" nil (k aw1 r iy0 z))
("cows" nil (k aw1 z))
("cowser" nil (k aw1 z er0))
("cowsert" nil (k aw1 s er0 t))
("cowslip" nil (k aw1 s l ih0 p))
("cox" nil (k aa1 k s))
("coxe" nil (k aa1 k s))
("coxen" nil (k aa1 k s ax n))
("coxon" nil (k aa1 k s ax n))
("coxwell" nil (k aa1 k s w eh1 l))
("coy" nil (k oy1))
("coye" nil (k oy1))
("coyer" nil (k oy1 er0))
("coykendall" nil (k oy0 k eh1 n d ax l))
("coyle" nil (k oy1 l))
("coyly" nil (k oy1 l iy0))
("coym" nil (k oy1 m))
("coyne" nil (k oy1 n))
("coyner" nil (k oy1 n er0))
("coyote" nil (k ay0 ow1 t iy0))
("coyotes" nil (k ay0 ow1 t iy0 s))
("cozad" nil (k ow1 z ax d))
("cozart" nil (k aa1 z aa0 r t))
("cozby" nil (k aa1 z b iy0))
("cozine" nil (k ow0 z iy1 n iy0))
("coziness" nil (k ow1 z iy0 n ax s))
("cozman" nil (k ow1 z m ax n))
("cozy" nil (k ow1 z iy0))
("cozying" nil (k ow1 z iy0 ih0 ng))
("cozza" nil (k ow1 t s ax))
("cozzens" nil (k aa1 z ax n z))
("cozzi" nil (k ow1 t s iy0))
("cozzolino" nil (k ow0 t s ow0 l iy1 n ow0))
("crab" nil (k r ae1 b))
("crabb" nil (k r ae1 b))
("crabbe" nil (k r ae1 b))
("crabbed" nil (k r ae1 b d))
("crabbs" nil (k r ae1 b z))
("crabby" nil (k r ae1 b iy0))
("crabeater" nil (k r ae1 b iy1 t er0))
("crabill" nil (k r ae1 b ax l))
("crable" nil (k r ey1 b ax l))
("crabmeat" nil (k r ae1 b m iy1 t))
("crabs" nil (k r ae1 b z))
("crabtree" nil (k r ae1 b t r iy1))
("cracchiolo" nil (k r aa0 k iy0 ow1 l ow0))
("crace" nil (k r ey1 s))
("crack" nil (k r ae1 k))
("crackdown" nil (k r ae1 k d aw1 n))
("crackdowns" nil (k r ae1 k d aw1 n z))
("cracked" nil (k r ae1 k t))
("crackel" nil (k r ae1 k ax l))
("cracker" nil (k r ae1 k er0))
("crackerjack" nil (k r ae1 k er0 jh ae1 k))
("crackers" nil (k r ae1 k er0 z))
("cracking" nil (k r ae1 k ih0 ng))
("crackle" nil (k r ae1 k ax l))
("crackled" nil (k r ae1 k ax l d))
("crackles" nil (k r ae1 k ax l z))
("crackling" nil (k r ae1 k l ih0 ng))
("crackpot" nil (k r ae1 k p aa1 t))
("cracks" nil (k r ae1 k s))
("cracraft" nil (k r aa1 k r ae0 f t))
("craddock" nil (k r ae1 d ax k))
("crader" nil (k r ey1 d er0))
("cradic" nil (k r ae1 d ih0 k))
("cradle" nil (k r ey1 d ax l))
("cradles" nil (k r ey1 d ax l z))
("cradling" nil (k r ey1 d ax l ih0 ng))
("cradock" nil (k r ae1 d ax k))
("crady" nil (k r ey1 d iy0))
("craft" nil (k r ae1 f t))
("crafted" nil (k r ae1 f t ih0 d))
("crafting" nil (k r ae1 f t ih0 ng))
("crafton" nil (k r ae1 f t ax n))
("crafts" nil (k r ae1 f t s))
("craftsman" nil (k r ae1 f t s m ax n))
("craftsmanship" nil (k r ae1 f t s m ax n sh ih1 p))
("craftsmen" nil (k r ae1 f t s m ax n))
("crafty" nil (k r ae1 f t iy0))
("crager" nil (k r ey1 jh er0))
("cragg" nil (k r ae1 g))
("craggs" nil (k r ae1 g z))
("craggy" nil (k r ae1 g iy0))
("craghead" nil (k r ae1 g hh eh1 d))
("cragin" nil (k r ae1 jh ih0 n))
("cragle" nil (k r ey1 g ax l))
("crago" nil (k r aa1 g ow0))
("cragun" nil (k r ae1 g ax n))
("craib" nil (k r ey1 b))
("craig" nil (k r ey1 g))
("craighead" nil (k r ey1 g hh eh1 d))
("craigie" nil (k r ey1 g iy0))
("craigo" nil (k r ey1 g ow0))
("crail" nil (k r ey1 l))
("crain" nil (k r ey1 n))
("craine" nil (k r ey1 n))
("craker" nil (k r ey1 k er0))
("crall" nil (k r ao1 l))
("cram" nil (k r ae1 m))
("cramblit" nil (k r ae1 m b l ih0 t))
("cramer" nil (k r ey1 m er0))
("crames" nil (k r ey1 m z))
("crammed" nil (k r ae1 m d))
("crammer" nil (k r ae1 m er0))
("cramming" nil (k r ae1 m ih0 ng))
("cramp" nil (k r ae1 m p))
("cramped" nil (k r ae1 m p t))
("cramping" nil (k r ae1 m p ih0 ng))
("cramps" nil (k r ae1 m p s))
("crampton" nil (k r ae1 m p t ax n))
("crams" nil (k r ae1 m z))
("cramton" nil (k r ae1 m t ax n))
("cranberries" nil (k r ae1 n b eh1 r iy0 z))
("cranberry" nil (k r ae1 n b eh1 r iy0))
("crance" nil (k r ae1 n s))
("crandall" nil (k r ae1 n d ax l))
("crandell" nil (k r ae1 n d ax l))
("crane" nil (k r ey1 n))
("craner" nil (k r ey1 n er0))
("cranes" nil (k r ey1 n z))
("cranesbills" nil (k r ey1 n z b ih1 l z))
("craney" nil (k r ey1 n iy0))
("cranfield" nil (k r ae1 n f iy1 l d))
("cranfill" nil (k r ae1 n f ax l))
("cranford" nil (k r ae1 n f er0 d))
("crank" nil (k r ae1 ng k))
("cranked" nil (k r ae1 ng k t))
("cranking" nil (k r ae1 ng k ih0 ng))
("cranks" nil (k r ae1 ng k s))
("crankshaft" nil (k r ae1 ng k sh ae1 f t))
("crankshafts" nil (k r ae1 ng k sh ae1 f t s))
("cranky" nil (k r ae1 ng k iy0))
("cranley" nil (k r ae1 n l iy0))
("cranmer" nil (k r ae1 n m er0))
("cranmore" nil (k r aa1 n m ao0 r))
("crannell" nil (k r ae1 n ax l))
("cranney" nil (k r ae1 n iy0))
("crannies" nil (k r ae1 n iy0 z))
("cranny" nil (k r ae1 n iy0))
("cranor" nil (k r ey1 n er0))
("crans" nil (k r ae1 n z))
("cranshaw" nil (k r ae1 n sh ao1))
("cranston" nil (k r ae1 n s t ax n))
("crap" nil (k r ae1 p))
("crapo" nil (k r aa1 p ow0))
("crappie" nil (k r ae1 p iy0))
("crapps" nil (k r ae1 p s))
("craps" nil (k r ae1 p s))
("crapser" nil (k r ae1 p s er0))
("crapshoot" nil (k r ae1 p sh uw1 t))
("crary" nil (k er1 eh0 r iy0))
("crase" nil (k r ey1 z))
("crash" nil (k r ae1 sh))
("crashed" nil (k r ae1 sh t))
("crashes" nil (k r ae1 sh ih0 z))
("crashing" nil (k r ae1 sh ih0 ng))
("crasnianski" nil (k r ae1 s n iy0 ae1 n s k iy0))
("crass" nil (k r ae1 s))
("crate" nil (k r ey1 t))
("crater" nil (k r ey1 t er0))
("cratered" nil (k r ey1 t er0 d))
("craters" nil (k r ey1 t er0 z))
("crates" nil (k r ey1 t s))
("crating" nil (k r ey1 t ih0 ng))
("craton" nil (k r ae1 t ax n))
("cratty" nil (k r ae1 t iy0))
("craugh" nil (k r ao1))
("craun" nil (k r ao1 n))
("cravath" nil (k r ae1 v ax th))
("crave" nil (k r ey1 v))
("craven" nil (k r ey1 v ax n))
("cravens" nil (k r ey1 v ax n z))
("craver" nil (k r ey1 v er0))
("craves" nil (k r ey1 v z))
("cravey" nil (k r ey1 v iy0))
("craving" nil (k r ey1 v ih0 ng))
("cravings" nil (k r ey1 v ih0 ng z))
("craw" nil (k r ao1))
("crawfish" nil (k r ao1 f ih1 sh))
("crawford" nil (k r ao1 f er0 d))
("crawfordsville" nil (k r ao1 f er0 d z v ih1 l))
("crawl" nil (k r ao1 l))
("crawled" nil (k r ao1 l d))
("crawley" nil (k r ao1 l iy0))
("crawling" nil (k r ao1 l ih0 ng))
("crawls" nil (k r ao1 l z))
("crawly" nil (k r ao1 l iy0))
("crawmer" nil (k r ao1 m er0))
("crawshaw" nil (k r ao1 sh ao1))
("craxi" nil (k r ae1 k s iy0))
("cray" nil (k r ey1))
("craycraft" nil (k r ey1 k r ae1 f t))
("crayfish" nil (k r ey1 f ih0 sh))
("crayne" nil (k r ey1 n))
("crayon" nil (k r ey1 aa1 n))
("crayons" nil (k r ey1 aa1 n z))
("crays" nil (k r ey1 z))
("crayton" nil (k r ey1 t ax n))
("craze" nil (k r ey1 z))
("crazed" nil (k r ey1 z d))
("crazier" nil (k r ey1 z iy0 er0))
("crazies" nil (k r ey1 z iy0 z))
("craziest" nil (k r ey1 z iy0 ax s t))
("crazily" nil (k r ey1 z ax l iy0))
("craziness" nil (k r ey1 z iy0 n ax s))
("crazy" nil (k r ey1 z iy0))
("crea" nil (k r iy1))
("creach" nil (k r iy1 ch))
("creager" nil (k r iy1 ih0 jh er0))
("creagh" nil (k r iy1 g))
("creaked" nil (k r iy1 k t))
("creaking" nil (k r iy1 k ih0 ng))
("creaky" nil (k r iy1 k iy0))
("creal" nil (k r iy1 l))
("cream" nil (k r iy1 m))
("creamed" nil (k r iy1 m d))
("creamer" nil (k r iy1 m er0))
("creamery" nil (k r iy1 m er0 iy0))
("creamier" nil (k r iy1 m iy0 er0))
("creamiest" nil (k r iy1 m iy0 ih0 s t))
("creams" nil (k r iy1 m z))
("creamy" nil (k r iy1 m iy0))
("crean" nil (k r iy1 n))
("crear" nil (k r ih1 r))
("crease" nil (k r iy1 s))
("creasey" nil (k r iy1 s iy0))
("creasman" nil (k r iy1 z m ax n))
("creason" nil (k r iy1 s ax n))
("creasy" nil (k r iy1 s iy0))
("create" nil (k r iy0 ey1 t))
("created" nil (k r iy0 ey1 t ax d))
("creates" nil (k r iy0 ey1 t s))
("creath" nil (k r eh1 th))
("creating" nil (k r iy0 ey1 t ih0 ng))
("creation" nil (k r iy0 ey1 sh ax n))
("creationism" nil (k r iy0 ey1 sh ax n ih1 z ax m))
("creations" nil (k r iy0 ey1 sh ax n z))
("creative" nil (k r iy0 ey1 t ih0 v))
("creatively" nil (k r iy0 ey1 t ih0 v l iy0))
("creativeness" nil (k r iy0 ey1 t ih0 v n ax s))
("creativity" nil (k r iy1 ey0 t ih1 v ax t iy0))
("creator" nil (k r iy0 ey1 t er0))
("creators" nil (k r iy0 ey1 t er0 z))
("creature" nil (k r iy1 ch er0))
("creatures" nil (k r iy1 ch er0 z))
("crecelius" nil (k r ih0 s iy1 l iy0 ih0 s))
("credence" nil (k r iy1 d ax n s))
("credential" nil (k r ih0 d eh1 n ch ax l))
("credentials" nil (k r ax d eh1 n sh ax l z))
("credeur" nil (k r ih0 d er1))
("credibility" nil (k r eh1 d ax b ih1 l ih0 t iy0))
("credible" nil (k r eh1 d ax b ax l))
("credibly" nil (k r eh1 d ax b l iy0))
("credit" nil (k r eh1 d ax t))
("creditable" nil (k r eh1 d ax t ax b ax l))
("creditably" nil (k r eh1 d ax t ax b l iy0))
("creditanstalt" nil (k r eh1 d ih1 t ax n sh t ao1 l t))
("creditbank" nil (k r eh1 d ih0 t b ae1 ng k))
("credited" nil (k r eh1 d ax t ax d))
("credithrift" nil (k r eh1 d ih0 th r ih1 f t))
("crediting" nil (k r eh1 d ax t ih0 ng))
("credito" nil (k r eh0 d iy1 t ow0))
("creditor" nil (k r eh1 d ax t er0))
("creditors" nil (k r eh1 d ih0 t er0 z))
("credits" nil (k r eh1 d ih0 t s))
("creditwatch" nil (k r eh1 d ih0 t w aa1 ch))
("creditworthiness" nil (k r eh1 d ih0 t w er1 dh iy0 n ax s))
("creditworthy" nil (k r eh1 d ih0 t w er1 dh iy0))
("credle" nil (k r eh1 d ax l))
("credo" nil (k r ey1 d ow0))
("credulity" nil (k r ih0 d uw1 l ax t iy0))
("cree" nil (k r iy1))
("creech" nil (k r iy1 ch))
("creecy" nil (k r iy1 s iy0))
("creed" nil (k r iy1 d))
("creeden" nil (k r iy1 d ax n))
("creedon" nil (k r iy1 d ax n))
("creeds" nil (k r iy1 d z))
("creegan" nil (k r iy1 g ax n))
("creek" nil (k r iy1 k))
("creekmore" nil (k r iy1 k m ao0 r))
("creekmur" nil (k r iy1 k m er0))
("creeks" nil (k r iy1 k s))
("creel" nil (k r iy1 l))
("creelman" nil (k r iy1 l m ax n))
("creely" nil (k r iy1 l iy0))
("creep" nil (k r iy1 p))
("creeping" nil (k r iy1 p ih0 ng))
("creeps" nil (k r iy1 p s))
("creepy" nil (k r iy1 p iy0))
("creer" nil (k r ih1 r))
("crees" nil (k r iy1 z))
("cref" nil (k r eh1 f))
("cregan" nil (k r iy1 g ax n))
("cregar" nil (k r iy1 g er0))
("creger" nil (k r iy1 jh er0))
("cregg" nil (k r eh1 g))
("cregger" nil (k r eh1 g er0))
("crego" nil (k r eh1 g ow0))
("crehan" nil (k r eh1 hh ax n))
("creighton" nil (k r ey1 t ax n))
("crellin" nil (k r eh1 l ih0 n))
("cremate" nil (k r iy1 m ey0 t))
("cremation" nil (k r iy0 m ey1 sh ax n))
("creme" nil (k r iy1 m))
("cremeans" nil (k r eh1 m ax n z))
("cremeens" nil (k r ih0 m iy1 n z))
("cremer" nil (k r iy1 m er0))
("cremin" nil (k r eh1 m ih0 n))
("crenelated" nil (k r eh1 n ax l ey1 t ax d))
("crenshaw" nil (k r eh1 n sh ao1))
("crenwelge" nil (k r eh1 n w ih0 l jh))
("creole" nil (k r iy1 ow0 l))
("creoles" nil (k r iy0 ow1 l z))
("creolized" nil (k r iy1 ow1 l ay1 z d))
("creosote" nil (k r iy1 ax s ow1 t))
("crepe" nil (k r ey1 p))
("crepeau" nil (k r ih0 p ow1))
("crepes" nil (k r ey1 p s))
("creps" nil (k r eh1 p s))
("crept" nil (k r eh1 p t))
("creque" nil (k r eh1 k))
("cresap" nil (k r eh1 s ax p))
("crescendo" nil (k r ih0 sh eh1 n d ow0))
("crescent" nil (k r eh1 s ax n t))
("crescenzi" nil (k r eh0 s ch eh1 n z iy0))
("crescenzo" nil (k r eh0 s ch eh1 n z ow0))
("cresci" nil (k r eh1 s iy0))
("crescott" nil (k r eh1 s k aa0 t))
("creson" nil (k r eh1 s ax n))
("crespi" nil (k r eh1 s p iy0))
("crespin" nil (k r eh1 s p ih0 n))
("crespo" nil (k r eh1 s p ow0))
("cress" nil (k r eh1 s))
("cresses" nil (k r eh1 s ax z))
("cressey" nil (k r eh1 s iy0))
("cressler" nil (k r eh1 s l er0))
("cressman" nil (k r eh1 s m ax n))
("cresson" nil (k r eh1 s ax n))
("cresswell" nil (k r eh1 s w eh1 l))
("cressy" nil (k r eh1 s iy0))
("crest" nil (k r eh1 s t))
("crestar" nil (k r eh1 s t aa1 r))
("crested" nil (k r eh1 s t ax d))
("crestfallen" nil (k r eh1 s t f ao1 l ax n))
("crestmont" nil (k r eh1 s t m aa1 n t))
("creswell" nil (k r eh1 s w eh1 l))
("crete" nil (k r iy1 t))
("cretella" nil (k r eh0 t eh1 l ax))
("cretsinger" nil (k r eh1 t s ih0 n jh er0))
("creveling" nil (k r eh1 v ax l ih0 ng))
("crevice" nil (k r eh1 v ax s))
("crevices" nil (k r eh1 v ax s ih0 z))
("crevier" nil (k r iy1 v iy0 er0))
("creviston" nil (k r eh1 v ih0 s t aa0 n))
("crew" nil (k r uw1))
("crewe" nil (k r uw1))
("crewel" nil (k r uw1 ax l))
("crewes" nil (k r uw1 z))
("crewman" nil (k r uw1 m ax n))
("crewmen" nil (k r uw1 m ih0 n))
("crews" nil (k r uw1 z))
("crib" nil (k r ih1 b))
("cribari" nil (k r iy0 b aa1 r iy0))
("cribb" nil (k r ih1 b))
("cribbage" nil (k r ih1 b ih0 jh))
("cribbs" nil (k r ih1 b z))
("cribs" nil (k r ih1 b z))
("crichlow" nil (k r ih1 k l ow0))
("crichton" nil (k r ih1 ch t ax n))
("crick" nil (k r ih1 k))
("cricket" nil (k r ih1 k ax t))
("crickets" nil (k r ih1 k ax t s))
("criddle" nil (k r ih1 d ax l))
("crider" nil (k r ay1 d er0))
("cried" nil (k r ay1 d))
("crier" nil (k r ay1 er0))
("cries" nil (k r ay1 z))
("criger" nil (k r ay1 jh er0))
("crigger" nil (k r ih1 g er0))
("crigler" nil (k r ih1 g l er0))
("crihfield" nil (k r ih1 f iy0 l d))
("crill" nil (k r ih1 l))
("crilley" nil (k r ih1 l iy0))
("crilly" nil (k r ih1 l iy0))
("crim" nil (k r ih1 m))
("crime" nil (k r ay1 m))
("crimea" nil (k r ay0 m iy1 ax))
("crimean" nil (k r ih0 m iy1 n))
("crimes" nil (k r ay1 m z))
("crimi" nil (k r iy1 m iy0))
("criminal" nil (k r ih1 m ax n ax l))
("criminalist" nil (k r ih1 m ax n ax l ih0 s t))
("criminalists" nil (k r ih1 m ax n ax l ih0 s t s))
("criminality" nil (k r ih1 m ax n ae1 l ih0 t iy0))
("criminalize" nil (k r ih1 m ax n ax l ay1 z))
("criminalizing" nil (k r ih1 m ax n ax l ay1 z ih0 ng))
("criminally" nil (k r ih1 m ax n ax l iy0))
("criminals" nil (k r ih1 m ax n ax l z))
("criminologist" nil (k r ih1 m ih0 n aa1 l ax jh ih0 s t))
("criminologists" nil (k r ih1 m ih0 n aa1 l ax jh ih0 s t s))
("criminology" nil (k r ih1 m ih0 n aa1 l ax jh iy0))
("crimm" nil (k r ih1 m))
("crimmins" nil (k r ih1 m ih0 n z))
("crimp" nil (k r ih1 m p))
("crimped" nil (k r ih1 m p t))
("crimping" nil (k r ih1 m p ih0 ng))
("crimps" nil (k r ih1 m p s))
("crimson" nil (k r ih1 m z ax n))
("criner" nil (k r ay1 n er0))
("cringe" nil (k r ih1 n jh))
("cringed" nil (k r ih1 n jh d))
("cringes" nil (k r ih1 n jh ih0 z))
("cripe" nil (k r ay1 p))
("crippen" nil (k r ih1 p ax n))
("cripple" nil (k r ih1 p ax l))
("crippled" nil (k r ih1 p ax l d))
("cripples" nil (k r ih1 p ax l z))
("crippling" nil (k r ih1 p ax l ih0 ng))
("cripps" nil (k r ih1 p s))
("crips" nil (k r ih1 p s))
("crisafi" nil (k r iy0 s aa1 f iy0))
("crisafulli" nil (k r iy0 s aa0 f uw1 l iy0))
("crisanti" nil (k r ih0 s ae1 n t iy0))
("crisci" nil (k r ih1 s iy0))
("criscione" nil (k r iy0 s ch ow1 n iy0))
("crisco" nil (k r iy1 s k ow0))
("criscuolo" nil (k r iy0 s k w ow1 l ow0))
("crise" nil (k r ay1 z))
("crises" nil (k r ay1 s iy0 z))
("crisis" nil (k r ay1 s ax s))
("crisler" nil (k r ih1 s ax l er0))
("crislip" nil (k r ih1 s l ih0 p))
("crisman" nil (k r ih1 s m ax n))
("crismon" nil (k r ih1 z m ax n))
("crisostomo" nil (k r iy0 s ow0 s t ow1 m ow0))
("crisp" nil (k r ih1 s p))
("crispell" nil (k r ih1 s p ax l))
("crispen" nil (k r ih1 s p ax n))
("crisper" nil (k r ih1 s p er0))
("crispin" nil (k r ih1 s p ih0 n))
("crispina" nil (k r iy0 s p iy1 n ax))
("crispino" nil (k r iy0 s p iy1 n ow0))
("crisply" nil (k r ih1 s p l iy0))
("crispness" nil (k r ih1 s p n ax s))
("crispo" nil (k r iy1 s p ow0))
("crispy" nil (k r ih1 s p iy0))
("criss" nil (k r ih1 s))
("crisscross" nil (k r ih1 s k r ao1 s))
("crisscrossed" nil (k r ih1 s k r ao1 s t))
("crisscrossing" nil (k r ih1 s k r ao1 s ih0 ng))
("crissey" nil (k r ih1 s iy0))
("crissinger" nil (k r ih1 s ih0 n jh er0))
("crissman" nil (k r ih1 s m ax n))
("crist" nil (k r ih1 s t))
("cristabel" nil (k r ih1 s t ax b eh0 l))
("cristal" nil (k r ih1 s t ax l))
("cristiani" nil (k r ih1 s t iy0 aa1 n iy0))
("cristiano" nil (k r iy0 s t iy0 aa1 n ow0))
("cristina" nil (k r ih0 s t iy1 n ax))
("cristobal" nil (k r ih1 s t ax b ax l))
("cristobel" nil (k r ih1 s t ax b eh0 l))
("criswell" nil (k r ih1 s w eh0 l))
("critcher" nil (k r ih1 ch er0))
("critchfield" nil (k r ih1 ch f iy0 l d))
("critchley" nil (k r ih1 ch l iy0))
("critchlow" nil (k r ih1 ch l ow0))
("critelli" nil (k r ih0 t eh1 l iy0))
("criteria" nil (k r ay0 t ih1 r iy0 ax))
("criterion" nil (k r ay0 t ih1 r iy0 ax n))
("crites" nil (k r ay1 t s))
("critic" nil (k r ih1 t ih0 k))
("critical" nil (k r ih1 t ih0 k ax l))
("criticality" nil (k r ih1 t ax k ae1 l ax t iy0))
("critically" nil (k r ih1 t ih0 k ax l iy0))
("criticism" nil (k r ih1 t ih0 s ih1 z ax m))
("criticisms" nil (k r ih1 t ih0 s ih1 z ax m z))
("criticize" nil (k r ih1 t ih0 s ay1 z))
("criticized" nil (k r ih1 t ax s ay1 z d))
("criticizes" nil (k r ih1 t ih0 s ay1 z ih0 z))
("criticizing" nil (k r ih1 t ih0 s ay1 z ih0 ng))
("critics" nil (k r ih1 t ih0 k s))
("critique" nil (k r ax t iy1 k))
("critiques" nil (k r ih0 t iy1 k s))
("criton" nil (k r ay1 t ax n))
("critser" nil (k r ih1 t s er0))
("crittenden" nil (k r ih0 t eh1 n d ax n))
("crittendon" nil (k r ih1 t ax n d ih0 n))
("critter" nil (k r ih1 t er0))
("critters" nil (k r ih1 t er0 z))
("critz" nil (k r ih1 t s))
("critzer" nil (k r ih1 t z er0))
("crivelli" nil (k r ih0 v eh1 l iy0))
("crivello" nil (k r ih0 v eh1 l ow0))
("crnkovich" nil (s er1 n ax v ih1 ch))
("cro" nil (k r ow1))
("croak" nil (k r ow1 k))
("croaker" nil (k r ow1 k er0))
("croaking" nil (k r ow1 k ih0 ng))
("croat" nil (k r ow1 aa1 t))
("croatia" nil (k r ow0 ey1 sh ax))
("croatian" nil (k r ow0 ey1 sh ax n))
("croatians" nil (k r ow0 ey1 sh ax n z))
("croats" nil (k r ow1 aa1 t s))
("croc" nil (k r aa1 k))
("crocco" nil (k r aa1 k ow0))
("croce" nil (k r ow1 ch iy0))
("crochet" nil (k r ow0 sh ey1))
("crocheted" nil (k r ow0 sh ey1 d))
("crocheting" nil (k r ow0 sh ey1 ih0 ng))
("crock" nil (k r aa1 k))
("crocker" nil (k r aa1 k er0))
("crockery" nil (k r aa1 k er0 iy0))
("crockett" nil (k r aa1 k ih0 t))
("crockford" nil (k r aa1 k f er0 d))
("crocodile" nil (k r aa1 k ax d ay1 l))
("crocodiles" nil (k r aa1 k ax d ay1 l z))
("crocus" nil (k r ow1 k ax s))
("crocuses" nil (k r ow1 k y uw0 s ih0 z))
("croff" nil (k r ao1 f))
("crofford" nil (k r aa1 f er0 d))
("crofoot" nil (k r ow1 f uh0 t))
("croft" nil (k r ao1 f t))
("crofton" nil (k r aa1 f t ax n))
("crofts" nil (k r ao1 f t s))
("crogan" nil (k r ow1 g ax n))
("croghan" nil (k r aa1 g hh ax n))
("croissant" nil (k w aa1 s aa1 n t))
("croissants" nil (k w aa1 s aa1 n t s))
("croix" nil (k r oy1))
("croke" nil (k r ow1 k))
("croker" nil (k r ow1 k er0))
("croley" nil (k r ow1 l iy0))
("croll" nil (k r ow1 l))
("crom" nil (k r aa1 m))
("cromartie" nil (k r aa1 m aa0 r t iy0))
("crombie" nil (k r aa1 m b iy0))
("cromer" nil (k r ow1 m er0))
("cromie" nil (k r aa1 m iy0))
("cromley" nil (k r aa1 m l iy0))
("crompton" nil (k r aa1 m p t ax n))
("cromwell" nil (k r aa1 m w ax l))
("cron" nil (k r aa1 n))
("cronan" nil (k r ow1 n ax n))
("cronauer" nil (k r aa1 n aw0 r))
("cronce" nil (k r aa1 n s))
("crone" nil (k r ow1 n))
("cronen" nil (k r ow1 n ax n))
("croner" nil (k r ow1 n er0))
("croney" nil (k r ow1 n iy0))
("cronic" nil (k r aa1 n ih0 k))
("cronies" nil (k r ow1 n iy0 z))
("cronin" nil (k r ow1 n ih0 n))
("cronk" nil (k r aa1 ng k))
("cronkhite" nil (k r aa1 ng k hh ay1 t))
("cronkite" nil (k r aa1 ng k ay1 t))
("cronkright" nil (k r aa1 ng k r ay1 t))
("cronquist" nil (k r aa1 n k w ih1 s t))
("cronus" nil (k r ow1 n ax s))
("crony" nil (k r ow1 n iy0))
("cronyism" nil (k r ow1 n iy0 ih1 z ax m))
("crook" nil (k r uh1 k))
("crooke" nil (k r uh1 k))
("crooked" nil (k r uh1 k ax d))
("crookedness" nil (k r uh1 k ax d n ax s))
("crooker" nil (k r uh1 k er0))
("crooklyn" nil (k r uh1 k l ih1 n))
("crooks" nil (k r uh1 k s))
("crookshank" nil (k r uh1 k sh ae1 ng k))
("crookshanks" nil (k r uh1 k sh ae1 ng k s))
("crookston" nil (k r uh1 k s t ax n))
("croom" nil (k r uw1 m))
("crooms" nil (k r uw1 m z))
("crooner" nil (k r uw1 n er0))
("crooning" nil (k r uw1 n ih0 ng))
("croons" nil (k r uw1 n z))
("crop" nil (k r aa1 p))
("cropland" nil (k r aa1 p l ae1 n d))
("cropley" nil (k r aa1 p l iy0))
("cropp" nil (k r aa1 p))
("cropped" nil (k r aa1 p t))
("cropper" nil (k r aa1 p er0))
("cropping" nil (k r aa1 p ih0 ng))
("crops" nil (k r aa1 p s))
("cropsey" nil (k r aa1 p s iy0))
("croquet" nil (k r ow0 k ey1))
("cros" nil (k r ao1 s))
("crosbey" nil (k r aa1 s b iy0))
("crosbie" nil (k r ao1 z b iy0))
("crosby" nil (k r ao1 z b iy0))
("crose" nil (k r ow1 z))
("croshaw" nil (k r aa1 sh ao0))
("croskey" nil (k r aa1 s k iy0))
("crosland" nil (k r aa1 s l ax n d))
("crosley" nil (k r aa1 s l iy0))
("croslin" nil (k r aa1 s l ih0 n))
("croson" nil (k r ow1 s ax n))
("cross" nil (k r ao1 s))
("crossan" nil (k r aa1 s ax n))
("crossbills" nil (k r ao1 s b ih1 l z))
("crossbones" nil (k r ao1 s b ow1 n z))
("crossborder" nil (k r ao1 s b ao1 r d er0))
("crossbow" nil (k r ao1 s b ow1))
("crosscurrents" nil (k r ao1 s k er1 ax n t s))
("crosse" nil (k r aa1 s))
("crossed" nil (k r ao1 s t))
("crossen" nil (k r ao1 s ax n))
("crosser" nil (k r ao1 s er0))
("crosses" nil (k r ao1 s ih0 z))
("crossett" nil (k r aa1 s ih0 t))
("crossfield" nil (k r ao1 s f iy1 l d))
("crossfire" nil (k r ao1 s f ay1 r))
("crossin" nil (k r aa1 s ih0 n))
("crossing" nil (k r ao1 s ih0 ng))
("crossings" nil (k r ao1 s ih0 ng z))
("crossland" nil (k r ao1 s l ae1 n d))
("crossley" nil (k r aa1 s l iy0))
("crosslin" nil (k r aa1 s l ih0 n))
("crossman" nil (k r ao1 s m ax n))
("crossno" nil (k r ow1 s n ow0))
("crosson" nil (k r aa1 s ax n))
("crossover" nil (k r ao1 s ow1 v er0))
("crosspiece" nil (k r ao1 s p iy1 s))
("crosspieces" nil (k r ao1 s p iy1 s ax z))
("crossroads" nil (k r ao1 s r ow1 d z))
("crosstalk" nil (k r ao1 s t ao1 k))
("crosstown" nil (k r ao1 s t aw1 n))
("crosswhite" nil (k r aa1 s w ay0 t))
("crosswise" nil (k r ao1 s w ay1 z))
("crossword" nil (k r ao1 s w er1 d))
("crosswords" nil (k r ao1 s w er1 d z))
("crosthwait" nil (k r aa1 s th w ax t))
("crosthwaite" nil (k r aa1 s th w ax t))
("croston" nil (k r aa1 s t ax n))
("croswell" nil (k r aa1 s w eh0 l))
("crotch" nil (k r aa1 ch))
("crotchety" nil (k r aa1 ch ax t iy0))
("croteau" nil (k r ax t ow1))
("crothers" nil (k r ah1 dh er0 z))
("crotonville" nil (k r ow1 t ax n v ih1 l))
("crotteau" nil (k r ax t ow1))
("crotts" nil (k r aa1 t s))
("crotty" nil (k r aa1 t iy0))
("crotwell" nil (k r aa1 t w eh1 l))
("crotzer" nil (k r ow1 t z er0))
("crouch" nil (k r aw1 ch))
("crouched" nil (k r aw1 ch t))
("croucher" nil (k r aw1 ch er0))
("crough" nil (k r aw1))
("crounse" nil (k r aw1 n s))
("crouse" nil (k r aw1 s))
("crouser" nil (k r aw1 s er0))
("crout" nil (k r aw1 t))
("crouthamel" nil (k r aw1 th ax m eh0 l))
("crovitz" nil (k r ow1 v ih0 t s))
("crovl" nil (k r ow1 v ax l))
("crovls" nil (k r ow1 v ax l z))
("crow" nil (k r ow1))
("crowbar" nil (k r ow1 b aa1 r))
("crowborough" nil (k r ow1 b er0 ow0))
("crowd" nil (k r aw1 d))
("crowded" nil (k r aw1 d ax d))
("crowden" nil (k r aw1 d ax n))
("crowder" nil (k r aw1 d er0))
("crowding" nil (k r aw1 d ih0 ng))
("crowds" nil (k r aw1 d z))
("crowe" nil (k r ow1))
("crowed" nil (k r ow1 d))
("crowell" nil (k r ow1 ax l))
("crowing" nil (k r ow1 ih0 ng))
("crowkeepers" nil (k r ow1 k iy1 p er0 z))
("crowl" nil (k r aw1 l))
("crowley" nil (k r aw1 l iy0))
("crown" nil (k r aw1 n))
("crowne" nil (k r aw1 n))
("crowned" nil (k r aw1 n d))
("crowner" nil (k r aw1 n er0))
("crowning" nil (k r aw1 n ih0 ng))
("crownlike" nil (k r aw1 n l ay1 k))
("crownover" nil (k r aw1 n ow1 v er0))
("crowns" nil (k r aw1 n z))
("crownx" nil (k r aw1 n eh1 k s))
("crows" nil (k r ow1 z))
("crowson" nil (k r aw1 s ax n))
("crowther" nil (k r aw1 dh er0))
("crowthers" nil (k r ow1 th er0 z))
("croxton" nil (k r aa1 k s t ax n))
("croy" nil (k r oy1))
("croyle" nil (k r oy1 l))
("crozier" nil (k r ow1 zh er0))
("cruce" nil (k r uw1 s))
("cruces" nil (k r uw1 s iy0 z))
("crucial" nil (k r uw1 sh ax l))
("crucially" nil (k r uw1 sh ax l l iy0))
("crucible" nil (k r uw1 s ax b ax l))
("crucified" nil (k r uw1 s ax f ay1 d))
("crucifix" nil (k r uw1 s ax f ih1 k s))
("crucifixes" nil (k r uw1 s ax f ih1 k s ih0 z))
("crucifixion" nil (k r uw1 s ih0 f ih1 k sh ax n))
("crucify" nil (k r uw1 s ax f ay1))
("crud" nil (k r ah1 d))
("cruddy" nil (k r ah1 d iy0))
("crude" nil (k r uw1 d))
("crudele" nil (k r uw1 d ax l))
("crudely" nil (k r uw1 d l iy0))
("crudes" nil (k r uw1 d z))
("crudup" nil (k r ah1 d ah1 p))
("cruea" nil (k r uw1 iy0 ax))
("cruel" nil (k r uw1 ax l))
("cruelest" nil (k r uw1 l ax s t))
("cruelly" nil (k r uw1 l iy0))
("cruelties" nil (k r uw1 l t iy0 z))
("cruelty" nil (k r uw1 l t iy0))
("cruey" nil (k r ah1 iy0))
("cruger" nil (k r uw1 jh er0))
("cruickshank" nil (k r uh1 k sh ae1 ng k))
("cruikshank" nil (k r uw1 ih0 k sh ae1 ng k))
("cruise" nil (k r uw1 z))
("cruised" nil (k r uw1 z d))
("cruiser" nil (k r uw1 z er0))
("cruisers" nil (k r uw1 z er0 z))
("cruises" nil (k r uw1 z ih0 z))
("cruising" nil (k r uw1 z ih0 ng))
("crull" nil (k r ah1 l))
("crum" nil (k r ah1 m))
("crumb" nil (k r ah1 m))
("crumbaugh" nil (k r ah1 m b ao1))
("crumble" nil (k r ah1 m b ax l))
("crumbled" nil (k r ah1 m b ax l d))
("crumbles" nil (k r ah1 m b ax l z))
("crumbley" nil (k r ah1 m b l iy0))
("crumbling" nil (k r ah1 m b ax l ih0 ng))
("crumbs" nil (k r ah1 m z))
("crumby" nil (k r ah1 m b iy0))
("crume" nil (k r uw1 m))
("crumitie" nil (k r uw1 m ih0 t ay0))
("crumley" nil (k r ah1 m l iy0))
("crumly" nil (k r ah1 m l iy0))
("crumm" nil (k r ah1 m))
("crummett" nil (k r ah1 m ih0 t))
("crummey" nil (k r ah1 m iy0))
("crummy" nil (k r ah1 m iy0))
("crump" nil (k r ah1 m p))
("crumpacker" nil (k r ah1 m p ax k er0))
("crumple" nil (k r ah1 m p ax l))
("crumpled" nil (k r ah1 m p ax l d))
("crumpler" nil (k r ah1 m p ax l er0))
("crumpton" nil (k r ah1 m p t ax n))
("crumrine" nil (k r ah1 m r iy1 n))
("crunch" nil (k r ah1 n ch))
("crunched" nil (k r ah1 n ch t))
("cruncher" nil (k r ah1 n ch er0))
("crunchers" nil (k r ah1 n ch er0 z))
("crunches" nil (k r ah1 n ch ih0 z))
("crunching" nil (k r ah1 n ch ih0 ng))
("crunchy" nil (k r ah1 n ch iy0))
("crunk" nil (k r ah1 ng k))
("crunkleton" nil (k r ah1 ng k l ih0 t aa0 n))
("crupi" nil (k r uw1 p iy0))
("crusade" nil (k r uw0 s ey1 d))
("crusader" nil (k r uw0 s ey1 d er0))
("crusaders" nil (k r uw0 s ey1 d er0 z))
("crusades" nil (k r uw0 s ey1 d z))
("crusading" nil (k r uw0 s ey1 d ih0 ng))
("crusan" nil (k r uw1 z ax n))
("cruse" nil (k r uw1 z))
("cruser" nil (k r uw1 z er0))
("crush" nil (k r ah1 sh))
("crushed" nil (k r ah1 sh t))
("crusher" nil (k r ah1 sh er0))
("crushers" nil (k r ah1 sh er0 z))
("crushes" nil (k r ah1 sh ih0 z))
("crushing" nil (k r ah1 sh ih0 ng))
("crusoe" nil (k r uw1 s ow0))
("crust" nil (k r ah1 s t))
("crustacean" nil (k r ax s t ey1 sh ax n))
("crustaceans" nil (k r ax s t ey1 sh ax n z))
("crustal" nil (k r ah1 s t ax l))
("crusted" nil (k r ah1 s t ih0 d))
("crusts" nil (k r ah1 s t s))
("crusty" nil (k r ah1 s t iy0))
("crutch" nil (k r ah1 ch))
("crutcher" nil (k r ah1 ch er0))
("crutches" nil (k r ah1 ch ih0 z))
("crutchfield" nil (k r ah1 ch f iy1 l d))
("crutchley" nil (k r ah1 ch l iy0))
("crute" nil (k r uw1 t))
("cruthirds" nil (k r ah1 th er0 d z))
("crux" nil (k r ah1 k s))
("cruz" nil (k r uw1 z))
("cruzado" nil (k r uw1 z aa1 d ow0))
("cruzados" nil (k r uw1 z aa1 d ow0 s))
("cruzan" nil (k r uw1 z ax n))
("cruze" nil (k r uw1 z))
("cruzen" nil (k r uw1 z ax n))
("cry" nil (k r ay1))
("cryan" nil (k r ay1 ax n))
("cryder" nil (k r ay1 d er0))
("cryderman" nil (k r ay1 d er0 m ax n))
("crye" nil (k r ay1))
("cryer" nil (k r ay1 er0))
("crying" nil (k r ay1 ih0 ng))
("crymes" nil (k r ay1 m z))
("cryolite" nil (k r ay1 ax l ay1 t))
("crypt" nil (k r ih1 p t))
("cryptic" nil (k r ih1 p t ih0 k))
("crypto" nil (k r ih1 p t ow0))
("cryptoclearance" nil (k r ih1 p t ow0 l ih1 r ax n s))
("crypts" nil (k r ih1 p t s))
("crysler" nil (k r ih1 s ax l er0))
("crystal" nil (k r ih1 s t ax l))
("crystalline" nil (k r ih1 s t ax l ay1 n))
("crystallize" nil (k r ih1 s t ax l ay1 z))
("crystallized" nil (k r ih1 s t ax l ay1 z d))
("crystallizing" nil (k r ih1 s t ax l ay1 z ih0 ng))
("crystallographer" nil (k r ih1 s t ax l aa1 g r ax f er0))
("crystallography" nil (k r ih1 s t ax l aa1 g r ax f iy0))
("crystals" nil (k r ih1 s t ax l z))
("cryster" nil (k r ay1 s t er0))
("crytzer" nil (k r ay1 t z er0))
("csaszar" nil (k ax s aa1 sh er0))
("cuadra" nil (k uw0 aa1 d r ax))
("cuadrado" nil (k uw0 aa0 d r aa1 d ow0))
("cuajone" nil (k y uw1 ax jh ow1 n))
("cuauhtemoc" nil (k y uw0 aw1 t ax m aa0 k))
("cub" nil (k ah1 b))
("cuba" nil (k y uw1 b ax))
("cuban" nil (k y uw1 b ax n))
("cubans" nil (k y uw1 b ax n z))
("cubbage" nil (k ah1 b ih0 jh))
("cubbies" nil (k ah1 b iy0 z))
("cubbison" nil (k ah1 b ih0 s ax n))
("cubbyhole" nil (k ah1 b iy0 hh ow1 l))
("cube" nil (k y uw1 b))
("cubes" nil (k y uw1 b z))
("cubic" nil (k y uw1 b ih0 k))
("cubicle" nil (k y uw1 b ih0 k ax l))
("cubicles" nil (k y uw1 b ih0 k ax l z))
("cubism" nil (k y uw1 b ih0 z ax m))
("cubist" nil (k y uw1 b ih0 s t))
("cubit" nil (k y uw1 b ih0 t))
("cubs" nil (k ah1 b z))
("cuccaro" nil (k uw0 k aa1 r ow0))
("cucchi" nil (k uw1 k iy0))
("cucchiara" nil (k uw0 k iy0 aa1 r ax))
("cucci" nil (k uw1 ch iy0))
("cuccia" nil (k uw1 ch ax))
("cuccio" nil (k uw1 ch iy0 ow0))
("cuckoo" nil (k ah1 k uw1))
("cuckoos" nil (k uw1 k uw0 z))
("cucumber" nil (k y uw1 k ax m b er0))
("cucumbers" nil (k y uw1 k ax m er0 z))
("cudahy" nil (k ah1 d ax hh iy0))
("cudd" nil (k ah1 d))
("cuddeback" nil (k ah1 d b ae0 k))
("cuddihy" nil (k ah1 d ih0 hh iy0))
("cuddle" nil (k ah1 d ax l))
("cuddled" nil (k ah1 d ax l d))
("cuddling" nil (k ah1 d l ih0 ng))
("cuddly" nil (k ah1 d l iy0))
("cuddy" nil (k ah1 d iy0))
("cude" nil (k y uw1 d))
("cudgel" nil (k ah1 jh ax l))
("cudgels" nil (k ah1 jh ax l z))
("cudmore" nil (k ah1 d m ao0 r))
("cudney" nil (k ah1 d n iy0))
("cudworth" nil (k ah1 d w er1 th))
("cue" nil (k y uw1))
("cued" nil (k y uw1 d))
("cuellar" nil (k y uw1 l er0))
("cuello" nil (k uw0 eh1 l ow0))
("cuenca" nil (k w eh1 n k ax))
("cuero" nil (k w eh1 r ow0))
("cuervo" nil (k uh1 r v ow0))
("cues" nil (k y uw1 z))
("cuesta" nil (k w eh1 s t ax))
("cueto" nil (k w eh1 t ow0))
("cueva" nil (k w eh1 v ax))
("cuevas" nil (k w eh1 v aa0 z))
("cuff" nil (k ah1 f))
("cuffe" nil (k ah1 f))
("cuffee" nil (k ah1 f iy1))
("cuffs" nil (k ah1 f s))
("cugini" nil (k uw0 jh iy1 n iy0))
("cuhney" nil (k uw1 n iy0))
("cuisinart" nil (k w iy1 s ih0 n er0 t))
("cuisinarts" nil (k w iy1 z ax n aa1 r t s))
("cuisine" nil (k w ih0 z iy1 n))
("cuisines" nil (k w ih0 z iy1 n z))
("culberson" nil (k ah1 l b er0 s ax n))
("culbert" nil (k ah1 l b er0 t))
("culbertson" nil (k ah1 l b er0 t s ax n))
("culbreath" nil (k ah1 l b r eh1 th))
("culbreth" nil (k ah1 l b r ih0 th))
("culbro" nil (k ah1 l b r ow0))
("culhane" nil (k ah1 l hh ey1 n))
("culinary" nil (k y uw1 l ih0 n eh1 r iy0))
("culinova" nil (k y uw1 l ih0 n ow1 v ax))
("culkin" nil (k ah1 l k ih0 n))
("cull" nil (k ah1 l))
("cullan" nil (k ah1 l ax n))
("culled" nil (k ah1 l d))
("cullen" nil (k ah1 l ax n))
("cullens" nil (k ah1 l ax n z))
("culler" nil (k ah1 l er0))
("cullers" nil (k ah1 l er0 z))
("cullerton" nil (k ah1 l er0 t ax n))
("culleton" nil (k uw1 l ih0 t aa0 n))
("culley" nil (k ah1 l iy0))
("cullifer" nil (k ah1 l ih0 f er0))
("culligan" nil (k ah1 l ih0 g ax n))
("cullimore" nil (k ah1 l iy0 m ao0 r))
("cullin" nil (k ah1 l ih0 n))
("cullinan" nil (k ah1 l ih0 n ax n))
("cullinane" nil (k ah1 l ih0 n ey1 n))
("cullinet" nil (k ah1 l ih0 n eh1 t))
("culling" nil (k ah1 l ih0 ng))
("cullins" nil (k ah1 l ih0 n z))
("cullipher" nil (k ah1 l ih0 f er0))
("cullison" nil (k ah1 l ih0 s ax n))
("cullman" nil (k ah1 l m ax n))
("cullom" nil (k ah1 l ax m))
("cullop" nil (k ah1 l ax p))
("cullum" nil (k ah1 l ax m))
("cully" nil (k ah1 l iy0))
("culmer" nil (k ah1 l m er0))
("culminate" nil (k ah1 l m ih0 n ey1 t))
("culminated" nil (k ah1 l m ax n ey1 t ax d))
("culminates" nil (k ah1 l m ih0 n ey1 t s))
("culminating" nil (k ah1 l m ax n ey1 t ih0 ng))
("culmination" nil (k ah1 l m ax n ey1 sh ax n))
("culp" nil (k ah1 l p))
("culpa" nil (k ah1 l p ax))
("culpability" nil (k ah1 l p ax b ih1 l ih0 t iy0))
("culpable" nil (k ah1 l p ax b ax l))
("culpepper" nil (k ah1 l p ih0 p er0))
("culprit" nil (k ah1 l p r ih0 t))
("culprits" nil (k ah1 l p r ih0 t s))
("cult" nil (k ah1 l t))
("cultic" nil (k ah1 l t ih0 k))
("cultist" nil (k ah1 l t ax s t))
("cultivate" nil (k ah1 l t ax v ey1 t))
("cultivated" nil (k ah1 l t ax v ey1 t ax d))
("cultivates" nil (k ah1 l t ih0 v ey1 t s))
("cultivating" nil (k ah1 l t ih0 v ey1 t ih0 ng))
("cultivation" nil (k ah1 l t ih0 v ey1 sh ax n))
("culton" nil (k ah1 l t ax n))
("cults" nil (k ah1 l t s))
("cultural" nil (k ah1 l ch er0 ax l))
("culturally" nil (k ah1 l ch er0 ax l iy0))
("culture" nil (k ah1 l ch er0))
("cultured" nil (k ah1 l ch er0 d))
("cultures" nil (k ah1 l ch er0 z))
("culturing" nil (k ah1 l ch er0 ih0 ng))
("culvahouse" nil (k ah1 l v ax hh aw1 s))
("culver" nil (k ah1 l v er0))
("culverhouse" nil (k ah1 l v er0 hh aw1 s))
("culwell" nil (k ah1 l w eh1 l))
("cum" nil (k ah1 m))
("cumbee" nil (k ah1 m b iy1))
("cumber" nil (k ah1 m b er0))
("cumberbatch" nil (k ah1 m b er0 b ae1 ch))
("cumberland" nil (k ah1 m b er0 l ax n d))
("cumberledge" nil (k ah1 m b er0 l eh1 jh))
("cumbersome" nil (k ah1 m b er0 s ax m))
("cumbie" nil (k ah1 m b iy0))
("cumbo" nil (k ah1 m b ow0))
("cumby" nil (k ah1 m b iy0))
("cumings" nil (k uw1 m ih0 ng z))
("cumming" nil (k ah1 m ih0 ng))
("cummings" nil (k ah1 m ih0 ng z))
("cummington" nil (k ah1 m ih0 ng t ax n))
("cummins" nil (k ah1 m ih0 n z))
("cummiskey" nil (k ah1 m ih0 s k iy0))
("cumpston" nil (k ah1 m p s t ax n))
("cumpton" nil (k ah1 m p t ax n))
("cumulative" nil (k y uw1 m y ax l ax t ih0 v))
("cumulatively" nil (k y uw1 m y ax l ax t ih0 v l iy0))
("cunard" nil (k y uw1 n er0 d))
("cundall" nil (k ah1 n d ax l))
("cundari" nil (k uw0 n d aa1 r iy0))
("cundiff" nil (k ah1 n d ih0 f))
("cundill" nil (k ah1 n d ih0 l))
("cundy" nil (k ah1 n d iy0))
("cuneiform" nil (k y uw1 n iy0 ax f ao1 r m))
("cuneo" nil (k y uw1 n iy0 ow0))
("cunha" nil (k ah1 n hh ax))
("cunliffe" nil (k ah1 n l ih0 f))
("cunnane" nil (k ah1 n ax n))
("cunneen" nil (k ax n iy1 n))
("cunniff" nil (k ah1 n ih0 f))
("cunning" nil (k ah1 n ih0 ng))
("cunningham" nil (k ah1 n ih0 ng hh ae1 m))
("cunnington" nil (k ah1 n ih0 ng t ax n))
("cuny" nil (k y uw1 n iy0))
("cuoco" nil (k w ow1 k ow0))
("cuomo" nil (k w ow1 m ow0))
("cuong" nil (k w ao1 ng))
("cuozzo" nil (k w ow1 z ow0))
("cup" nil (k ah1 p))
("cupboard" nil (k ah1 b er0 d))
("cupboards" nil (k ah1 b er0 d z))
("cupcake" nil (k ah1 p k ey1 k))
("cupcakes" nil (k ah1 p k ey1 k s))
("cupertino" nil (k uw1 p er0 t iy1 n ow0))
("cupid" nil (k y uw1 p ih0 d))
("cupids" nil (k y uw1 p ih0 d z))
("cupit" nil (k y uw1 p ih0 t))
("cupo" nil (k y uw1 p ow0))
("cupp" nil (k ah1 p))
("cuppett" nil (k ah1 p ih0 t))
("cupples" nil (k ah1 p ax l z))
("cupps" nil (k ah1 p s))
("cuppy" nil (k ah1 p iy0))
("cups" nil (k ah1 p s))
("curable" nil (k y uh1 r ax b ax l))
("curacao" nil (k y uh1 r ax s aw1))
("curate" nil (k y uh1 r ax t))
("curative" nil (k y uh1 r ax t ih0 v))
("curatolo" nil (k uh0 r aa0 t ow1 l ow0))
("curator" nil (k y uh0 r ey1 t er0))
("curatorial" nil (k y uh1 r ax t ao1 r iy0 ax l))
("curators" nil (k y uh1 r ax t er0 z))
("curb" nil (k er1 b))
("curbed" nil (k er1 b d))
("curbelo" nil (k uh0 r b eh1 l ow0))
("curbing" nil (k er1 b ih0 ng))
("curbow" nil (k er1 b ow0))
("curbs" nil (k er1 b z))
("curbside" nil (k er1 b s ay1 d))
("curbstone" nil (k er1 b s t ow1 n))
("curby" nil (k er1 b iy0))
("curci" nil (k uh1 r ch iy0))
("curcio" nil (k uh1 r ch iy0 ow0))
("curcuru" nil (k uh0 r k uh1 r uw0))
("curd" nil (k er1 d))
("cure" nil (k y uh1 r))
("cured" nil (k y uh1 r d))
("cures" nil (k y uh1 r z))
("cureton" nil (k y uh1 r t ax n))
("curfew" nil (k er1 f y uw0))
("curfews" nil (k er1 f y uw0 z))
("curfman" nil (k er1 f m ax n))
("curiale" nil (k uh0 r iy0 aa1 l iy0))
("curie" nil (k y uh0 r iy1))
("curiel" nil (k y uw1 r iy0 l))
("curing" nil (k y uh1 r ih0 ng))
("curington" nil (k y uh1 r ih0 ng t ax n))
("curiosities" nil (k y uh1 r iy0 aa1 s ax t iy0 z))
("curiosity" nil (k y uh1 r iy0 aa1 s ax t iy0))
("curious" nil (k y uh1 r iy0 ax s))
("curiously" nil (k y uh1 r iy0 ax s l iy0))
("curl" nil (k er1 l))
("curle" nil (k ao1 r ax l))
("curled" nil (k er1 l d))
("curlee" nil (k er1 l iy1))
("curless" nil (k er1 l ax s))
("curlett" nil (k er1 l ih0 t))
("curley" nil (k er1 l iy0))
("curlin" nil (k er1 l ih0 n))
("curling" nil (k er1 l ih0 ng))
("curls" nil (k er1 l z))
("curly" nil (k er1 l iy0))
("curmudgeon" nil (k er0 m ah1 jh ih0 n))
("curnow" nil (k er1 n ow0))
("curnutt" nil (k er1 n ax t))
("curnutte" nil (k er0 n ah1 t))
("curragh" nil (k ah1 r ax))
("curran" nil (k er1 ax n))
("curren" nil (k er1 ax n))
("currence" nil (k er1 ax n s))
("currencies" nil (k er1 ax n s iy0 z))
("currency" nil (k er1 ax n s iy0))
("currencywatch" nil (k er1 ax n s iy0 w aa1 ch))
("currens" nil (k er1 ax n z))
("current" nil (k er1 ax n t))
("currently" nil (k er1 ax n t l iy0))
("currents" nil (k er1 ax n t s))
("curreri" nil (k uh0 r eh1 r iy0))
("currey" nil (k er1 iy0))
("curricula" nil (k er0 ih1 k y ax l ax))
("curricular" nil (k er0 ih1 k y ax l er0))
("curriculum" nil (k er0 ih1 k y ax l ax m))
("curriculums" nil (k er0 ih1 k y ax l ax m z))
("currie" nil (k er1 iy0))
("curried" nil (k er1 iy0 d))
("currier" nil (k er1 iy0 er0))
("curries" nil (k er1 iy0 z))
("currin" nil (k ao1 r ih0 n))
("currington" nil (k er1 ih0 ng t ax n))
("curro" nil (k uh1 r ow0))
("curry" nil (k ah1 r iy0))
("curse" nil (k er1 s))
("cursed" nil (k er1 s t))
("curses" nil (k er1 s ih0 z))
("cursing" nil (k er1 s ih0 ng))
("cursor" nil (k er1 s er0))
("cursory" nil (k er1 s er0 iy0))
("curt" nil (k er1 t))
("curtail" nil (k er0 t ey1 l))
("curtailed" nil (k er0 t ey1 l d))
("curtailing" nil (k er0 t ey1 l ih0 ng))
("curtailment" nil (k er0 t ey1 l m ax n t))
("curtailments" nil (k er0 t ey1 l m ax n t s))
("curtails" nil (k er0 t ey1 l z))
("curtain" nil (k er1 t ax n))
("curtains" nil (k er1 t ax n z))
("curti" nil (k uh1 r t iy0))
("curtice" nil (k uh1 r t ih0 s))
("curtin" nil (k er1 t ih0 n))
("curtis" nil (k er1 t ax s))
("curtiss" nil (k er1 t ih0 s))
("curtly" nil (k er1 t l iy0))
("curtner" nil (k er1 t n er0))
("curto" nil (k uh1 r t ow0))
("curtright" nil (k er1 t r ay1 t))
("curts" nil (k er1 t s))
("curtsinger" nil (k er1 t s ih0 n jh er0))
("curve" nil (k er1 v))
("curved" nil (k er1 v d))
("curves" nil (k er1 v z))
("curvin" nil (k er1 v ih0 n))
("curving" nil (k er1 v ih0 ng))
("curvy" nil (k er1 v iy0))
("curzio" nil (k er1 z iy0 ow0))
("cusack" nil (k y uw1 z ax k))
("cusano" nil (k uw0 s aa1 n ow0))
("cush" nil (k ah1 sh))
("cushey" nil (k uh1 sh iy0))
("cushing" nil (k uh1 sh ih0 ng))
("cushion" nil (k uh1 sh ax n))
("cushioned" nil (k uh1 sh ax n d))
("cushioning" nil (k uh1 sh ax n ih0 ng))
("cushions" nil (k uh1 sh ax n z))
("cushitic" nil (k ax sh ih1 t ih0 k))
("cushman" nil (k uh1 sh m ax n))
("cushy" nil (k uh1 sh iy0))
("cusiana" nil (k y uw1 z iy0 ae1 n ax))
("cusic" nil (k y uw1 z ih0 k))
("cusick" nil (k y uw1 z ih0 k))
("cusimano" nil (k uw0 s iy0 m aa1 n ow0))
("cusip" nil (k ah1 s ih0 p))
("cusk" nil (k ah1 s k))
("cusmano" nil (k uw0 s m aa1 n ow0))
("cuso" nil (k y uw1 s ow0))
("cusp" nil (k ah1 s p))
("cuss" nil (k ah1 s))
("cusson" nil (k ah1 s ax n))
("custard" nil (k ah1 s t er0 d))
("custer" nil (k ah1 s t er0))
("custis" nil (k ah1 s t ih0 s))
("custodial" nil (k ax s t ow1 d iy0 ax l))
("custodian" nil (k ax s t ow1 d iy0 ax n))
("custodians" nil (k ax s t ow1 d iy0 ax n z))
("custodio" nil (k uw0 s t ow1 d iy0 ow0))
("custody" nil (k ah1 s t ax d iy0))
("custom" nil (k ah1 s t ax m))
("customarily" nil (k ah1 s t ax m eh1 r ax l iy0))
("customary" nil (k ah1 s t ax m eh1 r iy0))
("customer" nil (k ah1 s t ax m er0))
("customers" nil (k ah1 s t ax m er0 z))
("customize" nil (k ah1 s t ax m ay1 z))
("customized" nil (k ah1 s t ax m ay1 z d))
("customizing" nil (k ah1 s t ax m ay1 z ih0 ng))
("customs" nil (k ah1 s t ax m z))
("cusumano" nil (k uw0 s uw0 m aa1 n ow0))
("cut" nil (k ah1 t))
("cutaia" nil (k uw0 t aa1 y ax))
("cutback" nil (k ah1 t b ae1 k))
("cutbacks" nil (k ah1 t b ae1 k s))
("cutbirth" nil (k ah1 t b er1 th))
("cutchall" nil (k ah1 ch ax l))
("cutcher" nil (k ah1 ch er0))
("cutchin" nil (k ah1 ch ih0 n))
("cutchins" nil (k ah1 ch ih0 n z))
("cute" nil (k y uw1 t))
("cuteness" nil (k y uw1 t n ax s))
("cutesy" nil (k y uw1 t s iy0))
("cuthbert" nil (k ah1 th b er0 t))
("cuthbertson" nil (k ah1 th b er0 t s ax n))
("cuthrell" nil (k ah1 th r ax l))
("cuticle" nil (k y uw1 t ax k ax l))
("cutillo" nil (k y uw1 t ih1 l ow0))
("cutlass" nil (k ah1 t l ax s))
("cutler" nil (k ah1 t l er0))
("cutlery" nil (k ah1 t l er0 iy0))
("cutlip" nil (k ah1 t l ih0 p))
("cutoff" nil (k ah1 t ao1 f))
("cutoffs" nil (k ah1 t ao1 f s))
("cutout" nil (k ah1 t aw1 t))
("cutouts" nil (k ah1 t aw1 t s))
("cutrale" nil (k ah1 t r ey1 l))
("cutrell" nil (k ah1 t r ax l))
("cutrer" nil (k ah1 t r er0))
("cutright" nil (k ah1 t r ay1 t))
("cutrona" nil (k uw0 t r ow1 n ax))
("cutrone" nil (k uw0 t r ow1 n iy0))
("cuts" nil (k ah1 t s))
("cutsforth" nil (k ah1 t s f ao1 r th))
("cutshall" nil (k ah1 ch ax l))
("cutshaw" nil (k ah1 ch ao1))
("cutsinger" nil (k ah1 t s ih0 n jh er0))
("cutter" nil (k ah1 t er0))
("cutters" nil (k ah1 t er0 z))
("cutthroat" nil (k ah1 t th r ow1 t))
("cutting" nil (k ah1 t ih0 ng))
("cuttings" nil (k ah1 t ih0 ng z))
("cuttino" nil (k uw0 t iy1 n ow0))
("cuttlefish" nil (k ah1 t ax l f ih1 sh))
("cutts" nil (k ah1 t s))
("cutty" nil (k ah1 t iy0))
("cutugno" nil (k y uw0 t ah1 n y ow0))
("cutworm" nil (k ah1 t w er1 m))
("cutworms" nil (k ah1 t w er1 m z))
("cuvelier" nil (k y uw1 v l iy0 er0))
("cuyahoga" nil (k ay1 ax hh ow1 g ax))
("cuyler" nil (k ay1 l er0))
("cuzzort" nil (k ah1 z er0 t))
("cwiertnia" nil (k w iy1 r t n iy0 ax))
("cwik" nil (k w ih1 k))
("cwikla" nil (k w ih1 k l ax))
("cwiklinski" nil (k w ih0 k l ih1 n s k iy0))
("cwynar" nil (k w ih1 n er0))
("cxc" nil (s iy1 eh1 k s s iy1))
("cy" nil (s ay1))
("cyacq" nil (s ay1 ae0 k))
("cyanamid" nil (s ay0 ae1 n ax m ih0 d))
("cyanazine" nil (s ay1 ax n ax z iy1 n))
("cyanide" nil (s ay1 ax n ay1 d))
("cyanuric" nil (s ay0 ae1 n er0 ih0 k))
("cyber" nil (s ay1 b er0))
("cybercash" nil (s ay1 b er0 k ae1 sh))
("cyberspace" nil (s ay1 b er0 s p ey1 s))
("cybill" nil (s ay1 b ih1 l))
("cybulski" nil (k ih0 b ah1 l s k iy0))
("cycads" nil (s ay1 k ae0 d z))
("cycare" nil (s ay1 k eh1 r))
("cyclades" nil (s ih1 k l ax d iy1 z))
("cycle" nil (s ay1 k ax l))
("cycled" nil (s ay1 k ax l d))
("cycles" nil (s ay1 k ax l z))
("cyclical" nil (s ay1 k l ih0 k ax l))
("cyclicality" nil (s ih1 k l ih0 k ae1 l ih0 t iy0))
("cyclicals" nil (s ih1 k l ih0 k ax l z))
("cycling" nil (s ay1 k ax l ih0 ng))
("cyclist" nil (s ay1 k ax l ih0 s t))
("cyclohexane" nil (s ay1 k l ax hh eh1 k s ey0 n))
("cyclone" nil (s ih0 k l ow1 n))
("cyclopean" nil (s ay1 k l ax p iy1 ax n))
("cyclops" nil (s ay1 k l ao1 p s))
("cyclosporine" nil (s ih0 k l ao1 s p er0 iy1 n))
("cyclostomes" nil (s ay1 k l ax s t ow1 m z))
("cycolor" nil (s ih1 k ax l er0))
("cydonia" nil (s ih0 d ow1 n iy0 ax))
("cydrome" nil (s ih0 d r ow1 m))
("cygan" nil (s ay1 g ax n))
("cygne" nil (s ih1 g n ax))
("cygnus" nil (s ih1 g n ax s))
("cyhexatin" nil (s ay0 hh eh1 k s ax t ih0 n))
("cylinder" nil (s ih1 l ax n d er0))
("cylinders" nil (s ih1 l ax n d er0 z))
("cylindrical" nil (s ax l ih1 n d r ih0 k ax l))
("cymbal" nil (s ih1 m b ax l))
("cymbals" nil (s ih1 m b ax l z))
("cymrot" nil (s ih1 m r ax t))
("cynara" nil (k ih0 n aa1 r ax))
("cyndi" nil (s ih1 n d iy0))
("cynic" nil (s ih1 n ih0 k))
("cynical" nil (s ih1 n ih0 k ax l))
("cynically" nil (s ih1 n ih0 k ax l iy0))
("cynicism" nil (s ih1 n ih0 s ih1 z ax m))
("cynics" nil (s ih1 n ih0 k s))
("cynric" nil (s ih1 n r ih0 k))
("cynth" nil (s ih1 n th))
("cynthia" nil (s ih1 n th iy0 ax))
("cynthie" nil (s ih1 n th iy0))
("cynwyd" nil (s ih1 n w ih0 d))
("cypert" nil (s ay1 p er0 t))
("cypher" nil (s ay1 f er0))
("cyphers" nil (s ay1 f er0 z))
("cyphert" nil (s ay1 f er0 t))
("cypress" nil (s ay1 p r ax s))
("cyprian" nil (s ih1 p r iy0 ax n))
("cypriot" nil (s ih1 p r iy0 ax t))
("cypriots" nil (s ih1 p r iy0 ax t s))
("cypris" nil (s ay1 p r ih0 s))
("cyprus" nil (s ay1 p r ax s))
("cyr" nil (s ih1 r))
("cyran" nil (k ih1 r ax n))
("cyrano" nil (k iy0 r aa1 n ow0))
("cyrena" nil (k ih0 r iy1 n ax))
("cyril" nil (s ih1 r ax l))
("cyrilla" nil (s ih0 r ih1 l ax))
("cyrillic" nil (s er0 ih1 l ih0 k))
("cyrix" nil (s ay1 r ih1 k s))
("cyrus" nil (s ay1 r ax s))
("cyst" nil (s ih1 s t))
("cystic" nil (s ih1 s t ih0 k))
("cysts" nil (s ih1 s t s))
("cytherea" nil (s ih1 th er0 iy1 ax))
("cytogen" nil (s ay1 t ow0 jh eh0 n))
("cytology" nil (s ay0 t aa1 l ax jh iy0))
("cytomegalovirus" nil (s ay1 t ax m eh1 g ax l ow0 v ay1 r ax s))
("cytoplasm" nil (s ay1 t ax p l ae1 z ax m))
("cytoplasmic" nil (s ay1 t ax p l ae1 z m ih0 k))
("cytotech" nil (s ay1 t ow0 t eh1 k))
("cytotechs" nil (s ay1 t ow0 t eh1 k s))
("cywinski" nil (k ih0 v ih1 n s k iy0))
("czaja" nil (ch ay1 ax))
("czajka" nil (ch ay1 k ax))
("czajkowski" nil (ch ay0 k ao1 f s k iy0))
("czapla" nil (ch aa1 p l ax))
("czaplewski" nil (ch ax p l eh1 f s k iy0))
("czaplicki" nil (ch ax p l ih1 t s k iy0))
("czar" nil (z aa1 r))
("czarist" nil (z aa1 r ih0 s t))
("czarnecki" nil (ch er0 n eh1 t s k iy0))
("czarnik" nil (ch aa1 r n ih0 k))
("czarny" nil (ch aa1 r n iy0))
("czars" nil (z aa1 r z))
("czech" nil (ch eh1 k))
("czechoslovak" nil (ch eh1 k ax s l ow1 v aa0 k))
("czechoslovakia" nil (ch eh1 k ax s l ow0 v aa1 k iy0 ax))
("czechoslovakian" nil (ch eh1 ch ax s l ow0 v aa1 k iy0 ax n))
("czechoslovaks" nil (ch eh1 k ax s l ow1 v aa0 k s))
("czechowski" nil (ch ih0 hh ao1 f s k iy0))
("czechs" nil (ch eh1 k s))
("czekajewski" nil (ch eh1 k ax jh uw1 s k iy0))
("czepiel" nil (ch eh1 p iy0 l))
("czerniak" nil (ch er1 n iy0 ae0 k))
("czerwinski" nil (ch er0 v ih1 n s k iy0))
("czerwonka" nil (ch er0 v aa1 ng k ax))
("czeslaw" nil (ch eh0 s l ao1))
("czyz" nil (ch ih1 z))
("czyzewski" nil (ch ih0 z eh1 f s k iy0))
("d" nil (d iy1))
("da" nil (d aa1))
("daane" nil (d aa1 n))
("dab" nil (d ae1 b))
("dabah" nil (d ae1 b ax))
("dabble" nil (d ae1 b ax l))
("dabbled" nil (d ae1 b ax l d))
("dabbles" nil (d ae1 b ax l z))
("dabbling" nil (d ae1 b ax l ih0 ng))
("dabbs" nil (d ae1 b z))
("dabchick" nil (d ae1 b ch ih0 k))
("dabhol" nil (d ae1 b ow0 l))
("dabkowski" nil (d ax b k ao1 f s k iy0))
("dabney" nil (d ae1 b n iy0))
("dabrowski" nil (d ax b r ao1 f s k iy0))
("dac" nil (d ae1 k))
("dace" nil (d ey1 s))
("dacey" nil (d ey1 s iy0))
("dachshund" nil (d aa1 k s hh uh1 n d))
("dachshunds" nil (d aa1 k s hh uh1 n t s))
("dacia" nil (d ey1 sh iy0 ax))
("dack" nil (d ae1 k))
("dacosta" nil (d aa0 k ow1 s t ax))
("dacquisto" nil (d ae1 k w ih0 s t ow0))
("dacron" nil (d ae1 k r aa1 n))
("dacruz" nil (d aa1 k r uw0 z))
("dacs" nil (d ae1 k s))
("dacunha" nil (d ax k ah1 n hh ax))
("dacus" nil (d ae1 k ih0 s))
("dacy" nil (d ey1 s iy0))
("dad" nil (d ae1 d))
("dada" nil (d aa1 d aa1))
("dadamo" nil (d aa0 d aa1 m ow0))
("daddario" nil (d aa0 d aa1 r iy0 ow0))
("daddies" nil (d ae1 d iy0 z))
("daddona" nil (d aa0 d ow1 n ax))
("daddy" nil (d ae1 d iy0))
("dade" nil (d ey1 d))
("dadeland" nil (d ey1 d l ax n d))
("dadfar" nil (d ae1 d f aa1 r))
("dadisman" nil (d ae1 d ih0 s m ax n))
("dado" nil (d ey1 d ow1))
("dads" nil (d ae1 d z))
("dady" nil (d ey1 d iy0))
("dae" nil (d ey1))
("daedalus" nil (d eh1 d ax l ax s))
("daelim" nil (d ey1 l iy1 m))
("daenzer" nil (d eh1 n z er0))
("daewoo" nil (d ey1 w uw1))
("daffern" nil (d ae1 f er0 n))
("daffin" nil (d ae1 f ih0 n))
("daffodil" nil (d ae1 f ax d ih1 l))
("daffodils" nil (d ae1 f ax d ih1 l z))
("daffron" nil (d ae1 f r ax n))
("daffy" nil (d ae1 f iy0))
("daffynition" nil (d ae1 f ih0 n ih1 sh ax n))
("dafna" nil (d ae1 f n ax))
("dafoe" nil (d ae1 f ow0))
("dafsa" nil (d ae1 f s ax))
("daft" nil (d ae1 f t))
("dag" nil (d ae1 g))
("dagan" nil (d ey1 g ax n))
("dagata" nil (d aa0 g aa1 t ax))
("dagen" nil (d ae1 g ax n))
("dagenais" nil (d ae1 zh ih0 n ey0))
("dagenham" nil (d ae1 g ax n hh ae1 m))
("dagenhart" nil (d ae1 g ax n hh aa1 r t))
("dager" nil (d ey1 g er0))
("dagg" nil (d ae1 g))
("dagger" nil (d ae1 g er0))
("daggers" nil (d ae1 g er0 z))
("daggett" nil (d ae1 g ih0 t))
("daggs" nil (d ae1 g z))
("daggy" nil (d ae1 g iy0))
("dagle" nil (d ey1 g ax l))
("dagley" nil (d ae1 g l iy0))
("dagmar" nil (d ae1 g m aa1 r))
("dagon" nil (d ey1 g ax n))
("dagostino" nil (d aa0 g ow0 s t iy1 n ow0))
("dague" nil (d aa1 g))
("dagwood" nil (d ae1 g w uh1 d))
("daher" nil (d aa1 er0))
("dahill" nil (d aa1 hh ih0 l))
("dahl" nil (d aa1 l))
("dahlberg" nil (d aa1 l b er0 g))
("dahle" nil (d aa1 ax l))
("dahlem" nil (d aa1 l ih0 m))
("dahlen" nil (d aa1 l ax n))
("dahler" nil (d aa1 l er0))
("dahlgren" nil (d ae1 l g r ih0 n))
("dahlheimer" nil (d aa1 l hh ay1 m er0))
("dahlia" nil (d ae1 l y ax))
("dahlin" nil (d aa1 l ih0 n))
("dahlke" nil (d aa1 l k))
("dahlman" nil (d aa1 l m ax n))
("dahlquist" nil (d aa1 l k w ih1 s t))
("dahlstrom" nil (d aa1 l s t r ax m))
("dahm" nil (d ae1 m))
("dahman" nil (d aa1 m ax n))
("dahmen" nil (d aa1 m eh0 n))
("dahmer" nil (d aa1 m er0))
("dahms" nil (d aa1 m z))
("dahn" nil (d ae1 n))
("dahnke" nil (d ae1 ng k))
("dai" nil (d ay1))
("daido" nil (d ey1 d ow0))
("daidone" nil (d ey1 d ow1 n))
("daiei" nil (d ay1 ey1))
("daigle" nil (d ey1 g ax l))
("daigler" nil (d ey1 g l er0))
("daignault" nil (d eh0 g n ow1))
("daigneault" nil (d eh0 g n ow1))
("daigre" nil (d ey1 g er0))
("daigrepont" nil (d ey1 g r ih0 p aa0 n t))
("daihatsu" nil (d ay1 hh ae1 t s uw0))
("daiichi" nil (d ay1 iy1 ch iy0))
("daikin" nil (d ey1 k ih0 n))
("dail" nil (d ey1 l))
("daile" nil (d ey1 l))
("dailey" nil (d ey1 l iy0))
("dailies" nil (d ey1 l iy0 z))
("daily" nil (d ey1 l iy0))
("daimler" nil (d ey1 m l er0))
("daimones" nil (d ey1 m ow1 n z))
("dain" nil (d ey1 n))
("daines" nil (d ey1 n z))
("daini" nil (d ey1 n iy0))
("dainippon" nil (d ey1 n ih0 p aa1 n))
("dains" nil (d ey1 n z))
("dainty" nil (d ey1 n t iy0))
("dairies" nil (d eh1 r iy0 z))
("dairy" nil (d eh1 r iy0))
("dairying" nil (d eh1 r iy0 ih0 ng))
("dairymen" nil (d ey1 r iy0 m ax n))
("dais" nil (d ey1 z))
("daise" nil (d ey1 z))
("daisey" nil (d ey1 s iy0))
("daishowa" nil (d ey1 sh aw1 ax))
("daisies" nil (d ey1 z iy0 z))
("daisy" nil (d ey1 z iy0))
("daiwa" nil (d ey1 w ax))
("dajun" nil (d ey1 jh ax n))
("dakar" nil (d aa0 k aa1 r))
("dake" nil (d ey1 k))
("dakin" nil (d ey1 k ih0 n))
("dakota" nil (d ax k ow1 t ax))
("dakotans" nil (d ax k ow1 t ax n z))
("dakotas" nil (d ax k ow1 t ax z))
("dal" nil (d ae1 l))
("dalafield" nil (d ae1 l ax f iy1 l d))
("dalai" nil (d ae1 l ey1))
("dalal" nil (d ey1 l ax l))
("dalbar" nil (d ae1 l b aa0 r))
("dalbec" nil (d ae1 l b ih0 k))
("dalbert" nil (d ae1 l b er0 t))
("dalbey" nil (d ae1 l b iy0))
("dalby" nil (d ao1 l b iy0))
("dale" nil (d ey1 l))
("daleiden" nil (d ae1 l ay0 d ax n))
("dalen" nil (d ae1 l ax n))
("daleo" nil (d aa1 l iy0 ow0))
("dales" nil (d ey1 l z))
("dalesandro" nil (d aa0 l eh0 s aa1 n d r ow0))
("dalesio" nil (d ax l iy1 s iy0 ow0))
("dalessandro" nil (d aa0 l eh0 s aa1 n d r ow0))
("dalessio" nil (d ax l eh1 s iy0 ow0))
("daleth" nil (d aa1 l eh1 th))
("daley" nil (d ey1 l iy0))
("dalfen" nil (d ae1 l f ax n))
("dalfonso" nil (d ae1 l f aa1 n s ow0))
("dalfort" nil (d ao1 l f er0 t))
("dalgety" nil (d ae1 l g eh1 t iy0))
("dalgleish" nil (d ae1 l g ax l ih0 sh))
("dalhouse" nil (d ao1 l hh aw1 s))
("dali" nil (d aa1 l iy0))
("dalia" nil (d aa1 l y ax))
("dalian" nil (d ey1 l iy0 ax n))
("dalila" nil (d ax l ay1 l ax))
("dalis" nil (d ae1 l ih0 s))
("dalke" nil (d ey1 l k))
("dalkon" nil (d ae1 l k ax n))
("dall" nil (d ao1 l))
("dalla" nil (d ae1 l ax))
("dallaire" nil (d aa1 l eh0 r))
("dallara" nil (d ae1 l aa1 r ax))
("dallas" nil (d ae1 l ax s))
("dalley" nil (d ae1 l iy0))
("dallhold" nil (d ao1 l hh ow1 l d))
("dalliance" nil (d ae1 l iy0 ax n s))
("dallman" nil (d ao1 l m ax n))
("dallmann" nil (d ao1 l m ax n))
("dally" nil (d ae1 l iy0))
("dalman" nil (d ae1 l m ax n))
("dalmatian" nil (d ae0 l m ey1 sh ax n))
("dalmatians" nil (d ae0 l m ey1 sh ax n z))
("dalmo" nil (d ao1 l m ow0))
("dalo" nil (d aa1 l ow0))
("daloia" nil (d aa0 l ow1 y ax))
("daloisio" nil (d aa0 l oy1 s iy0 ow0))
("dalonzo" nil (d ax l aa1 n z ow0))
("dalpe" nil (d ey1 l p))
("dalpiaz" nil (d aa0 l p iy1 aa0 z))
("dalporto" nil (d aa0 l p ao1 r t ow0))
("dalrymple" nil (d ae1 l r ih0 m p ax l))
("dalto" nil (d aa1 l t ow0))
("dalton" nil (d ao1 l t ax n))
("daltons" nil (d ao1 l t ax n z))
("daly" nil (d ey1 l iy0))
("dalzell" nil (d ae1 l z ax l))
("dalziel" nil (d ae1 l z iy1 l))
("dam" nil (d ae1 m))
("dama" nil (d aa1 m ax))
("damage" nil (d ae1 m ax jh))
("damaged" nil (d ae1 m ax jh d))
("damages" nil (d ae1 m ax jh ax z))
("damaging" nil (d ae1 m ih0 jh ih0 ng))
("daman" nil (d ey1 m ax n))
("damaris" nil (d ax m aa1 r ax s))
("damas" nil (d aa1 m ax z))
("damascus" nil (d ax m ae1 s k ax s))
("damask" nil (d ae1 m ax s k))
("damasks" nil (d ae1 m ax s k s))
("damato" nil (d aa0 m aa1 t ow0))
("dambach" nil (d ae1 m b aa1 k))
("dambra" nil (d ae1 m b r ax))
("dambrosia" nil (d aa0 m b r ow1 s iy0 ax))
("dambrosio" nil (d ae1 m b r ow1 s iy0 ow0))
("dame" nil (d ey1 m))
("damelio" nil (d ax m iy1 l iy0 ow0))
("dameron" nil (d aa0 m eh0 r ao1 n))
("dames" nil (d ey1 m z))
("damewood" nil (d ey1 m w uh1 d))
("damgard" nil (d ae1 m g aa1 r d))
("damian" nil (d ey1 m iy0 ax n))
("damiani" nil (d aa0 m iy0 aa1 n iy0))
("damiano" nil (d aa0 m iy0 aa1 n ow0))
("damico" nil (d aa0 m iy1 k ow0))
("damien" nil (d ey1 m iy0 ax n))
("damietta" nil (d ae1 m iy0 eh1 t ax))
("daminozide" nil (d ae1 m ih1 n ax z ay1 d))
("damita" nil (d aa0 m iy1 t ax))
("damitz" nil (d ae1 m ih0 t s))
("damm" nil (d ae1 m))
("dammam" nil (d ae1 m ax m))
("damman" nil (d ae1 m ax n))
("dammann" nil (d ae1 m ax n))
("damme" nil (d ae1 m))
("dammed" nil (d ae1 m d))
("dammen" nil (d ae1 m ax n))
("dammer" nil (d ae1 m er0))
("dammerman" nil (d ae1 m er0 m ax n))
("damn" nil (d ae1 m))
("damnation" nil (d ae0 m n ey1 sh ax n))
("damned" nil (d ae1 m d))
("damning" nil (d ae1 m ih0 ng))
("damocles" nil (d ae1 m ax k l iy1 z))
("damon" nil (d ey1 m ax n))
("damone" nil (d ax m ow1 n))
("damons" nil (d ey1 m ax n z))
("damoose" nil (d ax m uw1 s))
("damore" nil (d ey1 m ao1 r))
("damour" nil (d ax m uh1 r))
("damp" nil (d ae1 m p))
("damped" nil (d ae1 m p t))
("dampen" nil (d ae1 m p ax n))
("dampened" nil (d ae1 m p ax n d))
("dampening" nil (d ae1 m p ax n ih0 ng))
("damper" nil (d ae1 m p er0))
("dampers" nil (d ae1 m p er0 z))
("dampier" nil (d ae1 m p iy0 er0))
("damping" nil (d ae1 m p ih0 ng))
("damps" nil (d ae1 m p s))
("damron" nil (d ae1 m r ax n))
("damrow" nil (d ae1 m r ow1))
("dams" nil (d ae1 m z))
("damsel" nil (d ae1 m z ax l))
("damson" nil (d ae1 m s ax n))
("damuth" nil (d ae1 m uw0 th))
("dan" nil (d ae1 n))
("dana" nil (d ey1 n ax))
("danaher" nil (d ae1 n ax hh er0))
("danahy" nil (d ae1 n ax hh iy0))
("danbury" nil (d ae1 n b er0 iy0))
("danby" nil (d ae1 n b iy0))
("danca" nil (d aa1 ng k ax))
("dancanet" nil (d ae1 ng k ax n eh1 t))
("dance" nil (d ae1 n s))
("danced" nil (d ae1 n s t))
("dancer" nil (d ae1 n s er0))
("dancers" nil (d ae1 n s er0 z))
("dances" nil (d ae1 n s ax z))
("dancey" nil (d ae1 n s iy0))
("dancing" nil (d ae1 n s ih0 ng))
("dancsak" nil (d ae1 n k s ae0 k))
("dancy" nil (d ae1 n s iy0))
("dandelion" nil (d ae1 n d ax l ay1 ax n))
("dandelions" nil (d ae1 n d ax l ay1 ax n z))
("dandeneau" nil (d ae1 n d ih0 n ow0))
("dando" nil (d ae1 n d ow0))
("dandrea" nil (d ae1 n d r iy0 ax))
("dandridge" nil (d ae1 n d r ih0 jh))
("dandruff" nil (d ae1 n d r ax f))
("dandurand" nil (d ae1 n d er0 ax n d))
("dandy" nil (d ae1 n d iy0))
("dandyism" nil (d ae1 n d iy0 ih1 z ax m))
("dane" nil (d ey1 n))
("danek" nil (d ae1 n ih0 k))
("danella" nil (d ax n eh1 l ax))
("danelle" nil (d ax n eh1 l))
("danes" nil (d ey1 n z))
("danese" nil (d aa0 n ey1 z iy0))
("daney" nil (d ey1 n iy0))
("danford" nil (d ae1 n f er0 d))
("danforth" nil (d ae1 n f er0 th))
("dang" nil (d ae1 ng))
("dangel" nil (d ey1 ng g ax l))
("dangelo" nil (d ae1 n jh ax l ow0))
("danger" nil (d ey1 n jh er0))
("dangerfield" nil (d ey1 n jh er0 f iy1 l d))
("dangerous" nil (d ey1 n jh er0 ax s))
("dangerously" nil (d ey1 n jh er0 ax s l iy0))
("dangers" nil (d ey1 n jh er0 z))
("dangle" nil (d ae1 ng g ax l))
("dangled" nil (d ae1 ng g ax l d))
("dangler" nil (d ae1 ng g ax l er0))
("dangles" nil (d ae1 ng g ax l z))
("dangling" nil (d ae1 ng g ax l ih0 ng))
("dango" nil (d ae1 ng g ow0))
("dani" nil (d aa1 n iy0))
("danica" nil (d ae1 n ih0 k ax))
("daniel" nil (d ae1 n y ax l))
("daniela" nil (d aa0 n y eh1 l ax))
("daniele" nil (d aa0 n y eh1 l ey0))
("daniell" nil (d ae1 n iy0 l))
("danielle" nil (d ae1 n iy0 eh1 l))
("daniello" nil (d aa0 n iy0 eh1 l ow0))
("daniels" nil (d ae1 n y ax l z))
("danielsen" nil (d ae1 n iy0 l s ax n))
("danielski" nil (d ax n iy1 l s k iy0))
("danielson" nil (d ae1 n y ax l s ax n))
("daniloff" nil (d ae1 n ax l ao0 f))
("danis" nil (d aa1 n iy0 z))
("danish" nil (d ey1 n ih0 sh))
("danju" nil (d ae1 n jh uw0))
("danjub" nil (d ae1 n jh uw0 b))
("danjube" nil (d ae1 n jh uw0 b))
("dank" nil (d ae1 ng k))
("danker" nil (d ae1 ng k er0))
("dankert" nil (d ae1 ng k er0 t))
("dankner" nil (d ae1 ng k n er0))
("danko" nil (d ae1 ng k ow0))
("danks" nil (d ae1 ng k s))
("danley" nil (d ae1 n l iy0))
("danly" nil (d ae1 n l iy0))
("dann" nil (d ae1 n))
("danna" nil (d ae1 n ax))
("dannelly" nil (d ae1 n ax l iy0))
("dannels" nil (d ae1 n ax l z))
("dannemeyer" nil (d ae1 n ax m ay1 r))
("dannemiller" nil (d ae1 n ax m ih1 l er0))
("dannen" nil (d ae1 n ax n))
("dannenberg" nil (d ae1 n ax n b er0 g))
("danner" nil (d ae1 n er0))
("danners" nil (d ae1 n er0 z))
("dannie" nil (d ae1 n iy0))
("dannunzio" nil (d aa0 n uw1 n z iy0 ow0))
("danny" nil (d ae1 n iy0))
("dano" nil (d aa1 n ow0))
("danos" nil (d ey1 n ow0 z))
("danowski" nil (d ax n ao1 f s k iy0))
("dansby" nil (d ae1 n s b iy0))
("danser" nil (d ae1 n s er0))
("dansereau" nil (d ae1 n s er0 ow1))
("dansforth" nil (d ae1 n s f ao1 r th))
("dansie" nil (d ae1 n s iy0))
("danske" nil (d ae1 n s k))
("danskin" nil (d ae1 n s k ih0 n))
("dansky" nil (d ae1 n s k iy0))
("danson" nil (d ae1 n s ax n))
("dant" nil (d ae1 n t))
("dante" nil (d aa1 n t ey0))
("dantin" nil (d ae1 n t ih0 n))
("danton" nil (d ae1 n t ax n))
("dantoni" nil (d aa0 n t ow1 n iy0))
("dantonio" nil (d ae1 n t ow1 n iy0 ow0))
("dantuono" nil (d aa0 n t w ow1 n ow0))
("dantzler" nil (d ae1 n t s l er0))
("danube" nil (d ae1 n y uw0 b))
("danvers" nil (d ae1 n v er0 z))
("danville" nil (d ae1 n v ih1 l))
("danylyszyn" nil (d ae1 n ih0 l iy1 z ih0 n))
("danz" nil (d ae1 n z))
("danza" nil (d ae1 n z ax))
("danzer" nil (d ae1 n z er0))
("danzig" nil (d ae1 n z ih0 g))
("danziger" nil (d ae1 n z ih0 g er0))
("danzy" nil (d ae1 n z iy0))
("dao" nil (d aw1))
("daoud" nil (d aw1 d))
("daoust" nil (d aw1 s t))
("daphne" nil (d ae1 f n iy0))
("daphnis" nil (d ae1 f n ax s))
("dapice" nil (d aa1 p ih0 s))
("dapolito" nil (d aa0 p ow0 l iy1 t ow0))
("daponte" nil (d aa0 p ow1 n t ey0))
("dapozzo" nil (d ax p aa1 z ow1))
("dapp" nil (d ae1 p))
("dapper" nil (d ae1 p er0))
("dapuzzo" nil (d ax p ah1 z ow0))
("daquila" nil (d aa0 k w iy1 l ax))
("dar" nil (d aa1 r))
("dara" nil (d ae1 r ax))
("darak" nil (d eh1 r ae0 k))
("darblay" nil (d aa1 r b l ey0))
("darbonne" nil (d aa1 r b ax n))
("darby" nil (d aa1 r b iy0))
("darbyshire" nil (d aa1 r b ih0 sh ay1 r))
("darcangelo" nil (d aa0 r k aa0 ng g eh1 l ow0))
("darcey" nil (d aa1 r s iy0))
("darcie" nil (d aa1 r k iy0))
("darco" nil (d aa1 r k ow0))
("darcy" nil (d aa1 r s iy0))
("dardar" nil (d aa0 r d aa1 r))
("darden" nil (d aa1 r d ax n))
("dardis" nil (d aa1 r d ih0 s))
("dare" nil (d eh1 r))
("dared" nil (d eh1 r d))
("daredevil" nil (d eh1 r d eh1 v ax l))
("daremblum" nil (d eh1 r ax m b l uw1 m))
("darensbourg" nil (d ae1 r ih0 n s b er0 g))
("dares" nil (d eh1 r z))
("darga" nil (d aa1 r g ax))
("dargan" nil (d aa1 r g ax n))
("dargis" nil (d aa1 r g ih0 s))
("daria" nil (d aa1 r iy0 ax))
("darice" nil (d aa1 r ih0 s))
("darien" nil (d eh1 r iy0 ax n))
("darienzo" nil (d aa0 r iy1 n z ow0))
("darin" nil (d ae1 r ih0 n))
("daring" nil (d eh1 r ih0 ng))
("dario" nil (d eh1 r iy0 ow1))
("darity" nil (d ae1 r ih0 t iy0))
("darius" nil (d er0 ay1 ax s))
("darjeeling" nil (d aa1 r jh iy1 l ih0 ng))
("dark" nil (d aa1 r k))
("darke" nil (d aa1 r k))
("darken" nil (d aa1 r k ax n))
("darkened" nil (d aa1 r k ax n d))
("darkening" nil (d aa1 r k ax n ih0 ng))
("darker" nil (d aa1 r k er0))
("darkest" nil (d aa1 r k ax s t))
("darkie" nil (d aa1 r k iy0))
("darkly" nil (d aa1 r k l iy0))
("darkness" nil (d aa1 r k n ax s))
("darkroom" nil (d aa1 r k r uw1 m))
("darland" nil (d aa1 r l ax n d))
("darlene" nil (d aa1 r l iy1 n))
("darley" nil (d aa1 r l iy0))
("darlin" nil (d aa1 r l ih0 n))
("darline" nil (d aa1 r l ay1 n))
("darling" nil (d aa1 r l ih0 ng))
("darlings" nil (d aa1 r l ih0 ng z))
("darlington" nil (d aa1 r l ih0 ng t ax n))
("darman" nil (d aa1 r m ax n))
("darmstadt" nil (d aa1 r m s t ae1 t))
("darn" nil (d aa1 r n))
("darnall" nil (d aa1 r n ax l))
("darned" nil (d aa1 r n d))
("darnel" nil (d aa1 r n ax l))
("darnell" nil (d aa0 r n eh1 l))
("darner" nil (d aa1 r n er0))
("darnold" nil (d aa1 r n ow1 l d))
("darns" nil (d aa1 r n z))
("darocha" nil (d aa0 r ow1 k ax))
("darosa" nil (d aa0 r ow1 s ax))
("darpino" nil (d aa0 r p iy1 n ow0))
("darr" nil (d eh1 r))
("darragh" nil (d eh1 r ax))
("darrah" nil (d ae1 r ax))
("darrell" nil (d eh1 r ax l))
("darrelle" nil (d er0 eh1 l))
("darren" nil (d aa1 r ax n))
("darrick" nil (d ae1 r ih0 k))
("darrigo" nil (d aa1 r iy1 g ow0))
("darrington" nil (d ae1 r ih0 ng t ax n))
("darroch" nil (d ae1 r ax k))
("darrough" nil (d ae1 r aw0))
("darrow" nil (d eh1 r ow0))
("darryl" nil (d eh1 r ax l))
("darsey" nil (d aa1 r s iy0))
("darst" nil (d aa1 r s t))
("darsy" nil (d aa1 r s iy0))
("dart" nil (d aa1 r t))
("dartboard" nil (d aa1 r t b ao1 r d))
("darted" nil (d aa1 r t ih0 d))
("darter" nil (d aa1 r t er0))
("dartez" nil (d aa0 r t eh1 z))
("darth" nil (d aa1 r th))
("darting" nil (d aa1 r t ih0 ng))
("dartmouth" nil (d aa1 r t m ax th))
("darton" nil (d aa1 r t ax n))
("darts" nil (d aa1 r t s))
("dartt" nil (d aa1 r t))
("darty" nil (d aa1 r t iy0))
("darville" nil (d aa1 r v ih1 l))
("darwin" nil (d aa1 r w ih0 n))
("darwinian" nil (d aa1 r w ih1 n iy0 ax n))
("darwinism" nil (d aa1 r w ih0 n ih1 z ax m))
("darwish" nil (d aa1 r w ih0 sh))
("daryl" nil (d eh1 r ax l))
("das" nil (d ae1 s))
("dasa" nil (d aa1 s ax))
("dasaro" nil (d aa0 s aa1 r ow0))
("dasburg" nil (d ae1 s b er0 g))
("dascenzo" nil (d aa0 s ch eh1 n z ow0))
("dasch" nil (d ae1 sh))
("daschle" nil (d ae1 sh l iy0))
("dascoli" nil (d aa0 s k ow1 l iy0))
("dase" nil (d ey1 z))
("daseke" nil (d ey1 s ax k iy0))
("dash" nil (d ae1 sh))
("dashboard" nil (d ae1 sh b ao1 r d))
("dashboards" nil (d ae1 sh b ao1 r d z))
("dashed" nil (d ae1 sh t))
("dasher" nil (d ae1 sh er0))
("dashes" nil (d ae1 sh ih0 z))
("dashiell" nil (d ae1 sh iy0 l))
("dashing" nil (d ae1 sh ih0 ng))
("dashnaw" nil (d ae1 sh n ao0))
("dashner" nil (d ae1 sh n er0))
("dashwood" nil (d ae1 sh w uh1 d))
("dasilva" nil (d ax s ih1 l v ax))
("daspin" nil (d ae1 s p ih0 n))
("daspit" nil (d ae1 s p ih0 t))
("dass" nil (d ae1 s))
("dassault" nil (d ae1 s ao0 l t))
("dassler" nil (d ae1 s l er0))
("dassow" nil (d ae1 s ow0))
("dastardly" nil (d ae1 s t er0 d l iy0))
("dat" nil (d ae1 t))
("data" nil (d ey1 t ax))
("database" nil (d ey1 t ax b ey1 s))
("databases" nil (d ey1 t ax b ey1 s ih0 z))
("datacard" nil (d ey1 t ax k aa1 r d))
("datacomm" nil (d ey1 t ax k aa1 m))
("datacomp" nil (d ey1 t ax k aa1 m p))
("datacopy" nil (d ey1 t ax k aa1 p iy0))
("datagraphix" nil (d ey1 t ax g r ae1 f ih0 k s))
("datametrics" nil (d ey1 t ax m eh1 t r ih0 k s))
("datapoint" nil (d ey1 t ax p oy1 n t))
("datapower" nil (d ey1 t ax p aw1 r))
("dataproducts" nil (d ey1 t ax p r aa1 d ax k t s))
("dataquest" nil (d ey1 t ax k w eh1 s t))
("dataram" nil (d ey1 t er0 ae1 m))
("datarex" nil (d ey1 t er0 eh1 k s))
("datas" nil (d ey1 t ax z))
("date" nil (d ey1 t))
("dated" nil (d ey1 t ih0 d))
("datek" nil (d ae1 t eh0 k))
("dateline" nil (d ey1 t l ay1 n))
("datelines" nil (d ey1 t l ay1 n z))
("dates" nil (d ey1 t s))
("datext" nil (d ae1 t eh1 k s t))
("dating" nil (d ey1 t ih0 ng))
("dato" nil (d aa1 t ow0))
("datron" nil (d ae1 t r ax n))
("dats" nil (d ae1 t s))
("datsun" nil (d ae1 t s ax n))
("dattilio" nil (d aa0 t iy1 l iy0 ow0))
("dattilo" nil (d aa0 t iy1 l ow0))
("datuk" nil (d aa1 t uw1 k))
("datum" nil (d ae1 t ax m))
("datura" nil (d ax t uh1 r ax))
("datz" nil (d ae1 t s))
("dau" nil (d ow1))
("daub" nil (d ao1 b))
("daube" nil (d ao1 b))
("daubed" nil (d ao1 b d))
("daubenspeck" nil (d aw1 b ih0 n s p ih0 k))
("dauber" nil (d aw1 b er0))
("daubert" nil (d aw1 b er0 t))
("dauch" nil (d aw1 ch))
("daudelin" nil (d ow1 d ih0 l ae0 n))
("dauenhauer" nil (d aw1 ax n hh aw0 er0))
("dauer" nil (d aw1 er0))
("daughdrill" nil (d ao1 d r ih0 l))
("daughenbaugh" nil (d ao0 eh1 n b ao0))
("daugherty" nil (d aa1 k er0 t iy0))
("daughety" nil (d ao1 ih0 t iy0))
("daughney" nil (d ao1 n iy0))
("daughter" nil (d ao1 t er0))
("daughters" nil (d ao1 t er0 z))
("daughtery" nil (d ao1 t er0 iy0))
("daughton" nil (d ao1 t ax n))
("daughtrey" nil (d ao1 t r iy0))
("daughtridge" nil (d ao1 t r ih0 jh))
("daughtry" nil (d ao1 t r iy0))
("daul" nil (d ao1 l))
("dault" nil (d ao1 l t))
("daulton" nil (d ao1 l t ax n))
("daum" nil (d ao1 m))
("daun" nil (d ao1 n))
("daunted" nil (d ao1 n t ih0 d))
("daunting" nil (d ao1 n t ih0 ng))
("dauphin" nil (d aw1 f ih0 n))
("dauphinais" nil (d ow1 f ih0 n ey0))
("dauphinee" nil (d ao0 f ih0 n iy1))
("dauria" nil (d ao1 r iy0 ax))
("daus" nil (d ao1 z))
("dauster" nil (d aw1 s t er0))
("dauterive" nil (d ow1 t er0 ih0 v))
("dauzat" nil (d aw1 z ax t))
("davalos" nil (d aa0 v aa1 l ow0 z))
("davanzo" nil (d ax v ae1 n z ow0))
("davao" nil (d ax v ow1))
("davault" nil (d ax v ow1))
("davco" nil (d ae1 v k ow0))
("dave" nil (d ey1 v))
("davee" nil (d ae1 v iy0))
("davenport" nil (d ae1 v ax n p ao1 r t))
("davern" nil (d ae1 v er0 n))
("daversa" nil (d aa0 v eh1 r s ax))
("daves" nil (d ey1 v z))
("davey" nil (d ey1 v iy0))
("davi" nil (d aa1 v iy0))
("davia" nil (d aa1 v iy0 ax))
("david" nil (d ey1 v ih0 d))
("davida" nil (d aa0 v iy1 d ax))
("davide" nil (d ah1 v iy1 d ey1))
("davidge" nil (d ae1 v ih0 jh))
("davidian" nil (d ax v ih1 d iy0 ax n))
("davidians" nil (d ax v ih1 d iy0 ax n z))
("davidoff" nil (d ey1 v ih0 d ao0 f))
("davidow" nil (d ae1 v ih0 d ow0))
("davids" nil (d ey1 v ih0 d z))
("davidson" nil (d ey1 v ih0 d s ax n))
("davie" nil (d ey1 v iy0))
("davies" nil (d ey1 v iy0 z))
("davignon" nil (d aa0 v iy0 g n ao1 n))
("davila" nil (d ax v ih1 l ax))
("davilla" nil (d ax v ih1 l ax))
("davin" nil (d ae1 v ih0 n))
("davina" nil (d aa0 v iy1 n ax))
("davino" nil (d aa0 v iy1 n ow0))
("davis" nil (d ey1 v ax s))
("davison" nil (d ey1 v ih0 s ax n))
("davisson" nil (d ae1 v ih0 s ax n))
("davitt" nil (d ax v ih1 t))
("davlin" nil (d ae1 v l ih0 n))
("davoli" nil (d aa0 v ow1 l iy0))
("davos" nil (d aa1 v ow0 s))
("davox" nil (d ae1 v aa0 k s))
("davy" nil (d ey1 v iy0))
("davydov" nil (d ey1 v ih0 d ao1 v))
("daw" nil (d ao1))
("dawdle" nil (d ao1 d ax l))
("dawdling" nil (d ao1 d l ih0 ng))
("dawdy" nil (d ao1 d iy0))
("dawe" nil (d ao1))
("dawes" nil (d ao1 z))
("dawit" nil (d ae1 w ih0 t))
("dawkins" nil (d ao1 k ih0 n z))
("dawley" nil (d ao1 l iy0))
("dawn" nil (d ao1 n))
("dawned" nil (d ao1 n d))
("dawning" nil (d ao1 n ih0 ng))
("dawns" nil (d ao1 n z))
("daws" nil (d ao1 z))
("dawsey" nil (d ao1 s iy0))
("dawson" nil (d ao1 s ax n))
("dax" nil (d ae1 k s))
("daxor" nil (d ae1 k s er0))
("day" nil (d ey1))
("daya" nil (d ay1 ax))
("dayan" nil (d ey1 ax n))
("daybreak" nil (d ey1 b r ey1 k))
("daycare" nil (d ey1 k eh1 r))
("dayco" nil (d ey1 k ow0))
("daydream" nil (d ey1 d r iy1 m))
("daydreamed" nil (d ey1 d r iy1 m d))
("daye" nil (d ey1))
("dayhoff" nil (d ey1 hh ao0 f))
("dayhuff" nil (d ey1 hh ah1 f))
("dayle" nil (d ey1 l))
("dayley" nil (d ey1 l iy0))
("daylight" nil (d ey1 l ay1 t))
("daylights" nil (d ey1 l ay1 t s))
("daylong" nil (d ey1 l ao1 ng))
("daynard" nil (d ey1 n er0 d))
("dayne" nil (d ey1 n))
("days" nil (d ey1 z))
("daytime" nil (d ey1 t ay1 m))
("daytimes" nil (d ey1 t ay1 m z))
("dayton" nil (d ey1 t ax n))
("daytona" nil (d ey0 t ow1 n ax))
("daytop" nil (d ey1 t aa1 p))
("daywalt" nil (d ey1 w ax l t))
("daze" nil (d ey1 z))
("dazed" nil (d ey1 z d))
("dazey" nil (d ey1 z iy0))
("dazs" nil (d aa1 s))
("dazzle" nil (d ae1 z ax l))
("dazzled" nil (d ae1 z ax l d))
("dazzling" nil (d ae1 z ax l ih0 ng))
("dazzo" nil (d ae1 z ow0))
("dbase" nil (d iy1 b ey1 s))
("ddt" nil (d iy1 d iy1 t iy1))
("de" nil (d iy1))
("dea" nil (d iy1))
("deacon" nil (d iy1 k ax n))
("deaconess" nil (d iy1 k ax n ax s))
("deacons" nil (d iy1 k ax n z))
("deactivated" nil (d iy1 ae1 k t ih0 v ey1 t ih0 d))
("dead" nil (d eh1 d))
("deadbeat" nil (d eh1 d b iy1 t))
("deadbeats" nil (d eh1 d b iy1 t s))
("deadening" nil (d eh1 d ax n ih0 ng))
("deader" nil (d eh1 d er0))
("deadliest" nil (d eh1 d l iy0 ax s t))
("deadline" nil (d eh1 d l ay1 n))
("deadlines" nil (d eh1 d l ay1 n z))
("deadliness" nil (d eh1 d l iy0 n ax s))
("deadlock" nil (d eh1 d l aa1 k))
("deadlocked" nil (d eh1 d l aa1 k t))
("deadlocks" nil (d eh1 d l aa1 k s))
("deadly" nil (d eh1 d l iy0))
("deadpan" nil (d eh1 d p ae1 n))
("deadweight" nil (d eh1 d w ey1 t))
("deadwood" nil (d eh1 d w uh1 d))
("deadwyler" nil (d eh1 d w ay1 l er0))
("deady" nil (d eh1 d iy0))
("deaf" nil (d eh1 f))
("deafening" nil (d eh1 f ax n ih0 ng))
("deafness" nil (d eh1 f n ax s))
("deahl" nil (d iy1 l))
("deak" nil (d iy1 k))
("deakin" nil (d iy1 k ih0 n))
("deakins" nil (d iy1 k ih0 n z))
("deal" nil (d iy1 l))
("dealba" nil (d iy1 ae1 l b ax))
("dealer" nil (d iy1 l er0))
("dealerline" nil (d iy1 l er0 l ay1 n))
("dealers" nil (d iy1 l er0 z))
("dealership" nil (d iy1 l er0 sh ih1 p))
("dealerships" nil (d iy1 l er0 sh ih1 p s))
("dealey" nil (d iy1 l iy0))
("dealfish" nil (d iy1 l f ih1 sh))
("dealing" nil (d iy1 l ih0 ng))
("dealings" nil (d iy1 l ih0 ng z))
("dealmaker" nil (d iy1 l m ey1 k er0))
("dealmakers" nil (d iy1 l m ey1 k er0 z))
("dealmaking" nil (d iy1 l m ey1 k ih0 ng))
("dealmeida" nil (d ax l m iy1 d ax))
("deals" nil (d iy1 l z))
("dealt" nil (d eh1 l t))
("dealy" nil (d iy1 l iy0))
("deam" nil (d iy1 m))
("deamer" nil (d iy1 m er0))
("dean" nil (d iy1 n))
("deana" nil (d iy1 ae1 n ax))
("deanda" nil (d iy1 ae1 n d ax))
("deandrade" nil (d ax n d r aa1 d iy0))
("deandrea" nil (d ae1 d r iy0 ax))
("deane" nil (d iy1 n))
("deaner" nil (d iy1 n er0))
("deangelis" nil (d iy0 ae1 n jh ax l ax s))
("deangelo" nil (d ax ng g eh1 l ow0))
("deanna" nil (d iy1 ae1 n ax))
("deans" nil (d iy1 n z))
("dear" nil (d ih1 r))
("dearborn" nil (d ih1 r b ao1 r n))
("dearden" nil (d ih1 r d ax n))
("deardorff" nil (d ih1 r d ao1 r f))
("deardourff" nil (d ih1 r d ao1 r f))
("dearest" nil (d ih1 r ax s t))
("dearing" nil (d ih1 r ih0 ng))
("dearinger" nil (d ih1 r ih0 ng er0))
("dearly" nil (d ih1 r l iy0))
("dearman" nil (d ih1 r m ax n))
("dearmas" nil (d er1 m ax z))
("dearment" nil (d ih1 r m ax n t))
("dearmon" nil (d er1 m ax n))
("dearmond" nil (d er1 m ax n d))
("dearth" nil (d er1 th))
("deary" nil (d ih1 r iy0))
("deas" nil (d iy1 z))
("dease" nil (d iy1 s))
("deason" nil (d iy1 z ax n))
("deasy" nil (d iy1 s iy0))
("deater" nil (d iy1 t er0))
("death" nil (d eh1 th))
("deathbed" nil (d eh1 th b eh1 d))
("deatherage" nil (d eh1 th er0 ih0 jh))
("deathly" nil (d eh1 th l iy0))
("deaths" nil (d eh1 th s))
("deathwatch" nil (d eh1 th w aa1 ch))
("deatley" nil (d iy1 t l iy0))
("deaton" nil (d iy1 t ax n))
("deatrick" nil (d iy1 t r ih0 k))
("deats" nil (d iy1 t s))
("deaver" nil (d iy1 v er0))
("deavers" nil (d iy1 v er0 z))
("deavila" nil (d ax v iy1 l ax))
("deb" nil (d eh1 b))
("debacker" nil (d iy1 b ae0 k er0))
("debacle" nil (d ax b aa1 k ax l))
("debacles" nil (d ey0 b aa1 k ax l z))
("debakey" nil (d ih0 b ey1 k iy0))
("debarment" nil (d ih0 b aa1 r m ax n t))
("debarr" nil (d ih0 b ae1 r))
("debarros" nil (d ey0 b aa1 r ow0 z))
("debartolo" nil (d ih0 b aa0 r t ow1 l ow0))
("debartolos" nil (d ax b aa1 r t ax l ow0 z))
("debase" nil (d ax b ey1 s))
("debased" nil (d ax b ey1 s t))
("debasement" nil (d ax b ey1 s m ax n t))
("debasing" nil (d ih0 b ey1 s ih0 ng))
("debatable" nil (d ax b ey1 t ax b ax l))
("debate" nil (d ax b ey1 t))
("debated" nil (d ax b ey1 t ih0 d))
("debater" nil (d ax b ey1 t er0))
("debates" nil (d ax b ey1 t s))
("debating" nil (d ax b ey1 t ih0 ng))
("debauche" nil (d eh1 b aw0 k))
("debauchery" nil (d ax b ao1 ch er0 iy0))
("debaun" nil (d eh1 b aw0 n))
("debbie" nil (d eh1 b iy0))
("debby" nil (d eh1 b iy0))
("debeer" nil (d eh1 b ih0 r))
("debeers" nil (d ih0 b ih1 r z))
("debelak" nil (d eh1 b ih0 l ax k))
("debell" nil (d iy1 b eh0 l))
("debella" nil (d ih0 b eh1 l ax))
("debellis" nil (d eh1 b ih0 l ih0 s))
("debello" nil (d ih0 b eh1 l ow0))
("debenedetto" nil (d ih0 b ih0 n ax d eh1 t ow0))
("debenedictis" nil (d eh1 b ih0 n ax d ih0 k t ih0 s))
("debenture" nil (d ax b eh1 n ch er0))
("debentures" nil (d ax b eh1 n ch er0 z))
("debernardi" nil (d ih0 b er0 n aa1 r d iy0))
("deberry" nil (d iy1 b eh0 r iy0))
("debes" nil (d iy1 b z))
("debevoise" nil (d eh1 b eh0 v w aa1 z))
("debi" nil (d eh1 b iy0))
("debiase" nil (d ih0 b iy0 aa1 s iy0))
("debilitated" nil (d ax b ih1 l ax t ey1 t ih0 d))
("debilitating" nil (d ax b ih1 l ax t ey1 t ih0 ng))
("debility" nil (d ax b ih1 l ax t iy0))
("debit" nil (d eh1 b ih0 t))
("deblanc" nil (d ih0 b l ae1 ng k))
("deblasio" nil (d ih0 b l aa1 s iy0 ow0))
("deblock" nil (d eh1 b l ax k))
("deblois" nil (d eh1 b l uw1))
("debnam" nil (d eh1 b n ax m))
("debo" nil (d iy1 b ow0))
("deboard" nil (d iy1 b ao0 r d))
("deboe" nil (d ih0 b ow1))
("deboer" nil (d iy1 b ow0 er0))
("debold" nil (d eh1 b ow0 l d))
("debolt" nil (d eh1 b ow0 l t))
("debona" nil (d ih0 b ow1 n ax))
("debonair" nil (d eh1 b ax n eh1 r))
("debonis" nil (d eh1 b ax n ih0 s))
("debono" nil (d ih0 b ow1 n ow0))
("debor" nil (d eh1 b ao0 r))
("debora" nil (d eh1 b r ax))
("deborah" nil (d eh1 b er0 ax))
("debord" nil (d ih0 b ao1 r d))
("deborde" nil (d ih0 b ao1 r d))
("debose" nil (d eh1 b ax s))
("debow" nil (d eh1 b ow0))
("deboy" nil (d ih0 b oy1))
("debra" nil (d eh1 b r ax))
("debriefed" nil (d ih0 b r iy1 f t))
("debriefing" nil (d ih0 b r iy1 f ih0 ng))
("debris" nil (d ax b r iy1))
("debrosse" nil (d eh1 b r ax s))
("debruhl" nil (d eh1 b r ax l))
("debruin" nil (d eh1 b r uw0 ih0 n))
("debruler" nil (d eh1 b r uw0 l er0))
("debruyn" nil (d eh1 b r ay0 n))
("debruyne" nil (d eh1 b r ay0 n))
("debs" nil (d eh1 b z))
("debt" nil (d eh1 t))
("debtholder" nil (d eh1 t hh ow1 l d er0))
("debtholders" nil (d eh1 t hh ow1 l d er0 z))
("debtor" nil (d eh1 t er0))
("debtors" nil (d eh1 t er0 z))
("debts" nil (d eh1 t s))
("debugging" nil (d iy0 b ah1 g ih0 ng))
("debuhr" nil (d eh1 b uh0 r))
("debunk" nil (d ih0 b ah1 ng k))
("debunked" nil (d ih0 b ah1 ng k t))
("debunking" nil (d ih0 b ah1 ng k ih0 ng))
("debus" nil (d eh1 b ih0 s))
("debusk" nil (d eh1 b ax s k))
("debussy" nil (d ih0 b ah1 s iy0))
("debut" nil (d ey0 b y uw1))
("debutante" nil (d eh1 b y ax t aa1 n t))
("debutantes" nil (d eh1 b y ax t aa1 n t s))
("debuted" nil (d ey0 b y uw1 d))
("debuts" nil (d ey1 b y uw0 z))
("dec" nil (d eh1 k))
("decade" nil (d eh0 k ey1 d))
("decadence" nil (d eh1 k ax d ax n s))
("decadent" nil (d eh1 k ax d ax n t))
("decades" nil (d eh0 k ey1 d z))
("decaffeinate" nil (d iy0 k ae1 f ax n ey1 t))
("decaffeinated" nil (d iy0 k ae1 f ax n ey1 t ih0 d))
("decaffeinating" nil (d iy0 k ae1 f ax n ey1 t ih0 ng))
("decaire" nil (d iy1 k eh0 r))
("decals" nil (d iy1 k ae1 l z))
("decamillo" nil (d eh1 k ax m ih1 l ow0))
("decamp" nil (d ax k ae1 m p))
("decamped" nil (d iy0 k ae1 m p t))
("decandia" nil (d ih0 k aa1 n d iy0 ax))
("decanting" nil (d ax k ae1 n t ih0 ng))
("decapitated" nil (d iy0 k ae1 p ax t ey1 t ih0 d))
("decapitation" nil (d ih0 k ae1 p ih0 t ey1 sh ax n))
("decaprio" nil (d ih0 k aa1 p r iy0 ow0))
("decapua" nil (d ih0 k aa0 p uw1 ax))
("decarli" nil (d ih0 k aa1 r l iy0))
("decarlo" nil (d ih0 k aa1 r l ow0))
("decaro" nil (d ih0 k aa1 r ow0))
("decarolis" nil (d eh1 k er0 ax l ih0 s))
("decastro" nil (d ih0 k ae1 s t r ow0))
("decato" nil (d ih0 k aa1 t ow0))
("decatur" nil (d ih0 k ey1 t er0))
("decay" nil (d ax k ey1))
("decayed" nil (d ax k ey1 d))
("decaying" nil (d ax k ey1 ih0 ng))
("decays" nil (d ax k ey1 z))
("decca" nil (d eh1 k ax))
("deceased" nil (d ih0 s iy1 s t))
("dececco" nil (d ih0 ch eh1 k ow0))
("decedent" nil (d ih0 s iy1 d ax n t))
("deceit" nil (d ax s iy1 t))
("deceitful" nil (d ax s iy1 t f ax l))
("deceive" nil (d ih0 s iy1 v))
("deceived" nil (d ih0 s iy1 v d))
("deceiving" nil (d ih0 s iy1 v ih0 ng))
("decelerate" nil (d ih0 s eh1 l er0 ey1 t))
("decelerated" nil (d ih0 s eh1 l er0 ey1 t ih0 d))
("decelerating" nil (d ih0 s eh1 l er0 ey1 t ih0 ng))
("deceleration" nil (d ih0 s eh1 l er0 ey1 sh ax n))
("decelle" nil (d ih0 s eh1 l))
("decelles" nil (d eh1 s ax l z))
("december" nil (d ih0 s eh1 m b er0))
("decency" nil (d iy1 s ax n s iy0))
("decennial" nil (d ax s eh1 n iy0 ax l))
("decent" nil (d iy1 s ax n t))
("decently" nil (d iy1 s ax n t l iy0))
("decentralization" nil (d ih0 s eh1 n t r ax l ih0 z ey1 sh ax n))
("decentralize" nil (d ih0 s eh1 n t r ax l ay1 z))
("decentralized" nil (d ih0 s eh1 n t r ax l ay1 z d))
("decentralizing" nil (d ih0 s eh1 n t r ax l ay1 z ih0 ng))
("deception" nil (d ih0 s eh1 p sh ax n))
("deceptions" nil (d ih0 s eh1 p sh ax n z))
("deceptive" nil (d ih0 s eh1 p t ih0 v))
("deceptively" nil (d ih0 s eh1 p t ih0 v l iy0))
("decertification" nil (d iy0 s er1 t ax f ax k ey1 sh ax n))
("decertified" nil (d iy0 s er1 t ax f ay1 d))
("decesare" nil (d ih0 ch eh0 s aa1 r iy0))
("decesaris" nil (d ih0 s eh1 s er0 ih0 s))
("dech" nil (d eh1 k))
("dechant" nil (d ey1 ch ax n t))
("dechellis" nil (d eh1 k ih0 l ih0 s))
("dechene" nil (d eh1 k iy0 n))
("decherd" nil (d eh1 ch er0 d))
("dechert" nil (d eh1 k er0 t))
("dechristopher" nil (d eh1 k r ih0 s t aa0 f er0))
("decibel" nil (d eh1 s ax b eh1 l))
("decibels" nil (d eh1 s ax b ax l z))
("decicco" nil (d ih0 ch iy1 k ow0))
("decide" nil (d ih0 s ay1 d))
("decided" nil (d ih0 s ay1 d ih0 d))
("decidedly" nil (d ih0 s ay1 d ax d l iy0))
("decides" nil (d ih0 s ay1 d z))
("deciding" nil (d ax s ay1 d ih0 ng))
("deciduous" nil (d ih0 s ih1 jh uw0 ax s))
("deciliter" nil (d eh1 s ax l iy1 t er0))
("decima" nil (d ih0 ch iy1 m ax))
("decimal" nil (d eh1 s ax m ax l))
("decimals" nil (d eh1 s ax m ax l z))
("decimate" nil (d eh1 s ax m ey1 t))
("decimated" nil (d eh1 s ax m ey1 t ih0 d))
("decipher" nil (d ih0 s ay1 f er0))
("deciphered" nil (d ih0 s ay1 f er0 d))
("deciphering" nil (d ax s ay1 f er0 ih0 ng))
("decision" nil (d ih0 s ih1 zh ax n))
("decisionmaking" nil (d ih0 s ih1 zh ax n m ey1 k ih0 ng))
("decisions" nil (d ih0 s ih1 zh ax n z))
("decisive" nil (d ih0 s ay1 s ih0 v))
("decisively" nil (d ih0 s ay1 s ih0 v l iy0))
("decisiveness" nil (d ih0 s ay1 s ih0 v n ax s))
("deck" nil (d eh1 k))
("deckard" nil (d ih0 k aa1 r d))
("decked" nil (d eh1 k t))
("decker" nil (d eh1 k er0))
("deckert" nil (d eh1 k er0 t))
("decking" nil (d eh1 k ih0 ng))
("deckman" nil (d eh1 k m ax n))
("decks" nil (d eh1 k s))
("declaimed" nil (d ih0 k l ey1 m d))
("declaration" nil (d eh1 k l er0 ey1 sh ax n))
("declarations" nil (d eh1 k l er0 ey1 sh ax n z))
("declaratory" nil (d ih0 k l eh1 r ax t ao1 r iy0))
("declare" nil (d ih0 k l eh1 r))
("declared" nil (d ih0 k l eh1 r d))
("declares" nil (d ih0 k l eh1 r z))
("declaring" nil (d ih0 k l eh1 r ih0 ng))
("declassified" nil (d ih0 k l ae1 s ax f ay1 d))
("declassify" nil (d ih0 k l ae1 s ax f ay1))
("declerck" nil (d ax k l er1 k))
("declercq" nil (d ax k l er1 k))
("declerk" nil (d ax k l er1 k))
("declerque" nil (d ax k l er1 k))
("decline" nil (d ih0 k l ay1 n))
("declined" nil (d ih0 k l ay1 n d))
("decliner" nil (d ih0 k l ay1 n er0))
("decliners" nil (d ih0 k l ay1 n er0 z))
("declines" nil (d ih0 k l ay1 n z))
("declining" nil (d ih0 k l ay1 n ih0 ng))
("declue" nil (d eh1 k l uw0))
("deco" nil (d eh1 k ow0))
("decode" nil (d ih0 k ow1 d))
("decoder" nil (d ih0 k ow1 d er0))
("decoders" nil (d ih0 k ow1 d er0 z))
("decoding" nil (d ih0 k ow1 d ih0 ng))
("decola" nil (d ih0 k ow1 l ax))
("decom" nil (d eh1 k aa1 m))
("decommissioned" nil (d iy0 k ax m ih1 sh ax n d))
("decommissioning" nil (d iy0 k ax m ih1 sh ax n ih0 ng))
("decompose" nil (d iy1 k ax m p ow1 z))
("decomposed" nil (d iy1 k ax m p ow1 z d))
("decomposes" nil (d iy1 k ax m p ow1 z ih0 z))
("decompression" nil (d iy1 k ax m p r eh1 sh ax n))
("deconcini" nil (d iy1 k ax n s iy1 n iy0))
("decongestants" nil (d ih0 k ax n jh eh1 s t ax n t s))
("deconstruction" nil (d iy0 k ax n s t r ah1 k sh ax n))
("decontamination" nil (d iy0 k ax n t ae1 m ax n ey1 sh ax n))
("decontrol" nil (d iy1 k ax n t r ow1 l))
("decontrolled" nil (d iy1 k ax n t r ow1 l d))
("decook" nil (d eh1 k uh0 k))
("decor" nil (d ih0 k ao1 r))
("decorate" nil (d eh1 k er0 ey1 t))
("decorated" nil (d eh1 k er0 ey1 t ax d))
("decorating" nil (d eh1 k er0 ey1 t ih0 ng))
("decoration" nil (d eh1 k er0 ey1 sh ax n))
("decorations" nil (d eh1 k er0 ey1 sh ax n z))
("decorative" nil (d eh1 k r ax t ih0 v))
("decorator" nil (d eh1 k er0 ey1 t er0))
("decorators" nil (d eh1 k er0 ey1 t er0 z))
("decordova" nil (d iy1 k ao1 r d ow1 v ax))
("decorous" nil (d eh1 k er0 ax s))
("decorte" nil (d ih0 k ao1 r t iy0))
("decorum" nil (d ih0 k ao1 r ax m))
("decosta" nil (d ih0 k ow1 s t ax))
("decoste" nil (d ih0 k ow1 s t iy0))
("decoster" nil (d eh1 k ax s t er0))
("decoteau" nil (d eh1 k ax t ow0))
("decou" nil (d ih0 k uw1))
("decouple" nil (d iy0 k ah1 p ax l))
("decoupling" nil (d iy0 k ah1 p l ih0 ng))
("decourcy" nil (d eh1 k uh0 r k iy0))
("decoursey" nil (d eh1 k ao0 r s iy0))
("decoy" nil (d ax k oy1))
("decoys" nil (d iy1 k oy0 z))
("decrane" nil (d ax k r ey1 n))
("decrease" v (d ih0 k r iy1 s))
("decrease" n (d iy1 k r iy0 s))
("decreased" nil (d ih0 k r iy1 s t))
("decreases" v (d ih0 k r iy1 s ax z))
("decreases" n (d iy1 k r iy0 s ax z))
("decreasing" nil (d ih0 k r iy1 s ih0 ng))
("decree" nil (d ih0 k r iy1))
("decreed" nil (d ih0 k r iy1 d))
("decrees" nil (d ih0 k r iy1 z))
("decrepit" nil (d ax k r eh1 p ih0 t))
("decrescenzo" nil (d ih0 k r eh0 s ch eh1 n z ow0))
("decribed" nil (d ih0 k r ay1 b d))
("decried" nil (d ih0 k r ay1 d))
("decries" nil (d ih0 k r ay1 z))
("decriminalization" nil (d iy0 k r ih1 m ax n ax l ax z ey1 sh ax n))
("decristofaro" nil (d ih0 k r iy0 s t ow0 f aa1 r ow0))
("decry" nil (d ih0 k r ay1))
("decrying" nil (d ih0 k r ay1 ih0 ng))
("decter" nil (d eh1 k t er0))
("decuir" nil (d eh1 k ih0 r))
("decworld" nil (d eh1 k w er0 l d))
("dede" nil (d iy1 d))
("dedeaux" nil (d ih0 d ow1))
("dedecker" nil (d eh1 d ih0 k er0))
("dederick" nil (d eh1 d er0 ih0 k))
("dedeurwaerder" nil (d ax d er1 w aa1 r d er0))
("dedham" nil (d eh1 d ax m))
("dedic" nil (d eh1 d ih0 k))
("dedicate" nil (d eh1 d ax k ey1 t))
("dedicated" nil (d eh1 d ax k ey0 t ax d))
("dedicates" nil (d eh1 d ih0 k ey1 t s))
("dedicating" nil (d eh1 d ih0 k ey1 t ih0 ng))
("dedication" nil (d eh1 d ax k ey1 sh ax n))
("dedios" nil (d ey0 d iy1 ow0 z))
("dedman" nil (d eh1 d m ax n))
("dedmon" nil (d eh1 d m ax n))
("dedo" nil (d ey1 d ow0))
("dedominicis" nil (d ey0 d ow0 m iy0 n iy1 s ih0 s))
("dedrick" nil (d eh1 d r ih0 k))
("deduce" nil (d ih0 d uw1 s))
("deduced" nil (d ih0 d uw1 s t))
("deduct" nil (d ih0 d ah1 k t))
("deducted" nil (d ih0 d ah1 k t ih0 d))
("deductibility" nil (d ih0 d ah1 k t ax b ih1 l ax t iy0))
("deductible" nil (d ih0 d ah1 k t ax b ax l))
("deductibles" nil (d ih0 d ah1 k t ax b ax l z))
("deducting" nil (d ih0 d ah1 k t ih0 ng))
("deduction" nil (d ih0 d ah1 k sh ax n))
("deductions" nil (d ih0 d ah1 k sh ax n z))
("deducts" nil (d ih0 d ah1 k t s))
("dee" nil (d iy1))
("deeb" nil (d iy1 b))
("deed" nil (d iy1 d))
("deeded" nil (d iy1 d ax d))
("deeds" nil (d iy1 d z))
("deeg" nil (d iy1 g))
("deegan" nil (d iy1 g ax n))
("deel" nil (d iy1 l))
("deeley" nil (d iy1 l iy0))
("deely" nil (d iy1 l iy0))
("deem" nil (d iy1 m))
("deemed" nil (d iy1 m d))
("deemer" nil (d iy1 m er0))
("deemphasizing" nil (d iy0 eh1 m f ax s ay1 z ih0 ng))
("deems" nil (d iy1 m z))
("deen" nil (d iy1 n))
("deener" nil (d iy1 n er0))
("deeney" nil (d iy1 n iy0))
("deep" nil (d iy1 p))
("deepak" nil (d iy1 p ae1 k))
("deepen" nil (d iy1 p ax n))
("deepened" nil (d iy1 p ax n d))
("deepening" nil (d iy1 p ax n ih0 ng))
("deepens" nil (d iy1 p ax n z))
("deeper" nil (d iy1 p er0))
("deepest" nil (d iy1 p ax s t))
("deeply" nil (d iy1 p l iy0))
("deepwater" nil (d iy1 p w ao1 t er0))
("deer" nil (d ih1 r))
("deere" nil (d ih1 r))
("deerfield" nil (d ih1 r f iy0 l d))
("deering" nil (d ih1 r ih0 ng))
("deerman" nil (d ih1 r m ax n))
("deerskin" nil (d ih1 r s k ih1 n))
("deery" nil (d ih1 r iy0))
("dees" nil (d iy1 z))
("deese" nil (d iy1 z))
("deeter" nil (d iy1 t er0))
("deets" nil (d iy1 t s))
("deetz" nil (d iy1 t s))
("defabio" nil (d ih0 f aa1 b iy0 ow0))
("defaced" nil (d ih0 f ey1 s t))
("defalco" nil (d ih0 f aa1 l k ow0))
("defamation" nil (d eh1 f ax m ey1 sh ax n))
("defamatory" nil (d ih0 f ae1 m ax t ao1 r iy0))
("defamed" nil (d ih0 f ey1 m d))
("defarges" nil (d ih0 f aa1 r jh ih0 z))
("default" nil (d ih0 f ao1 l t))
("defaulted" nil (d ih0 f ao1 l t ih0 d))
("defaulters" nil (d ih0 f ao1 l t er0 z))
("defaulting" nil (d ih0 f ao1 l t ih0 ng))
("defaults" nil (d ih0 f ao1 l t s))
("defazio" nil (d ih0 f aa1 z iy0 ow0))
("defeasance" nil (d ih0 f iy1 z ax n s))
("defeat" nil (d ih0 f iy1 t))
("defeated" nil (d ih0 f iy1 t ax d))
("defeating" nil (d ih0 f iy1 t ih0 ng))
("defeatism" nil (d ih0 f iy1 t ih0 z ax m))
("defeatist" nil (d ih0 f iy1 t ih0 s t))
("defeats" nil (d ih0 f iy1 t s))
("defect" v (d ih1 f eh1 k t))
("defect" n (d iy1 f eh0 k t))
("defected" nil (d ih0 f eh1 k t ih0 d))
("defecting" nil (d ih0 f eh1 k t ih0 ng))
("defection" nil (d ih0 f eh1 k sh ax n))
("defections" nil (d ih0 f eh1 k sh ax n z))
("defective" nil (d ih0 f eh1 k t ih0 v))
("defector" nil (d ih0 f eh1 k t er0))
("defectors" nil (d ih0 f eh1 k t er0 z))
("defects" v (d ih1 f eh1 k t))
("defects" n (d iy1 f eh0 k t))
("defee" nil (d eh1 f iy0))
("defelice" nil (d ih0 f eh1 l ih0 s))
("defenbaugh" nil (d eh1 f ih0 n b aw0))
("defence" nil (d ih0 f eh1 n s))
("defend" nil (d ih0 f eh1 n d))
("defendant" nil (d ih0 f eh1 n d ax n t))
("defendants" nil (d ih0 f eh1 n d ax n t s))
("defended" nil (d ih0 f eh1 n d ax d))
("defender" nil (d ih0 f eh1 n d er0))
("defenders" nil (d ih0 f eh1 n d er0 z))
("defending" nil (d ih0 f eh1 n d ih0 ng))
("defends" nil (d ih0 f eh1 n d z))
("defense" nil (d ih0 f eh1 n s))
("defenseless" nil (d ih0 f eh1 n s l ax s))
("defenses" nil (d ih0 f eh1 n s ax z))
("defensible" nil (d ih0 f eh1 n s ax b ax l))
("defensive" nil (d ih0 f eh1 n s ih0 v))
("defensively" nil (d ih0 f eh1 n s ih0 v l iy0))
("defensiveness" nil (d ih0 f eh1 n s ih0 v n ax s))
("defeo" nil (d iy1 f iy0 ow0))
("defer" nil (d ih0 f er1))
("deference" nil (d eh1 f er0 ax n s))
("deferential" nil (d eh1 f er0 eh1 n ch ax l))
("deferment" nil (d ih0 f er1 m ax n t))
("deferments" nil (d ih0 f er1 m ax n t s))
("deferral" nil (d ih0 f er1 ax l))
("deferrals" nil (d ih0 f er1 ax l z))
("deferred" nil (d ih0 f er1 d))
("deferring" nil (d ih0 f er1 ih0 ng))
("defers" nil (d ih0 f er1 z))
("defex" nil (d eh1 f eh1 k s))
("deffenbaugh" nil (d eh1 f ih0 n b aw0))
("deffeyes" nil (d eh0 f ay1 z))
("defiance" nil (d ih0 f ay1 ax n s))
("defiant" nil (d ih0 f ay1 ax n t))
("defiantly" nil (d ih0 f ay1 ax n t l iy0))
("defibaugh" nil (d eh1 f ih0 b ao1))
("defibrillator" nil (d iy0 f ih1 b r ih0 l ey1 t er0))
("defibrillators" nil (d iy0 f ih1 b r ih0 l ey1 t er0 z))
("deficiencies" nil (d ih0 f ih1 sh ax n s iy0 z))
("deficiency" nil (d ih0 f ih1 sh ax n s iy0))
("deficient" nil (d ih0 f ih1 sh ax n t))
("deficit" nil (d eh1 f ax s ax t))
("deficits" nil (d eh1 f ih0 s ih0 t s))
("defied" nil (d ih0 f ay1 d))
("defies" nil (d ih0 f ay1 z))
("defilippis" nil (d eh1 f ih0 l ih0 p ih0 s))
("defilippo" nil (d ih0 f iy0 l iy1 p ow0))
("defina" nil (d ih0 f iy1 n ax))
("definable" nil (d ih0 f ay1 n ax b ax l))
("define" nil (d ih0 f ay1 n))
("defined" nil (d ih0 f ay1 n d))
("defines" nil (d ih0 f ay1 n z))
("defining" nil (d ih0 f ay1 n ih0 ng))
("definite" nil (d eh1 f ax n ax t))
("definitely" nil (d eh1 f ax n ax t l iy0))
("definition" nil (d eh1 f ax n ih1 sh ax n))
("definitions" nil (d eh1 f ax n ih1 sh ax n z))
("definitive" nil (d ih0 f ih1 n ih0 t ih0 v))
("definitively" nil (d ih0 f ih1 n ih0 t ih0 v l iy0))
("defino" nil (d ih0 f iy1 n ow0))
("defiore" nil (d ih0 f iy0 ao1 r iy0))
("deflate" nil (d ih0 f l ey1 t))
("deflated" nil (d ih0 f l ey1 t ih0 d))
("deflating" nil (d ih0 f l ey1 t ih0 ng))
("deflation" nil (d ih0 f l ey1 sh ax n))
("deflationary" nil (d ih0 f l ey1 sh ax n eh1 r iy0))
("deflator" nil (d ih0 f l ey1 t er0))
("deflect" nil (d ih0 f l eh1 k t))
("deflected" nil (d ih0 f l eh1 k t ih0 d))
("deflecting" nil (d ih0 f l eh1 k t ih0 ng))
("deflects" nil (d ih0 f l eh1 k t s))
("defoe" nil (d ih0 f ow1))
("defoliant" nil (d ih0 f ow1 l iy0 ax n t))
("defoliants" nil (d ih0 f ow1 l iy0 ax n t s))
("defoor" nil (d eh1 f uh0 r))
("deford" nil (d eh1 f er0 d))
("defore" nil (d iy1 f ao0 r))
("deforest" nil (d ih0 f ao1 r ax s t))
("deforestation" nil (d ih0 f ao1 r ih0 s t ey1 sh ax n))
("deforge" nil (d eh1 f er0 g))
("deformation" nil (d iy1 f ao0 r m ey1 sh ax n))
("deformed" nil (d ih0 f ao1 r m d))
("deformities" nil (d ih0 f ao1 r m ax t iy0 z))
("deforrest" nil (d ey0 f ao1 r ih0 s t))
("defrain" nil (d ih0 f r ey1 n))
("defrance" nil (d iy1 f r ax n s))
("defrancesco" nil (d ih0 f r aa0 n ch eh1 s k ow0))
("defrancisco" nil (d ih0 f r aa0 n ch iy1 s k ow0))
("defranco" nil (d ih0 f r aa1 n k ow0))
("defrank" nil (d eh1 f r ax ng k))
("defrates" nil (d eh1 f er0 ey0 t s))
("defraud" nil (d ih0 f r ao1 d))
("defrauded" nil (d ih0 f r ao1 d ih0 d))
("defrauding" nil (d ih0 f r ao1 d ih0 ng))
("defray" nil (d ih0 f r ey1))
("defrays" nil (d ih0 f r ey1 z))
("defrees" nil (d ih0 f r iy1 z))
("defreese" nil (d eh1 f r iy0 s))
("defreitas" nil (d eh1 f r ay0 t ax z))
("defries" nil (d ih0 f r iy1 z))
("defrost" nil (d ih0 f r ao1 s t))
("defrosting" nil (d ih0 f r ao1 s t ih0 ng))
("deft" nil (d eh1 f t))
("defterios" nil (d eh1 f t eh1 r iy0 ow0 z))
("deftly" nil (d eh1 f t l iy0))
("defunct" nil (d ih0 f ah1 ng k t))
("defund" nil (d iy0 f ah1 n d))
("defunding" nil (d iy0 f ah1 n d ih0 ng))
("defusco" nil (d ih0 f uw1 s k ow0))
("defuse" nil (d ih0 f y uw1 z))
("defused" nil (d ih0 f y uw1 z d))
("defusing" nil (d ih0 f y uw1 z ih0 ng))
("defy" nil (d ih0 f ay1))
("defying" nil (d ih0 f ay1 ih0 ng))
("degaetano" nil (d ih0 g aa0 eh0 t aa1 n ow0))
("degan" nil (d iy1 g ax n))
("degarmo" nil (d ih0 g aa1 r m ow0))
("degas" nil (d ih0 g ae1 s))
("degaulle" nil (d ax g aa1 l))
("degen" nil (d eh1 g ax n))
("degener" nil (d eh1 g iy0 n er0))
("degeneracy" nil (d ih0 jh eh1 n er0 ax s iy0))
("degenerate" nil (d ih0 jh eh1 n er0 ax t))
("degenerated" nil (d ih0 jh eh1 n er0 ey1 t ih0 d))
("degenerates" nil (d ih0 jh eh1 n er0 ax t s))
("degenerating" nil (d ih0 jh eh1 n er0 ey1 t ih0 ng))
("degeneration" nil (d ih0 jh eh1 n er0 ey1 sh ax n))
("degenerative" nil (d ih0 jh eh1 n er0 ax t ih0 v))
("degenhardt" nil (d eh1 g ih0 n hh aa0 r t))
("degenhart" nil (d eh1 g ax n hh aa1 r t))
("degennaro" nil (d ih0 jh eh0 n aa1 r ow0))
("degeorge" nil (d eh1 g iy0 er0 g))
("deger" nil (d iy1 g er0))
("degiacomo" nil (d iy1 jh ax k ow0 m ow0))
("degidio" nil (d ih0 jh iy1 d iy0 ow0))
("degirolamo" nil (d ih0 jh ih0 r ow0 l aa1 m ow0))
("degler" nil (d eh1 g l er0))
("degnan" nil (d eh1 g n ax n))
("degner" nil (d eh1 g n er0))
("degood" nil (d eh1 g uh0 d))
("degraaf" nil (d eh1 g r aa0 f))
("degrace" nil (d ih0 g r aa1 ch iy0))
("degradable" nil (d ih0 g r ey1 d ax b ax l))
("degradation" nil (d eh1 g r ax d ey1 sh ax n))
("degradations" nil (d eh1 g r ax d ey1 sh ax n z))
("degrade" nil (d ih0 g r ey1 d))
("degraded" nil (d ih0 g r ey1 d ax d))
("degrades" nil (d ih0 g r ey1 d z))
("degrading" nil (d ih0 g r ey1 d ih0 ng))
("degraff" nil (d eh1 g r ax f))
("degraffenreid" nil (d eh1 g r ax f ih0 n r ay0 d))
("degrand" nil (d eh1 g r ae0 n d))
("degrange" nil (d eh1 g r ey0 n jh))
("degrasse" nil (d ih0 g r aa1 s iy0))
("degrave" nil (d ih0 g r aa1 v iy0))
("degraw" nil (d eh1 g r ao0))
("degray" nil (d eh1 g r ey0))
("degrazia" nil (d ih0 g r aa1 z iy0 ax))
("degree" nil (d ih0 g r iy1))
("degreed" nil (d ih0 g r iy1 d))
("degrees" nil (d ih0 g r iy1 z))
("degregorio" nil (d ih0 g r eh0 g ao1 r iy0 ow0))
("degregory" nil (d eh1 g r ih0 g er0 iy0))
("degroat" nil (d eh1 g r ow0 t))
("degroff" nil (d eh1 g r ao0 f))
("degroot" nil (d eh1 g r uw0 t))
("degroote" nil (d ax g r uw1 t))
("deguire" nil (d ey0 g w ih1 r ey0))
("degussa" nil (d ih0 g y uw1 s ax))
("degutare" nil (d eh1 g uw0 t aa1 r iy0))
("deguzman" nil (d ey0 g uw0 z m ae1 n))
("dehaan" nil (d eh1 hh aa0 n))
("dehaas" nil (d eh1 hh aa0 z))
("deharbe" nil (d ax hh aa1 r b iy0))
("dehart" nil (d eh1 hh aa0 r t))
("dehaven" nil (d eh1 hh ax v ax n))
("dehecq" nil (d ih0 hh eh1 k))
("deherrera" nil (d ey0 hh eh0 r eh1 r ax))
("dehler" nil (d eh1 l er0))
("dehm" nil (d eh1 m))
("dehmer" nil (d eh1 m er0))
("dehn" nil (d eh1 n))
("dehne" nil (d eh1 n))
("dehner" nil (d eh1 n er0))
("dehnert" nil (d eh1 n er0 t))
("dehoff" nil (d eh1 hh ao0 f))
("dehoyos" nil (d eh1 hh oy0 ow0 z))
("dehumanization" nil (d iy1 hh y uw1 m ax n ax z ey1 sh ax n))
("dehumanized" nil (d ih0 hh y uw1 m ax n ay0 z d))
("dehumanizing" nil (d iy0 hh y uw1 m ax n ay1 z ih0 ng))
("dehydrated" nil (d ih0 hh ay1 d r ey0 t ax d))
("dehydration" nil (d iy1 hh ay0 d r ey1 sh ax n))
("deibel" nil (d ay1 b ax l))
("deibert" nil (d ay1 b er0 t))
("deibler" nil (d ay1 b ax l er0))
("deichert" nil (d ay1 k er0 t))
("deidre" nil (d iy1 d r ax))
("deification" nil (d iy1 ax f ax k ey1 sh ax n))
("deify" nil (d iy1 ax f ay1))
("deighan" nil (d ey1 g hh ax n))
("deighton" nil (d ey1 t ax n))
("deignan" nil (d ax g n ae1 n))
("deihl" nil (d ay1 l))
("deike" nil (d iy1 k))
("deikel" nil (d ay1 k ax l))
("deily" nil (d iy1 l iy0))
("deines" nil (d iy1 n z))
("deininger" nil (d ay1 n ih0 ng er0))
("deinstitutionalization" nil (d iy0 ih1 n s t ih0 t uw1 sh ax n ax l ax z ey1 sh ax n))
("deis" nil (d iy1 z))
("deisher" nil (d iy1 ih0 sh er0))
("deism" nil (d iy1 ih0 z ax m))
("deiss" nil (d ay1 s))
("deist" nil (d iy1 ih0 s t))
("deitch" nil (d ay1 ch))
("deiter" nil (d ay1 t er0))
("deiters" nil (d ay1 t er0 z))
("deities" nil (d iy1 ax t iy0 z))
("deitrich" nil (d ay1 t r ih0 k))
("deitrick" nil (d ay1 t r ih0 k))
("deitsch" nil (d ay1 ch))
("deity" nil (d iy1 ax t iy0))
("deitz" nil (d iy1 t s))
("deja" nil (d iy1 jh ax))
("dejager" nil (d eh1 jh ey0 g er0))
("dejarnett" nil (d ih0 jh aa1 r n ih0 t))
("dejarnette" nil (d eh1 zh aa0 r n eh0 t))
("dejean" nil (d ih0 zh iy1 n))
("dejected" nil (d ih0 jh eh1 k t ih0 d))
("dejesus" nil (d ih0 jh iy1 z ax s))
("dejohn" nil (d ax jh aa1 n))
("dejong" nil (d ax jh ao1 ng))
("dejonge" nil (d ax jh ao1 ng))
("dejongh" nil (d ax jh ao1 ng))
("dejoseph" nil (d ax jh ow1 s ax f))
("dejoy" nil (d ax jh oy1))
("dejulio" nil (d ax jh uw1 l iy0 ow0))
("dekalb" nil (d ih0 k ae1 l b))
("dekay" nil (d ax k ey1))
("dekeyser" nil (d ax k ay1 z er0))
("dekker" nil (d eh1 k er0))
("dekle" nil (d eh1 k ax l))
("deklerk" nil (d ax k l er1 k))
("dekom" nil (d eh1 k ax m))
("dekoning" nil (d eh1 k ax n ih0 ng))
("dekuyper" nil (d ih0 k ay1 p er0))
("del" nil (d eh1 l))
("dela" nil (d eh1 l ax))
("delacerda" nil (d eh0 l aa0 ch eh1 r d ax))
("delacruz" nil (d ey0 l aa1 k r uw0 z))
("delacy" nil (d ax l ao1 s iy0))
("delafield" nil (d eh1 l ax f iy1 l d))
("delafuente" nil (d ey0 l aa0 f w eh1 n t ey0))
("delagarza" nil (d eh0 l aa0 g aa1 r z ax))
("delage" nil (d eh1 l ih0 jh))
("delagrange" nil (d eh0 l aa1 g r aa0 n jh))
("delahanty" nil (d eh1 l ax hh ax n t iy0))
("delahoussaye" nil (d eh0 l ax hh aw1 s ey0))
("delahunt" nil (d eh1 l ax hh ax n t))
("delahunty" nil (d eh1 l ax hh ax n t iy0))
("delaine" nil (d ih0 l ey1 n))
("delair" nil (d ih0 l eh1 r))
("delamar" nil (d ey0 l aa0 m aa1 r))
("delamater" nil (d eh1 l ax m ey0 t er0))
("delancey" nil (d eh1 l ax n s iy0))
("delancy" nil (d eh1 l ax n s iy0))
("deland" nil (d ih0 l ae1 n d))
("delane" nil (d eh1 l ax n))
("delaney" nil (d ax l ey1 n iy0))
("delange" nil (d eh1 l ey0 n jh))
("delano" nil (d ih0 l aa1 n ow0))
("delanoy" nil (d eh1 l ax n oy0))
("delany" nil (d eh1 l ax n iy0))
("delao" nil (d eh1 l aw0))
("delap" nil (d eh1 l ax p))
("delapaz" nil (d ey0 l aa1 p aa0 z))
("delapena" nil (d eh0 l aa0 p eh1 n ax))
("delapp" nil (d eh1 l ax p))
("delara" nil (d eh0 l aa1 r ax))
("delariva" nil (d eh0 l aa0 r iy1 v ax))
("delarosa" nil (d eh0 l aa0 r ow1 s ax))
("delashmit" nil (d eh1 l ax sh m ih0 t))
("delashmutt" nil (d eh1 l ax sh m ax t))
("delatorre" nil (d eh0 l aa0 t ao1 r iy0))
("delatte" nil (d ih0 l ae1 t))
("delauder" nil (d eh1 l aw0 d er0))
("delaughter" nil (d eh1 l ao0 t er0))
("delaune" nil (d eh1 l ao0 n))
("delaura" nil (d eh0 l ao1 r ax))
("delaurentiis" nil (d iy0 l ao0 r eh1 n t iy1 z))
("delaurentis" nil (d ey0 l aw0 r ey1 n t ih0 s))
("delauter" nil (d eh1 l aw0 t er0))
("delaval" nil (d eh1 l ax v ae0 l))
("delavan" nil (d eh1 l ax v ax n))
("delavega" nil (d ey0 l aa0 v ey1 g ax))
("delaware" nil (d eh1 l ax w eh1 r))
("delawder" nil (d eh1 l ao0 d er0))
("delay" nil (d ih0 l ey1))
("delayed" nil (d ih0 l ey1 d))
("delaying" nil (d ih0 l ey1 ih0 ng))
("delays" nil (d ih0 l ey1 z))
("delbarco" nil (d eh0 l b aa1 r k ow0))
("delbene" nil (d eh1 l b ih0 n ax))
("delbert" nil (d eh1 l b er0 t))
("delbianco" nil (d eh0 l b iy0 aa1 n k ow0))
("delbosque" nil (d ih0 l b ow1 s k))
("delbridge" nil (d eh1 l b r ih0 jh))
("delbuono" nil (d eh1 l b w ow1 n ow0))
("delcambre" nil (d eh0 l k aa1 m b r iy0))
("delcamp" nil (d eh1 l k ax m p))
("delcampo" nil (d eh0 l k aa1 m p ow0))
("delcarlo" nil (d eh1 l k aa1 r l ow0))
("delcastillo" nil (d eh0 l k aa0 s t ih1 l ow0))
("delchamps" nil (d eh1 l ch ae1 m p s))
("delcine" nil (d eh0 l ch iy1 n iy0))
("delco" nil (d eh1 l k ow0))
("delconte" nil (d eh0 l k ow1 n t iy0))
("delcor" nil (d eh1 l k ao1 r))
("delduca" nil (d eh1 l d uw1 k ax))
("delebarre" nil (d eh1 l ax b aa1 r))
("delectable" nil (d ih0 l eh1 k t ax b ax l))
("delee" nil (d eh1 l iy0))
("deleeuw" nil (d eh1 l iy0 uw0))
("delegate" nil (d eh1 l ax g ey1 t))
("delegated" nil (d eh1 l ax g ey1 t ax d))
("delegates" nil (d eh1 l ax g ey1 t s))
("delegating" nil (d eh1 l ax g ey1 t ih0 ng))
("delegation" nil (d eh1 l ax g ey1 sh ax n))
("delegations" nil (d eh1 l ax g ey1 sh ax n z))
("delehanty" nil (d eh1 l ih0 hh ax n t iy0))
("delellis" nil (d eh1 l ih0 l ih0 s))
("deleo" nil (d eh1 l iy0 ow0))
("deleon" nil (d eh1 l iy0 ax n))
("deleonardis" nil (d eh1 l iy0 ax n aa0 r d ih0 s))
("delete" nil (d ih0 l iy1 t))
("deleted" nil (d ih0 l iy1 t ax d))
("deleterious" nil (d eh1 l ax t ih1 r iy0 ax s))
("deleting" nil (d ih0 l iy1 t ih0 ng))
("deletion" nil (d ih0 l iy1 sh ax n))
("deletions" nil (d ih0 l iy1 sh ax n z))
("delfavero" nil (d eh0 l f aa0 v eh1 r ow0))
("delfin" nil (d eh1 l f ih0 n))
("delfina" nil (d eh1 l f iy1 n ax))
("delfine" nil (d eh0 l f iy1 n iy0))
("delfino" nil (d eh1 l f iy1 n ow0))
("delfosse" nil (d eh1 l f ax s))
("delfs" nil (d eh1 l f s))
("delft" nil (d eh1 l f t))
("delftware" nil (d eh1 l f t w eh1 r))
("delgadillo" nil (d eh0 l g aa0 d ih1 l ow0))
("delgado" nil (d eh0 l g aa1 d ow0))
("delgaudio" nil (d eh0 l g ao1 d iy0 ow0))
("delgiorno" nil (d eh0 l jh ao1 r n ow0))
("delgiudice" nil (d eh0 l jh uw1 d ax s))
("delgrande" nil (d eh1 l g r ae0 n d iy0))
("delgreco" nil (d eh1 l g r eh1 k ow0))
("delgrosso" nil (d eh1 l g r ow1 s ow0))
("delguercio" nil (d eh0 l g eh1 r ch iy0 ow0))
("delguidice" nil (d eh0 l g ay1 d ih0 s))
("delhaize" nil (d eh1 l hh ey1 z))
("delhi" nil (d eh1 l iy0))
("deli" nil (d eh1 l iy0))
("delia" nil (d iy1 l y ax))
("deliberate" j (d ih0 l ih1 b er0 ih0 t))
("deliberate" v (d ih0 l ih1 b er0 ey1 t))
("deliberated" nil (d ih0 l ih1 b er0 ey1 t ih0 d))
("deliberately" nil (d ih0 l ih1 b er0 ax t l iy0))
("deliberating" nil (d ih0 l ih1 b er0 ey1 t ih0 ng))
("deliberation" nil (d ih0 l ih1 b er0 ey1 sh ax n))
("deliberations" nil (d ih0 l ih1 b er0 ey1 sh ax n z))
("deliberative" nil (d ih0 l ih1 b er0 ey1 t ih0 v))
("delicacies" nil (d eh1 l ih0 k ax s iy0 z))
("delicacy" nil (d eh1 l ax k ax s iy0))
("delicate" nil (d eh1 l ax k ax t))
("delicately" nil (d eh1 l ax k ax t l iy0))
("delicatessen" nil (d eh1 l ih0 k ax t eh1 s ax n))
("delicatessens" nil (d eh1 l ih0 k ax t eh1 s ax n z))
("delich" nil (d eh1 l ih0 k))
("delicia" nil (d eh0 l iy1 ch ax))
("delicious" nil (d ih0 l ih1 sh ax s))
("deliciously" nil (d ih0 l ih1 sh ax sh l iy0))
("delight" nil (d ih0 l ay1 t))
("delighted" nil (d ih0 l ay1 t ax d))
("delightful" nil (d ih0 l ay1 t f ax l))
("delightfully" nil (d ih0 l ay1 t f ax l iy0))
("delighting" nil (d ih0 l ay1 t ih0 ng))
("delights" nil (d ih0 l ay1 t s))
("delila" nil (d eh0 l iy1 l ax))
("delilah" nil (d ax l ay1 l ax))
("delillo" nil (d ax l ih1 l ow0))
("delima" nil (d eh0 l iy1 m ax))
("deline" nil (d eh0 l iy1 n iy0))
("delineate" nil (d ih0 l ih1 n iy0 ey1 t))
("delineated" nil (d ih0 l ih1 n iy0 ey1 t ih0 d))
("delineates" nil (d ih0 l ih1 n iy0 ey1 t s))
("delineating" nil (d ih0 l ih1 n iy0 ey1 t ih0 ng))
("delineation" nil (d ih0 l ih1 n iy0 ey1 sh ax n))
("delinquencies" nil (d ih0 l ih1 ng k w ax n s iy0 z))
("delinquency" nil (d ih0 l ih1 ng k w ax n s iy0))
("delinquent" nil (d ih0 l ih1 ng k w ax n t))
("delinquents" nil (d ih0 l ih1 ng k w ax n t s))
("delio" nil (d ey1 l iy0 ow0))
("delirious" nil (d ih0 l ih1 r iy0 ax s))
("delirium" nil (d ih0 l ih1 r iy0 ax m))
("delisa" nil (d eh0 l iy1 s ax))
("delise" nil (d eh1 l ay0 z))
("delisi" nil (d eh0 l iy1 s iy0))
("delisio" nil (d eh0 l iy1 s iy0 ow0))
("delisle" nil (d ih0 l ay1 l))
("delist" nil (d iy1 l ih1 s t))
("delisted" nil (d iy1 l ih1 s t ih0 d))
("delisting" nil (d iy0 l ih1 s t ih0 ng))
("deliver" nil (d ih0 l ih1 v er0))
("deliverable" nil (d ih0 l ih1 v er0 ax b ax l))
("deliverance" nil (d ih0 l ih1 v er0 ax n s))
("delivered" nil (d ih0 l ih1 v er0 d))
("deliverers" nil (d ih0 l ih1 v er0 er0 z))
("deliveries" nil (d ih0 l ih1 v er0 iy0 z))
("delivering" nil (d ih0 l ih1 v er0 ih0 ng))
("delivers" nil (d ih0 l ih1 v er0 z))
("delivery" nil (d ih0 l ih1 v er0 iy0))
("delk" nil (d eh1 l k))
("delker" nil (d eh1 l k er0))
("dell" nil (d eh1 l))
("della" nil (d eh1 l ax))
("dellapenna" nil (d eh1 l ax p eh1 n ax))
("dellaquila" nil (d ey0 l aa0 k w iy1 l ax))
("dellarocco" nil (d eh1 l er0 ow0 k ow0))
("dellavalle" nil (d eh1 l ax v aa0 l iy0))
("dellavecchia" nil (d eh1 l ax v eh0 k iy0 ax))
("delle" nil (d eh1 l))
("deller" nil (d eh1 l er0))
("delligatti" nil (d eh0 l iy0 g aa1 t iy0))
("delling" nil (d eh1 l ih0 ng))
("dellinger" nil (d eh1 l ih0 ng er0))
("dellis" nil (d eh1 l ih0 s))
("dellolio" nil (d eh0 l ow1 l iy0 ow0))
("dellums" nil (d eh1 l ax m z))
("delma" nil (d eh1 l m ax))
("delman" nil (d eh1 l m ax n))
("delmar" nil (d eh1 l m er0))
("delmarva" nil (d eh1 l m aa1 r v ax))
("delmas" nil (d eh1 l m ax z))
("delmastro" nil (d eh1 l m ae1 s t r ow0))
("delmed" nil (d eh1 l m eh1 d))
("delmer" nil (d eh1 l m er0))
("delmonaco" nil (d eh0 l m ow0 n aa1 k ow0))
("delmonico" nil (d eh0 l m ow0 n iy1 k ow0))
("delmont" nil (d ey1 l m ow0 n t))
("delmonte" nil (d eh0 l m ow1 n t iy0))
("delmore" nil (d eh1 l m ao0 r))
("delnegro" nil (d eh1 l n eh1 g r ow0))
("delnero" nil (d eh0 l n eh1 r ow0))
("delo" nil (d eh1 l ow0))
("deloach" nil (d eh1 l ow0 ch))
("deloatch" nil (d eh1 l ow0 ch))
("deloitte" nil (d ax l oy1 t))
("deloney" nil (d eh1 l ax n iy0))
("delong" nil (d ax l ao1 ng))
("delora" nil (d eh0 l ao1 r ax))
("delore" nil (d ax l ao1 r))
("delorean" nil (d ax l ao1 r iy0 ax n))
("delorenzo" nil (d eh0 l ao0 r eh1 n z ow0))
("delores" nil (d ax l ao1 r ih0 s))
("delorey" nil (d eh1 l er0 iy0))
("deloria" nil (d eh0 l ao1 r iy0 ax))
("deloris" nil (d eh1 l er0 ih0 s))
("delorme" nil (d eh0 l ao1 r m iy0))
("delors" nil (d ax l ao1 r z))
("delosh" nil (d eh1 l ax sh))
("delosreyes" nil (d ih0 l aa1 s er0 ay0 z))
("delossantos" nil (d ey0 l ow0 s aa1 n t ow0 z))
("delouis" nil (d eh1 l uw0 iy1 z))
("delozier" nil (d eh1 l ax z iy0 er0))
("delp" nil (d eh1 l p))
("delph" nil (d eh1 l f))
("delphax" nil (d eh1 l f ae0 k s))
("delphi" nil (d eh1 l f ay0))
("delphia" nil (d eh1 l f iy0 ax))
("delphic" nil (d eh1 l f ih0 k))
("delphina" nil (d eh0 l f iy1 n ax))
("delphine" nil (d eh0 l f iy1 n iy0))
("delpino" nil (d eh1 l p iy1 n ow0))
("delpizzo" nil (d eh0 l p iy1 z ow0))
("delponte" nil (d eh0 l p ow1 n t iy0))
("delpozo" nil (d eh1 l p ow1 z ow0))
("delprete" nil (d eh1 l p r iy0 t))
("delpriore" nil (d eh0 l p r iy0 ao1 r iy0))
("delray" nil (d eh1 l r ey1))
("delre" nil (d eh1 l r))
("delreal" nil (d eh1 l r ax l))
("delrina" nil (d eh1 l r iy1 n ax))
("delrio" nil (d eh1 l r iy0 ow0))
("delrosario" nil (d eh0 l r ow0 s aa1 r iy0 ow0))
("delrossi" nil (d eh0 l r ow1 s iy0))
("delrosso" nil (d eh0 l r ow1 s ow0))
("delsanto" nil (d eh0 l s aa1 n t ow0))
("delsignore" nil (d eh0 l s iy0 g n ao1 r iy0))
("delta" nil (d eh1 l t ax))
("deltacorp" nil (d eh1 l t ax k ao1 r p))
("deltaic" nil (d eh0 l t ey1 ih0 k))
("deltak" nil (d eh1 l t ae1 k))
("deltec" nil (d eh1 l t eh1 k))
("deltona" nil (d eh1 l t ow1 n ax))
("deltoro" nil (d eh0 l t ao1 r ow0))
("deluca" nil (d ih0 l uw1 k ax))
("delucca" nil (d eh0 l uw1 k ax))
("delucchi" nil (d eh0 l uw1 k iy0))
("deluccia" nil (d eh0 l uw1 ch ax))
("delucia" nil (d eh0 l uw1 ch ax))
("delude" nil (d ih0 l uw1 d))
("deluded" nil (d ih0 l uw1 d ih0 d))
("deluding" nil (d ih0 l uw1 d ih0 ng))
("deluge" nil (d eh1 l y uw0 jh))
("deluged" nil (d eh1 l y uw0 jh d))
("deluise" nil (d eh0 l uw1 s iy0))
("deluna" nil (d eh0 l uw1 n ax))
("delusion" nil (d ih0 l uw1 zh ax n))
("delusions" nil (d ih0 l uw1 zh ax n z))
("deluxe" nil (d ax l ah1 k s))
("delvalle" nil (d eh1 l v ae1 l iy0))
("delve" nil (d eh1 l v))
("delvecchio" nil (d eh1 l v eh1 k iy0 ow0))
("delved" nil (d eh1 l v d))
("delves" nil (d eh1 l v z))
("delving" nil (d eh1 l v ih0 ng))
("delwin" nil (d eh1 l w ih0 n))
("delwyn" nil (d eh1 l w ih0 n))
("delzell" nil (d eh1 l z ax l))
("delzer" nil (d eh1 l z er0))
("demaggio" nil (d ih0 m aa1 jh iy0 ow0))
("demagogic" nil (d eh1 m ax g aa1 jh ih0 k))
("demagogue" nil (d eh1 m ax g aa1 g))
("demagoguery" nil (d eh1 m ax g aa1 g er0 iy0))
("demagogues" nil (d eh1 m ax g aa1 g z))
("demagogy" nil (d eh1 m ax g aa1 jh iy0))
("demain" nil (d ih0 m ey1 n))
("demaio" nil (d ih0 m aa1 iy0 ow0))
("deman" nil (d iy1 m ax n))
("demand" nil (d ih0 m ae1 n d))
("demanded" nil (d ih0 m ae1 n d ax d))
("demanding" nil (d ih0 m ae1 n d ih0 ng))
("demandingly" nil (d ih0 m ae1 n d ih0 ng l iy0))
("demands" nil (d ih0 m ae1 n d z))
("demar" nil (d ih0 m aa1 r))
("demarais" nil (d eh1 m er0 ey0))
("demaray" nil (d eh1 m er0 ey0))
("demarcation" nil (d iy1 m aa0 r k ey1 sh ax n))
("demarchi" nil (d ih0 m aa1 r k iy0))
("demarco" nil (d ih0 m aa1 r k ow0))
("demarcus" nil (d eh1 m aa0 r k ih0 s))
("demaree" nil (d eh0 m er0 iy1))
("demarest" nil (d ey0 m aa0 r ey1 ih0 s t))
("demaria" nil (d ih0 m aa1 r iy0 ax))
("demarinis" nil (d eh1 m er0 ih0 n ih0 s))
("demarino" nil (d ih0 m aa0 r iy1 n ow0))
("demario" nil (d ih0 m aa1 r iy0 ow0))
("demaris" nil (d eh1 m er0 ih0 s))
("demark" nil (d ax m aa1 r k))
("demars" nil (d eh1 m er0 z))
("demarsh" nil (d eh1 m aa0 r sh))
("demartin" nil (d ih0 m aa1 r t ih0 n))
("demartini" nil (d ih0 m aa0 r t iy1 n iy0))
("demartino" nil (d ih0 m aa0 r t iy1 n ow0))
("demary" nil (d eh1 m eh0 r iy0))
("demarzo" nil (d ih0 m aa1 r z ow0))
("demas" nil (d iy1 m ax s))
("demasi" nil (d ih0 m aa1 s iy0))
("demasters" nil (d iy1 m ae0 s t er0 z))
("dematteis" nil (d eh1 m ax t ay0 z))
("dematteo" nil (d ih0 m aa1 t iy0 ow0))
("demattia" nil (d ih0 m aa1 sh ax))
("demauro" nil (d ih0 m ao1 r ow0))
("demay" nil (d eh1 m ey0))
("demayo" nil (d ey0 m ey1 ow0))
("dembeck" nil (d eh1 m b eh1 k))
("dembinski" nil (d ih0 m b ih1 n s k iy0))
("dembowski" nil (d ih0 m b ao1 f s k iy0))
("dembski" nil (d eh1 m s k iy0))
("demby" nil (d eh1 m b iy0))
("demchak" nil (d eh1 m ch ax k))
("demean" nil (d ih0 m iy1 n))
("demeaning" nil (d ih0 m iy1 n ih0 ng))
("demeanor" nil (d ih0 m iy1 n er0))
("demeans" nil (d ih0 m iy1 n z))
("demel" nil (d eh1 m ax l))
("demello" nil (d ih0 m eh1 l ow0))
("demelo" nil (d ih0 m eh1 l ow0))
("dement" nil (d ax m eh1 n t))
("demented" nil (d ih0 m eh1 n t ih0 d))
("dementia" nil (d ih0 m eh1 n sh iy0 ax))
("demeo" nil (d iy1 m iy0 ow0))
("demerger" nil (d iy0 m er1 jh er0))
("demeritt" nil (d eh1 m er0 ih0 t))
("demers" nil (d iy1 m er0 z))
("demery" nil (d ih0 m er1 iy0))
("demeter" nil (d ih0 m iy1 t er0))
("demetre" nil (d eh0 m iy1 t er0))
("demetria" nil (d ih0 m eh1 t r iy0 ax))
("demetrio" nil (d ih0 m eh1 t r iy0 ow0))
("demetriou" nil (d ih0 m eh0 t r iy1 uw0))
("demeyer" nil (d eh1 m ay0 er0))
("demi" nil (d eh1 m iy0))
("demicco" nil (d ih0 m iy1 k ow0))
("demichael" nil (d eh1 m ih0 k eh0 l))
("demichele" nil (d eh1 m ih0 k ax l))
("demick" nil (d eh1 m ih0 k))
("demilio" nil (d ih0 m iy1 l iy0 ow0))
("demilitarization" nil (d iy0 m ih1 l ax t er0 ax z ey1 sh ax n))
("demilitarize" nil (d iy0 m ih1 l ax t er0 ay1 z))
("demilitarized" nil (d iy0 m ih1 l ax t er0 ay1 z d))
("demilitarizes" nil (d iy0 m ih1 l ax t er0 ay1 z ih0 z))
("demilitarizing" nil (d iy0 m ih1 l ax t er0 ay1 z ih0 ng))
("demille" nil (d ih0 m iy1 l iy0))
("demilo" nil (d ih0 m ih1 l ow0))
("deming" nil (d eh1 m ih0 ng))
("demint" nil (d ey1 m iy0 n t))
("demirag" nil (d ey1 m ih0 r aa1 jh))
("demirel" nil (d ax m ih1 r ax l))
("demirjian" nil (d ih0 m er1 jh iy0 ax n))
("demisch" nil (d ax m ih1 sh))
("demise" nil (d ih0 m ay1 z))
("demjanjuk" nil (d eh0 m y aa1 n y uw0 k))
("demko" nil (d eh1 m k ow0))
("demler" nil (d eh1 m l er0))
("demma" nil (d iy1 m ax))
("demme" nil (d eh1 m))
("demmer" nil (d eh1 m er0))
("demming" nil (d eh1 m ih0 ng))
("demmon" nil (d eh1 m ax n))
("demmons" nil (d eh1 m ax n z))
("demo" nil (d eh1 m ow0))
("demobilization" nil (d iy0 m ow1 b ax l ay0 z ey1 sh ax n))
("demobilize" nil (d ih0 m ow1 b ax l ay1 z))
("democracies" nil (d ih0 m aa1 k r ax s iy0 z))
("democracy" nil (d ih0 m aa1 k r ax s iy0))
("democrat" nil (d eh1 m ax k r ae1 t))
("democratic" nil (d eh1 m ax k r ae1 t ih0 k))
("democratica" nil (d eh1 m ax k r ae1 t ih0 k ax))
("democratically" nil (d eh1 m ax k r ae1 t ih0 k l iy0))
("democratization" nil (d ih0 m aa1 k r ax t ax z ey1 sh ax n))
("democratize" nil (d ih0 m aa1 k r ax t ay1 z))
("democrats" nil (d eh1 m ax k r ae1 t s))
("demodulation" nil (d iy1 m aa1 jh ax l ey1 sh ax n))
("demographer" nil (d ih0 m aa1 g r ax f er0))
("demographers" nil (d ih0 m aa1 g r ax f er0 z))
("demographic" nil (d eh1 m ax g r ae1 f ih0 k))
("demographically" nil (d eh1 m ax g r ae1 f ih0 k l iy0))
("demographics" nil (d eh1 m ax g r ae1 f ih0 k s))
("demography" nil (d ih0 m aa1 g r ax f iy0))
("demolish" nil (d ih0 m aa1 l ih0 sh))
("demolished" nil (d ih0 m aa1 l ih0 sh t))
("demolishing" nil (d ih0 m aa1 l ih0 sh ih0 ng))
("demolition" nil (d eh1 m ax l ih1 sh ax n))
("demon" nil (d iy1 m ax n))
("demond" nil (d ax m aa1 n d))
("demonic" nil (d ih0 m aa1 n ih0 k))
("demonize" nil (d iy1 m ax n ay1 z))
("demonized" nil (d iy1 m ax n ay1 z d))
("demonizer" nil (d iy1 m ax n ay1 z er0))
("demonizez" nil (d iy1 m ax n ay1 z ih0 z))
("demonizing" nil (d iy1 m ax n ay1 z ih0 ng))
("demons" nil (d iy1 m ax n z))
("demonstrable" nil (d eh1 m ax n s t r ax b ax l))
("demonstrably" nil (d ih0 m aa1 n s t r ax b l iy0))
("demonstrate" nil (d eh1 m ax n s t r ey1 t))
("demonstrated" nil (d eh1 m ax n s t r ey1 t ax d))
("demonstrates" nil (d eh1 m ax n s t r ey1 t s))
("demonstrating" nil (d eh1 m ax n s t r ey1 t ih0 ng))
("demonstration" nil (d eh1 m ax n s t r ey1 sh ax n))
("demonstrations" nil (d eh1 m ax n s t r ey1 sh ax n z))
("demonstrative" nil (d ih0 m aa1 n s t r ax t ih0 v))
("demonstrator" nil (d eh1 m ax n s t r ey1 t er0))
("demonstrators" nil (d eh1 m ax n s t r ey1 t er0 z))
("demont" nil (d eh1 m ax n t))
("demonte" nil (d ax m aa1 n t iy0))
("demopoulos" nil (d ax m aa1 p ax l ih0 s))
("demoralization" nil (d ih0 m ao1 r ax l ih0 z ey1 sh ax n))
("demoralize" nil (d ih0 m ao1 r ax l ay1 z))
("demoralized" nil (d ih0 m ao1 r ax l ay1 z d))
("demoralizing" nil (d ih0 m ao1 r ax l ay1 z ih0 ng))
("demore" nil (d eh1 m ao0 r))
("demorest" nil (d ey0 m ao1 r ih0 s t))
("demory" nil (d ih0 m er1 iy0))
("demos" nil (d eh1 m ow1 z))
("demoss" nil (d ax m aa1 s))
("demoted" nil (d ih0 m ow1 t ih0 d))
("demotion" nil (d ih0 m ow1 sh ax n))
("demott" nil (d ax m aa1 t))
("demoulin" nil (d eh1 m uw0 l ae0 n))
("demov" nil (d eh1 m aa0 v))
("dempewolf" nil (d eh1 m p y uw0 uh0 l f))
("demps" nil (d eh1 m p s))
("dempsey" nil (d eh1 m p s iy0))
("dempster" nil (d eh1 m p s t er0))
("demski" nil (d eh1 m s k iy0))
("demsky" nil (d eh1 m s k iy0))
("demur" nil (d ih0 m er1))
("demure" nil (d ih0 m y uh1 r))
("demurely" nil (d ih0 m y uh1 r l iy0))
("demuro" nil (d ih0 m uh1 r ow0))
("demurred" nil (d ih0 m er1 d))
("demurring" nil (d ih0 m er1 ih0 ng))
("demurs" nil (d ih0 m er1 z))
("demus" nil (d iy1 m ax s))
("demuth" nil (d iy1 m ax th))
("demyan" nil (d eh1 m y ax n))
("demystify" nil (d iy0 m ih1 s t ax f ay1))
("den" nil (d eh1 n))
("dena" nil (d iy1 n ax))
("denapoli" nil (d ih0 n aa1 p ax l iy0))
("denard" nil (d ih0 n aa1 r d))
("denardo" nil (d ih0 n aa1 r d ow0))
("denarii" nil (d ih0 n ae1 r iy0))
("denarius" nil (d ih0 n ae1 r iy0 ax s))
("denaro" nil (d ih0 n aa1 r ow0))
("denatale" nil (d ih0 n aa0 t aa1 l iy0))
("denationalization" nil (d iy1 n ae1 sh ax n ax l ih0 z ey1 sh ax n))
("denationalizations" nil (d iy0 n ae1 sh ax n ax l ih0 z ey1 sh ax n z))
("denationalize" nil (d ih0 n ae1 sh ax n ax l ay1 z))
("denationalized" nil (d ih0 n ae1 sh ax n ax l ay1 z d))
("denationalizing" nil (d ih0 n ae1 sh ax n ax l ay1 z ih0 ng))
("denatured" nil (d ih0 n ey1 ch er0 d))
("denault" nil (d ih0 n ow1))
("denbo" nil (d iy1 n b ow0))
("denboer" nil (d eh1 n b ow0 er0))
("denbow" nil (d eh1 n b ow0))
("denby" nil (d eh1 n b iy0))
("denden" nil (d eh1 n d ax n))
("dendinger" nil (d iy1 n d ih0 ng er0))
("dendritic" nil (d eh0 n d r ih1 t ih0 k))
("dendrochronology" nil (d eh1 n d r ow1 k r ax n aa1 l ax jh iy0))
("dendy" nil (d eh1 n d iy0))
("deneau" nil (d ih0 n ow1))
("deneault" nil (d ih0 n ow1))
("deneen" nil (d ih0 n iy1 n))
("deneke" nil (d eh1 n ih0 k))
("denenberg" nil (d eh1 n ax n b er0 g))
("denes" nil (d iy1 n z))
("deneve" nil (d eh1 n ih0 v))
("deng" nil (d eh1 ng))
("dengel" nil (d eh1 ng g ax l))
("dengler" nil (d ih1 ng ax l er0))
("denham" nil (d eh1 n ax m))
("denhart" nil (d eh1 n hh aa1 r t))
("denhartog" nil (d eh1 n hh aa0 r t ax g))
("denherder" nil (d eh1 n hh er1 d er0))
("denholm" nil (d eh1 n hh ow1 l m))
("deniability" nil (d ih0 n ay1 ax b ih1 l ih0 t iy0))
("denial" nil (d ih0 n ay1 ax l))
("denials" nil (d ih0 n ay1 ax l z))
("denice" nil (d ih0 n iy1 s))
("denicola" nil (d ih0 n iy0 k ow1 l ax))
("denied" nil (d ih0 n ay1 d))
("denies" nil (d ih0 n ay1 z))
("denigrate" nil (d eh1 n ax g r ey1 t))
("denigrated" nil (d eh1 n ih0 g r ey1 t ih0 d))
("denigrating" nil (d eh1 n ih0 g r ey1 t ih0 ng))
("denigris" nil (d eh1 n ih0 g r ih0 s))
("denike" nil (d eh1 n ih0 k))
("denim" nil (d eh1 n ax m))
("denio" nil (d iy1 n iy0 ow0))
("deniro" nil (d ih0 n ih1 r ow0))
("denis" nil (d eh1 n ih0 s))
("denise" nil (d ih0 n iy1 s))
("denison" nil (d eh1 n ih0 s ax n))
("deniston" nil (d eh1 n ih0 s t aa0 n))
("deniz" nil (d ey1 n iy0 z))
("denizens" nil (d eh1 n ax z ax n z))
("denk" nil (d eh1 ng k))
("denker" nil (d eh1 ng k er0))
("denki" nil (d eh1 ng k iy0))
("denkins" nil (d eh1 ng k ih0 n z))
("denko" nil (d eh1 ng k ow0))
("denktas" nil (d eh1 ng k t ax s))
("denlea" nil (d eh1 n l iy1))
("denley" nil (d eh1 n l iy0))
("denlinger" nil (d eh1 n ax l ih0 ng er0))
("denman" nil (d eh1 n m ax n))
("denmark" nil (d eh1 n m aa1 r k))
("denmon" nil (d eh1 n m ax n))
("denn" nil (d eh1 n))
("dennard" nil (d ih0 n aa1 r d))
("denne" nil (d eh1 n))
("dennehy" nil (d eh1 n ih0 hh iy0))
("dennen" nil (d eh1 n ax n))
("denner" nil (d eh1 n er0))
("dennett" nil (d eh1 n ih0 t))
("denney" nil (d eh1 n iy0))
("dennie" nil (d eh1 n iy0))
("dennin" nil (d eh1 n ih0 n))
("denning" nil (d eh1 n ih0 ng))
("denninger" nil (d eh1 n ih0 ng er0))
("dennington" nil (d eh1 n ih0 ng t ax n))
("dennis" nil (d eh1 n ih0 s))
("dennison" nil (d eh1 n ih0 s ax n))
("denniston" nil (d eh1 n ih0 s t aa0 n))
("denno" nil (d iy1 n ow0))
("denny" nil (d eh1 n iy0))
("deno" nil (d iy1 n ow0))
("denoble" nil (d eh1 n ow0 b ax l))
("denominated" nil (d ih0 n aa1 m ax n ey1 t ih0 d))
("denomination" nil (d ih0 n ao1 m ax n ey1 sh ax n))
("denominations" nil (d ih0 n ao1 m ax n ey1 sh ax n z))
("denominator" nil (d ih0 n aa1 m ax n ey1 t er0))
("denomme" nil (d eh1 n ax m))
("denote" nil (d ih0 n ow1 t))
("denoted" nil (d ih0 n ow1 t ax d))
("denotes" nil (d ih0 n ow1 t s))
("denouement" nil (d ey1 n uw1 m aa1 n))
("denounce" nil (d ih0 n aw1 n s))
("denounced" nil (d ih0 n aw1 n s t))
("denounces" nil (d ih0 n aw1 n s ih0 z))
("denouncing" nil (d ih0 n aw1 n s ih0 ng))
("denoyer" nil (d eh1 n oy0 er0))
("dens" nil (d eh1 n z))
("dense" nil (d eh1 n s))
("densely" nil (d eh1 n s l iy0))
("denser" nil (d eh1 n s er0))
("densest" nil (d eh1 n s ax s t))
("denshin" nil (d eh1 n sh ih0 n))
("densities" nil (d eh1 n s ax t iy0 z))
("densitometer" nil (d eh1 n s ax t aa1 m ax t er0))
("density" nil (d eh1 n s ax t iy0))
("densley" nil (d eh1 n s l iy0))
("denslow" nil (d eh1 n s l ow1))
("densmore" nil (d iy1 n s m ao0 r))
("denson" nil (d eh1 n s ax n))
("dent" nil (d eh1 n t))
("dental" nil (d eh1 n t ax l))
("dentals" nil (d eh1 n t ax l z))
("dente" nil (d eh1 n t))
("dented" nil (d eh1 n t ih0 d))
("dentin" nil (d eh1 n t ax n))
("dentine" nil (d eh1 n t iy0 n))
("denting" nil (d eh1 n t ih0 ng))
("dentino" nil (d ih0 n t iy1 n ow0))
("dentist" nil (d eh1 n t ax s t))
("dentistry" nil (d eh1 n t ax s t r iy0))
("dentists" nil (d eh1 n t ax s t s))
("dentition" nil (d eh0 n t ih1 sh ax n))
("dentler" nil (d eh1 n t l er0))
("denton" nil (d eh1 n t ax n))
("dentremont" nil (d ey0 n t r ey1 m ow0 n t))
("dents" nil (d eh1 n t s))
("dentsu" nil (d eh1 n t s uw0))
("dentton" nil (d eh1 n t ax n))
("denture" nil (d eh1 n ch er0))
("dentures" nil (d eh1 n ch er0 z))
("denucci" nil (d ih0 n uw1 ch iy0))
("denuclearization" nil (d iy0 n uw1 k l iy0 er0 ax z ey1 sh ax n))
("denuclearized" nil (d ih0 n uw1 k l iy0 er0 ay1 z d))
("denuded" nil (d ih0 n uw1 d ih0 d))
("denunciation" nil (d ih0 n ah1 n s iy0 ey1 sh ax n))
("denunciations" nil (d ih0 n ah1 n s iy0 ey1 sh ax n z))
("denunzio" nil (d ax n ah1 n z iy0 ow0))
("denver" nil (d eh1 n v er0))
("denwa" nil (d eh1 n w aa1))
("deny" nil (d ih0 n ay1))
("denying" nil (d ih0 n ay1 ih0 ng))
("denys" nil (d eh1 n ih0 s))
("denyse" nil (d eh1 n ay0 s))
("denz" nil (d eh1 n z))
("denzel" nil (d eh1 n z ax l))
("denzer" nil (d eh1 n z er0))
("denzil" nil (d eh1 n z ax l))
("denzler" nil (d eh1 n z l er0))
("deo" nil (d iy1 ow0))
("deodorant" nil (d iy0 ow1 d er0 ax n t))
("deodorants" nil (d iy0 ow1 d er0 ax n t s))
("deoliveira" nil (d iy1 aa1 l ih0 v ey1 r ax))
("deon" nil (d iy1 aa0 n))
("deoxyribonucleic" nil (d iy0 aa1 k s iy0 r ay1 b ow0 n uw0 k l ey1 ih0 k))
("dep" nil (d ih0 p aa1 r t m ax n t))
("depace" nil (d ih0 p aa1 ch iy0))
("depalma" nil (d ih0 p aa1 l m ax))
("depalo" nil (d ih0 p aa1 l ow0))
("depaola" nil (d ih0 p aw1 l ax))
("depaoli" nil (d ih0 p aw1 l iy0))
("depaolis" nil (d eh0 p aw1 l ih0 s))
("depaolo" nil (d ih0 p aw1 l ow0))
("depart" nil (d ih0 p aa1 r t))
("departed" nil (d ih0 p aa1 r t ax d))
("departing" nil (d ih0 p aa1 r t ih0 ng))
("department" nil (d ih0 p aa1 r t m ax n t))
("departmental" nil (d ih0 p aa1 r t m eh1 n t ax l))
("departmentalized" nil (d ih0 p aa1 r t m eh1 n t ax l ay1 z d))
("departments" nil (d ih0 p aa1 r t m ax n t s))
("departs" nil (d ih0 p aa1 r t s))
("departure" nil (d ih0 p aa1 r ch er0))
("departures" nil (d ih0 p aa1 r ch er0 z))
("depascale" nil (d ih0 p aa0 s k aa1 l iy0))
("depasquale" nil (d ih0 p aa0 s k w aa1 l iy0))
("depass" nil (d ih0 p ae1 s))
("depaul" nil (d ih0 p ao1 l))
("depaula" nil (d ih0 p ao1 l ax))
("depaulo" nil (d ih0 p ao1 l ow0))
("depauw" nil (d ax p aw1))
("depend" nil (d ih0 p eh1 n d))
("dependability" nil (d ih0 p eh1 n d ax b ih1 l ih0 t iy0))
("dependable" nil (d ih0 p eh1 n d ax b ax l))
("depended" nil (d ih0 p eh1 n d ax d))
("dependence" nil (d ih0 p eh1 n d ax n s))
("dependencies" nil (d ih0 p eh1 n d ax n s iy0 z))
("dependency" nil (d ih0 p eh1 n d ax n s iy0))
("dependent" nil (d ih0 p eh1 n d ax n t))
("dependents" nil (d ih0 p eh1 n d ax n t s))
("depending" nil (d ih0 p eh1 n d ih0 ng))
("depends" nil (d ih0 p eh1 n d z))
("depetro" nil (d ih0 p eh1 t r ow0))
("depew" nil (d ax p y uw1))
("dephillips" nil (d eh1 f ih0 l ih0 p s))
("depict" nil (d ih0 p ih1 k t))
("depicted" nil (d ih0 p ih1 k t ax d))
("depicting" nil (d ih0 p ih1 k t ih0 ng))
("depiction" nil (d ih0 p ih1 k sh ax n))
("depictions" nil (d ih0 p ih1 k sh ax n z))
("depicts" nil (d ih0 p ih1 k t s))
("depietro" nil (d ih0 p iy1 t r ow0))
("depilatory" nil (d ih0 p ih1 l ax t ao1 r iy0))
("depina" nil (d ih0 p iy1 n ax))
("depinto" nil (d ih0 p iy1 n t ow0))
("deplete" nil (d ih0 p l iy1 t))
("depleted" nil (d ih0 p l iy1 t ih0 d))
("depleters" nil (d ih0 p l iy1 t er0 z))
("depletes" nil (d ih0 p l iy1 t s))
("depleting" nil (d ih0 p l iy1 t ih0 ng))
("depletion" nil (d ih0 p l iy1 sh ax n))
("deplorable" nil (d ih0 p l ao1 r ax b ax l))
("deplore" nil (d ih0 p l ao1 r))
("deplored" nil (d ih0 p l ao1 r d))
("deplores" nil (d ih0 p l ao1 r z))
("deploring" nil (d ih0 p l ao1 r ih0 ng))
("deploy" nil (d ih0 p l oy1))
("deployable" nil (d ih0 p l oy1 ax b ax l))
("deployed" nil (d ih0 p l oy1 d))
("deploying" nil (d ih0 p l oy1 ih0 ng))
("deployment" nil (d ih0 p l oy1 m ax n t))
("deployments" nil (d ih0 p l oy1 m ax n t s))
("deploys" nil (d ih0 p l oy1 z))
("depner" nil (d eh1 p n er0))
("depo" nil (d iy1 p ow0))
("depolo" nil (d ih0 p ow1 l ow0))
("deponte" nil (d ih0 p ow1 n t iy0))
("depopulation" nil (d ih0 p aa1 p y ax l ey1 sh ax n))
("deport" nil (d ih0 p ao1 r t))
("deportation" nil (d iy1 p ao0 r t ey1 sh ax n))
("deportations" nil (d iy1 p ao0 r t ey1 sh ax n z))
("deported" nil (d ih0 p ao1 r t ax d))
("deportees" nil (d iy1 p ao0 r t iy1 z))
("deporting" nil (d ih0 p ao1 r t ih0 ng))
("deportment" nil (d ax p ao1 r t m ax n t))
("depose" nil (d ih0 p ow1 z))
("deposed" nil (d ih0 p ow1 z d))
("deposit" nil (d ax p aa1 z ih0 t))
("depositary" nil (d ax p aa1 z ih0 t eh1 r iy0))
("deposited" nil (d ax p aa1 z ih0 t ih0 d))
("depositing" nil (d ax p aa1 z ih0 t ih0 ng))
("deposition" nil (d eh1 p ax z ih1 sh ax n))
("depositional" nil (d eh1 p ax z ih1 sh ax n ax l))
("depositions" nil (d eh1 p ax z ih1 sh ax n z))
("depositor" nil (d ax p aa1 z ih0 t er0))
("depositors" nil (d ax p aa1 z ih0 t er0 z))
("depository" nil (d ih0 p aa1 z ax t ao1 r iy0))
("deposits" nil (d ax p aa1 z ih0 t s))
("depot" nil (d iy1 p ow0))
("depots" nil (d iy1 p ow0 z))
("depoy" nil (d eh1 p oy0))
("depp" nil (d eh1 p))
("deppe" nil (d eh1 p))
("deppen" nil (d eh1 p ax n))
("depravity" nil (d ih0 p r ae1 v ax t iy0))
("deprecating" nil (d eh1 p r ax k ey1 t ih0 ng))
("depreciable" nil (d ih0 p r ih1 sh ax b ax l))
("depreciate" nil (d ih0 p r iy1 sh iy0 ey1 t))
("depreciated" nil (d ih0 p r iy1 sh iy0 ey1 t ih0 d))
("depreciates" nil (d ih0 p r iy1 sh iy0 ey1 t s))
("depreciating" nil (d ih0 p r iy1 sh iy0 ey1 t ih0 ng))
("depreciation" nil (d ih0 p r iy1 sh iy0 ey1 sh ax n))
("depredation" nil (d eh1 p r ax d ey1 sh ax n))
("depredations" nil (d eh0 p r ax d ey1 sh ax n z))
("depree" nil (d ih0 p r iy1))
("deprenyl" nil (d eh1 p r ax n ih1 l))
("depress" nil (d ih0 p r eh1 s))
("depressant" nil (d ih0 p r eh1 s ax n t))
("depressants" nil (d ih0 p r eh1 s ax n t s))
("depressed" nil (d ih0 p r eh1 s t))
("depresses" nil (d ih0 p r eh1 s ax z))
("depressing" nil (d ih0 p r eh1 s ih0 ng))
("depressingly" nil (d ih0 p r eh1 s ih0 ng l iy0))
("depression" nil (d ih0 p r eh1 sh ax n))
("depressions" nil (d ih0 p r eh1 sh ax n z))
("depressive" nil (d ih0 p r eh1 s ih0 v))
("deprey" nil (d eh1 p r iy0))
("deprez" nil (d ey0 p r eh1 z))
("depriest" nil (d eh1 p er0 iy0 ih0 s t))
("deprivation" nil (d eh1 p r ax v ey1 sh ax n))
("deprivations" nil (d eh1 p r ax v ey1 sh ax n z))
("deprive" nil (d ih0 p r ay1 v))
("deprived" nil (d ih0 p r ay1 v d))
("deprives" nil (d ih0 p r ay1 v z))
("depriving" nil (d ih0 p r ay1 v ih0 ng))
("depth" nil (d eh1 p th))
("depths" nil (d eh1 p th s))
("deptula" nil (d ih0 p t uw1 l ax))
("depue" nil (d ax p y uw1))
("deputies" nil (d eh1 p y ax t iy0 z))
("deputy" nil (d eh1 p y ax t iy0))
("depuy" nil (d ih0 p ay1))
("dequeker" nil (d ih0 k w eh1 k er0))
("der" nil (d er1))
("derail" nil (d ih0 r ey1 l))
("derailed" nil (d ih0 r ey1 l d))
("derailing" nil (d ih0 r ey1 l ih0 ng))
("derailment" nil (d ih0 r ey1 l m ax n t))
("derailments" nil (d ih0 r ey1 l m ax n t s))
("derails" nil (d ih0 r ey1 l z))
("deramo" nil (d ih0 r aa1 m ow0))
("deramus" nil (d eh1 r ax m ih0 s))
("deranged" nil (d ih0 r ey1 n jh d))
("derasmo" nil (d ih0 r aa1 s m ow0))
("derby" nil (d er1 b iy0))
("derchin" nil (d er1 ch ih0 n))
("dercole" nil (d ih0 r k ow1 l iy0))
("derden" nil (d er1 d ax n))
("derderian" nil (d er0 d ih1 r iy0 ax n))
("derecktor" nil (d er0 eh1 k t er0))
("deregt" nil (d er0 eh1 k t))
("deregulate" nil (d iy0 r eh1 g y ax l ey0 t))
("deregulated" nil (d iy0 r eh1 g y ax l ey0 t ih0 d))
("deregulating" nil (d iy0 r eh1 g y ax l ey1 t ih0 ng))
("deregulation" nil (d iy0 r eh1 g y ax l ey1 sh ax n))
("deregulators" nil (d iy0 r eh1 g y ax l ey0 t er0 z))
("deregulatory" nil (d iy0 r eh1 g y ax l ax t ao1 r iy0))
("derek" nil (d eh1 r ih0 k))
("derelict" nil (d eh1 r ax l ih1 k t))
("dereliction" nil (d eh1 r ax l ih1 k sh ax n))
("deremer" nil (d eh1 r iy0 m er0))
("deren" nil (d ih1 r ax n))
("derenzo" nil (d ih0 r eh1 n z ow0))
("derflinger" nil (d er1 f ax l ih0 ng er0))
("derham" nil (d er1 hh ax m))
("derhammer" nil (d er1 hh ax m er0))
("derick" nil (d eh1 r ih0 k))
("derickson" nil (d eh1 r ih0 k s ax n))
("deridder" nil (d eh1 r ih0 d er0))
("deride" nil (d ih0 r ay1 d))
("derided" nil (d ih0 r ay1 d ih0 d))
("derides" nil (d ih0 r ay1 d z))
("deriding" nil (d ih0 r ay1 d ih0 ng))
("derienzo" nil (d ih0 r iy1 n z ow0))
("derik" nil (d eh1 r ih0 k))
("dering" nil (d ih1 r ih0 ng))
("deringer" nil (d eh1 r ih0 n jh er0))
("derington" nil (d er1 ih0 ng t ax n))
("derise" nil (d eh1 r ay0 z))
("derision" nil (d er0 ih1 zh ax n))
("derisive" nil (d er0 ih1 s ih0 v))
("derisively" nil (d er0 ih1 s ih0 v l iy0))
("deriso" nil (d ih0 r iy1 s ow0))
("derivation" nil (d eh1 r ax v ey1 sh ax n))
("derivative" nil (d er0 ih1 v ax t ih0 v))
("derivatives" nil (d er0 ih1 v ax t ih0 v z))
("derive" nil (d er0 ay1 v))
("derived" nil (d er0 ay1 v d))
("derives" nil (d er0 ay1 v z))
("deriving" nil (d er0 ay1 v ih0 ng))
("derk" nil (d er1 k))
("derks" nil (d er1 k s))
("derksen" nil (d er1 k s ax n))
("derleth" nil (d er1 l ih0 th))
("derma" nil (d er1 m ax))
("dermal" nil (d er1 m ax l))
("derman" nil (d er1 m ax n))
("dermatological" nil (d er1 m ax t ax l aa1 jh ih0 k ax l))
("dermatologist" nil (d er1 m ax t aa1 l ax jh ih0 s t))
("dermatologists" nil (d er1 m ax t aa1 l ax jh ih0 s t s))
("dermatology" nil (d er1 m ax t aa1 l ax jh iy0))
("dermer" nil (d er1 m er0))
("dermis" nil (d er1 m ax s))
("dermody" nil (d er1 m ax d iy0))
("dermot" nil (d er1 m ax t))
("dermott" nil (d er1 m ax t))
("dern" nil (d er1 n))
("derner" nil (d er1 n er0))
("derobertis" nil (d eh1 r ax b er0 t ih0 s))
("derocco" nil (d ih0 r ow1 k ow0))
("deroche" nil (d eh1 r ax k))
("derocher" nil (d eh1 r ax k er0))
("derogatory" nil (d er0 aa1 g ax t ao1 r iy0))
("deroo" nil (d eh1 r uw0))
("deroos" nil (d ih1 r uw0 z))
("derosa" nil (d ih0 r ow1 s ax))
("derose" nil (d eh1 r ax s))
("derosia" nil (d ih0 r ow1 s iy0 ax))
("derosier" nil (d eh1 r ax s iy0 er0))
("derossett" nil (d eh1 r ax s eh0 t))
("derouen" nil (d er0 w eh1 n))
("derouin" nil (d er0 w iy1 n))
("derousse" nil (d er0 uw1 s))
("deroy" nil (d ih1 r oy0))
("derr" nil (d eh1 r))
("derrick" nil (d eh1 r ih0 k))
("derrickson" nil (d eh1 r ih0 k s ax n))
("derrico" nil (d ih0 r iy1 k ow0))
("derrig" nil (d eh1 r ih0 g))
("derring" nil (d eh1 r ih0 ng))
("derringer" nil (d eh1 r ax n jh er0))
("derrington" nil (d eh1 r ih0 ng t ax n))
("derrow" nil (d eh1 r ow0))
("derry" nil (d eh1 r iy0))
("derryberry" nil (d eh1 r iy0 b eh1 r iy0))
("dersch" nil (d er1 sh))
("dershem" nil (d er1 sh ih0 m))
("dershowitz" nil (d er1 sh ax w ih1 t s))
("derstine" nil (d er1 s t iy0 n))
("derthick" nil (d er1 th ih0 k))
("derubeis" nil (d eh1 r ax b ay0 z))
("deruiter" nil (d ih1 r ih0 t er0))
("deruko" nil (d ih0 r uw1 k ow0))
("derus" nil (d eh1 r ih0 s))
("derusha" nil (d eh1 r ax sh ax))
("deruyter" nil (d ih1 r ay0 t er0))
("dervin" nil (d er1 v ih0 n))
("dervish" nil (d er1 v ih0 sh))
("derward" nil (d er1 w er0 d))
("derwin" nil (d er1 w ih0 n))
("derwinski" nil (d er0 w ih1 n s k iy0))
("dery" nil (d eh1 r iy0))
("deryck" nil (d eh1 r ih0 k))
("deryle" nil (d eh1 r ax l))
("des" nil (d eh1))
("desai" nil (d ey0 s aa1 iy0))
("desalination" nil (d iy0 s ey1 l ih0 n ey1 sh ax n))
("desalinization" nil (d iy0 s ey1 l ih0 n ax z ey1 sh ax n))
("desalvo" nil (d ih0 s aa1 l v ow0))
("desanctis" nil (d eh1 s ax ng k t ih0 s))
("desanti" nil (d ih0 s aa1 n t iy0))
("desantiago" nil (d ih0 s aa0 n t iy0 aa1 g ow0))
("desantis" nil (d ey0 s aa1 n t ih0 s))
("desanto" nil (d ih0 s aa1 n t ow0))
("desantos" nil (d ey0 s aa1 n t ow0 z))
("desaulniers" nil (d eh1 s ow0 l n iy0 er0 z))
("desautel" nil (d eh1 s ow0 t ax l))
("desautels" nil (d eh1 s ow0 t ax l z))
("descarpentries" nil (d ey0 k aa1 r p ax n t r iy0 z))
("descartes" nil (d ey0 k aa1 r t))
("descend" nil (d ih0 s eh1 n d))
("descendant" nil (d ih0 s eh1 n d ax n t))
("descendants" nil (d ih0 s eh1 n d ax n t s))
("descended" nil (d ih0 s eh1 n d ax d))
("descendents" nil (d ih0 s eh1 n d ax n t s))
("descending" nil (d ih0 s eh1 n d ih0 ng))
("descends" nil (d ih0 s eh1 n d z))
("descent" nil (d ih0 s eh1 n t))
("descenza" nil (d eh0 sh eh1 n z ax))
("desch" nil (d eh1 sh))
("deschaine" nil (d ih0 s k ey1 n))
("deschamps" nil (d eh1 sh ax m p s))
("deschene" nil (d eh1 sh iy0 n))
("deschenes" nil (d eh1 sh iy0 n z))
("deschepper" nil (d eh1 sh ih0 p er0))
("deschler" nil (d eh1 sh ax l er0))
("deschner" nil (d eh1 sh n er0))
("descoteaux" nil (d eh1 s k ax t ow0))
("describable" nil (d ih0 s k r ay1 b ax b ax l))
("describe" nil (d ih0 s k r ay1 b))
("described" nil (d ih0 s k r ay1 b d))
("describes" nil (d ih0 s k r ay1 b z))
("describing" nil (d ih0 s k r ay1 b ih0 ng))
("description" nil (d ih0 s k r ih1 p sh ax n))
("descriptions" nil (d ih0 s k r ih1 p sh ax n z))
("descriptive" nil (d ih0 s k r ih1 p t ih0 v))
("desecration" nil (d eh0 s ax k r ey1 sh ax n))
("desegregate" nil (d ih0 s eh1 g r ax g ey1 t))
("desegregated" nil (d ih0 s eh1 g r ih0 g ey1 t ih0 d))
("desegregation" nil (d ih0 s eh1 g r ax g ey1 sh ax n))
("desena" nil (d ih0 s eh1 n ax))
("desensitize" nil (d ih0 s eh1 n s ax t ay1 z))
("deseret" nil (d eh1 s er0 eh1 t))
("desert" n (d eh1 z er0 t))
("desert" v (d ih0 z er1 t))
("deserted" nil (d ih0 z er1 t ih0 d))
("deserter" nil (d eh1 z er0 t er0))
("deserters" nil (d eh1 z er0 t er0 z))
("deserting" nil (d eh1 z er0 t ih0 ng))
("desertion" nil (d ih0 z er1 sh ax n))
("desertions" nil (d ih0 z er1 sh ax n z))
("deserts" n (d eh1 z er0 t s))
("deserts" v (d ih0 z er1 t s))
("deserve" nil (d ih0 z er1 v))
("deserved" nil (d ih0 z er1 v d))
("deservedly" nil (d ih0 z er1 v ax d l iy0))
("deserves" nil (d ih0 z er1 v z))
("deserving" nil (d ih0 z er1 v ih0 ng))
("desha" nil (d eh1 sh ax))
("deshaies" nil (d ih0 sh ey1 z))
("deshane" nil (d eh1 sh ax n))
("deshaw" nil (d eh1 sh ao0))
("deshazer" nil (d eh1 sh ax z er0))
("deshazo" nil (d ey0 sh aa1 z ow0))
("deshields" nil (d eh1 sh iy0 l d z))
("deshler" nil (d eh1 sh l er0))
("deshon" nil (d eh1 sh ax n))
("deshong" nil (d eh1 sh ao0 ng))
("deshotel" nil (d eh1 sh ax t ax l))
("deshotels" nil (d eh1 sh ax t ax l z))
("desiccation" nil (d eh1 s ax k ey1 sh ax n))
("desiderio" nil (d ih0 s iy0 d eh1 r iy0 ow0))
("design" nil (d ih0 z ay1 n))
("designate" nil (d eh1 z ax g n ey1 t))
("designated" nil (d eh1 z ih0 g n ey1 t ih0 d))
("designates" nil (d eh1 z ih0 g n ey1 t s))
("designating" nil (d eh1 z ih0 g n ey1 t ih0 ng))
("designation" nil (d eh1 z ax g n ey1 sh ax n))
("designations" nil (d eh1 z ax g n ey1 sh ax n z))
("designcraft" nil (d ih0 z ay1 n k r ae1 f t))
("designed" nil (d ih0 z ay1 n d))
("designees" nil (d eh1 z ih0 g n iy1 z))
("designer" nil (d ih0 z ay1 n er0))
("designers" nil (d ih0 z ay1 n er0 z))
("designing" nil (d ih0 z ay1 n ih0 ng))
("designs" nil (d ih0 z ay1 n z))
("desilets" nil (d eh1 s ih0 l ih0 t s))
("desillers" nil (d ih0 s ih1 l er0 z))
("desilva" nil (d ih0 s iy1 l v ax))
("desimone" nil (d ih0 s iy0 m ow1 n iy0))
("desio" nil (d iy1 s iy0 ow0))
("desir" nil (d ih0 s ih1 r))
("desirability" nil (d ih0 z ay1 r ax b ih1 l ih0 t iy0))
("desirable" nil (d ih0 z ay1 r ax b ax l))
("desire" nil (d ih0 z ay1 er0))
("desired" nil (d ih0 z ay1 er0 d))
("desiree" nil (d eh1 s ay0 r iy0))
("desires" nil (d ih0 z ay1 er0 z))
("desiring" nil (d ih0 z ay1 er0 ih0 ng))
("desist" nil (d ih0 s ih1 s t))
("desisto" nil (d ih0 s iy1 s t ow0))
("desjardin" nil (d eh1 s zh aa0 r d ae0 n))
("desjardins" nil (d eh1 s zh aa0 r d ih0 n z))
("desjarlais" nil (d eh1 s zh aa0 r l ey0))
("desk" nil (d eh1 s k))
("deskin" nil (d eh1 s k ih0 n))
("deskins" nil (d eh1 s k ih0 n z))
("deskjet" nil (d eh1 s k jh eh1 t))
("deskpro" nil (d eh1 s k p r ow1))
("desks" nil (d eh1 s k s))
("desktop" nil (d eh1 s k t aa1 p))
("desktops" nil (d eh1 s k t aa1 p s))
("deslatte" nil (d ih0 s l ae1 t))
("deslauriers" nil (d eh1 s l ao0 r iy0 er0 z))
("desma" nil (d iy1 s m ax))
("desman" nil (d eh1 s m ax n))
("desmarais" nil (d eh1 z m er0 ey0))
("desmet" nil (d eh1 s m ih0 t))
("desmids" nil (d eh1 s m ax d z))
("desmith" nil (d eh1 s m ih0 th))
("desmona" nil (d ih0 s m ow1 n ax))
("desmond" nil (d eh1 z m ax n d))
("desnoyers" nil (d eh1 s n oy0 er0 z))
("desolate" nil (d eh1 s ax l ax t))
("desolation" nil (d eh1 s ax l ey1 sh ax n))
("desormeaux" nil (d eh1 s er0 m ow0))
("desoto" nil (d ih0 s ow1 t ow0))
("desousa" nil (d ih0 s aw1 s ax))
("desouza" nil (d ey0 s uw1 z ax))
("despain" nil (d ih0 s p ey1 n))
("despair" nil (d ih0 s p eh1 r))
("despaired" nil (d ih0 s p eh1 r d))
("despairing" nil (d ih0 s p eh1 r ih0 ng))
("despairs" nil (d ih0 s p eh1 r z))
("desper" nil (d eh1 s p er0))
("desperadoes" nil (d eh1 s p er0 aa1 d ow0 z))
("desperate" nil (d eh1 s p r ih0 t))
("desperately" nil (d eh1 s p er0 ax t l iy0))
("desperation" nil (d eh1 s p er0 ey1 sh ax n))
("despicable" nil (d ih0 s p ih1 k ax b ax l))
("despise" nil (d ih0 s p ay1 z))
("despised" nil (d ih0 s p ay1 z d))
("despises" nil (d ih0 s p ay1 z ih0 z))
("despite" nil (d ih0 s p ay1 t))
("despondency" nil (d ih0 s p aa1 n d ax n s iy0))
("despondent" nil (d ih0 s p aa1 n d ax n t))
("desposito" nil (d ih0 s p ow0 s iy1 t ow0))
("despot" nil (d eh1 s p ax t))
("despotic" nil (d ih0 s p aa1 t ih0 k))
("despotism" nil (d eh1 s p ax t ih1 z ax m))
("despres" nil (d eh1 s p er0 z))
("desrochers" nil (d ey0 r ow1 sh er0 z))
("desroches" nil (d ey0 r ow1 sh ih0 z))
("desrosier" nil (d ey0 r ow1 sh iy0 er0))
("desrosiers" nil (d ey0 r ow1 sh iy0 er0 z))
("dessauer" nil (d ih0 s aw1 r))
("desselle" nil (d ih0 s eh1 l))
("dessent" nil (d ih0 s eh1 n t))
("dessert" nil (d ih0 z er1 t))
("desserts" nil (d ih0 z er1 t s))
("dest" nil (d eh1 s t))
("destabilization" nil (d iy0 s t ey1 b ax l ax z ey1 sh ax n))
("destabilize" nil (d ih0 s t ey1 b ax l ay1 z))
("destabilized" nil (d ih0 s t ey1 b ax l ay1 z d))
("destabilizing" nil (d ih0 s t ey1 b ax l ay1 z ih0 ng))
("destec" nil (d eh1 s t eh1 k))
("destefanis" nil (d eh1 s t ih0 f ax n ih0 s))
("destefano" nil (d ih0 s t eh0 f aa1 n ow0))
("destination" nil (d eh1 s t ax n ey1 sh ax n))
("destinations" nil (d eh1 s t ax n ey1 sh ax n z))
("destined" nil (d eh1 s t ih0 n d))
("destinies" nil (d eh1 s t ax n iy0 z))
("destiny" nil (d eh1 s t ax n iy0))
("destitute" nil (d eh1 s t ax t uw1 t))
("destroy" nil (d ih0 s t r oy1))
("destroyed" nil (d ih0 s t r oy1 d))
("destroyer" nil (d ih0 s t r oy1 er0))
("destroyers" nil (d ih0 s t r oy1 er0 z))
("destroying" nil (d ih0 s t r oy1 ih0 ng))
("destroys" nil (d ih0 s t r oy1 z))
("destruct" nil (d ih0 s t r ah1 k t))
("destruction" nil (d ih0 s t r ah1 k sh ax n))
("destructive" nil (d ih0 s t r ah1 k t ih0 v))
("desultory" nil (d eh1 s ax l t ao1 r iy0))
("detach" nil (d ih0 t ae1 ch))
("detachable" nil (d ih0 t ae1 ch ax b ax l))
("detached" nil (d ih0 t ae1 ch t))
("detachment" nil (d ih0 t ae1 ch m ax n t))
("detail" nil (d ih0 t ey1 l))
("detailed" nil (d ih0 t ey1 l d))
("detailing" nil (d ih0 t ey1 l ih0 ng))
("details" nil (d ih0 t ey1 l z))
("detain" nil (d ih0 t ey1 n))
("detained" nil (d ih0 t ey1 n d))
("detainee" nil (d iy1 t ey0 n iy1))
("detainees" nil (d ih0 t ey1 n iy1 z))
("detaining" nil (d ih0 t ey1 n ih0 ng))
("detamore" nil (d ih0 t aa1 m ao0 r))
("detar" nil (d ih0 t aa1 r))
("detect" nil (d ih0 t eh1 k t))
("detectable" nil (d ih0 t eh1 k t ax b ax l))
("detected" nil (d ih0 t eh1 k t ax d))
("detecting" nil (d ih0 t eh1 k t ih0 ng))
("detection" nil (d ih0 t eh1 k sh ax n))
("detective" nil (d ih0 t eh1 k t ih0 v))
("detectives" nil (d ih0 t eh1 k t ih0 v z))
("detector" nil (d ih0 t eh1 k t er0))
("detectors" nil (d ih0 t eh1 k t er0 z))
("detects" nil (d ih0 t eh1 k t s))
("detemple" nil (d eh1 t ih0 m p ax l))
("detente" nil (d ey0 t aa1 n t))
("detention" nil (d ih0 t eh1 n sh ax n))
("detentions" nil (d ih0 t eh1 n sh ax n z))
("deter" nil (d ih0 t er1))
("deterding" nil (d eh1 t er0 d ih0 ng))
("detergent" nil (d ih0 t er1 jh ax n t))
("detergents" nil (d ih0 t er1 jh ax n t s))
("deteriorate" nil (d ih0 t ih1 r iy0 er0 ey1 t))
("deteriorated" nil (d ih0 t ih1 r iy0 er0 ey1 t ih0 d))
("deteriorates" nil (d ih0 t ih1 r iy0 er0 ey1 t s))
("deteriorating" nil (d ih0 t ih1 r iy0 er0 ey1 t ih0 ng))
("deterioration" nil (d ih0 t ih1 r iy0 er0 ey1 sh ax n))
("determan" nil (d iy1 t er0 m ax n))
("determent" nil (d ih0 t er1 m ax n t))
("determinable" nil (d ih0 t er1 m ax n ax b ax l))
("determinant" nil (d ih0 t er1 m ax n ax n t))
("determinants" nil (d ih0 t er1 m ax n ax n t s))
("determinate" nil (d ih0 t er1 m ax n ey1 t))
("determination" nil (d ih0 t er1 m ax n ey1 sh ax n))
("determinations" nil (d ax t er1 m ih0 n ey1 sh ax n z))
("determine" nil (d ax t er1 m ax n))
("determined" nil (d ih0 t er1 m ax n d))
("determinedly" nil (d ax t er1 m ax n ax d l iy0))
("determines" nil (d ax t er1 m ax n z))
("determining" nil (d ih0 t er1 m ax n ih0 ng))
("determinism" nil (d ih0 t er1 m ax n ih1 z ax m))
("determinist" nil (d ih0 t er1 m ax n ax s t))
("deterministic" nil (d ih0 t er1 m ax n ih1 s t ih0 k))
("deterred" nil (d ih0 t er1 d))
("deterrence" nil (d ih0 t er1 ax n s))
("deterrent" nil (d ih0 t er1 r ax n t))
("deterrents" nil (d ih0 t er1 ax n t s))
("deterring" nil (d ih0 t er1 ih0 ng))
("deters" nil (d ih0 t er1 z))
("detert" nil (d eh1 t er0 t))
("detest" nil (d ih0 t eh1 s t))
("detested" nil (d ih0 t eh1 s t ih0 d))
("detherage" nil (d eh1 dh er0 ih0 jh))
("dethlefs" nil (d eh1 th l ih0 f s))
("dethlefsen" nil (d eh1 th l ih0 f s ax n))
("dethloff" nil (d eh1 th l ao0 f))
("dethomas" nil (d ih0 th ow1 m ax z))
("dethrone" nil (d ih0 th r ow1 n))
("dethroned" nil (d ih0 th r ow1 n d))
("detienne" nil (d eh1 t iy0 eh0 n))
("detjen" nil (d eh1 t jh ax n))
("detlefsen" nil (d eh1 t l ih0 f s ax n))
("detloff" nil (d eh1 t l ao0 f))
("detmer" nil (d eh1 t m er0))
("detonate" nil (d eh1 t ax n ey1 t))
("detonated" nil (d eh1 t ax n ey1 t ax d))
("detonating" nil (d eh1 t ax n ey1 t ih0 ng))
("detonation" nil (d eh1 t ax n ey1 sh ax n))
("detore" nil (d ih0 t ao1 r iy0))
("detour" nil (d ih0 t uh1 r))
("detox" nil (d iy1 t aa1 k s))
("detoxication" nil (d ih0 t aa1 k s ax k ey1 sh ax n))
("detoxification" nil (d ih0 t aa1 k s ih0 f ih0 k ey1 sh ax n))
("detoxify" nil (d ih0 t aa1 k s ax f ay1))
("detract" nil (d ih0 t r ae1 k t))
("detracted" nil (d ih0 t r ae1 k t ih0 d))
("detracting" nil (d ih0 t r ae1 k t ih0 ng))
("detractors" nil (d ih0 t r ae1 k t er0 z))
("detracts" nil (d ih0 t r ae1 k t s))
("detrich" nil (d eh1 t r ih0 k))
("detrick" nil (d eh1 t r ih0 k))
("detriment" nil (d eh1 t r ax m ax n t))
("detrimental" nil (d eh1 t r ax m eh1 n t ax l))
("detritus" nil (d ih0 t r ay1 t ax s))
("detro" nil (d iy1 t r ow0))
("detroit" nil (d ih0 t r oy1 t))
("detroiters" nil (d ax t r oy1 t er0 z))
("detter" nil (d eh1 t er0))
("dettinger" nil (d eh1 t ih0 n jh er0))
("dettling" nil (d eh1 t l ih0 ng))
("dettloff" nil (d eh1 t l ao0 f))
("dettman" nil (d eh1 t m ax n))
("dettmann" nil (d eh1 t m ax n))
("dettmer" nil (d eh1 t m er0))
("dettore" nil (d ih0 t ao1 r iy0))
("detty" nil (d eh1 t iy0))
("deturk" nil (d eh1 t er0 k))
("detweiler" nil (d eh1 t w ay1 l er0))
("detwiler" nil (d eh1 t w ay1 l er0))
("detzel" nil (d eh1 t z ax l))
("deubel" nil (d oy1 b ax l))
("deubler" nil (d oy1 b ax l er0))
("deuce" nil (d uw1 s))
("deuel" nil (d uw1 eh0 l))
("deukmejian" nil (d uw0 k m ey1 jh iy0 ax n))
("deupree" nil (d uw0 p r iy1))
("deuschle" nil (d oy1 sh ax l))
("deuser" nil (d oy1 s er0))
("deuss" nil (d uw1 s))
("deutch" nil (d oy1 ch))
("deuterium" nil (d uw0 t iy1 r iy0 ax m))
("deuterons" nil (d uw1 t er0 aa1 n z))
("deutsch" nil (d oy1 ch))
("deutsche" nil (d oy1 ch))
("deutschemark" nil (d oy1 ch m aa1 r k))
("deutschemarks" nil (d oy1 ch m aa1 r k s))
("deutscher" nil (d oy1 ch er0))
("deutschland" nil (d oy1 ch l ae0 n d))
("deutschman" nil (d oy1 ch m ax n))
("deutz" nil (d oy1 t s))
("deux" nil (d uw1))
("deva" nil (d ey1 v ax))
("deval" nil (d ih0 v aa1 l))
("devall" nil (d ey0 v aa1 l))
("devaluated" nil (d ih0 v ae1 l y uw0 ey1 t ih0 d))
("devaluation" nil (d ih0 v ae1 l y uw0 ey1 sh ax n))
("devaluations" nil (d iy1 v ae0 l y uw0 ey1 sh ax n z))
("devalue" nil (d ih0 v ae1 l y uw1))
("devalued" nil (d ih0 v ae1 l y uw1 d))
("devaluing" nil (d ih0 v ae1 l y uw0 ih0 ng))
("devan" nil (d eh1 v ax n))
("devane" nil (d ih0 v ey1 n))
("devaney" nil (d eh1 v ax n iy0))
("devany" nil (d eh1 v ax n iy0))
("devastate" nil (d eh1 v ax s t ey1 t))
("devastated" nil (d eh1 v ax s t ey1 t ax d))
("devastating" nil (d eh1 v ax s t ey1 t ih0 ng))
("devastatingly" nil (d eh1 v ax s t ey1 t ih0 ng l iy0))
("devastation" nil (d eh1 v ax s t ey1 sh ax n))
("devaughn" nil (d eh1 v ao0 n))
("devaul" nil (d ih0 v ow1 l))
("devault" nil (d ih0 v ow1))
("devaux" nil (d ih0 v ow1))
("deveau" nil (d ih0 v ow1))
("deveaux" nil (d ih0 v ow1))
("develcon" nil (d ax v eh1 l k ax n))
("develle" nil (d ax v eh1 l))
("develop" nil (d ih0 v eh1 l ax p))
("developable" nil (d ih0 v eh1 l ax p ax b ax l))
("developed" nil (d ih0 v eh1 l ax p t))
("developer" nil (d ih0 v eh1 l ax p er0))
("developers" nil (d ih0 v eh1 l ax p er0 z))
("developing" nil (d ih0 v eh1 l ax p ih0 ng))
("development" nil (d ih0 v eh1 l ax p m ax n t))
("developmental" nil (d ih0 v eh1 l ax p m eh1 n t ax l))
("developments" nil (d ih0 v eh1 l ax p m ax n t s))
("develops" nil (d ih0 v eh1 l ax p s))
("devendorf" nil (d eh1 v ih0 n d ao0 r f))
("deveney" nil (d eh1 v ih0 n iy0))
("devenney" nil (d eh1 v ih0 n iy0))
("devenny" nil (d eh1 v ih0 n iy0))
("devenport" nil (d ih0 v eh1 n p ao0 r t))
("devens" nil (d iy1 v ax n z))
("deveny" nil (d ih0 v iy1 n iy0))
("dever" nil (d iy1 v er0))
("devera" nil (d ey0 v eh1 r ax))
("deveraux" nil (d eh1 v er0 ow0))
("devere" nil (d ey0 v eh1 r ey0))
("devereaux" nil (d eh1 v er0 ow0))
("deverell" nil (d ey0 v eh0 r ey1 l))
("devereux" nil (d eh1 v er0 uw1))
("devers" nil (d eh1 v er0 z))
("devey" nil (d ih0 v ey1))
("deviance" nil (d iy1 v iy0 ax n s))
("deviant" nil (d iy1 v iy0 ax n t))
("deviants" nil (d iy1 v iy0 ax n t s))
("deviate" nil (d iy1 v iy0 ey1 t))
("deviated" nil (d iy1 v iy0 ey1 t ih0 d))
("deviates" nil (d iy1 v iy0 ey1 t s))
("deviation" nil (d iy1 v iy0 ey1 sh ax n))
("deviations" nil (d iy0 v iy0 ey1 sh ax n z))
("device" nil (d ih0 v ay1 s))
("devices" nil (d ih0 v ay1 s ax z))
("devil" nil (d eh1 v ax l))
("devilbiss" nil (d eh1 v ih0 l b ih0 s))
("devilish" nil (d eh1 v l ih0 sh))
("devilishly" nil (d eh1 v ax l ih0 sh l iy0))
("deville" nil (d ax v ih1 l))
("devillier" nil (d ax v ih1 l y er0))
("devils" nil (d eh1 v ax l z))
("devilwood" nil (d eh1 v ax l w uh1 d))
("devin" nil (d eh1 v ih0 n))
("devincent" nil (d ey0 v iy1 n s ax n t))
("devincentis" nil (d eh0 v ih0 n s eh1 n t ih0 s))
("devincenzi" nil (d ih0 v iy0 n ch eh1 n z iy0))
("devincenzo" nil (d ih0 v iy0 n ch eh1 n z ow0))
("devine" nil (d ax v ay1 n))
("deviney" nil (d eh1 v ih0 n iy0))
("devinney" nil (d eh1 v ih0 n iy0))
("devino" nil (d ih0 v iy1 n ow0))
("devins" nil (d eh1 v ih0 n z))
("devious" nil (d iy1 v iy0 ax s))
("devise" nil (d ih0 v ay1 z))
("devised" nil (d ih0 v ay1 z d))
("devises" nil (d ih0 v ay1 z ih0 z))
("devising" nil (d ih0 v ay1 z ih0 ng))
("devita" nil (d ih0 v iy1 t ax))
("devito" nil (d ih0 v iy1 t ow0))
("devitt" nil (d ih0 v ih1 t))
("devivo" nil (d ih0 v iy1 v ow0))
("devlin" nil (d eh1 v l ih0 n))
("devoe" nil (d ih0 v ow1))
("devoid" nil (d ih0 v oy1 d))
("devol" nil (d eh1 v ao0 l))
("devolder" nil (d eh1 v ow0 l d er0))
("devoll" nil (d eh1 v ax l))
("devolution" nil (d eh1 v ax l uw1 sh ax n))
("devolved" nil (d ih0 v aa1 l v d))
("devon" nil (d eh1 v ax n))
("devona" nil (d eh1 v ax n ax))
("devonian" nil (d ih0 v ow1 n iy0 ax n))
("devonshire" nil (d ih0 v aa1 n sh ay1 r))
("devor" nil (d ih0 v ao1 r))
("devore" nil (d eh1 v er0))
("devos" nil (d iy1 v ow0 z))
("devoss" nil (d ih0 v aa1 s))
("devote" nil (d ih0 v ow1 t))
("devoted" nil (d ih0 v ow1 t ax d))
("devotee" nil (d eh1 v ax t iy1))
("devotees" nil (d eh1 v ax t iy1 z))
("devotes" nil (d ih0 v ow1 t s))
("devoting" nil (d ih0 v ow1 t ih0 ng))
("devotion" nil (d ih0 v ow1 sh ax n))
("devotional" nil (d ih0 v ow1 sh ax n ax l))
("devoto" nil (d ih0 v ow1 t ow0))
("devour" nil (d ih0 v aw1 er0))
("devoured" nil (d ih0 v aw1 er0 d))
("devouring" nil (d ih0 v aw1 er0 ih0 ng))
("devours" nil (d ih0 v aw1 er0 z))
("devout" nil (d ih0 v aw1 t))
("devoutly" nil (d ih0 v aw1 t l iy0))
("devoy" nil (d eh1 v oy0))
("devries" nil (d ih0 v r iy1 s))
("devry" nil (d eh1 v r iy0))
("dew" nil (d uw1))
("dewaard" nil (d uw0 aa1 r d))
("dewaele" nil (d uw1 eh0 l))
("dewald" nil (d uw1 ax l d))
("dewall" nil (d uw1 ax l))
("dewalt" nil (d uw1 ax l t))
("dewan" nil (d uw1 ax n))
("dewar" nil (d uw1 er0))
("dewart" nil (d uw1 aa0 r t))
("dewberry" nil (d uw1 b eh1 r iy0))
("dewbre" nil (d uw1 b er0))
("dewclaw" nil (d uw1 k l ao1))
("deweerd" nil (d uw1 ih0 r d))
("dewees" nil (d uw1 iy0 z))
("deweese" nil (d uw1 iy0 z))
("dewell" nil (d eh1 w eh0 l))
("dewey" nil (d uw1 iy0))
("dewhirst" nil (d eh1 w er0 s t))
("dewhurst" nil (d eh1 w er0 s t))
("dewilde" nil (d iy1 w ay0 l d))
("dewine" nil (d ax w ay1 n))
("dewing" nil (d uw1 ih0 ng))
("dewinter" nil (d uw1 ih0 n t er0))
("dewire" nil (d uw1 ay0 r))
("dewit" nil (d uw1 ih0 t))
("dewitt" nil (d ax w ih1 t))
("dewitte" nil (d uw0 ih1 t))
("dewitz" nil (d eh1 w ih0 t s))
("dewolf" nil (d uw1 uh0 l f))
("dewolfe" nil (d uw1 uh0 l f))
("dewoody" nil (d iy1 w uh0 d iy0))
("dews" nil (d uw1 z))
("dewulf" nil (d uw1 ax l f))
("dewy" nil (d uw1 iy0))
("dex" nil (d eh1 k s))
("dexheimer" nil (d eh1 k s hh ay0 m er0))
("dexter" nil (d eh1 k s t er0))
("dexterity" nil (d eh0 k s t eh1 r ax t iy0))
("dextra" nil (d eh1 k s t r ax))
("dextran" nil (d eh1 k s t r ae1 n))
("dextro" nil (d eh1 k s t r ow0))
("dextrose" nil (d eh1 k s t r ow0 s))
("dextrous" nil (d eh1 k s t r ax s))
("dey" nil (d ey1))
("deyo" nil (d ey1 ow0))
("deyoe" nil (d ey1 ow0))
("deyoung" nil (d eh1 y ax ng))
("deyton" nil (d ih0 t ao1 n))
("dezarn" nil (d ey0 z aa1 r n))
("dezeeuw" nil (d ih0 z iy1 uw0))
("dezern" nil (d ey0 z eh1 r n))
("deziel" nil (d eh1 z iy0 l))
("dhabi" nil (d aa1 b iy0))
("dhaka" nil (d aa1 k ax))
("dharma" nil (d aa1 r m ax))
("dhein" nil (d ay1 n))
("dhia" nil (d iy1 ax))
("dhillon" nil (d ih1 l ax n))
("dhlakama" nil (d ax l ax k aa1 m ax))
("dhole" nil (d ow1 l))
("dhondt" nil (d hh aa1 n t))
("dhows" nil (d aw1 z))
("di" nil (d iy1))
("dia" nil (d iy1 ax))
("diab" nil (d ay1 ax b))
("diabase" nil (d ay1 ax b ey1 s))
("diabasic" nil (d ay1 ax b ey1 s ih0 k))
("diabetes" nil (d ay1 ax b iy1 t iy0 z))
("diabetic" nil (d ay1 ax b eh1 t ih0 k))
("diabetics" nil (d ay1 ax b eh1 t ih0 k s))
("diablo" nil (d ay1 ae1 b l ow0))
("diabolical" nil (d ay1 ax b aa1 l ih0 k ax l))
("diaconate" nil (d ay0 ae1 k ax n ax t))
("diacritical" nil (d ay1 ax k r ih1 t ax k ax l))
("diadem" nil (d ay1 ax d eh1 m))
("diagnose" nil (d ay1 ax g n ow1 s))
("diagnosed" nil (d ay1 ax g n ow1 s t))
("diagnoses" nil (d ay1 ax g n ow1 s iy0 z))
("diagnosing" nil (d ay1 ax g n ow1 s ih0 ng))
("diagnosis" nil (d ay1 ax g n ow1 s ax s))
("diagnostek" nil (d ay1 ax g n aa1 s t eh0 k))
("diagnostic" nil (d ay1 ax g n aa1 s t ih0 k))
("diagnostics" nil (d ay1 ax g n aa1 s t ih0 k s))
("diagonal" nil (d ay0 ae1 g ax n ax l))
("diagonally" nil (d ay0 ae1 g ax n ax l iy0))
("diagram" nil (d ay1 ax g r ae1 m))
("diagraming" nil (d ay1 ax g r ae1 m ih0 ng))
("diagrammatic" nil (d ay1 ax g r ax m ae1 t ih0 k))
("diagrams" nil (d ay1 ax g r ae1 m z))
("dial" nil (d ay1 ax l))
("dialect" nil (d ay1 ax l eh1 k t))
("dialectic" nil (d ay1 ax l eh1 k t ih0 k))
("dialectical" nil (d ay1 ax l eh1 k t ih0 k ax l))
("dialects" nil (d ay1 ax l eh1 k t s))
("dialed" nil (d ay1 ax l d))
("dialing" nil (d ay1 ax l ih0 ng))
("dialog" nil (d ay1 ax l ao0 g))
("dialogue" nil (d ay1 ax l ao1 g))
("dialogues" nil (d ay1 ax l ao1 g z))
("dials" nil (d ay1 ax l z))
("dialtone" nil (d ay1 ax l t ow1 n))
("dialysis" nil (d ay0 ae1 l ax s ax s))
("diamagnetic" nil (d ay1 ax m ae0 g n eh1 t ih0 k))
("diamagnetism" nil (d ay1 ax m ae1 g n ih0 t ih1 z ax m))
("diamandis" nil (d ay1 ax m ae1 n d ih0 s))
("diamant" nil (d ay1 ax m ax n t))
("diamanta" nil (d ay1 ax m ae1 n t ax))
("diamante" nil (d ay1 ax m aa1 n t iy0))
("diameter" nil (d ay0 ae1 m ax t er0))
("diametrically" nil (d ay1 ax m eh1 t r ih0 k ax l iy0))
("diamond" nil (d ay1 m ax n d))
("diamonds" nil (d ay1 m ax n d z))
("dian" nil (d ay1 ax n))
("diana" nil (d ay0 ae1 n ax))
("diane" nil (d ay0 ae1 n))
("diangelo" nil (d ay0 ax ng g eh1 l ow0))
("dianna" nil (d ay1 ae1 n ax))
("dianne" nil (d ay0 ae1 n))
("diantha" nil (d ay1 ae1 n th ax))
("dianthe" nil (d ay0 ae1 n dh iy0))
("dianthia" nil (d ay1 ae1 n th iy0 ax))
("diantonio" nil (d ay1 ax n t ow1 n iy0 ow0))
("diaper" nil (d ay1 p er0))
("diapering" nil (d ay1 p er0 ih0 ng))
("diapers" nil (d ay1 ax p er0 z))
("diaphonia" nil (d ay1 ax f ow1 n iy0 ax))
("diaphragm" nil (d ay1 ax f r ae1 m))
("diapsid" nil (d ay1 ae1 p s ih0 d))
("diaries" nil (d ay1 er0 iy0 z))
("diario" nil (d ay0 eh1 r iy0 ow0))
("diarrhea" nil (d ay1 er0 iy1 ax))
("diarrhoea" nil (d ay1 er0 iy1 ax))
("diary" nil (d ay1 er0 iy0))
("dias" nil (d ay1 ax z))
("diasa" nil (d iy0 aa1 s ax))
("diasonics" nil (d ay1 ax s aa1 n ih0 k s))
("diaspora" nil (d ay0 ae1 s p er0 ax))
("diastase" nil (d ay1 ax s t ey1 s))
("diastole" nil (d ay0 ae1 s t ax l iy1))
("diastolic" nil (d ay1 ax s t aa1 l ih0 k))
("diastrophism" nil (d ay0 ae1 s t r ax f ih1 z ax m))
("diathermy" nil (d ay1 ax th er1 m iy0))
("diatomic" nil (d ay1 ax t aa1 m ih0 k))
("diatoms" nil (d ay1 ax t aa1 m z))
("diatonic" nil (d ay1 ax t aa1 n ih0 k))
("diatribe" nil (d ay1 ax t r ay1 b))
("diaz" nil (d iy1 aa0 s))
("diazo" nil (d ay0 ae1 z ow1))
("dibacco" nil (d ih0 b aa1 k ow0))
("dibari" nil (d ih0 b aa1 r iy0))
("dibartolo" nil (d iy1 b aa0 r t ow1 l ow0))
("dibartolomeo" nil (d ih0 b aa0 r t ow0 l ow1 m iy0 ow0))
("dibattista" nil (d ih0 b aa0 t iy1 s t ax))
("dibb" nil (d ih1 b))
("dibbern" nil (d ih1 b er0 n))
("dibble" nil (d ih1 b ax l))
("dibbled" nil (d ih1 b ax l d))
("dibella" nil (d ih0 b eh1 l ax))
("dibello" nil (d ih0 b eh1 l ow0))
("dibenedetto" nil (d ih0 b ih0 n ax d eh1 t ow0))
("dibernardo" nil (d ih0 b er0 n aa1 r d ow0))
("dibert" nil (d ih1 b er0 t))
("dibiase" nil (d iy1 b iy0 aa1 s iy0))
("dibiasio" nil (d ih0 b iy0 aa1 s iy0 ow0))
("diblasi" nil (d ih0 b l aa1 s iy0))
("diblasio" nil (d ih0 b l aa1 s iy0 ow0))
("dible" nil (d ay1 b ax l))
("diboll" nil (d ih1 b ax l))
("dibona" nil (d ih0 b ow1 n ax))
("dibrell" nil (d ih1 b r ax l))
("dibs" nil (d ih1 b z))
("dibuono" nil (d ih0 b w ow1 n ow0))
("dic" nil (d ih1 k))
("dicamba" nil (d ih0 k ae1 m b ax))
("dicamillo" nil (d ih0 k aa0 m ih1 l ow0))
("dicaprio" nil (d ih0 k ae1 p r iy0 ow0))
("dicarlo" nil (d ih0 k aa1 r l ow0))
("dice" nil (d ay1 s))
("dicecco" nil (d ih0 s eh1 k ow0))
("diced" nil (d ay1 s t))
("dicello" nil (d ih0 s eh1 l ow0))
("dicenso" nil (d ih0 s eh1 n s ow0))
("dicenzo" nil (d ih0 s eh1 n z ow0))
("diceon" nil (d ih1 s iy0 ax n))
("dicesare" nil (d ih0 ch eh0 s aa1 r iy0))
("dicey" nil (d ay1 s iy0))
("dichiara" nil (d ih0 k iy0 aa1 r ax))
("dichotomy" nil (d ay0 k aa1 t ax m iy0))
("dichroic" nil (d ay0 k r ow1 ih0 k))
("dichromate" nil (d ay0 k r ow1 m ey1 t))
("dichter" nil (d ih1 k t er0))
("dicicco" nil (d ih0 s ih1 k ow0))
("dicier" nil (d ay1 s iy0 er0))
("dicioccio" nil (d ih0 ch ow1 ch iy0 ow0))
("dick" nil (d ih1 k))
("dickard" nil (d ih1 k er0 d))
("dickason" nil (d ih1 k ax s ax n))
("dicke" nil (d ih1 k))
("dickel" nil (d ih1 k ax l))
("dicken" nil (d ih1 k ax n))
("dickens" nil (d ih1 k ax n z))
("dickensheets" nil (d ih1 k ax n sh iy1 t s))
("dickensian" nil (d ih0 k eh1 n z iy0 ax n))
("dickenson" nil (d ih1 k ih0 n s ax n))
("dicker" nil (d ih1 k er0))
("dickered" nil (d ih1 k er0 d))
("dickering" nil (d ih1 k er0 ih0 ng))
("dickerman" nil (d ih1 k er0 m ax n))
("dickerson" nil (d ih1 k er0 s ax n))
("dickert" nil (d ih1 k er0 t))
("dickes" nil (d ih1 k s))
("dickeson" nil (d ih1 k ih0 s ax n))
("dickey" nil (d ih1 k iy0))
("dickhaut" nil (d ih1 k hh aw1 t))
("dickie" nil (d ih1 k iy0))
("dickinson" nil (d ih1 k ih0 n s ax n))
("dickison" nil (d ih1 k ih0 s ax n))
("dickler" nil (d ih1 k l er0))
("dickman" nil (d ih1 k m ax n))
("dickmann" nil (d ih1 k m ax n))
("dickmeyer" nil (d ih1 k m ay0 er0))
("dicks" nil (d ih1 k s))
("dickson" nil (d ih1 k s ax n))
("dickstein" nil (d ih1 k s t ay0 n))
("dicky" nil (d ih1 k iy0))
("diclemente" nil (d ih1 k l ax m eh1 n t ey0))
("dicocco" nil (d ih0 k ow1 k ow0))
("dicola" nil (d ih0 k ow1 l ax))
("dicomed" nil (d ih1 k ax m eh0 d))
("dicostanzo" nil (d ih0 k ow0 s t aa1 n z ow0))
("dicots" nil (d ay1 k aa0 t s))
("dictaphone" nil (d ih1 k t ax f ow1 n))
("dictate" v (d ih0 k t ey1 t))
("dictate" n (d ih1 k t ey1 t))
("dictated" nil (d ih0 k t ey1 t ax d))
("dictates" v (d ih0 k t ey1 t s))
("dictates" n (d ih1 k t ey1 t s))
("dictating" nil (d ih1 k t ey1 t ih0 ng))
("dictation" nil (d ih0 k t ey1 sh ax n))
("dictator" nil (d ih0 k t ey1 t er0))
("dictatorial" nil (d ih1 k t ax t ao1 r iy0 ax l))
("dictators" nil (d ih0 k t ey1 t er0 z))
("dictatorship" nil (d ih0 k t ey1 t er0 sh ih1 p))
("dictatorships" nil (d ih0 k t ey1 t er0 sh ih1 p s))
("diction" nil (d ih1 k sh ax n))
("dictionaries" nil (d ih1 k sh ax n eh1 r iy0 z))
("dictionary" nil (d ih1 k sh ax n eh1 r iy0))
("dictum" nil (d ih1 k t ax m))
("dicus" nil (d ay1 k ax s))
("did" nil (d ih1 d))
("didactic" nil (d ay0 d ae1 k t ih0 k))
("diddy" nil (d ih1 d iy0))
("didemeyer" nil (d iy1 d ax m ay1 er0))
("dideoxycytidine" nil (d ih1 d iy0 aa1 k s iy0 s ay1 t ih0 d ay1 n))
("didi" nil (d iy1 d iy0))
("didier" nil (d ih1 d iy0 er0))
("didinium" nil (d ih0 d ih1 n iy0 ax m))
("didion" nil (d ih1 d iy0 ax n))
("dido" nil (d ay1 d ow0))
("didomenico" nil (d ih0 d ow0 m eh0 n iy1 k ow0))
("didonato" nil (d ih0 d ow0 n aa1 t ow0))
("didonna" nil (d ih0 d aa1 n ax))
("die" nil (d ay1))
("diebel" nil (d iy1 b ax l))
("diebold" nil (d ay1 b ow1 l d))
("dieck" nil (d iy1 k))
("dieckman" nil (d iy1 k m ax n))
("dieckmann" nil (d iy1 k m ax n))
("died" nil (d ay1 d))
("diede" nil (d iy1 d))
("diederich" nil (d iy1 d er0 ih0 k))
("diedrich" nil (d iy1 d r ih0 k))
("diedrick" nil (d iy1 d r ih0 k))
("diefenbach" nil (d iy1 f ih0 n b aa0 k))
("diefenderfer" nil (d iy1 f ih0 n d er0 f er0))
("diefendorf" nil (d iy1 f ih0 n d ao0 r f))
("dieffenbach" nil (d iy1 f ih0 n b aa0 k))
("diegans" nil (d iy1 g ax n z))
("diegel" nil (d iy1 g ax l))
("diego" nil (d iy0 ey1 g ow0))
("dieguez" nil (d ih0 g eh1 z))
("diehard" nil (d ay1 hh aa1 r d))
("diehards" nil (d ay1 hh aa1 r d z))
("diehl" nil (d iy1 l))
("diehm" nil (d iy1 m))
("diekman" nil (d iy1 k m ax n))
("diekmann" nil (d iy1 k m ax n))
("diel" nil (d iy1 l))
("diem" nil (d iy1 m))
("diemer" nil (d iy1 m er0))
("diemert" nil (d iy1 m er0 t))
("diener" nil (d iy1 n er0))
("dienes" nil (d iy1 n eh0 z))
("dienst" nil (d iy1 n s t))
("diep" nil (d iy1 p))
("dier" nil (d iy1 er0))
("diercks" nil (d iy1 r k s))
("dieringer" nil (d ih1 r ih0 n jh er0))
("dierker" nil (d iy1 r k er0))
("dierkes" nil (d iy1 r k s))
("dierking" nil (d ay1 er0 k ih0 ng))
("dierks" nil (d iy1 r k s))
("dierolf" nil (d ih1 r ow0 l f))
("diers" nil (d iy1 er0 z))
("diery" nil (d ih1 r iy0))
("dies" nil (d ay1 z))
("diesel" nil (d iy1 s ax l))
("diesels" nil (d iy1 z ax l z))
("dieses" nil (d ay1 z ih0 z))
("diesing" nil (d iy1 s ih0 ng))
("diet" nil (d ay1 ax t))
("dietary" nil (d ay1 ax t eh1 r iy0))
("dietel" nil (d iy1 t ax l))
("dieter" nil (d iy1 t er0))
("dieterich" nil (d iy1 t er0 ih0 k))
("dieterle" nil (d iy1 t er0 ax l))
("dieters" nil (d ay1 ax t er0 z))
("dietetic" nil (d ay1 ax t eh1 t ih0 k))
("dieting" nil (d ay1 ax t ih0 ng))
("dietitian" nil (d ay1 ax t ih1 sh ax n))
("dietitians" nil (d ay1 ax t ih1 sh ax n z))
("dietl" nil (d ay1 ax t l))
("dietrich" nil (d iy1 t r ih0 k))
("dietrick" nil (d iy1 t r ih0 k))
("diets" nil (d ay1 ih0 t s))
("dietsch" nil (d iy1 ch))
("dietsche" nil (d iy1 ch))
("dietz" nil (d iy1 t s))
("dietze" nil (d ay1 ax t z))
("dietzel" nil (d iy1 t z ax l))
("dietzen" nil (d iy1 t z ax n))
("dietzler" nil (d iy1 t s l er0))
("dietzman" nil (d iy1 t s m ax n))
("dievler" nil (d iy1 v l er0))
("diez" nil (d ay1 ax z))
("difabio" nil (d ih0 f ey1 b iy0 ow0))
("difalco" nil (d ih0 f ae1 l k ow0))
("difazio" nil (d ih0 f ey1 z iy0 ow0))
("difelice" nil (d ih0 f eh1 l ih0 s))
("diff" nil (d ih1 f))
("diffee" nil (d ih1 f iy0))
("diffenderfer" nil (d ih1 f ih0 n d er0 f er0))
("differ" nil (d ih1 f er0))
("differed" nil (d ih1 f er0 d))
("difference" nil (d ih1 f er0 ax n s))
("differences" nil (d ih1 f er0 ax n s ih0 z))
("different" nil (d ih1 f er0 ax n t))
("differential" nil (d ih1 f er0 eh1 n ch ax l))
("differentials" nil (d ih1 f er0 eh1 n ch ax l z))
("differentiate" nil (d ih1 f er0 eh1 n sh iy0 ey1 t))
("differentiated" nil (d ih1 f er0 eh1 n ch iy0 ey1 t ih0 d))
("differentiates" nil (d ih0 f er0 eh1 n sh iy0 ey1 t s))
("differentiating" nil (d ih1 f er0 eh1 n ch iy0 ey1 t ih0 ng))
("differentiation" nil (d ih0 f er0 eh1 n sh iy0 ey1 sh ax n))
("differently" nil (d ih1 f r ax n t l iy0))
("differing" nil (d ih1 f er0 ih0 ng))
("differs" nil (d ih1 f er0 z))
("difficult" nil (d ih1 f ax k ax l t))
("difficulties" nil (d ih1 f ax k ax l t iy0 z))
("difficulty" nil (d ih1 f ax k ax l t iy0))
("diffin" nil (d ih1 f ih0 n))
("diffley" nil (d ih1 f l iy0))
("diffract" nil (d ih0 f r ae1 k t))
("diffraction" nil (d ih0 f r ae1 k sh ax n))
("diffuse" j (d ih0 f y uw1 s))
("diffuse" v (d ih0 f y uw1 z))
("diffused" nil (d ih0 f y uw1 z d))
("diffuses" nil (d ih0 f y uw1 z ax z))
("diffusion" nil (d ih0 f y uw1 zh ax n))
("difilippo" nil (d ih0 f iy0 l iy1 p ow0))
("difiore" nil (d ih0 f iy0 ao1 r iy0))
("difm" nil (d ih1 f m))
("difonzo" nil (d ih0 f aa1 n z ow0))
("difrancesco" nil (d ih0 f r aa0 n ch eh1 s k ow0))
("difranco" nil (d ih0 f r aa1 n k ow0))
("dig" nil (d ih1 g))
("digaetano" nil (d ih0 jh ax t aa1 n ow0))
("digalakis" nil (d ih0 jh ax l aa1 k ax s))
("digangi" nil (d ih0 g ae1 n jh iy0))
("digate" nil (d ay1 g ey1 t))
("digby" nil (d ih1 g b iy0))
("digennaro" nil (d ih0 jh eh0 n aa1 r ow0))
("digenova" nil (d iy1 jh eh0 n ow1 v ax))
("digeronimo" nil (d ih0 jh er0 ow0 n iy1 m ow0))
("digest" v (d ay0 jh eh1 s t))
("digest" n (d ay1 jh eh0 s t))
("digested" nil (d ay1 jh eh1 s t ih0 d))
("digester" nil (d ay1 jh eh1 s t er0))
("digestible" nil (d ay0 jh eh1 s t ax b ax l))
("digesting" nil (d ay0 jh eh1 s t ih0 ng))
("digestion" nil (d ay0 jh eh1 s ch ax n))
("digestive" nil (d ay0 jh eh1 s t ih0 v))
("digests" v (d ay0 jh eh1 s t s))
("digests" n (d ay1 jh eh0 s t s))
("digger" nil (d ih1 g er0))
("diggers" nil (d ih1 g er0 z))
("digges" nil (d ih1 g z))
("digging" nil (d ih1 g ih0 ng))
("diggins" nil (d ih1 g ih0 n z))
("diggle" nil (d ih1 g ax l))
("diggs" nil (d ih1 g z))
("digiacomo" nil (d iy1 jh ax k ow0 m ow0))
("digicon" nil (d ih1 jh ih0 k aa1 n))
("digidyne" nil (d ih1 jh ih0 d ay1 n))
("digilio" nil (d ih0 jh iy1 l iy0 ow0))
("digioia" nil (d ih0 jh ow1 y ax))
("digiorgio" nil (d ih0 jh ao1 r jh iy0 ow0))
("digiovanna" nil (d ih0 jh ow0 v aa1 n ax))
("digiovanni" nil (d ih0 jh ow0 v aa1 n iy0))
("digirolamo" nil (d ih0 jh ih0 r ow0 l aa1 m ow0))
("digit" nil (d ih1 jh ax t))
("digital" nil (d ih1 jh ax t ax l))
("digitalis" nil (d ih1 jh ax t ae1 l ax s))
("digitally" nil (d ih1 jh ax t ax l iy0))
("digitech" nil (d ih1 jh ax t eh1 k))
("digitized" nil (d ih1 jh ax t ay1 z d))
("digits" nil (d ih1 jh ax t s))
("digiulio" nil (d ih0 jh uw1 l iy0 ow0))
("digiuseppe" nil (d iy1 jh uw0 s eh1 p iy0))
("digman" nil (d ih1 g m ax n))
("dignan" nil (d ih1 g n ax n))
("dignified" nil (d ih1 g n ax f ay1 d))
("dignify" nil (d ih1 g n ax f ay1))
("dignitaries" nil (d ih1 g n ax t eh1 r iy0 z))
("dignitary" nil (d ih1 g n ax t eh1 r iy0))
("dignity" nil (d ih1 g n ax t iy0))
("digrazia" nil (d ih0 g r aa1 z iy0 ax))
("digregorio" nil (d ih0 g r eh0 g ao1 r iy0 ow0))
("digress" nil (d ay0 g r eh1 s))
("digressing" nil (d ay0 g r eh1 s ih0 ng))
("digressions" nil (d ay0 g r eh1 sh ax n z))
("digs" nil (d ih1 g z))
("diguglielmo" nil (d ih0 g uw1 g l iy0 eh1 l m ow0))
("diianni" nil (d iy1 ae1 n iy0))
("diiorio" nil (d ih0 y ao1 r iy0 ow0))
("dijker" nil (d iy1 k er0))
("dijon" nil (d iy1 zh aa1 n))
("dike" nil (d ay1 k))
("dikeman" nil (d ay1 k m ax n))
("dikes" nil (d ay1 k s))
("dilapidated" nil (d ax l ae1 p ax d ey1 t ih0 d))
("dilatation" nil (d ih1 l ax t ey1 sh ax n))
("dilate" nil (d ay0 l ey1 t))
("dilated" nil (d ay0 l ey1 t ax d))
("dilation" nil (d ay0 l ey1 sh ax n))
("dilatory" nil (d ih1 l ax t ao1 r iy0))
("dilaura" nil (d ih0 l ao1 r ax))
("dilauro" nil (d ih0 l ao1 r ow0))
("dilbeck" nil (d ih1 l b eh1 k))
("dilday" nil (d ih1 l d ey1))
("dildine" nil (d ih0 l d iy1 n iy0))
("dildy" nil (d ih1 l d iy0))
("dilella" nil (d ih0 l eh1 l ax))
("dilello" nil (d ih0 l eh1 l ow0))
("dilemma" nil (d ih0 l eh1 m ax))
("dilemmas" nil (d ax l eh1 m ax z))
("dilenschneider" nil (d ay1 l ax n sh n ay1 d er0))
("dileo" nil (d ih1 l iy0 ow0))
("dileonardo" nil (d ih0 l iy0 ax n aa1 r d ow0))
("diles" nil (d ay1 l z))
("dilg" nil (d ih1 l g))
("dilger" nil (d ih1 l g er0))
("diliberto" nil (d ih0 l iy0 b eh1 r t ow0))
("diligence" nil (d ih1 l ax jh ax n s))
("diligent" nil (d ih1 l ih0 jh ax n t))
("diligently" nil (d ih1 l ax jh ax n t l iy0))
("dilip" nil (d ih1 l ih0 p))
("dilithium" nil (d ay0 l ih1 th iy0 ax m))
("dilks" nil (d ih1 l k s))
("dill" nil (d ih1 l))
("dillahunt" nil (d ih1 l ax hh ah1 n t))
("dillahunty" nil (d ih1 l ax hh ah1 n t iy0))
("dillard" nil (d ih1 l er0 d))
("dille" nil (d ih1 l))
("dillehay" nil (d ih1 l ih0 hh ey0))
("dillen" nil (d ih1 l ax n))
("dillenbeck" nil (d ih1 l ax n b eh1 k))
("dillenburg" nil (d ih1 l ax n b er0 g))
("diller" nil (d ih1 l er0))
("dilley" nil (d ih1 l iy0))
("dillie" nil (d ih1 l iy0))
("dillin" nil (d ih1 l ih0 n))
("dilling" nil (d ih1 l ih0 ng))
("dillinger" nil (d ih1 l ih0 ng er0))
("dillingham" nil (d ih1 l ih0 ng hh ae1 m))
("dillion" nil (d ih1 l y ax n))
("dillman" nil (d ih1 l m ax n))
("dillon" nil (d ih1 l ax n))
("dillow" nil (d ih1 l ow0))
("dills" nil (d ih1 l z))
("dillworth" nil (d ih1 l w er1 th))
("dilly" nil (d ih1 l iy0))
("dilmore" nil (d iy1 l m ao0 r))
("dilorenzo" nil (d ih0 l ao0 r eh1 n z ow0))
("diloreto" nil (d ih0 l ao0 r eh1 t ow0))
("dilorio" nil (d ih0 l ao1 r iy0 ow0))
("dils" nil (d ih1 l z))
("dilsaver" nil (d ih1 l s ax v er0))
("dilson" nil (d ih1 l s ax n))
("diltiazem" nil (d ih0 l t iy1 ax z eh1 m))
("dilts" nil (d ih1 l t s))
("diltz" nil (d ih1 l t s))
("dilullo" nil (d ih0 l uw1 l ow0))
("dilute" nil (d ay0 l uw1 t))
("diluted" nil (d ay0 l uw1 t ax d))
("dilutes" nil (d ay0 l uw1 t s))
("diluting" nil (d ay0 l uw1 t ih0 ng))
("dilution" nil (d ay0 l uw1 sh ax n))
("dilutive" nil (d ax l uw1 t ih0 v))
("diluzio" nil (d ih0 l uw1 z iy0 ow0))
("dilworth" nil (d ih1 l w er0 th))
("dim" nil (d ih1 m))
("dimaggio" nil (d ih0 m ae1 jh iy0 ow0))
("dimaio" nil (d ih0 m aa1 iy0 ow0))
("dimambro" nil (d ih0 m ae1 m b r ow0))
("dimarco" nil (d ih0 m aa1 r k ow0))
("dimare" nil (d ih0 m aa1 r iy0))
("dimaria" nil (d ih0 m aa1 r iy0 ax))
("dimarino" nil (d ih0 m aa0 r iy1 n ow0))
("dimario" nil (d ih0 m aa1 r iy0 ow0))
("dimartino" nil (d iy1 m aa0 r t iy1 n ow0))
("dimarzio" nil (d ih0 m aa1 r z iy0 ow0))
("dimarzo" nil (d ih0 m aa1 r z ow0))
("dimas" nil (d ay1 m ax z))
("dimascio" nil (d ih0 m ae1 s iy0 ow0))
("dimasi" nil (d ih0 m aa1 s iy0))
("dimatteo" nil (d ih0 m aa1 t iy0 ow0))
("dimauro" nil (d ih0 m ao1 r ow0))
("dime" nil (d ay1 m))
("dimeglio" nil (d ih0 m eh1 g l iy0 ow0))
("dimension" nil (d ih0 m eh1 n sh ax n))
("dimensional" nil (d ih0 m eh1 n sh ax n ax l))
("dimensionality" nil (d ih0 m eh1 n sh ax n ae1 l ax t iy0))
("dimensioned" nil (d ax m eh1 n ch ax n d))
("dimensions" nil (d ih0 m eh1 n sh ax n z))
("dimeo" nil (d iy1 m iy0 ow0))
("dimer" nil (d ay1 m er0))
("dimercurio" nil (d ih0 m er0 k uh1 r iy0 ow0))
("dimes" nil (d ay1 m z))
("dimetapp" nil (d ay1 m ax t ae1 p))
("dimiceli" nil (d ih0 m iy0 ch eh1 l iy0))
("dimichele" nil (d ih0 m iy0 k eh1 l iy0))
("dimick" nil (d ih1 m ih0 k))
("diming" nil (d ay1 m ih0 ng))
("diminish" nil (d ih0 m ih1 n ih0 sh))
("diminished" nil (d ih0 m ih1 n ih0 sh t))
("diminishes" nil (d ih0 m ih1 n ih0 sh ih0 z))
("diminishing" nil (d ih0 m ih1 n ih0 sh ih0 ng))
("dimino" nil (d ih0 m iy1 n ow0))
("diminution" nil (d ih1 m ax n uw1 sh ax n))
("diminutive" nil (d ih0 m ih1 n y ax t ih0 v))
("dimitri" nil (d ih0 m iy1 t r iy0))
("dimitrios" nil (d ih0 m iy1 t r iy0 ow0 s))
("dimitrius" nil (d ih0 m iy1 t r iy0 ax s))
("dimitroff" nil (d ih1 m ih0 t r ao0 f))
("dimitruk" nil (d ih0 m iy1 t r uh1 k))
("dimly" nil (d ih1 m l iy0))
("dimmed" nil (d ih1 m d))
("dimmer" nil (d ih1 m er0))
("dimmers" nil (d ih1 m er0 z))
("dimmick" nil (d ih1 m ih0 k))
("dimming" nil (d ih1 m ih0 ng))
("dimmitt" nil (d ih1 m ih0 t))
("dimock" nil (d ih1 m ax k))
("dimon" nil (d ih1 m ax n))
("dimona" nil (d ih0 m ow1 n ax))
("dimond" nil (d ay1 m ax n d))
("dimorphic" nil (d ay0 m ao1 r f ih0 k))
("dimorphism" nil (d ay0 m ao1 r f ih0 z ax m))
("dimperio" nil (d ih0 m p eh1 r iy0 ow0))
("dimpled" nil (d ih1 m p ax l d))
("dims" nil (d ih1 m z))
("dimsdale" nil (d ih1 m z d ey1 l))
("dimuro" nil (d ih0 m uh1 r ow0))
("dimuzio" nil (d ih0 m uw1 z iy0 ow0))
("din" nil (d ih1 n))
("dina" nil (d iy1 n ax))
("dinah" nil (d ay1 n ax))
("dinan" nil (d ih1 n ax n))
("dinapoli" nil (d ih0 n ae1 p ax l iy0))
("dinar" nil (d ih0 n aa1 r))
("dinardo" nil (d ih0 n aa1 r d ow0))
("dinars" nil (d ay1 n er0 z))
("dinatale" nil (d ih0 n aa0 t aa1 l iy0))
("dinda" nil (d ih1 n d ax))
("dine" nil (d ay1 n))
("dined" nil (d ay1 n d))
("dineen" nil (d ih0 n iy1 n))
("dinehart" nil (d ay1 n hh aa1 r t))
("diner" nil (d ay1 n er0))
("diners" nil (d ay1 n er0 z))
("dines" nil (d ay1 n z))
("ding" nil (d ih1 ng))
("dingee" nil (d ih1 ng g iy0))
("dingel" nil (d ih1 ng g ax l))
("dingell" nil (d ih1 ng g ax l))
("dinger" nil (d ih1 ng er0))
("dinges" nil (d ih1 n jh ih0 z))
("dingess" nil (d ih1 ng g ih0 s))
("dingham" nil (d ih1 ng ax m))
("dinghy" nil (d ih1 ng iy0))
("dingle" nil (d ih1 ng g ax l))
("dingledine" nil (d ih1 ng g l ih0 d ay0 n))
("dingler" nil (d ih1 ng g ax l er0))
("dingley" nil (d ih1 ng g l iy0))
("dingman" nil (d ih1 ng m ax n))
("dingo" nil (d ih1 ng g ow0))
("dings" nil (d ih1 ng z))
("dingus" nil (d ih1 ng g ih0 s))
("dingwall" nil (d ih1 ng g w ax l))
("dingy" nil (d ih1 n jh iy0))
("dinh" nil (d ih1 n))
("dini" nil (d iy1 n iy0))
("dinicola" nil (d ih0 n iy0 k ow1 l ax))
("dining" nil (d ay1 n ih0 ng))
("dinino" nil (d ih0 n iy1 n ow0))
("dinius" nil (d ay1 n iy0 ih0 s))
("dinkel" nil (d ih1 ng k ax l))
("dinkins" nil (d ih1 ng k ih0 n z))
("dinky" nil (d ih1 ng k iy0))
("dinmukhamed" nil (d ih1 n m uw0 k aa1 m eh0 d))
("dinneen" nil (d ih0 n iy1 n))
("dinner" nil (d ih1 n er0))
("dinners" nil (d ih1 n er0 z))
("dinnertime" nil (d ih1 n er0 t ay1 m))
("dinnerware" nil (d ih1 n er0 w eh1 r))
("dinning" nil (d ih1 n ih0 ng))
("dino" nil (d iy1 n ow0))
("dinosaur" nil (d ay1 n ax s ao1 r))
("dinosaurs" nil (d ay1 n ax s ao1 r z))
("dinoseb" nil (d ay1 n ow0 s eh1 b))
("dinoto" nil (d ih0 n ow1 t ow0))
("dinovo" nil (d ih0 n ow1 v ow0))
("dinsa" nil (d ih1 n s ax))
("dinsdale" nil (d ih1 n z d ey1 l))
("dinse" nil (d ih1 n s))
("dinsmore" nil (d iy1 n s m ao0 r))
("dint" nil (d ih1 n t))
("dinunzio" nil (d ih0 n ah1 n z iy0 ow0))
("dinwiddie" nil (d ih1 n w ih0 d iy0))
("diocesan" nil (d ay0 aa1 s ax s ax n))
("diocese" nil (d ay1 ax s iy1 z))
("dioceses" nil (d ay1 ax s iy1 z))
("diodati" nil (d iy0 ow0 d aa1 t iy0))
("diodato" nil (d iy0 ow0 d aa1 t ow0))
("diode" nil (d ay1 ow1 d))
("diodes" nil (d ay1 ow1 d z))
("dioguardi" nil (d iy0 ow0 g aa1 r d iy0))
("dion" nil (d ay1 ax n))
("diona" nil (d iy0 ow1 n ax))
("dione" nil (d iy1 aa0 n))
("dionisio" nil (d ay1 ax n ih1 s iy0 ow0))
("dionne" nil (d iy1 aa0 n))
("dionysius" nil (d ay1 ax n ih1 s iy0 ax s))
("dior" nil (d iy1 ao1 r))
("diorio" nil (d iy0 ao1 r iy0 ow0))
("diorite" nil (d ay1 er0 ay1 t))
("dios" nil (d iy1 ow0 s))
("dioxide" nil (d ay0 aa1 k s ay1 d))
("dioxides" nil (d ay0 aa1 k s ay0 d z))
("dioxin" nil (d ay1 aa1 k s ih0 n))
("dioxins" nil (d ay1 aa1 k s ih0 n z))
("dip" nil (d ih1 p))
("dipalma" nil (d ih0 p aa1 l m ax))
("dipaola" nil (d ih0 p aa0 ow1 l ax))
("dipaolo" nil (d ih0 p aa0 ow1 l ow0))
("dipasquale" nil (d ih0 p aa0 s k w aa1 l iy0))
("diperna" nil (d ih0 p eh1 r n ax))
("diphtheria" nil (d ih0 f th ih1 r iy0 ax))
("dipiazza" nil (d ih0 p iy0 aa1 t s ax))
("dipiero" nil (d ih0 p ih1 r ow0))
("dipierro" nil (d ih0 p ih1 r ow0))
("dipietro" nil (d ih0 p iy1 t r ow0))
("dipinto" nil (d ih0 p ih1 n t ow0))
("dipirro" nil (d ih0 p ih1 r ow0))
("diploma" nil (d ih0 p l ow1 m ax))
("diplomacy" nil (d ih0 p l ow1 m ax s iy0))
("diplomas" nil (d ih0 p l ow1 m ax z))
("diplomat" nil (d ih1 p l ax m ae1 t))
("diplomatic" nil (d ih1 p l ax m ae1 t ih0 k))
("diplomatically" nil (d ih1 p l ax m ae1 t ih0 k l iy0))
("diplomats" nil (d ih1 p l ax m ae1 t s))
("dipole" nil (d ay1 p ow1 l))
("dipped" nil (d ih1 p t))
("dippel" nil (d ih1 p ax l))
("dipper" nil (d ih1 p er0))
("dippers" nil (d ih1 p er0 z))
("dipping" nil (d ih1 p ih0 ng))
("dipple" nil (d ih1 p ax l))
("dippold" nil (d ih1 p ow1 l d))
("dippolito" nil (d ih0 p ow0 l iy1 t ow0))
("dippy" nil (d ih1 p iy0))
("diprima" nil (d ih0 p r iy1 m ax))
("dips" nil (d ih1 p s))
("dipstick" nil (d ih1 p s t ih1 k))
("dircks" nil (d er1 k s))
("dire" nil (d ay1 r))
("direct" nil (d er0 eh1 k t))
("directed" nil (d er0 eh1 k t ax d))
("directing" nil (d er0 eh1 k t ih0 ng))
("direction" nil (d er0 eh1 k sh ax n))
("directional" nil (d er0 eh1 k sh ax n ax l))
("directionless" nil (d er0 eh1 k sh ax n l ax s))
("directions" nil (d er0 eh1 k sh ax n z))
("directive" nil (d er0 eh1 k t ih0 v))
("directives" nil (d ay0 r eh1 k t ih0 v z))
("directly" nil (d er0 eh1 k t l iy0))
("directness" nil (d er0 eh1 k t n ax s))
("director" nil (d er0 eh1 k t er0))
("directorate" nil (d er0 eh1 k t er0 ax t))
("directorial" nil (d er0 eh0 k t ao1 r iy0 ax l))
("directories" nil (d ay0 r eh1 k t er0 iy0 z))
("directors" nil (d er0 eh1 k t er0 z))
("directorship" nil (d er0 eh1 k t er0 sh ih1 p))
("directorships" nil (d er0 eh1 k t er0 sh ih1 p s))
("directory" nil (d er0 eh1 k t er0 iy0))
("directs" nil (d er0 eh1 k t s))
("directv" nil (d er0 eh1 k t iy1 v iy1))
("direnzo" nil (d ih0 r eh1 n z ow0))
("direst" nil (d ay1 r ax s t))
("dirge" nil (d er1 jh))
("dirhams" nil (d er1 ax m z))
("dirickson" nil (d ao1 r ih0 k s ax n))
("dirienzo" nil (d ih0 r iy1 n z ow0))
("dirk" nil (d er1 k))
("dirkes" nil (d er1 k s))
("dirks" nil (d er1 k s))
("dirkse" nil (d er1 k s))
("dirksen" nil (d er1 k s ax n))
("dirlam" nil (d er0 l ae1 m))
("dirocco" nil (d ih0 r aa1 k ow0))
("dirosa" nil (d ih0 r ow1 s ax))
("dirr" nil (d er1))
("dirt" nil (d er1 t))
("dirtier" nil (d er1 t iy0 er0))
("dirtiest" nil (d er1 t iy0 ax s t))
("dirty" nil (d er1 t iy0))
("dirusso" nil (d ih0 r uw1 s ow0))
("disa" nil (d ih1 s ax))
("disabatino" nil (d ih0 s aa0 b aa0 t iy1 n ow0))
("disabato" nil (d ih0 s aa0 b aa1 t ow0))
("disabilities" nil (d ih0 s ax b ih1 l ax t iy0 z))
("disability" nil (d ih1 s ax b ih1 l ih0 t iy0))
("disable" nil (d ih0 s ey1 b ax l))
("disabled" nil (d ih0 s ey1 b ax l d))
("disables" nil (d ih0 s ey1 b ax l z))
("disabling" nil (d ih0 s ey1 b ax l ih0 ng))
("disadvantage" nil (d ih1 s ax d v ae1 n t ih0 jh))
("disadvantaged" nil (d ih0 s ax d v ae1 n t ih0 jh d))
("disadvantageous" nil (d ih1 s ae1 d v ae1 n t ey1 jh ax s))
("disadvantages" nil (d ih1 s ax d v ae1 n t ih0 jh ih0 z))
("disaffected" nil (d ih1 s ax f eh1 k t ih0 d))
("disaffection" nil (d ih1 s ax f eh1 k sh ax n))
("disagree" nil (d ih0 s ax g r iy1))
("disagreeable" nil (d ih1 s ax g r iy1 ax b ax l))
("disagreed" nil (d ih0 s ax g r iy1 d))
("disagreeing" nil (d ih0 s ax g r iy1 ih0 ng))
("disagreement" nil (d ih0 s ax g r iy1 m ax n t))
("disagreements" nil (d ih1 s ax g r iy1 m ax n t s))
("disagrees" nil (d ih0 s ax g r iy1 z))
("disallow" nil (d ih1 s ax l aw1))
("disallowance" nil (d ih1 s ax l aw1 ax n s))
("disallowances" nil (d ih1 s ax l aw1 ax n s ih0 z))
("disallowed" nil (d ih1 s ax l aw1 d))
("disallowing" nil (d ih0 s ax l aw1 ih0 ng))
("disalvo" nil (d ih0 s aa1 l v ow0))
("disano" nil (d ih0 s aa1 n ow0))
("disanti" nil (d ih0 s ae1 n t iy0))
("disantis" nil (d ih0 s aa1 n t ih0 s))
("disanto" nil (d ih0 s ae1 n t ow0))
("disappear" nil (d ih1 s ax p ih1 r))
("disappearance" nil (d ih1 s ax p ih1 r ax n s))
("disappearances" nil (d ih1 s ax p ih1 r ax n s ih0 z))
("disappeared" nil (d ih1 s ax p ih1 r d))
("disappearing" nil (d ih1 s ax p ih1 r ih0 ng))
("disappears" nil (d ih1 s ax p ih1 r z))
("disappoint" nil (d ih1 s ax p oy1 n t))
("disappointed" nil (d ih1 s ax p oy1 n t ih0 d))
("disappointing" nil (d ih1 s ax p oy1 n t ih0 ng))
("disappointingly" nil (d ih0 s ax p oy1 n t ih0 ng l iy0))
("disappointment" nil (d ih1 s ax p oy1 n t m ax n t))
("disappointments" nil (d ih0 s ax p oy1 n t m ax n t s))
("disappoints" nil (d ih1 s ax p oy1 n t s))
("disapproval" nil (d ih0 s ax p r uw1 v ax l))
("disapprove" nil (d ih1 s ax p r uw1 v))
("disapproved" nil (d ih1 s ax p r uw1 v d))
("disapproves" nil (d ih0 s ax p r uw1 v z))
("disapproving" nil (d ih0 s ax p r uw1 v ih0 ng))
("disarm" nil (d ih0 s aa1 r m))
("disarmament" nil (d ih0 s aa1 r m ax m ax n t))
("disarmed" nil (d ih0 s aa1 r m d))
("disarming" nil (d ih0 s aa1 r m ih0 ng))
("disarmingly" nil (d ih0 s aa1 r m ih0 ng l iy0))
("disarray" nil (d ih1 s er0 ey1))
("disassembled" nil (d ih1 s ax s eh1 m b ax l d))
("disassociate" nil (d ih1 s ax s ow1 sh iy0 ey0 t))
("disaster" nil (d ih0 z ae1 s t er0))
("disasters" nil (d ih0 z ae1 s t er0 z))
("disastrous" nil (d ih0 z ae1 s t r ax s))
("disastrously" nil (d ih0 z ae1 s t r ax s l iy0))
("disavow" nil (d ih1 s ax v aw1))
("disavowed" nil (d ih1 s ax v aw1 d))
("disavowing" nil (d ih0 s ax v aw1 ih0 ng))
("disband" nil (d ih0 s b ae1 n d))
("disbanded" nil (d ih0 s b ae1 n d ih0 d))
("disbanding" nil (d ih0 s b ae1 n d ih0 ng))
("disbarment" nil (d ih1 s b aa1 r m ax n t))
("disbarred" nil (d ih0 s b aa1 r d))
("disbelief" nil (d ih1 s b ih0 l iy1 f))
("disbelieve" nil (d ih0 s b ax l iy1 v))
("disbro" nil (d ih1 s b r ow0))
("disbrow" nil (d ih1 s b r aw0))
("disburse" nil (d ih0 s b er1 s))
("disbursed" nil (d ih0 s b er1 s t))
("disbursement" nil (d ih0 s b er1 s m ax n t))
("disbursements" nil (d ih0 s b er1 s m ax n t s))
("disbursing" nil (d ih0 s b er1 s ih0 ng))
("disc" nil (d ih1 s k))
("discard" nil (d ih0 s k aa1 r d))
("discarded" nil (d ih0 s k aa1 r d ih0 d))
("discarding" nil (d ih0 s k aa1 r d ih0 ng))
("discards" nil (d ih0 s k aa1 r d z))
("discenza" nil (d ih0 s ch eh1 n z ax))
("discern" nil (d ih0 s er1 n))
("discerned" nil (d ih0 s er1 n d))
("discernible" nil (d ih0 s er1 n ax b ax l))
("discerning" nil (d ih0 s er1 n ih0 ng))
("discernment" nil (d ih0 s er1 n m ax n t))
("disch" nil (d ih1 sh))
("discharge" v (d ih0 s ch aa1 r jh))
("discharge" n (d ih1 s ch aa0 r jh))
("discharged" nil (d ih0 s ch aa1 r jh d))
("discharges" v (d ih0 s ch aa1 r jh ax z))
("discharges" n (d ih1 s ch aa0 r jh ax z))
("discharging" nil (d ih0 s ch aa1 r jh ih0 ng))
("discher" nil (d ih1 sh er0))
("dischinger" nil (d ih1 sh ih0 n jh er0))
("disciple" nil (d ih0 s ay1 p ax l))
("disciples" nil (d ih0 s ay1 p ax l z))
("disciplinarian" nil (d ih1 s ih0 p l ih0 n eh1 r iy0 ax n))
("disciplinarians" nil (d ih1 s ih0 p l ih0 n eh1 r iy0 ax n z))
("disciplinary" nil (d ih1 s ax p l ax n eh1 r iy0))
("discipline" nil (d ih1 s ax p l ax n))
("disciplined" nil (d ih1 s ax p l ax n d))
("disciplines" nil (d ih1 s ax p l ax n z))
("disciplining" nil (d ih1 s ax p l ax n ih0 ng))
("disclaimed" nil (d ih0 s k l ey1 m d))
("disclaimer" nil (d ih0 s k l ey1 m er0))
("disclaimers" nil (d ih0 s k l ey1 m er0 z))
("disclaiming" nil (d ih0 s k l ey1 m ih0 ng))
("disclaims" nil (d ih0 s k l ey1 m z))
("discland" nil (d ih1 s k l ae1 n d))
("disclose" nil (d ih0 s k l ow1 z))
("disclosed" nil (d ih0 s k l ow1 z d))
("discloses" nil (d ih0 s k l ow1 z ih0 z))
("disclosing" nil (d ih0 s k l ow1 z ih0 ng))
("disclosure" nil (d ih0 s k l ow1 zh er0))
("disclosures" nil (d ih0 s k l ow1 zh er0 z))
("disco" nil (d ih1 s k ow0))
("discolor" nil (d ih0 s k ah1 l er0))
("discolors" nil (d ih0 s k ah1 l er0 z))
("discomfort" nil (d ih0 s k ah1 m f er0 t))
("disconcerting" nil (d ih1 s k ax n s er1 t ih0 ng))
("disconnect" nil (d ih0 s k ax n eh1 k t))
("disconnected" nil (d ih1 s k ax n eh1 k t ih0 d))
("disconnecting" nil (d ih1 s k ax n eh1 k t ih0 ng))
("disconnection" nil (d ih1 s k ax n eh1 k sh ax n))
("discontent" nil (d ih0 s k ax n t eh1 n t))
("discontented" nil (d ih1 s k ax n t eh1 n t ih0 d))
("discontents" nil (d ih1 s k ax n t eh1 n t s))
("discontinuance" nil (d ih1 s k ax n t ih1 n y uw0 ax n s))
("discontinuation" nil (d ih1 s k ax n t ih1 n y uw0 ey1 sh ax n))
("discontinue" nil (d ih0 s k ax n t ih1 n y uw0))
("discontinued" nil (d ih0 s k ax n t ih1 n y uw0 d))
("discontinuing" nil (d ih0 s k ax n t ih1 n y uw0 ih0 ng))
("discord" nil (d ih1 s k ao0 r d))
("discordant" nil (d ih0 s k ao1 r d ax n t))
("discos" nil (d ih1 s k ow0 z))
("discotheque" nil (d ih1 s k ow0 t eh1 k))
("discount" v (d ih0 s k aw1 n t))
("discount" n (d ih1 s k aw0 n t))
("discountable" nil (d ih1 s k aw1 n t ax b ax l))
("discounted" nil (d ih1 s k aw1 n t ih0 d))
("discounter" nil (d ih1 s k aw1 n t er0))
("discounters" nil (d ih0 s k aw1 n t er0 z))
("discounting" nil (d ih1 s k aw1 n t ih0 ng))
("discounts" v (d ih0 s k aw1 n t s))
("discounts" n (d ih1 s k aw0 n t s))
("discourage" nil (d ih0 s k er1 ih0 jh))
("discouraged" nil (d ih0 s k er1 ax jh d))
("discouragement" nil (d ih0 s k er1 ih0 jh m ax n t))
("discourages" nil (d ih0 s k er1 ih0 jh ih0 z))
("discouraging" nil (d ih0 s k er1 ax jh ih0 ng))
("discourse" nil (d ih1 s k ao0 r s))
("discourses" nil (d ih0 s k ao1 r s ih0 z))
("discover" nil (d ih0 s k ah1 v er0))
("discovered" nil (d ih0 s k ah1 v er0 d))
("discoveries" nil (d ih0 s k ah1 v er0 iy0 z))
("discovering" nil (d ih0 s k ah1 v er0 ih0 ng))
("discovers" nil (d ih0 s k ah1 v er0 z))
("discovery" nil (d ih0 s k ah1 v er0 iy0))
("discredit" nil (d ih0 s k r eh1 d ax t))
("discredited" nil (d ih0 s k r eh1 d ih0 t ih0 d))
("discrediting" nil (d ih0 s k r eh1 d ax t ih0 ng))
("discredits" nil (d ih0 s k r eh1 d ax t s))
("discreet" nil (d ih0 s k r iy1 t))
("discreetly" nil (d ih0 s k r iy1 t l iy0))
("discrepancies" nil (d ih0 s k r eh1 p ax n s iy0 z))
("discrepancy" nil (d ih0 s k r eh1 p ax n s iy0))
("discrete" nil (d ih0 s k r iy1 t))
("discretion" nil (d ih0 s k r eh1 sh ax n))
("discretionary" nil (d ih0 s k r eh1 sh ax n eh1 r iy0))
("discriminate" nil (d ih0 s k r ih1 m ax n ey1 t))
("discriminated" nil (d ih0 s k r ih1 m ax n ey1 t ax d))
("discriminates" nil (d ih0 s k r ih1 m ax n ey1 t s))
("discriminating" nil (d ih0 s k r ih1 m ax n ey1 t ih0 ng))
("discrimination" nil (d ih0 s k r ih1 m ax n ey1 sh ax n))
("discriminatory" nil (d ih0 s k r ih1 m ax n ax t ao1 r iy0))
("discs" nil (d ih1 s k s))
("discursive" nil (d ih0 s k er1 s ih0 v))
("discus" nil (d ih1 s k ax s))
("discuss" nil (d ih0 s k ah1 s))
("discussed" nil (d ih0 s k ah1 s t))
("discusses" nil (d ih0 s k ah1 s ax z))
("discussing" nil (d ih0 s k ah1 s ih0 ng))
("discussion" nil (d ih0 s k ah1 sh ax n))
("discussions" nil (d ih0 s k ah1 sh ax n z))
("disdain" nil (d ih0 s d ey1 n))
("disdained" nil (d ih0 s d ey1 n d))
("disdainful" nil (d ih0 s d ey1 n f ax l))
("disdaining" nil (d ih0 s d ey1 n ih0 ng))
("disdains" nil (d ih0 s d ey1 n z))
("dise" nil (d ay1 s))
("disease" nil (d ih0 z iy1 z))
("diseased" nil (d ih0 z iy1 z d))
("diseases" nil (d ih0 z iy1 z ax z))
("disembarkation" nil (d ih0 s eh1 m b aa0 r k ey1 sh ax n))
("disembodied" nil (d ih0 s ih0 b aa1 d iy0 d))
("disenchanted" nil (d ih0 s ih0 n ch ae1 n t ih0 d))
("disenchantment" nil (d ih0 s ih0 n ch ae1 n t m ax n t))
("disenfranchise" nil (d ih0 s ih0 n f r ae1 n ch ay1 z))
("disenfranchised" nil (d ih0 s ih0 n f r ae1 n ch ay1 z d))
("disenfranchisement" nil (d ih0 s ih0 n f r ae1 n ch ay1 z m ax n t))
("disengage" nil (d ih0 s ih0 n g ey1 jh))
("disengaged" nil (d ih0 s ih0 n g ey1 jh d))
("disengagement" nil (d ih0 s ih0 n g ey1 jh m ax n t))
("disengaging" nil (d ih0 s ih0 n g ey1 jh ih0 ng))
("disentangle" nil (d ih1 s ax n t ae1 ng g ax l))
("disequilibrium" nil (d ih0 s iy1 k w ax l ih1 b r iy0 ax m))
("disestablishment" nil (d ih0 s ih0 s t ae1 b l ih0 sh m ax n t))
("disfavor" nil (d ih0 s f ey1 v er0))
("disfigure" nil (d ih0 s f ih1 g y er0))
("disfigured" nil (d ih0 s f ih1 g y er0 d))
("disfiguring" nil (d ih0 s f ih1 g y er0 ih0 ng))
("disgorge" nil (d ih0 s g ao1 r jh))
("disgorged" nil (d ih0 s g ao1 r jh d))
("disgorgement" nil (d ih0 s g ao1 r jh m ax n t))
("disgrace" nil (d ih0 s g r ey1 s))
("disgraced" nil (d ih0 s g r ey1 s t))
("disgraceful" nil (d ih0 s g r ey1 s f ax l))
("disgruntled" nil (d ih0 s g r ah1 n t ax l d))
("disgruntlement" nil (d ih0 s g r ah1 n t ax l m ax n t))
("disgruntling" nil (d ih0 s g r ah1 n t ax l ih0 ng))
("disguise" nil (d ih0 s g ay1 z))
("disguised" nil (d ih0 s g ay1 z d))
("disguises" nil (d ih0 s g ay1 z ih0 z))
("disguising" nil (d ih0 s g ay1 z ih0 ng))
("disgust" nil (d ih0 s g ah1 s t))
("disgusted" nil (d ih0 s g ah1 s t ax d))
("disgusting" nil (d ih0 s g ah1 s t ih0 ng))
("disgusts" nil (d ih0 s g ah1 s t s))
("dish" nil (d ih1 sh))
("disharmony" nil (d ih0 s hh aa1 r m ax n iy0))
("disharoon" nil (d ih0 sh er0 uw1 n))
("dishaw" nil (d ih1 sh ao1))
("disheartened" nil (d ih0 s hh aa1 r t ax n d))
("disheartening" nil (d ih0 s hh aa1 r t ax n ih0 ng))
("dished" nil (d ih1 sh t))
("disher" nil (d ih1 sh er0))
("dishes" nil (d ih1 sh ax z))
("disheveled" nil (d ih0 sh eh1 v ax l d))
("dishing" nil (d ih1 sh ih0 ng))
("dishman" nil (d ih1 sh m ax n))
("dishner" nil (d ih1 sh n er0))
("dishon" nil (d ih1 s ax n))
("dishonest" nil (d ih0 s aa1 n ax s t))
("dishonestly" nil (d ih0 s ao1 n ax s t l iy0))
("dishonesty" nil (d ih0 s aa1 n ax s t iy0))
("dishong" nil (d ih1 s ao0 ng))
("dishonor" nil (d ih0 s aa1 n er0))
("dishonorable" nil (d ih0 s aa1 n er0 ax b ax l))
("dishwasher" nil (d ih1 sh w aa1 sh er0))
("dishwashers" nil (d ih1 sh w aa1 sh er0 z))
("dishwashing" nil (d ih1 sh w aa1 sh ih0 ng))
("disillusion" nil (d ih1 s ih0 l uw1 zh ax n))
("disillusioned" nil (d ih1 s ih0 l uw1 zh ax n d))
("disillusioning" nil (d ih1 s ih0 l uw1 zh ax n ih0 ng))
("disillusionment" nil (d ih1 s ih0 l uw1 zh ax n m ax n t))
("disilvestro" nil (d ih0 s iy0 l v eh1 s t r ow0))
("disimone" nil (d ih0 s iy0 m ow1 n iy0))
("disincentive" nil (d ih1 s ih0 n s eh1 n t ih0 v))
("disincentives" nil (d ih1 s ih0 n s eh1 n t ih0 v z))
("disinclination" nil (d ih0 s ih0 n k l ax n ey1 sh ax n))
("disinclined" nil (d ih1 s ih0 n k l ay1 n d))
("disinfectant" nil (d ih0 s ih0 n f eh1 k t ax n t))
("disinfectants" nil (d ih0 s ih0 n f eh1 k t ax n t s))
("disinfection" nil (d ih0 s ih0 n f eh1 k sh ax n))
("disinflation" nil (d ih1 s ih0 n f l ey1 sh ax n))
("disinflationary" nil (d ih1 s ih0 n f l ey1 sh ax n eh1 r iy0))
("disinformation" nil (d ih0 z ih1 n f er0 m ey1 sh ax n))
("disingenuous" nil (d ih0 s ih0 n jh eh1 n y uw0 ax s))
("disingenuousness" nil (d ih1 s ih0 n jh eh1 n y uw0 ax s n ax s))
("disintegrate" nil (d ih0 s ih1 n t ax g r ey1 t))
("disintegrated" nil (d ih0 s ih1 n t ax g r ey1 t ih0 d))
("disintegrates" nil (d ih0 s ih1 n t ax g r ey1 t s))
("disintegrating" nil (d ih0 s ih1 n t ax g r ey1 t ih0 ng))
("disintegration" nil (d ih0 s ih1 n t ax g r ey1 sh ax n))
("disinterest" nil (d ih0 s ih1 n t er0 ax s t))
("disinterested" nil (d ih0 s ih1 n t r ih0 s t ih0 d))
("disinvestment" nil (d ih0 s ih0 n v eh1 s t m ax n t))
("disjointed" nil (d ih0 s jh oy1 n t ih0 d))
("disk" nil (d ih1 s k))
("diskette" nil (d ih0 s k eh1 t))
("diskettes" nil (d ih0 s k eh1 t s))
("diskin" nil (d ih1 s k ih0 n))
("diskless" nil (d ih1 s k l ax s))
("disks" nil (d ih1 s k s))
("dislike" nil (d ih0 s l ay1 k))
("disliked" nil (d ih0 s l ay1 k t))
("dislikes" nil (d ih0 s l ay1 k s))
("dislocated" nil (d ih1 s l ow0 k ey0 t ih0 d))
("dislocation" nil (d ih0 s l ow1 k ey1 sh ax n))
("dislocations" nil (d ih0 s l ow1 k ey1 sh ax n z))
("dislodge" nil (d ih0 s l aa1 jh))
("dislodged" nil (d ih0 s l aa1 jh d))
("dislodging" nil (d ih0 s l aa1 jh ih0 ng))
("disloyal" nil (d ih0 s l oy1 ax l))
("disloyalty" nil (d ih0 s l oy1 ax l t iy0))
("dismal" nil (d ih1 z m ax l))
("dismally" nil (d ih1 z m ax l iy0))
("dismantle" nil (d ih0 s m ae1 n t ax l))
("dismantled" nil (d ih0 s m ae1 n t ax l d))
("dismantlement" nil (d ih0 s m ae1 n t ax l m ax n t))
("dismantles" nil (d ih0 s m ae1 n t ax l z))
("dismantling" nil (d ih0 s m ae1 n t ax l ih0 ng))
("dismay" nil (d ih0 s m ey1))
("dismayed" nil (d ih0 s m ey1 d))
("dismaying" nil (d ih0 s m ey1 ih0 ng))
("dismays" nil (d ih0 s m ey1 z))
("dismember" nil (d ih0 s m eh1 m b er0))
("dismembered" nil (d ih0 s m eh1 m b er0 d))
("dismembering" nil (d ih0 s m eh1 m b er0 ih0 ng))
("dismemberment" nil (d ih0 s m eh1 m b er0 m ax n t))
("dismiss" nil (d ih0 s m ih1 s))
("dismissal" nil (d ih0 s m ih1 s ax l))
("dismissals" nil (d ih0 s m ih1 s ax l z))
("dismissed" nil (d ih0 s m ih1 s t))
("dismisses" nil (d ih0 s m ih1 s ih0 z))
("dismissing" nil (d ih0 s m ih1 s ih0 ng))
("dismissive" nil (d ih0 s m ih1 s ih0 v))
("dismore" nil (d ih1 s m ao0 r))
("dismuke" nil (d ih1 s m y uw0 k))
("dismukes" nil (d ih1 s m y uw0 k s))
("dismutase" nil (d ih1 s m y uw0 t ey1 z))
("disney" nil (d ih1 z n iy0))
("disneyland" nil (d ih1 z n iy0 l ae1 n d))
("disobedience" nil (d ih1 s ax b iy1 d iy0 ax n s))
("disobey" nil (d ih1 s ax b ey1))
("disobeyed" nil (d ih1 s ow0 b ey1 d))
("dison" nil (d ih1 s ax n))
("disorder" nil (d ih0 s ao1 r d er0))
("disorderly" nil (d ih0 s ao1 r d er0 l iy0))
("disorders" nil (d ih0 s ao1 r d er0 z))
("disorganization" nil (d ih0 s ao1 r g ax n ax z ey1 sh ax n))
("disorganized" nil (d ih0 s ao1 r g ax n ay1 z d))
("disorient" nil (d ih0 s ao1 r iy0 eh1 n t))
("disorientation" nil (d ih0 s ao1 r iy0 ax n t ey1 sh ax n))
("disoriented" nil (d ih0 s ao1 r iy0 eh1 n t ih0 d))
("disorienting" nil (d ih0 s ao1 r iy0 eh1 n t ih0 ng))
("disowned" nil (d ih0 s ow1 n d))
("dispair" nil (d ih0 s p eh1 r))
("disparage" nil (d ih0 s p eh1 r ih0 jh))
("disparaged" nil (d ih0 s p eh1 r ih0 jh d))
("disparages" nil (d ih0 s p eh1 r ih0 jh ih0 z))
("disparaging" nil (d ih0 s p eh1 r ih0 jh ih0 ng))
("disparagingly" nil (d ih0 s p eh1 r ih0 jh ih0 ng l iy0))
("disparate" nil (d ih1 s p er0 ih0 t))
("disparities" nil (d ih0 s p eh1 r ax t iy0 z))
("disparity" nil (d ih0 s p eh1 r ax t iy0))
("dispassionate" nil (d ih0 s p ae1 sh ax n ax t))
("dispassionately" nil (d ih0 s p ae1 sh ax n ax t l iy0))
("dispatch" nil (d ih0 s p ae1 ch))
("dispatched" nil (d ih0 s p ae1 ch t))
("dispatcher" nil (d ih0 s p ae1 ch er0))
("dispatchers" nil (d ih0 s p ae1 ch er0 z))
("dispatches" nil (d ih0 s p ae1 ch ih0 z))
("dispatching" nil (d ih0 s p ae1 ch ih0 ng))
("dispel" nil (d ih0 s p eh1 l))
("dispell" nil (d ih0 s p eh1 l))
("dispelled" nil (d ih0 s p eh1 l d))
("dispelling" nil (d ih0 s p eh1 l ih0 ng))
("dispels" nil (d ih0 s p eh1 l z))
("dispensable" nil (d ih0 s p eh1 n s ax b ax l))
("dispensation" nil (d ih1 s p ax n s ey1 sh ax n))
("dispense" nil (d ih0 s p eh1 n s))
("dispensed" nil (d ih0 s p eh1 n s t))
("dispenser" nil (d ih0 s p eh1 n s er0))
("dispensers" nil (d ih0 s p eh1 n s er0 z))
("dispenses" nil (d ih0 s p eh1 n s ih0 z))
("dispensing" nil (d ih0 s p eh1 n s ih0 ng))
("dispenza" nil (d ih0 s p eh1 n z ax))
("dispersal" nil (d ih0 s p er1 s ax l))
("dispersant" nil (d ih1 s p er1 s ax n t))
("dispersants" nil (d ih1 s p er1 s ax n t s))
("disperse" nil (d ih0 s p er1 s))
("dispersed" nil (d ih0 s p er1 s t))
("dispersing" nil (d ih0 s p er1 s ih0 ng))
("dispersion" nil (d ih0 s p er1 zh ax n))
("dispersive" nil (d ih0 s p er1 z ih0 v))
("dispirited" nil (d ih0 s p ih1 r ax t ih0 d))
("dispiriting" nil (d ih0 s p ih1 r ih0 t ih0 ng))
("displace" nil (d ih0 s p l ey1 s))
("displaced" nil (d ih0 s p l ey1 s t))
("displacement" nil (d ih0 s p l ey1 s m ax n t))
("displacements" nil (d ih0 s p l ey1 s m ax n t s))
("displaces" nil (d ih0 s p l ey1 s ih0 z))
("displacing" nil (d ih0 s p l ey1 s ih0 ng))
("display" nil (d ih0 s p l ey1))
("displayed" nil (d ih0 s p l ey1 d))
("displaying" nil (d ih0 s p l ey1 ih0 ng))
("displays" nil (d ih0 s p l ey1 z))
("displaywrite" nil (d ih0 s p l ey1 r ay1 t))
("displease" nil (d ih0 s p l iy1 z))
("displeased" nil (d ih0 s p l iy1 z d))
("displeasure" nil (d ih0 s p l eh1 zh er0))
("disposable" nil (d ih0 s p ow1 z ax b ax l))
("disposables" nil (d ih0 s p ow1 z ax b ax l z))
("disposal" nil (d ih0 s p ow1 z ax l))
("disposals" nil (d ih0 s p ow1 z ax l z))
("dispose" nil (d ih0 s p ow1 z))
("disposed" nil (d ih0 s p ow1 z d))
("disposer" nil (d ih0 s p ow1 z er0))
("disposes" nil (d ih0 s p ow1 z ih0 z))
("disposing" nil (d ih0 s p ow1 z ih0 ng))
("disposition" nil (d ih1 s p ax z ih1 sh ax n))
("dispositions" nil (d ih1 s p ax z ih1 sh ax n z))
("dispossessed" nil (d ih1 s p ax z eh1 s t))
("disproportionate" nil (d ih1 s p r ax p ao1 r sh ax n ih0 t))
("disproportionately" nil (d ih1 s p r ax p ao1 r sh ax n ax t l iy0))
("disprove" nil (d ih0 s p r uw1 v))
("disproved" nil (d ih0 s p r uw1 v d))
("disproves" nil (d ih0 s p r uw1 v z))
("disputation" nil (d ih0 s p y uw1 t ey1 sh ax n))
("dispute" nil (d ih0 s p y uw1 t))
("disputed" nil (d ih0 s p y uw1 t ax d))
("disputes" nil (d ih0 s p y uw1 t s))
("disputing" nil (d ih0 s p y uw1 t ih0 ng))
("disqualification" nil (d ih0 s k w aa1 l ax f ax k ey1 sh ax n))
("disqualified" nil (d ih0 s k w aa1 l ax f ay1 d))
("disqualifies" nil (d ih0 s k w aa1 l ax f ay1 z))
("disqualify" nil (d ih0 s k w aa1 l ax f ay1))
("disqualifying" nil (d ih0 s k w aa1 l ax f ay1 ih0 ng))
("disque" nil (d ih1 s k))
("disquiet" nil (d ih0 s k w ay1 ax t))
("disquieting" nil (d ih0 s k w ay1 ax t ih0 ng))
("disraeli" nil (d ih0 z r ey1 l iy0))
("disregard" nil (d ih1 s r ih0 g aa1 r d))
("disregarded" nil (d ih1 s r ih0 g aa1 r d ih0 d))
("disregarding" nil (d ih1 s r ih0 g aa1 r d ih0 ng))
("disregards" nil (d ih1 s r ih0 g aa1 r d z))
("disrepair" nil (d ih1 s r ih0 p eh1 r))
("disreputable" nil (d ih0 s r eh1 p y ax t ax b ax l))
("disrepute" nil (d ih1 s r ih0 p y uw1 t))
("disrespect" nil (d ih1 s r ih0 s p eh1 k t))
("disrespected" nil (d ih1 s r ih0 s p eh1 k t ih0 d))
("disrespectful" nil (d ih1 s r ih0 s p eh1 k t f ax l))
("disrespecting" nil (d ih1 s r ih0 s p eh1 k t ih0 ng))
("disrespects" nil (d ih1 s r ih0 s p eh1 k t s))
("disrupt" nil (d ih0 s r ah1 p t))
("disrupted" nil (d ih0 s r ah1 p t ih0 d))
("disrupting" nil (d ih0 s r ah1 p t ih0 ng))
("disruption" nil (d ih0 s r ah1 p sh ax n))
("disruptions" nil (d ih0 s r ah1 p sh ax n z))
("disruptive" nil (d ih0 s r ah1 p t ih0 v))
("disrupts" nil (d ih0 s r ah1 p t s))
("diss" nil (d ih1 s))
("dissatisfaction" nil (d ih1 s ae0 t ih0 s f ae1 k sh ax n))
("dissatisfied" nil (d ih0 s ae1 t ax s f ay1 d))
("dissect" nil (d ay0 s eh1 k t))
("dissected" nil (d ay0 s eh1 k t ax d))
("dissecting" nil (d ay0 s eh1 k t ih0 ng))
("dissection" nil (d ay0 s eh1 k sh ax n))
("dissections" nil (d ay0 s eh1 k sh ax n z))
("dissects" nil (d ay0 s eh1 k t s))
("disseminate" nil (d ih0 s eh1 m ax n ey1 t))
("disseminated" nil (d ih0 s eh1 m ax n ey1 t ax d))
("disseminating" nil (d ih0 s eh1 m ax n ey1 t ih0 ng))
("dissemination" nil (d ih0 s eh1 m ax n ey1 sh ax n))
("dissension" nil (d ih0 s eh1 n sh ax n))
("dissent" nil (d ih0 s eh1 n t))
("dissented" nil (d ih0 s eh1 n t ih0 d))
("dissenter" nil (d ih0 s eh1 n t er0))
("dissenters" nil (d ih0 s eh1 n t er0 z))
("dissenting" nil (d ih0 s eh1 n t ih0 ng))
("dissents" nil (d ih0 s eh1 n t s))
("dissertation" nil (d ih1 s er0 t ey1 sh ax n))
("disservice" nil (d ih0 s s er1 v ax s))
("dissidence" nil (d ih1 s ax d ih0 n s))
("dissident" nil (d ih1 s ax d ih0 n t))
("dissidents" nil (d ih1 s ax d ax n t s))
("dissimilar" nil (d ih0 s s ih1 m ax l er0))
("dissimilarity" nil (d ih0 s s ih1 m ax l ae1 r ax t iy0))
("dissinger" nil (d ih1 s ih0 n jh er0))
("dissipate" nil (d ih1 s ax p ey1 t))
("dissipated" nil (d ih1 s ax p ey1 t ih0 d))
("dissipates" nil (d ih1 s ax p ey1 t s))
("dissipating" nil (d ih1 s ax p ey1 t ih0 ng))
("dissipation" nil (d ih1 s ih0 p ey1 sh ax n))
("dissipative" nil (d ih1 s ax p ey1 t ih0 v))
("dissociate" nil (d ih0 s ow1 s iy0 ey0 t))
("dissociation" nil (d ih0 s ow1 s iy0 ey1 sh ax n))
("dissolution" nil (d ih1 s ax l uw1 sh ax n))
("dissolve" nil (d ih0 z aa1 l v))
("dissolved" nil (d ih0 z aa1 l v d))
("dissolver" nil (d ih0 z aa1 l v er0))
("dissolvers" nil (d ih0 z aa1 l v er0 z))
("dissolves" nil (d ih0 z aa1 l v z))
("dissolving" nil (d ih0 z ao1 l v ih0 ng))
("dissonance" nil (d ih1 s ax n ax n s))
("dissonant" nil (d ih1 s ax n ax n t))
("dissuade" nil (d ih0 s w ey1 d))
("dissuaded" nil (d ih0 s w ey1 d ih0 d))
("dissymmetry" nil (d ih0 s ih1 m ax t r iy0))
("distad" nil (d ih1 s t ax d))
("distal" nil (d ih1 s t ax l))
("distance" nil (d ih1 s t ax n s))
("distanced" nil (d ih1 s t ax n s t))
("distances" nil (d ih1 s t ax n s ax z))
("distancing" nil (d ih1 s t ax n s ih0 ng))
("distant" nil (d ih1 s t ax n t))
("distasi" nil (d ih0 s t aa1 s iy0))
("distasio" nil (d ih0 s t aa1 s iy0 ow0))
("distaste" nil (d ih0 s t ey1 s t))
("distasteful" nil (d ih0 s t ey1 s t f ax l))
("distefano" nil (d ih0 s t eh0 f aa1 n ow0))
("distel" nil (d ih1 s t ax l))
("distemper" nil (d ih0 s t eh1 m p er0))
("distended" nil (d ih0 s t eh1 n d ih0 d))
("distill" nil (d ih0 s t ih1 l))
("distillate" nil (d ih1 s t ax l ey1 t))
("distillates" nil (d ih1 s t ih0 l ey1 t s))
("distillation" nil (d ih1 s t ih0 l ey1 sh ax n))
("distilled" nil (d ih0 s t ih1 l d))
("distiller" nil (d ih0 s t ih1 l er0))
("distilleries" nil (d ih0 s t ih1 l er0 iy0 z))
("distillers" nil (d ih0 s t ih1 l er0 z))
("distillery" nil (d ih0 s t ih1 l er0 iy0))
("distilling" nil (d ih0 s t ih1 l ih0 ng))
("distills" nil (d ih0 s t ih1 l z))
("distinct" nil (d ih0 s t ih1 ng k t))
("distinction" nil (d ih0 s t ih1 ng k sh ax n))
("distinctions" nil (d ih0 s t ih1 ng k sh ax n z))
("distinctive" nil (d ih0 s t ih1 ng k t ih0 v))
("distinctively" nil (d ih0 s t ih1 ng k t ih0 v l iy0))
("distinctiveness" nil (d ih0 s t ih1 ng k t ih0 v n ax s))
("distinctly" nil (d ih0 s t ih1 ng k t l iy0))
("distinguish" nil (d ih0 s t ih1 ng g w ih0 sh))
("distinguishable" nil (d ih0 s t ih1 ng g w ih0 sh ax b ax l))
("distinguished" nil (d ih0 s t ih1 ng g w ih0 sh t))
("distinguishes" nil (d ih0 s t ih1 ng g w ih0 sh ih0 z))
("distinguishing" nil (d ih0 s t ih1 ng g w ih0 sh ih0 ng))
("distler" nil (d ih1 s t l er0))
("distort" nil (d ih0 s t ao1 r t))
("distorted" nil (d ih0 s t ao1 r t ax d))
("distorting" nil (d ih0 s t ao1 r t ih0 ng))
("distortion" nil (d ih0 s t ao1 r sh ax n))
("distortions" nil (d ih0 s t ao1 r sh ax n z))
("distorts" nil (d ih0 s t ao1 r t s))
("distract" nil (d ih0 s t r ae1 k t))
("distracted" nil (d ih0 s t r ae1 k t ax d))
("distracting" nil (d ih0 s t r ae1 k t ih0 ng))
("distraction" nil (d ih0 s t r ae1 k sh ax n))
("distractions" nil (d ih0 s t r ae1 k sh ax n z))
("distracts" nil (d ih0 s t r ae1 k t s))
("distraught" nil (d ih0 s t r ao1 t))
("distress" nil (d ih0 s t r eh1 s))
("distressed" nil (d ih0 s t r eh1 s t))
("distresses" nil (d ih0 s t r eh1 s ih0 z))
("distressing" nil (d ih0 s t r eh1 s ih0 ng))
("distressingly" nil (d ih0 s t r eh1 s ih0 ng l iy0))
("distribute" nil (d ih0 s t r ih1 b y uw0 t))
("distributed" nil (d ih0 s t r ih1 b y ax t ax d))
("distributes" nil (d ih0 s t r ih1 b y uw0 t s))
("distributing" nil (d ih0 s t r ih1 b y uw0 t ih0 ng))
("distribution" nil (d ih1 s t r ax b y uw1 sh ax n))
("distributions" nil (d ih1 s t r ax b y uw1 sh ax n z))
("distributive" nil (d ih0 s t r ih1 b y uw0 t ih0 v))
("distributor" nil (d ih0 s t r ih1 b y ax t er0))
("distributors" nil (d ih0 s t r ih1 b y ax t er0 z))
("distributorship" nil (d ih0 s t r ih1 b y uw0 t er0 sh ih1 p))
("distributorships" nil (d ih0 s t r ih1 b y uw0 t er0 sh ih1 p s))
("district" nil (d ih1 s t r ih0 k t))
("districts" nil (d ih1 s t r ih0 k t s))
("distrigas" nil (d ih0 s t r iy1 g ax s))
("distron" nil (d ih1 s t r aa1 n))
("distrust" nil (d ih0 s t r ah1 s t))
("distrusted" nil (d ih0 s t r ah1 s t ax d))
("distrustful" nil (d ih0 s t r ah1 s t f ax l))
("distrusts" nil (d ih0 s t r ah1 s t s))
("disturb" nil (d ih0 s t er1 b))
("disturbance" nil (d ih0 s t er1 b ax n s))
("disturbances" nil (d ih0 s t er1 b ax n s ax z))
("disturbed" nil (d ih0 s t er1 b d))
("disturbing" nil (d ih0 s t er1 b ih0 ng))
("disturbingly" nil (d ih0 s t er1 b ih0 ng l iy0))
("disturbs" nil (d ih0 s t er1 b z))
("disunion" nil (d ih0 s y uw1 n y ax n))
("disunity" nil (d ih0 s y uw1 n ax t iy0))
("disuse" nil (d ih0 s y uw1 s))
("ditch" nil (d ih1 ch))
("ditched" nil (d ih1 ch t))
("ditches" nil (d ih1 ch ax z))
("ditching" nil (d ih1 ch ih0 ng))
("dith" nil (d ih1 th))
("dithering" nil (d ih1 dh er0 ih0 ng))
("dithers" nil (d ih1 dh er0 z))
("dithyramb" nil (d ih1 th er0 ae1 m))
("ditka" nil (d ih1 t k ax))
("ditlow" nil (d ih1 t l ow0))
("ditmars" nil (d ih1 t m er0 z))
("ditmer" nil (d ih1 t m er0))
("ditmore" nil (d ih1 t m ao0 r))
("ditomasso" nil (d ih0 t ow0 m aa1 s ow0))
("ditommaso" nil (d ih0 t ow0 m aa1 s ow0))
("dittberner" nil (d ih1 t b er0 n er0))
("dittemore" nil (d ih0 t eh1 m ao0 r))
("ditter" nil (d ih1 t er0))
("ditties" nil (d ih1 t iy0 z))
("dittman" nil (d ih1 t m ax n))
("dittmann" nil (d ih1 t m ax n))
("dittmar" nil (d ih1 t m er0))
("dittmer" nil (d ih1 t m er0))
("ditto" nil (d ih1 t ow0))
("ditton" nil (d ih1 t ax n))
("dittrich" nil (d ih1 t r ih0 k))
("dittus" nil (d ih1 t ax s))
("ditty" nil (d ih1 t iy0))
("ditullio" nil (d ih0 t ah1 l iy0 ow0))
("ditzel" nil (d ih1 t z ax l))
("ditzler" nil (d ih1 t s l er0))
("diuretic" nil (d ay1 uw0 r eh1 t ih0 k))
("diuretics" nil (d ay1 uw0 r eh1 t ih0 k s))
("diurnal" nil (d ay0 er1 n ax l))
("diurnally" nil (d ay0 er1 n ax l iy0))
("diva" nil (d iy1 v ax))
("divad" nil (d ih1 v ae0 d))
("divalent" nil (d ay0 v ey1 l ax n t))
("divall" nil (d ih1 v ax l))
("divan" nil (d ih0 v ae1 n))
("dive" nil (d ay1 v))
("dived" nil (d ay1 v d))
("divelbiss" nil (d ih0 v eh1 l b ih0 s))
("diveley" nil (d ih1 v ih0 l iy0))
("dively" nil (d ay1 v l iy0))
("diven" nil (d ay1 v ax n))
("divens" nil (d ay1 v ax n z))
("diver" nil (d ay1 v er0))
("diverge" nil (d ih0 v er1 jh))
("diverged" nil (d ay0 v er1 jh d))
("divergence" nil (d ay0 v er1 jh ax n s))
("divergences" nil (d ay0 v er1 jh ax n s ih0 z))
("divergent" nil (d ay0 v er1 jh ax n t))
("diverges" nil (d ay0 v er1 jh ih0 z))
("diverging" nil (d ay0 v er1 jh ih0 ng))
("divers" nil (d ay1 v er0 z))
("diverse" nil (d ay0 v er1 s))
("diversicare" nil (d ih1 v er0 s ih0 k eh1 r))
("diversification" nil (d ay0 v er1 s ax f ax k ey1 sh ax n))
("diversifications" nil (d ay0 v er1 s ax f ax k ey1 sh ax n z))
("diversified" nil (d ay0 v er1 s ax f ay1 d))
("diversify" nil (d ay0 v er1 s ax f ay1))
("diversifying" nil (d ay0 v er1 s ax f ay1 ih0 ng))
("diversion" nil (d ay0 v er1 zh ax n))
("diversionary" nil (d ay0 v er1 zh ax n eh1 r iy0))
("diversions" nil (d ih0 v er1 zh ax n z))
("diversity" nil (d ih0 v er1 s ax t iy0))
("divert" nil (d ay0 v er1 t))
("diverted" nil (d ay0 v er1 t ih0 d))
("diverticula" nil (d ay1 v er0 t ih1 k y ax l ax))
("diverticulum" nil (d ay1 v er0 t ih1 k y ax l ax m))
("divertimento" nil (d ih0 v er1 t ax m eh1 n t ow1))
("diverting" nil (d ay0 v er1 t ih0 ng))
("diverts" nil (d ay0 v er1 t s))
("dives" nil (d ay1 v z))
("divest" nil (d ay0 v eh1 s t))
("divested" nil (d ay0 v eh1 s t ih0 d))
("divesting" nil (d ay0 v eh1 s t ih0 ng))
("divestiture" nil (d ih0 v eh1 s t ih0 ch er0))
("divestitures" nil (d ih0 v eh1 s t ih0 ch er0 z))
("divestment" nil (d ay0 v eh1 s t m ax n t))
("divestments" nil (d ay0 v eh1 s t m ax n t s))
("divests" nil (d ay0 v eh1 s t s))
("divi" nil (d iy1 v iy0))
("divide" nil (d ih0 v ay1 d))
("divided" nil (d ih0 v ay1 d ax d))
("dividend" nil (d ih1 v ih0 d eh1 n d))
("dividends" nil (d ih1 v ax d eh1 n d z))
("divider" nil (d ih0 v ay1 d er0))
("divides" nil (d ih0 v ay1 d z))
("dividing" nil (d ih0 v ay1 d ih0 ng))
("divination" nil (d ih1 v ax n ey1 sh ax n))
("divincenzo" nil (d ih0 v iy0 n ch eh1 n z ow0))
("divine" nil (d ih0 v ay1 n))
("diviney" nil (d ih1 v ih0 n iy0))
("diving" nil (d ay1 v ih0 ng))
("divining" nil (d ax v ay1 n ih0 ng))
("divinitas" nil (d ih1 v ih0 n iy1 t ax s))
("divinities" nil (d ih0 v ih1 n ax t iy0 z))
("divinity" nil (d ih0 v ih1 n ax t iy0))
("divirgilio" nil (d ih0 v ih0 r jh iy1 l iy0 ow0))
("divis" nil (d iy1 v ih0 s))
("divisible" nil (d ih0 v ih1 z ax b ax l))
("division" nil (d ih0 v ih1 zh ax n))
("divisional" nil (d ih0 v ih1 zh ax n ax l))
("divisions" nil (d ih0 v ih1 zh ax n z))
("divisive" nil (d ih0 v ay1 s ih0 v))
("divisiveness" nil (d ih0 v ay1 s ih0 v n ax s))
("divison" nil (d ih0 v ih1 zh ax n))
("divisor" nil (d ih0 v ay1 z er0))
("divita" nil (d ih0 v iy1 t ax))
("divito" nil (d ih0 v iy1 t ow0))
("divorce" nil (d ih0 v ao1 r s))
("divorced" nil (d ih0 v ao1 r s t))
("divorcee" nil (d ax v ao1 r s iy1))
("divorces" nil (d ih0 v ao1 r s ih0 z))
("divorcing" nil (d ih0 v ao1 r s ih0 ng))
("divulge" nil (d ih0 v ah1 l jh))
("divulged" nil (d ih0 v ah1 l jh d))
("divulging" nil (d ih0 v ah1 l jh ih0 ng))
("divvied" nil (d ih1 v iy0 d))
("divvy" nil (d ih1 v iy0))
("dix" nil (d ih1 k s))
("dixie" nil (d ih1 k s iy0))
("dixieland" nil (d ih1 k s iy0 l ae1 n d))
("dixon" nil (d ih1 k s ax n))
("dixons" nil (d ih1 k s ax n z))
("dixson" nil (d ih1 k s ax n))
("dizon" nil (d ih1 z ax n))
("dizziness" nil (d ih1 z iy0 n ax s))
("dizzy" nil (d ih1 z iy0))
("dizzying" nil (d ih1 z iy0 ih0 ng))
("djakarta" nil (jh ax k aa1 r t ax))
("djibouti" nil (jh ih0 b uw1 t iy1))
("djurdjevic" nil (jh er1 jh ax v ih0 k))
("dk" nil (d iy1 k ey1))
("dlouhy" nil (d ax l aw1 iy0))
("dlugos" nil (d ax l uw1 g ow0 s))
("dlugosz" nil (d ax l uw1 g ow0 s))
("dmitri" nil (d ax m iy1 t r iy0))
("do" nil (d uw1))
("doable" nil (d uw1 ax b ax l))
("doak" nil (d ow1 k))
("doan" nil (d ow1 n))
("doane" nil (d ow1 n))
("doanh" nil (d ow1 n))
("doanna" nil (d ow1 n ax))
("dobb" nil (d aa1 b))
("dobberstein" nil (d aa1 b er0 s t iy1 n))
("dobbie" nil (d aa1 b iy0))
("dobbin" nil (d aa1 b ih0 n))
("dobbins" nil (d aa1 b ih0 n z))
("dobbs" nil (d aa1 b z))
("dobek" nil (d ow1 b ih0 k))
("dober" nil (d ow1 b er0))
("doberman" nil (d ow1 b er0 m ax n))
("doberstein" nil (d ow1 b er0 s t ay0 n))
("dobesh" nil (d aa1 b ih0 sh))
("dobey" nil (d aa1 b iy0))
("dobias" nil (d ow0 b iy1 ax z))
("dobie" nil (d aa1 b iy0))
("dobies" nil (d ow1 b iy0 z))
("dobis" nil (d ow1 b ih0 s))
("dobkin" nil (d aa1 b k ih0 n))
("dobkins" nil (d aa1 b k ih0 n z))
("doble" nil (d ow1 b ax l))
("dobler" nil (d ow1 b ax l er0))
("dobmeier" nil (d aa1 b m ay0 er0))
("dobos" nil (d ow1 b ow0 z))
("dobosz" nil (d aa1 b ax sh))
("dobransky" nil (d ax b r ae1 n s k iy0))
("dobratz" nil (d aa1 b r ax t s))
("dobrin" nil (d aa1 b r ih0 n))
("dobrins" nil (d aa1 b r ih0 n z))
("dobrinski" nil (d ax b r ih1 n s k iy0))
("dobrowolski" nil (d ax b r aw0 ow1 l s k iy0))
("dobrowski" nil (d ax b r ao1 f s k iy0))
("dobry" nil (d aa1 b r iy0))
("dobrynin" nil (d aa1 b r ih0 n ih0 n))
("dobrzynski" nil (d ow1 b r ax zh ih1 n s k iy0))
("dobson" nil (d aa1 b s ax n))
("dobsons" nil (d aa1 b s ax n z))
("doby" nil (d ow1 b iy0))
("dobyns" nil (d ow1 b ih0 n z))
("doc" nil (d aa1 k))
("docherty" nil (d aa1 ch er0 t iy0))
("dochow" nil (d ow1 ch aw0))
("dochterman" nil (d aa1 k t er0 m ax n))
("docie" nil (d aa1 k iy0))
("docila" nil (d aa1 s ih0 l ax))
("docile" nil (d aa1 s ax l))
("docility" nil (d aa0 s ih1 l ax t iy0))
("dock" nil (d aa1 k))
("docked" nil (d aa1 k t))
("docken" nil (d aa1 k ax n))
("dockendorf" nil (d aa1 k ih0 n d ao0 r f))
("dockers" nil (d aa1 k er0 z))
("dockery" nil (d aa1 k er0 iy0))
("docket" nil (d aa1 k ax t))
("dockets" nil (d aa1 k ax t s))
("dockham" nil (d aa1 k hh ax m))
("docking" nil (d aa1 k ih0 ng))
("dockins" nil (d aa1 k ih0 n z))
("docklands" nil (d aa1 k l ax n d z))
("docks" nil (d aa1 k s))
("dockside" nil (d aa1 k s ay1 d))
("dockson" nil (d aa1 k s ax n))
("dockstader" nil (d aa1 k s t ax d er0))
("dockter" nil (d aa1 k t er0))
("dockworkers" nil (d aa1 k w er1 k er0 z))
("dockyard" nil (d aa1 k y aa1 r d))
("docter" nil (d aa1 k t er0))
("doctor" nil (d aa1 k t er0))
("doctoral" nil (d aa1 k t er0 ax l))
("doctorate" nil (d aa1 k t er0 ax t))
("doctorates" nil (d aa1 k t er0 ax t s))
("doctored" nil (d aa1 k t er0 d))
("doctoring" nil (d aa1 k t er0 ih0 ng))
("doctors" nil (d aa1 k t er0 z))
("doctrinaire" nil (d aa1 k t r ax n eh1 r))
("doctrinal" nil (d aa1 k t r ax n ax l))
("doctrine" nil (d aa1 k t r ax n))
("doctrines" nil (d aa1 k t r ax n z))
("docudrama" nil (d ow1 k ax d r ae1 m ax))
("document" nil (d aa1 k y ax m eh0 n t))
("documenta" nil (d aa1 k y uw0 m eh1 n t ax))
("documentaries" nil (d aa1 k y ax m eh1 n t er0 iy0 z))
("documentary" nil (d aa1 k y ax m eh1 n t er0 iy0))
("documentation" nil (d aa1 k y ax m eh0 n t ey1 sh ax n))
("documented" nil (d aa1 k y ax m eh1 n t ax d))
("documenting" nil (d aa1 k y ax m ax n t ih0 ng))
("documents" nil (d aa1 k y ax m ax n t s))
("dodaro" nil (d ow0 d aa1 r ow0))
("dodd" nil (d aa1 d))
("doddering" nil (d aa1 d er0 ih0 ng))
("doddington" nil (d aa1 d ih0 n t ax n))
("doddridge" nil (d aa1 d r ih0 jh))
("dodds" nil (d aa1 d z))
("dodge" nil (d aa1 jh))
("dodged" nil (d aa1 jh d))
("dodgen" nil (d aa1 jh ax n))
("dodger" nil (d aa1 jh er0))
("dodgers" nil (d aa1 jh er0 z))
("dodges" nil (d aa1 jh ih0 z))
("dodging" nil (d aa1 jh ih0 ng))
("dodo" nil (d ow1 d ow0))
("dodos" nil (d ow1 d ow0 z))
("dodrill" nil (d aa1 d r ax l))
("dodson" nil (d aa1 d s ax n))
("dodsworth" nil (d aa1 d s w er0 th))
("doe" nil (d ow1))
("doebler" nil (d ow1 b ax l er0))
("doeden" nil (d ow1 d ax n))
("doege" nil (d ow1 jh))
("doehring" nil (d ao1 r ih0 ng))
("doell" nil (d ow1 l))
("doenges" nil (d ow1 n jh ih0 z))
("doepke" nil (d ow1 p k))
("doepker" nil (d ow1 p k er0))
("doer" nil (d uw1 r))
("doerfler" nil (d ao1 r f ax l er0))
("doerflinger" nil (d ao1 r f ax l ih0 ng er0))
("doering" nil (d uw1 er0 ih0 ng))
("doernberg" nil (d ao1 r n b er0 g))
("doerner" nil (d ao1 r n er0))
("doerr" nil (d ao1 r))
("doers" nil (d uw1 er0 z))
("doersam" nil (d ao1 r s ax m))
("does" v (d ah1 z))
("does" n (d ow1 z))
("doescher" nil (d ow1 sh er0))
("doetsch" nil (d ow1 ch))
("dofasco" nil (d ax f ae1 s k ow0))
("doffing" nil (d ao1 f ih0 ng))
("dog" nil (d ao1 g))
("dogan" nil (d ow1 g ax n))
("dogbane" nil (d ao1 g b ey1 n))
("dogberry" nil (d ao1 g b eh1 r iy0))
("doge" nil (d ow1 jh))
("dogfight" nil (d aa1 g f ay1 t))
("dogfights" nil (d ao1 g f ay1 t s))
("dogfish" nil (d ao1 g f ih1 sh))
("dogged" nil (d ao1 g d))
("doggedly" nil (d ao1 g ax d l iy0))
("doggerel" nil (d aa1 g er0 ax l))
("doggett" nil (d aa1 g ih0 t))
("doggie" nil (d ao1 g iy0))
("doggies" nil (d ao1 g iy0 z))
("dogging" nil (d ao1 g ih0 ng))
("doggy" nil (d ao1 g iy0))
("doghouse" nil (d ao1 g hh aw1 s))
("doglike" nil (d ao1 g l ay1 k))
("dogma" nil (d aa1 g m ax))
("dogmatic" nil (d aa0 g m ae1 t ih0 k))
("dogmatically" nil (d aa0 g m ae1 t ih0 k l iy0))
("dogmatism" nil (d aa1 g m ax t ih1 z ax m))
("dogs" nil (d aa1 g z))
("dogwood" nil (d ao1 g w uh1 d))
("dogwoods" nil (d ao1 g w uh1 d z))
("doh" nil (d ow1))
("doheny" nil (d aa1 hh ih0 n iy0))
("doherty" nil (d ow1 er0 t iy0))
("dohm" nil (d aa1 m))
("dohman" nil (d ow1 m ax n))
("dohme" nil (d ow1 m))
("dohmen" nil (d ow1 m eh0 n))
("dohn" nil (d aa1 n))
("dohnanyi" nil (d ow1 n aa1 n y iy0))
("dohner" nil (d ow1 n er0))
("dohr" nil (d ao1 r))
("dohrman" nil (d ao1 r m ax n))
("dohrmann" nil (d ao1 r m ax n))
("dohse" nil (d ow1 s))
("doi" nil (d oy1))
("doidge" nil (d oy1 jh))
("doig" nil (d oy1 g))
("doilies" nil (d oy1 l iy0 z))
("doing" nil (d uw1 ih0 ng))
("doings" nil (d uw1 ih0 ng z))
("doiron" nil (d oy0 r ao1 n))
("doke" nil (d ow1 k))
("dokken" nil (d aa1 k ax n))
("doktor" nil (d aa1 k t er0))
("dol" nil (d aa1 l))
("dolak" nil (d ow1 l ax k))
("dolan" nil (d ow1 l ax n))
("doland" nil (d uw1 l ax n d))
("dolata" nil (d ow0 l aa1 t ax))
("dolbow" nil (d ow1 l b ow0))
("dolby" nil (d ow1 l b iy0))
("dolce" nil (d ow1 l ch ey1))
("dolch" nil (d ow1 l ch))
("dold" nil (d ow1 l d))
("dolder" nil (d ow1 l d er0))
("doldrums" nil (d ow1 l d r ax m z))
("dole" nil (d ow1 l))
("dolecki" nil (d ax l eh1 t s k iy0))
("doled" nil (d ow1 l d))
("doleful" nil (d ow1 l f ax l))
("dolen" nil (d ow1 l ax n))
("dolence" nil (d ow1 l ax n s))
("doles" nil (d ow1 l z))
("doley" nil (d ow1 l iy0))
("dolezal" nil (d ow0 l ey0 z ae1 l))
("dolf" nil (d ow1 l f))
("dolfi" nil (d ow1 l f iy0))
("dolgen" nil (d ow1 l jh eh0 n))
("dolgin" nil (d ow1 l jh ih0 n))
("dolin" nil (d ow1 l ih0 n))
("dolinar" nil (d aa1 l ih0 n er0))
("doling" nil (d ow1 l ih0 ng))
("dolinger" nil (d ow1 l ih0 ng er0))
("dolinski" nil (d ax l ih1 n s k iy0))
("dolinsky" nil (d ax l ih1 n s k iy0))
("doll" nil (d aa1 l))
("dollar" nil (d aa1 l er0))
("dollard" nil (d aa1 l er0 d))
("dollarhide" nil (d aa1 l er0 hh ay1 d))
("dollars" nil (d aa1 l er0 z))
("dolle" nil (d aa1 l))
("dolled" nil (d aa1 l d))
("dollens" nil (d aa1 l ax n z))
("dolley" nil (d aa1 l iy0))
("dollie" nil (d aa1 l iy0))
("dollinger" nil (d aa1 l ih0 ng er0))
("dollins" nil (d aa1 l ih0 n z))
("dollison" nil (d aa1 l ih0 s ax n))
("dolliver" nil (d aa1 l ih0 v er0))
("dolloff" nil (d aa1 l ao1 f))
("dollop" nil (d aa1 l ax p))
("dolls" nil (d aa1 l z))
("dolly" nil (d aa1 l iy0))
("dolman" nil (d aa1 l m ax n))
("dolney" nil (d ow1 l n iy0))
("dolomite" nil (d ow1 l ax m ay1 t))
("dolomites" nil (d ow1 l ax m ay1 t s))
("dolores" nil (d ax l ao1 r ih0 s))
("dolorita" nil (d ow0 l ao0 r iy1 t ax))
("dolph" nil (d ow1 l f))
("dolphin" nil (d aa1 l f ax n))
("dolphins" nil (d aa1 l f ax n z))
("dols" nil (d aa1 l z))
("dolson" nil (d ow1 l s ax n))
("dolton" nil (d ow1 l t ax n))
("dom" nil (d aa1 m))
("domagala" nil (d ow0 m aa0 g aa1 l ax))
("domagalski" nil (d ax m ax g aa1 l s k iy0))
("domain" nil (d ow0 m ey1 n))
("domaine" nil (d ow0 m ey1 n))
("domains" nil (d ow0 m ey1 n z))
("doman" nil (d uw1 m ax n))
("domangue" nil (d ow1 m aa0 ng))
("domanico" nil (d ow0 m aa0 n iy1 k ow0))
("domanski" nil (d ax m ae1 n s k iy0))
("dombeck" nil (d aa1 m b eh1 k))
("dombek" nil (d aa1 m b ih0 k))
("dombkowski" nil (d ax m k ao1 f s k iy0))
("dombroski" nil (d ax m b r aw1 s k iy0))
("dombrosky" nil (d ax m b r ow1 s k iy0))
("dombrowski" nil (d ax m b r ao1 f s k iy0))
("dome" nil (d ow1 m))
("domecq" nil (d ow1 m eh1 k))
("domed" nil (d ow1 m d))
("domeier" nil (d aa1 m ay0 er0))
("domek" nil (d ow1 m eh0 k))
("domenech" nil (d aa1 m ih0 n ih0 k))
("domenici" nil (d ax m eh1 n ax ch iy0))
("domenick" nil (d aa1 m ih0 n ih0 k))
("domenico" nil (d ow0 m ey1 n iy0 k ow0))
("domeniconi" nil (d ow0 m eh1 n ih0 k ow1 n iy0))
("domer" nil (d ow1 m er0))
("domes" nil (d ow1 m z))
("domestic" nil (d ax m eh1 s t ih0 k))
("domestically" nil (d ax m eh1 s t ih0 k l iy0))
("domesticated" nil (d ax m eh1 s t ax k ey1 t ax d))
("domesticating" nil (d ax m eh1 s t ax k ey1 t ih0 ng))
("domestication" nil (d ax m eh1 s t ax k ey1 sh ax n))
("domesticity" nil (d ow1 m eh1 s t ih1 s ax t iy0))
("domestics" nil (d ax m eh1 s t ih0 k s))
("domical" nil (d aa1 m ax k ax l))
("domin" nil (d ow1 m ih0 n))
("domina" nil (d ow0 m iy1 n ax))
("dominance" nil (d aa1 m ax n ax n s))
("dominant" nil (d aa1 m ax n ax n t))
("dominate" nil (d aa1 m ax n ey1 t))
("dominated" nil (d aa1 m ax n ey1 t ax d))
("dominates" nil (d aa1 m ax n ey1 t s))
("dominating" nil (d aa1 m ax n ey1 t ih0 ng))
("domination" nil (d aa1 m ax n ey1 sh ax n))
("domine" nil (d ow0 m iy1 n iy0))
("dominee" nil (d ow1 m ih0 n ey1))
("domineering" nil (d aa1 m ax n ih1 r ih0 ng))
("dominelli" nil (d ow1 m ih0 n eh1 l iy0))
("dominey" nil (d aa1 m ih0 n iy0))
("domingo" nil (d ow0 m ih1 ng g ow0))
("domingos" nil (d ax m ih1 ng g ow0 z))
("domingue" nil (d ow1 m ih0 ng))
("domingues" nil (d ow0 m iy1 n g eh0 s))
("dominguez" nil (d ax m ih1 ng ih0 z))
("dominiak" nil (d ax m ih1 n iy0 ae0 k))
("dominic" nil (d aa1 m ax n ih0 k))
("dominica" nil (d ax m ih1 n ih0 k ax))
("dominican" nil (d ax m ih1 n ax k ax n))
("dominicana" nil (d ow0 m ih1 n ih0 k aa1 n ax))
("dominicans" nil (d ow0 m ih1 n ih0 k ax n z))
("dominici" nil (d ow0 m iy0 n iy1 ch iy0))
("dominick" nil (d aa1 m ax n ih0 k))
("dominik" nil (d ax m ih1 n ih0 k))
("dominion" nil (d ax m ih1 n y ax n))
("dominique" nil (d ao0 m ih0 n iy1 k))
("domino" nil (d aa1 m ax n ow1))
("dominoes" nil (d aa1 m ax n ow1 z))
("dominos" nil (d aa1 m ih0 n ow1 z))
("dominquez" nil (d ow0 m iy1 n k w eh0 z))
("dominski" nil (d ax m ih1 n s k iy0))
("dominus" nil (d ow0 m iy1 n ax s))
("dominy" nil (d ax m ay1 n iy0))
("domke" nil (d aa1 m k))
("dommer" nil (d aa1 m er0))
("dommie" nil (d aa1 m iy0))
("domtar" nil (d aa1 m t er0))
("domzalski" nil (d ax m z aa1 l s k iy0))
("don" nil (d aa1 n))
("dona" nil (d ow1 n ax))
("donadio" nil (d ow0 n aa1 d iy0 ow0))
("donaghey" nil (d aa1 n ax g hh iy0))
("donaghue" nil (d aa1 n ax g hh uw0))
("donaghy" nil (d aa1 n ax g hh iy0))
("donahey" nil (d aa1 n ax hh iy0))
("donaho" nil (d ow0 n aa1 hh ow0))
("donahoe" nil (d aa1 n ax hh ow1))
("donahoo" nil (d aa1 n ax hh uw1))
("donahue" nil (d aa1 n ax hh y uw1))
("donais" nil (d ax n ey1))
("donald" nil (d aa1 n ax l d))
("donalda" nil (d ow0 n aa1 l d ax))
("donaldo" nil (d ow0 n aa1 l d ow0))
("donaldson" nil (d aa1 n ax l d s ax n))
("donaldsons" nil (d aa1 n ax l d s ax n z))
("donalson" nil (d aa1 n ax l s ax n))
("donapria" nil (d ax n ae1 p r iy0 ax))
("donar" nil (d aa1 n er0))
("donat" nil (d ow1 n ax t))
("donata" nil (d ax n aa1 t ax))
("donate" nil (d ow1 n ey1 t))
("donated" nil (d ow1 n ey1 t ax d))
("donatelli" nil (d ow0 n aa0 t eh1 l iy0))
("donates" nil (d ow1 n ey1 t s))
("donath" nil (d aa1 n ax th))
("donathan" nil (d aa1 n ax th ax n))
("donati" nil (d ow0 n aa1 t iy0))
("donating" nil (d ow1 n ey1 t ih0 ng))
("donation" nil (d ow0 n ey1 sh ax n))
("donations" nil (d ow0 n ey1 sh ax n z))
("donatists" nil (d aa1 n ax t ax s t s))
("donato" nil (d ax n aa1 t ow0))
("donavan" nil (d aa1 n ax v ae1 n))
("donaway" nil (d aa1 n ax w ey1))
("donde" nil (d aa1 n d))
("dondero" nil (d ow0 n d eh1 r ow0))
("dondlinger" nil (d aa1 n d ax l ih0 ng er0))
("done" nil (d ah1 n))
("donegan" nil (d aa1 n ih0 g ae0 n))
("donehoo" nil (d ow0 n ey1 hh uw0))
("donelan" nil (d aa1 n ih0 l ae0 n))
("donella" nil (d ow0 n eh1 l ax))
("donelson" nil (d aa1 n ih0 l s ax n))
("doner" nil (d ao1 n er0))
("dones" nil (d ah1 n z))
("doney" nil (d aa1 n iy0))
("dong" nil (d ao1 ng))
("dongen" nil (d ao1 ng g ax n))
("dongmei" nil (d ow1 ng m ay1))
("donham" nil (d aa1 n hh ax m))
("donia" nil (d ow1 n iy0 ax))
("donica" nil (d aa1 n ih0 k ax))
("donigan" nil (d aa1 n ih0 g ax n))
("doniger" nil (d aa1 n ih0 g er0))
("donilon" nil (d aa1 n ax l aa0 n))
("donis" nil (d ow1 n ih0 s))
("donizetti" nil (d aa1 n ax z eh1 t iy0))
("donkey" nil (d aa1 ng k iy0))
("donkeys" nil (d aa1 ng k iy0 z))
("donlan" nil (d aa1 n l ax n))
("donley" nil (d aa1 n l iy0))
("donlin" nil (d aa1 n l ih0 n))
("donlon" nil (d aa1 n l ax n))
("donmoyer" nil (d aa1 n m oy1 er0))
("donn" nil (d aa1 n))
("donna" nil (d aa1 n ax))
("donnan" nil (d aa1 n ax n))
("donnas" nil (d aa1 n ax z))
("donnay" nil (d aa1 n ey0))
("donne" nil (d ah1 n))
("donned" nil (d aa1 n d))
("donnell" nil (d aa1 n ih0 l))
("donnellan" nil (d aa1 n ih0 l ae0 n))
("donnelley" nil (d aa1 n ax l iy0))
("donnelly" nil (d aa1 n ax l iy0))
("donner" nil (d aa1 n er0))
("donnie" nil (d aa1 n iy0))
("donning" nil (d aa1 n ih0 ng))
("donny" nil (d aa1 n iy0))
("donnybrook" nil (d aa1 n iy0 b r uh1 k))
("donofrio" nil (d ow0 n ow1 f r iy0 ow0))
("donoghue" nil (d aa1 n ax hh y uw0))
("donoho" nil (d aa1 n ax hh ow0))
("donohoe" nil (d aa1 n ax hh ow0))
("donohoo" nil (d aa1 n ax hh uw1))
("donohue" nil (d aa1 n ax hh y uw1))
("donor" nil (d ow1 n er0))
("donors" nil (d ow1 n er0 z))
("donovan" nil (d aa1 n ax v ax n))
("dons" nil (d aa1 n z))
("donsbach" nil (d aa1 n z b aa1 k))
("donut" nil (d ow1 n ah1 t))
("donuts" nil (d ow1 n ah1 t s))
("donvan" nil (d aa1 n v ax n))
("donze" nil (d aa1 n z))
("doo" nil (d uw1))
("doodads" nil (d uw1 d ae1 d z))
("doodle" nil (d uw1 d ax l))
("doodles" nil (d uw1 d ax l z))
("doody" nil (d uw1 d iy0))
("doolan" nil (d uw1 l ax n))
("doolen" nil (jh uw1 l ax n))
("dooley" nil (d uw1 l iy0))
("doolin" nil (d uw1 l ih0 n))
("dooling" nil (d uw1 l ih0 ng))
("doolittle" nil (d uw1 l ih1 t ax l))
("doom" nil (d uw1 m))
("doomed" nil (d uw1 m d))
("dooming" nil (d uw1 m ih0 ng))
("dooms" nil (d uw1 m z))
("doomsayers" nil (d uw1 m s ey1 er0 z))
("doomsday" nil (d uw1 m z d ey1))
("doonan" nil (d uw1 n ax n))
("dooner" nil (d uw1 n er0))
("doonesbury" nil (d uw1 n z b eh1 r iy0))
("door" nil (d ao1 r))
("doordarshan" nil (d uw1 r d aa1 r sh ax n))
("doorenbos" nil (d uh1 r eh0 n b ow1 s))
("doorkeeper" nil (d ao1 r k iy1 p er0))
("doorknob" nil (d uw1 r k n aa0 b))
("doorman" nil (d ao1 r m ae1 n))
("doormat" nil (d ao1 r m ae1 t))
("doormats" nil (d ao1 r m ae1 t s))
("doorn" nil (d ao1 r n))
("doornail" nil (d ao1 r n ey1 l))
("doornbos" nil (d ao1 r n b ow0 z))
("doors" nil (d ao1 r z))
("doorstep" nil (d ao1 r s t eh1 p))
("doorsteps" nil (d ao1 r s t eh1 p s))
("doorway" nil (d ao1 r w ey1))
("doorways" nil (d ao1 r w ey1 z))
("doose" nil (d uw1 s))
("doozy" nil (d uw1 z iy0))
("dopa" nil (d ow1 p ax))
("dopamine" nil (d aa1 p ax m ay1 n))
("dope" nil (d ow1 p))
("doped" nil (d ow1 p t))
("dopey" nil (d ow1 p iy0))
("dopp" nil (d aa1 p))
("doppler" nil (d aa1 p l er0))
("dopson" nil (d aa1 p s ax n))
("dora" nil (d ao1 r ax))
("dorado" nil (d ao0 r aa1 d ow0))
("dorais" nil (d er0 ey1))
("doral" nil (d ao0 r ae1 l))
("doralin" nil (d ao0 r aa0 l iy1 n))
("doralynne" nil (d ao1 r ax l ay0 n))
("doran" nil (d ao0 r ae1 n))
("doraville" nil (d ao1 r ax v ih0 l))
("dorazio" nil (d ao0 r aa1 z iy0 ow0))
("dorch" nil (d ao1 r k))
("dorchester" nil (d ao1 r ch eh1 s t er0))
("dordies" nil (d ao1 r d iy0 z))
("dore" nil (d ao1 r))
("dorea" nil (d ao1 r iy0 ax))
("doreen" nil (d ao0 r iy1 n))
("dorelia" nil (d ao0 r eh1 l iy0 ax))
("doremus" nil (d ao1 r ih0 m ih0 s))
("doren" nil (d ao1 r ax n))
("dorena" nil (d ao1 r ih0 n ax))
("dorene" nil (d ao1 r iy0 n))
("dorer" nil (d ao1 r er0))
("dorette" nil (d er0 eh1 t))
("doretti" nil (d ao0 r eh1 t iy0))
("dorey" nil (d ao1 r iy0))
("dorf" nil (d ao1 r f))
("dorff" nil (d ao1 r f))
("dorfman" nil (d ao1 r f m ax n))
("dorgan" nil (d ao1 r g ax n))
("dori" nil (d ao1 r iy0))
("doria" nil (d ao1 r iy0 ax))
("dorian" nil (d ao1 r iy0 ax n))
("dorians" nil (d ao1 r iy0 ax n z))
("doric" nil (d ao1 r ih0 k))
("dorice" nil (d ao1 r ih0 s))
("dorin" nil (d ao1 r ih0 n))
("dorinda" nil (d ao0 r iy1 n d ax))
("dorine" nil (d ao0 r iy1 n iy0))
("doring" nil (d ao1 r ih0 ng))
("dorio" nil (d ao1 r iy0 ow0))
("dorion" nil (d ao0 r iy0 ao1 n))
("doris" nil (d ao1 r ax s))
("dorise" nil (d ao1 r ay0 z))
("doritos" nil (d ao1 r iy1 t ow0 z))
("dority" nil (d ao1 r ih0 t iy0))
("dorko" nil (d ao1 r k ow0))
("dorland" nil (d ao1 r l ax n d))
("dorm" nil (d ao1 r m))
("dorman" nil (d ao1 r m ax n))
("dormancy" nil (d ao1 r m ax n s iy0))
("dormant" nil (d ao1 r m ax n t))
("dormer" nil (d ao1 r m er0))
("dorminey" nil (d ao1 r m ih0 n iy0))
("dormitories" nil (d ao1 r m ax t ao1 r iy0 z))
("dormitory" nil (d ao1 r m ax t ao1 r iy0))
("dormouse" nil (d ao1 r m aw1 s))
("dorms" nil (d ao1 r m z))
("dorn" nil (d ao1 r n))
("dornak" nil (d ao1 r n ax k))
("dornan" nil (d ao1 r n ax n))
("dornbusch" nil (d ao1 r n b ax sh))
("dornbush" nil (d ao1 r n b ax sh))
("dorner" nil (d ao1 r n er0))
("dorney" nil (d ao1 r n iy0))
("dornfeld" nil (d ao1 r n f eh0 l d))
("dornier" nil (d ao1 r n ih1 r))
("dornin" nil (d ao1 n ih0 n))
("doro" nil (d ao1 r ow0))
("doron" nil (d ao1 r ax n))
("dorotea" nil (d ao1 r ax sh ax))
("dorothea" nil (d ao1 r ax th iy1 ax))
("dorothy" nil (d ao1 r ax th iy0))
("dorough" nil (d ao1 r aw0))
("dorow" nil (d ao1 r ow0))
("dorr" nil (d ao1 r))
("dorrance" nil (d ao1 r ax n s))
("dorrell" nil (d ao0 r ey1 l))
("dorrie" nil (d ao1 r iy0))
("dorries" nil (d ao1 r iy0 z))
("dorris" nil (d ao1 r ih0 s))
("dorroh" nil (d ao1 r ow0))
("dorrough" nil (d ao1 r aw0))
("dorsa" nil (d ao1 r s ax))
("dorsal" nil (d ao1 r s ax l))
("dorsally" nil (d ao1 r s ax l iy0))
("dorsch" nil (d ao1 r sh))
("dorset" nil (d ao1 r s ih0 t))
("dorsett" nil (d ao1 r s ih0 t))
("dorsey" nil (d ao1 r s iy0))
("dorsi" nil (d ao1 r s iy0))
("dorst" nil (d ao1 r s t))
("dort" nil (d ao1 r t))
("dorta" nil (d ao1 r t ax))
("dortch" nil (d ao1 r ch))
("dorthea" nil (d ao1 r dh iy0 ax))
("dorthy" nil (d ao1 r th iy0))
("dorton" nil (d ao1 r t ax n))
("dorval" nil (d ao1 r v ax l))
("dorward" nil (d ao1 r w er0 d))
("dorwart" nil (d ao1 r w ao0 r t))
("dory" nil (d ao1 r iy0))
("dos" nil (d ao1 s))
("dosage" nil (d ow1 s ax jh))
("dosages" nil (d ow1 s ih0 jh ih0 z))
("dosch" nil (d ao1 sh))
("doscher" nil (d ao1 sh er0))
("dose" nil (d ow1 s))
("doser" nil (d ow1 s er0))
("doses" nil (d ow1 s ax z))
("dosh" nil (d aa1 sh))
("dosher" nil (d aa1 sh er0))
("doshi" nil (d ow1 sh iy0))
("doshier" nil (d aa1 sh iy0 er0))
("dosia" nil (d ow1 s iy0 ax))
("dosie" nil (d aa1 s iy0))
("dosier" nil (d ow1 s iy0 er0))
("dosimeters" nil (d ow0 s ih1 m ax t er0 z))
("doskocil" nil (d ao1 s k ax s ih0 l))
("doss" nil (d ao1 s))
("dossantos" nil (d ow0 s aa1 n t ow0 z))
("dossett" nil (d aa1 s ih0 t))
("dossey" nil (d aa1 s iy0))
("dossier" nil (d ao1 s y ey1))
("dossiers" nil (d ao1 s y ey1 z))
("dost" nil (d aa1 s t))
("dostal" nil (d aa1 s t ax l))
("doster" nil (d aa1 s t er0))
("dostie" nil (d aa1 s t iy0))
("dostoevski" nil (d ao1 s t ow1 v s k iy0))
("dostoevsky" nil (d ao1 s t ow1 v s k iy0))
("doswell" nil (d aa1 s w eh0 l))
("dot" nil (d aa1 t))
("dotan" nil (d ow1 t ax n))
("doten" nil (d ow1 t ax n))
("dothan" nil (d aa1 th ax n))
("doting" nil (d ow1 t ih0 ng))
("doto" nil (d ow1 t ow0))
("dots" nil (d aa1 t s))
("dotson" nil (d aa1 t s ax n))
("dott" nil (d aa1 t))
("dottavio" nil (d ow0 t aa1 v iy0 ow0))
("dotted" nil (d aa1 t ax d))
("dotter" nil (d aa1 t er0))
("dotterer" nil (d aa1 t er0 er0))
("dottie" nil (d aa1 t iy0))
("dotting" nil (d aa1 t ih0 ng))
("dotts" nil (d aa1 t s))
("dotty" nil (d aa1 t iy0))
("doty" nil (d ow1 t iy0))
("dotzler" nil (d aa1 t s l er0))
("doub" nil (d aw1 b))
("doubek" nil (d aw1 b ih0 k))
("doubet" nil (d aw1 b ih0 t))
("double" nil (d ah1 b ax l))
("doubled" nil (d ah1 b ax l d))
("doubleday" nil (d ah1 b ax l d ey1))
("doubles" nil (d ah1 b ax l z))
("doublespeak" nil (d ah1 b ax l s p iy1 k))
("doublethink" nil (d ah1 b ax l th ih1 ng k))
("doubletree" nil (d ah1 b ax l t r iy1))
("doubling" nil (d ah1 b ax l ih0 ng))
("doubly" nil (d ah1 b l iy0))
("doubrava" nil (d aw1 b r ax v ax))
("doubt" nil (d aw1 t))
("doubted" nil (d aw1 t ih0 d))
("doubters" nil (d aw1 t er0 z))
("doubtfire" nil (d aw1 t f ay1 er0))
("doubtfires" nil (d aw1 t f ay1 er0 z))
("doubtful" nil (d aw1 t f ax l))
("doubting" nil (d aw1 t ih0 ng))
("doubtless" nil (d aw1 t l ax s))
("doubts" nil (d aw1 t s))
("doucet" nil (d uw0 s eh1 t))
("doucette" nil (d uw1 s eh1 t))
("doud" nil (d aw1 d))
("doudna" nil (d aw1 d n ax))
("douds" nil (d aw1 d z))
("doug" nil (d ah1 g))
("dougal" nil (d uw1 g ax l))
("dougall" nil (d aw1 g ax l))
("dougan" nil (d aw1 g ax n))
("dough" nil (d ow1))
("doughboy" nil (d ow1 b oy1))
("dougher" nil (d ow1 er0))
("dougherty" nil (d ow1 er0 t iy0))
("doughman" nil (d ow1 m ax n))
("doughnut" nil (d ow1 n ah1 t))
("doughnuts" nil (d ow1 n ah1 t s))
("doughten" nil (d ao1 t ax n))
("doughtie" nil (d ao1 t iy0))
("doughty" nil (d ao1 t iy0))
("doughy" nil (d ow1 iy0))
("douglas" nil (d ah1 g l ax s))
("doukas" nil (d aw1 k ax z))
("doulton" nil (d ow1 l t ax n))
("douma" nil (d ow1 m ax))
("doupe" nil (d uw1 p))
("dour" nil (d aw1 er0))
("douse" nil (d aw1 s))
("doused" nil (d aw1 s t))
("dousing" nil (d aw1 s ih0 ng))
("douthat" nil (d aw1 th ax t))
("douthett" nil (d aw1 th ih0 t))
("douthit" nil (d uw0 th ih1 t))
("douthitt" nil (d uw0 th ih1 t))
("doutt" nil (d aw1 t))
("douty" nil (d aw1 t iy0))
("douville" nil (d uw0 v ih1 l))
("douwe" nil (d uw1))
("dov" nil (d aa1 v))
("dove" nil (d ah1 v))
("dovel" nil (d ow0 v eh1 l))
("dover" nil (d ow1 v er0))
("doverspike" nil (d ax v er1 s p ih0 k))
("doves" nil (d ah1 v z))
("dovetail" nil (d ah1 v t ey1 l))
("dovetailed" nil (d ah1 v t ey1 l d))
("dovetails" nil (d ah1 v t ey1 l z))
("dovey" nil (d ax v ey1))
("dovidio" nil (d ow0 v iy1 d iy0 ow0))
("dovish" nil (d ow1 v ih0 sh))
("dow" nil (d aw1))
("dowager" nil (d aw1 ax jh er0))
("dowd" nil (d aw1 d))
("dowda" nil (d aw1 d ax))
("dowdall" nil (d aw1 d ax l))
("dowdell" nil (d aw1 d ax l))
("dowden" nil (d aw1 d ax n))
("dowding" nil (d aw1 d ih0 ng))
("dowdle" nil (d aw1 d ax l))
("dowds" nil (d aw1 d z))
("dowdy" nil (d aw1 d iy0))
("dowe" nil (d aw1))
("dowel" nil (d aw1 ax l))
("dowell" nil (d aa1 w eh0 l))
("dowels" nil (d aw1 ax l z))
("dowen" nil (d aw1 ax n))
("dower" nil (d aw1 r))
("dowers" nil (d aw1 r z))
("dowie" nil (d aw1 iy0))
("dowis" nil (d aw1 ih0 s))
("dowland" nil (d aw1 l ax n d))
("dowlen" nil (d aw1 l ax n))
("dowler" nil (d aw1 l er0))
("dowless" nil (d aw1 l ax s))
("dowling" nil (d aw1 l ih0 ng))
("down" nil (d aw1 n))
("downard" nil (d aw1 n er0 d))
("downbeat" nil (d aw0 n b iy1 t))
("downcast" nil (d aw1 n k ae1 s t))
("downdraft" nil (d aw1 n d r ae1 f t))
("downe" nil (d aw1 n))
("downed" nil (d aw1 n d))
("downen" nil (d aw1 n ax n))
("downer" nil (d aw1 n er0))
("downers" nil (d aw1 n er0 z))
("downes" nil (d aw1 n z))
("downey" nil (d aw1 n iy0))
("downfall" nil (d aw1 n f ao1 l))
("downgrade" nil (d aw1 n g r ey1 d))
("downgraded" nil (d aw1 n g r ey1 d ax d))
("downgrades" nil (d aw1 n g r ey1 d z))
("downgrading" nil (d aw1 n g r ey1 d ih0 ng))
("downgradings" nil (d aw1 n g r ey1 d ih0 ng z))
("downham" nil (d aw1 n hh ax m))
("downhill" nil (d aw1 n hh ih1 l))
("downie" nil (d aw1 n iy0))
("downing" nil (d aw1 n ih0 ng))
("downingtown" nil (d aw1 n ih0 ng t aw1 n))
("download" nil (d aw1 n l ow1 d))
("downloaded" nil (d aw1 n l ow1 d ih0 d))
("downloading" nil (d aw1 n l ow1 d ih0 ng))
("downpayment" nil (d aw1 n p ey1 m ax n t))
("downplay" nil (d aw1 n p l ey1))
("downplayed" nil (d aw1 n p l ey1 d))
("downplaying" nil (d aw1 n p l ey1 ih0 ng))
("downplays" nil (d aw1 n p l ey1 z))
("downpour" nil (d aw1 n p ao0 r))
("downpours" nil (d aw1 n p ao1 r z))
("downrange" nil (d aw1 n r ey1 n jh))
("downright" nil (d aw1 n r ay1 t))
("downriver" nil (d aw1 n r ih1 v er0))
("downs" nil (d aw1 n z))
("downside" nil (d aw1 n s ay1 d))
("downsize" nil (d aw1 n s ay1 z))
("downsized" nil (d aw1 n s ay1 z d))
("downsizing" nil (d aw1 n s ay1 z ih0 ng))
("downstage" nil (d aw1 n s t ey1 jh))
("downstairs" nil (d aw1 n s t eh1 r z))
("downstate" nil (d aw1 n s t ey1 t))
("downstream" nil (d aw1 n s t r iy1 m))
("downswing" nil (d aw1 n s w ih1 ng))
("downtime" nil (d aw1 n t ay1 m))
("downtown" nil (d aw1 n t aw1 n))
("downtowns" nil (d aw1 n t aw1 n z))
("downtrend" nil (d aw1 n t r eh1 n d))
("downtrodden" nil (d aw1 n t r aa1 d ax n))
("downturn" nil (d aw1 n t er1 n))
("downturns" nil (d aw1 n t er1 n z))
("downum" nil (d aw1 n ax m))
("downward" nil (d aw1 n w er0 d))
("downwardly" nil (d aw1 n w er0 d l iy0))
("downwards" nil (d aw1 n w er0 d z))
("downy" nil (d aw1 n iy0))
("dowry" nil (d aw1 r iy0))
("dowse" nil (d aw1 s))
("dowty" nil (d aw1 t iy0))
("doxey" nil (d aa1 k s iy0))
("doxie" nil (d aa1 k s iy0))
("doxologies" nil (d aa0 k s aa1 l ax jh iy0 z))
("doxology" nil (d aa0 k s aa1 l ax jh iy0))
("doxtater" nil (d aa1 k s t ey0 t er0))
("doxy" nil (d aa1 k s iy0))
("doyal" nil (d oy0 aa1 l))
("doyel" nil (d oy1 ax l))
("doyen" nil (d oy1 ih0 n))
("doyle" nil (d oy1 l))
("doyon" nil (d oy1 ax n))
("dozed" nil (d ow1 z d))
("dozen" nil (d ah1 z ax n))
("dozens" nil (d ah1 z ax n z))
("dozier" nil (d ow1 z iy0 er0))
("dozing" nil (d ow1 z ih0 ng))
("drab" nil (d r ae1 b))
("drabble" nil (d r ae1 b ax l))
("drabek" nil (d r ae1 b ih0 k))
("drabenstott" nil (d r ax b eh1 n s t ax t))
("drabik" nil (d r aa1 b ih0 k))
("drabinsky" nil (d r ax b ih1 n s k iy0))
("drabkin" nil (d r ae1 b k ih1 n))
("drabs" nil (d r ae1 b z))
("drace" nil (d r ey1 s))
("drach" nil (d r ae1 ch))
("drachenberg" nil (d r ae1 k ax n b er0 g))
("drachma" nil (d r ae1 k m ax))
("drachmas" nil (d r aa1 k m ax z))
("drackett" nil (d r ae1 k ih0 t))
("draconian" nil (d r ey0 k ow1 n iy0 ax n))
("dracula" nil (d r ae1 k y uw0 l ax))
("draeger" nil (d r eh1 g er0))
("draffen" nil (d r ae1 f ax n))
("draft" nil (d r ae1 f t))
("drafted" nil (d r ae1 f t ih0 d))
("draftees" nil (d r ae1 f t iy1 z))
("drafter" nil (d r ae1 f t er0))
("drafters" nil (d r ae1 f t er0 z))
("drafting" nil (d r ae1 f t ih0 ng))
("drafts" nil (d r ae1 f t s))
("draftsman" nil (d r ae1 f t s m ax n))
("draftsmanship" nil (d r ae1 f t s m ax n sh ih1 p))
("draftsmen" nil (d r ae1 f t s m ax n))
("drafty" nil (d r ae1 f t iy0))
("drag" nil (d r ae1 g))
("dragan" nil (d r aa1 g ax n))
("drage" nil (d r ey1 jh))
("drager" nil (d r ey1 g er0))
("dragged" nil (d r ae1 g d))
("dragging" nil (d r ae1 g ih0 ng))
("draggy" nil (d r ae1 g iy0))
("dragline" nil (d r ae1 g l ay1 n))
("dragnet" nil (d r ae1 g n eh1 t))
("dragnets" nil (d r ae1 g n eh1 t s))
("drago" nil (d r aa1 g ow0))
("dragon" nil (d r ae1 g ax n))
("dragonair" nil (d r ae1 g ax n eh1 r))
("dragone" nil (d r ax g ow1 n))
("dragonhead" nil (d r ae1 g ax n hh eh1 d))
("dragons" nil (d r ae1 g ax n z))
("dragoo" nil (d r aa1 g uw0))
("dragovich" nil (d r ae1 g ax v ih0 ch))
("drags" nil (d r ae1 g z))
("draheim" nil (d r ae1 hh ay0 m))
("drahos" nil (d r ey1 hh ow0 z))
("drahuschak" nil (d r ae1 hh ax sh ae0 k))
("drain" nil (d r ey1 n))
("drainage" nil (d r ey1 n ax jh))
("draine" nil (d r ey1 n))
("drained" nil (d r ey1 n d))
("drainers" nil (d r ey1 n er0 z))
("draining" nil (d r ey1 n ih0 ng))
("drains" nil (d r ey1 n z))
("drake" nil (d r ey1 k))
("drakeford" nil (d r ae1 k ih0 f ao0 r d))
("drakes" nil (d r ey1 k s))
("drakos" nil (d r ey1 k ow0 z))
("dralle" nil (d r ey1 l))
("dram" nil (d r ae1 m))
("drama" nil (d r aa1 m ax))
("dramamine" nil (d r ae1 m ax m iy1 n))
("dramas" nil (d r aa1 m ax z))
("dramatic" nil (d r ax m ae1 t ih0 k))
("dramatically" nil (d r ax m ae1 t ih0 k l iy0))
("dramatist" nil (d r aa1 m ax t ih0 s t))
("dramatization" nil (d r ae1 m ax t ax z ey1 sh ax n))
("dramatizations" nil (d r ae1 m ax t ax z ey1 sh ax n z))
("dramatize" nil (d r aa1 m ax t ay1 z))
("dramatized" nil (d r ae1 m ax t ay1 z d))
("dramatizes" nil (d r ae1 m ax t ay1 z ih0 z))
("dramatizing" nil (d r ae1 m ax t ay1 z ih0 ng))
("drams" nil (d r ae1 m z))
("drane" nil (d r ey1 n))
("draney" nil (d r ey1 n iy0))
("drank" nil (d r ae1 ng k))
("drape" nil (d r ey1 p))
("drapeau" nil (d r ax p ow1))
("draped" nil (d r ey1 p t))
("draper" nil (d r ey1 p er0))
("draperies" nil (d r ey1 p er0 iy0 z))
("drapery" nil (d r ey1 p er0 iy0))
("drapes" nil (d r ey1 p s))
("draping" nil (d r ey1 p ih0 ng))
("drapkin" nil (d r ae1 p k ih0 n))
("drasner" nil (d r ae1 s n er0))
("drastic" nil (d r ae1 s t ih0 k))
("drastically" nil (d r ae1 s t ih0 k l iy0))
("draughn" nil (d r ao1 n))
("draughon" nil (d r ao1 ax n))
("draught" nil (d r ae1 f t))
("draughts" nil (d r ae1 f t s))
("draus" nil (d r ao1 z))
("draves" nil (d r ey1 v z))
("dravis" nil (d r ae1 v ih0 s))
("dravo" nil (d r ae1 v ow0))
("draw" nil (d r ao1))
("drawback" nil (d r ao1 b ae1 k))
("drawbacks" nil (d r ao1 b ae1 k s))
("drawbaugh" nil (d r ao1 b ao1))
("drawbridge" nil (d r ao1 b r ih1 jh))
("drawdown" nil (d r ao1 d aw1 n))
("drawdowns" nil (d r ao1 d aw1 n z))
("drawdy" nil (d r ao1 d iy0))
("drawer" nil (d r ao1 r))
("drawers" nil (d r ao1 r z))
("drawing" nil (d r ao1 ih0 ng))
("drawings" nil (d r ao1 ih0 ng z))
("drawl" nil (d r ao1 l))
("drawled" nil (d r ao1 l d))
("drawls" nil (d r ao1 l z))
("drawn" nil (d r ao1 n))
("draws" nil (d r ao1 z))
("dray" nil (d r ey1))
("drayer" nil (d r ey1 er0))
("drayton" nil (d r ey1 t ax n))
("dread" nil (d r eh1 d))
("dreaded" nil (d r eh1 d ih0 d))
("dreadful" nil (d r eh1 d f ax l))
("dreadfully" nil (d r eh1 d f ax l iy0))
("dreading" nil (d r eh1 d ih0 ng))
("dreadnought" nil (d r eh1 d n ao1 t))
("dream" nil (d r iy1 m))
("dreamed" nil (d r iy1 m d))
("dreamer" nil (d r iy1 m er0))
("dreamers" nil (d r iy1 m er0 z))
("dreaming" nil (d r iy1 m ih0 ng))
("dreamlike" nil (d r iy1 m l ay1 k))
("dreams" nil (d r iy1 m z))
("dreamt" nil (d r eh1 m t))
("dreamworks" nil (d r iy1 m w er1 k s))
("dreamworld" nil (d r iy1 m w er1 l d))
("dreamy" nil (d r iy1 m iy0))
("dreariness" nil (d r iy1 r iy0 n ax s))
("dreary" nil (d r ih1 r iy0))
("drebsky" nil (d r eh1 b s k iy0))
("drechsel" nil (d r eh1 k s ax l))
("drechsler" nil (d r eh1 k s ax l er0))
("dred" nil (d r eh1 d))
("dredge" nil (d r eh1 jh))
("dredged" nil (d r eh1 jh d))
("dredges" nil (d r eh1 jh ax z))
("dredging" nil (d r eh1 jh ih0 ng))
("drees" nil (d r iy1 z))
("dreese" nil (d r iy1 z))
("dreessen" nil (d r iy1 s ax n))
("dreger" nil (d r eh1 g er0))
("dregs" nil (d r eh1 g z))
("dreher" nil (d r eh1 r))
("dreibelbis" nil (d r ay1 b ih0 l b ih0 s))
("dreier" nil (d r ay1 er0))
("dreiling" nil (d r ay1 l ih0 ng))
("dreis" nil (d r iy1 z))
("dreisbach" nil (d r ay1 s b aa1 k))
("dreman" nil (d r iy1 m ax n))
("drench" nil (d r eh1 n ch))
("drenched" nil (d r eh1 n ch t))
("drenching" nil (d r eh1 n ch ih0 ng))
("drennan" nil (d r eh1 n ax n))
("drennen" nil (d r eh1 n ax n))
("drenning" nil (d r eh1 n ih0 ng))
("drennon" nil (d r eh1 n ax n))
("drenth" nil (d r eh1 n th))
("drepung" nil (d r ey1 p ah1 ng))
("dresbach" nil (d r eh1 s b aa1 k))
("dresch" nil (d r eh1 sh))
("drescher" nil (d r eh1 sh er0))
("dresden" nil (d r eh1 z d ih0 n))
("dresdner" nil (d r eh1 z n er0))
("dresen" nil (d r iy1 z ax n))
("dresher" nil (d r eh1 sh er0))
("dresner" nil (d r eh1 z n er0))
("dress" nil (d r eh1 s))
("dressage" nil (d r eh0 s aa1 zh))
("dressed" nil (d r eh1 s t))
("dressel" nil (d r eh1 s ax l))
("dressen" nil (d r eh1 s ax n))
("dresser" nil (d r eh1 s er0))
("dressers" nil (d r eh1 s er0 z))
("dresses" nil (d r eh1 s ax z))
("dressier" nil (d r eh1 s iy0 er0))
("dressing" nil (d r eh1 s ih0 ng))
("dressings" nil (d r eh1 s ih0 ng z))
("dressler" nil (d r eh1 s l er0))
("dressmaker" nil (d r eh1 s m ey1 k er0))
("dressmaking" nil (d r eh1 s m ey1 k ih0 ng))
("dressy" nil (d r eh1 s iy0))
("drew" nil (d r uw1))
("drewery" nil (d r uw1 er0 iy0))
("drewes" nil (d r uw1 z))
("drewett" nil (d r uw1 ih0 t))
("drewry" nil (d r uw1 r iy0))
("drews" nil (d r uw1 z))
("drexel" nil (d r eh1 k s ax l))
("drexler" nil (d r eh1 k s l er0))
("drey" nil (d r ey1))
("dreyer" nil (d r ey1 er0))
("dreyfus" nil (d r ay1 f ax s))
("dreyfuss" nil (d r ey1 f ax s))
("dribble" nil (d r ih1 b ax l))
("dribbled" nil (d r ih1 b ax l d))
("dribbling" nil (d r ih1 b ax l ih0 ng))
("dried" nil (d r ay1 d))
("driehaus" nil (d r iy1 hh aw1 s))
("drier" nil (d r ay1 er0))
("dries" nil (d r ay1 z))
("driessen" nil (d r iy1 s ax n))
("driest" nil (d r ay1 ax s t))
("driever" nil (d r iy1 v er0))
("drift" nil (d r ih1 f t))
("drifted" nil (d r ih1 f t ax d))
("drifter" nil (d r ih1 f t er0))
("drifting" nil (d r ih1 f t ih0 ng))
("driftnet" nil (d r ih1 f t n eh1 t))
("drifts" nil (d r ih1 f t s))
("driftwood" nil (d r ih1 f t w uh1 d))
("driggers" nil (d r ih1 g er0 z))
("driggs" nil (d r ih1 g z))
("drill" nil (d r ih1 l))
("drillbit" nil (d r ih1 l b ih1 t))
("drilled" nil (d r ih1 l d))
("driller" nil (d r ih1 l er0))
("drillers" nil (d r ih1 l er0 z))
("drilling" nil (d r ih1 l ih0 ng))
("drills" nil (d r ih1 l z))
("drina" nil (d iy1 n ax))
("drinas" nil (d iy1 n ax z))
("dring" nil (d r ih1 ng))
("drink" nil (d r ih1 ng k))
("drinkard" nil (d r ih1 ng k er0 d))
("drinker" nil (d r ih1 ng k er0))
("drinkers" nil (d r ih1 ng k er0 z))
("drinking" nil (d r ih1 ng k ih0 ng))
("drinks" nil (d r ih1 ng k s))
("drinkwater" nil (d r ih1 ng k w ao1 t er0))
("drinkwine" nil (d r ih1 ng k w ay1 n))
("drinnon" nil (d r ih1 n ax n))
("drip" nil (d r ih1 p))
("dripped" nil (d r ih1 p t))
("dripping" nil (d r ih1 p ih0 ng))
("dripps" nil (d r ih1 p s))
("drips" nil (d r ih1 p s))
("driscoll" nil (d r ih1 s k ax l))
("driskell" nil (d r ih1 s k ax l))
("driskill" nil (d r ih1 s k ih0 l))
("drivable" nil (d r ay1 v ax b ax l))
("drive" nil (d r ay1 v))
("drivel" nil (d r ih1 v ax l))
("driven" nil (d r ih1 v ax n))
("driver" nil (d r ay1 v er0))
("drivers" nil (d r ay1 v er0 z))
("drives" nil (d r ay1 v z))
("driveway" nil (d r ay1 v w ey1))
("driveways" nil (d r ay1 v w ey1 z))
("driving" nil (d r ay1 v ih0 ng))
("drizzle" nil (d r ih1 z ax l))
("drizzling" nil (d r ih1 z ax l ih0 ng))
("drizzly" nil (d r ih1 z l iy0))
("drobny" nil (d r aa1 b n iy0))
("droddy" nil (d r aa1 d iy0))
("droege" nil (d r ow1 jh))
("droessler" nil (d r ow1 s ax l er0))
("droge" nil (d r ow1 jh))
("drogoul" nil (d r ow0 g uw1 l))
("drogue" nil (d r ow1 g))
("drohan" nil (d r ow1 ax n))
("droke" nil (d r ow1 k))
("drolet" nil (d r ow1 l ih0 t))
("droll" nil (d r ow1 l))
("drollinger" nil (d r ow1 l ih0 ng er0))
("dromedary" nil (d r aa1 m ax d eh1 r iy0))
("dromer" nil (d r ow1 m er0))
("dromey" nil (d r ow1 m iy0))
("dromgoole" nil (d r aa1 m g uw1 l))
("dromgooles" nil (d r aa0 m g uw1 l z))
("dromon" nil (d r ow1 m ax n))
("drone" nil (d r ow1 n))
("droned" nil (d r ow1 n d))
("drones" nil (d r ow1 n z))
("dronet" nil (d r ow1 n ih0 t))
("droney" nil (d r ow1 n iy0))
("droning" nil (d r ow1 n ih0 ng))
("drool" nil (d r uw1 l))
("drooling" nil (d r uw1 l ih0 ng))
("droop" nil (d r uw1 p))
("drooped" nil (d r uw1 p t))
("drooping" nil (d r uw1 p ih0 ng))
("droopy" nil (d r uw1 p iy0))
("drop" nil (d r aa1 p))
("dropkin" nil (d r aa1 p k ih0 n))
("droplet" nil (d r aa1 p l ax t))
("droplets" nil (d r aa1 p l ax t s))
("dropoff" nil (d r aa1 p ao1 f))
("dropout" nil (d r aa1 p aw1 t))
("dropouts" nil (d r aa1 p aw1 t s))
("dropped" nil (d r aa1 p t))
("droppers" nil (d r aa1 p er0 z))
("dropping" nil (d r aa1 p ih0 ng))
("droppings" nil (d r aa1 p ih0 ng z))
("drops" nil (d r aa1 p s))
("dropsy" nil (d r aa1 p s iy0))
("droser" nil (d r ow1 z er0))
("dross" nil (d r ao1 s))
("drost" nil (d r aa1 s t))
("droste" nil (d r ow1 s t))
("drotar" nil (d r ow1 t er0))
("drought" nil (d r aw1 t))
("droughts" nil (d r aw1 t s))
("drouillard" nil (d r w iy0 l aa1 r d))
("drouin" nil (d r w iy1 n))
("drove" nil (d r ow1 v))
("drover" nil (d r ow1 v er0))
("droves" nil (d r ow1 v z))
("drown" nil (d r aw1 n))
("drowned" nil (d r aw1 n d))
("drowning" nil (d r aw1 n ih0 ng))
("drownings" nil (d r aw1 n ih0 ng z))
("drowns" nil (d r aw1 n z))
("drowsiness" nil (d r aw1 z iy0 n ax s))
("drowsy" nil (d r aw1 z iy0))
("droz" nil (d r aa1 z))
("drozd" nil (d r aa1 z d))
("drozda" nil (d r aa1 z d ax))
("drozdowski" nil (d r ax z d ao1 f s k iy0))
("dru" nil (d r uw1))
("drubbed" nil (d r ah1 b d))
("drubbing" nil (d r ah1 b ih0 ng))
("druce" nil (d r uw1 s))
("drucie" nil (d r ah1 k iy0))
("druck" nil (d r ah1 k))
("druckenmiller" nil (d r ah1 k ih0 n m ih1 l er0))
("drucker" nil (d r ah1 k er0))
("druckman" nil (d r ah1 k m ax n))
("drudge" nil (d r ah1 jh))
("drudgery" nil (d r ah1 jh er0 iy0))
("druella" nil (d r uw1 eh1 l ax))
("drug" nil (d r ah1 g))
("drugan" nil (d r uw1 g ax n))
("drugged" nil (d r ah1 g d))
("druggist" nil (d r ah1 g ih0 s t))
("druggists" nil (d r ah1 g ax s t s))
("drugmaker" nil (d r ah1 g m ey1 k er0))
("drugmakers" nil (d r ah1 g m ey1 k er0 z))
("drugs" nil (d r ah1 g z))
("drugstore" nil (d r ah1 g s t ao1 r))
("drugstores" nil (d r ah1 g s t ao1 r z))
("druid" nil (d r uw1 ih0 d))
("druidism" nil (d r uw1 ax d ih1 z ax m))
("druids" nil (d r uw1 ih0 d z))
("druisilla" nil (d r ih0 s ih1 l ax))
("drum" nil (d r ah1 m))
("drumbeat" nil (d r ah1 m b iy1 t))
("drumhead" nil (d r ah1 m hh eh1 d))
("drumheller" nil (d r ah1 m hh eh1 l er0))
("drumm" nil (d r ah1 m))
("drummed" nil (d r ah1 m d))
("drummer" nil (d r ah1 m er0))
("drummers" nil (d r ah1 m er0 z))
("drummey" nil (d r ah1 m iy0))
("drumming" nil (d r ah1 m ih0 ng))
("drummond" nil (d r ah1 m ax n d))
("drummonds" nil (d r ah1 m ax n d z))
("drumright" nil (d r ah1 m r ay1 t))
("drums" nil (d r ah1 m z))
("drumwright" nil (d r ah1 m r ay1 t))
("drunk" nil (d r ah1 ng k))
("drunkard" nil (d r ah1 ng k er0 d))
("drunkards" nil (d r ah1 ng k er0 d z))
("drunken" nil (d r ah1 ng k ax n))
("drunkenness" nil (d r ah1 ng k ax n n ax s))
("drunks" nil (d r ah1 ng k s))
("drupe" nil (d r uw1 p))
("drupes" nil (d r uw1 p s))
("drury" nil (d r uw1 r iy0))
("drusa" nil (d r uw1 s ax))
("druse" nil (d r uw1 z))
("drusie" nil (d r ah1 s iy0))
("drusilla" nil (d r uw1 s ih1 l ax))
("druthers" nil (d r ah1 dh er0 z))
("druze" nil (d r uw1 z))
("dry" nil (d r ay1))
("dryclean" nil (d r ay1 k l iy1 n))
("dryden" nil (d r ay1 d ax n))
("drye" nil (d r ay1))
("dryer" nil (d r ay1 er0))
("dryers" nil (d r ay1 er0 z))
("drying" nil (d r ay1 ih0 ng))
("dryly" nil (d r ay1 l iy0))
("dryness" nil (d r ay1 n ax s))
("drypers" nil (d r ay1 p er0 z))
("drypoint" nil (d r ay1 p oy1 n t))
("drysdale" nil (d r ay1 z d ey1 l))
("drzewiecki" nil (jh uw0 iy1 t s k iy0))
("dsouza" nil (d ax s uw1 z ax))
("du" nil (d uw1))
("dual" nil (d uw1 ax l))
("dualism" nil (d uw1 ax l ih1 z ax m))
("dualisms" nil (d uw1 ax l ih1 z ax m z))
("dualist" nil (d uw1 ax l ih0 s t))
("dualistic" nil (d uw1 ax l ih1 s t ih0 k))
("duality" nil (d uw0 ae1 l ax t iy0))
("duals" nil (d uw1 ax l z))
("duana" nil (d uw0 ae1 n ax))
("duane" nil (d w ey1 n))
("duarte" nil (d w aa1 r t ey1))
("dub" nil (d ah1 b))
("duba" nil (d uw1 b ax))
("dubach" nil (d ah1 b aa0 k))
("dubai" nil (d uw0 b ay1))
("dubarry" nil (d uw1 b eh1 r iy0))
("dubas" nil (d uw1 b ax z))
("dubay" nil (d ah1 b ey0))
("dubbed" nil (d ah1 b d))
("dubberly" nil (d ah1 b er0 l iy0))
("dubbing" nil (d ah1 b ih0 ng))
("dubbs" nil (d ah1 b z))
("dubcek" nil (d ah1 b ch eh1 k))
("dube" nil (d uw1 b))
("dubeau" nil (d ax b ow1))
("duberstein" nil (d uw1 b er0 s t ay1 n))
("dubey" nil (d ah1 b iy0))
("dubicki" nil (d ax b ih1 t s k iy0))
("dubie" nil (d ah1 b iy0))
("dubiel" nil (d ah1 b iy0 l))
("dubilier" nil (d uw1 b ax l ih1 r))
("dubin" nil (d uw1 b ih0 n))
("dubinin" nil (d uw0 b ih1 n ih0 n))
("dubinsky" nil (d ax b ih1 n s k iy0))
("dubious" nil (d uw1 b iy0 ax s))
("dubis" nil (d uw1 b ih0 s))
("dublin" nil (d ah1 b l ih0 n))
("duboff" nil (d ah1 b ao1 f))
("dubois" nil (d uw0 b oy1 s))
("duboise" nil (d ax b w aa1))
("dubord" nil (d ax b ao1 r d))
("dubose" nil (d uw0 b ow1 z))
("dubow" nil (d uw1 b ow0))
("dubray" nil (d ah1 b r ey1))
("dubree" nil (d ax b r iy1))
("dubreuil" nil (d ah1 b r uw0 l))
("dubroc" nil (d ah1 b r ax k))
("dubrow" nil (d ah1 b r ow1))
("dubs" nil (d ah1 b z))
("dubuc" nil (d uw1 b ax k))
("dubuisson" nil (d ah1 b ih0 s ax n))
("dubuque" nil (d ax b y uw1 k))
("duby" nil (d uw1 b iy0))
("duc" nil (d ah1 k))
("duca" nil (d uw1 k ax))
("ducal" nil (d uw1 k ax l))
("duce" nil (d uw1 s))
("ducey" nil (d ah1 s iy0))
("duch" nil (d ah1 ch))
("duchaine" nil (d ax sh ey1 n))
("duchamp" nil (d uw0 sh aa1 m p))
("ducharme" nil (d ax sh aa1 r m))
("duchemin" nil (d ah1 sh ih0 m ae0 n))
("duchene" nil (d ah1 k iy0 n))
("duchenne" nil (d uw0 sh eh1 n))
("duchesneau" nil (d ah1 sh ih0 s n ow0))
("duchess" nil (d ah1 ch ax s))
("duchon" nil (d ah1 ch ax n))
("duchossois" nil (d uw0 ch ao1 s w aa1))
("duchow" nil (d ah1 ch ow0))
("duchy" nil (d ah1 ch iy0))
("duck" nil (d ah1 k))
("ducked" nil (d ah1 k t))
("ducker" nil (d ah1 k er0))
("duckett" nil (d ah1 k ih0 t))
("ducking" nil (d ah1 k ih0 ng))
("duckling" nil (d ah1 k l ih0 ng))
("ducklings" nil (d ah1 k l ih0 ng z))
("ducks" nil (d ah1 k s))
("ducksworth" nil (d ah1 k s w er1 th))
("duckwall" nil (d ah1 k w ao1 l))
("duckweed" nil (d ah1 k w iy1 d))
("duckworth" nil (d ah1 k w er1 th))
("duclos" nil (d ax k l ow1 z))
("ducommun" nil (d uw0 k aa1 m ax n))
("ducote" nil (d ax k ow1 t))
("duct" nil (d ah1 k t))
("ductile" nil (d ah1 k t ax l))
("ductility" nil (d ax k t ih1 l ax t iy0))
("ductless" nil (d ah1 k t l ax s))
("ducts" nil (d ah1 k t s))
("dud" nil (d ah1 d))
("duda" nil (d uw1 d ax))
("dudack" nil (d uw1 d ae0 k))
("dudar" nil (d uw1 d er0))
("dudas" nil (d uw1 d ax z))
("dudash" nil (d ah1 d ax sh))
("dudayev" nil (d uw0 d ay1 eh1 v))
("dudd" nil (d ah1 d))
("dudding" nil (d ah1 d ih0 ng))
("duddy" nil (d ah1 d iy0))
("dude" nil (d uw1 d))
("dudeck" nil (d uw1 d eh0 k))
("dudek" nil (d uw1 d ih0 k))
("duden" nil (d uw1 d ax n))
("dudenhoeffer" nil (d ah1 d ih0 n hh ow0 f er0))
("dudes" nil (d y uw1 d z))
("dudgeon" nil (d ah1 jh ax n))
("dudik" nil (d uw1 d ih0 k))
("dudley" nil (d ah1 d l iy0))
("dudman" nil (d ah1 d m ax n))
("dudney" nil (d ah1 d n iy0))
("duds" nil (d ah1 d z))
("dudziak" nil (d ah1 jh iy0 ae0 k))
("dudzik" nil (d ah1 d z ih0 k))
("dudzinski" nil (d ax jh ih1 n s k iy0))
("due" nil (d uw1))
("duecker" nil (d uw1 k er0))
("dueitt" nil (d uw1 ax t))
("dueker" nil (d uw1 k er0))
("duel" nil (d uw1 ax l))
("dueling" nil (d uw1 l ih0 ng))
("duelist" nil (d uw1 ax l ih0 s t))
("duell" nil (jh uw1 l))
("duels" nil (d uw1 ax l z))
("duena" nil (d uw0 eh1 n ax))
("duenas" nil (d uw0 eh1 n ax s))
("duenez" nil (d w eh0 n eh1 z))
("duenow" nil (d uw1 n ow0))
("duensing" nil (d uh1 n s ih0 ng))
("duer" nil (d uw1 er0))
("duerksen" nil (d uh1 r k s ax n))
("duerr" nil (d uh1 r))
("duerson" nil (d uh1 r s ax n))
("duerst" nil (d uh1 r s t))
("dues" nil (d uw1 z))
("duesberg" nil (d uw1 z b er0 g))
("duesing" nil (d uw1 s ih0 ng))
("duesler" nil (d uw1 s ax l er0))
("duesseldorf" nil (d uw1 s ax l d ao1 r f))
("duet" nil (d uw0 eh1 t))
("duets" nil (d uw0 eh1 t s))
("duey" nil (d uw1 iy0))
("dufault" nil (d ax f ow1))
("dufek" nil (d uw1 f ih0 k))
("duff" nil (d ah1 f))
("duffee" nil (d ah1 f iy0))
("duffek" nil (d ah1 f ih0 k))
("duffel" nil (d ah1 f ax l))
("duffell" nil (d ah1 f ax l))
("duffer" nil (d ah1 f er0))
("duffett" nil (d ah1 f ih0 t))
("duffey" nil (d ah1 f iy0))
("duffie" nil (d ah1 f iy0))
("duffield" nil (d ah1 f iy1 l d))
("duffin" nil (d ah1 f ih0 n))
("duffner" nil (d ah1 f n er0))
("dufford" nil (d ah1 f er0 d))
("duffour" nil (d ah1 f ao0 r))
("duffus" nil (d ah1 f ax s))
("duffy" nil (d ah1 f iy0))
("dufner" nil (d ah1 f n er0))
("duford" nil (d ah1 f er0 d))
("dufort" nil (d ah1 f er0 t))
("dufour" nil (d ax f uh1 r))
("dufrane" nil (d ax f r ey1 n))
("dufrene" nil (d ax f r iy1 n))
("dufresne" nil (d ax f r eh1 n))
("dufur" nil (d ax f er1))
("dug" nil (d ah1 g))
("dugal" nil (d uw1 jh ax l))
("dugald" nil (d ah1 g ax l d))
("dugan" nil (d ah1 g ax n))
("dugar" nil (d uw1 g er0))
("dugas" nil (d uw1 g ax z))
("dugdale" nil (d ah1 g d ey1 l))
("duggan" nil (d ah1 g ax n))
("duggar" nil (d ah1 g er0))
("dugger" nil (d ah1 g er0))
("duggin" nil (d ah1 g ih0 n))
("duggins" nil (d ah1 g ih0 n z))
("dugo" nil (d uw1 g ow0))
("dugout" nil (d ah1 g aw1 t))
("dugouts" nil (d ah1 g aw1 t s))
("duguay" nil (d ah1 g ey0))
("duguid" nil (d ah1 g w ih0 d))
("duhaime" nil (d uw1 ay0 m))
("duhamel" nil (d ah1 hh ax m eh0 l))
("duhart" nil (d ah1 hh aa0 r t))
("duhe" nil (d uw1 hh))
("duhon" nil (d uw1 hh ax n))
("duignan" nil (d ih0 g n ae1 n))
("duis" nil (d uw1 ih0 z))
("duisburg" nil (d uw1 s b er0 g))
("duitsman" nil (d uw1 t s m ax n))
("duk" nil (d ah1 k))
("dukakis" nil (d uw0 k aa1 k ih0 s))
("dukakises" nil (d uw0 k aa1 k ih0 s ih0 z))
("dukart" nil (d ah1 k aa0 r t))
("duke" nil (d uw1 k))
("dukedom" nil (d uw1 k d ax m))
("dukeman" nil (d uw1 k m ax n))
("duker" nil (d uw1 k er0))
("dukes" nil (d uw1 k s))
("dula" nil (d uw1 l ax))
("dulac" nil (d ax l ae1 k))
("dulak" nil (d uw1 l ax k))
("dulaney" nil (d y uw1 l ax n iy0))
("dulany" nil (d y uw0 l ao1 n iy0))
("dulay" nil (d y uw1 l ey0))
("dulce" nil (d ah1 l s))
("dulcea" nil (d ah1 l s iy0 ax))
("dulciana" nil (d uw0 l ch ae1 n ax))
("dulcibelle" nil (d ah1 l s ih0 b ax l))
("dulcie" nil (d ah1 l k iy0))
("dulcimer" nil (d ah1 l s ih0 m er0))
("dulcine" nil (d ah1 l s ih0 n))
("dulcinea" nil (d ah1 l s ih0 n iy1 ax))
("duley" nil (d y uw1 l iy0))
("dulin" nil (d uw1 l ih0 n))
("duling" nil (d y uw1 l ih0 ng))
("dull" nil (d ah1 l))
("dulle" nil (d ah1 l))
("dullea" nil (d ah1 l iy0 ax))
("dulled" nil (d ah1 l d))
("duller" nil (d ah1 l er0))
("dulles" nil (d ah1 l ax s))
("dullest" nil (d ah1 l ax s t))
("dulling" nil (d ah1 l ih0 ng))
("dullness" nil (d ah1 l n ax s))
("dulong" nil (d y uw1 l ao0 ng))
("dulski" nil (d ah1 l s k iy0))
("dulude" nil (d y uw1 l uw1 d))
("duluth" nil (d ax l uw1 th))
("dulwich" nil (d ah1 l w ih1 ch))
("dulworth" nil (d ah1 l w er0 th))
("duly" nil (d uw1 l iy0))
("dum" nil (d ah1 m))
("duma" nil (d uw1 m ax))
("dumaine" nil (d ax m ey1 n))
("dumais" nil (d ax m ey1))
("duman" nil (d uw1 m ax n))
("dumas" nil (d uw1 m ax z))
("dumb" nil (d ah1 m))
("dumbbell" nil (d ah1 m b eh1 l))
("dumbbells" nil (d ah1 m b eh1 l z))
("dumber" nil (d ah1 m er0))
("dumbest" nil (d ah1 m ax s t))
("dumbfounded" nil (d ah1 m f aw1 n d ih0 d))
("dumenil" nil (d uw1 m ax n ih1 l))
("dumez" nil (d uw1 m eh0 z))
("dumford" nil (d ah1 m f er0 d))
("dumire" nil (d uw0 m ih1 r ey0))
("dumke" nil (d ah1 m k iy0))
("dumlao" nil (d uw1 m l aw0))
("dumler" nil (d ah1 m l er0))
("dumm" nil (d ah1 m))
("dummer" nil (d ah1 m er0))
("dummies" nil (d ah1 m iy0 z))
("dummitt" nil (d ah1 m ih0 t))
("dummy" nil (d ah1 m iy0))
("dumond" nil (d ax m aa1 n d))
("dumont" nil (d uw0 m aa1 n t))
("dumoulin" nil (d ah1 m uw0 l ae0 n))
("dump" nil (d ah1 m p))
("dumped" nil (d ah1 m p t))
("dumper" nil (d ah1 m p er0))
("dumping" nil (d ah1 m p ih0 ng))
("dumplings" nil (d ah1 m p l ih0 ng z))
("dumps" nil (d ah1 m p s))
("dumpster" nil (d ah1 m p s t er0))
("dumpsters" nil (d ah1 m p s t er0 z))
("dumpty" nil (d ah1 m p t iy0))
("dumpy" nil (d ah1 m p iy0))
("dun" nil (d ah1 n))
("duna" nil (d uw1 n ax))
("dunagan" nil (d uw0 n aa1 g aa0 n))
("dunahoo" nil (d uw0 n aa1 hh uw0))
("dunaj" nil (d uw1 n ax jh))
("dunavan" nil (d ah1 n ax v ae1 n))
("dunavant" nil (d uw0 n aa1 v ax n t))
("dunaway" nil (d ah1 n ax w ey1))
("dunay" nil (d ah1 n ey0))
("dunbar" nil (d ah1 n b aa0 r))
("dunc" nil (d ah1 ng k))
("duncan" nil (d ah1 ng k ax n))
("duncanson" nil (d ah1 ng k ax n s ax n))
("dunckel" nil (d ah1 ng k ax l))
("duncombe" nil (d ah1 ng k ax m))
("dundas" nil (d ah1 n d ax z))
("dundee" nil (d ax n d iy1))
("dundon" nil (d ah1 n d ax n))
("dundore" nil (d ah1 n d er0))
("dune" nil (d uw1 n))
("dunedin" nil (d uw1 n ax d ih0 n))
("dunegan" nil (d ah1 n ih0 g ae0 n))
("dunes" nil (d uw1 n z))
("dunfee" nil (d ah1 n f iy1))
("dunford" nil (d ah1 n f er0 d))
("dung" nil (d ah1 ng))
("dungan" nil (d ah1 ng g ax n))
("dungey" nil (d ah1 n jh iy0))
("dunham" nil (d ah1 n ax m))
("dunhill" nil (d ah1 n hh ih1 l))
("dunigan" nil (d ah1 n ih0 g ae0 n))
("dunite" nil (d uw0 n ay1 t))
("dunivan" nil (d ah1 n ih0 v ae0 n))
("dunk" nil (d ah1 ng k))
("dunked" nil (d ah1 ng k t))
("dunkel" nil (d ah1 ng k ax l))
("dunkelberg" nil (d ah1 ng k ax l b er0 g))
("dunkelberger" nil (d ah1 ng k ax l b er0 g er0))
("dunker" nil (d ah1 ng k er0))
("dunkerley" nil (d ah1 ng k er0 l iy0))
("dunkin" nil (d ah1 ng k ih0 n))
("dunkirk" nil (d ah1 n k er0 k))
("dunkle" nil (d ah1 ng k ax l))
("dunkleberger" nil (d ah1 ng k ax l b er0 g er0))
("dunklee" nil (d ah1 ng k l iy1))
("dunkley" nil (d ah1 ng k l iy0))
("dunklin" nil (d ah1 ng k l ih0 n))
("dunks" nil (d ah1 ng k s))
("dunlaevy" nil (d ax n l ey1 v iy0))
("dunlap" nil (d ah1 n l ae0 p))
("dunlavey" nil (d ax n l ax v ey1))
("dunlavy" nil (d ah1 n l ax v iy0))
("dunlay" nil (d ah1 n l ey1))
("dunleavy" nil (d uw1 n l ax v iy0))
("dunlevy" nil (d uw1 n l ih0 v iy0))
("dunley" nil (d ah1 n l iy0))
("dunlop" nil (d ah1 n l aa1 p))
("dunman" nil (d ah1 n m ax n))
("dunmire" nil (d uw0 n m ih1 r ey0))
("dunn" nil (d ah1 n))
("dunnagan" nil (d ah1 n ax g ae0 n))
("dunnam" nil (d ah1 n ax m))
("dunnavant" nil (d ah1 n ax v ax n t))
("dunnaway" nil (d ah1 n ax w ey0))
("dunne" nil (d ah1 n))
("dunned" nil (d ah1 n d))
("dunnell" nil (d ah1 n ax l))
("dunnett" nil (d ah1 n ih0 t))
("dunnigan" nil (d ah1 n ih0 g ax n))
("dunning" nil (d ah1 n ih0 ng))
("dunnington" nil (d ah1 n ih0 ng t ax n))
("dunphy" nil (d ah1 n f iy0))
("duns" nil (d ah1 n z))
("dunshee" nil (d ah1 n sh iy0))
("dunsmoor" nil (d ah1 n z m uh1 r))
("dunsmore" nil (d ah1 n s m ao0 r))
("dunson" nil (d ah1 n s ax n))
("dunst" nil (d ah1 n s t))
("dunstan" nil (d ah1 n s t ax n))
("dunster" nil (d ah1 n s t er0))
("dunston" nil (d ah1 n s t ax n))
("dunsworth" nil (d ah1 n z w er1 th))
("dunton" nil (d ah1 n t ax n))
("dunwoody" nil (d ah1 n w uh1 d iy0))
("dunworth" nil (d ah1 n w er1 th))
("duo" nil (d uw1 ow0))
("duodenal" nil (d uw1 ax d iy1 n ax l))
("duong" nil (d uw1 ow0 ng))
("duopoly" nil (d uw1 ow0 p aa1 l iy0))
("dupe" nil (d uw1 p))
("duped" nil (d uw1 p t))
("dupee" nil (d uw1 p iy1))
("duper" nil (d uw1 p er0))
("dupin" nil (d ax p ae1 n))
("duplantis" nil (d ax p l ae1 n t ih0 s))
("duplechain" nil (d uw1 p l ih0 sh ey0 n))
("duplechin" nil (d uw1 p l ih0 k ih0 n))
("dupler" nil (d uw1 p ax l er0))
("duplessis" nil (d uw1 p l ih0 s ih0 s))
("duplex" nil (d uw1 p l eh1 k s))
("duplicate" n (d uw1 p l ax k ax t))
("duplicate" v (d uw1 p l ax k ey1 t))
("duplicated" nil (d uw1 p l ih0 k ey1 t ih0 d))
("duplicates" v (d y uw1 p l ax k ey1 t s))
("duplicates" n (d y uw1 p l ax k ax t s))
("duplicating" nil (d uw1 p l ih0 k ey1 t ih0 ng))
("duplication" nil (d y uw1 p l ax k ey1 sh ax n))
("duplications" nil (d uw1 p l ih0 k ey1 sh ax n z))
("duplicative" nil (d uw0 p l ih1 k ax t ih0 v))
("duplicitous" nil (d uw0 p l ih1 s ih0 t ax s))
("duplicity" nil (d uw0 p l ih1 s ih0 t iy0))
("dupont" nil (d uw1 p aa0 n t))
("dupras" nil (d ax p r aa1 z))
("dupre" nil (d ah1 p er0))
("dupree" nil (d ax p r iy1))
("duprey" nil (d ah1 p r iy0))
("dupriest" nil (d ah1 p er0 iy0 ih0 s t))
("dupuis" nil (d ah1 p uw0 ih0 z))
("dupuy" nil (d ax p ay1))
("duque" nil (d uw1 k))
("duquesne" nil (d uw0 k ey1 n))
("duquette" nil (d ax k eh1 t))
("dura" nil (d uh1 r ax))
("durability" nil (d er0 ax b ih1 l ih0 t iy0))
("durable" nil (d uh1 r ax b ax l))
("durables" nil (d uh1 r ax b ax l z))
("duracell" nil (d uh1 r ax s eh1 l))
("durakon" nil (d uh1 r ax k ih0 n))
("durall" nil (d y uw1 r ax l))
("duramed" nil (d uh1 r ax m eh1 d))
("duran" nil (d er0 ae1 n))
("durando" nil (d uh0 r aa1 n d ow0))
("durango" nil (d er0 ae1 ng g ow0))
("durant" nil (d uh1 r ax n t))
("durante" nil (d uh0 r aa1 n t iy0))
("duration" nil (d uh1 r ey1 sh ax n))
("duray" nil (d uh1 r ey0))
("durazo" nil (d uh0 r aa1 z ow0))
("durban" nil (d er1 b ax n))
("durben" nil (d er1 b ax n))
("durbin" nil (d er1 b ih0 n))
("durchholz" nil (d er1 ch ow1 l t s))
("durden" nil (d er1 d ax n))
("durdin" nil (d er1 d ih0 n))
("duree" nil (d uh1 r iy1))
("durell" nil (d y uw1 r ax l))
("duren" nil (d uh1 r ax n))
("durenberger" nil (d uh1 r ax n b er0 g er0))
("durene" nil (d y uw1 r iy0 n))
("durer" nil (d uh1 r er0))
("duress" nil (d uh1 r eh0 s))
("durette" nil (d er0 eh1 t))
("durfee" nil (d er1 f iy0))
("durfey" nil (d er1 f iy0))
("durflinger" nil (d er1 f ax l ih0 ng er0))
("durgan" nil (d er1 g ax n))
("durgin" nil (d er1 jh ih0 n))
("durham" nil (d er1 ax m))
("durick" nil (d y uw1 r ih0 k))
("durie" nil (d uh1 r iy0))
("during" nil (d uh1 r ih0 ng))
("durio" nil (d uh1 r iy0 ow0))
("duriron" nil (d uw0 r ih1 r ax n))
("duris" nil (d y uw1 r ih0 s))
("durkee" nil (d er1 k iy0))
("durkin" nil (d er1 k ih0 n))
("durland" nil (d er1 l ax n d))
("durley" nil (d er1 l iy0))
("durling" nil (d er1 l ih0 ng))
("durn" nil (d er1 n))
("durnell" nil (d er1 n ax l))
("durney" nil (d er1 n iy0))
("durnil" nil (d er1 n ax l))
("durnin" nil (d er1 n ih0 n))
("durning" nil (d er1 n ih0 ng))
("duro" nil (d uh1 r ow1))
("durocher" nil (d y uw1 r ax k er0))
("duron" nil (d y uw1 r ax n))
("duross" nil (d y uw1 r ax s))
("durr" nil (d er1))
("durrah" nil (d ao1 r ax))
("durrance" nil (d uh1 r ax n s))
("durrant" nil (d ao1 r ax n t))
("durrell" nil (d ao1 r ax l))
("durrence" nil (d ao1 r ax n s))
("durrett" nil (d ao1 r ih0 t))
("durso" nil (d uh1 r s ow0))
("durst" nil (d er1 s t))
("durum" nil (d uh1 r ax m))
("durward" nil (d er1 w er0 d))
("durwin" nil (d er1 w ih0 n))
("dury" nil (d uh1 r iy0))
("duryea" nil (d uh0 r iy1 ax))
("duryee" nil (d uh0 r iy1))
("dusch" nil (d ah1 sh))
("dusek" nil (d uw1 s eh0 k))
("dusenberry" nil (d uw1 s ax n b eh0 r iy0))
("dusenbery" nil (d ax s eh1 n b er0 iy0))
("dusenbury" nil (d uw1 s ax n b eh0 r iy0))
("dush" nil (d ah1 sh))
("dushane" nil (d ah1 sh ax n))
("dusing" nil (d uw1 s ih0 ng))
("dusk" nil (d ah1 s k))
("duskin" nil (d ah1 s k ih0 n))
("dussault" nil (d ax s ow1))
("dusseau" nil (d ax s ow1))
("dusseault" nil (d ax s ow1))
("dusseldorf" nil (d uw1 s ax l d ao1 r f))
("dust" nil (d ah1 s t))
("dusted" nil (d ah1 s t ih0 d))
("duster" nil (d ah1 s t er0))
("dusters" nil (d ah1 s t er0 z))
("dustin" nil (d ah1 s t ih0 n))
("dusting" nil (d ah1 s t ih0 ng))
("dustman" nil (d ah1 s t m ax n))
("duston" nil (d ah1 s t ax n))
("dusts" nil (d ah1 s t s))
("dusty" nil (d ah1 s t iy0))
("dusza" nil (d ah1 sh ax))
("duszynski" nil (d ax sh ih1 n s k iy0))
("dutch" nil (d ah1 ch))
("dutcher" nil (d ah1 ch er0))
("dutchman" nil (d ah1 ch m ax n))
("duthie" nil (d ah1 th iy0))
("duties" nil (d uw1 t iy0 z))
("dutiful" nil (d uw1 t iy0 f ax l))
("dutifully" nil (d uw1 t iy0 f ax l iy0))
("dutil" nil (d ax t ih1 l))
("dutka" nil (d ah1 t k ax))
("dutkiewicz" nil (d ah1 t k ax v ih0 ch))
("dutko" nil (d ah1 t k ow0))
("dutoit" nil (d uw0 t oy1 t))
("dutra" nil (d uw1 t r ax))
("dutro" nil (d ah1 t r ow0))
("dutrow" nil (d ah1 t r ow0))
("dutson" nil (d ah1 t s ax n))
("dutt" nil (d ah1 t))
("dutter" nil (d ah1 t er0))
("dutton" nil (d ah1 t ax n))
("duty" nil (d uw1 t iy0))
("duva" nil (d uw1 v ax))
("duval" nil (d uw0 v ae1 l))
("duvalier" nil (d uw0 v ae1 l y er0))
("duvaliers" nil (d uw0 v ey1 l y er0 z))
("duvall" nil (d uw0 v aa1 l))
("duve" nil (d uw1 v))
("duvernay" nil (d ax v er1 n ey0))
("duwayne" nil (d uw0 w ey1 n))
("duwe" nil (d uw1 w))
("dux" nil (d ah1 k s))
("duzan" nil (d uw1 z ax n))
("dvoracek" nil (d v ao1 r ax ch eh0 k))
("dvorak" nil (d v ao1 r ax k))
("dvorsky" nil (d v ao1 r s k iy0))
("dwan" nil (d w aa1 n))
("dwana" nil (d w aa1 n ax))
("dwarf" nil (d w ao1 r f))
("dwarfed" nil (d w ao1 r f t))
("dwarfing" nil (d w ao1 r f ih0 ng))
("dwarfism" nil (d w ao1 r f ih0 z ax m))
("dwarfs" nil (d w ao1 r f s))
("dwayne" nil (d w ey1 n))
("dweck" nil (d w eh1 k))
("dwell" nil (d w eh1 l))
("dwelle" nil (d w eh1 l))
("dwelled" nil (d w eh1 l d))
("dweller" nil (d w eh1 l er0))
("dwellers" nil (d w eh1 l er0 z))
("dwelley" nil (d w eh1 l iy0))
("dwelling" nil (d w eh1 l ih0 ng))
("dwellings" nil (d w eh1 l ih0 ng z))
("dwells" nil (d w eh1 l z))
("dwelt" nil (d w eh1 l t))
("dwiggins" nil (d w ih1 g ih0 n z))
("dwight" nil (d w ay1 t))
("dwindle" nil (d w ih1 n d ax l))
("dwindled" nil (d w ih1 n d ax l d))
("dwindles" nil (d w ih1 n d ax l z))
("dwindling" nil (d w ih1 n d ax l ih0 ng))
("dwinell" nil (d w ih1 n ax l))
("dwire" nil (d w ay1 r))
("dwivedi" nil (d w ih0 v eh1 d iy0))
("dworak" nil (d w er1 ax k))
("dworin" nil (d w ao1 r ih0 n))
("dworkin" nil (d w ao1 r k ih0 n))
("dworsky" nil (d w er1 s k iy0))
("dwyer" nil (d w ay1 er0))
("dyad" nil (d ay1 ae1 d))
("dyal" nil (d ay1 ax l))
("dyana" nil (d ay0 ae1 n ax))
("dyane" nil (d ay0 ae1 n))
("dyansen" nil (d ay1 ax n s ax n))
("dyar" nil (d ay1 er0))
("dyas" nil (d ay1 ax s))
("dyatron" nil (d ay1 ax t r ax n))
("dyazide" nil (d ay1 ax z ay1 d))
("dybas" nil (d ay1 b ax z))
("dyce" nil (d ay1 s))
("dyche" nil (d ay1 ch))
("dyches" nil (d ay1 ch ih0 z))
("dyck" nil (d ay1 k))
("dyckman" nil (d ih1 k m ax n))
("dyco" nil (d ay1 k ow0))
("dycus" nil (d ay1 k ax s))
("dye" nil (d ay1))
("dyed" nil (d ay1 d))
("dyeing" nil (d ay1 ih0 ng))
("dyer" nil (d ay1 er0))
("dyes" nil (d ay1 z))
("dyess" nil (d ay1 ax s))
("dyestuffs" nil (d ay1 s t ah1 f s))
("dygert" nil (d ih1 g er0 t))
("dying" nil (d ay1 ih0 ng))
("dyk" nil (d ih1 k))
("dykas" nil (d ay1 k ax z))
("dyke" nil (d ay1 k))
("dykeman" nil (d ay1 k m ax n))
("dykes" nil (d ay1 k s))
("dykhouse" nil (d ih1 k hh aw1 s))
("dykman" nil (d ih1 k m ax n))
("dykstra" nil (d ay1 k s t r ax))
("dylan" nil (d ih1 l ax n))
("dylewski" nil (d ih0 l eh1 f s k iy0))
("dylex" nil (d ay1 l ax k s))
("dymally" nil (d ih1 m ax l iy0))
("dymek" nil (d ih1 m eh0 k))
("dyment" nil (d ih1 m ax n t))
("dymond" nil (d ay1 m ax n d))
("dynafac" nil (d ay1 n ax f ae1 k))
("dynalectric" nil (d ay1 n ax l eh1 k t r ih0 k))
("dynalectron" nil (d ay1 n ax l eh1 k t r ax n))
("dynamic" nil (d ay0 n ae1 m ih0 k))
("dynamics" nil (d ay0 n ae1 m ih0 k s))
("dynamism" nil (d ay1 n ax m ih1 z ax m))
("dynamite" nil (d ay1 n ax m ay1 t))
("dynamo" nil (d ay1 n ax m ow1))
("dynapac" nil (d ay1 n ax p ae1 k))
("dynascan" nil (d ay1 n ax s k ae1 n))
("dynastic" nil (d ay0 n ae1 s t ih0 k))
("dynasties" nil (d ay1 n ax s t iy0 z))
("dynasty" nil (d ay1 n ax s t iy0))
("dynatech" nil (d ih1 n ax t eh1 k))
("dyncorp" nil (d ih1 n k ao1 r p))
("dyneer" nil (d ih0 n ih1 r))
("dynes" nil (d ay1 n z))
("dysan" nil (d ay1 s ax n))
("dysart" nil (d ih1 s er0 t))
("dysentery" nil (d ih1 s ax n t eh1 r iy0))
("dysert" nil (d ih1 s er0 t))
("dysfunction" nil (d ih0 s f ah1 ng k sh ax n))
("dysfunctional" nil (d ih0 s f ah1 ng k sh ax n ax l))
("dysinger" nil (d ih1 s ih0 n jh er0))
("dyson" nil (d ay1 s ax n))
("dysplasia" nil (d ih0 s p l ey1 zh ax))
("dystrophin" nil (d ih1 s t r ax f ih0 n))
("dystrophy" nil (d ih1 s t r ax f iy0))
("dzhirkvelov" nil (d ax z er1 k v ax l aa0 v))
("dzhokhar" nil (jh ow1 k aa1 r))
("dziak" nil (d z iy1 ae0 k))
("dzialo" nil (jh iy0 aa1 l ow0))
("dziedzic" nil (jh iy1 jh ih0 k))
("dziekan" nil (jh iy1 k ax n))
("dzik" nil (d z ih1 k))
("dzikowski" nil (jh ih0 k ao1 f s k iy0))
("dziuba" nil (jh iy0 uw1 b ax))
("dziuk" nil (jh iy0 uw1 k))
("e" nil (iy1))
("each" nil (iy1 ch))
("eachan" nil (iy1 ch ax n))
("eachus" nil (iy1 ch ax s))
("eaddy" nil (iy1 d iy0))
("eade" nil (iy1 d))
("eader" nil (iy1 d er0))
("eades" nil (iy1 d z))
("eadie" nil (eh1 d iy0))
("eads" nil (iy1 d z))
("eady" nil (iy1 d iy0))
("eagan" nil (iy1 g ax n))
("eagar" nil (iy1 g er0))
("eagen" nil (iy1 g ax n))
("eager" nil (iy1 g er0))
("eagerly" nil (iy1 g er0 l iy0))
("eagerness" nil (iy1 g er0 n ax s))
("eagle" nil (iy1 g ax l))
("eagleburger" nil (iy1 g ax l b er0 g er0))
("eagles" nil (iy1 g ax l z))
("eagleson" nil (iy1 g l ih0 s ax n))
("eagleton" nil (iy1 g ax l t ax n))
("eagleye" nil (iy1 g ax l ay1))
("eaglin" nil (iy1 g l ih0 n))
("eagon" nil (iy1 g ax n))
("eaken" nil (iy1 k ax n))
("eaker" nil (iy1 k er0))
("eakes" nil (iy1 k s))
("eakin" nil (iy1 k ih0 n))
("eakins" nil (iy1 k ih0 n z))
("eakle" nil (iy1 k ax l))
("eales" nil (iy1 l z))
("ealey" nil (iy1 l iy0))
("ealing" nil (iy1 l ih0 ng))
("ealy" nil (iy1 l iy0))
("eamer" nil (iy1 m er0))
("eames" nil (iy1 m z))
("eamon" nil (iy1 m ax n))
("eanes" nil (iy1 n z))
("ear" nil (ih1 r))
("eardley" nil (ih1 r d l iy0))
("eared" nil (ih1 r d))
("earful" nil (ih1 r f ax l))
("eargle" nil (ih1 r g ax l))
("earhart" nil (ih1 r hh aa0 r t))
("earl" nil (er1 l))
("earldom" nil (er1 l d ax m))
("earle" nil (er1 l))
("earlene" nil (er1 l iy0 n))
("earles" nil (er1 l z))
("earless" nil (ih1 r l ax s))
("earley" nil (er1 l iy0))
("earlie" nil (er1 l iy0))
("earlier" nil (er1 l iy0 er0))
("earliest" nil (er1 l iy0 ax s t))
("earline" nil (er1 l ay0 n))
("earll" nil (er1 l))
("earls" nil (er1 l z))
("early" nil (er1 l iy0))
("earlywine" nil (er1 l iy0 w ay1 n))
("earmark" nil (ih1 r m aa1 r k))
("earmarked" nil (ih1 r m aa1 r k t))
("earmarking" nil (ih1 r m aa1 r k ih0 ng))
("earmarks" nil (ih1 r m aa1 r k s))
("earmuffs" nil (ih1 r m ah1 f s))
("earn" nil (er1 n))
("earned" nil (er1 n d))
("earner" nil (er1 n er0))
("earners" nil (er1 n er0 z))
("earnest" nil (er1 n ih0 s t))
("earnestly" nil (er1 n ax s t l iy0))
("earnestness" nil (er1 n ax s t n ax s))
("earney" nil (er1 n iy0))
("earnhardt" nil (er1 n hh aa1 r t))
("earnhart" nil (er1 n hh aa1 r t))
("earnheart" nil (er1 n hh aa1 r t))
("earnie" nil (er1 n iy0))
("earning" nil (er1 n ih0 ng))
("earnings" nil (er1 n ih0 ng z))
("earns" nil (er1 n z))
("earnshaw" nil (er1 n sh ao1))
("earp" nil (er1 p))
("earphones" nil (ih1 r f ow1 n z))
("earplugs" nil (ih1 r p l ah1 g z))
("earring" nil (ih1 r ih0 ng))
("earrings" nil (ih1 r ih0 ng z))
("ears" nil (ih1 r z))
("earshot" nil (ih1 r sh aa1 t))
("earth" nil (er1 th))
("eartha" nil (er1 th ax))
("earthbound" nil (er1 th b aw1 n d))
("earthen" nil (er1 th ax n))
("earthenware" nil (er1 th ax n w eh1 r))
("earthlings" nil (er1 th l ih0 ng z))
("earthly" nil (er1 th l iy0))
("earthmoving" nil (er1 th m uw1 v ih0 ng))
("earthquake" nil (er1 th k w ey1 k))
("earthquakes" nil (er1 th k w ey1 k s))
("earths" nil (er1 th s))
("earthshaking" nil (er1 th sh ey1 k ih0 ng))
("earthshine" nil (er1 th sh ay1 n))
("earthstar" nil (er1 th s t aa1 r))
("earthwork" nil (er1 th w er1 k))
("earthworm" nil (er1 th w er1 m))
("earthworms" nil (er1 th w er1 m z))
("earthy" nil (er1 th iy0))
("earwax" nil (ih1 r w ae1 k s))
("earwood" nil (ih1 r w uh1 d))
("eary" nil (ih1 r iy0))
("easco" nil (iy1 s k ow0))
("ease" nil (iy1 z))
("eased" nil (iy1 z d))
("easel" nil (iy1 z ax l))
("easement" nil (iy1 z m ax n t))
("eases" nil (iy1 z ih0 z))
("eash" nil (iy1 sh))
("easier" nil (iy1 z iy0 er0))
("easiest" nil (iy1 z iy0 ax s t))
("easily" nil (iy1 z ax l iy0))
("easing" nil (iy1 z ih0 ng))
("easler" nil (iy1 z l er0))
("easley" nil (iy1 z l iy0))
("easom" nil (iy1 z ax m))
("eason" nil (iy1 z ax n))
("east" nil (iy1 s t))
("eastbound" nil (iy1 s t b aw1 n d))
("eastburn" nil (iy1 s t b er1 n))
("eastdil" nil (iy1 s t d ih1 l))
("eastenders" nil (iy1 z t eh1 n d er0 z))
("eastep" nil (iy1 z t ih0 p))
("easter" nil (iy1 s t er0))
("easterbrook" nil (iy1 s t er0 b r uh1 k))
("easterday" nil (iy1 s t er0 d ey1))
("easterlin" nil (ax z t er1 l ih0 n))
("easterling" nil (iy1 s t er0 l ih0 ng))
("easterly" nil (iy1 s t er0 l iy0))
("eastern" nil (iy1 s t er0 n))
("easterner" nil (iy1 s t er0 n er0))
("easterners" nil (iy1 s t er0 n er0 z))
("easterwood" nil (iy1 s t er0 w uh1 d))
("eastes" nil (iy1 s t s))
("eastgroup" nil (iy1 s t g r uw1 p))
("eastham" nil (iy1 s t ax m))
("eastin" nil (iy1 z t ih0 n))
("eastland" nil (iy1 s t l ax n d))
("eastlick" nil (iy1 s t l ih1 k))
("eastman" nil (iy1 s t m ax n))
("eastmet" nil (iy1 s t m eh1 t))
("easton" nil (iy1 s t ax n))
("eastover" nil (iy1 s t ow1 v er0))
("eastpac" nil (iy1 s t p ae1 k))
("eastridge" nil (iy1 s t r ih1 jh))
("eastward" nil (iy1 s t w er0 d))
("eastwick" nil (iy1 s t w ih1 k))
("eastwood" nil (iy1 s t w uh1 d))
("easudes" nil (ey1 s uw1 d ey0 s))
("easy" nil (iy1 z iy0))
("easygoing" nil (iy1 z iy0 g ow1 ih0 ng))
("eat" nil (iy1 t))
("eatable" nil (iy1 t ax b ax l))
("eaten" nil (iy1 t ax n))
("eater" nil (iy1 t er0))
("eateries" nil (iy1 t er0 iy0 z))
("eaters" nil (iy1 t er0 z))
("eatery" nil (iy1 t er0 iy0))
("eatherly" nil (eh1 dh er0 l iy0))
("eatherton" nil (eh0 th er1 t ax n))
("eating" nil (iy1 t ih0 ng))
("eatman" nil (iy1 t m ax n))
("eatmon" nil (iy1 t m ax n))
("eaton" nil (iy1 t ax n))
("eats" nil (iy1 t s))
("eau" nil (ow1))
("eaux" nil (ow1))
("eave" nil (iy1 v))
("eavenson" nil (iy1 v ih0 n s ax n))
("eaves" nil (iy1 v z))
("eavesdrop" nil (iy1 v z d r aa1 p))
("eavesdropping" nil (iy1 v z d r aa1 p ih0 ng))
("eb" nil (eh1 b))
("eba" nil (iy1 b ax))
("eban" nil (eh1 b ax n))
("ebanks" nil (eh1 b ax ng k s))
("ebano" nil (eh0 b aa1 n ow0))
("ebarb" nil (eh1 b aa0 r b))
("ebasco" nil (eh0 b ae1 s k ow0))
("ebaugh" nil (eh1 b ao0))
("ebb" nil (eh1 b))
("ebba" nil (eh1 b ax))
("ebbed" nil (eh1 b d))
("ebben" nil (eh1 b ax n))
("ebbers" nil (eh1 b er0 z))
("ebbert" nil (eh1 b er0 t))
("ebbing" nil (eh1 b ih0 ng))
("ebbs" nil (eh1 b z))
("ebby" nil (eh1 b iy0))
("ebel" nil (eh1 b ax l))
("ebeling" nil (eh1 b ax l ih0 ng))
("eben" nil (eh1 b ax n))
("ebeneezer" nil (eh1 b ax n iy1 z er0))
("ebenezer" nil (eh1 b ih0 n iy1 z er0))
("eber" nil (eh1 b er0))
("eberhard" nil (eh1 b er0 hh aa0 r t))
("eberhardt" nil (eh1 b er0 hh aa1 r t))
("eberhart" nil (eh1 b er0 hh aa1 r t))
("eberl" nil (eh1 b er0 l))
("eberle" nil (eh1 b er0 ax l))
("eberlein" nil (eh1 b er0 l ay1 n))
("eberlin" nil (eh1 b er0 l ih0 n))
("eberling" nil (eh1 b er0 l ih0 ng))
("eberly" nil (eh1 b er0 l iy0))
("ebers" nil (eh1 b er0 z))
("ebersol" nil (eh1 b er0 s aa0 l))
("ebersold" nil (eh1 b er0 s ow1 l d))
("ebersole" nil (eh1 b er0 s ow1 l))
("eberstadt" nil (eh1 b er0 s t ae1 t))
("ebert" nil (eh1 b er0 t))
("eberts" nil (eh1 b er0 t s))
("eberwein" nil (eh1 b er0 w ay1 n))
("ebey" nil (eh1 b iy0))
("ebinger" nil (eh1 b ih0 ng er0))
("eble" nil (eh1 b ax l))
("eblen" nil (eh1 b ax l ax n))
("eblin" nil (eh1 b l ih0 n))
("ebling" nil (eh1 b ax l ih0 ng))
("ebner" nil (eh1 b n er0))
("ebola" nil (iy0 b ow1 l ax))
("ebony" nil (eh1 b ax n iy0))
("ebright" nil (ih0 b r ay1 t))
("ebro" nil (iy1 b r ow1))
("ebron" nil (eh1 b r ax n))
("ebullience" nil (ih0 b uh1 l y ax n s))
("ebullient" nil (ih0 b ah1 l y ax n t))
("eby" nil (iy1 b iy0))
("ecad" nil (iy1 k ae1 d))
("eccentric" nil (ih0 k s eh1 n t r ih0 k))
("eccentricities" nil (eh1 k s eh0 n t r ih1 s ih0 t iy0 z))
("eccentricity" nil (eh1 k s ax n t r ih1 s ax t iy0))
("eccentrics" nil (ih0 k s eh1 n t r ih0 k s))
("eccles" nil (eh1 k ax l z))
("ecclesiastical" nil (ih0 k l iy1 z iy0 ae1 s t ih0 k ax l))
("eccleston" nil (eh1 k l ih0 s t aa0 n))
("ecevit" nil (eh1 s ax v ih0 t))
("echard" nil (eh1 ch er0 d))
("echavarria" nil (eh1 ch ax v ae1 r iy0 ax))
("echelon" nil (eh1 sh ax l aa1 n))
("echelons" nil (eh1 sh ax l aa1 n z))
("echenberg" nil (eh1 k ax n b er0 g))
("echevarria" nil (ey0 ch ey0 v aa1 r iy0 ax))
("echeverria" nil (ey0 ch ey0 v eh1 r iy0 ax))
("echlin" nil (eh1 k l ih0 n))
("echo" nil (eh1 k ow0))
("echoed" nil (eh1 k ow0 d))
("echoes" nil (eh1 k ow0 z))
("echohawk" nil (eh1 k ow0 hh ao1 k))
("echoing" nil (eh1 k ow0 ih0 ng))
("echolocation" nil (eh1 k ow0 l ow0 k ey1 sh ax n))
("echols" nil (eh1 k ax l z))
("echos" nil (eh1 k ow0 z))
("eck" nil (eh1 k))
("eckard" nil (eh1 k er0 d))
("eckardt" nil (eh1 k er0 t))
("eckart" nil (eh1 k aa0 r t))
("eckberg" nil (eh1 k b er0 g))
("eckel" nil (eh1 k ax l))
("eckelberry" nil (eh1 k ax l b eh1 r iy0))
("eckels" nil (eh1 k ax l z))
("eckenfelder" nil (eh1 k ax n f eh1 l d er0))
("eckenrod" nil (eh1 k ih0 n r ax d))
("eckenrode" nil (eh1 k ih0 n r ow1 d))
("eckenroth" nil (eh1 k ih0 n r ao0 th))
("ecker" nil (eh1 k er0))
("eckerd" nil (eh1 k er0 d))
("eckerle" nil (eh1 k er0 ax l))
("eckerman" nil (eh1 k er0 m ax n))
("eckersley" nil (eh1 k er0 s l iy0))
("eckerson" nil (eh1 k er0 s ax n))
("eckert" nil (eh1 k er0 t))
("eckes" nil (eh1 k s))
("eckhard" nil (eh1 k hh aa1 r d))
("eckhardt" nil (eh1 k hh aa1 r t))
("eckhart" nil (eh1 k hh aa1 r t))
("eckhoff" nil (eh1 k hh ao1 f))
("eckl" nil (eh1 k ax l))
("eckland" nil (eh1 k l ax n d))
("eckler" nil (eh1 k l er0))
("eckles" nil (eh1 k ax l z))
("eckley" nil (eh1 k l iy0))
("ecklund" nil (eh1 k l ax n d))
("eckman" nil (eh1 k m ax n))
("eckmann" nil (eh1 k m ax n))
("eckrich" nil (eh1 k r ih0 k))
("eckrote" nil (eh1 k r ax t))
("eckroth" nil (eh1 k r ao1 th))
("eckstein" nil (eh1 k s t iy1 n))
("eckstrom" nil (eh1 k s t r ax m))
("eclectic" nil (ih0 k l eh1 k t ih0 k))
("eclipse" nil (ih0 k l ih1 p s))
("eclipsed" nil (ih0 k l ih1 p s t))
("eclipses" nil (ih0 k l ih1 p s ax z))
("eclipsing" nil (ih0 k l ih1 p s ih0 ng))
("ecliptic" nil (ih0 k l ih1 p t ih0 k))
("eco" nil (iy1 k ow0))
("ecogen" nil (iy1 k ow0 g ax n))
("ecolab" nil (iy1 k ow0 l ae1 b))
("ecolaire" nil (iy1 k ow0 l eh1 r))
("ecole" nil (ih0 k ow1 l))
("ecological" nil (iy0 k ax l aa1 jh ih0 k ax l))
("ecologically" nil (iy0 k ax l aa1 jh ih0 k l iy0))
("ecologist" nil (ih0 k aa1 l ax jh ih0 s t))
("ecologists" nil (ih0 k aa1 l ax jh ih0 s t s))
("ecology" nil (ih0 k aa1 l ax jh iy0))
("econ" nil (iy1 k aa1 n))
("econocom" nil (iy0 k aa1 n ow0 k aa1 m))
("econoline" nil (iy0 k aa1 n ow0 l ay1 n))
("econometric" nil (ih0 k aa1 n ax m eh1 t r ih0 k))
("econometrics" nil (ih0 k aa1 n ax m eh1 t r ih0 k s))
("economic" nil (eh1 k ax n aa1 m ih0 k))
("economical" nil (eh1 k ax n aa1 m ih0 k ax l))
("economically" nil (eh1 k ax n aa1 m ih0 k l iy0))
("economico" nil (iy0 k aa1 n ax m iy1 k ow0))
("economics" nil (eh1 k ax n aa1 m ih0 k s))
("economies" nil (ih0 k aa1 n ax m iy0 z))
("economist" nil (ih0 k aa1 n ax m ih0 s t))
("economists" nil (ih0 k aa1 n ax m ih0 s t s))
("economize" nil (ih0 k aa1 n ax m ay1 z))
("economos" nil (eh0 k ax n ow1 m ow0 z))
("economou" nil (eh0 k ow0 n ow1 m uw0))
("economy" nil (ih0 k aa1 n ax m iy0))
("econships" nil (iy1 k aa1 n sh ih1 p s))
("ecosystem" nil (iy1 k ow0 s ih1 s t ax m))
("ecosystems" nil (iy1 k ow0 s ih1 s t ax m z))
("ecru" nil (eh1 k r uw0))
("ecstasy" nil (eh1 k s t ax s iy0))
("ecstatic" nil (eh0 k s t ae1 t ih0 k))
("ecstatically" nil (eh0 k s t ae1 t ih0 k l iy0))
("ecton" nil (eh1 k t ax n))
("ector" nil (eh1 k t er0))
("ecuador" nil (eh1 k w ax d ao1 r))
("ecuadoran" nil (eh1 k w ax d ao1 r ax n))
("ecuadorean" nil (eh1 k w ax d ao1 r iy0 ax n))
("ecuadorian" nil (eh1 k w ax d ao1 r iy0 ax n))
("ecumena" nil (eh1 k y uw0 m iy1 n ax))
("ecumenical" nil (eh1 k y uw0 m eh1 n ih0 k ax l))
("eczema" nil (eh1 k s ax m ax))
("ed" nil (eh1 d))
("eda" nil (iy1 d ax))
("edam" nil (iy1 d ax m))
("edan" nil (iy1 d ax n))
("edana" nil (eh0 d ae1 n ax))
("edberg" nil (eh1 d b er0 g))
("edbert" nil (eh1 d b er0 t))
("eddie" nil (eh1 d iy0))
("eddies" nil (eh1 d iy0 z))
("eddinger" nil (eh1 d ih0 ng er0))
("eddings" nil (eh1 d ih0 ng z))
("eddington" nil (eh1 d ih0 ng t ax n))
("eddins" nil (eh1 d ih0 n z))
("eddleman" nil (eh1 d ax l m ax n))
("eddlemon" nil (eh1 d l ih0 m aa0 n))
("edds" nil (eh1 d z))
("eddy" nil (eh1 d iy0))
("ede" nil (iy1 d))
("edel" nil (eh1 d ax l))
("edelen" nil (eh1 d ax l ax n))
("edeline" nil (eh1 d ih0 l ay0 n))
("edell" nil (ih0 d eh1 l))
("edelman" nil (eh1 d ax l m ax n))
("edelmann" nil (eh1 d ax l m ax n))
("edelmar" nil (eh1 d ih0 l m er0))
("edelson" nil (eh1 d ih0 l s ax n))
("edelstein" nil (eh1 d ax l s t ay1 n))
("edelweiss" nil (ey1 d ax l v ay1 s))
("edema" nil (ih0 d iy1 m ax))
("eden" nil (iy1 d ax n))
("edenfield" nil (ih0 d eh1 n f iy0 l d))
("edens" nil (iy1 d ax n z))
("eder" nil (eh1 d er0))
("ederer" nil (eh1 d er0 er0))
("edes" nil (iy1 d z))
("edgar" nil (eh1 d g er0))
("edgcomb" nil (eh1 jh k ax m))
("edge" nil (eh1 jh))
("edgecomb" nil (eh1 jh k ao0 m))
("edgecombe" nil (eh1 jh k ow0 m))
("edged" nil (eh1 jh d))
("edgell" nil (eh1 jh ax l))
("edgemon" nil (eh1 jh m ax n))
("edger" nil (eh1 jh er0))
("edgerly" nil (eh1 jh er0 l iy0))
("edgerton" nil (eh1 jh er0 t ax n))
("edges" nil (eh1 jh ax z))
("edgett" nil (eh1 jh ih0 t))
("edgeways" nil (eh1 jh w ey1 z))
("edgewood" nil (eh1 jh w uh1 d))
("edgeworth" nil (eh1 jh w er0 th))
("edgin" nil (eh1 jh ih0 n))
("edginess" nil (eh1 jh iy0 n ax s))
("edging" nil (eh1 jh ih0 ng))
("edgington" nil (eh1 jh ih0 ng t ax n))
("edgley" nil (eh1 jh l iy0))
("edgmon" nil (eh1 jh m ax n))
("edgren" nil (eh1 d g r eh0 n))
("edgy" nil (eh1 jh iy0))
("edi" nil (iy1 d iy0))
("edible" nil (eh1 d ax b ax l))
("edibles" nil (eh1 d ax b ax l z))
("edick" nil (eh1 d ih0 k))
("edict" nil (iy1 d ih0 k t))
("edicts" nil (iy1 d ih0 k t s))
("edie" nil (eh1 d iy0))
("edification" nil (eh1 d ax f ax k ey1 sh ax n))
("edifice" nil (eh1 d ax f ax s))
("edifying" nil (eh1 d ax f ay1 ih0 ng))
("ediger" nil (eh1 d ih0 g er0))
("edin" nil (eh1 d ih0 n))
("edina" nil (ax d iy1 n ax))
("edinboro" nil (eh1 d ax n b er0 ow0))
("edinburgh" nil (eh1 d ax n b er0 ow0))
("edinger" nil (eh1 d ih0 ng er0))
("edington" nil (eh1 d ih0 ng t ax n))
("edison" nil (eh1 d ih0 s ax n))
("edisto" nil (eh1 d ih0 s t ow0))
("edit" nil (eh1 d ax t))
("edited" nil (eh1 d ax t ax d))
("edith" nil (iy1 d ih0 th))
("editha" nil (eh1 d ih0 dh ax))
("edithe" nil (eh1 d ih0 dh))
("editing" nil (eh1 d ax t ih0 ng))
("edition" nil (ax d ih1 sh ax n))
("editions" nil (ih0 d ih1 sh ax n z))
("editor" nil (eh1 d ax t er0))
("editorial" nil (eh1 d ax t ao1 r iy0 ax l))
("editorialist" nil (eh1 d ax t ao1 r iy0 ax l ih0 s t))
("editorialists" nil (eh1 d ax t ao1 r iy0 ax l ih0 s t s))
("editorialize" nil (eh1 d ax t ao1 r iy0 ax l ay1 z))
("editorialized" nil (eh1 d ax t ao1 r iy0 ax l ay1 z d))
("editorially" nil (eh1 d ax t ao1 r iy0 ax l iy0))
("editorials" nil (eh1 d ax t ao1 r iy0 ax l z))
("editors" nil (eh1 d ih0 t er0 z))
("editorship" nil (eh1 d ax t er0 sh ih1 p))
("edits" nil (eh1 d ih0 t s))
("ediva" nil (eh0 d iy1 v ax))
("edizione" nil (eh0 d iy1 z iy0 ow1 n iy0))
("edler" nil (eh1 d l er0))
("edley" nil (eh1 d l iy0))
("edlin" nil (eh1 d l ih0 n))
("edling" nil (eh1 d l ih0 ng))
("edlund" nil (eh1 d l ax n d))
("edlyn" nil (eh1 d l ih0 n))
("edman" nil (eh1 d m ax n))
("edmands" nil (eh1 d m ax n d z))
("edmar" nil (eh1 d m aa0 r))
("edminster" nil (ih0 d m ih1 n s t er0))
("edmison" nil (eh1 d m ih0 s ax n))
("edmisten" nil (ih0 d m ih1 s ax n))
("edmister" nil (eh1 d m ih0 s t er0))
("edmiston" nil (eh1 d m ih0 s t aa0 n))
("edmond" nil (eh1 d m ax n d))
("edmonda" nil (eh1 d m aa1 n d ax))
("edmonds" nil (eh1 d m ax n d z))
("edmondson" nil (eh1 d m ax n d s ax n))
("edmonson" nil (eh1 d m ax n s ax n))
("edmonston" nil (ih0 d m aa1 n s t ax n))
("edmonton" nil (eh1 d m ax n t ax n))
("edmund" nil (eh1 d m ax n d))
("edmunda" nil (eh1 d m ah1 n d ax))
("edmundo" nil (eh1 d m ah1 n d ow0))
("edmunds" nil (eh1 d m ax n d z))
("edmundson" nil (eh1 d m ax n d s ax n))
("edna" nil (eh1 d n ax))
("edney" nil (eh1 d n iy0))
("edo" nil (iy1 d ow0))
("edoardo" nil (eh1 d w aa1 r d ow0))
("edolf" nil (eh1 d ow0 l f))
("edouard" nil (eh1 d uw0 aa0 r d))
("edper" nil (eh1 d p er0))
("edquist" nil (eh1 d k w ih1 s t))
("edra" nil (eh1 d r ax))
("edrea" nil (eh1 d r iy0 ax))
("edric" nil (eh1 d r ih0 k))
("edrington" nil (eh1 d er0 ih0 ng t ax n))
("edris" nil (ih0 d r iy1 s))
("edsall" nil (ih0 d s ao1 l))
("edsel" nil (eh1 d s ax l))
("edson" nil (eh1 d s ax n))
("edstrom" nil (eh1 d s t r ax m))
("eduard" nil (eh1 d w er0 d))
("eduardo" nil (eh0 d w aa1 r d ow0))
("educate" nil (eh1 jh ax k ey1 t))
("educated" nil (eh1 jh ax k ey1 t ax d))
("educating" nil (eh1 jh ax k ey1 t ih0 ng))
("education" nil (eh1 jh ax k ey1 sh ax n))
("educational" nil (eh1 jh ax k ey1 sh ax n ax l))
("educationally" nil (eh1 jh ax k ey1 sh ax n ax l iy0))
("educations" nil (eh1 jh ax k ey1 sh ax n z))
("educator" nil (eh1 jh ax k ey1 t er0))
("educators" nil (eh1 jh ax k ey1 t er0 z))
("edwald" nil (ih0 d w ao1 l d))
("edward" nil (eh1 d w er0 d))
("edwardian" nil (eh0 d w ao1 r d iy0 ax n))
("edwardine" nil (ih0 d w ao1 r d ay0 n))
("edwards" nil (eh1 d w er0 d z))
("edwardson" nil (eh1 d w ao0 r d s ax n))
("edwin" nil (eh1 d w ax n))
("edwina" nil (eh0 d w iy1 n ax))
("edyth" nil (eh1 d ih0 th))
("edythe" nil (eh1 d ay0 dh))
("edzard" nil (eh1 d z er0 d))
("ee" nil (iy1))
("eeg" nil (iy1 g))
("eel" nil (iy1 l))
("eelam" nil (iy1 l ae0 m))
("eelgrass" nil (iy1 l g r ae1 s))
("eellike" nil (iy1 l l ay1 k))
("eells" nil (iy1 l z))
("eelpout" nil (iy1 l p aw1 t))
("eelpouts" nil (iy1 l p aw1 t s))
("eels" nil (iy1 l z))
("eerie" nil (ih1 r iy0))
("eerily" nil (ih1 r ax l iy0))
("efaw" nil (eh1 f ao0))
("effacing" nil (ih0 f ey1 s ih0 ng))
("effect" nil (ih0 f eh1 k t))
("effected" nil (ih0 f eh1 k t ax d))
("effecting" nil (ih0 f eh1 k t ih0 ng))
("effective" nil (ih0 f eh1 k t ih0 v))
("effectively" nil (ih0 f eh1 k t ih0 v l iy0))
("effectiveness" nil (ih0 f eh1 k t ih0 v n ax s))
("effects" nil (ih0 f eh1 k t s))
("efferent" nil (eh1 f er0 ax n t))
("efferson" nil (eh1 f er0 s ax n))
("effertz" nil (eh1 f er0 t s))
("effervescent" nil (eh1 f er0 v eh1 s ax n t))
("effete" nil (eh0 f iy1 t))
("efficacious" nil (eh1 f ax k ey1 sh ax s))
("efficacy" nil (eh1 f ih0 k ae1 s iy0))
("efficiences" nil (eh0 f ih1 sh ax n s ih0 z))
("efficiencies" nil (ih0 f ih1 sh ax n s iy0 z))
("efficiency" nil (ih0 f ih1 sh ax n s iy0))
("efficient" nil (ih0 f ih1 sh ax n t))
("efficiently" nil (ih0 f ih1 sh ax n t l iy0))
("effie" nil (eh1 f iy0))
("effigy" nil (eh1 f ih0 jh iy0))
("effinger" nil (eh1 f ih0 ng er0))
("effler" nil (eh1 f l er0))
("effluent" nil (eh1 f l uw0 ax n t))
("efflux" nil (eh1 f l ax k s))
("effort" nil (eh1 f er0 t))
("effortless" nil (eh1 f er0 t l ax s))
("effortlessly" nil (eh1 f er0 t l ax s l iy0))
("efforts" nil (eh1 f er0 t s))
("effron" nil (eh1 f r ax n))
("effrontery" nil (ih0 f r ah1 n t er0 iy0))
("effusive" nil (eh1 f y uw0 s ih0 v))
("effusively" nil (ih0 f y uw1 s ih0 v l iy0))
("effy" nil (eh1 f iy0))
("efird" nil (eh1 f er0 d))
("efron" nil (eh1 f r ax n))
("efta" nil (eh1 f t ax))
("egalitarian" nil (ih0 g ae1 l ax t eh1 r iy0 ax n))
("egalitarianism" nil (iy0 g ae1 l ax t eh1 r iy0 ax n ih1 z ax m))
("egan" nil (iy1 g ax n))
("egbert" nil (eh1 g b er0 t))
("egberta" nil (ey0 g b eh1 r t ax))
("egbertina" nil (eh0 g b er0 t iy1 n ax))
("egbertine" nil (eh1 g b er0 t iy1 n))
("egberts" nil (eh1 g b er0 t s))
("ege" nil (iy1 jh))
("egeland" nil (eh1 g ih0 l ax n d))
("egeler" nil (eh1 g ax l er0))
("egelhoff" nil (eh1 g ih0 l hh ao0 f))
("egelston" nil (eh1 g ih0 l s t ax n))
("eger" nil (iy1 g er0))
("egerer" nil (eh1 g er0 er0))
("egert" nil (eh1 g er0 t))
("egerton" nil (eh1 g er0 t ax n))
("egg" nil (eh1 g))
("egge" nil (eh1 g))
("eggebrecht" nil (eh1 g ih0 b r ih0 k t))
("egged" nil (eh1 g d))
("eggemeyer" nil (eh1 g ih0 m ay1 er0))
("eggen" nil (eh1 g ax n))
("eggenberger" nil (eh1 g ax n b er0 g er0))
("egger" nil (eh1 g er0))
("eggers" nil (eh1 g er0 z))
("eggert" nil (eh1 g er0 t))
("egghead" nil (eh1 g hh eh1 d))
("eggleston" nil (eh1 g ax l s t ax n))
("eggleton" nil (eh1 g l ih0 t aa0 n))
("eggplant" nil (eh1 g p l ae1 n t))
("eggplants" nil (eh1 g p l ae1 n t s))
("eggs" nil (eh1 g z))
("eggshell" nil (eh1 g sh eh1 l))
("eggum" nil (eh1 g ax m))
("egipt" nil (iy1 jh ih0 p t))
("egland" nil (eh1 g l ax n d))
("eglantine" nil (eh1 g l ax n t ay1 n))
("egle" nil (eh1 g ax l))
("egler" nil (eh1 g l er0))
("egleston" nil (eh1 g l ih0 s t aa0 n))
("egley" nil (eh1 g l iy0))
("egli" nil (eh1 g l iy0))
("eglin" nil (eh1 g l ih0 n))
("eglise" nil (eh1 g l iy1 s))
("egloff" nil (eh1 g l ao0 f))
("egly" nil (eh1 g l iy0))
("egner" nil (eh1 g n er0))
("egnew" nil (ih0 g n uw1))
("egnor" nil (eh1 g n er0))
("ego" nil (iy1 g ow0))
("egocentric" nil (iy1 g ow0 s eh1 n t r ih0 k))
("egoism" nil (iy1 g ow0 ih1 z ax m))
("egolf" nil (eh1 g ow0 l f))
("egomaniac" nil (eh1 g ow0 m ey1 n iy0 ae0 k))
("egon" nil (iy1 g ax n))
("egos" nil (iy1 g ow0 z))
("egotism" nil (iy1 g ax t ih1 z ax m))
("egotist" nil (iy1 g ax t ih0 s t))
("egotistical" nil (eh1 g ax t ih1 s t ih0 k ax l))
("egregious" nil (ih0 g r iy1 jh ax s))
("egregiously" nil (ih0 g r iy1 jh ax s l iy0))
("egypt" nil (iy1 jh ax p t))
("egyptair" nil (iy1 jh ih0 p t eh1 r))
("egyptian" nil (ih0 jh ih1 p sh ax n))
("egyptians" nil (ih0 jh ih1 p sh ax n z))
("egyptology" nil (iy1 jh ax p t aa1 l ax jh iy0))
("eh" nil (eh1))
("ehinger" nil (eh1 hh ih0 n jh er0))
("ehle" nil (eh1 l))
("ehlen" nil (eh1 l ax n))
("ehler" nil (eh1 l er0))
("ehlers" nil (eh1 l er0 z))
("ehlert" nil (eh1 l er0 t))
("ehlinger" nil (eh1 l ih0 ng er0))
("ehlke" nil (eh1 l k))
("ehly" nil (eh1 l iy0))
("ehman" nil (eh1 m ax n))
("ehmann" nil (eh1 m ax n))
("ehmen" nil (eh1 m eh0 n))
("ehmke" nil (eh1 m k iy0))
("ehren" nil (eh1 r ax n))
("ehrenberg" nil (eh1 r ax n b er0 g))
("ehrenfeld" nil (eh1 r ih0 n f eh0 l d))
("ehrenhalt" nil (eh1 r ax n hh ao1 l t))
("ehrenkrantz" nil (eh1 r ax n k r ae1 n t s))
("ehrenreich" nil (eh1 r ih0 n r ay0 k))
("ehresman" nil (eh1 r ih0 s m ax n))
("ehret" nil (eh1 r ih0 t))
("ehrhard" nil (eh1 r hh er0 d))
("ehrhardt" nil (eh1 r hh aa0 r t))
("ehrhart" nil (eh1 r hh aa0 r t))
("ehrich" nil (eh1 r ih0 k))
("ehrig" nil (eh1 r ih0 g))
("ehrke" nil (eh1 r k))
("ehrler" nil (eh1 r l er0))
("ehrlich" nil (er1 l ih0 k))
("ehrlichman" nil (er1 l ih0 k m ax n))
("ehrman" nil (eh1 r m ax n))
("ehrmann" nil (eh1 r m ax n))
("ehrsam" nil (eh1 r s ax m))
("eiben" nil (ay1 b ax n))
("eich" nil (ay1 k))
("eichberg" nil (ay1 k b er0 g))
("eichel" nil (ay1 k ax l))
("eichelberger" nil (ay1 k ax l b er0 g er0))
("eichen" nil (ay1 k ax n))
("eichenbaum" nil (ay1 k ax n b aw1 m))
("eichenberg" nil (ay1 k ax n b er0 g))
("eichenberger" nil (ay1 k ax n b er0 g er0))
("eichenlaub" nil (ay1 k ih0 n l aw0 b))
("eicher" nil (ay1 k er0))
("eichholz" nil (ay1 k hh ow0 l z))
("eichhorn" nil (ay1 k hh er0 n))
("eichhorst" nil (ay1 k hh ao0 r s t))
("eichinger" nil (ay1 k ih0 n jh er0))
("eichler" nil (ay1 k ax l er0))
("eichman" nil (ay1 k m ax n))
("eichmann" nil (ay1 k m ax n))
("eichner" nil (ay1 k n er0))
("eicholtz" nil (ay1 k ow0 l t s))
("eichorn" nil (ay1 k ao0 r n))
("eichorst" nil (ay1 k hh ao0 r s t))
("eichstadt" nil (ay1 k s t ae0 t))
("eichstaedt" nil (ay1 k s t eh0 t))
("eick" nil (ay1 k))
("eickhoff" nil (ay1 k hh ao1 f))
("eickholt" nil (ay1 k hh ow1 l t))
("eickmeyer" nil (ay1 k m ay0 er0))
("eid" nil (ay1 d))
("eide" nil (ay1 d))
("eidem" nil (ay1 d ih0 m))
("eiden" nil (ay1 d ax n))
("eidson" nil (iy1 d s ax n))
("eiermann" nil (ay1 r m ax n))
("eifert" nil (ay1 f er0 t))
("eiffel" nil (ay1 f ax l))
("eifler" nil (ay1 f ax l er0))
("eigen" nil (ay1 g ax n))
("eighmey" nil (ey1 m iy0))
("eighmy" nil (ey1 g m iy0))
("eight" nil (ey1 t))
("eighteen" nil (ey0 t iy1 n))
("eighteens" nil (ey0 t iy1 n z))
("eighteenth" nil (ey0 t iy1 n th))
("eightfold" nil (ey1 t f ow1 l d))
("eighth" nil (ey1 t th))
("eighths" nil (ey1 t th s))
("eighties" nil (ey1 t iy0 z))
("eightieth" nil (ey1 t iy0 ih0 th))
("eights" nil (ey1 t s))
("eighty" nil (ey1 t iy0))
("eigner" nil (ay1 g n er0))
("eiichi" nil (ey0 iy1 ch iy0))
("eiji" nil (ey1 jh iy0))
("eike" nil (ay1 k))
("eikenberry" nil (iy1 k ax n b eh0 r iy0))
("eilan" nil (ay1 l ax n))
("eiland" nil (ay1 l ax n d))
("eileen" nil (ay0 l iy1 n))
("eiler" nil (ay1 l er0))
("eilerman" nil (ay1 l er0 m ax n))
("eilers" nil (ay1 l er0 z))
("eilert" nil (ay1 l er0 t))
("eilts" nil (ay1 l t s))
("eimer" nil (ay1 m er0))
("eimers" nil (ay1 m er0 z))
("ein" nil (ay1 n))
("einar" nil (ay1 n er0))
("einbender" nil (ay1 n b eh1 n d er0))
("eindhoven" nil (ay1 n d hh ow1 v ax n))
("einhorn" nil (ay1 n hh ao1 r n))
("einspahr" nil (ay1 n s p aa0 r))
("einstein" nil (ay1 n s t ay0 n))
("eir" nil (ay1 r))
("eirena" nil (er0 ey1 n ax))
("eirich" nil (ay1 r ih0 k))
("eis" nil (ay1 z))
("eisa" nil (ey1 s ax))
("eisai" nil (ay1 s ay1))
("eisaman" nil (ay1 s ax m ax n))
("eischeid" nil (ay1 sh ay0 d))
("eischen" nil (ay1 sh ax n))
("eischens" nil (ay1 sh ax n z))
("eisel" nil (ay1 s ax l))
("eisele" nil (ay1 s ax l))
("eiseman" nil (ay1 s m ax n))
("eisemann" nil (ay1 s m ax n))
("eisen" nil (ay1 s ax n))
("eisenach" nil (ay1 z ax n aa1 k))
("eisenbach" nil (ay1 z ax n b aa0 k))
("eisenbarth" nil (ay1 z ax n b aa0 r th))
("eisenbeis" nil (ay1 z ax n b ay0 s))
("eisenberg" nil (ay1 z ax n b er0 g))
("eisenberger" nil (ay1 z ax n b er0 g er0))
("eisenbraun" nil (ay1 z ax n b r aw0 n))
("eisenhardt" nil (ay1 z ax n hh aa0 r t))
("eisenhart" nil (ay1 z ax n hh aa0 r t))
("eisenhauer" nil (ay1 z ax n hh aw0 er0))
("eisenhour" nil (ay1 z ax n aw0 r))
("eisenhower" nil (ay1 z ax n hh aw1 er0))
("eisenhowers" nil (ay1 z ax n hh aw1 er0 z))
("eisenhut" nil (ay1 z ax n hh ax t))
("eisenhuth" nil (ay1 z ax n hh uw0 th))
("eisenman" nil (ay1 z ax n m ax n))
("eisenmann" nil (ay1 z ax n m ax n))
("eisenmenger" nil (ay1 z ax n m eh0 ng er0))
("eisenstadt" nil (ay1 z ax n s t ae0 t))
("eisenstein" nil (ay1 z ax n s t ay0 n))
("eiserman" nil (ay1 z er0 m ax n))
("eisert" nil (ay1 s er0 t))
("eishi" nil (ey1 sh iy0))
("eisinger" nil (ay1 s ih0 n jh er0))
("eisler" nil (ay1 s l er0))
("eisman" nil (ay1 s m ax n))
("eisner" nil (ay1 s n er0))
("eison" nil (ay1 z ax n))
("eissler" nil (ay1 s ax l er0))
("eiszner" nil (ay1 z n er0))
("eitel" nil (ay1 t ax l))
("either" nil (iy1 dh er0))
("eitzen" nil (ay1 t z ax n))
("eizenstat" nil (ay1 z ax n s t ae1 t))
("eject" nil (ih0 jh eh1 k t))
("ejected" nil (ih0 jh eh1 k t ih0 d))
("ejection" nil (ih0 jh eh1 k sh ax n))
("ejup" nil (iy1 jh ax p))
("ek" nil (eh1 k))
("eka" nil (eh1 k ax))
("ekaterina" nil (ey0 k aa0 t eh0 r iy1 n ax))
("ekberg" nil (eh1 k b er0 g))
("ekblad" nil (eh1 k b l ae1 d))
("ekco" nil (eh1 k ow0))
("ekdahl" nil (eh1 k d aa1 l))
("eke" nil (iy1 k))
("eked" nil (iy1 k t))
("ekern" nil (eh1 k er0 n))
("ekeus" nil (iy1 k uw1 s))
("ekholm" nil (eh1 k hh ow1 l m))
("eking" nil (iy1 k ih0 ng))
("ekins" nil (eh1 k ih0 n z))
("ekkehard" nil (eh1 k hh aa1 r d))
("eklund" nil (eh1 k l ax n d))
("ekman" nil (eh1 k m ax n))
("eko" nil (eh1 k ow0))
("ekofisk" nil (eh1 k ax f ih0 s k))
("eksportfinans" nil (eh1 k s p ao1 r t f ih0 n ax n z))
("ekstrand" nil (eh1 k s t r ax n d))
("ekstrom" nil (eh1 k s t r ax m))
("el" nil (eh1 l))
("ela" nil (eh1 l ax))
("elaborate" n (ih0 l ae1 b r ax t))
("elaborate" v (ih0 l ae1 b r ey0 t))
("elaborated" nil (ih0 l ae1 b er0 ey0 t ax d))
("elaborately" nil (ih0 l ae1 b r ax t l iy0))
("elaborates" nil (ih0 l ae1 b er0 ey1 t s))
("elaborating" nil (ih0 l ae1 b er0 ey1 t ih0 ng))
("elaboration" nil (ih0 l ae1 b er0 ey1 sh ax n))
("elaine" nil (ih0 l ey1 n))
("elam" nil (eh1 l ax m))
("elamin" nil (eh1 l ax m ih0 n))
("elan" nil (iy1 l ax n))
("eland" nil (iy1 l ax n d))
("elane" nil (ih0 l ey1 n))
("elapse" nil (ih0 l ae1 p s))
("elapsed" nil (ih0 l ae1 p s t))
("elardo" nil (eh0 l aa1 r d ow0))
("elastic" nil (ih0 l ae1 s t ih0 k))
("elasticity" nil (iy1 l ae1 s t ih1 s ax t iy0))
("elastomers" nil (ih0 l ae1 s t ax m er0 z))
("elata" nil (eh0 l aa1 t ax))
("elated" nil (ih0 l ey1 t ax d))
("elaters" nil (eh1 l ax t er0 z))
("elation" nil (ih0 l ey1 sh ax n))
("elayne" nil (ih0 l ey1 n))
("elbaum" nil (eh1 l b aw1 m))
("elbe" nil (eh1 l b))
("elberson" nil (eh1 l b er0 s ax n))
("elbert" nil (eh1 l b er0 t))
("elberta" nil (eh0 l b eh1 r t ax))
("elbertine" nil (eh1 l b er0 t iy1 n))
("elbow" nil (eh1 l b ow1))
("elbowed" nil (eh1 l b ow1 d))
("elbowing" nil (eh1 l b ow1 ih0 ng))
("elbowroom" nil (eh1 l b ow1 r uw1 m))
("elbows" nil (eh1 l b ow1 z))
("elbrus" nil (eh1 l b r ax s))
("elco" nil (eh1 l k ow0))
("elcock" nil (ih0 l k aa1 k))
("elcor" nil (eh1 l k ao1 r))
("elcotel" nil (eh1 l k ow0 t eh1 l))
("elden" nil (eh1 l d ax n))
("elder" nil (eh1 l d er0))
("elderkin" nil (eh1 l d er0 k ih0 n))
("elderly" nil (eh1 l d er0 l iy0))
("elders" nil (eh1 l d er0 z))
("eldest" nil (eh1 l d ax s t))
("eldin" nil (eh1 l d ih0 n))
("eldon" nil (ih0 l d aa1 n))
("eldora" nil (eh0 l d ao1 r ax))
("eldorado" nil (eh1 l d er0 aa1 d ow0))
("eldred" nil (eh1 l d er0 d))
("eldredge" nil (ih0 l d r eh1 jh))
("eldreth" nil (ih0 l d r eh1 th))
("eldric" nil (eh1 l d r ih0 k))
("eldrida" nil (eh0 l d r iy1 d ax))
("eldridge" nil (eh1 l d r ih1 jh))
("eldritch" nil (eh1 l d r ih0 ch))
("eldwin" nil (ih0 l d w ih1 n))
("eleanor" nil (eh1 l ax n ao0 r))
("eleanora" nil (eh1 l ax n ao1 r ax))
("eleanore" nil (eh1 l ax n ao0 r))
("eleazer" nil (eh1 l ax z er0))
("elect" nil (ih0 l eh1 k t))
("electability" nil (ih0 l eh1 k t ax b ih1 l ax t iy0))
("electable" nil (ih0 l eh1 k t ax b ax l))
("elected" nil (ih0 l eh1 k t ax d))
("electing" nil (ih0 l eh1 k t ih0 ng))
("election" nil (ih0 l eh1 k sh ax n))
("electioneering" nil (ih0 l eh1 k sh ax n ih1 r ih0 ng))
("elections" nil (ih0 l eh1 k sh ax n z))
("elective" nil (ih0 l eh1 k t ih0 v))
("electives" nil (ih0 l eh1 k t ih0 v z))
("electoral" nil (ih0 l eh1 k t er0 ax l))
("electorate" nil (ih0 l eh1 k t er0 ax t))
("electorates" nil (ih0 l eh1 k t er0 ax t s))
("electors" nil (ih0 l eh1 k t er0 z))
("electra" nil (ih0 l eh1 k t r ax))
("electric" nil (ih0 l eh1 k t r ih0 k))
("electrical" nil (ih0 l eh1 k t r ih0 k ax l))
("electrically" nil (ih0 l eh1 k t r ih0 k ax l iy0))
("electricals" nil (ih0 l eh1 k t r ih0 k ax l z))
("electricar" nil (ih0 l eh1 k t r ih0 k aa1 r))
("electrician" nil (ih0 l eh0 k t r ih1 sh ax n))
("electricians" nil (ih0 l eh0 k t r ih1 sh ax n z))
("electricite" nil (ax l eh1 k t r ih1 s ih1 t ey0))
("electricity" nil (ih0 l eh1 k t r ih1 s ax t iy0))
("electrification" nil (ih0 l eh1 k t r ax f ih0 k ey1 sh ax n))
("electrified" nil (ih0 l eh1 k t r ax f ay1 d))
("electrify" nil (ih0 l eh1 k t r ax f ay1))
("electrifying" nil (ih0 l eh1 k t r ax f ay1 ih0 ng))
("electrique" nil (eh1 l eh0 k t r iy1 k))
("electro" nil (ih0 l eh1 k t r ow0))
("electrobiology" nil (ih0 l eh1 k t r ow0 b ay0 aa1 l ax jh iy0))
("electrocardiogram" nil (ih0 l eh1 k t r ow0 k aa1 r d iy0 ax g r ae1 m))
("electrocardiograms" nil (ih0 l eh1 k t r ow0 k aa1 r d iy0 ax g r ae1 m z))
("electrochemical" nil (ax l eh1 k t r ow0 k eh1 m ih0 k ax l))
("electrocom" nil (ih0 l eh1 k t r ow0 k aa1 m))
("electrocute" nil (ih0 l eh1 k t r ax k y uw1 t))
("electrocuted" nil (ih0 l eh1 k t r ax k y uw1 t ih0 d))
("electrocution" nil (ih0 l eh1 k t r ax k y uw1 sh ax n))
("electrocutions" nil (ax l eh1 k t r ax k y uw1 sh ax n z))
("electrode" nil (ih0 l eh1 k t r ow0 d))
("electrodes" nil (ih0 l eh1 k t r ow0 d z))
("electrodynamics" nil (ih0 l eh1 k t r ow0 d ay1 n ae1 m ih0 k s))
("electrolux" nil (ih0 l eh1 k t r ax l ax k s))
("electrolysis" nil (ih0 l eh1 k t r aa1 l ax s ax s))
("electrolytic" nil (ih0 l eh1 k t r ax l ih1 t ih0 k))
("electromagnet" nil (ih0 l eh1 k t r ow0 m ae1 g n ax t))
("electromagnetic" nil (ih0 l eh1 k t r ow0 m ae0 g n eh1 t ih0 k))
("electromagnetism" nil (ih0 l eh1 k t r ow0 m ae1 g n ax t ih1 z ax m))
("electromagnets" nil (ih0 l eh1 k t r ow0 m ae1 g n ax t s))
("electromechanical" nil (ih0 l eh1 k t r ow0 m ax k ae1 n ih0 k ax l))
("electromedics" nil (ih0 l eh1 k t r ow0 m eh1 d ih0 k s))
("electron" nil (ih0 l eh1 k t r aa0 n))
("electronic" nil (ih0 l eh1 k t r aa1 n ih0 k))
("electronically" nil (ih0 l eh1 k t r aa1 n ih0 k ax l iy0))
("electronics" nil (ih0 l eh1 k t r aa1 n ih0 k s))
("electrons" nil (ih0 l eh1 k t r aa0 n z))
("electroplating" nil (ax l eh1 k t r ax p l ey1 t ih0 ng))
("electroshock" nil (ih0 l eh1 k t r ow1 sh aa1 k))
("electrosound" nil (ih0 l eh1 k t r ow0 s aw1 n d))
("electrospace" nil (ih0 l eh1 k t r ow0 s p ey1 s))
("electrostatic" nil (ih0 l eh1 k t r ow0 s t ae1 t ih0 k))
("elects" nil (ih0 l eh1 k t s))
("eledge" nil (eh1 l ih0 jh))
("eleen" nil (eh1 l iy0 n))
("elefante" nil (eh0 l eh0 f aa1 n t iy0))
("elegance" nil (eh1 l ax g ax n s))
("elegant" nil (eh1 l ax g ax n t))
("elegantly" nil (eh1 l ih0 g ax n t l iy0))
("elek" nil (eh1 l ih0 k))
("elektra" nil (eh0 l eh1 k t r ax))
("elektrisk" nil (ih0 l eh1 k t r ih1 s k))
("elektrizitaetswerk" nil (eh1 l ih0 k t r ih1 z ih0 t ae1 t s w er0 k))
("element" nil (eh1 l ax m ax n t))
("elemental" nil (eh1 l ax m eh1 n t ax l))
("elementary" nil (eh1 l ax m eh1 n t r iy0))
("elements" nil (eh1 l ax m ax n t s))
("elena" nil (eh1 l ax n ax))
("elenbaas" nil (eh1 l ih0 n b aa0 z))
("elene" nil (eh1 l iy0 n))
("elenore" nil (eh1 l ih0 n er0))
("eleonore" nil (eh0 l iy0 ax n ao1 r iy0))
("elephant" nil (eh1 l ax f ax n t))
("elephantine" nil (eh1 l ax f ae1 n t iy1 n))
("elephants" nil (eh1 l ax f ax n t s))
("eletr" nil (eh1 l ax t er0))
("elevate" nil (eh1 l ax v ey1 t))
("elevated" nil (eh1 l ax v ey1 t ih0 d))
("elevates" nil (eh1 l ax v ey1 t s))
("elevating" nil (eh1 l ax v ey1 t ih0 ng))
("elevation" nil (eh1 l ax v ey1 sh ax n))
("elevations" nil (eh1 l ax v ey1 sh ax n z))
("elevator" nil (eh1 l ax v ey1 t er0))
("elevators" nil (eh1 l ax v ey1 t er0 z))
("eleven" nil (ih0 l eh1 v ax n))
("elevens" nil (ih0 l eh1 v ax n z))
("eleventh" nil (ih0 l eh1 v ax n th))
("elexis" nil (eh0 l eh1 k s ih0 s))
("eley" nil (iy1 l iy0))
("elf" nil (eh1 l f))
("elfers" nil (eh1 l f er0 z))
("elfie" nil (eh1 l f iy0))
("elfin" nil (eh1 l f ih0 n))
("elfman" nil (eh1 l f m ax n))
("elford" nil (eh1 l f er0 d))
("elfreda" nil (eh0 l f r eh1 d ax))
("elfrida" nil (eh0 l f r iy1 d ax))
("elfrieda" nil (eh0 l f r iy1 d ax))
("elfrink" nil (eh1 l f r ih0 ng k))
("elfstrom" nil (eh1 l f s t r ax m))
("elg" nil (eh1 l g))
("elga" nil (ih0 l g aa1))
("elgar" nil (eh1 l g er0))
("elgart" nil (ey1 l g aa0 r t))
("elger" nil (eh1 l g er0))
("elgersma" nil (eh0 l jh eh1 r s m ax))
("elgie" nil (eh1 l jh iy0))
("elgin" nil (eh1 l jh ih0 n))
("eli" nil (iy1 l ay0))
("elia" nil (ax l ay1 ax))
("eliades" nil (iy1 l iy1 ey0 d z))
("elias" nil (ax l ay1 ax s))
("eliason" nil (ax l ay1 ax s ax n))
("eliassen" nil (ax l ay1 ax s ax n))
("elich" nil (eh1 l ih0 k))
("elicit" nil (ih0 l ih1 s ih0 t))
("elicited" nil (ih0 l ih1 s ih0 t ih0 d))
("eliciting" nil (ih0 l ih1 s ax t ih0 ng))
("elicits" nil (ih0 l ih1 s ax t s))
("elick" nil (eh1 l ih0 k))
("elicker" nil (eh1 l ih0 k er0))
("elie" nil (eh1 l iy0))
("eligibility" nil (eh1 l ih0 jh ax b ih1 l ih0 t iy0))
("eligible" nil (eh1 l ax jh ax b ax l))
("elihu" nil (eh1 l ih0 hh uw0))
("elijah" nil (eh0 l ay1 jh ax))
("eliminate" nil (ih0 l ih1 m ax n ey1 t))
("eliminated" nil (ih0 l ih1 m ax n ey1 t ax d))
("eliminates" nil (ih0 l ih1 m ax n ey1 t s))
("eliminating" nil (ih0 l ih1 m ax n ey1 t ih0 ng))
("elimination" nil (ih0 l ih1 m ax n ey1 sh ax n))
("eliminations" nil (ih0 l ih1 m ih0 n ey1 sh ax n z))
("eline" nil (eh1 l ay0 n))
("elinor" nil (eh1 l ih0 n er0))
("elinore" nil (eh0 l iy0 n ao1 r iy0))
("elio" nil (eh1 l iy0 ow0))
("eliopoulos" nil (eh0 l iy0 aa1 p ax l ih0 s))
("eliot" nil (eh1 l iy0 ax t))
("eliott" nil (eh1 l iy0 aa0 t))
("elisa" nil (ax l iy1 s ax))
("elisabeth" nil (ih0 l ih1 z ax b ih0 th))
("elise" nil (ax l iy1 s))
("elish" nil (eh1 l ih0 sh))
("elisha" nil (eh1 l ih0 sh ax))
("elison" nil (eh1 l ih0 s ax n))
("elissa" nil (eh0 l iy1 s ax))
("elite" nil (ih0 l iy1 t))
("elites" nil (ih0 l iy1 t s))
("elitism" nil (eh1 l ih0 t ih1 z ax m))
("elitist" nil (ey0 l iy1 t ih0 s t))
("elitists" nil (ey0 l iy1 t ih0 s t s))
("elixir" nil (ih0 l ih1 k s er0))
("eliza" nil (ih0 l ay1 z ax))
("elizabeth" nil (ih0 l ih1 z ax b ax th))
("elizabethan" nil (eh1 l ih0 z ae1 b ax th ax n))
("elizabethtown" nil (ax l ih1 z ax b eh0 th t aw1 n))
("elizalde" nil (eh0 l iy0 z aa1 l d iy0))
("elizondo" nil (eh1 l ih0 z aa1 n d ow0))
("eljer" nil (eh1 l jh er0))
("elk" nil (eh1 l k))
("elkes" nil (eh1 l k s))
("elkhart" nil (eh1 l k hh aa1 r t))
("elkhorn" nil (eh1 l k hh ao1 r n))
("elkin" nil (ih0 l k ih1 n))
("elkind" nil (ih0 l k ay1 n d))
("elkington" nil (eh1 l k ih0 ng t ax n))
("elkins" nil (eh1 l k ih0 n z))
("elko" nil (eh1 l k ow0))
("elks" nil (eh1 l k s))
("elkton" nil (eh1 l k t ax n))
("ell" nil (eh1 l))
("ella" nil (eh1 l ax))
("ellamay" nil (eh1 l ax m ey1))
("ellan" nil (eh1 l ax n))
("ellard" nil (eh1 l er0 d))
("elle" nil (eh1 l))
("elledge" nil (eh1 l ih0 jh))
("ellefson" nil (eh1 l ih0 f s ax n))
("ellegood" nil (eh1 l ih0 g uh0 d))
("ellemann" nil (eh1 l ax m ax n))
("ellen" nil (eh1 l ax n))
("ellena" nil (eh0 l eh1 n ax))
("ellenbecker" nil (eh1 l ih0 n b eh0 k er0))
("ellenberg" nil (eh1 l ax n b er0 g))
("ellenberger" nil (eh1 l ax n b er0 g er0))
("ellenbogen" nil (eh1 l ih0 n b ax g ax n))
("ellenburg" nil (eh1 l ax n b er0 g))
("ellender" nil (eh1 l eh0 n d er0))
("ellene" nil (eh1 l iy1 n))
("ellenson" nil (eh1 l ih0 n s ax n))
("ellenwood" nil (eh1 l ax n w uh1 d))
("eller" nil (eh1 l er0))
("ellerbe" nil (eh1 l er0 b))
("ellerbee" nil (ih0 l er1 b iy0))
("ellerbrock" nil (ih0 l er1 b r ax k))
("ellerey" nil (eh1 l er0 iy0))
("ellerman" nil (eh1 l er0 m ax n))
("ellers" nil (eh1 l er0 z))
("ellert" nil (eh1 l er0 t))
("ellertson" nil (eh1 l er0 t s ax n))
("ellery" nil (eh1 l er0 iy0))
("ellesmere" nil (eh1 l z m ih1 r))
("ellesse" nil (eh0 l eh1 s))
("ellestad" nil (eh1 l ih0 s t ax d))
("ellett" nil (eh1 l ih0 t))
("ellette" nil (ih0 l eh1 t))
("elley" nil (eh1 l iy0))
("ellice" nil (eh1 l ih0 s))
("ellicott" nil (eh1 l ih0 k aa0 t))
("ellie" nil (eh1 l iy0))
("elliff" nil (eh1 l ih0 f))
("elliman" nil (eh1 l ih0 m ax n))
("elling" nil (eh1 l ih0 ng))
("ellinger" nil (eh1 l ih0 ng er0))
("ellingsen" nil (eh1 l ih0 ng s ax n))
("ellingson" nil (eh1 l ih0 ng s ax n))
("ellingsworth" nil (eh1 l ih0 ng z w er1 th))
("ellington" nil (eh1 l ih0 ng t ax n))
("ellingwood" nil (eh1 l ih0 ng w uh1 d))
("ellinwood" nil (eh1 l ih0 n w uh1 d))
("elliot" nil (eh1 l iy0 ax t))
("elliott" nil (eh1 l iy0 ax t))
("ellipse" nil (ih0 l ih1 p s))
("ellipsoids" nil (ih0 l ih1 p s oy0 d z))
("elliptical" nil (ih0 l ih1 p t ih0 k ax l))
("ellis" nil (eh1 l ih0 s))
("ellison" nil (eh1 l ih0 s ax n))
("ellisor" nil (eh1 l ih0 s er0))
("elliston" nil (eh1 l ih0 s t aa0 n))
("ellithorpe" nil (eh1 l ih0 th er0 p))
("ellman" nil (eh1 l m ax n))
("ellmann" nil (eh1 l m ax n))
("ellner" nil (eh1 l n er0))
("ells" nil (eh1 l z))
("ellsberg" nil (eh1 l z b er0 g))
("ellsworth" nil (eh1 l z w er0 th))
("ellwanger" nil (eh1 l w ao0 ng er0))
("ellwood" nil (eh1 l w uh1 d))
("elly" nil (eh1 l iy0))
("ellyn" nil (eh1 l ih0 n))
("ellyson" nil (eh1 l ih0 s ax n))
("ellzey" nil (eh1 l z iy0))
("elm" nil (eh1 l m))
("elma" nil (eh1 l m ax))
("elman" nil (eh1 l m ax n))
("elmendorf" nil (eh1 l m ih0 n d ao0 r f))
("elmer" nil (eh1 l m er0))
("elmes" nil (eh1 l m z))
("elmhurst" nil (eh1 l m hh er0 s t))
("elmira" nil (eh0 l m ay1 r ax))
("elmo" nil (eh1 l m ow0))
("elmootazbell" nil (eh0 l m uw1 t ax z b eh1 l))
("elmore" nil (eh1 l m ao0 r))
("elmquist" nil (eh1 l m k w ih1 s t))
("elms" nil (eh1 l m z))
("elmsford" nil (eh1 l m z f er0 d))
("elmwood" nil (eh1 l m w uh1 d))
("elna" nil (ih0 l n aa1))
("elnora" nil (eh0 l n ao1 r ax))
("elnore" nil (ih0 l n ao1 r))
("elnozahy" nil (eh1 l n ow1 z aa1 hh iy0))
("eloisa" nil (eh0 l oy1 s ax))
("eloise" nil (ih0 l oy1 z))
("elongate" nil (ih0 l ao1 ng g ey0 t))
("elongated" nil (ih0 l ao1 ng g ey0 t ax d))
("elongation" nil (iy1 l ao0 ng g ey1 sh ax n))
("elopes" nil (ih0 l ow1 p s))
("eloquence" nil (eh1 l ax k w ax n s))
("eloquent" nil (eh1 l ax k w ax n t))
("eloquently" nil (eh1 l ax k w ax n t l iy0))
("elpers" nil (eh1 l p er0 z))
("elrica" nil (eh1 l r ih0 k ax))
("elrick" nil (eh1 l r ih0 k))
("elrod" nil (ih0 l r aa1 d))
("elron" nil (eh1 l r ax n))
("elroy" nil (ih0 l r oy1))
("els" nil (eh1 l z))
("elsa" nil (eh1 l s ax))
("elsas" nil (eh1 l s ax z))
("elsasser" nil (eh1 l s ax s er0))
("elsberry" nil (eh1 l z b eh1 r iy0))
("elsbury" nil (eh1 l z b eh1 r iy0))
("elsdon" nil (eh1 l s d ax n))
("else" nil (eh1 l s))
("elsea" nil (eh1 l s iy0 ax))
("elsen" nil (eh1 l s ax n))
("elser" nil (eh1 l s er0))
("elses" nil (eh1 l s ih0 z))
("elsesser" nil (eh1 l s ih0 s er0))
("elsevier" nil (eh0 l s eh1 v y er0))
("elsewhere" nil (eh1 l s w eh1 r))
("elsey" nil (eh1 l s iy0))
("elsie" nil (eh1 l s iy0))
("elsinore" nil (eh1 l s ax n ao1 r))
("elsner" nil (eh1 l s n er0))
("elson" nil (eh1 l s ax n))
("elstad" nil (eh1 l s t ax d))
("elster" nil (eh1 l s t er0))
("elston" nil (ih0 l s t aa1 n))
("elswick" nil (eh1 l z w ih1 k))
("elsworth" nil (eh1 l z w er1 th))
("elting" nil (eh1 l t ih0 ng))
("elton" nil (eh1 l t ax n))
("eltringham" nil (eh1 l t r ih0 ng ae0 m))
("eltzroth" nil (eh1 l t s r ao0 th))
("elucidated" nil (ih0 l uw1 s ax d ey1 t ax d))
("elude" nil (ih0 l uw1 d))
("eluded" nil (ih0 l uw1 d ih0 d))
("eludes" nil (ih0 l uw1 d z))
("eluding" nil (ih0 l uw1 d ih0 ng))
("elusive" nil (ih0 l uw1 s ih0 v))
("elusiveness" nil (ih0 l uw1 s ih0 v n ax s))
("elva" nil (eh1 l v ax))
("elvera" nil (ey0 l v eh1 r ax))
("elvers" nil (eh1 l v er0 z))
("elves" nil (eh1 l v z))
("elvgren" nil (eh1 l v g r eh0 n))
("elvia" nil (eh1 l v iy0 ax))
("elvie" nil (eh1 l v iy0))
("elvin" nil (eh1 l v ih0 n))
("elvina" nil (eh0 l v iy1 n ax))
("elvington" nil (eh1 l v ih0 ng t ax n))
("elvira" nil (eh0 l v ay1 r ax))
("elvire" nil (eh1 l v ay1 r))
("elvis" nil (eh1 l v ih0 s))
("elvy" nil (eh1 l v iy0))
("elway" nil (eh1 l w ey1))
("elwell" nil (ih0 l w eh1 l))
("elwin" nil (eh1 l w ih0 n))
("elwood" nil (eh1 l w uh1 d))
("elxsi" nil (eh1 l k s iy0))
("ely" nil (iy1 l ay0))
("elyn" nil (eh1 l ih0 n))
("elyria" nil (ih0 l ih1 r iy0 ax))
("elyse" nil (eh1 l ay0 s))
("elysee" nil (eh1 l ih0 s iy1))
("elysees" nil (eh1 l ih0 s iy1 z))
("elysia" nil (ih0 l ih1 zh iy0 ax))
("elysium" nil (ih0 l ih1 z iy0 ax m))
("elza" nil (eh1 l z ax))
("elzey" nil (eh1 l z iy0))
("elzinga" nil (eh0 l z iy1 ng g ax))
("elzy" nil (eh1 l z iy0))
("em" nil (eh1 m))
("ema" nil (iy1 m ax))
("emaciated" nil (ih0 m ey1 sh iy0 ey1 t ih0 d))
("email" nil (ax m ey1 l))
("emailed" nil (ax m ey1 l d))
("emailing" nil (ax m ey1 l ih0 ng))
("emails" nil (ax m ey1 l z))
("emanate" nil (eh1 m ax n ey1 t))
("emanated" nil (eh1 m ax n ey1 t ih0 d))
("emanating" nil (eh1 m ax n ey1 t ih0 ng))
("emanations" nil (eh1 m ax n ey1 sh ax n z))
("emancipate" nil (ih0 m ae1 n s ax p ey1 t))
("emancipated" nil (ih0 m ae1 n s ax p ey1 t ih0 d))
("emancipation" nil (ih0 m ae1 n s ax p ey1 sh ax n))
("emanuel" nil (ih0 m ae1 n y uw0 ax l))
("emanuele" nil (ey0 m aa0 n uw0 eh1 l ey0))
("emanuelson" nil (ih0 m ae1 n uw0 l s ax n))
("emard" nil (eh1 m er0 d))
("emasculate" nil (ax m ae1 s k y uw0 l ih0 t))
("emasculated" nil (ax m ae1 s k y uw0 l ey1 t ih0 d))
("embalmed" nil (eh0 m b aa1 m d))
("embankment" nil (eh0 m b ae1 ng k m ax n t))
("embarcadero" nil (eh0 m b aa1 r k ax d eh1 r ow0))
("embargo" nil (eh0 m b aa1 r g ow0))
("embargoed" nil (ih0 m b aa1 r g ow0 d))
("embargoes" nil (eh0 m b aa1 r g ow0 z))
("embark" nil (eh0 m b aa1 r k))
("embarkation" nil (eh1 m b aa0 r k ey1 sh ax n))
("embarked" nil (eh0 m b aa1 r k t))
("embarking" nil (eh0 m b aa1 r k ih0 ng))
("embarks" nil (ih0 m b aa1 r k s))
("embarrass" nil (ih0 m b eh1 r ax s))
("embarrassed" nil (ih0 m b eh1 r ax s t))
("embarrasses" nil (ih0 m b ae1 r ax s ih0 z))
("embarrassing" nil (ih0 m b eh1 r ax s ih0 ng))
("embarrassingly" nil (ih0 m b eh1 r ax s ih0 ng l iy0))
("embarrassment" nil (ih0 m b eh1 r ax s m ax n t))
("embarrassments" nil (ih0 m b eh1 r ax s m ax n t s))
("embassies" nil (eh1 m b ax s iy0 z))
("embassy" nil (eh1 m b ax s iy0))
("embattled" nil (eh0 m b ae1 t ax l d))
("embayment" nil (eh0 m b ey1 m ax n t))
("embed" nil (ih0 m b eh1 d))
("embedded" nil (eh0 m b eh1 d ih0 d))
("embellish" nil (ih0 m b eh1 l ih0 sh))
("embellished" nil (eh0 m b eh1 l ih0 sh t))
("embellishment" nil (eh0 m b eh1 l ih0 sh m ax n t))
("embers" nil (eh1 m b er0 z))
("emberson" nil (eh1 m b er0 s ax n))
("emberton" nil (ih0 m b er1 t ax n))
("embezzle" nil (ih0 m b eh1 z ax l))
("embezzled" nil (ih0 m b eh1 z ax l d))
("embezzlement" nil (eh0 m b eh1 z ax l m ax n t))
("embezzler" nil (ih0 m b eh1 z ax l er0))
("embezzlers" nil (ih0 m b eh1 z ax l er0 z))
("embezzles" nil (ih0 m b eh1 z ax l z))
("embezzling" nil (ih0 m b eh1 z ax l ih0 ng))
("embittered" nil (eh0 m b ih1 t er0 d))
("emblazoned" nil (eh0 m b l ey1 z ax n d))
("emblem" nil (eh1 m b l ax m))
("emblematic" nil (eh1 m b l ax m ae1 t ih0 k))
("emblems" nil (eh1 m b l ax m z))
("embler" nil (eh1 m b l er0))
("embleton" nil (eh1 m b l ih0 t aa0 n))
("embodied" nil (ih0 m b aa1 d iy0 d))
("embodies" nil (eh0 m b aa1 d iy0 z))
("embodiment" nil (eh0 m b aa1 d iy0 m ax n t))
("embody" nil (ih0 m b aa1 d iy0))
("embodying" nil (ih0 m b aa1 d iy0 ih0 ng))
("embolden" nil (eh0 m b ow1 l d ax n))
("emboldened" nil (eh0 m b ow1 l d ax n d))
("embolism" nil (eh1 m b ow0 l ih1 z ax m))
("embolisms" nil (eh1 m b ow0 l ih1 z ax m z))
("embossed" nil (ih0 m b ao1 s t))
("embrace" nil (eh0 m b r ey1 s))
("embraced" nil (eh0 m b r ey1 s t))
("embraces" nil (eh0 m b r ey1 s ih0 z))
("embracing" nil (eh0 m b r ey1 s ih0 ng))
("embree" nil (ih0 m b r iy1))
("embrey" nil (eh1 m b r iy0))
("embroider" nil (ih0 m b r oy1 d er0))
("embroidered" nil (eh0 m b r oy1 d er0 d))
("embroiderers" nil (eh0 m b r oy1 d er0 er0 z))
("embroideries" nil (ih0 m b r oy1 d er0 iy0 z))
("embroidering" nil (eh0 m b r oy1 d er0 ih0 ng))
("embroidery" nil (eh0 m b r oy1 d er0 iy0))
("embroil" nil (eh0 m b r oy1 l))
("embroiled" nil (eh0 m b r oy1 l d))
("embry" nil (eh1 m b r iy0))
("embryo" nil (eh1 m b r iy0 ow1))
("embryology" nil (eh1 m b r iy0 aa1 l ax jh iy0))
("embryonic" nil (eh1 m b r iy0 aa1 n ih0 k))
("embryos" nil (eh1 m b r iy0 ow1 z))
("embury" nil (eh1 m b eh1 r iy0))
("emcee" nil (eh1 m s iy1))
("emch" nil (eh1 m ch))
("emco" nil (eh1 m k ow0))
("emde" nil (eh1 m d))
("emel" nil (eh1 m ax l))
("emelda" nil (eh0 m eh1 l d ax))
("emelie" nil (eh1 m ax l iy0))
("emelina" nil (eh1 m eh0 l iy1 n ax))
("emeline" nil (eh1 m ih0 l ay1 n))
("emelita" nil (eh0 m eh0 l iy1 t ax))
("emelyne" nil (eh1 m ih0 l ay0 n))
("emens" nil (eh1 m eh0 n z))
("emera" nil (eh0 m eh1 r ax))
("emerald" nil (eh1 m r ax l d))
("emeralds" nil (eh1 m r ax l d z))
("emerant" nil (ey0 m eh1 r ax n t))
("emeraude" nil (eh1 m er0 aw1 d))
("emerge" nil (ih0 m er1 jh))
("emerged" nil (ih0 m er1 jh d))
("emergence" nil (ih0 m er1 jh ax n s))
("emergencies" nil (ih0 m er1 jh ax n s iy0 z))
("emergency" nil (ih0 m er1 jh ax n s iy0))
("emergent" nil (ih0 m er1 jh ax n t))
("emerges" nil (ih0 m er1 jh ih0 z))
("emerging" nil (ih0 m er1 jh ih0 ng))
("emerich" nil (eh1 m er0 ih0 k))
("emerick" nil (eh1 m er0 ih0 k))
("emerine" nil (eh1 m er0 iy1 n))
("emeritus" nil (ih0 m eh1 r ax t ax s))
("emersed" nil (ih0 m er1 s t))
("emerson" nil (eh1 m er0 s ax n))
("emert" nil (eh1 m er0 t))
("emerton" nil (ih0 m er1 t ax n))
("emery" nil (eh1 m er0 iy0))
("emeryville" nil (eh1 m er0 iy0 v ih1 l))
("emetic" nil (ih0 m eh1 t ih0 k))
("emfinger" nil (eh1 m f ih0 ng er0))
("emge" nil (eh1 m jh))
("emhart" nil (eh1 m hh aa1 r t))
("emick" nil (eh1 m ih0 k))
("emig" nil (eh1 m ih0 g))
("emigh" nil (eh1 m ay0))
("emigrant" nil (eh1 m ax g r ax n t))
("emigrants" nil (eh1 m ax g r ax n t s))
("emigrate" nil (eh1 m ax g r ey1 t))
("emigrated" nil (eh1 m ax g r ey1 t ih0 d))
("emigrating" nil (eh1 m ax g r ey1 t ih0 ng))
("emigration" nil (eh1 m ax g r ey1 sh ax n))
("emigrations" nil (eh1 m ax g r ey1 sh ax n z))
("emigre" nil (eh1 m ax g r ey1))
("emigres" nil (eh1 m ax g r ey1 z))
("emil" nil (eh0 m iy1 l))
("emile" nil (ey0 m iy1 l))
("emilia" nil (ax m iy1 l iy0 ax))
("emilie" nil (eh1 m ax l iy0))
("emilio" nil (ax m iy1 l iy0 ow0))
("emily" nil (eh1 m ih0 l iy0))
("emina" nil (eh0 m iy1 n ax))
("eminase" nil (eh1 m ih0 n aa1 s iy0))
("eminence" nil (eh1 m ax n ax n s))
("eminences" nil (eh1 m ax n ax n s ih0 z))
("eminent" nil (eh1 m ax n ax n t))
("eminently" nil (eh1 m ax n ax n t l iy0))
("emir" nil (ih0 m ih1 r))
("emirate" nil (eh1 m er0 ax t))
("emirates" nil (eh1 m er0 ax t s))
("emison" nil (eh1 m ih0 s ax n))
("emissaries" nil (eh1 m ax s eh1 r iy0 z))
("emissary" nil (eh1 m ax s eh1 r iy0))
("emission" nil (ih0 m ih1 sh ax n))
("emissions" nil (ih0 m ih1 sh ax n z))
("emit" nil (ih0 m ih1 t))
("emits" nil (ih0 m ih1 t s))
("emitted" nil (ih0 m ih1 t ax d))
("emitting" nil (ih0 m ih1 t ih0 ng))
("emler" nil (eh1 m l er0))
("emley" nil (eh1 m l iy0))
("emling" nil (eh1 m l ih0 ng))
("emlyn" nil (ih0 m l ih1 n))
("emlynne" nil (ih0 m l ay1 n))
("emma" nil (eh1 m ax))
("emmaline" nil (eh0 m aa0 l iy1 n iy0))
("emmanuel" nil (ih0 m ae1 n y uw0 ax l))
("emmaus" nil (eh1 m aw0 s))
("emme" nil (eh1 m))
("emmel" nil (eh1 m ax l))
("emmeline" nil (eh1 m ax l ay1 n))
("emmendorfer" nil (eh1 m ih0 n d ao0 r f er0))
("emmer" nil (eh1 m er0))
("emmerich" nil (eh1 m er0 ih0 k))
("emmerling" nil (eh1 m er0 l ih0 ng))
("emmerson" nil (eh1 m er0 s ax n))
("emmert" nil (eh1 m er0 t))
("emmery" nil (eh1 m er0 iy0))
("emmet" nil (eh1 m ih0 t))
("emmett" nil (eh1 m ih0 t))
("emmi" nil (eh1 m iy0))
("emmick" nil (eh1 m ih0 k))
("emmie" nil (eh1 m iy0))
("emminger" nil (eh1 m ih0 ng er0))
("emmis" nil (eh1 m ih0 s))
("emmit" nil (eh1 m ih0 t))
("emmitt" nil (eh1 m ih0 t))
("emmons" nil (eh1 m ax n z))
("emmott" nil (eh1 m ax t))
("emmy" nil (eh1 m iy0))
("emmys" nil (eh1 m iy0 z))
("emogene" nil (eh1 m ax g iy0 n))
("emond" nil (eh1 m ax n d))
("emory" nil (eh1 m er0 iy0))
("emotion" nil (ih0 m ow1 sh ax n))
("emotional" nil (ih0 m ow1 sh ax n ax l))
("emotionalism" nil (ih0 m ow1 sh ax n ax l ih1 z ax m))
("emotionally" nil (ih0 m ow1 sh n ax l iy0))
("emotions" nil (ih0 m ow1 sh ax n z))
("empanel" nil (ih0 m p ae1 n ax l))
("empaneled" nil (ih0 m p ae1 n ax l d))
("empaneling" nil (eh0 m p ae1 n ax l ih0 ng))
("empathetic" nil (eh1 m p ax th eh1 t ih0 k))
("empathize" nil (eh1 m p ax th ay1 z))
("empathy" nil (eh1 m p ax th iy0))
("emperor" nil (eh1 m p er0 er0))
("emperors" nil (eh1 m p er0 er0 z))
("empey" nil (eh1 m p iy0))
("emphases" nil (eh1 m f ax s iy1 z))
("emphasis" nil (eh1 m f ax s ax s))
("emphasize" nil (eh1 m f ax s ay1 z))
("emphasized" nil (eh1 m f ax s ay1 z d))
("emphasizes" nil (eh1 m f ax s ay1 z ax z))
("emphasizing" nil (eh1 m f ax s ay1 z ih0 ng))
("emphatic" nil (eh0 m f ae1 t ih0 k))
("emphatically" nil (eh0 m f ae1 t ih0 k l iy0))
("emphysema" nil (eh0 m f ay1 s m aa0))
("empie" nil (eh1 m p iy0))
("empire" nil (eh1 m p ay0 er0))
("empires" nil (eh1 m p ay0 er0 z))
("empirical" nil (eh1 m p ih1 r ih0 k ax l))
("empirically" nil (eh0 m p ih1 r ih0 k ax l iy0))
("empiricism" nil (eh0 m p ih1 r ax s ih1 z ax m))
("empiricist" nil (ih0 m p ih1 r ax s ax s t))
("emplacement" nil (ih0 m p l ey1 s m ax n t))
("employ" nil (eh0 m p l oy1))
("employable" nil (eh0 m p l oy1 ax b ax l))
("employed" nil (eh0 m p l oy1 d))
("employee" nil (eh0 m p l oy1 iy0))
("employees" nil (eh0 m p l oy1 iy0 z))
("employer" nil (eh0 m p l oy1 er0))
("employers" nil (eh0 m p l oy1 er0 z))
("employes" nil (eh0 m p l oy1 z))
("employing" nil (eh0 m p l oy1 ih0 ng))
("employment" nil (eh0 m p l oy1 m ax n t))
("employments" nil (eh0 m p l oy1 m ax n t s))
("employs" nil (eh0 m p l oy1 z))
("emporia" nil (eh0 m p ao1 r iy0 ax))
("emporium" nil (eh1 m p ao1 r iy0 ax m))
("empower" nil (ih0 m p aw1 er0))
("empowered" nil (ih0 m p aw1 er0 d))
("empowering" nil (ih0 m p aw1 er0 ih0 ng))
("empowerment" nil (ih0 m p aw1 er0 m ax n t))
("empowers" nil (ih0 m p aw1 er0 z))
("empresa" nil (eh0 m p r eh1 s ax))
("empresas" nil (eh0 m p r eh1 s ax z))
("empress" nil (eh1 m p r eh0 s))
("emprise" nil (eh0 m p r ay1 z))
("empson" nil (eh1 m p s ax n))
("empt" nil (eh1 m p t))
("empted" nil (eh1 m p t ih0 d))
("emptied" nil (eh1 m p t iy0 d))
("emptier" nil (eh1 m p t iy0 er0))
("empties" nil (eh1 m p t iy0 z))
("emptiness" nil (eh1 m p t iy0 n ax s))
("empting" nil (eh1 m p t ih0 ng))
("emption" nil (eh1 m p sh ax n))
("emptive" nil (eh1 m p t ih0 v))
("emptor" nil (eh1 m p t er0))
("empts" nil (eh1 m p t s))
("empty" nil (eh1 m p t iy0))
("emptying" nil (eh1 m p t iy0 ih0 ng))
("emrich" nil (eh1 m r ih0 k))
("emrick" nil (eh1 m r ih0 k))
("emry" nil (eh1 m r iy0))
("emslie" nil (eh1 m s ax l iy0))
("emswiler" nil (eh1 m z w ay1 l er0))
("emu" nil (iy1 m y uw1))
("emuil" nil (eh1 m y uw0 ih1 l))
("emulate" nil (eh1 m y ax l ey1 t))
("emulated" nil (eh1 m y ax l ey1 t ih0 d))
("emulating" nil (eh1 m y ax l ey1 t ih0 ng))
("emulation" nil (eh1 m y ax l ey1 sh ax n))
("emulex" nil (eh1 m y uw0 l ax k s))
("emulsifier" nil (ih0 m ah1 l s ax f ay1 er0))
("emulsion" nil (ih0 m ah1 l sh ax n))
("en" nil (eh1 n))
("ena" nil (eh1 n ax))
("enable" nil (eh0 n ey1 b ax l))
("enabled" nil (eh0 n ey1 b ax l d))
("enables" nil (eh0 n ey1 b ax l z))
("enabling" nil (eh0 n ey1 b ax l ih0 ng))
("enact" nil (ih0 n ae1 k t))
("enacted" nil (ih0 n ae1 k t ax d))
("enacting" nil (ax n ae1 k t ih0 ng))
("enactment" nil (eh0 n ae1 k t m ax n t))
("enactments" nil (ih1 n ae1 k t m ax n t s))
("enacts" nil (ih1 n ae1 k t s))
("enamel" nil (ih0 n ae1 m ax l))
("enameled" nil (ih0 n ae1 m ax l d))
("enamels" nil (ih0 n ae1 m ax l z))
("enamored" nil (eh0 n ae1 m er0 d))
("enasa" nil (ey0 n aa1 s ax))
("enberg" nil (eh1 n b er0 g))
("encamped" nil (ih0 n k ae1 m p t))
("encapsulated" nil (eh0 n k ae1 p s ax l ey1 t ih0 d))
("encapsulating" nil (eh0 n k ae1 p s ax l ey1 t ih0 ng))
("encarnacion" nil (ih0 n k aa0 r n aa0 s iy0 ao1 n))
("encarta" nil (eh0 n k aa1 r t ah1))
("encased" nil (eh0 n k ey1 s t))
("encata" nil (eh0 n k aa1 t ah1))
("encephalitis" nil (eh0 n s eh1 f ax l ay1 t ax s))
("enchanted" nil (eh0 n ch ae1 n t ih0 d))
("enchanters" nil (eh0 n ch ae1 n t er0 z))
("enchanting" nil (eh0 n ch ae1 n t ih0 ng))
("enchantment" nil (eh0 n ch ae1 n t m ax n t))
("encinas" nil (eh1 n s ih0 n ax z))
("encinias" nil (ih0 n s iy0 n iy1 ax z))
("encino" nil (eh0 n s iy1 n ow0))
("encircle" nil (eh0 n s er1 k ax l))
("encircled" nil (ih0 n s er1 k ax l d))
("encircling" nil (eh0 n s er1 k ax l ih0 ng))
("enciso" nil (ih0 n s iy1 s ow0))
("enck" nil (eh1 ng k))
("enclave" nil (aa1 n k l ey1 v))
("enclaves" nil (aa1 n k l ey1 v z))
("enclose" nil (ih0 n k l ow1 z))
("enclosed" nil (eh0 n k l ow1 z d))
("enclosing" nil (eh0 n k l ow1 z ih0 ng))
("enclosure" nil (eh0 n k l ow1 zh er0))
("enclosures" nil (ih0 n k l ow1 zh er0 z))
("encode" nil (eh0 n k ow1 d))
("encoded" nil (eh0 n k ow1 d ih0 d))
("encoding" nil (eh0 n k ow1 d ih0 ng))
("encompass" nil (eh0 n k ah1 m p ax s))
("encompassed" nil (eh0 n k ah1 m p ax s t))
("encompasses" nil (eh0 n k ah1 m p ax s ax z))
("encompassing" nil (eh0 n k ah1 m p ax s ih0 ng))
("encor" nil (eh1 n k ao1 r))
("encore" nil (aa1 n k ao1 r))
("encores" nil (aa1 n k ao1 r z))
("encounter" nil (ih0 n k aw1 n t er0))
("encountered" nil (ih0 n k aw1 n t er0 d))
("encountering" nil (eh0 n k aw1 n t er0 ih0 ng))
("encounters" nil (ih0 n k aw1 n t er0 z))
("encourage" nil (eh0 n k er1 ih0 jh))
("encouraged" nil (eh0 n k er1 ih0 jh d))
("encouragement" nil (eh0 n k er1 ih0 jh m ax n t))
("encourages" nil (eh0 n k er1 ih0 jh ih0 z))
("encouraging" nil (eh0 n k er1 ih0 jh ih0 ng))
("encroach" nil (ih0 n k r ow1 ch))
("encroached" nil (ih0 n k r ow1 ch t))
("encroaches" nil (ih0 n k r ow1 ch ih0 z))
("encroaching" nil (ih0 n k r ow1 ch ih0 ng))
("encroachment" nil (eh0 n k r ow1 ch m ax n t))
("encroachments" nil (ih0 n k r ow1 ch m ax n t s))
("encrusted" nil (eh0 n k r ah1 s t ih0 d))
("encrusting" nil (eh0 n k r ah1 s t ih0 ng))
("encrypted" nil (eh0 n k r ih1 p t ih0 d))
("encryption" nil (eh0 n k r ih1 p sh ax n))
("encumber" nil (eh0 n k ah1 m b er0))
("encumbered" nil (eh0 n k ah1 m b er0 d))
("encyclical" nil (eh0 n s ih1 k l ih0 k ax l))
("encyclicals" nil (eh0 n s ih1 k l ih0 k ax l z))
("encyclopaedia" nil (ih0 n s ay1 k l ax p iy1 d iy0 ax))
("encyclopedia" nil (ih0 n s ay1 k l ax p iy1 d iy0 ax))
("encyclopedias" nil (ih0 n s ay1 k l ax p iy1 d iy0 ax z))
("encyclopedic" nil (ih0 n s ay1 k l ax p iy1 d ih0 k))
("encyclopedist" nil (ih0 n s ay1 k l ax p iy1 d ax s t))
("end" nil (eh1 n d))
("endaka" nil (eh0 n d aa1 k ax))
("endanger" nil (eh0 n d ey1 n jh er0))
("endangered" nil (eh0 n d ey1 n jh er0 d))
("endangering" nil (eh0 n d ey1 n jh er0 ih0 ng))
("endangerment" nil (eh0 n d ey1 n jh er0 m ax n t))
("endangers" nil (eh0 n d ey1 n jh er0 z))
("endara" nil (eh1 n d aa1 r ax))
("ende" nil (eh1 n d))
("endear" nil (eh0 n d iy1 r))
("endeared" nil (eh0 n d iy1 r d))
("endearing" nil (eh0 n d iy1 r ih0 ng))
("endearment" nil (ih0 n d ih1 r m ax n t))
("endeavor" nil (ih0 n d eh1 v er0))
("endeavored" nil (ih0 n d eh1 v er0 d))
("endeavors" nil (ih0 n d eh1 v er0 z))
("endeavour" nil (ih0 n d eh1 v er0))
("ended" nil (eh1 n d ax d))
("endemic" nil (eh0 n d eh1 m ih0 k))
("ender" nil (eh1 n d er0))
("enderby" nil (eh1 n d er0 b iy0))
("enderle" nil (eh1 n d er0 ax l))
("enderlin" nil (eh1 n d er0 l ih0 n))
("enders" nil (eh1 n d er0 z))
("enderson" nil (eh1 n d er0 s ax n))
("endevco" nil (eh0 n d eh1 v k ow0))
("endicott" nil (eh1 n d ih0 k aa1 t))
("ending" nil (eh1 n d ih0 ng))
("endings" nil (eh1 n d ih0 ng z))
("endive" nil (eh1 n d ih0 v))
("endler" nil (eh1 n d l er0))
("endless" nil (eh1 n d l ax s))
("endlessly" nil (eh1 n d l ax s l iy0))
("endlich" nil (eh1 n d l ih0 k))
("endo" nil (eh1 n d ow0))
("endocrine" nil (eh1 n d ow0 k r ay1 n))
("endocrinologist" nil (eh1 n d ow0 k r ax n aa1 l ax jh ax s t))
("endocrinologists" nil (eh1 n d ow0 k r ax n aa1 l ax jh ax s t s))
("endocrinology" nil (eh1 n d ow0 k r ax n aa1 l ax jh iy0))
("endodermal" nil (eh1 n d ow0 d er1 m ax l))
("endometriosis" nil (eh1 n d ow0 m eh1 t r iy0 ow1 s ih0 s))
("endorse" nil (eh0 n d ao1 r s))
("endorsed" nil (eh0 n d ao1 r s t))
("endorsement" nil (eh0 n d ao1 r s m ax n t))
("endorsements" nil (eh0 n d ao1 r s m ax n t s))
("endorser" nil (ih0 n d ao1 r s er0))
("endorsers" nil (ih0 n d ao1 r s er0 z))
("endorses" nil (eh0 n d ao1 r s ih0 z))
("endorsing" nil (eh0 n d ao1 r s ih0 ng))
("endoscopic" nil (eh1 n d ow0 s k aa1 p ih0 k))
("endosperm" nil (eh1 n d ax s p er1 m))
("endothermic" nil (eh1 n d ow0 th er1 m ih0 k))
("endotronics" nil (eh1 n d ow0 t r aa1 n ih0 k s))
("endow" nil (eh0 n d aw1))
("endowed" nil (eh0 n d aw1 d))
("endowing" nil (eh0 n d aw1 ih0 ng))
("endowment" nil (eh0 n d aw1 m ax n t))
("endowments" nil (eh0 n d aw1 m ax n t s))
("endpoint" nil (eh1 n d p oy1 n t))
("endres" nil (eh1 n d er0 z))
("endress" nil (eh1 n d r ih0 s))
("endrizzi" nil (ih0 n d r iy1 t s iy0))
("ends" nil (eh1 n d z))
("endsley" nil (eh1 n d s l iy0))
("endued" nil (eh0 n d uw1 d))
("endurance" nil (eh1 n d er0 ax n s))
("endure" nil (eh0 n d y uh1 r))
("endured" nil (eh0 n d y uh1 r d))
("endures" nil (eh0 n d y uh1 r z))
("enduring" nil (eh0 n d y uh1 r ih0 ng))
("endy" nil (eh1 n d iy0))
("enea" nil (eh1 n iy0 ax))
("enema" nil (eh1 n ax m ax))
("enemas" nil (eh1 n ax m ax z))
("enemies" nil (eh1 n ax m iy0 z))
("enemy" nil (eh1 n ax m iy0))
("energas" nil (eh1 n er0 g ae1 s))
("energen" nil (eh1 n er0 jh eh1 n))
("energetic" nil (eh1 n er0 jh eh1 t ih0 k))
("energetically" nil (eh1 n er0 jh eh1 t ih0 k l iy0))
("energies" nil (eh1 n er0 jh iy0 z))
("energize" nil (eh1 n er0 jh ay1 z))
("energized" nil (eh1 n er0 jh ay1 z d))
("energizer" nil (eh1 n er0 jh ay1 z er0))
("energizing" nil (eh1 n er0 jh ay1 z ih0 ng))
("energy" nil (eh1 n er0 jh iy0))
("enerson" nil (eh1 n er0 s ax n))
("enex" nil (iy1 n ax k s))
("enfant" nil (eh1 n f aa1 n t))
("enfeebled" nil (eh0 n f iy1 b ax l d))
("enfield" nil (eh1 n f iy0 l d))
("enfinger" nil (eh1 n f ih0 ng er0))
("enfold" nil (ih0 n f ow1 l d))
("enforce" nil (eh0 n f ao1 r s))
("enforceability" nil (eh0 n f ao1 r s ax b ih1 l ih0 t iy0))
("enforceable" nil (eh0 n f ao1 r s ax b ax l))
("enforced" nil (eh0 n f ao1 r s t))
("enforcement" nil (eh0 n f ao1 r s m ax n t))
("enforcer" nil (eh0 n f ao1 r s er0))
("enforcers" nil (eh0 n f ao1 r s er0 z))
("enforces" nil (eh0 n f ao1 r s ih0 z))
("enforcing" nil (eh0 n f ao1 r s ih0 ng))
("enfranchise" nil (ih0 n f r ae1 n ch ay1 z))
("enfranchised" nil (eh0 n f r ae1 n ch ay1 z d))
("enfranchises" nil (eh0 n f r ae1 n ch ay1 z ih0 z))
("eng" nil (eh1 ng))
("engage" nil (eh0 n g ey1 jh))
("engaged" nil (eh0 n g ey1 jh d))
("engagement" nil (eh0 n g ey1 jh m ax n t))
("engagements" nil (eh0 n g ey1 jh m ax n t s))
("engages" nil (eh0 n g ey1 jh ih0 z))
("engaging" nil (eh0 n g ey1 jh ih0 ng))
("engberg" nil (eh1 ng b er0 g))
("engdahl" nil (eh1 ng d aa0 l))
("enge" nil (eh1 n jh))
("engebretsen" nil (eh1 ng g ih0 b r ih0 t s ax n))
("engebretson" nil (eh1 ng g ih0 b r ih0 t s ax n))
("engel" nil (eh1 n g ax l))
("engelberg" nil (eh1 ng g ax l b er0 g))
("engelbert" nil (eh1 ng g ih0 l b er0 t))
("engelberta" nil (eh0 ng g eh0 l b eh1 r t ax))
("engelbrecht" nil (eh1 ng g ih0 l b r ih0 k t))
("engeleiter" nil (eh1 ng g ax l ay1 t er0))
("engelhard" nil (eh1 ng g ax l hh aa1 r d))
("engelhardt" nil (eh1 ng g ih0 l hh aa0 r t))
("engelhart" nil (eh1 ng g ax l hh aa1 r t))
("engelke" nil (eh1 ng g ih0 l k))
("engelken" nil (eh1 ng g ih0 l k ax n))
("engelking" nil (eh1 ng g ih0 l k ih0 ng))
("engelman" nil (eh1 ng g ax l m ax n))
("engelmann" nil (eh1 ng g ax l m ax n))
("engels" nil (eh1 ng g ax l z))
("engelson" nil (eh1 ng g ih0 l s ax n))
("engelstad" nil (eh1 ng g ih0 l s t ax d))
("engeman" nil (eh1 n jh m ax n))
("engen" nil (eh1 ng ax n))
("engender" nil (eh0 n jh eh1 n d er0))
("engendered" nil (eh0 n jh eh1 n d er0 d))
("engenders" nil (eh1 ng g ax n d er0 z))
("enger" nil (eh1 ng g er0))
("engert" nil (eh1 ng g er0 t))
("engesser" nil (eh1 ng g ih0 s er0))
("engh" nil (eh1 ng))
("engholm" nil (eh1 ng hh ow0 l m))
("engine" nil (eh1 n jh ax n))
("engined" nil (eh1 n jh ax n d))
("engineer" nil (eh1 n jh ax n ih1 r))
("engineered" nil (eh1 n jh ax n iy1 r d))
("engineering" nil (eh1 n jh ax n ih1 r ih0 ng))
("engineers" nil (eh1 n jh ax n ih1 r z))
("engines" nil (eh1 n jh ax n z))
("england" nil (ih1 ng g l ax n d))
("englander" nil (ih1 ng g l ax n d er0))
("englanders" nil (ih1 ng g l ax n d er0 z))
("englands" nil (ih1 ng g l ax n d z))
("engle" nil (eh1 ng g ax l))
("englebert" nil (ih1 ng l ih0 b er0 t))
("englehardt" nil (ih1 ng l ih0 hh aa0 r t))
("englehart" nil (ih1 ng ax l hh aa0 r t))
("engleman" nil (ih1 ng ax l m ax n))
("engler" nil (eh1 ng g ax l er0))
("englert" nil (ih1 ng l er0 t))
("englerth" nil (ih1 ng l er0 th))
("engles" nil (ih1 ng ax l z))
("englewood" nil (eh1 ng g ax l w uh1 d))
("english" nil (ih1 ng g l ih0 sh))
("englishman" nil (ih1 ng g l ih0 sh m ax n))
("englishmen" nil (eh1 ng l ih1 sh m ax n))
("englishwoman" nil (ih1 ng g l ih0 sh w uh1 m ax n))
("englund" nil (ih1 ng l ax n d))
("engman" nil (eh1 ng m ax n))
("engquist" nil (eh1 ng k w ih0 s t))
("engram" nil (eh1 n g r ae1 m))
("engrave" nil (ih0 n g r ey1 v))
("engraved" nil (ih0 n g r ey1 v d))
("engraver" nil (ih0 n g r ey1 v er0))
("engraving" nil (ih0 n g r ey1 v ih0 ng))
("engravings" nil (ih0 n g r ey1 v ih0 ng z))
("engross" nil (ih0 n g r ow1 s))
("engrossed" nil (ih0 n g r ow1 s t))
("engrossing" nil (ih0 n g r ow1 s ih0 ng))
("engstrand" nil (eh1 ng s t r ax n d))
("engstrom" nil (eh1 ng s t r ax m))
("engulf" nil (ih0 n g ah1 l f))
("engulfed" nil (ih0 n g ah1 l f t))
("engulfing" nil (ih0 n g ah1 l f ih0 ng))
("engwall" nil (ih0 ng w ao1 l))
("enhance" nil (eh0 n hh ae1 n s))
("enhanced" nil (eh0 n hh ae1 n s t))
("enhancement" nil (eh0 n hh ae1 n s m ax n t))
("enhancements" nil (eh0 n hh ae1 n s m ax n t s))
("enhancer" nil (eh0 n hh ae1 n s er0))
("enhances" nil (eh0 n hh ae1 n s ih0 z))
("enhancing" nil (eh0 n hh ae1 n s ih0 ng))
("enholm" nil (eh1 n hh ow1 l m))
("enholme" nil (eh1 n hh ow1 l m))
("enichem" nil (eh1 n ih0 ch ax m))
("enid" nil (iy1 n ih0 d))
("enigma" nil (ih0 n ih1 g m ax))
("enigmatic" nil (eh1 n ih0 g m ae1 t ih0 k))
("enimont" nil (iy1 n ih0 m aa1 n t))
("enis" nil (eh1 n ih0 s))
("enix" nil (eh1 n ih0 k s))
("enjoin" nil (eh0 n jh oy1 n))
("enjoined" nil (eh0 n jh oy1 n d))
("enjoining" nil (eh0 n jh oy1 n ih0 ng))
("enjoy" nil (eh0 n jh oy1))
("enjoyable" nil (eh0 n jh oy1 ax b ax l))
("enjoyed" nil (eh0 n jh oy1 d))
("enjoying" nil (eh0 n jh oy1 ih0 ng))
("enjoyment" nil (eh0 n jh oy1 m ax n t))
("enjoyments" nil (eh0 n jh oy1 m ax n t s))
("enjoys" nil (eh0 n jh oy1 z))
("enke" nil (eh1 ng k))
("enlarge" nil (eh0 n l aa1 r jh))
("enlarged" nil (eh0 n l aa1 r jh d))
("enlargement" nil (ih0 n l aa1 r jh m ax n t))
("enlargements" nil (ih0 n l aa1 r jh m ax n t s))
("enlarger" nil (ih0 n l aa1 r g er0))
("enlarges" nil (ih0 n l aa1 r jh ih0 z))
("enlarging" nil (ih0 n l aa1 r jh ih0 ng))
("enlighten" nil (eh0 n l ay1 t ax n))
("enlightened" nil (eh0 n l ay1 t ax n d))
("enlightening" nil (eh0 n l ay1 t ax n ih0 ng))
("enlightenment" nil (eh0 n l ay1 t ax n m ax n t))
("enlist" nil (eh0 n l ih1 s t))
("enlisted" nil (eh0 n l ih1 s t ih0 d))
("enlistees" nil (ih0 n l ih1 s t iy1 z))
("enlisting" nil (eh0 n l ih1 s t ih0 ng))
("enlistment" nil (eh0 n l ih1 s t m ax n t))
("enlists" nil (eh0 n l ih1 s t s))
("enliven" nil (eh0 n l ay1 v ax n))
("enlivened" nil (eh0 n l ay1 v ax n d))
("enlo" nil (eh1 n l ow0))
("enloe" nil (ih0 n l ow1))
("enlow" nil (ih0 n l ow1))
("enman" nil (eh1 n m ax n))
("enmeshed" nil (eh0 n m eh1 sh t))
("enmity" nil (eh1 n m ax t iy0))
("ennea" nil (eh1 n iy0 ax))
("enneking" nil (eh1 n ih0 k ih0 ng))
("ennen" nil (eh1 n ax n))
("ennes" nil (eh1 n z))
("ennis" nil (eh1 n ih0 s))
("ennoble" nil (ih0 n ow1 b ax l))
("ennosuke" nil (eh1 n ax s uw0 k))
("enns" nil (eh1 n z))
("ennui" nil (eh0 n uw1 iy0))
("eno" nil (eh1 n ow0))
("enoch" nil (iy1 n ax k))
("enochs" nil (eh1 n ax k s))
("enola" nil (ih0 n ow1 l ax))
("enomoto" nil (ih0 n ow0 m ow1 t ow0))
("enormity" nil (ih0 n ao1 r m ax t iy0))
("enormous" nil (ih0 n ao1 r m ax s))
("enormously" nil (ih0 n ao1 r m ax s l iy0))
("enough" nil (ih0 n ah1 f))
("enquirer" nil (ih0 n k w ay1 r er0))
("enquist" nil (eh1 n k w ih1 s t))
("enrage" nil (eh0 n r ey1 jh))
("enraged" nil (eh0 n r ey1 jh d))
("enraging" nil (eh0 n r ey1 jh ih0 ng))
("enraptured" nil (eh0 n r ae1 p ch er0 d))
("enrica" nil (ih0 n r iy1 k ax))
("enrich" nil (eh0 n r ih1 ch))
("enriched" nil (eh0 n r ih1 ch t))
("enriches" nil (eh0 n r ih1 ch ih0 z))
("enriching" nil (eh0 n r ih1 ch ih0 ng))
("enrichment" nil (eh0 n r ih1 ch m ax n t))
("enrico" nil (eh0 n r iy1 k ow0))
("enright" nil (ih0 n r ay1 t))
("enrile" nil (eh0 n r iy1 l))
("enrique" nil (eh0 n r iy1 k ey0))
("enriquez" nil (ih0 n r iy1 k w eh0 z))
("enro" nil (eh1 n r ow0))
("enroll" nil (eh0 n r ow1 l))
("enrolled" nil (eh0 n r ow1 l d))
("enrollees" nil (eh0 n r ow1 l iy1 z))
("enrolling" nil (eh0 n r ow1 l ih0 ng))
("enrollment" nil (eh0 n r ow1 l m ax n t))
("enrollments" nil (eh0 n r ow1 l m ax n t s))
("enrolls" nil (eh0 n r ow1 l z))
("enron" nil (eh1 n r aa0 n))
("enroute" nil (eh0 n r uw1 t))
("ensco" nil (eh1 n s k ow0))
("ensconced" nil (ih0 n s k aa1 n s t))
("enseco" nil (eh0 n s eh1 k ow0))
("ensemble" nil (aa0 n s aa1 m b ax l))
("ensembles" nil (aa0 n s aa1 m b ax l z))
("enserch" nil (eh1 n s er0 ch))
("ensey" nil (eh1 n z iy0))
("enshrine" nil (eh0 n sh r ay1 n))
("enshrined" nil (eh0 n sh r ay1 n d))
("enshrouded" nil (ih0 n sh r aw1 d ax d))
("ensign" nil (eh1 n s ax n))
("ensing" nil (eh1 n s ih0 ng))
("ensinger" nil (eh1 n s ih0 n jh er0))
("enskilda" nil (eh0 n s k ih1 l d ax))
("enslaved" nil (eh0 n s l ey1 v d))
("enslavement" nil (eh0 n s l ey1 v m ax n t))
("enslen" nil (eh1 n s ax l ax n))
("ensley" nil (eh1 n s l iy0))
("enslin" nil (eh1 n s l ih0 n))
("enslow" nil (ih0 n s l ow1))
("ensminger" nil (eh1 n s ax m ih0 ng er0))
("ensnare" nil (ih0 n s n eh1 r))
("ensnared" nil (eh0 n s n eh1 r d))
("ensnarled" nil (ih0 n s n aa1 r l d))
("ensor" nil (eh1 n s er0))
("ensource" nil (eh0 n s ao1 r s))
("ensrud" nil (eh1 n z r uw1 d))
("enstrom" nil (eh1 n s t r ax m))
("ensue" nil (ih0 n s uw1))
("ensued" nil (ih0 n s uw1 d))
("ensues" nil (ih0 n s uw1 z))
("ensuing" nil (eh1 n s uw0 ih0 ng))
("ensure" nil (eh0 n sh uh1 r))
("ensured" nil (eh0 n sh uh1 r d))
("ensures" nil (eh0 n sh uh1 r z))
("ensuring" nil (eh0 n sh uh1 r ih0 ng))
("ensz" nil (eh1 n sh))
("ent" nil (eh1 n t))
("entail" nil (eh0 n t ey1 l))
("entailed" nil (ih0 n t ey1 l d))
("entailing" nil (ih0 n t ey1 l ih0 ng))
("entails" nil (ih0 n t ey1 l z))
("entangle" nil (eh0 n t ae1 ng g ax l))
("entangled" nil (eh0 n t ae1 ng g ax l d))
("entanglement" nil (eh0 n t ae1 ng g ax l m ax n t))
("entanglements" nil (eh0 n t ae1 ng g ax l m ax n t s))
("entangling" nil (ih0 n t ae1 ng l ih0 ng))
("ente" nil (eh1 n t ey0))
("entebbe" nil (eh1 n t eh1 b iy0))
("entendre" nil (aa0 n t aa1 n d r ax))
("enter" nil (eh1 n t er0))
("entered" nil (eh1 n t er0 d))
("entergy" nil (eh1 n t er0 jh iy0))
("entering" nil (eh1 n t er0 ih0 ng))
("enteritidis" nil (eh1 n t er0 ih1 t ih0 d ih0 s))
("enterline" nil (eh1 n t er0 l ay1 n))
("enterprise" nil (eh1 n t er0 p r ay1 z))
("enterprises" nil (eh1 n t er0 p r ay1 z ih0 z))
("enterprising" nil (eh1 n t er0 p r ay1 z ih0 ng))
("enterra" nil (eh0 n t eh1 r ax))
("enters" nil (eh1 n t er0 z))
("entertain" nil (eh1 n t er0 t ey1 n))
("entertained" nil (eh1 n t er0 t ey1 n d))
("entertainer" nil (eh1 n t er0 t ey1 n er0))
("entertainers" nil (eh1 n t er0 t ey1 n er0 z))
("entertaining" nil (eh1 n t er0 t ey1 n ih0 ng))
("entertainment" nil (eh1 n t er0 t ey1 n m ax n t))
("entertainments" nil (eh1 n t er0 t ey1 n m ax n t s))
("entertains" nil (eh1 n t er0 t ey1 n z))
("entex" nil (eh1 n t eh1 k s))
("enthralled" nil (eh0 n th r ao1 l d))
("enthused" nil (ih0 n th uw1 z d))
("enthusiasm" nil (ih0 n th uw1 z iy0 ae1 z ax m))
("enthusiasms" nil (ih0 n th uw1 z iy0 ae1 z ax m z))
("enthusiast" nil (eh0 n th uw1 z iy0 ae1 s t))
("enthusiastic" nil (ih0 n th uw1 z iy0 ae1 s t ih0 k))
("enthusiastically" nil (ih0 n th uw1 z iy0 ae1 s t ih0 k l iy0))
("enthusiasts" nil (eh0 n th uw1 z iy0 ae1 s t s))
("entice" nil (ih0 n t ay1 s))
("enticed" nil (ih0 n t ay1 s t))
("enticement" nil (ih0 n t ay1 s m ax n t))
("enticements" nil (ih0 n t ay1 s m ax n t s))
("enticing" nil (eh0 n t ay1 s ih0 ng))
("entin" nil (eh1 n t ih0 n))
("entire" nil (ih0 n t ay1 er0))
("entirely" nil (ih0 n t ay1 er0 l iy0))
("entirety" nil (ih0 n t ay1 er0 t iy0))
("entities" nil (eh1 n t ih0 t iy0 z))
("entitle" nil (eh0 n t ay1 t ax l))
("entitled" nil (eh0 n t ay1 t ax l d))
("entitlement" nil (eh0 n t ay1 t ax l m ax n t))
("entitlements" nil (eh0 n t ay1 t ax l m ax n t s))
("entitles" nil (eh0 n t ay1 t ax l z))
("entitling" nil (eh0 n t ay1 t ax l ih0 ng))
("entity" nil (eh1 n t ax t iy0))
("entler" nil (eh1 n t l er0))
("entoffen" nil (eh1 n t ax f ax n))
("entombed" nil (eh0 n t uw1 m d))
("entombment" nil (ih0 n t uw1 m m ax n t))
("entomologist" nil (eh1 n t ax m aa1 l ax jh ax s t))
("entomologists" nil (eh1 n t ax m aa1 l ax jh ax s t s))
("entomology" nil (eh1 n t ax m aa1 l ax jh iy0))
("entourage" nil (aa1 n t uh0 r aa1 zh))
("entrails" nil (eh1 n t r ax l z))
("entrance" n (eh1 n t r ax n s))
("entrance" v (eh0 n t r aa1 n s))
("entranced" nil (ih0 n t r ae1 n s t))
("entrances" n (eh1 n t r ax n s ax z))
("entrances" v (eh0 n t r aa1 n s ax z))
("entrant" nil (eh1 n t r ax n t))
("entrants" nil (eh1 n t r ax n t s))
("entrapment" nil (ih0 n t r ae1 p m ax n t))
("entre" nil (aa1 n t r ey0))
("entreaties" nil (eh0 n t r iy1 t iy0 z))
("entree" nil (aa1 n t r ey1))
("entrees" nil (aa1 n t r ey1 z))
("entregrowth" nil (eh1 n t r ax g r ow0 th))
("entrekin" nil (eh1 n t r ih0 k ih0 n))
("entrench" nil (eh0 n t r eh1 n ch))
("entrenched" nil (eh0 n t r eh1 n ch t))
("entrenches" nil (eh0 n t r eh1 n ch ih0 z))
("entrenching" nil (eh0 n t r eh1 n ch ih0 ng))
("entrenchment" nil (eh0 n t r eh1 n ch m ax n t))
("entrepreneur" nil (aa1 n t r ax p r ax n er1))
("entrepreneurial" nil (aa1 n t r ax p r ax n er1 iy0 ax l))
("entrepreneurialism" nil (eh1 n t r ax p r ax n uw1 r iy1 ax l ih1 z ax m))
("entrepreneurs" nil (aa1 n t r ax p r ax n er1 z))
("entrepreneurship" nil (aa1 n t r ax p r ax n er1 sh ih0 p))
("entries" nil (eh1 n t r iy0 z))
("entrikin" nil (eh1 n t r ih0 k ih0 n))
("entringer" nil (eh1 n t er0 ih0 ng er0))
("entropy" nil (eh1 n t r ax p iy0))
("entrust" nil (eh0 n t r ah1 s t))
("entrusted" nil (eh0 n t r ah1 s t ih0 d))
("entrusting" nil (eh0 n t r ah1 s t ih0 ng))
("entry" nil (eh1 n t r iy0))
("entsminger" nil (eh1 n t s ax m ih0 ng er0))
("entwined" nil (eh0 n t w ay1 n d))
("entwisle" nil (eh1 n t w ay0 ax l))
("entwistle" nil (ih0 n t w ih1 s ax l))
("entz" nil (eh1 n t s))
("entzminger" nil (eh1 n t s m ih0 ng er0))
("enumerated" nil (ih0 n uw1 m er0 ey1 t ih0 d))
("enumeration" nil (ih0 n uw1 m er0 ey1 sh ax n))
("enunciated" nil (ih0 n ah1 n s iy0 ey1 t ih0 d))
("envelop" nil (ih0 n v eh1 l ax p))
("envelope" nil (eh1 n v ax l ow1 p))
("enveloped" nil (eh0 n v eh1 l ax p t))
("envelopes" nil (eh1 n v ax l ow1 p s))
("enveloping" nil (ih0 n v eh1 l ax p ih0 ng))
("enviable" nil (eh1 n v iy0 ax b ax l))
("envied" nil (eh1 n v iy0 d))
("envious" nil (eh1 n v iy0 ax s))
("enviously" nil (eh1 n v iy0 ax s l iy0))
("envirodyne" nil (eh0 n v ay1 r ow0 d ay1 n))
("environics" nil (eh1 n v ay0 r ao1 n ih0 k s))
("environment" nil (ih0 n v ay1 r ax n m ax n t))
("environmental" nil (ih0 n v ay1 r ax n m eh1 n t ax l))
("environmentalism" nil (eh0 n v ay1 r ax eh0 n m eh1 n t ax l ih1 z ax m))
("environmentalist" nil (ih0 n v ay1 r ax n m eh1 n t ax l ih0 s t))
("environmentalists" nil (eh0 n v ay1 r ax n m eh1 n t ax l ih0 s t s))
("environmentally" nil (ih0 n v ay1 r ax n m eh1 n t ax l iy0))
("environments" nil (ih0 n v ay1 r ax n m ax n t s))
("environs" nil (ih0 n v ay1 r ax n z))
("enviropact" nil (ih0 n v ay1 r ow0 p ae1 k t))
("envirosafe" nil (ih0 n v ay1 r ow0 s ey1 f))
("envirotest" nil (ih1 n v ay1 r ow0 t eh1 s t))
("envisage" nil (eh0 n v ih1 z ih0 jh))
("envisaged" nil (eh0 n v ih1 z ih0 jh d))
("envisages" nil (eh0 n v ih1 z ih0 jh ih0 z))
("envision" nil (eh0 n v ih1 zh ax n))
("envisioned" nil (eh0 n v ih1 zh ax n d))
("envisioning" nil (eh0 n v ih1 zh ax n ih0 ng))
("envisions" nil (eh0 n v ih1 zh ax n z))
("envos" nil (eh1 n v ow0 s))
("envoy" nil (eh1 n v oy0))
("envoys" nil (eh1 n v oy0 z))
("envy" nil (eh1 n v iy0))
("enwright" nil (ih0 n w r ay1 t))
("enyart" nil (eh1 n y aa0 r t))
("enyeart" nil (eh1 n y er0 t))
("enz" nil (eh1 n z))
("enzo" nil (eh1 n z ow0))
("enzon" nil (eh1 n z aa0 n))
("enzor" nil (eh1 n z er0))
("enzymatic" nil (eh1 n z ay0 m ae1 t ih0 k))
("enzyme" nil (eh1 n z ay1 m))
("enzymes" nil (eh1 n z ay1 m z))
("eocene" nil (iy1 ax s iy1 n))
("eoff" nil (ey1 ao0 f))
("eohippus" nil (iy1 ow0 hh ih1 p ax s))
("eolande" nil (ey1 ow0 l aa1 n d iy0))
("eon" nil (iy1 ax n))
("eons" nil (iy1 ax n z))
("epcot" nil (eh1 p k aa0 t))
("epeda" nil (eh0 p ey1 d ax))
("epes" nil (iy1 p s))
("ephedrine" nil (ih0 f eh1 d r ih0 n))
("ephemeral" nil (ih0 f eh1 m er0 ax l))
("ephlin" nil (eh1 f l ih0 n))
("ephraim" nil (iy1 f r ax m))
("ephron" nil (eh1 f r ax n))
("epic" nil (eh1 p ih0 k))
("epicenter" nil (eh1 p ax s eh1 n t er0))
("epics" nil (eh1 p ih0 k s))
("epicure" nil (eh1 p ih0 k y uh1 r))
("epicurean" nil (eh1 p ax k y uh0 r iy1 ax n))
("epidemic" nil (eh1 p ax d eh1 m ih0 k))
("epidemics" nil (eh1 p ax d eh1 m ih0 k s))
("epidemiological" nil (eh1 p ax d iy0 m iy0 ax l aa1 jh ih0 k ax l))
("epidemiologist" nil (eh1 p ax d iy0 m iy0 aa1 l ax jh ih0 s t))
("epidemiologists" nil (eh1 p ax d iy0 m iy0 aa1 l ax jh ih0 s t s))
("epidemiology" nil (eh1 p ax d eh1 m iy0 aa1 l ax jh iy0))
("epidermal" nil (eh1 p ax d er1 m ax l))
("epidermis" nil (eh1 p ax d er1 m ax s))
("epigenetic" nil (eh1 p ax jh ax n eh1 t ih0 k))
("epigram" nil (eh1 p ax g r ae1 m))
("epigraphic" nil (eh1 p ax g r ae1 f ih0 k))
("epilepsies" nil (eh1 p ax l eh1 p s iy0 z))
("epilepsy" nil (eh1 p ax l eh1 p s iy0))
("epileptic" nil (eh1 p ax l eh1 p t ih0 k))
("epilogue" nil (eh1 p ax l ao1 g))
("epiphany" nil (ih0 p ih1 f ax n iy0))
("episcopal" nil (ih0 p ih1 s k ax p ax l))
("episcopo" nil (ih0 p ih0 s k ow1 p ow0))
("episode" nil (eh1 p ax s ow1 d))
("episodes" nil (eh1 p ax s ow1 d z))
("episodic" nil (eh1 p ax s aa1 d ih0 k))
("episteme" nil (eh1 p ih0 s t iy1 m))
("epistemology" nil (ih0 p ih1 s t ax m aa1 l ax jh iy0))
("epistle" nil (ih0 p ih1 s ax l))
("epistolary" nil (ih0 p ih1 s t ax l eh1 r iy0))
("epitaph" nil (eh1 p ax t ae1 f))
("epitaphs" nil (eh1 p ax t ae1 f s))
("epithet" nil (eh1 p ax th eh1 t))
("epithets" nil (eh1 p ax th eh1 t s))
("epitome" nil (ih0 p ih1 t ax m iy0))
("epitomize" nil (ih0 p ih1 t ax m ay1 z))
("epitomized" nil (ih0 p ih1 t ax m ay1 z d))
("epitomizes" nil (ih0 p ih1 t ax m ay1 z ih0 z))
("epitope" nil (eh1 p ih0 t ow1 p))
("epler" nil (eh1 p l er0))
("epley" nil (eh1 p l iy0))
("eplin" nil (eh1 p l ih0 n))
("epling" nil (eh1 p l ih0 ng))
("epoch" nil (eh1 p ax k))
("epochal" nil (eh1 p ax k ax l))
("epochs" nil (eh1 p ax k s))
("epogen" nil (eh1 p ax jh eh0 n))
("epoxy" nil (ih0 p aa1 k s iy0))
("epp" nil (eh1 p))
("eppard" nil (eh1 p er0 d))
("eppel" nil (eh1 p ax l))
("epperly" nil (eh1 p er0 l iy0))
("eppers" nil (eh1 p er0 z))
("epperson" nil (eh1 p er0 s ax n))
("eppes" nil (eh1 p s))
("eppich" nil (eh1 p ih0 ch))
("epping" nil (eh1 p ih0 ng))
("eppinger" nil (eh1 p ih0 ng er0))
("epple" nil (eh1 p ax l))
("eppler" nil (eh1 p l er0))
("eppley" nil (eh1 p l iy0))
("eppner" nil (eh1 p n er0))
("eppolito" nil (eh0 p ow0 l iy1 t ow0))
("epps" nil (eh1 p s))
("eprom" nil (eh1 p r ax m))
("eproms" nil (eh1 p r ax m z))
("epsco" nil (eh1 p s k ow0))
("epsilon" nil (eh1 p s ax l aa1 n))
("epsom" nil (eh1 p s ax m))
("epsomite" nil (eh1 p s ax m ay1 t))
("epson" nil (eh1 p s ax n))
("epstein" nil (eh1 p s t iy1 n))
("epting" nil (eh1 p t ih0 ng))
("equable" nil (eh1 k w ax b ax l))
("equal" nil (iy1 k w ax l))
("equaled" nil (iy1 k w ax l d))
("equaling" nil (iy1 k w ax l ih0 ng))
("equality" nil (ih0 k w aa1 l ax t iy0))
("equalization" nil (iy1 k w ax l ih0 z ey1 sh ax n))
("equalize" nil (iy1 k w ax l ay1 z))
("equalized" nil (iy1 k w ax l ay1 z d))
("equalizer" nil (iy1 k w ax l ay1 z er0))
("equalizing" nil (iy1 k w ax l ay1 z ih0 ng))
("equally" nil (iy1 k w ax l iy0))
("equals" nil (iy1 k w ax l z))
("equanimity" nil (iy1 k w ax n ih1 m ih0 t iy0))
("equate" nil (ih0 k w ey1 t))
("equated" nil (ih0 k w ey1 t ih0 d))
("equates" nil (ih0 k w ey1 t s))
("equating" nil (ih0 k w ey1 t ih0 ng))
("equation" nil (ih0 k w ey1 zh ax n))
("equations" nil (ih0 k w ey1 zh ax n z))
("equator" nil (ih0 k w ey1 t er0))
("equatorial" nil (iy1 k w ax t ao1 r iy0 ax l))
("equestrian" nil (ih0 k w eh1 s t r iy0 ax n))
("equiangular" nil (iy1 k w ax ae1 ng g y ax l er0))
("equibank" nil (eh1 k w ax b ae1 ng k))
("equicor" nil (eh1 k w ih1 k ao1 r))
("equifax" nil (eh1 k w ih0 f ae1 k s))
("equilibria" nil (iy1 k w ax l ih1 b r iy0 ax))
("equilibrium" nil (iy1 k w ax l ih1 b r iy0 ax m))
("equilink" nil (eh1 k w ax l ih1 ng k))
("equimark" nil (eh1 k w ih0 m aa1 r k))
("equine" nil (iy1 k w ay1 n))
("equinox" nil (iy1 k w ax n aa1 k s))
("equion" nil (eh1 k w iy0 aa0 n))
("equip" nil (ih0 k w ih1 p))
("equipment" nil (ih0 k w ih1 p m ax n t))
("equipments" nil (ih0 k w ih1 p m ax n t s))
("equipped" nil (ih0 k w ih1 p t))
("equipping" nil (ih0 k w ih1 p ih0 ng))
("equips" nil (ih0 k w ih1 p s))
("equitable" nil (eh1 k w ax t ax b ax l))
("equitably" nil (eh1 k w ih0 t ax b l iy0))
("equitas" nil (eh1 k w ax t ax s))
("equitation" nil (eh1 k w ax t ey1 sh ax n))
("equitec" nil (eh1 k w ax t eh1 k))
("equitex" nil (eh1 k w ax t eh1 k s))
("equiticorp" nil (eh1 k w ih0 t iy0 k ao1 r p))
("equities" nil (eh1 k w ax t iy0 z))
("equity" nil (eh1 k w ax t iy0))
("equivalence" nil (ih0 k w ih1 v ax l ax n s))
("equivalency" nil (ih0 k w ih1 v ax l ax n s iy0))
("equivalent" nil (ih0 k w ih1 v ax l ax n t))
("equivalents" nil (ih0 k w ih1 v ax l ax n t s))
("equivocal" nil (ih0 k w ih1 v ax k ax l))
("er" nil (er0))
("era" nil (eh1 r ax))
("eradicate" nil (ih0 r ae1 d ax k ey1 t))
("eradicated" nil (ih0 r ae1 d ax k ey1 t ih0 d))
("eradicating" nil (ih0 r ae1 d ax k ey1 t ih0 ng))
("eradication" nil (ih0 r ae1 d ax k ey1 sh ax n))
("eralp" nil (eh1 r ao0 l p))
("eramo" nil (eh0 r aa1 m ow0))
("eranthe" nil (eh1 r ax n th))
("eras" nil (ih1 r ax z))
("erasable" nil (ih0 r ey1 s ax b ax l))
("erase" nil (ih0 r ey1 s))
("erased" nil (ih0 r ey1 s t))
("eraser" nil (ih0 r ey1 s er0))
("erasers" nil (ih0 r ey1 s er0 z))
("erases" nil (ih0 r ey1 s ih0 z))
("erasing" nil (ih0 r ey1 s ih0 ng))
("erasmus" nil (ih0 r ae1 z m ax s))
("erastus" nil (ih0 r ae1 s t ax s))
("erawan" nil (eh1 r ax w aa1 n))
("erazo" nil (eh0 r aa1 z ow0))
("erb" nil (er1 b))
("erbacher" nil (er1 b aa0 k er0))
("erbamont" nil (er1 b ax m aa1 n t))
("erbe" nil (er1 b))
("erber" nil (er1 b er0))
("erbes" nil (er1 b z))
("erburu" nil (er0 b uh1 r uw0))
("erby" nil (er1 b iy0))
("erceg" nil (er1 s ih0 g))
("erck" nil (er1 k))
("ercole" nil (er0 k ow1 l iy0))
("ercros" nil (er1 k r ow0 z))
("erda" nil (eh1 r d ax))
("erdahl" nil (er1 d aa0 l))
("erdman" nil (er1 d m ax n))
("erdmann" nil (er1 d m ax n))
("erdos" nil (er1 d ow0 z))
("erdrich" nil (er1 d r ih0 k))
("erect" nil (ih0 r eh1 k t))
("erected" nil (ih0 r eh1 k t ax d))
("erecting" nil (ih0 r eh1 k t ih0 ng))
("erection" nil (ih0 r eh1 k sh ax n))
("erections" nil (ih0 r eh1 k sh ax n z))
("erects" nil (ih0 r eh1 k t s))
("erena" nil (er0 eh1 n ax))
("erensel" nil (eh1 r ax n s eh0 l))
("ergle" nil (er1 g ax l))
("ergo" nil (er1 g ow0))
("ergonomic" nil (er1 g ax n aa1 m ih0 k))
("ergonomics" nil (er1 g ax n aa1 m ih0 k s))
("ergot" nil (er1 g ax t))
("ergotism" nil (er1 g ax t ih1 z ax m))
("ergots" nil (er1 g ax t s))
("erhard" nil (eh1 r hh aa0 r d))
("erhardt" nil (er1 hh aa0 r t))
("erhart" nil (er1 hh aa0 r t))
("eric" nil (eh1 r ih0 k))
("erica" nil (eh1 r ih0 k ax))
("erich" nil (eh1 r ih0 k))
("erichsen" nil (eh1 r ih0 k s ax n))
("erick" nil (eh1 r ih0 k))
("ericksen" nil (eh1 r ih0 k s ax n))
("erickson" nil (eh1 r ih0 k s ax n))
("ericson" nil (eh1 r ih0 k s ax n))
("ericsson" nil (eh1 r ih0 k s ax n))
("eridania" nil (eh1 r ih0 d ae1 n iy0 ax))
("erie" nil (ih1 r iy0))
("erik" nil (eh1 r ih0 k))
("erika" nil (eh1 r ih0 k ax))
("eriksen" nil (eh1 r ih0 k s ax n))
("erikson" nil (eh1 r ih0 k s ax n))
("eriksson" nil (eh1 r ih0 k s ax n))
("erin" nil (eh1 r ih0 n))
("erina" nil (er0 iy1 n ax))
("erion" nil (eh0 r iy0 ao1 n))
("erisa" nil (eh0 r iy1 s ax))
("erisman" nil (eh1 r ih0 s m ax n))
("eritrea" nil (eh1 r ih0 t r iy1 ax))
("eritrean" nil (eh1 r ih0 t r iy1 ax n))
("eritreans" nil (eh1 r ih0 t r iy1 n z))
("erk" nil (er1 k))
("erker" nil (er1 k er0))
("erkkila" nil (er1 k ih0 l ax))
("erl" nil (er1 l))
("erlach" nil (er1 l aa1 k))
("erland" nil (er1 l ax n d))
("erlandson" nil (er1 l ax n d s ax n))
("erlangen" nil (er0 l ae1 ng g ax n))
("erlanger" nil (eh1 r l ae0 ng er0))
("erle" nil (ao1 r ax l))
("erlene" nil (er1 l iy0 n))
("erler" nil (er1 l er0))
("erlich" nil (er1 l ih0 k))
("erlick" nil (er1 l ih0 k))
("erline" nil (er1 l ay0 n))
("erling" nil (er1 l ih0 ng))
("erly" nil (er1 l iy0))
("erma" nil (er1 m ax))
("erman" nil (er1 m ax n))
("ermer" nil (er1 m er0))
("ermin" nil (er1 m ih0 n))
("ermina" nil (er0 m iy1 n ax))
("ermine" nil (er1 m ax n))
("erminia" nil (er0 m iy1 n iy0 ax))
("erminie" nil (er1 m ih0 n iy0))
("ermis" nil (er1 m ih0 s))
("erna" nil (er1 n ax))
("ernaline" nil (er1 n ax l ay0 n))
("erne" nil (er1 n))
("ernest" nil (er1 n ax s t))
("ernestine" nil (er1 n ih0 s t iy1 n))
("ernesto" nil (er0 n eh1 s t ow0))
("ernests" nil (er1 n ax s t s))
("erney" nil (er1 n iy0))
("ernie" nil (er1 n iy0))
("erno" nil (eh1 r n ow0))
("ernsberger" nil (er1 n z b er0 g er0))
("ernst" nil (er1 n s t))
("ernster" nil (er1 n s t er0))
("erny" nil (er1 n iy0))
("ero" nil (ih1 r ow0))
("erode" nil (ih0 r ow1 d))
("eroded" nil (ih0 r ow1 d ax d))
("erodes" nil (ih0 r ow1 d z))
("erodible" nil (ih0 r ow1 d ax b ax l))
("eroding" nil (ih0 r ow1 d ih0 ng))
("eroh" nil (eh1 r ow0))
("eroica" nil (er0 oy1 k ax))
("eros" nil (ih1 r aa0 s))
("erosion" nil (ih0 r ow1 zh ax n))
("erosional" nil (ih0 r ow1 zh ax n ax l))
("erosive" nil (ih0 r ow1 s ih0 v))
("erotic" nil (ih0 r aa1 t ih0 k))
("eroticism" nil (er0 aa1 t ih0 s ih1 z ax m))
("erpelding" nil (er1 p ih0 l d ih0 ng))
("erps" nil (er1 p s))
("err" nil (eh1 r))
("erra" nil (eh1 r ax))
("errand" nil (eh1 r ax n d))
("errands" nil (eh1 r ax n d z))
("errant" nil (eh1 r ax n t))
("erratic" nil (ih0 r ae1 t ih0 k))
("erratically" nil (eh0 r ae1 t ih0 k l iy0))
("erred" nil (eh1 r d))
("errett" nil (eh1 r ih0 t))
("erria" nil (eh1 r iy0 ax))
("errickson" nil (eh1 r ih0 k s ax n))
("errico" nil (er0 iy1 k ow0))
("errington" nil (eh1 r ih0 ng t ax n))
("errol" nil (eh1 r ax l))
("erroneous" nil (eh0 r ow1 n iy0 ax s))
("erroneously" nil (eh0 r ow1 n iy0 ax s l iy0))
("error" nil (eh1 r er0))
("errors" nil (eh1 r er0 z))
("errs" nil (eh1 r z))
("ers" nil (er1 z))
("ersatz" nil (eh1 r s aa1 t s))
("ershad" nil (er1 sh ae1 d))
("erskin" nil (er1 s k ih0 n))
("erskine" nil (er1 s k ay1 n))
("erstwhile" nil (er1 s t w ay1 l))
("erte" nil (er1 t))
("ertel" nil (er1 t ax l))
("ertha" nil (er1 dh ax))
("ertl" nil (er1 t ax l))
("ertle" nil (er1 t ax l))
("ertman" nil (er1 t m ax n))
("ertz" nil (er1 t s))
("erudite" nil (eh1 r ax d ay1 t))
("erudition" nil (eh1 r ax d ih1 sh ax n))
("erupt" nil (ih0 r ah1 p t))
("erupted" nil (ih0 r ah1 p t ax d))
("erupting" nil (ih0 r ah1 p t ih0 ng))
("eruption" nil (ih0 r ah1 p sh ax n))
("eruptions" nil (ih0 r ah1 p sh ax n z))
("eruptive" nil (ih0 r ah1 p t ih0 v))
("erupts" nil (ih0 r ah1 p t s))
("erven" nil (er1 v ax n))
("ervin" nil (er1 v ih0 n))
("erving" nil (er1 v ih0 ng))
("erway" nil (er1 w ey0))
("erwin" nil (er1 w ih0 n))
("erwina" nil (er0 v ay1 n ax))
("erxleben" nil (er0 k s l eh1 b ax n))
("erythropoietin" nil (eh0 r ih1 th r ax p oy1 t ih0 n))
("es" nil (eh1 s))
("esarey" nil (eh1 s er0 iy0))
("esau" nil (iy1 s ao1))
("esbenshade" nil (eh1 s b ih0 n sh ax d))
("esber" nil (eh1 s b er0))
("escada" nil (eh1 s k aa1 d ax))
("escalante" nil (eh0 s k aa0 l aa1 n t iy0))
("escalate" nil (eh1 s k ax l ey1 t))
("escalated" nil (eh1 s k ax l ey1 t ih0 d))
("escalates" nil (eh1 s k ax l ey1 t s))
("escalating" nil (eh1 s k ax l ey1 t ih0 ng))
("escalation" nil (eh1 s k ax l ey1 sh ax n))
("escalator" nil (eh1 s k ax l ey1 t er0))
("escalators" nil (eh1 s k ax l ey1 t er0 z))
("escalera" nil (eh0 s k aa0 l eh1 r ax))
("escalona" nil (eh0 s k aa0 l ow1 n ax))
("escambia" nil (eh1 s k ae1 m b iy0 ax))
("escamez" nil (eh0 s k aa1 m eh0 z))
("escamilla" nil (eh0 s k aa0 m ih1 l ax))
("escanaba" nil (eh1 s k ax n aa1 b ax))
("escandon" nil (ih0 s k ae1 n d ax n))
("escapade" nil (eh1 s k ax p ey1 d))
("escapades" nil (eh1 s k ax p ey1 d z))
("escape" nil (ih0 s k ey1 p))
("escaped" nil (ih0 s k ey1 p t))
("escapees" nil (ih0 s k ey1 p iy1 z))
("escapement" nil (ih0 s k ey1 p m ax n t))
("escapes" nil (ih0 s k ey1 p s))
("escaping" nil (ih0 s k ey1 p ih0 ng))
("escapism" nil (ih0 s k ey1 p ih1 z ax m))
("escarcega" nil (eh0 s k aa0 r ch eh1 g ax))
("escareno" nil (eh0 s k aa0 r eh1 n ow0))
("escarpment" nil (eh0 s k aa1 r p m ax n t))
("escarpments" nil (eh0 s k aa1 r p m ax n t s))
("esch" nil (eh1 sh))
("eschatological" nil (eh1 s k ax t ax l aa1 jh ih0 k ax l))
("eschbach" nil (eh1 sh b aa1 k))
("esche" nil (eh1 sh))
("eschen" nil (eh1 sh ax n))
("eschenbach" nil (eh1 sh ih0 n b aa0 k))
("eschenburg" nil (eh1 sh ax n b er0 g))
("escher" nil (eh1 sh er0))
("eschete" nil (eh1 sh iy0 t))
("eschew" nil (eh0 s ch uw1))
("eschewed" nil (eh1 sh uw1 d))
("eschewing" nil (eh1 sh uw1 ih0 ng))
("eschews" nil (eh0 s ch uw1 z))
("eschmann" nil (eh1 sh m ax n))
("esco" nil (eh1 s k ow0))
("escobar" nil (eh1 s k ow0 b aa1 r))
("escobedo" nil (eh0 s k ow0 b ey1 d ow0))
("escoe" nil (ih0 s k ow1))
("escondido" nil (eh1 s k aa0 n d iy1 d ow0))
("escort" v (eh0 s k ao1 r t))
("escort" n (eh1 s k ao0 r t))
("escorted" nil (eh0 s k ao1 r t ih0 d))
("escorting" nil (eh1 s k ao0 r t ih0 ng))
("escorts" n (eh1 s k ao0 r t s))
("escorts" v (eh0 s k ao1 r t s))
("escoto" nil (eh0 s k ow1 t ow0))
("escott" nil (eh1 s k ax t))
("escrow" v (eh0 s k r ow1))
("escrow" n (eh1 s k r ow0))
("escrowed" nil (eh1 s k r ow0 d))
("escudero" nil (eh0 s k uw0 d eh1 r ow0))
("escudo" nil (eh0 s k uw1 d ow0))
("escudos" nil (eh0 s k uw1 d ow0 z))
("escue" nil (ey1 s k y uw0))
("esh" nil (eh1 sh))
("esham" nil (eh1 sh ax m))
("eshbach" nil (eh1 sh b aa1 k))
("eshbaugh" nil (ih0 sh b ao1))
("eshelman" nil (eh1 sh ax l m ax n))
("eshleman" nil (eh1 sh ax l m ax n))
("eskandarian" nil (eh1 s k ax n d eh1 r iy0 ax n))
("eskelson" nil (eh1 s k ih0 l s ax n))
("esker" nil (eh1 s k er0))
("eskew" nil (eh1 s k y uw0))
("eskey" nil (eh1 s k iy0))
("eskimo" nil (eh1 s k ax m ow1))
("eskimos" nil (eh1 s k ax m ow1 z))
("eskin" nil (ih0 s k ih1 n))
("eskridge" nil (eh1 s k r ih1 jh))
("esler" nil (eh1 s ax l er0))
("eslick" nil (eh1 s l ih0 k))
("eslinger" nil (eh1 s ax l ih0 ng er0))
("esmark" nil (eh1 s m aa1 r k))
("esme" nil (eh1 z m))
("esmerelda" nil (eh0 s m er0 eh1 l d ax))
("esmond" nil (eh1 z m ax n d))
("esophagus" nil (ih0 s aa1 f ax g ax s))
("esoteric" nil (eh1 s ax t eh1 r ih0 k))
("espada" nil (ey0 s p aa1 d ax))
("espalier" nil (eh0 s p ae1 l y er0))
("espana" nil (eh0 s p ae1 n ax))
("espanol" nil (eh1 s p aa0 n y ow1 l))
("espanola" nil (eh1 s p ax n ow1 l ax))
("esparza" nil (eh0 s p aa1 r z ax))
("espe" nil (eh1 s p))
("especially" nil (ih0 s p eh1 sh l iy0))
("espectador" nil (eh0 s p eh1 k t ax d ao1 r))
("espejo" nil (ey0 s p ey1 y ow0))
("espeland" nil (eh1 s p ih0 l ax n d))
("espenschied" nil (eh1 s p ih0 n sh iy0 d))
("espenshade" nil (eh1 s p ih0 n sh ax d))
("esper" nil (eh1 s p er0))
("esperanto" nil (eh1 s p er0 ae1 n t ow0))
("espeseth" nil (eh1 s p ih0 s ih0 th))
("espey" nil (eh1 s p iy0))
("espinal" nil (ey0 s p iy1 n ax l))
("espino" nil (ey0 s p iy1 n ow0))
("espinola" nil (eh0 s p iy0 n ow1 l ax))
("espinosa" nil (eh0 s p ih0 n ow1 z ax))
("espinoza" nil (ey0 s p iy0 n ow1 z ax))
("espionage" nil (eh1 s p iy0 ax n aa0 jh))
("espirito" nil (eh1 s ih0 r iy1 t ow0))
("espiritu" nil (eh0 s p ih0 r iy1 ch uw0))
("espitia" nil (eh0 s p iy1 sh ax))
("esplanade" nil (eh1 s p l ax n aa1 d))
("esplin" nil (eh1 s p l ih0 n))
("esposito" nil (eh0 s p ax z iy1 t ow0))
("espouse" nil (ih0 s p aw1 z))
("espoused" nil (ih0 s p aw1 z d))
("espouses" nil (ih0 s p aw1 z ih0 z))
("espousing" nil (ih0 s p aw1 z ih0 ng))
("espresso" nil (eh1 s p r eh1 s ow0))
("esprit" nil (eh0 s p r iy1))
("espy" nil (eh1 s p iy0))
("esqueda" nil (eh0 s k w ey1 d ax))
("esquer" nil (ih0 s k er1))
("esquibel" nil (ey0 s k w iy0 b eh1 l))
("esquire" nil (eh1 s k w ay1 r))
("esquivel" nil (ey0 s k w iy0 v eh1 l))
("esrey" nil (eh1 s r iy0))
("ess" nil (eh1 s))
("essa" nil (eh1 s ax))
("essary" nil (eh1 s eh0 r iy0))
("essay" nil (eh0 s ey1))
("essayist" nil (eh0 s ey1 ih0 s t))
("essays" nil (eh0 s ey1 z))
("esse" nil (eh1 s))
("esselman" nil (eh1 s ax l m ax n))
("esselte" nil (eh0 s eh1 l t iy0))
("essen" nil (eh1 s ax n))
("essenburg" nil (eh1 s ax n b er0 g))
("essence" nil (eh1 s ax n s))
("essential" nil (ih0 s eh1 n sh ax l))
("essentially" nil (ih0 s eh1 n sh ax l iy0))
("essentials" nil (eh0 s eh1 n ch ax l z))
("esser" nil (eh1 s er0))
("esses" nil (eh1 s ih0 z))
("essex" nil (eh1 s ih0 k s))
("essick" nil (eh1 s ih0 k))
("essie" nil (eh1 s iy0))
("essig" nil (eh1 s ih0 g))
("esslinger" nil (eh1 s ax l ih0 ng er0))
("essman" nil (eh1 s m ax n))
("essner" nil (eh1 s n er0))
("esso" nil (eh1 s ow0))
("esson" nil (eh1 s ax n))
("est" nil (ax s t ey1 t))
("esta" nil (eh1 s t ax))
("establish" nil (ih0 s t ae1 b l ih0 sh))
("established" nil (ih0 s t ae1 b l ih0 sh t))
("establishes" nil (ih0 s t ae1 b l ih0 sh ih0 z))
("establishing" nil (ih0 s t ae1 b l ih0 sh ih0 ng))
("establishment" nil (ih0 s t ae1 b l ih0 sh m ax n t))
("establishments" nil (eh0 s t ae1 b l ih0 sh m ax n t s))
("estabrook" nil (eh1 s t ax b r uh1 k))
("estabrooks" nil (eh1 s t ax b r uh0 k s))
("estai" nil (eh1 s t ey0))
("estate" nil (ih0 s t ey1 t))
("estates" nil (ih0 s t ey1 t s))
("este" nil (eh1 s t))
("esteban" nil (eh1 s t ax b aa0 n))
("estee" nil (eh1 s t iy0))
("esteem" nil (ih0 s t iy1 m))
("esteemed" nil (ih0 s t iy1 m d))
("estel" nil (eh1 s t ax l))
("estell" nil (eh1 s t ax l))
("estella" nil (eh0 s t eh1 l ax))
("estelle" nil (eh0 s t eh1 l))
("estenson" nil (eh1 s t ih0 n s ax n))
("estep" nil (eh1 s t ih0 p))
("estepp" nil (eh1 s t ih0 p))
("ester" nil (eh1 s t er0))
("esterline" nil (eh1 s t er0 l ay1 n))
("esterly" nil (eh1 s t er0 l iy0))
("esters" nil (eh1 s t er0 z))
("estes" nil (eh1 s t iy0 z))
("esteve" nil (eh1 s t ih0 v))
("esteves" nil (ey0 s t ey1 v eh0 s))
("estevez" nil (ey0 s t ey1 v eh0 z))
("estey" nil (eh1 s t iy0))
("esther" nil (eh1 s t er0))
("estill" nil (ey0 s t iy1 l))
("estimable" nil (eh1 s t ax m ax b ax l))
("estimate" n (eh1 s t ax m ax t))
("estimate" v (eh1 s t ax m ey1 t))
("estimated" nil (eh1 s t ax m ey1 t ax d))
("estimates" n (eh1 s t ax m ax t s))
("estimates" v (eh1 s t ax m ey0 t s))
("estimating" nil (eh1 s t ax m ey1 t ih0 ng))
("estimation" nil (eh1 s t ax m ey1 sh ax n))
("estimators" nil (ax s t ih1 m ax t er0 z))
("estis" nil (eh1 s t ih0 s))
("estle" nil (eh1 s ax l))
("estock" nil (eh1 s t aa0 k))
("estok" nil (ih0 s t aa1 k))
("estonia" nil (eh0 s t ow1 n iy0 ax))
("estonian" nil (eh0 s t ow1 n iy0 ax n))
("estonians" nil (eh0 s t ow1 n iy0 ax n z))
("estrada" nil (eh0 s t r aa1 d ax))
("estranged" nil (eh0 s t r ey1 n jh d))
("estrangement" nil (ih0 s t r ey1 n jh m ax n t))
("estrella" nil (eh1 s t r eh1 l ax))
("estrich" nil (eh1 s t r ih0 k))
("estridge" nil (ih0 s t r ih1 jh))
("estrin" nil (eh1 s t r ih0 n))
("estrogen" nil (eh1 s t r ax jh ax n))
("estrogens" nil (eh1 s t r ax jh ih0 n z))
("estrous" nil (eh1 s t r ax s))
("estrus" nil (eh1 s t r ax s))
("estuaries" nil (eh1 s ch uw0 eh1 r iy0 z))
("estuary" nil (eh1 s ch uw0 eh1 r iy0))
("esty" nil (eh1 s t iy0))
("esworthy" nil (eh1 s w er0 dh iy0))
("et" nil (eh1 t))
("etablissements" nil (eh1 t ae0 b l ih1 s ax m aa1 n t s))
("etc" nil (eh1 t s eh1 t er0 ax))
("etcetera" nil (eh1 t s eh1 t er0 ax))
("etch" nil (eh1 ch))
("etched" nil (eh1 ch t))
("etches" nil (eh1 ch ih0 z))
("etcheverry" nil (eh1 ch ih0 v eh0 r iy0))
("etching" nil (eh1 ch ih0 ng))
("etchings" nil (eh1 ch ih0 ng z))
("etchison" nil (eh1 ch ih0 s ax n))
("eternal" nil (ih0 t er1 n ax l))
("eternally" nil (ih0 t er1 n ax l iy0))
("eternit" nil (eh1 t er0 n ih0 t))
("eternity" nil (ih0 t er1 n ax t iy0))
("etess" nil (eh1 t eh1 s))
("ethan" nil (iy1 th ax n))
("ethane" nil (eh1 th ey1 n))
("ethanol" nil (eh1 th ax n ao1 l))
("ethel" nil (eh1 th ax l))
("ethelbert" nil (eh1 th ih0 l b er0 t))
("ethelda" nil (ih0 th eh1 l d ax))
("ethelinda" nil (eh0 th eh0 l iy1 n d ax))
("etheline" nil (eh1 th ih0 l ay1 n))
("etheljean" nil (eh1 th ih0 l y ax n))
("ethelyn" nil (eh1 th ih0 l ih0 n))
("ethene" nil (eh1 th iy1 n))
("ether" nil (iy1 th er0))
("ethereal" nil (ih0 th ih1 r iy0 ax l))
("etheredge" nil (eh1 th er0 eh0 jh))
("etheridge" nil (eh1 th er0 ih1 jh))
("etherington" nil (eh1 dh er0 ih0 ng t ax n))
("ethernet" nil (eh1 th er0 n eh1 t))
("ethers" nil (iy1 th er0 z))
("etherton" nil (ih0 th er1 t ax n))
("ethic" nil (eh1 th ih0 k))
("ethical" nil (eh1 th ax k ax l))
("ethically" nil (eh1 th ih0 k ax l iy0))
("ethicist" nil (eh1 th ih0 s ih0 s t))
("ethicists" nil (eh1 th ih0 s ih0 s t s))
("ethicon" nil (eh1 th ih0 k aa1 n))
("ethics" nil (eh1 th ih0 k s))
("ethier" nil (eh1 th iy0 er0))
("ethington" nil (eh1 th ih0 ng t ax n))
("ethiopia" nil (iy1 th iy0 ow1 p iy0 ax))
("ethiopian" nil (iy1 th iy0 ow1 p iy0 ax n))
("ethiopians" nil (iy1 th iy0 ow1 p iy0 ax n z))
("ethnic" nil (eh1 th n ih0 k))
("ethnically" nil (eh1 th n ih0 k l iy0))
("ethnicity" nil (eh0 th n ih1 s ih0 t iy0))
("ethnics" nil (eh1 th n ih0 k s))
("ethnocentric" nil (eh1 th n ow0 s eh1 n t r ih0 k))
("ethnocentrism" nil (eh1 th n ow0 s eh1 n t r ih0 z ax m))
("ethnographic" nil (eh1 th n ax g r ae1 f ih0 k))
("ethnology" nil (eh0 th n aa1 l ax jh iy0))
("ethnos" nil (eh1 th n ow0 s))
("ethology" nil (ih0 th aa1 l ax jh iy0))
("ethos" nil (iy1 th aa0 s))
("ethridge" nil (ih0 th r ih1 jh))
("ethyl" nil (eh1 th ax l))
("ethylene" nil (eh1 th ax l iy1 n))
("ethyol" nil (eh1 th ay0 aa0 l))
("etienne" nil (eh1 t iy0 eh0 n))
("etiologies" nil (iy1 t iy0 aa1 l ax jh iy0 z))
("etiology" nil (iy1 t iy0 aa1 l ax jh iy0))
("etiquette" nil (eh1 t ax k ax t))
("etling" nil (eh1 t l ih0 ng))
("eton" nil (iy1 t ax n))
("etops" nil (iy1 t aa0 p s))
("etruscan" nil (ih0 t r ah1 s k ax n))
("etruscans" nil (ih0 t r ah1 s k ax n z))
("etsu" nil (eh1 t s uw0))
("etta" nil (eh1 t ax))
("etter" nil (eh1 t er0))
("etters" nil (eh1 t er0 z))
("ettie" nil (eh1 t iy0))
("ettinger" nil (eh1 t ih0 ng er0))
("ettlinger" nil (eh1 t l ih0 ng er0))
("etty" nil (eh1 t iy0))
("etudes" nil (ey1 t uw1 d z))
("etymology" nil (eh1 t ax m aa1 l ax jh iy0))
("etzel" nil (eh1 t z ax l))
("etzioni" nil (eh1 t s iy1 ow1 n iy0))
("etzkorn" nil (eh1 t s k er0 n))
("etzler" nil (eh1 t s l er0))
("euan" nil (y uw1 ax n))
("eubank" nil (y uw1 b ae0 ng k))
("eubanks" nil (y uw1 b ae0 ng k s))
("eucalypti" nil (y uw1 k ax l ih1 p t ay0))
("eucalyptus" nil (y uw1 k ax l ih1 p t ax s))
("eucharist" nil (y uw1 k er0 ax s t))
("euchre" nil (y uw1 k er0))
("euchred" nil (y uw1 k er0 d))
("euclea" nil (y uw1 k l iy0 ax))
("euclid" nil (y uw1 k l ih0 d))
("eudocia" nil (y uw0 d ow1 ch ax))
("eudora" nil (y uw0 d ao1 r ax))
("eudosia" nil (y uw0 d ow1 s iy0 ax))
("eudoxia" nil (y uw1 d aa1 k s iy0 ax))
("eudy" nil (y uw1 d iy0))
("eugene" nil (y uw0 jh iy1 n))
("eugenia" nil (y uw1 jh iy1 n iy0 ax))
("eugenics" nil (y uw0 jh eh1 n ih0 k s))
("eugenio" nil (y uw1 jh iy1 n iy0 ow0))
("eula" nil (y uw1 l ax))
("euler" nil (oy1 l er0))
("eulich" nil (y uw1 l ih0 k))
("eulogies" nil (y uw1 l ax jh iy0 z))
("eulogy" nil (y uw1 l ax jh iy0))
("eunice" nil (y uw1 n ax s))
("euphemia" nil (y uw0 f iy1 m iy0 ax))
("euphemism" nil (y uw1 f ax m ih1 z ax m))
("euphemisms" nil (y uw1 f ax m ih1 z ax m z))
("euphemistic" nil (y uw1 f ax m ih1 s t ih0 k))
("euphemistically" nil (y uw1 f ax m ih1 s t ih0 k l iy0))
("euphoria" nil (y uw0 f ao1 r iy0 ax))
("euphoric" nil (y uw0 f aa1 r ih0 k))
("euphrates" nil (y uw0 f r ey1 t iy0 z))
("eurasia" nil (y uh0 r ey1 zh ax))
("eurasian" nil (y uh0 r ey1 zh ih0 n))
("eure" nil (er1))
("eureka" nil (y uh0 r iy1 k ax))
("eurest" nil (y uh1 r eh0 s t))
("eurich" nil (oy1 r ih0 k))
("euro" nil (y uw1 r ow0))
("eurobond" nil (y uw1 r ow0 b aa1 n d))
("eurobonds" nil (y uw1 r ow0 b aa1 n d z))
("eurocom" nil (y uw1 r ax k ax m))
("eurocommercial" nil (y uw1 r ow0 k ax m er1 sh ax l))
("eurocopter" nil (y uh1 r ow0 k aa1 p t er0))
("eurocrats" nil (y uw1 r ax k r ae0 t s))
("eurodebentures" nil (y uw1 r ow0 d ax b eh1 n ch er0 z))
("eurodebt" nil (y uw1 r ow0 d eh1 t))
("eurodeposits" nil (y uw1 r ow0 d ih0 p aa1 z ih0 t s))
("eurodisney" nil (y uw1 r ow0 d ih1 z n iy0))
("eurodollar" nil (y uw1 r ow0 d aa1 l er0))
("eurodollars" nil (y uw1 r ow0 d aa1 l er0 z))
("eurofighter" nil (y uw1 r ow0 f ay1 t er0))
("eurofima" nil (y uw1 r ax f iy1 m ax))
("euromark" nil (y uw1 r ax m aa0 r k))
("euromarket" nil (y uw1 r ow0 m aa1 r k ih0 t))
("euromarkets" nil (y uw1 r ow0 m aa1 r k ih0 t s))
("euromissiles" nil (y uw1 r ow1 m ih0 s iy1 l eh0 z))
("euromobiliare" nil (y uw1 r ow0 m ow0 b ih0 l eh1 r))
("euromoney" nil (y uw1 r ow0 m ah1 n iy0))
("euronotes" nil (y uw1 r ax n ow1 t s))
("europa" nil (y uh0 r ow1 p ax))
("europe" nil (y uh1 r ax p))
("european" nil (y uh1 r ax p iy1 ax n))
("europeans" nil (y uh1 r ax p iy1 ax n z))
("europeenne" nil (y uw1 r ow0 p iy1 n))
("europewide" nil (y uh1 r ax p w ay1 d))
("europhile" nil (y uh1 r ow0 f ay0 l))
("europhiles" nil (y uh1 r ow0 f ay0 l z))
("europium" nil (y uh0 r ow1 p iy0 ax m))
("eurosid" nil (y uw1 r ax s ih0 d))
("eurostar" nil (y uh1 r ow0 s t aa1 r))
("eurostat" nil (y uw1 r ao1 s t ae0 t))
("eurosterling" nil (y uw1 r ow0 s t er1 l ih0 ng))
("eurotunnel" nil (y uw1 r ow0 t ah1 n ax l))
("euroyen" nil (y uw1 r ow0 y eh1 n))
("eury" nil (y uw1 r iy0))
("eusebio" nil (y uw1 s iy1 b iy0 ow0))
("eustace" nil (y uw1 s t ax s))
("eustachian" nil (y uw0 s t ey1 k iy0 ax n))
("eustacia" nil (y uw0 s t aa1 ch ax))
("eustice" nil (y uw1 s t ih0 s))
("eustis" nil (y uw1 s t ax s))
("euthanasia" nil (y uw1 th ax n ey1 zh ax))
("eutsey" nil (y uw1 t s iy0))
("eutsler" nil (y uw1 t s l er0))
("ev" nil (eh1 v))
("eva" nil (ey1 v ax))
("evacuate" nil (ih0 v ae1 k y ax ey1 t))
("evacuated" nil (ih0 v ae1 k y ax w ey1 t ih0 d))
("evacuating" nil (ih0 v ae1 k y ax w ey1 t ih0 ng))
("evacuation" nil (ih0 v ae1 k y ax w ey1 sh ax n))
("evacuations" nil (ih0 v ae1 k y ax w ey1 sh ax n z))
("evacuee" nil (ih0 v ae1 k y uw0 iy0))
("evacuees" nil (ih0 v ae1 k y uw0 iy0 z))
("evade" nil (ih0 v ey1 d))
("evaded" nil (ih0 v ey1 d ih0 d))
("evaders" nil (ih0 v ey1 d er0 z))
("evades" nil (ih0 v ey1 d z))
("evading" nil (ih0 v ey1 d ih0 ng))
("evadne" nil (ih0 v eh1 d n iy0))
("evald" nil (eh1 v ax l d))
("evaleen" nil (eh1 v ax l iy1 n))
("evaluate" nil (ih0 v ae1 l y uw0 ey1 t))
("evaluated" nil (ih0 v ae1 l y uw0 ey1 t ax d))
("evaluates" nil (ih0 v ae1 l y uw0 ey1 t s))
("evaluating" nil (ih0 v ae1 l y uw0 ey1 t ih0 ng))
("evaluation" nil (ih0 v ae1 l y uw0 ey1 sh ax n))
("evaluations" nil (ih0 v ae1 l y uw0 ey1 sh ax n z))
("evan" nil (eh1 v ax n))
("evancho" nil (eh0 v aa1 n k ow0))
("evanescent" nil (eh1 v ax n eh1 s ax n t))
("evangelical" nil (iy1 v ae0 n jh eh1 l ih0 k ax l))
("evangelicals" nil (iy1 v ae1 n jh eh1 l ih0 k ax l z))
("evangeline" nil (ih0 v ae1 n jh ih0 l ay1 n))
("evangelism" nil (ih0 v ae1 n jh ax l ih1 z ax m))
("evangelist" nil (ih0 v ae1 n jh ax l ax s t))
("evangelista" nil (eh0 v aa0 ng g eh0 l iy1 s t ax))
("evangelists" nil (ih0 v ae1 n jh ax l ih0 s t s))
("evangelize" nil (ih0 v ae1 n jh ax l ay1 z))
("evanko" nil (ih0 v ae1 ng k ow0))
("evanoff" nil (eh1 v ax n ao0 f))
("evans" nil (eh1 v ax n z))
("evanson" nil (eh1 v ax n s ax n))
("evanston" nil (eh1 v ax n s t ax n))
("evansville" nil (eh1 v ax n z v ih1 l))
("evaporate" nil (ih0 v ae1 p er0 ey1 t))
("evaporated" nil (ih0 v ae1 p er0 ey1 t ax d))
("evaporates" nil (ih0 v ae1 p er0 ey1 t s))
("evaporating" nil (ih0 v ae1 p er0 ey1 t ih0 ng))
("evaporation" nil (ih0 v ae1 p er0 ey1 sh ax n))
("evaporator" nil (ih0 v ae1 p er0 ey1 t er0))
("evarts" nil (eh1 v er0 t s))
("evasion" nil (ih0 v ey1 zh ax n))
("evasions" nil (ih0 v ey1 zh ax n z))
("evasive" nil (ih0 v ey1 s ih0 v))
("evatt" nil (eh1 v ax t))
("eve" nil (iy1 v))
("evel" nil (eh1 v ax l))
("eveland" nil (iy1 v l ax n d))
("eveleth" nil (eh1 v ih0 l ih0 th))
("evelina" nil (eh1 v eh0 l iy1 n ax))
("eveline" nil (eh1 v ax l ih0 n))
("evelyn" nil (eh1 v ax l ax n))
("even" nil (iy1 v ax n))
("evendale" nil (eh1 v ax n d ey1 l))
("evened" nil (iy1 v ax n d))
("evenhanded" nil (iy1 v ax n hh ae1 n d ih0 d))
("evenhandedly" nil (eh1 v ax n hh ae1 n d ih0 d l iy0))
("evening" nil (iy1 v n ih0 ng))
("evenings" nil (iy1 v n ih0 ng z))
("evenly" nil (iy1 v ax n l iy0))
("evenness" nil (iy1 v ax n n ax s))
("evens" nil (iy1 v ax n z))
("evensen" nil (eh1 v ih0 n s ax n))
("evensky" nil (iy1 v eh1 n s k iy0))
("evenson" nil (eh1 v ih0 n s ax n))
("event" nil (ih0 v eh1 n t))
("eventful" nil (ih0 v eh1 n t f ax l))
("events" nil (ih0 v eh1 n t s))
("eventual" nil (ax v eh1 n ch uw0 ax l))
("eventuality" nil (ax v eh1 n ch uw0 ae1 l ih0 t iy0))
("eventually" nil (ih0 v eh1 n ch ax w ax l iy0))
("ever" nil (eh1 v er0))
("everage" nil (eh1 v er0 ih0 jh))
("everard" nil (eh1 v er0 er0 d))
("everbach" nil (eh1 v er0 b aa1 k))
("everding" nil (eh1 v er0 d ih0 ng))
("eveready" nil (eh1 v er0 r eh1 d iy0))
("evered" nil (eh1 v er0 d))
("everest" nil (eh1 v er0 ax s t))
("everett" nil (eh1 v er0 ax t))
("everette" nil (eh1 v er0 eh1 t))
("everetts" nil (eh1 v er0 ax t s))
("everex" nil (eh1 v er0 ax k s))
("everglades" nil (eh1 v er0 g l ey1 d z))
("evergood" nil (eh1 v er0 g uh1 d))
("evergreen" nil (eh1 v er0 g r iy1 n))
("evergreens" nil (eh1 v er0 g r iy1 n z))
("everhardt" nil (eh1 v er0 hh aa1 r t))
("everhart" nil (eh1 v er0 hh aa1 r t))
("everidge" nil (eh1 v er0 ih1 jh))
("everingham" nil (eh1 v er0 ih0 ng hh ae1 m))
("everington" nil (eh1 v er0 ih1 ng t ax n))
("everitt" nil (eh1 v er0 ih0 t))
("everlasting" nil (eh1 v er0 l ae1 s t ih0 ng))
("everlastings" nil (eh1 v er0 l ae1 s t ih0 ng z))
("everley" nil (eh1 v er0 l iy0))
("everling" nil (eh1 v er0 l ih0 ng))
("everly" nil (eh1 v er0 l iy0))
("everman" nil (eh1 v er0 m ax n))
("evermore" nil (eh1 v er0 m ao1 r))
("evers" nil (eh1 v er0 z))
("eversman" nil (eh1 v er0 s m ax n))
("eversole" nil (eh1 v er0 s ow1 l))
("everson" nil (eh1 v er0 s ax n))
("evert" nil (ih0 v er1 t))
("everton" nil (eh1 v er0 t ax n))
("everts" nil (eh1 v er0 t s))
("every" nil (eh1 v er0 iy0))
("everybody" nil (eh1 v r iy0 b aa1 d iy0))
("everyday" nil (eh1 v r iy0 d ey1))
("everyman" nil (eh1 v r iy0 m ax n))
("everyone" nil (eh1 v r iy0 w ah1 n))
("everything" nil (eh1 v r iy0 th ih1 ng))
("everytime" nil (eh1 v r iy0 t ay1 m))
("everywhere" nil (eh1 v r iy0 w eh1 r))
("eves" nil (iy1 v z))
("evett" nil (eh1 v ih0 t))
("evetts" nil (eh1 v ih0 t s))
("evian" nil (eh1 v iy0 ax n))
("evict" nil (ih0 v ih1 k t))
("evicted" nil (ih0 v ih1 k t ih0 d))
("evicting" nil (ih0 v ih1 k t ih0 ng))
("eviction" nil (ih0 v ih1 k sh ax n))
("evictions" nil (ih0 v ih1 k sh ax n z))
("evidence" nil (eh1 v ax d ax n s))
("evidenced" nil (eh1 v ih0 d ax n s t))
("evidences" nil (eh1 v ax d ax n s ax z))
("evident" nil (eh1 v ax d ax n t))
("evidentiary" nil (eh1 v ih0 d eh1 n sh iy0 eh0 r iy0))
("evidently" nil (eh1 v ax d ax n t l iy0))
("evil" nil (iy1 v ax l))
("evildoer" nil (iy1 v ax l d uw1 er0))
("evildoers" nil (iy1 v ax l d uw1 er0 z))
("evils" nil (iy1 v ax l z))
("evilsizer" nil (iy1 v ax l s ay1 z er0))
("evilsizor" nil (iy1 v ax l s ay1 z er0))
("evin" nil (eh1 v ih0 n))
("evinced" nil (ih0 v ih1 n s t))
("evinger" nil (iy1 v ih0 ng er0))
("evins" nil (eh1 v ih0 n z))
("eviscerate" nil (ax v ih1 s er0 ey1 t))
("evita" nil (eh0 v iy1 t ax))
("evitt" nil (eh1 v ih0 t))
("evitts" nil (eh1 v ih0 t s))
("evjen" nil (eh1 v jh ax n))
("evlyn" nil (eh1 v l ax n))
("evocation" nil (iy1 v ow0 k ey1 sh ax n))
("evocative" nil (ih0 v aa1 k ax t ih0 v))
("evoke" nil (ih0 v ow1 k))
("evoked" nil (ih0 v ow1 k t))
("evokes" nil (ih0 v ow1 k s))
("evoking" nil (ih0 v ow1 k ih0 ng))
("evola" nil (eh0 v ow1 l ax))
("evolution" nil (eh1 v ax l uw1 sh ax n))
("evolutionary" nil (eh1 v ax l uw1 sh ax n eh1 r iy0))
("evolve" nil (ih0 v aa1 l v))
("evolved" nil (ih0 v aa1 l v d))
("evolves" nil (ih0 v aa1 l v z))
("evolving" nil (ih0 v aa1 l v ih0 ng))
("evon" nil (eh1 v ax n))
("evonne" nil (eh1 v aa1 n))
("evrard" nil (eh1 v r er0 d))
("evren" nil (eh1 v r ax n))
("ewald" nil (y uw1 ax l d))
("ewalt" nil (y uw1 ax l t))
("ewan" nil (y uw1 ax n))
("ewart" nil (y uw1 er0 t))
("ewbank" nil (y uw1 b ax ng k))
("ewell" nil (y uw1 ax l))
("ewen" nil (y uw1 ax n))
("ewer" nil (y uw1 er0))
("ewers" nil (y uw1 er0 z))
("ewert" nil (y uw1 er0 t))
("ewig" nil (y uw1 ih0 g))
("ewing" nil (y uw1 ih0 ng))
("ewings" nil (y uw1 ih0 ng z))
("ewoldt" nil (y uw1 ow0 l t))
("eworld" nil (iy1 w er1 l d))
("ewton" nil (y uw1 t ax n))
("ewy" nil (y uw1 iy0))
("ex" nil (eh1 k s))
("exabyte" nil (eh1 k s ax b ay1 t))
("exacerbate" nil (ih0 g z ae1 s er0 b ey1 t))
("exacerbated" nil (ih0 g z ae1 s er0 b ey1 t ax d))
("exacerbates" nil (ih0 g z ae1 s er0 b ey1 t s))
("exacerbating" nil (ih0 g z ae1 s er0 b ey1 t ih0 ng))
("exacerbations" nil (ih0 g z ae1 s er0 b ey1 sh ax n z))
("exact" nil (ih0 g z ae1 k t))
("exacted" nil (ih0 g z ae1 k t ih0 d))
("exacting" nil (ih0 g z ae1 k t ih0 ng))
("exactions" nil (ih0 g z ae1 k sh ax n z))
("exactitude" nil (ih0 g z ae1 k t ax t uw1 d))
("exactly" nil (ih0 g z ae1 k t l iy0))
("exaggerate" nil (ih0 g z ae1 jh er0 ey1 t))
("exaggerated" nil (ih0 g z ae1 jh er0 ey1 t ax d))
("exaggerates" nil (ih0 g z ae1 jh er0 ey1 t s))
("exaggerating" nil (ih0 g z ae1 jh er0 ey1 t ih0 ng))
("exaggeration" nil (ih0 g z ae1 jh er0 ey1 sh ax n))
("exaggerations" nil (ih0 g z ae1 jh er0 ey1 sh ax n z))
("exalt" nil (ih0 g z ao1 l t))
("exalted" nil (ih0 g z ao1 l t ih0 d))
("exam" nil (ih0 g z ae1 m))
("examination" nil (ih0 g z ae1 m ax n ey1 sh ax n))
("examinations" nil (ih0 g z ae1 m ax n ey1 sh ax n z))
("examine" nil (ih0 g z ae1 m ih0 n))
("examined" nil (ih0 g z ae1 m ax n d))
("examiner" nil (ih0 g z ae1 m ax n er0))
("examiners" nil (ih0 g z ae1 m ih0 n er0 z))
("examines" nil (ih0 g z ae1 m ax n z))
("examining" nil (ih0 g z ae1 m ih0 n ih0 ng))
("example" nil (ih0 g z ae1 m p ax l))
("examples" nil (ih0 g z ae1 m p ax l z))
("exams" nil (ih0 g z ae1 m z))
("exar" nil (eh1 k s er0))
("exasperated" nil (ih0 g z ae1 s p er0 ey1 t ih0 d))
("exasperating" nil (ih0 g z ae1 s p er0 ey1 t ih0 ng))
("exasperation" nil (eh1 k s ae1 s p er0 ey1 sh ax n))
("exboyfriend" nil (eh1 k s b oy1 f r eh1 n d))
("exbroker" nil (eh1 k s b r ow1 k er0))
("exbrokers" nil (eh1 k s b r ow1 k er0 z))
("excalibur" nil (eh1 k s k ae1 l ax b er0))
("excavate" nil (eh1 k s k ax v ey1 t))
("excavated" nil (eh1 k s k ax v ey1 t ih0 d))
("excavating" nil (eh1 k s k ax v ey1 t ih0 ng))
("excavation" nil (eh1 k s k ax v ey1 sh ax n))
("excavators" nil (eh1 k s k ax v ey1 t er0 z))
("exceed" nil (ih0 k s iy1 d))
("exceeded" nil (ih0 k s iy1 d ax d))
("exceeding" nil (ih0 k s iy1 d ih0 ng))
("exceedingly" nil (ih0 k s iy1 d ih0 ng l iy0))
("exceeds" nil (ih0 k s iy1 d z))
("excel" nil (ih0 k s eh1 l))
("excelan" nil (eh1 k s eh1 l ax n))
("excelled" nil (ih0 k s eh1 l d))
("excellence" nil (eh1 k s ax l ax n s))
("excellent" nil (eh1 k s ax l ax n t))
("excellently" nil (eh1 k s ax l ax n t l iy0))
("excels" nil (ih0 k s eh1 l z))
("excelsior" nil (ih0 k s eh1 l s iy0 er0))
("except" nil (ih0 k s eh1 p t))
("excepted" nil (ih0 k s eh1 p t ih0 d))
("excepting" nil (eh1 k s eh1 p t ih0 ng))
("exception" nil (ih0 k s eh1 p sh ax n))
("exceptional" nil (ih0 k s eh1 p sh ax n ax l))
("exceptionally" nil (ih0 k s eh1 p sh ax n ax l iy0))
("exceptions" nil (ih0 k s eh1 p sh ax n z))
("excerpt" nil (eh1 k s er0 p t))
("excerpted" nil (eh1 k s er1 p t ih0 d))
("excerpts" nil (eh1 k s er0 p t s))
("excess" j (eh1 k s eh1 s))
("excess" n (eh1 k s eh1 s))
("excesses" n (eh1 k s eh1 s ih0 z))
("excesses" j (ih0 k s eh1 s ih0 z))
("excessive" nil (ih0 k s eh1 s ih0 v))
("excessively" nil (ih0 k s eh1 s ih0 v l iy0))
("exchange" nil (ih0 k s ch ey1 n jh))
("exchangeable" nil (ih0 k s ch ey1 n jh ax b ax l))
("exchanged" nil (ih0 k s ch ey1 n jh d))
("exchanger" nil (ih0 k s ch ey1 n jh er0))
("exchangers" nil (ih0 k s ch ey1 n jh er0 z))
("exchanges" nil (ih0 k s ch ey1 n jh ax z))
("exchanging" nil (ih0 k s ch ey1 n jh ih0 ng))
("exchequer" nil (eh1 k s ch eh1 k er0))
("excimer" nil (eh1 k s ih1 m er0))
("excise" nil (eh0 k s ay1 s))
("excised" nil (eh1 k s ay1 z d))
("excises" nil (eh1 k s ay1 z ih0 z))
("excision" nil (ih0 k s ih1 zh ax n))
("excitable" nil (ih0 k s ay1 t ax b ax l))
("excitation" nil (eh1 k s ay0 t ey1 sh ax n))
("excite" nil (ih0 k s ay1 t))
("excited" nil (ih0 k s ay1 t ax d))
("excitedly" nil (ih0 k s ay1 t ax d l iy0))
("excitement" nil (ih0 k s ay1 t m ax n t))
("excites" nil (ih0 k s ay1 t s))
("exciting" nil (ih0 k s ay1 t ih0 ng))
("exclaim" nil (ih0 k s k l ey1 m))
("exclaimed" nil (ih0 k s k l ey1 m d))
("exclaiming" nil (ih0 k s k l ey1 m ih0 ng))
("exclaims" nil (ih0 k s k l ey1 m z))
("exclamation" nil (eh1 k s k l ax m ey1 sh ax n))
("exclamations" nil (eh1 k s k l ax m ey1 sh ax n z))
("excludable" nil (ih0 k s k l uw1 d ax b ax l))
("exclude" nil (ih0 k s k l uw1 d))
("excluded" nil (ih0 k s k l uw1 d ax d))
("excludes" nil (ih0 k s k l uw1 d z))
("excluding" nil (ih0 k s k l uw1 d ih0 ng))
("exclusion" nil (ih0 k s k l uw1 zh ax n))
("exclusionary" nil (ih0 k s k l uw1 zh ax n eh1 r iy0))
("exclusions" nil (ih0 k s k l uw1 zh ax n z))
("exclusive" nil (ih0 k s k l uw1 s ih0 v))
("exclusively" nil (ih0 k s k l uw1 s ih0 v l iy0))
("exclusivity" nil (eh1 k s k l uw1 s ih1 v ax t iy0))
("exco" nil (eh1 k s k ow0))
("excoa" nil (eh1 k s k ow1 ax))
("excommunicated" nil (eh1 k s k ax m y uw1 n ax k ey1 t ax d))
("excommunication" nil (eh1 k s k ax m y uw1 n ax k ey1 sh ax n))
("excoriated" nil (eh0 k s k ao1 r iy0 ey1 t ih0 d))
("excoriating" nil (eh0 k s k ao1 r iy0 ey1 t ih0 ng))
("excrement" nil (eh1 k s k r ax m ax n t))
("excrete" nil (ih0 k s k r iy1 t))
("excretion" nil (ih0 k s k r iy1 sh ax n))
("excretory" nil (eh1 k s k r ax t ao1 r iy0))
("excruciating" nil (ih0 k s k r uw1 sh iy0 ey1 t ih0 ng))
("excruciatingly" nil (eh1 k s k r uw1 s iy0 ey1 t ih0 ng l iy0))
("exculpatory" nil (eh1 k s k ah1 l p ax t ao1 r iy0))
("excursion" nil (ih0 k s k er1 zh ax n))
("excursions" nil (ih0 k s k er1 zh ax n z))
("excuse" n (ih0 k s k y uw1 s))
("excuse" v (ih0 k s k y uw1 z))
("excused" nil (ih0 k s k y uw1 z d))
("excuses" n (ih0 k s k y uw1 s ih0 z))
("excuses" v (ih0 k s k y uw1 z ih0 z))
("exec" nil (eh1 g z eh1 k))
("execrable" nil (eh1 g z eh1 k r ax b ax l))
("execs" nil (eh1 g z eh1 k s))
("execute" nil (eh1 k s ax k y uw1 t))
("executed" nil (eh1 k s ax k y uw1 t ax d))
("executes" nil (eh1 k s ax k y uw1 t s))
("executing" nil (eh1 k s ax k y uw1 t ih0 ng))
("execution" nil (eh1 k s ax k y uw1 sh ax n))
("executioner" nil (eh1 k s ax k y uw1 sh ax n er0))
("executions" nil (eh1 k s ax k y uw1 sh ax n z))
("executive" nil (ih0 g z eh1 k y ax t ih0 v))
("executives" nil (ih0 g z eh1 k y ax t ih0 v z))
("executone" nil (eh1 g z eh1 k y uw0 t ow1 n))
("executor" nil (ih0 g z eh1 k y ax t er0))
("executors" nil (ih0 g z eh1 k y ax t er0 z))
("exel" nil (eh1 k s ax l))
("exemplary" nil (ih0 g z eh1 m p l er0 iy0))
("exemplified" nil (ih0 g z eh1 m p l ax f ay1 d))
("exemplifies" nil (ih0 g z eh1 m p l ax f ay1 z))
("exemplify" nil (ih0 g z eh1 m p l ax f ay1))
("exemplifying" nil (ih0 g z eh1 m p l ax f ay1 ih0 ng))
("exemplum" nil (ih0 g z eh1 m p l ax m))
("exempt" nil (ih0 g z eh1 m p t))
("exempted" nil (ih0 g z eh1 m p t ih0 d))
("exempting" nil (ih0 g z eh1 m p t ih0 ng))
("exemption" nil (ih0 g z eh1 m p sh ax n))
("exemptions" nil (ih0 g z eh1 m p sh ax n z))
("exempts" nil (ih0 g z eh1 m p t s))
("exercisable" nil (eh1 k s er0 s ay1 z ax b ax l))
("exercise" nil (eh1 k s er0 s ay1 z))
("exercised" nil (eh1 k s er0 s ay1 z d))
("exerciser" nil (eh1 k s er0 s ay1 z er0))
("exercisers" nil (eh1 k s er0 s ay1 z er0 z))
("exercises" nil (eh1 k s er0 s ay1 z ax z))
("exercising" nil (eh1 k s er0 s ay1 z ih0 ng))
("exert" nil (ih0 g z er1 t))
("exerted" nil (ih0 g z er1 t ih0 d))
("exerting" nil (ih0 g z er1 t ih0 ng))
("exertion" nil (ih0 g z er1 sh ax n))
("exertions" nil (ih0 g z er1 sh ax n z))
("exerts" nil (ih0 g z er1 t s))
("exes" nil (eh1 k s ih0 z))
("exeter" nil (eh1 k s ih0 t er0))
("exfoliation" nil (eh0 k s f ow1 l iy0 ey1 sh ax n))
("exhalation" nil (eh1 k s hh ax l ey1 sh ax n))
("exhaled" nil (eh0 k s hh ey1 l d))
("exhaust" nil (ih0 g z ao1 s t))
("exhausted" nil (ih0 g z ao1 s t ax d))
("exhausting" nil (ih0 g z ao1 s t ih0 ng))
("exhaustion" nil (ih0 g z ao1 s ch ax n))
("exhaustive" nil (ih0 g z ao1 s t ih0 v))
("exhaustively" nil (ih0 g z aa1 s t ih0 v l iy0))
("exhausts" nil (ih0 g z ao1 s t s))
("exhibit" nil (ih0 g z ih1 b ih0 t))
("exhibited" nil (ih0 g z ih1 b ax t ax d))
("exhibiting" nil (ih0 g z ih1 b ih0 t ih0 ng))
("exhibition" nil (eh1 k s ax b ih1 sh ax n))
("exhibitionist" nil (eh1 k s ax b ih1 sh ax n ax s t))
("exhibitionists" nil (eh1 k s ax b ih1 sh ax n ax s t s))
("exhibitions" nil (eh1 k s ax b ih1 sh ax n z))
("exhibitor" nil (ih0 g z ih1 b ax t er0))
("exhibitors" nil (ih0 g z ih1 b ax t er0 z))
("exhibits" nil (ih0 g z ih1 b ax t s))
("exhilarated" nil (ih0 g z ih1 l er0 ey1 t ih0 d))
("exhilarating" nil (ih0 g z ih1 l er0 ey1 t ih0 ng))
("exhilaration" nil (ih0 g z ih1 l er0 ey1 sh ax n))
("exhort" nil (ih0 g z ao1 r t))
("exhortation" nil (eh1 g z ao1 r t ey1 sh ax n))
("exhortations" nil (eh1 g z ao1 r t ey1 sh ax n z))
("exhorted" nil (ih0 g z ao1 r t ih0 d))
("exhorting" nil (ih0 g z ao1 r t ih0 ng))
("exhorts" nil (ih0 g z ao1 r t s))
("exide" nil (eh1 k s ay1 d))
("exigencies" nil (eh1 k s ih1 jh ax n s iy0 z))
("exigent" nil (eh1 k s ih0 jh ax n t))
("exigents" nil (eh1 k s ih0 jh ax n t s))
("exile" nil (eh1 g z ay1 l))
("exiled" nil (eh1 g z ay1 l d))
("exiles" nil (eh1 g z ay1 l z))
("exim" nil (eh1 k s ih0 m))
("exisiting" nil (eh1 g z ih1 s ih0 t ih0 ng))
("exist" nil (ih0 g z ih1 s t))
("existed" nil (ih0 g z ih1 s t ax d))
("existence" nil (eh0 g z ih1 s t ax n s))
("existent" nil (eh0 g z ih1 s t ax n t))
("existential" nil (eh1 k s ih0 s t eh1 n ch ax l))
("existing" nil (ih0 g z ih1 s t ih0 ng))
("exists" nil (ih0 g z ih1 s t s))
("exit" nil (eh1 g z ih0 t))
("exited" nil (eh1 g z ax t ih0 d))
("exiting" nil (eh1 g z ih0 t ih0 ng))
("exits" nil (eh1 g z ih0 t s))
("exler" nil (eh1 k s l er0))
("exley" nil (eh1 k s l iy0))
("exline" nil (eh1 k s l ay0 n))
("exner" nil (eh1 k s n er0))
("exocet" nil (eh1 k s ow0 s eh1 t))
("exodus" nil (eh1 k s ax d ax s))
("exogenous" nil (eh1 k s ow1 jh ax n ax s))
("exon" nil (eh1 k s ao1 n))
("exonerate" nil (ih0 g z aa1 n er0 ey1 t))
("exonerated" nil (ih0 g z aa1 n er0 ey1 t ih0 d))
("exonerating" nil (ih0 g z aa1 n er0 ey1 t ih0 ng))
("exorbitant" nil (ih0 g z ao1 r b ih0 t ax n t))
("exorcist" nil (eh1 k s er0 s ax s t))
("exoskeleton" nil (eh1 k s ow0 s k eh1 l ax t ax n))
("exothermic" nil (eh1 k s ow0 th er1 m ih0 k))
("exotic" nil (ih0 g z aa1 t ih0 k))
("exotics" nil (eh0 g z aa1 t ih0 k s))
("exovir" nil (eh1 k s ow0 v ih1 r))
("expand" nil (ih0 k s p ae1 n d))
("expandable" nil (ih0 k s p ae1 n d ax b ax l))
("expanded" nil (ih0 k s p ae1 n d ax d))
("expanding" nil (ih0 k s p ae1 n d ih0 ng))
("expands" nil (ih0 k s p ae1 n d z))
("expanse" nil (ih0 k s p ae1 n s))
("expanses" nil (ih0 k s p ae1 n s ih0 z))
("expansion" nil (ih0 k s p ae1 n sh ax n))
("expansionary" nil (ih0 k s p ae1 n sh ax n eh1 r iy0))
("expansionism" nil (ih0 k s p ae1 n ch ax n ih1 z ax m))
("expansionist" nil (ih0 k s p ae1 n ch ax n ih0 s t))
("expansions" nil (ih0 k s p ae1 n sh ax n z))
("expansive" nil (ih0 k s p ae1 n s ih0 v))
("expatriate" nil (eh0 k s p ey1 t r iy0 ey1 t))
("expatriates" nil (eh0 k s p ey1 t r iy0 ey1 t s))
("expatriation" nil (eh0 k s p ey1 t r iy0 ey1 sh ax n))
("expect" nil (ih0 k s p eh1 k t))
("expectancies" nil (ih0 k s p eh1 k t ax n s iy0 z))
("expectancy" nil (ih0 k s p eh1 k t ax n s iy0))
("expectant" nil (ih0 k s p eh1 k t ax n t))
("expectation" nil (eh1 k s p eh0 k t ey1 sh ax n))
("expectations" nil (eh1 k s p eh0 k t ey1 sh ax n z))
("expected" nil (ih0 k s p eh1 k t ax d))
("expecting" nil (ih0 k s p eh1 k t ih0 ng))
("expectorant" nil (ih0 k s p eh1 k t er0 ax n t))
("expects" nil (ih0 k s p eh1 k t s))
("expedience" nil (ih0 k s p iy1 d iy0 ax n s))
("expediency" nil (ih0 k s p iy1 d iy0 ax n s iy0))
("expedient" nil (ih0 k s p iy1 d iy0 ax n t))
("expedite" nil (eh1 k s p ih0 d ay1 t))
("expedited" nil (eh1 k s p ih0 d ay1 t ih0 d))
("expediting" nil (eh1 k s p ax d ay1 t ih0 ng))
("expedition" nil (eh1 k s p ax d ih1 sh ax n))
("expeditionary" nil (eh1 k s p ax d ih1 sh ax n eh1 r iy0))
("expeditions" nil (eh1 k s p ax d ih1 sh ax n z))
("expeditious" nil (eh1 k s p ax d ih1 sh ax s))
("expeditiously" nil (eh1 k s p ax d ih1 sh ax s l iy0))
("expel" nil (ih0 k s p eh1 l))
("expelled" nil (ih0 k s p eh1 l d))
("expelling" nil (ih0 k s p eh1 l ih0 ng))
("expend" nil (ih0 k s p eh1 n d))
("expendable" nil (ih0 k s p eh1 n d ax b ax l))
("expended" nil (ih0 k s p eh1 n d ih0 d))
("expending" nil (eh1 k s p eh1 n d ih0 ng))
("expenditure" nil (ih0 k s p eh1 n d ax ch er0))
("expenditures" nil (ih0 k s p eh1 n d ax ch er0 z))
("expense" nil (ih0 k s p eh1 n s))
("expenses" nil (ih0 k s p eh1 n s ax z))
("expensing" nil (ih0 k s p eh1 n s ih0 ng))
("expensive" nil (ih0 k s p eh1 n s ih0 v))
("expensively" nil (eh1 k s p eh1 n s ih0 v l iy0))
("experience" nil (ih0 k s p ih1 r iy0 ax n s))
("experienced" nil (ih0 k s p ih1 r iy0 ax n s t))
("experiences" nil (ih0 k s p ih1 r iy0 ax n s ih0 z))
("experiencing" nil (ih0 k s p ih1 r iy0 ax n s ih0 ng))
("experiment" nil (ih0 k s p eh1 r ax m ax n t))
("experimental" nil (ih0 k s p eh1 r ax m eh1 n t ax l))
("experimentalist" nil (ih0 k s p eh1 r ax m eh1 n t ax l ih0 s t))
("experimentally" nil (ih0 k s p eh0 r ax m eh1 n t ax l iy0))
("experimentation" nil (ih0 k s p eh1 r ax m ax n t ey1 sh ax n))
("experimented" nil (ih0 k s p eh1 r ax m ax n t ax d))
("experimenters" nil (ih0 k s p eh1 r ax m eh1 n t er0 z))
("experimenting" nil (eh0 k s p eh1 r ax m eh0 n t ih0 ng))
("experiments" nil (ih0 k s p eh1 r ax m ax n t s))
("expert" nil (eh1 k s p er0 t))
("expertise" nil (eh1 k s p er0 t iy1 z))
("expertly" nil (eh1 k s p er0 t l iy0))
("experts" nil (eh1 k s p er0 t s))
("expiate" nil (eh1 k s p iy0 ey1 t))
("expiration" nil (eh1 k s p er0 ey1 sh ax n))
("expirations" nil (eh1 k s p er0 ey1 sh ax n z))
("expiratory" nil (ih0 k s p ay1 r ax t ao1 r iy0))
("expire" nil (ih0 k s p ay1 r))
("expired" nil (ih0 k s p ay1 r d))
("expires" nil (ih0 k s p ay1 er0 z))
("expiring" nil (ih0 k s p ay1 r ih0 ng))
("expiry" nil (eh1 k s p ay1 r iy0))
("explain" nil (ih0 k s p l ey1 n))
("explainable" nil (ih0 k s p l ey1 n ax b ax l))
("explained" nil (ih0 k s p l ey1 n d))
("explaining" nil (ih0 k s p l ey1 n ih0 ng))
("explains" nil (ih0 k s p l ey1 n z))
("explanation" nil (eh1 k s p l ax n ey1 sh ax n))
("explanations" nil (eh1 k s p l ax n ey1 sh ax n z))
("explanatory" nil (ih0 k s p l ae1 n ax t ao1 r iy0))
("expletive" nil (eh1 k s p l ax t ih0 v))
("expletives" nil (eh1 k s p l ax t ih0 v z))
("explication" nil (eh1 k s p l ax k ey1 sh ax n))
("explicit" nil (ih0 k s p l ih1 s ax t))
("explicitly" nil (ih0 k s p l ih1 s ax t l iy0))
("explode" nil (ih0 k s p l ow1 d))
("exploded" nil (ih0 k s p l ow1 d ax d))
("explodes" nil (ih0 k s p l ow1 d z))
("exploding" nil (ih0 k s p l ow1 d ih0 ng))
("exploit" n (eh1 k s p l oy0 t))
("exploit" v (ih0 k s p l oy1 t))
("exploitation" nil (eh1 k s p l oy1 t ey1 sh ax n))
("exploitative" nil (eh1 k s p l oy1 t ax t ih0 v))
("exploited" nil (eh1 k s p l oy1 t ax d))
("exploiting" nil (eh1 k s p l oy1 t ih0 ng))
("exploits" n (eh1 k s p l oy1 t s))
("exploits" v (ih0 k s p l oy1 t s))
("exploration" nil (eh1 k s p l er0 ey1 sh ax n))
("explorations" nil (eh1 k s p l er0 ey1 sh ax n z))
("exploratory" nil (ih0 k s p l ao1 r ax t ao1 r iy0))
("explore" nil (ih0 k s p l ao1 r))
("explored" nil (ih0 k s p l ao1 r d))
("explorer" nil (ih0 k s p l ao1 r er0))
("explorers" nil (ih0 k s p l ao1 r er0 z))
("explores" nil (ih0 k s p l ao1 r z))
("exploring" nil (ih0 k s p l ao1 r ih0 ng))
("explosion" nil (ih0 k s p l ow1 zh ax n))
("explosions" nil (ih0 k s p l ow1 zh ax n z))
("explosive" nil (ih0 k s p l ow1 s ih0 v))
("explosively" nil (eh1 k s p l ow1 s ih0 v l iy0))
("explosives" nil (ih0 k s p l ow1 s ih0 v z))
("explosivos" nil (eh1 k s p l ax s iy1 v ow0 s))
("expo" nil (eh1 k s p ow0))
("exponent" nil (eh1 k s p ow1 n ax n t))
("exponential" nil (eh1 k s p ow0 n eh1 n ch ax l))
("exponentially" nil (eh1 k s p ow0 n eh1 n sh ax l iy0))
("exponents" nil (ih0 k s p ow1 n ax n t s))
("export" n (eh1 k s p ao0 r t))
("export" v (ih0 k s p ao1 r t))
("exportable" nil (eh0 k s p ao1 r t ax b ax l))
("exported" nil (ih0 k s p ao1 r t ax d))
("exporter" nil (ih0 k s p ao1 r t er0))
("exporters" nil (ih0 k s p ao1 r t er0 z))
("exporting" nil (ih0 k s p ao1 r t ih0 ng))
("exports" n (eh1 k s p ao0 r t s))
("exports" v (ih0 k s p ao1 r t s))
("expos" nil (eh1 k s p ow0 z))
("expose" nil (ih0 k s p ow1 z))
("exposed" nil (ih0 k s p ow1 z d))
("exposes" nil (ih0 k s p ow1 z ih0 z))
("exposing" nil (ih0 k s p ow1 z ih0 ng))
("exposition" nil (eh1 k s p ax z ih1 sh ax n))
("expositions" nil (eh1 k s p ax z ih1 sh ax n z))
("exposito" nil (eh0 k s p ax s ay1 t ow0))
("exposure" nil (ih0 k s p ow1 zh er0))
("exposures" nil (ih0 k s p ow1 zh er0 z))
("expound" nil (ih0 k s p aw1 n d))
("expounded" nil (ih0 k s p aw1 n d ax d))
("expounding" nil (ih0 k s p aw1 n d ih0 ng))
("expounds" nil (ih0 k s p aw1 n d z))
("express" nil (ih0 k s p r eh1 s))
("expressed" nil (ih0 k s p r eh1 s t))
("expresses" nil (ih0 k s p r eh1 s ax z))
("expressing" nil (ih0 k s p r eh1 s ih0 ng))
("expression" nil (ih0 k s p r eh1 sh ax n))
("expressionism" nil (ih0 k s p r eh1 sh ax n ih1 z ax m))
("expressionist" nil (ih0 k s p r eh1 sh ax n ax s t))
("expressionistic" nil (ih0 k s p r eh1 sh ax n ih1 s t ih0 k))
("expressions" nil (ih0 k s p r eh1 sh ax n z))
("expressive" nil (ih0 k s p r eh1 s ih0 v))
("expressivity" nil (eh1 k s p r ax s ih1 v ih0 t iy0))
("expressly" nil (ih0 k s p r eh1 s l iy0))
("expressway" nil (ih0 k s p r eh1 s w ey1))
("expropriate" nil (eh0 k s p r ow1 p r iy0 ey1 t))
("expropriated" nil (eh0 k s p r ow1 p r iy0 ey1 t ih0 d))
("expropriation" nil (eh1 k s p r ow1 p r iy0 ey1 sh ax n))
("expropriations" nil (eh1 k s p r ow1 p r iy0 ey1 sh ax n z))
("expulsion" nil (ih0 k s p ah1 l sh ax n))
("expulsions" nil (ih0 k s p ah1 l sh ax n z))
("expunged" nil (ih0 k s p ah1 n jh d))
("exquisite" nil (eh1 k s k w ax z ax t))
("exquisitely" nil (eh1 k s k w ih1 z ih0 t l iy0))
("extant" nil (eh1 k s t ax n t))
("extel" nil (eh1 k s t eh1 l))
("extend" nil (ih0 k s t eh1 n d))
("extendable" nil (eh1 k s t eh1 n d ax b ax l))
("extended" nil (ih0 k s t eh1 n d ax d))
("extenders" nil (eh1 k s t eh1 n d er0 z))
("extendible" nil (eh1 k s t eh1 n d ih0 b ax l))
("extending" nil (ih0 k s t eh1 n d ih0 ng))
("extends" nil (ih0 k s t eh1 n d z))
("extension" nil (ih0 k s t eh1 n sh ax n))
("extensions" nil (ih0 k s t eh1 n sh ax n z))
("extensive" nil (ih0 k s t eh1 n s ih0 v))
("extensively" nil (ih0 k s t eh1 n s ih0 v l iy0))
("extent" nil (ih0 k s t eh1 n t))
("extenuating" nil (ih0 k s t eh1 n y uw0 ey1 t ih0 ng))
("exterior" nil (ih0 k s t ih1 r iy0 er0))
("exteriors" nil (eh0 k s t ih1 r iy0 er0 z))
("exterminating" nil (ih0 k s t er1 m ax n ey1 t ih0 ng))
("extermination" nil (ih0 k s t er1 m ax n ey1 sh ax n))
("exterminator" nil (ih0 k s t er1 m ax n ey1 t er0))
("exterminators" nil (ih0 k s t er1 m ax n ey1 t er0 z))
("external" nil (ih0 k s t er1 n ax l))
("externally" nil (ih0 k s t er1 n ax l iy0))
("extinct" nil (ih0 k s t ih1 ng k t))
("extinction" nil (ih0 k s t ih1 ng k sh ax n))
("extinguish" nil (ih0 k s t ih1 ng g w ih0 sh))
("extinguished" nil (ih0 k s t ih1 ng g w ih0 sh t))
("extinguisher" nil (ih0 k s t ih1 ng g w ih0 sh er0))
("extinguishers" nil (ih0 k s t ih1 ng g w ih0 sh er0 z))
("extinguishing" nil (ih0 k s t ih1 ng g w ih0 sh ih0 ng))
("extinguishment" nil (ih0 k s t ih1 ng g w ih0 sh m ax n t))
("extirpate" nil (eh1 k s t er0 p ey1 t))
("extol" nil (ih0 k s t ow1 l))
("extolled" nil (ih0 k s t ow1 l d))
("extolling" nil (ih0 k s t ow1 l ih0 ng))
("extols" nil (ih0 k s t ow1 l z))
("exton" nil (eh1 k s t ax n))
("extort" nil (ih0 k s t ao1 r t))
("extorted" nil (ih0 k s t ao1 r t ih0 d))
("extorting" nil (ih0 k s t ao1 r t ih0 ng))
("extortion" nil (ih0 k s t ao1 r sh ax n))
("extortionate" nil (ih0 k s t ao1 r sh ax n ax t))
("extra" nil (eh1 k s t r ax))
("extracellularly" nil (eh1 k s t r ax s eh1 l y ax l er0 l iy0))
("extract" v (ih0 k s t r ae1 k t))
("extract" n (eh1 k s t r ae1 k t))
("extracted" nil (ih0 k s t r ae1 k t ax d))
("extracting" nil (ih0 k s t r ae1 k t ih0 ng))
("extraction" nil (ih0 k s t r ae1 k sh ax n))
("extractions" nil (ih0 k s t r ae1 k sh ax n z))
("extracts" v (ih0 k s t r ae1 k t s))
("extracts" n (eh1 k s t r ae1 k t s))
("extracurricular" nil (eh1 k s t r ax k er0 ih1 k y ax l er0))
("extradite" nil (eh1 k s t r ax d ay1 t))
("extradited" nil (eh1 k s t r ax d ay1 t ih0 d))
("extraditing" nil (eh1 k s t r ax d ay1 t ih0 ng))
("extradition" nil (eh1 k s t r ax d ih1 sh ax n))
("extragalactic" nil (eh1 k s t r ax g ax l ae1 k t ih0 k))
("extralegal" nil (eh1 k s t r ax l iy1 g ax l))
("extramarital" nil (eh1 k s t r ax m eh1 r ax t ax l))
("extraneous" nil (eh0 k s t r ey1 n iy0 ax s))
("extraordinaire" nil (eh1 k s t r ax ao1 r d ih0 n eh1 r))
("extraordinarily" nil (ih0 k s t r ao1 r d ax n eh1 r ax l iy0))
("extraordinary" nil (ih0 k s t r ao1 r d ax n eh1 r iy0))
("extrapolate" nil (ih0 k s t r ae1 p ax l ey1 t))
("extrapolated" nil (ih0 k s t r ae1 p ax l ey1 t ih0 d))
("extrapolating" nil (ih0 k s t r ae1 p ax l ey1 t ih0 ng))
("extrapolation" nil (ih0 k s t r ae1 p ax l ey1 sh ax n))
("extras" nil (eh1 k s t r ax z))
("extrasensory" nil (eh1 k s t r ax s eh1 n s er0 iy0))
("extraterrestrial" nil (eh1 k s t r ax t er0 eh1 s t r iy0 ax l))
("extraterrestrials" nil (eh1 k s t r ax t er1 eh1 s t r iy0 ax l z))
("extraterritorial" nil (eh1 k s t r ax t eh1 r ih0 t ao1 r iy0 ax l))
("extraterritoriality" nil (eh1 k s t r ax t eh1 r ax t ao1 r iy0 ae1 l ax t iy0))
("extravagance" nil (ih0 k s t r ae1 v ax g ax n s))
("extravagant" nil (ih0 k s t r ae1 v ax g ax n t))
("extravagantly" nil (eh1 k s t r ae1 v ax g ax n t l iy0))
("extravaganza" nil (ih0 k s t r ae1 v ax g ae1 n z ax))
("extravaganzas" nil (ih0 k s t r ae1 v ax g ae1 n z ax z))
("extreme" nil (ih0 k s t r iy1 m))
("extremely" nil (ih0 k s t r iy1 m l iy0))
("extremes" nil (ih0 k s t r iy1 m z))
("extremism" nil (eh1 k s t r eh1 m ih0 z ax m))
("extremist" nil (ih0 k s t r iy1 m ih0 s t))
("extremists" nil (ih0 k s t r iy1 m ax s t s))
("extremities" nil (ih0 k s t r eh1 m ax t iy0 z))
("extremity" nil (ih0 k s t r eh1 m ax t iy0))
("extricate" nil (eh1 k s t r ax k ey1 t))
("extricated" nil (eh1 k s t r ih0 k ey1 t ih0 d))
("extroverted" nil (eh1 k s t r ax v er1 t ih0 d))
("extruded" nil (ih0 k s t r uw1 d ax d))
("extruding" nil (ih0 k s t r uw1 d ih0 ng))
("extrusion" nil (ih0 k s t r uw1 zh ax n))
("exuberance" nil (ih0 g z uw1 b er0 ax n s))
("exuberant" nil (ih0 g z uw1 b er0 ax n t))
("exude" nil (ih0 g z uw1 d))
("exuded" nil (ih0 g z uw1 d ih0 d))
("exudes" nil (ih0 g z uw1 d z))
("exultant" nil (ih0 g z ah1 l t ax n t))
("exultantly" nil (ih0 g z ah1 l t ax n t l iy0))
("exulted" nil (ih0 g z ah1 l t ih0 d))
("exults" nil (ih0 g z ah1 l t s))
("exum" nil (ih0 g z ah1 m))
("exxon" nil (eh1 k s aa0 n))
("eyde" nil (ey1 d))
("eydie" nil (ey1 d iy0))
("eye" nil (ay1))
("eyeball" nil (ay1 b ao1 l))
("eyeballs" nil (ay1 b ao1 l z))
("eyebrow" nil (ay1 b r aw1))
("eyebrows" nil (ay1 b r aw1 z))
("eyecare" nil (ay1 k eh1 r))
("eyed" nil (ay1 d))
("eyedropper" nil (ay1 d r aa1 p er0))
("eyedrops" nil (ay1 d r aa1 p s))
("eyeglass" nil (ay1 g l ae1 s))
("eyeglasses" nil (ay1 g l ae1 s ax z))
("eyeing" nil (ay1 ih0 ng))
("eyelab" nil (ay1 l ae1 b))
("eyelash" nil (ay1 l ae1 sh))
("eyelashes" nil (ay1 l ae1 sh ih0 z))
("eyeless" nil (ay1 l ax s))
("eyelet" nil (ay1 l ax t))
("eyelets" nil (ay1 l ax t s))
("eyelids" nil (ay1 l ih1 d z))
("eyelike" nil (ay1 l ay1 k))
("eyepiece" nil (ay1 p iy1 s))
("eyer" nil (ay1 er0))
("eyerly" nil (iy1 er0 l iy0))
("eyerman" nil (iy1 er0 m ax n))
("eyes" nil (ay1 z))
("eyeshade" nil (ay1 sh ey1 d))
("eyesight" nil (ay1 s ay1 t))
("eyesore" nil (ay1 s ao1 r))
("eyespot" nil (ay1 s p aa1 t))
("eyestone" nil (ay1 s t ow1 n))
("eyestrain" nil (ay1 s t r ey1 n))
("eyetech" nil (ay1 t eh1 k))
("eyewear" nil (ay1 w eh1 r))
("eyewitness" nil (ay1 w ih1 t n ax s))
("eyewitnesses" nil (ay1 w ih1 t n ax s ih0 z))
("eyler" nil (ey1 l er0))
("eyman" nil (ey1 m ax n))
("eynon" nil (ey1 n ax n))
("eyre" nil (eh1 r))
("eyrich" nil (eh1 r ih0 ch))
("eyrie" nil (eh1 r iy0))
("eyring" nil (ey1 r ih0 ng))
("eyster" nil (ey1 s t er0))
("eyton" nil (ey1 t ax n))
("ezekiel" nil (eh1 z ih0 k iy1 l))
("ezell" nil (ax z eh1 l))
("ezelle" nil (ih0 z eh1 l))
("ezoe" nil (eh1 z ow0))
("ezola" nil (ey1 z ow1 l ax))
("ezra" nil (eh1 z r ax))
("ezzell" nil (eh1 z ax l))
("ezzo" nil (eh1 z ow0))
("f" nil (eh1 f))
("faal" nil (f aa1 l))
("faas" nil (f aa1 z))
("fab" nil (f ae1 b))
("fabbri" nil (f ae1 b r iy0))
("fabel" nil (f ae1 b ax l))
("fabela" nil (f aa0 b eh1 l ax))
("faber" nil (f ey1 b er0))
("faberge" nil (f ae1 b er0 jh))
("faberman" nil (f ey1 b er0 m ax n))
("fabia" nil (f aa1 b iy0 ax))
("fabian" nil (f ey1 b iy0 ax n))
("fabiani" nil (f aa0 b iy0 aa1 n iy0))
("fabiano" nil (f aa0 b iy0 aa1 n ow0))
("fabien" nil (f ae1 b iy0 n))
("fabio" nil (f aa1 b iy0 ow0))
("fable" nil (f ey1 b ax l))
("fabled" nil (f ey1 b ax l d))
("fables" nil (f ey1 b ax l z))
("fabre" nil (f ey1 b er0))
("fabri" nil (f ae1 b r iy0))
("fabric" nil (f ae1 b r ih0 k))
("fabricant" nil (f ae1 b r ih0 k ax n t))
("fabricate" nil (f ae1 b r ax k ey1 t))
("fabricated" nil (f ae1 b r ih0 k ey1 t ax d))
("fabricates" nil (f ae1 b r ih0 k ey1 t s))
("fabricating" nil (f ae1 b r ih0 k ey1 t ih0 ng))
("fabrication" nil (f ae1 b r ih0 k ey1 sh ax n))
("fabrications" nil (f ae1 b r ih0 k ey1 sh ax n z))
("fabricator" nil (f ae1 b r ih0 k ey1 t er0))
("fabricators" nil (f ae1 b r ih0 k ey1 t er0 z))
("fabrics" nil (f ae1 b r ih0 k s))
("fabris" nil (f ae1 b r ih0 s))
("fabrizi" nil (f aa0 b r iy1 z iy0))
("fabrizio" nil (f aa0 b r iy1 t s iy0 ow0))
("fabrizius" nil (f ae1 b r iy1 z iy0 ax s))
("fabron" nil (f ae1 b r ax n))
("fabry" nil (f ae1 b r iy0))
("fabulous" nil (f ae1 b y ax l ax s))
("fabulously" nil (f ae1 b y uw0 l ax s l iy0))
("fac" nil (f ae1 k))
("facade" nil (f ax s aa1 d))
("facades" nil (f ax s aa1 d z))
("facchini" nil (f aa0 k iy1 n iy0))
("facciola" nil (f ae1 ch iy0 ow1 l ax))
("face" nil (f ey1 s))
("faced" nil (f ey1 s t))
("facedown" nil (f ey1 s d aw1 n))
("faceless" nil (f ey1 s l ax s))
("facelift" nil (f ey1 s l ih1 f t))
("facemire" nil (f aa0 ch eh0 m ih1 r iy0))
("facer" nil (f ey1 s er0))
("faces" nil (f ey1 s ax z))
("facet" nil (f ae1 s ax t))
("faceted" nil (f ae1 s ax t ih0 d))
("facetious" nil (f ax s iy1 sh ax s))
("facetiously" nil (f ax s iy1 sh ax s l iy0))
("facets" nil (f ae1 s ax t s))
("facey" nil (f ey1 s iy0))
("facial" nil (f ey1 sh ax l))
("faciane" nil (f aa0 s iy0 aa1 n ey0))
("facie" nil (f ey1 s iy0))
("facile" nil (f ae1 s ax l))
("facilitate" nil (f ax s ih1 l ax t ey1 t))
("facilitated" nil (f ax s ih1 l ax t ey1 t ih0 d))
("facilitates" nil (f ax s ih1 l ax t ey1 t s))
("facilitating" nil (f ax s ih1 l ax t ey1 t ih0 ng))
("facilitation" nil (f ax s ih1 l ax t ey1 sh ax n))
("facilitator" nil (f ax s ih1 l ax t ey1 t er0))
("facilitators" nil (f ax s ih1 l ax t ey1 t er0 z))
("facilities" nil (f ax s ih1 l ax t iy0 z))
("facility" nil (f ax s ih1 l ih0 t iy0))
("facing" nil (f ey1 s ih0 ng))
("facings" nil (f ey1 s ih0 ng z))
("fackler" nil (f ae1 k l er0))
("fackrell" nil (f ae1 k r ax l))
("facsimile" nil (f ae0 k s ih1 m ax l iy0))
("facsimiles" nil (f ae0 k s ih1 m ax l iy0 z))
("fact" nil (f ae1 k t))
("facteau" nil (f ax k t ow1))
("faction" nil (f ae1 k sh ax n))
("factional" nil (f ae1 k sh ax n ax l))
("factionalism" nil (f ae1 k sh ax n ax l ih1 z ax m))
("factions" nil (f ae1 k sh ax n z))
("factly" nil (f ae1 k t l iy0))
("facto" nil (f ae1 k t ow0))
("factor" nil (f ae1 k t er0))
("factored" nil (f ae1 k t er0 d))
("factories" nil (f ae1 k t er0 iy0 z))
("factoring" nil (f ae1 k t er0 ih0 ng))
("factors" nil (f ae1 k t er0 z))
("factory" nil (f ae1 k t er0 iy0))
("facts" nil (f ae1 k t s))
("factual" nil (f ae1 k ch uw0 ax l))
("factually" nil (f ae1 k ch uw0 ax l iy0))
("facultative" nil (f ae1 k ax l t ey1 t ih0 v))
("faculties" nil (f ae1 k ax l t iy0 z))
("faculty" nil (f ae1 k ax l t iy0))
("fad" nil (f ae1 d))
("fadden" nil (f ae1 d ax n))
("faddis" nil (f ae1 d ih0 s))
("faddish" nil (f ae1 d ih0 sh))
("fade" nil (f ey1 d))
("faded" nil (f ey1 d ax d))
("fadel" nil (f ae1 d ax l))
("fadeley" nil (f ae1 d ih0 l iy0))
("fadely" nil (f ey1 d l iy0))
("faden" nil (f ey1 d ax n))
("fader" nil (f ey1 d er0))
("fades" nil (f ey1 d z))
("fading" nil (f ey1 d ih0 ng))
("fadlallah" nil (f ae1 d l ae1 l ax))
("fadness" nil (f ae1 d n ax s))
("fads" nil (f ae1 d z))
("fae" nil (f ay1))
("faerber" nil (f eh1 r b er0))
("faeth" nil (f iy1 th))
("fagan" nil (f ey1 g ax n))
("fagen" nil (f ae1 g ax n))
("fager" nil (f ey1 g er0))
("fagerberg" nil (f ey1 g er0 b er0 g))
("fagerstrom" nil (f ey1 g er0 s t r ax m))
("fagg" nil (f ae1 g))
("faggart" nil (f ae1 g aa0 r t))
("fagin" nil (f ey1 g ih0 n))
("fagley" nil (f ae1 g l iy0))
("fagnant" nil (f ae1 g n ax n t))
("fagoth" nil (f ae1 g ax th))
("fagundes" nil (f ae1 g ax n d z))
("fahd" nil (f aa1 d))
("faherty" nil (f ae1 hh er0 t iy0))
("fahey" nil (f ae1 hh iy0))
("fahl" nil (f aa1 l))
("fahlman" nil (f aa1 l m ax n))
("fahmy" nil (f aa1 m iy0))
("fahnestock" nil (f aa1 n s t aa1 k))
("fahr" nil (f aa1 r))
("fahrenheit" nil (f eh1 r ax n hh ay1 t))
("fahrenkopf" nil (f aa1 r ax n k aa1 p f))
("fahrer" nil (f aa1 r er0))
("fahringer" nil (f aa1 r ih0 ng er0))
("fahrner" nil (f aa1 r n er0))
("fahrney" nil (f aa1 r n iy0))
("fahs" nil (f ae1 s))
("fahy" nil (f ey1 hh iy0))
("faidley" nil (f ey1 d l iy0))
("faiella" nil (f ay1 eh1 l ax))
("fail" nil (f ey1 l))
("faile" nil (f ey1 l))
("failed" nil (f ey1 l d))
("failing" nil (f ey1 l ih0 ng))
("failings" nil (f ey1 l ih0 ng z))
("failla" nil (f ey1 l ax))
("failor" nil (f ey1 l er0))
("fails" nil (f ey1 l z))
("failsafe" nil (f ey1 l s ey1 f))
("failure" nil (f ey1 l y er0))
("failures" nil (f ey1 l y er0 z))
("fain" nil (f ey1 n))
("faint" nil (f ey1 n t))
("fainted" nil (f ey1 n t ih0 d))
("fainter" nil (f ey1 n t er0))
("faintest" nil (f ey1 n t ax s t))
("fainthearted" nil (f ey1 n t hh aa1 r t ih0 d))
("fainting" nil (f ey1 n t ih0 ng))
("faintly" nil (f ey1 n t l iy0))
("faintness" nil (f ey1 n t n ax s))
("faiola" nil (f ay1 ow0 l ax))
("fair" nil (f eh1 r))
("fairall" nil (f eh0 r ao1 l))
("fairbairn" nil (f eh1 r b er0 n))
("fairbank" nil (f eh1 r b ax ng k))
("fairbanks" nil (f eh1 r b ax ng k s))
("fairbrother" nil (f eh1 r b r ax dh er0))
("fairburn" nil (f eh1 r b er1 n))
("fairchild" nil (f eh1 r ch ay1 l d))
("faircloth" nil (f eh1 r k l ax th))
("fairclough" nil (f eh1 r k l aw0))
("faire" nil (f eh1 r))
("fairer" nil (f eh1 r er0))
("faires" nil (f eh1 r z))
("fairest" nil (f eh1 r ih0 s t))
("fairey" nil (f eh1 r iy0))
("fairfax" nil (f eh1 r f ae1 k s))
("fairfield" nil (f eh1 r f iy1 l d))
("fairgrounds" nil (f eh1 r g r aw1 n d z))
("fairhaven" nil (f eh1 r hh ey1 v ax n))
("fairhurst" nil (f ay1 r hh er0 s t))
("fairies" nil (f eh1 r iy0 z))
("fairlawn" nil (f eh1 r l ao1 n))
("fairleigh" nil (f eh1 r l ax))
("fairless" nil (f eh0 r l iy1 s))
("fairley" nil (f eh1 r l iy0))
("fairlie" nil (f eh1 r l iy0))
("fairly" nil (f eh1 r l iy0))
("fairman" nil (f ay1 r m ax n))
("fairmont" nil (f eh1 r m aa1 n t))
("fairness" nil (f eh1 r n ax s))
("fairs" nil (f eh1 r z))
("fairview" nil (f eh1 r v y uw1))
("fairway" nil (f eh1 r w ey1))
("fairways" nil (f eh1 r w ey1 z))
("fairweather" nil (f eh1 r w eh1 dh er0))
("fairy" nil (f eh1 r iy0))
("fairyland" nil (f eh1 r iy0 l ae1 n d))
("faisal" nil (f ey1 z ax l))
("faison" nil (f ey1 z ax n))
("faist" nil (f aa1 ih0 s t))
("fait" nil (f ey1 t))
("faith" nil (f ey1 th))
("faithful" nil (f ey1 th f ax l))
("faithfully" nil (f ey1 th f ax l iy0))
("faithfuls" nil (f ey1 th f ax l z))
("faiths" nil (f ey1 th s))
("faivre" nil (f ey1 v er0))
("fajardo" nil (f aa0 y aa1 r d ow0))
("fajita" nil (f ax jh iy1 t ax))
("fajitas" nil (f ax jh iy1 t ax z))
("fake" nil (f ey1 k))
("faked" nil (f ey1 k t))
("fakers" nil (f ey1 k er0 z))
("fakes" nil (f ey1 k s))
("faking" nil (f ey1 k ih0 ng))
("falafel" nil (f ax l aa1 f ax l))
("falanga" nil (f aa0 l aa1 ng g ax))
("falardeau" nil (f ae1 l aa0 r d ow1))
("falasco" nil (f aa0 l aa1 s k ow0))
("falb" nil (f ao1 l b))
("falbo" nil (f ae1 l b ow0))
("falcigno" nil (f ae1 l s ih1 g n ow0))
("falck" nil (f ae1 l k))
("falco" nil (f ae1 l k ow0))
("falcoff" nil (f ae1 l k ao0 f))
("falcon" nil (f ae1 l k ax n))
("falconbridge" nil (f ae1 l k ax n b r ih1 jh))
("falcone" nil (f aa0 l k ow1 n iy0))
("falconer" nil (f ae1 l k ax n er0))
("falconers" nil (f ae1 l k ax n er0 z))
("falconets" nil (f ae1 l k ax n eh1 t s))
("falconi" nil (f aa0 l k ow1 n iy0))
("falconry" nil (f ae1 l k ax n r iy0))
("falcons" nil (f ae1 l k ax n z))
("faldo" nil (f aa1 l d ow0))
("faler" nil (f ey1 l er0))
("fales" nil (f ey1 l z))
("faley" nil (f ey1 l iy0))
("falgoust" nil (f aa1 l g uw0 s t))
("falgout" nil (f ae1 l g aw0 t))
("falik" nil (f aa1 l ih0 k))
("falin" nil (f ae1 l ih0 n))
("falise" nil (f ax l iy1 s))
("falk" nil (f ao1 k))
("falke" nil (f ao1 k))
("falkenberg" nil (f ao1 k ax n b er0 g))
("falkenstein" nil (f ao1 k ax n s t ay0 n))
("falkland" nil (f ao1 k l ax n d))
("falklands" nil (f ao1 k l ax n d z))
("falkner" nil (f ao1 k n er0))
("falkowski" nil (f ax l k ao1 f s k iy0))
("fall" nil (f ao1 l))
("falla" nil (f ae1 l ax))
("fallacies" nil (f ae1 l ax s iy0 z))
("fallacious" nil (f ax l ey1 sh ax s))
("fallaciously" nil (f ax l ey1 sh ax s l iy0))
("fallacy" nil (f ae1 l ax s iy0))
("fallaw" nil (f ae1 l ao1))
("fallback" nil (f ao1 l b ae1 k))
("fallen" nil (f ao1 l ax n))
("faller" nil (f ao1 l er0))
("fallert" nil (f ae1 l er0 t))
("fallibility" nil (f ae1 l ih0 b ih1 l ih0 t iy0))
("fallible" nil (f ae1 l ax b ax l))
("fallin" nil (f ae1 l ih0 n))
("falling" nil (f ao1 l ih0 ng))
("fallis" nil (f ae1 l ih0 s))
("falloff" nil (f ao1 l ao1 f))
("fallon" nil (f ae1 l ax n))
("fallout" nil (f ao1 l aw1 t))
("fallouts" nil (f ao1 l aw1 t s))
("fallow" nil (f ae1 l ow1))
("fallows" nil (f ae1 l ow1 z))
("falls" nil (f ao1 l z))
("falmouth" nil (f ae1 l m ax th))
("falor" nil (f ae1 l er0))
("falotico" nil (f aa0 l ow0 t iy1 k ow0))
("false" nil (f ao1 l s))
("falsehood" nil (f ae1 l s hh uh1 d))
("falsehoods" nil (f ae1 l s hh uh1 d z))
("falsely" nil (f ao1 l s l iy0))
("falsetto" nil (f ao0 l s eh1 t ow1))
("falsification" nil (f ae1 l s ax f ax k ey1 sh ax n))
("falsified" nil (f ao1 l s ax f ay1 d))
("falsify" nil (f ao1 l s ax f ay1))
("falsifying" nil (f ao1 l s ax f ay1 ih0 ng))
("falsity" nil (f ao1 l s ax t iy0))
("falsone" nil (f ao1 l s ax n))
("falstaff" nil (f ao1 l s t ae1 f))
("falter" nil (f ao1 l t er0))
("faltered" nil (f ao1 l t er0 d))
("faltering" nil (f ao1 l t er0 ih0 ng))
("falters" nil (f ao1 l t er0 z))
("falvey" nil (f ae1 l v iy0))
("falvo" nil (f aa1 l v ow0))
("falwell" nil (f ao1 l w eh1 l))
("falzarano" nil (f aa0 l z aa0 r aa1 n ow0))
("falzon" nil (f aa1 l z ax n))
("falzone" nil (f aa0 l z ow1 n iy0))
("fama" nil (f aa1 m ax))
("fambro" nil (f ae1 m b r ow0))
("fambrough" nil (f ae1 m b r aw0))
("fame" nil (f ey1 m))
("famed" nil (f ey1 m d))
("famer" nil (f ey1 m er0))
("famiglietti" nil (f ax m ih1 g l iy0 eh1 t iy0))
("familar" nil (f ae1 m ax l er0))
("familial" nil (f ax m ih1 l y ax l))
("familiar" nil (f ax m ih1 l y er0))
("familiarity" nil (f ax m ih1 l y eh1 r ax t iy0))
("familiarize" nil (f ax m ih1 l y er0 ay1 z))
("familiarized" nil (f ax m ih1 l y er0 ay1 z d))
("families" nil (f ae1 m ax l iy0 z))
("family" nil (f ae1 m ax l iy0))
("famine" nil (f ae1 m ax n))
("famines" nil (f ae1 m ax n z))
("famous" nil (f ey1 m ax s))
("famously" nil (f ey1 m ax s l iy0))
("famularo" nil (f aa0 m uw0 l aa1 r ow0))
("fan" nil (f ae1 n))
("fanara" nil (f aa0 n aa1 r ax))
("fanatic" nil (f ax n ae1 t ih0 k))
("fanatical" nil (f ax n ae1 t ih0 k ax l))
("fanatically" nil (f ax n ae1 t ih0 k ax l iy0))
("fanaticism" nil (f ax n ae1 t ax s ih1 z ax m))
("fanatics" nil (f ax n ae1 t ih0 k s))
("fancher" nil (f ae1 n ch er0))
("fanchon" nil (f ae1 n ch ax n))
("fancied" nil (f ae1 n s iy0 d))
("fancier" nil (f ae1 n s iy0 er0))
("fanciers" nil (f ae1 n s iy0 er0 z))
("fancies" nil (f ae1 n s iy0 z))
("fanciest" nil (f ae1 n s iy0 ax s t))
("fanciful" nil (f ae1 n s ih0 f ax l))
("fancy" nil (f ae1 n s iy0))
("fandango" nil (f ae0 n d ae1 ng g ow1))
("fandel" nil (f ae1 n d ax l))
("fandrich" nil (f ae1 n d r ih0 k))
("fane" nil (f ey1 n))
("fanech" nil (f ae1 n eh1 k))
("fanelli" nil (f ax n eh1 l iy0))
("faneuil" nil (f ae1 n y uw0 ih0 l))
("fanfare" nil (f ae1 n f eh1 r))
("fang" nil (f ae1 ng))
("fanger" nil (f ae1 ng er0))
("fangled" nil (f ae1 ng g ax l d))
("fangman" nil (f ae1 ng m ax n))
("fangs" nil (f ae1 ng z))
("fanguy" nil (f ax ng g w iy1))
("fankhauser" nil (f ae1 ng k hh aw1 z er0))
("fanlike" nil (f ae1 n l ay1 k))
("fann" nil (f ae1 n))
("fanned" nil (f ae1 n d))
("fannie" nil (f ae1 n iy0))
("fannin" nil (f ae1 n ih0 n))
("fanning" nil (f ae1 n ih0 ng))
("fannon" nil (f ae1 n ax n))
("fanny" nil (f ae1 n iy0))
("fans" nil (f ae1 n z))
("fansler" nil (f ae1 n s ax l er0))
("fant" nil (f ae1 n t))
("fanta" nil (f aa1 n t ax))
("fantail" nil (f ae1 n t ey1 l))
("fantasia" nil (f ae0 n t ey1 zh ax))
("fantasies" nil (f ae1 n t ax s iy0 z))
("fantasize" nil (f ae1 n t ax s ay1 z))
("fantasized" nil (f ae1 n t ih0 s ay1 z d))
("fantasizes" nil (f ae1 n t ax s ay1 z ih0 z))
("fantasizing" nil (f ae1 n t ax s ay1 z ih0 ng))
("fantastic" nil (f ae0 n t ae1 s t ih0 k))
("fantastically" nil (f ae0 n t ae1 s t ih0 k l iy0))
("fantastik" nil (f ae1 n t ae1 s t ih0 k))
("fantasy" nil (f ae1 n t ax s iy0))
("fantini" nil (f aa0 n t iy1 n iy0))
("fantle" nil (f ae1 n t ax l))
("fanton" nil (f ae1 n t ax n))
("fantozzi" nil (f aa0 n t ow1 t s iy0))
("fantroy" nil (f ae1 n t r oy1))
("fanuc" nil (f ae1 n ax k))
("fanucchi" nil (f aa0 n uw1 k iy0))
("fanueil" nil (f ae1 n y ax l))
("fao" nil (f aw1))
("far" nil (f aa1 r))
("farabaugh" nil (f ae1 r ax b ao1))
("farabee" nil (f ae1 r ax b iy0))
("farabundo" nil (f eh1 r ax b uw1 n d ow1))
("farace" nil (f aa0 r aa1 ch iy0))
("faraci" nil (f aa0 r aa1 ch iy0))
("faraday" nil (f ae1 r ax d ey1))
("faragher" nil (f ae1 r ax g hh er0))
("farago" nil (f aa0 r aa1 g ow0))
("farah" nil (f eh1 r ax))
("faranda" nil (f er0 ae1 n d ax))
("faraone" nil (f aa0 r aa0 ow1 n iy0))
("faraway" nil (f aa1 r ax w ey1))
("farb" nil (f aa1 r b))
("farben" nil (f aa1 r b ax n))
("farber" nil (f aa1 r b er0))
("farberware" nil (f aa1 r b er0 w eh1 r))
("farbman" nil (f aa1 r b m ax n))
("farce" nil (f aa1 r s))
("farcical" nil (f aa1 r s ax k ax l))
("fare" nil (f eh1 r))
("fared" nil (f eh1 r d))
("fares" nil (f eh1 r z))
("farese" nil (f aa0 r ey1 z iy0))
("farewell" nil (f eh1 r w eh1 l))
("farfan" nil (f aa1 r f ax n))
("farfetched" nil (f aa1 r f eh1 ch t))
("farge" nil (f aa1 r jh))
("fargnoli" nil (f aa0 r g n ow1 l iy0))
("fargo" nil (f aa1 r g ow1))
("farha" nil (f aa1 r hh ax))
("farhat" nil (f aa1 r hh ae1 t))
("faria" nil (f aa1 r iy0 ax))
("farias" nil (f eh1 r iy0 ax z))
("farica" nil (f aa0 r iy1 k ax))
("fariello" nil (f aa0 r iy0 eh1 l ow0))
("faries" nil (f eh0 r iy1 z))
("farina" nil (f er0 iy1 n ax))
("farinacci" nil (f aa0 r iy0 n aa1 ch iy0))
("farinas" nil (f er0 iy1 n ax z))
("farinella" nil (f aa0 r iy0 n eh1 l ax))
("faring" nil (f eh1 r iy0 ng))
("farino" nil (f aa0 r iy1 n ow0))
("faris" nil (f eh1 r ih0 s))
("farish" nil (f eh1 r ih0 sh))
("fariss" nil (f er0 ih1 s))
("farkas" nil (f aa1 r k ax s))
("farland" nil (f aa1 r l ax n d))
("farlee" nil (f aa1 r l iy1))
("farler" nil (f aa1 r l er0))
("farless" nil (f aa1 r l ax s))
("farley" nil (f aa1 r l iy0))
("farling" nil (f aa1 r l ih0 ng))
("farlow" nil (f aa1 r l ow1))
("farly" nil (f aa1 r l iy0))
("farm" nil (f aa1 r m))
("farman" nil (f aa1 r m ax n))
("farmboy" nil (f aa1 r m b oy1))
("farmed" nil (f aa1 r m d))
("farmer" nil (f aa1 r m er0))
("farmers" nil (f aa1 r m er0 z))
("farmhand" nil (f aa1 r m hh ae1 n d))
("farmhands" nil (f aa1 r m hh ae1 n d z))
("farmhouse" nil (f aa1 r m hh aw1 s))
("farmhouses" nil (f aa1 r m hh aw1 s ax z))
("farming" nil (f aa1 r m ih0 ng))
("farmingdale" nil (f aa1 r m ih0 ng d ey1 l))
("farmington" nil (f aa1 r m ih0 ng t ax n))
("farmland" nil (f aa1 r m l ae1 n d))
("farmlands" nil (f aa1 r m l ae1 n d z))
("farms" nil (f aa1 r m z))
("farmstead" nil (f aa1 r m s t eh1 d))
("farmworker" nil (f aa1 r m w er1 k er0))
("farmworkers" nil (f aa1 r m w er1 k er0 z))
("farmyard" nil (f aa1 r m y aa1 r d))
("farnall" nil (f aa1 r n ax l))
("farnam" nil (f aa1 r n ax m))
("farnan" nil (f aa1 r n ax n))
("farnborough" nil (f aa1 r n b er0 ow0))
("farnell" nil (f aa1 r n ax l))
("farner" nil (f aa1 r n er0))
("farnes" nil (f aa1 r n z))
("farney" nil (f aa1 r n iy0))
("farnham" nil (f aa1 r n ax m))
("farnley" nil (f aa1 r n l iy0))
("farnsworth" nil (f aa1 r n s w er0 th))
("farnum" nil (f aa1 r n ax m))
("farnworth" nil (f aa1 r n w er0 th))
("faro" nil (f eh1 r ow0))
("farold" nil (f ae1 r ow0 l d))
("faron" nil (f eh1 r ax n))
("farone" nil (f er0 ow1 n))
("farquar" nil (f aa0 r k w aa1 r))
("farquhar" nil (f aa1 r k w er0))
("farquharson" nil (f aa0 r k ah1 hh aa0 r s ax n))
("farr" nil (f aa1 r))
("farra" nil (f aa1 r ax))
("farragut" nil (f eh1 r ax g ax t))
("farrah" nil (f eh1 r ax))
("farrakhan" nil (f aa1 r ax k aa1 n))
("farran" nil (f ae1 r ax n))
("farrand" nil (f ae1 r ax n d))
("farrant" nil (f aa1 r ax n t))
("farrar" nil (f er0 aa1 r))
("farrel" nil (f eh1 r ax l))
("farrell" nil (f eh1 r ih0 l))
("farrelly" nil (f eh1 r ax l iy0))
("farren" nil (f eh1 r ax n))
("farrens" nil (f ae1 r ax n z))
("farrer" nil (f aa1 r er0))
("farrey" nil (f ae1 r iy0))
("farrier" nil (f eh1 r iy0 er0))
("farrington" nil (f eh1 r ih0 ng t ax n))
("farrior" nil (f aa0 r iy1 er0))
("farris" nil (f eh1 r ih0 s))
("farrish" nil (f ae1 r ih0 sh))
("farro" nil (f aa1 r ow0))
("farrow" nil (f eh1 r ow1))
("farrowing" nil (f ae1 r ow1 ih0 ng))
("farruggia" nil (f aa0 r uw1 g jh ax))
("farrugia" nil (f aa0 r uw1 jh ax))
("farry" nil (f ae1 r iy0))
("farsi" nil (f aa1 r s iy0))
("farsighted" nil (f aa1 r s ay1 t ax d))
("farsightedness" nil (f aa1 r s ay1 t ih0 d n ax s))
("farson" nil (f aa1 r s ax n))
("farther" nil (f aa1 r dh er0))
("farthest" nil (f aa1 r dh ax s t))
("farthing" nil (f aa1 r dh ih0 ng))
("farthingale" nil (f aa1 r dh ih0 ng g ey1 l))
("farve" nil (f aa1 r v))
("farver" nil (f aa1 r v er0))
("farwell" nil (f aa1 r w eh1 l))
("farwest" nil (f aa1 r w eh1 s t))
("fasano" nil (f aa0 s aa1 n ow0))
("fasbender" nil (f ae1 s b eh1 n d er0))
("fascell" nil (f ax s eh1 l))
("fasching" nil (f ae1 sh ih0 ng))
("fasciano" nil (f aa0 s ch iy0 aa1 n ow0))
("fascinate" nil (f ae1 s ax n ey1 t))
("fascinated" nil (f ae1 s ax n ey1 t ax d))
("fascinates" nil (f ae1 s ax n ey1 t s))
("fascinating" nil (f ae1 s ax n ey1 t ih0 ng))
("fascination" nil (f ae1 s ax n ey1 sh ax n))
("fascism" nil (f ae1 sh ih1 z ax m))
("fascist" nil (f ae1 sh ax s t))
("fascists" nil (f ae1 sh ax s t s))
("fashion" nil (f ae1 sh ax n))
("fashionable" nil (f ae1 sh ax n ax b ax l))
("fashionably" nil (f ae1 sh ax n ax b l iy0))
("fashioned" nil (f ae1 sh ax n d))
("fashioning" nil (f ae1 sh ax n ih0 ng))
("fashions" nil (f ae1 sh ax n z))
("fasick" nil (f ae1 s ih0 k))
("fasig" nil (f ae1 s ih0 g))
("fasnacht" nil (f ae1 s n ax k t))
("faso" nil (f aa1 s ow0))
("fason" nil (f ae1 s ax n))
("fasone" nil (f aa0 s ow1 n iy0))
("fass" nil (f ae1 s))
("fassbender" nil (f ae1 s b eh1 n d er0))
("fassett" nil (f ae1 s ih0 t))
("fassler" nil (f ae1 s l er0))
("fassnacht" nil (f ae1 s n ax k t))
("fast" nil (f ae1 s t))
("fastball" nil (f ae1 s t b ao1 l))
("fasten" nil (f ae1 s ax n))
("fastened" nil (f ae1 s ax n d))
("fastener" nil (f ae1 s ax n er0))
("fasteners" nil (f ae1 s ax n er0 z))
("fastening" nil (f ae1 s ax n ih0 ng))
("fastenings" nil (f ae1 s ax n ih0 ng z))
("faster" nil (f ae1 s t er0))
("fastest" nil (f ae1 s t ax s t))
("fastfood" nil (f ae1 s t f uw1 d))
("fastidious" nil (f ae0 s t ih1 d iy0 ax s))
("fasting" nil (f ae1 s t ih0 ng))
("fastness" nil (f ae1 s t n ax s))
("fasts" nil (f ae1 s t s))
("fasulo" nil (f ax s uw1 l ow0))
("fat" nil (f ae1 t))
("fata" nil (f aa1 t ax))
("fatah" nil (f aa1 t ax))
("fatal" nil (f ey1 t ax l))
("fatalism" nil (f ey1 t ax l ih1 z ax m))
("fatalities" nil (f ax t ae1 l ax t iy0 z))
("fatality" nil (f ax t ae1 l ax t iy0))
("fatally" nil (f ey1 t ax l iy0))
("fate" nil (f ey1 t))
("fated" nil (f ey1 t ih0 d))
("fateful" nil (f ey1 t f ax l))
("fates" nil (f ey1 t s))
("fath" nil (f ae1 th))
("father" nil (f aa1 dh er0))
("fathered" nil (f aa1 dh er0 d))
("fatheree" nil (f ae0 th er0 iy1))
("fatherhood" nil (f aa1 dh er0 hh uh1 d))
("fathering" nil (f aa1 dh er0 ih0 ng))
("fatherland" nil (f aa1 dh er0 l ae1 n d))
("fatherless" nil (f aa1 dh er0 l ax s))
("fatherly" nil (f aa1 dh er0 l iy0))
("fathers" nil (f aa1 dh er0 z))
("fathi" nil (f ae1 th iy0))
("fathom" nil (f ae1 dh ax m))
("fathomable" nil (f ae1 dh ax m ax b ax l))
("fathoms" nil (f ae1 dh ax m z))
("fatigue" nil (f ax t iy1 g))
("fatigued" nil (f ax t iy1 g d))
("fatigues" nil (f ax t iy1 g z))
("fats" nil (f ae1 t s))
("fatten" nil (f ae1 t ax n))
("fattened" nil (f ae1 t ax n d))
("fattening" nil (f ae1 t ax n ih0 ng))
("fatter" nil (f ae1 t er0))
("fattest" nil (f ae1 t ax s t))
("fatties" nil (f ae1 t iy0 z))
("fatty" nil (f ae1 t iy0))
("fatula" nil (f aa0 t uw1 l ax))
("fatuous" nil (f ae1 ch ax w ax s))
("fatzinger" nil (f ey1 t z ih0 ng er0))
("faubel" nil (f aw1 b ax l))
("fauber" nil (f aw1 b er0))
("faubert" nil (f aw1 b er0 t))
("faubion" nil (f aw1 b iy0 ax n))
("fauble" nil (f ao1 b ax l))
("faucet" nil (f ao1 s ax t))
("faucets" nil (f ao1 s ax t s))
("faucett" nil (f ao1 s ih0 t))
("faucette" nil (f ow0 s eh1 t))
("faucher" nil (f aw1 k er0))
("faucheux" nil (f ow0 sh ow1))
("fauci" nil (f ao1 s iy0))
("faughn" nil (f ao1 n))
("faughnan" nil (f ao1 n ax n))
("faught" nil (f ao1 t))
("faul" nil (f ao1 l))
("faulcon" nil (f ao1 l k ax n))
("faulconer" nil (f ao1 l k ax n er0))
("faulding" nil (f ao1 l d ih0 ng))
("faulds" nil (f ao1 l d z))
("faulhaber" nil (f aw1 l hh ax b er0))
("faulk" nil (f ao1 k))
("faulkenberry" nil (f ao1 l k ax n b eh1 r iy0))
("faulkner" nil (f ao1 k n er0))
("faulks" nil (f ao1 k s))
("faull" nil (f ao1 l))
("faulstich" nil (f ao1 l s t ih0 ch))
("fault" nil (f ao1 l t))
("faulted" nil (f ao1 l t ih0 d))
("faulting" nil (f ao1 l t ih0 ng))
("faults" nil (f ao1 l t s))
("faulty" nil (f ao1 l t iy0))
("fauna" nil (f ao1 n ax))
("faunal" nil (f aa1 n ax l))
("faunce" nil (f ao1 n s))
("fauntleroy" nil (f ao1 n t l er0 oy1))
("fauntroy" nil (f ao1 n t r oy1))
("faupel" nil (f ow0 p eh1 l))
("faure" nil (f ao1 r))
("faurot" nil (f ao0 r ow1))
("fauroux" nil (f ao0 r uw1))
("faus" nil (f ao1 z))
("fauser" nil (f aw1 s er0))
("fausett" nil (f ao1 s ih0 t))
("fausey" nil (f ao1 s iy0))
("fausnaugh" nil (f ao1 s n ao0))
("fauss" nil (f ao1 s))
("faust" nil (f aw1 s t))
("fausta" nil (f ao1 s t ax))
("faustian" nil (f ao1 s ch ax n))
("faustina" nil (f ao1 s t iy1 n ax))
("faustine" nil (f aw1 s t iy0 n))
("faustino" nil (f ao1 s t iy1 n ow0))
("fausto" nil (f ao1 s t ow0))
("faustus" nil (f ao1 s t ax s))
("fauteux" nil (f ow0 t ow1))
("fauth" nil (f ao1 th))
("fauver" nil (f aw1 v er0))
("faux" nil (f ao1 k s))
("fava" nil (f aa1 v ax))
("favale" nil (f aa0 v aa1 l iy0))
("favaloro" nil (f aa0 v aa0 l ao1 r ow0))
("favaro" nil (f aa0 v aa1 r ow0))
("favata" nil (f aa0 v aa1 t ax))
("favazza" nil (f aa0 v aa1 t s ax))
("favela" nil (f aa0 v eh1 l ax))
("faver" nil (f ey1 v er0))
("favero" nil (f aa0 v eh1 r ow0))
("favia" nil (f aa1 v iy0 ax))
("favinger" nil (f ey1 v ih0 ng er0))
("favor" nil (f ey1 v er0))
("favorability" nil (f ae1 v er0 ax b ih1 l ih0 t iy0))
("favorable" nil (f ey1 v er0 ax b ax l))
("favorably" nil (f ey1 v er0 ax b l iy0))
("favored" nil (f ey1 v er0 d))
("favoring" nil (f ey1 v er0 ih0 ng))
("favorite" nil (f ey1 v er0 ih0 t))
("favorites" nil (f ey1 v er0 ih0 t s))
("favoritism" nil (f ey1 v er0 ih0 t ih1 z ax m))
("favors" nil (f ey1 v er0 z))
("favre" nil (f ey1 v er0))
("favreau" nil (f ax v r ow1))
("favro" nil (f ae1 v r ow0))
("faw" nil (f ao1))
("fawbush" nil (f ao1 b ax sh))
("fawcett" nil (f ao1 s ih0 t))
("fawkes" nil (f ao1 k s))
("fawley" nil (f ao1 l iy0))
("fawn" nil (f ao1 n))
("fawning" nil (f ao1 n ih0 ng))
("fawver" nil (f ao1 v er0))
("fax" nil (f ae1 k s))
("faxed" nil (f ae1 k s t))
("faxes" nil (f ae1 k s ih0 z))
("faxing" nil (f ae1 k s ih0 ng))
("faxon" nil (f ae1 k s ax n))
("fay" nil (f ey1))
("fayanne" nil (f ey1 ih0 n))
("fayard" nil (f ax y aa1 r d))
("faye" nil (f ey1))
("fayed" nil (f ey1 d))
("fayette" nil (f ey1 eh1 t))
("fayetteville" nil (f ey1 eh1 t v ih1 l))
("fayez" nil (f ey1 eh0 z))
("fayme" nil (f ey1 m))
("faymonville" nil (f ey1 m ax n v ih1 l))
("fayne" nil (f ey1 n))
("fayrouz" nil (f ey1 r uw1 z))
("faz" nil (f ae1 z))
("faze" nil (f ey1 z))
("fazed" nil (f ey1 z d))
("fazekas" nil (f ae1 z ih0 k ax z))
("fazenbaker" nil (f ey1 z ax n b ey1 k er0))
("fazio" nil (f ey1 z iy0 ow0))
("fazzino" nil (f aa0 t s iy1 n ow0))
("fazzio" nil (f ae1 z iy0 ow0))
("fe" nil (f ey1))
("feagan" nil (f ey1 g ax n))
("feagans" nil (f iy1 g ax n z))
("feagin" nil (f iy1 jh ih0 n))
("feagins" nil (f iy1 jh ih0 n z))
("feagle" nil (f iy1 g ax l))
("fealty" nil (f iy1 ax l t iy0))
("fear" nil (f ih1 r))
("feared" nil (f ih1 r d))
("fearful" nil (f ih1 r f ax l))
("fearing" nil (f ih1 r ih0 ng))
("fearless" nil (f ih1 r l ax s))
("fearn" nil (f er1 n))
("fearnow" nil (f er1 n ow0))
("fearon" nil (f ih1 r ax n))
("fears" nil (f ih1 r z))
("fearsome" nil (f ih1 r s ax m))
("feasel" nil (f iy1 z ax l))
("feasibility" nil (f iy1 z ax b ih1 l ax t iy0))
("feasible" nil (f iy1 z ax b ax l))
("feast" nil (f iy1 s t))
("feasted" nil (f iy1 s t ih0 d))
("feaster" nil (f iy1 s t er0))
("feasting" nil (f iy1 s t ih0 ng))
("feasts" nil (f iy1 s t s))
("feat" nil (f iy1 t))
("feather" nil (f eh1 dh er0))
("featherbedding" nil (f eh1 dh er0 b eh1 d ih0 ng))
("feathered" nil (f eh1 dh er0 d))
("feathering" nil (f eh1 dh er0 ih0 ng))
("featherless" nil (f eh1 dh er0 l ax s))
("featherly" nil (f eh1 dh er0 l iy0))
("feathers" nil (f eh1 dh er0 z))
("featherston" nil (f eh0 th er1 s t ax n))
("featherstone" nil (f eh1 dh er0 s t ow1 n))
("featherweight" nil (f eh1 dh er0 w ey1 t))
("feathery" nil (f eh1 dh er0 iy0))
("feats" nil (f iy1 t s))
("feature" nil (f iy1 ch er0))
("featured" nil (f iy1 ch er0 d))
("featureless" nil (f iy1 ch er0 l ax s))
("features" nil (f iy1 ch er0 z))
("featuring" nil (f iy1 ch er0 ih0 ng))
("feazel" nil (f iy1 z ax l))
("feazell" nil (f iy1 z ax l))
("feb" nil (f eh1 b y ax w eh1 r iy0))
("febles" nil (f eh1 b ax l z))
("febres" nil (f eh1 b r ax s))
("february" nil (f eh1 b y ax w eh1 r iy0))
("fecal" nil (f iy1 k ax l))
("feces" nil (f iy1 s iy1 z))
("fecher" nil (f eh1 k er0))
("fechner" nil (f eh1 k n er0))
("fecht" nil (f eh1 k t))
("fechter" nil (f eh1 k t er0))
("feck" nil (f eh1 k))
("feckless" nil (f eh1 k l ih0 s))
("fecteau" nil (f ih0 k t ow1))
("fed" nil (f eh1 d))
("fedak" nil (f eh1 d ax k))
("fedder" nil (f eh1 d er0))
("fedders" nil (f eh1 d er0 z))
("feddersen" nil (f eh1 d er0 s ax n))
("fede" nil (f iy1 d))
("fedeccredito" nil (f eh0 d eh1 k r eh0 d iy1 t ow0))
("fedele" nil (f eh1 d ax l))
("fedeli" nil (f eh0 d eh1 l iy0))
("feder" nil (f eh1 d er0))
("federal" nil (f eh1 d er0 ax l))
("federalism" nil (f eh1 d er0 ax l ih1 z ax m))
("federalist" nil (f eh1 d er0 ax l ih0 s t))
("federalists" nil (f eh1 d er0 ax l ih0 s t s))
("federalized" nil (f eh1 d er0 ax l ay1 z d))
("federally" nil (f eh1 d er0 ax l iy0))
("federals" nil (f eh1 d er0 ax l z))
("federated" nil (f eh1 d er0 ey1 t ih0 d))
("federation" nil (f eh1 d er0 ey1 sh ax n))
("federations" nil (f eh1 d er0 ey1 sh ax n z))
("federative" nil (f eh1 d er0 ax t ih0 v))
("federbush" nil (f eh1 d er0 b uh1 sh))
("federer" nil (f eh1 d er0 er0))
("federica" nil (f eh0 d er0 iy1 k ax))
("federici" nil (f eh0 d er0 iy1 ch iy0))
("federico" nil (f eh0 d er0 iy1 k ow0))
("federle" nil (f eh1 d er0 ax l))
("federman" nil (f iy1 d er0 m ax n))
("federoff" nil (f eh1 d er0 ao1 f))
("federspiel" nil (f eh1 d er0 s p iy0 l))
("fedewa" nil (f ih0 d uw1 ax))
("fedex" nil (f eh1 d eh1 k s))
("fediay" nil (f iy1 d iy0 ey1))
("fedler" nil (f eh1 d l er0))
("fedor" nil (f eh1 d er0))
("fedora" nil (f ih0 d ao1 r ax))
("fedorchak" nil (f eh1 d er0 k ax k))
("fedorko" nil (f ih0 d ao1 r k ow0))
("fedrick" nil (f eh1 d r ih0 k))
("feds" nil (f eh1 d z))
("fee" nil (f iy1))
("feeback" nil (f iy1 b ae1 k))
("feeble" nil (f iy1 b ax l))
("feebly" nil (f iy1 b l iy0))
("feed" nil (f iy1 d))
("feedback" nil (f iy1 d b ae1 k))
("feeder" nil (f iy1 d er0))
("feeders" nil (f iy1 d er0 z))
("feeding" nil (f iy1 d ih0 ng))
("feedings" nil (f iy1 d ih0 ng z))
("feedlot" nil (f iy1 d l aa1 t))
("feedlots" nil (f iy1 d l aa1 t s))
("feeds" nil (f iy1 d z))
("feedstock" nil (f iy1 d s t aa1 k))
("feedstocks" nil (f iy1 d s t aa1 k s))
("feehan" nil (f iy1 ax n))
("feel" nil (f iy1 l))
("feeler" nil (f iy1 l er0))
("feelers" nil (f iy1 l er0 z))
("feeley" nil (f iy1 l iy0))
("feeling" nil (f iy1 l ih0 ng))
("feelings" nil (f iy1 l ih0 ng z))
("feels" nil (f iy1 l z))
("feely" nil (f iy1 l iy0))
("feemster" nil (f iy1 m s t er0))
("feeney" nil (f iy1 n iy0))
("feenstra" nil (f iy1 n s t r ax))
("feeny" nil (f iy1 n iy0))
("fees" nil (f iy1 z))
("feese" nil (f iy1 z))
("feeser" nil (f iy1 z er0))
("feet" nil (f iy1 t))
("feezor" nil (f iy1 z er0))
("feffer" nil (f eh1 f er0))
("fegan" nil (f eh1 g ax n))
("feger" nil (f iy1 g er0))
("fegley" nil (f eh1 g l iy0))
("feher" nil (f eh1 hh er0))
("fehl" nil (f eh1 l))
("fehling" nil (f eh1 l ih0 ng))
("fehlman" nil (f eh1 l m ax n))
("fehn" nil (f eh1 n))
("fehnel" nil (f eh1 n ax l))
("fehr" nil (f eh1 r))
("fehrenbach" nil (f eh1 r ih0 n b aa0 k))
("fehrenbacher" nil (f eh1 r ih0 n b aa0 k er0))
("fehring" nil (f eh1 r ih0 ng))
("fehringer" nil (f eh1 r ih0 ng er0))
("fehrman" nil (f eh1 r m ax n))
("feick" nil (f iy1 k))
("feickert" nil (f ay1 k er0 t))
("feig" nil (f iy1 g))
("feige" nil (f iy1 jh))
("feigel" nil (f ay1 g ax l))
("feigen" nil (f ay1 g ax n))
("feigenbaum" nil (f ay1 g ax n b aw1 m))
("feighan" nil (f ey1 g ax n))
("feighner" nil (f ey1 n er0))
("feight" nil (f ey1 t))
("feigin" nil (f ay1 g ih0 n))
("feign" nil (f ey1 n))
("feigned" nil (f ey1 n d))
("feigning" nil (f ey1 n ih0 ng))
("feil" nil (f iy1 l))
("feild" nil (f iy1 l d))
("feiler" nil (f ay1 l er0))
("feimster" nil (f ay1 m s t er0))
("fein" nil (f ay1 n))
("feinauer" nil (f ay1 n aw0 er0))
("feinberg" nil (f ay1 n b er0 g))
("feiner" nil (f ay1 n er0))
("feingold" nil (f ay1 ng g ow0 l d))
("feinman" nil (f ay1 n m ax n))
("feinstein" nil (f ay1 n s t ay1 n))
("feint" nil (f ey1 n t))
("feist" nil (f ay1 s t))
("feister" nil (f ay1 s t er0))
("feisty" nil (f ay1 s t iy0))
("feit" nil (f ay1 t))
("feith" nil (f ay1 th))
("feitz" nil (f ay1 t s))
("fejes" nil (f ih0 zh iy1 z))
("fekete" nil (f eh1 k iy0 t))
("fel" nil (f eh1 l))
("fela" nil (f eh1 l ax))
("felan" nil (f eh1 l ax n))
("felbatol" nil (f eh1 l b ax t aa0 l))
("felber" nil (f eh1 l b er0))
("felch" nil (f eh1 l ch))
("feld" nil (f eh1 l d))
("felda" nil (f eh1 l d ax))
("feldberg" nil (f eh1 l d b er0 g))
("felde" nil (f eh1 l d))
("feldene" nil (f eh0 l d iy1 n))
("felder" nil (f eh1 l d er0))
("felderman" nil (f eh1 l d er0 m ax n))
("feldhaus" nil (f eh1 l d hh aw1 s))
("feldkamp" nil (f eh1 l d k ae1 m p))
("feldman" nil (f eh1 l d m ax n))
("feldmann" nil (f eh1 l d m ax n))
("feldmeier" nil (f eh1 l d m ay0 er0))
("feldmuehle" nil (f eh1 l d m y uw1 l ax))
("feldner" nil (f eh1 l d n er0))
("feldpausch" nil (f eh1 l d p aw1 sh))
("feldspar" nil (f eh1 l d s p aa1 r))
("feldspars" nil (f eh1 l d s p aa1 r z))
("feldstein" nil (f eh1 l d s t ay0 n))
("feldt" nil (f eh1 l t))
("felgenhauer" nil (f eh1 l g ih0 n hh aw0 er0))
("felger" nil (f eh1 l g er0))
("felice" nil (f ax l iy1 s))
("felicetti" nil (f eh0 l iy0 ch eh1 t iy0))
("felicia" nil (f ax l iy1 sh ax))
("feliciano" nil (f ax l iy0 s iy0 aa1 n ow0))
("felicite" nil (f eh1 l ih0 s ay1 t))
("felicitous" nil (f ih0 l ih1 s ax t ax s))
("felicity" nil (f ih0 l ih1 s ax t iy0))
("feline" nil (f iy1 l ay1 n))
("felines" nil (f iy1 l ay1 n z))
("felipe" nil (f ax l iy1 p))
("felis" nil (f eh1 l ih0 s))
("felix" nil (f iy1 l ih0 k s))
("feliz" nil (f eh1 l ih0 z))
("felkel" nil (f eh1 l k ax l))
("felker" nil (f eh1 l k er0))
("felkins" nil (f eh1 l k ih0 n z))
("felkner" nil (f eh1 l k n er0))
("fell" nil (f eh1 l))
("fella" nil (f eh1 l ax))
("felland" nil (f eh1 l ax n d))
("fellas" nil (f eh1 l ax s))
("felled" nil (f eh1 l d))
("fellenz" nil (f eh1 l ih0 n s))
("feller" nil (f eh1 l er0))
("fellers" nil (f eh1 l er0 z))
("fellheimer" nil (f eh1 l hh ay1 m er0))
("fellin" nil (f eh1 l ih0 n))
("felling" nil (f eh1 l ih0 ng))
("fellinger" nil (f eh1 l ih0 ng er0))
("fellman" nil (f eh1 l m ax n))
("fellmeth" nil (f eh1 l m ax th))
("fellner" nil (f eh1 l n er0))
("fellow" nil (f eh1 l ow0))
("fellowes" nil (f eh1 l ow1 z))
("fellows" nil (f eh1 l ow0 z))
("fellowship" nil (f eh1 l ow0 sh ih1 p))
("fellowships" nil (f eh1 l ow0 sh ih1 p s))
("fells" nil (f eh1 l z))
("fellsway" nil (f eh1 l z w ey1))
("felman" nil (f eh1 l m ax n))
("felmlee" nil (f eh1 l m l iy0))
("felon" nil (f eh1 l ax n))
("felonies" nil (f eh1 l ax n iy0 z))
("felons" nil (f eh1 l ax n z))
("felony" nil (f eh1 l ax n iy0))
("felps" nil (f eh1 l p s))
("felrice" nil (f eh1 l r ay1 s))
("fels" nil (f eh1 l z))
("felsenthal" nil (f eh1 l z ax n th aa1 l))
("felske" nil (f eh1 l s k))
("felt" nil (f eh1 l t))
("felten" nil (f eh1 l t ax n))
("felter" nil (f eh1 l t er0))
("feltes" nil (f eh1 l t s))
("feltman" nil (f eh1 l t m ax n))
("feltner" nil (f eh1 l t n er0))
("felton" nil (f eh1 l t ax n))
("feltrax" nil (f eh1 l t r ae0 k s))
("felts" nil (f eh1 l t s))
("feltsman" nil (f eh1 l t s m ax n))
("feltus" nil (f eh1 l t ax s))
("felty" nil (f eh1 l t iy0))
("feltz" nil (f eh1 l t s))
("felucca" nil (f ih0 l ah1 k ax))
("felver" nil (f eh1 l v er0))
("female" nil (f iy1 m ey1 l))
("females" nil (f iy1 m ey1 l z))
("femia" nil (f iy1 m iy0 ax))
("femina" nil (f eh1 m ih0 n ax))
("feminine" nil (f eh1 m ax n ax n))
("femininity" nil (f eh1 m ax n ih1 n ax t iy0))
("feminism" nil (f eh1 m ih0 n ih1 z ax m))
("feminist" nil (f eh1 m ax n ax s t))
("feminists" nil (f eh1 m ax n ax s t s))
("femme" nil (f eh1 m))
("femoral" nil (f eh1 m er0 ax l))
("femrite" nil (f eh1 m r ay1 t))
("femsa" nil (f eh1 m s ax))
("femur" nil (f iy1 m er0))
("fence" nil (f eh1 n s))
("fenced" nil (f eh1 n s t))
("fenceless" nil (f eh1 n s l ax s))
("fences" nil (f eh1 n s ax z))
("fencing" nil (f eh1 n s ih0 ng))
("fencl" nil (f eh1 ng k ax l))
("fend" nil (f eh1 n d))
("fended" nil (f eh1 n d ih0 d))
("fender" nil (f eh1 n d er0))
("fenders" nil (f eh1 n d er0 z))
("fenderson" nil (f eh1 n d er0 s ax n))
("fendi" nil (f eh1 n d iy0))
("fending" nil (f eh1 n d ih0 ng))
("fendler" nil (f eh1 n d l er0))
("fendley" nil (f eh1 n d l iy0))
("fendrick" nil (f eh1 n d r ih0 k))
("fends" nil (f eh1 n d z))
("fendt" nil (f eh1 n t))
("fenech" nil (f eh1 n ih0 k))
("fenella" nil (f eh0 n eh1 l ax))
("fenelon" nil (f eh1 n ih0 l aa1 n))
("feng" nil (f eh1 ng))
("fenger" nil (f eh1 ng er0))
("fenian" nil (f iy1 n iy0 ax n))
("fenichell" nil (f eh1 n ih0 ch ax l))
("fenimore" nil (f eh1 n ih0 m ao1 r))
("fenley" nil (f eh1 n l iy0))
("fenlon" nil (f eh1 n l ax n))
("fenn" nil (f eh1 n))
("fennel" nil (f eh1 n ax l))
("fennell" nil (f eh1 n ax l))
("fennelly" nil (f eh1 n ax l iy0))
("fennema" nil (f eh1 n ih0 m ax))
("fenner" nil (f eh1 n er0))
("fennessey" nil (f eh1 n ih0 s iy0))
("fennessy" nil (f eh1 n ih0 s iy0))
("fennewald" nil (f eh1 n ih0 w ao0 l d))
("fenney" nil (f eh1 n iy0))
("fennimore" nil (f eh1 n ih0 m ao1 r))
("fenno" nil (f eh1 n ow0))
("fenoglio" nil (f eh0 n ow1 g l iy0 ow0))
("fenske" nil (f eh1 n s k))
("fenster" nil (f eh1 n s t er0))
("fenstermacher" nil (f eh1 n s t er0 m ax k er0))
("fenstermaker" nil (f eh1 n s t er0 m ey1 k er0))
("fensterstock" nil (f eh1 n s t er0 s t aa1 k))
("fent" nil (f eh1 n t))
("fenter" nil (f eh1 n t er0))
("fenton" nil (f eh1 n t ax n))
("fentress" nil (f eh1 n t r ih0 s))
("fenugreek" nil (f eh1 n uw0 g r iy1 k))
("fenway" nil (f eh1 n w ey1))
("fenwick" nil (f eh1 n w ih1 k))
("fenwood" nil (f eh1 n w uh1 d))
("fenyvessy" nil (f eh1 n ih0 v eh1 s iy0))
("fenzel" nil (f eh1 n z ax l))
("feodor" nil (f iy1 ax d er0))
("feodora" nil (f iy0 ax d ao1 r ax))
("feola" nil (f iy0 aa1 l ax))
("fer" nil (f er1))
("fera" nil (f eh1 r ax))
("feral" nil (f eh1 r ax l))
("ferber" nil (f er1 b er0))
("ferch" nil (f er1 k))
("ferd" nil (f er1 d))
("ferderer" nil (f er1 d er0 er0))
("ferdie" nil (f er1 d iy0))
("ferdig" nil (f er1 d ih0 g))
("ferdinand" nil (f er1 d ih0 n ae1 n d))
("ferdinanda" nil (f er0 d iy0 n aa1 n d ax))
("ferdon" nil (f eh0 r d ao1 n))
("ferebee" nil (f eh1 r ih0 b iy0))
("ferenc" nil (f er0 eh1 n s))
("ference" nil (f ih1 r ax n s))
("ferencz" nil (f eh1 r ih0 n ch))
("ferg" nil (f er1 g))
("fergason" nil (f er1 g ax s ax n))
("ferger" nil (f er1 g er0))
("fergerson" nil (f er1 g er0 s ax n))
("fergeson" nil (f er1 g ax s ax n))
("fergie" nil (f er1 g iy0))
("fergus" nil (f er1 g ax s))
("ferguson" nil (f er1 g ax s ax n))
("fergusson" nil (f er1 g ax s ax n))
("feria" nil (f eh1 r iy0 ax))
("ferko" nil (f er1 k ow0))
("ferland" nil (f er1 l ax n d))
("ferm" nil (f er1 m))
("ferman" nil (f er1 m ax n))
("ferment" nil (f er0 m eh1 n t))
("fermenta" nil (f er0 m eh1 n t ax))
("fermentation" nil (f er1 m ax n t ey1 sh ax n))
("fermented" nil (f er0 m eh1 n t ax d))
("fermenting" nil (f er0 m eh1 n t ih0 ng))
("ferments" nil (f er0 m eh1 n t s))
("fermi" nil (f er1 m iy0))
("fermilab" nil (f er1 m ih0 l ae1 b))
("fermin" nil (f er1 m ih0 n))
("fermium" nil (f eh1 r m iy0 ax m))
("fermoyle" nil (f er1 m oy1 l))
("fern" nil (f er1 n))
("fernald" nil (f er0 n aa1 l d))
("fernand" nil (f er0 n ae1 n d))
("fernanda" nil (f er0 n ae1 n d ax))
("fernandes" nil (f er0 n aa1 n d eh0 s))
("fernandez" nil (f er0 n ae1 n d eh0 z))
("fernandina" nil (f er0 n ax n d iy1 n ax))
("fernando" nil (f er0 n ae1 n d ow0))
("fernao" nil (f er0 n aw1))
("fernau" nil (f er1 n aw0))
("fernberg" nil (f er1 n b er0 g))
("ferndale" nil (f er1 n d ey1 l))
("ferner" nil (f er1 n er0))
("fernery" nil (f er1 n er0 iy0))
("fernholz" nil (f er1 n hh ow0 l z))
("fernicola" nil (f er0 n iy0 k ow1 l ax))
("fernlike" nil (f er1 n l ay1 k))
("ferns" nil (f er1 n z))
("fernstrom" nil (f er1 n s t r ax m))
("fero" nil (f eh1 r ow0))
("ferocious" nil (f er0 ow1 sh ax s))
("ferociously" nil (f er0 ow1 sh ih0 s l iy0))
("ferocity" nil (f er0 aa1 s ax t iy0))
("ferol" nil (f eh1 r ax l))
("ferra" nil (f eh1 r ax))
("ferraioli" nil (f eh1 r ay0 ow0 l iy0))
("ferrall" nil (f eh1 r ax l))
("ferran" nil (f eh1 r ax n))
("ferrand" nil (f eh1 r ax n d))
("ferrandino" nil (f er0 aa0 n d iy1 n ow0))
("ferrando" nil (f eh1 r ae1 n d ow0))
("ferrant" nil (f eh1 r ax n t))
("ferrante" nil (f er0 aa1 n t iy0))
("ferranti" nil (f eh0 r ae1 n t iy0))
("ferrara" nil (f er0 aa1 r ax))
("ferrari" nil (f er0 aa1 r iy0))
("ferrario" nil (f er0 aa1 r iy0 ow0))
("ferraris" nil (f er0 aa1 r ih0 s))
("ferraro" nil (f er0 aa1 r ow0))
("ferre" nil (f eh1 r))
("ferrebee" nil (f eh1 r ih0 b iy0))
("ferree" nil (f er0 iy1))
("ferreira" nil (f eh0 r ey1 r ax))
("ferreiro" nil (f eh0 r eh1 r ow0))
("ferrel" nil (f eh0 r eh1 l))
("ferrell" nil (f eh1 r ih0 l))
("ferrelli" nil (f er0 eh1 l iy0))
("ferren" nil (f eh1 r ax n))
("ferrency" nil (f eh1 r eh1 n s iy0))
("ferrentino" nil (f er0 eh0 n t iy1 n ow0))
("ferrer" nil (f eh1 r er0))
("ferrera" nil (f eh0 r eh1 r ax))
("ferreri" nil (f er0 eh1 r iy0))
("ferrero" nil (f eh0 r eh1 r ow0))
("ferret" nil (f eh1 r ax t))
("ferreted" nil (f eh1 r ax t ax d))
("ferreting" nil (f eh1 r ax t ih0 ng))
("ferrets" nil (f eh1 r ax t s))
("ferretti" nil (f er0 eh1 t iy0))
("ferri" nil (f eh1 r iy0))
("ferrick" nil (f eh1 r ih0 k))
("ferrie" nil (f eh1 r iy0))
("ferried" nil (f eh1 r iy0 d))
("ferrier" nil (f eh1 r iy0 er0))
("ferriera" nil (f er0 ih1 r ax))
("ferries" nil (f eh1 r iy0 z))
("ferrigno" nil (f er0 iy1 g n ow0))
("ferrill" nil (f eh0 r iy1 l))
("ferrin" nil (f eh1 r ih0 n))
("ferrington" nil (f eh1 r ih0 ng t ax n))
("ferrini" nil (f er0 iy1 n iy0))
("ferris" nil (f eh1 r ih0 s))
("ferriss" nil (f eh1 r ih0 s ih0 z))
("ferrite" nil (f eh1 r ay0 t))
("ferriter" nil (f eh1 r ay0 t er0))
("ferriz" nil (f eh1 r ih0 z))
("ferro" nil (f eh1 r ow0))
("ferrochrome" nil (f eh1 r ax k r ow1 m))
("ferrofluidics" nil (f eh1 r ow0 f l uw0 ih1 d ih0 k s))
("ferromagnet" nil (f eh1 r ow0 m ae1 g n ax t))
("ferromagnetic" nil (f eh1 r ow0 m ae1 g n eh1 t ih0 k))
("ferromagnetism" nil (f eh1 r ow0 m ae1 g n ax t ih1 z ax m))
("ferron" nil (f eh1 r ax n))
("ferrone" nil (f er0 ow1 n iy0))
("ferronickel" nil (f eh1 r ax n ih1 k ax l))
("ferrotype" nil (f eh1 r ax t ay1 p))
("ferrous" nil (f eh1 r ax s))
("ferrucci" nil (f er0 uw1 ch iy0))
("ferruzzi" nil (f eh0 r uw1 z iy0))
("ferry" nil (f eh1 r iy0))
("ferrying" nil (f eh1 r iy0 ih0 ng))
("ferryman" nil (f eh1 r iy0 m ax n))
("fertig" nil (f er1 t ih0 g))
("fertile" nil (f er1 t ax l))
("fertility" nil (f er0 t ih1 l ax t iy0))
("fertilization" nil (f er1 t ax l ih0 z ey1 sh ax n))
("fertilize" nil (f er1 t ax l ay1 z))
("fertilized" nil (f er1 t ax l ay1 z d))
("fertilizer" nil (f er1 t ax l ay1 z er0))
("fertilizers" nil (f er1 t ax l ay1 z er0 z))
("fertilizes" nil (f er1 t ax l ay1 z ih0 z))
("fertilizing" nil (f er1 t ax l ay1 z ih0 ng))
("fertitta" nil (f er0 t iy1 t ax))
("ferullo" nil (f er0 uw1 l ow0))
("fervent" nil (f er1 v ax n t))
("fervently" nil (f er1 v ax n t l iy0))
("fervid" nil (f er1 v ax d))
("fervor" nil (f er1 v er0))
("fery" nil (f eh1 r iy0))
("feschbach" nil (f eh1 sh b aa1 k))
("fescue" nil (f eh1 s k y uw1))
("feser" nil (f iy1 z er0))
("feshbach" nil (f eh1 sh b aa1 k))
("fesler" nil (f eh1 s ax l er0))
("fesmire" nil (f eh1 s m ay1 r))
("fesperman" nil (f eh1 s p er0 m ax n))
("fess" nil (f eh1 s))
("fessel" nil (f eh1 s ax l))
("fessenden" nil (f eh1 s ax n d ax n))
("fessler" nil (f eh1 s l er0))
("fest" nil (f eh1 s t))
("festa" nil (f eh1 s t ax))
("fester" nil (f eh1 s t er0))
("festering" nil (f eh1 s t er0 ih0 ng))
("festiva" nil (f eh0 s t iy1 v ax))
("festival" nil (f eh1 s t ax v ax l))
("festivals" nil (f eh1 s t ax v ax l z))
("festive" nil (f eh1 s t ih0 v))
("festivities" nil (f eh0 s t ih1 v ax t iy0 z))
("festivity" nil (f eh0 s t ih1 v ax t iy0))
("festooned" nil (f eh1 s t uw1 n d))
("festspielhaus" nil (f eh1 s t s p iy1 l hh aw1 s))
("festus" nil (f eh1 s t ax s))
("fetal" nil (f iy1 t ax l))
("fetch" nil (f eh1 ch))
("fetched" nil (f eh1 ch t))
("fetches" nil (f eh1 ch ih0 z))
("fetching" nil (f eh1 ch ih0 ng))
("fete" nil (f ey1 t))
("feted" nil (f ey1 t ih0 d))
("fetherolf" nil (f eh1 th er0 ow0 l f))
("fetherston" nil (f eh1 th er0 s t ax n))
("fetid" nil (f eh1 t ax d))
("fetish" nil (f eh1 t ih0 sh))
("fetishism" nil (f eh1 t ih0 sh ih1 z ax m))
("fetishists" nil (f eh1 t ih0 sh ax s t s))
("fetner" nil (f eh1 t n er0))
("fetrow" nil (f eh1 t r ow0))
("fetsch" nil (f eh1 ch))
("fetsko" nil (f eh1 t s k ow0))
("fett" nil (f eh1 t))
("fette" nil (f eh1 t))
("fetter" nil (f eh1 t er0))
("fettered" nil (f eh1 t er0 d))
("fetterhoff" nil (f eh1 t er0 hh ao0 f))
("fetterly" nil (f eh1 t er0 l iy0))
("fetterman" nil (f eh1 t er0 m ax n))
("fetterolf" nil (f eh1 t er0 ow0 l f))
("fetters" nil (f eh1 t er0 z))
("fettes" nil (f eh1 t s))
("fettig" nil (f eh1 t ih0 g))
("fetting" nil (f eh1 t ih0 ng))
("fetty" nil (f eh1 t iy0))
("fetus" nil (f iy1 t ax s))
("fetuses" nil (f iy1 t ax s ih0 z))
("fetz" nil (f eh1 t s))
("fetzer" nil (f eh1 t s er0))
("feucht" nil (f oy1 k t))
("feud" nil (f y uw1 d))
("feudal" nil (f y uw1 d ax l))
("feudalism" nil (f y uw1 d ax l ih1 z ax m))
("feudalistic" nil (f y uw1 d ax l ih1 s t ih0 k))
("feuded" nil (f y uw1 d ih0 d))
("feuding" nil (f y uw1 d ih0 ng))
("feuds" nil (f y uw1 d z))
("feuer" nil (f y uw1 er0))
("feuerborn" nil (f oy1 er0 b er0 n))
("feuerman" nil (f y uw1 er0 m ax n))
("feuerstein" nil (f y uw1 er0 s t iy1 n))
("feulner" nil (f oy1 l n er0))
("feustel" nil (f oy1 s t ax l))
("feutz" nil (f y uw1 t s))
("fever" nil (f iy1 v er0))
("fevered" nil (f iy1 v er0 d))
("feverfew" nil (f iy1 v er0 f y uw1))
("feverfews" nil (f iy1 v er0 f y uw1 z))
("feverish" nil (f iy1 v er0 ih0 sh))
("feverishly" nil (f eh1 v er0 ih1 sh l iy0))
("fevers" nil (f iy1 v er0 z))
("few" nil (f y uw1))
("fewell" nil (f eh1 w eh0 l))
("fewer" nil (f y uw1 er0))
("fewest" nil (f y uw1 ax s t))
("fey" nil (f ey1))
("feyen" nil (f ay1 n))
("fez" nil (f eh1 z))
("fi" nil (f ay1))
("fiacco" nil (f iy0 aa1 k ow0))
("fiala" nil (f iy0 aa1 l ax))
("fialkowski" nil (f iy0 ax l k ao1 f s k iy0))
("fiance" nil (f iy0 aa1 n s ey1))
("fiancee" nil (f iy0 ae1 n s iy0))
("fianna" nil (f iy0 ae1 n ax))
("fiasco" nil (f iy0 ae1 s k ow0))
("fiascos" nil (f iy0 ae1 s k ow0 z))
("fiat" nil (f ay1 ae0 t))
("fiber" nil (f ay1 b er0))
("fiberboard" nil (f ay1 b er0 b ao1 r d))
("fiberglas" nil (f ay1 b er0 g l ae1 s))
("fiberglass" nil (f ay1 b er0 g l ae1 s))
("fiberoptic" nil (f ay1 b er0 aa1 p t ih0 k))
("fibers" nil (f ay1 b er0 z))
("fiberweb" nil (f ay1 b er0 w eh1 b))
("fibre" nil (f ay1 b er0))
("fibreboard" nil (f ay1 b er0 b ao1 r d))
("fibrillation" nil (f ih1 b r ih0 l ey1 sh ax n))
("fibrin" nil (f ay1 b r ax n))
("fibronics" nil (f ih0 b r aa1 n ih0 k s))
("fibrosis" nil (f ay0 b r ow1 s ax s))
("fibrous" nil (f ay1 b r ax s))
("fibula" nil (f ih1 b y ax l ax))
("ficarra" nil (f iy0 k aa1 r ax))
("ficco" nil (f ih1 k ow0))
("ficek" nil (f ih1 s ih0 k))
("fichera" nil (f iy0 k eh1 r ax))
("fichtel" nil (f ih1 ch t eh1 l))
("fichter" nil (f ih1 k t er0))
("fichtner" nil (f ih1 k t n er0))
("fichus" nil (f ih1 ch ax s))
("fick" nil (f ih1 k))
("ficke" nil (f ih1 k))
("fickel" nil (f ih1 k ax l))
("ficken" nil (f ih1 k ax n))
("ficker" nil (f ih1 k er0))
("fickes" nil (f ih1 k s))
("fickett" nil (f ih1 k ih0 t))
("fickle" nil (f ih1 k ax l))
("fickleness" nil (f ih1 k ax l n ax s))
("ficklin" nil (f ih1 k l ih0 n))
("fickling" nil (f ih1 k l ih0 ng))
("fico" nil (f iy1 k ow0))
("ficom" nil (f ih1 k ax m))
("ficorca" nil (f ih0 k ao1 r k ax))
("fiction" nil (f ih1 k sh ax n))
("fictional" nil (f ih1 k sh ax n ax l))
("fictionalized" nil (f ih1 k sh ax n ax l ay1 z d))
("fictions" nil (f ih1 k sh ax n z))
("fictitious" nil (f ih0 k t ih1 sh ax s))
("fidalgo" nil (f iy0 d aa1 l g ow0))
("fidata" nil (f ih0 d aa1 t ax))
("fiddle" nil (f ih1 d ax l))
("fiddled" nil (f ih1 d ax l d))
("fiddler" nil (f ih1 d ax l er0))
("fiddles" nil (f ih1 d ax l z))
("fiddling" nil (f ih1 d l ih0 ng))
("fide" nil (f ay1 d))
("fidel" nil (f ih0 d eh1 l))
("fidelco" nil (f ih0 d eh1 l k ow0))
("fidelcor" nil (f ih1 d ax l k ao1 r))
("fidelia" nil (f ih0 d iy1 l y ax))
("fidelity" nil (f ax d eh1 l ax t iy0))
("fidenas" nil (f ih1 d ih0 n ax s))
("fides" nil (f ay1 d z))
("fidget" nil (f ih1 jh ih0 t))
("fidgeting" nil (f ih1 jh ax t ih0 ng))
("fidler" nil (f ay1 d ax l er0))
("fido" nil (f ay1 d ow0))
("fiduciares" nil (f ih0 d uw1 s iy0 eh1 r z))
("fiduciaries" nil (f ih0 d uw1 sh iy0 eh1 r iy0 z))
("fiduciary" nil (f ax d uw1 sh iy0 eh1 r iy0))
("fiebelkorn" nil (f iy1 b ih0 l k er0 n))
("fieber" nil (f iy1 b er0))
("fiebig" nil (f iy1 b ih0 g))
("fiebiger" nil (f iy1 b ih0 g er0))
("fiechter" nil (f ih1 k t er0))
("fiedler" nil (f iy1 d l er0))
("fief" nil (f iy1 f))
("fiefdom" nil (f iy1 f d ax m))
("fiefdoms" nil (f iy1 f d ax m z))
("fiegel" nil (f iy1 g ax l))
("fieger" nil (f iy1 g er0))
("fiel" nil (f iy1 l))
("field" nil (f iy1 l d))
("fieldcrest" nil (f iy1 l d k r eh1 s t))
("fielded" nil (f iy1 l d ih0 d))
("fielden" nil (f iy1 l d ax n))
("fielder" nil (f iy1 l d er0))
("fielders" nil (f iy1 l d er0 z))
("fieldhouse" nil (f iy1 l d hh aw1 s))
("fielding" nil (f iy1 l d ih0 ng))
("fields" nil (f iy1 l d z))
("fieldwork" nil (f iy1 l d w er1 k))
("fiend" nil (f iy1 n d))
("fiendish" nil (f iy1 n d ih0 sh))
("fiends" nil (f iy1 n d z))
("fiene" nil (f iy1 n))
("fier" nil (f ih1 r))
("fierce" nil (f ih1 r s))
("fiercely" nil (f ih1 r s l iy0))
("fiercer" nil (f ih1 r s er0))
("fiercest" nil (f ih1 r s ax s t))
("fierman" nil (f ih1 r m ax n))
("fiero" nil (f iy1 r ow0))
("fieros" nil (f iy0 eh1 r ow0 s))
("fierro" nil (f ih1 r ow0))
("fierros" nil (f ih1 r ow0 z))
("fiers" nil (f iy1 r z))
("fierst" nil (f iy1 r s t))
("fierstein" nil (f iy1 r s t iy1 n))
("fiery" nil (f ay1 er0 iy0))
("fieser" nil (f iy1 s er0))
("fiest" nil (f iy1 ih0 s t))
("fiesta" nil (f iy0 eh1 s t ax))
("fife" nil (f ay1 f))
("fifer" nil (f ay1 f er0))
("fifi" nil (f iy1 f iy0))
("fifield" nil (f ih1 f iy0 l d))
("fifteen" nil (f ih0 f t iy1 n))
("fifteens" nil (f ih0 f t iy1 n z))
("fifteenth" nil (f ih0 f t iy1 n th))
("fifth" nil (f ih1 f th))
("fifths" nil (f ih1 f th s))
("fifties" nil (f ih1 f t iy0 z))
("fiftieth" nil (f ih1 f t iy0 ih0 th))
("fifty" nil (f ih1 f t iy0))
("fig" nil (f ih1 g))
("figallo" nil (f ih0 g aa1 l ow0))
("figaro" nil (f ih1 g er0 ow1))
("figaroa" nil (f ih1 g er0 ow1 ax))
("figel" nil (f ih1 g ax l))
("figg" nil (f ih1 g))
("figge" nil (f ih1 g))
("figgie" nil (f ih1 g iy0))
("figgins" nil (f ih1 g ih0 n z))
("figgs" nil (f ih1 g z))
("fight" nil (f ay1 t))
("fighter" nil (f ay1 t er0))
("fighters" nil (f ay1 t er0 z))
("fighting" nil (f ay1 t ih0 ng))
("fightmaster" nil (f ay1 t m ae1 s t er0))
("fights" nil (f ay1 t s))
("figler" nil (f ay1 g ax l er0))
("figley" nil (f ih1 g l iy0))
("figone" nil (f ih1 g ax n))
("figs" nil (f ih1 g z))
("figueira" nil (f iy0 g eh1 r ax))
("figueiredo" nil (f iy0 g er0 ey1 d ow0))
("figueras" nil (f iy0 g eh1 r aa0 z))
("figueredo" nil (f iy0 g eh0 r ey1 d ow0))
("figueroa" nil (f iy0 g eh0 r ow1 ax))
("figura" nil (f iy0 g uh1 r ax))
("figuration" nil (f ih1 g y er0 ey1 sh ax n))
("figurative" nil (f ih1 g y er0 ax t ih0 v))
("figuratively" nil (f ih0 g y uh1 r ax t ih0 v l iy0))
("figure" nil (f ih1 g y er0))
("figured" nil (f ih1 g y er0 d))
("figurehead" nil (f ih1 g y er0 hh eh1 d))
("figures" nil (f ih1 g y er0 z))
("figurines" nil (f ih1 g y er0 iy1 n z))
("figuring" nil (f ih1 g y er0 ih0 ng))
("figurski" nil (f ih0 g er1 s k iy0))
("fiji" nil (f iy1 jh iy0))
("fijian" nil (f iy1 jh iy0 ax n))
("fijians" nil (f ih1 jh iy0 ax n z))
("fike" nil (f ay1 k))
("fikes" nil (f ay1 k s))
("fil" nil (f ih1 l))
("fila" nil (f ay1 l ax))
("filament" nil (f ih1 l ax m ax n t))
("filaments" nil (f ih1 l ax m ax n t s))
("filbert" nil (f ih1 l b er0 t))
("filberts" nil (f ih1 l b er0 t s))
("filby" nil (f ih1 l b iy0))
("filched" nil (f ih1 l ch t))
("fildes" nil (f ih1 l d z))
("file" nil (f ay1 l))
("filed" nil (f ay1 l d))
("filenet" nil (f ay1 l n eh1 t))
("filer" nil (f ay1 l er0))
("filers" nil (f ay1 l er0 z))
("files" nil (f ay1 l z))
("filet" nil (f ih0 l ey1))
("filets" nil (f ih0 l ey1 z))
("filial" nil (f ih1 l iy0 ax l))
("filibuster" nil (f ih1 l ax b ah1 s t er0))
("filibustered" nil (f ih1 l ax b ah1 s t er0 d))
("filibustering" nil (f ih1 l ax b ah1 s t er0 ih0 ng))
("filibusters" nil (f ih1 l ax b ah1 s t er0 z))
("filice" nil (f iy1 l ih0 s))
("filigree" nil (f ih1 l ax g r iy1))
("filigrees" nil (f ih1 l ax g r iy1 z))
("filing" nil (f ay1 l ih0 ng))
("filings" nil (f ay1 l ih0 ng z))
("filinski" nil (f ih0 l ih1 n s k iy0))
("filion" nil (f ih1 l y ax n))
("filip" nil (f ih1 l ih0 p))
("filipa" nil (f iy0 l iy1 p ax))
("filipek" nil (f ih1 l ih0 p ih0 k))
("filipiak" nil (f ih0 l ih1 p iy0 ae0 k))
("filipino" nil (f ih1 l ax p iy1 n ow0))
("filipinos" nil (f ih1 l ax p iy1 n ow0 z))
("filipkowski" nil (f ih0 l ih0 p k ao1 f s k iy0))
("filipowicz" nil (f ih0 l ih1 p ax v ih0 ch))
("filippelli" nil (f iy0 l iy0 p eh1 l iy0))
("filippello" nil (f iy1 l ih0 p eh1 l ow0))
("filippi" nil (f ih0 l ih1 p iy0))
("filippini" nil (f iy0 l iy0 p iy1 n iy0))
("filippo" nil (f ih0 l iy1 p ow0))
("filippone" nil (f ih1 l ih0 p ow1 n))
("filipski" nil (f ih0 l ih1 p s k iy0))
("filkins" nil (f ih1 l k ih0 n z))
("fill" nil (f ih1 l))
("filla" nil (f ih1 l ax))
("fillauer" nil (f ih1 l aw1 er0))
("filled" nil (f ih1 l d))
("filler" nil (f ih1 l er0))
("fillers" nil (f ih1 l er0 z))
("fillet" nil (f ax l ey1))
("filleted" nil (f ih1 l ax t ih0 d))
("fillets" nil (f ih1 l ih0 t s))
("filley" nil (f ih1 l iy0))
("fillies" nil (f ih1 l iy0 z))
("filling" nil (f ih1 l ih0 ng))
("fillingame" nil (f iy0 l ih0 ng g aa1 m iy0))
("fillinger" nil (f ih1 l ih0 ng er0))
("fillingham" nil (f ih1 l ih0 ng hh ae1 m))
("fillingim" nil (f ih1 l ih0 ng g ih0 m))
("fillings" nil (f ih1 l ih0 ng z))
("fillion" nil (f ih1 l y ax n))
("fillip" nil (f ih1 l ax p))
("fillman" nil (f ih1 l m ax n))
("fillmore" nil (f ih1 l m ao0 r))
("fills" nil (f ih1 l z))
("filly" nil (f ih1 l iy0))
("film" nil (f ih1 l m))
("filma" nil (f ih1 l m ax))
("filmed" nil (f ih1 l m d))
("filmer" nil (f ih1 l m er0))
("filming" nil (f ih1 l m ih0 ng))
("filmless" nil (f ih1 l m l ax s))
("filmmaker" nil (f ih1 l m ey1 k er0))
("filmmakers" nil (f ih1 l m ey1 k er0 z))
("filmmaking" nil (f ih1 l m ey1 k ih0 ng))
("filmore" nil (f ih1 l m ao1 r))
("films" nil (f ih1 l m z))
("filo" nil (f iy1 l ow0))
("filofax" nil (f ih1 l ow0 f ae1 k s))
("filosa" nil (f iy0 l ow1 s ax))
("filsinger" nil (f ih1 l s ih0 ng er0))
("filson" nil (f ih1 l s ax n))
("filter" nil (f ih1 l t er0))
("filtered" nil (f ih1 l t er0 d))
("filtering" nil (f ih1 l t er0 ih0 ng))
("filters" nil (f ih1 l t er0 z))
("filtertek" nil (f ih1 l t er0 t eh1 k))
("filth" nil (f ih1 l th))
("filthy" nil (f ih1 l th iy0))
("filtration" nil (f ih0 l t r ey1 sh ax n))
("filtrol" nil (f ih1 l t r aa0 l))
("filyaw" nil (f ih0 l iy1 aa0 v))
("fimbres" nil (f ih1 m b er0 z))
("fin" nil (f ih1 n))
("fina" nil (f ay1 n ax))
("finagle" nil (f ih1 n ax g ax l))
("final" nil (f ay1 n ax l))
("finalco" nil (f ih0 n ae1 l k ow0))
("finale" nil (f ax n ae1 l iy0))
("finalist" nil (f ay1 n ax l ih0 s t))
("finalists" nil (f ay1 n ax l ax s t s))
("finality" nil (f ay0 n ae1 l ax t iy0))
("finalize" nil (f ay1 n ax l ay1 z))
("finalized" nil (f ay1 n ax l ay1 z d))
("finalizing" nil (f ay1 n ax l ay1 z ih0 ng))
("finally" nil (f ay1 n ax l iy0))
("finals" nil (f ay1 n ax l z))
("finamore" nil (f ih1 n ax m ao1 r))
("finan" nil (f ih1 n ax n))
("finance" nil (f ax n ae1 n s))
("financed" nil (f ih0 n ae1 n s t))
("financement" nil (f ih0 n ae1 n s m ax n t))
("finances" nil (f ih0 n ae1 n s ih0 z))
("financial" nil (f ax n ae1 n sh ax l))
("financially" nil (f ax n ae1 n sh ax l iy0))
("financials" nil (f ay1 n ae1 n sh ax l z))
("financier" nil (f ih1 n ax n s ih1 r))
("financiera" nil (f ih0 n ae1 n s iy0 eh1 r ax))
("financiere" nil (f ih1 n ae0 n s ih1 r))
("financiero" nil (f ih0 n ae1 n s iy0 eh1 r ow0))
("financiers" nil (f ih1 n ax n s ih1 r z))
("financing" nil (f ax n ae1 n s ih0 ng))
("financings" nil (f ax n ae1 n s ih0 ng z))
("financo" nil (f ih0 n ae1 ng k ow0))
("finanz" nil (f ih1 n ae0 n z))
("finanza" nil (f ih0 n ae1 n z ax))
("finanziaria" nil (f ih0 n ae1 n z iy0 eh1 r iy0 ax))
("finanziario" nil (f ih0 n ae1 n z iy0 eh1 r iy0 ow0))
("finarte" nil (f ih1 n aa0 r t))
("finazzo" nil (f iy0 n aa1 z ow0))
("finberg" nil (f ih1 n b er0 g))
("fincannon" nil (f ih1 n k ax n aa0 n))
("finch" nil (f ih1 n ch))
("fincham" nil (f ih1 n ch ax m))
("fincher" nil (f ih1 n ch er0))
("finches" nil (f ih1 n ch ih0 z))
("finchum" nil (f ih1 n k ax m))
("finck" nil (f ih1 ng k))
("fincke" nil (f ih1 ng k))
("finckle" nil (f ih1 ng k ax l))
("find" nil (f ay1 n d))
("finder" nil (f ay1 n d er0))
("finders" nil (f ay1 n d er0 z))
("finding" nil (f ay1 n d ih0 ng))
("findings" nil (f ay1 n d ih0 ng z))
("findlay" nil (f ih1 n d l iy0))
("findley" nil (f ih1 n d l iy0))
("findling" nil (f ih1 n d ax l ih0 ng))
("finds" nil (f ay1 n d z))
("fine" nil (f ay1 n))
("fineberg" nil (f ay1 n b er0 g))
("fined" nil (f ay1 n d))
("finefrock" nil (f ay1 n f r aa1 k))
("finegan" nil (f ih1 n ih0 g ae0 n))
("finegold" nil (f ay1 n g ow1 l d))
("finella" nil (f ih0 n eh1 l ax))
("finelli" nil (f ih0 n eh1 l iy0))
("finely" nil (f ay1 n l iy0))
("fineman" nil (f ay1 n m ax n))
("finer" nil (f ay1 n er0))
("fineran" nil (f ih1 n er0 ae0 n))
("finerty" nil (f ih1 n er0 t iy0))
("finery" nil (f ay1 n er0 iy0))
("fines" nil (f ay1 n z))
("finesse" nil (f ih0 n eh1 s))
("finessed" nil (f ax n eh1 s t))
("finest" nil (f ay1 n ax s t))
("finestone" nil (f ay1 n s t ow1 n))
("finevest" nil (f ay1 n v eh1 s t))
("finex" nil (f ih1 n eh1 k s))
("finfrock" nil (f ih1 n f r aa1 k))
("finger" nil (f ih1 ng g er0))
("fingerboard" nil (f ih1 ng g er0 b ao1 r d))
("fingered" nil (f ih1 ng g er0 d))
("fingerhut" nil (f ih1 ng g er0 hh ah1 t))
("fingering" nil (f ih1 ng g er0 ih0 ng))
("fingernail" nil (f ih1 ng g er0 n ey1 l))
("fingernails" nil (f ih1 ng g er0 n ey1 l z))
("fingerprint" nil (f ih1 ng g er0 p r ih1 n t))
("fingerprinted" nil (f ih1 ng g er0 p r ih1 n t ih0 d))
("fingerprinting" nil (f ih1 ng g er0 p r ih1 n t ih0 ng))
("fingerprints" nil (f ih1 ng g er0 p r ih1 n t s))
("fingers" nil (f ih1 ng g er0 z))
("fingertips" nil (f ih1 ng g er0 t ih1 p s))
("fini" nil (f iy1 n iy0))
("finicky" nil (f ih1 n ax k iy0))
("finigan" nil (f ih1 n ih0 g ax n))
("fining" nil (f ay1 n ih0 ng))
("fininvest" nil (f ih1 n ih1 n v eh1 s t))
("finish" nil (f ih1 n ih0 sh))
("finished" nil (f ih1 n ih0 sh t))
("finisher" nil (f ih1 n ih0 sh er0))
("finishers" nil (f ih1 n ih0 sh er0 z))
("finishes" nil (f ih1 n ih0 sh ih0 z))
("finishing" nil (f ih1 n ih0 sh ih0 ng))
("finite" nil (f ay1 n ay1 t))
("fink" nil (f ih1 ng k))
("finkbeiner" nil (f ih1 ng k b ay1 n er0))
("finkbiner" nil (f ih1 ng k b ay1 n er0))
("finke" nil (f ih1 ng k))
("finkel" nil (f ih1 ng k ax l))
("finkelman" nil (f ih1 ng k ax l m ax n))
("finkelson" nil (f ih1 ng k ax l s ax n))
("finkelstein" nil (f ih1 ng k ax l s t iy1 n))
("finken" nil (f ih1 ng k ax n))
("finkenbinder" nil (f ih1 ng k ih0 n b ih0 n d er0))
("finkielstain" nil (f ih0 ng k iy1 l s t ih0 n))
("finkle" nil (f ih1 ng k ax l))
("finklea" nil (f ih1 ng k l iy0 ax))
("finkler" nil (f ih1 ng k l er0))
("finkley" nil (f ih1 ng k l iy0))
("finks" nil (f ih1 ng k s))
("finland" nil (f ih1 n l ae1 n d))
("finlandization" nil (f ih1 n l ae0 n d ih0 z ey1 sh ax n))
("finlay" nil (f ih1 n l iy0))
("finlayson" nil (f ih1 n l iy0 s ax n))
("finlets" nil (f ih1 n l ax t s))
("finley" nil (f ih1 n l iy0))
("finmeccanica" nil (f ih1 n m ax k ae1 n ih0 k ax))
("finn" nil (f ih1 n))
("finnair" nil (f ih1 n eh1 r))
("finnan" nil (f ih1 n ax n))
("finne" nil (f ih1 n))
("finnegan" nil (f ih1 n ax g ax n))
("finnell" nil (f ih1 n ax l))
("finneran" nil (f ih1 n er0 ae0 n))
("finnerty" nil (f ih1 n er0 t iy0))
("finney" nil (f ih1 n iy0))
("finnicum" nil (f ih1 n ih0 k ax m))
("finnie" nil (f ih1 n iy0))
("finnigan" nil (f ih1 n ih0 g ax n))
("finnish" nil (f ih1 n ih0 sh))
("finns" nil (f ih1 n z))
("fino" nil (f iy1 n ow0))
("finocchiaro" nil (f iy0 n ow0 k iy0 aa1 r ow0))
("finocchio" nil (f iy0 n ow1 k iy0 ow0))
("fins" nil (f ih1 n z))
("finscher" nil (f ih1 n sh er0))
("finsider" nil (f ih1 n s ay0 d er0))
("finstad" nil (f ih1 n s t ax d))
("finster" nil (f ih1 n s t er0))
("finstrom" nil (f ih1 n s t r ax m))
("finton" nil (f ih1 n t ax n))
("finucane" nil (f ih1 n ax k ey0 n))
("finzel" nil (f ih1 n z ax l))
("fiola" nil (f iy0 ow1 l ax))
("fiona" nil (f iy0 ow1 n ax))
("fioravanti" nil (f iy0 ao0 r aa0 v aa1 n t iy0))
("fiore" nil (f iy0 ao1 r iy0))
("fiorella" nil (f iy0 ao0 r eh1 l ax))
("fiorelli" nil (f iy0 ao0 r eh1 l iy0))
("fiorello" nil (f iy0 ao0 r eh1 l ow0))
("fiorentino" nil (f iy0 ao0 r eh0 n t iy1 n ow0))
("fiorenza" nil (f iy0 ao0 r eh1 n z ax))
("fioretti" nil (f iy0 ao0 r eh1 t iy0))
("fiori" nil (f iy0 ao1 r iy0))
("fiorillo" nil (f iy0 ao0 r ih1 l ow0))
("fiorina" nil (f iy0 ao0 r iy1 n ax))
("fiorini" nil (f iy0 ao0 r iy1 n iy0))
("fiorino" nil (f iy0 ao0 r iy1 n ow0))
("fiorito" nil (f iy0 ao0 r iy1 t ow0))
("fiorucci" nil (f iy0 ao0 r uw1 ch iy0))
("fipple" nil (f ih1 p ax l))
("fipps" nil (f ih1 p s))
("fir" nil (f er1))
("fire" nil (f ay1 er0))
("firearm" nil (f ay1 er0 aa1 r m))
("firearms" nil (f ay1 er0 aa1 r m z))
("fireball" nil (f ay1 er0 b ao1 l))
("firebaugh" nil (f ay1 r b ao1))
("firebird" nil (f ay1 er0 b er1 d))
("firebirds" nil (f ay1 er0 b er1 d z))
("firebomb" nil (f ay1 er0 b aa1 m))
("firebombed" nil (f ay1 r b aa1 m d))
("firebombing" nil (f ay1 er0 b aa1 m ih0 ng))
("firebombs" nil (f ay1 r b aa1 m z))
("firebrand" nil (f ay1 er0 b r ae1 n d))
("firebrands" nil (f ay1 er0 b r ae1 n d z))
("firebrick" nil (f ay1 er0 b r ih1 k))
("firebug" nil (f ay1 er0 b ah1 g))
("firebush" nil (f ay1 er0 b uh1 sh))
("firecracker" nil (f ay1 er0 k r ae1 k er0))
("firecrackers" nil (f ay1 er0 k r ae1 k er0 z))
("fired" nil (f ay1 er0 d))
("firedamp" nil (f ay1 er0 d ae1 m p))
("firefight" nil (f ay1 r f ay1 t))
("firefighter" nil (f ay1 r f ay1 t er0))
("firefighters" nil (f ay1 er0 f ay1 t er0 z))
("firefighting" nil (f ay1 r f ay1 t ih0 ng))
("firefly" nil (f ay1 er0 f l ay1))
("firehouse" nil (f ay1 er0 hh aw1 s))
("firehouses" nil (f ay1 er0 hh aw1 s ih0 z))
("fireman" nil (f ay1 r m ax n))
("firemen" nil (f ay1 r m ax n))
("firenza" nil (f ih0 r eh1 n z ax))
("firenzas" nil (f er0 eh1 n z ax z))
("fireplace" nil (f ay1 er0 p l ey1 s))
("fireplaces" nil (f ay1 er0 p l ey1 s ax z))
("firepower" nil (f ay1 r p aw1 er0))
("fireproof" nil (f ay1 er0 p r uw1 f))
("fireproofing" nil (f ay1 er0 p r uw1 f ih0 ng))
("fires" nil (f ay1 er0 z))
("fireside" nil (f ay1 er0 s ay1 d))
("firesign" nil (f ay1 er0 s ay1 n))
("firestine" nil (f ay0 r s t iy1 n iy0))
("firestone" nil (f ay1 r s t ow1 n))
("firestorm" nil (f ay1 r s t ao1 r m))
("fireweed" nil (f ay1 er0 w iy1 d))
("firewood" nil (f ay1 er0 w uh1 d))
("fireworks" nil (f ay1 r w er1 k s))
("firing" nil (f ay1 r ih0 ng))
("firings" nil (f ay1 r ih0 ng z))
("firino" nil (f ih0 r iy1 n ow0))
("firkins" nil (f er1 k ih0 n z))
("firkus" nil (f er1 k ih0 s))
("firm" nil (f er1 m))
("firma" nil (f er1 m ax))
("firman" nil (f er1 m ax n))
("firmans" nil (f er1 m ax n z))
("firmed" nil (f er1 m d))
("firmer" nil (f er1 m er0))
("firmest" nil (f er1 m ax s t))
("firmin" nil (f er1 m ih0 n))
("firming" nil (f er1 m ih0 ng))
("firmly" nil (f er1 m l iy0))
("firmness" nil (f er1 m n ax s))
("firms" nil (f er1 m z))
("firpo" nil (f ih1 r p ow0))
("first" nil (f er1 s t))
("firstar" nil (f er1 s t aa1 r))
("firstborn" nil (f er1 s t b ao1 r n))
("firstfed" nil (f er1 s t f eh1 d))
("firsthand" nil (f er1 s t hh ae1 n d))
("firstier" nil (f er1 s t y er0))
("firstly" nil (f er1 s t l iy0))
("firsts" nil (f er1 s t s))
("firstsouth" nil (f er1 s t s aw1 th))
("firth" nil (f er1 th))
("firzite" nil (f er1 z ay1 t))
("fis" nil (f ih1 s))
("fisc" nil (f ih1 s k))
("fiscal" nil (f ih1 s k ax l))
("fiscally" nil (f ih1 s k ax l iy0))
("fisch" nil (f ih1 sh))
("fischbach" nil (f ih1 sh b aa1 k))
("fischbein" nil (f ih1 sh b ay1 n))
("fischel" nil (f ih1 sh ax l))
("fischer" nil (f ih1 sh er0))
("fischetti" nil (f iy0 s k eh1 t iy0))
("fischl" nil (f ih1 s k ax l))
("fischler" nil (f ih1 sh ax l er0))
("fischman" nil (f ih1 sh m ax n))
("fiscus" nil (f ih1 s k ax s))
("fiser" nil (f ay1 z er0))
("fiserv" nil (f ih1 s er0 v))
("fisette" nil (f ih0 s eh1 t))
("fish" nil (f ih1 sh))
("fishback" nil (f ih1 sh b ae1 k))
("fishbaine" nil (f ih1 sh b ey1 n))
("fishbaugh" nil (f ih1 sh b ao1))
("fishbeck" nil (f ih1 sh b eh1 k))
("fishbein" nil (f ih1 sh b ay1 n))
("fishbowl" nil (f ih1 sh b ow1 l))
("fishburn" nil (f ih1 sh b er1 n))
("fishburne" nil (f ih1 sh b er0 n))
("fisheaters" nil (f ih1 sh iy1 t er0 z))
("fished" nil (f ih1 sh t))
("fishel" nil (f ih1 sh ax l))
("fishell" nil (f ih1 sh ax l))
("fisher" nil (f ih1 sh er0))
("fisheries" nil (f ih1 sh er0 iy0 z))
("fisherman" nil (f ih1 sh er0 m ae1 n))
("fishermen" nil (f ih1 sh er0 m ih0 n))
("fishers" nil (f ih1 sh er0 z))
("fishery" nil (f ih1 sh er0 iy0))
("fishes" nil (f ih1 sh ax z))
("fishing" nil (f ih1 sh ih0 ng))
("fishkill" nil (f ih1 sh k ih1 l))
("fishkin" nil (f ih1 sh k ih0 n))
("fishlow" nil (f ih1 sh l ow1))
("fishman" nil (f ih1 sh m ae1 n))
("fishmonger" nil (f ih1 sh m aa1 ng g er0))
("fishy" nil (f ih1 sh iy0))
("fisk" nil (f ih1 s k))
("fiske" nil (f ih1 s k))
("fisler" nil (f ih1 s ax l er0))
("fisons" nil (f ay1 z ax n z))
("fiss" nil (f ih1 s))
("fissel" nil (f ih1 s ax l))
("fissell" nil (f ih1 s ax l))
("fission" nil (f ih1 sh ax n))
("fissionable" nil (f ih1 sh ax n ax b ax l))
("fissure" nil (f ih1 sh er0))
("fissured" nil (f ih1 sh er0 d))
("fissures" nil (f ih1 sh er0 z))
("fist" nil (f ih1 s t))
("fisted" nil (f ih1 s t ih0 d))
("fister" nil (f ih1 s t er0))
("fistful" nil (f ih1 s t f ax l))
("fisticuffs" nil (f ih1 s t ih0 k ah1 f s))
("fists" nil (f ih1 s t s))
("fit" nil (f ih1 t))
("fitak" nil (f ih1 t ae1 k))
("fitch" nil (f ih1 ch))
("fitchburg" nil (f ih1 ch b er0 g))
("fitchett" nil (f ih1 ch ih0 t))
("fite" nil (f ay1 t))
("fites" nil (f ay1 t s))
("fitful" nil (f ih1 t f ax l))
("fitfully" nil (f ih1 t f ax l iy0))
("fithian" nil (f ih1 th iy0 ax n))
("fitness" nil (f ih1 t n ax s))
("fits" nil (f ih1 t s))
("fitt" nil (f ih1 t))
("fitted" nil (f ih1 t ax d))
("fitter" nil (f ih1 t er0))
("fitterer" nil (f ih1 t er0 er0))
("fittest" nil (f ih1 t ax s t))
("fitting" nil (f ih1 t ih0 ng))
("fittingly" nil (f ih1 t ih0 ng l iy0))
("fittings" nil (f ih1 t ih0 ng z))
("fitton" nil (f ih1 t ax n))
("fittro" nil (f ih1 t r ow0))
("fitts" nil (f ih1 t s))
("fitz" nil (f ih1 t s))
("fitzer" nil (f ih1 t s er0))
("fitzgerald" nil (f ih0 t s jh eh1 r ax l d))
("fitzgibbon" nil (f ih1 t s jh ih1 b ax n))
("fitzgibbons" nil (f ih1 t s jh ih1 b ax n z))
("fitzharris" nil (f ih0 t s hh ae1 r ih0 s))
("fitzhenry" nil (f ih0 t s hh eh1 n er0 iy0))
("fitzhugh" nil (f ih0 t s hh y uw1))
("fitzmaurice" nil (f ih0 t s m ao1 r ih0 s))
("fitzmorris" nil (f ih0 t s m ao1 r ih0 s))
("fitzner" nil (f ih1 t s n er0))
("fitzpatrick" nil (f ih1 t s p ae1 t r ih0 k))
("fitzsimmons" nil (f ih0 t s ih1 m ax n z))
("fitzsimons" nil (f ih0 t s ih1 m ax n z))
("fitzwater" nil (f ih1 t s w ao1 t er0))
("fitzwilliam" nil (f ih0 t s w ih1 l y ax m))
("fiumara" nil (f iy1 uw0 m aa1 r ax))
("five" nil (f ay1 v))
("fiveash" nil (f ay1 v ae1 sh))
("fivecoat" nil (f ay1 v k ow1 t))
("fivefold" nil (f ay1 v f ow1 l d))
("fives" nil (f ay1 v z))
("fix" nil (f ih1 k s))
("fixable" nil (f ih1 k s ax b ax l))
("fixate" nil (f ih1 k s ey1 t))
("fixated" nil (f ih1 k s ey1 t ih0 d))
("fixation" nil (f ih0 k s ey1 sh ax n))
("fixatives" nil (f ih1 k s ax t ih0 v z))
("fixed" nil (f ih1 k s t))
("fixer" nil (f ih1 k s er0))
("fixes" nil (f ih1 k s ih0 z))
("fixing" nil (f ih1 k s ih0 ng))
("fixings" nil (f ih1 k s ih0 ng z))
("fixler" nil (f ih1 k s l er0))
("fixture" nil (f ih1 k s ch er0))
("fixtures" nil (f ih1 k s ch er0 z))
("fizer" nil (f ay1 z er0))
("fizz" nil (f ih1 z))
("fizzle" nil (f ih1 z ax l))
("fizzled" nil (f ih1 z ax l d))
("fizzles" nil (f ih1 z ax l z))
("fizzling" nil (f ih1 z ax l ih0 ng))
("fjeld" nil (f y eh1 l d))
("fjelstad" nil (f y eh1 l s t ax d))
("fjords" nil (f y ao1 r d z))
("flab" nil (f l ae1 b))
("flabbergasted" nil (f l ae1 b er0 g ae1 s t ih0 d))
("flabby" nil (f l ae1 b iy0))
("flaccid" nil (f l ae1 k s ih0 d))
("flach" nil (f l ae1 ch))
("flack" nil (f l ae1 k))
("flad" nil (f l ae1 d))
("flag" nil (f l ae1 g))
("flagella" nil (f l ax jh eh1 l ax))
("flagellated" nil (f l ae1 jh ax l ey1 t ax d))
("flagellum" nil (f l ax jh eh1 l ax m))
("flageolet" nil (f l ae1 jh ax l eh1 t))
("flagg" nil (f l ae1 g))
("flagged" nil (f l ae1 g d))
("flagging" nil (f l ae1 g ih0 ng))
("flagler" nil (f l ae1 g l er0))
("flagpole" nil (f l ae1 g p ow1 l))
("flagpoles" nil (f l ae1 g p ow1 l z))
("flagrant" nil (f l ey1 g r ax n t))
("flagrantly" nil (f l ey1 g r ax n t l iy0))
("flags" nil (f l ae1 g z))
("flagship" nil (f l ae1 g sh ih1 p))
("flagships" nil (f l ae1 g sh ih1 p s))
("flagstaff" nil (f l ae1 g s t ae1 f))
("flagstar" nil (f l ae1 g s t aa1 r))
("flagstone" nil (f l ae1 g s t ow1 n))
("flaharty" nil (f l eh1 r t iy0))
("flaherty" nil (f l eh1 r t iy0))
("flahive" nil (f l ae1 hh ih0 v))
("flaig" nil (f l ey1 g))
("flail" nil (f l ey1 l))
("flailing" nil (f l ey1 l ih0 ng))
("flaim" nil (f l ey1 m))
("flair" nil (f l eh1 r))
("flak" nil (f l ae1 k))
("flake" nil (f l ey1 k))
("flakes" nil (f l ey1 k s))
("flaking" nil (f l ey1 k ih0 ng))
("flaky" nil (f l ey1 k iy0))
("flam" nil (f l ae1 m))
("flamboyance" nil (f l ae0 m b oy1 ax n s))
("flamboyant" nil (f l ae0 m b oy1 ax n t))
("flamboyantly" nil (f l ae0 m b oy1 ax n t l iy0))
("flame" nil (f l ey1 m))
("flamed" nil (f l ey1 m d))
("flamemaster" nil (f l ey1 m ae1 s t er0))
("flamenco" nil (f l ax m eh1 ng k ow1))
("flamer" nil (f l ey1 m er0))
("flames" nil (f l ey1 m z))
("flaming" nil (f l ey1 m ih0 ng))
("flamingo" nil (f l ax m ih1 ng g ow0))
("flamingos" nil (f l ax m ih1 ng g ow0 z))
("flaminian" nil (f l ax m ih1 n iy0 ax n))
("flamm" nil (f l ae1 m))
("flammability" nil (f l ae1 m ax b ih1 l ih0 t iy0))
("flammable" nil (f l ae1 m ax b ax l))
("flammang" nil (f l ae1 m ax ng))
("flammer" nil (f l ae1 m er0))
("flammia" nil (f l ae1 m iy0 ax))
("flamson" nil (f l ae1 m s ax n))
("flan" nil (f l ae1 n))
("flanagan" nil (f l ae1 n ax g ax n))
("flanagin" nil (f l ae1 n ax g ih0 n))
("flanary" nil (f l ae1 n er0 iy0))
("flanders" nil (f l ae1 n d er0 z))
("flanery" nil (f l ey1 n er0 iy0))
("flange" nil (f l ae1 n jh))
("flanges" nil (f l ae1 n jh ax z))
("flanigan" nil (f l ae1 n ih0 g ax n))
("flank" nil (f l ae1 ng k))
("flanked" nil (f l ae1 ng k t))
("flanking" nil (f l ae1 ng k ih0 ng))
("flanks" nil (f l ae1 ng k s))
("flann" nil (f l ae1 n))
("flanna" nil (f l ae1 n ax))
("flannagan" nil (f l ae1 n ax g ax n))
("flannel" nil (f l ae1 n ax l))
("flannels" nil (f l ae1 n ax l z))
("flannery" nil (f l ae1 n er0 iy0))
("flannigan" nil (f l ae1 n ih0 g ax n))
("flansburg" nil (f l ae1 n s b er0 g))
("flap" nil (f l ae1 p))
("flaplike" nil (f l ae1 p l ay1 k))
("flappers" nil (f l ae1 p er0 z))
("flapping" nil (f l ae1 p ih0 ng))
("flaps" nil (f l ae1 p s))
("flare" nil (f l eh1 r))
("flared" nil (f l eh1 r d))
("flares" nil (f l eh1 r z))
("flaring" nil (f l eh1 r ih0 ng))
("flasch" nil (f l ae1 sh))
("flash" nil (f l ae1 sh))
("flashback" nil (f l ae1 sh b ae1 k))
("flashbacks" nil (f l ae1 sh b ae1 k s))
("flashbulb" nil (f l ae1 sh b ax l b))
("flashdance" nil (f l ae1 sh d ae1 n s))
("flashed" nil (f l ae1 sh t))
("flasher" nil (f l ae1 sh er0))
("flashers" nil (f l ae1 sh er0 z))
("flashes" nil (f l ae1 sh ih0 z))
("flashier" nil (f l ae1 sh iy0 er0))
("flashing" nil (f l ae1 sh ih0 ng))
("flashlight" nil (f l ae1 sh l ay1 t))
("flashlights" nil (f l ae1 sh l ay1 t s))
("flashpoint" nil (f l ae1 sh p oy1 n t))
("flashy" nil (f l ae1 sh iy0))
("flask" nil (f l ae1 s k))
("flasks" nil (f l ae1 s k s))
("flat" nil (f l ae1 t))
("flatau" nil (f l ae1 t aw0))
("flatbed" nil (f l ae1 t b eh1 d))
("flatboat" nil (f l ae1 t b ow1 t))
("flaten" nil (f l ae1 t ax n))
("flater" nil (f l ey1 t er0))
("flatfish" nil (f l ae1 t f ih1 sh))
("flatfishes" nil (f l ae1 t f ih1 sh ih0 z))
("flath" nil (f l ae1 th))
("flathead" nil (f l ae1 t hh eh1 d))
("flatheads" nil (f l ae1 t hh eh1 d z))
("flathers" nil (f l ae1 dh er0 z))
("flatland" nil (f l ae1 t l ae1 n d))
("flatlands" nil (f l ae1 t l ae0 n d z))
("flatley" nil (f l ae1 t l iy0))
("flatly" nil (f l ae1 t l iy0))
("flatness" nil (f l ae1 t n ax s))
("flats" nil (f l ae1 t s))
("flatt" nil (f l ae1 t))
("flatten" nil (f l ae1 t ax n))
("flattened" nil (f l ae1 t ax n d))
("flattening" nil (f l ae1 t ax n ih0 ng))
("flatter" nil (f l ae1 t er0))
("flattered" nil (f l ae1 t er0 d))
("flattering" nil (f l ae1 t er0 ih0 ng))
("flattery" nil (f l ae1 t er0 iy0))
("flattish" nil (f l ae1 t ih0 sh))
("flatulent" nil (f l ae1 ch ax l ax n t))
("flatware" nil (f l ae1 t w eh1 r))
("flatworm" nil (f l ae1 t w er0 m))
("flaubert" nil (f l aw1 b er0 t))
("flaugh" nil (f l ao1))
("flaugher" nil (f l ao1 er0))
("flaum" nil (f l ao1 m))
("flaunt" nil (f l ao1 n t))
("flaunted" nil (f l ao1 n t ih0 d))
("flaunting" nil (f l ao1 n t ih0 ng))
("flaunts" nil (f l ao1 n t s))
("flavell" nil (f l ae1 v ax l))
("flavia" nil (f l ae1 v iy0 ax))
("flavin" nil (f l ey1 v ih0 n))
("flavius" nil (f l ey1 v iy0 ih0 s))
("flavor" nil (f l ey1 v er0))
("flavored" nil (f l ey1 v er0 d))
("flavorful" nil (f l ey1 v er0 f ax l))
("flavoring" nil (f l ey1 v er0 ih0 ng))
("flavorings" nil (f l ey1 v er0 ih0 ng z))
("flavorists" nil (f l ey1 v er0 ih0 s t s))
("flavors" nil (f l ey1 v er0 z))
("flavr" nil (f l ey1 v er0))
("flaw" nil (f l ao1))
("flawed" nil (f l ao1 d))
("flawless" nil (f l ao1 l ax s))
("flawlessly" nil (f l ao1 l ax s l iy0))
("flawn" nil (f l ao1 n))
("flaws" nil (f l ao1 z))
("flax" nil (f l ae1 k s))
("flaxman" nil (f l ae1 k s m ax n))
("flaxseed" nil (f l ae1 k s iy1 d))
("flayed" nil (f l ey1 d))
("flea" nil (f l iy1))
("fleagle" nil (f l iy1 g ax l))
("fleas" nil (f l iy1 z))
("fleck" nil (f l eh1 k))
("fleckenstein" nil (f l eh1 k ax n s t ay1 n))
("flecks" nil (f l eh1 k s))
("fled" nil (f l eh1 d))
("fledermaus" nil (f l eh1 d er0 m aw0 s))
("fledged" nil (f l eh1 jh d))
("fledging" nil (f l eh1 jh ih0 ng))
("fledgling" nil (f l eh1 jh l ih0 ng))
("flee" nil (f l iy1))
("fleece" nil (f l iy1 s))
("fleeced" nil (f l iy1 s t))
("fleecy" nil (f l iy1 s iy0))
("fleeger" nil (f l iy1 g er0))
("fleeing" nil (f l iy1 ih0 ng))
("fleek" nil (f l iy1 k))
("fleeman" nil (f l iy1 m ax n))
("fleener" nil (f l iy1 n er0))
("fleenor" nil (f l iy1 n er0))
("fleer" nil (f l ih1 r))
("flees" nil (f l iy1 z))
("fleet" nil (f l iy1 t))
("fleeting" nil (f l iy1 t ih0 ng))
("fleets" nil (f l iy1 t s))
("fleetwood" nil (f l iy1 t w uh1 d))
("flegal" nil (f l iy1 g ax l))
("flegel" nil (f l eh1 g ax l))
("fleharty" nil (f l eh1 hh aa0 r t iy0))
("fleig" nil (f l iy1 g))
("fleisch" nil (f l ay1 sh))
("fleischauer" nil (f l ay1 sh aw0 er0))
("fleischer" nil (f l ay1 sh er0))
("fleischhacker" nil (f l ay1 sh hh ax k er0))
("fleischman" nil (f l ay1 sh m ax n))
("fleischmann" nil (f l ay1 sh m ax n))
("fleisher" nil (f l ay1 sh er0))
("fleishman" nil (f l ay1 sh m ax n))
("fleiss" nil (f l ay1 sh))
("fleissner" nil (f l ay1 s n er0))
("fleitas" nil (f l ay1 t ax s))
("fleming" nil (f l eh1 m ih0 ng))
("flemings" nil (f l eh1 m ih0 ng z))
("flemington" nil (f l eh1 m ih0 ng t ax n))
("flemish" nil (f l eh1 m ih0 sh))
("flemister" nil (f l eh1 m ih0 s t er0))
("flemmer" nil (f l eh1 m er0))
("flemming" nil (f l eh1 m ih0 ng))
("flemons" nil (f l eh1 m ax n z))
("flener" nil (f l iy1 n er0))
("flenner" nil (f l eh1 n er0))
("flenniken" nil (f l eh1 n ih0 k ax n))
("flesch" nil (f l eh1 sh))
("flesh" nil (f l eh1 sh))
("fleshed" nil (f l eh1 sh t))
("flesher" nil (f l eh1 sh er0))
("fleshman" nil (f l eh1 sh m ax n))
("fleshy" nil (f l eh1 sh iy0))
("flesner" nil (f l eh1 s n er0))
("flessner" nil (f l eh1 s n er0))
("fleta" nil (f l iy1 t ax))
("fletchall" nil (f l eh1 ch ax l))
("fletcher" nil (f l eh1 ch er0))
("flett" nil (f l eh1 t))
("fleur" nil (f l er1))
("fleurette" nil (f l er0 eh1 t))
("fleuri" nil (f l er1 r iy1))
("fleury" nil (f l uh1 r iy0))
("flew" nil (f l uw1))
("flewellen" nil (f l uw1 eh1 l ax n))
("flewelling" nil (f l uw1 eh1 l ih0 ng))
("flex" nil (f l eh1 k s))
("flexed" nil (f l eh1 k s t))
("flexer" nil (f l eh1 k s er0))
("flexes" nil (f l eh1 k s ih0 z))
("flexi" nil (f l eh1 k s iy0))
("flexibility" nil (f l eh1 k s ax b ih1 l ax t iy0))
("flexible" nil (f l eh1 k s ax b ax l))
("flexibly" nil (f l eh1 k s ax b l iy0))
("flexing" nil (f l eh1 k s ih0 ng))
("flextime" nil (f l eh1 k s t ay1 m))
("flextronics" nil (f l eh1 k s t r aa1 n ih0 k s))
("flicek" nil (f l ih1 ch eh0 k))
("flick" nil (f l ih1 k))
("flicker" nil (f l ih1 k er0))
("flickered" nil (f l ih1 k er0 d))
("flickering" nil (f l ih1 k er0 ih0 ng))
("flickers" nil (f l ih1 k er0 z))
("flicking" nil (f l ih1 k ih0 ng))
("flickinger" nil (f l ih1 k ih0 ng er0))
("flicks" nil (f l ih1 k s))
("flied" nil (f l ay1 d))
("fliegel" nil (f l iy1 g ax l))
("flier" nil (f l ay1 er0))
("fliers" nil (f l ay1 er0 z))
("flies" nil (f l ay1 z))
("flight" nil (f l ay1 t))
("flightless" nil (f l ay1 t l ax s))
("flights" nil (f l ay1 t s))
("flightsafety" nil (f l ay1 t s ey1 f t iy0))
("flighty" nil (f l ay1 t iy0))
("flim" nil (f l ih1 m))
("flimflam" nil (f l ih1 m f l ae1 m))
("flimsiest" nil (f l ih1 m z iy0 ax s t))
("flimsy" nil (f l ih1 m z iy0))
("flinch" nil (f l ih1 n ch))
("flinchbaugh" nil (f l ih1 n ch b ao1))
("flinched" nil (f l ih1 n ch t))
("flinchum" nil (f l ih1 n k ax m))
("flinders" nil (f l ih1 n d er0 z))
("fling" nil (f l ih1 ng))
("flinging" nil (f l ih1 ng ih0 ng))
("flings" nil (f l ih1 ng z))
("flink" nil (f l ih1 ng k))
("flinn" nil (f l ih1 n))
("flinner" nil (f l ih1 n er0))
("flint" nil (f l ih1 n t))
("flintlocks" nil (f l ih1 n t l aa1 k s))
("flints" nil (f l ih1 n t s))
("flintstones" nil (f l ih1 n t s t ow1 n z))
("flinty" nil (f l ih1 n t iy0))
("flip" nil (f l ih1 p))
("flippant" nil (f l ih1 p ax n t))
("flipped" nil (f l ih1 p t))
("flippen" nil (f l ih1 p ax n))
("flipper" nil (f l ih1 p er0))
("flippers" nil (f l ih1 p er0 z))
("flippin" nil (f l ih1 p ih0 n))
("flipping" nil (f l ih1 p ih0 ng))
("flippo" nil (f l ih1 p ow0))
("flips" nil (f l ih1 p s))
("flirt" nil (f l er1 t))
("flirtation" nil (f l er0 t ey1 sh ax n))
("flirtations" nil (f l er0 t ey1 sh ax n z))
("flirtatious" nil (f l er0 t ey1 sh ax s))
("flirted" nil (f l er1 t ih0 d))
("flirting" nil (f l er1 t ih0 ng))
("flirts" nil (f l er1 t s))
("flis" nil (f l ih1 s))
("fliss" nil (f l ih1 s))
("flit" nil (f l ih1 t))
("flitting" nil (f l ih1 t ih0 ng))
("flo" nil (f l ow1))
("float" nil (f l ow1 t))
("floated" nil (f l ow1 t ax d))
("floater" nil (f l ow1 t er0))
("floaters" nil (f l ow1 t er0 z))
("floating" nil (f l ow1 t ih0 ng))
("floats" nil (f l ow1 t s))
("floc" nil (f l aa1 k))
("flock" nil (f l aa1 k))
("flocked" nil (f l aa1 k t))
("flockhart" nil (f l aa1 k hh aa1 r t))
("flocking" nil (f l aa1 k ih0 ng))
("flocks" nil (f l aa1 k s))
("flodin" nil (f l ow1 d ih0 n))
("floe" nil (f l ow1))
("floersheim" nil (f l ao1 r sh ay1 m))
("flogging" nil (f l aa1 g ih0 ng))
("flohr" nil (f l aa1 r))
("floirenda" nil (f l oy1 r eh1 n d ax))
("flom" nil (f l aa1 m))
("flonorial" nil (f l aa1 n ao1 r iy0 ax l))
("flood" nil (f l ah1 d))
("flooded" nil (f l ah1 d ax d))
("floodgates" nil (f l ah1 d g ey1 t s))
("flooding" nil (f l ah1 d ih0 ng))
("floodlights" nil (f l ah1 d l ay1 t s))
("floodplain" nil (f l ah1 d p l ey1 n))
("floods" nil (f l ah1 d z))
("floodwater" nil (f l ah1 d w ao1 t er0))
("floodwaters" nil (f l ah1 d w ao1 t er0 z))
("flook" nil (f l uh1 k))
("floor" nil (f l ao1 r))
("floorboards" nil (f l ao1 r b ao1 r d z))
("floored" nil (f l ao1 r d))
("flooring" nil (f l ao1 r ih0 ng))
("floors" nil (f l ao1 r z))
("flop" nil (f l aa1 p))
("flopped" nil (f l aa1 p t))
("flopping" nil (f l aa1 p ih0 ng))
("floppy" nil (f l aa1 p iy0))
("flops" nil (f l aa1 p s))
("floptical" nil (f l aa1 p t ih0 k ax l))
("flor" nil (f l ao1 r))
("flora" nil (f l ao1 r ax))
("florafax" nil (f l ao1 r ax f ae1 k s))
("floral" nil (f l ao1 r ax l))
("florance" nil (f l ao1 r ax n s))
("flore" nil (f l ao1 r))
("florea" nil (f l ao1 r iy0 ax))
("florek" nil (f l ao1 r ih0 k))
("floren" nil (f l ao1 r ax n))
("florence" nil (f l ao1 r ax n s))
("florentina" nil (f l ao1 r eh0 n t iy1 n ax))
("florentine" nil (f l ao1 r ax n t iy1 n))
("florentino" nil (f l ao0 r eh0 n t iy1 n ow0))
("florer" nil (f l ao1 r er0))
("flores" nil (f l ao1 r eh1 z))
("florescue" nil (f l ao1 r ax s k y uw0))
("florey" nil (f l ao1 r iy0))
("florez" nil (f l ao0 r eh1 z))
("flori" nil (f l ao1 r iy0))
("floria" nil (f l ao1 r iy0 ax))
("florian" nil (f l ao1 r iy0 ax n))
("florid" nil (f l ao1 r ax d))
("florida" nil (f l ao1 r ax d ax))
("floridabanc" nil (f l ao1 r ax d ax b ae1 ng k))
("floridian" nil (f l ao0 r ih1 d iy0 ax n))
("floridians" nil (f l ao0 r ih1 d iy0 ax n z))
("florin" nil (f l ao1 r ih0 n))
("florinda" nil (f l ao0 r iy1 n d ax))
("florine" nil (f l ao0 r iy1 n iy0))
("florio" nil (f l ao1 r iy0 ow0))
("floris" nil (f l ao1 r ih0 s))
("florist" nil (f l aa1 r ax s t))
("florists" nil (f l ao1 r ih0 s t s))
("floro" nil (f l ao1 r ow0))
("florrie" nil (f l ao1 r iy0))
("florry" nil (f l ao1 r iy0))
("florsheim" nil (f l ao1 r sh ay1 m))
("flory" nil (f l ao1 r iy0))
("floss" nil (f l aa1 s))
("flossie" nil (f l ao1 s iy0))
("flotation" nil (f l ow0 t ey1 sh ax n))
("flotilla" nil (f l ow0 t ih1 l ax))
("flotow" nil (f l aa1 t aw0))
("flotsam" nil (f l aa1 t s ax m))
("flott" nil (f l aa1 t))
("flotta" nil (f l aa1 t ax))
("flounces" nil (f l aw1 n s ih0 z))
("flounder" nil (f l aw1 n d er0))
("floundered" nil (f l aw1 n d er0 d))
("floundering" nil (f l aw1 n d er0 ih0 ng))
("flounders" nil (f l aw1 n d er0 z))
("flour" nil (f l aw1 er0))
("flourish" nil (f l er1 ih0 sh))
("flourished" nil (f l er1 ih0 sh t))
("flourishes" nil (f l er1 ih0 sh ax z))
("flourishing" nil (f l er1 ih0 sh ih0 ng))
("flournoy" nil (f l uh0 r n oy1))
("flours" nil (f l aw1 er0 z))
("flout" nil (f l aw1 t))
("flouted" nil (f l aw1 t ih0 d))
("flouting" nil (f l aw1 t ih0 ng))
("flow" nil (f l ow1))
("flowe" nil (f l ow1))
("flowed" nil (f l ow1 d))
("flower" nil (f l aw1 er0))
("flowered" nil (f l aw1 er0 d))
("flowering" nil (f l aw1 er0 ih0 ng))
("flowerpot" nil (f l aw1 er0 p aa1 t))
("flowers" nil (f l aw1 er0 z))
("flowery" nil (f l aw1 er0 iy0))
("flowing" nil (f l ow1 ih0 ng))
("flown" nil (f l ow1 n))
("flows" nil (f l ow1 z))
("flowton" nil (f l ow1 t ax n))
("floy" nil (f l oy1))
("floyd" nil (f l oy1 d))
("flu" nil (f l uw1))
("flubs" nil (f l ah1 b z))
("fluck" nil (f l ah1 k))
("fluctuate" nil (f l ah1 k ch ax w ey1 t))
("fluctuated" nil (f l ah1 k ch ax w ey1 t ih0 d))
("fluctuates" nil (f l ah1 k ch uw0 ey1 t s))
("fluctuating" nil (f l ah1 k ch ax w ey1 t ih0 ng))
("fluctuation" nil (f l ah1 k ch uw0 ey1 sh ax n))
("fluctuations" nil (f l ah1 k ch uw0 ey1 sh ax n z))
("flud" nil (f l ah1 d))
("fludd" nil (f l ah1 d))
("flue" nil (f l uw1))
("fluegel" nil (f l uh1 g ax l))
("fluegge" nil (f l uw1 g))
("fluency" nil (f l uw1 ax n s iy0))
("fluent" nil (f l uw1 ax n t))
("fluet" nil (f l uw1 t))
("fluff" nil (f l ah1 f))
("fluffed" nil (f l ah1 f t))
("fluffier" nil (f l ah1 f iy0 er0))
("fluffs" nil (f l ah1 f s))
("fluffy" nil (f l ah1 f iy0))
("fluharty" nil (f l ah1 er0 t iy0))
("fluhr" nil (f l er1))
("fluid" nil (f l uw1 ax d))
("fluidity" nil (f l uw0 ih1 d ax t iy0))
("fluids" nil (f l uw1 ax d z))
("fluitt" nil (f l uw1 ax t))
("fluke" nil (f l uw1 k))
("fluker" nil (f l uw1 k er0))
("flukes" nil (f l uw1 k s))
("flulike" nil (f l uw1 l ay1 k))
("flume" nil (f l uw1 m))
("flumenbaum" nil (f l uw1 m ax n b aw1 m))
("flumes" nil (f l uw1 m z))
("flung" nil (f l ah1 ng))
("flunk" nil (f l ah1 ng k))
("flunked" nil (f l ah1 ng k t))
("flunking" nil (f l ah1 ng k ih0 ng))
("flunks" nil (f l ah1 ng k s))
("fluor" nil (f l uw1 er0))
("fluoresce" nil (f l uh1 r eh1 s))
("fluorescent" nil (f l uh1 r eh1 s ax n t))
("fluorescently" nil (f l uh1 r eh1 s ax n t l iy0))
("fluorescents" nil (f l uh1 r eh1 s ax n t s))
("fluoridation" nil (f l uh1 r ax d ey1 sh ax n))
("fluoride" nil (f l uh1 r ay1 d))
("fluorides" nil (f l uh1 r ay1 d z))
("fluorine" nil (f l uh1 r iy1 n))
("fluorite" nil (f l uh1 r ay1 t))
("fluorocarbon" nil (f l uh1 r ow0 k aa1 r b ax n))
("fluorocarbons" nil (f l uh1 r ow0 k aa1 r b ax n z))
("fluorometer" nil (f l uh1 r aa1 m ax t er0))
("fluorspar" nil (f l uh1 r s p aa1 r))
("flurried" nil (f l er1 iy0 d))
("flurries" nil (f l er1 iy0 z))
("flurry" nil (f l er1 iy0))
("flury" nil (f l uw1 r iy0))
("flus" nil (f l uw1 z))
("flush" nil (f l ah1 sh))
("flushed" nil (f l ah1 sh t))
("flushes" nil (f l ah1 sh ih0 z))
("flushing" nil (f l ah1 sh ih0 ng))
("flustered" nil (f l ah1 s t er0 d))
("flute" nil (f l uw1 t))
("flutes" nil (f l uw1 t s))
("flutist" nil (f l uw1 t ih0 s t))
("flutter" nil (f l ah1 t er0))
("fluttered" nil (f l ah1 t er0 d))
("fluttering" nil (f l ah1 t er0 ih0 ng))
("flutters" nil (f l ah1 t er0 z))
("fluty" nil (f l uw1 t iy0))
("fluvial" nil (f l uw1 v iy0 ax l))
("flux" nil (f l ah1 k s))
("fluxional" nil (f l ah1 k sh ax n ax l))
("fly" nil (f l ay1))
("flyby" nil (f l ay1 b ay1))
("flycatcher" nil (f l ay1 k ae1 ch er0))
("flye" nil (f l ay1))
("flyer" nil (f l ay1 er0))
("flyers" nil (f l ay1 er0 z))
("flying" nil (f l ay1 ih0 ng))
("flynn" nil (f l ih1 n))
("flynt" nil (f l ih1 n t))
("flythe" nil (f l ay1 dh))
("flytrap" nil (f l ay1 t r ae1 p))
("flyway" nil (f l ay1 w ey1))
("flywheel" nil (f l ay1 w iy1 l))
("foal" nil (f ow1 l))
("foaling" nil (f ow1 l ih0 ng))
("foam" nil (f ow1 m))
("foams" nil (f ow1 m z))
("foamy" nil (f ow1 m iy0))
("foard" nil (f ao1 r d))
("fobbs" nil (f aa1 b z))
("fobes" nil (f ow1 b z))
("focal" nil (f ow1 k ax l))
("focht" nil (f aa1 k t))
("fochtman" nil (f aa1 k t m ax n))
("fockler" nil (f aa1 k l er0))
("focus" nil (f ow1 k ax s))
("focused" nil (f ow1 k ax s t))
("focuses" nil (f ow1 k ax s ih0 z))
("focusing" nil (f ow1 k ax s ih0 ng))
("focussed" nil (f ow1 k ax s t))
("focusses" nil (f ow1 k ax s ih0 z))
("focussing" nil (f ow1 k ax s ih0 ng))
("fodder" nil (f aa1 d er0))
("fodera" nil (f ow0 d eh1 r ax))
("fodor" nil (f ow1 d er0))
("foe" nil (f ow1))
("foehlich" nil (f ow1 l ih0 ch))
("foell" nil (f iy1 l))
("foeller" nil (f ow1 l er0))
("foerster" nil (f ao1 r s t er0))
("foertsch" nil (f ao1 r ch))
("foes" nil (f ow1 z))
("fog" nil (f aa1 g))
("fogal" nil (f ow1 jh ax l))
("fogarty" nil (f aa1 g aa1 r t iy0))
("fogel" nil (f ow1 g ax l))
("fogelberg" nil (f ow1 g ax l b er0 g))
("fogelman" nil (f ow1 g ax l m ax n))
("fogelson" nil (f ow1 g ax l s ax n))
("fogerty" nil (f aa1 jh er0 t iy0))
("fogg" nil (f aa1 g))
("foggiest" nil (f aa1 g iy0 ax s t))
("foggy" nil (f aa1 g iy0))
("fogle" nil (f ow1 g ax l))
("fogleman" nil (f ow1 g ax l m ax n))
("fogler" nil (f ow1 g ax l er0))
("foglesong" nil (f aa1 g l ih0 s ao0 ng))
("foglia" nil (f aa1 g l iy0 ax))
("foglio" nil (f aa1 g l iy0 ow0))
("fogt" nil (f aa1 g t))
("fogy" nil (f ow1 g iy0))
("fohl" nil (f ow1 l))
("foible" nil (f oy1 b ax l))
("foibles" nil (f oy1 b ax l z))
("foie" nil (f oy1))
("foil" nil (f oy1 l))
("foiled" nil (f oy1 l d))
("foiles" nil (f aa1 ay0 l z))
("foiling" nil (f oy1 l ih0 ng))
("foils" nil (f oy1 l z))
("foist" nil (f oy1 s t))
("foisted" nil (f oy1 s t ih0 d))
("foister" nil (f oy1 s t er0))
("foisy" nil (f oy1 z iy0))
("fojtik" nil (f aa1 y t ih0 k))
("fokker" nil (f aa1 k er0))
("folan" nil (f ow1 l ax n))
("foland" nil (f ow1 l ax n d))
("folck" nil (f ow1 l k))
("fold" nil (f ow1 l d))
("folded" nil (f ow1 l d ax d))
("folden" nil (f ow1 l d ax n))
("folder" nil (f ow1 l d er0))
("folders" nil (f ow1 l d er0 z))
("folding" nil (f ow1 l d ih0 ng))
("folds" nil (f ow1 l d z))
("foley" nil (f ow1 l iy0))
("folger" nil (f ow1 l jh er0))
("folgers" nil (f ow1 l jh er0 z))
("folha" nil (f ow1 l hh ax))
("foliage" nil (f ow1 l ih0 jh))
("foliate" nil (f ow1 l iy0 ey1 t))
("foliation" nil (f ow1 l iy0 ey1 sh ax n))
("folic" nil (f aa1 l ih0 k))
("folino" nil (f ow0 l iy1 n ow0))
("folk" nil (f ow1 k))
("folker" nil (f ow1 k er0))
("folkers" nil (f ow1 k er0 z))
("folkert" nil (f ow1 k er0 t))
("folkerts" nil (f ow1 k er0 t s))
("folkes" nil (f ow1 k s))
("folkish" nil (f ow1 k ih0 sh))
("folklore" nil (f ow1 k l ao1 r))
("folkman" nil (f ow1 k m ax n))
("folkrock" nil (f ow1 k r aa1 k))
("folks" nil (f ow1 k s))
("folkstone" nil (f ow1 k s t ow1 n))
("folksy" nil (f ow1 k s iy0))
("folktale" nil (f ow1 k t ey1 l))
("folkways" nil (f ow1 k w ey1 z))
("folland" nil (f aa1 l ax n d))
("follett" nil (f aa1 l ih0 t))
("follette" nil (f aa1 l eh1 t))
("folley" nil (f aa1 l iy0))
("folliard" nil (f aa1 l y er0 d))
("follicle" nil (f aa1 l ax k ax l))
("follicles" nil (f aa1 l ih0 k ax l z))
("follies" nil (f aa1 l iy0 z))
("follin" nil (f aa1 l ih0 n))
("follis" nil (f aa1 l ih0 s))
("follman" nil (f aa1 l m ax n))
("follmer" nil (f aa1 l m er0))
("follow" nil (f aa1 l ow0))
("followed" nil (f aa1 l ow0 d))
("followell" nil (f aa1 l ax w eh0 l))
("follower" nil (f aa1 l ow0 er0))
("followers" nil (f aa1 l ow0 er0 z))
("following" nil (f aa1 l ow0 ih0 ng))
("followings" nil (f aa1 l ow0 ih0 ng z))
("follows" nil (f aa1 l ow0 z))
("followup" nil (f aa1 l ow0 ah1 p))
("folly" nil (f aa1 l iy0))
("folmar" nil (f ow1 l m er0))
("folmer" nil (f ow1 l m er0))
("folse" nil (f ow1 l s))
("folsom" nil (f ow1 l s ax m))
("folson" nil (f ow1 l s ax n))
("folta" nil (f ow1 l t ax))
("foltene" nil (f ow0 l t iy1 n))
("folts" nil (f ow1 l t s))
("foltz" nil (f ow1 l t s))
("folwell" nil (f ow1 l w eh1 l))
("folz" nil (f ow1 l z))
("fomby" nil (f aa1 m b iy0))
("foment" nil (f ow1 m eh0 n t))
("fomented" nil (f ow1 m eh0 n t ih0 d))
("fomenting" nil (f ow1 m eh0 n t ih0 ng))
("fomento" nil (f ow1 m eh1 n t ow0))
("fomon" nil (f ow1 m ax n))
("fonar" nil (f aa1 n er0))
("foncier" nil (f aa1 n s y er0))
("fond" nil (f aa1 n d))
("fonda" nil (f aa1 n d ax))
("fonder" nil (f aa1 n d er0))
("fondest" nil (f aa1 n d ax s t))
("fondiaria" nil (f aa1 n d iy0 eh1 r iy0 ax))
("fondkommission" nil (f aa1 n d k ax m ih1 sh ax n))
("fondly" nil (f aa1 n d l iy0))
("fondness" nil (f aa1 n d n ax s))
("fondren" nil (f aa1 n d er0 ax n))
("fondue" nil (f aa0 n d y uw1))
("fondues" nil (f aa1 n d uw0 z))
("fone" nil (f ow1 n))
("foner" nil (f ow1 n er0))
("fones" nil (f ow1 n z))
("fong" nil (f ao1 ng))
("fonger" nil (f ao1 ng er0))
("fonner" nil (f aa1 n er0))
("fons" nil (f aa1 n z))
("fonseca" nil (f ow0 n s eh1 k ax))
("font" nil (f aa1 n t))
("fontaine" nil (f ao0 n t ey1 n))
("fontana" nil (f ao0 n t ae1 n ax))
("fontanella" nil (f aa1 n t ax n eh1 l ax))
("fontanez" nil (f ow0 n t aa1 n eh0 z))
("fontanilla" nil (f aa1 n t ax n ih1 l ax))
("fonte" nil (f aa1 n t))
("fontenette" nil (f aa1 n t ih0 n eh0 t))
("fontenot" nil (f aa1 n t ih0 n ax t))
("fontes" nil (f ow1 n t eh0 s))
("fonts" nil (f aa1 n t s))
("fonville" nil (f ow1 n v ih0 l))
("foo" nil (f uw1))
("food" nil (f uw1 d))
("foodarama" nil (f uw1 d er0 ae1 m ax))
("foodmaker" nil (f uw1 d m ey1 k er0))
("foods" nil (f uw1 d z))
("foodservice" nil (f uw1 d s er1 v ih0 s))
("foodstuff" nil (f uw1 d s t ah1 f))
("foodstuffs" nil (f uw1 d s t ah1 f s))
("foodtown" nil (f uw1 d t aw1 n))
("foodways" nil (f uw1 d w ey1 z))
("fooks" nil (f uh1 k s))
("fool" nil (f uw1 l))
("fooled" nil (f uw1 l d))
("foolery" nil (f uw1 l er0 iy0))
("foolhardy" nil (f uw1 l hh aa1 r d iy0))
("fooling" nil (f uw1 l ih0 ng))
("foolish" nil (f uw1 l ih0 sh))
("foolishly" nil (f uw1 l ih0 sh l iy0))
("foolishness" nil (f uw1 l ih0 sh n ax s))
("foolproof" nil (f uw1 l p r uw1 f))
("fools" nil (f uw1 l z))
("foong" nil (f uw1 ng))
("foor" nil (f uh1 r))
("foos" nil (f uw1 z))
("foose" nil (f uw1 s))
("fooshee" nil (f uw1 sh iy0))
("foot" nil (f uh1 t))
("footage" nil (f uh1 t ih0 jh))
("football" nil (f uh1 t b ao1 l))
("footballs" nil (f uh1 t b ao1 l z))
("foote" nil (f uh1 t))
("footed" nil (f uh1 t ih0 d))
("footer" nil (f uh1 t er0))
("footfall" nil (f uh1 t f ao1 l))
("foothill" nil (f uh1 t hh ih1 l))
("foothills" nil (f uh1 t hh ih1 l z))
("foothold" nil (f uh1 t hh ow1 l d))
("footholds" nil (f uh1 t hh ow1 l d z))
("footing" nil (f uh1 t ih0 ng))
("footlights" nil (f uh1 t l ay1 t s))
("footloose" nil (f uh1 t l uw1 s))
("footman" nil (f uh1 t m ax n))
("footnote" nil (f uh1 t n ow1 t))
("footnoted" nil (f uh1 t n ow1 t ih0 d))
("footnotes" nil (f uh1 t n ow1 t s))
("footnoting" nil (f uh1 t n ow1 t ih0 ng))
("footpath" nil (f uh1 t p ae1 th))
("footprint" nil (f uh1 t p r ih1 n t))
("footprints" nil (f uh1 t p r ih1 n t s))
("footrace" nil (f uh1 t r ey1 s))
("foots" nil (f uh1 t s))
("footsteps" nil (f uh1 t s t eh1 p s))
("footwall" nil (f uh1 t w ao1 l))
("footware" nil (f uh1 t w eh1 r))
("footwear" nil (f uh1 t w eh1 r))
("footwork" nil (f uh1 t w er1 k))
("foppiano" nil (f ow0 p iy0 aa1 n ow0))
("for" nil (f ao1 r))
("forage" nil (f ao1 r ih0 jh))
("forages" nil (f ao1 r ax jh ax z))
("foraging" nil (f ao1 r ih0 jh ih0 ng))
("foraker" nil (f ao1 r ax k er0))
("foramen" nil (f er0 ey1 m ax n))
("foran" nil (f ao1 r ax n))
("forand" nil (f ao1 r ax n d))
("foray" nil (f ao1 r ey0))
("forays" nil (f ao1 r ey0 z))
("forbade" nil (f er0 b ey1 d))
("forbearance" nil (f ao0 r b eh1 r ax n s))
("forbes" nil (f ao1 r b z))
("forbess" nil (f ao1 r b z ih0 z))
("forbid" nil (f er0 b ih1 d))
("forbidden" nil (f ao1 r b ih0 d ax n))
("forbidding" nil (f er0 b ih1 d ih0 ng))
("forbids" nil (f er0 b ih1 d z))
("forbis" nil (f ao1 r b ih0 s))
("forbus" nil (f ao1 r b ih0 s))
("forbush" nil (f ao1 r b ax sh))
("force" nil (f ao1 r s))
("forced" nil (f ao1 r s t))
("forceful" nil (f ao1 r s f ax l))
("forcefully" nil (f ao1 r s f ax l iy0))
("forcefulness" nil (f ao1 r s f ax l n ax s))
("forces" nil (f ao1 r s ih0 z))
("forcible" nil (f ao1 r s ax b ax l))
("forcibly" nil (f ao1 r s ax b l iy0))
("forcier" nil (f ao1 r k iy0 er0))
("forcing" nil (f ao1 r s ih0 ng))
("forcum" nil (f ao1 r k ax m))
("ford" nil (f ao1 r d))
("forde" nil (f ao1 r d))
("fordham" nil (f ao1 r d ax m))
("fords" nil (f ao1 r d z))
("fordyce" nil (f ao1 r d ay1 s))
("fore" nil (f ao1 r))
("forearm" nil (f ao0 r aa1 r m))
("forebearance" nil (f ao1 r b eh1 r ax n s))
("forebears" nil (f ao1 r b eh1 r z))
("foreboding" nil (f ao0 r b ow1 d ih0 ng))
("forebrain" nil (f ao1 r b r ey1 n))
("forecast" nil (f ao1 r k ae1 s t))
("forecasted" nil (f ao1 r k ae1 s t ih0 d))
("forecaster" nil (f ao1 r k ae1 s t er0))
("forecasters" nil (f ao1 r k ae1 s t er0 z))
("forecasting" nil (f ao1 r k ae1 s t ih0 ng))
("forecasts" nil (f ao0 r k ae1 s t s))
("foreclose" nil (f ao0 r k l ow1 z))
("foreclosed" nil (f ao0 r k l ow1 z d))
("forecloses" nil (f ao0 r k l ow1 z ih0 z))
("foreclosing" nil (f ao0 r k l ow1 z ih0 ng))
("foreclosure" nil (f ao0 r k l ow1 zh er0))
("foreclosures" nil (f ao0 r k l ow1 zh er0 z))
("foree" nil (f ao1 r iy1))
("forefathers" nil (f ao1 r f aa1 dh er0 z))
("forefinger" nil (f ao1 r f ih1 ng g er0))
("forefoot" nil (f ao1 r f uh1 t))
("forefront" nil (f ao1 r f r ah1 n t))
("forego" nil (f ao0 r g ow1))
("foregoing" nil (f ao0 r g ow1 ih0 ng))
("foregone" nil (f ao1 r g ao1 n))
("foreground" nil (f ao1 r g r aw1 n d))
("forehand" nil (f ao1 r hh ae1 n d))
("forehands" nil (f ao1 r hh ae1 n d z))
("forehead" nil (f ao1 r hh eh0 d))
("foreheads" nil (f ao1 r hh eh1 d z))
("foreign" nil (f ao1 r ax n))
("foreigner" nil (f ao1 r ax n er0))
("foreigners" nil (f ao1 r ax n er0 z))
("forelimbs" nil (f ao1 r l ih1 m z))
("foreman" nil (f ao1 r m ax n))
("foremen" nil (f ao1 r m ax n))
("foremost" nil (f ao1 r m ow1 s t))
("forensic" nil (f er0 eh1 n s ih0 k))
("forensics" nil (f er0 eh1 n s ih0 k s))
("forero" nil (f ao1 r ow0))
("forerunner" nil (f ao1 r ah1 n er0))
("forerunners" nil (f ao1 r ah1 n er0 z))
("foresaw" nil (f ao1 r s ao1))
("foresee" nil (f ao0 r s iy1))
("foreseeable" nil (f ao0 r s iy1 ax b ax l))
("foreseeing" nil (f ao0 r s iy1 ih0 ng))
("foreseen" nil (f ao1 r s iy1 n))
("foresees" nil (f ao0 r s iy1 z))
("foreshadow" nil (f ao0 r sh ae1 d ow0))
("foreshadowed" nil (f ao0 r sh ae1 d ow0 d))
("foreshadowing" nil (f ao0 r sh ae1 d ow0 ih0 ng))
("foreshadows" nil (f ao0 r sh ae1 d ow0 z))
("foresight" nil (f ao1 r s ay1 t))
("foresman" nil (f ao1 r s m ax n))
("forest" nil (f ao1 r ax s t))
("foresta" nil (f ao1 r s t ax))
("forestall" nil (f ao0 r s t ao1 l))
("forestalled" nil (f ao1 r s t aa1 l d))
("forestalling" nil (f ao1 r s t aa1 l ih0 ng))
("forested" nil (f ao1 r ax s t ax d))
("forester" nil (f ao1 r ax s t er0))
("foresters" nil (f ao1 r ax s t er0 z))
("forestry" nil (f ao1 r ax s t r iy0))
("forests" nil (f ao1 r ax s t s))
("foret" nil (f ao1 r t))
("foretaste" nil (f ao0 r t ey1 s t))
("foretell" nil (f ao0 r t eh1 l))
("foretelling" nil (f ao0 r t eh1 l ih0 ng))
("forethought" nil (f ao1 r th ao1 t))
("foretold" nil (f ao0 r t ow1 l d))
("forever" nil (f er0 eh1 v er0))
("forewarned" nil (f ao0 r w ao1 r n d))
("forewings" nil (f ao1 r w ih1 ng z))
("foreword" nil (f ao1 r w er1 d))
("forex" nil (f ao1 r eh0 k s))
("forfeit" nil (f ao1 r f ih0 t))
("forfeitable" nil (f ao1 r f ax t ax b ax l))
("forfeited" nil (f ao1 r f ih0 t ih0 d))
("forfeiting" nil (f ao1 r f ax t ih0 ng))
("forfeiture" nil (f ao1 r f ax ch er0))
("forfeitures" nil (f ao1 r f ax ch er0 z))
("forgave" nil (f er0 g ey1 v))
("forge" nil (f ao1 r jh))
("forged" nil (f ao1 r jh d))
("forger" nil (f ao1 r jh er0))
("forgeries" nil (f ao1 r jh er0 iy0 z))
("forgers" nil (f ao1 r jh er0 z))
("forgery" nil (f ao1 r jh er0 iy0))
("forges" nil (f ao1 r jh ih0 z))
("forget" nil (f er0 g eh1 t))
("forgetful" nil (f ao0 r g eh1 t f ax l))
("forgets" nil (f er0 g eh1 t s))
("forgettable" nil (f ao0 r g eh1 t ax b ax l))
("forgette" nil (f er0 zh eh1 t))
("forgetting" nil (f er0 g eh1 t ih0 ng))
("forgey" nil (f ao1 r jh iy0))
("forgie" nil (f ao1 r jh iy0))
("forging" nil (f ao1 r jh ih0 ng))
("forgings" nil (f ao1 jh ih0 ng z))
("forgione" nil (f ao0 r jh ow1 n iy0))
("forgivable" nil (f ao0 r g ih1 v ax b ax l))
("forgive" nil (f er0 g ih1 v))
("forgiven" nil (f er0 g ih1 v ax n))
("forgiveness" nil (f er0 g ih1 v n ax s))
("forgiving" nil (f er0 g ih1 v ih0 ng))
("forgo" nil (f ao0 r g ow1))
("forgoes" nil (f ao0 r g ow1 z))
("forgoing" nil (f ao0 r g ow1 ih0 ng))
("forgone" nil (f ao0 r g aa1 n))
("forgot" nil (f er0 g aa1 t))
("forgotten" nil (f er0 g aa1 t ax n))
("forgue" nil (f ao1 r g))
("forgy" nil (f ao1 r jh iy0))
("forhan" nil (f ao1 r hh ax n))
("forie" nil (f ao1 r iy0))
("forinash" nil (f ao1 r ih0 n ae0 sh))
("forint" nil (f ao1 r ih0 n t))
("forints" nil (f ao1 r ih0 n t s))
("forister" nil (f ao1 r ih0 s t er0))
("fork" nil (f ao1 r k))
("forked" nil (f ao1 r k t))
("forker" nil (f ao1 r k er0))
("forkey" nil (f ao1 r k iy1))
("forking" nil (f ao1 r k ih0 ng))
("forklift" nil (f ao1 r k l ih1 f t))
("forklifts" nil (f ao1 r k l ih1 f t s))
("forkner" nil (f ao1 r k n er0))
("forks" nil (f ao1 r k s))
("forlenza" nil (f ao0 r l eh1 n z ax))
("forlorn" nil (f er0 l ao1 r n))
("form" nil (f ao1 r m))
("forma" nil (f ao1 r m ax))
("formal" nil (f ao1 r m ax l))
("formaldehyde" nil (f ao0 r m ae1 l d ax hh ay1 d))
("formalism" nil (f ao1 r m ax l ih1 z ax m))
("formalist" nil (f ao1 r m ax l ax s t))
("formalities" nil (f ao0 r m ae1 l ax t iy0 z))
("formality" nil (f ao0 r m ae1 l ax t iy0))
("formalization" nil (f ao1 r m ax l ax z ey0 sh ax n))
("formalize" nil (f ao1 r m ax l ay1 z))
("formalized" nil (f ao1 r m ax l ay1 z d))
("formalizes" nil (f ao1 r m ax l ay1 z ih0 z))
("formalizing" nil (f ao1 r m ax l ay1 z ih0 ng))
("formally" nil (f ao1 r m ax l iy0))
("forman" nil (f ao1 r m ae1 n))
("formanek" nil (f ao1 r m ax n ih0 k))
("formant" nil (f ao1 r m ax n t))
("formants" nil (f ao1 r m ax n t s))
("format" nil (f ao1 r m ae1 t))
("formation" nil (f ao0 r m ey1 sh ax n))
("formations" nil (f ao0 r m ey1 sh ax n z))
("formative" nil (f ao1 r m ax t ih0 v))
("formato" nil (f ao0 r m aa1 t ow0))
("formats" nil (f ao1 r m ae1 t s))
("formby" nil (f ao1 r m b iy0))
("formed" nil (f ao1 r m d))
("former" nil (f ao1 r m er0))
("formerly" nil (f ao1 r m er0 l iy0))
("formic" nil (f ao1 r m ih0 k))
("formica" nil (f ao0 r m ay1 k ax))
("formidable" nil (f ao1 r m ax d ax b ax l))
("formidably" nil (f ao1 r m ax d ax b l iy0))
("forming" nil (f ao1 r m ih0 ng))
("formosa" nil (f ao0 r m ow1 s ax))
("formoso" nil (f ao0 r m ow1 s ow0))
("forms" nil (f ao1 r m z))
("formula" nil (f ao1 r m y ax l ax))
("formulaic" nil (f ao1 r m y ax l ey1 ih0 k))
("formulary" nil (f ao1 r m y ax l eh1 r iy0))
("formulas" nil (f ao1 r m y ax l ax z))
("formulate" nil (f ao1 r m y ax l ey1 t))
("formulated" nil (f ao1 r m y ax l ey1 t ax d))
("formulates" nil (f ao1 r m y ax l ey1 t s))
("formulating" nil (f ao1 r m y ax l ey1 t ih0 ng))
("formulation" nil (f ao1 r m y ax l ey1 sh ax n))
("formulations" nil (f ao1 r m y uw0 l ey1 sh ax n z))
("fornal" nil (f ao1 r n ax l))
("forner" nil (f ao1 r n er0))
("fornes" nil (f ao1 r n z))
("forness" nil (f er1 n ih0 s))
("forney" nil (f ao1 r n iy0))
("forni" nil (f ao1 r n iy0))
("fornoff" nil (f ao1 r n ao0 f))
("fornwalt" nil (f ao1 r n w ax l t))
("forquer" nil (f ao1 r k er0))
("forrer" nil (f ao1 er0 r))
("forrest" nil (f ao1 r ax s t))
("forrestal" nil (f ao1 r ax s t ax l))
("forrester" nil (f ao1 r ax s t er0))
("forrey" nil (f ao1 r iy0))
("forry" nil (f ao1 r iy0))
("fors" nil (f er1 z))
("forsake" nil (f ao0 r s ey1 k))
("forsaken" nil (f ao0 r s ey1 k ax n))
("forsaking" nil (f ao0 r s ey1 k ih0 ng))
("forsberg" nil (f ao1 r s b er0 g))
("forsburg" nil (f ao1 r s b er0 g))
("forschner" nil (f ao1 r sh n er0))
("forse" nil (f ao1 r s))
("forsee" nil (f er0 s iy1))
("forseeable" nil (f ao1 r s iy1 ax b ax l))
("forsell" nil (f ao1 r s ax l))
("forseth" nil (f ao1 r s ih0 th))
("forsgren" nil (f ao1 r s g r eh0 n))
("forshan" nil (f ao1 r sh ax n))
("forshee" nil (f ao1 r sh iy0))
("forshey" nil (f ao1 r sh iy0))
("forslund" nil (f ao1 r s l ax n d))
("forsman" nil (f ao1 r s m ax n))
("forson" nil (f ao1 r s ax n))
("forsook" nil (f ao0 r s uh1 k))
("forst" nil (f ao1 r s t))
("forster" nil (f ao1 r s t er0))
("forstmann" nil (f ao1 r s t m ax n))
("forstner" nil (f ao1 r s t n er0))
("forstrom" nil (f ao1 r s t r ax m))
("forswear" nil (f ao0 r s w eh1 r))
("forsworn" nil (f ao1 r s w ao1 r n))
("forsyth" nil (f ao1 r s ay1 th))
("forsythe" nil (f ao1 r s ay0 dh))
("forsythias" nil (f ao0 r s ih1 th iy0 ax z))
("fort" nil (f ao1 r t))
("fortas" nil (f ao1 r t ax s))
("forte" nil (f ao1 r t ey0))
("fortenberry" nil (f ao1 r t ax n b eh0 r iy0))
("fortes" nil (f ao1 r t ey0 z))
("forth" nil (f ao1 r th))
("forthcoming" nil (f ao1 r th k ah1 m ih0 ng))
("forthright" nil (f ao1 r th r ay1 t))
("forthrightly" nil (f ao1 r th r ay1 t l iy0))
("forthrightness" nil (f ao1 r th r ay1 t n ax s))
("forthwith" nil (f ao1 r th w ih1 th))
("forti" nil (f ao1 r t iy0))
("fortier" nil (f ao1 r t iy0 er0))
("forties" nil (f ao1 r t iy0 z))
("fortieth" nil (f ao1 r t iy0 ih0 th))
("fortifications" nil (f ao1 r t ax f ax k ey1 sh ax n z))
("fortified" nil (f ao1 r t ax f ay1 d))
("fortifiers" nil (f ao1 r t ax f ay1 er0 z))
("fortify" nil (f ao1 r t ih0 f ay1))
("fortifying" nil (f ao1 r t ih0 f ay1 ih0 ng))
("fortin" nil (f ao1 r t ih0 n))
("fortini" nil (f ao0 r t iy1 n iy0))
("fortino" nil (f ao0 r t iy1 n ow0))
("fortis" nil (f ao1 r t ih0 s))
("fortitude" nil (f ao1 r t ih0 t uw1 d))
("fortman" nil (f ao1 r t m ax n))
("fortna" nil (f ao1 r t n ax))
("fortner" nil (f ao1 r t n er0))
("fortney" nil (f ao1 r t n iy0))
("fortnight" nil (f ao1 r t n ay1 t))
("fortnightly" nil (f ao1 r t n ay1 t l iy0))
("forton" nil (f ao1 r t ax n))
("fortress" nil (f ao1 r t r ax s))
("forts" nil (f ao1 r t s))
("fortson" nil (f ao1 r t s ax n))
("fortuitous" nil (f ao0 r t uw1 ih0 t ax s))
("fortuna" nil (f ao0 r t uw1 n ax))
("fortunate" nil (f ao1 r ch ax n ax t))
("fortunately" nil (f ao1 r ch ax n ax t l iy0))
("fortunato" nil (f ao0 r t uw0 n aa1 t ow0))
("fortune" nil (f ao1 r ch ax n))
("fortunes" nil (f ao1 r ch ax n z))
("forty" nil (f ao1 r t iy0))
("forum" nil (f ao1 r ax m))
("forums" nil (f ao1 r ax m z))
("forward" nil (f ao1 r w er0 d))
("forwarded" nil (f ao1 r w er0 d ih0 d))
("forwarder" nil (f ao1 r w er0 d er0))
("forwarders" nil (f ao1 r w er0 d er0 z))
("forwarding" nil (f ao1 r w er0 d ih0 ng))
("forwards" nil (f ao1 r w er0 d z))
("forwood" nil (f ao1 r w uh1 d))
("forys" nil (f ao1 r iy0 z))
("forza" nil (f ao1 r z ax))
("fosamax" nil (f aa1 s ax m ae1 k s))
("fosback" nil (f aa1 s b ae1 k))
("fosberg" nil (f aa1 s b er0 g))
("fosco" nil (f aa1 s k ow0))
("fosdick" nil (f aa1 s d ih0 k))
("foshee" nil (f aa1 sh iy0))
("foskett" nil (f aa1 s k ih0 t))
("foskey" nil (f aa1 s k iy0))
("fosler" nil (f aa1 s ax l er0))
("fosnaugh" nil (f aa1 s n ao0))
("foss" nil (f aa1 s))
("fosse" nil (f aa1 s))
("fossel" nil (f aa1 s ax l))
("fossen" nil (f aa1 s ax n))
("fossett" nil (f aa1 s ih0 t))
("fossey" nil (f aa1 s iy0))
("fossil" nil (f aa1 s ax l))
("fossiliferous" nil (f aa1 s ax l ih1 f er0 ax s))
("fossilized" nil (f aa1 s ax l ay1 z d))
("fossils" nil (f aa1 s ax l z))
("fossum" nil (f aa1 s ax m))
("foster" nil (f aa1 s t er0))
("fostered" nil (f aa1 s t er0 d))
("fostering" nil (f aa1 s t er0 ih0 ng))
("fosters" nil (f aa1 s t er0 z))
("foth" nil (f aa1 th))
("fothergill" nil (f ah1 dh er0 g ih1 l))
("fotheringham" nil (f ah1 dh er0 ih0 ng hh ae0 m))
("foti" nil (f ow1 t iy0))
("fotopoulos" nil (f ax t aa1 p ax l ih0 s))
("fouch" nil (f aw1 ch))
("fouche" nil (f aw1 ch))
("foucher" nil (f aw1 k er0))
("fougere" nil (f aw1 g er0))
("fought" nil (f ao1 t))
("foul" nil (f aw1 l))
("foulds" nil (f ow1 l d z))
("fouled" nil (f aw1 l d))
("fouling" nil (f aw1 l ih0 ng))
("foulk" nil (f aw1 l k))
("foulke" nil (f aw1 l k))
("foulkes" nil (f aw1 l k s))
("foulks" nil (f uw1 l k s))
("foulness" nil (f aw1 l n ax s))
("fouls" nil (f aw1 l z))
("found" nil (f aw1 n d))
("foundation" nil (f aw0 n d ey1 sh ax n))
("foundations" nil (f aw0 n d ey1 sh ax n z))
("founded" nil (f aw1 n d ax d))
("founder" nil (f aw1 n d er0))
("foundered" nil (f aw1 n d er0 d))
("foundering" nil (f aw1 n d er0 ih0 ng))
("founders" nil (f aw1 n d er0 z))
("founding" nil (f aw1 n d ih0 ng))
("foundling" nil (f aw1 n d l ih0 ng))
("foundries" nil (f aw1 n d r iy0 z))
("foundry" nil (f aw1 n d r iy0))
("fountain" nil (f aw1 n t ax n))
("fountaine" nil (f uw0 n t ey1 n))
("fountains" nil (f aw1 n t ax n z))
("four" nil (f ao1 r))
("fourfold" nil (f ao1 r f ow1 l d))
("fourman" nil (f ao1 r m ax n))
("fournet" nil (f uh0 r n eh1 t))
("fournier" nil (f ao1 r n iy0 er0))
("fours" nil (f ao1 r z))
("foursome" nil (f ao1 r s ax m))
("foursquare" nil (f ao1 r s k w eh1 r))
("fourteen" nil (f ao1 r t iy1 n))
("fourteens" nil (f ao1 r t iy1 n z))
("fourteenth" nil (f ao1 r t iy1 n th))
("fourth" nil (f ao1 r th))
("fourthquarter" nil (f ao1 r th k w ao1 r t er0))
("fourths" nil (f ao1 r th s))
("fourtou" nil (f ao0 r t uw1))
("fouse" nil (f aw1 s))
("fousek" nil (f aw1 s ih0 k))
("foushee" nil (f aw1 sh iy0))
("foust" nil (f aw1 s t))
("fout" nil (f aw1 t))
("foutch" nil (f aw1 ch))
("fouts" nil (f aw1 t s))
("fouty" nil (f aw1 t iy0))
("foutz" nil (f aw1 t s))
("foward" nil (f aw1 r d))
("fowble" nil (f aw1 b ax l))
("fowers" nil (f aw1 er0 z))
("fowkes" nil (f aw1 k s))
("fowl" nil (f aw1 l))
("fowle" nil (f aw1 ax l))
("fowler" nil (f aw1 l er0))
("fowles" nil (f aw1 ax l z))
("fowlkes" nil (f aw1 l k s))
("fox" nil (f aa1 k s))
("foxboro" nil (f aa1 k s b er0 ow0))
("foxes" nil (f aa1 k s ax z))
("foxglove" nil (f aa1 k s g l ah1 v))
("foxhole" nil (f aa1 k s hh ow1 l))
("foxholes" nil (f aa1 k s hh ow1 l z))
("foxhound" nil (f aa1 k s hh aw1 n d))
("foxman" nil (f aa1 k s m ax n))
("foxmeyer" nil (f aa1 k s m ay1 r))
("foxtail" nil (f aa1 k s t ey1 l))
("foxwell" nil (f aa1 k s w eh1 l))
("foxwoods" nil (f aa1 k s w uh1 d z))
("foxworth" nil (f aa1 k s w er1 th))
("foxworthy" nil (f aa1 k s w er1 dh iy0))
("foxx" nil (f aa1 k s k s))
("foxy" nil (f aa1 k s iy0))
("foy" nil (f oy1))
("foye" nil (f oy1))
("foyer" nil (f oy1 er0))
("foyle" nil (f oy1 l))
("foyt" nil (f oy1 t))
("fraas" nil (f r aa1 z))
("frable" nil (f r ey1 b ax l))
("fracas" nil (f r ey1 k ax s))
("frace" nil (f r ey1 s))
("fraction" nil (f r ae1 k sh ax n))
("fractional" nil (f r ae1 k sh ax n ax l))
("fractionally" nil (f r ae1 k sh ax n ax l iy0))
("fractions" nil (f r ae1 k sh ax n z))
("fractious" nil (f r ae1 k sh ax s))
("fractiousness" nil (f r ae1 k sh ax s n ax s))
("fracture" nil (f r ae1 k ch er0))
("fractured" nil (f r ae1 k ch er0 d))
("fractures" nil (f r ae1 k ch er0 z))
("fracturing" nil (f r ae1 k ch er0 ih0 ng))
("fradette" nil (f r ax d eh1 t))
("fradkin" nil (f r ae1 d k ih0 n))
("frady" nil (f r ey1 d iy0))
("fraga" nil (f r aa1 g ax))
("fragale" nil (f r aa0 g aa1 l iy0))
("frager" nil (f r ey1 g er0))
("fragile" nil (f r ae1 jh ax l))
("fragility" nil (f r ax jh ih1 l ax t iy0))
("fragment" n (f r ae1 g m ax n t))
("fragment" v (f r ae0 g m eh1 n t))
("fragmentary" nil (f r ae1 g m ax n t eh1 r iy0))
("fragmentation" nil (f r ae1 g m ax n t ey1 sh ax n))
("fragmented" nil (f r ae1 g m ax n t ih0 d))
("fragmenting" nil (f r ae1 g m ax n t ih0 ng))
("fragments" n (f r ae1 g m ax n t s))
("fragments" v (f r ae0 g m eh1 n t s))
("frago" nil (f r ey1 g ow0))
("fragonard" nil (f r ae1 g ax n er0 d))
("fragoso" nil (f r aa0 g ow1 s ow0))
("fragrance" nil (f r ey1 g r ax n s))
("fragrances" nil (f r ey1 g r ax n s ax z))
("fragrant" nil (f r ey1 g r ax n t))
("fraher" nil (f r aa1 er0))
("frahm" nil (f r ae1 m))
("frail" nil (f r ey1 l))
("frailey" nil (f r ey1 l iy0))
("frailties" nil (f r ey1 l t iy0 z))
("frailty" nil (f r ey1 l t iy0))
("fraim" nil (f r ey1 m))
("frain" nil (f r ey1 n))
("fraine" nil (f r ey1 n))
("fraioli" nil (f r ay0 ow1 l iy0))
("fraire" nil (f r eh1 r))
("fraiser" nil (f r ey1 zh er0))
("fraizer" nil (f r ey1 zh er0))
("fraker" nil (f r ey1 k er0))
("frakes" nil (f r ey1 k s))
("fraleigh" nil (f r ey1 l iy0))
("fraley" nil (f r ey1 l iy0))
("fralick" nil (f r ae1 l ih0 k))
("fralin" nil (f r ae1 l ih0 n))
("fralix" nil (f r ae1 l ih0 k s))
("fram" nil (f r ae1 m))
("framatome" nil (f r ae1 m ax t ow1 m))
("frame" nil (f r ey1 m))
("framed" nil (f r ey1 m d))
("framer" nil (f r ey1 m er0))
("framers" nil (f r ey1 m er0 z))
("frames" nil (f r ey1 m z))
("framework" nil (f r ey1 m w er1 k))
("framing" nil (f r ey1 m ih0 ng))
("framingham" nil (f r ey1 m ih0 ng hh ae1 m))
("frampton" nil (f r ae1 m p t ax n))
("fran" nil (f r ae1 n))
("frana" nil (f r ae1 n ax))
("franc" nil (f r ae1 ng k))
("franca" nil (f r ae1 ng k ax))
("francais" nil (f r aa0 n s ey1))
("francaise" nil (f r aa0 n s eh1 z))
("francaises" nil (f r aa0 n s eh1 z))
("francavilla" nil (f r aa0 n k aa0 v ih1 l ax))
("france" nil (f r ae1 n s))
("francek" nil (f r ae1 n ch eh1 k))
("frances" nil (f r ae1 n s ih0 s))
("francesca" nil (f r ae0 n ch eh1 s k ax))
("franceschi" nil (f r aa0 n ch eh1 s k iy0))
("franceschini" nil (f r aa0 n ch eh0 s k iy1 n iy0))
("francesco" nil (f r ae0 n ch eh1 s k ow0))
("francesconi" nil (f r aa0 n ch eh0 s k ow1 n iy0))
("francese" nil (f r aa0 n ch ey1 z iy0))
("francesmary" nil (f r ae1 n s ax s m eh1 r iy0))
("franchi" nil (f r aa1 n k iy0))
("franchik" nil (f r ae1 n ch ih0 k))
("franchini" nil (f r aa0 n k iy1 n iy0))
("franchino" nil (f r aa0 n k iy1 n ow0))
("franchise" nil (f r ae1 n ch ay1 z))
("franchised" nil (f r ae1 n ch ay0 z d))
("franchisee" nil (f r ae1 n ch ay1 z iy1))
("franchisees" nil (f r ae1 n ch ay0 z iy1 z))
("franchiser" nil (f r ae1 n ch ay1 z er0))
("franchisers" nil (f r ae1 n ch ay1 z er0 z))
("franchises" nil (f r ae1 n ch ay1 z ih0 z))
("franchising" nil (f r ae1 n ch ay0 z ih0 ng))
("franchot" nil (f r ae1 n k ax t))
("francia" nil (f r aa1 n ch ax))
("francie" nil (f r ae1 ng k iy0))
("francies" nil (f r ax n s iy1 z))
("francine" nil (f r ae0 n s iy1 n))
("francines" nil (f r ae0 n s iy1 n z))
("francis" nil (f r ae1 n s ax s))
("franciscan" nil (f r ae0 n s ih1 s k ax n))
("franciscans" nil (f r ae0 n s ih1 s k ax n z))
("francisco" nil (f r ae0 n s ih1 s k ow0))
("franciso" nil (f r ae0 n s iy1 s ow0))
("francisville" nil (f r ae1 n s ih0 s v ih1 l))
("franck" nil (f r ae1 ng k))
("francke" nil (f r ae1 ng k))
("francklin" nil (f r ae1 ng k l ih0 n))
("francklyn" nil (f r ae1 ng k l ih0 n))
("franckowiak" nil (f r ax n s k aw1 iy0 ae0 k))
("franco" nil (f r ae1 ng k ow0))
("francoeur" nil (f r ax n k er1))
("francois" nil (f r aa0 n s w aa1))
("francoise" nil (f r ae0 n s w aa1 z))
("francom" nil (f r ae1 ng k ax m))
("francophile" nil (f r ae1 ng k ax f ay1 l))
("francorp" nil (f r ae1 n k ao1 r p))
("francs" nil (f r ae1 ng k s))
("francy" nil (f r ae1 n s iy0))
("francyne" nil (f r ae1 n s ay1 n))
("franczak" nil (f r ae1 n ch ae0 k))
("frandsen" nil (f r ae1 n d s ax n))
("frane" nil (f r ey1 n))
("franek" nil (f r ae1 n ih0 k))
("franey" nil (f r ey1 n iy0))
("frangos" nil (f r ae1 ng g ow0 z))
("frank" nil (f r ae1 ng k))
("franke" nil (f r ae1 ng k))
("franked" nil (f r ae1 ng k t))
("frankel" nil (f r ae1 ng k ax l))
("franken" nil (f r ae1 ng k ax n))
("frankenberg" nil (f r ae1 ng k ax n b er0 g))
("frankenberger" nil (f r ae1 ng k ax n b er0 g er0))
("frankenberry" nil (f r ae1 ng k ax n b eh1 r iy0))
("frankenfield" nil (f r ae1 ng k ax n f iy1 l d))
("frankenstein" nil (f r ae1 ng k ax n s t ay1 n))
("frankfort" nil (f r ae1 ng k f er0 t))
("frankfurt" nil (f r ae1 ng k f er0 t))
("frankfurter" nil (f r ae1 ng k f er0 t er0))
("frankfurters" nil (f r ae1 ng k f er0 t er0 z))
("frankhouser" nil (f r ae1 ng k hh aw1 s er0))
("frankie" nil (f r ae1 ng k iy0))
("frankiewicz" nil (f r ae1 n k ax v ih0 ch))
("frankincense" nil (f r ae1 ng k ax n s eh1 n s))
("franking" nil (f r ae1 ng k ih0 ng))
("frankino" nil (f r ae0 ng k iy1 n ow0))
("frankish" nil (f r ae1 ng k ih0 sh))
("frankl" nil (f r ae1 ng k ax l))
("frankland" nil (f r ae1 ng k l ax n d))
("franklin" nil (f r ae1 ng k l ih0 n))
("franklinite" nil (f r ae1 ng k l ih0 n ay1 t))
("frankly" nil (f r ae1 ng k l iy0))
("franklyn" nil (f r ae1 ng k l ih0 n))
("frankness" nil (f r ae1 ng k n ax s))
("franko" nil (f r ae1 ng k ow0))
("frankovich" nil (f r ae1 ng k ax v ih0 ch))
("frankowski" nil (f r ax ng k ao1 f s k iy0))
("franks" nil (f r ae1 ng k s))
("frankson" nil (f r ae1 ng k s ax n))
("frankum" nil (f r ae1 ng k ax m))
("frannie" nil (f r ae1 n iy0))
("franny" nil (f r ae1 n iy0))
("frans" nil (f r ae1 n z))
("fransen" nil (f r ae1 n s ax n))
("franson" nil (f r ae1 n s ax n))
("franssen" nil (f r ae1 n s ax n))
("franta" nil (f r ae1 n t ax))
("frantic" nil (f r ae1 n t ih0 k))
("frantically" nil (f r ae1 n t ax k ax l iy0))
("frantom" nil (f r ae1 n t ax m))
("frantz" nil (f r ae1 n t s))
("frantzen" nil (f r ae1 n t z ax n))
("franyo" nil (f r aa1 n y ow0))
("franz" nil (f r ae1 n z))
("franze" nil (f r ae1 n z))
("franzel" nil (f r ae1 n z ax l))
("franzen" nil (f r ae1 n z ax n))
("franzese" nil (f r aa0 n z ey1 z iy0))
("franzone" nil (f r aa0 n z ow1 n iy0))
("franzoni" nil (f r aa0 n z ow1 n iy0))
("fraph" nil (f r ae1 f))
("frappier" nil (f r ae1 p iy0 er0))
("frary" nil (f r eh1 r iy0))
("frasca" nil (f r aa1 s k ax))
("frascella" nil (f r aa0 s ch eh1 l ax))
("frasch" nil (f r ae1 sh))
("frasco" nil (f r aa1 s k ow0))
("frase" nil (f r ey1 z))
("fraser" nil (f r ey1 z er0))
("frasher" nil (f r ae1 sh er0))
("frashier" nil (f r ey1 zh y er0))
("frasier" nil (f r ey1 s iy0 er0))
("frasure" nil (f r aa1 zh er0))
("fratangelo" nil (f r aa0 t aa0 ng g eh1 l ow0))
("frate" nil (f r ey1 t))
("frater" nil (f r ey1 t er0))
("fraternal" nil (f r ax t er1 n ax l))
("fraternities" nil (f r ax t er1 n ax t iy0 z))
("fraternity" nil (f r ax t er1 n ax t iy0))
("fraternizing" nil (f r ae1 t er0 n ay1 z ih0 ng))
("frates" nil (f r ey1 t s))
("fratus" nil (f r ae1 t ih0 s))
("fratzke" nil (f r ae1 t s k))
("frau" nil (f r aw1))
("fraud" nil (f r ao1 d))
("frauds" nil (f r ao1 d z))
("fraudster" nil (f r ao1 d s t er0))
("fraudsters" nil (f r ao1 d s t er0 z))
("fraudulent" nil (f r ao1 jh ax l ax n t))
("fraudulently" nil (f r ao1 d uw0 l ax n t l iy0))
("fraught" nil (f r ao1 t))
("frausto" nil (f r ao1 s t ow0))
("fravel" nil (f r ae1 v ax l))
("frawley" nil (f r ao1 l iy0))
("fray" nil (f r ey1))
("frayed" nil (f r ey1 d))
("frayer" nil (f r ey1 er0))
("fraying" nil (f r ey1 ih0 ng))
("frayn" nil (f r ey1 n))
("frayne" nil (f r ey1 n))
("fraze" nil (f r ey1 z))
("frazee" nil (f r ax z iy1))
("frazer" nil (f r ey1 z er0))
("frazier" nil (f r ey1 zh er0))
("frazzano" nil (f r ax z aa1 n ow0))
("frazzini" nil (f r aa0 t s iy1 n iy0))
("frazzled" nil (f r ae1 z ax l d))
("fread" nil (f r eh1 d))
("freak" nil (f r iy1 k))
("freaked" nil (f r iy1 k t))
("freaking" nil (f r iy1 k ih0 ng))
("freaks" nil (f r iy1 k s))
("frean" nil (f r iy1 n))
("frear" nil (f r ih1 r))
("frears" nil (f r iy1 r z))
("freas" nil (f r iy1 z))
("frease" nil (f r iy1 z))
("freberg" nil (f r iy1 b er0 g))
("frech" nil (f r eh1 k))
("frechette" nil (f r ih0 sh eh1 t))
("freck" nil (f r eh1 k))
("freckled" nil (f r eh1 k ax l d))
("fred" nil (f r eh1 d))
("freda" nil (f r eh1 d))
("freddie" nil (f r eh1 d iy0))
("freddy" nil (f r eh1 d iy0))
("frede" nil (f r iy1 d))
("fredell" nil (f r eh1 d ax l))
("fredella" nil (f r eh0 d eh1 l ax))
("fredenburg" nil (f r iy1 d ax n b er0 g))
("frederic" nil (f r eh1 d r ih0 k))
("frederica" nil (f r eh1 d er0 iy1 k ax))
("frederick" nil (f r eh1 d r ih0 k))
("fredericka" nil (f r ey0 d eh0 r iy1 k ax))
("fredericks" nil (f r eh1 d r ih0 k s))
("fredericksburg" nil (f r eh1 d r ih0 k s b er0 g))
("fredericksen" nil (f r eh1 d er0 ih0 k s ax n))
("frederickson" nil (f r eh1 d er0 ih0 k s ax n))
("frederico" nil (f r eh0 d er0 iy1 k ow0))
("frederika" nil (f r ey0 d eh0 r iy1 k ax))
("frederiksen" nil (f r eh1 d er0 ih0 k s ax n))
("frederking" nil (f r eh1 d er0 k ih1 ng))
("fredette" nil (f r ih0 d eh1 t))
("frediani" nil (f r eh0 d iy0 aa1 n iy0))
("fredin" nil (f r eh1 d ih0 n))
("fredkin" nil (f r eh1 d k ih0 n))
("fredlund" nil (f r eh1 d l ax n d))
("fredman" nil (f r eh1 d m ax n))
("fredo" nil (f r iy1 d ow0))
("fredric" nil (f r eh1 d r ih0 k))
("fredrich" nil (f r eh1 d r ih0 k))
("fredrick" nil (f r eh1 d r ih0 k))
("fredricks" nil (f r eh1 d r ih0 k s))
("fredricksen" nil (f r eh1 d r ih0 k s ax n))
("fredrickson" nil (f r eh1 d r ih0 k s ax n))
("fredriksen" nil (f r ih0 d r ih1 k s ax n))
("fredrikson" nil (f r eh1 d r ih0 k s ax n))
("free" nil (f r iy1))
("freeberg" nil (f r iy1 b er0 g))
("freebie" nil (f r iy1 b iy0))
("freebies" nil (f r iy1 b iy0 z))
("freeborn" nil (f r iy1 b er0 n))
("freeburg" nil (f r iy1 b er0 g))
("freeburn" nil (f r iy1 b er1 n))
("freed" nil (f r iy1 d))
("freedenberg" nil (f r iy1 d eh1 n b er0 g))
("freedland" nil (f r iy1 d l ax n d))
("freedle" nil (f r iy1 d ax l))
("freedman" nil (f r iy1 d m ax n))
("freedom" nil (f r iy1 d ax m))
("freedoms" nil (f r iy1 d ax m z))
("freefall" nil (f r iy1 f ao1 l))
("freeform" nil (f r iy1 f ao1 r m))
("freeh" nil (f r iy1))
("freehand" nil (f r iy1 hh ae1 n d))
("freehling" nil (f r iy1 l ih0 ng))
("freehold" nil (f r iy1 hh ow1 l d))
("freeholders" nil (f r iy1 hh ow1 l d er0 z))
("freeing" nil (f r iy1 ih0 ng))
("freel" nil (f r iy1 l))
("freelance" nil (f r iy1 l ae1 n s))
("freeland" nil (f r iy1 l ax n d))
("freeloaders" nil (f r iy1 l ow1 d er0 z))
("freelove" nil (f r iy1 l ah1 v))
("freels" nil (f r iy1 l z))
("freely" nil (f r iy1 l iy0))
("freeman" nil (f r iy1 m ax n))
("freemans" nil (f r iy1 m ax n z))
("freemason" nil (f r iy1 m ey1 s ax n))
("freemasonry" nil (f r iy1 m ey1 s ax n r iy0))
("freemon" nil (f r iy1 m ax n))
("freemont" nil (f r iy1 m aa1 n t))
("freemyer" nil (f r iy1 m iy0 er0))
("freen" nil (f r iy1 n))
("freeney" nil (f r iy1 n iy0))
("freeny" nil (f r iy1 n iy0))
("freeport" nil (f r iy1 p ao1 r t))
("freer" nil (f r iy1 er0))
("frees" nil (f r iy1 z))
("freese" nil (f r iy1 z))
("freest" nil (f r iy1 ih0 s t))
("freestanding" nil (f r iy1 s t ae1 n d ih0 ng))
("freestone" nil (f r iy1 s t ow1 n))
("freestyle" nil (f r iy1 s t ay1 l))
("freet" nil (f r iy1 t))
("freethinkers" nil (f r iy1 th ih1 ng k er0 z))
("freeway" nil (f r iy1 w ey1))
("freeways" nil (f r iy1 w ey1 z))
("freewheeling" nil (f r iy1 w iy1 l ih0 ng))
("freeze" nil (f r iy1 z))
("freezer" nil (f r iy1 z er0))
("freezers" nil (f r iy1 z er0 z))
("freezes" nil (f r iy1 z ih0 z))
("freezing" nil (f r iy1 z ih0 ng))
("fregeau" nil (f r ih0 zh ow1))
("fregia" nil (f r iy1 jh iy0 ax))
("fregoso" nil (f r eh0 g ow1 s ow0))
("frei" nil (f r ay1))
("freia" nil (f r iy1 ax))
("freiberg" nil (f r ay1 b er0 g))
("freiberger" nil (f r ay1 b er0 g er0))
("freiburger" nil (f r ay1 b er0 g er0))
("freid" nil (f r iy1 d))
("freidel" nil (f r ay1 d ax l))
("freidman" nil (f r ay1 d m ax n))
("freier" nil (f r ey1 er0))
("freiermuth" nil (f r ay1 er0 m uw0 th))
("freight" nil (f r ey1 t))
("freighter" nil (f r ey1 t er0))
("freighters" nil (f r ey1 t er0 z))
("freightliner" nil (f r ey1 t l ay1 n er0))
("freightways" nil (f r ey1 t w ey1 z))
("freiheit" nil (f r ay1 hh ay1 t))
("freije" nil (f r iy1 jh))
("freilich" nil (f r ay1 l ih0 k))
("freiman" nil (f r ay1 m ax n))
("freimark" nil (f r ay1 m aa0 r k))
("freimuth" nil (f r ay1 m uw0 th))
("frein" nil (f r ey1 n))
("freind" nil (f r ay1 n d))
("freire" nil (f r ih1 r))
("freis" nil (f r iy1 z))
("freise" nil (f r iy1 s))
("freitag" nil (f r ay1 t ax g))
("freitas" nil (f r ay1 t ax z))
("freiwald" nil (f r ay1 w ao1 l d))
("frel" nil (f r eh1 l))
("frelimo" nil (f r eh1 l ih0 m ow1))
("frels" nil (f r eh1 l z))
("fremd" nil (f r eh1 m d))
("fremin" nil (f r eh1 m ih0 n))
("fremont" nil (f r iy1 m aa0 n t))
("french" nil (f r eh1 n ch))
("frenchman" nil (f r eh1 n ch m ax n))
("frenchmen" nil (f r eh1 n ch m ax n))
("frenetic" nil (f r ax n eh1 t ih0 k))
("frenette" nil (f r ih0 n eh1 t))
("freni" nil (f r eh1 n iy0))
("frenkel" nil (f r eh1 ng k ax l))
("frentz" nil (f r eh1 n t s))
("frenz" nil (f r eh1 n z))
("frenzel" nil (f r eh1 n z ax l))
("frenzied" nil (f r eh1 n z iy0 d))
("frenzy" nil (f r eh1 n z iy0))
("freon" nil (f r iy1 aa0 n))
("freons" nil (f r iy1 aa0 n z))
("frequencies" nil (f r iy1 k w ax n s iy0 z))
("frequency" nil (f r iy1 k w ax n s iy0))
("frequent" j (f r iy1 k w ax n t))
("frequent" v (f r ih0 k w eh1 n t))
("frequented" nil (f r iy1 k w ax n t ih0 d))
("frequently" nil (f r iy1 k w ax n t l iy0))
("frequents" nil (f r iy1 k w ax n t s))
("frere" nil (f r ih1 r))
("freres" nil (f r iy1 r z))
("frerichs" nil (f r eh1 r ih0 k s))
("frericks" nil (f r eh1 r ih0 k s))
("frerika" nil (f r eh1 r ih0 k ax))
("frerking" nil (f r er1 k ih0 ng))
("fresco" nil (f r eh1 s k ow0))
("frescoed" nil (f r eh1 s k ow0 d))
("frescoes" nil (f r eh1 s k ow0 z))
("frese" nil (f r iy1 z))
("fresenius" nil (f r ax s iy1 n iy0 ax s))
("fresh" nil (f r eh1 sh))
("freshbake" nil (f r eh1 sh b ey1 k))
("freshen" nil (f r eh1 sh ax n))
("freshening" nil (f r eh1 sh ax n ih0 ng))
("fresher" nil (f r eh1 sh er0))
("freshest" nil (f r eh1 sh ax s t))
("freshley" nil (f r eh1 sh l iy0))
("freshly" nil (f r eh1 sh l iy0))
("freshman" nil (f r eh1 sh m ax n))
("freshmen" nil (f r eh1 sh m ih0 n))
("freshness" nil (f r eh1 sh n ax s))
("freshour" nil (f r eh1 s aw0 r))
("freshwater" nil (f r eh1 sh w ao1 t er0))
("fresno" nil (f r eh1 z n ow0))
("fresquez" nil (f r ey0 s k w eh1 z))
("freston" nil (f r eh1 s t ax n))
("fret" nil (f r eh1 t))
("fretful" nil (f r eh1 t f ax l))
("frets" nil (f r eh1 t s))
("frett" nil (f r eh1 t))
("fretted" nil (f r eh1 t ih0 d))
("fretter" nil (f r eh1 t er0))
("fretting" nil (f r eh1 t ih0 ng))
("fretwell" nil (f r eh1 t w eh1 l))
("fretz" nil (f r eh1 t s))
("freud" nil (f r oy1 d))
("freudenberg" nil (f r oy1 d ax n b er0 g))
("freudenthal" nil (f r oy1 d ih0 n th ax l))
("freudian" nil (f r uw1 d iy0 ax n))
("freund" nil (f r uw1 n d))
("freundlich" nil (f r oy1 n d l ih0 k))
("frevert" nil (f r eh1 v er0 t))
("frew" nil (f r uw1))
("frewen" nil (f r uw1 ax n))
("frewin" nil (f r uw1 ih0 n))
("frey" nil (f r ey1))
("freya" nil (f r ey1 ax))
("freyer" nil (f r ey1 er0))
("freyermuth" nil (f er1 ay0 r m uw0 th))
("freyman" nil (f r ey1 m ax n))
("freymiller" nil (f r ey1 m ih1 l er0))
("freyne" nil (f r ey1 n))
("freyre" nil (f r eh1 r))
("freytag" nil (f r ey1 t ax g))
("frezza" nil (f r eh1 z ax))
("friar" nil (f r ay1 er0))
("friary" nil (f r ay1 er0 iy0))
("frias" nil (f r iy1 ax z))
("friberg" nil (f r ay1 b er0 g))
("fribourg" nil (f r ay1 b ao1 r g))
("fricano" nil (f r iy0 k aa1 n ow0))
("frick" nil (f r ih1 k))
("fricke" nil (f r ih1 k))
("fricker" nil (f r ih1 k er0))
("frickey" nil (f r ih1 k iy0))
("fricks" nil (f r ih1 k s))
("friction" nil (f r ih1 k sh ax n))
("frictionless" nil (f r ih1 k sh ax n l ax s))
("frictions" nil (f r ih1 k sh ax n z))
("frida" nil (f r iy1 d ax))
("friday" nil (f r ay1 d iy0))
("fridays" nil (f r ay1 d iy0 z))
("friddle" nil (f r ih1 d ax l))
("fridge" nil (f r ih1 jh))
("fridley" nil (f r ih1 d l iy0))
("fridman" nil (f r ih1 d m ax n))
("fridolf" nil (f r ih1 d ow0 l f))
("fridovich" nil (f r ih1 d ax v ih0 ch))
("fridrich" nil (f r ih1 d r ih0 k))
("fridson" nil (f r ih1 d s ax n))
("frie" nil (f er1 iy0))
("friebel" nil (f r iy1 b ax l))
("fried" nil (f r ay1 d))
("frieda" nil (f r iy1 d ax))
("friedberg" nil (f r ay1 d b er0 g))
("friede" nil (f r iy1 d))
("friedel" nil (f r iy1 d ax l))
("frieden" nil (f r iy1 d ax n))
("friedenberg" nil (f r iy1 d ax n b er0 g))
("frieder" nil (f r ay1 d er0))
("friederich" nil (f r iy1 d er0 ih0 k))
("friederike" nil (f r iy1 d er0 ih0 k))
("friedhelm" nil (f r iy1 d hh eh1 l m))
("friedhoff" nil (f r iy1 d hh ao1 f))
("friedl" nil (f r iy1 d ax l))
("friedland" nil (f r iy1 d l ax n d))
("friedlander" nil (f r iy1 d l ax n d er0))
("friedley" nil (f r iy1 d l iy0))
("friedli" nil (f r iy1 d l iy1))
("friedline" nil (f r iy1 d l ay1 n))
("friedly" nil (f r iy1 d l iy0))
("friedman" nil (f r iy1 d m ax n))
("friedmann" nil (f r iy1 d m ax n))
("friedrich" nil (f r iy1 d r ih0 k))
("friedrichs" nil (f r iy1 d r ih0 k s))
("friedrichsen" nil (f r iy1 d r ih0 k s ax n))
("friedrick" nil (f r iy1 d r ih0 k))
("friedsam" nil (f r iy1 d s ae1 m))
("friedson" nil (f r iy1 d s ax n))
("friedt" nil (f r iy1 d t))
("friel" nil (f r iy1 l))
("frieling" nil (f r iy1 l ih0 ng))
("frieman" nil (f r iy1 m ax n))
("friend" nil (f r eh1 n d))
("friendlier" nil (f r eh1 n d l iy0 er0))
("friendliest" nil (f r eh1 n d l iy0 ax s t))
("friendliness" nil (f r eh1 n d l iy0 n ih0 s))
("friendly" nil (f r eh1 n d l iy0))
("friends" nil (f r eh1 n d z))
("friendship" nil (f r eh1 n d sh ih0 p))
("friendships" nil (f r eh1 n d sh ih0 p s))
("frier" nil (f r ay1 er0))
("frierson" nil (f r ih1 r s ax n))
("fries" nil (f r ay1 z))
("friese" nil (f r iy1 z))
("friesen" nil (f r iy1 s ax n))
("friesenhahn" nil (f r iy1 s ih0 n hh ax n))
("friesner" nil (f r iy1 s n er0))
("frieson" nil (f r iy1 z ax n))
("friess" nil (f r iy1 s))
("friesz" nil (f r iy1 sh))
("frieze" nil (f r iy1 z))
("frigate" nil (f r ih1 g ax t))
("frigates" nil (f r ih1 g ax t s))
("fright" nil (f r ay1 t))
("frighten" nil (f r ay1 t ax n))
("frightened" nil (f r ay1 t ax n d))
("frightening" nil (f r ay1 t ax n ih0 ng))
("frighteningly" nil (f r ay1 t ax n ih0 ng l iy0))
("frightens" nil (f r ay1 t ax n z))
("frightful" nil (f r ay1 t f ax l))
("frightfully" nil (f r ay1 t f ax l iy0))
("frigid" nil (f r ih1 jh ax d))
("frigo" nil (f r iy1 g ow0))
("frigon" nil (f r ih1 g ax n))
("friis" nil (f r iy1 z))
("frikkie" nil (f r ih1 k iy0))
("fril" nil (f r ih1 l))
("friley" nil (f r ih1 l iy0))
("frill" nil (f r ih1 l))
("frills" nil (f r ih1 l z))
("frilly" nil (f r ih1 l iy0))
("fringe" nil (f r ih1 n jh))
("fringer" nil (f r ih1 n jh er0))
("fringes" nil (f r ih1 n jh ih0 z))
("frink" nil (f r ih1 ng k))
("friona" nil (f r iy0 ow1 n ax))
("frisbee" nil (f r ih1 s b iy0))
("frisbie" nil (f r ih1 s b iy0))
("frisby" nil (f r ih1 s b iy0))
("frisch" nil (f r ih1 sh))
("friscia" nil (f r iy1 s ch ax))
("frisco" nil (f r iy1 s k ow0))
("frisella" nil (f r ih0 s eh1 l ax))
("frishman" nil (f r ih1 sh m ax n))
("frisina" nil (f r iy0 s iy1 n ax))
("frisinger" nil (f r ih1 s ih0 n jh er0))
("frisk" nil (f r ih1 s k))
("friske" nil (f r ih1 s k))
("frisked" nil (f r ih1 s k t))
("frisky" nil (f r ih1 s k iy0))
("frison" nil (f r ih1 s ax n))
("frist" nil (f r ih1 s t))
("fristoe" nil (f r ih1 s t ow0))
("fritch" nil (f r ih1 ch))
("fritcher" nil (f r ih1 ch er0))
("fritchman" nil (f r ih1 ch m ax n))
("frith" nil (f r ih1 th))
("frito" nil (f r iy1 t ow0))
("frits" nil (f r ih1 t s))
("fritsch" nil (f r ih1 ch))
("fritsche" nil (f r ih1 ch))
("fritter" nil (f r ih1 t er0))
("frittered" nil (f r ih1 t er0 d))
("frittering" nil (f r ih1 t er0 ih0 ng))
("fritts" nil (f r ih1 t s))
("fritz" nil (f r ih1 t s))
("fritze" nil (f r ih1 t z))
("fritzen" nil (f r ih1 t z ax n))
("fritzi" nil (f r iy1 t z iy0))
("fritzie" nil (f r ih1 t z iy0))
("fritzinger" nil (f r ih1 t z ih0 ng er0))
("fritzler" nil (f r ih1 t s l er0))
("fritzsche" nil (f r ih1 t z sh))
("frivolity" nil (f r ax v aa1 l ax t iy0))
("frivolous" nil (f r ih1 v ax l ax s))
("frix" nil (f r ih1 k s))
("frizell" nil (f r ih1 z ax l))
("frizzell" nil (f r ih1 z ax l))
("froberg" nil (f r ow1 b er0 g))
("frock" nil (f r aa1 k))
("frocks" nil (f r aa1 k s))
("froedge" nil (f r ow1 jh))
("froehlich" nil (f r ow1 l ih0 k))
("froelich" nil (f r ow1 l ih0 k))
("froemming" nil (f r ow1 m ih0 ng))
("froese" nil (f r ow1 s))
("frog" nil (f r aa1 g))
("frogg" nil (f r aa1 g))
("frogge" nil (f r aa1 g))
("frogman" nil (f r aa1 g m ae1 n))
("frogmen" nil (f r aa1 g m ih0 n))
("frogmouths" nil (f r aa1 g m aw1 th s))
("frogs" nil (f r aa1 g z))
("froh" nil (f r ow1))
("frohlich" nil (f r ow1 l ih0 k))
("frohman" nil (f r ow1 m ax n))
("frohn" nil (f r aa1 n))
("frohnmayer" nil (f r ao1 n m ay1 er0))
("froio" nil (f r ow1 iy0 ow0))
("frolic" nil (f r aa1 l ih0 k))
("frolicking" nil (f r aa1 l ih0 k ih0 ng))
("from" nil (f r ah1 m))
("froman" nil (f r ow1 m ax n))
("frome" nil (f r ow1 m))
("fromer" nil (f r ow1 m er0))
("fromm" nil (f r aa1 m))
("fromme" nil (f r aa1 m))
("frommelt" nil (f r aa1 m ih0 l t))
("frommer" nil (f r aa1 m er0))
("fromstein" nil (f r aa1 m s t iy1 n))
("fronczak" nil (f r aa1 n ch ae0 k))
("frond" nil (f r aa1 n d))
("fronde" nil (f r aa1 n d))
("fronds" nil (f r aa1 n d z))
("fronek" nil (f r ow1 n ih0 k))
("froning" nil (f r ow1 n ih0 ng))
("fronk" nil (f r aa1 ng k))
("front" nil (f r ah1 n t))
("frontage" nil (f r ah1 n t ih0 jh))
("frontal" nil (f r ah1 n t ax l))
("fronted" nil (f r ah1 n t ih0 d))
("frontera" nil (f r ow0 n t eh1 r ax))
("fronterhouse" nil (f r ah1 n t er0 hh aw1 s))
("frontier" nil (f r ax n t ih1 r))
("frontiere" nil (f r aa0 n t y eh1 r))
("frontiers" nil (f r ax n t ih1 r z))
("fronting" nil (f r ah1 n t ih0 ng))
("frontline" nil (f r ah1 n t l ay1 n))
("frontrunner" nil (f r ah1 n t r ah1 n er0))
("frontrunners" nil (f r ah1 n t r ah1 n er0 z))
("fronts" nil (f r ah1 n t s))
("frontz" nil (f r aa1 n t s))
("frosch" nil (f r ao1 sh))
("fross" nil (f r ao1 s))
("frost" nil (f r ao1 s t))
("frostban" nil (f r ao1 s t b ae1 n))
("frostbite" nil (f r ao1 s t b ay1 t))
("frostbitten" nil (f r ao1 s t b ih1 t ax n))
("frosted" nil (f r ao1 s t ax d))
("frosting" nil (f r ao1 s t ih0 ng))
("frostings" nil (f r ao1 s t ih0 ng z))
("frosts" nil (f r ao1 s t s))
("frosty" nil (f r ao1 s t iy0))
("froth" nil (f r ao1 th))
("frothingham" nil (f r ao1 th ih0 ng hh ae1 m))
("froths" nil (f r ao1 th s))
("frothy" nil (f r ao1 th iy0))
("frown" nil (f r aw1 n))
("frowned" nil (f r aw1 n d))
("frownfelter" nil (f r aw1 n f eh1 l t er0))
("frowning" nil (f r aw1 n ih0 ng))
("frowningly" nil (f r aw1 n ih0 ng l iy0))
("frowns" nil (f r aw1 n z))
("froze" nil (f r ow1 z))
("frozen" nil (f r ow1 z ax n))
("frucher" nil (f r uw1 k er0))
("fruchey" nil (f r ah1 ch iy0))
("fruchter" nil (f r ah1 k t er0))
("fruchtman" nil (f r ah1 k t m ax n))
("fructose" nil (f r ah1 k t ow1 s))
("frueh" nil (f r uw1))
("fruehauf" nil (f r uw1 hh ao1 f))
("fruehling" nil (f r uw1 l ih0 ng))
("frugal" nil (f r uw1 g ax l))
("frugality" nil (f r uw0 g ae1 l ax t iy0))
("frugally" nil (f r uw1 g ax l iy0))
("fruge" nil (f r uw1 jh))
("fruin" nil (f r uw1 ih0 n))
("fruit" nil (f r uw1 t))
("fruitcake" nil (f r uw1 t k ey1 k))
("fruitcakes" nil (f r uw1 t k ey1 k s))
("fruitful" nil (f r uw1 t f ax l))
("fruiting" nil (f r uw1 t ih0 ng))
("fruition" nil (f r uw0 ih1 sh ax n))
("fruitless" nil (f r uw1 t l ax s))
("fruitlessly" nil (f r uw1 t l ax s l iy0))
("fruitopia" nil (f r uw1 t ow1 p iy0 ax))
("fruits" nil (f r uw1 t s))
("fruity" nil (f r uw1 t iy0))
("frum" nil (f r ah1 m))
("frumkin" nil (f r ah1 m k ih0 n))
("frump" nil (f r ah1 m p))
("frumpy" nil (f r ah1 m p iy0))
("frunze" nil (f r ah1 n z))
("frusen" nil (f r uw1 z ax n))
("frush" nil (f r ah1 sh))
("frustrate" nil (f r ah1 s t r ey1 t))
("frustrated" nil (f r ah1 s t r ey1 t ax d))
("frustrates" nil (f r ah1 s t r ey1 t s))
("frustrating" nil (f r ah1 s t r ey1 t ih0 ng))
("frustration" nil (f r ax s t r ey1 sh ax n))
("frustrations" nil (f r ax s t r ey1 sh ax n z))
("frutchey" nil (f r ah1 ch iy0))
("fruth" nil (f r uw1 th))
("frutiger" nil (f r ah1 t ih0 g er0))
("fry" nil (f r ay1))
("fryar" nil (f r ay1 er0))
("fryberger" nil (f r ay1 b er0 g er0))
("frye" nil (f r ay1))
("fryer" nil (f r ay1 er0))
("fryers" nil (f r ay1 er0 z))
("frying" nil (f r ay1 ih0 ng))
("fryling" nil (f r ay1 l ih0 ng))
("fryman" nil (f r ay1 m ax n))
("frymire" nil (f r ih0 m ay1 r))
("fryrear" nil (f r ay1 r ih1 r))
("frysinger" nil (f r ih1 s ih0 n jh er0))
("fs" nil (f s))
("fsi" nil (f s iy1))
("fthenakis" nil (f th eh0 n aa1 k ih0 s))
("fu" nil (f uw1))
("fucci" nil (f uw1 ch iy0))
("fuchs" nil (f y uw1 k s))
("fuchsias" nil (f y uw1 sh ax z))
("fudala" nil (f uw0 d aa1 l ax))
("fuddruckers" nil (f ah1 d r ax k er0 z))
("fuddy" nil (f ah1 d iy0))
("fudge" nil (f ah1 jh))
("fudging" nil (f ah1 jh ih0 ng))
("fudgy" nil (f ah1 jh iy0))
("fudo" nil (f y uw1 d ow0))
("fudosan" nil (f y uw1 d ax s aa0 n))
("fuel" nil (f y uw1 ax l))
("fueled" nil (f y uw1 ax l d))
("fueling" nil (f y uw1 l ih0 ng))
("fuelled" nil (f y uw1 ax l d))
("fuelling" nil (f y uw1 l ih0 ng))
("fuels" nil (f y uw1 ax l z))
("fuente" nil (f w eh1 n t ey1))
("fuentes" nil (f w eh1 n t ey1 z))
("fuentez" nil (f w eh1 n t ey1 z))
("fuer" nil (f y uw1 r))
("fuerst" nil (f w eh1 r s t))
("fuerstenberg" nil (f uh1 r s t ax n b er0 g))
("fugard" nil (f y uw1 g aa1 r d))
("fugate" nil (f ah1 g ey0 t))
("fugazy" nil (f y uw1 g ax z iy0))
("fuge" nil (f y uw1 jh))
("fugere" nil (f y uw1 g er0))
("fugett" nil (f y uw1 jh ih0 t))
("fugitive" nil (f y uw1 jh ax t ih0 v))
("fugitives" nil (f y uw1 jh ih0 t ih0 v z))
("fugitt" nil (f y uw1 jh ih0 t))
("fugue" nil (f y uw1 g))
("fugues" nil (f y uw1 g z))
("fuhr" nil (f uh1 r))
("fuhrer" nil (f uh1 r er0))
("fuhrman" nil (f uh1 r m ax n))
("fuhrmann" nil (f uh1 r m ax n))
("fuhs" nil (f ah1 s))
("fuji" nil (f uw1 jh iy0))
("fujian" nil (f y uw1 jh iy0 ax n))
("fujii" nil (f uw0 jh iy1 iy0))
("fujikawa" nil (f uw1 jh iy0 k aa1 w ax))
("fujikura" nil (f uw1 jh ih0 k uw1 r ax))
("fujimori" nil (f uw1 jh iy0 m ao1 r iy0))
("fujimoto" nil (f uw1 jh iy0 m ow1 t ow0))
("fujioka" nil (f uw1 jh iy0 ow1 k ax))
("fujis" nil (f y uw1 jh ih0 s))
("fujisankei" nil (f uw1 jh ih0 s aa1 ng k ey0))
("fujisawa" nil (f uw1 jh ih0 s aa1 w ax))
("fujita" nil (f uw0 jh iy1 t ax))
("fujitsu" nil (f uw0 jh ih1 t s uw0))
("fujiwara" nil (f uw0 jh iy0 w aa1 r ax))
("fujiya" nil (f uw0 jh iy1 y ax))
("fujiyama" nil (f uw1 jh iy0 y aa1 m ax))
("fuke" nil (f uw1 k))
("fuklah" nil (f uw1 k l ax))
("fukuda" nil (f uw0 k uw1 d ax))
("fukui" nil (f uw0 k uw1 iy0))
("fukumoto" nil (f uw0 k uw0 m ow1 t ow0))
("fukunaga" nil (f uw0 k uw0 n aa1 g ax))
("fukuoka" nil (f uw1 k uw1 ow1 k ax))
("fukushima" nil (f uw0 k uw0 sh iy1 m ax))
("fukuyama" nil (f uw1 k uw1 y aa1 m ax))
("fulani" nil (f uw1 l aa1 n iy0))
("fulbright" nil (f uh1 l b r ay1 t))
("fulcher" nil (f uh1 l ch er0))
("fulco" nil (f uh1 l k ow0))
("fulcrum" nil (f uh1 l k r ax m))
("fuld" nil (f uh1 l d))
("fulenwider" nil (f y uw1 l ih0 n w ay0 d er0))
("fulfer" nil (f uh1 l f er0))
("fulfill" nil (f uh0 l f ih1 l))
("fulfilled" nil (f uh0 l f ih1 l d))
("fulfilling" nil (f uh0 l f ih1 l ih0 ng))
("fulfillment" nil (f uh0 l f ih1 l m ax n t))
("fulfills" nil (f uh0 l f ih1 l z))
("fulford" nil (f uh1 l f er0 d))
("fulgham" nil (f ah1 l g ax m))
("fulghum" nil (f ah1 l g ax m))
("fulginiti" nil (f uh1 l jh ih0 n ih1 t iy0))
("fulk" nil (f ah1 l k))
("fulkerson" nil (f ah1 l k er0 s ax n))
("fulks" nil (f ah1 l k s))
("full" nil (f uh1 l))
("fullam" nil (f uh1 l ax m))
("fullard" nil (f uh1 l er0 d))
("fullback" nil (f uh1 l b ae1 k))
("fullbright" nil (f uh1 l b r ay1 t))
("fullen" nil (f uh1 l ax n))
("fullenkamp" nil (f uh1 l ax n k ae1 m p))
("fullenwider" nil (f uh1 l ax n w ay1 d er0))
("fuller" nil (f uh1 l er0))
("fullerton" nil (f uh1 l er0 t ax n))
("fullest" nil (f uh1 l ax s t))
("fullilove" nil (f uw0 l iy0 l ow1 v iy0))
("fullington" nil (f uh1 l ih0 ng t ax n))
("fullman" nil (f uh1 l m ax n))
("fullmer" nil (f uh1 l m er0))
("fullness" nil (f uh1 l n ax s))
("fulltime" nil (f uh1 l t ay1 m))
("fullwood" nil (f uh1 l w uh1 d))
("fully" nil (f uh1 l iy0))
("fulmer" nil (f ah1 l m er0))
("fulminate" nil (f uh1 l m ax n ey1 t))
("fulmore" nil (f uh1 l m ao0 r))
("fulop" nil (f uw1 l ax p))
("fulp" nil (f uh1 l p))
("fulsome" nil (f uh1 l s ax m))
("fulton" nil (f uh1 l t ax n))
("fults" nil (f uh1 l t s))
("fultz" nil (f uh1 l t s))
("fulvia" nil (f uh1 l v iy0 ax))
("fulwider" nil (f ah1 l w ay0 d er0))
("fulwiler" nil (f ah1 l w ay0 l er0))
("fulwood" nil (f ax l w uh1 d))
("fumaroles" nil (f y uw1 m er0 ow1 l z))
("fumble" nil (f ah1 m b ax l))
("fumbled" nil (f ah1 m b ax l d))
("fumbles" nil (f ah1 m b ax l z))
("fumbling" nil (f ah1 m b ax l ih0 ng))
("fume" nil (f y uw1 m))
("fumed" nil (f y uw1 m d))
("fumes" nil (f y uw1 m z))
("fumigate" nil (f y uw1 m ax g ey1 t))
("fumigation" nil (f y uw1 m ax g ey1 sh ax n))
("fuming" nil (f y uw1 m ih0 ng))
("fumio" nil (f y uw1 m iy0 ow0))
("fun" nil (f ah1 n))
("funari" nil (f uw0 n aa1 r iy0))
("funaro" nil (f uw0 n aa1 r ow0))
("funches" nil (f ah1 n ch ih0 z))
("funchess" nil (f ah1 n k ih0 s))
("funck" nil (f ah1 ng k))
("function" nil (f ah1 ng k sh ax n))
("functional" nil (f ah1 ng k sh ax n ax l))
("functionality" nil (f ah1 ng k sh ax n ae1 l ih0 t iy0))
("functionally" nil (f ah1 ng k sh ax n ax l iy0))
("functionaries" nil (f ah1 ng k sh ax n eh1 r iy0 z))
("functionary" nil (f ah1 ng k sh ax n eh1 r iy0))
("functioned" nil (f ah1 ng k sh ax n d))
("functioning" nil (f ah1 ng k sh ax n ih0 ng))
("functions" nil (f ah1 ng k sh ax n z))
("fund" nil (f ah1 n d))
("fundamental" nil (f ah1 n d ax m eh1 n t ax l))
("fundamentalism" nil (f ah1 n d ax m eh1 n t ax l ih1 z ax m))
("fundamentalist" nil (f ah1 n d ax m eh1 n t ax l ih0 s t))
("fundamentalists" nil (f ah1 n d ax m eh1 n t ax l ih0 s t s))
("fundamentally" nil (f ah1 n d ax m eh1 n t ax l iy0))
("fundamentals" nil (f ah1 n d ax m eh1 n t ax l z))
("funded" nil (f ah1 n d ax d))
("funderburg" nil (f ah1 n d er0 b er0 g))
("funderburk" nil (f ah1 n d er0 b er0 k))
("funderburke" nil (f ah1 n d er0 b er1 k))
("funders" nil (f ah1 n d er0 z))
("funding" nil (f ah1 n d ih0 ng))
("fundora" nil (f uw0 n d ao1 r ax))
("fundraiser" nil (f ah1 n d r ey1 z er0))
("fundraisers" nil (f ah1 n d r ey1 z er0 z))
("fundraising" nil (f ah1 n d r ey1 s ih0 ng))
("funds" nil (f ah1 n d z))
("funeral" nil (f y uw1 n er0 ax l))
("funerals" nil (f y uw1 n er0 ax l z))
("funerary" nil (f y uw1 n er0 eh1 r iy0))
("funes" nil (f y uw1 n z))
("fung" nil (f ah1 ng))
("fungal" nil (f ah1 ng g ax l))
("fungi" nil (f ah1 n jh ay0))
("fungible" nil (f ah1 n jh ih0 b ax l))
("fungicide" nil (f ah1 n jh ax s ay1 d))
("fungicides" nil (f ah1 n jh ax s ay1 d z))
("fungus" nil (f ah1 ng g ax s))
("funk" nil (f ah1 ng k))
("funke" nil (f ah1 ng k))
("funkhouser" nil (f ah1 ng k hh aw1 s er0))
("funks" nil (f ah1 ng k s))
("funky" nil (f ah1 ng k iy0))
("funnel" nil (f ah1 n ax l))
("funneled" nil (f ah1 n ax l d))
("funneling" nil (f ah1 n ax l ih0 ng))
("funnell" nil (f ah1 n ax l))
("funnels" nil (f ah1 n ax l z))
("funnier" nil (f ah1 n iy0 er0))
("funniest" nil (f ah1 n iy0 ax s t))
("funniness" nil (f ah1 n iy0 n ax s))
("funny" nil (f ah1 n iy0))
("funs" nil (f ah1 n z))
("funston" nil (f ah1 n s t ax n))
("funtime" nil (f ah1 n t ay1 m))
("fuoco" nil (f uw0 ow1 k ow0))
("fuoss" nil (f uw1 s))
("fuqua" nil (f uw1 k w ax))
("fuquay" nil (f uw1 k ey0))
("fur" nil (f er1))
("furash" nil (f y er0 ae1 sh))
("furbee" nil (f er1 b iy1))
("furber" nil (f er1 b er0))
("furbishing" nil (f er1 b ih0 sh ih0 ng))
("furbush" nil (f er1 b uh1 sh))
("furby" nil (f er1 b iy0))
("furches" nil (f er0 sh iy1 z))
("furer" nil (f y uh1 r er0))
("furey" nil (f y uh1 r iy0))
("furfaro" nil (f uh0 r f aa1 r ow0))
("furgason" nil (f er1 g ax s ax n))
("furgerson" nil (f er1 g er0 s ax n))
("furgeson" nil (f er1 g ih0 s ax n))
("furia" nil (f y uh1 r iy0 ax))
("furini" nil (f uh0 r iy1 n iy0))
("furino" nil (f uh0 r iy1 n ow0))
("furious" nil (f y uh1 r iy0 ax s))
("furiouser" nil (f y uh1 r iy0 ax s er0))
("furiously" nil (f y uh1 r iy0 ax s l iy0))
("furlan" nil (f er1 l ax n))
("furlaud" nil (f er0 l ow1))
("furlett" nil (f er0 l eh1 t))
("furlong" nil (f er1 l ao1 ng))
("furlough" nil (f er1 l ow0))
("furloughed" nil (f er1 l ow0 d))
("furloughs" nil (f er1 l ow0 z))
("furlow" nil (f er1 l ow1))
("furman" nil (f er1 m ae1 n))
("furmark" nil (f er1 m aa1 r k))
("furnace" nil (f er1 n ax s))
("furnaces" nil (f er1 n ax s ax z))
("furnari" nil (f uh0 r n aa1 r iy0))
("furnas" nil (f er1 n ax s))
("furner" nil (f er1 n er0))
("furness" nil (f er1 n ih0 s))
("furney" nil (f er1 n iy0))
("furnish" nil (f er1 n ih0 sh))
("furnished" nil (f er1 n ih0 sh t))
("furnishes" nil (f er1 n ih0 sh ax z))
("furnishing" nil (f er1 n ih0 sh ih0 ng))
("furnishings" nil (f er1 n ih0 sh ih0 ng z))
("furniss" nil (f er0 n ih1 s))
("furniture" nil (f er1 n ih0 ch er0))
("furno" nil (f uh1 r n ow0))
("furor" nil (f y uh1 r ao0 r))
("furr" nil (f er1))
("furrer" nil (f er1 er0))
("furrh" nil (f er1))
("furrier" nil (f er1 iy0 er0))
("furriers" nil (f er1 iy0 er0 z))
("furrow" nil (f er1 ow0))
("furry" nil (f er1 iy0))
("furs" nil (f er1 z))
("furse" nil (f er1 s))
("furst" nil (f er1 s t))
("furstenberg" nil (f er1 s ax n b er0 g))
("furtado" nil (f uh0 r t aa1 d ow0))
("furtak" nil (f er1 t ax k))
("furtaw" nil (f er1 t ao0))
("furth" nil (f er1 th))
("further" nil (f er1 dh er0))
("furtherance" nil (f er1 th er0 ax n s))
("furthered" nil (f er1 dh er0 d))
("furthering" nil (f er1 dh er0 ih0 ng))
("furthermore" nil (f er1 dh er0 m ao1 r))
("furthers" nil (f er1 dh er0 z))
("furthest" nil (f er1 th ax s t))
("furtick" nil (f er1 t ih0 k))
("furtive" nil (f er1 t ih0 v))
("furtively" nil (f er1 t ih0 v l iy0))
("furukawa" nil (f uh0 r uw0 k aa1 w ax))
("furuta" nil (f er0 uw1 t ax))
("furuya" nil (f uh0 r uw1 y ax))
("fury" nil (f y uh1 r iy0))
("fusaro" nil (f uw0 s aa1 r ow0))
("fusco" nil (f uw1 s k ow0))
("fuse" nil (f y uw1 z))
("fused" nil (f y uw1 z d))
("fuselage" nil (f y uw1 s ax l aa1 jh))
("fuselages" nil (f y uw1 s ax l ey1 jh ih0 z))
("fuselier" nil (f y uw1 s l iy0 er0))
("fuses" nil (f y uw1 z ax z))
("fusible" nil (f y uw1 z ax b ax l))
("fusillade" nil (f y uw1 s ih0 l ey1 d))
("fusilli" nil (f y uw0 s ih1 l iy0))
("fusing" nil (f y uw1 z ih0 ng))
("fusion" nil (f y uw1 zh ax n))
("fuson" nil (f uw1 s ax n))
("fuss" nil (f ah1 s))
("fussed" nil (f ah1 s t))
("fussell" nil (f ah1 s ax l))
("fusselman" nil (f ah1 s ax l m ax n))
("fusses" nil (f ah1 s ih0 z))
("fussing" nil (f ah1 s ih0 ng))
("fussner" nil (f ah1 s n er0))
("fussy" nil (f ah1 s iy0))
("fust" nil (f ah1 s t))
("fustat" nil (f ah1 s t ae0 t))
("fustok" nil (f ah1 s t aa0 k))
("fuston" nil (f ah1 s t ax n))
("futch" nil (f ah1 ch))
("futher" nil (f ah1 dh er0))
("futile" nil (f y uw1 t ax l))
("futility" nil (f y uw0 t ih1 l ax t iy0))
("futral" nil (f ah1 t r ax l))
("futrell" nil (f y uw0 t r eh1 l))
("futterman" nil (f ah1 t er0 m ax n))
("future" nil (f y uw1 ch er0))
("futures" nil (f y uw1 ch er0 z))
("futurism" nil (f y uw1 ch er0 ih1 z ax m))
("futurist" nil (f y uw1 ch er0 ih0 s t))
("futuristic" nil (f y uw1 ch er0 ih1 s t ih0 k))
("futurists" nil (f y uw1 ch er0 ih0 s t s))
("fuzes" nil (f y uw1 z ih0 z))
("fuzz" nil (f ah1 z))
("fuzzier" nil (f ah1 z iy0 er0))
("fuzzy" nil (f ah1 z iy0))
("fye" nil (f ay1))
("fyfe" nil (f ay1 f))
("fyffe" nil (f ay1 f))
("fyffes" nil (f ih1 f s))
("fyke" nil (f ay1 k))
("fyock" nil (f y aa1 k))
("fyodor" nil (f y ow1 d er0))
("fyodorov" nil (f y ow1 d er0 ao1 v))
("g" nil (jh iy1))
("ga" nil (g aa1))
("gaal" nil (g aa1 l))
("gaar" nil (g aa1 r))
("gaarder" nil (g aa1 r d er0))
("gab" nil (g ae1 b))
("gaba" nil (g aa1 b ax))
("gabaldon" nil (g aa0 b aa0 l d ao1 n))
("gabardine" nil (g ae1 b er0 d iy1 n))
("gabardines" nil (g ae1 b er0 d iy1 n z))
("gabay" nil (g ae1 b ey0))
("gabbard" nil (g ax b aa1 r d))
("gabbert" nil (g ae1 b er0 t))
("gabbing" nil (g ae1 b ih0 ng))
("gabbro" nil (g ae1 b r ow0))
("gabbroic" nil (g ae0 b r ow1 ih0 k))
("gabby" nil (g ae1 b iy0))
("gabe" nil (g ey1 b))
("gabehart" nil (g ey1 b hh aa1 r t))
("gabel" nil (g ax b eh1 l))
("gabele" nil (g ax b eh1 l ey0))
("gabelli" nil (g ax b eh1 l iy0))
("gaber" nil (g ey1 b er0))
("gabert" nil (g ae1 b er0 t))
("gabhart" nil (g ae1 b hh aa1 r t))
("gabie" nil (g ae1 b iy0))
("gable" nil (g ey1 b ax l))
("gabler" nil (g ey1 b ax l er0))
("gables" nil (g ey1 b ax l z))
("gabon" nil (g ax b aa1 n))
("gabor" nil (g ax b ao1 r))
("gaboriault" nil (g ae1 b er0 iy0 ow0))
("gaboury" nil (g ae1 b uh0 r iy0))
("gabrall" nil (g ey1 b r ax l))
("gabriel" nil (g ey1 b r iy0 ax l))
("gabriela" nil (g aa0 b r iy0 eh1 l ax))
("gabriele" nil (g aa0 b r iy0 eh1 l))
("gabriella" nil (g aa0 b r iy0 eh1 l ax))
("gabrielle" nil (g ae1 b r iy0 eh0 l))
("gabrielli" nil (g aa0 b r iy0 eh1 l iy0))
("gabrielsen" nil (g ae1 b r iy0 l s ax n))
("gabrielson" nil (g ae1 b r iy0 l s ax n))
("gabrys" nil (g ae1 b er0 iy0 z))
("gaby" nil (g ae1 b iy0))
("gaccione" nil (g aa0 k ch ow1 n iy0))
("gacek" nil (g aa1 ch eh1 k))
("gach" nil (g ae1 ch))
("gacy" nil (g ey1 s iy0))
("gad" nil (g ae1 d))
("gadberry" nil (g ae1 d b eh1 r iy0))
("gadbois" nil (g ae1 d b w aa1))
("gadd" nil (g ae1 d))
("gaddie" nil (g ae1 d iy0))
("gadding" nil (g ae1 d ih0 ng))
("gaddis" nil (g ae1 d ih0 s))
("gaddum" nil (g ae1 d ax m))
("gaddy" nil (g ae1 d iy0))
("gade" nil (g ey1 d))
("gadflies" nil (g ae1 d f l ay1 z))
("gadfly" nil (g ae1 d f l ay1))
("gadget" nil (g ae1 jh ax t))
("gadgetry" nil (g ae1 jh ax t r iy0))
("gadgets" nil (g ae1 jh ax t s))
("gadhafi" nil (g ax d aa1 f iy0))
("gadomski" nil (g ax d aa1 m s k iy0))
("gadoury" nil (g ae1 d uh0 r iy0))
("gads" nil (g ae1 d z))
("gadsby" nil (g ae1 d z b iy0))
("gadsden" nil (g ae1 d z d ax n))
("gadson" nil (g ae1 d s ax n))
("gadway" nil (g ae1 d w ey1))
("gadzinski" nil (g ax jh ih1 n s k iy0))
("gae" nil (g ay1))
("gaea" nil (g iy1 ax))
("gaebel" nil (g eh1 b ax l))
("gaede" nil (g iy1 d))
("gael" nil (g ey1 l))
("gaelic" nil (g ey1 l ih0 k))
("gaertner" nil (g eh1 r t n er0))
("gaeta" nil (g iy1 t ax))
("gaetano" nil (g ay0 t aa1 n ow0))
("gaeth" nil (g iy1 th))
("gaetz" nil (g iy1 t s))
("gaf" nil (g ae1 f))
("gaff" nil (g ae1 f))
("gaffe" nil (g ae1 f))
("gaffes" nil (g ae1 f s))
("gaffey" nil (g ae1 f iy0))
("gaffin" nil (g ae1 f ih0 n))
("gaffney" nil (g ae1 f n iy0))
("gafford" nil (g ae1 f er0 d))
("gag" nil (g ae1 g))
("gagan" nil (g ey1 g ax n))
("gage" nil (g ey1 jh))
("gagel" nil (g ae1 g ax l))
("gagen" nil (g ae1 g ax n))
("gager" nil (g ey1 g er0))
("gagged" nil (g ae1 g d))
("gaggle" nil (g ae1 g ax l))
("gagliano" nil (g aa0 g l iy0 aa1 n ow0))
("gagliardi" nil (g aa0 g l iy0 aa1 r d iy0))
("gagliardo" nil (g aa0 g l iy0 aa1 r d ow0))
("gaglio" nil (g ae1 g l iy0 ow0))
("gaglione" nil (g aa0 g l iy0 ow1 n iy0))
("gagne" nil (g ey1 n iy0))
("gagner" nil (g ae1 g n er0))
("gagnier" nil (g ae1 g n iy0 er0))
("gagnon" nil (g ae1 n y ax n))
("gags" nil (g ae1 g z))
("gahagan" nil (g aa0 hh aa1 g aa0 n))
("gahan" nil (g ae1 hh ax n))
("gahli" nil (g aa1 l iy0))
("gahm" nil (g ae1 m))
("gahn" nil (g ae1 n))
("gahr" nil (g aa1 r))
("gaia" nil (g ay1 ax))
("gaidar" nil (g ay1 d aa1 r))
("gaier" nil (g ey1 er0))
("gaige" nil (g ey1 jh))
("gail" nil (g ey1 l))
("gaile" nil (g ey1 l))
("gailey" nil (g ey1 l iy0))
("gaillard" nil (g ey1 l er0 d))
("gaily" nil (g ey1 l iy0))
("gain" nil (g ey1 n))
("gained" nil (g ey1 n d))
("gainer" nil (g ey1 n er0))
("gainers" nil (g ey1 n er0 z))
("gaines" nil (g ey1 n z))
("gainesville" nil (g ey1 n z v ih1 l))
("gainey" nil (g ey1 n iy0))
("gainful" nil (g ey1 n f ax l))
("gainfully" nil (g ey1 n f ax l iy0))
("gaining" nil (g ey1 n ih0 ng))
("gainor" nil (g ey1 n er0))
("gainous" nil (g ey1 n ax s))
("gains" nil (g ey1 n z))
("gainsay" nil (g ey1 n s ey1))
("gainsboro" nil (g ey1 n z b er0 ow0))
("gainsco" nil (g ey1 n s k ow0))
("gair" nil (g eh1 r))
("gaiser" nil (g ey1 z er0))
("gait" nil (g ey1 t))
("gaitan" nil (g ay1 t aa1 n))
("gaiter" nil (g ey1 t er0))
("gaither" nil (g ey1 th er0))
("gaithersburg" nil (g ey1 th er0 z b er0 g))
("gaitskill" nil (g ey1 t s k ih1 l))
("gajda" nil (g ay1 d ax))
("gajewski" nil (g ay0 eh1 f s k iy0))
("gal" nil (g ae1 l))
("gala" nil (g ae1 l ax))
("galactic" nil (g ax l ae1 k t ih0 k))
("galactose" nil (g ax l ae1 k t ow0 s))
("galahad" nil (g ae1 l ax hh ae1 d))
("galambos" nil (g aa0 l aa1 m b ow0 z))
("galan" nil (g ey1 l ax n))
("galane" nil (g ax l ey1 n))
("galang" nil (g ae1 l ax ng))
("galanis" nil (g ax l aa1 n ih0 s))
("galant" nil (g aa1 l ax n t))
("galante" nil (g aa0 l aa1 n t iy0))
("galanter" nil (g ax l ae1 n t er0))
("galanti" nil (g ax l ae1 n t iy0))
("galarneau" nil (g ae1 l aa0 r n ow1))
("galarza" nil (g ax l aa1 r z ax))
("galas" nil (g ey1 l ax z))
("galassi" nil (g aa0 l aa1 s iy0))
("galasso" nil (g aa0 l aa1 s ow0))
("galatas" nil (g aa0 l aa1 t aa0 z))
("galatea" nil (g ae1 l ax t iy1 ax))
("galati" nil (g aa0 l aa1 t iy0))
("galaviz" nil (g ae1 l ax v ih0 z))
("galax" nil (g ae1 l ae1 k s))
("galaxies" nil (g ae1 l ax k s iy0 z))
("galaxy" nil (g ae1 l ax k s iy0))
("galban" nil (g ae1 l b ax n))
("galbo" nil (g ae1 l b ow0))
("galbraith" nil (g ae1 l b r ey1 th))
("galbreath" nil (g ae1 l b r eh1 th))
("gale" nil (g ey1 l))
("galea" nil (g ey1 l iy0 ax))
("galeano" nil (g aa0 l iy1 n ow0))
("galef" nil (g ae1 l ax f))
("galen" nil (g ey1 l ax n))
("galena" nil (g ax l iy1 n ax))
("galenical" nil (g ax l eh1 n ih0 k ax l))
("galentine" nil (g aa0 l eh0 n t iy1 n iy0))
("galer" nil (g ey1 l er0))
("galerias" nil (g ae1 l er0 iy1 ax z))
("galeries" nil (g ae1 l er0 iy0 z))
("gales" nil (g ey1 l z))
("galesburg" nil (g ey1 l z b er0 g))
("galesi" nil (g ax l eh1 s iy0))
("galey" nil (g ey1 l iy0))
("galford" nil (g ae1 l f er0 d))
("galgano" nil (g aa0 l g aa1 n ow0))
("gali" nil (g aa1 l iy0))
("galiano" nil (g aa0 l iy0 aa1 n ow0))
("galicia" nil (g ax l ih1 sh iy0 ax))
("galie" nil (g ey1 l iy0))
("galiena" nil (g aa0 l iy1 n ax))
("galik" nil (g ae1 l ih0 k))
("galilean" nil (g ae1 l ax l iy1 ax n))
("galilee" nil (g ae1 l ax l iy1))
("galileo" nil (g ae1 l ax l iy1 ow0))
("galin" nil (g ae1 l ih0 n))
("galindo" nil (g aa0 l iy1 n d ow0))
("galinski" nil (g ax l ih1 n s k iy0))
("galinsky" nil (g ax l ih1 n s k iy0))
("galioto" nil (g aa0 l iy0 ow1 t ow0))
("galipault" nil (g ae1 l ih0 p ao1 l t))
("galipeau" nil (g ae1 l ih0 p ow1))
("galka" nil (g ae1 l k ax))
("gall" nil (g ao1 l))
("galla" nil (g ae1 l ax))
("gallacher" nil (g ae1 l ax k er0))
("gallager" nil (g ao1 l ih0 jh er0))
("gallagher" nil (g ae1 l ax g er0))
("gallahan" nil (g ae1 l ax hh ae0 n))
("gallaher" nil (g ae1 l ax hh er0))
("gallamore" nil (g aa0 l aa1 m ao0 r))
("galland" nil (g ae1 l ax n d))
("gallant" nil (g ae1 l ax n t))
("gallantry" nil (g ae1 l ax n t r iy0))
("gallard" nil (g ax l aa1 r d))
("gallardo" nil (g aa0 l aa1 r d ow0))
("gallas" nil (g ae1 l ax z))
("gallatin" nil (g ae1 l ax t ih0 n))
("gallaway" nil (g ao1 l ax w ey1))
("gallbladder" nil (g ao1 l b l ae1 d er0))
("galle" nil (g ey1 l))
("gallego" nil (g aa0 l eh1 g ow0))
("gallegos" nil (g ae1 l ih0 g ow0 z))
("gallemore" nil (g aa0 l eh1 m ao0 r))
("gallen" nil (g ao1 l ax n))
("gallentine" nil (g aa0 l eh0 n t iy1 n iy0))
("galleon" nil (g ae1 l iy0 ax n))
("galleons" nil (g ae1 l iy0 ax n z))
("galler" nil (g ao1 l er0))
("gallerani" nil (g aa0 l er0 aa1 n iy0))
("galleria" nil (g ae1 l er0 iy1 ax))
("galleries" nil (g ae1 l er0 iy0 z))
("gallery" nil (g ae1 l er0 iy0))
("galles" nil (g ey1 l z))
("gallet" nil (g ae1 l ih0 t))
("galletti" nil (g aa0 l eh1 t iy0))
("galley" nil (g ae1 l iy0))
("galleys" nil (g ae1 l iy0 z))
("galli" nil (g ae1 l iy0))
("galliano" nil (g aa0 l iy0 aa1 n ow0))
("gallic" nil (g ae1 l ih0 k))
("gallick" nil (g ae1 l ih0 k))
("gallien" nil (g ax l iy1 n))
("gallier" nil (g ae1 l iy0 er0))
("galligan" nil (g ae1 l ih0 g ax n))
("galliher" nil (g ae1 l ih0 hh er0))
("gallik" nil (g ae1 l ih0 k))
("gallimard" nil (g ae1 l ih0 m er0 d))
("gallimore" nil (g aa0 l iy1 m ao0 r))
("gallina" nil (g aa0 l iy1 n ax))
("galling" nil (g ao1 l ih0 ng))
("gallinger" nil (g ao1 l ih0 ng er0))
("gallion" nil (g ae1 l y ax n))
("gallium" nil (g ae1 l iy0 ax m))
("gallivan" nil (g ae1 l ih0 v ax n))
("gallman" nil (g ao1 l m ax n))
("gallo" nil (g ae1 l ow0))
("gallogly" nil (g ae1 l ax g l iy0))
("gallon" nil (g ae1 l ax n))
("gallons" nil (g ae1 l ax n z))
("galloon" nil (g ax l uw1 n))
("gallop" nil (g ae1 l ax p))
("galloped" nil (g ae1 l ax p t))
("galloping" nil (g ae1 l ax p ih0 ng))
("gallow" nil (g ae1 l ow0))
("galloway" nil (g ae1 l ow0 w ey1))
("galloways" nil (g ae1 l ow0 w ey1 z))
("gallows" nil (g ae1 l ow0 z))
("galls" nil (g ao1 l z))
("gallstone" nil (g ao1 l s t ow1 n))
("gallstones" nil (g aa1 l s t ow1 n z))
("gallucci" nil (g aa0 l uw1 ch iy0))
("galluccio" nil (g aa0 l uw1 ch iy0 ow0))
("gallup" nil (g ae1 l ax p))
("gallus" nil (g ae1 l ax s))
("galluzzo" nil (g aa0 l uw1 z ow0))
("gallway" nil (g ao1 l w ey1))
("galoob" nil (g ax l uw1 b))
("galore" nil (g ax l ao1 r))
("galoshes" nil (g ax l aa1 sh ax z))
("galotti" nil (g ax l aa1 t iy0))
("galpin" nil (g ae1 l p ih0 n))
("gals" nil (g ae1 l z))
("galster" nil (g ae1 l s t er0))
("galsworthy" nil (g ae1 l z w er1 dh iy0))
("galt" nil (g ao1 l t))
("galton" nil (g ae1 l t ax n))
("galus" nil (g ae1 l ih0 s))
("galusha" nil (g ae1 l ax sh ax))
("galuska" nil (g ax l ah1 s k ax))
("galvan" nil (g aa0 l v aa1 n))
("galvanic" nil (g ae0 l v ae1 n ih0 k))
("galvanize" nil (g ae1 l v ax n ay1 z))
("galvanized" nil (g ae1 l v ax n ay1 z d))
("galvanizing" nil (g ae1 l v ax n ay1 z ih0 ng))
("galven" nil (g aa1 l v ax n))
("galveston" nil (g ae1 l v ax s t ax n))
("galvez" nil (g aa0 l v eh1 z))
("galvin" nil (g ae1 l v ih0 n))
("galway" nil (g aa1 l w ey0))
("galyean" nil (g ae1 l iy0 ax n))
("galyen" nil (g ae1 l iy0 eh0 n))
("galyon" nil (g ae1 l iy0 aa0 n))
("gama" nil (g aa1 m ax))
("gamache" nil (g aa1 m ey0 k))
("gamage" nil (g ae1 m ih0 jh))
("gamal" nil (g ax m aa1 l))
("gamba" nil (g ae1 m b ax))
("gambale" nil (g aa0 m b aa1 l iy0))
("gambardella" nil (g aa0 m b aa0 r d eh1 l ax))
("gamber" nil (g ae1 m b er0))
("gambia" nil (g ae1 m b iy0 ax))
("gambill" nil (g ax m b ih1 l))
("gambino" nil (g ae0 m b iy1 n ow0))
("gambit" nil (g ae1 m b ih0 t))
("gambits" nil (g ae1 m b ax t s))
("gamble" nil (g ae1 m b ax l))
("gambled" nil (g ae1 m b ax l d))
("gambler" nil (g ae1 m b l er0))
("gamblers" nil (g ae1 m b l er0 z))
("gambles" nil (g ae1 m b ax l z))
("gamblin" nil (g ae1 m b l ih0 n))
("gambling" nil (g ae1 m b ax l ih0 ng))
("gamboa" nil (g aa0 m b ow1 ax))
("gambone" nil (g aa1 m b ow0 n))
("gambrel" nil (g ae1 m b r ax l))
("gambrell" nil (g ae1 m b r ax l))
("gambrill" nil (g ae1 m b r ih0 l))
("gambro" nil (g ae1 m b r ow0))
("gamco" nil (g ae1 m k ow0))
("game" nil (g ey1 m))
("gamel" nil (g aa1 m ax l))
("gamelin" nil (g ae1 m ih0 l ih0 n))
("gamello" nil (g ax m eh1 l ow0))
("gamely" nil (g ey1 m l iy0))
("gamer" nil (g ey1 m er0))
("games" nil (g ey1 m z))
("gamesmanship" nil (g ey1 m z m ax n sh ih1 p))
("gamet" nil (g ae1 m ih0 t))
("gametangia" nil (g ae1 m ax t ae1 n jh iy0 ax))
("gamete" nil (g ae1 m iy0 t))
("gametophyte" nil (g ax m iy1 t ax f ay1 t))
("gamez" nil (g aa0 m eh1 z))
("gaming" nil (g ey1 m ih0 ng))
("gamino" nil (g aa0 m iy1 n ow0))
("gamm" nil (g ae1 m))
("gamma" nil (g ae1 m ax))
("gammage" nil (g ae1 m ih0 jh))
("gammel" nil (g ae1 m ax l))
("gammell" nil (g ae1 m ax l))
("gammill" nil (g ae1 m ih0 l))
("gammon" nil (g ae1 m ax n))
("gammons" nil (g ae1 m ax n z))
("gamp" nil (g ae1 m p))
("gamper" nil (g ae1 m p er0))
("gamunde" nil (g ax m uw1 n d))
("gamut" nil (g ae1 m ax t))
("gan" nil (g ae1 n))
("ganas" nil (g ae1 n ax z))
("ganaway" nil (g ae1 n ax w ey0))
("gancarz" nil (g aa1 n k aa0 r z))
("ganci" nil (g ae1 n s iy0))
("ganda" nil (g ae1 n d ax))
("gandalf" nil (g ae1 n d ao0 l f))
("gandara" nil (g aa0 n d aa1 r ax))
("gandee" nil (g ae1 n d iy0))
("gander" nil (g ae1 n d er0))
("gandhi" nil (g aa1 n d iy0))
("gandolfi" nil (g aa0 n d ow1 l f iy0))
("gandolfo" nil (g aa0 n d ow1 l f ow0))
("gandy" nil (g ae1 n d iy0))
("ganem" nil (g ae1 n ih0 m))
("ganes" nil (g ey1 n z))
("ganey" nil (g ey1 n iy0))
("gang" nil (g ae1 ng))
("gangbusters" nil (g ae1 ng b ah1 s t er0 z))
("gange" nil (g ae1 n jh))
("gangemi" nil (g aa0 ng g eh1 m iy0))
("ganger" nil (g ae1 ng er0))
("gangi" nil (g ae1 n jh iy0))
("ganging" nil (g ae1 ng ih0 ng))
("gangl" nil (g ae1 ng g ax l))
("ganglia" nil (g ae1 ng g l iy0 ax))
("ganglionic" nil (g ae1 ng g l iy0 aa1 n ih0 k))
("gangloff" nil (g ae1 ng g l ao0 f))
("gangly" nil (g ae1 ng l iy0))
("gangplank" nil (g ae1 ng p l ae1 ng k))
("gangs" nil (g ae1 ng z))
("gangsta" nil (g ae1 ng s t ax))
("gangster" nil (g ae1 ng s t er0))
("gangsters" nil (g ae1 ng s t er0 z))
("gangwer" nil (g ae1 ng w er0))
("ganic" nil (g ae1 n ih0 k))
("ganim" nil (g ae1 n ih0 m))
("ganis" nil (g ae1 n ih0 s))
("ganley" nil (g ae1 n l iy0))
("gann" nil (g ae1 n))
("gannaway" nil (g ae1 n ax w ey0))
("gannett" nil (g ae1 n ih0 t))
("gannon" nil (g ae1 n ax n))
("gano" nil (g aa1 n ow0))
("ganoe" nil (g ae1 n ow0))
("ganong" nil (g ae1 n ao0 ng))
("gans" nil (g ae1 n z))
("gansen" nil (g ae1 n s ax n))
("ganser" nil (g ae1 n s er0))
("ganske" nil (g ae1 n s k))
("ganson" nil (g ae1 n s ax n))
("ganster" nil (g ae1 n s t er0))
("gant" nil (g ae1 n t))
("gantenbein" nil (g ae1 n t ih0 n b ay0 n))
("ganter" nil (g ae1 n t er0))
("gantlet" nil (g ao1 n t l ax t))
("gantner" nil (g ae1 n t n er0))
("gantos" nil (g ae1 n t ow0 s))
("gantry" nil (g ae1 n t r iy0))
("gantt" nil (g ae1 n t))
("gantz" nil (g ae1 n t s))
("ganus" nil (g ey1 n ih0 s))
("ganymede" nil (g ae1 n ax m iy1 d))
("ganz" nil (g ae1 n z))
("ganzel" nil (g ae1 n z ax l))
("ganzer" nil (g ae1 n z er0))
("gaona" nil (g aa0 ow1 n ax))
("gap" nil (g ae1 p))
("gape" nil (g ey1 p))
("gaping" nil (g ey1 p ih0 ng))
("gapinski" nil (g ax p ih1 n s k iy0))
("gapp" nil (g ae1 p))
("gappa" nil (g ae1 p ax))
("gaps" nil (g ae1 p s))
("gar" nil (g aa1 r))
("gara" nil (g ae1 r ax))
("garabedian" nil (g ae1 r ax b iy1 d iy0 ax n))
("garafalo" nil (g aa0 r aa0 f aa1 l ow0))
("garafola" nil (g aa0 r aa0 f ow1 l ax))
("garage" nil (g er0 aa1 zh))
("garages" nil (g er0 aa1 zh ih0 z))
("garagiola" nil (g er0 ae1 jh iy0 ow1 l ax))
("garajda" nil (g er0 aa1 zh d ax))
("garamendi" nil (g eh1 r ax m eh1 n d iy0))
("garand" nil (g ae1 r ax n d))
("garant" nil (g aa1 r ax n t))
("garard" nil (g er0 aa1 r d))
("garavaglia" nil (g aa0 r aa0 v aa1 g l iy0 ax))
("garay" nil (g ae1 r ey0))
("garb" nil (g aa1 r b))
("garbacz" nil (g aa1 r b ax ch))
("garbage" nil (g aa1 r b ih0 jh))
("garbarini" nil (g aa0 r b aa0 r iy1 n iy0))
("garbarino" nil (g aa0 r b aa0 r iy1 n ow0))
("garbe" nil (g aa1 r b))
("garbed" nil (g aa1 r b d))
("garber" nil (g aa1 r b er0))
("garbers" nil (g aa1 r b er0 z))
("garbett" nil (g aa1 r b ih0 t))
("garbled" nil (g aa1 r b ax l d))
("garbo" nil (g aa1 r b ow0))
("garbutt" nil (g aa1 r b ax t))
("garceau" nil (g aa0 r s ow1))
("garces" nil (g aa1 r s eh0 s))
("garcetti" nil (g aa1 r ch eh1 t iy0))
("garcia" nil (g aa1 r s iy1 ax))
("garczynski" nil (g er0 ch ih1 n s k iy0))
("gard" nil (g aa1 r d))
("garda" nil (g aa1 r d ax))
("garde" nil (g aa1 r d))
("gardea" nil (g aa1 r d iy0 ax))
("gardella" nil (g aa1 r d eh1 l ax))
("garden" nil (g aa1 r d ax n))
("gardena" nil (g aa0 r d iy1 n ax))
("gardenamerica" nil (g aa1 r d ax n ax m eh1 r ih0 k ax))
("gardened" nil (g aa1 r d ax n d))
("gardener" nil (g aa1 r d ax n er0))
("gardeners" nil (g aa1 r d ax n er0 z))
("gardenhire" nil (g aa1 r d ax n hh ay1 r))
("gardenia" nil (g aa0 r d iy1 n y ax))
("gardenias" nil (g aa0 r d iy1 n y ax z))
("gardening" nil (g aa1 r d ax n ih0 ng))
("gardens" nil (g aa1 r d ax n z))
("gardin" nil (g aa1 r d ih0 n))
("gardiner" nil (g aa1 r d n er0))
("gardini" nil (g aa0 r d iy1 n iy0))
("gardinier" nil (g aa1 r d ih0 n iy0 er0))
("gardner" nil (g aa1 r d n er0))
("garduno" nil (g aa0 r d uw1 n ow0))
("gardyne" nil (g aa1 r d ay1 n))
("gare" nil (g eh1 r))
("gareau" nil (g er0 ow1))
("garelick" nil (g ae1 r ih0 l ih0 k))
("gareth" nil (g eh1 r ih0 th))
("garett" nil (g ae1 r ih0 t))
("garey" nil (g ae1 r iy0))
("garfield" nil (g aa1 r f iy1 l d))
("garfinkel" nil (g aa1 r f ih0 ng k ax l))
("garfinkle" nil (g aa1 r f ih1 ng k ax l))
("garg" nil (g aa1 r g))
("gargan" nil (g aa1 r g ax n))
("gargano" nil (g aa0 r g aa1 n ow0))
("gargantuan" nil (g aa0 r g ae1 n ch uw0 ax n))
("garger" nil (g aa1 r g er0))
("garges" nil (g aa1 r jh ih0 z))
("gargill" nil (g aa1 r jh ih0 l))
("gargis" nil (g aa1 r g ih0 s))
("gargiulo" nil (g aa1 r jh uw0 l ow0))
("gargoyles" nil (g aa1 r g oy1 l z))
("garguilo" nil (g aa0 r g w iy1 l ow0))
("gargus" nil (g aa1 r g ax s))
("gari" nil (g aa1 r iy0))
("garibaldi" nil (g ae1 r ax b ao1 l d iy0))
("garibay" nil (g ae1 r ih0 b ey0))
("gariepy" nil (g er0 iy1 p iy0))
("garin" nil (g eh1 r ih0 n))
("garing" nil (g eh1 r ih0 ng))
("garinger" nil (g eh1 r ih0 ng er0))
("garis" nil (g ae1 r ih0 s))
("garish" nil (g eh1 r ih0 sh))
("garity" nil (g ae1 r ih0 t iy0))
("garl" nil (g aa1 r l))
("garlan" nil (g aa1 r l ax n))
("garland" nil (g aa1 r l ax n d))
("garlic" nil (g aa1 r l ih0 k))
("garlick" nil (g aa1 r l ih0 k))
("garlicky" nil (g aa1 r l ih0 k iy0))
("garling" nil (g aa1 r l ih0 ng))
("garlinger" nil (g aa1 r l ih0 ng er0))
("garlinghouse" nil (g aa1 r l ih0 ng hh aw1 s))
("garlington" nil (g aa1 r l ih0 ng t ax n))
("garlitz" nil (g aa1 r l ih0 t s))
("garlock" nil (g aa1 r l ax k))
("garlow" nil (g aa1 r l ow0))
("garman" nil (g aa1 r m ax n))
("garmany" nil (g er0 m ao1 n iy0))
("garment" nil (g aa1 r m ax n t))
("garments" nil (g aa1 r m ax n t s))
("garmon" nil (g aa1 r m ax n))
("garmond" nil (g aa1 r m ax n d))
("garms" nil (g aa1 r m z))
("garmund" nil (g aa1 r m ax n d))
("garn" nil (g aa1 r n))
("garneau" nil (g aa0 r n ow1))
("garner" nil (g aa1 r n er0))
("garnered" nil (g aa1 r n er0 d))
("garnering" nil (g aa1 r n er0 ih0 ng))
("garners" nil (g aa1 r n er0 z))
("garnes" nil (g aa1 r n z))
("garnet" nil (g aa1 r n ax t))
("garnett" nil (g aa1 r n ih0 t))
("garnette" nil (g aa0 r n eh1 t))
("garney" nil (g aa1 r n iy0))
("garnica" nil (g aa0 r n iy1 k ax))
("garnier" nil (g aa1 r n iy0 er0))
("garnish" nil (g aa1 r n ih0 sh))
("garnishment" nil (g aa1 r n ih0 sh m ax n t))
("garno" nil (g aa1 r n ow0))
("garnock" nil (g aa1 r n ax k))
("garnsey" nil (g aa1 r n s iy0))
("garo" nil (g eh1 r ow0))
("garofalo" nil (g aa0 r ow0 f aa1 l ow0))
("garofano" nil (g aa0 r ow0 f aa1 n ow0))
("garofolo" nil (g eh0 r ax f ow1 l ax))
("garon" nil (g aa0 r ao1 n))
("garone" nil (g er0 ow1 n))
("garonzik" nil (g er0 aa1 n z ih0 k))
("garoutte" nil (g er0 uw1 t))
("garr" nil (g ae1 r))
("garrabrant" nil (g aa0 r aa1 b r ax n t))
("garrahan" nil (g ae1 r ax hh ae0 n))
("garramone" nil (g ae1 r ax m ow1 n))
("garrard" nil (g ae1 r er0 d))
("garratt" nil (g eh1 r ax t))
("garraway" nil (g ae1 r ax w ey0))
("garrell" nil (g aa0 r ey1 l))
("garrels" nil (g ae1 r ax l z))
("garrelts" nil (g ae1 r ih0 l t s))
("garren" nil (g aa1 r ax n))
("garret" nil (g eh1 r ih0 t))
("garrets" nil (g eh1 r ax t s))
("garretson" nil (g ae1 r ih0 t s ax n))
("garrett" nil (g eh1 r ih0 t))
("garrette" nil (g er0 eh1 t))
("garrey" nil (g ae1 r iy0))
("garrick" nil (g eh1 r ih0 k))
("garrido" nil (g aa0 r iy1 d ow0))
("garriga" nil (g ae1 r ih0 g ax))
("garrigan" nil (g ae1 r ih0 g ax n))
("garrigues" nil (g aa0 r iy1 g eh0 s))
("garrigus" nil (g aa0 r iy1 g ih0 s))
("garringer" nil (g ae1 r ih0 ng er0))
("garriott" nil (g ae1 r iy0 ax t))
("garris" nil (g ae1 r ih0 s))
("garrison" nil (g eh1 r ih0 s ax n))
("garrisoned" nil (g ae1 r ax s ax n d))
("garritano" nil (g aa0 r iy0 t aa1 n ow0))
("garritt" nil (g ae1 r ih0 t))
("garrity" nil (g eh1 r ih0 t iy0))
("garro" nil (g aa1 r ow0))
("garrod" nil (g ae1 r ax d))
("garron" nil (g ae1 r ax n))
("garros" nil (g eh1 r ow0 s))
("garrott" nil (g ae1 r ax t))
("garrow" nil (g eh1 r ow0))
("garroway" nil (g ae1 r ow0 w ey1))
("garrulous" nil (g eh1 r ax l ax s))
("garry" nil (g ae1 r iy0))
("gars" nil (g aa1 r z))
("garside" nil (g aa1 r s ay1 d))
("garske" nil (g aa1 r s k))
("garson" nil (g aa1 r s ax n))
("garst" nil (g aa1 r s t))
("garsten" nil (g aa1 r s t eh0 n))
("garstka" nil (g aa1 r s t k ax))
("garten" nil (g aa1 r t ax n))
("gartenberg" nil (g aa1 r t ax n b er0 g))
("garter" nil (g aa1 r t er0))
("garth" nil (g aa1 r th))
("garthwaite" nil (g aa1 r th w ey1 t))
("gartin" nil (g aa1 r t ih0 n))
("gartland" nil (g aa1 r t l ax n d))
("gartley" nil (g aa1 r t l iy0))
("gartman" nil (g aa1 r t m ax n))
("gartmore" nil (g aa1 r t m ao1 r))
("gartner" nil (g aa1 r t n er0))
("garton" nil (g aa1 r t ax n))
("gartrell" nil (g aa1 r t r ax l))
("gartside" nil (g aa1 r t s ay1 d))
("gartzke" nil (g aa1 r t z k))
("garver" nil (g aa1 r v er0))
("garverick" nil (g aa1 r v er0 ih0 k))
("garvey" nil (g aa1 r v iy0))
("garvie" nil (g aa1 r v iy0))
("garvin" nil (g aa1 r v ih0 n))
("garwin" nil (g aa1 r w ih0 n))
("garwood" nil (g aa1 r w uh1 d))
("gary" nil (g eh1 r iy0))
("garza" nil (g aa1 r z ax))
("garzarelli" nil (g aa1 r z er0 eh1 l iy0))
("garzon" nil (g aa1 r z ax n))
("gas" nil (g ae1 s))
("gasaway" nil (g ae1 s ax w ey1))
("gasbarro" nil (g aa0 s b aa1 r ow0))
("gasca" nil (g aa1 s k ax))
("gascon" nil (g ae1 s k ax n))
("gascons" nil (g ae1 s k ax n z))
("gase" nil (g ey1 z))
("gaseous" nil (g ae1 s iy0 ax s))
("gases" nil (g ae1 s ax z))
("gash" nil (g ae1 sh))
("gashes" nil (g ae1 sh ax z))
("gasich" nil (g ey1 z ih0 k))
("gasification" nil (g ae1 s ax f ax k ey1 sh ax n))
("gasior" nil (g ae1 s iy0 er0))
("gasiorowski" nil (g ax s iy0 ao0 r ao1 f s k iy0))
("gaska" nil (g aa1 s k ax))
("gaskamp" nil (g ae1 s k ae1 m p))
("gaskell" nil (g ae1 s k ax l))
("gasket" nil (g ae1 s k ax t))
("gaskets" nil (g ae1 s k ax t s))
("gaskey" nil (g ae1 s k iy1))
("gaskill" nil (g ae1 s k ih1 l))
("gaskin" nil (g ae1 s k ih0 n))
("gaskins" nil (g ae1 s k ih0 n z))
("gasner" nil (g ae1 s n er0))
("gasohol" nil (g ae1 s ax hh ao1 l))
("gasoline" nil (g ae1 s ax l iy1 n))
("gasolines" nil (g ae1 s ax l iy1 n z))
("gasp" nil (g ae1 s p))
("gaspar" nil (g ae1 s p er0))
("gaspard" nil (g ax s p aa1 r d))
("gaspari" nil (g aa0 s p aa1 r iy0))
("gasparini" nil (g aa0 s p aa0 r iy1 n iy0))
("gasparro" nil (g aa0 s p aa1 r ow0))
("gaspe" nil (g ae1 s p iy0))
("gasped" nil (g ae1 s p t))
("gasper" nil (g ae1 s p er0))
("gasperini" nil (g aa0 s p er0 iy1 n iy0))
("gasping" nil (g ae1 s p ih0 ng))
("gasps" nil (g ae1 s p s))
("gasque" nil (g ey1 s k))
("gass" nil (g ae1 s))
("gassaway" nil (g ae1 s ax w ey0))
("gassed" nil (g ae1 s t))
("gassee" nil (g ae1 s iy1))
("gassen" nil (g ae1 s ax n))
("gasser" nil (g ae1 s er0))
("gassert" nil (g ae1 s er0 t))
("gasses" nil (g ae1 s ih0 z))
("gassett" nil (g ae1 s ih0 t))
("gassing" nil (g ae1 s ih0 ng))
("gassman" nil (g ae1 s m ax n))
("gassmann" nil (g ae1 s m ax n))
("gassner" nil (g ae1 s n er0))
("gast" nil (g ae1 s t))
("gastelum" nil (g ey1 s t ih0 l ax m))
("gaster" nil (g ae1 s t er0))
("gastineau" nil (g ae1 s t ih0 n ow1))
("gaston" nil (g ae1 s t ax n))
("gastric" nil (g ae1 s t r ih0 k))
("gastritis" nil (g ae0 s t r ay1 t ax s))
("gastrointestinal" nil (g ae1 s t r ow0 ih0 n t eh1 s t ax n ax l))
("gastronomic" nil (g ax s t r aa1 n aa1 m ih0 k))
("gastronomy" nil (g ae0 s t r aa1 n ax m iy0))
("gastroscope" nil (g ae1 s t r ax s k ow1 p))
("gastrovascular" nil (g ae1 s t r ow0 v ae1 s k y ax l er0))
("gastrulate" nil (g ae1 s t r ax l ey1 t))
("gastrulation" nil (g ae1 s t r ax l ey1 sh ax n))
("gat" nil (g ae1 t))
("gatch" nil (g ae1 ch))
("gatchel" nil (g ae1 ch ax l))
("gatchell" nil (g ae1 ch ax l))
("gate" nil (g ey1 t))
("gatekeeper" nil (g ey1 t k iy1 p er0))
("gatekeepers" nil (g ey1 t k iy1 p er0 z))
("gateley" nil (g ae1 t ih0 l iy0))
("gately" nil (g ey1 t l iy0))
("gates" nil (g ey1 t s))
("gateway" nil (g ey1 t w ey1))
("gateways" nil (g ey1 t w ey1 z))
("gatewood" nil (g ey1 t w uh1 d))
("gath" nil (g ae1 th))
("gather" nil (g ae1 dh er0))
("gathered" nil (g ae1 dh er0 d))
("gatherer" nil (g ae1 dh er0 er0))
("gatherers" nil (g ae1 dh er0 er0 z))
("gathering" nil (g ae1 dh er0 ih0 ng))
("gatherings" nil (g ae1 dh er0 ih0 ng z))
("gathers" nil (g ae1 dh er0 z))
("gathings" nil (g ae1 th ih0 ng z))
("gathman" nil (g ae1 th m ax n))
("gathright" nil (g ae1 th r ay1 t))
("gatley" nil (g ae1 t l iy0))
("gatliff" nil (g ae1 t l ih0 f))
("gatlin" nil (g ae1 t l ih0 n))
("gatling" nil (g ae1 t l ih0 ng))
("gatoil" nil (g ax t oy1 l))
("gator" nil (g ey1 t er0))
("gatorade" nil (g ae1 t er0 ey1 d))
("gators" nil (g ey1 t er0 z))
("gatos" nil (g aa1 t ow1 s))
("gatrell" nil (g ae1 t r ax l))
("gatsby" nil (g ae1 t s b iy0))
("gatson" nil (g ae1 t s ax n))
("gatt" nil (g ae1 t))
("gatten" nil (g ae1 t ax n))
("gatti" nil (g ae1 t iy0))
("gatting" nil (g ae1 t ih0 ng))
("gattis" nil (g ae1 t ih0 s))
("gatton" nil (g ae1 t ax n))
("gattuso" nil (g aa0 t uw1 s ow0))
("gatward" nil (g ae1 t w er0 d))
("gatwick" nil (g ae1 t w ih1 k))
("gatx" nil (g ae1 t eh1 k s))
("gatz" nil (g ae1 t s))
("gatza" nil (g aa1 t z ax))
("gatzke" nil (g ae1 t z k))
("gau" nil (g ow1))
("gaub" nil (g ao1 b))
("gaubatz" nil (g aw1 b ax t s))
("gaubert" nil (g aw1 b er0 t))
("gauch" nil (g ao1 ch))
("gauche" nil (g ow1 sh))
("gaucher" nil (g ow1 sh er0))
("gaucho" nil (g aw1 ch ow0))
("gauchos" nil (g aw1 ch ow0 z))
("gaudet" nil (g ow0 d eh1 t))
("gaudette" nil (g ow0 d eh1 t))
("gaudin" nil (g ow0 d ae1 n))
("gaudino" nil (g ao1 d iy1 n ow0))
("gaudio" nil (g ao1 d iy0 ow0))
("gaudioso" nil (g ao0 d iy0 ow1 s ow0))
("gaudreau" nil (g ow0 d r ow1))
("gaudy" nil (g ao1 d iy0))
("gauer" nil (g aw1 er0))
("gauerke" nil (g aw1 er0 k))
("gaufman" nil (g ao1 f m ax n))
("gauge" nil (g ey1 jh))
("gauged" nil (g ey1 jh d))
("gauger" nil (g ey1 jh er0))
("gauges" nil (g ey1 jh ax z))
("gaugh" nil (g ao1))
("gaughan" nil (g ao1 ax n))
("gaughran" nil (g ao1 r ax n))
("gauging" nil (g ey1 jh ih0 ng))
("gaugler" nil (g ao1 g l er0))
("gauguin" nil (g ao1 g w ih0 n))
("gaul" nil (g ao1 l))
("gaulden" nil (g aw1 d ax n))
("gaulding" nil (g ao1 l d ih0 ng))
("gaulin" nil (g ow0 l ae1 n))
("gaulke" nil (g ao1 l k))
("gaulle" nil (g ao1 l))
("gaullist" nil (g ao1 l ih0 s t))
("gauls" nil (g ao1 l z))
("gault" nil (g ao1 l t))
("gaultney" nil (g ao1 l t n iy0))
("gaumer" nil (g aw1 m er0))
("gaumond" nil (g ow0 m aa1 n d))
("gauna" nil (g ao1 n ax))
("gaunce" nil (g ao1 n s))
("gaunt" nil (g ao1 n t))
("gauntlet" nil (g ao1 n t l ax t))
("gauntt" nil (g ao1 n t))
("gaus" nil (g ao1 z))
("gause" nil (g ao1 z))
("gausman" nil (g aw1 s m ax n))
("gauss" nil (g aw1 s))
("gaustad" nil (g aw1 s t ax d))
("gaut" nil (g ao1 t))
("gauthier" nil (g aw1 th iy0 er0))
("gauthreaux" nil (g ow0 th r ow1))
("gautier" nil (g aw1 t iy0 er0))
("gautney" nil (g ao1 t n iy0))
("gautreau" nil (g ow0 t r ow1))
("gautreaux" nil (g ow0 t r ow1))
("gauvin" nil (g ow0 v ae1 n))
("gauvreau" nil (g ow0 v r ow1))
("gauze" nil (g ao1 z))
("gavan" nil (g ey1 v ax n))
("gave" nil (g ey1 v))
("gavel" nil (g ae1 v ax l))
("gaven" nil (g ey1 v ax n))
("gaver" nil (g ey1 v er0))
("gavigan" nil (g ae1 v ih0 g ax n))
("gavin" nil (g ae1 v ih0 n))
("gaviria" nil (g ax v ih1 r iy0 ax))
("gavitt" nil (g ae1 v ih0 t))
("gavras" nil (g ae1 v r ax s))
("gaw" nil (g ao1))
("gawain" nil (g aa1 w ax n))
("gawel" nil (g ao1 ax l))
("gawen" nil (g ao1 ax n))
("gawk" nil (g ao1 k))
("gawking" nil (g ao1 k ih0 ng))
("gawky" nil (g ao1 k iy0))
("gawlik" nil (g ao1 l ih0 k))
("gawne" nil (g ao1 n))
("gawron" nil (g ao1 r ax n))
("gawronski" nil (g aa0 v r aa1 n s k iy0))
("gawthrop" nil (g ao1 th r ax p))
("gaxiola" nil (g ae0 k s iy0 ow1 l ax))
("gay" nil (g ey1))
("gayda" nil (g ey1 d ax))
("gayden" nil (g ey1 d ax n))
("gaydos" nil (g ey1 d ow0 z))
("gaydosh" nil (g ey1 d ax sh))
("gaye" nil (g ey1))
("gayer" nil (g ey1 er0))
("gayhart" nil (g ey1 hh aa1 r t))
("gayheart" nil (g ey1 hh aa1 r t))
("gayle" nil (g ey1 l))
("gayler" nil (g ey1 l er0))
("gayles" nil (g ey1 l z))
("gaylor" nil (g ey1 l er0))
("gaylord" nil (g ey1 l ao1 r d))
("gayman" nil (g ey0 m ae1 n))
("gaymon" nil (g ey1 m ax n))
("gayner" nil (g ey1 n er0))
("gaynor" nil (g ey1 n er0))
("gays" nil (g ey1 z))
("gaytan" nil (g ey1 t ax n))
("gayton" nil (g ey1 t ax n))
("gaz" nil (g aa1 z))
("gaza" nil (g aa1 z ax))
("gazaway" nil (g aa1 z ax w ey0))
("gazda" nil (g ae1 z d ax))
("gazdik" nil (g ae1 z d ih0 k))
("gaze" nil (g ey1 z))
("gazed" nil (g ey1 z d))
("gazella" nil (g ax z eh1 l ax))
("gazelle" nil (g ax z eh1 l))
("gazelles" nil (g ax z eh1 l z))
("gazes" nil (g ey1 z ax z))
("gazeta" nil (g ax z ey1 t ax))
("gazette" nil (g ax z eh1 t))
("gazing" nil (g ey1 z ih0 ng))
("gazonsky" nil (g ax z aa1 n s k iy0))
("gazprom" nil (g ae1 z p r aa1 m))
("gazzola" nil (g aa0 t s ow1 l ax))
("gdansk" nil (g ax d ae1 n s k))
("geac" nil (g iy1 k))
("geagea" nil (jh iy1 ax jh iy1 ax))
("gean" nil (jh iy1 n))
("gear" nil (g ih1 r))
("gearan" nil (g ih1 r ax n))
("gearbox" nil (g ih1 r b aa1 k s))
("gearboxes" nil (g ih1 r b aa0 k s ih0 z))
("geared" nil (g ih1 r d))
("gearey" nil (g ih1 r iy0))
("gearhart" nil (g ih1 r hh aa0 r t))
("gearheart" nil (g ih1 r hh aa0 r t))
("gearin" nil (g ih1 r ih0 n))
("gearing" nil (g ih1 r ih0 ng))
("gears" nil (g ih1 r z))
("geary" nil (g ih1 r iy0))
("gebauer" nil (g eh1 b aw0 er0))
("gebbia" nil (jh eh1 b iy0 ax))
("gebbie" nil (jh eh1 b iy0))
("gebel" nil (g eh1 b ax l))
("gebert" nil (g eh1 b er0 t))
("gebhard" nil (g eh1 b hh er0 d))
("gebhardt" nil (g eh1 b hh aa0 r t))
("gebhart" nil (g eh1 b hh aa0 r t))
("gebler" nil (g eh1 b ax l er0))
("gebo" nil (jh ey1 b ow0))
("gechem" nil (g eh1 ch ax m))
("geck" nil (jh eh1 k))
("gecko" nil (g eh1 k ow0))
("geckos" nil (g eh1 k ow0 z))
("geddes" nil (g eh1 d ax s))
("geddie" nil (jh eh1 d iy0))
("geddings" nil (jh eh1 d ih0 ng z))
("geddis" nil (g eh1 d ih0 s))
("gedeon" nil (g eh1 d iy0 ax n))
("gedney" nil (jh eh1 d n iy0))
("geduld" nil (g eh1 d ax l d))
("gee" nil (jh iy1))
("geeing" nil (jh iy1 ih0 ng))
("geek" nil (g iy1 k))
("geeks" nil (g iy1 k s))
("geena" nil (g iy1 n ax))
("geenen" nil (g iy1 n ax n))
("geer" nil (g ih1 r))
("geerdes" nil (g ih1 r d z))
("geers" nil (g iy1 er0 z))
("geerts" nil (g ih1 r t s))
("gees" nil (jh iy1 s))
("geesaman" nil (g iy1 s ax m ax n))
("geese" nil (g iy1 s))
("geesey" nil (g iy1 s iy0))
("geeslin" nil (g iy1 s l ih0 n))
("geeting" nil (g iy1 t ih0 ng))
("geez" nil (jh iy1 z))
("geffen" nil (g eh1 f ax n))
("geffert" nil (g eh1 f er0 t))
("geffner" nil (g eh1 f n er0))
("geffre" nil (jh eh1 f er0))
("gefinor" nil (g eh1 f ih0 n er0))
("gegenheimer" nil (g eh1 g ih0 n hh ay0 m er0))
("gegg" nil (jh eh1 g))
("gehl" nil (g eh1 l))
("gehle" nil (jh eh1 hh ax l))
("gehlhausen" nil (g eh1 l hh aw0 z ax n))
("gehling" nil (g eh1 l ih0 ng))
("gehm" nil (jh eh1 m))
("gehman" nil (g eh1 m ax n))
("gehr" nil (jh eh1 r))
("gehres" nil (jh eh1 r z))
("gehret" nil (g eh1 r ih0 t))
("gehrig" nil (g eh1 r ih0 g))
("gehring" nil (g eh1 r ih0 ng))
("gehringer" nil (g eh1 r ih0 ng er0))
("gehris" nil (g eh1 r ih0 s))
("gehrke" nil (jh eh1 r k))
("gehrman" nil (g eh1 r m ax n))
("gehrmann" nil (g eh1 r m ax n))
("gehrt" nil (g eh1 r t))
("gehry" nil (g eh1 r iy0))
("geib" nil (g ay1 b))
("geibel" nil (g ay1 b ax l))
("geico" nil (g ay1 k ow0))
("geidel" nil (g ay1 d ax l))
("geier" nil (g ay1 er0))
("geiger" nil (g ay1 g er0))
("geigle" nil (g ay1 g ax l))
("geigy" nil (g ay1 g iy0))
("geiken" nil (g ay1 k ax n))
("geil" nil (g ay1 l))
("geiler" nil (g ay1 l er0))
("geiman" nil (g ay1 m ax n))
("geimer" nil (g ay1 m er0))
("geis" nil (g ay1 z))
("geise" nil (g ay1 s))
("geisel" nil (g ay1 s ax l))
("geiselman" nil (g ay1 s ax l m ax n))
("geisen" nil (g ay1 s ax n))
("geiser" nil (g ay1 s er0))
("geisert" nil (g ay1 s er0 t))
("geisha" nil (g ey1 sh ax))
("geisinger" nil (g ay1 s ih0 n jh er0))
("geisler" nil (g ay1 s l er0))
("geiss" nil (g ay1 s))
("geissinger" nil (g ay1 s ih0 n jh er0))
("geissler" nil (g ay1 s l er0))
("geist" nil (g ay1 s t))
("geister" nil (g ay1 s t er0))
("geiszler" nil (g ay1 s l er0))
("geitner" nil (g ay1 t n er0))
("geitz" nil (g ay1 t s))
("gejdenson" nil (g ey1 d ax n s ax n))
("gekko" nil (g eh1 k ow0))
("gel" nil (jh eh1 l))
("gelardi" nil (jh eh0 l aa1 r d iy0))
("gelardin" nil (g ax l aa1 r d ih0 n))
("gelasia" nil (jh eh0 l aa1 s iy0 ax))
("gelatin" nil (jh eh1 l ax t ax n))
("gelatinous" nil (jh ax l ae1 t ax n ax s))
("gelb" nil (jh eh1 l b))
("gelbart" nil (g eh1 l b aa1 r t))
("gelber" nil (g eh1 l b er0))
("gelco" nil (jh eh1 l k ow0))
("gelder" nil (g eh1 l d er0))
("geldermann" nil (g eh1 l d er0 m ax n))
("gelett" nil (jh eh1 l ih0 t))
("gelette" nil (zh ih0 l eh1 t))
("gelfand" nil (g eh1 l f ax n d))
("gelinas" nil (g eh1 l ih0 n ax z))
("gelineau" nil (zh eh1 l ih0 n ow0))
("gell" nil (jh eh1 l))
("gellatly" nil (jh eh1 l ax t l iy0))
("geller" nil (g eh1 l er0))
("gellerman" nil (g eh1 l er0 m ax n))
("gellert" nil (g eh1 l er0 t))
("gelles" nil (jh eh1 l z))
("gelli" nil (g eh1 l iy0))
("gellis" nil (g eh1 l ih0 s))
("gellman" nil (g eh1 l m ax n))
("gellner" nil (g eh1 l n er0))
("gelman" nil (g eh1 l m ax n))
("gelpi" nil (jh eh1 l p iy0))
("gels" nil (jh eh1 l z))
("gelsinger" nil (g eh1 l s ih0 n jh er0))
("geltz" nil (g eh1 l t s))
("gelvin" nil (g eh1 l v ih0 n))
("gem" nil (jh eh1 m))
("gemayel" nil (g ax m ey1 ax l))
("gemberling" nil (g eh1 m b er0 l ih0 ng))
("gemco" nil (jh eh1 m k ow0))
("gemcraft" nil (jh eh1 m k r ae1 f t))
("gemex" nil (jh eh1 m eh0 k s))
("gemfibrozil" nil (g eh1 m f ih0 b r ow0 z ax l))
("gemignani" nil (jh eh0 m iy0 g n aa1 n iy0))
("gemina" nil (g eh0 m iy1 n ax))
("geminate" nil (jh eh1 m ax n ax t))
("gemini" nil (jh eh1 m ax n ay1))
("gemma" nil (jh eh1 m ax))
("gemme" nil (jh eh1 m))
("gemmell" nil (g eh1 m ax l))
("gemmer" nil (g eh1 m er0))
("gemmill" nil (g eh1 m ax l))
("gems" nil (jh eh1 m z))
("gemsbok" nil (g eh1 m z b aa0 k))
("gemstone" nil (jh eh1 m s t ow1 n))
("gemstones" nil (jh eh1 m s t ow1 n z))
("gen" nil (jh eh1 n er0 ax l))
("gena" nil (jh eh1 n ax))
("gencarelli" nil (jh eh0 n k aa0 r eh1 l iy0))
("genco" nil (jh eh1 ng k ow0))
("gencor" nil (jh eh1 n k ao1 r))
("gencorp" nil (jh eh1 n k ao1 r p))
("gendarme" nil (zh aa1 n d aa1 r m))
("gender" nil (jh eh1 n d er0))
("genders" nil (jh eh1 n d er0 z))
("gendler" nil (jh eh1 n d l er0))
("gendreau" nil (zh ih0 n d r ow1))
("gendrisek" nil (jh eh1 d r ih0 s eh1 k))
("gendron" nil (jh eh1 n d r ax n))
("gene" nil (jh iy1 n))
("genealogy" nil (jh iy1 n iy0 aa1 l ax jh iy0))
("geneen" nil (jh ax n iy1 n))
("genelabs" nil (jh eh1 n ax l ae1 b z))
("genemedicine" nil (jh eh1 n ax m eh1 d ax s ax n))
("genencor" nil (jh eh1 n ax n k ao1 r))
("genentech" nil (jh eh1 n ax n t eh1 k))
("genera" nil (jh eh1 n er0 ax))
("general" nil (jh eh1 n er0 ax l))
("generale" nil (jh eh1 n er0 ae1 l))
("generales" nil (jh eh1 n eh0 r aa1 l eh0 s))
("generali" nil (jh eh1 n er0 aa1 l iy0))
("generalissimo" nil (jh eh0 n eh1 r ax l ih1 s ih0 m ow1))
("generalist" nil (jh eh1 n er0 ax l ih0 s t))
("generalists" nil (jh eh1 n er0 ax l ih0 s t s))
("generalities" nil (jh eh1 n er0 ae1 l ax t iy0 z))
("generality" nil (jh eh1 n er0 ae1 l ax t iy0))
("generalization" nil (jh eh1 n er0 ax l ih0 z ey1 sh ax n))
("generalizations" nil (jh eh1 n er0 ax l ax z ey1 sh ax n z))
("generalize" nil (jh eh1 n er0 ax l ay1 z))
("generalized" nil (jh eh1 n er0 ax l ay1 z d))
("generally" nil (jh eh1 n er0 ax l iy0))
("generals" nil (jh eh1 n er0 ax l z))
("generalship" nil (jh eh1 n er0 ax l sh ih1 p))
("generate" nil (jh eh1 n er0 ey1 t))
("generated" nil (jh eh1 n er0 ey1 t ax d))
("generates" nil (jh eh1 n er0 ey1 t s))
("generating" nil (jh eh1 n er0 ey1 t ih0 ng))
("generation" nil (jh eh1 n er0 ey1 sh ax n))
("generational" nil (jh eh1 n er0 ey1 sh ax n ax l))
("generationaly" nil (jh eh1 n er0 ey1 sh ax n ax l iy0))
("generations" nil (jh eh1 n er0 ey1 sh ax n z))
("generative" nil (jh eh1 n er0 ax t ih0 v))
("generator" nil (jh eh1 n er0 ey1 t er0))
("generators" nil (jh eh1 n er0 ey1 t er0 z))
("genereux" nil (zh eh1 n er0 ow0))
("generic" nil (jh ax n eh1 r ih0 k))
("generically" nil (jh ax n eh1 r ih0 k l iy0))
("generics" nil (jh ax n eh1 r ih0 k s))
("generosity" nil (jh eh1 n er0 aa1 s ax t iy0))
("generous" nil (jh eh1 n er0 ax s))
("generously" nil (jh eh1 n er0 ax s l iy0))
("genes" nil (jh iy1 n z))
("genesco" nil (jh eh0 n eh1 s k ow0))
("genesee" nil (jh eh1 n ax s iy1))
("genesis" nil (jh eh1 n ax s ax s))
("genet" nil (jh eh1 n ih0 t))
("genetic" nil (jh ax n eh1 t ih0 k))
("genetically" nil (jh ax n eh1 t ih0 k l iy0))
("geneticist" nil (jh ax n eh1 t ax s ax s t))
("geneticists" nil (jh ax n eh1 t ax s ax s t s))
("genetics" nil (jh ax n eh1 t ih0 k s))
("geneva" nil (jh ax n iy1 v ax))
("geneve" nil (jh ax n iy1 v))
("genevieve" nil (jh eh1 n ax v iy1 v))
("genex" nil (jh eh1 n eh0 k s))
("geng" nil (jh eh1 ng))
("genger" nil (jh eh1 ng g er0))
("genghis" nil (jh eh1 ng hh ih0 s))
("gengler" nil (g ih1 ng ax l er0))
("genial" nil (jh iy1 n y ax l))
("geniality" nil (jh iy1 n iy0 ae1 l ax t iy0))
("genicom" nil (jh eh1 n ih0 k ax m))
("genie" nil (jh iy1 n iy0))
("geniere" nil (jh eh0 n y eh1 r))
("genis" nil (g eh1 n ih0 s))
("genisco" nil (jh eh0 n ih1 s k ow0))
("genital" nil (jh eh1 n ax t ax l))
("genitals" nil (jh eh1 n ax t ax l z))
("genius" nil (jh iy1 n y ax s))
("geniuses" nil (jh iy1 n y ax s ih0 z))
("genk" nil (jh eh1 ng k))
("genlyte" nil (jh eh1 n l ay1 t))
("genmar" nil (jh eh1 n m aa0 r))
("genna" nil (jh eh1 n ax))
("gennadi" nil (jh eh0 n aa1 d iy0))
("gennady" nil (g ax n aa1 d iy0))
("gennaro" nil (jh ax n aa1 r ow0))
("gennett" nil (jh eh1 n ih0 t))
("gennifer" nil (jh eh1 n ih0 f er0))
("geno" nil (jh iy1 n ow0))
("genoa" nil (jh eh1 n ow0 ax))
("genocide" nil (jh eh1 n ax s ay1 d))
("genoese" nil (jh eh1 n ow0 s))
("genome" nil (jh iy1 n ow1 m))
("genossenschaftsbank" nil (g eh0 n ow1 s eh0 n sh ae0 f t s b ae1 ng k))
("genotype" nil (jh eh1 n ax t ay1 p))
("genotypes" nil (jh eh1 n ax t ay1 p s))
("genova" nil (jh eh1 n ow0 v ax))
("genovese" nil (g eh1 n ax v iy0 z))
("genovesi" nil (jh eh1 n ow0 v eh1 s iy0))
("genpharm" nil (jh eh1 n f aa1 r m))
("genrad" nil (jh eh1 n r ae0 d))
("genre" nil (zh aa1 n r ax))
("genres" nil (zh aa1 n r ax z))
("genrich" nil (g eh1 n r ih0 k))
("gens" nil (jh eh1 n z))
("genscher" nil (g eh1 n sh er0))
("gensel" nil (g eh1 n s ax l))
("gensia" nil (jh eh1 n s iy1 ax))
("genske" nil (jh eh1 n s k))
("gensler" nil (g eh1 n s ax l er0))
("genson" nil (jh eh1 n s ax n))
("genstar" nil (jh eh1 n s t aa1 r))
("gent" nil (jh eh1 n t))
("genteel" nil (jh eh0 n t iy1 l))
("genter" nil (jh eh1 n t er0))
("genther" nil (g eh1 n dh er0))
("genthner" nil (jh eh1 n th n er0))
("gentian" nil (jh eh1 n sh ax n))
("gentile" nil (jh eh1 n t ay1 l))
("gentiles" nil (jh eh1 n t ay1 l z))
("gentility" nil (jh eh0 n t ih1 l ih0 t iy0))
("gentle" nil (jh eh1 n t ax l))
("gentleman" nil (jh eh1 n t ax l m ax n))
("gentlemanly" nil (jh eh1 n t ax l m ax n l iy0))
("gentlemen" nil (jh eh1 n t ax l m ih0 n))
("gentleness" nil (jh eh1 n t ax l n ax s))
("gentler" nil (jh eh1 n t l er0))
("gentles" nil (jh eh1 n t ax l z))
("gentlest" nil (jh eh1 n t ax l ax s t))
("gentlewoman" nil (jh eh1 n t ax l w uh1 m ax n))
("gentlewomen" nil (jh eh1 n t ax l w ih1 m ax n))
("gently" nil (jh eh1 n t l iy0))
("gentner" nil (jh eh1 n t n er0))
("gentrification" nil (jh eh1 n t r ih0 f ih0 k ey1 sh ax n))
("gentrified" nil (jh eh1 n t r ih0 f ay1 d))
("gentrifying" nil (jh eh1 n t r ih0 f ay1 ih0 ng))
("gentry" nil (jh eh1 n t r iy0))
("gentz" nil (jh eh1 n t s))
("gentzler" nil (jh eh1 n t s l er0))
("genuine" nil (jh eh1 n y ax w ax n))
("genuinely" nil (jh eh1 n y ax w ax n l iy0))
("genung" nil (jh eh1 n ax ng))
("genus" nil (jh iy1 n ax s))
("genz" nil (jh eh1 n z))
("genzyme" nil (jh eh1 n z ay1 m))
("geo" nil (jh iy1 ow0))
("geocentric" nil (jh iy1 ow0 s eh1 n t r ih0 k))
("geochemistry" nil (jh iy1 ow0 k eh1 m ax s t r iy0))
("geodesic" nil (jh iy1 ax d eh1 s ih0 k))
("geodesy" nil (jh iy0 aa1 d ax s iy0))
("geodyne" nil (jh iy1 ow0 d ay1 n))
("geoff" nil (jh eh1 f))
("geoffrey" nil (jh eh1 f r iy0))
("geoffrion" nil (jh iy1 aa1 f r iy0 ax n))
("geoffroy" nil (jh iy1 ax f r oy0))
("geoghegan" nil (g ax hh ey1 g ax n))
("geographer" nil (jh iy0 aa1 g r ax f er0))
("geographic" nil (jh iy1 ax g r ae1 f ih0 k))
("geographical" nil (jh iy1 ax g r ae1 f ih0 k ax l))
("geographically" nil (jh iy1 ax g r ae1 f ih0 k ax l iy0))
("geography" nil (jh iy0 aa1 g r ax f iy0))
("geoid" nil (jh iy1 oy0 d))
("geologic" nil (jh iy1 ax l aa1 jh ih0 k))
("geological" nil (jh iy1 ax l aa1 jh ih0 k ax l))
("geologist" nil (jh iy0 aa1 l ax jh ax s t))
("geologists" nil (jh iy0 aa1 l ax jh ih0 s t s))
("geology" nil (jh iy0 aa1 l ax jh iy0))
("geomagnetic" nil (jh iy1 ow0 m ae0 g n eh1 t ih0 k))
("geometric" nil (jh iy1 ax m eh1 t r ih0 k))
("geometrical" nil (jh iy1 ax m eh1 t r ih0 k ax l))
("geometrically" nil (jh iy1 ax m eh1 t r ih0 k l iy0))
("geometries" nil (jh iy0 aa1 m ax t r iy0 z))
("geometry" nil (jh iy0 aa1 m ax t r iy0))
("geomorphology" nil (jh iy1 ax m ao1 r f aa1 l ax jh iy0))
("geon" nil (jh iy1 aa0 n))
("geophysical" nil (jh iy1 ow0 f ih1 z ax k ax l))
("geopolitical" nil (jh iy1 ow0 p ax l ih1 t ih0 k ax l))
("geopolitically" nil (jh iy1 ow0 p ax l ih1 t ih0 k l iy0))
("geopolitics" nil (jh iy1 ow0 p aa1 l ax t ih0 k s))
("geordie" nil (jh iy1 er0 d iy0))
("georg" nil (g ey1 ao0 r g))
("georgakis" nil (jh ao1 r jh aa1 k ax s))
("george" nil (jh ao1 r jh))
("georgene" nil (jh ao1 r jh iy1 n))
("georges" nil (jh ao1 r jh ax z))
("georgeson" nil (jh iy1 er0 g ih0 s ax n))
("georgetown" nil (jh ao1 r jh t aw1 n))
("georgette" nil (jh ao0 r jh eh1 t))
("georgi" nil (jh iy0 aa1 r jh iy0))
("georgia" nil (jh ao1 r jh ax))
("georgiades" nil (jh ao1 r jh iy0 aa1 d ax s))
("georgiadis" nil (jh ao1 r jh iy0 aa1 d ih0 s))
("georgian" nil (jh ao1 r jh ax n))
("georgiana" nil (jh ao1 r jh iy0 ae1 n ax))
("georgians" nil (jh ao1 r jh ax n z))
("georgie" nil (jh ao1 r jh iy0))
("georgienne" nil (jh ao1 r jh iy0 eh1 n))
("georgina" nil (jh ao1 r jh iy1 n ax))
("georgine" nil (jh ao1 r jh iy1 n))
("georgiou" nil (jh ao1 r jh ow1))
("georgopoulos" nil (jh ao1 r jh aa1 p ax l ih0 s))
("georgy" nil (jh ao1 r jh iy0))
("geostrophic" nil (jh iy1 ow0 s t r aa1 f ih0 k))
("geosyncline" nil (jh iy1 ow0 s ih1 n k l ay0 n))
("geotaxis" nil (jh iy1 ow0 t ae1 k s ax s))
("geotek" nil (g iy1 ow0 t eh1 k))
("geothermal" nil (jh iy1 ow0 th er1 m ax l))
("geotropic" nil (jh iy1 ax t r aa1 p ih0 k))
("geotropism" nil (jh iy0 aa1 t r ax p ih1 z ax m))
("gephardt" nil (g eh1 p hh aa1 r t))
("gephart" nil (g eh1 p hh aa0 r t))
("geppert" nil (g eh1 p er0 t))
("gera" nil (jh eh1 r ax))
("gerace" nil (jh er0 aa1 ch iy0))
("geraci" nil (jh er0 aa1 ch iy0))
("geraghty" nil (jh eh1 r ax t iy0))
("gerais" nil (jh er0 ey1))
("gerald" nil (jh eh1 r ax l d))
("geraldina" nil (jh er0 aa0 l d iy1 n ax))
("geraldine" nil (jh eh0 r ax l d iy1 n))
("geraldo" nil (jh er0 aa1 l d ow0))
("geralds" nil (jh eh1 r ax l d z))
("geran" nil (jh eh1 r ax n))
("geranium" nil (jh er0 ey1 n iy0 ax m))
("geraniums" nil (jh er0 ey1 n iy0 ax m z))
("gerard" nil (jh er0 aa1 r d))
("gerardi" nil (jh er0 aa1 r d iy0))
("gerardo" nil (jh er0 aa1 r d ow0))
("gerashchenko" nil (g eh1 ax sh ey1 ng k ow0))
("gerasimov" nil (jh er0 ae1 s ih0 m aa0 v))
("gerbasi" nil (jh er0 b aa1 s iy0))
("gerber" nil (g er1 b er0))
("gerberding" nil (g er1 b er0 d ih0 ng))
("gerbert" nil (g er1 b er0 t))
("gerbig" nil (g er1 b ih0 g))
("gerbino" nil (jh er0 b iy1 n ow0))
("gerbrandt" nil (g er1 b r ae1 n t))
("gerd" nil (g er1 d))
("gerda" nil (g er1 d ax))
("gerdeman" nil (g er1 d m ax n))
("gerdes" nil (zh er1 d z))
("gerding" nil (g eh1 r t ih0 ng))
("gerdts" nil (jh er1 d t s))
("gere" nil (jh ih1 r))
("geremia" nil (jh er0 iy1 m iy0 ax))
("geren" nil (g ih1 r ax n))
("gerena" nil (jh er0 eh1 n ax))
("gerety" nil (zh eh1 r ih0 t iy0))
("gergel" nil (g er1 g ax l))
("gergely" nil (jh er1 jh l iy0))
("gergen" nil (g er1 g ax n))
("gerger" nil (g er1 g er0))
("gergrude" nil (g er1 g r uw0 d))
("gerhard" nil (g er1 hh aa1 r d))
("gerhardt" nil (g er1 hh aa0 r t))
("gerhart" nil (g er1 hh aa0 r t))
("gerhold" nil (g er1 hh ow0 l d))
("geriatric" nil (jh eh1 r iy0 ae1 t r ih0 k))
("geriatricians" nil (jh eh1 r iy0 ax t r ih1 sh ax n z))
("geriatrics" nil (jh eh1 r iy0 ae1 t r ih0 k s))
("gerich" nil (g eh1 r ih0 k))
("gericke" nil (jh eh1 r ih0 k))
("gerig" nil (jh eh1 r ih0 g))
("gering" nil (g ih1 r ih0 ng))
("geringer" nil (g eh1 r ih0 n jh er0))
("gerk" nil (jh er1 k))
("gerke" nil (jh er1 k))
("gerken" nil (g er1 k ax n))
("gerkin" nil (jh er1 k ih0 n))
("gerlach" nil (g er1 l ax k))
("gerleman" nil (g ao1 r ax l m ax n))
("gerlich" nil (g er1 l ih0 k))
("gerling" nil (g er1 l ih0 ng))
("gerlock" nil (g er1 l ax k))
("gerloff" nil (g er1 l ao0 f))
("germ" nil (jh er1 m))
("germain" nil (jh er0 m ey1 n))
("germaine" nil (zh er0 m ey1 n))
("german" nil (jh er1 m ax n))
("germane" nil (jh er0 m ey1 n))
("germani" nil (jh er0 m aa1 n iy0))
("germania" nil (jh er0 m ey1 n iy0 ax))
("germanic" nil (jh er0 m ae1 n ih0 k))
("germann" nil (g er1 m ax n))
("germano" nil (jh er0 m aa1 n ow0))
("germans" nil (jh er1 m ax n z))
("germantown" nil (jh er1 m ax n t aw1 n))
("germany" nil (jh er1 m ax n iy0))
("germanys" nil (jh er1 m ax n iy0 z))
("germe" nil (jh er1 m))
("germer" nil (jh er1 m er0))
("germicide" nil (jh er1 m ax s ay1 d))
("germinate" nil (jh er1 m ax n ey1 t))
("germinated" nil (jh er1 m ax n ey1 t ih0 d))
("germination" nil (jh er1 m ax n ey1 sh ax n))
("germiston" nil (jh er1 m ax s t aa1 n))
("germond" nil (g er1 m ax n d))
("germs" nil (jh er1 m z))
("gerner" nil (g er1 n er0))
("gernert" nil (g er1 n er0 t))
("gero" nil (jh eh1 r ow0))
("gerold" nil (g eh1 r ow0 l d))
("gerome" nil (g eh1 r ax m))
("geron" nil (jh eh1 r ax n))
("gerontology" nil (jh eh1 r ax n t aa1 l ax jh iy0))
("geroux" nil (zh er0 uw1))
("gerow" nil (jh eh1 r ow0))
("gerrald" nil (jh eh1 r ax l d))
("gerrick" nil (jh eh1 r ih0 k))
("gerrie" nil (jh eh1 r iy0))
("gerringer" nil (jh eh1 r ih0 ng er0))
("gerrish" nil (jh eh1 r ih0 sh))
("gerrit" nil (jh eh1 r ih0 t))
("gerrits" nil (jh eh1 r ih0 t s))
("gerritsen" nil (g eh1 r ih0 t s ax n))
("gerrity" nil (jh eh1 r ih0 t iy0))
("gerry" nil (jh eh1 r iy0))
("gerrymander" nil (jh eh1 r iy0 m ae1 n d er0))
("gerrymandered" nil (jh eh1 r iy0 m ae1 n d er0 d))
("gerrymandering" nil (jh eh1 r iy0 m ae1 n d er0 ih0 ng))
("gerrymanders" nil (jh eh1 r iy0 m ae1 n d er0 z))
("gersch" nil (g er1 sh))
("gersh" nil (g er1 sh))
("gershman" nil (g er1 sh m ax n))
("gershon" nil (g er1 sh ax n))
("gershowitz" nil (g er1 sh ax w ih0 t s))
("gershwin" nil (g er1 sh w ih0 n))
("gerson" nil (jh er1 s ax n))
("gerst" nil (g er1 s t))
("gerstein" nil (g er1 s t ay0 n))
("gerstel" nil (g er1 s t ax l))
("gersten" nil (g er1 s t ax n))
("gerstenberger" nil (g er1 s ax n b er0 g er0))
("gerstenhaber" nil (g er1 s t ax n hh ey1 b er0))
("gerster" nil (g er1 s t er0))
("gerstman" nil (g er1 s t m ax n))
("gerstner" nil (g er1 s t n er0))
("gerszewski" nil (g er0 sh eh1 f s k iy0))
("gert" nil (g er1 t))
("gerth" nil (g er1 th))
("gertie" nil (jh er1 t iy0))
("gertler" nil (g er1 t ax l er0))
("gertner" nil (g er1 t n er0))
("gertrud" nil (g er1 t r uw0 d))
("gertrude" nil (g er1 t r uw0 d))
("gertsch" nil (g er1 ch))
("gerty" nil (jh er1 t iy0))
("gertz" nil (g er1 t s))
("gervais" nil (zh er0 v ey1))
("gervase" nil (g er1 v ax s))
("gervasi" nil (jh er0 v aa1 s iy0))
("gervasio" nil (jh er0 v aa1 s iy0 ow0))
("gerwig" nil (g er1 w ih0 g))
("gerwin" nil (g er1 w ih0 n))
("gery" nil (jh eh1 r iy0))
("gesamtmetall" nil (g eh0 s ae1 m t m eh1 t ax l))
("geschke" nil (g eh1 sh k))
("gesco" nil (g eh1 s k ow0))
("gesell" nil (g eh1 s ax l))
("gesellschaft" nil (g ax s eh1 l sh ae0 f t))
("geske" nil (jh eh1 s k))
("gesner" nil (g eh1 s n er0))
("gess" nil (jh eh1 s))
("gessel" nil (g eh1 s ax l))
("gessert" nil (g eh1 s er0 t))
("gessler" nil (g eh1 s ax l er0))
("gessner" nil (g eh1 s n er0))
("gest" nil (jh eh1 s t))
("gestal" nil (jh eh1 s t ax l))
("gestapo" nil (g ax s t aa1 p ow0))
("gestation" nil (jh eh0 s t ey1 sh ax n))
("gestetner" nil (g ax s t eh1 t n er0))
("gesture" nil (jh eh1 s ch er0))
("gestures" nil (jh eh1 s ch er0 z))
("gesturing" nil (jh eh1 s ch er0 ih0 ng))
("gesualdi" nil (jh eh0 s uw0 aa1 l d iy0))
("get" nil (g eh1 t))
("getaway" nil (g eh1 t ax w ey1))
("getaways" nil (g eh1 t ax w ey1 z))
("getchell" nil (g eh1 ch ax l))
("geter" nil (g eh1 t er0))
("gethers" nil (g eh1 dh er0 z))
("getman" nil (g eh1 t m ax n))
("gets" nil (g eh1 t s))
("getsinger" nil (g eh1 t s ih0 n jh er0))
("gettel" nil (g eh1 t ax l))
("getter" nil (g eh1 t er0))
("getters" nil (g eh1 t er0 z))
("getting" nil (g eh1 t ih0 ng))
("gettinger" nil (g eh1 t ih0 ng er0))
("gettings" nil (g eh1 t ih0 ng z))
("gettis" nil (g eh1 t ih0 s))
("gettle" nil (g eh1 t ax l))
("gettler" nil (g eh1 t ax l er0))
("gettman" nil (g eh1 t m ax n))
("getts" nil (g eh1 t s))
("getty" nil (g eh1 t iy0))
("gettys" nil (g eh1 t iy0 z))
("gettysburg" nil (g eh1 t iy0 z b er0 g))
("getulio" nil (g ax ch uw1 l iy0 ow0))
("getz" nil (g eh1 t s))
("geurin" nil (zh er0 ae1 n))
("geurts" nil (zh er1 t s))
("gevaert" nil (g ax v eh1 r t))
("geving" nil (jh iy1 v ih0 ng))
("gewirtz" nil (g uw1 er0 t s))
("geyelin" nil (jh ey1 ih1 l ih0 n))
("geyer" nil (g ey1 er0))
("geyser" nil (g ay1 z er0))
("geysers" nil (g ay1 z er0 z))
("gfeller" nil (g ax f eh1 l er0))
("ghafar" nil (g ax f aa1 r))
("ghali" nil (g aa1 l iy0))
("ghana" nil (g ae1 n ax))
("ghanaian" nil (g ax n ay1 ax n))
("ghanaians" nil (g ax n ay1 ax n z))
("gharbonifar" nil (g aa0 r b aa1 n ih0 f aa0 r))
("ghastliness" nil (g ae1 s t l iy0 n ax s))
("ghastly" nil (g ae1 s t l iy0))
("ghee" nil (g iy1))
("gheen" nil (g iy1 n))
("ghent" nil (g eh1 n t))
("gherardi" nil (g er0 aa1 r d iy0))
("gherman" nil (g er1 m ax n))
("ghetto" nil (g eh1 t ow0))
("ghettos" nil (g eh1 t ow0 z))
("ghez" nil (g eh1 z))
("ghezzi" nil (g eh1 z iy0))
("ghidella" nil (g ih0 d eh1 l ax))
("ghio" nil (g ay1 ow0))
("gholamreza" nil (g ow1 l ax m r ey1 z ax))
("gholson" nil (g ow1 l s ax n))
("gholston" nil (g ow1 l s t ax n))
("ghorbanifar" nil (g ao0 r b ae1 n ih0 f aa0 r))
("ghormley" nil (g ao1 r m l iy0))
("ghose" nil (g ow1 z))
("ghosh" nil (g aa1 sh))
("ghost" nil (g ow1 s t))
("ghostbusters" nil (g ow1 s t b ah1 s t er0 z))
("ghostlike" nil (g ow1 s t l ay1 k))
("ghostly" nil (g ow1 s t l iy0))
("ghosts" nil (g ow1 s t s))
("ghoulish" nil (g uw1 l ih0 sh))
("ghrist" nil (g r ih1 s t))
("ghulam" nil (g y uw1 l ae0 m))
("ghuloum" nil (g uw1 l ow1 m))
("giacalone" nil (jh aa1 k aa0 l ow0 n iy0))
("giacco" nil (jh aa1 k ow0))
("giacinta" nil (jh aa1 ch iy0 n t ax))
("giacobbe" nil (jh aa1 k ow0 b iy0))
("giacomelli" nil (jh ax k ow0 m eh1 l iy0))
("giacomini" nil (jh ax k ow0 m iy1 n iy0))
("giacomo" nil (jh aa1 k ax m ow0))
("giaconda" nil (jh iy1 ax k aa1 n d ax))
("giacone" nil (jh iy1 ax k ow1 n iy0))
("giaimo" nil (jh ey1 m ow0))
("gialanella" nil (jh ax l aa0 n eh1 l ax))
("giamatti" nil (jh iy1 ax m aa1 t iy0))
("giambalvo" nil (jh aa1 m b aa0 l v ow0))
("giambra" nil (jh aa1 m b r ax))
("giambrone" nil (jh aa1 m b r ow0 n iy0))
("giammalva" nil (jh iy1 ax m ao1 l v ax))
("giammarco" nil (jh aa1 m aa0 r k ow0))
("giammarino" nil (jh ax m aa0 r iy1 n ow0))
("giampa" nil (jh aa1 m p ax))
("giampaolo" nil (jh aa0 m p aw1 l ow0))
("giampietro" nil (jh aa1 m p iy0 t r ow0))
("gian" nil (jh iy1 aa0 n))
("giancarlo" nil (jh iy1 ae0 ng k aa1 r l ow0))
("giancola" nil (jh aa1 ng k ow0 l ax))
("gianelli" nil (jh ax n eh1 l iy0))
("gianfrancesco" nil (jh aa1 n f r ae0 n ch eh1 s k ow0))
("giang" nil (jh iy0 aa1 ng))
("giangrande" nil (jh aa1 n g r ae0 n d iy0))
("gianino" nil (jh ax n iy1 n ow0))
("giannattasio" nil (jh aa1 n aa0 t aa0 s iy0 ow0))
("giannelli" nil (jh ax n eh1 l iy0))
("giannetti" nil (jh ax n eh1 t iy0))
("giannetto" nil (jh ax n eh1 t ow0))
("gianni" nil (jh aa1 n iy0))
("giannini" nil (jh ax n iy1 n iy0))
("giannino" nil (jh iy1 ax n iy1 n ow0))
("giannola" nil (jh aa1 n ow0 l ax))
("giannone" nil (jh aa1 n ow0 n iy0))
("giannotti" nil (jh aa1 n ow0 t iy0))
("gianotti" nil (jh aa1 n ow0 t iy0))
("giant" nil (jh ay1 ax n t))
("giants" nil (jh ay1 ax n t s))
("giaquinto" nil (jh aa1 k w iy0 n t ow0))
("giard" nil (jh iy0 aa1 r d))
("giardina" nil (jh aa1 r d iy0 n ax))
("giardini" nil (jh er0 d iy1 n iy0))
("giardino" nil (jh er0 d iy1 n ow0))
("giarratano" nil (jh aa1 r ax t aa1 n ow0))
("giarrusso" nil (jh aa0 r uw1 s ow0))
("gib" nil (g ih1 b))
("gibas" nil (jh ay1 b ax z))
("gibb" nil (jh ih1 b))
("gibbard" nil (zh ih0 b aa1 r d))
("gibbens" nil (g ih1 b ax n z))
("gibberish" nil (g ih1 b er0 ih0 sh))
("gibbins" nil (jh ih1 b ih0 n z))
("gibble" nil (jh ih1 b ax l))
("gibbon" nil (g ih1 b ax n))
("gibboney" nil (jh ih1 b ax n iy0))
("gibbons" nil (g ih1 b ax n z))
("gibbs" nil (g ih1 b z))
("gibby" nil (jh ih1 b iy0))
("gibeau" nil (zh ih0 b ow1))
("gibeault" nil (zh ih0 b ow1))
("giberson" nil (jh ih1 b er0 s ax n))
("gibert" nil (g ih1 b er0 t))
("gibian" nil (g ih1 b iy0 ax n))
("giblen" nil (g ih1 b l ax n))
("gibler" nil (jh ih1 b ax l er0))
("giblet" nil (jh ih1 b l ax t))
("giblin" nil (jh ih1 b l ih0 n))
("gibney" nil (jh ih1 b n iy0))
("giboney" nil (jh ih1 b ax n iy0))
("gibraltar" nil (jh ih0 b r ao1 l t er0))
("gibson" nil (g ih1 b s ax n))
("gick" nil (jh ih1 k))
("gidcumb" nil (g ih1 d k ax m))
("giddens" nil (g ih1 d ax n z))
("giddings" nil (g ih1 d ih0 ng z))
("giddy" nil (g ih1 d iy0))
("gidel" nil (g ay1 d eh1 l))
("gideon" nil (g ih1 d iy0 ax n))
("gidget" nil (g ih1 jh ax t))
("gidley" nil (g ih1 d l iy0))
("gidney" nil (g ih1 d n iy0))
("gidwitz" nil (g ih1 d w ih0 t s))
("giebel" nil (g iy1 b ax l))
("giebler" nil (g iy1 b ax l er0))
("gieck" nil (jh iy1 k))
("giefer" nil (g iy1 f er0))
("gieger" nil (g iy1 g er0))
("giegerich" nil (g iy1 g er0 ih0 k))
("giel" nil (jh iy1 l))
("gielgud" nil (g iy1 l g ax d))
("gielow" nil (jh iy1 l ow0))
("gienger" nil (g iy1 n jh er0))
("gienow" nil (g iy1 n aw0))
("gier" nil (jh ih1 r))
("giere" nil (jh ih1 r))
("gierhart" nil (g ih1 r hh aa0 r t))
("gierke" nil (jh ih1 r k))
("gierman" nil (g ih1 r m ax n))
("giersch" nil (g ih1 r sh))
("gies" nil (g iy1 z))
("giesbrecht" nil (g iy1 s b r ih0 k t))
("giese" nil (jh iy1 s))
("giesecke" nil (g iy1 s ih0 k))
("gieseke" nil (g iy1 s ih0 k))
("gieseler" nil (g iy1 s ax l er0))
("gieselman" nil (g iy1 s ax l m ax n))
("giesen" nil (g iy1 s ax n))
("gieser" nil (g iy1 s er0))
("giesey" nil (jh iy1 s iy0))
("gieske" nil (jh iy1 s k))
("giesler" nil (g iy1 s ax l er0))
("giessen" nil (g iy1 z s ax n))
("gietzen" nil (g iy1 t z ax n))
("giff" nil (g ih1 f))
("giffard" nil (g ih1 f er0 d))
("giffen" nil (g ih1 f ax n))
("gifferd" nil (g ih1 f er0 d))
("giffin" nil (g ih1 f ih0 n))
("gifford" nil (g ih1 f er0 d))
("giffy" nil (g ih1 f iy0))
("gift" nil (g ih1 f t))
("gifted" nil (g ih1 f t ax d))
("giftrust" nil (g ih1 f t r ah1 s t))
("gifts" nil (g ih1 f t s))
("giftware" nil (g ih1 f t w eh1 r))
("gig" nil (g ih1 g))
("gigabytes" nil (g ih1 g ax b ay1 t s))
("gigaflops" nil (g ih1 g ax f l aa1 p s))
("gigante" nil (jh iy0 g aa1 n t iy0))
("gigantic" nil (jh ay0 g ae1 n t ih0 k))
("giger" nil (g ay1 g er0))
("gigging" nil (g ih1 g ih0 ng))
("giggle" nil (g ih1 g ax l))
("giggled" nil (g ih1 g ax l d))
("giggles" nil (g ih1 g ax l z))
("giggling" nil (g ih1 g ax l ih0 ng))
("giggly" nil (g ih1 g ax l iy0))
("giglia" nil (jh iy1 g l iy0 ax))
("giglio" nil (jh ih1 g l iy0 ow0))
("gigliotti" nil (jh iy0 g l iy0 ow1 t iy0))
("gignac" nil (g ih1 g n ax k))
("gignoux" nil (g ih0 n uw1))
("gigot" nil (jh ih1 g ax t))
("gigs" nil (g ih1 g z))
("giguere" nil (jh iy0 g eh1 r ey0))
("gikas" nil (g ay1 k ax z))
("gil" nil (g ih1 l))
("gilardi" nil (jh iy0 l aa1 r d iy0))
("gilb" nil (g ih1 l b))
("gilberg" nil (g ih1 l b er0 g))
("gilbert" nil (g ih1 l b er0 t))
("gilberta" nil (jh iy0 l b eh1 r t ax))
("gilberte" nil (g ih1 l b er0 t))
("gilberti" nil (jh iy0 l b eh1 r t iy0))
("gilbertina" nil (jh iy0 l b er0 t iy1 n ax))
("gilbertine" nil (jh iy0 l b er0 t iy1 n iy0))
("gilberto" nil (g ih0 l b eh1 r t ow1))
("gilbertson" nil (g ih1 l b er0 t s ax n))
("gilbey" nil (g ih1 l b iy0))
("gilbo" nil (jh iy1 l b ow0))
("gilboy" nil (g ih1 l b oy0))
("gilbreath" nil (g ih1 l b r eh1 th))
("gilbreth" nil (g ih1 l b r ih0 th))
("gilbride" nil (g ih1 l b r ay1 d))
("gilby" nil (g ih1 l b iy0))
("gilchrest" nil (g ih1 l k er0 ih0 s t))
("gilchrist" nil (g ih1 l k r ih0 s t))
("gilcrease" nil (g ih0 l k r iy1 s))
("gilcrest" nil (g ih1 l k er0 ih0 s t))
("gilda" nil (g ih1 l d ax))
("gilday" nil (g ih1 l d ey1))
("gildea" nil (jh iy1 l d iy0 ax))
("gilded" nil (g ih1 l d ih0 d))
("gilden" nil (g ih1 l d ax n))
("gilder" nil (g ih1 l d er0))
("gildersleeve" nil (g ih1 l d er0 s l iy1 v))
("gilding" nil (g ih1 l d ih0 ng))
("gildner" nil (g ih1 l d n er0))
("gildon" nil (g ih1 l d ax n))
("gile" nil (g ay1 l))
("gilead" nil (g ih0 l iy1 d))
("giles" nil (jh ay1 l z))
("gilfillan" nil (g ih1 l f ih1 l ax n))
("gilford" nil (g ih1 l f er0 d))
("gilger" nil (g ih1 l g er0))
("gilgit" nil (g ih1 l jh ih0 t))
("gilgore" nil (g ih1 l g ao1 r))
("gilham" nil (g ih1 l hh ax m))
("gilhooly" nil (g ih1 l hh uw0 l iy0))
("gilkerson" nil (g ih1 l k er0 s ax n))
("gilkes" nil (g ih1 l k s))
("gilkeson" nil (g ih1 l k ih0 s ax n))
("gilkey" nil (g ih1 l k iy0))
("gilkison" nil (g ih1 l k ih0 s ax n))
("gill" nil (g ih1 l))
("gillam" nil (g ih1 l ax m))
("gillan" nil (g ih1 l ax n))
("gilland" nil (g ih1 l ax n d))
("gillard" nil (zh ih0 l aa1 r d))
("gillaspie" nil (g ih1 l ax s p iy0))
("gillaspy" nil (g ih1 l ax s p iy0))
("gille" nil (g ay1 l))
("gillean" nil (g ih1 l ax n))
("gilleland" nil (g ih1 l ih0 l ae0 n d))
("gillem" nil (g ih1 l ih0 m))
("gillen" nil (g ih1 l ax n))
("gillentine" nil (g ih1 l ax n t ay1 n))
("giller" nil (g ih1 l er0))
("gilleran" nil (g ih1 l er0 ae0 n))
("gillers" nil (g ih1 l er0 z))
("gilles" nil (zh iy1 l))
("gillespie" nil (g ax l eh1 s p iy0))
("gillet" nil (g ih1 l ih0 t))
("gillett" nil (jh ih0 l ih1 t))
("gillette" nil (jh ih0 l eh1 t))
("gilley" nil (g ih1 l iy0))
("gillham" nil (g ih1 l hh ax m))
("gilliam" nil (g ih1 l iy0 ax m))
("gillian" nil (jh ih1 l iy0 ax n))
("gilliand" nil (g ih1 l iy0 ax n d))
("gilliard" nil (g ih1 l iy0 er0 d))
("gilliatt" nil (g ih1 l iy0 ae0 t))
("gillick" nil (g ih1 l ih0 k))
("gillie" nil (g ih1 l iy0))
("gillies" nil (g ih1 l iy0 z))
("gillig" nil (g ih1 l ih0 g))
("gilligan" nil (g ih1 l ax g ax n))
("gillihan" nil (g ih1 l ax hh ae0 n))
("gillikin" nil (g ih1 l ax k ax n))
("gillilan" nil (g ih1 l ax l ax n))
("gilliland" nil (g ih1 l ax l ax n d))
("gillin" nil (g ih1 l ih0 n))
("gillingham" nil (g ih1 l ih0 ng hh ae1 m))
("gillings" nil (g ih1 l ih0 ng z))
("gillinov" nil (g ih1 l ih0 n aa0 v))
("gillins" nil (g ih1 l ih0 n z))
("gillis" nil (g ih1 l ih0 s))
("gillison" nil (g ih1 l ih0 s ax n))
("gillispie" nil (g ax l eh1 s p iy0))
("gillman" nil (g ih1 l m ax n))
("gillmore" nil (jh iy1 l m ao0 r))
("gillock" nil (g ih1 l ax k))
("gillogly" nil (g ih1 l ax g l iy0))
("gillon" nil (g ih1 l ax n))
("gillooly" nil (g ih1 l uw0 l iy0))
("gillott" nil (g ih1 l ax t))
("gillotti" nil (jh iy0 l ow1 t iy0))
("gills" nil (g ih1 l z))
("gillson" nil (g ih1 l s ax n))
("gillum" nil (g ih1 l ax m))
("gilly" nil (g ih1 l iy0))
("gilman" nil (g ih1 l m ax n))
("gilmartin" nil (g ih0 l m aa1 r t ih0 n))
("gilmer" nil (g ih1 l m er0))
("gilmore" nil (g ih1 l m ao0 r))
("gilmour" nil (zh ih0 l m uh1 r))
("gilpatrick" nil (g ih1 l p ax t r ih0 k))
("gilpin" nil (g ih1 l p ih0 n))
("gilreath" nil (g ih1 l r eh0 th))
("gilroy" nil (g ih1 l r oy1))
("gilsdorf" nil (g ih1 l s d ao0 r f))
("gilson" nil (g ih1 l s ax n))
("gilstrap" nil (g ih1 l s t r ax p))
("gilt" nil (g ih1 l t))
("giltner" nil (g ih1 l t n er0))
("gilts" nil (g ih1 l t s))
("gilvin" nil (g ih1 l v ih0 n))
("gilyard" nil (zh ax l y aa1 r d))
("gimbel" nil (g ih1 m b ax l))
("gimenez" nil (jh iy0 m ey1 n eh0 z))
("gimlin" nil (g ih1 m l ih0 n))
("gimme" nil (g ih1 m iy0))
("gimmick" nil (g ih1 m ih0 k))
("gimmickry" nil (g ih1 m ih0 k r iy0))
("gimmicks" nil (g ih1 m ih0 k s))
("gimmicky" nil (g ih0 m ih1 k iy0))
("gimpel" nil (g ih1 m p ax l))
("gin" nil (jh ih1 n))
("gina" nil (jh iy1 n ax))
("ginandjar" nil (jh ih0 n ae1 n jh er0))
("ginder" nil (g ay1 n d er0))
("gindlesperger" nil (g ih1 n d l ih0 s p er0 g er0))
("gines" nil (jh ay1 n z))
("ginevra" nil (jh ih0 n eh1 v r ax))
("ging" nil (jh ih1 ng))
("gingell" nil (g ih1 ng g ax l))
("ginger" nil (jh ih1 n jh er0))
("gingerbread" nil (jh ih1 n jh er0 b r eh1 d))
("gingerich" nil (g ih1 ng g er0 ih0 k))
("gingerly" nil (jh ih1 n jh er0 l iy0))
("gingery" nil (jh ih1 n jh er0 iy0))
("ginghams" nil (g ih1 ng ax m z))
("gingles" nil (jh ih1 ng g ax l z))
("gingold" nil (jh ih1 n g ow1 l d))
("gingras" nil (g ih1 ng g r ax z))
("gingrich" nil (g ih1 ng g r ih0 ch))
("ginley" nil (jh ih1 n l iy0))
("ginn" nil (jh ih1 n))
("ginnie" nil (jh ih1 n iy0))
("ginning" nil (jh ih1 n ih0 ng))
("ginny" nil (jh ih1 n iy0))
("gino" nil (jh iy1 n ow0))
("ginocchio" nil (jh iy0 n ow1 k iy0 ow0))
("ginsberg" nil (g ih1 n s b er0 g))
("ginsburg" nil (g ih1 n z b er0 g))
("ginseng" nil (jh ih1 n s eh1 ng))
("ginsu" nil (g ih1 n s uw0))
("gintel" nil (jh ih1 n t eh1 l))
("ginter" nil (g ih1 n t er0))
("ginther" nil (g ih1 n dh er0))
("ginting" nil (jh ih1 n t ih1 ng))
("ginty" nil (jh ih1 n t iy0))
("gintz" nil (g ih1 n t s))
("ginyard" nil (jh ih1 n y aa1 r d))
("ginza" nil (g ih1 n z ax))
("ginzberg" nil (g ih1 n z b er0 g))
("gioia" nil (jh ow1 y ax))
("gionet" nil (jh iy0 ow1 n ey0 t))
("gionfriddo" nil (jh ow0 n f r iy1 d ow0))
("giordani" nil (jh ao0 r d aa1 n iy0))
("giordano" nil (jh ao0 r d aa1 n ow0))
("giorgi" nil (jh ao1 r jh iy0))
("giorgia" nil (jh ao1 r jh ax))
("giorgio" nil (jh ao1 r jh iy0 ow0))
("giovanelli" nil (jh ow0 v aa0 n eh1 l iy0))
("giovanetti" nil (jh ow0 v aa0 n eh1 t iy0))
("giovannetti" nil (jh ow0 v aa0 n eh1 t iy0))
("giovanni" nil (jh iy1 ow0 v aa1 n iy0))
("giovanniello" nil (jh ow0 v aa1 n iy0 eh1 l ow0))
("giovannini" nil (jh ow0 v aa0 n iy1 n iy0))
("giovannoni" nil (jh ow0 v aa0 n ow1 n iy0))
("giovenco" nil (jh ow0 v eh1 n k ow0))
("giovinazzo" nil (jh ow0 v iy0 n aa1 z ow0))
("gipe" nil (jh ay1 p))
("gipp" nil (jh ih1 p))
("gipper" nil (g ih1 p er0))
("gipple" nil (jh ih1 p ax l))
("gipson" nil (g iy1 p s ax n))
("giraffe" nil (jh er0 ae1 f))
("giraffes" nil (jh er0 ae1 f s))
("giraldo" nil (jh ih0 r aa1 l d ow0))
("girard" nil (jh er0 aa1 r d))
("girardi" nil (jh ih0 r aa1 r d iy0))
("girardin" nil (zh ao1 r aa0 r d ae0 n))
("girardot" nil (zh ao1 r aa0 r d ow0))
("giraud" nil (zh ay0 r ow1))
("gird" nil (g er1 d))
("girders" nil (g er1 d er0 z))
("girding" nil (g er1 d ih0 ng))
("girdle" nil (g er1 d ax l))
("girdler" nil (g er1 d ax l er0))
("girdley" nil (g er1 d l iy0))
("girdner" nil (g er1 d n er0))
("girds" nil (g er1 d z))
("gire" nil (g ay1 r))
("girgenti" nil (jh ih0 r jh eh1 n t iy0))
("girgis" nil (g er1 g ih0 s))
("girija" nil (g ih1 r iy1 jh ax))
("girl" nil (g er1 l))
("girlfriend" nil (g er1 l f r eh1 n d))
("girlfriends" nil (g er1 l f r eh1 n d z))
("girlie" nil (g er1 l iy0))
("girlish" nil (g er1 l ih0 sh))
("girlishly" nil (g er1 l ih0 sh l iy0))
("girls" nil (g er1 l z))
("girobank" nil (g ih1 r ow0 b ae1 ng k))
("girod" nil (zh er0 aa1 d))
("giroir" nil (zh ay0 r w aa1 r))
("girolamo" nil (jh ih0 r ow0 l aa1 m ow0))
("giroldi" nil (g ih0 r ow1 l d iy0))
("giron" nil (g ao1 r ax n))
("girouard" nil (zh ay1 r uw0 er0 d))
("giroux" nil (g ih0 r uw1))
("girozentrale" nil (g ih0 r ow1 z ax n t r aa1 l))
("girsky" nil (g er1 s k iy0))
("girt" nil (g er1 t))
("girten" nil (g er1 t ax n))
("girth" nil (g er1 th))
("girtman" nil (g er1 t m ax n))
("girton" nil (g er1 t ax n))
("girvan" nil (g er1 v ax n))
("girven" nil (g er1 v ax n))
("girvin" nil (g er1 v ih0 n))
("giscard" nil (g ih0 s k aa1 r d))
("gisclair" nil (zh ih0 s k l eh1 r))
("gisela" nil (g iy1 z ax l ax))
("gisella" nil (jh ih0 s eh1 l ax))
("giselle" nil (zh ih0 s eh1 l))
("gish" nil (jh ih1 sh))
("gisi" nil (jh iy1 s iy0))
("gisler" nil (g ih1 s ax l er0))
("gismondi" nil (jh iy0 s m ow1 n d iy0))
("gissendanner" nil (g ih1 s ih0 n d ax n er0))
("gissi" nil (g ih1 s iy0))
("gist" nil (jh ih1 s t))
("gitana" nil (jh iy0 t ae1 n ax))
("gitano" nil (g ih0 t aa1 n ow0))
("gitchell" nil (jh ih1 ch ax l))
("githa" nil (jh ih1 dh ax))
("githens" nil (g ih1 th ax n z))
("gitlin" nil (jh ih1 t l ih0 n))
("gittelman" nil (g ih1 t ax l m ax n))
("gittens" nil (g ih1 t ax n z))
("gitter" nil (g ih1 t er0))
("gittings" nil (jh ih1 t ih0 ng z))
("gittins" nil (jh ih1 t ih0 n z))
("gittis" nil (jh ih1 t ax s))
("gittleman" nil (g ih1 t ax l m ax n))
("gittler" nil (g ih1 t l er0))
("gitto" nil (jh iy1 t ow0))
("giudice" nil (jh uw1 d ih0 s iy0))
("giudici" nil (jh uw1 d ih0 ch iy0))
("giuffre" nil (jh uw1 f r iy0))
("giuffrida" nil (jh uw1 f r iy0 d ax))
("giuliani" nil (jh uw1 l iy0 aa1 n iy0))
("giuliano" nil (jh uw1 l iy0 aa1 n ow0))
("giulio" nil (jh uw1 l iy0 ow0))
("giunta" nil (jh uw1 n t ax))
("giurescu" nil (jh uw1 r eh1 s k uw0))
("giuseppe" nil (jh ih0 s eh1 p iy0))
("giusti" nil (jh uw1 s t iy0))
("giusto" nil (jh uw1 s t ow0))
("givan" nil (g ih1 v ax n))
("give" nil (g ih1 v))
("giveaway" nil (g ih1 v ax w ey1))
("giveaways" nil (g ih1 v ax w ey1 z))
("givebacks" nil (g ay1 v b ae1 k s))
("given" nil (g ih1 v ax n))
("givenchy" nil (g ih0 v eh1 n ch iy0))
("givenness" nil (g ih1 v ax n ax s))
("givens" nil (g ih1 v ax n z))
("giver" nil (g ih1 v er0))
("givers" nil (g ih1 v er0 z))
("gives" nil (g ih1 v z))
("giveth" nil (g ih1 v eh0 th))
("givhan" nil (g ih1 v hh ax n))
("giving" nil (g ih1 v ih0 ng))
("givins" nil (g ih1 v ih0 n z))
("givler" nil (g ih1 v l er0))
("gizbert" nil (g ih1 z b er0 t))
("gizmo" nil (g ih1 z m ow1))
("gizmos" nil (g ih1 z m ow0 z))
("gizzard" nil (g ih1 z er0 d))
("gizzi" nil (jh iy1 t s iy0))
("gjelten" nil (jh eh1 l t ax n))
("gjerde" nil (jh er1 d))
("glaab" nil (g l aa1 b))
("glab" nil (g l ae1 b))
("glace" nil (g l ey1 s))
("glacial" nil (g l ey1 sh ax l))
("glaciated" nil (g l ey1 sh iy0 ey1 t ax d))
("glaciation" nil (g l ey1 sh iy0 ey1 sh ax n))
("glacier" nil (g l ey1 sh er0))
("glaciers" nil (g l ey1 sh er0 z))
("glacis" nil (g l ey1 s ax s))
("glacken" nil (g l ae1 k ax n))
("glackin" nil (g l ae1 k ih0 n))
("glad" nil (g l ae1 d))
("gladd" nil (g l ae1 d))
("gladden" nil (g l ae1 d ax n))
("gladding" nil (g l ae1 d ih0 ng))
("glade" nil (g l ey1 d))
("gladfelter" nil (g l ae1 d f eh1 l t er0))
("gladhill" nil (g l ae1 d hh ih1 l))
("gladiator" nil (g l ae1 d iy0 ey1 t er0))
("gladiators" nil (g l ae1 d iy0 ey1 t er0 z))
("gladieux" nil (g l ae1 d iy0 ow0))
("gladiolus" nil (g l ae1 d iy0 ow1 l ax s))
("gladis" nil (g l ae1 d ih0 s))
("gladish" nil (g l ae1 d ih0 sh))
("gladje" nil (g l ae1 d jh iy1))
("gladly" nil (g l ae1 d l iy0))
("gladman" nil (g l ae1 d m ax n))
("gladney" nil (g l ae1 d n iy0))
("gladson" nil (g l ae1 d s ax n))
("gladstein" nil (g l ae1 d s t ay1 n))
("gladstone" nil (g l ae1 d s t ow1 n))
("gladstones" nil (g l ae1 d s t ow1 n z))
("gladu" nil (g l ey1 d uw0))
("gladwell" nil (g l ae1 d w eh1 l))
("gladwin" nil (g l ae1 d w ih0 n))
("gladys" nil (g l ae1 d ih0 s))
("glaeser" nil (g l ey1 z er0))
("glahn" nil (g l ae1 n))
("glamor" nil (g l ae1 m er0))
("glamorous" nil (g l ae1 m er0 ax s))
("glamour" nil (g l ae1 m er0))
("glance" nil (g l ae1 n s))
("glanced" nil (g l ae1 n s t))
("glances" nil (g l ae1 n s ih0 z))
("glancing" nil (g l ae1 n s ih0 ng))
("glancy" nil (g l ae1 n s iy0))
("gland" nil (g l ae1 n d))
("glander" nil (g l ae1 n d er0))
("glandon" nil (g l ae1 n d ax n))
("glands" nil (g l ae1 n d z))
("glandular" nil (g l ae1 n jh ax l er0))
("glanton" nil (g l ae1 n t ax n))
("glantz" nil (g l ae1 n t s))
("glanville" nil (g l ae1 n v ih1 l))
("glanz" nil (g l ae1 n z))
("glanzer" nil (g l ae1 n z er0))
("glanzman" nil (g l ae1 n z m ax n))
("glare" nil (g l eh1 r))
("glared" nil (g l eh1 r d))
("glares" nil (g l eh1 r z))
("glaring" nil (g l eh1 r ih0 ng))
("glaris" nil (g l eh1 r ih0 s))
("glas" nil (g l ae1 s))
("glasby" nil (g l ae1 s b iy0))
("glasco" nil (g l aa1 s k ow0))
("glascock" nil (g l ae1 s k ax k))
("glascoe" nil (g l ae1 s k ow0))
("glaser" nil (g l ey1 z er0))
("glasgow" nil (g l ae1 s k ow1))
("glasheen" nil (g l ax sh iy1 n))
("glashow" nil (g l ae1 sh aw0))
("glasner" nil (g l ae1 s n er0))
("glasnost" nil (g l ae1 s n aa0 s t))
("glasow" nil (g l ey1 z ow0))
("glasper" nil (g l ae1 s p er0))
("glaspie" nil (g l ae1 s p iy0))
("glaspy" nil (g l ae1 s p iy0))
("glass" nil (g l ae1 s))
("glassberg" nil (g l ae1 s b er0 g))
("glassblower" nil (g l ae1 s b l ow1 er0))
("glassblowing" nil (g l ae1 s b l ow1 ih0 ng))
("glassburn" nil (g l ae1 s b er1 n))
("glassco" nil (g l ae1 s k ow0))
("glasscock" nil (g l ae1 s k aa1 k))
("glassed" nil (g l ae1 s t))
("glasser" nil (g l ae1 s er0))
("glasses" nil (g l ae1 s ax z))
("glassey" nil (g l ae1 s iy0))
("glassford" nil (g l ae1 s f ao0 r d))
("glasslike" nil (g l ae1 s l ay1 k))
("glassmaker" nil (g l ae1 s m ey1 k er0))
("glassmaking" nil (g l ae1 s m ey1 k ih0 ng))
("glassman" nil (g l ae1 s m ae1 n))
("glassmeyer" nil (g l ae1 s m ay0 er0))
("glassner" nil (g l ae1 s n er0))
("glasson" nil (g l ae1 s ax n))
("glassware" nil (g l ae1 s w eh1 r))
("glassy" nil (g l ae1 s iy0))
("glastetter" nil (g l ae1 s t ih0 t er0))
("glatfelter" nil (g l ae1 t f ih0 l t er0))
("glatt" nil (g l ae1 t))
("glatz" nil (g l ae1 t s))
("glatzer" nil (g l ey1 t z er0))
("glaub" nil (g l ao1 b))
("glauber" nil (g l aw1 b er0))
("glaucoma" nil (g l ao0 k ow1 m ax))
("glaude" nil (g l ao1 d))
("glaus" nil (g l ao1 z))
("glauser" nil (g l aw1 s er0))
("glavan" nil (g l ey1 v ax n))
("glavin" nil (g l ae1 v ih0 n))
("glawe" nil (g l ao1))
("glaxo" nil (g l ae1 k s ow0))
("glaza" nil (g l aa1 z ax))
("glaze" nil (g l ey1 z))
("glazebrook" nil (g l ey1 z b r uh1 k))
("glazed" nil (g l ey1 z d))
("glazener" nil (g l ae1 z iy0 n er0))
("glazer" nil (g l ey1 z er0))
("glazes" nil (g l ey1 z ax z))
("glazier" nil (g l ey1 z iy0 er0))
("glazing" nil (g l ey1 z ih0 ng))
("glazner" nil (g l ae1 z n er0))
("glazunov" nil (g l ae1 z uw0 n aa0 v))
("gleacher" nil (g l iy1 ch er0))
("gleam" nil (g l iy1 m))
("gleamed" nil (g l iy1 m d))
("gleaming" nil (g l iy1 m ih0 ng))
("gleams" nil (g l iy1 m z))
("glean" nil (g l iy1 n))
("gleaned" nil (g l iy1 n d))
("gleason" nil (g l iy1 s ax n))
("gleaton" nil (g l iy1 t ax n))
("gleave" nil (g l iy1 v))
("gleaves" nil (g l iy1 v z))
("gleba" nil (g l iy1 b ax))
("gleckler" nil (g l eh1 k l er0))
("gleda" nil (g l ey1 d ax))
("gledhill" nil (g l eh1 d hh ih1 l))
("glee" nil (g l iy1))
("gleeful" nil (g l iy1 f ax l))
("gleefully" nil (g l iy1 f ax l iy0))
("gleeson" nil (g l iy1 z ax n))
("gleghorn" nil (g l eh1 g hh er0 n))
("gleich" nil (g l ay1 k))
("gleichauf" nil (g l ay1 k ao0 f))
("gleim" nil (g l iy1 m))
("gleisner" nil (g l ay1 s n er0))
("glemp" nil (g l eh1 m p))
("glen" nil (g l eh1 n))
("glenayre" nil (g l eh1 n eh1 r))
("glenbrook" nil (g l eh1 n b r uh1 k))
("glenda" nil (g l eh1 n d ax))
("glendale" nil (g l eh1 n d ey1 l))
("glendening" nil (g l eh1 n d ax n ih0 ng))
("glendenning" nil (g l eh1 n d ih0 n ih0 ng))
("glendinning" nil (g l eh1 n d ih0 n ih0 ng))
("glendon" nil (g l eh1 n d ow0 n))
("glenfed" nil (g l eh1 n f eh1 d))
("glengarry" nil (g l eh1 n g eh1 r iy0))
("glenmore" nil (g l eh1 n m ao1 r))
("glenn" nil (g l eh1 n))
("glenna" nil (g l eh1 n ax))
("glennie" nil (g l eh1 n iy0))
("glennon" nil (g l eh1 n ax n))
("glenny" nil (g l eh1 n iy0))
("glens" nil (g l eh1 n z))
("glenview" nil (g l eh1 n v y uw1))
("glenville" nil (g l eh1 n v ih0 l))
("glenwood" nil (g l eh1 n w uh1 d))
("glessner" nil (g l eh1 s n er0))
("glew" nil (g l uw1))
("glib" nil (g l ih1 b))
("glibly" nil (g l ih1 b l iy0))
("glick" nil (g l ih1 k))
("glickenhaus" nil (g l ih1 k ax n hh aw1 s))
("glickman" nil (g l ih1 k m ax n))
("glicksman" nil (g l ih1 k s m ax n))
("glickstein" nil (g l ih1 k s t iy1 n))
("glidden" nil (g l ih1 d ax n))
("glide" nil (g l ay1 d))
("glided" nil (g l ay1 d ih0 d))
("glider" nil (g l ay1 d er0))
("gliders" nil (g l ay1 d er0 z))
("glides" nil (g l ay1 d z))
("glidewell" nil (g l ay1 d w eh1 l))
("gliding" nil (g l ay1 d ih0 ng))
("glimcher" nil (g l ih1 m ch er0))
("glimmer" nil (g l ih1 m er0))
("glimmering" nil (g l ih1 m er0 ih0 ng))
("glimmers" nil (g l ih1 m er0 z))
("glimpse" nil (g l ih1 m p s))
("glimpsed" nil (g l ih1 m p s t))
("glimpses" nil (g l ih1 m p s ih0 z))
("glines" nil (g l ay1 n z))
("glinka" nil (g l ih1 ng k ax))
("glinski" nil (g l ih1 n s k iy0))
("glint" nil (g l ih1 n t))
("glissandi" nil (g l ax s aa1 n d iy0))
("glisson" nil (g l ih1 s ax n))
("glistened" nil (g l ih1 s ax n d))
("glistening" nil (g l ih1 s ax n ih0 ng))
("glitch" nil (g l ih1 ch))
("glitches" nil (g l ih1 ch ih0 z))
("glitter" nil (g l ih1 t er0))
("glittered" nil (g l ih1 t er0 d))
("glittering" nil (g l ih1 t er0 ih0 ng))
("glitters" nil (g l ih1 t er0 z))
("glittery" nil (g l ih1 t er0 iy0))
("glitz" nil (g l ih1 t s))
("glitzy" nil (g l ih1 t s iy0))
("glo" nil (g l ow1))
("gloat" nil (g l ow1 t))
("gloated" nil (g l ow1 t ih0 d))
("gloating" nil (g l ow1 t ih0 ng))
("gloats" nil (g l ow1 t s))
("global" nil (g l ow1 b ax l))
("globalization" nil (g l ow1 b ax l ih0 z ey1 sh ax n))
("globalize" nil (g l ow1 b ax l ay1 z))
("globalized" nil (g l ow1 b ax l ay1 z d))
("globally" nil (g l ow1 b ax l iy0))
("globalstar" nil (g l ow1 b ax l s t aa1 r))
("globcom" nil (g l aa1 b k ax m))
("globe" nil (g l ow1 b))
("globex" nil (g l ow1 b ax k s))
("globo" nil (g l ow1 b ow0))
("globs" nil (g l aa1 b z))
("globular" nil (g l aa1 b y ax l er0))
("globulin" nil (g l aa1 b y ax l ih0 n))
("globulins" nil (g l aa1 b y ax l ih0 n z))
("globus" nil (g l ow1 b ax s))
("glock" nil (g l aa1 k))
("glockenspiel" nil (g l aa1 k ax n s p iy1 l))
("glockner" nil (g l aa1 k n er0))
("glod" nil (g l aa1 d))
("glodowski" nil (g l ax d ao1 f s k iy0))
("gloe" nil (g l ow1))
("gloeckner" nil (g l ow1 k n er0))
("gloede" nil (g l ow1 d))
("glogowski" nil (g l ax g ao1 f s k iy0))
("glomar" nil (g l ow1 m aa0 r))
("glomb" nil (g l aa1 m))
("glomski" nil (g l aa1 m s k iy1))
("gloom" nil (g l uw1 m))
("gloomier" nil (g l uw1 m iy0 er0))
("gloomily" nil (g l uw1 m ax l iy0))
("gloomy" nil (g l uw1 m iy0))
("gloor" nil (g l uh1 r))
("glop" nil (g l aa1 p))
("glor" nil (g l ao1 r))
("glore" nil (g l ao1 r))
("glori" nil (g l ao1 r iy0))
("gloria" nil (g l ao1 r iy0 ax))
("gloriana" nil (g l ao1 r iy0 ae1 n ax))
("gloriane" nil (g l ao1 r iy0 ae1 n))
("glories" nil (g l ao1 r iy0 z))
("glorified" nil (g l ao1 r ax f ay1 d))
("glorify" nil (g l ao1 r ax f ay1))
("glorioso" nil (g l ao0 r iy0 ow1 s ow0))
("glorious" nil (g l ao1 r iy0 ax s))
("gloriously" nil (g l ao1 r iy0 ax s l iy0))
("glory" nil (g l ao1 r iy0))
("gloss" nil (g l ao1 s))
("glossary" nil (g l ao1 s er0 iy0))
("glossed" nil (g l ao1 s t))
("glosser" nil (g l ao1 s er0))
("glosses" nil (g l ao1 s ih0 z))
("glossier" nil (g l ao1 s iy0 er0))
("glossman" nil (g l ao1 s m ax n))
("glosson" nil (g l aa1 s ax n))
("glossy" nil (g l ao1 s iy0))
("gloster" nil (g l aa1 s t er0))
("glotfelty" nil (g l aa1 t f ih0 l t iy0))
("glottal" nil (g l aa1 t ax l))
("glottis" nil (g l aa1 t ax s))
("glotzbach" nil (g l aa1 t s b aa0 k))
("gloucester" nil (g l ao1 s t er0))
("glove" nil (g l ah1 v))
("gloved" nil (g l ah1 v d))
("glover" nil (g l ah1 v er0))
("gloves" nil (g l ah1 v z))
("glow" nil (g l ow1))
("glowacki" nil (g l aw0 aa1 t s k iy0))
("glowed" nil (g l ow1 d))
("glowered" nil (g l aw1 er0 d))
("glowering" nil (g l aw1 er0 ih0 ng))
("glowing" nil (g l ow1 ih0 ng))
("glowingly" nil (g l ow1 ih0 ng l iy0))
("glows" nil (g l ow1 z))
("gloyd" nil (g l oy1 d))
("gluck" nil (g l ah1 k))
("gluckman" nil (g l ah1 k m ax n))
("glucksman" nil (g l ah1 k s m ax n))
("glucose" nil (g l uw1 k ow1 s))
("glucosides" nil (g l uw1 k ax s ay1 d z))
("glue" nil (g l uw1))
("glueck" nil (g l uw1 k))
("glued" nil (g l uw1 d))
("glues" nil (g l uw1 z))
("glum" nil (g l ah1 m))
("glumly" nil (g l ah1 m l iy0))
("glunt" nil (g l ah1 n t))
("gluntz" nil (g l ah1 n t s))
("glunz" nil (g l ah1 n z))
("gluskin" nil (g l ah1 s k ih0 n))
("glut" nil (g l ah1 t))
("glutamate" nil (g l uw1 t ax m ey1 t))
("glutamic" nil (g l uw0 t ae1 m ih0 k))
("glutaric" nil (g l uw0 t ae1 r ih0 k))
("gluten" nil (g l uw1 t ax n))
("gluth" nil (g l uw1 th))
("gluts" nil (g l ah1 t s))
("glutted" nil (g l ah1 t ih0 d))
("gluttonous" nil (g l ah1 t ax n ax s))
("gluttons" nil (g l ah1 t ax n z))
("gluttony" nil (g l ah1 t ax n iy0))
("glycel" nil (g l ih1 s ax l))
("glycerol" nil (g l ih1 s er0 ow1 l))
("glycine" nil (g l ay1 s iy1 n))
("glycogen" nil (g l ay1 k ax jh ih0 n))
("glycol" nil (g l ay1 k ao1 l))
("glycomed" nil (g l ay1 k ow1 m d))
("glycoside" nil (g l ay1 k ax s ay1 d))
("glymph" nil (g l ih1 m f))
("glyn" nil (g l ih1 n))
("glyndebourne" nil (g l ih1 n d b ao1 r n))
("glynis" nil (g l ih1 n ih0 s))
("glynn" nil (g l ih1 n))
("glynnie" nil (g l ih1 n iy0))
("glynnis" nil (g l ih1 n ih0 s))
("gmbh" nil (g ax m))
("gnagey" nil (n ae1 jh iy0))
("gnaizda" nil (n ey1 z d ax))
("gnann" nil (n ae1 n))
("gnarled" nil (n aa1 r l d))
("gnashing" nil (n ae1 sh ih0 ng))
("gnats" nil (n ae1 t s))
("gnau" nil (n aw1))
("gnaw" nil (n ao1))
("gnawed" nil (n ao1 d))
("gnawing" nil (n ao1 ih0 ng))
("gnaws" nil (n ao1 z))
("gnc" nil (n k))
("gneiss" nil (n ay1 s))
("gnomes" nil (n ow1 m z))
("gnomic" nil (n ow1 m ih0 k))
("gnomonic" nil (n ow0 m aa1 n ih0 k))
("gnosticism" nil (n aa1 s t ih0 s ih1 z ax m))
("gnu" nil (n uw1))
("go" nil (g ow1))
("goa" nil (g ow1 ax))
("goad" nil (g ow1 d))
("goaded" nil (g ow1 d ih0 d))
("goading" nil (g ow1 d ih0 ng))
("goal" nil (g ow1 l))
("goalie" nil (g ow1 l iy0))
("goalkeeper" nil (g ow1 l k iy1 p er0))
("goalpost" nil (g ow1 l p ow1 s t))
("goals" nil (g ow1 l z))
("goan" nil (g ow1 n))
("goans" nil (g ow1 n z))
("goar" nil (g ao1 r))
("goard" nil (g ao1 r d))
("goat" nil (g ow1 t))
("goatee" nil (g ow1 t iy1))
("goates" nil (g ow1 ey0 t s))
("goatley" nil (g ow1 t l iy0))
("goats" nil (g ow1 t s))
("gob" nil (g aa1 b))
("gobain" nil (g ow1 b ih0 n))
("gobble" nil (g aa1 b ax l))
("gobbled" nil (g aa1 b ax l d))
("gobbledygook" nil (g aa1 b ax l d iy0 g uh1 k))
("gobblers" nil (g aa1 b ax l er0 z))
("gobbling" nil (g aa1 b ax l ih0 ng))
("gobel" nil (g ow1 b ax l))
("goben" nil (g aa1 b ax n))
("gober" nil (g ow1 b er0))
("goberstein" nil (g ow1 b er0 s t iy1 n))
("gobert" nil (g aa1 b er0 t))
("gobi" nil (g ow1 b iy0))
("gobie" nil (g ow1 b iy0))
("gobin" nil (g ow1 b ih0 n))
("gobinot" nil (g ow1 b ih0 n aa0 t))
("gobioff" nil (g ow1 b iy0 ao0 f))
("goble" nil (g ow1 b ax l))
("goblet" nil (g aa1 b l ax t))
("goblirsch" nil (g aa1 b l er0 sh))
("gobs" nil (g aa1 b z))
("gochenour" nil (g aa1 sh ih0 n uh0 r))
("gochnauer" nil (g aa1 k n aw0 er0))
("gochnour" nil (g ax sh n uh1 r))
("gocke" nil (g aa1 k))
("gockel" nil (g aa1 k ax l))
("gockley" nil (g aa1 k l iy0))
("god" nil (g aa1 d))
("goda" nil (g ow1 d ax))
("godar" nil (g ax d aa1 r))
("godard" nil (g ax d aa1 r d))
("godart" nil (g aa1 d aa0 r t))
("godbee" nil (g aa1 d b iy1))
("godbey" nil (g aa1 d b iy0))
("godbold" nil (g aa1 d b ow1 l d))
("godbolt" nil (g aa1 d b ow1 l t))
("godbout" nil (g aa1 d b aw1 t))
("godby" nil (g aa1 d b iy0))
("goddamn" nil (g aa1 d d ae1 m))
("goddard" nil (g aa1 d er0 d))
("goddart" nil (g aa1 d aa0 r t))
("goddeau" nil (g aa1 d ow1))
("godden" nil (g aa1 d ax n))
("goddess" nil (g aa1 d ax s))
("goddesses" nil (g aa1 d ax s ih0 z))
("godding" nil (g aa1 d ih0 ng))
("godek" nil (g ow1 d ih0 k))
("godette" nil (g ax d eh1 t))
("godfather" nil (g aa1 d f aa1 dh er0))
("godfrey" nil (g aa1 d f r iy0))
("godhead" nil (g aa1 d hh eh1 d))
("godin" nil (g ow1 d ih0 n))
("godina" nil (g ow0 d iy1 n ax))
("godine" nil (g ow0 d iy1 n))
("godines" nil (g aa1 d ay0 n z))
("godinez" nil (g ow0 d iy1 n eh0 z))
("goding" nil (g ow1 d ih0 ng))
("godino" nil (g ow0 d iy1 n ow0))
("godiva" nil (g ax d ay1 v ax))
("godkin" nil (g aa1 d k ih0 n))
("godleski" nil (g ax d l eh1 s k iy0))
("godless" nil (g aa1 d l ax s))
("godlewski" nil (g ax d l eh1 f s k iy0))
("godley" nil (g aa1 d l iy0))
("godlove" nil (g aa1 d l ah1 v))
("godly" nil (g aa1 d l iy0))
("godman" nil (g aa1 d m ax n))
("godmother" nil (g aa1 d m ah1 dh er0))
("godot" nil (g ow1 d ax t))
("godown" nil (g ow1 d aw1 n))
("godoy" nil (g aa1 d oy0))
("godrej" nil (g aa1 d r ey0))
("gods" nil (g aa1 d z))
("godsell" nil (g aa1 d s ax l))
("godsend" nil (g aa1 d s eh1 n d))
("godsey" nil (g aa1 d z iy0))
("godshalk" nil (g aa1 d sh ax k))
("godshall" nil (g aa1 d sh ax l))
("godunov" nil (g ow1 d uw0 n aa0 v))
("godwin" nil (g aa1 d w ih0 n))
("godzilla" nil (g aa1 d z ih1 l ax))
("goe" nil (g ow1))
("goebbels" nil (g ow1 b ax l z))
("goebel" nil (g ow1 b ax l))
("goecke" nil (g ow1 k))
("goedde" nil (g ow1 d))
("goede" nil (g ow1 d))
("goedecke" nil (g ow1 d ih0 k))
("goeden" nil (g ow1 d ax n))
("goedert" nil (g ow1 d er0 t))
("goedken" nil (g ow1 d k ax n))
("goeglein" nil (g ow1 g l ay1 n))
("goehner" nil (g ow1 n er0))
("goehring" nil (g ao1 r ih0 ng))
("goeke" nil (g ow1 k))
("goeken" nil (g ow1 k ax n))
("goeller" nil (g ow1 l er0))
("goellner" nil (g ow1 l n er0))
("goelz" nil (g ow1 l z))
("goelzer" nil (g ow1 l z er0))
("goeman" nil (g ow1 m ax n))
("goemon" nil (g ow1 ey1 m ax n))
("goen" nil (g ow1 n))
("goens" nil (g ow1 n z))
("goepfert" nil (g ow1 p f er0 t))
("goer" nil (g ow1 er0))
("goergen" nil (g ao1 r g ax n))
("goering" nil (g ow1 er0 ih0 ng))
("goerke" nil (g ao1 r k))
("goerlitz" nil (g ao1 r l ih0 t s))
("goerner" nil (g ao1 r n er0))
("goers" nil (g ow1 er0 z))
("goertz" nil (g ao1 r t s))
("goertzen" nil (g ao1 r t z ax n))
("goes" nil (g ow1 z))
("goeser" nil (g ow1 z er0))
("goetabanken" nil (g ow1 t ax b ae1 ng k ax n))
("goethals" nil (g ow1 th ax l z))
("goethe" nil (g ow1 th))
("goetsch" nil (g ow1 ch))
("goette" nil (g eh1 t))
("goettel" nil (g ow1 t ax l))
("goetting" nil (g ow1 t ih0 ng))
("goettl" nil (g ow1 t ax l))
("goettsch" nil (g ow1 t ch))
("goetz" nil (g eh1 t s))
("goetze" nil (g ow1 t z))
("goetzinger" nil (g ow1 t z ih0 ng er0))
("goetzke" nil (g ow1 t s k iy0))
("goetzman" nil (g ow1 t s m ax n))
("goewey" nil (g aa1 uw0 iy0))
("goff" nil (g ao1 f))
("goffin" nil (g aa1 f ih0 n))
("goffinet" nil (g aa1 f ih0 n ih0 t))
("goffman" nil (g aa1 f m ax n))
("goffney" nil (g aa1 f n iy0))
("goffredo" nil (g ow0 f r ey1 d ow0))
("goforth" nil (g ow1 f ao1 r th))
("goga" nil (g ow1 g ax))
("gogan" nil (g ow1 g ax n))
("gogel" nil (g ow1 g ax l))
("goggans" nil (g aa1 g ax n z))
("goggin" nil (g aa1 g ih0 n))
("goggins" nil (g aa1 g ih0 n z))
("goggle" nil (g aa1 g ax l))
("goggles" nil (g aa1 g ax l z))
("gogh" nil (g ow1))
("goglia" nil (g aa1 g l iy0 ax))
("gogo" nil (g ow1 g ow1))
("goguen" nil (g ow1 g w ax n))
("goh" nil (g ow1))
("goheen" nil (g ax hh iy1 n))
("gohl" nil (g ow1 l))
("gohlke" nil (g aa1 k))
("gohman" nil (g ow1 m ax n))
("gohn" nil (g aa1 n))
("gohr" nil (g aa1 r))
("goin" nil (g oy1 n))
("goines" nil (g oy1 n z))
("going" nil (g ow1 ih0 ng))
("goings" nil (g ow1 ih0 ng z))
("goins" nil (g oy1 n z))
("goiter" nil (g oy1 t er0))
("goizueta" nil (g oy1 z w ey1 t ax))
("gokey" nil (g ow1 k iy1))
("gola" nil (g ow1 l ax))
("golab" nil (g ow1 l ae1 b))
("golan" nil (g ow1 l ax n))
("goland" nil (g ow1 l ax n d))
("golas" nil (g ow1 l ax z))
("golaszewski" nil (g ax l ax sh eh1 f s k iy0))
("golay" nil (g aa1 l ey0))
("golba" nil (g ow1 l b ax))
("golberg" nil (g ow1 l b er0 g))
("golconda" nil (g aa0 l k aa1 n d ax))
("gold" nil (g ow1 l d))
("golda" nil (g ow1 l d ax))
("goldade" nil (g ow1 l d ey1 d))
("goldammer" nil (g ow1 l d ax m er0))
("goldbach" nil (g ow1 l d b aa1 k))
("goldbaum" nil (g ow1 l d b aw1 m))
("goldbeck" nil (g ow1 l d b eh1 k))
("goldberg" nil (g ow1 l d b er0 g))
("goldberger" nil (g ow1 l d b er0 g er0))
("goldblatt" nil (g ow1 l d b l ax t))
("goldblum" nil (g ow1 l d b l ax m))
("goldcor" nil (g ow1 l d k ao1 r))
("goldcreek" nil (g ow1 l d k r iy1 k))
("goldcrest" nil (g ow1 l d k r eh1 s t))
("golde" nil (g ow1 l d))
("golden" nil (g ow1 l d ax n))
("goldenberg" nil (g ow1 l d ax n b er0 g))
("goldenrod" nil (g ow1 l d ax n r aa1 d))
("goldenson" nil (g ow1 l d ax n s ax n))
("goldenstein" nil (g ow1 l d ax n s t ay1 n))
("golder" nil (g ow1 l d er0))
("goldey" nil (g ow1 l d iy0))
("goldfarb" nil (g ow1 l d f aa1 r b))
("goldfeder" nil (g ow1 l d f eh1 d er0))
("goldfield" nil (g ow1 l d f iy1 l d))
("goldfields" nil (g ow1 l d f iy1 l d z))
("goldfinch" nil (g ow1 l d f ih1 n ch))
("goldfinches" nil (g ow1 l d f ih1 n ch ih0 z))
("goldfine" nil (g ow1 l d f ay1 n))
("goldfinger" nil (g ow1 l d f ih1 ng g er0))
("goldfish" nil (g ow1 l d f ih1 sh))
("goldfus" nil (g ow1 l d f ah1 s))
("goldhammer" nil (g ow1 l d hh ae1 m er0))
("goldie" nil (g ow1 l d iy0))
("goldin" nil (g ow1 l d ih0 n))
("golding" nil (g ow1 l d ih0 ng))
("goldinger" nil (g ow1 l d ih0 ng er0))
("goldman" nil (g ow1 l d m ax n))
("goldmann" nil (g ow1 l d m ax n))
("goldmark" nil (g ow1 l d m aa1 r k))
("goldmine" nil (g ow1 l d m ay1 n))
("goldner" nil (g ow1 l d n er0))
("goldome" nil (g ow1 l d ow1 m))
("goldress" nil (g ow1 l d r eh1 s))
("goldrick" nil (g ow1 l d r ih0 k))
("goldring" nil (g ow1 l d r ih1 ng))
("golds" nil (g ow1 l d z))
("goldsberry" nil (g ow1 l d z b eh1 r iy0))
("goldsborough" nil (g ow1 l d z b er0 ow0))
("goldsby" nil (g ow1 l d z b iy0))
("goldschmid" nil (g ow1 l d sh m ih1 d))
("goldschmidt" nil (g ow1 l d sh m ih1 t))
("goldsmith" nil (g ow1 l d s m ih1 th))
("goldson" nil (g ow1 l d s ax n))
("goldstar" nil (g ow1 l d s t aa1 r))
("goldstein" nil (g ow1 l d s t ay1 n))
("goldstock" nil (g ow1 l d s t aa1 k))
("goldston" nil (g ow1 l d s t ax n))
("goldstone" nil (g ow1 l d s t ow1 n))
("goldstrike" nil (g ow1 l d s t r ay1 k))
("goldsworthy" nil (g ow1 l d z w er1 dh iy0))
("goldthwaite" nil (g ow1 l d th w ey1 t))
("goldwasser" nil (g ow1 l d w ao0 s er0))
("goldwater" nil (g ow1 l d w ao1 t er0))
("goldwin" nil (g ow1 l d w ih0 n))
("goldwire" nil (g ow1 l d w ay1 r))
("goldwyn" nil (g ow1 l d w ih0 n))
("goldy" nil (g ow1 l d iy0))
("golec" nil (g ow1 l ih0 k))
("golem" nil (g ow1 l ax m))
("goleman" nil (g ow1 l m ax n))
("golembeski" nil (g ax l ih0 m b eh1 s k iy0))
("golembiewski" nil (g ax l ih0 m b iy0 eh1 f s k iy0))
("golen" nil (g aa1 l ax n))
("goley" nil (g ow1 l iy0))
("golf" nil (g aa1 l f))
("golfed" nil (g aa1 l f t))
("golfer" nil (g aa1 l f er0))
("golfers" nil (g aa1 l f er0 z))
("golfing" nil (g aa1 l f ih0 ng))
("golfs" nil (g aa1 l f s))
("golgi" nil (g ow1 l jh iy0))
("golgo" nil (g ow1 l g ow0))
("goliath" nil (g ax l ay1 ax th))
("goliaths" nil (g ow1 l iy0 ae0 th s))
("golightly" nil (g ow1 l ay1 t l iy0))
("golinski" nil (g ax l ih1 n s k iy0))
("golisano" nil (g ow1 l ih0 s aa1 n ow0))
("golkar" nil (g ow1 l k aa0 r))
("goll" nil (g aa1 l))
("golla" nil (g aa1 l ax))
("golladay" nil (g aa1 l ax d ey1))
("golle" nil (g aa1 l))
("goller" nil (g aa1 l er0))
("golliday" nil (g aa1 l iy0 d ey0))
("golliher" nil (g aa1 l ih0 hh er0))
("gollnick" nil (g aa1 l n ih0 k))
("gollust" nil (g ow1 l ax s t))
("golly" nil (g aa1 l iy0))
("golob" nil (g ow1 l ax b))
("golomb" nil (g aa1 l ax m))
("golonka" nil (g ow0 l ow1 ng k ax))
("goloven" nil (g ow1 l ax v ax n))
("golphin" nil (g ow1 l f ih0 n))
("golson" nil (g ow1 l s ax n))
("golston" nil (g ow1 l s t ax n))
("goltz" nil (g ow1 l t s))
("golub" nil (g ow1 l ax b))
("golubski" nil (g ax l ah1 b s k iy0))
("golz" nil (g ow1 l z))
("goma" nil (g ow1 m ax))
("gombar" nil (g ax m b aa1 r))
("gomberg" nil (g aa1 m b er0 g))
("gombert" nil (g aa1 m b er0 t))
("gombos" nil (g ow1 m b ow0 z))
("gomer" nil (g ow1 m er0))
("gomery" nil (g ow1 m er0 iy0))
("gomes" nil (g ow1 m eh1 z))
("gomez" nil (g ow1 m eh0 z))
("gomillion" nil (g aa1 m ih0 l y ax n))
("gomoll" nil (g aa1 m ax l))
("gomory" nil (g ow1 m er0 iy0))
("gonaives" nil (g ow0 n ay1 v z))
("goncalves" nil (g ow0 n k aa1 l v eh0 s))
("gonce" nil (g aa1 n s))
("goncharov" nil (g aa1 n ch er0 aa0 v))
("gond" nil (g aa1 n d))
("gonda" nil (g aa1 n d ax))
("gondek" nil (g aa1 n d ih0 k))
("gonder" nil (g aa1 n d er0))
("gondola" nil (g aa1 n d ax l ax))
("gondolas" nil (g aa1 n d ax l ax z))
("gondolier" nil (g aa1 n d ax l ih1 r))
("gone" nil (g ao1 n))
("gonet" nil (g ow1 n ih0 t))
("gong" nil (g ao1 ng))
("gongaware" nil (g aa1 ng g ax w eh0 r))
("gongora" nil (g ow0 ng g ao1 r ax))
("gongs" nil (g ao1 ng z))
("gonia" nil (g ow1 n iy0 ax))
("goniometer" nil (g ow1 n iy0 aa1 m ax t er0))
("gonium" nil (g ow1 n iy0 ax m))
("gonna" nil (g aa1 n ax))
("gonnella" nil (g ow0 n eh1 l ax))
("gonnerman" nil (g aa1 n er0 m ax n))
("gonococcal" nil (g aa1 n ax k aa1 k ax l))
("gonorrhea" nil (g aa1 n er0 iy1 ax))
("gonsales" nil (g ow1 n s ey0 l z))
("gonsalez" nil (g ow0 n s aa1 l eh0 z))
("gonsalves" nil (g ow0 n s aa1 l v eh0 s))
("gonser" nil (g aa1 n s er0))
("gonsoulin" nil (g aa1 n s aw0 l ih0 n))
("gonterman" nil (g aa1 n t er0 m ax n))
("gonya" nil (g ow1 n y ax))
("gonyea" nil (g ow1 n y eh0))
("gonyer" nil (g ow1 n y er0))
("gonyo" nil (g ow1 n y ow0))
("gonzales" nil (g aa0 n z aa1 l ax s))
("gonzalez" nil (g aa0 n z aa1 l ax z))
("gonzalo" nil (g aa0 n z aa1 l ow0))
("gonzo" nil (g aa1 n z ow0))
("goo" nil (g uw1))
("gooch" nil (g uw1 ch))
("good" nil (g uh1 d))
("gooda" nil (g uh1 d ax))
("goodale" nil (g uh1 d ey1 l))
("goodall" nil (g uh1 d ao1 l))
("goodbar" nil (g uh1 d b aa1 r))
("goodbread" nil (g uh1 d b r eh1 d))
("goodby" nil (g uh1 d b ay1))
("goodbye" nil (g uh1 d b ay1))
("goodchild" nil (g uh1 d ch ay1 l d))
("goode" nil (g uh1 d))
("goodell" nil (g uh1 d eh0 l))
("gooden" nil (g uh1 d ax n))
("goodenough" nil (g uh1 d ax n aw1))
("goodenow" nil (g uh1 d eh0 n ow0))
("gooder" nil (g uh1 d er0))
("gooderham" nil (g uh1 d er0 hh ae1 m))
("goodfellow" nil (g uh1 d f eh1 l ow0))
("goodfriend" nil (g uh1 d f r eh1 n d))
("goodgame" nil (g uh1 d g ey1 m))
("goodhart" nil (g uh1 d hh aa1 r t))
("goodheart" nil (g uh1 d hh aa1 r t))
("goodhue" nil (g uh1 d hh y uw1))
("goodies" nil (g uh1 d iy0 z))
("goodin" nil (g uh1 d ax n))
("goodine" nil (g uh1 d ay0 n))
("gooding" nil (g uh1 d ih0 ng))
("goodison" nil (g uh1 d ih0 s ax n))
("goodkin" nil (g uh1 d k ih1 n))
("goodlet" nil (g uh1 d l eh1 t))
("goodlett" nil (g uh1 d l eh1 t))
("goodley" nil (g uh1 d l iy0))
("goodlin" nil (g uh1 d l ih0 n))
("goodling" nil (g uh1 d l ih0 ng))
("goodloe" nil (g uh1 d l ow1))
("goodlow" nil (g uh1 d l ow1))
("goodly" nil (g uh1 d l iy0))
("goodman" nil (g uh1 d m ax n))
("goodner" nil (g uh1 d n er0))
("goodness" nil (g uh1 d n ax s))
("goodnight" nil (g uh1 d n ay1 t))
("goodnough" nil (g uh1 d n ah1 f))
("goodnow" nil (g uh1 d n ow0))
("goodpaster" nil (g uh1 d p ae1 s t er0))
("goodpasture" nil (g uh1 d p ae1 s ch er0))
("goodreau" nil (g uh1 d r ow1))
("goodrich" nil (g uh1 d r ih1 ch))
("goodrick" nil (g uh1 d r ih0 k))
("goodridge" nil (g uh1 d r ih1 jh))
("goodroe" nil (g uh1 d r ow1))
("goodrow" nil (g uh1 d r ow1))
("goodrum" nil (g uh1 d r ah1 m))
("goods" nil (g uh1 d z))
("goodsell" nil (g uh1 d s eh1 l))
("goodson" nil (g uh1 d s ax n))
("goodspeed" nil (g uh1 d s p iy1 d))
("goodstein" nil (g uh1 d s t ay1 n))
("goodtab" nil (g uh1 d t ae1 b))
("goodwill" nil (g uh1 d w ih1 l))
("goodwin" nil (g uh1 d w ih0 n))
("goodwine" nil (g uh1 d w ay1 n))
("goodwyn" nil (g uh1 d w ih1 n))
("goody" nil (g uh1 d iy0))
("goodyear" nil (g uh1 d y ih0 r))
("gooey" nil (g uw1 iy0))
("goof" nil (g uw1 f))
("goofed" nil (g uw1 f t))
("goofing" nil (g uw1 f ih0 ng))
("goofs" nil (g uw1 f s))
("goofy" nil (g uw1 f iy0))
("googe" nil (g uw1 jh))
("googin" nil (g uw1 g ih0 n))
("googins" nil (g uw1 g ih0 n z))
("gook" nil (g uh1 k))
("gookin" nil (g uh1 k ih0 n))
("goold" nil (g uw1 l d))
("gooley" nil (g uw1 l iy0))
("goolsby" nil (g uw1 l s b iy0))
("goon" nil (g uw1 n))
("goonan" nil (g uw1 n ax n))
("goons" nil (g uw1 n z))
("goop" nil (g uw1 p))
("goos" nil (g uw1 z))
("goosby" nil (g uw1 s b iy0))
("goose" nil (g uw1 s))
("gooseberry" nil (g uw1 s b eh1 r iy0))
("goosefish" nil (g uw1 s f ih1 sh))
("goosefoot" nil (g uw1 s f uh1 t))
("goosen" nil (g uw1 s ax n))
("goosey" nil (g uw1 s iy0))
("goossen" nil (g uw1 s ax n))
("goossens" nil (g uw1 s ax n z))
("goostree" nil (g uw0 s t r iy1))
("gootee" nil (g uw1 t iy0))
("gopac" nil (g ow1 p ae1 k))
("gopher" nil (g ow1 f er0))
("gophers" nil (g ow1 f er0 z))
("gora" nil (g ao1 r ax))
("goracke" nil (g ao1 r ax k))
("goradze" nil (g ao1 r aa1 d z ax))
("goral" nil (g ao1 r ax l))
("goralski" nil (g er0 aa1 l s k iy0))
("goranson" nil (g ao1 r ax n s ax n))
("gorazde" nil (g ao1 r aa1 zh d ax))
("gorbachev" nil (g ao1 r b ax ch eh0 v))
("gorbachevs" nil (g ao1 r b ax ch eh0 v z))
("gorby" nil (g ao1 r b iy0))
("gorczyca" nil (g er0 ch ih1 k ax))
("gorczynski" nil (g er0 ch ih1 n s k iy0))
("gorda" nil (g ao1 r d ax))
("gordan" nil (g ao1 r d ax n))
("gorden" nil (g ao1 r d ax n))
("gorder" nil (g ao1 r d er0))
("gordeyev" nil (g ao0 r d ay1 ax v))
("gordian" nil (g ao1 r d iy0 ax n))
("gordie" nil (g ao1 r d iy0))
("gordillo" nil (g ao1 r d ih1 l ow0))
("gordin" nil (g ao1 r d ih0 n))
("gordinier" nil (g ao1 r d ih0 n iy0 er0))
("gordji" nil (g ao1 r jh iy0))
("gordner" nil (g ao1 r d n er0))
("gordon" nil (g ao1 r d ax n))
("gordy" nil (g ao1 r d iy0))
("gore" nil (g ao1 r))
("gorecki" nil (g er0 eh1 t s k iy0))
("gored" nil (g ao1 r d))
("goree" nil (g ao1 r iy1))
("gorelick" nil (g ao1 r ih0 l ih0 k))
("goren" nil (g ao1 r ax n))
("gorenflo" nil (g ao0 r eh1 n f l ow0))
("gores" nil (g ao1 r z))
("gorey" nil (g ao1 r iy0))
("gorgas" nil (g ao1 r g ax z))
("gorge" nil (g ao1 r jh))
("gorgeous" nil (g ao1 r jh ax s))
("gorges" nil (g ao1 r jh ax z))
("gorgon" nil (g ao1 r g ax n))
("gorgone" nil (g ao1 r g ax n))
("gorgonian" nil (g ao0 r g ow1 n iy0 ax n))
("gorgons" nil (g ao1 r g ax n z))
("gorguze" nil (g ao1 r g y uw0 z))
("gorham" nil (g ao1 r ax m))
("gori" nil (g ao1 r iy0))
("goria" nil (g ao1 r iy0 ax))
("gorilla" nil (g er0 ih1 l ax))
("gorillas" nil (g er0 ih1 l ax z))
("gorin" nil (g ao1 r ax n))
("goring" nil (g ao1 r ih0 ng))
("goris" nil (g ao1 r ax s))
("gorka" nil (g ao1 r k ax))
("gorki" nil (g ao1 r k iy0))
("gorky" nil (g ao1 r k iy0))
("gorley" nil (g ao1 r l iy0))
("gorman" nil (g ao1 r m ax n))
("gormley" nil (g ao1 r m l iy0))
("gormly" nil (g ao1 r m l iy0))
("gorney" nil (g ao1 r n iy0))
("gorniak" nil (g ao1 r n iy0 ae0 k))
("gornick" nil (g ao1 r n ih0 k))
("gornik" nil (g ao1 r n ih0 k))
("gornto" nil (g ao1 r n t ow0))
("gorny" nil (g ao1 r n iy0))
("gorospe" nil (g ao0 r ow1 s p ey0))
("gorr" nil (g ao1 r))
("gorrell" nil (g ao0 r ey1 l))
("gorski" nil (g ao1 r s k iy0))
("gorsky" nil (g ao1 r s k iy0))
("gorsline" nil (g ao1 r s l ay1 n))
("gorsuch" nil (g ao1 r s ax ch))
("gort" nil (g ao1 r t))
("gortari" nil (g ao0 r t aa1 r iy0))
("gorter" nil (g ao1 r t er0))
("gortney" nil (g ao1 r t n iy0))
("gorton" nil (g ao1 r t ax n))
("gorum" nil (g ao1 r ax m))
("gory" nil (g ao1 r iy0))
("gosa" nil (g ow1 s ax))
("gosbank" nil (g ao1 s b ae1 ng k))
("gosch" nil (g ao1 sh))
("gosdin" nil (g aa1 s d ih0 n))
("gose" nil (g ow1 z))
("gosh" nil (g aa1 sh))
("gosha" nil (g ow1 sh ax))
("goshawk" nil (g aa1 s hh ao1 k))
("goshorn" nil (g aa1 sh er0 n))
("goslee" nil (g aa1 s l iy0))
("goslin" nil (g aa1 s l ih0 n))
("gosline" nil (g aa1 s l ay0 n))
("gosman" nil (g aa1 s m ax n))
("gosnell" nil (g aa1 s n ax l))
("gosney" nil (g aa1 s n iy0))
("gospel" nil (g aa1 s p ax l))
("gospels" nil (g aa1 s p ax l z))
("gosplan" nil (g ao1 s p l ae1 n))
("goss" nil (g ao1 s))
("gossage" nil (g ao1 s ih0 jh))
("gossamer" nil (g aa1 s ax m er0))
("gossard" nil (g ax s aa1 r d))
("gosse" nil (g aa1 s))
("gosselin" nil (g aa1 s ih0 l ih0 n))
("gossen" nil (g ao1 s ax n))
("gosser" nil (g ao1 s er0))
("gossett" nil (g aa1 s ih0 t))
("gossip" nil (g aa1 s ax p))
("gossiping" nil (g aa1 s ax p ih0 ng))
("gossips" nil (g aa1 s ax p s))
("gossipy" nil (g aa1 s ax p iy0))
("gossman" nil (g ao1 s m ax n))
("gostev" nil (g ao1 s t ax v))
("gostomski" nil (g ax s t aa1 m s k iy0))
("goswick" nil (g aa1 s w ih0 k))
("got" nil (g aa1 t))
("gotaas" nil (g aa1 t aa1 s))
("gotch" nil (g aa1 ch))
("gotcha" nil (g aa1 ch ax))
("gotcher" nil (g aa1 ch er0))
("goth" nil (g aa1 th))
("gotham" nil (g aa1 th ax m))
("gothard" nil (g aa1 th er0 d))
("gothenburg" nil (g ow1 t ax n b er0 g))
("gothic" nil (g aa1 th ih0 k))
("gotlieb" nil (g aa1 t l iy1 b))
("goto" nil (g ow1 t uw1))
("gotsch" nil (g aa1 ch))
("gotschal" nil (g aa1 ch ax l))
("gotschall" nil (g aa1 ch ax l))
("gotshal" nil (g aa1 ch ax l))
("gotshall" nil (g aa1 ch ax l))
("gott" nil (g aa1 t))
("gotta" nil (g aa1 t ax))
("gotten" nil (g aa1 t ax n))
("gotterdammerung" nil (g aa1 t er0 d ae1 m er0 ax ng))
("gottesman" nil (g aa1 t s m ax n))
("gottfried" nil (g ao1 t f r iy0 d))
("gotthardt" nil (g aa1 th aa0 r t))
("gotthelf" nil (g aa1 t hh eh1 l f))
("gotti" nil (g aa1 t iy0))
("gottis" nil (g aa1 t ih0 s))
("gottlieb" nil (g aa1 t l iy1 b))
("gottman" nil (g aa1 t m ax n))
("gotts" nil (g aa1 t s))
("gottsch" nil (g aa1 ch))
("gottschalk" nil (g aa1 ch ax k))
("gottschalks" nil (g aa1 ch ax k s))
("gottschall" nil (g aa1 ch ax l))
("gottshall" nil (g aa1 ch ax l))
("gottwald" nil (g aa1 t w ax l d))
("gotwalt" nil (g aa1 t w ax l t))
("gotz" nil (g aa1 t s))
("goucher" nil (g aw1 k er0))
("goude" nil (g aw1 d))
("goudeau" nil (g uw1 d ow1))
("goudie" nil (g aw1 d iy0))
("goudreau" nil (g uw1 d r ow1))
("goudy" nil (g aw1 d iy0))
("gouge" nil (g aw1 jh))
("gouged" nil (g aw1 jh d))
("gougeon" nil (g aw1 jh ih0 n))
("gouger" nil (g aw1 jh er0))
("gough" nil (g ao1 f))
("goughnour" nil (g aw1 n er0))
("gouging" nil (g aw1 jh ih0 ng))
("gouin" nil (g w iy1 n))
("gouker" nil (g aw1 k er0))
("goulart" nil (g uw0 l aa1 r t))
("goulash" nil (g uw1 l aa1 sh))
("gould" nil (g uw1 l d))
("goulden" nil (g uh1 d ax n))
("goulding" nil (g uw1 l d ih0 ng))
("goulet" nil (g uw0 l eh1 t))
("goulette" nil (g uw1 l eh1 t))
("goupil" nil (g uw1 p ax l))
("gourd" nil (g ao1 r d))
("gourdine" nil (g uh0 r d ay1 n))
("gourds" nil (g ao1 r d z))
("gourlay" nil (g ao1 r l ey0))
("gourley" nil (g ao1 r l iy0))
("gourment" nil (g ao1 r m eh1 n t))
("gourmet" nil (g uh1 r m ey1))
("gourmets" nil (g uh1 r m ey1 z))
("gousha" nil (g uw1 sh ax))
("gout" nil (g aw1 t))
("goutal" nil (g uw1 t ax l))
("gouty" nil (g aw1 t iy0))
("gouveia" nil (g uw0 v ey1 iy0 ax))
("govan" nil (g ow1 v ax n))
("gove" nil (g ow1 v))
("govea" nil (g ah1 v iy0 ax))
("gover" nil (g ah1 v er0))
("goverment" nil (g ah1 v er0 m ax n t))
("govern" nil (g ah1 v er0 n))
("governale" nil (g ah1 v er0 n ey1 l))
("governance" nil (g ah1 v er0 n ax n s))
("governed" nil (g ah1 v er0 n d))
("governess" nil (g ah1 v er0 n ax s))
("governing" nil (g ah1 v er0 n ih0 ng))
("government" nil (g ah1 v er0 m ax n t))
("governmental" nil (g ah1 v er0 m eh1 n t ax l))
("governmentally" nil (g ah1 v er0 m eh1 n t ax l iy0))
("governments" nil (g ah1 v er0 m ax n t s))
("governor" nil (g ah1 v er0 n er0))
("governors" nil (g ah1 v er0 n er0 z))
("governorship" nil (g ah1 v er0 n er0 sh ih1 p))
("governorships" nil (g ah1 v er0 n er0 sh ih1 p s))
("governs" nil (g ah1 v er0 n z))
("govett" nil (g ah1 v ax t))
("govier" nil (g ow1 v iy0 er0))
("govoni" nil (g ow0 v ow1 n iy0))
("govpx" nil (g ah1 v p iy1 eh1 k s))
("govs" nil (g aa1 v z))
("gow" nil (g aw1))
("gowan" nil (g aw1 ax n))
("gowans" nil (g aw1 ax n z))
("goward" nil (g ow1 w er0 d))
("gowdy" nil (g aw1 d iy0))
("gowell" nil (g aa1 w eh0 l))
("gowen" nil (g aw1 ax n))
("gowens" nil (g aw1 ax n z))
("gower" nil (g aw1 er0))
("gowin" nil (g aw1 ih0 n))
("gowing" nil (g aw1 ih0 ng))
("gowins" nil (g aw1 ih0 n z))
("gown" nil (g aw1 n))
("gowns" nil (g aw1 n z))
("goy" nil (g oy1))
("goya" nil (g oy1 ax))
("goyer" nil (g oy1 er0))
("goyette" nil (g oy1 eh1 t))
("goyim" nil (g oy1 ih0 m))
("goyne" nil (g oy1 n))
("goynes" nil (g oy1 n z))
("goza" nil (g ow1 z ax))
("gozo" nil (g ow1 z ow0))
("grab" nil (g r ae1 b))
("grabau" nil (g r ae1 b aw0))
("grabbe" nil (g r ae1 b))
("grabbed" nil (g r ae1 b d))
("grabber" nil (g r ae1 b er0))
("grabbers" nil (g r ae1 b er0 z))
("grabbing" nil (g r ae1 b ih0 ng))
("grabe" nil (g r ey1 b))
("grabel" nil (g r ae1 b ax l))
("graben" nil (g r ae1 b ax n))
("grabens" nil (g r aa1 b ax n z))
("grabenstein" nil (g r ae1 b ax n s t ay1 n))
("graber" nil (g r ey1 b er0))
("grabert" nil (g r ae1 b er0 t))
("grabill" nil (g r ax b ih1 l))
("grabinski" nil (g r ax b ih1 n s k iy0))
("grable" nil (g r ey1 b ax l))
("grabner" nil (g r ae1 b n er0))
("graboski" nil (g r ax b aw1 s k iy0))
("grabow" nil (g r ae1 b ow0))
("grabowski" nil (g r ax b ao1 f s k iy0))
("grabs" nil (g r ae1 b z))
("grabski" nil (g r ae1 b s k iy1))
("grace" nil (g r ey1 s))
("graced" nil (g r ey1 s t))
("graceffa" nil (g r ax s eh1 f ax))
("graceful" nil (g r ey1 s f ax l))
("gracefully" nil (g r ey1 s f ax l iy0))
("graceland" nil (g r ey1 s l ax n d))
("graceless" nil (g r ey1 s l ax s))
("graces" nil (g r ey1 s ih0 z))
("gracey" nil (g r ey1 s iy0))
("grachev" nil (g r aa1 ch eh0 v))
("graci" nil (g r aa1 ch iy0))
("gracia" nil (g r aa1 ch ax))
("graciano" nil (g r aa0 ch iy0 aa1 n ow0))
("gracie" nil (g r ey1 s iy0))
("gracile" nil (g r ae1 s ax l))
("gracious" nil (g r ey1 sh ax s))
("graciously" nil (g r ey1 sh ax s l iy0))
("grackle" nil (g r ae1 k ax l))
("grackles" nil (g r ae1 k ax l z))
("graco" nil (g r ae1 k ow0))
("gracy" nil (g r ey1 s iy0))
("graczyk" nil (g r aa1 ch ih0 k))
("grad" nil (g r ae1 d))
("gradation" nil (g r ey0 d ey1 sh ax n))
("gradco" nil (g r ae1 d k ow0))
("graddy" nil (g r ae1 d iy0))
("grade" nil (g r ey1 d))
("graded" nil (g r ey1 d ax d))
("gradel" nil (g r ae1 d ax l))
("graden" nil (g r ey1 d ax n))
("grader" nil (g r ey1 d er0))
("graders" nil (g r ey1 d er0 z))
("grades" nil (g r ey1 d z))
("grading" nil (g r ey1 d ih0 ng))
("gradison" nil (g r ae1 d ih0 s ax n))
("gradney" nil (g r ae1 d n iy0))
("grado" nil (g r aa1 d ow0))
("grads" nil (g r ae1 d z))
("gradstein" nil (g r ae1 d s t iy0 n))
("gradual" nil (g r ae1 jh uw0 ax l))
("gradualism" nil (g r ae1 jh ax w ax l ih1 z ax m))
("gradualist" nil (g r ae1 jh ax w ax l ih0 s t))
("gradually" nil (g r ae1 jh uw0 ax l iy0))
("graduate" n (g r ae1 jh ax w ax t))
("graduate" v (g r ae1 jh ax w ey0 t))
("graduated" nil (g r ae1 jh uw0 ey1 t ih0 d))
("graduates" n (g r ae1 jh ax w ax t s))
("graduates" v (g r ae1 jh ax w ey0 t s))
("graduating" nil (g r ae1 jh ax w ey1 t ih0 ng))
("graduation" nil (g r ae1 jh uw0 ey1 sh ax n))
("gradus" nil (g r ey1 d ax s))
("grady" nil (g r ey1 d iy0))
("graeber" nil (g r eh1 b er0))
("graebner" nil (g r eh1 b n er0))
("graef" nil (g r iy1 f))
("graefe" nil (g r iy1 f))
("graeff" nil (g r iy1 f))
("graeme" nil (g r ey1 m))
("graeser" nil (g r ey1 z er0))
("graessle" nil (g r eh1 s ax l))
("graetz" nil (g r iy1 t s))
("graf" nil (g r ae1 f))
("grafe" nil (g r ey1 f))
("graff" nil (g r ae1 f))
("graffam" nil (g r ae1 f ax m))
("graffeo" nil (g r aa1 f iy0 ow0))
("graffiti" nil (g r ax f iy1 t iy0))
("graffius" nil (g r ae1 f iy0 ih0 s))
("graft" nil (g r ae1 f t))
("grafted" nil (g r ae1 f t ih0 d))
("grafting" nil (g r ae1 f t ih0 ng))
("grafton" nil (g r ae1 f t ax n))
("grafts" nil (g r ae1 f t s))
("grage" nil (g r ey1 jh))
("gragert" nil (g r ey1 g er0 t))
("gragg" nil (g r ae1 g))
("graham" nil (g r ey1 ax m))
("grahams" nil (g r ae1 m z))
("grahek" nil (g r ae1 hh ih0 k))
("grahl" nil (g r aa1 l))
("grahn" nil (g r ae1 n))
("graichen" nil (g r ay1 k ax n))
("grail" nil (g r ey1 l))
("grain" nil (g r ey1 n))
("grained" nil (g r ey1 n d))
("grainery" nil (g r ey1 n er0 iy0))
("grainger" nil (g r aa1 ih0 ng er0))
("grains" nil (g r ey1 n z))
("grainy" nil (g r ey1 n iy0))
("grajeda" nil (g r ay0 iy1 d ax))
("graley" nil (g r ae1 l iy0))
("grall" nil (g r ao1 l))
("gram" nil (g r ae1 m))
("grama" nil (g r ae1 m ax))
("gramaphone" nil (g r ae1 m ax f ow1 n))
("grambling" nil (g r ae1 m b l ih0 ng))
("gramer" nil (g r ey1 m er0))
("gramercy" nil (g r ax m er1 s iy0))
("grames" nil (g r ey1 m z))
("gramley" nil (g r ae1 m l iy0))
("gramlich" nil (g r ae1 m l ih0 k))
("gramling" nil (g r ae1 m l ih0 ng))
("gramm" nil (g r ae1 m))
("grammar" nil (g r ae1 m er0))
("grammatical" nil (g r ax m ae1 t ax k ax l))
("grammatico" nil (g r aa0 m aa0 t iy1 k ow0))
("grammer" nil (g r ae1 m er0))
("grammophon" nil (g r ae1 m ax f aa0 n))
("grammy" nil (g r ae1 m iy0))
("grammys" nil (g r ae1 m iy0 z))
("grams" nil (g r ae1 m z))
("gramza" nil (g r ae1 m z ax))
("gran" nil (g r ae1 n))
("grana" nil (g r ae1 n ax))
("granada" nil (g r ax n aa1 d ax))
("granade" nil (g r ax n ey1 d))
("granado" nil (g r aa0 n aa1 d ow0))
("granados" nil (g r aa0 n aa1 d ow0 z))
("granahan" nil (g r ae1 n ax hh ae0 n))
("granat" nil (g r ae1 n ax t))
("granata" nil (g r aa0 n aa1 t ax))
("granato" nil (g r aa0 n aa1 t ow0))
("granberg" nil (g r ae1 n b er0 g))
("granberry" nil (g r ae1 n b eh1 r iy0))
("grancare" nil (g r ae1 n k eh1 r))
("grand" nil (g r ae1 n d))
("granda" nil (g r ae1 n d ax))
("grandad" nil (g r ae1 n d ae1 d))
("grandbaby" nil (g r ae1 n d b ey1 b iy0))
("grandberry" nil (g r ae1 n d b eh1 r iy0))
("grandchamp" nil (g r ae1 n d ch ae1 m p))
("grandchild" nil (g r ae1 n d ch ay1 l d))
("grandchildren" nil (g r ae1 n ch ih1 l d r ax n))
("granddaddy" nil (g r ae1 n d ae1 d iy0))
("granddaughter" nil (g r ae1 n d ao1 t er0))
("granddaughters" nil (g r ae1 n d ao1 t er0 z))
("grande" nil (g r ae1 n d))
("grander" nil (g r ae1 n d er0))
("granderson" nil (g r ae1 n d er0 s ax n))
("grandest" nil (g r ae1 n d ax s t))
("grandeur" nil (g r ae0 n d uw1 r))
("grandfather" nil (g r ae1 n d f aa1 dh er0))
("grandfathered" nil (g r ae1 n d f aa1 dh er0 d))
("grandfathering" nil (g r ae1 n d f aa1 dh er0 ih0 ng))
("grandfatherly" nil (g r ae1 n d f aa1 dh er0 l iy0))
("grandfathers" nil (g r ae1 n d f aa1 dh er0 z))
("grandfield" nil (g r ae1 n d f iy1 l d))
("grandi" nil (g r ae1 n d iy0))
("grandillo" nil (g r ae0 n d ih1 l ow0))
("grandiloquent" nil (g r ae0 n d ih1 l ax k w ax n t))
("grandin" nil (g r ae1 n d ih1 n))
("grandinetti" nil (g r ae0 n d iy0 n eh1 t iy0))
("grandiose" nil (g r ae1 n d iy0 ow1 s))
("grandison" nil (g r ae1 n d ih0 s ax n))
("grandkids" nil (g r ae1 n d k ih1 d z))
("grandly" nil (g r ae1 n d l iy0))
("grandma" nil (g r ae1 m aa0))
("grandmaison" nil (g r ae1 n d m ay1 z aa1 n))
("grandmaster" nil (g r ae1 n d m ae1 s t er0))
("grandmet" nil (g r ae1 n d m eh1 t))
("grandmother" nil (g r ae1 n d m ah1 dh er0))
("grandmotherly" nil (g r ae1 n d m ah1 dh er0 l iy0))
("grandmothers" nil (g r ae1 n d m ah1 dh er0 z))
("grandnephew" nil (g r ae1 n d n eh1 f y uw0))
("grandon" nil (g r ae1 n d aa0 n))
("grandpa" nil (g r ae1 n d p aa1))
("grandparent" nil (g r ae1 n d p eh1 r ax n t))
("grandparents" nil (g r ae1 n d p eh1 r ax n t s))
("grandpre" nil (g r ae1 n d p r ey1))
("grands" nil (g r ae1 n d z))
("grandson" nil (g r ae1 n d s ah1 n))
("grandsons" nil (g r ae1 n d s ah1 n z))
("grandstaff" nil (g r ae1 n d s t ae1 f))
("grandstand" nil (g r ae1 n d s t ae1 n d))
("grandstanding" nil (g r ae1 n d s t ae1 n d ih0 ng))
("grandt" nil (g r ae1 n t))
("granduncle" nil (g r ae1 n d ah1 ng k ax l))
("grandview" nil (g r ae1 n d v y uw1))
("grandy" nil (g r ae1 n d iy0))
("graner" nil (g r ey1 n er0))
("granese" nil (g r ae1 n iy0 z))
("graney" nil (g r ey1 n iy0))
("granfield" nil (g r ae1 n f iy1 l d))
("grange" nil (g r ey1 n jh))
("granger" nil (g r ey1 n jh er0))
("grangers" nil (g r ey1 n jh er0 z))
("granges" nil (g r ey1 n jh ih0 z))
("granholm" nil (g r ae1 n hh ow1 l m))
("granier" nil (g r ey1 n iy0 er0))
("granieri" nil (g r aa0 n ih1 r iy0))
("granillo" nil (g r ax n ih1 l ow0))
("graningeverken" nil (g r ae1 n ih0 ng g eh1 v er0 k ax n))
("granite" nil (g r ae1 n ax t))
("granitic" nil (g r ax n ih1 t ih0 k))
("granito" nil (g r aa0 n iy1 t ow0))
("granlund" nil (g r ae1 n l ax n d))
("grannan" nil (g r ae1 n ax n))
("grannis" nil (g r ae1 n ih0 s))
("granny" nil (g r ae1 n iy0))
("grano" nil (g r aa1 n ow0))
("granoff" nil (g r ae1 n ao0 f))
("granola" nil (g r ax n ow1 l ax))
("granquist" nil (g r ae1 n k w ih1 s t))
("granstrom" nil (g r ae1 n s t r ax m))
("grant" nil (g r ae1 n t))
("granted" nil (g r ae1 n t ax d))
("grantham" nil (g r ae1 n th ax m))
("granting" nil (g r ae1 n t ih0 ng))
("grantland" nil (g r ae1 n t l ax n d))
("grantor" nil (g r ae1 n t er0))
("grantors" nil (g r ae1 n t er0 z))
("grantree" nil (g r ae1 n t r iy1))
("grants" nil (g r ae1 n t s))
("grantz" nil (g r ae1 n t s))
("granular" nil (g r ae1 n y ax l er0))
("granulation" nil (g r ae1 n y ax l ey1 sh ax n))
("granulocyte" nil (g r ax n uw1 l ow0 s ay1 t))
("granum" nil (g r ae1 n ax m))
("granville" nil (g r ae1 n v ih0 l))
("granz" nil (g r ae1 n t s))
("granzow" nil (g r ae1 n z ow0))
("grape" nil (g r ey1 p))
("grapefruit" nil (g r ey1 p f r uw1 t))
("grapefruits" nil (g r ey1 p f r uw1 t s))
("graper" nil (g r ey1 p er0))
("grapes" nil (g r ey1 p s))
("grapeshot" nil (g r ey1 p sh aa1 t))
("grapevine" nil (g r ey1 p v ay1 n))
("grapevines" nil (g r ey1 p v ay1 n z))
("graph" nil (g r ae1 f))
("graphic" nil (g r ae1 f ih0 k))
("graphical" nil (g r ae1 f ih0 k ax l))
("graphically" nil (g r ae1 f ih0 k l iy0))
("graphics" nil (g r ae1 f ih0 k s))
("graphite" nil (g r ae1 f ay1 t))
("graphology" nil (g r ax f aa1 l ax jh iy0))
("graphs" nil (g r ae1 f s))
("grapple" nil (g r ae1 p ax l))
("grappled" nil (g r ae1 p ax l d))
("grapples" nil (g r ae1 p ax l z))
("grappling" nil (g r ae1 p l ih0 ng))
("gras" nil (g r ae1 s))
("graser" nil (g r ey1 z er0))
("grasmick" nil (g r ae1 z m ih0 k))
("grasp" nil (g r ae1 s p))
("grasped" nil (g r ae1 s p t))
("grasping" nil (g r ae1 s p ih0 ng))
("grasps" nil (g r ae1 s p s))
("grass" nil (g r ae1 s))
("grasse" nil (g r ae1 s))
("grassed" nil (g r ae1 s t))
("grassel" nil (g r ae1 s ax l))
("grasser" nil (g r ae1 s er0))
("grassers" nil (g r ae1 s er0 z))
("grasses" nil (g r ae1 s ax z))
("grassgreen" nil (g r ae1 s g r iy1 n))
("grasshopper" nil (g r ae1 s hh aa1 p er0))
("grasshoppers" nil (g r ae1 s hh aa1 p er0 z))
("grassi" nil (g r ae1 s iy0))
("grassia" nil (g r aa1 s iy0 ax))
("grassl" nil (g r ae1 s ax l))
("grassland" nil (g r ae1 s l ae1 n d))
("grasslands" nil (g r ae1 s l ae1 n d z))
("grassley" nil (g r ae1 s l iy0))
("grasslike" nil (g r ae1 s l ay1 k))
("grassman" nil (g r ae1 s m ax n))
("grasso" nil (g r ae1 s ow0))
("grassroots" nil (g r ae1 s r uw1 t s))
("grassy" nil (g r ae1 s iy0))
("grasty" nil (g r ae1 s t iy0))
("grata" nil (g r aa1 t ax))
("grate" nil (g r ey1 t))
("grated" nil (g r ey1 t ih0 d))
("grateful" nil (g r ey1 t f ax l))
("gratefully" nil (g r ey1 t f ax l iy0))
("grater" nil (g r ey1 t er0))
("grates" nil (g r ey1 t s))
("grathwohl" nil (g r ae1 th w ow1 l))
("gratification" nil (g r ae1 t ax f ax k ey1 sh ax n))
("gratified" nil (g r ae1 t ax f ay1 d))
("gratify" nil (g r ae1 t ax f ay1))
("gratifying" nil (g r ae1 t ax f ay1 ih0 ng))
("grating" nil (g r ey1 t ih0 ng))
("gratis" nil (g r ae1 t ax s))
("gratitude" nil (g r ae1 t ax t uw1 d))
("grattan" nil (g r ae1 t ax n))
("gratton" nil (g r ae1 t ax n))
("gratuities" nil (g r ax t uw1 ih0 t iy0 z))
("gratuitous" nil (g r ax t uw1 ax t ax s))
("gratuitously" nil (g r ax t uw1 ax t ax s l iy0))
("gratuity" nil (g r ax t uw1 ih0 t iy0))
("gratz" nil (g r ae1 t s))
("gratzer" nil (g r ey1 t z er0))
("grau" nil (g r aw1))
("grauberger" nil (g r aw1 b er0 g er0))
("graue" nil (g r aw1))
("grauel" nil (g r aw1 ax l))
("grauer" nil (g r aw1 er0))
("graul" nil (g r ao1 l))
("grauman" nil (g r ao1 m ax n))
("graumann" nil (g r ao1 m ax n))
("graunke" nil (g r ao1 ng k))
("gravatt" nil (g r ae1 v ax t))
("grave" nil (g r ey1 v))
("gravel" nil (g r ae1 v ax l))
("graveline" nil (g r ey1 v l ay1 n))
("gravell" nil (g r ae1 v ax l))
("gravelle" nil (g r ax v eh1 l))
("gravelly" nil (g r ae1 v ax l iy0))
("gravely" nil (g r ey1 v l iy0))
("graven" nil (g r ey1 v ax n))
("graver" nil (g r ey1 v er0))
("graves" nil (g r ey1 v z))
("gravest" nil (g r ae1 v ax s t))
("gravett" nil (g r ae1 v ih0 t))
("gravette" nil (g r ax v eh1 t))
("graveyard" nil (g r ey1 v y aa1 r d))
("graveyards" nil (g r ey1 v y aa1 r d z))
("gravies" nil (g r ey1 v iy0 z))
("gravimeter" nil (g r ae1 v ax m iy1 t er0))
("gravimetric" nil (g r ae1 v ax m eh1 t r ih0 k))
("gravina" nil (g r aa0 v iy1 n ax))
("gravino" nil (g r aa0 v iy1 n ow0))
("gravitate" nil (g r ae1 v ih0 t ey1 t))
("gravitated" nil (g r ae1 v ax t ey1 t ih0 d))
("gravitates" nil (g r ae1 v ih0 t ey1 t s))
("gravitating" nil (g r ae1 v ih0 t ey1 t ih0 ng))
("gravitational" nil (g r ae1 v ih0 t ey1 sh ax n ax l))
("gravitationally" nil (g r ae1 v ax t ey1 sh n ax l iy0))
("gravitt" nil (g r ae1 v ih0 t))
("gravity" nil (g r ae1 v ax t iy0))
("gravley" nil (g r ae1 v l iy0))
("gravlin" nil (g r ae1 v l ih0 n))
("gravois" nil (g r ax v w aa1))
("gravy" nil (g r ey1 v iy0))
("graw" nil (g r ao1))
("grawe" nil (g r ao1))
("gray" nil (g r ey1))
("graybeal" nil (g r ey1 b ax l))
("graybill" nil (g r ey1 b ih1 l))
("graydon" nil (g r ey1 d ax n))
("graye" nil (g r ey1))
("grayer" nil (g r ey1 er0))
("graying" nil (g r ey1 ih0 ng))
("grayish" nil (g r ey1 ih0 sh))
("grayling" nil (g r ey1 l ih0 ng))
("grays" nil (g r ey1 z))
("grayson" nil (g r ey1 s ax n))
("graze" nil (g r ey1 z))
("grazer" nil (g r ey1 z er0))
("graziani" nil (g r aa0 z iy0 aa1 n iy0))
("graziano" nil (g r aa0 t s iy0 aa1 n ow0))
("grazier" nil (g r ey1 z iy0 er0))
("grazing" nil (g r ey1 z ih0 ng))
("greaney" nil (g r iy1 n iy0))
("grear" nil (g r ih1 r))
("grease" nil (g r iy1 s))
("greased" nil (g r iy1 s t))
("greaser" nil (g r iy1 s er0))
("greasewood" nil (g r iy1 s w uh1 d))
("greasing" nil (g r iy1 s ih0 ng))
("greason" nil (g r iy1 s ax n))
("greasy" nil (g r iy1 s iy0))
("great" nil (g r ey1 t))
("greater" nil (g r ey1 t er0))
("greatest" nil (g r ey1 t ax s t))
("greathouse" nil (g r ey1 t hh aw1 s))
("greatly" nil (g r ey1 t l iy0))
("greatness" nil (g r ey1 t n ax s))
("greats" nil (g r ey1 t s))
("greave" nil (g r iy1 v))
("greaves" nil (g r iy1 v z))
("greb" nil (g r eh1 b))
("grebe" nil (g r iy1 b))
("greber" nil (g r iy1 b er0))
("grebner" nil (g r eh1 b n er0))
("grecco" nil (g r eh1 k ow0))
("grech" nil (g r eh1 k))
("grecian" nil (g r iy1 sh ax n))
("greco" nil (g r eh1 k ow0))
("grede" nil (g r iy1 d))
("gredel" nil (g r eh1 d ax l))
("greditor" nil (g r eh1 d ih0 t er0))
("greear" nil (g r iy1 er0))
("greece" nil (g r iy1 s))
("greed" nil (g r iy1 d))
("greedily" nil (g r iy1 d ax l iy0))
("greedy" nil (g r iy1 d iy0))
("greek" nil (g r iy1 k))
("greeks" nil (g r iy1 k s))
("greeley" nil (g r iy1 l iy0))
("greely" nil (g r iy1 l iy0))
("green" nil (g r iy1 n))
("greenan" nil (g r iy1 n ax n))
("greenawalt" nil (g r iy1 n ax w ao1 l t))
("greenaway" nil (g r iy1 n ax w ey1))
("greenback" nil (g r iy1 n b ae1 k))
("greenbacks" nil (g r iy1 n b ae1 k s))
("greenbaum" nil (g r iy1 n b aw1 m))
("greenbelt" nil (g r iy1 n b eh1 l t))
("greenberg" nil (g r iy1 n b er0 g))
("greenberger" nil (g r iy1 n b er0 g er0))
("greenblatt" nil (g r iy1 n b l ax t))
("greenburg" nil (g r iy1 n b er0 g))
("greenbury" nil (g r iy1 n b er0 iy0))
("greencastle" nil (g r iy1 n k ae1 s ax l))
("greene" nil (g r iy1 n))
("greened" nil (g r iy1 n d))
("greener" nil (g r iy1 n er0))
("greenery" nil (g r iy1 n er0 iy0))
("greenfeld" nil (g r iy1 n f eh1 l d))
("greenfell" nil (g r iy1 n f eh1 l))
("greenfield" nil (g r iy1 n f iy1 l d))
("greenhalgh" nil (g r iy1 n hh ah1 l g))
("greenham" nil (g r iy1 n ax m))
("greenhaw" nil (g r iy1 n hh ao1))
("greenhill" nil (g r iy1 n hh ih1 l))
("greenhoe" nil (g r iy1 n hh ow1))
("greenhouse" nil (g r iy1 n hh aw1 s))
("greenhouses" nil (g r iy1 n hh aw1 s ih0 z))
("greenhut" nil (g r iy1 n hh ax t))
("greeniaus" nil (g r iy1 n iy0 aw0 s))
("greenidge" nil (g r iy1 n ih0 jh))
("greening" nil (g r iy1 n ih0 ng))
("greenish" nil (g r iy1 n ih0 sh))
("greenland" nil (g r iy1 n l ax n d))
("greenlaw" nil (g r iy1 n l ao1))
("greenleaf" nil (g r iy1 n l iy1 f))
("greenlee" nil (g r iy1 n l iy1))
("greenlees" nil (g r iy1 n l iy1 z))
("greenley" nil (g r iy1 n l iy0))
("greenly" nil (g r iy1 n l iy0))
("greenmail" nil (g r iy1 n m ey1 l))
("greenmailer" nil (g r iy1 n m ey1 l er0))
("greenman" nil (g r iy1 n m ax n))
("greenness" nil (g r iy1 n n ax s))
("greeno" nil (g r iy1 n ow0))
("greenough" nil (g r iy1 n ax f))
("greenpeace" nil (g r iy1 n p iy1 s))
("greens" nil (g r iy1 n z))
("greensboro" nil (g r iy1 n z b er0 r ow0))
("greensburg" nil (g r iy1 n z b er0 g))
("greenshields" nil (g r iy1 n sh iy1 l d z))
("greenslade" nil (g r iy1 n s l ax d))
("greenslet" nil (g r iy1 n s l eh1 t))
("greenspan" nil (g r iy1 n s p ae1 n))
("greenspon" nil (g r iy1 n s p aa1 n))
("greenspun" nil (g r iy1 n s p ah1 n))
("greenstein" nil (g r iy1 n s t ay1 n))
("greenstone" nil (g r iy1 n s t ow1 n))
("greenstreet" nil (g r iy1 n s t r iy1 t))
("greenup" nil (g r iy1 n ah1 p))
("greenvale" nil (g r iy1 n v ey1 l))
("greenville" nil (g r iy1 n v ih0 l))
("greenwald" nil (g r iy1 n w ao1 l d))
("greenwalt" nil (g r iy1 n w ax l t))
("greenway" nil (g r iy1 n w ey1))
("greenwell" nil (g r iy1 n w eh1 l))
("greenwich" nil (g r eh1 n ih0 ch))
("greenwood" nil (g r iy1 n w uh1 d))
("greer" nil (g r ih1 r))
("greeson" nil (g r iy1 s ax n))
("greet" nil (g r iy1 t))
("greeted" nil (g r iy1 t ax d))
("greetham" nil (g r iy1 th ax m))
("greeting" nil (g r iy1 t ih0 ng))
("greetings" nil (g r iy1 t ih0 ng z))
("greets" nil (g r iy1 t s))
("greever" nil (g r iy1 v er0))
("grefe" nil (g r iy1 f))
("greff" nil (g r eh1 f))
("greg" nil (g r eh1 g))
("grega" nil (g r iy1 g ax))
("gregarious" nil (g r ax g eh1 r iy0 ax s))
("greger" nil (g r eh1 g er0))
("gregersen" nil (g r eh1 g er0 s ax n))
("gregerson" nil (g r eh1 g er0 s ax n))
("gregg" nil (g r eh1 g))
("greggs" nil (g r eh1 g z))
("grego" nil (g r eh1 g ow0))
("gregoire" nil (g r ih0 g w aa1 r))
("gregor" nil (g r eh1 g er0))
("gregori" nil (g r eh0 g ao1 r iy0))
("gregoria" nil (g r eh0 g ao1 r iy0 ax))
("gregorian" nil (g r ax g ao1 r iy0 ax n))
("gregorich" nil (g r eh1 g er0 ih0 k))
("gregorie" nil (g r eh1 g er0 iy0))
("gregorio" nil (g r ih0 g ao1 r iy0 ow0))
("gregory" nil (g r eh1 g er0 iy0))
("gregson" nil (g r eh1 g s ax n))
("gregston" nil (g r eh1 g s t ax n))
("gregus" nil (g r iy1 g ax s))
("greider" nil (g r ay1 d er0))
("greif" nil (g r iy1 f))
("greiff" nil (g r iy1 f))
("greig" nil (g r iy1 g))
("greim" nil (g r iy1 m))
("greiman" nil (g r ay1 m ax n))
("grein" nil (g r ey1 n))
("greiner" nil (g r ay1 n er0))
("greinke" nil (g r ey1 ng k))
("greis" nil (g r iy1 z))
("greiwe" nil (g r iy1 w))
("grell" nil (g r eh1 l))
("grella" nil (g r eh1 l ax))
("grelle" nil (g r eh1 l))
("gremban" nil (g r eh1 m b ae1 n))
("gremillion" nil (g r eh1 m ih0 l y ax n))
("gren" nil (g r eh1 n))
("grenada" nil (g r ih0 n ey1 d ax))
("grenade" nil (g r ax n ey1 d))
("grenades" nil (g r ax n ey1 d z))
("grenda" nil (g r eh1 n d ax))
("grendel" nil (g r eh1 n d ax l))
("grenell" nil (g r eh1 n ax l))
("grenfell" nil (g r eh1 n f ax l))
("grenier" nil (g r iy1 n iy0 er0))
("greninger" nil (g r eh1 n ih0 ng er0))
("grennan" nil (g r eh1 n ax n))
("grenoble" nil (g r ax n ow1 b ax l))
("grenon" nil (g r eh1 n ax n))
("grenouille" nil (g r ax n uw1 iy0))
("grenz" nil (g r eh1 n z))
("gresh" nil (g r eh1 sh))
("gresham" nil (g r eh1 sh ax m))
("gresko" nil (g r eh1 s k ow0))
("gress" nil (g r eh1 s))
("gresser" nil (g r eh1 s er0))
("gressett" nil (g r eh1 s ih0 t))
("gressley" nil (g r eh1 s l iy0))
("gressman" nil (g r eh1 s m ax n))
("greta" nil (g r iy1 t ax))
("gretal" nil (g r eh1 t ax l))
("gretchen" nil (g r eh1 ch ax n))
("grete" nil (g r iy1 t))
("gretel" nil (g r eh1 t ax l))
("greth" nil (g r eh1 th))
("grethel" nil (g r eh1 th ax l))
("grether" nil (g r eh1 dh er0))
("gretna" nil (g r eh1 t n ax))
("grettenberger" nil (g r eh1 t ax n b er0 g er0))
("gretz" nil (g r eh1 t s))
("gretzinger" nil (g r eh1 t z ih0 ng er0))
("gretzky" nil (g r eh1 t s k iy1))
("greubel" nil (g r oy1 b ax l))
("greuel" nil (g r uw1 ax l))
("greulich" nil (g r oy1 l ih0 k))
("greunke" nil (g r uw1 ng k))
("greve" nil (g r iy1 v))
("grever" nil (g r eh1 v er0))
("grew" nil (g r uw1))
("grewal" nil (g r uw1 ax l))
("grewe" nil (g r uw1))
("grewell" nil (g r eh1 w eh0 l))
("grey" nil (g r ey1))
("greyhound" nil (g r ey1 hh aw1 n d))
("greylag" nil (g r ey1 l ae1 g))
("gribben" nil (g r ih1 b ax n))
("gribbin" nil (g r ih1 b ih0 n))
("gribbins" nil (g r ih1 b ih0 n z))
("gribble" nil (g r ih1 b ax l))
("gribbles" nil (g r ih1 b ax l z))
("gribbon" nil (g r ih1 b ax n))
("grice" nil (g r ay1 s))
("grid" nil (g r ih1 d))
("grider" nil (g r ay1 d er0))
("gridiron" nil (g r ih1 d ay1 er0 n))
("gridley" nil (g r ih1 d l iy0))
("gridlock" nil (g r ih1 d l aa1 k))
("gridlocked" nil (g r ih1 d l aa1 k t))
("grids" nil (g r ih1 d z))
("grieb" nil (g r iy1 b))
("griebel" nil (g r iy1 b ax l))
("grieco" nil (g r iy1 k ow0))
("grieder" nil (g r iy1 d er0))
("grief" nil (g r iy1 f))
("grieger" nil (g r iy1 g er0))
("griego" nil (g r iy1 g ow0))
("grieme" nil (g r iy1 m))
("griep" nil (g r iy1 p))
("griepentrog" nil (g r iy1 p ih0 n t r ax g))
("grier" nil (g r ay1 er0))
("grierson" nil (g r ih1 r s ax n))
("gries" nil (g r ay1 z))
("griesa" nil (g r iy0 eh1 s ax))
("griesbach" nil (g r iy1 s b aa0 k))
("griese" nil (g r iy1 z))
("griesemer" nil (g r iy1 s iy0 m er0))
("grieser" nil (g r iy1 s er0))
("grieshaber" nil (g r iy1 sh ax b er0))
("griesinger" nil (g r iy1 s ih0 ng er0))
("griess" nil (g r iy1 s))
("griesser" nil (g r iy1 s er0))
("griest" nil (g r ay1 ih0 s t))
("grievance" nil (g r iy1 v ax n s))
("grievances" nil (g r iy1 v ax n s ax z))
("grieve" nil (g r iy1 v))
("grievers" nil (g r iy1 v er0 z))
("grieves" nil (g r iy1 v z))
("grieveson" nil (g r iy1 v ax s ax n))
("grieving" nil (g r iy1 v ih0 ng))
("grievous" nil (g r iy1 v ax s))
("grievously" nil (g r iy1 v ax s l iy0))
("griff" nil (g r ih1 f))
("griffee" nil (g r ih1 f iy0))
("griffen" nil (g r ih1 f ax n))
("griffeth" nil (g r ih1 f ih0 th))
("griffey" nil (g r ih1 f iy0))
("griffie" nil (g r ih1 f iy0))
("griffin" nil (g r ih1 f ih0 n))
("griffing" nil (g r ih1 f ih0 ng))
("griffis" nil (g r ih1 f ih0 s))
("griffith" nil (g r ih1 f ax th))
("griffiths" nil (g r ih1 f ih0 th s))
("griffitts" nil (g r ih1 f ih0 t s))
("griffo" nil (g r ih1 f ow0))
("griffon" nil (g r ih1 f ax n))
("griffy" nil (g r ih1 f iy0))
("grigas" nil (g r ay1 g ax z))
("grigg" nil (g r ih1 g))
("griggs" nil (g r ih1 g z))
("griggy" nil (g r ih1 g iy0))
("grignon" nil (g r ih1 g n ax n))
("grigoli" nil (g r ih0 g ow1 l iy0))
("grigorovich" nil (g r ih0 g ao1 r ax v ih0 ch))
("grigoryants" nil (g r ih0 g ao1 r y ae0 n t s))
("grigsby" nil (g r ih1 g z b iy0))
("grijalva" nil (g r iy0 y aa1 l v ax))
("grill" nil (g r ih1 l))
("grille" nil (g r ih1 l))
("grilled" nil (g r ih1 l d))
("grilli" nil (g r ih1 l iy0))
("grilling" nil (g r ih1 l ih0 ng))
("grilliot" nil (g r ih1 l iy0 ax t))
("grillo" nil (g r ih1 l ow0))
("grillot" nil (g r ih1 l ax t))
("grills" nil (g r ih1 l z))
("grim" nil (g r ih1 m))
("grima" nil (g r iy1 m ax))
("grimaced" nil (g r ih1 m ax s t))
("grimaces" nil (g r ih1 m ax s ih0 z))
("grimaldi" nil (g r iy0 m aa1 l d iy0))
("grimaldo" nil (g r iy0 m aa1 l d ow0))
("grime" nil (g r ay1 m))
("grimes" nil (g r ay1 m z))
("grimley" nil (g r ih1 m l iy0))
("grimly" nil (g r ih1 m l iy0))
("grimm" nil (g r ih1 m))
("grimme" nil (g r ih1 m))
("grimmer" nil (g r ih1 m er0))
("grimmest" nil (g r ih1 m ax s t))
("grimmett" nil (g r ih1 m ih0 t))
("grimness" nil (g r ih1 m n ax s))
("grimshaw" nil (g r ih1 m sh ao1))
("grimsley" nil (g r ih1 m z l iy0))
("grimstad" nil (g r ih1 m s t ax d))
("grimwood" nil (g r ih1 m w uh1 d))
("grimy" nil (g r ay1 m iy0))
("grin" nil (g r ih1 n))
("grinage" nil (g r ih1 n ih0 jh))
("grinberg" nil (g r ih1 n b er0 g))
("grinch" nil (g r ih1 n ch))
("grind" nil (g r ay1 n d))
("grinde" nil (g r ih1 n d))
("grinder" nil (g r ay1 n d er0))
("grinders" nil (g r ay1 n d er0 z))
("grinding" nil (g r ay1 n d ih0 ng))
("grindlays" nil (g r ay1 n d l ey1 z))
("grindle" nil (g r ih1 n d ax l))
("grindley" nil (g r ih1 n d l iy0))
("grindrod" nil (g r ay1 n d r aa1 d))
("grinds" nil (g r ay1 n d z))
("grindstaff" nil (g r ay1 n d s t ae1 f))
("grindstone" nil (g r ay1 n d s t ow1 n))
("grine" nil (g r ay1 n))
("griner" nil (g r ay1 n er0))
("gring" nil (g r ih1 ng))
("gringo" nil (g r ih1 ng g ow0))
("gringos" nil (g r ih1 ng g ow0 z))
("grinned" nil (g r ih1 n d))
("grinnell" nil (g r ih0 n eh1 l))
("grinning" nil (g r ih1 n ih0 ng))
("grins" nil (g r ih1 n z))
("grinstead" nil (g r ih1 n s t eh1 d))
("grinstein" nil (g r ih1 n s t iy1 n))
("grip" nil (g r ih1 p))
("gripe" nil (g r ay1 p))
("griped" nil (g r ay1 p t))
("gripes" nil (g r ay1 p s))
("griping" nil (g r ay1 p ih0 ng))
("gripp" nil (g r ih1 p))
("gripped" nil (g r ih1 p t))
("grippi" nil (g r ih1 p iy0))
("gripping" nil (g r ih1 p ih0 ng))
("grippo" nil (g r ih1 p ow0))
("grips" nil (g r ih1 p s))
("grisanti" nil (g r ih0 s ae1 n t iy0))
("grisby" nil (g r ih1 s b iy0))
("griscom" nil (g r ih1 s k ax m))
("grise" nil (g r ay1 z))
("grisham" nil (g r ih1 sh ax m))
("grishilda" nil (g r ih0 sh ih1 l d ax))
("grishman" nil (g r ih1 sh m ax n))
("grisly" nil (g r ih1 z l iy0))
("grismer" nil (g er1 ih0 z ax m er0))
("grismore" nil (g r iy1 s m ao0 r))
("grissett" nil (g r ih1 s ih0 t))
("grissinger" nil (g r ih1 s ih0 ng er0))
("grisso" nil (g r ih1 s ow0))
("grissom" nil (g r ih1 s ax m))
("grisson" nil (g r ih1 s ax n))
("grist" nil (g r ih1 s t))
("gristle" nil (g r ih1 s ax l))
("griswell" nil (g r ih1 s w eh0 l))
("griswold" nil (g r ih1 s w ow1 l d))
("grit" nil (g r ih1 t))
("grits" nil (g r ih1 t s))
("gritter" nil (g r ih1 t er0))
("gritting" nil (g r ih1 t ih0 ng))
("gritton" nil (g r ih1 t ax n))
("gritty" nil (g r ih1 t iy0))
("gritz" nil (g r ih1 t s))
("gritzmacher" nil (g r ih1 t s m aa1 k er0))
("grivas" nil (g r iy1 v aa0 z))
("grizelda" nil (g r iy0 z eh1 l d ax))
("grizzard" nil (g r ih1 z er0 d))
("grizzell" nil (g r ih1 z ax l))
("grizzle" nil (g r ih1 z ax l))
("grizzled" nil (g r ih1 z ax l d))
("grizzlies" nil (g r ih1 z l iy0 z))
("grizzly" nil (g r ih1 z l iy0))
("gro" nil (g r ow1))
("groan" nil (g r ow1 n))
("groaned" nil (g r ow1 n d))
("groaning" nil (g r ow1 n ih0 ng))
("groans" nil (g r ow1 n z))
("groat" nil (g r ow1 t))
("grob" nil (g r aa1 b))
("grobe" nil (g r ow1 b))
("groben" nil (g r aa1 b ax n))
("grober" nil (g r ow1 b er0))
("grobler" nil (g r aa1 b l er0))
("groce" nil (g r ow1 s))
("grocer" nil (g r ow1 s er0))
("groceries" nil (g r ow1 s er0 iy0 z))
("grocers" nil (g r ow1 s er0 z))
("grocery" nil (g r ow1 s er0 iy0))
("groch" nil (g r aa1 k))
("grochowski" nil (g r ax hh ao1 f s k iy0))
("grode" nil (g r ow1 d))
("groden" nil (g r ow1 d ax n))
("grodin" nil (g r ow1 d ih0 n))
("grodsky" nil (g r aa1 d s k iy0))
("groebner" nil (g r ow1 b n er0))
("groeger" nil (g r ow1 g er0))
("groen" nil (g r ow1 n))
("groene" nil (g r aa1 iy0 n))
("groeneveld" nil (g r ow1 n ih0 v ih0 l d))
("groenewold" nil (g r ow1 n uw0 ow0 l d))
("groening" nil (g r aa1 ax n ih0 ng))
("groep" nil (g r ow1 p))
("groesbeck" nil (g r ow1 s b eh0 k))
("groff" nil (g r ao1 f))
("groft" nil (g r aa1 f t))
("grogan" nil (g r ow1 g ax n))
("grogg" nil (g r aa1 g))
("groggy" nil (g r aa1 g iy0))
("groh" nil (g r ow1))
("grohman" nil (g r ow1 m ax n))
("grohs" nil (g r ow1 s))
("groin" nil (g r oy1 n))
("groined" nil (g r oy1 n d))
("groins" nil (g r oy1 n z))
("groleau" nil (g r ax l ow1))
("grolier" nil (g r ow1 l y er0))
("groll" nil (g r ow1 l))
("grom" nil (g r aa1 m))
("groma" nil (g r aa1 m ax))
("groman" nil (g r ow1 m ax n))
("gromek" nil (g r ow1 m ih0 k))
("gromer" nil (g r ow1 m er0))
("gromes" nil (g r ow1 m z))
("gromyko" nil (g r ow0 m iy1 k ow0))
("gronau" nil (g r ow1 n aw0))
("gronberg" nil (g r aa1 n b er0 g))
("grondahl" nil (g r aa1 n d aa1 l))
("grondin" nil (g r aa1 n d ih0 n))
("grone" nil (g r ow1 n))
("gronemeyer" nil (g r aa1 n ih0 m ay0 er0))
("groner" nil (g r ow1 n er0))
("gronewold" nil (g r aa1 n uw0 ow0 l d))
("groninger" nil (g r ow1 n ih0 ng er0))
("gronlund" nil (g r aa1 n l ax n d))
("gronowski" nil (g r ax n ao1 f s k iy0))
("gronseth" nil (g r aa1 n s ih0 th))
("gronski" nil (g r aa1 n s k iy0))
("groom" nil (g r uw1 m))
("groome" nil (g r uw1 m))
("groomed" nil (g r uw1 m d))
("groomers" nil (g r uw1 m er0 z))
("groomes" nil (g r uw1 m z))
("grooming" nil (g r uw1 m ih0 ng))
("grooms" nil (g r uw1 m z))
("groopman" nil (g r uw1 p m ax n))
("groos" nil (g r uw1 z))
("groot" nil (g r uw1 t))
("grooters" nil (g r uw1 t er0 z))
("groove" nil (g r uw1 v))
("groover" nil (g r uw1 v er0))
("grooves" nil (g r uw1 v z))
("grooviest" nil (g r uw1 v iy0 ax s t))
("groovy" nil (g r uw1 v iy0))
("grope" nil (g r ow1 p))
("groping" nil (g r ow1 p ih0 ng))
("gropp" nil (g r aa1 p))
("gropper" nil (g r aa1 p er0))
("gros" nil (g r ow1 s))
("grosbeak" nil (g r ow1 s b iy1 k))
("grosbeaks" nil (g r ow1 s b iy1 k s))
("grosch" nil (g r ao1 sh))
("grose" nil (g r ow1 z))
("groseclose" nil (g r aa0 s ih0 k l ow1 z))
("grosh" nil (g r aa1 sh))
("groshek" nil (g r aa1 sh ih0 k))
("groshong" nil (g r aa1 sh ao0 ng))
("grosjean" nil (g r ax s zh iy1 n))
("groskopf" nil (g r aa1 s k ao0 p f))
("gross" nil (g r ow1 s))
("grossbard" nil (g r ow1 s b aa1 r d))
("grossberg" nil (g r ow1 s b er0 g))
("grosse" nil (g r aa1 s))
("grossed" nil (g r ow1 s t))
("grossenbacher" nil (g r aa1 s ih0 n b aa0 k er0))
("grosser" nil (g r ow1 s er0))
("grosses" nil (g r ow1 s ih0 z))
("grossfeld" nil (g r ow1 s f eh1 l d))
("grosshans" nil (g r aa1 sh ax n z))
("grossi" nil (g r ow1 s iy0))
("grossing" nil (g r ow1 s ih0 ng))
("grosskopf" nil (g r ow1 s k ao0 p f))
("grossly" nil (g r ow1 s l iy0))
("grossman" nil (g r ow1 s m ax n))
("grossmann" nil (g r ao1 s m ax n))
("grossnickle" nil (g r aa1 s n ih0 k ax l))
("grosso" nil (g r ow1 s ow0))
("grosvenor" nil (g r ow1 v n er0))
("grosz" nil (g r ow1 s))
("grote" nil (g r ow1 t))
("grotesque" nil (g r ow0 t eh1 s k))
("grotesquely" nil (g r ow0 t eh1 s k l iy0))
("groth" nil (g r aa1 th))
("grothaus" nil (g r aa1 t hh aw1 s))
("grothe" nil (g r ow1 dh))
("grotheer" nil (g r ao1 th ih0 r))
("groton" nil (g r aa1 t ax n))
("grotto" nil (g r aa1 t ow1))
("grotz" nil (g r aa1 t s))
("grouch" nil (g r aw1 ch))
("groucho" nil (g r uw1 ch ow0))
("groulx" nil (g r aw1 l k s))
("ground" nil (g r aw1 n d))
("groundbreaking" nil (g r aw1 n d b r ey1 k ih0 ng))
("grounded" nil (g r aw1 n d ih0 d))
("groundhog" nil (g r aw1 n d hh aa1 g))
("grounding" nil (g r aw1 n d ih0 ng))
("groundless" nil (g r aw1 n d l ax s))
("groundling" nil (g r aw1 n d l ih0 ng))
("groundnut" nil (g r aw1 n d n ah1 t))
("groundnuts" nil (g r aw1 n d n ah1 t s))
("grounds" nil (g r aw1 n d z))
("groundswell" nil (g r aw1 n d s w eh1 l))
("groundwater" nil (g r aw1 n d w aa1 t er0))
("groundwork" nil (g r aw1 n d w er1 k))
("group" nil (g r uw1 p))
("groupe" nil (g r uw1 p))
("grouped" nil (g r uw1 p t))
("groupement" nil (g r uw1 p m ax n t))
("grouper" nil (g r uw1 p er0))
("groupers" nil (g r uw1 p er0 z))
("groupie" nil (g r uw1 p iy0))
("groupies" nil (g r uw1 p iy0 z))
("grouping" nil (g r uw1 p ih0 ng))
("groupings" nil (g r uw1 p ih0 ng z))
("groups" nil (g r uw1 p s))
("groupware" nil (g r uw1 p w eh1 r))
("grouse" nil (g r aw1 s))
("groused" nil (g r aw1 s t))
("grouses" nil (g r aw1 s ih0 z))
("grousing" nil (g r aw1 s ih0 ng))
("groussman" nil (g r aw1 s m ax n))
("grout" nil (g r aw1 t))
("grove" nil (g r ow1 v))
("grovel" nil (g r aa1 v ax l))
("groveling" nil (g r ao1 v ax l ih0 ng))
("groveman" nil (g r ow1 v m ax n))
("grover" nil (g r ow1 v er0))
("grovers" nil (g r ow1 v er0 z))
("groves" nil (g r ow1 v z))
("grow" nil (g r ow1))
("growe" nil (g r ow1))
("grower" nil (g r ow1 er0))
("growers" nil (g r ow1 er0 z))
("growing" nil (g r ow1 ih0 ng))
("growl" nil (g r aw1 l))
("growled" nil (g r aw1 l d))
("growling" nil (g r ow1 l ih0 ng))
("growls" nil (g r aw1 l z))
("grown" nil (g r ow1 n))
("grownup" nil (g r ow1 n ah1 p))
("grownups" nil (g r ow1 n ah1 p s))
("grows" nil (g r ow1 z))
("growth" nil (g r ow1 th))
("growths" nil (g r ow1 th s))
("grozny" nil (g r ow1 z n iy0))
("grua" nil (g r uw1 ax))
("grub" nil (g r ah1 b))
("gruba" nil (g r uw1 b ax))
("grubaugh" nil (g r ah1 b ao0))
("grubb" nil (g r ah1 b))
("grubbs" nil (g r ah1 b z))
("grubby" nil (g r ah1 b iy0))
("grube" nil (g r uw1 b))
("gruben" nil (g r ah1 b ax n))
("gruber" nil (g r uw1 b er0))
("gruberova" nil (g r uw1 b eh0 r ow1 v ax))
("grubman" nil (g r ah1 b m ax n))
("grubs" nil (g r ah1 b z))
("grubstein" nil (g r ah1 b s t iy1 n))
("grudge" nil (g r ah1 jh))
("grudges" nil (g r ah1 jh ih0 z))
("grudging" nil (g r ah1 jh ih0 ng))
("grudgingly" nil (g r ah1 jh ih0 ng l iy0))
("grudzien" nil (g r ah1 d z iy0 n))
("grudzinski" nil (g r ax jh ih1 n s k iy0))
("gruel" nil (g r uw1 ih0 l))
("grueling" nil (g r uw1 ih0 l ih0 ng))
("gruen" nil (g r uw1 n))
("gruenberg" nil (g r uh1 n b er0 g))
("gruener" nil (g r uh1 n er0))
("gruenewald" nil (g r uh1 n ih0 w ao0 l d))
("gruenhagen" nil (g r uh1 n hh ax g ax n))
("gruenwald" nil (g r uh1 n w ao0 l d))
("gruesome" nil (g r uw1 s ax m))
("gruet" nil (g r uw1 ih0 t))
("gruetzmacher" nil (g r uh1 t s m aa1 k er0))
("gruff" nil (g r ah1 f))
("gruhlke" nil (g r uw1 l k))
("gruhn" nil (g r ah1 n))
("gruis" nil (g r uw1 ih0 z))
("grulke" nil (g r ah1 l k))
("grullon" nil (g r ah1 l ax n))
("grum" nil (g r ah1 m))
("grumbine" nil (g r ah1 m b ay1 n))
("grumble" nil (g r ah1 m b ax l))
("grumbled" nil (g r ah1 m b ax l d))
("grumbles" nil (g r ah1 m b ax l z))
("grumbling" nil (g r ah1 m b ax l ih0 ng))
("grumblings" nil (g r ah1 m b ax l ih0 ng z))
("grumman" nil (g r ah1 m ax n))
("grummond" nil (g r ah1 m ax n d))
("grump" nil (g r ah1 m p))
("grumpy" nil (g r ah1 m p iy0))
("grun" nil (g r ah1 n))
("grunberg" nil (g r ah1 n b er0 g))
("grund" nil (g r ah1 n d))
("grunden" nil (g r ah1 n d ax n))
("grunder" nil (g r ah1 n d er0))
("grundfest" nil (g r ah1 n d f eh1 s t))
("grundhofer" nil (g r ah1 n d hh aa1 f er0))
("grundig" nil (g r ah1 n d ih0 g))
("grundman" nil (g r ah1 n d m ax n))
("grundstrom" nil (g r ah1 n d s t r ax m))
("grundy" nil (g r ah1 n d iy0))
("grunebaum" nil (g r uw1 n b aw1 m))
("gruneich" nil (g r uw0 n ay1 k))
("gruner" nil (g r uw1 n er0))
("grunert" nil (g r uw1 n er0 t))
("grunewald" nil (g r ah1 n ih0 w ao0 l d))
("grunge" nil (g r ah1 n jh))
("grungy" nil (g r ah1 n jh iy0))
("grunion" nil (g r ah1 n y ax n))
("grunow" nil (g r uw1 n ow0))
("grunt" nil (g r ah1 n t))
("gruntal" nil (g r ah1 n t ax l))
("grunted" nil (g r ah1 n t ih0 d))
("grunts" nil (g r ah1 n t s))
("grunwald" nil (g r ah1 n w ao1 l d))
("grupe" nil (g r uw1 p))
("grupo" nil (g r uw1 p ow0))
("grupp" nil (g r ah1 p))
("gruppo" nil (g r uw1 p ow0))
("grush" nil (g r ah1 sh))
("grushow" nil (g r ah1 sh ow0))
("gruss" nil (g r ah1 s))
("grussing" nil (g r ah1 s ih0 ng))
("gruszka" nil (g r ah1 sh k ax))
("grutman" nil (g r ah1 t m ax n))
("gruver" nil (g r uw1 v er0))
("gruwell" nil (g r ah1 w eh0 l))
("gryder" nil (g r ay1 d er0))
("grygiel" nil (g r ih1 jh iy0 l))
("gryphons" nil (g r ih1 f ax n z))
("grzelak" nil (g er1 z ax l ae1 k))
("grzesiak" nil (g er0 z eh1 s iy0 ae0 k))
("grzeskowiak" nil (g er0 z ih0 s k aw1 iy0 ae0 k))
("grzyb" nil (g er1 z ih1 b))
("grzybowski" nil (g er1 z ih0 b aw1 s k iy0))
("grzywacz" nil (g er1 z ih0 v ae1 ch))
("grzywinski" nil (g er0 z ih0 w ih1 n s k iy0))
("gschwind" nil (g ax sh w ay1 n d))
("gsell" nil (g ax s eh1 l))
("gtech" nil (g ax t eh1 k))
("gu" nil (g uw1))
("guadagno" nil (g aa0 d aa1 g n ow0))
("guadalajara" nil (g w aa1 d ah1 l ax hh aa1 r ax))
("guadalcanal" nil (g w aa1 d ax l k ax n ae1 l))
("guadalupe" nil (g w aa1 d ax l uw1 p))
("guadarrama" nil (g uw0 aa0 d aa0 r aa1 m ax))
("guadeloupe" nil (g w aa1 d ax l uw1 p))
("guagliardo" nil (g w aa1 g l iy0 aa1 r d ow0))
("guajardo" nil (g w aa0 y aa1 r d ow0))
("gualdoni" nil (g aa0 l d ow1 n iy0))
("gualtieri" nil (g aa0 l t ih1 r iy0))
("guam" nil (g w aa1 m))
("guanaco" nil (g w ax n aa1 k ow1))
("guandjo" nil (g w aa1 n jh ow1))
("guandjong" nil (g w aa1 n jh ow1 ng))
("guandong" nil (g w aa1 n d ow1 ng))
("guangdong" nil (g w ae1 ng d ao1 ng))
("guangjo" nil (g w aa1 ng jh ow1))
("guangzhou" nil (g w ae1 ng z uw1))
("guanine" nil (g w aa1 n iy1 n))
("guano" nil (g w aa1 n ow1))
("guantanamo" nil (g w aa1 n t aa1 n ax m ow1))
("guarani" nil (g w aa1 r ax n iy1))
("guarantee" nil (g eh1 r ax n t iy1))
("guaranteed" nil (g eh1 r ax n t iy1 d))
("guaranteeing" nil (g eh1 r ax n t iy1 ih0 ng))
("guarantees" nil (g eh1 r ax n t iy1 z))
("guaranties" nil (g eh1 r ax n t iy1 z))
("guarantor" nil (g eh1 r ax n t ao1 r))
("guarantors" nil (g eh1 r ax n t ao1 r z))
("guaranty" nil (g eh1 r ax n t iy1))
("guard" nil (g aa1 r d))
("guardado" nil (g aa0 r d aa1 d ow0))
("guarded" nil (g aa1 r d ax d))
("guardedly" nil (g aa1 r d ih0 d l iy0))
("guardfish" nil (g aa1 r d f ih1 sh))
("guardia" nil (g w aa1 r d iy0 ax))
("guardian" nil (g aa1 r d iy0 ax n))
("guardians" nil (g aa1 r d iy0 ax n z))
("guardianship" nil (g aa1 r d iy0 ax n sh ih0 p))
("guarding" nil (g aa1 r d ih0 ng))
("guardino" nil (g aa0 r d iy1 n ow0))
("guardiola" nil (g aa0 r d iy0 ow1 l ax))
("guardrails" nil (g aa1 r d r ey1 l z))
("guards" nil (g aa1 r d z))
("guardsman" nil (g aa1 r d z m ae1 n))
("guardsmen" nil (g aa1 r d z m ih0 n))
("guariglia" nil (g aa0 r iy1 g l iy0 ax))
("guarini" nil (g aa0 r iy1 n iy0))
("guarino" nil (g aa0 r iy1 n ow0))
("guarisco" nil (g aa0 r iy1 s k ow0))
("guarnaccia" nil (g aa0 r n ae1 ch iy0 ax))
("guarneri" nil (g aa0 r n eh1 r iy0))
("guarnieri" nil (g aa0 r n ih1 r iy0))
("guasch" nil (g w ae1 sh))
("guastella" nil (g aa0 s t eh1 l ax))
("guatemala" nil (g w aa1 t ax m aa1 l ax))
("guatemalan" nil (g w aa1 t ax m aa1 l ax n))
("guatemalans" nil (g w aa1 t ax m aa1 l ax n z))
("guattery" nil (g w aa1 t er0 iy0))
("guavas" nil (g w aa1 v ax z))
("guay" nil (g ey1))
("guayabera" nil (g w ay1 ax b eh1 r ax))
("guba" nil (g uw1 b ax))
("gubbels" nil (g ah1 b ax l z))
("gubbins" nil (g ah1 b ih0 n z))
("guber" nil (g y uw1 b er0))
("guberman" nil (g uw1 b er0 m ax n))
("gubernatorial" nil (g uw0 b er0 n ax t ao1 r iy0 ax l))
("gubler" nil (g y uw1 b ax l er0))
("gubser" nil (g ah1 b s er0))
("gucci" nil (g uw1 ch iy0))
("gucciardo" nil (g uw0 ch aa1 r d ow0))
("guccio" nil (g uw1 ch iy0 ow0))
("guccione" nil (g uw0 k ch ow1 n iy0))
("guck" nil (g ah1 k))
("guckert" nil (g ah1 k er0 t))
("gucwa" nil (g ah1 k w ax))
("gude" nil (g y uw1 d))
("guderian" nil (g y uw1 d iy1 r iy0 ax n))
("gudgel" nil (g ah1 jh ax l))
("gudgeon" nil (g ah1 jh ax n))
("gudger" nil (g ah1 jh er0))
("gudino" nil (g uw0 d iy1 n ow0))
("gudmundson" nil (g ah1 d m ax n d s ax n))
("gue" nil (g y uw1))
("guedry" nil (g eh1 d r iy0))
("guel" nil (g eh1 l))
("guenette" nil (g ih0 n eh1 t))
("guenna" nil (g ey1 n ax))
("guenter" nil (g eh1 n t er0))
("guenther" nil (g uh1 n th er0))
("guenthner" nil (g eh1 n th n er0))
("guerard" nil (g er0 aa1 r d))
("guercio" nil (g eh1 r s iy0 ow0))
("guerette" nil (g er0 eh1 t))
("guerilla" nil (g er0 ih1 l ax))
("guerillas" nil (g er0 ih1 l ax z))
("guerin" nil (g eh1 r ih0 n))
("guerino" nil (g eh0 r iy1 n ow0))
("guernsey" nil (g er1 n z iy0))
("guerra" nil (g w eh1 r ax))
("guerrant" nil (g eh1 r ax n t))
("guerre" nil (g eh1 r))
("guerrera" nil (g w er0 eh1 r ax))
("guerrero" nil (g er0 eh1 r ow0))
("guerrette" nil (g er0 eh1 t))
("guerrier" nil (g eh1 r iy0 er0))
("guerrieri" nil (g er0 ih1 r iy0))
("guerriero" nil (g er0 ih1 r ow0))
("guerrilla" nil (g er0 ih1 l ax))
("guerrillas" nil (g er0 ih1 l ax z))
("guerrini" nil (g er0 iy1 n iy0))
("guerry" nil (g er0 iy1))
("guertin" nil (g eh0 r t iy1 n))
("guess" nil (g eh1 s))
("guessed" nil (g eh1 s t))
("guesser" nil (g eh1 s er0))
("guessers" nil (g eh1 s er0 z))
("guesses" nil (g eh1 s ax z))
("guessing" nil (g eh1 s ih0 ng))
("guesstimate" nil (g eh1 s t ih0 m ih0 t))
("guesstimates" nil (g eh1 s t ih0 m ih0 t s))
("guesswork" nil (g eh1 s w er1 k))
("guest" nil (g eh1 s t))
("guests" nil (g eh1 s t s))
("guettler" nil (g eh1 t ax l er0))
("guevara" nil (g ey0 v aa1 r ax))
("guez" nil (g eh1 z))
("guffaws" nil (g ax f ao1 z))
("guffey" nil (g ah1 f iy0))
("guffin" nil (g ah1 f ih0 n))
("guffy" nil (g ah1 f iy0))
("gugel" nil (g uw1 g ax l))
("guggenheim" nil (g uw1 g ax n hh ay1 m))
("guggisberg" nil (g ah1 g ih0 s b er0 g))
("gugino" nil (g uw0 jh iy1 n ow0))
("guglielmetti" nil (g uw0 g l iy0 eh0 l m eh1 t iy0))
("guglielmi" nil (g uw0 g l iy0 eh1 l m iy0))
("guglielmo" nil (g uw0 g l iy0 eh1 l m ow0))
("gugliotta" nil (g uw0 g l iy0 ow1 t ax))
("gugliotti" nil (g uw0 g l iy0 ow1 t iy0))
("gugliuzza" nil (g uw0 g l iy0 uw1 t s ax))
("guhl" nil (g ah1 l))
("guiana" nil (g iy0 aa1 n ax))
("guice" nil (g ay1 s))
("guichard" nil (g ih0 sh aa1 r d))
("guida" nil (g iy1 d ax))
("guidance" nil (g ay1 d ax n s))
("guide" nil (g ay1 d))
("guidebook" nil (g ay1 d b uh1 k))
("guidebooks" nil (g ay1 d b uh1 k s))
("guided" nil (g ay1 d ax d))
("guideline" nil (g ay1 d l ay1 n))
("guidelines" nil (g ay1 d l ay1 n z))
("guidepost" nil (g ay1 d p ow1 s t))
("guideposts" nil (g ay1 d p ow1 s t s))
("guider" nil (g ay1 d er0))
("guidera" nil (g iy0 d eh1 r ax))
("guides" nil (g ay1 d z))
("guidi" nil (g iy1 d iy0))
("guidice" nil (g ay1 d ih0 s))
("guiding" nil (g ay1 d ih0 ng))
("guido" nil (g w iy1 d ow0))
("guidone" nil (g iy0 d ow1 n iy0))
("guidotti" nil (g iy0 d ow1 t iy0))
("guidroz" nil (g w iy1 d r ow0 z))
("guidry" nil (g ih1 d er0 iy0))
("guier" nil (g ay1 er0))
("guiffre" nil (g ih1 f er0))
("guilbault" nil (g ih0 l b ow1))
("guilbeau" nil (g ih0 l b ow1))
("guilbeault" nil (g ih0 l b ow1))
("guilbeaux" nil (g ih0 l b ow1))
("guilbert" nil (g ih1 l b er0 t))
("guild" nil (g ih1 l d))
("guilder" nil (g ih1 l d er0))
("guilders" nil (g ih1 l d er0 z))
("guildhall" nil (g ih1 l d hh ao1 l))
("guilds" nil (g ih1 l d z))
("guile" nil (g ay1 l))
("guileless" nil (g ay1 l l ax s))
("guiles" nil (g ay1 l z))
("guilfoil" nil (g ih0 l f oy1 l))
("guilford" nil (g ih1 l f er0 d))
("guilfoyle" nil (g ih1 l f oy0 l))
("guiliani" nil (g iy1 l iy0 aa1 n iy0))
("guiliano" nil (g iy1 l iy0 aa1 n ow0))
("guilin" nil (g ih1 l ih0 n))
("guill" nil (g ih1 l))
("guilla" nil (g w ih1 l ax))
("guillaume" nil (g ih1 l aw1 m))
("guillemette" nil (g ih1 l ax m eh1 t))
("guillen" nil (g ih1 l ax n))
("guillermo" nil (g w ih0 l y eh1 r m ow0))
("guillet" nil (g ih0 l eh1 t))
("guillette" nil (g ih0 l eh1 t))
("guilliams" nil (g w iy0 l iy1 aa0 m z))
("guillory" nil (g ih1 l er0 iy0))
("guillot" nil (g ih0 l ow1))
("guillotine" nil (g ih1 l ax t iy1 n))
("guillotte" nil (g ih0 l ao1 t))
("guilmartin" nil (g ih0 l m aa1 r t ih0 n))
("guilmette" nil (g ih0 l m eh1 t))
("guilt" nil (g ih1 l t))
("guilty" nil (g ih1 l t iy0))
("guimaraes" nil (g ih1 m er0 ey1 z))
("guimond" nil (g ih0 m aa1 n d))
("guimont" nil (g ih0 m aa1 n t))
("guin" nil (g ih1 n))
("guinan" nil (g w iy0 n aa1 n))
("guinarou" nil (g w ih0 n aa1 r ow1))
("guindon" nil (g ih1 n d ax n))
("guinea" nil (g ih1 n iy0))
("guineas" nil (g ih1 n iy0 z))
("guiney" nil (g ih1 n iy0))
("guinier" nil (g w ih0 n ih1 r))
("guinn" nil (g ih1 n))
("guinness" nil (g ih1 n ax s))
("guinta" nil (g w iy1 n t ax))
("guinther" nil (g ih1 n dh er0))
("guinto" nil (g w iy1 n t ow0))
("guinyard" nil (g ih0 n y aa1 r d))
("guion" nil (g w iy0 ao1 n))
("guire" nil (g ay1 r))
("guise" nil (g ay1 z))
("guises" nil (g ay1 z ax z))
("guisewite" nil (g ay1 s w ay1 t))
("guisinger" nil (g ih1 s ih0 n jh er0))
("guitar" nil (g ih0 t aa1 r))
("guitarist" nil (g ih1 t aa1 r ax s t))
("guitarists" nil (g ih1 t aa1 r ax s t s))
("guitarro" nil (g ih0 t aa1 r ow0))
("guitars" nil (g ih0 t aa1 r z))
("guiterrez" nil (g w iy0 t eh1 r eh0 z))
("guitierrez" nil (g w iy0 t ih1 r eh0 z))
("guizar" nil (g w iy0 z aa1 r))
("gujarat" nil (g y uw1 jh er0 ae1 t))
("gul" nil (g ah1 l))
("gula" nil (g y uw1 l ax))
("gulag" nil (g y uw1 l ae0 g))
("gulas" nil (g y uw1 l ax z))
("gulbrandsen" nil (g ah1 l b r ax n d s ax n))
("gulbrandson" nil (g ah1 l b r ax n d s ax n))
("gulbransen" nil (g ah1 l b r ax n s ax n))
("gulbranson" nil (g ah1 l b r ax n s ax n))
("gulbuddin" nil (g ah1 b uw1 d ih0 n))
("gulch" nil (g ah1 l ch))
("guldin" nil (g ah1 l d ih0 n))
("gulf" nil (g ah1 l f))
("gulfstream" nil (g ah1 l f s t r iy1 m))
("gulick" nil (g y uw1 l ih0 k))
("gulino" nil (g uw0 l iy1 n ow0))
("gull" nil (g ah1 l))
("gulla" nil (g ah1 l ax))
("gullah" nil (g ah1 l ax))
("gullatt" nil (g ah1 l ax t))
("gulledge" nil (g uw1 l ih0 jh))
("gullet" nil (g ah1 l ax t))
("gullett" nil (g uw1 l ih0 t))
("gullette" nil (g uw0 l eh1 t))
("gulley" nil (g ah1 l iy0))
("gulli" nil (g uw1 l iy0))
("gullibility" nil (g ah1 l ax b ih1 l ih0 t iy0))
("gullible" nil (g ah1 l ax b ax l))
("gullick" nil (g ah1 l ih0 k))
("gullickson" nil (g ah1 l ih0 k s ax n))
("gullies" nil (g ah1 l iy0 z))
("gulliford" nil (g ah1 l ih0 f er0 d))
("gullikson" nil (g ah1 l ih0 k s ax n))
("gullion" nil (g ah1 l y ax n))
("gulliver" nil (g ah1 l ih0 v er0))
("gullo" nil (g uw1 l ow0))
("gully" nil (g ah1 l iy0))
("gulp" nil (g ah1 l p))
("gulped" nil (g ah1 l p t))
("gulping" nil (g ah1 l p ih0 ng))
("gulps" nil (g ah1 l p s))
("gulyas" nil (g uw1 l y ax s))
("gum" nil (g ah1 m))
("gumaer" nil (g ah1 m aw0 er0))
("gumbel" nil (g ah1 m b ax l))
("gumbert" nil (g ah1 m b er0 t))
("gumbi" nil (g ah1 m b iy0))
("gumbiner" nil (g ah1 m b ay1 n er0))
("gumbinger" nil (g ah1 m b ih1 ng er0))
("gumbo" nil (g ah1 m b ow0))
("gumbs" nil (g ah1 m z))
("gumi" nil (g uw1 m iy0))
("gumina" nil (g uw0 m iy1 n ax))
("gumm" nil (g ah1 m))
("gummed" nil (g ah1 m d))
("gummi" nil (g y uw1 m iy0))
("gummy" nil (g ah1 m iy0))
("gump" nil (g ah1 m p))
("gumpert" nil (g ah1 m p er1 t))
("gumport" nil (g ah1 m p ao1 r t))
("gumpp" nil (g ah1 m p))
("gumption" nil (g ah1 m p sh ax n))
("gums" nil (g ah1 m z))
("gumshoe" nil (g ah1 m sh uw1))
("gumucio" nil (g y uw1 m y uw1 s iy0 ow0))
("gumz" nil (g ah1 m z))
("gun" nil (g ah1 n))
("gunboat" nil (g ah1 n b ow1 t))
("gunboats" nil (g ah1 n b ow1 t s))
("gunby" nil (g ah1 n b iy0))
("guncotton" nil (g ah1 n k aa1 t ax n))
("gundel" nil (g ah1 n d ax l))
("gunder" nil (g ah1 n d er0))
("gunderman" nil (g ah1 n d er0 m ax n))
("gundersen" nil (g ah1 n d er0 s ax n))
("gunderson" nil (g ah1 n d er0 s ax n))
("gundlach" nil (g ah1 n d l ax k))
("gundle" nil (g ah1 n d ax l))
("gundog" nil (g ah1 n d ao1 g))
("gundrum" nil (g ah1 n d r ax m))
("gundry" nil (g ah1 n d er0 iy0))
("gundy" nil (g ah1 n d iy0))
("gunfight" nil (g ah1 n f ay1 t))
("gunfighter" nil (g ah1 n f ay1 t er0))
("gunfighters" nil (g ah1 n f ay1 t er0 z))
("gunfighting" nil (g ah1 n f ay1 t ih0 ng))
("gunfights" nil (g ah1 n f ay1 t s))
("gunfire" nil (g ah1 n f ay1 er0))
("gunflint" nil (g ah1 n f l ih0 n t))
("gung" nil (g ah1 ng))
("gunhilda" nil (g uw0 n hh iy1 l d ax))
("gunia" nil (g uw1 n iy0 ax))
("gunkel" nil (g ah1 ng k ax l))
("gunlicks" nil (g ah1 n l ih0 k s))
("gunman" nil (g ah1 n m ax n))
("gunmen" nil (g ah1 n m eh1 n))
("gunn" nil (g ah1 n))
("gunnar" nil (g ah1 n er0))
("gunnarson" nil (g ah1 n aa0 r s ax n))
("gunned" nil (g ah1 n d))
("gunnell" nil (g ah1 n ax l))
("gunnells" nil (g ah1 n ax l z))
("gunnels" nil (g ah1 n ax l z))
("gunner" nil (g ah1 n er0))
("gunners" nil (g ah1 n er0 z))
("gunnerson" nil (g ah1 n er0 s ax n))
("gunnery" nil (g ah1 n er0 iy0))
("gunning" nil (g ah1 n ih0 ng))
("gunnison" nil (g ah1 n ih0 s ax n))
("gunnoe" nil (g ah1 n ow0))
("gunny" nil (g ah1 n iy0))
("gunpoint" nil (g ah1 n p oy1 n t))
("gunpowder" nil (g ah1 n p aw1 d er0))
("gunrunner" nil (g ah1 n r ah1 n er0))
("guns" nil (g ah1 n z))
("gunsalus" nil (g ax n s aa1 l ax s))
("gunship" nil (g ah1 n sh ih1 p))
("gunships" nil (g ah1 n sh ih1 p s))
("gunshot" nil (g ah1 n sh aa1 t))
("gunshots" nil (g ah1 n sh aa1 t s))
("gunslinger" nil (g ah1 n s l ih1 ng er0))
("gunslingers" nil (g ah1 n s l ih1 ng er0 z))
("gunst" nil (g ah1 n s t))
("gunter" nil (g ah1 n t er0))
("gunther" nil (g ah1 n th er0))
("guntur" nil (g ah1 n t er0))
("gunty" nil (g ah1 n t iy0))
("guppies" nil (g ah1 p iy0 z))
("gupta" nil (g uh1 p t ax))
("guptill" nil (g ah1 p t ih0 l))
("gupton" nil (g ah1 p t ax n))
("gura" nil (g uh1 r ax))
("gural" nil (g y uh1 r ax l))
("gurevich" nil (g y uw1 r ih0 v ih0 ch))
("gurganious" nil (g er1 g ax n iy0 ih0 s))
("gurganus" nil (g er1 g ax n ih0 s))
("gurgle" nil (g er1 g ax l))
("gurgling" nil (g er1 g ax l ih0 ng))
("gurian" nil (g y uh1 r iy0 ax n))
("gurion" nil (g y uh1 r iy0 ax n))
("gurit" nil (g y uh1 r ih0 t))
("gurka" nil (g er1 k ax))
("gurkin" nil (g er1 k ih0 n))
("gurley" nil (g er1 l iy0))
("gurnards" nil (g er1 n er0 d z))
("gurnee" nil (g er0 n iy1))
("gurney" nil (g er1 n iy0))
("gurnsey" nil (g er1 n s iy0))
("gurr" nil (g er1))
("gurria" nil (g er1 iy0 ax))
("gurrola" nil (g uh0 r ow1 l ax))
("gurry" nil (g er1 iy0))
("gurski" nil (g er1 s k iy0))
("gursky" nil (g er1 s k iy0))
("gurtler" nil (g er1 t ax l er0))
("gurtner" nil (g er1 t n er0))
("gurtz" nil (g er1 t s))
("guru" nil (g uw1 r uw1))
("gurule" nil (g er0 uw1 l))
("gurus" nil (g uw1 r uw1 z))
("gus" nil (g ah1 s))
("guse" nil (g y uw1 z))
("gusella" nil (g y uw1 s eh1 l ax))
("guseman" nil (g uw1 s m ax n))
("gush" nil (g ah1 sh))
("gushed" nil (g ah1 sh t))
("gusher" nil (g ah1 sh er0))
("gushers" nil (g ah1 sh er0 z))
("gushes" nil (g ah1 sh ih0 z))
("gushing" nil (g ah1 sh ih0 ng))
("gusinsky" nil (g ax s ih1 n s k iy0))
("gusky" nil (g ah1 s k iy0))
("gusler" nil (g ah1 s ax l er0))
("gusman" nil (g ah1 s m ax n))
("guss" nil (g ah1 s))
("gussie" nil (g ah1 s iy0))
("gussied" nil (g ah1 s iy0 d))
("gussman" nil (g ah1 s m ax n))
("gussy" nil (g ah1 s iy0))
("gust" nil (g ah1 s t))
("gustaf" nil (g uw1 s t aa0 f))
("gustafson" nil (g ah1 s t ax f s ax n))
("gustafsson" nil (g uw1 s t aa0 f s ax n))
("gustav" nil (g uh1 s t aa0 f))
("gustava" nil (g uw0 s t aa1 v ax))
("gustave" nil (g uw0 s t aa1 v iy0))
("gustaveson" nil (g uw0 s t aa0 v ey1 s ax n))
("gustavo" nil (g ax s t aa1 v ow0))
("gustavson" nil (g ah1 s t ax v s ax n))
("guste" nil (g ah1 s t))
("gusted" nil (g ah1 s t ih0 d))
("guster" nil (g ah1 s t er0))
("gustin" nil (g ah1 s t ih0 n))
("gusto" nil (g ah1 s t ow1))
("guston" nil (g ah1 s t ax n))
("gusts" nil (g ah1 s t s))
("gustus" nil (g ah1 s t ax s))
("gusty" nil (g ah1 s t iy0))
("gut" nil (g ah1 t))
("gutekunst" nil (g ah1 t ih0 k ax n s t))
("guterman" nil (g y uw1 t er0 m ax n))
("gutermuth" nil (g ah1 t er0 m uw0 th))
("gutfeld" nil (g ah1 t f eh1 l d))
("gutfreund" nil (g uh1 t f r eh1 n d))
("guth" nil (g uw1 th))
("gutherie" nil (g ah1 th er0 iy0))
("guthery" nil (g ah1 th er0 iy0))
("guthmiller" nil (g ah1 th m ax l er0))
("guthridge" nil (g ah1 th r ih0 jh))
("guthrie" nil (g ah1 th r iy0))
("gutierez" nil (g uw0 t ih1 r eh0 z))
("gutierres" nil (g uw0 t y eh1 r eh0 z))
("gutierrez" nil (g uw0 t y eh1 r eh0 z))
("gutknecht" nil (g uw1 t n eh0 k t))
("gutkowski" nil (g ax t k ao1 f s k iy0))
("gutman" nil (g ah1 t m ax n))
("gutmann" nil (g ah1 t m ax n))
("gutowski" nil (g ax t ao1 f s k iy0))
("gutridge" nil (g ah1 t r ih1 jh))
("guts" nil (g ah1 t s))
("gutshall" nil (g ah1 ch ax l))
("gutsy" nil (g ah1 t s iy1))
("gutt" nil (g ah1 t))
("gutted" nil (g ah1 t ih0 d))
("guttenberg" nil (g ah1 t ax n b er0 g))
("gutter" nil (g ah1 t er0))
("guttered" nil (g ah1 t er0 d))
("guttering" nil (g ah1 t er0 ih0 ng))
("gutterman" nil (g ah1 t er0 m ax n))
("gutters" nil (g ah1 t er0 z))
("gutting" nil (g ah1 t ih0 ng))
("guttmacher" nil (g ah1 t m aa1 k er0))
("guttman" nil (g ah1 t m ax n))
("guttmann" nil (g ah1 t m ax n))
("guttural" nil (g ah1 t er0 ax l))
("gutwein" nil (g ah1 t w ay1 n))
("gutzman" nil (g ah1 t s m ax n))
("gutzmer" nil (g ah1 t s m er0))
("gutzwiller" nil (g ah1 t s w ih1 l er0))
("guy" nil (g ay1))
("guyana" nil (g ay1 aa1 n ax))
("guyer" nil (g ay1 er0))
("guyett" nil (g ay1 eh1 t))
("guyette" nil (g ay1 eh1 t))
("guymon" nil (g ay1 m ax n))
("guynes" nil (g ay1 n z))
("guynn" nil (g ay1 n))
("guyon" nil (g ay1 ax n))
("guyot" nil (g ay1 ax t))
("guys" nil (g ay1 z))
("guyton" nil (g ay1 t ax n))
("guza" nil (g y uw1 z ax))
("guzek" nil (g uw1 z eh0 k))
("guzik" nil (g y uw1 z ih0 k))
("guzman" nil (g ah1 z m ax n))
("guzowski" nil (g ax z ao1 f s k iy0))
("guzy" nil (g y uw1 z iy0))
("guzzardo" nil (g uw0 t s aa1 r d ow0))
("guzzetta" nil (g uw0 t s eh1 t ax))
("guzzetti" nil (g y uw0 z eh1 t iy0))
("guzzi" nil (g uw1 t s iy0))
("guzzle" nil (g ah1 z ax l))
("guzzler" nil (g ah1 z l er0))
("guzzlers" nil (g ah1 z ax l er0 z))
("guzzling" nil (g ah1 z ax l ih0 ng))
("guzzo" nil (g uw1 z ow0))
("gvaryahu" nil (g ax v eh0 r y aa1 hh uw0))
("gwaltney" nil (g w ao1 l t n iy0))
("gwartney" nil (g w ao1 r t n iy0))
("gwen" nil (g w eh1 n))
("gwenda" nil (g w eh1 n d ax))
("gwendolyn" nil (g w eh1 n d ax l ih0 n))
("gwennie" nil (g w eh1 n iy0))
("gwenore" nil (g w eh1 n er0))
("gwin" nil (g w ih1 n))
("gwinn" nil (g w ih1 n))
("gwinner" nil (g w ih1 n er0))
("gwinnett" nil (g w ih0 n eh1 t))
("gwizdala" nil (g w ih0 z d aa1 l ax))
("gwozdz" nil (g w aa1 z d z))
("gwyn" nil (g w ih1 n))
("gwynn" nil (g w ih1 n))
("gwynne" nil (g w ih1 n))
("gyger" nil (g ay1 g er0))
("gyi" nil (g iy1))
("gyles" nil (jh ay1 l z))
("gyllenhammar" nil (jh ih1 l eh0 n hh ae1 m er0))
("gym" nil (jh ih1 m))
("gymboree" nil (jh ih1 m b ao0 r iy1))
("gymnasia" nil (jh ih0 m n ey1 z iy0 ax))
("gymnasium" nil (jh ih0 m n ey1 z iy0 ax m))
("gymnasiums" nil (jh ih0 m n ey1 z iy0 ax m z))
("gymnast" nil (jh ih1 m n ax s t))
("gymnastic" nil (jh ih0 m n ae1 s t ih0 k))
("gymnastics" nil (jh ih0 m n ae1 s t ih0 k s))
("gymnasts" nil (jh ih1 m n ae0 s t s))
("gyms" nil (jh ih1 m z))
("gynecologic" nil (g ay1 n ax k ax l aa1 jh ih0 k))
("gynecological" nil (g ay1 n ax k ax l aa1 jh ih0 k ax l))
("gynecologist" nil (g ay1 n ax k aa1 l ax jh ax s t))
("gynecologists" nil (g ay1 n ax k aa1 l ax jh ax s t s))
("gynecology" nil (g ay1 n ax k aa1 l ax jh iy0))
("gynex" nil (jh ih1 n ax k s))
("gyohten" nil (g y ow1 t ax n))
("gyosai" nil (g y ow1 s ey1))
("gypped" nil (jh ih1 p t))
("gypsies" nil (jh ih1 p s iy0 z))
("gypsum" nil (jh ih1 p s ax m))
("gypsy" nil (jh ih1 p s iy0))
("gyr" nil (jh ay1 r))
("gyrate" nil (jh ay1 r ey1 t))
("gyrated" nil (jh ay1 r ey1 t ih0 d))
("gyrating" nil (jh ay1 r ey1 t ih0 ng))
("gyrations" nil (jh ay0 r ey1 sh ax n z))
("gyro" nil (jh ay1 r ow1))
("gyrocompass" nil (jh ay1 r ow0 k ah1 m p ax s))
("gyropilot" nil (jh ay1 r ow0 p ay1 l ax t))
("gyros" nil (jh ay1 r ow1 z))
("gyroscope" nil (jh ay1 r ax s k ow1 p))
("gyroscopes" nil (jh ay1 r ax s k ow1 p s))
("gyroscopic" nil (jh ay1 r ax s k aa1 p ih0 k))
("gytha" nil (jh ay1 dh ax))
("gyuhama" nil (g y uw1 hh aa1 m ax))
("h" nil (ey1 ch))
("ha" nil (hh aa1))
("haab" nil (hh aa1 b))
("haack" nil (hh aa1 k))
("haacke" nil (hh aa1 k))
("haaf" nil (hh aa1 f))
("haag" nil (hh aa1 g))
("haagen" nil (hh aa1 g ax n))
("haagenson" nil (hh aa1 g ih0 n s ax n))
("haak" nil (hh aa1 k))
("haake" nil (hh aa1 k))
("haakenson" nil (hh aa1 k ih0 n s ax n))
("haaland" nil (hh aa1 l ax n d))
("haan" nil (hh aa1 n))
("haapala" nil (hh aa1 p aa1 l ax))
("haar" nil (hh aa1 r))
("haas" nil (hh aa1 s))
("haase" nil (hh aa1 s))
("haass" nil (hh aa1 s))
("haavelmo" nil (hh aa1 v eh1 l m ow0))
("habben" nil (hh ae1 b ax n))
("habeas" nil (hh ae1 b iy0 ax s))
("habeck" nil (hh aa1 b eh0 k))
("habecker" nil (hh ae1 b eh0 k er0))
("habeeb" nil (hh ae1 b iy0 b))
("habegger" nil (hh ae1 b ih0 g er0))
("habel" nil (hh ae1 b ax l))
("habenicht" nil (hh ae1 b ih0 n ih0 k t))
("haber" nil (hh ey1 b er0))
("haberdashery" nil (hh ae1 b er0 d ae1 sh er0 iy0))
("haberer" nil (hh ae1 b er0 er0))
("haberkorn" nil (hh ae1 b er0 k er0 n))
("haberl" nil (hh ae1 b er0 l))
("haberland" nil (hh ae1 b er0 l ax n d))
("haberle" nil (hh ae1 b er0 ax l))
("haberman" nil (hh ey1 b er0 m ax n))
("habermann" nil (hh ey1 b er0 m ax n))
("habermehl" nil (hh ae1 b er0 m ax l))
("haberstroh" nil (hh aa0 b eh1 r s t r ow0))
("habib" nil (hh ax b iy1 b))
("habibie" nil (hh ax b iy1 b iy0))
("habich" nil (hh ae1 b ih0 k))
("habicht" nil (hh ae1 b ih0 k t))
("habig" nil (hh ae1 b ih0 g))
("habiger" nil (hh ae1 b ih0 g er0))
("habit" nil (hh ae1 b ax t))
("habitable" nil (hh ae1 b ax t ax b ax l))
("habitat" nil (hh ae1 b ax t ae1 t))
("habitation" nil (hh ae1 b ax t ey1 sh ax n))
("habitats" nil (hh ae1 b ax t ae1 t s))
("habits" nil (hh ae1 b ax t s))
("habitual" nil (hh ax b ih1 ch uw0 ax l))
("habitually" nil (hh ax b ih1 ch uw0 ax l iy0))
("habitues" nil (hh ae1 b ih0 ch uw0 z))
("hable" nil (hh ey1 b ax l))
("habsburg" nil (hh ae1 p s b er0 g))
("haby" nil (hh ey1 b iy0))
("hach" nil (hh ae1 ch))
("hachette" nil (hh ax sh eh1 t))
("hachey" nil (hh ae1 ch iy0))
("hachtel" nil (hh ae1 k t ax l))
("hacienda" nil (hh ae1 s iy0 eh1 n d ax))
("hack" nil (hh ae1 k))
("hackathorn" nil (hh ae1 k ax th er0 n))
("hackbart" nil (hh ae1 k b aa1 r t))
("hackbarth" nil (hh ae1 k b aa1 r th))
("hackberry" nil (hh ae1 k b eh1 r iy0))
("hacke" nil (hh ae1 k))
("hacked" nil (hh ae1 k t))
("hackel" nil (hh ae1 k ax l))
("hackenberg" nil (hh ae1 k ax n b er0 g))
("hackensack" nil (hh ae1 k ax n s ae1 k))
("hacker" nil (hh ae1 k er0))
("hackers" nil (hh ae1 k er0 z))
("hackert" nil (hh ae1 k er0 t))
("hackett" nil (hh ae1 k ih0 t))
("hackford" nil (hh ae1 k f er0 d))
("hacking" nil (hh ae1 k ih0 ng))
("hackl" nil (hh ae1 k ax l))
("hackleman" nil (hh ae1 k ax l m ax n))
("hackler" nil (hh ae1 k ax l er0))
("hackles" nil (hh ae1 k ax l z))
("hackley" nil (hh ae1 k l iy0))
("hackman" nil (hh ae1 k m ae1 n))
("hackmann" nil (hh ae1 k m ax n))
("hackmatack" nil (hh ae1 k m ax t ae1 k))
("hackney" nil (hh ae1 k n iy0))
("hackneyed" nil (hh ae1 k n iy0 d))
("hacks" nil (hh ae1 k s))
("hacksaw" nil (hh ae1 k s ao1))
("hackstaff" nil (hh ae1 k s t ae1 f))
("hackworth" nil (hh ae1 k w er1 th))
("had" nil (hh ae1 d))
("hada" nil (hh aa1 d ax))
("hadad" nil (hh ae1 d ax d))
("hadaway" nil (hh aa1 d ax w ey0))
("hadd" nil (hh ae1 d))
("haddad" nil (hh ae1 d ax d))
("haddan" nil (hh ae1 d ax n))
("haddaway" nil (hh ae1 d ax w ey0))
("hadden" nil (hh ae1 d ax n))
("haddock" nil (hh ae1 d ax k))
("haddon" nil (hh ae1 d ax n))
("haddow" nil (hh ae1 d ow0))
("hade" nil (hh ey1 d))
("haden" nil (hh ey1 d ax n))
("hader" nil (hh ey1 d er0))
("hades" nil (hh ey1 d iy0 z))
("hadfield" nil (hh ae1 d f iy1 l d))
("hadid" nil (hh aa0 d iy1 d))
("hadleigh" nil (hh ae1 d l ax))
("hadler" nil (hh ey1 d ax l er0))
("hadley" nil (hh ae1 d l iy0))
("hadlock" nil (hh ae1 d l ax k))
("hadnot" nil (hh ae1 d n ax t))
("hadria" nil (hh ae1 d r iy0 ax))
("hadrian" nil (hh ae1 d r iy0 ax n))
("hadsall" nil (hh ae1 d s ax l))
("hadsell" nil (hh ae1 d s ax l))
("hadson" nil (hh ae1 d s ax n))
("hadwin" nil (hh ae1 d w ih0 n))
("haeberle" nil (hh eh1 b er0 ax l))
("haecker" nil (hh eh1 k er0))
("haefele" nil (hh eh1 f ax l))
("haeffner" nil (hh eh1 f n er0))
("haefner" nil (hh eh1 f n er0))
("haegele" nil (hh eh1 g ax l))
("haeger" nil (hh eh1 g er0))
("haen" nil (hh iy1 n))
("haenel" nil (hh eh1 n ax l))
("haering" nil (hh aa1 er0 ih0 ng))
("haertel" nil (hh eh1 r t ax l))
("haese" nil (hh iy1 s))
("haessly" nil (hh ae1 s l iy0))
("haeussler" nil (hh aw1 s l er0))
("hafele" nil (hh ae1 f ax l))
("hafeman" nil (hh ey1 f m ax n))
("hafen" nil (hh ae1 f ax n))
("hafer" nil (hh ey1 f er0))
("hafey" nil (hh ey1 f iy0))
("hafez" nil (hh aa1 f eh0 z))
("haff" nil (hh ae1 f))
("haffey" nil (hh ae1 f iy0))
("haffner" nil (hh ae1 f n er0))
("hafford" nil (hh ae1 f er0 d))
("hafif" nil (hh aa0 f iy1 f))
("hafley" nil (hh ae1 f l iy0))
("hafner" nil (hh ae1 f n er0))
("hafnia" nil (hh ae1 f n iy0 ax))
("hafnium" nil (hh ae1 f n iy0 ax m))
("haft" nil (hh ae1 f t))
("hafta" nil (hh ae1 f t ax))
("hafts" nil (hh ae1 f t s))
("hag" nil (hh ae1 g))
("haga" nil (hh aa1 g ax))
("hagadorn" nil (hh ae1 g ax d ao0 r n))
("hagaman" nil (hh ae1 g ax m ax n))
("hagan" nil (hh ey1 g ax n))
("hagans" nil (hh ey1 g ax n z))
("hagar" nil (hh ey1 g aa0 r))
("hagarty" nil (hh ae1 g aa0 r t iy0))
("hagberg" nil (hh ae1 g b er0 g))
("hage" nil (hh ey1 jh))
("hagedorn" nil (hh ae1 g ih0 d er0 n))
("hagee" nil (hh ae1 jh iy0))
("hagel" nil (hh ae1 g ax l))
("hagelin" nil (hh ae1 g ih0 l ih0 n))
("hageman" nil (hh ey1 jh m ax n))
("hagemann" nil (hh ey1 jh m ax n))
("hagemeier" nil (hh ae1 g ih0 m ay0 er0))
("hagemeister" nil (hh ae1 g ih0 m ay0 s t er0))
("hagemeyer" nil (hh ae1 g ih0 m ay0 er0))
("hagen" nil (hh ey1 g ax n))
("hagenbuch" nil (hh ae1 g ih0 n b ax k))
("hagenlocker" nil (hh ey1 g ax n l aa1 k er0))
("hagenow" nil (hh ae1 jh ih0 n ow0))
("hagens" nil (hh ey1 g ax n z))
("hager" nil (hh ey1 g er0))
("hagerman" nil (hh ey1 g er0 m ax n))
("hagerstown" nil (hh ey1 g er0 z t aw1 n))
("hagerty" nil (hh ae1 jh er0 t iy0))
("hagewood" nil (hh ey1 jh w uh0 d))
("hagey" nil (hh ae1 jh iy0))
("hagfish" nil (hh ae1 g f ih0 sh))
("hagg" nil (hh ae1 g))
("haggan" nil (hh ae1 g ax n))
("haggar" nil (hh ae1 g er0))
("haggard" nil (hh ae1 g er0 d))
("haggart" nil (hh ae1 g er0 t))
("haggarty" nil (hh ae1 g aa1 r t iy0))
("hagge" nil (hh ae1 g))
("haggerty" nil (hh ae1 g er0 t iy0))
("haggett" nil (hh ae1 g ih0 t))
("haggins" nil (hh ae1 g ih0 n z))
("haggle" nil (hh ae1 g ax l))
("haggled" nil (hh ae1 g ax l d))
("haggling" nil (hh ae1 g ax l ih0 ng))
("hagglund" nil (hh ae1 g l ax n d))
("haggstrom" nil (hh ae1 g s t r ax m))
("hagin" nil (hh ae1 jh ih0 n))
("hagins" nil (hh ae1 g ih0 n z))
("hagiography" nil (hh ae1 g iy0 aa1 g r ax f iy0))
("hagiwara" nil (hh aa1 g ih0 w aa1 r ax))
("hagle" nil (hh ey1 g ax l))
("hagler" nil (hh ae1 g l er0))
("hagley" nil (hh ae1 g l iy0))
("haglund" nil (hh ae1 g l ax n d))
("hagman" nil (hh ae1 g m ax n))
("hagmann" nil (hh ae1 g m ax n))
("hagner" nil (hh ae1 g n er0))
("hagood" nil (hh ae1 g uh0 d))
("hagopian" nil (hh ax g ow1 p iy0 ax n))
("hagstrom" nil (hh ae1 g s t r ax m))
("hague" nil (hh ey1 g))
("hagwood" nil (hh ae1 g w uh1 d))
("hagy" nil (hh ey1 g iy0))
("hah" nil (hh aa1))
("hahl" nil (hh aa1 l))
("hahm" nil (hh ae1 m))
("hahn" nil (hh aa1 n))
("hahne" nil (hh ey1 n))
("hahner" nil (hh aa1 n er0))
("hahs" nil (hh aa1 s))
("haid" nil (hh ey1 d))
("haidee" nil (hh ey1 d iy0))
("haider" nil (hh ey1 d er0))
("haidet" nil (hh ey1 d ih0 t))
("haifa" nil (hh ey1 f ax))
("haig" nil (hh ey1 g))
("haigh" nil (hh ey1))
("haight" nil (hh ey1 t))
("haigler" nil (hh ey1 g l er0))
("haik" nil (hh ey1 k))
("haiku" nil (hh ay1 k uw0))
("hail" nil (hh ey1 l))
("hailand" nil (hh ey1 l ax n d))
("haile" nil (hh ey1 l))
("hailed" nil (hh ey1 l d))
("hailes" nil (hh ey1 l z))
("hailey" nil (hh ey1 l iy0))
("hailing" nil (hh ey1 l ih0 ng))
("hails" nil (hh ey1 l z))
("hailstones" nil (hh ey1 l s t ow1 n z))
("hailstorm" nil (hh ey1 l s t ao1 r m))
("haim" nil (hh ay1 m))
("haimes" nil (hh ey1 m z))
("haimovitch" nil (hh ey1 m ax v ih0 ch))
("haimovitz" nil (hh ey1 m ax v ih0 t s))
("haimowitz" nil (hh ay1 m ax w ih0 t s))
("hain" nil (hh ey1 n))
("hainan" nil (hh ey1 n ax n))
("hainer" nil (hh ey1 n er0))
("haines" nil (hh ey1 n z))
("hainey" nil (hh ey1 n iy0))
("hainley" nil (hh ey1 n l iy0))
("hainline" nil (hh ey1 n l ay1 n))
("hains" nil (hh ey1 n z))
("hainsworth" nil (hh ey1 n s w er0 th))
("hair" nil (hh eh1 r))
("haircut" nil (hh eh1 r k ah1 t))
("haircuts" nil (hh eh1 r k ah1 t s))
("hairdo" nil (hh eh1 r d uw1))
("hairdos" nil (hh eh1 r d uw1 z))
("hairdresser" nil (hh eh1 r d r eh1 s er0))
("hairdressers" nil (hh eh1 r d r eh1 s er0 z))
("hairdressing" nil (hh eh1 r d r eh1 s ih0 ng))
("haire" nil (hh eh1 r))
("haired" nil (hh eh1 r d))
("hairfield" nil (hh eh1 r f iy1 l d))
("hairgrove" nil (hh ay1 r g r ax v))
("hairiness" nil (hh eh1 r iy0 n ax s))
("hairless" nil (hh eh1 r l ax s))
("hairline" nil (hh eh1 r l ay1 n))
("hairr" nil (hh eh1 r))
("hairs" nil (hh eh1 r z))
("hairspray" nil (hh eh1 r s p r ey1))
("hairston" nil (hh ay1 r s t ax n))
("hairstyle" nil (hh eh1 r s t ay1 l))
("hairy" nil (hh eh1 r iy0))
("haisley" nil (hh ey1 z l iy0))
("haislip" nil (hh ay1 s l ih0 p))
("haist" nil (hh aa1 ih0 s t))
("haisten" nil (hh ay1 s ax n))
("hait" nil (hh ey1 t))
("haith" nil (hh ey1 th))
("haithcock" nil (hh ey1 th k aa1 k))
("haiti" nil (hh ey1 t iy0))
("haitian" nil (hh ey1 sh ax n))
("haitians" nil (hh ey1 sh ax n z))
("haitien" nil (hh ey1 sh ax n))
("haitiens" nil (hh ey1 sh ax n z))
("haizlip" nil (hh ey1 z l ih0 p))
("hajdu" nil (hh aa1 y d uw0))
("hajduk" nil (hh ay1 d ax k))
("hajek" nil (hh ay1 eh0 k))
("hajime" nil (hh aa0 jh iy1 m iy0))
("hajj" nil (hh ae1 jh))
("hajjar" nil (hh aa0 y aa1 r))
("hakala" nil (hh ax k aa1 l ax))
("hakan" nil (hh ey1 k ax n))
("hakanson" nil (hh ae1 k ax n s ax n))
("hake" nil (hh ey1 k))
("hakes" nil (hh ey1 k s))
("hakim" nil (hh aa0 k iy1 m))
("hakko" nil (hh ae1 k ow0))
("hakon" nil (hh ae1 k ax n))
("hakuhodo" nil (hh aa1 k uw1 hh ow1 d ow0))
("hal" nil (hh ae1 l))
("halama" nil (hh aa0 l aa1 m ax))
("halamandaris" nil (hh ae1 l ax m ae1 n d er0 ih0 s))
("halas" nil (hh aa1 l ax z))
("halasz" nil (hh aa1 l ax sh))
("halbach" nil (hh ae1 l b aa0 k))
("halberg" nil (hh ae1 l b er0 g))
("halberstadt" nil (hh ae1 l b er0 sh t ae0 t))
("halberstam" nil (hh ae1 l b er0 s t ae0 m))
("halbert" nil (hh ae1 l b er0 t))
("halbig" nil (hh ae1 l b ih0 g))
("halbrook" nil (hh ae1 l b r uh0 k))
("halbrooks" nil (hh ae1 l b r uh0 k s))
("halbur" nil (hh ae1 l b er0))
("halcion" nil (hh ae1 l s iy0 ax n))
("halcomb" nil (hh ae1 l k ax m))
("halcyon" nil (hh ae1 l s iy0 ax n))
("halcyone" nil (hh ae1 l s iy0 ow0 n))
("hald" nil (hh ao1 l d))
("haldan" nil (hh ae1 l d ax n))
("haldana" nil (hh ax l d ae1 n ax))
("haldeman" nil (hh ey1 l d m ax n))
("halden" nil (hh ao1 l d ax n))
("halder" nil (hh ao1 l d er0))
("halderman" nil (hh ao1 l d er0 m ax n))
("hale" nil (hh ey1 l))
("hales" nil (hh ey1 l z))
("haley" nil (hh ey1 l iy0))
("half" nil (hh ae1 f))
("halfacre" nil (hh ae1 f ey1 k er0))
("halfback" nil (hh ae1 f b ae1 k))
("halfdan" nil (hh ae1 l f d ax n))
("halferty" nil (hh ae1 f er0 t iy0))
("halfhearted" nil (hh ae1 f hh aa1 r t ih0 d))
("halfhill" nil (hh ae1 f hh ih1 l))
("halfman" nil (hh ae1 f m ax n))
("halfmann" nil (hh ae1 f m ax n))
("halford" nil (hh ae1 f er0 d))
("halfrida" nil (hh ae1 f r ih0 d ax))
("halftime" nil (hh ae1 f t ay1 m))
("halftone" nil (hh ae1 f t ow1 n))
("halfway" nil (hh ae1 f w ey1))
("halgren" nil (hh ae1 l g r eh0 n))
("haliburton" nil (hh ae1 l ih0 b er1 t ax n))
("halibut" nil (hh ae1 l ax b ax t))
("halide" nil (hh ae1 l ay1 d))
("halifax" nil (hh ae1 l ih0 f ae1 k s))
("halik" nil (hh ae1 l ih0 k))
("halimeda" nil (hh aa0 l iy0 m ey1 d ax))
("halite" nil (hh ae1 l ay0 t))
("halko" nil (hh ae1 l k ow0))
("hall" nil (hh ao1 l))
("halla" nil (hh ae1 l ax))
("halladay" nil (hh ae1 l ax d ey1))
("hallahan" nil (hh ae1 l ax hh ae0 n))
("hallam" nil (hh ae1 l ax m))
("hallanan" nil (hh ae1 l ax n ax n))
("hallandale" nil (hh ae1 l ax n d ey1 l))
("hallas" nil (hh ae1 l ax z))
("hallauer" nil (hh ae1 l aw0 er0))
("hallbauer" nil (hh ao1 l b aw1 r))
("hallberg" nil (hh ao1 l b er0 g))
("halle" nil (hh ae1 l))
("halleck" nil (hh ae1 l ih0 k))
("hallelujah" nil (hh ae1 l ax l uw1 y ax))
("hallen" nil (hh ao1 l ax n))
("hallenbeck" nil (hh ao1 l ax n b eh1 k))
("haller" nil (hh ao1 l er0))
("halleran" nil (hh ae1 l er0 ae0 n))
("hallet" nil (hh ae1 l ih0 t))
("hallett" nil (hh ae1 l ih0 t))
("halley" nil (hh ae1 l iy0))
("hallford" nil (hh ae1 l f er0 d))
("hallgarten" nil (hh ao1 l g aa1 r t ax n))
("hallgren" nil (hh ae1 l g r eh0 n))
("halliburton" nil (hh ae1 l ih0 b er1 t ax n))
("halliday" nil (hh ae1 l ih0 d ey1))
("hallie" nil (hh ao1 l iy0))
("halligan" nil (hh ae1 l ih0 g ax n))
("hallin" nil (hh ae1 l ih0 n))
("hallinan" nil (hh ae1 l ih0 n ax n))
("halling" nil (hh ao1 l ih0 ng))
("hallingby" nil (hh ao1 l ih0 ng b iy0))
("hallisey" nil (hh ae1 l ih0 s iy0))
("hallman" nil (hh ao1 l m ax n))
("hallmark" nil (hh ao1 l m aa1 r k))
("hallmarks" nil (hh ao1 l m aa1 r k s))
("hallock" nil (hh ae1 l ax k))
("halloran" nil (hh ae1 l er0 ax n))
("halloway" nil (hh ae1 l ow0 w ey1))
("hallowed" nil (hh ae1 l ow0 d))
("halloween" nil (hh ae1 l ax w iy1 n))
("hallowell" nil (hh ae1 l ax w eh0 l))
("hallows" nil (hh ae1 l ow0 z))
("hallquist" nil (hh ae1 l k w ih0 s t))
("halls" nil (hh ao1 l z))
("hallstrom" nil (hh ae1 l s t r ax m))
("hallucinations" nil (hh ax l uw1 s ax n ey1 sh ax n z))
("hallucinatory" nil (hh ax l uw1 s ax n ax t ao1 r iy0))
("hallucinogenic" nil (hh ax l uw1 s ax n ax jh eh1 n ih0 k))
("hallum" nil (hh ae1 l ax m))
("hallums" nil (hh ae1 l ax m z))
("hallward" nil (hh ao1 l w er0 d))
("hallway" nil (hh ao1 l w ey1))
("hallways" nil (hh ao1 l w ey1 z))
("hallwood" nil (hh ao1 l w uh1 d))
("hally" nil (hh ae1 l iy0))
("halm" nil (hh aa1 m))
("halmi" nil (hh ao1 l m iy0))
("halmos" nil (hh ao1 l m ow0 s))
("halo" nil (hh ey1 l ow0))
("halogen" nil (hh ae1 l ax jh ax n))
("halogenated" nil (hh ae1 l ax jh ax n ey1 t ih0 d))
("halon" nil (hh ey1 l aa1 n))
("halophytic" nil (hh ae1 l ax f ih1 t ih0 k))
("halos" nil (hh ey1 l ow0 z))
("halper" nil (hh ae1 l p er0))
("halperin" nil (hh ae1 l p er0 ih0 n))
("halpern" nil (hh ae1 l p er0 n))
("halpert" nil (hh ae1 l p er0 t))
("halpin" nil (hh ae1 l p ih0 n))
("halprin" nil (hh ae1 l p r ih0 n))
("halsell" nil (hh ae1 l s ax l))
("halseth" nil (hh ae1 l s ih0 th))
("halsey" nil (hh ae1 l s iy0))
("halstead" nil (hh ae1 l s t eh0 d))
("halsted" nil (hh ae1 l s t ih0 d))
("halston" nil (hh ao1 l s t ax n))
("halt" nil (hh ao1 l t))
("halted" nil (hh ao1 l t ax d))
("halteman" nil (hh ey1 l t m ax n))
("halter" nil (hh ao1 l t er0))
("halterman" nil (hh ao1 l t er0 m ax n))
("halters" nil (hh ao1 l t er0 z))
("halting" nil (hh ao1 l t ih0 ng))
("haltingly" nil (hh ao1 l t ih0 ng l iy0))
("haltiwanger" nil (hh ae1 l t ih0 w ax ng er0))
("haltom" nil (hh ae1 l t ax m))
("halton" nil (hh ae1 l t ax n))
("halts" nil (hh ao1 l t s))
("haluska" nil (hh ax l ah1 s k ax))
("halve" nil (hh ae1 v))
("halved" nil (hh ae1 v d))
("halverson" nil (hh ae1 l v er0 s ax n))
("halves" nil (hh ae1 v z))
("halving" nil (hh ae1 v ih0 ng))
("halvorsen" nil (hh ae0 l v ao1 r s ax n))
("halvorson" nil (hh ae1 l v er0 s ax n))
("ham" nil (hh ae1 m))
("hama" nil (hh aa1 m ax))
("hamacher" nil (hh ae1 m ax k er0))
("hamad" nil (hh ae1 m ax d))
("hamada" nil (hh aa0 m aa1 d ax))
("hamadei" nil (hh ae1 m ax d ey1))
("hamadi" nil (hh ax m aa1 d iy0))
("hamaker" nil (hh aa1 m ey0 k er0))
("hamal" nil (hh ey1 m ax l))
("hamamoto" nil (hh aa0 m aa0 m ow1 t ow0))
("haman" nil (hh ey1 m ax n))
("hamann" nil (hh aa1 m ax n))
("hamar" nil (hh ax m aa1 r))
("hamas" nil (hh aa1 m aa1 s))
("hamasaki" nil (hh aa1 m aa0 s aa1 k iy0))
("hambelton" nil (hh ax m b eh1 l t ax n))
("hamberg" nil (hh ae1 m b er0 g))
("hamberger" nil (hh ae1 m b er0 g er0))
("hamblen" nil (hh ae1 m b ax l ax n))
("hamblet" nil (hh ae1 m b l ih0 t))
("hambleton" nil (hh ae1 m b l ih0 t aa0 n))
("hambley" nil (hh ae1 m b l iy0))
("hamblin" nil (hh ae1 m b l ih0 n))
("hambly" nil (hh ae1 m b l iy0))
("hambrecht" nil (hh ae1 m b r eh1 k t))
("hambrick" nil (hh ae1 m b r ih1 k))
("hambright" nil (hh ae1 m b r ay1 t))
("hambro" nil (hh ae1 m b r ow0))
("hambros" nil (hh ae1 m b r ow0 s))
("hamburg" nil (hh ae1 m b er0 g))
("hamburger" nil (hh ae1 m b er0 g er0))
("hamburgers" nil (hh ae1 m b er0 g er0 z))
("hamby" nil (hh ae1 m b iy0))
("hamdan" nil (hh ae1 m d ax n))
("hamdoon" nil (hh ae0 m d uw1 n))
("hamed" nil (hh ae1 m d))
("hameister" nil (hh ae1 m ay0 s t er0))
("hamel" nil (hh ae1 m ax l))
("hamelin" nil (hh ae1 m ax l ih0 n))
("hamer" nil (hh ae1 m er0))
("hames" nil (hh ey1 m z))
("hamid" nil (hh ax m iy1 d))
("hamiel" nil (hh ae1 m iy0 l))
("hamil" nil (hh ae1 m ax l))
("hamill" nil (hh ae1 m ax l))
("hamilton" nil (hh ae1 m ax l t ax n))
("hamish" nil (hh ae1 m ih0 sh))
("hamiter" nil (hh ae1 m ay0 t er0))
("hamitic" nil (hh ae0 m ih1 t ih0 k))
("hamler" nil (hh ae1 m ax l er0))
("hamlet" nil (hh ae1 m l ax t))
("hamlets" nil (hh ae1 m l ax t s))
("hamlett" nil (hh ae1 m l ih0 t))
("hamley" nil (hh ae1 m l iy0))
("hamlin" nil (hh ae1 m l ih0 n))
("hamling" nil (hh ae1 m l ih0 ng))
("hamm" nil (hh ae1 m))
("hammac" nil (hh ae1 m ax k))
("hammacher" nil (hh ae1 m aa1 k er0))
("hammack" nil (hh ae1 m ax k))
("hammaker" nil (hh ae1 m ey1 k er0))
("hamman" nil (hh ae1 m ax n))
("hammann" nil (hh ae1 m ax n))
("hammar" nil (hh ae1 m er0))
("hammas" nil (hh ax m aa1 s))
("hamme" nil (hh ae1 m))
("hammel" nil (hh ae1 m ax l))
("hammell" nil (hh ae1 m ax l))
("hammen" nil (hh ae1 m ax n))
("hammer" nil (hh ae1 m er0))
("hammered" nil (hh ae1 m er0 d))
("hammering" nil (hh ae1 m er0 ih0 ng))
("hammerle" nil (hh ae1 m er0 ax l))
("hammerlock" nil (hh ae1 m er0 l aa1 k))
("hammerman" nil (hh ae1 m er0 m ax n))
("hammermeister" nil (hh ae1 m er0 m ay1 s t er0))
("hammermill" nil (hh ae1 m er0 m ih1 l))
("hammers" nil (hh ae1 m er0 z))
("hammerschmidt" nil (hh ae1 m er0 sh m ih1 t))
("hammersley" nil (hh ae1 m er0 s l iy0))
("hammersmith" nil (hh ae1 m er0 s m ih1 th))
("hammerson" nil (hh ae1 m er0 s ax n))
("hammerstein" nil (hh ae1 m er0 s t iy1 n))
("hammerstrom" nil (hh ae1 m er0 s t r ax m))
("hammes" nil (hh ae1 m z))
("hammett" nil (hh ae1 m ih0 t))
("hammill" nil (hh ae1 m ax l))
("hammitt" nil (hh ae1 m ih0 t))
("hammock" nil (hh ae1 m ax k))
("hammocks" nil (hh ae1 m ax k s))
("hammon" nil (hh ae1 m ax n))
("hammond" nil (hh ae1 m ax n d))
("hammonds" nil (hh ae1 m ax n d z))
("hammons" nil (hh ae1 m ax n z))
("hammontree" nil (hh ae0 m ax n t r iy1))
("hamner" nil (hh ae1 m n er0))
("hamon" nil (hh ae1 m ax n))
("hamor" nil (hh ae1 m er0))
("hamp" nil (hh ae1 m p))
("hampe" nil (hh ae1 m p))
("hampel" nil (hh ae1 m p ax l))
("hamper" nil (hh ae1 m p er0))
("hampered" nil (hh ae1 m p er0 d))
("hampering" nil (hh ae1 m p er0 ih0 ng))
("hampers" nil (hh ae1 m p er0 z))
("hample" nil (hh ae1 m p ax l))
("hampshire" nil (hh ae1 m p sh er0))
("hampshirites" nil (hh ae1 m p sh er0 ay1 t s))
("hampson" nil (hh ae1 m p s ax n))
("hampton" nil (hh ae1 m p t ax n))
("hamptons" nil (hh ae1 m p t ax n z))
("hamre" nil (hh ae1 m er0))
("hamric" nil (hh ae1 m r ih0 k))
("hamrick" nil (hh ae1 m r ih0 k))
("hamrock" nil (hh ae1 m r aa1 k))
("hams" nil (hh ae1 m z))
("hamsher" nil (hh ae1 m sh er0))
("hamson" nil (hh ae1 m s ax n))
("hamsphire" nil (hh ae1 m s f ay1 r))
("hamster" nil (hh ae1 m s t er0))
("hamsters" nil (hh ae1 m s t er0 z))
("hamstra" nil (hh ae1 m s t r ax))
("hamstring" nil (hh ae1 m s t r ih1 ng))
("hamstrings" nil (hh ae1 m s t r ih1 ng z))
("hamstrung" nil (hh ae1 m s t r ax ng))
("hamtramck" nil (hh ae0 m t r ae1 m ih0 k))
("han" nil (hh aa1 n))
("hana" nil (hh ae1 n ax))
("hanafin" nil (hh ae1 n ax f ih0 n))
("hanagan" nil (hh aa0 n aa1 g aa0 n))
("hanahan" nil (hh ae1 n ax hh ae0 n))
("hanak" nil (hh aa1 n ax k))
("hanan" nil (hh ey1 n ax n))
("hanas" nil (hh ae1 n ax z))
("hanauer" nil (hh ae1 n aw0 er0))
("hanawalt" nil (hh ae1 n ax w ao1 l t))
("hanaway" nil (hh ae1 n ax w ey0))
("hanback" nil (hh ae1 n b ae1 k))
("hanberry" nil (hh ae1 n b eh1 r iy0))
("hanbury" nil (hh ae1 n b eh1 r iy0))
("hanby" nil (hh ae1 n b iy0))
("hance" nil (hh ae1 n s))
("hancher" nil (hh ae1 n ch er0))
("hanchett" nil (hh ae1 n ch ih0 t))
("hanchey" nil (hh ae1 n ch iy0))
("hancock" nil (hh ae1 n k aa1 k))
("hancox" nil (hh ae1 n k aa0 k s))
("hand" nil (hh ae1 n d))
("handa" nil (hh ae1 n d ax))
("handbag" nil (hh ae1 n d b ae1 g))
("handbags" nil (hh ae1 n d b ae1 g z))
("handball" nil (hh ae1 n d b ao1 l))
("handbills" nil (hh ae1 n d b ih1 l z))
("handbook" nil (hh ae1 n d b uh1 k))
("handbooks" nil (hh ae1 n d b uh1 k s))
("handclasp" nil (hh ae1 n d k l ae1 s p))
("handcraft" nil (hh ae1 n d k r ae1 f t))
("handcrafted" nil (hh ae1 n d k r ae1 f t ih0 d))
("handcrafts" nil (hh ae1 n d k r ae1 f t s))
("handcuff" nil (hh ae1 n d k ah1 f))
("handcuffed" nil (hh ae1 n d k ah1 f t))
("handcuffs" nil (hh ae1 n d k ah1 f s))
("handed" nil (hh ae1 n d ax d))
("handedly" nil (hh ae1 n d ih0 d l iy0))
("handedness" nil (hh ae1 n d ax d n ax s))
("handel" nil (hh ae1 n d ax l))
("handeland" nil (hh ae1 n d ih0 l ax n d))
("handelman" nil (hh ae1 n d ax l m ax n))
("handels" nil (hh ae1 n d ax l z))
("handelsbank" nil (hh ae1 n d ax l z b ae1 ng k))
("handelsbanken" nil (hh ae1 n d ax l s b ae1 ng k ax n))
("handelsman" nil (hh ae1 n d ih0 l s m ax n))
("hander" nil (hh ae1 n d er0))
("handers" nil (hh ae1 n d er0 z))
("handford" nil (hh ae1 n d f er0 d))
("handful" nil (hh ae1 n d f uh1 l))
("handgun" nil (hh ae1 n d g ah1 n))
("handguns" nil (hh ae1 n d g ah1 n z))
("handheld" nil (hh ae1 n d hh eh1 l d))
("handholding" nil (hh ae1 n d hh ow1 l d ih0 ng))
("handicap" nil (hh ae1 n d iy0 k ae1 p))
("handicapped" nil (hh ae1 n d iy0 k ae1 p t))
("handicapper" nil (hh ae1 n d iy0 k ae1 p er0))
("handicappers" nil (hh ae1 n d iy0 k ae1 p er0 z))
("handicapping" nil (hh ae1 n d iy0 k ae1 p ih0 ng))
("handicaps" nil (hh ae1 n d iy0 k ae1 p s))
("handicraft" nil (hh ae1 n d iy0 k r ae1 f t))
("handicrafts" nil (hh ae1 n d iy0 k r ae1 f t s))
("handier" nil (hh ae1 n d iy0 er0))
("handily" nil (hh ae1 n d ax l iy0))
("handing" nil (hh ae1 n d ih0 ng))
("handiwork" nil (hh ae1 n d iy0 w er1 k))
("handke" nil (hh ae1 n d k iy0))
("handkerchief" nil (hh ae1 ng k er0 ch ih0 f))
("handkerchiefs" nil (hh ae1 ng k er0 ch ax f s))
("handle" nil (hh ae1 n d ax l))
("handlebars" nil (hh ae1 n d ax l b aa1 r z))
("handled" nil (hh ae1 n d ax l d))
("handler" nil (hh ae1 n d l er0))
("handlers" nil (hh ae1 n d l er0 z))
("handles" nil (hh ae1 n d ax l z))
("handley" nil (hh ae1 n d l iy0))
("handlin" nil (hh ae1 n d l ih0 n))
("handling" nil (hh ae1 n d l ih0 ng))
("handlon" nil (hh ae1 n d l ax n))
("handlooms" nil (hh ae1 n d l uw1 m z))
("handly" nil (hh ae1 n d l iy0))
("handmade" nil (hh ae1 n d m ey1 d))
("handout" nil (hh ae1 n d aw1 t))
("handouts" nil (hh ae1 n d aw1 t s))
("handpicked" nil (hh ae1 n d p ih1 k t))
("handrich" nil (hh ae1 n d r ih0 k))
("handros" nil (hh ae1 n d r ow0 s))
("hands" nil (hh ae1 n d z))
("handsaw" nil (hh ae1 n d s ao1))
("handset" nil (hh ae1 n d s eh1 t))
("handsets" nil (hh ae1 n d s eh1 t s))
("handshake" nil (hh ae1 n d sh ey1 k))
("handshakes" nil (hh ae1 n d sh ey1 k s))
("handsome" nil (hh ae1 n s ax m))
("handsomely" nil (hh ae1 n s ax m l iy0))
("handstands" nil (hh ae1 n d s t ae1 n d z))
("handwerk" nil (hh ae1 n d w er0 k))
("handwerker" nil (hh ae1 n d w er0 k er0))
("handwoven" nil (hh ae1 n d w ow1 v ax n))
("handwriting" nil (hh ae1 n d r ay1 t ih0 ng))
("handwritten" nil (hh ae1 n d r ih1 t ax n))
("handy" nil (hh ae1 n d iy0))
("handyman" nil (hh ae1 n d iy0 m ae1 n))
("handymen" nil (hh ae1 n d iy0 m eh1 n))
("hane" nil (hh ey1 n))
("haneda" nil (hh ax n ey1 d ax))
("hanel" nil (hh ae1 n ax l))
("haneline" nil (hh ae1 n ih0 l ay1 n))
("hanemann" nil (hh ey1 n m ax n))
("haner" nil (hh ey1 n er0))
("hanes" nil (hh ey1 n z))
("haney" nil (hh ey1 n iy0))
("hanf" nil (hh ae1 n f))
("hanford" nil (hh ae1 n f er0 d))
("hanft" nil (hh ae1 n f t))
("hang" nil (hh ae1 ng))
("hangar" nil (hh ae1 ng er0))
("hangars" nil (hh ae1 ng er0 z))
("hangartner" nil (hh ae1 ng g aa0 r t n er0))
("hanged" nil (hh ae1 ng d))
("hangen" nil (hh ae1 ng ax n))
("hanger" nil (hh ae1 ng er0))
("hangers" nil (hh ae1 ng er0 z))
("hanging" nil (hh ae1 ng ih0 ng))
("hangman" nil (hh ae1 ng m ax n))
("hangout" nil (hh ae1 ng aw1 t))
("hangouts" nil (hh ae1 ng aw1 t s))
("hangover" nil (hh ae1 ng ow1 v er0))
("hangovers" nil (hh ae1 ng ow1 v er0 z))
("hangs" nil (hh ae1 ng z))
("hangsang" nil (hh ae1 ng s ae1 ng))
("hanifen" nil (hh ae1 n ih0 f ax n))
("hanifin" nil (hh ae1 n ih0 f ih0 n))
("hanigan" nil (hh ae1 n ih0 g ax n))
("haning" nil (hh ey1 n ih0 ng))
("hanisch" nil (hh ae1 n ih0 sh))
("hanisee" nil (hh ae1 n ih0 s iy0))
("hanish" nil (hh ae1 n ih0 sh))
("hanjin" nil (hh ae1 n jh ih0 n))
("hank" nil (hh ae1 ng k))
("hanke" nil (hh ae1 ng k))
("hankel" nil (hh ae1 ng k ax l))
("hanken" nil (hh ae1 ng k ax n))
("hanker" nil (hh ae1 ng k er0))
("hankering" nil (hh ae1 ng k er0 ih0 ng))
("hankerson" nil (hh ae1 ng k er0 s ax n))
("hankes" nil (hh ae1 ng k s))
("hankey" nil (hh ae1 ng k iy0))
("hankin" nil (hh ae1 ng k ih0 n))
("hankins" nil (hh ae1 ng k ih0 n z))
("hankinson" nil (hh ae1 ng k ih0 n s ax n))
("hankla" nil (hh ae1 ng k l ax))
("hanko" nil (hh ae1 ng k ow0))
("hanks" nil (hh ae1 ng k s))
("hanky" nil (hh ae1 ng k iy0))
("hanley" nil (hh ae1 n l iy0))
("hanlin" nil (hh ae1 n l ih0 n))
("hanlon" nil (hh ae1 n l ax n))
("hanly" nil (hh ae1 n l iy0))
("hanmer" nil (hh ae1 n m er0))
("hann" nil (hh ae1 n))
("hanna" nil (hh ae1 n ax))
("hannaford" nil (hh ae1 n ax f er0 d))
("hannagan" nil (hh ae1 n ax g ae0 n))
("hannah" nil (hh ae1 n ax))
("hannahs" nil (hh ae1 n ax z))
("hannam" nil (hh ae1 n ax m))
("hannaman" nil (hh ae1 n ax m ax n))
("hannan" nil (hh ae1 n ax n))
("hannay" nil (hh ae1 n ey0))
("hanneken" nil (hh ae1 n ih0 k ax n))
("hanneman" nil (hh ae1 n m ax n))
("hannemann" nil (hh ae1 n m ax n))
("hannen" nil (hh ae1 n ax n))
("hanner" nil (hh ae1 n er0))
("hanners" nil (hh ae1 n er0 z))
("hannes" nil (hh ae1 n z))
("hannesson" nil (hh ae1 n ax s ax n))
("hanney" nil (hh ae1 n iy0))
("hanni" nil (hh ae1 n iy0))
("hannibal" nil (hh ae1 n ih0 b ax l))
("hannie" nil (hh ae1 n iy0))
("hannifin" nil (hh ae1 n ih0 f ih0 n))
("hannig" nil (hh ae1 n ih0 g))
("hannigan" nil (hh ae1 n ih0 g ax n))
("hanning" nil (hh ae1 n ih0 ng))
("hannis" nil (hh ae1 n ih0 s))
("hannoch" nil (hh ae1 n ax k))
("hannold" nil (hh ae1 n ow1 l d))
("hannon" nil (hh ae1 n ax n))
("hannula" nil (hh ae1 n uw0 l ax))
("hannum" nil (hh ae1 n ax m))
("hanny" nil (hh ae1 n iy0))
("hano" nil (hh aa1 n ow0))
("hanoi" nil (hh ae1 n oy0))
("hanold" nil (hh ae1 n ow0 l d))
("hanover" nil (hh ae1 n ow0 v er0))
("hanoverian" nil (hh ae1 n ow0 v ih1 r iy0 ax n))
("hanrahan" nil (hh ae1 n r ax hh ae0 n))
("hanratty" nil (hh ae1 n r ax t iy0))
("hans" nil (hh aa1 n s))
("hansa" nil (hh ae1 n s ax))
("hansard" nil (hh ae1 n s er0 d))
("hansberger" nil (hh ae1 n s b er0 g er0))
("hansberry" nil (hh ae1 n s b eh1 r iy0))
("hansbrough" nil (hh ae1 n s b r aw0))
("hansbury" nil (hh ae1 n s b eh0 r iy0))
("hansche" nil (hh ae1 n sh iy0))
("hanscom" nil (hh ae1 n s k ax m))
("hanseatic" nil (hh ae1 n s iy0 ae1 t ih0 k))
("hansel" nil (hh ae1 n s ax l))
("hansell" nil (hh ae1 n s ax l))
("hanselman" nil (hh ae1 n s ax l m ax n))
("hansen" nil (hh ae1 n s ax n))
("hanser" nil (hh aa1 n s er0))
("hansford" nil (hh ae1 n s f er0 d))
("hanshaw" nil (hh ae1 n sh ao1))
("hanshew" nil (hh ae1 n sh uw0))
("hanshin" nil (hh ae1 n sh ih0 n))
("hansley" nil (hh ae1 n s l iy0))
("hansman" nil (hh ae1 n s m ax n))
("hansmann" nil (hh ae1 n s m ax n))
("hansom" nil (hh ae1 n s ax m))
("hanson" nil (hh ae1 n s ax n))
("hanssen" nil (hh ae1 n s ax n))
("hansson" nil (hh ae1 n s ax n))
("hanten" nil (hh ae1 n t ax n))
("hanthorn" nil (hh ae1 n th er0 n))
("hantman" nil (hh ae1 n t m ax n))
("hantz" nil (hh ae1 n t s))
("hanukkah" nil (hh aa1 n ax k ax))
("hanukkahs" nil (hh aa1 n ax k ax z))
("hanus" nil (hh ey1 n ih0 s))
("hanvey" nil (hh ae1 n v iy0))
("hanwa" nil (hh ae1 n w ax))
("hanway" nil (hh ae1 n w ey1))
("hany" nil (hh ey1 n iy0))
("hanzel" nil (hh ae1 n z ax l))
("hanzlik" nil (hh ae1 n z l ih0 k))
("hao" nil (hh aw1))
("hapag" nil (hh ey1 p ae1 g))
("hapeman" nil (hh ey1 p m ax n))
("hapgood" nil (hh ae1 p g uh1 d))
("haphazard" nil (hh ae0 p hh ae1 z er0 d))
("haphazardly" nil (hh ae1 f ax z er0 d l iy0))
("hapke" nil (hh ey1 p k iy0))
("hapless" nil (hh ae1 p l ax s))
("haploid" nil (hh ae1 p l oy0 d))
("hapner" nil (hh ae1 p n er0))
("hapoalim" nil (hh ax p ow1 l ih0 m))
("happ" nil (hh ae1 p))
("happe" nil (hh ae1 p))
("happel" nil (hh ae1 p ax l))
("happen" nil (hh ae1 p ax n))
("happened" nil (hh ae1 p ax n d))
("happening" nil (hh ae1 p ax n ih0 ng))
("happenings" nil (hh ae1 p ax n ih0 ng z))
("happens" nil (hh ae1 p ax n z))
("happenstance" nil (hh ae1 p ax n s t ae1 n s))
("happier" nil (hh ae1 p iy0 er0))
("happiest" nil (hh ae1 p iy0 ax s t))
("happily" nil (hh ae1 p ax l iy0))
("happiness" nil (hh ae1 p iy0 n ax s))
("happy" nil (hh ae1 p iy0))
("hapsburg" nil (hh ae1 p s b er0 g))
("haq" nil (hh ae1 k))
("haque" nil (hh ae1 k))
("hara" nil (hh eh1 r ax))
("harada" nil (hh aa0 r aa1 d ax))
("harald" nil (hh aa1 r ax l d))
("haralda" nil (hh aa0 r aa1 l d ax))
("haralson" nil (hh ae1 r ax l s ax n))
("haran" nil (hh ae1 r ax n))
("harangue" nil (hh er0 ae1 ng))
("harangued" nil (hh er0 ae1 ng d))
("harangues" nil (hh er0 ae1 ng z))
("haranguing" nil (hh er0 ae1 ng ih0 ng))
("harare" nil (hh er0 aa1 r iy0))
("harass" nil (hh er0 ae1 s))
("harassed" nil (hh er0 ae1 s t))
("harassing" nil (hh er0 ae1 s ih0 ng))
("harassment" nil (hh er0 ae1 s m ax n t))
("haraszti" nil (hh er0 ae1 s t iy0))
("haraway" nil (hh aa1 r ax w ey0))
("harb" nil (hh aa1 r b))
("harbach" nil (hh aa1 r b aa1 k))
("harbaugh" nil (hh aa1 r b ao1))
("harbeck" nil (hh aa1 r b eh1 k))
("harber" nil (hh aa1 r b er0))
("harbert" nil (hh aa1 r b er0 t))
("harberts" nil (hh aa1 r b er0 t s))
("harbeson" nil (hh aa1 r b ih0 s ax n))
("harbin" nil (hh aa1 r b ih0 n))
("harbinger" nil (hh aa1 r b ih0 n jh er0))
("harbingers" nil (hh aa1 r b ih0 ng er0 z))
("harbinson" nil (hh aa1 r b ih0 n s ax n))
("harbison" nil (hh aa1 r b ih0 s ax n))
("harbold" nil (hh aa1 r b ow1 l d))
("harbor" nil (hh aa1 r b er0))
("harbored" nil (hh aa1 r b er0 d))
("harboring" nil (hh aa1 r b er0 ih0 ng))
("harbors" nil (hh aa1 r b er0 z))
("harborside" nil (hh aa1 r b er0 s ay1 d))
("harbour" nil (hh aa1 r b er0))
("harbuck" nil (hh aa1 r b ax k))
("harc" nil (hh aa1 r k))
("harclerode" nil (hh aa1 r k ax l er0 ow0 d))
("harcourt" nil (hh aa1 r k ao1 r t))
("harcrow" nil (hh aa1 r k r ow0))
("harcum" nil (hh aa1 r k ax m))
("hard" nil (hh aa1 r d))
("hardacre" nil (hh aa1 r d ey1 k er0))
("hardage" nil (hh aa1 r d ih0 jh))
("hardart" nil (hh aa1 r d aa1 r t))
("hardaway" nil (hh aa1 r d ax w ey1))
("hardback" nil (hh aa1 r d b ae1 k))
("hardball" nil (hh aa1 r d b ao1 l))
("hardboard" nil (hh aa1 r d b ao1 r d))
("hardcastle" nil (hh aa1 r d k ae1 s ax l))
("hardcore" nil (hh aa1 r d k ao1 r))
("hardcover" nil (hh aa1 r d k ah1 v er0))
("hardebeck" nil (hh aa1 r d b eh0 k))
("hardee" nil (hh aa1 r d iy1))
("hardegree" nil (hh aa0 r d ih0 g r iy1))
("hardeman" nil (hh aa1 r d m ax n))
("harden" nil (hh aa1 r d ax n))
("hardenbrook" nil (hh aa1 r d ax n b r uh1 k))
("hardened" nil (hh aa1 r d ax n d))
("hardener" nil (hh aa1 r d ax n er0))
("hardening" nil (hh aa1 r d ax n ih0 ng))
("hardens" nil (hh aa1 r d ax n z))
("harder" nil (hh aa1 r d er0))
("harders" nil (hh aa1 r d er0 z))
("hardest" nil (hh aa1 r d ax s t))
("hardesty" nil (hh aa1 r d ax s t iy0))
("hardey" nil (hh aa1 r d iy0))
("hardgoods" nil (hh aa1 r d g uh1 d z))
("hardgrave" nil (hh aa1 r d g r ey1 v))
("hardgrove" nil (hh aa1 r d g r ow1 v))
("hardheaded" nil (hh aa1 r d hh eh1 d ih0 d))
("hardick" nil (hh aa1 r d ih0 k))
("hardie" nil (hh aa1 r d iy0))
("hardier" nil (hh aa1 r d iy0 er0))
("hardigree" nil (hh aa0 r d ih0 g r iy1))
("hardiman" nil (hh aa1 r d ih0 m ax n))
("hardimon" nil (hh aa1 r d ih0 m aa0 n))
("hardin" nil (hh aa1 r d ih0 n))
("harding" nil (hh aa1 r d ih0 ng))
("hardinger" nil (hh aa1 r d ih0 ng er0))
("hardison" nil (hh aa1 r d ih0 s ax n))
("hardister" nil (hh aa1 r d ih0 s t er0))
("hardisty" nil (hh aa1 r d ih0 s t iy0))
("hardline" nil (hh aa1 r d l ay1 n))
("hardliner" nil (hh aa1 r d l ay1 n er0))
("hardliners" nil (hh aa1 r d l ay1 n er0 z))
("hardly" nil (hh aa1 r d l iy0))
("hardman" nil (hh aa1 r d m ax n))
("hardness" nil (hh aa1 r d n ax s))
("hardnett" nil (hh aa1 r d n ih0 t))
("hardpressed" nil (hh aa1 r d p r eh1 s t))
("hardrick" nil (hh aa1 r d r ih0 k))
("hardscrabble" nil (hh aa1 r d s k r ae1 b ax l))
("hardship" nil (hh aa1 r d sh ih0 p))
("hardships" nil (hh aa1 r d sh ih0 p s))
("hardt" nil (hh aa1 r t))
("hardtke" nil (hh aa1 r d k iy0))
("hardware" nil (hh aa1 r d w eh1 r))
("hardway" nil (hh aa1 r d w ey1))
("hardwick" nil (hh aa1 r d w ih1 k))
("hardwicke" nil (hh aa1 r d w ih0 k))
("hardwin" nil (hh aa1 r d w ih0 n))
("hardwood" nil (hh aa1 r d w uh1 d))
("hardwoods" nil (hh aa1 r d w uh1 d z))
("hardworking" nil (hh aa1 r d w er1 k ih0 ng))
("hardy" nil (hh aa1 r d iy0))
("hardymon" nil (hh aa1 r d iy0 m aa1 n))
("hare" nil (hh eh1 r))
("harebrained" nil (hh eh1 r b r ey1 n d))
("harelson" nil (hh ae1 r ih0 l s ax n))
("harem" nil (hh eh1 r ax m))
("haren" nil (hh eh1 r ax n))
("harer" nil (hh eh1 r er0))
("hares" nil (hh eh1 r z))
("harewood" nil (hh eh1 r w uh1 d))
("harff" nil (hh aa1 r f))
("harford" nil (hh aa1 r f er0 d))
("hargadon" nil (hh aa0 r g aa0 d ao1 n))
("hargan" nil (hh aa1 r g ax n))
("hargens" nil (hh aa1 r g ax n z))
("harger" nil (hh aa1 r g er0))
("hargett" nil (hh aa1 r jh ih0 t))
("hargis" nil (hh aa1 r g ih0 s))
("hargrave" nil (hh aa1 r g r ey1 v))
("hargraves" nil (hh aa1 r g r ey1 v z))
("hargreaves" nil (hh aa1 r g r iy1 v z))
("hargrove" nil (hh aa1 r g r ow1 v))
("hargus" nil (hh aa1 r g ax s))
("hari" nil (hh aa1 r iy0))
("harig" nil (hh ae1 r ih0 g))
("harima" nil (hh eh0 r iy1 m ax))
("haring" nil (hh eh1 r ih0 ng))
("hariri" nil (hh er0 ih1 r iy0))
("haris" nil (hh aa1 r iy0 s))
("haritos" nil (hh eh0 r iy1 t ow0 s))
("hariz" nil (hh eh1 r ih0 z))
("harjo" nil (hh aa1 r jh ow0))
("harju" nil (hh aa1 r y uw0))
("hark" nil (hh aa1 r k))
("harkavy" nil (hh aa1 r k ax v iy0))
("harke" nil (hh aa1 r k))
("harken" nil (hh aa1 r k ax n))
("harker" nil (hh aa1 r k er0))
("harkey" nil (hh aa1 r k iy0))
("harkin" nil (hh aa1 r k ih0 n))
("harking" nil (hh aa1 r k ih0 ng))
("harkins" nil (hh aa1 r k ih0 n z))
("harkleroad" nil (hh aa1 r k l er0 ow0 d))
("harkless" nil (hh aa1 r k l ax s))
("harkness" nil (hh aa1 r k n ax s))
("harkrader" nil (hh aa1 r k r ax d er0))
("harkrider" nil (hh aa1 r k r ay1 d er0))
("harks" nil (hh aa1 r k s))
("harl" nil (hh aa1 r l))
("harlacher" nil (hh aa1 r l ax k er0))
("harlan" nil (hh aa1 r l ax n))
("harland" nil (hh aa1 r l ax n d))
("harle" nil (hh aa1 r ax l))
("harlem" nil (hh aa1 r l ax m))
("harleman" nil (hh aa1 r ax l m ax n))
("harlequin" nil (hh aa1 r l ax k w ax n))
("harless" nil (hh aa1 r l ih0 s))
("harley" nil (hh aa1 r l iy0))
("harleysville" nil (hh aa1 r l iy0 z v ih1 l))
("harlin" nil (hh aa1 r l ih0 n))
("harling" nil (hh aa1 r l ih0 ng))
("harlingen" nil (hh aa1 r l ih0 ng ax n))
("harloff" nil (hh aa1 r l ao0 f))
("harlot" nil (hh aa1 r l ax t))
("harlow" nil (hh aa1 r l ow0))
("harm" nil (hh aa1 r m))
("harman" nil (hh aa1 r m ax n))
("harmata" nil (hh aa0 r m aa1 t ax))
("harmattan" nil (hh aa1 r m ax t ae1 n))
("harmed" nil (hh aa1 r m d))
("harmel" nil (hh aa1 r m ax l))
("harmening" nil (hh aa1 r m ax n ih0 ng))
("harmer" nil (hh aa1 r m er0))
("harmes" nil (hh aa1 r m z))
("harmetz" nil (hh aa1 r m eh1 t s))
("harmeyer" nil (hh aa1 r m ay1 er0))
("harmful" nil (hh aa1 r m f ax l))
("harmfulness" nil (hh aa1 r m f ax l n ax s))
("harming" nil (hh aa1 r m ih0 ng))
("harmison" nil (hh aa1 r m ih0 s ax n))
("harmless" nil (hh aa1 r m l ax s))
("harmlessly" nil (hh aa1 r m l ax s l iy0))
("harmon" nil (hh aa1 r m ax n))
("harmonia" nil (hh aa0 r m ow1 n iy0 ax))
("harmonic" nil (hh aa0 r m aa1 n ih0 k))
("harmonica" nil (hh aa0 r m aa1 n ih0 k ax))
("harmonics" nil (hh aa0 r m aa1 n ih0 k s))
("harmonie" nil (hh aa1 r m ax n iy0))
("harmonies" nil (hh aa1 r m ax n iy0 z))
("harmonious" nil (hh aa0 r m ow1 n iy0 ax s))
("harmonium" nil (hh aa0 r m ow1 n iy0 ax m))
("harmonization" nil (hh aa1 r m ax n ih0 z ey1 sh ax n))
("harmonize" nil (hh aa1 r m ax n ay1 z))
("harmonized" nil (hh aa1 r m ax n ay1 z d))
("harmonizing" nil (hh aa1 r m ax n ay1 z ih0 ng))
("harmony" nil (hh aa1 r m ax n iy0))
("harms" nil (hh aa1 r m z))
("harmsen" nil (hh aa1 r m s ax n))
("harn" nil (hh aa1 r n))
("harnack" nil (hh aa1 r n ax k))
("harnage" nil (hh aa1 r n ih0 jh))
("harnden" nil (hh aa1 r n d ax n))
("harne" nil (hh aa1 r n))
("harned" nil (hh aa1 r n d))
("harner" nil (hh aa1 r n er0))
("harness" nil (hh aa1 r n ax s))
("harnessed" nil (hh aa1 r n ax s t))
("harnesses" nil (hh aa1 r n ax s ax z))
("harnessing" nil (hh aa1 r n ax s ih0 ng))
("harnett" nil (hh aa1 r n ih0 t))
("harney" nil (hh aa1 r n iy0))
("harnisch" nil (hh aa1 r n ih0 sh))
("harnischfeger" nil (hh aa1 r n ih0 sh f eh1 g er0))
("harnish" nil (hh aa1 r n ih0 sh))
("harnois" nil (hh aa0 r n w aa1))
("haro" nil (hh aa1 r ow0))
("harold" nil (hh eh1 r ax l d))
("haroldson" nil (hh ae1 r ow0 l d s ax n))
("harp" nil (hh aa1 r p))
("harpe" nil (hh aa1 r p))
("harped" nil (hh aa1 r p t))
("harpel" nil (hh aa0 r p eh1 l))
("harpenau" nil (hh aa1 r p ih0 n ow0))
("harper" nil (hh aa1 r p er0))
("harpers" nil (hh aa1 r p er0 z))
("harpham" nil (hh aa1 r f ax m))
("harpin" nil (hh aa0 r p ae1 n))
("harping" nil (hh aa1 r p ih0 ng))
("harpist" nil (hh aa1 r p ih0 s t))
("harpists" nil (hh aa1 r p ih0 s t s))
("harpley" nil (hh aa1 r p l iy0))
("harpo" nil (hh aa1 r p ow0))
("harpold" nil (hh aa1 r p ow1 l d))
("harpole" nil (hh aa1 r p ow1 l))
("harpoon" nil (hh aa0 r p uw1 n))
("harps" nil (hh aa1 r p s))
("harpsichord" nil (hh aa1 r p s ax k ao1 r d))
("harpst" nil (hh aa1 r p s t))
("harpster" nil (hh aa1 r p s t er0))
("harquebus" nil (hh aa1 r k w ax b ax s))
("harr" nil (hh ae1 r))
("harrah" nil (hh ae1 r ax))
("harral" nil (hh ae1 r ax l))
("harralson" nil (hh ae1 r ax l s ax n))
("harre" nil (hh ae1 r))
("harrel" nil (hh ae1 r ax l))
("harreld" nil (hh ae1 r ih0 l d))
("harrell" nil (hh eh1 r ax l))
("harrelson" nil (hh eh1 r ih0 l s ax n))
("harren" nil (hh ae1 r ax n))
("harrer" nil (hh aa1 r er0))
("harried" nil (hh eh1 r iy0 d))
("harrier" nil (hh eh1 r iy0 er0))
("harries" nil (hh eh1 r iy0 z))
("harriet" nil (hh eh1 r iy0 ax t))
("harriette" nil (hh ae1 r iy0 eh0 t))
("harrigan" nil (hh eh1 r ih0 g ax n))
("harriger" nil (hh ae1 r ih0 g er0))
("harrill" nil (hh ae1 r ax l))
("harriman" nil (hh eh1 r ih0 m ax n))
("harring" nil (hh ae1 r ih0 ng))
("harrington" nil (hh eh1 r ih0 ng t ax n))
("harriott" nil (hh ae1 r iy0 ax t))
("harris" nil (hh eh1 r ih0 s))
("harrisburg" nil (hh ae1 r ih0 s b er0 g))
("harrison" nil (hh eh1 r ih0 s ax n))
("harriss" nil (hh ae1 r ih0 s))
("harrity" nil (hh ae1 r ih0 t iy0))
("harrod" nil (hh eh1 r ax d))
("harrods" nil (hh eh1 r ax d z))
("harrold" nil (hh eh1 r ax l d))
("harron" nil (hh ae1 r ax n))
("harrop" nil (hh eh1 r ax p))
("harroun" nil (hh er0 uw1 n))
("harrow" nil (hh ae1 r ow0))
("harrower" nil (hh ae1 r ow0 w er0))
("harrowing" nil (hh eh1 r ow0 ih0 ng))
("harry" nil (hh eh1 r iy0))
("harryman" nil (hh ae1 r iy0 m ax n))
("harsch" nil (hh aa1 r sh))
("harsco" nil (hh aa1 r s k ow0))
("harsh" nil (hh aa1 r sh))
("harsha" nil (hh aa1 r sh ax))
("harshaw" nil (hh aa1 r sh ao1))
("harshbarger" nil (hh aa1 r sh b aa1 r g er0))
("harshberger" nil (hh aa1 r sh b er0 g er0))
("harsher" nil (hh aa1 r sh er0))
("harshest" nil (hh aa1 r sh ax s t))
("harshfield" nil (hh aa1 r sh f iy1 l d))
("harshly" nil (hh aa1 r sh l iy0))
("harshman" nil (hh aa1 r sh m ax n))
("harshness" nil (hh aa1 r sh n ax s))
("harstad" nil (hh aa1 r s t ax d))
("harston" nil (hh aa1 r s t ax n))
("hart" nil (hh aa1 r t))
("harte" nil (hh aa1 r t))
("hartel" nil (hh aa1 r t ax l))
("hartell" nil (hh aa1 r t ax l))
("harten" nil (hh aa1 r t ax n))
("hartenstein" nil (hh aa1 r t ax n s t ay1 n))
("harter" nil (hh aa1 r t er0))
("hartfiel" nil (hh aa1 r t f iy1 l))
("hartfield" nil (hh aa1 r t f iy1 l d))
("hartford" nil (hh aa1 r t f er0 d))
("hartgraves" nil (hh aa1 r t g r ey1 v z))
("hartgrove" nil (hh aa1 r t g r ow1 v))
("harth" nil (hh aa1 r th))
("hartig" nil (hh aa1 r t ih0 g))
("hartigan" nil (hh aa1 r t ih0 g ax n))
("hartin" nil (hh aa1 r t ih0 n))
("harting" nil (hh aa1 r t ih0 ng))
("hartinger" nil (hh aa1 r t ih0 ng er0))
("hartis" nil (hh aa1 r t ih0 s))
("hartje" nil (hh aa1 r t jh ey0))
("hartke" nil (hh aa1 r t k iy0))
("hartkopf" nil (hh aa1 r t k ao0 f))
("hartl" nil (hh aa1 r t ax l))
("hartlage" nil (hh aa1 r t l ih0 jh))
("hartland" nil (hh aa1 r t l ax n d))
("hartlaub" nil (hh aa1 r t l aw1 b))
("hartle" nil (hh aa1 r t ax l))
("hartless" nil (hh aa1 r t l ax s))
("hartley" nil (hh aa1 r t l iy0))
("hartlieb" nil (hh aa1 r t l iy1 b))
("hartline" nil (hh aa1 r t l ay1 n))
("hartling" nil (hh aa1 r t l ih0 ng))
("hartman" nil (hh aa1 r t m ax n))
("hartmann" nil (hh aa1 r t m ax n))
("hartmarx" nil (hh aa1 r t m aa1 r k s))
("hartnell" nil (hh aa1 r t n ax l))
("hartner" nil (hh aa1 r t n er0))
("hartness" nil (hh aa1 r t n ax s))
("hartnett" nil (hh aa1 r t n ih0 t))
("hartney" nil (hh aa1 r t n iy0))
("hartog" nil (hh aa1 r t ax g))
("harton" nil (hh aa1 r t ax n))
("hartong" nil (hh aa1 r t ao0 ng))
("hartranft" nil (hh aa1 r t r ah1 n f t))
("harts" nil (hh aa1 r t s))
("hartsell" nil (hh aa1 r t s ax l))
("hartsfield" nil (hh aa1 r t s f iy1 l d))
("hartshorn" nil (hh aa1 r t s hh ao1 r n))
("hartshorne" nil (hh aa1 r ch er0 n))
("hartsock" nil (hh aa1 r t s ax k))
("hartsoe" nil (hh aa1 r t s ow0))
("hartson" nil (hh aa1 r t s ax n))
("hartsook" nil (hh aa1 r t s uh0 k))
("hartsough" nil (hh aa1 r t s aw0))
("hartstein" nil (hh aa1 r t s t ay1 n))
("hartt" nil (hh aa1 r t))
("hartter" nil (hh aa1 r t er0))
("hartung" nil (hh aa1 r t ax ng))
("hartwell" nil (hh aa1 r t w eh1 l))
("hartwick" nil (hh aa1 r t w ih1 k))
("hartwig" nil (hh aa1 r t w ih0 k))
("hartwood" nil (hh aa1 r t w uh1 d))
("harty" nil (hh aa1 r t iy0))
("hartz" nil (hh aa1 r t s))
("hartzel" nil (hh aa1 r t z ax l))
("hartzell" nil (hh aa1 r t z ax l))
("hartzler" nil (hh aa1 r t s l er0))
("hartzog" nil (hh aa1 r t z ax g))
("haruo" nil (hh aa0 r uw1 ow0))
("harv" nil (hh aa1 r v))
("harvard" nil (hh aa1 r v er0 d))
("harvath" nil (hh aa1 r v ax th))
("harve" nil (hh aa1 r v))
("harvel" nil (hh aa1 r v ax l))
("harvell" nil (hh aa1 r v ax l))
("harvest" nil (hh aa1 r v ax s t))
("harvestable" nil (hh aa1 r v ax s t ax b ax l))
("harvested" nil (hh aa1 r v ax s t ax d))
("harvester" nil (hh aa1 r v ax s t er0))
("harvesters" nil (hh aa1 r v ax s t er0 z))
("harvesting" nil (hh aa1 r v ax s t ih0 ng))
("harvests" nil (hh aa1 r v ax s t s))
("harvey" nil (hh aa1 r v iy0))
("harvie" nil (hh aa1 r v iy0))
("harvill" nil (hh aa1 r v ih0 l))
("harville" nil (hh aa1 r v ih1 l))
("harvin" nil (hh aa1 r v ih0 n))
("harvison" nil (hh aa1 r v ih0 s ax n))
("harward" nil (hh aa1 r w er0 d))
("harwell" nil (hh aa1 r w eh1 l))
("harwick" nil (hh aa1 r w ih0 k))
("harwood" nil (hh aa1 r w uh1 d))
("haryana" nil (hh eh0 r y aa1 n ax))
("has" nil (hh ae1 z))
("hasan" nil (hh ey1 z ax n))
("hasbro" nil (hh ae1 z b r ow0))
("hasbrook" nil (hh ae1 s b r uh0 k))
("hasbrouck" nil (hh ae1 z b r uh1 k))
("hascall" nil (hh ae1 s k ax l))
("hasch" nil (hh ae1 sh))
("haschke" nil (hh ae1 sh k))
("hase" nil (hh ey1 z))
("hasegawa" nil (hh aa1 s ey0 g aa1 w ax))
("hasek" nil (hh aa1 s eh0 k))
("haselden" nil (hh ae1 s ih0 l d ax n))
("haseley" nil (hh ae1 s ih0 l iy0))
("haselhorst" nil (hh ae1 s ih0 l hh ao0 r s t))
("haseltine" nil (hh ae1 s ih0 l t iy0 n))
("haselton" nil (hh ax s eh1 l t ax n))
("haseman" nil (hh ey1 s m ax n))
("hasenauer" nil (hh ae1 s ih0 n aw0 er0))
("hasenfus" nil (hh ey1 z ax n f ah1 s))
("haser" nil (hh eh1 z er0))
("hash" nil (hh ae1 sh))
("hashagen" nil (hh ae1 sh ax g ax n))
("hashed" nil (hh ae1 sh t))
("hashem" nil (hh ae1 sh ih0 m))
("hashemi" nil (hh ax sh ey1 m iy0))
("hashemite" nil (hh ae1 sh ax m ay1 t))
("hashers" nil (hh ae1 sh er0 z))
("hashi" nil (hh ae1 sh iy0))
("hashim" nil (hh ae1 sh ih0 m))
("hashimoto" nil (hh aa0 sh iy0 m ow1 t ow0))
("hashing" nil (hh ae1 sh ih0 ng))
("hashish" nil (hh ae1 sh ih0 sh))
("hashman" nil (hh ae1 sh m ax n))
("hasidic" nil (hh ax s ih1 d ih0 k))
("haske" nil (hh ey1 s k))
("haskell" nil (hh ae1 s k ax l))
("haskett" nil (hh ae1 s k ih0 t))
("haskew" nil (hh ae1 s k y uw0))
("haskin" nil (hh ae1 s k ih0 n))
("haskins" nil (hh ae1 s k ih0 n z))
("haslam" nil (hh ae1 s l ax m))
("hasler" nil (hh ae1 s ax l er0))
("haslett" nil (hh ae1 s l ih0 t))
("hasley" nil (hh ae1 z l iy0))
("haso" nil (hh aa1 s ow1))
("haspel" nil (hh ae1 s p ax l))
("hassan" nil (hh ax s aa1 n))
("hasse" nil (hh aa1 s))
("hassebrock" nil (hh ae1 s ih0 b r aa1 k))
("hassel" nil (hh ae1 s ax l))
("hasselbach" nil (hh ae1 s ih0 l b aa0 k))
("hasselbring" nil (hh ae1 s ih0 l b r ih0 ng))
("hassell" nil (hh ae1 s ax l))
("hasselman" nil (hh ae1 s ax l m ax n))
("hassen" nil (hh ae1 s ax n))
("hassenberg" nil (hh ae1 s ax n b er0 g))
("hassenfeld" nil (hh ae1 s ax n f eh1 l d))
("hassett" nil (hh ae1 s ih0 t))
("hassey" nil (hh ae1 s iy0))
("hassig" nil (hh ae1 s ih0 g))
("hassing" nil (hh ae1 s ih0 ng))
("hassinger" nil (hh ae1 s ih0 n jh er0))
("hassle" nil (hh ae1 s ax l))
("hassled" nil (hh ae1 s ax l d))
("hassler" nil (hh ae1 s l er0))
("hassles" nil (hh ae1 s ax l z))
("hassman" nil (hh ae1 s m ax n))
("hasson" nil (hh ae1 s ax n))
("hast" nil (hh ae1 s t))
("haste" nil (hh ey1 s t))
("hasten" nil (hh ey1 s ax n))
("hastened" nil (hh ey1 s ax n d))
("hastening" nil (hh ey1 s ax n ih0 ng))
("hastens" nil (hh ey1 s ax n z))
("hastey" nil (hh ey1 s t iy0))
("hastie" nil (hh ey1 s t iy0))
("hastily" nil (hh ey1 s t ax l iy0))
("hasting" nil (hh ey1 s t ih0 ng))
("hastings" nil (hh ey1 s t ih0 ng z))
("haston" nil (hh ae1 s t ax n))
("hasty" nil (hh ey1 s t iy0))
("haswell" nil (hh ae1 s w eh0 l))
("hasz" nil (hh aa1 sh))
("hat" nil (hh ae1 t))
("hata" nil (hh aa1 t ax))
("hatala" nil (hh ae1 t ax l ax))
("hataway" nil (hh ae1 t ax w ey1))
("hatch" nil (hh ae1 ch))
("hatchback" nil (hh ae1 ch b ae1 k))
("hatched" nil (hh ae1 ch t))
("hatchel" nil (hh ae1 ch ax l))
("hatchell" nil (hh ae1 ch ax l))
("hatcher" nil (hh ae1 ch er0))
("hatcheries" nil (hh ae1 ch er0 iy0 z))
("hatchery" nil (hh ae1 ch er0 iy0))
("hatches" nil (hh ae1 ch ih0 z))
("hatchet" nil (hh ae1 ch ax t))
("hatchets" nil (hh ae1 ch ax t s))
("hatchett" nil (hh ae1 ch ih0 t))
("hatching" nil (hh ae1 ch ih0 ng))
("hate" nil (hh ey1 t))
("hated" nil (hh ey1 t ax d))
("hateful" nil (hh ey1 t f ax l))
("hatem" nil (hh ae1 t ih0 m))
("hater" nil (hh ey1 t er0))
("haters" nil (hh ey1 t er0 z))
("hates" nil (hh ey1 t s))
("hatfield" nil (hh ae1 t f iy1 l d))
("hath" nil (hh ae1 th))
("hathaway" nil (hh ae1 th ax w ey1))
("hathcoat" nil (hh ae1 th k ow1 t))
("hathcock" nil (hh ae1 th k ax k))
("hatheway" nil (hh ey1 dh w ey0))
("hathorn" nil (hh aa1 th ao0 r n))
("hathorne" nil (hh ae1 th er0 n))
("hating" nil (hh ey1 t ih0 ng))
("hatler" nil (hh ey1 t ax l er0))
("hatlestad" nil (hh ae1 t l ih0 s t ax d))
("hatley" nil (hh ae1 t l iy0))
("hatmaker" nil (hh ae1 t m ey1 k er0))
("hatred" nil (hh ey1 t r ax d))
("hatreds" nil (hh ey1 t r ax d z))
("hats" nil (hh ae1 t s))
("hatt" nil (hh ae1 t))
("hattabaugh" nil (hh ae1 t ax b ao0))
("hattan" nil (hh ae1 t ax n))
("hattaway" nil (hh ae1 t ax w ey0))
("hatten" nil (hh ae1 t ax n))
("hattendorf" nil (hh ae1 t ih0 n d ao0 r f))
("hatter" nil (hh ae1 t er0))
("hatteras" nil (hh ae1 t er0 ax s))
("hattersley" nil (hh ae1 t er0 z l iy0))
("hattery" nil (hh ae1 t er0 iy0))
("hattie" nil (hh ae1 t iy0))
("hatton" nil (hh ae1 t ax n))
("hattori" nil (hh aa0 t ao1 r iy0))
("hattusas" nil (hh ax t uw1 s ax s))
("hatty" nil (hh ae1 t iy0))
("hatz" nil (hh ae1 t s))
("hau" nil (hh aw1))
("haub" nil (hh ao1 b))
("hauber" nil (hh aw1 b er0))
("haubert" nil (hh aw1 b er0 t))
("haubner" nil (hh aw1 b n er0))
("haubrich" nil (hh aw1 b r ih0 k))
("hauch" nil (hh ao1 ch))
("hauck" nil (hh ao1 k))
("hauenstein" nil (hh aw1 ax n s t ay0 n))
("hauer" nil (hh aw1 er0))
("haueter" nil (hh aw1 t er0))
("hauf" nil (hh ao1 f))
("hauff" nil (hh ao1 f))
("haug" nil (hh ao1 g))
("haugan" nil (hh ao1 g ax n))
("hauge" nil (hh ao1 jh))
("haugen" nil (hh aw1 g ax n))
("hauger" nil (hh aw1 g er0))
("haugh" nil (hh ao1))
("haughey" nil (hh ao1 k iy0))
("haughn" nil (hh ao1 n))
("haught" nil (hh ao1 t))
("haughton" nil (hh ao1 t ax n))
("haughty" nil (hh ao1 t iy0))
("haugland" nil (hh ao1 g l ax n d))
("hauk" nil (hh ao1 k))
("hauke" nil (hh ao1 k))
("haul" nil (hh ao1 l))
("hauled" nil (hh ao1 l d))
("hauler" nil (hh ao1 l er0))
("haulers" nil (hh ao1 l er0 z))
("hauling" nil (hh ao1 l ih0 ng))
("hauls" nil (hh ao1 l z))
("haun" nil (hh ao1 n))
("haunt" nil (hh ao1 n t))
("haunted" nil (hh ao1 n t ax d))
("haunting" nil (hh ao1 n t ih0 ng))
("hauntingly" nil (hh ao1 n t ih0 ng l iy0))
("haunts" nil (hh ao1 n t s))
("haupert" nil (hh aw1 p er0 t))
("hauppauge" nil (hh aw1 p ao0 jh))
("haupt" nil (hh ao1 p t))
("hauptfuhrer" nil (hh aw1 p t f y uh1 r er0))
("hauptman" nil (hh aw1 p t m ax n))
("hauptmann" nil (hh aw1 p t m ax n))
("haury" nil (hh ao1 r iy0))
("haus" nil (hh aw1 s))
("hausa" nil (hh aw1 s ax))
("hausauer" nil (hh aw1 s aw0 er0))
("hausch" nil (hh aw1 sh))
("hauschild" nil (hh aw1 s ch ay1 l d))
("hauschildt" nil (hh aw1 sh ih0 l t))
("hause" nil (hh aw1 s))
("hausen" nil (hh aw1 z ax n))
("hauser" nil (hh aw1 z er0))
("hauserman" nil (hh aw1 z er0 m ax n))
("hausfeld" nil (hh aw1 s f eh1 l d))
("hausler" nil (hh aw1 s ax l er0))
("hausman" nil (hh aw1 s m ax n))
("hausmann" nil (hh aw1 s m ax n))
("hausner" nil (hh aw1 s n er0))
("hauss" nil (hh aw1 s))
("hausser" nil (hh aw1 s er0))
("haussler" nil (hh aw1 s ax l er0))
("haussmann" nil (hh aw1 s m ax n))
("hauswirth" nil (hh aw1 s w er0 th))
("haut" nil (hh ao1 t))
("hautala" nil (hh ao1 t ax l ax))
("haute" nil (hh ow1 t))
("hauter" nil (hh aw1 t er0))
("hauth" nil (hh ao1 th))
("hauver" nil (hh aw1 v er0))
("havana" nil (hh ax v ae1 n ax))
("havard" nil (hh ae1 v er0 d))
("havas" nil (hh aa1 v aa0 z))
("have" nil (hh ae1 v))
("havel" nil (hh ae1 v ax l))
("havelka" nil (hh ax v eh1 l k ax))
("havelock" nil (hh ae1 v l aa1 k))
("haveman" nil (hh ey1 v m ax n))
("havemann" nil (hh ae1 v ax m ax n))
("haven" nil (hh ey1 v ax n))
("havener" nil (hh ae1 v iy0 n er0))
("havens" nil (hh ey1 v ax n z))
("haver" nil (hh eh1 v er0))
("haverfield" nil (hh ae1 v er0 f iy1 l d))
("haverford" nil (hh ae1 v er0 f er0 d))
("haverkamp" nil (hh ae1 v er0 k ae1 m p))
("haverland" nil (hh ae1 v er0 l ax n d))
("haverly" nil (hh ey1 v er0 l iy0))
("haverstick" nil (hh ey1 v er0 s t ih0 k))
("haverstock" nil (hh ey1 v er0 s t aa0 k))
("haverty" nil (hh ae1 v er0 t iy0))
("haves" nil (hh ae1 v z))
("havey" nil (hh ey1 v iy0))
("haviland" nil (hh ey1 v iy0 l ax n d))
("havill" nil (hh aa0 v iy1 l))
("havilland" nil (hh ae1 v ax l ax n d))
("having" nil (hh ae1 v ih0 ng))
("havins" nil (hh ae1 v ih0 n z))
("havis" nil (hh ae1 v ih0 s))
("havlicek" nil (hh aa1 v l ih0 ch eh0 k))
("havlik" nil (hh ae1 v l ih0 k))
("havlin" nil (hh ae1 v l ih0 n))
("havner" nil (hh ae1 v n er0))
("havoc" nil (hh ae1 v ax k))
("havran" nil (hh ae1 v r ax n))
("havranek" nil (hh ax v r ae1 n eh0 k))
("havrilla" nil (hh ae1 v r ih1 l ax))
("havron" nil (hh ae1 v r ax n))
("haw" nil (hh ao1))
("hawaii" nil (hh ax w ay1 iy1))
("hawaiian" nil (hh ax w ay1 ax n))
("hawaiians" nil (hh ax w ay1 ax n z))
("hawass" nil (hh ae1 w aa0 s))
("hawbaker" nil (hh ao1 b ey1 k er0))
("hawe" nil (hh ao1))
("hawes" nil (hh ao1 z))
("hawing" nil (hh ao1 ih0 ng))
("hawk" nil (hh ao1 k))
("hawkbill" nil (hh ao1 k b ih1 l))
("hawke" nil (hh ao1 k))
("hawked" nil (hh ao1 k t))
("hawken" nil (hh ao1 k ax n))
("hawker" nil (hh ao1 k er0))
("hawkers" nil (hh ao1 k er0 z))
("hawkes" nil (hh ao1 k s))
("hawkey" nil (hh ao1 k iy1))
("hawkeye" nil (hh ao1 k ay1))
("hawkiness" nil (hh ao1 k n ax s))
("hawking" nil (hh ao1 k ih0 ng))
("hawkins" nil (hh ao1 k ih0 n z))
("hawkinson" nil (hh ao1 k ih0 n s ax n))
("hawkish" nil (hh ao1 k ih0 sh))
("hawks" nil (hh ao1 k s))
("hawksley" nil (hh ao1 k s l iy0))
("hawley" nil (hh ao1 l iy0))
("hawn" nil (hh ao1 n))
("haworth" nil (hh ae1 w er0 th))
("haws" nil (hh ao1 z))
("hawthorn" nil (hh ao1 th ao1 r n))
("hawthorne" nil (hh ao1 th ao0 r n))
("hawthorns" nil (hh ao1 th ao1 r n z))
("hawver" nil (hh ao1 v er0))
("haxton" nil (hh ae1 k s t ax n))
("hay" nil (hh ey1))
("hayashi" nil (hh aa0 y aa1 sh iy0))
("hayashida" nil (hh aa0 y aa0 sh iy1 d ax))
("haycock" nil (hh ey1 k aa1 k))
("haycraft" nil (hh ey1 k r ae1 f t))
("haydel" nil (hh ey1 d ax l))
("hayden" nil (hh ey1 d ax n))
("haydn" nil (hh ay1 d ax n))
("haydock" nil (hh ey1 d aa1 k))
("haydon" nil (hh ey1 d ax n))
("haydu" nil (hh ey1 d uw0))
("hayduk" nil (hh ey1 d ax k))
("haye" nil (hh ey1))
("hayek" nil (hh ey1 ih0 k))
("hayen" nil (hh ey1 ax n))
("hayenga" nil (hh ey0 ey1 ng g ax))
("hayer" nil (hh ey1 er0))
("hayes" nil (hh ey1 z))
("hayfields" nil (hh ey1 f iy1 l d z))
("hayford" nil (hh ey1 f er0 d))
("haygood" nil (hh ey1 g uh1 d))
("hayhurst" nil (hh ey1 hh er0 s t))
("hayles" nil (hh ey1 l z))
("hayley" nil (hh ey1 l iy0))
("hayloft" nil (hh ey1 l ao1 f t))
("haymaker" nil (hh ey1 m ey1 k er0))
("hayman" nil (hh ey1 m ax n))
("haymarket" nil (hh ey1 m aa1 r k ih0 t))
("haymes" nil (hh ey1 m z))
("haymon" nil (hh ey1 m ax n))
("haymond" nil (hh ey1 m ax n d))
("haymore" nil (hh ey1 m ao0 r))
("hayn" nil (hh ey1 n))
("hayne" nil (hh ey1 n))
("hayner" nil (hh ey1 n er0))
("haynes" nil (hh ey1 n z))
("haynesworth" nil (hh ey1 n z w er0 th))
("haynie" nil (hh ey1 n iy0))
("haynsworth" nil (hh ey1 n z w er0 th))
("hays" nil (hh ey1 z))
("hayse" nil (hh ey1 z))
("hayslett" nil (hh ey1 s l ih0 t))
("hayslip" nil (hh ey1 s l ih0 p))
("haystack" nil (hh ey1 s t ae1 k))
("hayter" nil (hh ey1 t er0))
("hayton" nil (hh ey1 t ax n))
("hayward" nil (hh ey1 w er0 d))
("haywire" nil (hh ey1 w ay1 r))
("haywood" nil (hh ey1 w uh1 d))
("hayworth" nil (hh ey1 w er1 th))
("hazan" nil (hh ey1 z ax n))
("hazard" nil (hh ae1 z er0 d))
("hazardous" nil (hh ae1 z er0 d ax s))
("hazards" nil (hh ae1 z er0 d z))
("haze" nil (hh ey1 z))
("hazel" nil (hh ey1 z ax l))
("hazelbaker" nil (hh ey1 z ax l b ey1 k er0))
("hazelett" nil (hh ae1 z ih0 l eh0 t))
("hazelip" nil (hh ae1 z ih0 l ih0 p))
("hazell" nil (hh ae1 z ax l))
("hazelnut" nil (hh ey1 z ax l n ah1 t))
("hazelrigg" nil (hh ax z eh1 l r ih0 g))
("hazeltine" nil (hh ey1 z ax l t ay1 n))
("hazelton" nil (hh ey1 z ax l t ax n))
("hazelwood" nil (hh ey1 z ax l w uh1 d))
("hazen" nil (hh ey1 z ax n))
("hazing" nil (hh ey1 z ih0 ng))
("hazle" nil (hh ey1 z ax l))
("hazleton" nil (hh ey1 z ax l t ax n))
("hazlett" nil (hh ae1 z l ih0 t))
("hazlewood" nil (hh ey1 z ax l w uh1 d))
("hazy" nil (hh ey1 z iy0))
("hazzard" nil (hh ae1 z er0 d))
("hbox" nil (ey1 ch b ao1 k s))
("hces" nil (ey1 ch s iy1 iy1 eh1 s))
("he" nil (hh iy1))
("heaberlin" nil (hh iy1 b er0 l ih0 n))
("heacock" nil (hh iy1 k ax k))
("heacox" nil (hh iy1 k aa0 k s))
("head" nil (hh eh1 d))
("headache" nil (hh eh1 d ey1 k))
("headaches" nil (hh eh1 d ey1 k s))
("headband" nil (hh eh1 d b ae1 n d))
("headbands" nil (hh eh1 d b ae1 n d z))
("headboard" nil (hh eh1 d b ao1 r d))
("headcheese" nil (hh eh1 d ch iy1 z))
("headcount" nil (hh eh1 d k aw1 n t))
("headdress" nil (hh eh1 d r eh1 s))
("headdresses" nil (hh eh1 d d r eh1 s ax z))
("headed" nil (hh eh1 d ax d))
("headen" nil (hh eh1 d ax n))
("header" nil (hh eh1 d er0))
("headfirst" nil (hh eh1 d f er1 s t))
("headgear" nil (hh eh1 d g ih1 r))
("headhunter" nil (hh eh1 d hh ah1 n t er0))
("headhunters" nil (hh eh1 d hh ah1 n t er0 z))
("heading" nil (hh eh1 d ih0 ng))
("headings" nil (hh eh1 d ih0 ng z))
("headington" nil (hh eh1 d ih0 ng t ax n))
("headlee" nil (hh eh1 d l iy1))
("headless" nil (hh eh1 d l ax s))
("headley" nil (hh eh1 d l iy0))
("headlight" nil (hh eh1 d l ay1 t))
("headlights" nil (hh eh1 d l ay1 t s))
("headline" nil (hh eh1 d l ay1 n))
("headlined" nil (hh eh1 d l ay1 n d))
("headliner" nil (hh eh1 d l ay1 n er0))
("headlines" nil (hh eh1 d l ay1 n z))
("headlining" nil (hh eh1 d l ay1 n ih0 ng))
("headlong" nil (hh eh1 d l ao1 ng))
("headlund" nil (hh eh1 d l ax n d))
("headman" nil (hh eh1 d m ax n))
("headmaster" nil (hh eh1 d m ae1 s t er0))
("headmistress" nil (hh eh1 d m ih1 s t r ax s))
("headphones" nil (hh eh1 d f ow1 n z))
("headquartered" nil (hh eh1 d k ao1 r t er0 d))
("headquarters" nil (hh eh1 d k w ao1 r t er0 z))
("headrests" nil (hh eh1 d r eh1 s t s))
("headrick" nil (hh eh1 d r ih0 k))
("headroom" nil (hh eh1 d r uw1 m))
("heads" nil (hh eh1 d z))
("headset" nil (hh eh1 d s eh1 t))
("headsets" nil (hh eh1 d s eh1 t s))
("headship" nil (hh eh1 d sh ih1 p))
("headsman" nil (hh eh1 d z m ax n))
("headstart" nil (hh eh1 d s t aa1 r t))
("headstrong" nil (hh eh1 d s t r ao1 ng))
("headwaters" nil (hh eh1 d w ao1 t er0 z))
("headway" nil (hh eh1 d w ey1))
("heady" nil (hh eh1 d iy0))
("heafner" nil (hh iy1 f n er0))
("heagle" nil (hh iy1 g ax l))
("heagney" nil (hh iy1 g n iy0))
("heagy" nil (hh iy1 jh iy0))
("heal" nil (hh iy1 l))
("heald" nil (hh iy1 l d))
("healed" nil (hh iy1 l d))
("healer" nil (hh iy1 l er0))
("healers" nil (hh iy1 l er0 z))
("healey" nil (hh iy1 l iy0))
("healing" nil (hh iy1 l ih0 ng))
("heals" nil (hh iy1 l z))
("health" nil (hh eh1 l th))
("healthamerica" nil (hh iy1 l th ax m eh1 r ih0 k ax))
("healthcare" nil (hh eh1 l th k eh1 r))
("healthcorp" nil (hh eh1 l th k ao1 r p))
("healthdyne" nil (hh eh1 l th d ay1 n))
("healthful" nil (hh eh1 l th f ax l))
("healthier" nil (hh eh1 l th iy0 er0))
("healthiest" nil (hh eh1 l th iy0 ih0 s t))
("healthsource" nil (hh eh1 l th s ao1 r s))
("healthsouth" nil (hh eh1 l th s aw1 th))
("healthtrust" nil (hh eh1 l th t r ah1 s t))
("healthvest" nil (hh eh1 l th v eh1 s t))
("healthy" nil (hh eh1 l th iy0))
("healy" nil (hh iy1 l iy0))
("heaney" nil (hh iy1 n iy0))
("heap" nil (hh iy1 p))
("heape" nil (hh iy1 p))
("heaped" nil (hh iy1 p t))
("heaphy" nil (hh iy1 f iy0))
("heaping" nil (hh iy1 p ih0 ng))
("heaps" nil (hh iy1 p s))
("hear" nil (hh ih1 r))
("heard" nil (hh er1 d))
("hearers" nil (hh ih1 r er0 z))
("hearin" nil (hh ih1 r ih0 n))
("hearing" nil (hh ih1 r ih0 ng))
("hearings" nil (hh ih1 r ih0 ng z))
("hearl" nil (hh er1 l))
("hearn" nil (hh er1 n))
("hearne" nil (hh er1 n))
("hearns" nil (hh er1 n z))
("hearon" nil (hh ih1 r ax n))
("hearron" nil (hh ao1 r ax n))
("hears" nil (hh ih1 r z))
("hearsay" nil (hh ih1 r s ey1))
("hearse" nil (hh er1 s))
("hearst" nil (hh er1 s t))
("heart" nil (hh aa1 r t))
("heartache" nil (hh aa1 r t ey1 k))
("heartbeat" nil (hh aa1 r t b iy1 t))
("heartbeats" nil (hh aa1 r t b iy1 t s))
("heartbreak" nil (hh aa1 r t b r ey1 k))
("heartbreaking" nil (hh aa1 r t b r ey1 k ih0 ng))
("heartbroken" nil (hh aa1 r t b r ow1 k ax n))
("heartburn" nil (hh aa1 r t b er1 n))
("hearted" nil (hh aa1 r t ax d))
("heartedly" nil (hh aa1 r t ih0 d l iy0))
("hearten" nil (hh aa1 r t ax n))
("heartened" nil (hh aa1 r t ax n d))
("heartening" nil (hh aa1 r t ax n ih0 ng))
("heartfelt" nil (hh aa1 r t f eh1 l t))
("hearth" nil (hh aa1 r th))
("hearths" nil (hh aa1 r th s))
("heartily" nil (hh aa1 r t ax l iy0))
("heartland" nil (hh aa1 r t l ae1 n d))
("heartless" nil (hh aa1 r t l ax s))
("hearts" nil (hh aa1 r t s))
("heartthrob" nil (hh aa1 r t th r aa1 b))
("heartwarming" nil (hh aa1 r t w ao1 r m ih0 ng))
("heartwise" nil (hh aa1 r t w ay1 z))
("heartwood" nil (hh aa1 r t w uh1 d))
("hearty" nil (hh aa1 r t iy0))
("heasley" nil (hh iy1 z l iy0))
("heaslip" nil (hh iy1 s l ih0 p))
("heaston" nil (hh iy1 s t ax n))
("heat" nil (hh iy1 t))
("heated" nil (hh iy1 t ax d))
("heatedly" nil (hh iy1 t ih0 d l iy0))
("heater" nil (hh iy1 t er0))
("heaters" nil (hh iy1 t er0 z))
("heath" nil (hh iy1 th))
("heathcliff" nil (hh eh1 th k l ih0 f))
("heathcock" nil (hh eh1 th k ax k))
("heathcote" nil (hh eh1 th k ax t))
("heathen" nil (hh iy1 dh ax n))
("heather" nil (hh eh1 dh er0))
("heatherington" nil (hh eh1 dh er0 ih0 ng t ax n))
("heatherly" nil (hh eh1 dh er0 l iy0))
("heathers" nil (hh eh1 dh er0 z))
("heathman" nil (hh iy1 th m ax n))
("heathrow" nil (hh iy1 th r ow0))
("heathwood" nil (hh iy1 th w uh1 d))
("heating" nil (hh iy1 t ih0 ng))
("heatley" nil (hh iy1 t l iy0))
("heaton" nil (hh iy1 t ax n))
("heats" nil (hh iy1 t s))
("heatwole" nil (hh iy1 t w ow1 l))
("heave" nil (hh iy1 v))
("heaved" nil (hh iy1 v d))
("heaven" nil (hh eh1 v ax n))
("heavener" nil (hh eh1 v ax n er0))
("heavenly" nil (hh eh1 v ax n l iy0))
("heavens" nil (hh eh1 v ax n z))
("heaves" nil (hh iy1 v z))
("heavey" nil (hh iy1 v iy0))
("heavier" nil (hh eh1 v iy0 er0))
("heavies" nil (hh eh1 v iy0 z))
("heaviest" nil (hh eh1 v iy0 ax s t))
("heavily" nil (hh eh1 v ax l iy0))
("heavin" nil (hh eh1 v ih0 n))
("heaving" nil (hh iy1 v ih0 ng))
("heavner" nil (hh iy1 v n er0))
("heavrin" nil (hh iy1 v r ih0 n))
("heavy" nil (hh eh1 v iy0))
("heavyhanded" nil (hh eh1 v iy0 hh ae1 n d ih0 d))
("heavyset" nil (hh eh1 v iy0 s eh1 t))
("heavyweight" nil (hh eh1 v iy0 w ey1 t))
("heavyweights" nil (hh eh1 v iy0 w ey1 t s))
("hebard" nil (hh eh1 b er0 d))
("hebb" nil (hh eh1 b))
("hebda" nil (hh eh1 b d ax))
("hebding" nil (hh eh1 b d ih0 ng))
("hebe" nil (hh iy1 b))
("hebel" nil (hh eh1 b ax l))
("hebeler" nil (hh eh1 b ax l er0))
("hebenstreit" nil (hh eh1 b ih0 n s t r ay0 t))
("heber" nil (hh iy1 b er0))
("heberer" nil (hh eh1 b er0 er0))
("heberle" nil (hh eh1 b er0 ax l))
("heberlein" nil (hh eh1 b er0 l ay1 n))
("heberling" nil (hh eh1 b er0 l ih0 ng))
("hebert" nil (hh eh1 b er0 t))
("hebner" nil (hh eh1 b n er0))
("hebrew" nil (hh iy1 b r uw0))
("hebrides" nil (hh eh1 b r ih0 d iy0 z))
("hebron" nil (hh eh1 b r ax n))
("hecco" nil (hh eh1 k ow0))
("hechinger" nil (hh eh1 k ih0 n jh er0))
("hechler" nil (hh eh1 k l er0))
("hecht" nil (hh eh1 k t))
("hechtman" nil (hh eh1 k t m ax n))
("heck" nil (hh eh1 k))
("heckaman" nil (hh eh1 k ax m ax n))
("heckard" nil (hh eh1 k er0 d))
("heckart" nil (hh eh1 k er0 t))
("heckathorn" nil (hh eh1 k ax th er0 n))
("heckbert" nil (hh eh1 k b er0 t))
("heckel" nil (hh eh1 k ax l))
("heckendorn" nil (hh eh1 k eh0 n d ao0 r n))
("hecker" nil (hh eh1 k er0))
("heckert" nil (hh eh1 k er0 t))
("heckle" nil (hh eh1 k ax l))
("heckled" nil (hh eh1 k ax l d))
("heckler" nil (hh eh1 k l er0))
("hecklers" nil (hh eh1 k l er0 z))
("heckling" nil (hh eh1 k l ih0 ng))
("heckman" nil (hh eh1 k m ax n))
("heckmann" nil (hh eh1 k m ax n))
("heckuva" nil (hh eh0 k y uw1 v ax))
("hecla" nil (hh eh1 k l ax))
("hecox" nil (hh eh1 k aa0 k s))
("hectare" nil (hh eh1 k t aa1 r))
("hectares" nil (hh eh1 k t aa1 r z))
("hectic" nil (hh eh1 k t ih0 k))
("hectograph" nil (hh eh1 k t ax g r ae1 f))
("hector" nil (hh eh1 k t er0))
("hectoring" nil (hh eh1 k t er0 ih0 ng))
("hecuba" nil (hh eh1 k y ax b ax))
("heda" nil (hh ey1 d ax))
("hedberg" nil (hh eh1 d b er0 g))
("hedda" nil (hh eh1 d ax))
("hedden" nil (hh eh1 d ax n))
("hedding" nil (hh eh1 d ih0 ng))
("heddy" nil (hh eh1 d iy0))
("hedeen" nil (hh eh1 d iy0 n))
("hedge" nil (hh eh1 jh))
("hedgecock" nil (hh eh1 jh k aa1 k))
("hedged" nil (hh eh1 jh d))
("hedgehog" nil (hh eh1 jh hh aa1 g))
("hedgehogs" nil (hh eh1 jh hh aa1 g z))
("hedgepath" nil (hh eh1 jh p ae1 th))
("hedgepeth" nil (hh eh1 jh ih0 p eh0 th))
("hedger" nil (hh eh1 jh er0))
("hedgers" nil (hh eh1 jh er0 z))
("hedges" nil (hh eh1 jh ih0 z))
("hedging" nil (hh eh1 jh ih0 ng))
("hedglin" nil (hh eh1 jh l ih0 n))
("hedgpeth" nil (hh eh1 jh p ih0 th))
("hedi" nil (hh eh1 d iy0))
("hediger" nil (hh eh1 d ih0 g er0))
("hedin" nil (hh eh1 d ih0 n))
("hedley" nil (hh eh1 d l iy0))
("hedlund" nil (hh eh1 d l ax n d))
("hedman" nil (hh eh1 d m ax n))
("hedonic" nil (hh ax d aa1 n ih0 k))
("hedonism" nil (hh iy1 d ax n ih1 z ax m))
("hedonistic" nil (hh iy1 d ax n ih1 s t ih0 k))
("hedquist" nil (hh eh1 d k w ih1 s t))
("hedrich" nil (hh eh1 d r ih0 k))
("hedrick" nil (hh eh1 d r ih0 k))
("hedstrom" nil (hh eh1 d s t r ax m))
("hedtke" nil (hh eh1 d k iy0))
("hedwig" nil (hh eh1 d w ih0 g))
("hedwiga" nil (hh eh1 d w ih0 g ax))
("hedy" nil (hh iy1 d iy0))
("hee" nil (hh iy1))
("heeb" nil (hh iy1 b))
("heebner" nil (hh iy1 b n er0))
("heed" nil (hh iy1 d))
("heeded" nil (hh iy1 d ax d))
("heeding" nil (hh iy1 d ih0 ng))
("heeds" nil (hh iy1 d z))
("heefner" nil (hh iy1 f n er0))
("heeg" nil (hh iy1 g))
("heeke" nil (hh iy1 k))
("heekin" nil (hh iy1 k ih0 n))
("heel" nil (hh iy1 l))
("heelan" nil (hh iy1 l ax n))
("heeled" nil (hh iy1 l d))
("heels" nil (hh iy1 l z))
("heemstra" nil (hh iy1 m s t r ax))
("heenan" nil (hh iy1 n ax n))
("heeney" nil (hh iy1 n iy0))
("heer" nil (hh iy1 er0))
("heeren" nil (hh ih1 r ax n))
("heermann" nil (hh ih1 r m ax n))
("hees" nil (hh iy1 z))
("heesch" nil (hh iy1 sh))
("heese" nil (hh iy1 z))
("heeter" nil (hh iy1 t er0))
("hefei" nil (hh ax f ey1))
("heffel" nil (hh eh1 f ax l))
("heffelfinger" nil (hh eh1 f ih0 l f ih0 ng er0))
("hefferan" nil (hh eh1 f er0 ax n))
("hefferman" nil (hh eh1 f er0 m ax n))
("heffern" nil (hh eh1 f er0 n))
("heffernan" nil (hh eh1 f er0 n ax n))
("hefferon" nil (hh eh1 f er0 ax n))
("heffington" nil (hh eh1 f ih0 ng t ax n))
("heffler" nil (hh eh1 f l er0))
("heffley" nil (hh eh1 f l iy0))
("heffner" nil (hh eh1 f n er0))
("heffron" nil (hh eh1 f r ax n))
("hefler" nil (hh eh1 f l er0))
("hefley" nil (hh eh1 f l iy0))
("heflin" nil (hh eh1 f l ih0 n))
("hefner" nil (hh eh1 f n er0))
("heft" nil (hh eh1 f t))
("hefter" nil (hh eh1 f t er0))
("hefti" nil (hh eh1 f t iy0))
("heftier" nil (hh eh1 f t iy0 er0))
("heftiest" nil (hh eh1 f t iy0 ax s t))
("hefty" nil (hh eh1 f t iy0))
("hegadorn" nil (hh eh1 g ax d ao1 r n))
("hegarty" nil (hh eh1 g aa0 r t iy0))
("hege" nil (hh iy1 jh))
("hegedus" nil (hh eh1 g ih0 d ih0 s))
("hegel" nil (hh eh1 g ax l))
("hegelian" nil (hh iy0 jh iy1 l iy0 ax n))
("hegeman" nil (hh iy1 g m ax n))
("hegemonic" nil (hh eh1 g ax m aa1 n ih0 k))
("hegemony" nil (hh iy0 jh eh1 m ax n iy0))
("hegenna" nil (hh eh0 g eh1 n ax))
("heger" nil (hh iy1 g er0))
("hegg" nil (hh eh1 g))
("hegge" nil (hh eh1 g))
("heggen" nil (hh eh1 g ax n))
("heggie" nil (hh eh1 g iy0))
("hegland" nil (hh eh1 g l ax n d))
("hegler" nil (hh eh1 g l er0))
("heglund" nil (hh eh1 g l ax n d))
("hegna" nil (hh eh1 g n ax))
("hegner" nil (hh eh1 g n er0))
("hegstrom" nil (hh eh1 g s t r ax m))
("hegwood" nil (hh eh1 g w uh1 d))
("hegyi" nil (hh ey1 g y iy0))
("heh" nil (hh eh1))
("hehir" nil (hh eh1 hh ih0 r))
("hehl" nil (hh eh1 l))
("hehman" nil (hh eh1 m ax n))
("hehmeyer" nil (hh eh1 m ay1 r))
("hehn" nil (hh eh1 n))
("hehr" nil (hh eh1 r))
("heibel" nil (hh ay1 b ax l))
("heiberg" nil (hh ay1 b er0 g))
("heiberger" nil (hh ay1 b er0 g er0))
("heichel" nil (hh ay1 k ax l))
("heichelbech" nil (hh ay1 k ih0 l b ih0 k))
("heick" nil (hh ay1 k))
("heico" nil (hh ay1 k ow0))
("heid" nil (hh ay1 d))
("heidbreder" nil (hh ay1 d b r ih0 d er0))
("heidbrink" nil (hh ay1 d b r ih0 ng k))
("heide" nil (hh ay1 d))
("heidecker" nil (hh ay1 d ih0 k er0))
("heidel" nil (hh ay1 d ax l))
("heidelberg" nil (hh ay1 d ax l b er0 g))
("heidelberger" nil (hh ay1 d ax l b er0 g er0))
("heideman" nil (hh ay1 d m ax n))
("heidemann" nil (hh ay1 d m ax n))
("heiden" nil (hh ay1 d ax n))
("heidenreich" nil (hh ay1 d ih0 n r ay0 k))
("heider" nil (hh ay1 d er0))
("heidi" nil (hh ay1 d iy0))
("heidinger" nil (hh ay1 d ih0 ng er0))
("heidler" nil (hh ay1 d ax l er0))
("heidorn" nil (hh ay1 d er0 n))
("heidrich" nil (hh ay1 d r ih0 k))
("heidrick" nil (hh ay1 d r ih0 k))
("heidt" nil (hh ay1 d t))
("heier" nil (hh ay1 er0))
("heifers" nil (hh eh1 f er0 z))
("heifetz" nil (hh ay1 f ih0 t s))
("heifner" nil (hh iy1 f n er0))
("heiges" nil (hh ay1 jh ih0 z))
("height" nil (hh ay1 t))
("heighten" nil (hh ay1 t ax n))
("heightened" nil (hh ay1 t ax n d))
("heightening" nil (hh ay1 t ax n ih0 ng))
("heightens" nil (hh ay1 t ax n z))
("heights" nil (hh ay1 t s))
("heigl" nil (hh ay1 g ax l))
("heiken" nil (hh ay1 k ax n))
("heikes" nil (hh ay1 k ax z))
("heikkila" nil (hh ay1 k ih0 l ax))
("heikkinen" nil (hh ay1 k ih0 n ax n))
("heil" nil (hh ay1 l))
("heiland" nil (hh ay1 l ax n d))
("heilbrun" nil (hh ay1 l b r ax n))
("heileman" nil (hh ay1 l m ax n))
("heiler" nil (hh ay1 l er0))
("heilig" nil (hh ay1 l ih0 g))
("heilman" nil (hh ay1 l m ax n))
("heilmann" nil (hh ay1 l m ax n))
("heim" nil (hh ay1 m))
("heiman" nil (hh ay1 m ax n))
("heimann" nil (hh ay1 m ax n))
("heimbach" nil (hh ay1 m b aa1 k))
("heimberger" nil (hh ay1 m b er0 g er0))
("heimbigner" nil (hh ay1 m b ay0 g n er0))
("heimbuch" nil (hh ay1 m b ax k))
("heimburger" nil (hh ay1 m b er0 g er0))
("heimer" nil (hh ay1 m er0))
("heimerl" nil (hh ay1 m er0 l))
("heimlich" nil (hh ay1 m l ih0 k))
("heims" nil (hh ay1 m z))
("heimsoth" nil (hh ay1 m s ax th))
("hein" nil (hh ay1 n))
("heinbach" nil (hh ay1 n b aa1 k))
("heinbaugh" nil (hh ay1 n b aw0))
("heindel" nil (hh ay1 n d ax l))
("heindl" nil (hh ay1 n d ax l))
("heine" nil (hh ay1 n))
("heinecke" nil (hh ay1 n ih0 k))
("heineken" nil (hh ay1 n ax k ax n))
("heineman" nil (hh ay1 n m ax n))
("heinemann" nil (hh ay1 n m ax n))
("heinen" nil (hh ay1 n ax n))
("heiner" nil (hh ay1 n er0))
("heines" nil (hh ay1 n z))
("heiney" nil (hh ay1 n iy0))
("heinicke" nil (hh ay1 n ih0 k))
("heinig" nil (hh ay1 n ih0 g))
("heininger" nil (hh ay1 n ih0 ng er0))
("heinisch" nil (hh ay1 n ih0 sh))
("heinke" nil (hh ay1 ng k))
("heinkel" nil (hh ay1 ng k ax l))
("heinl" nil (hh ay1 n ax l))
("heinle" nil (hh ay1 n ax l))
("heinlein" nil (hh ay1 n l ay1 n))
("heinlen" nil (hh ay1 n ax l ax n))
("heinly" nil (hh ay1 n l iy0))
("heino" nil (hh ay1 n ow0))
("heinold" nil (hh ay1 n ow0 l d))
("heinonen" nil (hh ay1 n ax n ax n))
("heinous" nil (hh ey1 n ax s))
("heinrich" nil (hh ay1 n r ih0 k))
("heinrichs" nil (hh ay1 n r ih0 k s))
("heins" nil (hh ay1 n z))
("heinsohn" nil (hh ay1 n s ax n))
("heintz" nil (hh ay1 n t s))
("heintze" nil (hh ay1 n t s))
("heintzelman" nil (hh ay1 n t z ax l m ax n))
("heintzman" nil (hh ay1 n t s m ax n))
("heiny" nil (hh ay1 n iy0))
("heinz" nil (hh ay1 n z))
("heinze" nil (hh ay1 n z))
("heinzel" nil (hh ay1 n z ax l))
("heinzelman" nil (hh ay1 n z ax l m ax n))
("heinzen" nil (hh ay1 n z ax n))
("heinzman" nil (hh ay1 n z m ax n))
("heinzmann" nil (hh ay1 n z m ax n))
("heiple" nil (hh ay1 p ax l))
("heir" nil (eh1 r))
("heiress" nil (eh1 r ax s))
("heirloom" nil (eh1 r l uw1 m))
("heirlooms" nil (eh1 r l uw1 m z))
("heironimus" nil (eh1 r aa0 n ih0 m uw0 s))
("heirs" nil (eh1 r z))
("heisbourg" nil (hh ay1 s b ao1 r g))
("heise" nil (hh ay1 s))
("heisel" nil (hh ay1 s ax l))
("heiser" nil (hh ay1 s er0))
("heiserman" nil (hh ay1 s er0 m ax n))
("heisey" nil (hh ay1 s iy0))
("heishman" nil (hh iy1 ih0 sh m ax n))
("heisinger" nil (hh ay1 s ih0 n jh er0))
("heiskell" nil (hh ay1 s k ax l))
("heisler" nil (hh ay1 s ax l er0))
("heisman" nil (hh ay1 s m ax n))
("heisner" nil (hh ay1 s n er0))
("heiss" nil (hh ay1 s))
("heist" nil (hh ay1 s t))
("heistand" nil (hh ay1 s t ax n d))
("heister" nil (hh ay1 s t er0))
("heit" nil (hh ay1 t))
("heitkamp" nil (hh ay1 t k ae1 m p))
("heitman" nil (hh ay1 t m ax n))
("heitmann" nil (hh ay1 t m ax n))
("heitmeyer" nil (hh ay1 t m ay0 er0))
("heitner" nil (hh ay1 t n er0))
("heitz" nil (hh ay1 t s))
("heitzenrater" nil (hh ay1 t z ih0 n r ey0 t er0))
("heitzman" nil (hh ay1 t s m ax n))
("heiwa" nil (hh ay1 w ax))
("heizer" nil (hh ay1 z er0))
("hejl" nil (hh eh1 jh l))
("hejna" nil (hh eh1 jh n ax))
("hekker" nil (hh eh1 k er0))
("hekmatyar" nil (hh eh1 k m ax t y aa1 r))
("helaba" nil (hh eh0 l aa1 b ax))
("helander" nil (hh eh1 l ax n d er0))
("helane" nil (hh ax l ey1 n))
("helber" nil (hh eh1 l b er0))
("helberg" nil (hh eh1 l b er0 g))
("helbert" nil (hh eh1 l b er0 t))
("helbig" nil (hh eh1 l b ih0 g))
("helbing" nil (hh eh1 l b ih0 ng))
("helbling" nil (hh eh1 l b ax l ih0 ng))
("held" nil (hh eh1 l d))
("heldenbrand" nil (hh eh1 l d ih0 n b r ax n d))
("heldentenor" nil (hh eh1 l d eh0 n t eh1 n er0))
("helder" nil (hh eh1 l d er0))
("helderman" nil (hh eh1 l d er0 m ax n))
("heldman" nil (hh eh1 l d m ax n))
("heldor" nil (hh eh1 l d er0))
("heldreth" nil (hh eh1 l d r ih0 th))
("heldring" nil (hh eh1 l d r ih0 ng))
("helds" nil (hh eh1 l d z))
("heldt" nil (hh eh1 l t))
("helen" nil (hh eh1 l ax n))
("helena" nil (hh eh1 l ax n ax))
("helene" nil (hh ax l iy1 n))
("helens" nil (hh eh1 l ax n z))
("helf" nil (hh eh1 l f))
("helfand" nil (hh eh1 l f ax n d))
("helfer" nil (hh eh1 l f er0))
("helfgott" nil (hh eh1 l f g aa1 t))
("helfman" nil (hh eh1 l f m ax n))
("helfrich" nil (hh eh1 l f r ih0 k))
("helga" nil (hh eh1 l g ax))
("helgerson" nil (hh eh1 l g er0 s ax n))
("helgesen" nil (hh eh1 l g iy0 z ax n))
("helgeson" nil (hh eh1 l g ih0 s ax n))
("helget" nil (hh eh1 l g ih0 t))
("helgren" nil (hh eh1 l g r eh0 n))
("helical" nil (hh eh1 l ih0 k ax l))
("helice" nil (hh eh1 l ih0 s))
("helicon" nil (hh eh1 l ih0 k aa1 n))
("helicons" nil (hh eh1 l ih0 k ax n z))
("helicopter" nil (hh eh1 l ih0 k aa1 p t er0))
("helicopters" nil (hh eh1 l ih0 k aa1 p t er0 z))
("helie" nil (hh eh1 l iy0))
("helin" nil (hh eh1 l ih0 n))
("heling" nil (hh iy1 l ih0 ng))
("helinski" nil (hh ih0 l ih1 n s k iy0))
("helionetics" nil (hh iy1 l iy0 ow0 n eh1 t ih0 k s))
("heliopolis" nil (hh iy1 l iy0 aa1 p ax l ax s))
("helios" nil (hh iy1 l iy0 aa1 s))
("heliotrope" nil (hh iy1 l iy0 ax t r ow1 p))
("helium" nil (hh iy1 l iy0 ax m))
("helix" nil (hh iy1 l ih0 k s))
("helke" nil (hh eh1 l k))
("hell" nil (hh eh1 l))
("hellacious" nil (hh eh1 l ey1 sh ax s))
("hellaciously" nil (hh eh1 l ey1 sh ax s l iy0))
("hellaciousness" nil (hh eh1 l ey1 sh ax s n ax s))
("hellams" nil (hh eh1 l ax m z))
("helland" nil (hh eh1 l ax n d))
("hellard" nil (hh eh1 l er0 d))
("hellberg" nil (hh eh1 l b er0 g))
("hellbusch" nil (hh eh1 l b ax sh))
("helle" nil (hh eh1 l))
("hellen" nil (hh eh1 l ax n))
("hellenbrand" nil (hh eh1 l ax n b r ae1 n d))
("hellenic" nil (hh ax l eh1 n ih0 k))
("hellenism" nil (hh eh1 l ax n ih1 z ax m))
("hellenistic" nil (hh eh1 l ax n ih1 s t ih0 k))
("hellenized" nil (hh eh1 l ax n ay1 z d))
("heller" nil (hh eh1 l er0))
("hellerman" nil (hh eh1 l er0 m ax n))
("hellfire" nil (hh eh1 l f ay1 r))
("hellickson" nil (hh eh1 l ih0 k s ax n))
("helling" nil (hh eh1 l ih0 ng))
("hellinger" nil (hh eh1 l ih0 ng er0))
("hellish" nil (hh eh1 l ih0 sh))
("hellman" nil (hh eh1 l m ax n))
("hellmann" nil (hh eh1 l m ax n))
("hellmer" nil (hh eh1 l m er0))
("hellmuth" nil (hh eh1 l m uw1 th))
("hellner" nil (hh eh1 l n er0))
("hello" nil (hh ax l ow1))
("hellstrom" nil (hh eh1 l s t r ax m))
("hellums" nil (hh eh1 l ax m z))
("helluva" nil (hh eh1 l uw1 v ax))
("hellwig" nil (hh eh1 l w ih0 g))
("hellyer" nil (hh eh1 l iy0 er0))
("helm" nil (hh eh1 l m))
("helma" nil (hh eh1 l m ax))
("helman" nil (hh eh1 l m ax n))
("helmbrecht" nil (hh eh1 l m b r ih0 k t))
("helme" nil (hh eh1 l m))
("helmer" nil (hh eh1 l m er0))
("helmerich" nil (hh eh1 l m er0 ih0 k))
("helmers" nil (hh eh1 l m er0 z))
("helmes" nil (hh eh1 l m z))
("helmet" nil (hh eh1 l m ax t))
("helmets" nil (hh eh1 l m ax t s))
("helmich" nil (hh eh1 l m ih0 k))
("helmick" nil (hh eh1 l m ih0 k))
("helmig" nil (hh eh1 l m ih0 g))
("helming" nil (hh eh1 l m ih0 ng))
("helminiak" nil (hh eh1 l m ih0 n iy0 ae0 k))
("helminski" nil (hh ih0 l m ih1 n s k iy0))
("helminth" nil (hh eh1 l m ih0 n th))
("helmkamp" nil (hh eh1 l m k ae1 m p))
("helmke" nil (hh eh1 l m k))
("helmont" nil (hh eh1 l m aa1 n t))
("helms" nil (hh eh1 l m z))
("helmsley" nil (hh eh1 l m z l iy0))
("helmsleys" nil (hh eh1 l m z l iy0 z))
("helmsman" nil (hh eh1 l m z m ae1 n))
("helmstetter" nil (hh eh1 l m s t ih0 t er0))
("helmut" nil (hh eh1 l m ax t))
("helmuth" nil (hh eh1 l m uw1 th))
("helot" nil (hh eh1 l ax t))
("helotism" nil (hh eh1 l ax t ih1 z ax m))
("helotry" nil (hh eh1 l ax t r iy0))
("helots" nil (hh eh1 l ax t s))
("help" nil (hh eh1 l p))
("helped" nil (hh eh1 l p t))
("helper" nil (hh eh1 l p er0))
("helpers" nil (hh eh1 l p er0 z))
("helpful" nil (hh eh1 l p f ax l))
("helpfully" nil (hh eh1 l p f ax l iy0))
("helping" nil (hh eh1 l p ih0 ng))
("helpings" nil (hh eh1 l p ih0 ng z))
("helpless" nil (hh eh1 l p l ax s))
("helplessly" nil (hh eh1 l p l ax s l iy0))
("helplessness" nil (hh eh1 l p l ax s n ax s))
("helprin" nil (hh eh1 l p r ih0 n))
("helps" nil (hh eh1 l p s))
("helsel" nil (hh eh1 l s ax l))
("helser" nil (hh eh1 l s er0))
("helseth" nil (hh eh1 l s ih0 th))
("helsinki" nil (hh eh1 l s ih0 ng k iy0))
("helsley" nil (hh eh1 l s l iy0))
("helstrom" nil (hh eh1 l s t r ax m))
("helt" nil (hh eh1 l t))
("helter" nil (hh eh1 l t er0))
("helton" nil (hh eh1 l t ax n))
("heltsley" nil (hh eh1 l t s l iy0))
("heltzel" nil (hh eh1 l t z ax l))
("helvey" nil (hh eh1 l v iy0))
("helvie" nil (hh eh1 l v iy0))
("helwig" nil (hh eh1 l w ih0 g))
("helzer" nil (hh eh1 l z er0))
("hem" nil (hh eh1 m))
("heman" nil (hh iy1 m ax n))
("hemann" nil (hh eh1 m ax n))
("hemant" nil (hh eh1 m ax n t))
("hematite" nil (hh eh1 m ax t ay1 t))
("hematology" nil (hh eh1 m ax t aa1 l ax jh iy0))
("hemberger" nil (hh eh1 m b er0 g er0))
("hembree" nil (hh ih0 m b r iy1))
("hembrick" nil (hh eh1 m b r ih1 k))
("hemby" nil (hh eh1 m b iy0))
("hemdale" nil (hh eh1 m d ey1 l))
("hemel" nil (hh eh1 m ax l))
("hemenway" nil (hh eh1 m ax n w ey1))
("hemimorphite" nil (hh eh1 m ax m ao1 r f ay1 t))
("heming" nil (hh eh1 m ih0 ng))
("heminger" nil (hh eh1 m ih0 ng er0))
("hemingway" nil (hh eh1 m ih0 ng w ey1))
("hemiplegia" nil (hh eh1 m ax p l iy1 jh iy0 ax))
("hemisphere" nil (hh eh1 m ih0 s f ih1 r))
("hemispheric" nil (hh eh1 m ax s f ih1 r ih0 k))
("hemker" nil (hh eh1 m k er0))
("hemler" nil (hh eh1 m l er0))
("hemline" nil (hh eh1 m l ay1 n))
("hemlines" nil (hh eh1 m l ay1 n z))
("hemlo" nil (hh eh1 m l ow0))
("hemlock" nil (hh eh1 m l aa1 k))
("hemm" nil (hh eh1 m))
("hemme" nil (hh eh1 m))
("hemmed" nil (hh eh1 m d))
("hemmelgarn" nil (hh eh1 m ih0 l g aa0 r n))
("hemmen" nil (hh eh1 m ax n))
("hemmer" nil (hh eh1 m er0))
("hemmerich" nil (hh eh1 m er0 ih0 k))
("hemmerle" nil (hh eh1 m er0 l iy0))
("hemmerling" nil (hh eh1 m er0 l ih0 ng))
("hemmert" nil (hh eh1 m er0 t))
("hemmeter" nil (hh eh1 m ih0 t er0))
("hemming" nil (hh eh1 m ih0 ng))
("hemminger" nil (hh eh1 m ih0 ng er0))
("hemminghaus" nil (hh eh1 m ih0 ng hh aw1 s))
("hemmings" nil (hh eh1 m ih0 ng z))
("hemmingsen" nil (hh eh1 m ih0 ng s ax n))
("hemmingson" nil (hh eh1 m ih0 ng s ax n))
("hemocyanin" nil (hh iy1 m ax s ay1 ax n ax n))
("hemodynamics" nil (hh eh1 m ow0 d ay0 n ae1 m ih0 k s))
("hemoglobin" nil (hh iy1 m ax g l ow1 b ax n))
("hemolytic" nil (hh iy1 m ax l ih1 t ih0 k))
("hemond" nil (hh eh1 m ax n d))
("hemophilia" nil (hh iy1 m ax f iy1 l iy0 ax))
("hemophiliacs" nil (hh iy1 m ax f ih1 l iy0 ae1 k s))
("hemorrhage" nil (hh eh1 m er0 ih0 jh))
("hemorrhaged" nil (hh eh1 m er0 ih0 jh d))
("hemorrhagic" nil (hh eh1 m er0 ae1 g ih0 k))
("hemorrhaging" nil (hh eh1 m er0 ih0 jh ih0 ng))
("hemorrhoid" nil (hh eh1 m er0 oy1 d))
("hemorrhoids" nil (hh eh1 m er0 oy1 d z))
("hemotec" nil (hh eh1 m ow0 t eh1 k))
("hemp" nil (hh eh1 m p))
("hempel" nil (hh eh1 m p ax l))
("hempen" nil (hh eh1 m p ax n))
("hempfling" nil (hh eh1 m p f ax l ih0 ng))
("hemphill" nil (hh eh1 m p hh ih1 l))
("hempstead" nil (hh eh1 m p s t eh0 d))
("hemric" nil (hh eh1 m r ih0 k))
("hemrick" nil (hh eh1 m r ih0 k))
("hemry" nil (hh eh1 m r iy0))
("hems" nil (hh eh1 m z))
("hemsley" nil (hh eh1 m z l iy0))
("hemstreet" nil (hh eh1 m s t r iy1 t))
("hemy" nil (hh eh1 m iy0))
("hen" nil (hh eh1 n))
("henao" nil (hh ey1 n aw0))
("henard" nil (hh eh1 n er0 d))
("henault" nil (hh eh1 n aw0 l t))
("henbane" nil (hh eh1 n b ey1 n))
("hence" nil (hh eh1 n s))
("henceforth" nil (hh eh1 n s f ao1 r th))
("hench" nil (hh eh1 n ch))
("henchman" nil (hh eh1 n ch m ax n))
("henchmen" nil (hh eh1 n ch m ax n))
("henckel" nil (hh eh1 n k ax l))
("hendee" nil (hh eh1 n d iy0))
("hendel" nil (hh eh1 n d ax l))
("hendershot" nil (hh eh1 n d er0 sh ax t))
("hendershott" nil (hh eh1 n d er0 sh ax t))
("henderson" nil (hh eh1 n d er0 s ax n))
("hendler" nil (hh eh1 n d l er0))
("hendley" nil (hh eh1 n d l iy0))
("hendon" nil (hh eh1 n d ow0 n))
("hendra" nil (hh eh1 n d r ax))
("hendren" nil (hh eh1 n d er0 ax n))
("hendrich" nil (hh eh1 n d r ih0 k))
("hendrick" nil (hh eh1 n d r ih0 k))
("hendricks" nil (hh eh1 n d r ih0 k s))
("hendricksen" nil (hh eh1 n d r ih0 k s ax n))
("hendrickson" nil (hh eh1 n d r ih0 k s ax n))
("hendrie" nil (hh eh1 n d er0 iy0))
("hendrik" nil (hh eh1 n d r ih0 k))
("hendriks" nil (hh eh1 n d r ih0 k s))
("hendriksen" nil (hh eh1 n d r ih0 k s ax n))
("hendrix" nil (hh eh1 n d r ih0 k s))
("hendrixson" nil (hh eh1 n d r ih0 k s ax n))
("hendron" nil (hh eh1 n d r ax n))
("hendry" nil (hh eh1 n d r iy0))
("hendryx" nil (hh eh1 n d r ih0 k s))
("hendy" nil (hh eh1 n d iy0))
("henegar" nil (hh eh1 n ih0 g er0))
("heneghan" nil (hh ih0 n eh1 g hh ax n))
("henehan" nil (hh eh1 n ih0 hh ae0 n))
("henery" nil (hh eh1 n er0 iy0))
("henes" nil (hh iy1 n z))
("heney" nil (hh eh1 n iy0))
("heng" nil (hh eh1 ng))
("hengel" nil (hh eh1 ng g ax l))
("hengst" nil (hh eh1 ng g s t))
("henhouse" nil (hh eh1 n hh aw1 s))
("henigan" nil (hh eh1 n ih0 g ax n))
("henin" nil (hh eh1 n ih0 n))
("heninger" nil (hh eh1 n ih0 ng er0))
("henion" nil (hh eh1 n y ax n))
("henk" nil (hh eh1 ng k))
("henke" nil (hh eh1 ng k))
("henkel" nil (hh eh1 ng k ax l))
("henkelman" nil (hh eh1 ng k ax l m ax n))
("henkels" nil (hh eh1 ng k ax l z))
("henken" nil (hh eh1 ng k ax n))
("henkes" nil (hh eh1 ng k s))
("henkin" nil (hh eh1 ng k ih0 n))
("henkle" nil (hh eh1 ng k ax l))
("henle" nil (hh eh1 n ax l))
("henley" nil (hh eh1 n l iy0))
("henleys" nil (hh eh1 n l iy0 z))
("henline" nil (hh eh1 n l ay1 n))
("henly" nil (hh eh1 n l iy0))
("henman" nil (hh eh1 n m ax n))
("henn" nil (hh eh1 n))
("henna" nil (hh eh1 n ax))
("henne" nil (hh eh1 n))
("henneberger" nil (hh eh1 n b er0 g er0))
("henneberry" nil (hh eh1 n b eh0 r iy0))
("henneke" nil (hh eh1 n ih0 k))
("hennelly" nil (hh eh1 n ax l iy0))
("henneman" nil (hh eh1 n m ax n))
("hennen" nil (hh eh1 n ax n))
("hennepin" nil (hh eh1 n ih0 p ih0 n))
("henner" nil (hh eh1 n er0))
("hennes" nil (hh eh1 n z))
("henness" nil (hh eh1 n ih0 s))
("hennessee" nil (hh eh1 n ih0 s iy0))
("hennessey" nil (hh eh1 n ax s iy0))
("hennessy" nil (hh eh1 n ax s iy0))
("henney" nil (hh eh1 n iy0))
("hennick" nil (hh eh1 n ih0 k))
("hennig" nil (hh eh1 n ih0 g))
("hennigan" nil (hh eh1 n ih0 g ax n))
("hennigar" nil (hh eh1 n ih0 g er0))
("henning" nil (hh eh1 n ih0 ng))
("henninger" nil (hh eh1 n ih0 ng er0))
("hennings" nil (hh eh1 n ih0 ng z))
("henningsen" nil (hh eh1 n ih0 ng s ax n))
("hennington" nil (hh eh1 n ih0 ng t ax n))
("hennis" nil (hh eh1 n ih0 s))
("hennon" nil (hh eh1 n ax n))
("henpecked" nil (hh eh1 n p eh1 k t))
("henri" nil (hh eh1 n r iy0))
("henrich" nil (hh eh1 n r ih0 k))
("henrichs" nil (hh eh1 n r ih0 k s))
("henrichsen" nil (hh eh1 n r ih0 k s ax n))
("henrick" nil (hh eh1 n r ih0 k))
("henricks" nil (hh eh1 n r ih0 k s))
("henricksen" nil (hh eh1 n r ih0 k s ax n))
("henrickson" nil (hh eh1 n r ih0 k s ax n))
("henrie" nil (hh eh1 n er0 iy0))
("henrietta" nil (hh eh1 n r iy0 eh1 t ax))
("henriette" nil (hh eh1 n r iy0 eh1 t))
("henrik" nil (hh eh1 n r ih0 k))
("henrika" nil (hh eh1 n r ih0 k ax))
("henriksen" nil (hh eh1 n r ih0 k s ax n))
("henrikson" nil (hh eh1 n r ih0 k s ax n))
("henrique" nil (aa0 n r iy1 k))
("henriques" nil (hh eh0 n r iy1 k eh0 z))
("henriquez" nil (hh eh0 n r iy1 k eh0 z))
("henry" nil (hh eh1 n r iy0))
("hens" nil (hh eh1 n z))
("hensarling" nil (hh eh1 n s aa0 r l ih0 ng))
("hensch" nil (hh eh1 n sh))
("henschel" nil (hh eh1 n sh ax l))
("henschen" nil (hh eh1 n sh ax n))
("hense" nil (hh eh1 n s))
("hensel" nil (hh eh1 n s ax l))
("hensen" nil (hh eh1 n s ax n))
("henshaw" nil (hh eh1 n sh ao1))
("henske" nil (hh eh1 n s k iy0))
("henslee" nil (hh eh1 n z l iy1))
("hensler" nil (hh eh1 n s ax l er0))
("hensley" nil (hh eh1 n z l iy0))
("henson" nil (hh eh1 n s ax n))
("hentges" nil (hh eh1 n t jh ih0 z))
("henthorn" nil (hh eh1 n th er0 n))
("henthorne" nil (hh eh1 n th er0 n))
("hentic" nil (hh eh1 n t ih0 k))
("hentoff" nil (hh eh1 n t ao0 f))
("henton" nil (hh eh1 n t ax n))
("hentschel" nil (hh eh1 n ch ax l))
("hentz" nil (hh eh1 n t s))
("henwood" nil (hh eh1 n w uh1 d))
("henze" nil (hh eh1 n z))
("henzel" nil (hh eh1 n z ax l))
("henzler" nil (hh eh1 n z l er0))
("heon" nil (hh iy1 ax n))
("hep" nil (hh eh1 p))
("heparin" nil (hh eh1 p er0 ih0 n))
("hepatic" nil (hh ax p ae1 t ih0 k))
("hepatitis" nil (hh eh1 p ax t ay1 t ax s))
("hepburn" nil (hh eh1 p b er0 n))
("hepfer" nil (hh eh1 p f er0))
("hepker" nil (hh eh1 p k er0))
("hepler" nil (hh eh1 p l er0))
("hepner" nil (hh eh1 p n er0))
("hepp" nil (hh eh1 p))
("heppe" nil (hh eh1 p))
("hepper" nil (hh eh1 p er0))
("heppler" nil (hh eh1 p l er0))
("heppner" nil (hh eh1 p n er0))
("heptathlon" nil (hh eh0 p t ae1 th l aa0 n))
("hepworth" nil (hh eh1 p w er0 th))
("her" nil (hh er0))
("hera" nil (hh ih1 r ax))
("herald" nil (hh eh1 r ax l d))
("heralded" nil (hh eh1 r ax l d ih0 d))
("heraldic" nil (hh eh0 r ae1 l d ih0 k))
("heralding" nil (hh eh1 r ax l d ih0 ng))
("heraldry" nil (hh eh1 r ax l d r iy0))
("heralds" nil (hh eh1 r ax l d z))
("herb" nil (er1 b))
("herbaceous" nil (er0 b ey1 sh ax s))
("herbal" nil (er1 b ax l))
("herbalife" nil (hh er1 b ax l ay1 f))
("herbalist" nil (er1 b ax l ax s t))
("herbalists" nil (er1 b ax l ax s t s))
("herbariums" nil (hh er0 b eh1 r iy0 ax m z))
("herbeck" nil (hh er1 b eh0 k))
("herbel" nil (hh er1 b ax l))
("herber" nil (hh er1 b er0))
("herberg" nil (hh er1 b er0 g))
("herberger" nil (hh er1 b er0 g er0))
("herbers" nil (hh er1 b er0 z))
("herbert" nil (hh er1 b er0 t))
("herbicide" nil (hh er1 b ih0 s ay1 d))
("herbicides" nil (er1 b ax s ay1 d z))
("herbie" nil (hh er1 b iy0))
("herbig" nil (hh er1 b ih0 g))
("herbin" nil (hh er1 b ih0 n))
("herbison" nil (hh er1 b ih0 s ax n))
("herbivore" nil (hh er1 b ih0 v ao1 r))
("herbivorous" nil (hh er0 b ih1 v er0 ax s))
("herbold" nil (hh er1 b ow0 l d))
("herbs" nil (er1 b z))
("herbst" nil (hh er1 b s t))
("herbster" nil (hh er1 b s t er0))
("herceg" nil (hh er1 s ih0 g))
("hercegovina" nil (hh eh1 r t s ax g ow0 v iy1 n ax))
("herculean" nil (hh er0 k y uw1 l iy0 ax n))
("hercules" nil (hh er1 k y ax l iy1 z))
("herczeg" nil (hh er1 ch ih0 g))
("herd" nil (hh er1 d))
("herda" nil (hh eh1 r d ax))
("herded" nil (hh er1 d ih0 d))
("herder" nil (hh eh1 r d er0))
("herding" nil (hh er1 d ih0 ng))
("herdman" nil (hh er1 d m ax n))
("herds" nil (hh er1 d z))
("herdsmen" nil (hh er1 d z m ih0 n))
("herdt" nil (hh er1 t))
("here" nil (hh ih1 r))
("hereabout" nil (hh ih1 r ax b aw1 t))
("hereabouts" nil (hh ih1 r ax b aw1 t s))
("hereafter" nil (hh ih0 r ae1 f t er0))
("hereby" nil (hh ih0 r b ay1))
("heredia" nil (hh er0 eh1 d iy0 ax))
("hereditary" nil (hh er0 eh1 d ax t eh1 r iy0))
("heredity" nil (hh er0 eh1 d ax t iy0))
("hereford" nil (hh eh1 r ax f er0 d))
("herein" nil (hh ih0 r ih1 n))
("herendeen" nil (hh ih1 r n d iy0 n))
("heresy" nil (hh eh1 r ax s iy0))
("heretic" nil (hh eh1 r ax t ih0 k))
("heretical" nil (hh er0 eh1 t ih0 k ax l))
("heretofore" nil (hh ih1 r t ax f ao1 r))
("herewith" nil (hh ih1 r w ih1 th))
("herford" nil (hh er1 f er0 d))
("herfurth" nil (hh er1 f er0 th))
("hergert" nil (hh er1 g er0 t))
("herget" nil (hh er1 g ih0 t))
("hergott" nil (hh er1 g ax t))
("herin" nil (hh eh1 r ih0 n))
("hering" nil (hh er1 ih0 ng))
("heringer" nil (hh eh1 r ih0 n jh er0))
("herington" nil (hh eh1 r ih0 ng t ax n))
("heritable" nil (hh eh1 r ax t ax b ax l))
("heritage" nil (hh eh1 r ax t ax jh))
("herk" nil (hh er1 k))
("herkert" nil (hh er1 k er0 t))
("herl" nil (hh er1 l))
("herlihy" nil (hh er1 l ih0 hh iy0))
("herling" nil (hh er1 l ih0 ng))
("herlong" nil (hh er1 l ao0 ng))
("herm" nil (hh er1 m))
("herman" nil (hh er1 m ax n))
("hermance" nil (hh er1 m ax n s))
("hermann" nil (hh er1 m ax n))
("hermanns" nil (hh er1 m aa0 n z))
("hermans" nil (hh er1 m ax n z))
("hermansen" nil (hh er1 m ax n s ax n))
("hermanson" nil (hh er1 m ax n s ax n))
("hermaphrodite" nil (hh er0 m ae1 f r ax d ay1 t))
("hermaphroditic" nil (hh er0 m ae1 f r ax d ih1 t ih0 k))
("hermes" nil (hh er1 m iy0 z))
("hermetically" nil (hh er0 m eh1 t ih0 k ax l iy0))
("hermia" nil (hh er1 m iy0 ax))
("hermias" nil (hh er1 m iy0 ax z))
("hermida" nil (hh eh0 r m iy1 d ax))
("hermie" nil (hh er1 m iy0))
("hermina" nil (hh er1 m ih0 n ax))
("hermine" nil (hh er1 m ih0 n))
("herminia" nil (hh eh0 r m iy1 n iy0 ax))
("herminie" nil (hh er1 m ih0 n iy0))
("hermit" nil (hh er1 m ax t))
("hermitage" nil (hh er1 m ax t ax jh))
("hermits" nil (hh er1 m ax t s))
("hermon" nil (hh er1 m ax n))
("hermosa" nil (hh eh0 r m ow1 s ax))
("hermosillo" nil (hh er0 m ax s ih1 l ow0))
("herms" nil (hh er1 m z))
("hermsen" nil (hh er1 m s ax n))
("hern" nil (hh er1 n))
("hernan" nil (hh er1 n ax n))
("hernandes" nil (hh er1 n ih0 n d z))
("hernandez" nil (hh er0 n ae1 n d eh0 z))
("hernando" nil (hh er0 n aa1 n d ow0))
("herndon" nil (hh er1 n d ax n))
("herne" nil (hh er1 n))
("herner" nil (hh er1 n er0))
("hernia" nil (hh er1 n iy0 ax))
("herniates" nil (hh er1 n iy0 ey1 t s))
("hernon" nil (hh er1 n ax n))
("hero" nil (hh ih1 r ow0))
("herod" nil (hh eh1 r ax d))
("heroes" nil (hh ih1 r ow0 z))
("heroic" nil (hh ih0 r ow1 ih0 k))
("heroically" nil (hh ih1 r ow1 ih0 k l iy0))
("heroics" nil (hh ih0 r ow1 ih0 k s))
("heroin" nil (hh eh1 r ow0 ax n))
("heroine" nil (hh eh1 r ow0 ax n))
("heroines" nil (hh eh1 r ow1 ax n z))
("heroism" nil (hh eh1 r ow0 ih1 z ax m))
("heroized" nil (hh iy1 r ow0 ay1 z d))
("herold" nil (hh eh1 r ax l d))
("heron" nil (hh eh1 r ax n))
("herons" nil (hh eh1 r ax n z))
("heros" nil (hh ih1 r ow0 z))
("heroux" nil (hh er0 uw1))
("herpes" nil (hh er1 p iy0 z))
("herr" nil (hh eh1 r))
("herre" nil (hh eh1 r))
("herreid" nil (hh eh1 r ay0 d))
("herrell" nil (hh eh1 r ax l))
("herren" nil (hh eh1 r ax n))
("herrera" nil (hh er0 eh1 r ax))
("herrero" nil (hh eh0 r eh1 r ow0))
("herrhausen" nil (hh eh1 r hh aw1 z ax n))
("herriage" nil (hh eh1 r iy0 ih0 jh))
("herrick" nil (hh eh1 r ih0 k))
("herridge" nil (hh eh1 r ih0 jh))
("herrig" nil (hh eh1 r ih0 g))
("herriman" nil (hh eh1 r ih0 m ax n))
("herrin" nil (hh eh1 r ih0 n))
("herring" nil (hh eh1 r ih0 ng))
("herrings" nil (hh eh1 r ih0 ng z))
("herringshaw" nil (hh eh1 r ih0 ng sh ao1))
("herrington" nil (hh eh1 r ih0 ng t ax n))
("herriott" nil (hh eh1 r iy0 aa0 t))
("herrle" nil (hh eh1 r ax l))
("herrlinger" nil (hh eh1 r l ih1 ng er0))
("herrman" nil (hh eh1 r m ax n))
("herrmann" nil (hh eh1 r m ax n))
("herro" nil (hh eh1 r ow0))
("herrod" nil (hh eh1 r ax d))
("herrold" nil (hh eh1 r ow1 l d))
("herron" nil (hh eh1 r ax n))
("herronimo" nil (hh er0 aa1 n ax m ow0))
("hers" nil (hh er0 z))
("hersant" nil (hh er1 s ax n t))
("hersch" nil (hh er1 sh))
("herschel" nil (hh er1 sh ax l))
("herschell" nil (hh er1 sh ax l))
("herscu" nil (hh er0 s k uw1))
("herself" nil (hh er0 s eh1 l f))
("hersey" nil (hh er1 s iy0))
("hersh" nil (hh er1 sh))
("hershberger" nil (hh er1 sh b er0 g er0))
("hershey" nil (hh er1 sh iy0))
("hershiser" nil (hh er1 sh ay0 z er0))
("hershkowitz" nil (hh er1 sh k ax w ih0 t s))
("hershman" nil (hh er1 sh m ax n))
("hershner" nil (hh er1 sh n er0))
("herskovitz" nil (hh er1 s k ax v ih0 t s))
("herskowitz" nil (hh er1 s k ax w ih0 t s))
("hersman" nil (hh er1 s m ax n))
("hersom" nil (hh er1 s ax m))
("herson" nil (hh er1 s ax n))
("herst" nil (hh er1 s t))
("hert" nil (hh er1 t))
("herta" nil (hh er1 t ax))
("hertel" nil (hh er1 t ax l))
("hertenstein" nil (hh er1 t ax n s t ay0 n))
("herter" nil (hh er1 t er0))
("hertha" nil (hh er1 th ax))
("hertig" nil (hh er1 t ih0 g))
("herting" nil (hh er1 t ih0 ng))
("hertlein" nil (hh er1 t l ay0 n))
("hertog" nil (hh er1 t aa0 g))
("hertz" nil (hh eh1 r t s))
("hertzberg" nil (hh er1 t s b er0 g))
("hertzenleben" nil (hh er1 t ax n l ey1 b ax n))
("hertzler" nil (hh er1 t z ax l er0))
("hertzog" nil (hh er1 t z ax g))
("herve" nil (hh er1 v))
("hervey" nil (hh er0 v ey1))
("herwick" nil (hh er1 w ih1 k))
("herwig" nil (hh er1 w ih0 g))
("herz" nil (hh er1 z))
("herzberg" nil (hh er1 z b er0 g))
("herzberger" nil (hh er1 z b er0 g er0))
("herzegovina" nil (hh eh1 r t s ax g ow0 v iy1 n ax))
("herzer" nil (hh er1 z er0))
("herzfeld" nil (hh er1 z f eh0 l d))
("herzig" nil (hh er1 z ih0 g))
("herzing" nil (hh er1 z ih0 ng))
("herzlinger" nil (hh er1 z l ih1 ng er0))
("herzog" nil (hh er1 z aa0 g))
("hesch" nil (hh eh1 sh))
("heseltine" nil (hh eh1 s ax l t iy1 n))
("heselton" nil (hh ih0 s eh1 l t ax n))
("hesiod" nil (hh iy1 s iy0 ax d))
("hesitancy" nil (hh eh1 z ih0 t ax n s iy0))
("hesitant" nil (hh eh1 z ih0 t ax n t))
("hesitantly" nil (hh eh1 z ih0 t ax n t l iy0))
("hesitate" nil (hh eh1 z ax t ey1 t))
("hesitated" nil (hh eh1 z ih0 t ey1 t ih0 d))
("hesitates" nil (hh eh1 z ax t ey1 t s))
("hesitating" nil (hh eh1 z ax t ey1 t ih0 ng))
("hesitation" nil (hh eh1 z ax t ey1 sh ax n))
("hesketh" nil (hh eh1 s k ih0 th))
("heskett" nil (hh eh1 s k ih0 t))
("heslep" nil (hh eh1 s l ih0 p))
("hesler" nil (hh eh1 s ax l er0))
("heslin" nil (hh eh1 s l ih0 n))
("heslop" nil (hh eh1 s l ax p))
("hesper" nil (hh eh1 s p er0))
("hespera" nil (hh ey0 s p eh1 r ax))
("hess" nil (hh eh1 s))
("hesse" nil (hh eh1 s))
("hessel" nil (hh eh1 s ax l))
("hesseltine" nil (hh eh1 s ih0 l t iy0 n))
("hesser" nil (hh eh1 s er0))
("hessian" nil (hh eh1 sh ax n))
("hessing" nil (hh eh1 s ih0 ng))
("hessinger" nil (hh eh1 s ih0 n jh er0))
("hession" nil (hh eh1 sh ih0 n))
("hessite" nil (hh eh1 s ay0 t))
("hessler" nil (hh eh1 s l er0))
("hessling" nil (hh eh1 s ax l ih0 ng))
("hesson" nil (hh eh1 s ax n))
("hesston" nil (hh eh1 s t ax n))
("hestand" nil (hh eh1 s t ax n d))
("hester" nil (hh eh1 s t er0))
("hesther" nil (hh eh1 s dh er0))
("hestia" nil (hh eh1 s t iy0 ax))
("heston" nil (hh eh1 s t ax n))
("heterocercal" nil (hh eh1 t er0 ow0 s er1 k ax l))
("heterodox" nil (hh eh1 t er0 ax d aa1 k s))
("heterodoxy" nil (hh eh1 t er0 ax d aa1 k s iy0))
("heterodyne" nil (hh eh1 t er0 ax d ay1 n))
("heterogeneity" nil (hh eh1 t er0 ax jh ih0 n iy1 ax t iy0))
("heterogeneous" nil (hh eh1 t er0 ax jh iy1 n y ax s))
("heterosexual" nil (hh eh1 t er0 ow0 s eh1 k sh ax w ax l))
("heterosexuals" nil (hh eh1 t er0 ow0 s eh1 k sh ax w ax l z))
("heterosis" nil (hh eh1 t er0 ow1 s ax s))
("heterosporous" nil (hh eh1 t er0 aa1 s p er0 ax s))
("heterotrophic" nil (hh eh1 t er0 ax t r aa1 f ih0 k))
("heterozygous" nil (hh eh1 t er0 ax z ay1 g ax s))
("heth" nil (hh eh1 th))
("hetherington" nil (hh eh1 dh er0 ih0 ng t ax n))
("hetland" nil (hh eh1 t l ax n d))
("hetman" nil (hh eh1 t m ax n))
("hetrick" nil (hh eh1 t r ih0 k))
("hett" nil (hh eh1 t))
("hettel" nil (hh eh1 t ax l))
("hetti" nil (hh eh1 t iy0))
("hettick" nil (hh eh1 t ih0 k))
("hettie" nil (hh eh1 t iy0))
("hettler" nil (hh eh1 t l er0))
("hettrick" nil (hh eh1 t r ih0 k))
("hetty" nil (hh eh1 t iy0))
("hetu" nil (hh iy1 ch uw0))
("hetz" nil (hh eh1 t s))
("hetzel" nil (hh eh1 t z ax l))
("hetzer" nil (hh eh1 t z er0))
("hetzler" nil (hh eh1 t s l er0))
("heuberger" nil (hh oy1 b er0 g er0))
("heublein" nil (hh y uw1 b l ay1 n))
("heuer" nil (hh y uw1 er0))
("heuerman" nil (hh oy1 er0 m ax n))
("heuermann" nil (hh oy1 er0 m ax n))
("heuman" nil (hh y uw1 m ax n))
("heumann" nil (hh y uw1 m ax n))
("heun" nil (hh y uw1 n))
("heupel" nil (hh oy1 p ax l))
("heuring" nil (hh er1 ih0 ng))
("heusen" nil (hh y uw1 s ax n))
("heuser" nil (hh oy1 s er0))
("heusser" nil (hh y uw1 s er0))
("hevener" nil (hh eh1 v iy0 n er0))
("heverly" nil (hh eh1 v er0 l iy0))
("hevey" nil (hh ih0 v ey1))
("hevia" nil (hh ey1 v iy0 ax))
("hew" nil (hh y uw1))
("heward" nil (hh y uw1 er0 d))
("hewe" nil (hh y uw1))
("hewell" nil (hh eh1 w eh0 l))
("hewes" nil (hh y uw1 z))
("hewett" nil (hh y uw1 ih0 t))
("hewey" nil (hh y uw1 iy0))
("hewing" nil (hh y uw1 ih0 ng))
("hewins" nil (hh y uw1 ih0 n z))
("hewitt" nil (hh y uw1 ih0 t))
("hewler" nil (hh y uw1 l er0))
("hewlett" nil (hh y uw1 l ih0 t))
("hewn" nil (hh y uw1 n))
("hews" nil (hh y uw1 z))
("hewson" nil (hh y uw1 s ax n))
("hexachlorophene" nil (hh eh1 k s ax k l ao1 r ax f iy1 n))
("hexagon" nil (hh eh1 k s ax g aa1 n))
("hexagonal" nil (hh eh0 k s ae1 g ax n ax l))
("hexcel" nil (hh eh1 k s ax l))
("hext" nil (hh eh1 k s t))
("hey" nil (hh ey1))
("heyboer" nil (hh ey1 b ow0 er0))
("heyd" nil (hh ey1 d))
("heyday" nil (hh ey1 d ey1))
("heyde" nil (hh ey1 d))
("heyden" nil (hh ey1 d ax n))
("heydon" nil (hh ey1 d ax n))
("heydt" nil (hh ey1 d t))
("heye" nil (hh ay1))
("heyen" nil (hh ay1 n))
("heyer" nil (hh ey1 er0))
("heying" nil (hh ey1 ih0 ng))
("heyl" nil (hh ey1 l))
("heyman" nil (hh ey1 m ax n))
("heymann" nil (hh ey1 m ax n))
("heyn" nil (hh ey1 n))
("heyne" nil (hh ey1 n))
("heys" nil (hh ey1 z))
("heyse" nil (hh ey1 s))
("heyser" nil (hh ey1 z er0))
("heyward" nil (hh ey1 w er0 d))
("heywood" nil (hh ey1 w uh1 d))
("hezbollah" nil (hh eh0 z b aa1 l ax))
("hfdf" nil (ey1 ch eh1 f d iy1 eh1 f))
("hgh" nil (ey1 ch jh iy1 ey1 ch))
("hi" nil (hh ay1))
("hialeah" nil (hh ay1 ax l iy1 ax))
("hiam" nil (hh ay1 ax m))
("hiatt" nil (hh ay1 ax t))
("hiatus" nil (hh ay0 ey1 t ax s))
("hiawatha" nil (hh ay1 ax w aa1 th ax))
("hibachi" nil (hh ax b aa1 ch iy0))
("hibbard" nil (hh ih1 b er0 d))
("hibben" nil (hh ih1 b ax n))
("hibberd" nil (hh ih1 b er0 d))
("hibbert" nil (hh ih1 b er0 t))
("hibbett" nil (hh ih1 b ih0 t))
("hibbitts" nil (hh ih1 b ih0 t s))
("hibbler" nil (hh ih1 b l er0))
("hibbs" nil (hh ih1 b z))
("hibdon" nil (hh ih1 b d ax n))
("hibernate" nil (hh ay1 b er0 n ey1 t))
("hibernation" nil (hh ay1 b er0 n ey1 sh ax n))
("hibernia" nil (hh ay1 b er1 n iy0 ax))
("hibler" nil (hh ao1 b ax l er0))
("hibma" nil (hh ih1 b m ax))
("hibner" nil (hh ih1 b n er0))
("hibor" nil (hh ay1 b er0))
("hibshman" nil (hh ih1 b sh m ax n))
("hiccup" nil (hh ih1 k ax p))
("hiccups" nil (hh ih1 k ax p s))
("hice" nil (hh ay1 s))
("hichens" nil (hh ih1 k ax n z))
("hick" nil (hh ih1 k))
("hickam" nil (hh ih1 k ax m))
("hickcox" nil (hh ih1 k aa0 k s))
("hickel" nil (hh ih1 k ax l))
("hicken" nil (hh ih1 k ax n))
("hickerson" nil (hh ih1 k er0 s ax n))
("hickey" nil (hh ih1 k iy0))
("hickle" nil (hh ih1 k ax l))
("hicklin" nil (hh ih1 k l ih0 n))
("hickling" nil (hh ih1 k l ih0 ng))
("hickman" nil (hh ih1 k m ax n))
("hickmon" nil (hh ih1 k m ax n))
("hickok" nil (hh ih1 k ax k))
("hickories" nil (hh ih1 k er0 iy0 z))
("hickory" nil (hh ih1 k er0 iy0))
("hickox" nil (hh ih1 k aa0 k s))
("hicks" nil (hh ih1 k s))
("hickson" nil (hh ih1 k s ax n))
("hicksville" nil (hh ih1 k s v ih1 l))
("hid" nil (hh ih1 d))
("hidalgo" nil (hh ax d ae1 l g ow1))
("hidden" nil (hh ih1 d ax n))
("hide" nil (hh ay1 d))
("hideaki" nil (hh iy1 d ey0 aa1 k iy0))
("hideaway" nil (hh ay1 d ax w ey1))
("hidebound" nil (hh ay1 d b aw1 n d))
("hideo" nil (hh ih0 d ey1 ow0))
("hideous" nil (hh ih1 d iy0 ax s))
("hideously" nil (hh ih1 d iy0 ax s l iy0))
("hideout" nil (hh ay1 d aw1 t))
("hideouts" nil (hh ay1 d aw1 t s))
("hider" nil (hh ay1 d er0))
("hides" nil (hh ay1 d z))
("hiding" nil (hh ay1 d ih0 ng))
("hidy" nil (hh ay1 d iy0))
("hieb" nil (hh iy1 b))
("hieber" nil (hh iy1 b er0))
("hiebert" nil (hh iy1 b er0 t))
("hiegel" nil (hh iy1 g ax l))
("hiems" nil (hh iy1 m z))
("hiemstra" nil (hh iy1 m s t r ax))
("hier" nil (hh ay1 er0))
("hierarchical" nil (hh ay1 r aa1 r k ax k ax l))
("hierarchies" nil (hh ay1 r aa1 r k iy0 z))
("hierarchy" nil (hh ay1 er0 aa1 r k iy0))
("hierholzer" nil (hh ay1 r hh ow0 l z er0))
("hieroglyphic" nil (hh ay1 r ow0 g l ih1 f ih0 k))
("hieroglyphics" nil (hh ay1 r ow0 g l ih1 f ih0 k s))
("hiers" nil (hh iy1 r z))
("hiestand" nil (hh iy1 s t ax n d))
("hiester" nil (hh ay1 ih0 s t er0))
("hietala" nil (hh ay1 t ax l ax))
("hietpas" nil (hh ay1 t p ax z))
("hiett" nil (hh ay1 t))
("higa" nil (hh iy1 g ax))
("higashi" nil (hh iy0 g aa1 sh iy0))
("higbee" nil (hh ih1 g b iy1))
("higbie" nil (hh ih1 g b iy0))
("higby" nil (hh ih1 g b iy0))
("higdon" nil (hh ih1 g d ax n))
("higgason" nil (hh ih1 g ax s ax n))
("higgenbotham" nil (hh ih1 g ih0 n b ax th ax m))
("higgens" nil (hh ih1 g ax n z))
("higgerson" nil (hh ih1 g er0 s ax n))
("higginbotham" nil (hh ih0 g ih0 n b aa1 th ax m))
("higginbottom" nil (hh ih0 g ih0 n b aa1 t ax m))
("higgins" nil (hh ih1 g ih0 n z))
("higginson" nil (hh ih1 g ih0 n s ax n))
("higgs" nil (hh ih1 g z))
("high" nil (hh ay1))
("higham" nil (hh ay1 ax m))
("highberger" nil (hh ay1 b er0 g er0))
("highboy" nil (hh ay1 b oy1))
("highbrow" nil (hh ay1 b r aw1))
("highbrows" nil (hh ay1 b r aw1 z))
("higher" nil (hh ay1 er0))
("highest" nil (hh ay1 ax s t))
("highfalutin" nil (hh ay1 f ax l uw1 t ih0 n))
("highfield" nil (hh ay1 f iy1 l d))
("highfill" nil (hh ay1 f ih1 l))
("highflier" nil (hh ay1 f l ay1 er0))
("highfliers" nil (hh ay1 f l ay1 er0 z))
("highflying" nil (hh ay1 f l ay1 ih0 ng))
("highland" nil (hh ay1 l ax n d))
("highlander" nil (hh ay1 l ae1 n d er0))
("highlanders" nil (hh ay1 l ae1 n d er0 z))
("highlands" nil (hh ay1 l ax n d z))
("highley" nil (hh ay1 l iy0))
("highlight" nil (hh ay1 l ay1 t))
("highlighted" nil (hh ay1 l ay1 t ih0 d))
("highlighting" nil (hh ay1 l ay1 t ih0 ng))
("highlights" nil (hh ay1 l ay1 t s))
("highly" nil (hh ay1 l iy0))
("highman" nil (hh ay1 m ax n))
("highness" nil (hh ay1 n ax s))
("highs" nil (hh ay1 z))
("highsmith" nil (hh ay1 s m ih1 th))
("highspeed" nil (hh ay1 s p iy1 d))
("hight" nil (hh ay1 t))
("hightech" nil (hh ay1 t eh1 k))
("hightower" nil (hh ay1 t aw1 er0))
("highway" nil (hh ay1 w ey1))
("highways" nil (hh ay1 w ey1 z))
("highyield" nil (hh ay1 y iy1 l d))
("higinbotham" nil (hh ih1 g ih0 n b ax th ax m))
("higley" nil (hh ih1 g l iy0))
("higman" nil (hh ih1 g m ax n))
("hignight" nil (hh ih1 g n ay1 t))
("hignite" nil (hh ih1 g n ay1 t))
("higuchi" nil (hh iy0 g uw1 ch iy0))
("higuera" nil (hh iy0 g eh1 r ax))
("hijack" nil (hh ay1 jh ae1 k))
("hijacked" nil (hh ay1 jh ae1 k t))
("hijacker" nil (hh ay1 jh ae1 k er0))
("hijackers" nil (hh ay1 jh ae1 k er0 z))
("hijacking" nil (hh ay1 jh ae1 k ih0 ng))
("hijackings" nil (hh ay1 jh ae1 k ih0 ng z))
("hijinks" nil (hh ih1 jh ih0 ng k s))
("hike" nil (hh ay1 k))
("hiked" nil (hh ay1 k t))
("hiker" nil (hh ay1 k er0))
("hikers" nil (hh ay1 k er0 z))
("hikes" nil (hh ay1 k s))
("hiking" nil (hh ay1 k ih0 ng))
("hiland" nil (hh ih1 l ax n d))
("hilaria" nil (hh iy0 l aa1 r iy0 ax))
("hilario" nil (hh iy0 l aa1 r iy0 ow0))
("hilarious" nil (hh ih0 l eh1 r iy0 ax s))
("hilariously" nil (hh ih0 l eh1 r iy0 ax s l iy0))
("hilarity" nil (hh ih0 l eh1 r ax t iy0))
("hilary" nil (hh ih1 l er0 iy0))
("hilb" nil (hh ih1 l b))
("hilberg" nil (hh ih1 l b er0 g))
("hilbert" nil (hh ih1 l b er0 t))
("hilborn" nil (hh ih1 l b er0 n))
("hilbun" nil (hh ih1 l b ax n))
("hilburn" nil (hh ih1 l b er0 n))
("hild" nil (hh ih1 l d))
("hilda" nil (hh ih1 l d ax))
("hilde" nil (hh ih1 l d))
("hildebran" nil (hh ih1 l d ih0 b r ax n))
("hildebrand" nil (hh ih1 l d ih0 b r ae1 n d))
("hildebrandt" nil (hh ih1 l d ih0 b r ax n t))
("hildebrant" nil (hh ih1 l d ih0 b r ax n t))
("hildegard" nil (hh ih1 l d ax g aa1 r d))
("hildegarde" nil (hh ih1 l d ih0 g aa1 r d))
("hildegardes" nil (hh ih1 l d ax g aa1 r d z))
("hildemar" nil (hh ih1 l d ih0 m er0))
("hilden" nil (hh ay1 l d ax n))
("hildenbrand" nil (hh ih1 l d ih0 n b r ax n d))
("hilder" nil (hh ih1 l d er0))
("hilderbrand" nil (hh ih1 l d er0 b r ax n d))
("hilderbrandt" nil (hh ih1 l d er0 b r ax n t))
("hildie" nil (hh ay1 l d iy0))
("hilditch" nil (hh ih1 l d ih0 ch))
("hildreth" nil (hh ih1 l d r ih0 th))
("hildy" nil (hh ih1 l d iy0))
("hile" nil (hh ay1 l))
("hileman" nil (hh ay1 l m ax n))
("hilemon" nil (hh ay1 l m ax n))
("hiler" nil (hh ay1 l er0))
("hiles" nil (hh ay1 l z))
("hiley" nil (hh ih1 l iy0))
("hilfiger" nil (hh ih1 l f ay0 g er0))
("hilfiker" nil (hh ih1 l f ih0 k er0))
("hilgart" nil (hh ih1 l g aa1 r t))
("hilgeman" nil (hh ih1 l g m ax n))
("hilgenberg" nil (hh ih1 l g ax n b er0 g))
("hilgendorf" nil (hh ih1 l g ih0 n d ao0 r f))
("hilger" nil (hh ih1 l g er0))
("hilgers" nil (hh ih1 l g er0 z))
("hilgert" nil (hh ih1 l g er0 t))
("hilinski" nil (hh ih0 l ih1 n s k iy0))
("hilke" nil (hh ih1 l k ax))
("hilker" nil (hh ih1 l k er0))
("hill" nil (hh ih1 l))
("hilla" nil (hh ih1 l ax))
("hillard" nil (hh ih1 l er0 d))
("hillary" nil (hh ih1 l er0 iy0))
("hillas" nil (hh ih1 l ax s))
("hillbillies" nil (hh ih1 l b ih1 l iy0 z))
("hillbilly" nil (hh ih1 l b ih0 l iy0))
("hille" nil (hh ih1 l))
("hilleary" nil (hh ih1 l er0 iy0))
("hillebrand" nil (hh ih1 l ih0 b r ax n d))
("hillegas" nil (hh ih1 l ih0 g ax z))
("hillegass" nil (hh ih1 l ih0 g ax s))
("hillel" nil (hh ih1 l eh1 l))
("hillen" nil (hh ih1 l ax n))
("hillenbrand" nil (hh ih1 l ax n b r ae1 n d))
("hillenburg" nil (hh ih1 l ax n b er0 g))
("hiller" nil (hh ih1 l er0))
("hillery" nil (hh ih1 l er0 iy0))
("hillesheim" nil (hh ih1 l ih0 s hh ay0 m))
("hillestad" nil (hh ih1 l ih0 s t ax d))
("hilley" nil (hh ih1 l iy0))
("hillhaven" nil (hh ih1 l hh ey1 v ax n))
("hillhouse" nil (hh ih1 l hh aw1 s))
("hilliard" nil (hh ih1 l y aa0 r d))
("hillier" nil (hh ih1 l iy0 er0))
("hilligoss" nil (hh ih1 l ih0 g aa0 s))
("hilliker" nil (hh ih1 l ay0 k er0))
("hillin" nil (hh ih1 l ih0 n))
("hilling" nil (hh ih1 l ih0 ng))
("hillis" nil (hh ih1 l ih0 s))
("hillman" nil (hh ih1 l m ae1 n))
("hillmann" nil (hh ih1 l m ax n))
("hillmer" nil (hh ih1 l m er0))
("hillock" nil (hh ih1 l ax k))
("hills" nil (hh ih1 l z))
("hillsboro" nil (hh ih1 l z b er0 ow0))
("hillsborough" nil (hh ih1 l z b er0 ow0))
("hillsdale" nil (hh ih1 l z d ey1 l))
("hillsdown" nil (hh ih1 l z d aw1 n))
("hillside" nil (hh ih1 l s ay1 d))
("hillsides" nil (hh ih1 l s ay1 d z))
("hillsman" nil (hh ih1 l s m ax n))
("hillson" nil (hh ih1 l s ax n))
("hillstrom" nil (hh ih1 l s t r ax m))
("hilltop" nil (hh ih1 l t aa1 p))
("hilly" nil (hh ih1 l iy0))
("hillyard" nil (hh ih1 l y aa1 r d))
("hillyer" nil (hh ih1 l iy0 er0))
("hilma" nil (hh ih1 l m ax))
("hilmer" nil (hh ih1 l m er0))
("hilmes" nil (hh ih1 l m z))
("hilo" nil (hh iy1 l ow0))
("hilpert" nil (hh ih1 l p er0 t))
("hilsabeck" nil (hh ih1 l s ax b eh1 k))
("hilscher" nil (hh ih1 l sh er0))
("hilsinger" nil (hh ih1 l s ih0 ng er0))
("hilsman" nil (hh ih1 l z m ax n))
("hilson" nil (hh ih1 l s ax n))
("hilt" nil (hh ih1 l t))
("hiltner" nil (hh ih1 l t n er0))
("hilton" nil (hh ih1 l t ax n))
("hilts" nil (hh ih1 l t s))
("hiltunen" nil (hh ih1 l t ax n ax n))
("hilty" nil (hh ih1 l t iy0))
("hiltz" nil (hh ih1 l t s))
("hilyard" nil (hh ax l y aa1 r d))
("hilyer" nil (hh ih1 l iy0 er0))
("him" nil (hh ih1 m))
("himalayan" nil (hh ih1 m ax l ey1 ax n))
("himalayas" nil (hh ih1 m ax l ay1 ax s))
("hime" nil (hh ay1 m))
("himebaugh" nil (hh ih1 m ih0 b ao0))
("himel" nil (hh ih1 m ax l))
("himes" nil (hh ay1 m z))
("himmel" nil (hh ih1 m ax l))
("himmelberger" nil (hh ih1 m ax l b er0 g er0))
("himmelfarb" nil (hh ih1 m ax l f aa1 r b))
("himmelsbach" nil (hh ih1 m ih0 l s b aa0 k))
("himmelstein" nil (hh ih1 m ax l s t ay0 n))
("himont" nil (hh ih1 m ax n t))
("himself" nil (hh ih0 m s eh1 l f))
("hinch" nil (hh ih1 n ch))
("hinchcliff" nil (hh ih1 n ch k l ih0 f))
("hinchcliffe" nil (hh ih1 n ch k l ih0 f))
("hinchey" nil (hh ih1 n ch iy0))
("hinchliffe" nil (hh ih1 n k l ih0 f))
("hinchman" nil (hh ih1 ng k m ax n))
("hinck" nil (hh ih1 ng k))
("hinckley" nil (hh ih1 ng k l iy0))
("hind" nil (hh ay1 n d))
("hinde" nil (hh ih1 n d))
("hindelong" nil (hh ih1 n d ax l ao0 ng))
("hinder" nil (hh ih1 n d er0))
("hindered" nil (hh ih1 n d er0 d))
("hinderer" nil (hh ih1 n d er0 er0))
("hindering" nil (hh ih1 n d er0 ih0 ng))
("hinderliter" nil (hh ih1 n d er0 l iy0 t er0))
("hinderman" nil (hh ay1 n d er0 m ax n))
("hinders" nil (hh ih1 n d er0 z))
("hindes" nil (hh ih1 n d z))
("hindi" nil (hh ih1 n d iy0))
("hindle" nil (hh ih1 n d ax l))
("hindley" nil (hh ih1 n d l iy0))
("hindman" nil (hh ay1 n d m ax n))
("hindquarters" nil (hh ay1 n d k w ao1 r t er0 z))
("hindrance" nil (hh ih1 n d r ax n s))
("hindrances" nil (hh ih1 n d r ax n s ih0 z))
("hinds" nil (hh ay1 n d z))
("hindsight" nil (hh ay1 n d s ay1 t))
("hindu" nil (hh ih1 n d uw1))
("hinduism" nil (hh ih1 n jh uw0 ih1 z ax m))
("hindus" nil (hh ih1 n d uw1 z))
("hindustan" nil (hh ih1 n d uw0 s t ae1 n))
("hine" nil (hh ay1 n))
("hinebaugh" nil (hh ih1 n ih0 b ao0))
("hineline" nil (hh ih1 n ih0 l ay1 n))
("hinely" nil (hh ay1 n l iy0))
("hineman" nil (hh ay1 n m ax n))
("hiner" nil (hh ay1 n er0))
("hinerman" nil (hh ay1 n er0 m ax n))
("hines" nil (hh ay1 n z))
("hinesley" nil (hh ih1 n ih0 s l iy0))
("hiney" nil (hh ih1 n iy0))
("hing" nil (hh ih1 ng))
("hinge" nil (hh ih1 n jh))
("hinged" nil (hh ih1 n jh d))
("hinger" nil (hh ih1 n jh er0))
("hinges" nil (hh ih1 n jh ih0 z))
("hingham" nil (hh ih1 ng ax m))
("hingle" nil (hh ih1 ng g ax l))
("hingst" nil (hh ih1 ng s t))
("hingston" nil (hh ih1 ng s t ax n))
("hink" nil (hh ih1 ng k))
("hinkel" nil (hh ih1 ng k ax l))
("hinkelman" nil (hh ih1 ng k ax l m ax n))
("hinkle" nil (hh ih1 ng k ax l))
("hinkley" nil (hh ih1 ng k l iy0))
("hinkson" nil (hh ih1 ng k s ax n))
("hinman" nil (hh ih1 n m ax n))
("hinnant" nil (hh ih1 n ax n t))
("hinnenkamp" nil (hh ih1 n ih0 n k ae0 m p))
("hinners" nil (hh ih1 n er0 z))
("hinny" nil (hh ih1 n iy0))
("hino" nil (hh iy1 n ow0))
("hinojos" nil (hh iy0 n ow1 y ow0 z))
("hinojosa" nil (hh iy0 n ow0 jh ow1 s ax))
("hinote" nil (hh ih0 n ow1 t))
("hinrichs" nil (hh ih1 n r ih0 k s))
("hinrichsen" nil (hh ih1 n r ih0 k s ax n))
("hinsch" nil (hh ih1 n sh))
("hinsdale" nil (hh ih1 n s d ey1 l))
("hinshaw" nil (hh ih1 n sh ao1))
("hinsley" nil (hh ih1 n s l iy0))
("hinson" nil (hh ih1 n s ax n))
("hint" nil (hh ih1 n t))
("hinted" nil (hh ih1 n t ax d))
("hinterland" nil (hh ih1 n t er0 l ae1 n d))
("hinterlands" nil (hh ih1 n t er0 l ae1 n d z))
("hinting" nil (hh ih1 n t ih0 ng))
("hinton" nil (hh ih1 n t ax n))
("hints" nil (hh ih1 n t s))
("hintz" nil (hh ih1 n t s))
("hintze" nil (hh ih1 n t z))
("hinz" nil (hh ih1 n z))
("hinzack" nil (hh ih1 n z ae0 k))
("hinze" nil (hh ih1 n z))
("hinzman" nil (hh ih1 n z m ax n))
("hiott" nil (hh ay1 ax t))
("hip" nil (hh ih1 p))
("hipbones" nil (hh ih1 p b ow1 n z))
("hipkins" nil (hh ih1 p k ih0 n z))
("hipolito" nil (iy1 p ow0 l iy1 t ow0))
("hipp" nil (hh ih1 p))
("hippe" nil (hh ih1 p))
("hippen" nil (hh ih1 p ax n))
("hippensteel" nil (hh ih1 p ih0 n s t iy0 l))
("hipper" nil (hh ih1 p er0))
("hippert" nil (hh ih1 p er0 t))
("hippie" nil (hh ih1 p iy0))
("hippies" nil (hh ih1 p iy0 z))
("hipple" nil (hh ih1 p ax l))
("hippler" nil (hh ih1 p l er0))
("hippo" nil (hh ih1 p ow0))
("hippocrates" nil (hh ih1 p ax k r ey1 t s))
("hippodrome" nil (hh ih1 p ax d r ow1 m))
("hippolytus" nil (hh ax p aa1 l ax t ax s))
("hippopotamus" nil (hh ih1 p ax p aa1 t ax m ax s))
("hipps" nil (hh ih1 p s))
("hips" nil (hh ih1 p s))
("hipsher" nil (hh ih1 p sh er0))
("hipskind" nil (hh ih1 p s k ih0 n d))
("hipwell" nil (hh ih1 p w eh1 l))
("hirai" nil (hh ih0 r aa1 iy0))
("hiram" nil (hh ay1 r ax m))
("hirano" nil (hh ih0 r aa1 n ow0))
("hirata" nil (hh ih0 r aa1 t ax))
("hirayama" nil (hh ih0 r aa0 y aa1 m ax))
("hird" nil (hh er1 d))
("hire" nil (hh ay1 er0))
("hired" nil (hh ay1 er0 d))
("hires" nil (hh ay1 er0 z))
("hiriart" nil (hh ih1 r iy0 aa0 r t))
("hiring" nil (hh ay1 r ih0 ng))
("hirings" nil (hh ay1 r ih0 ng z))
("hirn" nil (hh er1 n))
("hiro" nil (hh ih1 r ow0))
("hiroaki" nil (hh ih1 r ow0 aa1 k iy0))
("hirohito" nil (hh ih1 r ow0 hh iy1 t ow1))
("hiromasa" nil (hh ih1 r ow0 m aa1 s ax))
("hirons" nil (hh ao1 r ax n z))
("hirose" nil (hh ih0 r ow1 s ey0))
("hiroshi" nil (hh ih0 r ow1 sh iy0))
("hiroshima" nil (hh ih1 r ow0 sh iy1 m ax))
("hirota" nil (hh ih0 r ow1 t ax))
("hiroyuki" nil (hh ih1 r ow0 y uw1 k iy0))
("hirsch" nil (hh er1 sh))
("hirschberg" nil (hh er1 sh b er0 g))
("hirschfeld" nil (hh er1 sh f eh0 l d))
("hirschfield" nil (hh er1 s k f iy0 l d))
("hirschhorn" nil (hh er1 sh hh er0 n))
("hirschi" nil (hh ih1 r s k iy0))
("hirschman" nil (hh er1 sh m ax n))
("hirschmann" nil (hh er1 sh m ax n))
("hirschy" nil (hh er1 sh iy0))
("hirsh" nil (hh er1 sh))
("hirshberg" nil (hh er1 sh b er0 g))
("hirshfield" nil (hh er1 sh f iy0 l d))
("hirshhorn" nil (hh er1 sh hh ao1 r n))
("hirshman" nil (hh er1 sh m ax n))
("hirst" nil (hh er1 s t))
("hirt" nil (hh er1 t))
("hirth" nil (hh er1 th))
("hirtle" nil (hh er1 t ax l))
("hirtz" nil (hh er1 t s))
("hirulog" nil (hh ih1 r uw1 l aa1 g))
("hirzel" nil (hh er1 z ax l))
("his" nil (hh ih1 z))
("hisao" nil (hh ih0 s aa1 ow0))
("hisaw" nil (hh ay1 s ao0))
("hiscock" nil (hh ih1 s k ax k))
("hiscox" nil (hh ih1 s k aa0 k s))
("hise" nil (hh ay1 z))
("hisel" nil (hh ih1 s ax l))
("hiser" nil (hh ay1 z er0))
("hisey" nil (hh ih1 s iy0))
("hisham" nil (hh ih1 sh ax m))
("hisle" nil (hh ay1 ax l))
("hislop" nil (hh ih1 s l ax p))
("hispanic" nil (hh ih0 s p ae1 n ih0 k))
("hispanics" nil (hh ih0 s p ae1 n ih0 k s))
("hispano" nil (hh ih0 s p aa1 n ow0))
("hispanoil" nil (hh ih1 s p ax n oy1 l))
("hiss" nil (hh ih1 s))
("hissed" nil (hh ih1 s t))
("hisses" nil (hh ih1 s ih0 z))
("hissing" nil (hh ih1 s ih0 ng))
("hissong" nil (hh ih1 s ao1 ng))
("histadrut" nil (hh ih1 s t ax d r ax t))
("histamine" nil (hh ih1 s t ax m iy1 n))
("histidine" nil (hh ih1 s t ax d iy1 n))
("histogram" nil (hh ih1 s t ax g r ae1 m))
("histology" nil (hh ih0 s t aa1 l ax jh iy0))
("historian" nil (hh ih0 s t ao1 r iy0 ax n))
("historians" nil (hh ih0 s t ao1 r iy0 ax n z))
("historic" nil (hh ih0 s t ao1 r ih0 k))
("historical" nil (hh ih0 s t ao1 r ih0 k ax l))
("historically" nil (hh ih0 s t ao1 r ih0 k ax l iy0))
("histories" nil (hh ih1 s t er0 iy0 z))
("historiography" nil (hh ih0 s t ao1 r iy0 aa1 g r ax f iy0))
("history" nil (hh ih1 s t er0 iy0))
("histrionic" nil (hh ih1 s t r iy0 aa1 n ih0 k))
("histrionics" nil (hh ih1 s t r iy0 aa1 n ih0 k s))
("hit" nil (hh ih1 t))
("hitachi" nil (hh ih0 t aa1 ch iy0))
("hitch" nil (hh ih1 ch))
("hitchcock" nil (hh ih1 ch k aa1 k))
("hitched" nil (hh ih1 ch t))
("hitchens" nil (hh ih1 ch ax n z))
("hitches" nil (hh ih1 ch ih0 z))
("hitching" nil (hh ih1 ch ih0 ng))
("hitchings" nil (hh ih1 ch ih0 ng z))
("hitchins" nil (hh ih1 ch ih0 n z))
("hitchman" nil (hh ih1 ch m ax n))
("hitchner" nil (hh ih1 ch n er0))
("hite" nil (hh ay1 t))
("hites" nil (hh ay1 t s))
("hither" nil (hh ih1 dh er0))
("hitherto" nil (hh ih1 dh er1 t uw1))
("hitler" nil (hh ih1 t l er0))
("hitoshi" nil (hh ih0 t ow1 sh iy0))
("hits" nil (hh ih1 t s))
("hitschler" nil (hh ih1 t s l er0))
("hitsman" nil (hh ih1 t s m ax n))
("hitson" nil (hh ih1 t s ax n))
("hitt" nil (hh ih1 t))
("hitter" nil (hh ih1 t er0))
("hitters" nil (hh ih1 t er0 z))
("hitting" nil (hh ih1 t ih0 ng))
("hittite" nil (hh ih1 t ay0 t))
("hittle" nil (hh ih1 t ax l))
("hittner" nil (hh ih1 t n er0))
("hitty" nil (hh ih1 t iy0))
("hitz" nil (hh ih1 t s))
("hitzeman" nil (hh ih1 t s m ax n))
("hive" nil (hh ay1 v))
("hively" nil (hh ay1 v l iy0))
("hives" nil (hh ay1 v z))
("hiway" nil (hh ay1 w ey1))
("hix" nil (hh ih1 k s))
("hixenbaugh" nil (hh ih0 g z eh1 n b ao0))
("hixon" nil (hh ih1 k s ax n))
("hixson" nil (hh ih1 k s ax n))
("hizer" nil (hh ay1 z er0))
("hjelm" nil (hh ax jh eh1 l m))
("hjerpe" nil (hh ax jh er1 p))
("hjort" nil (hh ax jh ao1 r t))
("hlad" nil (hh l ae1 d))
("hladik" nil (hh l ae1 d ih0 k))
("hladky" nil (hh l ae1 d k iy0))
("hlavac" nil (hh l aa1 v ax k))
("hlavacek" nil (hh l aa1 v ax ch eh0 k))
("hlavaty" nil (hh l ax v aa1 t iy0))
("hmmm" nil (hh m))
("hmong" nil (m ao1 ng))
("hnat" nil (hh n ae1 t))
("ho" nil (hh ow1))
("hoadley" nil (hh ow1 d l iy0))
("hoag" nil (hh ow1 g))
("hoagie" nil (hh ow1 g iy0))
("hoagland" nil (hh ow1 g l ax n d))
("hoaglin" nil (hh ow1 g l ih0 n))
("hoaglund" nil (hh ow1 g l ax n d))
("hoagy" nil (hh ow1 g iy0))
("hoak" nil (hh ow1 k))
("hoang" nil (hh ow1 ng))
("hoar" nil (hh ao1 r))
("hoard" nil (hh ao1 r d))
("hoarded" nil (hh ao1 r d ih0 d))
("hoarding" nil (hh ao1 r d ih0 ng))
("hoards" nil (hh ao1 r d z))
("hoare" nil (hh ao1 r))
("hoarse" nil (hh ao1 r s))
("hoarseness" nil (hh ao1 r s n ax s))
("hoary" nil (hh ao1 r iy0))
("hoax" nil (hh ow1 k s))
("hoaxes" nil (hh ow1 k s ih0 z))
("hob" nil (hh aa1 b))
("hoback" nil (hh ow1 b ae1 k))
("hoban" nil (hh ow1 b ax n))
("hobart" nil (hh ow1 b aa1 r t))
("hobaugh" nil (hh aa1 b ao0))
("hobbes" nil (hh aa1 b iy0 z))
("hobbie" nil (hh aa1 b iy0))
("hobbies" nil (hh aa1 b iy0 z))
("hobbing" nil (hh aa1 b ih0 ng))
("hobbins" nil (hh aa1 b ih0 n z))
("hobble" nil (hh aa1 b ax l))
("hobbled" nil (hh aa1 b ax l d))
("hobbles" nil (hh aa1 b ax l z))
("hobbling" nil (hh aa1 b ax l ih0 ng))
("hobbs" nil (hh aa1 b z))
("hobby" nil (hh aa1 b iy0))
("hobbyist" nil (hh aa1 b iy0 ih0 s t))
("hobbyists" nil (hh aa1 b iy0 ih0 s t s))
("hobday" nil (hh aa1 b d ey1))
("hobdy" nil (hh aa1 b d iy0))
("hoben" nil (hh aa1 b ax n))
("hoberg" nil (hh ow1 b er0 g))
("hoberman" nil (hh ow1 b er0 m ax n))
("hobert" nil (hh aa1 b er0 t))
("hobgood" nil (hh aa1 b g uh1 d))
("hobin" nil (hh ow1 b ih0 n))
("hoblit" nil (hh aa1 b l ih0 t))
("hobnail" nil (hh aa1 b n ey1 l))
("hobnob" nil (hh aa1 b n aa1 b))
("hobnobbing" nil (hh aa1 b n aa1 b ih0 ng))
("hoboken" nil (hh ow1 b ow0 k ax n))
("hobs" nil (hh aa1 b z))
("hobsbawm" nil (hh aa0 b s b ao1 m))
("hobson" nil (hh aa1 b s ax n))
("hoc" nil (hh aa1 k))
("hocevar" nil (hh ow0 s ey0 v aa1 r))
("hoch" nil (hh aa1 k))
("hochberg" nil (hh aa1 k b er0 g))
("hochhalter" nil (hh aa1 k hh ax l t er0))
("hochhauser" nil (hh aa1 k hh aw1 z er0))
("hochman" nil (hh aa1 k m ax n))
("hochmuth" nil (hh aa1 k m uw1 th))
("hochstatter" nil (hh aa1 k s t ax t er0))
("hochstedler" nil (hh aa1 k s t ih0 d ax l er0))
("hochstein" nil (hh aa1 k s t ay0 n))
("hochstetler" nil (hh aa1 k s t ih0 t ax l er0))
("hochtief" nil (hh aa1 k t iy1 f))
("hock" nil (hh aa1 k))
("hockaday" nil (hh aa1 k ax d ey1))
("hockenberry" nil (hh aa1 k ax n b eh1 r iy0))
("hockenbury" nil (hh aa1 k ax n b eh1 r iy0))
("hockensmith" nil (hh aa1 k ax n s m ih1 th))
("hocker" nil (hh aa1 k er0))
("hockersmith" nil (hh aa1 k er0 s m ih1 th))
("hockett" nil (hh aa1 k ih0 t))
("hockey" nil (hh aa1 k iy0))
("hockin" nil (hh aa1 k ih0 n))
("hocking" nil (hh aa1 k ih0 ng))
("hockley" nil (hh aa1 k l iy0))
("hockman" nil (hh aa1 k m ax n))
("hockney" nil (hh aa1 k n iy0))
("hoctor" nil (hh aa1 k t er0))
("hocus" nil (hh ow1 k ax s))
("hocutt" nil (hh aa1 k ax t))
("hodak" nil (hh ow1 d ax k))
("hodapp" nil (hh aa1 d ax p))
("hodde" nil (hh aa1 d))
("hodder" nil (hh aa1 d er0))
("hodding" nil (hh aa1 d ih0 ng))
("hodel" nil (hh ow1 d ax l))
("hodes" nil (hh ow1 d z))
("hodgdon" nil (hh aa1 jh d ax n))
("hodge" nil (hh aa1 jh))
("hodgen" nil (hh aa1 jh ax n))
("hodgens" nil (hh aa1 jh ax n z))
("hodgepodge" nil (hh aa1 jh p aa1 jh))
("hodges" nil (hh aa1 jh ih0 z))
("hodgin" nil (hh aa1 jh ih0 n))
("hodgins" nil (hh aa1 jh ih0 n z))
("hodgkin" nil (hh aa1 jh k ih0 n))
("hodgkins" nil (hh aa1 jh k ih0 n z))
("hodgkinson" nil (hh aa1 jh k ih0 n s ax n))
("hodgkiss" nil (hh aa1 jh k ih0 s))
("hodgman" nil (hh aa1 jh m ax n))
("hodgson" nil (hh aa1 jh s ax n))
("hodkinson" nil (hh aa1 d k ih0 n s ax n))
("hodnett" nil (hh aa1 d n ih0 t))
("hodo" nil (hh ow1 d ow0))
("hodsdon" nil (hh aa1 d z d ax n))
("hodson" nil (hh aa1 d s ax n))
("hoe" nil (hh ow1))
("hoechst" nil (hh ow1 k s t))
("hoeck" nil (hh ow1 k))
("hoecker" nil (hh ow1 k er0))
("hoedown" nil (hh ow1 d aw1 n))
("hoefer" nil (hh ow1 f er0))
("hoeffner" nil (hh ow1 f n er0))
("hoefle" nil (hh ow1 f ax l))
("hoefler" nil (hh ow1 f ax l er0))
("hoeflich" nil (hh ow1 f l ih0 k))
("hoefling" nil (hh ow1 f ax l ih0 ng))
("hoefs" nil (hh ow1 f s))
("hoeft" nil (hh ow1 f t))
("hoeg" nil (hh ow1 g))
("hoeger" nil (hh ow1 g er0))
("hoehn" nil (hh ow1 n))
("hoehne" nil (hh ow1 n))
("hoeing" nil (hh ow1 ih0 ng))
("hoek" nil (hh ow1 k))
("hoeksema" nil (hh ow1 k s ih0 m ax))
("hoekstra" nil (hh ow1 k s t r ax))
("hoel" nil (hh ow1 l))
("hoell" nil (hh ow1 l))
("hoelle" nil (hh ow1 l))
("hoelscher" nil (hh ow1 l sh er0))
("hoelterhoff" nil (hh ow1 l t er0 hh ao1 f))
("hoelting" nil (hh ow1 l t ih0 ng))
("hoelzel" nil (hh ow1 l z ax l))
("hoelzer" nil (hh ow1 l z er0))
("hoen" nil (hh ow1 n))
("hoene" nil (hh aa1 iy0 n))
("hoener" nil (hh ow1 n er0))
("hoenig" nil (hh ow1 n ih0 g))
("hoeper" nil (hh ow1 p er0))
("hoepfner" nil (hh ow1 p f n er0))
("hoepner" nil (hh ow1 p n er0))
("hoeppner" nil (hh ow1 p n er0))
("hoerig" nil (hh ao1 r ih0 g))
("hoerner" nil (hh ao1 r n er0))
("hoerr" nil (hh ao1 r))
("hoes" nil (hh ow1 z))
("hoesch" nil (hh ow1 sh))
("hoeschen" nil (hh ow1 sh ax n))
("hoesly" nil (hh ow1 s l iy0))
("hoeveler" nil (hh ow1 v l er0))
("hoevet" nil (hh ow1 v eh1 t))
("hoey" nil (hh ow1 iy0))
("hof" nil (hh aa1 f))
("hofacker" nil (hh aa1 f ax k er0))
("hofbauer" nil (hh aa1 f b aw0 er0))
("hofer" nil (hh ow1 f er0))
("hoff" nil (hh ao1 f))
("hoffa" nil (hh ao1 f ax))
("hoffacker" nil (hh ao1 f ax k er0))
("hoffart" nil (hh ao1 f aa0 r t))
("hoffarth" nil (hh ao1 f aa0 r th))
("hoffecker" nil (hh ao1 f ih0 k er0))
("hoffenberg" nil (hh ao1 f ax n b er0 g))
("hoffer" nil (hh aa1 f er0))
("hofferber" nil (hh ao1 f er0 b er0))
("hoffert" nil (hh ao1 f er0 t))
("hoffler" nil (hh ao1 f ax l er0))
("hoffman" nil (hh ao1 f m ax n))
("hoffmann" nil (hh ao1 f m ax n))
("hoffmaster" nil (hh ao1 f m ae1 s t er0))
("hoffmeier" nil (hh ao1 f m ay0 er0))
("hoffmeister" nil (hh ao1 f m ay1 s t er0))
("hoffmeyer" nil (hh ao1 f m ay0 er0))
("hoffner" nil (hh ao1 f n er0))
("hoffpauir" nil (hh ao1 f p aw0 er0))
("hofi" nil (hh ow1 f iy0))
("hofland" nil (hh aa1 f l ax n d))
("hofler" nil (hh aa1 f l er0))
("hofman" nil (hh aa1 f m ax n))
("hofmann" nil (hh aa1 f m ax n))
("hofmeister" nil (hh aa1 f m ay0 s t er0))
("hofrichter" nil (hh aa1 f r ih0 k t er0))
("hofstad" nil (hh aa1 f s t ae0 d))
("hofstetter" nil (hh aa1 f s t eh0 t er0))
("hofstra" nil (hh aa1 f s t r ax))
("hog" nil (hh aa1 g))
("hogan" nil (hh ow1 g aa1 n))
("hogans" nil (hh ow1 g aa1 n z))
("hoganson" nil (hh aa1 g ax n s ax n))
("hogarth" nil (hh ow1 g aa1 r th))
("hogarty" nil (hh aa1 g aa1 r t iy0))
("hogberg" nil (hh aa1 g b er0 g))
("hoge" nil (hh ow1 jh))
("hogeland" nil (hh aa1 g ih0 l ax n d))
("hogen" nil (hh aa1 g ax n))
("hogenson" nil (hh aa1 jh ih0 n s ax n))
("hoger" nil (hh ow1 g er0))
("hogg" nil (hh aa1 g))
("hoggan" nil (hh aa1 g ax n))
("hoggard" nil (hh aa1 g er0 d))
("hoggatt" nil (hh aa1 g ax t))
("hogge" nil (hh aa1 g))
("hogland" nil (hh aa1 g l ax n d))
("hogle" nil (hh ow1 g ax l))
("hoglund" nil (hh ao1 g l ax n d))
("hoglunds" nil (hh ao1 g l ax n d z))
("hognose" nil (hh aa1 g n ow1 z))
("hogrefe" nil (hh aa1 g r ih0 f))
("hogs" nil (hh aa1 g z))
("hogsed" nil (hh ow1 g s t))
("hogsett" nil (hh aa1 g s ih0 t))
("hogston" nil (hh aa1 g s t ax n))
("hogue" nil (hh ow1 g))
("hogwash" nil (hh aa1 g w aa1 sh))
("hogwood" nil (hh ao1 g w uh1 d))
("hoh" nil (hh ow1))
("hoheisel" nil (hh ow1 ay0 s ax l))
("hohenberger" nil (hh ow1 ax n b er0 g er0))
("hohensee" nil (hh aa0 hh ih0 n s iy1))
("hohenstein" nil (hh ow1 ax n s t ay0 n))
("hohl" nil (hh ow1 l))
("hohler" nil (hh ow1 l er0))
("hohlt" nil (hh ow1 l t))
("hohman" nil (hh ow1 m ax n))
("hohmann" nil (hh ow1 m ax n))
("hohn" nil (hh aa1 n))
("hohner" nil (hh ow1 n er0))
("hohnstein" nil (hh ow1 n s t ay0 n))
("hohorst" nil (hh ow1 hh ao1 r s t))
("hoi" nil (hh oy1))
("hoiland" nil (hh oy0 l ae1 n d))
("hoilman" nil (hh oy1 l m ax n))
("hoisington" nil (hh oy1 z ih0 ng t ax n))
("hoist" nil (hh oy1 s t))
("hoisted" nil (hh oy1 s t ax d))
("hoisting" nil (hh oy1 s t ih0 ng))
("hoists" nil (hh oy1 s t s))
("hoit" nil (hh oy1 t))
("hoium" nil (hh aa1 iy0 ax m))
("hojnacki" nil (hh ax y n aa1 t s k iy0))
("hokanson" nil (hh aa1 k ax n s ax n))
("hoke" nil (hh ow1 k))
("hokenson" nil (hh aa1 k ih0 n s ax n))
("hokey" nil (hh ow1 k iy0))
("hokkaido" nil (hh ow0 k ay1 d ow0))
("hokum" nil (hh ow1 k ax m))
("hokuriku" nil (hh aa1 k er0 iy1 k uw1))
("holaday" nil (hh ow1 l ax d ey0))
("holahan" nil (hh aa1 l ax hh ae0 n))
("holan" nil (hh ow1 l ax n))
("holaway" nil (hh ow1 l ax w ey0))
("holbein" nil (hh ow1 l b ay0 n))
("holben" nil (hh ow1 l b ax n))
("holberg" nil (hh ow1 l b er0 g))
("holbert" nil (hh ow1 l b er0 t))
("holbrook" nil (hh ow1 l b r uh1 k))
("holbrooke" nil (hh ow1 l b r uh0 k))
("holbrooks" nil (hh ow1 l b r uh0 k s))
("holck" nil (hh ow1 l k))
("holcomb" nil (hh ow1 l k ax m))
("holcombe" nil (hh ow1 l k ax m))
("hold" nil (hh ow1 l d))
("holda" nil (hh ow1 l d ax))
("holdaway" nil (hh ow1 l d ax w ey1))
("holde" nil (hh ow1 l d))
("holdeman" nil (hh ow1 l d m ax n))
("holden" nil (hh ow1 l d ax n))
("holder" nil (hh ow1 l d er0))
("holderbank" nil (hh ow1 l d er0 b ae1 ng k))
("holderbaum" nil (hh ow1 l d er0 b aw0 m))
("holderby" nil (hh ow1 l d er0 b iy0))
("holderfield" nil (hh ow1 l d er0 f iy1 l d))
("holderman" nil (hh ow1 l d er0 m ax n))
("holderness" nil (hh ow1 l d er0 n ax s))
("holders" nil (hh ow1 l d er0 z))
("holdfast" nil (hh ow1 l f ae1 s t))
("holding" nil (hh ow1 l d ih0 ng))
("holdings" nil (hh ow1 l d ih0 ng z))
("holdman" nil (hh ow1 l d m ax n))
("holdorf" nil (hh ow1 l d ao0 r f))
("holdout" nil (hh ow1 l d aw1 t))
("holdouts" nil (hh ow1 l d aw1 t s))
("holdover" nil (hh ow1 l d ow1 v er0))
("holdovers" nil (hh ow1 l d ow1 v er0 z))
("holdren" nil (hh ow1 l d er0 ax n))
("holdridge" nil (hh ow1 l d r ih0 jh))
("holds" nil (hh ow1 l d z))
("holdsworth" nil (hh ow1 l d z w er1 th))
("holdup" nil (hh ow1 l d ah1 p))
("holdups" nil (hh ow1 l d ah1 p s))
("hole" nil (hh ow1 l))
("holecek" nil (hh aa1 l ih0 ch eh0 k))
("holed" nil (hh ow1 l d))
("holeman" nil (hh ow1 l m ax n))
("holen" nil (hh ow1 l ax n))
("holes" nil (hh ow1 l z))
("holewinski" nil (hh ax l uw0 ih1 n s k iy0))
("holford" nil (hh ow1 l f er0 d))
("holgate" nil (hh ow1 l g ey1 t))
("holgerson" nil (hh ow1 l g er0 s ax n))
("holguin" nil (hh ow1 l g ih0 n))
("holian" nil (hh ow1 l iy0 ax n))
("holick" nil (hh aa1 l ih0 k))
("holiday" nil (hh aa1 l ax d ey1))
("holidays" nil (hh aa1 l ax d ey1 z))
("holien" nil (hh ow1 l iy0 ax n))
("holier" nil (hh ow1 l iy0 er0))
("holiest" nil (hh ow1 l iy0 ih0 s t))
("holifield" nil (hh aa1 l ih0 f iy1 l d))
("holihan" nil (hh aa1 l ih0 hh ae0 n))
("holik" nil (hh ow1 l ih0 k))
("holiman" nil (hh aa1 l ih0 m ax n))
("holiness" nil (hh ow1 l iy0 n ax s))
("holistic" nil (hh ow0 l ih1 s t ih0 k))
("holl" nil (hh aa1 l))
("hollabaugh" nil (hh aa1 l ax b ao1))
("holladay" nil (hh aa1 l ax d ey1))
("hollan" nil (hh aa1 l ax n))
("holland" nil (hh aa1 l ax n d))
("hollander" nil (hh aa1 l ax n d er0))
("hollanders" nil (hh aa1 l ax n d er0 z))
("hollands" nil (hh aa1 l ax n d z))
("hollandsworth" nil (hh aa1 l ax n d z w er1 th))
("hollar" nil (hh aa1 l er0))
("hollars" nil (hh aa1 l er0 z))
("hollatz" nil (hh aa1 l ax t s))
("hollaway" nil (hh aa1 l ax w ey0))
("holle" nil (hh aa1 l))
("holleman" nil (hh ow1 l m ax n))
("hollen" nil (hh aa1 l ax n))
("hollenbach" nil (hh aa1 l ih0 n b aa0 k))
("hollenback" nil (hh aa1 l ax n b ae1 k))
("hollenbaugh" nil (hh ax l eh1 n b ao0))
("hollenbeck" nil (hh aa1 l ax n b eh1 k))
("hollenberg" nil (hh aa1 l ax n b er0 g))
("hollender" nil (hh aa1 l eh0 n d er0))
("hollenkamp" nil (hh aa1 l ih0 n k ae0 m p))
("holler" nil (hh aa1 l er0))
("holleran" nil (hh aa1 l er0 ax n))
("hollerbach" nil (hh aa1 l er0 b aa1 k))
("hollered" nil (hh aa1 l er0 d))
("hollering" nil (hh aa1 l er0 ih0 ng))
("hollern" nil (hh aa1 l er0 n))
("hollers" nil (hh aa1 l er0 z))
("hollett" nil (hh aa1 l ih0 t))
("holley" nil (hh aa1 l iy0))
("hollibaugh" nil (hh aa1 l ih0 b ao1))
("hollick" nil (hh aa1 l ih0 k))
("holliday" nil (hh aa1 l ih0 d ey1))
("hollie" nil (hh aa1 l iy0))
("hollier" nil (hh ao1 l iy0 er0))
("hollies" nil (hh aa1 l iy0 z))
("hollifield" nil (hh aa1 l ih0 f iy1 l d))
("holliman" nil (hh aa1 l ih0 m ax n))
("hollimon" nil (hh aa1 l ih0 m ax n))
("hollin" nil (hh aa1 l ih0 n))
("holling" nil (hh aa1 l ih0 ng))
("hollinger" nil (hh aa1 l ih0 ng er0))
("hollings" nil (hh aa1 l ih0 ng z))
("hollingshead" nil (hh aa1 l ih0 ng z hh eh1 d))
("hollingsworth" nil (hh aa1 l ih0 ng z w er1 th))
("hollingworth" nil (hh aa1 l ih0 ng g w er1 th))
("hollins" nil (hh aa1 l ih0 n z))
("hollinshead" nil (hh aa1 l ih0 n s hh eh1 d))
("hollinsworth" nil (hh aa1 l ih0 n s w er1 th))
("hollis" nil (hh aa1 l ih0 s))
("hollister" nil (hh aa1 l ih0 s t er0))
("holliston" nil (hh aa1 l ih0 s t ax n))
("hollman" nil (hh aa1 l m ax n))
("hollmann" nil (hh aa1 l m ax n))
("hollo" nil (hh aa1 l ow1))
("hollobaugh" nil (hh aa1 l ax b ao0))
("holloman" nil (hh aa1 l ow0 m ax n))
("hollomon" nil (hh aa1 l ax m aa0 n))
("hollon" nil (hh aa1 l ax n))
("hollopeter" nil (hh aa1 l ax p iy0 t er0))
("holloran" nil (hh aa1 l er0 ax n))
("hollow" nil (hh aa1 l ow0))
("holloway" nil (hh aa1 l ow0 w ey1))
("hollowed" nil (hh aa1 l ow0 d))
("hollowell" nil (hh aa1 l ax w eh0 l))
("hollowing" nil (hh aa1 l ow0 ih0 ng))
("hollows" nil (hh aa1 l ow0 z))
("hollstein" nil (hh aa1 l s t ay0 n))
("holly" nil (hh aa1 l iy0))
("hollyfield" nil (hh aa1 l ih0 f iy1 l d))
("hollyhocks" nil (hh aa1 l iy0 hh aa1 k s))
("hollywood" nil (hh aa1 l iy0 w uh1 d))
("holm" nil (hh ow1 m))
("holman" nil (hh aa1 l m ax n))
("holmberg" nil (hh ow1 l m b er0 g))
("holme" nil (hh ow1 l m))
("holmen" nil (hh aa1 l m eh0 n))
("holmer" nil (hh ow1 l m er0))
("holmes" nil (hh ow1 m z))
("holmgren" nil (hh ow1 l m g r eh0 n))
("holmium" nil (hh ow1 l m iy0 ax m))
("holmlund" nil (hh ow1 l m l ax n d))
("holmquist" nil (hh ow1 l m k w ih1 s t))
("holmstrom" nil (hh ow1 l m s t r ax m))
("holness" nil (hh aa1 l n ih0 s))
("holocaust" nil (hh aa1 l ax k ao1 s t))
("hologram" nil (hh aa1 l ax g r ae1 m))
("holograms" nil (hh aa1 l ax g r ae1 m z))
("holographic" nil (hh aa1 l ax g r ae1 f ih0 k))
("holohan" nil (hh aa1 l ax hh ae0 n))
("holquin" nil (hh ow1 l k w ih0 n))
("holroyd" nil (hh ow1 l r oy1 d))
("holsapple" nil (hh ow1 l s ax p ax l))
("holscher" nil (hh ow1 l sh er0))
("holsclaw" nil (hh ow1 l s k l ao0))
("holsey" nil (hh ow1 l s iy0))
("holshouser" nil (hh ow1 l s hh aw1 s er0))
("holsinger" nil (hh ow1 l s ih0 ng er0))
("holsomback" nil (hh ow1 l s ax m b ae1 k))
("holsonback" nil (hh ow1 l s ax n b ae1 k))
("holsopple" nil (hh ow1 l s ax p ax l))
("holst" nil (hh ow1 l s t))
("holstad" nil (hh ow1 l s t ax d))
("holste" nil (hh ow1 l s t))
("holstead" nil (hh ow1 l s t eh1 d))
("holstein" nil (hh ow1 l s t iy1 n))
("holsten" nil (hh ow1 l s ax n))
("holster" nil (hh ow1 l s t er0))
("holstine" nil (hh ow1 l s t ay1 n))
("holston" nil (hh ow1 l s t ax n))
("holstrom" nil (hh ow1 l s t r ax m))
("holsworth" nil (hh ow1 l s w er0 th))
("holt" nil (hh ow1 l t))
("holtan" nil (hh ow1 l t ax n))
("holte" nil (hh ow1 l t))
("holten" nil (hh ow1 l t ax n))
("holter" nil (hh ow1 l t er0))
("holterman" nil (hh ow1 l t er0 m ax n))
("holthaus" nil (hh ow1 l t hh aw1 s))
("holthus" nil (hh ow1 l th ax s))
("holtkamp" nil (hh ow1 l t k ae1 m p))
("holtman" nil (hh ow1 l t m ax n))
("holtmann" nil (hh ow1 l t m ax n))
("holton" nil (hh ow1 l t ax n))
("holtorf" nil (hh ow1 l t er0 f))
("holtrop" nil (hh ow1 l t r ax p))
("holtry" nil (hh ow1 l t r iy0))
("holts" nil (hh ow1 l t s))
("holtsclaw" nil (hh ow1 l t s k l ao1))
("holtz" nil (hh ow1 l t s))
("holtzapple" nil (hh ow1 l t z ax p ax l))
("holtzclaw" nil (hh ow1 l t s k l ao0))
("holtzer" nil (hh ow1 l t z er0))
("holtzinger" nil (hh ow1 l t z ih0 ng er0))
("holtzman" nil (hh ow1 l t s m ax n))
("holub" nil (hh ow1 l ax b))
("holum" nil (hh ow1 l ax m))
("holverson" nil (hh aa1 l v er0 s ax n))
("holvis" nil (hh ow1 l v ax s))
("holway" nil (hh aa1 l w ey0))
("holweger" nil (hh ow1 l w ih0 g er0))
("holwerda" nil (hh ow0 l w er1 d ax))
("holy" nil (hh ow1 l iy0))
("holycross" nil (hh ow1 l iy0 k r ao1 s))
("holyfield" nil (hh ow1 l iy0 f iy1 l d))
("holyoak" nil (hh ow1 l iy0 ow1 k))
("holyoke" nil (hh ow1 l iy0 ow1 k))
("holz" nil (hh ow1 l z))
("holzapfel" nil (hh ow1 l z ax p f ax l))
("holzer" nil (hh ow1 l z er0))
("holzhauer" nil (hh ow1 l z hh aw0 er0))
("holzhauser" nil (hh ow1 l z hh aw0 z er0))
("holzheimer" nil (hh ow1 l z hh ay0 m er0))
("holzinger" nil (hh ow1 l z ih0 ng er0))
("holzman" nil (hh ow1 l z m ax n))
("holzmann" nil (hh ow1 l z m ax n))
("holzschuh" nil (hh ow1 l sh uw0))
("holzstoff" nil (hh ow1 l s t ao1 f))
("holzwarth" nil (hh ow1 l z w er0 th))
("holzworth" nil (hh ow1 l z w er0 th))
("hom" nil (hh aa1 m))
("homa" nil (hh ow1 m ax))
("homac" nil (hh ow1 m ae0 k))
("homage" nil (aa1 m ax jh))
("homan" nil (hh ow1 m ax n))
("homann" nil (hh ow1 m ax n))
("homans" nil (hh ow1 m ax n z))
("homart" nil (hh ow1 m aa1 r t))
("hombre" nil (hh aa1 m b r ax))
("homburg" nil (hh aa1 m b er0 g))
("home" nil (hh ow1 m))
("homebuilder" nil (hh ow1 m b ih1 l d er0))
("homebuilders" nil (hh ow1 m b ih1 l d er0 z))
("homebuilding" nil (hh ow1 m b ih1 l d ih0 ng))
("homebuyers" nil (hh ow1 m b ay1 er0 z))
("homecare" nil (hh ow1 m k eh1 r))
("homeclub" nil (hh ow1 m k l ah1 b))
("homecoming" nil (hh ow1 m k ah1 m ih0 ng))
("homed" nil (hh ow1 m d))
("homedco" nil (hh ow1 m eh1 d k ow1))
("homefed" nil (hh ow1 m f eh1 d))
("homegrown" nil (hh ow1 m g r ow1 n))
("homeland" nil (hh ow1 m l ae1 n d))
("homelands" nil (hh ow1 m l ae1 n d z))
("homeless" nil (hh ow1 m l ax s))
("homelessness" nil (hh ow1 m l ax s n ax s))
("homelike" nil (hh ow1 m l ay1 k))
("homely" nil (hh ow1 m l iy0))
("homemade" nil (hh ow1 m m ey1 d))
("homemaker" nil (hh ow1 m m ey1 k er0))
("homemakers" nil (hh ow1 m m ey1 k er0 z))
("homemaking" nil (hh ow1 m m ey1 k ih0 ng))
("homen" nil (hh ow1 m ax n))
("homeostasis" nil (hh ow1 m iy0 ow0 s t ey1 s ax s))
("homeostatic" nil (hh ow1 m iy0 ow0 s t ae1 t ih0 k))
("homeowner" nil (hh ow1 m ow1 n er0))
("homeowners" nil (hh ow1 m ow1 n er0 z))
("homeownership" nil (hh ow1 m ow1 n er0 sh ih1 p))
("homeporting" nil (hh ow1 m p ao1 r t ih0 ng))
("homer" nil (hh ow1 m er0))
("homeric" nil (hh ow0 m eh1 r ih0 k))
("homers" nil (hh ow1 m er0 z))
("homes" nil (hh ow1 m z))
("homesick" nil (hh ow1 m s ih1 k))
("homesickness" nil (hh ow1 m s ih1 k n ax s))
("homesley" nil (hh ow1 m z l iy0))
("homespun" nil (hh ow1 m s p ah1 n))
("homestake" nil (hh ow1 m s t ey1 k))
("homestate" nil (hh ow1 m s t ey1 t))
("homestead" nil (hh ow1 m s t eh1 d))
("homesteaded" nil (hh ow1 m s t eh1 d ih0 d))
("homestretch" nil (hh ow1 m s t r eh1 ch))
("hometown" nil (hh ow1 m t aw1 n))
("homeward" nil (hh ow1 m w er0 d))
("homewood" nil (hh ow1 m w uh1 d))
("homework" nil (hh ow1 m w er1 k))
("homeworkers" nil (hh ow1 m w er1 k er0 z))
("homey" nil (hh ow1 m iy0))
("homeyer" nil (hh ow1 m iy0 er0))
("homicidal" nil (hh aa1 m ax s ay1 d ax l))
("homicide" nil (hh aa1 m ax s ay1 d))
("homicides" nil (hh aa1 m ax s ay1 d z))
("homiletic" nil (hh aa1 m ax l eh1 t ih0 k))
("homilies" nil (hh ow1 m ax l iy0 z))
("homily" nil (hh aa1 m ax l iy0))
("hominem" nil (hh aa1 m ih0 n eh0 m))
("homing" nil (hh ow1 m ih0 ng))
("hominy" nil (hh aa1 m ih0 n iy0))
("homme" nil (hh aa1 m))
("hommel" nil (hh aa1 m ax l))
("hommes" nil (hh aa1 m z))
("homo" nil (hh ow1 m ow0))
("homogeneity" nil (hh aa1 m ax jh ax n iy1 ax t iy0))
("homogeneous" nil (hh ow1 m ax jh iy1 n iy0 ax s))
("homogenized" nil (hh ow0 m aa1 jh ax n ay1 z d))
("homogenous" nil (hh ax m aa1 jh ax n ax s))
("homogeny" nil (hh ow0 m aa1 jh ax n iy0))
("homola" nil (hh ax m aa1 l ax))
("homolka" nil (hh ax m aa1 l k ax))
("homologous" nil (hh ax m aa1 l ax g ax s))
("homophobia" nil (hh ow1 m ax f ow1 b iy0 ax))
("homosexual" nil (hh ow1 m ow0 s eh1 k sh ax w ax l))
("homosexuality" nil (hh ow1 m ow0 s eh1 k sh ax w ae1 l ax t iy0))
("homosexuals" nil (hh ow1 m ow0 s eh1 k sh ax w ax l z))
("homosporous" nil (hh ax m aa1 s p er0 ax s))
("homozygous" nil (hh ow1 m ow0 z ay1 g ax s))
("homrich" nil (hh aa1 m r ih0 k))
("homs" nil (hh aa1 m z))
("homsey" nil (hh aa1 m z iy0))
("homuth" nil (hh aa1 m uw0 th))
("hon" nil (hh aa1 n))
("honaker" nil (hh aa1 n ax k er0))
("honan" nil (hh ow1 n ax n))
("honasan" nil (hh aa1 n ax z ax n))
("honda" nil (hh ao1 n d ax))
("hondas" nil (hh aa1 n d ax s))
("honderich" nil (hh aa1 n d er0 ih0 k))
("hondo" nil (hh aa1 n d ow0))
("honduran" nil (hh aa0 n d uh1 r ax n))
("hondurans" nil (hh aa0 n d uh1 r ax n z))
("honduras" nil (hh aa0 n d uh1 r ax s))
("hone" nil (hh ow1 n))
("honea" nil (hh aa1 n iy0 ax))
("honeck" nil (hh ow1 n eh0 k))
("honecker" nil (hh ow1 n eh0 k er0))
("honed" nil (hh ow1 n d))
("honer" nil (hh ow1 n er0))
("honest" nil (aa1 n ax s t))
("honestly" nil (aa1 n ax s t l iy0))
("honesty" nil (aa1 n ax s t iy0))
("honey" nil (hh ah1 n iy0))
("honeybee" nil (hh ah1 n iy0 b iy1))
("honeybees" nil (hh ah1 n iy0 b iy1 z))
("honeycomb" nil (hh ah1 n iy0 k ow1 m))
("honeycutt" nil (hh ah1 n iy0 k ax t))
("honeydew" nil (hh ah1 n iy0 d uw1))
("honeyman" nil (hh aa1 n iy0 m ax n))
("honeymoon" nil (hh ah1 n iy0 m uw1 n))
("honeymooned" nil (hh ah1 n iy0 m uw1 n d))
("honeymooners" nil (hh ah1 n iy0 m uw1 n er0 z))
("honeys" nil (hh ah1 n iy0 z))
("honeysuckles" nil (hh ah1 n iy0 s ah1 k ax l z))
("honeywell" nil (hh ah1 n iy0 w eh1 l))
("hong" nil (hh ao1 ng))
("hongkong" nil (hh ao1 ng k ao0 ng))
("honig" nil (hh aa1 n ih0 g))
("honing" nil (hh ow1 n ih0 ng))
("honk" nil (hh aa1 ng k))
("honking" nil (hh ao1 ng k ih0 ng))
("honky" nil (hh ao1 ng k iy0))
("honn" nil (hh aa1 n))
("honnold" nil (hh aa1 n ow1 l d))
("honold" nil (hh aa1 n ow0 l d))
("honolulu" nil (hh aa1 n ax l uw1 l uw0))
("honor" nil (aa1 n er0))
("honora" nil (ao0 n ao1 r ax))
("honorable" nil (aa1 n er0 ax b ax l))
("honorably" nil (aa1 n er0 ax b l iy0))
("honoraria" nil (aa1 n er0 eh1 r iy0 ax))
("honorarium" nil (aa1 n er0 eh1 r iy0 ax m))
("honorariums" nil (aa1 n er0 eh1 r iy0 ax m z))
("honorary" nil (aa1 n er0 eh1 r iy0))
("honore" nil (aa1 n er0))
("honored" nil (aa1 n er0 d))
("honoria" nil (aa0 n er1 iy0 ax))
("honorific" nil (aa1 n er0 ih1 f ih0 k))
("honoring" nil (aa1 n er0 ih0 ng))
("honors" nil (aa1 n er0 z))
("hons" nil (hh aa1 n z))
("honsberger" nil (hh aa1 n s b er0 g er0))
("honse" nil (hh aa1 n s))
("honshu" nil (hh aa1 n sh uw0))
("honsinger" nil (hh aa1 n s ih0 ng er0))
("hontz" nil (hh aa1 n t s))
("hoo" nil (hh uw1))
("hoobler" nil (hh uw1 b l er0))
("hooch" nil (hh uw1 ch))
("hoock" nil (hh uw1 k))
("hood" nil (hh uh1 d))
("hooded" nil (hh uh1 d ih0 d))
("hoodlum" nil (hh uh1 d l ax m))
("hoodlums" nil (hh uw1 d l ax m z))
("hoods" nil (hh uh1 d z))
("hoodwink" nil (hh uh1 d w ih1 ng k))
("hoodwinked" nil (hh uh1 d w ih1 ng k t))
("hooey" nil (hh uw1 iy0))
("hoof" nil (hh uw1 f))
("hoofed" nil (hh uh1 f t))
("hoofs" nil (hh uh1 f s))
("hoog" nil (hh uw1 g))
("hooge" nil (hh uw1 jh))
("hoogland" nil (hh uw1 g l ax n d))
("hoogovens" nil (hh uw1 g ax v ih0 n z))
("hook" nil (hh uh1 k))
("hooke" nil (hh uh1 k))
("hooked" nil (hh uh1 k t))
("hooker" nil (hh uh1 k er0))
("hookers" nil (hh uh1 k er0 z))
("hooking" nil (hh uh1 k ih0 ng))
("hooks" nil (hh uh1 k s))
("hookup" nil (hh uh1 k ah1 p))
("hookups" nil (hh uh1 k ah1 p s))
("hookworm" nil (hh uh1 k w er0 m))
("hoole" nil (hh uw1 l))
("hooley" nil (hh uw1 l iy0))
("hooliganism" nil (hh uw1 l ih0 g ax n ih1 z ax m))
("hooligans" nil (hh uw1 l ih0 g ax n z))
("hoolihan" nil (hh uw1 l ih0 hh ae0 n))
("hoon" nil (hh uw1 n))
("hoop" nil (hh uw1 p))
("hooper" nil (hh uw1 p er0))
("hooperman" nil (hh uw1 p er0 m ax n))
("hoopes" nil (hh uw1 p s))
("hoopingarner" nil (hh uw0 p ih1 ng g aa0 r n er0))
("hoopla" nil (hh uw1 p l aa1))
("hoopoe" nil (hh uw1 p uw1))
("hoops" nil (hh uw1 p s))
("hooray" nil (hh uh0 r ey1))
("hoos" nil (hh uw1 z))
("hoose" nil (hh uw1 s))
("hooser" nil (hh uw1 z er0))
("hoosier" nil (hh uw1 zh er0))
("hoosiers" nil (hh uw1 z y er0 z))
("hoot" nil (hh uw1 t))
("hooted" nil (hh uw1 t ih0 d))
("hooten" nil (hh uw1 t ax n))
("hootman" nil (hh uw1 t m ax n))
("hooton" nil (hh uw1 t ax n))
("hoots" nil (hh uw1 t s))
("hooven" nil (hh uw1 v ax n))
("hoover" nil (hh uw1 v er0))
("hooves" nil (hh uh1 v z))
("hop" nil (hh aa1 p))
("hopburg" nil (hh aa1 p b er0 g))
("hope" nil (hh ow1 p))
("hoped" nil (hh ow1 p t))
("hopeful" nil (hh ow1 p f ax l))
("hopefully" nil (hh ow1 p f ax l iy0))
("hopefuls" nil (hh ow1 p f ax l z))
("hopeless" nil (hh ow1 p l ax s))
("hopelessly" nil (hh ow1 p l ax s l iy0))
("hopelessness" nil (hh ow1 p l ax s n ax s))
("hopes" nil (hh ow1 p s))
("hopewell" nil (hh ow1 p w eh1 l))
("hopf" nil (hh aa1 p f))
("hopfensperger" nil (hh aa1 p f ih0 n s p er0 g er0))
("hopfer" nil (hh aa1 p f er0))
("hopfinger" nil (hh aa1 p f ih0 ng er0))
("hopgood" nil (hh aa1 p g uh1 d))
("hopi" nil (hh ow1 p iy0))
("hoping" nil (hh ow1 p ih0 ng))
("hopke" nil (hh ow1 p k))
("hopkin" nil (hh aa1 p k ih0 n))
("hopkins" nil (hh aa1 p k ih0 n z))
("hopkinson" nil (hh aa1 p k ih0 n s ax n))
("hopkinton" nil (hh aa1 p k ih0 n t ax n))
("hopkyns" nil (hh aa1 p k ih0 n z))
("hopman" nil (hh aa1 p m ax n))
("hopp" nil (hh aa1 p))
("hoppa" nil (hh aa1 p ax))
("hoppe" nil (hh aa1 p))
("hopped" nil (hh aa1 p t))
("hoppel" nil (hh aa1 p ax l))
("hopper" nil (hh aa1 p er0))
("hoppers" nil (hh aa1 p er0 z))
("hoppert" nil (hh aa1 p er0 t))
("hoppes" nil (hh aa1 p s))
("hopping" nil (hh aa1 p ih0 ng))
("hopple" nil (hh aa1 p ax l))
("hopps" nil (hh aa1 p s))
("hops" nil (hh aa1 p s))
("hopscotch" nil (hh aa1 p s k aa1 ch))
("hopson" nil (hh aa1 p s ax n))
("hopton" nil (hh aa1 p t ax n))
("hopwood" nil (hh aa1 p w uh1 d))
("hora" nil (hh ao1 r ax))
("horace" nil (hh ao1 r ax s))
("horacek" nil (hh ao1 r ax ch eh0 k))
("horacia" nil (hh ao0 r aa1 s iy0 ax))
("horacio" nil (hh ao0 r ey1 s iy0 ow0))
("horak" nil (hh ao1 r ax k))
("horan" nil (hh ao1 r ax n))
("horatia" nil (hh ao0 r aa1 sh ax))
("horatian" nil (hh er0 ey1 sh ax n))
("horatio" nil (hh er0 ey1 sh ow0))
("horatius" nil (hh ao0 r ey1 sh ax s))
("horch" nil (hh ao1 r k))
("horchow" nil (hh ao1 r ch ow0))
("hord" nil (hh ao1 r d))
("horde" nil (hh ao1 r d))
("hordes" nil (hh ao1 r d z))
("horehound" nil (hh aa1 r hh aw1 n d))
("horejsi" nil (hh ao0 r ey1 y s iy0))
("horelick" nil (hh ao1 r ax l ih0 k))
("horen" nil (hh ao1 r ax n))
("horenstein" nil (hh ao1 r ax n s t ay0 n))
("horgan" nil (hh ao1 r g ax n))
("horger" nil (hh ao1 r g er0))
("hori" nil (hh ao1 r iy0))
("horiguchi" nil (hh ao1 r ih0 g uw1 ch iy0))
("horine" nil (hh ao1 r iy0 n))
("horiuchi" nil (hh ao0 r iy0 uw1 ch iy0))
("horizon" nil (hh er0 ay1 z ax n))
("horizons" nil (hh er0 ay1 z ax n z))
("horizontal" nil (hh ao1 r ax z aa1 n t ax l))
("horizontally" nil (hh ao1 r ih0 z aa1 n t ax l iy0))
("horkey" nil (hh ao1 r k iy0))
("horky" nil (hh ao1 r k iy0))
("horlacher" nil (hh ao1 r l ax k er0))
("horlick" nil (hh ao1 r l ih0 k))
("horman" nil (hh ao1 r m ax n))
("hormann" nil (hh ao1 r m ax n))
("hormats" nil (hh ao1 r m ae0 t s))
("hormel" nil (hh ao0 r m eh1 l))
("hormonal" nil (hh ao1 r m ow1 n ax l))
("hormone" nil (hh ao1 r m ow1 n))
("hormones" nil (hh ao1 r m ow1 n z))
("hormuz" nil (hh ao1 r m uw0 z))
("horn" nil (hh ao1 r n))
("hornack" nil (hh ao1 r n ax k))
("hornaday" nil (hh ao1 r n ax d ey1))
("hornak" nil (hh ao1 r n ax k))
("hornback" nil (hh ao1 r n b ae1 k))
("hornbaker" nil (hh ao1 r n b ax k er0))
("hornbeak" nil (hh ao1 r n b ax k))
("hornbeck" nil (hh ao1 r n b eh1 k))
("hornberger" nil (hh ao1 r n b er0 g er0))
("hornbills" nil (hh ao1 r n b ih1 l z))
("hornblende" nil (hh ao1 r n b l eh1 n d))
("hornbook" nil (hh ao1 r n b uh1 k))
("hornbostel" nil (hh ao1 r n b ax s t ax l))
("hornbrook" nil (hh ao1 r n b r uh1 k))
("hornbuckle" nil (hh ao1 r n b ax k ax l))
("hornburg" nil (hh ao1 r n b er0 g))
("hornby" nil (hh ao1 r n b iy0))
("horne" nil (hh ao1 r n))
("horned" nil (hh ao1 r n d))
("horner" nil (hh ao1 r n er0))
("hornet" nil (hh ao1 r n ih0 t))
("hornets" nil (hh ao1 r n ax t s))
("horney" nil (hh ao1 r n iy0))
("hornick" nil (hh ao1 r n ih0 k))
("hornig" nil (hh ao1 r n ih0 g))
("hornik" nil (hh ao1 r n ih0 k))
("horning" nil (hh ao1 r n ih0 ng))
("hornless" nil (hh ao1 r n l ax s))
("hornlike" nil (hh ao1 r n l ay1 k))
("hornor" nil (hh ao1 r n er0))
("horns" nil (hh ao1 r n z))
("hornsby" nil (hh ao1 r n z b iy0))
("hornstein" nil (hh ao1 r n s t ay1 n))
("hornung" nil (hh ao1 r n ax ng))
("horny" nil (hh ao1 r n iy0))
("hornyak" nil (hh ao1 r n y ae0 k))
("horoscope" nil (hh ao1 r ax s k ow1 p))
("horoscopes" nil (hh ao1 r ax s k ow1 p s))
("horovitz" nil (hh aa1 r ax v ih0 t s))
("horowitz" nil (hh ao1 r ow0 ih0 t s))
("horr" nil (hh ao1 r))
("horrell" nil (hh ao1 r ax l))
("horrendous" nil (hh ao1 r eh1 n d ax s))
("horrendously" nil (hh ao1 r eh1 n d ax s l iy0))
("horrible" nil (hh ao1 r ax b ax l))
("horribly" nil (hh ao1 r ax b l iy0))
("horrid" nil (hh ao1 r ax d))
("horrific" nil (hh ao0 r ih1 f ih0 k))
("horrified" nil (hh ao1 r ax f ay1 d))
("horrifying" nil (hh ao1 r ax f ay1 ih0 ng))
("horrigan" nil (hh ao1 r ax g ax n))
("horrocks" nil (hh ao1 r ax k s))
("horror" nil (hh ao1 r er0))
("horrors" nil (hh ao1 r er0 z))
("hors" nil (hh ao1 r z))
("horsch" nil (hh ao1 r sh))
("horse" nil (hh ao1 r s))
("horseback" nil (hh ao1 r s b ae1 k))
("horseflesh" nil (hh ao1 r s f l eh1 sh))
("horsehead" nil (hh ao1 r s hh eh1 d))
("horseman" nil (hh ao1 r s m ax n))
("horsemanship" nil (hh ao1 r s m ax n sh ih0 p))
("horsemen" nil (hh ao1 r s m ax n))
("horseplay" nil (hh ao1 r s p l ey1))
("horsepower" nil (hh ao1 r s p aw1 er0))
("horseradish" nil (hh ao1 r s r ae1 d ih0 sh))
("horses" nil (hh ao1 r s ax z))
("horseshoe" nil (hh ao1 r s sh uw1))
("horseshoes" nil (hh ao1 r s sh uw1 z))
("horsetails" nil (hh ao1 r s t ey1 l z))
("horsey" nil (hh ao1 r s iy0))
("horsfall" nil (hh ao1 r s f ax l))
("horsham" nil (hh ao1 r sh ax m))
("horsing" nil (hh ao1 r s ih0 ng))
("horsley" nil (hh ao1 r s l iy0))
("horsman" nil (hh ao1 r s m ax n))
("horst" nil (hh ao1 r s t))
("horstman" nil (hh ao1 r s t m ax n))
("horstmann" nil (hh ao1 r s t m ax n))
("horta" nil (hh ao1 r t ax))
("horten" nil (hh ao1 r t ax n))
("horter" nil (hh ao1 r t er0))
("horticultural" nil (hh ao1 r t ax k ah1 l ch er0 ax l))
("horticulture" nil (hh ao1 r t ih0 k ah1 l ch er0))
("horticulturist" nil (hh ao1 r t ih0 k ah1 l ch er0 ih0 s t))
("hortman" nil (hh ao1 r t m ax n))
("horton" nil (hh ao1 r t ax n))
("horvac" nil (hh ao1 r v ae0 k))
("horvat" nil (hh ao1 r v ax t))
("horvath" nil (hh ao1 r v ae0 th))
("horvitz" nil (hh ao1 r v ih0 t s))
("horwath" nil (hh ao1 r w ao0 th))
("horwich" nil (hh ao1 r w ih0 k))
("horwitz" nil (hh ao1 r w ih0 t s))
("hosack" nil (hh aa1 s ax k))
("hosch" nil (hh ao1 sh))
("hose" nil (hh ow1 z))
("hosea" nil (hh ow0 s iy1 ax))
("hosed" nil (hh ow1 z d))
("hosek" nil (hh ow1 s eh0 k))
("hoselton" nil (hh ax s eh1 l t ax n))
("hoses" nil (hh ow1 z ih0 z))
("hosey" nil (hh ow1 z iy0))
("hosford" nil (hh aa1 s f er0 d))
("hoshaw" nil (hh aa1 sh ao0))
("hosick" nil (hh aa1 s ih0 k))
("hosie" nil (hh ow1 z iy0))
("hosiery" nil (hh ow1 zh er0 iy0))
("hoskin" nil (hh aa1 s k ih0 n))
("hosking" nil (hh aa1 s k ih0 ng))
("hoskins" nil (hh aa1 s k ih0 n z))
("hoskinson" nil (hh aa1 s k ih0 n s ax n))
("hoskyns" nil (hh ao1 s k ih0 n z))
("hosler" nil (hh aa1 s ax l er0))
("hosley" nil (hh aa1 s l iy0))
("hosman" nil (hh aa1 s m ax n))
("hosner" nil (hh aa1 s n er0))
("hosni" nil (hh ow1 z n iy0))
("hosoka" nil (hh ow1 s aa1 k ax))
("hosokawa" nil (hh ow1 s ax k aa1 w ax))
("hospice" nil (hh aa1 s p ax s))
("hospices" nil (hh aa1 s p ax s ih0 z))
("hospitable" nil (hh aa1 s p ih1 t ax b ax l))
("hospital" nil (hh aa1 s p ih1 t ax l))
("hospitality" nil (hh aa1 s p ax t ae1 l ax t iy0))
("hospitalization" nil (hh aa1 s p ih0 t ax l ax z ey1 sh ax n))
("hospitalizations" nil (hh aa1 s p ih0 t ax l ax z ey1 sh ax n z))
("hospitalize" nil (hh ao1 s p ih1 t ax l ay1 z))
("hospitalized" nil (hh aa1 s p ih0 t ax l ay1 z d))
("hospitals" nil (hh aa1 s p ih1 t ax l z))
("hoss" nil (hh aa1 s))
("hossack" nil (hh aa1 s ax k))
("hossain" nil (hh ax s ey1 n))
("hossein" nil (hh ao1 s ey1 n))
("hosseini" nil (hh ow0 s ey0 iy1 n iy0))
("hossler" nil (hh aa1 s ax l er0))
("host" nil (hh ow1 s t))
("hostage" nil (hh aa1 s t ih0 jh))
("hostages" nil (hh aa1 s t ax jh ax z))
("hosted" nil (hh ow1 s t ih0 d))
("hostel" nil (hh aa1 s t ax l))
("hostels" nil (hh aa1 s t ax l z))
("hostench" nil (hh ao1 s t ax n ch))
("hoster" nil (hh ow1 s t er0))
("hosterman" nil (hh ow1 s t er0 m ax n))
("hostess" nil (hh ow1 s t ax s))
("hostesses" nil (hh ow1 s t ax s ax z))
("hostetler" nil (hh aa1 s t ih0 t ax l er0))
("hostetter" nil (hh aa1 s t ih0 t er0))
("hostettler" nil (hh aa1 s t ih0 t ax l er0))
("hostile" nil (hh aa1 s t ax l))
("hostilities" nil (hh aa0 s t ih1 l ax t iy0 z))
("hostility" nil (hh aa0 s t ih1 l ax t iy0))
("hosting" nil (hh ow1 s t ih0 ng))
("hostler" nil (hh aa1 s l er0))
("hosts" nil (hh ow1 s t s))
("hostutler" nil (hh aa1 s t uw0 t ax l er0))
("hot" nil (hh aa1 t))
("hotaling" nil (hh aa1 t ax l ih0 ng))
("hotard" nil (hh aa1 t er0 d))
("hotbed" nil (hh aa1 t b eh1 d))
("hotbeds" nil (hh aa1 t b eh1 d z))
("hotcakes" nil (hh aa1 t k ey1 k s))
("hotchkin" nil (hh aa1 ch k ih0 n))
("hotchkiss" nil (hh aa1 ch k ih0 s))
("hotdog" nil (hh aa1 t d ao1 g))
("hotdogs" nil (hh aa1 t d ao1 g z))
("hotel" nil (hh ow0 t eh1 l))
("hotelier" nil (hh ow0 t eh1 l y er0))
("hoteliers" nil (hh ow0 t eh1 l y er0 z))
("hotels" nil (hh ow0 t eh1 l z))
("hoth" nil (hh aa1 th))
("hothouse" nil (hh aa1 t hh aw1 s))
("hotline" nil (hh aa1 t l ay1 n))
("hotlines" nil (hh aa1 t l ay1 n z))
("hotly" nil (hh aa1 t l iy0))
("hots" nil (hh aa1 t s))
("hotshot" nil (hh aa1 ch aa1 t))
("hott" nil (hh aa1 t))
("hottel" nil (hh aa1 t ax l))
("hottelet" nil (hh aa1 t l eh0 t))
("hottenstein" nil (hh aa1 t ax n s t ay0 n))
("hotter" nil (hh aa1 t er0))
("hottest" nil (hh aa1 t ax s t))
("hottinger" nil (hh aa1 t ih0 ng er0))
("hottle" nil (hh aa1 t ax l))
("hottman" nil (hh aa1 t m ax n))
("hotwired" nil (hh aa1 t w ay1 er0 d))
("hotz" nil (hh aa1 t s))
("hotze" nil (hh ow1 t z))
("hou" nil (hh uw1))
("houchen" nil (hh aw1 k ax n))
("houchens" nil (hh aw1 k ax n z))
("houchin" nil (hh aw1 k ih0 n))
("houchins" nil (hh aw1 k ih0 n z))
("houck" nil (hh aw1 k))
("houdaille" nil (hh uw1 d ey1 l))
("houde" nil (hh aw1 d))
("houdek" nil (hh aw1 d ih0 k))
("houdeshell" nil (hh uw1 d ih0 sh ax l))
("houdini" nil (hh uw0 d iy1 n iy0))
("houff" nil (hh ow1 f))
("houg" nil (hh aw1 g))
("houge" nil (hh aw1 jh))
("hougen" nil (hh aw1 g ax n))
("hough" nil (hh ah1 f))
("hougham" nil (hh aw1 ax m))
("houghland" nil (hh aw1 l ax n d))
("houghs" nil (hh aw1 z))
("houghtaling" nil (hh ao1 t ax l ih0 ng))
("houghton" nil (hh ao1 t ax n))
("hougland" nil (hh aw1 g l ax n d))
("houk" nil (hh aw1 k))
("houle" nil (hh aw1 l))
("houlihan" nil (hh uw1 l ih0 hh ae0 n))
("houlton" nil (hh ow1 l t ax n))
("hound" nil (hh aw1 n d))
("hounded" nil (hh aw1 n d ih0 d))
("hounding" nil (hh aw1 n d ih0 ng))
("hounds" nil (hh aw1 n d z))
("hounshell" nil (hh aw1 n sh ax l))
("houp" nil (hh uw1 p))
("houpt" nil (hh uw1 p t))
("hour" nil (aw1 er0))
("hourglass" nil (aw1 er0 g l ae1 s))
("hourigan" nil (aw1 r ih0 g ae0 n))
("hourihan" nil (aw0 r iy1 hh aa0 n))
("hourlong" nil (aw1 r l ao1 ng))
("hourly" nil (aw1 r l iy0))
("hours" nil (aw1 er0 z))
("housand" nil (hh aw1 s ax n d))
("housden" nil (hh aw1 s d ax n))
("house" n (hh aw1 s))
("house" v (hh aw1 z))
("houseal" nil (hh aw1 s ax l))
("houseboat" nil (hh aw1 s b ow1 t))
("housebroken" nil (hh aw1 s b r ow1 k ax n))
("housecleaning" nil (hh aw1 s k l iy1 n ih0 ng))
("housed" nil (hh aw1 z d))
("houseful" nil (hh aw1 s f ax l))
("household" nil (hh aw1 s hh ow1 l d))
("householder" nil (hh aw1 s hh ow1 l d er0))
("householders" nil (hh aw1 s hh ow1 l d er0 z))
("households" nil (hh aw1 s hh ow1 l d z))
("housekeeper" nil (hh aw1 s k iy1 p er0))
("housekeepers" nil (hh aw1 s k iy1 p er0 z))
("housekeeping" nil (hh aw1 s k iy1 p ih0 ng))
("houseknecht" nil (hh aw1 s k n ih0 k t))
("housel" nil (hh aw1 s ax l))
("houseman" nil (hh aw1 s m ax n))
("houser" nil (hh aw1 z er0))
("houses" n (hh aw1 s ax z))
("houses" v (hh aw1 z ih0 z))
("houseware" nil (hh aw1 s w eh1 r))
("housewares" nil (hh aw1 s w eh1 r z))
("housewife" nil (hh aw1 s w ay1 f))
("housewives" nil (hh aw1 s w ay1 v z))
("housework" nil (hh aw1 s w er1 k))
("houseworth" nil (hh aw1 s w er1 th))
("housewright" nil (hh aw1 s r ay1 t))
("housh" nil (hh aw1 sh))
("housholder" nil (hh aw1 sh ow0 l d er0))
("housing" nil (hh aw1 z ih0 ng))
("housings" nil (hh aw1 z ih0 ng z))
("houska" nil (hh aw1 s k ax))
("housley" nil (hh aw1 s l iy0))
("housman" nil (hh aw1 s m ax n))
("houston" nil (hh y uw1 s t ax n))
("houstonian" nil (hh uw1 s t ow1 n iy0 ax n))
("hout" nil (hh aw1 t))
("houtchens" nil (hh aw1 ch ax n z))
("houten" nil (hh aw1 t eh0 n))
("houtman" nil (hh aw1 t m ax n))
("houts" nil (hh aw1 t s))
("houtz" nil (hh aw1 t s))
("houx" nil (hh uw1))
("houy" nil (hh aa1 ay0))
("houze" nil (hh aw1 z))
("hovan" nil (hh ow1 v ax n))
("hovanec" nil (hh ax v ae1 n ih0 k))
("hovater" nil (hh ow1 v ax t er0))
("hovatter" nil (hh aa1 v ax t er0))
("hovda" nil (hh aa1 v d ax))
("hovde" nil (hh ow1 v d))
("hovden" nil (hh aa1 v d ax n))
("hove" nil (hh ow1 v))
("hovel" nil (hh ah1 v ax l))
("hovels" nil (hh ah1 v ax l z))
("hoven" nil (hh ow1 v ax n))
("hover" nil (hh ah1 v er0))
("hovercraft" nil (hh ah1 v er0 k r ae1 f t))
("hovered" nil (hh ah1 v er0 d))
("hoverflies" nil (hh ah1 v er0 f l ay1 z))
("hovering" nil (hh ah1 v er0 ih0 ng))
("hovermale" nil (hh ah1 v er0 m ax l))
("hovers" nil (hh ah1 v er0 z))
("hoverson" nil (hh ah1 v er0 s ax n))
("hovey" nil (hh ow1 v iy0))
("hoving" nil (hh ow1 v ih0 ng))
("hovious" nil (hh ow1 v iy0 ax s))
("hovis" nil (hh ow1 v ih0 s))
("hovland" nil (hh aa1 v l ax n d))
("hovnanian" nil (hh aa1 v n ey1 n iy0 ax n))
("hovorka" nil (hh ax v ao1 r k ax))
("hovsepian" nil (hh ax v s iy1 p iy0 ax n))
("hovy" nil (hh ow1 v iy0))
("how" nil (hh aw1))
("howald" nil (hh aw1 ax l d))
("howard" nil (hh aw1 er0 d))
("howarth" nil (hh aw1 aa0 r th))
("howat" nil (hh aw1 ax t))
("howatt" nil (hh aw1 ax t))
("howcroft" nil (hh aw1 k r ax f t))
("howden" nil (hh aw1 d ax n))
("howdeshell" nil (hh aw1 d ih0 sh eh0 l))
("howdy" nil (hh aw1 d iy0))
("howdyshell" nil (hh aw1 d ih0 sh eh0 l))
("howe" nil (hh aw1))
("howell" nil (hh aw1 ax l))
("howells" nil (hh aw1 ax l z))
("howenstine" nil (hh aw1 ih0 n s t iy0 n))
("hower" nil (hh aw1 er0))
("howerter" nil (hh aw1 er0 t er0))
("howerton" nil (hh aw0 er1 t ax n))
("howery" nil (hh aw1 er0 iy0))
("howes" nil (hh aw1 z))
("howeth" nil (hh aw1 ih0 th))
("however" nil (hh aw1 eh1 v er0))
("howey" nil (hh aw1 iy0))
("howie" nil (hh aw1 iy0))
("howington" nil (hh aw1 ih0 ng t ax n))
("howison" nil (hh aw1 ih0 s ax n))
("howitt" nil (hh aw1 ih0 t))
("howitzer" nil (hh aw1 ax t s er0))
("howitzers" nil (hh aw1 ax t s er0 z))
("howk" nil (hh aw1 k))
("howl" nil (hh aw1 l))
("howland" nil (hh aw1 l ax n d))
("howle" nil (hh aw1 ax l))
("howled" nil (hh aw1 l d))
("howler" nil (hh aw1 l er0))
("howlett" nil (hh aw1 l ih0 t))
("howley" nil (hh aw1 l iy0))
("howling" nil (hh aw1 l ih0 ng))
("howls" nil (hh aw1 l z))
("howmet" nil (hh aw1 m ax t))
("howorth" nil (hh aa1 w er0 th))
("howry" nil (hh aw1 r iy0))
("howsare" nil (hh aw1 s er0))
("howse" nil (hh aw1 z))
("howser" nil (hh aw1 z er0))
("howson" nil (hh aw1 s ax n))
("howtek" nil (hh aw1 t eh1 k))
("howton" nil (hh aw1 t ax n))
("howze" nil (hh aw1 z))
("hoxie" nil (hh aa1 k s iy0))
("hoxsey" nil (hh aa1 k s iy0))
("hoxsie" nil (hh aa1 k s iy0))
("hoxworth" nil (hh aa1 k s w er0 th))
("hoy" nil (hh oy1))
("hoye" nil (hh oy1))
("hoyer" nil (hh oy1 er0))
("hoying" nil (hh oy1 ih0 ng))
("hoylake" nil (hh oy1 l ey1 k))
("hoyland" nil (hh oy1 l ax n d))
("hoyle" nil (hh oy1 l))
("hoyne" nil (hh oy1 n))
("hoyos" nil (hh oy1 ow0 z))
("hoyt" nil (hh oy1 t))
("hoyvald" nil (hh oy1 v ax l d))
("hrabak" nil (hh r aa1 b ax k))
("hrawi" nil (hh er0 w aa1 w iy0))
("hrdlicka" nil (hh er0 d l ih1 s k ax))
("hreha" nil (hh r iy1 hh ax))
("hren" nil (hh r eh1 n))
("hribar" nil (hh r ih0 b aa1 r))
("hritz" nil (hh r ih1 t s))
("hrivnak" nil (hh r ih1 v n ax k))
("hrncir" nil (hh er1 n ch er0))
("hron" nil (hh r aa1 n))
("hronek" nil (hh r ow1 n ih0 k))
("hrovat" nil (hh r ow1 v ax t))
("hrubik" nil (hh ih0 r uw1 b ih0 k))
("hruby" nil (hh r uw1 b iy0))
("hruska" nil (hh r ah1 s k ax))
("hsia" nil (sh aa1))
("hsiao" nil (sh aw1))
("hsieh" nil (sh iy0 eh1))
("hsiung" nil (sh iy0 ah1 ng))
("hsu" nil (sh uw1))
("hu" nil (hh uw1))
("hua" nil (w aa1))
("huachuca" nil (hh w aa0 ch uw1 k ax))
("huallaga" nil (hh w aa0 l aa1 g ax))
("huan" nil (hh w aa1 n))
("huaneng" nil (hh w aa1 n eh1 ng))
("huang" nil (hh w ae1 ng))
("huard" nil (hh w aa1 r d))
("hub" nil (hh ah1 b))
("hubbard" nil (hh ah1 b er0 d))
("hubbart" nil (hh ah1 b er0 t))
("hubbell" nil (hh ah1 b ax l))
("hubbert" nil (hh ah1 b er0 t))
("hubble" nil (hh ah1 b ax l))
("hubbs" nil (hh ah1 b z))
("hubbub" nil (hh ah1 b ax b))
("hubby" nil (hh ah1 b iy0))
("hubcaps" nil (hh ah1 b k ae1 p s))
("hubco" nil (hh ah1 b k ow0))
("hube" nil (hh y uw1 b))
("huber" nil (hh y uw1 b er0))
("huberman" nil (hh uw1 b er0 m ax n))
("hubers" nil (hh uw1 b er0 z))
("hubert" nil (hh y uw1 b er0 t))
("huberta" nil (hh uw0 b eh1 r t ax))
("huberto" nil (hh uw0 b eh1 r t ow0))
("huberty" nil (hh ah1 b er0 t iy0))
("hubka" nil (hh ah1 b k ax))
("hubler" nil (hh y uw1 b ax l er0))
("hubley" nil (hh ah1 b l iy0))
("hubner" nil (hh ah1 b n er0))
("hubris" nil (hh y uw1 b r ax s))
("hubs" nil (hh ah1 b z))
("huch" nil (hh ah1 ch))
("huck" nil (hh ah1 k))
("huckaba" nil (hh ah1 k ax b ax))
("huckabay" nil (hh ah1 k ax b ey1))
("huckabee" nil (hh ah1 k ax b iy0))
("huckaby" nil (hh ah1 k ax b iy0))
("hucke" nil (hh ah1 k))
("huckeba" nil (hh ah1 k ih0 b ax))
("hucker" nil (hh ah1 k er0))
("huckins" nil (hh ah1 k ih0 n z))
("huckle" nil (hh ah1 k ax l))
("huckleberry" nil (hh ah1 k ax l b eh1 r iy0))
("hucks" nil (hh ah1 k s))
("huckstep" nil (hh ah1 k s t ih0 p))
("huckster" nil (hh ah1 k s t er0))
("hucksters" nil (hh ah1 k s t er0 z))
("hud" nil (hh ah1 d))
("hudak" nil (hh uw1 d ax k))
("huddle" nil (hh ah1 d ax l))
("huddled" nil (hh ah1 d ax l d))
("huddles" nil (hh ah1 d ax l z))
("huddleson" nil (hh ah1 d l ih0 s ax n))
("huddleston" nil (hh ah1 d ax l s t ax n))
("huddling" nil (hh ah1 d ax l ih0 ng))
("huddy" nil (hh ah1 d iy0))
("hudec" nil (hh uw1 d ih0 k))
("hudecek" nil (hh ah1 d ih0 ch eh0 k))
("hudek" nil (hh uw1 d ih0 k))
("hudelson" nil (hh ah1 d ih0 l s ax n))
("hudgens" nil (hh ah1 jh ax n z))
("hudgins" nil (hh ah1 jh ih0 n z))
("hudkins" nil (hh ah1 d k ih0 n z))
("hudler" nil (hh uw1 d ax l er0))
("hudlow" nil (hh ah1 d l ow0))
("hudman" nil (hh ah1 d m ax n))
("hudnall" nil (hh ah1 d n ax l))
("hudnell" nil (hh ah1 d n ax l))
("hudnut" nil (hh ah1 d n ah1 t))
("hudock" nil (hh ah1 d ax k))
("hudon" nil (hh uw1 d ax n))
("hudson" nil (hh ah1 d s ax n))
("hudspeth" nil (hh ah1 d s p ih0 th))
("hudwon" nil (hh ah1 d w ax n))
("hudy" nil (hh y uw1 d iy0))
("hudzik" nil (hh ah1 d z ih0 k))
("hue" nil (hh y uw1))
("huebel" nil (hh uh1 b ax l))
("hueber" nil (hh uh1 b er0))
("huebert" nil (hh uh1 b er0 t))
("huebner" nil (hh y uw1 b n er0))
("huebsch" nil (hh uh1 b sh))
("hued" nil (hh y uw1 d))
("huegel" nil (hh uh1 g ax l))
("hueglin" nil (hh y uw1 g l ih0 n))
("huelskamp" nil (hh uh1 l s k ae0 m p))
("huelsman" nil (hh uh1 l s m ax n))
("huelsmann" nil (hh uh1 l s m ax n))
("huenink" nil (hh uh1 n ih0 ng k))
("huerta" nil (hh w eh1 r t ah1))
("huerter" nil (hh er1 t er0))
("hues" nil (hh y uw1 z))
("huesman" nil (hh uh1 s m ax n))
("huestis" nil (hh uh1 s t ih0 s))
("hueston" nil (hh uh1 s t ax n))
("huether" nil (hh uh1 dh er0))
("huett" nil (hh uw1 t))
("huetta" nil (hh uw0 eh1 t ax))
("huette" nil (hh uw1 t))
("huettl" nil (hh uh1 t ax l))
("huettner" nil (hh uh1 t n er0))
("huey" nil (hh y uw1 iy0))
("hufbauer" nil (hh ah1 f b aw1 er0))
("huff" nil (hh ah1 f))
("huffaker" nil (hh ah1 f ax k er0))
("huffed" nil (hh ah1 f t))
("huffer" nil (hh ah1 f er0))
("huffine" nil (hh ah1 f ay1 n))
("huffines" nil (hh ah1 f ay1 n z))
("huffing" nil (hh ah1 f ih0 ng))
("huffington" nil (hh ah1 f ih0 ng t ax n))
("huffman" nil (hh ah1 f m ax n))
("huffmaster" nil (hh ah1 f m ae1 s t er0))
("hufford" nil (hh ah1 f er0 d))
("huffs" nil (hh ah1 f s))
("huffstetler" nil (hh ah1 f s t ih0 t ax l er0))
("huffstutler" nil (hh ah1 f s t uw0 t ax l er0))
("huffy" nil (hh ah1 f iy0))
("hufnagel" nil (hh ah1 f n ey1 g ax l))
("hufnagle" nil (hh ah1 f n ey1 g ax l))
("hufstedler" nil (hh ah1 f s t ih0 d ax l er0))
("hufstetler" nil (hh ah1 f s t ih0 t ax l er0))
("hug" nil (hh ah1 g))
("huge" nil (hh y uw1 jh))
("hugel" nil (hh uw1 g ax l))
("hugely" nil (hh y uw1 jh l iy0))
("hugest" nil (hh y uw1 jh ax s t))
("hugette" nil (hh ax zh eh1 t))
("hugg" nil (hh ah1 g))
("huggard" nil (hh ah1 g er0 d))
("hugged" nil (hh ah1 g d))
("hugger" nil (hh ah1 g er0))
("huggett" nil (hh ah1 g ih0 t))
("huggies" nil (hh ah1 g iy0 z))
("hugging" nil (hh ah1 g ih0 ng))
("huggins" nil (hh ah1 g ih0 n z))
("huggler" nil (hh ah1 g l er0))
("hugh" nil (hh y uw1))
("hughart" nil (hh ah1 g hh aa1 r t))
("hughbanks" nil (hh ah1 b ax ng k s))
("hughen" nil (hh y uw1 ax n))
("hughes" nil (hh y uw1 z))
("hughett" nil (hh y uw1 ih0 t))
("hughette" nil (hh y uw1 eh1 t))
("hughey" nil (hh ah1 g iy0))
("hughie" nil (hh y uw1 iy0))
("hughlett" nil (hh y uw1 l ih0 t))
("hughley" nil (hh ah1 g l iy0))
("hughs" nil (y uw1 z))
("hughson" nil (hh ah1 g s ax n))
("hughston" nil (hh ah1 g s t ax n))
("hughy" nil (hh y uw1 iy0))
("hugill" nil (hh ah1 jh ax l))
("hugley" nil (hh ah1 g l iy0))
("hugo" nil (hh y uw1 g ow0))
("hugoton" nil (hh y uw1 g ow0 t ax n))
("hugs" nil (hh ah1 g z))
("huguenin" nil (hh uw0 g ey0 n iy1 n))
("huguenot" nil (hh y uw1 g ax n aa1 t))
("huguenots" nil (hh y uw1 g ax n aa1 t s))
("huguet" nil (hh uw1 g ey0 t))
("huguley" nil (hh ah1 g y uw0 l iy0))
("hugus" nil (hh y uw1 g ax s))
("huh" nil (hh ah1))
("huhn" nil (hh ah1 n))
("huhta" nil (hh uw1 t ax))
("hui" nil (hh uw1 ih0))
("huibregtse" nil (hh uw1 b r eh0 k t s))
("huie" nil (hh y uw1 iy0))
("huish" nil (hh y uw1 ih0 sh))
("huisman" nil (hh uw1 s m ax n))
("huitt" nil (hh uw1 t))
("huizar" nil (hh ih0 z aa1 r))
("huizenga" nil (hh ih0 z ey1 ng g ax))
("huizinga" nil (hh ih0 z iy1 ng g ax))
("hukill" nil (hh y uw1 k ih0 l))
("hula" nil (hh uw1 l ax))
("hulbard" nil (hh ah1 l b er0 d))
("hulbert" nil (hh ah1 l b er0 t))
("hulburd" nil (hh ah1 l b er0 d))
("hulburt" nil (hh ah1 l b er0 t))
("hulce" nil (hh ah1 l s))
("huldie" nil (hh ah1 d iy0))
("huldy" nil (hh ah1 l d iy0))
("hulen" nil (hh ah1 l ax n))
("hulet" nil (hh uw1 l ih0 t))
("hulett" nil (hh y uw1 l ih0 t))
("hulette" nil (hh y uw1 l eh1 t))
("hulgan" nil (hh ah1 l g ax n))
("hulick" nil (hh y uw1 l ih0 k))
("hulin" nil (hh y uw1 l ih0 n))
("huling" nil (hh y uw1 l ih0 ng))
("hulings" nil (hh y uw1 l ih0 ng z))
("hulk" nil (hh ah1 l k))
("hulking" nil (hh ah1 l k ih0 ng))
("hulks" nil (hh ah1 l k s))
("hull" nil (hh ah1 l))
("hullabaloo" nil (hh ah1 l ax b ax l uw1))
("hulled" nil (hh ah1 l d))
("hullender" nil (hh uw1 l eh0 n d er0))
("hullett" nil (hh uw1 l ih0 t))
("hullinger" nil (hh ah1 l ih0 ng er0))
("hulls" nil (hh ah1 l z))
("hullum" nil (hh ah1 l ax m))
("hulme" nil (hh ah1 l m))
("hulon" nil (hh y uw1 l ax n))
("hulond" nil (hh y uw1 l ax n d))
("huls" nil (hh ah1 l z))
("hulse" nil (hh ah1 l s))
("hulsebus" nil (hh ah1 l s ih0 b ih0 s))
("hulsey" nil (hh ah1 l s iy0))
("hulsizer" nil (hh ah1 l s ay1 z er0))
("hulslander" nil (hh ah1 l s l ax n d er0))
("hulsman" nil (hh ah1 l s m ax n))
("hulst" nil (hh ah1 l s t))
("hult" nil (hh ah1 l t))
("hultberg" nil (hh ah1 l t b er0 g))
("hultgren" nil (hh ah1 l t g r eh0 n))
("hultman" nil (hh ah1 l t m ax n))
("hulton" nil (hh ah1 l t ax n))
("hultquist" nil (hh ah1 l t k w ih0 s t))
("hults" nil (hh ah1 l t s))
("hultz" nil (hh ah1 l t s))
("hulvey" nil (hh ah1 l v iy0))
("hum" nil (hh ah1 m))
("human" nil (hh y uw1 m ax n))
("humana" nil (hh y uw0 m ae1 n ax))
("humane" nil (hh y uw0 m ey1 n))
("humanism" nil (hh y uw1 m ax n ih1 z ax m))
("humanist" nil (hh y uw1 m ax n ih0 s t))
("humanistic" nil (hh y uw1 m ax n ih1 s t ih0 k))
("humanists" nil (hh y uw1 m ax n ax s t s))
("humanitarian" nil (hh y uw1 m ae1 n ax t eh1 r iy0 ax n))
("humanities" nil (hh y uw0 m ae1 n ih0 t iy0 z))
("humanity" nil (hh y uw0 m ae1 n ih0 t iy0))
("humanize" nil (hh y uw1 m ax n ay1 z))
("humanizing" nil (hh y uw1 m ax n ay1 z ih0 ng))
("humankind" nil (hh y uw1 m ax n k ay1 n d))
("humanly" nil (hh y uw1 m ax n l iy0))
("humann" nil (hh y uw1 m ax n))
("humanness" nil (hh y uw1 m ax n n ax s))
("humans" nil (hh y uw1 m ax n z))
("humbarger" nil (hh ah1 m b aa1 r g er0))
("humber" nil (hh ah1 m b er0))
("humbert" nil (hh ah1 m b er0 t))
("humberto" nil (hh ax m b er1 t ow1))
("humble" nil (hh ah1 m b ax l))
("humbled" nil (hh ah1 m b ax l d))
("humbler" nil (hh ah1 m b ax l er0))
("humbles" nil (hh ah1 m b ax l z))
("humblest" nil (hh ah1 m b ax l ax s t))
("humbling" nil (hh ah1 m b ax l ih0 ng))
("humbly" nil (hh ah1 m b l iy0))
("humboldt" nil (hh ah1 m b ow1 l t))
("humbug" nil (hh ah1 m b ah1 g))
("humburg" nil (hh ah1 m b er0 g))
("humdinger" nil (hh ah1 m d ih0 ng er0))
("humdrum" nil (hh ah1 m d r ah1 m))
("hume" nil (hh y uw1 m))
("humenik" nil (hh y uw1 m ih0 n ih0 k))
("humerous" nil (hh y uw1 m er0 ax s))
("humerus" nil (hh y uw1 m er0 ax s))
("humes" nil (hh y uw1 m z))
("humfrey" nil (hh ah1 m f r iy0))
("humfry" nil (hh ah1 m f er0 iy0))
("humi" nil (hh y uw1 m iy0))
("humid" nil (hh y uw1 m ax d))
("humidifiers" nil (hh y uw0 m ih1 d ax f ay1 er0 z))
("humidity" nil (hh y uw0 m ih1 d ax t iy0))
("humiliate" nil (hh y uw0 m ih1 l iy0 ey1 t))
("humiliated" nil (hh y uw0 m ih1 l iy0 ey1 t ih0 d))
("humiliating" nil (hh y uw0 m ih1 l iy0 ey1 t ih0 ng))
("humiliation" nil (hh y uw0 m ih1 l iy0 ey1 sh ax n))
("humiliations" nil (hh y uw1 m ih1 l iy0 ey1 sh ax n z))
("humility" nil (hh y uw0 m ih1 l ih0 t iy0))
("humiston" nil (hh y uw1 m ih0 s t aa0 n))
("humke" nil (hh ah1 m k iy0))
("huml" nil (hh ah1 m ax l))
("humm" nil (hh ah1 m))
("hummel" nil (hh ah1 m ax l))
("hummell" nil (hh ah1 m ax l))
("hummer" nil (hh ah1 m er0))
("humming" nil (hh ah1 m ih0 ng))
("hummingbird" nil (hh ah1 m ih0 ng b er1 d))
("hummingbirds" nil (hh ah1 m ih0 ng b er1 d z))
("humongous" nil (hh y uw0 m ao1 ng g ax s))
("humor" nil (hh y uw1 m er0))
("humoral" nil (hh y uw1 m er0 ax l))
("humored" nil (hh y uw1 m er0 d))
("humorist" nil (hh y uw1 m er0 ax s t))
("humorists" nil (hh y uw1 m er0 ih0 s t s))
("humorless" nil (hh y uw1 m er0 l ax s))
("humorous" nil (hh y uw1 m er0 ax s))
("hump" nil (hh ah1 m p))
("humpal" nil (hh ah1 m p ax l))
("humpback" nil (hh ah1 m p b ae1 k))
("humped" nil (hh ah1 m p t))
("humpert" nil (hh ah1 m p er1 t))
("humph" nil (hh ah1 m f))
("humphery" nil (hh ah1 m f er0 iy0))
("humpherys" nil (hh ah1 m f er0 iy0 z))
("humphrey" nil (hh ah1 m f r iy0))
("humphreys" nil (hh ah1 m f r iy0 z))
("humphries" nil (hh ah1 m f er0 iy0 z))
("humphry" nil (hh ah1 m f r iy0))
("humpty" nil (hh ah1 m p t iy0))
("hums" nil (hh ah1 m z))
("humulin" nil (hh y uw1 m y uw1 l ih0 n))
("humus" nil (hh y uw1 m ax s))
("humvee" nil (hh ah1 m v iy1))
("humvees" nil (hh ah1 m v iy1 z))
("hun" nil (hh ah1 n))
("hunan" nil (hh uw1 n aa0 n))
("hunch" nil (hh ah1 n ch))
("hunchback" nil (hh ah1 n ch b ae1 k))
("hunched" nil (hh ah1 n ch t))
("hunches" nil (hh ah1 n ch ih0 z))
("hund" nil (hh ah1 n d))
("hundertmark" nil (hh ah1 n d er0 t m aa1 r k))
("hundley" nil (hh ah1 n d l iy0))
("hundred" nil (hh ah1 n d r ax d))
("hundreds" nil (hh ah1 n d r ax d z))
("hundredth" nil (hh ah1 n d r ax d th))
("hundredths" nil (hh ah1 n d r ax d th s))
("hundredweight" nil (hh ah1 n d r ax d w ey1 t))
("hundt" nil (hh ah1 n t))
("huneke" nil (hh ah1 n ih0 k))
("huneycutt" nil (hh ah1 n iy0 k ax t))
("hung" nil (hh ah1 ng))
("hungarian" nil (hh ax ng g eh1 r iy0 ax n))
("hungarians" nil (hh ax ng g eh1 r iy0 ax n z))
("hungary" nil (hh ah1 ng g er0 iy0))
("hungate" nil (hh ah1 ng ey0 t))
("hunger" nil (hh ah1 ng g er0))
("hungerford" nil (hh ah1 ng g er0 f er0 d))
("hungerfords" nil (hh ah1 ng g er0 f er0 d z))
("hungering" nil (hh ah1 ng g er0 ih0 ng))
("hungrier" nil (hh ah1 ng g r iy0 er0))
("hungrily" nil (hh ah1 ng g r ax l iy0))
("hungry" nil (hh ah1 ng g r iy0))
("hunk" nil (hh ah1 ng k))
("hunke" nil (hh ah1 ng k))
("hunkele" nil (hh ah1 ng k ax l))
("hunker" nil (hh ah1 ng k er0))
("hunkered" nil (hh ah1 ng k er0 d))
("hunkering" nil (hh ah1 ng k er0 ih0 ng))
("hunkins" nil (hh ah1 ng k ih0 n z))
("hunks" nil (hh ah1 ng k s))
("hunky" nil (hh ah1 ng k iy0))
("hunley" nil (hh ah1 n l iy0))
("hunn" nil (hh ah1 n))
("hunnell" nil (hh ah1 n ax l))
("hunnewell" nil (hh ah1 n ih0 w eh0 l))
("hunnicutt" nil (hh ah1 n ih0 k ax t))
("hunsaker" nil (hh ah1 n s ax k er0))
("hunsberger" nil (hh ah1 n s b er0 g er0))
("hunsicker" nil (hh ah1 n s ih0 k er0))
("hunsinger" nil (hh ah1 n s ih0 ng er0))
("hunsley" nil (hh ah1 n s l iy0))
("hunsucker" nil (hh ah1 n s ax k er0))
("hunt" nil (hh ah1 n t))
("huntco" nil (hh ah1 n t k ow0))
("hunte" nil (hh ah1 n t))
("hunted" nil (hh ah1 n t ax d))
("hunter" nil (hh ah1 n t er0))
("hunters" nil (hh ah1 n t er0 z))
("hunting" nil (hh ah1 n t ih0 ng))
("huntingdon" nil (hh ah1 n t ih0 ng d ih0 n))
("huntington" nil (hh ah1 n t ih0 ng t ax n))
("huntley" nil (hh ah1 n t l iy0))
("hunton" nil (hh ah1 n t ax n))
("huntoon" nil (hh ax n t uw1 n))
("huntress" nil (hh ah1 n t r ih0 s))
("hunts" nil (hh ah1 n t s))
("huntsinger" nil (hh ah1 n t s ih0 n jh er0))
("huntsman" nil (hh ah1 n t s m ax n))
("huntsville" nil (hh ah1 n t s v ih0 l))
("huntway" nil (hh ah1 n t w ey1))
("huntzinger" nil (hh ah1 n t z ih0 ng er0))
("hunza" nil (hh ah1 n z ax))
("hunzeker" nil (hh ah1 n z ih0 k er0))
("hunziker" nil (hh ah1 n z ih0 k er0))
("huot" nil (hh y uw1 ax t))
("hupe" nil (hh y uw1 p))
("hupfer" nil (hh ah1 p f er0))
("hupp" nil (hh ah1 p))
("huppert" nil (hh ah1 p er0 t))
("hur" nil (hh er1))
("hurco" nil (hh er1 k ow1))
("hurd" nil (hh er1 d))
("hurdle" nil (hh er1 d ax l))
("hurdler" nil (hh er1 d ax l er0))
("hurdles" nil (hh er1 d ax l z))
("hurdling" nil (hh er1 d ax l ih0 ng))
("hurdman" nil (hh er1 d m ax n))
("hurford" nil (hh er1 f er0 d))
("hurl" nil (hh er1 l))
("hurlbert" nil (hh er1 l b er0 t))
("hurlburt" nil (hh er1 l b er0 t))
("hurlbut" nil (hh er1 l b ax t))
("hurlbutt" nil (hh er1 l b ax t))
("hurled" nil (hh er1 l d))
("hurless" nil (hh er1 l ax s))
("hurley" nil (hh er1 l iy0))
("hurling" nil (hh er1 l ih0 ng))
("hurlock" nil (hh er1 l ax k))
("hurm" nil (hh er1 m))
("hurn" nil (hh er1 n))
("hurney" nil (hh er1 n iy0))
("huron" nil (hh y uw1 r aa1 n))
("hurrah" nil (hh uh0 r aa1))
("hurrell" nil (hh ao1 r ax l))
("hurricane" nil (hh er1 ax k ey1 n))
("hurricanes" nil (hh er1 ax k ey1 n z))
("hurried" nil (hh er1 iy0 d))
("hurriedly" nil (hh er1 iy0 d l iy0))
("hurries" nil (hh er1 iy0 z))
("hurry" nil (hh er1 iy0))
("hurrying" nil (hh er1 iy0 ih0 ng))
("hursey" nil (hh er1 s iy0))
("hursh" nil (hh er1 sh))
("hurst" nil (hh er1 s t))
("hurston" nil (hh er1 s t ax n))
("hurt" nil (hh er1 t))
("hurta" nil (hh er1 t ax))
("hurtado" nil (hh er0 t aa1 d ow0))
("hurteau" nil (hh er0 t ow1))
("hurtful" nil (hh er1 t f ax l))
("hurtig" nil (hh er1 t ih0 g))
("hurting" nil (hh er1 t ih0 ng))
("hurtle" nil (hh er1 t ax l))
("hurtling" nil (hh er1 t l ih0 ng))
("hurts" nil (hh er1 t s))
("hurtt" nil (hh er1 t))
("hurtubise" nil (hh er1 t ax b ay0 z))
("hurvitz" nil (hh er1 v ih0 t s))
("hurwitz" nil (hh er1 w ih0 t s))
("husain" nil (hh ah1 s ay0 n))
("husak" nil (hh uw1 s ax k))
("husar" nil (hh uw1 s er0))
("husband" nil (hh ah1 z b ax n d))
("husbandry" nil (hh ah1 z b ax n d r iy0))
("husbands" nil (hh ah1 z b ax n d z))
("husby" nil (hh ah1 s b iy0))
("huschka" nil (hh ah1 sh k ax))
("huse" nil (hh y uw1 z))
("huseby" nil (hh ah1 s ih0 b iy0))
("huseman" nil (hh uw1 s m ax n))
("husen" nil (hh uw1 s ax n))
("huser" nil (hh y uw1 z er0))
("hush" nil (hh ah1 sh))
("hushed" nil (hh ah1 sh t))
("husic" nil (hh y uw1 z ih0 k))
("husk" nil (hh ah1 s k))
("huska" nil (hh ah1 s k ax))
("huskey" nil (hh ah1 s k iy0))
("huskins" nil (hh ah1 s k ih0 n z))
("husks" nil (hh ah1 s k s))
("husky" nil (hh ah1 s k iy0))
("husman" nil (hh ah1 s m ax n))
("husmann" nil (hh ah1 s m ax n))
("huson" nil (hh uw1 s ax n))
("huss" nil (hh ah1 s))
("hussain" nil (hh uw0 s ey1 n))
("hussar" nil (hh ah1 s er0))
("hussein" nil (hh uw0 s ey1 n))
("husseini" nil (hh y uw0 s ey1 n iy0))
("husser" nil (hh ah1 s er0))
("hussey" nil (hh ah1 s iy0))
("hussite" nil (hh ah1 s ay1 t))
("hussman" nil (hh ah1 s m ax n))
("husson" nil (hh ah1 s ax n))
("hussong" nil (hh ah1 s ao1 ng))
("hussy" nil (hh ah1 s iy0))
("hust" nil (hh ah1 s t))
("hustad" nil (hh ah1 s t ax d))
("hustead" nil (hh ah1 s t eh0 d))
("husted" nil (hh ah1 s t ih0 d))
("huster" nil (hh ah1 s t er0))
("hustings" nil (hh ah1 s t ih0 ng z))
("hustle" nil (hh ah1 s ax l))
("hustled" nil (hh ah1 s ax l d))
("hustler" nil (hh ah1 s ax l er0))
("hustlers" nil (hh ah1 s ax l er0 z))
("hustles" nil (hh ah1 s ax l z))
("hustling" nil (hh ah1 s ax l ih0 ng))
("huston" nil (hh ah1 s t ax n))
("hut" nil (hh ah1 t))
("huta" nil (hh uw1 t ax))
("hutch" nil (hh ah1 ch))
("hutchcraft" nil (hh ah1 ch k r ae1 f t))
("hutchens" nil (hh ah1 ch ax n z))
("hutcheon" nil (hh ah1 ch iy0 ax n))
("hutcherson" nil (hh ah1 ch er0 s ax n))
("hutcheson" nil (hh ah1 ch ih0 s ax n))
("hutchings" nil (hh ah1 ch ih0 ng z))
("hutchins" nil (hh ah1 t ch ih1 n z))
("hutchinson" nil (hh ah1 ch ih0 n s ax n))
("hutchison" nil (hh ah1 ch ih0 s ax n))
("huth" nil (hh uw1 th))
("hutmacher" nil (hh ah1 t m ax k er0))
("hutnick" nil (hh ah1 t n ih0 k))
("huto" nil (hh uw1 t ow1))
("huts" nil (hh ah1 t s))
("hutsell" nil (hh ah1 t s ax l))
("hutson" nil (hh ah1 t s ax n))
("hutt" nil (hh ah1 t))
("hutter" nil (hh ah1 t er0))
("huttner" nil (hh ah1 t n er0))
("hutto" nil (hh uw1 t ow0))
("hutton" nil (hh ah1 t ax n))
("hutu" nil (hh uw1 t uw1))
("hutus" nil (hh uw1 t uw1 z))
("hutzel" nil (hh ah1 t z ax l))
("hutzell" nil (hh ah1 t z ax l))
("hutzelman" nil (hh ah1 t s ax l m ax n))
("hutzler" nil (hh ah1 t s l er0))
("huval" nil (hh uw0 v ae1 l))
("huwe" nil (hh uw1 w))
("hux" nil (hh ah1 k s))
("huxford" nil (hh ah1 k s f er0 d))
("huxley" nil (hh ah1 k s l iy0))
("huxtable" nil (hh ah1 k s t ax b ax l))
("huxtables" nil (hh ah1 k s t ax b ax l z))
("huyck" nil (hh ay1 k))
("huyett" nil (hh ay1 ih0 t))
("huyler" nil (hh ay1 l er0))
("huynh" nil (hh ay1 n))
("huyser" nil (hh ay1 s er0))
("hwa" nil (hh w aa1))
("hwan" nil (hh w aa1 n))
("hwang" nil (hh w ae1 ng))
("hy" nil (hh ay1))
("hyacinth" nil (hh ay1 ax s ih1 n th))
("hyacintha" nil (hh ay1 ax s ih1 n th ax))
("hyacinthe" nil (hh ay1 ax s ih0 n th))
("hyacinthia" nil (hh ay1 ax s ih1 n th iy0 ax))
("hyacinthie" nil (hh ay1 ax s ih1 n th iy0))
("hyacinths" nil (hh ay1 ax s ih0 n th s))
("hyades" nil (hh ay1 ax d iy1 z))
("hyaluronic" nil (hh ay1 ax l er0 aa1 n ih0 k))
("hyams" nil (hh ay1 ax m z))
("hyannis" nil (hh ay0 ae1 n ih0 s))
("hyatt" nil (hh ay1 ax t))
("hybl" nil (hh ih1 b ax l))
("hybrid" nil (hh ay1 b r ax d))
("hybridization" nil (hh ay1 b r ax d ax z ey1 sh ax n))
("hybridize" nil (hh ay1 b r ax d ay1 z))
("hybrids" nil (hh ay1 b r ax d z))
("hybritech" nil (hh ay1 b r ih0 t eh1 k))
("hyche" nil (hh ay1 ch))
("hycor" nil (hh ay1 k ao1 r))
("hycroft" nil (hh ay1 k r ao1 f t))
("hyde" nil (hh ay1 d))
("hyden" nil (hh ay1 d ax n))
("hyder" nil (hh ay1 d er0))
("hydock" nil (hh ay1 d ax k))
("hydra" nil (hh ay1 d r ax))
("hydrant" nil (hh ay1 d r ax n t))
("hydras" nil (hh ay1 d r ax z))
("hydrate" nil (hh ay1 d r ey1 t))
("hydrated" nil (hh ay1 d r ey1 t ax d))
("hydration" nil (hh ay0 d r ey1 sh ax n))
("hydraulic" nil (hh ay0 d r ao1 l ih0 k))
("hydraulics" nil (hh ay0 d r ao1 l ih0 k s))
("hydrazine" nil (hh ay1 d r ax z iy1 n))
("hydrick" nil (hh ih1 d r ih0 k))
("hydride" nil (hh ay1 d r ay1 d))
("hydro" nil (hh ay1 d r ow1))
("hydrocarbon" nil (hh ay1 d r ow0 k aa1 r b ax n))
("hydrocarbons" nil (hh ay1 d r ow0 k aa1 r b ax n z))
("hydroelectric" nil (hh ay1 d r ow0 ih0 l eh1 k t r ih0 k))
("hydrofoil" nil (hh ay1 d r ax f oy1 l))
("hydrogen" nil (hh ay1 d r ax jh ax n))
("hydrogenate" nil (hh ay1 d r ax jh ax n ey1 t))
("hydrogenated" nil (hh ay1 d r ax jh ax n ey1 t ih0 d))
("hydrogenates" nil (hh ay1 d r ax jh ax n ey1 t s))
("hydrogenating" nil (hh ay1 d r ax jh ax n ey1 t ih0 ng))
("hydrogenation" nil (hh ay1 d r aa1 jh ax n ey1 sh ax n))
("hydrogens" nil (hh ay1 d r ax jh ax n z))
("hydrographic" nil (hh ay1 d r ax g r ae1 f ih0 k))
("hydrolysis" nil (hh ay0 d r aa1 l ax s ax s))
("hydrolyzed" nil (hh ay1 d r ax l ay1 z d))
("hydrolyzing" nil (hh ay1 d r ax l ay1 z ih0 ng))
("hydrometer" nil (hh ay0 d r aa1 m ax t er0))
("hydron" nil (hh ay1 d r ax n))
("hydrophilic" nil (hh ay1 d r ax f ih1 l ih0 k))
("hydroponic" nil (hh ay1 d r ax p aa1 n ih0 k))
("hydropower" nil (hh ay1 d r ow0 p aw1 r))
("hydrosol" nil (hh ay1 d r ax s aa1 l))
("hydrosulfide" nil (hh ay1 d r ow0 s ah1 l f ay1 d))
("hydrotherapy" nil (hh ay1 d r ow0 th eh1 r ax p iy0))
("hydrothermal" nil (hh ay1 d r ow0 th er1 m ax l))
("hydrous" nil (hh ay1 d r ax s))
("hydroxide" nil (hh ay0 d r aa1 k s ay0 d))
("hydroxides" nil (hh ay0 d r aa1 k s ay0 d z))
("hyena" nil (hh ay0 iy1 n ax))
("hyenas" nil (hh ay0 iy1 n ax z))
("hyer" nil (hh ay1 er0))
("hyers" nil (hh ay1 er0 z))
("hygeia" nil (hh ay1 jh ey1 ax))
("hygiene" nil (hh ay1 jh iy1 n))
("hygienist" nil (hh ay1 g iy1 n ih0 s t))
("hygienists" nil (hh ay1 g iy1 n ih0 s t s))
("hygrometer" nil (hh ay0 g r aa1 m ax t er0))
("hykes" nil (hh ay1 k s))
("hyland" nil (hh ay1 l ax n d))
("hyle" nil (hh ay1 l))
("hyler" nil (hh ay1 l er0))
("hylsa" nil (hh ay1 l s ax))
("hyman" nil (hh ay1 m ax n))
("hymans" nil (hh ay1 m ax n z))
("hymas" nil (hh ay1 m ax z))
("hymel" nil (hh ay1 m ax l))
("hymer" nil (hh ay1 m er0))
("hymes" nil (hh ay1 m z))
("hymn" nil (hh ih1 m))
("hymnals" nil (hh ih1 m n ax l z))
("hymnology" nil (hh ih0 m n aa1 l ax jh iy0))
("hymns" nil (hh ih1 m z))
("hymowitz" nil (hh ih1 m ax w ih0 t s))
("hynd" nil (hh ih1 n d))
("hyndman" nil (hh ih1 n d m ax n))
("hynds" nil (hh ih1 n d z))
("hynek" nil (hh ay1 n ih0 k))
("hynes" nil (hh ay1 n z))
("hynson" nil (hh ih1 n s ax n))
("hyogo" nil (hh y ow1 g ow0))
("hyotan" nil (hh y ow1 t ae1 n))
("hype" nil (hh ay1 p))
("hyped" nil (hh ay1 p t))
("hyper" nil (hh ay1 p er0))
("hyperactive" nil (hh ay1 p er0 ae1 k t ih0 v))
("hyperactivity" nil (hh ay1 p er0 ae0 k t ih1 v ih0 t iy0))
("hyperbola" nil (hh ay0 p er1 b ax l ax))
("hyperbole" nil (hh ay0 p er1 b ax l iy1))
("hyperbolic" nil (hh ay1 p er0 b aa1 l ih0 k))
("hyperborean" nil (hh ay1 p er0 b ao1 r iy0 ax n))
("hypercard" nil (hh ay1 p er0 k aa1 r d))
("hypercritical" nil (hh ay1 p er0 k r ih1 t ih0 k ax l))
("hyperinflation" nil (hh ay1 p er0 ih0 n f l ey1 sh ax n))
("hyperion" nil (hh ay0 p ih1 r iy0 ax n))
("hyperkinetic" nil (hh ay1 p er0 k ih0 n eh1 t ih0 k))
("hypermarket" nil (hh ay1 p er0 m aa1 r k ih0 t))
("hypermarkets" nil (hh ay1 p er0 m aa1 r k ih0 t s))
("hyperopia" nil (hh ay1 p er0 ow1 p iy0 ax))
("hypersensitive" nil (hh ay1 p er0 s eh1 n s ih0 t ih0 v))
("hypersonic" nil (hh ay1 p er0 s aa1 n ih0 k))
("hypertension" nil (hh ay1 p er0 t eh1 n sh ax n))
("hypertensive" nil (hh ay1 p er0 t eh1 n s ih0 v))
("hypertext" nil (hh ay1 p er0 t eh1 k s t))
("hypertonic" nil (hh ay1 p er0 t aa1 n ih0 k))
("hypes" nil (hh ay1 p s))
("hyphae" nil (hh ay1 f iy1))
("hyphen" nil (hh ay1 f ax n))
("hyphenated" nil (hh ay1 f ax n ey1 t ih0 d))
("hyping" nil (hh ay1 p ih0 ng))
("hypnosis" nil (hh ih0 p n ow1 s ax s))
("hypnotic" nil (hh ih0 p n aa1 t ih0 k))
("hypnotics" nil (hh ih0 p n aa1 t ih0 k s))
("hypnotism" nil (hh ih1 p n ax t ih1 z ax m))
("hypnotized" nil (hh ih1 p n ax t ay1 z d))
("hypo" nil (hh ay1 p ow0))
("hypochondria" nil (hh ay1 p ax k aa1 n d r iy0 ax))
("hypocrisy" nil (hh ih0 p aa1 k r ax s iy0))
("hypocrite" nil (hh ih1 p ax k r ih1 t))
("hypocrites" nil (hh ih1 p ax k r ih1 t s))
("hypocritical" nil (hh ih1 p ax k r ih1 t ih0 k ax l))
("hypodermic" nil (hh ay1 p ax d er1 m ih0 k))
("hypoglycemia" nil (hh ay1 p ow0 g l ay0 s iy1 m iy0 ax))
("hypoglycemic" nil (hh ay1 p ow0 g l ay0 s iy1 m ih0 k))
("hypolite" nil (hh ay1 p ax l ay0 t))
("hyponex" nil (hh ay1 p ow0 n eh1 k s))
("hypotension" nil (hh ay1 p ow0 t eh1 n sh ax n))
("hypothalamic" nil (hh ay1 p ow0 th ax l ae1 m ih0 k))
("hypotheken" nil (hh ay1 p aa1 th ax k ax n))
("hypothermia" nil (hh ay1 p ax th er1 m iy0 ax))
("hypotheses" nil (hh ay0 p aa1 th ax s iy1 z))
("hypothesis" nil (hh ay0 p aa1 th ax s ax s))
("hypothesize" nil (hh ay0 p aa1 th ax s ay1 z))
("hypothesized" nil (hh ay0 p aa1 th ax s ay1 z d))
("hypothetical" nil (hh ay1 p ax th eh1 t ax k ax l))
("hypothetically" nil (hh ay1 p ax th eh1 t ih0 k l iy0))
("hypoxia" nil (hh ay0 p aa1 k s iy0 ax))
("hyraxes" nil (hh ay1 r ae0 k s ax z))
("hyre" nil (hh ay1 r))
("hysell" nil (hh ay1 s ax l))
("hyser" nil (hh ay1 z er0))
("hyslop" nil (hh ay1 s l ax p))
("hyson" nil (hh ay1 s ax n))
("hysong" nil (hh ay1 s ao1 ng))
("hyssop" nil (hh ih1 s ax p))
("hyster" nil (hh ih1 s t er0))
("hysterectomy" nil (hh ih1 s t er0 eh1 k t ax m iy0))
("hysteria" nil (hh ih0 s t eh1 r iy0 ax))
("hysterical" nil (hh ih0 s t eh1 r ih0 k ax l))
("hysterically" nil (hh ih1 s t eh1 r ih0 k l iy0))
("hyun" nil (hh ay1 ax n))
("hyundai" nil (hh y ah1 n d ey1))
("hyundais" nil (hh y ah1 n d ey1 z))
("i" nil (ay1))
("ia" nil (iy1 ax))
("iacobelli" nil (iy0 aa1 k ow0 b eh1 l iy0))
("iacobellis" nil (iy0 aa1 k ow0 b eh1 l ih0 s))
("iacobucci" nil (iy0 aa0 k ow0 b uw1 ch iy0))
("iacocca" nil (ay1 ax k ow1 k ax))
("iacona" nil (ay1 ax k ow1 n ax))
("iacono" nil (ay1 ax k ow1 n ow0))
("iacovelli" nil (iy0 aa1 k ow0 v eh1 l iy0))
("iafrate" nil (ay1 ax f r ey1 t iy0))
("iago" nil (iy0 aa1 g ow0))
("iain" nil (iy0 ey1 n))
("iakovos" nil (iy0 aa1 k ow0 v ow0 s))
("iams" nil (iy1 aa0 m z))
("ian" nil (iy1 ax n))
("iannaccone" nil (iy0 aa1 n ax k ow1 n iy0))
("iannacone" nil (iy0 aa1 n ax k ow1 n iy0))
("iannamico" nil (iy0 aa1 n ax m iy1 k ow0))
("iannelli" nil (iy0 ax n eh1 l iy0))
("iannello" nil (iy0 ax n eh1 l ow0))
("ianni" nil (iy0 aa1 n iy0))
("ianniello" nil (iy0 aa1 n iy0 eh1 l ow0))
("iannone" nil (iy0 ax n ow1 n iy0))
("iannotti" nil (iy0 ax n ow1 t iy0))
("iannucci" nil (iy0 ax n uw1 ch iy0))
("iannuzzi" nil (iy0 ax n uw1 t s iy0))
("iantha" nil (iy0 aa1 n th ax))
("ianthe" nil (iy0 aa1 n th ey0))
("ianthina" nil (iy0 ax n th iy1 n ax))
("iason" nil (iy1 ax s ax n))
("iavarone" nil (iy0 aa1 v er0 ow1 n iy0))
("ib" nil (ih1 b))
("ibach" nil (ih1 b aa0 k))
("ibanez" nil (ih0 b aa1 n eh0 z))
("ibaraki" nil (ay1 b er0 aa1 k iy0))
("ibarra" nil (ih0 b aa1 r ax))
("ibbotson" nil (ih1 b ax t s ax n))
("iberia" nil (ay0 b ih1 r iy0 ax))
("iberian" nil (ay0 b ih1 r iy0 ax n))
("ibex" nil (ay1 b eh0 k s))
("ibis" nil (ay1 b ax s))
("ibmer" nil (ih1 b m er0))
("ibmers" nil (ih1 b m er0 z))
("ibn" nil (ih1 b ax n))
("ibrahim" nil (ih1 b r aa0 hh iy1 m))
("ibsen" nil (ih1 b s ax n))
("ibuprofen" nil (ay1 b y uw0 p r ow1 f ax n))
("icahn" nil (ay1 k aa0 n))
("icard" nil (ih0 k aa1 r d))
("icarus" nil (ih1 k er1 ax s))
("ice" nil (ay1 s))
("iceberg" nil (ay1 s b er0 g))
("icebergs" nil (ay1 s b er0 g z))
("icebox" nil (ay1 s b aa1 k s))
("icebreaker" nil (ay1 s b r ey1 k er0))
("icebreakers" nil (ay1 s b r ey1 k er0 z))
("iced" nil (ay1 s t))
("iceland" nil (ay1 s l ax n d))
("icelandair" nil (ay1 s l ae0 n d eh1 r))
("icelandic" nil (ay0 s l ae1 n d ih0 k))
("icenhour" nil (ih1 s ax n aw0 r))
("icenhower" nil (ih1 s ax n hh aw1 er0))
("icenogle" nil (ih1 s ax n ow1 g ax l))
("ices" nil (ay1 s ax z))
("iceskating" nil (ay1 s s k ey1 t ih0 ng))
("ich" nil (ih1 ch))
("ichi" nil (iy1 ch iy0))
("ichikawa" nil (ih0 ch iy0 k aa1 w ax))
("ichiro" nil (iy1 ch ih0 r ow0))
("icicles" nil (ay1 s ih0 k ax l z))
("icing" nil (ay1 s ih0 ng))
("ickes" nil (ih1 k s))
("icon" nil (ay1 k aa0 n))
("iconoclasm" nil (ay1 k aa1 n ax k l ae1 z ax m))
("iconoclast" nil (ay1 k aa1 n ax k l ae1 s t))
("iconoclastic" nil (ay1 k ax n ax k l ae1 s t ih0 k))
("iconography" nil (ay1 k ax n aa1 g r ax f iy0))
("icons" nil (ay1 k aa1 n z))
("icy" nil (ay1 s iy0))
("id" nil (ih1 d))
("ida" nil (ay1 d ax))
("idaho" nil (ay1 d ax hh ow1))
("idalia" nil (ih0 d aa1 l y ax))
("idalina" nil (ih0 d aa0 l iy1 n ax))
("idaline" nil (ih1 d ax l ay0 n))
("iddings" nil (ih1 d ih0 ng z))
("ide" nil (ay1 d))
("idea" nil (ay0 d iy1 ax))
("ideal" nil (ay0 d iy1 l))
("idealism" nil (ay0 d iy1 l ih0 z ax m))
("idealist" nil (ay0 d iy1 l ih0 s t))
("idealistic" nil (ay0 d iy1 ax l ih1 s t ih0 k))
("idealists" nil (ay0 d iy1 l ih0 s t s))
("idealize" nil (ay0 d iy1 l ay1 z))
("idealized" nil (ay0 d iy1 ax l ay1 z d))
("ideally" nil (ay0 d iy1 l iy0))
("ideals" nil (ay0 d iy1 l z))
("ideas" nil (ay0 d iy1 ax z))
("idec" nil (ay1 d ax k))
("iden" nil (ay1 d ax n))
("identa" nil (ay0 d eh1 n t ax))
("identical" nil (ay0 d eh1 n t ih0 k ax l))
("identics" nil (ay0 d eh1 n t ih0 k s))
("identifiable" nil (ay0 d eh1 n t ax f ay1 ax b ax l))
("identification" nil (ay0 d eh1 n t ax f ax k ey1 sh ax n))
("identifications" nil (ay0 d eh1 n t ax f ax k ey1 sh ax n z))
("identified" nil (ay0 d eh1 n t ax f ay1 d))
("identifies" nil (ay0 d eh1 n t ax f ay1 z))
("identify" nil (ay0 d eh1 n t ax f ay1))
("identifying" nil (ay0 d eh1 n t ax f ay1 ih0 ng))
("identities" nil (ay0 d eh1 n t ih0 t iy0 z))
("identity" nil (ay0 d eh1 n t ax t iy0))
("ideological" nil (ay1 d iy0 ax l aa1 jh ih0 k ax l))
("ideologically" nil (ay1 d iy0 ax l aa1 jh ih0 k l iy0))
("ideologies" nil (ay1 d iy0 aa1 l ax jh iy0 z))
("ideologist" nil (ay1 d iy0 aa1 l ax jh ax s t))
("ideologists" nil (ay1 d iy0 aa1 l ax jh ih0 s t s))
("ideologue" nil (ay1 d iy0 ax l ow0 g))
("ideologues" nil (ay1 d iy0 ax l ao0 g z))
("ideology" nil (ay1 d iy0 aa1 l ax jh iy0))
("ideonomy" nil (ay1 d iy0 aa1 n ax m iy0))
("idette" nil (ay1 d eh1 t))
("idiocy" nil (ih1 d iy0 ax s iy0))
("idiom" nil (ih1 d iy0 ax m))
("idiomatic" nil (ih1 d iy0 ax m ae1 t ih0 k))
("idioms" nil (ih1 d iy0 ax m z))
("idiosyncratic" nil (ih0 d iy0 ow0 s ih0 n k r ae1 t ih0 k))
("idiot" nil (ih1 d iy0 ax t))
("idiotic" nil (ih1 d iy0 aa1 t ih0 k))
("idiotically" nil (ih1 d iy0 aa1 t ih0 k l iy0))
("idiots" nil (ih1 d iy0 ax t s))
("idle" nil (ay1 d ax l))
("idled" nil (ay1 d ax l d))
("idleman" nil (ay1 d ax l m ax n))
("idleness" nil (ay1 d ax l n ax s))
("idler" nil (ay1 d ax l er0))
("idles" nil (ay1 d ax l z))
("idling" nil (ay1 d ax l ih0 ng))
("idly" nil (ay1 d l iy0))
("ido" nil (iy1 d ow1))
("idol" nil (ay1 d ax l))
("idola" nil (ih0 d ow1 l ax))
("idolatrous" nil (ay0 d aa1 l ax t r ax s))
("idolize" nil (ay1 d ax l ay1 z))
("idolized" nil (ay1 d ax l ay1 z d))
("idolizes" nil (ay1 d ax l ay1 z ih0 z))
("idolizing" nil (ay1 d ax l ay1 z ih0 ng))
("idols" nil (ay1 d ax l z))
("idona" nil (ih0 d ow1 n ax))
("idonia" nil (ih0 d ow1 n iy0 ax))
("idrocarburi" nil (ih0 d r aa1 k aa1 r b uh1 r iy0))
("iduna" nil (ih0 d uw1 n ax))
("idyll" nil (ay1 d ax l))
("idyllic" nil (ay0 d ih1 l ih0 k))
("ierardi" nil (iy0 er0 aa1 r d iy0))
("ierne" nil (ih1 r n))
("iezzi" nil (iy0 eh1 t s iy0))
("if" nil (ih1 f))
("iffy" nil (ih1 f iy0))
("ifil" nil (ay0 f iy1 l))
("ifill" nil (ih0 f iy1 l))
("ifint" nil (ay1 f ih0 n t))
("ifs" nil (ih1 f s))
("igarashi" nil (iy1 g aa0 r aa1 sh iy0))
("ige" nil (ay1 jh))
("igel" nil (ih1 g ax l))
("igene" nil (ay0 jh iy1 n))
("iger" nil (ay1 g er0))
("iglehart" nil (ay1 g ax l hh aa0 r t))
("iglesia" nil (iy1 g l ey1 s iy0 ax))
("iglesias" nil (ih0 g l ey1 s iy0 ih0 s))
("igloo" nil (ih1 g l uw0))
("ignace" nil (ih1 g n ax s))
("ignacio" nil (ih0 g n aa1 s iy0 ow0))
("ignasiak" nil (ih0 g n aa1 s iy0 ae0 k))
("ignatia" nil (ih0 g n aa1 sh ax))
("ignatius" nil (ih0 g n ey1 sh ax s))
("ignatowski" nil (ih0 g n ax t ao1 f s k iy0))
("ignatz" nil (ih1 g n ax t s))
("igneous" nil (ih1 g n iy0 ax s))
("ignite" nil (ih0 g n ay1 t))
("ignited" nil (ih0 g n ay1 t ih0 d))
("ignites" nil (ih0 g n ay1 t s))
("igniting" nil (ih0 g n ay1 t ih0 ng))
("ignition" nil (ih0 g n ih1 sh ax n))
("ignoble" nil (ih0 g n ow1 b ax l))
("ignominious" nil (ih1 g n ax m ih1 n iy0 ax s))
("ignominy" nil (ih1 g n ow0 m ih1 n iy0))
("ignoramus" nil (ih1 g n er0 ey1 m ax s))
("ignorance" nil (ih1 g n er0 ax n s))
("ignorant" nil (ih1 g n er0 ax n t))
("ignore" nil (ih0 g n ao1 r))
("ignored" nil (ih0 g n ao1 r d))
("ignores" nil (ih0 g n ao1 r z))
("ignoring" nil (ih0 g n ao1 r ih0 ng))
("igo" nil (iy1 g ow0))
("igoe" nil (iy1 g ow0))
("igon" nil (ay1 g ax n))
("igor" nil (ih0 g ao1 r))
("igou" nil (ih0 g uw1))
("iguana" nil (ay0 g w aa1 n ax))
("iguanas" nil (ih0 g w aa1 n ax z))
("ihasz" nil (ay1 hh ae0 s))
("ihde" nil (ih1 d))
("ihle" nil (ay1 hh ax l))
("ihlenfeldt" nil (ih1 l ih0 n f ih0 l t))
("ihnen" nil (ih1 n ax n))
("ihrig" nil (ih1 r ih0 g))
("ihrke" nil (ih1 r k))
("iiams" nil (iy1 ax m z))
("iida" nil (iy1 d ax))
("iie" nil (iy1 iy0))
("ijames" nil (ih0 y aa1 m eh0 s))
("ijaz" nil (ih1 jh ax z))
("ikard" nil (ih1 k er0 d))
("ike" nil (ay1 k))
("ikea" nil (ay1 k iy1 ax))
("ikeda" nil (ih0 k ey1 d ax))
("ikenberry" nil (ay1 k ax n b eh1 r iy0))
("ikerd" nil (ih1 k er0 d))
("ikeuchi" nil (iy0 k uw1 ch iy0))
("ikey" nil (ih1 k iy0))
("ikie" nil (ih1 k iy0))
("ikle" nil (ih1 k ax l))
("ikner" nil (ih1 k n er0))
("il" nil (ih1 l))
("ila" nil (iy1 l ax))
("ilalis" nil (ih1 l ae1 l ax s))
("ilan" nil (iy1 l aa1 n))
("ilardi" nil (ih0 l aa1 r d iy0))
("ilbo" nil (ih1 l b ow0))
("ileana" nil (ih1 l iy0 ae1 n ax))
("ilene" nil (ih1 l iy0 n))
("iler" nil (ay1 l er0))
("iles" nil (ay1 l z))
("ileto" nil (ay0 l ey1 d ow0))
("ilg" nil (ih1 l g))
("ilgenfritz" nil (ih1 l g ih0 n f r ih0 t s))
("ilhae" nil (ih0 l hh ey1))
("iliad" nil (ih1 l iy0 ax d))
("iliano" nil (ih1 l iy0 aa1 n ow0))
("iliescu" nil (ih1 l iy0 eh1 s k y uw1))
("iliff" nil (ih1 l ih0 f))
("iljin" nil (ih1 l jh ih0 n))
("ilk" nil (ih1 l k))
("ilka" nil (ih1 l k ax))
("ill" nil (ih1 l))
("illegal" nil (ih0 l iy1 g ax l))
("illegalities" nil (ih1 l iy0 g ae1 l ih0 t iy0 z))
("illegality" nil (ih1 l iy0 g ae1 l ih0 t iy0))
("illegally" nil (ih0 l iy1 g ax l iy0))
("illegals" nil (ih0 l iy1 g ax l z))
("illegible" nil (ih0 l eh1 jh ax b ax l))
("illegitimacy" nil (ih1 l ih0 jh ih1 t ax m ax s iy0))
("illegitimate" nil (ih1 l ih0 jh ih1 t ax m ih0 t))
("illes" nil (ay1 l z))
("illg" nil (ih1 l g))
("illicit" nil (ih0 l ih1 s ax t))
("illicitly" nil (ih1 l ih0 s ih0 t l iy0))
("illig" nil (ih1 l ih0 g))
("illingworth" nil (ih1 l ih0 ng w er1 th))
("illinois" nil (ih1 l ax n oy1))
("illiquid" nil (ih0 l ih1 k w ih0 d))
("illiquidity" nil (ih1 l ih0 k w ih1 d ih0 t iy0))
("illiteracy" nil (ih0 l ih1 t er0 ax s iy0))
("illiterate" nil (ih0 l ih1 t er0 ax t))
("illiterates" nil (ih1 l ih1 t er0 ax t s))
("illness" nil (ih1 l n ax s))
("illnesses" nil (ih1 l n ax s ih0 z))
("illogic" nil (ih0 l aa1 jh ih0 k))
("illogical" nil (ih0 l aa1 jh ih0 k ax l))
("ills" nil (ih1 l z))
("illuminate" nil (ih0 l uw1 m ih0 n ih0 t))
("illuminated" nil (ih0 l uw1 m ax n ey1 t ax d))
("illuminates" nil (ih0 l uw1 m ax n ey1 t s))
("illuminating" nil (ih0 l uw1 m ax n ey1 t ih0 ng))
("illumination" nil (ih0 l uw1 m ax n ey1 sh ax n))
("illuminator" nil (ih0 l uw1 m ax n ey1 t er0))
("illumined" nil (ih0 l uw1 m ax n d))
("illusion" nil (ih0 l uw1 zh ax n))
("illusionary" nil (ih0 l uw1 zh ax n eh1 r iy0))
("illusionism" nil (ih0 l uw1 zh ax n ih1 z ax m))
("illusionist" nil (ih0 l uw1 zh ax n ax s t))
("illusionists" nil (ih0 l uw1 zh ax n ax s t s))
("illusions" nil (ih0 l uw1 zh ax n z))
("illusory" nil (ih0 l uw1 s er0 iy0))
("illustrate" nil (ih1 l ax s t r ey1 t))
("illustrated" nil (ih1 l ax s t r ey1 t ax d))
("illustrates" nil (ih1 l ax s t r ey1 t s))
("illustrating" nil (ih1 l ax s t r ey1 t ih0 ng))
("illustration" nil (ih1 l ax s t r ey1 sh ax n))
("illustrations" nil (ih1 l ax s t r ey1 sh ax n z))
("illustrative" nil (ih0 l ah1 s t r ax t ih0 v))
("illustrator" nil (ih1 l ax s t r ey1 t er0))
("illustrators" nil (ih1 l ax s t r ey1 t er0 z))
("illustrious" nil (ih0 l ah1 s t r iy0 ax s))
("illyrian" nil (ih0 l ih1 r iy0 ax n))
("ilo" nil (ay1 l ow0))
("ilona" nil (ih0 l ow1 n ax))
("ilopango" nil (iy1 l ow0 p ae1 ng g ow0))
("ilsa" nil (ih1 l s ax))
("ilse" nil (ih1 l s))
("ilsley" nil (ih1 l z l iy0))
("ilva" nil (ih1 l v ax))
("ilya" nil (ih1 l y ax))
("im" nil (ih1 m))
("image" nil (ih1 m ax jh))
("imageries" nil (ih1 m ih0 jh r iy0 z))
("imagery" nil (ih1 m ax jh r iy0))
("images" nil (ih1 m ax jh ax z))
("imaginable" nil (ih0 m ae1 jh ax n ax b ax l))
("imaginary" nil (ih0 m ae1 jh ax n eh1 r iy0))
("imagination" nil (ih0 m ae1 jh ax n ey1 sh ax n))
("imaginations" nil (ih0 m ae1 jh ax n ey1 sh ax n z))
("imaginative" nil (ih0 m ae1 jh ax n ax t ih0 v))
("imaginatively" nil (ih0 m ae1 jh ax n ax t ih0 v l iy0))
("imagine" nil (ih0 m ae1 jh ax n))
("imagined" nil (ih0 m ae1 jh ax n d))
("imagineering" nil (ih0 m ae1 jh ax n ih1 r ih0 ng))
("imagines" nil (ih0 m ae1 jh ax n z))
("imaging" nil (ih1 m ih0 jh ih0 ng))
("imagining" nil (ih0 m ae1 jh ax n ih0 ng))
("imai" nil (ih0 m ay1))
("imam" nil (ay1 m ae0 m))
("imamura" nil (ih0 m aa0 m uh1 r ax))
("iman" nil (ay1 m ax n))
("imari" nil (iy0 m aa1 r iy0))
("imasco" nil (ih0 m ae1 s ow0))
("imax" nil (ay1 m ae1 k s))
("imbalance" nil (ih0 m b ae1 l ax n s))
("imbalances" nil (ih0 m b ae1 l ax n s ih0 z))
("imbedded" nil (ih0 m b eh1 d ih0 d))
("imber" nil (ih1 m b er0))
("imbibe" nil (ih0 m b ay1 b))
("imbibed" nil (ih0 m b ay1 b d))
("imboden" nil (ih1 m b ow0 d ax n))
("imbroglio" nil (ih0 m b r ow1 l y ow0))
("imbrogno" nil (ih0 m b r ow1 g n ow0))
("imbued" nil (ih0 m b y uw1 d))
("imburgia" nil (ih1 m b er0 g iy0 ax))
("imcera" nil (ih1 m s eh1 r ax))
("imel" nil (ih1 m ax l))
("imelda" nil (ih0 m eh1 l d ax))
("imes" nil (ay1 m z))
("imhof" nil (ih1 m hh ax f))
("imhoff" nil (ih1 m hh ao1 f))
("imig" nil (ih1 m ih0 g))
("imitable" nil (ih1 m ax t ax b ax l))
("imitate" nil (ih1 m ax t ey1 t))
("imitated" nil (ih1 m ax t ey1 t ih0 d))
("imitates" nil (ih1 m ax t ey1 t s))
("imitating" nil (ih1 m ax t ey1 t ih0 ng))
("imitation" nil (ih1 m ax t ey1 sh ax n))
("imitations" nil (ih1 m ih0 t ey1 sh ax n z))
("imitative" nil (ih1 m ax t ey1 t ih0 v))
("imitator" nil (ih1 m ax t ey1 t er0))
("imitators" nil (ih1 m ih0 t ey1 t er0 z))
("imlay" nil (ih0 m l ey1))
("imler" nil (ih1 m l er0))
("imm" nil (ay1 eh1 m eh1 m))
("immaculate" nil (ih0 m ae1 k y uw0 l ih0 t))
("immaculately" nil (ih0 m ae1 k y ax l ax t l iy0))
("immanence" nil (ih1 m ax n ax n s))
("immanent" nil (ih1 m ax n ax n t))
("immanuel" nil (ih1 m ax n uh0 l))
("immaterial" nil (ih1 m ax t ih1 r iy0 ax l))
("immature" nil (ih1 m ax t y uh1 r))
("immaturity" nil (ih1 m ax ch uh1 r ih0 t iy0))
("immeasurably" nil (ih0 m eh1 z er0 ae1 b l iy0))
("immediacy" nil (ih0 m iy1 d iy0 ax s iy0))
("immediate" nil (ih0 m iy1 d iy0 ax t))
("immediately" nil (ih0 m iy1 d iy0 ax t l iy0))
("immel" nil (ih1 m ax l))
("immemorial" nil (ih1 m ax m ao1 r iy0 ax l))
("immense" nil (ih0 m eh1 n s))
("immensely" nil (ih0 m eh1 n s l iy0))
("immerman" nil (ih1 m er0 m ax n))
("immerse" nil (ih0 m er1 s))
("immersed" nil (ih0 m er1 s t))
("immersion" nil (ih0 m er1 zh ax n))
("immigrant" nil (ih1 m ax g r ax n t))
("immigrants" nil (ih1 m ax g r ax n t s))
("immigrate" nil (ih1 m ax g r ey1 t))
("immigrated" nil (ih1 m ax g r ey1 t ih0 d))
("immigration" nil (ih1 m ax g r ey1 sh ax n))
("imminent" nil (ih1 m ax n ax n t))
("imminently" nil (ih1 m ax n ax n t l iy0))
("immiscible" nil (ih0 m ih1 s ax b ax l))
("immobile" nil (ih0 m ow1 b ax l))
("immobility" nil (ih1 m ow0 b ih1 l ih0 t iy0))
("immobilized" nil (ih0 m ow1 b ax l ay1 z d))
("immobilizing" nil (ih0 m ow1 b ax l ay1 z ih0 ng))
("immoral" nil (ih0 m ao1 r ax l))
("immorality" nil (ih1 m er0 ae1 l ih0 t iy0))
("immortal" nil (ih0 m ao1 r t ax l))
("immortality" nil (ih1 m ao0 r t ae1 l ih0 t iy0))
("immortalize" nil (ih0 m ao1 r t ax l ay0 z))
("immortalized" nil (ih0 m ao1 r t ax l ay0 z d))
("immortalizes" nil (ih0 m ao1 r t ax l ay0 z ih0 z))
("immortalizing" nil (ih0 m ao1 r t ax l ay0 z ih0 ng))
("immotile" nil (ih0 m ow1 t ax l))
("immovable" nil (ih0 m uw1 v ax b ax l))
("immulogic" nil (ih1 m y uw0 l aa1 jh ih0 k))
("immune" nil (ih0 m y uw1 n))
("immunetech" nil (ih1 m y uw0 n t eh1 k))
("immunex" nil (ih1 m y uw0 n eh0 k s))
("immunities" nil (ih0 m y uw1 n ih0 t iy0 z))
("immunity" nil (ih0 m y uw1 n ax t iy0))
("immunization" nil (ih1 m y uw0 n ax z ey1 sh ax n))
("immunizations" nil (ih1 m y uw0 n ax z ey1 sh ax n z))
("immunize" nil (ih1 m y uw0 n ay1 z))
("immunized" nil (ih1 m y ax n ay1 z d))
("immunizes" nil (ih1 m y ax n ay1 z ih0 z))
("immunizing" nil (ih1 m y ax n ay1 z ih0 ng))
("immuno" nil (ih0 m y uw1 n ow0))
("immunodeficiency" nil (ih1 m y uw0 n ow0 d ih1 f ih1 sh ax n s iy0))
("immunological" nil (ih1 m y uw0 n ax l aa1 jh ih0 k ax l))
("immunologist" nil (ih1 m y uw0 n aa1 l ax jh ih0 s t))
("immunology" nil (ih1 m y uw0 n aa1 l ax jh iy0))
("immunomedics" nil (ih1 m y uw0 n ow0 m eh1 d ih0 k s))
("immunotherapy" nil (ih1 m y uw0 n ow0 th eh1 r ax p iy0))
("immutable" nil (ih0 m y uw1 t ax b ax l))
("imnet" nil (ih1 m n eh1 t))
("imo" nil (ay1 m ow0))
("imogen" nil (ih1 m ax g ax n))
("imogene" nil (ih1 m ax jh iy1 n))
("imondi" nil (ih0 m ow1 n d iy0))
("impact" v (ih0 m p ae1 k t))
("impact" n (ih1 m p ae0 k t))
("impacted" nil (ih1 m p ae1 k t ih0 d))
("impacting" nil (ih0 m p ae1 k t ih0 ng))
("impacts" v (ih0 m p ae1 k t s))
("impacts" n (ih1 m p ae0 k t s))
("impair" nil (ih0 m p eh1 r))
("impaired" nil (ih0 m p eh1 r d))
("impairing" nil (ih0 m p eh1 r ih0 ng))
("impairment" nil (ih0 m p eh1 r m ax n t))
("impairs" nil (ih0 m p eh1 r z))
("impala" nil (ih0 m p aa1 l ax))
("impaled" nil (ih0 m p ey1 l d))
("impart" nil (ih0 m p aa1 r t))
("imparted" nil (ih0 m p aa1 r t ih0 d))
("impartial" nil (ih0 m p aa1 r sh ax l))
("impartiality" nil (ih0 m p aa1 r sh iy0 ae1 l ih0 t iy0))
("impartially" nil (ih0 m p aa1 r sh ax l iy0))
("imparting" nil (ih0 m p aa1 r t ih0 ng))
("impassable" nil (ih0 m p ae1 s ax b ax l))
("impasse" nil (ih0 m p ae1 s))
("impassioned" nil (ih0 m p ae1 sh ax n d))
("impassive" nil (ih0 m p ae1 s ih0 v))
("impassively" nil (ih0 m p ae1 s ih0 v l iy0))
("impastato" nil (ih0 m p aa0 s t aa1 t ow0))
("impatience" nil (ih0 m p ey1 sh ax n s))
("impatiens" nil (ih0 m p ey1 sh ax n z))
("impatient" nil (ih0 m p ey1 sh ax n t))
("impatiently" nil (ih0 m p ey1 sh ax n t l iy0))
("impeach" nil (ih0 m p iy1 ch))
("impeached" nil (ih0 m p iy1 ch t))
("impeachment" nil (ih0 m p iy1 ch m ax n t))
("impeachments" nil (ih0 m p iy1 ch m ax n t s))
("impeccable" nil (ih0 m p eh1 k ax b ax l))
("impeccably" nil (ih0 m p eh1 k ax b l iy0))
("impedance" nil (ih0 m p iy1 d ax n s))
("impede" nil (ih0 m p iy1 d))
("impeded" nil (ih0 m p iy1 d ih0 d))
("impedes" nil (ih0 m p iy1 d z))
("impediment" nil (ih0 m p eh1 d ax m ax n t))
("impediments" nil (ih0 m p eh1 d ax m ax n t s))
("impeding" nil (ih0 m p iy1 d ih0 ng))
("impelled" nil (ih0 m p eh1 l d))
("impending" nil (ih0 m p eh1 n d ih0 ng))
("impenetrable" nil (ih0 m p eh1 n ax t r ax b ax l))
("imperative" nil (ih0 m p eh1 r ax t ih0 v))
("imperatives" nil (ih0 m p eh1 r ax t ih0 v z))
("imperato" nil (ih0 m p er0 aa1 t ow0))
("imperceptible" nil (ih1 m p er0 s eh1 p t ih0 b ax l))
("imperceptibly" nil (ih1 m p er0 s eh1 p t ih0 b l iy0))
("imperfect" nil (ih0 m p er1 f ih0 k t))
("imperfection" nil (ih1 m p er0 f eh1 k sh ax n))
("imperfections" nil (ih1 m p er0 f eh1 k sh ax n z))
("imperfectly" nil (ih0 m p er1 f ih0 k t l iy0))
("imperia" nil (ih0 m p iy1 r iy0 ax))
("imperial" nil (ih0 m p ih1 r iy0 ax l))
("imperiale" nil (ih0 m p er0 iy0 aa1 l iy0))
("imperialism" nil (ih0 m p ih1 r iy0 ax l ih1 z ax m))
("imperialist" nil (ih0 m p ih1 r iy0 ax l ih0 s t))
("imperialistic" nil (ih0 m p iy1 r iy0 ax l ih1 s t ih0 k))
("imperialists" nil (ih0 m p ih1 r iy0 ax l ih0 s t s))
("imperil" nil (ih0 m p eh1 r ax l))
("imperiled" nil (ih0 m p eh1 r ax l d))
("imperiling" nil (ih0 m p eh1 r ax l ih0 ng))
("imperils" nil (ih0 m p eh1 r ax l z))
("imperious" nil (ih0 m p ih1 r iy0 ax s))
("impermissible" nil (ih1 m p er0 m ih1 s ih0 b ax l))
("impersonal" nil (ih0 m p er1 s ax n ax l))
("impersonality" nil (ih0 m p er1 s ax n ae1 l ax t iy0))
("impersonating" nil (ih0 m p er1 s ax n ey1 t ih0 ng))
("impersonation" nil (ih1 m p er0 s ax n ey1 sh ax n))
("impersonator" nil (ih0 m p er1 s ax n ey0 t er0))
("impersonators" nil (ih0 m p er1 s ax n ey1 t er0 z))
("impertinent" nil (ih0 m p er1 t ax n ax n t))
("impervious" nil (ih0 m p er1 v iy0 ax s))
("impetuous" nil (ih0 m p eh1 ch w ax s))
("impetus" nil (ih1 m p ax t ax s))
("impinge" nil (ih0 m p ih1 n jh))
("impish" nil (ih1 m p ih0 sh))
("implacable" nil (ih0 m p l ae1 k ax b ax l))
("implant" nil (ih0 m p l ae1 n t))
("implantable" nil (ih1 m p l ae1 n t ax b ax l))
("implantation" nil (ih0 m p l ae1 n t ey1 sh ax n))
("implanted" nil (ih0 m p l ae1 n t ih0 d))
("implanting" nil (ih0 m p l ae1 n t ih0 ng))
("implants" nil (ih0 m p l ae1 n t s))
("implausible" nil (ih0 m p l ao1 z ax b ax l))
("implausibly" nil (ih0 m p l ao1 z ax b l iy0))
("implement" n (ih1 m p l ax m ax n t))
("implement" v (ih1 m p l ax m eh1 n t))
("implementation" nil (ih1 m p l ax m eh0 n t ey1 sh ax n))
("implemented" nil (ih1 m p l ax m eh1 n t ax d))
("implementing" nil (ih1 m p l ax m eh1 n t ih0 ng))
("implements" n (ih1 m p l ax m ax n t s))
("implements" v (ih1 m p l ax m eh1 n t s))
("implicate" nil (ih1 m p l ih0 k ey1 t))
("implicated" nil (ih1 m p l ih0 k ey1 t ih0 d))
("implicating" nil (ih1 m p l ih0 k ey1 t ih0 ng))
("implication" nil (ih1 m p l ax k ey1 sh ax n))
("implications" nil (ih1 m p l ax k ey1 sh ax n z))
("implicit" nil (ih0 m p l ih1 s ax t))
("implicitly" nil (ih0 m p l ih1 s ax t l iy0))
("implied" nil (ih0 m p l ay1 d))
("implies" nil (ih0 m p l ay1 z))
("implode" nil (ih0 m p l ow1 d))
("imploded" nil (ih0 m p l ow1 d ih0 d))
("implore" nil (ih0 m p l ao1 r))
("implored" nil (ih0 m p l ao1 r d))
("implores" nil (ih0 m p l ao1 r z))
("imploring" nil (ih0 m p l ao1 r ih0 ng))
("implosion" nil (ih0 m p l ow1 zh ax n))
("imply" nil (ih0 m p l ay1))
("implying" nil (ih0 m p l ay1 ih0 ng))
("impolite" nil (ih0 m p ax l ay1 t))
("impolitic" nil (ih0 m p ao1 l ih1 t ih1 k))
("imponderable" nil (ih0 m p aa1 n d er0 ax b ax l))
("imponderables" nil (ih0 m p aa1 n d er0 ax b ax l z))
("import" v (ih0 m p ao1 r t))
("import" n (ih1 m p ao0 r t))
("importance" nil (ih0 m p ao1 r t ax n s))
("important" nil (ih0 m p ao1 r t ax n t))
("importantly" nil (ih0 m p ao1 r t ax n t l iy0))
("importation" nil (ih1 m p ao0 r t ey1 sh ax n))
("imported" nil (ih0 m p ao1 r t ih0 d))
("importer" nil (ih0 m p ao1 r t er0))
("importers" nil (ih0 m p ao1 r t er0 z))
("importing" nil (ih0 m p ao1 r t ih0 ng))
("imports" v (ih0 m p ao1 r t s))
("imports" n (ih1 m p ao0 r t s))
("impose" nil (ih0 m p ow1 z))
("imposed" nil (ih0 m p ow1 z d))
("imposes" nil (ih0 m p ow1 z ax z))
("imposing" nil (ih0 m p ow1 z ih0 ng))
("imposition" nil (ih1 m p ax z ih1 sh ax n))
("impossibility" nil (ih0 m p ao1 s ih0 b ih1 l ih0 t iy0))
("impossible" nil (ih0 m p aa1 s ax b ax l))
("impossibly" nil (ih0 m p aa1 s ax b l iy0))
("impostors" nil (ih0 m p ao1 s t er0 z))
("impotence" nil (ih1 m p ax t ax n s))
("impotent" nil (ih1 m p ax t ax n t))
("impound" nil (ih0 m p aw1 n d))
("impounded" nil (ih0 m p aw1 n d ih0 d))
("impoundment" nil (ih0 m p aw1 n d m ax n t))
("impoundments" nil (ih0 m p aw1 n d m ax n t s))
("impoverish" nil (ih0 m p aa1 v r ih0 sh))
("impoverished" nil (ih0 m p aa1 v r ih0 sh t))
("impoverishes" nil (ih0 m p aa1 v r ih0 sh ax z))
("impoverishment" nil (ih0 m p aa1 v r ih0 sh m ax n t))
("impracticable" nil (ih0 m p r ae1 k t ih0 k ax b ax l))
("impractical" nil (ih0 m p r ae1 k t ax k ax l))
("imprecise" nil (ih1 m p r ax s ay1 s))
("impregnable" nil (ih0 m p r eh1 g n ax b ax l))
("impregnated" nil (ih0 m p r eh1 g n ey1 t ax d))
("impregnation" nil (ih0 m p r eh1 g n ey1 sh ax n))
("impresario" nil (ih1 m p r ih0 s aa1 r iy0 ow1))
("impress" v (ih0 m p r eh1 s))
("impress" n (ih1 m p r eh0 s))
("impressed" nil (ih0 m p r eh1 s t))
("impresses" v (ih0 m p r eh1 s ih0 z))
("impresses" n (ih1 m p r eh0 s ih0 z))
("impressing" nil (ih0 m p r eh1 s ih0 ng))
("impression" nil (ih0 m p r eh1 sh ax n))
("impressionable" nil (ih0 m p r eh1 sh ax n ax b ax l))
("impressionism" nil (ih0 m p r eh1 sh ax n ih1 z ax m))
("impressionist" nil (ih0 m p r eh1 sh ax n ax s t))
("impressionistic" nil (ih0 m p r eh1 sh ax n ih1 s t ih0 k))
("impressionists" nil (ih0 m p r eh1 sh ax n ih0 s t s))
("impressions" nil (ih0 m p r eh1 sh ax n z))
("impressive" nil (ih0 m p r eh1 s ih0 v))
("impressively" nil (ih0 m p r eh1 s ih0 v l iy0))
("impressment" nil (ih0 m p r eh1 s m ax n t))
("imprimatur" nil (ih1 m p r ih0 m aa1 t er0))
("imprimis" nil (ih0 m p r iy1 m ih0 s))
("imprint" v (ih0 m p r ih1 n t))
("imprint" n (ih1 m p r ih0 n t))
("imprinted" nil (ih0 m p r ih1 n t ih0 d))
("imprints" v (ih0 m p r ih1 n t s))
("imprints" n (ih1 m p r ih0 n t s))
("imprison" nil (ih0 m p r ih1 z ax n))
("imprisoned" nil (ih0 m p r ih1 z ax n d))
("imprisoning" nil (ih0 m p r ih1 z ax n ih0 ng))
("imprisonment" nil (ih0 m p r ih1 z ax n m ax n t))
("improbable" nil (ih0 m p r aa1 b ax b ax l))
("improbably" nil (ih0 m p r aa1 b ax b l iy0))
("impromptu" nil (ih0 m p r aa1 m p t uw0))
("improper" nil (ih0 m p r aa1 p er0))
("improperly" nil (ih0 m p r aa1 p er0 l iy0))
("improprieties" nil (ih1 m p r ax p r ay1 ax t iy0 z))
("impropriety" nil (ih1 m p r ax p r ay1 ax t iy0))
("improv" nil (ih1 m p r aa1 v))
("improve" nil (ih0 m p r uw1 v))
("improved" nil (ih0 m p r uw1 v d))
("improvement" nil (ih0 m p r uw1 v m ax n t))
("improvements" nil (ih0 m p r uw1 v m ax n t s))
("improves" nil (ih0 m p r uw1 v z))
("improving" nil (ih0 m p r uw1 v ih0 ng))
("improvisation" nil (ih1 m p r ax v ih0 z ey1 sh ax n))
("improvisational" nil (ih1 m p r ax v ih0 z ey1 sh ax n ax l))
("improvise" nil (ih1 m p r ax v ay1 z))
("improvised" nil (ih1 m p r ax v ay1 z d))
("improvising" nil (ih1 m p r ax v ay1 z ih0 ng))
("imprudence" nil (ih0 m p r uw1 d ax n s))
("imprudent" nil (ih0 m p r uw1 d ax n t))
("imprudently" nil (ih0 m p r uw1 d ax n t l iy0))
("impson" nil (ih1 m p s ax n))
("impugn" nil (ih0 m p y uw1 n))
("impugned" nil (ih0 m p y uw1 n d))
("impugning" nil (ih0 m p y uw1 n ih0 ng))
("impulse" nil (ih1 m p ax l s))
("impulses" nil (ih1 m p ax l s ih0 z))
("impulsive" nil (ih0 m p ah1 l s ih0 v))
("impulsively" nil (ih0 m p ah1 l s ih0 v l iy0))
("impunity" nil (ih0 m p y uw1 n ih0 t iy0))
("impure" nil (ih0 m p y uh1 r))
("impurities" nil (ih0 m p y uh1 r ax t iy0 z))
("impurity" nil (ih0 m p y uh1 r ax t iy0))
("imputation" nil (ih1 m p y ax t ey1 sh ax n))
("imputed" nil (ih0 m p y uw1 t ih0 d))
("imre" nil (ih1 m r ax))
("imreg" nil (ih1 m r eh1 g))
("imrie" nil (ih1 m er0 iy0))
("imus" nil (ay1 m ax s))
("in" nil (ih0 n))
("ina" nil (iy1 n ax))
("inability" nil (ih1 n ax b ih1 l ih0 t iy0))
("inabinet" nil (ih0 n ax b ih1 n ih0 t))
("inaccessible" nil (ih1 n ax k s eh1 s ax b ax l))
("inaccuracies" nil (ih0 n ae1 k y er0 ae1 s iy0 z))
("inaccuracy" nil (ih0 n ae1 k y er0 ax s iy0))
("inaccurate" nil (ih0 n ae1 k y er0 ax t))
("inaccurately" nil (ih0 n ae1 k y er0 ax t l iy0))
("inacom" nil (ih1 n ax k aa1 m))
("inacomp" nil (ay1 n ax k aa1 m p))
("inaction" nil (ih0 n ae1 k sh ax n))
("inactivated" nil (ih0 n ae1 k t ih0 v ey1 t ih0 d))
("inactivation" nil (ih0 n ae1 k t ih0 v ey1 sh ax n))
("inactive" nil (ih0 n ae1 k t ih0 v))
("inactivity" nil (ih0 n ae0 k t ih1 v ih0 t iy0))
("inadequacies" nil (ih0 n ae1 d ax k w ax s iy0 z))
("inadequacy" nil (ih0 n ae1 d ih0 k w ax s iy0))
("inadequate" nil (ih0 n ae1 d ax k w ax t))
("inadequately" nil (ih0 n ae1 d ax k w ax t l iy0))
("inadmissible" nil (ih0 n ax d m ih1 s ax b ax l))
("inadvertent" nil (ih1 n ax d v er1 t ax n t))
("inadvertently" nil (ih1 n ax d v er1 t ax n t l iy0))
("inadvisable" nil (ih0 n ax d v ay1 z ax b ax l))
("inalienable" nil (ih0 n ey1 l y ax n ax b ax l))
("inamura" nil (ih1 n ax m uh1 r ax))
("inane" nil (ih0 n ey1 n))
("inanimate" nil (ih0 n ae1 n ax m ax t))
("inapplicable" nil (ih0 n ae1 p l ih0 k ax b ax l))
("inappropriate" nil (ih1 n ax p r ow1 p r iy0 ih0 t))
("inappropriately" nil (ih0 n ax p r aa1 p r iy0 ax t l iy0))
("inarticulate" nil (ih0 n aa0 r t ih1 k y ax l ax t))
("inasmuch" nil (ih0 n ae1 s m ax k))
("inattention" nil (ih0 n ax t eh1 n ch ax n))
("inattentive" nil (ih1 n ax t eh1 n t ih0 v))
("inaudible" nil (ih0 n ao1 d ax b ax l))
("inaugural" nil (ih0 n ao1 g er0 ax l))
("inaugurate" nil (ih0 n ao1 g y er0 ih0 t))
("inaugurated" nil (ih0 n ao1 g er0 ey1 t ih0 d))
("inaugurates" nil (ih0 n ao1 g y er0 ih0 t s))
("inaugurating" nil (ih0 n ao1 g y er0 ey1 t ih0 ng))
("inauguration" nil (ih0 n ao1 g y er0 ey1 sh ax n))
("inauspicious" nil (ih0 n aw1 s p ih1 sh ih0 s))
("inboard" nil (ih1 n b ao1 r d))
("inboden" nil (ih1 n b ow0 d ax n))
("inbody" nil (ih1 n b aa0 d iy0))
("inborn" nil (ih1 n b ao1 r n))
("inbound" nil (ih0 n b aw1 n d))
("inbred" nil (ih1 n b r eh1 d))
("inc" nil (ih1 ng k))
("inca" nil (ih1 ng k ax))
("incalculable" nil (ih0 n k ae1 l k y ax l ax b ax l))
("incandescent" nil (ih1 n k ax n d eh1 s ax n t))
("incantation" nil (ih0 n k ae1 n t ey1 sh ax n))
("incantatory" nil (ih0 n k ae1 n t ax t ao1 r iy0))
("incapable" nil (ih0 n k ey1 p ax b ax l))
("incapacitated" nil (ih0 n k ax p ae1 s ih0 t ey1 t ih0 d))
("incapacitation" nil (ih0 n k ax p ae1 s ih0 t ey1 sh ax n))
("incapacity" nil (ih0 n k ax p ae1 s ax t iy0))
("incarcerate" nil (ih0 n k aa1 r s er0 ey1 t))
("incarcerated" nil (ih0 n k aa1 r s er0 ey1 t ih0 d))
("incarcerating" nil (ih0 n k aa1 r s er0 ey1 t ih0 ng))
("incarceration" nil (ih0 n k aa1 r s er0 ey1 sh ax n))
("incarnate" nil (ih0 n k aa1 r n ax t))
("incarnation" nil (ih0 n k aa1 r n ey1 sh ax n))
("incarnations" nil (ih0 n k aa0 r n ey1 sh ax n z))
("incas" nil (ih1 ng k ax z))
("incase" nil (ih0 n k ey1 s))
("incata" nil (ih0 ng k aa1 t ax))
("ince" nil (ih1 n s))
("incendiary" nil (ih0 n s eh1 n d iy0 eh0 r iy0))
("incense" v (ih0 n s eh1 n s))
("incense" n (ih1 n s eh1 n s))
("incensed" nil (ih1 n s eh1 n s t))
("incentive" nil (ih0 n s eh1 n t ih0 v))
("incentives" nil (ih0 n s eh1 n t ih0 v z))
("inception" nil (ih0 n s eh1 p sh ax n))
("incessant" nil (ih0 n s eh1 s ax n t))
("incessantly" nil (ih0 n s eh1 s ax n t l iy0))
("incest" nil (ih1 n s eh1 s t))
("incestuous" nil (ih0 n s eh1 s ch w ax s))
("inch" nil (ih1 n ch))
("inchcape" nil (ih1 n ch k ey1 p))
("inched" nil (ih1 n ch t))
("inches" nil (ih1 n ch ax z))
("inching" nil (ih1 n ch ih0 ng))
("inchon" nil (ih1 n ch ax n))
("incidence" nil (ih1 n s ax d ax n s))
("incidences" nil (ih1 n s ax d ax n s ih0 z))
("incident" nil (ih1 n s ax d ax n t))
("incidental" nil (ih1 n s ih0 d eh1 n t ax l))
("incidentally" nil (ih1 n s ih0 d eh1 n t ax l iy0))
("incidentals" nil (ih1 n s ih0 d eh1 n t ax l z))
("incidents" nil (ih1 n s ax d ax n t s))
("incinerate" nil (ih0 n s ih1 n er0 ey1 t))
("incinerated" nil (ih0 n s ih1 n er0 ey1 t ih0 d))
("incinerating" nil (ih0 n s ih1 n er0 ey1 t ih0 ng))
("incineration" nil (ih0 n s ih1 n er0 ey1 sh ax n))
("incinerator" nil (ih0 n s ih1 n er0 ey1 t er0))
("incinerators" nil (ih0 n s ih1 n er0 ey1 t er0 z))
("incipient" nil (ih0 n s ih1 p iy0 ax n t))
("incision" nil (ih0 n s ih1 zh ax n))
("incisive" nil (ih0 n s ay1 s ih0 v))
("incisor" nil (ih0 n s ay1 z er0))
("incisors" nil (ih0 n s ay1 z er0 z))
("incite" nil (ih0 n s ay1 t))
("incited" nil (ih0 n s ay1 t ih0 d))
("inciting" nil (ih0 n s ay1 t ih0 ng))
("inclement" nil (ih0 n k l eh1 m ax n t))
("inclination" nil (ih1 n k l ax n ey1 sh ax n))
("inclinations" nil (ih1 n k l ax n ey1 sh ax n z))
("incline" v (ih0 n k l ay1 n))
("incline" n (ih1 n k l ay0 n))
("inclined" nil (ih0 n k l ay1 n d))
("inclines" v (ih0 n k l ay1 n z))
("inclines" n (ih1 n k l ay0 n z))
("inclosure" nil (ih0 n k l ow1 zh er0))
("include" nil (ih0 n k l uw1 d))
("included" nil (ih0 n k l uw1 d ax d))
("includes" nil (ih0 n k l uw1 d z))
("including" nil (ih0 n k l uw1 d ih0 ng))
("inclusion" nil (ih0 n k l uw1 zh ax n))
("inclusions" nil (ih0 n k l uw1 zh ax n z))
("inclusive" nil (ih0 n k l uw1 s ih0 v))
("inco" nil (ih1 n k ow1))
("incoherence" nil (ih0 n k ow0 hh ih1 r ax n s))
("incoherent" nil (ih0 n k ow0 hh ih1 r ax n t))
("incoherently" nil (ih0 n k ow0 hh ih1 r ax n t l iy0))
("incom" nil (ih1 ng k ax m))
("income" nil (ih1 n k ah1 m))
("incomes" nil (ih1 n k ah1 m z))
("incoming" nil (ih1 n k ah1 m ih0 ng))
("incommunicado" nil (ih1 n k ax m y uw1 n ax k aa1 d ow0))
("incomparable" nil (ih0 n k aa1 m p er0 ax b ax l))
("incomparably" nil (ih0 n k aa1 m p er0 ax b l iy0))
("incompatibility" nil (ih0 n k aa1 m p ax t ih0 b ih1 l ih0 t iy0))
("incompatible" nil (ih0 n k ax m p ae1 t ax b ax l))
("incompetence" nil (ih0 n k aa1 m p ax t ax n s))
("incompetent" nil (ih0 n k aa1 m p ax t ax n t))
("incompetents" nil (ih0 ng k aa1 m p ax t ax n t s))
("incomplete" nil (ih0 n k ax m p l iy1 t))
("incomprehensible" nil (ih0 ng k aa1 m p r ax hh eh1 n s ih0 b ax l))
("incompressible" nil (ih0 n k ax m p r eh1 s ax b ax l))
("inconceivable" nil (ih1 n k ax n s iy1 v ax b ax l))
("inconclusive" nil (ih0 n k ax n k l uw1 s ih0 v))
("inconclusively" nil (ih0 ng k aa1 n k l uw0 s ih0 v l iy0))
("incongruity" nil (ih1 ng k ao0 ng r uw1 ih0 t iy0))
("incongruous" nil (ih0 ng k ao1 ng r uw0 ax s))
("incongruously" nil (ih0 ng k ao1 ng r uw0 ax s l iy0))
("inconsequential" nil (ih0 ng k aa1 n s ax k w eh1 n ch ax l))
("inconsistencies" nil (ih0 ng k aa1 n s ih0 s t eh1 n s iy0 z))
("inconsistency" nil (ih1 n k ax n s ih1 s t ax n s iy0))
("inconsistent" nil (ih1 n k ax n s ih1 s t ax n t))
("inconspicuous" nil (ih0 ng k aa1 n s p ih0 k w ax s))
("inconstancy" nil (ih0 n k aa1 n s t ax n s iy0))
("incontinence" nil (ih0 n k aa1 n t ax n ax n s))
("incontinent" nil (ih0 n k aa1 n t ax n ax n t))
("incontrovertible" nil (ih0 ng k aa1 n t r ow0 v er1 t ih0 b ax l))
("inconvenience" nil (ih1 n k ax n v iy1 n y ax n s))
("inconvenienced" nil (ih1 n k ax n v iy1 n y ax n s t))
("inconveniences" nil (ih1 n k ax n v iy1 n y ax n s ih0 z))
("inconvenient" nil (ih1 n k ax n v iy1 n y ax n t))
("incoordination" nil (ih0 n k ow1 ao1 r d ax n ey1 sh ax n))
("incorporate" nil (ih0 n k ao1 r p er0 ey1 t))
("incorporated" nil (ih0 n k ao1 r p er0 ey1 t ax d))
("incorporates" nil (ih0 n k ao1 r p er0 ey1 t s))
("incorporating" nil (ih0 n k ao1 r p er0 ey1 t ih0 ng))
("incorporation" nil (ih0 n k ao1 r p er0 ey1 sh ax n))
("incorporations" nil (ih0 n k ao1 r p er0 ey1 sh ax n z))
("incorrect" nil (ih0 n k er0 eh1 k t))
("incorrectly" nil (ih0 n k er0 eh1 k t l iy0))
("incorrigible" nil (ih0 n k aa1 r ax jh ax b ax l))
("incorvaia" nil (ih0 n k ao0 r v aa1 y ax))
("increase" v (ih0 n k r iy1 s))
("increase" n (ih1 n k r iy1 s))
("increased" nil (ih0 n k r iy1 s t))
("increases" v (ih0 n k r iy1 s ax z))
("increases" n (ih1 n k r iy0 s ax z))
("increasing" nil (ih0 n k r iy1 s ih0 ng))
("increasingly" nil (ih0 n k r iy1 s ih0 ng l iy0))
("incredible" nil (ih0 n k r eh1 d ax b ax l))
("incredibly" nil (ih0 n k r eh1 d ax b l iy0))
("incredulity" nil (ih1 n k r ax d uw1 l ih0 t iy0))
("incredulous" nil (ih0 n k r eh1 jh ax l ax s))
("increment" nil (ih1 n k r ax m ax n t))
("incremental" nil (ih1 n k r ax m eh1 n t ax l))
("incrementally" nil (ih0 n k r ax m eh1 n t ax l iy0))
("increments" nil (ih1 ng k r ax m ax n t s))
("incriminate" nil (ih0 n k r ih1 m ax n ey1 t))
("incriminating" nil (ih0 n k r ih1 m ax n ey1 t ih0 ng))
("incrimination" nil (ih0 n k r ih1 m ax n ey1 sh ax n))
("incrustation" nil (ih1 n k r ax s t ey1 sh ax n))
("incstar" nil (ih1 ng k s t aa1 r))
("incubating" nil (ih1 n k y ax b ey1 t ih0 ng))
("incubation" nil (ih1 ng k y uw0 b ey1 sh ax n))
("incubator" nil (ih1 ng k y ax b ey1 t er0))
("incubators" nil (ih1 ng k y uw0 b ey1 t er0 z))
("inculcate" nil (ih1 ng k ax l k ey1 t))
("incumbency" nil (ih0 n k ah1 m b ax n s iy0))
("incumbent" nil (ih0 n k ah1 m b ax n t))
("incumbents" nil (ih0 n k ah1 m b ax n t s))
("incur" nil (ih0 n k er1))
("incurable" nil (ih0 n k y uh1 r ax b ax l))
("incurred" nil (ih0 n k er1 d))
("incurring" nil (ih0 n k er1 ih0 ng))
("incurs" nil (ih0 n k er1 z))
("incursion" nil (ih0 n k er1 zh ax n))
("incursions" nil (ih0 n k er1 zh ax n z))
("inda" nil (iy1 n d ax))
("indaba" nil (ih0 n d aa1 b ax))
("indal" nil (ih1 n d ax l))
("indata" nil (ih0 n d aa1 t ax))
("indebted" nil (ih0 n d eh1 t ax d))
("indebtedness" nil (ih0 n d eh1 t ih0 d n ih0 s))
("indecency" nil (ih0 n d iy1 s ax n s iy0))
("indecent" nil (ih0 n d iy1 s ax n t))
("indecision" nil (ih0 n d ih0 s ih1 zh ax n))
("indecisive" nil (ih1 n d ih0 s ay1 s ih0 v))
("indecisiveness" nil (ih0 n d eh1 s ih0 s ih0 v n ax s))
("indeed" nil (ih0 n d iy1 d))
("indefatigable" nil (ih1 n d ih0 f ae1 t ih0 g ax b ax l))
("indefensible" nil (ih0 n d ih0 f eh1 n s ax b ax l))
("indefinable" nil (ih0 n d ih0 f ay1 n ax b ax l))
("indefinite" nil (ih0 n d eh1 f ax n ax t))
("indefinitely" nil (ih0 n d eh1 f ax n ax t l iy0))
("indelible" nil (ih0 n d eh1 l ih0 b ax l))
("indelibly" nil (ih0 n d eh1 l ax b l iy0))
("indelicate" nil (ih0 n d eh1 l ih0 k ax t))
("indelicato" nil (ih0 n d eh0 l iy0 k aa1 t ow0))
("indemnification" nil (ih0 n d eh1 m n ax f ih0 k ey1 sh ax n))
("indemnified" nil (ih0 n d eh1 m n ax f ay1 d))
("indemnify" nil (ih0 n d eh1 m n ax f ay1))
("indemnifying" nil (ih0 n d eh1 m n ih0 f ay1 ih0 ng))
("indemnities" nil (ih0 n d eh1 m n ih0 t iy0 z))
("indemnity" nil (ih0 n d eh1 m n ax t iy0))
("indent" v (ih0 n d eh1 n t))
("indent" n (ih1 n d eh0 n t))
("indents" v (ih0 n d eh1 n t s))
("indents" n (ih1 n d eh0 n t s))
("indenture" nil (ih0 n d eh1 n ch er0))
("indentured" nil (ih0 n d eh1 n ch er0 d))
("indentures" nil (ih0 n d eh1 n ch er0 z))
("independence" nil (ih1 n d ih0 p eh1 n d ax n s))
("independent" nil (ih1 n d ih0 p eh1 n d ax n t))
("independently" nil (ih1 n d ih0 p eh1 n d ax n t l iy0))
("independents" nil (ih1 n d ih0 p eh1 n d ax n t s))
("inderal" nil (ih1 n d er0 ax l))
("indestructibility" nil (ih1 n d ax s t r ah1 k t ih0 b ih1 l ih0 t iy0))
("indestructible" nil (ih1 n d ax s t r ah1 k t ih0 b ax l))
("indeterminate" nil (ih1 n d ih0 t er1 m ih0 n ih0 t))
("index" nil (ih1 n d eh0 k s))
("indexation" nil (ih1 n d eh1 k s ey1 sh ax n))
("indexed" nil (ih1 n d eh0 k s t))
("indexers" nil (ih1 n d eh1 k s er0 z))
("indexes" nil (ih1 n d eh0 k s ih0 z))
("indexing" nil (ih1 n d eh0 k s ih0 ng))
("india" nil (ih1 n d iy0 ax))
("indian" nil (ih1 n d iy0 ax n))
("indiana" nil (ih1 n d iy0 ae1 n ax))
("indianapolis" nil (ih1 n d iy0 ax n ae1 p ax l ih0 s))
("indianians" nil (ih1 n d iy0 ae1 n iy0 ax n z))
("indians" nil (ih1 n d iy0 ax n z))
("indic" nil (ih1 n d ih0 k))
("indicate" nil (ih1 n d ax k ey1 t))
("indicated" nil (ih1 n d ax k ey1 t ax d))
("indicates" nil (ih1 n d ax k ey1 t s))
("indicating" nil (ih1 n d ax k ey1 t ih0 ng))
("indication" nil (ih1 n d ax k ey1 sh ax n))
("indications" nil (ih1 n d ax k ey1 sh ax n z))
("indicative" nil (ih0 n d ih1 k ax t ih0 v))
("indicator" nil (ih1 n d ax k ey1 t er0))
("indicators" nil (ih1 n d ax k ey1 t er0 z))
("indices" nil (ih1 n d ih0 s iy1 z))
("indict" nil (ih0 n d ay1 t))
("indicted" nil (ih0 n d ay1 t ax d))
("indicting" nil (ih0 n d ay1 t ih0 ng))
("indictment" nil (ih0 n d ay1 t m ax n t))
("indictments" nil (ih0 n d ay1 t m ax n t s))
("indies" nil (ih1 n d iy0 z))
("indifference" nil (ih0 n d ih1 f er0 ax n s))
("indifferent" nil (ih0 n d ih1 f r ax n t))
("indigenous" nil (ih0 n d ih1 jh ax n ax s))
("indigenously" nil (ih0 n d ih1 jh ax n ax s l iy0))
("indigent" nil (ih1 n d ih0 jh ax n t))
("indigents" nil (ih1 n d ih0 jh ax n t s))
("indigestion" nil (ih1 n d ay0 jh eh1 s ch ax n))
("indignant" nil (ih0 n d ih1 g n ax n t))
("indignantly" nil (ih0 n d ih1 g n ax n t l iy0))
("indignation" nil (ih1 n d ih0 g n ey1 sh ax n))
("indignities" nil (ih0 n d ih1 g n ax t iy0 z))
("indignity" nil (ih0 n d ih1 g n ax t iy0))
("indigo" nil (ih1 n d ax g ow1))
("indira" nil (ih0 n d ih1 r ax))
("indirect" nil (ih0 n d er0 eh1 k t))
("indirectly" nil (ih0 n d er0 eh1 k t l iy0))
("indiscreet" nil (ih0 n d ih0 s k r iy1 t))
("indiscretions" nil (ih1 n d ih0 s k r eh1 sh ax n z))
("indiscriminate" nil (ih0 n d ih0 s k r ih1 m ax n ax t))
("indiscriminately" nil (ih1 n d ih0 s k r ih1 m ax n ax t l iy0))
("indispensable" nil (ih1 n d ih0 s p eh1 n s ax b ax l))
("indispensible" nil (ih1 n d ih0 s p eh1 n s ih0 b ax l))
("indisputable" nil (ih1 n d ih0 s p y uw1 t ax b ax l))
("indisputably" nil (ih0 n d ih1 s p y uw0 t ae1 b l iy0))
("indistinct" nil (ih0 n d ih0 s t ih1 ng k t))
("indistinguishable" nil (ih0 n d ih0 s t ih1 ng g w ih0 sh ax b ax l))
("indite" nil (ih0 n d ay1 t))
("indited" nil (ih0 n d ay1 t ih0 d))
("indium" nil (ih1 n d iy0 ax m))
("individual" nil (ih1 n d ax v ih1 jh ax w ax l))
("individualism" nil (ih1 n d ih0 v ih0 d uw1 ax l ih1 z ax m))
("individualist" nil (ih1 n d ih0 v ih0 d uw1 ax l ih0 s t))
("individualistic" nil (ih1 n d ih0 v ih1 jh uw0 ax l ih1 s t ih0 k))
("individualists" nil (ih1 n d ax v ih1 jh uw0 ax l ih0 s t s))
("individuality" nil (ih0 n d ih1 v ih0 jh uw0 ae1 l ih0 t iy0))
("individualized" nil (ih1 n d ih0 v ih1 jh uw0 ax l ay0 z d))
("individually" nil (ih1 n d ih0 v ih1 jh uw0 ax l iy0))
("individuals" nil (ih1 n d ax v ih1 jh ax w ax l z))
("indivisible" nil (ih1 n d ih0 v ih1 s ih0 b ax l))
("indo" nil (ih1 n d ow0))
("indochina" nil (ih1 n d ow0 ch ay1 n ax))
("indochinese" nil (ih1 n d ow0 ch ay1 n iy1 z))
("indoctrinated" nil (ih0 n d aa1 k t r ax n ey1 t ih0 d))
("indoctrination" nil (ih0 n d aa1 k t r ax n ey1 sh ax n))
("indolent" nil (ih1 n d ax l ax n t))
("indomitable" nil (ih0 n d aa1 m ax t ax b ax l))
("indonesia" nil (ih1 n d ow0 n iy1 zh ax))
("indonesian" nil (ih1 n d ow0 n iy1 zh ax n))
("indonesians" nil (ih1 n d ow0 n iy1 s iy0 ax n z))
("indoor" nil (ih1 n d ao1 r))
("indoors" nil (ih1 n d ao1 r z))
("indosuez" nil (ih1 n d ow0 s uw0 ey1 z))
("indovina" nil (ih0 n d ow0 v iy1 n ax))
("indri" nil (ih1 n d r iy1))
("induce" nil (ih0 n d uw1 s))
("induced" nil (ih0 n d uw1 s t))
("inducement" nil (ih0 n d uw1 s m ax n t))
("inducements" nil (ih0 n d uw1 s m ax n t s))
("induces" nil (ih0 n d uw1 s ih0 z))
("inducing" nil (ih0 n d uw1 s ih0 ng))
("inductance" nil (ih0 n d ah1 k t ax n s))
("inducted" nil (ih0 n d ah1 k t ax d))
("inductees" nil (ih0 n d ah1 k t iy1 z))
("induction" nil (ih0 n d ah1 k sh ax n))
("inductor" nil (ih0 n d ah1 k t er0))
("indulge" nil (ih0 n d ah1 l jh))
("indulged" nil (ih0 n d ah1 l jh d))
("indulgence" nil (ih0 n d ah1 l jh ax n s))
("indulgences" nil (ih0 n d ah1 l jh ax n s ax z))
("indulgent" nil (ih0 n d ah1 l jh ax n t))
("indulges" nil (ih0 n d ah1 l jh ih0 z))
("indulging" nil (ih0 n d ah1 l jh ih0 ng))
("indus" nil (ih1 n d ax s))
("industri" nil (ih0 n d ah1 s t r iy0))
("industria" nil (ih0 n d ah1 s t r iy0 ax))
("industrial" nil (ih0 n d ah1 s t r iy0 ax l))
("industriale" nil (ih1 n d ax s t r iy0 aa1 l iy0))
("industriali" nil (ih0 n d ah1 s t r iy0 aa1 l iy0))
("industrialist" nil (ih0 n d ah1 s t r iy0 ax l ih0 s t))
("industrialists" nil (ih0 n d ah1 s t r iy0 ax l ih0 s t s))
("industrialization" nil (ih0 n d ah1 s t r iy0 ax l ih0 z ey1 sh ax n))
("industrialize" nil (ih0 n d ah1 s t r iy0 ax l ay1 z))
("industrialized" nil (ih0 n d ah1 s t r iy0 ax l ay1 z d))
("industrializing" nil (ih0 n d ah1 s t r iy0 ax l ay1 z ih0 ng))
("industrially" nil (ih0 n d ah1 s t r iy0 ax l iy0))
("industrials" nil (ih0 n d ah1 s t r iy0 ax l z))
("industrie" nil (ih1 n d ax s t r iy0))
("industrielle" nil (ih0 n d ah1 s t r iy0 eh1 l))
("industrier" nil (ih1 n d ax s t r iy1 er0))
("industries" nil (ih1 n d ax s t r iy0 z))
("industrious" nil (ih0 n d ah1 s t r iy0 ax s))
("industrivaerden" nil (ih1 n d ax s t r iy0 v eh1 r d ax n))
("industry" nil (ih1 n d ax s t r iy0))
("industrywide" nil (ih1 n d ax s t r iy0 w ay1 d))
("indy" nil (ih1 n d iy0))
("inedible" nil (ih0 n eh1 d ax b ax l))
("ineffable" nil (ih0 n eh1 f ax b ax l))
("ineffective" nil (ih0 n ih0 f eh1 k t ih0 v))
("ineffectiveness" nil (ih1 n ax f eh1 k t ih0 v n ax s))
("ineffectual" nil (ih1 n ax f eh1 k ch uw0 ax l))
("inefficiencies" nil (ih0 n ih0 f ih1 sh ax n s iy0 z))
("inefficiency" nil (ih0 n ih0 f ih1 sh ax n s iy0))
("inefficient" nil (ih0 n ih0 f ih1 sh ax n t))
("inefficiently" nil (ih0 n ih0 f ih1 sh ax n t l iy0))
("inelastic" nil (ih1 n ax l ae1 s t ih0 k))
("ineligible" nil (ih0 n eh1 l ih0 jh ax b ax l))
("inept" nil (ih0 n eh1 p t))
("ineptitude" nil (ih0 n eh1 p t ih0 t uw1 d))
("ineptly" nil (ih0 n eh1 p t l iy0))
("inequalities" nil (ih1 n ax k w aa1 l ih0 t iy0 z))
("inequality" nil (ih1 n ih0 k w aa1 l ax t iy0))
("inequitable" nil (ih0 n eh1 k w ax t ax b ax l))
("inequities" nil (ih0 n eh1 k w ax t iy0 z))
("inequity" nil (ih0 n eh1 k w ax t iy0))
("inerco" nil (ih0 n er1 k ow0))
("inert" nil (ih0 n er1 t))
("inertia" nil (ih0 n er1 sh ax))
("inertial" nil (ih0 n er1 sh ax l))
("ines" nil (ih1 n ih0 s))
("inescapable" nil (ih1 n ih0 s k ey1 p ax b ax l))
("inevitability" nil (ih0 n eh1 v ih0 t ax b ih1 l ih0 t iy0))
("inevitable" nil (ih0 n eh1 v ax t ax b ax l))
("inevitably" nil (ih0 n eh1 v ax t ax b l iy0))
("inexact" nil (ih0 n ih0 g z ae1 k t))
("inexcusable" nil (ih0 n ih0 k s k y uw1 z ax b ax l))
("inexhaustible" nil (ih0 n ih0 g z ao1 s t ax b ax l))
("inexorable" nil (ih1 n eh1 k s er0 ax b ax l))
("inexorably" nil (ih0 n eh1 k s er0 ax b l iy0))
("inexpensive" nil (ih1 n ih0 k s p eh1 n s ih0 v))
("inexpensively" nil (ih1 n ax k s p eh1 n s ih0 v l iy0))
("inexperience" nil (ih1 n ih0 k s p ih1 r iy0 ax n s))
("inexperienced" nil (ih0 n ih0 k s p ih1 r iy0 ax n s t))
("inexplicable" nil (ih1 n ax k s p l ih1 s ax b ax l))
("inexplicably" nil (ih1 n ih0 k s p l ih1 k ax b l iy0))
("inextricably" nil (ih0 n eh1 k s t r ih0 k ax b l iy0))
("inez" nil (ih0 n eh1 z))
("infallibility" nil (ih0 n f ae1 l ih0 b ih1 l ih0 t iy0))
("infallible" nil (ih0 n f ae1 l ax b ax l))
("infamous" nil (ih1 n f ax m ax s))
("infamy" nil (ih1 n f ax m iy0))
("infancy" nil (ih1 n f ax n s iy0))
("infant" nil (ih1 n f ax n t))
("infante" nil (ih0 n f aa1 n t ey1))
("infanticide" nil (ih0 n f ae1 n t ax s ay1 d))
("infantile" nil (ih1 n f ax n t ih0 l))
("infantino" nil (ih0 n f aa0 n t iy1 n ow0))
("infantry" nil (ih1 n f ax n t r iy0))
("infantryman" nil (ih1 n f ae0 n t r iy0 m ax n))
("infantrymen" nil (ih1 n f ae0 n t r iy0 m ax n))
("infants" nil (ih1 n f ax n t s))
("infarction" nil (ih0 n f aa1 r k sh ax n))
("infatuated" nil (ih0 n f ae1 ch uw0 ey1 t ih0 d))
("infatuation" nil (ih0 n f ae1 ch uw0 ey1 sh ax n))
("infeasible" nil (ih0 n f iy1 z ax b ax l))
("infect" nil (ih0 n f eh1 k t))
("infected" nil (ih0 n f eh1 k t ax d))
("infecting" nil (ih0 n f eh1 k t ih0 ng))
("infection" nil (ih0 n f eh1 k sh ax n))
("infections" nil (ih0 n f eh1 k sh ax n z))
("infectious" nil (ih0 n f eh1 k sh ax s))
("infective" nil (ih0 n f eh1 k t ih0 v))
("infects" nil (ih0 n f eh1 k t s))
("infer" nil (ih0 n f er1))
("inference" nil (ih1 n f er0 ax n s))
("inferences" nil (ih1 n f er0 eh1 n s ih0 z))
("inferior" nil (ih0 n f ih1 r iy0 er0))
("inferiority" nil (ih1 n f ih0 r iy0 ao1 r ih0 t iy0))
("infernal" nil (ih0 n f er1 n ax l))
("inferno" nil (ih0 n f er1 n ow0))
("inferred" nil (ih0 n f er1 d))
("inferring" nil (ih0 n f er1 ih0 ng))
("infertile" nil (ih0 n f er1 t ax l))
("infertility" nil (ih1 n f er0 t ih1 l ih0 t iy0))
("infestation" nil (ih0 n f eh1 s t ey1 sh ax n))
("infestations" nil (ih1 n f eh1 s t ey1 sh ax n z))
("infested" nil (ih0 n f eh1 s t ax d))
("infests" nil (ih0 n f eh1 s t s))
("infidel" nil (ih1 n f ih0 d eh1 l))
("infidelities" nil (ih1 n f ih0 d eh1 l ih0 t iy0 z))
("infidelity" nil (ih1 n f ih0 d eh1 l ih0 t iy0))
("infield" nil (ih1 n f iy1 l d))
("infielder" nil (ih1 n f iy1 l d er0))
("infielders" nil (ih1 n f iy1 l d er0 z))
("infighting" nil (ih1 n f ay1 t ih0 ng))
("infiltrate" nil (ih0 n f ih1 l t r ey1 t))
("infiltrated" nil (ih0 n f ih1 l t r ey1 t ih0 d))
("infiltrating" nil (ih0 n f ih1 l t r ey1 t ih0 ng))
("infiltration" nil (ih1 n f ih0 l t r ey1 sh ax n))
("infinger" nil (ih1 n f ih0 ng er0))
("infinite" nil (ih1 n f ax n ax t))
("infinitely" nil (ih1 n f ax n ax t l iy0))
("infinitesimal" nil (ih1 n f ih0 n ih0 t eh1 s ih0 m ax l))
("infiniti" nil (ih0 n f ih1 n ih0 t iy0))
("infinity" nil (ih0 n f ih1 n ax t iy0))
("infirm" nil (ih0 n f er1 m))
("infirmary" nil (ih0 n f er1 m er0 iy0))
("inflame" nil (ih0 n f l ey1 m))
("inflamed" nil (ih0 n f l ey1 m d))
("inflaming" nil (ih0 n f l ey1 m ih0 ng))
("inflammable" nil (ih0 n f l ae1 m ax b ax l))
("inflammation" nil (ih1 n f l ax m ey1 sh ax n))
("inflammatory" nil (ih0 n f l ae1 m ax t ao1 r iy0))
("inflatable" nil (ih0 n f l ey1 t ax b ax l))
("inflate" nil (ih0 n f l ey1 t))
("inflated" nil (ih0 n f l ey1 t ax d))
("inflates" nil (ih0 n f l ey1 t s))
("inflating" nil (ih0 n f l ey1 t ih0 ng))
("inflation" nil (ih0 n f l ey1 sh ax n))
("inflationary" nil (ih0 n f l ey1 sh ax n eh1 r iy0))
("inflator" nil (ih0 n f l ey1 t er0))
("inflected" nil (ih0 n f l eh1 k t ax d))
("inflection" nil (ih0 n f l eh1 k sh ax n))
("inflections" nil (ih0 n f l eh1 k sh ax n z))
("inflexibility" nil (ih0 n f l eh1 k s ih0 b ih1 l ih0 t iy0))
("inflexible" nil (ih0 n f l eh1 k s ax b ax l))
("inflict" nil (ih0 n f l ih1 k t))
("inflicted" nil (ih0 n f l ih1 k t ih0 d))
("inflicting" nil (ih0 n f l ih1 k t ih0 ng))
("infliction" nil (ih0 n f l ih1 k sh ax n))
("inflicts" nil (ih0 n f l ih1 k t s))
("inflight" nil (ih1 n f l ay1 t))
("inflow" nil (ih1 n f l ow1))
("inflows" nil (ih1 n f l ow1 z))
("influence" nil (ih1 n f l uw0 ax n s))
("influenced" nil (ih1 n f l uw0 ax n s t))
("influences" nil (ih1 n f l uw1 ax n s ih0 z))
("influencing" nil (ih1 n f l uw1 ax n s ih0 ng))
("influential" nil (ih1 n f l uw0 eh1 n ch ax l))
("influenza" nil (ih1 n f l uw0 eh1 n z ax))
("influx" nil (ih1 n f l ah1 k s))
("info" nil (ih1 n f ow0))
("infocorp" nil (ih1 n f ow0 k ao1 r p))
("infomercial" nil (ih1 n f ow0 m er1 sh ax l))
("infomercials" nil (ih1 n f ow0 m er1 sh ax l z))
("inform" nil (ih0 n f ao1 r m))
("informal" nil (ih0 n f ao1 r m ax l))
("informality" nil (ih1 n f er0 m ae1 l ih0 t iy0))
("informally" nil (ih0 n f ao1 r m ax l iy0))
("informals" nil (ih0 n f ao1 r m ax l z))
("informant" nil (ih0 n f ao1 r m ax n t))
("informants" nil (ih0 n f ao1 r m ax n t s))
("informatics" nil (ih1 n f er0 m ae1 t ih0 k s))
("information" nil (ih1 n f er0 m ey1 sh ax n))
("informational" nil (ih1 n f er0 m ey1 sh ax n ax l))
("informations" nil (ih1 n f er0 m ey1 sh ax n z))
("informative" nil (ih0 n f ao1 r m ax t ih0 v))
("informed" nil (ih0 n f ao1 r m d))
("informer" nil (ih0 n f ao1 r m er0))
("informers" nil (ih0 n f ao1 r m er0 z))
("informing" nil (ih0 n f ao1 r m ih0 ng))
("informix" nil (ih0 n f ao1 r m ih0 k s))
("informs" nil (ih0 n f ao1 r m z))
("infoscan" nil (ih1 n f ow0 s k ae1 n))
("infotech" nil (ih1 n f ow0 t eh1 k))
("infotechnology" nil (ih1 n f ow0 t eh0 k n aa1 l ax jh iy0))
("infotron" nil (ih1 n f ow0 t r aa1 n))
("infoworld" nil (ih1 n f ow0 w er1 l d))
("infra" nil (ih1 n f r ax))
("infraction" nil (ih0 n f r ae1 k sh ax n))
("infractions" nil (ih0 n f r ae1 k sh ax n z))
("infrared" nil (ih1 n f r er0 eh1 d))
("infrastructural" nil (ih1 n f r ax s t r ah1 k ch er0 ax l))
("infrastructure" nil (ih1 n f r ax s t r ah1 k ch er0))
("infrastructures" nil (ih1 n f r ax s t r ah1 k ch er0 z))
("infrequency" nil (ih0 n f r iy1 k w ax n s iy0))
("infrequent" nil (ih0 n f r iy1 k w ax n t))
("infrequently" nil (ih0 n f r iy1 k w ax n t l iy0))
("infringe" nil (ih0 n f r ih1 n jh))
("infringed" nil (ih0 n f r ih1 n jh d))
("infringement" nil (ih0 n f r ih1 n jh m ax n t))
("infringements" nil (ih0 n f r ih1 n jh m ax n t s))
("infringes" nil (ih0 n f r ih1 n jh ih0 z))
("infringing" nil (ih0 n f r ih1 n jh ih0 ng))
("infuriate" nil (ih0 n f y uh1 r iy0 ey1 t))
("infuriated" nil (ih0 n f y uh1 r iy0 ey1 t ax d))
("infuriates" nil (ih0 n f y uh1 r iy0 ey1 t s))
("infuriating" nil (ih0 n f y uh1 r iy0 ey1 t ih0 ng))
("infuse" nil (ih0 n f y uw1 z))
("infused" nil (ih0 n f y uw1 z d))
("infuses" nil (ih0 n f y uw1 z ih0 z))
("infusing" nil (ih0 n f y uw1 z ih0 ng))
("infusion" nil (ih0 n f y uw1 zh ax n))
("infusions" nil (ih0 n f y uw1 zh ax n z))
("ing" nil (ih1 ng))
("inga" nil (iy1 ng g ax))
("ingalls" nil (ih0 ng g ao1 l z))
("ingalsbe" nil (ih1 ng g ax l s b iy0))
("ingar" nil (ih1 ng g er0))
("ingber" nil (ih1 ng b er0))
("inge" nil (ih1 n jh))
("ingelheim" nil (ih1 ng g ax l hh ay1 m))
("ingels" nil (ih1 ng g ax l z))
("ingemar" nil (ih1 ng g ih0 m er0))
("ingenious" nil (ih0 n jh iy1 n y ax s))
("ingeniously" nil (ih0 n jh iy1 n y ax s l iy0))
("ingenito" nil (ih0 ng g eh0 n iy1 t ow0))
("ingenue" nil (ih0 n jh eh1 n y uw0))
("ingenuity" nil (ih0 n jh ax n uw1 ax t iy0))
("ingenuous" nil (ih0 n jh eh1 n y uw0 ax s))
("inger" nil (ih1 ng er0))
("ingersoll" nil (ih1 ng g er0 s ao0 l))
("ingerson" nil (ih1 ng g er0 s ax n))
("ingest" nil (ih0 n jh eh1 s t))
("ingested" nil (ih0 n jh eh1 s t ax d))
("ingesting" nil (ih0 n jh eh1 s t ih0 ng))
("ingestion" nil (ih0 n jh eh1 s ch ax n))
("ingham" nil (ih1 ng ax m))
("inghram" nil (ih0 ng g r ae1 m))
("ingimarson" nil (ih1 ng ax m aa1 r s ax n))
("ingle" nil (ih1 ng g ax l))
("inglebert" nil (ih1 ng g l ih0 b er0 t))
("inglenook" nil (ih1 ng g ax l n uh1 k))
("ingles" nil (ih1 ng g ax l z))
("inglese" nil (ih1 ng g l iy0 z))
("inglett" nil (ih0 ng g l eh1 t))
("inglewood" nil (ih1 ng g ax l w uh1 d))
("inglis" nil (ih1 ng g l ih0 s))
("inglish" nil (ih1 ng g ax l ih0 sh))
("inglorious" nil (ih0 n g l ao1 r iy0 ax s))
("ingman" nil (ih1 ng m ax n))
("ingmar" nil (ih1 ng m aa0 r))
("ingmire" nil (ih1 ng m ay0 r))
("ingo" nil (ih1 ng g ow0))
("ingoglia" nil (ih0 ng g ow1 g l iy0 ax))
("ingold" nil (ih1 ng g ow0 l d))
("ingot" nil (ih1 ng g ax t))
("ingots" nil (ih1 ng g ax t s))
("ingraham" nil (ih1 ng g r ax hh ae1 m))
("ingrained" nil (ih0 n g r ey1 n d))
("ingram" nil (ih1 ng g r ax m))
("ingrao" nil (iy1 ng g r aw0))
("ingrassia" nil (ih0 ng g r aa1 sh ax))
("ingratiate" nil (ih0 n g r ey1 sh iy0 ey1 t))
("ingratiating" nil (ih0 n g r ey1 sh iy0 ey1 t ih0 ng))
("ingredient" nil (ih0 n g r iy1 d iy0 ax n t))
("ingredients" nil (ih0 n g r iy1 d iy0 ax n t s))
("ingria" nil (ih1 ng g r iy0 ax))
("ingrid" nil (ih1 ng g r ih0 d))
("ingrum" nil (ih1 ng g r ax m))
("ingvar" nil (ih1 ng v aa0 r))
("ingwersen" nil (ih1 ng g w er0 s ax n))
("inhabit" nil (ih0 n hh ae1 b ax t))
("inhabitants" nil (ih0 n hh ae1 b ax t ax n t s))
("inhabitation" nil (ih0 n hh ae1 b ax t ey1 sh ax n))
("inhabited" nil (ih0 n hh ae1 b ax t ax d))
("inhalable" nil (ih0 n hh ey1 l ax b ax l))
("inhalation" nil (ih0 n ax l ey1 sh ax n))
("inhale" nil (ih0 n hh ey1 l))
("inhaled" nil (ih0 n hh ey1 l d))
("inhaler" nil (ih0 n hh ey1 l er0))
("inhaling" nil (ih0 n hh ey1 l ih0 ng))
("inherent" nil (ih0 n hh ih1 r ax n t))
("inherently" nil (ih0 n hh ih1 r ax n t l iy0))
("inherit" nil (ih0 n hh eh1 r ax t))
("inheritable" nil (ih0 n hh eh1 r ax t ax b ax l))
("inheritance" nil (ih0 n hh eh1 r ax t ax n s))
("inherited" nil (ih0 n hh eh1 r ax t ih0 d))
("inheriting" nil (ih0 n hh eh1 r ax t ih0 ng))
("inherits" nil (ih0 n hh eh1 r ax t s))
("inhibit" nil (ih0 n hh ih1 b ax t))
("inhibited" nil (ih0 n hh ih1 b ax t ih0 d))
("inhibiting" nil (ih0 n hh ih1 b ax t ih0 ng))
("inhibition" nil (ih1 n hh ax b ih1 sh ax n))
("inhibitions" nil (ih1 n hh ax b ih1 sh ax n z))
("inhibitor" nil (ih0 n hh ih1 b ax t er0))
("inhibitors" nil (ih0 n hh ih1 b ax t er0 z))
("inhibitory" nil (ih0 n hh ih1 b ax t ao1 r iy0))
("inhibits" nil (ih0 n hh ih1 b ax t s))
("inhofe" nil (ih1 n hh ow1 f))
("inhospitable" nil (ih0 n hh aa1 s p ax t ax b ax l))
("inhouse" nil (ih1 n hh aw1 s))
("inhuman" nil (ih0 n hh y uw1 m ax n))
("inhumane" nil (ih0 n hh y uw0 m ey1 n))
("iniga" nil (ih0 n iy1 g ax))
("iniguez" nil (ih0 n iy1 g eh0 z))
("iniki" nil (iy0 n iy1 k iy0))
("inimical" nil (ih0 n ih1 m ih0 k ax l))
("inimitable" nil (ih0 n ih1 m ax t ax b ax l))
("iniss" nil (ih1 n ih0 s))
("initial" nil (ih0 n ih1 sh ax l))
("initialed" nil (ih0 n ih1 sh ax l d))
("initially" nil (ih0 n ih1 sh ax l iy0))
("initials" nil (ih0 n ih1 sh ax l z))
("initiate" n (ih0 n ih1 sh iy0 ax t))
("initiate" v (ih0 n ih1 sh iy0 ey1 t))
("initiated" nil (ih0 n ih1 sh iy0 ey1 t ax d))
("initiates" n (ih0 n ih1 sh iy0 ax t s))
("initiates" v (ih0 n ih1 sh iy0 ey1 t s))
("initiating" nil (ih0 n ih1 sh iy0 ey1 t ih0 ng))
("initiation" nil (ih0 n ih1 sh iy0 ey1 sh ax n))
("initiative" nil (ih0 n ih1 sh ax t ih0 v))
("initiatives" nil (ih0 n ih1 sh ax t ih0 v z))
("initiator" nil (ih1 n ih0 sh iy1 ey0 t er0))
("initiators" nil (ih0 n ih1 sh iy0 ey0 t er0 z))
("initio" nil (ih0 n ih1 t iy0 ow0))
("iniziativa" nil (ih1 n ih0 z iy1 ax t iy1 v ax))
("inject" nil (ih0 n jh eh1 k t))
("injectable" nil (ih0 n jh eh1 k t ax b ax l))
("injected" nil (ih0 n jh eh1 k t ax d))
("injecting" nil (ih0 n jh eh1 k t ih0 ng))
("injection" nil (ih0 n jh eh1 k sh ax n))
("injections" nil (ih0 n jh eh1 k sh ax n z))
("injector" nil (ih0 n jh eh1 k t er0))
("injectors" nil (ih0 n jh eh1 k t er0 z))
("injects" nil (ih0 n jh eh1 k t s))
("injuction" nil (ih0 n jh ah1 k sh ax n))
("injunction" nil (ih0 n jh ah1 ng k sh ax n))
("injunctions" nil (ih0 n jh ah1 ng k sh ax n z))
("injunctive" nil (ih0 n jh ah1 ng k t ih0 v))
("injure" nil (ih1 n jh er0))
("injured" nil (ih1 n jh er0 d))
("injures" nil (ih1 n jh er0 z))
("injuries" nil (ih1 n jh er0 iy0 z))
("injuring" nil (ih1 n jh er0 ih0 ng))
("injurious" nil (ih0 n jh uh1 r iy0 ax s))
("injury" nil (ih1 n jh er0 iy0))
("injustice" nil (ih0 n jh ah1 s t ih0 s))
("injustices" nil (ih0 n jh ah1 s t ax s ih0 z))
("ink" nil (ih1 ng k))
("inkatha" nil (ih0 ng k ae1 th ax))
("inkblot" nil (ih1 ng k b l aa1 t))
("inkjet" nil (ih1 ng k jh eh1 t))
("inkling" nil (ih1 ng k l ih0 ng))
("inks" nil (ih1 ng k s))
("inky" nil (ih1 ng k iy0))
("inlaid" nil (ih1 n l ey1 d))
("inland" nil (ih1 n l ae1 n d))
("inlaws" nil (ih0 n l ao1 z))
("inlay" nil (ih1 n l ey1))
("inlet" nil (ih1 n l eh1 t))
("inlets" nil (ih1 n l eh1 t s))
("inlow" nil (ih0 n l ow1))
("inmac" nil (ih1 n m ae0 k))
("inman" nil (ih1 n m ax n))
("inmarsat" nil (ih0 n m aa1 r s ae1 t))
("inmate" nil (ih1 n m ey1 t))
("inmates" nil (ih1 n m ey1 t s))
("inmex" nil (ih1 n m eh1 k s))
("inmobiliaria" nil (ih0 n m ow1 b ax l iy0 eh1 r iy0 ax))
("inmon" nil (ih1 n m ax n))
("inmont" nil (ih1 n m aa1 n t))
("inn" nil (ih1 n))
("innards" nil (ih1 n er0 d z))
("innate" nil (ih0 n ey1 t))
("inner" nil (ih1 n er0))
("innermost" nil (ih1 n er0 m ow1 s t))
("innerspace" nil (ih1 n er0 s p ey1 s))
("innes" nil (ih1 n ax s))
("inness" nil (ih1 n ih0 s))
("inning" nil (ih1 n ih0 ng))
("innings" nil (ih1 n ih0 ng z))
("innis" nil (ih1 n ih0 s))
("inniss" nil (ih1 n ih0 s))
("innkeeper" nil (ih1 n k iy1 p er0))
("innkeepers" nil (ih1 n k iy1 p er0 z))
("inno" nil (ih1 n ow0))
("innocence" nil (ih1 n ax s ax n s))
("innocent" nil (ih1 n ax s ax n t))
("innocenti" nil (ih0 n ow0 ch eh1 n t iy0))
("innocently" nil (ih1 n ax s ax n t l iy0))
("innocents" nil (ih1 n ax s ax n t s))
("innocuous" nil (ih0 n aa1 k y uw0 ax s))
("innominate" nil (ih0 n aa1 m ax n ax t))
("innopac" nil (ih1 n ax p ae1 k))
("innovate" nil (ih1 n ax v ey1 t))
("innovated" nil (ih1 n ax v ey1 t ih0 d))
("innovation" nil (ih1 n ax v ey1 sh ax n))
("innovations" nil (ih1 n ax v ey1 sh ax n z))
("innovative" nil (ih1 n ax v ey1 t ih0 v))
("innovator" nil (ih1 n ax v ey1 t er0))
("innovators" nil (ih1 n ax v ey1 t er0 z))
("inns" nil (ih1 n z))
("innuendo" nil (ih0 n y uw0 eh1 n d ow0))
("innuendoes" nil (ih0 n y uw0 eh1 n d ow0 z))
("innuendos" nil (ih0 n y uw0 eh1 n d ow0 z))
("innumerable" nil (ih0 n uw1 m er0 ax b ax l))
("inocencio" nil (ih0 n ow0 ch eh1 n ch iy0 ow0))
("inoculate" nil (ih0 n aa1 k y ax l ey1 t))
("inoculated" nil (ih0 n aa1 k y ax l ey1 t ih0 d))
("inoculation" nil (ih0 n aa1 k y ax l ey1 sh ax n))
("inoffensive" nil (ih0 n ax f eh1 n s ih0 v))
("inoperable" nil (ih0 n aa1 p er0 ax b ax l))
("inoperative" nil (ih0 n aa1 p er0 ax t ih0 v))
("inopportune" nil (ih0 n aa1 p er0 t uw1 n))
("inordinate" nil (ih0 n ao1 r d ax n ih0 t))
("inordinately" nil (ih0 n ao1 r d ax n ax t l iy0))
("inorganic" nil (ih0 n ao0 r g ae1 n ih0 k))
("inoue" nil (ih0 n ow1 ey0))
("inouye" nil (ih0 n uw1 ey0))
("inpatient" nil (ih1 n p ey1 sh ax n t))
("inpatients" nil (ih0 n p ey1 sh ax n t s))
("input" nil (ih1 n p uh1 t))
("inputs" nil (ih1 n p uh1 t s))
("inquest" nil (ih1 n k w eh1 s t))
("inquire" nil (ih0 n k w ay1 r))
("inquired" nil (ih0 n k w ay1 er0 d))
("inquirer" nil (ih0 n k w ay1 r er0))
("inquires" nil (ih0 n k w ay1 er0 z))
("inquiries" nil (ih0 n k w ay1 er0 iy0 z))
("inquiring" nil (ih0 n k w ay1 er0 ih0 ng))
("inquiry" nil (ih0 n k w ay1 r iy0))
("inquisition" nil (ih1 n k w ax z ih1 sh ax n))
("inquisitive" nil (ih0 n k w ih1 z ih0 t ih0 v))
("inquisitors" nil (ih0 n k w ih1 z ax t er0 z))
("inroad" nil (ih1 n r ow1 d))
("inroads" nil (ih1 n r ow1 d z))
("ins" nil (ih1 n z))
("insalaco" nil (ih0 n s aa0 l aa1 k ow0))
("insane" nil (ih0 n s ey1 n))
("insanity" nil (ih0 n s ae1 n ax t iy0))
("insatiable" nil (ih0 n s ey1 sh ax b ax l))
("inscho" nil (ih1 n sh ow0))
("insco" nil (iy1 n s k ow0))
("inscoe" nil (ih0 n s k ow1))
("inscore" nil (ih0 n s k ao1 r iy0))
("inscribed" nil (ih0 n s k r ay1 b d))
("inscription" nil (ih0 n s k r ih1 p sh ax n))
("inscriptions" nil (ih0 n s k r ih1 p sh ax n z))
("inscrutable" nil (ih0 n s k r uw1 t ax b ax l))
("insect" nil (ih1 n s eh1 k t))
("insecticide" nil (ih0 n s eh1 k t ax s ay1 d))
("insecticides" nil (ih0 n s eh1 k t ax s ay1 d z))
("insectivorous" nil (ih0 n s eh0 k t ih1 v er0 ax s))
("insects" nil (ih1 n s eh1 k t s))
("insecure" nil (ih1 n s ax k y er0))
("insecurities" nil (ih1 n s ax k y uh1 r ih0 t iy0 z))
("insecurity" nil (ih1 n s ih0 k y uh1 r ih0 t iy0))
("insel" nil (ih1 n s ax l))
("insemination" nil (ih0 n s eh1 m ax n ey1 sh ax n))
("insensitive" nil (ih0 n s eh1 n s ax t ih0 v))
("insensitivity" nil (ih0 n s eh1 n s ax t ih1 v ax t iy0))
("inseparable" nil (ih0 n s eh1 p er0 ax b ax l))
("inserra" nil (ih0 n s eh1 r ax))
("insert" v (ih0 n s er1 t))
("insert" n (ih1 n s er r t))
("inserted" nil (ih0 n s er1 t ax d))
("inserting" nil (ih0 n s er1 t ih0 ng))
("insertion" nil (ih0 n s er1 sh ax n))
("inserts" v (ih0 n s er1 t s))
("inserts" n (ih1 n s er0 t s))
("inset" nil (ih1 n s eh1 t))
("inshore" nil (ih1 n sh ao1 r))
("inside" nil (ih0 n s ay1 d))
("insider" nil (ih0 n s ay1 d er0))
("insiders" nil (ih0 n s ay1 d er0 z))
("insides" nil (ih0 n s ay1 d z))
("insidious" nil (ih0 n s ih1 d iy0 ax s))
("insight" nil (ih1 n s ay1 t))
("insightful" nil (ih1 n s ay1 t f ax l))
("insights" nil (ih1 n s ay1 t s))
("insignia" nil (ih0 n s ih1 g n iy0 ax))
("insignificance" nil (ih1 n s ih0 g n y ih1 f ih0 k ax n s))
("insignificant" nil (ih1 n s ih0 g n y ih1 f ih0 k ax n t))
("insilco" nil (ih0 n s ih1 l k ow0))
("insincere" nil (ih1 n s ih0 n s ih1 r))
("insinuate" nil (ih0 n s ih1 n y uw0 ey0 t))
("insinuating" nil (ih0 n s ih1 n y uw0 ey0 t ih0 ng))
("insinuation" nil (ih0 n s ih1 n y uw0 ey1 sh ax n))
("insinuations" nil (ih0 n s ih1 n y uw0 ey1 sh ax n z))
("insist" nil (ih0 n s ih1 s t))
("insisted" nil (ih0 n s ih1 s t ax d))
("insistence" nil (ih0 n s ih1 s t ax n s))
("insistent" nil (ih0 n s ih1 s t ax n t))
("insistently" nil (ih0 n s ih1 s t ax n t l iy0))
("insisting" nil (ih0 n s ih1 s t ih0 ng))
("insists" nil (ih0 n s ih1 s t s))
("insituform" nil (ih0 n s ih1 t uw1 f ao0 r m))
("inskeep" nil (ih1 n z k iy1 p))
("insko" nil (ih1 n s k ow0))
("insley" nil (ih1 n s l iy0))
("insofar" nil (ih1 n s ax f aa0 r))
("insolence" nil (ih1 n s ax l ax n s))
("insolent" nil (ih1 n s ax l ax n t))
("insoluble" nil (ih0 n s aa1 l y ax b ax l))
("insolvencies" nil (ih0 n s aa1 l v ax n s iy0 z))
("insolvency" nil (ih0 n s aa1 l v ax n s iy0))
("insolvent" nil (ih0 n s aa1 l v ax n t))
("insomnia" nil (ih0 n s aa1 m n iy0 ax))
("inspect" nil (ih0 n s p eh1 k t))
("inspected" nil (ih0 n s p eh1 k t ih0 d))
("inspecting" nil (ih0 n s p eh1 k t ih0 ng))
("inspection" nil (ih0 n s p eh1 k sh ax n))
("inspections" nil (ih0 n s p eh1 k sh ax n z))
("inspector" nil (ih0 n s p eh1 k t er0))
("inspectorate" nil (ih0 n s p eh1 k t er0 ax t))
("inspectors" nil (ih0 n s p eh1 k t er0 z))
("inspects" nil (ih0 n s p eh1 k t s))
("inspeech" nil (ih0 n s p iy1 ch))
("inspiration" nil (ih1 n s p er0 ey1 sh ax n))
("inspirational" nil (ih1 n s p er0 ey1 sh ax n ax l))
("inspirations" nil (ih1 n s p er0 ey1 sh ax n z))
("inspire" nil (ih0 n s p ay1 r))
("inspired" nil (ih0 n s p ay1 er0 d))
("inspires" nil (ih0 n s p ay1 r z))
("inspiring" nil (ih0 n s p ay1 r ih0 ng))
("instabilities" nil (ih1 n s t ax b ih1 l ih0 t iy0 z))
("instability" nil (ih1 n s t ax b ih1 l ih0 t iy0))
("install" nil (ih0 n s t ao1 l))
("installation" nil (ih1 n s t ax l ey1 sh ax n))
("installations" nil (ih1 n s t ax l ey1 sh ax n z))
("installed" nil (ih0 n s t ao1 l d))
("installer" nil (ih0 n s t ao1 l er0))
("installers" nil (ih0 n s t ao1 l er0 z))
("installing" nil (ih0 n s t ao1 l ih0 ng))
("installment" nil (ih0 n s t ao1 l m ax n t))
("installments" nil (ih0 n s t ao1 l m ax n t s))
("installs" nil (ih0 n s t ao1 l z))
("instance" nil (ih1 n s t ax n s))
("instances" nil (ih1 n s t ax n s ax z))
("instant" nil (ih1 n s t ax n t))
("instantaneous" nil (ih1 n s t ax n t ae1 n iy0 ax s))
("instantaneously" nil (ih1 n s t ax n t ae1 n iy0 ax s l iy0))
("instantly" nil (ih1 n s t ax n t l iy0))
("instead" nil (ih0 n s t eh1 d))
("instigate" nil (ih1 n s t ax g ey1 t))
("instigated" nil (ih1 n s t ax g ey1 t ih0 d))
("instigating" nil (ih1 n s t ax g ey1 t ih0 ng))
("instigation" nil (ih1 n s t ih0 g ey1 sh ax n))
("instigator" nil (ih1 n s t ax g ey1 t er0))
("instill" nil (ih0 n s t ih1 l))
("instilled" nil (ih0 n s t ih1 l d))
("instilling" nil (ih0 n s t ih1 l ih0 ng))
("instinct" nil (ih1 n s t ih0 ng k t))
("instinctive" nil (ih0 n s t ih1 ng k t ih0 v))
("instinctively" nil (ih0 n s t ih1 ng k t ih0 v l iy0))
("instincts" nil (ih1 n s t ih0 ng k t s))
("instinet" nil (ih1 n s t ih0 n eh1 t))
("institut" nil (ih1 n s t ih0 t uw0 t))
("institute" nil (ih1 n s t ax t uw1 t))
("instituted" nil (ih1 n s t ax t uw1 t ax d))
("institutes" nil (ih1 n s t ax t uw1 t s))
("instituting" nil (ih1 n s t ih0 t uw1 t ih0 ng))
("institution" nil (ih1 n s t ih0 t uw1 sh ax n))
("institutional" nil (ih1 n s t ih0 t uw1 sh ax n ax l))
("institutionalist" nil (ih1 n s t ih0 t uw1 sh ax n ax l ax s t))
("institutionalists" nil (ih1 n s t ih0 t uw1 sh ax n ax l ax s t s))
("institutionalization" nil (ih0 n s t ih0 t uw1 sh ax n ax l ih0 z ey1 sh ax n))
("institutionalize" nil (ih1 n s t ih0 t uw1 sh ax n ax l ay0 z))
("institutionalized" nil (ih1 n s t ih0 t uw1 sh ax n ax l ay0 z d))
("institutionalizes" nil (ih1 n s t ih0 t uw1 sh ax n ax l ay0 z ih0 z))
("institutionalizing" nil (ih1 n s t ih0 t uw1 sh ax n ax l ay0 z ih0 ng))
("institutionally" nil (ih1 n s t ih0 t uw1 sh ax n ax l iy0))
("institutions" nil (ih1 n s t ih0 t uw1 sh ax n z))
("instituto" nil (ih1 n s t ih0 t uw1 t ow0))
("instone" nil (ih1 n s t ow1 n))
("instruct" nil (ih0 n s t r ah1 k t))
("instructed" nil (ih0 n s t r ah1 k t ax d))
("instructing" nil (ih0 n s t r ah1 k t ih0 ng))
("instruction" nil (ih0 n s t r ah1 k sh ax n))
("instructional" nil (ih0 n s t r ah1 k sh ax n ax l))
("instructions" nil (ih0 n s t r ah1 k sh ax n z))
("instructive" nil (ih0 n s t r ah1 k t ih0 v))
("instructor" nil (ih0 n s t r ah1 k t er0))
("instructors" nil (ih0 n s t r ah1 k t er0 z))
("instructs" nil (ih0 n s t r ah1 k t s))
("instrument" nil (ih1 n s t r ax m ax n t))
("instrumental" nil (ih1 n s t r ax m eh1 n t ax l))
("instrumentalists" nil (ih1 n s t r ax m eh1 n t ax l ih0 s t s))
("instrumentals" nil (ih1 n s t r ax m eh1 n t ax l z))
("instrumentation" nil (ih1 n s t r ax m eh1 n t ey1 sh ax n))
("instruments" nil (ih1 n s t r ax m ax n t s))
("insubordination" nil (ih0 n s ax b ao1 r d ax n ey1 sh ax n))
("insubstantial" nil (ih0 n s ax b s t ae1 n ch ax l))
("insufferable" nil (ih0 n s ah1 f er0 ax b ax l))
("insufficient" nil (ih0 n s ax f ih1 sh ax n t))
("insufficiently" nil (ih1 n s ax f ih1 sh ax n t l iy0))
("insular" nil (ih1 n s ax l er0))
("insularity" nil (ih1 n s ax l eh1 r ih0 t iy0))
("insulate" nil (ih1 n s ax l ey1 t))
("insulated" nil (ih1 n s ax l ey1 t ax d))
("insulating" nil (ih1 n s ax l ey1 t ih0 ng))
("insulation" nil (ih1 n s ax l ey1 sh ax n))
("insulator" nil (ih1 n s ax l ey1 t er0))
("insulators" nil (ih1 n s ax l ey1 t er0 z))
("insulin" nil (ih1 n s ax l ax n))
("insult" v (ih0 n s ah1 l t))
("insult" n (ih1 n s ax l t))
("insulted" nil (ih0 n s ah1 l t ih0 d))
("insulting" nil (ih0 n s ah1 l t ih0 ng))
("insults" v (ih1 n s ax l t s))
("insults" n (ih0 n s ah1 l t s))
("insupportable" nil (ih0 n s ax p ao1 r t ax b ax l))
("insurance" nil (ih0 n sh uh1 r ax n s))
("insurances" nil (ih0 n sh uh1 r ax n s ih0 z))
("insure" nil (ih0 n sh uh1 r))
("insured" nil (ih0 n sh uh1 r d))
("insurer" nil (ih0 n sh uh1 r er0))
("insurers" nil (ih0 n sh uh1 r er0 z))
("insures" nil (ih0 n sh uh1 r z))
("insurgencies" nil (ih0 n s er1 jh ax n s iy0 z))
("insurgency" nil (ih0 n s er1 jh ax n s iy0))
("insurgent" nil (ih0 n s er1 jh ax n t))
("insurgents" nil (ih0 n s er1 jh ax n t s))
("insuring" nil (ih0 n sh uh1 r ih0 ng))
("insurmountable" nil (ih1 n s er0 m aw1 n t ax b ax l))
("insurrection" nil (ih1 n s er0 eh1 k sh ax n))
("intact" nil (ih0 n t ae1 k t))
("intaglio" nil (ih0 n t ae1 l y ow0))
("intake" nil (ih1 n t ey1 k))
("intan" nil (ih1 n t ax n))
("intangible" nil (ih0 n t ae1 n jh ax b ax l))
("intangibles" nil (ih0 n t ae1 n jh ax b ax l z))
("intefadeh" nil (ih1 n t ax f aa1 d ax))
("integers" nil (ih1 n t ax jh er0 z))
("integon" nil (ih1 n t ax g aa0 n))
("integra" nil (ih0 n t eh1 g r ax))
("integral" nil (ih1 n t ax g r ax l))
("integrate" nil (ih1 n t ax g r ey1 t))
("integrated" nil (ih1 n t ax g r ey1 t ax d))
("integrates" nil (ih1 n t ax g r ey1 t s))
("integrating" nil (ih1 n t ax g r ey1 t ih0 ng))
("integration" nil (ih1 n t ax g r ey1 sh ax n))
("integrations" nil (ih1 n t ax g r ey1 sh ax n z))
("integrator" nil (ih1 n t ax g r ey1 t er0))
("integrators" nil (ih1 n t ax g r ey1 t er0 z))
("integrelin" nil (ih1 n t ax g r eh1 l ax n))
("integrity" nil (ih0 n t eh1 g r ax t iy0))
("integument" nil (ih0 n t eh1 g y ax m ax n t))
("intel" nil (ih1 n t eh1 l))
("intelco" nil (ih0 n t eh1 l k ow0))
("intelcom" nil (ih1 n t eh1 l k aa0 m))
("intellect" nil (ih1 n t ax l eh1 k t))
("intellects" nil (ih1 n t ax l eh1 k t s))
("intellectual" nil (ih1 n t ax l eh1 k ch uw0 ax l))
("intellectualism" nil (ih0 n t eh1 l ax k ch uw1 ax l ih1 z ax m))
("intellectually" nil (ih1 n t ax l eh1 k ch uw0 ax l iy0))
("intellectuals" nil (ih1 n t ax l eh1 k ch uw0 ax l z))
("intellicall" nil (ih0 n t eh1 l ih0 k ao1 l))
("intellicorp" nil (ih0 n t eh1 l ih0 k ao1 r p))
("intelligence" nil (ih0 n t eh1 l ax jh ax n s))
("intelligent" nil (ih0 n t eh1 l ax jh ax n t))
("intelligently" nil (ih0 n t eh1 l ih0 jh ax n t l iy0))
("intelligentsia" nil (ih0 n t eh1 l ax jh eh1 n t s iy0 ax))
("intelligible" nil (ih0 n t eh1 l ax jh ax b ax l))
("intelogic" nil (ih1 n t ax l aa1 jh ih0 k))
("intelsat" nil (ih1 n t eh1 l s ae0 t))
("intemperate" nil (ih0 n t eh1 m p er0 ax t))
("intend" nil (ih0 n t eh1 n d))
("intended" nil (ih0 n t eh1 n d ax d))
("intending" nil (ih0 n t eh1 n d ih0 ng))
("intends" nil (ih0 n t eh1 n d z))
("intense" nil (ih0 n t eh1 n s))
("intensely" nil (ih0 n t eh1 n s l iy0))
("intensification" nil (ih0 n t eh1 n s ax f ax k ey1 sh ax n))
("intensified" nil (ih0 n t eh1 n s ax f ay1 d))
("intensifies" nil (ih0 n t eh1 n s ax f ay1 z))
("intensify" nil (ih0 n t eh1 n s ax f ay1))
("intensifying" nil (ih0 n t eh1 n s ax f ay1 ih0 ng))
("intensity" nil (ih0 n t eh1 n s ax t iy0))
("intensive" nil (ih0 n t eh1 n s ih0 v))
("intensively" nil (ih0 n t eh1 n s ih0 v l iy0))
("intent" nil (ih0 n t eh1 n t))
("intention" nil (ih0 n t eh1 n ch ax n))
("intentional" nil (ih0 n t eh1 n sh ax n ax l))
("intentionally" nil (ih0 n t eh1 n sh ax n ax l iy0))
("intentioned" nil (ih0 n t eh1 n ch ax n d))
("intentions" nil (ih0 n t eh1 n ch ax n z))
("intently" nil (ih0 n t eh1 n t l iy0))
("intents" nil (ih0 n t eh1 n t s))
("inter" nil (ih0 n t er1))
("interacciones" nil (ih1 n t er0 ae1 k s iy0 ow1 n z))
("interact" nil (ih1 n t er0 ae1 k t))
("interacting" nil (ih1 n t er0 ae1 k t ih0 ng))
("interaction" nil (ih1 n t er0 ae1 k sh ax n))
("interactions" nil (ih1 n t er0 ae1 k sh ax n z))
("interactive" nil (ih1 n t er0 ae1 k t ih0 v))
("interactivity" nil (ih1 n t er0 ae1 k t ih1 v ih0 t iy0))
("interacts" nil (ih1 n t er0 ae1 k t s))
("interagency" nil (ih1 n t er0 ey1 jh ax n s iy0))
("interamerican" nil (ih1 n t er0 ax m eh1 r ax k ax n))
("interand" nil (ih1 n t er0 ax n d))
("interbank" nil (ih1 n t er0 b ae1 ng k))
("interbred" nil (ih1 n t er0 b r eh1 d))
("interbrew" nil (ih1 n t er1 b r uw1))
("intercable" nil (ih1 n t er0 k ey1 b ax l))
("intercare" nil (ih1 n t er0 k eh1 r))
("intercede" nil (ih1 n t er0 s iy1 d))
("interceded" nil (ih1 n t er0 s iy1 d ih0 d))
("interceding" nil (ih1 n t er0 s iy1 d ih0 ng))
("intercellular" nil (ih1 n t er0 s eh1 l y ax l er0))
("intercept" nil (ih1 n t er0 s eh1 p t))
("intercepted" nil (ih1 n t er0 s eh1 p t ax d))
("intercepting" nil (ih1 n t er0 s eh1 p t ih0 ng))
("interception" nil (ih1 n t er0 s eh1 p sh ax n))
("interceptions" nil (ih1 n t er0 s eh1 p sh ax n z))
("interceptor" nil (ih1 n t er0 s eh1 p t er0))
("interceptors" nil (ih1 n t er0 s eh1 p t er0 z))
("intercepts" nil (ih1 n t er0 s eh1 p t s))
("intercession" nil (ih1 n t er0 s eh1 sh ax n))
("interchange" nil (ih1 n t er0 ch ey1 n jh))
("interchangeable" nil (ih1 n t er0 ch ey1 n jh ax b ax l))
("interchangeably" nil (ih1 n t er0 ch ey1 n jh ax b l iy0))
("interchanges" nil (ih1 n t er0 ch ey1 n jh ih0 z))
("intercity" nil (ih1 n t er0 s ih1 t iy0))
("interco" nil (ih1 n t er0 k ow1))
("intercollegiate" nil (ih1 n t er0 k ax l iy1 jh ax t))
("intercom" nil (ih1 n t er0 k aa1 m))
("intercompany" nil (ih1 n t er0 k ah1 m p ax n iy0))
("interconnect" nil (ih1 n t er0 k ax n eh1 k t))
("interconnected" nil (ih1 n t er0 k ax n eh1 k t ih0 d))
("interconnection" nil (ih1 n t er0 k ax n eh1 k sh ax n))
("interconnections" nil (ih1 n t er0 k ax n eh1 k sh ax n z))
("intercontinental" nil (ih1 n t er0 k aa1 n t ax n eh1 n t ax l))
("intercontinentale" nil (ih1 n t er0 k ax n t ih1 n ax n t aa1 l))
("intercorp" nil (ih1 n t er0 k ao1 r p))
("intercorporation" nil (ih0 n t er0 k ao1 r p er0 ey1 sh ax n))
("intercourse" nil (ih1 n t er0 k ao1 r s))
("intercultural" nil (ih1 n t er0 k ah1 l ch er0 ax l))
("intercurrent" nil (ih1 n t er0 k er1 ax n t))
("interdealer" nil (ih1 n t er0 d iy1 l er0))
("interdependence" nil (ih1 n t er0 d ax p eh1 n d ax n s))
("interdependent" nil (ih1 n t er0 d ih0 p eh1 n d ax n t))
("interdict" nil (ih1 n t er0 d ih1 k t))
("interdicted" nil (ih1 n t er0 d ih1 k t ih0 d))
("interdictes" nil (ih1 n t er0 d ih1 k t s))
("interdicting" nil (ih1 n t er0 d ih1 k t ih0 ng))
("interdiction" nil (ih1 n t er0 d ih1 k sh ax n))
("interdigital" nil (ih1 n t er0 d ih1 jh ax t ax l))
("interdisciplinary" nil (ih1 n t er0 d ih1 s ax p l ax n eh1 r iy0))
("interdyne" nil (ih1 n t er0 d ay1 n))
("interest" nil (ih1 n t r ax s t))
("interested" nil (ih1 n t r ax s t ax d))
("interesting" nil (ih1 n t r ax s t ih0 ng))
("interestingly" nil (ih1 n t er0 eh1 s t ih0 ng l iy0))
("interestrate" nil (ih1 n t er0 ax s t r ey1 t))
("interests" nil (ih1 n t r ax s t s))
("interface" nil (ih1 n t er0 f ey1 s))
("interfaces" nil (ih1 n t er0 f ey1 s ih0 z))
("interfaith" nil (ih1 n t er0 f ey1 th))
("interfax" nil (ih1 n t er0 f ae1 k s))
("interfere" nil (ih1 n t er0 f ih1 r))
("interfered" nil (ih1 n t er0 f ih1 r d))
("interference" nil (ih1 n t er0 f ih1 r ax n s))
("interferes" nil (ih1 n t er0 f ih1 r z))
("interfering" nil (ih1 n t er0 f ih1 r ih0 ng))
("interferometer" nil (ih1 n t er0 f er0 aa1 m ax t er0))
("interferon" nil (ih1 n t er0 f eh1 r aa0 n))
("interferons" nil (ih0 n t er0 f ih1 r aa0 n z))
("interfirst" nil (ih1 n t er0 f er1 s t))
("interflug" nil (ih1 n t er0 f l ax g))
("interfunding" nil (ih1 n t er0 f ah1 n d ih0 ng))
("intergenerational" nil (ih1 n t er0 jh eh1 n er0 ey1 sh ax n ax l))
("intergovernmental" nil (ih1 n t er0 g ah1 v er0 n m eh1 n t ax l))
("intergraph" nil (ih1 n t er0 g r ae1 f))
("intergroup" nil (ih1 n t er0 g r uw1 p))
("interim" nil (ih1 n t er0 ax m))
("interior" nil (ih0 n t ih1 r iy0 er0))
("interiors" nil (ih0 n t ih1 r iy0 er0 z))
("interject" nil (ih1 n t er0 jh eh1 k t))
("interjected" nil (ih1 n t er0 jh eh1 k t ih0 d))
("interjections" nil (ih1 n t er0 jh eh1 k sh ax n z))
("interjects" nil (ih1 n t er0 jh eh1 k t s))
("interlaced" nil (ih1 n t er0 l ey1 s t))
("interlake" nil (ih1 n t er0 l ey1 k))
("interleaf" nil (ih1 n t er0 l iy1 f))
("interleaving" nil (ih1 n t er0 l iy1 v ih0 ng))
("interleukin" nil (ih1 n t er0 l uw1 k ih0 n))
("interlink" nil (ih1 n t er0 l ih1 ng k))
("interlinked" nil (ih1 n t er0 l ih1 ng k t))
("interlock" nil (ih1 n t er0 l aa1 k))
("interlocking" nil (ih1 n t er0 l aa1 k ih0 ng))
("interlocutors" nil (ih1 n t er0 l aa1 k y ax t er0 z))
("interloper" nil (ih1 n t er0 l ow1 p er0))
("interlopers" nil (ih1 n t er0 l ow1 p er0 z))
("interlude" nil (ih1 n t er0 l uw1 d))
("interludes" nil (ih1 n t er0 l uw1 d z))
("intermagnetics" nil (ih1 n t er0 m ae0 g n eh1 t ih0 k s))
("intermark" nil (ih1 n t er0 m aa1 r k))
("intermarket" nil (ih1 n t er0 m aa1 r k ih0 t))
("intermarriage" nil (ih1 n t er0 m eh1 r ih0 jh))
("intermarried" nil (ih1 n t er0 m ae1 r iy0 d))
("intermec" nil (ih1 n t er0 m ax k))
("intermedia" nil (ih1 n t er0 m iy1 d iy0 ax))
("intermediaries" nil (ih1 n t er0 m iy1 d iy0 eh1 r iy0 z))
("intermediary" nil (ih1 n t er0 m iy1 d iy0 eh0 r iy0))
("intermediate" nil (ih1 n t er0 m iy1 d iy0 ih0 t))
("intermediates" nil (ih1 n t er0 m iy1 d iy0 ax t s))
("intermedics" nil (ih1 n t er0 m eh1 d ih0 k s))
("intermet" nil (ih1 n t er0 m eh1 t))
("interminable" nil (ih0 n t er1 m ax n ax b ax l))
("interminably" nil (ih1 n t er0 m ih0 n ae1 b l iy0))
("intermingle" nil (ih1 n t er0 m ih1 ng g ax l))
("intermingled" nil (ih1 n t er0 m ih1 ng g ax l d))
("intermingling" nil (ih1 n t er0 m ih1 ng g ax l ih0 ng))
("intermission" nil (ih1 n t er0 m ih1 sh ax n))
("intermissions" nil (ih1 n t er0 m ih1 sh ax n z))
("intermittent" nil (ih1 n t er0 m ih1 t ax n t))
("intermittently" nil (ih1 n t er0 m ih1 t ax n t l iy0))
("intermixed" nil (ih1 n t er0 m ih1 k s t))
("intermodal" nil (ih1 n t er0 m ow1 d ax l))
("intermolecular" nil (ih1 n t er0 m ax l eh1 k y ax l er0))
("intermountain" nil (ih0 n t er0 m aw1 n t ih0 n))
("intern" n (ih1 n t er0 n))
("intern" v (ih0 n t er1 n))
("internacional" nil (ih1 n t er0 n ae1 sh ax n ax l))
("internal" nil (ih0 n t er1 n ax l))
("internally" nil (ih0 n t er1 n ax l iy0))
("international" nil (ih1 n t er0 n ae1 sh ax n ax l))
("internationale" nil (ih0 n t er0 n ae1 sh ax n aa1 l iy0))
("internationalism" nil (ih0 n t er0 n ae1 sh ax n ax l ih1 z ax m))
("internationalist" nil (ih0 n t er0 n ae1 sh ax n ax l ih0 s t))
("internationalists" nil (ih1 n t er0 n ae1 sh ax n ax l ih0 s t s))
("internationalization" nil (ih1 n t er0 n ae1 sh ax n ax l ih0 z ey1 sh ax n))
("internationalize" nil (ih1 n t er0 n ae1 sh ax n ax l ay1 z))
("internationalized" nil (ih0 n t er0 n ae1 sh ax n ax l ay0 z d))
("internationally" nil (ih1 n t er0 n ae1 sh ax n ax l iy0))
("internationals" nil (ih1 n t er0 n ae1 sh ax n ax l z))
("internatonal" nil (ih1 n t er0 n ae1 sh ax n ax l))
("internecine" nil (ih0 n t er1 n ax s iy1 n))
("interned" nil (ih1 n t er1 n d))
("internees" nil (ih0 n t er0 n iy1 z))
("internet" nil (ih1 n t er0 n eh1 t))
("internist" nil (ih0 n t er1 n ih0 s t))
("internists" nil (ih0 n t er1 n ih0 s t s))
("internment" nil (ih0 n t er1 n m ax n t))
("internorth" nil (ih1 n t er0 n ao0 r th))
("interns" v (ih1 n t er1 n z))
("interns" n (ih1 n t er0 n z))
("internship" nil (ih1 n t er0 n sh ih1 p))
("internships" nil (ih1 n t er0 n sh ih1 p s))
("interoffice" nil (ih1 n t er0 ao1 f ax s))
("interpart" nil (ih1 n t er0 p aa1 r t))
("interpersonal" nil (ih1 n t er0 p er1 s ax n ax l))
("interplanetary" nil (ih1 n t er0 p l ae1 n ax t eh1 r iy0))
("interplay" nil (ih1 n t er0 p l ey1))
("interpol" nil (ih1 n t er0 p ow1 l))
("interpolated" nil (ih1 t er1 p ax l ey1 t ih0 d))
("interpret" nil (ih0 n t er1 p r ax t))
("interpretation" nil (ih0 n t er1 p r ih0 t ey1 sh ax n))
("interpretations" nil (ih0 n t er1 p r ih0 t ey1 sh ax n z))
("interpreted" nil (ih0 n t er1 p r ax t ax d))
("interpreter" nil (ih0 n t er1 p r ax t er0))
("interpreters" nil (ih0 n t er1 p r ax t er0 z))
("interpreting" nil (ih1 n t er0 p r eh1 t ih0 ng))
("interpretive" nil (ih0 n t er1 p r ax t ih0 v))
("interprets" nil (ih0 n t er1 p r ax t s))
("interprovincial" nil (ih1 n t er0 p r ow0 v ih1 n sh ax l))
("interpublic" nil (ih1 n t er0 p ah1 b l ih0 k))
("interracial" nil (ih1 n t er0 r ey1 sh ax l))
("interrante" nil (ih0 n t er0 r aa1 n t iy0))
("interred" nil (ih1 n t er1 d))
("interregnum" nil (ih1 n t er0 r eh1 g n ax m))
("interrelate" nil (ih1 n t er0 r ih0 l ey1 t))
("interrelated" nil (ih1 n t er0 r ih0 l ey1 t ih0 d))
("interrent" nil (ih1 n t er0 eh1 n t))
("interrogate" nil (ih0 n t eh1 r ax g ey1 t))
("interrogated" nil (ih0 n t eh1 r ax g ey1 t ih0 d))
("interrogating" nil (ih0 n t eh1 r ax g ey1 t ih0 ng))
("interrogation" nil (ih0 n t eh1 r ax g ey1 sh ax n))
("interrogations" nil (ih0 n t eh1 r ax g ey1 sh ax n z))
("interrogator" nil (ih1 n t eh1 r ax g ey1 t er0))
("interrogators" nil (ih1 n t eh1 r ax g ey1 t er0 z))
("interrupt" nil (ih1 n t er0 ah1 p t))
("interrupted" nil (ih1 n t er0 ah1 p t ih0 d))
("interruptible" nil (ih1 n t er0 r ah1 p t ih0 b ax l))
("interrupting" nil (ih1 n t er0 ah1 p t ih0 ng))
("interruption" nil (ih1 n t er0 ah1 p sh ax n))
("interruptions" nil (ih1 n t er0 ah1 p sh ax n z))
("interrupts" nil (ih1 n t er0 ah1 p t s))
("interscholastic" nil (ih1 n t er0 s k ax l ae1 s t ih0 k))
("interscope" nil (ih1 n t er0 s k ow1 p))
("intersec" nil (ih1 n t er0 s eh0 k))
("intersect" nil (ih1 n t er0 s eh1 k t))
("intersected" nil (ih1 n t er0 s eh1 k t ih0 d))
("intersecting" nil (ih1 n t er0 s eh1 k t ih0 ng))
("intersection" nil (ih1 n t er0 s eh1 k sh ax n))
("intersections" nil (ih1 n t er0 s eh1 k sh ax n z))
("intersects" nil (ih1 n t er0 s eh1 k t s))
("interspec" nil (ih1 n t er0 s p eh0 k))
("interspersed" nil (ih1 n t er0 s p er1 s t))
("interstate" nil (ih1 n t er0 s t ey1 t))
("interstates" nil (ih1 n t er0 s t ey1 t s))
("interstellar" nil (ih1 n t er0 s t eh1 l er0))
("intertan" nil (ih1 n t er0 t ae1 n))
("intertech" nil (ih1 n t er0 t eh1 k))
("intertechnology" nil (ih1 n t er0 t ax k n aa1 l ax jh iy0))
("intertect" nil (ih1 n t er0 t eh1 k t))
("intertidal" nil (ih1 n t er0 t ay1 d ax l))
("intertrans" nil (ih1 n t er0 t r ae1 n z))
("intertribal" nil (ih1 n t er0 t r ay1 b ax l))
("intertwined" nil (ih0 n t er0 t w ay1 n d))
("intertwining" nil (ih1 n t er0 t w ay1 n ih0 ng))
("interurban" nil (ih1 n t er0 er1 b ax n))
("interval" nil (ih1 n t er0 v ax l))
("intervals" nil (ih1 n t er0 v ax l z))
("intervene" nil (ih1 n t er0 v iy1 n))
("intervened" nil (ih1 n t er0 v iy1 n d))
("intervenes" nil (ih1 n t er0 v iy1 n z))
("intervening" nil (ih1 n t er0 v iy1 n ih0 ng))
("intervenors" nil (ih1 n t er0 v iy1 n er0 z))
("intervention" nil (ih1 n t er0 v eh1 n ch ax n))
("interventionism" nil (ih1 n t er0 v eh1 n ch ax n ih1 z ax m))
("interventionist" nil (ih1 n t er0 v eh1 n sh ax n ih0 s t))
("interventionists" nil (ih1 n t er0 v eh1 n ch ax n ih0 s t s))
("interventions" nil (ih1 n t er0 v eh1 n ch ax n z))
("interview" nil (ih1 n t er0 v y uw1))
("interviewed" nil (ih1 n t er0 v y uw1 d))
("interviewee" nil (ih0 n t er0 v y uw0 iy1))
("interviewees" nil (ih0 n t er0 v y uw0 iy1 z))
("interviewer" nil (ih1 n t er0 v y uw1 er0))
("interviewers" nil (ih1 n t er0 v y uw1 er0 z))
("interviewing" nil (ih1 n t er0 v y uw1 ih0 ng))
("interviews" nil (ih1 n t er0 v y uw1 z))
("intervoice" nil (ih1 n t er0 v oy1 s))
("interweaving" nil (ih1 n t er0 w iy1 v ih0 ng))
("interwoven" nil (ih1 n t er0 w ow1 v ax n))
("intestinal" nil (ih0 n t eh1 s t ax n ax l))
("intestine" nil (ih0 n t eh1 s t ax n))
("intestines" nil (ih0 n t eh1 s t ax n z))
("intex" nil (ih1 n t eh1 k s))
("inti" nil (ih1 n t iy0))
("intifada" nil (ih1 n t ih0 f aa1 d ax))
("intimacy" nil (ih1 n t ax m ax s iy0))
("intimate" nil (ih1 n t ax m ax t))
("intimated" nil (ih1 n t ax m ey1 t ih0 d))
("intimately" nil (ih1 n t ax m ax t l iy0))
("intimates" nil (ih1 n t ax m ax t s))
("intimations" nil (ih1 n t ih0 m ey1 sh ax n z))
("intimidate" nil (ih0 n t ih1 m ih0 d ey1 t))
("intimidated" nil (ih0 n t ih1 m ih0 d ey1 t ih0 d))
("intimidates" nil (ih0 n t ih1 m ih0 d ey1 t s))
("intimidating" nil (ih0 n t ih1 m ih0 d ey1 t ih0 ng))
("intimidation" nil (ih0 n t ih1 m ih0 d ey1 sh ax n))
("intis" nil (ih1 n t ih0 s))
("into" nil (ih0 n t uw1))
("intolerable" nil (ih0 n t aa1 l er0 ax b ax l))
("intolerably" nil (ih0 n t aa1 l er0 ax b l iy0))
("intolerance" nil (ih0 n t aa1 l er0 ax n s))
("intolerant" nil (ih0 n t aa1 l er0 ax n t))
("intoned" nil (ih0 n t ow1 n d))
("intones" nil (ih0 n t ow1 n z))
("intoning" nil (ih0 n t ow1 n ih0 ng))
("intourist" nil (ih0 n t uh1 r ih0 s t))
("intoxicated" nil (ih0 n t aa1 k s ax k ey1 t ax d))
("intoxicating" nil (ih0 n t aa1 k s ih0 k ey1 t ih0 ng))
("intoxication" nil (ih0 n t aa1 k s ax k ey1 sh ax n))
("intra" nil (ih1 n t r ax))
("intracompany" nil (ih1 n t r ax k aa1 m p ax n iy0))
("intracranial" nil (ih1 n t r ax k r ey1 n iy0 ax l))
("intractable" nil (ih0 n t r ae1 k t ax b ax l))
("intraday" nil (ih1 n t r ax d ey1))
("intramarginal" nil (ih1 n t r ax m aa1 r jh ih0 n ax l))
("intramural" nil (ih1 n t r ax m y uh1 r ax l))
("intransigence" nil (ih0 n t r ae1 n s ax jh ax n s))
("intransigent" nil (ih0 n t r ae1 n s ax jh ax n t))
("intraocular" nil (ih1 n t r ax ow1 k y uw0 l er0))
("intraparty" nil (ih1 n t r ax p aa1 r t iy0))
("intrastate" nil (ih1 n t r ax s t ey1 t))
("intrauterine" nil (ih1 n t r ax y uw1 t er0 ih0 n))
("intravenous" nil (ih1 n t r ax v iy1 n ax s))
("intravenously" nil (ih0 n t r ae1 v ax n ax s l iy0))
("intrawest" nil (ih1 n t r ax w ax s t))
("intrepid" nil (ih0 n t r eh1 p ax d))
("intrex" nil (ih1 n t r ax k s))
("intricacies" nil (ih1 n t r ax k ax s iy0 z))
("intricate" nil (ih1 n t r ax k ax t))
("intricately" nil (ih1 n t r ax k ax t l iy0))
("intrieri" nil (ih0 n t r ih1 r iy0))
("intrigue" nil (ih0 n t r iy1 g))
("intrigued" nil (ih1 n t r iy0 g d))
("intrigues" nil (ih0 n t r iy1 g z))
("intriguing" nil (ih0 n t r iy1 g ih0 ng))
("intriguingly" nil (ih0 n t r iy1 g ih0 ng l iy0))
("intrinsic" nil (ih0 n t r ih1 n s ih0 k))
("intrinsically" nil (ih0 n t r ih1 n s ih0 k ax l iy0))
("introduce" nil (ih1 n t r ax d uw1 s))
("introduced" nil (ih1 n t r ax d uw1 s t))
("introduces" nil (ih1 n t r ax d uw1 s ih0 z))
("introducing" nil (ih1 n t r ax d uw1 s ih0 ng))
("introduction" nil (ih1 n t r ax d ah1 k sh ax n))
("introductions" nil (ih1 n t r ax d ah1 k sh ax n z))
("introductory" nil (ih1 n t r ax d ah1 k t er0 iy0))
("intron" nil (ih1 n t r ax n))
("introspection" nil (ih1 n t r ax s p eh1 k sh ax n))
("introspective" nil (ih1 n t r ax s p eh1 k t ih0 v))
("introverted" nil (ih1 n t r ow0 v er1 t ih0 d))
("intrude" nil (ih0 n t r uw1 d))
("intruded" nil (ih0 n t r uw1 d ax d))
("intruder" nil (ih0 n t r uw1 d er0))
("intruders" nil (ih0 n t r uw1 d er0 z))
("intrudes" nil (ih0 n t r uw1 d z))
("intruding" nil (ih0 n t r uw1 d ih0 ng))
("intrusion" nil (ih0 n t r uw1 zh ax n))
("intrusions" nil (ih0 n t r uw1 zh ax n z))
("intrusive" nil (ih0 n t r uw1 s ih0 v))
("intuit" nil (ih0 n t uw1 ax t))
("intuition" nil (ih1 n t uw0 ih1 sh ax n))
("intuitive" nil (ih0 n t uw1 ax t ih0 v))
("intuitively" nil (ih0 n t uw1 ih0 t ih0 v l iy0))
("inuit" nil (ih1 n uw0 t))
("inundate" nil (ih1 n ax n d ey1 t))
("inundated" nil (ih1 n ax n d ey1 t ih0 d))
("inundations" nil (ih1 n ax n d ey1 sh ax n z))
("inured" nil (ih0 n y uh1 r d))
("invacare" nil (ih1 n v ax k eh1 r))
("invade" nil (ih0 n v ey1 d))
("invaded" nil (ih0 n v ey1 d ax d))
("invader" nil (ih0 n v ey1 d er0))
("invaders" nil (ih0 n v ey1 d er0 z))
("invades" nil (ih0 n v ey1 d z))
("invading" nil (ih0 n v ey1 d ih0 ng))
("invalid" n (ih1 n v ax l ax d))
("invalid" j (ih1 n v ah1 l ax d))
("invalidate" nil (ih0 n v ae1 l ih0 d ey1 t))
("invalidated" nil (ih0 n v ae1 l ax d ey1 t ax d))
("invalidating" nil (ih0 n v ae1 l ax d ey1 t ih0 ng))
("invalidation" nil (ih0 n v ae1 l ax d ey1 sh ax n))
("invalids" nil (ih1 n v ax l ax d z))
("invaluable" nil (ih0 n v ae1 l y ax b ax l))
("invariably" nil (ih0 n v eh1 r iy0 ax b l iy0))
("invariance" nil (ih0 n v eh1 r iy0 ax n s))
("invariant" nil (ih0 n v eh1 r iy0 ax n t))
("invasion" nil (ih0 n v ey1 zh ax n))
("invasions" nil (ih0 n v ey1 zh ax n z))
("invasive" nil (ih0 n v ey1 s ih0 v))
("invective" nil (ih0 n v eh1 k t ih0 v))
("invent" nil (ih0 n v eh1 n t))
("invented" nil (ih0 n v eh1 n t ax d))
("inventing" nil (ih0 n v eh1 n t ih0 ng))
("invention" nil (ih0 n v eh1 n sh ax n))
("inventions" nil (ih0 n v eh1 n sh ax n z))
("inventive" nil (ih0 n v eh1 n t ih0 v))
("inventiveness" nil (ih0 n v eh1 n t ih0 v n ax s))
("inventor" nil (ih0 n v eh1 n t er0))
("inventoried" nil (ih1 n v ax n t ao1 r iy0 d))
("inventories" nil (ih1 n v ax n t ao1 r iy0 z))
("inventors" nil (ih0 n v eh1 n t er0 z))
("inventory" nil (ih1 n v ax n t ao1 r iy0))
("invents" nil (ih0 n v eh1 n t s))
("inverlat" nil (ih1 n v er0 l ae1 t))
("inverness" nil (ih1 n v er0 n eh1 s))
("inverse" nil (ih0 n v er1 s))
("inversely" nil (ih0 n v er1 s l iy0))
("inversion" nil (ih0 n v er1 zh ax n))
("invertebrates" nil (ih0 n v er1 t ax b r ax t s))
("inverted" nil (ih0 n v er1 t ih0 d))
("invesco" nil (ih0 n v eh1 s k ow0))
("invest" nil (ih0 n v eh1 s t))
("investable" nil (ih0 n v eh1 s t ax b ax l))
("investcorp" nil (ih0 n v eh1 s t k ao0 r p))
("invested" nil (ih0 n v eh1 s t ax d))
("investigate" nil (ih0 n v eh1 s t ax g ey1 t))
("investigated" nil (ih0 n v eh1 s t ax g ey1 t ax d))
("investigates" nil (ih0 n v eh1 s t ax g ey1 t s))
("investigating" nil (ih0 n v eh1 s t ax g ey1 t ih0 ng))
("investigation" nil (ih0 n v eh1 s t ax g ey1 sh ax n))
("investigational" nil (ih0 n v eh0 s t ih0 g ey1 sh ax n ax l))
("investigations" nil (ih0 n v eh1 s t ax g ey1 sh ax n z))
("investigative" nil (ih0 n v eh1 s t ax g ey1 t ih0 v))
("investigator" nil (ih0 n v eh1 s t ax g ey1 t er0))
("investigators" nil (ih0 n v eh1 s t ax g ey1 t er0 z))
("investigatory" nil (ih0 n v eh1 s t ax g ax t ao1 r iy0))
("investimento" nil (ih0 n v eh1 s t ih0 m eh1 n t ow0))
("investing" nil (ih0 n v eh1 s t ih0 ng))
("investissements" nil (ih1 n v eh1 s t iy1 z ih0 m aa0 n t s))
("investiture" nil (ih0 n v eh1 s t ax ch er0))
("investment" nil (ih0 n v eh1 s t m ax n t))
("investments" nil (ih0 n v eh1 s t m ax n t s))
("investnet" nil (ih0 n v eh1 s t n eh1 t))
("investor" nil (ih0 n v eh1 s t er0))
("investors" nil (ih0 n v eh1 s t er0 z))
("invests" nil (ih0 n v eh1 s t s))
("inveterate" nil (ih0 n v eh1 t er0 ax t))
("invidious" nil (ih0 n v ih1 d iy0 ax s))
("invigorate" nil (ih0 n v ih1 g er0 ih0 t))
("invigorated" nil (ih0 n v ih1 g er0 ey1 t ih0 d))
("invigorating" nil (ih0 n v ih1 g er0 ey1 t ih0 ng))
("invincibility" nil (ih0 n v ih1 n s ax b ih1 l ih0 t iy0))
("invincible" nil (ih0 n v ih1 n s ax b ax l))
("inviolable" nil (ih0 n v ay1 ax l ax b ax l))
("inviolate" nil (ih0 n v ay1 ax l ih0 t))
("invirase" nil (ih1 n v ay1 r ey1 z))
("invisibility" nil (ih0 n v ih1 z ax b ih1 l ax t iy0))
("invisible" nil (ih0 n v ih1 z ax b ax l))
("invisibles" nil (ih1 n v ih1 z ax b ax l z))
("invitation" nil (ih1 n v ih0 t ey1 sh ax n))
("invitational" nil (ih1 n v ax t ey1 sh ax n ax l))
("invitations" nil (ih1 n v ih0 t ey1 sh ax n z))
("invite" v (ih0 n v ay1 t))
("invite" n (ih1 n v ay0 t))
("invited" nil (ih0 n v ay1 t ax d))
("invites" v (ih0 n v ay1 t s))
("invites" n (ih1 n v ay0 t s))
("inviting" nil (ih0 n v ay1 t ih0 ng))
("invitron" nil (ih1 n v ih0 t r aa0 n))
("invocation" nil (ih1 n v ax k ey1 sh ax n))
("invoice" nil (ih1 n v oy0 s))
("invoices" nil (ih1 n v oy0 s ih0 z))
("invoicing" nil (ih1 n v oy1 s ih0 ng))
("invoke" nil (ih0 n v ow1 k))
("invoked" nil (ih0 n v ow1 k t))
("invokes" nil (ih0 n v ow1 k s))
("invoking" nil (ih0 n v ow1 k ih0 ng))
("involuntarily" nil (ih1 n v ow0 l ah1 n t er0 ih1 l iy0))
("involuntary" nil (ih0 n v aa1 l ax n t eh1 r iy0))
("involve" nil (ih0 n v aa1 l v))
("involved" nil (ih0 n v aa1 l v d))
("involvement" nil (ih0 n v aa1 l v m ax n t))
("involves" nil (ih0 n v aa1 l v z))
("involving" nil (ih0 n v aa1 l v ih0 ng))
("invulnerability" nil (ih0 n v ah1 l n er0 ax b ih1 l ih0 t iy0))
("invulnerable" nil (ih0 n v ah1 l n er0 ax b ax l))
("inward" nil (ih1 n w er0 d))
("inwardly" nil (ih1 n w er0 d l iy0))
("inwood" nil (ih1 n w uh1 d))
("iny" nil (ih1 n iy0))
("inzer" nil (ih1 n z er0))
("io" nil (ay1 ow0))
("iodice" nil (ay1 ax d ay1 s))
("iodide" nil (ay1 ax d ay1 d))
("iodides" nil (ay1 ax d ay1 d z))
("iodine" nil (ay1 ax d ay1 n))
("iola" nil (ay0 ow1 l ax))
("iolande" nil (iy0 ow0 l aa1 n d iy0))
("iole" nil (iy0 ow1 l iy0))
("iomega" nil (ay1 ow0 m ey1 g ax))
("ion" nil (ay1 ax n))
("iona" nil (ay0 ow1 n ax))
("ione" nil (ay0 ow1 n iy0))
("ionic" nil (ay0 aa1 n ih0 k))
("ionics" nil (ay0 aa1 n ih0 k s))
("ionization" nil (ay1 ax n ax z ey1 sh ax n))
("ionizers" nil (ay1 ax n ay1 z er0 z))
("ionizing" nil (ay1 ax n ay1 z ih0 ng))
("ionospheric" nil (ay0 aa1 n ax s f eh1 r ih0 k))
("ions" nil (ay1 ax n z))
("iorio" nil (iy0 ao1 r iy0 ow0))
("ios" nil (iy1 ow0 s))
("iosif" nil (ay1 ax s ih0 f))
("iosue" nil (ay0 ow1 s uw0))
("iota" nil (ay0 ow1 t ax))
("iott" nil (ay1 ax t))
("iou" nil (ay1 ow1 y uw1))
("iovine" nil (iy0 ow0 v iy1 n iy0))
("iovino" nil (iy0 ow0 v iy1 n ow0))
("iowa" nil (ay1 ax w ax))
("iowan" nil (ay1 ax w ax n))
("iowans" nil (ay1 ax w ax n z))
("ip" nil (ih1 p))
("ipalco" nil (iy0 p ae1 l k ow0))
("ipco" nil (ih1 p k ow0))
("ipock" nil (ih1 p ax k))
("ippolito" nil (ih1 p ow0 l iy1 t ow0))
("ipsco" nil (ih1 p s k ow0))
("ipsen" nil (ih1 p s ax n))
("iptay" nil (ih1 p t ey0))
("iqbal" nil (ih1 k b ax l))
("ira" nil (ay1 r ax))
("iran" nil (ih0 r aa1 n))
("iranamok" nil (ay1 r ax n aa1 m aa0 k))
("irangate" nil (ih0 r aa1 n g ey1 t))
("irani" nil (ih0 r aa1 n iy0))
("iranian" nil (ih0 r aa1 n iy0 ax n))
("iranians" nil (ay0 r ey1 n iy0 ax n z))
("iranscam" nil (ay0 r ae1 n s k ae0 m))
("iraq" nil (ih0 r aa1 k))
("iraqi" nil (ih0 r ae1 k iy0))
("iraqis" nil (ih0 r ae1 k iy0 z))
("iras" nil (ay1 r ax z))
("irascible" nil (ih0 r ae1 s ih0 b ax l))
("irate" nil (ay0 r ey1 t))
("irby" nil (er1 b iy0))
("ire" nil (ay1 r))
("ireene" nil (ay0 r iy1 n))
("irelan" nil (ih0 r ey0 l aa1 n))
("ireland" nil (ay1 er0 l ax n d))
("irell" nil (ay0 r eh1 l))
("irena" nil (ih0 r ey1 n ax))
("irene" nil (ay0 r iy1 n))
("ireson" nil (ih1 r ih0 s ax n))
("ireta" nil (ih0 r ey1 t ax))
("ireton" nil (ih1 r ih0 t aa0 n))
("iretta" nil (ih0 r eh1 t ax))
("irette" nil (ih0 r eh1 t))
("irey" nil (ay1 r iy0))
("irian" nil (ay1 r iy0 ax n))
("irianese" nil (ay0 r iy1 ax n iy1 s))
("irick" nil (ih1 r ih0 k))
("iridescent" nil (ih1 r ax d eh1 s ax n t))
("iridium" nil (ih0 r ih1 d iy0 ax m))
("irimajiri" nil (ay0 r iy1 m ax jh ih1 r iy0))
("irina" nil (ih0 r iy1 n ax))
("iris" nil (ay1 r ax s))
("irises" nil (ay1 r ax s ih0 z))
("irish" nil (ay1 r ih0 sh))
("irishman" nil (ay1 r ih0 sh m ax n))
("irizarry" nil (ih1 r ih0 z ae0 r iy0))
("irk" nil (er1 k))
("irked" nil (er1 k t))
("irks" nil (er1 k s))
("irksome" nil (er1 k s ax m))
("irkutsk" nil (er0 k uh1 t s k))
("irlbeck" nil (er1 l b eh0 k))
("irma" nil (er1 m ax))
("iron" nil (ay1 er0 n))
("ironclad" nil (ay1 er0 n k l ae1 d))
("ironed" nil (ay1 er0 n d))
("ironic" nil (ay0 r aa1 n ih0 k))
("ironical" nil (ay0 r aa1 n ih0 k ax l))
("ironically" nil (ay0 r aa1 n ih0 k l iy0))
("ironies" nil (ay1 r ax n iy0 z))
("ironing" nil (ay1 er0 n ih0 ng))
("ironize" nil (ay1 er0 n ay1 z))
("ironized" nil (ay1 er0 n ay1 z d))
("ironizer" nil (ay1 er0 n ay1 z er0))
("ironizes" nil (ay1 er0 n ay1 z ih0 z))
("ironizing" nil (ay1 er0 n ay1 z ih0 ng))
("irons" nil (ay1 er0 n z))
("ironside" nil (ay1 er0 n s ay1 d))
("ironsides" nil (ay1 er0 n s ay1 d z))
("ironton" nil (ay1 r n t ax n))
("ironwood" nil (ay1 er0 n w uh1 d))
("irony" nil (ay1 r ax n iy0))
("iroquois" nil (ih1 r ax k w oy1))
("irradiated" nil (ih0 r ey1 d iy0 ey1 t ih0 d))
("irradiation" nil (ih0 r ey1 d iy0 ey1 sh ax n))
("irrational" nil (ih0 r ae1 sh ax n ax l))
("irrationality" nil (ih0 r ae1 sh ax n ae1 l ax t iy0))
("irrationally" nil (ih0 r ae1 sh ax n ax l iy0))
("irreconcilable" nil (ih0 r eh1 k ax n s ay1 l ax b ax l))
("irredentism" nil (ih1 r ax d eh1 n t ih0 z ax m))
("irrefutable" nil (ih0 r ax f y uw1 t ax b ax l))
("irregular" nil (ih0 r eh1 g y ax l er0))
("irregularities" nil (ih0 r eh0 g y ax l eh1 r ax t iy0 z))
("irregularity" nil (ih0 r eh1 g y ax l eh1 r ax t iy0))
("irregularly" nil (ih0 r eh1 g y ax l er0 l iy0))
("irrelevance" nil (ih0 r eh1 l ax v ax n s))
("irrelevancy" nil (ih0 r eh1 l ax v ax n s iy0))
("irrelevant" nil (ih0 r eh1 l ax v ax n t))
("irremediable" nil (ih1 r ih0 m iy1 d iy0 ax b ax l))
("irreparable" nil (ih0 r eh1 p er0 ax b ax l))
("irreparably" nil (ih0 r eh1 p er0 ax b l iy0))
("irreplaceable" nil (ih0 r ax p l ey1 s ax b ax l))
("irrepressible" nil (ih0 r ax p r eh1 s ax b ax l))
("irresistible" nil (ih1 r ih0 z ih1 s t ax b ax l))
("irresistibly" nil (ih1 r ih0 z ih1 s t ax b l iy0))
("irrespective" nil (ih0 r ax s p eh1 k t ih0 v))
("irresponsibility" nil (ih0 r ax s p aa1 n s ax b ih1 l ax t iy0))
("irresponsible" nil (ih0 r ax s p aa1 n s ax b ax l))
("irresponsibly" nil (ih0 r ax s p aa1 n s ax b l iy0))
("irretrievably" nil (ih0 r ax t r iy1 v ax b l iy0))
("irreverence" nil (ih0 r eh1 v er0 ax n s))
("irreverent" nil (ih0 r eh1 v er0 ax n t))
("irreversible" nil (ih1 r ih0 v er1 s ax b ax l))
("irreversibly" nil (ih1 r ih0 v er1 s ax b l iy0))
("irrevocable" nil (ih0 r eh1 v ax k ax b ax l))
("irrevocably" nil (ih0 r eh1 v ax k ax b l iy0))
("irrigate" nil (ih1 r ax g ey1 t))
("irrigated" nil (ih1 r ax g ey1 t ih0 d))
("irrigation" nil (ih1 r ax g ey1 sh ax n))
("irrigators" nil (ih1 r ax g ey1 t er0 z))
("irritable" nil (ih1 r ax t ax b ax l))
("irritant" nil (ih1 r ax t ax n t))
("irritants" nil (ih1 r ax t ax n t s))
("irritate" nil (ih1 r ih0 t ey1 t))
("irritated" nil (ih1 r ax t ey1 t ax d))
("irritates" nil (ih1 r ih0 t ey1 t s))
("irritating" nil (ih1 r ax t ey1 t ih0 ng))
("irritation" nil (ih1 r ih0 t ey1 sh ax n))
("irv" nil (er1 v))
("irvette" nil (er0 v eh1 t))
("irvin" nil (er1 v ih0 n))
("irvine" nil (er1 v ay0 n))
("irving" nil (er1 v ih0 ng))
("irwin" nil (er1 w ax n))
("irwindale" nil (er1 w ih0 n d ey1 l))
("is" nil (ih1 z))
("isa" nil (iy1 s ax))
("isaac" nil (ay1 z ax k))
("isaacks" nil (ih1 s aa0 k s))
("isaacs" nil (ay1 z ih0 k s))
("isaacson" nil (ay1 z ih0 k s ax n))
("isaak" nil (ih0 s aa1 k))
("isabel" nil (ih1 z ax b eh1 l))
("isabell" nil (ih0 s aa0 b eh1 l))
("isabella" nil (ih1 z ax b eh1 l ax))
("isabelle" nil (ih1 z ax b eh1 l))
("isackson" nil (ih1 s ax k s ax n))
("isadore" nil (ih0 s aa0 d ao1 r ey0))
("isaiah" nil (ay1 z ey1 ax))
("isaksen" nil (ih1 s ax k s ax n))
("isakson" nil (ih1 s ax k s ax n))
("isaly" nil (ih1 s ax l iy0))
("isautier" nil (ay0 s ao1 t y er0))
("isbell" nil (ih1 s b eh0 l))
("isbill" nil (ih0 s b ih1 l))
("iscariot" nil (ih0 s k eh1 r iy0 ax t))
("isch" nil (ih1 sh))
("ise" nil (ay1 z))
("iselin" nil (ih1 s ih0 l ih0 n))
("iseman" nil (ay1 s m ax n))
("iseminger" nil (ih1 s iy0 m ih0 ng er0))
("isenberg" nil (ay1 z ax n b er0 g))
("isenhart" nil (ay1 z ax n hh aa1 r t))
("isenhour" nil (ih1 s ih0 n aw0 r))
("isenhower" nil (ih1 s ih0 n hh ow0 er0))
("isensee" nil (ay1 z ax n s iy1))
("iser" nil (ay1 z er0))
("isetan" nil (ih1 s ih0 t ax n))
("isgrigg" nil (ih0 s g r ih1 g))
("isgro" nil (iy1 s g r ow0))
("isgur" nil (ih1 s g er0))
("ish" nil (ih1 sh))
("isham" nil (ih1 sh ax m))
("ishaq" nil (ih1 sh ae0 k))
("ishee" nil (ih1 sh iy0))
("isherwood" nil (ih1 sh er0 w uh1 d))
("ishi" nil (ih1 sh iy0))
("ishibashi" nil (ih0 sh iy0 b aa1 sh iy0))
("ishida" nil (ih0 sh iy1 d ax))
("ishihara" nil (ih0 sh iy0 hh aa1 r ax))
("ishii" nil (ih0 sh iy1 iy0))
("ishikawa" nil (ih0 sh iy0 k aa1 w ax))
("ishikawajima" nil (iy1 sh ih0 k aa1 w ax jh iy1 m ax))
("ishikura" nil (ih1 sh ih0 k uh1 r ax))
("ishimura" nil (iy1 sh ih0 m uw1 r ax))
("ishler" nil (ih1 sh l er0))
("ishmael" nil (ih1 sh m iy0 l))
("ishman" nil (ih1 sh m ax n))
("ishtar" nil (ih1 sh t aa0 r))
("isidore" nil (ih1 z ih0 d ao1 r))
("isis" nil (ay1 s ax s))
("iskra" nil (ih1 s k r ax))
("islam" nil (ih0 s l aa1 m))
("islamabad" nil (ih0 s l ae1 m ax b ae0 d))
("islami" nil (ih0 z l aa1 m iy0))
("islamic" nil (ih0 z l aa1 m ih0 k))
("islamization" nil (ih1 z l aa0 m ax z ey1 sh ax n))
("island" nil (ay1 l ax n d))
("islander" nil (ay1 l ax n d er0))
("islanders" nil (ay1 l ax n d er0 z))
("islandia" nil (ay1 l ae1 n d iy0 ax))
("islands" nil (ay1 l ax n d z))
("islas" nil (ay1 l ax z))
("isle" nil (ay1 l))
("isleen" nil (ay1 l iy0 n))
("isler" nil (ay1 l er0))
("isles" nil (ay1 l z))
("islets" nil (ay1 l ax t s))
("isley" nil (ay1 l iy0))
("islip" nil (ay1 s l ih0 p))
("ism" nil (ih1 z ax m))
("ismail" nil (ih1 s m ey0 l))
("isms" nil (ih1 z ax m z))
("isner" nil (ih1 s n er0))
("isocyanate" nil (ay1 s ax s ay1 ax n ey1 t))
("isoda" nil (iy1 s ow1 d ax))
("isoelectronic" nil (ay1 s ow0 ih0 l eh0 k t r aa1 n ih0 k))
("isoetec" nil (ay1 s ow0 t eh1 k))
("isola" nil (ay0 s aa1 l ax))
("isolate" nil (ay1 s ax l ey1 t))
("isolated" nil (ay1 s ax l ey1 t ax d))
("isolates" nil (ay1 s ax l ey1 t s))
("isolating" nil (ay1 s ax l ey1 t ih0 ng))
("isolation" nil (ay1 s ax l ey1 sh ax n))
("isolationism" nil (ay1 s ax l ey1 sh ax n ih1 z ax m))
("isolationist" nil (ay1 s ax l ey1 sh ax n ax s t))
("isolationists" nil (ay1 s ax l ey1 sh ax n ih0 s t s))
("isolde" nil (ih0 s ow1 l d ax))
("isom" nil (ay1 s ax m))
("isomedix" nil (ay1 s ow0 m eh1 d ih0 k s))
("isomer" nil (ay1 s ax m er0))
("isomers" nil (ay1 s ax m er0 z))
("isomorphism" nil (ay1 s ax m ao1 r f ih0 z ax m))
("ison" nil (ih1 s ax n))
("isoprinosine" nil (ih1 s ax p r ih1 n ax s iy1 n))
("isosceles" nil (ih0 s ao1 s eh1 l eh0 z))
("isothermal" nil (ay1 s ax th er1 m ax l))
("isotoner" nil (ih1 z ow0 t ah1 n er0))
("isotonic" nil (ay1 s ax t aa1 n ih0 k))
("isotope" nil (ay1 s ax t ow1 p))
("isotopes" nil (ay1 s ax t ow1 p s))
("isotopic" nil (ay1 s ax t aa1 p ih0 k))
("isoxicam" nil (ih0 s aa1 k s ih0 k ax m))
("ispra" nil (ih1 s p r ax))
("israel" nil (ih1 z r iy0 ax l))
("israeli" nil (ih0 z r ey1 l iy0))
("israelis" nil (ih0 z r ey1 l iy0 z))
("israeloff" nil (ih0 s r ey1 l ao0 f))
("israelson" nil (ih1 s r eh0 l s ax n))
("isreal" nil (ih1 z r ax l))
("issa" nil (ih1 s ax))
("issac" nil (ih1 s ax k))
("issam" nil (ih1 s ax m))
("issar" nil (iy1 s aa1 r))
("issie" nil (ih1 s iy0))
("issing" nil (ih1 s ih0 ng))
("issuable" nil (ih1 sh uw0 ax b ax l))
("issuance" nil (ih1 sh uw0 ax n s))
("issuances" nil (ih1 sh uw0 ax n s ih0 z))
("issue" nil (ih1 sh uw0))
("issued" nil (ih1 sh uw0 d))
("issueless" nil (ih1 sh uw0 l ax s))
("issuer" nil (ih1 sh uw0 er0))
("issuers" nil (ih1 sh uw0 er0 z))
("issues" nil (ih1 sh uw0 z))
("issuing" nil (ih1 sh uw0 ih0 ng))
("issy" nil (ih1 s iy0))
("istanbul" nil (ih1 s t aa0 n b uw1 l))
("istat" nil (ih1 s t ae0 t))
("istel" nil (ih1 s t eh0 l))
("isthmus" nil (ih1 s m ax s))
("istituto" nil (ih1 s t ih0 t uw1 t ow0))
("istook" nil (ih1 s t uh1 k))
("istre" nil (ih1 s t er0))
("isuzu" nil (ih0 s uw1 z uw0))
("it" nil (ih1 t))
("ita" nil (ay1 t ax))
("italia" nil (iy0 t ae1 l iy0 ax))
("italian" nil (ih0 t ae1 l y ax n))
("italiana" nil (ih0 t ae1 l iy0 ae1 n ax))
("italianate" nil (ih0 t ae1 l y ax n ey1 t))
("italiano" nil (ih0 t aa0 l iy0 aa1 n ow0))
("italians" nil (ih0 t ae1 l y ax n z))
("italic" nil (ay0 t ae1 l ih0 k))
("italics" nil (ay0 t ae1 l ih0 k s))
("italtel" nil (ih0 t ae1 l t eh1 l))
("italy" nil (ih1 t ax l iy0))
("itamar" nil (iy1 t ax m aa1 r))
("itami" nil (ih0 t aa1 m iy0))
("itar" nil (iy1 t aa1 r))
("itasca" nil (ay0 t ae1 s k ax))
("itch" nil (ih1 ch))
("itches" nil (ih1 ch ax z))
("itching" nil (ih1 ch ih0 ng))
("itchy" nil (ih1 ch iy0))
("itek" nil (ay1 t eh1 k))
("itel" nil (ay1 t eh1 l))
("item" nil (ay1 t ax m))
("itemize" nil (ay1 t ax m ay1 z))
("itemized" nil (ay1 t ax m ay1 z d))
("itemizers" nil (ay1 t ax m ay1 z er0 z))
("itemizing" nil (ay1 t ax m ay1 z ih0 ng))
("items" nil (ay1 t ax m z))
("iten" nil (ay1 t ax n))
("iterative" nil (ih1 t er0 ey1 t ih0 v))
("ithaca" nil (ih1 th ax k ax))
("itinerant" nil (ay0 t ih1 n er0 ax n t))
("itineraries" nil (ay0 t ih1 n er0 eh1 r iy0 z))
("itinerary" nil (ay0 t ih1 n er0 eh1 r iy0))
("itkin" nil (ih0 t k ih1 n))
("ito" nil (iy1 t ow0))
("itochu" nil (iy1 t ow1 ch uw1))
("itoh" nil (iy1 t ow0))
("its" nil (ih1 t s))
("itself" nil (ih0 t s eh1 l f))
("ittner" nil (ih1 t n er0))
("itzhak" nil (ih1 t s aa0 k))
("itzkowitz" nil (ih1 t s k ax w ih0 t s))
("iuliano" nil (iy0 uw0 l iy0 aa1 n ow0))
("iva" nil (iy1 v ax))
("ivaco" nil (ay1 v ax k ow1))
("ivan" nil (ay1 v ax n))
("ivana" nil (ih0 v aa1 n ax))
("ivancic" nil (ih0 v ae1 ng k ih0 k))
("ivane" nil (ih1 v ax n))
("ivanhoe" nil (ay1 v ax n hh ow1))
("ivanna" nil (ih0 v aa1 n ax))
("ivanoff" nil (ih1 v ax n ao0 f))
("ivanov" nil (ih1 v ax n aa0 v))
("ivar" nil (ih1 v er0))
("ivatans" nil (ay1 v ax t ax n z))
("ivax" nil (ay1 v ae0 k s))
("iven" nil (ay1 v ax n))
("ivens" nil (ay1 v ax n z))
("iver" nil (ay1 v er0))
("ivern" nil (ay1 v er0 n))
("ivers" nil (ay1 v er0 z))
("iversen" nil (ih0 v er1 s ax n))
("iverson" nil (ay1 v er0 s ax n))
("ivery" nil (ay1 v er0 iy0))
("ives" nil (ay1 v z))
("ivester" nil (ay1 v eh1 s t er0))
("ivey" nil (ay1 v iy0))
("ivie" nil (ay1 v iy0))
("ivins" nil (ih1 v ih0 n z))
("ivor" nil (ay1 v er0))
("ivorians" nil (ay0 v ao1 r iy0 ax n z))
("ivory" nil (ay1 v er0 iy0))
("ivy" nil (ay1 v iy0))
("iwai" nil (ay0 w ay1))
("iwaki" nil (ay0 w aa1 k iy0))
("iwamoto" nil (ih0 w aa0 m ow1 t ow0))
("iwan" nil (iy1 w aa0 n))
("iwanicki" nil (ih0 v ax n ih1 t s k iy0))
("iwanski" nil (ih0 v ae1 n s k iy0))
("iwasaki" nil (iy1 w aa0 s aa1 k iy0))
("iwata" nil (ih0 w aa1 t ax))
("iwate" nil (ay0 w ey1 t))
("iwen" nil (ih1 w ax n))
("iwerks" nil (ay1 w er1 k s))
("iwinski" nil (ih0 v ih1 n s k iy0))
("iwo" nil (ay1 w ow0))
("ixion" nil (ih0 k s ay1 ax n))
("iyer" nil (ay1 er0))
("izaguirre" nil (ih0 z ax g w ih1 r ey0))
("izansky" nil (ih0 z ae1 n s k iy0))
("izard" nil (ih1 z er0 d))
("izatt" nil (iy1 z aa0 t))
("izetbegovic" nil (ih1 z eh0 t b eh1 g ow0 v ih1 ch))
("izod" nil (ay1 z aa0 d))
("izquierdo" nil (ih0 z k w ih1 r d ow0))
("izumi" nil (ih0 z uw1 m iy0))
("izvestia" nil (ih0 z v eh1 s t iy0 ax))
("izzard" nil (ih1 z er0 d))
("izzi" nil (ih1 z iy0))
("izzo" nil (ih1 z ow0))
("izzy" nil (ih1 z iy0))
("j" nil (jh ey1))
("ja" nil (y aa1))
("jab" nil (jh ae1 b))
("jabbar" nil (jh ax b aa1 r))
("jabbed" nil (jh ae1 b d))
("jabbing" nil (jh ae1 b ih0 ng))
("jabbour" nil (zh ax b uh1 r))
("jaber" nil (jh ey1 b er0))
("jablon" nil (jh ae1 b l ax n))
("jablonowski" nil (y ax b l ax n ao1 f s k iy0))
("jablonski" nil (y ax b l aa1 n s k iy0))
("jablonsky" nil (y ax b l aa1 n s k iy0))
("jabs" nil (jh ae1 b z))
("jac" nil (jh ae1 k))
("jacek" nil (jh aa1 ch eh1 k))
("jachim" nil (jh ae1 k ih0 m))
("jacinda" nil (jh ax s ih1 n d ax))
("jacinta" nil (jh ax s ih1 n t ax))
("jacinth" nil (jh ae1 s ih0 n th))
("jacintha" nil (jh ax s ih1 n th ax))
("jacinthe" nil (jh ae1 s ih0 n th))
("jacinto" nil (jh ax s ih1 n t ow0))
("jack" nil (jh ae1 k))
("jacka" nil (jh ae1 k ax))
("jackal" nil (jh ae1 k ax l))
("jackass" nil (jh ae1 k ae1 s))
("jacked" nil (jh ae1 k t))
("jackel" nil (jh ae1 k ax l))
("jackelyn" nil (jh ae1 k ih0 l ih0 n))
("jacket" nil (jh ae1 k ax t))
("jacketed" nil (jh ae1 k ax t ih0 d))
("jackets" nil (jh ae1 k ih0 t s))
("jackett" nil (jh ae1 k ih0 t))
("jackhammer" nil (jh ae1 k hh ae1 m er0))
("jackhammers" nil (jh ae1 k hh ae1 m er0 z))
("jacki" nil (jh ae1 k iy0))
("jackie" nil (jh ae1 k iy0))
("jacking" nil (jh ae1 k ih0 ng))
("jackley" nil (jh ae1 k l iy0))
("jacklin" nil (jh ae1 k l ih0 n))
("jackman" nil (jh ae1 k m ax n))
("jacko" nil (jh ae1 k ow0))
("jackovich" nil (jh ae1 k ax v ih0 ch))
("jackowski" nil (y ax s k ao1 f s k iy0))
("jackpot" nil (jh ae1 k p aa1 t))
("jackpots" nil (jh ae1 k p aa1 t s))
("jackrabbit" nil (jh ae1 k r ae1 b ax t))
("jacks" nil (jh ae1 k s))
("jackson" nil (jh ae1 k s ax n))
("jacksonian" nil (jh ae0 k s ow1 n iy0 ax n))
("jacksons" nil (jh ae1 k s ax n z))
("jacksonville" nil (jh ae1 k s ax n v ih1 l))
("jacky" nil (jh ae1 k iy0))
("jaclyn" nil (jh ae1 k l ih0 n))
("jaco" nil (y aa1 k ow0))
("jacob" nil (jh ey1 k ax b))
("jacoba" nil (y aa0 k ow1 b ax))
("jacobe" nil (jh ae1 k ax b))
("jacobean" nil (jh ae1 k ax b iy1 ax n))
("jacober" nil (jh ey1 k ax b er0))
("jacobi" nil (y aa0 k ow1 b iy0))
("jacobin" nil (jh ae1 k ax b ax n))
("jacobina" nil (y ax k ax b ay1 n ax))
("jacobine" nil (jh ae1 k ax b ay1 n))
("jacobite" nil (jh ae1 k ax b ay1 t))
("jacobites" nil (jh ae1 k ax b ay1 t s))
("jacobo" nil (y aa0 k ow1 b ow0))
("jacoboski" nil (jh ae1 k ow0 b ao1 s k iy0))
("jacobowitz" nil (y ax k aa1 b ax w ih0 t s))
("jacobs" nil (jh ey1 k ax b z))
("jacobsen" nil (jh ey1 k ax b s ax n))
("jacobson" nil (jh ey1 k ax b s ax n))
("jacobus" nil (jh ax k ow1 b ax s))
("jacoby" nil (jh ax k ow1 b iy0))
("jacome" nil (jh ax k ow1 m))
("jacor" nil (jh ey1 k ao1 r))
("jacot" nil (jh ae1 k ax t))
("jacox" nil (jh ae1 k aa0 k s))
("jacque" nil (zh ey1 k))
("jacqueline" nil (jh ae1 k w ax l ih0 n))
("jacquelyn" nil (jh ae1 k w ih0 l ih0 n))
("jacques" nil (zh aa1 k))
("jacquet" nil (y aa1 k ey0 t))
("jacquetta" nil (jh aa0 k eh1 t ax))
("jacquez" nil (y aa0 k w eh1 z))
("jacquie" nil (jh ae1 k w iy0))
("jacquin" nil (jh ae1 k w ih0 n))
("jacquot" nil (zh ax k w ow1))
("jacuzzi" nil (jh ax k y uw1 z iy0))
("jade" nil (jh ey1 d))
("jaded" nil (jh ey1 d ax d))
("jadeite" nil (jh ey1 d ay1 t))
("jades" nil (jh ey1 d z))
("jadwin" nil (jh ae1 d w ih0 n))
("jae" nil (jh ey1))
("jaeckel" nil (y eh1 k ax l))
("jaeger" nil (y eh1 g er0))
("jaegers" nil (y eh1 g er0 z))
("jaekel" nil (y eh1 k ax l))
("jaenicke" nil (y eh1 n ih0 k))
("jafco" nil (jh ae1 f k ow0))
("jaffa" nil (jh ae1 f ax))
("jaffar" nil (jh ax f aa1 r))
("jaffe" nil (jh ae1 f iy0))
("jaffee" nil (jh ae1 f iy0))
("jaffna" nil (jh ae1 f n ax))
("jaffray" nil (jh ae1 f r ey0))
("jaffrey" nil (jh ae1 f r iy0))
("jager" nil (jh ey1 g er0))
("jagged" nil (jh ae1 g d))
("jagger" nil (jh ae1 g er0))
("jaggers" nil (jh ae1 g er0 z))
("jagiello" nil (jh aa0 jh eh1 l ow0))
("jagielski" nil (y ax g iy1 l s k iy0))
("jaglom" nil (jh ae1 g l ax m))
("jago" nil (y aa1 g ow0))
("jagoda" nil (jh aa0 g ow1 d ax))
("jagodzinski" nil (y ax g ax jh ih1 n s k iy0))
("jagow" nil (y aa1 g ow0))
("jags" nil (jh ae1 g z))
("jagt" nil (jh ae1 t))
("jaguar" nil (jh ae1 g w aa1 r))
("jaguars" nil (jh ae1 g w aa1 r z))
("jahn" nil (jh ae1 n))
("jahner" nil (jh aa1 n er0))
("jahnke" nil (jh ae1 ng k))
("jahns" nil (jh aa1 n z))
("jahr" nil (jh aa1 r))
("jai" nil (jh ay1))
("jail" nil (jh ey1 l))
("jailed" nil (jh ey1 l d))
("jailhouse" nil (jh ey1 l hh aw1 s))
("jailing" nil (jh ey1 l ih0 ng))
("jails" nil (jh ey1 l z))
("jaime" nil (hh ay1 m iy0))
("jaimes" nil (hh ay1 m iy0 z))
("jain" nil (jh ay1 n))
("jainism" nil (jh ey1 n ih0 z ax m))
("jakab" nil (jh ae1 k ax b))
("jakarta" nil (jh ax k aa1 r t ax))
("jake" nil (jh ey1 k))
("jakel" nil (jh ae1 k ax l))
("jakes" nil (jh ey1 k s))
("jakeway" nil (jh ey1 k w ey1))
("jakie" nil (jh ae1 k iy0))
("jakob" nil (jh ae1 k ax b))
("jakobsen" nil (jh ae1 k ax b s ax n))
("jakubek" nil (y ax k uw1 b eh0 k))
("jakubiak" nil (y ax k ah1 b iy0 ae0 k))
("jakubik" nil (y ax k uw1 b ih0 k))
("jakubowicz" nil (y ax k ah1 b ax v ih0 ch))
("jakubowski" nil (y ax k ax b ao1 f s k iy0))
("jalalabad" nil (jh ax l ae1 l ax b ae0 d))
("jalbert" nil (jh ae1 l b er0 t))
("jalopy" nil (jh ax l aa1 p iy0))
("jam" nil (jh ae1 m))
("jama" nil (jh ae1 m ax))
("jamaica" nil (jh ax m ey1 k ax))
("jamaican" nil (jh ax m ey1 k ax n))
("jamaicans" nil (jh ax m ey1 k ax n z))
("jamail" nil (jh ax m ey1 l))
("jamal" nil (jh ax m aa1 l))
("jamar" nil (y aa0 m aa1 r))
("jamb" nil (jh ae1 m))
("jambor" nil (y aa0 m b ao1 r))
("jamerson" nil (jh ae1 m er0 s ax n))
("james" nil (jh ey1 m z))
("jameson" nil (jh ey1 m s ax n))
("jamestown" nil (jh ey1 m z t aw1 n))
("jamesway" nil (jh ey1 m z w ey1))
("jamie" nil (jh ey1 m iy0))
("jamieson" nil (jh ey1 m ih0 s ax n))
("jamison" nil (jh ey1 m ih0 s ax n))
("jammed" nil (jh ae1 m d))
("jammers" nil (jh ae1 m er0 z))
("jamming" nil (jh ae1 m ih0 ng))
("jammu" nil (jh ax m uw1))
("jamrog" nil (jh ae1 m r ax g))
("jamroz" nil (y aa1 m r ow0 z))
("jams" nil (jh ae1 m z))
("jamshedpur" nil (jh ae1 m sh eh1 d p er0))
("jan" nil (jh ae1 n))
("jana" nil (jh ae1 n ax))
("janacek" nil (y ae1 n ax ch eh0 k))
("janachowski" nil (jh ae1 n ax ch aw1 s k iy0))
("janak" nil (y ae1 n ax k))
("janardhan" nil (jh ax n aa1 r d ax n))
("janas" nil (jh ae1 n ax z))
("janca" nil (jh ae1 ng k ax))
("janco" nil (jh ae1 ng k ow0))
("janczak" nil (y ae1 n ch ae0 k))
("janda" nil (jh ae1 n d ax))
("jandreau" nil (zh ax n d r ow1))
("jandt" nil (jh ae1 n t))
("jane" nil (jh ey1 n))
("janecek" nil (y ae1 n ih0 ch eh0 k))
("janeczko" nil (y ax n eh1 ch k ow0))
("janeiro" nil (jh ax n eh1 r ow1))
("janek" nil (y ae1 n eh0 k))
("janel" nil (jh ae1 n ax l))
("janella" nil (jh ax n eh1 l ax))
("janelle" nil (zh ax n eh1 l))
("janes" nil (jh ey1 n z))
("janesville" nil (jh ey1 n z v ih1 l))
("janet" nil (jh ae1 n ax t))
("janette" nil (jh ax n eh1 t))
("janeway" nil (jh ey1 n w ey1))
("janey" nil (jh ey1 n iy0))
("jang" nil (jh ae1 ng))
("jangled" nil (jh ae1 ng g ax l d))
("jani" nil (y aa1 n iy0))
("janiak" nil (y ae1 n iy0 ae0 k))
("janice" nil (jh ae1 n ax s))
("janicek" nil (y ae1 n ih0 ch eh0 k))
("janick" nil (jh ae1 n ih0 k))
("janicke" nil (jh ae1 n ih0 k))
("janicki" nil (jh ax n ih1 k iy0))
("janie" nil (jh ey1 n iy0))
("janiga" nil (jh ae1 n ih0 g ax))
("janik" nil (y ae1 n ih0 k))
("janikowski" nil (y ax n ih0 k ao1 f s k iy0))
("janina" nil (jh ax n iy1 n ax))
("janine" nil (jh ax n iy1 n))
("janis" nil (jh ae1 n ih0 s))
("janisch" nil (jh ae1 n ih0 sh))
("janish" nil (y ae1 n ih0 sh))
("janiszewski" nil (y ax n ih0 sh eh1 f s k iy0))
("janitor" nil (jh ae1 n ax t er0))
("janitorial" nil (jh ae1 n ih0 t ao1 r iy0 ax l))
("janitors" nil (jh ae1 n ax t er0 z))
("jank" nil (jh ae1 ng k))
("janka" nil (jh ae1 ng k ax))
("janke" nil (jh ae1 ng k))
("jankiewicz" nil (y ae1 n k ax v ih0 ch))
("janklow" nil (jh ae1 ng k l ow0))
("janko" nil (jh ae1 ng k ow0))
("jankovic" nil (jh ae1 ng k ax v ih0 k))
("jankovich" nil (jh ae1 ng k ax v ih0 ch))
("jankowiak" nil (y ax ng k aw1 iy0 ae0 k))
("jankowski" nil (y ax ng k ao1 f s k iy0))
("jann" nil (jh ae1 n))
("janna" nil (jh ae1 n ax))
("janney" nil (jh ae1 n iy0))
("janning" nil (jh ae1 n ih0 ng))
("jannock" nil (jh ae1 n ax k))
("jannotta" nil (jh ax n aa1 t ax))
("janoff" nil (jh ae1 n ao0 f))
("janofsky" nil (jh ax n ao1 f s k iy0))
("janos" nil (jh ae1 n ow0 s))
("janosik" nil (y ax n ow1 s ih0 k))
("janoski" nil (y ax n aw1 s k iy0))
("janosko" nil (y ax n ow1 s k ow0))
("janosky" nil (y ax n ow1 s k iy0))
("janota" nil (y ax n ow1 t ax))
("janousek" nil (y ax n uw1 s eh0 k))
("janovsky" nil (jh ax n aa1 v s k iy0))
("janow" nil (y ae1 n ow0))
("janowiak" nil (y ax n aw1 iy0 ae0 k))
("janowicz" nil (y ae1 n ax v ih0 ch))
("janowitz" nil (y ae1 n ax w ih0 t s))
("janowski" nil (y ax n ao1 f s k iy0))
("jans" nil (jh ae1 n z))
("jansa" nil (jh ae1 n s ax))
("jansen" nil (jh ae1 n s ax n))
("jansenism" nil (jh ae1 n s ih0 n ih1 z ax m))
("jansenist" nil (jh ae1 n s ax n ax s t))
("jansenists" nil (jh ae1 n s ax n ax s t s))
("jansky" nil (y ae1 n s k iy0))
("jansma" nil (y ae1 n s m ax))
("janson" nil (jh ae1 n s ax n))
("janssen" nil (jh ae1 n s ax n))
("janssens" nil (jh ae1 n s ax n z))
("jansson" nil (jh ae1 n s ax n))
("janthina" nil (y aa0 n th iy1 n ax))
("jantz" nil (jh ae1 n t s))
("jantzen" nil (jh ae1 n t z ax n))
("january" nil (jh ae1 n y uw0 eh1 r iy0))
("janus" nil (jh ey1 n ax s))
("janusz" nil (y aa1 n uh0 sh))
("januszewski" nil (y ax n ax sh eh1 f s k iy0))
("janvrin" nil (jh ae1 n v r ih0 n))
("janz" nil (jh ae1 n z))
("janzen" nil (jh ae1 n z ax n))
("jap" nil (jh ae1 p))
("japan" nil (jh ax p ae1 n))
("japanese" nil (jh ae1 p ax n iy1 z))
("japans" nil (jh ax p ae1 n z))
("japonica" nil (jh ax p aa1 n ax k ax))
("japs" nil (jh ae1 p s))
("jaqua" nil (y ae1 k w ax))
("jaquay" nil (zh ax k ey1))
("jaques" nil (jh ae1 k s))
("jaquess" nil (y aa1 k w ih0 s))
("jaquez" nil (y aa0 k w eh1 z))
("jaquish" nil (jh ae1 k w ih0 sh))
("jaquith" nil (jh ae1 k w ih0 th))
("jar" nil (jh aa1 r))
("jara" nil (jh aa1 r ax))
("jaramillo" nil (jh ae1 r ax m ih1 l ow0))
("jarboe" nil (jh aa1 r b ow1))
("jarchow" nil (jh aa1 r ch aw1))
("jardin" nil (jh aa1 r d ih0 n))
("jardine" nil (jh aa1 r d ay1 n))
("jarecki" nil (y er0 eh1 t s k iy0))
("jared" nil (jh eh1 r ih0 d))
("jarema" nil (y er0 iy1 m ax))
("jares" nil (zh eh1 r z))
("jargon" nil (jh aa1 r g ax n))
("jarislowsky" nil (jh eh1 r ih0 s l aw1 s k iy0))
("jarman" nil (jh aa1 r m ax n))
("jarmon" nil (y aa0 r m ao1 n))
("jarnagin" nil (jh aa1 r n ax jh ih0 n))
("jarnigan" nil (jh aa1 r n ih0 g ae0 n))
("jaros" nil (y aa1 r ow0 z))
("jarosz" nil (y aa1 r ax sh))
("jaroszewski" nil (y er0 ax sh eh1 f s k iy0))
("jaroszynski" nil (jh eh1 r ax z ih1 n s k iy0))
("jarrard" nil (zh er0 aa1 r d))
("jarratt" nil (jh ae1 r ax t))
("jarreau" nil (zh er0 ow1))
("jarred" nil (jh aa1 r d))
("jarrell" nil (y aa0 r ey1 l))
("jarrells" nil (jh ae1 r ax l z))
("jarrett" nil (jh eh1 r ax t))
("jarriel" nil (zh er0 iy1 l))
("jarring" nil (jh aa1 r ih0 ng))
("jarryd" nil (jh ae1 r ih0 d))
("jars" nil (jh aa1 r z))
("jaruzelski" nil (jh eh1 r ax z eh1 l s k iy0))
("jarvi" nil (jh aa1 r v iy0))
("jarvia" nil (jh aa1 r v iy0 ax))
("jarvie" nil (jh aa1 r v iy0))
("jarvik" nil (jh aa1 r v ih0 k))
("jarvinen" nil (jh aa1 r v ih0 n ax n))
("jarvis" nil (jh aa1 r v ax s))
("jarzombek" nil (y er0 z aa1 m b eh0 k))
("jas" nil (jh ey1 m z))
("jasek" nil (y aa1 s eh0 k))
("jasinowski" nil (jh ae1 s ih0 n aw1 s k iy0))
("jasinski" nil (y ax s ih1 n s k iy0))
("jaskiewicz" nil (y aa1 s k ax v ih0 ch))
("jasko" nil (y aa1 s k ow0))
("jaskolski" nil (y ax s k ow1 l s k iy0))
("jaskot" nil (y aa1 s k ax t))
("jaskowiak" nil (y ax s k aw1 iy0 ae0 k))
("jaskulski" nil (y ax s k ah1 l s k iy0))
("jasmer" nil (jh ae1 s ax m er0))
("jasmin" nil (jh ae1 z m ih0 n))
("jasmina" nil (y ax s m ay1 n ax))
("jasmine" nil (jh ae1 z m ax n))
("jaso" nil (y aa1 s ow0))
("jason" nil (jh ey1 s ax n))
("jaspal" nil (jh ae1 s p aa0 l))
("jasper" nil (jh ae1 s p er0))
("jaspers" nil (jh ae1 s p er0 z))
("jasperson" nil (jh ae1 s p er0 s ax n))
("jass" nil (jh ae1 s))
("jasso" nil (jh ae1 s ow0))
("jaster" nil (jh ae1 s t er0))
("jauch" nil (jh ao1 ch))
("jaudon" nil (y aw0 d ao1 n))
("jaundice" nil (jh ao1 n d ax s))
("jaundiced" nil (jh ao1 n d ih0 s t))
("jaunt" nil (jh ao1 n t))
("jauntily" nil (jh ao1 n t ax l iy0))
("jaunts" nil (jh ao1 n t s))
("jaunty" nil (jh ao1 n t iy0))
("jauregui" nil (y aw0 r ey1 g w iy0))
("jaurequi" nil (y aw0 r ey1 k w iy0))
("java" nil (jh aa1 v ax))
("javanese" nil (jh aa1 v ax n iy1 z))
("javelin" nil (jh ae1 v ax l ax n))
("javett" nil (jh ae1 v ax t))
("javier" nil (hh aa1 v iy0 eh1 r))
("javits" nil (jh ae1 v ih0 t s))
("javorsky" nil (y ax v ao1 r s k iy0))
("jaw" nil (jh ao1))
("jawad" nil (jh ax w aa1 d))
("jawbone" nil (jh ao1 b ow1 n))
("jawboning" nil (jh ao1 b ow1 n ih0 ng))
("jawed" nil (jh ao1 d))
("jaworowski" nil (y ax w er0 ao1 f s k iy0))
("jaworski" nil (jh ax w ao1 r s k iy0))
("jaws" nil (jh ao1 z))
("jay" nil (jh ey1))
("jaya" nil (jh ay1 ax))
("jayashankar" nil (jh ay1 ax sh aa1 ng k aa0 r))
("jaycees" nil (jh ey1 s iy1 z))
("jaycox" nil (jh ey1 k aa1 k s))
("jaye" nil (jh ey1))
("jayewardene" nil (jh ey1 w ao0 r d iy1 n))
("jayme" nil (jh ey1 m iy0))
("jaymes" nil (jh ey1 m iy0 z))
("jayne" nil (jh ey1 n))
("jaynes" nil (jh ey1 n z))
("jayroe" nil (jh ey1 r ow1))
("jays" nil (jh ey1 z))
("jayson" nil (jh ey1 z ax n))
("jaywalking" nil (jh ey1 w ao1 k ih0 ng))
("jazmin" nil (jh ae1 z m ih0 n))
("jazz" nil (jh ae1 z))
("jazzed" nil (jh ae1 z d))
("jazzman" nil (jh ae1 z m ae1 n))
("jazzmen" nil (jh ae1 z m ax n))
("jazzy" nil (jh ae1 z iy0))
("je" nil (jh iy1))
("jealous" nil (jh eh1 l ax s))
("jealousies" nil (jh eh1 l ax s iy0 z))
("jealously" nil (jh eh1 l ax s l iy0))
("jealousy" nil (jh eh1 l ax s iy0))
("jean" nil (jh iy1 n))
("jeana" nil (jh iy1 n ax))
("jeanbaptiste" nil (zh eh1 n b ax p t ih0 s t))
("jeancourt" nil (jh iy1 n k ao1 r t))
("jeane" nil (jh iy1 n))
("jeanes" nil (jh iy1 n z))
("jeanette" nil (jh ax n eh1 t))
("jeaney" nil (jh iy1 n iy0))
("jeanie" nil (jh iy1 n iy0))
("jeanlouis" nil (zh aa1 n l uw0 iy1 z))
("jeanmichele" nil (zh aa1 n m ax sh eh1 l))
("jeanne" nil (jh iy1 n))
("jeannette" nil (jh ax n eh1 t))
("jeannie" nil (jh iy1 n iy0))
("jeanniot" nil (jh iy1 n iy0 ax t))
("jeanpierre" nil (zh ao0 n p iy0 eh1 r))
("jeans" nil (jh iy1 n z))
("jeansonne" nil (jh eh1 n s ax n))
("jeb" nil (jh eh1 b))
("jech" nil (jh eh1 k))
("jed" nil (jh eh1 d))
("jedlicka" nil (y ih0 d l ih1 s k ax))
("jee" nil (jh iy1))
("jeep" nil (jh iy1 p))
("jeepney" nil (jh iy1 p n iy0))
("jeeps" nil (jh iy1 p s))
("jeered" nil (jh ih1 r d))
("jeering" nil (jh iy1 r ih0 ng))
("jeers" nil (jh iy1 r z))
("jefcoat" nil (jh eh1 f k ow1 t))
("jeff" nil (jh eh1 f))
("jeffcoat" nil (jh eh1 f k ow0 t))
("jeffer" nil (jh eh1 f er0))
("jefferey" nil (jh eh1 f er0 iy0))
("jefferies" nil (jh eh1 f r iy0 z))
("jefferis" nil (jh eh1 f er0 ih0 s))
("jeffers" nil (jh eh1 f er0 z))
("jefferson" nil (jh eh1 f er0 s ax n))
("jeffersonian" nil (jh eh1 f er0 s ow1 n iy0 ax n))
("jeffery" nil (jh eh1 f er0 iy0))
("jefferys" nil (jh eh1 f er0 iy0 z))
("jeffords" nil (jh eh1 f er0 d z))
("jeffress" nil (jh eh1 f r ih0 s))
("jeffrey" nil (jh eh1 f r iy0))
("jeffreys" nil (jh eh1 f r iy0 z))
("jeffries" nil (jh eh1 f r iy0 z))
("jeffry" nil (jh eh1 f r iy0))
("jeffs" nil (jh eh1 f s))
("jeffus" nil (jh eh1 f ax s))
("jeffy" nil (jh eh1 f iy0))
("jehle" nil (jh eh1 hh ax l))
("jehovah" nil (jh ax hh ow1 v ax))
("jekel" nil (jh eh1 k ax l))
("jekyll" nil (jh eh1 k ax l))
("jelen" nil (jh eh1 l ax n))
("jelinek" nil (jh eh1 l ax n ih0 k))
("jelinski" nil (y ih0 l ih1 n s k iy0))
("jelks" nil (jh eh1 l k s))
("jell" nil (jh eh1 l))
("jelley" nil (jh eh1 l iy0))
("jellied" nil (jh eh1 l iy0 d))
("jellies" nil (jh eh1 l iy0 z))
("jellison" nil (jh eh1 l ih0 s ax n))
("jello" nil (jh eh1 l ow0))
("jelly" nil (jh eh1 l iy0))
("jellyfish" nil (jh eh1 l iy0 f ih1 sh))
("jem" nil (jh eh1 m))
("jemie" nil (jh eh1 m iy0))
("jemima" nil (jh eh0 m iy1 m ax))
("jemison" nil (jh eh1 m ih0 s ax n))
("jemmie" nil (jh eh1 m iy0))
("jemmott" nil (jh eh1 m ax t))
("jen" nil (jh eh1 n))
("jena" nil (jh iy1 n ax))
("jencks" nil (jh eh1 ng k s))
("jendro" nil (jh eh1 n d r ow0))
("jenison" nil (jh eh1 n ih0 s ax n))
("jenkens" nil (jh eh1 ng k ax n z))
("jenkin" nil (jh eh1 ng k ih0 n))
("jenkins" nil (jh eh1 ng k ax n z))
("jenkinson" nil (jh eh1 ng k ih0 n s ax n))
("jenks" nil (jh eh1 ng k s))
("jenn" nil (jh eh1 n))
("jenne" nil (jh eh1 n))
("jenner" nil (jh eh1 n er0))
("jenness" nil (jh eh1 n ih0 s))
("jennett" nil (jh eh1 n ih0 t))
("jennette" nil (zh ih0 n eh1 t))
("jennewein" nil (jh eh1 n uw0 ay1 n))
("jenney" nil (jh eh1 n iy0))
("jenni" nil (jh eh1 n iy0))
("jennie" nil (jh eh1 n iy0))
("jennifer" nil (jh eh1 n ax f er0))
("jenning" nil (jh eh1 n ih0 ng))
("jenninger" nil (jh eh1 n ih0 ng er0))
("jennings" nil (jh eh1 n ih0 ng z))
("jennison" nil (jh eh1 n ih0 s ax n))
("jenny" nil (jh eh1 n iy0))
("jenrette" nil (jh eh1 n r eh1 t))
("jens" nil (jh eh1 n z))
("jensen" nil (jh eh1 n s ax n))
("jenson" nil (jh eh1 n s ax n))
("jenssen" nil (jh eh1 n s ax n))
("jent" nil (jh eh1 n t))
("jentsch" nil (jh eh1 n ch))
("jentz" nil (jh eh1 n t s))
("jeong" nil (jh eh1 ng))
("jeopardize" nil (jh eh1 p er0 d ay1 z))
("jeopardized" nil (jh eh1 p er0 d ay1 z d))
("jeopardizes" nil (jh eh1 p er0 d ay1 z ih0 z))
("jeopardizing" nil (jh eh1 p er0 d ay1 z ih0 ng))
("jeopardy" nil (jh eh1 p er0 d iy0))
("jeppesen" nil (jh eh1 p iy0 z ax n))
("jeppson" nil (jh eh1 p s ax n))
("jepsen" nil (jh eh1 p s ax n))
("jepson" nil (jh eh1 p s ax n))
("jerabek" nil (jh eh1 r ax b ih0 k))
("jerald" nil (jh eh1 r ax l d))
("jerboas" nil (jh er0 b ow1 ax z))
("jerde" nil (jh er1 d))
("jerden" nil (jh er1 d ax n))
("jere" nil (jh eh1 r iy0))
("jereb" nil (jh eh1 r ih0 b))
("jereissati" nil (jh eh0 r ey1 s aa1 t iy0))
("jerel" nil (jh eh1 r ax l))
("jereld" nil (jh eh1 r ih0 l d))
("jeremia" nil (y eh0 r iy1 m iy0 ax))
("jeremiah" nil (jh eh1 r ax m ay1 ax))
("jeremy" nil (jh eh1 r ax m iy0))
("jerez" nil (y eh0 r eh1 z))
("jergens" nil (jh er1 g ax n z))
("jergenson" nil (jh er1 g ih0 n s ax n))
("jerger" nil (jh er1 g er0))
("jeri" nil (jh eh1 r iy0))
("jericho" nil (jh eh1 r ih0 k ow1))
("jerk" nil (jh er1 k))
("jerked" nil (jh er1 k t))
("jerking" nil (jh er1 k ih0 ng))
("jerkins" nil (jh er1 k ih0 n z))
("jerks" nil (jh er1 k s))
("jerky" nil (jh er1 k iy0))
("jerman" nil (jh er1 m ax n))
("jermyn" nil (jh er1 m ih0 n))
("jernberg" nil (jh er1 n b er0 g))
("jernigan" nil (jh er1 n ih0 g ae0 n))
("jeroboam" nil (jh eh1 r ax b ow1 ax m))
("jerold" nil (jh eh1 r ow0 l d))
("jerome" nil (jh er0 ow1 m))
("jerr" nil (jh er1))
("jerrell" nil (y eh0 r ey1 l))
("jerri" nil (jh eh1 r iy0))
("jerrico" nil (jh eh1 r ih0 k ow1))
("jerrie" nil (jh eh1 r iy0))
("jerritts" nil (jh eh1 r ih0 t s))
("jerrold" nil (jh eh1 r ax l d))
("jerrome" nil (y eh0 r ow1 m ey0))
("jerry" nil (jh eh1 r iy0))
("jersey" nil (jh er1 z iy0))
("jerseys" nil (jh er1 z iy0 z))
("jersusalem" nil (jh er0 uw1 s ax l ax m))
("jerusalem" nil (jh er0 uw1 s ax l ax m))
("jerusha" nil (jh eh1 r ax sh ax))
("jervis" nil (jh er1 v ih0 s))
("jerzak" nil (y er1 z ax k))
("jerzy" nil (jh er1 z iy0))
("jeschke" nil (jh eh1 sh k))
("jeske" nil (jh eh1 s k))
("jesmer" nil (jh eh1 s ax m er0))
("jesper" nil (jh eh1 s p er0))
("jespersen" nil (jh eh1 s p er0 s ax n))
("jesperson" nil (jh eh1 s p er0 s ax n))
("jess" nil (jh eh1 s))
("jessalynn" nil (jh eh1 s ax l ih0 n))
("jessamine" nil (jh eh1 s ax m ih0 n))
("jessamyn" nil (jh eh1 s ax m ih0 n))
("jesse" nil (jh eh1 s iy0))
("jessee" nil (jh eh1 s iy1))
("jesselson" nil (jh eh1 s ax l s ax n))
("jessen" nil (jh eh1 s ax n))
("jesser" nil (jh eh1 s er0))
("jessica" nil (jh eh1 s ih0 k ax))
("jessie" nil (jh eh1 s iy0))
("jesson" nil (jh eh1 s ax n))
("jessop" nil (jh eh1 s ax p))
("jessup" nil (jh eh1 s ah1 p))
("jessye" nil (jh eh1 s iy0))
("jest" nil (jh eh1 s t))
("jester" nil (jh eh1 s t er0))
("jests" nil (jh eh1 s t s))
("jesudason" nil (jh eh1 s uw1 d ax s ax n))
("jesuit" nil (jh eh1 zh uw0 ih0 t))
("jesuits" nil (jh eh1 zh uw0 ih0 t s))
("jesup" nil (jh eh1 s ax p))
("jesus" nil (jh iy1 z ax s))
("jet" nil (jh eh1 t))
("jetborne" nil (jh eh1 t b ao1 r n))
("jeter" nil (jh iy1 t er0))
("jethro" nil (jh eh1 th r ow0))
("jetliner" nil (jh eh1 t l ay1 n er0))
("jetliners" nil (jh eh1 t l ay1 n er0 z))
("jetro" nil (jh eh1 t r ow0))
("jets" nil (jh eh1 t s))
("jetstream" nil (jh eh1 t s t r iy1 m))
("jetstreams" nil (jh eh1 t s t r iy1 m z))
("jett" nil (jh eh1 t))
("jetta" nil (jh eh1 t ax))
("jette" nil (jh eh1 t))
("jetted" nil (jh eh1 t ih0 d))
("jetter" nil (jh eh1 t er0))
("jetting" nil (jh eh1 t ih0 ng))
("jettison" nil (jh eh1 t ih0 s ax n))
("jettisoned" nil (jh eh1 t ih0 s ax n d))
("jettisoning" nil (jh eh1 t ax s ax n ih0 ng))
("jetton" nil (jh eh1 t ax n))
("jetty" nil (jh eh1 t iy0))
("jeumont" nil (jh uw1 m aa1 n t))
("jew" nil (jh uw1))
("jewel" nil (jh uw1 ax l))
("jewelcor" nil (jh uw1 ax l k ao0 r))
("jeweler" nil (jh uw1 ax l er0))
("jewelers" nil (jh uw1 ax l er0 z))
("jewell" nil (jh uw1 ih0 l))
("jewellers" nil (jh uw1 ax l er0 z))
("jewelmasters" nil (jh uw1 ax l m ae1 s t er0 z))
("jewelry" nil (jh uw1 ax l r iy0))
("jewels" nil (jh uw1 ax l z))
("jewess" nil (jh uw1 ax s))
("jewett" nil (jh uw1 ih0 t))
("jewfishes" nil (jh uw1 f ih1 sh ih0 z))
("jewish" nil (jh uw1 ih0 sh))
("jewkes" nil (jh uw1 k s))
("jewry" nil (jh uw1 r iy0))
("jews" nil (jh uw1 z))
("jex" nil (jh eh1 k s))
("jeyaretnam" nil (jh ey1 er0 eh1 t n ax m))
("jezek" nil (y eh1 z eh0 k))
("jezewski" nil (y ih0 z eh1 f s k iy0))
("jezierski" nil (y ih0 z ih1 r s k iy0))
("jeziorski" nil (y ih0 z iy0 ao1 r s k iy0))
("jfet" nil (jh f eh1 t))
("jha" nil (jh aa1))
("jhirmack" nil (jh er1 m ae0 k))
("ji" nil (jh iy1))
("jia" nil (jh iy1 ax))
("jiang" nil (jh aa1 ng))
("jiangsu" nil (jh y aa1 ng s uw0))
("jiar" nil (jh iy0 aa1 r))
("jiawen" nil (jh iy1 w eh1 n))
("jibe" nil (jh ay1 b))
("jibes" nil (jh ay1 b z))
("jicha" nil (jh ih1 ch ax))
("jie" nil (jh iy1))
("jif" nil (jh ih1 f))
("jiffy" nil (jh ih1 f iy0))
("jig" nil (jh ih1 g))
("jiggetts" nil (jh ih1 g ih0 t s))
("jiggling" nil (jh ih1 g ax l ih0 ng))
("jigs" nil (jh ih1 g z))
("jigsaw" nil (jh ih1 g s ao1))
("jihad" nil (jh iy1 hh ae0 d))
("jilek" nil (jh ih1 l ih0 k))
("jiles" nil (jh ay1 l z))
("jill" nil (jh ih1 l))
("jillion" nil (jh ih1 l y ax n))
("jillions" nil (jh ih1 l y ax n z))
("jillson" nil (jh ih1 l s ax n))
("jilted" nil (jh ih1 l t ih0 d))
("jim" nil (jh ih1 m))
("jima" nil (jh iy1 m ax))
("jimenez" nil (hh ih1 m eh1 n eh1 z))
("jimerson" nil (jh ih1 m er0 s ax n))
("jimi" nil (jh ih1 m iy0))
("jiminez" nil (hh ih0 m eh1 n eh1 z))
("jimison" nil (jh ih1 m ih0 s ax n))
("jimmerson" nil (jh ih1 m er0 s ax n))
("jimmie" nil (jh ih1 m iy0))
("jimmied" nil (jh ih1 m iy0 d))
("jimmy" nil (jh ih1 m iy0))
("jin" nil (jh ih1 n))
("jindo" nil (jh ih1 n d ow0))
("jindra" nil (jh ih1 n d r ax))
("jines" nil (jh ay1 n z))
("jing" nil (jh ih1 ng))
("jingle" nil (jh ih1 ng g ax l))
("jingles" nil (jh ih1 ng g ax l z))
("jingoism" nil (jh ih1 ng g ow1 ih0 z ax m))
("jingoistic" nil (jh ih0 ng g ow0 ih1 s t ih0 k))
("jingsheng" nil (jh ih1 ng sh eh0 ng))
("jinkins" nil (jh ih1 ng k ih0 n z))
("jinks" nil (jh ih1 ng k s))
("jinright" nil (jh ih1 n r ay1 t))
("jinx" nil (jh ih1 ng k s))
("jipson" nil (jh ih1 p s ax n))
("jirak" nil (jh ao1 r ax k))
("jirga" nil (jh er1 g ax))
("jiri" nil (jh ih1 r iy0))
("jirik" nil (jh ao1 r ih0 k))
("jiron" nil (y ih0 r ao1 n))
("jirsa" nil (y ih1 r s ax))
("jitney" nil (jh ih1 t n iy0))
("jitterbugging" nil (jh ih1 t er0 b ah1 g ih0 ng))
("jitters" nil (jh ih1 t er0 z))
("jittery" nil (jh ih1 t er0 iy0))
("jive" nil (jh ay1 v))
("jividen" nil (y iy0 v ay1 d ax n))
("jiving" nil (jh ay1 v ih0 ng))
("jo" nil (jh ow1))
("joachim" nil (y ow1 aa1 k ih0 m))
("joachims" nil (y ow1 aa1 k ih0 m z))
("joakima" nil (jh ow1 k iy1 m ax))
("joan" nil (jh ow1 n))
("joann" nil (jh ow1 ae1 n))
("joanna" nil (jh ow0 ae1 n ax))
("joanne" nil (jh ow1 ae1 n))
("joanou" nil (jh ow1 n uw1))
("joao" nil (jh ow1 ow0))
("joaquim" nil (w aa1 k iy1 m))
("joaquin" nil (w aa1 k iy1 n))
("job" nil (jh aa1 b))
("jobbers" nil (jh aa1 b er0 z))
("jobe" nil (jh ow1 b))
("jobes" nil (jh ow1 b z))
("jobete" nil (jh ow0 b iy1 t))
("jobin" nil (jh ow1 b ih0 n))
("jobina" nil (y ow0 b iy1 n ax))
("jobless" nil (jh aa1 b l ax s))
("joblessness" nil (jh aa1 b l ax s n ax s))
("jobs" nil (jh aa1 b z))
("jobson" nil (jh aa1 b s ax n))
("jobst" nil (jh aa1 b s t))
("jobyna" nil (y ax b iy1 n ax))
("jocelin" nil (jh ao1 s l ih1 n))
("joceline" nil (jh ao1 s l ih1 n))
("jocelyn" nil (jh ao1 s l ih1 n))
("jocelyne" nil (jh ao1 s l ih1 n))
("jochem" nil (jh aa1 k ih0 m))
("jochen" nil (jh ow1 k ax n))
("jochim" nil (jh aa1 k ih0 m))
("jochum" nil (jh aa1 k ax m))
("jock" nil (jh aa1 k))
("jockey" nil (jh aa1 k iy0))
("jockeying" nil (jh aa1 k iy0 ih0 ng))
("jockeys" nil (jh aa1 k iy0 z))
("jocks" nil (jh aa1 k s))
("jocosa" nil (y ow0 k ow1 s ax))
("jocular" nil (jh aa1 k y ax l er0))
("jocylan" nil (jh ao1 s l ax n))
("jodi" nil (jh ow1 d iy0))
("jodie" nil (jh ow1 d iy0))
("jodoin" nil (zh ax d oy1 n))
("jody" nil (jh ow1 d iy0))
("joe" nil (jh ow1))
("joedicke" nil (jh ow1 d ih0 k))
("joel" nil (jh ow1 ax l))
("joelson" nil (jh ow1 l s ax n))
("joens" nil (jh ow1 n z))
("joerg" nil (jh ao1 r g))
("joerger" nil (y er1 g er0))
("joers" nil (jh ow1 er0 z))
("joes" nil (jh ow1 z))
("joette" nil (zh ow0 eh1 t))
("joey" nil (jh ow1 iy0))
("joffe" nil (jh aa1 f))
("joffrey" nil (jh ao1 f r iy0))
("joffrion" nil (y ow0 f r iy0 ao1 n))
("jog" nil (jh aa1 g))
("jogged" nil (jh aa1 g d))
("jogger" nil (jh aa1 g er0))
("joggers" nil (jh aa1 g er0 z))
("jogging" nil (jh aa1 g ih0 ng))
("jogs" nil (jh aa1 g z))
("joh" nil (jh ow1))
("johan" nil (y ow0 hh aa1 n))
("johann" nil (y ow1 hh aa0 n))
("johanna" nil (jh ax hh ae1 n ax))
("johannes" nil (jh ow0 hh ae1 n ax s))
("johannesburg" nil (jh ow1 hh ae1 n ih0 s b er0 g))
("johannesen" nil (jh ow1 hh ae1 n ih0 s ax n))
("johannessen" nil (jh ow1 hh ae1 n ih0 s ax n))
("johanning" nil (y ow1 hh ae1 n ih0 ng))
("johanns" nil (y ow1 hh aa0 n z))
("johannsen" nil (jh ow1 hh ae1 n s ax n))
("johansen" nil (jh ow1 hh ae1 n s ax n))
("johanson" nil (jh ow1 hh ae1 n s ax n))
("johansson" nil (y ow1 hh aa1 n s ax n))
("john" nil (jh aa1 n))
("johnathon" nil (jh aa1 n ax th ax n))
("johndrow" nil (jh aa1 n d r ow0))
("johnnie" nil (jh aa1 n iy0))
("johnny" nil (jh aa1 n iy0))
("johns" nil (jh aa1 n z))
("johnsen" nil (jh aa1 n s ax n))
("johnsey" nil (jh aa1 n z iy0))
("johnson" nil (jh aa1 n s ax n))
("johnsons" nil (jh aa1 n s ax n z))
("johnsrud" nil (jh aa1 n s r ax d))
("johnston" nil (jh aa1 n s t ax n))
("johnstone" nil (jh aa1 n s t ow1 n))
("johnstown" nil (jh aa1 n z t aw1 n))
("johson" nil (jh aa1 s ax n))
("joice" nil (jh oy1 s))
("joie" nil (zh w aa1))
("join" nil (jh oy1 n))
("joined" nil (jh oy1 n d))
("joiner" nil (jh oy1 n er0))
("joinery" nil (jh oy1 n er0 iy0))
("joines" nil (jh oy1 n z))
("joining" nil (jh oy1 n ih0 ng))
("joins" nil (jh oy1 n z))
("joint" nil (jh oy1 n t))
("jointed" nil (jh oy1 n t ax d))
("jointly" nil (jh oy1 n t l iy0))
("jointness" nil (jh oy1 n t n ax s))
("joints" nil (jh oy1 n t s))
("joist" nil (jh oy1 s t))
("joists" nil (jh oy1 s t s))
("jojola" nil (y ow0 y ow1 l ax))
("joke" nil (jh ow1 k))
("joked" nil (jh ow1 k t))
("jokela" nil (jh aa1 k ih0 l ax))
("joker" nil (jh ow1 k er0))
("jokers" nil (jh ow1 k er0 z))
("jokerst" nil (jh aa1 k er0 s t))
("jokes" nil (jh ow1 k s))
("jokhang" nil (jh aa1 k hh ae0 ng))
("joki" nil (jh ow1 k iy0))
("jokinen" nil (jh ax k ih1 n ax n))
("joking" nil (jh ow1 k ih0 ng))
("jokingly" nil (jh ow1 k ih0 ng l iy0))
("joles" nil (jh ow1 l z))
("joletta" nil (jh ow0 l eh1 t ax))
("jolette" nil (jh ow1 l eh1 t))
("jolicoeur" nil (zh aa1 l ih0 k er0))
("joliet" nil (jh ow1 l iy0 eh1 t))
("jolin" nil (jh ow1 l ih0 n))
("jolivette" nil (zh aa1 l ih0 v eh0 t))
("jolla" nil (hh oy1 y ax))
("jolley" nil (jh aa1 l iy0))
("jollie" nil (jh aa1 l iy0))
("jollies" nil (jh aa1 l iy0 z))
("jolliff" nil (jh aa1 l ih0 f))
("jolliffe" nil (jh aa1 l ih0 f))
("jolly" nil (jh aa1 l iy0))
("jollying" nil (jh aa1 l iy0 ih0 ng))
("jolson" nil (jh ow1 l s ax n))
("jolt" nil (jh ow1 l t))
("jolted" nil (jh ow1 l t ih0 d))
("jolting" nil (jh ow1 l t ih0 ng))
("jolts" nil (jh ow1 l t s))
("joly" nil (jh ow1 l iy0))
("jon" nil (jh aa1 n))
("jonah" nil (jh ow1 n ax))
("jonas" nil (jh ow1 n ax s))
("jonassaint" nil (jh ow1 n ax s ey1 n t))
("jonassen" nil (jh aa1 n ax s ax n))
("jonathan" nil (jh aa1 n ax th ax n))
("jonathon" nil (jh aa1 n ax th ax n))
("joncas" nil (jh aa1 ng k ax z))
("jone" nil (jh ow1 n))
("jones" nil (jh ow1 n z))
("joneses" nil (jh ow1 n z ih0 z))
("jong" nil (jh ao1 ng))
("joni" nil (jh ow1 n iy0))
("jonker" nil (jh aa1 ng k er0))
("jonn" nil (jh aa1 n))
("jonothan" nil (jh aa1 n ax th ax n))
("jonson" nil (jh aa1 n s ax n))
("jonsson" nil (jh aa1 n s ax n))
("joo" nil (jh uw1))
("joong" nil (jh uh1 ng))
("joos" nil (jh uw1 z))
("joost" nil (jh uw1 s t))
("joosten" nil (y uw1 s ax n))
("joplin" nil (jh aa1 p l ih0 n))
("jopling" nil (jh ow1 p ax l ih0 ng))
("jopp" nil (jh aa1 p))
("jordache" nil (jh ao1 r d ae1 sh))
("jordahl" nil (jh ao1 r d aa0 l))
("jordan" nil (jh ao1 r d ax n))
("jordana" nil (y ao0 r d ae1 n ax))
("jordanian" nil (jh ao1 r d ey1 n iy0 ax n))
("jordanians" nil (jh ao0 r d ey1 n iy0 ax n z))
("jorde" nil (jh ao1 r d))
("jorden" nil (jh ao1 r d ax n))
("jorge" nil (hh ao1 r hh ey0))
("jorgensen" nil (jh ao1 r g ax n s ax n))
("jorgenson" nil (jh ao1 r jh ih0 n s ax n))
("jorio" nil (jh ao1 r iy0 ow0))
("jornal" nil (jh ao1 r n ax l))
("jorstad" nil (jh ao1 r s t ax d))
("jory" nil (jh ao1 r iy0))
("jos" nil (jh ow1 z))
("josceline" nil (jh aa1 s ih0 l ay1 n))
("joscelyn" nil (jh aa1 s ih0 l ih0 n))
("joscelyne" nil (jh aa1 s ih0 l ay1 n))
("jose" nil (hh ow1 z ey1))
("josef" nil (jh ow1 s ax f))
("joseph" nil (jh ow1 s ax f))
("josepha" nil (jh ow0 s eh1 f ax))
("josephina" nil (jh aa0 s ih0 f ay1 n ax))
("josephine" nil (jh ow1 s ax f iy1 n))
("josephs" nil (jh ow1 s ih0 f s))
("josephson" nil (jh ow1 s ax f s ax n))
("josephthal" nil (jh ow1 s ax f th ao1 l))
("josephus" nil (jh ow0 s iy1 f ax s))
("joser" nil (jh ow1 z er0))
("josette" nil (zh ax s eh1 t))
("josey" nil (jh ow1 z iy0))
("josh" nil (jh aa1 sh))
("joshi" nil (jh ow1 sh iy0))
("joshua" nil (jh aa1 sh uw0 ax))
("josiah" nil (jh ow0 s ay1 ax))
("josie" nil (jh ow1 z iy0))
("josleyn" nil (jh aa1 s l ih1 n))
("joslin" nil (jh aa1 s l ih0 n))
("joslyn" nil (jh ao1 s l ih0 n))
("jospin" nil (jh aa1 s p ih0 n))
("joss" nil (jh ao1 s))
("josselyn" nil (jh aa1 s ih0 l ih0 n))
("jost" nil (jh aa1 s t))
("josten" nil (jh aa1 s ax n))
("jostens" nil (jh ao1 s t ax n z))
("jostes" nil (jh ow1 s t s))
("jostle" nil (jh aa1 s ax l))
("jostled" nil (jh aa1 s ax l d))
("jostling" nil (jh aa1 s ax l ih0 ng))
("joswick" nil (jh aa1 s w ih0 k))
("jot" nil (jh aa1 t))
("jotted" nil (jh aa1 t ih0 d))
("joubert" nil (y aw1 b er0 t))
("jouett" nil (jh aa1 uw0 t))
("joule" nil (jh uw1 l))
("joules" nil (jh uw1 l z))
("jour" nil (jh aw1 r))
("jourdan" nil (zh uh0 r d ae1 n))
("jouret" nil (zh w aa0 r ey1))
("journal" nil (jh er1 n ax l))
("journalese" nil (jh er1 n ax l iy1 z))
("journalism" nil (jh er1 n ax l ih1 z ax m))
("journalist" nil (jh er1 n ax l ax s t))
("journalistic" nil (jh er1 n ax l ih1 s t ih0 k))
("journalists" nil (jh er1 n ax l ax s t s))
("journals" nil (jh er1 n ax l z))
("journey" nil (jh er1 n iy0))
("journeyed" nil (jh er1 n iy0 d))
("journeying" nil (jh er1 n iy0 ih0 ng))
("journeyman" nil (jh er1 n iy0 m ae1 n))
("journeys" nil (jh er1 n iy0 z))
("jousting" nil (jh aw1 s t ih0 ng))
("jovan" nil (jh ow1 v ax n))
("jovanovic" nil (y ax v ax n aa1 v ih0 k))
("jovanovich" nil (jh ow0 v ae1 n ax v ih0 ch))
("jovi" nil (jh ow1 v iy0))
("jovial" nil (jh ow1 v iy0 ax l))
("jovian" nil (jh ow1 v iy0 ax n))
("jovita" nil (y ax v ay1 t ax))
("jovito" nil (jh ow0 v iy1 t ow0))
("jowers" nil (jh aw1 er0 z))
("jowett" nil (jh aw1 ih0 t))
("jowl" nil (jh aw1 l))
("jowley" nil (jh ow1 l iy0))
("jowls" nil (jh aw1 l z))
("jowly" nil (jh aw1 l iy0))
("joy" nil (jh oy1))
("joyal" nil (jh oy1 ax l))
("joyce" nil (jh oy1 s))
("joycelyn" nil (jh oy1 s l ih1 n))
("joye" nil (jh oy1))
("joyful" nil (jh oy1 f ax l))
("joyfully" nil (jh oy1 f ax l iy0))
("joyner" nil (jh oy1 n er0))
("joynes" nil (jh oy1 n z))
("joynt" nil (jh oy1 n t))
("joyoni" nil (jh oy0 ow1 n iy0))
("joyous" nil (jh oy1 ax s))
("joyride" nil (jh oy1 r ay1 d))
("joys" nil (jh oy1 z))
("joystick" nil (jh oy1 s t ih1 k))
("jozef" nil (jh ow1 z ax f))
("jozwiak" nil (y aa1 z v iy0 ae0 k))
("jr" nil (jh uw1 n y er0))
("ju" nil (jh uw1))
("jua" nil (jh uw1 ax))
("juan" nil (w aa1 n))
("juana" nil (w aa1 n ax))
("juanita" nil (w aa0 n iy1 t ax))
("juarez" nil (w aa1 r eh1 z))
("juba" nil (jh uw1 b ax))
("jubb" nil (jh ah1 b))
("jubilant" nil (jh uw1 b ax l ax n t))
("jubilation" nil (jh uw1 b ax l ey1 sh ax n))
("jubilee" nil (jh uw1 b ax l iy1))
("juckett" nil (jh ah1 k ih0 t))
("judah" nil (jh uw1 d ax))
("judaism" nil (jh uw1 d ey0 ih1 z ax m))
("judas" nil (jh uw1 d ax s))
("juday" nil (jh uw1 d ey1))
("judd" nil (jh ah1 d))
("judds" nil (jh ah1 d z))
("jude" nil (jh uw1 d))
("judea" nil (jh uw0 d iy1 ax))
("judeo" nil (jh uw0 d ey1 ow0))
("judge" nil (jh ah1 jh))
("judged" nil (jh ah1 jh d))
("judgement" nil (jh ah1 jh m ax n t))
("judgements" nil (jh ah1 jh m ax n t s))
("judges" nil (jh ah1 jh ih0 z))
("judgeship" nil (jh ah1 jh sh ih1 p))
("judgeships" nil (jh ah1 jh sh ih1 p s))
("judging" nil (jh ah1 jh ih0 ng))
("judgment" nil (jh ah1 jh m ax n t))
("judgmental" nil (jh ax jh m eh1 n t ax l))
("judgments" nil (jh ah1 jh m ax n t s))
("judicate" nil (jh uw1 d ih0 k ey1 t))
("judice" nil (jh uw1 d ih0 s))
("judicial" nil (jh uw0 d ih1 sh ax l))
("judicially" nil (jh uw0 d ih1 sh ax l iy0))
("judiciary" nil (jh uw0 d ih1 sh iy0 eh1 r iy0))
("judicious" nil (jh uw0 d ih1 sh ax s))
("judiciously" nil (jh uw0 d ih1 sh ih0 s l iy0))
("judie" nil (jh ah1 d iy0))
("judishe" nil (jh uw1 d iy1 sh))
("judith" nil (jh uw1 d ax th))
("judkins" nil (jh ah1 d k ih0 n z))
("judo" nil (jh uw1 d ow1))
("judson" nil (jh ah1 d s ax n))
("judy" nil (jh uw1 d iy0))
("jue" nil (jh uw1))
("juedes" nil (jh w eh1 d eh0 s))
("juel" nil (jh uw1 l))
("juenemann" nil (y uw1 n m ax n))
("juenger" nil (y uw1 ng g er0))
("juergen" nil (y er1 g ax n))
("juergens" nil (y er1 g ax n z))
("jueteng" nil (jh uw1 t eh1 ng))
("juett" nil (jh uw1 t))
("jug" nil (jh ah1 g))
("juge" nil (jh uw1 jh))
("juggernaut" nil (jh ah1 g er0 n ao1 t))
("juggle" nil (jh ah1 g ax l))
("juggled" nil (jh ah1 g ax l d))
("juggler" nil (jh ah1 g ax l er0))
("jugglers" nil (jh ah1 g ax l er0 z))
("juggles" nil (jh ah1 g ax l z))
("juggling" nil (jh ah1 g ax l ih0 ng))
("jugs" nil (jh ah1 g z))
("jugular" nil (jh uw1 g y ax l er0))
("juhas" nil (jh uw1 ax z))
("juhasz" nil (y ah1 hh ax sh))
("juhl" nil (jh ah1 l))
("juhlin" nil (jh uw1 l ih0 n))
("juhnke" nil (jh ah1 ng k))
("juice" nil (jh uw1 s))
("juices" nil (jh uw1 s ax z))
("juicier" nil (jh uw0 ih1 s y er0))
("juiciest" nil (jh uw1 s iy0 ax s t))
("juicy" nil (jh uw1 s iy0))
("juilliard" nil (jh uw1 l iy0 aa0 r d))
("jujitsu" nil (jh uw0 jh ih1 t s uw1))
("jujo" nil (jh uw1 jh ow0))
("jukebox" nil (jh uw1 k b aa1 k s))
("jukes" nil (jh uw1 k s))
("juki" nil (jh uw1 k iy0))
("jule" nil (jh uw1 l))
("jules" nil (jh uw1 l z))
("juli" nil (jh uw1 l iy0))
("julia" nil (jh uw1 l y ax))
("julian" nil (jh uw1 l iy0 ax n))
("juliana" nil (jh uw1 l iy0 ae1 n ax))
("julianne" nil (jh uw1 l iy0 eh1 n))
("juliano" nil (jh uw1 l iy0 aa1 n ow0))
("julie" nil (jh uw1 l iy0))
("julien" nil (jh uw1 l iy0 ax n))
("juliet" nil (jh uw1 l iy0 eh1 t))
("julieta" nil (y uw0 l iy1 t ax))
("julietta" nil (jh uw0 l iy0 eh1 t ax))
("juliette" nil (jh uw0 l iy0 eh1 t))
("julin" nil (jh uw1 l ih0 n))
("julina" nil (y uw0 l iy1 n ax))
("juline" nil (jh uw1 l ay0 n))
("julio" nil (jh uw1 l iy0 ow0))
("julius" nil (jh uw1 l y ax s))
("juliusz" nil (jh uw1 l iy0 uw0 s))
("julson" nil (jh ah1 l s ax n))
("july" nil (jh uw1 l ay1))
("jumble" nil (jh ah1 m b ax l))
("jumbled" nil (jh ah1 m b ax l d))
("jumbo" nil (jh ah1 m b ow0))
("jumbos" nil (jh ah1 m b ow1 z))
("jumonville" nil (zh ah1 m ax n v ih0 l))
("jump" nil (jh ah1 m p))
("jumped" nil (jh ah1 m p t))
("jumper" nil (jh ah1 m p er0))
("jumpers" nil (jh ah1 m p er0 z))
("jumping" nil (jh ah1 m p ih0 ng))
("jumps" nil (jh ah1 m p s))
("jumpstart" nil (jh ah1 m p s t aa1 r t))
("jumpsuit" nil (jh ah1 m p s uw1 t))
("jumpy" nil (jh ah1 m p iy0))
("jun" nil (jh ah1 n))
("junco" nil (jh ah1 ng k ow0))
("junction" nil (jh ah1 ng k sh ax n))
("juncture" nil (jh ah1 ng k ch er0))
("junctures" nil (jh ah1 ng k ch er0 z))
("junda" nil (jh ah1 n d ax))
("jundt" nil (jh ah1 n t))
("june" nil (jh uw1 n))
("juneau" nil (jh uw1 n ow0))
("junejo" nil (jh uw0 n ey1 hh ow0))
("junek" nil (jh uw1 n ih0 k))
("junella" nil (jh uw1 n eh1 l ax))
("junes" nil (jh uw1 n z))
("junette" nil (jh uw1 n eh1 t))
("jung" nil (y uh1 ng))
("jungbluth" nil (jh ah1 ng b l uw0 th))
("junge" nil (jh ah1 n g))
("jungels" nil (jh ah1 ng g ax l z))
("junger" nil (jh ah1 ng er0))
("jungers" nil (jh ah1 ng er0 z))
("junghans" nil (jh ah1 ng g ax n z))
("jungle" nil (jh ah1 ng g ax l))
("jungles" nil (jh ah1 ng g ax l z))
("jungman" nil (jh ah1 ng m ax n))
("jungwirth" nil (jh ah1 ng g w er0 th))
("junia" nil (y uw1 n iy0 ax))
("juniata" nil (y uw0 n iy0 aa1 t ax))
("junine" nil (jh ah1 n ih0 n))
("junior" nil (jh uw1 n y er0))
("juniors" nil (jh uw1 n y er0 z))
("juniper" nil (jh uw1 n ax p er0))
("junipers" nil (jh uw1 n ih0 p er0 z))
("junius" nil (jh uw1 n iy0 ih0 s))
("junji" nil (jh ah1 n jh iy0))
("junk" nil (jh ah1 ng k))
("junkbond" nil (jh ah1 ng k b aa1 n d))
("junked" nil (jh ah1 ng k t))
("junker" nil (jh ah1 ng k er0))
("junket" nil (jh ah1 ng k ih0 t))
("junkets" nil (jh ah1 ng k ih0 t s))
("junkholders" nil (jh ah1 ng k hh ow1 l d er0 z))
("junkie" nil (jh ah1 ng k iy0))
("junkier" nil (jh ah1 ng k iy0 er0))
("junkies" nil (jh ah1 ng k iy0 z))
("junkiest" nil (jh ah1 ng k iy0 ax s t))
("junkin" nil (jh ah1 ng k ih0 n))
("junking" nil (jh ah1 ng k ih0 ng))
("junkins" nil (jh ah1 ng k ih1 n z))
("junky" nil (jh ah1 ng k iy0))
("junkyard" nil (jh ah1 ng k y aa1 r d))
("junkyards" nil (jh ah1 ng k y aa1 r d z))
("juno" nil (jh uw1 n ow0))
("junod" nil (jh uw1 n ax d))
("junot" nil (jh uw1 n ax t))
("junta" nil (hh uh1 n t ax))
("juntunen" nil (jh ah1 n t ax n ax n))
("jupin" nil (jh uw1 p ih0 n))
("jupiter" nil (jh uw1 p ax t er0))
("juppe" nil (jh uw1 p iy0))
("jurado" nil (y uh0 r aa1 d ow0))
("juran" nil (y uh0 r aa1 n))
("juranek" nil (jh uh1 r ax n ih0 k))
("juras" nil (jh uh1 r ax z))
("jurassic" nil (jh uh0 r ae1 s ih0 k))
("jurczak" nil (y er1 ch ae0 k))
("jurczyk" nil (y er1 ch ih0 k))
("jure" nil (jh uh1 r))
("jurek" nil (jh uh1 r eh0 k))
("jurewicz" nil (jh uh1 r ax v ih0 ch))
("jurgen" nil (jh er1 g ax n))
("jurgens" nil (jh er1 g ax n z))
("jurgensen" nil (jh er1 g ih0 n s ax n))
("jurgensmeyer" nil (jh er1 g ax n z m ay1 r))
("jurgenson" nil (jh er1 g ih0 n s ax n))
("jurica" nil (jh uh1 r ih0 k ax))
("jurich" nil (jh uh1 r ih0 k))
("juries" nil (jh uh1 r iy0 z))
("juris" nil (jh uh1 r ih0 s))
("jurisdiction" nil (jh uh1 r ax s d ih1 k sh ax n))
("jurisdictional" nil (jh uh1 r ax s d ih1 k sh ax n ax l))
("jurisdictions" nil (jh uh1 r ih0 s d ih1 k sh ax n z))
("jurisprudence" nil (jh uh1 r ax s p r uw1 d ax n s))
("jurisprudential" nil (jh uh1 r ax s p r uw1 d eh1 n sh ax l))
("jurist" nil (jh uh1 r ax s t))
("jurists" nil (jh uh1 r ih0 s t s))
("jurkiewicz" nil (y er1 k ax v ih0 ch))
("jurkovich" nil (y er1 k ax v ih0 ch))
("jurkowski" nil (y er0 k ao1 f s k iy0))
("jurney" nil (jh er1 n iy0))
("juror" nil (jh uh1 r er0))
("jurors" nil (jh uh1 r er0 z))
("jurs" nil (jh er1 z))
("jury" nil (jh uh1 r iy0))
("jusco" nil (jh ah1 s k ow0))
("jusino" nil (y uw0 s iy1 n ow0))
("just" nil (jh ah1 s t))
("justa" nil (jh ah1 s t ax))
("justen" nil (jh ah1 s t ax n))
("juster" nil (jh ah1 s t er0))
("justesen" nil (jh ah1 s t iy0 z ax n))
("justice" nil (jh ah1 s t ax s))
("justices" nil (jh ah1 s t ax s ax z))
("justifiable" nil (jh ah1 s t ax f ay1 ax b ax l))
("justifiably" nil (jh ah1 s t ax f ay1 ax b l iy0))
("justification" nil (jh ah1 s t ax f ax k ey1 sh ax n))
("justifications" nil (jh ah1 s t ih0 f ih0 k ey1 sh ax n z))
("justified" nil (jh ah1 s t ax f ay1 d))
("justifies" nil (jh ah1 s t ax f ay1 z))
("justify" nil (jh ah1 s t ax f ay1))
("justifying" nil (jh ah1 s t ax f ay1 ih0 ng))
("justin" nil (jh ah1 s t ax n))
("justina" nil (y uw0 s t iy1 n ax))
("justine" nil (jh ax s t iy1 n))
("justiniano" nil (jh uw0 s t iy0 n iy0 aa1 n ow0))
("justino" nil (jh ax s t iy1 n ow0))
("justis" nil (y uw1 s t ih0 s))
("justiss" nil (y uw1 s t iy0 s))
("justly" nil (jh ah1 s t l iy0))
("justman" nil (jh ah1 s t m ax n))
("justo" nil (jh ah1 s t ow0))
("justus" nil (jh ah1 s t ax s))
("justy" nil (jh ah1 s t iy0))
("jute" nil (jh uw1 t))
("jutland" nil (jh ah1 t l ax n d))
("jutras" nil (y uw1 t r aa0 z))
("jutting" nil (jh ah1 t ih0 ng))
("juul" nil (jh uw1 ax l))
("juve" nil (jh uw1 v))
("juvenile" nil (jh uw1 v ax n ax l))
("juveniles" nil (jh uw1 v ax n ax l z))
("juxtaposed" nil (jh ah1 k s t ax p ow1 z d))
("juxtaposition" nil (jh ah1 k s t ax p ax z ih1 sh ax n))
("juxtapositions" nil (jh ah1 k s t ax p ax z ih1 sh ax n z))
("jyishane" nil (jh iy1 sh ey1 n))
("jynx" nil (jh ih1 ng k s))
("k" nil (k ey1))
("ka" nil (k aa1))
("kaas" nil (k aa1 z))
("kaatz" nil (k aa1 t s))
("kabat" nil (k ae1 b ax t))
("kabbalah" nil (k ax b aa1 l ax))
("kabbani" nil (k ax b aa1 n iy0))
("kabel" nil (k ae1 b ax l))
("kabi" nil (k ae1 b iy0))
("kabivitrum" nil (k ax b ih1 v ih0 t r ax m))
("kable" nil (k ey1 b ax l))
("kabler" nil (k ey1 b ax l er0))
("kabral" nil (k ax b r aa1 l))
("kabuki" nil (k ax b uw1 k iy1))
("kabul" nil (k aa1 b uh0 l))
("kacer" nil (k ey1 s er0))
("kach" nil (k ae1 ch))
("kachel" nil (k ae1 k ax l))
("kachmar" nil (k ae1 k m er0))
("kachur" nil (k ae1 ch er0))
("kackley" nil (k ae1 k l iy0))
("kaczmarczyk" nil (k aa1 ch m er0 ch ih0 k))
("kaczmarek" nil (k ax ch m aa1 r eh0 k))
("kaczmarski" nil (k ax ch m aa1 r s k iy0))
("kaczor" nil (k aa1 ch er0))
("kaczorowski" nil (k ax ch er0 ao1 f s k iy0))
("kaczynski" nil (k ax ch ih1 n s k iy0))
("kadar" nil (k ae1 d er0))
("kade" nil (k ey1 d))
("kadel" nil (k ae1 d ax l))
("kaden" nil (k ey1 d ax n))
("kader" nil (k ey1 d er0))
("kading" nil (k ey1 d ih0 ng))
("kadish" nil (k ey1 d ih0 sh))
("kadlec" nil (k aa1 d l ih0 k))
("kadow" nil (k aa1 d ow0))
("kadrmas" nil (k ae1 d er0 m aa1 z))
("kady" nil (k ey1 d iy0))
("kaeding" nil (k eh1 d ih0 ng))
("kaehler" nil (k eh1 l er0))
("kaelin" nil (k eh1 l ih0 n))
("kaercher" nil (k eh1 r k er0))
("kaeser" nil (k ey1 z er0))
("kaestner" nil (k eh1 s t n er0))
("kaetzel" nil (k eh1 t z ax l))
("kafer" nil (k ey1 f er0))
("kaffenberger" nil (k ae1 f ax n b er0 g er0))
("kafka" nil (k aa1 f k ax))
("kafkaesque" nil (k aa1 f k ax eh1 s k))
("kagami" nil (k ae1 g ax m iy0))
("kagan" nil (k ey1 g ax n))
("kagarlitsky" nil (k ae1 g er0 l ih1 t s k iy1))
("kagawa" nil (k ax g aa1 w ax))
("kage" nil (k ey1 jh))
("kagel" nil (k ey1 g ax l))
("kagey" nil (k ey1 jh iy0))
("kageyama" nil (k aa1 g iy0 aa1 m ax))
("kagin" nil (k ey1 g ih0 n))
("kagins" nil (k ey1 g ih0 n z))
("kagler" nil (k ae1 g l er0))
("kagy" nil (k ey1 g iy0))
("kahan" nil (k ax hh aa1 n))
("kahane" nil (k ax hh aa1 n ey1))
("kahl" nil (k aa1 l))
("kahle" nil (k aa1 l))
("kahler" nil (k aa1 l er0))
("kahley" nil (k aa1 l iy0))
("kahn" nil (k aa1 n))
("kahng" nil (k aa1 ng))
("kahr" nil (k aa1 r))
("kahre" nil (k eh1 r))
("kahrs" nil (k aa1 r z))
("kai" nil (k ay1))
("kaifu" nil (k ay1 f uw1))
("kaigler" nil (k ey1 g l er0))
("kail" nil (k ey1 l))
("kailash" nil (k ey1 l ax sh))
("kaim" nil (k ey1 m))
("kain" nil (k ey1 n))
("kaine" nil (k ey1 n))
("kainer" nil (k ey1 n er0))
("kainz" nil (k ey1 n z))
("kairamo" nil (k eh1 r aa1 m ow0))
("kairey" nil (k eh1 r iy0))
("kaiser" nil (k ay1 z er0))
("kaiseraugst" nil (k ay1 z er0 ao0 g s t))
("kaisertech" nil (k ay1 z er0 t eh1 k))
("kaisha" nil (k ey1 sh ax))
("kaji" nil (k aa1 jh iy0))
("kajima" nil (k aa1 jh iy1 m ax))
("kakadu" nil (k aa1 k aa1 d uw0))
("kakimoto" nil (k aa1 k ih0 m ow1 t ow0))
("kakos" nil (k ey1 k ow0 z))
("kaktovik" nil (k ae1 k t ow1 v ih0 k))
("kakuei" nil (k ae1 k y uw0 iy0))
("kakumaru" nil (k aa1 k uw0 m aa1 r uw0))
("kalafut" nil (k ae1 l ax f ax t))
("kalal" nil (k ey1 l ax l))
("kalamazoo" nil (k ae1 l ax m ax z uw1))
("kalan" nil (k ey1 l ax n))
("kalas" nil (k aa1 l ax z))
("kalashnikov" nil (k ax l ae1 sh n ih0 k aa1 v))
("kalata" nil (k ax l aa1 t ax))
("kalb" nil (k ae1 l b))
("kalbach" nil (k ae1 l b aa1 k))
("kalberer" nil (k ae1 l b er0 er0))
("kalbfleisch" nil (k ae1 l b f l ay1 sh))
("kale" nil (k ey1 l))
("kaleel" nil (k ae1 l iy0 l))
("kaleida" nil (k ax l ay1 d ax))
("kaleidoscope" nil (k ax l ay1 d ax s k ow1 p))
("kalen" nil (k ey1 l ax n))
("kaler" nil (k ey1 l er0))
("kaleta" nil (k ae1 l ih0 t ax))
("kaley" nil (k ey1 l iy0))
("kalgoorlie" nil (k ae1 l g uw1 r l iy0))
("kalikow" nil (k ae1 l ih0 k ow0))
("kalil" nil (k ae1 l ax l))
("kalin" nil (k ae1 l ih0 n))
("kalina" nil (k ax l ay1 n ax))
("kalinoski" nil (k ax l ih0 n aw1 s k iy0))
("kalinowski" nil (k ax l ih0 n ao1 f s k iy0))
("kalinske" nil (k ax l ih1 n s k iy0))
("kalinski" nil (k ax l ih1 n s k iy0))
("kalis" nil (k ae1 l ih0 s))
("kalish" nil (k ae1 l ih0 sh))
("kalisz" nil (k aa1 l ih0 sh))
("kaliszewski" nil (k ax l ih0 sh eh1 f s k iy0))
("kalivoda" nil (k ax l ih0 v ow1 d ax))
("kalk" nil (k ao1 k))
("kalka" nil (k ae1 l k ax))
("kalkbrenner" nil (k ae1 l k b r ih0 n er0))
("kall" nil (k ao1 l))
("kalla" nil (k ae1 l ax))
("kallal" nil (k ae1 l ax l))
("kallam" nil (k ae1 l ax m))
("kallas" nil (k ae1 l ax z))
("kallay" nil (k ae1 l ey0))
("kallen" nil (k ao1 l ax n))
("kallenbach" nil (k ae1 l ih0 n b aa0 k))
("kallenberger" nil (k ao1 l ax n b er0 g er0))
("kallhoff" nil (k ae1 l hh ao0 f))
("kalliel" nil (k ae1 l iy0 ax l))
("kallio" nil (k ae1 l iy0 ow0))
("kallis" nil (k ae1 l ih0 s))
("kallman" nil (k ao1 l m ax n))
("kallmeyer" nil (k ae1 l m ay0 er0))
("kallstrom" nil (k ae1 l s t r ax m))
("kallus" nil (k ae1 l ax s))
("kalman" nil (k ae1 l m ax n))
("kalmanovitz" nil (k ae1 l m ae1 n ax v ih0 t s))
("kalmar" nil (k ae1 l m er0))
("kalmbach" nil (k ae1 l m b aa0 k))
("kalmus" nil (k ae1 l m ih0 s))
("kalnins" nil (k ae1 l n ih0 n z))
("kalo" nil (k ey1 l ow0))
("kalok" nil (k ae1 l aa1 k))
("kalous" nil (k ae1 l ax s))
("kalp" nil (k ae1 l p))
("kalt" nil (k ao1 l t))
("kaltenbach" nil (k ae1 l t ih0 n b aa0 k))
("kaltenbacher" nil (k aa1 l t ax n b aa1 k er0))
("kalter" nil (k ao1 l t er0))
("kalthoff" nil (k ae1 l th))
("kaluza" nil (k ax l uw1 z ax))
("kaluzny" nil (k ax l ah1 z n iy0))
("kalvar" nil (k ae1 l v aa0 r))
("kam" nil (k ae1 m))
("kama" nil (k aa1 m ax))
("kamakau" nil (k aa1 m ax k aa1 uw0))
("kamal" nil (k ey1 m ax l))
("kamali" nil (k ax m aa1 l iy0))
("kaman" nil (k ey1 m ax n))
("kamber" nil (k ae1 m b er0))
("kamchatka" nil (k ae1 m ch ae1 t k ax))
("kamehameha" nil (k ax m ey1 ax m ey1 ax))
("kamei" nil (k ae1 m iy0))
("kamel" nil (k aa1 m ax l))
("kamen" nil (k aa1 m eh0 n))
("kamenar" nil (k ae1 m ax n er0))
("kamens" nil (k aa1 m eh0 n z))
("kamentsev" nil (k ax m eh1 n t s aa1 v))
("kamer" nil (k ey1 m er0))
("kamerer" nil (k ae1 m er0 er0))
("kamikaze" nil (k aa1 m ax k aa1 z iy0))
("kamin" nil (k aa1 m iy0 n))
("kaminer" nil (k ae1 m ih0 n er0))
("kamins" nil (k ae1 m ih0 n z))
("kaminski" nil (k ax m ih1 n s k iy0))
("kaminsky" nil (k ax m ih1 n s k iy0))
("kamke" nil (k ae1 m k iy0))
("kamler" nil (k ae1 m ax l er0))
("kamm" nil (k ae1 m))
("kamman" nil (k ae1 m ax n))
("kammer" nil (k ae1 m er0))
("kammerer" nil (k ae1 m er0 er0))
("kammerzell" nil (k ae1 m er0 z ax l))
("kammeyer" nil (k ae1 m ay0 er0))
("kamp" nil (k ae1 m p))
("kampa" nil (k ae1 m p ax))
("kampe" nil (k ae1 m p))
("kampelman" nil (k ae1 m p ax l m ax n))
("kampen" nil (k ae1 m p ax n))
("kamper" nil (k ae1 m p er0))
("kampf" nil (k ae1 m p f))
("kampfer" nil (k ae1 m p f er0))
("kamphaus" nil (k ae1 m p hh aw1 s))
("kampman" nil (k ae1 m p m ax n))
("kampmann" nil (k ae1 m p m ax n))
("kamps" nil (k ae1 m p s))
("kampschulte" nil (k ae1 m p sh uh1 l t iy0))
("kamradt" nil (k ae1 m r ax t))
("kamrath" nil (k ae1 m r ax th))
("kamstra" nil (k ae1 m s t r ax))
("kan" nil (k ae1 n))
("kana" nil (k ae1 n ax))
("kanade" nil (k ax n aa1 d ey1))
("kanady" nil (k ae1 n ax d iy0))
("kanagy" nil (k ae1 n ax jh iy0))
("kanak" nil (k ae1 n ax k))
("kanan" nil (k ey1 n ax n))
("kanarek" nil (k ae1 n er0 ih0 k))
("kanawa" nil (k aa1 n aa1 w ax))
("kanawha" nil (k ax n ao1 hh ax))
("kanazawa" nil (k aa1 n aa0 z aa1 w ax))
("kanda" nil (k ae1 n d ax))
("kandahar" nil (k ae1 n d ax hh aa0 r))
("kandel" nil (k ae1 n d ax l))
("kandler" nil (k ae1 n d l er0))
("kandt" nil (k ae1 n t))
("kane" nil (k ey1 n))
("kaneb" nil (k ae1 n ax b))
("kanegsberg" nil (k ae1 n ax g z b er0 g))
("kaneko" nil (k aa0 n ey1 k ow0))
("kanemaru" nil (k ae1 n eh0 m aa1 r uw0))
("kaner" nil (k ey1 n er0))
("kaneshiro" nil (k aa0 n ey0 sh ih1 r ow0))
("kaney" nil (k ey1 n iy0))
("kang" nil (k ae1 ng))
("kangaroo" nil (k ae1 ng g er0 uw1))
("kangaroos" nil (k ae1 ng g er0 uw1 z))
("kangas" nil (k ae1 ng g ax z))
("kanghua" nil (k ae1 ng hh y uw0 ax))
("kangyo" nil (k ae1 n jh y ow0))
("kania" nil (k aa0 n iy1 ax))
("kaniewski" nil (k aa0 n iy0 eh1 f s k iy0))
("kanipe" nil (k ae1 n ih0 p))
("kanis" nil (k ae1 n ih0 s))
("kanitz" nil (k ae1 n ih0 t s))
("kanji" nil (k ae1 n jh iy0))
("kankakee" nil (k ae1 ng k ih0 k iy0))
("kankaku" nil (k aa1 n k aa1 k uw0))
("kann" nil (k ae1 n))
("kannan" nil (k aa1 n ax n))
("kanne" nil (k ae1 n))
("kannenberg" nil (k ae1 n ax n b er0 g))
("kanner" nil (k ae1 n er0))
("kanno" nil (k ae1 n ow0))
("kano" nil (k aa1 n ow0))
("kanode" nil (k ax n ow1 d))
("kanon" nil (k ey1 n ax n))
("kanouse" nil (k aa0 n ow0 uw1 s ey0))
("kansai" nil (k ae0 n s ay1))
("kansallis" nil (k ae1 n s ae1 l ih0 s))
("kansan" nil (k ae1 n z ax n))
("kansas" nil (k ae1 n z ax s))
("kant" nil (k ae1 n t))
("kanter" nil (k ae1 n t er0))
("kantian" nil (k ae1 n t iy0 ax n))
("kantner" nil (k ae1 n t n er0))
("kantola" nil (k ax n t ow1 l ax))
("kantor" nil (k ae1 n t er0))
("kantrowitz" nil (k ae1 n t r ax w ih0 t s))
("kantz" nil (k ae1 n t s))
("kanz" nil (k ae1 n z))
("kanzler" nil (k ae1 n z l er0))
("kao" nil (k aw1))
("kaohsiung" nil (k ey1 ow1 s iy0 ax ng))
("kaolin" nil (k aw1 l ih0 n))
("kaori" nil (k ao1 r iy0))
("kapaun" nil (k ae1 p aw0 n))
("kapfer" nil (k ae1 p f er0))
("kapinos" nil (k ae1 p ih0 n ow0 z))
("kapitan" nil (k ae1 p ih0 t ax n))
("kaplain" nil (k ae1 p l ax n))
("kaplan" nil (k ae1 p l ax n))
("kapler" nil (k ey1 p ax l er0))
("kaplin" nil (k ae1 p l ih0 n))
("kaplow" nil (k ae1 p l ow0))
("kaplowitz" nil (k aa1 p l ax w ih0 t s))
("kapner" nil (k ae1 p n er0))
("kapnick" nil (k ae1 p n ih0 k))
("kapoor" nil (k ax p uw1 r))
("kapor" nil (k ey1 p er0))
("kapp" nil (k ae1 p))
("kappa" nil (k ae1 p ax))
("kappel" nil (k ae1 p ax l))
("kappeler" nil (k ae1 p ax l er0))
("kappelman" nil (k ae1 p ax l m ax n))
("kapper" nil (k ae1 p er0))
("kappes" nil (k ae1 p s))
("kappler" nil (k ae1 p l er0))
("kappus" nil (k ae1 p ax s))
("kapral" nil (k ae1 p r ax l))
("kaps" nil (k ae1 p s))
("kapton" nil (k ae1 p t ax n))
("kaptur" nil (k ae1 p t er0))
("kapur" nil (k ae1 p er0))
("kapuscinski" nil (k ax p ax s ch ih1 n s k iy0))
("kapusta" nil (k ax p ah1 s t ax))
("kara" nil (k eh1 r ax))
("karabakh" nil (k eh1 r ax b ae0 k))
("karabin" nil (k aa0 r aa1 b iy0 n))
("karachi" nil (k aa0 r aa1 ch iy0))
("karadzic" nil (k aa0 r aa1 jh ih0 k))
("karaffa" nil (k ae1 r ax f ax))
("karajan" nil (k eh1 r ax jh ax n))
("karam" nil (k ae1 r ax m))
("karami" nil (k er0 aa1 m iy0))
("karan" nil (k eh1 r ax n))
("karaoke" nil (k eh1 r iy0 ow1 k iy0))
("karas" nil (k aa1 r ax z))
("karasawa" nil (k aa1 r ax s aa1 w ax))
("karasek" nil (k er0 aa1 s eh0 k))
("karasik" nil (k er0 aa1 s ih0 k))
("karat" nil (k eh1 r ax t))
("karate" nil (k er0 aa1 t iy0))
("karatsu" nil (k er0 ae1 t s uw1))
("karatz" nil (k eh1 r ax t s))
("karban" nil (k aa1 r b ax n))
("karbassioun" nil (k aa1 r b ae1 s iy0 uw1 n))
("karber" nil (k aa1 r b er0))
("karbowski" nil (k er0 b ao1 f s k iy0))
("karch" nil (k aa1 r k))
("karcher" nil (k aa1 r ch er0))
("karcz" nil (k aa1 r ch))
("karczewski" nil (k er0 ch eh1 f s k iy0))
("kardashian" nil (k aa1 d ax sh ey1 n))
("kardell" nil (k aa1 r d ax l))
("kardos" nil (k aa1 r d ow0 z))
("kareem" nil (k er0 iy1 m))
("karel" nil (k eh1 r ax l))
("karelian" nil (k er0 iy1 l iy0 ax n))
("karels" nil (k eh1 r ax l z))
("karen" nil (k eh1 r ax n))
("karenina" nil (k eh1 r eh0 n iy1 n ax))
("karet" nil (k eh1 r ax t))
("karg" nil (k aa1 r g))
("karger" nil (k aa1 r g er0))
("karges" nil (k aa1 r jh ih0 z))
("kari" nil (k eh1 r iy0))
("karibu" nil (k eh1 r iy1 b uw0))
("karim" nil (k er0 iy1 m))
("karimi" nil (k aa0 r iy1 m iy0))
("karin" nil (k eh1 r ih0 n))
("karino" nil (k eh1 r iy1 n ow0))
("kariotis" nil (k aa1 r iy0 ow1 t ih0 s))
("karis" nil (k eh1 r iy0 z))
("karl" nil (k aa1 r l))
("karla" nil (k aa1 r l ax))
("karle" nil (k aa1 r ax l))
("karlen" nil (k aa1 r l ax n))
("karlheinz" nil (k aa1 r l hh ay1 n z))
("karlik" nil (k aa1 r l ih0 k))
("karlin" nil (k aa1 r l ih0 n))
("karline" nil (k aa1 r l ay1 n))
("karls" nil (k aa1 r l z))
("karlsen" nil (k aa1 r l s ax n))
("karlson" nil (k aa1 r l s ax n))
("karlsruhe" nil (k aa1 r l z r uw1))
("karlsson" nil (k aa1 r l s ax n))
("karma" nil (k aa1 r m ax))
("karman" nil (k aa1 r m ax n))
("karn" nil (k aa1 r n))
("karner" nil (k aa1 r n er0))
("karnes" nil (k aa1 r n z))
("karney" nil (k aa1 r n iy0))
("karns" nil (k aa1 r n z))
("karnsund" nil (k aa1 r n s ax n d))
("karol" nil (k eh1 r ao0 l))
("karolina" nil (k eh1 r ax l ay1 n ax))
("karoline" nil (k eh1 r ax l ay1 n))
("karolinska" nil (k eh1 r ow0 l ih1 n s k ax))
("karoly" nil (k er0 ow1 l iy0))
("karolyi" nil (k er0 ow1 l y iy0))
("karolyn" nil (k ae1 r ax l ih0 n))
("karon" nil (k eh1 r ax n))
("karos" nil (k aa1 r ow0 s))
("karoun" nil (k er0 uw1 n))
("karow" nil (k aa1 r ow0))
("karp" nil (k aa1 r p))
("karpatkin" nil (k aa1 r p ae1 t k ih0 n))
("karpel" nil (k aa1 r p ax l))
("karpen" nil (k aa1 r p ax n))
("karpf" nil (k aa1 r p f))
("karpinski" nil (k er0 p ih1 n s k iy0))
("karpov" nil (k aa1 r p aa1 v))
("karpowicz" nil (k aa1 r p ax v ih0 ch))
("karr" nil (k aa1 r))
("karraker" nil (k aa1 r ax k er0))
("karras" nil (k ae1 r ax z))
("karren" nil (k ae1 r ax n))
("karrer" nil (k aa1 r er0))
("karrick" nil (k ae1 r ih0 k))
("karriker" nil (k ae1 r ih0 k er0))
("karry" nil (k eh1 r iy0))
("karsh" nil (k aa1 r sh))
("karshner" nil (k aa1 r sh n er0))
("karson" nil (k aa1 r s ax n))
("karst" nil (k aa1 r s t))
("karstadt" nil (k aa1 r s t ae1 t))
("karsten" nil (k aa1 r s ax n))
("karstens" nil (k aa1 r s ax n z))
("karstetter" nil (k aa1 r s t ih0 t er0))
("kartasasmita" nil (k aa0 r t ae1 s ax s m iy1 t ax))
("kartchner" nil (k aa1 r ch n er0))
("karter" nil (k aa1 r t er0))
("kartes" nil (k aa1 r t s))
("karun" nil (k er0 uw1 n))
("karvonen" nil (k aa1 r v ax n ax n))
("karwoski" nil (k er0 v aw1 s k iy0))
("karwowski" nil (k er0 v ao1 f s k iy0))
("kary" nil (k eh1 r iy0))
("karydakis" nil (k eh1 r iy0 d ae1 k ih0 s))
("karyn" nil (k ae1 r ih0 n))
("kasal" nil (k ey1 s ax l))
("kasch" nil (k ae1 sh))
("kaschak" nil (k ae1 sh ax k))
("kasdorf" nil (k ae1 s d ao0 r f))
("kase" nil (k ey1 z))
("kasel" nil (k ae1 s ax l))
("kasell" nil (k ae1 s ax l))
("kaseman" nil (k ey1 s m ax n))
("kaser" nil (k ey1 z er0))
("kasey" nil (k ae1 s iy0))
("kash" nil (k ae1 sh))
("kashiwagi" nil (k ae1 sh ih0 w ae1 g iy0))
("kashiwahara" nil (k aa1 sh iy0 w ax hh aa1 r ax))
("kashmir" nil (k ae1 sh m ih0 r))
("kashner" nil (k ae1 sh n er0))
("kashoggi" nil (k ax sh aa1 jh iy0))
("kashuba" nil (k ax sh uw1 b ax))
("kasich" nil (k aa1 s ih0 ch))
("kasik" nil (k aa1 s ih0 k))
("kasinger" nil (k ae1 s ih0 n jh er0))
("kaska" nil (k aa1 s k ax))
("kaske" nil (k ae1 s k iy0))
("kaskel" nil (k ae1 s k ax l))
("kasler" nil (k ae1 s ax l er0))
("kasmer" nil (k ae1 s ax m er0))
("kasmira" nil (k ax s m ay1 r ax))
("kasner" nil (k ae1 s n er0))
("kaspar" nil (k ae1 s p er0))
("kasparek" nil (k ae1 s p er0 ih0 k))
("kasparian" nil (k ax s p eh1 r iy0 ax n))
("kasparov" nil (k ae1 s p er0 aa0 v))
("kasper" nil (k ae1 s p er0))
("kasperek" nil (k ae1 s p er0 ih0 k))
("kasperski" nil (k ax s p er1 s k iy0))
("kasprowicz" nil (k aa1 s p r ax v ih0 ch))
("kasprzak" nil (k aa1 s p er0 z ax k))
("kasprzyk" nil (k aa1 s p er0 z ih0 k))
("kasputys" nil (k ae1 s p y uw1 t ih0 s))
("kasriel" nil (k ae1 s r iy0 ax l))
("kass" nil (k ae1 s))
("kassa" nil (k ae1 s ax))
("kassab" nil (k ae1 s ax b))
("kassan" nil (k ae1 s ax n))
("kassar" nil (k ae1 s er0))
("kassebaum" nil (k ae1 s ax b aw1 m))
("kassel" nil (k ae1 s ax l))
("kassem" nil (k ae1 s ax m))
("kassen" nil (k ae1 s ax n))
("kassin" nil (k ae1 s ih0 n))
("kassing" nil (k ae1 s ih0 ng))
("kassis" nil (k ae1 s ih0 s))
("kassler" nil (k ae1 s l er0))
("kassner" nil (k ae1 s n er0))
("kasson" nil (k ae1 s ax n))
("kast" nil (k ae1 s t))
("kastel" nil (k ey1 s t ax l))
("kastelic" nil (k ax s t eh1 l ih0 k))
("kasten" nil (k ae1 s t ax n))
("kastenmeier" nil (k ae1 s t ax n m ay1 r))
("kastens" nil (k ey1 s ax n z))
("kaster" nil (k ae1 s t er0))
("kastl" nil (k ae1 s t ax l))
("kastle" nil (k ae1 s ax l))
("kastler" nil (k ae1 s t l er0))
("kastner" nil (k ae1 s t n er0))
("kasun" nil (k aa1 s uw0 n))
("kasza" nil (k aa1 sh ax))
("kaszuba" nil (k ax sh uw1 b ax))
("kat" nil (k ae1 t))
("kataoka" nil (k aa0 t aa0 ow1 k ax))
("katarina" nil (k aa1 t er0 iy1 n ax))
("katashiba" nil (k ae1 t ax sh iy1 b ax))
("katayama" nil (k aa0 t aa0 y aa1 m ax))
("katcher" nil (k ae1 ch er0))
("kate" nil (k ey1 t))
("katen" nil (k ey1 t ax n))
("kater" nil (k ey1 t er0))
("kates" nil (k ey1 t s))
("kath" nil (k ae1 th))
("kathan" nil (k ae1 th ax n))
("katharina" nil (k ae1 th eh0 r iy1 n ax))
("katharine" nil (k ae1 th r ih0 n))
("katherine" nil (k ae1 th er0 ih0 n))
("kathi" nil (k ae1 th iy0))
("kathie" nil (k ae1 th iy0))
("kathleen" nil (k ae0 th l iy1 n))
("kathlene" nil (k ae1 th l iy1 n))
("kathline" nil (k ae1 th l ay1 n))
("kathman" nil (k ae1 th m ax n))
("kathryn" nil (k ae1 th r ih0 n))
("kathy" nil (k ae1 th iy0))
("katia" nil (k aa1 t iy0 ax))
("katie" nil (k ae1 t iy0))
("katmandu" nil (k ae1 t m ae0 n d uw1))
("kato" nil (k ey1 t ow0))
("katona" nil (k aa0 t ow1 n ax))
("katragadda" nil (k aa1 t ax g aa1 d ax))
("kats" nil (k ae1 t s))
("katsanos" nil (k ae1 t s aa1 n ow0 s))
("katsaros" nil (k ae1 t s er0 ow0 z))
("katsumi" nil (k aa1 t s uw1 m iy0))
("katsushi" nil (k ae1 t s uw1 sh iy0))
("katt" nil (k ae1 t))
("kattner" nil (k ae1 t n er0))
("katy" nil (k ey1 t iy0))
("katyn" nil (k ey1 t ih1 n))
("katz" nil (k ae1 t s))
("katzen" nil (k ae1 t z ax n))
("katzenbach" nil (k ae1 t s ax n b aa1 k))
("katzenberg" nil (k ae1 t s ax n b er0 g))
("katzenberger" nil (k ae1 t z ax n b er0 g er0))
("katzenstein" nil (k ae1 t s ax n s t ay0 n))
("katzer" nil (k ae1 t s er0))
("katzin" nil (k ae1 t s ih0 n))
("katzman" nil (k ae1 t s m ax n))
("kau" nil (k aw1))
("kauai" nil (k aw1 ay1))
("kauble" nil (k ao1 b ax l))
("kaucher" nil (k aw1 k er0))
("kauer" nil (k aw1 er0))
("kaufer" nil (k ao1 f er0))
("kauffman" nil (k ao1 f m ax n))
("kauffmann" nil (k ao1 f m ax n))
("kaufhof" nil (k ao1 f hh ao0 f))
("kaufhold" nil (k aw1 f hh ow0 l d))
("kaufman" nil (k ao1 f m ax n))
("kaufmann" nil (k ao1 f m ax n))
("kauk" nil (k ao1 k))
("kaul" nil (k ao1 l))
("kaunda" nil (k ao1 n d ax))
("kaup" nil (k ao1 p))
("kaupp" nil (k ao1 p))
("kauppi" nil (k ao1 p iy0))
("kaus" nil (k ao1 z))
("kausch" nil (k aw1 sh))
("kauth" nil (k ao1 th))
("kautz" nil (k ao1 t s))
("kautzman" nil (k aw1 t s m ax n))
("kauzlarich" nil (k aw1 z l er0 ih0 k))
("kavan" nil (k ey1 v ax n))
("kavanagh" nil (k ae1 v ax n ao1))
("kavanaugh" nil (k ae1 v ax n ao1))
("kaveney" nil (k ae1 v ih0 n iy0))
("kavner" nil (k ae1 v n er0))
("kawa" nil (k aa1 w ax))
("kawaguchi" nil (k aa1 w aa0 g uw1 ch iy0))
("kawahara" nil (k aa1 w aa0 hh aa1 r ax))
("kawai" nil (k aa0 w aa1 iy0))
("kawakami" nil (k aa1 w aa0 k aa1 m iy0))
("kawamoto" nil (k aa0 w aa0 m ow1 t ow0))
("kawamura" nil (k aw1 aa0 m uh1 r ax))
("kawano" nil (k aa0 w aa1 n ow0))
("kawasaki" nil (k aa1 w aa0 s aa1 k iy0))
("kawashima" nil (k aa1 w aa0 sh iy1 m ax))
("kawate" nil (k aa1 w aa1 t ey1))
("kawecki" nil (k aa0 v eh1 t s k iy0))
("kaweske" nil (k aa0 v eh1 s k iy0))
("kay" nil (k ey1))
("kaya" nil (k aa1 y ax))
("kayak" nil (k ay1 ae0 k))
("kayaking" nil (k ay1 ae1 k ih0 ng))
("kayaks" nil (k ay1 ae0 k s))
("kaydon" nil (k ey1 d ax n))
("kaye" nil (k ey1))
("kayes" nil (k ey1 z))
("kaylie" nil (k ey1 l iy0))
("kaylor" nil (k ey1 l er0))
("kayne" nil (k ey1 n))
("kayo" nil (k ey0 ow1))
("kaypro" nil (k ey1 p r ow0))
("kays" nil (k ey1 z))
("kayser" nil (k ey1 z er0))
("kaysersberg" nil (k ay1 z er0 z b er0 g))
("kazakh" nil (k ae1 z ae0 k))
("kazakhs" nil (k ae1 z ae0 k s))
("kazakhstan" nil (k aa1 z aa0 k s t aa1 n))
("kazan" nil (k ey1 z ax n))
("kazanjian" nil (k ax z ae1 n jh iy0 ax n))
("kazarian" nil (k ax z eh1 r iy0 ax n))
("kazda" nil (k aa1 z d ax))
("kazee" nil (k aa1 z iy0))
("kazempour" nil (k ax z eh1 m p aw0 r))
("kazen" nil (k ae1 z ax n))
("kazikaev" nil (k ae1 z ih0 k ey1 v))
("kazin" nil (k ey1 z ih0 n))
("kazis" nil (k ae1 z ih0 s))
("kazlauskas" nil (k ae1 z l aw0 s k ax z))
("kazmer" nil (k ae1 z m er0))
("kazmierczak" nil (k aa1 z m ih0 r ch ae0 k))
("kazmierski" nil (k ax z m ih1 r s k iy0))
("kazoo" nil (k ax z uw1))
("kazuhiko" nil (k aa1 z uw0 hh iy1 k ow0))
("kazuo" nil (k aa1 z uw1 ow0))
("kea" nil (k iy1))
("keach" nil (k iy1 ch))
("keadle" nil (k iy1 d ax l))
("keady" nil (k iy1 d iy0))
("keagle" nil (k iy1 g ax l))
("keagy" nil (k iy1 jh iy0))
("keahey" nil (k iy1 hh iy0))
("keal" nil (k iy1 l))
("kealey" nil (k iy1 l iy0))
("kealy" nil (k iy1 l iy0))
("kean" nil (k iy1 n))
("keane" nil (k iy1 n))
("keaney" nil (k iy1 n iy0))
("kear" nil (k ih1 r))
("kearby" nil (k er1 b iy0))
("kearfott" nil (k ih1 r f aa0 t))
("kearley" nil (k er1 l iy0))
("kearn" nil (k er1 n))
("kearney" nil (k er1 n iy0))
("kearns" nil (k er1 n z))
("kearny" nil (k er1 n iy0))
("kearse" nil (k er1 s))
("keary" nil (k ih1 r iy0))
("keas" nil (k iy1 z))
("keasler" nil (k iy1 z l er0))
("keasling" nil (k iy1 z l ih0 ng))
("keast" nil (k iy1 s t))
("keath" nil (k iy1 th))
("keathley" nil (k iy1 th l iy0))
("keating" nil (k iy1 t ih0 ng))
("keatley" nil (k iy1 t l iy0))
("keaton" nil (k iy1 t ax n))
("keator" nil (k iy1 t er0))
("keats" nil (k iy1 t s))
("keatts" nil (k iy1 t s))
("keaveney" nil (k iy1 v ih0 n iy0))
("keaveny" nil (k iy1 v ih0 n iy0))
("keay" nil (k iy1 iy0))
("keck" nil (k eh1 k))
("keckler" nil (k eh1 k l er0))
("kedar" nil (k iy1 d er0))
("keddy" nil (k eh1 d iy0))
("kedrowski" nil (k ih0 d r ao1 f s k iy0))
("keds" nil (k eh1 d z))
("kedzierski" nil (k ih0 jh ih1 r s k iy0))
("kee" nil (k iy1))
("keeble" nil (k iy1 b ax l))
("keebler" nil (k iy1 b l er0))
("keech" nil (k iy1 ch))
("keedy" nil (k iy1 d iy0))
("keef" nil (k iy1 f))
("keefe" nil (k iy1 f))
("keefer" nil (k iy1 f er0))
("keeffe" nil (k iy1 f))
("keegali" nil (k iy1 g aa1 l iy0))
("keegan" nil (k iy1 g ax n))
("keehan" nil (k iy1 ax n))
("keehn" nil (k iy1 n))
("keehner" nil (k iy1 n er0))
("keel" nil (k iy1 l))
("keelan" nil (k iy1 l ax n))
("keele" nil (k iy1 l))
("keeler" nil (k iy1 l er0))
("keeley" nil (k iy1 l iy0))
("keelin" nil (k iy1 l ih0 n))
("keeling" nil (k iy1 l ih0 ng))
("keels" nil (k iy1 l z))
("keelson" nil (k eh1 l s ax n))
("keely" nil (k iy1 l iy0))
("keen" nil (k iy1 n))
("keena" nil (k iy1 n ax))
("keenan" nil (k iy1 n ax n))
("keene" nil (k iy1 n))
("keeneland" nil (k iy1 n ax l ax n d))
("keener" nil (k iy1 n er0))
("keenest" nil (k iy1 n ax s t))
("keeney" nil (k iy1 n iy0))
("keenly" nil (k iy1 n l iy0))
("keenum" nil (k iy1 n ax m))
("keeny" nil (k iy1 n iy0))
("keep" nil (k iy1 p))
("keeper" nil (k iy1 p er0))
("keepers" nil (k iy1 p er0 z))
("keeping" nil (k iy1 p ih0 ng))
("keeps" nil (k iy1 p s))
("keepsakes" nil (k iy1 p s ey1 k s))
("keeran" nil (k ih1 r ax n))
("kees" nil (k iy1 z))
("keese" nil (k iy1 z))
("keesee" nil (k iy0 s iy1))
("keesey" nil (k iy1 s iy0))
("keesler" nil (k iy1 z l er0))
("keesling" nil (k iy1 z l ih0 ng))
("keetch" nil (k iy1 ch))
("keeter" nil (k iy1 t er0))
("keeth" nil (k iy1 th))
("keeton" nil (k iy1 t ax n))
("keever" nil (k iy1 v er0))
("keevil" nil (k iy1 v ax l))
("keezer" nil (k iy1 z er0))
("kefauver" nil (k eh1 f aw0 v er0))
("keffer" nil (k eh1 f er0))
("keg" nil (k eh1 g))
("kegel" nil (k eh1 jh ax l))
("kegg" nil (k eh1 g))
("kegler" nil (k eh1 g l er0))
("kegley" nil (k eh1 g l iy0))
("kegs" nil (k eh1 g z))
("kehl" nil (k eh1 l))
("kehler" nil (k eh1 l er0))
("kehm" nil (k eh1 m))
("kehn" nil (k eh1 n))
("kehne" nil (k eh1 n))
("kehoe" nil (k eh1 hh ow0))
("kehr" nil (k eh1 r))
("kehrer" nil (k eh1 r er0))
("kehres" nil (k eh1 r z))
("keicher" nil (k ay1 k er0))
("keidanren" nil (k ay1 d ae1 n r ax n))
("keidel" nil (k ay1 d ax l))
("keifer" nil (k iy1 f er0))
("keiffer" nil (k iy1 f er0))
("keigley" nil (k iy1 g l iy0))
("keiichi" nil (k ey1 iy1 ch iy0))
("keiji" nil (k iy1 jh iy0))
("keiko" nil (k ey1 k ow0))
("keil" nil (k iy1 l))
("keillor" nil (k iy1 l er0))
("keilman" nil (k ay1 l m ax n))
("keim" nil (k iy1 m))
("keimig" nil (k iy1 m ih0 g))
("keinath" nil (k ay1 n ax th))
("keiner" nil (k iy1 n er0))
("keio" nil (k ey1 ow0))
("keiper" nil (k iy1 p er0))
("keir" nil (k iy1 r))
("keiretsu" nil (k ih1 r eh1 t s uw0))
("keirn" nil (k ih1 r n))
("keirns" nil (k ay1 r n z))
("keirsey" nil (k ih1 r s iy0))
("keisei" nil (k ay1 s ey1))
("keiser" nil (k ay1 s er0))
("keisler" nil (k ay1 s ax l er0))
("keisling" nil (k ay1 s ax l ih0 ng))
("keister" nil (k iy1 ih0 s t er0))
("keisuke" nil (k ey1 s uw1 k iy0))
("keitel" nil (k ay1 t ax l))
("keiter" nil (k iy1 t er0))
("keith" nil (k iy1 th))
("keithley" nil (k iy1 th l iy0))
("keithly" nil (k iy1 th l iy0))
("keitt" nil (k iy1 t))
("keitz" nil (k iy1 t s))
("keizai" nil (k iy1 z ey0))
("keizer" nil (k ay1 z er0))
("kekst" nil (k eh1 k s t))
("kelbaugh" nil (k eh1 l b ao1))
("kelber" nil (k eh1 l b er0))
("kelby" nil (k eh1 l b iy0))
("kelcey" nil (k eh1 l s iy0))
("kelch" nil (k eh1 l ch))
("kelchner" nil (k eh1 l k n er0))
("kelda" nil (k eh1 l d ax))
("kelder" nil (k eh1 l d er0))
("keleher" nil (k eh1 l ih0 hh er0))
("keleman" nil (k iy1 l m ax n))
("kelemen" nil (k iy1 l m eh0 n))
("kelker" nil (k eh1 l k er0))
("kell" nil (k eh1 l))
("kellam" nil (k eh1 l ax m))
("kellams" nil (k eh1 l ax m z))
("kellar" nil (k eh1 l er0))
("kelleher" nil (k eh1 l ih0 hh er0))
("kellems" nil (k eh1 l ih0 m z))
("kellen" nil (k eh1 l ax n))
("kellenberger" nil (k eh1 l ax n b er0 g er0))
("kellenyi" nil (k eh1 l eh1 n y iy0))
("keller" nil (k eh1 l er0))
("kellerman" nil (k eh1 l er0 m ax n))
("kellermann" nil (k eh1 l er0 m ax n))
("kellett" nil (k eh1 l ih0 t))
("kelley" nil (k eh1 l iy0))
("kelli" nil (k eh1 l iy0))
("kelliher" nil (k eh1 l ih0 hh er0))
("kelling" nil (k eh1 l ih0 ng))
("kellis" nil (k eh1 l ih0 s))
("kellison" nil (k eh1 l ih0 s ax n))
("kellman" nil (k eh1 l m ax n))
("kellner" nil (k eh1 l n er0))
("kellogg" nil (k eh1 l ao0 g))
("kellough" nil (k eh1 l aw0))
("kellow" nil (k eh1 l ow0))
("kells" nil (k eh1 l z))
("kellum" nil (k eh1 l ax m))
("kellwood" nil (k eh1 l w uh1 d))
("kelly" nil (k eh1 l iy0))
("kelm" nil (k eh1 l m))
("kelman" nil (k eh1 l m ax n))
("kelner" nil (k eh1 l n er0))
("kelnhofer" nil (k eh1 l n hh ax f er0))
("kelp" nil (k eh1 l p))
("kelps" nil (k eh1 l p s))
("kelsall" nil (k eh1 l s ax l))
("kelsay" nil (k eh1 l s ey0))
("kelsch" nil (k eh1 l sh))
("kelsey" nil (k eh1 l s iy0))
("kelso" nil (k eh1 l s ow0))
("kelsoe" nil (k eh1 l s ow0))
("kelson" nil (k eh1 l s ax n))
("kelter" nil (k eh1 l t er0))
("kelting" nil (k eh1 l t ih0 ng))
("keltner" nil (k eh1 l t n er0))
("kelton" nil (k eh1 l t ax n))
("kelty" nil (k eh1 l t iy0))
("keltz" nil (k eh1 l t s))
("kelvan" nil (k eh1 l v ax n))
("kelven" nil (k eh1 l v ax n))
("kelvin" nil (k eh1 l v ax n))
("kem" nil (k eh1 m))
("kemal" nil (k ax m aa1 l))
("kembel" nil (k eh1 m b ax l))
("kemble" nil (k eh1 m b ax l))
("kemeny" nil (k eh1 m ih0 n iy0))
("kemerer" nil (k eh1 m er0 er0))
("kemery" nil (k eh1 m er0 iy0))
("kemler" nil (k eh1 m l er0))
("kemmer" nil (k eh1 m er0))
("kemmerer" nil (k eh1 m er0 er0))
("kemmerling" nil (k eh1 m er0 l ih0 ng))
("kemmons" nil (k eh1 m ax n z))
("kemner" nil (k eh1 m n er0))
("kemnitz" nil (k eh1 m n ih0 t s))
("kemp" nil (k eh1 m p))
("kempa" nil (k eh1 m p ax))
("kempe" nil (k eh1 m p))
("kempel" nil (k eh1 m p ax l))
("kempen" nil (k eh1 m p ax n))
("kemper" nil (k eh1 m p er0))
("kempf" nil (k eh1 m p f))
("kempfer" nil (k eh1 m p f er0))
("kempke" nil (k eh1 m p k))
("kempker" nil (k eh1 m p k er0))
("kemple" nil (k eh1 m p ax l))
("kempler" nil (k eh1 m p l er0))
("kemplin" nil (k eh1 m p l ih0 n))
("kempner" nil (k eh1 m p n er0))
("kemppainen" nil (k eh1 m p ay0 n ax n))
("kempski" nil (k eh1 m p s k iy0))
("kempson" nil (k eh1 m p s ax n))
("kempster" nil (k eh1 m p s t er0))
("kempton" nil (k eh1 m p t ax n))
("ken" nil (k eh1 n))
("kenaf" nil (k eh1 n ae0 f))
("kenagy" nil (k eh1 n ax jh iy0))
("kenan" nil (k iy1 n ax n))
("kendal" nil (k eh1 n d ax l))
("kendall" nil (k eh1 n d ax l))
("kendavis" nil (k eh1 n d ey1 v ih0 s))
("kendell" nil (k eh1 n d ax l))
("kender" nil (k eh1 n d er0))
("kendig" nil (k eh1 n d ih0 g))
("kendle" nil (k eh1 n d ax l))
("kendra" nil (k eh1 n d r ax))
("kendrick" nil (k eh1 n d r ih0 k))
("kendricks" nil (k eh1 n d r ih0 k s))
("kendzierski" nil (k ih0 n jh ih1 r s k iy0))
("kendzior" nil (k ih0 n jh iy1 er0))
("kenealy" nil (k eh1 n iy0 ax l iy0))
("kenefick" nil (k eh1 n ih0 f ih0 k))
("kenerson" nil (k eh1 n er0 s ax n))
("kenetech" nil (k eh1 n ih0 t eh0 k))
("kenfield" nil (k eh1 n f iy1 l d))
("kenichi" nil (k eh1 n iy1 ch iy0))
("kenilworth" nil (k eh1 n ax l w er1 th))
("kenison" nil (k eh1 n ih0 s ax n))
("keniston" nil (k eh1 n ih0 s t aa0 n))
("kenji" nil (k eh1 n jh iy0))
("kenkel" nil (k eh1 ng k ax l))
("kenley" nil (k eh1 n l iy0))
("kenmare" nil (k eh1 n m eh1 r))
("kenmore" nil (k eh1 n m ao1 r))
("kenn" nil (k eh1 n))
("kenna" nil (k eh1 n ax))
("kennamer" nil (k eh1 n ax m er0))
("kennametal" nil (k eh1 n ax m eh1 t ax l))
("kennan" nil (k eh1 n ax n))
("kennard" nil (k eh1 n er0 d))
("kenneally" nil (k eh1 n ax l iy0))
("kennebeck" nil (k eh1 n b eh0 k))
("kennebrew" nil (k eh1 n ih0 b r uw0))
("kennebunkport" nil (k eh1 n ax b ah1 ng k p ao1 r t))
("kennecott" nil (k eh1 n ax k aa1 t))
("kennedy" nil (k eh1 n ax d iy0))
("kennedys" nil (k eh1 n ax d iy0 z))
("kennel" nil (k eh1 n ax l))
("kennell" nil (k eh1 n ax l))
("kennelly" nil (k eh1 n ax l iy0))
("kennels" nil (k eh1 n ax l z))
("kennemer" nil (k eh1 n iy0 m er0))
("kennemore" nil (k eh1 n m ao0 r))
("kenner" nil (k eh1 n er0))
("kennerly" nil (k eh1 n er0 l iy0))
("kennerson" nil (k eh1 n er0 s ax n))
("kennesaw" nil (k eh1 n ax s ao1))
("kenneth" nil (k eh1 n ih0 th))
("kennett" nil (k eh1 n ax t))
("kenney" nil (k eh1 n iy0))
("kenning" nil (k eh1 n ih0 ng))
("kennington" nil (k eh1 n ih0 ng t ax n))
("kennison" nil (k eh1 n ih0 s ax n))
("kennon" nil (k eh1 n ax n))
("kenny" nil (k eh1 n iy0))
("keno" nil (k iy1 n ow0))
("kenosha" nil (k ih0 n ow1 sh ax))
("kenoyer" nil (k eh1 n oy0 er0))
("kenrick" nil (k eh1 n r ih0 k))
("kensinger" nil (k eh1 n s ih0 n jh er0))
("kensington" nil (k eh1 n z ih0 ng t ax n))
("kensler" nil (k eh1 n s ax l er0))
("kent" nil (k eh1 n t))
("kente" nil (k eh1 n t ey0))
("kentner" nil (k eh1 n t n er0))
("kenton" nil (k eh1 n t ax n))
("kentuckians" nil (k eh1 n t ah1 k iy0 ax n z))
("kentucky" nil (k ax n t ah1 k iy0))
("kenward" nil (k eh1 n w er0 d))
("kenway" nil (k eh1 n w ey1))
("kenwood" nil (k eh1 n w uh1 d))
("kenworth" nil (k eh1 n w er1 th))
("kenworthy" nil (k eh1 n w er1 dh iy0))
("kenya" nil (k eh1 n y ax))
("kenyan" nil (k eh1 n y ax n))
("kenyon" nil (k eh1 n y ax n))
("kenzie" nil (k eh1 n z iy0))
("kenzo" nil (k eh1 n z ow0))
("keo" nil (k iy1 ow0))
("keogh" nil (k iy1 ow0))
("keohane" nil (k iy1 ax hh ey1 n))
("keough" nil (k iy1 ow0))
("keown" nil (k iy1 ow0 n))
("kepco" nil (k eh1 p k ow0))
("kephart" nil (k eh1 f aa0 r t))
("kepler" nil (k eh1 p l er0))
("kepley" nil (k eh1 p l iy0))
("keplinger" nil (k eh1 p ax l ih0 ng er0))
("kepner" nil (k eh1 p n er0))
("kepp" nil (k eh1 p))
("keppel" nil (k eh1 p ax l))
("kepple" nil (k eh1 p ax l))
("keppler" nil (k eh1 p l er0))
("kept" nil (k eh1 p t))
("ker" nil (k er1))
("kera" nil (k eh1 r ax))
("kerala" nil (k eh1 r aa1 l ax))
("keranen" nil (k eh1 r ax n ax n))
("keratin" nil (k eh1 r ax t ax n))
("kerbel" nil (k er1 b ax l))
("kerber" nil (k er1 b er0))
("kerbow" nil (k er1 b ow0))
("kerbs" nil (k er1 b z))
("kerby" nil (k er1 b iy0))
("kerce" nil (k er1 s))
("kercher" nil (k er1 k er0))
("kercheval" nil (k er1 ch ih0 v ax l))
("kerchiefs" nil (k er1 ch ax f s))
("kerchner" nil (k er1 k n er0))
("kerekes" nil (k eh1 r ih0 k s))
("kerestes" nil (k eh1 r ih0 s t s))
("keresztes" nil (k eh1 r ax s t iy0 z))
("kerfoot" nil (k er1 f uh0 t))
("kerien" nil (k eh1 r iy0 ax n))
("kerin" nil (k eh1 r ih0 n))
("kerins" nil (k eh1 r ih0 n z))
("kerker" nil (k er1 k er0))
("kerkhoff" nil (k er1 k hh ao0 f))
("kerkman" nil (k er1 k m ax n))
("kerkorian" nil (k er0 k ao1 r iy0 ax n))
("kerl" nil (k er1 l))
("kerley" nil (k er1 l iy0))
("kerlin" nil (k er1 l ih0 n))
("kerman" nil (k er1 m ax n))
("kermit" nil (k er1 m ih0 t))
("kern" nil (k er1 n))
("kernaghan" nil (k er0 n ae1 g hh ax n))
("kernan" nil (k er1 n ax n))
("kernel" nil (k er1 n ax l))
("kernels" nil (k er1 n ax l z))
("kernen" nil (k er1 n ax n))
("kerner" nil (k er1 n er0))
("kernes" nil (k er1 n z))
("kerney" nil (k er1 n iy0))
("kernite" nil (k er1 n ay0 t))
("kernodle" nil (k er1 n ow0 d ax l))
("kerns" nil (k er1 n z))
("kerosene" nil (k eh1 r ax s iy1 n))
("kerpedjiev" nil (k er0 p eh1 jh iy0 eh1 v))
("kerper" nil (k er1 p er0))
("kerr" nil (k er1))
("kerrey" nil (k eh1 r iy0))
("kerrick" nil (k eh1 r ih0 k))
("kerridge" nil (k eh1 r ih0 jh))
("kerrigan" nil (k eh1 r ih0 g ax n))
("kerrville" nil (k er1 v ih1 l))
("kerry" nil (k eh1 r iy0))
("kersch" nil (k er1 sh))
("kerscher" nil (k er1 sh er0))
("kerschner" nil (k er1 sh n er0))
("kersee" nil (k er1 s iy0))
("kersey" nil (k er1 s iy0))
("kersh" nil (k er1 sh))
("kershaw" nil (k er1 sh aa1))
("kershner" nil (k er1 sh n er0))
("kerst" nil (k er1 s t))
("kerstein" nil (k er1 s t ay0 n))
("kersten" nil (k er1 s ax n))
("kerstetter" nil (k er1 s t ih0 t er0))
("kersting" nil (k er1 s t ih0 ng))
("kertesz" nil (k er1 t ih0 sh))
("kerth" nil (k er1 th))
("kertz" nil (k er1 t s))
("kervin" nil (k er1 v ih0 n))
("kerwen" nil (k er1 w ax n))
("kerwin" nil (k er1 w ih0 n))
("kerwood" nil (k er1 w uh0 d))
("kerzner" nil (k er1 z n er0))
("kesel" nil (k eh1 s ax l))
("keshishian" nil (k ih0 sh ih1 sh iy0 ax n))
("kesinger" nil (k eh1 s ih0 ng er0))
("keske" nil (k eh1 s k))
("keslar" nil (k eh1 s l er0))
("kesler" nil (k eh1 s ax l er0))
("kesling" nil (k eh1 s ax l ih0 ng))
("kesner" nil (k eh1 s n er0))
("kess" nil (k eh1 s))
("kessel" nil (k eh1 s ax l))
("kessell" nil (k eh1 s ax l))
("kesselman" nil (k eh1 s ax l m ax n))
("kesselring" nil (k eh1 s ih0 l r ih0 ng))
("kessen" nil (k eh1 s ax n))
("kessenich" nil (k eh1 s ih0 n ih0 k))
("kessinger" nil (k eh1 s ih0 ng er0))
("kessler" nil (k eh1 s l er0))
("kessner" nil (k eh1 s n er0))
("kestel" nil (k eh1 s t ax l))
("kesten" nil (k eh1 s ax n))
("kestenbaum" nil (k eh1 s ax n b aw0 m))
("kester" nil (k eh1 s t er0))
("kesterson" nil (k eh1 s t er0 s ax n))
("kesting" nil (k eh1 s t ih0 ng))
("kestler" nil (k eh1 s t l er0))
("kestner" nil (k eh1 s t n er0))
("kestrels" nil (k eh1 s t r ax l z))
("keswick" nil (k eh1 s w ih1 k))
("ketch" nil (k eh1 ch))
("ketcham" nil (k eh1 ch ax m))
("ketchem" nil (k eh1 ch ih0 m))
("ketchen" nil (k eh1 ch ax n))
("ketcher" nil (k eh1 ch er0))
("ketcherside" nil (k eh1 ch er0 s ay1 d))
("ketchie" nil (k eh1 ch iy0))
("ketchikan" nil (k eh1 ch ih0 k ae0 n))
("ketchledge" nil (k eh1 ch l eh1 jh))
("ketchum" nil (k eh1 ch ax m))
("ketchup" nil (k eh1 ch ax p))
("ketelhut" nil (k eh1 t ih0 l hh ax t))
("ketelsen" nil (k eh1 t ih0 l s ax n))
("ketema" nil (k eh1 t ey1 m ax))
("keteyian" nil (k ax t ey1 ax n))
("ketler" nil (k eh1 t l er0))
("ketner" nil (k eh1 t n er0))
("keto" nil (k ey1 t ow0))
("ketola" nil (k eh1 t ax l ax))
("ketone" nil (k iy1 t ow0 n))
("ketou" nil (k ax t uw1))
("ketron" nil (k eh1 t r ax n))
("kett" nil (k eh1 t))
("kettell" nil (k eh1 t ax l))
("ketter" nil (k eh1 t er0))
("ketterer" nil (k eh1 t er0 er0))
("kettering" nil (k eh1 t er0 ih0 ng))
("ketterling" nil (k eh1 t er0 l ih0 ng))
("ketterman" nil (k eh1 t er0 m ax n))
("ketti" nil (k eh1 t iy0))
("kettle" nil (k eh1 t ax l))
("kettler" nil (k eh1 t ax l er0))
("kettles" nil (k eh1 t ax l z))
("kettlewell" nil (k eh1 t ax l w eh1 l))
("kettner" nil (k eh1 t n er0))
("ketura" nil (k eh1 t uh0 r ax))
("ketz" nil (k eh1 t s))
("ketzel" nil (k eh1 t s ax l))
("keune" nil (k y uw1 n))
("kevan" nil (k eh1 v ax n))
("keven" nil (k iy1 v ax n))
("kever" nil (k eh1 v er0))
("kevex" nil (k eh1 v ax k s))
("keville" nil (k iy1 v ih0 l))
("kevin" nil (k eh1 v ih0 n))
("kevlar" nil (k eh1 v l er0))
("kevlin" nil (k eh1 v l ih0 n))
("kevorkian" nil (k ax v ao1 r k iy0 ax n))
("kew" nil (k y uw1))
("kewaunee" nil (k y uw0 ao1 n iy0))
("kewley" nil (k y uw1 l iy0))
("key" nil (k iy1))
("keyboard" nil (k iy1 b ao1 r d))
("keyboards" nil (k iy1 b ao1 r d z))
("keycorp" nil (k iy1 k ao0 r p))
("keye" nil (k ay1))
("keyed" nil (k iy1 d))
("keyes" nil (k iy1 z))
("keyhole" nil (k iy1 hh ow1 l))
("keying" nil (k iy1 ih0 ng))
("keyless" nil (k iy1 l ax s))
("keylon" nil (k ey1 l ax n))
("keynes" nil (k ey1 n z))
("keynesian" nil (k ey1 n z iy0 ax n))
("keynesians" nil (k ey1 n iy1 s iy0 ax n z))
("keynote" nil (k iy1 n ow1 t))
("keypad" nil (k iy1 p ae1 d))
("keypads" nil (k iy1 p ae1 d z))
("keys" nil (k iy1 z))
("keyser" nil (k ay1 z er0))
("keysor" nil (k iy1 s er0))
("keystone" nil (k iy1 s t ow1 n))
("keyton" nil (k iy1 t ax n))
("keyworth" nil (k iy1 w er1 th))
("keziah" nil (k ax z iy1 ax))
("kganakga" nil (k ax g ax n ae1 g ax))
("kgori" nil (k ax g ao1 r iy0))
("khabarovsk" nil (k ae1 b er0 aa1 f s k))
("khad" nil (k ae1 d))
("khaki" nil (k aa1 k iy0))
("khakis" nil (k ae1 k iy0 z))
("khalaf" nil (k ae1 l ax f))
("khalid" nil (k aa1 l ih0 d))
("khalifa" nil (k ax l iy1 f ax))
("khalil" nil (k ae1 l ax l))
("khalsa" nil (k aa1 l s ax))
("khamenei" nil (k ax m ey1 n iy1))
("khan" nil (k aa1 n))
("khanate" nil (k aa1 n ey0 t))
("khanna" nil (k ae1 n ax))
("kharg" nil (k aa1 r g))
("khartoum" nil (k aa1 r t uw1 m))
("khasbulatov" nil (k aa1 s b ax l aa1 t aa0 f))
("khashoggi" nil (k ax sh aa1 jh iy0))
("kheel" nil (k iy1 l))
("khem" nil (k eh1 m))
("khmer" nil (k m eh1 r))
("khomeini" nil (k ow0 m ey1 n iy0))
("khoo" nil (k uw1))
("khosla" nil (k ao1 s l ax))
("khost" nil (k ow1 s t))
("khouri" nil (k aw1 r iy0))
("khoury" nil (k aw1 r iy0))
("khrushchev" nil (k r uw1 s ch eh1 v))
("khumalo" nil (k y uw0 m aa1 l ow0))
("khuu" nil (k uw1))
("ki" nil (k iy1))
("kia" nil (k iy1 er0))
("kiam" nil (k iy1 ax m))
("kiawah" nil (k ay1 ax w ax))
("kibbe" nil (k ih1 b))
("kibbee" nil (k ih1 b iy1))
("kibbey" nil (k ih1 b iy0))
("kibble" nil (k ih1 b ax l))
("kibbutz" nil (k ih0 b uh1 t s))
("kibbutzim" nil (k ih1 b uh0 t s ih1 m))
("kibbutzniks" nil (k ih0 b uh1 t s n ih0 k s))
("kibby" nil (k ih1 b iy0))
("kibler" nil (k ao1 b ax l er0))
("kibodeaux" nil (k ih1 b ax d ow0))
("kichline" nil (k ih1 k l ay1 n))
("kick" nil (k ih1 k))
("kickback" nil (k ih1 k b ae1 k))
("kickbacks" nil (k ih1 k b ae1 k s))
("kicked" nil (k ih1 k t))
("kicker" nil (k ih1 k er0))
("kickers" nil (k ih1 k er0 z))
("kicking" nil (k ih1 k ih0 ng))
("kicklighter" nil (k ih1 k l ay1 t er0))
("kickoff" nil (k ih1 k ao1 f))
("kicks" nil (k ih1 k s))
("kid" nil (k ih1 d))
("kida" nil (k iy1 d ax))
("kidd" nil (k ih1 d))
("kidde" nil (k ih1 d))
("kidded" nil (k ih1 d ih0 d))
("kidder" nil (k ih1 d er0))
("kiddie" nil (k ih1 d iy0))
("kiddies" nil (k ih1 d iy0 z))
("kidding" nil (k ih1 d ih0 ng))
("kiddy" nil (k ih1 d iy0))
("kidman" nil (k ih1 d m ax n))
("kidnap" nil (k ih1 d n ae1 p))
("kidnaped" nil (k ih1 d n ae1 p t))
("kidnaping" nil (k ih1 d n ae1 p ih0 ng))
("kidnapped" nil (k ih1 d n ae1 p t))
("kidnapper" nil (k ih1 d n ae1 p er0))
("kidnappers" nil (k ih1 d n ae1 p er0 z))
("kidnapping" nil (k ih1 d n ae1 p ih0 ng))
("kidnappings" nil (k ih1 d n ae1 p ih0 ng z))
("kidnaps" nil (k ih1 d n ae1 p s))
("kidney" nil (k ih1 d n iy0))
("kidneys" nil (k ih1 d n iy0 z))
("kido" nil (k iy1 d ow0))
("kids" nil (k ih1 d z))
("kidstock" nil (k ih1 d s t aa1 k))
("kidwell" nil (k ih1 d w eh1 l))
("kiecker" nil (k iy1 k er0))
("kiedrowski" nil (k iy0 d r ao1 f s k iy0))
("kief" nil (k iy1 f))
("kiefer" nil (k iy1 f er0))
("kieffer" nil (k iy1 f er0))
("kieft" nil (k iy1 f t))
("kiehl" nil (k iy1 l))
("kiehn" nil (k iy1 n))
("kiehne" nil (k iy1 n))
("kiel" nil (k iy1 l))
("kielar" nil (k iy1 l er0))
("kielb" nil (k iy1 l b))
("kielbasa" nil (k iy0 l b aa1 s ax))
("kieler" nil (k iy1 l er0))
("kielley" nil (k iy1 l iy0))
("kielman" nil (k iy1 l m ax n))
("kielty" nil (k iy1 l t iy0))
("kiely" nil (k iy1 l iy0))
("kienan" nil (k iy1 n ax n))
("kienast" nil (k iy1 n ax s t))
("kiene" nil (k iy1 n))
("kiener" nil (k iy1 n er0))
("kieninger" nil (k iy1 n ih0 ng er0))
("kienitz" nil (k iy1 n ih0 t s))
("kienle" nil (k iy1 n ax l))
("kientz" nil (k iy1 n t s))
("kienzle" nil (k iy1 n z ax l))
("kieper" nil (k iy1 p er0))
("kier" nil (k ih1 r))
("kieran" nil (k iy1 r ax n))
("kiernan" nil (k ih1 r n ax n))
("kierscht" nil (k ih1 r sh t))
("kierstead" nil (k ih1 r s t eh0 d))
("kierulff" nil (k iy1 r ax l f))
("kies" nil (k ay1 s))
("kieschnick" nil (k iy1 sh n ih0 k))
("kiesel" nil (k iy1 s ax l))
("kieselmann" nil (k iy1 z ax l m ax n))
("kieser" nil (k iy1 s er0))
("kiesewetter" nil (k iy1 s uw0 ih0 t er0))
("kiesler" nil (k iy1 z l er0))
("kiesling" nil (k iy1 z l ih0 ng))
("kiesow" nil (k iy1 s ow0))
("kiess" nil (k iy1 s))
("kiessling" nil (k iy1 s l ih0 ng))
("kiester" nil (k ay1 ih0 s t er0))
("kietzman" nil (k iy1 t s m ax n))
("kiev" nil (k iy1 eh1 v))
("kievan" nil (k iy1 v ax n))
("kievit" nil (k iy1 v ih0 t))
("kiewit" nil (k iy1 w ih0 t))
("kifer" nil (k ay1 f er0))
("kiff" nil (k ih1 f))
("kigale" nil (k iy0 g aa1 l iy0))
("kigali" nil (k iy0 g aa1 l iy0))
("kiger" nil (k ay1 g er0))
("kiggins" nil (k ih1 g ih0 n z))
("kight" nil (k ay1 t))
("kightlinger" nil (k ay1 t ax l ih0 ng er0))
("kihn" nil (k ih1 n))
("kiichi" nil (k iy0 iy1 ch iy0))
("kijowski" nil (k ih0 y ao1 f s k iy0))
("kika" nil (k ih1 k ax))
("kiker" nil (k ay1 k er0))
("kiki" nil (k iy1 k iy0))
("kikkoman" nil (k iy1 k ow0 m aa0 n))
("kikta" nil (k ih1 k t ax))
("kikuchi" nil (k iy0 k uw1 ch iy0))
("kikumura" nil (k iy1 k uw1 m uw1 r ax))
("kilbane" nil (k ih1 l b ax n))
("kilborn" nil (k ih1 l b er0 n))
("kilborne" nil (k ih1 l b ao1 r n))
("kilbourn" nil (k ih1 l b er0 n))
("kilbourne" nil (k ih1 l b er0 n))
("kilbride" nil (k ih1 l b r ay1 d))
("kilburg" nil (k ih1 l b er0 g))
("kilburn" nil (k ih1 l b er0 n))
("kilbury" nil (k ih1 l b eh1 r iy0))
("kilby" nil (k ih1 l b iy0))
("kilcoyne" nil (k ih1 l k oy0 n))
("kilcrease" nil (k ih0 l k r iy1 s))
("kilcullen" nil (k ih0 l k ah1 l ax n))
("kilday" nil (k ih1 l d ey1))
("kildee" nil (k ih1 l d iy0))
("kildow" nil (k ih1 l d ow0))
("kilduff" nil (k ih1 l d ax f))
("kile" nil (k ay1 l))
("kilen" nil (k ih1 l ax n))
("kiley" nil (k ay1 l iy0))
("kilgo" nil (k ih1 l g ow0))
("kilgore" nil (k ih1 l g ao0 r))
("kilgour" nil (k ih1 l g er0))
("kilgus" nil (k ih1 l g ax s))
("kilian" nil (k ih1 l iy0 ax n))
("kilimanjaro" nil (k ih0 l iy1 m ax n jh aa1 r ow0))
("kilker" nil (k ih1 l k er0))
("kill" nil (k ih1 l))
("killam" nil (k ih1 l ax m))
("kille" nil (k ih1 l))
("killeagh" nil (k ih0 l iy1 g))
("killebrew" nil (k ih1 l ih0 b r uw0))
("killed" nil (k ih1 l d))
("killeen" nil (k ih0 l iy1 n))
("killen" nil (k ih1 l ax n))
("killer" nil (k ih1 l er0))
("killers" nil (k ih1 l er0 z))
("killey" nil (k ih1 l iy0))
("killgore" nil (k ih1 l g ao1 r))
("killian" nil (k ih1 l y ax n))
("killifish" nil (k ih1 l ih0 f ih0 sh))
("killilea" nil (k ih0 l ih0 l iy1 ax))
("killin" nil (k ih1 l ih0 n))
("killing" nil (k ih1 l ih0 ng))
("killingbeck" nil (k ih1 l ih0 ng b eh1 k))
("killinger" nil (k ih1 l ih0 ng er0))
("killings" nil (k ih1 l ih0 ng z))
("killingsworth" nil (k ih1 l ih0 ng z w er1 th))
("killington" nil (k ih1 l ih0 ng t ax n))
("killion" nil (k ih1 l y ax n))
("killman" nil (k ih1 l m ax n))
("killmer" nil (k ih1 l m er0))
("killmon" nil (k ih1 l m ax n))
("killoran" nil (k ih1 l er0 ax n))
("killory" nil (k ih1 l er0 iy0))
("killough" nil (k ih1 l aw0))
("killpack" nil (k ih1 l p ae1 k))
("kills" nil (k ih1 l z))
("kilman" nil (k ih1 l m ax n))
("kilmarnock" nil (k ih0 l m aa1 r n aa1 k))
("kilmartin" nil (k ih0 l m aa1 r t ih0 n))
("kilmer" nil (k ih1 l m er0))
("kiln" nil (k ih1 l n))
("kilns" nil (k ih1 l n z))
("kilo" nil (k ih1 l ow1))
("kilobit" nil (k ih1 l ax b ih0 t))
("kilobytes" nil (k ih1 l ow0 b ay1 t s))
("kilogram" nil (k ih1 l ax g r ae1 m))
("kilograms" nil (k ih1 l ax g r ae1 m z))
("kilometer" nil (k ax l aa1 m ax t er0))
("kilometers" nil (k ax l aa1 m ax t er0 z))
("kilos" nil (k iy1 l ow1 z))
("kilowatt" nil (k ih1 l ax w aa1 t))
("kilowatts" nil (k ih1 l ax w aa1 t s))
("kilpatrick" nil (k ih1 l p ae1 t r ih0 k))
("kilroy" nil (k ih1 l r oy1))
("kilter" nil (k ih1 l t er0))
("kilton" nil (k ih1 l t ax n))
("kilts" nil (k ih1 l t s))
("kilty" nil (k ih1 l t iy0))
("kilzer" nil (k ih1 l z er0))
("kim" nil (k ih1 m))
("kimba" nil (k ih1 m b ax))
("kimball" nil (k ih1 m b ax l))
("kimbel" nil (k ih1 m b ax l))
("kimbell" nil (k ih1 m b eh0 l))
("kimber" nil (k ih1 m b er0))
("kimberley" nil (k ih1 m b er0 l iy0))
("kimberlin" nil (k ih1 m b er0 l ih0 n))
("kimberling" nil (k ih1 m b er0 l ih0 ng))
("kimberly" nil (k ih1 m b er0 l iy0))
("kimble" nil (k ih1 m b ax l))
("kimbler" nil (k ih1 m b l er0))
("kimbley" nil (k ih1 m b l iy0))
("kimbrel" nil (k ih1 m b r ax l))
("kimbrell" nil (k ih1 m b r ax l))
("kimbriel" nil (k ih1 m b r iy0 ax l))
("kimbro" nil (k ih1 m b r ow0))
("kimbrough" nil (k ih1 m b r aw0))
("kimche" nil (k ih1 m ch iy0))
("kimchi" nil (k ih1 m ch iy0))
("kimco" nil (k ih1 m k ow0))
("kime" nil (k ay1 m))
("kimel" nil (k ih1 m ax l))
("kimery" nil (k ih1 m er0 iy0))
("kimes" nil (k ay1 m z))
("kimler" nil (k ih1 m ax l er0))
("kimm" nil (k ih1 m))
("kimmel" nil (k ih1 m ax l))
("kimmell" nil (k ih1 m ax l))
("kimmelman" nil (k ih1 m ax l m ax n))
("kimmer" nil (k ih1 m er0))
("kimmerle" nil (k ih1 m er0 ax l))
("kimmet" nil (k ih1 m ih0 t))
("kimmey" nil (k ih1 m iy0))
("kimmich" nil (k ih1 m ih0 ch))
("kimmins" nil (k ih1 m ih0 n z))
("kimmitt" nil (k ih1 m ih0 t))
("kimmons" nil (k ih1 m ax n z))
("kimono" nil (k ax m ow1 n ax))
("kimonos" nil (k ax m ow1 n ax z))
("kimoto" nil (k iy0 m ow1 t ow0))
("kimpel" nil (k ih1 m p ax l))
("kimple" nil (k ih1 m p ax l))
("kimpo" nil (k ih1 m p ow0))
("kimpton" nil (k ih1 m p t ax n))
("kimrey" nil (k ih1 m r iy0))
("kims" nil (k ih1 m z))
("kimsey" nil (k ih1 m z iy0))
("kimura" nil (k iy0 m uh1 r ax))
("kimzey" nil (k ih1 m z iy0))
("kin" nil (k ih1 n))
("kinard" nil (k ih1 n er0 d))
("kinark" nil (k ih1 n aa0 r k))
("kinburn" nil (k ih1 n b er1 n))
("kincade" nil (k ih1 n k ey1 d))
("kincaid" nil (k ih1 n k ey1 d))
("kincannon" nil (k ih1 n k ae1 n ax n))
("kincer" nil (k ih1 n s er0))
("kinch" nil (k ih1 n ch))
("kincheloe" nil (k ih1 n ch ih0 l ow0))
("kinchen" nil (k ih1 ng k ax n))
("kincy" nil (k ih1 n s iy0))
("kind" nil (k ay1 n d))
("kinda" nil (k ih1 n d ax))
("kindall" nil (k ih1 n d ax l))
("kindel" nil (k ih1 n d ax l))
("kindell" nil (k ih1 n d ax l))
("kinder" nil (k ay1 n d er0))
("kindergarten" nil (k ih1 n d er0 g aa1 r t ax n))
("kindergartens" nil (k ih1 n d er0 g aa1 r t ax n z))
("kindergartner" nil (k ih1 n d er0 g aa1 r t n er0))
("kindergartners" nil (k ih1 n d er0 g aa1 r t n er0 z))
("kinderman" nil (k ay1 n d er0 m ax n))
("kindest" nil (k ay1 n d ax s t))
("kindig" nil (k ih1 n d ih0 g))
("kindle" nil (k ih1 n d ax l))
("kindleberger" nil (k ih1 n d ax l b er0 g er0))
("kindled" nil (k ih1 n d ax l d))
("kindler" nil (k ih1 n d ax l er0))
("kindley" nil (k ih1 n d l iy0))
("kindling" nil (k ih1 n d l ih0 ng))
("kindly" nil (k ay1 n d l iy0))
("kindness" nil (k ay1 n d n ax s))
("kindran" nil (k ih1 n d r ax n))
("kindred" nil (k ih1 n d r ih0 d))
("kindrick" nil (k ih1 n d r ih0 k))
("kinds" nil (k ay1 n d z))
("kindt" nil (k ih1 n t))
("kindy" nil (k ay1 n d iy0))
("kinepolis" nil (k ih0 n eh1 p ax l ih0 s))
("kiner" nil (k ay1 n er0))
("kines" nil (k ay1 n z))
("kinesiology" nil (k ih1 n ih0 s iy1 aa1 l ax jh iy0))
("kinesthetic" nil (k ih1 n ax s th eh1 t ih0 k))
("kineta" nil (k ih1 n ih0 t ax))
("kinetic" nil (k ax n eh1 t ih0 k))
("kinetics" nil (k ax n eh1 t ih0 k s))
("king" nil (k ih1 ng))
("kingbirds" nil (k ih1 ng b er1 d z))
("kingdom" nil (k ih1 ng d ax m))
("kingdoms" nil (k ih1 ng d ax m z))
("kingdon" nil (k ih1 ng d ax n))
("kingen" nil (k ih1 ng ax n))
("kingery" nil (k ih1 ng g er0 iy0))
("kingfisher" nil (k ih1 ng f ih1 sh er0))
("kingfishers" nil (k ih1 ng f ih1 sh er0 z))
("kingham" nil (k ih1 ng hh ae1 m))
("kinghorn" nil (k ih1 ng hh er0 n))
("kingly" nil (k ih1 ng l iy0))
("kingma" nil (k ih1 ng m ax))
("kingmaker" nil (k ih1 ng m ey1 k er0))
("kingman" nil (k ih1 ng m ax n))
("kingon" nil (k ih1 ng ao1 n))
("kingpin" nil (k ih1 ng p ih1 n))
("kingpins" nil (k ih1 ng p ih1 n z))
("kingrey" nil (k ih1 ng g r iy0))
("kingry" nil (k ih1 ng g er0 iy0))
("kings" nil (k ih1 ng z))
("kingsborough" nil (k ih1 ng z b er1 ow0))
("kingsbridge" nil (k ih1 ng z b r ih1 jh))
("kingsbury" nil (k ih1 ng z b eh1 r iy0))
("kingsey" nil (k ih1 ng z iy0))
("kingsford" nil (k ih1 ng s f er0 d))
("kingship" nil (k ih1 ng sh ih0 p))
("kingsland" nil (k ih1 ng z l ae0 n d))
("kingsley" nil (k ih1 ng z l iy0))
("kingsolver" nil (k ih1 ng s aa0 l v er0))
("kingsport" nil (k ih1 ng s p ao1 r t))
("kingston" nil (k ih1 ng s t ax n))
("kingsville" nil (k ih1 ng z v ih1 l))
("kingswell" nil (k ih1 ng g z w eh0 l))
("kington" nil (k ih1 ng t ax n))
("kinion" nil (k ih1 n y ax n))
("kiniry" nil (k ih1 n ay0 r iy0))
("kinkade" nil (k ih1 ng k ax d))
("kinkead" nil (k ih1 ng k eh0 d))
("kinkel" nil (k ih1 ng k ax l))
("kinker" nil (k ih1 ng k er0))
("kinkle" nil (k ih1 ng k ax l))
("kinks" nil (k ih1 ng k s))
("kinky" nil (k ih1 ng k iy0))
("kinlaw" nil (k ih1 n l ao1))
("kinley" nil (k ih1 n l iy0))
("kinloch" nil (k ih1 n l ax k))
("kinman" nil (k ih1 n m ax n))
("kinn" nil (k ih1 n))
("kinnaird" nil (k ih1 n er0 d))
("kinnaman" nil (k ih1 n ax m ax n))
("kinnamon" nil (k ih1 n ax m ax n))
("kinnan" nil (k ih1 n ax n))
("kinnard" nil (k ih1 n er0 d))
("kinne" nil (k ih1 n))
("kinnear" nil (k ih1 n ih1 r))
("kinnebrew" nil (k ih1 n ih0 b r uw0))
("kinnell" nil (k ih1 n ax l))
("kinner" nil (k ih1 n er0))
("kinnett" nil (k ih1 n ih0 t))
("kinney" nil (k ih1 n iy0))
("kinnick" nil (k ih1 n ih0 k))
("kinnie" nil (k ih1 n iy0))
("kinnison" nil (k ih1 n ih0 s ax n))
("kinnock" nil (k ih1 n ax k))
("kinnunen" nil (k ih0 n ah1 n ax n))
("kinoshita" nil (k iy0 n ow0 sh iy1 t ax))
("kinsel" nil (k ih1 n s ax l))
("kinsell" nil (k ih1 n s ax l))
("kinsella" nil (k iy0 n s eh1 l ax))
("kinser" nil (k ih1 n s er0))
("kinsey" nil (k ih1 n z iy0))
("kinship" nil (k ih1 n sh ih1 p))
("kinsinger" nil (k ih1 n s ih0 n jh er0))
("kinsler" nil (k ih1 n s ax l er0))
("kinsley" nil (k ih1 n z l iy0))
("kinslow" nil (k ih1 n s l ow1))
("kinsman" nil (k ih1 n z m ae1 n))
("kinst" nil (k ih1 n s t))
("kinstler" nil (k ih1 n s t l er0))
("kint" nil (k ih1 n t))
("kinter" nil (k ih1 n t er0))
("kintigh" nil (k ih1 n t ay0))
("kintner" nil (k ih1 n t n er0))
("kinton" nil (k ih1 n t ax n))
("kintz" nil (k ih1 n t s))
("kintzel" nil (k ih1 n t z ax l))
("kinyon" nil (k ih1 n y ax n))
("kinzel" nil (k ih1 n z ax l))
("kinzer" nil (k ih1 n z er0))
("kinzey" nil (k ih1 n z iy0))
("kinzie" nil (k ih1 n z iy0))
("kinzler" nil (k ih1 n z l er0))
("kinzlmaier" nil (k ih1 n z ax l m ay1 er0))
("kiosk" nil (k iy1 ao1 s k))
("kiosks" nil (k iy1 ao1 s k s))
("kious" nil (k ay1 ax s))
("kip" nil (k ih1 p))
("kiper" nil (k ay1 p er0))
("kipfer" nil (k ih1 p f er0))
("kipling" nil (k ih1 p l ih0 ng))
("kiplinger" nil (k ay1 p ax l ih0 ng er0))
("kipnis" nil (k ih1 p n ih0 s))
("kipp" nil (k ih1 p))
("kipper" nil (k ih1 p er0))
("kippers" nil (k ih1 p er0 z))
("kippes" nil (k ih1 p s))
("kippur" nil (k ih1 p er0))
("kiracofe" nil (k ih1 r ax k ow1 f))
("kiraly" nil (k ih1 r ax l iy0))
("kirby" nil (k er1 b iy0))
("kirch" nil (k er1 k))
("kirchberg" nil (k er1 k b er0 g))
("kirchberger" nil (k er1 ch b er0 g er0))
("kirchen" nil (k er1 k ax n))
("kircher" nil (k er1 k er0))
("kirchgessner" nil (k er1 k g ih0 s n er0))
("kirchhofer" nil (k er1 k hh ax f er0))
("kirchhoff" nil (k er1 k hh ao0 f))
("kirchman" nil (k er1 k m ax n))
("kirchner" nil (k er1 k n er0))
("kirchoff" nil (k er1 k hh ao0 f))
("kirgan" nil (k er1 g ax n))
("kiri" nil (k ih1 r iy0))
("kiribati" nil (k ih1 r ih1 b aa1 t iy0))
("kirin" nil (k ih1 r ih0 n))
("kirk" nil (k er1 k))
("kirkbride" nil (k er1 k b r ay0 d))
("kirkby" nil (k er1 k b iy0))
("kirkeby" nil (k er1 k ih0 b iy0))
("kirkendall" nil (k er0 k eh1 n d ax l))
("kirkendoll" nil (k er0 k eh1 n d ow0 l))
("kirker" nil (k er1 k er0))
("kirkey" nil (k er1 k iy0))
("kirkham" nil (k er1 k hh ax m))
("kirkhart" nil (k er1 k hh aa0 r t))
("kirkland" nil (k er1 k l ax n d))
("kirkley" nil (k er1 k l iy0))
("kirklin" nil (k er1 k l ih0 n))
("kirkman" nil (k er1 k m ax n))
("kirkner" nil (k er1 k n er0))
("kirkpatrick" nil (k er0 k p ae1 t r ih0 k))
("kirks" nil (k er1 k s))
("kirksey" nil (k er1 k s iy0))
("kirkuk" nil (k ih1 r k uh1 k))
("kirkwood" nil (k er1 k w uh0 d))
("kirley" nil (k er1 l iy0))
("kirlin" nil (k er1 l ih0 n))
("kirmse" nil (k er1 m s iy0))
("kirn" nil (k er1 n))
("kirnan" nil (k er1 n ax n))
("kirner" nil (k er1 n er0))
("kirouac" nil (k ay1 r aw0 ae0 k))
("kirov" nil (k ih1 r aa0 v))
("kirsch" nil (k er1 sh))
("kirschbaum" nil (k er1 sh b aw0 m))
("kirschenbaum" nil (k er1 sh ax n b aw0 m))
("kirschenmann" nil (k er1 sh ax n m ax n))
("kirschman" nil (k er1 sh m ax n))
("kirschner" nil (k er1 sh n er0))
("kirsh" nil (k er1 sh))
("kirshbaum" nil (k er1 sh b aw1 m))
("kirshenbaum" nil (k er1 sh ax n b aw0 m))
("kirshner" nil (k er1 sh n er0))
("kirst" nil (k er1 s t))
("kirstein" nil (k er1 s t ay0 n))
("kirsten" nil (k er1 s t ax n))
("kirstie" nil (k er1 s t iy0))
("kirstin" nil (k er1 s t ih0 n))
("kirt" nil (k er1 t))
("kirtland" nil (k er1 t l ax n d))
("kirtley" nil (k er1 t l iy0))
("kirton" nil (k er1 t ax n))
("kirts" nil (k er1 t s))
("kirven" nil (k er1 v ax n))
("kirwan" nil (k er1 w ao0 n))
("kirwin" nil (k er1 w ih0 n))
("kiryas" nil (k ih1 r y ax s))
("kiryat" nil (k ih1 r y ax t))
("kis" nil (k ih1 s))
("kisamore" nil (k iy0 s aa1 m ao0 r))
("kisan" nil (k ih1 z ax n))
("kisch" nil (k ih1 sh))
("kisco" nil (k ih1 s k ow0))
("kise" nil (k ay1 z))
("kiser" nil (k ay1 z er0))
("kish" nil (k ih1 sh))
("kisha" nil (k ih1 sh ax))
("kishbaugh" nil (k ih1 sh b aw0))
("kishi" nil (k iy1 sh iy0))
("kisiel" nil (k ih1 s iy0 l))
("kiska" nil (k ih1 s k ax))
("kisler" nil (k ih1 s ax l er0))
("kisling" nil (k ih1 s ax l ih0 ng))
("kismet" nil (k ih1 z m ih0 t))
("kismets" nil (k ih1 z m ih0 t s))
("kisner" nil (k ih1 s n er0))
("kisor" nil (k ay1 z er0))
("kiss" nil (k ih1 s))
("kissack" nil (k ih1 s ax k))
("kissam" nil (k ih1 s ax m))
("kissane" nil (k ih1 s ax n))
("kissed" nil (k ih1 s t))
("kissee" nil (k ih1 s iy1))
("kissel" nil (k ih1 s ax l))
("kissell" nil (k ih1 s ax l))
("kisses" nil (k ih1 s ax z))
("kissick" nil (k ih1 s ih0 k))
("kissimmee" nil (k ih0 s ih1 m iy0))
("kissing" nil (k ih1 s ih0 ng))
("kissinger" nil (k ih1 s ih0 n jh er0))
("kissler" nil (k ih1 s l er0))
("kissling" nil (k ih1 s l ih0 ng))
("kissner" nil (k ih1 s n er0))
("kist" nil (k ih1 s t))
("kister" nil (k ih1 s t er0))
("kistler" nil (k ih1 s t l er0))
("kistner" nil (k ih1 s t n er0))
("kiszczak" nil (k ih1 z ae0 k))
("kit" nil (k ih1 t))
("kita" nil (k iy1 t ax))
("kitagawa" nil (k iy0 t aa0 g aa1 w ax))
("kitajima" nil (k iy1 t ax jh iy1 m ax))
("kitamura" nil (k iy0 t aa0 m uh1 r ax))
("kitcat" nil (k ih1 t k ae1 t))
("kitch" nil (k ih1 ch))
("kitchel" nil (k ih1 ch ax l))
("kitchell" nil (k ih1 ch ax l))
("kitchen" nil (k ih1 ch ax n))
("kitchenaid" nil (k ih1 ch ax n ey1 d))
("kitchenette" nil (k ih1 ch ax n eh1 t))
("kitchens" nil (k ih1 ch ax n z))
("kitchenware" nil (k ih1 ch ax n w eh1 r))
("kitchin" nil (k ih1 ch ih0 n))
("kitching" nil (k ih1 ch ih0 ng))
("kitchings" nil (k ih1 ch ih0 ng z))
("kite" nil (k ay1 t))
("kites" nil (k ay1 t s))
("kithcart" nil (k ih1 th k aa1 r t))
("kitimat" nil (k ih1 t ih0 m ae0 t))
("kiting" nil (k ay1 t ih0 ng))
("kitner" nil (k ih1 t n er0))
("kitowski" nil (k ih0 t ao1 f s k iy0))
("kits" nil (k ih1 t s))
("kitsch" nil (k ih1 ch))
("kitson" nil (k ih1 t s ax n))
("kitt" nil (k ih1 t))
("kittaneh" nil (k ih0 t aa1 n eh1))
("kittel" nil (k ih1 t ax l))
("kittell" nil (k ih1 t ax l))
("kittelson" nil (k ih1 t ih0 l s ax n))
("kitten" nil (k ih1 t ax n))
("kittens" nil (k ih1 t ax n z))
("kitterman" nil (k ih1 t er0 m ax n))
("kittinger" nil (k ih1 t ih0 ng er0))
("kittiwake" nil (k ih1 t ih0 w ey1 k))
("kittle" nil (k ih1 t ax l))
("kittler" nil (k ih1 t ax l er0))
("kittles" nil (k ih1 t ax l z))
("kittleson" nil (k ih1 t l ih0 s ax n))
("kittner" nil (k ih1 t n er0))
("kittredge" nil (k ih1 t r ih0 jh))
("kittrell" nil (k ih1 t r ax l))
("kitts" nil (k ih1 t s))
("kitty" nil (k ih1 t iy0))
("kitz" nil (k ih1 t s))
("kitzman" nil (k ih1 t s m ax n))
("kitzmiller" nil (k ih1 t s m ih1 l er0))
("kivela" nil (k ih1 v ih0 l ax))
("kivett" nil (k ih1 v ih0 t))
("kivi" nil (k ih1 v iy0))
("kiwanis" nil (k ih0 w aa1 n ih0 s))
("kiwi" nil (k iy1 w iy0))
("kiwis" nil (k ih1 w ih0 s))
("kiyoshi" nil (k ih0 y ow1 sh iy0))
("kiyotaka" nil (k iy1 ow0 t aa1 k ax))
("kizer" nil (k ay1 z er0))
("kizziah" nil (k ih1 z iy0 ax))
("kjar" nil (k ax jh aa1 r))
("kjell" nil (k y eh1 l))
("kjellberg" nil (jh eh1 l b er0 g))
("kjenstad" nil (k y eh1 n s t ae1 d))
("kjos" nil (k ax jh aa1 s))
("klaas" nil (k l aa1 s))
("klaassen" nil (k l aa1 s ax n))
("klabunde" nil (k l ae1 b ax n d))
("kladstrup" nil (k l ae1 d s t r ax p))
("klages" nil (k l ey1 jh ih0 z))
("klahn" nil (k l ae1 n))
("klahr" nil (k l ae1 r))
("klaiber" nil (k l ey1 b er0))
("klamer" nil (k l ey1 m er0))
("klamm" nil (k l ae1 m))
("klammer" nil (k l ae1 m er0))
("klamon" nil (k l ey1 m ax n))
("klan" nil (k l ae1 n))
("klang" nil (k l ae1 ng))
("klann" nil (k l ae1 n))
("klansmen" nil (k l ae1 n z m ax n))
("klapp" nil (k l ae1 p))
("klapper" nil (k l ae1 p er0))
("klapperich" nil (k l ae1 p er0 ih0 k))
("klar" nil (k l aa1 r))
("klare" nil (k l eh1 r))
("klarich" nil (k l ae1 r ih0 k))
("klarman" nil (k l aa1 r m ax n))
("klas" nil (k l ae1 s))
("klase" nil (k l ey1 z))
("klasen" nil (k l ey1 s ax n))
("klass" nil (k l ae1 s))
("klassen" nil (k l ae1 s ax n))
("klatsky" nil (k l ae1 t s k iy0))
("klatt" nil (k l ae1 t))
("klauber" nil (k l aw1 b er0))
("klauer" nil (k l aw1 er0))
("klaus" nil (k l aw1 s))
("klauser" nil (k l aw1 s er0))
("klausing" nil (k l aw1 s ih0 ng))
("klausner" nil (k l aw1 s n er0))
("klaver" nil (k l ey1 v er0))
("klawitter" nil (k l ao1 ih0 t er0))
("klay" nil (k l ey1))
("klayman" nil (k l ey1 m ax n))
("kleban" nil (k l eh1 b ax n))
("klebba" nil (k l eh1 b ax))
("klebe" nil (k l iy1 b))
("kleber" nil (k l iy1 b er0))
("klecka" nil (k l eh1 k ax))
("klecker" nil (k l eh1 k er0))
("kleckner" nil (k l eh1 k n er0))
("klee" nil (k l iy1))
("kleeman" nil (k l iy1 m ax n))
("kleen" nil (k l iy1 n))
("kleenex" nil (k l iy1 n ax k s))
("kleer" nil (k l ih1 r))
("klees" nil (k l iy1 z))
("kleffner" nil (k l eh1 f n er0))
("kleiber" nil (k l ay1 b er0))
("kleier" nil (k l ay1 er0))
("kleiman" nil (k l ay1 m ax n))
("klein" nil (k l ay1 n))
("kleinberg" nil (k l ay1 n b er0 g))
("kleindienst" nil (k l ay1 n d iy1 n s t))
("kleine" nil (k l ay1 n))
("kleiner" nil (k l ay1 n er0))
("kleinert" nil (k l ay1 n er0 t))
("kleinfeld" nil (k l ay1 n f eh1 l d))
("kleinfelter" nil (k l ay1 n f ih0 l t er0))
("kleinhans" nil (k l ay1 n hh ax n z))
("kleinhenz" nil (k l ay1 n hh ih0 n s))
("kleinknecht" nil (k l ay1 ng k n ih0 k t))
("kleinman" nil (k l ay1 n m ax n))
("kleinpaste" nil (k l ay1 n p ey1 s t))
("kleinpeter" nil (k l ay1 n p iy0 t er0))
("kleinsasser" nil (k l ay1 n s ax s er0))
("kleinschmidt" nil (k l ay1 n sh m ih1 t))
("kleinsmith" nil (k l ay1 n s m ih0 th))
("kleinwort" nil (k l ay1 n w er0 t))
("kleis" nil (k l ay1 z))
("kleiss" nil (k l ay1 s))
("kleist" nil (k l ay1 s t))
("klejna" nil (k l ey1 n er0))
("klem" nil (k l eh1 m))
("klema" nil (k l eh1 m ax))
("kleman" nil (k l eh1 m ax n))
("klemann" nil (k l eh1 m ax n))
("klemens" nil (k l eh1 m ax n s))
("klement" nil (k l eh1 m ax n t))
("klemm" nil (k l eh1 m))
("klemme" nil (k l eh1 m))
("klemmer" nil (k l eh1 m er0))
("klemp" nil (k l eh1 m p))
("klempner" nil (k l eh1 m p n er0))
("klemz" nil (k l eh1 m z))
("klenk" nil (k l eh1 ng k))
("klenke" nil (k l eh1 ng k))
("klepac" nil (k l eh1 p ax k))
("klepfer" nil (k l eh1 p f er0))
("kleppe" nil (k l eh1 p))
("klepper" nil (k l eh1 p er0))
("kleppinger" nil (k l eh1 p ih0 ng er0))
("klerk" nil (k l er1 k))
("klesch" nil (k l eh1 sh))
("klesken" nil (k l eh1 s k ax n))
("kless" nil (k l eh1 s))
("klett" nil (k l eh1 t))
("kleve" nil (k l iy1 v))
("kleven" nil (k l iy1 v ax n))
("klever" nil (k l eh1 v er0))
("kley" nil (k l ey1))
("klich" nil (k l ih1 ch))
("klick" nil (k l ih1 k))
("kliebert" nil (k l iy1 b er0 t))
("klieg" nil (k l iy1 g))
("klier" nil (k l ay1 er0))
("kliethermes" nil (k l iy1 th er0 m z))
("kliewer" nil (k l iy1 w er0))
("kligman" nil (k l ih1 g m ax n))
("klim" nil (k l ih1 m))
("klima" nil (k l ay1 m ax))
("klimas" nil (k l ay1 m ax z))
("klimaszewski" nil (k l ih0 m ax sh eh1 f s k iy0))
("klimczak" nil (k l ih1 m ch ae0 k))
("klimek" nil (k l ih1 m ih0 k))
("kliment" nil (k l ay1 m ax n t))
("klimowicz" nil (k l ih1 m ax v ih0 ch))
("klinck" nil (k l ih1 ng k))
("klindt" nil (k l ih1 n t))
("klindworth" nil (k l ih1 n d w er0 th))
("kline" nil (k l ay1 n))
("klinedinst" nil (k l ih1 n ih0 d ih0 n s t))
("klinefelter" nil (k l ih1 n ih0 f ih0 l t er0))
("kling" nil (k l ih1 ng))
("klingaman" nil (k l ih1 ng g ax m ax n))
("klingbeil" nil (k l ih1 ng b ay0 l))
("klingberg" nil (k l ih1 ng b er0 g))
("klinge" nil (k l ih1 n jh))
("klingel" nil (k l ih1 ng g ax l))
("klingenberg" nil (k l ih1 ng ax n b er0 g))
("klingensmith" nil (k l ih1 ng g ih0 n s m ih0 th))
("klinger" nil (k l ih1 ng er0))
("klingerman" nil (k l ih1 ng er0 m ax n))
("klinghoffer" nil (k l ih1 ng hh ao1 f er0))
("klingler" nil (k l ih1 ng g ax l er0))
("klingman" nil (k l ih1 ng m ax n))
("klink" nil (k l ih1 ng k))
("klinke" nil (k l ih1 ng k))
("klinker" nil (k l ih1 ng k er0))
("klinkhammer" nil (k l ih1 ng k hh ae0 m er0))
("klinkner" nil (k l ih1 ng k n er0))
("klint" nil (k l ih1 n t))
("klintworth" nil (k l ih1 n t w er0 th))
("klinzing" nil (k l ih1 n z ih0 ng))
("klipfel" nil (k l ih1 p f ax l))
("klipp" nil (k l ih1 p))
("klippel" nil (k l ih1 p ax l))
("klish" nil (k l ih1 sh))
("klitz" nil (k l ih1 t s))
("klitzke" nil (k l ih1 t z k))
("kloberdanz" nil (k l aa1 b er0 d ax n s))
("kloc" nil (k l aa1 k))
("klocek" nil (k l ow1 s ih0 k))
("klock" nil (k l aa1 k))
("klocke" nil (k l aa1 k))
("kloeckner" nil (k l ow1 k n er0))
("kloehn" nil (k l ow1 n))
("kloepfer" nil (k l ow1 p f er0))
("kloeppel" nil (k l ow1 p ax l))
("kloepper" nil (k l ow1 p er0))
("kloiber" nil (k l oy1 b er0))
("klomp" nil (k l aa1 m p))
("klondike" nil (k l aa1 n d ay1 k))
("klonowski" nil (k l ax n ao1 f s k iy0))
("klontz" nil (k l aa1 n t s))
("kloos" nil (k l uw1 z))
("klooster" nil (k l uw1 s t er0))
("kloosterman" nil (k l uw1 s t er0 m ax n))
("klopf" nil (k l aa1 p f))
("klopfenstein" nil (k l aa1 p f ax n s t ay0 n))
("klopfer" nil (k l aa1 p f er0))
("klopp" nil (k l aa1 p))
("kloppenburg" nil (k l aa1 p ax n b er0 g))
("klos" nil (k l aa1 s))
("klose" nil (k l ow1 z))
("klosinski" nil (k l ax s ih1 n s k iy0))
("klosowski" nil (k l ax s ao1 f s k iy0))
("kloss" nil (k l ao1 s))
("klossner" nil (k l aa1 s n er0))
("kloster" nil (k l ao1 s t er0))
("klosterman" nil (k l aa1 s t er0 m ax n))
("klostermann" nil (k l aa1 s t er0 m ax n))
("kloth" nil (k l aa1 th))
("klotz" nil (k l aa1 t s))
("klotzbach" nil (k l aa1 t s b aa0 k))
("klucevsek" nil (k l uw0 s eh1 v s eh0 k))
("kluck" nil (k l ah1 k))
("kludt" nil (k l ah1 d t))
("kluender" nil (k l uh1 n d er0))
("kluesner" nil (k l uh1 s n er0))
("kluever" nil (k l uh1 v er0))
("klug" nil (k l ah1 g))
("kluge" nil (k l uw1 jh))
("kluger" nil (k l uw1 g er0))
("klugh" nil (k l ah1))
("klugman" nil (k l ah1 g m ax n))
("klugt" nil (k l ah1 t))
("klukas" nil (k l uw1 k ax z))
("klumb" nil (k l ah1 m))
("klump" nil (k l ah1 m p))
("klumpp" nil (k l ah1 m p))
("klunder" nil (k l ah1 n d er0))
("klundt" nil (k l ah1 n t))
("klunk" nil (k l ah1 ng k))
("klus" nil (k l ah1 s))
("klusman" nil (k l ah1 s m ax n))
("klute" nil (k l uw1 t))
("kluth" nil (k l uw1 th))
("klutts" nil (k l ah1 t s))
("kluttz" nil (k l ah1 t s))
("klutz" nil (k l ah1 t s))
("klutznick" nil (k l ah1 t s n ih0 k))
("kluver" nil (k l uw1 v er0))
("kluwer" nil (k l uw1 w er0))
("klux" nil (k l ah1 k s))
("klym" nil (k l ih1 m))
("klynveld" nil (k l ih1 n v eh1 l d))
("kmart" nil (k ey1 m aa1 r t))
("kmetz" nil (k ax m eh1 t s))
("kmiec" nil (k ax m iy1 k))
("kmiecik" nil (k ax m iy1 ch ih0 k))
("knaack" nil (n aa1 k))
("knaak" nil (n aa1 k))
("knab" nil (n ae1 b))
("knabb" nil (n ae1 b))
("knabe" nil (n ey1 b))
("knable" nil (n ey1 b ax l))
("knack" nil (n ae1 k))
("knacks" nil (n ae1 k s))
("knackstedt" nil (n ae1 k s t ih0 t))
("knaggs" nil (n ae1 g z))
("knape" nil (n ey1 p))
("knapik" nil (n ae1 p ih0 k))
("knapke" nil (n ey1 p k))
("knapp" nil (n ae1 p))
("knappenberger" nil (n ae1 p ax n b er0 g er0))
("knapper" nil (n ae1 p er0))
("knapton" nil (n ae1 p t ax n))
("knarr" nil (n ae1 r))
("knaub" nil (n ao1 b))
("knauer" nil (n aw1 er0))
("knauf" nil (n ao1 f))
("knauff" nil (n ao1 f))
("knaup" nil (n ao1 p))
("knaus" nil (n ao1 z))
("knauss" nil (n ao1 s))
("knave" nil (n ey1 v))
("knaves" nil (n ey1 v z))
("kneading" nil (n iy1 d ih0 ng))
("kneafsey" nil (n iy1 f s iy0))
("kneale" nil (n iy1 l))
("knebel" nil (n eh1 b ax l))
("knecht" nil (n eh1 k t))
("knechtel" nil (n eh1 k t ax l))
("knee" nil (n iy1))
("kneebone" nil (n iy1 b ow1 n))
("kneecap" nil (n iy1 k ae1 p))
("kneece" nil (n iy1 s))
("kneed" nil (n iy1 d))
("kneel" nil (n iy1 l))
("kneeland" nil (n iy1 l ax n d))
("kneeling" nil (n iy1 l ih0 ng))
("kneer" nil (n ih1 r))
("knees" nil (n iy1 z))
("kneip" nil (n iy1 p))
("kneisel" nil (n ay1 s ax l))
("kneisley" nil (n iy1 s l iy0))
("knell" nil (n eh1 l))
("kneller" nil (n eh1 l er0))
("knelt" nil (n eh1 l t))
("knepp" nil (n eh1 p))
("knepper" nil (n eh1 p er0))
("knerr" nil (n eh1 r))
("kness" nil (n eh1 s))
("knesset" nil (n eh1 s ax t))
("knew" nil (n uw1))
("knezevich" nil (n eh1 z ih0 v ih0 ch))
("knicely" nil (n ay1 s l iy0))
("knick" nil (n ih1 k))
("knickerbocker" nil (n ih1 k er0 b aa1 k er0))
("knickerbockered" nil (n ih1 k er0 b aa1 k er0 d))
("knickerbockers" nil (n ih1 k er0 b aa1 k er0 z))
("knickers" nil (n ih1 k er0 z))
("knickknack" nil (n ih1 k n ae1 k))
("knickknacks" nil (n ih1 k n ae1 k s))
("knicks" nil (n ih1 k s))
("knief" nil (n iy1 f))
("kniep" nil (n iy1 p))
("knieriem" nil (n iy1 r iy1 m))
("knierim" nil (n ih1 r ih0 m))
("knies" nil (n ay1 z))
("kniess" nil (n iy1 s))
("knievel" nil (k ax n iy1 v ax l))
("knife" nil (n ay1 f))
("knifelike" nil (n ay1 f l ay1 k))
("kniffen" nil (n ih1 f ax n))
("kniffin" nil (n ih1 f ih0 n))
("knigge" nil (n ih1 g))
("knight" nil (n ay1 t))
("knighten" nil (n ay1 t ax n))
("knighthood" nil (n ay1 t hh uh1 d))
("knightly" nil (n ay1 t l iy0))
("knighton" nil (n ay1 t ax n))
("knights" nil (n ay1 t s))
("knill" nil (n ih1 l))
("knipe" nil (n ay1 p))
("knipfer" nil (n ih1 p f er0))
("knipp" nil (n ih1 p))
("knippa" nil (n ih1 p ax))
("knippel" nil (n ih1 p ax l))
("knippenberg" nil (n ih1 p ax n b er0 g))
("knipper" nil (n ih1 p er0))
("knipple" nil (n ih1 p ax l))
("knisely" nil (n ay1 z l iy0))
("kniskern" nil (n ih1 s k er0 n))
("knisley" nil (n ih1 s l iy0))
("knispel" nil (n ih1 s p ax l))
("kniss" nil (n ih1 s))
("knit" nil (n ih1 t))
("knits" nil (n ih1 t s))
("knitted" nil (n ih1 t ax d))
("knittel" nil (n ih1 t ax l))
("knitter" nil (n ih1 t er0))
("knitting" nil (n ih1 t ih0 ng))
("knittle" nil (n ih1 t ax l))
("knitwear" nil (n ih1 t w eh1 r))
("knives" nil (n ay1 v z))
("knob" nil (n aa1 b))
("knobbe" nil (n aa1 b))
("knobby" nil (n aa1 b iy0))
("knobel" nil (n ow1 b ax l))
("knoblauch" nil (n aa1 b l aw0 k))
("knoble" nil (n ow1 b ax l))
("knobloch" nil (n aa1 b l ax k))
("knoblock" nil (n aa1 b l aa0 k))
("knobs" nil (n aa1 b z))
("knoch" nil (n aa1 k))
("knoche" nil (n aa1 ch))
("knochel" nil (n aa1 k ax l))
("knock" nil (n aa1 k))
("knockdown" nil (n aa1 k d aw1 n))
("knocked" nil (n aa1 k t))
("knocking" nil (n aa1 k ih0 ng))
("knockoff" nil (n aa1 k ao1 f))
("knockoffs" nil (n aa1 k ao1 f s))
("knockout" nil (n aa1 k aw1 t))
("knocks" nil (n aa1 k s))
("knode" nil (n ow1 d))
("knodel" nil (n ow1 d ax l))
("knodle" nil (n ow1 d ax l))
("knoebel" nil (n ow1 b ax l))
("knoedler" nil (n ow1 d ax l er0))
("knoell" nil (n ow1 l))
("knoff" nil (n ao1 f))
("knogo" nil (n ow1 g ow0))
("knoke" nil (n ow1 k))
("knoles" nil (n ow1 l z))
("knoll" nil (n ow1 l))
("knoop" nil (n uw1 p))
("knop" nil (n aa1 p))
("knope" nil (n ow1 p))
("knopf" nil (n aa1 p f))
("knopp" nil (n aa1 p))
("knorr" nil (n ao1 r))
("knost" nil (n aa1 s t))
("knot" nil (n aa1 t))
("knotek" nil (n ow1 t ih0 k))
("knoth" nil (n aa1 th))
("knots" nil (n aa1 t s))
("knott" nil (n aa1 t))
("knotted" nil (n aa1 t ih0 d))
("knotts" nil (n aa1 t s))
("knotty" nil (n aa1 t iy0))
("knouff" nil (n ow1 f))
("knous" nil (n ao1 s))
("knouse" nil (n aw1 s))
("know" nil (n ow1))
("knowed" nil (n ow1 d))
("knower" nil (n ow1 er0))
("knowing" nil (n ow1 ih0 ng))
("knowingly" nil (n ow1 ih0 ng l iy0))
("knowledge" nil (n aa1 l ax jh))
("knowledgeable" nil (n aa1 l ax jh ax b ax l))
("knowledgeably" nil (n aa1 l ih0 jh ax b l iy0))
("knowledgeware" nil (n aa1 l ih0 jh w eh1 r))
("knowles" nil (n ow1 l z))
("knowlton" nil (n ow1 l t ax n))
("known" nil (n ow1 n))
("knows" nil (n ow1 z))
("knox" nil (n aa1 k s))
("knoxville" nil (n aa1 k s v ih0 l))
("knoy" nil (n oy1))
("knuckle" nil (n ah1 k ax l))
("knuckles" nil (n ah1 k ax l z))
("knudsen" nil (n uw1 d s ax n))
("knudson" nil (n uw1 d s ax n))
("knudtson" nil (n uw1 t s ax n))
("knueppel" nil (n uw1 p ax l))
("knupp" nil (n ah1 p))
("knust" nil (n ah1 s t))
("knut" nil (n ah1 t))
("knuth" nil (n uw1 th))
("knuts" nil (n ah1 t s))
("knutsen" nil (n ah1 t s ax n))
("knutson" nil (k n uw1 t s ax n))
("knutzen" nil (n ah1 t z ax n))
("ko" nil (k ow1))
("koala" nil (k ow0 aa1 l ax))
("koba" nil (k ow1 b ax))
("kobacker" nil (k ow1 b ae1 k er0))
("kobak" nil (k ow1 b ax k))
("kobayashi" nil (k ow1 b aa0 y aa1 sh iy0))
("kobe" nil (k ow1 b))
("kobel" nil (k ow1 b ax l))
("kober" nil (k ow1 b er0))
("koberstein" nil (k ow1 b er0 s t ay0 n))
("kobes" nil (k ow1 b z))
("koble" nil (k ow1 b ax l))
("kobler" nil (k ow1 b ax l er0))
("kobren" nil (k aa1 b r ax n))
("kobrin" nil (k aa1 b r ih0 n))
("kobs" nil (k aa1 b z))
("kobus" nil (k ow1 b ax s))
("koby" nil (k ow1 b iy0))
("kobylarz" nil (k ax b ih1 l er0 z))
("kobylinski" nil (k ax b ih0 l ih1 n s k iy0))
("kobza" nil (k aa1 b z ax))
("kocak" nil (k ow1 k ax k))
("koch" nil (k ao1 ch))
("kochan" nil (k aa1 k ax n))
("kochanek" nil (k aa1 k ax n ih0 k))
("kochanowski" nil (k ax hh ax n ao1 f s k iy0))
("kochanski" nil (k ax hh ae1 n s k iy0))
("kochel" nil (k aa1 k ax l))
("kochendorfer" nil (k aa1 k ih0 n d ao0 r f er0))
("kocher" nil (k ao1 ch er0))
("kochevar" nil (k ax hh eh1 v er0))
("kochis" nil (k aa1 k ih0 s))
("kochman" nil (k aa1 k m ax n))
("koci" nil (k ow1 s iy0))
("kocian" nil (k ow1 sh ax n))
("kociemba" nil (k ow1 s iy0 eh1 m b ax))
("kocinski" nil (k ax ch ih1 n s k iy0))
("kociolek" nil (k ow1 s iy0 ow1 l eh0 k))
("kocis" nil (k ow1 s ih0 s))
("kock" nil (k aa1 k))
("kocourek" nil (k ax k uh1 r eh0 k))
("kocsis" nil (k aa1 k s ih0 s))
("kocur" nil (k ow1 k er0))
("kocurek" nil (k ax k y uw1 r eh0 k))
("kodacolor" nil (k ow1 d ax k ah1 l er0))
("kodak" nil (k ow1 d ae1 k))
("kodama" nil (k ow0 d aa1 m ax))
("kodiak" nil (k ow1 d iy0 ae1 k))
("kodo" nil (k ow1 d ow0))
("koebel" nil (k ow1 b ax l))
("koedinger" nil (k ow1 d ih0 ng er0))
("koegel" nil (k ow1 g ax l))
("koegler" nil (k ow1 g ax l er0))
("koehl" nil (k ow1 l))
("koehler" nil (k ow1 l er0))
("koehn" nil (k ow1 n))
("koehne" nil (k ow1 n))
("koehnen" nil (k ow1 n ax n))
("koeki" nil (k ow1 k iy0))
("koelle" nil (k ow1 l))
("koeller" nil (k ow1 l er0))
("koelling" nil (k ow1 l ih0 ng))
("koelsch" nil (k ow1 l sh))
("koelzer" nil (k ow1 l z er0))
("koen" nil (k ow1 n))
("koeneman" nil (k aa1 iy0 n m ax n))
("koenen" nil (k ow1 n ax n))
("koenig" nil (k er1 n ih0 g))
("koenigs" nil (k er1 n ih0 g z))
("koenigsberg" nil (k ow1 n ih0 g z b er0 g))
("koenigsfeld" nil (k ow1 n ih0 g z f eh1 l d))
("koep" nil (k ow1 p))
("koepke" nil (k ow1 p k))
("koepp" nil (k ow1 p))
("koeppe" nil (k ow1 p))
("koeppel" nil (k ow1 p ax l))
("koeppen" nil (k ow1 p ax n))
("koepsel" nil (k ow1 p s ax l))
("koepsell" nil (k ow1 p s ax l))
("koerber" nil (k ao1 r b er0))
("koerner" nil (k ao1 r n er0))
("koester" nil (k ow1 s t er0))
("koesters" nil (k ow1 s t er0 z))
("koestler" nil (k ow1 s ax l er0))
("koestner" nil (k ow1 s t n er0))
("koether" nil (k ow1 dh er0))
("koetje" nil (k ow1 t jh))
("koetter" nil (k ow1 t er0))
("koetting" nil (k ow1 t ih0 ng))
("koff" nil (k ao1 f))
("koffler" nil (k aa1 f ax l er0))
("koffman" nil (k ao1 f m ax n))
("kofler" nil (k ow1 f ax l er0))
("kofoed" nil (k ow1 f ow0 d))
("kofron" nil (k aa1 f r ax n))
("koga" nil (k ow1 g ax))
("kogan" nil (k ow1 g ax n))
("kogel" nil (k ow1 g ax l))
("koger" nil (k ow1 g er0))
("kogler" nil (k ow1 g ax l er0))
("koglin" nil (k aa1 g l ih0 n))
("kogut" nil (k ow1 g ax t))
("kogyo" nil (k ow1 jh y ow0))
("koh" nil (k ow1))
("kohan" nil (k ow1 hh aa0 n))
("kohen" nil (k ow1 ax n))
("kohl" nil (k ow1 l))
("kohlbeck" nil (k ow1 l b eh1 k))
("kohlberg" nil (k ow1 l b er0 g))
("kohlenberg" nil (k ow1 l ax n b er0 g))
("kohler" nil (k ow1 l er0))
("kohles" nil (k ow1 hh ax l z))
("kohlhepp" nil (k ow1 l hh ih0 p))
("kohlhoff" nil (k ow1 l hh ao1 f))
("kohli" nil (k ow1 l iy0))
("kohlman" nil (k ow1 l m ax n))
("kohlmann" nil (k ow1 l m ax n))
("kohlmeier" nil (k ow1 l m ay0 er0))
("kohlmeyer" nil (k ow1 l m ay0 er0))
("kohls" nil (k ow1 l z))
("kohn" nil (k aa1 n))
("kohne" nil (k ow1 n))
("kohnen" nil (k ow1 n ax n))
("kohner" nil (k ow1 n er0))
("kohnke" nil (k aa1 ng k))
("kohout" nil (k ow1 aw0 t))
("kohr" nil (k ao1 r))
("kohring" nil (k ao1 r ih0 ng))
("kohrs" nil (k ao1 r z))
("kohtoro" nil (k ow0 t ao1 r ow0))
("kohut" nil (k ow1 ax t))
("koichi" nil (k ow0 iy1 ch iy0))
("koido" nil (k oy1 d ow0))
("koike" nil (k oy1 k))
("koito" nil (k oy1 t ow0))
("koivisto" nil (k oy1 v ih1 s t ow0))
("koji" nil (k ow1 jh iy0))
("kojima" nil (k ax y ay1 m ax))
("kok" nil (k aa1 k))
("kokan" nil (k ow1 k ax n))
("kokate" nil (k ow1 k aa1 t ey1))
("koke" nil (k ow1 k))
("koken" nil (k ow1 k ax n))
("koker" nil (k ow1 k er0))
("kokes" nil (k ow1 k s))
("kokesh" nil (k aa1 k ih0 sh))
("kokinda" nil (k ax k ih1 n d ax))
("koko" nil (k ow1 k ow0))
("kokomo" nil (k ow1 k ax m ow1))
("kokoschka" nil (k ax k ao1 sh k ax))
("kokoszka" nil (k ax k aa1 sh k ax))
("kokusai" nil (k aa1 k y uw0 s ay1))
("kol" nil (k ow1 l))
("kolakowski" nil (k ax l ax k ao1 f s k iy0))
("kolander" nil (k aa1 l ax n d er0))
("kolar" nil (k ow1 l er0))
("kolarik" nil (k ax l aa1 r ih0 k))
("kolasa" nil (k ax l aa1 s ax))
("kolasinski" nil (k ax l ax s ih1 n s k iy0))
("kolb" nil (k ow1 l b))
("kolbe" nil (k ow1 l b))
("kolbeck" nil (k aa1 l b eh0 k))
("kolber" nil (k ow1 l b er0))
("kolberg" nil (k ow1 l b er0 g))
("kolbo" nil (k ow1 l b ow0))
("kolden" nil (k ow1 l d ax n))
("kole" nil (k ow1 l))
("kolek" nil (k ow1 l eh0 k))
("kolenda" nil (k ax l eh1 n d ax))
("kolesar" nil (k ax l eh1 s er0))
("kolich" nil (k aa1 l ih0 hh))
("kolin" nil (k ow1 l ih0 n))
("kolinski" nil (k ax l ih1 n s k iy0))
("kolk" nil (k ow1 k))
("kolka" nil (k ow1 l k ax))
("kolker" nil (k ow1 k er0))
("koll" nil (k aa1 l))
("kollar" nil (k aa1 l er0))
("kollasch" nil (k aa1 l ax sh))
("kollath" nil (k aa1 l ax th))
("kolle" nil (k aa1 l))
("koller" nil (k aa1 l er0))
("kolling" nil (k aa1 l ih0 ng))
("kollman" nil (k aa1 l m ax n))
("kollmann" nil (k aa1 l m ax n))
("kollmeyer" nil (k aa1 l m ay0 er0))
("kollmorgen" nil (k ow0 l m ao1 r g ax n))
("kolm" nil (k ow1 m))
("kolman" nil (k aa1 l m ax n))
("kolodny" nil (k ax l aa1 d n iy0))
("kolodziej" nil (k ax l aa1 d z iy0 jh))
("kolodziejski" nil (k ax l aa1 jh iy0 ey1 s k iy0))
("kolojejchick" nil (k ow1 l ow0 jh ey1 ch ih0 k))
("kolokoff" nil (k aa1 l ax k ao1 f))
("koloski" nil (k ax l aw1 s k iy0))
("kolowich" nil (k aa1 l ax w ih0 ch))
("kolp" nil (k ow1 l p))
("kolski" nil (k ow1 l s k iy0))
("kolsky" nil (k ow1 l s k iy0))
("kolstad" nil (k ow1 l s t ax d))
("kolter" nil (k ow1 l t er0))
("kolterman" nil (k ow1 l t er0 m ax n))
("kolton" nil (k ow1 l t ax n))
("koltz" nil (k ow1 l t s))
("komaba" nil (k ow0 m aa1 b ax))
("komag" nil (k ow1 m ae0 g))
("koman" nil (k ow1 m ax n))
("komansky" nil (k ow0 m ae1 n s k iy0))
("komar" nil (k ow1 m er0))
("komara" nil (k ow0 m aa1 r ax))
("komarek" nil (k ow0 m aa1 r eh0 k))
("komarik" nil (k ow0 m aa1 r ih0 k))
("komatsu" nil (k ow0 m aa1 t s uw1))
("kombat" nil (k aa1 m b ae0 t))
("komer" nil (k ow1 m er0))
("komercni" nil (k ow1 m er1 ch n iy0))
("komi" nil (k ow1 m iy0))
("komine" nil (k ow1 m ay1 n))
("komineft" nil (k aa1 m ih0 n eh0 f t))
("komisar" nil (k ax m ih1 s er0))
("kommer" nil (k aa1 m er0))
("komori" nil (k ow0 m ao1 r iy0))
("komorny" nil (k ow0 m ao1 r n iy0))
("komorowski" nil (k ax m er0 ao1 f s k iy0))
("komoto" nil (k ow0 m ow1 t ow0))
("komp" nil (k aa1 m p))
("komsomol" nil (k aa1 m s ow0 m ax l))
("komura" nil (k ow1 m uh1 r ax))
("kon" nil (k aa1 n))
("konarski" nil (k ax n aa1 r s k iy0))
("konczal" nil (k aa1 n ch ax l))
("kondas" nil (k aa1 n d ax z))
("kondo" nil (k aa1 n d ow0))
("kondracki" nil (k ax n d r aa1 t s k iy0))
("kondrat" nil (k aa1 n d r ax t))
("konecny" nil (k ax n eh1 k n iy0))
("konen" nil (k aa1 n ax n))
("kong" nil (k ao1 ng))
("kongsberg" nil (k ao1 ng z b er0 g))
("konica" nil (k aa1 n ih0 k ax))
("konicek" nil (k aa1 n ih0 ch eh1 k))
("konicki" nil (k ax n ih1 t s k iy0))
("konieczka" nil (k ax n iy1 ch k ax))
("konieczny" nil (k ax n iy1 ch n iy0))
("konig" nil (k aa1 n ih0 g))
("koning" nil (k ow1 n ih0 ng))
("koninklijke" nil (k aa1 n ih0 ng k l iy1 k iy0))
("konishi" nil (k ow0 n iy1 sh iy0))
("konishiroku" nil (k aa1 n ih1 sh ih0 r ow1 k uw1))
("konitzer" nil (k aa1 n ih0 t z er0))
("konkel" nil (k aa1 ng k ax l))
("konkle" nil (k aa1 ng k ax l))
("konkol" nil (k aa1 ng k ao0 l))
("konner" nil (k aa1 n er0))
("kono" nil (k ow1 n ow0))
("konold" nil (k aa1 n ow0 l d))
("konop" nil (k ow1 n ax p))
("konopka" nil (k ax n ow1 p k ax))
("konrad" nil (k aa1 n r ax d))
("konrath" nil (k aa1 n r ax th))
("konstantin" nil (k aa1 n s t ih0 n t iy1 n))
("konsultat" nil (k aa1 n s ax l t aa1 t))
("kontos" nil (k aa1 n t ow0 z))
("kontras" nil (k aa1 n t r ax s))
("kontrollbank" nil (k aa1 n t r ax l b ae1 ng k))
("konwinski" nil (k ax n v ih1 n s k iy0))
("konz" nil (k aa1 n z))
("konzen" nil (k aa1 n z ax n))
("konzi" nil (k aa1 n z iy0))
("koo" nil (k uw1))
("koob" nil (k uw1 b))
("koogler" nil (k uw1 g l er0))
("kooi" nil (k uw1 iy0))
("kooiker" nil (k uw1 ih0 k er0))
("kooiman" nil (k uw1 ih0 m ax n))
("kooistra" nil (k uw0 ih1 z t r ax))
("kook" nil (k uh1 k))
("kooken" nil (k uw1 k ax n))
("kooker" nil (k uh1 k er0))
("kooks" nil (k uh1 k s))
("kool" nil (k uw1 l))
("koon" nil (k uw1 n))
("koonce" nil (k uw1 n s))
("koone" nil (k uw1 n))
("kooning" nil (k uw1 n ih0 ng))
("koons" nil (k uw1 n z))
("koonts" nil (k uw1 n t s))
("koontz" nil (k uw1 n t s))
("koop" nil (k uw1 p))
("koopman" nil (k uw1 p m ax n))
("koopmann" nil (k uw1 p m ax n))
("koops" nil (k uw1 p s))
("koor" nil (k uw1 r))
("koors" nil (k uh1 r z))
("koos" nil (k uw1 z))
("kooser" nil (k uw1 z er0))
("kootenay" nil (k uw1 t ax n ey1))
("kooy" nil (k uw1 iy0))
("kooyman" nil (k aa1 oy0 m ax n))
("kopacz" nil (k aa1 p ax ch))
("kopas" nil (k ow1 p ax z))
("kopczynski" nil (k ax p ch ih1 n s k iy0))
("kope" nil (k ow1 p))
("kopec" nil (k ow1 p ih0 k))
("kopecky" nil (k ax p eh1 t s k iy0))
("kopeks" nil (k ow1 p ax k s))
("kopel" nil (k ow1 p ax l))
("kopelman" nil (k ow1 p ax l m ax n))
("koper" nil (k ow1 p er0))
("kopera" nil (k ax p ih1 r ax))
("koperski" nil (k ax p er1 s k iy0))
("kopetski" nil (k ax p eh1 t s k iy0))
("kopf" nil (k ao1 p f))
("kopinski" nil (k ax p ih1 n s k iy0))
("kopischke" nil (k ax p ih1 sh k iy0))
("kopit" nil (k aa1 p ih0 t))
("kopka" nil (k ow1 p k ax))
("kopke" nil (k ow1 p k))
("kopko" nil (k ow1 p k ow0))
("koplan" nil (k aa1 p l ax n))
("koplin" nil (k aa1 p l ih0 n))
("koplovitz" nil (k aa1 p l ax v ih0 t s))
("kopp" nil (k aa1 p))
("koppe" nil (k aa1 p))
("koppel" nil (k aa1 p ax l))
("koppell" nil (k aa1 p ax l))
("koppelman" nil (k aa1 p ax l m ax n))
("koppen" nil (k aa1 p ax n))
("koppenhaver" nil (k aa1 p ih0 n hh ax v er0))
("kopper" nil (k aa1 p er0))
("koppers" nil (k aa1 p er0 z))
("koppes" nil (k aa1 p s))
("kopplin" nil (k aa1 p l ih0 n))
("kopriva" nil (k aa1 p r ih0 v ax))
("koprowski" nil (k ax p r ao1 f s k iy0))
("kops" nil (k aa1 p s))
("kopy" nil (k aa1 p iy0))
("kora" nil (k ao1 r ax))
("koral" nil (k ao1 r ax l))
("koran" nil (k ao0 r aa1 n))
("koranda" nil (k er0 ae1 n d ax))
("koranic" nil (k ao0 r ae1 n ih0 k))
("korb" nil (k ao1 r b))
("korba" nil (k ao1 r b ax))
("korbel" nil (k ao1 r b ax l))
("korber" nil (k ao1 r b er0))
("korbren" nil (k ao1 r b r eh0 n))
("korby" nil (k ao1 r b iy0))
("korczak" nil (k ao1 r ch ae0 k))
("korda" nil (k ao1 r d ax))
("korea" nil (k ao0 r iy1 ax))
("koreagate" nil (k ao0 r iy1 ax g ey1 t))
("korean" nil (k ao0 r iy1 ax n))
("koreans" nil (k ao0 r iy1 ax n z))
("koreas" nil (k ao1 r iy0 ax z))
("korell" nil (k ao1 r ax l))
("koren" nil (k ao1 r ax n))
("korenek" nil (k ao1 r ih0 n ih0 k))
("koresh" nil (k ao1 r eh1 sh))
("koret" nil (k ao1 r ax t))
("korey" nil (k ao1 r iy0))
("korf" nil (k ao1 r f))
("korff" nil (k ao1 r f))
("korfhage" nil (k ao1 r f hh ih0 jh))
("korhonen" nil (k ao1 r hh ax n ax n))
("korinek" nil (k ao1 r ih0 n ih0 k))
("korman" nil (k ao1 r m ax n))
("kormos" nil (k ao1 r m ow0 z))
("korn" nil (k ao1 r n))
("kornacki" nil (k er0 n aa1 t s k iy0))
("kornberg" nil (k ao1 r n b er0 g))
("kornblum" nil (k ao1 r n b l ax m))
("kornbluth" nil (k ao1 r n b l uw0 th))
("kornegay" nil (k ao1 r n ih0 g ey0))
("korner" nil (k ao1 r n er0))
("kornfeld" nil (k ao1 r n f eh0 l d))
("kornhauser" nil (k ao1 r n hh aw0 z er0))
("kornreich" nil (k ao1 r n r ay0 k))
("korns" nil (k ao1 n z))
("korol" nil (k ao1 r ao0 l))
("korologos" nil (k ao0 r aa1 l ow1 g ow0 s))
("korona" nil (k er0 ow1 n ax))
("korotich" nil (k ao1 r ax t ih0 ch))
("korpela" nil (k er0 p iy1 l ax))
("korpi" nil (k ao1 r p iy0))
("korry" nil (k ao1 r iy0))
("korson" nil (k ao1 r s ax n))
("kort" nil (k ao1 r t))
("korte" nil (k ao1 r t))
("korten" nil (k ao1 r t ax n))
("korth" nil (k ao1 r th))
("korthals" nil (k ao1 r th ax l z))
("kortman" nil (k ao1 r t m ax n))
("kortum" nil (k ao1 r t ax m))
("kortz" nil (k ao1 r t s))
("koruna" nil (k ao0 r uw1 n ax))
("korver" nil (k ao1 r v er0))
("kory" nil (k ao1 r iy0))
("koryagin" nil (k ao1 r y aa1 g ih0 n))
("korzeniewski" nil (k er0 z ih1 n iy0 eh1 f s k iy0))
("kos" nil (k aa1 s))
("kosa" nil (k ow1 s ax))
("kosak" nil (k ow1 s ax k))
("kosakowski" nil (k ax s ax k ao1 f s k iy0))
("kosan" nil (k ow1 z ax n))
("kosanke" nil (k aa1 s ax ng k))
("kosanovich" nil (k ax s ae1 n ax v ih0 ch))
("kosar" nil (k ow1 s er0))
("kosberg" nil (k ao1 z b er0 g))
("kosbie" nil (k aa1 z b iy0))
("kosch" nil (k ao1 sh))
("koscielniak" nil (k ax s ch iy1 l n iy0 ae0 k))
("koscinski" nil (k ax s ch ih1 n s k iy0))
("kosco" nil (k ow1 s k ow0))
("kosecoff" nil (k ow1 s ax k ao0 f))
("kosek" nil (k ow1 s eh0 k))
("kosel" nil (k ow1 s ax l))
("koser" nil (k ow1 z er0))
("kosh" nil (k aa1 sh))
("kosher" nil (k ow1 sh er0))
("kosiba" nil (k ow0 s iy1 b ax))
("kosier" nil (k ow1 s iy0 er0))
("kosik" nil (k ow1 s ih0 k))
("kosin" nil (k ow1 s ih0 n))
("kosinski" nil (k ax s ih1 n s k iy0))
("koska" nil (k ow1 s k ax))
("koskela" nil (k ax s k iy1 l ax))
("koskey" nil (k aa1 s k iy0))
("koski" nil (k aw1 s k iy0))
("koskinen" nil (k aa1 s k ih0 n ax n))
("kosko" nil (k ow1 s k ow0))
("koskotas" nil (k ao1 s k ow1 t ax s))
("kosky" nil (k aa1 s k iy0))
("kosloski" nil (k ax s l aw1 s k iy0))
("koslosky" nil (k ax s l ow1 s k iy0))
("koslow" nil (k aa1 s l ow0))
("koslowski" nil (k ax s l ao1 f s k iy0))
("kosman" nil (k aa1 s m ax n))
("kosmatka" nil (k ax s m aa1 t k ax))
("kosmetsky" nil (k aa1 z m eh1 t s k iy0))
("kosmicki" nil (k ax s m ih1 t s k iy0))
("kosnovsky" nil (k aa1 z n aa1 f s k iy0))
("kosovo" nil (k ow1 s ow0 v ow1))
("kosowski" nil (k ax s ao1 f s k iy0))
("kosowsky" nil (k ax s aw1 s k iy0))
("koss" nil (k ao1 s))
("kossman" nil (k ao1 s m ax n))
("kossow" nil (k aa1 s ow0))
("kost" nil (k aa1 s t))
("kosta" nil (k ow1 s t ax))
("kostal" nil (k aa1 s t ax l))
("kostas" nil (k ow1 s t ax z))
("kostecki" nil (k ax s t eh1 t s k iy0))
("kostek" nil (k aa1 s t eh0 k))
("kostelecky" nil (k ax s t eh0 l eh1 t s k iy0))
("kostelnik" nil (k ax s t eh1 l n ih0 k))
("koster" nil (k aa1 s t er0))
("kostic" nil (k aa1 s t ih0 k))
("kostick" nil (k ow1 s t ih0 k))
("kostka" nil (k aa1 s t k ax))
("kostmayer" nil (k ao1 s t m ey1 er0))
("kostoff" nil (k aa1 s t ao0 f))
("kostrzewa" nil (k ax s t r z uw1 ax))
("kostrzewski" nil (k ao1 s t er0 z eh1 f s k iy0))
("kosub" nil (k ow1 s ax b))
("kosy" nil (k ow1 s iy0))
("kosyakov" nil (k ow1 s y ax k aa0 v))
("kosygin" nil (k ow1 s ih0 g ih0 n))
("kot" nil (k aa1 t))
("kotara" nil (k ow0 t aa1 r ax))
("kotarski" nil (k ax t aa1 r s k iy0))
("kotas" nil (k ow1 t ax z))
("kotch" nil (k aa1 ch))
("kote" nil (k ow1 t))
("kotecki" nil (k ax t eh1 t s k iy0))
("kotek" nil (k ow1 t eh1 k))
("koteles" nil (k aa1 t eh0 l eh0 z))
("koth" nil (k aa1 th))
("kothari" nil (k aa1 th er0 iy0))
("kothe" nil (k ow1 dh))
("kotila" nil (k ax t ay1 l ax))
("kotlarz" nil (k aa1 t l er0 z))
("kotler" nil (k ow1 t ax l er0))
("kotlowitz" nil (k aa1 t l ax w ih0 t s))
("kotowski" nil (k ax t ao1 f s k iy0))
("kotsonis" nil (k ae1 t s ow1 n ax s))
("kott" nil (k aa1 t))
("kotter" nil (k aa1 t er0))
("kottke" nil (k aa1 t k iy0))
("kottler" nil (k aa1 t l er0))
("kottwitz" nil (k aa1 t w ih0 t s))
("kotula" nil (k ax t uw1 l ax))
("kotz" nil (k aa1 t s))
("kouba" nil (k uw1 b ax))
("koudelka" nil (k aw0 d eh1 l k ax))
("kough" nil (k aw1))
("kouns" nil (k aw1 n z))
("kountz" nil (k aw1 n t s))
("kouri" nil (k ow0 uh1 r iy0))
("kouril" nil (k uw1 r ax l))
("kourou" nil (k uw1 r uw1))
("koury" nil (k aw1 r iy0))
("kovac" nil (k ow1 v ax k))
("kovacevic" nil (k ax v ax ch eh1 v ih0 k))
("kovacevich" nil (k ax v aa1 ch ih0 v ih0 ch))
("kovach" nil (k ow1 v aa0 k))
("kovacic" nil (k ax v aa1 k ih0 k))
("kovacich" nil (k ax v aa1 ch ih0 hh))
("kovacik" nil (k aa1 v ax ch ih0 k))
("kovack" nil (k aa1 v ax k))
("kovacs" nil (k ow1 v ae0 k s))
("koval" nil (k ow1 v ax l))
("kovalchik" nil (k ax v aa1 l hh ih0 k))
("kovalcik" nil (k aa1 v ax l ch ih0 k))
("kovaleski" nil (k ax v ax l eh1 s k iy0))
("kovalik" nil (k ax v aa1 l ih0 k))
("kovalsky" nil (k ax v aa1 l s k iy0))
("kovar" nil (k ow1 v er0))
("kovarik" nil (k ax v aa1 r ih0 k))
("kovatch" nil (k aa1 v ax ch))
("kovats" nil (k ow1 v ax t s))
("koven" nil (k ow1 v ax n))
("kover" nil (k ow1 v er0))
("koversada" nil (k ah1 v er0 s aa1 t ax))
("kovich" nil (k aa1 v ih0 ch))
("kowal" nil (k aw1 ax l))
("kowalchuk" nil (k aw0 aa1 l hh ax k))
("kowalczyk" nil (k aw1 ax l ch ih0 k))
("kowaleski" nil (k aw0 ax l eh1 s k iy0))
("kowalewski" nil (k aw0 ax l eh1 f s k iy0))
("kowalik" nil (k aw0 aa1 l ih0 k))
("kowalke" nil (k aa1 w ao1 k))
("kowalkowski" nil (k aa1 w ao0 k ao1 f s k iy0))
("kowall" nil (k aw1 ax l))
("kowalski" nil (k ax w aa1 l s k iy0))
("kowalsky" nil (k aw0 aa1 l s k iy0))
("kowitz" nil (k aa1 w ih0 t s))
("kownacki" nil (k aw1 n aa1 k iy0))
("koyama" nil (k ow0 y aa1 m ax))
("koyo" nil (k oy1 ow0))
("koza" nil (k ow1 z ax))
("kozak" nil (k ow1 z ax k))
("kozakiewicz" nil (k ax z aa1 k ax v ih0 ch))
("kozar" nil (k ow1 z er0))
("kozberg" nil (k aa1 z b er0 g))
("kozel" nil (k ow1 z ax l))
("kozicki" nil (k ax z ih1 t s k iy0))
("koziel" nil (k aa1 z iy0 l))
("kozik" nil (k ow1 z ih0 k))
("kozikowski" nil (k ax z ih0 k ao1 f s k iy0))
("kozinski" nil (k ax z ih1 n s k iy0))
("koziol" nil (k aa1 z iy0 ao0 l))
("kozlik" nil (k aa1 z l ih0 k))
("kozloff" nil (k aa1 z l ao0 f))
("kozloski" nil (k ax z l aw1 s k iy0))
("kozlow" nil (k aa1 z l ow0))
("kozlowski" nil (k ax z l ao1 f s k iy0))
("kozma" nil (k ow1 z m ax))
("kozminski" nil (k ax z m ih1 n s k iy0))
("koznovsky" nil (k aa1 z n aa1 f s k iy0))
("kozo" nil (k ow1 z ow0))
("kozol" nil (k ow1 z ax l))
("kozub" nil (k ow1 z ax b))
("kozuch" nil (k aa1 z ax hh))
("kozyrev" nil (k aa1 z er0 eh1 v))
("kraai" nil (k r aa1 iy0))
("kraatz" nil (k r aa1 t s))
("krabbe" nil (k r ae1 b))
("krabbenhoft" nil (k r ae1 b ih0 n hh ax f t))
("krabill" nil (k r ae1 b ax l))
("krach" nil (k r ae1 ch))
("kracht" nil (k r ae1 k t))
("krack" nil (k r ae1 k))
("kracke" nil (k r ae1 k))
("kraeger" nil (k r eh1 g er0))
("kraemer" nil (k r eh1 m er0))
("kraeutler" nil (k r aw1 t l er0))
("krafft" nil (k r ae1 f t))
("kraft" nil (k r ae1 f t))
("kraftwerk" nil (k r ae1 f t w er0 k))
("krage" nil (k r ey1 jh))
("krager" nil (k r ey1 g er0))
("kragh" nil (k r ae1 g))
("kragt" nil (k r ae1 g t))
("krah" nil (k r aa1))
("krahenbuhl" nil (k r aa1 ih0 n b ax l))
("krahl" nil (k r aa1 l))
("krahn" nil (k r ae1 n))
("krain" nil (k r ey1 n))
("krajewski" nil (k r ay0 eh1 f s k iy0))
("krajicek" nil (k r ay1 ih0 ch eh0 k))
("krajina" nil (k r ay1 n ax))
("kraker" nil (k r ey1 k er0))
("krakow" nil (k r aa1 k aw0))
("krakower" nil (k r ae1 k ow0 er0))
("krakowski" nil (k r ax k ao1 f s k iy0))
("kral" nil (k r ae1 l))
("kralicek" nil (k r aa1 l ih0 ch eh0 k))
("kralik" nil (k r aa1 l ih0 k))
("krall" nil (k r ao1 l))
("kram" nil (k r ae1 m))
("kramar" nil (k r ae1 m er0))
("kramer" nil (k r ey1 m er0))
("kramlich" nil (k r ae1 m l ih0 k))
("kramm" nil (k r ae1 m))
("kramme" nil (k r ae1 m))
("krammer" nil (k r ae1 m er0))
("krammes" nil (k r ae1 m z))
("kramp" nil (k r ae1 m p))
("krampe" nil (k r ae1 m p))
("krane" nil (k r ey1 n))
("kraner" nil (k r ey1 n er0))
("kranich" nil (k r ae1 n ih0 ch))
("krans" nil (k r ae1 n z))
("krantz" nil (k r ae1 n t s))
("kranz" nil (k r ae1 n z))
("kranzler" nil (k r ae1 n z l er0))
("krapels" nil (k r ae1 p ax l z))
("krapf" nil (k r ae1 p f))
("krapp" nil (k r ae1 p))
("kras" nil (k r ae1 s))
("krasinski" nil (k r ax s ih1 n s k iy0))
("kraska" nil (k r aa1 s k ax))
("krasner" nil (k r ae1 s n er0))
("krasnoff" nil (k r ae1 s n ao0 f))
("krasnow" nil (k r aa1 s n ow0))
("krasnoyarsk" nil (k r ae0 s n oy1 aa0 r s k))
("krasny" nil (k r ae1 z n iy0))
("krasowski" nil (k r ax s ao1 f s k iy0))
("krass" nil (k r ae1 s))
("kraszewski" nil (k r ax sh eh1 f s k iy0))
("krat" nil (k r ae1 t))
("kratky" nil (k r ae1 t k iy0))
("kratochvil" nil (k r ae1 t ax k v ax l))
("kratt" nil (k r ae1 t))
("kratz" nil (k r ae1 t s))
("kratzer" nil (k r ey1 t z er0))
("kratzke" nil (k r ae1 t z k))
("krauer" nil (k r aw1 r))
("kraus" nil (k r aw1 s))
("krause" nil (k r ao1 s))
("krauser" nil (k r aw1 s er0))
("kraushaar" nil (k r aw1 sh aa1 r))
("krauskopf" nil (k r aw1 s k ao0 f))
("krauss" nil (k r aw1 s))
("krausse" nil (k r ao1 s))
("krausz" nil (k r aw1 sh))
("kraut" nil (k r aw1 t))
("krauter" nil (k r aw1 t er0))
("krauth" nil (k r ao1 th))
("krauze" nil (k r aw1 z))
("kravchuk" nil (k r aa1 v ch uh1 k))
("kravetz" nil (k r ae1 v ih0 t s))
("kravis" nil (k r ae1 v ih0 s))
("kravitz" nil (k r ae1 v ih0 t s))
("krawccykiewi" nil (k r aw1 ch ih0 k uw1 iy0))
("krawchuk" nil (k r ao1 ch ax k))
("krawczak" nil (k r aa1 v ch ae0 k))
("krawczyk" nil (k r aa1 v ch ih0 k))
("krawiec" nil (k r aa1 v iy0 k))
("krawitz" nil (k r aa1 w ih0 t s))
("kray" nil (k r ey1))
("kraynak" nil (k r ey1 n ax k))
("kreager" nil (k r iy1 g er0))
("kreamer" nil (k r iy1 m er0))
("krebbs" nil (k r eh1 b z))
("krebs" nil (k r eh1 b z))
("krebsbach" nil (k r eh1 b z b aa0 k))
("krech" nil (k r eh1 k))
("kredietbank" nil (k r eh0 d iy0 t b ae1 ng k))
("kredit" nil (k r eh1 t ih0 t))
("kreditanstalt" nil (k r eh0 d ih1 t ah1 n s t ao1 l t))
("kreeger" nil (k r iy1 g er0))
("kreft" nil (k r eh1 f t))
("kreg" nil (k r eh1 g))
("kregel" nil (k r eh1 g ax l))
("kreger" nil (k r iy1 g er0))
("kreh" nil (k r eh1))
("krehbiel" nil (k r eh1 b iy0 l))
("kreher" nil (k r eh1 r))
("kreicher" nil (k r ay1 k er0))
("kreider" nil (k r ay1 d er0))
("kreidler" nil (k r ay1 d ax l er0))
("kreifels" nil (k r ay1 f ax l z))
("kreig" nil (k r iy1 g))
("kreiger" nil (k r ay1 g er0))
("kreiling" nil (k r ay1 l ih0 ng))
("kreimer" nil (k r ay1 m er0))
("krein" nil (k r ey1 n))
("kreiner" nil (k r ay1 n er0))
("kreis" nil (k r iy1 z))
("kreisberg" nil (k r ay1 s b er0 g))
("kreischer" nil (k r ay1 sh er0))
("kreisel" nil (k r ay1 s ax l))
("kreiser" nil (k r ay1 s er0))
("kreisher" nil (k r iy1 ih0 sh er0))
("kreisler" nil (k r ay1 s l er0))
("kreisman" nil (k r ay1 s m ax n))
("kreiss" nil (k r ay1 s))
("kreiter" nil (k r ay1 t er0))
("kreitman" nil (k r ay1 t m ax n))
("kreitner" nil (k r ay1 t n er0))
("kreitz" nil (k r iy1 t s))
("kreitzberg" nil (k r ay1 t s b er0 g))
("kreitzer" nil (k r ay1 t z er0))
("krejci" nil (k r eh1 jh s iy0))
("krell" nil (k r eh1 l))
("kremer" nil (k r iy1 m er0))
("kremers" nil (k r iy1 m er0 z))
("kremlin" nil (k r eh1 m l ax n))
("kremlinologists" nil (k r eh1 m l ih0 n aa1 l ax jh ih0 s t s))
("krempa" nil (k r eh1 m p ax))
("krempasky" nil (k r ih0 m p aa1 s k iy0))
("kren" nil (k r eh1 n))
("krenek" nil (k r eh1 n ih0 k))
("krengel" nil (k r eh1 ng g ax l))
("krenik" nil (k r eh1 n ih0 k))
("krenke" nil (k r eh1 ng k))
("krenn" nil (k r eh1 n))
("krentz" nil (k r eh1 n t s))
("krenwinkle" nil (k r eh1 n w ih1 ng k ax l))
("krenz" nil (k r eh1 n z))
("krenzer" nil (k r eh1 n z er0))
("krepps" nil (k r eh1 p s))
("kreps" nil (k r eh1 p s))
("kresa" nil (k r iy1 s er0))
("kresge" nil (k r eh1 s g iy0))
("kress" nil (k r eh1 s))
("kresse" nil (k r eh1 s))
("kresser" nil (k r eh1 s er0))
("kressin" nil (k r eh1 s ih0 n))
("kressler" nil (k r eh1 s l er0))
("kretchman" nil (k r eh1 ch m ax n))
("kretchmer" nil (k r eh1 ch m er0))
("kretsch" nil (k r eh1 ch))
("kretschmar" nil (k r eh1 ch m er0))
("kretschmer" nil (k r eh1 ch m er0))
("kretz" nil (k r eh1 t s))
("kretzer" nil (k r eh1 t z er0))
("kretzschmar" nil (k r eh1 ch m er0))
("kreuger" nil (k r oy1 g er0))
("kreul" nil (k r uw1 l))
("kreuser" nil (k r oy1 s er0))
("kreuter" nil (k r oy1 t er0))
("kreutz" nil (k r uw1 t s))
("kreuzer" nil (k r uw1 z er0))
("krewson" nil (k r uw1 s ax n))
("krey" nil (k r ey1))
("kribs" nil (k r ih1 b z))
("krichbaum" nil (k r ih1 k b aw0 m))
("krick" nil (k r ih1 k))
("krider" nil (k r ay1 d er0))
("kridler" nil (k r ih1 d l er0))
("kriebel" nil (k r iy1 b ax l))
("krieg" nil (k r iy1 g))
("kriegel" nil (k r iy1 g ax l))
("krieger" nil (k r iy1 g er0))
("krienke" nil (k r iy1 ng k))
("krier" nil (k r ay1 er0))
("kriese" nil (k r iy1 z))
("kriesel" nil (k r iy1 s ax l))
("kriete" nil (k r iy1 t))
("krigbaum" nil (k r ih1 g b aw1 m))
("kriger" nil (k r ay1 g er0))
("krikalev" nil (k r ih1 k ax l eh1 v))
("krikorian" nil (k r ih0 k ao1 r iy0 ax n))
("krill" nil (k r ih1 l))
("krim" nil (k r ih1 m))
("krimmel" nil (k r ih1 m ax l))
("kriner" nil (k r ay1 n er0))
("kring" nil (k r ih1 ng))
("kringen" nil (k r ih1 ng ax n))
("krings" nil (k r ih1 ng z))
("krinke" nil (k r ih1 ng k))
("krinsky" nil (k r ih1 n s k iy0))
("kris" nil (k r ih1 s))
("krisch" nil (k r ih1 sh))
("krischer" nil (k r ih1 sh er0))
("krise" nil (k r ay1 z))
("krisher" nil (k r ih1 sh er0))
("krishna" nil (k r ih1 sh n ax))
("krishnan" nil (k r ih1 sh n ax n))
("krisko" nil (k r ih1 s k ow0))
("krispies" nil (k r ih1 s p iy0 z))
("kriss" nil (k r ih1 s))
("krist" nil (k r ih1 s t))
("krista" nil (k r ih1 s t ax))
("kristallnacht" nil (k r ih1 s t ax l n aa1 k t))
("kristen" nil (k r ih1 s ax n))
("kristensen" nil (k r ih1 s t ax n s ax n))
("kristi" nil (k r ih1 s t iy0))
("kristiansen" nil (k r ih1 s ch ax n s ax n))
("kristin" nil (k r ih1 s t ih0 n))
("kristine" nil (k r ih0 s t iy1 n))
("kristof" nil (k r ih1 s t ax f))
("kristoff" nil (k r ih1 s t ao0 f))
("kristoffe" nil (k r ih1 s ao0 f))
("kristol" nil (k r ih1 s t ax l))
("kristy" nil (k r ih1 s t iy0))
("kritz" nil (k r ih1 t s))
("kritzer" nil (k r ih1 t z er0))
("krivanek" nil (k r ih1 v ax n ih0 k))
("kriz" nil (k r ih1 z))
("krizan" nil (k r ih1 z ax n))
("krizek" nil (k r ih1 z eh0 k))
("krob" nil (k r aa1 b))
("kroboth" nil (k r aa1 b ax th))
("kroc" nil (k r aa1 k))
("krock" nil (k r aa1 k))
("kroeber" nil (k r ow1 b er0))
("kroeger" nil (k r ow1 g er0))
("kroegers" nil (k r ow1 g er0 z))
("kroeker" nil (k r ow1 k er0))
("kroener" nil (k r ow1 n er0))
("kroening" nil (k r aa1 ax n ih0 ng))
("kroenke" nil (k r ow1 ng k))
("kroes" nil (k r ow1 z))
("kroese" nil (k r ow1 s))
("kroeze" nil (k r ow1 z))
("kroft" nil (k r aa1 f t))
("krog" nil (k r aa1 g))
("kroger" nil (k r ow1 g er0))
("krogh" nil (k r ow1))
("krogman" nil (k r aa1 g m ax n))
("krogstad" nil (k r aa1 g s t ax d))
("kroh" nil (k r ow1))
("krohn" nil (k r ow1 n))
("krok" nil (k r aa1 k))
("krol" nil (k r ao1 l))
("krolak" nil (k r ow1 l ax k))
("krolczyk" nil (k r ow1 l ch ih0 k))
("krolick" nil (k r aa1 l ih0 k))
("krolikowski" nil (k r ax l ih0 k ao1 f s k iy0))
("kroll" nil (k r ao1 l))
("krom" nil (k r aa1 m))
("krome" nil (k r ow1 m))
("kromer" nil (k r ow1 m er0))
("kromm" nil (k r aa1 m))
("kron" nil (k r aa1 n))
("krona" nil (k r ow1 n ax))
("kronberg" nil (k r aa1 n b er0 g))
("krone" nil (k r ow1 n ax))
("kronenberg" nil (k r aa1 n ax n b er0 g))
("kronenberger" nil (k r aa1 n ax n b er0 g er0))
("kroner" nil (k r ow1 n er0))
("kronick" nil (k r aa1 n ih0 k))
("kronish" nil (k r aa1 n ih0 sh))
("kronk" nil (k r aa1 ng k))
("kronor" nil (k r ow1 n er0))
("kronos" nil (k r ow1 n ow0 s))
("krontz" nil (k r aa1 n t s))
("kroon" nil (k r uw1 n))
("kropf" nil (k r aa1 p f))
("kropp" nil (k r aa1 p))
("kross" nil (k r ao1 s))
("krossel" nil (k r ao1 s ax l))
("krotkov" nil (k r ao1 t k ao0 v))
("krotz" nil (k r aa1 t s))
("krotzer" nil (k r ow1 t z er0))
("krouner" nil (k r uw1 n er0))
("kroupa" nil (k r uw1 p ax))
("krouse" nil (k r aw1 s))
("krout" nil (k r aw1 t))
("krowe" nil (k r ow1))
("krowitz" nil (k r aw1 ih0 t s))
("kroy" nil (k r oy1))
("kruchten" nil (k r ah1 k t ax n))
("kruck" nil (k r ah1 k))
("kruckeberg" nil (k r ah1 k b er0 g))
("kruckenberg" nil (k r ah1 k ax n b er0 g))
("kruczek" nil (k r ah1 ch eh0 k))
("krudman" nil (k r ah1 d m ax n))
("krueger" nil (k r uw1 g er0))
("kruer" nil (k r uw1 er0))
("krug" nil (k r ah1 g))
("kruger" nil (k r uw1 g er0))
("krugerrand" nil (k r uw0 g eh1 r ae0 n d))
("krugh" nil (k r ah1))
("krugman" nil (k r ah1 g m ax n))
("kruk" nil (k r ah1 k))
("krukowski" nil (k r ax k ao1 f s k iy0))
("krul" nil (k r ah1 l))
("krull" nil (k r ah1 l))
("krum" nil (k r ah1 m))
("krumholz" nil (k r ah1 m hh ow1 l z))
("krumm" nil (k r ah1 m))
("krumme" nil (k r ah1 m))
("krummel" nil (k r ah1 m ax l))
("krumrey" nil (k r ah1 m r iy0))
("krumwiede" nil (k r ah1 m w iy1 d))
("krupa" nil (k r uw1 p ax))
("krupicka" nil (k r uw1 p ih1 k ax))
("krupinski" nil (k r ax p ih1 n s k iy0))
("krupka" nil (k r ah1 p k ax))
("krupman" nil (k r ah1 p m ax n))
("krupnick" nil (k r ah1 p n ih0 k))
("krupp" nil (k r ah1 p))
("kruppa" nil (k r ah1 p ax))
("krupski" nil (k r ah1 p s k iy0))
("krus" nil (k r ah1 s))
("kruschke" nil (k r ah1 sh k))
("kruse" nil (k r uw1 z))
("krusemark" nil (k r ah1 s ih0 m aa0 r k))
("krusinski" nil (k r ax s ih1 n s k iy0))
("kruszewski" nil (k r ax sh eh1 f s k iy0))
("kruszka" nil (k r ah1 sh k ax))
("kruszynski" nil (k r ax sh ih1 n s k iy0))
("krutick" nil (k r uw1 t ih1 k))
("krutsinger" nil (k r ah1 t s ih0 n jh er0))
("kruttschnitt" nil (k r ah1 ch n ih0 t))
("krutz" nil (k r ah1 t s))
("kruzel" nil (k r uw1 z ax l))
("krych" nil (k r ih1 ch))
("kryder" nil (k r ay1 d er0))
("kryger" nil (k r ay1 g er0))
("krygier" nil (k r ay1 g iy0 er0))
("krypton" nil (k r ih1 p t aa0 n))
("kryptos" nil (k r ih1 p t ow0 s))
("krysiak" nil (k r ih1 s iy0 ae0 k))
("krzeminski" nil (k r ih0 m ih1 n s k iy0))
("krzysztof" nil (k r ay1 s t ao0 f))
("krzywicki" nil (k r ih0 v ih1 t s k iy0))
("krzyzanowski" nil (k r ih0 z ax n ao1 f s k iy0))
("ksiazek" nil (k ax s iy0 aa1 z eh0 k))
("ku" nil (k uw1))
("kuala" nil (k w aa1 l ax))
("kuan" nil (k w aa1 n))
("kuba" nil (k y uw1 b ax))
("kubacki" nil (k ax b aa1 t s k iy0))
("kubala" nil (k ax b aa1 l ax))
("kuban" nil (k y uw1 b ax n))
("kubas" nil (k uw1 b ax z))
("kubat" nil (k uw1 b ax t))
("kube" nil (k y uw1 b))
("kubena" nil (k ax b iy1 n ax))
("kuberski" nil (k ax b er1 s k iy0))
("kubes" nil (k y uw1 b z))
("kubiak" nil (k uw1 b iy0 ae0 k))
("kubic" nil (k y uw1 b ih0 k))
("kubica" nil (k y uw1 b ih0 k ax))
("kubicek" nil (k ah1 b ih0 ch eh0 k))
("kubick" nil (k y uw1 b ih0 k))
("kubicki" nil (k ax b ih1 t s k iy0))
("kubik" nil (k y uw1 b ih0 k))
("kubin" nil (k y uw1 b ih0 n))
("kubinski" nil (k ax b ih1 n s k iy0))
("kubis" nil (k uw1 b ih0 s))
("kubisiak" nil (k ax b ih1 s iy0 ae0 k))
("kubitz" nil (k y uw1 b ih0 t s))
("kubler" nil (k y uw1 b ax l er0))
("kubly" nil (k ah1 b l iy0))
("kubo" nil (k y uw1 b ow0))
("kubota" nil (k uw0 b ow1 t ax))
("kubrick" nil (k y uw1 b r ih1 k))
("kuby" nil (k y uw1 b iy0))
("kuc" nil (k ah1 k))
("kucan" nil (k y uw1 k ax n))
("kucera" nil (k ax ch ih1 r ax))
("kucewicz" nil (k y uw1 s ih0 w ih0 t s))
("kuch" nil (k ah1 ch))
("kuchar" nil (k ah1 k er0))
("kucharski" nil (k ax ch aa1 r s k iy0))
("kuchenbecker" nil (k ah1 k ih0 n b eh0 k er0))
("kucher" nil (k ah1 k er0))
("kuchera" nil (k ah1 ch er0 ax))
("kuchinski" nil (k ax ch ih1 n s k iy0))
("kuchinsky" nil (k ax ch ih1 n s k iy0))
("kuchler" nil (k ah1 k ax l er0))
("kuchma" nil (k uw1 ch m ax))
("kuchta" nil (k ah1 ch t ax))
("kucinski" nil (k ax ch ih1 n s k iy0))
("kuck" nil (k ah1 k))
("kuczek" nil (k ah1 ch eh0 k))
("kuczynski" nil (k ax ch ih1 n s k iy0))
("kuder" nil (k y uw1 d er0))
("kudla" nil (k ah1 d l ax))
("kudlow" nil (k ah1 d l ow0))
("kudner" nil (k ah1 d n er0))
("kudos" nil (k uw1 d ow0 s))
("kudrna" nil (k ah1 d er1 n ax))
("kudzu" nil (k ah1 d z uw0))
("kuebler" nil (k uh1 b ax l er0))
("kuechler" nil (k uh1 k ax l er0))
("kueck" nil (k uw1 k))
("kuecker" nil (k uh1 k er0))
("kuehl" nil (k uh1 l))
("kuehler" nil (k uh1 l er0))
("kuehn" nil (k uw1 n))
("kuehne" nil (k uw1 n))
("kuehnel" nil (k uh1 n ax l))
("kuehner" nil (k uh1 n er0))
("kuehnle" nil (k uh1 n ax l))
("kueker" nil (k uh1 k er0))
("kuenheim" nil (k y uw1 ax n hh ay1 m))
("kuennen" nil (k uh1 n ax n))
("kuenstler" nil (k uh1 n s ax l er0))
("kuenzel" nil (k uh1 n z ax l))
("kuenzi" nil (k uw0 ey1 n z iy0))
("kuenzli" nil (k uh1 n z l iy0))
("kuester" nil (k uh1 s t er0))
("kuether" nil (k uh1 dh er0))
("kufahl" nil (k ah1 f aa0 l))
("kuffel" nil (k ah1 f ax l))
("kugel" nil (k uw1 g ax l))
("kugelman" nil (k ah1 g ax l m ax n))
("kugler" nil (k uw1 g ax l er0))
("kuhar" nil (k uw1 er0))
("kuhl" nil (k ah1 l))
("kuhle" nil (k uw1 ax l))
("kuhlenschmidt" nil (k y uw1 l ax n sh m ih1 t))
("kuhlman" nil (k uw1 l m ax n))
("kuhlmann" nil (k uw1 l m ax n))
("kuhn" nil (k uw1 n))
("kuhne" nil (k ah1 n))
("kuhner" nil (k uw1 n er0))
("kuhnert" nil (k uw1 n er0 t))
("kuhnke" nil (k ah1 ng k))
("kuhnle" nil (k ah1 n ax l))
("kuhns" nil (k uw1 n z))
("kuhr" nil (k er1))
("kuhrt" nil (k er1 t))
("kuiken" nil (k uw1 k ax n))
("kuiper" nil (k uw1 p er0))
("kuipers" nil (k uw1 p er0 z))
("kujala" nil (k ay0 aa1 l ax))
("kujawa" nil (k uw0 jh aa1 w ax))
("kujawski" nil (k ax y aa1 f s k iy0))
("kuk" nil (k ah1 k))
("kuker" nil (k y uw1 k er0))
("kukje" nil (k uw1 k y ih0))
("kukla" nil (k ah1 k l ax))
("kuklinski" nil (k ax k l ih1 n s k iy0))
("kukowski" nil (k ax k ao1 f s k iy0))
("kukuk" nil (k uw1 k ax k))
("kula" nil (k uw1 l ax))
("kulaga" nil (k uw0 l aa1 g ax))
("kulakowski" nil (k y uw0 l ax k ao1 f s k iy0))
("kulas" nil (k y uw1 l ax z))
("kulesa" nil (k y uw0 l iy1 s ax))
("kulesza" nil (k y uw0 l eh1 sh ax))
("kulhanek" nil (k ah1 l hh ax n eh0 k))
("kulich" nil (k y uw1 l ih0 k))
("kulick" nil (k y uw1 l ih0 k))
("kulig" nil (k y uw1 l ih0 g))
("kuligowski" nil (k y uw0 l ih0 g ao1 f s k iy0))
("kulik" nil (k y uw1 l ih0 k))
("kulikowski" nil (k y uw0 l ih0 k ao1 f s k iy0))
("kulinski" nil (k y uw0 l ih1 n s k iy0))
("kulis" nil (k uw1 l ih0 s))
("kulish" nil (k y uw1 l ih0 sh))
("kulka" nil (k ah1 l k ax))
("kulkarni" nil (k ax l k aa1 r n iy0))
("kull" nil (k ah1 l))
("kulla" nil (k ah1 l ax))
("kullberg" nil (k ah1 l b er0 g))
("kullman" nil (k ah1 l m ax n))
("kully" nil (k ah1 l iy0))
("kulow" nil (k y uw1 l ow0))
("kulp" nil (k ah1 l p))
("kulpa" nil (k ah1 l p ax))
("kulwicki" nil (k ax l v ih1 t s k iy0))
("kulzer" nil (k ah1 l z er0))
("kumagai" nil (k uw0 m aa0 g aa1 iy0))
("kumar" nil (k uw0 m aa1 r))
("kumble" nil (k ah1 m b ax l))
("kume" nil (k y uw1 m))
("kumler" nil (k ah1 m l er0))
("kumm" nil (k ah1 m))
("kummer" nil (k ah1 m er0))
("kumouri" nil (k uw1 m ao1 r iy0))
("kump" nil (k ah1 m p))
("kumpf" nil (k ah1 m p f))
("kumquat" nil (k ah1 m k w aa0 t))
("kun" nil (k ah1 n))
("kuna" nil (k y uw1 n ax))
("kunaev" nil (k y uw0 n ey1 v))
("kunath" nil (k ah1 n ax th))
("kunayev" nil (k y uw0 n ey1 ax v))
("kunce" nil (k ah1 n s))
("kunda" nil (k ah1 n d ax))
("kunde" nil (k ah1 n d))
("kundera" nil (k ax n d eh1 r ax))
("kundert" nil (k ah1 n d er0 t))
("kundinger" nil (k ah1 n d ih0 ng er0))
("kundrat" nil (k ah1 n d r ax t))
("kunert" nil (k y uw1 n er0 t))
("kunes" nil (k y uw1 n z))
("kunesh" nil (k ah1 n ih0 sh))
("kuney" nil (k y uw1 n iy0))
("kung" nil (k ah1 ng))
("kunin" nil (k y uw1 n ih0 n))
("kunio" nil (k y uw1 n iy0 ow0))
("kunka" nil (k ah1 ng k ax))
("kunkel" nil (k ah1 ng k ax l))
("kunkle" nil (k ah1 ng k ax l))
("kunkler" nil (k ah1 ng k l er0))
("kuns" nil (k ah1 n z))
("kunselman" nil (k ah1 n s ax l m ax n))
("kunsman" nil (k ah1 n s m ax n))
("kunst" nil (k ah1 n s t))
("kunstler" nil (k ah1 n s t l er0))
("kuntz" nil (k ah1 n t s))
("kuntze" nil (k ah1 n t z))
("kuntzman" nil (k ah1 n t s m ax n))
("kunz" nil (k ah1 n z))
("kunze" nil (k ah1 n z))
("kunzelman" nil (k ah1 n z ax l m ax n))
("kunzler" nil (k ah1 n z l er0))
("kunzman" nil (k ah1 n z m ax n))
("kuo" nil (k uw1 ow0))
("kuomintang" nil (k w ow1 m ih1 n t ae1 ng))
("kupec" nil (k uw1 p ih0 k))
("kuper" nil (k y uw1 p er0))
("kuperman" nil (k uw1 p er0 m ax n))
("kupfer" nil (k ah1 p f er0))
("kupferman" nil (k ah1 p f er0 m ax n))
("kupiec" nil (k ah1 p iy0 k))
("kupka" nil (k ah1 p k ax))
("kupor" nil (k y uw1 p er0))
("kupper" nil (k ah1 p er0))
("kuralt" nil (k y er0 ao1 l t))
("kuramoto" nil (k uh0 r aa0 m ow1 t ow0))
("kuranari" nil (k uh1 r ax n aa1 r iy0))
("kuras" nil (k uh1 r ax z))
("kurd" nil (k er1 d))
("kurdish" nil (k er1 d ih0 sh))
("kurdistan" nil (k er1 d ih0 s t ae1 n))
("kurds" nil (k er1 d z))
("kurdziel" nil (k er1 d z iy0 l))
("kurek" nil (k y uw1 r eh0 k))
("kurian" nil (k y uh1 r iy0 ax n))
("kuriansky" nil (k uh1 r iy0 ae1 n s k iy0))
("kurihara" nil (k uw1 r ih0 hh aa1 r ax))
("kurilla" nil (k er0 ih1 l ax))
("kurinsky" nil (k y er1 ih1 n s k iy0))
("kurk" nil (k er1 k))
("kurka" nil (k er1 k ax))
("kurkjian" nil (k er1 k jh iy0 ax n))
("kurkowski" nil (k er0 k ao1 f s k iy0))
("kurlak" nil (k er1 l ae0 k))
("kurland" nil (k er1 l ax n d))
("kurlander" nil (k er1 l ax n d er0))
("kurman" nil (k er1 m ax n))
("kurmel" nil (k er1 m ax l))
("kurnit" nil (k er1 n ih0 t))
("kuroda" nil (k er0 ow1 d ax))
("kurokawa" nil (k uw1 r ow0 k aa1 w ax))
("kurosawa" nil (k uh1 r ow0 s aa1 w ax))
("kurowski" nil (k er0 ao1 f s k iy0))
("kurt" nil (k er1 t))
("kurtenbach" nil (k er1 t ih0 n b aa0 k))
("kurth" nil (k er1 th))
("kurtis" nil (k er1 t ih0 s))
("kurtz" nil (k er1 t s))
("kurtzman" nil (k er1 t s m ax n))
("kuruman" nil (k y uh1 r uw0 m ax n))
("kurumizowa" nil (k uh1 r uw0 m iy0 z ow1 ax))
("kury" nil (k y uw1 r iy0))
("kurylo" nil (k er0 ih1 l ow0))
("kurz" nil (k er1 z))
("kurzawa" nil (k uh0 r z aa1 w ax))
("kurzweil" nil (k er0 z w ay1 l))
("kus" nil (k ah1 s))
("kusch" nil (k ah1 sh))
("kuschel" nil (k ah1 sh ax l))
("kuse" nil (k y uw1 z))
("kusek" nil (k uw1 s eh0 k))
("kusel" nil (k uw1 s ax l))
("kush" nil (k uh1 sh))
("kushner" nil (k ah1 sh n er0))
("kushnir" nil (k ah1 sh n er0))
("kusiak" nil (k ah1 s iy0 ae0 k))
("kuske" nil (k ah1 s k))
("kusler" nil (k ah1 s ax l er0))
("kuss" nil (k ah1 s))
("kusserow" nil (k ah1 s er0 ow0))
("kussman" nil (k ah1 s m ax n))
("kuster" nil (k ah1 s t er0))
("kustra" nil (k ah1 s t r ax))
("kutch" nil (k ah1 ch))
("kutcher" nil (k ah1 ch er0))
("kuter" nil (k y uw1 t er0))
("kutner" nil (k ah1 t n er0))
("kutsch" nil (k ah1 ch))
("kutscher" nil (k ah1 ch er0))
("kutter" nil (k ah1 t er0))
("kuttner" nil (k ah1 t n er0))
("kutz" nil (k ah1 t s))
("kutzer" nil (k ah1 t z er0))
("kuwahara" nil (k uw1 w aa0 hh aa1 r ax))
("kuwait" nil (k uw0 w ey1 t))
("kuwaiti" nil (k uw0 w ey1 t iy0))
("kuwaitis" nil (k uw0 w ey1 t iy0 z))
("kuwam" nil (k y uw1 w aa0 m))
("kuykendall" nil (k ay1 k eh0 n d aa1 l))
("kuyper" nil (k ay1 p er0))
("kuzara" nil (k y uw0 z aa1 r ax))
("kuze" nil (k y uw1 z))
("kuzel" nil (k uw1 z ax l))
("kuzma" nil (k ah1 z m ax))
("kuzminski" nil (k ax z m ih1 n s k iy0))
("kuznets" nil (k ah1 z n eh1 t s))
("kuznia" nil (k ah1 z n iy0 ax))
("kuzniar" nil (k ah1 z n y er0))
("kuznicki" nil (k ax z n ih1 t s k iy0))
("kvale" nil (k ax v ey1 l))
("kvam" nil (k ax v ae1 m))
("kvamme" nil (k ax v ae1 m))
("kvasnicka" nil (k ax v ax s n ih1 s k ax))
("kvetch" nil (k ax v eh1 ch))
("kveton" nil (k ax v eh1 t ax n))
("kvistad" nil (k ax v ih1 s t ae1 d))
("kvitsinsky" nil (k ax v ih0 t s ih1 n s k iy0))
("kwak" nil (k w ae1 k))
("kwan" nil (k w aa1 n))
("kwang" nil (k w aa1 ng))
("kwangju" nil (k w aa0 ng jh uw1))
("kwanza" nil (k w aa1 n z ax))
("kwasniewski" nil (k w ax z n eh1 f s k iy0))
("kwasnik" nil (k w aa1 z n ih0 k))
("kwasny" nil (k w aa1 z n iy0))
("kwazulu" nil (k w aa0 z uw1 l uw0))
("kweisi" nil (k w ay1 z iy0))
("kwh" nil (k ey1 d ah1 b ax l y uw0 ey1 ch))
("kwiatek" nil (k w iy0 aa1 t eh0 k))
("kwiatkowski" nil (k w iy0 ax t k ao1 f s k iy0))
("kwiecien" nil (k w iy1 s iy0 n))
("kwiecinski" nil (k w iy0 ch ih1 n s k iy0))
("kwik" nil (k w ih1 k))
("kwitny" nil (k w ih1 t n iy0))
("kwok" nil (k w aa1 k))
("kwolek" nil (k w ow1 l eh0 k))
("kwon" nil (k w aa1 n))
("kwong" nil (k w ao1 ng))
("kyd" nil (k ih1 d))
("kyer" nil (k ay1 er0))
("kyes" nil (k ay1 z))
("kyger" nil (k ay1 g er0))
("kyi" nil (k iy1))
("kyker" nil (k ay1 k er0))
("kyle" nil (k ay1 l))
("kyler" nil (k ay1 l er0))
("kyles" nil (k ay1 l z))
("kyllo" nil (k ih1 l ow0))
("kyllonen" nil (k ih0 l aa1 n ax n))
("kyna" nil (k ih1 n ax))
("kynard" nil (k ih1 n er0 d))
("kyne" nil (k ay1 n))
("kynikos" nil (k ih0 n iy1 k ow0 s))
("kyocera" nil (k y ow0 s eh1 r ax))
("kyodo" nil (k y ow1 d ow0))
("kyokuto" nil (k y aa0 k uw1 t ow0))
("kyong" nil (k y ao1 ng))
("kyongsang" nil (k y ao1 ng s ae0 ng))
("kyoto" nil (k y ow1 t ow0))
("kyowa" nil (k y ow1 ax))
("kyrgyzstan" nil (k ih1 r g ih0 s t ae1 n))
("kysar" nil (k ih1 s er0))
("kyser" nil (k ay1 s er0))
("kysor" nil (k ay1 s er0))
("kyte" nil (k ay1 t))
("kytril" nil (k ih1 t r ih0 l))
("kyu" nil (k y uw1))
("kyung" nil (k y ah1 ng))
("kyushu" nil (k y ah1 sh uw0))
("kyzar" nil (k ih1 z er0))
("kyzer" nil (k ay1 z er0))
("l" nil (eh1 l))
("la" nil (l aa1))
("laabs" nil (l aa1 b z))
("laack" nil (l aa1 k))
("laaco" nil (l aa1 k ow0))
("laake" nil (l aa1 k))
("laakso" nil (l aa1 k s ow0))
("laas" nil (l aa1 z))
("laatsch" nil (l aa1 ch))
("lab" nil (l ae1 b))
("laba" nil (l aa1 b ax))
("labadie" nil (l ae1 b ax d iy0))
("laban" nil (l ey1 b ax n))
("laband" nil (l aa0 b ae1 n d))
("labant" nil (l ax b ae1 n t))
("labar" nil (l ax b aa1 r))
("labarbera" nil (l aa0 b aa0 r b eh1 r ax))
("labarge" nil (l ae1 b aa0 r g))
("labarr" nil (l ax b ae1 r))
("labarre" nil (l aa0 b aa1 r ey0))
("labat" nil (l aa1 b aa0 t))
("labate" nil (l aa1 b ey0 t))
("labaton" nil (l ae1 b ax t ax n))
("labatt" nil (l ax b ae1 t))
("labauve" nil (l ax b ow1 v))
("labay" nil (l ae1 b ey0))
("labbe" nil (l ae1 b))
("labe" nil (l ey1 b))
("labeau" nil (l ax b ow1))
("label" nil (l ey1 b ax l))
("labeled" nil (l ey1 b ax l d))
("labeling" nil (l ey1 b ax l ih0 ng))
("labell" nil (l ax b eh1 l))
("labella" nil (l ax b eh1 l ax))
("labelle" nil (l ax b eh1 l))
("labelled" nil (l ey1 b ax l d))
("labels" nil (l ey1 b ax l z))
("labenski" nil (l aa0 b eh1 n s k iy0))
("laber" nil (l ey1 b er0))
("laberge" nil (l aa1 b er0 g))
("labianca" nil (l aa0 b iy0 aa1 ng k ax))
("labine" nil (l aa0 b iy1 n iy0))
("lablanc" nil (l ax b l ae1 ng k))
("labo" nil (l aa1 b ow0))
("laboda" nil (l aa0 b ow1 d ax))
("labombard" nil (l ae1 b ax m b er0 d))
("labonte" nil (l ax b aa1 n t))
("labor" nil (l ey1 b er0))
("laboratoires" nil (l ax b ao1 r ax t w aa1 z))
("laboratories" nil (l ae1 b r ax t ao1 r iy0 z))
("laboratory" nil (l ae1 b r ax t ao1 r iy0))
("laborde" nil (l ax b ao1 r d))
("labore" nil (l ax b ao1 r))
("labored" nil (l ey1 b er0 d))
("laborer" nil (l ey1 b er0 er0))
("laborers" nil (l ey1 b er0 er0 z))
("laboring" nil (l ey1 b er0 ih0 ng))
("laborious" nil (l ax b ao1 r iy0 ax s))
("laboriously" nil (l ax b ao1 r iy0 ax s l iy0))
("labors" nil (l ey1 b er0 z))
("labossiere" nil (l ae1 b ax s iy0 eh0 r))
("labouisse" nil (l aa0 b uw1 s))
("labounty" nil (l ax b aw1 n t iy0))
("labour" nil (l ey1 b aw1 r))
("labove" nil (l ax b ah1 v))
("labow" nil (l ae1 b ow0))
("laboy" nil (l ax b oy1))
("labrador" nil (l ae1 b r ax d ao1 r))
("labrake" nil (l ae1 b r ax k))
("labranche" nil (l aa0 b r aa1 n k iy0))
("labreck" nil (l ae1 b r ih0 k))
("labrecque" nil (l ax b r eh1 k))
("labree" nil (l ax b r iy1))
("labrie" nil (l ae1 b er0 iy0))
("labrioche" nil (l aa1 b r iy0 ow1 sh))
("labriola" nil (l aa0 b r iy0 ow1 l ax))
("labrosse" nil (l aa0 b r ow1 s iy0))
("labs" nil (l ae1 b z))
("labuda" nil (l ax b y uw1 d ax))
("labus" nil (l ae1 b ih0 s))
("labyrinth" nil (l ae1 b er0 ih1 n th))
("labyrinthine" nil (l ae1 b er0 ih1 n th iy1 n))
("lac" nil (l ae1 k))
("lacana" nil (l aa0 k ae1 n ax))
("lacasse" nil (l aa0 k aa1 s iy0))
("lacava" nil (l aa0 k aa1 v ax))
("lacayo" nil (l aa0 k ey1 ow0))
("lacaze" nil (l aa0 k aa1 z iy0))
("laccoliths" nil (l ae1 k ax l ih0 th s))
("lace" nil (l ey1 s))
("laced" nil (l ey1 s t))
("lacefield" nil (l ey1 s f iy1 l d))
("lacer" nil (l ey1 s er0))
("lacerate" nil (l ae1 s er0 ey1 t))
("laceration" nil (l ae1 s er0 ey1 sh ax n))
("lacerations" nil (l ae1 s er0 ey1 sh ax n z))
("lacerte" nil (l aa0 ch eh1 r t iy0))
("laces" nil (l ey1 s ax z))
("lacewell" nil (l ey1 s w eh1 l))
("lacey" nil (l ey1 s iy0))
("lach" nil (l ae1 ch))
("lachance" nil (l aa1 k ax n s))
("lachapelle" nil (l ae1 sh ax p ax l))
("lacharite" nil (l ae1 ch er0 ay1 t))
("lache" nil (l ae1 ch))
("lachenbruch" nil (l ae1 k ax n b r uw1 k))
("lacher" nil (l ae1 k er0))
("lachica" nil (l ae1 ch ih0 k ax))
("lachlan" nil (l ae1 k l ax n))
("lachman" nil (l ae1 k m ax n))
("lachmar" nil (l ae1 k m aa0 r))
("lachney" nil (l ae1 k n iy0))
("lachowicz" nil (l aa1 hh ax v ih0 ch))
("lacina" nil (l aa0 ch iy1 n ax))
("lacivita" nil (l aa0 ch iy0 v iy1 t ax))
("lack" nil (l ae1 k))
("lackadaisical" nil (l ae1 k ax d ey1 z ih0 k ax l))
("lackawanna" nil (l ae1 k ax w aa1 n ax))
("lacked" nil (l ae1 k t))
("lackey" nil (l ae1 k iy0))
("lackeys" nil (l ae1 k iy0 z))
("lackie" nil (l ae1 k iy0))
("lacking" nil (l ae1 k ih0 ng))
("lackluster" nil (l ae1 k l ah1 s t er0))
("lackman" nil (l ae1 k m ax n))
("lackner" nil (l ae1 k n er0))
("lacko" nil (l ae1 k ow0))
("lackovic" nil (l ae1 k ax v ih0 k))
("lacks" nil (l ae1 k s))
("laclair" nil (l ae1 k l er0))
("laclede" nil (l aa0 k l iy1 d))
("lacock" nil (l ae1 k ax k))
("lacomb" nil (l ae1 k ax m))
("lacombe" nil (l aa0 k ow1 m b iy0))
("laconic" nil (l aa0 k aa1 n ih0 k))
("laconte" nil (l aa0 k ow1 n t iy0))
("lacorte" nil (l aa0 k ao1 r t iy0))
("lacoss" nil (l ax k aa1 s))
("lacosse" nil (l aa0 k ow1 s iy0))
("lacoste" nil (l aa0 k ao1 s t))
("lacount" nil (l ax k uw1 n t))
("lacour" nil (l ax k uh1 r))
("lacourse" nil (l ax k uh1 r s))
("lacoursiere" nil (l ae1 k uh0 r s iy0 eh0 r))
("lacouture" nil (l ae1 k uw0 ch er0))
("lacovara" nil (l aa0 k ow0 v aa1 r ax))
("lacquer" nil (l ae1 k er0))
("lacquered" nil (l ae1 k er0 d))
("lacroix" nil (l ax k r oy1))
("lacross" nil (l ax k r ao1 s))
("lacrosse" nil (l ax k r ao1 s))
("lactase" nil (l ae1 k t ey1 s))
("lactating" nil (l ae1 k t ey0 t ih0 ng))
("lactation" nil (l ae0 k t ey1 sh ax n))
("lacteals" nil (l ae1 k t iy1 l z))
("lacter" nil (l ae1 k t er0))
("lactic" nil (l ae1 k t ih0 k))
("lactobacillus" nil (l ae1 k t ow0 b ax s ih1 l ax s))
("lactone" nil (l ae1 k t ow0 n))
("lactose" nil (l ae1 k t ow0 s))
("lacy" nil (l ey1 s iy0))
("lad" nil (l ae1 d))
("lada" nil (l aa1 d ax))
("ladas" nil (l aa1 d ax z))
("ladbroke" nil (l ae1 d b r ow1 k))
("ladd" nil (l ae1 d))
("ladder" nil (l ae1 d er0))
("ladders" nil (l ae1 d er0 z))
("lade" nil (l ey1 d))
("ladehoff" nil (l ae1 d ax hh ao0 f))
("laden" nil (l ey1 d ax n))
("ladenburg" nil (l ey1 d ax n b er0 g))
("lader" nil (l ey1 d er0))
("ladewig" nil (l ae1 d uw0 ih0 g))
("ladies" nil (l ey1 d iy0 z))
("lading" nil (l ey1 d ih0 ng))
("ladino" nil (l ax d iy1 n ow0))
("ladish" nil (l aa0 d ih1 sh))
("ladle" nil (l ey1 d ax l))
("ladled" nil (l ey1 d ax l d))
("ladles" nil (l ey1 d ax l z))
("ladley" nil (l ae1 d l iy0))
("ladner" nil (l ae1 d n er0))
("ladnier" nil (l ae1 d n iy0 er0))
("ladouceur" nil (l ae1 d uw0 s er0))
("ladow" nil (l ae1 d ow0))
("lads" nil (l ae1 d z))
("ladson" nil (l ae1 d s ax n))
("laduca" nil (l aa0 d uw1 k ax))
("ladue" nil (l aa1 d w eh0))
("laduke" nil (l aa0 d uw1 k ey0))
("ladwig" nil (l ae1 d w ih0 g))
("lady" nil (l ey1 d iy0))
("ladybird" nil (l ey1 d iy0 b er1 d))
("ladybugs" nil (l ey1 d iy0 b ah1 g z))
("ladylike" nil (l ey1 d iy0 l ay1 k))
("laenderbank" nil (l ae1 n d er0 b ae1 ng k))
("laevo" nil (l ey1 v ow0))
("lafalce" nil (l aa0 f ae1 l s iy0))
("lafarge" nil (l aa0 f aa1 r jh))
("lafata" nil (l aa0 f aa1 t ax))
("lafauci" nil (l aa0 f ao1 ch iy0))
("lafave" nil (l ax f ey1 v))
("lafavor" nil (l ae1 f ax v er0))
("lafayette" nil (l aa1 f iy0 eh1 t))
("lafer" nil (l ey1 f er0))
("laferriere" nil (l ae1 f er0 iy0 eh0 r))
("lafever" nil (l ae1 f eh0 v er0))
("lafevers" nil (l ae1 f eh0 v er0 z))
("lafeyette" nil (l aa1 f iy0 eh1 t))
("laff" nil (l ae1 f))
("laffer" nil (l ae1 f er0))
("lafferty" nil (l ae1 f er0 t iy0))
("laffey" nil (l ae1 f iy0))
("laffin" nil (l ae1 f ih0 n))
("laffitte" nil (l ax f ih1 t))
("laffoon" nil (l ax f uw1 n))
("lafite" nil (l aa0 f ay1 t))
("lafitte" nil (l ax f ih1 t))
("laflam" nil (l ax f l ae1 m))
("laflamme" nil (l ae1 f l ih0 m))
("lafleche" nil (l ax f l eh1 sh))
("lafler" nil (l ey1 f ax l er0))
("lafleur" nil (l ax f l er1))
("laflin" nil (l ae1 f l ih0 n))
("lafoe" nil (l ax f ow1))
("lafollette" nil (l ae1 f ax l eh0 t))
("lafon" nil (l ae1 f ax n))
("lafond" nil (l ax f aa1 n d))
("lafont" nil (l ax f aa1 n t))
("lafontaine" nil (l ae1 f ax n t ey1 n))
("lafontant" nil (l aa0 f aa1 n t ax n t))
("laforce" nil (l ax f ao1 r s))
("laforest" nil (l ax f ao1 r ax s t))
("laforge" nil (l ax f ao1 r g))
("laforte" nil (l ax f ao1 r t))
("lafortune" nil (l ae1 f er0 t uw0 n))
("lafosse" nil (l ax f aa1 s))
("lafountain" nil (l aa1 f aa1 n t ey1 n))
("lafountaine" nil (l aa1 f aa1 n t ey1 n))
("lafoy" nil (l ax f oy1))
("laframboise" nil (l ah1 f r ae1 m b w aa1))
("lafrance" nil (l ax f r ae1 n s))
("lafreniere" nil (l ae1 f r ih0 n iy0 eh0 r))
("lafrenz" nil (l ae1 f r ih0 n s))
("lafuente" nil (l aa0 f w eh1 n t ey0))
("lag" nil (l ae1 g))
("lagace" nil (l aa0 g aa1 ch iy0))
("lagan" nil (l ae1 g ax n))
("lagana" nil (l aa0 g ae1 n ax))
("lagarde" nil (l aa0 g aa1 r d iy0))
("lagardere" nil (l aa0 g aa0 r d ih1 r))
("lagasse" nil (l aa0 g aa1 s iy0))
("lagattuta" nil (l aa0 g aa0 t uw1 t ax))
("lage" nil (l ey1 jh))
("lager" nil (l aa1 g er0))
("lagergren" nil (l ey1 g er0 g r eh0 n))
("lagerquist" nil (l ey1 g er0 k w ih0 s t))
("lagerstrom" nil (l ey1 g er0 s t r ax m))
("lagesse" nil (l ae1 g eh0 s))
("laggard" nil (l ae1 g er0 d))
("laggards" nil (l ae1 g er0 d z))
("lagged" nil (l ae1 g d))
("lagging" nil (l ae1 g ih0 ng))
("lagle" nil (l ey1 g ax l))
("lagnado" nil (l aa1 g n aa1 d ow0))
("lago" nil (l aa1 g ow0))
("lagomarsino" nil (l aa0 g ow1 m aa0 r s iy1 n ow0))
("lagoon" nil (l ax g uw1 n))
("lagoons" nil (l ax g uw1 n z))
("lagos" nil (l ey1 g ao0 s))
("lagow" nil (l ae1 g ow0))
("lagrange" nil (l ae1 g r ey1 n jh))
("lagreca" nil (l aa0 g r eh1 k ax))
("lagroceria" nil (l aa0 g r ow1 s er0 iy1 ah1))
("lagrone" nil (l aa0 g r ow1 n iy0))
("lagrow" nil (l ae1 g r ow1))
("lags" nil (l ae1 g z))
("laguardia" nil (l ax g w aa1 r d iy0 ax))
("lague" nil (l aa1 g))
("laguna" nil (l ax g uw1 n ax))
("lagunas" nil (l ax g uw1 n ax z))
("lahaie" nil (l ae1 hh iy0))
("laham" nil (l ae1 hh ax m))
("lahaye" nil (l ae1 hh ey0))
("lahey" nil (l ey1 hh iy0))
("lahiff" nil (l ae1 hh ih0 f))
("lahm" nil (l ae1 m))
("lahman" nil (l aa1 m ax n))
("lahmann" nil (l aa1 m ax n))
("lahn" nil (l ae1 n))
("lahood" nil (l aa1 hh uh0 d))
("lahore" nil (l aa0 hh ao1 r))
("lahr" nil (l aa1 r))
("lahti" nil (l aa1 t iy0))
("lahue" nil (l ae1 hh y uw0))
("lai" nil (l ay1))
("laib" nil (l ey1 b))
("laible" nil (l ey1 b ax l))
("laibowitz" nil (l ey1 b ax w ih0 t s))
("laibrook" nil (l ey1 b r uh1 k))
("laiche" nil (l ey1 ch))
("laid" nil (l ey1 d))
("laidig" nil (l ey1 d ih0 g))
("laidlaw" nil (l ey1 d l ao1))
("laidler" nil (l ey1 d l er0))
("laidley" nil (l ey1 d l iy0))
("lail" nil (l ey1 l))
("laimbeer" nil (l ey1 m b ih1 r))
("lain" nil (l ey1 n))
("laine" nil (l ey1 n))
("laing" nil (l aa1 ih0 ng))
("lainhart" nil (l ay1 n hh aa0 r t))
("laino" nil (l ey1 n ow0))
("lair" nil (l eh1 r))
("laird" nil (l eh1 r d))
("lairmore" nil (l eh1 r m ao0 r))
("lairson" nil (l eh1 r s ax n))
("lais" nil (l ey1 z))
("laissez" nil (l ey1 z ey1))
("lait" nil (l ey1))
("laitinen" nil (l ay1 t ih0 n ax n))
("laity" nil (l ey1 ax t iy0))
("lajeunesse" nil (l ae1 y oy0 n eh0 s))
("lajoie" nil (l ae1 jh w aa0))
("lajous" nil (l ax zh uw1 s))
("lak" nil (l ae1 k))
("lakatos" nil (l ae1 k ax t ow0 z))
("lake" nil (l ey1 k))
("lakefield" nil (l ey1 k f iy1 l d))
("lakefront" nil (l ey1 k f r ah1 n t))
("lakeland" nil (l ey1 k l ax n d))
("lakeman" nil (l ey1 k m ax n))
("laker" nil (l ey1 k er0))
("lakers" nil (l ey1 k er0 z))
("lakes" nil (l ey1 k s))
("lakeshore" nil (l ey1 k sh ao1 r))
("lakeside" nil (l ey1 k s ay1 d))
("lakeview" nil (l ey1 k v y uw1))
("lakewood" nil (l ey1 k w uh1 d))
("lakey" nil (l ey1 k iy0))
("lakin" nil (l ae1 k ih0 n))
("lakins" nil (l ae1 k ih0 n z))
("lal" nil (l ae1 l))
("lala" nil (l aa1 l ax))
("lalande" nil (l aa0 l aa1 n d iy0))
("laliberte" nil (l aa0 l iy0 b eh1 r t iy0))
("lalit" nil (l ax l iy1 t))
("lalita" nil (l ax l iy1 t ax))
("lalitesh" nil (l aa1 l ih1 t eh1 sh))
("lall" nil (l ao1 l))
("lalla" nil (l ae1 l ax))
("lalley" nil (l ae1 l iy0))
("lalli" nil (l ae1 l iy0))
("lallie" nil (l ao1 l iy0))
("lallier" nil (l ae1 l iy0 er0))
("lally" nil (l ae1 l iy0))
("lalonde" nil (l aa0 l ow1 n d iy0))
("lalone" nil (l aa0 l ow1 n iy0))
("lalor" nil (l ae0 l ao1 r))
("lam" nil (l ae1 m))
("lama" nil (l aa1 m ax))
("lamacchia" nil (l aa0 m aa1 k iy0 ax))
("lamagna" nil (l aa0 m aa1 g n ax))
("lamaist" nil (l aa1 m ay0 ih0 s t))
("laman" nil (l ey1 m ax n))
("lamanna" nil (l ae1 m ih0 n ax))
("lamantia" nil (l aa0 m aa1 n sh ax))
("lamar" nil (l ax m aa1 r))
("lamarca" nil (l aa0 m aa1 r k ax))
("lamarche" nil (l aa0 m aa1 r k iy0))
("lamarr" nil (l aa0 m aa1 r))
("lamarre" nil (l aa0 m aa1 r iy0))
("lamartina" nil (l aa0 m aa0 r t iy1 n ax))
("lamaster" nil (l aa1 m ae0 s t er0))
("lamastus" nil (l ae1 m ax s t ih0 s))
("lamattina" nil (l aa0 m aa0 t iy1 n ax))
("lamaur" nil (l ax m ao1 r))
("lamay" nil (l ae1 m ey0))
("lamb" nil (l ae1 m))
("lambaste" nil (l ae1 m b ae1 s t))
("lambasted" nil (l ae1 m b ae1 s t ih0 d))
("lambastes" nil (l ae1 m b ae1 s t s))
("lambasting" nil (l ae0 m b ey1 s t ih0 ng))
("lambda" nil (l ae1 m d ax))
("lambdin" nil (l ae1 m d ih0 n))
("lambe" nil (l ae1 m))
("lamberg" nil (l ae1 m b er0 g))
("lamberson" nil (l ae1 m b er0 s ax n))
("lambert" nil (l ae1 m b er0 t))
("lamberth" nil (l ae1 m b er0 th))
("lamberti" nil (l aa0 m b eh1 r t iy0))
("lamberto" nil (l aa0 m b eh1 r t ow1))
("lamberton" nil (l ae1 m b er0 t ax n))
("lambertson" nil (l ae1 m b er0 t s ax n))
("lamberty" nil (l ae1 m b er0 t iy0))
("lambeth" nil (l ae1 m b ih0 th))
("lambiase" nil (l aa0 m b iy0 aa1 s iy0))
("lambie" nil (l ae1 m b iy0))
("lambing" nil (l ae1 m ih0 ng))
("lamborghini" nil (l ae1 m b er0 g iy1 n iy0))
("lamborghinis" nil (l ae1 m b er0 g iy1 n iy0 z))
("lamborn" nil (l ax m b ao1 r n))
("lamboy" nil (l ae1 m b oy1))
("lambrecht" nil (l ae1 m b r ih0 k t))
("lambright" nil (l ae1 m b r ay1 t))
("lambro" nil (l ae1 m b r ow0))
("lambros" nil (l ae1 m b r ow0 z))
("lambs" nil (l ae1 m z))
("lambsdorff" nil (l ae1 m z d ao0 r f))
("lambson" nil (l ae1 m s ax n))
("lamby" nil (l ae1 m b iy0))
("lame" nil (l ey1 m))
("lamellar" nil (l ax m eh1 l er0))
("lamely" nil (l ey1 m l iy0))
("lamendola" nil (l aa0 m eh0 n d ow1 l ax))
("lament" nil (l ax m eh1 n t))
("lamentable" nil (l ax m eh1 n t ax b ax l))
("lamented" nil (l ax m eh1 n t ih0 d))
("lamenting" nil (l ax m eh1 n t ih0 ng))
("laments" nil (l ax m eh1 n t s))
("lamer" nil (l ey1 m er0))
("lamere" nil (l aa0 m eh1 r ey0))
("lamers" nil (l ey1 m er0 z))
("lamey" nil (l ey1 m iy0))
("lamia" nil (l ey1 m iy0 ax))
("lamica" nil (l ae1 m ih0 k ax))
("lamie" nil (l ey1 m iy0))
("lamina" nil (l ae1 m ax n ax))
("laminack" nil (l ae1 m ih0 n ax k))
("laminated" nil (l ae1 m ax n ey1 t ax d))
("laminates" nil (l ae1 m ax n ax t s))
("laminating" nil (l ae1 m ax n ey1 t ih0 ng))
("lamine" nil (l ax m iy1 n))
("lamirande" nil (l aa0 m ih0 r aa1 n d iy0))
("lamison" nil (l ae1 m ih0 s ax n))
("lamke" nil (l ae1 m k iy0))
("lamkin" nil (l ae1 m k ih0 n))
("lamle" nil (l ae1 m l iy0))
("lamm" nil (l ae1 m))
("lamme" nil (l ae1 m))
("lammers" nil (l ae1 m er0 z))
("lammert" nil (l ae1 m er0 t))
("lammey" nil (l ae1 m iy0))
("lammi" nil (l ae1 m iy0))
("lammon" nil (l ae1 m ax n))
("lammond" nil (l ae1 m ax n d))
("lammons" nil (l ae1 m ax n z))
("lamon" nil (l aa0 m ao1 n))
("lamond" nil (l ax m aa1 n d))
("lamonica" nil (l aa0 m ow0 n iy1 k ax))
("lamons" nil (l aa1 m ow0 n z))
("lamont" nil (l ax m aa1 n t))
("lamontagne" nil (l aa1 m aa0 n t ey1 n))
("lamonte" nil (l aa0 m aa1 n t))
("lamora" nil (l aa0 m ao1 r ax))
("lamore" nil (l aa1 m ao0 r))
("lamoreaux" nil (l ae1 m er0 ow0))
("lamoreux" nil (l ae1 m er0 ow0))
("lamorte" nil (l aa0 m ao1 r t iy0))
("lamos" nil (l aa1 m ow0 s))
("lamothe" nil (l ae1 m ax dh))
("lamott" nil (l ax m aa1 t))
("lamotta" nil (l aa0 m ow1 t ax))
("lamotte" nil (l ax m ao1 t))
("lamountain" nil (l ae1 m uw0 n t ey0 n))
("lamoureaux" nil (l ae1 m uh0 r ow0))
("lamoureux" nil (l ae1 m uh0 r ow0))
("lamp" nil (l ae1 m p))
("lampe" nil (l ae1 m p))
("lampert" nil (l ae1 m p er1 t))
("lampf" nil (l ae1 m p f))
("lamphear" nil (l ae1 m f er0))
("lamphere" nil (l ae1 m f er0))
("lamphier" nil (l ae1 m f iy0 er0))
("lampi" nil (l ae1 m p iy0))
("lamping" nil (l ae1 m p ih0 ng))
("lampkin" nil (l ae1 m p k ih0 n))
("lampkins" nil (l ae1 m p k ih0 n z))
("lampl" nil (l ae1 m p ax l))
("lampley" nil (l ae1 m p l iy0))
("lamplighters" nil (l ae1 m p l ay1 t er0 z))
("lampman" nil (l ae1 m p m ax n))
("lampo" nil (l ae1 m p ow0))
("lampoon" nil (l ae0 m p uw1 n))
("lampooned" nil (l ae0 m p uw1 n d))
("lamport" nil (l ae1 m p ao1 r t))
("lampp" nil (l ae1 m p))
("lampposts" nil (l ae1 m p p ow1 s t s))
("lamprey" nil (l ae1 m p r iy0))
("lampreys" nil (l ae1 m p r iy0 z))
("lampron" nil (l ae1 m p r ax n))
("lampros" nil (l ae1 m p r ow0 z))
("lamps" nil (l ae1 m p s))
("lampson" nil (l ae1 m p s ax n))
("lampton" nil (l ae1 m p t ax n))
("lamson" nil (l ae1 m s ax n))
("lamston" nil (l ae1 m s t ax n))
("lamy" nil (l ey1 m iy0))
("lan" nil (l ae1 n))
("lana" nil (l ae1 n ax))
("lanagan" nil (l ae1 n ax g ae0 n))
("lanahan" nil (l ae1 n ax hh ae0 n))
("lanasa" nil (l aa0 n aa1 s ax))
("lancashire" nil (l ae1 ng k ax sh ay1 r))
("lancaster" nil (l ae1 n k ae1 s t er0))
("lancastrian" nil (l ae1 ng k ae1 s t r iy0 ax n))
("lance" nil (l ae1 n s))
("lancelet" nil (l ae1 n s l ax t))
("lancelot" nil (l ae1 n s ax l aa1 t))
("lanceolate" nil (l ae1 n s iy0 ax l ey1 t))
("lancer" nil (l ae1 n s er0))
("lancers" nil (l ae1 n s er0 z))
("lancet" nil (l ae1 n s ax t))
("lancets" nil (l ae1 n s ax t s))
("lancia" nil (l aa1 n ch ax))
("lancing" nil (l ae1 n s ih0 ng))
("lancit" nil (l ae1 n s ih0 t))
("lanclos" nil (l ae1 n k l ow0 z))
("lancome" nil (l ae1 n k ax m))
("lancon" nil (l ae1 ng k ax n))
("lancour" nil (l ax n k uh1 r))
("lanctot" nil (l ae1 ng k t ax t))
("lancz" nil (l ae1 n ch))
("land" nil (l ae1 n d))
("landa" nil (l ae1 n d ax))
("landau" nil (l ae1 n d aw1))
("landauer" nil (l ae1 n d aw1 er0))
("landay" nil (l ae1 n d ey1))
("landbank" nil (l ae1 n d b ae1 ng k))
("landberg" nil (l ae1 n d b er0 g))
("lande" nil (l ae1 n d))
("landeck" nil (l ae1 n d ih0 k))
("landed" nil (l ae1 n d ax d))
("landefeld" nil (l ae1 n d ih0 f eh0 l d))
("landell" nil (l ae0 n d eh1 l))
("landen" nil (l ae1 n d ax n))
("landenberger" nil (l ae1 n d ax n b er0 g er0))
("lander" nil (l ae1 n d er0))
("landeros" nil (l ae1 n d er0 ow0 z))
("landers" nil (l ae1 n d er0 z))
("landes" nil (l ae1 n d z))
("landesbank" nil (l ae1 n d iy0 z b ae1 ng k))
("landesman" nil (l ae1 n d ih0 s m ax n))
("landess" nil (l ae1 n d ax s))
("landfair" nil (l ae1 n d f eh1 r))
("landfall" nil (l ae1 n d f ao1 l))
("landfill" nil (l ae1 n d f ih1 l))
("landfilling" nil (l ae1 n d f ih0 l ih0 ng))
("landfills" nil (l ae1 n d f ih1 l z))
("landforms" nil (l ae1 n d f ao1 r m z))
("landfried" nil (l ae1 n d f er0 iy0 d))
("landgraf" nil (l ae1 n jh r ax f))
("landgrave" nil (l ae1 n d g r ey1 v))
("landgrebe" nil (l ae1 n jh r ih0 b))
("landgren" nil (l ae1 n d g r eh0 n))
("landholders" nil (l ae1 n d hh ow1 l d er0 z))
("landholding" nil (l ae1 n d hh ow1 l d ih0 ng))
("landholdings" nil (l ae1 n d hh ow1 l d ih0 ng z))
("landi" nil (l ae1 n d iy0))
("landin" nil (l ae1 n d ih0 n))
("landing" nil (l ae1 n d ih0 ng))
("landingham" nil (l ae1 n d ih0 ng hh ae1 m))
("landings" nil (l ae1 n d ih0 ng z))
("landini" nil (l aa0 n d iy1 n iy0))
("landino" nil (l aa0 n d iy1 n ow0))
("landis" nil (l ae1 n d ih0 s))
("landlady" nil (l ae1 n d l ey1 d iy0))
("landler" nil (l ae1 n d l er0))
("landless" nil (l ae1 n d l ax s))
("landlocked" nil (l ae1 n d l aa1 k t))
("landlord" nil (l ae1 n d l ao1 r d))
("landlords" nil (l ae1 n d l ao1 r d z))
("landman" nil (l ae1 n d m ax n))
("landmark" nil (l ae1 n d m aa1 r k))
("landmarks" nil (l ae1 n d m aa1 r k s))
("landmass" nil (l ae1 n d m ae1 s))
("landmesser" nil (l ae1 n d m eh1 s er0))
("lando" nil (l ae1 n d ow0))
("landolfi" nil (l aa0 n d ow1 l f iy0))
("landolt" nil (l ae1 n d ow1 l t))
("landon" nil (l ae1 n d ax n))
("landor" nil (l ae1 n d er0))
("landover" nil (l ae1 n d ow1 v er0))
("landow" nil (l ae1 n d ow0))
("landowner" nil (l ae1 n d ow1 n er0))
("landowners" nil (l ae1 n d ow1 n er0 z))
("landowski" nil (l ax n d ao1 f s k iy0))
("landreneau" nil (l ae1 n d r ih0 n ow0))
("landress" nil (l ae1 n d r ih0 s))
("landreth" nil (l ae1 n d r ih0 th))
("landrigan" nil (l ae1 n d r ih0 g ax n))
("landrith" nil (l ae1 n d r ih0 th))
("landrum" nil (l ae1 n d r ax m))
("landrus" nil (l ae1 n d r ax s))
("landry" nil (l ae1 n d r iy0))
("lands" nil (l ae1 n d z))
("landsat" nil (l ae1 n d s ae1 t))
("landsberg" nil (l ae1 n d z b er0 g))
("landsberger" nil (l ae1 n d z b er0 g er0))
("landsbergis" nil (l ae1 n d z b er0 g ax s))
("landscape" nil (l ae1 n d s k ey1 p))
("landscaped" nil (l ae1 n d s k ey1 p t))
("landscaper" nil (l ae1 n d s k ey1 p er0))
("landscapers" nil (l ae1 n d s k ey1 p er0 z))
("landscapes" nil (l ae1 n d s k ey1 p s))
("landscaping" nil (l ae1 n d s k ey1 p ih0 ng))
("landslide" nil (l ae1 n d s l ay1 d))
("landslides" nil (l ae1 n d s l ay1 d z))
("landsman" nil (l ae1 n d z m ax n))
("landstrom" nil (l ae1 n d s t r ax m))
("landt" nil (l ae1 n t))
("landward" nil (l ae1 n d w er0 d))
("landwehr" nil (l ae1 n d w ih0 r))
("landy" nil (l ae1 n d iy0))
("lane" nil (l ey1 n))
("lanehart" nil (l ey1 n hh aa1 r t))
("lanes" nil (l ey1 n z))
("lanese" nil (l aa0 n ey1 z iy0))
("lanete" nil (l ax n iy1 t))
("laneve" nil (l ax n iy1 v))
("laney" nil (l ey1 n iy0))
("lanfear" nil (l ae1 n f er0))
("lanford" nil (l ae1 n f er0 d))
("lang" nil (l ae1 ng))
("langan" nil (l ae1 ng g ax n))
("langbehn" nil (l ae1 ng b ih0 n))
("langbein" nil (l ae1 ng b ay0 n))
("langbo" nil (l ae1 ng b ow0))
("langdale" nil (l ae1 ng d ey1 l))
("langdon" nil (l ae1 ng d ax n))
("lange" nil (l ae1 ng))
("langel" nil (l ey1 ng g ax l))
("langeland" nil (l ey1 ng g ih0 l ax n d))
("langelier" nil (l ey1 n jh l iy0 er0))
("langella" nil (l ae0 n jh eh1 l er0))
("langen" nil (l ae1 ng ax n))
("langenbach" nil (l ey1 ng g ih0 n b aa0 k))
("langenberg" nil (l ae1 ng ax n b er0 g))
("langenderfer" nil (l ey1 ng g ih0 n d er0 f er0))
("langendorf" nil (l ey1 ng g ih0 n d ao0 r f))
("langenfeld" nil (l ey1 ng g ih0 n f eh0 l d))
("langer" nil (l ae1 ng er0))
("langerman" nil (l ae1 ng er0 m ax n))
("langevin" nil (l ey1 ng g ih0 v ih0 n))
("langfeldt" nil (l ae1 ng f eh0 l t))
("langfitt" nil (l ae1 ng f ih0 t))
("langford" nil (l ae1 ng f er0 d))
("langguth" nil (l ae1 n g uw0 th))
("langham" nil (l ae1 ng hh ae0 m))
("langhans" nil (l ae1 ng hh ax n z))
("langhoff" nil (l ae1 ng hh ao0 f))
("langhorne" nil (l ae1 ng hh ao0 r n))
("langhorst" nil (l ae1 ng hh ao0 r s t))
("langill" nil (l ae1 ng g ax l))
("langille" nil (l aa0 ng g iy1 l iy0))
("langlais" nil (l ax ng g l ey1))
("langland" nil (l ae1 ng l ax n d))
("langley" nil (l ae1 ng l iy0))
("langlinais" nil (l ae1 ng g l ih0 n ey0))
("langlois" nil (l ax ng l w aa1))
("langman" nil (l ae1 ng m ax n))
("langner" nil (l ae1 ng n er0))
("lango" nil (l aa1 ng g ow0))
("langone" nil (l aa0 ng g ow1 n iy0))
("langowski" nil (l ax ng g ao1 f s k iy0))
("langrehr" nil (l ae1 ng g r ih0 r))
("langridge" nil (l ae1 ng g r ih0 jh))
("langsam" nil (l ae1 ng s ax m))
("langsdorf" nil (l ae1 ng z d ao0 r f))
("langseth" nil (l ae1 ng s eh0 th))
("langsford" nil (l ae1 ng z f er0 d))
("langstaff" nil (l ae1 ng s t ae0 f))
("langston" nil (l ae1 ng s t ax n))
("langton" nil (l ae1 ng t ax n))
("language" nil (l ae1 ng g w ax jh))
("languages" nil (l ae1 ng g w ax jh ax z))
("languid" nil (l ae1 ng g w ax d))
("languish" nil (l ae1 ng g w ih0 sh))
("languished" nil (l ae1 ng g w ih0 sh t))
("languishes" nil (l ae1 ng g w ih0 sh ih0 z))
("languishing" nil (l ae1 ng g w ih0 sh ih0 ng))
("langwell" nil (l ae1 ng g w eh0 l))
("langworth" nil (l ae1 ng g w er0 th))
("langworthy" nil (l ae1 ng w er0 dh iy0))
("lanham" nil (l ae1 n ax m))
("lani" nil (l aa1 n iy0))
("lanier" nil (l ax n ih1 r))
("lanigan" nil (l ae1 n ih0 g ax n))
("laning" nil (l ey1 n ih0 ng))
("laningham" nil (l ae1 n ih0 ng hh ae1 m))
("lanius" nil (l ey1 n iy0 ih0 s))
("lank" nil (l ae1 ng k))
("lanka" nil (l ae1 ng k ax))
("lankan" nil (l ae1 ng k ax n))
("lankans" nil (l ae1 ng k ax n z))
("lankford" nil (l ae1 ng k f er0 d))
("lanktree" nil (l ae1 ng k t r iy1))
("lanky" nil (l ae1 ng k iy0))
("lanman" nil (l ae1 n m ax n))
("lann" nil (l ae1 n))
("lanna" nil (l ae1 n ax))
("lannan" nil (l ae1 n ax n))
("lannen" nil (l ae1 n ax n))
("lanners" nil (l ae1 n er0 z))
("lanni" nil (l ae1 n iy0))
("lanning" nil (l ae1 n ih0 ng))
("lannom" nil (l ae1 n ax m))
("lannon" nil (l ae1 n ax n))
("lanny" nil (l ae1 n iy0))
("lano" nil (l aa1 n ow0))
("lanolin" nil (l ae1 n ax l ax n))
("lanoue" nil (l ae1 n uw0 ey1))
("lanouette" nil (l ae1 n uw0 eh0 t))
("lanphear" nil (l ae1 n f ih1 r))
("lanpher" nil (l ae1 n f er0))
("lanphere" nil (l ae1 n f ih1 r))
("lanphier" nil (l ae1 n f ih0 r))
("lansberry" nil (l ae1 n z b eh1 r iy0))
("lansbury" nil (l ae1 n z b er1 iy0))
("lansdale" nil (l ae1 n z d ey1 l))
("lansdell" nil (l ae1 n z d ax l))
("lansdown" nil (l ae1 n z d aw0 n))
("lansdowne" nil (l ae1 n s d aw1 n))
("lanser" nil (l ae1 n s er0))
("lansford" nil (l ae1 n s f er0 d))
("lansing" nil (l ae1 n s ih0 ng))
("lansky" nil (l ae1 n s k iy0))
("lanston" nil (l ae1 n s t ax n))
("lant" nil (l ae1 n t))
("lantana" nil (l ae0 n t ae1 n ax))
("lanter" nil (l ae1 n t er0))
("lanterman" nil (l ae1 n t er0 m ax n))
("lantern" nil (l ae1 n t er0 n))
("lanterns" nil (l ae1 n t er0 n z))
("lanthanum" nil (l ae1 n th ax n ax m))
("lanthier" nil (l ae1 n th iy0 er0))
("lanting" nil (l ae1 n t ih0 ng))
("lantis" nil (l aa1 n t ih0 s))
("lantos" nil (l ae1 n t ow0 s))
("lantrip" nil (l ae1 n t r ih0 p))
("lantry" nil (l ae1 n t r iy0))
("lantz" nil (l ae1 n t s))
("lantzy" nil (l ae1 n t z iy0))
("lanum" nil (l ae1 n ax m))
("lanvin" nil (l ae1 n v ih0 n))
("lanyi" nil (l aa1 n y iy0))
("lanyon" nil (l ae1 n y ax n))
("lanz" nil (l ae1 n z))
("lanza" nil (l ae1 n z ax))
("lanzelotti" nil (l ae1 n z ax l aa1 t iy0))
("lanzer" nil (l ae1 n z er0))
("lanzet" nil (l ae1 n z ax t))
("lanzi" nil (l ae1 n z iy0))
("lanzillo" nil (l ae1 n z ih1 l ow0))
("lanzo" nil (l ae1 n z ow0))
("lao" nil (l aw1))
("laos" nil (l aa1 ow0 s))
("laotian" nil (l ey0 ow1 sh ax n))
("lap" nil (l ae1 p))
("lapage" nil (l aa1 p ih0 jh))
("lapaglia" nil (l aa0 p aa1 g l iy0 ax))
("lapalme" nil (l aa0 p aa1 l m iy0))
("lapan" nil (l ax p ae1 n))
("laparoscopic" nil (l ae1 p er0 aa1 s k ax p ih0 k))
("lapdog" nil (l ae1 p d ao1 g))
("lape" nil (l ey1 p))
("lapel" nil (l ax p eh1 l))
("lapenna" nil (l aa0 p eh1 n ax))
("lapenta" nil (l aa0 p eh1 n t ax))
("laperle" nil (l ae1 p er0 ax l))
("lapeyrouse" nil (l ae1 p ih0 r uw0 s))
("lapham" nil (l ae1 f ax m))
("lapiana" nil (l aa0 p iy0 ae1 n ax))
("lapidary" nil (l ae1 p ax d eh1 r iy0))
("lapides" nil (l ax p ay1 d z))
("lapidus" nil (l ae1 p ih0 d ih0 s))
("lapier" nil (l ey1 p iy0 er0))
("lapierre" nil (l ae1 p iy0 eh0 r))
("lapin" nil (l ae1 p ih0 n))
("lapine" nil (l aa0 p iy1 n iy0))
("lapinski" nil (l ax p ih1 n s k iy0))
("lapinsky" nil (l ax p ih1 n s k iy0))
("lapka" nil (l aa1 p k ax))
("laplaca" nil (l ax p l aa1 k ax))
("lapland" nil (l ae1 p l ae1 n d))
("laplant" nil (l ax p l ao1 n t))
("laplante" nil (l ax p l aa1 n t iy0))
("laplume" nil (l ax p l uw1 m))
("lapoint" nil (l ax p oy1 n t))
("lapointe" nil (l ax p oy1 n t))
("lapolla" nil (l ax p ow1 l ax))
("laporta" nil (l ax p ao1 r t ax))
("laporte" nil (l ax p ao1 r t))
("lapp" nil (l ae1 p))
("lappe" nil (l ae1 p))
("lapped" nil (l ae1 p t))
("lappen" nil (l ae1 p ax n))
("lappin" nil (l ax p ae1 n))
("lapping" nil (l ae1 p ih0 ng))
("lapps" nil (l ae1 p s))
("laprade" nil (l aa0 p r aa1 d iy0))
("laprairie" nil (l ax p r eh1 r iy0))
("lapre" nil (l ey1 p er0))
("laprise" nil (l aa1 p r ay0 z))
("laps" nil (l ae1 p s))
("lapse" nil (l ae1 p s))
("lapsed" nil (l ae1 p s t))
("lapses" nil (l ae1 p s ih0 z))
("lapsing" nil (l ae1 p s ih0 ng))
("lapsley" nil (l ae1 p s l iy0))
("laptop" nil (l ae1 p t aa1 p))
("laptops" nil (l ae1 p t aa1 p s))
("lar" nil (l aa1 r))
("lara" nil (l aa1 r ax))
("larabee" nil (l ae1 r ax b iy0))
("laraia" nil (l aa0 r aa1 y ax))
("laraine" nil (l er0 ey1 n))
("laramee" nil (l ae1 r ax m iy0))
("laramie" nil (l eh1 r ax m iy0))
("laramore" nil (l ae1 r ax m ao1 r))
("laraway" nil (l ae1 r ax w ey1))
("larcenous" nil (l aa1 r s ax n ax s))
("larceny" nil (l aa1 r s ax n iy0))
("larch" nil (l aa1 r ch))
("larche" nil (l aa1 r ch iy0))
("larchmont" nil (l aa1 r ch m aa1 n t))
("larco" nil (l aa1 r k ow1))
("larcom" nil (l aa1 r k aa1 m))
("lard" nil (l aa1 r d))
("larded" nil (l aa1 r d ih0 d))
("larder" nil (l aa1 r d er0))
("lardner" nil (l aa1 r d n er0))
("lare" nil (l eh1 r))
("lareau" nil (l er0 ow1))
("laredo" nil (l er0 ey1 d ow0))
("lareina" nil (l aa0 r ey0 iy1 n ax))
("larena" nil (l aa0 r eh1 n ax))
("larew" nil (l ae1 r uw0))
("large" nil (l aa1 r jh))
("largely" nil (l aa1 r jh l iy0))
("largen" nil (l aa1 r g ax n))
("largent" nil (l aa1 r jh ax n t))
("larger" nil (l aa1 r jh er0))
("largess" nil (l aa1 r jh ax s))
("largesse" nil (l aa0 r g eh1 s))
("largest" nil (l aa1 r jh ax s t))
("largo" nil (l aa1 r g ow0))
("lariat" nil (l eh1 r iy0 ax t))
("lariccia" nil (l aa0 r iy1 ch ax))
("larijani" nil (l eh1 r ih0 jh aa1 n iy0))
("larimer" nil (l ae1 r ih0 m er0))
("larimore" nil (l aa0 r iy1 m ao0 r))
("larina" nil (l aa0 r iy1 n ax))
("larine" nil (l aa0 r iy1 n iy0))
("larios" nil (l aa0 r iy1 ow0 z))
("lariscy" nil (l er0 ih1 s k iy0))
("larison" nil (l ae1 r ih0 s ax n))
("larissa" nil (l aa0 r iy1 s ax))
("larivee" nil (l ae1 r ih0 v iy0))
("lariviere" nil (l ae1 r ih0 v iy0 eh0 r))
("larizza" nil (l er0 ih1 z ax))
("lark" nil (l aa1 r k))
("larke" nil (l aa1 r k))
("larkey" nil (l aa1 r k iy0))
("larkin" nil (l aa1 r k ih0 n))
("larkins" nil (l aa1 r k ih0 n z))
("larks" nil (l aa1 r k s))
("larkspur" nil (l aa1 r k s p er1))
("larmer" nil (l aa1 r m er0))
("larmon" nil (l aa1 r m ax n))
("larmore" nil (l aa1 r m ao0 r))
("larned" nil (l aa1 r n d))
("larner" nil (l aa1 r n er0))
("larocca" nil (l aa0 r ow1 k ax))
("larocco" nil (l aa0 r ow1 k ow0))
("laroche" nil (l aa0 r ow1 sh))
("larochelle" nil (l ae1 r ax sh ax l))
("larock" nil (l ae1 r ax k))
("larocque" nil (l er0 ow1 k))
("laroe" nil (l ae1 r ow0))
("laroque" nil (l er0 ow1 k))
("larosa" nil (l aa0 r ow1 s ax))
("larose" nil (l aa0 r ow1 s iy0))
("larosiere" nil (l aa0 r ow0 s y eh1 r))
("larouche" nil (l aa0 r uw1 sh))
("laroux" nil (l er0 uw1))
("larowe" nil (l ae1 r aw0))
("larrabee" nil (l ae1 r ax b iy0))
("larrance" nil (l ae1 r ax n s))
("larrea" nil (l ae1 r iy0 ax))
("larrick" nil (l ae1 r ih0 k))
("larrimore" nil (l aa0 r iy1 m ao0 r))
("larrison" nil (l ae1 r ih0 s ax n))
("larrivee" nil (l ae1 r ih0 v iy0))
("larroquette" nil (l aa1 r ow0 k eh1 t))
("larrow" nil (l ae1 r ow0))
("larry" nil (l eh1 r iy0))
("lars" nil (l aa1 r z))
("larsen" nil (l aa1 r s ax n))
("larsh" nil (l aa1 r sh))
("larson" nil (l aa1 r s ax n))
("larsson" nil (l aa1 r s ax n))
("larue" nil (l er0 uw1))
("larussa" nil (l aa0 r uw1 s ax))
("larusso" nil (l aa0 r uw1 s ow0))
("larva" nil (l aa1 r v ax))
("larvae" nil (l aa1 r v iy0))
("larval" nil (l aa1 r v ax l))
("lary" nil (l eh1 r iy0))
("laryngeal" nil (l er0 ih1 n jh iy0 ax l))
("laryngitis" nil (l eh1 r ax n jh ay1 t ax s))
("larynx" nil (l eh1 r ih0 ng k s))
("larzelere" nil (l er0 z eh1 l er0))
("las" nil (l aa1 s))
("lasagna" nil (l aa0 s aa1 n y ax))
("lasala" nil (l aa0 s aa1 l ax))
("lasalle" nil (l ax s ae1 l))
("lasater" nil (l ae1 s ey0 t er0))
("lasca" nil (l aa1 s k ax))
("lascala" nil (l aa0 s k aa1 l ax))
("lasch" nil (l ae1 sh))
("lascivious" nil (l ax s ih1 v iy0 ax s))
("lasco" nil (l aa1 s k ow0))
("lascola" nil (l aa0 s k ow1 l ax))
("lase" nil (l ey1 z))
("lasecki" nil (l ax s eh1 t s k iy0))
("lasek" nil (l aa1 s eh0 k))
("laser" nil (l ey1 z er0))
("lasers" nil (l ey1 z er0 z))
("lasersonics" nil (l aa0 s er0 s aa1 n ih0 k s))
("laseter" nil (l ae1 s iy0 t er0))
("lash" nil (l ae1 sh))
("lashbrook" nil (l ae1 sh b r uh1 k))
("lashed" nil (l ae1 sh t))
("lasher" nil (l ae1 sh er0))
("lashes" nil (l ae1 sh ih0 z))
("lashing" nil (l ae1 sh ih0 ng))
("lashings" nil (l ae1 sh ih0 ng z))
("lashlee" nil (l ae1 sh l iy1))
("lashley" nil (l ae1 sh l iy0))
("lashomb" nil (l ae1 sh ax m))
("lashua" nil (l ae1 sh uw0 ae0))
("lashway" nil (l ae1 sh w ey1))
("lasiter" nil (l ae1 s ay0 t er0))
("lask" nil (l ae1 s k))
("laska" nil (l aa1 s k ax))
("laske" nil (l ey1 s k))
("lasker" nil (l ae1 s k er0))
("laskey" nil (l ae1 s k iy0))
("laski" nil (l aa1 s k iy0))
("laskin" nil (l ae1 s k ih0 n))
("lasko" nil (l ae1 s k ow0))
("laskowski" nil (l ax s k ao1 f s k iy0))
("lasky" nil (l ae1 s k iy0))
("lasley" nil (l ae1 s l iy0))
("laslo" nil (l aa1 s l ow0))
("lasmo" nil (l ae1 s m ow0))
("lasorda" nil (l ax s ao1 r d ax))
("lasorsa" nil (l aa0 s ao1 r s ax))
("lasota" nil (l aa0 s ow1 t ax))
("laspina" nil (l aa0 s p iy1 n ax))
("lass" nil (l ae1 s))
("lasseigne" nil (l ax s ey1 n y))
("lassen" nil (l ae1 s ax n))
("lasser" nil (l ae1 s er0))
("lasseter" nil (l ae1 s iy0 t er0))
("lassetter" nil (l ae1 s ih0 t er0))
("lassie" nil (l ae1 s iy0))
("lassila" nil (l aa0 s iy1 l ax))
("lassiter" nil (l ae1 s ay0 t er0))
("lassman" nil (l ae1 s m ax n))
("lassner" nil (l ae1 s n er0))
("lasso" nil (l ae1 s ow0))
("lasswell" nil (l ae1 s w eh1 l))
("last" nil (l ae1 s t))
("lasted" nil (l ae1 s t ax d))
("laster" nil (l ae1 s t er0))
("lastest" nil (l ae1 s t ax s t))
("lasting" nil (l ae1 s t ih0 ng))
("lastinger" nil (l ae1 s t ih0 ng er0))
("lastly" nil (l ae1 s t l iy0))
("lastra" nil (l ae1 s t r ax))
("lastrapes" nil (l aa0 s t r aa1 p eh0 s))
("lasts" nil (l ae1 s t s))
("lasumma" nil (l ax s uw1 m ax))
("lasure" nil (l aa1 zh er0))
("laswell" nil (l ae1 s w eh1 l))
("laszlo" nil (l ae1 z l ow0))
("lat" nil (l aa1 t))
("lata" nil (l aa1 t ax))
("lataif" nil (l aa0 t ey1 f))
("latanze" nil (l ax t aa1 n z ey1))
("latch" nil (l ae1 ch))
("latchaw" nil (l ae1 ch ao0))
("latched" nil (l ae1 ch t))
("latches" nil (l ae1 ch ax z))
("latchford" nil (l ae1 ch f er0 d))
("latching" nil (l ae1 ch ih0 ng))
("latchkey" nil (l ae1 ch k iy1))
("late" nil (l ey1 t))
("latecomers" nil (l aa0 t eh1 k ax m er0 z))
("latella" nil (l aa0 t eh1 l ax))
("lately" nil (l ey1 t l iy0))
("latency" nil (l ey1 t ax n s iy0))
("latendresse" nil (l ae1 t ih0 n d r eh0 s))
("lateness" nil (l ey1 t n ax s))
("latent" nil (l ey1 t ax n t))
("later" nil (l ey1 t er0))
("lateral" nil (l ae1 t er0 ax l))
("laterites" nil (l ae1 t er0 ay1 t s))
("laterriere" nil (l aa0 t eh1 r y eh1 r))
("laterza" nil (l aa0 t eh1 r z ax))
("latest" nil (l ey1 t ax s t))
("latex" nil (l ey1 t eh1 k s))
("latexes" nil (l ey1 t eh0 k s ax z))
("lath" nil (l ae1 th))
("latham" nil (l ae1 th ax m))
("lathan" nil (l ae1 th ax n))
("lathe" nil (l ey1 dh))
("lathem" nil (l ae1 th ih0 m))
("lather" nil (l ae1 dh er0))
("lathers" nil (l ae1 dh er0 z))
("lathes" nil (l ey1 dh z))
("lathon" nil (l ae1 th ax n))
("lathouris" nil (l ax th uw1 r ih0 s))
("lathrop" nil (l ae1 th r ax p))
("latigo" nil (l ax t iy1 g ow0))
("latimer" nil (l ae1 t ax m er0))
("latimore" nil (l aa0 t iy1 m ao0 r))
("latin" nil (l ae1 t ax n))
("latina" nil (l aa0 t iy1 n ax))
("latinate" nil (l ae1 t ax n ey1 t))
("latini" nil (l aa0 t iy1 n iy0))
("latino" nil (l aa0 t iy1 n ow0))
("latinos" nil (l aa0 t iy1 n ow0 s))
("latins" nil (l ae1 t ax n z))
("latiolais" nil (l ae1 t iy0 ax l ey0))
("latitude" nil (l ae1 t ax t uw1 d))
("latitudes" nil (l ae1 t ax t uw1 d z))
("latitudinal" nil (l ae1 t ax t uw1 d ax n ax l))
("latka" nil (l aa1 t k ax))
("latner" nil (l ae1 t n er0))
("lato" nil (l aa1 t ow0))
("laton" nil (l ae1 t ax n))
("latona" nil (l aa0 t ow1 n ax))
("latonia" nil (l aa0 t ow1 n iy0 ax))
("latorre" nil (l aa0 t ao1 r iy0))
("latour" nil (l ax t uh1 r))
("latourette" nil (l ae1 t uh0 r eh0 t))
("latrobe" nil (l aa0 t r ow1 b))
("lats" nil (l aa1 t s))
("latsch" nil (l ae1 ch))
("latshaw" nil (l ae1 ch ao1))
("latson" nil (l ae1 t s ax n))
("latta" nil (l ae1 t ax))
("lattanzi" nil (l aa0 t aa1 n z iy0))
("lattanzio" nil (l aa0 t aa1 n z iy0 ow0))
("latter" nil (l ae1 t er0))
("lattice" nil (l ae1 t ax s))
("latticework" nil (l ae1 t ax s w er1 k))
("lattimer" nil (l ae1 t ax m er0))
("lattimore" nil (l aa0 t iy1 m ao0 r))
("lattin" nil (l ae1 t ih0 n))
("lattner" nil (l ae1 t n er0))
("latty" nil (l ae1 t iy0))
("latulippe" nil (l aa0 t uw0 l iy1 p iy0))
("latvala" nil (l aa0 t v aa1 l ax))
("latvia" nil (l ae1 t v iy0 ax))
("latvian" nil (l ae1 t v iy0 ax n))
("latvians" nil (l ae1 t v iy0 ax n z))
("latz" nil (l ae1 t s))
("lau" nil (l aw1))
("laub" nil (l ao1 b))
("laubach" nil (l aw1 b aa1 k))
("laubacher" nil (l aw1 b aa1 k er0))
("laube" nil (l ao1 b))
("laubenstein" nil (l aw1 b ax n s t ay0 n))
("laubenthal" nil (l aw1 b ih0 n th ax l))
("lauber" nil (l aw1 b er0))
("laubhan" nil (l aw1 b hh ax n))
("laubscher" nil (l aw1 b sh er0))
("lauck" nil (l ao1 k))
("laud" nil (l ao1 d))
("laudable" nil (l ao1 d ax b ax l))
("laudanum" nil (l ao1 d ax n ax m))
("laudatory" nil (l ao1 d ax t ao1 r iy0))
("laude" nil (l ao1 d))
("lauded" nil (l ao1 d ih0 d))
("laudenslager" nil (l aw1 d ih0 n s l ey0 g er0))
("lauder" nil (l ao1 d er0))
("lauderback" nil (l aw1 d er0 b ae0 k))
("lauderbaugh" nil (l ao0 d er1 b ao0))
("lauderdale" nil (l ao1 d er0 d ey1 l))
("laudicina" nil (l ao0 d iy0 ch iy1 n ax))
("lauding" nil (l ao1 d ih0 ng))
("laudise" nil (l ao1 d ay1 z))
("laudon" nil (l ao1 d ax n))
("lauds" nil (l ao1 d z))
("laue" nil (l aw1 iy0))
("lauer" nil (l aw1 er0))
("lauerman" nil (l aw1 er0 m ax n))
("lauf" nil (l ao1 f))
("laufenberg" nil (l aw1 f ax n b er0 g))
("laufer" nil (l ao1 f er0))
("lauffer" nil (l ao1 f er0))
("laugh" nil (l ae1 f))
("laughable" nil (l ae1 f ax b ax l))
("laughed" nil (l ae1 f t))
("laughery" nil (l ae1 f er0 iy0))
("laughing" nil (l ae1 f ih0 ng))
("laughinghouse" nil (l ae1 f ih0 ng hh aw1 s))
("laughingly" nil (l ae1 f ih0 ng l iy0))
("laughingstock" nil (l ae1 f ih0 ng s t aa1 k))
("laughlin" nil (l aa1 k l ih0 n))
("laughman" nil (l ae1 f m ax n))
("laughner" nil (l ae1 f n er0))
("laughrey" nil (l ae1 f r iy0))
("laughridge" nil (l ae1 f r ih1 jh))
("laughs" nil (l ae1 f s))
("laughter" nil (l ae1 f t er0))
("laughton" nil (l ao1 t ax n))
("lauinger" nil (l aw1 ih0 ng er0))
("lauman" nil (l aw1 m ax n))
("laumann" nil (l aw1 m ax n))
("laun" nil (l ao1 n))
("launcelot" nil (l ao1 n s ih0 l aa1 t))
("launch" nil (l ao1 n ch))
("launched" nil (l ao1 n ch t))
("launcher" nil (l ao1 n ch er0))
("launchers" nil (l ao1 n ch er0 z))
("launches" nil (l ao1 n ch ax z))
("launching" nil (l ao1 n ch ih0 ng))
("launchings" nil (l ao1 n ch ih0 ng z))
("launder" nil (l ao1 n d er0))
("laundered" nil (l ao1 n d er0 d))
("launderer" nil (l ao1 n d er0 er0))
("launderers" nil (l ao1 n d er0 er0 z))
("laundering" nil (l ao1 n d er0 ih0 ng))
("laundries" nil (l ao1 n d r iy0 z))
("laundromat" nil (l ao1 n d r ax m ae1 t))
("laundry" nil (l ao1 n d r iy0))
("launer" nil (l ao1 n er0))
("launius" nil (l aw1 n iy0 ih0 s))
("lauper" nil (l aw1 p er0))
("laur" nil (l ao1 r))
("laura" nil (l ao1 r ax))
("laurance" nil (l ao1 r ax n s))
("laurdan" nil (l ao1 r d ax n))
("laure" nil (l ao1 r))
("laureano" nil (l ao0 r iy1 n ow0))
("laureate" nil (l ao1 r iy0 ax t))
("laureates" nil (l ao1 r iy0 ax t s))
("laureen" nil (l ao0 r iy1 n))
("laurel" nil (l ao1 r ax l))
("laurels" nil (l ao1 r ax l z))
("lauren" nil (l ao1 r ax n))
("laurena" nil (l ao0 r eh1 n ax))
("laurence" nil (l ao1 r ax n s))
("laurene" nil (l ao0 r iy1 n))
("laurens" nil (l ao1 r ax n z))
("laurent" nil (l ao1 r ax n t))
("laurentian" nil (l ao0 r eh1 n sh ax n))
("laurentiis" nil (l ao1 r ax n t iy1 z))
("laurenz" nil (l aw1 r ey0 n z))
("lauretarian" nil (l ao1 r ih0 t eh1 r iy0 ax n))
("lauretta" nil (l er0 eh1 t ax))
("laurette" nil (l ao0 r eh1 t))
("lauria" nil (l ao1 r iy0 ax))
("lauriats" nil (l aa1 r iy0 ih0 t s))
("laurice" nil (l ao1 r ih0 s))
("lauricella" nil (l ao1 r ih0 s eh1 l ax))
("laurich" nil (l aw1 r ih0 k))
("lauridsen" nil (l ao0 r ih1 d s ax n))
("laurie" nil (l ao1 r iy0))
("laurin" nil (l ao1 r ih0 n))
("laurino" nil (l ao0 r iy1 n ow0))
("lauritsen" nil (l aw1 r ih0 t s ax n))
("lauritzen" nil (l aw1 r ih0 t z ax n))
("lauro" nil (l ao1 r ow0))
("laursen" nil (l aw1 r s ax n))
("laury" nil (l ao1 r iy0))
("lausanne" nil (l ao1 s ae1 n))
("lausch" nil (l aw1 sh))
("laut" nil (l ao1 t))
("lauten" nil (l aw1 t ax n))
("lautenbach" nil (l aw1 t ax n b aa0 k))
("lautenberg" nil (l aw1 t ax n b er0 g))
("lautenschlager" nil (l aw1 t ih0 n sh l ey0 g er0))
("lauter" nil (l aw1 t er0))
("lauterbach" nil (l aw1 t er0 b aa0 k))
("lauth" nil (l ao1 th))
("lautner" nil (l aw1 t n er0))
("lautzenheiser" nil (l aw1 t z ax n hh ay0 z er0))
("lauver" nil (l aw1 v er0))
("laux" nil (l ao1 k s))
("lauzon" nil (l aw1 z ax n))
("lava" nil (l aa1 v ax))
("laval" nil (l ax v aa1 l))
("lavalle" nil (l aa0 v aa1 l iy0))
("lavallee" nil (l ae1 v ax l iy0))
("lavalley" nil (l ae1 v ax l iy0))
("lavan" nil (l ey1 v ax n))
("lavarnway" nil (l ax v aa1 r n w ey0))
("lavatories" nil (l ae1 v ax t ao1 r iy0 z))
("lavatory" nil (l ae1 v ax t ao1 r iy0))
("lave" nil (l ey1 v))
("lavecchia" nil (l aa0 v eh1 k iy0 ax))
("laveda" nil (l aa0 v ey1 d ax))
("lavell" nil (l aa0 v ey1 l))
("lavelle" nil (l ax v eh1 l))
("lavely" nil (l ey1 v l iy0))
("laven" nil (l ey1 v ax n))
("lavender" nil (l ae1 v ax n d er0))
("laventhol" nil (l ey1 v ax n th aa0 l))
("laventure" nil (l aa0 v eh1 n ch er0))
("laver" nil (l ey1 v er0))
("laverdiere" nil (l aa0 v er0 d ih1 r iy0))
("laverdure" nil (l aa0 v er0 d uh1 r iy0))
("lavere" nil (l aa0 v eh1 r ey0))
("lavergne" nil (l ax v er1 n y))
("laverne" nil (l ax v er1 n))
("lavers" nil (l ey1 v er0 z))
("laverty" nil (l ae1 v er0 t iy0))
("lavery" nil (l ey1 v er0 iy0))
("lavetta" nil (l aa0 v eh1 t ax))
("lavette" nil (l ax v eh1 t))
("lavey" nil (l ey1 v iy0))
("lavi" nil (l ey1 v iy0))
("lavie" nil (l ey1 v iy0))
("lavigne" nil (l ax v ih1 n y))
("lavin" nil (l ae1 v ih0 n))
("lavina" nil (l aa0 v iy1 n ax))
("lavinder" nil (l ae1 v ih0 n d er0))
("lavine" nil (l aa0 v iy1 n iy0))
("lavinia" nil (l ax v ih1 n y ax))
("laviolette" nil (l ae1 v iy0 ow0 l eh1 t))
("lavis" nil (l ae1 v ih0 s))
("lavish" nil (l ae1 v ih0 sh))
("lavished" nil (l ae1 v ih0 sh t))
("lavishes" nil (l ae1 v ih0 sh ax z))
("lavishing" nil (l ae1 v ih0 sh ih0 ng))
("lavishly" nil (l ae1 v ih0 sh l iy0))
("lavoie" nil (l ax v w aa1))
("lavorgna" nil (l ax v ao1 r n ax))
("lavoro" nil (l ax v ao1 r ow0))
("lavoy" nil (l ax v oy1))
("lavy" nil (l ey1 v iy0))
("law" nil (l ao1))
("lawall" nil (l aw1 ax l))
("laware" nil (l ax w eh1 r))
("lawbreakers" nil (l ao1 b r ey1 k er0 z))
("lawbreaking" nil (l ao1 b r ey1 k ih0 ng))
("lawes" nil (l ao1 z))
("lawford" nil (l ao1 f er0 d))
("lawful" nil (l ao1 f ax l))
("lawfully" nil (l ao1 f ax l iy0))
("lawhead" nil (l ao1 hh eh1 d))
("lawhon" nil (l ao1 hh ax n))
("lawhorn" nil (l ae1 w er0 n))
("lawhorne" nil (l ae1 w er0 n))
("lawing" nil (l ao1 ih0 ng))
("lawler" nil (l ao1 l er0))
("lawless" nil (l ao1 l ax s))
("lawlessness" nil (l ao1 l ax s n ax s))
("lawley" nil (l ao1 l iy0))
("lawlis" nil (l ao1 l ih0 s))
("lawlor" nil (l ao1 l er0))
("lawmaker" nil (l ao1 m ey1 k er0))
("lawmakers" nil (l ao1 m ey1 k er0 z))
("lawmaking" nil (l ao1 m ey1 k ih0 ng))
("lawman" nil (l ao1 m ax n))
("lawmen" nil (l ao1 m eh1 n))
("lawn" nil (l ao1 n))
("lawner" nil (l ao1 n er0))
("lawniczak" nil (l aa1 v n ih0 ch ae0 k))
("lawnmower" nil (l ao1 n m ow1 er0))
("lawns" nil (l ao1 n z))
("lawnsdale" nil (l ao1 n z d ey1 l))
("lawplan" nil (l ao1 p l ae1 n))
("lawrance" nil (l ao1 r ax n s))
("lawrence" nil (l ao1 r ax n s))
("lawrenceburg" nil (l ao1 r ax n s b er0 g))
("lawrenceville" nil (l ao1 r ax n s v ih1 l))
("lawrencium" nil (l ao0 r eh1 n s iy0 ax m))
("lawrenz" nil (l aa1 v r ih0 n z))
("lawrie" nil (l ao1 r iy0))
("lawry" nil (l ao1 r iy0))
("laws" nil (l ao1 z))
("lawshe" nil (l ao1 sh))
("lawson" nil (l ao1 s ax n))
("lawsuit" nil (l ao1 s uw1 t))
("lawsuits" nil (l ao1 s uw1 t s))
("lawter" nil (l ao1 t er0))
("lawther" nil (l ao1 dh er0))
("lawton" nil (l ao1 t ax n))
("lawver" nil (l ao1 v er0))
("lawyer" nil (l ao1 y er0))
("lawyering" nil (l ao1 y er0 ih0 ng))
("lawyerly" nil (l ao1 y er0 l iy0))
("lawyers" nil (l ao1 y er0 z))
("lax" nil (l ae1 k s))
("laxalt" nil (l ae1 k s aa0 l t))
("laxative" nil (l ae1 k s ax t ih0 v))
("laxatives" nil (l ae1 k s ax t ih0 v z))
("laxey" nil (l ae1 k s iy0))
("laxity" nil (l ae1 k s ax t iy0))
("laxson" nil (l ae1 k s ax n))
("laxton" nil (l ae1 k s t ax n))
("lay" nil (l ey1))
("layborne" nil (l ey1 b ao1 r n))
("laycock" nil (l ey1 k aa1 k))
("layden" nil (l ey1 d ax n))
("laye" nil (l ey1))
("layer" nil (l ey1 er0))
("layered" nil (l ey1 er0 d))
("layering" nil (l ey1 er0 ih0 ng))
("layers" nil (l ey1 er0 z))
("layfield" nil (l ey1 f iy1 l d))
("laying" nil (l ey1 ih0 ng))
("layland" nil (l ey1 l ax n d))
("layman" nil (l ey1 m ax n))
("laymen" nil (l ey1 m ax n))
("laymon" nil (l ey1 m ax n))
("layne" nil (l ey1 n))
("layoff" nil (l ey1 ao1 f))
("layoffs" nil (l ey1 ao1 f s))
("layout" nil (l ey1 aw1 t))
("layouts" nil (l ey1 aw1 t s))
("layover" nil (l ey1 ow1 v er0))
("layovers" nil (l ey1 ow1 v er0 z))
("lays" nil (l ey1 z))
("layson" nil (l ey1 z ax n))
("layton" nil (l ey1 t ax n))
("lazar" nil (l ey1 z er0))
("lazard" nil (l ax z aa1 r d))
("lazare" nil (l aa0 z aa1 r ey0))
("lazaro" nil (l ax z aa1 r ow0))
("lazaroff" nil (l ae1 z er0 ao0 f))
("lazarski" nil (l ax z aa1 r s k iy0))
("lazarus" nil (l ae1 z er0 ax s))
("lazarz" nil (l aa1 z aa0 r z))
("lazcano" nil (l aa0 z k aa1 n ow0))
("lazear" nil (l ax z ih1 r))
("lazenby" nil (l ae1 z ih0 n b iy0))
("lazer" nil (l ey1 z er0))
("lazier" nil (l ey1 z iy0 er0))
("laziest" nil (l ey1 z iy0 ax s t))
("lazily" nil (l ae1 z ax l iy0))
("laziness" nil (l ey1 z iy0 n ax s))
("lazo" nil (l aa1 z ow0))
("lazor" nil (l aa0 z ao1 r))
("lazuli" nil (l ax z uw1 l iy0))
("lazulite" nil (l ae1 z ax l ay1 t))
("lazur" nil (l ae1 z er0))
("lazy" nil (l ey1 z iy0))
("lazzara" nil (l aa0 t s aa1 r ax))
("lazzari" nil (l aa0 t s aa1 r iy0))
("lazzarini" nil (l aa0 t s aa0 r iy1 n iy0))
("lazzaro" nil (l aa0 t s aa1 r ow0))
("lazzell" nil (l ae1 z ax l))
("lcs" nil (eh1 l s iy1 eh1 s))
("ldbrinkman" nil (l ax d b r ih1 ng k m ax n))
("le" nil (l ax))
("lea" nil (l iy1))
("leabo" nil (l iy1 b ow0))
("leach" nil (l iy1 ch))
("leached" nil (l iy1 ch t))
("leaches" nil (l iy1 ch ih0 z))
("leaching" nil (l iy1 ch ih0 ng))
("leachman" nil (l iy1 ch m ax n))
("leacock" nil (l iy1 k aa1 k))
("lead" n (l eh1 d))
("lead" v_p (l eh1 d))
("lead" v (l iy1 d))
("leadbetter" nil (l iy1 d b ih0 t er0))
("leaded" nil (l eh1 d ih0 d))
("leaden" nil (l eh1 d ax n))
("leader" nil (l iy1 d er0))
("leaderless" nil (l iy1 d er0 l ax s))
("leaders" nil (l iy1 d er0 z))
("leadership" nil (l iy1 d er0 sh ih1 p))
("leaderships" nil (l iy1 d er0 sh ih1 p s))
("leading" nil (l iy1 d ih0 ng))
("leadingham" nil (l iy1 d ih0 ng hh ae1 m))
("leadoff" nil (l iy1 d ao1 f))
("leads" nil (l iy1 d z))
("leadville" nil (l eh1 d v ih1 l))
("leadwork" nil (l eh1 d w er1 k))
("leaf" nil (l iy1 f))
("leafed" nil (l iy1 f t))
("leafhoppers" nil (l iy1 f hh aa1 p er0 z))
("leafing" nil (l iy1 f ih0 ng))
("leafless" nil (l iy1 f l ax s))
("leaflet" nil (l iy1 f l ax t))
("leaflets" nil (l iy1 f l ax t s))
("leaflike" nil (l iy1 f l ay1 k))
("leafs" nil (l iy1 f s))
("leafy" nil (l iy1 f iy0))
("league" nil (l iy1 g))
("leaguer" nil (l iy1 g er0))
("leaguers" nil (l iy1 g er0 z))
("leagues" nil (l iy1 g z))
("leah" nil (l iy1 ax))
("leahey" nil (l ey1 hh iy0))
("leahy" nil (l ey1 hh iy0))
("leak" nil (l iy1 k))
("leakage" nil (l iy1 k ax jh))
("leake" nil (l iy1 k))
("leaked" nil (l iy1 k t))
("leaker" nil (l iy1 k er0))
("leakers" nil (l iy1 k er0 z))
("leakey" nil (l iy1 k iy0))
("leaking" nil (l iy1 k ih0 ng))
("leaks" nil (l iy1 k s))
("leaky" nil (l iy1 k iy0))
("leal" nil (l iy1 l))
("leala" nil (l iy1 l ax))
("lealia" nil (l iy1 l iy0 ax))
("lealie" nil (l iy1 l iy0))
("leaman" nil (l iy1 m ax n))
("leamer" nil (l iy1 m er0))
("leaming" nil (l iy1 m ih0 ng))
("leamon" nil (l iy1 m ax n))
("leamy" nil (l iy1 m iy0))
("lean" nil (l iy1 n))
("leana" nil (l iy1 n ax))
("leander" nil (l iy0 ae1 n d er0))
("leandro" nil (l iy0 aa1 n d r ow0))
("leaned" nil (l iy1 n d))
("leaner" nil (l iy1 n er0))
("leaners" nil (l iy1 n er0 z))
("leanest" nil (l iy1 n ax s t))
("leaning" nil (l iy1 n ih0 ng))
("leanings" nil (l iy1 n ih0 ng z))
("leann" nil (l iy0 ae1 n))
("leans" nil (l iy1 n z))
("leap" nil (l iy1 p))
("leaped" nil (l eh1 p t))
("leapfrog" nil (l iy1 p f r ao1 g))
("leapfrogged" nil (l iy1 p f r ao1 g d))
("leapfrogging" nil (l iy1 p f r ao1 g ih0 ng))
("leaphart" nil (l iy1 p hh aa1 r t))
("leaping" nil (l iy1 p ih0 ng))
("leapley" nil (l iy1 p l iy0))
("leaps" nil (l iy1 p s))
("leapt" nil (l eh1 p t))
("lear" nil (l ih1 r))
("leard" nil (l er1 d))
("learjet" nil (l ih1 r jh eh1 t))
("learn" nil (l er1 n))
("learned" nil (l er1 n d))
("learner" nil (l er1 n er0))
("learners" nil (l er1 n er0 z))
("learning" nil (l er1 n ih0 ng))
("learns" nil (l er1 n z))
("learonal" nil (l iy1 r ax n ax l))
("leary" nil (l ih1 r iy0))
("leas" nil (l iy1 z))
("leasable" nil (l iy1 s ax b ax l))
("leasco" nil (l iy1 s k ow0))
("lease" nil (l iy1 s))
("leaseback" nil (l iy1 s b ae1 k))
("leasebacks" nil (l iy1 s b ae1 k s))
("leased" nil (l iy1 s t))
("leasehold" nil (l iy1 s hh ow1 l d))
("leaser" nil (l iy1 s er0))
("leases" nil (l iy1 s ih0 z))
("leaseway" nil (l iy1 s w ey1))
("leash" nil (l iy1 sh))
("leashes" nil (l iy1 sh ih0 z))
("leasing" nil (l iy1 s ih0 ng))
("leask" nil (l iy1 s k))
("leason" nil (l iy1 s ax n))
("least" nil (l iy1 s t))
("leasure" nil (l eh1 zh er0))
("leath" nil (l iy1 th))
("leatham" nil (l iy1 th ax m))
("leathem" nil (l eh1 th ih0 m))
("leather" nil (l eh1 dh er0))
("leatherback" nil (l eh1 dh er0 b ae1 k))
("leatherbacks" nil (l eh1 dh er0 b ae1 k s))
("leatherberry" nil (l eh1 dh er0 b eh1 r iy0))
("leatherbury" nil (l eh1 dh er0 b eh1 r iy0))
("leatherman" nil (l eh1 dh er0 m ax n))
("leathers" nil (l eh1 dh er0 z))
("leatherwood" nil (l eh1 dh er0 w uh1 d))
("leathery" nil (l eh1 dh er0 iy0))
("leaton" nil (l iy1 t ax n))
("leatrice" nil (l iy1 t r ih0 s))
("leave" nil (l iy1 v))
("leavell" nil (l iy1 v ax l))
("leavelle" nil (l ax v eh1 l))
("leavened" nil (l eh1 v ax n d))
("leavening" nil (l eh1 v ax n ih0 ng))
("leavens" nil (l eh1 v ax n z))
("leavenworth" nil (l eh1 v ax n w er1 th))
("leaver" nil (l iy1 v er0))
("leaverton" nil (l iy1 v er0 t ax n))
("leaves" nil (l iy1 v z))
("leavey" nil (l iy1 v iy0))
("leaving" nil (l iy1 v ih0 ng))
("leavings" nil (l iy1 v ih0 ng z))
("leavins" nil (l iy1 v ih0 n z))
("leavitt" nil (l eh1 v ih0 t))
("leavy" nil (l iy1 v iy0))
("leazer" nil (l iy1 z er0))
("lebanese" nil (l eh1 b ax n iy1 z))
("lebanon" nil (l eh1 b ax n ax n))
("lebar" nil (l ih0 b aa1 r))
("lebaron" nil (l ax b eh1 r ax n))
("lebarron" nil (l ax b eh1 r ax n))
("lebeau" nil (l ih0 b ow1))
("lebeck" nil (l iy1 b eh0 k))
("lebeda" nil (l ey0 b ey1 d ax))
("lebegue" nil (l eh1 b ih0 g))
("lebel" nil (l eh1 b ax l))
("leben" nil (l eh1 b ax n))
("lebenthal" nil (l eh1 b ax n th ao1 l))
("leber" nil (l iy1 b er0))
("lebert" nil (l eh1 b er0 t))
("leblanc" nil (l ax b l ae1 ng k))
("leble" nil (l eh1 b ax l))
("lebleu" nil (l eh1 b l uw0))
("leblond" nil (l ih0 b l aa1 n d))
("lebo" nil (l ey1 b ow0))
("leboeuf" nil (l ax b ah1 f))
("lebold" nil (l eh1 b ow0 l d))
("lebon" nil (l eh1 b ax n))
("lebouef" nil (l ax b ah1 f))
("lebovitz" nil (l eh1 b ax v ih0 t s))
("lebow" nil (l ax b ow1))
("lebowitz" nil (l eh1 b ax w ih0 t s))
("lebrecht" nil (l eh1 b r ih0 k t))
("lebreton" nil (l eh1 b r ih0 t aa0 n))
("lebron" nil (l eh1 b r ax n))
("lebrun" nil (l eh1 b r ax n))
("lebsack" nil (l eh1 b s ax k))
("lecates" nil (l ih0 k ey1 t s))
("leccese" nil (l eh0 ch ey1 z iy0))
("lech" nil (l eh1 k))
("lecher" nil (l eh1 ch er0))
("lecherous" nil (l eh1 ch er0 ax s))
("lechler" nil (l eh1 k l er0))
("lechman" nil (l eh1 k m ax n))
("lechmere" nil (l eh1 k m ih1 r))
("lechner" nil (l eh1 k n er0))
("lechtenberg" nil (l eh1 k t ax n b er0 g))
("lechters" nil (l eh1 k t er0 z))
("lechuga" nil (l eh1 ch uw0 g ax))
("lecithin" nil (l eh1 s ax th ax n))
("leck" nil (l eh1 k))
("leckey" nil (l eh1 k iy0))
("leckie" nil (l eh1 k iy0))
("leckrone" nil (l eh1 k r ax n))
("leclair" nil (l eh1 k l er0))
("leclaire" nil (l ih0 k l eh1 r))
("leclerc" nil (l ax k l er1 k))
("leclercq" nil (l eh1 k l er0 k))
("leclere" nil (l eh1 k l er0))
("lecli" nil (l eh1 k l iy0))
("lecocq" nil (l eh1 k ax k))
("lecompte" nil (l eh1 k ax m p t))
("lecomte" nil (l ih0 k aa1 m t))
("leconte" nil (l eh0 k ow1 n t iy0))
("lecount" nil (l ih0 k uw1 n t))
("lecrone" nil (l eh0 k r ow1 n iy0))
("lecroy" nil (l eh1 k r oy0))
("lectec" nil (l eh1 k t eh1 k))
("lectern" nil (l eh1 k t er0 n))
("lector" nil (l eh1 k t er0))
("lecture" nil (l eh1 k ch er0))
("lectured" nil (l eh1 k ch er0 d))
("lecturer" nil (l eh1 k ch er0 er0))
("lecturers" nil (l eh1 k ch er0 er0 z))
("lectures" nil (l eh1 k ch er0 z))
("lecturing" nil (l eh1 k ch er0 ih0 ng))
("lecuyer" nil (l eh1 k ay0 er0))
("lecy" nil (l iy1 s iy0))
("led" nil (l eh1 d))
("leda" nil (l iy1 d ax))
("leday" nil (l iy1 d ey0))
("ledbetter" nil (l eh1 d b er0 t er0))
("ledden" nil (l eh1 d ax n))
("leddy" nil (l eh1 d iy0))
("ledeen" nil (l ax d iy1 n))
("leder" nil (l eh1 d er0))
("lederer" nil (l eh1 d er0 er0))
("lederle" nil (l eh1 d er0 l iy0))
("lederman" nil (l iy1 d er0 m ax n))
("ledesma" nil (l eh0 d eh1 s m ax))
("ledet" nil (l eh1 d ih0 t))
("ledezma" nil (l ey0 d ey1 z m ax))
("ledford" nil (l eh1 d f er0 d))
("ledge" nil (l eh1 jh))
("ledger" nil (l eh1 jh er0))
("ledgers" nil (l eh1 jh er0 z))
("ledgerwood" nil (l eh1 jh er0 w uh1 d))
("ledges" nil (l eh1 jh ih0 z))
("ledin" nil (l eh1 d ih0 n))
("ledlow" nil (l eh1 d l ow1))
("ledo" nil (l ey1 d ow0))
("ledonne" nil (l eh1 d ax n))
("ledoux" nil (l ih0 d uw1))
("ledvina" nil (l eh0 d v iy1 n ax))
("ledwell" nil (l eh1 d w eh1 l))
("ledwith" nil (l eh1 d w ih1 th))
("ledyard" nil (l eh1 d y er0 d))
("lee" nil (l iy1))
("leeb" nil (l iy1 b))
("leece" nil (l iy1 s))
("leech" nil (l iy1 ch))
("leeches" nil (l iy1 ch ih0 z))
("leeco" nil (l iy1 k ow0))
("leed" nil (l iy1 d))
("leeder" nil (l iy1 d er0))
("leedham" nil (l iy1 d hh ax m))
("leedom" nil (l iy1 d ax m))
("leeds" nil (l iy1 d z))
("leedy" nil (l iy1 d iy0))
("leek" nil (l iy1 k))
("leeks" nil (l iy1 k s))
("leeman" nil (l iy1 m ax n))
("leeming" nil (l iy1 m ih0 ng))
("leemon" nil (l iy1 m ax n))
("leen" nil (l iy1 n))
("leep" nil (l iy1 p))
("leeper" nil (l iy1 p er0))
("leery" nil (l ih1 r iy0))
("lees" nil (l iy1 z))
("leesburg" nil (l iy1 z b er0 g))
("leese" nil (l iy1 s))
("leeser" nil (l iy1 s er0))
("leeson" nil (l iy1 s ax n))
("leet" nil (l iy1 t))
("leetch" nil (l iy1 ch))
("leete" nil (l iy1 t))
("leeth" nil (l iy1 th))
("leeuwen" nil (l uw1 ax n))
("leever" nil (l iy1 v er0))
("leeward" nil (l iy1 w er0 d))
("leeway" nil (l iy1 w ey1))
("lefauve" nil (l ax f aa1 v))
("lefave" nil (l ih0 f ey1 v))
("lefcourt" nil (l eh1 f k ao1 r t))
("lefeber" nil (l ax f ey1 b er0))
("lefebre" nil (l ax f ey1 b er0))
("lefebure" nil (l eh1 f ih0 b y uw0 r))
("lefebvre" nil (l ax f ey1 b er0))
("lefever" nil (l ax f ey1 v er0))
("lefevers" nil (l ax f ey1 v er0 z))
("lefevre" nil (l ax f ey1 v er0))
("leff" nil (l eh1 f))
("leffel" nil (l eh1 f ax l))
("leffert" nil (l eh1 f er0 t))
("lefferts" nil (l eh1 f er0 t s))
("leffew" nil (l eh1 f y uw0))
("leffingwell" nil (l eh1 f ih0 ng g w eh0 l))
("leffler" nil (l eh1 f l er0))
("lefkowitz" nil (l eh1 f k ax w ih0 t s))
("lefler" nil (l eh1 f l er0))
("leflore" nil (l eh1 f l er0))
("lefort" nil (l eh1 f er0 t))
("lefrak" nil (l eh1 f r ae0 k))
("lefrancois" nil (l eh1 f r ax n k w aa0))
("lefrere" nil (l ax f r eh1 r))
("left" nil (l eh1 f t))
("lefties" nil (l eh1 f t iy0 z))
("leftism" nil (l eh1 f t ih1 z ax m))
("leftist" nil (l eh1 f t ih0 s t))
("leftists" nil (l eh1 f t ih0 s t s))
("lefton" nil (l eh1 f t ax n))
("leftover" nil (l eh1 f t ow1 v er0))
("leftovers" nil (l eh1 f t ow1 v er0 z))
("leftridge" nil (l eh1 f t r ih1 jh))
("leftward" nil (l eh1 f t w er0 d))
("leftwich" nil (l eh1 f t w ih0 k))
("leftwing" nil (l eh1 f t w ih1 ng))
("lefty" nil (l eh1 f t iy0))
("leg" nil (l eh1 g))
("legacies" nil (l eh1 g ax s iy0 z))
("legacy" nil (l eh1 g ax s iy0))
("legal" nil (l iy1 g ax l))
("legalese" nil (l eh1 g ax l iy1 s))
("legalism" nil (l iy1 g ax l ih1 z ax m))
("legalistic" nil (l eh1 g ax l ih1 s t ih0 k))
("legalities" nil (l iy0 g ae1 l ih0 t iy0 z))
("legality" nil (l iy0 g ae1 l ax t iy0))
("legalization" nil (l iy1 g ax l ax z ey1 sh ax n))
("legalize" nil (l iy1 g ax l ay1 z))
("legalized" nil (l iy1 g ax l ay1 z d))
("legalizing" nil (l iy1 g ax l ay1 z ih0 ng))
("legally" nil (l iy1 g ax l iy0))
("legals" nil (l iy1 g ax l z))
("legan" nil (l eh1 g ax n))
("legare" nil (l eh0 g aa1 r iy0))
("legaspi" nil (l eh0 g aa1 s p iy0))
("legate" nil (l eh1 g ax t))
("legates" nil (l eh1 g ax t s))
("legault" nil (l ih0 g ow1))
("lege" nil (l iy1 jh))
("legend" nil (l eh1 jh ax n d))
("legendary" nil (l eh1 jh ax n d eh1 r iy0))
("legendre" nil (l eh1 g ih0 n d r ey0))
("legends" nil (l eh1 jh ax n d z))
("legent" nil (l eh1 jh ax n t))
("leger" nil (l eh1 jh er0))
("legerdemain" nil (l eh1 jh er0 d ax m ey1 n))
("legere" nil (l eh1 g er0))
("legette" nil (l ih0 zh eh1 t))
("legg" nil (l eh1 g))
("legge" nil (l eh1 g))
("legged" nil (l eh1 g ax d))
("leggett" nil (l eh1 g ih0 t))
("leggette" nil (l eh1 g eh1 t))
("leggings" nil (l eh1 g ih0 ng z))
("leggio" nil (l eh1 jh iy0 ow0))
("leggitt" nil (l eh1 g ih0 t))
("leggy" nil (l eh1 g iy0))
("leghorn" nil (l eh1 g hh ao0 r n))
("leghorns" nil (l eh1 g hh ao0 r n z))
("legibility" nil (l eh1 jh ax b ih1 l ax t iy0))
("legible" nil (l eh0 g ax b ax l))
("legion" nil (l iy1 jh ax n))
("legionaries" nil (l iy1 jh ax n eh1 r iy0 z))
("legions" nil (l iy1 jh ax n z))
("legislate" nil (l eh1 jh ih0 s l ey1 t))
("legislated" nil (l eh1 jh ax s l ey1 t ax d))
("legislates" nil (l eh1 jh ih0 s l ey1 t s))
("legislating" nil (l eh1 jh ih0 s l ey1 t ih0 ng))
("legislation" nil (l eh1 jh ax s l ey1 sh ax n))
("legislative" nil (l eh1 jh ax s l ey1 t ih0 v))
("legislatively" nil (l eh1 jh ih0 s l ey1 t ih0 v l iy0))
("legislator" nil (l eh1 jh ax s l ey1 t er0))
("legislators" nil (l eh1 jh ax s l ey1 t er0 z))
("legislature" nil (l eh1 jh ax s l ey1 ch er0))
("legislatures" nil (l eh1 jh ih0 s l ey1 ch er0 z))
("legit" nil (l eh1 jh ih0 t))
("legitimacy" nil (l ax jh ih1 t ax m ax s iy0))
("legitimate" nil (l ax jh ih1 t ax m ax t))
("legitimately" nil (l ax jh ih1 t ax m ax t l iy0))
("legitimize" nil (l ax jh ih1 t ax m ay1 z))
("legitimized" nil (l ih0 jh ih1 t ax m ay1 z d))
("legitimizes" nil (l ih0 jh ih1 t ax m ay1 z ih0 z))
("legitimizing" nil (l ih0 jh ih1 t ax m ay1 z ih0 ng))
("legler" nil (l eh1 g l er0))
("legless" nil (l eh1 g l ax s))
("legner" nil (l eh1 g n er0))
("lego" nil (l eh1 g ow0))
("legore" nil (l eh1 g ao1 r))
("legrand" nil (l eh1 g r ae0 n d))
("legrande" nil (l eh1 g r ae0 n d))
("legree" nil (l ih0 g r iy1))
("legroom" nil (l ax g r uw1 m))
("legros" nil (l eh1 g r ow0 z))
("legs" nil (l eh1 g z))
("legume" nil (l eh1 g y uw1 m))
("legumes" nil (l eh1 g y uw1 m z))
("legwork" nil (l eh1 g w er1 k))
("leh" nil (l eh1))
("lehan" nil (l iy1 hh ae0 n))
("lehane" nil (l ax hh ey1 n))
("lehder" nil (l eh1 d er0))
("lehenbauer" nil (l ey1 ax n b aw0 er0))
("lehew" nil (l ey1 hh y uw0))
("lehigh" nil (l iy1 hh ay1))
("lehl" nil (l eh1 l))
("lehman" nil (l iy1 m ax n))
("lehmann" nil (l ey1 m ax n))
("lehmer" nil (l eh1 m er0))
("lehmkuhl" nil (l eh1 m k ax l))
("lehn" nil (l eh1 n))
("lehne" nil (l eh1 n))
("lehnen" nil (l eh1 n ax n))
("lehner" nil (l eh1 n er0))
("lehnert" nil (l eh1 n er0 t))
("lehnhoff" nil (l eh1 n hh ao0 f))
("lehr" nil (l eh1 r))
("lehrer" nil (l eh1 r er0))
("lehrke" nil (l eh1 r k))
("lehrman" nil (l eh1 r m ax n))
("lehrmann" nil (l eh1 r m ax n))
("lehtinen" nil (l eh1 t ih0 n ax n))
("lehtonen" nil (l ih0 t aa1 n ax n))
("lei" nil (l ey1))
("leib" nil (l iy1 b))
("leibel" nil (l ay1 b ax l))
("leibensperger" nil (l ay1 b ih0 n s p er0 g er0))
("leiber" nil (l iy1 b er0))
("leibert" nil (l ay1 b er0 t))
("leibfried" nil (l ay1 b f er0 iy0 d))
("leibler" nil (l iy1 b l er0))
("leibman" nil (l iy1 b m ax n))
("leibniz" nil (l iy1 b n ih0 z))
("leibold" nil (l ay1 b ow1 l d))
("leibovit" nil (l iy1 b ax v ih0 t))
("leibovitz" nil (l iy1 b ax v ih0 t s))
("leibowitz" nil (l iy1 b ow0 ih0 t s))
("leibrand" nil (l ay1 b r ae1 n d))
("leibrock" nil (l ay1 b r ax k))
("leiby" nil (l iy1 b iy0))
("leicester" nil (l eh1 s t er0))
("leichliter" nil (l ay1 k l iy0 t er0))
("leichner" nil (l ay1 k n er0))
("leicht" nil (l ay1 k t))
("leichter" nil (l ay1 k t er0))
("leichtman" nil (l ay1 k t m ax n))
("leichty" nil (l ay1 k t iy0))
("leick" nil (l iy1 k))
("leider" nil (l ay1 d er0))
("leiderman" nil (l ay1 d er0 m ax n))
("leidig" nil (l ay1 d ih0 g))
("leidner" nil (l ay1 d n er0))
("leidy" nil (l iy1 d iy0))
("leier" nil (l ay1 er0))
("leif" nil (l iy1 f))
("leifer" nil (l ay1 f er0))
("leifeste" nil (l ay1 f ih0 s t))
("leifheit" nil (l ay1 f hh ay0 t))
("leigh" nil (l iy1))
("leight" nil (l ey1 t))
("leighton" nil (l ey1 t ax n))
("leighty" nil (l ey1 t iy0))
("leija" nil (l iy1 y ax))
("leikam" nil (l ay1 k ax m))
("leiken" nil (l ay1 k ax n))
("leiker" nil (l ay1 k er0))
("leila" nil (l iy1 l ax))
("leilani" nil (l ax l aa1 n iy0))
("leilia" nil (l iy1 l iy0 ax))
("leiman" nil (l ay1 m ax n))
("leimbach" nil (l ay1 m b aa1 k))
("leimer" nil (l ay1 m er0))
("lein" nil (l iy1 n))
("leinart" nil (l ay1 n aa0 r t))
("leinbach" nil (l ay1 n b aa1 k))
("leinberger" nil (l ay1 n b er0 g er0))
("leindecker" nil (l ay1 n d ih0 k er0))
("leinen" nil (l ay1 n ax n))
("leiner" nil (l ay1 n er0))
("leingang" nil (l ay1 ng g ax ng))
("leininger" nil (l ay1 n ih0 ng er0))
("leino" nil (l ey0 iy1 n ow0))
("leinonen" nil (l ay1 n ax n ax n))
("leins" nil (l iy1 n z))
("leinsdorf" nil (l ay1 n z d ao1 r f))
("leinweber" nil (l ay1 n w ih0 b er0))
("leiper" nil (l iy1 p er0))
("leiphart" nil (l ay1 p hh aa0 r t))
("leipold" nil (l ay1 p ow0 l d))
("leipzig" nil (l ay1 p s ih0 g))
("leis" nil (l ey1 z))
("leischner" nil (l ay1 sh n er0))
("leise" nil (l iy1 s))
("leisenring" nil (l ay1 s ih0 n r ih0 ng))
("leiser" nil (l ay1 s er0))
("leisey" nil (l iy1 s iy0))
("leishman" nil (l iy1 sh m ax n))
("leising" nil (l ay1 s ih0 ng))
("leisinger" nil (l ay1 s ih0 n jh er0))
("leisner" nil (l ay1 s n er0))
("leiss" nil (l ay1 s))
("leist" nil (l iy1 ih0 s t))
("leister" nil (l iy1 s t er0))
("leistikow" nil (l ay0 s t ih1 k ow0))
("leistner" nil (l ay1 s t n er0))
("leisure" nil (l eh1 zh er0))
("leisurely" nil (l iy1 z er0 l iy0))
("leisy" nil (l iy1 s iy0))
("leitch" nil (l iy1 ch))
("leite" nil (l iy1 t))
("leiter" nil (l ay1 t er0))
("leiterman" nil (l ay1 t er0 m ax n))
("leith" nil (l iy1 th))
("leitha" nil (l iy1 dh ax))
("leithia" nil (l iy1 dh iy0 ax))
("leitman" nil (l ay1 t m ax n))
("leitmotif" nil (l ay1 t m ow0 t iy1 f))
("leitner" nil (l ay1 t n er0))
("leitz" nil (l iy1 t s))
("leitzel" nil (l ay1 t z ax l))
("leitzke" nil (l ay1 t z k))
("leiva" nil (l iy1 v ax))
("leja" nil (l ey1 y ax))
("lejeune" nil (l eh1 y oy0 n))
("lek" nil (l eh1 k))
("lekachman" nil (l eh1 k aa1 k m ax n))
("lekas" nil (l iy1 k ax z))
("lekberg" nil (l eh1 k b er0 g))
("lela" nil (l iy1 l ax))
("lelah" nil (l eh1 l ax))
("leland" nil (l iy1 l ax n d))
("leleux" nil (l ih0 l ow1))
("lelia" nil (l iy1 l y ax))
("lellouche" nil (l eh0 l uw1 sh))
("lem" nil (l eh1 m))
("lema" nil (l iy1 m ax))
("lemaire" nil (l ax m eh1 r))
("lemaitre" nil (l ih0 m ey1 t er0))
("leman" nil (l iy1 m ax n))
("lemans" nil (l eh1 m ax n z))
("lemanski" nil (l ih0 m ae1 n s k iy0))
("lemar" nil (l ih0 m aa1 r))
("lemarr" nil (l eh1 m er0))
("lemaster" nil (l iy1 m ae0 s t er0))
("lemasters" nil (l ax m ae1 s t er0 z))
("lemay" nil (l eh1 m ey0))
("lembcke" nil (l eh1 m b k))
("lemberg" nil (l eh1 m b er0 g))
("lemberger" nil (l eh1 m b er0 g er0))
("lembke" nil (l eh1 m b k))
("lembo" nil (l eh1 m b ow0))
("lemburg" nil (l eh1 m b er0 g))
("lemcke" nil (l eh1 m k))
("lemelin" nil (l eh1 m ih0 l ih0 n))
("lemelle" nil (l ih0 m eh1 l))
("lemen" nil (l iy1 m eh0 n))
("lemercier" nil (l ax m er1 s iy0 er0))
("lemere" nil (l eh1 m er0))
("lemery" nil (l eh1 m er0 iy0))
("lemieux" nil (l ax m y uw1))
("leming" nil (l iy1 m ih0 ng))
("lemire" nil (l ey0 m ih1 r ey0))
("lemke" nil (l eh1 m k iy0))
("lemler" nil (l eh1 m l er0))
("lemley" nil (l eh1 m l iy0))
("lemm" nil (l eh1 m))
("lemma" nil (l eh1 m ax))
("lemme" nil (l eh1 m ax))
("lemmen" nil (l eh1 m eh0 n))
("lemmer" nil (l eh1 m er0))
("lemmerman" nil (l eh1 m er0 m ax n))
("lemmie" nil (l eh1 m iy0))
("lemming" nil (l eh1 m ih0 ng))
("lemmings" nil (l eh1 m ih0 ng z))
("lemmo" nil (l eh1 m ow0))
("lemmon" nil (l eh1 m ax n))
("lemmond" nil (l eh1 m ax n d))
("lemmons" nil (l eh1 m ax n z))
("lemoine" nil (l ih0 m oy1 n))
("lemon" nil (l eh1 m ax n))
("lemonade" nil (l eh1 m ax n ey1 d))
("lemond" nil (l eh1 m ax n d))
("lemonds" nil (l eh1 m ax n d z))
("lemons" nil (l eh1 m ax n z))
("lemont" nil (l ax m aa1 n t))
("lemos" nil (l iy1 m ow0 z))
("lemp" nil (l eh1 m p))
("lempert" nil (l eh1 m p er0 t))
("lempke" nil (l eh1 m p k))
("lemuela" nil (l eh0 m uw1 l ax))
("lemurs" nil (l iy1 m er0 z))
("lemus" nil (l iy1 m ax s))
("len" nil (l eh1 n))
("lena" nil (l iy1 n ax))
("lenahan" nil (l eh1 n ax hh ae0 n))
("lenard" nil (l eh1 n er0 d))
("lenart" nil (l eh1 n er0 t))
("lenarz" nil (l ey1 n aa0 r z))
("lencioni" nil (l eh0 n ch ow1 n iy0))
("lend" nil (l eh1 n d))
("lender" nil (l eh1 n d er0))
("lenderman" nil (l eh1 n d er0 m ax n))
("lenders" nil (l eh1 n d er0 z))
("lending" nil (l eh1 n d ih0 ng))
("lendings" nil (l eh1 n d ih0 ng z))
("lendl" nil (l eh1 n d ax l))
("lends" nil (l eh1 n d z))
("lene" nil (l iy1 n))
("lenehan" nil (l eh1 n ih0 hh ae0 n))
("leneta" nil (l eh0 n eh1 t ax))
("lenexa" nil (l eh0 n eh1 k s ax))
("leng" nil (l eh1 ng))
("lengacher" nil (l eh1 ng g ax k er0))
("lengel" nil (l eh1 ng g ax l))
("lenger" nil (l eh1 ng er0))
("lengle" nil (l ih1 ng ax l))
("length" nil (l eh1 ng k th))
("lengthen" nil (l eh1 ng th ax n))
("lengthened" nil (l eh1 ng th ax n d))
("lengthening" nil (l eh1 ng th ax n ih0 ng))
("lengthens" nil (l eh1 ng th ax n z))
("lengths" nil (l eh1 ng k th s))
("lengthways" nil (l eh1 ng th w ey1 z))
("lengthwise" nil (l eh1 ng th w ay1 z))
("lengthy" nil (l eh1 ng th iy0))
("lengyel" nil (l eh1 ng y eh1 l))
("lenhard" nil (l eh1 n er0 d))
("lenhardt" nil (l eh1 n hh aa1 r t))
("lenhart" nil (l eh1 n hh aa1 r t))
("lenhoff" nil (l eh1 n hh ao1 f))
("leniency" nil (l iy1 n y ax n s iy0))
("lenient" nil (l iy1 n iy0 ax n t))
("leniently" nil (l iy1 n y ax n t l iy0))
("lenig" nil (l eh1 n ih0 g))
("lenihan" nil (l eh1 n ih0 hh ae0 n))
("lenin" nil (l eh1 n ax n))
("leningrad" nil (l eh1 n ax n g r ae1 d))
("lenington" nil (l eh1 n ih0 ng t ax n))
("leninism" nil (l eh1 n ih0 n ih1 z ax m))
("leninist" nil (l eh1 n ih0 n ih0 s t))
("lenis" nil (l iy1 n ax s))
("lenita" nil (l eh0 n iy1 t ax))
("lenius" nil (l iy1 n iy0 ih0 s))
("lenk" nil (l eh1 ng k))
("lenke" nil (l eh1 ng k))
("lenker" nil (l eh1 ng k er0))
("lennane" nil (l eh0 n ey1 n))
("lennar" nil (l eh1 n er0))
("lennard" nil (l eh1 n er0 d))
("lennartz" nil (l eh1 n aa0 r t s))
("lennie" nil (l eh1 n iy0))
("lennig" nil (l eh1 n ih0 g))
("lenning" nil (l eh1 n ih0 ng))
("lennon" nil (l eh1 n ax n))
("lennox" nil (l eh1 n ax k s))
("lenny" nil (l eh1 n iy0))
("leno" nil (l eh1 n ow0))
("lenon" nil (l eh1 n ax n))
("lenora" nil (l eh1 n er0 ax))
("lenore" nil (l ax n ao1 r))
("lenos" nil (l iy1 n ow0 z))
("lenox" nil (l eh1 n ax k s))
("lens" nil (l eh1 n z))
("lenscrafters" nil (l eh1 n z k r ae1 f t er0 z))
("lenses" nil (l eh1 n z ax z))
("lensing" nil (l eh1 n s ih0 ng))
("lent" nil (l eh1 n t))
("lentil" nil (l eh1 n t ax l))
("lentils" nil (l eh1 n t ax l z))
("lentinan" nil (l eh1 n t ih0 n ax n))
("lentine" nil (l eh0 n t iy1 n iy0))
("lentini" nil (l eh0 n t iy1 n iy0))
("lentner" nil (l eh1 n t n er0))
("lento" nil (l eh1 n t ow0))
("lenton" nil (l eh1 n t ax n))
("lents" nil (l eh1 n t s))
("lentsch" nil (l eh1 n ch))
("lentz" nil (l eh1 n t s))
("lentzsch" nil (l eh1 n t s))
("leny" nil (l eh1 n iy0))
("lenz" nil (l eh1 n z))
("lenzen" nil (l eh1 n z ax n))
("lenzi" nil (l eh1 n z iy0))
("lenzini" nil (l eh0 n z iy1 n iy0))
("lenzo" nil (l eh1 n z ow0))
("leo" nil (l iy1 ow0))
("leoda" nil (l eh1 d ax))
("leoine" nil (l ih0 oy1 n))
("leola" nil (l eh1 l ax))
("leoma" nil (l ih0 ow1 m ax))
("leominster" nil (l iy1 ow0 m ih1 n s t er0))
("leon" nil (l iy1 aa0 n))
("leona" nil (l iy1 ow0 n ax))
("leonara" nil (l eh0 n aa1 r ax))
("leonard" nil (l eh1 n er0 d))
("leonarda" nil (l eh0 n aa1 r d ax))
("leonardi" nil (l eh0 n aa1 r d iy0))
("leonardis" nil (l eh1 n aa0 r d ih0 s))
("leonardo" nil (l iy1 ax n aa1 r d ow0))
("leone" nil (l iy0 ow1 n))
("leonel" nil (l iy1 ow0 n ax l))
("leonelle" nil (l eh0 n eh1 l))
("leonelli" nil (l eh0 n eh1 l iy0))
("leonerd" nil (l iy1 ax n er0 d))
("leones" nil (l iy0 ow1 n z))
("leonetti" nil (l eh0 n eh1 t iy0))
("leong" nil (l eh1 ng))
("leonhard" nil (l iy1 ax n hh aa0 r t))
("leonhardt" nil (l eh1 n hh aa0 r t))
("leonhart" nil (l eh1 n hh aa0 r t))
("leoni" nil (l eh0 ow1 n iy0))
("leonid" nil (l iy1 ax n ih0 d))
("leonie" nil (l eh1 n iy0))
("leonora" nil (l iy1 ax n ao1 r ax))
("leonore" nil (l iy1 ax n ao0 r))
("leontine" nil (l eh0 n t iy1 n iy0))
("leontyne" nil (l eh1 n t ay0 n))
("leopard" nil (l eh1 p er0 d))
("leopards" nil (l eh1 p er0 d z))
("leopold" nil (l iy1 ax p ow1 l d))
("leopoldina" nil (l eh0 p ow0 l d iy1 n ax))
("leopoldine" nil (l eh0 p ow0 l d iy1 n iy0))
("leopoldo" nil (l iy1 ax p ow1 l d ow0))
("leora" nil (l eh1 r ax))
("leos" nil (l iy1 ow0 z))
("leota" nil (l eh1 t ax))
("leotard" nil (l iy1 ax t aa1 r d))
("lep" nil (l eh1 p))
("lepage" nil (l eh1 p ih0 jh))
("lepak" nil (l eh1 p ax k))
("lepard" nil (l eh1 p er0 d))
("lepe" nil (l iy1 p))
("leper" nil (l eh1 p er0))
("lepera" nil (l eh0 p eh1 r ax))
("lepere" nil (l ey0 p eh1 r ey0))
("lepers" nil (l eh1 p er0 z))
("lepine" nil (l eh0 p iy1 n iy0))
("lepisto" nil (l eh0 p iy1 s t ow0))
("lepkowski" nil (l ih0 p k ao1 f s k iy0))
("lepley" nil (l eh1 p l iy0))
("lepore" nil (l eh0 p ao1 r iy0))
("lepp" nil (l eh1 p))
("leppanen" nil (l eh1 p ax n ax n))
("leppard" nil (l eh1 p er0 d))
("leppek" nil (l eh1 p ih0 k))
("lepper" nil (l eh1 p er0))
("leppert" nil (l eh1 p er0 t))
("leppla" nil (l eh1 p l ax))
("leppo" nil (l eh1 p ow0))
("lepre" nil (l eh1 p er0))
("leprosy" nil (l eh1 p r ax s iy0))
("leptonic" nil (l eh0 p t aa1 n ih0 k))
("lequire" nil (l ey0 k w ih1 r ey0))
("lerach" nil (l eh1 r ax k))
("lerch" nil (l er1 k))
("lerche" nil (l er1 ch))
("lereah" nil (l eh1 r ey1 ax))
("lerette" nil (l er0 eh1 t))
("lerew" nil (l eh1 r uw0))
("lerma" nil (l er1 m ax))
("lerman" nil (l eh1 r m ax n))
("lerner" nil (l er1 n er0))
("leroux" nil (l er0 uw1))
("leroy" nil (l iy1 r oy1))
("lerro" nil (l eh1 r ow0))
("lersch" nil (l er1 sh))
("lerum" nil (l eh1 r ax m))
("les" nil (l eh1 s))
("lesa" nil (l iy1 s ax))
("lesabre" nil (l ax s ey1 b er0))
("lesabres" nil (l eh1 s ax b r ax s))
("lesage" nil (l eh1 s ih0 jh))
("lesane" nil (l ey0 s aa1 n ey0))
("lesbian" nil (l eh1 z b iy0 ax n))
("lesbians" nil (l eh1 z b iy0 ax n z))
("lescaze" nil (l ax s k ey1 z))
("lesch" nil (l eh1 sh))
("leschly" nil (l eh1 sh l iy0))
("leser" nil (l eh1 s er0))
("lesesne" nil (l eh1 s ih0 n))
("lesh" nil (l eh1 sh))
("lesharo" nil (l eh0 sh aa1 r ow0))
("lesher" nil (l eh1 sh er0))
("lesiak" nil (l eh1 s iy0 ae0 k))
("lesieur" nil (l eh1 s iy0 er0))
("lesinski" nil (l ih0 s ih1 n s k iy0))
("lesion" nil (l iy1 zh ax n))
("lesions" nil (l iy1 zh ax n z))
("leske" nil (l eh1 s k))
("lesko" nil (l eh1 s k ow0))
("lesley" nil (l eh1 s l iy0))
("leslie" nil (l eh1 s l iy0))
("lesmeister" nil (l eh1 s m ay1 s t er0))
("lesner" nil (l eh1 s n er0))
("lesniak" nil (l eh1 s n iy0 ae0 k))
("lesnick" nil (l eh1 s n ih0 k))
("lesniewski" nil (l eh0 s n eh1 f s k iy0))
("lesotho" nil (l eh0 s ow1 th ow0))
("lesperance" nil (l eh0 s p er0 aa1 n s))
("lespinasse" nil (l eh1 s p ih0 n ae0 s))
("less" nil (l eh1 s))
("lessard" nil (l eh1 s er0 d))
("lessee" nil (l eh0 s iy1))
("lessees" nil (l eh0 s iy1 z))
("lessen" nil (l eh1 s ax n))
("lessened" nil (l eh1 s ax n d))
("lessening" nil (l eh1 s ax n ih0 ng))
("lessens" nil (l eh1 s ax n z))
("lesser" nil (l eh1 s er0))
("lessig" nil (l eh1 s ih0 g))
("lessin" nil (l eh1 s ih0 n))
("lessing" nil (l eh1 s ih0 ng))
("lessley" nil (l eh1 s l iy0))
("lessman" nil (l eh1 s m ax n))
("lessner" nil (l eh1 s n er0))
("lesson" nil (l eh1 s ax n))
("lessons" nil (l eh1 s ax n z))
("lessor" nil (l eh1 s er0))
("lessors" nil (l eh1 s er0 z))
("lest" nil (l eh1 s t))
("lestat" nil (l eh1 s t ae1 t))
("lester" nil (l eh1 s t er0))
("lestrange" nil (l eh0 s t r ey1 n jh))
("lesuer" nil (l eh0 s uw1 er0))
("lesueur" nil (l eh0 s uw1 er0))
("lesure" nil (l eh0 sh uh1 r))
("leszczynski" nil (l eh0 sh ih1 n s k iy0))
("leszek" nil (l eh1 s eh0 k))
("let" nil (l eh1 t))
("leta" nil (l eh1 t ax))
("letarte" nil (l eh1 t aa0 r t))
("letcher" nil (l eh1 ch er0))
("letchworth" nil (l eh1 ch w er0 th))
("letdown" nil (l eh1 t d aw1 n))
("letellier" nil (l eh1 t ax l iy0 er0))
("letendre" nil (l ax t aa1 n d r ax))
("leth" nil (l eh1 th))
("letha" nil (l eh1 th ax))
("lethal" nil (l iy1 th ax l))
("lethality" nil (l iy0 th ae1 l ax t iy0))
("lethargic" nil (l ax th aa1 r jh ih0 k))
("lethargy" nil (l eh1 th er0 jh iy0))
("lethco" nil (l eh1 th k ow0))
("lethia" nil (l eh1 th iy0 ax))
("leticia" nil (l ax t ih1 sh ax))
("letitia" nil (l ih0 t ih1 sh iy0 ax))
("letizia" nil (l eh0 t iy1 z iy0 ax))
("letner" nil (l eh1 t n er0))
("leto" nil (l iy1 t ow0))
("letourneau" nil (l eh1 t uh0 r n ow0))
("lets" nil (l eh1 t s))
("letsch" nil (l eh1 ch))
("letsinger" nil (l eh1 t s ih0 n jh er0))
("letson" nil (l eh1 t s ax n))
("lett" nil (l eh1 t))
("lettau" nil (l ih0 t ow1))
("letter" nil (l eh1 t er0))
("lettered" nil (l eh1 t er0 d))
("letterhead" nil (l eh1 t er0 hh eh1 d))
("lettering" nil (l eh1 t er0 ih0 ng))
("letterman" nil (l eh1 t er0 m ax n))
("letterpress" nil (l eh1 t er0 p r eh1 s))
("letters" nil (l eh1 t er0 z))
("lettice" nil (l eh1 t ih0 s))
("lettie" nil (l eh1 t iy0))
("lettiere" nil (l eh1 t iy0 eh0 r))
("lettieri" nil (l eh0 t ih1 r iy0))
("letting" nil (l eh1 t ih0 ng))
("lettish" nil (l eh1 t ih0 sh))
("letts" nil (l eh1 t s))
("lettuce" nil (l eh1 t ax s))
("letty" nil (l eh1 t iy0))
("letup" nil (l eh1 t ah1 p))
("letz" nil (l eh1 t s))
("leu" nil (l uw1))
("leubert" nil (l uw1 b er0 t))
("leucadia" nil (l uw0 k ey1 d iy0 ax))
("leuck" nil (l uw1 k))
("leuenberger" nil (l uw1 ax n b er0 g er0))
("leuffer" nil (l uw1 f er0))
("leugers" nil (l oy1 g er0 z))
("leukemia" nil (l uw0 k iy1 m iy0 ax))
("leukocyte" nil (l uw1 k ax s ay1 t))
("leumi" nil (l uw1 m iy0))
("leung" nil (l uw1 ng))
("leupold" nil (l oy1 p ow0 l d))
("leuschner" nil (l oy1 sh n er0))
("leuthold" nil (l oy1 th ow0 l d))
("leutwiler" nil (l uw1 t w ay1 l er0))
("leuzzi" nil (l uw1 z iy0))
("lev" nil (l eh1 v))
("leva" nil (l eh1 v ax))
("leval" nil (l ax v aa1 l))
("levalley" nil (l eh1 v ax l iy0))
("levamisole" nil (l ax v ae1 m ih0 s ow1 l))
("levan" nil (l eh1 v ax n))
("levana" nil (l ih0 v ae1 n ax))
("levander" nil (l eh1 v ax n d er0))
("levandoski" nil (l ih0 v ax n d aw1 s k iy0))
("levandowski" nil (l ih0 v ax n d ao1 f s k iy0))
("levangie" nil (l eh1 v ax ng iy0))
("levant" nil (l ax v ae1 n t))
("levario" nil (l ey0 v aa1 r iy0 ow0))
("levasseur" nil (l eh1 v ax s er0))
("levay" nil (l eh1 v ey0))
("levchenko" nil (l eh1 v ch eh1 ng k ow0))
("leve" nil (l iy1 v))
("leveck" nil (l eh1 v ih0 k))
("levee" nil (l eh1 v iy0))
("levees" nil (l eh1 v iy0 z))
("leveille" nil (l eh1 v ay0 l))
("level" nil (l eh1 v ax l))
("leveled" nil (l eh1 v ax l d))
("levelers" nil (l eh1 v ax l er0 z))
("leveling" nil (l eh1 v ax l ih0 ng))
("levell" nil (l ey0 v ey1 l))
("levelled" nil (l eh1 v ax l d))
("levels" nil (l eh1 v ax l z))
("leven" nil (l iy1 v ax n))
("levenberg" nil (l iy1 v ax n b er0 g))
("levendusky" nil (l ih0 v ih0 n d ah1 s k iy0))
("levene" nil (l eh1 v iy0 n))
("levengood" nil (l eh1 v ih0 n g uh0 d))
("levenhagen" nil (l eh1 v ih0 n hh ax g ax n))
("levens" nil (l iy1 v ax n z))
("levenson" nil (l eh1 v ih0 n s ax n))
("levenstein" nil (l iy1 v ax n s t ay0 n))
("leventhal" nil (l eh1 v ax n th ao1 l))
("leveque" nil (l ax v iy1 k))
("lever" nil (l eh1 v er0))
("leverage" nil (l eh1 v er0 ih0 jh))
("leveraged" nil (l iy1 v er0 ih0 jh d))
("leveraging" nil (l eh1 v r ih0 jh ih0 ng))
("leverenz" nil (l eh1 v er0 ih0 n s))
("leverett" nil (l eh1 v er0 eh0 t))
("leverette" nil (l eh1 v er0 eh0 t))
("leverich" nil (l eh1 v er0 ih0 k))
("levering" nil (l eh1 v er0 ih0 ng))
("leverone" nil (l eh0 v er0 ow1 n iy0))
("levers" nil (l eh1 v er0 z))
("leverson" nil (l eh1 v er0 s ax n))
("levert" nil (l eh1 v er0 t))
("leverton" nil (l ih0 v er1 t ax n))
("levesque" nil (l ih0 v eh1 s k))
("levett" nil (l eh1 v ih0 t))
("levey" nil (l ih0 v ey1))
("levi" nil (l iy1 v ay0))
("leviathan" nil (l ax v ay1 ax th ax n))
("levick" nil (l eh1 v ih0 k))
("levie" nil (l iy1 v iy0))
("levied" nil (l eh1 v iy0 d))
("levien" nil (l eh1 v iy0 ax n))
("levies" nil (l eh1 v iy0 z))
("levin" nil (l eh1 v ih0 n))
("levina" nil (l eh0 v iy1 n ax))
("levine" nil (l ax v iy1 n))
("leviner" nil (l eh1 v ih0 n er0))
("levings" nil (l eh1 v ih0 ng z))
("levingston" nil (l ih1 v ih0 ng s t ax n))
("levins" nil (l eh1 v ih0 n z))
("levinsky" nil (l ih0 v ih1 n s k iy0))
("levinson" nil (l eh1 v ih0 n s ax n))
("levis" nil (l eh1 v ih0 s))
("levison" nil (l eh1 v ih0 s ax n))
("levit" nil (l eh1 v ih0 t))
("levitan" nil (l eh1 v ih0 t ax n))
("levitated" nil (l eh1 v ih0 t ey1 t ih0 d))
("levitating" nil (l eh1 v ax t ey1 t ih0 ng))
("levitation" nil (l eh1 v ih0 t ey1 sh ax n))
("levitch" nil (l eh1 v ih0 ch))
("leviton" nil (l ih0 v ih1 t ax n))
("levitsky" nil (l ih0 v ih1 t s k iy0))
("levitt" nil (l eh1 v ih0 t))
("levitte" nil (l eh1 v ih0 t))
("levittown" nil (l eh1 v ih0 t aw1 n))
("levity" nil (l eh1 v ih0 t iy0))
("levitz" nil (l eh1 v ih0 t s))
("levy" nil (l eh1 v iy0))
("levying" nil (l eh1 v iy0 ih0 ng))
("lew" nil (l uw1))
("lewallen" nil (l uw0 ao1 l ax n))
("lewan" nil (l uw1 ax n))
("lewandoski" nil (l uw0 ax n d aw1 s k iy0))
("lewandowski" nil (l uw0 ax n d ao1 f s k iy0))
("lewanna" nil (l uw1 ih0 n ax))
("lewd" nil (l uw1 d))
("lewellen" nil (l uw1 eh1 l ax n))
("lewelling" nil (l uw1 eh1 l ih0 ng))
("lewellyn" nil (l uw1 eh1 l ih0 n))
("lewensky" nil (l uw1 eh1 n s k iy0))
("lewers" nil (l uw1 er0 z))
("lewey" nil (l uw1 iy0))
("lewicki" nil (l uw0 ih1 t s k iy0))
("lewin" nil (l uw1 ih0 n))
("lewing" nil (l uw1 ih0 ng))
("lewins" nil (l uw1 ih0 n z))
("lewinski" nil (l uw0 ih1 n s k iy0))
("lewinsky" nil (l uw0 ih1 n s k iy0))
("lewinsohn" nil (l uw1 ih0 n s ax n))
("lewinton" nil (l uw1 ih0 n t ax n))
("lewis" nil (l uw1 ih0 s))
("lewisburg" nil (l uw1 ih0 s b er0 g))
("lewiston" nil (l uw1 ax s t ax n))
("lewkowicz" nil (l uw1 k ax v ih0 ch))
("lewman" nil (l uw1 m ax n))
("lewter" nil (l uw1 t er0))
("lewton" nil (l uw1 t ax n))
("lewy" nil (l uw1 iy0))
("lex" nil (l eh1 k s))
("lexical" nil (l eh1 k s ih0 k ax l))
("lexicographer" nil (l eh1 k s ih0 k aa1 g r ax f er0))
("lexicon" nil (l eh1 k s ih0 k aa1 n))
("lexie" nil (l eh1 k s iy0))
("lexine" nil (l eh1 k s ay0 n))
("lexington" nil (l eh1 k s ih0 ng t ax n))
("lexis" nil (l eh1 k s ih0 s))
("lexmark" nil (l eh1 k s m aa1 r k))
("lexus" nil (l eh1 k s ax s))
("ley" nil (l ey1))
("leya" nil (l ey1 ax))
("leyba" nil (l ey1 b ax))
("leyda" nil (l ey1 d ax))
("leyden" nil (l ay1 d ax n))
("leyendecker" nil (l ay1 n d ih0 k er0))
("leyh" nil (l ey1))
("leyland" nil (l ey1 l ax n d))
("leyrer" nil (l ey1 r er0))
("leys" nil (l ey1 z))
("leysen" nil (l ey1 s ax n))
("leyva" nil (l ey1 v ax))
("lezotte" nil (l ih0 z ao1 t))
("lhasa" nil (l aa1 s ax))
("lheureux" nil (l er0 ow1))
("lhommedieu" nil (l ow1 m eh0 d y uw1))
("li" nil (l iy1))
("lia" nil (l iy1 ax))
("liabilities" nil (l ay1 ax b ih1 l ax t iy0 z))
("liability" nil (l ay1 ax b ih1 l ih0 t iy0))
("liable" nil (l ay1 ax b ax l))
("liacos" nil (l ay0 aa1 k ow0 s))
("liaison" nil (l iy0 ey1 z aa1 n))
("liaisons" nil (l iy0 ey1 s aa1 n z))
("liam" nil (l iy1 ax m))
("lian" nil (l y aa1 n))
("liana" nil (l iy0 aa1 n ax))
("liane" nil (l iy0 ae1 n))
("liang" nil (l y ae1 ng))
("lianna" nil (l iy0 aa1 n ax))
("lianne" nil (l iy0 ae1 n))
("liao" nil (l iy0 aw1))
("liar" nil (l ay1 er0))
("liars" nil (l ay1 er0 z))
("liasson" nil (l ey1 z aa0 n))
("lib" nil (l ih1 b))
("libbey" nil (l ih1 b iy0))
("libby" nil (l ih1 b iy0))
("libel" nil (l ay1 b ax l))
("libeled" nil (l ay1 b ax l d))
("libelous" nil (l ay1 b ax l ax s))
("liberace" nil (l ih1 b er0 ax s))
("liberal" nil (l ih1 b er0 ax l))
("liberalism" nil (l ih1 b er0 ax l ih1 z ax m))
("liberality" nil (l ih1 b er0 ae1 l ax t iy0))
("liberalization" nil (l ih1 b r ax l ih0 z ey1 sh ax n))
("liberalizations" nil (l ih0 b er0 ax l ih0 z ey1 sh ax n z))
("liberalize" nil (l ih1 b er0 ax l ay1 z))
("liberalized" nil (l ih1 b er0 ax l ay1 z d))
("liberalizing" nil (l ih1 b er0 ax l ay1 z ih0 ng))
("liberally" nil (l ih1 b er0 ax l iy0))
("liberals" nil (l ih1 b er0 ax l z))
("liberate" nil (l ih1 b er0 ey1 t))
("liberated" nil (l ih1 b er0 ey1 t ih0 d))
("liberati" nil (l iy0 b er0 aa1 t iy0))
("liberating" nil (l ih1 b er0 ey1 t ih0 ng))
("liberation" nil (l ih1 b er0 ey1 sh ax n))
("liberato" nil (l iy0 b er0 aa1 t ow0))
("liberatore" nil (l iy0 b er0 aa0 t ao1 r iy0))
("liberia" nil (l ay0 b ih1 r iy0 ax))
("liberian" nil (l ay0 b ih1 r iy0 ax n))
("liberman" nil (l ih1 b er0 m ax n))
("libert" nil (l ih1 b er0 t))
("libertarian" nil (l ih1 b er0 t eh1 r iy0 ax n))
("libertarians" nil (l ih1 b er0 t eh1 r iy0 ax n z))
("liberti" nil (l iy0 b eh1 r t iy0))
("liberties" nil (l ih1 b er0 t iy0 z))
("libertine" nil (l ih1 b er0 t iy1 n))
("libertines" nil (l ih1 b er0 t iy1 n z))
("liberto" nil (l iy0 b eh1 r t ow0))
("liberty" nil (l ih1 b er0 t iy0))
("libido" nil (l ax b iy1 d ow0))
("libman" nil (l ih1 b m ax n))
("libor" nil (l iy1 b er0))
("libra" nil (l iy1 b r ax))
("librarian" nil (l ay0 b r eh1 r iy0 ax n))
("librarians" nil (l ay0 b r eh1 r iy0 ax n z))
("libraries" nil (l ay1 b r eh1 r iy0 z))
("library" nil (l ay1 b r eh1 r iy0))
("libration" nil (l ay0 b r ey1 sh ax n))
("librettist" nil (l ax b r eh1 t ax s t))
("libretto" nil (l ax b r eh1 t ow0))
("librizzi" nil (l iy0 b r iy1 t s iy0))
("liby" nil (l ay1 b iy0))
("libya" nil (l ih1 b iy0 ax))
("libyan" nil (l ih1 b iy0 ax n))
("libyans" nil (l ih1 b iy0 ax n z))
("licari" nil (l iy0 k aa1 r iy0))
("licata" nil (l iy0 k aa1 t ax))
("licausi" nil (l iy0 k ao1 s iy0))
("licavoli" nil (l iy0 k aa0 v ow1 l iy0))
("licciardi" nil (l iy0 ch aa1 r d iy0))
("lice" nil (l ay1 s))
("licea" nil (l ih1 s iy0 ax))
("licence" nil (l ay1 s ax n s))
("licences" nil (l ay1 s ax n s ih0 z))
("license" nil (l ay1 s ax n s))
("licensed" nil (l ay1 s ax n s t))
("licensee" nil (l ay1 s ax n s iy1))
("licensees" nil (l ay1 s ax n s iy1 z))
("licenser" nil (l ay1 s ax n s er0))
("licenses" nil (l ay1 s ax n s ih0 z))
("licensing" nil (l ay1 s ax n s ih0 ng))
("licensor" nil (l ay1 s ax n s er0))
("licensure" nil (l ay1 s ax n ch er0))
("licentious" nil (l ay0 s eh1 n ch ax s))
("lich" nil (l ih1 ch))
("lichen" nil (l ay1 k ax n))
("lichens" nil (l ay1 k ax n z))
("lichlyter" nil (l ih1 k l ay0 t er0))
("licht" nil (l ih1 k t))
("lichtblau" nil (l ih1 ch t b l aw1))
("lichte" nil (l ih1 ch t))
("lichten" nil (l ih1 k t ax n))
("lichtenberg" nil (l ih1 k t ax n b er0 g))
("lichtenberger" nil (l ih1 k t ax n b er0 g er0))
("lichtenfels" nil (l ih1 k t ih0 n f ax l z))
("lichtenstein" nil (l ih1 k t ax n s t iy1 n))
("lichtenwalner" nil (l ih1 k t ih0 n w ax l n er0))
("lichtenwalter" nil (l ih1 k t ih0 n w ax l t er0))
("lichter" nil (l ih1 k t er0))
("lichterman" nil (l ih1 k t er0 m ax n))
("lichtman" nil (l ih1 k t m ax n))
("lichty" nil (l ih1 ch t iy0))
("licio" nil (l ih1 s iy0 ow0))
("licitra" nil (l iy0 ch iy1 t r ax))
("lick" nil (l ih1 k))
("licked" nil (l ih1 k t))
("licker" nil (l ih1 k er0))
("licking" nil (l ih1 k ih0 ng))
("licklider" nil (l ih1 k l ay0 d er0))
("licks" nil (l ih1 k s))
("lickteig" nil (l ih1 k t ay1 g))
("lico" nil (l iy1 k ow0))
("licon" nil (l ih1 k ax n))
("licorice" nil (l ih1 k er0 ih0 sh))
("lid" nil (l ih1 d))
("liddell" nil (l ih1 d ax l))
("liddick" nil (l ih1 d ih0 k))
("liddicoat" nil (l ih1 d ih0 k ow1 t))
("liddle" nil (l ih1 d ax l))
("liddy" nil (l ih1 d iy0))
("lide" nil (l ay1 d))
("liden" nil (l ay1 d ax n))
("lidgerwood" nil (l ih1 jh er0 w uh1 d))
("lido" nil (l iy1 d ow0))
("lids" nil (l ih1 d z))
("lie" nil (l ay1))
("lieb" nil (l iy1 b))
("liebe" nil (l iy1 b))
("liebel" nil (l iy1 b ax l))
("liebeler" nil (l iy1 b ax l er0))
("liebelt" nil (l iy1 b ih0 l t))
("liebenow" nil (l iy1 b ih0 n ow0))
("lieber" nil (l iy1 b er0))
("lieberman" nil (l iy1 b er0 m ax n))
("liebermann" nil (l iy1 b er0 m ax n))
("liebert" nil (l iy1 b er0 t))
("lieberthal" nil (l iy1 b er0 th ao1 l))
("liebig" nil (l iy1 b ih0 g))
("liebl" nil (l iy1 b ax l))
("liebler" nil (l iy1 b l er0))
("liebling" nil (l iy1 b ax l ih0 ng))
("liebman" nil (l iy1 b m ax n))
("liebmann" nil (l iy1 b m ax n))
("liebowitz" nil (l iy1 b ax w ih0 t s))
("liechtenstein" nil (l ih1 k t ax n s t ay1 n))
("liechty" nil (l iy1 ch t iy0))
("lied" nil (l ay1 d))
("liedel" nil (l iy1 d ax l))
("lieder" nil (l iy1 d er0))
("liederman" nil (l iy1 d er0 m ax n))
("liedtke" nil (l iy1 t k iy0))
("lief" nil (l iy1 f))
("liefer" nil (l iy1 f er0))
("liege" nil (l iy1 jh))
("liem" nil (l iy1 m))
("liemandt" nil (l iy1 m ae0 n t))
("lien" nil (l iy1 n))
("lienau" nil (l ax n ow1))
("lienemann" nil (l iy1 n ax m ax n))
("liener" nil (l iy1 n er0))
("lienhard" nil (l iy1 n hh aa1 r d))
("lienhart" nil (l iy1 n hh aa1 r t))
("liens" nil (l iy1 n z))
("lierman" nil (l ih1 r m ax n))
("liermann" nil (l ih1 r m ax n))
("lies" nil (l ay1 z))
("liesch" nil (l iy1 sh))
("liese" nil (l iy1 z))
("lieser" nil (l iy1 s er0))
("lieske" nil (l iy1 s k iy0))
("lietz" nil (l iy1 t s))
("lietzke" nil (l iy1 t z k iy0))
("lieu" nil (l uw1))
("lieurance" nil (l uw1 r ax n s))
("lieutenant" nil (l uw0 t eh1 n ax n t))
("lieutenants" nil (l uw0 t eh1 n ax n t s))
("life" nil (l ay1 f))
("lifeblood" nil (l ay1 f b l ah1 d))
("lifeboat" nil (l ay1 f b ow1 t))
("lifeboats" nil (l ay1 f b ow1 t s))
("lifeco" nil (l iy1 f k ow1))
("lifeguard" nil (l ay1 f g aa1 r d))
("lifeless" nil (l ay1 f l ax s))
("lifelike" nil (l ay1 f l ay1 k))
("lifeline" nil (l ay1 f l ay1 n))
("lifelines" nil (l ay1 f l ay1 n z))
("lifelong" nil (l ay1 f l ao1 ng))
("lifer" nil (l ay1 f er0))
("lifers" nil (l ay1 f er0 z))
("lifes" nil (l ay1 f s))
("lifesaver" nil (l ay1 f s ey1 v er0))
("lifesavers" nil (l ay1 f s ey1 v er0 z))
("lifesaving" nil (l ay1 f s ey1 v ih0 ng))
("lifespan" nil (l ay1 f s p ae1 n))
("lifestyle" nil (l ay1 f s t ay1 l))
("lifestyles" nil (l ay1 f s t ay1 l z))
("lifetime" nil (l ay1 f t ay1 m))
("lifetimes" nil (l ay1 f t ay1 m z))
("liff" nil (l ih1 f))
("liffe" nil (l ih1 f))
("lifland" nil (l ih1 f l ax n d))
("lifo" nil (l iy1 f ow0))
("liford" nil (l ih1 f er0 d))
("lifsey" nil (l ih1 f s iy0))
("lifshitz" nil (l ih1 f sh ih0 t s))
("lifson" nil (l ih1 f s aa1 n))
("lift" nil (l ih1 f t))
("lifted" nil (l ih1 f t ax d))
("lifter" nil (l ih1 f t er0))
("lifters" nil (l ih1 f t er0 z))
("liftin" nil (l ih1 f t ih0 n))
("lifting" nil (l ih1 f t ih0 ng))
("liftoff" nil (l ih1 f t ao1 f))
("lifton" nil (l ih1 f t ax n))
("lifts" nil (l ih1 f t s))
("ligachev" nil (l ih1 g ax ch eh0 v))
("ligament" nil (l ih1 g ax m ax n t))
("ligaments" nil (l ih1 g ax m ax n t s))
("ligand" nil (l ih1 g ax n d))
("ligands" nil (l ih1 g ax n d z))
("ligas" nil (l ay1 g ax z))
("ligation" nil (l ay0 g ey1 sh ax n))
("ligget" nil (l ih1 g ih0 t))
("liggett" nil (l ih1 g ih0 t))
("liggins" nil (l ih1 g ih0 n z))
("liggio" nil (l iy1 jh iy0 ow0))
("light" nil (l ay1 t))
("lightbulb" nil (l ay1 t b ah1 l b))
("lightbulbs" nil (l ay1 t b ah1 l b z))
("lightcap" nil (l ay1 t k ae1 p))
("lighted" nil (l ay1 t ax d))
("lighten" nil (l ay1 t ax n))
("lightened" nil (l ay1 t ax n d))
("lightening" nil (l ay1 t ax n ih0 ng))
("lighter" nil (l ay1 t er0))
("lighters" nil (l ay1 t er0 z))
("lightest" nil (l ay1 t ax s t))
("lightfastness" nil (l ay1 t f ae1 s t n ax s))
("lightfoot" nil (l ay1 t f uh1 t))
("lighthall" nil (l ay1 t hh ao1 l))
("lightheadedness" nil (l ay1 t hh eh1 d ax d n ax s))
("lighthearted" nil (l ay1 t hh aa1 r t ih0 d))
("lighthizer" nil (l ay1 t hh ay1 z er0))
("lighthouse" nil (l ay1 t hh aw1 s))
("lighthouses" nil (l ay1 t hh aw1 s ih0 z))
("lighting" nil (l ay1 t ih0 ng))
("lightle" nil (l ay1 t ax l))
("lightly" nil (l ay1 t l iy0))
("lightner" nil (l ay1 t n er0))
("lightness" nil (l ay1 t n ax s))
("lightning" nil (l ay1 t n ih0 ng))
("lights" nil (l ay1 t s))
("lightsey" nil (l ay1 t s iy0))
("lightship" nil (l ay1 t sh ih1 p))
("lightships" nil (l ay1 t sh ih1 p s))
("lightstone" nil (l ay1 t s t ow1 n))
("lightweight" nil (l ay1 t w ey1 t))
("lighty" nil (l ay1 t iy0))
("ligman" nil (l ih1 g m ax n))
("lignin" nil (l ih1 g n ih0 n))
("lignite" nil (l ih1 g n ay1 t))
("ligon" nil (l ih1 g ax n))
("liguori" nil (l iy0 g ao1 r iy0))
("lijewski" nil (l ih0 y eh1 f s k iy0))
("likable" nil (l ay1 k ax b ax l))
("like" nil (l ay1 k))
("likeable" nil (l ay1 k ax b ax l))
("liked" nil (l ay1 k t))
("likelier" nil (l ay1 k l iy0 er0))
("likeliest" nil (l ay1 k l iy0 ax s t))
("likelihood" nil (l ay1 k l iy0 hh uh1 d))
("likely" nil (l ay1 k l iy0))
("liken" nil (l ay1 k ax n))
("likened" nil (l ay1 k ax n d))
("likeness" nil (l ay1 k n ax s))
("likenesses" nil (l ay1 k n ax s ih0 z))
("likening" nil (l ay1 k ax n ih0 ng))
("likens" nil (l ay1 k ax n z))
("likes" nil (l ay1 k s))
("likewise" nil (l ay1 k w ay1 z))
("likhachov" nil (l ih1 k ax ch aa0 v))
("likhyani" nil (l ih0 k y aa1 n iy0))
("liking" nil (l ay1 k ih0 ng))
("likins" nil (l ih1 k ih0 n z))
("likud" nil (l ih1 k ax d))
("lil" nil (l ih1 l))
("lila" nil (l iy1 l ax))
("lilac" nil (l ay1 l ae1 k))
("lilacs" nil (l ay1 l ae1 k s))
("lilah" nil (l ih1 l ax))
("lilco" nil (l ih1 l k ow0))
("lile" nil (l ay1 l))
("liles" nil (l ay1 l z))
("liley" nil (l ih1 l iy0))
("lili" nil (l ih1 l iy0))
("lilia" nil (l iy1 l iy0 ax))
("lilian" nil (l ih1 l iy0 ax n))
("liliana" nil (l ih1 l iy0 ae1 n ax))
("liliane" nil (l ih0 l iy0 ae1 n))
("lilias" nil (l ay1 l iy0 ax z))
("lilien" nil (l ih1 l iy0 ax n))
("lilienthal" nil (l ih1 l iy0 n th ax l))
("lilies" nil (l ih1 l iy0 z))
("lilith" nil (l ih1 l ih0 th))
("lilja" nil (l iy1 l y ax))
("liljedahl" nil (l ih1 l jh ih0 d aa0 l))
("liljegren" nil (l ih1 l jh ih0 g r eh0 n))
("lill" nil (l ih1 l))
("lilla" nil (l ih1 l ax))
("lillard" nil (l ih1 l er0 d))
("lille" nil (l ih1 l))
("lillehamer" nil (l ih1 l iy0 hh ae1 m er0))
("lillehammer" nil (l ih1 l iy0 hh ae1 m er0))
("liller" nil (l ih1 l er0))
("lilley" nil (l ih1 l iy0))
("lilli" nil (l ih1 l iy0))
("lillian" nil (l ih1 l iy0 ax n))
("lillibridge" nil (l ih1 l ih0 b r ih1 jh))
("lillich" nil (l ih1 l ih0 k))
("lillicrop" nil (l ih1 l ih0 k r aa0 p))
("lillie" nil (l ih1 l iy0))
("lilliputian" nil (l ih1 l ax p y uw1 sh ax n))
("lillis" nil (l ih1 l ih0 s))
("lillo" nil (l ih1 l ow0))
("lilly" nil (l ih1 l iy0))
("lilt" nil (l ih1 l t))
("lily" nil (l ih1 l iy0))
("lilyan" nil (l ih1 l iy0 ax n))
("lilybell" nil (l ih1 l iy0 b eh1 l))
("lim" nil (l ih1 m))
("lima" nil (l ay1 m ax))
("liman" nil (l ay1 m ax n))
("limas" nil (l ay1 m ax z))
("limb" nil (l ih1 m))
("limbach" nil (l ih1 m b aa1 k))
("limbaugh" nil (l ih1 m b ao1))
("limbed" nil (l ih1 m d))
("limber" nil (l ih1 m b er0))
("limberg" nil (l ih1 m b er0 g))
("limbers" nil (l ih1 m b er0 z))
("limbert" nil (l ih1 m b er0 t))
("limbless" nil (l ih1 m l ax s))
("limbo" nil (l ih1 m b ow0))
("limbrick" nil (l ih1 m b r ih0 k))
("limbs" nil (l ih1 m z))
("limburg" nil (l ih1 m b er0 g))
("lime" nil (l ay1 m))
("limehouse" nil (l ay1 m hh aw1 s))
("limelight" nil (l ay1 m l ay1 t))
("limerick" nil (l ih1 m er0 ih0 k))
("limericks" nil (l ih1 m er0 ih0 k s))
("limes" nil (l ay1 m z))
("limestone" nil (l ay1 m s t ow1 n))
("limestones" nil (l ay1 m s t ow1 n z))
("liming" nil (l ay1 m ih0 ng))
("limit" nil (l ih1 m ax t))
("limitation" nil (l ih1 m ih0 t ey1 sh ax n))
("limitations" nil (l ih1 m ih0 t ey1 sh ax n z))
("limited" nil (l ih1 m ax t ax d))
("limiting" nil (l ih1 m ax t ih0 ng))
("limitless" nil (l ih1 m ax t l ax s))
("limits" nil (l ih1 m ax t s))
("limmer" nil (l ih1 m er0))
("limnology" nil (l ih0 m n aa1 l ax jh iy0))
("limo" nil (l ih1 m ow0))
("limoges" nil (l ih0 m ow1 jh ih0 z))
("limon" nil (l ih1 m ax n))
("limonite" nil (l ay1 m ax n ay1 t))
("limousine" nil (l ih1 m ax z iy1 n))
("limousines" nil (l ih1 m ax z iy1 n z))
("limp" nil (l ih1 m p))
("limped" nil (l ih1 m p t))
("limpert" nil (l ih1 m p er0 t))
("limpets" nil (l ih1 m p ax t s))
("limping" nil (l ih1 m p ih0 ng))
("lin" nil (l ih1 n))
("lina" nil (l iy1 n ax))
("linafelter" nil (l ih1 n ax f eh1 l t er0))
("linage" nil (l ay1 n ih0 jh))
("linam" nil (l ih1 n ax m))
("linares" nil (l ih1 n er0 z))
("linc" nil (l ih1 ng k))
("lincare" nil (l ih1 n k eh1 r))
("lince" nil (l ih1 n s))
("lincecum" nil (l ih1 n s ih0 k ax m))
("linch" nil (l ih1 n ch))
("linchpin" nil (l ih1 n ch p ih1 n))
("lincicome" nil (l ih1 n s ih0 k ow1 m))
("linck" nil (l ih1 ng k))
("lincks" nil (l ih1 ng k s))
("lincoln" nil (l ih1 ng k ax n))
("lincolns" nil (l ih1 ng k ax n z))
("lincolnshire" nil (l ih1 ng k ax n sh ih1 r))
("lind" nil (l ih1 n d))
("linda" nil (l ih1 n d ax))
("lindahl" nil (l ih1 n d aa1 l))
("lindaman" nil (l ih1 n d ax m ax n))
("lindamood" nil (l ih1 n d ax m uw1 d))
("lindane" nil (l ih1 n d ey1 n))
("lindau" nil (l ih1 n d aw0))
("lindauer" nil (l ih1 n d aw0 er0))
("lindbeck" nil (l ay1 n d b eh0 k))
("lindberg" nil (l ay1 n d b er0 g))
("lindbergh" nil (l ih1 n d b er0 g))
("lindblad" nil (l ih1 n d b l ax d))
("lindblom" nil (l ih1 n d b l ax m))
("lindbloom" nil (l ih1 n d b l uw1 m))
("lindburg" nil (l ay1 n d b er0 g))
("linde" nil (l ih1 n d))
("lindeen" nil (l ih1 n d iy0 n))
("lindell" nil (l ih1 n d ax l))
("lindeman" nil (l ih1 n d m ax n))
("lindemann" nil (l ih1 n d ax m ax n))
("lindemuth" nil (l ih1 n d ih0 m uw0 th))
("linden" nil (l ih1 n d ax n))
("lindenbaum" nil (l ay1 n d ax n b aw0 m))
("lindenberg" nil (l ih1 n d ax n b er0 g))
("lindenberger" nil (l ih1 n d ax n b er0 g er0))
("lindenmuth" nil (l ih1 n d ih0 n m uw0 th))
("linder" nil (l ih1 n d er0))
("linderman" nil (l ay1 n d er0 m ax n))
("lindfors" nil (l ih1 n d f er0 z))
("lindgren" nil (l ih1 n d g r eh0 n))
("lindh" nil (l ih1 n d))
("lindholm" nil (l ih1 n d hh ow1 l m))
("lindhorst" nil (l ih1 n d hh ao0 r s t))
("lindig" nil (l ih1 n d ih0 g))
("lindler" nil (l ih1 n d l er0))
("lindley" nil (l ih1 n d l iy0))
("lindman" nil (l ih1 n d m ax n))
("lindmark" nil (l ih1 n d m aa1 r k))
("lindner" nil (l ih1 n d n er0))
("lindo" nil (l ih1 n d ow0))
("lindon" nil (l ih1 n d ax n))
("lindow" nil (l ih1 n d ow0))
("lindquist" nil (l ih1 n d k w ih1 s t))
("lindroth" nil (l ih1 n d r ao1 th))
("lindsay" nil (l ih1 n d z iy0))
("lindseth" nil (l ih1 n d s ih0 th))
("lindsey" nil (l ih1 n d z iy0))
("lindskog" nil (l ih1 n d s k ax g))
("lindsley" nil (l ih1 n d s l iy0))
("lindstedt" nil (l ih1 n d s t ih0 t))
("lindstrand" nil (l ih1 n d s t r ax n d))
("lindstrom" nil (l ih1 n d s t r ax m))
("lindvall" nil (l ih1 n d v ax l))
("lindy" nil (l ih1 n d iy0))
("line" nil (l ay1 n))
("lineage" nil (l ih1 n iy0 ax jh))
("lineages" nil (l ih1 n iy0 ih0 jh ih0 z))
("lineal" nil (l ih1 n iy0 ax l))
("linear" nil (l ih1 n iy0 er0))
("linearly" nil (l ih1 n iy0 er0 l iy0))
("lineback" nil (l ay1 n b ae1 k))
("linebacker" nil (l ay1 n b ae1 k er0))
("linebackers" nil (l ay1 n b ae1 k er0 z))
("linebarger" nil (l ih1 n ih0 b aa0 r g er0))
("linebaugh" nil (l ih1 n ih0 b ao0))
("lineberger" nil (l ay1 n b er0 g er0))
("lineberry" nil (l ay1 n b eh1 r iy0))
("lined" nil (l ay1 n d))
("linehan" nil (l ih1 n ih0 hh ae0 n))
("lineman" nil (l ay1 n m ax n))
("linemen" nil (l ay1 n m ax n))
("linen" nil (l ih1 n ax n))
("linenberger" nil (l ih1 n ax n b er0 g er0))
("linens" nil (l ih1 n ax n z))
("liner" nil (l ay1 n er0))
("linerboard" nil (l ay1 n er0 b ao1 r d))
("liners" nil (l ay1 n er0 z))
("lines" nil (l ay1 n z))
("linette" nil (l ih0 n eh1 t))
("lineup" nil (l ay1 n ah1 p))
("lineups" nil (l ay1 n ah1 p s))
("lineweaver" nil (l ay1 n w iy1 v er0))
("linford" nil (l ih1 n f er0 d))
("ling" nil (l ih1 ng))
("lingafelter" nil (l ih1 ng g ax f ih0 l t er0))
("lingard" nil (l ih1 ng g er0 d))
("lingelbach" nil (l ih1 ng g ih0 l b aa0 k))
("lingenfelter" nil (l ih1 ng g ih0 n f ih0 l t er0))
("linger" nil (l ih1 ng g er0))
("lingered" nil (l ih1 ng g er0 d))
("lingerfelt" nil (l ih1 ng g er0 f eh1 l t))
("lingerie" nil (l aa1 n zh er0 ey1))
("lingering" nil (l ih1 ng g er0 ih0 ng))
("lingers" nil (l ih1 ng g er0 z))
("lingg" nil (l ih1 ng g))
("lingle" nil (l ih1 ng g ax l))
("lingner" nil (l ih1 ng n er0))
("lingo" nil (l ih1 ng g ow0))
("lingua" nil (l ih1 ng g w ax))
("linguist" nil (l ih1 ng g w ax s t))
("linguistic" nil (l ih0 ng g w ih1 s t ih0 k))
("linguistically" nil (l ih0 ng g w ih1 s t ih0 k l iy0))
("linguistics" nil (l ih0 ng g w ih1 s t ih0 k s))
("linguists" nil (l ih1 ng g w ax s t s))
("lingus" nil (l ih1 ng g ax s))
("linh" nil (l ih1 n))
("linhardt" nil (l ih1 n hh aa1 r t))
("linhares" nil (l ih1 n hh er0 z))
("linhart" nil (l ih1 n hh aa1 r t))
("lini" nil (l iy1 n iy0))
("lining" nil (l ay1 n ih0 ng))
("lininger" nil (l ay1 n ih0 ng er0))
("linings" nil (l ay1 n ih0 ng z))
("link" nil (l ih1 ng k))
("linkage" nil (l ih1 ng k ax jh))
("linkages" nil (l ih1 ng k ih0 jh ih0 z))
("linke" nil (l ih1 ng k))
("linked" nil (l ih1 ng k t))
("linkenhoker" nil (l ih1 ng k ih0 n hh ax k er0))
("linker" nil (l ih1 ng k er0))
("linking" nil (l ih1 ng k ih0 ng))
("linkous" nil (l ih1 ng k ax s))
("links" nil (l ih1 ng k s))
("linkup" nil (l ih1 ng k ah1 p))
("linkups" nil (l ih1 ng k ah1 p s))
("linley" nil (l ih1 n l iy0))
("linn" nil (l ih1 n))
("linnane" nil (l ih1 n ax n))
("linnas" nil (l ih1 n ax s))
("linne" nil (l ih1 n))
("linnea" nil (l ih1 n iy0 ax))
("linnehan" nil (l ih1 n ih0 hh ae0 n))
("linnell" nil (l ih1 n ax l))
("linneman" nil (l ih1 n m ax n))
("linnemann" nil (l ih1 n m ax n))
("linnet" nil (l ih1 n ih0 t))
("linney" nil (l ih1 n iy0))
("linnik" nil (l ih1 n ih0 k))
("lino" nil (l iy1 n ow0))
("linoleum" nil (l ax n ow1 l iy0 ax m))
("linotype" nil (l ih1 n ow0 t ay1 p))
("linowes" nil (l ih1 n ow0 z))
("linquist" nil (l ih1 n k w ih0 s t))
("lins" nil (l ih1 n z))
("linsay" nil (l ih1 n s ey0))
("linscomb" nil (l ih1 n s k ax m))
("linscott" nil (l ih1 n s k ax t))
("linse" nil (l ih1 n s))
("linseed" nil (l ih1 n s iy1 d))
("linsey" nil (l ih1 n z iy0))
("linskey" nil (l ih1 n s k iy0))
("linsky" nil (l ih1 n s k iy0))
("linsley" nil (l ih1 n s l iy0))
("linson" nil (l ih1 n s ax n))
("linstrom" nil (l ih1 n s t r ax m))
("lint" nil (l ih1 n t))
("lintas" nil (l ih1 n t ax s))
("lintel" nil (l ih1 n t ax l))
("linters" nil (l ih1 n t er0 z))
("linthicum" nil (l ih1 n th ih0 k ax m))
("lintner" nil (l ih1 n t n er0))
("linton" nil (l ih1 n t ax n))
("lintz" nil (l ih1 n t s))
("linus" nil (l ay1 n ax s))
("linville" nil (l iy1 n v ih0 l))
("linwick" nil (l ih1 n w ih1 k))
("linwood" nil (l ih1 n w uh1 d))
("linz" nil (l ih1 n z))
("linzer" nil (l ih1 n z er0))
("linzey" nil (l ih1 n z iy0))
("linzy" nil (l ih1 n z iy0))
("lion" nil (l ay1 ax n))
("lionberger" nil (l ay1 ax n b er0 g er0))
("lionel" nil (l ay1 ax n ax l))
("lionetti" nil (l iy0 ax n eh1 t iy0))
("lionheart" nil (l ay1 ax n hh aa1 r t))
("lionized" nil (l ay1 ax n ay1 z d))
("lions" nil (l ay1 ax n z))
("liotier" nil (l iy0 ow1 t iy0 ey1))
("liotta" nil (l iy0 ow1 t ax))
("liou" nil (l iy0 uw1))
("lip" nil (l ih1 p))
("lipa" nil (l iy1 p ax))
("lipari" nil (l iy0 p aa1 r iy0))
("lipe" nil (l ay1 p))
("lipford" nil (l ih1 p f er0 d))
("lipham" nil (l ih1 f ax m))
("lipid" nil (l ay1 p ax d))
("lipinski" nil (l ih0 p ih1 n s k iy0))
("lipka" nil (l ih1 p k ax))
("lipke" nil (l ih1 p k))
("lipkin" nil (l ih1 p k ih0 n))
("lipkind" nil (l ih1 p k ih0 n d))
("lipman" nil (l ih1 p m ax n))
("lipnick" nil (l ih1 p n ih1 k))
("lipoprotein" nil (l ih1 p ax p r ow1 t iy0 n))
("lipoproteins" nil (l ih1 p ow0 p r ow1 t iy0 n z))
("liposome" nil (l ih1 p ax s ow1 m))
("liposomes" nil (l ih1 p ax s ow1 m z))
("liposuction" nil (l ih1 p ow0 s ah1 k sh ax n))
("lipovsky" nil (l ih0 p aa1 v s k iy0))
("lipp" nil (l ih1 p))
("lippa" nil (l ih1 p ax))
("lippard" nil (l ih1 p er0 d))
("lippe" nil (l ih1 p))
("lipped" nil (l ih1 p t))
("lippens" nil (l ih1 p ax n z))
("lipper" nil (l ih1 p er0))
("lippert" nil (l ih1 p er0 t))
("lippi" nil (l ih1 p iy0))
("lippincott" nil (l ih1 p ih0 n k ax t))
("lippitt" nil (l ih1 p ih0 t))
("lippman" nil (l ih1 p m ax n))
("lippmann" nil (l ih1 p m ax n))
("lippo" nil (l ih1 p ow0))
("lippold" nil (l ih1 p ow1 l d))
("lipps" nil (l ih1 p s))
("lippy" nil (l ih1 p iy0))
("lips" nil (l ih1 p s))
("lipschitz" nil (l ih1 p sh ih0 t s))
("lipschultz" nil (l ih1 p sh ax l t s))
("lipschutz" nil (l ih1 p sh ax t s))
("lipscomb" nil (l ih1 p s k ax m))
("lipsett" nil (l ih1 p s ih0 t))
("lipsey" nil (l ih1 p s iy0))
("lipshie" nil (l ih1 p sh iy0))
("lipshutz" nil (l ih1 p sh ax t s))
("lipsig" nil (l ih1 p s ih0 g))
("lipsitz" nil (l ih1 p s ih0 t s))
("lipski" nil (l ih1 p s k iy1))
("lipsky" nil (l ih1 p s k ay1))
("lipson" nil (l ih1 p s ax n))
("lipstein" nil (l ih1 p s t iy1 n))
("lipstick" nil (l ih1 p s t ih1 k))
("lipsticks" nil (l ih1 p s t ih1 k s))
("liptak" nil (l ih1 p t ax k))
("lipton" nil (l ih1 p t ax n))
("lipuma" nil (l iy0 p uw1 m ax))
("liquefaction" nil (l ih1 k w ax f ae1 k sh ax n))
("liquefied" nil (l ih1 k w ax f ay1 d))
("liquefy" nil (l ih1 k w ax f ay1))
("liqueur" nil (l ih0 k er1))
("liqueurs" nil (l ih0 k er1 z))
("liquid" nil (l ih1 k w ax d))
("liquidate" nil (l ih1 k w ih0 d ey1 t))
("liquidated" nil (l ih1 k w ih0 d ey1 t ih0 d))
("liquidates" nil (l ih1 k w ih0 d ey1 t s))
("liquidating" nil (l ih1 k w ih0 d ey1 t ih0 ng))
("liquidation" nil (l ih1 k w ih0 d ey1 sh ax n))
("liquidations" nil (l ih1 k w ih0 d ey1 sh ax n z))
("liquidator" nil (l ih1 k w ih0 d ey1 t er0))
("liquidators" nil (l ih1 k w ih0 d ey1 t er0 z))
("liquide" nil (l ih0 k w ay1 d))
("liquidities" nil (l ih0 k w ih1 d ax t iy0 z))
("liquidity" nil (l ih0 k w ih1 d ax t iy0))
("liquids" nil (l ih1 k w ax d z))
("liqun" nil (l ih0 k uw1 n))
("liquor" nil (l ih1 k er0))
("liquori" nil (l iy0 k ao1 r iy0))
("liquors" nil (l ih1 k er0 z))
("lira" nil (l ih1 r ax))
("lire" nil (l ih1 r ax))
("lirette" nil (l er0 eh1 t))
("liro" nil (l ih1 r ow0))
("lis" nil (l ih1 s))
("lisa" nil (l iy1 s ax))
("lisabet" nil (l ih1 s ax b eh0 t))
("lisabeth" nil (l ih1 s ax b eh0 th))
("lisak" nil (l ih1 s ax k))
("lisanti" nil (l ih0 s ae1 n t iy0))
("lisbeth" nil (l ih1 s b ih0 th))
("lisbon" nil (l ih1 z b ax n))
("lisby" nil (l ih1 s b iy0))
("lisco" nil (l ih1 s k ow0))
("liscom" nil (l ih1 s k ax m))
("lise" nil (l ay1 z))
("lisec" nil (l ih1 z ax k))
("lisenbee" nil (l ih0 s eh1 n b iy0))
("lisenby" nil (l ih1 s ih0 n b iy0))
("lisette" nil (l ih0 s eh1 t))
("lish" nil (l ih1 sh))
("lishman" nil (l ih1 sh m ax n))
("lisi" nil (l iy1 s iy0))
("lisiecki" nil (l ih0 s iy1 t s k iy0))
("lisk" nil (l ih1 s k))
("liska" nil (l ih1 s k ax))
("liske" nil (l ih1 s k))
("liskey" nil (l ih1 s k iy0))
("lisko" nil (l ih1 s k ow0))
("lisle" nil (l ay1 ax l))
("lisman" nil (l ih1 z m ax n))
("lisowski" nil (l ih0 s ao1 f s k iy0))
("lisp" nil (l ih1 s p))
("liss" nil (l ih1 s))
("lissa" nil (l ih1 s ax))
("lissack" nil (l ih1 s ax k))
("lissie" nil (l ih1 s iy0))
("lissner" nil (l ih1 s n er0))
("lissy" nil (l ih1 s iy0))
("list" nil (l ih1 s t))
("listed" nil (l ih1 s t ax d))
("listen" nil (l ih1 s ax n))
("listened" nil (l ih1 s ax n d))
("listener" nil (l ih1 s ax n er0))
("listeners" nil (l ih1 s ax n er0 z))
("listening" nil (l ih1 s ax n ih0 ng))
("listens" nil (l ih1 s ax n z))
("lister" nil (l ih1 s t er0))
("listeria" nil (l ih0 s t ih1 r iy0 ax))
("listerine" nil (l ih1 s t er0 iy1 n))
("listerines" nil (l ih1 s t er0 iy1 n z))
("listing" nil (l ih1 s t ih0 ng))
("listings" nil (l ih1 s t ih0 ng z))
("listless" nil (l ih1 s t l ax s))
("listlessly" nil (l ih1 s t l ax s l iy0))
("liston" nil (l ih1 s t ax n))
("lists" nil (l ih1 s t s))
("liszewski" nil (l ih0 sh eh1 f s k iy0))
("liszka" nil (l ih1 sh k ax))
("liszt" nil (l ih1 s t))
("lit" nil (l ih1 t))
("lita" nil (l iy1 t ax))
("litaker" nil (l ih1 t ey0 k er0))
("litalien" nil (l ih1 t ax l iy0 n))
("litan" nil (l ay1 t ax n))
("litanies" nil (l ih1 t ax n iy0 z))
("litany" nil (l ih1 t ax n iy0))
("litchfield" nil (l ih1 ch f iy0 l d))
("litchford" nil (l ih1 ch f er0 d))
("litco" nil (l ih1 t k ow1))
("lite" nil (l ay1 t))
("liter" nil (l iy1 t er0))
("literacy" nil (l ih1 t er0 ax s iy0))
("literal" nil (l ih1 t er0 ax l))
("literally" nil (l ih1 t er0 ax l iy0))
("literary" nil (l ih1 t er0 eh1 r iy0))
("literate" nil (l ih1 t er0 ax t))
("literati" nil (l ih1 t er0 aa1 t iy0))
("literature" nil (l ih1 t er0 ax ch er0))
("literaturnaya" nil (l ih0 t eh1 r ax t er0 n ay1 ax))
("liters" nil (l iy1 t er0 z))
("lites" nil (l ay1 t s))
("lithe" nil (l ay1 dh))
("litherland" nil (l ih1 th er0 l ax n d))
("lithgow" nil (l ih1 th g aw0))
("lithic" nil (l ih1 th ih0 k))
("lithium" nil (l ih1 th iy0 ax m))
("lithograph" nil (l ih1 th ax g r ae1 f))
("lithographic" nil (l ih1 th ax g r ae1 f ih0 k))
("lithographs" nil (l ih1 th ax g r ae1 f s))
("lithography" nil (l ax th aa1 g r ax f iy0))
("lithotripter" nil (l ih1 th ax t r ih1 p t er0))
("lithuania" nil (l ih1 th ax w ey1 n iy0 ax))
("lithuanian" nil (l ih1 th ax w ey1 n iy0 ax n))
("lithuanians" nil (l ih1 th ax w ey1 n iy0 ax n z))
("litigant" nil (l ih1 t ih0 g ax n t))
("litigants" nil (l ih1 t ih0 g ax n t s))
("litigate" nil (l ih1 t ih0 g ey1 t))
("litigated" nil (l ih1 t ih0 g ey1 t ih0 d))
("litigating" nil (l ih1 t ih0 g ey1 t ih0 ng))
("litigation" nil (l ih1 t ax g ey1 sh ax n))
("litigator" nil (l ih1 t ax g ey1 t er0))
("litigators" nil (l ih1 t ax g ey1 t er0 z))
("litigious" nil (l ih0 t ih1 jh ax s))
("litke" nil (l ih1 t k iy0))
("litle" nil (l ay1 t ax l))
("litman" nil (l ih1 t m ax n))
("litmus" nil (l ih1 t m ax s))
("lits" nil (l ih1 t s))
("litsey" nil (l ih1 t s iy0))
("litt" nil (l ih1 t))
("litteken" nil (l ih1 t ih0 k ax n))
("littell" nil (l ih1 t ax l))
("litten" nil (l ih1 t ax n))
("litter" nil (l ih1 t er0))
("litteral" nil (l ih1 t er0 ax l))
("littered" nil (l ih1 t er0 d))
("littering" nil (l ih1 t er0 ih0 ng))
("litters" nil (l ih1 t er0 z))
("littig" nil (l ih1 t ih0 g))
("little" nil (l ih1 t ax l))
("littleboy" nil (l ih1 t ax l b oy1))
("littlechild" nil (l ih1 t ax l ch ay1 l d))
("littlefield" nil (l ih1 t ax l f iy1 l d))
("littleford" nil (l ih1 t l ih0 f er0 d))
("littlejohn" nil (l ih1 t ax l jh aa1 n))
("littlepage" nil (l ih1 t ax l p ey1 jh))
("littler" nil (l ih1 t ax l er0))
("littles" nil (l ih1 t ax l z))
("littlest" nil (l ih1 t ax l ax s t))
("littleton" nil (l ih1 t ax l t ax n))
("littlewood" nil (l ih1 t ax l w uh1 d))
("littman" nil (l ih1 t m ax n))
("littmann" nil (l ih1 t m ax n))
("litton" nil (l ih1 t ax n))
("littoral" nil (l ih1 t er0 ax l))
("littrell" nil (l ih1 t r ax l))
("litts" nil (l ih1 t s))
("litty" nil (l ih1 t iy0))
("liturgical" nil (l ax t er1 jh ih0 k ax l))
("liturgy" nil (l ih1 t er0 jh iy0))
("litvack" nil (l ih1 t v ae0 k))
("litvak" nil (l ih1 t v ax k))
("litvin" nil (l ih1 t v ih0 n))
("litwack" nil (l ih1 t w ao0 k))
("litwak" nil (l ih1 t w ax k))
("litwiller" nil (l ih0 t w ih1 l er0))
("litwin" nil (l ih1 t w ih0 n))
("litz" nil (l ih1 t s))
("litzenberg" nil (l ih1 t z ax n b er0 g))
("litzenberger" nil (l ih1 t z ax n b er0 g er0))
("litzinger" nil (l ih1 t z ih0 ng er0))
("liu" nil (l y uw1))
("liuzza" nil (l iy0 uw1 t s ax))
("liuzzi" nil (l iy0 uw1 t s iy0))
("livable" nil (l ih1 v ax b ax l))
("live" j (l ay1 v))
("live" v (l ih1 v))
("lived" nil (l ih1 v d))
("livelier" nil (l ih0 v eh1 l y er0))
("liveliest" nil (l ih1 v ax l iy1 s t))
("livelihood" nil (l ay1 v l iy0 hh uh1 d))
("livelihoods" nil (l ih1 v ax l ih0 hh uh1 d z))
("liveliness" nil (l ay1 v l iy0 n ax s))
("lively" nil (l ay1 v l iy0))
("liven" nil (l ay1 v ax n))
("livened" nil (l ay1 v ax n d))
("livengood" nil (l ih1 v ih0 n g uh0 d))
("liver" nil (l ih1 v er0))
("livergood" nil (l ih1 v er0 g uh1 d))
("liveried" nil (l ih1 v r iy0 d))
("liverman" nil (l ih1 v er0 m ax n))
("livermore" nil (l ih0 v er0 m ao1 r))
("livernois" nil (l ih1 v er0 n w aa1))
("liverpool" nil (l ih1 v er0 p uw1 l))
("livers" nil (l ih1 v er0 z))
("liverworts" nil (l ih1 v er0 w er0 t s))
("lives" v (l ih1 v z))
("lives" n (l ay1 v z))
("livesay" nil (l ih1 v ih0 s ey0))
("livesey" nil (l ih1 v ih0 s iy0))
("livestock" nil (l ay1 v s t aa1 k))
("livezey" nil (l ih1 v ih0 z iy0))
("livia" nil (l ih1 v iy0 ax))
("livid" nil (l ih1 v ih0 d))
("living" nil (l ih1 v ih0 ng))
("livingood" nil (l ih1 v ih0 n g uh1 d))
("livings" nil (l ih1 v ih0 ng z))
("livingston" nil (l ih1 v ih0 ng s t ax n))
("livingstone" nil (l ih1 v ih0 ng s t ow1 n))
("livingwell" nil (l ih1 v ih0 ng w eh1 l))
("livolsi" nil (l iy0 v ow1 l s iy0))
("livonia" nil (l ih0 v ow1 n y ax))
("livoti" nil (l iy0 v ow1 t iy0))
("livsey" nil (l ih1 v z iy0))
("livvie" nil (l ih1 v iy0))
("liz" nil (l ih1 z))
("liza" nil (l iy1 z ax))
("lizabeth" nil (l ih1 z ax b eh0 th))
("lizak" nil (l ih1 z ax k))
("lizard" nil (l ih1 z er0 d))
("lizards" nil (l ih1 z er0 d z))
("lizarraga" nil (l iy0 z aa0 r aa1 g ax))
("lizhi" nil (l ih1 z iy0))
("lizotte" nil (l ih0 z ao1 t))
("lizzie" nil (l ih1 z iy0))
("lizzy" nil (l ih1 z iy0))
("ljubomir" nil (l y uw1 b ow0 m ih1 r))
("llama" nil (l aa1 m ax))
("llamas" nil (l aa1 m ax z))
("llana" nil (l ae1 n ax))
("llanas" nil (l ae1 n ax z))
("llanes" nil (l ey1 n z))
("llano" nil (l aa1 n ow0))
("llanos" nil (l aa1 n ow0 z))
("llerena" nil (l eh0 r ey1 n ax))
("llewellyn" nil (l uw1 eh1 l ih0 n))
("llewelyn" nil (l uw1 ih0 l ih0 n))
("llorens" nil (l ao0 r ey1 ax n z))
("llorente" nil (l ao0 r ey1 n t ey0))
("llosa" nil (l ow1 s ax))
("llovio" nil (l ow1 v iy0 ow0))
("lloyd" nil (l oy1 d))
("lloyds" nil (l oy1 d z))
("lo" nil (l ow1))
("load" nil (l ow1 d))
("loaded" nil (l ow1 d ax d))
("loader" nil (l ow1 d er0))
("loaders" nil (l ow1 d er0 z))
("loadholt" nil (l ow1 d hh ow1 l t))
("loading" nil (l ow1 d ih0 ng))
("loadings" nil (l ow1 d ih0 ng z))
("loadman" nil (l ow1 d m ax n))
("loads" nil (l ow1 d z))
("loaf" nil (l ow1 f))
("loafers" nil (l ow1 f er0 z))
("loafs" nil (l ow1 f s))
("loaiza" nil (l ow0 aa0 iy1 z ax))
("loam" nil (l ow1 m))
("loamy" nil (l ow1 m iy0))
("loan" nil (l ow1 n))
("loaned" nil (l ow1 n d))
("loaning" nil (l ow1 n ih0 ng))
("loanloss" nil (l ow1 n l ao1 s))
("loans" nil (l ow1 n z))
("loar" nil (l ao1 r))
("loath" nil (l ow1 th))
("loathe" nil (l ow1 dh))
("loathed" nil (l ow1 dh d))
("loathing" nil (l ow1 th ih0 ng))
("loathsome" nil (l ow1 dh s ax m))
("loaves" nil (l ow1 v z))
("lob" nil (l aa1 b))
("lobato" nil (l ow0 b aa1 t ow0))
("lobaugh" nil (l aa1 b ao0))
("lobb" nil (l aa1 b))
("lobban" nil (l aa1 b ax n))
("lobbed" nil (l aa1 b d))
("lobbia" nil (l aa1 b iy0 ax))
("lobbied" nil (l aa1 b iy0 d))
("lobbies" nil (l aa1 b iy0 z))
("lobbing" nil (l aa1 b ih0 ng))
("lobby" nil (l aa1 b iy0))
("lobbying" nil (l aa1 b iy0 ih0 ng))
("lobbyist" nil (l aa1 b iy0 ax s t))
("lobbyists" nil (l aa1 b iy0 ih0 s t s))
("lobdell" nil (l aa1 b d ax l))
("lobe" nil (l ow1 b))
("lobed" nil (l ow1 b d))
("lobel" nil (l ow1 b ax l))
("lobell" nil (l ow0 b eh1 l))
("lobello" nil (l ow0 b eh1 l ow0))
("lober" nil (l ow1 b er0))
("loberg" nil (l ow1 b er0 g))
("lobes" nil (l ow1 b z))
("lobianco" nil (l ow0 b iy0 aa1 n k ow0))
("loblaw" nil (l aa0 b l ao1))
("loblolly" nil (l aa1 b l aa1 l iy0))
("lobo" nil (l ow1 b ow0))
("lobos" nil (l ow1 b ow0 s))
("lobosco" nil (l ow0 b ow1 s k ow0))
("lobotomy" nil (l ow1 b ow1 t ow1 m iy0))
("lobster" nil (l aa1 b s t er0))
("lobsterman" nil (l aa1 b s t er0 m ax n))
("lobstermen" nil (l aa1 b s t er0 m ih0 n))
("lobsters" nil (l aa1 b s t er0 z))
("lobue" nil (l ow1 b w eh0))
("lobules" nil (l aa1 b y uw1 l z))
("local" nil (l ow1 k ax l))
("locale" nil (l ow0 k ae1 l))
("locales" nil (l ow0 k ae1 l z))
("localities" nil (l ow0 k ae1 l ih0 t iy0 z))
("locality" nil (l ow0 k ae1 l ax t iy0))
("localization" nil (l ow1 k ax l ax z ey1 sh ax n))
("localized" nil (l ow1 k ax l ay1 z d))
("locally" nil (l ow1 k ax l iy0))
("locals" nil (l ow1 k ax l z))
("locascio" nil (l ax k ae1 s iy0 ow0))
("locastro" nil (l ax k ae1 s t r ow0))
("locate" nil (l ow1 k ey1 t))
("located" nil (l ow1 k ey1 t ax d))
("locatelli" nil (l ow0 k aa0 t eh1 l iy0))
("locates" nil (l ow1 k ey1 t s))
("locating" nil (l ow1 k ey1 t ih0 ng))
("location" nil (l ow0 k ey1 sh ax n))
("locations" nil (l ow0 k ey1 sh ax n z))
("locator" nil (l ow1 k ey1 t er0))
("locey" nil (l ow1 s iy0))
("loch" nil (l aa1 k))
("locher" nil (l aa1 k er0))
("lochhead" nil (l aa1 k hh eh1 d))
("lochner" nil (l aa1 k n er0))
("lochridge" nil (l aa1 k r ih0 jh))
("locicero" nil (l ow0 ch iy0 ch eh1 r ow0))
("lock" nil (l aa1 k))
("lockaby" nil (l aa1 k ax b iy0))
("lockamy" nil (l aa1 k ax m iy0))
("lockard" nil (l aa1 k er0 d))
("lockart" nil (l aa1 k aa1 r t))
("locke" nil (l aa1 k))
("locked" nil (l aa1 k t))
("locken" nil (l aa1 k ax n))
("locker" nil (l aa1 k er0))
("lockerbie" nil (l aa1 k er0 b iy0))
("lockerby" nil (l aa1 k er0 b iy0))
("lockerman" nil (l aa1 k er0 m ax n))
("lockers" nil (l aa1 k er0 z))
("lockert" nil (l aa1 k er0 t))
("lockett" nil (l aa1 k ih0 t))
("lockey" nil (l aa1 k iy0))
("lockhart" nil (l aa1 k hh aa1 r t))
("lockheed" nil (l aa1 k hh iy1 d))
("lockie" nil (l aa1 k iy0))
("locking" nil (l aa1 k ih0 ng))
("locklair" nil (l aa1 k l er0))
("locklar" nil (l aa1 k l er0))
("locklear" nil (l aa1 k l er0))
("lockley" nil (l aa1 k l iy0))
("locklin" nil (l aa1 k l ih0 n))
("lockman" nil (l aa1 k m ax n))
("lockmiller" nil (l aa1 k m ih1 l er0))
("lockner" nil (l aa1 k n er0))
("lockout" nil (l aa1 k aw1 t))
("lockouts" nil (l aa1 k aw1 t s))
("lockrem" nil (l aa1 k r ih0 m))
("lockridge" nil (l aa1 k r ih1 jh))
("locks" nil (l aa1 k s))
("locksmith" nil (l aa1 k s m ih1 th))
("lockstep" nil (l aa1 k s t eh1 p))
("lockup" nil (l aa1 k ah1 p))
("lockwood" nil (l aa1 k w uh1 d))
("lockyer" nil (l aa1 k y er0))
("loco" nil (l ow1 k ow0))
("lococo" nil (l ow0 k ow1 k ow0))
("locomotion" nil (l ow1 k ax m ow1 sh ax n))
("locomotive" nil (l ow1 k ax m ow1 t ih0 v))
("locomotives" nil (l ow1 k ax m ow1 t ih0 v z))
("locomotor" nil (l ow1 k ax m ow1 t er0))
("loconte" nil (l ow0 k ow1 n t iy0))
("locsin" nil (l aa1 k s ih0 n))
("loctite" nil (l aa1 k t ay1 t))
("locurto" nil (l ow0 k uh1 r t ow0))
("locus" nil (l ow1 k ax s))
("locust" nil (l ow1 k ax s t))
("locusts" nil (l ow1 k ax s t s))
("locy" nil (l ow1 s iy0))
("lodato" nil (l ow0 d aa1 t ow0))
("lode" nil (l ow1 d))
("lodema" nil (l ow0 d eh1 m ax))
("loden" nil (l ow1 d ax n))
("loder" nil (l ow1 d er0))
("lodes" nil (l ow1 d z))
("lodestar" nil (l ow1 d s t aa1 r))
("lodestone" nil (l ow1 d s t ow1 n))
("lodge" nil (l aa1 jh))
("lodged" nil (l aa1 jh d))
("lodgepole" nil (l aa1 jh p ow1 l))
("lodges" nil (l aa1 jh ax z))
("lodging" nil (l aa1 jh ih0 ng))
("lodgings" nil (l aa1 jh ih0 ng z))
("lodi" nil (l ow1 d ay0))
("lodico" nil (l ow0 d iy1 k ow0))
("lodley" nil (l aa1 d l iy0))
("lodwick" nil (l aa1 d w ih0 k))
("lodwrick" nil (l aa1 d r ih0 k))
("loe" nil (l ow1))
("loeb" nil (l ow1 b))
("loeber" nil (l ow1 b er0))
("loeffel" nil (l ow1 f ax l))
("loeffelholz" nil (l ow1 f ih0 l hh ow0 l z))
("loeffler" nil (l ow1 f l er0))
("loehr" nil (l ao1 r))
("loella" nil (l ow1 eh1 l ax))
("loeper" nil (l ow1 p er0))
("loepfe" nil (l ow1 p f iy0))
("loera" nil (l ao1 r ax))
("loesch" nil (l ow1 sh))
("loesche" nil (l ow1 sh))
("loescher" nil (l ow1 sh er0))
("loeser" nil (l ow1 z er0))
("loess" nil (l ow1 ax s))
("loew" nil (l aa1 uw0))
("loewe" nil (l aa1 uw0))
("loewen" nil (l aa1 uw0 ax n))
("loewenson" nil (l ow1 w ax n s ax n))
("loewenstein" nil (l aa1 uw0 ax n s t ay0 n))
("loewenthal" nil (l aa1 uw0 ih0 n th ax l))
("loewi" nil (l ow1 w iy0))
("loews" nil (l ow1 z))
("loewy" nil (l ow1 iy0))
("lofaro" nil (l ow0 f aa1 r ow0))
("lofaso" nil (l ow0 f aa1 s ow0))
("loffredo" nil (l ow0 f r ey1 d ow0))
("lofgren" nil (l aa1 f g r eh0 n))
("lofland" nil (l aa1 f l ax n d))
("loflin" nil (l aa1 f l ih0 n))
("lofquist" nil (l aa1 f k w ih0 s t))
("lofstrom" nil (l aa1 f s t r ax m))
("loft" nil (l ao1 f t))
("lofthouse" nil (l ao1 f t hh aw1 s))
("lofthus" nil (l aa1 f th ih0 s))
("loftier" nil (l ao1 f t iy0 er0))
("loftin" nil (l aa1 f t ih0 n))
("loftis" nil (l aa1 f t ih0 s))
("lofton" nil (l aa1 f t ax n))
("lofts" nil (l ao1 f t s))
("loftus" nil (l ao1 f t ax s))
("lofty" nil (l ao1 f t iy0))
("log" nil (l ao1 g))
("logan" nil (l ow1 g ax n))
("logar" nil (l ow1 g er0))
("logarithm" nil (l aa1 g er0 ih1 dh ax m))
("logarithmic" nil (l aa1 g er0 ih1 dh m ih0 k))
("logarithms" nil (l aa1 g er0 ih1 dh ax m z))
("logbook" nil (l ao1 g b uh1 k))
("loge" nil (l ow1 jh))
("loges" nil (l ow1 jh ih0 z))
("logged" nil (l ao1 g d))
("loggerhead" nil (l ao1 g er0 hh eh1 d))
("loggerheads" nil (l aa1 g er0 hh eh1 d z))
("loggers" nil (l aa1 g er0 z))
("loggia" nil (l aa1 g iy0 ax))
("logging" nil (l ao1 g ih0 ng))
("loggins" nil (l aa1 g ih0 n z))
("loghry" nil (l aa1 g r iy0))
("logic" nil (l aa1 jh ih0 k))
("logica" nil (l aa1 jh ih0 k ax))
("logical" nil (l aa1 jh ih0 k ax l))
("logically" nil (l aa1 jh ih0 k l iy0))
("logician" nil (l ow0 jh ih1 sh ax n))
("logicon" nil (l aa1 jh ih0 k aa1 n))
("logie" nil (l ow1 jh iy0))
("logistic" nil (l ax jh ih1 s t ih0 k))
("logistical" nil (l ax jh ih1 s t ih0 k ax l))
("logistically" nil (l ax jh ih1 s t ih0 k l iy0))
("logistics" nil (l ax jh ih1 s t ih0 k s))
("logiudice" nil (l ow0 jh uw1 d ax s))
("logjam" nil (l aa1 g jh ae1 m))
("logjams" nil (l aa1 g jh ae1 m z))
("logo" nil (l ow1 g ow0))
("logograms" nil (l ao1 g ax g r ae1 m z))
("logographic" nil (l ao1 g ax g r ae1 f ih0 k))
("logos" nil (l ow1 g ow1 z))
("logothetis" nil (l ow1 g ow0 th eh1 t ih0 s))
("lograsso" nil (l ow0 g r aa1 s ow0))
("logs" nil (l ao1 g z))
("logsdon" nil (l aa1 g z d ax n))
("logston" nil (l aa1 g s t ax n))
("logue" nil (l ow1 g))
("loguidice" nil (l ow0 g ay1 d ih0 s))
("logwood" nil (l aa1 g w uh1 d))
("loh" nil (l ow1))
("lohengrin" nil (l ow1 ax n g r ih0 n))
("lohman" nil (l ow1 m ax n))
("lohmann" nil (l ow1 m ax n))
("lohmeier" nil (l ow1 m ay0 er0))
("lohmeyer" nil (l ow1 m ay0 er0))
("lohn" nil (l aa1 n))
("lohner" nil (l ow1 n er0))
("lohnes" nil (l ow1 n z))
("lohr" nil (l ao1 r))
("lohrenz" nil (l ao1 r ih0 n s))
("lohrey" nil (l aa1 r iy0))
("lohrman" nil (l ao1 r m ax n))
("lohrmann" nil (l ao1 r m ax n))
("lohse" nil (l ow1 s))
("loiacano" nil (l ow0 y ax k aa1 n ow0))
("loiacono" nil (l ow0 y ax k ow1 n ow0))
("loibl" nil (l oy1 b ax l))
("loin" nil (l oy1 n))
("loincloth" nil (l oy1 n k l ao1 th))
("loire" nil (l oy1 r))
("lois" nil (l ow1 ax s))
("loise" nil (l uw1 ax s))
("loiseau" nil (l uw1 ax s ow1))
("loisel" nil (l oy0 z eh1 l))
("loiselle" nil (l oy0 z eh1 l))
("loiter" nil (l oy1 t er0))
("loitering" nil (l oy1 t er0 ih0 ng))
("lok" nil (l aa1 k))
("loken" nil (l ow1 k ax n))
("loker" nil (l ow1 k er0))
("lokey" nil (l ow1 k iy0))
("lokken" nil (l aa1 k ax n))
("lola" nil (l ow1 l ax))
("loleta" nil (l ow0 l eh1 t ax))
("lolita" nil (l ow0 l iy1 t ax))
("loll" nil (l aa1 l))
("lollapalooza" nil (l aa1 l ax p ax l uw1 z ax))
("lollapaloozas" nil (l aa1 l ax p ax l uw1 z ax z))
("lollar" nil (l aa1 l er0))
("lollard" nil (l aa1 l er0 d))
("lolley" nil (l aa1 l iy0))
("lolli" nil (l aa1 l iy0))
("lolling" nil (l aa1 l ih0 ng))
("lollipop" nil (l aa1 l iy0 p aa1 p))
("lollipops" nil (l aa1 l iy0 p aa1 p s))
("lollis" nil (l ax l iy1 s))
("lolly" nil (l aa1 l iy0))
("loma" nil (l ow1 m ax))
("lomak" nil (l ow1 m ae0 k))
("loman" nil (l ow1 m ax n))
("lomas" nil (l ow1 m ax z))
("lomasney" nil (l aa1 m ax s n iy0))
("lomax" nil (l ow1 m ae1 k s))
("lomb" nil (l aa1 m))
("lomba" nil (l aa1 m b ax))
("lombard" nil (l aa1 m b aa0 r d))
("lombardi" nil (l aa0 m b aa1 r d iy0))
("lombardo" nil (l aa0 m b aa1 r d ow0))
("lombards" nil (l aa1 m b aa0 r d z))
("lomeli" nil (l ow0 m eh1 l iy0))
("lommen" nil (l ow1 m ax n))
("lomonaco" nil (l ow0 m ow0 n aa1 k ow0))
("lompoc" nil (l aa1 m p aa0 k))
("lon" nil (l aa1 n))
("lona" nil (l ow1 n ax))
("lonardo" nil (l ow0 n aa1 r d ow0))
("lonas" nil (l ow1 n ax z))
("loncar" nil (l aa1 n k aa0 r))
("londo" nil (l aa1 n d ow0))
("london" nil (l ah1 n d ax n))
("londonderry" nil (l ah1 n d ax n d eh1 r iy0))
("londoner" nil (l ah1 n d ax n er0))
("londoners" nil (l ah1 n d ax n er0 z))
("londono" nil (l ow0 n d ow1 n ow0))
("lone" nil (l ow1 n))
("loneliness" nil (l ow1 n l iy0 n ax s))
("lonely" nil (l ow1 n l iy0))
("loner" nil (l ow1 n er0))
("lonergan" nil (l aa1 n er0 g ax n))
("lonero" nil (l ow0 n eh1 r ow0))
("lones" nil (l ow1 n z))
("lonesome" nil (l ow1 n s ax m))
("lonetree" nil (l ow1 n t r iy1))
("loney" nil (l ow1 n iy0))
("long" nil (l ao1 ng))
("longacre" nil (l ao1 ng ey1 k er0))
("longaker" nil (l aa1 ng g ax k er0))
("longan" nil (l aa1 ng g ax n))
("longanecker" nil (l aa1 ng g ax n eh0 k er0))
("longbow" nil (l ao1 ng b ow1))
("longbrake" nil (l ao1 ng b r ey1 k))
("longcor" nil (l ow0 ng g k ao1 r))
("longden" nil (l ao1 ng d eh1 n))
("longe" nil (l aa1 n jh))
("longed" nil (l ao1 ng d))
("longenecker" nil (l aa1 n jh n eh0 k er0))
("longer" nil (l ao1 ng g er0))
("longerbeam" nil (l ao1 ng g er0 b iy1 m))
("longest" nil (l ao1 ng g ax s t))
("longevity" nil (l ao0 n jh eh1 v ax t iy0))
("longfellow" nil (l ao1 ng f eh1 l ow0))
("longfield" nil (l ao1 ng f iy1 l d))
("longhair" nil (l ao1 ng hh eh1 r))
("longhand" nil (l ao1 ng hh ae1 n d))
("longhi" nil (l ow1 n g iy0))
("longhorn" nil (l ao1 ng hh ao1 r n))
("longhorns" nil (l ao1 ng hh ao1 r n z))
("longhouse" nil (l ao1 ng hh aw1 s))
("longhurst" nil (l aa1 ng hh er0 s t))
("longing" nil (l ao1 ng ih0 ng))
("longingly" nil (l ao1 ng ih0 ng l iy0))
("longings" nil (l ao1 ng ih0 ng z))
("longino" nil (l ow0 ng g iy1 n ow0))
("longish" nil (l ao1 ng ih0 sh))
("longitude" nil (l aa1 n jh ax t uw1 d))
("longitudes" nil (l aa1 n jh ax t uw1 d z))
("longitudinal" nil (l aa1 n jh ax t uw1 d ax n ax l))
("longleaf" nil (l ao1 ng l iy1 f))
("longlegs" nil (l ao1 ng l eh1 g z))
("longley" nil (l aa1 ng g l iy0))
("longman" nil (l ao1 ng m ax n))
("longmeyer" nil (l ao1 ng m ay1 er0))
("longmire" nil (l aa1 ng m ay0 r))
("longmont" nil (l ao1 ng m aa1 n t))
("longmore" nil (l ow1 ng m ao0 r))
("longnecker" nil (l ao1 ng n eh1 k er0))
("longo" nil (l ao1 ng g ow0))
("longobardi" nil (l ow0 ng g ow0 b aa1 r d iy0))
("longoria" nil (l ow0 ng g ao1 r iy0 ax))
("longpre" nil (l aa1 ng p er0))
("longrange" nil (l ao1 ng r ey1 n jh))
("longs" nil (l ao1 ng z))
("longshore" nil (l ao1 ng sh ao1 r))
("longshoremen" nil (l ao1 ng sh ao1 r m ih0 n))
("longshot" nil (l ao1 ng sh aa1 t))
("longspurs" nil (l ao1 ng s p er1 z))
("longstaff" nil (l ao1 ng s t ae1 f))
("longstanding" nil (l ao1 ng s t ae1 n d ih0 ng))
("longstreet" nil (l ao1 ng s t r iy1 t))
("longstreth" nil (l aa1 ng s t r eh0 th))
("longsworth" nil (l ao1 ng z w er1 th))
("longterm" nil (l ao1 ng t er1 m))
("longtime" nil (l ao1 ng t ay1 m))
("longtin" nil (l ao1 ng t ih0 n))
("longton" nil (l ao1 ng t ax n))
("longuet" nil (l ao1 ng w eh1 t))
("longview" nil (l ao1 ng v y uw1))
("longwell" nil (l ao1 ng w eh1 l))
("longwood" nil (l ao1 ng w uh1 d))
("longworth" nil (l ao1 ng w er1 th))
("lonigro" nil (l ow0 n iy1 g r ow0))
("lonnie" nil (l ao1 n iy0))
("lonon" nil (l ow1 n ax n))
("lonrho" nil (l aa1 n r ow0))
("lons" nil (l aa1 n z))
("lonsdale" nil (l aa1 n s d ey1 l))
("lonski" nil (l aa1 n s k iy0))
("lonsky" nil (l aa1 n s k iy0))
("lonsway" nil (l aa1 n s w ey0))
("loo" nil (l uw1))
("looart" nil (l uw1 aa0 r t))
("looby" nil (l uw1 b iy0))
("look" nil (l uh1 k))
("lookabaugh" nil (l uw1 k ax b ao0))
("lookabill" nil (l uh1 k ax b ih1 l))
("lookalike" nil (l uh1 k ax l ay1 k))
("looked" nil (l uh1 k t))
("looker" nil (l uh1 k er0))
("looking" nil (l uh1 k ih0 ng))
("lookingbill" nil (l uh1 k ih0 ng b ih1 l))
("lookout" nil (l uh1 k aw1 t))
("looks" nil (l uh1 k s))
("loom" nil (l uw1 m))
("looman" nil (l uw1 m ax n))
("loomed" nil (l uw1 m d))
("loomer" nil (l uw1 m er0))
("looming" nil (l uw1 m ih0 ng))
("loomis" nil (l uw1 m ih0 s))
("looms" nil (l uw1 m z))
("loon" nil (l uw1 n))
("looney" nil (l uw1 n iy0))
("loong" nil (l uh1 ng))
("loons" nil (l uw1 n z))
("loony" nil (l uw1 n iy0))
("loop" nil (l uw1 p))
("looped" nil (l uw1 p t))
("looper" nil (l uw1 p er0))
("loopers" nil (l uw1 p er0 z))
("loophole" nil (l uw1 p hh ow1 l))
("loopholes" nil (l uw1 p hh ow1 l z))
("loops" nil (l uw1 p s))
("loopy" nil (l uw1 p iy0))
("loos" nil (l uw1 z))
("loose" nil (l uw1 s))
("loosed" nil (l uw1 s t))
("loosely" nil (l uw1 s l iy0))
("loosen" nil (l uw1 s ax n))
("loosened" nil (l uw1 s ax n d))
("loosening" nil (l uw1 s ax n ih0 ng))
("loosens" nil (l uw1 s ax n z))
("looser" nil (l uw1 s er0))
("looses" nil (l uw1 s ih0 z))
("loosestrife" nil (l uw1 s s t r ay1 f))
("loosestrifes" nil (l uw1 s s t r ay1 f s))
("loosing" nil (l uw1 s ih0 ng))
("loot" nil (l uw1 t))
("looted" nil (l uw1 t ax d))
("lootens" nil (l uw1 t ax n z))
("looter" nil (l uw1 t er0))
("looters" nil (l uw1 t er0 z))
("looting" nil (l uw1 t ih0 ng))
("lop" nil (l aa1 p))
("lopardo" nil (l ow0 p aa1 r d ow0))
("lopata" nil (l ow0 p aa1 t ax))
("lopatin" nil (l ow0 p aa0 t iy1 n))
("lope" nil (l ow1 p))
("lopeman" nil (l ow1 p m ax n))
("loper" nil (l ow1 p er0))
("lopes" nil (l ow1 p s))
("lopez" nil (l ow1 p eh0 z))
("lopiccolo" nil (l ow0 p iy0 k ow1 l ow0))
("lopid" nil (l ow1 p ih0 d))
("lopinto" nil (l ax p ih1 n t ow0))
("lopp" nil (l aa1 p))
("lopped" nil (l aa1 p t))
("loppers" nil (l aa1 p er0 z))
("lopping" nil (l aa1 p ih0 ng))
("loppnow" nil (l aa1 p n aw0))
("lopresti" nil (l ow0 p r eh1 s t iy0))
("lopresto" nil (l ow0 p r eh1 s t ow0))
("loprete" nil (l ow1 p r iy0 t))
("lopsided" nil (l aa1 p s ay1 d ih0 d))
("loquacious" nil (l ow0 k w ey1 sh ax s))
("loquat" nil (l ow1 k w aa1 t))
("lor" nil (l ao1 r))
("lora" nil (l ao1 r ax))
("lorah" nil (l ao1 r ax))
("lorain" nil (l er0 ey1 n))
("loraine" nil (l er0 ey1 n))
("loral" nil (l ao1 r ax l))
("loran" nil (l ao1 r ae1 n))
("lorance" nil (l ao1 r ax n s))
("lorang" nil (l ao1 r ax ng))
("loranger" nil (l ao1 r ey0 n jh er0))
("lorber" nil (l ao1 r b er0))
("lorch" nil (l ao1 r k))
("lord" nil (l ao1 r d))
("lorden" nil (l ao1 r d ax n))
("lordi" nil (l ao1 r d iy0))
("lords" nil (l ao1 r d z))
("lordstown" nil (l ao1 r d z t aw1 n))
("lore" nil (l ao1 r))
("lorean" nil (l ao1 r iy0 ax n))
("loredo" nil (l ao0 r ey1 d ow0))
("loree" nil (l ao1 r iy1))
("lorek" nil (l ao1 r ih0 k))
("lorelei" nil (l ao1 r ih0 l ay1))
("lorelli" nil (l ao0 r eh1 l iy0))
("lorello" nil (l ao0 r eh1 l ow0))
("loren" nil (l ao1 r ax n))
("lorena" nil (l ao0 r eh1 n ax))
("lorenc" nil (l ao1 r ih0 ng k))
("lorence" nil (l ao1 r ax n s))
("lorene" nil (l ao0 r iy1 n))
("lorensen" nil (l ao1 r ih0 n s ax n))
("lorenson" nil (l ao1 r ih0 n s ax n))
("lorentz" nil (l ao1 r ih0 n t s))
("lorentzen" nil (l ao1 r ih0 n t z ax n))
("lorenz" nil (l er0 eh1 n t s))
("lorenza" nil (l ao0 r eh1 n z ax))
("lorenzana" nil (l ao0 r eh0 n z ae1 n ax))
("lorenzen" nil (l ao1 r ih0 n z ax n))
("lorenzetti" nil (l ao0 r eh0 n z eh1 t iy0))
("lorenzi" nil (l ao0 r eh1 n z iy0))
("lorenzini" nil (l ao0 r eh0 n z iy1 n iy0))
("lorenzo" nil (l er0 eh1 n z ow0))
("loreto" nil (l ao0 r eh1 t ow0))
("loretta" nil (l er0 eh1 t ax))
("lorette" nil (l ao0 r eh1 t))
("loretto" nil (l ao0 r eh1 t ow0))
("lorey" nil (l ao1 r iy0))
("lorge" nil (l ao1 r jh))
("lori" nil (l ao1 r iy0))
("loria" nil (l ao1 r iy0 ax))
("lorick" nil (l ao1 r ih0 k))
("lorie" nil (l ao1 r iy0))
("lorillard" nil (l ao1 r ax l aa0 r d))
("lorimar" nil (l ao1 r ax m aa0 r))
("lorimer" nil (l ao1 r ih0 m er0))
("lorimor" nil (l ao1 r ih0 m er0))
("lorin" nil (l ao1 r ih0 n))
("lorincz" nil (l ao1 r ih0 n ch))
("loring" nil (l ao1 r ih0 ng))
("lorino" nil (l ao0 r iy1 n ow0))
("lorio" nil (l ao1 r iy0 ow0))
("loris" nil (l ao1 r ih0 s))
("lorita" nil (l ao0 r iy1 t ax))
("lorman" nil (l ao1 r m ax n))
("lormand" nil (l ao1 r m ax n d))
("lorna" nil (l ao1 r n ax))
("lorne" nil (l ao1 r n))
("lorraine" nil (l er0 ey1 n))
("lorrie" nil (l ao1 r iy0))
("lorry" nil (l ao1 r iy0))
("lorson" nil (l ao1 r s ax n))
("lortel" nil (l ao1 r t eh1 l))
("lortie" nil (l ao1 r t iy0))
("lorton" nil (l ao1 r t ax n))
("lortz" nil (l ao1 r t s))
("lorusso" nil (l ao0 r uw1 s ow0))
("lory" nil (l ao1 r iy0))
("los" nil (l ow1 s))
("losada" nil (l ow0 s aa1 d ax))
("losasso" nil (l ow0 s aa1 s ow0))
("loscalzo" nil (l ow0 s k aa1 l z ow0))
("losch" nil (l ao1 sh))
("loschiavo" nil (l ow0 s k iy0 aa1 v ow0))
("lose" nil (l uw1 z))
("losec" nil (l ow1 z ax k))
("losee" nil (l ax s iy1))
("loseke" nil (l aa1 s ih0 k))
("loser" nil (l uw1 z er0))
("losers" nil (l uw1 z er0 z))
("loses" nil (l uw1 z ax z))
("losey" nil (l aa1 s iy0))
("losh" nil (l aa1 sh))
("losing" nil (l uw1 z ih0 ng))
("losinski" nil (l ax s ih1 n s k iy0))
("losito" nil (l ow0 s iy1 t ow0))
("loso" nil (l ow1 s ow0))
("losoya" nil (l aa1 s oy0 ae0))
("loss" nil (l ao1 s))
("losses" nil (l ao1 s ax z))
("lossing" nil (l ao1 s ih0 ng))
("lost" nil (l ao1 s t))
("losurdo" nil (l ow0 s uh1 r d ow0))
("lot" nil (l aa1 t))
("loth" nil (l aa1 th))
("lothian" nil (l aa1 th iy0 ax n))
("lothrop" nil (l aa1 th r ax p))
("lothson" nil (l aa1 th s ax n))
("lotion" nil (l ow1 sh ax n))
("lotions" nil (l ow1 sh ax n z))
("lotito" nil (l ow0 t iy1 t ow0))
("loton" nil (l ow1 t ax n))
("lots" nil (l aa1 t s))
("lotspeich" nil (l aa1 t s p ay0 k))
("lott" nil (l aa1 t))
("lotta" nil (l aa1 t ax))
("lotte" nil (l aa1 t))
("lotter" nil (l aa1 t er0))
("lotteries" nil (l aa1 t er0 iy0 z))
("lottery" nil (l aa1 t er0 iy0))
("lottes" nil (l aa1 t s))
("lottie" nil (l aa1 t iy0))
("lottman" nil (l aa1 t m ax n))
("lotto" nil (l aa1 t ow1))
("lotts" nil (l aa1 t s))
("lotty" nil (l aa1 t iy0))
("lotus" nil (l ow1 t ax s))
("lotz" nil (l aa1 t s))
("lotze" nil (l ow1 t z))
("lou" nil (l uw1))
("louanna" nil (l aw0 aa1 n ax))
("louart" nil (l uw1 aa0 r t))
("louck" nil (l aw1 k))
("loucks" nil (l aw1 k s))
("loud" nil (l aw1 d))
("louden" nil (l aw1 d ax n))
("loudenslager" nil (l aw1 d ih0 n s l ey0 g er0))
("louder" nil (l aw1 d er0))
("louderback" nil (l aw1 d er0 b ae1 k))
("loudermilk" nil (l aw1 d er0 m ih1 l k))
("loudest" nil (l aw1 d ax s t))
("loudin" nil (l uw0 d ae1 n))
("loudly" nil (l aw1 d l iy0))
("loudon" nil (l aw1 d ax n))
("loudspeaker" nil (l aw1 d s p iy1 k er0))
("loudspeakers" nil (l aw1 d s p iy1 k er0 z))
("louella" nil (l aw0 eh1 l ax))
("louganis" nil (l uw1 g ey1 n ih0 s))
("lougee" nil (l aw1 jh iy0))
("lough" nil (l aw1))
("lougheed" nil (l aw1 iy0 d))
("loughery" nil (l aw1 er0 iy0))
("loughlin" nil (l ao1 f l ih0 n))
("loughman" nil (l aw1 m ax n))
("loughmiller" nil (l aw1 m ax l er0))
("loughner" nil (l aw1 n er0))
("loughney" nil (l aw1 n iy0))
("loughran" nil (l aw1 r ax n))
("loughrey" nil (l aw1 r iy0))
("loughridge" nil (l aw1 r ih0 jh))
("loughry" nil (l aw1 r iy0))
("louie" nil (l uw0 iy1))
("louis" nil (l uw1 ih0 s))
("louisa" nil (l uw0 iy1 z ax))
("louise" nil (l uw0 iy1 z))
("louisette" nil (l w iy0 z eh1 t))
("louisiana" nil (l uw0 iy1 z iy0 ae1 n ax))
("louisville" nil (l uw1 iy0 v ih1 l))
("louk" nil (l aw1 k))
("louks" nil (l aw1 k s))
("lounge" nil (l aw1 n jh))
("lounges" nil (l aw1 n jh ih0 z))
("lounging" nil (l aw1 n jh ih0 ng))
("lounsberry" nil (l aw1 n s b eh0 r iy0))
("loup" nil (l uw1 p))
("loupe" nil (l uw1 p))
("louque" nil (l uw1 k))
("lourdes" nil (l ao1 r d z))
("loureiro" nil (l uh0 r eh1 r ow0))
("lourenco" nil (l aw0 r eh1 n k ow0))
("lourie" nil (l aw1 r iy0))
("loury" nil (l uw1 r iy0))
("louse" nil (l aw1 s))
("lousiness" nil (l aw1 z iy0 n ax s))
("lousy" nil (l aw1 z iy0))
("louth" nil (l aw1 th))
("louthan" nil (l aw1 th ax n))
("loutzenhiser" nil (l aw1 t z ih0 n hh ay0 z er0))
("louvier" nil (l aw1 v iy0 er0))
("louviere" nil (l uw1 v iy0 eh0 r))
("louvre" nil (l uw1 v r ax))
("louw" nil (l uw1))
("loux" nil (l uw1))
("lovaas" nil (l aa1 v aa0 z))
("lovable" nil (l ah1 v ax b ax l))
("lovage" nil (l ah1 v ax jh))
("lovallo" nil (l ow0 v aa1 l ow0))
("lovan" nil (l ow1 v ax n))
("lovas" nil (l ow1 v ax z))
("lovastatin" nil (l ow0 v ae1 s t ax t ih0 n))
("lovato" nil (l ow0 v aa1 t ow0))
("love" nil (l ah1 v))
("loveall" nil (l ah1 v ao1 l))
("lovecchio" nil (l ax v eh1 k iy0 ow0))
("loved" nil (l ah1 v d))
("loveday" nil (l ah1 v d ey1))
("lovegrove" nil (l ah1 v g r ow1 v))
("lovejoy" nil (l ah1 v jh oy1))
("lovelace" nil (l ah1 v l ey1 s))
("loveland" nil (l ah1 v l ax n d))
("loveless" nil (l ah1 v l ax s))
("lovell" nil (l ah1 v ax l))
("lovely" nil (l ah1 v l iy0))
("loven" nil (l ah1 v ax n))
("lover" nil (l ah1 v er0))
("loverde" nil (l ow0 v eh1 r d iy0))
("loveridge" nil (l ah1 v r ih1 jh))
("lovering" nil (l ah1 v er0 ih0 ng))
("lovern" nil (l ah1 v er0 n))
("lovers" nil (l ah1 v er0 z))
("loves" nil (l ah1 v z))
("lovett" nil (l ah1 v ih0 t))
("lovette" nil (l ah1 v eh1 t))
("lovewell" nil (l ah1 v w eh1 l))
("lovgren" nil (l aa1 v g r eh0 n))
("lovick" nil (l aa1 v ih0 k))
("lovie" nil (l ah1 v iy0))
("lovin" nil (l ow1 v ih0 n))
("loving" nil (l ah1 v ih0 ng))
("lovinger" nil (l ah1 v ih0 ng er0))
("lovingly" nil (l ah1 v ih0 ng l iy0))
("lovingood" nil (l aa1 v ih0 n g uh0 d))
("lovings" nil (l ah1 v ih0 ng z))
("lovins" nil (l ow1 v ih0 n z))
("lovitt" nil (l aa1 v ih0 t))
("lovler" nil (l ah1 v l er0))
("lovvorn" nil (l aa1 v er0 n))
("low" nil (l ow1))
("lowborn" nil (l ow1 b ao1 r n))
("lowden" nil (l ow1 d ax n))
("lowder" nil (l ow1 d er0))
("lowdermilk" nil (l ow0 d er1 m ih0 l k))
("lowe" nil (l ow1))
("lowekamp" nil (l ow1 k ae1 m p))
("lowell" nil (l ow1 ax l))
("lowen" nil (l ow1 ax n))
("lowenberg" nil (l ow1 ax n b er0 g))
("lowenstein" nil (l ow1 ax n s t ay1 n))
("lowenthal" nil (l ow1 ih0 n th ax l))
("lower" nil (l ow1 er0))
("lowered" nil (l ow1 er0 d))
("lowering" nil (l ow1 er0 ih0 ng))
("lowers" nil (l ow1 er0 z))
("lowery" nil (l aw1 er0 iy0))
("lowes" nil (l ow1 z))
("lowest" nil (l ow1 ax s t))
("lowey" nil (l ow1 iy0))
("lowing" nil (l ow1 ih0 ng))
("lowlands" nil (l ow1 l ae1 n d z))
("lowly" nil (l ow1 l iy0))
("lowman" nil (l ow1 m ax n))
("lown" nil (l aw1 n))
("lowndes" nil (l ow1 n d iy0 z))
("lowney" nil (l ow1 n iy0))
("lowrance" nil (l ao1 r ax n s))
("lowrey" nil (l aw1 r iy0))
("lowrie" nil (l ao1 r iy0))
("lowrimore" nil (l ao1 r iy0 m ao0 r))
("lowry" nil (l aw1 r iy0))
("lows" nil (l ow1 z))
("lowther" nil (l ow1 dh er0))
("lowy" nil (l ow1 iy0))
("lox" nil (l aa1 k s))
("loy" nil (l oy1))
("loya" nil (l oy1 ax))
("loyal" nil (l oy1 ax l))
("loyalist" nil (l oy1 ax l ih0 s t))
("loyalists" nil (l oy1 ax l ih0 s t s))
("loyall" nil (l oy1 ax l))
("loyally" nil (l oy1 ax l iy0))
("loyalties" nil (l oy1 ax l t iy0 z))
("loyalty" nil (l oy1 ax l t iy0))
("loyce" nil (l oy1 s))
("loyd" nil (l oy1 d))
("loye" nil (l oy1))
("loyer" nil (l oy1 er0))
("loynd" nil (l oy1 n d))
("loyola" nil (l oy1 ow1 l ax))
("loza" nil (l ow1 z ax))
("lozada" nil (l ow0 z aa1 d ax))
("lozano" nil (l ow0 z aa1 n ow0))
("lozeau" nil (l ax z ow1))
("lozenges" nil (l ao1 z ax n jh ax z))
("lozier" nil (l ow1 z iy0 er0))
("lozinski" nil (l ax z ih1 n s k iy0))
("lozito" nil (l ow0 z iy1 t ow0))
("lozon" nil (l ow1 z ax n))
("lozoya" nil (l ow0 z oy1 ax))
("lp" nil (eh1 l p iy1))
("lpn" nil (eh1 l p iy1 eh1 n))
("ls" nil (eh1 l eh1 s))
("lsd" nil (eh1 l eh1 s d iy1))
("ltd" nil (l ih1 m ih0 t ih0 d))
("lu" nil (l uw1))
("lua" nil (l uw1 ax))
("luallen" nil (l uw0 ae1 l ax n))
("luana" nil (l uw0 ae1 n ax))
("luanda" nil (l uw0 aa1 n d ax))
("luane" nil (l uw1 ax n))
("luau" nil (l uw1 aw1))
("luba" nil (l uw1 b ax))
("lubar" nil (l uw1 b aa0 r))
("lubarsky" nil (l ax b aa1 r s k iy0))
("lubas" nil (l uw1 b ax z))
("lubavitcher" nil (l uw1 b ax v ih1 ch er0))
("lubavitchers" nil (l uw1 b ax v ih1 ch er0 z))
("lubben" nil (l ah1 b ax n))
("lubbers" nil (l ah1 b er0 z))
("lubbock" nil (l ah1 b ax k))
("lube" nil (l uw1 b))
("lubeck" nil (l uw1 b eh1 k))
("lubell" nil (l uw1 b eh0 l))
("lubensky" nil (l uw0 b eh1 n s k iy0))
("luber" nil (l uw1 b er0))
("lubicons" nil (l uw1 b ih0 k aa1 n z))
("lubin" nil (l uw1 b ih0 n))
("lubinski" nil (l ax b ih1 n s k iy0))
("lubinsky" nil (l ax b ih1 n s k iy0))
("lubitz" nil (l uw1 b ih0 t s))
("lubke" nil (l ah1 b k))
("lublin" nil (l ah1 b l ih0 n))
("lubow" nil (l uw1 b ow0))
("lubowski" nil (l uw0 b aw1 s k iy0))
("lubrano" nil (l uw0 b r aa1 n ow0))
("lubricant" nil (l uw1 b r ax k ax n t))
("lubricants" nil (l uw1 b r ax k ax n t s))
("lubricate" nil (l uw1 b r ih0 k ey1 t))
("lubricated" nil (l uw1 b r ax k ey1 t ih0 d))
("lubricating" nil (l uw1 b r ax k ey1 t ih0 ng))
("lubrication" nil (l uw1 b r ih0 k ey1 sh ax n))
("lubrizol" nil (l uw1 b r ih0 z aa0 l))
("luby" nil (l uw1 b iy0))
("luc" nil (l uw1 k))
("luca" nil (l uw1 k ax))
("lucado" nil (l uw0 k aa1 d ow0))
("lucarelli" nil (l uw0 k aa0 r eh1 l iy0))
("lucas" nil (l uw1 k ax s))
("lucasfilm" nil (l uw0 k ae1 s f ih0 l m))
("lucca" nil (l uw1 k ax))
("lucchese" nil (l uw0 k ey1 z iy0))
("lucchesi" nil (l uw0 k eh1 s iy0))
("lucchetti" nil (l uw0 k eh1 t iy0))
("lucci" nil (l uw1 ch iy0))
("lucco" nil (l uw1 k ow0))
("luce" nil (l uw1 s))
("lucente" nil (l uw0 ch eh1 n t iy0))
("lucerne" nil (l uw1 s er0 n))
("lucero" nil (l uw0 ch eh1 r ow0))
("lucette" nil (l uw1 s eh1 t))
("lucey" nil (l uw1 s iy0))
("luchini" nil (l uw0 k iy1 n iy0))
("luchsinger" nil (l ah1 k s ih0 n jh er0))
("lucht" nil (l ah1 k t))
("lucia" nil (l uw1 sh ax))
("lucian" nil (l uw1 sh ax n))
("luciani" nil (l uw1 ch ax n iy0))
("lucianna" nil (l uw1 ch ax n ax))
("luciano" nil (l uw0 ch iy0 aa1 n ow0))
("lucich" nil (l ah1 ch ih0 hh))
("lucid" nil (l uw1 s ax d))
("lucida" nil (l uw0 ch iy1 d ax))
("lucido" nil (l uw0 ch iy1 d ow0))
("lucie" nil (l uw1 s iy0))
("lucien" nil (l uw1 s iy0 ax n))
("lucienne" nil (l uw0 ch iy1 eh0 n))
("lucier" nil (l uw1 s iy0 er0))
("lucifer" nil (l uw1 s ax f er0))
("lucile" nil (l uw0 s iy1 l))
("lucille" nil (l uw0 s iy1 l))
("lucinda" nil (l uw0 s ih1 n d ax))
("lucio" nil (l uw1 s iy0 ow0))
("lucite" nil (l uw1 s ay1 t))
("lucius" nil (l uw1 sh ax s))
("luciw" nil (l uw1 s ih0))
("luck" nil (l ah1 k))
("lucke" nil (l ah1 k))
("lucken" nil (l ah1 k ax n))
("luckenbach" nil (l ah1 k ih0 n b aa0 k))
("luckenbaugh" nil (l ax k eh1 n b ao0))
("luckenbill" nil (l ah1 k ih0 n b ax l))
("lucker" nil (l ah1 k er0))
("luckett" nil (l ah1 k ih0 t))
("luckey" nil (l ah1 k iy0))
("luckhardt" nil (l ah1 k hh aa1 r t))
("luckie" nil (l ah1 k iy0))
("luckier" nil (l ah1 k iy0 er0))
("luckiest" nil (l ax k iy1 s t))
("luckily" nil (l ah1 k ax l iy0))
("lucking" nil (l ah1 k ih0 ng))
("luckless" nil (l ah1 k l ax s))
("luckman" nil (l ah1 k m ax n))
("luckow" nil (l ah1 s k ow0))
("lucks" nil (l ah1 k s))
("lucky" nil (l ah1 k iy0))
("luckyn" nil (l ah1 k ih0 n))
("lucrative" nil (l uw1 k r ax t ih0 v))
("lucrecia" nil (l uw0 k r iy1 sh ax))
("lucretia" nil (l uw0 k r iy1 sh ax))
("lucus" nil (l uw1 k ax s))
("lucy" nil (l uw1 s iy0))
("luczak" nil (l ah1 ch ae0 k))
("ludcke" nil (l ah1 d k iy0))
("ludden" nil (l ah1 d ax n))
("luddite" nil (l ah1 d ay1 t))
("luddites" nil (l ah1 d ay1 t s))
("luddy" nil (l ah1 d iy0))
("ludeke" nil (l ah1 d ih0 k))
("ludella" nil (l uw1 d eh1 l ax))
("ludeman" nil (l uw1 d m ax n))
("ludemann" nil (l uw1 d m ax n))
("ludewig" nil (l ah1 d uw0 ih0 g))
("ludicrous" nil (l uw1 d ax k r ax s))
("ludicrously" nil (l uw1 d ax k r ax s l iy0))
("ludington" nil (l ah1 d ih0 ng t ax n))
("ludite" nil (l uw1 d ay0 t))
("ludites" nil (l uw1 d ay0 t s))
("ludke" nil (l ah1 d k iy0))
("ludlam" nil (l ah1 d l ax m))
("ludlow" nil (l ah1 d l ow1))
("ludlum" nil (l ah1 d l ax m))
("ludmer" nil (l ah1 d m er0))
("ludmila" nil (l ax d m ay1 l ax))
("ludmilla" nil (l ax d m ih1 l ax))
("ludolph" nil (l ah1 d ow0 l f))
("ludtke" nil (l ah1 d k iy0))
("ludvigsen" nil (l ah1 d v ih0 g s ax n))
("ludvigson" nil (l ah1 d v ih0 g s ax n))
("ludvik" nil (l ah1 d v ih0 k))
("ludwick" nil (l ah1 d w ih0 k))
("ludwig" nil (l ah1 d w ih0 g))
("ludwiga" nil (l ax d v ay1 g ax))
("ludwigshafen" nil (l ax d w ih1 g sh ax f ax n))
("ludwin" nil (l ah1 d w ih0 n))
("ludy" nil (l uw1 d iy0))
("lue" nil (l uw1))
("luebbe" nil (l uw1 b))
("luebbers" nil (l uh1 b er0 z))
("luebbert" nil (l uh1 b er0 t))
("luebke" nil (l uw1 b k))
("lueck" nil (l uw1 ih0 k))
("luecke" nil (l uw1 k))
("luedecke" nil (l uh1 d ih0 k))
("lueder" nil (l uh1 d er0))
("lueders" nil (l uh1 d er0 z))
("luedke" nil (l uw1 d k iy0))
("luedtke" nil (l uh1 d k iy0))
("lueh" nil (l w eh1))
("luehring" nil (l uh1 r ih0 ng))
("luehrs" nil (l uh1 r z))
("lueken" nil (l uh1 k ax n))
("luella" nil (l uw1 eh1 l ax))
("luelle" nil (l uw1 l))
("luellen" nil (l uh1 l ax n))
("luepke" nil (l uw1 p k iy0))
("luera" nil (l uw0 eh1 r ax))
("lueras" nil (l uh1 r ax z))
("luers" nil (l uw1 er0 z))
("luerssen" nil (l uw1 r s ax n))
("lueth" nil (l uw1 th))
("luetkemeyer" nil (l uh1 t k ih0 m ay0 er0))
("luevano" nil (l uw0 eh0 v aa1 n ow0))
("luff" nil (l ah1 f))
("luffed" nil (l ah1 f t))
("luffman" nil (l ah1 f m ax n))
("lufkin" nil (l ah1 f k ih0 n))
("luft" nil (l ah1 f t))
("lufthansa" nil (l ax f t ae1 n z ax))
("luftig" nil (l ah1 f t ih0 g))
("lufttransport" nil (l ah1 f t r ae1 n z p ao1 r t))
("lug" nil (l ah1 g))
("lugano" nil (l uw0 g aa1 n ow0))
("lugar" nil (l uw1 g er0))
("luge" nil (l uw1 jh))
("luger" nil (l uw1 g er0))
("lugers" nil (l uw1 g er0 z))
("luggage" nil (l ah1 g ax jh))
("lugging" nil (l ah1 g ih0 ng))
("lugi" nil (l uw1 g iy0))
("luginbill" nil (l ah1 g ih0 n b ax l))
("luginbuhl" nil (l ah1 g ih0 n b ax l))
("lugo" nil (l uw1 g ow0))
("lugs" nil (l ah1 g z))
("lugubrious" nil (l uw0 g y uw1 b r iy0 ax s))
("lugworms" nil (l ah1 g w er0 m z))
("luhman" nil (l ah1 m ax n))
("luhmann" nil (l ah1 m ax n))
("luhn" nil (l ah1 n))
("luhr" nil (l er1))
("luhring" nil (l uh1 r ih0 ng))
("luhrs" nil (l uh1 r z))
("lui" nil (l uw1 ih0))
("luick" nil (l uw1 k))
("luigi" nil (l uw0 iy1 jh iy0))
("luikart" nil (l uw1 k aa0 r t))
("luis" nil (l uw0 iy1 s))
("luisa" nil (l uw0 iy1 z ax))
("luisi" nil (l uw1 s iy0))
("luiz" nil (l uw1 z))
("lujan" nil (l uw0 y aa1 n))
("luk" nil (l ah1 k))
("luka" nil (l uw1 k ax))
("lukach" nil (l ah1 k ax k))
("lukacs" nil (l ah1 k ax k s))
("lukas" nil (l uw1 k ax z))
("lukash" nil (l uw0 k ae1 sh))
("lukasiewicz" nil (l ax k aa1 s ax v ih0 ch))
("lukasik" nil (l ax k aa1 s ih0 k))
("lukaszewski" nil (l ax k ax sh eh1 f s k iy0))
("luke" nil (l uw1 k))
("lukehart" nil (l uw1 k hh aa0 r t))
("luken" nil (l uw1 k ax n))
("lukenbill" nil (l ah1 k ih0 n b ax l))
("lukens" nil (l uw1 k ax n z))
("luker" nil (l uw1 k er0))
("lukes" nil (l uw1 k s))
("lukewarm" nil (l uw1 k w ao1 r m))
("lukin" nil (l uw1 k ih0 n))
("lukins" nil (l uw1 k ih0 n z))
("lukman" nil (l ah1 k m ax n))
("lukoil" nil (l uw1 k oy1 l))
("lukowski" nil (l ax k ao1 f s k iy0))
("luks" nil (l ah1 k s))
("lukyanov" nil (l uw1 k y ax n aa0 v))
("lula" nil (l uw1 l ax))
("lulie" nil (l ah1 l iy0))
("lull" nil (l ah1 l))
("lullaby" nil (l ah1 l ax b ay1))
("lulled" nil (l ah1 l d))
("lulling" nil (l ah1 l ih0 ng))
("lulls" nil (l ah1 l z))
("lulu" nil (l uw1 l uw1))
("lum" nil (l ah1 m))
("luma" nil (l uw1 m ax))
("luman" nil (l uw1 m ax n))
("lumb" nil (l ah1 m))
("lumbago" nil (l ax m b ey1 g ow1))
("lumbar" nil (l ah1 m b aa1 r))
("lumbard" nil (l ah1 m b er0 d))
("lumber" nil (l ah1 m b er0))
("lumbering" nil (l ah1 m b er0 ih0 ng))
("lumberjack" nil (l ah1 m b er0 jh ae1 k))
("lumbert" nil (l ah1 m b er0 t))
("lumberyard" nil (l ah1 m b er0 y aa1 r d))
("lumberyards" nil (l ah1 m b er0 y aa1 r d z))
("lumbra" nil (l ah1 m b r ax))
("lumen" nil (l uw1 m ax n))
("lumex" nil (l uw1 m ax k s))
("lumia" nil (l uw1 m iy0 ax))
("lumina" nil (l uw1 m ih0 n ax))
("luminal" nil (l uw1 m ax n ax l))
("luminance" nil (l uw1 m ax n ax n s))
("luminaries" nil (l uw1 m ax n eh1 r iy0 z))
("luminescence" nil (l uw1 m ax n eh1 s ax n s))
("luminescent" nil (l uw1 m ax n eh1 s ax n t))
("luminoso" nil (l uw1 m ih0 n ow1 s ow0))
("luminous" nil (l uw1 m ax n ax s))
("lumley" nil (l ah1 m l iy0))
("lumm" nil (l ah1 m))
("lummus" nil (l ah1 m ax s))
("lumonics" nil (l uw0 m aa1 n ih0 k s))
("lump" nil (l ah1 m p))
("lumpectomies" nil (l ah1 m p eh1 k t ax m iy0 z))
("lumpectomy" nil (l ah1 m p eh1 k t ax m iy0))
("lumped" nil (l ah1 m p t))
("lumping" nil (l ah1 m p ih0 ng))
("lumpkin" nil (l ah1 m p k ih0 n))
("lumpkins" nil (l ah1 m p k ih0 n z))
("lumpp" nil (l ah1 m p))
("lumps" nil (l ah1 m p s))
("lumpur" nil (l ah1 m p uh1 r))
("lumpy" nil (l ah1 m p iy0))
("lumsden" nil (l ah1 m s d ax n))
("luna" nil (l uw1 n ax))
("lunacy" nil (l uw1 n ax s iy0))
("lunar" nil (l uw1 n er0))
("lunati" nil (l uw0 n aa0 t iy1))
("lunatic" nil (l uw1 n ax t ih1 k))
("lunatics" nil (l uw1 n ax t ih1 k s))
("lunceford" nil (l ah1 n s ih0 f ao0 r d))
("lunch" nil (l ah1 n ch))
("lunched" nil (l ah1 n ch t))
("luncheon" nil (l ah1 n ch ax n))
("luncheonette" nil (l ah1 n ch ih0 n eh1 t))
("luncheons" nil (l ah1 n ch ax n z))
("lunches" nil (l ah1 n ch ih0 z))
("lunching" nil (l ah1 n ch ih0 ng))
("lunchroom" nil (l ah1 n ch r uw1 m))
("lunchtime" nil (l ah1 n ch t ay1 m))
("lund" nil (l ah1 n d))
("lundahl" nil (l ah1 n d aa1 l))
("lunday" nil (l ah1 n d ey1))
("lundberg" nil (l ah1 n d b er0 g))
("lundblad" nil (l ah1 n d b l ax d))
("lundborg" nil (l ah1 n d b ao0 r g))
("lunde" nil (l ah1 n d))
("lundeen" nil (l ah1 n d iy0 n))
("lundell" nil (l ah1 n d ax l))
("lunden" nil (l ah1 n d ax n))
("lunder" nil (l ah1 n d er0))
("lundgren" nil (l ah1 n d g r eh0 n))
("lundholm" nil (l ah1 n d hh ow1 l m))
("lundin" nil (l ah1 n d ih0 n))
("lundmark" nil (l ah1 n d m aa1 r k))
("lundquist" nil (l ah1 n d k w ih1 s t))
("lundstedt" nil (l ah1 n d s t ih0 t))
("lundsten" nil (l ah1 n d s ax n))
("lundstrom" nil (l ah1 n d s t r ax m))
("lundt" nil (l ah1 n t))
("lundy" nil (l ah1 n d iy0))
("lunetta" nil (l uw0 n eh1 t ax))
("lung" nil (l ah1 ng))
("lunge" nil (l ah1 n jh))
("lunged" nil (l ah1 n jh d))
("lunger" nil (l ah1 ng er0))
("lunges" nil (l ah1 n jh ih0 z))
("lungfish" nil (l ah1 ng f ih1 sh))
("lunging" nil (l ah1 n jh ih0 ng))
("lungren" nil (l ah1 n g r eh0 n))
("lungs" nil (l ah1 ng z))
("lunn" nil (l ah1 n))
("lunney" nil (l ah1 n iy0))
("lunny" nil (l ah1 n iy0))
("lunsford" nil (l ah1 n s f er0 d))
("lunt" nil (l ah1 n t))
("luntz" nil (l ah1 n t s))
("lunz" nil (l ah1 n z))
("luo" nil (l w ow1))
("luoma" nil (l uw0 ow1 m ax))
("luong" nil (l uw0 ao1 ng))
("luongo" nil (l uw0 ow1 ng g ow0))
("lupa" nil (l uw1 p ax))
("lupatkin" nil (l uw1 p aa1 t k ih0 n))
("lupe" nil (l uw1 p))
("luper" nil (l uw1 p er0))
("lupfer" nil (l ah1 p f er0))
("lupi" nil (l uw1 p iy0))
("lupica" nil (l uw0 p iy1 k ax))
("lupien" nil (l ah1 p iy0 n))
("lupin" nil (l uw1 p ax n))
("lupinacci" nil (l uw0 p iy0 n aa1 ch iy0))
("lupine" nil (l uw1 p ay1 n))
("lupinski" nil (l ax p ih1 n s k iy0))
("luplow" nil (l ah1 p l ow0))
("lupo" nil (l uw1 p ow0))
("luppino" nil (l uw0 p iy1 n ow0))
("luptak" nil (l ah1 p t ax k))
("lupton" nil (l ah1 p t ax n))
("lupus" nil (l uw1 p ax s))
("luque" nil (l uw1 k))
("luquette" nil (l ax k eh1 t))
("lura" nil (l uh1 r ax))
("lurch" nil (l er1 ch))
("lurched" nil (l er1 ch t))
("lurches" nil (l er1 ch ih0 z))
("lurching" nil (l er1 ch ih0 ng))
("lure" nil (l uh1 r))
("lured" nil (l uh1 r d))
("lures" nil (l uh1 r z))
("lurette" nil (l er0 eh1 t))
("lurgi" nil (l er1 jh iy0))
("luria" nil (l uh1 r iy0 ax))
("lurid" nil (l uh1 r ax d))
("lurie" nil (l uh1 r iy0))
("luring" nil (l uh1 r ih0 ng))
("lurk" nil (l er1 k))
("lurked" nil (l er1 k t))
("lurking" nil (l er1 k ih0 ng))
("lurks" nil (l er1 k s))
("lurleen" nil (l er0 l iy1 n))
("lurlene" nil (l er1 l iy0 n))
("lurline" nil (l er1 l ay0 n))
("lurvey" nil (l er0 v ey1))
("lurz" nil (l er1 z))
("lusaka" nil (l uw0 s aa1 k ax))
("lusardi" nil (l uw0 s aa1 r d iy0))
("lusby" nil (l ah1 s b iy0))
("lusch" nil (l ah1 sh))
("luscher" nil (l ah1 sh er0))
("luscombe" nil (l uw0 s k ow1 m b iy0))
("luse" nil (l uw1 z))
("lush" nil (l ah1 sh))
("lusher" nil (l ah1 sh er0))
("lushly" nil (l ah1 sh l iy0))
("lusignan" nil (l ah1 s ih0 g n ax n))
("lusinchi" nil (l uw0 s ih1 n ch iy0))
("lusitania" nil (l uw1 s ax t ey1 n iy0 ax))
("lusitanias" nil (l uw1 s ax t ey1 n iy0 ax z))
("lusk" nil (l ah1 s k))
("luskin" nil (l ah1 s k ih0 n))
("lusky" nil (l ah1 s k iy0))
("lusser" nil (l ah1 s er0))
("lussier" nil (l ah1 s iy0 er0))
("lust" nil (l ah1 s t))
("lusted" nil (l ah1 s t ih0 d))
("luster" nil (l ah1 s t er0))
("lustful" nil (l ah1 s t f ax l))
("lustgarten" nil (l ah1 s t g aa1 r d ax n))
("lustig" nil (l ah1 s t ih0 g))
("lusting" nil (l ah1 s t ih0 ng))
("lustre" nil (l ah1 s t er0))
("lustrous" nil (l ah1 s t r ax s))
("lusty" nil (l ah1 s t iy0))
("lute" nil (l uw1 t))
("luten" nil (l uw1 t ax n))
("luter" nil (l uw1 t er0))
("lutes" nil (l uw1 t s))
("lutey" nil (l uw1 t iy0))
("lutfalla" nil (l ax t f ae1 l ax))
("lutgen" nil (l ah1 t g ax n))
("luth" nil (l uw1 th))
("luther" nil (l uw1 th er0))
("lutheran" nil (l uw1 th er0 ax n))
("luthi" nil (l uw1 dh iy0))
("luthy" nil (l ah1 th iy0))
("lutman" nil (l ah1 t m ax n))
("luton" nil (l uw1 t ax n))
("lutsky" nil (l ah1 t s k iy0))
("lutter" nil (l ah1 t er0))
("lutterman" nil (l ah1 t er0 m ax n))
("luttman" nil (l ah1 t m ax n))
("lutton" nil (l ah1 t ax n))
("luttrell" nil (l ah1 t r ax l))
("luttrull" nil (l ah1 t r ax l))
("luttwak" nil (l ah1 t w aa0 k))
("luty" nil (l uw1 t iy0))
("lutz" nil (l ah1 t s))
("lutze" nil (l ah1 t z))
("lutzke" nil (l ah1 t z k))
("luu" nil (l uw1 uw0))
("luvena" nil (l ah1 v ih0 n ax))
("luvs" nil (l ah1 v z))
("luwana" nil (l ax w ae1 n ax))
("lux" nil (l ah1 k s))
("luxembourg" nil (l ah1 k s ax m b ao1 r g))
("luxenberg" nil (l ah1 k s ax n b er0 g))
("luxor" nil (l ah1 k s er0))
("luxottica" nil (l ah1 k s ow0 t iy1 k ax))
("luxton" nil (l ah1 k s t ax n))
("luxuriant" nil (l ax g zh er1 iy0 ax n t))
("luxuries" nil (l ah1 g zh er0 iy0 z))
("luxurious" nil (l ax g zh er1 iy0 ax s))
("luxury" nil (l ah1 g zh er0 iy0))
("luyster" nil (l ay1 s t er0))
("luz" nil (l ah1 z))
("luzader" nil (l ah1 z ax d er0))
("luzar" nil (l uw1 z er0))
("luzhkov" nil (l uw1 zh k ao1 v))
("luzier" nil (l uw1 z iy0 er0))
("luzon" nil (l uw1 z aa0 n))
("luzzi" nil (l uw1 t s iy0))
("lwin" nil (l w ih1 n))
("lxi" nil (l k s iy1))
("ly" nil (l ay1))
("lyall" nil (l ay1 ax l))
("lybarger" nil (l ay1 b aa1 r g er0))
("lybeck" nil (l ay1 b eh1 k))
("lybrand" nil (l ay1 b r ae1 n d))
("lycan" nil (l ay1 k ax n))
("lycoming" nil (l ay1 k ax m ih0 ng))
("lycra" nil (l ay1 k r ax))
("lyda" nil (l ay1 d ax))
("lydall" nil (l ay1 d ax l))
("lyday" nil (l ay1 d ey1))
("lyde" nil (l ay1 d))
("lyden" nil (l ay1 d ax n))
("lydia" nil (l ih1 d iy0 ax))
("lydic" nil (l ih1 d ih0 k))
("lydick" nil (l ih1 d ih0 k))
("lydie" nil (l ih1 d iy0))
("lydon" nil (l ih1 d ax n))
("lye" nil (l ay1))
("lyell" nil (l ay1 eh1 l))
("lyerla" nil (l ay1 er1 l ax))
("lyerly" nil (l ay1 er0 l iy0))
("lyford" nil (l ih1 f er0 d))
("lying" nil (l ay1 ih0 ng))
("lyke" nil (l ay1 k))
("lykens" nil (l ay1 k ax n z))
("lykes" nil (l ay1 k s))
("lykin" nil (l ay1 k ax n))
("lykins" nil (l ih1 k ax n z))
("lyle" nil (l ay1 l))
("lyles" nil (l ay1 l z))
("lyman" nil (l ay1 m ax n))
("lyme" nil (l ay1 m))
("lymph" nil (l ih1 m f))
("lymphadenopathy" nil (l ih1 m f ax d eh1 n ax p ae1 th iy0))
("lymphocytes" nil (l ih1 m f ax s ay1 t s))
("lymphoma" nil (l ih0 m f ow1 m ax))
("lymphomas" nil (l ih0 m f ow1 m ax z))
("lyn" nil (l ih1 n))
("lynam" nil (l ih1 n ax m))
("lynch" nil (l ih1 n ch))
("lynchburg" nil (l ih1 n ch b er0 g))
("lynched" nil (l ih1 n ch t))
("lynching" nil (l ih1 n ch ih0 ng))
("lynd" nil (l ih1 n d))
("lynda" nil (l ih1 n d ax))
("lynde" nil (l ih1 n d))
("lynden" nil (l ih1 n d ax n))
("lyndhurst" nil (l ih1 n d hh er0 s t))
("lyndon" nil (l ih1 n d ax n))
("lynds" nil (l ih1 n d z))
("lyne" nil (l ay1 n))
("lynes" nil (l ay1 n z))
("lyness" nil (l ay1 n eh1 s))
("lynette" nil (l ay1 n eh1 t))
("lynford" nil (l ih1 n f er0 d))
("lyng" nil (l ih1 ng))
("lynk" nil (l ih1 ng k))
("lynn" nil (l ih1 n))
("lynna" nil (l ay1 n ax))
("lynne" nil (l ih1 n))
("lynott" nil (l ay1 n ax t))
("lynskey" nil (l ih1 n s k iy0))
("lynton" nil (l ih1 n t ax n))
("lynx" nil (l ih1 ng k s))
("lynxes" nil (l ih1 ng k s ih0 z))
("lyon" nil (l ay1 ax n))
("lyondell" nil (l ay1 ax n d eh1 l))
("lyonnais" nil (l iy1 ax n ey1))
("lyonnaise" nil (l ay1 ax n ey1 z))
("lyons" nil (l ay1 ax n z))
("lyphomed" nil (l ay1 f ax m eh0 d))
("lyra" nil (l ay1 r ax))
("lyre" nil (l ay1 r))
("lyric" nil (l ih1 r ih0 k))
("lyrical" nil (l ih1 r ih0 k ax l))
("lyricism" nil (l ay1 r ih0 s ih1 z ax m))
("lyricist" nil (l ay1 r ih0 s ih0 s t))
("lyrics" nil (l ih1 r ih0 k s))
("lyris" nil (l ih1 r ih0 s))
("lyrist" nil (l ay1 r ih0 s t))
("lysaght" nil (l ay1 s aa0 t))
("lysander" nil (l ay1 s ae1 n d er0))
("lysandra" nil (l ay1 s ae1 n d r ax))
("lysine" nil (l ay1 s iy0 n))
("lysis" nil (l ay1 s ih0 s))
("lyski" nil (l ay1 s k iy0))
("lysne" nil (l ay1 n))
("lysol" nil (l ay1 s ao1 l))
("lyssy" nil (l ih1 s iy0))
("lyster" nil (l ih1 s t er0))
("lyter" nil (l ay1 t er0))
("lythgoe" nil (l ih1 th g ow0))
("lytle" nil (l ay1 t ax l))
("lyttle" nil (l ih1 t ax l))
("lytton" nil (l ih1 t ax n))
("lyubimov" nil (l y uw1 b ax m aa0 v))
("lyvers" nil (l ay1 v er0 z))
("m" nil (eh1 m))
("ma" nil (m aa1))
("maack" nil (m aa1 k))
("maaco" nil (m ey1 k ow0))
("maag" nil (m aa1 g))
("maahs" nil (m aa1 z))
("maalox" nil (m ey1 l aa0 k s))
("maas" nil (m aa1 z))
("maass" nil (m aa1 s))
("maassen" nil (m aa1 s ax n))
("maastricht" nil (m aa1 s t r ih1 k t))
("mab" nil (m ae1 b))
("mabe" nil (m ey1 b))
("mabee" nil (m ae1 b iy0))
("mabel" nil (m ey1 b ax l))
("mabelle" nil (m ax b eh1 l))
("maben" nil (m ae1 b ax n))
("maberry" nil (m aa1 b eh0 r iy0))
("mabery" nil (m ae1 b er0 iy0))
("mabey" nil (m ey1 b iy0))
("mabie" nil (m ae1 b iy0))
("mabile" nil (m aa1 b ax l))
("mabin" nil (m ae1 b ih0 n))
("mable" nil (m ey1 b ax l))
("mabley" nil (m ae1 b l iy0))
("mabon" nil (m ey1 b ax n))
("mabrey" nil (m ae1 b r iy0))
("mabry" nil (m ae1 b er0 iy0))
("mabus" nil (m ae1 b ih0 s))
("mac" nil (m ae1 k))
("macabre" nil (m ax k aa1 b r ax))
("macadam" nil (m ax k ae1 d ax m))
("macadamia" nil (m ae1 k ax d ey1 m iy0 ax))
("macallister" nil (m ax k ae1 l ih0 s t er0))
("macalpine" nil (m ax k ae1 l p ay1 n))
("macaluso" nil (m ae1 k ax l uw1 s ow0))
("macandrews" nil (m ax k ae1 n d r uw1 z))
("macao" nil (m ax k aw1))
("macaques" nil (m ax k aa1 k s))
("macari" nil (m aa0 k aa1 r iy0))
("macaroni" nil (m ae1 k er0 ow1 n iy0))
("macarthur" nil (m ax k aa1 r th er0))
("macartney" nil (m ax k aa1 r t n iy0))
("macaulay" nil (m ax k ao1 l iy0))
("macauley" nil (m ax k ao1 l iy0))
("macaw" nil (m ax k ao1))
("macaws" nil (m ax k ao1 z))
("macbeth" nil (m ax k b eh1 th))
("macbride" nil (m ax k b r ay1 d))
("maccabean" nil (m ae1 k ax b iy1 ax n))
("maccabees" nil (m ae1 k ax b iy1 z))
("maccallum" nil (m ax k ae1 l ax m))
("maccaquano" nil (m ae1 k ax k w aa1 n ow0))
("maccarone" nil (m ae1 k er0 ow1 n))
("maccarthy" nil (m ax k aa1 r th iy0))
("macchi" nil (m ae1 k iy0))
("macchia" nil (m ae1 k iy0 ax))
("macchio" nil (m ae1 k iy0 ow0))
("macconnell" nil (m ax k aa1 n ax l))
("maccormack" nil (m ax k ao1 r m ax k))
("macdermott" nil (m ax k d er1 m ax t))
("macdiarmid" nil (m ax k d ih1 r m ih0 d))
("macdill" nil (m ax k d ih1 l))
("macdonald" nil (m ax k d aa1 n ax l d))
("macdonell" nil (m ax k d aa1 n ax l))
("macdonnell" nil (m ax k d aa1 n ax l))
("macdonough" nil (m ax k d aa1 n ax f))
("macdougal" nil (m ax k d uw1 g ax l))
("macdougall" nil (m ax k d uw1 g ax l))
("macduff" nil (m ax k d ah1 f))
("mace" nil (m ey1 s))
("maceachern" nil (m ax k iy1 ch er0 n))
("maceda" nil (m ax s ey1 d ax))
("macedo" nil (m ax s ey1 d ow0))
("macedonia" nil (m ae1 s ax d ow1 n iy0 ax))
("macedonian" nil (m ae1 s ax d ow1 n iy0 ax n))
("macek" nil (m ih0 k eh1 k))
("macera" nil (m ax s eh1 r ax))
("macewan" nil (m ax ch uw1 ax n))
("macewen" nil (m ax k uw1 ax n))
("macey" nil (m ey1 s iy0))
("macfadden" nil (m ax k f ae1 d ax n))
("macfadyen" nil (m ax k f ae1 d iy0 ax n))
("macfarland" nil (m ax k f aa1 r l ax n d))
("macfarlane" nil (m ax k f aa1 r l ax n))
("macgowan" nil (m ax g aw1 ax n))
("macgregor" nil (m ax g r eh1 g er0))
("macgyver" nil (m ax g ay1 v er0))
("mach" nil (m aa1 k))
("macha" nil (m ae1 ch ax))
("machacek" nil (m ae1 k ax s ih0 k))
("machado" nil (m ax ch aa1 d ow0))
("machala" nil (m ax ch aa1 l ax))
("machamer" nil (m ae1 k ax m er0))
("mache" nil (m ae1 ch))
("machel" nil (m ae1 ch ax l))
("machen" nil (m ae1 k ax n))
("macher" nil (m ae1 k er0))
("macheski" nil (m ax ch eh1 s k iy0))
("machete" nil (m ax sh eh1 t iy1))
("machetes" nil (m ax sh eh1 t iy1 z))
("machi" nil (m aa1 k iy0))
("machiavelli" nil (m aa1 k iy0 ax v eh1 l iy0))
("machiavellian" nil (m aa1 k iy0 ax v eh1 l iy0 ax n))
("machida" nil (m ax ch iy1 d ax))
("machin" nil (m ae1 ch ih0 n))
("machination" nil (m ae1 k ax n ey1 sh ax n))
("machinations" nil (m ae1 k ax n ey1 sh ax n z))
("machine" nil (m ax sh iy1 n))
("machinea" nil (m ae1 sh ax n iy1 ax))
("machined" nil (m ax sh iy1 n d))
("machineries" nil (m ax sh iy1 n er0 iy0 z))
("machinery" nil (m ax sh iy1 n er0 iy0))
("machines" nil (m ax sh iy1 n z))
("machining" nil (m ax sh iy1 n ih0 ng))
("machinist" nil (m ax sh iy1 n ax s t))
("machinists" nil (m ax sh iy1 n ih0 s t s))
("machismo" nil (m ax ch ih1 z m ow0))
("machnik" nil (m ae1 k n ih0 k))
("macho" nil (m aa1 ch ow0))
("machold" nil (m ax hh ow1 l d))
("machowski" nil (m ax ch ao1 f s k iy0))
("macht" nil (m ae1 ch t))
("machtley" nil (m ae1 t ch l iy0))
("machuca" nil (m ax ch uw1 k ax))
("machuga" nil (m ax ch uw1 g ax))
("maciag" nil (m ax k ey1 g))
("macias" nil (m aa0 s iy1 ax s))
("maciejewski" nil (m ax ch eh1 f s k iy0))
("maciel" nil (m ih0 k iy1 l))
("macik" nil (m aa1 ch ih0 k))
("macinnes" nil (m ax k ih1 n ax s))
("macinnis" nil (m ax k ih1 n ax s))
("macintax" nil (m ae1 k ax n t ae1 k s))
("macintosh" nil (m ae1 k ax n t ao1 sh))
("macintoshes" nil (m ae1 k ax n t ao1 sh ih0 z))
("macintyre" nil (m ae1 k ax n t ay1 r))
("macioce" nil (m aa0 ch ow1 ch iy0))
("maciolek" nil (m ax ch iy0 ow1 l eh0 k))
("macisaac" nil (m ax ch ih1 s ae1 k))
("maciver" nil (m iy1 k ih0 v er0))
("mack" nil (m ae1 k))
("mackall" nil (m ae1 k ax l))
("mackay" nil (m ax k ey1))
("macke" nil (m ae1 k))
("mackel" nil (m ae1 k ax l))
("mackellar" nil (m ax k eh1 l er0))
("macken" nil (m ae1 k ax n))
("mackenzie" nil (m ax k eh1 n z iy0))
("mackerel" nil (m ae1 k er0 ax l))
("mackert" nil (m ae1 k er0 t))
("mackey" nil (m ae1 k iy0))
("mackie" nil (m ae1 k iy0))
("mackiewicz" nil (m ae1 k iy0 ax w ih0 t s))
("mackin" nil (m ae1 k ih0 n))
("mackinaw" nil (m ae1 k ax n ao1))
("mackinley" nil (m ax k ih1 n l iy0))
("mackinney" nil (m ax k ih1 n iy0))
("mackinnon" nil (m ax k ih1 n ax n))
("mackintosh" nil (m ae1 k ax n t aa1 sh))
("macklem" nil (m ae1 k l ax m))
("mackler" nil (m ae1 k l er0))
("mackley" nil (m ae1 k l iy0))
("macklin" nil (m ae1 k l ih0 n))
("macknight" nil (m ax k n ay1 t))
("macko" nil (m ae1 k ow0))
("mackowiak" nil (m ax s k aw1 iy0 ae0 k))
("mackowski" nil (m ax s k ao1 f s k iy0))
("macks" nil (m ae1 k s))
("macktal" nil (m ae1 k t ax l))
("maclachlan" nil (m ax k l ae1 k l ax n))
("maclaine" nil (m ax k l ey1 n))
("maclaren" nil (m ax k l eh1 r ax n))
("maclaughlin" nil (m ax k l ao1 k l ih0 n))
("maclay" nil (m ax k l ey1))
("maclean" nil (m ax k l ey1 n))
("maclellan" nil (m ax k l eh1 l ax n))
("maclennan" nil (m ax k l eh1 n ax n))
("macleod" nil (m ax k l aw1 d))
("maclin" nil (m ae1 k l ax n))
("macmahon" nil (m ax k m ae1 n))
("macmaster" nil (m ax k m ae1 s t er0))
("macmillan" nil (m ax k m ih1 l ax n))
("macmullen" nil (m ax k m ah1 l ax n))
("macmurray" nil (m ax k m er1 iy0))
("macnab" nil (m ax k n ae1 b))
("macnair" nil (m ax k n eh1 r))
("macnamara" nil (m ae1 k n ax m eh1 r ax))
("macnaughton" nil (m ax k n ao1 t ax n))
("macneal" nil (m ax k n iy1 l))
("macneil" nil (m ax k n iy1 l))
("macneill" nil (m ax k n iy1 l))
("macomb" nil (m ey1 k ax m))
("macomber" nil (m ax k aa1 m b er0))
("macon" nil (m ey1 k ax n))
("macoute" nil (m ax k uw1 t))
("macoutes" nil (m ax k uw1 t s))
("macphail" nil (m ax k f ey1 l))
("macphee" nil (m ax k f iy1))
("macpherson" nil (m ax k f ih1 r s ax n))
("macquarrie" nil (m ax k eh1 r iy0))
("macqueen" nil (m ax k w iy1 n))
("macrae" nil (m ax k r ey1))
("macrame" nil (m ae1 k r ax m ey1))
("macri" nil (m ae1 k r iy0))
("macro" nil (m ae1 k r ow0))
("macrodantin" nil (m ae1 k r ow0 d ae1 n t ih0 n))
("macroeconomic" nil (m ae1 k r ow0 eh0 k ax n aa1 m ih0 k))
("macroeconomics" nil (m ae1 k r ow0 eh0 k ax n aa1 m ih0 k s))
("macromedia" nil (m ae1 k r ow0 m iy1 d iy0 ax))
("macrophage" nil (m ae1 k r ow0 f ey1 jh))
("macrophages" nil (m ae1 k r ow0 f ey1 jh ih0 z))
("macrovision" nil (m ae1 k r ow0 v ih1 zh ax n))
("macs" nil (m ae1 k s))
("macsharry" nil (m ax k sh eh1 r iy0))
("mactaggart" nil (m ax k t ae1 g er0 t))
("mactan" nil (m ax k t ae1 n))
("mactavish" nil (m ax k t ae1 v ih0 sh))
("macumber" nil (m ax k ah1 m b er0))
("macumolo" nil (m ax k uw1 m ow0 l ow0))
("macvicar" nil (m ax k v ih1 k er0))
("macwilliams" nil (m ax k w ih1 l y ax m z))
("macworld" nil (m ae1 k w er1 l d))
("macy" nil (m ey1 s iy0))
("maczko" nil (m aa1 ch k ow0))
("mad" nil (m ae1 d))
("mada" nil (m aa1 d ax))
("madagascar" nil (m ae1 d ax g ae1 s k er0))
("madalena" nil (m ae1 d ax l ey1 n ax))
("madam" nil (m ae1 d ax m))
("madame" nil (m ae1 d ax m))
("madan" nil (m ey1 d ax n))
("madar" nil (m ae1 d er0))
("madara" nil (m aa0 d aa1 r ax))
("madaras" nil (m aa0 d aa1 r aa0 z))
("madaris" nil (m ae1 d er0 ih0 s))
("maday" nil (m aa1 d ey0))
("madcap" nil (m ae1 d k ae1 p))
("madd" nil (m ae1 d))
("maddalena" nil (m aa0 d aa0 l eh1 n ax))
("maddamma" nil (m ax d aa1 m ax))
("madden" nil (m ae1 d ax n))
("maddening" nil (m ae1 d ax n ih0 ng))
("maddeningly" nil (m ae1 d ax n ih0 ng l iy0))
("madder" nil (m ae1 d er0))
("maddie" nil (m ae1 d iy0))
("madding" nil (m ae1 d ih0 ng))
("maddison" nil (m ae1 d ih0 s ax n))
("maddock" nil (m ae1 d ax k))
("maddocks" nil (m ae1 d ax k s))
("maddox" nil (m ae1 d ax k s))
("maddux" nil (m ae1 d ax k s))
("maddy" nil (m ae1 d iy0))
("made" nil (m ey1 d))
("madeira" nil (m ax d ih1 r ax))
("madej" nil (m ae1 d ih0 jh))
("madel" nil (m ae1 d ax l))
("madelaine" nil (m ae1 d ih0 l ey0 n))
("madeleine" nil (m ae1 d ax l eh1 n))
("madelena" nil (m aa0 d eh0 l eh1 n ax))
("madelene" nil (m ae0 d ax l iy1 n))
("madelin" nil (m ae1 d ax l ih0 n))
("madeline" nil (m ae1 d ax l ih0 n))
("madella" nil (m ax d eh1 l ax))
("madelle" nil (m ax d eh1 l))
("madelon" nil (m aa0 d ey0 l ao1 n))
("mademoiselle" nil (m ae1 d ax m ax z eh1 l))
("maden" nil (m ey1 d ax n))
("mader" nil (m ey1 d er0))
("madera" nil (m aa0 d eh1 r ax))
("madere" nil (m ae1 d er0))
("madero" nil (m aa0 d eh1 r ow0))
("madewell" nil (m ae1 d ih0 w eh0 l))
("madey" nil (m ey1 d iy0))
("madge" nil (m ae1 jh))
("madhouse" nil (m ae1 d hh aw1 s))
("madhusudan" nil (m aa1 d hh uw0 s uw1 d ax n))
("madia" nil (m aa1 d iy0 ax))
("madigan" nil (m ae1 d ih0 g ax n))
("madill" nil (m aa0 d iy1 l))
("madison" nil (m ae1 d ax s ax n))
("madkins" nil (m ae1 d k ih0 n z))
("madl" nil (m ae1 d ax l))
("madlen" nil (m ae1 d ax l ax n))
("madlin" nil (m ae1 d l ih0 n))
("madlock" nil (m ae1 d l aa1 k))
("madly" nil (m ae1 d l iy0))
("madman" nil (m ae1 d m ae1 n))
("madmen" nil (m ae1 d m ax n))
("madness" nil (m ae1 d n ax s))
("madoc" nil (m ae1 d ax k))
("madock" nil (m ae1 d ax k))
("madoff" nil (m ae1 d ao1 f))
("madog" nil (m ae1 d ax g))
("madole" nil (m ax d ow1 l))
("madonia" nil (m aa0 d ow1 n iy0 ax))
("madonna" nil (m ax d aa1 n ax))
("madora" nil (m ax d ao1 r ax))
("madore" nil (m ax d ao1 r ey0))
("madra" nil (m aa1 d r ax))
("madras" nil (m ae1 d r ax s))
("madre" nil (m ae1 d r ax))
("madrid" nil (m ax d r ih1 d))
("madrigal" nil (m ae1 d r ax g ax l))
("madrigals" nil (m ae1 d r ax g ax l z))
("madril" nil (m ae1 d r ih0 l))
("madron" nil (m ae1 d r ax n))
("madruga" nil (m ae1 d r uw0 g ax))
("madry" nil (m ae1 d r iy0))
("madsen" nil (m ae1 d s ax n))
("madson" nil (m ae1 d s ax n))
("madstones" nil (m ae1 d s t ow1 n z))
("mady" nil (m ey1 d iy0))
("mae" nil (m ey1))
("maeda" nil (m ey0 iy1 d ax))
("maeder" nil (m eh1 d er0))
("maekawa" nil (m aa1 ih0 k aa1 w ax))
("maelstrom" nil (m ey1 l s t r ax m))
("maenza" nil (m aa0 eh1 n z ax))
("maersk" nil (m eh1 r s k))
("maertens" nil (m eh1 r t ax n z))
("maertz" nil (m eh1 r t s))
("maerz" nil (m eh1 r z))
("maes" nil (m ey1 z))
("maese" nil (m iy1 s))
("maestas" nil (m eh1 s t ax z))
("maestri" nil (m aa0 eh1 s t r iy0))
("maestro" nil (m ay1 s t r ow0))
("maez" nil (m ay0 eh1 z))
("maffei" nil (m ae1 f ay0))
("maffeo" nil (m aa1 f iy0 ow0))
("maffett" nil (m ae1 f ih0 t))
("maffia" nil (m ae1 f iy0 ax))
("maffucci" nil (m aa0 f uw1 ch iy0))
("mafia" nil (m aa1 f iy0 ax))
("mag" nil (m ae1 g))
("magallanes" nil (m ae1 g ax l ey1 n z))
("magallon" nil (m ae1 g ax l aa0 n))
("magan" nil (m ey1 g ax n))
("magana" nil (m aa0 g ae1 n ax))
("magar" nil (m ae1 g er0))
("magarian" nil (m ax g eh1 r iy0 ax n))
("magaw" nil (m ae1 g ao0))
("magazine" nil (m ae1 g ax z iy1 n))
("magaziner" nil (m ae1 g ax z iy1 n er0))
("magazines" nil (m ae1 g ax z iy1 n z))
("magby" nil (m ae1 g b iy0))
("magda" nil (m ae1 g d ax))
("magdala" nil (m aa0 g d aa1 l ax))
("magdalen" nil (m ae1 g d ax l ax n))
("magdalena" nil (m ae1 g d ax l iy1 n ax))
("magdalene" nil (m ae1 g d ax l iy1 n))
("magdaleno" nil (m aa0 g d aa0 l ey1 n ow0))
("magee" nil (m ax g iy1))
("magel" nil (m ae1 g ax l))
("magellan" nil (m ax jh eh1 l ax n))
("magellanic" nil (m ae1 jh ax l ae1 n ih0 k))
("magenta" nil (m ax jh eh1 n t ax))
("mager" nil (m ae1 g er0))
("magerman" nil (m ae1 g er0 m ax n))
("magers" nil (m ae1 g er0 z))
("mages" nil (m ey1 jh ih0 z))
("maggard" nil (m ae1 g er0 d))
("maggart" nil (m ae1 g er0 t))
("maggi" nil (m ae1 jh iy0))
("maggie" nil (m ae1 g iy0))
("maggio" nil (m aa1 jh iy0 ow0))
("maggot" nil (m ae1 g ax t))
("maggots" nil (m ae1 g ax t s))
("maggs" nil (m ae1 g z))
("magi" nil (m ey1 jh ay0))
("magic" nil (m ae1 jh ih0 k))
("magical" nil (m ae1 jh ih0 k ax l))
("magically" nil (m ae1 jh ih0 k ax l iy0))
("magician" nil (m ax jh ih1 sh ax n))
("magicians" nil (m ax jh ih1 sh ax n z))
("magid" nil (m ae1 jh ih0 d))
("magie" nil (m ey1 jh iy0))
("magiera" nil (m aa0 jh ih1 r ax))
("magill" nil (m ae1 jh ax l))
("magin" nil (m ae1 jh ih0 n))
("maginn" nil (m ae1 jh ih0 n))
("maginnis" nil (m ae1 jh ih0 n ih0 s))
("maginot" nil (m ae1 jh ih0 n aa0))
("magistad" nil (m ae1 jh ih0 s t ae1 d))
("magisterial" nil (m ae1 jh ih0 s t iy1 r iy0 ax l))
("magistrate" nil (m ae1 jh ax s t r ey1 t))
("magistrates" nil (m ae1 jh ih0 s t r ey1 t s))
("magistro" nil (m aa0 jh iy1 s t r ow0))
("maglaj" nil (m aa1 g l ay1))
("maglev" nil (m ae1 g l eh1 v))
("magley" nil (m ae1 g l iy0))
("magliano" nil (m aa0 g l iy0 aa1 n ow0))
("maglica" nil (m ae1 g l ih0 k ax))
("maglio" nil (m ae1 g l iy0 ow0))
("magliocco" nil (m aa0 g l iy0 ow1 k ow0))
("maglione" nil (m aa0 g l iy0 ow1 n iy0))
("magma" nil (m ae1 g m ax))
("magna" nil (m ae1 g n ax))
("magnan" nil (m ae1 g n ax n))
("magnani" nil (m aa0 g n aa1 n iy0))
("magnanimous" nil (m ae0 g n ae1 n ax m ax s))
("magnano" nil (m aa0 g n aa1 n ow0))
("magnant" nil (m ae1 g n ax n t))
("magnate" nil (m ae1 g n ax t))
("magnates" nil (m ae1 g n ey1 t s))
("magnavox" nil (m ae1 g n ax v aa0 k s))
("magner" nil (m ae1 g n er0))
("magnesia" nil (m ae0 g n iy1 zh ax))
("magnesite" nil (m ae1 g n ax s ay1 t))
("magnesium" nil (m ae0 g n iy1 z iy0 ax m))
("magness" nil (m ax g n iy1 s))
("magnet" nil (m ae1 g n ax t))
("magnetek" nil (m ae1 g n eh0 t eh1 k))
("magneti" nil (m ae0 g n eh1 t iy0))
("magnetic" nil (m ae0 g n eh1 t ih0 k))
("magnetically" nil (m ae0 g n eh1 t ih0 k ax l iy0))
("magnetics" nil (m ae0 g n eh1 t ih0 k s))
("magnetism" nil (m ae1 g n ax t ih1 z ax m))
("magnetite" nil (m ae1 g n ax t ay1 t))
("magnetization" nil (m ae1 g n ax t ax z ey1 sh ax n))
("magnetized" nil (m ae1 g n ih0 t ay1 z d))
("magnetometer" nil (m ae1 g n ax t aa1 m ax t er0))
("magnetron" nil (m ae1 g n ax t r aa1 n))
("magnets" nil (m ae1 g n ax t s))
("magnification" nil (m ae1 g n ax f ax k ey1 sh ax n))
("magnifications" nil (m ae1 g n ax f ax k ey1 sh ax n z))
("magnificent" nil (m ae0 g n ih1 f ax s ax n t))
("magnificently" nil (m ae0 g n ih1 f ax s ax n t l iy0))
("magnified" nil (m ae1 g n ax f ay1 d))
("magnifiers" nil (m ae1 g n ax f ay1 er0 z))
("magnifies" nil (m ae1 g n ax f ay1 z))
("magnify" nil (m ae1 g n ax f ay1))
("magnifying" nil (m ae1 g n ax f ay1 ih0 ng))
("magnin" nil (m ae1 g n ih0 n))
("magnitogorsk" nil (m ae0 g n ih1 t ax g ao0 r s k))
("magnitude" nil (m ae1 g n ax t uw1 d))
("magnitudes" nil (m ae1 g n ax t uw1 d z))
("magno" nil (m ae1 g n ow0))
("magnolia" nil (m ae0 g n ow1 l y ax))
("magnolias" nil (m ae0 g n ow1 l y ax z))
("magnone" nil (m aa0 g n ow1 n iy0))
("magnum" nil (m ae1 g n ax m))
("magnus" nil (m ae1 g n ax s))
("magnuson" nil (m ae1 g n ax s ax n))
("magnussen" nil (m ae1 g n ax s ax n))
("magnusson" nil (m ae1 g n ax s ax n))
("magoon" nil (m ax g uw1 n))
("magouirk" nil (m ax g w er1 k))
("magowan" nil (m aa0 g ow1 w aa0 n))
("magpie" nil (m ae1 g p ay1))
("magpies" nil (m ae1 g p ay1 z))
("magrane" nil (m ae1 g r ey1 n))
("magrath" nil (m ax g r ae1 th))
("magri" nil (m ae1 g r iy0))
("magrini" nil (m aa0 g r iy1 n iy0))
("magro" nil (m ae1 g r ow0))
("magruder" nil (m ax g r uw1 d er0))
("mags" nil (m ae1 g z))
("magten" nil (m ae1 g t eh1 n))
("maguire" nil (m ax g w ay1 r))
("magwood" nil (m ae1 g w uh1 d))
("magyar" nil (m ae1 g y aa1 r))
("magyars" nil (m ae1 g y aa1 r z))
("mah" nil (m aa1))
("mahabharata" nil (m aa0 hh aa0 b er0 aa1 t ax))
("mahady" nil (m ae1 hh ax d iy0))
("mahaffey" nil (m ax hh ae1 f iy0))
("mahaffy" nil (m ae1 hh ax f iy0))
("mahal" nil (m ax hh aa1 l))
("mahala" nil (m ae1 hh ax l ax))
("mahalah" nil (m ae1 hh ax l ax))
("mahalia" nil (m ax hh ae1 l iy0 ax))
("mahan" nil (m ey1 ax n))
("mahaney" nil (m ax hh ey1 n iy0))
("mahannah" nil (m ax hh ae1 n ax))
("mahany" nil (m ae1 hh ax n iy0))
("mahar" nil (m ae1 hh er0))
("maharaja" nil (m aa1 hh er0 aa1 zh ah1))
("maharajahs" nil (m aa0 hh er0 aa1 jh ax z))
("maharashtra" nil (m aa1 hh er0 aa1 sh t r ah1))
("mahathir" nil (m ae1 hh ax th er0))
("mahatma" nil (m ax hh aa1 t m ax))
("mahayana" nil (m aa1 hh ax y aa1 n ax))
("mahdi" nil (m aa1 d iy0))
("mahe" nil (m ey1 hh iy0))
("maher" nil (m aa1 r))
("mahesh" nil (m ax hh eh1 sh))
("maheu" nil (m ae1 hh y uw0))
("mahfouz" nil (m aa0 f uw1 z))
("mahin" nil (m ae1 hh ih0 n))
("mahindra" nil (m ax hh ih1 n d r ax))
("mahl" nil (m aa1 l))
("mahle" nil (m ey1 hh ax l))
("mahler" nil (m aa1 l er0))
("mahlman" nil (m aa1 l m ax n))
("mahlmann" nil (m aa1 l m ax n))
("mahlstedt" nil (m aa1 l s t ih0 t))
("mahlum" nil (m aa1 l ax m))
("mahmood" nil (m ae1 m uw0 d))
("mahmud" nil (m ae1 m uw0 d))
("mahn" nil (m ae1 n))
("mahnke" nil (m ae1 ng k))
("mahnken" nil (m aa1 ng k ax n))
("mahogany" nil (m ax hh aa1 g ax n iy0))
("mahon" nil (m ey1 ax n))
("mahone" nil (m ae1 hh ax n))
("mahoney" nil (m ax hh ow1 n iy0))
("mahony" nil (m ae1 hh ax n iy0))
("mahood" nil (m aa1 hh uh0 d))
("mahowald" nil (m ae1 hh aw0 ao0 l d))
("mahr" nil (m aa1 r))
("mahrt" nil (m aa1 r t))
("mahtar" nil (m aa1 t er0))
("mahurin" nil (m ae1 hh y er0 ih0 n))
("mahwah" nil (m aa1 w ax))
("mai" nil (m ay1))
("maia" nil (m ay1 ax))
("maid" nil (m ey1 d))
("maida" nil (m ey1 d ax))
("maidel" nil (m ey1 d ax l))
("maiden" nil (m ey1 d ax n))
("maidenform" nil (m ey1 d ax n f ao1 r m))
("maidens" nil (m ey1 d ax n z))
("maidie" nil (m ey1 d iy0))
("maids" nil (m ey1 d z))
("maiello" nil (m ay1 eh1 l ow0))
("maier" nil (m ey1 er0))
("maiers" nil (m ey1 er0 z))
("maietta" nil (m ay0 eh1 t ax))
("mail" nil (m ey1 l))
("mailbox" nil (m ey1 l b aa1 k s))
("mailboxes" nil (m ey1 l b aa1 k s ih0 z))
("maile" nil (m ey1 l))
("mailed" nil (m ey1 l d))
("mailer" nil (m ey1 l er0))
("mailers" nil (m ey1 l er0 z))
("mailey" nil (m ey1 l iy0))
("mailgram" nil (m ey1 l g r ae1 m))
("mailgrams" nil (m ey1 l g r ae1 m z))
("mailhot" nil (m ay1 l hh ax t))
("mailing" nil (m ey1 l ih0 ng))
("mailings" nil (m ey1 l ih0 ng z))
("maillard" nil (m eh0 l aa1 r d))
("maille" nil (m ey1 l))
("maillet" nil (m eh0 l eh1 t))
("mailloux" nil (m eh0 l uw1))
("mailman" nil (m ey1 l m ae1 n))
("mailroom" nil (m ey1 l r uw1 m))
("mails" nil (m ey1 l z))
("mailson" nil (m ey1 l s ax n))
("maim" nil (m ey1 m))
("maimed" nil (m ey1 m d))
("maimone" nil (m ey1 m ow1 n))
("maims" nil (m ey1 m z))
("main" nil (m ey1 n))
("maina" nil (m ey1 n ax))
("mainard" nil (m ey1 n er0 d))
("maine" nil (m ey1 n))
("mainer" nil (m ey1 n er0))
("maines" nil (m ey1 n z))
("mainframe" nil (m ey1 n f r ey1 m))
("mainframes" nil (m ey1 n f r ey1 m z))
("mainichi" nil (m aa1 ih0 n iy1 ch iy0))
("mainland" nil (m ey1 n l ae1 n d))
("mainlanders" nil (m ey1 n l ae1 n d er0 z))
("mainline" nil (m ey1 n l ay1 n))
("mainly" nil (m ey1 n l iy0))
("maino" nil (m ey1 n ow0))
("mainor" nil (m ey1 n er0))
("mainord" nil (m eh0 n ao1 r d))
("mains" nil (m ey1 n z))
("mainstay" nil (m ey1 n s t ey1))
("mainstays" nil (m ey1 n s t ey1 z))
("mainstream" nil (m ey1 n s t r iy1 m))
("mainstreet" nil (m ey1 n s t r iy1 t))
("maintain" nil (m ey0 n t ey1 n))
("maintained" nil (m ey0 n t ey1 n d))
("maintaining" nil (m ey0 n t ey1 n ih0 ng))
("maintains" nil (m ey0 n t ey1 n z))
("maintenance" nil (m ey1 n t ax n ax n s))
("mainville" nil (m ey1 n v ih1 l))
("mainwaring" nil (m ey1 n w eh1 r ih0 ng))
("mainz" nil (m ey1 n z))
("maio" nil (m ey1 ow0))
("maiolo" nil (m ay1 ow0 l ow0))
("maione" nil (m ay1 ow0 n iy0))
("maiorana" nil (m ay0 ao0 r ae1 n ax))
("maiorano" nil (m ay1 ao0 r aa0 n ow0))
("maiorino" nil (m ay0 ao0 r iy1 n ow0))
("mair" nil (m eh1 r))
("maire" nil (m eh1 r))
("mairs" nil (m eh1 r z))
("mais" nil (m ey1 z))
("maisano" nil (m ax s aa1 n ow0))
("maisch" nil (m ay1 sh))
("maisel" nil (m ay1 s ax l))
("maish" nil (m aa1 ih0 sh))
("maisie" nil (m ey1 z iy0))
("maison" nil (m ey1 z ax n))
("maisonet" nil (m eh1 s ax n ih0 t))
("maitland" nil (m ey1 t l ax n d))
("maitre" nil (m ey1 t r ax))
("maize" nil (m ey1 z))
("maiziere" nil (m ey1 z ih1 r))
("majcher" nil (m ae1 jh k er0))
("majchrzak" nil (m ay1 r z ax k))
("majer" nil (m ey1 jh er0))
("majerus" nil (m ae1 jh er0 ih0 s))
("majeske" nil (m ay0 eh1 s k iy0))
("majeski" nil (m ay0 eh1 s k iy0))
("majesta" nil (m aa0 y ey1 s t ax))
("majestic" nil (m ax jh eh1 s t ih0 k))
("majestically" nil (m ax jh eh1 s t ih0 k ax l iy0))
("majesty" nil (m ae1 jh ax s t iy0))
("majette" nil (m ax zh eh1 t))
("majeure" nil (m ax jh uw1 r))
("majewski" nil (m ay0 eh1 f s k iy0))
("majik" nil (m ae1 jh ih0 k))
("majka" nil (m ay1 k ax))
("majkowski" nil (m ay0 k ao1 f s k iy0))
("majnoon" nil (m ae1 jh n uw1 n))
("major" nil (m ey1 jh er0))
("majored" nil (m ey1 jh er0 d))
("majoring" nil (m ey1 jh er0 ih0 ng))
("majoritarian" nil (m ae1 jh er0 ih0 t eh1 r iy0 ax n))
("majorities" nil (m ax jh ao1 r ih0 t iy0 z))
("majority" nil (m ax jh ao1 r ax t iy0))
("majoros" nil (m aa0 y ao1 r ow0 z))
("majors" nil (m ey1 jh er0 z))
("majrooh" nil (m ae1 jh r uw0))
("majure" nil (m aa0 y uh1 r ey0))
("mak" nil (m ae1 k))
("makar" nil (m ae1 k er0))
("makara" nil (m aa0 k aa1 r ax))
("makarewicz" nil (m ax k aa1 r ih0 v ih0 ch))
("make" nil (m ey1 k))
("makegoods" nil (m ey1 k g uh1 d z))
("makela" nil (m ae1 k ih0 l ax))
("makeover" nil (m ae1 k iy0 ow1 v er0))
("makepeace" nil (m ae1 k ih0 p iy0 s))
("maker" nil (m ey1 k er0))
("makers" nil (m ey1 k er0 z))
("makes" nil (m ey1 k s))
("makeshift" nil (m ey1 k sh ih1 f t))
("makeup" nil (m ey1 k ah1 p))
("makhoul" nil (m ax k uw1 l))
("maki" nil (m aa1 k iy0))
("makin" nil (m ae1 k ih0 n))
("makinen" nil (m ae1 k ih0 n ax n))
("making" nil (m ey1 k ih0 ng))
("makings" nil (m ey1 k ih0 ng z))
("makinson" nil (m ae1 k ih0 n s ax n))
("makita" nil (m ax k iy1 t ax))
("mako" nil (m aa1 k ow0))
("makos" nil (m ey1 k ow0 z))
("makoto" nil (m ax k ow1 t ow0))
("makowski" nil (m ax k ao1 f s k iy0))
("makowsky" nil (m ax k aw1 s k iy0))
("makris" nil (m ae1 k r ih0 s))
("makro" nil (m ae1 k r ow0))
("maksoud" nil (m ae0 k s uw1 d))
("maktoum" nil (m ae0 k t uw1 m))
("maktoums" nil (m ae0 k t uw1 m z))
("makuch" nil (m ae1 k ax k))
("mal" nil (m ae1 l))
("mala" nil (m aa1 l ax))
("malabre" nil (m ae1 l ax b r ax))
("malacanang" nil (m ax l ae1 k ax n ae0 ng))
("malacca" nil (m ax l ae1 k ax))
("malachowski" nil (m ax l ax hh ao1 f s k iy0))
("maladies" nil (m ae1 l ax d iy0 z))
("maladjusted" nil (m ae1 l ax jh ah1 s t ih0 d))
("maladroit" nil (m ae1 l ax d r oy1 t))
("malady" nil (m ae1 l ax d iy0))
("malaga" nil (m ae1 l ax g ax))
("malagasy" nil (m ae1 l ax g ae1 s iy0))
("malaise" nil (m ae0 l ey1 z))
("malak" nil (m ae1 l ax k))
("malamud" nil (m ae1 l ax m ax d))
("malamute" nil (m ae1 l ax m y uw1 t))
("malan" nil (m ey1 l ax n))
("maland" nil (m ae1 l ax n d))
("malaney" nil (m ax l ey1 n iy0))
("malanga" nil (m aa0 l aa1 ng g ax))
("malanowski" nil (m ax l ax n ao1 f s k iy0))
("malaprop" nil (m ae1 l ax p r aa1 p))
("malara" nil (m aa0 l aa1 r ax))
("malaria" nil (m ax l eh1 r iy0 ax))
("malarial" nil (m ax l eh1 r iy0 ax l))
("malarkey" nil (m ax l aa1 r k iy0))
("malaspina" nil (m aa0 l aa0 s p iy1 n ax))
("malatesta" nil (m aa0 l aa0 t eh1 s t ax))
("malave" nil (m aa0 l aa1 v iy0))
("malawi" nil (m ax l aa1 w iy0))
("malay" nil (m ey1 l ey0))
("malayan" nil (m ax l ey1 ax n))
("malays" nil (m ax l ey1 z))
("malaysia" nil (m ax l ey1 zh ax))
("malaysian" nil (m ax l ey1 zh ax n))
("malaysians" nil (m ax l ey1 zh ax n z))
("malbon" nil (m ae1 l b ax n))
("malbrough" nil (m ae1 l b r aw0))
("malburg" nil (m ae1 l b er0 g))
("malchow" nil (m aa1 l hh aw0))
("malcolm" nil (m ae1 l k ax m))
("malcolmson" nil (m ae1 l k ax m s ax n))
("malcom" nil (m ae1 l k ax m))
("malcomb" nil (m ae1 l k ax m))
("malcomson" nil (m ae1 l k ax m s ax n))
("malcontents" nil (m ae1 l k ax n t eh1 n t s))
("malczewski" nil (m ax l ch eh1 f s k iy0))
("malden" nil (m ao1 l d ax n))
("maldives" nil (m ao1 l d ay1 v z))
("maldonado" nil (m ae0 l d ax n aa1 d ow0))
("maldutis" nil (m aa0 l d uw1 t ih0 s))
("male" nil (m ey1 l))
("malec" nil (m aa1 l ih0 k))
("malecha" nil (m ax l eh1 ch ax))
("malecki" nil (m ax l eh1 k iy0))
("malefactors" nil (m ae1 l ax f ae1 k t er0 z))
("malek" nil (m aa1 l eh0 k))
("malena" nil (m ae1 l ih0 n ax))
("malenfant" nil (m aa0 l ey1 n f ax n t))
("maler" nil (m ey1 l er0))
("malerba" nil (m aa0 l eh1 r b ax))
("males" nil (m ey1 l z))
("maleski" nil (m ax l eh1 s k iy0))
("malesky" nil (m ax l eh1 s k iy0))
("malevolent" nil (m ax l eh1 v ax l ax n t))
("maley" nil (m ey1 l iy0))
("malfeasance" nil (m ae1 l f iy1 z ax n s))
("malfitano" nil (m ae1 l f ih0 t aa1 n ow0))
("malformations" nil (m ae1 l f ao0 r m ey1 sh ax n z))
("malformed" nil (m ae1 l f ao1 r m d))
("malfunction" nil (m ae0 l f ah1 ng k sh ax n))
("malfunctioned" nil (m ae0 l f ah1 ng k sh ax n d))
("malfunctioning" nil (m ae0 l f ah1 ng k sh ax n ih0 ng))
("malfunctions" nil (m ae0 l f ah1 ng k sh ax n z))
("malhotra" nil (m ae1 l hh aa1 t r ax))
("mali" nil (m aa1 l iy0))
("malia" nil (m aa1 l iy0 ax))
("malibu" nil (m ae1 l ax b uw0))
("malice" nil (m ae1 l ax s))
("malicious" nil (m ax l ih1 sh ax s))
("maliciously" nil (m ax l ih1 sh ih0 s l iy0))
("malick" nil (m ae1 l ih0 k))
("malicki" nil (m ax l ih1 k iy0))
("malicoat" nil (m ae1 l ih0 k ow1 t))
("malignancies" nil (m ax l ih1 g n ax n s iy0 z))
("malignancy" nil (m ax l ih1 g n ax n s iy0))
("malignant" nil (m ax l ih1 g n ax n t))
("maligned" nil (m ax l ay1 n d))
("malik" nil (m ae1 l ih0 k))
("malin" nil (m ae1 l ih0 n))
("malina" nil (m aa0 l iy1 n ax))
("malinak" nil (m ae1 l ih0 n ae1 k))
("malinda" nil (m aa0 l iy1 n d ax))
("malinde" nil (m ae1 l ih0 n d))
("maline" nil (m aa0 l iy1 n iy0))
("malinoski" nil (m ax l ih0 n aw1 s k iy0))
("malinowski" nil (m ax l ih0 n ao1 f s k iy0))
("malinski" nil (m ax l ih1 n s k iy0))
("malis" nil (m aa1 l iy0 z))
("maliszewski" nil (m ax l ih0 sh eh1 f s k iy0))
("malizia" nil (m ax l ih1 z iy0 ax))
("malkiel" nil (m ao1 l k iy0 ax l))
("malkiewicz" nil (m aa1 l k ax v ih0 ch))
("malkin" nil (m ae1 l k ih0 n))
("malkovich" nil (m ao1 l k ax v ih0 ch))
("malkowski" nil (m ax l k ao1 f s k iy0))
("mall" nil (m ao1 l))
("mallard" nil (m ae1 l er0 d))
("mallards" nil (m ae1 l er0 d z))
("malle" nil (m ae1 l))
("malleability" nil (m ae1 l iy0 ax b ih1 l ax t iy0))
("malleable" nil (m ae1 l iy0 ax b ax l))
("mallek" nil (m ae1 l ih0 k))
("mallen" nil (m ao1 l ax n))
("maller" nil (m ao1 l er0))
("mallery" nil (m ae1 l er0 iy0))
("mallet" nil (m ae1 l ih0 t))
("malleton" nil (m ae1 l ax t ax n))
("mallett" nil (m ae1 l ih0 t))
("mallette" nil (m ax l eh1 t))
("malley" nil (m ae1 l iy0))
("mallia" nil (m aa1 l iy0 ax))
("mallick" nil (m ae1 l ih0 k))
("mallicoat" nil (m ae1 l ih0 k ow0 t))
("mallie" nil (m ao1 l iy0))
("mallightco" nil (m ae1 l ay1 t k ow0))
("mallin" nil (m ae1 l ih0 n))
("mallinckrodt" nil (m ae1 l ih0 ng k r aa1 t))
("mallinger" nil (m ao1 l ih0 ng er0))
("mallinson" nil (m ae1 l ih0 n s ax n))
("mallis" nil (m ae1 l ih0 s))
("mallison" nil (m ae1 l ih0 s ax n))
("mallo" nil (m aa1 l ow0))
("malloch" nil (m ae1 l ax k))
("mallon" nil (m ae1 l ax n))
("mallonee" nil (m ae0 l ax n iy1))
("mallory" nil (m ae1 l er0 iy0))
("mallow" nil (m ae1 l ow0))
("mallows" nil (m ae1 l ow0 z))
("malloy" nil (m ax l oy1))
("mallozzi" nil (m ae1 l aa1 z iy0))
("malls" nil (m ao1 l z))
("mally" nil (m ae1 l iy0))
("malm" nil (m aa1 m))
("malmberg" nil (m aa1 l m b er0 g))
("malmgren" nil (m ae1 l m g r eh0 n))
("malmquist" nil (m ae1 l m k w ih0 s t))
("malmstrom" nil (m ae1 l m s t r ax m))
("malnar" nil (m ae1 l n er0))
("malnourished" nil (m ae0 l n er1 ih0 sh t))
("malnutrition" nil (m ae1 l n uw0 t r ih1 sh ax n))
("malo" nil (m aa1 l ow0))
("malon" nil (m ae1 l ax n))
("malone" nil (m ax l ow1 n))
("maloney" nil (m ax l ow1 n iy0))
("maloof" nil (m ax l uw1 f))
("malott" nil (m ax l aa1 t))
("malouf" nil (m ae1 l ow0 f))
("maloy" nil (m ae1 l oy0))
("malpass" nil (m ae1 l p ax s))
("malphrus" nil (m ae1 l f r ax s))
("malpractice" nil (m ae0 l p r ae1 k t ax s))
("malrite" nil (m ae1 l r ay1 t))
("malsom" nil (m ae1 l s ax m))
("malson" nil (m ae1 l s ax n))
("malstrom" nil (m ae1 l s t r ax m))
("malt" nil (m ao1 l t))
("malta" nil (m ao1 l t ax))
("maltais" nil (m ax l t ey1))
("maltase" nil (m ao1 l t ey1 s))
("maltbie" nil (m ae1 l t b iy0))
("maltby" nil (m ae1 l t b iy0))
("malted" nil (m ao1 l t ax d))
("malter" nil (m ao1 l t er0))
("maltese" nil (m ao0 l t iy1 z))
("malting" nil (m ao1 l t ih0 ng))
("maltose" nil (m ao1 l t ow0 s))
("maltreated" nil (m ae0 l t r iy1 t ih0 d))
("maltreatment" nil (m ae0 l t r iy1 t m ax n t))
("malts" nil (m ao1 l t s))
("maltz" nil (m ae1 l t s))
("malueg" nil (m ae1 l uh0 g))
("maluken" nil (m ae1 l uw1 k ih0 n))
("malva" nil (m aa1 l v ax))
("malval" nil (m aa1 l v aa0 l))
("malveaux" nil (m ae0 l v ow1))
("malvern" nil (m ae1 l v er0 n))
("malvie" nil (m ao1 l v iy0))
("malvin" nil (m ae1 l v ih0 n))
("malvina" nil (m aa0 l v iy1 n ax))
("malvinas" nil (m ao0 l v iy1 n ax s))
("maly" nil (m ey1 l iy0))
("malzahn" nil (m ae1 l z ax n))
("mama" nil (m aa1 m ax))
("mamaroneck" nil (m ax m eh1 r ax n eh0 k))
("mamas" nil (m aa1 m ax z))
("mamba" nil (m aa1 m b ax))
("mambas" nil (m aa1 m b ax z))
("mambo" nil (m aa1 m b ow0))
("mame" nil (m ey1 m))
("mameluke" nil (m ae1 m ax l uw1 k))
("mamet" nil (m ae1 m ax t))
("mamie" nil (m ey1 m iy0))
("mamis" nil (m ae1 m ih0 s))
("mamma" nil (m aa1 m ax))
("mammal" nil (m ae1 m ax l))
("mammalian" nil (m ax m ey1 l iy0 ax n))
("mammallike" nil (m ae1 m ax l l ay1 k))
("mammals" nil (m ae1 m ax l z))
("mammary" nil (m ae1 m er0 iy0))
("mammen" nil (m ae1 m ax n))
("mammogram" nil (m ae1 m ow0 g r ae1 m))
("mammograms" nil (m ae1 m ow0 g r ae1 m z))
("mammography" nil (m ax m ow1 g r ax f iy0))
("mammoth" nil (m ae1 m ax th))
("mammoths" nil (m ae1 m ax th s))
("mamone" nil (m ax m ow1 n))
("mamula" nil (m ae1 m y uw0 l ax))
("man" nil (m ae1 n))
("mana" nil (m aa1 n ax))
("manac" nil (m ae1 n ae0 k))
("manafort" nil (m ae1 n ax f ao0 r t))
("manage" nil (m ae1 n ax jh))
("manageable" nil (m ae1 n ih0 jh ax b ax l))
("managed" nil (m ae1 n ax jh d))
("management" nil (m ae1 n ax jh m ax n t))
("managements" nil (m ae1 n ih0 jh m ax n t s))
("manager" nil (m ae1 n ax jh er0))
("managerial" nil (m ae1 n ih0 jh ih1 r iy0 ax l))
("managers" nil (m ae1 n ax jh er0 z))
("manages" nil (m ae1 n ih0 jh ih0 z))
("managing" nil (m ae1 n ih0 jh ih0 ng))
("managment" nil (m ae1 n ih0 jh m ax n t))
("managua" nil (m ax n aa1 g w ax))
("manahan" nil (m ae1 n ax hh ae0 n))
("manak" nil (m ae1 n ax k))
("manalo" nil (m aa0 n aa1 l ow0))
("manard" nil (m ae1 n er0 d))
("manas" nil (m aa1 n ax z))
("manasco" nil (m aa0 n aa1 s k ow0))
("manasion" nil (m ae1 n ax sh ax n))
("manassas" nil (m ax n aa1 s ax s))
("manatee" nil (m ae1 n ax t iy1))
("manatees" nil (m ae1 n ax t iy1 z))
("manatt" nil (m ae1 n ax t))
("manbeck" nil (m ae1 n b eh1 k))
("mance" nil (m ae1 n s))
("mancebo" nil (m aa0 n ch eh1 b ow0))
("mancera" nil (m ae0 n s eh1 r ax))
("mancha" nil (m aa1 n k ax))
("manchester" nil (m ae1 n ch eh1 s t er0))
("manchu" nil (m ae1 n ch uw0))
("manchuria" nil (m ae0 n ch uh1 r iy0 ax))
("mancil" nil (m ae1 n s ih0 l))
("mancilla" nil (m ae1 n s ih1 l ax))
("mancillas" nil (m ax n s ih1 l ax z))
("mancinelli" nil (m aa0 n ch iy0 n eh1 l iy0))
("mancini" nil (m aa0 n ch iy1 n iy0))
("mancino" nil (m aa0 n ch iy1 n ow0))
("manco" nil (m ae1 ng k ow0))
("mancusi" nil (m aa0 n k uw1 s iy0))
("mancuso" nil (m ae1 ng k y uw1 s ow0))
("manda" nil (m ae1 n d ax))
("mandalay" nil (m ae1 n d ax l ey1))
("mandalit" nil (m ae1 n d ax l ih1 t))
("mandamus" nil (m ae0 n d ey1 m ax s))
("mandarin" nil (m ae1 n d er0 ax n))
("mandarino" nil (m aa0 n d aa0 r iy1 n ow0))
("mandarins" nil (m ae1 n d er0 ax n z))
("mandate" nil (m ae1 n d ey1 t))
("mandated" nil (m ae1 n d ey1 t ih0 d))
("mandates" nil (m ae1 n d ey1 t s))
("mandating" nil (m ae1 n d ey1 t ih0 ng))
("mandato" nil (m aa0 n d aa1 t ow0))
("mandatory" nil (m ae1 n d ax t ao1 r iy0))
("mandel" nil (m ae1 n d ax l))
("mandela" nil (m ae1 n d eh1 l ax))
("mandelbaum" nil (m ae1 n d ax l b aw1 m))
("mandell" nil (m ae1 n d ax l))
("mandella" nil (m ae1 n d eh1 l ax))
("mander" nil (m ae1 n d er0))
("manders" nil (m ae1 n d er0 z))
("manderscheid" nil (m ae1 n d er0 sh ay1 d))
("manderson" nil (m ae1 n d er0 s ax n))
("mandeville" nil (m ae1 n d ax v ih1 l))
("mandible" nil (m ae1 n d ax b ax l))
("mandich" nil (m ae1 n d ih0 k))
("mandie" nil (m ae1 n d iy0))
("mandigo" nil (m aa0 n d iy1 g ow0))
("mandl" nil (m ae1 n d ax l))
("mandle" nil (m ae1 n d ax l))
("mandler" nil (m ae1 n d l er0))
("mandolin" nil (m ae1 n d ax l ih1 n))
("mandracchia" nil (m ae1 n d r ae1 k iy0 ax))
("mandrake" nil (m ae1 n d r ey1 k))
("mandrell" nil (m ae1 n d r ax l))
("mandresh" nil (m ae0 n d r eh1 sh))
("mandrill" nil (m ae1 n d r ih0 l))
("mandt" nil (m ae1 n t))
("mandujano" nil (m aa0 n d uw0 y aa1 n ow0))
("mandy" nil (m ae1 n d iy0))
("mane" nil (m ey1 n))
("maned" nil (m ey1 n d))
("manella" nil (m ax n eh1 l ax))
("maner" nil (m ey1 n er0))
("maners" nil (m ey1 n er0 z))
("manes" nil (m ey1 n z))
("maness" nil (m aa1 n ih0 s))
("manet" nil (m ae0 n ey1))
("manette" nil (m ax n eh1 t))
("maneuver" nil (m ax n uw1 v er0))
("maneuverability" nil (m ax n uw1 v er0 ax b ih1 l ih0 t iy0))
("maneuverable" nil (m ax n uw1 v er0 ax b ax l))
("maneuvered" nil (m ax n uw1 v er0 d))
("maneuvering" nil (m ax n uw1 v er0 ih0 ng))
("maneuverings" nil (m ax n uw1 v er0 ih0 ng z))
("maneuvers" nil (m ax n uw1 v er0 z))
("maneval" nil (m aa0 n ey0 v ae1 l))
("maney" nil (m ey1 n iy0))
("manford" nil (m ae1 n f er0 d))
("manfra" nil (m ae1 n f r ax))
("manfre" nil (m ae1 n f er0))
("manfred" nil (m ae1 n f r ih0 d))
("manfredi" nil (m aa0 n f r eh1 d iy0))
("manfredo" nil (m aa0 n f r ey1 d ow0))
("manfully" nil (m ae1 n f ax l iy0))
("mang" nil (m ae1 ng))
("manga" nil (m ae1 ng g ax))
("mangan" nil (m ae1 ng g ax n))
("manganaro" nil (m aa0 ng g aa0 n aa1 r ow0))
("manganello" nil (m aa0 ng g aa0 n eh1 l ow0))
("manganese" nil (m ae1 ng g ax n iy1 z))
("manganiello" nil (m aa0 ng g aa0 n iy0 eh1 l ow0))
("mangano" nil (m aa0 ng g aa1 n ow0))
("mangas" nil (m ae1 ng g ax z))
("mange" nil (m ey1 n jh))
("mangel" nil (m ey1 ng g ax l))
("mangels" nil (m ey1 ng g ax l z))
("mangement" nil (m ey1 n jh m ax n t))
("mangen" nil (m ae1 ng ax n))
("manger" nil (m ey1 n jh er0))
("manges" nil (m ey1 n jh ih0 z))
("mangham" nil (m ae1 ng g ax m))
("mangiapane" nil (m ae1 n jh iy0 ax p ey1 n))
("mangiaracina" nil (m aa1 n jh er0 aa0 ch iy0 n ax))
("mangieri" nil (m aa0 ng g ih1 r iy0))
("mangin" nil (m ae1 ng g ih0 n))
("mangine" nil (m aa0 ng g iy1 n iy0))
("mangini" nil (m aa0 ng g iy1 n iy0))
("mangino" nil (m aa0 ng g iy1 n ow0))
("mangione" nil (m aa0 n jh ow1 n iy0))
("manglapus" nil (m ae1 ng l ax p ax s))
("mangle" nil (m ae1 ng g ax l))
("mangled" nil (m ae1 ng g ax l d))
("mangling" nil (m ae1 ng g ax l ih0 ng))
("mango" nil (m ae1 ng g ow0))
("mangoes" nil (m ae1 ng g ow0 z))
("mangold" nil (m ae1 n g ow1 l d))
("mangone" nil (m aa0 ng g ow1 n iy0))
("mangope" nil (m ae0 ng g ow1 p ey1))
("mangosteen" nil (m ae1 ng g ow0 s t iy1 n))
("mangosteens" nil (m ae1 ng g ow0 s t iy1 n z))
("mangosuthu" nil (m ae1 ng g ax s ah1 th uw0))
("mangrove" nil (m ae1 n g r ow1 v))
("mangrum" nil (m ae1 ng g r ax m))
("mangual" nil (m ae1 n g ax l))
("mangum" nil (m ae1 ng g ax m))
("mangus" nil (m ae1 ng g ih0 s))
("mangy" nil (m ey1 n jh iy0))
("manhandled" nil (m ae1 n hh ae1 n d ax l d))
("manhart" nil (m ae1 n hh aa1 r t))
("manhasset" nil (m ae0 n hh ae1 s eh0 t))
("manhattan" nil (m ae0 n hh ae1 t ax n))
("manheim" nil (m ae1 n hh ay0 m))
("manhole" nil (m ae1 n hh ow1 l))
("manhood" nil (m ae1 n hh uh1 d))
("manhunt" nil (m ae1 n hh ah1 n t))
("mani" nil (m aa1 n iy0))
("mania" nil (m ey1 n iy0 ax))
("maniac" nil (m ey1 n iy0 ae1 k))
("maniacal" nil (m ax n ay1 ax k ax l))
("maniaci" nil (m aa0 n iy0 aa1 ch iy0))
("maniacs" nil (m ey1 n iy0 ae1 k s))
("maniatis" nil (m ae1 n iy0 aa1 t ih0 s))
("manic" nil (m ae1 n ih0 k))
("manicure" nil (m ae1 n ih0 k y er0))
("manicured" nil (m ae1 n ih0 k y er0 d))
("manier" nil (m eh1 n iy0 er0))
("manifest" nil (m ae1 n ax f eh1 s t))
("manifestation" nil (m ae1 n ax f eh0 s t ey1 sh ax n))
("manifestations" nil (m ae1 n ax f eh0 s t ey1 sh ax n z))
("manifested" nil (m ae1 n ax f eh1 s t ax d))
("manifestly" nil (m ae1 n ax f eh0 s t l iy0))
("manifesto" nil (m ae1 n ax f eh1 s t ow1))
("manifests" nil (m ae1 n ax f eh1 s t s))
("manifold" nil (m ae1 n ax f ow1 l d))
("manigat" nil (m ae1 n ih0 g ae0 t))
("manigault" nil (m ae1 n ih0 g ao0 l t))
("manigo" nil (m aa0 n iy1 g ow0))
("manila" nil (m ax n ih1 l ax))
("manioc" nil (m ae1 n iy0 aa1 k))
("manion" nil (m aa0 n y ao1 n))
("maniples" nil (m ae1 n ax p ax l z))
("manipulate" nil (m ax n ih1 p y ax l ey1 t))
("manipulated" nil (m ax n ih1 p y ax l ey1 t ih0 d))
("manipulates" nil (m ax n ih1 p y ax l ey1 t s))
("manipulating" nil (m ax n ih1 p y ax l ey1 t ih0 ng))
("manipulation" nil (m ax n ih1 p y ax l ey1 sh ax n))
("manipulations" nil (m ax n ih1 p y ax l ey1 sh ax n z))
("manipulative" nil (m ax n ih1 p y ax l ey1 t ih0 v))
("manipulator" nil (m ax n ih1 p y ax l ey1 t er0))
("manipulators" nil (m ax n ih1 p y ax l ey1 t er0 z))
("manis" nil (m ae1 n ih0 s))
("maniscalco" nil (m aa0 n iy0 s k aa1 l k ow0))
("manischewitz" nil (m ae1 n ih0 sh eh1 v ih0 t s))
("manish" nil (m ae1 n ih0 sh))
("manitoba" nil (m ae1 n ih0 t ow1 b ax))
("manitowoc" nil (m ae1 n ih0 t ax w aa0 k))
("manjarrez" nil (m aa0 n y aa1 r eh0 z))
("mank" nil (m ae1 ng k))
("manka" nil (m ae1 ng k ax))
("manke" nil (m ae1 ng k))
("manker" nil (m ae1 ng k er0))
("mankey" nil (m ae1 n k iy1))
("mankiewicz" nil (m ae1 ng k ih0 w ih0 t s))
("mankin" nil (m ae1 ng k ih0 n))
("mankind" nil (m ae1 n k ay1 n d))
("mankinds" nil (m ae1 n k ay1 n d z))
("mankins" nil (m ae1 ng k ih0 n z))
("manko" nil (m ae1 ng k ow0))
("mankowski" nil (m ax ng k ao1 f s k iy0))
("manley" nil (m ae1 n l iy0))
("manlove" nil (m ae1 n l ah1 v))
("manly" nil (m ae1 n l iy0))
("manmade" nil (m ae1 n m ey1 d))
("mann" nil (m ae1 n))
("manna" nil (m ae1 n ax))
("mannarino" nil (m ae1 n er0 iy0 n ow0))
("manne" nil (m ae1 n))
("manned" nil (m ae1 n d))
("mannella" nil (m aa0 n eh1 l ax))
("mannen" nil (m ae1 n ax n))
("mannequin" nil (m ae1 n ax k ih0 n))
("mannequins" nil (m ae1 n ax k ih0 n z))
("manner" nil (m ae1 n er0))
("mannered" nil (m ae1 n er0 d))
("mannering" nil (m ae1 n er0 ih0 ng))
("mannerism" nil (m ae1 n er0 ih1 z ax m))
("mannerisms" nil (m ae1 n er0 ih1 z ax m z))
("mannerist" nil (m ae1 n er0 ax s t))
("manners" nil (m ae1 n er0 z))
("mannes" nil (m ae1 n z))
("mannesmann" nil (m ae1 n ax s m ax n))
("manney" nil (m ae1 n iy0))
("mannheim" nil (m ae1 n hh ay0 m))
("manni" nil (m ae1 n iy0))
("mannie" nil (m ae1 n iy0))
("mannina" nil (m ae1 n ih0 n ax))
("manninen" nil (m ae1 n ih0 n ax n))
("manning" nil (m ae1 n ih0 ng))
("mannino" nil (m ae1 n iy0 n ow0))
("mannion" nil (m ae1 n y ax n))
("mannis" nil (m ae1 n ih0 s))
("mannix" nil (m ae1 n ih0 k s))
("manno" nil (m ae1 n ow0))
("mannon" nil (m ae1 n ax n))
("manns" nil (m ae1 n z))
("manny" nil (m ae1 n iy0))
("mano" nil (m aa1 n ow0))
("manocchio" nil (m aa0 n ow1 k iy0 ow0))
("manolis" nil (m ae1 n ax l ih0 s))
("manon" nil (m aa0 n ao1 n))
("manoogian" nil (m ax n uw1 jh iy0 ax n))
("manor" nil (m ae1 n er0))
("manors" nil (m ae1 n er0 z))
("manos" nil (m ey1 n ow0 z))
("manpower" nil (m ae1 n p aw1 er0))
("manring" nil (m ae1 n r ih1 ng))
("manrique" nil (m ax n r iy1 k))
("manriquez" nil (m aa0 n r iy1 k w eh0 z))
("manry" nil (m ae1 n r iy0))
("mans" nil (m ae1 n z))
("manseau" nil (m ax n s ow1))
("mansel" nil (m ae1 n s ax l))
("mansell" nil (m ae1 n s ax l))
("manser" nil (m ae1 n s er0))
("mansfield" nil (m ae1 n z f iy1 l d))
("manship" nil (m ae1 n sh ih1 p))
("mansion" nil (m ae1 n sh ax n))
("mansions" nil (m ae1 n ch ax n z))
("manske" nil (m ae1 n s k))
("mansker" nil (m ae1 n s k er0))
("manslaughter" nil (m ae1 n s l ao1 t er0))
("manso" nil (m ae1 n s ow0))
("manson" nil (m ae1 n s ax n))
("mansour" nil (m ae1 n s er0))
("mansueto" nil (m ae0 n s w ey1 t ow0))
("mansur" nil (m ae1 n s er0))
("manta" nil (m ae1 n t ax))
("mantei" nil (m ae1 n t ay0))
("mantel" nil (m ae1 n t ax l))
("mantell" nil (m ae0 n t eh1 l))
("manter" nil (m ae1 n t er0))
("manternach" nil (m ae1 n t er0 n ax k))
("manteufel" nil (m ae1 n t oy0 f ax l))
("mantey" nil (m ae1 n t iy0))
("manthe" nil (m ae1 n dh))
("manthei" nil (m ae1 n dh ay0))
("manthey" nil (m ae1 n th iy0))
("mantia" nil (m aa1 n sh ax))
("mantilla" nil (m ae0 n t ih1 l ax))
("mantione" nil (m aa0 n t iy0 ow1 n iy0))
("mantle" nil (m ae1 n t ax l))
("mantles" nil (m ae1 n t ax l z))
("manton" nil (m ae1 n t ax n))
("mantooth" nil (m ae1 n t uw1 th))
("mantra" nil (m ae1 n t r ax))
("mantua" nil (m ae1 n ch uw0 ax))
("mantz" nil (m ae1 n t s))
("manual" nil (m ae1 n y uw0 ax l))
("manually" nil (m ae1 n y uw0 ax l iy0))
("manuals" nil (m ae1 n y uw0 ax l z))
("manucher" nil (m ae1 n uw0 k er0))
("manuel" nil (m aa0 n w eh1 l))
("manuela" nil (m ae0 n w ey1 l ax))
("manuele" nil (m ae1 n uh0 l))
("manuevering" nil (m ax n uw1 v er0 ih0 ng))
("manufacture" nil (m ae1 n y ax f ae1 k ch er0))
("manufactured" nil (m ae1 n y ax f ae1 k ch er0 d))
("manufacturer" nil (m ae1 n y ax f ae1 k ch er0 er0))
("manufacturers" nil (m ae1 n y ax f ae1 k ch er0 er0 z))
("manufactures" nil (m ae1 n y ax f ae1 k ch er0 z))
("manufacturing" nil (m ae1 n y ax f ae1 k ch er0 ih0 ng))
("manure" nil (m ax n uh1 r))
("manus" nil (m ey1 n ih0 s))
("manuscript" nil (m ae1 n y ax s k r ih1 p t))
("manuscripts" nil (m ae1 n y ax s k r ih1 p t s))
("manvel" nil (m ae1 n v ax l))
("manvil" nil (m ae1 n v ih0 l))
("manville" nil (m ae1 n v ih0 l))
("manwaring" nil (m ae1 n w eh1 r ih0 ng))
("manwarren" nil (m ax n w ao1 r ax n))
("manweb" nil (m ae1 n w eh1 b))
("manwell" nil (m ae1 n w eh1 l))
("manwiller" nil (m ae1 n w ih1 l er0))
("manx" nil (m ae1 ng k s))
("many" nil (m eh1 n iy0))
("manz" nil (m ae1 n z))
("manzanares" nil (m aa0 n z aa0 n aa1 r eh0 s))
("manzanilla" nil (m ae1 n z ax n ih1 l ax))
("manzano" nil (m aa0 n z aa1 n ow0))
("manzella" nil (m ae1 n z eh1 l ax))
("manzer" nil (m ae1 n z er0))
("manzi" nil (m ae1 n z iy0))
("manzione" nil (m aa0 n z iy0 ow1 n iy0))
("manzo" nil (m ae1 n z ow0))
("mao" nil (m aw1))
("maoist" nil (m aw1 ih0 s t))
("maoists" nil (m aw1 ih0 s t s))
("maori" nil (m aw1 r iy0))
("maoris" nil (m ey1 er0 ih0 s))
("map" nil (m ae1 p))
("mapco" nil (m ae1 p k ow0))
("mapel" nil (m ae1 p ax l))
("mapes" nil (m ey1 p s))
("mapi" nil (m ae1 p iy0))
("maple" nil (m ey1 p ax l))
("maples" nil (m ey1 p ax l z))
("maplewood" nil (m ey1 p ax l w uh1 d))
("mapp" nil (m ae1 p))
("mapped" nil (m ae1 p t))
("mapping" nil (m ae1 p ih0 ng))
("mapplethorpe" nil (m ae1 p l ax th ao0 r p))
("maps" nil (m ae1 p s))
("maputo" nil (m ax p uw1 t ow0))
("maquila" nil (m ax k w iy1 l ax))
("maquiladora" nil (m ae1 k w ih0 l ae1 d er0 ax))
("maquiladoras" nil (m ae0 k iy1 y ax d ao1 r ax s))
("maquilas" nil (m ae1 k w ax l ax s))
("mar" nil (m aa1 r))
("mara" nil (m aa1 r ax))
("marabella" nil (m ae1 r ax b eh1 l ax))
("marable" nil (m eh1 r ax b ax l))
("marabou" nil (m eh1 r ax b uw1))
("maracle" nil (m aa1 r ax k ax l))
("maradona" nil (m aa1 r ax d ow1 n ah1))
("maradonna" nil (m aa1 r ax d ow1 n ah1))
("marafat" nil (m eh1 r ax f ae0 t))
("marais" nil (m eh1 r ey1))
("marak" nil (m ae1 r ax k))
("maralina" nil (m aa0 r aa0 l iy1 n ax))
("maraline" nil (m aa0 r aa0 l iy1 n iy0))
("maran" nil (m aa0 r aa1 n))
("maranda" nil (m er0 ae1 n d ax))
("marando" nil (m er0 ae1 n d ow0))
("marano" nil (m aa0 r aa1 n ow0))
("maranon" nil (m eh1 r ax n aa0 n))
("marantette" nil (m eh1 r ax n t eh1 t))
("maranto" nil (m er0 ae1 n t ow0))
("marantz" nil (m ae1 r ax n t s))
("maranville" nil (m aa0 r aa1 n v ih0 l))
("maras" nil (m aa1 r ax z))
("marasco" nil (m aa0 r aa1 s k ow0))
("marasek" nil (m er0 aa1 s eh0 k))
("marash" nil (m aa1 r ax sh))
("marathon" nil (m eh1 r ax th aa1 n))
("marathons" nil (m eh1 r ax th aa1 n z))
("marauders" nil (m er0 ao1 d er0 z))
("maravilla" nil (m aa0 r aa0 v ih1 l ax))
("marbach" nil (m aa1 r b aa1 k))
("marbella" nil (m aa1 r b eh1 l ax))
("marberry" nil (m aa1 r b eh1 r iy0))
("marbil" nil (m aa1 r b ih0 l))
("marble" nil (m aa1 r b ax l))
("marbled" nil (m aa1 r b ax l d))
("marblehead" nil (m aa1 r b ax l hh eh1 d))
("marbles" nil (m aa1 r b ax l z))
("marbod" nil (m aa1 r b aa0 d))
("marbry" nil (m aa1 r b r iy0))
("marburger" nil (m aa1 r b er0 g er0))
("marbury" nil (m aa1 r b eh1 r iy0))
("marbut" nil (m aa1 r b ax t))
("marc" nil (m aa1 r k))
("marcade" nil (m aa1 r k ey1 d))
("marcano" nil (m aa0 r k aa1 n ow0))
("marcantel" nil (m aa0 r k aa0 n t eh1 l))
("marcantonio" nil (m aa1 r k ax n t ow1 n iy0 ow0))
("marceau" nil (m aa0 r s ow1))
("marceaux" nil (m aa0 r s ow1))
("marcel" nil (m aa0 r s eh1 l))
("marcela" nil (m aa0 r ch eh1 l ax))
("marcelia" nil (m aa0 r ch eh1 l iy0 ax))
("marcelino" nil (m aa0 r ch eh0 l iy1 n ow0))
("marcell" nil (m aa0 r s ey1 l))
("marcella" nil (m aa0 r s eh1 l ax))
("marcelle" nil (m aa0 r s eh1 l))
("marcelli" nil (m aa0 r ch eh1 l iy0))
("marcellina" nil (m aa0 r ch eh0 l iy1 n ax))
("marcelline" nil (m aa0 r ch eh0 l iy1 n iy0))
("marcellino" nil (m aa0 r ch eh0 l iy1 n ow0))
("marcello" nil (m aa1 r s eh1 l ow0))
("marcellus" nil (m aa0 r s eh1 l ax s))
("marcelo" nil (m aa0 r ch eh1 l ow0))
("march" nil (m aa1 r ch))
("marchak" nil (m aa1 r ch ax k))
("marchal" nil (m aa1 r ch ax l))
("marchand" nil (m aa0 r k hh ae1 n d))
("marchant" nil (m aa1 r ch ax n t))
("marchbanks" nil (m aa1 r ch b ae1 ng k s))
("marche" nil (m aa1 r sh))
("marched" nil (m aa1 r ch t))
("marchers" nil (m aa1 r ch er0 z))
("marches" nil (m aa1 r ch ih0 z))
("marchesani" nil (m aa0 r k eh0 s aa1 n iy0))
("marchesano" nil (m aa0 r k eh0 s aa1 n ow0))
("marcheschi" nil (m aa0 r ch eh1 sh iy0))
("marchese" nil (m aa0 r k iy1 z iy0))
("marchesi" nil (m aa0 r k eh1 s iy0))
("marchessault" nil (m aa1 r sh ih0 s ow0))
("marchetta" nil (m aa0 r k eh1 t ax))
("marchetti" nil (m aa0 r k eh1 t iy0))
("marchewka" nil (m er0 ch y uw1 k ax))
("marchi" nil (m aa1 r k iy0))
("marching" nil (m aa1 r ch ih0 ng))
("marchini" nil (m aa0 r k iy1 n iy0))
("marchio" nil (m aa1 r k iy0 ow0))
("marchione" nil (m aa0 r k iy0 ow1 n iy0))
("marchita" nil (m aa0 r k iy1 t ax))
("marchitto" nil (m aa0 r k iy1 t ow0))
("marchman" nil (m aa1 r k m ax n))
("marcia" nil (m aa1 r sh ax))
("marcial" nil (m aa0 r s iy0 aa1 l))
("marciano" nil (m aa0 r ch iy0 aa1 n ow0))
("marciante" nil (m aa1 r ch ax n t iy0))
("marcie" nil (m aa1 r k iy0))
("marcil" nil (m aa1 r s ih0 l))
("marcile" nil (m aa1 r ch ax l))
("marcilio" nil (m aa0 r s iy1 l iy0 ow0))
("marcille" nil (m aa1 r s ih0 l))
("marcin" nil (m aa0 r s iy1 n))
("marcinek" nil (m er0 ch ih1 n eh0 k))
("marciniak" nil (m er0 ch ih1 n iy0 ae0 k))
("marcinko" nil (m aa1 r s ih1 ng k ow0))
("marcinkowski" nil (m er0 ch ih0 ng k ao1 f s k iy0))
("marcinkus" nil (m aa1 r s ih1 ng k ax s))
("marckesano" nil (m aa1 r k eh1 s aa1 n ow0))
("marcks" nil (m aa1 r k s))
("marcmann" nil (m aa1 r k m ax n))
("marco" nil (m aa1 r k ow0))
("marcoe" nil (m aa1 r k ow0))
("marcom" nil (m aa1 r k ax m))
("marcon" nil (m aa1 r k ax n))
("marcone" nil (m aa0 r k ow1 n iy0))
("marconi" nil (m aa0 r k ow1 n iy0))
("marcor" nil (m aa1 r k ao1 r))
("marcos" nil (m aa1 r k ow0 s))
("marcoses" nil (m aa0 r k ow1 s ih0 z))
("marcott" nil (m aa0 r k aa1 t))
("marcotte" nil (m aa0 r k ao1 t))
("marcou" nil (m aa0 r k uw1))
("marcoux" nil (m aa0 r k uw1))
("marcrum" nil (m aa1 r k r ax m))
("marcucci" nil (m aa0 r k uw1 ch iy0))
("marcum" nil (m aa1 r k ax m))
("marcus" nil (m aa1 r k ax s))
("marcussen" nil (m aa1 r k ax s ax n))
("marcy" nil (m aa1 r s iy0))
("marczak" nil (m aa1 r ch ae0 k))
("marden" nil (m aa1 r d ax n))
("marder" nil (m aa1 r d er0))
("mardi" nil (m aa1 r t iy0))
("mardis" nil (m aa1 r d ih0 s))
("mardon" nil (m aa1 r d ax n))
("mare" nil (m eh1 r))
("maready" nil (m er0 eh1 d iy0))
("marean" nil (m aa0 r iy1 n))
("marek" nil (m aa1 r eh0 k))
("marelda" nil (m aa0 r eh1 l d ax))
("marella" nil (m er0 eh1 l ax))
("marelli" nil (m eh0 r eh1 l iy0))
("mareno" nil (m aa0 r eh1 n ow0))
("mares" nil (m eh1 r z))
("maresca" nil (m aa0 r eh1 s k ax))
("maresco" nil (m aa0 r eh1 s k ow0))
("maresh" nil (m ae1 r ih0 sh))
("maret" nil (m aa1 r ey0 t))
("marett" nil (m ae1 r ih0 t))
("maretta" nil (m aa0 r eh1 t ax))
("marette" nil (m er0 eh1 t))
("marez" nil (m aa0 r eh1 z))
("marfa" nil (m aa1 r f ax))
("marga" nil (m aa1 r g ax))
("margalo" nil (m aa0 r g aa1 l ow0))
("margaret" nil (m aa1 r g er0 ih0 t))
("margareta" nil (m aa0 r g aa0 r eh1 t ax))
("margarete" nil (m aa1 r g er0 iy1 t))
("margarethe" nil (m aa0 r g aa0 r eh1 dh iy0))
("margaretten" nil (m aa1 r g er0 eh1 t ax n))
("margarine" nil (m aa1 r jh er0 ax n))
("margarita" nil (m aa1 r g er0 iy1 t ax))
("margaritas" nil (m aa1 r g eh0 r iy1 t ax s))
("margaux" nil (m aa0 r g ow1))
("marge" nil (m aa1 r jh))
("margeotes" nil (m aa1 r jh iy0 ow1 t iy0 z))
("margerum" nil (m aa1 r g er0 ax m))
("margery" nil (m aa1 r jh er0 iy0))
("margeson" nil (m aa1 r g ih0 s ax n))
("marget" nil (m aa1 r g ih0 t))
("margette" nil (m aa0 r zh eh1 t))
("margie" nil (m aa1 r jh iy0))
("margin" nil (m aa1 r jh ax n))
("marginal" nil (m aa1 r jh ax n ax l))
("marginalize" nil (m aa1 r jh ax n ax l ay1 z))
("marginalized" nil (m aa1 r jh ax n ax l ay1 z d))
("marginalizes" nil (m aa1 r jh ax n ax l ay1 z ih0 z))
("marginalizing" nil (m aa1 r jh ax n ax l ay1 z ih0 ng))
("marginally" nil (m aa1 r jh ax n ax l iy0))
("margined" nil (m aa1 r jh ax n d))
("margining" nil (m aa1 r jh ax n ih0 ng))
("margins" nil (m aa1 r jh ax n z))
("margiotta" nil (m aa0 r jh ow1 t ax))
("margittai" nil (m aa1 r jh ih0 t ay1))
("margo" nil (m aa1 r g ow0))
("margolies" nil (m aa0 r g ow1 l iy0 z))
("margolin" nil (m aa1 r g ax l ih0 n))
("margolis" nil (m aa0 r g ow1 l ih0 s))
("margory" nil (m er0 g er1 iy0))
("margoshes" nil (m aa0 r g ao1 sh ih0 z))
("margot" nil (m aa1 r g ow0))
("margraf" nil (m aa1 r g r ax f))
("margrave" nil (m aa1 r g r ey1 v))
("margret" nil (m aa1 g r eh0 t))
("marguerite" nil (m aa1 r g er0 iy1 t))
("margulies" nil (m aa1 r g uw0 l iy0 z))
("margulis" nil (m aa1 r g ax l ih0 s))
("margy" nil (m aa1 r jh iy0))
("marhefka" nil (m er0 hh eh1 f k ax))
("mari" nil (m aa1 r iy0))
("maria" nil (m er0 iy1 ax))
("mariachi" nil (m aa1 r iy0 aa1 ch iy0))
("mariah" nil (m er0 ay1 ax))
("mariam" nil (m eh1 r iy0 ae0 m))
("marian" nil (m eh1 r iy0 ax n))
("mariana" nil (m eh1 r iy0 ae1 n ax))
("marianas" nil (m eh1 r iy0 ae1 n ax s))
("mariani" nil (m aa0 r iy0 aa1 n iy0))
("marianna" nil (m aa0 r iy0 aa1 n ax))
("marianne" nil (m eh1 r iy0 ae1 n))
("mariano" nil (m eh1 r iy0 aa1 n ow0))
("maribel" nil (m ae1 r ih0 b eh0 l))
("marice" nil (m aa1 r ih0 s))
("marich" nil (m ae1 r ih0 k))
("maricle" nil (m ae1 r ih0 k ax l))
("maricopa" nil (m eh1 r ih0 k ow1 p ax))
("marie" nil (m er0 iy1))
("mariel" nil (m eh1 r iy0 ax l))
("marielitos" nil (m eh0 r iy1 eh0 l iy1 t ow0 s))
("marien" nil (m eh1 r iy0 n))
("marienbonn" nil (m eh1 r iy0 n b ao1 n))
("marier" nil (m eh1 r iy0 er0))
("marietta" nil (m eh1 r iy0 eh1 t ax))
("mariette" nil (m eh1 r iy0 eh1 t))
("marigold" nil (m eh1 r ax g ow1 l d))
("marigolds" nil (m eh1 r ax g ow1 l d z))
("marijuana" nil (m eh1 r ax w aa1 n ax))
("marik" nil (m ae1 r ih0 k))
("marilda" nil (m aa0 r iy1 l d ax))
("marilin" nil (m ae1 r ih0 l ih0 n))
("marilla" nil (m aa0 r ih1 l ax))
("marilyn" nil (m eh1 r ax l ax n))
("marilynn" nil (m eh1 r ax l ax n))
("marimba" nil (m er0 ih1 m b ax))
("marin" nil (m eh1 r ih0 n))
("marina" nil (m er0 iy1 n ax))
("marinaccio" nil (m aa0 r ih0 n aa1 ch iy0 ow0))
("marinaro" nil (m aa0 r ih0 n aa1 r ow0))
("marinas" nil (m er0 iy1 n ax z))
("marinated" nil (m eh1 r ax n ey1 t ih0 d))
("marine" nil (m er0 iy1 n))
("marineau" nil (m eh1 r ih0 n ow0))
("marineland" nil (m er0 iy1 n l ax n d))
("marinelli" nil (m aa0 r ih0 n eh1 l iy0))
("marinello" nil (m aa0 r ih0 n eh1 l ow0))
("mariner" nil (m eh1 r ax n er0))
("mariners" nil (m eh1 r ax n er0 z))
("marines" nil (m er0 iy1 n z))
("marinez" nil (m aa0 r iy1 n eh0 z))
("maring" nil (m eh1 r ih0 ng))
("marini" nil (m aa0 r iy1 n iy0))
("marinkovich" nil (m er0 ih1 ng k ax v ih0 ch))
("marino" nil (m er0 iy1 n ow0))
("marinoff" nil (m ae1 r ih0 n ao0 f))
("marinos" nil (m er0 iy1 n ow0 z))
("marinucci" nil (m aa0 r iy0 n uw1 ch iy0))
("marinvest" nil (m er0 ih1 n v ax s t))
("mario" nil (m aa1 r iy0 ow0))
("marion" nil (m eh1 r iy0 ax n))
("mariotta" nil (m eh1 r iy0 aa1 t ax))
("mariotti" nil (m aa0 r iy0 ow1 t iy0))
("maris" nil (m eh1 r ih0 s))
("mariscal" nil (m aa1 r ax s k ax l))
("marital" nil (m eh1 r ax t ax l))
("maritime" nil (m eh1 r ax t ay1 m))
("maritz" nil (m eh1 r ih0 t s))
("mariucci" nil (m eh1 r iy0 uw1 ch iy0))
("marius" nil (m eh1 r iy0 ax s))
("marjai" nil (m aa1 r jh ey0))
("marje" nil (m aa1 r jh))
("marjie" nil (m aa1 r jh iy0))
("marjorie" nil (m aa1 r jh er0 iy0))
("marjory" nil (m aa1 r jh er0 iy0))
("marjy" nil (m aa1 r jh iy0))
("mark" nil (m aa1 r k))
("markair" nil (m aa1 r k eh1 r))
("markarian" nil (m aa1 r k eh1 r iy0 ax n))
("markdown" nil (m aa1 r k d aw1 n))
("markdowns" nil (m aa1 r k d aw1 n z))
("marke" nil (m aa1 r k))
("marked" nil (m aa1 r k t))
("markedly" nil (m aa1 r k ax d l iy0))
("markee" nil (m aa1 r k iy1))
("markel" nil (m aa1 r k ax l))
("markell" nil (m aa1 r k ax l))
("marken" nil (m aa1 r k ax n))
("marker" nil (m aa1 r k er0))
("markers" nil (m aa1 r k er0 z))
("markert" nil (m aa1 r k er0 t))
("markese" nil (m aa0 r k iy1 s))
("market" nil (m aa1 r k ax t))
("marketability" nil (m aa1 r k ih0 t ax b ih1 l ih0 t iy0))
("marketable" nil (m aa1 r k ax t ax b ax l))
("marketamerica" nil (m aa1 r k ax t ax m eh1 r ih0 k ax))
("marketed" nil (m aa1 r k ax t ax d))
("marketeer" nil (m aa1 r k ax t ih1 r))
("marketeers" nil (m aa1 r k ax t ih1 r z))
("marketer" nil (m aa1 r k ax t er0))
("marketers" nil (m aa1 r k ax t er0 s))
("marketing" nil (m aa1 r k ax t ih0 ng))
("marketings" nil (m aa1 r k ax t ih0 ng z))
("marketmakers" nil (m aa1 r k ax t m ey1 k er0 z))
("marketplace" nil (m aa1 r k ax t p l ey1 s))
("marketplaces" nil (m aa1 r k ax t p l ey1 s ih0 z))
("markets" nil (m aa1 r k ax t s))
("marketscope" nil (m aa1 r k ax t s k ow1 p))
("marketwide" nil (m aa1 r k ax t w ay1 d))
("markey" nil (m aa1 r k iy1))
("markgraf" nil (m aa1 r k g r ax f))
("markham" nil (m aa1 r k ax m))
("markiewicz" nil (m aa1 r k ax v ih0 ch))
("markin" nil (m aa1 r k ih0 n))
("marking" nil (m aa1 r k ih0 ng))
("markings" nil (m aa1 r k ih0 ng z))
("markka" nil (m aa1 r k ax))
("markkaa" nil (m aa0 r k aa1))
("markkas" nil (m aa1 r k ax s))
("markland" nil (m aa1 r k l ax n d))
("markle" nil (m aa1 r k ax l))
("markley" nil (m aa1 r k l iy0))
("markman" nil (m aa1 r k m ax n))
("marko" nil (m aa1 r k ow0))
("markoff" nil (m aa1 r k ao1 f))
("markos" nil (m aa1 r k ow0 z))
("markovic" nil (m aa1 r k ax v ih0 k))
("markovich" nil (m aa1 r k ax v ih0 ch))
("markovitz" nil (m aa1 r k ax v ih0 t s))
("markow" nil (m aa1 r k ow0))
("markowicz" nil (m aa1 r k ax v ih0 ch))
("markowitz" nil (m aa1 r k ax w ih0 t s))
("markowski" nil (m er0 k ao1 f s k iy0))
("marks" nil (m aa1 r k s))
("marksberry" nil (m aa1 r k s b eh1 r iy0))
("marksmanship" nil (m aa1 r k s m ax n sh ih0 p))
("markson" nil (m aa1 r k s ax n))
("markum" nil (m aa1 r k ax m))
("markunas" nil (m aa1 r k uw0 n ax z))
("markup" nil (m aa1 r k ah1 p))
("markups" nil (m aa1 r k ah1 p s))
("markus" nil (m aa1 r k ax s))
("markuson" nil (m aa1 r k ax s ax n))
("markwardt" nil (m aa1 r k w ao0 r t))
("markwell" nil (m aa1 r k w eh1 l))
("markwood" nil (m aa1 r k w uh1 d))
("marla" nil (m aa1 r l ax))
("marland" nil (m aa1 r l ax n d))
("marlar" nil (m aa1 r l er0))
("marlatt" nil (m aa1 r l ax t))
("marlboro" nil (m aa1 r l b er0 ow0))
("marlboros" nil (m aa1 r l b er0 r ow0 z))
("marlborough" nil (m aa1 r l b er0 ow0))
("marleen" nil (m aa0 r l iy1 n))
("marleene" nil (m aa0 r l iy1 n))
("marlen" nil (m aa1 r l ax n))
("marlena" nil (m aa0 r l ey1 n ax))
("marlene" nil (m aa1 r l iy1 n))
("marlens" nil (m aa1 r l ax n z))
("marler" nil (m aa1 r l er0))
("marlett" nil (m aa1 r l ih0 t))
("marlette" nil (m aa1 r l eh1 t))
("marley" nil (m aa1 r l iy0))
("marliave" nil (m aa1 r l iy0 eh1 v))
("marlin" nil (m aa1 r l ax n))
("marling" nil (m aa1 r l ih0 ng))
("marlins" nil (m aa1 r l ax n z))
("marlo" nil (m aa1 r l ow0))
("marlon" nil (m aa1 r l ax n))
("marlow" nil (m aa1 r l ow0))
("marlowe" nil (m aa1 r l ow1))
("marlton" nil (m aa1 r l t ax n))
("marmaduke" nil (m aa1 r m ax d uw1 k))
("marmalade" nil (m aa1 r m ax l ey1 d))
("marmara" nil (m aa0 r m aa1 r ax))
("marmer" nil (m aa1 r m er0))
("marmion" nil (m aa1 r m iy0 ax n))
("marmo" nil (m aa1 r m ow0))
("marmolejo" nil (m aa0 r m ow0 l ey1 y ow0))
("marmon" nil (m aa1 r m ax n))
("marmor" nil (m aa1 r m er0))
("marmoset" nil (m aa1 r m ax z eh1 t))
("marnell" nil (m aa1 r n ax l))
("marner" nil (m aa1 r n er0))
("marney" nil (m aa1 r n iy0))
("maro" nil (m aa1 r ow0))
("marogen" nil (m eh1 r ow0 g ax n))
("marohl" nil (m ae1 r ow0 l))
("marohn" nil (m aa1 r ax n))
("marois" nil (m er0 w aa1))
("marold" nil (m ae1 r ow0 l d))
("marolf" nil (m ae1 r ow0 l f))
("marolt" nil (m ae1 r ow0 l t))
("maron" nil (m aa0 r ao1 n))
("marone" nil (m er0 ow1 n))
("maroney" nil (m er0 ow1 n iy0))
("maroni" nil (m aa0 r ow1 n iy0))
("maronite" nil (m eh1 r ax n ay1 t))
("maroon" nil (m er0 uw1 n))
("marooned" nil (m er0 uw1 n d))
("marose" nil (m er0 ow1 s))
("marotta" nil (m er0 aa1 t ax))
("marotte" nil (m er0 ao1 t))
("marotto" nil (m aa0 r ow1 t ow0))
("marotz" nil (m ae1 r ax t s))
("marous" nil (m eh1 r ax s))
("marovich" nil (m aa1 r ax v ih0 ch))
("marplan" nil (m aa1 r p l ae1 n))
("marple" nil (m aa1 r p ax l))
("marquand" nil (m aa1 r k w ax n d))
("marquard" nil (m aa0 r k w aa1 r d))
("marquardt" nil (m aa1 r k w aa0 r t))
("marquart" nil (m aa1 r k w aa0 r t))
("marque" nil (m aa1 r k))
("marquee" nil (m aa0 r k iy1))
("marques" nil (m aa1 r k s))
("marquess" nil (m aa1 r k w ax s))
("marquest" nil (m aa1 r k w eh1 s t))
("marquette" nil (m aa0 r k eh1 t))
("marquez" nil (m aa0 r k eh1 z))
("marquis" nil (m aa1 r k iy1))
("marquiss" nil (m aa1 r k w iy0 s))
("marquita" nil (m aa0 r k w iy1 t ax))
("marr" nil (m aa1 r))
("marra" nil (m aa1 r ax))
("marrazzo" nil (m aa1 r aa1 z ow0))
("marred" nil (m aa1 r d))
("marren" nil (m aa1 r ax n))
("marrero" nil (m aa1 r eh1 r ow0))
("marrett" nil (m eh1 r ih0 t))
("marriage" nil (m eh1 r ih0 jh))
("marriages" nil (m eh1 r ih0 jh ih0 z))
("married" nil (m eh1 r iy0 d))
("marries" nil (m eh1 r iy0 z))
("marrin" nil (m ae1 r ih0 n))
("marriner" nil (m eh1 r ih0 n er0))
("marring" nil (m aa1 r ih0 ng))
("marriott" nil (m eh1 r iy0 ax t))
("marris" nil (m ae1 r ih0 s))
("marro" nil (m aa1 r ow0))
("marrocco" nil (m aa0 r ow1 k ow0))
("marron" nil (m eh1 r ax n))
("marrone" nil (m aa0 r ow1 n iy0))
("marroquin" nil (m aa0 r ow0 k w iy1 n))
("marrow" nil (m eh1 r ow0))
("marrs" nil (m aa1 r z))
("marrufo" nil (m aa1 r uw1 f ow0))
("marrujo" nil (m aa0 r uw1 y ow0))
("marry" nil (m eh1 r iy0))
("marrying" nil (m eh1 r iy0 ih0 ng))
("mars" nil (m aa1 r z))
("marsal" nil (m aa1 r s ax l))
("marsala" nil (m aa0 r s aa1 l ax))
("marsalis" nil (m aa1 r s ax l ih0 s))
("marsam" nil (m aa1 r s ae0 m))
("marsch" nil (m aa1 r sh))
("marschalk" nil (m aa1 r sh ao1 k))
("marschall" nil (m aa1 r sh ax l))
("marschke" nil (m aa1 r sh k))
("marschner" nil (m aa1 r sh n er0))
("marsden" nil (m aa1 r s d ax n))
("marsdon" nil (m aa1 r s d ax n))
("marse" nil (m aa1 r s))
("marsee" nil (m aa1 r s iy1))
("marseille" nil (m aa0 r s ay1 l))
("marseilles" nil (m aa0 r s ay1 l ax s))
("marsella" nil (m aa1 r s eh1 l ax))
("marsh" nil (m aa1 r sh))
("marsha" nil (m aa1 r sh ax))
("marshak" nil (m aa1 r sh ax k))
("marshal" nil (m aa1 r sh ax l))
("marshaled" nil (m aa1 r sh ax l d))
("marshaling" nil (m aa1 r sh ax l ih0 ng))
("marshall" nil (m aa1 r sh ax l))
("marshalled" nil (m aa1 r sh ax l d))
("marshalls" nil (m aa1 r sh ao1 l z))
("marshalltown" nil (m aa1 r sh ax l t aw1 n))
("marshals" nil (m aa1 r sh ax l z))
("marshburn" nil (m aa1 r sh b er1 n))
("marshes" nil (m aa1 r sh ax z))
("marshfield" nil (m aa1 r sh f iy1 l d))
("marshmallow" nil (m aa1 r sh m eh1 l ow0))
("marshmallows" nil (m aa1 r sh m eh1 l ow0 z))
("marshman" nil (m aa1 r sh m ax n))
("marshy" nil (m aa1 r sh iy0))
("marsicano" nil (m aa0 r s iy0 k aa1 n ow0))
("marsico" nil (m aa0 r s iy1 k ow0))
("marsiglia" nil (m aa1 r s ih1 g l iy0 ax))
("marsili" nil (m aa0 r s iy1 l iy0))
("marsland" nil (m aa1 r s l ax n d))
("marso" nil (m aa1 r s ow0))
("marsolek" nil (m aa1 r s ax l eh0 k))
("marson" nil (m aa1 r s ax n))
("marsteller" nil (m aa1 r z t eh1 l er0))
("marsters" nil (m aa1 r s t er0 z))
("marston" nil (m aa1 r s t ax n))
("marsupial" nil (m aa0 r s uw1 p iy0 ax l))
("marszalek" nil (m er0 sh aa1 l eh0 k))
("mart" nil (m aa1 r t))
("marta" nil (m aa1 r t ax))
("marte" nil (m aa1 r t))
("martech" nil (m aa1 r t eh1 k))
("martel" nil (m aa1 r t eh1 l))
("martell" nil (m aa1 r t eh1 l))
("martella" nil (m aa1 r t eh1 l ax))
("martelle" nil (m aa0 r t eh1 l))
("martelli" nil (m aa0 r t eh1 l iy0))
("martellini" nil (m aa1 r t eh1 l iy1 n iy0))
("martello" nil (m aa0 r t eh1 l ow0))
("marten" nil (m aa1 r t ax n))
("marteney" nil (m aa1 r t ih0 n iy0))
("martens" nil (m aa1 r t ax n z))
("martensen" nil (m aa1 r t ih0 n s ax n))
("martenson" nil (m aa1 r t ih0 n s ax n))
("marter" nil (m aa1 r t er0))
("marth" nil (m aa1 r th))
("martha" nil (m aa1 r th ax))
("marthaler" nil (m aa1 r th ax l er0))
("marthe" nil (m aa1 r dh))
("marthena" nil (m aa1 r th ih0 n ax))
("marti" nil (m aa1 r t iy0))
("martial" nil (m aa1 r sh ax l))
("martian" nil (m aa1 r sh ax n))
("martie" nil (m aa1 r t iy0))
("martin" nil (m aa1 r t ax n))
("martina" nil (m aa0 r t iy1 n ax))
("martindale" nil (m aa1 r t ih0 n d ey0 l))
("martine" nil (m aa0 r t iy1 n))
("martineau" nil (m aa1 r t ih0 n ow1))
("martinec" nil (m aa1 r t ih0 n eh0 k))
("martinek" nil (m aa1 r t ih0 n ih0 k))
("martines" nil (m aa0 r t iy1 n z))
("martinez" nil (m aa0 r t iy1 n eh0 z))
("marting" nil (m aa1 r t ih0 ng))
("martingale" nil (m aa1 r t ax n g ey1 l))
("martini" nil (m aa0 r t iy1 n iy0))
("martinique" nil (m aa0 r t ih0 n iy1 k))
("martinis" nil (m aa0 r t iy1 n iy0 z))
("martinka" nil (m aa1 r t ih1 ng k ax))
("martinko" nil (m aa1 r t ih1 ng k ow0))
("martino" nil (m aa0 r t iy1 n ow0))
("martinovich" nil (m aa1 r t ih1 n ax v ih0 ch))
("martins" nil (m aa1 r t ih0 n z))
("martinsen" nil (m aa0 r t ih1 n s ax n))
("martinson" nil (m aa1 r t ih0 n s ax n))
("martiny" nil (m aa1 r t ih0 n iy0))
("martire" nil (m aa0 r t ih1 r iy0))
("martirosov" nil (m aa0 r t ih1 r ax s aa0 v))
("martis" nil (m aa1 r t ih0 s))
("martita" nil (m aa0 r t iy1 t ax))
("martocci" nil (m aa0 r t ow1 ch iy0))
("marton" nil (m aa1 r t ax n))
("martone" nil (m aa0 r t ow1 n iy0))
("martorana" nil (m aa0 r t ao0 r ae1 n ax))
("martorano" nil (m aa0 r t ao0 r aa1 n ow0))
("martorelli" nil (m aa0 r t ao0 r eh1 l iy0))
("marts" nil (m aa1 r t s))
("martucci" nil (m aa0 r t uw1 ch iy0))
("marturano" nil (m aa0 r t uh0 r aa1 n ow0))
("martus" nil (m aa1 r t ax s))
("marty" nil (m aa1 r t iy0))
("martyn" nil (m aa1 r t ih0 n))
("martyr" nil (m aa1 r t er0))
("martyrdom" nil (m aa1 r t er0 d ax m))
("martyre" nil (m aa0 r t ay1 r))
("martyred" nil (m aa1 r t er0 d))
("martyrs" nil (m aa1 r t er0 z))
("martz" nil (m aa1 r t s))
("marubeni" nil (m aa1 r uw0 b iy1 n iy0))
("maruca" nil (m er0 uw1 k ax))
("marucci" nil (m aa0 r uw1 ch iy0))
("marui" nil (m er0 uw1 iy0))
("marukin" nil (m er0 uw1 k ih0 n))
("marulanda" nil (m ae1 r uw0 l ae1 n d ax))
("marullo" nil (m aa0 r uw1 l ow0))
("maruska" nil (m er0 ah1 s k ax))
("maruti" nil (m er0 uw1 t iy0))
("maruyama" nil (m aa1 r uw0 y aa1 m ax))
("maruyu" nil (m eh1 r uw0 y uw0))
("marv" nil (m aa1 r v))
("marva" nil (m aa1 r v ax))
("marvel" nil (m aa1 r v ax l))
("marvela" nil (m aa0 r v ey1 l ax))
("marveled" nil (m aa1 r v ax l d))
("marvell" nil (m aa0 r v eh1 l))
("marvella" nil (m aa1 r v eh1 l ax))
("marvelle" nil (m aa0 r v eh1 l))
("marvelous" nil (m aa1 r v ax l ax s))
("marvelously" nil (m aa1 r v ax l ax s l iy0))
("marvels" nil (m aa1 r v ax l z))
("marver" nil (m aa1 r v er0))
("marvin" nil (m aa1 r v ih0 n))
("marwick" nil (m aa1 r w ih1 k))
("marwin" nil (m aa1 r w ih0 n))
("marwood" nil (m aa1 r w uh1 d))
("marx" nil (m aa1 r k s))
("marxen" nil (m aa1 r k s ax n))
("marxism" nil (m aa1 r k s ih0 z ax m))
("marxist" nil (m aa1 r k s ax s t))
("marxists" nil (m aa1 r k s ih0 s t s))
("mary" nil (m eh1 r iy0))
("marya" nil (m aa1 r y ax))
("maryann" nil (m eh0 r iy0 ae1 n))
("maryanne" nil (m eh0 r iy0 ae1 n))
("maryland" nil (m eh1 r ax l ax n d))
("marylin" nil (m ae1 r ih0 l ih0 n))
("marylou" nil (m ae1 r ih0 l uw0))
("marymount" nil (m eh1 r iy0 m aw1 n t))
("maryruth" nil (m ae1 r ih0 r uw0 th))
("maryse" nil (m aa0 r iy1 s))
("marysville" nil (m eh1 r iy0 z v ih1 l))
("maryville" nil (m eh1 r iy0 v ih1 l))
("marz" nil (m aa1 r z))
("marzan" nil (m aa1 r z ax n))
("marzano" nil (m aa0 r z aa1 n ow0))
("marzec" nil (m aa1 r z ih0 k))
("marzette" nil (m aa1 r z eh1 t))
("marzilli" nil (m aa0 r z iy1 l iy0))
("marzo" nil (m aa1 r z ow0))
("marzolf" nil (m aa1 r z ow1 l f))
("marzotto" nil (m aa1 r z aa1 t ow0))
("marzullo" nil (m aa0 r z uw1 l ow0))
("mas" nil (m aa1 z))
("masa" nil (m aa1 s ax))
("masaaki" nil (m aa1 s aa0 aa1 k iy0))
("masahiko" nil (m aa1 s aa0 hh iy1 k ow0))
("masahiro" nil (m aa1 s aa0 hh ih1 r ow0))
("masai" nil (m aa0 s aa1 iy0))
("masakazu" nil (m aa1 s aa0 k aa1 z uw0))
("masaki" nil (m aa0 s aa1 k iy0))
("masami" nil (m aa0 s aa1 m iy0))
("masao" nil (m aa0 s aa1 ow0))
("masaru" nil (m aa0 s aa1 r uw0))
("masaya" nil (m aa0 s aa1 y ax))
("masayoshi" nil (m aa1 s aa0 y ow1 sh iy0))
("masayuki" nil (m aa1 s aa0 y uw1 k iy0))
("mascara" nil (m ae0 s k eh1 r ax))
("mascarenas" nil (m ae1 s k er0 ih0 n ax z))
("mascari" nil (m aa0 s k aa1 r iy0))
("mascaro" nil (m aa0 s k aa1 r ow0))
("masch" nil (m ae1 sh))
("maschino" nil (m aa0 s k iy1 n ow0))
("maschke" nil (m ae1 sh k))
("masci" nil (m ae1 s iy0))
("mascia" nil (m aa1 s ch ax))
("masciarelli" nil (m aa0 s ch er0 eh1 l iy0))
("mascio" nil (m ae1 s iy0 ow0))
("mascioli" nil (m aa0 s ch ow1 l iy0))
("masco" nil (m ae1 s k ow0))
("mascola" nil (m aa0 s k ow1 l ax))
("mascolo" nil (m aa0 s k ow1 l ow0))
("mascorro" nil (m aa0 s k ao1 r ow0))
("mascot" nil (m ae1 s k ax t))
("masculine" nil (m ae1 s k y ax l ax n))
("masculinity" nil (m ae1 s k y ax l ih1 n ax t iy0))
("masden" nil (m ae1 s d ax n))
("mase" nil (m ey1 z))
("masefield" nil (m ey1 z f iy1 l d))
("masek" nil (m aa1 s eh0 k))
("masella" nil (m ax s eh1 l ax))
("maselli" nil (m ax s eh1 l iy0))
("maseng" nil (m ae1 s ih0 ng))
("maser" nil (m ey1 z er0))
("maserati" nil (m ae1 s eh0 r aa1 t iy0))
("maseri" nil (m ax s eh1 r iy0))
("mash" nil (m ae1 sh))
("mashaw" nil (m ae1 sh ao0))
("mashburn" nil (m ae1 sh b er1 n))
("mashed" nil (m ae1 sh t))
("mashek" nil (m ae1 sh ih0 k))
("masi" nil (m aa1 s iy0))
("masiello" nil (m aa0 s iy0 eh1 l ow0))
("masin" nil (m aa1 s ih0 n))
("masini" nil (m aa0 s iy1 n iy0))
("masino" nil (m aa0 s iy1 n ow0))
("masius" nil (m ae1 s iy0 ax s))
("mask" nil (m ae1 s k))
("maske" nil (m ey1 s k))
("masked" nil (m ae1 s k t))
("maskell" nil (m ae1 s k ax l))
("masker" nil (m ae1 s k er0))
("masking" nil (m ae1 s k ih0 ng))
("masks" nil (m ae1 s k s))
("maslanka" nil (m ax s l ae1 ng k ax))
("maslen" nil (m ae1 s ax l ax n))
("masley" nil (m ae1 s l iy0))
("maslin" nil (m ae1 z l ih0 n))
("maslow" nil (m aa1 s l ow0))
("maslowski" nil (m ax s l ao1 f s k iy0))
("maslyukov" nil (m ae1 s l y uw0 k aa1 v))
("masochism" nil (m ae1 s ax k ih0 z ax m))
("mason" nil (m ey1 s ax n))
("masone" nil (m ax s ow1 n))
("masoner" nil (m ey1 s ax n er0))
("masonic" nil (m ax s aa1 n ih0 k))
("masonite" nil (m ae1 s ax n ay1 t))
("masonry" nil (m ey1 s ax n r iy0))
("masons" nil (m ey1 s ax n z))
("masquerade" nil (m ae1 s k er0 ey1 d))
("masquerading" nil (m ae1 s k er0 ey1 d ih0 ng))
("masri" nil (m aa1 s r iy0))
("mass" nil (m ae1 s))
("massa" nil (m ae1 s ax))
("massachusetts" nil (m ae1 s ax ch uw1 s ax t s))
("massachussetts" nil (m ae1 s ax ch uw1 s ax t s))
("massacre" nil (m ae1 s ax k er0))
("massacred" nil (m ae1 s ax k er0 d))
("massacres" nil (m ae1 s ih0 k er0 z))
("massacring" nil (m ae1 s ax k er0 ih0 ng))
("massad" nil (m ae1 s ax d))
("massage" nil (m ax s aa1 zh))
("massaged" nil (m ax s aa1 zh d))
("massager" nil (m ax s aa1 zh er0))
("massages" nil (m ax s aa1 zh ih0 z))
("massaging" nil (m ax s aa1 zh ih0 ng))
("massar" nil (m ae1 s er0))
("massari" nil (m aa0 s aa1 r iy0))
("massaro" nil (m ax s aa1 r ow0))
("masse" nil (m ae1 s))
("massed" nil (m ae1 s t))
("massenburg" nil (m ae1 s ax n b er0 g))
("massengale" nil (m ae1 s ax n g ey1 l))
("massengill" nil (m ae1 s ax n g ih1 l))
("masser" nil (m ae1 s er0))
("masses" nil (m ae1 s ax z))
("massett" nil (m ae1 s ih0 t))
("massey" nil (m ae1 s iy0))
("massi" nil (m ae1 s iy0))
("massicotte" nil (m ae1 s ih0 k ao0 t))
("massie" nil (m ae1 s iy0))
("massieu" nil (m ae1 s iy0 uw0))
("massif" nil (m ae0 s iy1 f))
("massimino" nil (m aa0 s iy0 m iy1 n ow0))
("massimo" nil (m ae1 s ih0 m ow1))
("massing" nil (m ae1 s ih0 ng))
("massingale" nil (m aa0 s ih0 ng g aa1 l iy0))
("massingill" nil (m ae1 s ih0 ng g ax l))
("massive" nil (m ae1 s ih0 v))
("massively" nil (m ae1 s ih0 v l iy0))
("massman" nil (m ae1 s m ax n))
("massmann" nil (m ae1 s m ax n))
("massmutual" nil (m ae1 s m y uw1 ch uw0 ax l))
("masso" nil (m ae1 s ow0))
("masson" nil (m ae1 s ax n))
("massoni" nil (m aa0 s ow1 n iy0))
("massoth" nil (m ae1 s ax th))
("massport" nil (m ae1 s p ao1 r t))
("massucci" nil (m aa0 s uw1 ch iy0))
("mast" nil (m ae1 s t))
("mastandrea" nil (m aa0 s t aa1 n d r iy0 ax))
("mastectomy" nil (m ae0 s t eh1 k t ax m iy0))
("masted" nil (m ae1 s t ax d))
("mastel" nil (m ey1 s t ax l))
("masteller" nil (m ey1 s t ax l er0))
("masten" nil (m ae1 s t ax n))
("master" nil (m ae1 s t er0))
("mastercard" nil (m ae1 s t er0 k aa1 r d))
("mastercards" nil (m ae1 s t er0 k aa1 r d z))
("mastered" nil (m ae1 s t er0 d))
("masterful" nil (m ae1 s t er0 f ax l))
("masterfully" nil (m ae1 s t er0 f ax l iy0))
("mastergate" nil (m ae1 s t er0 g ey1 t))
("mastering" nil (m ae1 s t er0 ih0 ng))
("masterly" nil (m ae1 s t er0 l iy0))
("masterman" nil (m ae1 s t er0 m ax n))
("mastermind" nil (m ae1 s t er0 m ay1 n d))
("masterminded" nil (m ae1 s t er0 m ay1 n d ih0 d))
("masterminding" nil (m ae1 s t er0 m ay1 n d ih0 ng))
("masterpiece" nil (m ae1 s t er0 p iy1 s))
("masterpieces" nil (m ae1 s t er0 p iy1 s ax z))
("masters" nil (m ae1 s t er0 z))
("masterson" nil (m ae1 s t er0 s ax n))
("masterstroke" nil (m ae1 s t er0 s t r ow1 k))
("masterwork" nil (m ae1 s t er0 w er1 k))
("masterworks" nil (m ae1 s t er0 w er1 k s))
("mastery" nil (m ae1 s t er0 iy0))
("masthead" nil (m ae1 s t hh eh1 d))
("mastif" nil (m ae1 s t ax f))
("mastiff" nil (m ae1 s t ax f))
("mastifs" nil (m ae1 s t ax f s))
("mastin" nil (m ae1 s t ih0 n))
("mastoiditis" nil (m ae1 s t oy0 d ay1 t ax s))
("maston" nil (m ae1 s t ax n))
("mastrangelo" nil (m aa0 s t r aa0 ng g eh1 l ow0))
("mastrianni" nil (m aa0 s t r iy0 aa1 n iy0))
("mastriano" nil (m aa0 s t r iy0 aa1 n ow0))
("mastrich" nil (m ae1 s t r ih0 ch))
("mastro" nil (m ae1 s t r ow0))
("mastrocola" nil (m ae0 s t r ow0 k ow1 l ax))
("mastrogiovanni" nil (m ae0 s t r ow0 jh ow0 v aa1 n iy0))
("mastroianni" nil (m ae0 s t r ow0 y aa1 n iy0))
("mastronardi" nil (m ae0 s t r ow0 n aa1 r d iy0))
("mastropietro" nil (m ae0 s t r ow0 p iy1 t r ow0))
("masturbate" nil (m ae1 s t er0 b ey1 t))
("masturbated" nil (m ae1 s t er0 b ey1 t ih0 d))
("masturbates" nil (m ae1 s t er0 b ey1 t s))
("masturbation" nil (m ae1 s t er0 b ey1 sh ax n))
("masucci" nil (m aa0 s uw1 ch iy0))
("masuda" nil (m ax s uw1 d ax))
("masullo" nil (m aa0 s uw1 l ow0))
("masur" nil (m ae1 s er0))
("mat" nil (m ae1 t))
("mata" nil (m aa1 t ax))
("matador" nil (m ae1 t ax d ao1 r))
("matagorda" nil (m ae1 t ax g ao1 r d ax))
("matala" nil (m ax t aa1 l ax))
("matalin" nil (m ae1 t ax l ih0 n))
("matalon" nil (m aa0 t aa0 l ao1 n))
("matamoros" nil (m aa0 t aa0 m ao1 r ow0 s))
("matarazzo" nil (m aa0 t aa0 r aa1 z ow0))
("matarese" nil (m aa0 t aa0 r ey1 z iy0))
("matas" nil (m aa1 t ax z))
("matassa" nil (m aa0 t aa1 s ax))
("mataya" nil (m aa0 t aa1 y ax))
("match" nil (m ae1 ch))
("matchbook" nil (m ae1 ch b uh1 k))
("matchbox" nil (m ae1 ch b aa1 k s))
("matched" nil (m ae1 ch t))
("matches" nil (m ae1 ch ax z))
("matchett" nil (m ae1 ch ih0 t))
("matching" nil (m ae1 ch ih0 ng))
("matchlocks" nil (m ae1 ch l aa1 k s))
("matchmaker" nil (m ae1 ch m ey1 k er0))
("matchmakers" nil (m ae1 ch m ey1 k er0 z))
("matchmaking" nil (m ae1 ch m ey1 k ih0 ng))
("matchup" nil (m ae1 ch ax p))
("matchups" nil (m ae1 ch ax p s))
("mate" nil (m ey1 t))
("matecki" nil (m ax t eh1 k iy0))
("mateer" nil (m ax t ih1 r))
("mateja" nil (m aa0 t ey1 y ax))
("matejka" nil (m ax t ey1 k ax))
("matell" nil (m ax t eh1 l))
("matelski" nil (m ax t eh1 l s k iy0))
("mateo" nil (m aa0 t ey1 ow0))
("mater" nil (m aa1 t er0))
("matera" nil (m aa0 t eh1 r ax))
("material" nil (m ax t ih1 r iy0 ax l))
("materialism" nil (m ax t ih1 r iy0 ax l ih1 z ax m))
("materialist" nil (m ax t ih1 r iy0 ax l ih0 s t))
("materialistic" nil (m ax t ih1 r iy0 ax l ih1 s t ih0 k))
("materialize" nil (m ax t ih1 r iy0 ax l ay1 z))
("materialized" nil (m ax t ih1 r iy0 ax l ay1 z d))
("materializes" nil (m ax t ih1 r iy0 ax l ay1 z ih0 z))
("materializing" nil (m ax t ih1 r iy0 ax l ay1 z ih0 ng))
("materially" nil (m ax t ih1 r iy0 ax l iy0))
("materials" nil (m ax t ih1 r iy0 ax l z))
("materiel" nil (m ax t ih1 r iy0 eh1 l))
("matern" nil (m ae1 t er0 n))
("materna" nil (m aa0 t eh1 r n ax))
("maternal" nil (m ax t er1 n ax l))
("maternity" nil (m ax t er1 n ih0 t iy0))
("mates" nil (m ey1 t s))
("matewan" nil (m ae1 t uw1 aa0 n))
("matey" nil (m ey1 t iy0))
("math" nil (m ae1 th))
("mathai" nil (m aa0 dh aa1 iy0))
("mathe" nil (m ey1 dh))
("mathea" nil (m ae1 dh iy0 ax))
("matheis" nil (m ae1 dh ay0 z))
("mathematical" nil (m ae1 th ax m ae1 t ih0 k ax l))
("mathematically" nil (m ae1 th ax m ae1 t ih0 k ax l iy0))
("mathematician" nil (m ae1 th ax m ax t ih1 sh ax n))
("mathematicians" nil (m ae1 th ax m ax t ih1 sh ax n z))
("mathematics" nil (m ae1 th ax m ae1 t ih0 k s))
("mathena" nil (m ae1 th ih0 n ax))
("matheney" nil (m ae1 th ih0 n iy0))
("matheny" nil (m ae1 th ih0 n iy0))
("mather" nil (m ae1 dh er0))
("matherly" nil (m ae1 dh er0 l iy0))
("mathern" nil (m ae1 dh er0 n))
("matherne" nil (m ae1 th er0 n))
("mathers" nil (m ae1 dh er0 z))
("matherson" nil (m ae1 th er0 s ax n))
("mathes" nil (m ae1 th ih0 s))
("matheson" nil (m ae1 th ax s ax n))
("matheus" nil (m ae1 dh oy0 z))
("mathew" nil (m ae1 th y uw0))
("mathews" nil (m ae1 th y uw0 z))
("mathewson" nil (m ae1 th y uw0 s ax n))
("mathey" nil (m ae1 dh iy0))
("mathia" nil (m aa1 dh iy0 ax))
("mathias" nil (m ax th ay1 ax s))
("mathiasen" nil (m ax th ay1 ax s ax n))
("mathiason" nil (m ax th ay1 ax s ax n))
("mathie" nil (m ae1 th iy0))
("mathies" nil (m ae1 th iy0 z))
("mathiesen" nil (m ae1 th iy0 s ax n))
("mathieson" nil (m ax th iy1 z ax n))
("mathieu" nil (m ae1 th y uw0))
("mathilda" nil (m ax t ih1 l d ax))
("mathilde" nil (m ae1 th ih0 l d))
("mathis" nil (m ae1 th ih0 s))
("mathisen" nil (m ae1 th ay0 z ax n))
("mathison" nil (m ae1 th ih0 s ax n))
("mathur" nil (m ae1 th er0))
("mathwig" nil (m ae1 th w ih0 g))
("mathy" nil (m ae1 th iy0))
("mathys" nil (m ey1 th iy0 z))
("matias" nil (m aa1 sh ax z))
("matic" nil (m ae1 t ih0 k))
("matif" nil (m aa0 t iy1 f))
("matilda" nil (m ax t ih1 l d ax))
("matinee" nil (m ae1 t ih0 n ey1))
("mating" nil (m ey1 t ih0 ng))
("matis" nil (m aa1 t ih0 s))
("matisse" nil (m ae1 t ih0 s))
("matkin" nil (m ae1 t k ih0 n))
("matkins" nil (m ae1 t k ih0 n z))
("matkovich" nil (m aa1 t k ax v ih0 ch))
("matlack" nil (m ae1 t l ae1 k))
("matlick" nil (m ae1 t l ih0 k))
("matlin" nil (m ae1 t l ih0 n))
("matlock" nil (m ae1 t l aa1 k))
("matney" nil (m ae1 t n iy0))
("mato" nil (m aa1 t ow0))
("matos" nil (m aa1 t ow0 z))
("matott" nil (m ae1 t ax t))
("matousek" nil (m ax t uw1 s eh0 k))
("matra" nil (m ae1 t r ax))
("matranga" nil (m aa0 t r aa1 ng g ax))
("matriarch" nil (m ey1 t r iy0 aa1 r k))
("matriarchal" nil (m ey1 t r iy0 aa1 r k ax l))
("matriarchy" nil (m ey1 t r iy0 aa1 r k iy0))
("matrilineal" nil (m ae1 t r ih0 l ih1 n iy0 ax l))
("matrimonial" nil (m ae1 t r ax m ow1 n iy0 ax l))
("matrimony" nil (m ae1 t r ax m ow1 n iy0))
("matrix" nil (m ey1 t r ih0 k s))
("matron" nil (m ey1 t r ax n))
("matronly" nil (m ey1 t r ax n l iy0))
("matrons" nil (m ey1 t r ax n z))
("mats" nil (m ae1 t s))
("matsapa" nil (m ae0 t s aa1 p ax))
("matsen" nil (m aa1 t s ax n))
("matsko" nil (m aa1 t s k ow0))
("matson" nil (m aa1 t s ax n))
("matsubara" nil (m aa1 t s uw1 b aa1 r ax))
("matsuda" nil (m aa0 t s uw1 d ax))
("matsui" nil (m aa0 t s uw1 iy0))
("matsukawa" nil (m aa1 t s uw1 k aa1 w ax))
("matsumoto" nil (m aa0 t s uw0 m ow1 t ow0))
("matsumura" nil (m aa0 t s uw0 m uh1 r ax))
("matsunaga" nil (m aa0 t s uw0 n aa1 g ax))
("matsuo" nil (m aa0 t s uw1 ow0))
("matsuoka" nil (m aa0 t s uw0 ow1 k ax))
("matsushita" nil (m aa0 t s uw0 sh iy1 t ax))
("matsuura" nil (m aa0 t s uw0 uh1 r ax))
("matt" nil (m ae1 t))
("matta" nil (m ae1 t ax))
("matte" nil (m ae1 t))
("mattea" nil (m ae1 sh ax))
("mattei" nil (m ae1 t ay0))
("matteis" nil (m ax t ay1 s))
("mattel" nil (m ax t eh1 l))
("matteo" nil (m aa0 t ey1 ow0))
("matter" nil (m ae1 t er0))
("mattera" nil (m aa0 t eh1 r ax))
("mattered" nil (m ae1 t er0 d))
("mattern" nil (m ae1 t er0 n))
("matters" nil (m ae1 t er0 z))
("mattes" nil (m ae1 t s))
("matteson" nil (m ae1 t ih0 s ax n))
("matteucci" nil (m aa0 t uw1 ch iy0))
("matthea" nil (m ae1 t dh iy0 ax))
("mattheis" nil (m ae1 t dh ay0 z))
("matthes" nil (m ae1 t dh z))
("matthew" nil (m ae1 th y uw0))
("matthews" nil (m ae1 th y uw0 z))
("matthewson" nil (m ae1 th y uw0 s ax n))
("matthey" nil (m ae1 t dh iy0))
("matthia" nil (m aa1 t dh iy0 ax))
("matthias" nil (m ax th ay1 ax s))
("matthies" nil (m ae1 th iy0 z))
("matthiesen" nil (m ae1 th iy0 s ax n))
("matthis" nil (m ae1 th ih0 s))
("matthys" nil (m ae1 th iy0 z))
("mattias" nil (m ae1 sh ax z))
("mattice" nil (m aa1 t ih0 s))
("mattick" nil (m ae1 t ih0 k))
("mattie" nil (m ae1 t iy0))
("mattila" nil (m aa0 t iy1 l ax))
("mattina" nil (m aa0 t iy1 n ax))
("matting" nil (m ae1 t ih0 ng))
("mattingley" nil (m ae1 t ih0 ng g l iy0))
("mattingly" nil (m ae1 t ih0 ng l iy0))
("mattioli" nil (m aa0 t iy0 ow1 l iy0))
("mattis" nil (m ae1 t ih0 s))
("mattison" nil (m ae1 t ih0 s ax n))
("mattke" nil (m ae1 t k iy0))
("mattocks" nil (m ae1 t ax k s))
("matton" nil (m ae1 t ax n))
("mattoon" nil (m ax t uw1 n))
("mattos" nil (m ae1 t ow0 z))
("mattox" nil (m ae1 t ax k s))
("mattress" nil (m ae1 t r ax s))
("mattresses" nil (m ae1 t r ax s ax z))
("matts" nil (m ae1 t s))
("mattson" nil (m ae1 t s ax n))
("mattucci" nil (m aa0 t uw1 ch iy0))
("matty" nil (m ae1 t iy0))
("matula" nil (m aa0 t uw1 l ax))
("matulich" nil (m ax t y uw1 l ih0 hh))
("matulis" nil (m ae1 t ax l ih0 s))
("maturation" nil (m ae1 ch er0 ey1 sh ax n))
("mature" nil (m ax ch uh1 r))
("matured" nil (m ax t y uh1 r d))
("matures" nil (m ax ch uh1 r z))
("maturing" nil (m ax ch uh1 r ih0 ng))
("maturities" nil (m ax ch uh1 r ih0 t iy0 z))
("maturity" nil (m ax ch uh1 r ax t iy0))
("maturo" nil (m aa0 t uh1 r ow0))
("matus" nil (m ae1 t ih0 s))
("matuschka" nil (m ax t ah1 sh k ax))
("matusek" nil (m ax t uw1 s eh0 k))
("matusiak" nil (m ax t ah1 s iy0 ae0 k))
("matusik" nil (m ax t uw1 s ih0 k))
("matuska" nil (m ax t ah1 s k ax))
("matuszak" nil (m ax t ah1 sh ax k))
("matuszewski" nil (m ax t ax sh eh1 f s k iy0))
("matyas" nil (m ae1 t y ax s))
("matz" nil (m ae1 t s))
("matza" nil (m aa1 t z ax))
("matzek" nil (m ae1 t z ih0 k))
("matzen" nil (m ae1 t z ax n))
("matzinger" nil (m ey1 t z ih0 ng er0))
("matzke" nil (m ae1 t z k))
("matzner" nil (m ae1 t s n er0))
("mau" nil (m ow1))
("mauceri" nil (m ao0 ch eh1 r iy0))
("mauch" nil (m ao1 ch))
("maucher" nil (m aw1 ch er0))
("mauck" nil (m ao1 k))
("maud" nil (m ao1 d))
("maude" nil (m ao1 d))
("maudlin" nil (m ao1 d l ih0 n))
("maue" nil (m aw1 iy0))
("mauer" nil (m aw1 er0))
("mauger" nil (m ao1 g er0))
("maugeri" nil (m ao0 jh eh1 r iy0))
("maughan" nil (m ao1 g ax n))
("maui" nil (m aw1 iy0))
("mauk" nil (m ao1 k))
("maul" nil (m ao1 l))
("maulden" nil (m aw1 l d ax n))
("mauldin" nil (m aw1 l d ax n))
("maulding" nil (m ao1 l d ih0 ng))
("maule" nil (m ao1 l))
("mauled" nil (m ao1 l d))
("mauler" nil (m ao1 l er0))
("maull" nil (m ao1 l))
("maulsby" nil (m ao1 l s b iy0))
("mault" nil (m ao1 l t))
("maultasch" nil (m ao1 l t ae0 sh))
("maultsby" nil (m ao1 l t s b iy0))
("maumee" nil (m aw1 m iy0))
("mauna" nil (m ao1 n ax))
("maund" nil (m ao1 n d))
("maune" nil (m ao1 n))
("mauney" nil (m ao1 n iy0))
("maung" nil (m ao1 ng))
("maupin" nil (m ow0 p ae1 n))
("maura" nil (m ao1 r ax))
("maureen" nil (m ao0 r iy1 n))
("maurer" nil (m ao1 r er0))
("mauri" nil (m ao1 r iy0))
("maurice" nil (m ao0 r iy1 s))
("mauricio" nil (m aw0 r iy1 s iy0 ow0))
("maurie" nil (m ao1 r iy0))
("mauriello" nil (m ao1 r iy0 eh1 l ow0))
("maurin" nil (m ao1 r ih0 n))
("maurine" nil (m ao0 r iy1 n iy0))
("maurita" nil (m ao0 r iy1 t ax))
("mauritania" nil (m ao1 r ih0 t ey1 n iy0 ax))
("mauritius" nil (m ao0 r ih1 t iy0 ax s))
("mauritz" nil (m ao1 r ih0 t s))
("maurizio" nil (m ao0 r ih1 z iy0 ow0))
("mauro" nil (m ao1 r ow0))
("maurus" nil (m ao1 r ax s))
("maury" nil (m ao1 r iy0))
("maus" nil (m ao1 z))
("mauser" nil (m aw1 s er0))
("mausoleum" nil (m ao1 s ax l iy1 ax m))
("mausolf" nil (m aw1 s ow0 l f))
("mauss" nil (m ao1 s))
("mausser" nil (m aw1 s er0))
("maust" nil (m ao1 s t))
("maute" nil (m ao1 t))
("mauthe" nil (m ao1 dh))
("mautner" nil (m ao1 t n er0))
("mautz" nil (m ao1 t s))
("mauve" nil (m ao1 v))
("mauzey" nil (m ao1 z iy0))
("mauzy" nil (m ao1 z iy0))
("mave" nil (m ey1 v))
("maven" nil (m ey1 v ax n))
("mavens" nil (m ey1 v ax n z))
("maverick" nil (m ae1 v er0 ih0 k))
("mavericks" nil (m ae1 v er0 ih0 k s))
("maverix" nil (m ae1 v er0 ih0 k s))
("maves" nil (m ey1 v z))
("mavis" nil (m ey1 v ih0 s))
("mavrodi" nil (m ae0 v r ow1 d iy0))
("mavroules" nil (m ae0 v r uw1 l z))
("maw" nil (m ao1))
("mawhinney" nil (m ae1 w ih0 n iy0))
("mawkish" nil (m ao1 k ih0 sh))
("mawr" nil (m ao1 er0))
("mawson" nil (m ao1 s ax n))
("mawyer" nil (m ao1 iy0 er0))
("max" nil (m ae1 k s))
("maxa" nil (m ae1 k s ax))
("maxam" nil (m ae1 k s ax m))
("maxcy" nil (m ae1 k s iy0))
("maxed" nil (m ae1 k s t))
("maxell" nil (m ae0 k s eh1 l))
("maxey" nil (m ae1 k s iy0))
("maxfield" nil (m ae1 k s f iy0 l d))
("maxham" nil (m ae1 k s ax m))
("maxi" nil (m ae1 k s iy0))
("maxicare" nil (m ae1 k s iy0 k eh1 r))
("maxie" nil (m ae1 k s iy0))
("maxilla" nil (m ae0 k s ih1 l ax))
("maxim" nil (m ae1 k s ax m))
("maxima" nil (m ae1 k s ax m ax))
("maximal" nil (m ae1 k s ax m ax l))
("maxime" nil (m ae0 k s iy1 m))
("maximilian" nil (m ae1 k s ih0 m ih1 l iy0 ax n))
("maximization" nil (m ae1 k s ih0 m ih0 z ey1 sh ax n))
("maximize" nil (m ae1 k s ax m ay1 z))
("maximized" nil (m ae1 k s ax m ay1 z d))
("maximizes" nil (m ae1 k s ax m ay1 z ih0 z))
("maximizing" nil (m ae1 k s ax m ay1 z ih0 ng))
("maxims" nil (m ae1 k s ax m z))
("maximum" nil (m ae1 k s ax m ax m))
("maxine" nil (m ae0 k s iy1 n))
("maxion" nil (m ae1 k s iy0 aa0 n))
("maxis" nil (m ae1 k s iy0 z))
("maxon" nil (m ae1 k s ax n))
("maxpharma" nil (m ae0 k s f aa1 r m ax))
("maxsaver" nil (m ae1 k s ey1 v er0))
("maxsavers" nil (m ae1 k s ax v er0 z))
("maxson" nil (m ae1 k s ax n))
("maxton" nil (m ae1 k s t ax n))
("maxtor" nil (m ae1 k s t er0))
("maxus" nil (m ae1 k s ax s))
("maxwell" nil (m ae1 k s w eh1 l))
("maxx" nil (m ae1 k s))
("maxxam" nil (m ae1 k s ax m))
("maxxim" nil (m ae1 k s ih0 m))
("maxy" nil (m ae1 k s iy0))
("maxzide" nil (m ae0 k s iy1 t ax))
("may" nil (m ey1))
("maya" nil (m ay1 ax))
("mayall" nil (m ey1 ao1 l))
("mayan" nil (m ay1 ax n))
("maybe" nil (m ey1 b iy0))
("maybee" nil (m ey1 b iy1))
("maybelle" nil (m ey1 b eh1 l))
("maybelline" nil (m ey1 b ax l iy1 n))
("mayben" nil (m ey1 b ax n))
("mayberry" nil (m ey1 b eh1 r iy0))
("maybin" nil (m ey1 b ih0 n))
("maybury" nil (m ey1 b eh1 r iy0))
("maycock" nil (m ey1 k aa1 k))
("mayda" nil (m ey1 d ax))
("mayde" nil (m ey1 d))
("mayden" nil (m ey1 d ax n))
("maydena" nil (m ey1 d ih0 n ax))
("maye" nil (m ey1))
("mayeaux" nil (m ey1 ow0))
("mayeda" nil (m ay0 iy1 d ax))
("mayer" nil (m ey1 er0))
("mayernik" nil (m ey0 er1 n ih0 k))
("mayers" nil (m ey1 er0 z))
("mayerson" nil (m ey1 er0 s ax n))
("mayes" nil (m ey1 z))
("mayeux" nil (m ey0 ow1))
("mayfair" nil (m ey1 f eh1 r))
("mayfield" nil (m ey1 f iy1 l d))
("mayflower" nil (m ey1 f l aw1 er0))
("mayhall" nil (m ey1 hh ao1 l))
("mayhan" nil (m ey1 ax n))
("mayhem" nil (m ey1 hh eh1 m))
("mayher" nil (m ey1 er0))
("mayhew" nil (m ey1 hh y uw0))
("mayhue" nil (m ey1 hh y uw1))
("mayhugh" nil (m ey1 hh ax))
("mayland" nil (m ey1 l ax n d))
("mayle" nil (m ey1 l))
("mayme" nil (m ey1 m))
("maynard" nil (m ey1 n er0 d))
("mayne" nil (m ey1 n))
("mayner" nil (m ey1 n er0))
("maynes" nil (m ey1 n z))
("maynor" nil (m ey1 n er0))
("mayo" nil (m ey1 ow0))
("mayon" nil (m ey0 ao1 n))
("mayonnaise" nil (m ey1 ax n ey1 z))
("mayor" nil (m ey1 er0))
("mayoral" nil (m ey1 er0 ax l))
("mayoralty" nil (m ey1 er0 ax l t iy0))
("mayorga" nil (m ey1 ao1 r g ax))
("mayors" nil (m ey1 er0 z))
("mayotte" nil (m ey0 ao1 t))
("mayr" nil (m ey1 r))
("mays" nil (m ey1 z))
("mayse" nil (m ey1 z))
("mayson" nil (m ey1 z ax n))
("maytag" nil (m ey1 t ae1 g))
("mayton" nil (m ey1 t ax n))
("mayville" nil (m ey1 v ih1 l))
("mayweather" nil (m ey1 w eh1 dh er0))
("maz" nil (m aa1 z))
("maza" nil (m aa1 z ax))
("mazanec" nil (m ax z ae1 n ih0 k))
("mazankowski" nil (m ae1 z ax ng k aw1 s k iy0))
("mazda" nil (m aa1 z d ax))
("maze" nil (m ey1 z))
("mazeika" nil (m ax z ay1 k ax))
("mazeltov" nil (m aa1 z ax l t aa1 f))
("mazer" nil (m ey1 z er0))
("mazes" nil (m ey1 z ih0 z))
("maziarz" nil (m ax z iy1 er0 z))
("mazie" nil (m ey1 z iy0))
("mazo" nil (m aa1 z ow0))
("mazola" nil (m ax z ow1 l ax))
("mazon" nil (m aa0 z ao1 n))
("mazor" nil (m aa0 z ao1 r))
("mazowiecki" nil (m ax z ow1 iy0 eh1 k iy0))
("mazur" nil (m ae1 z er0))
("mazurek" nil (m ax z y uw1 r eh0 k))
("mazurkas" nil (m ax z er1 k ax z))
("mazurkiewicz" nil (m ax z er1 k ax v ih0 ch))
("mazurowski" nil (m ax z er0 ao1 f s k iy0))
("mazyck" nil (m ae1 z ih0 k))
("mazza" nil (m ae1 z ax))
("mazzaferro" nil (m ae1 z ax f eh1 r ow0))
("mazzanti" nil (m aa0 t s aa1 n t iy0))
("mazzara" nil (m aa0 t s aa1 r ax))
("mazzarella" nil (m aa0 t s aa0 r eh1 l ax))
("mazzei" nil (m aa0 t s ey1 iy0))
("mazzella" nil (m aa0 t s eh1 l ax))
("mazzeo" nil (m ae1 z iy0 ow0))
("mazzetti" nil (m aa0 t s eh1 t iy0))
("mazzie" nil (m ae1 z iy0))
("mazziotti" nil (m aa0 t s iy0 ow1 t iy0))
("mazzo" nil (m ae1 z ow0))
("mazzocchi" nil (m aa0 t s ow1 k iy0))
("mazzocco" nil (m aa0 t s ow1 k ow0))
("mazzola" nil (m aa0 t s ow1 l ax))
("mazzoli" nil (m ax z ow1 l iy0))
("mazzone" nil (m aa0 t s ow1 n iy0))
("mazzoni" nil (m aa0 t s ow1 n iy0))
("mazzorana" nil (m ae1 z er0 aa1 n ax))
("mazzotta" nil (m aa0 t s ow1 t ax))
("mazzuca" nil (m aa0 t s uw1 k ax))
("mazzucco" nil (m aa0 t s uw1 k ow0))
("mbank" nil (eh1 m b ae1 ng k))
("mbeki" nil (m b eh1 k iy0))
("mc" nil (m ih0 k))
("mcabee" nil (m ax k ae1 b iy0))
("mcadam" nil (m ax k ae1 d ax m))
("mcadams" nil (m ax k ae1 d ax m z))
("mcadoo" nil (m ae1 k ax d uw1))
("mcadory" nil (m ax k ae1 d er0 iy0))
("mcadow" nil (m ax k ae1 d ow0))
("mcafee" nil (m ae1 k ax f iy1))
("mcaffee" nil (m ae1 k ax f iy0))
("mcaleer" nil (m ae1 k ax l ih0 r))
("mcaleese" nil (m ae1 k ax l iy0 z))
("mcalexander" nil (m ax k ae1 l ih0 g z ae1 n d er0))
("mcalinden" nil (m ax k l ih1 n d ax n))
("mcalister" nil (m ax k ae1 l ax s t er0))
("mcallen" nil (m ax k ao1 l ax n))
("mcallister" nil (m ax k ae1 l ax s t er0))
("mcaloon" nil (m ae1 k ax l uw1 n))
("mcalpin" nil (m ax k ae1 l p ax n))
("mcalpine" nil (m ax k ae1 l p ay1 n))
("mcamis" nil (m ax k ae1 m ax s))
("mcan" nil (m ax k ae1 n))
("mcanally" nil (m ae1 k ax n ae1 l iy0))
("mcanany" nil (m ax k ae1 n ax n iy0))
("mcandrew" nil (m ax k ae1 n d r uw1))
("mcandrews" nil (m ax k ae1 n d r uw1 z))
("mcanelly" nil (m ae1 k ax n ae1 l iy0))
("mcaninch" nil (m ax k ae1 n ih0 n ch))
("mcannally" nil (m ae1 k ax n ae1 l iy0))
("mcanulty" nil (m ae1 k ax n ax l t iy0))
("mcardle" nil (m ax k aa1 r d ax l))
("mcarthur" nil (m ax k aa1 r th er0))
("mcartor" nil (m ax k aa1 r t er0))
("mcatee" nil (m ax k ae1 t iy0))
("mcateer" nil (m ae1 k ax t ih1 r))
("mcaulay" nil (m ax k ao1 l ey0))
("mcauley" nil (m ax k ao1 l iy0))
("mcauliff" nil (m ax k ao1 l ax f))
("mcauliffe" nil (m ax k ao1 l ax f))
("mcavity" nil (m ax k ae1 v ax t iy0))
("mcavoy" nil (m ae1 k ax v oy1))
("mcbain" nil (m ax k b ey1 n))
("mcbane" nil (m ax k b ey1 n))
("mcbay" nil (m ax k b ey1))
("mcbean" nil (m ax k b iy1 n))
("mcbeath" nil (m ax k b eh1 th))
("mcbee" nil (m ax k b iy1))
("mcbeth" nil (m ax k b eh1 th))
("mcbirney" nil (m ax k b er1 n iy0))
("mcbrayer" nil (m ax k b r ey1 er0))
("mcbrearty" nil (m ax k b r ih1 r t iy0))
("mcbreen" nil (m ax k b r iy1 n))
("mcbride" nil (m ax k b r ay1 d))
("mcbridge" nil (m ax k b r ih1 jh))
("mcbrien" nil (m ax k b r ay1 ax n))
("mcbroom" nil (m ax k b r uw1 m))
("mcbryar" nil (m ax k b r ay1 er0))
("mcbryde" nil (m ax k b r ay1 d))
("mcburnett" nil (m ax k b er1 n ax t))
("mcburney" nil (m ax k b er1 n iy0))
("mccaa" nil (m ax k aa1))
("mccabe" nil (m ax k ey1 b))
("mccadden" nil (m ax k ae1 d ax n))
("mccafferty" nil (m ax k ae1 f er0 t iy0))
("mccaffery" nil (m ax k ae1 f r iy0))
("mccaffrey" nil (m ax k ae1 f r iy0))
("mccaghren" nil (m ax k ae1 g r ax n))
("mccague" nil (m ax k ey1 g))
("mccahan" nil (m ax k ae1 n))
("mccahill" nil (m ax k aa1 hh ih1 l))
("mccaig" nil (m ax k ey1 g))
("mccain" nil (m ax k ey1 n))
("mccaleb" nil (m ax k ey1 l ax b))
("mccalip" nil (m ax k ae1 l ih1 p))
("mccalister" nil (m ax k ae1 l ax s t er0))
("mccall" nil (m ax k ao1 l))
("mccalla" nil (m ax k ae1 l ax))
("mccallen" nil (m ax k ao1 l ax n))
("mccalley" nil (m ax k ae1 l iy0))
("mccallie" nil (m ax k ao1 l iy0))
("mccallion" nil (m ax k ae1 l y ax n))
("mccallister" nil (m ax k ae1 l ax s t er0))
("mccallon" nil (m ax k ae1 l ax n))
("mccallum" nil (m ax k ae1 l ax m))
("mccalmont" nil (m ax k ae1 l m ax n t))
("mccamant" nil (m ax k ae1 m ax n t))
("mccambridge" nil (m ax k ae1 m b r ih1 jh))
("mccamey" nil (m ax k ae1 m iy0))
("mccamish" nil (m ax k ae1 m ih1 sh))
("mccammon" nil (m ax k ae1 m ax n))
("mccampbell" nil (m ax k ae1 m p b ax l))
("mccamy" nil (m ax k ey1 m iy0))
("mccan" nil (m ax k ae1 n))
("mccance" nil (m ax k ae1 n s))
("mccandless" nil (m ax k ae1 n d l ax s))
("mccandlish" nil (m ax k ae1 n d l ih0 sh))
("mccane" nil (m ax k ey1 n))
("mccanless" nil (m ax k ae1 n l ax s))
("mccann" nil (m ax k ae1 n))
("mccanna" nil (m ax k ae1 n ax))
("mccannon" nil (m ax k ae1 n ax n))
("mccants" nil (m ax k ae1 n t s))
("mccard" nil (m ax k aa1 r d))
("mccardell" nil (m ax k aa1 r d ax l))
("mccardle" nil (m ax k aa1 r d ax l))
("mccarey" nil (m ax k ae1 r iy0))
("mccargar" nil (m ax k aa1 r g er0))
("mccargo" nil (m ax k aa1 r g ow0))
("mccarl" nil (m ax k aa1 r l))
("mccarley" nil (m ax k aa1 r l iy0))
("mccarn" nil (m ax k aa1 r n))
("mccarney" nil (m ax k aa1 r n iy0))
("mccarran" nil (m ax k eh1 r ax n))
("mccarrell" nil (m ax k ae1 r ax l))
("mccarren" nil (m ax k eh1 r ax n))
("mccarrick" nil (m ax k ae1 r ih0 k))
("mccarroll" nil (m ax k ae1 r ax l))
("mccarron" nil (m ax k ae1 r ax n))
("mccarry" nil (m ax k ae1 r iy0))
("mccarson" nil (m ax k aa1 r s ax n))
("mccart" nil (m ax k aa1 r t))
("mccartan" nil (m ax k aa1 r t ax n))
("mccarten" nil (m ax k aa1 r t ax n))
("mccarter" nil (m ax k aa1 r t er0))
("mccartha" nil (m ax k aa1 r th ax))
("mccarthy" nil (m ax k aa1 r th iy0))
("mccarthyism" nil (m ax k aa1 r th iy0 ih0 z ax m))
("mccartin" nil (m ax k aa1 r t ax n))
("mccartney" nil (m ax k aa1 r t n iy0))
("mccartt" nil (m ax k aa1 r t))
("mccarty" nil (m ax k aa1 r t iy0))
("mccarver" nil (m ax k aa1 r v er0))
("mccarville" nil (m ax k aa0 r v ax l))
("mccary" nil (m iy1 k eh0 r iy0))
("mccaskey" nil (m ax k ae1 s k iy0))
("mccaskill" nil (m ax k ae1 s k ax l))
("mccasland" nil (m ax k ae1 z l ax n d))
("mccaslin" nil (m ax k ae1 z l ax n))
("mccaughan" nil (m ax k ao1 n))
("mccaughey" nil (m ax k ao1 iy0))
("mccaul" nil (m ax k ao1 l))
("mccauley" nil (m ax k ao1 l iy0))
("mccaulley" nil (m ax k ao1 l iy0))
("mccausland" nil (m ax k ao1 z l ax n d))
("mccauslin" nil (m ax k ao1 z l ax n))
("mccaw" nil (m ax k ao1))
("mccawley" nil (m ax k ao1 l iy0))
("mccay" nil (m ax k ey1))
("mcchesney" nil (m ax k ch eh1 z n iy0))
("mcchristian" nil (m ax k r ih1 s ch ax n))
("mcclafferty" nil (m ax k l ae1 f er0 t iy0))
("mcclaflin" nil (m ax k l ae1 f l ax n))
("mcclain" nil (m ax k l ey1 n))
("mcclaine" nil (m ax k l ey1 n))
("mcclam" nil (m ax k l ae1 m))
("mcclanahan" nil (m ax k l ae1 n ax hh ae0 n))
("mcclane" nil (m ax k l ey1 n))
("mcclaran" nil (m ax k l ae1 r ax n))
("mcclard" nil (m iy1 k l er0 d))
("mcclaren" nil (m ax k l eh1 r ax n))
("mcclarnon" nil (m ax k l aa1 r n ax n))
("mcclarty" nil (m ax k l aa1 r t iy0))
("mcclary" nil (m ax k l eh1 r iy0))
("mcclaskey" nil (m ax k l ae1 s k iy0))
("mcclatchey" nil (m ax k l ae1 ch iy0))
("mcclatchy" nil (m ax k l ae1 ch iy0))
("mcclaugherty" nil (m ax k l ae1 f er0 t iy0))
("mcclave" nil (m ax k l ey1 v))
("mcclay" nil (m ax k l ey1))
("mccleaf" nil (m ax k l iy1 f))
("mcclean" nil (m ax k l iy1 n))
("mccleary" nil (m ax k l ih1 r iy0))
("mccleave" nil (m ax k l iy1 v))
("mccleery" nil (m ax k l ih1 r iy0))
("mcclees" nil (m ax k l iy1 z))
("mccleese" nil (m ax k l iy1 s))
("mcclellan" nil (m ax k l eh1 l ax n))
("mcclelland" nil (m ax k l eh1 l ax n d))
("mcclellen" nil (m ax k l eh1 l ax n))
("mcclements" nil (m ax k l eh1 m ax n t s))
("mcclenaghan" nil (m ax k l eh1 n ax g ax n))
("mcclenahan" nil (m ax k l eh1 n ax hh ae0 n))
("mcclenathan" nil (m ax k l eh1 n ax th ax n))
("mcclendon" nil (m ax k l ey1 n d ax n))
("mcclenny" nil (m ax k l eh1 n iy0))
("mccleskey" nil (m ax k l eh1 s k iy0))
("mcclimans" nil (m ax k l ay1 m ax n z))
("mcclintic" nil (m ax k l ih1 n t ih0 k))
("mcclintick" nil (m ax k l ih1 n t ih0 k))
("mcclintock" nil (m ax g l ih1 n t aa0 k))
("mcclinton" nil (m ax k l ih1 n t ax n))
("mcclish" nil (m ax k l ih1 sh))
("mcclory" nil (m ax k l ao1 r iy0))
("mccloskey" nil (m ax k l ao1 s k iy0))
("mcclosky" nil (m ax k l ao1 s k iy0))
("mccloud" nil (m ax k l aw1 d))
("mccloy" nil (m ax k l oy1))
("mccluer" nil (m ax k l uw1 r))
("mcclune" nil (m ax k l uw1 n))
("mccluney" nil (m ax k l uw1 n iy0))
("mcclung" nil (m ax k l ah1 ng))
("mcclure" nil (m ax k l uw1 r))
("mcclurg" nil (m ax k l er1 g))
("mcclurkin" nil (m ax k l er1 k ax n))
("mccluskey" nil (m ax k l ah1 s k iy0))
("mccoig" nil (m ax k oy1 g))
("mccoin" nil (m ax k oy1 n))
("mccole" nil (m ax k ow1 l))
("mccolgan" nil (m ax k ow1 l g ax n))
("mccoll" nil (m ax k ow1 l))
("mccollam" nil (m ax k aa1 l ax m))
("mccolley" nil (m ax k aa1 l iy0))
("mccollister" nil (m ax k aa1 l ax s t er0))
("mccolloch" nil (m ax k aa1 l ax k))
("mccollom" nil (m ax k aa1 l ax m))
("mccollough" nil (m ax k aa1 l ax))
("mccollum" nil (m ax k ao1 l ax m))
("mccolm" nil (m ax k ow1 m))
("mccomas" nil (m ax k ow1 m ax s))
("mccomb" nil (m ax k ow1 m))
("mccomber" nil (m ax k ow1 m b er0))
("mccombie" nil (m ax k ow1 m b iy0))
("mccombs" nil (m ax k aa1 m z))
("mccommon" nil (m ax k aa1 m ax n))
("mccommons" nil (m ax k aa1 m ax n z))
("mccomsey" nil (m ax k aa1 m s iy0))
("mcconaghy" nil (m ax k aa1 n ax g iy0))
("mcconaha" nil (m ax k aa1 n ax hh aa0))
("mcconahay" nil (m ax k aa1 n ax hh ey1))
("mcconahy" nil (m ax k aa1 n ax hh iy0))
("mcconathy" nil (m ax k aa1 n ax th iy0))
("mcconaughey" nil (m ax k aa1 n ax g ey0))
("mcconaughy" nil (m ax k aa1 n ax g iy0))
("mccone" nil (m ax k ow1 n))
("mcconico" nil (m ax k aa1 n ax k ow0))
("mcconkey" nil (m ax k aa1 ng k iy0))
("mcconn" nil (m ax k aa1 n))
("mcconnaughey" nil (m ax k aa1 n ax g ey0))
("mcconnel" nil (m ax k aa1 n ax l))
("mcconnell" nil (m ax k aa1 n ax l))
("mcconnon" nil (m ax k aa1 n ax n))
("mcconville" nil (m ax k aa1 n v ih1 l))
("mccooey" nil (m ax k uw1 iy0))
("mccook" nil (m ax k uh1 k))
("mccool" nil (m ax k uw1 l))
("mccord" nil (m ax k ao1 r d))
("mccorkel" nil (m ax k ao1 r k ax l))
("mccorkell" nil (m ax k ao1 r k ax l))
("mccorkindale" nil (m ax k ao1 r k ax n d ey1 l))
("mccorkle" nil (m ax k ao1 r k ax l))
("mccormac" nil (m ax k ao1 r m ax k))
("mccormack" nil (m ax k ao1 r m ax k))
("mccormick" nil (m ax k ao1 r m ih0 k))
("mccorquodale" nil (m ax k ao1 r k ax d ey1 l))
("mccorry" nil (m ax k ao1 r iy0))
("mccort" nil (m ax k ao1 r t))
("mccorvey" nil (m ax k ao1 r v iy0))
("mccosh" nil (m ax k aa1 sh))
("mccoskey" nil (m ax k aa1 s k iy0))
("mccotter" nil (m ax k aa1 t er0))
("mccoun" nil (m ax k aw1 n))
("mccourt" nil (m ax k ao1 r t))
("mccowan" nil (m ax k aw1 ax n))
("mccowen" nil (m ax k aw1 ax n))
("mccowin" nil (m ax k aw1 ih0 n))
("mccown" nil (m ax k aw1 n))
("mccoy" nil (m ax k oy1))
("mccoys" nil (m ax k oy1 z))
("mccracken" nil (m ax k r ae1 k ax n))
("mccrackin" nil (m ax k r ae1 k ax n))
("mccrady" nil (m ax k r ey1 d iy0))
("mccrae" nil (m ax k r ey1))
("mccraney" nil (m ax k r ae1 n iy0))
("mccranie" nil (m ax k r ey1 n iy0))
("mccrary" nil (m ax k r eh1 r iy0))
("mccravy" nil (m ax k r ey1 v iy0))
("mccraw" nil (m ax k r ao1))
("mccray" nil (m ax k r ey1))
("mccrea" nil (m ax k r ey1))
("mccreadie" nil (m ax k r iy1 d iy0))
("mccready" nil (m ax k r iy1 d iy0))
("mccreary" nil (m ax k r ih1 r iy0))
("mccredie" nil (m ax k r iy1 d iy0))
("mccree" nil (m ax k r iy1))
("mccreedy" nil (m ax k r iy1 d iy0))
("mccreery" nil (m ax k r ih1 r iy0))
("mccreight" nil (m ax k r ey1 t))
("mccreless" nil (m ax k r iy1 l ax s))
("mccrickard" nil (m ax k r ih1 k er0 d))
("mccright" nil (m ax k r ay1 t))
("mccrillis" nil (m ax k r ih1 l ax s))
("mccrimmon" nil (m ax k r ih1 m ax n))
("mccrocklin" nil (m ax k r aa1 k l ax n))
("mccrone" nil (m ax k r ow1 n))
("mccrorey" nil (m ax k r ao1 r iy0))
("mccrory" nil (m ax k r ao1 r iy0))
("mccroskey" nil (m ax k r aa1 s k iy0))
("mccrossen" nil (m ax k r ao1 s ax n))
("mccrudden" nil (m ax k r ah1 d ax n))
("mccrum" nil (m ax k r ah1 m))
("mccrumb" nil (m ax k r ah1 m))
("mccrystal" nil (m ax k r ih1 s t ax l))
("mccuan" nil (m ax k uw1 ax n))
("mccubbin" nil (m ax k ah1 b ax n))
("mccubbins" nil (m ax k ah1 b ax n z))
("mccue" nil (m ax k y uw1))
("mccuen" nil (m ax k y uw1 n))
("mccuin" nil (m ax k uw1 ax n))
("mccuistion" nil (m ax k w ih1 sh ax n))
("mccuiston" nil (m ax k w ih1 s t ax n))
("mcculla" nil (m ax k ah1 l ax))
("mccullagh" nil (m ax k ah1 l ax g))
("mccullah" nil (m ax k ah1 l ax))
("mccullar" nil (m ax k ah1 l er0))
("mccullars" nil (m ax k ah1 l er0 z))
("mccullen" nil (m ax k ah1 l ax n))
("mcculler" nil (m ax k ah1 l er0))
("mccullers" nil (m ax k ah1 l er0 z))
("mcculley" nil (m ax k ah1 l iy0))
("mcculloch" nil (m ax k ah1 l ax k))
("mcculloh" nil (m ax k ah1 l ax))
("mccullough" nil (m ax k ah1 l ax))
("mccullum" nil (m ax k ah1 l ax m))
("mccully" nil (m ax k ah1 l iy0))
("mccumber" nil (m ax k ah1 m b er0))
("mccune" nil (m ax k y uw1 n))
("mccurdy" nil (m ax k er1 d iy0))
("mccurley" nil (m ax k er1 l iy0))
("mccurry" nil (m ax k er1 iy0))
("mccusker" nil (m ax k ah1 s k er0))
("mccutchan" nil (m ax k ah1 ch ax n))
("mccutchen" nil (m ax k ah1 ch ax n))
("mccutcheon" nil (m ax k ah1 ch ax n))
("mcdade" nil (m ax k d ey1 d))
("mcdaid" nil (m ax k d ey1 d))
("mcdanel" nil (m ax k d ae1 n ax l))
("mcdaniel" nil (m ax k d ae1 n y ax l))
("mcdaniels" nil (m ax k d ae1 n y ax l z))
("mcdannel" nil (m ax k d ae1 n ax l))
("mcdaris" nil (m ax k d ae1 r ax s))
("mcdavid" nil (m ax k d ey1 v ax d))
("mcdavitt" nil (m ax k d ae1 v ax t))
("mcdeal" nil (m ax k d iy1 l))
("mcdearmon" nil (m ax k d er1 m ax n))
("mcdermid" nil (m ax k d er1 m ax d))
("mcdermitt" nil (m ax k d er1 m ax t))
("mcdermott" nil (m ax k d er1 m ax t))
("mcdevitt" nil (m ax k d eh1 v ax t))
("mcdiarmid" nil (m ax k d eh1 r m ax d))
("mcdill" nil (m ax k d ih1 l))
("mcdivett" nil (m ax d ih1 v ax t))
("mcdivitt" nil (m ax k d ih1 v ax t))
("mcdole" nil (m ax k d ow1 l))
("mcdonagh" nil (m ax k d ah1 n ax))
("mcdonald" nil (m ax k d aa1 n ax l d))
("mcdonalds" nil (m ax k d aa1 n ax l d z))
("mcdonell" nil (m ax k d aa1 n ax l))
("mcdonnell" nil (m ax k d aa1 n ax l))
("mcdonough" nil (m ax k d ah1 n ax))
("mcdorman" nil (m ax k d ao1 r m ax n))
("mcdougal" nil (m ax k d uw1 g ax l))
("mcdougald" nil (m ax k d uw1 g ax l d))
("mcdougall" nil (m ax k d uw1 g ax l))
("mcdougals" nil (m ax k d uw1 g ax l z))
("mcdougle" nil (m ax k d uw1 g ax l))
("mcdow" nil (m ax k d aw1))
("mcdowall" nil (m ax k d aw1 ax l))
("mcdowell" nil (m ax k d aw1 ax l))
("mcduff" nil (m ax k d ah1 f))
("mcduffee" nil (m ax k d ah1 f iy0))
("mcduffie" nil (m ax k d ah1 f iy0))
("mcduffy" nil (m ax k d ah1 f iy0))
("mceachern" nil (m ax k iy1 ch er0 n))
("mceachin" nil (m ax k iy1 ch ax n))
("mcelderry" nil (m ae1 k ax l d ih1 r iy0))
("mceldowney" nil (m ae1 k ax l d aw1 n iy0))
("mcelfresh" nil (m ae1 k ax l f r eh1 sh))
("mcelhaney" nil (m ae1 k ax l hh ey1 n iy0))
("mcelhannon" nil (m ae1 k ax l hh ae1 n ax n))
("mcelhany" nil (m ae1 k ax l hh ey1 n iy0))
("mcelheney" nil (m ae1 k ax l hh ey1 n iy0))
("mcelheny" nil (m ae1 k ax l hh iy1 n iy0))
("mcelhiney" nil (m ae1 k ax l hh ih1 n iy0))
("mcelhinney" nil (m ae1 k ax l hh ih1 n iy0))
("mcelhinny" nil (m ae1 k ax l hh ih1 n iy0))
("mcelhone" nil (m ae1 k ax l hh ow1 n))
("mcelligott" nil (m ax k eh1 l ax g ax t))
("mcelmurray" nil (m ae1 k ax l m er1 iy0))
("mcelmurry" nil (m ae1 k ax l m er1 r iy0))
("mcelrath" nil (m ae1 k ax l r ae1 th))
("mcelravy" nil (m ae1 k ax l r ey1 v iy0))
("mcelreath" nil (m ae1 k ax l r ae1 th))
("mcelroy" nil (m ae1 k ax l r oy1))
("mcelvain" nil (m ae1 k ax l v ey1 n))
("mcelvaine" nil (m ae1 k ax l v ey1 n))
("mcelveen" nil (m ae1 k ax l v iy1 n))
("mcelwain" nil (m ae1 k ax l w ey1 n))
("mcelwaine" nil (m ae1 k ax l w ey1 n))
("mcelwee" nil (m ae1 k ax l w iy0))
("mcelyea" nil (m ae1 k ax l y ey1))
("mcenaney" nil (m ae1 k ax n ey1 n iy0))
("mcenany" nil (m ae1 k ax n ey1 n iy0))
("mcendree" nil (m ax k eh1 n d r iy1))
("mcenerney" nil (m ae1 k ax n er0 n iy0))
("mcenery" nil (m ax k eh1 n er0 iy0))
("mcenroe" nil (m ae1 k ax n r ow0))
("mcentee" nil (m ae1 k ax n t iy0))
("mcentire" nil (m ae1 k ax n t ay0 r))
("mcentyre" nil (m ae1 k ax n t ay0 r))
("mcerlean" nil (m ax k er1 l iy0 n))
("mceuen" nil (m ax k y uw1 ax n))
("mcever" nil (m ax k eh1 v er0))
("mcevers" nil (m ax k eh1 v er0 z))
("mcevilly" nil (m ax k eh1 v ax l iy0))
("mcevoy" nil (m ae1 k ax v oy1))
("mcewan" nil (m ae1 k uw0 ae0 n))
("mcewen" nil (m ax k y uw1 ax n))
("mcfadden" nil (m ax k f ae1 d ax n))
("mcfaddin" nil (m ax k f ae1 d ax n))
("mcfadin" nil (m ax k f ae1 d ax n))
("mcfadyen" nil (m ax k f ae1 d iy0 ax n))
("mcfall" nil (m ax k f ao1 l))
("mcfalls" nil (m ax k f ao1 l z))
("mcfann" nil (m ax k f ae1 n))
("mcfarlan" nil (m ax k f aa1 r l ax n))
("mcfarland" nil (m ax k f aa1 r l ax n d))
("mcfarlane" nil (m ax k f aa1 r l ax n))
("mcfarlin" nil (m ax k f aa1 r l ax n))
("mcfarling" nil (m ax k f aa1 r l ih0 ng))
("mcfarren" nil (m ax k f eh1 r ax n))
("mcfate" nil (m ax k f ey1 t))
("mcfatridge" nil (m ax k f ae1 t r ih0 jh))
("mcfatter" nil (m ax k f ae1 t er0))
("mcfaul" nil (m ax k f ao1 l))
("mcfayden" nil (m ax k f ey1 d ax n))
("mcfee" nil (m ax k f iy1))
("mcfeely" nil (m ax k f iy1 l iy0))
("mcfeeters" nil (m ax k f iy1 t er0 z))
("mcferran" nil (m ax k f eh1 r ax n))
("mcferren" nil (m ax k f eh1 r ax n))
("mcferrin" nil (m ax k f eh1 r ax n))
("mcferron" nil (m ax k f eh1 r ax n))
("mcfetridge" nil (m ax k f eh1 t r ih0 jh))
("mcgaffey" nil (m ax g ae1 f iy0))
("mcgagh" nil (m ax g ao1))
("mcgaha" nil (m ax g aa1 hh ax))
("mcgahan" nil (m ax g ae1 hh ax n))
("mcgahee" nil (m ax g ae1 hh iy0))
("mcgahey" nil (m ax g ae1 hh iy0))
("mcgalliard" nil (m ax g ae1 l iy0 aa0 r d))
("mcgann" nil (m ax g ae1 n))
("mcgannon" nil (m ax g ae1 n ax n))
("mcgarity" nil (m ax g ae1 r ax t iy0))
("mcgarr" nil (m ax g aa1 r))
("mcgarrah" nil (m ax g ae1 r ax))
("mcgarrigle" nil (m ax g ae1 r ax g ax l))
("mcgarrity" nil (m ax g ae1 r ax t iy0))
("mcgarry" nil (m ax g eh1 r iy0))
("mcgarvey" nil (m ax g aa0 r v ey1))
("mcgary" nil (m ax g eh1 r iy0))
("mcgath" nil (m ax g ae1 th))
("mcgaugh" nil (m ax g ao1))
("mcgaughey" nil (m ax g ao1 iy0))
("mcgaughy" nil (m ax g ao1 iy0))
("mcgauley" nil (m ax g ao1 l iy0))
("mcgavin" nil (m ax g ae1 v ax n))
("mcgavock" nil (m ax g ae1 v ax k))
("mcgaw" nil (m ax g ao1))
("mcgeachy" nil (m ax g iy1 ch iy0))
("mcgeary" nil (m ax g ih1 r iy0))
("mcgee" nil (m ax g iy1))
("mcgeehan" nil (m ax g iy1 hh ax n))
("mcgeever" nil (m ax g iy1 v er0))
("mcgegan" nil (m ax g eh1 g ax n))
("mcgeorge" nil (m ax k jh ao1 r jh))
("mcgeough" nil (m ax g ah1 f))
("mcgettigan" nil (m ax g eh1 t ax g ax n))
("mcghee" nil (m ax g iy1))
("mcghie" nil (m ax g iy1))
("mcgibbon" nil (m ax g ih1 b ax n))
("mcgill" nil (m ax g ih1 l))
("mcgillen" nil (m ax g ih1 l ax n))
("mcgillicuddy" nil (m ax g ih1 l ax k ax d iy0))
("mcgillis" nil (m ax g ih1 l ax s))
("mcgillivray" nil (m ax g ih1 l ax v r ey0))
("mcgilton" nil (m ax g ih1 l t ax n))
("mcgilvery" nil (m ax g ih1 l v er0 iy0))
("mcgilvray" nil (m ax g ih1 l v r iy0))
("mcginley" nil (m ax g ih1 n l iy0))
("mcginn" nil (m ax g ih1 n))
("mcginnes" nil (m ax g ih1 n z))
("mcginness" nil (m ax g ih1 n ax s))
("mcginnis" nil (m ax g ih1 n ax s))
("mcginniss" nil (m ax g ih1 n ax s))
("mcginnity" nil (m ax g ih1 n ax t iy0))
("mcginty" nil (m ax g ih1 n t iy0))
("mcgirr" nil (m ax g er1))
("mcgirt" nil (m ax g er1 t))
("mcgivern" nil (m ax g ih1 v er0 n))
("mcgivney" nil (m ax g ih1 v n iy0))
("mcglade" nil (m ax g l ey1 d))
("mcglamery" nil (m ax g l ae1 m er0 iy0))
("mcglashan" nil (m ax g l ae1 sh ax n))
("mcglasson" nil (m ax g l ae1 s ax n))
("mcglaughlin" nil (m ax g l ao1 f l ax n))
("mcglaun" nil (m ax g l ao1 n))
("mcglinchey" nil (m ax g l ih1 n ch iy0))
("mcglinn" nil (m ax g l ih1 n))
("mcglocklin" nil (m ax g l aa1 k l ax n))
("mcgloin" nil (m ax g l oy1 n))
("mcglone" nil (m ax g l ow1 n))
("mcglory" nil (m ax g l ao1 r iy0))
("mcglothen" nil (m ax g l aa1 th ax n))
("mcglothin" nil (m ax g l aa1 th ax n))
("mcglothlin" nil (m ax g l aa1 th l ax n))
("mcglynn" nil (m ax g l ih1 n))
("mcgoey" nil (m ax g aa1 iy0))
("mcgoff" nil (m ax g ao1 f))
("mcgoldrick" nil (m ax g ow1 l d r ax k))
("mcgols" nil (m ax k g aa1 l z))
("mcgonagle" nil (m ax g aa1 n ax g ax l))
("mcgonigal" nil (m ax g aa1 n ax g ax l))
("mcgonigle" nil (m ax g aa1 n ax g ax l))
("mcgough" nil (m ax g aw1))
("mcgourty" nil (m ax g uh1 r t iy0))
("mcgovern" nil (m ax g ah1 v er0 n))
("mcgowan" nil (m ax g aw1 ax n))
("mcgowen" nil (m ax g aw1 ax n))
("mcgowin" nil (m ax k g aw1 ax n))
("mcgown" nil (m ax g aw1 n))
("mcgrady" nil (m ax g r ey1 d iy0))
("mcgrail" nil (m ax g r ey1 l))
("mcgrain" nil (m ax g r ey1 n))
("mcgranahan" nil (m ax g r ae1 n ax hh ae1 n))
("mcgrane" nil (m ax g r ey1 n))
("mcgrath" nil (m ax g r ae1 th))
("mcgraw" nil (m ax g r ao1))
("mcgray" nil (m ax g r ey1))
("mcgreal" nil (m ax g r iy1 l))
("mcgreevey" nil (m ax g r iy1 v iy0))
("mcgreevy" nil (m ax g r iy1 v iy0))
("mcgregor" nil (m ax g r eh1 g er0))
("mcgregory" nil (m ax g r eh1 g er0 iy0))
("mcgrevin" nil (m ax g r eh1 v ax n))
("mcgrew" nil (m ax g r uw1))
("mcgriff" nil (m ax g r ih1 f))
("mcgroarty" nil (m ax g r ao1 r t iy0))
("mcgrogan" nil (m ax g r ow1 g ax n))
("mcgrory" nil (m ax g r ao1 r iy0))
("mcgruder" nil (m ax g r uw1 d er0))
("mcguane" nil (m ax g w ey1 n))
("mcguckin" nil (m ax g ah1 k ax n))
("mcgue" nil (m ax g y uw1))
("mcguffee" nil (m ax g ah1 f iy0))
("mcguffey" nil (m ax g ah1 f iy0))
("mcguffie" nil (m ax g ah1 f iy0))
("mcguffin" nil (m ax g ah1 f ax n))
("mcguigan" nil (m ax g ih1 g ax n))
("mcguiness" nil (m ax g ih1 n ax s))
("mcguinn" nil (m ax g ih1 n))
("mcguinness" nil (m ax g ih1 n ax s))
("mcguire" nil (m ax g w ay1 r))
("mcguirk" nil (m ax g er1 k))
("mcguirt" nil (m ax g er1 t))
("mcgurk" nil (m ax g er1 k))
("mcgurn" nil (m ax g er1 n))
("mcguyer" nil (m ax g ay1 er0))
("mcgwire" nil (m ax g w ay1 r))
("mchaffie" nil (m ax k ae1 f iy0))
("mchale" nil (m ax k ey1 l))
("mcham" nil (m ax k ae1 m))
("mchan" nil (m ax k ae1 n))
("mchaney" nil (m ax k ae1 n iy0))
("mchargue" nil (m ax k aa1 r g))
("mchatton" nil (m ax k ae1 t ax n))
("mchenry" nil (m ax k eh1 n r iy0))
("mchone" nil (m ax k ow1 n))
("mchugh" nil (m ax k y uw1))
("mcilhenny" nil (m ae1 k ih1 l hh eh1 n iy0))
("mcilrath" nil (m ae1 k ax l r ae1 th))
("mcilroy" nil (m ae1 k ax l r oy1))
("mcilvain" nil (m ae1 k ih1 l v ey1 n))
("mcilvaine" nil (m ae1 k ih1 l v ey1 n))
("mcilveen" nil (m ae1 k ih1 l v iy1 n))
("mcilwain" nil (m ae1 k ih1 l w ey1 n))
("mcinerney" nil (m ae1 k ax n er0 n iy0))
("mcinerny" nil (m ae1 k ax n er0 n iy0))
("mcingvale" nil (m ae1 k ih0 ng v ey1 l))
("mcinnes" nil (m ax g ih1 n ax s))
("mcinnis" nil (m ax g ih1 n ax s))
("mcinroy" nil (m ae1 k ih1 n r oy1))
("mcintee" nil (m ae1 k ih1 n t iy1))
("mcintire" nil (m ae1 k ih1 n t ay1 r))
("mcintosh" nil (m ae1 k ax n t ao1 sh))
("mcinturf" nil (m ae1 k ih1 n t er1 f))
("mcinturff" nil (m ae1 k ih1 n t er1 f))
("mcintyre" nil (m ae1 k ih1 n t ay1 r))
("mcinvale" nil (m ae1 k ih1 n v ey1 l))
("mcisaac" nil (m ax k ay1 z ax k))
("mciver" nil (m ax k ih1 v er0))
("mcivor" nil (m ax k ih1 v er0))
("mcjunkin" nil (m ax k jh ah1 ng k ax n))
("mcjunkins" nil (m ax k jh ah1 ng k ax n z))
("mckaig" nil (m ax k ey1 g))
("mckain" nil (m ax k ey1 n))
("mckamey" nil (m ax k ae1 m iy0))
("mckane" nil (m ax k ey1 n))
("mckanna" nil (m ax k ae1 n ax))
("mckay" nil (m ax k ey1))
("mckeag" nil (m ax k iy1 g))
("mckeague" nil (m ax k iy1 g))
("mckean" nil (m ax k iy1 n))
("mckeand" nil (m ax k iy1 n d))
("mckechnie" nil (m ax k eh1 k n iy0))
("mckee" nil (m ax k iy1))
("mckeegan" nil (m ax k iy1 g ax n))
("mckeehan" nil (m ax k iy1 hh ax n))
("mckeel" nil (m ax k iy1 l))
("mckeeman" nil (m ax k iy1 m ax n))
("mckeen" nil (m ax k iy1 n))
("mckeesport" nil (m ax k iy1 s p ao1 r t))
("mckeever" nil (m ax k iy1 v er0))
("mckeithan" nil (m ax k iy1 th ax n))
("mckeithen" nil (m ax k iy1 th ax n))
("mckell" nil (m ax k eh1 l))
("mckellar" nil (m ax k eh1 l er0))
("mckeller" nil (m ax k eh1 l er0))
("mckellips" nil (m ax k eh1 l ih0 p s))
("mckelvey" nil (m ae1 k ax l v ey1))
("mckelvie" nil (m ae1 k ax l v iy1))
("mckelvy" nil (m ae1 k ax l v iy1))
("mckemie" nil (m ax k eh1 m iy0))
("mckendree" nil (m ax k eh1 n d r iy0))
("mckendrick" nil (m ax k eh1 n d r ih0 k))
("mckendry" nil (m ax k eh1 n d r iy0))
("mckenna" nil (m ax k eh1 n ax))
("mckenney" nil (m ax k eh1 n iy0))
("mckennon" nil (m ax k eh1 n ax n))
("mckenny" nil (m ax k eh1 n iy0))
("mckenrick" nil (m ax k eh1 n r ih0 k))
("mckenzie" nil (m ax k eh1 n z iy0))
("mckeon" nil (m ax k iy1 ax n))
("mckeone" nil (m ax k iy1 ax n))
("mckeough" nil (m ax k iy1 ow0))
("mckeown" nil (m ax k y uw1 ax n))
("mckercher" nil (m ax k er1 ch er0))
("mckern" nil (m ax k er1 n))
("mckernan" nil (m ax k er1 n ax n))
("mckesson" nil (m ax k eh1 s ax n))
("mckethan" nil (m ax k eh1 th ax n))
("mckevitt" nil (m ax k eh1 v ax t))
("mckey" nil (m ax k iy1))
("mckibben" nil (m ax k ih1 b ax n))
("mckibbin" nil (m ax k ih1 b ax n))
("mckibbon" nil (m ax k ih1 b ax n))
("mckids" nil (m ax k ih1 d z))
("mckie" nil (m ax k iy1))
("mckiernan" nil (m ax k ih1 r n ax n))
("mckillip" nil (m ax k ih1 l ax p))
("mckillop" nil (m ax k ih1 l ax p))
("mckim" nil (m ax k ih1 m))
("mckimmey" nil (m ax k ih1 m iy0))
("mckimmy" nil (m ax k ih1 m iy0))
("mckinlay" nil (m ax k ih1 n l iy0))
("mckinley" nil (m ax k ih1 n l iy0))
("mckinney" nil (m ax k ih1 n iy0))
("mckinnie" nil (m ax k ih1 n iy0))
("mckinnis" nil (m ax k ih1 n ax s))
("mckinnon" nil (m ax k ih1 n ax n))
("mckinsey" nil (m ax k ih1 n z iy0))
("mckinstry" nil (m ax k ih1 n s t r iy0))
("mckinzie" nil (m ax k ih1 n z iy0))
("mckissack" nil (m ax k ih1 s ax k))
("mckissic" nil (m ax k ih1 s ih0 k))
("mckissick" nil (m ax k ih1 s ih0 k))
("mckitrick" nil (m ax k ih1 t r ih0 k))
("mckittrick" nil (m ax k ih1 t r ih0 k))
("mckneely" nil (m ax k n iy1 l iy0))
("mcknew" nil (m ax k n uw1))
("mcknight" nil (m ax k n ay1 t))
("mckone" nil (m ax k ow1 n))
("mckowen" nil (m ax k aw1 ax n))
("mckown" nil (m ax k ow1 n))
("mckoy" nil (m ax k oy1))
("mckune" nil (m ax k y uw1 n))
("mclachlan" nil (m ax k l aa1 k l ax n))
("mclafferty" nil (m ax k l ae1 f er0 t iy0))
("mclain" nil (m ax k l ey1 n))
("mclamb" nil (m ax k l ae1 m))
("mclanahan" nil (m ax k l ae1 n ax hh ae0 n))
("mclane" nil (m ax k l ey1 n))
("mclaren" nil (m ax k l eh1 r ax n))
("mclarney" nil (m ax k l aa1 r n iy0))
("mclarty" nil (m ax k l aa1 r t iy0))
("mclauchlin" nil (m ax k l ao1 k l ax n))
("mclaughlin" nil (m ax g l aa1 k l ax n))
("mclaurin" nil (m ax k l ao1 r ax n))
("mclaury" nil (m ax k l ao1 r iy0))
("mclawhorn" nil (m ax k l ae1 w er0 n))
("mclay" nil (m ax k l ey1))
("mclean" nil (m ax k l iy1 n))
("mclear" nil (m ax k l ih1 r))
("mcleary" nil (m ax k l ih1 r iy0))
("mclees" nil (m ax k l iy1 z))
("mcleish" nil (m ax k l iy1 sh))
("mcleland" nil (m ax k l eh1 l ax n d))
("mclellan" nil (m ax k l eh1 l ax n))
("mclelland" nil (m ax k l eh1 l ax n d))
("mclendon" nil (m ax k l eh1 n d ax n))
("mclennan" nil (m ax k l eh1 n ax n))
("mcleod" nil (m ax k l aw1 d))
("mcleroy" nil (m ax k l iy1 r oy0))
("mclerran" nil (m ax k l eh1 r ax n))
("mclester" nil (m ax k l eh1 s t er0))
("mclin" nil (m ax k l ih1 n))
("mclinden" nil (m ax k l ih1 n d ax n))
("mclinn" nil (m ax k l ih1 n))
("mclish" nil (m ax k l ih1 sh))
("mcloud" nil (m ax k l aw1 d))
("mclouth" nil (m ax k l aw1 th))
("mclucas" nil (m ax k l uw1 k ax z))
("mcluckie" nil (m ax k l ah1 k iy0))
("mclure" nil (m ax k l uw1 r))
("mcmackin" nil (m ax k m ae1 k ax n))
("mcmahan" nil (m ax k m ey1 hh ax n))
("mcmahen" nil (m ax k m ey1 hh ax n))
("mcmahill" nil (m ax k m ey1 hh ih1 l))
("mcmahon" nil (m ax k m ey1 ax n))
("mcmains" nil (m ax k m ey1 n z))
("mcmaken" nil (m ax k m ey1 k ax n))
("mcmakin" nil (m ax k m ae1 k ax n))
("mcmanama" nil (m ax k m ae1 n ax m ax))
("mcmanaman" nil (m ax k m ae1 n ax m ax n))
("mcmanamon" nil (m ax k m ae1 n ax m ax n))
("mcmanaway" nil (m ax k m ae1 n ax w ey0))
("mcmanigal" nil (m ax k m ae1 n ax g ax l))
("mcmanis" nil (m ax k m ae1 n ax s))
("mcmann" nil (m ax k m ae1 n))
("mcmannis" nil (m ax k m ae1 n ax s))
("mcmanus" nil (m ax k m ae1 n ax s))
("mcmartin" nil (m ax k m aa1 r t ax n))
("mcmaster" nil (m ax k m ae1 s t er0))
("mcmasters" nil (m ax k m ae1 s t er0 z))
("mcmath" nil (m ax k m ae1 th))
("mcmeans" nil (m ax k m iy1 n z))
("mcmeekin" nil (m ax k m iy1 k ax n))
("mcmeen" nil (m ax k m iy1 n))
("mcmenamin" nil (m ax k m eh1 n ax m ax n))
("mcmenamy" nil (m ax k m eh1 n ax m iy0))
("mcmenemy" nil (m ax k m eh1 n ax m iy0))
("mcmennamin" nil (m ax k m eh1 n ax m ax n))
("mcmichael" nil (m ax k m ay1 k ax l))
("mcmichen" nil (m ax k m ih1 ch ax n))
("mcmickle" nil (m ax k m ih1 k ax l))
("mcmillan" nil (m ax k m ih1 l ax n))
("mcmillen" nil (m ax k m ih1 l ax n))
("mcmiller" nil (m ax k m ih1 l er0))
("mcmillian" nil (m ax k m ih1 l y ax n))
("mcmillin" nil (m ax k m ih1 l ih0 n))
("mcmillion" nil (m ax k m ih1 l y ax n))
("mcmillon" nil (m ax k m ih1 l ax n))
("mcminn" nil (m ax k m ih1 n))
("mcmonagle" nil (m ax k m aa1 n ax g ax l))
("mcmonigle" nil (m ax k m aa1 n ax g ax l))
("mcmoran" nil (m ax k m ao1 r ax n))
("mcmorran" nil (m ax k m ao1 r ax n))
("mcmorris" nil (m ax k m ao1 r ax s))
("mcmorrow" nil (m ax k m ao1 r ow0))
("mcmuffin" nil (m ax k m ah1 f ax n))
("mcmullan" nil (m ax k m ah1 l ax n))
("mcmullen" nil (m ax k ah1 l ax n))
("mcmullin" nil (m ax k m ah1 l ax n))
("mcmunn" nil (m ax k m ah1 n))
("mcmurdo" nil (m ax k m er1 d ow0))
("mcmurphy" nil (m ax k m er1 f iy0))
("mcmurray" nil (m ax k m er1 ey0))
("mcmurrey" nil (m ax k m er1 iy0))
("mcmurry" nil (m ax k m er1 iy0))
("mcmurtrey" nil (m ax k m er1 t r iy0))
("mcmurtrie" nil (m ax k m er1 t er0 iy0))
("mcmurtry" nil (m ax k m er1 t r iy0))
("mcnab" nil (m ax k n ae1 b))
("mcnabb" nil (m ax k n ae1 b))
("mcnair" nil (m ax k n eh1 r))
("mcnairy" nil (m ax k n eh1 r iy0))
("mcnall" nil (m ax k n ao1 l))
("mcnalley" nil (m ax k n ae1 l iy0))
("mcnally" nil (m ax k n ae1 l iy0))
("mcnamara" nil (m ae1 k n ax m eh1 r ax))
("mcnamee" nil (m ae1 k n ax m iy0))
("mcnamer" nil (m ax k n ey1 m er0))
("mcnaney" nil (m ax k n ae1 n iy0))
("mcnary" nil (m ax k n eh1 r iy0))
("mcnatt" nil (m ax k n ae1 t))
("mcnaught" nil (m ax k n ao1 t))
("mcnaughton" nil (m ax k n ao1 t ax n))
("mcnay" nil (m ax k n ey1))
("mcneal" nil (m ax k n iy1 l))
("mcnealy" nil (m ax k n iy1 l iy0))
("mcnear" nil (m ax k n ih1 r))
("mcneary" nil (m ax k n ih1 r iy0))
("mcnease" nil (m ax k n iy1 z))
("mcnee" nil (m ax k n iy1))
("mcneece" nil (m ax k n iy1 s))
("mcneel" nil (m ax k n iy1 l))
("mcneeley" nil (m ax k n iy1 l iy0))
("mcneely" nil (m ax k n iy1 l iy0))
("mcneer" nil (m ax k n ih1 r))
("mcnees" nil (m ax k n iy1 z))
("mcneese" nil (m ax k n iy1 s))
("mcneff" nil (m ax k n eh1 f))
("mcneice" nil (m ax k n iy1 s))
("mcneil" nil (m ax k n iy1 l))
("mcneill" nil (m ax k n iy1 l))
("mcneilly" nil (m ax k n iy1 l iy0))
("mcneish" nil (m ax k n iy1 sh))
("mcnelis" nil (m ax k n eh1 l ax s))
("mcnellis" nil (m ax k n eh1 l ax s))
("mcnelly" nil (m ax k n eh1 l iy0))
("mcnemar" nil (m ae1 k n ax m aa1 r))
("mcnerney" nil (m ax k n er1 n iy0))
("mcnett" nil (m ax k n eh1 t))
("mcnevin" nil (m ax k n eh1 v ax n))
("mcnew" nil (m ax k n uw1))
("mcnichol" nil (m ax k n ih1 k ax l))
("mcnicholas" nil (m ax k n ih1 l ax l ax s))
("mcnichols" nil (m ax k n ih1 k ax l z))
("mcnickle" nil (m ax k n ih1 k ax l))
("mcnicol" nil (m ax k n ih1 k ao0 l))
("mcniel" nil (m ax k n iy1 l))
("mcniff" nil (m ax k n ih1 f))
("mcninch" nil (m ax k n ih1 n ch))
("mcnish" nil (m ax k n ih1 sh))
("mcnitt" nil (m ax k n ih1 t))
("mcnorton" nil (m ax k n ao1 r t ax n))
("mcnuggets" nil (m ax k n ah1 g ax t s))
("mcnulty" nil (m ax k n ah1 l t iy0))
("mcnutt" nil (m ax k n ah1 t))
("mcomber" nil (m ax k aa1 m b er0))
("mcorp" nil (eh1 m k ao1 r p))
("mcpaper" nil (m ax k p ey1 p er0))
("mcparland" nil (m ax k p aa1 r l ax n d))
("mcpartland" nil (m ax k p aa1 r t l ax n d))
("mcpartlin" nil (m ax k p aa1 r t l ax n))
("mcpeak" nil (m ax k p iy1 k))
("mcpeake" nil (m ax k p iy1 k))
("mcpeck" nil (m ax k p eh1 k))
("mcpeek" nil (m ax k p iy1 k))
("mcpeters" nil (m ax k p iy1 t er0 z))
("mcphail" nil (m ax k f ey1 l))
("mcphatter" nil (m ax k f ae1 t er0))
("mcphaul" nil (m ax k f ao1 l))
("mcphearson" nil (m ax k f er1 s ax n))
("mcphee" nil (m ax k f iy1))
("mcpheeters" nil (m ax k f iy1 t er0 z))
("mcpheron" nil (m ax k f eh1 r ax n))
("mcpherson" nil (m ax k f er1 s ax n))
("mcphie" nil (m ax k f iy1))
("mcphillips" nil (m ax k f ih1 l ax p s))
("mcpike" nil (m ax k p ay1 k))
("mcquade" nil (m ax k w ey1 d))
("mcquaid" nil (m ax k w ey1 d))
("mcquaide" nil (m ax k w ey1 d))
("mcquaig" nil (m ax k w ey1 g))
("mcquain" nil (m ax k w ey1 n))
("mcquarrie" nil (m ax k w ao1 r iy0))
("mcquary" nil (m iy1 k w eh0 r iy0))
("mcquay" nil (m ax k ey1))
("mcqueary" nil (m ax k w ih1 r iy0))
("mcqueen" nil (m ax k w iy1 n))
("mcqueeney" nil (m ax k w iy1 n iy0))
("mcquerry" nil (m ax k w eh1 r iy0))
("mcquigg" nil (m ax k w ih1 g))
("mcquilkin" nil (m ax k w ih1 l k ax n))
("mcquillan" nil (m ax k w ih1 l ax n))
("mcquillen" nil (m ax k w ih1 l ax n))
("mcquillin" nil (m ax k w ih1 l ax n))
("mcquinn" nil (m ax k w ih1 n))
("mcquire" nil (m ax k w ay1 r))
("mcquiston" nil (m ax k w ih1 s t ax n))
("mcquitty" nil (m ax k w ih1 t iy0))
("mcquown" nil (m ax k w aw1 n))
("mcrae" nil (m ax k r ey1))
("mcraney" nil (m ax k r ae1 n iy0))
("mcray" nil (m ax k r ey1))
("mcree" nil (m ax k r iy1))
("mcreynolds" nil (m ax k r ey1 n ax l d z))
("mcright" nil (m ax k r ay1 t))
("mcroberts" nil (m ax k r aa1 b er0 t s))
("mcrorie" nil (m ax k r ao1 r iy0))
("mcroy" nil (m ax k r oy1))
("mcshan" nil (m ax k sh ae1 n))
("mcshane" nil (m ax k sh ey1 n))
("mcshea" nil (m ax k sh ey1))
("mcsherry" nil (m ax k sh eh1 r iy0))
("mcsleep" nil (m ax k s l iy1 p))
("mcsorley" nil (m ax k s ao1 r l iy0))
("mcspadden" nil (m ax k s p ae1 d ax n))
("mcstay" nil (m ax k s t ey1))
("mcswain" nil (m ax k s w ey1 n))
("mcsween" nil (m ax k s w iy1 n))
("mcsweeney" nil (m ax k s w iy1 n iy0))
("mctaggart" nil (m ax k t ae1 g er0 t))
("mctague" nil (m ax k t ey1 g))
("mctavish" nil (m ax k t ey1 v ih0 sh))
("mcteer" nil (m ax k t ih1 r))
("mcternan" nil (m ax k t er1 n ax n))
("mctier" nil (m ax k t ay1 er0))
("mctiernan" nil (m ax k t ay1 r n ax n))
("mctighe" nil (m ax k t ay1 g))
("mctigue" nil (m ax k t iy1 g))
("mcvay" nil (m ax k v ey1))
("mcvea" nil (m ax k v iy1))
("mcveigh" nil (m ax k v ey1))
("mcvey" nil (m ax k v ey1))
("mcvicar" nil (m ax k v ih1 k er0))
("mcvicker" nil (m ax k v ih1 k er0))
("mcvoy" nil (m ax k v oy1))
("mcwain" nil (m ax k w ey1 n))
("mcwaters" nil (m ax k w ao1 t er0 z))
("mcwatters" nil (m ax k w ao1 t er0 z))
("mcweeney" nil (m ax k w iy1 n iy0))
("mcwethy" nil (m ax k w eh1 th iy0))
("mcwherter" nil (m ax k w er1 t er0))
("mcwhinney" nil (m ax k w ih1 n iy0))
("mcwhirt" nil (m ax k w er1 t))
("mcwhirter" nil (m ax k w er1 t er0))
("mcwhite" nil (m ax k w ay1 t))
("mcwhorter" nil (m ax k w ao1 r t er0))
("mcwilliam" nil (m ax k w ih1 l y ax m))
("mcwilliams" nil (m ax k w ih1 l y ax m z))
("mcwright" nil (m ax k r ay1 t))
("mczeal" nil (m ax k z iy1 l))
("me" nil (m iy1))
("mea" nil (m iy1))
("meacham" nil (m iy1 ch ax m))
("meachum" nil (m iy1 ch ax m))
("mead" nil (m iy1 d))
("meade" nil (m iy1 d))
("meader" nil (m iy1 d er0))
("meaders" nil (m iy1 d er0 z))
("meador" nil (m iy1 d er0))
("meadors" nil (m iy1 d er0 z))
("meadow" nil (m eh1 d ow1))
("meadowlands" nil (m eh1 d ow0 l ae1 n d z))
("meadowlark" nil (m eh1 d ow0 l aa1 r k))
("meadows" nil (m eh1 d ow1 z))
("meads" nil (m iy1 d z))
("meager" nil (m iy1 g er0))
("meagher" nil (m aa1 r))
("meaker" nil (m iy1 k er0))
("meakin" nil (m iy1 k ih0 n))
("meal" nil (m iy1 l))
("mealer" nil (m iy1 l er0))
("mealey" nil (m iy1 l iy0))
("mealing" nil (m iy1 l ih0 ng))
("mealor" nil (m iy1 l er0))
("meals" nil (m iy1 l z))
("mealtime" nil (m iy1 l t ay1 m))
("mealy" nil (m iy1 l iy0))
("mealynosed" nil (m iy1 l iy0 n ow1 z d))
("mean" nil (m iy1 n))
("meander" nil (m iy0 ae1 n d er0))
("meandered" nil (m iy0 ae1 n d er0 d))
("meandering" nil (m iy0 ae1 n d er0 ih0 ng))
("meanders" nil (m iy0 ae1 n d er0 z))
("meaner" nil (m iy1 n er0))
("meanest" nil (m iy1 n ax s t))
("meaney" nil (m iy1 n iy0))
("meaning" nil (m iy1 n ih0 ng))
("meaningful" nil (m iy1 n ih0 ng f ax l))
("meaningfully" nil (m iy1 n ih0 ng f ax l iy0))
("meaningless" nil (m iy1 n ih0 ng l ax s))
("meanings" nil (m iy1 n ih0 ng z))
("meanness" nil (m iy1 n n ax s))
("meanor" nil (m iy1 n er0))
("means" nil (m iy1 n z))
("meant" nil (m eh1 n t))
("meantime" nil (m iy1 n t ay1 m))
("meanwhile" nil (m iy1 n w ay1 l))
("meany" nil (m iy1 n iy0))
("mear" nil (m ih1 r))
("meara" nil (m iy1 r ax))
("meares" nil (m iy1 r z))
("mearns" nil (m er1 n z))
("mears" nil (m ih1 r z))
("mease" nil (m iy1 z))
("measel" nil (m iy1 z ax l))
("measles" nil (m iy1 z ax l z))
("measly" nil (m iy1 z l iy0))
("measurable" nil (m eh1 zh er0 ax b ax l))
("measurably" nil (m eh1 zh er0 ax b l iy0))
("measure" nil (m eh1 zh er0))
("measured" nil (m eh1 zh er0 d))
("measurement" nil (m eh1 zh er0 m ax n t))
("measurements" nil (m eh1 zh er0 m ax n t s))
("measures" nil (m eh1 zh er0 z))
("measurex" nil (m eh1 z er0 ax k s))
("measuring" nil (m eh1 zh er0 ih0 ng))
("meat" nil (m iy1 t))
("meatball" nil (m iy1 t b ao1 l))
("meatballs" nil (m iy1 t b ao1 l z))
("meath" nil (m iy1 th))
("meatier" nil (m iy1 t iy0 er0))
("meatpacker" nil (m iy1 t p ae1 k er0))
("meatpackers" nil (m iy1 t p ae1 k er0 z))
("meatpacking" nil (m iy1 t p ae1 k ih0 ng))
("meats" nil (m iy1 t s))
("meaty" nil (m iy1 t iy0))
("meaux" nil (m ow1))
("meave" nil (m iy1 v))
("mebane" nil (m eh1 b ax n))
("meca" nil (m eh1 k ax))
("mecca" nil (m eh1 k ax))
("mech" nil (m eh1 k))
("mecham" nil (m eh1 ch ax m))
("mechanic" nil (m ax k ae1 n ih0 k))
("mechanical" nil (m ax k ae1 n ih0 k ax l))
("mechanically" nil (m ax k ae1 n ih0 k l iy0))
("mechanics" nil (m ax k ae1 n ih0 k s))
("mechanicsburg" nil (m ax k ae1 n ih0 k s b er0 g))
("mechanism" nil (m eh1 k ax n ih1 z ax m))
("mechanisms" nil (m eh1 k ax n ih1 z ax m z))
("mechanistic" nil (m eh1 k ax n ih1 s t ih0 k))
("mechanization" nil (m eh1 k ax n ax z ey1 sh ax n))
("mechanized" nil (m eh1 k ax n ay1 z d))
("meche" nil (m eh1 ch))
("mechem" nil (m eh1 k ih0 m))
("mechler" nil (m eh1 k l er0))
("mechling" nil (m eh1 k l ih0 ng))
("meciar" nil (m eh1 s iy0 aa0 r))
("meck" nil (m eh1 k))
("meckel" nil (m eh1 k ax l))
("meckes" nil (m eh1 k s))
("mecklenburg" nil (m eh1 k l ax n b er0 g))
("meckler" nil (m eh1 k l er0))
("meckley" nil (m eh1 k l iy0))
("meckstroth" nil (m eh1 k s t r ao0 th))
("mecum" nil (m eh1 k ax m))
("med" nil (m eh1 d))
("meda" nil (m ey1 d ax))
("medaglia" nil (m eh0 d aa1 g l iy0 ax))
("medal" nil (m eh1 d ax l))
("medalist" nil (m eh1 d ax l ih0 s t))
("medalists" nil (m eh1 d ax l ih0 s t s))
("medallion" nil (m ax d ae1 l y ax n))
("medallions" nil (m ax d ae1 l y ax n z))
("medals" nil (m eh1 d ax l z))
("medaphis" nil (m eh1 d ax f ih1 s))
("medar" nil (m eh1 d er0))
("medarex" nil (m eh1 d er0 eh1 k s))
("medaris" nil (m ey0 d aa1 r ih0 s))
("medcalf" nil (m eh1 d k ae0 l f))
("medchem" nil (m eh1 d k eh1 m))
("medco" nil (m eh1 d k ow0))
("meddaugh" nil (m eh1 d ao0))
("medders" nil (m eh1 d er0 z))
("meddle" nil (m eh1 d ax l))
("meddlesome" nil (m eh1 d ax l s ax m))
("meddling" nil (m eh1 d ax l ih0 ng))
("medea" nil (m ax d iy1 ax))
("medearis" nil (m eh1 d er0 ih0 s))
("medeiros" nil (m ey0 d ih1 r ow0 z))
("medel" nil (m eh1 d ax l))
("medellin" nil (m ih0 d eh1 l ih0 n))
("medema" nil (m eh0 d eh1 m ax))
("medendorp" nil (m eh1 d eh0 n d ao0 r p))
("meder" nil (m iy1 d er0))
("mederos" nil (m eh1 d er0 ow0 z))
("medes" nil (m iy1 d z))
("medeva" nil (m eh1 d eh1 v ax))
("medex" nil (m eh1 d ax k s))
("medfacts" nil (m eh1 d f ae1 k t s))
("medfirst" nil (m eh1 d f er1 s t))
("medfly" nil (m eh1 d f l iy0))
("medford" nil (m eh1 d f er0 d))
("medgar" nil (m eh1 d g er0))
("medi" nil (m eh1 d iy0))
("media" nil (m iy1 d iy0 ax))
("mediamark" nil (m iy1 d iy0 ax m aa1 r k))
("median" nil (m iy1 d iy0 ax n))
("medianews" nil (m iy1 d iy0 ax y uw1 z))
("mediaset" nil (m iy1 d iy0 ax s eh1 t))
("mediate" nil (m iy1 d iy0 ey1 t))
("mediated" nil (m iy1 d iy0 ey1 t ih0 d))
("mediating" nil (m iy1 d iy0 ey1 t ih0 ng))
("mediation" nil (m iy1 d iy0 ey1 sh ax n))
("mediator" nil (m iy1 d iy0 ey1 t er0))
("mediators" nil (m iy1 d iy0 ey1 t er0 z))
("medic" nil (m eh1 d ih0 k))
("medicaid" nil (m eh1 d ax k ey1 d))
("medical" nil (m eh1 d ax k ax l))
("medically" nil (m eh1 d ax k l iy0))
("medicare" nil (m eh1 d ax k eh1 r))
("medicate" nil (m eh1 d ih0 k ey1 t))
("medication" nil (m eh1 d ax k ey1 sh ax n))
("medications" nil (m eh1 d ax k ey1 sh ax n z))
("medici" nil (m eh0 d iy1 s iy0))
("medicinal" nil (m ax d ih1 s ax n ax l))
("medicinally" nil (m ax d ih1 s ax n ax l iy0))
("medicine" nil (m eh1 d ax s ax n))
("medicines" nil (m eh1 d ax s ax n z))
("medico" nil (m eh1 d ih0 k ow1))
("medics" nil (m eh1 d ih0 k s))
("medicus" nil (m eh1 d ih0 k ax s))
("medieval" nil (m ih0 d iy1 v ax l))
("medigap" nil (m eh1 d ih0 g ae0 p))
("medimmune" nil (m eh1 d ih0 m y uw1 n))
("medin" nil (m ey0 d iy1 n))
("medina" nil (m ax d ay1 n ax))
("medinger" nil (m iy1 d ih0 ng er0))
("medio" nil (m iy1 d iy0 ow0))
("mediobanca" nil (m ih0 d iy1 ow0 b ae1 ng k ax))
("mediocre" nil (m iy1 d iy0 ow1 k er0))
("mediocrity" nil (m iy1 d iy0 aa1 k r ax t iy0))
("mediplex" nil (m eh1 d ih0 p l eh1 k s))
("mediq" nil (m eh0 d iy1 k))
("medisgroups" nil (m eh1 d ih0 s g r uw1 p s))
("meditate" nil (m eh1 d ax t ey1 t))
("meditating" nil (m eh1 d ax t ey1 t ih0 ng))
("meditation" nil (m eh1 d ax t ey1 sh ax n))
("meditations" nil (m eh1 d ih0 t ey1 sh ax n z))
("meditative" nil (m eh1 d ax t ey1 t ih0 v))
("mediterranean" nil (m eh1 d ax t er0 ey1 n iy0 ax n))
("meditrust" nil (m eh1 d ih0 t r ah1 s t))
("meditz" nil (m eh1 d ih0 t s))
("medium" nil (m iy1 d iy0 ax m))
("mediums" nil (m iy1 d iy0 ax m z))
("medland" nil (m eh1 d l ax n d))
("medlar" nil (m eh1 d l er0))
("medlen" nil (m eh1 d ax l ax n))
("medler" nil (m eh1 d l er0))
("medley" nil (m eh1 d l iy0))
("medlin" nil (m eh1 d l ih0 n))
("medlock" nil (m eh1 d l ax k))
("mednick" nil (m eh1 d n ih0 k))
("medoff" nil (m eh1 d ao0 f))
("medora" nil (m ey0 d ao1 r ax))
("medrano" nil (m eh0 d r aa1 n ow0))
("medserv" nil (m eh1 d s er0 v))
("medsker" nil (m eh1 d s k er0))
("medstone" nil (m eh1 d s t ow1 n))
("medtronic" nil (m eh0 d t r aa1 n ih0 k))
("medulla" nil (m ih0 d ah1 l ax))
("medusa" nil (m ax d uw1 s ax))
("medusas" nil (m ax d uw1 s ax z))
("medved" nil (m eh1 d v d))
("medvedev" nil (m eh1 d v ax d eh1 v))
("medwin" nil (m eh1 d w ih0 n))
("mee" nil (m iy1))
("meece" nil (m iy1 s))
("meech" nil (m iy1 ch))
("meeder" nil (m iy1 d er0))
("meegan" nil (m iy1 g ax n))
("meehan" nil (m iy1 ax n))
("meehans" nil (m iy1 hh ae1 n z))
("meehl" nil (m iy1 l))
("meek" nil (m iy1 k))
("meeker" nil (m iy1 k er0))
("meekins" nil (m iy1 k ih0 n z))
("meekly" nil (m iy1 k l iy0))
("meeks" nil (m iy1 k s))
("meeler" nil (m iy1 l er0))
("meenaghan" nil (m iy1 n ax hh ae0 n))
("meenan" nil (m iy1 n ax n))
("meents" nil (m iy1 n t s))
("meer" nil (m iy1 er0))
("meers" nil (m iy1 er0 z))
("mees" nil (m iy1 z))
("meese" nil (m iy1 s))
("meester" nil (m iy1 s t er0))
("meet" nil (m iy1 t))
("meeting" nil (m iy1 t ih0 ng))
("meetings" nil (m iy1 t ih0 ng z))
("meets" nil (m iy1 t s))
("meetze" nil (m iy1 t z))
("meeuwsen" nil (m iy1 uw1 s ax n))
("mefferd" nil (m eh1 f er0 d))
("meffert" nil (m eh1 f er0 t))
("mefford" nil (m eh1 f er0 d))
("meg" nil (m eh1 g))
("mega" nil (m eh1 g ax))
("megabit" nil (m eh1 g ax b ih0 t))
("megabucks" nil (m eh1 g ax b ah1 k s))
("megabyte" nil (m eh1 g ax b ay1 t))
("megabytes" nil (m eh1 g ax b ay1 t s))
("megacarriers" nil (m eh1 g ax k ae1 r y er0 z))
("megace" nil (m iy1 g ax s))
("megadeal" nil (m eh1 g ax d iy1 l))
("megadeals" nil (m eh1 g ax d iy1 l z))
("megadeath" nil (m eh1 g ax d eh1 th))
("megafoods" nil (m eh1 g ax f uw1 d z))
("megahertz" nil (m eh1 g ax hh er0 t s))
("megahouse" nil (m eh1 g ax hh aw1 s))
("megahouses" nil (m eh1 g ax hh aw1 s ih0 z))
("megalomania" nil (m eh1 g ax l ow0 m ey1 n iy0 ax))
("megalomaniac" nil (m eh1 g ax l ow0 m ey1 n iy0 ae1 k))
("megalopolis" nil (m eh1 g ax l aa1 p ax l ax s))
("megamergers" nil (m eh1 g ax m er1 jh er0 z))
("megan" nil (m ey1 g ax n))
("megaphone" nil (m eh1 g ax f ow1 n))
("megaphones" nil (m eh1 g ax f ow1 n z))
("megaquest" nil (m eh1 g ax k w eh1 s t))
("megarry" nil (m eh1 g er0 iy0))
("megastores" nil (m eh1 g ax s t ao1 r z))
("megawatt" nil (m eh1 g ax w aa1 t))
("megawatts" nil (m eh1 g ax w aa1 t s))
("megee" nil (m eh1 jh iy0))
("meger" nil (m eh1 g er0))
("megginson" nil (m eh1 g ih0 n s ax n))
("meggison" nil (m eh1 g ih0 s ax n))
("meggs" nil (m eh1 g z))
("meghdar" nil (m eh1 g d aa0 r))
("megill" nil (m eh1 jh ax l))
("megna" nil (m eh1 g n ax))
("mehaffey" nil (m eh1 hh ax f iy0))
("mehall" nil (m ax hh ao1 l))
("mehan" nil (m ey1 hh ae0 n))
("meharg" nil (m ey1 hh aa1 r g))
("mehdi" nil (m eh1 d iy0))
("mehetabel" nil (m ih0 hh eh1 t ax b ih0 l))
("mehitabel" nil (m eh1 hh ih0 t ax b ax l))
("mehitabelle" nil (m eh1 hh ih0 t ax b ax l))
("mehl" nil (m eh1 l))
("mehlberg" nil (m eh1 l b er0 g))
("mehle" nil (m eh1 hh ax l))
("mehlenbacher" nil (m eh1 l ih0 n b aa0 k er0))
("mehler" nil (m eh1 l er0))
("mehlhaff" nil (m eh1 l hh ax f))
("mehlhoff" nil (m eh1 l hh ao0 f))
("mehlhorn" nil (m eh1 l hh er0 n))
("mehling" nil (m eh1 l ih0 ng))
("mehlman" nil (m eh1 l m ax n))
("mehmet" nil (m eh1 m ax t))
("mehner" nil (m eh1 n er0))
("mehnert" nil (m eh1 n er0 t))
("mehr" nil (m eh1 r))
("mehrabian" nil (m eh1 r ey1 b iy0 ax n))
("mehran" nil (m eh1 r ax n))
("mehrens" nil (m eh1 r ax n z))
("mehrer" nil (m eh1 r er0))
("mehring" nil (m eh1 r ih0 ng))
("mehringer" nil (m eh1 r ih0 ng er0))
("mehrtens" nil (m eh1 r t ax n z))
("mehta" nil (m eh1 t ax))
("mei" nil (m ay1))
("meidinger" nil (m ay1 d ih0 ng er0))
("meidl" nil (m iy1 d ax l))
("meier" nil (m ay1 er0))
("meierfeld" nil (m ay1 r f eh1 l d))
("meighan" nil (m ey1 g hh ax n))
("meigher" nil (m ey1 g er0))
("meiji" nil (m ey1 jh iy1))
("meikle" nil (m iy1 k ax l))
("mein" nil (m iy1 n))
("meincke" nil (m ay1 ng k))
("meinders" nil (m ay1 n d er0 z))
("meindl" nil (m ay1 n d ax l))
("meine" nil (m iy1 n))
("meinecke" nil (m ay1 n ih0 k))
("meineke" nil (m ay1 n ih0 k))
("meinen" nil (m ay1 n ax n))
("meiner" nil (m ay1 n er0))
("meiners" nil (m ay1 n er0 z))
("meinert" nil (m ay1 n er0 t))
("meinertzhagen" nil (m ay1 n er0 t s hh aa1 g ax n))
("meinhardt" nil (m ay1 n hh aa1 r t))
("meinhart" nil (m ay1 n hh aa1 r t))
("meinhold" nil (m ay1 n hh ow1 l d))
("meininger" nil (m ay1 n ih0 ng er0))
("meinke" nil (m iy1 ng k))
("meints" nil (m ay1 n t s))
("meinzer" nil (m ay1 n z er0))
("meiosis" nil (m ay0 ow1 s ax s))
("meir" nil (m ih1 r))
("meiring" nil (m ay1 r ih0 ng))
("meis" nil (m iy1 z))
("meisch" nil (m ay1 sh))
("meise" nil (m iy1 s))
("meisel" nil (m ay1 s ax l))
("meisels" nil (m ay1 s ax l z))
("meisenheimer" nil (m ay1 s ih0 n hh ay0 m er0))
("meiser" nil (m ay1 s er0))
("meishan" nil (m ay1 sh ax n))
("meisinger" nil (m ay1 s ih0 n jh er0))
("meisler" nil (m ay1 s ax l er0))
("meisner" nil (m ay1 s n er0))
("meiss" nil (m ay1 s))
("meissner" nil (m ay1 s n er0))
("meister" nil (m ay1 s t er0))
("meitz" nil (m iy1 t s))
("meitzler" nil (m ay1 t s l er0))
("meixner" nil (m iy1 k s n er0))
("meiyuh" nil (m ey0 y uw1))
("mejia" nil (m ey1 y iy0 ax))
("mejias" nil (m ey0 y iy1 ax z))
("mekeel" nil (m eh1 k iy0 l))
("mekong" nil (m ey1 k aa0 ng))
("mel" nil (m eh1 l))
("melamed" nil (m eh1 l ax m eh0 d))
("melamine" nil (m eh1 l ax m iy1 n))
("melancholic" nil (m eh1 l ax n k aa1 l ih0 k))
("melancholy" nil (m eh1 l ax n k aa1 l iy0))
("melancon" nil (m ih0 l ae1 n k ax n))
("meland" nil (m eh1 l ax n d))
("melander" nil (m eh1 l ax n d er0))
("melanesian" nil (m eh1 l ax n iy1 z ax n))
("melanesians" nil (m eh1 l ax n iy1 s iy0 ax n z))
("melange" nil (m eh1 l ae0 ng))
("melanie" nil (m eh1 l ax n iy0))
("melanin" nil (m eh1 l ax n ax n))
("melanoma" nil (m eh1 l ax n ow1 m ax))
("melanson" nil (m eh1 l ax n s ax n))
("melantha" nil (m ih0 l ae1 n dh ax))
("melany" nil (m eh1 l ax n iy0))
("melaragno" nil (m eh0 l aa0 r aa1 g n ow0))
("melatonin" nil (m eh1 l ax t ow1 n ax n))
("melba" nil (m eh1 l b ax))
("melberg" nil (m eh1 l b er0 g))
("melbourne" nil (m eh1 l b er0 n))
("melburn" nil (m eh1 l b er0 n))
("melby" nil (m eh1 l b iy0))
("melcher" nil (m eh1 l ch er0))
("melchert" nil (m eh1 l ch er0 t))
("melching" nil (m eh1 l ch ih0 ng))
("melchior" nil (m ey0 l ch iy1 er0))
("melchiorre" nil (m eh0 l k iy0 ao1 r ey0))
("melchor" nil (m eh1 l ch er0))
("meld" nil (m eh1 l d))
("melded" nil (m eh1 l d ax d))
("melder" nil (m eh1 l d er0))
("melding" nil (m eh1 l d ih0 ng))
("meldon" nil (m eh1 l d ax n))
("meldrum" nil (m eh1 l d r ax m))
("mele" nil (m iy1 l))
("melear" nil (m eh1 l er0))
("melee" nil (m ey1 l ey1))
("melendez" nil (m ax l eh1 n d eh0 z))
("melendrez" nil (m ey0 l ey1 n d r eh0 z))
("melendy" nil (m ih0 l eh1 n d iy0))
("melero" nil (m ey0 l eh1 r ow0))
("meleski" nil (m ih0 l eh1 s k iy0))
("melessa" nil (m eh0 l eh1 s ax))
("melfi" nil (m eh1 l f iy0))
("melgaard" nil (m eh1 l g aa1 r d))
("melgar" nil (m ey0 l g aa1 r))
("melgoza" nil (m eh0 l g ow1 z ax))
("melhorn" nil (m eh1 l hh er0 n))
("meli" nil (m eh1 l iy0))
("melia" nil (m eh1 l iy0 ax))
("melican" nil (m eh1 l ih0 k ax n))
("melicent" nil (m eh1 l ih0 s ax n t))
("melichar" nil (m eh1 l ih0 k er0))
("melick" nil (m eh1 l ih0 k))
("melikian" nil (m ih0 l ih1 k iy0 ax n))
("melillo" nil (m eh0 l ih1 l ow0))
("melin" nil (m eh1 l ih0 n))
("melina" nil (m eh0 l iy1 n ax))
("melinda" nil (m ax l ih1 n d ax))
("meline" nil (m eh1 l ay0 n))
("meling" nil (m eh1 l ih0 ng))
("melisent" nil (m eh1 l ih0 s ax n t))
("melissa" nil (m ax l ih1 s ax))
("melisse" nil (m eh1 l ih0 s))
("melita" nil (m eh0 l iy1 t ax))
("melito" nil (m eh0 l iy1 t ow0))
("melitta" nil (m eh0 l iy1 t ax))
("melius" nil (m iy1 l iy0 ih0 s))
("melka" nil (m eh1 l k ax))
("melkar" nil (m eh1 l k aa1 r))
("melkonian" nil (m eh1 l k ow1 n iy0 ax n))
("mell" nil (m eh1 l))
("mella" nil (m eh1 l ax))
("melland" nil (m eh1 l ax n d))
("mellaril" nil (m eh1 l er0 ax l))
("melle" nil (m eh1 l))
("mellem" nil (m eh1 l ih0 m))
("mellema" nil (m eh0 l eh1 m ax))
("mellen" nil (m eh1 l ax n))
("meller" nil (m eh1 l er0))
("mellett" nil (m eh1 l ih0 t))
("melley" nil (m eh1 l iy0))
("mellgren" nil (m eh1 l g r eh0 n))
("melli" nil (m eh1 l iy0))
("mellicent" nil (m ey0 l iy1 s ax n t))
("mellick" nil (m eh1 l ih0 k))
("mellie" nil (m eh1 l iy0))
("mellin" nil (m eh1 l ih0 n))
("melling" nil (m eh1 l ih0 ng))
("mellinger" nil (m eh1 l ih0 ng er0))
("mellis" nil (m eh1 l ih0 s))
("mellish" nil (m eh1 l ih0 sh))
("mellman" nil (m eh1 l m ax n))
("mello" nil (m eh1 l ow0))
("melloan" nil (m eh0 l ow1 n))
("mellon" nil (m eh1 l ax n))
("mellons" nil (m eh1 l ax n z))
("mellor" nil (m eh1 l er0))
("mellott" nil (m eh1 l ax t))
("mellow" nil (m eh1 l ow0))
("mellowed" nil (m eh1 l ow0 d))
("mellowing" nil (m eh1 l ow0 ih0 ng))
("melly" nil (m eh1 l iy0))
("melman" nil (m eh1 l m ax n))
("melnick" nil (m eh1 l n ih0 k))
("melnik" nil (m eh1 l n ih0 k))
("melnor" nil (m eh1 l n er0))
("melnyk" nil (m eh1 l n ih0 k))
("melo" nil (m eh1 l ow0))
("meloche" nil (m eh0 l ow1 k iy0))
("melodic" nil (m ax l aa1 d ih0 k))
("melodies" nil (m eh1 l ax d iy0 z))
("melodious" nil (m ax l ow1 d iy0 ax s))
("melodrama" nil (m eh1 l ax d r aa1 m ax))
("melodramas" nil (m eh1 l ax d r aa1 m ax z))
("melodramatic" nil (m eh1 l ax d r ax m ae1 t ih0 k))
("melody" nil (m eh1 l ax d iy0))
("melon" nil (m eh1 l ax n))
("melone" nil (m eh0 l ow1 n iy0))
("meloni" nil (m eh0 l ow1 n iy0))
("melons" nil (m eh1 l ax n z))
("meloy" nil (m eh1 l oy0))
("melquist" nil (m eh1 l k w ih1 s t))
("melridge" nil (m eh1 l r ih1 jh))
("melrod" nil (m eh1 l r aa1 d))
("melrose" nil (m eh1 l r ow1 z))
("melroy" nil (m eh1 l r oy1))
("melson" nil (m eh1 l s ax n))
("melt" nil (m eh1 l t))
("meltdown" nil (m eh1 l t d aw1 n))
("melted" nil (m eh1 l t ax d))
("melting" nil (m eh1 l t ih0 ng))
("melton" nil (m eh1 l t ax n))
("melts" nil (m eh1 l t s))
("meltwater" nil (m eh1 l t w aa1 t er0))
("meltz" nil (m eh1 l t s))
("meltzer" nil (m eh1 l t s er0))
("melucci" nil (m eh0 l uw1 ch iy0))
("melugin" nil (m eh1 l ax g ih0 n))
("melva" nil (m eh1 l v ax))
("melvie" nil (m eh1 l v iy0))
("melville" nil (m eh1 l v ih0 l))
("melvin" nil (m eh1 l v ih0 n))
("melvina" nil (m eh0 l v iy1 n ax))
("melvine" nil (m eh1 l v ay1 n))
("melvyn" nil (m eh1 l v ih0 n))
("melzer" nil (m eh1 l z er0))
("member" nil (m eh1 m b er0))
("membered" nil (m eh1 m b er0 d))
("members" nil (m eh1 m b er0 z))
("membership" nil (m eh1 m b er0 sh ih1 p))
("memberships" nil (m eh1 m b er0 sh ih1 p s))
("membrane" nil (m eh1 m b r ey1 n))
("membranes" nil (m eh1 m b r ey1 n z))
("membranous" nil (m eh1 m b r ax n ax s))
("memel" nil (m eh1 m ax l))
("memento" nil (m ih0 m eh1 n t ow0))
("mementos" nil (m ih0 m eh1 n t ow0 z))
("memmer" nil (m eh1 m er0))
("memmott" nil (m eh1 m ax t))
("memnon" nil (m eh1 m n aa1 n))
("memo" nil (m eh1 m ow1))
("memoir" nil (m eh1 m w aa1 r))
("memoirs" nil (m eh1 m w aa1 r z))
("memoli" nil (m eh0 m ow1 l iy0))
("memorabilia" nil (m eh1 m er0 ax b iy1 l y ax))
("memorable" nil (m eh1 m er0 ax b ax l))
("memorably" nil (m eh1 m er0 ax b l iy0))
("memoranda" nil (m eh1 m er0 ae1 n d ax))
("memorandum" nil (m eh1 m er0 ae1 n d ax m))
("memorandums" nil (m eh1 m er0 ae1 n d ax m z))
("memorex" nil (m eh1 m ao0 r eh1 k s))
("memorial" nil (m ax m ao1 r iy0 ax l))
("memorialized" nil (m ax m ao1 r iy0 ax l ay1 z d))
("memorials" nil (m ax m ao1 r iy0 ax l z))
("memories" nil (m eh1 m er0 iy0 z))
("memorize" nil (m eh1 m er0 ay1 z))
("memorized" nil (m eh1 m er0 ay1 z d))
("memorizing" nil (m eh1 m er0 ay1 z ih0 ng))
("memory" nil (m eh1 m er0 iy0))
("memos" nil (m eh1 m ow0 z))
("memotec" nil (m eh1 m ow0 t eh1 k))
("memphis" nil (m eh1 m f ax s))
("memtec" nil (m eh1 m t eh1 k))
("men" nil (m eh1 n))
("mena" nil (m iy1 n ax))
("menace" nil (m eh1 n ax s))
("menachem" nil (m ax n aa1 hh ax m))
("menacing" nil (m eh1 n ax s ih0 ng))
("menacingly" nil (m eh1 n ax s ih0 ng l iy0))
("menagerie" nil (m ax n ae1 jh er0 iy0))
("menahem" nil (m ax n aa1 hh ax m))
("menaker" nil (m eh1 n ax k er0))
("menapace" nil (m eh0 n aa0 p aa1 ch iy0))
("menard" nil (m ih0 n aa1 r d))
("menasco" nil (m eh0 n aa1 s k ow0))
("menasion" nil (m eh0 n ae1 s iy0 ax n))
("menatep" nil (m eh1 n ax t eh1 p))
("mencer" nil (m eh1 n s er0))
("mench" nil (m eh1 n ch))
("menchaca" nil (m eh0 n k aa1 k ax))
("mencher" nil (m eh1 n ch er0))
("mencken" nil (m eh1 ng k ax n))
("menconi" nil (m eh0 n k ow1 n iy0))
("mend" nil (m eh1 n d))
("mendacity" nil (m eh0 n d ae1 s ih0 t iy0))
("mende" nil (m eh1 n d))
("mended" nil (m eh1 n d ih0 d))
("mendel" nil (m eh1 n d ax l))
("mendell" nil (m eh1 n d eh1 l))
("mendelsohn" nil (m eh1 n d ax l s ax n))
("mendelson" nil (m eh1 n d ax l s ax n))
("mendenhall" nil (m eh1 n d ax n hh ao1 l))
("mendes" nil (m ey1 n d eh0 s))
("mendez" nil (m eh0 n d eh1 z))
("mendicino" nil (m eh0 n d iy0 ch iy1 n ow0))
("mendieta" nil (m eh0 n d iy1 t ax))
("mendillo" nil (m eh1 n d ih1 l ow0))
("mending" nil (m eh1 n d ih0 ng))
("mendiola" nil (m eh1 n d iy0 ow1 l ax))
("mendivil" nil (m ey0 n d iy0 v iy1 l))
("mendlowitz" nil (m eh1 n d l ax w ih0 t s))
("mendocino" nil (m eh1 n d ax s iy1 n ow0))
("mendola" nil (m eh0 n d ow1 l ax))
("mendolia" nil (m eh0 n d ow1 l iy0 ax))
("mendonca" nil (m eh0 n d ow1 n k ax))
("mendonsa" nil (m eh1 n d aa1 n s ax))
("mendosa" nil (m eh0 n d ow1 s ax))
("mendoza" nil (m eh0 n d ow1 z ax))
("mendyk" nil (m eh1 n d ih0 k))
("menear" nil (m ih0 n ih1 r))
("meneely" nil (m ih0 n iy1 l iy0))
("menees" nil (m eh1 n iy1 z))
("menefee" nil (m eh1 n ih0 f iy0))
("menem" nil (m eh1 n ax m))
("menendez" nil (m eh0 n eh1 n d eh0 z))
("meneses" nil (m ey0 n ey1 s eh0 s))
("menezes" nil (m ey0 n ey1 z eh0 s))
("meng" nil (m eh1 ng))
("menge" nil (m eh1 n jh))
("mengel" nil (m eh1 ng g ax l))
("menger" nil (m eh1 n jh er0))
("mengers" nil (m eh1 ng g er0 z))
("menges" nil (m eh1 n jh ih0 z))
("menghini" nil (m eh0 n g iy1 n iy0))
("mengistu" nil (m eh1 ng g iy1 s t uw0))
("menhaden" nil (m eh0 n hh ey1 d ax n))
("menial" nil (m iy1 n iy0 ax l))
("menifee" nil (m eh1 n ih0 f iy1))
("menil" nil (m eh0 n iy1 l))
("meningitis" nil (m eh1 n ax n jh ay1 t ax s))
("menjivar" nil (m ey0 n y iy0 v aa1 r))
("menk" nil (m eh1 ng k))
("menka" nil (m eh1 ng k ax))
("menke" nil (m eh1 ng k))
("menken" nil (m eh1 ng k ax n))
("menkes" nil (m eh1 ng k s))
("menlo" nil (m eh1 n l ow0))
("menn" nil (m eh1 n))
("menna" nil (m eh1 n ax))
("mennan" nil (m eh1 n ax n))
("menne" nil (m eh1 n))
("mennella" nil (m eh1 n eh1 l ax))
("mennen" nil (m eh1 n ax n))
("mennenga" nil (m ih0 n eh1 ng g ax))
("menning" nil (m eh1 n ih0 ng))
("menninger" nil (m eh1 n ih0 ng er0))
("mennini" nil (m eh0 n iy1 n iy0))
("mennonite" nil (m eh1 n ax n ay1 t))
("mennonites" nil (m eh1 n ax n ay1 t s))
("meno" nil (m ey1 n ow0))
("menon" nil (m ey0 n ao1 n))
("menopausal" nil (m eh1 n ax p aw1 s ax l))
("menopause" nil (m eh1 n ax p aw1 s))
("menor" nil (m eh1 n er0))
("menorah" nil (m ax n ao1 r ax))
("menorahs" nil (m ax n ao1 r ax z))
("menotomy" nil (m ih0 n aa1 t ih0 m iy0))
("mens" nil (m eh1 n z))
("mensah" nil (m eh1 n s ax))
("mensch" nil (m eh1 n sh))
("menser" nil (m eh1 n s er0))
("mensik" nil (m eh1 n s ih0 k))
("mensing" nil (m eh1 n s ih0 ng))
("mensinger" nil (m eh1 n s ih0 n jh er0))
("menstrual" nil (m eh1 n s t r uw0 ax l))
("menswear" nil (m eh1 n z w ey1 r))
("mental" nil (m eh1 n t ax l))
("mentality" nil (m eh0 n t ae1 l ax t iy0))
("mentally" nil (m eh1 n t ax l iy0))
("mente" nil (m eh1 n t))
("mentel" nil (m ey0 n t eh1 l))
("menter" nil (m eh1 n t er0))
("menthol" nil (m eh1 n th ao0 l))
("mentholatum" nil (m eh1 n th ax l ey1 t ax m))
("mentink" nil (m eh1 n t ih0 ng k))
("mention" nil (m eh1 n sh ax n))
("mentioned" nil (m eh1 n sh ax n d))
("mentioning" nil (m eh1 n sh ax n ih0 ng))
("mentions" nil (m eh1 n sh ax n z))
("mento" nil (m eh1 n t ow0))
("menton" nil (m eh1 n t ax n))
("mentor" nil (m eh1 n t ao1 r))
("mentoring" nil (m eh1 n t er0 ih0 ng))
("mentors" nil (m eh1 n t er0 z))
("mentz" nil (m eh1 n t s))
("mentzel" nil (m eh1 n t z ax l))
("mentzer" nil (m eh1 n t z er0))
("menu" nil (m eh1 n y uw0))
("menuhin" nil (m ax n uw1 hh ih0 n))
("menus" nil (m eh1 n y uw0 z))
("menz" nil (m eh1 n z))
("menze" nil (m eh1 n z))
("menzel" nil (m eh1 n z ax l))
("menzer" nil (m eh1 n z er0))
("menzie" nil (m eh1 n z iy0))
("menzies" nil (m eh1 n z iy0 z))
("meo" nil (m iy1 ow0))
("meola" nil (m iy0 aa1 l ax))
("meow" nil (m iy0 aw1))
("mequon" nil (m eh1 k w ax n))
("mer" nil (m eh1 r))
("merabank" nil (m eh1 r ax b ae1 ng k))
("meranda" nil (m er0 aa1 n d ax))
("meraz" nil (m eh1 r aa0 z))
("merc" nil (m er1 k))
("mercadante" nil (m er0 k aa0 d aa1 n t iy0))
("mercado" nil (m er0 k aa1 d ow0))
("mercantil" nil (m er0 k ae1 n t ih0 l))
("mercantile" nil (m er1 k ax n t ay1 l))
("mercantilism" nil (m er0 k ae1 n t ax l ih1 z ax m))
("mercantilist" nil (m er0 k ae1 n t ax l ih0 s t))
("mercator" nil (m er0 k ey1 t er0))
("merce" nil (m er1 s))
("mercedes" nil (m er0 s ey1 d iy0 z))
("mercedeses" nil (m er1 s ey1 d iy1 z ih0 z))
("mercenaries" nil (m er1 s ax n eh1 r iy0 z))
("mercenary" nil (m er1 s ax n eh1 r iy0))
("mercer" nil (m er1 s er0))
("merchandise" nil (m er1 ch ax n d ay1 z))
("merchandiser" nil (m er1 ch ax n d ay1 z er0))
("merchandisers" nil (m er1 ch ax n d ay1 z er0 z))
("merchandising" nil (m er1 ch ax n d ay1 z ih0 ng))
("merchant" nil (m er1 ch ax n t))
("merchantmen" nil (m er1 ch ax n t m ih0 n))
("merchants" nil (m er1 ch ax n t s))
("merchantsbank" nil (m er1 ch ax n t s b ae1 ng k))
("mercier" nil (m er1 s iy0 er0))
("mercies" nil (m er1 s iy0 z))
("merciful" nil (m er1 s ih0 f ax l))
("mercifully" nil (m er1 s ih0 f ax l iy0))
("merciless" nil (m er1 s ax l ax s))
("mercilessly" nil (m er1 s ax l ax s l iy0))
("merck" nil (m er1 k))
("mercosur" nil (m er1 k ow1 s er1))
("mercure" nil (m er0 k uh1 r iy0))
("mercuri" nil (m er0 k uh1 r iy0))
("mercurial" nil (m er0 k y uh1 r iy0 ax l))
("mercuric" nil (m er0 k y uh1 r ih0 k))
("mercurio" nil (m er0 k uh1 r iy0 ow0))
("mercury" nil (m er1 k y er0 iy0))
("mercy" nil (m er1 s iy0))
("merdyce" nil (m er1 d ay0 s))
("mere" nil (m ih1 r))
("meredith" nil (m eh1 r ih0 d ih0 th))
("merely" nil (m ih1 r l iy0))
("merenda" nil (m eh0 r ey1 n d ax))
("merendino" nil (m er0 eh0 n d iy1 n ow0))
("merest" nil (m eh1 r ax s t))
("merfeld" nil (m er1 f eh0 l d))
("merge" nil (m er1 jh))
("merged" nil (m er1 jh d))
("mergen" nil (m er1 g ax n))
("merger" nil (m er1 jh er0))
("mergers" nil (m er1 jh er0 z))
("merges" nil (m er1 jh ih0 z))
("merging" nil (m er1 jh ih0 ng))
("merhige" nil (m er0 hh iy1 jh))
("meriam" nil (m ih1 r iy0 ih0 m))
("merica" nil (m eh1 r ih0 k ax))
("mericantante" nil (m eh0 r iy1 k ax n t aa1 n t ey0))
("mericle" nil (m eh1 r ih0 k ax l))
("merida" nil (m er0 iy1 d ax))
("meriden" nil (m eh1 r ih0 d ax n))
("merideth" nil (m eh1 r ih0 d ih0 th))
("meridian" nil (m er0 ih1 d iy0 ax n))
("meridien" nil (m er0 ih1 d iy0 ax n))
("meridionale" nil (m er0 ih1 d iy0 ax n aa1 l iy0))
("meridith" nil (m eh1 r ih0 d ih0 th))
("meridor" nil (m eh1 r ih0 d ao0 r))
("meriel" nil (m ih1 r iy0 l))
("merieux" nil (m eh1 r iy0 uw1))
("merillat" nil (m eh1 r ih0 l ae0 t))
("merino" nil (m er0 iy1 n ow0))
("merion" nil (m eh1 r iy0 ax n))
("meris" nil (m eh1 r ax s))
("merisel" nil (m eh1 r ih0 s eh1 l))
("merit" nil (m eh1 r ax t))
("merited" nil (m eh1 r ih0 t ih0 d))
("merithew" nil (m er0 ih1 th y uw0))
("meritless" nil (m eh1 r ih0 t l ax s))
("meritor" nil (m eh1 r ax t er0))
("meritorious" nil (m eh1 r ax t ao1 r iy0 ax s))
("merits" nil (m eh1 r ax t s))
("meritt" nil (m eh1 r ih0 t))
("meriweather" nil (m eh1 r ih0 w eh1 dh er0))
("meriwether" nil (m eh1 r ih0 w eh1 dh er0))
("merk" nil (m er1 k))
("merkel" nil (m er1 k ax l))
("merker" nil (m er1 k er0))
("merkey" nil (m er1 k iy0))
("merkin" nil (m er1 k ih0 n))
("merkle" nil (m er1 k ax l))
("merkley" nil (m er1 k l iy0))
("merklin" nil (m er1 k l ih0 n))
("merksamer" nil (m er1 k s ax m er0))
("merkt" nil (m er1 k t))
("merkur" nil (m er1 k er0))
("merl" nil (m er1 l))
("merle" nil (m er1 l))
("merlin" nil (m er1 l ih0 n))
("merlina" nil (m er0 l iy1 n ax))
("merline" nil (m er1 l ay0 n))
("merlino" nil (m er0 l iy1 n ow0))
("merlins" nil (m er1 l ih0 n z))
("merlis" nil (m er1 l iy0 z))
("merlo" nil (m eh1 r l ow0))
("merlot" nil (m er1 l ax t))
("mermaid" nil (m er1 m ey1 d))
("mermaids" nil (m er1 m ey1 d z))
("merman" nil (m er1 m ae1 n))
("mermelstein" nil (m er1 m ax l s t ay0 n))
("merna" nil (m eh1 r n ax))
("merner" nil (m er1 n er0))
("mernick" nil (m er1 n ih0 k))
("mero" nil (m eh1 r ow0))
("merola" nil (m er0 ow1 l ax))
("merolla" nil (m er0 ow1 l ax))
("meroney" nil (m eh1 r ax n iy0))
("merow" nil (m eh1 r ow0))
("merrell" nil (m eh1 r ax l))
("merrett" nil (m eh1 r ih0 t))
("merriam" nil (m eh1 r iy0 ax m))
("merrick" nil (m eh1 r ih0 k))
("merricks" nil (m eh1 r ih0 k s))
("merrie" nil (m eh1 r iy0))
("merrier" nil (m eh1 r iy0 er0))
("merrifield" nil (m eh1 r ih0 f iy1 l d))
("merrigan" nil (m eh1 r ih0 g ax n))
("merrihew" nil (m er0 ih1 hh y uw0))
("merrill" nil (m eh1 r ax l))
("merrily" nil (m eh1 r ax l iy0))
("merrimac" nil (m eh1 r ih0 m ae0 k))
("merrimack" nil (m eh1 r ih0 m ae1 k))
("merriman" nil (m eh1 r ih0 m ax n))
("merrin" nil (m eh1 r ih0 n))
("merriott" nil (m eh1 r iy0 ax t))
("merritt" nil (m eh1 r ih0 t))
("merritts" nil (m eh1 r ih0 t s))
("merriweather" nil (m eh0 r ih0 w eh1 dh er0))
("merriwether" nil (m eh1 r ih0 w ih0 dh er0))
("merrow" nil (m eh1 r ow0))
("merry" nil (m eh1 r iy0))
("merryfield" nil (m eh1 r iy0 f iy1 l d))
("merryman" nil (m eh1 r iy0 m ax n))
("mers" nil (m er1 z))
("mersch" nil (m er1 sh))
("mersereau" nil (m er1 s er0 ow0))
("mershon" nil (m er1 sh ax n))
("mersman" nil (m er1 s m ax n))
("merson" nil (m er1 s ax n))
("merta" nil (m eh1 r t ax))
("merten" nil (m er1 t ax n))
("mertens" nil (m er1 t ax n z))
("mertes" nil (m eh1 r t eh0 s))
("mertice" nil (m eh1 r t ih0 s))
("mertins" nil (m er1 t ih0 n z))
("mertle" nil (m er1 t ax l))
("merton" nil (m er1 t ax n))
("mertz" nil (m er1 t s))
("merv" nil (m er1 v))
("mervin" nil (m er1 v ih0 n))
("mervine" nil (m er1 v ay0 n))
("mervis" nil (m er1 v ih0 s))
("mervyn" nil (m er1 v ih0 n))
("merwe" nil (m er1 w iy0))
("merwin" nil (m er1 w ih0 n))
("merwyn" nil (m er1 w ih0 n))
("meryl" nil (m eh1 r ax l))
("merz" nil (m er1 z))
("mesa" nil (m ey1 s ax))
("mesaba" nil (m eh0 s aa1 b ax))
("mesaros" nil (m ey0 s aa1 r ow0 z))
("mesbic" nil (m eh1 s b ih0 k))
("mesbics" nil (m eh1 s b ih0 k s))
("mescaline" nil (m eh1 s k ax l iy1 n))
("mesch" nil (m eh1 sh))
("mescher" nil (m eh1 sh er0))
("meschke" nil (m eh1 sh k))
("meselsohn" nil (m eh1 z ax l s ax n))
("meselson" nil (m eh1 z ax l s ax n))
("mesenbrink" nil (m eh1 s ih0 n b r ih0 ng k))
("mesenteric" nil (m eh1 s ax n t eh1 r ih0 k))
("meserole" nil (m eh0 s er0 ow1 l iy0))
("meserve" nil (m eh1 s er0 v))
("meservey" nil (m eh0 z er0 v ey1))
("mesh" nil (m eh1 sh))
("meshed" nil (m eh1 sh t))
("meshell" nil (m eh1 sh ax l))
("meshes" nil (m eh1 sh ih0 z))
("meshing" nil (m eh1 sh ih0 ng))
("meshulam" nil (m eh1 sh uw0 l ae0 m))
("mesick" nil (m eh1 s ih0 k))
("mesirow" nil (m eh1 s ih0 r ow0))
("meske" nil (m eh1 s k))
("mesker" nil (m eh1 s k er0))
("meskill" nil (m eh1 s k ih0 l))
("meskimen" nil (m eh1 s k iy0 m eh0 n))
("mesko" nil (m eh1 s k ow0))
("mesler" nil (m eh1 s ax l er0))
("mesmer" nil (m eh1 z m er0))
("mesmerism" nil (m eh1 s m er0 ih1 z ax m))
("mesmerized" nil (m eh1 z m er0 ay1 z d))
("mesmerizing" nil (m eh1 z m er0 ay1 z ih0 ng))
("mesner" nil (m eh1 s n er0))
("mesodermal" nil (m eh1 z ax d er1 m ax l))
("mesolithic" nil (m eh1 z ax l ih1 th ih0 k))
("meson" nil (m ey1 z aa1 n))
("mesons" nil (m iy1 z aa1 n z))
("mesopotamia" nil (m eh1 s ax p ax t ey1 m iy0 ax))
("mesopotamian" nil (m eh1 s ax p ax t ey1 m iy0 ax n))
("mesopotamians" nil (m eh1 s ax p ax t ey1 m iy0 ax n z))
("mesothelioma" nil (m eh1 s ax th iy1 l iy0 ow1 m ax))
("mesothorax" nil (m eh1 z ax th ao1 r ae1 k s))
("mesozoic" nil (m eh1 s ax z ow1 ih0 k))
("mesquite" nil (m eh1 s k iy1 t))
("mess" nil (m eh1 s))
("messa" nil (m eh1 s ax))
("message" nil (m eh1 s ax jh))
("messagepad" nil (m eh1 s ax jh p ae1 d))
("messages" nil (m eh1 s ax jh ax z))
("messaging" nil (m eh1 s ih0 jh ih0 ng))
("messamore" nil (m eh0 s aa1 m ao0 r))
("messana" nil (m eh0 s ae1 n ax))
("messed" nil (m eh1 s t))
("messel" nil (m eh1 s ax l))
("messenger" nil (m eh1 s ax n jh er0))
("messengers" nil (m eh1 s ax n jh er0 z))
("messer" nil (m eh1 s er0))
("messerli" nil (m eh1 s er0 l iy0))
("messerly" nil (m eh1 s er0 l iy0))
("messerschmidt" nil (m eh1 s er0 sh m ih1 t))
("messerschmitt" nil (m eh1 s er0 sh m ih1 t))
("messersmith" nil (m eh1 s er0 s m ih1 th))
("messes" nil (m eh1 s ih0 z))
("messiaen" nil (m eh1 s iy0 ey1 n))
("messiah" nil (m ax s ay1 ax))
("messiahs" nil (m ax s ay1 ax z))
("messianic" nil (m eh1 s iy0 ae1 n ih0 k))
("messick" nil (m eh1 s ih0 k))
("messier" nil (m eh1 s iy0 er0))
("messimer" nil (m eh1 s ih0 m er0))
("messina" nil (m ih0 s iy1 n ax))
("messineo" nil (m eh1 s ih1 n iy0 ow0))
("messing" nil (m eh1 s ih0 ng))
("messinger" nil (m eh1 s ih0 ng er0))
("messler" nil (m eh1 s l er0))
("messman" nil (m eh1 s m ax n))
("messmer" nil (m eh1 s m er0))
("messmore" nil (m eh1 s m ao0 r))
("messner" nil (m eh1 s n er0))
("messrs" nil (m eh1 s er0 z))
("messy" nil (m eh1 s iy0))
("mest" nil (m eh1 s t))
("mesta" nil (m eh1 s t ax))
("mestas" nil (m eh1 s t ax z))
("mestek" nil (m eh1 s t eh1 k))
("mester" nil (m eh1 s t er0))
("mestizo" nil (m eh0 s t iy1 z ow0))
("meston" nil (m eh1 s t ax n))
("mestrallet" nil (m eh1 s t r ax l eh1 t))
("mestre" nil (m eh1 s t er0))
("meszaros" nil (m ih0 sh aa1 r ow0 z))
("met" nil (m eh1 t))
("meta" nil (m iy1 t ax))
("metabolic" nil (m eh1 t ax b aa1 l ih0 k))
("metabolism" nil (m ax t ae1 b ax l ih1 z ax m))
("metabolisms" nil (m ax t ae1 b ax l ih1 z ax m z))
("metabolize" nil (m ax t ae1 b ax l ay1 z))
("metacarpals" nil (m eh1 t ax k aa1 r p ax l z))
("metairie" nil (m ax t eh1 r iy0))
("metal" nil (m eh1 t ax l))
("metall" nil (m eh1 t ao1 l))
("metallgesellschaft" nil (m eh1 t ax l g eh1 s ax l sh ae1 f t))
("metallic" nil (m ax t ae1 l ih0 k))
("metallo" nil (m eh0 t aa1 l ow0))
("metallurgical" nil (m eh1 t ax l er1 jh ih0 k ax l))
("metallurgy" nil (m eh1 t ax l er0 jh iy0))
("metals" nil (m eh1 t ax l z))
("metalwork" nil (m eh1 t ax l w er1 k))
("metalworker" nil (m eh1 t ax l w er1 k er0))
("metalworkers" nil (m eh1 t ax l w er1 k er0 z))
("metalworking" nil (m eh1 t ax l w er1 k ih0 ng))
("metamorphic" nil (m eh1 t ax m ao1 r f ih0 k))
("metamorphose" nil (m eh1 t ax m ao1 r f ow0 z))
("metamorphosis" nil (m eh1 t ax m ao1 r f ax s ax s))
("metamucil" nil (m eh1 t ax m y uw1 s ax l))
("metaphor" nil (m eh1 t ax f ao0 r))
("metaphorical" nil (m eh1 t ax f ao1 r ih0 k ax l))
("metaphors" nil (m eh1 t ax f ao0 r z))
("metaphysical" nil (m eh1 t ax f ih1 z ih0 k ax l))
("metaphysics" nil (m eh1 t ax f ih1 z ih0 k s))
("metathorax" nil (m eh1 t ax th ao1 r ae1 k s))
("metcalf" nil (m eh1 t k ae1 f))
("metcalfe" nil (m eh1 t k ax l f))
("metcom" nil (m eh1 t k aa0 m))
("mete" nil (m iy1 t))
("meted" nil (m iy1 t ih0 d))
("meteor" nil (m iy1 t iy0 er0))
("meteoric" nil (m iy1 t iy0 ao1 r ih0 k))
("meteorological" nil (m iy1 t iy0 ao1 r ax l aa1 jh ih0 k ax l))
("meteorologist" nil (m iy1 t iy0 er0 aa1 l ax jh ih0 s t))
("meteorologists" nil (m iy1 t iy0 er0 aa1 l ax jh ih0 s t s))
("meteorology" nil (m iy1 t iy0 er0 aa1 l ax jh iy0))
("meter" nil (m iy1 t er0))
("metering" nil (m iy1 t er0 ih0 ng))
("meters" nil (m iy1 t er0 z))
("metex" nil (m eh1 t eh1 k s))
("meth" nil (m eh1 th))
("methadone" nil (m eh1 th ax d ow1 n))
("methamphetamine" nil (m eh1 th ae0 m f eh1 t ax m iy0 n))
("methane" nil (m eh1 th ey1 n))
("methanex" nil (m eh1 th ax n eh1 k s))
("methanol" nil (m eh1 th ax n aa1 l))
("methazine" nil (m eh1 th ax z iy1 n))
("metheney" nil (m eh1 th ih0 n iy0))
("metheny" nil (m eh1 th ih0 n iy0))
("methner" nil (m eh1 th n er0))
("method" nil (m eh1 th ax d))
("methode" nil (m ax th ow1 d))
("methodical" nil (m ax th aa1 d ax k ax l))
("methodically" nil (m ax th aa1 d ih0 k ax l iy0))
("methodism" nil (m eh1 th ax d ih1 z ax m))
("methodisms" nil (m eh1 th ax d ih1 z ax m z))
("methodist" nil (m eh1 th ax d ax s t))
("methodological" nil (m eh1 th ax d ax l aa1 jh ih0 k ax l))
("methodologies" nil (m eh1 th ow0 d aa1 l ax jh iy0 z))
("methodology" nil (m eh1 th ax d aa1 l ax jh iy0))
("methods" nil (m eh1 th ax d z))
("methot" nil (m eh1 th ax t))
("methotrexate" nil (m eh1 th ow0 t r eh1 k s ey1 t))
("methuselah" nil (m ax th y uw1 z ax l ax))
("methvin" nil (m eh1 th v ih0 n))
("methyl" nil (m eh1 th ax l))
("methylene" nil (m eh1 th ih0 l iy1 n))
("metics" nil (m eh1 t ih0 k s))
("meticulous" nil (m ax t ih1 k y ax l ax s))
("meticulously" nil (m ax t ih1 k y ax l ax s l iy0))
("metier" nil (m eh1 t y er0))
("metis" nil (m eh1 t ih0 s))
("metivier" nil (m eh1 t ih0 v iy0 er0))
("metlife" nil (m eh1 t l ay1 f))
("metolachlor" nil (m ax t ow1 l ax k l er0))
("metoyer" nil (m eh1 t oy0 er0))
("metpath" nil (m eh1 t p ae1 th))
("metra" nil (m eh1 t r ax))
("metrahealth" nil (m eh1 t r ax hh eh1 l th))
("metric" nil (m eh1 t r ih0 k))
("metrical" nil (m eh1 t r ih0 k ax l))
("metrically" nil (m eh1 t r ih0 k ax l iy0))
("metrick" nil (m eh1 t r ih0 k))
("metrics" nil (m eh1 t r ih0 k s))
("metro" nil (m eh1 t r ow1))
("metrobanc" nil (m eh1 t r ow0 b ae1 ng k))
("metrobank" nil (m eh1 t r ow0 b ae1 ng k))
("metrocall" nil (m eh1 t r ow1 k ao1 l))
("metrocolor" nil (m eh1 t r ow0 k ah1 l er0))
("metrocorp" nil (m eh1 t r ow0 k ao1 r p))
("metrodome" nil (m eh1 t r ow0 d ow1 m))
("metromail" nil (m eh1 t r ow0 m ey1 l))
("metromedia" nil (m eh1 t r ow0 m iy1 d iy0 ax))
("metroplex" nil (m eh1 t r ow0 p l eh1 k s))
("metropol" nil (m eh1 t r ax p ow1 l))
("metropolis" nil (m ax t r aa1 p ax l ax s))
("metropolitain" nil (m eh1 t r ax p ao1 l ax t ax n))
("metropolitan" nil (m eh1 t r ax p aa1 l ax t ax n))
("metropolitans" nil (m eh1 t r ax p aa1 l ax t ax n z))
("mets" nil (m eh1 t s))
("metsker" nil (m eh1 t s k er0))
("mette" nil (m eh1 t))
("metter" nil (m eh1 t er0))
("metternich" nil (m eh1 t er0 n ih0 ch))
("mettle" nil (m eh1 t ax l))
("mettlen" nil (m eh1 t l ax n))
("mettler" nil (m eh1 t l er0))
("metts" nil (m eh1 t s))
("mettur" nil (m eh1 t er0))
("metz" nil (m eh1 t s))
("metzenbaum" nil (m eh1 t s ax n b aw1 m))
("metzer" nil (m eh1 t s er0))
("metzgar" nil (m eh1 t s g er0))
("metzger" nil (m eh1 t s g er0))
("metzinger" nil (m eh1 t z ih0 ng er0))
("metzker" nil (m eh1 t s k er0))
("metzler" nil (m eh1 t s l er0))
("metzner" nil (m eh1 t s n er0))
("meunier" nil (m oy1 n iy0 er0))
("meurer" nil (m er1 er0))
("meuse" nil (m y uw1 z))
("meuser" nil (m oy1 s er0))
("meuth" nil (m y uw1 th))
("mevacor" nil (m eh1 v ax k ao1 r))
("mew" nil (m y uw1))
("mewborn" nil (m y uw1 b er0 n))
("mewes" nil (m y uw1 z))
("mex" nil (m eh1 k s))
("mexicali" nil (m eh1 k s ih0 k aa1 l iy0))
("mexican" nil (m eh1 k s ax k ax n))
("mexicana" nil (m eh1 k s ih0 k ae1 n ax))
("mexicano" nil (m eh1 k s ih0 k aa1 n ow0))
("mexicanos" nil (m eh1 k s ih0 k aa1 n ow0 z))
("mexicans" nil (m eh1 k s ih0 k ax n z))
("mexico" nil (m eh1 k s ax k ow1))
("mey" nil (m ey1))
("meyer" nil (m ay1 er0))
("meyerbeer" nil (m ay1 er0 b ih1 r))
("meyerhoff" nil (m ay1 er0 hh ao0 f))
("meyering" nil (m ey1 er0 ih0 ng))
("meyerman" nil (m ay1 er0 m ax n))
("meyerowitz" nil (m ay1 er0 ax w ih0 t s))
("meyers" nil (m ay1 er0 z))
("meyerson" nil (m ay1 er0 s ax n))
("meyn" nil (m ey1 n))
("meyo" nil (m ey1 ow0))
("meyohas" nil (m ey0 ow1 hh aa0 s))
("meza" nil (m eh1 z ax))
("mezera" nil (m ey0 z eh1 r ax))
("mezey" nil (m eh1 z iy0))
("mezger" nil (m eh1 z jh er0))
("mezo" nil (m eh1 z ow0))
("mezvinsky" nil (m eh1 z v ih1 n s k iy0))
("mezzaluna" nil (m eh1 t s ax l uw1 n ax))
("mezzanine" nil (m eh1 z ax n iy1 n))
("mezzo" nil (m eh1 z ow0))
("mfume" nil (eh1 m f uw1 m ey1))
("mhoon" nil (m hh uw1 n))
("mi" nil (m iy1))
("mia" nil (m iy1 ax))
("miami" nil (m ay0 ae1 m iy0))
("miano" nil (m iy0 aa1 n ow0))
("miara" nil (m ay0 aa1 r ax))
("miasma" nil (m iy0 ae1 z m ax))
("miata" nil (m iy0 aa1 t ax))
("miazga" nil (m iy0 aa1 z g ax))
("mic" nil (m ih1 k))
("mica" nil (m ay1 k ax))
("micaela" nil (m iy0 k ey1 l ax))
("micale" nil (m iy0 k aa1 l iy0))
("micallef" nil (m ih1 k ax l eh0 f))
("micciche" nil (m iy0 ch iy1 k iy0))
("miccio" nil (m iy1 ch iy0 ow0))
("mice" nil (m ay1 s))
("micek" nil (m ih1 ch eh0 k))
("miceli" nil (m iy0 ch eh1 l iy0))
("mich" nil (m ih1 ch))
("michael" nil (m ay1 k ax l))
("michaela" nil (m ay0 k eh1 l ax))
("michaelina" nil (m ay1 k ax l iy1 n ax))
("michaeline" nil (m ay1 k ax l ay0 n))
("michaelis" nil (m ay1 k eh1 l ih0 s))
("michaella" nil (m ay1 k eh1 l ax))
("michaels" nil (m ay1 k ax l z))
("michaelsen" nil (m ay1 k ax l s ax n))
("michaelson" nil (m ay1 k ax l s ax n))
("michal" nil (m ih1 ch ax l))
("michalak" nil (m ih1 ch ax l ae0 k))
("michalec" nil (m ih0 hh aa1 l ih0 k))
("michalek" nil (m ih0 hh aa1 l eh0 k))
("michalik" nil (m ih1 k ax l ih0 k))
("michalowski" nil (m ih1 ch ax l ao1 f s k iy0))
("michals" nil (m ih1 ch ax l z))
("michalski" nil (m ih1 ch aa1 l s k iy0))
("michalsky" nil (m ih1 ch aa1 l s k iy0))
("michaud" nil (m ih0 sh ow1))
("michaux" nil (m ih0 sh ow1))
("michcon" nil (m ih1 sh k aa0 n))
("micheal" nil (m ay1 k ax l))
("micheaux" nil (m ih0 sh ow1))
("michel" nil (m ih0 sh eh1 l))
("michelangelo" nil (m ay1 k ax l ae1 n jh ax l ow1))
("michele" nil (m ih0 sh eh1 l))
("micheletti" nil (m ih0 k ax l eh1 t iy0))
("micheli" nil (m ih0 k eh1 l iy0))
("michelin" nil (m ih1 sh ax l ax n))
("michelina" nil (m ih0 k ax l iy1 n ax))
("micheline" nil (m ih1 sh ax l ax n))
("michelini" nil (m ih0 k ax l iy1 n iy0))
("michell" nil (m ih1 ch ax l))
("michelle" nil (m ih0 sh eh1 l))
("michelli" nil (m ih0 ch eh1 l iy0))
("michelman" nil (m ay1 k ax l m ax n))
("michelob" nil (m ih1 k ax l ow0 b))
("michelotti" nil (m ih0 k ax l ow1 t iy0))
("michels" nil (m ih0 sh eh1 l z))
("michelsen" nil (m ay1 k ax l s ax n))
("michelson" nil (m ay1 k ax l s ax n))
("michener" nil (m ih1 ch n er0))
("michie" nil (m ih1 ch iy0))
("michiels" nil (m ay1 k ax l z))
("michigan" nil (m ih1 sh ih0 g ax n))
("michio" nil (m ih1 ch iy0 ow0))
("michl" nil (m ih1 ch ax l))
("michler" nil (m ih1 ch l er0))
("michna" nil (m ih1 ch n ax))
("michoacan" nil (m ih0 ch ow1 k ax n))
("michon" nil (m ih1 ch ax n))
("mick" nil (m ih1 k))
("micka" nil (m ih1 k ax))
("micke" nil (m ih1 k))
("mickel" nil (m ih1 k ax l))
("mickelberry" nil (m ih1 k ax l b eh1 r iy0))
("mickels" nil (m ih1 k ax l z))
("mickelsen" nil (m ih1 k ax l s ax n))
("mickelson" nil (m ih1 k ax l s ax n))
("mickens" nil (m ih1 k ax n z))
("mickey" nil (m ih1 k iy0))
("mickie" nil (m ih1 k iy0))
("mickiewicz" nil (m ih1 k ax v ih0 ch))
("mickish" nil (m ih1 k ih0 sh))
("mickle" nil (m ih1 k ax l))
("mickler" nil (m ih1 k l er0))
("mickles" nil (m ih1 k ax l z))
("mickley" nil (m ih1 k l iy0))
("mickus" nil (m ih1 k ax s))
("micky" nil (m ih1 k iy0))
("micoin" nil (m ih0 k oy1 n))
("micom" nil (m ay1 k aa0 m))
("micro" nil (m ay1 k r ow1))
("microage" nil (m ay1 k r ow0 ey1 jh))
("microaire" nil (m ay1 k r ow0 eh1 r))
("microamerica" nil (m ay1 k r ow0 ax m eh1 r ih0 k ax))
("microbe" nil (m ay1 k r ow1 b))
("microbes" nil (m ay1 k r ow1 b z))
("microbial" nil (m ay0 k r ow1 b iy0 ax l))
("microbilt" nil (m ay1 k r ow0 b ih1 l t))
("microbiologist" nil (m ay1 k r ow0 b iy0 aa1 l ax jh ih0 s t))
("microbiology" nil (m ay1 k r ow0 b ay1 aa1 l ax jh iy0))
("microbreweries" nil (m ay1 k r ow1 b r uw1 er0 iy0 z))
("microbrewery" nil (m ay1 k r ow1 b r uw1 er0 iy0))
("microchip" nil (m ay1 k r ow1 ch ih1 p))
("microchips" nil (m ay1 k r ow1 ch ih1 p s))
("microcircuits" nil (m ay1 k r ow0 s er1 k ax t s))
("microclimates" nil (m ay1 k r ow0 k l ay1 m ax t s))
("microcode" nil (m ay1 k r ow0 k ow1 d))
("microcom" nil (m ay1 k r ow0 k aa1 m))
("microcomputer" nil (m ay1 k r ow1 k ax m p y uw1 t er0))
("microcomputers" nil (m ay1 k r ow1 k ax m p y uw1 t er0 z))
("microcosm" nil (m ay1 k r ax k aa1 z ax m))
("microdyne" nil (m ay1 k r ow0 d ay1 n))
("microeconomic" nil (m ay1 k r ow0 eh1 k ax n aa1 m ih0 k))
("microeconomics" nil (m ay1 k r ow0 eh1 k ax n aa1 m ih0 k s))
("microelectronic" nil (m ay1 k r ow0 ih0 l eh0 k t r aa1 n ih0 k))
("microelectronics" nil (m ay1 k r ow0 ih0 l eh0 k t r aa1 n ih0 k s))
("microelettronica" nil (m ay1 k r ow0 ih0 l eh0 t r aa1 n ih0 k ax))
("microfiche" nil (m ay1 k r ow0 f iy1 ch))
("microfilm" nil (m ay1 k r ax f ih1 l m))
("microfossils" nil (m ay1 k r ow1 f aa1 s ax l z))
("microgenesys" nil (m ay1 k r ow0 jh eh1 n ax s ih0 s))
("micrografx" nil (m ay1 k r ow0 g r ae1 f eh0 k s))
("micrograms" nil (m ay1 k r ow0 g r ae1 m z))
("micrographic" nil (m ay1 k r ow0 g r ae1 f ih0 k))
("micrographics" nil (m ay1 k r ow0 g r ae1 f ih0 k s))
("micromanage" nil (m ay1 k r ow0 m ae1 n ih0 jh))
("micromanagement" nil (m ay1 k r ow0 m ae1 n ih0 jh m ax n t))
("micrometer" nil (m ay0 k r aa1 m ax t er0))
("micron" nil (m ay1 k r aa1 n))
("micronesia" nil (m ay1 k r ow0 n iy1 zh ax))
("micronics" nil (m ay1 k r ao1 n ih1 k s))
("microns" nil (m ay1 k r aa1 n z))
("microorganism" nil (m ay1 k r ow0 ao1 r g ax n ih1 z ax m))
("microorganisms" nil (m ay1 k r ow0 ao1 r g ax n ih1 z ax m z))
("micropaleontology" nil (m ay1 k r ow0 p ey1 l iy0 ax n t aa1 l ax jh iy0))
("microphone" nil (m ay1 k r ax f ow1 n))
("microphones" nil (m ay1 k r ow0 f ow1 n z))
("micropolis" nil (m ay1 k r ao1 p ax l ih0 s))
("micropro" nil (m ay1 k r ow0 p r ow1))
("microprocessor" nil (m ay1 k r ow0 p r aa1 s eh0 s er0))
("microprocessors" nil (m ay1 k r ow0 p r aa1 s eh0 s er0 z))
("microprose" nil (m ay1 k r ow0 p r ow1 z))
("micros" nil (m ay1 k r ow0 z))
("microscope" nil (m ay1 k r ax s k ow1 p))
("microscopes" nil (m ay1 k r ax s k ow1 p s))
("microscopic" nil (m ay1 k r ax s k aa1 p ih0 k))
("microscopically" nil (m ay1 k r ax s k aa1 p ih0 k l iy0))
("microsemi" nil (m ay1 k r ow0 s eh1 m iy0))
("microsoft" nil (m ay1 k r ow1 s ao1 f t))
("microsystems" nil (m ay1 k r ow1 s ih1 s t ax m z))
("microtel" nil (m ay1 k r ow0 t eh1 l))
("microtubules" nil (m ay1 k r ow1 t uw1 b y uw0 l z))
("microvax" nil (m ay1 k r ow0 v ae1 k s))
("microware" nil (m ay1 k r ow1 w eh1 r))
("microwavable" nil (m ay1 k r ow0 w ey1 v ax b ax l))
("microwave" nil (m ay1 k r ax w ey1 v))
("microwaved" nil (m ay1 k r ow0 w ey1 v d))
("microwaves" nil (m ay1 k r ow0 w ey1 v z))
("microx" nil (m ih1 k r ax k s))
("micucci" nil (m iy0 k uw1 ch iy0))
("mid" nil (m ih1 d))
("midafternoon" nil (m ih1 d ae1 f t er0 n uw1 n))
("midair" nil (m ih1 d eh1 r))
("midamerica" nil (m ih1 d ax m eh1 r ih0 k ax))
("midas" nil (m ay1 d ax s))
("midcap" nil (m ih1 d k ae1 p))
("midcentury" nil (m ih0 d s eh1 n ch er0 iy0))
("midcon" nil (m ih1 d k aa1 n))
("midcontinent" nil (m ih0 d k aa1 n t ih0 n ax n t))
("midcourse" nil (m ih1 d k ao1 r s))
("middaugh" nil (m ih1 d ao0))
("midday" nil (m ih1 d d ey1))
("middendorf" nil (m ih1 d ax n d ao1 r f))
("middenstandsbank" nil (m ih0 d eh1 n s t ae0 n d z b ae1 ng k))
("middle" nil (m ih1 d ax l))
("middlebrook" nil (m ih1 d ax l b r uh1 k))
("middlebrooks" nil (m ih1 d ax l b r uh1 k s))
("middleburg" nil (m ih1 d ax l b er0 g))
("middlebury" nil (m ih1 d ax l b eh1 r iy0))
("middlekauff" nil (m ih1 d ax l k ow0 f))
("middleman" nil (m ih1 d ax l m ae1 n))
("middlemen" nil (m ih1 d ax l m eh1 n))
("middlemiss" nil (m ih1 d ax l m ih1 s))
("middlesex" nil (m ih1 d ax l s eh1 k s))
("middlesworth" nil (m ih1 d ax l z w er1 th))
("middleton" nil (m ih1 d ax l t ax n))
("middletown" nil (m ih1 d ax l t aw1 n))
("middleweight" nil (m ih1 d ax l w ey1 t))
("middling" nil (m ih1 d ax l ih0 ng))
("mideast" nil (m ih1 d iy1 s t))
("mideastern" nil (m ih1 d iy1 s t er0 n))
("midflight" nil (m ih1 d f l ay1 t))
("midges" nil (m ih1 jh ax z))
("midget" nil (m ih1 jh ax t))
("midgetman" nil (m ih1 jh ax t m ae1 n))
("midgett" nil (m ih1 jh ih0 t))
("midgette" nil (m ih0 jh eh1 t))
("midgley" nil (m ih1 jh l iy0))
("midi" nil (m iy1 d iy0))
("midkiff" nil (m ih1 d k ih0 f))
("midland" nil (m ih1 d l ae1 n d))
("midlands" nil (m ih1 d l ae1 n d z))
("midlantic" nil (m ih0 d l ae1 n t ih0 k))
("midler" nil (m ih1 d l er0))
("midlevel" nil (m ih1 d l ax v ax l))
("midlife" nil (m ih1 d l ay1 f))
("midmorning" nil (m ih1 d m ao1 r n ih0 ng))
("midnight" nil (m ih1 d n ay1 t))
("midpac" nil (m ih1 d p ae1 k))
("midpoint" nil (m ih1 d p oy1 n t))
("midpriced" nil (m ih1 d p r ay1 s t))
("midrange" nil (m ih1 d r ey1 n jh))
("midrib" nil (m ih1 d r ih0 b))
("midseason" nil (m ih1 d s iy1 z ax n))
("midsection" nil (m ih1 d s eh1 k sh ax n))
("midsession" nil (m ih1 d s eh1 sh ax n))
("midshipman" nil (m ih1 d sh ih1 p m ax n))
("midshipmen" nil (m ih1 d sh ih1 p m eh0 n))
("midsize" nil (m ih1 d s ay1 z))
("midsized" nil (m ih1 d s ay1 z d))
("midst" nil (m ih1 d s t))
("midstream" nil (m ih1 d s t r iy1 m))
("midsummer" nil (m ih1 d s ah1 m er0))
("midterm" nil (m ih1 d t er1 m))
("midthun" nil (m ih1 d th ax n))
("midtown" nil (m ih1 d t aw1 n))
("midura" nil (m iy0 d uh1 r ax))
("midway" nil (m ih1 d w ey1))
("midweek" nil (m ih1 d w iy1 k))
("midwest" nil (m ih1 d w eh1 s t))
("midwestern" nil (m ih0 d w eh1 s t er0 n))
("midwesterner" nil (m ih1 d w eh1 s t er0 n er0))
("midwesterners" nil (m ih0 d w eh1 s t er0 n er0 z))
("midwife" nil (m ih1 d w ay1 f))
("midwifery" nil (m ih1 d w ay1 f er0 iy0))
("midwinter" nil (m ih1 d w ih1 n t er0))
("midwives" nil (m ih1 d w ay1 v z))
("midyear" nil (m ih1 d y ih1 r))
("midyett" nil (m ih1 d y eh1 t))
("midyette" nil (m ih1 d y eh1 t))
("miears" nil (m iy0 ih1 r z))
("mieczkowski" nil (m iy0 ch k ao1 f s k iy0))
("mieczyslaw" nil (m iy1 ch ih0 s l ao1))
("miedema" nil (m iy0 d eh1 m ax))
("mielcarek" nil (m iy0 l k aa1 r eh0 k))
("miele" nil (m iy1 l))
("mielke" nil (m iy1 l k iy0))
("mien" nil (m iy1 n))
("mieno" nil (m iy1 n ow0))
("mier" nil (m ay1 er0))
("miera" nil (m iy1 r ax))
("mieras" nil (m ih0 r aa1 z))
("miers" nil (m ay1 er0 z))
("miert" nil (m ay1 er0 t))
("mierzejewski" nil (m ih0 r z ey0 eh1 f s k iy0))
("mierzwa" nil (m iy1 r z w ax))
("mies" nil (m ay1 z))
("miesner" nil (m iy1 z n er0))
("miesse" nil (m iy1 s))
("miffed" nil (m ih1 f t))
("mifflin" nil (m ih1 f l ih0 n))
("mifsud" nil (m ih1 f s ax d))
("mig" nil (m ih1 g))
("migdal" nil (m ih1 g d ax l))
("migent" nil (m ih1 jh ax n t))
("miggins" nil (m ih1 g ih0 n z))
("might" nil (m ay1 t))
("mightiest" nil (m ay1 t iy0 ax s t))
("mightily" nil (m ay1 t ax l iy0))
("mighty" nil (m ay1 t iy0))
("migliaccio" nil (m iy0 g l iy0 aa1 ch iy0 ow0))
("migliore" nil (m iy0 g l iy0 ao1 r iy0))
("mignanelli" nil (m ih0 g n ax n eh1 l iy0))
("mignano" nil (m iy0 g n aa1 n ow0))
("migneault" nil (m ih0 g n ow1))
("mignogna" nil (m iy0 g n ow1 g n ax))
("mignon" nil (m ih1 g n ax n))
("mignone" nil (m iy0 g n ow1 n iy0))
("mignons" nil (m ih1 g n ax n z))
("migra" nil (m ih1 g r ax))
("migraine" nil (m ay1 g r ey1 n))
("migraines" nil (m ay1 g r ey1 n z))
("migrant" nil (m ay1 g r ax n t))
("migrants" nil (m ay1 g r ax n t s))
("migrate" nil (m ay1 g r ey1 t))
("migrated" nil (m ay1 g r ey1 t ih0 d))
("migrating" nil (m ay1 g r ey1 t ih0 ng))
("migration" nil (m ay0 g r ey1 sh ax n))
("migrations" nil (m ay0 g r ey1 sh ax n z))
("migratory" nil (m ay1 g r ax t ao1 r iy0))
("migs" nil (m ih1 g z))
("miguel" nil (m ih0 g eh1 l))
("migues" nil (m iy1 g eh0 s))
("miguez" nil (m iy0 g eh1 z))
("mihai" nil (m iy1 hh ay1))
("mihal" nil (m ay1 hh ax l))
("mihalek" nil (m ih1 hh ax l eh0 k))
("mihalic" nil (m ih0 hh ae1 l ih0 k))
("mihalik" nil (m ih1 hh ax l ih0 k))
("mihalko" nil (m ih0 hh ae1 l k ow0))
("mihaly" nil (m ih1 hh ax l iy0))
("mihelich" nil (m ih1 hh ih0 l ih0 k))
("mihm" nil (m ih1 m))
("mihn" nil (m ih1 n))
("mihok" nil (m ih1 hh ax k))
("mijares" nil (m iy0 y aa1 r eh0 s))
("mika" nil (m iy1 k ax))
("mikado" nil (m ih0 k aa1 d ow0))
("mikaela" nil (m iy0 k ey1 l ax))
("mikal" nil (m iy1 k ax l))
("mike" nil (m ay1 k))
("mikel" nil (m ih1 k ax l))
("mikell" nil (m ih1 k ax l))
("mikels" nil (m ih1 k ax l z))
("mikelson" nil (m ih1 k ih0 l s ax n))
("mikes" nil (m ay1 k s))
("mikesell" nil (m ih1 k ih0 s ax l))
("mikesh" nil (m ih1 k ih0 sh))
("mikeska" nil (m ih0 k eh1 s k ax))
("mikey" nil (m ay1 k iy0))
("mikhail" nil (m ih0 k ey1 l))
("mikita" nil (m ih0 k iy1 t ax))
("mikkelsen" nil (m ih0 k eh1 l s ax n))
("mikkelson" nil (m ih1 k ih0 l s ax n))
("mikkola" nil (m ih0 k ow1 l ax))
("miklas" nil (m ay1 k l ax z))
("mikles" nil (m ay1 k ax l z))
("miklos" nil (m iy1 k l ow0 s))
("miko" nil (m ay1 k ow0))
("mikol" nil (m ih1 k ao0 l))
("mikolajczak" nil (m ih0 k aa1 l ay0 ch ae0 k))
("mikolajczyk" nil (m ih0 k aa1 l ay0 ch ih0 k))
("mikos" nil (m ay1 k ow0 z))
("mikrut" nil (m ih1 k r ax t))
("miksch" nil (m ih1 k sh))
("mikula" nil (m ih0 k uw1 l ax))
("mikulak" nil (m ih0 k uw1 l ax k))
("mikulec" nil (m ih0 k uw1 l ih0 k))
("mikulic" nil (m ih0 k uw1 l ih0 k))
("mikulich" nil (m ih0 k y uw1 l ih0 hh))
("mikulski" nil (m ih0 k ah1 l s k iy0))
("mikus" nil (m ay1 k ax s))
("mikva" nil (m ih1 k v ax))
("mil" nil (m ih1 l))
("milacron" nil (m ih1 l ax k r aa0 n))
("milagro" nil (m ih0 l ae1 g r ow0))
("milam" nil (m ih1 l ax m))
("milan" nil (m ax l aa1 n))
("milani" nil (m iy0 l aa1 n iy0))
("milano" nil (m iy0 l aa1 n ow0))
("milanowski" nil (m ih0 l ax n ao1 f s k iy0))
("milardo" nil (m iy0 l aa1 r d ow0))
("milas" nil (m ay1 l ax z))
("milazzo" nil (m iy0 l aa1 z ow0))
("milbank" nil (m ih1 l b ae1 ng k))
("milbauer" nil (m ih1 l b aw0 er0))
("milberg" nil (m ih1 l b er0 g))
("milberger" nil (m ih1 l b er0 g er0))
("milbourn" nil (m ih0 l b uh1 r n))
("milbourne" nil (m ih0 l b uh1 r n))
("milbrandt" nil (m ih1 l b r ax n t))
("milbrath" nil (m ih1 l b r ax th))
("milburn" nil (m ih1 l b er1 n))
("milby" nil (m ih1 l b iy0))
("milch" nil (m ih1 l ch))
("mild" nil (m ay1 l d))
("mildenberger" nil (m ay1 l d ax n b er0 g er0))
("milder" nil (m ay1 l d er0))
("mildest" nil (m ay1 l d ax s t))
("mildew" nil (m ih1 l d uw1))
("mildews" nil (m ih1 l d uw1 z))
("mildly" nil (m ay1 l d l iy0))
("mildred" nil (m ih1 l d r ih0 d))
("mildrid" nil (m ih1 l d er0 ih0 d))
("mile" nil (m ay1 l))
("mileage" nil (m ay1 l ax jh))
("mileham" nil (m ih1 l ih0 hh ae0 m))
("miler" nil (m ay1 l er0))
("miles" nil (m ay1 l z))
("mileski" nil (m ih0 l eh1 s k iy0))
("milestone" nil (m ay1 l s t ow1 n))
("milestones" nil (m ay1 l s t ow1 n z))
("miletich" nil (m ih1 l ih0 t ih0 k))
("milewski" nil (m ih0 l eh1 f s k iy0))
("miley" nil (m ay1 l iy0))
("milford" nil (m ih1 l f er0 d))
("milgram" nil (m ih1 l g r ae1 m))
("milham" nil (m ih1 l hh ax m))
("milhoan" nil (m ih1 l hh ow0 n))
("milholland" nil (m ih1 l hh ax l ax n d))
("milhollin" nil (m ih0 l hh aa1 l ih0 n))
("milhorn" nil (m ih1 l hh er0 n))
("milhous" nil (m ih1 l hh aw1 s))
("milhouse" nil (m ih1 l hh aw1 s))
("milian" nil (m ih1 l iy0 ax n))
("milicent" nil (m ih1 l ih0 s ih0 n t))
("milich" nil (m ih1 l ih0 k))
("milici" nil (m iy0 l iy1 ch iy0))
("milieu" nil (m ih0 l y uh1))
("milissent" nil (m ih1 l ih0 s ax n t))
("militancy" nil (m ih1 l ax t ax n s iy0))
("militant" nil (m ih1 l ax t ax n t))
("militants" nil (m ih1 l ax t ax n t s))
("militaries" nil (m ih1 l ax t eh1 r iy0 z))
("militarily" nil (m ih1 l ax t eh1 r ax l iy0))
("militarism" nil (m ih1 l ax t er0 ih1 z ax m))
("militaristic" nil (m ih1 l ih0 t er0 ih1 s t ih0 k))
("militarists" nil (m ih1 l ax t er0 ih0 s t s))
("militarized" nil (m ih1 l ax t er0 ay1 z d))
("military" nil (m ih1 l ax t eh1 r iy0))
("militate" nil (m ih1 l ih0 t ey1 t))
("militello" nil (m iy0 l iy0 t eh1 l ow0))
("militia" nil (m ax l ih1 sh ax))
("militiamen" nil (m ax l ih1 sh ax m ih0 n))
("militias" nil (m ax l ih1 sh ax z))
("milito" nil (m iy0 l iy1 t ow0))
("milius" nil (m ay1 l iy0 ih0 s))
("milk" nil (m ih1 l k))
("milke" nil (m ih1 l k))
("milked" nil (m ih1 l k t))
("milken" nil (m ih1 l k ax n))
("milkens" nil (m ih1 l k ax n z))
("milking" nil (m ih1 l k ih0 ng))
("milkman" nil (m ih1 l k m ae1 n))
("milko" nil (m ih1 l k ow0))
("milkovich" nil (m ih1 l k ax v ih0 ch))
("milkowski" nil (m ih0 l k ao1 f s k iy0))
("milks" nil (m ih1 l k s))
("milkshake" nil (m ih1 l k sh ey1 k))
("milkweed" nil (m ih1 l k w iy1 d))
("milky" nil (m ih1 l k iy0))
("mill" nil (m ih1 l))
("millage" nil (m ih1 l ih0 jh))
("millan" nil (m ih1 l ax n))
("millar" nil (m ih1 l er0))
("millard" nil (m ih1 l er0 d))
("millau" nil (m ih1 l aw0))
("millay" nil (m ih0 l ey1))
("millburn" nil (m ih1 l b er0 n))
("mille" nil (m ih1 l))
("milled" nil (m ih1 l d))
("milledge" nil (m ih1 l ih0 jh))
("millen" nil (m ih1 l ax n))
("millender" nil (m ih1 l eh0 n d er0))
("millennia" nil (m ax l eh1 n iy0 ax))
("millennial" nil (m ih0 l eh1 n iy0 ax l))
("millennium" nil (m ax l eh1 n iy0 ax m))
("millenniums" nil (m ax l eh1 n iy0 ax m z))
("miller" nil (m ih1 l er0))
("millerick" nil (m ih1 l er0 ih0 k))
("millers" nil (m ih1 l er0 z))
("milleson" nil (m ih1 l ih0 s ax n))
("millet" nil (m ih1 l ax t))
("millets" nil (m ih1 l ax t s))
("millett" nil (m ih1 l ih0 t))
("millette" nil (m ih0 l eh1 t))
("milley" nil (m ih1 l iy0))
("millhouse" nil (m ih1 l hh aw1 s))
("millibar" nil (m ih1 l ax b aa1 r))
("millican" nil (m ih1 l ih0 k ax n))
("millicent" nil (m ih1 l ih0 s ih0 n t))
("millicom" nil (m ih1 l ih0 k ax m))
("millie" nil (m ih1 l iy0))
("milliet" nil (m ih1 l iy0 eh1 t))
("milligal" nil (m ih1 l ih0 g ax l))
("milligan" nil (m ih1 l ih0 g ax n))
("milligram" nil (m ih1 l ax g r ae1 m))
("milligrams" nil (m ih1 l ax g r ae1 m z))
("millikan" nil (m ih1 l ax k ax n))
("milliken" nil (m ih1 l ih0 k ax n))
("millikin" nil (m ih1 l ih0 k ih0 n))
("milliliters" nil (m ih1 l ax l iy1 t er0 z))
("milliman" nil (m ih1 l ih0 m ax n))
("millimeter" nil (m ih1 l ax m iy1 t er0))
("millimeters" nil (m ih1 l ax m iy1 t er0 z))
("millin" nil (m ih1 l ax n))
("milliner" nil (m ih1 l ih0 n er0))
("milling" nil (m ih1 l ih0 ng))
("millington" nil (m ih1 l ih0 ng t ax n))
("million" nil (m ih1 l y ax n))
("millionaire" nil (m ih1 l y ax n eh1 r))
("millionaires" nil (m ih1 l y ax n eh1 r z))
("millions" nil (m ih1 l y ax n z))
("millionth" nil (m ih1 l y ax n th))
("millionths" nil (m ih1 l y ax n t th s))
("millipore" nil (m ih1 l ih0 p ao1 r))
("milliron" nil (m ih1 l er0 ax n))
("millirons" nil (m ih1 l er0 ow0 n z))
("millis" nil (m ih1 l ih0 s))
("millisent" nil (m ih1 l ax s ax n t))
("milllion" nil (m ih1 l y ax n))
("millman" nil (m ih1 l m ax n))
("milln" nil (m ih1 l n))
("millner" nil (m ih1 l n er0))
("millon" nil (m ih1 l ax n))
("milloy" nil (m ih1 l oy0))
("mills" nil (m ih1 l z))
("millsap" nil (m ih1 l s ae1 p))
("millsaps" nil (m ih1 l s ae1 p s))
("millspaugh" nil (m ih1 l s p ao0))
("millstein" nil (m ih1 l s t ay1 n))
("millstone" nil (m ih1 l s t ow1 n))
("millward" nil (m ih1 l w er0 d))
("millwood" nil (m ih1 l w uh1 d))
("milly" nil (m ih1 l iy0))
("milman" nil (m ih1 l m ax n))
("milne" nil (m ih1 l n))
("milner" nil (m ih1 l n er0))
("milnes" nil (m ih1 l n z))
("milo" nil (m ay1 l ow0))
("milonas" nil (m iy0 l ow1 n aa0 z))
("milone" nil (m ih0 l ow1 n))
("milos" nil (m iy1 l ow0 z))
("milosevic" nil (m ih0 l ow1 s ax v ih0 k))
("milosevich" nil (m ih0 l aa1 s ih0 v ih0 ch))
("milosh" nil (m ih0 l ao1 sh))
("milot" nil (m ih1 l ax t))
("milpitas" nil (m ih0 l p iy1 t ax s))
("milroy" nil (m ih1 l r oy1))
("mils" nil (m ih1 l z))
("milsap" nil (m ih1 l s ae1 p))
("milson" nil (m ih1 l s ax n))
("milstead" nil (m ih1 l s t eh1 d))
("milstein" nil (m ih1 l s t ay1 n))
("milt" nil (m ih1 l t))
("miltenberger" nil (m ih1 l t ax n b er0 g er0))
("miltner" nil (m ih1 l t n er0))
("milton" nil (m ih1 l t ax n))
("miltonic" nil (m ih0 l t aa1 n ih0 k))
("milum" nil (m ih1 l ax m))
("milunovich" nil (m ih0 l uw1 n ax v ih0 ch))
("milward" nil (m ih1 l w er0 d))
("milwaukee" nil (m ih0 l w ao1 k iy0))
("milz" nil (m ih1 l z))
("mim" nil (m ih1 m))
("mimbs" nil (m ih1 m z))
("mime" nil (m ay1 m))
("mimeograph" nil (m ih1 m iy0 ax g r ae1 f))
("mimi" nil (m iy1 m iy0))
("mimic" nil (m ih1 m ih0 k))
("mimicked" nil (m ih1 m ih0 k t))
("mimicking" nil (m ih1 m ih0 k ih0 ng))
("mimicry" nil (m ih1 m ih0 k r iy0))
("mimics" nil (m ih1 m ih0 k s))
("mimis" nil (m iy1 m iy0 z))
("mimms" nil (m ih1 m z))
("mimnaugh" nil (m ih1 m n aw0))
("mimosa" nil (m ih0 m ow1 s ax))
("mims" nil (m ih1 m z))
("min" nil (m ih1 n))
("mina" nil (m iy1 n ax))
("minahan" nil (m ih1 n ax hh ae0 n))
("minami" nil (m iy0 n aa1 m iy0))
("minamide" nil (m ih1 n ax m iy1 d ey0))
("minar" nil (m ay1 n er0))
("minard" nil (m ih0 n aa1 r d))
("minardos" nil (m ih0 n aa1 r d ow0 s))
("minarets" nil (m ih1 n er0 eh1 t s))
("minarik" nil (m ih1 n er0 ih0 k))
("minas" nil (m iy1 n ax s))
("minasian" nil (m ih0 n ae1 zh ih0 n))
("minassian" nil (m ih0 n ae1 s zh ih0 n))
("minatome" nil (m ih1 n ax t ow1 m))
("minc" nil (m ih1 ng k))
("mince" nil (m ih1 n s))
("minced" nil (m ih1 n s t))
("mincemeat" nil (m ih1 n s m iy1 t))
("mincer" nil (m ih1 n s er0))
("mincey" nil (m ih1 n s iy0))
("minch" nil (m ih1 n ch))
("mincher" nil (m ih1 n ch er0))
("minchew" nil (m ih1 n ch y uw0))
("minchey" nil (m ih1 n ch iy0))
("minchin" nil (m ih1 n ch ih0 n))
("mincing" nil (m ih1 n s ih0 ng))
("minckler" nil (m ih1 ng k l er0))
("mincks" nil (m ih1 ng k s))
("mincy" nil (m ih1 n s iy0))
("mind" nil (m ay1 n d))
("minda" nil (m ih1 n d ax))
("mindanao" nil (m ih1 n d ax n aw1))
("minded" nil (m ay1 n d ax d))
("mindedly" nil (m ay1 n d ih0 d l iy0))
("mindedness" nil (m ay1 n d ih0 d n ax s))
("mindel" nil (m ih1 n d ax l))
("minden" nil (m ay1 n d ax n))
("minder" nil (m ay1 n d er0))
("mindful" nil (m ay1 n d f ax l))
("minding" nil (m ay1 n d ih0 ng))
("mindless" nil (m ay1 n d l ax s))
("mindlessly" nil (m ay1 n d l ax s l iy0))
("mindlin" nil (m ih1 n d l ih0 n))
("minds" nil (m ay1 n d z))
("mindset" nil (m ay1 n d s eh1 t))
("mindy" nil (m ih1 n d iy0))
("mine" nil (m ay1 n))
("minea" nil (m ih0 n iy1 ax))
("minear" nil (m ih0 n ih1 r))
("mineau" nil (m ih0 n ow1))
("minebea" nil (m ih1 n ax b iy1 ax))
("mined" nil (m ay1 n d))
("mineer" nil (m ay1 n er0))
("minefield" nil (m ay1 n f iy1 l d))
("minehan" nil (m ih1 n ih0 hh ae0 n))
("minehart" nil (m ay1 n hh aa1 r t))
("minella" nil (m ih0 n eh1 l ax))
("minelli" nil (m ih0 n eh1 l iy0))
("mineo" nil (m ih1 n iy0 ow0))
("mineola" nil (m ih1 n iy0 ow1 l ax))
("mineowners" nil (m ay1 n ow1 n er0 z))
("miner" nil (m ay1 n er0))
("minera" nil (m ih0 n eh1 r ax))
("mineral" nil (m ih1 n er0 ax l))
("mineralization" nil (m ih1 n er0 ax l ax z ey1 sh ax n))
("mineralogically" nil (m ih1 n er0 ax l aa1 jh ih0 k ax l iy0))
("mineralogist" nil (m ih1 n er0 ae1 l ax jh ih0 s t))
("mineralogy" nil (m ih1 n er0 aa1 l ax jh iy0))
("minerals" nil (m ih1 n er0 ax l z))
("minerd" nil (m ih1 n er0 d))
("miners" nil (m ay1 n er0 z))
("minerva" nil (m ax n er1 v ax))
("minervini" nil (m iy1 n er0 v iy1 n iy0))
("mines" nil (m ay1 n z))
("minesweeper" nil (m ay1 n s w iy1 p er0))
("minesweepers" nil (m ay1 n s w iy1 p er0 z))
("minet" nil (m ih1 n ih0 t))
("mineta" nil (m ih0 n ey1 t ax))
("minette" nil (m ih0 n eh1 t))
("mineworkers" nil (m ay1 n w er1 k er0 z))
("ming" nil (m ih1 ng))
("minge" nil (m ih1 n jh))
("minger" nil (m ih1 ng er0))
("minges" nil (m ih1 n jh ih0 z))
("mingle" nil (m ih1 ng g ax l))
("mingled" nil (m ih1 ng g ax l d))
("mingles" nil (m ih1 ng g ax l z))
("mingling" nil (m ih1 ng g ax l ih0 ng))
("mingo" nil (m iy1 ng g ow0))
("mings" nil (m ih1 ng z))
("mingus" nil (m ih1 ng g ih0 s))
("minh" nil (m ih1 n))
("mini" nil (m ih1 n iy0))
("miniard" nil (m ih1 n y er0 d))
("miniature" nil (m ih1 n iy0 ax ch uh1 r))
("miniatures" nil (m ih1 n iy0 ax ch er0 z))
("miniaturization" nil (m ih1 n iy0 ax ch er0 ih0 z ey1 sh ax n))
("miniaturized" nil (m ih1 n ih0 ch er0 ay1 z d))
("minibuses" nil (m ih1 n iy0 b ah1 s ih0 z))
("minicar" nil (m ih1 n iy0 k aa1 r))
("minicars" nil (m ih1 n iy0 k aa1 r z))
("minich" nil (m ih1 n ih0 ch))
("minichiello" nil (m iy0 n iy0 k iy0 eh1 l ow0))
("minick" nil (m ih1 n ih0 k))
("minicomputer" nil (m ih1 n iy0 k ax m p y uw1 t er0))
("minicomputers" nil (m ih1 n iy0 k ax m p y uw1 t er0 z))
("minicucci" nil (m iy0 n iy0 k uw1 ch iy0))
("minidisc" nil (m ih1 n iy0 d ih1 s k))
("minier" nil (m ih1 n iy0 er0))
("minihan" nil (m ih1 n ih0 hh ae0 n))
("minilab" nil (m ih1 n iy0 l ae1 b))
("minilabs" nil (m ih1 n iy0 l ae1 b z))
("minimal" nil (m ih1 n ax m ax l))
("minimalism" nil (m ih1 n ax m ax l ih1 z ax m))
("minimalist" nil (m ih1 n ax m ax l ih0 s t))
("minimally" nil (m ih1 n ax m ax l iy0))
("minimill" nil (m ih1 n iy0 m aa1 l))
("minimills" nil (m ih1 n iy0 m ih1 l z))
("minimize" nil (m ih1 n ax m ay1 z))
("minimized" nil (m ih1 n ax m ay1 z d))
("minimizes" nil (m ih1 n ax m ay1 z ax z))
("minimizing" nil (m ih1 n ax m ay1 z ih0 ng))
("minimum" nil (m ih1 n ax m ax m))
("minimums" nil (m ih1 n ih0 m ax m z))
("mining" nil (m ay1 n ih0 ng))
("mininger" nil (m ay1 n ih0 ng er0))
("minion" nil (m ih1 n y ax n))
("minions" nil (m ih1 n y ax n z))
("minis" nil (m ih1 n iy0 z))
("miniscribe" nil (m ih1 n iy0 s k r ay1 b))
("miniscule" nil (m ih1 n ih0 s k y uw0 l))
("miniseries" nil (m ih1 n ih0 s eh1 r iy0 z))
("minish" nil (m ih1 n ih0 sh))
("miniskirt" nil (m ih1 n iy0 s k er1 t))
("miniskirts" nil (m ih1 n iy0 s k er1 t s))
("minister" nil (m ih1 n ax s t er0))
("ministerial" nil (m ih1 n ih0 s t iy1 r iy0 ax l))
("ministers" nil (m ih1 n ax s t er0 z))
("ministrations" nil (m ih1 n ax s t r ey1 sh ax n z))
("ministries" nil (m ih1 n ih0 s t r iy0 z))
("ministry" nil (m ih1 n ax s t r iy0))
("minisupercomputer" nil (m ih1 n iy0 s uw1 p er0 k ah1 m p y uw1 t er0))
("minisupercomputers" nil (m ih1 n iy0 s uw1 p er0 k ah1 m p y uw1 t er0 z))
("minit" nil (m ih1 n ih0 t))
("minitel" nil (m ih1 n ax t eh1 l))
("minium" nil (m ih1 n iy0 ax m))
("minivan" nil (m ih1 n iy0 v ae1 n))
("minivans" nil (m ih1 n iy0 v ae1 n z))
("minix" nil (m ih1 n ih0 k s))
("minjares" nil (m iy0 n y aa1 r eh0 s))
("minjarez" nil (m iy0 n y aa1 r eh0 z))
("mink" nil (m ih1 ng k))
("minke" nil (m ih1 ng k))
("minkel" nil (m ih1 ng k ax l))
("minkin" nil (m ih1 ng k ih0 n))
("minkler" nil (m ih1 ng k l er0))
("minkoff" nil (m ih1 ng k ao1 f))
("minkow" nil (m ih1 ng k aw0))
("minks" nil (m ih1 ng k s))
("minna" nil (m ih1 n ax))
("minnaar" nil (m ih0 n aa1 r))
("minneapolis" nil (m ih1 n iy0 ae1 p ax l ih0 s))
("minnelli" nil (m ih0 n eh1 l iy0))
("minner" nil (m ih1 n er0))
("minnesota" nil (m ih1 n ih0 s ow1 t ax))
("minnesotan" nil (m ih1 n ax s ow1 t ax n))
("minnesotans" nil (m ih1 n ax s ow1 t ax n z))
("minnetonka" nil (m ih1 n ih0 t ao1 ng k ax))
("minney" nil (m ih1 n iy0))
("minnich" nil (m ih1 n ih0 ch))
("minnick" nil (m ih1 n ih0 k))
("minnie" nil (m ih1 n iy0))
("minniear" nil (m ih1 n iy0 ih1 r))
("minnifield" nil (m ih1 n ax f iy1 l d))
("minnig" nil (m ih1 n ih0 g))
("minnis" nil (m ih1 n ih0 s))
("minniti" nil (m iy0 n iy1 t iy0))
("minnix" nil (m ih1 n ih0 k s))
("minnow" nil (m ih1 n ow0))
("minnows" nil (m ih1 n ow0 z))
("minns" nil (m ih1 n z))
("minntech" nil (m ih1 n t eh1 k))
("minny" nil (m ih1 n iy0))
("mino" nil (m iy1 n ow0))
("minoan" nil (m ax n ow1 ax n))
("minogue" nil (m iy1 n ao0 g))
("minoli" nil (m ih0 n ow1 l iy0))
("minolta" nil (m ih0 n aa1 l t ah1))
("minor" nil (m ay1 n er0))
("minorca" nil (m ax n ao1 r k ax))
("minorco" nil (m ih0 n ao1 r k ow0))
("minoring" nil (m ay1 n er0 ih0 ng))
("minorite" nil (m ih1 n er0 ay1 t))
("minorities" nil (m ay0 n ao1 r ax t iy0 z))
("minority" nil (m ay0 n ao1 r ax t iy0))
("minors" nil (m ay1 n er0 z))
("minoru" nil (m ih0 n ao1 r uw0))
("minot" nil (m ih1 n ax t))
("minott" nil (m ih1 n ax t))
("minow" nil (m ih1 n ow0))
("minoxidil" nil (m ih0 n aa1 k s ih0 d ih0 l))
("minpeco" nil (m ih0 n p eh1 k ow0))
("minshall" nil (m ih1 n sh ax l))
("minshew" nil (m ih1 n sh uw0))
("minsk" nil (m ih1 n s k))
("minskoff" nil (m ih1 n s k ao0 f))
("minsky" nil (m ih1 n s k iy0))
("minson" nil (m ih1 n s ax n))
("minstar" nil (m ih1 n s t aa1 r))
("minster" nil (m ih1 n s t er0))
("minstrel" nil (m ih1 n s t r ax l))
("minstrels" nil (m ih1 n s t r ax l z))
("mint" nil (m ih1 n t))
("minta" nil (m ih1 n t ax))
("mintage" nil (m ih1 n t ax jh))
("minted" nil (m ih1 n t ih0 d))
("minteer" nil (m ih1 n t ih1 r))
("minter" nil (m ih1 n t er0))
("mintha" nil (m ih1 n th ax))
("mintier" nil (m ih1 n t iy0 er0))
("minting" nil (m ih1 n t ih0 ng))
("minto" nil (m ih1 n t ow0))
("minton" nil (m ih1 n t ax n))
("mints" nil (m ih1 n t s))
("minturn" nil (m ih1 n t er1 n))
("mintz" nil (m ih1 n t s))
("mintzer" nil (m ih1 n t z er0))
("minuet" nil (m ih1 n y ax w eh1 t))
("minus" nil (m ay1 n ax s))
("minuscule" nil (m ih1 n ax s k y uw1 l))
("minuses" nil (m ay1 n ax s ih0 z))
("minute" n (m ih1 n ax t))
("minute" j (m ih1 n uw0 t))
("minutely" nil (m ih1 n ax t l iy0))
("minuteman" nil (m ih1 n ax t m ae1 n))
("minutes" nil (m ih1 n ax t s))
("minutiae" nil (m ih0 n uw1 sh iy0 ax))
("minyard" nil (m ih1 n y aa0 r d))
("miocene" nil (m ay1 ax s iy1 n))
("mion" nil (m ay1 ax n))
("mione" nil (m ay1 ow1 n))
("miot" nil (m ay1 ax t))
("miotke" nil (m ay1 aa1 t k iy0))
("mips" nil (m ih1 p s))
("mir" nil (m ih1 r))
("mira" nil (m ih1 r ax))
("mirabal" nil (m ih1 r ax b ax l))
("mirabel" nil (m ih0 r aa0 b eh1 l))
("mirabella" nil (m ih0 r aa0 b eh1 l ax))
("mirabelle" nil (m ao1 r ax b ax l))
("mirabelli" nil (m ih0 r aa0 b eh1 l iy0))
("mirabile" nil (m ih0 r aa1 b ax l))
("mirabito" nil (m ih0 r aa0 b iy1 t ow0))
("miracid" nil (m ih1 r ae1 s ax d))
("miracle" nil (m ih1 r ax k ax l))
("miracles" nil (m ih1 r ax k ax l z))
("miraculous" nil (m er0 ae1 k y ax l ax s))
("miraculously" nil (m er0 ae1 k y ax l ax s l iy0))
("mirada" nil (m ih0 r aa1 d ax))
("mirage" nil (m er0 aa1 zh))
("miraglia" nil (m ih0 r ae1 g l iy0 ax))
("mirak" nil (m ih1 r ae0 k))
("miramax" nil (m ih1 r ax m ae1 k s))
("miramontes" nil (m ih0 r aa0 m ow1 n t eh0 s))
("miranda" nil (m er0 ae1 n d ax))
("mirando" nil (m ih0 r ae1 n d ow0))
("mire" nil (m ay1 r))
("mired" nil (m ay1 r d))
("mireles" nil (m ih0 r ey1 l eh0 s))
("mirelez" nil (m ih0 r ey1 l eh0 z))
("mirella" nil (m ih0 r eh1 l ax))
("mirelle" nil (m er0 eh1 l))
("mirenda" nil (m ih0 r eh1 n d ax))
("mires" nil (m ay1 r z))
("miriam" nil (m ih1 r iy0 ax m))
("mirick" nil (m ih1 r ih0 k))
("mirilla" nil (m ih0 r ih1 l ax))
("mirkin" nil (m er1 k ih0 n))
("mirna" nil (m er1 n ax))
("miro" nil (m ih1 r ow0))
("miron" nil (m ih0 r ao1 n))
("mironenko" nil (m ih1 r ow0 n eh1 ng k ow0))
("miroslav" nil (m ih1 r ax s l aa0 v))
("mirra" nil (m ih1 r ax))
("mirror" nil (m ih1 r er0))
("mirrored" nil (m ih1 r er0 d))
("mirroring" nil (m ih1 r er0 ih0 ng))
("mirrors" nil (m ih1 r er0 z))
("mirsky" nil (m er1 s k iy0))
("mirth" nil (m er1 th))
("mirti" nil (m er1 t iy0))
("mirtle" nil (m er1 t ax l))
("mirto" nil (m ih1 r t ow0))
("mirv" nil (m er1 v))
("mirza" nil (m er1 z ax))
("mis" nil (m ih1 s))
("misadventures" nil (m ih0 s ax d v eh1 n ch er0 z))
("misallocated" nil (m ih0 s ae1 l ax k ey1 t ih0 d))
("misallocation" nil (m ih0 s ae1 l ax k ey1 sh ax n))
("misanthrope" nil (m ih1 s ax n th r ow1 p))
("misapplication" nil (m ih1 s ae0 p l ax k ey1 sh ax n))
("misapplied" nil (m ih1 s ax p l ay1 d))
("misapplying" nil (m ih0 s ax p l ay1 ih0 ng))
("misapprehension" nil (m ih0 s ae1 p r iy0 hh eh1 n sh ax n))
("misappropriate" nil (m ih0 s ax p r ow1 p r iy0 ey1 t))
("misappropriated" nil (m ih1 s ax p r ow1 p r iy0 ey1 t ih0 d))
("misappropriating" nil (m ih1 s ax p r ow1 p r iy0 ey1 t ih0 ng))
("misappropriation" nil (m ih1 s ax p r ow1 p r iy0 ey1 sh ax n))
("misawa" nil (m ih0 s aa1 w ax))
("misbegotten" nil (m ih1 s b ax g aa1 t ax n))
("misbehaving" nil (m ih1 s b ax hh ey1 v ih0 ng))
("misbehavior" nil (m ih1 s b ax hh ey1 v y er0))
("misbrener" nil (m ih1 s b r eh1 n er0))
("miscalculated" nil (m ih0 s k ae1 l k y ax l ey1 t ih0 d))
("miscalculation" nil (m ih0 s k ae1 l k y ax l ey1 sh ax n))
("miscalculations" nil (m ih0 s k ae1 l k y ax l ey1 sh ax n z))
("miscarriage" nil (m ih0 s k eh1 r ax jh))
("miscarriages" nil (m ih0 s k eh1 r ih0 jh ih0 z))
("miscast" nil (m ih0 s k ae1 s t))
("miscayuna" nil (m ih1 s k ay0 y uw1 n ax))
("miscellaneous" nil (m ih1 s ax l ey1 n iy0 ax s))
("miscellany" nil (m ih1 s ax l ey1 n iy0))
("misch" nil (m ih1 sh))
("mischa" nil (m ih1 sh ax))
("mischarged" nil (m ih0 s ch aa1 r jh d))
("mischarges" nil (m ih0 s ch aa1 r jh ih0 z))
("mischarging" nil (m ih0 s ch aa1 r jh ih0 ng))
("mischel" nil (m ih1 sh ax l))
("mischer" nil (m ih1 sh er0))
("mischief" nil (m ih1 s ch ax f))
("mischievous" nil (m ih1 s ch ax v ax s))
("mischke" nil (m ih1 sh k))
("mischler" nil (m ih1 sh ax l er0))
("miscibility" nil (m ih1 s ih0 b ih1 l ih0 t iy0))
("miscible" nil (m ih1 s ax b ax l))
("misconceived" nil (m ih0 s k ax n s iy1 v d))
("misconception" nil (m ih0 s k ax n s eh1 p sh ax n))
("misconceptions" nil (m ih1 s k ax n s eh1 p sh ax n z))
("misconduct" n (m ih0 s k aa1 n d ax k t))
("misconduct" v (m ih0 s k ax n d ah1 k t))
("misconstrue" nil (m ih0 s k ax n s t r uw1))
("misconstrued" nil (m ih1 s k ax n s t r uw1 d))
("miscount" nil (m ih1 s k aw1 n t))
("miscreants" nil (m ih1 s k r iy0 ax n t s))
("miscreations" nil (m ih1 s k r iy0 ey1 sh ax n z))
("miscues" nil (m ih1 s k y uw1 z))
("misdeed" nil (m ih1 s d iy1 d))
("misdeeds" nil (m ih0 s d iy1 d z))
("misdemeanor" nil (m ih1 s d ax m iy1 n er0))
("misdemeanors" nil (m ih1 s d ax m iy1 n er0 z))
("misdiagnosed" nil (m ih0 s d ay1 ih0 g n ow1 s t))
("misdirected" nil (m ih1 s d ih0 r eh1 k t ih0 d))
("misek" nil (m ih1 s eh0 k))
("misener" nil (m ih1 s iy0 n er0))
("misenheimer" nil (m ih1 s ih0 n hh ay0 m er0))
("miser" nil (m ay1 z er0))
("miserable" nil (m ih1 z er0 ax b ax l))
("miserables" nil (m ih1 z er0 ax b ax l z))
("miserably" nil (m ih1 z er0 ax b l iy0))
("miseries" nil (m ih1 z er0 iy0 z))
("miserly" nil (m ay1 z er0 l iy0))
("misery" nil (m ih1 z er0 iy0))
("mises" nil (m ay1 z ih0 z))
("misfeldt" nil (m ih1 s f ih0 l t))
("misfire" nil (m ih0 s f ay1 er0))
("misfit" nil (m ih0 s f ih1 t))
("misfits" nil (m ih1 s f ih1 t s))
("misfortune" nil (m ih0 s f ao1 r ch ax n))
("misfortunes" nil (m ih0 s f ao1 r ch ax n z))
("misgivings" nil (m ih0 s g ih1 v ih0 ng z))
("misgovernment" nil (m ih0 s g ah1 v er0 n m ax n t))
("misguided" nil (m ih0 s g ay1 d ih0 d))
("mish" nil (m ih1 sh))
("mishandle" nil (m ih0 s hh ae1 n d ax l))
("mishandled" nil (m ih0 s hh ae1 n d ax l d))
("mishandling" nil (m ih0 s hh ae1 n d l ih0 ng))
("mishap" nil (m ih1 s hh ae1 p))
("mishaps" nil (m ih1 s hh ae1 p s))
("mishawaka" nil (m ih1 sh ax w ao1 k ax))
("mishawaum" nil (m ih1 sh ax w aa1 m))
("mishawum" nil (m ih1 sh ax w ah1 m))
("mishkin" nil (m iy1 sh k iy0 n))
("mishler" nil (m ih1 sh l er0))
("mishmash" nil (m ih1 sh m ae1 sh))
("mishoe" nil (m iy1 sh uw0))
("mishra" nil (m ih1 sh r ax))
("misiak" nil (m ih1 s iy0 ae0 k))
("misiaszek" nil (m ih0 s iy0 aa1 sh eh0 k))
("misidentified" nil (m ih0 s ay0 d eh1 n t ih0 f ay1 d))
("misiewicz" nil (m ih1 s ax v ih0 ch))
("misinformation" nil (m ih0 z ih0 n f er0 m ey1 sh ax n))
("misinformed" nil (m ih0 s ih0 n f ao1 r m d))
("misinterpret" nil (m ih0 s ih0 n t er1 p r ax t))
("misinterpretation" nil (m ih0 s ih0 n t er1 p r ax t ey1 sh ax n))
("misinterpreted" nil (m ih0 s ih0 n t er1 p r ax t ih0 d))
("misinterpreting" nil (m ih0 s ih0 n t er1 p r ax t ih0 ng))
("misjudged" nil (m ih0 s jh ah1 jh d))
("misjudgment" nil (m ih0 s jh ah1 jh m ax n t))
("misjudgments" nil (m ih0 s jh ah1 jh m ax n t s))
("miska" nil (m ih1 s k ax))
("miske" nil (m ih1 s k))
("miskell" nil (m ih1 s k ax l))
("miskito" nil (m ih0 s k iy1 t ow0))
("miskitos" nil (m ih0 s k iy1 t ow0 s))
("misko" nil (m ih1 s k ow0))
("mislabeled" nil (m ih0 s l ey1 b ax l d))
("mislabeling" nil (m ih0 s l ey1 b ax l ih0 ng))
("mislaid" nil (m ih0 s l ey1 d))
("mislead" nil (m ih0 s l iy1 d))
("misleading" nil (m ih0 s l iy1 d ih0 ng))
("misleadingly" nil (m ih0 s l iy1 d ih0 ng l iy0))
("misleads" nil (m ih0 s l iy1 d z))
("misled" nil (m ih0 s l eh1 d))
("mismanaged" nil (m ih0 s m ae1 n ih0 jh d))
("mismanagement" nil (m ih0 s m ae1 n ih0 jh m ax n t))
("mismanaging" nil (m ih0 s m ae1 n ih0 jh ih0 ng))
("mismatch" nil (m ih0 s m ae1 ch))
("mismatched" nil (m ih0 s m ae1 ch t))
("mismatches" nil (m ih0 s m ae1 ch ih0 z))
("misner" nil (m ih1 z n er0))
("misnomer" nil (m ih0 s n ow1 m er0))
("misperceives" nil (m ih0 s p er0 s iy1 v z))
("misperception" nil (m ih1 s p er0 s eh1 p sh ax n))
("misperceptions" nil (m ih0 s p er0 s eh1 p sh ax n z))
("misplace" nil (m ih0 s p l ey1 s))
("misplaced" nil (m ih0 s p l ey1 s t))
("mispriced" nil (m ih0 s p r ay1 s t))
("misprint" v (m ih0 s p r ih1 n t))
("misprint" n (m ih1 s p r ih0 n t))
("misprints" v (m ih0 s p r ih1 n t s))
("misprints" n (m ih1 s p r ih0 n t s))
("misprision" nil (m ih0 s p r ih1 zh ax n))
("misquote" nil (m ih0 s k w ow1 t))
("misquoted" nil (m ih0 s k w ow1 t ih0 d))
("misread" nil (m ih0 s r iy1 d))
("misreading" nil (m ih0 s r iy1 d ih0 ng))
("misreported" nil (m ih0 s r ih0 p ao1 r t ih0 d))
("misrepresent" nil (m ih0 s r eh1 p r ax z eh1 n t))
("misrepresentation" nil (m ih1 s r eh0 p r ih0 z eh0 n t ey1 sh ax n))
("misrepresentations" nil (m ih1 s r eh0 p r ih0 z eh0 n t ey1 sh ax n z))
("misrepresented" nil (m ih1 s r eh0 p r ih0 z eh1 n t ih0 d))
("misrepresenting" nil (m ih0 s r eh1 p r ax z eh1 n t ih0 ng))
("misrepresents" nil (m ih0 s r eh1 p r ax z eh1 n t s))
("misrule" nil (m ih0 s r uw1 l))
("miss" nil (m ih1 s))
("missal" nil (m ih1 s ax l))
("missed" nil (m ih1 s t))
("missel" nil (m ih1 s ax l))
("misses" nil (m ih1 s ax z))
("missett" nil (m ih1 s ax t))
("missey" nil (m ih1 s iy0))
("misshapen" nil (m ih0 s sh ey1 p ax n))
("missie" nil (m ih1 s iy0))
("missildine" nil (m ih0 s iy0 l d iy1 n iy0))
("missile" nil (m ih1 s ax l))
("missiles" nil (m ih1 s ax l z))
("missimer" nil (m ih1 s ih0 m er0))
("missing" nil (m ih1 s ih0 ng))
("mission" nil (m ih1 sh ax n))
("missionaries" nil (m ih1 sh ax n eh1 r iy0 z))
("missionary" nil (m ih1 sh ax n eh1 r iy0))
("missions" nil (m ih1 sh ax n z))
("mississauga" nil (m ih1 s ih0 s ao1 g ax))
("mississippi" nil (m ih1 s ih0 s ih1 p iy0))
("mississippian" nil (m ih1 s ih0 s ih1 p iy0 ax n))
("missive" nil (m ih1 s ih0 v))
("missler" nil (m ih1 s l er0))
("missoula" nil (m ih0 z uw1 l ax))
("missouri" nil (m ax z uh1 r iy0))
("misspeak" nil (m ih0 s s p iy1 k))
("misspelled" nil (m ih0 s s p eh1 l d))
("misspelling" nil (m ih0 s s p eh1 l ih0 ng))
("misspending" nil (m ih0 s s p eh1 n d ih0 ng))
("misspent" nil (m ih0 s s p eh1 n t))
("misspoke" nil (m ih0 s s p ow1 k))
("misstated" nil (m ih0 s s t ey1 t ih0 d))
("misstatement" nil (m ih0 s t ey1 t m ax n t))
("misstatements" nil (m ih0 s t ey1 t m ax n t s))
("misstates" nil (m ih0 s s t ey1 t s))
("misstating" nil (m ih0 s s t ey1 t ih0 ng))
("misstep" nil (m ih0 s s t eh1 p))
("missteps" nil (m ih0 s s t eh1 p s))
("missy" nil (m ih1 s iy0))
("mist" nil (m ih1 s t))
("mistake" nil (m ih0 s t ey1 k))
("mistaken" nil (m ih0 s t ey1 k ax n))
("mistakenly" nil (m ih0 s t ey1 k ax n l iy0))
("mistakes" nil (m ih0 s t ey1 k s))
("mistaking" nil (m ih0 s t ey1 k ih0 ng))
("mister" nil (m ih1 s t er0))
("misters" nil (m ih1 s t er0 z))
("mistic" nil (m ih1 s t ih0 k))
("mistler" nil (m ih1 s t l er0))
("mistletoe" nil (m ih1 s ax l t ow1))
("mistook" nil (m ih0 s t uh1 k))
("mistral" nil (m ih1 s t r ax l))
("mistreat" nil (m ih0 s t r iy1 t))
("mistreated" nil (m ih0 s t r iy1 t ih0 d))
("mistreating" nil (m ih0 s t r iy1 t ih0 ng))
("mistreatment" nil (m ih0 s t r iy1 t m ax n t))
("mistreats" nil (m ih0 s t r iy1 t s))
("mistress" nil (m ih1 s t r ax s))
("mistresses" nil (m ih1 s t r ax s ax z))
("mistretta" nil (m ih0 s t r eh1 t ax))
("mistrial" nil (m ih0 s t r ay1 ax l))
("mistrials" nil (m ih0 s t r ay1 ax l z))
("mistrust" nil (m ih0 s t r ah1 s t))
("mistrusted" nil (m ih0 s t r ah1 s t ih0 d))
("mistry" nil (m ih1 s t r iy0))
("mists" nil (m ih1 s t s))
("misty" nil (m ih1 s t iy0))
("misunderstand" nil (m ih0 s ax n d er0 s t ae1 n d))
("misunderstanding" nil (m ih1 s ax n d er0 s t ae1 n d ih0 ng))
("misunderstandings" nil (m ih0 s ah1 n d er0 s t ae1 n d ih0 ng z))
("misunderstood" nil (m ih1 s ax n d er0 s t uh1 d))
("misuraca" nil (m ih0 s uh0 r aa1 k ax))
("misuse" n (m ih0 s y uw1 s))
("misuse" v (m ih0 s y uw1 z))
("misused" nil (m ih0 s y uw1 z d))
("misuses" n (m ih0 s y uw1 s ih0 z))
("misuses" v (m ih0 s y uw1 z ih0 z))
("misusing" nil (m ih0 s y uw1 z ih0 ng))
("mita" nil (m iy1 t ax))
("mitamura" nil (m iy1 t ax m uh1 r ax))
("mitch" nil (m ih1 ch))
("mitcham" nil (m ih1 ch ax m))
("mitchel" nil (m ih1 ch ax l))
("mitchell" nil (m ih1 ch ax l))
("mitchelson" nil (m ih1 ch ax l s ax n))
("mitcheltree" nil (m ih0 ch ih0 l t r iy1))
("mitchem" nil (m ih1 ch ih0 m))
("mitchener" nil (m ih1 ch iy0 n er0))
("mitchner" nil (m ih1 ch n er0))
("mitchum" nil (m ih1 ch ax m))
("mite" nil (m ay1 t))
("mitek" nil (m ay1 t eh1 k))
("mitel" nil (m ay1 t eh1 l))
("miter" nil (m ay1 t er0))
("mitering" nil (m ay1 t er0 ih0 ng))
("mites" nil (m ay1 t s))
("mithcell" nil (m ih1 th s ax l))
("mithridates" nil (m ih1 th r ih0 d ey1 t s))
("mithun" nil (m ih1 th ax n))
("miti" nil (m ih1 t iy1))
("mitigate" nil (m ih1 t ax g ey1 t))
("mitigated" nil (m ih1 t ax g ey1 t ih0 d))
("mitigating" nil (m ih1 t ax g ey1 t ih0 ng))
("mitigation" nil (m ih1 t ih0 g ey1 sh ax n))
("mitman" nil (m ih1 t m ax n))
("mitnick" nil (m ih1 t n ih0 k))
("mitofsky" nil (m ih0 t ao1 f s k iy0))
("mitra" nil (m ih1 t r ax))
("mitral" nil (m ay1 t r ax l))
("mitrano" nil (m iy0 t r aa1 n ow0))
("mitre" nil (m ih1 t r iy0))
("mitro" nil (m ih1 t r ow0))
("mitrovich" nil (m ih1 t r ax v ih0 ch))
("mitsch" nil (m ih1 ch))
("mitschke" nil (m ih1 ch k))
("mitsotakis" nil (m ih1 t s ow0 t aa1 k ih0 s))
("mitsuba" nil (m iy0 t s uw1 b ax))
("mitsubishi" nil (m ih1 t s uw1 b iy1 sh iy0))
("mitsui" nil (m iy0 t s uw1 iy0))
("mitsukoshi" nil (m iy1 t s uw1 k ow1 sh iy0))
("mitsuru" nil (m iy0 t s uw1 r uw0))
("mitt" nil (m ih1 t))
("mittag" nil (m ih1 t ax g))
("mittan" nil (m ih1 t ax n))
("mitteleuropa" nil (m ih1 t eh1 l uw1 r ow1 p ax))
("mittelman" nil (m ih1 t ax l m ax n))
("mittelstadt" nil (m ih1 t ih0 l sh t ae0 t))
("mittelstaedt" nil (m ih1 t ih0 l s t eh0 t))
("mittelsteadt" nil (m ih1 t ih0 l s t eh0 t))
("mitten" nil (m ih1 t ax n))
("mittendorf" nil (m ih1 t ih0 n d ao0 r f))
("mittens" nil (m ih1 t ax n z))
("mitterand" nil (m iy1 t er0 ao1 n d))
("mitterrand" nil (m iy1 t er0 ao1 n d))
("mittleman" nil (m ih1 t ax l m ax n))
("mittler" nil (m ih1 t l er0))
("mittman" nil (m ih1 t m ax n))
("mitton" nil (m ih1 t ax n))
("mitts" nil (m ih1 t s))
("mityunov" nil (m ih1 t y uw0 n aa0 v))
("mitzel" nil (m ih1 t z ax l))
("mitzi" nil (m ih1 t s iy0))
("mitzner" nil (m ih1 t s n er0))
("mitzvah" nil (m ih1 t s v ax))
("miura" nil (m iy0 uh1 r ax))
("mix" nil (m ih1 k s))
("mixed" nil (m ih1 k s t))
("mixer" nil (m ih1 k s er0))
("mixers" nil (m ih1 k s er0 z))
("mixes" nil (m ih1 k s ax z))
("mixing" nil (m ih1 k s ih0 ng))
("mixon" nil (m ih1 k s ax n))
("mixson" nil (m ih1 k s ax n))
("mixte" nil (m ih1 k s t))
("mixter" nil (m ih1 k s t er0))
("mixture" nil (m ih1 k s ch er0))
("mixtures" nil (m ih1 k s ch er0 z))
("mixup" nil (m ih1 k s ax p))
("miyagawa" nil (m iy1 aa0 g aa1 w ax))
("miyahara" nil (m iy1 y aa0 hh aa1 r ax))
("miyake" nil (m iy0 y aa1 k ey0))
("miyako" nil (m iy1 ah1 k ow1))
("miyamori" nil (m iy1 aa0 m ao1 r iy0))
("miyamoto" nil (m iy0 y aa0 m ow1 t ow0))
("miyasaki" nil (m iy1 y aa0 s aa1 k iy0))
("miyasato" nil (m iy0 y aa0 s aa1 t ow0))
("miyashiro" nil (m iy0 y aa0 sh ih1 r ow0))
("miyazaki" nil (m iy1 y aa0 z aa1 k iy0))
("miyazawa" nil (m iy1 y ax z aa1 w ax))
("mizar" nil (m ay1 z aa1 r))
("mize" nil (m ay1 z))
("mizel" nil (m ay1 z ax l))
("mizell" nil (m ih1 z ax l))
("mizelle" nil (m ih0 z eh1 l))
("mizen" nil (m ay1 z ax n))
("mizer" nil (m ay1 z er0))
("mizner" nil (m ih1 z n er0))
("mizrahi" nil (m iy0 z r aa1 hh iy0))
("mizuno" nil (m ih0 z uw1 n ow0))
("mizzell" nil (m ih1 z ax l))
("mkhatshwa" nil (m ax k ae1 ch w ax))
("mladic" nil (m l aa1 d ih0 ch))
("mlecko" nil (m l eh1 k ow0))
("mlotok" nil (m l ow1 t aa1 k))
("mme" nil (m iy1))
("mmonoxide" nil (m ax n aa1 k s ay0 d))
("mnemonics" nil (n ih0 m aa1 n ih0 k s))
("mnookin" nil (m n uh1 k ax n))
("mo" nil (m ow1))
("moab" nil (m ow1 ae0 b))
("moacq" nil (m ow1 k))
("moad" nil (m ow1 d))
("moak" nil (m ow1 k))
("moakley" nil (m ow1 k l iy0))
("moami" nil (m ow1 m iy0))
("moammar" nil (m ow1 ax m aa1 r))
("moan" nil (m ow1 n))
("moaned" nil (m ow1 n d))
("moaning" nil (m ow1 n ih0 ng))
("moans" nil (m ow1 n z))
("moat" nil (m ow1 t))
("moates" nil (m ow1 ey1 t s))
("moats" nil (m ow1 t s))
("mob" nil (m aa1 b))
("mobay" nil (m ow0 b ey1))
("mobbed" nil (m aa1 b d))
("mobbs" nil (m aa1 b z))
("mobe" nil (m ow1 b))
("moberg" nil (m ow1 b er0 g))
("moberly" nil (m ow1 b er0 l iy0))
("mobil" nil (m ow1 b ax l))
("mobile" nil (m ow1 b ax l))
("mobiles" nil (m ow1 b ax l z))
("mobilia" nil (m ow0 b iy1 l iy0 ax))
("mobiliare" nil (m ow1 b ax l eh1 r))
("mobility" nil (m ow0 b ih1 l ax t iy0))
("mobilization" nil (m ow1 b ax l ax z ey1 sh ax n))
("mobilizations" nil (m ow1 b ax l ax z ey1 sh ax n z))
("mobilize" nil (m ow1 b ax l ay1 z))
("mobilized" nil (m ow1 b ax l ay1 z d))
("mobilizing" nil (m ow1 b ax l ay1 z ih0 ng))
("mobius" nil (m ow1 b iy0 ax s))
("mobley" nil (m ow1 b l iy0))
("mobs" nil (m aa1 b z))
("mobster" nil (m aa1 b s t er0))
("mobsters" nil (m aa1 b s t er0 z))
("mobutu" nil (m ow0 b y uw1 t uw1))
("moby" nil (m ow1 b iy0))
("moca" nil (m ow1 k ax))
("mocarski" nil (m ax k aa1 r s k iy0))
("mocatta" nil (m ax k aa1 t ax))
("moccasin" nil (m aa1 k ax s ax n))
("moccia" nil (m ow1 ch ax))
("moccio" nil (m ow1 ch iy0 ow0))
("moceri" nil (m ow0 ch eh1 r iy0))
("moch" nil (m aa1 k))
("mocha" nil (m ow1 k ax))
("mochel" nil (m aa1 k ax l))
("mochida" nil (m ax ch iy1 d ax))
("mochizuki" nil (m ow0 ch iy0 z uw1 k iy0))
("mock" nil (m aa1 k))
("mocked" nil (m aa1 k t))
("mockery" nil (m aa1 k er0 iy0))
("mocking" nil (m aa1 k ih0 ng))
("mockingbird" nil (m aa1 k ih0 ng b er1 d))
("mockler" nil (m aa1 k l er0))
("mocks" nil (m aa1 k s))
("moczygemba" nil (m ax ch ih0 g eh1 m b ax))
("modalities" nil (m ow0 d ae1 l ax t iy0 z))
("modarressi" nil (m ow0 d aa1 r eh1 s iy0))
("mode" nil (m ow1 d))
("moded" nil (m ow1 d ih0 d))
("model" nil (m aa1 d ax l))
("modeled" nil (m aa1 d ax l d))
("modeling" nil (m aa1 d ax l ih0 ng))
("modell" nil (m ow0 d ey1 l))
("modelo" nil (m ow0 d eh1 l ow0))
("models" nil (m aa1 d ax l z))
("modem" nil (m ow1 d ax m))
("modems" nil (m ow1 d ax m z))
("moden" nil (m ow1 d ax n))
("modena" nil (m ow0 d eh1 n ax))
("moder" nil (m ow1 d er0))
("moderate" n (m aa1 d er0 ax t))
("moderate" v (m aa1 d er0 ey0 t))
("moderated" nil (m aa1 d er0 ey1 t ih0 d))
("moderately" nil (m aa1 d er0 ax t l iy0))
("moderates" n (m aa1 d er0 ax t s))
("moderates" y (m aa1 d er0 ey0 t s))
("moderating" nil (m aa1 d er0 ey1 t ih0 ng))
("moderation" nil (m aa1 d er0 ey1 sh ax n))
("moderator" nil (m aa1 d er0 ey1 t er0))
("modern" nil (m aa1 d er0 n))
("modernism" nil (m aa1 d er0 n ih1 z ax m))
("modernist" nil (m aa1 d er0 n ax s t))
("modernists" nil (m aa1 d er0 n ax s t s))
("modernity" nil (m aa0 d er1 n ax t iy0))
("modernization" nil (m aa1 d er0 n ax z ey1 sh ax n))
("modernize" nil (m aa1 d er0 n ay1 z))
("modernized" nil (m aa1 d er0 n ay1 z d))
("modernizing" nil (m aa1 d er0 n ay1 z ih0 ng))
("moderns" nil (m aa1 d er0 n z))
("modes" nil (m ow1 d z))
("modesitt" nil (m aa1 d ih0 s ih0 t))
("modest" nil (m aa1 d ax s t))
("modesta" nil (m ow0 d eh1 s t ax))
("modestia" nil (m ow0 d eh1 s t y ax))
("modestine" nil (m ow0 d eh0 s t iy1 n iy0))
("modestly" nil (m aa1 d ax s t l iy0))
("modesto" nil (m ow0 d eh1 s t ow0))
("modesty" nil (m aa1 d ax s t iy0))
("modglin" nil (m aa1 jh l ih0 n))
("modi" nil (m ow1 d iy0))
("modic" nil (m aa1 d ih0 k))
("modica" nil (m ow0 d iy1 k ax))
("modicum" nil (m aa1 d ih0 k ax m))
("modification" nil (m aa1 d ax f ax k ey1 sh ax n))
("modifications" nil (m aa1 d ax f ax k ey1 sh ax n z))
("modified" nil (m aa1 d ax f ay1 d))
("modifies" nil (m aa1 d ax f ay1 z))
("modify" nil (m aa1 d ax f ay1))
("modifying" nil (m aa1 d ax f ay1 ih0 ng))
("modigliani" nil (m ow0 d ih1 g l iy0 aa1 n iy0))
("modine" nil (m ow0 d iy1 n))
("modisette" nil (m aa1 d ih0 s eh0 t))
("modjeski" nil (m ax jh eh1 s k iy0))
("modlin" nil (m aa1 d l ih0 n))
("modrak" nil (m aa1 d r ax k))
("modrow" nil (m aa1 d r ow0))
("modugno" nil (m ow0 d uw1 g n ow0))
("modulaire" nil (m ao1 jh uw0 l eh1 r))
("modular" nil (m aa1 jh ax l er0))
("modulate" nil (m ao1 jh y uw0 l ey0 t))
("modulation" nil (m aa1 jh ax l ey1 sh ax n))
("modulator" nil (m aa1 jh ax l ey1 t er0))
("module" nil (m aa1 jh uw0 l))
("modules" nil (m aa1 jh uw0 l z))
("modus" nil (m ow1 d ax s))
("mody" nil (m ow1 d iy0))
("modzelewski" nil (m ax jh ih0 l eh1 f s k iy0))
("moe" nil (m ow1))
("moebius" nil (m ow1 b iy0 ih0 s))
("moeckel" nil (m ow1 k ax l))
("moede" nil (m ow1 d))
("moehle" nil (m ow1 hh ax l))
("moehlman" nil (m ow1 l m ax n))
("moehring" nil (m ao1 r ih0 ng))
("moeller" nil (m ow1 l er0))
("moellering" nil (m ow1 l er0 ih0 ng))
("moen" nil (m ow1 n))
("moench" nil (m ow1 ng k))
("moening" nil (m aa1 ax n ih0 ng))
("moerbe" nil (m ao1 r b))
("moerke" nil (m ao1 r k))
("moerman" nil (m ow1 er0 m ax n))
("moers" nil (m ow1 er0 z))
("moertel" nil (m ao1 r t eh1 l))
("moes" nil (m ow1 z))
("moesch" nil (m ow1 sh))
("moeser" nil (m ow1 z er0))
("moessner" nil (m ow1 s n er0))
("moet" nil (m ow1 ih0 t))
("mof" nil (m aa1 f))
("moffa" nil (m aa1 f ax))
("moffat" nil (m aa1 f ax t))
("moffatt" nil (m aa1 f ax t))
("moffet" nil (m aa1 f ih0 t))
("moffett" nil (m aa1 f ih0 t))
("moffit" nil (m aa1 f ih0 t))
("moffitt" nil (m aa1 f ih0 t))
("mofford" nil (m ao1 f er0 d))
("mofield" nil (m ow1 f iy1 l d))
("moga" nil (m ow1 g ax))
("mogadishu" nil (m ow1 g ax d iy1 sh uw1))
("mogan" nil (m ow1 g ax n))
("mogavero" nil (m ow0 g aa0 v eh1 r ow0))
("mogel" nil (m ow1 g ax l))
("mogensen" nil (m aa1 g ih0 n s ax n))
("moger" nil (m ow1 g er0))
("mogg" nil (m aa1 g))
("mogle" nil (m ow1 g ax l))
("moglia" nil (m aa1 g l iy0 ax))
("mogopa" nil (m ow0 g ow1 p ax))
("mogul" nil (m ow1 g ax l))
("moguls" nil (m ow1 g ax l z))
("mohair" nil (m ow1 hh eh1 r))
("mohamad" nil (m ax hh aa1 m ax d))
("mohamed" nil (m ow0 hh aa1 m eh0 d))
("mohammad" nil (m ow0 hh aa1 m eh0 d))
("mohammed" nil (m ow0 hh ae1 m ih0 d))
("mohan" nil (m ow1 hh aa0 n))
("mohar" nil (m ow1 hh er0))
("mohasco" nil (m ow0 hh ae1 s k ow0))
("mohawk" nil (m ow1 hh ao1 k))
("mohawks" nil (m ow1 hh ao1 k s))
("moher" nil (m aa1 hh er0))
("mohl" nil (m ow1 l))
("mohler" nil (m ow1 l er0))
("mohlman" nil (m ow1 l m ax n))
("mohn" nil (m aa1 n))
("mohney" nil (m aa1 n iy0))
("mohnkern" nil (m aa1 n k er0 n))
("mohon" nil (m ow1 hh ax n))
("mohr" nil (m ao1 r))
("mohring" nil (m ao1 r ih0 ng))
("mohrman" nil (m ao1 r m ax n))
("mohrmann" nil (m ao1 r m ax n))
("mohs" nil (m aa1 s))
("mohtashemi" nil (m ow1 t ax sh ey1 m iy0))
("moi" nil (m w aa1))
("moilanen" nil (m oy1 l ax n ax n))
("moina" nil (m oy1 n ax))
("moines" nil (m oy1 n z))
("moir" nil (m oy1 r))
("moira" nil (m oy1 r ax))
("moisan" nil (m oy0 z ae1 n))
("moise" nil (m oy1 z))
("moises" nil (m oy1 z ih0 z))
("moisi" nil (m oy1 s iy0))
("moist" nil (m oy1 s t))
("moistened" nil (m oy1 s ax n d))
("moisture" nil (m oy1 s ch er0))
("moisturizer" nil (m oy1 s ch er0 ay1 z er0))
("mojave" nil (m ow0 hh aa1 v iy0))
("mojica" nil (m ow0 y iy1 k ax))
("mojo" nil (m ow1 jh ow0))
("mok" nil (m aa1 k))
("moka" nil (m ow1 k ax))
("mokhiber" nil (m aa1 k hh ih0 b er0))
("mokoena" nil (m ax k ow1 n ax))
("mokry" nil (m aa1 k r iy0))
("mol" nil (m ao1 l))
("mola" nil (m ow1 l ax))
("molaison" nil (m ax l ey1 z ax n))
("moland" nil (m aa1 l ax n d))
("molander" nil (m aa1 l ax n d er0))
("molano" nil (m ow0 l aa1 n ow0))
("molasses" nil (m ax l ae1 s ax z))
("molchan" nil (m ow1 l ch ax n))
("mold" nil (m ow1 l d))
("moldable" nil (m ow1 l d ax b ax l))
("moldavia" nil (m ow1 l d ey1 v iy0 ax))
("moldavian" nil (m ow1 l d ey1 v iy0 ax n))
("molded" nil (m ow1 l d ax d))
("molden" nil (m ow1 l d ax n))
("moldenhauer" nil (m ow1 l d ih0 n hh aw0 er0))
("molder" nil (m ow1 l d er0))
("molders" nil (m ow1 l d er0 z))
("molding" nil (m ow1 l d ih0 ng))
("moldings" nil (m ow1 l d ih0 ng z))
("moldova" nil (m ow1 l d ow1 v ax))
("moldovan" nil (m ow1 l d ow0 v aa1 n))
("molds" nil (m ow1 l d z))
("moldy" nil (m ow1 l d iy0))
("mole" nil (m ow1 l))
("molecular" nil (m ax l eh1 k y ax l er0))
("molecule" nil (m aa1 l ax k y uw1 l))
("molecules" nil (m aa1 l ax k y uw1 l z))
("moleculon" nil (m ow0 l eh1 k y uw0 l aa0 n))
("molehill" nil (m ow1 l hh ih1 l))
("molelike" nil (m ow1 l l ay1 k))
("molen" nil (m ow1 l ax n))
("molenaar" nil (m aa1 l ih0 n aa0 r))
("molenda" nil (m ow0 l eh1 n d ax))
("moler" nil (m ow1 l er0))
("moles" nil (m ow1 l z))
("moleski" nil (m ax l eh1 s k iy0))
("molesky" nil (m ax l eh1 s k iy0))
("molest" nil (m ax l eh1 s t))
("molestation" nil (m ow1 l eh0 s t ey1 sh ax n))
("molested" nil (m ax l eh1 s t ih0 d))
("molester" nil (m ax l eh1 s t er0))
("molesting" nil (m ax l eh1 s t ih0 ng))
("molests" nil (m ax l eh1 s t s))
("molesworth" nil (m ow1 l z w er1 th))
("molex" nil (m ow1 l ax k s))
("moliere" nil (m ow0 l y eh1 r))
("molin" nil (m ow1 l ih0 n))
("molina" nil (m ax l iy1 n ax))
("molinar" nil (m aa1 l ih0 n er0))
("molinari" nil (m ao1 l ax n aa1 r iy0))
("molinaro" nil (m ow0 l iy0 n aa1 r ow0))
("moline" nil (m ow0 l iy1 n))
("molineaux" nil (m ao1 l ax n ow1))
("molinelli" nil (m ow0 l iy0 n eh1 l iy0))
("molino" nil (m ow0 l iy1 n ow0))
("molitor" nil (m ow0 l iy1 t er0))
("moll" nil (m aa1 l))
("molle" nil (m aa1 l))
("mollenhauer" nil (m aa1 l ih0 n hh aw0 er0))
("mollenkopf" nil (m aa1 l ax n k ao0 p f))
("moller" nil (m aa1 l er0))
("mollet" nil (m aa1 l ih0 t))
("mollett" nil (m aa1 l ih0 t))
("molley" nil (m aa1 l iy0))
("mollica" nil (m aa1 l ih0 k ax))
("mollie" nil (m aa1 l iy0))
("mollified" nil (m aa1 l ax f ay1 d))
("mollify" nil (m aa1 l ax f ay1))
("mollison" nil (m aa1 l ih0 s ax n))
("mollner" nil (m aa1 l n er0))
("mollo" nil (m aa1 l ow0))
("mollohan" nil (m aa1 l ax hh ae0 n))
("molloy" nil (m aa1 l oy0))
("mollusk" nil (m aa1 l ax s k))
("mollusks" nil (m aa1 l ax s k s))
("molly" nil (m aa1 l iy0))
("mollycoddle" nil (m aa1 l iy0 k aa1 d ax l))
("molnar" nil (m ow1 l n er0))
("molner" nil (m ow1 l n er0))
("molock" nil (m aa1 l ax k))
("molokai" nil (m aa1 l ax k ay1))
("moloney" nil (m ax l ow1 n iy0))
("molony" nil (m ax l ao1 n iy0))
("molotov" nil (m ao1 l ax t aa1 f))
("molpus" nil (m ao1 l p ax s))
("molson" nil (m ow1 l s ax n))
("molstad" nil (m ow1 l s t ax d))
("molt" nil (m ow1 l t))
("molten" nil (m ow1 l t ax n))
("molter" nil (m ow1 l t er0))
("molting" nil (m ow1 l t ih0 ng))
("molton" nil (m ow1 l t ax n))
("moltz" nil (m ow1 l t s))
("moly" nil (m ow1 l iy0))
("molybdenum" nil (m ax l ih1 b d ih0 n ax m))
("molyneaux" nil (m aa1 l ih0 n ow0))
("molzahn" nil (m ow1 l z ax n))
("mom" nil (m aa1 m))
("moma" nil (m ow1 m ax))
("moman" nil (m ow1 m ax n))
("momayez" nil (m ow0 m ey1 eh0 z))
("mombasa" nil (m aa0 m b aa1 s ax))
("moment" nil (m ow1 m ax n t))
("momentarily" nil (m ow1 m ax n t eh1 r ax l iy0))
("momentary" nil (m ow1 m ax n t eh1 r iy0))
("momentous" nil (m ow0 m eh1 n t ax s))
("moments" nil (m ow1 m ax n t s))
("momentum" nil (m ow0 m eh1 n t ax m))
("momigliano" nil (m ow0 m ih1 g l iy0 aa1 n ow0))
("momma" nil (m aa1 m ax))
("mommens" nil (m aa1 m ax n z))
("mommy" nil (m aa1 m iy0))
("momokawa" nil (m ow1 m ow0 k aa1 w ax))
("moms" nil (m aa1 m z))
("mon" nil (m ow1 n))
("mona" nil (m ow1 n ax))
("monacelli" nil (m ow0 n aa0 ch eh1 l iy0))
("monachino" nil (m ow0 n aa0 k iy1 n ow0))
("monaco" nil (m aa1 n ax k ow1))
("monad" nil (m ow1 n ae0 d))
("monadnock" nil (m aa1 n ae1 d n aa0 k))
("monagene" nil (m aa1 n ax jh iy1 n))
("monaghan" nil (m aa1 n ax hh ae0 n))
("monahan" nil (m aa1 n ax hh ae0 n))
("monarch" nil (m aa1 n aa1 r k))
("monarchies" nil (m aa1 n aa0 r k iy0 z))
("monarchs" nil (m aa1 n aa1 r k s))
("monarchy" nil (m aa1 n aa0 r k iy0))
("monarrez" nil (m ow0 n aa1 r eh0 z))
("monash" nil (m aa1 n ae1 sh))
("monasteries" nil (m aa1 n ax s t eh1 r iy0 z))
("monastery" nil (m aa1 n ax s t eh1 r iy0))
("monastic" nil (m ax n ae1 s t ih0 k))
("monasticism" nil (m ax n ae1 s t ax s ih1 z ax m))
("monatomic" nil (m aa1 n ax t aa1 m ih0 k))
("monca" nil (m ow1 n k ax))
("moncada" nil (m ow0 n k aa1 d ax))
("moncayo" nil (m ow0 n k ey1 ow0))
("monceaux" nil (m ax n s ow1))
("moncrief" nil (m aa1 n k r iy0 f))
("moncur" nil (m aa1 n k er0))
("moncure" nil (m ow0 n k uh1 r iy0))
("moncus" nil (m aa1 n k ih0 s))
("monda" nil (m aa1 n d ax))
("mondadori" nil (m aa1 n d ax d ao1 r iy0))
("mondale" nil (m aa1 n d ey1 l))
("mondavi" nil (m aa0 n d aa1 v iy0))
("monday" nil (m ah1 n d iy0))
("mondays" nil (m ah1 n d iy0 z))
("monde" nil (m aa1 n d))
("mondelli" nil (m ow0 n d eh1 l iy0))
("mondello" nil (m aa1 n d eh1 l ow0))
("mondeo" nil (m aa1 n d ey1 ow0))
("mondex" nil (m aa1 n d eh0 k s))
("mondo" nil (m aa1 n d ow0))
("mondor" nil (m aa1 n d er0))
("mondragon" nil (m ow0 n d r aa0 g ao1 n))
("mondry" nil (m aa1 n d r iy0))
("monds" nil (m aa1 n d z))
("mondschein" nil (m aa1 n d sh ay1 n))
("mondy" nil (m aa1 n d iy0))
("mone" nil (m ow1 n))
("monell" nil (m aa1 n ax l))
("mones" nil (m ow1 n z))
("monessen" nil (m ow1 n ax s ax n))
("monet" nil (m ow0 n ey1))
("monetarism" nil (m aa1 n ax t er0 ih1 z ax m))
("monetarist" nil (m aa1 n ax t er0 ih0 s t))
("monetarists" nil (m aa1 n ax t er0 ih1 s t s))
("monetary" nil (m aa1 n ax t eh1 r iy0))
("monett" nil (m aa1 n ih0 t))
("monette" nil (m ax n eh1 t))
("money" nil (m ah1 n iy0))
("moneycard" nil (m ah1 n iy0 k aa1 r d))
("moneycenter" nil (m ah1 n iy0 s eh1 n t er0))
("moneyed" nil (m ah1 n iy0 d))
("moneyline" nil (m ah1 n iy0 l ay1 n))
("moneymaker" nil (m ah1 n iy0 m ey1 k er0))
("moneymakers" nil (m ah1 n iy0 m ey1 k er0 z))
("moneymaking" nil (m ah1 n iy0 m ey1 k ih0 ng))
("moneypenny" nil (m ah1 n iy0 p eh1 n iy0))
("moneywatch" nil (m ah1 n iy0 w aa1 ch))
("moneyweek" nil (m ah1 n iy0 w iy1 k))
("monfils" nil (m aa1 n f ih0 l z))
("monfort" nil (m aa1 n f er0 t))
("monforte" nil (m ow0 n f ao1 r t iy0))
("mong" nil (m ao1 ng))
("mongan" nil (m aa1 ng g ax n))
("monge" nil (m aa1 n jh))
("mongeau" nil (m ax ng g ow1))
("mongelli" nil (m ow0 ng g eh1 l iy0))
("mongeon" nil (m aa1 n jh ih0 n))
("monger" nil (m ah1 ng g er0))
("mongering" nil (m ah1 ng g er0 ih0 ng))
("mongers" nil (m ah1 ng g er0 z))
("mongiello" nil (m ow0 n jh eh1 l ow0))
("mongillo" nil (m ow0 ng g ih1 l ow0))
("mongol" nil (m aa1 ng g ax l))
("mongold" nil (m aa1 n g ow1 l d))
("mongolia" nil (m aa0 ng g ow1 l iy0 ax))
("mongolian" nil (m aa0 ng g ow1 l iy0 ax n))
("mongoloid" nil (m aa1 ng g ax l oy1 d))
("mongols" nil (m aa1 ng g ax l z))
("mongoose" nil (m aa1 ng g uw0 s))
("mongooses" nil (m aa1 ng g uw0 s ax z))
("mongosutu" nil (m aa1 ng g ow0 s uw1 t uw0))
("mongstad" nil (m ao1 ng s t ae1 d))
("monhollen" nil (m aa1 n hh ax l ax n))
("monica" nil (m aa1 n ih0 k ax))
("monical" nil (m aa1 n ih0 k ax l))
("monico" nil (m ow0 n iy1 k ow0))
("monied" nil (m ah1 n iy0 d))
("monier" nil (m ow1 n iy0 er0))
("monies" nil (m ah1 n iy0 z))
("monieson" nil (m ow1 n iy0 s ax n))
("monigold" nil (m aa1 n ih0 g ow1 l d))
("moniker" nil (m aa1 n ih0 k er0))
("monikers" nil (m aa1 n ih0 k er0 z))
("moninger" nil (m ow1 n ih0 ng er0))
("monism" nil (m aa1 n ih0 z ax m))
("monisms" nil (m aa1 n ih0 z ax m z))
("monitor" nil (m aa1 n ax t er0))
("monitored" nil (m aa1 n ax t er0 d))
("monitoring" nil (m aa1 n ax t er0 ih0 ng))
("monitors" nil (m aa1 n ax t er0 z))
("moniz" nil (m aa1 n ih0 z))
("monje" nil (m aa1 n jh))
("monk" nil (m ah1 ng k))
("monkees" nil (m aa1 ng k iy0 z))
("monkey" nil (m ah1 ng k iy0))
("monkeylike" nil (m ah1 ng k iy0 l ay1 k))
("monkeys" nil (m ah1 ng k iy0 z))
("monks" nil (m ah1 ng k s))
("monmouth" nil (m aa1 n m ax th))
("monn" nil (m aa1 n))
("monnett" nil (m aa1 n ih0 t))
("monnier" nil (m aa1 n iy0 er0))
("monnig" nil (m aa1 n ih0 g))
("monnin" nil (m aa1 n ih0 n))
("mono" nil (m ow1 n ow0))
("monocarpic" nil (m aa1 n ax k aa1 r p ih0 k))
("monochromatic" nil (m aa1 n ax k r ow0 m ae1 t ih0 k))
("monochrome" nil (m aa1 n ax k r ow1 m))
("monoclate" nil (m aa1 n ax k l ey1 t))
("monoclinic" nil (m aa1 n ax k l ih1 n ih0 k))
("monoclonal" nil (m aa1 n ax k l ow1 n ax l))
("monogamous" nil (m ax n aa1 g ax m ax s))
("monogamy" nil (m ax n aa1 g ax m iy0))
("monogram" nil (m aa1 n ax g r ae1 m))
("monogrammed" nil (m aa1 n ax g r ae1 m d))
("monograph" nil (m aa1 n ax g r ae1 f))
("monohull" nil (m aa1 n ax hh ax l))
("monolingual" nil (m aa1 n ax l ih1 ng g w ax l))
("monolith" nil (m aa1 n ax l ih1 th))
("monolithic" nil (m aa1 n ax l ih1 th ih0 k))
("monologue" nil (m aa1 n ax l ao1 g))
("monologues" nil (m aa1 n ax l ao1 g z))
("monomer" nil (m aa1 n ax m er0))
("monomers" nil (m aa1 n ax m er0 z))
("monongahela" nil (m ax n ao1 ng g ax hh ey1 l ax))
("mononuclear" nil (m aa1 n ax n uw1 k l iy0 er0))
("monophonic" nil (m aa1 n ax f aa1 n ih0 k))
("monoplanes" nil (m aa1 n ax p l ey1 n z))
("monopole" nil (m aa1 n ax p ow1 l))
("monopoles" nil (m aa1 n ax p ow1 l z))
("monopolies" nil (m ax n aa1 p ax l iy0 z))
("monopolist" nil (m ax n aa1 p ax l ax s t))
("monopolistic" nil (m ax n aa1 p ax l ih1 s t ih0 k))
("monopolization" nil (m ax n aa1 p ax l ih0 z ey1 sh ax n))
("monopolize" nil (m ax n aa1 p ax l ay1 z))
("monopolized" nil (m ax n aa1 p ax l ay1 z d))
("monopolizes" nil (m ax n aa1 p ax l ay1 z ih0 z))
("monopolizing" nil (m ax n aa1 p ax l ay1 z ih0 ng))
("monopoly" nil (m ax n aa1 p ax l iy0))
("monorail" nil (m aa1 n er0 ey1 l))
("monorails" nil (m aa1 n er0 ey1 l z))
("monosaccharide" nil (m aa1 n ax s ae1 k er0 ay1 d))
("monoszon" nil (m aa1 n ax z aa1 n))
("monotheism" nil (m aa1 n ax th iy0 ih0 z ax m))
("monotone" nil (m aa1 n ax t ow1 n))
("monotonous" nil (m ax n aa1 t ax n ax s))
("monotony" nil (m ax n aa1 t ax n iy0))
("monotype" nil (m aa1 n ax t ay1 p))
("monovalent" nil (m aa1 n ax v ey1 l ax n t))
("monoxide" nil (m ax n aa1 k s ay0 d))
("monreal" nil (m aa1 n r ax l))
("monro" nil (m aa1 n r ow0))
("monroe" nil (m ax n r ow1))
("monroeville" nil (m aa0 n r ow1 v ih1 l))
("monrovia" nil (m aa1 n r ow1 v iy0 ax))
("monroy" nil (m aa1 n r oy1))
("monsanto" nil (m aa1 n s ae1 n t ow0))
("monsees" nil (m aa1 n s iy1 z))
("monsen" nil (m aa1 n s ax n))
("monsey" nil (m aa1 n z iy0))
("monsieur" nil (m ax s y er1))
("monsieurs" nil (m ax s y er1 z))
("monsignor" nil (m aa0 n s iy1 n y er0))
("monsignors" nil (m aa0 n s iy1 n y er0 z))
("monsky" nil (m aa1 n s k iy0))
("monsod" nil (m aa1 n s aa0 d))
("monson" nil (m aa1 n s ax n))
("monsoon" nil (m aa0 n s uw1 n))
("monsoons" nil (m aa0 n s uw1 n z))
("monsour" nil (m aa1 n s er0))
("monster" nil (m aa1 n s t er0))
("monsters" nil (m aa1 n s t er0 z))
("monstrosity" nil (m aa0 n s t r aa1 s ax t iy0))
("monstrous" nil (m aa1 n s t r ax s))
("mont" nil (m aa1 n t))
("montag" nil (m ax n t ae1 g))
("montage" nil (m aa1 n t ih0 jh))
("montagna" nil (m ow0 n t aa1 g n ax))
("montagne" nil (m ax n t ey1 n y))
("montagnier" nil (m aa0 n t ae1 n y er0))
("montagnino" nil (m ow0 n t aa0 g n iy1 n ow0))
("montagu" nil (m aa1 n t ax g y uw1))
("montague" nil (m aa1 n t ax g y uw1))
("montalban" nil (m aa0 n t ae1 l b ax n))
("montalbano" nil (m ow0 n t aa0 l b aa1 n ow0))
("montalbo" nil (m aa1 n t ae1 l b ow0))
("montalto" nil (m ow0 n t aa1 l t ow0))
("montalvo" nil (m ow0 n t aa1 l v ow0))
("montana" nil (m aa0 n t ae1 n ax))
("montanari" nil (m ow0 n t aa0 n aa1 r iy0))
("montanaro" nil (m ow0 n t aa0 n aa1 r ow0))
("montand" nil (m aa1 n t ax n d))
("montanez" nil (m ow0 n t aa1 n eh0 z))
("montante" nil (m ow0 n t aa1 n t iy0))
("montanye" nil (m ow0 n t aa1 n y ey0))
("montavon" nil (m ow0 n t aa0 v ao1 n))
("montazeri" nil (m aa1 n t ax z eh1 r iy0))
("montblanc" nil (m aa0 n t b l ae1 ng k))
("montclair" nil (m aa1 n t k l eh1 r))
("monte" nil (m aa1 n t iy0))
("monteagudo" nil (m ow0 n t ax g uw1 d ow0))
("montebello" nil (m aa1 n t ih0 b eh1 l ow0))
("montecalvo" nil (m ow0 n t eh0 k aa1 l v ow0))
("montedison" nil (m aa0 n t eh1 d ih0 s ax n))
("montee" nil (m aa1 n t iy0))
("montefiore" nil (m aa1 n t ax f iy0 ao1 r iy0))
("monteforte" nil (m ow0 n t eh0 f ao1 r t iy0))
("montefusco" nil (m ow0 n t eh0 f uw1 s k ow0))
("monteil" nil (m aa0 n t ay1 l))
("monteiro" nil (m aa0 n t eh1 r ow0))
("montejano" nil (m aa0 n t ey0 aa1 n ow0))
("montel" nil (m aa0 n t eh1 l))
("monteleone" nil (m aa0 n t ey0 l ey0 ow1 n iy0))
("montella" nil (m aa1 n t eh1 l ax))
("montello" nil (m aa1 n t eh1 l ow0))
("montelongo" nil (m aa0 n t eh0 l ow1 ng g ow0))
("montemarano" nil (m aa0 n t eh0 m aa0 r aa1 n ow0))
("montemayor" nil (m aa0 n t ey0 m ey0 ao1 r))
("montemurro" nil (m aa0 n t eh0 m uh1 r ow0))
("montenay" nil (m aa1 n t ax n ey1))
("montenegro" nil (m aa1 n t ax n ey1 g r ow0))
("monter" nil (m aa1 n t er0))
("monterey" nil (m aa1 n t er0 ey1))
("montero" nil (m aa0 n t eh1 r ow0))
("monterosso" nil (m ow0 n t er0 ow1 s ow0))
("monterrey" nil (m aa1 n t er0 ey1))
("montes" nil (m aa1 n t iy0 z))
("montesano" nil (m ow0 n t eh0 s aa1 n ow0))
("montesdeoca" nil (m ow0 n t eh0 s d iy0 aa1 k ax))
("montesi" nil (m ow0 n t eh1 s iy0))
("montesino" nil (m ow0 n t eh0 s iy1 n ow0))
("monteverde" nil (m ow0 n t eh0 v eh1 r d iy0))
("montevideo" nil (m aa1 n t ax v ih1 d iy0 ow1))
("montez" nil (m ow0 n t eh1 z))
("montezuma" nil (m aa1 n t ih0 z uw1 m ax))
("montford" nil (m ax n t f ao1 r d))
("montfort" nil (m aa1 n t f er0 t))
("montgomery" nil (m aa0 n t g ah1 m r iy0))
("montgoris" nil (m aa0 n t g ao1 r ih0 s))
("month" nil (m ah1 n th))
("monthlong" nil (m ah1 n th l ao1 ng))
("monthly" nil (m ah1 n th l iy0))
("months" nil (m ah1 n th s))
("monti" nil (m aa1 n t iy0))
("monticciolo" nil (m aa0 n t iy1 ch iy0 ow1 l ow0))
("monticello" nil (m aa1 n t ax s eh1 l ow0))
("montie" nil (m aa1 n t iy0))
("montiel" nil (m ow0 n t iy0 eh1 l))
("montien" nil (m aa1 n t iy1 eh0 n))
("montieth" nil (m aa1 n t iy0 ih0 th))
("montijo" nil (m ow0 n t iy1 y ow0))
("montini" nil (m ow0 n t iy1 n iy0))
("montjoy" nil (m aa1 n t jh oy1))
("montminy" nil (m aa1 n t m ih0 n iy0))
("montone" nil (m ow0 n t ow1 n iy0))
("montooth" nil (m aa1 n t uw1 th))
("montoya" nil (m aa0 n t oy1 ax))
("montpelier" nil (m aa0 n t p iy1 l iy0 er0))
("montreal" nil (m ah1 n t r iy0 ao1 l))
("montrone" nil (m aa0 n t r ow1 n))
("montrose" nil (m aa1 n t r ow1 z))
("montross" nil (m ow1 n t r ow0 s))
("montroy" nil (m aa1 n t r oy1))
("monts" nil (m aa1 n t s))
("montuori" nil (m ow0 n t w ao1 r iy0))
("montvale" nil (m aa1 n t v ey1 l))
("montville" nil (m ow1 n t v ih0 l))
("monty" nil (m ao1 n t iy0))
("montz" nil (m aa1 n t s))
("monument" nil (m aa1 n y ax m ax n t))
("monumental" nil (m aa1 n y ax m eh1 n t ax l))
("monuments" nil (m aa1 n y ax m ax n t s))
("monus" nil (m ow1 n ax s))
("mony" nil (m ow1 n iy0))
("monzert" nil (m aa1 n z er0 t))
("monzingo" nil (m ow0 n z iy1 ng g ow0))
("monzo" nil (m aa1 n z ow0))
("monzon" nil (m ow0 n z ao1 n))
("monzonite" nil (m aa1 n z ax n ay1 t))
("mooberry" nil (m uw1 b eh1 r iy0))
("mood" nil (m uw1 d))
("moodie" nil (m uw1 d iy0))
("moods" nil (m uw1 d z))
("moody" nil (m uw1 d iy0))
("mooers" nil (m uw1 er0 z))
("moog" nil (m uw1 g))
("mook" nil (m uh1 k))
("moomaw" nil (m uw1 m ao1))
("moomey" nil (m uw1 m iy0))
("moon" nil (m uw1 n))
("moondreamers" nil (m uw1 n d r iy1 m er0 z))
("moone" nil (m uw1 n))
("mooney" nil (m uw1 n iy0))
("mooneyham" nil (m uw1 n iy0 hh ax m))
("mooneyhan" nil (m uw1 n iy0 hh ax n))
("moonlight" nil (m uw1 n l ay1 t))
("moonlighting" nil (m uw1 n l ay1 t ih0 ng))
("moonlike" nil (m uw1 n l ay1 k))
("moonlit" nil (m uw1 n l ih1 t))
("moons" nil (m uw1 n z))
("moonshine" nil (m uw1 n sh ay1 n))
("moonstone" nil (m uw1 n s t ow1 n))
("moonstruck" nil (m uw1 n s t r ah1 k))
("moonves" nil (m uw1 n v eh0 z))
("moor" nil (m uh1 r))
("mooradian" nil (m uh0 r ey1 d iy0 ax n))
("moorco" nil (m uh1 r k ow0))
("moore" nil (m uh1 r))
("moored" nil (m uh1 r d))
("moorefield" nil (m uh1 r ih0 f iy0 l d))
("moorehead" nil (m uh1 r hh eh1 d))
("moorehouse" nil (m uh1 r hh aw1 s))
("moorer" nil (m uh1 r er0))
("moores" nil (m uh1 r z))
("moorestown" nil (m ao1 r z t aw1 n))
("moorhead" nil (m uh1 r hh eh1 d))
("moorhouse" nil (m uh1 r hh aw1 s))
("mooring" nil (m uh1 r ih0 ng))
("moorings" nil (m uw1 r ih0 ng z))
("moorish" nil (m uh1 r ih0 sh))
("moorland" nil (m uh1 r l ae1 n d))
("moorlands" nil (m uh1 r l ae1 n d z))
("moorman" nil (m uh1 r m ax n))
("moors" nil (m uh1 r z))
("moos" nil (m uw1 z))
("moosa" nil (m uw1 s ax))
("moose" nil (m uw1 s))
("moosehead" nil (m uw1 s hh eh1 d))
("moosman" nil (m uw1 s m ax n))
("moot" nil (m uw1 t))
("moothart" nil (m uw1 th aa0 r t))
("moots" nil (m uw1 t s))
("mooty" nil (m uw1 t iy0))
("mootz" nil (m uw1 t s))
("mop" nil (m aa1 p))
("mopboard" nil (m aa1 p b ao1 r d))
("moping" nil (m ow1 p ih0 ng))
("mopped" nil (m aa1 p t))
("moppes" nil (m aa1 p s))
("mopping" nil (m aa1 p ih0 ng))
("mops" nil (m aa1 p s))
("moquin" nil (m ow0 k w iy1 n))
("mor" nil (m ao1 r))
("mora" nil (m ao1 r ax))
("morabito" nil (m ao0 r aa0 b iy1 t ow0))
("morace" nil (m ao0 r aa1 ch iy0))
("morad" nil (m ao1 r ax d))
("morado" nil (m ao0 r aa1 d ow0))
("moraga" nil (m ao0 r aa1 g ax))
("moragne" nil (m er0 ey1 n y))
("morahan" nil (m ao1 r ax hh ae0 n))
("morain" nil (m er0 ey1 n))
("morainal" nil (m er0 ey1 n ax l))
("moraine" nil (m er0 ey1 n))
("morais" nil (m er0 ey1))
("moraitis" nil (m ao0 r ay1 t ih0 s))
("moral" nil (m ao1 r ax l))
("morale" nil (m er0 ae1 l))
("morales" nil (m er0 ae1 l z))
("moralez" nil (m ao0 r aa1 l eh0 z))
("moralism" nil (m ao1 r ax l ih1 z ax m))
("moralist" nil (m ao1 r ax l ih0 s t))
("moralistic" nil (m ao1 r ax l ih1 s t ih0 k))
("moralists" nil (m ao1 r ax l ih0 s t s))
("morality" nil (m er0 ae1 l ax t iy0))
("moralizing" nil (m ao1 r ax l ay1 z ih0 ng))
("morally" nil (m ao1 r ax l iy0))
("morals" nil (m ao1 r ax l z))
("moran" nil (m er0 ae1 n))
("morand" nil (m ao1 r ax n d))
("morandi" nil (m ao0 r aa1 n d iy0))
("morando" nil (m ao0 r aa1 n d ow0))
("morang" nil (m ao1 r ax ng))
("morano" nil (m ao0 r aa1 n ow0))
("morant" nil (m ao1 r ax n t))
("morante" nil (m ao0 r aa1 n t iy0))
("morash" nil (m ao1 r ax sh))
("moraski" nil (m er0 aa1 s k iy0))
("morass" nil (m er0 ae1 s))
("morasses" nil (m er0 ae1 s ih0 z))
("morath" nil (m ao1 r ax th))
("moratoria" nil (m ao1 r ax t ao1 r iy0 ax))
("moratorium" nil (m ao1 r ax t ao1 r iy0 ax m))
("moratoriums" nil (m ao1 r ax t ao1 r iy0 ax m z))
("moravec" nil (m er0 aa1 v ih0 k))
("moravek" nil (m ao1 r ax v ih0 k))
("moravian" nil (m er0 ey1 v iy0 ax n))
("morawski" nil (m er0 aa1 f s k iy0))
("moray" nil (m er0 ey1))
("morays" nil (m er0 ey1 z))
("morbid" nil (m ao1 r b ax d))
("morbidity" nil (m ao0 r b ih1 d ax t iy0))
("morby" nil (m ao1 r b iy0))
("morcom" nil (m ao1 r k ax m))
("mord" nil (m ao1 r d))
("mordant" nil (m ao1 r d ax n t))
("mordechai" nil (m ao1 r d ax k ay1))
("morden" nil (m ao1 r d ax n))
("more" nil (m ao1 r))
("morea" nil (m ao1 r iy0 ax))
("moreau" nil (m er0 ow1))
("moredock" nil (m ao1 r ih0 d aa0 k))
("moree" nil (m er0 iy1))
("moreen" nil (m ao0 r iy1 n))
("morefield" nil (m ao1 r ih0 f iy1 l d))
("morehart" nil (m ao1 r ih0 hh aa0 r t))
("morehead" nil (m ao1 r hh eh0 d))
("morehouse" nil (m ao1 r hh aw1 s))
("moreira" nil (m ao0 r eh1 r ax))
("morejon" nil (m ao1 r ih0 jh aa0 n))
("morel" nil (m er0 eh1 l))
("moreland" nil (m ao1 r l ax n d))
("morell" nil (m ao1 r ax l))
("morella" nil (m ao0 r eh1 l ax))
("morelli" nil (m ao0 r eh1 l iy0))
("morello" nil (m er0 eh1 l ow0))
("morelock" nil (m ao1 r ih0 l aa0 k))
("moreman" nil (m ao1 r m ax n))
("moren" nil (m ao1 r ax n))
("morena" nil (m ao0 r ey1 n ax))
("morency" nil (m ao0 r ao1 n s iy0))
("moreno" nil (m ao0 r iy1 n ow0))
("moreover" nil (m ao0 r ow1 v er0))
("morera" nil (m ao0 r eh1 r ax))
("mores" nil (m ao1 r ey1 z))
("moresco" nil (m ao0 r eh1 s k ow0))
("moret" nil (m ao1 r ih0 t))
("moreton" nil (m ao1 r ih0 t aa0 n))
("moretti" nil (m ao0 r eh1 t iy0))
("moretto" nil (m ao0 r eh1 t ow0))
("moretz" nil (m ao1 r ih0 t s))
("morever" nil (m ao1 r v er0))
("morey" nil (m ao1 r iy0))
("morfin" nil (m ao1 r f ih0 n))
("morford" nil (m ao1 r f er0 d))
("morga" nil (m ao1 r g ax))
("morgado" nil (m ao0 r g aa1 d ow0))
("morgan" nil (m ao1 r g ax n))
("morgana" nil (m ao0 r g ae1 n ax))
("morgans" nil (m ao1 r g ax n z))
("morganstern" nil (m ao1 r g ax n s t er0 n))
("morgante" nil (m ao0 r g aa1 n t iy0))
("morganti" nil (m ao0 r g aa1 n t iy0))
("morgantown" nil (m ao1 r g ax n t aw1 n))
("morgart" nil (m ao1 r g aa0 r t))
("morgen" nil (m ao1 r g ax n))
("morgenroth" nil (m ao1 r g ih0 n r ao0 th))
("morgenstern" nil (m ao1 r g ih0 n s t er0 n))
("morgenthaler" nil (m ao1 r g ih0 n th ax l er0))
("morgenthau" nil (m ao1 r g ax n th aw1))
("morgue" nil (m ao1 r g))
("morgun" nil (m ao1 r g ax n))
("mori" nil (m ao1 r iy0))
("moria" nil (m ao1 r iy0 ax))
("moriarity" nil (m ao1 r iy0 aa1 r ax t iy0))
("moriarty" nil (m ao1 r iy0 aa1 r t iy0))
("moribund" nil (m ao1 r ax b ax n d))
("moric" nil (m ao1 r ih0 k))
("morice" nil (m ao1 r ih0 s))
("morici" nil (m ao0 r iy1 ch iy0))
("morihiro" nil (m ao0 r iy0 hh ih1 r ow0))
("morikawa" nil (m ao0 r iy0 k aa1 w ax))
("morillo" nil (m ao0 r ih1 l ow0))
("morimoto" nil (m ao0 r iy0 m ow1 t ow0))
("morin" nil (m ao1 r ih0 n))
("morine" nil (m ao0 r iy1 n iy0))
("moring" nil (m ao1 r ih0 ng))
("morini" nil (m ao0 r iy1 n iy0))
("morino" nil (m ao0 r iy1 n ow0))
("morioka" nil (m ao0 r iy0 ow1 k ax))
("moris" nil (m ao1 r ih0 s))
("morisette" nil (m ao1 r ih0 s eh0 t))
("morishita" nil (m ao0 r iy0 sh iy1 t ax))
("morison" nil (m ao1 r ih0 s ax n))
("morissette" nil (m ao1 r ih0 s eh0 t))
("morita" nil (m ao0 r iy1 t ax))
("moritz" nil (m ao0 r ih1 t s))
("moriya" nil (m ao0 r iy1 ax))
("mork" nil (m ao1 r k))
("morken" nil (m ao1 r k ax n))
("morlan" nil (m ao1 r l ax n))
("morland" nil (m ao1 r l ax n d))
("morley" nil (m ao1 r l iy0))
("morlock" nil (m ao1 r l ax k))
("morman" nil (m ao1 r m ax n))
("mormile" nil (m ao1 r m ay0 l))
("mormino" nil (m ao0 r m iy1 n ow0))
("mormon" nil (m ao1 r m ax n))
("mormons" nil (m ao1 r m ax n z))
("morna" nil (m ao1 r n ax))
("morneau" nil (m er0 n ow1))
("morneault" nil (m er0 n ow1))
("morning" nil (m ao1 r n ih0 ng))
("mornings" nil (m ao1 r n ih0 ng z))
("morningstar" nil (m ao1 r n ih0 ng s t aa1 r))
("moro" nil (m ao1 r ow1))
("moroccan" nil (m er0 aa1 k ax n))
("moroccans" nil (m er0 aa1 k ax n z))
("morocco" nil (m er0 aa1 k ow0))
("moron" nil (m ao1 r aa1 n))
("morones" nil (m ao0 r ow1 n eh0 s))
("moroney" nil (m er0 ow1 n iy0))
("moroni" nil (m er0 ow1 n iy0))
("moros" nil (m ao1 r ow0 z))
("morose" nil (m er0 ow1 s))
("morosky" nil (m ao0 r ao1 s k iy0))
("morovcic" nil (m ao0 r aa1 v ch ih0 k))
("moroz" nil (m ao1 r ow0 z))
("morphew" nil (m ao1 r f y uw0))
("morphin" nil (m ao1 r f ax n))
("morphine" nil (m ao1 r f iy0 n))
("morphis" nil (m ao1 r f ih0 s))
("morphogenesis" nil (m ao1 r f ax jh eh1 n ax s ax s))
("morphological" nil (m ao1 r f ax l aa1 jh ih0 k ax l))
("morphology" nil (m ao0 r f aa1 l ax jh iy0))
("morphs" nil (m ao1 r f s))
("morr" nil (m ao1 r))
("morra" nil (m ao1 r ax))
("morrall" nil (m ao0 r aa1 l))
("morreale" nil (m ao0 r iy1 l iy0))
("morrell" nil (m ao0 r eh1 l))
("morren" nil (m ao1 r ax n))
("morrical" nil (m ao1 r ih0 k ax l))
("morrie" nil (m ao1 r iy0))
("morrill" nil (m ao0 r iy1 l))
("morrin" nil (m ao1 r ih0 n))
("morris" nil (m ao1 r ax s))
("morrisett" nil (m ao1 r ax s eh1 t))
("morrisette" nil (m ao1 r ih0 s eh0 t))
("morrisey" nil (m ao1 r ih0 s iy0))
("morrish" nil (m ao1 r ih0 sh))
("morrison" nil (m ao1 r ih0 s ax n))
("morrissette" nil (m ao1 r ih0 s eh0 t))
("morrissey" nil (m ao1 r ih0 s iy0))
("morristown" nil (m ao1 r ax s t aw1 n))
("morrisville" nil (m ao1 r ax s v ih1 l))
("morro" nil (m ao1 r ow0))
("morrone" nil (m ao0 r ow1 n iy0))
("morrow" nil (m aa1 r ow0))
("morsch" nil (m ao1 r sh))
("morse" nil (m ao1 r s))
("morsel" nil (m ao1 r s ax l))
("morsels" nil (m ao1 r s ax l z))
("morson" nil (m ao1 r s ax n))
("morss" nil (m ao1 r s))
("mort" nil (m ao1 r t))
("mortage" nil (m ao1 r t ih0 jh))
("mortages" nil (m ao1 r t ih0 jh ih0 z))
("mortal" nil (m ao1 r t ax l))
("mortality" nil (m ao0 r t ae1 l ax t iy0))
("mortally" nil (m ao1 r t ax l iy0))
("mortals" nil (m ao1 r t ax l z))
("mortar" nil (m ao1 r t er0))
("mortara" nil (m ao0 r t aa1 r ax))
("mortars" nil (m ao1 r t er0 z))
("mortell" nil (m ao0 r t ey1 l))
("mortellaro" nil (m ao0 r t eh0 l aa1 r ow0))
("mortem" nil (m ao1 r t ax m))
("morten" nil (m ao1 r t ax n))
("mortensen" nil (m ao1 r t ih0 n s ax n))
("mortenson" nil (m ao1 r t ih0 n s ax n))
("morter" nil (m ao1 r t er0))
("mortgage" nil (m ao1 r g ax jh))
("mortgaged" nil (m ao1 r g ih0 jh d))
("mortgagepower" nil (m ao1 r g ih0 jh p aw1 r))
("mortgages" nil (m ao1 r g ax jh ax z))
("mortgaging" nil (m ao1 r g ih0 jh ih0 ng))
("mortician" nil (m ao0 r t ih1 sh ax n))
("mortie" nil (m ao1 r t iy0))
("mortier" nil (m ao1 r t iy0 er0))
("mortified" nil (m ao1 r t ax f ay1 d))
("mortimer" nil (m ao1 r t ih0 m er0))
("mortimore" nil (m ao0 r t iy1 m ao0 r))
("mortis" nil (m ao1 r t ih0 s))
("mortland" nil (m ao1 r t l ax n d))
("mortner" nil (m ao1 r t n er0))
("morton" nil (m ao1 r t ax n))
("mortuary" nil (m ao1 r ch uw0 eh1 r iy0))
("moruhiro" nil (m ao1 r uw0 hh ih1 r ow0))
("morvant" nil (m ao1 r v ax n t))
("morven" nil (m ao1 r v ax n))
("morvillo" nil (m ao0 r v ih1 l ow0))
("mory" nil (m ao1 r iy0))
("morze" nil (m ao1 r z))
("mosaic" nil (m ow0 z ey1 ih0 k))
("mosaical" nil (m ow0 z ey1 ih0 k ax l))
("mosaics" nil (m ow0 z ey1 ih0 k s))
("mosbacher" nil (m aa1 s b aa0 k er0))
("mosbrucker" nil (m aa1 s b r ax k er0))
("mosca" nil (m ow1 s k ax))
("moscato" nil (m ow0 s k aa1 t ow0))
("moschella" nil (m ow0 s k eh1 l ax))
("moschetti" nil (m ow0 s k eh1 t iy0))
("mosco" nil (m ow1 s k ow0))
("moscom" nil (m ao1 s k ax m))
("moscoso" nil (m ow0 s k ow1 s ow0))
("moscow" nil (m aa1 s k ow1))
("mose" nil (m ow1 z))
("mosel" nil (m ow1 s ax l))
("moseley" nil (m ow1 z l iy0))
("moselle" nil (m ow0 z eh1 l))
("mosely" nil (m ow1 z l iy0))
("moseman" nil (m ow1 s m ax n))
("moser" nil (m ow1 z er0))
("moses" nil (m ow1 z ax s))
("mosey" nil (m ow1 z iy0))
("moshe" nil (m ow0 sh eh1))
("mosher" nil (m ow1 zh er0))
("moshier" nil (m aa1 sh iy0 er0))
("moshood" nil (m aa1 s hh uh1 d))
("mosie" nil (m aa1 s iy0))
("mosier" nil (m ow1 zh er0))
("mosiman" nil (m ow1 s iy0 m ax n))
("moskal" nil (m aa1 s k ax l))
("mosko" nil (m ow1 s k ow0))
("moskolenko" nil (m ao1 s k ow0 l eh1 ng k ow0))
("moskovitz" nil (m aa1 s k ax v ih0 t s))
("moskow" nil (m aa1 s k ow0))
("moskowitz" nil (m ao1 s k ax w ih0 t s))
("moskwa" nil (m aa1 s k v ax))
("mosle" nil (m ow1 s ax l))
("moslem" nil (m aa1 z l ax m))
("moslems" nil (m aa1 z l ax m z))
("mosler" nil (m aa1 s ax l er0))
("mosley" nil (m ow1 z l iy0))
("mosman" nil (m aa1 s m ax n))
("mosque" nil (m aa1 s k))
("mosqueda" nil (m ow0 s k w ey1 d ax))
("mosquera" nil (m ow0 s k w eh1 r ax))
("mosques" nil (m aa1 s k s))
("mosquito" nil (m ax s k iy1 t ow0))
("mosquitoes" nil (m ax s k iy1 t ow0 z))
("mosquitos" nil (m ax s k iy1 t ow0 z))
("moss" nil (m ao1 s))
("mossad" nil (m ow0 s ae1 d))
("mossbarger" nil (m aa1 s b aa0 r g er0))
("mossberg" nil (m ao1 s b er0 g))
("mossburg" nil (m ao1 s b er0 g))
("mosser" nil (m ao1 s er0))
("mosses" nil (m ao1 s ax z))
("mossey" nil (m aa1 s iy0))
("mossholder" nil (m ao1 s hh ow1 l d er0))
("mosslike" nil (m ao1 s l ay1 k))
("mossman" nil (m ao1 s m ax n))
("mosso" nil (m ow1 s ow0))
("most" nil (m ow1 s t))
("mostar" nil (m ow1 s t aa0 r))
("mostek" nil (m aa1 s t ih0 k))
("mosteller" nil (m aa1 s t ax l er0))
("moster" nil (m ow1 s t er0))
("mostly" nil (m ow1 s t l iy0))
("mostoller" nil (m aa1 s t ow0 l er0))
("mostow" nil (m aa1 s t aw0))
("mostrom" nil (m aa1 s t r ax m))
("mostyn" nil (m aa1 s t ih0 n))
("mosur" nil (m ow1 s uh1 r))
("moszkowski" nil (m aa1 s k aw1 s k iy0))
("mota" nil (m ow1 t ax))
("mote" nil (m ow1 t))
("motel" nil (m ow0 t eh1 l))
("motels" nil (m ow0 t eh1 l z))
("moten" nil (m ow1 t ax n))
("motes" nil (m ow1 t s))
("moteurs" nil (m ow0 t uw1 r z))
("moth" nil (m ao1 th))
("mothball" nil (m ao1 th b ao1 l))
("mothballed" nil (m ao1 th b ao1 l d))
("mothballing" nil (m ao1 th b ao1 l ih0 ng))
("mothballs" nil (m ao1 th b ao1 l z))
("mother" nil (m ah1 dh er0))
("mothercare" nil (m ah1 dh er0 k eh1 r))
("motherhood" nil (m ah1 dh er0 hh uh1 d))
("mothering" nil (m ah1 dh er0 ih0 ng))
("motherland" nil (m ah1 dh er0 l ae1 n d))
("motherly" nil (m ah1 dh er0 l iy0))
("mothers" nil (m ah1 dh er0 z))
("mothershead" nil (m ah1 dh er0 z hh eh1 d))
("mothershed" nil (m ah1 dh er0 sh eh1 d))
("mothon" nil (m aa1 th ax n))
("moths" nil (m ao1 th s))
("motif" nil (m ow0 t iy1 f))
("motifs" nil (m ow0 t iy1 f s))
("motile" nil (m ow1 t ax l))
("motility" nil (m ow0 t ih1 l ih0 t iy0))
("motion" nil (m ow1 sh ax n))
("motioning" nil (m ow1 sh ax n ih0 ng))
("motionless" nil (m ow1 sh ax n l ax s))
("motions" nil (m ow1 sh ax n z))
("motivate" nil (m ow1 t ax v ey1 t))
("motivated" nil (m ow1 t ax v ey1 t ax d))
("motivates" nil (m ow1 t ax v ey1 t s))
("motivating" nil (m ow1 t ax v ey1 t ih0 ng))
("motivation" nil (m ow1 t ax v ey1 sh ax n))
("motivational" nil (m ow1 t ax v ey1 sh ax n ax l))
("motivations" nil (m ow1 t ax v ey1 sh ax n z))
("motivator" nil (m ow1 t ax v ey0 t er0))
("motive" nil (m ow1 t ih0 v))
("motives" nil (m ow1 t ih0 v z))
("motl" nil (m aa1 t ax l))
("motley" nil (m aa1 t l iy0))
("moto" nil (m ow1 t ow0))
("motohashi" nil (m ow1 t ow0 hh aa1 sh iy0))
("motola" nil (m ow0 t ow1 l ax))
("moton" nil (m ow1 t ow0 n))
("motonori" nil (m ow1 t ow0 n ao1 r iy0))
("motor" nil (m ow1 t er0))
("motorbike" nil (m ow1 t er0 b ay1 k))
("motorbikes" nil (m ow1 t er0 b ay1 k s))
("motorboat" nil (m ow1 t er0 b ow1 t))
("motorcade" nil (m ow1 t er0 k ey1 d))
("motorcars" nil (m ow1 t er0 k aa1 r z))
("motorcycle" nil (m ow1 t er0 s ay1 k ax l))
("motorcycles" nil (m ow1 t er0 s ay1 k ax l z))
("motorcyclist" nil (m ow1 t er0 s ay1 k ax l ih0 s t))
("motorcyclists" nil (m ow1 t er0 s ay1 k ax l ih0 s t s))
("motoren" nil (m ow1 t er0 ax n))
("motoring" nil (m ow1 t er0 ih0 ng))
("motorist" nil (m ow1 t er0 ih0 s t))
("motorists" nil (m ow1 t er0 ax s t s))
("motorize" nil (m ow1 t er0 ay1 z))
("motorized" nil (m ow1 t er0 ay1 z d))
("motorola" nil (m ow1 t er0 ow1 l ax))
("motors" nil (m ow1 t er0 z))
("motown" nil (m ow1 t aw1 n))
("motrin" nil (m aa1 t r ih0 n))
("mots" nil (m aa1 t s))
("motsinger" nil (m aa1 t s ih0 n jh er0))
("motsoaledi" nil (m aa0 t s ow0 l ey1 d iy0))
("mott" nil (m aa1 t))
("motte" nil (m aa1 t))
("motter" nil (m aa1 t er0))
("mottern" nil (m aa1 t er0 n))
("mottl" nil (m aa1 t ax l))
("mottled" nil (m aa1 t ax l d))
("mottley" nil (m aa1 t l iy0))
("mottling" nil (m aa1 t ax l ih0 ng))
("motto" nil (m aa1 t ow0))
("mottola" nil (m ow0 t ow1 l ax))
("mottram" nil (m aa1 t r ax m))
("motts" nil (m aa1 t s))
("mottus" nil (m aa1 t ax s))
("motyka" nil (m aa1 t ay0 k ax))
("motyl" nil (m ow1 t ax l))
("motz" nil (m aa1 t s))
("motzer" nil (m ow1 t z er0))
("motzko" nil (m aa1 t s k ow0))
("moua" nil (m aw1 ax))
("moudry" nil (m ow1 d r iy0))
("moudy" nil (m aw1 d iy0))
("moueix" nil (m uw1 w ay1 k s))
("moul" nil (m aw1 l))
("mould" nil (m ow1 l d))
("moulden" nil (m ow1 l d ax n))
("moulder" nil (m ow1 l d er0))
("moulds" nil (m ow1 l d z))
("mouldy" nil (m ow1 l d iy0))
("moulin" nil (m uw0 l ae1 n))
("moulinex" nil (m uw1 l ih0 n eh0 k s))
("moulthrop" nil (m aw1 l th r ax p))
("moulton" nil (m ow1 l t ax n))
("moultrie" nil (m ow1 l t r iy0))
("mounce" nil (m aw1 n s))
("mound" nil (m aw1 n d))
("mounds" nil (m aw1 n d z))
("mounger" nil (m aw1 ng er0))
("mounsey" nil (m aw1 n s iy0))
("mount" nil (m aw1 n t))
("mountain" nil (m aw1 n t ax n))
("mountaineer" nil (m aw1 n t ih0 n ih1 r))
("mountainous" nil (m aw1 n t ax n ax s))
("mountains" nil (m aw1 n t ax n z))
("mountainside" nil (m aw1 n t ax n s ay1 d))
("mountainsides" nil (m aw1 n t ax n s ay1 d z))
("mountaintop" nil (m aw1 n t ax n t aa1 p))
("mountcastle" nil (m aw1 n t k ae1 s ax l))
("mounted" nil (m aw1 n t ax d))
("mountford" nil (m uw0 n t f ao1 r d))
("mounties" nil (m aw1 n t iy0 z))
("mounting" nil (m aw1 n t ih0 ng))
("mountjoy" nil (m aw1 n t jh oy1))
("mountleigh" nil (m aw1 n t l iy1))
("mounts" nil (m aw1 n t s))
("mountz" nil (m aw1 n t s))
("moura" nil (m uh1 r ax))
("mourad" nil (m uw1 r ae0 d))
("mouradian" nil (m ao0 r ey1 d iy0 ax n))
("mourer" nil (m ao1 r er0))
("mourn" nil (m ao1 r n))
("mourned" nil (m ao1 r n d))
("mourners" nil (m ao1 r n er0 z))
("mournful" nil (m ao1 r n f ax l))
("mourning" nil (m ao1 r n ih0 ng))
("mourns" nil (m ao1 r n z))
("mouse" nil (m aw1 s))
("mousehole" nil (m aw1 s hh ow1 l))
("mousel" nil (m uw0 s eh1 l))
("mouser" nil (m aw1 z er0))
("mousers" nil (m aw1 z er0 z))
("mousetrap" nil (m aw1 s t r ae1 p))
("mousley" nil (m aw1 s l iy0))
("moussa" nil (m aw1 s ax))
("mousse" nil (m uw1 s))
("mousseau" nil (m uw1 s ow1))
("moustache" nil (m ah1 s t ae1 sh))
("mouth" n (m aw1 th))
("mouth" v (m aw1 dh))
("mouthed" nil (m aw1 dh d))
("mouthful" nil (m aw1 th f uh1 l))
("mouthing" nil (m aw1 dh ih0 ng))
("mouthparts" nil (m aw1 th p aa1 r t s))
("mouthpiece" nil (m aw1 th p iy1 s))
("mouthpieces" nil (m aw1 th p iy1 s ih0 z))
("mouths" nil (m aw1 dh z))
("mouthwash" nil (m aw1 th w aa1 sh))
("mouthy" nil (m aw1 th iy0))
("moutray" nil (m uw0 t r ey1))
("moutse" nil (m uw1 t s iy0))
("mouw" nil (m aw1 w))
("mouzon" nil (m uw0 z ao1 n))
("movable" nil (m uw1 v ax b ax l))
("move" nil (m uw1 v))
("moved" nil (m uw1 v d))
("movement" nil (m uw1 v m ax n t))
("movements" nil (m uw1 v m ax n t s))
("mover" nil (m uw1 v er0))
("movers" nil (m uw1 v er0 z))
("moves" nil (m uw1 v z))
("movida" nil (m uw1 v iy1 d ax))
("movie" nil (m uw1 v iy0))
("moviegoers" nil (m uw1 v iy1 g ow1 er0 z))
("movielab" nil (m uw1 v iy0 l ae1 b))
("movies" nil (m uw1 v iy0 z))
("movietime" nil (m uw1 v iy0 t ay1 m))
("moving" nil (m uw1 v ih0 ng))
("movingly" nil (m uw1 v ih0 ng l iy0))
("mow" nil (m ow1))
("mowat" nil (m ow1 ax t))
("mowatt" nil (m ow1 ax t))
("mowbray" nil (m ow1 b r ey0))
("mowdy" nil (m ow1 d iy0))
("mowed" nil (m ow1 d))
("mowell" nil (m aa1 w eh0 l))
("mowen" nil (m ow1 ax n))
("mower" nil (m ow1 er0))
("mowers" nil (m ow1 er0 z))
("mowery" nil (m aw1 er0 iy0))
("mowing" nil (m aw1 ih0 ng))
("mowrer" nil (m ao1 r er0))
("mowrey" nil (m ao1 r iy0))
("mowry" nil (m ao1 r iy0))
("mows" nil (m aw1 z))
("moxie" nil (m aa1 k s iy0))
("moxley" nil (m aa1 k s l iy0))
("moxon" nil (m aa1 k s ax n))
("moy" nil (m oy1))
("moya" nil (m oy1 ax))
("moye" nil (m oy1))
("moyer" nil (m oy1 er0))
("moyers" nil (m oy1 er0 z))
("moyes" nil (m oy1 z))
("moylan" nil (m oy1 l ax n))
("moyle" nil (m oy1 l))
("moyna" nil (m oy1 n ax))
("moynahan" nil (m oy1 n ax hh ae0 n))
("moynihan" nil (m oy1 n ih0 hh ae0 n))
("moyse" nil (m oy1 s))
("mozambican" nil (m ow1 z ae0 m b iy0 k ax n))
("mozambicans" nil (m ow1 z ae0 m b iy0 k ax n z))
("mozambique" nil (m ow1 z ae0 m b iy1 k))
("mozart" nil (m ow1 z aa0 r t))
("mozartean" nil (m ow1 z aa1 r t iy0 ax n))
("mozee" nil (m aa1 z iy0))
("mozelle" nil (m ax z eh1 l))
("mozena" nil (m ax z iy1 n ax))
("mozer" nil (m ow1 z er0))
("mozingo" nil (m ow0 z iy1 ng g ow0))
("mozley" nil (m aa1 z l iy0))
("mpeg" nil (eh1 m p eh1 g))
("mpg" nil (eh1 m p iy1 jh iy1))
("mph" nil (eh1 m p iy1 ey1 ch))
("mr" nil (m ih1 s t er0))
("mraz" nil (m r ae1 z))
("mrazek" nil (m r aa1 z eh0 k))
("mrazik" nil (m r aa1 z ih0 k))
("mroczek" nil (m r aa1 ch eh0 k))
("mroczka" nil (m r aa1 ch k ax))
("mroczkowski" nil (m r ax ch k ao1 f s k iy0))
("mrotek" nil (m r ow1 t eh1 k))
("mroz" nil (m r aa1 z))
("mrozek" nil (m r ow1 z eh0 k))
("mrozinski" nil (m r ax z ih1 n s k iy0))
("mrs" nil (m ih1 s ih0 z))
("mruk" nil (m r ah1 k))
("ms" nil (m ih1 z))
("msgr" nil (m aa0 n s iy1 n y er0))
("mssrs" nil (m eh1 s er0 z))
("mt" nil (m aw1 n t))
("mtel" nil (eh1 m t eh1 l))
("mu" nil (m uw1))
("mubarak" nil (m uw0 b aa1 r ih0 k))
("mucci" nil (m uw1 ch iy0))
("muccio" nil (m uw1 ch iy0 ow0))
("much" nil (m ah1 ch))
("mucha" nil (m ah1 ch ax))
("muchmore" nil (m ah1 k m ao0 r))
("muchnick" nil (m ah1 k n ih0 k))
("muchow" nil (m ah1 ch ow0))
("muck" nil (m ah1 k))
("muckenfuss" nil (m ah1 k ax n f ah1 s))
("muckey" nil (m ah1 k iy0))
("mucking" nil (m ah1 k ih0 ng))
("muckle" nil (m ah1 k ax l))
("muckleroy" nil (m ah1 k l er0 oy0))
("muckraker" nil (m ah1 k r ey1 k er0))
("muckraking" nil (m ah1 k r ey1 k ih0 ng))
("mucky" nil (m ah1 k iy0))
("mucosa" nil (m y uw0 k ow1 s ax))
("mucosal" nil (m y uw0 k ow1 s ax l))
("mucous" nil (m y uw1 k ax s))
("mucus" nil (m y uw1 k ax s))
("mud" nil (m ah1 d))
("mudd" nil (m ah1 d))
("muddied" nil (m ah1 d iy0 d))
("muddier" nil (m ah1 d iy0 er0))
("muddle" nil (m ah1 d ax l))
("muddled" nil (m ah1 d ax l d))
("muddleheaded" nil (m ah1 d ax l hh eh1 d ax d))
("muddles" nil (m ah1 d ax l z))
("muddling" nil (m ah1 d ax l ih0 ng))
("muddy" nil (m ah1 d iy0))
("mudge" nil (m ah1 jh))
("mudgett" nil (m ah1 jh ih0 t))
("mudra" nil (m ah1 d r ax))
("mudrick" nil (m ah1 d r ih0 k))
("mudry" nil (m ah1 d r iy0))
("muds" nil (m ah1 d z))
("mudslinging" nil (m ah1 d s l ih1 ng ih0 ng))
("mudwagon" nil (m ah1 d w ae1 g ax n))
("muecke" nil (m uw1 k))
("muegge" nil (m uw1 g))
("muehl" nil (m y uw1 l))
("muehlbauer" nil (m y uw1 l b aw0 er0))
("muehlebach" nil (m y uw1 l b aa1 k))
("mueller" nil (m y uw1 l er0))
("muellner" nil (m y uw1 l n er0))
("muench" nil (m y uw1 ng k))
("muenchen" nil (m y uw1 n ch ax n))
("muenchow" nil (m uw1 n ch aw0))
("muenster" nil (m y uw1 n s t er0))
("muetzel" nil (m y uw1 t z ax l))
("muff" nil (m ah1 f))
("muffin" nil (m ah1 f ax n))
("muffins" nil (m ah1 f ax n z))
("muffle" nil (m ah1 f ax l))
("muffled" nil (m ah1 f ax l d))
("muffler" nil (m ah1 f l er0))
("mufflers" nil (m ah1 f l er0 z))
("muffley" nil (m ah1 f l iy0))
("muffoletto" nil (m uw0 f ow0 l eh1 t ow0))
("muffs" nil (m ah1 f s))
("muffy" nil (m ah1 f iy0))
("mug" nil (m ah1 g))
("mugabe" nil (m uw0 g aa1 b ey0))
("mugar" nil (m y uw1 g er0))
("mugavero" nil (m uw0 g aa0 v eh1 r ow0))
("mugford" nil (m ah1 g f er0 d))
("mugged" nil (m ah1 g d))
("mugger" nil (m ah1 g er0))
("muggeridge" nil (m ah1 g er0 ih1 jh))
("muggers" nil (m ah1 g er0 z))
("mugging" nil (m ah1 g ih0 ng))
("muggings" nil (m ah1 g ih0 ng z))
("muggsy" nil (m ah1 g s iy0))
("muggy" nil (m ah1 g iy0))
("mughniyeh" nil (m ah1 g n ih0 y ax))
("mugniyah" nil (m ah1 g n ih0 y ax))
("mugs" nil (m ah1 g z))
("muha" nil (m y uw1 hh ax))
("muhamed" nil (m uh0 hh aa1 m eh0 d))
("muhammad" nil (m uh0 hh aa1 m ax d))
("muhammed" nil (m uh0 hh aa1 m eh0 d))
("muharram" nil (m ax hh ae1 r ax m))
("muhl" nil (m ah1 l))
("muhlbauer" nil (m uw1 l b aw0 er0))
("muhlenkamp" nil (m uw1 l ih0 n k ae0 m p))
("muhr" nil (m uh1 r))
("muhs" nil (m ah1 s))
("mui" nil (m uw1 ih0))
("muilenburg" nil (m uw1 l ax n b er0 g))
("muir" nil (m y uh1 r))
("muirfield" nil (m y uh1 r f iy0 l d))
("muirhead" nil (m y uh1 r hh eh1 d))
("muise" nil (m uw1 z))
("mujahadeen" nil (m y uw0 jh ae1 hh ax d iy1 n))
("mujahedeen" nil (m uw1 jh ax hh eh0 d iy1 n))
("mujahideen" nil (m uw1 jh ax hh eh0 d iy1 n))
("mujica" nil (m y uw1 jh ih0 k ax))
("mujzel" nil (m ah1 jh z ax l))
("mukai" nil (m uw0 k aa1 iy0))
("mukasey" nil (m y uw1 k ih0 s iy0))
("mukherjee" nil (m ax k hh er1 jh iy0))
("mukhopadhyay" nil (m uw1 k ow0 p aa1 d y ay0))
("mukluk" nil (m ah1 k l ax k))
("mula" nil (m y uw1 l ax))
("mulanaphy" nil (m y uw1 l ax n ae1 f iy0))
("mulanax" nil (m y uw1 l ax n ae0 k s))
("mulatto" nil (m ax l aa1 t ow0))
("mulberry" nil (m ah1 l b eh1 r iy0))
("mulcahey" nil (m ah1 l k ax hh iy0))
("mulcahy" nil (m ax l k ey1 hh iy0))
("mulcare" nil (m ah1 l k er0))
("mulch" nil (m ah1 l ch))
("mulched" nil (m ah1 l ch t))
("mulches" nil (m ah1 l ch ih0 z))
("mulching" nil (m ah1 l ch ih0 ng))
("mulder" nil (m ah1 d er0))
("muldoon" nil (m ax l d uw1 n))
("muldowney" nil (m ah1 l d aw0 n iy0))
("muldrew" nil (m ah1 l d r uw0))
("muldrow" nil (m ah1 l d r aw0))
("mule" nil (m y uw1 l))
("mules" nil (m y uw1 l z))
("mulford" nil (m ah1 l f er0 d))
("mulgrew" nil (m ah1 l g r uw0))
("mulhall" nil (m ah1 l hh ax l))
("mulhearn" nil (m ah1 l hh er0 n))
("mulheren" nil (m ax l hh eh1 r ax n))
("mulherin" nil (m ah1 l hh er0 ih0 n))
("mulhern" nil (m ah1 l hh er0 n))
("mulholland" nil (m ah1 l hh aa1 l ax n d))
("mulhouse" nil (m ah1 l hh aw1 s))
("mulkern" nil (m ah1 l k er0 n))
("mulkey" nil (m ah1 l k iy0))
("mulkins" nil (m ah1 l k ih0 n z))
("mull" nil (m ah1 l))
("mullady" nil (m ah1 l ax d iy0))
("mullah" nil (m ah1 l ax))
("mullahs" nil (m ah1 l ax z))
("mullally" nil (m ah1 l ax l iy0))
("mullaly" nil (m ah1 l ax l iy0))
("mullan" nil (m ah1 l ax n))
("mullane" nil (m ah1 l ax n))
("mullaney" nil (m ah1 l ax n iy0))
("mullany" nil (m ah1 l ax n iy0))
("mullarkey" nil (m ax l aa1 r k iy0))
("mulled" nil (m ah1 l d))
("mullen" nil (m ah1 l ax n))
("mullenax" nil (m ah1 l ax n ae1 k s))
("mullendore" nil (m uw0 l eh1 n d ao0 r))
("mullenix" nil (m uw1 l ih0 n ih0 k s))
("mullens" nil (m ah1 l ax n z))
("muller" nil (m ah1 l er0))
("mullet" nil (m ah1 l ax t))
("mullett" nil (m uw1 l ih0 t))
("mullican" nil (m ah1 l ih0 k ax n))
("mulligan" nil (m ah1 l ih0 g ax n))
("mulliken" nil (m ah1 l ih0 k ax n))
("mullikin" nil (m ah1 l ih0 k ih0 n))
("mullin" nil (m ah1 l ih0 n))
("mullinax" nil (m ah1 l ih0 n ae0 k s))
("mullineaux" nil (m ah1 l ih0 n ow1))
("mulling" nil (m ah1 l ih0 ng))
("mullings" nil (m ah1 l ih0 ng z))
("mullinix" nil (m ah1 l ih0 n ih0 k s))
("mullins" nil (m ah1 l ih1 n z))
("mullis" nil (m ah1 l ih0 s))
("mulloy" nil (m ah1 l oy0))
("mulls" nil (m ah1 l z))
("mulnix" nil (m ah1 l n ih0 k s))
("mulqueen" nil (m ax l k w iy1 n))
("mulroney" nil (m ax l r ow1 n iy0))
("mulrooney" nil (m ah1 l r uw0 n iy0))
("mulroy" nil (m ah1 l r oy1))
("mulry" nil (m ah1 l r iy0))
("multer" nil (m ah1 l t er0))
("multi" nil (m ah1 l t iy0))
("multibank" nil (m ah1 l t iy0 b ae1 ng k))
("multibillion" nil (m ah1 l t ay1 b ih1 l y ax n))
("multicandidate" nil (m ah1 l t iy0 k ae1 n d ih0 d ey1 t))
("multicenter" nil (m ah1 l t iy0 s eh1 n t er0))
("multicolored" nil (m ah1 l t iy0 k ah1 l er0 d))
("multicultural" nil (m ah1 l t iy0 k ah1 l ch er0 ax l))
("multiculturalism" nil (m ah1 l t iy0 k ah1 l ch er0 ax l ih0 z ax m))
("multiemployer" nil (m ah1 l t iy0 ih0 m p l oy1 er0))
("multifaceted" nil (m ah1 l t iy0 f ae1 s ax t ih0 d))
("multifamily" nil (m ah1 l t ay0 f ae1 m ax l iy0))
("multifoods" nil (m ah1 l t iy0 f uw1 d z))
("multilateral" nil (m ah1 l t ih0 l ae1 t er0 ax l))
("multilayer" nil (m ah1 l t iy0 l ey1 er0))
("multilayered" nil (m ah1 l t iy0 l ey1 er0 d))
("multilevel" nil (m ah1 l t ay0 l eh1 v ax l))
("multiline" nil (m ah1 l t iy0 l ay1 n))
("multilingual" nil (m ah1 l t iy0 l ih1 ng w ax l))
("multimarket" nil (m ah1 l t iy0 m aa1 r k ih0 t))
("multimate" nil (m ah1 l t iy0 m ey1 t))
("multimedia" nil (m ah1 l t iy0 m iy1 d iy0 ax))
("multimillion" nil (m ah1 l t ay1 m ih1 l y ax n))
("multimillionaire" nil (m ah1 l t iy0 m ih1 l y ax n eh1 r))
("multimillionaires" nil (m ah1 l t iy0 m ih1 l y ax n eh1 r z))
("multinational" nil (m ah1 l t ay1 n ae1 sh ax n ax l))
("multinationals" nil (m ah1 l t ay1 n ae1 sh ax n ax l z))
("multipart" nil (m ah1 l t iy0 p aa1 r t))
("multiparty" nil (m ah1 l t iy0 p aa1 r t iy0))
("multiplayer" nil (m ah1 l t iy0 p l ey1 er0))
("multiple" nil (m ah1 l t ax p ax l))
("multiples" nil (m ah1 l t ax p ax l z))
("multiplex" nil (m ah1 l t iy0 p l eh1 k s))
("multiplexer" nil (m ah1 l t iy0 p l eh1 k s er0))
("multiplexers" nil (m ah1 l t iy0 p l eh1 k s er0 z))
("multiplication" nil (m ah1 l t ax p l ax k ey1 sh ax n))
("multiplicity" nil (m ah1 l t ax p l ih1 s ih0 t iy0))
("multiplied" nil (m ah1 l t ax p l ay1 d))
("multiplier" nil (m ah1 l t ax p l ay1 er0))
("multiplies" nil (m ah1 l t ax p l ay1 z))
("multiply" nil (m ah1 l t ax p l ay1))
("multiplying" nil (m ah1 l t ax p l ay1 ih0 ng))
("multiprocessor" nil (m ah1 l t iy0 p r aa1 s eh1 s er0))
("multipurpose" nil (m ah1 l t iy0 p er1 p ax s))
("multiracial" nil (m ah1 l t ay1 r ey1 sh ax l))
("multistate" nil (m ah1 l t iy0 s t ey1 t))
("multistory" nil (m ah1 l t iy0 s t ao1 r iy0))
("multitasking" nil (m ah1 l t iy0 t ae1 s k ih0 ng))
("multitude" nil (m ah1 l t ax t uw1 d))
("multitudes" nil (m ah1 l t ax t y uw1 d z))
("multiuser" nil (m ah1 l t iy0 y uw1 z er0))
("multivalve" nil (m ah1 l t iy0 v ae0 l v))
("multivision" nil (m ah1 l t iy0 v ih1 zh ax n))
("multiyear" nil (m ah1 l t iy0 y iy1 r))
("multnomah" nil (m ah1 l n ow1 m ax))
("mulvaney" nil (m ah1 l v ey1 n iy0))
("mulvany" nil (m ah1 l v ey1 n iy0))
("mulvehill" nil (m ah1 l v hh ih0 l))
("mulvey" nil (m ax l v ey1))
("mulvihill" nil (m ah1 l v iy0 hh ih0 l))
("mum" nil (m ah1 m))
("muma" nil (m y uw1 m ax))
("mumaw" nil (m uw1 m ao0))
("mumble" nil (m ah1 m b ax l))
("mumbled" nil (m ah1 m b ax l d))
("mumbles" nil (m ah1 m b ax l z))
("mumbling" nil (m ah1 m b ax l ih0 ng))
("mumbo" nil (m ah1 m b ow0))
("mumby" nil (m ah1 m b iy0))
("mumford" nil (m ah1 m f er0 d))
("mumm" nil (m ah1 m))
("mumma" nil (m ah1 m ax))
("mumme" nil (m ah1 m))
("mummert" nil (m ah1 m er0 t))
("mummey" nil (m ah1 m iy0))
("mummies" nil (m ah1 m iy0 z))
("mummification" nil (m ah1 m ih0 f ih0 k ey1 sh ax n))
("mummified" nil (m ah1 m ih0 f ay1 d))
("mummifying" nil (m ah1 m ax f ay1 ih0 ng))
("mummy" nil (m ah1 m iy0))
("mumper" nil (m ah1 m p er0))
("mumphrey" nil (m ah1 m f r iy0))
("mumpower" nil (m ah1 m p ow0 er0))
("mumps" nil (m ah1 m p s))
("mums" nil (m ah1 m z))
("mun" nil (m ah1 n))
("munafo" nil (m uw0 n aa1 f ow0))
("munce" nil (m ah1 n s))
("muncey" nil (m ah1 n s iy0))
("munch" nil (m ah1 n ch))
("munchies" nil (m ah1 n ch iy1 z))
("munching" nil (m ah1 n ch ih0 ng))
("muncie" nil (m ah1 n s iy0))
("muncipal" nil (m ah1 n s ih0 p ax l))
("muncy" nil (m ah1 n s iy0))
("mund" nil (m ah1 n d))
("mundane" nil (m ax n d ey1 n))
("munday" nil (m ah1 n d ey1))
("mundell" nil (m ah1 n d ax l))
("munden" nil (m ah1 n d ax n))
("mundi" nil (m ah1 n d iy0))
("mundie" nil (m ah1 n d iy0))
("mundinger" nil (m ah1 n d ih0 ng er0))
("mundis" nil (m ah1 n d ih0 s))
("mundo" nil (m ah1 n d ow0))
("mundorf" nil (m ah1 n d ao0 r f))
("mundorff" nil (m ah1 n d ao0 r f))
("mundt" nil (m ah1 n t))
("mundy" nil (m ah1 n d iy0))
("munford" nil (m ah1 n f er0 d))
("munger" nil (m ah1 ng er0))
("mungia" nil (m uw1 n jh ax))
("mungin" nil (m ah1 ng g ih0 n))
("mungle" nil (m ah1 ng g ax l))
("mungo" nil (m ah1 ng g ow0))
("munguia" nil (m uw1 n g w iy0 ax))
("muni" nil (m y uw1 n iy0))
("munich" nil (m y uw1 n ih0 k))
("municipal" nil (m y uw0 n ih1 s ax p ax l))
("municipalities" nil (m y uw1 n ih0 s ax p ae1 l ih0 t iy0 z))
("municipality" nil (m y uw1 n ih0 s ax p ae1 l ax t iy0))
("municipally" nil (m y uw0 n ih1 s ih0 p ax l iy0))
("municipals" nil (m y uw0 n ih1 s ih0 p ax l z))
("munier" nil (m y uw1 n iy0 er0))
("munis" nil (m y uw1 n ih0 s))
("munitions" nil (m y uw0 n ih1 sh ax n z))
("muniz" nil (m y uw1 n ih0 z))
("munk" nil (m ah1 ng k))
("munkres" nil (m ah1 ng k er0 z))
("munley" nil (m ah1 n l iy0))
("munn" nil (m ah1 n))
("munnell" nil (m ah1 n ax l))
("munnerlyn" nil (m ax n er1 l ih0 n))
("munns" nil (m ah1 n z))
("munos" nil (m uw1 n ow0 z))
("munoz" nil (m uw1 n y ow0 z))
("munro" nil (m ax n r ow1))
("munroe" nil (m ah1 n r ow0))
("muns" nil (m ah1 n z))
("munsch" nil (m ah1 n sh))
("munsell" nil (m ah1 n s ax l))
("munsey" nil (m ah1 n z iy0))
("munsingwear" nil (m ah1 n s ih0 ng w eh1 r))
("munson" nil (m ah1 n s ax n))
("munster" nil (m ah1 n s t er0))
("munsterman" nil (m ah1 n s t er0 m ax n))
("munt" nil (m ah1 n t))
("muntean" nil (m ax n t iy1 n))
("munter" nil (m ah1 n t er0))
("muntz" nil (m ah1 n t s))
("munyan" nil (m ah1 n y ax n))
("munyon" nil (m ah1 n y ax n))
("munz" nil (m ah1 n z))
("munzer" nil (m ah1 n z er0))
("muoio" nil (m w ow1 iy0 ow0))
("muppet" nil (m ah1 p ax t))
("muppets" nil (m ah1 p ih0 t s))
("mura" nil (m uh1 r ax))
("murad" nil (m y uh1 r ae0 d))
("murai" nil (m y er0 ay1))
("murakami" nil (m uh0 r aa0 k aa1 m iy0))
("mural" nil (m y uh1 r ax l))
("murali" nil (m er0 aa1 l iy0))
("murals" nil (m y uh1 r ax l z))
("muramatsu" nil (m uw1 r aa0 m aa1 t s uw1))
("muramoto" nil (m uh0 r aa0 m ow1 t ow0))
("muranaga" nil (m er0 r ax n aa1 g ax))
("murano" nil (m uh0 r aa1 n ow0))
("muraoka" nil (m uh0 r aa0 ow1 k ax))
("murase" nil (m y er1 ey0 z))
("muraski" nil (m er0 aa1 s k iy0))
("murasky" nil (m er0 ae1 s k iy0))
("murata" nil (m uh0 r aa1 t ax))
("muratore" nil (m uh0 r aa0 t ao1 r ey0))
("murawski" nil (m er0 aa1 f s k iy0))
("murayama" nil (m er1 ay0 aa1 m ax))
("murch" nil (m er1 k))
("murchie" nil (m er1 ch iy0))
("murchinson" nil (m er1 ch ih0 n s ax n))
("murchison" nil (m er1 ch ih0 s ax n))
("murdaugh" nil (m er1 d ao0))
("murden" nil (m er1 d ax n))
("murder" nil (m er1 d er0))
("murdered" nil (m er1 d er0 d))
("murderer" nil (m er1 d er0 er0))
("murderers" nil (m er1 d er0 er0 z))
("murdering" nil (m er1 d er0 ih0 ng))
("murderous" nil (m er1 d er0 ax s))
("murders" nil (m er1 d er0 z))
("murdick" nil (m er1 d ih0 k))
("murdoch" nil (m er1 d aa0 k))
("murdock" nil (m er1 d aa0 k))
("murdy" nil (m er1 d iy0))
("murff" nil (m er1 f))
("murfin" nil (m er1 f ih0 n))
("murguia" nil (m uh1 r g w iy0 ax))
("muri" nil (m uh1 r iy0))
("murial" nil (m y uh1 r iy0 ax l))
("murias" nil (m y uh1 r iy0 ax s))
("muriel" nil (m y uh1 r iy0 ax l))
("murihuro" nil (m uh1 r iy0 hh uh1 r ow0))
("murin" nil (m y uh1 r ih0 n))
("murine" nil (m y uh1 r iy1 n))
("murjani" nil (m er0 jh aa1 n iy0))
("murk" nil (m er1 k))
("murkier" nil (m er1 k iy0 er0))
("murkowski" nil (m er0 k aw1 s k iy0))
("murky" nil (m er1 k iy0))
("murley" nil (m er1 l iy0))
("murmansk" nil (m er0 m ae0 n s k))
("murmur" nil (m er1 m er0))
("murmured" nil (m er1 m er0 d))
("murmuring" nil (m er1 m er0 ih0 ng))
("murmurs" nil (m er1 m er0 z))
("murnan" nil (m er1 n ax n))
("murnane" nil (m er1 n ax n))
("muro" nil (m uh1 r ow0))
("murph" nil (m er1 f))
("murphey" nil (m er1 f iy0))
("murphree" nil (m er0 f r iy1))
("murphrey" nil (m er1 f r iy0))
("murphy" nil (m er1 f iy0))
("murphys" nil (m er1 f iy0 z))
("murr" nil (m er1))
("murrah" nil (m ao1 r ax))
("murray" nil (m er1 iy0))
("murree" nil (m er1 iy0))
("murrelet" nil (m er1 l ih0 t))
("murrell" nil (m ao1 r ax l))
("murren" nil (m er1 ax n))
("murrey" nil (m er1 iy0))
("murrie" nil (m er1 iy0))
("murrieta" nil (m uh0 r iy1 t ax))
("murrill" nil (m ao1 r ax l))
("murrin" nil (m ao1 r ih0 n))
("murrow" nil (m ao1 r ow0))
("murry" nil (m er1 iy0))
("murtagh" nil (m er1 t ax g))
("murtaugh" nil (m er1 t ao0))
("murtha" nil (m er1 th ax))
("murthy" nil (m er1 th iy0))
("murto" nil (m er1 t ow1))
("murton" nil (m er1 t ax n))
("murty" nil (m er1 t iy0))
("murveit" nil (m er0 v iy1 t))
("murzyn" nil (m er1 z ih0 n))
("musa" nil (m y uw1 s ax))
("musacchio" nil (m y uw1 s ae1 k iy0 ow0))
("musante" nil (m uw0 s aa1 n t iy0))
("musarra" nil (m uw0 s aa1 r ax))
("musavi" nil (m y uw0 s aa1 v iy0))
("musburger" nil (m ah1 s b er0 g er0))
("muscarella" nil (m uw0 s k aa0 r eh1 l ax))
("muscarello" nil (m uw0 s k aa0 r eh1 l ow0))
("muscat" nil (m ah1 s k ae0 t))
("muscato" nil (m uw0 s k aa1 t ow0))
("musch" nil (m ah1 sh))
("muscle" nil (m ah1 s ax l))
("muscled" nil (m ah1 s ax l d))
("muscles" nil (m ah1 s ax l z))
("muscling" nil (m ah1 s ax l ih0 ng))
("musco" nil (m uw1 s k ow0))
("muscocho" nil (m ax s k aa1 ch ow0))
("muscovite" nil (m ah1 s k ax v ay1 t))
("muscovites" nil (m ah1 s k ax v ay1 t s))
("muscular" nil (m ah1 s k y ax l er0))
("musculature" nil (m ah1 s k y ax l ax ch er0))
("muse" nil (m y uw1 z))
("mused" nil (m y uw1 z d))
("musee" nil (m y uw1 z iy1))
("muses" nil (m y uw1 z ax z))
("museum" nil (m y uw0 z iy1 ax m))
("museums" nil (m y uw0 z iy1 ax m z))
("musgrave" nil (m ah1 s g r ax v))
("musgrove" nil (m ah1 s g r ax v))
("mush" nil (m ah1 sh))
("mushroom" nil (m ah1 sh r uw0 m))
("mushroomed" nil (m ah1 sh r uh1 m d))
("mushrooming" nil (m ah1 sh r uh1 m ih0 ng))
("mushrooms" nil (m ah1 sh r uw0 m z))
("mushrush" nil (m ah1 sh r ax sh))
("mushtaq" nil (m ah1 sh t ae0 k))
("mushy" nil (m ah1 sh iy0))
("musial" nil (m y uw1 z iy0 ax l))
("music" nil (m y uw1 z ih0 k))
("musica" nil (m y uw1 z ih0 k ax))
("musical" nil (m y uw1 z ih0 k ax l))
("musicality" nil (m y uw1 z ih0 k ae1 l ax t iy0))
("musically" nil (m y uw1 z ih0 k l iy0))
("musicals" nil (m y uw1 z ih0 k ax l z))
("musich" nil (m y uw1 s ih0 k))
("musician" nil (m y uw0 z ih1 sh ax n))
("musicians" nil (m y uw0 z ih1 sh ax n z))
("musick" nil (m y uw1 s ih0 k))
("musicland" nil (m y uw1 z ih0 k l ae1 n d))
("musicologist" nil (m y uw1 z ih0 k aa1 l ax jh ax s t))
("musidora" nil (m uw0 s iy0 d ao1 r ax))
("musil" nil (m uw1 z ax l))
("musing" nil (m y uw1 z ih0 ng))
("musings" nil (m y uw1 z ih0 ng z))
("musk" nil (m ah1 s k))
("muska" nil (m ah1 s k ax))
("muske" nil (m ah1 s k))
("muskegon" nil (m ax s k iy1 g ih0 n))
("musket" nil (m ah1 s k ax t))
("muskie" nil (m ah1 s k iy0))
("muskmelon" nil (m ah1 s k m eh1 l ax n))
("muskogee" nil (m ax s k ow1 g iy0))
("muskopf" nil (m ah1 s k ao0 p f))
("muskrat" nil (m ah1 s k r ae1 t))
("musky" nil (m ah1 s k iy0))
("muslim" nil (m ah1 z l ax m))
("muslims" nil (m ah1 z l ax m z))
("muslin" nil (m ah1 z l ax n))
("musman" nil (m ah1 s m ax n))
("musolf" nil (m y uw1 s ow1 l f))
("musolino" nil (m uw0 s ow0 l iy1 n ow0))
("musschenbroek" nil (m ax s ch eh1 n b r ow0 k))
("mussell" nil (m ah1 s ax l))
("musselman" nil (m ah1 s ax l m ax n))
("mussels" nil (m ah1 s ax l z))
("musselwhite" nil (m ah1 s ax l w ay1 t))
("musser" nil (m ah1 s er0))
("mussman" nil (m ah1 s m ax n))
("musso" nil (m uw1 s ow0))
("mussolini" nil (m uw1 s ow0 l iy1 n iy0))
("musson" nil (m ah1 s ax n))
("must" nil (m ah1 s t))
("mustache" nil (m ah1 s t ae1 sh))
("mustached" nil (m ah1 s t ae1 sh t))
("mustaches" nil (m ah1 s t ae1 sh ih0 z))
("mustachioed" nil (m ax s t ae1 sh iy0 ow1 d))
("mustafa" nil (m uw0 s t aa1 f ax))
("mustain" nil (m ax s t ey1 n))
("mustang" nil (m ah1 s t ae1 ng))
("mustangs" nil (m ah1 s t ae1 ng z))
("mustard" nil (m ah1 s t er0 d))
("mustards" nil (m ah1 s t er0 d z))
("muster" nil (m ah1 s t er0))
("mustered" nil (m ah1 s t er0 d))
("mustering" nil (m ah1 s t er0 ih0 ng))
("musters" nil (m ah1 s t er0 z))
("mustin" nil (m ah1 s t ih0 n))
("musto" nil (m ah1 s t ow0))
("mustoe" nil (m ah1 s t ow0))
("muston" nil (m ah1 s t ax n))
("musty" nil (m ah1 s t iy0))
("musumeci" nil (m uw0 s uw0 m eh1 ch iy0))
("muszynski" nil (m ax sh ih1 n s k iy0))
("mutagens" nil (m y uw1 t ax jh eh0 n z))
("mutant" nil (m y uw1 t ax n t))
("mutate" nil (m y uw1 t ey1 t))
("mutated" nil (m y uw1 t ey1 t ih0 d))
("mutation" nil (m y uw0 t ey1 sh ax n))
("mutations" nil (m y uw0 t ey1 sh ax n z))
("mutch" nil (m ah1 ch))
("mutchler" nil (m ah1 ch l er0))
("mute" nil (m y uw1 t))
("muted" nil (m y uw1 t ih0 d))
("mutert" nil (m y uw1 t er0 t))
("muth" nil (m uw1 th))
("muti" nil (m uw1 t iy1))
("mutilate" nil (m y uw1 t ax l ey1 t))
("mutilated" nil (m y uw1 t ax l ey1 t ax d))
("mutilating" nil (m y uw1 t ax l ey1 t ih0 ng))
("mutilation" nil (m y uw1 t ax l ey1 sh ax n))
("mutineer" nil (m y uw1 t ax n ih1 r))
("mutineers" nil (m y uw1 t ih0 n ih1 r z))
("muting" nil (m y uw1 t ih0 ng))
("mutinied" nil (m y uw1 t ax n iy0 d))
("mutinies" nil (m y uw1 t ax n iy0 z))
("mutinous" nil (m y uw1 t ax n ax s))
("mutiny" nil (m y uw1 t ax n iy0))
("mutlangen" nil (m ax t l ae1 ng g ax n))
("mutsch" nil (m ah1 ch))
("mutschler" nil (m ah1 ch l er0))
("mutt" nil (m ah1 t))
("mutter" nil (m ah1 t er0))
("muttered" nil (m ah1 t er0 d))
("muttering" nil (m ah1 t er0 ih0 ng))
("mutterings" nil (m ah1 t er0 ih0 ng z))
("mutters" nil (m ah1 t er0 z))
("mutton" nil (m ah1 t ax n))
("mutts" nil (m ah1 t s))
("mutual" nil (m y uw1 ch ax w ax l))
("mutualism" nil (m y uw1 ch ax w ax l ih1 z ax m))
("mutually" nil (m y uw1 ch uw0 ax l iy0))
("mutz" nil (m ah1 t s))
("mutzman" nil (m ah1 t s m ax n))
("muzak" nil (m y uw1 z ae0 k))
("muzik" nil (m y uw1 z ih0 k))
("muzio" nil (m y uw1 z iy0 ow0))
("muzyka" nil (m y uw0 z ih1 k ax))
("muzzey" nil (m ah1 z iy0))
("muzzle" nil (m ah1 z ax l))
("muzzled" nil (m ah1 z ax l d))
("muzzles" nil (m ah1 z ax l z))
("muzzy" nil (m ah1 z iy0))
("my" nil (m ay1))
("myanmar" nil (m y aa1 n m aa1 r))
("myatt" nil (m ay1 ax t))
("mycenaean" nil (m ay1 s ax n iy1 ax n))
("mycogen" nil (m ay1 k ow0 g ax n))
("mycology" nil (m ay0 k aa1 l ax jh iy0))
("myelin" nil (m ay1 ih0 l ih0 n))
("myer" nil (m ay1 er0))
("myers" nil (m ay1 er0 z))
("myerson" nil (m ay1 r s ax n))
("myette" nil (m ay1 eh1 t))
("myhand" nil (m ay1 hh ae1 n d))
("myhre" nil (m ih1 r))
("myhren" nil (m ih1 r ax n))
("myklebust" nil (m ih1 k l ih0 b ax s t))
("mylan" nil (m ay1 l ax n))
("myler" nil (m ay1 l er0))
("myles" nil (m ay1 l z))
("mylod" nil (m ay1 l aa0 d))
("myna" nil (m ay1 n ax))
("mynatt" nil (m ih1 n ax t))
("mynhier" nil (m ih1 n hh iy0 er0))
("myocardial" nil (m ay1 ax k aa1 r d iy0 ax l))
("myocardium" nil (m ay1 ax k aa1 r d iy0 ax m))
("myojin" nil (m y ow1 jh ih1 n))
("myopia" nil (m ay0 ow1 p iy0 ax))
("myopic" nil (m ay0 aa1 p ih0 k))
("myoscint" nil (m y ao1 s ih0 n t))
("myosin" nil (m ay1 ax s ax n))
("myotrophin" nil (m ay1 ow0 t r ow1 f ih0 n))
("myra" nil (m ay1 r ax))
("myracle" nil (m ih1 r ax k ax l))
("myre" nil (m ay1 r))
("myren" nil (m ay1 r ax n))
("myreon" nil (m ay1 r iy0 ax n))
("myres" nil (m ay1 r z))
("myriad" nil (m ih1 r iy0 ax d))
("myrick" nil (m ay1 r ih0 k))
("myrilla" nil (m ih0 r ih1 l ax))
("myrlene" nil (m ih1 r l iy0 n))
("myrmidon" nil (m er1 m ih0 d ih0 n))
("myrmidons" nil (m er1 m ih0 d ih0 n z))
("myrna" nil (m er1 n ax))
("myron" nil (m ay1 r ax n))
("myrrh" nil (m er1))
("myrta" nil (m er1 t ax))
("myrtia" nil (m er1 sh ax))
("myrtice" nil (m ih1 r t ih0 s))
("myrtis" nil (m ih1 r t ih0 s))
("myrtle" nil (m er1 t ax l))
("myrtles" nil (m er1 t ax l z))
("myself" nil (m ay1 s eh1 l f))
("myslinski" nil (m ih0 s l ih1 n s k iy0))
("mysliwiec" nil (m ih0 s l ih1 v iy0 k))
("mystere" nil (m ih1 s t ih1 r))
("mysteries" nil (m ih1 s t er0 iy0 z))
("mysterious" nil (m ih0 s t ih1 r iy0 ax s))
("mysteriously" nil (m ih1 s t ih1 r iy0 ax s l iy0))
("mysterium" nil (m ih1 s t iy1 r iy0 ax m))
("mystery" nil (m ih1 s t er0 iy0))
("mystic" nil (m ih1 s t ih0 k))
("mystical" nil (m ih1 s t ih0 k ax l))
("mysticism" nil (m ih1 s t ih0 s ih1 z ax m))
("mystics" nil (m ih1 s t ih0 k s))
("mystified" nil (m ih1 s t ax f ay1 d))
("mystifying" nil (m ih1 s t ax f ay1 ih0 ng))
("mystique" nil (m ih0 s t iy1 k))
("myszka" nil (m ih1 sh k ax))
("myth" nil (m ih1 th))
("mythic" nil (m ih1 th ih0 k))
("mythical" nil (m ih1 th ax k ax l))
("mythological" nil (m ih1 th ax l aa1 jh ih0 k ax l))
("mythology" nil (m ax th aa1 l ax jh iy0))
("myths" nil (m ih1 th s))
("myung" nil (m y ah1 ng))
("n" nil (eh1 n))
("na" nil (n aa1))
("naab" nil (n aa1 b))
("naas" nil (n aa1 z))
("nab" nil (n ae1 b))
("nabb" nil (n ae1 b))
("nabbed" nil (n ae1 b d))
("nabbing" nil (n ae1 b ih0 ng))
("naber" nil (n ey1 b er0))
("nabers" nil (n ey1 b er0 z))
("nabi" nil (n aa1 b iy0))
("nabil" nil (n aa0 b iy1))
("nabisco" nil (n ax b ih1 s k ow0))
("nablus" nil (n ae1 b l ax s))
("nabob" nil (n ae1 b aa0 b))
("nabokov" nil (n ae1 b ax k aa1 v))
("nabors" nil (n ey1 b er0 z))
("nabozny" nil (n ax b aa1 z n iy0))
("naccarato" nil (n aa0 k er0 aa1 t ow0))
("nacchio" nil (n ae1 k iy0 ow0))
("nacco" nil (n ae1 k ow0))
("nace" nil (n ey1 s))
("nachbar" nil (n ae1 k b er0))
("nachman" nil (n ae1 k m ax n))
("nachmany" nil (n ae1 k m ax n iy0))
("nacho" nil (n ae1 ch ow0))
("nachreiner" nil (n ae1 k r ay0 n er0))
("nachtigal" nil (n ae1 ch t ih0 g ax l))
("nacional" nil (n ey1 s iy1 ax n ax l))
("nack" nil (n ae1 k))
("nacke" nil (n ae1 k))
("naclerio" nil (n aa0 k l eh1 r iy0 ow0))
("nacobre" nil (n ae1 k ax b r ax))
("nacogdoches" nil (n ax k aa1 g d ax ch ih0 z))
("nad" nil (n ae1 d))
("nada" nil (n aa1 d ax))
("nadeau" nil (n ax d ow1))
("nadein" nil (n ax d ay1 n))
("nadel" nil (n aa0 d eh1 l))
("nadell" nil (n aa0 d ey1 l))
("nader" nil (n ey1 d er0))
("naderites" nil (n ey1 d er0 ay1 t s))
("nadgwick" nil (n ae1 jh w ih1 k))
("nadine" nil (n ax d iy1 n))
("nading" nil (n ey1 d ih0 ng))
("nadir" nil (n ey1 d er0))
("nadja" nil (n ae1 d y ax))
("nadler" nil (n ey1 d ax l er0))
("nadolny" nil (n ax d ow1 l n iy0))
("nadolski" nil (n ax d ow1 l s k iy0))
("naef" nil (n iy1 f))
("naegele" nil (n eh1 g ax l))
("naegeli" nil (n eh1 g ih0 l iy0))
("naeger" nil (n eh1 g er0))
("naeve" nil (n iy1 v))
("nafco" nil (n ae1 f k ow0))
("naff" nil (n ae1 f))
("naffziger" nil (n ae1 f z ih0 g er0))
("nafta" nil (n ae1 f t ax))
("naftalis" nil (n ae0 f t aa1 l ih0 s))
("nafziger" nil (n ae1 f z ih0 g er0))
("nag" nil (n ae1 g))
("nagai" nil (n aa0 g aa1 iy0))
("nagano" nil (n aa0 g aa1 n ow0))
("nagao" nil (n aa0 g aa1 ow0))
("nagasaki" nil (n aa1 g aa0 s aa1 k iy0))
("nagata" nil (n aa0 g aa1 t ax))
("nagel" nil (n ey1 g ax l))
("nagele" nil (n ae1 g ax l))
("nagengast" nil (n ae1 g ih0 ng g ax s t))
("nager" nil (n ey1 g er0))
("nagged" nil (n ae1 g d))
("nagging" nil (n ae1 g ih0 ng))
("nagi" nil (n aa1 g iy0))
("nagin" nil (n ey1 g ih0 n))
("nagle" nil (n ey1 g ax l))
("nagler" nil (n ey1 g l er0))
("nagorno" nil (n ax g ao1 r n ow0))
("nagorski" nil (n ax g ao1 r s k iy0))
("nagoya" nil (n ax g oy1 ax))
("nagy" nil (n ey1 g iy0))
("nagymaros" nil (n ae1 jh ih0 m aa1 r ow0 s))
("nah" nil (n aa1))
("nahas" nil (n aa1 hh ax z))
("nahm" nil (n ae1 m))
("nahmias" nil (n aa1 m iy0 ax z))
("nahum" nil (n ae1 hh ax m))
("naidoo" nil (n ey1 d uw1))
("naify" nil (n ey1 f ay1))
("naik" nil (n ey1 k))
("nail" nil (n ey1 l))
("nailed" nil (n ey1 l d))
("nailing" nil (n ey1 l ih0 ng))
("naill" nil (n ey1 l))
("nailor" nil (n ey1 l er0))
("nails" nil (n ey1 l z))
("naim" nil (n ay1 ih0 m))
("naiman" nil (n ey1 m ax n))
("nair" nil (n eh1 r))
("naira" nil (n eh1 r ax))
("nairn" nil (n eh1 r n))
("nairne" nil (n eh1 r n))
("nairobi" nil (n ay1 r ow1 b iy0))
("naisbitt" nil (n ey1 s b ih0 t))
("naish" nil (n aa1 ih0 sh))
("naismith" nil (n ey1 z m ih0 th))
("naito" nil (n ey1 t ow0))
("naive" nil (n ay1 iy1 v))
("naively" nil (n aa0 iy1 v l iy0))
("naivete" nil (n aa0 iy1 v ax t ey1))
("najar" nil (n aa0 y aa1 r))
("najarian" nil (n ax jh eh1 r iy0 ax n))
("najera" nil (n aa0 jh eh1 r ax))
("najib" nil (n ax jh iy1 b))
("najibullah" nil (n ae1 jh ih0 b uh1 l ax))
("najjar" nil (n aa0 y aa1 r))
("nakagawa" nil (n aa0 k aa0 g aa1 w ax))
("nakahara" nil (n aa0 k aa0 hh aa1 r ax))
("nakai" nil (n aa0 k aa1 iy0))
("nakajima" nil (n aa0 k aa0 jh iy1 m ax))
("nakama" nil (n aa0 k aa1 m ax))
("nakamoto" nil (n aa0 k aa0 m ow1 t ow0))
("nakamura" nil (n aa0 k aa0 m uh1 r ax))
("nakanishi" nil (n aa0 k aa0 n iy1 sh iy0))
("nakano" nil (n aa0 k aa1 n ow0))
("nakao" nil (n aa0 k aa1 ow0))
("nakashima" nil (n aa0 k aa0 sh iy1 m ax))
("nakasone" nil (n aa1 k ax s ow1 n iy0))
("nakata" nil (n aa0 k aa1 t ax))
("nakatani" nil (n aa0 k aa0 t aa1 n iy0))
("nakayama" nil (n aa0 k aa0 y aa1 m ax))
("naked" nil (n ey1 k ax d))
("nakhamkin" nil (n ae1 k ae1 m k ih0 n))
("nalbandian" nil (n ae1 l b ae1 n d iy0 ax n))
("nalbone" nil (n ae1 l b ow1 n))
("nalcap" nil (n ae1 l k ae0 p))
("nalco" nil (n ae1 l k ow0))
("naldo" nil (n aa1 l d ow0))
("nale" nil (n ey1 l))
("nalepa" nil (n ax l iy1 p ax))
("nall" nil (n ao1 l))
("nalley" nil (n ae1 l iy0))
("nalls" nil (n ao1 l z))
("nally" nil (n ae1 l iy0))
("nam" nil (n ae1 m))
("namby" nil (n ae1 m b iy0))
("name" nil (n ey1 m))
("named" nil (n ey1 m d))
("nameless" nil (n ey1 m l ax s))
("namely" nil (n ey1 m l iy0))
("nameplate" nil (n ey1 m p l ey1 t))
("nameplates" nil (n ey1 m p l ey1 t s))
("namer" nil (n ey1 m er0))
("names" nil (n ey1 m z))
("namesake" nil (n ey1 m s ey1 k))
("namibia" nil (n ax m ih1 b iy0 ax))
("namibian" nil (n ax m ih1 b iy0 ax n))
("naming" nil (n ey1 m ih0 ng))
("namphy" nil (n ae1 m f iy0))
("nan" nil (n ae1 n))
("nana" nil (n ae1 n ax))
("nancarrow" nil (n ae1 ng k ae1 r ow0))
("nance" nil (n ae1 n s))
("nanchang" nil (n ae1 n ch ae1 ng))
("nancy" nil (n ae1 n s iy0))
("nandaime" nil (n ae1 n d ey1 m))
("nandi" nil (n aa1 n d iy0))
("nandita" nil (n aa0 n d iy1 t ax))
("nando" nil (n ae1 n d ow0))
("nanette" nil (n ax n eh1 t))
("nanez" nil (n aa0 n eh1 z))
("nang" nil (n ae1 ng))
("nangaku" nil (n ae1 ng g aa1 k uw1))
("nangle" nil (n ae1 ng g ax l))
("nania" nil (n aa1 n iy0 ax))
("nanjing" nil (n ae1 n jh ih0 ng))
("nankai" nil (n ae0 ng k ay1))
("nankervis" nil (n ae1 ng k er0 v ih0 s))
("nanking" nil (n ae1 ng k ih0 ng))
("nanna" nil (n ae1 n ax))
("nanney" nil (n ae1 n iy0))
("nanni" nil (n ae1 n iy0))
("nannies" nil (n ae1 n iy0 z))
("nanninga" nil (n aa0 n iy1 ng g ax))
("nannini" nil (n aa0 n iy1 n iy0))
("nanny" nil (n ae1 n iy0))
("nanos" nil (n aa1 n ow0 z))
("nanoseconds" nil (n ax n ow1 s ax k ax n d z))
("nantes" nil (n ae1 n t s))
("nantucket" nil (n ae0 n t ah1 k ih0 t))
("nantz" nil (n ae1 n t s))
("naoma" nil (n aa0 ow1 m ax))
("naomi" nil (n ey0 ow1 m iy0))
("nap" nil (n ae1 p))
("napa" nil (n ae1 p ax))
("napalm" nil (n ey1 p aa0 m))
("napco" nil (n ae1 p k ow0))
("napea" nil (n aa1 p iy0 ax))
("naperville" nil (n ey1 p er0 v ih1 l))
("naphtha" nil (n ae1 f th ax))
("naphthalene" nil (n ae1 f th ax l iy1 n))
("napier" nil (n ey1 p iy0 er0))
("napierala" nil (n aa0 p ih0 r aa1 l ax))
("napkin" nil (n ae1 p k ih0 n))
("napkins" nil (n ae1 p k ih0 n z))
("naples" nil (n ey1 p ax l z))
("napoleon" nil (n ax p ow1 l iy0 ax n))
("napoleonic" nil (n ax p ow1 l iy0 aa1 n ih0 k))
("napoles" nil (n ae1 p ax l z))
("napoletano" nil (n aa0 p ow0 l eh0 t aa1 n ow0))
("napoli" nil (n ae1 p ax l iy0))
("napolitan" nil (n aa0 p ow0 l iy0 t aa1 n))
("napolitano" nil (n aa0 p ow0 l iy0 t aa1 n ow0))
("napora" nil (n aa0 p ao1 r ax))
("napp" nil (n ae1 p))
("nappa" nil (n ae1 p ax))
("napped" nil (n ae1 p t))
("napper" nil (n ae1 p er0))
("nappi" nil (n ae1 p iy0))
("nappier" nil (n ae1 p iy0 er0))
("napping" nil (n ae1 p ih0 ng))
("nappo" nil (n ae1 p ow0))
("naprosyn" nil (n ae1 p r ax s ih0 n))
("naps" nil (n ae1 p s))
("naquin" nil (n ae1 k w ih0 n))
("naqvi" nil (n ae1 k v iy0))
("nara" nil (n aa1 r ax))
("naramore" nil (n aa0 r aa1 m ao0 r))
("naranjo" nil (n aa0 r aa1 n y ow0))
("narasimha" nil (n aa1 r ax s ih1 m hh aa1))
("narasimhan" nil (n aa1 r ax s ih1 m hh aa1 n))
("narayan" nil (n aa0 r aa1 y aa0 n))
("narayanan" nil (n aa1 r ay0 aa1 n ax n))
("narazaki" nil (n aa1 r aa0 z aa1 k iy0))
("narc" nil (n aa1 r k))
("narciso" nil (n aa0 r ch iy1 s ow0))
("narcisse" nil (n aa1 r s ih0 s))
("narcissism" nil (n aa1 r s ih0 s ih1 z ax m))
("narcissistic" nil (n aa1 r s ih0 s ih1 s t ih0 k))
("narcissus" nil (n aa0 r s ih1 s ax s))
("narco" nil (n aa1 r k ow0))
("narcolepsy" nil (n aa1 r k ax l eh1 p s iy0))
("narcos" nil (n aa1 r k ow0 s))
("narcotic" nil (n aa0 r k aa1 t ih0 k))
("narcotics" nil (n aa0 r k aa1 t ih0 k s))
("nard" nil (n aa1 r d))
("narda" nil (n aa1 r d ax))
("nardella" nil (n aa1 r d eh1 l ax))
("nardelli" nil (n aa0 r d eh1 l iy0))
("nardi" nil (n aa1 r d iy0))
("nardiello" nil (n aa0 r d iy0 eh1 l ow0))
("nardini" nil (n aa0 r d iy1 n iy0))
("nardo" nil (n aa1 r d ow0))
("nardone" nil (n aa0 r d ow1 n iy0))
("nardozzi" nil (n aa0 r d ow1 t s iy0))
("narducci" nil (n aa0 r d uw1 ch iy0))
("nares" nil (n ae1 r z))
("narita" nil (n eh0 r iy1 t ax))
("narjes" nil (n aa1 r y ax s))
("narlikar" nil (n aa1 r l ih0 k aa1 r))
("narlock" nil (n aa1 r l ax k))
("naro" nil (n aa1 r ow0))
("narodny" nil (n er0 aa1 d n iy0))
("naron" nil (n aa0 r ao1 n))
("narragansett" nil (n eh1 r ax g ae1 n s ax t))
("narramore" nil (n aa0 r aa1 m ao0 r))
("narrate" nil (n eh1 r ey1 t))
("narrated" nil (n eh1 r ey1 t ih0 d))
("narrates" nil (n eh0 r ey1 t s))
("narration" nil (n eh0 r ey1 sh ax n))
("narrative" nil (n ae1 r ax t ih0 v))
("narratives" nil (n ae1 r ax t ih0 v z))
("narrator" nil (n eh1 r ey0 t er0))
("narrators" nil (n eh1 r ey1 t er0 z))
("narron" nil (n ae1 r ax n))
("narrow" nil (n eh1 r ow0))
("narrowed" nil (n eh1 r ow0 d))
("narrower" nil (n eh1 r ow0 er0))
("narrowest" nil (n eh1 r ow0 ax s t))
("narrowing" nil (n eh1 r ow0 ih0 ng))
("narrowly" nil (n eh1 r ow0 l iy0))
("narrowness" nil (n eh1 r ow0 n ax s))
("narrows" nil (n eh1 r ow0 z))
("nars" nil (n aa1 r z))
("narum" nil (n er0 ah1 m))
("narvaez" nil (n aa0 r v aa1 eh0 z))
("narveson" nil (n aa1 r v ih0 s ax n))
("nary" nil (n eh1 r iy0))
("nasa" nil (n ae1 s ax))
("nasal" nil (n ey1 z ax l))
("nasally" nil (n ey1 z ax l iy0))
("nasby" nil (n ae1 s b iy0))
("nasca" nil (n aa1 s k ax))
("nascar" nil (n ae1 s k aa1 r))
("nascent" nil (n ey1 s ax n t))
("nascimento" nil (n ae1 s ih0 m eh1 n t ow0))
("nasco" nil (n ae1 s k ow0))
("nasdaq" nil (n ae1 z d ae0 k))
("nase" nil (n ey1 z))
("naser" nil (n ey1 z er0))
("nash" nil (n ae1 sh))
("nashashibi" nil (n ae1 sh ax sh iy1 b iy0))
("nashburg" nil (n ae1 sh b er0 g))
("nashua" nil (n ae1 sh uw0 ax))
("nashville" nil (n ae1 sh v ih0 l))
("nasional" nil (n ae1 s iy0 ax n ae1 l))
("naslund" nil (n ae1 s l ax n d))
("naso" nil (n aa1 s ow0))
("nason" nil (n ae1 s ax n))
("nasopharynx" nil (n ax s aa1 f er0 ih0 ng k s))
("nasr" nil (n aa1 s er0))
("nass" nil (n ae1 s))
("nassar" nil (n ae1 s er0))
("nassau" nil (n ae1 s ao0))
("nasser" nil (n ae1 s er0))
("nassif" nil (n ae1 s ih0 f))
("nast" nil (n ae1 s t))
("nasta" nil (n ae1 s t ax))
("nastasi" nil (n aa0 s t aa1 s iy0))
("nastier" nil (n ae1 s t iy0 er0))
("nastiest" nil (n ae1 s t iy0 ax s t))
("nastiness" nil (n ae1 s t iy0 n ax s))
("nasty" nil (n ae1 s t iy0))
("nasworthy" nil (n ae1 s w er0 dh iy0))
("nat" nil (n ae1 t))
("nata" nil (n aa1 t ax))
("natal" nil (n ey1 t ax l))
("natala" nil (n aa0 t aa1 l ax))
("natale" nil (n aa0 t aa1 l iy0))
("natali" nil (n aa0 t aa1 l iy0))
("natalia" nil (n ax t aa1 l y ax))
("natalie" nil (n ae1 t ax l iy0))
("natalle" nil (n ax t aa1 l ey0))
("natan" nil (n ey1 t ax n))
("natasha" nil (n ax t aa1 sh ax))
("natcher" nil (n ae1 ch er0))
("natchez" nil (n ae1 ch eh1 z))
("nate" nil (n ey1 t))
("nath" nil (n ae1 th))
("nathalia" nil (n ax th ae1 l iy0 ax))
("nathalie" nil (n ae1 t ax l iy0))
("nathan" nil (n ey1 th ax n))
("nathanael" nil (n ae1 th ax n ey1 l))
("nathania" nil (n ax th ae1 n iy0 ax))
("nathaniel" nil (n ax th ae1 n y ax l))
("nathanson" nil (n ae1 th ax n s ax n))
("nathe" nil (n ey1 dh))
("nathene" nil (n ax th iy1 n))
("natick" nil (n ey1 t ih0 k))
("nation" nil (n ey1 sh ax n))
("nationair" nil (n ey1 sh ax n eh1 r))
("national" nil (n ae1 sh ax n ax l))
("nationale" nil (n ae1 sh ax n ae1 l ey0))
("nationales" nil (n ae1 sh ax n aa1 l eh0 s))
("nationalism" nil (n ae1 sh ax n ax l ih1 z ax m))
("nationalist" nil (n ae1 sh ax n ax l ax s t))
("nationalistic" nil (n ae1 sh ax n ax l ih1 s t ih0 k))
("nationalists" nil (n ae1 sh ax n ax l ih0 s t s))
("nationalities" nil (n ae1 sh ax n ae1 l ih0 t iy0 z))
("nationality" nil (n ae1 sh ax n ae1 l ax t iy0))
("nationalization" nil (n ae1 sh ax n ax l ax z ey1 sh ax n))
("nationalizations" nil (n ae1 sh ax n ax l ax z ey1 sh ax n z))
("nationalize" nil (n ae1 sh ax n ax l ay1 z))
("nationalized" nil (n ae1 sh ax n ax l ay1 z d))
("nationalizing" nil (n ae1 sh ax n ax l ay1 z ih0 ng))
("nationally" nil (n ae1 sh ax n ax l iy0))
("nationals" nil (n ae1 sh ax n ax l z))
("nationhood" nil (n ey1 sh ax n hh uh1 d))
("nations" nil (n ey1 sh ax n z))
("nationsbanc" nil (n ey1 sh ax n z b ae1 ng k))
("nationsbank" nil (n ey1 sh ax n z b ae1 ng k))
("nationwide" nil (n ey1 sh ax n w ay1 d))
("native" nil (n ey1 t ih0 v))
("natives" nil (n ey1 t ih0 v z))
("natividad" nil (n ax t ih0 v ih0 d aa1 d))
("nativism" nil (n ey1 t ih0 v ih1 z ax m))
("nativist" nil (n ey1 t ih0 v ih1 s t))
("nativity" nil (n ax t ih1 v ax t iy0))
("nato" nil (n ey1 t ow0))
("natoli" nil (n aa0 t ow1 l iy0))
("natomas" nil (n ey1 t ow1 m ax z))
("nats" nil (n ae1 t s))
("natter" nil (n ae1 t er0))
("nattie" nil (n ae1 t iy0))
("nattily" nil (n ae1 t ax l iy0))
("natty" nil (n ae1 t iy0))
("natuna" nil (n ax t uw1 n ax))
("natural" nil (n ae1 ch er0 ax l))
("naturalism" nil (n ae1 ch er0 ax l ih1 z ax m))
("naturalist" nil (n ae1 ch er0 ax l ax s t))
("naturalistic" nil (n ae1 ch er0 ax l ih1 s t ih0 k))
("naturalists" nil (n ae1 ch er0 ax l ih0 s t s))
("naturalization" nil (n ae1 ch er0 ax l ax z ey1 sh ax n))
("naturalized" nil (n ae1 ch er0 ax l ay1 z d))
("naturally" nil (n ae1 ch er0 ax l iy0))
("nature" nil (n ey1 ch er0))
("natured" nil (n ey1 ch er0 d))
("naturedly" nil (n ey1 ch er0 d l iy0))
("natwest" nil (n ae1 t w eh1 s t))
("natzke" nil (n ae1 t z k))
("nau" nil (n ow1))
("nauer" nil (n aw1 er0))
("nauert" nil (n aw1 er0 t))
("naugatuck" nil (n ao1 g ax t ah1 k))
("naught" nil (n ao1 t))
("naughton" nil (n ao1 t ax n))
("naughty" nil (n ao1 t iy0))
("naugle" nil (n ao1 g ax l))
("naugles" nil (n ao1 g ax l z))
("nault" nil (n ao1 l t))
("nauman" nil (n aw1 m ax n))
("naumann" nil (n aw1 m ax n))
("naus" nil (n ao1 z))
("nausea" nil (n ao1 z iy0 ax))
("nauseam" nil (n aw1 z iy1 m))
("nauseated" nil (n ao1 z iy0 ey1 t ax d))
("nauseating" nil (n ao1 zh iy0 ey1 t ih0 ng))
("nauss" nil (n ao1 s))
("nauta" nil (n aa0 uw1 t ax))
("nautical" nil (n ao1 t ax k ax l))
("nautilus" nil (n ao1 t ax l ax s))
("nava" nil (n aa1 v ax))
("navajo" nil (n aa1 v ax hh ow1))
("naval" nil (n ey1 v ax l))
("navar" nil (n aa0 v aa1 r))
("navarette" nil (n ae1 v er0 eh1 t))
("navarra" nil (n aa0 v aa1 r ax))
("navarrete" nil (n ae1 v er0 iy1 t))
("navarrette" nil (n ae1 v er0 eh1 t))
("navarro" nil (n ax v aa1 r ow0))
("navas" nil (n aa1 v ax s))
("navcom" nil (n ae1 v k aa1 m))
("nave" nil (n ey1 v))
("navel" nil (n ey1 v ax l))
("navellier" nil (n ax v eh1 l y er0))
("navickas" nil (n aa0 v iy1 k aa0 z))
("navies" nil (n ey1 v iy0 z))
("navigable" nil (n ae1 v ax g ax b ax l))
("navigate" nil (n ae1 v ax g ey1 t))
("navigated" nil (n ae1 v ax g ey1 t ih0 d))
("navigating" nil (n ae1 v ax g ey1 t ih0 ng))
("navigation" nil (n ae1 v ax g ey1 sh ax n))
("navigational" nil (n ae1 v ax g ey1 sh ax n ax l))
("navigator" nil (n ae1 v ax g ey1 t er0))
("navigators" nil (n ae1 v ax g ey1 t er0 z))
("navin" nil (n aa0 v iy1 n))
("navis" nil (n aa1 v ih0 s))
("navistar" nil (n ae1 v ih0 s t aa1 r))
("navratilova" nil (n ae0 v r ae1 t ih0 l ow1 v ax))
("navstar" nil (n ae1 v s t aa1 r))
("navy" nil (n ey1 v iy0))
("nawrocki" nil (n aa0 v r ow1 t s k iy0))
("nawrot" nil (n ao1 r ax t))
("nay" nil (n ey1))
("nayden" nil (n ey1 d ih0 n))
("naylor" nil (n ey1 l er0))
("naysayers" nil (n ey1 s ey1 er0 z))
("nazar" nil (n aa0 z aa1 r))
("nazarian" nil (n ax z eh1 r iy0 ax n))
("nazario" nil (n aa0 z aa1 r iy0 ow0))
("nazer" nil (n ey1 z er0))
("nazi" nil (n aa1 t s iy0))
("nazionale" nil (n aa0 t s iy0 ow0 n aa1 l iy0))
("nazis" nil (n aa1 t s iy0 z))
("nazism" nil (n ae1 z ih0 z ax m))
("nazzaro" nil (n aa0 t s aa1 r ow0))
("ne" nil (n iy1))
("neace" nil (n iy1 s))
("nead" nil (n iy1 d))
("neagle" nil (n iy1 g ax l))
("neal" nil (n iy1 l))
("neala" nil (n iy1 l ax))
("neale" nil (n iy1 l))
("nealey" nil (n iy1 l iy0))
("nealis" nil (n iy1 l ih0 s))
("neall" nil (n iy1 l))
("nealon" nil (n iy1 l ax n))
("nealson" nil (n iy1 l s ax n))
("nealy" nil (n iy1 l iy0))
("neanderthal" nil (n iy0 ae1 n d er0 th ao1 l))
("neanderthals" nil (n iy0 ae1 n d er0 th ao1 l z))
("neapolitan" nil (n iy1 ax p aa1 l ax t ax n))
("near" nil (n ih1 r))
("nearby" nil (n ih1 r b ay1))
("neared" nil (n ih1 r d))
("nearer" nil (n ih1 r er0))
("nearest" nil (n ih1 r ax s t))
("nearhood" nil (n ih1 r hh uh0 d))
("nearing" nil (n ih1 r ih0 ng))
("nearly" nil (n ih1 r l iy0))
("nears" nil (n ih1 r z))
("nearsighted" nil (n iy1 r s ay1 t ih0 d))
("nearsightedness" nil (n iy1 r s ay1 t ih0 d n ax s))
("neary" nil (n ih1 r iy0))
("neas" nil (n iy1 z))
("nease" nil (n iy1 z))
("neat" nil (n iy1 t))
("neater" nil (n iy1 t er0))
("neatest" nil (n iy1 t ax s t))
("neathery" nil (n eh1 th er0 iy0))
("neatly" nil (n iy1 t l iy0))
("neatness" nil (n iy1 t n ax s))
("neault" nil (n ow1))
("neave" nil (n iy1 v))
("neaves" nil (n iy1 v z))
("nebeker" nil (n eh1 b ih0 k er0))
("nebel" nil (n eh1 b ax l))
("nebergall" nil (n iy1 b er0 g ax l))
("neblett" nil (n eh1 b l ih0 t))
("nebraska" nil (n ax b r ae1 s k ax))
("nebraskans" nil (n ax b r ae1 s k ax n z))
("nebula" nil (n eh1 b y ax l ax))
("nebulous" nil (n eh1 b y ax l ax s))
("nec" nil (n eh1 k))
("necaise" nil (n eh1 k ay0 s))
("necci" nil (n eh1 ch iy0))
("necessarilly" nil (n eh1 s ax s eh1 r ax l iy0))
("necessarily" nil (n eh1 s ax s eh1 r ax l iy0))
("necessary" nil (n eh1 s ax s eh1 r iy0))
("necessitate" nil (n ax s eh1 s ax t ey1 t))
("necessitated" nil (n ax s eh1 s ax t ey1 t ax d))
("necessitates" nil (n ax s eh1 s ax t ey1 t s))
("necessitating" nil (n ax s eh1 s ih0 t ey1 t ih0 ng))
("necessities" nil (n ax s eh1 s ih0 t iy0 z))
("necessity" nil (n ax s eh1 s ax t iy0))
("nechayev" nil (n eh1 ch ay1 eh0 v))
("nechyba" nil (n eh1 ch iy1 b ax))
("neck" nil (n eh1 k))
("necked" nil (n eh1 k t))
("necker" nil (n eh1 k er0))
("necklace" nil (n eh1 k l ax s))
("necklaces" nil (n eh1 k l ax s ih0 z))
("necklacing" nil (n eh1 k l ax s ih0 ng))
("necks" nil (n eh1 k s))
("necktie" nil (n eh1 k t ay1))
("neckties" nil (n eh1 k t ay1 z))
("neckwear" nil (n eh1 k w eh1 r))
("neco" nil (n iy1 k ow0))
("necromancy" nil (n eh1 k r ax m ae1 n s iy0))
("necropolis" nil (n ax k r aa1 p ax l ax s))
("necrosis" nil (n ax k r ow1 s ax s))
("nectar" nil (n eh1 k t er0))
("necula" nil (n eh1 k y uw0 l ax))
("ned" nil (n eh1 d))
("neda" nil (n ey1 d ax))
("nedd" nil (n eh1 d))
("nedda" nil (n eh1 d ax))
("neddick" nil (n eh1 d ih0 k))
("neddo" nil (n eh1 d ow0))
("nedeau" nil (n ih0 d ow1))
("nederland" nil (n eh1 d er0 l ax n d))
("nederlanden" nil (n eh1 d er0 l ae1 n d ax n))
("nederlander" nil (n eh1 d er0 l ae1 n d er0))
("nederlandsche" nil (n eh1 d er0 l ae1 n d sh iy0))
("nederlandse" nil (n eh1 d er0 l ae1 n d s iy0))
("nedlloyd" nil (n eh1 d ax l oy1 d))
("nedrow" nil (n eh1 d r ow0))
("nedved" nil (n eh1 d v d))
("nee" nil (n iy1))
("neeb" nil (n iy1 b))
("neece" nil (n iy1 s))
("neeco" nil (n iy1 k ow0))
("need" nil (n iy1 d))
("needed" nil (n iy1 d ax d))
("needelman" nil (n iy1 d ax l m ax n))
("needful" nil (n iy1 d f ax l))
("needham" nil (n iy1 d ax m))
("neediest" nil (n iy1 d iy0 ih0 s t))
("needing" nil (n iy1 d ih0 ng))
("needle" nil (n iy1 d ax l))
("needled" nil (n iy1 d ax l d))
("needlefish" nil (n iy1 d ax l f ih1 sh))
("needlelike" nil (n iy1 d ax l l ay1 k))
("needleman" nil (n iy1 d ax l m ax n))
("needlepoint" nil (n iy1 d ax l p oy1 n t))
("needler" nil (n iy1 d ax l er0))
("needles" nil (n iy1 d ax l z))
("needless" nil (n iy1 d l ax s))
("needlessly" nil (n iy1 d l ax s l iy0))
("needlework" nil (n iy1 d ax l w er1 k))
("needling" nil (n iy1 d l ih0 ng))
("needs" nil (n iy1 d z))
("needy" nil (n iy1 d iy0))
("neef" nil (n iy1 f))
("neel" nil (n iy1 l))
("neeld" nil (n iy1 l d))
("neeley" nil (n iy1 l iy0))
("neels" nil (n iy1 l z))
("neely" nil (n iy1 l iy0))
("neeman" nil (n iy1 m ax n))
("neenan" nil (n iy1 n ax n))
("neeper" nil (n iy1 p er0))
("neer" nil (n ih1 r))
("nees" nil (n iy1 z))
("neese" nil (n iy1 z))
("neeson" nil (n iy1 s ax n))
("neet" nil (n iy1 t))
("nefarious" nil (n ax f eh1 r iy0 ax s))
("neff" nil (n eh1 f))
("neg" nil (n eh1 g))
("negara" nil (n eh0 g aa1 r ax))
("negate" nil (n ih0 g ey1 t))
("negated" nil (n iy1 g ey0 t ih0 d))
("negates" nil (n ih0 g ey1 t s))
("negating" nil (n ih0 g ey1 t ih0 ng))
("negative" nil (n eh1 g ax t ih0 v))
("negatively" nil (n eh1 g ax t ih0 v l iy0))
("negatives" nil (n eh1 g ax t ih0 v z))
("negativism" nil (n eh1 g ax t ih0 v ih1 z ax m))
("negativity" nil (n eh1 g ax t ih1 v ax t iy0))
("negatrons" nil (n eh1 g ax t r aa0 n z))
("negev" nil (n eh1 g eh1 v))
("neglect" nil (n ax g l eh1 k t))
("neglected" nil (n ax g l eh1 k t ax d))
("neglectful" nil (n ih0 g l eh1 k t f ax l))
("neglecting" nil (n ih0 g l eh1 k t ih0 ng))
("neglects" nil (n ih0 g l eh1 k t s))
("negley" nil (n eh1 g l iy0))
("neglia" nil (n eh1 g l iy0 ax))
("negligence" nil (n eh1 g l ax jh ax n s))
("negligent" nil (n eh1 g l ax jh ax n t))
("negligently" nil (n eh1 g l ih0 jh ax n t l iy0))
("negligible" nil (n eh1 g l ax jh ax b ax l))
("negotiable" nil (n ax g ow1 sh ax b ax l))
("negotiate" nil (n ax g ow1 sh iy0 ey1 t))
("negotiated" nil (n ax g ow1 sh iy0 ey1 t ax d))
("negotiates" nil (n ih0 g ow1 sh iy0 ey1 t s))
("negotiating" nil (n ih0 g ow1 sh iy0 ey1 t ih0 ng))
("negotiation" nil (n ih0 g ow1 sh iy0 ey1 sh ax n))
("negotiations" nil (n ax g ow1 sh iy0 ey1 sh ax n z))
("negotiator" nil (n ax g ow1 sh iy0 ey1 t er0))
("negotiators" nil (n ih0 g ow1 sh iy0 ey1 t er0 z))
("negrete" nil (n eh1 g r iy1 t))
("negri" nil (n eh1 g r iy0))
("negrin" nil (n eh1 g r ih0 n))
("negro" nil (n iy1 g r ow0))
("negroes" nil (n iy1 g r ow0 z))
("negroid" nil (n iy1 g r oy0 d))
("negron" nil (n eh1 g r ax n))
("negroni" nil (n eh1 g r ow1 n iy0))
("negros" nil (n iy1 g r ow0 z))
("negs" nil (n eh1 g z))
("negus" nil (n iy1 g ax s))
("nehemiah" nil (n iy1 ax m ay1 ax))
("neher" nil (n ey1 er0))
("nehls" nil (n eh1 l z))
("nehring" nil (n eh1 r ih0 ng))
("nehru" nil (n ey1 r uw1))
("neibauer" nil (n ay1 b aw0 er0))
("neibert" nil (n iy1 b er0 t))
("neice" nil (n iy1 s))
("neider" nil (n iy1 d er0))
("neidhardt" nil (n ay1 d hh aa0 r t))
("neidhart" nil (n ay1 d hh aa0 r t))
("neidigh" nil (n iy1 d ay1))
("neidl" nil (n iy1 d ax l))
("neidlinger" nil (n ay1 d ax l ih0 ng er0))
("neier" nil (n ay1 er0))
("neifert" nil (n iy1 f er0 t))
("neiger" nil (n ay1 g er0))
("neighbor" nil (n ey1 b er0))
("neighborhood" nil (n ey1 b er0 hh uh1 d))
("neighborhoods" nil (n ey1 b er0 hh uh1 d z))
("neighboring" nil (n ey1 b er0 ih0 ng))
("neighborly" nil (n ey1 b er0 l iy0))
("neighbors" nil (n ey1 b er0 z))
("neighmond" nil (n ey1 m ax n d))
("neikirk" nil (n iy1 k er0 k))
("neil" nil (n iy1 l))
("neilan" nil (n iy1 l ax n))
("neild" nil (n iy1 l d))
("neill" nil (n iy1 l))
("neils" nil (n iy1 l z))
("neilsen" nil (n ay1 l s ax n))
("neilson" nil (n iy1 l s ax n))
("neiman" nil (n iy1 m ax n))
("neimeyer" nil (n ay1 m ay0 er0))
("nein" nil (n iy1 n))
("neira" nil (n eh1 r ax))
("neis" nil (n iy1 z))
("neisen" nil (n ay1 s ax n))
("neises" nil (n iy1 z ih0 z))
("neisler" nil (n ay1 s ax l er0))
("neiss" nil (n iy1 s))
("neiswender" nil (n ay1 s w eh0 n d er0))
("neither" nil (n iy1 dh er0))
("neitz" nil (n iy1 t s))
("neitzel" nil (n ay1 t z ax l))
("neitzke" nil (n ay1 t z k))
("nej" nil (n ey1))
("nekoosa" nil (n ih0 k uw1 s ax))
("nekton" nil (n eh1 k t ax n))
("nel" nil (n eh1 l))
("nela" nil (n eh1 l ax))
("nelda" nil (n eh1 l d ax))
("nelia" nil (n eh1 l iy0 ax))
("nelina" nil (n eh0 l iy1 n ax))
("nelita" nil (n eh0 l iy1 t ax))
("nell" nil (n eh1 l))
("nella" nil (n eh1 l ax))
("nellcor" nil (n eh1 l k ao1 r))
("nelle" nil (n eh1 l))
("neller" nil (n eh1 l er0))
("nelles" nil (n eh1 l z))
("nellette" nil (n eh1 l eh1 t))
("nelli" nil (n eh1 l iy0))
("nelliana" nil (n eh1 l iy0 ae1 n ax))
("nellie" nil (n eh1 l iy0))
("nelligan" nil (n eh1 l ih0 g ax n))
("nellis" nil (n eh1 l ih0 s))
("nellwyn" nil (n eh1 l w ih0 n))
("nelly" nil (n eh1 l iy0))
("nelms" nil (n eh1 l m z))
("neloms" nil (n eh1 l ax m z))
("nelon" nil (n eh1 l ax n))
("nels" nil (n eh1 l z))
("nelsen" nil (n eh1 l s ax n))
("nelson" nil (n eh1 l s ax n))
("nematode" nil (n eh1 m ax t ow1 d))
("nematodes" nil (n eh1 m ax t ow1 d z))
("nemean" nil (n iy1 m iy0 ax n))
("nemec" nil (n eh1 m ih0 k))
("nemecek" nil (n eh1 m ih0 s ih0 k))
("nemer" nil (n iy1 m er0))
("nemeroff" nil (n eh1 m er0 ao0 f))
("nemes" nil (n iy1 m z))
("nemesis" nil (n eh1 m ax s ih0 s))
("nemeth" nil (n ey1 m ih0 th))
("nemetz" nil (n eh1 m ih0 t s))
("nemitz" nil (n eh1 m ih0 t s))
("nemmers" nil (n eh1 m er0 z))
("nemo" nil (n eh1 m ow0))
("nemos" nil (n iy1 m ow0 z))
("nemours" nil (n ih0 m ao1 r z))
("nendick" nil (n eh1 n d ih0 k))
("nene" nil (n iy1 n))
("nenninger" nil (n eh1 n ih0 ng er0))
("neo" nil (n iy1 ow0))
("neoax" nil (n iy1 ow0 ae1 k s))
("neoclassic" nil (n iy1 ow0 k l ae1 s ih0 k))
("neoclassical" nil (n iy1 ow0 k l ae1 s ih0 k ax l))
("neoconservative" nil (n iy1 ow0 k ax n s er1 v ax t ih0 v))
("neoconservatives" nil (n iy1 ow0 k ax n s er1 v ax t ih0 v z))
("neola" nil (n iy0 aa1 l ax))
("neoliberal" nil (n iy1 ow0 l ih1 b er0 ax l))
("neoliberals" nil (n iy1 ow0 l ih1 b er0 ax l z))
("neoma" nil (n ey0 ow1 m ax))
("neon" nil (n iy1 aa0 n))
("neonatal" nil (n iy1 ow0 n ey1 t ax l))
("neons" nil (n iy1 aa1 n z))
("neophyte" nil (n iy1 ax f ay1 t))
("neophytes" nil (n iy1 ax f ay1 t s))
("neoplasm" nil (n iy1 ax p l ae1 z ax m))
("neoplatonic" nil (n iy1 ow0 p l ax t aa1 n ih0 k))
("neoplatonist" nil (n iy1 ow0 p l ey1 t ax n ax s t))
("neoprene" nil (n iy1 ax p r iy1 n))
("neoprobe" nil (n iy1 ow0 p r ow1 b))
("neorx" nil (n iy1 ow0 r eh1 k s))
("neotenic" nil (n iy1 ax t iy1 n ih0 k))
("neoteny" nil (n iy0 aa1 t ax n iy0))
("nepa" nil (n iy1 p ax))
("nepal" nil (n ax p ao1 l))
("nepalese" nil (n eh1 p ax l iy1 z))
("nepali" nil (n ax p ao1 l iy0))
("nephew" nil (n eh1 f y uw0))
("nephews" nil (n eh1 f y uw0 z))
("nephridium" nil (n ax f r ih1 d iy0 ax m))
("nephrite" nil (n eh1 f r ay0 t))
("nephron" nil (n eh1 f r aa0 n))
("nephrosis" nil (n ax f r ow1 s ax s))
("nepl" nil (n eh1 p ax l))
("neponset" nil (n ax p aa1 n s ih0 t))
("nepool" nil (n ax p uw1 l))
("nepotism" nil (n eh1 p ax t ih1 z ax m))
("neptune" nil (n eh1 p t uw0 n))
("neptunium" nil (n eh0 p t uw1 n iy0 ax m))
("nerco" nil (n er1 k ow0))
("nerd" nil (n er1 d))
("nerds" nil (n er1 d z))
("nerdy" nil (n er1 d iy0))
("nerenberg" nil (n ih1 r ax n b er0 g))
("neri" nil (n eh1 r iy0))
("nerice" nil (n eh1 r ih0 s))
("nerine" nil (n eh1 r iy0 n))
("nerio" nil (n eh1 r iy0 ow0))
("neritic" nil (n er0 ih1 t ih0 k))
("nerlich" nil (n er1 l ih0 k))
("nerney" nil (n er1 n iy0))
("nero" nil (n ih1 r ow0))
("nerone" nil (n eh1 r ax n))
("nerren" nil (n eh1 r ax n))
("nervana" nil (n er0 v ae1 n ax))
("nerve" nil (n er1 v))
("nerves" nil (n er1 v z))
("nervosa" nil (n er0 v ow1 s ax))
("nervous" nil (n er1 v ax s))
("nervously" nil (n er1 v ax s l iy0))
("nervousness" nil (n er1 v ax s n ax s))
("nervy" nil (n er1 v iy0))
("nes" nil (n eh1 s))
("nesbit" nil (n eh1 s b ih0 t))
("nesbitt" nil (n eh1 z b ih0 t))
("nesby" nil (n eh1 s b iy0))
("nesci" nil (n eh1 s iy0))
("nesheim" nil (n eh1 s hh ay1 m))
("nesi" nil (n eh1 s iy0))
("nesler" nil (n eh1 s ax l er0))
("nesmith" nil (n eh1 z m ih0 th))
("ness" nil (n eh1 s))
("nessa" nil (n eh1 s ax))
("nessel" nil (n eh1 s ax l))
("nessen" nil (n iy1 s ax n))
("nesser" nil (n eh1 s er0))
("nesseth" nil (n eh1 s ih0 th))
("nessi" nil (n eh1 s iy0))
("nessie" nil (n eh1 s iy0))
("nessler" nil (n eh1 s l er0))
("nest" nil (n eh1 s t))
("nesta" nil (n eh1 s t ax))
("neste" nil (n eh1 s t))
("nestea" nil (n eh1 s t iy0 ax))
("nested" nil (n eh1 s t ax d))
("nester" nil (n eh1 s t er0))
("nesters" nil (n eh1 s t er0 z))
("nesting" nil (n eh1 s t ih0 ng))
("nestle" nil (n eh1 s ax l))
("nestled" nil (n eh1 s ax l d))
("nestler" nil (n eh1 s ax l er0))
("nestles" nil (n eh1 s ax l z))
("nestling" nil (n eh1 s t l ih0 ng))
("nestlings" nil (n eh1 s t l ih0 ng z))
("nestor" nil (n eh1 s t er0))
("nestorian" nil (n eh0 s t ao1 r iy0 ax n))
("nestorianism" nil (n eh0 s t ao1 r iy0 ax n ih0 z ax m))
("nests" nil (n eh1 s t s))
("net" nil (n eh1 t))
("netanyahu" nil (n eh1 t ax n y aa1 hh uw1))
("netback" nil (n eh1 t b ae1 k))
("netcom" nil (n eh1 t k aa1 m))
("neth" nil (n eh1 th))
("nether" nil (n eh1 dh er0))
("nethercutt" nil (n eh1 th er0 k ax t))
("netherland" nil (n eh1 dh er0 l ax n d))
("netherlandic" nil (n eh1 dh er0 l ae1 n d ih0 k))
("netherlands" nil (n eh1 dh er0 l ax n d z))
("nethers" nil (n eh1 dh er0 z))
("netherton" nil (n eh1 dh er0 t ax n))
("netherworld" nil (n eh1 dh er0 w er1 l d))
("nethery" nil (n eh1 dh er0 iy0))
("nethuva" nil (n eh1 th uw1 v ax))
("netlike" nil (n eh1 t l ay1 k))
("neto" nil (n eh1 t ow0))
("nets" nil (n eh1 t s))
("netscape" nil (n eh1 t s k ey1 p))
("nett" nil (n eh1 t))
("netta" nil (n eh1 t ax))
("netted" nil (n eh1 t ih0 d))
("netter" nil (n eh1 t er0))
("netterville" nil (n eh1 t er0 v ih0 l))
("nettesheim" nil (n eh1 t ih0 s sh ay0 m))
("nettie" nil (n eh1 t iy0))
("netting" nil (n eh1 t ih0 ng))
("nettle" nil (n eh1 t ax l))
("nettled" nil (n eh1 t ax l d))
("nettles" nil (n eh1 t ax l z))
("nettlesome" nil (n eh1 t ax l s ax m))
("nettleton" nil (n eh1 t ax l t ax n))
("netto" nil (n eh1 t ow0))
("netty" nil (n eh1 t iy0))
("netview" nil (n eh1 t v y uw1))
("netware" nil (n eh1 t w eh1 r))
("network" nil (n eh1 t w er1 k))
("networked" nil (n eh1 t w er1 k t))
("networking" nil (n eh1 t w er1 k ih0 ng))
("networks" nil (n eh1 t w er1 k s))
("networth" nil (n eh1 t w er1 th))
("netz" nil (n eh1 t s))
("netzel" nil (n eh1 t z ax l))
("netzer" nil (n eh1 t z er0))
("netzley" nil (n eh1 t s l iy0))
("neu" nil (n oy1))
("neubauer" nil (n uw1 b aw0 er0))
("neubecker" nil (n uw1 b eh1 k er0))
("neuber" nil (n uw1 b er0))
("neuberger" nil (n uw1 b er0 g er0))
("neubert" nil (n uw1 b er0 t))
("neuburg" nil (n uw1 b er0 g))
("neuburger" nil (n uw1 b er0 g er0))
("neudecker" nil (n uw1 d ih0 k er0))
("neue" nil (n uw1 iy0))
("neuendorf" nil (n uw1 ax n d ao0 r f))
("neuendorffer" nil (n uw1 n d ao0 r f er0))
("neuenfeldt" nil (n uw1 ax n f eh0 l t))
("neuenschwander" nil (n uw1 ax n sh w ao0 n d er0))
("neuer" nil (n oy1 er0))
("neufeld" nil (n uw1 f eh1 l d))
("neugebauer" nil (n uw1 g ax b aw0 er0))
("neugent" nil (n uw1 jh ax n t))
("neuhart" nil (n uw1 hh aa1 r t))
("neuharth" nil (n uw1 hh aa1 r th))
("neuhaus" nil (n uw1 hh aw1 s))
("neuhauser" nil (n uw1 hh aw1 z er0))
("neuhoff" nil (n uw1 hh ao1 f))
("neukam" nil (n uw1 k ax m))
("neukirchen" nil (n uw1 k er0 ch ax n))
("neukom" nil (n uw1 k aa0 m))
("neumaier" nil (n uw1 m ay0 er0))
("neuman" nil (n uw1 m ax n))
("neumann" nil (n uw1 m ax n))
("neumayer" nil (n uw1 m ey1 er0))
("neumeier" nil (n uw1 m ay0 er0))
("neumeister" nil (n uw1 m ay1 s t er0))
("neumeyer" nil (n uw1 m ay0 er0))
("neumiller" nil (n uw1 m ax l er0))
("neuner" nil (n uw1 n er0))
("neupert" nil (n uw1 p er0 t))
("neupogen" nil (n uw1 p ow0 jh eh1 n))
("neural" nil (n uh1 r ax l))
("neurofibromatosis" nil (n uh1 r ow0 f ay0 b r ow1 m ax t ow1 s ih0 s))
("neurohr" nil (n uh1 r ao0 r))
("neurologic" nil (n uh1 r ax l aa1 jh ih0 k))
("neurological" nil (n uh1 r ax l aa1 jh ih0 k ax l))
("neurologist" nil (n uh0 r aa1 l ax jh ax s t))
("neurology" nil (n uh0 r aa1 l ax jh iy0))
("neurons" nil (n uh1 r aa0 n z))
("neuropathy" nil (n uh1 r ow0 p ae1 th iy0))
("neuroscience" nil (n y uh1 r ow0 s ay1 ax n s))
("neuroses" nil (n uh0 r ow1 s iy0 z))
("neurosis" nil (n uh0 r ow1 s ax s))
("neurosurgeon" nil (n uh1 r ow0 s er1 jh ax n))
("neurosurgeons" nil (n uh1 r ow0 s er1 jh ax n z))
("neuroth" nil (n uh1 r ao0 th))
("neurotic" nil (n uh0 r aa1 t ih0 k))
("neuser" nil (n uw1 s er0))
("neustadt" nil (n uw1 sh t ae0 t))
("neuter" nil (n uw1 t er0))
("neutered" nil (n uw1 t er0 d))
("neutering" nil (n uw1 t er0 ih0 ng))
("neutral" nil (n uw1 t r ax l))
("neutralism" nil (n uw1 t r ax l ih1 z ax m))
("neutralist" nil (n uw1 t r ax l ax s t))
("neutrality" nil (n uw0 t r ae1 l ax t iy0))
("neutralization" nil (n uw1 t r ax l ax z ey1 sh ax n))
("neutralize" nil (n uw1 t r ax l ay1 z))
("neutralized" nil (n uw1 t r ax l ay1 z d))
("neutralizes" nil (n uw1 t r ax l ay1 z ih0 z))
("neutralizing" nil (n uw1 t r ax l ay1 z ih0 ng))
("neutrino" nil (n uw0 t r iy1 n ow0))
("neutrinos" nil (n uw0 t r iy1 n ow0 z))
("neutrogena" nil (n uw1 t r ax jh iy1 n ax))
("neutron" nil (n uw1 t r aa1 n))
("neutrons" nil (n uw1 t r aa1 n z))
("neuville" nil (n uw1 v ih1 l))
("neuwirth" nil (n uw1 w er0 th))
("neuzil" nil (n uw1 z ax l))
("neva" nil (n ey1 v ax))
("nevada" nil (n ax v aa1 d ax))
("nevala" nil (n ey0 v aa1 l ax))
("nevarez" nil (n ey0 v aa1 r eh0 z))
("neve" nil (n iy1 v))
("neveau" nil (n ih0 v ow1))
("nevel" nil (n ey0 v eh1 l))
("nevels" nil (n eh1 v ax l z))
("never" nil (n eh1 v er0))
("nevers" nil (n eh1 v er0 z))
("nevertheless" nil (n eh1 v er0 dh ax l eh1 s))
("neves" nil (n iy1 v z))
("neveu" nil (n ih0 v uw1))
("nevil" nil (n ey0 v iy1 l))
("nevile" nil (n ey0 v ay1 l))
("nevill" nil (n eh1 v ih0 l))
("neville" nil (n eh1 v ih0 l))
("nevills" nil (n eh1 v ih0 l z))
("nevils" nil (n eh1 v ax l z))
("nevin" nil (n eh1 v ih0 n))
("nevins" nil (n eh1 v ih0 n z))
("nevis" nil (n eh1 v ih0 s))
("nevitt" nil (n eh1 v ih0 t))
("nevius" nil (n iy1 v iy0 ih0 s))
("new" nil (n uw1))
("newall" nil (n uw1 ao1 l))
("newark" nil (n uw1 er0 k))
("newbauer" nil (n uw1 b aw0 er0))
("newberg" nil (n uw1 b er0 g))
("newberger" nil (n uw1 b er0 g er0))
("newbern" nil (n uw1 b er0 n))
("newberry" nil (n uw1 b eh1 r iy0))
("newbery" nil (n uw1 b eh1 r iy0))
("newbill" nil (n uw1 b ih1 l))
("newbold" nil (n uw1 b ow1 l d))
("newborn" nil (n uw1 b ao0 r n))
("newborns" nil (n uw1 b ao1 r n z))
("newbridge" nil (n uw1 b r ih1 jh))
("newbrough" nil (n uw1 b r aw0))
("newburg" nil (n uw1 b er0 g))
("newburger" nil (n uw1 b er0 g er0))
("newburn" nil (n uw1 b er1 n))
("newbury" nil (n uw1 b eh1 r iy0))
("newby" nil (n uw1 b iy0))
("newcastle" nil (n uw1 k ae1 s ax l))
("newcom" nil (n uw1 k ax m))
("newcomb" nil (n uw1 k ax m))
("newcombe" nil (n uw1 k ax m))
("newcome" nil (n uw1 k ax m))
("newcomer" nil (n uw1 k ah1 m er0))
("newcomers" nil (n uw1 k ah1 m er0 z))
("newcor" nil (n uw1 k ao1 r))
("newedge" nil (n uw1 ax jh))
("newell" nil (n uw1 ax l))
("newer" nil (n uw1 er0))
("newest" nil (n uw1 ax s t))
("newey" nil (n uw1 iy0))
("newfangled" nil (n uw1 f ae1 ng g ax l d))
("newfield" nil (n uw1 f iy1 l d))
("newfound" nil (n uw1 f aw1 n d))
("newfoundland" nil (n uw1 f ax n d l ax n d))
("newgard" nil (n uw1 g er0 d))
("newgate" nil (n uw1 g ey0 t))
("newgateway" nil (n uw1 g ey1 t w ey1))
("newhall" nil (n uw1 hh ao1 l))
("newhard" nil (n uw1 hh aa1 r d))
("newhart" nil (n uw1 hh aa1 r t))
("newhouse" nil (n uw1 hh aw1 s))
("newill" nil (n iy0 w ih1 l))
("newingham" nil (n uw1 ih0 ng hh ae1 m))
("newish" nil (n uw1 ih0 sh))
("newkirk" nil (n uw1 k er0 k))
("newlan" nil (n uw1 l ax n))
("newland" nil (n uw1 l ax n d))
("newlin" nil (n uw1 l ih0 n))
("newlon" nil (n uw1 l ax n))
("newlun" nil (n uw1 l ax n))
("newly" nil (n uw1 l iy0))
("newlyn" nil (n uw1 l ih0 n))
("newlywed" nil (n uw1 l iy0 w eh1 d))
("newlyweds" nil (n uw1 l iy0 w eh1 d z))
("newman" nil (n uw1 m ax n))
("newmann" nil (n uw1 m ax n))
("newmark" nil (n uw1 m aa1 r k))
("newmarket" nil (n uw1 m aa1 r k ax t))
("newmeyer" nil (n uw1 m ay0 er0))
("newmont" nil (n uw1 m aa1 n t))
("newmyer" nil (n uw1 m iy0 er0))
("newnam" nil (n uw1 n ax m))
("newness" nil (n uw1 n ax s))
("neworld" nil (n eh1 w er1 l d))
("newport" nil (n uw1 p ao0 r t))
("newquist" nil (n uw1 k w ih1 s t))
("news" nil (n uw1 z))
("newscast" nil (n uw1 z k ae1 s t))
("newscaster" nil (n uw1 z k ae1 s t er0))
("newscasters" nil (n uw1 z k ae1 s t er0 z))
("newscasts" nil (n uw1 z k ae1 s t s))
("newscorp" nil (n uw1 z k ao1 r p))
("newsday" nil (n uw1 z d ey1))
("newsgroups" nil (n uw1 z g r uw1 p s))
("newsham" nil (n uw1 sh ax m))
("newshour" nil (n uw1 z aw1 r))
("newsletter" nil (n uw1 z l eh1 t er0))
("newsletters" nil (n uw1 z l eh1 t er0 z))
("newsmagazine" nil (n uw1 z m ae1 g ax z iy1 n))
("newsmaker" nil (n uw1 z m ey1 k er0))
("newsmakers" nil (n uw1 z m ey1 k er0 z))
("newsman" nil (n uw1 z m ae1 n))
("newsmen" nil (n uw1 z m ih0 n))
("newsom" nil (n uw1 z ax m))
("newsome" nil (n uw1 s ax m))
("newson" nil (n uw1 s ax n))
("newspaper" nil (n uw1 z p ey1 p er0))
("newspaperman" nil (n uw1 z p ey1 p er0 m ae1 n))
("newspapermen" nil (n uw1 z p ae1 p er0 m ax n))
("newspapers" nil (n uw1 z p ey1 p er0 z))
("newspeak" nil (n uw1 s p iy1 k))
("newsprint" nil (n uw1 z p r ih1 n t))
("newsreel" nil (n uw1 z r iy1 l))
("newsreels" nil (n uw1 z r iy1 l z))
("newsroom" nil (n uw1 z r uw1 m))
("newsrooms" nil (n uw1 z r uw1 m z))
("newsstand" nil (n uw1 z s t ae1 n d))
("newsstands" nil (n uw1 z s t ae1 n d z))
("newstrom" nil (n uw1 z t r ax m))
("newsum" nil (n uw1 z ax m))
("newswanger" nil (n uw1 z w ao0 ng er0))
("newsweek" nil (n uw1 z w iy1 k))
("newsweekly" nil (n uw1 z w iy1 k l iy0))
("newswire" nil (n uw1 z w ay1 r))
("newswires" nil (n uw1 z w ay1 r z))
("newsworthy" nil (n uw1 z w er1 dh iy0))
("newsy" nil (n uw1 z iy0))
("newt" nil (n uw1 t))
("newton" nil (n uw1 t ax n))
("newtonchik" nil (n uw1 t aa1 n ch ih0 k))
("newtonian" nil (n uw0 t ow1 n iy0 ax n))
("newtown" nil (n uw1 t aw1 n))
("newtowne" nil (n uw1 t aw1 n))
("newts" nil (n uw1 t s))
("newvector" nil (n uw1 v eh1 k t er0))
("newville" nil (n uw1 v ih1 l))
("newwave" nil (n uw1 w ey1 v))
("nexgen" nil (n eh1 k s jh eh1 n))
("nexis" nil (n eh1 k s ih0 s))
("next" nil (n eh1 k s t))
("nextel" nil (n eh1 k s t eh1 l))
("nextstep" nil (n eh1 k s t s t eh1 p))
("nexus" nil (n eh1 k s ax s))
("ney" nil (n ey1))
("neyens" nil (n ay1 n z))
("neyer" nil (n ey1 er0))
("neyhart" nil (n ey1 hh aa1 r t))
("neyland" nil (n ey1 l ax n d))
("neylon" nil (n ey1 l ax n))
("neyman" nil (n ey1 m ax n))
("neysa" nil (n ey1 s ax))
("nez" nil (n eh1 z))
("ng" nil (eh1 ng))
("ngai" nil (g ay1))
("ngema" nil (eh0 n jh eh1 m aa0))
("nghi" nil (g iy1))
("ngo" nil (g ow1))
("nguyen" nil (n uw0 y eh1 n))
("niacin" nil (n ay1 ax s ax n))
("niagara" nil (n ay0 ae1 g r ax))
("niaid" nil (n ay1 ey1 d))
("nial" nil (n ay1 ax l))
("niall" nil (n ay1 l))
("nib" nil (n ih1 b))
("nibbe" nil (n ih1 b))
("nibble" nil (n ih1 b ax l))
("nibbled" nil (n ih1 b ax l d))
("nibbles" nil (n ih1 b ax l z))
("nibbling" nil (n ih1 b ax l ih0 ng))
("nibert" nil (n ih1 b er0 t))
("niblack" nil (n ih1 b l ae1 k))
("niblett" nil (n ih1 b l ih0 t))
("niblock" nil (n ih1 b l aa1 k))
("nibs" nil (n ih1 b z))
("nicandros" nil (n ih0 k ae1 n d r ow0 s))
("nicaragua" nil (n ih1 k er0 aa1 g w ax))
("nicaraguan" nil (n ih1 k er0 aa1 g w ax n))
("nicaraguans" nil (n ih1 k er0 aa1 g w ax n z))
("nicastro" nil (n ih0 k ae1 s t r ow0))
("niccoli" nil (n iy0 k ow1 l iy0))
("niccolite" nil (n ih1 k ax l ay1 t))
("niccum" nil (n ih1 k ax m))
("nice" nil (n ay1 s))
("nicely" nil (n ay1 s l iy0))
("niceness" nil (n ay1 s n ax s))
("nicer" nil (n ay1 s er0))
("nicest" nil (n ay1 s ih0 s t))
("niceties" nil (n ay1 s ih0 t iy0 z))
("niche" nil (n ih1 ch))
("nichelson" nil (n ih1 ch ih0 l s ax n))
("niches" nil (n ih1 ch ih0 z))
("nichol" nil (n ih1 k ao0 l))
("nichola" nil (n ih0 hh ow1 l ax))
("nicholas" nil (n ih1 k ax l ax s))
("nicholes" nil (n ih1 k hh ow0 l z))
("nicholi" nil (n ih1 k ax l ay0))
("nicholl" nil (n ih1 k ax l))
("nicholls" nil (n ih1 k ax l z))
("nichols" nil (n ih1 k ax l z))
("nicholson" nil (n ih1 k ax l s ax n))
("nichter" nil (n ih1 k t er0))
("nick" nil (n ih1 k))
("nicked" nil (n ih1 k t))
("nickel" nil (n ih1 k ax l))
("nickell" nil (n ih1 k ax l))
("nickelodeon" nil (n ih1 k ih0 l ow1 d iy0 ax n))
("nickels" nil (n ih1 k ax l z))
("nickelson" nil (n ih1 k ih0 l s ax n))
("nickens" nil (n ih1 k ax n z))
("nickerson" nil (n ih1 k er0 s ax n))
("nickeson" nil (n ih1 k ih0 s ax n))
("nickey" nil (n ih1 k iy0))
("nicki" nil (n ih1 k iy0))
("nicklas" nil (n ih1 k l ax z))
("nicklaus" nil (n ih1 k l ax s))
("nickle" nil (n ih1 k ax l))
("nickleby" nil (n ih1 k ax l b iy0))
("nickles" nil (n ih1 k ax l z))
("nickless" nil (n ih1 k l ax s))
("nicklin" nil (n ih1 k l ih0 n))
("nicklow" nil (n ih1 k l ow1))
("nickname" nil (n ih1 k n ey1 m))
("nicknamed" nil (n ih1 k n ey1 m d))
("nicknames" nil (n ih1 k n ey1 m z))
("nickol" nil (n ih1 k ax l))
("nickolas" nil (n ih1 k ax l ax z))
("nickoloff" nil (n ih1 k ax l ao0 f))
("nickols" nil (n ih1 k ax l z))
("nickolson" nil (n ih1 k ow0 l s ax n))
("nicks" nil (n ih1 k s))
("nickson" nil (n ih1 k s ax n))
("nickum" nil (n ih1 k ax m))
("nicky" nil (n ih1 k iy0))
("niclaneshia" nil (n ih1 k l ax n eh1 sh ax))
("nicley" nil (n ih1 k l iy0))
("nico" nil (n iy1 k ow0))
("nicodemo" nil (n iy1 k ow0 d ey1 m ow0))
("nicoderm" nil (n ih1 k ax d er1 m))
("nicol" nil (n ih1 k ax l))
("nicola" nil (n ih0 k ow1 l ax))
("nicolae" nil (n ih1 k ow0 l ay1))
("nicolai" nil (n iy0 k ow0 l aa1 iy0))
("nicolaides" nil (n ih1 k ax l ey0 d z))
("nicolaisen" nil (n ih1 k ax l ay0 s ax n))
("nicolas" nil (n ih1 k ax l ax s))
("nicolau" nil (n ih1 k ax l aw0))
("nicolaus" nil (n ih1 k ax l aw0 z))
("nicolay" nil (n ih1 k ax l ey1))
("nicole" nil (n ih0 k ow1 l))
("nicolet" nil (n ih1 k ax l eh1 t))
("nicoletta" nil (n ih1 k ax l eh1 t ax))
("nicolette" nil (n ih1 k ax l eh1 t))
("nicoletti" nil (n iy0 k ow0 l eh1 t iy0))
("nicoli" nil (n iy0 k ow1 l iy0))
("nicolin" nil (n ih1 k ax l ih0 n))
("nicolina" nil (n iy1 k ow0 l iy1 n ax))
("nicoline" nil (n iy0 k ow0 l iy1 n iy0))
("nicolini" nil (n iy1 k ow0 l iy1 n iy0))
("nicoll" nil (n ih1 k ax l))
("nicolle" nil (n ih0 k ow1 l))
("nicolls" nil (n ih1 k ax l z))
("nicolo" nil (n ih1 k ax l ow0))
("nicoloff" nil (n ih1 k ax l ao0 f))
("nicolosi" nil (n iy0 k ow0 l ow1 s iy0))
("nicols" nil (n ih1 k ax l z))
("nicolson" nil (n ih1 k ow0 l s ax n))
("nicor" nil (n ay1 k ao1 r))
("nicorette" nil (n ih1 k er0 eh1 t))
("nicosia" nil (n ih0 k ax s iy1 ax))
("nicoski" nil (n ih0 k ao1 s k iy0))
("nicoson" nil (n ih1 k ax s ax n))
("nicotera" nil (n iy1 k ow0 t eh1 r ax))
("nicotine" nil (n ih1 k ax t iy1 n))
("nicotra" nil (n ih0 k aa1 t r ax))
("nida" nil (n iy1 d ax))
("nidal" nil (n ih0 d aa1 l))
("niday" nil (n ay1 d ey1))
("nidiffer" nil (n ih1 d ih0 f er0))
("nie" nil (n iy1))
("niebauer" nil (n iy1 b aw0 er0))
("niebling" nil (n iy1 b ax l ih0 ng))
("niebuhr" nil (n iy1 b uh0 r))
("niebur" nil (n iy1 b er0))
("niece" nil (n iy1 s))
("nieces" nil (n iy1 s ih0 z))
("nied" nil (n iy1 d))
("niedbala" nil (n iy0 d b aa1 l ax))
("niedbalski" nil (n iy0 d b aa1 l s k iy0))
("nieder" nil (n iy1 d er0))
("niederer" nil (n iy1 d er0 er0))
("niederhauser" nil (n iy1 d er0 hh aw0 z er0))
("niederman" nil (n ay1 d er0 m ax n))
("niedermeier" nil (n iy1 d er0 m ay0 er0))
("niedermeyer" nil (n iy1 d er0 m ay0 er0))
("niedzielski" nil (n iy0 jh iy1 l s k iy0))
("niedzwiecki" nil (n iy0 jh v iy1 t s k iy0))
("niehaus" nil (n iy1 hh aw1 s))
("niehoff" nil (n iy1 hh ao0 f))
("niekamp" nil (n iy1 k ae1 m p))
("nieland" nil (n iy1 l ax n d))
("nield" nil (n iy1 l d))
("nields" nil (n iy1 l d z))
("niels" nil (n iy1 l z))
("nielsen" nil (n iy1 l s ax n))
("nielson" nil (n iy1 l s ax n))
("nieman" nil (n iy1 m ax n))
("niemann" nil (n iy1 m ax n))
("niemczyk" nil (n iy1 m ch ih0 k))
("niemeier" nil (n iy1 m ay0 er0))
("niemela" nil (n iy0 m eh1 l ax))
("niemeyer" nil (n iy1 m ay0 er0))
("niemi" nil (n iy1 m iy0))
("niemiec" nil (n iy1 m iy1 k))
("niemuth" nil (n iy1 m ax th))
("nienaber" nil (n iy1 n ax b er0))
("nienhaus" nil (n iy1 n hh aw1 s))
("nienhuis" nil (n iy1 n hh uw0 ih0 z))
("nienow" nil (n iy1 n ow0))
("nier" nil (n iy1 er0))
("nierenberg" nil (n ih1 r ax n b er0 g))
("nierman" nil (n ih1 r m ax n))
("nies" nil (n ay1 z))
("niese" nil (n iy1 z))
("niesen" nil (n iy1 s ax n))
("niess" nil (n iy1 s))
("nieto" nil (n iy1 t ow0))
("nietzsche" nil (n iy1 ch iy0))
("nieves" nil (n iy0 eh1 v eh0 s))
("niezgoda" nil (n iy0 z g ow1 d ax))
("nifong" nil (n ih1 f ao0 ng))
("nifty" nil (n ih1 f t iy0))
("nigel" nil (n ay1 jh ax l))
("niger" nil (n ay1 jh er0))
("nigeria" nil (n ay0 jh ih1 r iy0 ax))
("nigerian" nil (n ay0 jh ih1 r iy0 ax n))
("nigerians" nil (n ay0 jh ih1 r iy0 ax n z))
("nigg" nil (n ih1 g))
("niggardliness" nil (n ih1 g er0 d l iy0 n ax s))
("niggardly" nil (n ih1 g er0 d l iy0))
("nigger" nil (n ih1 g er0))
("niggers" nil (n ih1 g er0 z))
("nigh" nil (n ay1))
("nighswonger" nil (n ay1 s w ao0 ng er0))
("night" nil (n ay1 t))
("nightclub" nil (n ay1 t k l ah1 b))
("nightclubs" nil (n ay1 t k l ah1 b z))
("nightengale" nil (n ay1 t ih0 ng g ax l))
("nighters" nil (n ay1 t er0 z))
("nightfall" nil (n ay1 t f ao1 l))
("nighthawk" nil (n ay1 t hh ao1 k))
("nightingale" nil (n ay1 t ih0 ng g ey0 l))
("nightlife" nil (n ay1 t l ay1 f))
("nightline" nil (n ay1 t l ay1 n))
("nightlines" nil (n ay1 t l ay1 n z))
("nightly" nil (n ay1 t l iy0))
("nightmare" nil (n ay1 t m eh1 r))
("nightmares" nil (n ay1 t m eh1 r z))
("nightmarish" nil (n ay1 t m eh1 r ih0 sh))
("nights" nil (n ay1 t s))
("nightshade" nil (n ay1 ch ey1 d))
("nightshirt" nil (n ay1 ch er1 t))
("nightstage" nil (n ay1 t s t ey1 jh))
("nighttime" nil (n ay1 t t ay1 m))
("nighttimes" nil (n ay1 t t ay1 m z))
("nigrelli" nil (n iy0 g r eh1 l iy0))
("nigris" nil (n ih1 g r ih0 s))
("nigro" nil (n ih1 g r ow0))
("niguel" nil (n ih0 g eh1 l))
("nihart" nil (n ih1 hh aa0 r t))
("nihei" nil (n ih0 hh ey1))
("nihilism" nil (n ay1 ax l ih1 z ax m))
("nihilists" nil (n ay1 ax l ax s t s))
("nihiser" nil (n ih1 hh ay0 z er0))
("nihon" nil (n ih1 hh aa0 n))
("niihau" nil (n iy1 hh aw0))
("nik" nil (n ih1 k))
("nike" nil (n ay1 k iy0))
("nikes" nil (n ay1 k s))
("nikita" nil (n ih1 k iy1 t ax))
("nikkei" nil (n ih0 k ey1))
("nikkel" nil (n ih1 k ax l))
("nikkhah" nil (n ih1 k hh aa0))
("nikki" nil (n ih1 k iy0))
("nikko" nil (n iy1 k ow0))
("nikolai" nil (n ih1 k ow0 l ay1))
("nikolaus" nil (n ih1 k ax l aw0 z))
("nikolic" nil (n ih0 k aa1 l ih0 k))
("nikolich" nil (n ih0 k aa1 l ih0 hh))
("nikon" nil (n ay1 k aa1 n))
("nil" nil (n ih1 l))
("nila" nil (n iy1 l ax))
("nilan" nil (n iy0 l aa1 n))
("niland" nil (n ay1 l ax n d))
("nile" nil (n ay1 l))
("niles" nil (n ay1 l z))
("nilges" nil (n ih1 l jh ih0 z))
("nill" nil (n ih1 l))
("nilles" nil (n ay1 l z))
("nilly" nil (n ih1 l iy0))
("nils" nil (n ih1 l z))
("nilsen" nil (n ih1 l s ax n))
("nilson" nil (n ih1 l s ax n))
("nilsson" nil (n ih1 l s ax n))
("nimble" nil (n ih1 m b ax l))
("nimbly" nil (n ih1 m b l iy0))
("nimbus" nil (n ih1 m b ax s))
("nimby" nil (n ih1 m b iy0))
("nimitz" nil (n ih1 m ih0 t s))
("nimmer" nil (n ih1 m er0))
("nimmo" nil (n ih1 m ow0))
("nimmons" nil (n ih1 m ax n z))
("nimrod" nil (n ih1 m r aa0 d))
("nimrodi" nil (n ih0 m r ow1 d iy0))
("nims" nil (n ih1 m z))
("nimtz" nil (n ih1 m t s))
("nina" nil (n ay1 n ax))
("ninagawa" nil (n iy1 n aa0 g aa1 w ax))
("nincompoop" nil (n ih1 ng k ax m p uw1 p))
("nincompoops" nil (n ih1 ng k ax m p uw1 p s))
("nine" nil (n ay1 n))
("ninefold" nil (n ih1 n f ow1 l d))
("niner" nil (n ay1 n er0))
("niners" nil (n ay1 n er0 z))
("nines" nil (n ay1 n z))
("nineteen" nil (n ay1 n t iy1 n))
("nineteenth" nil (n ay1 n t iy1 n th))
("nineties" nil (n ay1 n t iy0 z))
("ninetieth" nil (n ay1 n t iy0 ih0 th))
("ninette" nil (n ih0 n eh1 t))
("ninety" nil (n ay1 n t iy0))
("nineveh" nil (n ih1 n ax v ax))
("ning" nil (n ih1 ng))
("ninja" nil (n ih1 n jh ax))
("ninneman" nil (n ih1 n m ax n))
("nino" nil (n iy1 n ow0))
("ninon" nil (n ih1 n ax n))
("nintendo" nil (n ih0 n t eh1 n d ow0))
("ninth" nil (n ay1 n th))
("ninths" nil (n ay1 n th s))
("niobite" nil (n ay1 ow0 b ay1 t))
("niobium" nil (n ay1 ow1 b iy0 ax m))
("nip" nil (n ih1 p))
("nipon" nil (n ih1 p aa1 n))
("nipp" nil (n ih1 p))
("nipped" nil (n ih1 p t))
("nipper" nil (n ih1 p er0))
("nippert" nil (n ih1 p er0 t))
("nipping" nil (n ih1 p ih0 ng))
("nipple" nil (n ih1 p ax l))
("nippon" nil (n ih1 p aa1 n))
("nippondenso" nil (n ih1 p aa1 n d eh1 n s ow0))
("nippy" nil (n ih1 p iy0))
("nipsco" nil (n ih1 p s k ow0))
("niquette" nil (n ih0 k eh1 t))
("nir" nil (n ih1 r))
("nirenberg" nil (n ay1 r ax n b er0 g))
("niro" nil (n ih1 r ow0))
("nirvana" nil (n ih0 r v aa1 n ax))
("nisbet" nil (n ih1 z b ax t))
("nisbett" nil (n ih1 s b ih0 t))
("nisei" nil (n ih0 s ey1))
("nishi" nil (n iy1 sh iy0))
("nishida" nil (n iy0 sh iy1 d ax))
("nishikawa" nil (n iy0 sh iy0 k aa1 w ax))
("nishimo" nil (n ih0 sh iy1 m ow0))
("nishimoto" nil (n iy0 sh iy0 m ow1 t ow0))
("nishimura" nil (n iy0 sh iy0 m uh1 r ax))
("nishioka" nil (n iy1 sh iy0 ow1 k ax))
("nishiyama" nil (n iy0 sh iy0 y aa1 m ax))
("nishizawa" nil (n iy1 sh ih0 z aa1 w ax))
("niskanen" nil (n ih1 s k ax n ax n))
("nisley" nil (n ih1 z l iy0))
("nissa" nil (n ih1 s ax))
("nissan" nil (n iy1 s aa0 n))
("nissans" nil (n iy1 s aa0 n z))
("nissei" nil (n ih0 s ey1))
("nissen" nil (n ih1 s ax n))
("nissenbaum" nil (n ih1 s ax n b aw1 m))
("nisshin" nil (n ih1 sh ih0 n))
("nissho" nil (n ih1 sh ow0))
("nissim" nil (n ih1 s ih0 m))
("nissin" nil (n ih1 s ih0 n))
("nissley" nil (n ih1 s l iy0))
("nist" nil (n ih1 s t))
("nistler" nil (n ih1 s t l er0))
("niswander" nil (n ih1 s w ao0 n d er0))
("niswonger" nil (n ih1 s w ao0 ng er0))
("nit" nil (n ih1 t))
("nita" nil (n iy1 t ax))
("nite" nil (n ay1 t))
("nithuekan" nil (n ih1 th w ax k aa1 n))
("nitka" nil (n ih1 t k ax))
("nitpicking" nil (n ih1 t p ih1 k ih0 ng))
("nitrate" nil (n ay1 t r ey1 t))
("nitrating" nil (n ay1 t r ey1 t ih0 ng))
("nitration" nil (n ay0 t r ey1 sh ax n))
("nitric" nil (n ay1 t r ih0 k))
("nitride" nil (n ay1 t r ay0 d))
("nitriles" nil (n ay1 t r ax l z))
("nitro" nil (n ih1 t r ow0))
("nitrocellulose" nil (n ay1 t r ow0 s eh1 l y ax l ow1 s))
("nitrogen" nil (n ay1 t r ax jh ax n))
("nitrogenous" nil (n ay0 t r aa1 jh ax n ax s))
("nitroglycerin" nil (n ay1 t r ax g l ih1 s er0 ax n))
("nitrosamines" nil (n ih0 t r aa1 s ax m iy1 n eh0 z))
("nitrous" nil (n ih1 t r ax s))
("nitsch" nil (n ih1 ch))
("nitsche" nil (n ih1 ch))
("nitschke" nil (n ih1 ch k))
("nitta" nil (n ih1 t ax))
("nitto" nil (n ih1 t ow0))
("nitty" nil (n ih1 t iy0))
("nitz" nil (n ih1 t s))
("nitza" nil (n ih1 t z ax))
("nitzberg" nil (n ih1 t s b er0 g))
("nitze" nil (n ih1 t s iy0))
("nitzel" nil (n ih1 t z ax l))
("nitzsche" nil (n ih1 t z sh))
("niven" nil (n ay1 v ax n))
("nivens" nil (n ay1 v ax n z))
("niver" nil (n ay1 v er0))
("nivison" nil (n ih1 v ih0 s ax n))
("nix" nil (n ih1 k s))
("nixdorf" nil (n ih1 k s d ao1 r f))
("nixed" nil (n ih1 k s t))
("nixes" nil (n ih1 k s ih0 z))
("nixie" nil (n ih1 k s iy0))
("nixon" nil (n ih1 k s ax n))
("nizar" nil (n ay1 z aa0 r))
("nizhny" nil (n ih1 zh n iy0))
("niziolek" nil (n ih0 z iy0 ow1 l eh0 k))
("niznik" nil (n ih1 z n ih0 k))
("njt" nil (eh1 n jh ey1 t iy1))
("no" nil (n ow1))
("noa" nil (n ow1 ax))
("noack" nil (n ow1 k))
("noah" nil (n ow1 ax))
("noakes" nil (n ow1 k s))
("noam" nil (n ow1 m))
("noami" nil (n ow1 m iy0))
("nobbe" nil (n aa1 b))
("nobec" nil (n ow0 b eh1 k))
("nobel" nil (n ow0 b eh1 l))
("nobelium" nil (n ow0 b eh1 l iy0 ax m))
("nobile" nil (n aa1 b ax l))
("nobility" nil (n ow0 b ih1 l ax t iy0))
("nobis" nil (n ow1 b ih0 s))
("noble" nil (n ow1 b ax l))
("nobleman" nil (n ow1 b ax l m ax n))
("nobles" nil (n ow1 b ax l z))
("noblesse" nil (n ow0 b l eh1 s))
("noblet" nil (n aa1 b l ax t))
("noblett" nil (n aa1 b l ih0 t))
("noblewoman" nil (n ow1 b ax l w uh1 m ax n))
("noblin" nil (n aa1 b l ih0 n))
("noblitt" nil (n aa1 b l ih0 t))
("nobly" nil (n aa1 b l iy0))
("nobodies" nil (n ow1 b aa1 d iy1 z))
("nobody" nil (n ow1 b aa1 d iy1))
("noboru" nil (n ow0 b ao1 r uw0))
("nobrega" nil (n aa1 b r ih0 g ax))
("nobriga" nil (n aa1 b r ih0 g ax))
("nobuaki" nil (n ow1 b uw0 aa1 k iy0))
("nobuo" nil (n ow0 b uw1 ow0))
("nobuto" nil (n ow0 b uw1 t ow0))
("nobuyuki" nil (n ow1 b uw0 y uw1 k iy0))
("noce" nil (n ow1 s))
("nocella" nil (n ow0 ch eh1 l ax))
("nocera" nil (n ow0 ch eh1 r ax))
("nock" nil (n aa1 k))
("nocturnal" nil (n aa0 k t er1 n ax l))
("nod" nil (n aa1 d))
("noda" nil (n ow1 d ax))
("nodded" nil (n aa1 d ax d))
("nodding" nil (n aa1 d ih0 ng))
("node" nil (n ow1 d))
("nodes" nil (n ow1 d z))
("nodine" nil (n ow0 d iy1 n iy0))
("nods" nil (n aa1 d z))
("nodular" nil (n aa1 jh ax l er0))
("nodules" nil (n aa1 jh uw0 l z))
("noe" nil (n ow1))
("noecker" nil (n ow1 k er0))
("noel" nil (n ow0 eh1 l))
("noell" nil (n ow1 l))
("noelle" nil (n ow0 eh1 l))
("noes" nil (n ow1 z))
("noeth" nil (n ow1 th))
("noffke" nil (n aa1 f k))
("noffsinger" nil (n aa1 f s ih0 n jh er0))
("nofsinger" nil (n aa1 f s ih0 n jh er0))
("noftsker" nil (n ao1 f t s k er0))
("nofziger" nil (n ao1 f z ih0 g er0))
("noga" nil (n ow1 g ax))
("nogales" nil (n ow0 g aa1 l eh0 s))
("nogawa" nil (n ow0 g aa1 w ax))
("noggle" nil (n aa1 g ax l))
("nogle" nil (n ow1 g ax l))
("noguchi" nil (n ow0 g uw1 ch iy0))
("nogueira" nil (n ow0 g eh1 r ax))
("noguera" nil (n ow0 g eh1 r ax))
("noh" nil (n ow1))
("nohl" nil (n ow1 l))
("nohr" nil (n ao1 r))
("noir" nil (n oy1 r))
("noirs" nil (n oy1 r z))
("noise" nil (n oy1 z))
("noises" nil (n oy1 z ih0 z))
("noisier" nil (n oy1 z iy0 er0))
("noisiest" nil (n oy1 z iy0 ax s t))
("noisily" nil (n oy1 z ax l iy0))
("noisy" nil (n oy1 z iy0))
("nokes" nil (n ow1 k s))
("nokia" nil (n ow1 k iy0 ax))
("nokyo" nil (n ow1 k y ow0))
("nola" nil (n ow1 l ax))
("nolan" nil (n ow1 l ax n))
("nolana" nil (n ow0 l ae1 n ax))
("noland" nil (n ow1 l ax n d))
("nolasco" nil (n ow0 l aa1 s k ow0))
("nold" nil (n ow1 l d))
("nolde" nil (n ow1 l d))
("nolden" nil (n ow1 l d ax n))
("nolder" nil (n ow1 l d er0))
("nole" nil (n ow1 l))
("nolen" nil (n aa1 l ax n))
("noles" nil (n ow1 l z))
("nolet" nil (n ow1 l ih0 t))
("noleta" nil (n ow0 l eh1 t ax))
("nolette" nil (n ow1 l eh1 t))
("nolf" nil (n ow1 l f))
("nolie" nil (n aa1 l iy0))
("nolin" nil (n ow1 l ih0 n))
("nolita" nil (n ow0 l iy1 t ax))
("noll" nil (n ow1 l))
("nolla" nil (n ow1 l ax))
("nollan" nil (n aa1 l ax n))
("noller" nil (n ow1 l er0))
("nolley" nil (n aa1 l iy0))
("nollie" nil (n ow1 l iy0))
("nolt" nil (n ow1 l t))
("nolte" nil (n ow1 l t))
("nolting" nil (n ow1 l t ih0 ng))
("nom" nil (n aa1 m))
("nomad" nil (n ow1 m ae1 d))
("nomadic" nil (n ow0 m ae1 d ih0 k))
("nomads" nil (n ow1 m ae1 d z))
("nome" nil (n ow1 m))
("nomenclature" nil (n ow1 m ax n k l ey1 ch er0))
("nomenklatura" nil (n ow0 m eh1 ng k l ax ch uh1 r ax))
("nomi" nil (n ow1 m iy0))
("nominal" nil (n aa1 m ax n ax l))
("nominally" nil (n aa1 m ax n ax l iy0))
("nominate" nil (n aa1 m ax n ax t))
("nominated" nil (n aa1 m ax n ey1 t ax d))
("nominates" nil (n aa1 m ax n ey1 t s))
("nominating" nil (n aa1 m ax n ey1 t ih0 ng))
("nomination" nil (n aa1 m ax n ey1 sh ax n))
("nominations" nil (n aa1 m ax n ey1 sh ax n z))
("nominee" nil (n aa1 m ax n iy1))
("nominees" nil (n aa1 m ax n iy1 z))
("nomura" nil (n ow0 m uh1 r ax))
("non" nil (n aa1 n))
("nona" nil (n aa1 n ax))
("nonacademic" nil (n aa0 n ae1 k ax d eh1 m ih0 k))
("nonaccrual" nil (n aa1 n ax k r uw1 ax l))
("nonaccruing" nil (n aa1 n ax k r uw1 ih0 ng))
("nonaggression" nil (n aa1 n ax g r eh1 sh ax n))
("nonagricultural" nil (n aa1 n ae1 g r ih0 k ah1 l ch er0 ax l))
("nonalcoholic" nil (n aa1 n ae1 l k ax hh aa1 l ih0 k))
("nonaligned" nil (n aa1 n ax l ay1 n d))
("nonautomotive" nil (n aa1 n ao1 t ow0 m ow1 t ih0 v))
("nonbank" nil (n aa1 n b ae1 ng k))
("nonbanking" nil (n aa1 n b ae1 ng k ih0 ng))
("nonbelievers" nil (n aa1 n b ax l iy1 v er0 z))
("nonbinding" nil (n aa1 n b ay1 n d ih0 ng))
("nonbiting" nil (n aa0 n b ay1 t ih0 ng))
("nonbuilding" nil (n aa1 n b ih1 l d ih0 ng))
("nonbusiness" nil (n aa1 n b ih1 z n ax s))
("noncallable" nil (n aa0 n k ao1 l ax b ax l))
("noncash" nil (n aa1 n k ae1 sh))
("nonchalance" nil (n aa1 n sh ax l aa1 n s))
("nonchalant" nil (n aa1 n sh ax l aa1 n t))
("nonchalantly" nil (n aa1 n sh ax l aa1 n t l iy0))
("noncolor" nil (n aa0 n k ah1 l er0))
("noncombatants" nil (n aa1 n k ax m b ae1 t ax n t s))
("noncommercial" nil (n aa1 n k ax m er1 sh ax l))
("noncommittal" nil (n aa1 n k ax m ih1 t ax l))
("noncommunist" nil (n aa1 n k aa1 m y uw0 n ih0 s t))
("noncompete" nil (n aa0 n k ax m p iy1 t))
("noncompetitive" nil (n aa1 n k ax m p eh1 t ax t ih0 v))
("noncompliance" nil (n aa1 n k ax m p l ay1 ax n s))
("nonconformist" nil (n aa1 n k ax n f ao1 r m ih0 s t))
("nonconformists" nil (n aa1 n k ax n f ao1 r m ax s t s))
("nonconformity" nil (n aa1 n k ax n f ao1 r m ax t iy0))
("noncontract" nil (n aa0 n k aa1 n t r ae1 k t))
("noncontroversial" nil (n aa0 n k aa1 n t r ax v er1 sh ax l))
("nonconvertible" nil (n aa0 n k ax n v er1 t ax b ax l))
("noncore" nil (n aa1 n k ao1 r))
("noncorporate" nil (n aa0 n k ao1 r p r ax t))
("noncriminal" nil (n aa0 n k r ih1 m ih0 n ax l))
("noncumulative" nil (n aa0 n k y uw1 m y ax l ax t ih0 v))
("nondeductible" nil (n aa0 n d ih0 d ah1 k t ih0 b ax l))
("nondefense" nil (n aa0 n d ih0 f eh1 n s))
("nondescript" nil (n aa1 n d ih0 s k r ih1 p t))
("nondirect" nil (n aa1 n d er0 eh1 k t))
("nondiscrimination" nil (n aa0 n d ih1 s k r ih0 m ih0 n ey1 sh ax n))
("nondiscriminatory" nil (n aa1 n d ih0 s k r ih1 m ax n ax t ao1 r iy0))
("nondollar" nil (n aa1 n d aa1 l er0))
("nondurable" nil (n aa0 n d uh1 r ax b ax l))
("nondurables" nil (n aa0 n d uh1 r ax b ax l z))
("none" nil (n ah1 n))
("noneconomic" nil (n aa1 n eh1 k ax n aa1 m ih0 k))
("nonelectrical" nil (n aa0 n ih0 l eh1 k t r ih0 k ax l))
("nonemergency" nil (n aa1 n ih0 m er1 jh ax n s iy0))
("nonentity" nil (n aa0 n eh1 n t ax t iy0))
("nonessential" nil (n aa1 n ih0 s eh1 n ch ax l))
("nonesuch" nil (n ah1 n s ah1 ch))
("nonetheless" nil (n ah1 n dh ax l eh1 s))
("nonevent" nil (n aa1 n ih0 v eh1 n t))
("nonexclusive" nil (n aa1 n ih0 k s k l uw1 s ih0 v))
("nonexecutive" nil (n aa1 n ih0 g z eh1 k y ax t ih0 v))
("nonexistent" nil (n aa1 n ax g z ih1 s t ax n t))
("nonfarm" nil (n aa1 n f aa1 r m))
("nonfat" nil (n aa1 n f ae1 t))
("nonfatal" nil (n aa1 n f ey1 t ax l))
("nonferrous" nil (n aa0 n f eh1 r ax s))
("nonfiction" nil (n aa0 n f ih1 k sh ax n))
("nonfinancial" nil (n aa0 n f ax n ae1 n sh ax l))
("nonfood" nil (n aa1 n f uw1 d))
("nongovernment" nil (n aa0 n g ah1 v er0 n m ax n t))
("nongovernmental" nil (n aa0 n g ah1 v er0 n m eh1 n t ax l))
("nongreek" nil (n aa1 n g r iy1 k))
("nonhuman" nil (n aa0 n hh y uw1 m ax n))
("nonie" nil (n aa1 n iy0))
("noninflationary" nil (n aa1 n ih0 n f l ey1 sh ax n eh1 r iy0))
("noninterest" nil (n aa0 n ih1 n t r ax s t))
("noninterference" nil (n aa1 n ih1 n t er0 f ih1 r ax n s))
("nonintervention" nil (n aa1 n ih1 n t er0 v eh1 n ch ax n))
("nonjets" nil (n aa1 n jh eh1 t s))
("nonlethal" nil (n aa0 n l iy1 th ax l))
("nonlinear" nil (n aa0 n l ih1 n iy1 er0))
("nonliturgical" nil (n aa0 n l ax t er1 jh ih0 k ax l))
("nonmanagement" nil (n aa0 n m ae1 n ih0 jh m ax n t))
("nonmanufacturing" nil (n aa1 n m ae1 n y ax f ae1 k ch er0 ih0 ng))
("nonmember" nil (n aa0 n m eh1 m b er0))
("nonmembers" nil (n aa0 n m eh1 m b er0 z))
("nonmilitary" nil (n aa0 n m ih1 l ax t eh1 r iy0))
("nonnative" nil (n aa1 n ey1 t ih0 v))
("nonnemacher" nil (n aa1 n ih0 m ax k er0))
("nonnuclear" nil (n aa1 n uw1 k l iy1 er0))
("nonoperating" nil (n aa1 n ao1 p er0 ey1 t ih0 ng))
("nonpacifists" nil (n aa1 n p ae1 s ih0 f ih0 s t s))
("nonpartisan" nil (n aa0 n p aa1 r t ax z ax n))
("nonpaying" nil (n aa0 n p ey1 ih0 ng))
("nonpayment" nil (n aa0 n p ey1 m ax n t))
("nonperforming" nil (n aa0 n p er0 f ao1 r m ih0 ng))
("nonplanar" nil (n aa0 n p l ey1 n er0))
("nonplussed" nil (n aa0 n p l ah1 s t))
("nonpoisonous" nil (n aa0 n p oy1 z ax n ax s))
("nonpolitical" nil (n aa1 n p ax l ih1 t ih0 k ax l))
("nonprescription" nil (n aa1 n p r ax s k r ih1 p sh ax n))
("nonproductive" nil (n aa1 n p r ax d ah1 k t ih0 v))
("nonprofessional" nil (n aa1 n p r ax f eh1 sh ax n ax l))
("nonprofessionals" nil (n aa0 n p r ax f eh1 sh ax n ax l z))
("nonprofit" nil (n aa0 n p r aa1 f ax t))
("nonprofits" nil (n aa1 n p r aa1 f ih0 t s))
("nonproliferation" nil (n aa0 n p r ax l ih1 f er0 ey1 sh ax n))
("nonpublic" nil (n aa0 n p ah1 b l ih0 k))
("nonqualified" nil (n aa0 n k w aa1 l ax f ay1 d))
("nonracial" nil (n aa0 n r ey1 sh ax l))
("nonrecurring" nil (n aa0 n r ih0 k er1 ih0 ng))
("nonrefundable" nil (n aa0 n r ih0 f ah1 n d ax b ax l))
("nonregulated" nil (n aa0 n r eh1 g y ax l ey1 t ih0 d))
("nonreligious" nil (n aa1 n r ih0 l ih1 jh ax s))
("nonrenewable" nil (n aa0 n r iy0 n uw1 ax b ax l))
("nonresident" nil (n aa0 n r eh1 z ax d ax n t))
("nonresidential" nil (n aa1 n r eh1 z ax d eh1 n sh ax l))
("nonresidents" nil (n aa0 n r eh1 z ax d ax n t s))
("nonroman" nil (n aa0 n r ow1 m ax n))
("nonruling" nil (n aa0 n r uw1 l ih0 ng))
("nonsectarian" nil (n aa1 n s eh0 k t eh1 r iy0 ax n))
("nonsense" nil (n aa1 n s eh0 n s))
("nonsensical" nil (n aa0 n s eh1 n s ih0 k ax l))
("nonsmoker" nil (n aa0 n s m ow1 k er0))
("nonsmokers" nil (n aa0 n s m ow1 k er0 z))
("nonsmoking" nil (n aa1 n s m ow1 k ih0 ng))
("nonspecific" nil (n aa1 n s p ax s ih1 f ih0 k))
("nonsporting" nil (n aa0 n s p ao1 r t ih0 ng))
("nonstandard" nil (n aa0 n s t ae1 n d er0 d))
("nonstarter" nil (n aa0 n s t aa1 r t er0))
("nonstick" nil (n aa0 n s t ih1 k))
("nonstop" nil (n aa1 n s t aa1 p))
("nonstops" nil (n aa1 n s t aa1 p s))
("nonstrategic" nil (n aa0 n s t r ax t iy1 jh ih0 k))
("nonsurgical" nil (n aa0 n s er1 jh ih0 k ax l))
("nontaxable" nil (n aa0 n t ae1 k s ax b ax l))
("nontechnical" nil (n aa0 n t eh1 k n ih0 k ax l))
("nontoxic" nil (n aa0 n t aa1 k s ih0 k))
("nontraditional" nil (n aa1 n t r ax d ih1 sh ax n ax l))
("nontransferable" nil (n aa1 n t r ae1 n s f er1 ax b ax l))
("nonunion" nil (n aa0 n y uw1 n y ax n))
("nonunionized" nil (n aa0 n y uw1 n y ax n ay1 z d))
("nonutility" nil (n aa1 n y uw0 t ih1 l ih0 t iy0))
("nonviolence" nil (n aa0 n v ay1 ax l ax n s))
("nonviolent" nil (n aa0 n v ay1 ax l ax n t))
("nonvolatile" nil (n aa0 n v aa1 l ax t ax l))
("nonvoting" nil (n aa0 n v ow1 t ih0 ng))
("nonwhite" nil (n aa1 n w ay1 t))
("nonwhites" nil (n aa1 n w ay1 t s))
("nonwovens" nil (n aa1 n w ow1 v ax n z))
("noodle" nil (n uw1 d ax l))
("noodles" nil (n uw1 d ax l z))
("nook" nil (n uh1 k))
("nooks" nil (n uh1 k s))
("noon" nil (n uw1 n))
("noonan" nil (n uw1 n ax n))
("noone" nil (n uw1 n))
("nooney" nil (n uw1 n iy0))
("noons" nil (n uw1 n z))
("noontime" nil (n uw1 n t ay1 m))
("noorda" nil (n uh1 r d ax))
("noose" nil (n uw1 s))
("nope" nil (n ow1 p))
("nor" nil (n ao1 r))
("nora" nil (n ao1 r ax))
("norah" nil (n ao1 r ax))
("noramco" nil (n ao0 r ae1 m k ow0))
("noranda" nil (n ao0 r ae1 n d ax))
("norbeck" nil (n ao1 r b eh0 k))
("norberg" nil (n ao1 r b er0 g))
("norbert" nil (n ao1 r b er0 t))
("norberta" nil (n ao0 r b eh1 r t ax))
("norberto" nil (n ao0 r b er1 t ow1))
("norbury" nil (n ao1 r b eh0 r iy0))
("norby" nil (n ao1 r b iy0))
("norcen" nil (n ao1 r s ax n))
("norcia" nil (n ao1 r ch ax))
("norco" nil (n ao1 r k ow0))
("norcross" nil (n ao1 r k r ao0 s))
("nord" nil (n ao1 r d))
("nordahl" nil (n ao1 r d aa0 l))
("nordan" nil (n ao1 r d ax n))
("nordbanken" nil (n ao1 r d b ae1 ng k ax n))
("nordberg" nil (n ao1 r d b er0 g))
("nordby" nil (n ao1 r d b iy0))
("nordeen" nil (n er0 d iy1 n))
("nordell" nil (n ao1 r d ax l))
("norden" nil (n ao1 r d ax n))
("norder" nil (n ao1 r d er0))
("nordgren" nil (n ao1 r d g r eh0 n))
("nordic" nil (n ao1 r d ih0 k))
("nordica" nil (n ao1 r d ih0 k ax))
("nordictrack" nil (n ao1 r d ih0 k t r ae1 k))
("nordin" nil (n ao1 r d ih0 n))
("nordine" nil (n ao0 r d iy1 n iy0))
("nordio" nil (n ao1 r d iy0 ow0))
("nordisk" nil (n ao1 r d ih1 s k))
("nordling" nil (n ao1 r d ax l ih0 ng))
("nordlund" nil (n ao1 r d l ax n d))
("nordman" nil (n ao1 r d m ax n))
("nordmann" nil (n ao1 r d m ax n))
("nordmeyer" nil (n ao1 r d m ay0 er0))
("nordquist" nil (n ao1 r d k w ih0 s t))
("nordson" nil (n ao1 r d s ax n))
("nordstrom" nil (n ao1 r d s t r ax m))
("nordstroms" nil (n ao1 r d s t r ax m z))
("nordyke" nil (n ao1 r d ay1 k))
("noreco" nil (n ao0 r eh1 k ow0))
("nored" nil (n ao1 r d))
("noreen" nil (n er0 iy1 n))
("noreiga" nil (n ao0 r ey1 g ax))
("norelco" nil (n er0 eh1 l k ow0))
("norell" nil (n ao1 r ax l))
("norem" nil (n ao1 r ih0 m))
("noren" nil (n ao1 r ax n))
("norenberg" nil (n ao1 r ax n b er0 g))
("norex" nil (n ao1 r ax k s))
("norfleet" nil (n ao1 r f l iy1 t))
("norfolk" nil (n ao1 r f ax k))
("norgaard" nil (n ao1 r g aa0 r d))
("norgard" nil (n ao1 r g er0 d))
("norge" nil (n ao1 r jh))
("norgren" nil (n ao1 r g r ax n))
("noriega" nil (n ao1 r iy0 ey1 g ax))
("noriegas" nil (n ao1 r iy0 ey1 g ax s))
("norilsk" nil (n ao1 r ih1 l s k))
("norimatsu" nil (n ao1 r iy0 m aa1 t s uw0))
("norinchukin" nil (n ao1 r ih0 n ch uw1 k ih0 n))
("nork" nil (n ao1 r k))
("norkus" nil (n ao1 r k ih0 s))
("norland" nil (n ao1 r l ax n d))
("norlander" nil (n ao1 r l ax n d er0))
("norlin" nil (n ao1 r l ih0 n))
("norling" nil (n ao1 r l ih0 ng))
("norm" nil (n ao1 r m))
("norma" nil (n ao1 r m ax))
("normal" nil (n ao1 r m ax l))
("normalcy" nil (n ao1 r m ax l s iy0))
("normality" nil (n ao1 r m ae1 l ax t iy0))
("normalization" nil (n ao1 r m ax l ih0 z ey1 sh ax n))
("normalize" nil (n ao1 r m ax l ay1 z))
("normalized" nil (n ao1 r m ax l ay1 z d))
("normalizing" nil (n ao1 r m ax l ay1 z ih0 ng))
("normally" nil (n ao1 r m ax l iy0))
("norman" nil (n ao1 r m ax n))
("normand" nil (n ao1 r m ax n d))
("normandin" nil (n ao1 r m ax n d ih0 n))
("normandy" nil (n ao1 r m ax n d iy0))
("normative" nil (n ao1 r m ax t ih0 v))
("norment" nil (n ao1 r m ax n t))
("normick" nil (n ao1 r m ih0 k))
("normie" nil (n ao1 r m iy0))
("normile" nil (n ao1 r m ay0 l))
("normington" nil (n ao1 r m ih0 ng t ax n))
("normoyle" nil (n ao1 r m oy0 l))
("norms" nil (n ao1 r m z))
("normy" nil (n ao1 r m iy0))
("norna" nil (n ao1 r n ax))
("norodom" nil (n ao1 r ow0 d ah1 m))
("norplant" nil (n ao1 r p l ae1 n t))
("norquist" nil (n ao1 r k w ih0 s t))
("norred" nil (n ao1 r d))
("norrell" nil (n ao1 r ax l))
("norrie" nil (n ao1 r iy0))
("norrington" nil (n ao1 r ih0 ng t ax n))
("norris" nil (n ao1 r ih0 s))
("norrod" nil (n ao1 r ax d))
("norry" nil (n ao1 r iy0))
("norse" nil (n ao1 r s))
("norsemen" nil (n ao1 r s m ih0 n))
("norsk" nil (n ao1 r s k))
("norske" nil (n ao1 r s k iy0))
("norstan" nil (n ao1 r s t ax n))
("norstar" nil (n ao1 r s t aa1 r))
("norstrom" nil (n ao1 r s t r ax m))
("norsworthy" nil (n ao1 r s w er0 dh iy0))
("norte" nil (n ao1 r t))
("nortek" nil (n ao1 r t eh1 k))
("nortel" nil (n ao1 r t eh0 l))
("north" nil (n ao1 r th))
("northam" nil (n ao1 r th ax m))
("northampton" nil (n ao1 r th ae1 m p t ax n))
("northbrook" nil (n ao1 r th b r uh1 k))
("northcliff" nil (n ao1 r th k l ih0 f))
("northcott" nil (n ao1 r th k ax t))
("northcraft" nil (n ao1 r th k r ae1 f t))
("northcutt" nil (n ao1 r th k ax t))
("northeast" nil (n ao1 r th iy1 s t))
("northeastern" nil (n ao1 r th iy1 s t er0 n))
("northeasterners" nil (n ao0 r th iy1 s t er0 n er0 z))
("northeastward" nil (n ao1 r th iy1 s t w er0 d))
("norther" nil (n ao1 r dh er0))
("northerly" nil (n ao1 r dh er0 l iy0))
("northern" nil (n ao1 r dh er0 n))
("northerner" nil (n ao1 r dh er0 n er0))
("northerners" nil (n ao1 r dh er0 n er0 z))
("northernmost" nil (n ao1 r dh er0 n m ow1 s t))
("northey" nil (n ao1 r dh iy0))
("northfield" nil (n ao1 r th f iy1 l d))
("northgate" nil (n ao1 r th g ey1 t))
("northington" nil (n ao1 r th ih0 ng t ax n))
("northland" nil (n ao1 r th l ae1 n d))
("northridge" nil (n ao1 r th r ih1 jh))
("northrop" nil (n ao1 r th r ax p))
("northrup" nil (n ao1 r th r ax p))
("northstar" nil (n ao1 r th s t aa1 r))
("northup" nil (n ao1 r th ah1 p))
("northvale" nil (n ao1 r th v ey1 l))
("northview" nil (n ao1 r th v y uw1))
("northward" nil (n ao1 r th w er0 d))
("northway" nil (n ao1 r th w ey1))
("northwest" nil (n ao1 r th w eh1 s t))
("northwestern" nil (n ao1 r th w eh1 s t er0 n))
("northwood" nil (n ao1 r th w uh1 d))
("norton" nil (n ao1 r t ax n))
("norvel" nil (n ao1 r v ax l))
("norvell" nil (n ao1 r v ax l))
("norvil" nil (n ao1 r v ax l))
("norville" nil (n ao1 r v ih0 l))
("norvin" nil (n ao1 r v ih0 n))
("norwalk" nil (n ao1 r w ao1 k))
("norward" nil (n ao1 r w er0 d))
("norway" nil (n ao1 r w ey1))
("norweb" nil (n ao1 r w eh1 b))
("norwegian" nil (n ao1 r w iy1 jh ax n))
("norwegians" nil (n ao0 r w iy1 jh ax n z))
("norwell" nil (n ao1 r w eh0 l))
("norwest" nil (n ao1 r w eh1 s t))
("norwich" nil (n ao1 r w ih0 ch))
("norwin" nil (n ao1 r w ih0 n))
("norwitz" nil (n ao1 r w ih0 t s))
("norwood" nil (n ao1 r w uh0 d))
("norwyn" nil (n ao1 r w ih0 n))
("nosal" nil (n ow1 z ax l))
("nose" nil (n ow1 z))
("nosebleed" nil (n ow1 z b l iy1 d))
("nosed" nil (n ow1 z d))
("nosedive" nil (n ow1 z d ay1 v))
("nosedived" nil (n ow1 z d ay1 v d))
("nosek" nil (n ow1 s eh0 k))
("noses" nil (n ow1 z ih0 z))
("noseworthy" nil (n ow1 z w er1 dh iy0))
("nosiness" nil (n ow1 z iy0 n ax s))
("nosing" nil (n ow1 z ih0 ng))
("noska" nil (n ow1 s k ax))
("noss" nil (n ao1 s))
("nostalgia" nil (n ao0 s t ae1 l jh ax))
("nostalgic" nil (n ao0 s t ae1 l jh ih0 k))
("nostra" nil (n ao1 s t r ax))
("nostrand" nil (n aa1 s t r ax n d))
("nostrils" nil (n aa1 s t r ax l z))
("nostrums" nil (n aa1 s t r ax m z))
("not" nil (n aa1 t))
("notable" nil (n ow1 t ax b ax l))
("notables" nil (n ow1 t ax b ax l z))
("notably" nil (n ow1 t ax b l iy0))
("notarianni" nil (n ow0 t aa0 r iy0 aa1 n iy0))
("notaro" nil (n ow0 t aa1 r ow0))
("notary" nil (n ow1 t er0 iy0))
("notation" nil (n ow0 t ey1 sh ax n))
("notations" nil (n ow0 t ey1 sh ax n z))
("notch" nil (n aa1 ch))
("notched" nil (n aa1 ch t))
("notches" nil (n aa1 ch ax z))
("note" nil (n ow1 t))
("notebaert" nil (n ow1 t ax b eh1 r t))
("notebook" nil (n ow1 t b uh1 k))
("notebooks" nil (n ow1 t b uh1 k s))
("noteboom" nil (n aa1 t ih0 b uw0 m))
("noted" nil (n ow1 t ax d))
("noteholders" nil (n ow1 t hh ow1 l d er0 z))
("notes" nil (n ow1 t s))
("notestine" nil (n ow0 t eh0 s t iy1 n iy0))
("noteware" nil (n ow1 t w eh1 r))
("noteworthy" nil (n ow1 t w er1 dh iy0))
("noth" nil (n aa1 th))
("nothdurft" nil (n aa1 th d er0 f t))
("nothin" nil (n aa1 th ih0 n))
("nothing" nil (n ah1 th ih0 ng))
("nothingness" nil (n ah1 th ih0 ng n ax s))
("nothings" nil (n ah1 th ih0 ng z))
("nothnagel" nil (n aa1 th n ax g ax l))
("nothstein" nil (n aa1 th s t iy1 n))
("notice" nil (n ow1 t ax s))
("noticeable" nil (n ow1 t ax s ax b ax l))
("noticeably" nil (n ow1 t ih0 s ax b l iy0))
("noticed" nil (n ow1 t ax s t))
("notices" nil (n ow1 t ax s ax z))
("noticing" nil (n ow1 t ih0 s ih0 ng))
("notification" nil (n ow1 t ax f ax k ey1 sh ax n))
("notifications" nil (n ow1 t ax f ax k ey1 sh ax n z))
("notified" nil (n ow1 t ax f ay1 d))
("notifies" nil (n ow1 t ax f ay1 z))
("notify" nil (n ow1 t ax f ay1))
("notifying" nil (n ow1 t ax f ay1 ih0 ng))
("noting" nil (n ow1 t ih0 ng))
("notion" nil (n ow1 sh ax n))
("notions" nil (n ow1 sh ax n z))
("notis" nil (n ow1 t ih0 s))
("noto" nil (n ow1 t ow0))
("notochord" nil (n ow1 t ax k ao1 r d))
("notoriety" nil (n ow1 t er0 ay1 ax t iy0))
("notorious" nil (n ow0 t ao1 r iy0 ax s))
("notoriously" nil (n ow0 t ao1 r iy0 ax s l iy0))
("notre" nil (n ow1 t r ax))
("nots" nil (n aa1 t s))
("nott" nil (n aa1 t))
("nottage" nil (n aa1 t ih0 jh))
("notte" nil (n aa1 t))
("notter" nil (n aa1 t er0))
("nottingham" nil (n aa1 t ih0 ng hh ae1 m))
("notwithstanding" nil (n aa1 t w ih0 th s t ae1 n d ih0 ng))
("notz" nil (n aa1 t s))
("noun" nil (n aw1 n))
("nourish" nil (n er1 ih0 sh))
("nourished" nil (n er1 ih0 sh t))
("nourishing" nil (n er1 ih0 sh ih0 ng))
("nourishment" nil (n er1 ih0 sh m ax n t))
("nourse" nil (n ao1 r s))
("nouveau" nil (n uw1 v ow1))
("nouvel" nil (n uw1 v ax l))
("nouvelle" nil (n uw1 v eh1 l))
("nov" nil (n ow0 v eh1 m b er0))
("nova" nil (n ow1 v ax))
("novacare" nil (n ow1 v ax k eh1 r))
("novacek" nil (n aa1 v ax ch eh0 k))
("novack" nil (n aa1 v ax k))
("novacor" nil (n ow1 v ax k ao1 r))
("novak" nil (n ow1 v ae0 k))
("novakovich" nil (n ax v aa1 k ax v ih0 ch))
("novakowski" nil (n ax v ax k ao1 f s k iy0))
("novametrix" nil (n ow0 v ae1 m ax t r ih0 k s))
("novas" nil (n ow1 v ax z))
("novato" nil (n ow0 v aa1 t ow0))
("novel" nil (n aa1 v ax l))
("novelist" nil (n aa1 v ax l ax s t))
("novelistic" nil (n aa1 v ax l ih1 s t ih0 k))
("novelists" nil (n aa1 v ax l ax s t s))
("novell" nil (n ow0 v eh1 l))
("novella" nil (n ow0 v eh1 l ax))
("novelli" nil (n ow0 v eh1 l iy0))
("novello" nil (n ow0 v eh1 l ow0))
("novellus" nil (n ow0 v eh1 l ax s))
("novelly" nil (n ow1 v ax l iy0))
("novels" nil (n aa1 v ax l z))
("novelties" nil (n aa1 v ax l t iy0 z))
("novelty" nil (n aa1 v ax l t iy0))
("november" nil (n ow0 v eh1 m b er0))
("noverco" nil (n ow0 v er1 k ow0))
("novey" nil (n ow1 v iy0))
("novgorod" nil (n aa1 v g er0 aa1 d))
("novia" nil (n ow1 v iy0 ax))
("novice" nil (n aa1 v ax s))
("novices" nil (n aa1 v ax s ih0 z))
("novick" nil (n ow1 v ih0 k))
("novicki" nil (n ax v ih1 t s k iy0))
("novinger" nil (n ow1 v ih0 ng er0))
("novitski" nil (n ax v ih1 t s k iy0))
("novitsky" nil (n ax v ih1 t s k iy0))
("novo" nil (n ow1 v ow0))
("novoa" nil (n ow0 v ow1 ax))
("novopharm" nil (n ow1 v ow0 f aa1 r m))
("novosad" nil (n aa1 v ax s ae0 d))
("novosel" nil (n aa1 v ax s eh0 l))
("novosibirsk" nil (n ow0 v ax s ih0 b er1 s k))
("novosti" nil (n ow0 v ao1 s t iy0))
("novotney" nil (n aa1 v ax t n iy0))
("novotny" nil (n ax v ow1 t n iy0))
("novum" nil (n ow1 v ax m))
("novy" nil (n ow1 v iy0))
("now" nil (n aw1))
("nowa" nil (n aw1 ax))
("nowack" nil (n aw1 ax k))
("nowacki" nil (n aw0 aa1 t s k iy0))
("nowaczyk" nil (n aw1 ax ch ih1 k))
("nowadays" nil (n aw1 ax d ey1 z))
("nowak" nil (n ow1 v ae0 k))
("nowakowski" nil (n aw0 ax k ao1 f s k iy0))
("nowell" nil (n ow1 ax l))
("nowels" nil (n aw1 ax l z))
("nowhere" nil (n ow1 w eh1 r))
("nowicki" nil (n aw0 ih1 t s k iy0))
("nowinski" nil (n aw0 ih1 n s k iy0))
("nowlan" nil (n aw1 l ax n))
("nowland" nil (n aw1 l ax n d))
("nowlin" nil (n aw1 l ih0 n))
("nowling" nil (n aw1 l ih0 ng))
("nowotny" nil (n aw0 ow1 t n iy0))
("noxell" nil (n aa0 k s eh1 l))
("noxema" nil (n aa1 k s m aa0))
("noxious" nil (n aa1 k sh ax s))
("noxon" nil (n aa1 k s ax n))
("noxso" nil (n aa1 k s ow0))
("noyce" nil (n oy1 s))
("noyd" nil (n oy1 d))
("noyes" nil (n oy1 z))
("noyola" nil (n oy1 ow1 l ax))
("nozzle" nil (n aa1 z ax l))
("nozzles" nil (n aa1 z ax l z))
("nu" nil (n uw1))
("nuala" nil (n uw0 aa1 l ax))
("nuance" nil (n uw1 aa0 n s))
("nuanced" nil (n uw1 aa0 n s t))
("nuances" nil (n uw1 aa0 n s ih0 z))
("nub" nil (n ah1 b))
("nuber" nil (n uw1 b er0))
("nubian" nil (n y uw1 b iy0 ax n))
("nucci" nil (n uw1 ch iy0))
("nuccio" nil (n uw1 ch iy0 ow0))
("nuckles" nil (n ah1 k ax l z))
("nuckolls" nil (n ah1 k ax l z))
("nuckols" nil (n ah1 k ax l z))
("nuclear" nil (n uw1 k l iy0 er0))
("nucleation" nil (n uw1 k l iy0 ey1 sh ax n))
("nuclei" nil (n uw1 k l iy0 ay1))
("nucleic" nil (n uw0 k l ey1 ih0 k))
("nucleonics" nil (n ah1 k l iy0 aa1 n ih0 k s))
("nucleotide" nil (n uw1 k l iy0 ax t ay1 d))
("nucleus" nil (n uw1 k l iy0 ax s))
("nucor" nil (n uw1 k ao1 r))
("nucorp" nil (n uw1 k ao1 r p))
("nudd" nil (n ah1 d))
("nude" nil (n uw1 d))
("nudelman" nil (n ah1 d ax l m ax n))
("nudes" nil (n uw1 d z))
("nudge" nil (n ah1 jh))
("nudged" nil (n ah1 jh d))
("nudges" nil (n ah1 jh ih0 z))
("nudging" nil (n ah1 jh ih0 ng))
("nudist" nil (n uw1 d ih0 s t))
("nudity" nil (n uw1 d ih0 t iy0))
("nudo" nil (n uw1 d ow0))
("nuevo" nil (n w ey1 v ow0))
("nuexco" nil (n w ey1 k s k ow0))
("nufer" nil (n uw1 f er0))
("nuffer" nil (n ah1 f er0))
("nugan" nil (n uw1 g ax n))
("nugent" nil (n uw1 jh ax n t))
("nugget" nil (n ah1 g ih0 t))
("nuggets" nil (n ah1 g ax t s))
("nuhfer" nil (n uw1 f er0))
("nuhn" nil (n ah1 n))
("nuisance" nil (n uw1 s ax n s))
("nujoma" nil (n uw0 jh ow1 m ax))
("nuke" nil (n uw1 k))
("nukem" nil (n uw1 k ax m))
("nukes" nil (n uw1 k s))
("null" nil (n ah1 l))
("nullified" nil (n ah1 l ax f ay1 d))
("nullifies" nil (n ah1 l ax f ay1 z))
("nullify" nil (n ah1 l ax f ay1))
("nullifying" nil (n ah1 l ax f ay1 ih0 ng))
("nulph" nil (n ah1 l f))
("nulton" nil (n ah1 l t ax n))
("nulty" nil (n ah1 l t iy0))
("numac" nil (n uw1 m ae0 k))
("numb" nil (n ah1 m))
("numbed" nil (n ah1 m d))
("number" nil (n ah1 m b er0))
("numbered" nil (n ah1 m b er0 d))
("numbering" nil (n ah1 m b er0 ih0 ng))
("numbers" nil (n ah1 m b er0 z))
("numbing" nil (n ah1 m ih0 ng))
("numbingly" nil (n ah1 m ih0 ng l iy0))
("numbness" nil (n ah1 m n ax s))
("numed" nil (n uw0 m eh1 d))
("numeiri" nil (n uw0 m ey1 r iy0))
("numerals" nil (n uw1 m er0 ax l z))
("numerica" nil (n uw0 m eh1 r ih0 k ax))
("numerical" nil (n uw0 m eh1 r ax k ax l))
("numerically" nil (n uw0 m eh1 r ih0 k l iy0))
("numerous" nil (n uw1 m er0 ax s))
("numidia" nil (n uw0 m ih1 d iy0 ax))
("numismatic" nil (n uw1 m ih0 s m ae1 t ih0 k))
("numismatics" nil (n uw1 m ih0 s m ae1 t ih0 k s))
("numismatists" nil (n uw0 m ih1 s m ax t ih1 s t s))
("nummi" nil (n uw1 m iy0))
("nun" nil (n ah1 n))
("nunamaker" nil (n uw0 n aa1 m ey0 k er0))
("nunan" nil (n uw1 n aa0 n))
("nuncio" nil (n ah1 n s iy0 ow0))
("nunemaker" nil (n uw1 n m ey1 k er0))
("nunes" nil (n uw1 n z))
("nunez" nil (n uw1 n eh0 z))
("nungesser" nil (n ah1 ng g ih0 s er0))
("nunley" nil (n ah1 n l iy0))
("nunn" nil (n ah1 n))
("nunnally" nil (n ah1 n ax l iy0))
("nunnelley" nil (n ah1 n ih0 l iy0))
("nunnelly" nil (n ah1 n ax l iy0))
("nunnery" nil (n ah1 n er0 iy0))
("nuno" nil (n uw1 n ow0))
("nuns" nil (n ah1 n z))
("nunsense" nil (n ah1 n s eh1 n s))
("nunziata" nil (n uw0 n z iy0 aa1 t ax))
("nunziato" nil (n uw0 n z iy0 aa1 t ow0))
("nunzio" nil (n ah1 n z iy0 ow0))
("nuova" nil (n w ow1 v ax))
("nuovo" nil (n w ow1 v ow0))
("nupeng" nil (n y uw1 p eh0 ng))
("nuptial" nil (n ah1 p ch ax l))
("nuremberg" nil (n uh1 r ax m b er0 g))
("nureyev" nil (n er0 ay1 ax v))
("nurmi" nil (n uh1 r m iy0))
("nurnberger" nil (n er1 n b er0 g er0))
("nurock" nil (n uw1 r aa1 k))
("nurre" nil (n er1))
("nurse" nil (n er1 s))
("nursed" nil (n er1 s t))
("nurseries" nil (n er1 s er0 iy0 z))
("nursery" nil (n er1 s er0 iy0))
("nurseryman" nil (n er1 s er0 iy0 m ae1 n))
("nurserymen" nil (n er1 s er0 iy0 m ax n))
("nurses" nil (n er1 s ax z))
("nursing" nil (n er1 s ih0 ng))
("nurture" nil (n er1 ch er0))
("nurtured" nil (n er1 ch er0 d))
("nurturer" nil (n er1 t er0 er0))
("nurtures" nil (n er1 ch er0 z))
("nurturing" nil (n er1 ch er0 ih0 ng))
("nusbaum" nil (n iy1 ih0 s b aw0 m))
("nuss" nil (n ah1 s))
("nussbaum" nil (n ah1 s b aw1 m))
("nussbaumer" nil (n ah1 s b aw1 m er0))
("nusser" nil (n ah1 s er0))
("nussle" nil (n ah1 s ax l))
("nusz" nil (n ah1 sh))
("nut" nil (n ah1 t))
("nutcracker" nil (n ah1 t k r ae1 k er0))
("nute" nil (n y uw1 t))
("nutlets" nil (n ah1 t l ax t s))
("nutley" nil (n ah1 t l iy0))
("nutmeg" nil (n ah1 t m eh1 g))
("nutone" nil (n uw1 t ow1 n))
("nutrasweet" nil (n uw1 t r ax s w iy1 t))
("nutri" nil (n uw1 t r iy0))
("nutriclean" nil (n uw1 t r ih0 k l iy1 n))
("nutrient" nil (n uw1 t r iy0 ax n t))
("nutrients" nil (n uw1 t r iy0 ax n t s))
("nutrition" nil (n uw0 t r ih1 sh ax n))
("nutritional" nil (n uw0 t r ih1 sh ax n ax l))
("nutritionist" nil (n uw0 t r ih1 sh ax n ih0 s t))
("nutritionists" nil (n uw0 t r ih1 sh ax n ih0 s t s))
("nutritious" nil (n uw0 t r ih1 sh ax s))
("nutritive" nil (n uw1 t r ax t ih0 v))
("nuts" nil (n ah1 t s))
("nutshell" nil (n ah1 ch eh1 l))
("nutt" nil (n ah1 t))
("nuttall" nil (n ah1 t ax l))
("nutter" nil (n ah1 t er0))
("nutting" nil (n ah1 t ih0 ng))
("nuttle" nil (n ah1 t ax l))
("nutty" nil (n ah1 t iy0))
("nuveen" nil (n uw0 v iy1 n))
("nuxoll" nil (n ah1 k s ax l))
("nuys" nil (n ay1 z))
("nuzum" nil (n uw1 z ax m))
("nuzzi" nil (n uw1 t s iy0))
("nuzzle" nil (n ah1 z ax l))
("nuzzo" nil (n uw1 z ow0))
("nvhomes" nil (eh1 n v iy1 hh ow1 m z))
("nvryan" nil (eh0 n v r ay1 ax n))
("nyack" nil (n ay1 ae1 k))
("nyberg" nil (n ay1 b er0 g))
("nyborg" nil (n ih1 b ao0 r g))
("nyce" nil (n ay1 s))
("nycor" nil (n ay1 k ao1 r))
("nycum" nil (n ih1 k ax m))
("nycz" nil (n ih1 ch))
("nydam" nil (n ih1 d ax m))
("nydegger" nil (n ih1 d ih0 g er0))
("nydia" nil (n ih1 d iy0 ax))
("nye" nil (n ay1))
("nyenhuis" nil (n ay1 ax n hh uw1 s))
("nyerere" nil (n ay1 er0 ih1 r))
("nyers" nil (n ay1 r z))
("nyet" nil (n y eh1 t))
("nygaard" nil (n ih1 g aa0 r d))
("nygard" nil (n ih1 g er0 d))
("nygren" nil (n ih1 g r eh0 n))
("nyholm" nil (n ih1 hh ow0 l m))
("nyhus" nil (n ih1 hh ih0 s))
("nyland" nil (n ih1 l ax n d))
("nylander" nil (n ih1 l ax n d er0))
("nylen" nil (n ih1 l ax n))
("nylex" nil (n ay1 l ax k s))
("nylon" nil (n ay1 l aa1 n))
("nylons" nil (n ay1 l aa1 n z))
("nylund" nil (n ih1 l ax n d))
("nyman" nil (n ay1 m ax n))
("nymex" nil (n ay1 m eh1 k s))
("nymph" nil (n ih1 m f))
("nymphomaniac" nil (n ih1 m f ax m ey1 n iy0 ae1 k))
("nymphomaniacs" nil (n ih1 m f ax m ey1 n iy0 ae1 k s))
("nymphs" nil (n ih1 m f s))
("nynex" nil (n ay1 n eh1 k s))
("nyquist" nil (n ih1 k w ih0 s t))
("nyssa" nil (n ih1 s ax))
("nystrom" nil (n ay1 s t r ax m))
("nyx" nil (n ih1 k s))
("o" nil (ow1))
("oad" nil (ow1 ey1 d iy1))
("oahu" nil (ow1 aa1 hh uw0))
("oak" nil (ow1 k))
("oakar" nil (ow1 k aa1 r))
("oakbrook" nil (ow1 k b r uh1 k))
("oakdale" nil (ow1 k d ey1 l))
("oakes" nil (ow1 k s))
("oakey" nil (ow1 k iy0))
("oakite" nil (ow1 k ay1 t))
("oakland" nil (ow1 k l ax n d))
("oakleaf" nil (ow1 k l iy1 f))
("oakley" nil (ow1 k l iy0))
("oakman" nil (ow1 k m ax n))
("oakmark" nil (ow1 k m aa1 r k))
("oakmont" nil (ow1 k m aa1 n t))
("oaks" nil (ow1 k s))
("oakville" nil (ow1 k v ih1 l))
("oakwood" nil (ow1 k w uh1 d))
("oana" nil (ow1 n ax))
("oanh" nil (ow1 n))
("oar" nil (ao1 r))
("oared" nil (ao1 r d))
("oarlock" nil (ao1 r l aa1 k))
("oars" nil (ao1 r z))
("oarsman" nil (ao1 r z m ax n))
("oas" nil (ow1 z))
("oases" nil (ow0 ey1 s iy0 z))
("oasis" nil (ow0 ey1 s ax s))
("oat" nil (ow1 t))
("oates" nil (ow1 t s))
("oath" nil (ow1 th))
("oathout" nil (ow1 th aw1 t))
("oaths" nil (ow1 dh z))
("oatis" nil (ow1 t ih0 s))
("oatley" nil (ow1 t l iy0))
("oatman" nil (ow1 t m ax n))
("oatmeal" nil (ow1 t m iy1 l))
("oats" nil (ow1 t s))
("oaxaca" nil (ow1 k s ax k ax))
("oba" nil (ow1 b ax))
("obadiah" nil (ow1 b ax d ay1 ax))
("obando" nil (ow0 b ae1 n d ow0))
("obanion" nil (ow0 b aa0 n y ao1 n))
("obannon" nil (aa1 b ax n aa0 n))
("obar" nil (ax b aa1 r))
("obara" nil (ow0 b aa1 r ax))
("obarr" nil (ow0 b aa1 r))
("obedience" nil (ow0 b iy1 d iy0 ax n s))
("obedient" nil (ow0 b iy1 d iy0 ax n t))
("obediently" nil (ow0 b iy1 d iy0 ax n t l iy0))
("obeid" nil (ow0 b ay1 d))
("obeirne" nil (aa1 b ay0 r n))
("obelia" nil (ow0 b eh1 l iy0 ax))
("obenchain" nil (aa1 b ih0 n k ay0 n))
("obenshain" nil (aa1 b ih0 n sh ay0 n))
("ober" nil (ow1 b er0))
("oberbeck" nil (aa1 b er0 b eh0 k))
("oberdorf" nil (aa1 b er0 d ao0 r f))
("oberg" nil (ow1 b er0 g))
("oberhaus" nil (aa1 b er0 hh aw0 s))
("oberhausen" nil (ow1 b er0 hh aw1 z ax n))
("oberhelman" nil (aa1 b er0 hh ax l m ax n))
("oberholtzer" nil (aa1 b er0 hh ow0 l t z er0))
("oberlander" nil (aa1 b er0 l ax n d er0))
("oberle" nil (aa1 b er0 ax l))
("oberlin" nil (ow1 b er0 l ih0 n))
("oberly" nil (ow1 b er0 l iy0))
("obermaier" nil (ow1 b er0 m ay1 r))
("oberman" nil (aa1 b er0 m ax n))
("obermeier" nil (aa1 b er0 m ay0 er0))
("obermeyer" nil (aa1 b er0 m ay0 er0))
("obermiller" nil (aa1 b er0 m ax l er0))
("oberon" nil (ow1 b er0 aa1 n))
("oberry" nil (ow1 b eh0 r iy0))
("oberst" nil (aa1 b er0 s t))
("oberstar" nil (ow1 b er0 s t aa1 r))
("obert" nil (aa1 b er0 t))
("oberweis" nil (ow1 b er0 w ay1 s))
("obese" nil (ow0 b iy1 s))
("obesity" nil (ow0 b iy1 s ax t iy0))
("obey" nil (ow0 b ey1))
("obeyed" nil (ow0 b ey1 d))
("obeying" nil (ow0 b ey1 ih0 ng))
("obeys" nil (ow0 b ey1 z))
("obfuscate" nil (aa1 b f ax s k ey1 t))
("obfuscation" nil (aa1 b f ax s k ey1 sh ax n))
("obie" nil (ow1 b iy0))
("obispo" nil (ow0 b ih1 s p ow0))
("obit" nil (ow1 b ax t))
("obits" nil (aa1 b ih0 t s))
("obituaries" nil (ow0 b ih1 ch uw0 eh1 r iy0 z))
("obituary" nil (ow0 b ih1 ch uw0 eh1 r iy0))
("object" n (aa1 b jh eh0 k t))
("object" v (ax b jh eh1 k t))
("objected" nil (ax b jh eh1 k t ax d))
("objecting" nil (ax b jh eh1 k t ih0 ng))
("objection" nil (ax b jh eh1 k sh ax n))
("objectionable" nil (ax b jh eh1 k sh ax n ax b ax l))
("objections" nil (ax b jh eh1 k sh ax n z))
("objective" nil (ax b jh eh1 k t ih0 v))
("objectively" nil (aa0 b jh eh1 k t ih0 v l iy0))
("objectives" nil (ax b jh eh1 k t ih0 v z))
("objectivity" nil (aa1 b jh eh0 k t ih1 v ih0 t iy0))
("objector" nil (ax b jh eh1 k t er0))
("objectors" nil (ax b jh eh1 k t er0 z))
("objects" n (aa1 b jh eh0 k t s))
("objects" v (ax b jh eh1 k t s))
("oblak" nil (aa1 b l ax k))
("oblander" nil (aa1 b l ax n d er0))
("oblast" nil (aa1 b l ae0 s t))
("oblate" nil (aa0 b l ey1 t))
("obligate" nil (aa1 b l ax g ey1 t))
("obligated" nil (aa1 b l ax g ey1 t ih0 d))
("obligates" nil (aa1 b l ax g ey1 t s))
("obligation" nil (aa1 b l ax g ey1 sh ax n))
("obligations" nil (aa1 b l ax g ey1 sh ax n z))
("obligatory" nil (ax b l ih1 g ax t ao1 r iy0))
("oblige" nil (ax b l ay1 jh))
("obliged" nil (ax b l ay1 jh d))
("obliges" nil (ax b l ay1 jh ih0 z))
("obliging" nil (ax b l ay1 jh ih0 ng))
("obligingly" nil (ax b l ay1 jh ih0 ng l iy0))
("oblinger" nil (ow1 b ax l ih0 ng er0))
("oblique" nil (ax b l iy1 k))
("obliquely" nil (ow0 b l iy1 k l iy0))
("obliterate" nil (ax b l ih1 t er0 ey1 t))
("obliterated" nil (ax b l ih1 t er0 ey1 t ih0 d))
("oblivion" nil (ax b l ih1 v iy0 ax n))
("oblivious" nil (ax b l ih1 v iy0 ax s))
("oblong" nil (aa1 b l ao0 ng))
("obnoxious" nil (aa0 b n aa1 k sh ax s))
("oboe" nil (ow1 b ow0))
("oboist" nil (ow1 b ow0 ax s t))
("obon" nil (ow1 b ax n))
("oboyle" nil (aa1 b oy0 l))
("obradovich" nil (ax b r aa1 d ax v ih0 ch))
("obrecht" nil (aa1 b r ih0 k t))
("obregon" nil (aa1 b r ih0 g ax n))
("obremski" nil (ax b r eh1 m s k iy0))
("obrenovich" nil (ax b r eh1 n ax v ih0 ch))
("obrian" nil (aa1 b r iy0 ax n))
("obriant" nil (ow0 b r iy1 ax n t))
("obrien" nil (ow0 b r ay1 ax n))
("obringer" nil (aa1 b r ih0 ng er0))
("obrinsky" nil (ow0 b r ih1 n s k iy0))
("obryan" nil (ow0 b r ay1 ax n))
("obryant" nil (ow0 b r ay1 ax n t))
("obscene" nil (aa0 b s iy1 n))
("obscenely" nil (aa0 b s iy1 n ax l iy0))
("obscenities" nil (aa0 b s eh1 n ih0 t iy0 z))
("obscenity" nil (ax b s eh1 n ax t iy0))
("obscure" nil (ax b s k y uh1 r))
("obscured" nil (ax b s k y uh1 r d))
("obscures" nil (ax b s k y uh1 r z))
("obscuring" nil (ax b s k y uh1 r ih0 ng))
("obscurity" nil (ax b s k y uh1 r ax t iy0))
("obsequious" nil (ax b s iy1 k w iy0 ax s))
("observable" nil (ax b z er1 v ax b ax l))
("observance" nil (ax b z er1 v ax n s))
("observances" nil (ax b z er1 v ax n s ih0 z))
("observant" nil (ax b z er1 v ax n t))
("observateur" nil (aa0 b z er1 v ax t uh1 r))
("observation" nil (aa1 b z er0 v ey1 sh ax n))
("observations" nil (aa1 b z er0 v ey1 sh ax n z))
("observatories" nil (ax b z er1 v ax t ao1 r iy0 z))
("observatory" nil (ax b z er1 v ax t ao1 r iy0))
("observe" nil (ax b z er1 v))
("observed" nil (ax b z er1 v d))
("observer" nil (ax b z er1 v er0))
("observers" nil (ax b z er1 v er0 z))
("observes" nil (ax b z er1 v z))
("observing" nil (ax b z er1 v ih0 ng))
("obsessed" nil (ax b s eh1 s t))
("obsession" nil (ax b s eh1 sh ax n))
("obsessional" nil (ax b s eh1 sh ax n ax l))
("obsessions" nil (ax b s eh1 sh ax n z))
("obsessive" nil (ax b s eh1 s ih0 v))
("obsessively" nil (aa0 b s eh1 s ih0 v l iy0))
("obsidian" nil (ax b s ih1 d iy0 ax n))
("obsolescence" nil (aa1 b s ax l eh1 s ax n s))
("obsolescent" nil (aa1 b s ax l eh1 s ax n t))
("obsolete" nil (aa1 b s ax l iy1 t))
("obst" nil (aa1 b s t))
("obstacle" nil (aa1 b s t ax k ax l))
("obstacles" nil (aa1 b s t ax k ax l z))
("obstetrician" nil (aa1 b s t ax t r ih1 sh ax n))
("obstetricians" nil (aa1 b s t ax t r ih1 sh ax n z))
("obstetrics" nil (ax b s t eh1 t r ih0 k s))
("obstfeld" nil (aa1 b s t f eh1 l d))
("obstinacy" nil (aa1 b s t ax n ax s iy0))
("obstinate" nil (aa1 b s t ax n ax t))
("obstruct" nil (ax b s t r ah1 k t))
("obstructed" nil (ax b s t r ah1 k t ih0 d))
("obstructing" nil (ax b s t r ah1 k t ih0 ng))
("obstruction" nil (ax b s t r ah1 k sh ax n))
("obstructionism" nil (ax b s t r ah1 k sh ax n ih1 z ax m))
("obstructionist" nil (ax b s t r ah1 k sh ax n ax s t))
("obstructionists" nil (ax b s t r ah1 k sh ax n ax s t s))
("obstructive" nil (ax b s t r ah1 k t ih0 v))
("obtain" nil (ax b t ey1 n))
("obtainable" nil (ax b t ey1 n ax b ax l))
("obtained" nil (ax b t ey1 n d))
("obtaining" nil (ax b t ey1 n ih0 ng))
("obtains" nil (ax b t ey1 n z))
("obtrudes" nil (ax b t r uw1 d z))
("obtrusive" nil (ax b t r uw1 s ih0 v))
("obtuse" nil (aa0 b t uw1 s))
("obuchowski" nil (ax b ax hh ao1 f s k iy0))
("obverse" nil (ax b v er1 s))
("obviate" nil (aa1 b v iy0 ey1 t))
("obviating" nil (aa1 b v iy0 ey1 t ih0 ng))
("obvious" nil (aa1 b v iy0 ax s))
("obviously" nil (aa1 b v iy0 ax s l iy0))
("obyrne" nil (aa1 b ay0 r n))
("ocain" nil (ow0 k aa0 iy1 n))
("ocala" nil (ow0 k ae1 l ax))
("ocallaghan" nil (aa0 k ax l ae1 g hh ax n))
("ocampo" nil (ow0 k ae1 m p ow0))
("ocana" nil (ow0 k ae1 n ax))
("ocanas" nil (ow0 k ae1 n ax z))
("ocarroll" nil (ow0 k aa0 r ow1 l))
("ocasio" nil (ow0 k aa1 s iy0 ow0))
("ocaw" nil (ow0 k ao1))
("occasion" nil (ax k ey1 zh ax n))
("occasional" nil (ax k ey1 zh ax n ax l))
("occasionally" nil (ax k ey1 zh ax n ax l iy0))
("occasioned" nil (ax k ey1 zh ax n d))
("occasions" nil (ax k ey1 zh ax n z))
("occhino" nil (ow1 k iy1 n ow0))
("occhipinti" nil (ow0 k iy0 p iy1 n t iy0))
("occhoa" nil (ow1 ch ow1 ax))
("occidental" nil (aa1 k s ax d eh1 n t ax l))
("occidentale" nil (aa1 k s ih0 d eh1 n t ax l))
("occipital" nil (aa0 k s ih1 p ax t ax l))
("occlusion" nil (ax k l uw1 zh ax n))
("occult" nil (ax k ah1 l t))
("occupancy" nil (aa1 k y ax p ax n s iy0))
("occupant" nil (aa1 k y ax p ax n t))
("occupants" nil (aa1 k y ax p ax n t s))
("occupation" nil (aa1 k y ax p ey1 sh ax n))
("occupational" nil (aa0 k y ax p ey1 sh ax n ax l))
("occupations" nil (aa1 k y ax p ey1 sh ax n z))
("occupied" nil (aa1 k y ax p ay1 d))
("occupies" nil (aa1 k y ax p ay1 z))
("occupy" nil (aa1 k y ax p ay1))
("occupying" nil (aa1 k y ax p ay1 ih0 ng))
("occur" nil (ax k er1))
("occured" nil (ax k er1 d))
("occuring" nil (ax k er1 ih0 ng))
("occurred" nil (ax k er1 d))
("occurrence" nil (ax k er1 ax n s))
("occurrences" nil (ax k er1 ax n s ih0 z))
("occurring" nil (ax k er1 ih0 ng))
("occurs" nil (ax k er1 z))
("ocean" nil (ow1 sh ax n))
("oceaneering" nil (ow1 sh ax n ih1 r ih0 ng))
("oceanfront" nil (ow1 sh ax n f r ah1 n t))
("oceangoing" nil (ow1 sh ax n g ow1 ih0 ng))
("oceanic" nil (ow1 sh iy0 ae1 n ih0 k))
("oceanographic" nil (ow1 sh ax n ax g r ae1 f ih0 k))
("oceanography" nil (ow1 sh ax n aa1 g r ax f iy0))
("oceans" nil (ow1 sh ax n z))
("oceanside" nil (ow1 sh ax n s ay1 d))
("oceanview" nil (ow1 sh ax n v uw1))
("ocelot" nil (aa1 s ax l aa1 t))
("och" nil (aa1 k))
("ocheltree" nil (aa0 ch ih0 l t r iy1))
("ochoa" nil (aa1 ch ow1 ax))
("ochra" nil (aa1 k r ax))
("ochre" nil (ow1 k er0))
("ochs" nil (aa1 k s))
("ochsenschlager" nil (aa1 k s ax n sh l aa1 g er0))
("ochsner" nil (aa1 k s n er0))
("ocilla" nil (ow0 s ih1 l ax))
("ocker" nil (aa1 k er0))
("ockerbloom" nil (aa1 k er0 b l uw0 m))
("ockerman" nil (aa1 k er0 m ax n))
("oclair" nil (aa1 k l er0))
("ocon" nil (ax k aa1 n))
("oconnell" nil (ow0 k aa1 n ax l))
("oconner" nil (aa1 k ax n er0))
("oconnor" nil (ow0 k aa1 n er0))
("oct" nil (ao0 k t ow1 b er0))
("octagon" nil (aa1 k t ax g aa1 n))
("octagonal" nil (aa0 k t ae1 g ax n ax l))
("octahedral" nil (aa1 k t ax hh iy1 d r ax l))
("octane" nil (aa1 k t ey0 n))
("octave" nil (aa1 k t ih0 v))
("octaves" nil (aa1 k t ih0 v z))
("octavia" nil (aa0 k t ey1 v iy0 ax))
("octavio" nil (aa1 k t ey1 v iy0 ow0))
("octavius" nil (aa0 k t ey1 v iy0 ax s))
("octavus" nil (aa1 k t ax v uw0 s))
("octel" nil (aa1 k t eh1 l))
("octet" nil (aa0 k t eh1 t))
("octillion" nil (aa0 k t ih1 l y ax n))
("october" nil (aa0 k t ow1 b er0))
("octogenarian" nil (aa1 k t ax jh ih0 n eh1 r iy0 ax n))
("octopus" nil (aa1 k t ax p uh1 s))
("oda" nil (ow1 d ax))
("odaiko" nil (ow0 d ey1 k ow0))
("odaniel" nil (aa1 d ax n iy0 l))
("oday" nil (ow1 d ey0))
("odd" nil (aa1 d))
("oddball" nil (aa1 d b ao1 l))
("oddballs" nil (aa1 d b ao1 l z))
("odden" nil (aa1 d ax n))
("odder" nil (aa1 d er0))
("oddest" nil (aa1 d ax s t))
("oddities" nil (aa1 d ax t iy0 z))
("oddity" nil (aa1 d ax t iy0))
("oddler" nil (aa1 d l er0))
("oddly" nil (aa1 d l iy0))
("odds" nil (aa1 d z))
("oddsmakers" nil (aa1 d z m ey1 k er0 z))
("oddy" nil (aa1 d iy0))
("ode" nil (ow1 d))
("odea" nil (aa1 d iy0 ax))
("odegaard" nil (aa1 d ih0 g aa0 r d))
("odegard" nil (aa1 d ih0 g er0 d))
("odekirk" nil (aa1 d ih0 k er0 k))
("odele" nil (ow0 d eh1 l iy0))
("odelet" nil (aa1 d ih0 l ih0 t))
("odelette" nil (aa1 d ih0 l eh0 t))
("odelia" nil (ow0 d eh1 l iy0 ax))
("odelinda" nil (ow0 d eh0 l iy1 n d ax))
("odell" nil (ow0 d eh1 l))
("odella" nil (ow0 d eh1 l ax))
("odem" nil (ow1 d ih0 m))
("oden" nil (ow1 d ax n))
("odenthal" nil (aa1 d ih0 n th ax l))
("odeon" nil (ow1 d iy0 ax n))
("oder" nil (ow1 d er0))
("odes" nil (ow1 d z))
("odessa" nil (ow0 d eh1 s ax))
("odetics" nil (ow0 d eh1 t ih0 k s))
("odette" nil (ow1 d eh1 t))
("odgers" nil (aa1 jh er0 z))
("odier" nil (ow1 d iy0 er0))
("odilia" nil (ow0 d iy1 l iy0 ax))
("odin" nil (ow1 d ax n))
("odiorne" nil (ow0 d iy0 ao1 r n iy0))
("odious" nil (ow1 d iy0 ax s))
("odland" nil (aa1 d l ax n d))
("odle" nil (ow1 d ax l))
("odneal" nil (aa1 d n ax l))
("odoherty" nil (aa1 d ax hh er0 t iy0))
("odolf" nil (aa1 d ow0 l f))
("odom" nil (ow1 d ax m))
("odometer" nil (ow1 d aa1 m ax t er0))
("odometers" nil (ow1 d aa1 m ax t er0 z))
("odoms" nil (ow1 d ax m z))
("odonald" nil (aa1 d ax n ao0 l d))
("odonnel" nil (aa1 d ax n eh0 l))
("odonnell" nil (ow0 d aa1 n ax l))
("odonoghue" nil (aa1 d ax n aa0 g hh uw0))
("odonohue" nil (aa1 d ax n aa0 hh y uw0))
("odonovan" nil (aa0 d ax n ow1 v ax n))
("odor" nil (ow1 d er0))
("odorant" nil (ow1 d er0 ax n t))
("odorless" nil (ow1 d er0 l ax s))
("odorous" nil (ow1 d er0 ax s))
("odors" nil (ow1 d er0 z))
("odowd" nil (aa1 d aw0 d))
("odp" nil (ow1 d iy1 p iy1))
("odriscoll" nil (aa1 d r ih0 s k aa0 l))
("odum" nil (ow1 d ax m))
("odwyer" nil (aa1 d w iy0 er0))
("odysseus" nil (ow0 d ih1 s iy0 ax s))
("odyssey" nil (aa1 d ax s iy0))
("oedipal" nil (eh1 d ax p ax l))
("oedipus" nil (ow0 d iy1 p ax s))
("oehler" nil (ow1 l er0))
("oehlert" nil (ow1 l er0 t))
("oehlke" nil (ow1 l k))
("oehme" nil (ow1 m))
("oehmen" nil (ow1 m ax n))
("oehmens" nil (ow1 m ax n z))
("oehrlein" nil (ao1 r l ay0 n))
("oeien" nil (ow1 iy0 ax n))
("oelke" nil (ow1 l k))
("oelkers" nil (ow1 l k er0 z))
("oelman" nil (ow1 l m ax n))
("oelrich" nil (ow1 l r ih0 k))
("oelschlager" nil (ow1 l sh l ey0 g er0))
("oerlikon" nil (ao1 r l ih0 k aa1 n))
("oertel" nil (ao1 r t ax l))
("oesch" nil (ow1 sh))
("oeser" nil (ow1 z er0))
("oest" nil (ow1 ih0 s t))
("oesterle" nil (ow1 s t er0 ax l))
("oesterling" nil (ow1 s t er0 l ih0 ng))
("oesterreich" nil (ow1 s t er0 ay0 k))
("oesterreichische" nil (ow1 s t er0 ay1 k ih0 sh iy0))
("oestreich" nil (ow1 s t r ay1 k))
("oestreicher" nil (ow1 s t r ay1 k er0))
("oetken" nil (ow1 t k ax n))
("oetting" nil (ow1 t ih0 ng))
("oettinger" nil (ow1 t ih0 n jh er0))
("oeuvre" nil (uw1 v r ax))
("of" nil (ah1 v))
("ofallon" nil (aa1 f ax l aa0 n))
("ofarrell" nil (aa1 f er0 eh0 l))
("ofc" nil (ow1 eh1 f s iy1))
("ofelia" nil (ow0 f ey1 l iy0 ax))
("ofer" nil (ow1 f er0))
("off" nil (ao1 f))
("offbeat" nil (ao1 f b iy1 t))
("offen" nil (ao1 f ax n))
("offend" nil (ax f eh1 n d))
("offended" nil (ax f eh1 n d ax d))
("offender" nil (ax f eh1 n d er0))
("offenders" nil (ax f eh1 n d er0 z))
("offending" nil (ax f eh1 n d ih0 ng))
("offends" nil (ax f eh1 n d z))
("offense" nil (ax f eh1 n s))
("offenses" nil (ax f eh1 n s ih0 z))
("offensive" nil (ax f eh1 n s ih0 v))
("offensively" nil (ax f eh1 n s ih0 v l iy0))
("offensives" nil (ax f eh1 n s ih0 v z))
("offer" nil (ao1 f er0))
("offerdahl" nil (aa1 f er0 d aa0 l))
("offered" nil (ao1 f er0 d))
("offerer" nil (ao1 f er0 er0))
("offering" nil (ao1 f er0 ih0 ng))
("offerings" nil (ao1 f er0 ih0 ng z))
("offerman" nil (ao1 f er0 m ax n))
("offermann" nil (ao1 f er0 m ax n))
("offers" nil (ao1 f er0 z))
("offhand" nil (ao1 f hh ae1 n d))
("offical" nil (ax f ih0 sh ax l))
("officals" nil (ao1 f ih0 k ax l z))
("office" nil (ao1 f ax s))
("officeholder" nil (ao1 f ax s hh ow1 l d er0))
("officeholders" nil (ao1 f ax s hh ow1 l d er0 z))
("officemax" nil (ao1 f ax s m ae1 k s))
("officer" nil (ao1 f ax s er0))
("officers" nil (ao1 f ax s er0 z))
("offices" nil (ao1 f ax s ax z))
("official" nil (ax f ih1 sh ax l))
("officialdom" nil (ax f ih1 sh ax l d ax m))
("officially" nil (ax f ih1 sh ax l iy0))
("officials" nil (ax f ih1 sh ax l z))
("officiate" nil (ax f ih1 sh iy0 ey1 t))
("officiated" nil (ax f ih1 sh iy0 ey1 t ax d))
("officiating" nil (ax f ih1 sh iy0 ey1 t ih0 ng))
("officio" nil (ax f ih1 s iy0 ow0))
("offield" nil (aa1 f iy1 l d))
("offill" nil (ao1 f ih1 l))
("offing" nil (ao1 f ih0 ng))
("offner" nil (aa1 f n er0))
("offord" nil (aa1 f er0 d))
("offs" nil (ao1 f s))
("offset" nil (ao0 f s eh1 t))
("offsets" nil (ao1 f s eh1 t s))
("offsetting" nil (ao0 f s eh1 t ih0 ng))
("offshoot" nil (ao1 f sh uw1 t))
("offshoots" nil (ao1 f sh uw1 t s))
("offshore" nil (ao1 f sh ao1 r))
("offspring" nil (ao1 f s p r ih1 ng))
("offstage" nil (ao1 f s t ey1 jh))
("offutt" nil (aa1 f ax t))
("ofilia" nil (ow0 f iy1 l iy0 ax))
("oflaherty" nil (aa1 f l ax hh er0 t iy0))
("oflynn" nil (aa1 f l ih0 n))
("oft" nil (ao1 f t))
("oftedahl" nil (aa1 f t ih0 d aa0 l))
("oftel" nil (aa1 f t eh1 l))
("often" nil (ao1 f ax n))
("oftener" nil (ao1 f ax n er0))
("oftentimes" nil (ao1 f ax n t ay1 m z))
("og" nil (aa1 g))
("ogan" nil (ow1 g ax n))
("ogara" nil (ow0 g aa1 r ax))
("ogata" nil (ow0 g aa1 t ax))
("ogawa" nil (ow0 g aa1 w ax))
("ogborn" nil (aa1 g b er0 n))
("ogburn" nil (aa1 g b er0 n))
("ogden" nil (aa1 g d ax n))
("ogg" nil (aa1 g))
("ogier" nil (ow1 g iy0 er0))
("ogilvie" nil (aa1 jh ih0 l v iy0))
("ogilvy" nil (ow1 g ax l v iy0))
("ogle" nil (ow1 g ax l))
("oglebay" nil (ow1 g ax l b ey1))
("ogled" nil (ow1 g ax l d))
("ogles" nil (ow1 g ax l z))
("oglesbee" nil (ow1 g ax l z b iy1))
("oglesby" nil (aa1 g l ih0 s b iy0))
("oglethorpe" nil (ow1 g ax l th ao1 r p))
("ogletree" nil (ow1 g ax l t r iy1))
("ognibene" nil (ow0 g n iy0 b eh1 n ax))
("ogonyok" nil (ow0 g ow1 n y aa0 k))
("ogorman" nil (aa1 g er0 m ax n))
("ogrady" nil (ax g r aa1 d iy0))
("ogre" nil (ow1 g er0))
("ogren" nil (aa1 g r eh0 n))
("ogress" nil (ow1 g r ax s))
("oguin" nil (ow0 g uw1 iy0 n))
("oguinn" nil (aa1 g ih0 n))
("oh" nil (ow1))
("ohagan" nil (ow0 hh aa1 g aa0 n))
("ohair" nil (aa1 hh er0))
("ohalloran" nil (aa0 hh ax l ao1 r ax n))
("ohanesian" nil (aa0 hh ax n eh1 zh ih0 n))
("ohanian" nil (ax hh ey1 n iy0 ax n))
("ohanlon" nil (ax hh ae1 n l ax n))
("ohare" nil (ow0 hh aa1 r ey0))
("oharra" nil (aa1 hh er0 ax))
("ohashi" nil (ow0 hh aa1 sh iy0))
("ohaver" nil (aa1 hh ax v er0))
("ohba" nil (ow1 b ax))
("ohbayashi" nil (ow1 b aa0 y aa1 sh iy0))
("ohearn" nil (aa1 hh er0 n))
("ohern" nil (aa1 hh er0 n))
("oherron" nil (aa1 hh er0 aa0 n))
("ohio" nil (ow0 hh ay1 ow0))
("ohioans" nil (ow1 hh ay1 ow1 ax n z))
("ohkawara" nil (ow1 k aa1 w aa1 r ax))
("ohl" nil (ow1 l))
("ohland" nil (ow1 l ax n d))
("ohlendorf" nil (ow1 l ih0 n d ao0 r f))
("ohler" nil (ow1 l er0))
("ohlin" nil (ow1 l ih0 n))
("ohlinger" nil (ow1 l ih0 ng er0))
("ohlman" nil (ow1 l m ax n))
("ohlmann" nil (ow1 l m ax n))
("ohlmeyer" nil (ow1 l m ay1 r))
("ohlrich" nil (ow1 l r ih0 k))
("ohlsen" nil (ow1 l s ax n))
("ohlson" nil (ow1 l s ax n))
("ohlsson" nil (ow1 l s ax n))
("ohm" nil (ow1 m))
("ohmae" nil (ow1 m ey1))
("ohman" nil (ow1 m ax n))
("ohmann" nil (ow1 m ax n))
("ohmer" nil (ow1 m er0))
("ohms" nil (ow1 m z))
("ohmura" nil (ow0 m uw1 r ax))
("ohnemus" nil (ow1 n ih0 m ih0 s))
("ohnstad" nil (ow1 n s t ax d))
("ohr" nil (ao1 r))
("ohrt" nil (ao1 r t))
("ohs" nil (ow1 z))
("oi" nil (oy1))
("oie" nil (oy1))
("oien" nil (aa1 iy0 n))
("oil" nil (oy1 l))
("oiled" nil (oy1 l d))
("oiler" nil (oy1 l er0))
("oilers" nil (oy1 l er0 z))
("oilfield" nil (oy1 l f iy1 l d))
("oilfields" nil (oy1 l f iy1 l d z))
("oiling" nil (oy1 l ih0 ng))
("oilman" nil (oy1 l m ae1 n))
("oilmen" nil (oy1 l m eh1 n))
("oils" nil (oy1 l z))
("oilseed" nil (oy1 l s iy1 d))
("oilseeds" nil (oy1 l s iy1 d z))
("oilwell" nil (oy1 l w eh1 l))
("oily" nil (oy1 l iy0))
("ointment" nil (oy1 n t m ax n t))
("oishi" nil (ow0 iy1 sh iy0))
("oja" nil (ow1 y ax))
("ojala" nil (ow0 y aa1 l ax))
("ojeda" nil (ow0 y ey1 d ax))
("oji" nil (ow1 jh iy0))
("ojibwa" nil (ow0 jh ih1 b w ax))
("ok" nil (ow1 k ey1))
("oka" nil (ow1 k ax))
("okabe" nil (ow0 k aa1 b ey0))
("okada" nil (ow0 k aa1 d ax))
("okamoto" nil (ow1 k ax m ow1 t ow0))
("okamura" nil (ow1 k ax m uh1 r ax))
("okane" nil (ow0 k aa1 n ey0))
("okano" nil (ow0 k aa1 n ow0))
("okasaki" nil (ow1 k ax s aa1 k iy0))
("okasan" nil (ow0 k ey1 z ax n))
("okay" nil (ow1 k ey1))
("okayed" nil (ow1 k ey1 ax d))
("okays" nil (ow1 k ey1 z))
("okazaki" nil (ow0 k aa0 z aa1 k iy0))
("okeechobee" nil (ow0 k iy1 ch ow1 b iy0))
("okeefe" nil (aa1 k iy0 f))
("okeeffe" nil (aa1 k iy0 f))
("okelley" nil (aa1 k ih0 l iy0))
("okelly" nil (aa1 k ax l iy0))
("oken" nil (ow0 k ey1 ax n))
("okerlund" nil (aa1 k er0 l ax n d))
("okeson" nil (aa1 k ih0 s ax n))
("okey" nil (ow1 k iy0))
("oki" nil (ow1 k iy0))
("okie" nil (ow1 k iy0))
("okimoto" nil (ow0 k iy0 m ow1 t ow0))
("okin" nil (ow1 k ih0 n))
("okinawa" nil (ow1 k ih1 n aa1 w ax))
("okinawans" nil (ow1 k ih1 n aa1 w ax n z))
("oklahoma" nil (ow1 k l ax hh ow1 m ax))
("okon" nil (ow1 k ow0 n))
("okoniewski" nil (ow0 k aa1 n iy0 eh1 f s k iy0))
("okonski" nil (ax k aa1 n s k iy0))
("okpo" nil (aa1 k p ow0))
("okra" nil (ow1 k r ax))
("okray" nil (aa1 k r ey0))
("oksana" nil (aa1 k s ae1 n ax))
("oksenberg" nil (aa1 k s ax n b er0 g))
("okubo" nil (ow0 k uw1 b ow0))
("okuda" nil (ow0 k uw1 d ax))
("okuma" nil (ow0 k y uw1 m er0))
("okumura" nil (ow0 k uw0 m uh1 r ax))
("okun" nil (ow1 k uw0 n))
("okuno" nil (ow0 k y uw1 n ow0))
("okura" nil (ow0 k uw1 r ax))
("okurowski" nil (ow0 k er0 ow1 s k iy0))
("ola" nil (ow1 l ax))
("olack" nil (ow1 l ae0 k))
("olaf" nil (ow1 l aa0 f))
("olafson" nil (aa1 l ax f s ax n))
("olague" nil (ow1 l aa0 g))
("olah" nil (ax l aa1))
("olajuwon" nil (ax l ay1 jh uw0 aa1 n))
("oland" nil (aa1 l ax n d))
("olander" nil (aa1 l ax n d er0))
("olano" nil (ax l aa1 n ow0))
("olathe" nil (ow0 l ae1 th iy0))
("olaughlin" nil (ax l ao1 k l ih0 n))
("olay" nil (ow0 l ey1))
("olayan" nil (ow0 l ay1 ax n))
("olberding" nil (ow1 l b er0 d ih0 ng))
("olbrich" nil (ow1 l b r ih0 k))
("olcott" nil (ow1 l k ax t))
("olczak" nil (ow1 l ch ae0 k))
("old" nil (ow1 l d))
("oldaker" nil (ow1 l d ax k er0))
("oldani" nil (ow0 l d aa1 n iy0))
("olde" nil (ow1 l d))
("olden" nil (ow1 l d ax n))
("oldenburg" nil (ow1 l d ax n b er0 g))
("oldenkamp" nil (ow1 l d ih0 n k ae0 m p))
("older" nil (ow1 l d er0))
("oldest" nil (ow1 l d ax s t))
("oldfather" nil (ow1 l d f aa1 dh er0))
("oldfield" nil (ow1 l d f iy1 l d))
("oldham" nil (ow1 l d ax m))
("oldies" nil (ow1 l d iy0 z))
("oldroyd" nil (ow1 l d r oy1 d))
("olds" nil (ow1 l d z))
("oldsmobile" nil (ow1 l d z m ow0 b iy1 l))
("oldsmobiles" nil (ow1 l d z m ow0 b iy1 l z))
("oldsters" nil (ow1 l d s t er0 z))
("oldt" nil (ow1 l t))
("ole" nil (ow1 l))
("olea" nil (aa1 l iy0 ax))
("olean" nil (ow0 l iy1 n))
("oleander" nil (ow1 l iy0 ae1 n d er0))
("olear" nil (aa1 l er0))
("oleary" nil (aa1 l er0 iy0))
("oleaster" nil (ow1 l iy0 ae1 s t er0))
("olefins" nil (ow1 l ax f ih0 n z))
("oleg" nil (ow1 l ax g))
("olejniczak" nil (ax l ey1 n ih0 ch ae0 k))
("olejnik" nil (ax l ey1 n ih0 k))
("oleksiak" nil (ax l eh1 k s iy0 ae0 k))
("oleksy" nil (ax l eh1 k s iy0))
("olen" nil (aa1 l ax n))
("olender" nil (aa1 l eh0 n d er0))
("olenick" nil (aa1 l ih0 n ih0 k))
("olenik" nil (aa1 l ih0 n ih0 k))
("oleo" nil (ow1 l iy0 ow1))
("oler" nil (ow1 l er0))
("oles" nil (ow1 l z))
("olesen" nil (aa1 l iy0 z ax n))
("oleske" nil (ow1 l ax s k iy0))
("olesky" nil (ax l eh1 s k iy0))
("oleson" nil (aa1 l ih0 s ax n))
("olestra" nil (ow0 l eh1 s t r ax))
("olexa" nil (ax l iy1 k s ax))
("oley" nil (ow1 l iy0))
("olfactory" nil (ow0 l f ae1 k t er0 iy0))
("olga" nil (ow1 l g ax))
("olguin" nil (ow1 l g ih0 n))
("olick" nil (ow1 l ih0 k))
("oliff" nil (aa1 l ih0 f))
("oligarch" nil (ow1 l ih0 g aa1 r k))
("oligarchy" nil (aa1 l ax g aa1 r k iy0))
("oliger" nil (aa1 l ih0 g er0))
("oligocene" nil (aa1 l ax g ow0 s iy1 n))
("oligopolistic" nil (ow0 l ih1 g ax p ow0 l ih1 s t ih0 k))
("oligopoly" nil (aa1 l ih0 g aa1 p ax l iy0))
("olin" nil (ow1 l ih0 n))
("olinda" nil (ow0 l iy1 n d ax))
("olinde" nil (aa1 l ih0 n d))
("olinger" nil (aa1 l ih0 ng er0))
("oliphant" nil (aa1 l ih0 f ax n t))
("oliva" nil (ow0 l iy1 v ax))
("olivares" nil (ow0 l iy0 v aa1 r eh0 s))
("olivarez" nil (ow0 l iy0 v aa1 r eh0 z))
("olivas" nil (ow0 l iy1 v aa0 z))
("olive" nil (aa1 l ax v))
("oliveira" nil (aa1 l ih0 v ey1 r ax))
("oliver" nil (aa1 l ax v er0))
("olivera" nil (ow0 l iy0 v eh1 r ax))
("oliveras" nil (ow0 l iy0 v eh1 r aa0 z))
("oliveri" nil (ow0 l iy0 v eh1 r iy0))
("oliveria" nil (aa1 l ih0 v iy1 r iy0 ax))
("oliverio" nil (aa1 l ih0 v iy1 r iy0 ow0))
("olivero" nil (ow0 l iy0 v eh1 r ow0))
("oliveros" nil (ow0 l iy0 v eh1 r ow0 z))
("olives" nil (aa1 l ih0 v z))
("oliveto" nil (ow0 l iy0 v ey1 t ow0))
("olivette" nil (aa1 l ih0 v eh1 t))
("olivetti" nil (aa1 l ih0 v eh1 t iy0))
("olivia" nil (ow0 l ih1 v iy0 ax))
("olivier" nil (ow1 l ih1 v iy1 ey1))
("olivieri" nil (ow0 l iy0 v ih1 r iy0))
("olivine" nil (aa1 l ax v iy1 n))
("olivo" nil (ao0 l iy1 v ow0))
("olk" nil (ow1 k))
("olkowski" nil (ow0 l k ao1 f s k iy0))
("ollar" nil (aa1 l er0))
("oller" nil (aa1 l er0))
("olley" nil (aa1 l iy0))
("ollie" nil (aa1 l iy0))
("olliff" nil (aa1 l ih0 f))
("ollila" nil (aa1 l ih0 l ax))
("ollinger" nil (aa1 l ih0 ng er0))
("ollis" nil (ao1 l iy0 z))
("ollison" nil (aa1 l ih0 s ax n))
("olly" nil (aa1 l iy0))
("olmeda" nil (ow0 l m ey1 d ax))
("olmedo" nil (ow0 l m ey1 d ow0))
("olmo" nil (ow1 l m ow0))
("olmos" nil (ow1 l m ow0 z))
("olmstead" nil (ow1 l m s t eh1 d))
("olney" nil (ow1 l n iy0))
("olof" nil (ow1 l ao0 f))
("olofson" nil (aa1 l ax f s ax n))
("olokuei" nil (ow1 l ow0 k uw1 iy0))
("oloughlin" nil (ax l aw1 k l ih0 n))
("olsen" nil (ow1 l s ax n))
("olshan" nil (ow1 l sh ax n))
("olshansky" nil (ow1 l sh ax n s k iy0))
("olshefski" nil (ow0 l sh eh1 f s k iy0))
("olsher" nil (ow1 l sh er0))
("olson" nil (ow1 l s ax n))
("olsson" nil (ow1 l s ax n))
("olstad" nil (ow1 l s t ax d))
("olsten" nil (ow1 l s t ax n))
("olszewski" nil (ow0 l sh eh1 f s k iy0))
("olt" nil (ow1 l t))
("olthoff" nil (ow1 l t hh ao1 f))
("oltman" nil (ow1 l t m ax n))
("oltmann" nil (ow1 l t m ax n))
("oltmanns" nil (ow1 l t m ax n z))
("olund" nil (aa1 l ax n d))
("olva" nil (ow1 l v ax))
("olver" nil (ow1 l v er0))
("olvera" nil (ow0 l v eh1 r ax))
("olvey" nil (ow1 l v iy0))
("olympia" nil (ow0 l ih1 m p iy0 ax))
("olympiad" nil (ow0 l ih1 m p iy0 ae1 d))
("olympian" nil (ow0 l ih1 m p iy0 ax n))
("olympians" nil (ow0 l ih1 m p iy0 ax n z))
("olympias" nil (ow0 l ih1 m p iy0 ax s))
("olympic" nil (ow0 l ih1 m p ih0 k))
("olympics" nil (ow0 l ih1 m p ih0 k s))
("olympus" nil (ow0 l ih1 m p ax s))
("oma" nil (ow1 m ax))
("omaha" nil (ow1 m ax hh aa1))
("omahoney" nil (aa1 m ax hh aa0 n iy0))
("omahony" nil (aa1 m ax hh ax n iy0))
("omalley" nil (aa1 m ax l iy0))
("oman" nil (ow0 m aa1 n))
("omani" nil (ow0 m aa1 n iy0))
("omans" nil (ow0 m aa1 n z))
("omar" nil (ow1 m aa0 r))
("omara" nil (ow0 m aa1 r ax))
("omary" nil (ax m eh1 r iy0))
("ombudsman" nil (aa1 m b ax d z m ax n))
("omdahl" nil (aa1 m d aa1 l))
("omead" nil (ow0 m iy1 d))
("omeara" nil (ow1 m iy1 r ax))
("omega" nil (ow0 m eh1 g ax))
("omelet" nil (aa1 m l ax t))
("omelets" nil (aa1 m l ax t s))
("omelia" nil (ow0 m eh1 l iy0 ax))
("omen" nil (ow1 m ax n))
("omens" nil (ow1 m ax n z))
("omer" nil (ow1 m er0))
("ominous" nil (aa1 m ax n ax s))
("ominously" nil (aa1 m ax n ax s l iy0))
("omission" nil (ow0 m ih1 sh ax n))
("omissions" nil (ow0 m ih1 sh ax n z))
("omit" nil (ow0 m ih1 t))
("omits" nil (ow0 m ih1 t s))
("omitted" nil (ow0 m ih1 t ax d))
("omitting" nil (ow0 m ih1 t ih0 ng))
("omlor" nil (aa1 m l er0))
("ommen" nil (aa1 m ax n))
("omni" nil (aa1 m n iy0))
("omnibank" nil (aa1 m n ih0 b ae1 ng k))
("omnibook" nil (aa1 m n ih0 b uh1 k))
("omnibus" nil (aa1 m n ax b ax s))
("omnibuses" nil (aa1 m n ax b ax s ax z))
("omnicare" nil (aa1 m n ih0 k eh1 r))
("omnicom" nil (aa1 m n ih0 k aa0 m))
("omnipotence" nil (aa0 m n ih1 p ax t ax n s))
("omnipotent" nil (aa0 m n ih1 p ax t ax n t))
("omnipresence" nil (aa1 m n ax p r eh1 z ax n s))
("omnipresent" nil (aa1 m n ih0 p r eh1 z ax n t))
("omniscient" nil (aa0 m n ih1 sh ax n t))
("omnivorous" nil (aa0 m n ih1 v er0 ax s))
("omohundro" nil (ow0 m ow0 hh uw1 n d r ow0))
("omonia" nil (ax m ow1 n iy0 ax))
("omori" nil (ow0 m ao1 r iy0))
("omron" nil (aa1 m r ax n))
("on" nil (aa1 n))
("ona" nil (ax n aa1))
("onagers" nil (aa1 n ax jh er0 z))
("onan" nil (ow1 n ax n))
("onassis" nil (ow0 n ae1 s ih0 s))
("onate" nil (ow1 n ey0 t))
("onawa" nil (ow0 n aa1 w ax))
("onboard" nil (aa1 n b ao1 r d))
("oncale" nil (ow0 n k aa1 l iy0))
("once" nil (w ah1 n s))
("oncogen" nil (aa1 ng k ow0 g ax n))
("oncogene" nil (aa1 ng k ow0 jh iy1 n))
("oncogenes" nil (aa0 ng k aa1 jh eh1 n iy0 s))
("oncologist" nil (aa0 ng k aa1 l ax jh ih0 s t))
("oncologists" nil (aa0 ng k aa1 l ax jh ih0 s t s))
("oncology" nil (aa0 ng k aa1 l ax jh iy0))
("oncoming" nil (ao1 n k ah1 m ih0 ng))
("oncor" nil (aa1 n k ao1 r))
("ondaatje" nil (aa0 n d aa1 t y ax))
("onder" nil (aa1 n d er0))
("onderdonk" nil (aa1 n d er0 d ax ng k))
("ondo" nil (ao1 n d ow0))
("ondracek" nil (aa1 n d r ax s eh0 k))
("one" nil (w ah1 n))
("oneal" nil (ow0 n iy1 l))
("oneall" nil (aa1 n ax l))
("onecomm" nil (w ah1 n k aa1 m))
("oneida" nil (ow0 n ay1 d ax))
("oneil" nil (ow0 n iy1 l))
("oneill" nil (ow0 n iy1 l))
("oneness" nil (w ah1 n n ax s))
("oneok" nil (ow1 n iy0 aa0 k))
("onerous" nil (ow1 n er0 ax s))
("ones" nil (w ah1 n z))
("oneself" nil (w ah1 n s eh1 l f))
("onetime" nil (w ah1 n t ay1 m))
("onex" nil (w ah1 n eh1 k s))
("oney" nil (ow1 n iy0))
("oneyear" nil (w ah1 n y ih1 r))
("ong" nil (ao1 ng))
("ongoing" nil (aa1 n g ow1 ih0 ng))
("ongpin" nil (ao1 ng p ih0 n))
("onion" nil (ah1 n y ax n))
("onions" nil (ah1 n y ax n z))
("onishi" nil (ow0 n iy1 sh iy0))
("onken" nil (aa1 ng k ax n))
("onley" nil (aa1 n l iy0))
("online" nil (ao1 n l ay1 n))
("onlooker" nil (ao1 n l uh1 k er0))
("onlookers" nil (ao1 n l uh1 k er0 z))
("only" nil (ow1 n l iy0))
("onnen" nil (aa1 n ax n))
("onni" nil (aa1 n iy0))
("onno" nil (aa1 n ow0))
("ono" nil (ow1 n ow0))
("onoda" nil (ow0 n ow1 d er0))
("onofre" nil (ow0 n ao1 f r iy0))
("onofrio" nil (ow0 n ow1 f r iy0 ow0))
("onomastics" nil (aa1 n ax m ae1 s t ih0 k s))
("onondaga" nil (aa1 n ax n d ao1 g ax))
("onorato" nil (ow0 n ao0 r aa1 t ow0))
("onrushing" nil (aa1 n r ah1 sh ih0 ng))
("ons" nil (aa1 n z))
("onscreen" nil (aa1 n s k r iy1 n))
("onset" nil (aa1 n s eh1 t))
("onshore" nil (aa1 n sh ao1 r))
("onsite" nil (aa1 n s ay1 t))
("onslaught" nil (ao1 n s l ao1 t))
("onslow" nil (aa1 n s l ow0))
("onstad" nil (aa1 n s t ax d))
("onstage" nil (aa1 n s t ey1 jh))
("onstott" nil (aa1 n s t ax t))
("ontario" nil (aa0 n t eh1 r iy0 ow0))
("ontiveros" nil (ow0 n t iy0 v eh1 r ow0 z))
("ontko" nil (aa1 n t k ow0))
("onto" nil (aa1 n t uw0))
("ontogeny" nil (aa0 n t aa1 jh ax n iy0))
("ontological" nil (aa1 n t ax l aa1 jh ih0 k ax l))
("ontology" nil (aa0 n t aa1 l ax jh iy0))
("onus" nil (ow1 n ax s))
("onward" nil (ao1 n w er0 d))
("onwards" nil (aa1 n w er0 d z))
("onyx" nil (aa1 n ih0 k s))
("oodles" nil (uw1 d ax l z))
("ooh" nil (uw1))
("oohs" nil (uw1 z))
("ooley" nil (uw1 l iy0))
("oolong" nil (uw1 l ao0 ng))
("oomph" nil (uw1 m f))
("oona" nil (uw1 n ax))
("oooh" nil (uw1))
("ooohs" nil (uw1 z))
("oops" nil (uw1 p s))
("ooten" nil (ow0 ow0 t ey1 ax n))
("ooze" nil (uw1 z))
("oozed" nil (uw1 z d))
("oozes" nil (uw1 z ih0 z))
("oozing" nil (uw1 z ih0 ng))
("op" nil (aa1 p))
("opacity" nil (ow0 p ae1 s ax t iy0))
("opal" nil (ow1 p ax l))
("opalina" nil (ow0 p aa0 l iy1 n ax))
("opaline" nil (ow1 p ax l iy1 n))
("opalka" nil (ax p aa1 l k ax))
("opaque" nil (ow0 p ey1 k))
("opax" nil (ow1 p ae1 k s))
("opcom" nil (aa1 p k aa0 m))
("opdahl" nil (aa1 p d aa1 l))
("opdyke" nil (aa1 p d ay1 k))
("opec" nil (ow1 p eh1 k))
("opel" nil (ow1 p ax l))
("open" nil (ow1 p ax n))
("opened" nil (ow1 p ax n d))
("opener" nil (ow1 p ax n er0))
("openers" nil (ow1 p ax n er0 z))
("opening" nil (ow1 p ax n ih0 ng))
("openings" nil (ow1 p ax n ih0 ng z))
("openly" nil (ow1 p ax n l iy0))
("openness" nil (ow1 p ax n n ax s))
("opens" nil (ow1 p ax n z))
("openshaw" nil (ow1 p ax n sh ao1))
("openwork" nil (ow1 p ax n w er1 k))
("opera" nil (aa1 p r ax))
("operable" nil (aa1 p er0 ax b ax l))
("operandi" nil (aa1 p er0 ae1 n d iy0))
("operant" nil (aa1 p er0 ax n t))
("operas" nil (aa1 p r ax z))
("operate" nil (aa1 p er0 ey1 t))
("operated" nil (aa1 p er0 ey1 t ax d))
("operates" nil (aa1 p er0 ey1 t s))
("operatic" nil (aa1 p er0 ae1 t ih0 k))
("operating" nil (aa1 p er0 ey1 t ih0 ng))
("operation" nil (aa1 p er0 ey1 sh ax n))
("operational" nil (aa1 p er0 ey1 sh ax n ax l))
("operationally" nil (aa1 p er0 ey1 sh ax n ax l iy0))
("operations" nil (aa1 p er0 ey1 sh ax n z))
("operative" nil (aa1 p er0 ax t ih0 v))
("operatives" nil (aa1 p er0 ax t ih0 v z))
("operator" nil (aa1 p er0 ey1 t er0))
("operators" nil (aa1 p er0 ey1 t er0 z))
("opere" nil (ow0 p eh1 r))
("operetta" nil (aa1 p er0 eh1 t ax))
("operettas" nil (aa1 p er0 eh1 t ax z))
("opfer" nil (aa1 p f er0))
("opheim" nil (aa1 p hh ay1 m))
("ophelia" nil (ax f iy1 l y ax))
("ophthalmic" nil (aa0 f th ae1 l m ih0 k))
("ophthalmologist" nil (aa1 p th ax m aa1 l ax jh ih0 s t))
("ophthalmologists" nil (aa1 p th ax m aa1 l ax jh ih0 s t s))
("ophthalmology" nil (aa1 p th ax m aa1 l ax jh iy0))
("ophthalmoscope" nil (aa0 f th ae1 l m ax s k ow1 p))
("ophuls" nil (ow1 f ax l z))
("opiate" nil (ow1 p iy0 ax t))
("opic" nil (aa1 p ih0 k))
("opie" nil (aa1 p iy0))
("opiela" nil (ow1 p iy1 l ax))
("opine" nil (ow0 p ay1 n))
("opined" nil (ow0 p ay1 n d))
("opines" nil (ow0 p ay1 n z))
("opining" nil (ow0 p ay1 n ih0 ng))
("opinion" nil (ax p ih1 n y ax n))
("opinionated" nil (ax p ih1 n y ax n ey1 t ih0 d))
("opinions" nil (ax p ih1 n y ax n z))
("opitz" nil (aa1 p ih0 t s))
("opium" nil (ow1 p iy0 ax m))
("opland" nil (aa1 p l ax n d))
("ople" nil (ow1 p ax l))
("oplinger" nil (ow1 p ax l ih0 ng er0))
("opossum" nil (ow0 p aa1 s ax m))
("opp" nil (aa1 p))
("oppedisano" nil (ow0 p eh0 d iy0 s aa1 n ow0))
("oppel" nil (aa1 p ax l))
("oppelt" nil (aa1 p ih0 l t))
("oppenheim" nil (aa1 p ih0 n hh ay1 m))
("oppenheimer" nil (aa1 p ax n hh ay1 m er0))
("oppenheimers" nil (aa1 p ax n hh ay1 m er0 z))
("oppens" nil (aa1 p ax n z))
("opper" nil (aa1 p er0))
("opperman" nil (aa1 p er0 m ax n))
("oppermann" nil (aa1 p er0 m ax n))
("oppliger" nil (aa1 p l ih0 g er0))
("opponent" nil (ax p ow1 n ax n t))
("opponents" nil (ax p ow1 n ax n t s))
("opportune" nil (aa1 p er0 t uw1 n))
("opportunism" nil (aa1 p er0 t uw1 n ih1 z ax m))
("opportunist" nil (aa1 p er0 t uw1 n ih0 s t))
("opportunistic" nil (aa1 p er0 t uw1 n ih1 s t ih0 k))
("opportunists" nil (aa1 p er0 t uw1 n ih0 s t s))
("opportunites" nil (aa1 p er0 t uw0 n ay0 t s))
("opportunities" nil (aa1 p er0 t uw1 n ax t iy0 z))
("opportunity" nil (aa1 p er0 t uw1 n ax t iy0))
("oppose" nil (ax p ow1 z))
("opposed" nil (ax p ow1 z d))
("opposes" nil (ax p ow1 z ih0 z))
("opposing" nil (ax p ow1 z ih0 ng))
("opposite" nil (aa1 p ax z ax t))
("opposites" nil (aa1 p ax z ax t s))
("opposition" nil (aa1 p ax z ih1 sh ax n))
("oppress" nil (ax p r eh1 s))
("oppressed" nil (ax p r eh1 s t))
("oppression" nil (ax p r eh1 sh ax n))
("oppressive" nil (ax p r eh1 s ih0 v))
("oppressor" nil (ax p r eh1 s er0))
("oppressors" nil (ax p r eh1 s er0 z))
("opprobrium" nil (ax p r ow1 b r iy0 ax m))
("oprah" nil (ow1 p r ax))
("opry" nil (aa1 p r iy0))
("opryland" nil (aa1 p r iy0 l ax n d))
("ops" nil (aa1 p s))
("opsahl" nil (aa1 p s aa0 l))
("opsal" nil (aa1 p s ax l))
("opt" nil (aa1 p t))
("optation" nil (aa1 p t ey1 sh ax n))
("opted" nil (aa1 p t ih0 d))
("optek" nil (aa1 p t eh1 k))
("optic" nil (aa1 p t ih0 k))
("optica" nil (aa1 p t ih0 k ax))
("optical" nil (aa1 p t ih0 k ax l))
("optically" nil (aa1 p t ih0 k l iy0))
("opticians" nil (aa0 p t ih1 sh ax n z))
("optics" nil (aa1 p t ih0 k s))
("optima" nil (aa1 p t ax m ax))
("optimal" nil (aa1 p t ax m ax l))
("optimism" nil (aa1 p t ax m ih1 z ax m))
("optimist" nil (aa1 p t ax m ih0 s t))
("optimistic" nil (aa1 p t ax m ih1 s t ih0 k))
("optimistically" nil (aa1 p t ih0 m ih1 s t ih0 k ax l iy0))
("optimists" nil (aa1 p t ax m ih0 s t s))
("optimization" nil (aa0 p t ax m ax z ey1 sh ax n))
("optimize" nil (aa1 p t ax m ay1 z))
("optimum" nil (aa1 p t ax m ax m))
("opting" nil (aa1 p t ih0 ng))
("option" nil (aa1 p sh ax n))
("optional" nil (aa1 p sh ax n ax l))
("optioned" nil (aa1 p sh ax n d))
("options" nil (aa1 p sh ax n z))
("opto" nil (aa1 p t ow0))
("optometric" nil (aa1 p t ow0 m eh1 t r ih0 k))
("optometrists" nil (aa0 p t aa1 m ax t r ih0 s t s))
("optometry" nil (aa0 p t aa1 m ax t r iy0))
("opts" nil (aa1 p t s))
("optus" nil (aa1 p t ax s))
("opulence" nil (aa1 p y ax l ax n s))
("opulent" nil (aa1 p y ax l ax n t))
("opus" nil (ow1 p ax s))
("oquendo" nil (ow0 k w eh1 n d ow0))
("oquin" nil (ow0 k w iy1 n))
("oquinn" nil (ow0 k w iy1 n))
("or" nil (ao1 r))
("ora" nil (ao1 r ax))
("orabel" nil (ao0 r aa0 b eh1 l))
("orabelle" nil (ao1 r ax b ax l))
("oracle" nil (ao1 r ax k ax l))
("oracles" nil (ao1 r ax k ax l z))
("oradour" nil (ao1 r ax d ao1 r))
("oraflex" nil (ao1 r ax f l eh1 k s))
("orahood" nil (ao1 r ax hh uh1 d))
("oral" nil (ao1 r ax l))
("oralia" nil (ao0 r aa1 l iy0 ax))
("oralie" nil (ao1 r ax l iy0))
("orally" nil (ao1 r ax l iy0))
("oram" nil (ao1 r ax m))
("oran" nil (ao0 r aa1 n))
("orand" nil (ao1 r ax n d))
("orange" nil (ao1 r ax n jh))
("orangeburg" nil (ao1 r ax n jh b er0 g))
("oranges" nil (ao1 r ax n jh ax z))
("orangina" nil (ao0 r ae0 n jh iy1 n er0))
("oration" nil (ao0 r ey1 sh ax n))
("orator" nil (ao1 r ax t er0))
("oratorical" nil (ao1 r ax t ao1 r ax k ax l))
("oratorio" nil (aa1 r ax t ao1 r iy0 ow0))
("orators" nil (ao1 r ax t er0 z))
("oratory" nil (ao1 r ax t ao1 r iy0))
("oravec" nil (ao0 r aa1 v ih0 k))
("oravetz" nil (ao1 r ax v ih0 t s))
("orb" nil (ao1 r b))
("orbach" nil (ao1 r b aa0 k))
("orban" nil (ao1 r b ax n))
("orbanco" nil (ao0 r b ae1 ng k ow0))
("orben" nil (ao1 r b ax n))
("orbis" nil (ao1 r b ih0 s))
("orbit" nil (ao1 r b ax t))
("orbital" nil (ao1 r b ax t ax l))
("orbiter" nil (ao1 r b ax t er0))
("orbiting" nil (ao1 r b ax t ih0 ng))
("orbits" nil (ao1 r b ax t s))
("orcas" nil (ao1 r k ax s))
("orchard" nil (ao1 r ch er0 d))
("orchards" nil (ao1 r ch er0 d z))
("orchestra" nil (ao1 r k ax s t r ax))
("orchestral" nil (ao0 r k eh1 s t r ax l))
("orchestrally" nil (ao0 r k eh1 s t r ax l iy0))
("orchestras" nil (ao1 r k ax s t r ax z))
("orchestrate" nil (ao1 r k ih0 s t r ey1 t))
("orchestrated" nil (ao1 r k ih0 s t r ey1 t ih0 d))
("orchestrates" nil (ao1 r k ax s t r ey1 t s))
("orchestrating" nil (ao1 r k ih0 s t r ey1 t ih0 ng))
("orchestration" nil (ao1 r k ax s t r ey1 sh ax n))
("orchestrations" nil (ao1 r k ax s t r ey1 sh ax n z))
("orchid" nil (ao1 r k ax d))
("orchids" nil (ao1 r k ax d z))
("orcutt" nil (ao1 r k ax t))
("ord" nil (ao1 r d))
("ordain" nil (ao0 r d ey1 n))
("ordained" nil (ao0 r d ey1 n d))
("ordaz" nil (ao1 r d aa0 z))
("ordeal" nil (ao0 r d iy1 l))
("ordella" nil (ao1 r d eh1 l ax))
("order" nil (ao1 r d er0))
("ordered" nil (ao1 r d er0 d))
("ordering" nil (ao1 r d er0 ih0 ng))
("orderliness" nil (ao1 r d er0 l iy0 n ax s))
("orderly" nil (ao1 r d er0 l iy0))
("orders" nil (ao1 r d er0 z))
("ordinance" nil (ao1 r d ax n ax n s))
("ordinances" nil (ao1 r d ax n ax n s ih0 z))
("ordinaries" nil (ao1 r d ax n eh1 r iy0 z))
("ordinarily" nil (ao1 r d ax n eh1 r ax l iy0))
("ordinary" nil (ao1 r d ax n eh1 r iy0))
("ordination" nil (ao1 r d ax n ey1 sh ax n))
("ordiway" nil (ao1 r d iy0 w ey0))
("ordnance" nil (ao1 r d n ax n s))
("ordonez" nil (ao0 r d ow1 n eh0 z))
("ordoyne" nil (ao1 r d oy0 n))
("ordway" nil (ao1 r d w ey0))
("ore" nil (ao1 r))
("orea" nil (er0 iy1 ax))
("orear" nil (er0 ih1 r))
("orebaugh" nil (ao0 r eh1 b ao0))
("oreffice" nil (ao0 r eh1 f ih0 s))
("orefice" nil (ao0 r eh1 f ay0 s))
("oregan" nil (ao0 r eh1 g ax n))
("oregano" nil (ao1 r eh1 g ax n ow1))
("oregon" nil (ao1 r ax g ax n))
("oregonian" nil (ao1 r ax g ow1 n iy0 ax n))
("oreilly" nil (ao1 r ax l iy0))
("orel" nil (ao1 r ax l))
("orela" nil (ao0 r eh1 l ax))
("orelia" nil (ao0 r eh1 l iy0 ax))
("orellana" nil (ao0 r eh0 l ae1 n ax))
("orem" nil (ao1 r ih0 m))
("oren" nil (ao1 r ax n))
("orenda" nil (ao0 r eh1 n d ax))
("orendorff" nil (ao1 r ih0 n d ao0 r f))
("orens" nil (ao1 r ax n z))
("orenstein" nil (ao1 r ax n s t ay0 n))
("oreo" nil (ao1 r iy0 ow0))
("ores" nil (ao1 r z))
("oreskovich" nil (ao0 r eh1 s k ax v ih0 ch))
("orestes" nil (ao0 r eh1 s t iy0 z))
("orf" nil (ao1 r f))
("orfield" nil (ao1 r f iy0 l d))
("orford" nil (ao1 r f er0 d))
("org" nil (ao1 r g))
("organ" nil (ao1 r g ax n))
("organelles" nil (ao1 r g ax n eh1 l z))
("organic" nil (ao0 r g ae1 n ih0 k))
("organically" nil (ao0 r g ae1 n ih0 k l iy0))
("organisation" nil (ao1 r g ax n ih0 z ey1 sh ax n))
("organism" nil (ao1 r g ax n ih1 z ax m))
("organismic" nil (ao1 r g ax n ih1 z m ih0 k))
("organisms" nil (ao1 r g ax n ih1 z ax m z))
("organist" nil (ao1 r g ax n ax s t))
("organization" nil (ao1 r g ax n ax z ey1 sh ax n))
("organizational" nil (ao1 r g ax n ax z ey1 sh ax n ax l))
("organizations" nil (ao1 r g ax n ax z ey1 sh ax n z))
("organize" nil (ao1 r g ax n ay1 z))
("organized" nil (ao1 r g ax n ay1 z d))
("organizer" nil (ao1 r g ax n ay1 z er0))
("organizers" nil (ao1 r g ax n ay1 z er0 z))
("organizes" nil (ao1 r g ax n ay1 z ih0 z))
("organizing" nil (ao1 r g ax n ay1 z ih0 ng))
("organogenesis" nil (ao1 r g ax n ow0 jh eh1 n ax s ih0 s))
("organs" nil (ao1 r g ax n z))
("organum" nil (ao1 r g ax n ax m))
("orgasm" nil (ao1 r g ae0 z ax m))
("orgasms" nil (ao1 r g ae0 z ax m z))
("orgel" nil (ao1 r g ax l))
("orgeron" nil (ao1 r g er0 ax n))
("orgies" nil (ao1 r jh iy0 z))
("orgill" nil (ao1 r jh ax l))
("orginally" nil (ao1 r jh ih0 n ax l iy0))
("orgotein" nil (ao1 r g ow0 t ay1 n))
("orgy" nil (ao1 r jh iy0))
("ori" nil (ao1 r iy0))
("oriana" nil (ao0 r iy0 ae1 n ax))
("orick" nil (ao1 r ih0 k))
("orielda" nil (ao0 r iy1 l d ax))
("orielle" nil (ao1 r iy0 eh0 l))
("orient" nil (ao1 r iy0 eh1 n t))
("oriental" nil (ao1 r iy0 eh1 n t ax l))
("orientals" nil (ao1 r iy0 eh1 n t ax l z))
("orientation" nil (ao1 r iy0 eh0 n t ey1 sh ax n))
("oriented" nil (ao1 r iy0 eh1 n t ax d))
("orifices" nil (ao1 r ax f ax s ax z))
("origin" nil (ao1 r ax jh ax n))
("original" nil (er0 ih1 jh ax n ax l))
("originality" nil (er0 ih1 jh ax n ae1 l ih0 t iy0))
("originally" nil (er0 ih1 jh ax n ax l iy0))
("originals" nil (er0 ih1 jh ax n ax l z))
("originate" nil (er0 ih1 jh ax n ey1 t))
("originated" nil (er0 ih1 jh ax n ey1 t ax d))
("originates" nil (er0 ih1 jh ax n ey1 t s))
("originating" nil (er0 ih1 jh ax n ey1 t ih0 ng))
("origination" nil (er0 ih1 jh ax n ey1 sh ax n))
("originations" nil (er0 ih1 jh ax n ey1 sh ax n z))
("originator" nil (er0 ih1 jh ax n ey1 t er0))
("originators" nil (er0 ih1 jh ax n ey1 t er0 z))
("origins" nil (ao1 r ax jh ih0 n z))
("oriley" nil (ao1 r ih0 l iy0))
("orilla" nil (ao0 r ih1 l ax))
("orin" nil (ao1 r ih0 n))
("orinda" nil (er0 ih1 n d ax))
("orinoco" nil (ao1 r ax n ow1 k ow0))
("oriola" nil (ao0 r iy0 ow1 l ax))
("oriole" nil (ao1 r iy0 ow1 l))
("orioles" nil (ao1 r iy0 ow1 l z))
("orion" nil (ow0 r ay1 ax n))
("oriordan" nil (ow0 r ih1 r d ax n))
("oritz" nil (ao1 r ih0 t s))
("orix" nil (ao1 r ih0 k s))
("orkem" nil (ao1 r k ax m))
("orkin" nil (ao1 r k ih0 n))
("orkney" nil (ao1 r k n iy0))
("orky" nil (ao1 r k iy0))
("orlan" nil (ao1 r l ax n))
("orland" nil (ao1 r l ax n d))
("orlanda" nil (ao0 r l aa1 n d ax))
("orlandi" nil (ao0 r l aa1 n d iy0))
("orlando" nil (ao0 r l ae1 n d ow0))
("orleans" nil (ao1 r l iy0 ax n z))
("orlena" nil (ao1 r l ih0 n ax))
("orlene" nil (ao1 r l iy1 n))
("orlich" nil (ao1 r l ih0 k))
("orlick" nil (ao1 r l ih0 k))
("orloff" nil (ao1 r l ao0 f))
("orloski" nil (ao0 r l aw1 s k iy0))
("orlosky" nil (ao0 r l ow1 s k iy0))
("orlowski" nil (ao0 r l ao1 f s k iy0))
("orly" nil (ao1 r l iy0))
("orman" nil (ao1 r m ax n))
("ormand" nil (ao1 r m ax n d))
("orme" nil (ao1 r m))
("ormerod" nil (ao1 r m er0 ax d))
("ormes" nil (ao1 r m z))
("ormiston" nil (ao1 r m ih0 s t aa0 n))
("ormond" nil (ao1 r m ax n d))
("ormonde" nil (ao1 r m ax n d))
("ormsbee" nil (ao1 r m z b iy0))
("ormsby" nil (ao1 r m z b iy0))
("orn" nil (ao1 r n))
("orna" nil (ao1 r n ax))
("ornament" nil (ao1 r n ax m ax n t))
("ornamental" nil (ao1 r n ax m eh1 n t ax l))
("ornamentation" nil (ao1 r n ax m eh0 n t ey1 sh ax n))
("ornamented" nil (ao1 r n ax m eh1 n t ih0 d))
("ornaments" nil (ao1 r n ax m ax n t s))
("ornate" nil (ao0 r n ey1 t))
("ornda" nil (ao1 r n d ax))
("orndoff" nil (ao1 r n d ao0 f))
("orndorff" nil (ao1 r n d ao0 r f))
("orne" nil (ao1 r n))
("ornelas" nil (ao1 r n ih0 l ax z))
("ornellas" nil (ao0 r n eh1 l ax z))
("orner" nil (ao1 r n er0))
("ornery" nil (ao1 r n er0 iy0))
("ornstein" nil (ao1 r n s t ay0 n))
("oro" nil (ao1 r ow0))
("oroark" nil (ao1 r ao0 r k))
("orogenic" nil (ao1 r ax jh eh1 n ih0 k))
("orogil" nil (ao1 r ow0 g ih1 l))
("orographic" nil (ao1 r ax g r ae1 f ih0 k))
("orona" nil (ao0 r ow1 n ax))
("oropeza" nil (ao0 r ow0 p eh1 z ax))
("oros" nil (ao1 r ow0 z))
("orosco" nil (ao0 r ow1 s k ow0))
("orosz" nil (ao1 r ax sh))
("orourke" nil (er0 uh1 r k))
("orozco" nil (ao0 r ow1 z k ow0))
("orpah" nil (ao1 r p aa0))
("orphan" nil (ao1 r f ax n))
("orphanage" nil (ao1 r f ax n ax jh))
("orphanages" nil (ao1 r f ax n ih0 jh ih0 z))
("orphaned" nil (ao1 r f ax n d))
("orphaning" nil (ao1 r f ax n ih0 ng))
("orphans" nil (ao1 r f ax n z))
("orphee" nil (ao1 r f iy0))
("orpheum" nil (ao1 r f iy0 uw0 m))
("orpheus" nil (ao1 r f iy0 ax s))
("orphic" nil (ao1 r f ih0 k))
("orpiment" nil (ao1 r p ax m ax n t))
("orr" nil (ao1 r))
("orran" nil (ao0 r aa1 n))
("orrell" nil (ao0 r ey1 l))
("orren" nil (ao1 r ax n))
("orrery" nil (ao1 r er0 iy0))
("orrick" nil (ao1 r ih0 k))
("orrico" nil (ao0 r iy1 k ow0))
("orrin" nil (ao1 r ih0 n))
("orris" nil (ao1 r ih0 s))
("orrison" nil (ao1 r ih0 s ax n))
("orsa" nil (ao1 r s ax))
("orsak" nil (ao1 r s ax k))
("orsay" nil (ao1 r s ey1))
("orsborn" nil (ao1 r s b er0 n))
("orsburn" nil (ao1 r s b er0 n))
("orser" nil (ao1 r s er0))
("orsi" nil (ao1 r s iy0))
("orsini" nil (ao0 r s iy1 n iy0))
("orsino" nil (ao0 r s iy1 n ow0))
("orso" nil (ao1 r s ow0))
("orsola" nil (ao0 r s ow1 l ax))
("orson" nil (ao1 r s ax n))
("ort" nil (ao1 r t))
("orta" nil (ao1 r t ax))
("ortega" nil (ao0 r t ey1 g ax))
("ortegas" nil (ao0 r t ey1 g ax s))
("ortego" nil (ao0 r t ey1 g ow0))
("ortegon" nil (ao0 r t ey0 g ao1 n))
("orten" nil (ao1 r t ax n))
("ortenzio" nil (ao0 r t eh1 n z iy0 ow0))
("ortez" nil (ao0 r t eh1 z))
("orth" nil (ao1 r th))
("ortho" nil (ao1 r th ow1))
("orthoclase" nil (ao1 r th ax k l ey1 s))
("orthodontia" nil (ao1 r th ax d aa1 n ch iy0 ax))
("orthodontic" nil (ao1 r th ax d aa1 n t ih0 k))
("orthodontics" nil (ao1 r th ax d aa1 n t ih0 k s))
("orthodontist" nil (ao1 r th ax d aa1 n t ax s t))
("orthodontists" nil (ao1 r th ax d aa1 n t ax s t s))
("orthodox" nil (ao1 r th ax d aa1 k s))
("orthodoxy" nil (ao1 r th ax d aa1 k s iy0))
("orthography" nil (ao0 r th aa1 g r ax f iy0))
("orthopedic" nil (ao1 r th ax p iy1 d ih0 k))
("orthopedists" nil (ao1 r th ax p iy1 d ax s t s))
("ortis" nil (ao1 r t ih0 s))
("ortiz" nil (ao1 r t iy1 z))
("ortlieb" nil (ao1 r t l iy1 b))
("ortloff" nil (ao1 r t l ao0 f))
("ortman" nil (ao1 r t m ax n))
("ortmann" nil (ao1 r t m ax n))
("ortner" nil (ao1 r t n er0))
("ortolano" nil (ao0 r t ow0 l aa1 n ow0))
("orton" nil (ao1 r t ax n))
("ortwein" nil (ao1 r t w ay0 n))
("ortwin" nil (ao1 r t w ih0 n))
("orum" nil (ao1 r ax m))
("orva" nil (ao1 r v ax))
("orval" nil (ao1 r v ax l))
("orville" nil (ao1 r v ih0 l))
("orvin" nil (ao1 r v ih0 n))
("orvis" nil (ao1 r v ih0 s))
("orwell" nil (ao1 r w eh1 l))
("orwellian" nil (ao1 r w eh1 l iy0 ax n))
("orwick" nil (ao1 r w ih0 k))
("orwig" nil (ao1 r w ih0 g))
("ory" nil (ao1 r iy0))
("oryx" nil (ao1 r ih0 k s))
("orzech" nil (ao1 r z ih0 hh))
("orzechowski" nil (ao0 r z ih0 hh ao1 f s k iy0))
("orzel" nil (ao1 r z ax l))
("os" nil (aa1 s))
("osage" nil (ow0 s ey1 jh))
("osages" nil (ow0 s ey1 jh ax z))
("osaka" nil (ow0 s aa1 k ax))
("osake" nil (ow0 s aa1 k iy0))
("osaki" nil (ow0 s aa1 k iy0))
("osakowa" nil (ow1 s ax k ow1 ax))
("osamu" nil (ow0 s aa1 m uw0))
("osberg" nil (aa1 s b er0 g))
("osbert" nil (aa1 s b er0 t))
("osbon" nil (aa1 s b ax n))
("osborn" nil (ao1 z b ao1 r n))
("osborne" nil (ao1 z b ao1 r n))
("osbourn" nil (aa1 s b er0 n))
("osbourne" nil (aa1 s b er0 n))
("osbun" nil (aa1 s b ax n))
("osburn" nil (aa1 s b er1 n))
("osby" nil (aa1 s b iy0))
("oscar" nil (ao1 s k er0))
("oscars" nil (ao1 s k er0 z))
("oscarson" nil (aa1 s k aa0 r s ax n))
("osceola" nil (ao1 s k iy0 ow1 l ax))
("oscillates" nil (aa1 s ax l ey1 t s))
("oscillating" nil (aa1 s ax l ey1 t ih0 ng))
("oscillation" nil (aa1 s ax l ey1 sh ax n))
("oscillations" nil (aa1 s ax l ey1 sh ax n z))
("oscillator" nil (aa1 s ax l ey1 t er0))
("oscillators" nil (aa1 s ax l ey1 t er0 z))
("oscillatory" nil (aa1 s ax l ax t ao1 r iy0))
("oscilloscopes" nil (ao1 s ih1 l ax s k ow1 p s))
("osco" nil (aa1 s k ow0))
("ose" nil (ow1 z))
("oseguera" nil (ow0 s ey0 g eh1 r ax))
("oser" nil (ow1 s er0))
("osgood" nil (aa1 s g uh1 d))
("osha" nil (ow1 sh ax))
("oshatz" nil (ow0 sh ae1 t s))
("oshaughnessy" nil (ow1 sh aa1 n ih0 s iy0))
("oshawa" nil (ao1 sh aa1 w ax))
("oshea" nil (aa1 sh iy0 ax))
("oshel" nil (aa1 sh ax l))
("oshell" nil (aa1 sh ax l))
("osher" nil (aa1 sh er0))
("oshields" nil (aa1 sh iy0 l d z))
("oshima" nil (ow0 sh iy1 m ax))
("oshiro" nil (ow0 sh ih1 r ow0))
("oshita" nil (ow0 sh iy1 t ax))
("oshkosh" nil (ao1 sh k ao1 sh))
("oshman" nil (ao1 sh m ax n))
("osiecki" nil (ax s iy1 t s k iy0))
("osika" nil (ax s ay1 k ax))
("osinski" nil (ax s ih1 n s k iy0))
("oskar" nil (ao1 s k er0))
("osland" nil (aa1 s l ax n d))
("osler" nil (aa1 z l er0))
("osley" nil (aa1 s l iy0))
("oslin" nil (aa1 s l ih0 n))
("oslo" nil (aa1 z l ow0))
("oslund" nil (aa1 s l ax n d))
("osman" nil (aa1 s m ax n))
("osmar" nil (aa1 s m er0))
("osmena" nil (ao1 z m iy1 n ax))
("osment" nil (aa1 s m ax n t))
("osmer" nil (aa1 s ax m er0))
("osmium" nil (aa1 z m iy0 ax m))
("osmon" nil (aa1 s m ax n))
("osmond" nil (aa1 s m ax n d))
("osmonds" nil (ao1 z m ax n d z))
("osmosis" nil (ao1 z m ow1 s ih0 s))
("osmotic" nil (aa0 z m aa1 t ax k))
("osmun" nil (aa1 z m ax n))
("osmund" nil (aa1 s m ax n d))
("osmundson" nil (aa1 z m ax n d s ax n))
("osorio" nil (ow0 s ao1 r iy0 ow0))
("osowski" nil (ax s ao1 f s k iy0))
("ospina" nil (ow0 s p iy1 n ax))
("osprey" nil (ao1 s p r iy0))
("osred" nil (aa1 s er0 d))
("osric" nil (aa1 z r ih0 k))
("oss" nil (ao1 s))
("ossa" nil (aa1 s ax))
("ossicles" nil (aa1 s ax k ax l z))
("ossification" nil (aa1 s ax f ax k ey1 sh ax n))
("ossified" nil (aa1 s ax f ay1 d))
("ossify" nil (aa1 s ax f ay1))
("ossman" nil (aa1 s m ax n))
("osso" nil (ow1 s ow0))
("osswald" nil (aa1 s w ax l d))
("ost" nil (ow1 s t))
("ostberg" nil (aa1 s t b er0 g))
("ostby" nil (aa1 s t b iy0))
("osteen" nil (aa1 s t iy1 n))
("osten" nil (aa1 s ax n))
("ostendorf" nil (aa1 s t ax n d ao0 r f))
("ostensible" nil (aa0 s t eh1 n s ax b ax l))
("ostensibly" nil (aa0 s t eh1 n s ax b l iy0))
("ostenson" nil (aa1 s t ih0 n s ax n))
("ostentation" nil (ao1 s t eh0 n t ey1 sh ax n))
("ostentatious" nil (aa1 s t ax n t ey1 sh ax s))
("ostentatiously" nil (aa1 s t ax n t ey1 sh ax s l iy0))
("osteoarthritis" nil (aa1 s t iy0 ow1 aa0 r th r ay1 t ax s))
("osteopathic" nil (aa1 s t iy0 ax p ae1 th ih0 k))
("osteoporosis" nil (ao1 s t iy0 aa1 p er0 ow1 s ih0 s))
("oster" nil (aa1 s t er0))
("osterberg" nil (aa1 s t er0 b er0 g))
("ostergaard" nil (aa1 s t er0 g aa0 r d))
("ostergard" nil (aa1 s t er0 g er0 d))
("ostergren" nil (aa1 s t er0 g r eh0 n))
("osterhaus" nil (aa1 s t er0 hh aw0 s))
("osterhoff" nil (ow1 s t er0 hh ao1 f))
("osterholt" nil (aa1 s t er0 hh ow0 l t))
("osterhoudt" nil (aa1 s t er0 hh aw0 t))
("osterhout" nil (aa1 s t er0 hh aw0 t))
("osterkamp" nil (aa1 s t er0 k ae0 m p))
("osterling" nil (aa1 s t er0 l ih0 ng))
("osterloh" nil (ow0 s t eh1 r l ow0))
("osterlund" nil (aa1 s t er0 l ax n d))
("osterman" nil (aa1 s t er0 m ax n))
("ostermann" nil (aa1 s t er0 m ax n))
("ostermeier" nil (aa1 s t er0 m ay0 er0))
("ostermeyer" nil (aa1 s t er1 m ay1 er0))
("ostermiller" nil (aa1 s t er0 m ax l er0))
("osterreichische" nil (ao1 s t er0 r ay1 k ih0 sh iy0))
("ostertag" nil (aa1 s t er0 t ax g))
("osthoff" nil (aa1 s t hh ao0 f))
("ostia" nil (aa1 s t iy0 ax))
("ostin" nil (aa1 s t ax n))
("ostinato" nil (aa1 s t ax n aa1 t ow1))
("osting" nil (aa1 s t ih0 ng))
("ostling" nil (aa1 s ax l ih0 ng))
("ostlund" nil (aa1 s t l ax n d))
("ostman" nil (aa1 s t m ax n))
("ostpolitik" nil (ow1 s t p ow1 l ih0 t ih1 k))
("ostracism" nil (ao1 s t r ax s ih1 z ax m))
("ostracized" nil (ao1 s t r ax s ay1 z d))
("ostrand" nil (aa1 s t r ax n d))
("ostrander" nil (aa1 s t r ax n d er0))
("ostrem" nil (aa1 s t r ih0 m))
("ostrich" nil (ao1 s t r ih0 ch))
("ostriches" nil (aa1 s t r ih0 ch ih0 z))
("ostroff" nil (ao1 s t r ao0 f))
("ostrogoths" nil (aa1 s t r ax g aa1 th s))
("ostrom" nil (aa1 s t r ax m))
("ostroski" nil (ax s t r aw1 s k iy0))
("ostrosky" nil (ax s t r ow1 s k iy0))
("ostrow" nil (aa1 s t r aw0))
("ostrowski" nil (ax s t r ao1 f s k iy0))
("ostrowsky" nil (ax s t r aw1 s k iy0))
("ostrum" nil (aa1 s t r ax m))
("ostwald" nil (aa1 s t w ax l d))
("osuch" nil (aa1 s ax k))
("osullivan" nil (aa0 s ax l ih1 v ax n))
("osuna" nil (ow0 s uw1 n ax))
("oswald" nil (ao1 z w ao0 l d))
("oswalt" nil (aa1 s w ax l t))
("oswego" nil (aa0 s w iy1 g ow0))
("oswell" nil (aa1 s w eh1 l))
("ota" nil (ow1 t ax))
("otaiba" nil (ow0 t ey1 b ax))
("otani" nil (ow0 t aa1 n iy0))
("otello" nil (ow0 t eh1 l ow0))
("oteri" nil (ow0 t eh1 r iy0))
("otero" nil (ow0 t eh1 r ow0))
("otey" nil (ow1 t iy0))
("otha" nil (ax dh aa1))
("othello" nil (ax th eh1 l ow0))
("other" nil (ah1 dh er0))
("otherness" nil (ah1 dh er0 n ax s))
("others" nil (ah1 dh er0 z))
("otherwise" nil (ah1 dh er0 w ay1 z))
("otherworldly" nil (ah1 dh er0 w er1 l d l iy0))
("othilia" nil (ow0 th iy1 l iy0 ax))
("othman" nil (aa1 th m ax n))
("otis" nil (ow1 t ih0 s))
("otmar" nil (aa1 t m aa0 r))
("otology" nil (ow0 t aa1 l ax jh iy0))
("otomobil" nil (ow0 t ow1 m ow0 b ax l))
("otoole" nil (aa1 t uw0 l))
("otremba" nil (ow0 t r eh1 m b ax))
("otsego" nil (aa0 t s ey1 g ow0))
("otsuka" nil (ow0 t s uw1 k ax))
("ott" nil (aa1 t))
("ottaviani" nil (ow0 t aa0 v iy0 aa1 n iy0))
("ottaviano" nil (ow0 t aa0 v iy0 aa1 n ow0))
("ottawa" nil (aa1 t ax w aa1))
("ottaway" nil (aa1 t ax w ey0))
("otte" nil (aa1 t))
("otten" nil (aa1 t ax n))
("ottens" nil (aa1 t ax n z))
("otter" nil (aa1 t er0))
("otterbein" nil (aa1 t er0 b ay0 n))
("otterloo" nil (aa1 t er0 l uw1))
("otters" nil (aa1 t er0 z))
("otterson" nil (aa1 t er0 s ax n))
("ottesen" nil (aa1 t iy0 z ax n))
("otteson" nil (aa1 t ih0 s ax n))
("ottilie" nil (aa1 t ax l iy0))
("otting" nil (aa1 t ih0 ng))
("ottley" nil (aa1 t l iy0))
("ottman" nil (aa1 t m ax n))
("otto" nil (aa1 t ow1))
("ottoman" nil (aa1 t ax m ax n))
("ottosen" nil (aa1 t ow0 s ax n))
("ottoson" nil (aa1 t ax s ax n))
("otts" nil (aa1 t s))
("ottum" nil (aa1 t ax m))
("otwell" nil (aa1 t w eh1 l))
("ou" nil (uw1))
("oubre" nil (uw1 b er0))
("ouch" nil (aw1 ch))
("ouderkirk" nil (aw1 d er0 k er0 k))
("ouelette" nil (aa1 uw0 l eh0 t))
("ouellet" nil (aa1 uw0 l ih0 t))
("ouellette" nil (aa1 uw0 l eh0 t))
("ought" nil (ao1 t))
("oughta" nil (ao1 t ax))
("ouimet" nil (w iy0 m eh1 t))
("ouimette" nil (w iy0 m eh1 t))
("ounce" nil (aw1 n s))
("ounces" nil (aw1 n s ax z))
("ounsted" nil (aw1 n s t eh1 d))
("our" nil (aw1 er0))
("ourada" nil (ow0 uh0 r aa1 d ax))
("ours" nil (aw1 er0 z))
("ourself" nil (aw0 er0 s eh1 l f))
("ourselves" nil (aw0 er0 s eh1 l v z))
("ourso" nil (er1 s ow0))
("ousley" nil (aw1 s l iy0))
("oust" nil (aw1 s t))
("oustanding" nil (uw1 s t ae1 n d ih0 ng))
("ousted" nil (aw1 s t ih0 d))
("ouster" nil (aw1 s t er0))
("ousting" nil (aw1 s t ih0 ng))
("out" nil (aw1 t))
("outage" nil (aw1 t ax jh))
("outages" nil (aw1 t ih0 jh ih0 z))
("outback" nil (aw1 t b ae1 k))
("outbid" nil (aw1 t b ih1 d))
("outbidding" nil (aw1 t b ih1 d ih0 ng))
("outboard" nil (aw1 t b ao1 r d))
("outbound" nil (aw1 t b aw1 n d))
("outbreak" nil (aw1 t b r ey1 k))
("outbreaks" nil (aw1 t b r ey1 k s))
("outburst" nil (aw1 t b er1 s t))
("outbursts" nil (aw1 t b er1 s t s))
("outcalt" nil (aw1 t k ax l t))
("outcast" nil (aw1 t k ae1 s t))
("outcasts" nil (aw1 t k ae1 s t s))
("outclass" nil (aw1 t k l ae1 s))
("outcome" nil (aw1 t k ah1 m))
("outcomes" nil (aw1 t k ah1 m z))
("outcries" nil (aw1 t k r ay1 z))
("outcry" nil (aw1 t k r ay1))
("outdated" nil (aw1 t d ey1 t ih0 d))
("outdid" nil (aw1 t d ih1 d))
("outdistancing" nil (aw1 t d ih1 s t ax n s ih0 ng))
("outdo" nil (aw1 t d uw1))
("outdone" nil (aw1 t d ah1 n))
("outdoor" nil (aw1 t d ao1 r))
("outdoors" nil (aw1 t d ao1 r z))
("outen" nil (aw1 t ax n))
("outer" nil (aw1 t er0))
("outermost" nil (aw1 t er0 m ow1 s t))
("outerwear" nil (aw1 t er0 w eh1 r))
("outfield" nil (aw1 t f iy1 l d))
("outfielder" nil (aw1 t f iy1 l d er0))
("outfielders" nil (aw1 t f iy1 l d er0 z))
("outfit" nil (aw1 t f ih1 t))
("outfits" nil (aw1 t f ih1 t s))
("outfitted" nil (aw1 t f ih1 t ih0 d))
("outfitter" nil (aw1 t f ih1 t er0))
("outfitting" nil (aw1 t f ih1 t ih0 ng))
("outflank" nil (aw1 t f l ae1 ng k))
("outflanked" nil (aw0 t f l ae1 ng k t))
("outflow" nil (aw1 t f l ow1))
("outflows" nil (aw1 t f l ow1 z))
("outfoxed" nil (aw0 t f aa1 k s t))
("outgained" nil (aw1 t g ey1 n d))
("outgo" nil (aw1 t g ow1))
("outgoing" nil (aw1 t g ow1 ih0 ng))
("outgrew" nil (aw1 t g r uw1))
("outgrow" nil (aw1 t g r ow1))
("outgrowing" nil (aw1 t g r ow1 ih0 ng))
("outgrown" nil (aw1 t g r ow1 n))
("outgrowth" nil (aw1 t g r ow1 th))
("outguess" nil (aw1 t g eh1 s))
("outgunned" nil (aw1 t g ah1 n d))
("outhouse" nil (aw1 t hh aw1 s))
("outhouses" nil (aw1 t hh aw1 s ih0 z))
("outing" nil (aw1 t ih0 ng))
("outings" nil (aw1 t ih0 ng z))
("outland" nil (aw1 t l ae1 n d))
("outlandish" nil (aw0 t l ae1 n d ih0 sh))
("outlast" nil (aw1 t l ae1 s t))
("outlasted" nil (aw1 t l ae1 s t ih0 d))
("outlaw" nil (aw1 t l ao1))
("outlawed" nil (aw1 t l ao1 d))
("outlawing" nil (aw1 t l ao1 ih0 ng))
("outlawry" nil (aw1 t l ao1 r iy0))
("outlaws" nil (aw1 t l ao1 z))
("outlay" nil (aw1 t l ey1))
("outlays" nil (aw1 t l ey1 z))
("outler" nil (aw1 t ax l er0))
("outlet" nil (aw1 t l eh1 t))
("outlets" nil (aw1 t l eh1 t s))
("outley" nil (aw1 t l iy0))
("outline" nil (aw1 t l ay1 n))
("outlined" nil (aw1 t l ay1 n d))
("outlines" nil (aw1 t l ay1 n z))
("outlining" nil (aw1 t l ay1 n ih0 ng))
("outlive" nil (aw1 t l ih1 v))
("outlived" nil (aw1 t l ih1 v d))
("outlook" nil (aw1 t l uh1 k))
("outlooks" nil (aw1 t l uh1 k s))
("outlying" nil (aw1 t l ay1 ih0 ng))
("outman" nil (aw1 t m ax n))
("outmaneuver" nil (aw0 t m ax n uw1 v er0))
("outmaneuvered" nil (aw0 t m ax n uw1 v er0 d))
("outmoded" nil (aw1 t m ow1 d ax d))
("outnumber" nil (aw0 t n ah1 m b er0))
("outnumbered" nil (aw1 t n ah1 m b er0 d))
("outnumbering" nil (aw1 t n ah1 m b er0 ih0 ng))
("outokumpu" nil (uw1 t ax k ah1 m p uw0))
("outpace" nil (aw1 t p ey1 s))
("outpaced" nil (aw1 t p ey1 s t))
("outpaces" nil (aw1 t p ey1 s ih0 z))
("outpacing" nil (aw1 t p ey1 s ih0 ng))
("outpatient" nil (aw1 t p ey1 sh ax n t))
("outperform" nil (aw1 t p er0 f ao1 r m))
("outperformance" nil (aw1 t p er0 f ao1 r m ax n s))
("outperformed" nil (aw1 t p er0 f ao1 r m d))
("outperformer" nil (aw1 t p er0 f ao1 r m er0))
("outperforming" nil (aw1 t p er0 f ao1 r m ih0 ng))
("outperforms" nil (aw1 t p er0 f ao1 r m z))
("outplacement" nil (aw1 t p l ey1 s m ax n t))
("outpost" nil (aw1 t p ow1 s t))
("outposts" nil (aw1 t p ow1 s t s))
("outpouching" nil (aw1 t p aw1 ch ih0 ng))
("outpouring" nil (aw1 t p ao1 r ih0 ng))
("output" nil (aw1 t p uh1 t))
("outputs" nil (aw1 t p uh1 t s))
("outrage" nil (aw1 t r ey1 jh))
("outraged" nil (aw1 t r ey1 jh d))
("outrageous" nil (aw0 t r ey1 jh ax s))
("outrageously" nil (aw1 t r ey1 jh ax s l iy0))
("outrages" nil (aw1 t r ey1 jh ih0 z))
("outraging" nil (aw1 t r ey1 jh ih0 ng))
("outreach" nil (aw1 t r iy1 ch))
("outriders" nil (aw1 t r ay1 d er0 z))
("outrigger" nil (aw1 t r ih1 g er0))
("outright" nil (aw1 t r ay1 t))
("outrun" nil (aw0 t r ah1 n))
("outs" nil (aw1 t s))
("outscored" nil (aw0 t s k ao1 r d))
("outsell" nil (aw0 t s eh1 l))
("outselling" nil (aw0 t s eh1 l ih0 ng))
("outsells" nil (aw0 t s eh1 l z))
("outset" nil (aw1 t s eh1 t))
("outshine" nil (aw1 ch ay1 n))
("outshone" nil (aw0 t sh ow1 n))
("outside" nil (aw1 t s ay1 d))
("outsider" nil (aw0 t s ay1 d er0))
("outsiders" nil (aw0 t s ay1 d er0 z))
("outsides" nil (aw1 t s ay1 d z))
("outsized" nil (aw1 t s ay1 z d))
("outskirts" nil (aw1 t s k er1 t s))
("outsmart" nil (aw1 t s m aa1 r t))
("outsold" nil (aw0 t s ow1 l d))
("outsourcing" nil (aw1 t s ao1 r s ih0 ng))
("outspend" nil (aw1 t s p eh1 n d))
("outspending" nil (aw1 t s p eh1 n d ih0 ng))
("outspent" nil (aw0 t s p eh1 n t))
("outspoken" nil (aw1 t s p ow1 k ax n))
("outspokenness" nil (aw0 t s p ow1 k ax n ax s))
("outstanding" nil (aw1 t s t ae1 n d ih0 ng))
("outstretched" nil (aw0 t s t r eh1 ch t))
("outstrip" nil (aw0 t s t r ih1 p))
("outstripped" nil (aw0 t s t r ih1 p t))
("outstripping" nil (aw0 t s t r ih1 p ih0 ng))
("outstrips" nil (aw0 t s t r ih1 p s))
("outta" nil (uw1 t ax))
("outten" nil (aw1 t ax n))
("outvoted" nil (aw0 t v ow1 t ax d))
("outward" nil (aw1 t w er0 d))
("outwardly" nil (aw1 t w er0 d l iy0))
("outwards" nil (aw1 t w er0 d z))
("outweigh" nil (aw1 t w ey1))
("outweighed" nil (aw0 t w ey1 d))
("outweighing" nil (aw1 t w ey1 ih0 ng))
("outweighs" nil (aw1 t w ey1 z))
("outwit" nil (aw1 t w ih1 t))
("outwitting" nil (aw1 t w ih1 t ih0 ng))
("ouzts" nil (aw1 z t s))
("oval" nil (ow1 v ax l))
("ovalle" nil (aa1 v ey0 l))
("ovarian" nil (ow0 v eh1 r iy0 ax n))
("ovary" nil (ow1 v er0 iy0))
("ovate" nil (ow1 v ey0 t))
("ovation" nil (ow0 v ey1 sh ax n))
("ovations" nil (ow0 v ey1 sh ax n z))
("oven" nil (ah1 v ax n))
("ovens" nil (ah1 v ax n z))
("over" nil (ow1 v er0))
("overabundance" nil (ow1 v er0 ax b ah1 n d ax n s))
("overacker" nil (ow1 v er0 ax k er0))
("overacted" nil (ow1 v er0 ae1 k t ih0 d))
("overactive" nil (ow1 v er0 ae1 k t ih0 v))
("overall" j (ow1 v er0 ao1 l))
("overall" n (ow1 v er0 ao1 l))
("overallotment" nil (ow1 v er0 ax l aa1 t m ax n t))
("overallotments" nil (ow1 v er0 ax l aa1 t m ax n t s))
("overalls" nil (ow1 v er0 ao1 l z))
("overambitious" nil (ow1 v er0 ae0 m b ih1 sh ax s))
("overarching" nil (ow1 v er0 aa1 r ch ih0 ng))
("overbaugh" nil (ow0 v er1 b ao0))
("overbay" nil (ow1 v er0 b ey1))
("overbearing" nil (ow1 v er0 b eh1 r ih0 ng))
("overbeck" nil (ow1 v er0 b eh1 k))
("overbey" nil (ow1 v er0 b iy0))
("overbilling" nil (ow1 v er0 b ih1 l ih0 ng))
("overblown" nil (ow1 v er0 b l ow1 n))
("overboard" nil (ow1 v er0 b ao1 r d))
("overbooked" nil (ow1 v er0 b uh1 k t))
("overbooking" nil (ow1 v er0 b uh1 k ih0 ng))
("overbought" nil (ow1 v er0 b ao1 t))
("overbuilding" nil (ow1 v er0 b ih1 l d ih0 ng))
("overbuilt" nil (ow1 v er0 b ih1 l t))
("overburden" nil (ow1 v er0 b er1 d ax n))
("overburdened" nil (ow1 v er0 b er1 d ax n d))
("overby" nil (ow1 v er0 b iy0))
("overcame" nil (ow1 v er0 k ey1 m))
("overcapacity" nil (ow1 v er0 k ax p ae1 s ax t iy0))
("overcash" nil (ow1 v er0 k ae1 sh))
("overcast" nil (ow1 v er0 k ae1 s t))
("overcharge" nil (ow1 v er0 ch aa1 r jh))
("overcharged" nil (ow1 v er0 ch aa1 r jh d))
("overcharges" nil (ow1 v er0 ch aa1 r jh ih0 z))
("overcharging" nil (ow1 v er0 ch aa1 r jh ih0 ng))
("overcoat" nil (ow1 v er0 k ow1 t))
("overcome" nil (ow1 v er0 k ah1 m))
("overcomes" nil (ow1 v er0 k ah1 m z))
("overcoming" nil (ow1 v er0 k ah1 m ih0 ng))
("overconfidence" nil (ow1 v er0 k aa1 n f ih0 d ax n s))
("overconfident" nil (ow1 v er0 k aa1 n f ih0 d ax n t))
("overconsumption" nil (ow1 v er0 k ax n s ah1 m p sh ax n))
("overcooked" nil (ow1 v er0 k uh1 k t))
("overcrowded" nil (ow1 v er0 k r aw1 d ih0 d))
("overcrowding" nil (ow1 v er0 k r aw1 d ih0 ng))
("overdamping" nil (ow1 v er0 d ae1 m p ih0 ng))
("overdependence" nil (ow1 v er0 d ih0 p eh1 n d ax n s))
("overdid" nil (ow1 v er0 d ih1 d))
("overdo" nil (ow1 v er0 d uw1))
("overdoing" nil (ow1 v er0 d uw1 ih0 ng))
("overdone" nil (ow1 v er0 d ah1 n))
("overdorf" nil (ow1 v er0 d ao0 r f))
("overdose" nil (ow1 v er0 d ow1 s))
("overdosed" nil (ow1 v er0 d ow1 s t))
("overdoses" nil (ow1 v er0 d ow1 s ih0 z))
("overdraft" nil (ow1 v er0 d r ae1 f t))
("overdrafting" nil (ow1 v er0 d r ae1 f t ih0 ng))
("overdrafts" nil (ow1 v er0 d r ae1 f t s))
("overdraw" nil (ow1 v er0 d r ao1))
("overdrawn" nil (ow1 v er0 d r ao1 n))
("overdress" nil (ow1 v er0 d r eh1 s))
("overdrive" nil (ow1 v er0 d r ay1 v))
("overdue" nil (ow1 v er0 d uw1))
("overeager" nil (ow1 v er0 iy1 g er0))
("overeat" nil (ow1 v er0 iy1 t))
("overeating" nil (ow1 v er0 iy1 t ih0 ng))
("overemphasize" nil (ow1 v er0 eh1 m f ax s ay1 z))
("overestimate" nil (ow1 v er0 eh1 s t ax m ey1 t))
("overestimated" nil (ow1 v er0 eh1 s t ax m ey1 t ih0 d))
("overestimates" nil (ow1 v er0 eh1 s t ax m ey1 t s))
("overestimating" nil (ow1 v er0 eh1 s t ax m ey1 t ih0 ng))
("overexpansion" nil (ow1 v er0 ih0 k s p ae1 n sh ax n))
("overexposed" nil (ow1 v er0 ih0 k s p ow1 z d))
("overexposure" nil (ow1 v er0 ih0 k s p ow1 zh er0))
("overextended" nil (ow1 v er0 ih0 k s t eh1 n d ax d))
("overextending" nil (ow1 v er0 ih0 k s t eh1 n d ih0 ng))
("overfeed" nil (ow1 v er0 f iy1 d))
("overfelt" nil (ow1 v er0 f eh1 l t))
("overfield" nil (ow1 v er0 f iy1 l d))
("overfill" nil (ow1 v er0 f ih1 l))
("overfishing" nil (ow1 v er0 f ih1 sh ih0 ng))
("overflights" nil (ow1 v er0 f l ay1 t s))
("overflow" nil (ow1 v er0 f l ow1))
("overflowed" nil (ow1 v er0 f l ow1 d))
("overflowing" nil (ow1 v er0 f l ow1 ih0 ng))
("overflows" nil (ow1 v er0 f l ow1 z))
("overfunded" nil (ow1 v er0 f ah1 n d ih0 d))
("overfunding" nil (ow1 v er0 f ah1 n d ih0 ng))
("overgaard" nil (ow1 v er0 g aa1 r d))
("overgenerous" nil (ow1 v er0 jh eh1 n er0 ax s))
("overgrazing" nil (ow1 v er0 g r ey1 z ih0 ng))
("overgrown" nil (ow1 v er0 g r ow1 n))
("overhang" nil (ow1 v er0 hh ae1 ng))
("overhanging" nil (ow1 v er0 hh ae1 ng ih0 ng))
("overhangs" nil (ow1 v er0 hh ae1 ng z))
("overhaul" nil (ow1 v er0 hh ao1 l))
("overhauled" nil (ow1 v er0 hh ao1 l d))
("overhauling" nil (ow1 v er0 hh ao1 l ih0 ng))
("overhauls" nil (ow1 v er0 hh ao1 l z))
("overhead" nil (ow1 v er0 hh eh1 d))
("overheads" nil (ow1 v er0 hh eh1 d z))
("overhear" nil (ow1 v er0 hh ih1 r))
("overheard" nil (ow1 v er0 hh er1 d))
("overhearing" nil (ow1 v er0 hh ih1 r ih0 ng))
("overheat" nil (ow1 v er0 hh iy1 t))
("overheated" nil (ow1 v er0 hh iy1 t ih0 d))
("overheating" nil (ow1 v er0 hh iy1 t ih0 ng))
("overholser" nil (ow1 v er0 hh ow1 l s er0))
("overholt" nil (ow1 v er0 hh ow0 l t))
("overholtzer" nil (ow1 v er0 hh ow0 l t z er0))
("overjoyed" nil (ow1 v er0 jh oy1 d))
("overkill" nil (ow1 v er0 k ih1 l))
("overkilling" nil (ow1 v er0 k ih1 l ih0 ng))
("overlaid" nil (ow1 v er0 l ey1 d))
("overlain" nil (ow1 v er0 l ey1 n))
("overland" nil (ow1 v er0 l ae1 n d))
("overlap" nil (ow1 v er0 l ae1 p))
("overlapped" nil (ow1 v er0 l ae1 p t))
("overlapping" nil (ow1 v er0 l ae1 p ih0 ng))
("overlaps" nil (ow1 v er0 l ae1 p s))
("overlay" nil (ow1 v er0 l ey1))
("overlays" nil (ow1 v er0 l ey1 z))
("overleveraged" nil (ow1 v er0 l eh1 v r ih0 jh d))
("overley" nil (ow1 v er0 l iy0))
("overload" nil (ow1 v er0 l ow1 d))
("overloaded" nil (ow1 v er0 l ow1 d ih0 d))
("overloading" nil (ow1 v er0 l ow1 d ih0 ng))
("overloads" nil (ow1 v er0 l ow1 d z))
("overlock" nil (ow1 v er0 l aa1 k))
("overlook" nil (ow1 v er0 l uh1 k))
("overlooked" nil (ow1 v er0 l uh1 k t))
("overlooking" nil (ow1 v er0 l uh1 k ih0 ng))
("overlooks" nil (ow1 v er0 l uh1 k s))
("overlord" nil (ow1 v er0 l ao1 r d))
("overlords" nil (ow1 v er0 l ao1 r d z))
("overly" nil (ow1 v er0 l iy0))
("overlying" nil (ow1 v er0 l ay1 ih0 ng))
("overman" nil (ow1 v er0 m ax n))
("overmatched" nil (ow1 v er0 m ae1 ch t))
("overmyer" nil (ow1 v er0 m iy0 er0))
("overnight" nil (ow1 v er0 n ay1 t))
("overnighters" nil (ow1 v er0 n ay1 t er0 z))
("overnite" nil (ow1 v er0 n ay1 t))
("overoptimism" nil (ow1 v er0 aa1 p t ih0 m ih1 z ax m))
("overpaid" nil (ow1 v er0 p ey1 d))
("overpass" nil (ow1 v er0 p ae1 s))
("overpasses" nil (ow1 v er0 p ae1 s ih0 z))
("overpay" nil (ow1 v er0 p ey1))
("overpaying" nil (ow1 v er0 p ey1 ih0 ng))
("overpayment" nil (ow1 v er0 p ey1 m ax n t))
("overpayments" nil (ow1 v er0 p ey1 m ax n t s))
("overpeck" nil (ow1 v er0 p eh1 k))
("overplayed" nil (ow1 v er0 p l ey1 d))
("overpopulation" nil (ow1 v er0 p aa1 p y ax l ey1 sh ax n))
("overpowered" nil (ow1 v er0 p aw1 er0 d))
("overpowering" nil (ow1 v er0 p aw1 r ih0 ng))
("overpriced" nil (ow1 v er0 p r ay1 s t))
("overproduce" nil (ow1 v er0 p r ax d uw1 s))
("overproduced" nil (ow1 v er0 p r ax d uw1 s t))
("overproducers" nil (ow1 v er0 p r ax d uw1 s er0 z))
("overproducing" nil (ow1 v er0 p r ax d y uw1 s ih0 ng))
("overproduction" nil (ow1 v er0 p r ax d ah1 k sh ax n))
("overprotection" nil (ow1 v er0 p r ax t eh1 k sh ax n))
("overqualified" nil (ow1 v er0 k w aa1 l ax f ay1 d))
("overran" nil (ow1 v er0 r ae1 n))
("overrated" nil (ow1 v er0 r ey1 t ih0 d))
("overreach" nil (ow1 v er0 r iy1 ch))
("overreached" nil (ow1 v er0 r iy1 ch t))
("overreaches" nil (ow1 v er0 r iy1 ch ih0 z))
("overreaching" nil (ow1 v er0 r iy1 ch ih0 ng))
("overreact" nil (ow1 v er0 r iy0 ae1 k t))
("overreacted" nil (ow1 v er0 r iy0 ae1 k t ih0 d))
("overreacting" nil (ow1 v er0 r iy0 ae1 k t ih0 ng))
("overreaction" nil (ow1 v er0 r iy0 ae1 k sh ax n))
("overregulated" nil (ow1 v er0 r eh1 g y ax l ey1 t ih0 d))
("overregulation" nil (ow1 v er0 r eh1 g y ax l ey1 sh ax n))
("overreliance" nil (ow1 v er0 r ih0 l ay1 ax n s))
("overrepresented" nil (ow1 v er0 r eh1 p r ih0 z eh1 n t ih0 d))
("overridden" nil (ow1 v er0 r ih1 d ax n))
("override" nil (ow1 v er0 r ay1 d))
("overrides" nil (ow1 v er0 r ay1 d z))
("overriding" nil (ow1 v er0 r ay1 d ih0 ng))
("overripe" nil (ow1 v er0 r ay1 p))
("overrode" nil (ow1 v er0 r ow1 d))
("overrule" nil (ow1 v er0 r uw1 l))
("overruled" nil (ow1 v er0 r uw1 l d))
("overruling" nil (ow1 v er0 r uw1 l ih0 ng))
("overrun" nil (ow1 v er0 r ah1 n))
("overrunning" nil (ow1 v er0 r ah1 n ih0 ng))
("overruns" nil (ow1 v er0 r ah1 n z))
("overs" nil (ow1 v er0 z))
("oversaw" nil (ow1 v er0 s ao1))
("overseas" nil (ow1 v er0 s iy1 z))
("oversee" nil (ow1 v er0 s iy1))
("overseeing" nil (ow1 v er0 s iy1 ih0 ng))
("overseen" nil (ow1 v er0 s iy1 n))
("overseer" nil (ow1 v er0 s iy1 er0))
("overseers" nil (ow1 v er0 s iy1 er0 z))
("oversees" nil (ow1 v er0 s iy1 z))
("overselling" nil (ow1 v er0 s eh1 l ih0 ng))
("overshadow" nil (ow1 v er0 sh ae1 d ow0))
("overshadowed" nil (ow1 v er0 sh ae1 d ow0 d))
("overshadowing" nil (ow1 v er0 sh ae1 d ow0 ih0 ng))
("overshadows" nil (ow1 v er0 sh ae1 d ow0 z))
("overshoot" nil (ow1 v er0 sh uw1 t))
("overshooting" nil (ow1 v er0 sh uw1 t ih0 ng))
("overshot" nil (ow1 v er0 sh aa1 t))
("oversight" nil (ow1 v er0 s ay1 t))
("oversimplified" nil (ow1 v er0 s ih1 m p l ih0 f ay1 d))
("oversize" nil (ow1 v er0 s ay1 z))
("oversized" nil (ow1 v er0 s ay1 z d))
("oversizes" nil (ow1 v er0 s ay1 z ih0 z))
("overslept" nil (ow1 v er0 s l eh1 p t))
("oversold" nil (ow1 v er0 s ow1 l d))
("overson" nil (ow1 v er0 s ax n))
("overspend" nil (ow1 v er0 s p eh1 n d))
("overspending" nil (ow1 v er0 s p eh1 n d ih0 ng))
("overspends" nil (ow1 v er0 s p eh1 n d z))
("overspent" nil (ow1 v er0 s p eh1 n t))
("overstaffed" nil (ow1 v er0 s t ae1 f t))
("overstate" nil (ow1 v er0 s t ey1 t))
("overstated" nil (ow1 v er0 s t ey1 t ih0 d))
("overstatement" nil (ow1 v er0 s t ey1 t m ax n t))
("overstatements" nil (ow1 v er0 s t ey1 t m ax n t s))
("overstates" nil (ow1 v er0 s t ey1 t s))
("overstating" nil (ow1 v er0 s t ey1 t ih0 ng))
("overstayed" nil (ow1 v er0 s t ey1 d))
("overstep" nil (ow1 v er0 s t eh1 p))
("overstepped" nil (ow1 v er0 s t eh1 p t))
("overstepping" nil (ow1 v er0 s t eh1 p ih0 ng))
("overstocked" nil (ow1 v er0 s t aa1 k t))
("overstreet" nil (ow1 v er0 s t r iy1 t))
("overstrom" nil (ow1 v er0 s t r aa1 m))
("overstuffed" nil (ow1 v er0 s t ah1 f t))
("oversubscribed" nil (ow1 v er0 s ax b s k r ay1 b d))
("oversupplied" nil (ow1 v er0 s ax p l ay1 d))
("oversupply" nil (ow1 v er0 s ax p l ay1))
("overt" nil (ow0 v er1 t))
("overtake" nil (ow1 v er0 t ey1 k))
("overtaken" nil (ow1 v er0 t ey1 k ax n))
("overtaking" nil (ow1 v er0 t ey1 k ih0 ng))
("overtax" nil (ow1 v er0 t ae1 k s))
("overtaxed" nil (ow1 v er0 t ae1 k s t))
("overthrew" nil (ow1 v er0 th r uw1))
("overthrow" nil (ow1 v er0 th r ow1))
("overthrowing" nil (ow1 v er0 th r ow1 ih0 ng))
("overthrown" nil (ow1 v er0 th r ow1 n))
("overtime" nil (ow1 v er0 t ay1 m))
("overtly" nil (ow0 v er1 t l iy0))
("overton" nil (ow1 v er0 t ax n))
("overtones" nil (ow1 v er0 t ow1 n z))
("overtook" nil (ow1 v er0 t uh1 k))
("overture" nil (ow1 v er0 ch er0))
("overtures" nil (ow1 v er0 ch uh1 r z))
("overturf" nil (ow1 v er0 t er1 f))
("overturn" nil (ow1 v er0 t er1 n))
("overturned" nil (ow1 v er0 t er1 n d))
("overturning" nil (ow1 v er0 t er1 n ih0 ng))
("overturns" nil (ow1 v er0 t er1 n z))
("overuse" nil (ow1 v er0 y uw1 z))
("overused" nil (ow1 v er0 y uw1 z d))
("overvaluation" nil (ow1 v er0 v ae1 l y uw0 ey1 sh ax n))
("overvalued" nil (ow1 v er0 v ae1 l y uw0 d))
("overview" nil (ow1 v er0 v y uw1))
("overweight" nil (ow1 v er0 w ey1 t))
("overweighted" nil (ow1 v er0 w ey1 t ih0 d))
("overwhelm" nil (ow1 v er0 w eh1 l m))
("overwhelmed" nil (ow1 v er0 w eh1 l m d))
("overwhelming" nil (ow1 v er0 w eh1 l m ih0 ng))
("overwhelmingly" nil (ow1 v er0 w eh1 l m ih0 ng l iy0))
("overwhelms" nil (ow1 v er0 w eh1 l m z))
("overwinter" nil (ow1 v er0 w ih1 n t er0))
("overwork" nil (ow1 v er0 w er1 k))
("overworked" nil (ow1 v er0 w er1 k t))
("overwritten" nil (ow1 v er0 r ih1 t ax n))
("overwrought" nil (ow1 v er0 r ao1 t))
("overy" nil (ow1 v er0 iy0))
("overzealous" nil (ow1 v er0 z eh1 l ax s))
("oveson" nil (aa1 v ax s ax n))
("ovett" nil (ow1 v eh1 t))
("ovex" nil (ow1 v eh0 k s))
("oviatt" nil (ow1 v iy0 aa1 t))
("oviedo" nil (ow0 v iy0 ey1 d ow0))
("oviparous" nil (ow0 v ih1 p er0 ax s))
("ovitt" nil (ow1 v ih0 t))
("ovitz" nil (ow0 v ih1 t s))
("ovoid" nil (ow1 v oy1 d))
("ovonic" nil (ow0 v aa1 n ih0 k))
("ovoviviparous" nil (ow1 v ow1 v ay1 v ih1 p er0 ax s))
("ovshinsky" nil (aa0 v sh ih1 n s k iy0))
("ovulation" nil (ow1 v y ax l ey1 sh ax n))
("ovule" nil (ow1 v y uw0 l))
("ovum" nil (ow1 v ax m))
("ow" nil (ow1))
("owades" nil (ow1 aa0 d z))
("owczarzak" nil (aw0 ch aa1 r z ax k))
("owe" nil (ow1))
("owed" nil (ow1 d))
("owen" nil (ow1 ax n))
("owenby" nil (aw1 ih0 n b iy0))
("owens" nil (ow1 ax n z))
("owensboro" nil (ow1 ax n z b er0 ow0))
("owensby" nil (aw1 ih0 n s b iy0))
("owes" nil (ow1 z))
("owing" nil (ow1 ih0 ng))
("owings" nil (ow1 ih0 ng z))
("owl" nil (aw1 l))
("owls" nil (aw1 l z))
("own" nil (ow1 n))
("ownbey" nil (aw1 n b iy0))
("ownby" nil (aw1 n b iy0))
("owned" nil (ow1 n d))
("owner" nil (ow1 n er0))
("owners" nil (ow1 n er0 z))
("ownership" nil (ow1 n er0 sh ih1 p))
("owning" nil (ow1 n ih0 ng))
("owns" nil (ow1 n z))
("owosso" nil (ow0 ao1 s ow0))
("owsley" nil (aw1 s l iy0))
("owyhee" nil (ow1 ih0 hh iy0))
("ox" nil (aa1 k s))
("oxalates" nil (aa1 k s ax l ey1 t s))
("oxalis" nil (aa1 k s ax l ax s))
("oxbow" nil (aa1 k s b ow1))
("oxbridge" nil (aa1 k s b r ih1 jh))
("oxcart" nil (aa1 k s k aa1 r t))
("oxdon" nil (aa1 k s d aa1 n))
("oxen" nil (aa1 k s ax n))
("oxendine" nil (aa1 k s ih0 n d ay0 n))
("oxfam" nil (aa1 k s f ae1 m))
("oxford" nil (aa1 k s f er0 d))
("oxfords" nil (aa1 k s f er0 d z))
("oxidant" nil (aa1 k s ax d ax n t))
("oxidants" nil (aa1 k s ih0 d ax n t s))
("oxidation" nil (aa1 k s ax d ey1 sh ax n))
("oxide" nil (aa1 k s ay1 d))
("oxides" nil (aa1 k s ay1 d z))
("oxidized" nil (aa1 k s ax d ay1 z d))
("oxidizer" nil (aa1 k s ih0 d ay1 z er0))
("oxidizing" nil (aa1 k s ax d ay1 z ih0 ng))
("oxidyne" nil (aa1 k s ih0 d ay1 n))
("oxley" nil (aa1 k s l iy0))
("oxman" nil (aa1 k s m ax n))
("oxnard" nil (aa1 k s n er0 d))
("oxner" nil (aa1 k s n er0))
("oxoco" nil (aa0 k s ow1 k ow0))
("oxton" nil (aa1 k s t ax n))
("oxy" nil (aa1 k s iy0))
("oxygen" nil (aa1 k s ax jh ax n))
("oxygenate" nil (aa1 k s ax jh ax n ey1 t))
("oxygenated" nil (aa1 k s ax jh ax n ey1 t ax d))
("oxymoron" nil (aa0 k s ih1 m er0 aa0 n))
("oxytocin" nil (aa1 k s ax t ow1 s ax n))
("oy" nil (oy1))
("oyama" nil (ow0 y aa1 m ax))
("oye" nil (oy1))
("oyen" nil (oy1 ih0 n))
("oyer" nil (oy1 er0))
("oyler" nil (oy1 l er0))
("oyola" nil (oy1 ow1 l ax))
("oyster" nil (oy1 s t er0))
("oysters" nil (oy1 s t er0 z))
("oz" nil (aa1 z))
("ozaki" nil (ow0 z aa1 k iy0))
("ozal" nil (ow1 z ax l))
("ozanich" nil (ax z ae1 n ih0 hh))
("ozanne" nil (ow1 z ae0 n))
("ozark" nil (ow1 z aa1 r k))
("ozarks" nil (ow1 z aa1 r k s))
("ozawa" nil (ow0 z aa1 w ax))
("ozbun" nil (aa1 z b ax n))
("ozburn" nil (aa1 z b er0 n))
("ozelle" nil (ax z eh1 l))
("ozga" nil (ow1 z g ax))
("ozick" nil (ow1 z ih0 k))
("ozier" nil (ow1 z iy0 er0))
("ozimek" nil (ax z ih1 m eh0 k))
("ozment" nil (aa1 z m ax n t))
("ozmun" nil (aa1 z m ax n))
("ozolins" nil (ow0 z ow1 l iy0 n z))
("ozols" nil (ow1 z ow0 l z))
("ozone" nil (ow1 z ow1 n))
("ozora" nil (ax z ao1 r ax))
("ozuna" nil (ow0 z uw1 n ax))
("ozzie" nil (aa1 z iy0))
("p" nil (p iy1))
("pa" nil (p aa1))
("paap" nil (p aa1 p))
("paape" nil (p aa1 p))
("paar" nil (p aa1 r))
("paasch" nil (p aa1 sh))
("paavola" nil (p aa1 v ow1 l ax))
("pablo" nil (p aa1 b l ow0))
("pablum" nil (p ae1 b l ax m))
("pabon" nil (p aa0 b ao1 n))
("pabst" nil (p ae1 b s t))
("pac" nil (p ae1 k))
("paca" nil (p aa1 k ax))
("paccar" nil (p ax k aa1 r))
("paccione" nil (p aa0 k ch ow1 n iy0))
("pace" nil (p ey1 s))
("paced" nil (p ey1 s t))
("pacella" nil (p ax s eh1 l ax))
("pacelli" nil (p ax s eh1 l iy0))
("pacemaker" nil (p ey1 s m ey1 k er0))
("pacemakers" nil (p ey1 s m ey1 k er0 z))
("pacer" nil (p ey1 s er0))
("pacers" nil (p ey1 s er0 z))
("paces" nil (p ey1 s ih0 z))
("pacesetter" nil (p ey1 s eh1 t er0))
("paceway" nil (p ey1 s w ey1))
("pacey" nil (p ey1 s iy0))
("pacheco" nil (p ax ch eh1 k ow0))
("pachinko" nil (p ax ch ih1 ng k ow0))
("pacholder" nil (p ae1 ch ax l d er0))
("pacholski" nil (p ax hh ow1 l s k iy0))
("pachter" nil (p ae1 k t er0))
("pachysandra" nil (p ae1 ch ih0 s ae1 n d r ax))
("paci" nil (p aa1 ch iy0))
("pacific" nil (p ax s ih1 f ih0 k))
("pacifica" nil (p ax s ih1 f ih0 k ax))
("pacificare" nil (p ax s ih1 f ih0 k eh1 r))
("pacification" nil (p ae1 s ax f ax k ey1 sh ax n))
("pacifico" nil (p aa0 ch iy0 f iy1 k ow0))
("pacificorp" nil (p ax s ih1 f ih0 k ao0 r p))
("pacified" nil (p ae1 s ax f ay1 d))
("pacifier" nil (p ae1 s ax f ay1 er0))
("pacifiers" nil (p ae1 s ax f ay1 er0 z))
("pacifism" nil (p ae1 s ih0 f ih1 z ax m))
("pacifist" nil (p ae1 s ih0 f ih0 s t))
("pacifists" nil (p ae1 s ih0 f ih0 s t s))
("pacify" nil (p ae1 s ax f ay1))
("pacing" nil (p ey1 s ih0 ng))
("pacini" nil (p aa0 ch iy1 n iy0))
("pacino" nil (p ax s iy1 n ow0))
("pack" nil (p ae1 k))
("package" nil (p ae1 k ax jh))
("packaged" nil (p ae1 k ih0 jh d))
("packager" nil (p ae1 k ih0 jh er0))
("packagers" nil (p ae1 k ih0 jh er0 z))
("packages" nil (p ae1 k ax jh ax z))
("packaging" nil (p ae1 k ih0 jh ih0 ng))
("packard" nil (p ae1 k er0 d))
("packed" nil (p ae1 k t))
("packer" nil (p ae1 k er0))
("packers" nil (p ae1 k er0 z))
("packet" nil (p ae1 k ax t))
("packets" nil (p ae1 k ih0 t s))
("packett" nil (p ae1 k ih0 t))
("packham" nil (p ae1 k hh ax m))
("packing" nil (p ae1 k ih0 ng))
("packinghouse" nil (p ae1 k ih0 ng hh aw1 s))
("packman" nil (p ae1 k m ax n))
("packs" nil (p ae1 k s))
("packwood" nil (p ae1 k w uh1 d))
("pacman" nil (p ae1 k m ax n))
("paco" nil (p ey1 k ow0))
("pacs" nil (p ae1 k s))
("pact" nil (p ae1 k t))
("pactel" nil (p ae1 k t eh1 l))
("pacto" nil (p ae1 k t ow0))
("pacts" nil (p ae1 k t s))
("pacyna" nil (p aa1 k iy0 n ax))
("paczkowski" nil (p ax ch k ao1 f s k iy0))
("pad" nil (p ae1 d))
("paddack" nil (p ae1 d ax k))
("padded" nil (p ae1 d ax d))
("padden" nil (p ae1 d ax n))
("paddies" nil (p ae1 d iy0 z))
("padding" nil (p ae1 d ih0 ng))
("paddington" nil (p ae1 d ih0 ng t ax n))
("paddle" nil (p ae1 d ax l))
("paddled" nil (p ae1 d ax l d))
("paddles" nil (p ae1 d ax l z))
("paddling" nil (p ae1 d ax l ih0 ng))
("paddock" nil (p ae1 d ax k))
("paddy" nil (p ae1 d iy0))
("paden" nil (p ey1 d ax n))
("padfield" nil (p ae1 d f iy1 l d))
("padget" nil (p ae1 jh ih0 t))
("padgett" nil (p ae1 jh ih0 t))
("padgitt" nil (p ae1 jh ih0 t))
("padilla" nil (p ax d ih1 l ax))
("padley" nil (p ae1 d l iy0))
("padlock" nil (p ae1 d l aa1 k))
("padlocked" nil (p ae1 d l aa1 k t))
("padlocks" nil (p ae1 d l aa1 k s))
("padovano" nil (p aa0 d ow0 v aa1 n ow0))
("padre" nil (p ae1 d r ey1))
("padres" nil (p ae1 d r ey1 z))
("padrick" nil (p ae1 d r ih0 k))
("padro" nil (p aa1 d r ow0))
("padron" nil (p ae1 d r ax n))
("pads" nil (p ae1 d z))
("padua" nil (p ae1 d y uw0 ax))
("paduano" nil (p aa0 d uw0 aa1 n ow0))
("paducah" nil (p ax d uw1 k ax))
("padula" nil (p aa0 d uw1 l ax))
("pae" nil (p ay1))
("paean" nil (p iy1 ax n))
("paeans" nil (p iy1 ax n z))
("paek" nil (p iy1 k))
("paeth" nil (p iy1 th))
("paetz" nil (p iy1 t s))
("paez" nil (p ay0 eh1 z))
("paff" nil (p ae1 f))
("pafford" nil (p ae1 f er0 d))
("pagan" nil (p ey1 g ax n))
("paganelli" nil (p aa0 g aa0 n eh1 l iy0))
("pagani" nil (p aa0 g aa1 n iy0))
("paganini" nil (p ae1 g ax n iy1 n iy0))
("paganism" nil (p ey1 g ax n ih1 z ax m))
("pagano" nil (p aa0 g aa1 n ow0))
("pagar" nil (p ae1 g er0))
("page" nil (p ey1 jh))
("pageant" nil (p ae1 jh ax n t))
("pageantry" nil (p ae1 jh ax n t r iy0))
("pageants" nil (p ae1 jh ax n t s))
("paged" nil (p ey1 jh d))
("pagel" nil (p ae1 g ax l))
("pagels" nil (p ae1 g ax l z))
("pagemaker" nil (p ey1 jh m ey1 k er0))
("pagenkopf" nil (p ae1 g ax n k ao0 p f))
("pager" nil (p ey1 jh er0))
("pagers" nil (p ey1 jh er0 z))
("pages" nil (p ey1 jh ax z))
("paget" nil (p ae1 jh ax t))
("pagett" nil (p ae1 jh ax t))
("pagette" nil (p ae1 jh eh1 t))
("pagezy" nil (p ae1 jh eh1 z iy0))
("pagination" nil (p ae1 jh ax n ey1 sh ax n))
("paging" nil (p ey1 jh ih0 ng))
("paglia" nil (p ae1 g l iy0 ax))
("pagliaro" nil (p ae1 g l iy0 aa1 r ow0))
("pagliarulo" nil (p ae1 g l iy0 er0 uw1 l ow0))
("pagliuca" nil (p ae1 g l iy0 uw1 k ax))
("pagni" nil (p ae1 g n iy0))
("pagnotta" nil (p aa0 g n ow1 t ax))
("pagoda" nil (p ax g ow1 d ax))
("pagurian" nil (p ax g y uh1 r iy0 ax n))
("pah" nil (p ae1))
("pahl" nil (p aa1 l))
("pahnos" nil (p aa1 n ow0 s))
("pai" nil (p aa1 iy0))
("paid" nil (p ey1 d))
("paige" nil (p ey1 jh))
("paik" nil (p ey1 k))
("pail" nil (p ey1 l))
("paille" nil (p ey1 l))
("pails" nil (p ey1 l z))
("pain" nil (p ey1 n))
("paine" nil (p ey1 n))
("pained" nil (p ey1 n d))
("painewebber" nil (p ey1 n w eh1 b er0))
("painful" nil (p ey1 n f ax l))
("painfully" nil (p ey1 n f ax l iy0))
("painkiller" nil (p ey1 n k ih1 l er0))
("painkillers" nil (p ey1 n k ih1 l er0 z))
("painless" nil (p ey1 n l ax s))
("painlessly" nil (p ey1 n l ax s l iy0))
("paino" nil (p ey1 n ow0))
("pains" nil (p ey1 n z))
("painstaking" nil (p ey1 n s t ey1 k ih0 ng))
("painstakingly" nil (p ey1 n s t ey1 k ih0 ng l iy0))
("paint" nil (p ey1 n t))
("paintbrush" nil (p ey1 n t b r ah1 sh))
("paintbrushes" nil (p ey1 n t b r ah1 sh ih0 s))
("painted" nil (p ey1 n t ax d))
("painter" nil (p ey1 n t er0))
("painterly" nil (p ey1 n t er0 l iy0))
("painters" nil (p ey1 n t er0 z))
("painting" nil (p ey1 n t ih0 ng))
("paintings" nil (p ey1 n t ih0 ng z))
("paints" nil (p ey1 n t s))
("pair" nil (p eh1 r))
("paired" nil (p eh1 r d))
("pairing" nil (p eh1 r ih0 ng))
("pairs" nil (p eh1 r z))
("pais" nil (p ey1 z))
("paisley" nil (p ey1 z l iy0))
("pait" nil (p ey1 t))
("paiute" nil (p ay1 y uw0 t))
("paiva" nil (p ey1 v ax))
("paiz" nil (p ey1 z))
("pajak" nil (p ay1 ax k))
("pajama" nil (p ax jh aa1 m ax))
("pajamas" nil (p ax jh aa1 m ax z))
("pak" nil (p ae1 k))
("pake" nil (p ey1 k))
("pakeha" nil (p ax k ey1 hh aa0))
("pakistan" nil (p ae1 k ih0 s t ae1 n))
("pakistani" nil (p ae1 k ih0 s t ae1 n iy0))
("pakistanian" nil (p ae1 k ih0 s t ae1 n iy0 ax n))
("pakistanis" nil (p ae1 k ih0 s t ae1 n iy0 z))
("pakula" nil (p ax k uw1 l ax))
("pakulski" nil (p ax k ah1 l s k iy0))
("pal" nil (p ae1 l))
("palace" nil (p ae1 l ax s))
("palaces" nil (p ae1 l ax s ax z))
("palacio" nil (p ax l ey1 s iy0 ow0))
("palacios" nil (p aa0 l aa0 s iy1 ow0 z))
("paladino" nil (p aa0 l aa0 d iy1 n ow0))
("palafox" nil (p ae1 l ax f aa1 k s))
("palais" nil (p ax l ey1))
("palardy" nil (p ax l aa1 r d iy0))
("palash" nil (p ax l ae1 sh))
("palatability" nil (p ae1 l ax t ax b ih1 l ax t iy0))
("palatable" nil (p ae1 l ax t ax b ax l))
("palate" nil (p ae1 l ax t))
("palates" nil (p ae1 l ax t s))
("palatial" nil (p ax l ey1 sh ax l))
("palatine" nil (p ae1 l ax t ay1 n))
("palau" nil (p ae1 l aw0))
("palauans" nil (p ax l aw1 ax n z))
("palay" nil (p ey1 l ey1))
("palazzi" nil (p aa0 l aa1 t s iy0))
("palazzo" nil (p ax l aa1 z ow0))
("palazzola" nil (p aa0 l aa0 t s ow1 l ax))
("palazzolo" nil (p aa0 l aa0 t s ow1 l ow0))
("palca" nil (p ae1 l k ax))
("palco" nil (p ae1 l k ow0))
("pale" nil (p ey1 l))
("palecek" nil (p aa1 l ih0 ch eh0 k))
("paled" nil (p ey1 l d))
("palen" nil (p ae1 l ax n))
("paleobotany" nil (p ey1 l iy0 ow0 b aa1 t ax n iy0))
("paleocene" nil (p ey1 l iy0 ax s iy1 n))
("paleontology" nil (p ey1 l iy0 ax n t aa1 l ax jh iy0))
("paleozoic" nil (p ey1 l iy0 ax z ow1 ih0 k))
("palermo" nil (p ax l eh1 r m ow0))
("pales" nil (p ey1 l z))
("palese" nil (p aa0 l ey1 z iy0))
("palest" nil (p ey1 l ax s t))
("palestine" nil (p ae1 l ax s t ay1 n))
("palestinian" nil (p ae1 l ih0 s t ih1 n iy0 ax n))
("palestinians" nil (p ae1 l ih0 s t ih1 n iy0 ax n z))
("palette" nil (p ae1 l ax t))
("paley" nil (p ey1 l iy0))
("palfrey" nil (p ae1 l f r iy0))
("palin" nil (p ae1 l ih0 n))
("palinkas" nil (p ae1 l ih0 ng k ax z))
("palisades" nil (p ae1 l ih0 s ey1 d z))
("palka" nil (p ae1 l k ax))
("palko" nil (p ae1 l k ow0))
("palkovic" nil (p ax l k aa1 v ih0 k))
("pall" nil (p aa1 l))
("palla" nil (p ae1 l ax))
("palladino" nil (p aa0 l aa0 d iy1 n ow0))
("palladium" nil (p ax l ey1 d iy0 ax m))
("pallante" nil (p aa0 l aa1 n t iy0))
("pallas" nil (p ae1 l ax s))
("paller" nil (p ae1 l er0))
("palleschi" nil (p aa0 l eh1 s k iy0))
("pallet" nil (p ae1 l ax t))
("pallett" nil (p ae1 l ax t))
("palliative" nil (p ae1 l iy0 ax t ih0 v))
("palliatives" nil (p ae1 l iy0 ax t ih0 v z))
("pallid" nil (p ae1 l ax d))
("pallo" nil (p ae1 l ow0))
("pallone" nil (p aa0 l ow1 n iy0))
("palm" nil (p aa1 m))
("palma" nil (p aa1 l m ax))
("palmateer" nil (p ae1 l m ax t ih0 r))
("palmatier" nil (p ae1 l m ax t iy0 er0))
("palmdale" nil (p aa1 m d ey1 l))
("palme" nil (p aa1 m))
("palmer" nil (p aa1 m er0))
("palmeri" nil (p aa0 l m eh1 r iy0))
("palmerino" nil (p ao1 l m eh0 r iy1 n ow0))
("palmero" nil (p aa0 l m eh1 r ow0))
("palmerton" nil (p aa1 m er0 t ax n))
("palmertree" nil (p aa1 m er0 t r iy1))
("palmetto" nil (p ae0 l m eh1 t ow0))
("palmgren" nil (p ae1 l m g r eh0 n))
("palmieri" nil (p ao1 l m iy0 eh1 r iy0))
("palmira" nil (p aa0 l m ih1 r ax))
("palmisano" nil (p aa0 l m iy0 s aa1 n ow0))
("palmistry" nil (p aa1 m ih0 s t r iy0))
("palmiter" nil (p ae1 l m ay0 t er0))
("palmitic" nil (p ae0 l m ih1 t ih0 k))
("palmolive" nil (p aa0 l m aa1 l ih0 v))
("palmore" nil (p ae1 l m ao0 r))
("palmquist" nil (p ae1 l m k w ih0 s t))
("palms" nil (p aa1 m z))
("palmstierna" nil (p ao1 l m s t iy0 eh1 r n ax))
("palo" nil (p ae1 l ow0))
("paloma" nil (p aa0 l ow1 m ax))
("palomar" nil (p ae1 l ax m aa0 r))
("palomares" nil (p aa0 l ow0 m aa1 r eh0 s))
("palomba" nil (p aa0 l ow1 m b ax))
("palombi" nil (p ax l aa1 m b iy0))
("palombo" nil (p ax l aa1 m b ow0))
("palometa" nil (p aa0 l ow0 m eh1 t ax))
("palomino" nil (p ae1 l ax m iy1 n ow0))
("palomita" nil (p aa0 l ow0 m iy1 t ax))
("palomo" nil (p aa0 l ow1 m ow0))
("palone" nil (p ax l ow1 n))
("palos" nil (p aa1 l ow0 z))
("palpable" nil (p ae1 l p ax b ax l))
("palpably" nil (p ae1 l p ax b l iy0))
("palpitation" nil (p ae1 l p ax t ey1 sh ax n))
("palpitations" nil (p ae1 l p ih0 t ey1 sh ax n z))
("pals" nil (p ae1 l z))
("palsy" nil (p ao1 l z iy0))
("paltry" nil (p ao1 l t r iy0))
("paltz" nil (p ao1 l t s))
("paluch" nil (p ae1 l ax k))
("paluck" nil (p ae1 l ax k))
("palumbo" nil (p ax l ah1 m b ow0))
("paluzzi" nil (p aa0 l uw1 t s iy0))
("pam" nil (p ae1 m))
("pamby" nil (p ae1 m b iy0))
("pamela" nil (p ae1 m ax l ax))
("pamelina" nil (p aa0 m eh0 l iy1 n ax))
("pamella" nil (p ax m eh1 l ax))
("pamer" nil (p ey1 m er0))
("pammy" nil (p ae1 m iy0))
("pamour" nil (p ae1 m ao0 r))
("pampas" nil (p ae1 m p ax z))
("pampel" nil (p ae1 m p ax l))
("pamper" nil (p ae1 m p er0))
("pampered" nil (p ae1 m p er0 d))
("pamperin" nil (p ae1 m p er0 ih0 n))
("pampering" nil (p ae1 m p er0 ih0 ng))
("pampers" nil (p ae1 m p er0 z))
("pamphlet" nil (p ae1 m f l ax t))
("pamphleteer" nil (p ae1 m f l ax t ih1 r))
("pamphlets" nil (p ae1 m f l ax t s))
("pamplin" nil (p ae1 m p l ih0 n))
("pan" nil (p ae1 n))
("panacea" nil (p ae1 n ax s iy1 ax))
("panache" nil (p ax n aa1 sh))
("panaco" nil (p ae1 n ax k ow0))
("panagopoulos" nil (p ae0 n ax g aa1 p ax l ih0 s))
("panagos" nil (p aa0 n aa1 g ow0 z))
("panama" nil (p ae1 n ax m aa1))
("panamanian" nil (p ae1 n ax m ey1 n iy0 ax n))
("panamanians" nil (p ae1 n ax m ey1 n iy0 ax n z))
("panamsat" nil (p ax n ae1 m s ae0 t))
("panaro" nil (p aa0 n aa1 r ow0))
("panas" nil (p ae1 n ax z))
("panasonic" nil (p ae1 n ax s aa1 n ih0 k))
("pancake" nil (p ae1 n k ey1 k))
("pancakes" nil (p ae1 n k ey1 k s))
("pancanadian" nil (p ae1 ng k ax n ey1 d iy0 ax n))
("pancho" nil (p ae1 n ch ow0))
("panciera" nil (p aa0 n ch ih1 r ax))
("pancoast" nil (p ae1 n k ow1 s t))
("pancontinental" nil (p ae1 n k aa1 n t ax n eh1 n t ax l))
("pancreas" nil (p ae1 n k r iy0 ax s))
("pancreatic" nil (p ae1 n k r iy0 ae1 t ih0 k))
("panda" nil (p ae1 n d ax))
("pandanus" nil (p ae0 n d ey1 n ax s))
("pandas" nil (p ae1 n d ax z))
("pandemic" nil (p ae0 n d eh1 m ih0 k))
("pandemonium" nil (p ae1 n d ih0 m ow1 n iy0 ax m))
("pander" nil (p ae1 n d er0))
("pandered" nil (p ae1 n d er0 d))
("pandering" nil (p ae1 n d er0 ih0 ng))
("pandey" nil (p aa1 n d ey1))
("pandick" nil (p ae1 n d ih1 k))
("pandit" nil (p ah1 n d ax t))
("pando" nil (p aa1 n d ow0))
("pandolfi" nil (p aa0 n d ow1 l f iy0))
("pandolfo" nil (p aa0 n d ow1 l f ow0))
("pandora" nil (p ae0 n d ao1 r ax))
("pandya" nil (p aa1 n d y ax))
("pane" nil (p ey1 n))
("panebianco" nil (p aa0 n eh0 b iy0 aa1 n k ow0))
("panek" nil (p ae1 n ih0 k))
("panel" nil (p ae1 n ax l))
("paneled" nil (p ae1 n ax l d))
("paneling" nil (p ae1 n ax l ih0 ng))
("panelist" nil (p ae1 n ax l ax s t))
("panelists" nil (p ae1 n ax l ih0 s t s))
("panelization" nil (p ae1 n ax l ax z ey1 sh ax n))
("panelized" nil (p ae1 n ax l ay1 z d))
("panella" nil (p ax n eh1 l ax))
("panels" nil (p ae1 n ax l z))
("panem" nil (p ey1 n ax m))
("panepinto" nil (p aa0 n eh0 p iy1 n t ow0))
("panes" nil (p ey1 n z))
("panetta" nil (p ax n eh1 t ax))
("panfida" nil (p ae1 n f iy1 d ax))
("panfil" nil (p ae1 n f ih0 l))
("panfile" nil (p ae1 n f ay1 l))
("panfuls" nil (p ae1 n f ax l z))
("pang" nil (p ae1 ng))
("pangallo" nil (p aa0 ng g aa1 l ow0))
("pangborn" nil (p ae1 ng b ao1 r n))
("pangburn" nil (p ae1 ng b er1 n))
("pangels" nil (p ae1 ng g eh1 l z))
("pangle" nil (p ae1 ng g ax l))
("pangs" nil (p ae1 ng z))
("panhandle" nil (p ae1 n hh ae1 n d ax l))
("paniagua" nil (p aa0 n iy0 aa1 g ax))
("panic" nil (p ae1 n ih0 k))
("paniccia" nil (p aa0 n iy1 ch ax))
("panicked" nil (p ae1 n ih0 k t))
("panicking" nil (p ae1 n ih0 k ih0 ng))
("panicky" nil (p ae1 n ih0 k iy0))
("panico" nil (p aa0 n iy1 k ow0))
("panics" nil (p ae1 n ih0 k s))
("panik" nil (p ae1 n ih0 k))
("pankau" nil (p ae1 ng k aw0))
("pankey" nil (p ae1 n k iy1))
("pankki" nil (p ae1 ng k iy0))
("panko" nil (p ae1 ng k ow0))
("pankonin" nil (p ae1 ng k ax n ih0 n))
("pankow" nil (p ae1 ng k ow0))
("pankratz" nil (p ae1 ng k r ax t s))
("panky" nil (p ae1 ng k iy0))
("panmunjom" nil (p ae1 n m uw0 n jh ao1 m))
("panmure" nil (p ae1 n m uh1 r))
("panned" nil (p ae1 n d))
("pannell" nil (p ae1 n ax l))
("pannier" nil (p ae1 n iy0 er0))
("pannill" nil (p ae1 n ih0 l))
("panning" nil (p ae1 n ih0 ng))
("pannone" nil (p aa0 n ow1 n iy0))
("pannu" nil (p aa1 n uw0))
("pannullo" nil (p aa0 n uw1 l ow0))
("panny" nil (p ae1 n iy0))
("panoply" nil (p ae1 n aa0 p l iy0))
("panopolys" nil (p ax n aa1 p ax l iy0 z))
("panora" nil (p ax n ao1 r ax))
("panorama" nil (p ae1 n er0 ae1 m ax))
("panoramic" nil (p ae1 n er0 ae1 m ih0 k))
("panos" nil (p aa1 n ow0 s))
("panoz" nil (p ae1 n aa0 z))
("panozzo" nil (p ax n aa1 z ow0))
("panphila" nil (p ae1 n f ih0 l ax))
("pans" nil (p ae1 n z))
("pansies" nil (p ae1 n z iy0 z))
("pansophic" nil (p ae1 n s ax f ih0 k))
("pansy" nil (p ae1 n z iy0))
("pant" nil (p ae1 n t))
("pantaleo" nil (p aa0 n t aa1 l iy0 ow0))
("pantalone" nil (p ae1 n t ax l ow1 n))
("pantaloons" nil (p ae1 n t ax l uw1 n z))
("pantano" nil (p aa0 n t aa1 n ow0))
("panted" nil (p ae1 n t ih0 d))
("pantel" nil (p aa0 n t eh1 l))
("panter" nil (p ae1 n t er0))
("panthea" nil (p ae1 n th iy0 ax))
("pantheistic" nil (p ae1 n th iy0 ih1 s t ih0 k))
("pantheon" nil (p ae1 n th iy0 aa1 n))
("panther" nil (p ae1 n th er0))
("panthers" nil (p ae1 n th er0 z))
("panties" nil (p ae1 n t iy0 z))
("panting" nil (p ae1 n t ih0 ng))
("pantle" nil (p ae1 n t ax l))
("pantoja" nil (p aa0 n t ow1 y ax))
("pantomime" nil (p ae1 n t ax m ay1 m))
("panton" nil (p ae1 n t ax n))
("pantry" nil (p ae1 n t r iy0))
("pants" nil (p ae1 n t s))
("pantsuit" nil (p ae1 n t s uw1 t))
("pantuso" nil (p aa0 n t uw1 s ow0))
("panty" nil (p ae1 n t iy0))
("pantyhose" nil (p ae1 n t iy0 hh ow1 z))
("panza" nil (p ae1 n z ax))
("panzarella" nil (p aa0 n z aa0 r eh1 l ax))
("panzer" nil (p ae1 n z er0))
("panzhihua" nil (p ae1 n z ih0 hh y uw1 ax))
("panzica" nil (p ae1 n z ih0 k ax))
("pao" nil (p aw1))
("paolella" nil (p aa0 ow0 l eh1 l ax))
("paoletti" nil (p aa0 ow0 l eh1 t iy0))
("paoli" nil (p aw1 l iy0))
("paolillo" nil (p aa0 ow0 l ih1 l ow0))
("paolini" nil (p aa0 ow0 l iy1 n iy0))
("paolino" nil (p aa0 ow0 l iy1 n ow0))
("paolo" nil (p aw1 l ow0))
("paolucci" nil (p aa0 ow0 l uw1 ch iy0))
("paone" nil (p ey1 ow1 n))
("paonessa" nil (p aa0 ow0 n eh1 s ax))
("pap" nil (p ae1 p))
("papa" nil (p aa1 p ax))
("papacy" nil (p ey1 p ax s iy0))
("papadakis" nil (p aa0 p aa0 d aa1 k ih0 s))
("papadopoulos" nil (p ae1 p ax d aa1 p ax l ax s))
("papageorge" nil (p ae1 p ax jh ao1 r jh))
("papago" nil (p aa1 p ax g ow1))
("papal" nil (p ey1 p ax l))
("papale" nil (p aa0 p aa1 l iy0))
("papaleo" nil (p aa0 p ey0 l iy1 ax))
("papalia" nil (p aa0 p aa1 l iy0 ax))
("papandrea" nil (p aa0 p aa1 n d r iy0 ax))
("papandreou" nil (p ax p ae1 n d r iy0 uw1))
("papania" nil (p aa0 p aa1 n iy0 ax))
("paparella" nil (p aa0 p aa0 r eh1 l ax))
("papas" nil (p aa1 p ax z))
("papaw" nil (p ao1 p ao0))
("papay" nil (p ae1 p ey0))
("papaya" nil (p ax p ay1 ax))
("papayas" nil (p ax p ay1 ax z))
("papazian" nil (p ax p ey1 z iy0 ax n))
("pape" nil (p ey1 p))
("papenfuss" nil (p ae1 p ax n f ah1 s))
("paper" nil (p ey1 p er0))
("paperback" nil (p ey1 p er0 b ae1 k))
("paperbacks" nil (p ey1 p er0 b ae1 k s))
("paperboard" nil (p ey1 p er0 b ao1 r d))
("paperboy" nil (p ey1 p er0 b oy1))
("paperboys" nil (p ey1 p er0 b oy1 z))
("papered" nil (p ey1 p er0 d))
("papering" nil (p ey1 p er0 ih0 ng))
("paperless" nil (p ey1 p er0 l ax s))
("papermaking" nil (p ae1 p er0 m ey1 k ih0 ng))
("papermate" nil (p ey1 p er0 m ey1 t))
("papers" nil (p ey1 p er0 z))
("paperweight" nil (p ey1 p er0 w ey1 t))
("paperwork" nil (p ey1 p er0 w er1 k))
("paperworkers" nil (p ey1 p er0 w er1 k er0 z))
("papery" nil (p ey1 p er0 iy0))
("papesh" nil (p ae1 p ih0 sh))
("papier" nil (p ey1 p y er0))
("papilloma" nil (p ae1 p ih0 l ow1 m ax))
("papin" nil (p aa0 p iy1 n))
("papineau" nil (p ae1 p ih0 n ow1))
("papini" nil (p aa0 p iy1 n iy0))
("papke" nil (p ey1 p k))
("papp" nil (p ae1 p))
("pappa" nil (p ae1 p ax))
("pappadio" nil (p ax p ey1 d iy0 ow0))
("pappalardo" nil (p aa0 p aa0 l aa1 r d ow0))
("pappas" nil (p ae1 p ax s))
("pappert" nil (p ae1 p er0 t))
("paprika" nil (p ae0 p r iy1 k ax))
("paprocki" nil (p ax p r ow1 t s k iy0))
("papson" nil (p ae1 p s ax n))
("papua" nil (p ae1 p y uw0 ax))
("paque" nil (p ae1 k))
("paquet" nil (p aa1 k ey0 t))
("paquette" nil (p ax k eh1 t))
("paquin" nil (p ae1 k w ih0 n))
("par" nil (p aa1 r))
("para" nil (p eh1 r ax))
("parable" nil (p eh1 r ax b ax l))
("parables" nil (p eh1 r ax b ax l z))
("parabola" nil (p er0 ae1 b ax l ax))
("parabolic" nil (p eh1 r ax b aa1 l ih0 k))
("parachute" nil (p eh1 r ax sh uw1 t))
("parachuted" nil (p eh1 r ax sh uw1 t ih0 d))
("parachutes" nil (p eh1 r ax sh uw1 t s))
("parachuting" nil (p eh1 r ax sh uw1 t ih0 ng))
("parada" nil (p er0 ae1 d ax))
("parade" nil (p er0 ey1 d))
("paraded" nil (p er0 ey1 d ih0 d))
("paradee" nil (p eh1 r ax d iy0))
("parades" nil (p er0 ey1 d z))
("paradigm" nil (p eh1 r ax d ay1 m))
("paradigms" nil (p eh1 r ax d ay1 m z))
("parading" nil (p er0 ey1 d ih0 ng))
("paradis" nil (p er0 ae1 d ih0 s))
("paradise" nil (p eh1 r ax d ay1 s))
("paradiso" nil (p er0 ax d iy1 s ow0))
("parador" nil (p eh1 r ax d ao1 r))
("paradowski" nil (p er0 ax d ao1 f s k iy0))
("paradox" nil (p eh1 r ax d aa1 k s))
("paradoxes" nil (p eh1 r ax d aa1 k s ih0 z))
("paradoxical" nil (p eh1 r ax d aa1 k s ih0 k ax l))
("paradoxically" nil (p eh1 r ax d aa1 k s ax k l iy0))
("parady" nil (p er0 ae1 d iy0))
("paradyne" nil (p eh1 r ax d ay1 n))
("paraffin" nil (p eh1 r ax f ax n))
("paragon" nil (p eh1 r ax g aa1 n))
("paragraph" nil (p ae1 r ax g r ae1 f))
("paragraphs" nil (p ae1 r ax g r ae1 f s))
("paraguay" nil (p eh1 r ax g w ey1))
("paraguayan" nil (p eh1 r ax g w ay1 ax n))
("parakeet" nil (p eh1 r ax k iy1 t))
("parakeets" nil (p eh1 r ax k iy1 t s))
("paralegal" nil (p eh1 r ax l iy1 g ax l))
("paralegals" nil (p eh1 r ax l iy1 g ax l z))
("parallel" nil (p eh1 r ax l eh1 l))
("paralleled" nil (p eh1 r ax l eh1 l d))
("paralleling" nil (p eh1 r ax l eh1 l ih0 ng))
("parallelism" nil (p eh1 r ax l eh1 l ih1 z ax m))
("parallels" nil (p eh1 r ax l eh1 l z))
("paralympic" nil (p eh1 r ax l ih1 m p ih0 k))
("paralysis" nil (p er0 ae1 l ax s ax s))
("paralytic" nil (p eh1 r ax l ih1 t ih0 k))
("paralyze" nil (p eh1 r ax l ay1 z))
("paralyzed" nil (p eh1 r ax l ay1 z d))
("paralyzes" nil (p eh1 r ax l ay1 z ih0 z))
("paralyzing" nil (p eh1 r ax l ay1 z ih0 ng))
("paramagnetic" nil (p ae1 r ax m ae1 g n eh1 t ih0 k))
("paramax" nil (p eh1 r ax m ae0 k s))
("paramecium" nil (p eh1 r ax m iy1 s iy0 ax m))
("paramedics" nil (p eh1 r ax m eh1 d ih0 k s))
("parameter" nil (p er0 ae1 m ax t er0))
("parameters" nil (p er0 ae1 m ax t er0 z))
("parametric" nil (p eh1 r ax m eh1 t r ih0 k))
("paramilitaries" nil (p eh1 r ax m ih1 l ax t eh1 r iy0 z))
("paramilitary" nil (p eh1 r ax m ih1 l ax t eh1 r iy0))
("paramo" nil (p er0 ae1 m ow0))
("paramonova" nil (p eh1 r ax m ax n ow1 v ax))
("paramore" nil (p eh1 ax m ao1 r))
("paramount" nil (p eh1 r ax m aw1 n t))
("paramus" nil (p er0 ae1 m ax s))
("parana" nil (p er0 aa1 n ax))
("paranoia" nil (p eh1 r ax n oy1 ax))
("paranoiac" nil (p eh1 r ax n oy1 ae1 k))
("paranoid" nil (p eh1 r ax n oy1 d))
("paraphernalia" nil (p eh1 r ax f ax n ey1 l y ax))
("paraphrase" nil (p eh1 r ax f r ey1 z))
("paraphrasing" nil (p eh1 r ax f r ey1 z ih0 ng))
("paraplegia" nil (p eh1 r ax p l iy1 jh iy0 ax))
("paraplegic" nil (p eh1 r ax p l eh1 g ih0 k))
("parasite" nil (p eh1 r ax s ay1 t))
("parasites" nil (p eh1 r ax s ay1 t s))
("parasitic" nil (p eh1 r ax s ih1 t ih0 k))
("parasympathetic" nil (p eh1 r ax s ih1 m p ax th eh1 t ih0 k))
("paratore" nil (p er0 ax t ao1 r iy0))
("paratrooper" nil (p eh1 r ax t r uw1 p er0))
("paratroopers" nil (p eh1 r ax t r uw1 p er0 z))
("paratroops" nil (p eh1 r ax t r uw1 p s))
("parazoa" nil (p eh1 r ax z ow1 ax))
("parboil" nil (p aa1 r b oy1 l))
("parboiling" nil (p aa1 r b oy1 l ih0 ng))
("parc" nil (p aa1 r k))
("parcel" nil (p aa1 r s ax l))
("parceled" nil (p aa1 r s ax l d))
("parceling" nil (p aa1 r s ax l ih0 ng))
("parcell" nil (p aa0 r s ey1 l))
("parcells" nil (p aa1 r s ax l z))
("parcels" nil (p aa1 r s ax l z))
("parched" nil (p aa1 r ch t))
("parcher" nil (p aa1 r ch er0))
("parchman" nil (p aa1 r k m ax n))
("parchment" nil (p aa1 r ch m ax n t))
("parchmentlike" nil (p aa1 r ch m ax n t l ay1 k))
("parcplace" nil (p aa1 r k p l ey1 s))
("parde" nil (p aa1 r d))
("pardee" nil (p aa1 r d iy0))
("parden" nil (p aa1 r d ax n))
("pardi" nil (p aa1 r d iy0))
("pardini" nil (p aa0 r d iy1 n iy0))
("pardo" nil (p aa1 r d ow0))
("pardoe" nil (p aa1 r d ow1))
("pardon" nil (p aa1 r d ax n))
("pardoned" nil (p aa1 r d ax n d))
("pardoning" nil (p aa1 r d ax n ih0 ng))
("pardons" nil (p aa1 r d ax n z))
("pardue" nil (p aa1 r d uw1))
("pardus" nil (p aa1 r d ax s))
("pardy" nil (p aa1 r d iy0))
("pare" nil (p eh1 r))
("pared" nil (p eh1 r d))
("paredes" nil (p aa0 r ey1 d eh0 s))
("paredez" nil (p aa0 r ey1 d eh0 z))
("parekh" nil (p aa1 r ih0 k))
("parella" nil (p aa0 r eh1 l ax))
("parent" nil (p eh1 r ax n t))
("parentage" nil (p eh1 r ax n t ax jh))
("parental" nil (p er0 eh1 n t ax l))
("parente" nil (p aa0 r eh1 n t iy0))
("parenteau" nil (p aa0 r eh1 n t ow0))
("parentheses" nil (p er0 eh1 n th ax s iy1 z))
("parenthesis" nil (p er0 eh1 n th ax s ih0 s))
("parenthood" nil (p eh1 r ax n t hh uh1 d))
("parenti" nil (p aa0 r eh1 n t iy0))
("parenting" nil (p eh1 r ax n t ih0 ng))
("parents" nil (p eh1 r ax n t s))
("pares" nil (p eh1 r z))
("pareti" nil (p eh1 r eh1 t iy0))
("paretti" nil (p eh1 r eh1 t iy0))
("parfitt" nil (p aa1 r f ih0 t))
("parfums" nil (p er0 f y uw1 m z))
("parga" nil (p aa1 r g ax))
("pargesa" nil (p aa0 r jh eh1 s ax))
("parham" nil (p aa1 r hh ax m))
("pariah" nil (p er0 ay1 ax))
("pariahs" nil (p er0 ay1 ax z))
("paribas" nil (p eh0 r iy1 b ax s))
("parietal" nil (p er0 ay1 ax t ax l))
("parikh" nil (p ae1 r ih0 k))
("parillo" nil (p aa0 r ih1 l ow0))
("parimutuel" nil (p eh1 r ih0 m y uw1 ch uw0 ax l))
("paring" nil (p eh1 r ih0 ng))
("paris" nil (p eh1 r ih0 s))
("parise" nil (p aa1 r ay0 z))
("pariseau" nil (p ae1 r ih0 s ow0))
("parish" nil (p ae1 r ih0 sh))
("parishes" nil (p ae1 r ih0 sh ih0 z))
("parishioner" nil (p er0 ih1 sh ax n er0))
("parishioners" nil (p er0 ih1 sh ax n er0 z))
("parisi" nil (p aa0 r iy1 s iy0))
("parisian" nil (p er0 ih1 zh ax n))
("parisians" nil (p er0 ih1 s iy0 ax n z))
("parisienne" nil (p er0 ih1 s iy0 eh1 n))
("parities" nil (p eh1 r ax t iy0 z))
("parity" nil (p eh1 r ax t iy0))
("parizeau" nil (p eh1 r ih0 z ow1))
("parizek" nil (p er0 ih1 z eh0 k))
("parizo" nil (p aa0 r iy1 z ow0))
("park" nil (p aa1 r k))
("parke" nil (p aa1 r k))
("parked" nil (p aa1 r k t))
("parker" nil (p aa1 r k er0))
("parkers" nil (p aa1 r k er0 z))
("parkersburg" nil (p aa1 r k er0 z b er0 g))
("parkerson" nil (p aa1 r k er0 s ax n))
("parkes" nil (p aa1 r k s))
("parkey" nil (p aa1 r k iy1))
("parkhill" nil (p aa1 r k hh ih1 l))
("parkhurst" nil (p aa1 r k hh er0 s t))
("parkin" nil (p aa1 r k ih0 n))
("parking" nil (p aa1 r k ih0 ng))
("parkins" nil (p aa1 r k ih0 n z))
("parkinson" nil (p aa1 r k ih0 n s ax n))
("parkison" nil (p aa1 r k ih0 s ax n))
("parkland" nil (p aa1 r k l ae1 n d))
("parkman" nil (p aa1 r k m ax n))
("parks" nil (p aa1 r k s))
("parkside" nil (p aa1 r k s ay1 d))
("parkway" nil (p aa1 r k w ey1))
("parlance" nil (p aa1 r l ax n s))
("parlato" nil (p aa0 r l aa1 t ow0))
("parlay" nil (p aa1 r l ey0))
("parlayed" nil (p aa1 r l ey1 d))
("parlaying" nil (p aa1 r l ey0 ih0 ng))
("parlee" nil (p aa1 r l iy1))
("parler" nil (p aa1 r l er0))
("parlett" nil (p aa1 r l ih0 t))
("parlette" nil (p aa1 r l eh1 t))
("parley" nil (p aa1 r l iy0))
("parliament" nil (p aa1 r l ax m ax n t))
("parliamentarian" nil (p aa1 r l ax m ax n t eh1 r iy0 ax n))
("parliamentarians" nil (p aa1 r l ax m eh0 n t eh1 r iy0 ax n z))
("parliamentary" nil (p aa1 r l ax m eh1 n t er0 iy0))
("parliaments" nil (p aa1 r l ax m ax n t s))
("parlier" nil (p aa1 r l iy0 er0))
("parlin" nil (p aa1 r l ih0 n))
("parlodel" nil (p aa1 r l ow1 d eh1 l))
("parlor" nil (p aa1 r l er0))
("parlors" nil (p aa1 r l er0 z))
("parlour" nil (p aa1 r l er0))
("parlow" nil (p aa1 r l ow1))
("parma" nil (p aa1 r m ax))
("parmalat" nil (p aa1 r m ax l ae1 t))
("parman" nil (p aa1 r m ax n))
("parmar" nil (p aa1 r m aa1 r))
("parmele" nil (p aa0 r m eh1 l iy0))
("parmelee" nil (p aa1 r m ih0 l iy0))
("parmenter" nil (p aa1 r m ax n t er0))
("parmentier" nil (p aa1 r m ax n t iy0 er0))
("parmer" nil (p aa1 r m er0))
("parmesan" nil (p aa1 r m ax z aa1 n))
("parmeter" nil (p aa1 r m ih0 t er0))
("parmley" nil (p aa1 r m l iy0))
("parnassus" nil (p aa1 r n ae1 s ax s))
("parnell" nil (p aa1 r n ax l))
("parnella" nil (p aa0 r n eh1 l ax))
("parnes" nil (p aa1 r n z))
("parness" nil (p aa1 r n ax s))
("paro" nil (p aa1 r ow0))
("parochial" nil (p er0 ow1 k iy0 ax l))
("parochialism" nil (p er0 ow1 k iy0 ax l ih1 z ax m))
("parodi" nil (p aa0 r ow1 d iy0))
("parodied" nil (p eh1 r ax d iy0 d))
("parodies" nil (p eh1 r ax d iy0 z))
("parodist" nil (p ae1 r ax d ih0 s t))
("parody" nil (p eh1 r ax d iy0))
("parole" nil (p er0 ow1 l))
("paroled" nil (p er0 ow1 l d))
("parolees" nil (p er0 ow1 l iy1 z))
("paroles" nil (p er0 ow1 l z))
("paroling" nil (p er0 ow1 l ih0 ng))
("parpart" nil (p aa1 r p aa1 r t))
("parquet" nil (p aa0 r k ey1))
("parr" nil (p aa1 r))
("parra" nil (p aa1 r ax))
("parrack" nil (p ae1 r ax k))
("parramore" nil (p aa0 r aa1 m ao0 r))
("parras" nil (p eh1 r ax z))
("parrella" nil (p aa0 r eh1 l ax))
("parrent" nil (p aa1 r ax n t))
("parrett" nil (p ae1 r ih0 t))
("parretti" nil (p er0 eh1 t iy0))
("parried" nil (p eh1 r iy0 d))
("parrill" nil (p ae1 r ax l))
("parrilla" nil (p aa1 r ih1 l ax))
("parrillo" nil (p aa1 r ih1 l ow0))
("parrinello" nil (p aa0 r iy0 n eh1 l ow0))
("parrino" nil (p aa1 r iy1 n ow0))
("parriott" nil (p ae1 r iy0 ax t))
("parris" nil (p eh1 r ih0 s))
("parrish" nil (p eh1 r ih0 sh))
("parrishes" nil (p eh1 r ih0 sh ih0 z))
("parrot" nil (p eh1 r ax t))
("parrots" nil (p eh1 r ax t s))
("parrott" nil (p eh1 r ax t))
("parrotta" nil (p aa0 r ow1 t ax))
("parrow" nil (p ae1 r ow0))
("parry" nil (p eh1 r iy0))
("pars" nil (p aa1 r z))
("parse" nil (p aa1 r s))
("parsec" nil (p aa1 r s eh1 k))
("parsefal" nil (p aa1 r s ih0 f ax l))
("parsell" nil (p aa1 r s ax l))
("parshall" nil (p aa1 r sh ax l))
("parshley" nil (p aa1 r sh l iy0))
("parsifal" nil (p aa1 r s ih0 f ax l))
("parsimony" nil (p aa1 r s ax m ow1 n iy0))
("parsing" nil (p aa1 r s ih0 ng))
("parsippany" nil (p aa0 r s ih1 p ax n iy0))
("parsky" nil (p aa1 r s k iy0))
("parsley" nil (p aa1 r s l iy0))
("parslow" nil (p aa1 r s l ow1))
("parson" nil (p aa1 r s ax n))
("parsonage" nil (p aa1 r s ax n ih0 jh))
("parsons" nil (p aa1 r s ax n z))
("parsow" nil (p aa1 r s ow0))
("part" nil (p aa1 r t))
("partain" nil (p aa0 r t ey1 n))
("partake" nil (p aa0 r t ey1 k))
("partch" nil (p aa1 r ch))
("partecipazioni" nil (p aa1 r t eh1 s ih0 p ae1 z iy0 ow1 n iy0))
("parted" nil (p aa1 r t ax d))
("partee" nil (p aa1 r t iy1))
("parten" nil (p aa1 r t ax n))
("partenope" nil (p aa1 r t ax n ow1 p))
("parthenia" nil (p aa0 r th eh1 n iy0 ax))
("parthenogenetic" nil (p aa1 r th ax n ow0 jh ax n eh1 t ih0 k))
("parthenon" nil (p aa1 r th ax n aa1 n))
("parthian" nil (p aa1 r th iy0 ax n))
("parti" nil (p aa1 r t iy0))
("partial" nil (p aa1 r sh ax l))
("partiality" nil (p aa1 r sh iy0 ae1 l ax t iy0))
("partially" nil (p aa1 r sh ax l iy0))
("participant" nil (p aa0 r t ih1 s ax p ax n t))
("participants" nil (p aa0 r t ih1 s ax p ax n t s))
("participate" nil (p aa0 r t ih1 s ax p ey1 t))
("participated" nil (p aa0 r t ih1 s ax p ey1 t ax d))
("participates" nil (p aa0 r t ih1 s ax p ey1 t s))
("participating" nil (p aa0 r t ih1 s ax p ey1 t ih0 ng))
("participation" nil (p aa0 r t ih1 s ax p ey1 sh ax n))
("participations" nil (p aa0 r t ih1 s ax p ey1 sh ax n z))
("participative" nil (p aa1 r t ih1 s ax p ax t ih0 v))
("participatory" nil (p aa1 r t ih1 s ax p ax t ao1 r iy0))
("particle" nil (p aa1 r t ax k ax l))
("particles" nil (p aa1 r t ax k ax l z))
("particular" nil (p er0 t ih1 k y ax l er0))
("particularity" nil (p er0 t ih1 k y ax l ae1 r ax t iy0))
("particularly" nil (p aa1 r t ih1 k y ax l er0 l iy0))
("particulars" nil (p er0 t ih1 k y ax l er0 z))
("particulary" nil (p aa1 r t ih1 k y ax l eh1 r iy0))
("particulate" nil (p er0 t ih1 k y ax l ax t))
("particulates" nil (p aa1 r t ih1 k y ax l ax t s))
("partida" nil (p aa0 r t iy1 d ax))
("partido" nil (p aa0 r t iy1 d ow0))
("parties" nil (p aa1 r t iy0 z))
("partin" nil (p aa1 r t ih0 n))
("parting" nil (p aa1 r t ih0 ng))
("partington" nil (p aa1 r t ih0 ng t ax n))
("partisan" nil (p aa1 r t ax z ax n))
("partisans" nil (p aa1 r t ax z ax n z))
("partisanship" nil (p aa1 r t ax z ax n sh ih1 p))
("partition" nil (p aa0 r t ih1 sh ax n))
("partitioned" nil (p aa0 r t ih1 sh ax n d))
("partitions" nil (p aa0 r t ih1 sh ax n z))
("partlow" nil (p aa1 r t l ow1))
("partly" nil (p aa1 r t l iy0))
("partner" nil (p aa1 r t n er0))
("partnering" nil (p aa1 r t n er0 ih0 ng))
("partners" nil (p aa1 r t n er0 z))
("partnership" nil (p aa1 r t n er0 sh ih1 p))
("partnerships" nil (p aa1 r t n er0 sh ih1 p s))
("partney" nil (p aa1 r t n iy0))
("parton" nil (p aa1 r t ax n))
("partridge" nil (p aa1 r t r ax jh))
("partridges" nil (p aa1 r t r ih0 jh ih0 z))
("parts" nil (p aa1 r t s))
("parttime" nil (p aa1 r t t ay1 m))
("party" nil (p aa1 r t iy0))
("partying" nil (p aa1 r t iy0 ih0 ng))
("partyka" nil (p er0 t ih1 k ax))
("parvin" nil (p aa1 r v ih0 n))
("parziale" nil (p aa1 r z iy0 aa1 l iy0))
("parzych" nil (p aa1 r z ih0 hh))
("pas" nil (p aa1 z))
("pasadena" nil (p ae1 s ax d iy1 n er0))
("pascal" nil (p ae0 s k ae1 l))
("pascale" nil (p ae0 s k ae1 l))
("pascarella" nil (p aa0 s k aa0 r eh1 l ax))
("pascarelli" nil (p aa0 s k aa0 r eh1 l iy0))
("pasch" nil (p ae1 s k))
("paschal" nil (p ae1 sh ax l))
("paschall" nil (p ae1 sh ax l))
("paschen" nil (p ae1 sh ax n))
("paschke" nil (p ae1 sh k))
("pasco" nil (p aa1 s k ow0))
("pascoe" nil (p ae1 s k ow0))
("pascua" nil (p aa0 s k uw1 ax))
("pascual" nil (p ae1 s k uw0 ax l))
("pascucci" nil (p aa0 s k uw1 ch iy0))
("pascutto" nil (p ax s k y uw1 t ow0))
("pascuzzi" nil (p aa0 s k uw1 t s iy0))
("pasek" nil (p aa1 s eh0 k))
("paseo" nil (p aa0 s ey1 ow1))
("pash" nil (p ae1 sh))
("pasha" nil (p ax sh aa1))
("pashley" nil (p ae1 sh l iy0))
("pasillas" nil (p aa0 s ih1 l ax z))
("pasion" nil (p aa0 zh iy1 n))
("pask" nil (p ae1 s k))
("paske" nil (p ey1 s k))
("paskey" nil (p ae1 s k iy0))
("pasko" nil (p aa1 s k ow0))
("paslay" nil (p ae1 s l ey0))
("pasley" nil (p ae1 s l iy0))
("pasman" nil (p ae1 s m ax n))
("paso" nil (p ae1 s ow0))
("pasok" nil (p aa1 z aa0 k))
("pasqua" nil (p aa1 s k w ax))
("pasquale" nil (p aa0 s k w aa1 l ey0))
("pasquarella" nil (p aa0 s k w aa0 r eh1 l ax))
("pasquarelli" nil (p aa0 s k w aa0 r eh1 l iy0))
("pasquarello" nil (p aa0 s k w aa0 r eh1 l ow0))
("pasquariello" nil (p aa0 s k w aa0 r iy0 eh1 l ow0))
("pasque" nil (p ae1 s k))
("pasquinelli" nil (p aa0 s k w iy0 n eh1 l iy0))
("pasquini" nil (p aa0 s k w iy1 n iy0))
("pass" nil (p ae1 s))
("passable" nil (p ae1 s ax b ax l))
("passably" nil (p ae1 s ax b l iy0))
("passafiume" nil (p aa0 s aa0 f iy1 uw0 m))
("passage" nil (p ae1 s ax jh))
("passages" nil (p ae1 s ax jh ax z))
("passageway" nil (p ae1 s ax jh w ey1))
("passaic" nil (p ax s ey1 ih0 k))
("passalacqua" nil (p ae1 s ax l ae1 k w ax))
("passanisi" nil (p aa0 s aa0 n iy1 s iy0))
("passante" nil (p aa0 s aa1 n t iy0))
("passantino" nil (p aa0 s aa0 n t iy1 n ow0))
("passarella" nil (p aa0 s aa0 r eh1 l ax))
("passarelli" nil (p aa0 s aa0 r eh1 l iy0))
("passaretti" nil (p aa0 s aa0 r eh1 t iy0))
("passaro" nil (p aa0 s aa1 r ow0))
("passat" nil (p ae1 s ae0 t))
("passbook" nil (p ae1 s b uh1 k))
("passe" nil (p ae1 s ey1))
("passed" nil (p ae1 s t))
("passel" nil (p ae1 s ih0 l))
("passenger" nil (p ae1 s ax n jh er0))
("passengers" nil (p ae1 s ax n jh er0 z))
("passer" nil (p ae1 s er0))
("passero" nil (p aa0 s eh1 r ow0))
("passers" nil (p ae1 s er0 z))
("passersby" nil (p ae1 s er0 z b iy0))
("passes" nil (p ae1 s ax z))
("passey" nil (p ae1 s iy0))
("passim" nil (p ae0 s iy1 m))
("passing" nil (p ae1 s ih0 ng))
("passino" nil (p aa0 s iy1 n ow0))
("passion" nil (p ae1 sh ax n))
("passionate" nil (p ae1 sh ax n ax t))
("passionately" nil (p ae1 sh ax n ax t l iy0))
("passions" nil (p ae1 sh ax n z))
("passive" nil (p ae1 s ih0 v))
("passively" nil (p ae1 s ih0 v l iy0))
("passivity" nil (p ax s ih1 v ih0 t iy0))
("passman" nil (p ae1 s m ax n))
("passmore" nil (p aa1 s m ao0 r))
("passon" nil (p ae1 s ax n))
("passover" nil (p ae1 s ow1 v er0))
("passow" nil (p ae1 s ow0))
("passport" nil (p ae1 s p ao1 r t))
("passports" nil (p ae1 s p ao1 r t s))
("password" nil (p ae1 s w er1 d))
("passwords" nil (p ae1 s w er1 d z))
("past" nil (p ae1 s t))
("pasta" nil (p aa1 s t ax))
("pastas" nil (p aa1 s t ax z))
("paste" nil (p ey1 s t))
("pasted" nil (p ey1 s t ih0 d))
("pastel" nil (p ae0 s t eh1 l))
("pastels" nil (p ae0 s t eh1 l z))
("paster" nil (p ae1 s t er0))
("pasternack" nil (p ae1 s t er0 n ae0 k))
("pasternak" nil (p ae1 s t er0 n ae0 k))
("pastes" nil (p ey1 s t s))
("pasteur" nil (p ax s t uw1 r))
("pasteurization" nil (p ae1 s ch er0 ax z ey1 sh ax n))
("pasteurized" nil (p ae1 s ch er0 ay1 z d))
("pastiche" nil (p ae1 s t iy1 sh))
("pastime" nil (p ae1 s t ay1 m))
("pastimes" nil (p ae1 s t ay1 m z))
("pasting" nil (p ey1 s t ih0 ng))
("pastor" nil (p ae1 s t er0))
("pastora" nil (p ae0 s t ao1 r ax))
("pastoral" nil (p ae1 s t er0 ax l))
("pastoralism" nil (p ae1 s t er0 ax l ih1 z ax m))
("pastore" nil (p ae1 s t ao1 r))
("pastorino" nil (p aa0 s t ao0 r iy1 n ow0))
("pastorius" nil (p ae1 s t ao0 r iy0 ih0 s))
("pastors" nil (p ae1 s t er0 z))
("pastrami" nil (p ax s t r aa1 m iy0))
("pastrana" nil (p aa0 s t r ae1 n ax))
("pastries" nil (p ey1 s t r iy0 z))
("pastry" nil (p ey1 s t r iy0))
("pastula" nil (p aa0 s t uw1 l ax))
("pasture" nil (p ae1 s ch er0))
("pastures" nil (p ae1 s ch er0 z))
("paszek" nil (p aa1 sh eh0 k))
("paszkiewicz" nil (p aa1 sh k ax v ih0 ch))
("pasztor" nil (p ae1 s t er0))
("pat" nil (p ae1 t))
("patagonia" nil (p ae1 t ax g ow1 n iy0 ax))
("patagonian" nil (p ae1 t ax g ow1 n iy0 ax n))
("patak" nil (p ae1 t ax k))
("pataki" nil (p ax t aa1 k iy0))
("pataky" nil (p ae1 t ax k iy0))
("patalano" nil (p aa0 t aa0 l aa1 n ow0))
("patane" nil (p ae1 t ax n))
("patch" nil (p ae1 ch))
("patched" nil (p ae1 ch t))
("patchell" nil (p ae1 ch ax l))
("patchen" nil (p ae1 ch ax n))
("patches" nil (p ae1 ch ax z))
("patchett" nil (p ae1 ch ih0 t))
("patchin" nil (p ae1 ch ih0 n))
("patching" nil (p ae1 ch ih0 ng))
("patchwork" nil (p ae1 ch w er1 k))
("patchworks" nil (p ae1 ch w er1 k s))
("patchy" nil (p ae1 ch iy0))
("patco" nil (p ae1 t k ow0))
("pate" nil (p ey1 t))
("patek" nil (p aa1 t eh0 k))
("patel" nil (p ax t eh1 l))
("patella" nil (p ax t eh1 l ax))
("patenaude" nil (p ae1 t ih0 n ow0 d))
("patent" nil (p ae1 t ax n t))
("patentable" nil (p ae1 t ax n t ax b ax l))
("patented" nil (p ae1 t ax n t ax d))
("patenting" nil (p ae1 t ax n t ih0 ng))
("patently" nil (p ae1 t ax n t l iy0))
("patents" nil (p ae1 t ax n t s))
("pater" nil (p ey1 t er0))
("patera" nil (p aa0 t eh1 r ax))
("paternal" nil (p ax t er1 n ax l))
("paternalism" nil (p ax t er1 n ax l ih1 z ax m))
("paternalistic" nil (p ax t er1 n ax l ih1 s t ih0 k))
("paternity" nil (p ax t er1 n ih0 t iy0))
("paterno" nil (p aa0 t eh1 r n ow0))
("paternostro" nil (p aa0 t er0 n ow1 s t r ow0))
("paterson" nil (p ae1 t er0 s ax n))
("pates" nil (p ey1 t s))
("patey" nil (p ey1 t iy0))
("path" nil (p ae1 th))
("pathan" nil (p ae1 th ax n))
("pathans" nil (p ae1 th ax n z))
("pathe" nil (p ae1 th))
("pathetic" nil (p ax th eh1 t ih0 k))
("pathetically" nil (p ax th eh1 t ih0 k ax l iy0))
("pathfinder" nil (p ae1 th f ay1 n d er0))
("pathmark" nil (p ae1 th m aa1 r k))
("pathogenic" nil (p ae1 th ax jh eh1 n ih0 k))
("pathogens" nil (p ae1 th ax jh ax n z))
("pathological" nil (p ae1 th ax l aa1 jh ih0 k ax l))
("pathologies" nil (p ax th aa1 l ax jh iy0 z))
("pathologist" nil (p ax th aa1 l ax jh ax s t))
("pathologists" nil (p ax th aa1 l ax jh ax s t s))
("pathology" nil (p ax th aa1 l ax jh iy0))
("pathos" nil (p ey1 th aa0 s))
("paths" nil (p ae1 dh z))
("pathway" nil (p ae1 th w ey1))
("pathways" nil (p ae1 th w ey1 z))
("patience" nil (p ey1 sh ax n s))
("patient" nil (p ey1 sh ax n t))
("patiently" nil (p ey1 sh ax n t l iy0))
("patients" nil (p ey1 sh ax n t s))
("patillo" nil (p ax t ih1 l ow0))
("patin" nil (p ae1 t ih0 n))
("patina" nil (p ax t iy1 n ax))
("patinkin" nil (p ax t ih1 ng k ih0 n))
("patino" nil (p aa0 t iy1 n ow0))
("patio" nil (p ae1 t iy0 ow1))
("patios" nil (p ae1 t iy0 ow1 z))
("patlan" nil (p ae1 t l ax n))
("patlex" nil (p ae1 t l eh0 k s))
("patman" nil (p ae1 t m ax n))
("patmore" nil (p ae1 t m ao0 r))
("patnaude" nil (p aa0 t n ao1 d iy0))
("patnode" nil (p ae1 t n ow1 d))
("paton" nil (p ae1 t ax n))
("patons" nil (p ae1 t ax n z))
("patriarca" nil (p aa0 t r iy0 aa1 r k ax))
("patriarch" nil (p ey1 t r iy0 aa1 r k))
("patriarchal" nil (p ey1 t r iy0 aa1 r k ax l))
("patriarchate" nil (p ey1 t r iy0 aa1 r k ax t))
("patriarchs" nil (p ey1 t r iy0 aa1 r k s))
("patriarchy" nil (p ey1 t r iy0 aa1 r k iy0))
("patrice" nil (p ax t r iy1 s))
("patricelli" nil (p aa0 t r iy0 ch eh1 l iy0))
("patricia" nil (p ax t r ih1 sh ax))
("patrician" nil (p ax t r ih1 sh ax n))
("patricians" nil (p ax t r ih1 sh ax n z))
("patricio" nil (p ax t r ih1 s iy0 ow0))
("patrick" nil (p ae1 t r ih0 k))
("patricks" nil (p ae1 t r ih0 k s))
("patricof" nil (p ae1 t r ih0 k ao1 f))
("patridge" nil (p ae1 t r ih1 jh))
("patrie" nil (p ae1 t er0 iy0))
("patrilineal" nil (p ae1 t r ih0 l ih1 n iy0 ax l))
("patrimony" nil (p ae1 t r ax m ow1 n iy0))
("patriot" nil (p ey1 t r iy0 ax t))
("patriotic" nil (p ey1 t r iy0 aa1 t ih0 k))
("patriotism" nil (p ey1 t r iy0 ax t ih1 z ax m))
("patriots" nil (p ey1 t r iy0 ax t s))
("patristic" nil (p ax t r ih1 s t ih0 k))
("patrizio" nil (p aa0 t r iy1 z iy0 ow0))
("patrol" nil (p ax t r ow1 l))
("patrolled" nil (p ax t r ow1 l d))
("patrolling" nil (p ax t r ow1 l ih0 ng))
("patrolman" nil (p ax t r ow1 l m ae1 n))
("patrolmen" nil (p ax t r ow0 l m eh1 n))
("patrols" nil (p ax t r ow1 l z))
("patron" nil (p ey1 t r ax n))
("patronage" nil (p ae1 t r ax n ih0 jh))
("patrone" nil (p aa0 t r ow1 n iy0))
("patronize" nil (p ey1 t r ax n ay1 z))
("patronized" nil (p ey1 t r ax n ay1 z d))
("patronizing" nil (p ey1 t r ax n ay1 z ih0 ng))
("patrons" nil (p ey1 t r ax n z))
("patronymic" nil (p ae1 t r ax n ih1 m ih0 k))
("patry" nil (p ae1 t r iy0))
("pats" nil (p ae1 t s))
("patsies" nil (p ae1 t s iy0 z))
("patsy" nil (p ae1 t s iy0))
("patt" nil (p ae1 t))
("patted" nil (p ae1 t ax d))
("pattee" nil (p ae1 t iy1))
("patten" nil (p ae1 t ax n))
("patter" nil (p ae1 t er0))
("pattered" nil (p ae1 t er0 d))
("pattern" nil (p ae1 t er0 n))
("patterned" nil (p ae1 t er0 n d))
("patterns" nil (p ae1 t er0 n z))
("patterson" nil (p ae1 t er0 s ax n))
("patteson" nil (p ae1 t ih0 s ax n))
("patti" nil (p ae1 t iy0))
("pattie" nil (p ae1 t iy0))
("patties" nil (p ae1 t iy0 z))
("pattillo" nil (p aa0 t ih1 l ow0))
("pattin" nil (p ae1 t ih0 n))
("patting" nil (p ae1 t ih0 ng))
("pattinson" nil (p ae1 t ih0 n s ax n))
("pattis" nil (p ae1 t ih0 s))
("pattison" nil (p ae1 t ih0 s ax n))
("pattiz" nil (p ae1 t ih0 z))
("patton" nil (p ae1 t ax n))
("patty" nil (p ae1 t iy0))
("paty" nil (p ey1 t iy0))
("patz" nil (p ae1 t s))
("patzer" nil (p ey1 t z er0))
("patzke" nil (p ae1 t z k))
("paucity" nil (p ao1 s ax t iy0))
("paugh" nil (p ao1))
("paul" nil (p ao1 l))
("paula" nil (p ao1 l ax))
("paulding" nil (p ao1 l d ih0 ng))
("paule" nil (p ao1 l))
("paules" nil (p ow1 l z))
("pauletta" nil (p aa0 l eh1 t ax))
("paulette" nil (p ao0 l eh1 t))
("pauley" nil (p ao1 l iy0))
("paulhus" nil (p aw1 l hh ih0 s))
("pauli" nil (p ao1 l iy0))
("paulick" nil (p ao1 l ih0 k))
("paulik" nil (p ao1 l ih0 k))
("paulin" nil (p ao1 l ih0 n))
("paulina" nil (p ao1 l iy1 n ax))
("pauline" nil (p ao0 l iy1 n))
("pauling" nil (p ao1 l ih0 ng))
("paulino" nil (p ao0 l iy1 n ow0))
("paulita" nil (p ao0 l iy1 t ax))
("paulk" nil (p ao1 l k))
("paull" nil (p ao1 l))
("paullin" nil (p ao1 l ih0 n))
("paulo" nil (p ao1 l ow0))
("paulos" nil (p aw1 l ow0 z))
("pauls" nil (p ao1 l z))
("paulsen" nil (p aw1 l s ax n))
("paulson" nil (p ao1 l s ax n))
("paulus" nil (p ao1 l ax s))
("pauly" nil (p ao1 l iy0))
("paunchy" nil (p ao1 n ch iy0))
("paup" nil (p ao1 p))
("pauper" nil (p ao1 p er0))
("pause" nil (p ao1 z))
("paused" nil (p ao1 z d))
("pauses" nil (p ao1 z ax z))
("pausing" nil (p ao1 z ih0 ng))
("paustian" nil (p ao1 z ch ih0 n))
("pautler" nil (p aw1 t ax l er0))
("pautsch" nil (p aw1 ch))
("pautz" nil (p ao1 t s))
("pauwels" nil (p aw1 w ax l z))
("pavao" nil (p aa1 v aw0))
("pavarotti" nil (p ae1 v er0 aa1 t iy0))
("pave" nil (p ey1 v))
("paved" nil (p ey1 v d))
("pavek" nil (p ae1 v ih0 k))
("pavel" nil (p ae1 v ax l))
("pavelka" nil (p ax v eh1 l k ax))
("pavelko" nil (p ax v eh1 l k ow0))
("pavement" nil (p ey1 v m ax n t))
("pavements" nil (p ey1 v m ax n t s))
("paver" nil (p ey1 v er0))
("paves" nil (p ey1 v z))
("pavese" nil (p aa0 v ey1 z iy0))
("pavey" nil (p ey1 v iy0))
("pavia" nil (p ey1 v iy0 ax))
("pavich" nil (p ae1 v ih0 ch))
("pavilion" nil (p ax v ih1 l y ax n))
("pavilions" nil (p ax v ih1 l y ax n z))
("pavillion" nil (p ax v ih1 l y ax n))
("paving" nil (p ey1 v ih0 ng))
("pavlak" nil (p aa1 v l ax k))
("pavlic" nil (p ae1 v l ih0 k))
("pavlica" nil (p ae1 v l ih0 k ax))
("pavlicek" nil (p aa1 v l ih0 ch eh0 k))
("pavlich" nil (p aa1 v l ih0 hh))
("pavlick" nil (p ae1 v l ih0 k))
("pavlik" nil (p ae1 v l ih0 k))
("pavlis" nil (p ae1 v l ih0 s))
("pavlock" nil (p ae1 v l ax k))
("pavlov" nil (p ae1 v l aa0 v))
("pavlovian" nil (p ae1 v l ow1 v iy0 ax n))
("pavlovic" nil (p ax v l aa1 v ih0 k))
("pavlovich" nil (p ae1 v l ax v ih0 ch))
("pavon" nil (p aa1 v ax n))
("pavone" nil (p ax v ow1 n))
("paw" nil (p ao1))
("pawelek" nil (p aa0 v eh1 l eh0 k))
("pawelski" nil (p aa0 v eh1 l s k iy0))
("pawlak" nil (p ao1 l ax k))
("pawley" nil (p ao1 l iy0))
("pawlicki" nil (p aa0 v l ih1 t s k iy0))
("pawlik" nil (p ao1 l ih0 k))
("pawlikowski" nil (p aa0 v l ih0 k ao1 f s k iy0))
("pawling" nil (p ao1 l ih0 ng))
("pawloski" nil (p aa0 v l aw1 s k iy0))
("pawlowicz" nil (p aa1 v l ax v ih0 ch))
("pawlowski" nil (p aa0 v l ao1 f s k iy0))
("pawluk" nil (p aa1 v l ax k))
("pawn" nil (p ao1 n))
("pawned" nil (p ao1 n d))
("pawnee" nil (p ao1 n iy1))
("pawnees" nil (p ao1 n iy1 z))
("pawns" nil (p ao1 n z))
("pawnshop" nil (p ao1 n sh aa1 p))
("pawnshops" nil (p ao1 n sh aa1 p s))
("paws" nil (p ao1 z))
("pawson" nil (p ao1 s ax n))
("pawtucket" nil (p ao1 t ah1 k ih0 t))
("pawtuxet" nil (p ao1 t ah1 k s ax t))
("pax" nil (p ae1 k s))
("paxar" nil (p ae1 k s er0))
("paxman" nil (p ae1 k s m ax n))
("paxon" nil (p ae1 k s ax n))
("paxson" nil (p ae1 k s ax n))
("paxton" nil (p ae1 k s t ax n))
("pay" nil (p ey1))
("payable" nil (p ey1 ax b ax l))
("payables" nil (p ey1 ax b ax l z))
("payan" nil (p ey1 ax n))
("payback" nil (p ey1 b ae1 k))
("paycheck" nil (p ey1 ch eh1 k))
("paychecks" nil (p ey1 ch eh1 k s))
("paychex" nil (p ey1 ch eh1 k s))
("payco" nil (p ey1 k ow0))
("payday" nil (p ey1 d ey1))
("paye" nil (p ey1))
("payer" nil (p ey1 er0))
("payers" nil (p ey1 er0 z))
("payette" nil (p ey1 eh1 t))
("payeur" nil (p ey0 er1))
("paying" nil (p ey1 ih0 ng))
("payless" nil (p ey1 l eh1 s))
("payload" nil (p ey1 l ow1 d))
("payloads" nil (p ey1 l ow1 d z))
("paylor" nil (p ey1 l er0))
("payment" nil (p ey1 m ax n t))
("payments" nil (p ey1 m ax n t s))
("payne" nil (p ey1 n))
("paynter" nil (p ey1 n t er0))
("payoff" nil (p ey1 ao1 f))
("payoffs" nil (p ey1 ao1 f s))
("payola" nil (p ey1 ow1 l ax))
("payout" nil (p ey1 aw1 t))
("payouts" nil (p ey1 aw1 t s))
("payroll" nil (p ey1 r ow1 l))
("payrolls" nil (p ey1 r ow1 l z))
("pays" nil (p ey1 z))
("payson" nil (p ey1 z ax n))
("paysop" nil (p ey1 s aa1 p))
("paysops" nil (p ey1 s aa1 p s))
("payton" nil (p ey1 t ax n))
("paz" nil (p aa1 z))
("pazner" nil (p aa1 z n er0))
("pazos" nil (p aa1 z ow0 z))
("pea" nil (p iy1))
("peabody" nil (p iy1 b aa1 d iy0))
("peace" nil (p iy1 s))
("peaceable" nil (p iy1 s ax b ax l))
("peaceful" nil (p iy1 s f ax l))
("peacefully" nil (p iy1 s f ax l iy0))
("peacefulness" nil (p iy1 s f ax l n ax s))
("peacekeeper" nil (p iy1 s k iy1 p er0))
("peacekeepers" nil (p iy1 s k iy1 p er0 z))
("peacekeeping" nil (p iy1 s k iy1 p ih0 ng))
("peacemaker" nil (p iy1 s m ey1 k er0))
("peacemakers" nil (p iy1 s m ey1 k er0 z))
("peacemaking" nil (p iy1 s m ey1 k ih0 ng))
("peacetime" nil (p iy1 s t ay1 m))
("peach" nil (p iy1 ch))
("peacher" nil (p iy1 ch er0))
("peaches" nil (p iy1 ch ax z))
("peachey" nil (p iy1 ch iy0))
("peachtree" nil (p iy1 ch t r iy1))
("peachy" nil (p iy1 ch iy0))
("peacock" nil (p iy1 k aa1 k))
("peacocks" nil (p iy1 k aa1 k s))
("peaden" nil (p eh1 d ax n))
("peafowl" nil (p iy1 f aw1 l))
("peagler" nil (p iy1 g l er0))
("peak" nil (p iy1 k))
("peake" nil (p iy1 k))
("peaked" nil (p iy1 k t))
("peakes" nil (p iy1 k s))
("peaking" nil (p iy1 k ih0 ng))
("peaks" nil (p iy1 k s))
("peal" nil (p iy1 l))
("peale" nil (p iy1 l))
("pealer" nil (p iy1 l er0))
("peanut" nil (p iy1 n ax t))
("peanuts" nil (p iy1 n ax t s))
("peapack" nil (p iy1 p ae1 k))
("pear" nil (p eh1 r))
("pearce" nil (p ih1 r s))
("pearcy" nil (p er1 k iy0))
("pearl" nil (p er1 l))
("pearle" nil (p er1 l))
("pearline" nil (p er1 l ay0 n))
("pearling" nil (p er1 l ih0 ng))
("pearlman" nil (p er1 l m ae1 n))
("pearls" nil (p er1 l z))
("pearlstein" nil (p er1 l s t ay1 n))
("pearlstine" nil (p er1 l s t ay1 n))
("pearly" nil (p er1 l iy0))
("pearman" nil (p eh1 r m ax n))
("pears" nil (p eh1 r z))
("pearse" nil (p er1 s))
("pearson" nil (p ih1 r s ax n))
("peart" nil (p er1 t))
("peary" nil (p iy1 r iy0))
("peas" nil (p iy1 z))
("peasant" nil (p eh1 z ax n t))
("peasantry" nil (p eh1 z ax n t r iy0))
("peasants" nil (p eh1 z ax n t s))
("pease" nil (p iy1 z))
("peasey" nil (p iy1 z iy0))
("peaslee" nil (p iy1 z l iy1))
("peasley" nil (p iy1 z l iy0))
("peat" nil (p iy1 t))
("peatross" nil (p iy1 t r ax s))
("peaudouce" nil (p ow1 d uw1 s))
("peavey" nil (p iy1 v iy0))
("peavler" nil (p iy1 v l er0))
("peavy" nil (p iy1 v iy0))
("peay" nil (p iy1))
("pebble" nil (p eh1 b ax l))
("pebbles" nil (p eh1 b ax l z))
("pebereau" nil (p eh1 b er0 ow1))
("pebley" nil (p eh1 b l iy0))
("pecan" nil (p ax k aa1 n))
("pecans" nil (p ih0 k ae1 n z))
("pecci" nil (p eh1 ch iy0))
("pech" nil (p eh1 k))
("pecha" nil (p eh1 ch ax))
("pechacek" nil (p eh1 k ax s ih0 k))
("pechin" nil (p eh1 ch ih0 n))
("pechiney" nil (p eh1 ch ih0 n iy0))
("pechman" nil (p eh1 k m ax n))
("pechora" nil (p ax k ao1 r ax))
("pecht" nil (p eh1 k t))
("pecina" nil (p eh0 ch iy1 n ax))
("peck" nil (p eh1 k))
("pecka" nil (p eh1 k ax))
("peckenpaugh" nil (p ih0 k eh1 n p ao0))
("peckham" nil (p eh1 k ax m))
("pecking" nil (p eh1 k ih0 ng))
("peckinpaugh" nil (p ih0 k ih1 n p ao0))
("peckman" nil (p eh1 k m ax n))
("peco" nil (p ey1 k ow0))
("pecor" nil (p eh1 k er0))
("pecora" nil (p eh0 k ao1 r ax))
("pecoraro" nil (p eh0 k ao0 r aa1 r ow0))
("pecore" nil (p eh0 k ao1 r iy0))
("pecot" nil (p eh1 k ax t))
("pectic" nil (p eh1 k t ih0 k))
("pectin" nil (p eh1 k t ax n))
("pectoral" nil (p eh1 k t er0 ax l))
("peculiar" nil (p ax k y uw1 l y er0))
("peculiarities" nil (p ih0 k y uw1 l iy0 eh1 r ax t iy0 z))
("peculiarity" nil (p ih0 k y uw1 l iy0 eh1 r ax t iy0))
("peculiarly" nil (p ih0 k y uw1 l y er0 l iy0))
("pecuniary" nil (p eh0 k y uw1 n iy0 eh1 r iy0))
("pedagogical" nil (p eh1 d ax g aa1 jh ih0 k ax l))
("pedagogy" nil (p eh1 d ax g ow1 jh iy0))
("pedal" nil (p eh1 d ax l))
("pedaled" nil (p eh1 d ax l d))
("pedaling" nil (p eh1 d ax l ih0 ng))
("pedalled" nil (p eh1 d ax l d))
("pedals" nil (p eh1 d ax l z))
("pedantic" nil (p ax d ae1 n t ih0 k))
("pedantry" nil (p eh1 d ax n t r iy0))
("peddicord" nil (p eh1 d ih0 k ao0 r d))
("peddie" nil (p eh1 d iy0))
("peddle" nil (p eh1 d ax l))
("peddled" nil (p eh1 d ax l d))
("peddler" nil (p eh1 d l er0))
("peddlers" nil (p eh1 d l er0 z))
("peddles" nil (p eh1 d ax l z))
("peddling" nil (p eh1 d ax l ih0 ng))
("peddy" nil (p eh1 d iy0))
("peden" nil (p eh1 d ax n))
("pedersen" nil (p eh1 d er0 s ax n))
("pederson" nil (p eh1 d er0 s ax n))
("pedestal" nil (p eh1 d ax s t ax l))
("pedestals" nil (p eh1 d ax s t ax l z))
("pedestrian" nil (p ax d eh1 s t r iy0 ax n))
("pedestrians" nil (p ax d eh1 s t r iy0 ax n z))
("pediatric" nil (p iy1 d iy0 ae1 t r ih0 k))
("pediatrician" nil (p iy1 d iy0 ax t r ih1 sh ax n))
("pediatricians" nil (p iy1 d iy0 ax t r ih1 sh ax n z))
("pediatrics" nil (p iy1 d iy0 ae1 t r ih0 k s))
("pedicure" nil (p eh1 d ih0 k y er0))
("pedigo" nil (p eh0 d iy1 g ow0))
("pedigree" nil (p eh1 d ax g r iy0))
("pedley" nil (p eh1 d l iy0))
("pedone" nil (p ey0 d ow1 n ey0))
("pedowitz" nil (p eh1 d ow0 ih0 t s))
("pedraza" nil (p ey0 d r aa1 z ax))
("pedretti" nil (p eh0 d r eh1 t iy0))
("pedrick" nil (p eh1 d r ih0 k))
("pedro" nil (p ey1 d r ow0))
("pedroli" nil (p eh1 d r ow1 l iy0))
("pedros" nil (p ey1 d r ow0 z))
("pedrosa" nil (p ey0 d r ow1 s ax))
("pedroso" nil (p ey0 d r ow1 s ow0))
("pedrotti" nil (p eh0 d r ow1 t iy0))
("pedroza" nil (p ey0 d r ow1 z ax))
("pee" nil (p iy1))
("peebler" nil (p iy1 b l er0))
("peebles" nil (p iy1 b ax l z))
("peed" nil (p iy1 d))
("peedin" nil (p iy1 d ih0 n))
("peek" nil (p iy1 k))
("peeked" nil (p iy1 k t))
("peeking" nil (p iy1 k ih0 ng))
("peeks" nil (p iy1 k s))
("peel" nil (p iy1 l))
("peele" nil (p iy1 l))
("peeled" nil (p iy1 l d))
("peeler" nil (p iy1 l er0))
("peeling" nil (p iy1 l ih0 ng))
("peels" nil (p iy1 l z))
("peens" nil (p iy1 n z))
("peep" nil (p iy1 p))
("peeping" nil (p iy1 p ih0 ng))
("peeples" nil (p iy1 p ax l z))
("peer" nil (p ih1 r))
("peerage" nil (p ih1 r ax jh))
("peered" nil (p ih1 r d))
("peering" nil (p iy1 r ih0 ng))
("peerless" nil (p ih1 r l ih0 s))
("peers" nil (p ih1 r z))
("peerson" nil (p ih1 r s ax n))
("peery" nil (p iy1 r iy0))
("peet" nil (p iy1 t))
("peete" nil (p iy1 t))
("peeters" nil (p iy1 t er0 z))
("peets" nil (p iy1 t s))
("peetz" nil (p iy1 t s))
("peeve" nil (p iy1 v))
("peeved" nil (p iy1 v d))
("peeves" nil (p iy1 v z))
("peevey" nil (p iy1 v iy0))
("peevy" nil (p iy1 v iy0))
("peewee" nil (p iy1 w iy1))
("peffer" nil (p eh1 f er0))
("peffley" nil (p eh1 f l iy0))
("peformance" nil (p ax f ao1 r m ax n s))
("peg" nil (p eh1 g))
("pegasus" nil (p eh1 g ax s ax s))
("pegboards" nil (p eh1 g b ao1 r d z))
("pegg" nil (p eh1 g))
("pegged" nil (p eh1 g d))
("peggie" nil (p eh1 g iy0))
("pegging" nil (p eh1 g ih0 ng))
("peggs" nil (p eh1 g z))
("peggy" nil (p eh1 g iy0))
("peglow" nil (p eh1 g l ow1))
("pegmatite" nil (p eh1 g m ax t ay1 t))
("pegram" nil (p eh1 g r ae1 m))
("pegs" nil (p eh1 g z))
("peguero" nil (p ey0 g eh1 r ow0))
("pegues" nil (p ey1 g eh0 s))
("pehl" nil (p eh1 l))
("pehrson" nil (p eh1 r s ax n))
("pei" nil (p ey1))
("peifer" nil (p ay1 f er0))
("peiffer" nil (p ay1 f er0))
("peil" nil (p iy1 l))
("peinado" nil (p ey0 iy0 n aa1 d ow0))
("peine" nil (p iy1 n))
("peipu" nil (p ey1 p uw1))
("peirce" nil (p ih1 r s))
("peirson" nil (p iy1 r s ax n))
("peiser" nil (p ay1 s er0))
("peitz" nil (p iy1 t s))
("peixoto" nil (p ax k s ow1 t ow0))
("pejorative" nil (p ax jh ao1 r ax t ih0 v))
("pekala" nil (p ih0 k aa1 l ax))
("pekar" nil (p eh1 k er0))
("pekarek" nil (p eh1 k er0 ih0 k))
("pekin" nil (p iy1 k ax n))
("peking" nil (p iy1 k ih1 ng))
("peko" nil (p iy1 k ow0))
("pekrul" nil (p eh1 k r ax l))
("peladeau" nil (p eh1 l ax d ow1))
("pelaez" nil (p ey0 l aa1 eh0 z))
("pelagia" nil (p eh0 l aa1 jh ax))
("pelagians" nil (p ih0 l ey1 jh iy0 ax n z))
("pelagic" nil (p ax l ae1 jh ih0 k))
("pelayo" nil (p ey0 l ey1 ow0))
("pelc" nil (p eh1 l k))
("pelchat" nil (p eh1 l ch ax t))
("pelczar" nil (p eh1 l ch er0))
("pele" nil (p eh1 l ey0))
("pelekunu" nil (p eh1 l ax k uw1 n uw0))
("pelerin" nil (p eh1 l er0 ih0 n))
("pelfrey" nil (p eh1 l f r iy0))
("pelham" nil (p eh1 l ax m))
("pelican" nil (p eh1 l ax k ax n))
("pelicans" nil (p eh1 l ih0 k ax n z))
("pelikan" nil (p eh1 l ih0 k ae0 n))
("pelino" nil (p eh0 l iy1 n ow0))
("pelissier" nil (p eh1 l ih0 s iy0 er0))
("pelka" nil (p eh1 l k ax))
("pelkey" nil (p eh1 l k iy0))
("pelky" nil (p eh1 l k iy0))
("pell" nil (p eh1 l))
("pella" nil (p eh1 l ax))
("pellagra" nil (p ax l ae1 g r ax))
("pelland" nil (p eh1 l ax n d))
("pelle" nil (p eh1 l))
("pellecchia" nil (p eh1 l eh1 k iy0 ax))
("pellegrin" nil (p eh1 l ih0 g r ih0 n))
("pellegrini" nil (p eh1 l eh0 g r iy1 n iy0))
("pellegrino" nil (p eh1 l eh0 g r iy1 n ow0))
("peller" nil (p eh1 l er0))
("pellerin" nil (p eh1 l er0 ih0 n))
("pellerito" nil (p eh0 l er0 iy1 t ow0))
("pellet" nil (p eh1 l ax t))
("pelleted" nil (p eh1 l ax t ax d))
("pelletier" nil (p eh0 l ax t ih1 r))
("pellets" nil (p eh1 l ax t s))
("pellett" nil (p eh1 l ih0 t))
("pelleu" nil (p eh1 l uw0))
("pelley" nil (p eh1 l iy0))
("pellicane" nil (p eh1 l ih0 k ey1 n))
("pellicano" nil (p eh0 l iy0 k aa1 n ow0))
("pellman" nil (p eh1 l m ax n))
("pellow" nil (p eh1 l ow0))
("pellum" nil (p eh1 l ax m))
("pelon" nil (p eh1 l ax n))
("peloponnesian" nil (p eh1 l ax p ax n iy1 zh ax n))
("peloquin" nil (p ey0 l ow0 k w iy1 n))
("pelosi" nil (p eh0 l ow1 s iy0))
("peloso" nil (p eh0 l ow1 s ow0))
("pelot" nil (p eh1 l ax t))
("pelphrey" nil (p eh1 l f r iy0))
("pels" nil (p eh1 l z))
("pelster" nil (p eh1 l s t er0))
("pelt" nil (p eh1 l t))
("pelted" nil (p eh1 l t ih0 d))
("peltier" nil (p eh1 l t iy0 er0))
("pelto" nil (p eh1 l t ow0))
("pelton" nil (p eh1 l t ax n))
("pelts" nil (p eh1 l t s))
("peltz" nil (p eh1 l t s))
("peltzer" nil (p eh1 l t z er0))
("peluso" nil (p eh0 l uw1 s ow0))
("pelvic" nil (p eh1 l v ih0 k))
("pelvis" nil (p eh1 l v ax s))
("pelz" nil (p eh1 l z))
("pelzel" nil (p eh1 l z ax l))
("pelzer" nil (p eh1 l z er0))
("pember" nil (p eh1 m b er0))
("pemberton" nil (p eh1 m b er0 t ax n))
("pembina" nil (p eh1 m b ih0 n ax))
("pemble" nil (p eh1 m b ax l))
("pembleton" nil (p eh1 m b l ih0 t aa0 n))
("pembridge" nil (p eh1 m b r ih1 jh))
("pembroke" nil (p eh1 m b r ow1 k))
("pemex" nil (p eh1 m eh1 k s))
("pen" nil (p eh1 n))
("pena" nil (p eh1 n ax))
("penal" nil (p iy1 n ax l))
("penalize" nil (p eh1 n ax l ay1 z))
("penalized" nil (p iy1 n ax l ay1 z d))
("penalizes" nil (p eh1 n ax l ay1 z ih0 z))
("penalizing" nil (p iy1 n ax l ay1 z ih0 ng))
("penaloza" nil (p eh0 n aa0 l ow1 z ax))
("penalties" nil (p eh1 n ax l t iy0 z))
("penalty" nil (p eh1 n ax l t iy0))
("penance" nil (p eh1 n ax n s))
("penang" nil (p eh1 n ae0 ng))
("penans" nil (p eh1 n ax n z))
("penberthy" nil (p ih0 n b er1 th iy0))
("pence" nil (p eh1 n s))
("penchant" nil (p eh1 n ch ax n t))
("pencil" nil (p eh1 n s ax l))
("penciled" nil (p eh1 n s ax l d))
("pencils" nil (p eh1 n s ax l z))
("pendant" nil (p eh1 n d ax n t))
("pendarvis" nil (p eh1 n d aa0 r v ih0 s))
("pendell" nil (p eh1 n d ax l))
("pendelton" nil (p ih0 n d eh1 l t ax n))
("pender" nil (p eh1 n d er0))
("pendergast" nil (p eh1 n d er0 g ae1 s t))
("pendergraft" nil (p eh1 n d er0 g r ax f t))
("pendergraph" nil (p eh1 n d er0 g r ae1 f))
("pendergrass" nil (p eh1 n d er0 g r ax s))
("pendergrast" nil (p eh1 n d er0 g r ax s t))
("penders" nil (p eh1 n d er0 z))
("pending" nil (p eh1 n d ih0 ng))
("pendleton" nil (p eh1 n d ax l t ax n))
("pendley" nil (p eh1 n d l iy0))
("pendola" nil (p eh0 n d ow1 l ax))
("pendril" nil (p eh1 n d r ih0 l))
("pendry" nil (p eh1 n d r iy0))
("pendulous" nil (p eh1 n jh ax l ax s))
("pendulum" nil (p eh1 n jh ax l ax m))
("pendyala" nil (p eh1 d y aa1 l ax))
("penelope" nil (p ax n eh1 l ax p iy0))
("penetrate" nil (p eh1 n ax t r ey1 t))
("penetrated" nil (p eh1 n ax t r ey1 t ax d))
("penetrates" nil (p eh1 n ax t r ey1 t s))
("penetrating" nil (p eh1 n ax t r ey1 t ih0 ng))
("penetration" nil (p eh1 n ax t r ey1 sh ax n))
("penfield" nil (p eh1 n f iy1 l d))
("penfil" nil (p eh1 n f ih0 l))
("penfold" nil (p eh1 n f ow1 l d))
("peng" nil (p eh1 ng))
("pengassan" nil (p eh1 ng g ae1 s ax n))
("pengelly" nil (p eh1 ng g ax l iy0))
("pengo" nil (p eh1 ng g ow0))
("penguin" nil (p eh1 ng g w ax n))
("penguins" nil (p eh1 ng g w ax n z))
("penh" nil (p eh1 n))
("penicillin" nil (p eh1 n ax s ih1 l ax n))
("penicillins" nil (p eh1 n ax s ih1 l ax n z))
("penick" nil (p eh1 n ih0 k))
("penikese" nil (p eh1 n ih0 k iy1 z))
("penile" nil (p iy1 n ay0 l))
("peninger" nil (p eh1 n ih0 ng er0))
("peninsula" nil (p ax n ih1 n s ax l ax))
("peninsular" nil (p ax n ih1 n s ax l er0))
("penis" nil (p eh1 n ih0 s))
("peniston" nil (p eh1 n ih0 s t aa0 n))
("penitent" nil (p eh1 n ih0 t ih0 n t))
("penitentiary" nil (p eh1 n ih0 t eh1 n ch er0 iy0))
("penix" nil (p eh1 n ih0 k s))
("penkala" nil (p ih0 ng k aa1 l ax))
("penkava" nil (p eh0 ng k aa1 v ax))
("penland" nil (p eh1 n l ax n d))
("penley" nil (p eh1 n l iy0))
("penman" nil (p eh1 n m ax n))
("penn" nil (p eh1 n))
("penna" nil (p eh1 n ax))
("pennacchio" nil (p eh0 n aa1 k iy0 ow0))
("pennant" nil (p eh1 n ax n t))
("pennants" nil (p eh1 n ax n t s))
("pennbancorp" nil (p eh1 n b ae1 n k ao1 r p))
("penncorp" nil (p eh1 n k ao1 r p))
("pennebaker" nil (p eh1 n ih0 b ax k er0))
("penned" nil (p eh1 n d))
("pennel" nil (p eh1 n ax l))
("pennell" nil (p eh1 n ax l))
("pennella" nil (p eh1 n eh1 l ax))
("penner" nil (p eh1 n er0))
("pennex" nil (p eh1 n ax k s))
("penney" nil (p eh1 n iy0))
("pennick" nil (p eh1 n ih0 k))
("pennie" nil (p eh1 n iy0))
("pennies" nil (p eh1 n iy0 z))
("penniless" nil (p eh1 n iy0 l ax s))
("penniman" nil (p eh1 n ih0 m ax n))
("penning" nil (p eh1 n ih0 ng))
("penninger" nil (p eh1 n ih0 ng er0))
("pennings" nil (p eh1 n ih0 ng z))
("pennington" nil (p eh1 n ih0 ng t ax n))
("pennino" nil (p eh0 n iy1 n ow0))
("pennisi" nil (p eh0 n iy1 s iy0))
("pennix" nil (p eh1 n ih0 k s))
("pennock" nil (p eh1 n ax k))
("penns" nil (p eh1 n z))
("pennsauken" nil (p eh1 n s ao1 k ax n))
("pennsylvania" nil (p eh1 n s ax l v ey1 n y ax))
("pennsylvanian" nil (p eh1 n s ih0 l v ey1 n iy0 ax n))
("pennsylvanians" nil (p eh1 n s ih0 l v ey1 n iy0 ax n z))
("pennwalt" nil (p eh1 n w ao1 l t))
("penny" nil (p eh1 n iy0))
("pennypacker" nil (p eh1 n ih0 p ae0 k er0))
("pennywell" nil (p eh1 n iy0 w eh1 l))
("pennzoil" nil (p eh1 n z oy1 l))
("penobscot" nil (p ax n aa1 b s k aa0 t))
("penology" nil (p iy0 n aa1 l ax jh iy0))
("penoyer" nil (p eh1 n oy0 er0))
("penril" nil (p eh1 n r ih0 l))
("penrod" nil (p eh1 n r aa1 d))
("penrose" nil (p eh1 n r ow1 z))
("penry" nil (p eh1 n r iy0))
("pens" nil (p eh1 n z))
("pensacola" nil (p eh1 n s ax k ow1 l ax))
("pense" nil (p eh1 n s))
("pensinger" nil (p eh1 n s ih0 n jh er0))
("pension" nil (p eh1 n sh ax n))
("pensioner" nil (p eh1 n sh ax n er0))
("pensioners" nil (p eh1 n sh ax n er0 z))
("pensions" nil (p eh1 n sh ax n z))
("pensive" nil (p eh1 n s ih0 v))
("penske" nil (p eh1 n s k iy0))
("penson" nil (p eh1 n s ax n))
("pensyl" nil (p eh1 n s ih0 l))
("pent" nil (p eh1 n t))
("penta" nil (p eh1 n t ax))
("pentagon" nil (p eh1 n t ih0 g aa1 n))
("pentagons" nil (p eh1 n t ih0 g aa1 n z))
("pentair" nil (p eh1 n t eh1 r))
("pentamidine" nil (p eh1 n t ae1 m ih0 d iy1 n))
("pentane" nil (p eh1 n t ey1 n))
("pentathlon" nil (p eh0 n t ae1 th l ax n))
("pentax" nil (p eh1 n t ae1 k s))
("pentech" nil (p eh1 n t eh1 k))
("pentecost" nil (p eh1 n t ax k ao1 s t))
("pentecostal" nil (p eh1 n t ax k ao1 s t ax l))
("penthea" nil (p eh1 n th iy0 ax))
("penthouse" nil (p eh1 n t hh aw1 s))
("pentimento" nil (p eh1 n t ih0 m eh1 n t ow0))
("pentium" nil (p eh1 n t iy0 ax m))
("pentiums" nil (p eh1 n t iy0 ax m z))
("pentland" nil (p eh1 n t l ax n d))
("penton" nil (p eh1 n t ax n))
("pentron" nil (p eh1 n t r ax n))
("pentrust" nil (p eh1 n t r ah1 s t))
("pentz" nil (p eh1 n t s))
("penuel" nil (p eh1 n uh0 l))
("penumbra" nil (p ih0 n ah1 m b r ax))
("penwell" nil (p eh1 n w eh1 l))
("penwest" nil (p eh1 n w eh1 s t))
("penza" nil (p eh1 n z ax))
("penzer" nil (p eh1 n z er0))
("peon" nil (p iy1 ax n))
("peonies" nil (p iy1 ax n iy0 z))
("peons" nil (p iy1 ax n z))
("peony" nil (p iy1 ax n iy0))
("people" nil (p iy1 p ax l))
("peopled" nil (p iy1 p ax l d))
("peoples" nil (p iy1 p ax l z))
("peoria" nil (p iy0 ao1 r iy0 ax))
("pep" nil (p eh1 p))
("pepcid" nil (p eh1 p s ih0 d))
("pepe" nil (p eh1 p iy0))
("pepenadores" nil (p eh1 p eh1 n ax d ao1 r eh0 z))
("peper" nil (p eh1 p er0))
("pepi" nil (p eh1 p iy0))
("pepin" nil (p eh1 p ih0 n))
("pepita" nil (p eh0 p iy1 t ax))
("pepitone" nil (p eh1 p ih0 t ow1 n))
("peplinski" nil (p ih0 p l ih1 n s k iy0))
("peplow" nil (p eh1 p l ow1))
("peppard" nil (p eh1 p er0 d))
("peppel" nil (p eh1 p ax l))
("pepper" nil (p eh1 p er0))
("peppered" nil (p eh1 p er0 d))
("pepperell" nil (p eh1 p er0 ax l))
("pepperidge" nil (p eh1 p er0 ih1 jh))
("peppering" nil (p eh1 p er0 ih0 ng))
("pepperman" nil (p eh1 p er0 m ax n))
("peppermint" nil (p eh1 p er0 m ih1 n t))
("pepperoni" nil (p eh1 p er0 ow1 n iy0))
("peppers" nil (p eh1 p er0 z))
("peppi" nil (p eh1 p iy0))
("peppin" nil (p eh1 p ih0 n))
("pepple" nil (p eh1 p ax l))
("peppler" nil (p eh1 p l er0))
("peppy" nil (p eh1 p iy0))
("pepsi" nil (p eh1 p s iy0))
("pepsico" nil (p eh1 p s ih0 k ow0))
("peptic" nil (p eh1 p t ih0 k))
("peptide" nil (p eh1 p t ay1 d))
("peptides" nil (p eh1 p t ay1 d z))
("pequeno" nil (p ey0 k w ey1 n ow0))
("pequignot" nil (p ih0 k w ih1 g n ax t))
("per" nil (p er1))
("pera" nil (p er1 ax))
("peragine" nil (p er0 aa0 jh iy1 n iy0))
("peraino" nil (p er0 aa0 iy1 n ow0))
("perala" nil (p er0 aa1 l ax))
("perales" nil (p er0 aa1 l eh0 s))
("peralez" nil (p er0 aa1 l eh0 z))
("peralta" nil (p er0 aa1 l t ax))
("peras" nil (p eh1 r ax s))
("peraza" nil (p er0 aa1 z ax))
("perazzo" nil (p er0 aa1 z ow0))
("perc" nil (p er1 k))
("perce" nil (p er1 s))
("perceive" nil (p er0 s iy1 v))
("perceived" nil (p er0 s iy1 v d))
("perceives" nil (p er0 s iy1 v z))
("perceiving" nil (p er0 s iy1 v ih0 ng))
("percell" nil (p er0 s eh1 l))
("percent" nil (p er0 s eh1 n t))
("percentage" nil (p er0 s eh1 n t ax jh))
("percentages" nil (p er0 s eh1 n t ih0 jh ih0 z))
("percentile" nil (p er0 s eh1 n t ay1 l))
("percentiles" nil (p er0 s eh1 n t ay1 l z))
("perceptible" nil (p er0 s eh1 p t ax b ax l))
("perception" nil (p er0 s eh1 p sh ax n))
("perceptions" nil (p er0 s eh1 p sh ax n z))
("perceptive" nil (p er0 s eh1 p t ih0 v))
("perceptual" nil (p er0 s eh1 p ch ax w ax l))
("perceval" nil (p er0 s ey0 v ae1 l))
("perch" nil (p er1 ch))
("perched" nil (p er1 ch t))
("perches" nil (p er1 ch ih0 z))
("perchlorate" nil (p er0 k l ao1 r ey0 t))
("perchloric" nil (p er0 k l ao1 r ih0 k))
("percifield" nil (p er1 s ih0 f iy0 l d))
("percival" nil (p er1 s ih0 v ax l))
("percle" nil (p er1 k ax l))
("percolate" nil (p er1 k ax l ey1 t))
("percolating" nil (p er1 k ax l ey1 t ih0 ng))
("percs" nil (p er1 k s))
("percussion" nil (p er0 k ah1 sh ax n))
("percussionist" nil (p er0 k ah1 sh ax n ih0 s t))
("percutaneous" nil (p er0 k y uw0 t ey1 n iy0 ax s))
("percy" nil (p er1 s iy0))
("perdew" nil (p er1 d uw0))
("perdita" nil (p er0 d iy1 t ax))
("perdomo" nil (p er0 d ow1 m ow0))
("perdue" nil (p er0 d uw1))
("pere" nil (p er1))
("perea" nil (p er0 iy1 ax))
("pereda" nil (p er0 ey1 d ax))
("peregoy" nil (p er1 ih0 g oy0))
("peregrine" nil (p eh1 r ax g r iy1 n))
("peregrines" nil (p eh1 r ax g r iy1 n z))
("pereira" nil (p er0 eh1 r ax))
("perella" nil (p er0 eh1 l ax))
("perelman" nil (p eh1 r ax l m ax n))
("peremptory" nil (p er0 eh1 m p t er0 iy0))
("perennial" nil (p er0 eh1 n iy0 ax l))
("perennially" nil (p er0 eh1 n iy0 ax l iy0))
("perennials" nil (p er0 eh1 n iy0 ax l z))
("perera" nil (p er0 eh1 r ax))
("peres" nil (p eh1 r eh0 z))
("perestroika" nil (p eh1 r ax s t r oy1 k ax))
("peretti" nil (p er0 eh1 t iy0))
("peretz" nil (p er1 ih0 t s))
("pereyra" nil (p er0 eh1 r ax))
("perez" nil (p eh1 r eh0 z))
("perfect" j (p er1 f ax k t))
("perfect" v (p er0 f eh1 k t))
("perfecta" nil (p er0 f eh1 k t ax))
("perfected" nil (p er0 f eh1 k t ax d))
("perfecting" nil (p er0 f eh1 k t ih0 ng))
("perfection" nil (p er0 f eh1 k sh ax n))
("perfectionism" nil (p er0 f eh1 k sh ax n ih1 z ax m))
("perfectionist" nil (p er0 f eh1 k sh ax n ax s t))
("perfectly" nil (p er1 f ax k t l iy0))
("perfetti" nil (p er0 f eh1 t iy0))
("perfetto" nil (p er0 f eh1 t ow0))
("perfidy" nil (p er1 f ih0 d iy0))
("perforated" nil (p er1 f er0 ey1 t ax d))
("perforation" nil (p er1 f er0 ey1 sh ax n))
("perforations" nil (p er1 f er0 ey1 sh ax n z))
("perforce" nil (p er0 f ao1 r s))
("perform" nil (p er0 f ao1 r m))
("performa" nil (p er0 f ao1 r m ax))
("performance" nil (p er0 f ao1 r m ax n s))
("performances" nil (p er0 f ao1 r m ax n s ax z))
("performed" nil (p er0 f ao1 r m d))
("performer" nil (p er0 f ao1 r m er0))
("performers" nil (p er0 f ao1 r m er0 z))
("performing" nil (p er0 f ao1 r m ih0 ng))
("performs" nil (p er0 f ao1 r m z))
("perfume" nil (p er0 f y uw1 m))
("perfumed" nil (p er0 f y uw1 m d))
("perfumes" nil (p er0 f y uw1 m z))
("perfunctory" nil (p er0 f ah1 ng k t er0 iy0))
("pergamon" nil (p er1 g ax m ax n))
("pergande" nil (p er1 g ih0 n d))
("pergola" nil (p er0 g ow1 l ax))
("perham" nil (p er1 hh ax m))
("perhaps" nil (p er0 hh ae1 p s))
("perich" nil (p er1 ih0 k))
("perignon" nil (p er0 ih1 n y aa0 n))
("perigo" nil (p er0 iy1 g ow0))
("peril" nil (p eh1 r ax l))
("perillo" nil (p er0 ih1 l ow0))
("perilous" nil (p eh1 r ax l ax s))
("perilously" nil (p eh1 r ax l ax s l iy0))
("perils" nil (p eh1 r ax l z))
("periman" nil (p er1 iy0 m ax n))
("perimeter" nil (p er0 ih1 m ax t er0))
("perin" nil (p er1 ih0 n))
("perine" nil (p er0 iy1 n iy0))
("perini" nil (p er0 iy1 n iy0))
("perino" nil (p er0 iy1 n ow0))
("period" nil (p ih1 r iy0 ax d))
("periodic" nil (p ih1 r iy0 aa1 d ih0 k))
("periodical" nil (p ih1 r iy0 aa1 d ih0 k ax l))
("periodically" nil (p iy1 r iy0 aa1 d ih0 k ax l iy0))
("periodicals" nil (p ih1 r iy0 aa1 d ih0 k ax l z))
("periodontal" nil (p eh1 r iy0 ax d aa1 n t ax l))
("periods" nil (p ih1 r iy0 ax d z))
("peripatetic" nil (p eh1 r ax p ax t eh1 t ih0 k))
("peripheral" nil (p er0 ih1 f er0 ax l))
("peripherals" nil (p er0 ih1 f er0 ax l z))
("periphery" nil (p er0 ih1 f er0 iy0))
("periscope" nil (p eh1 r ax s k ow1 p))
("perish" nil (p eh1 r ih0 sh))
("perishable" nil (p eh1 r ih0 sh ax b ax l))
("perished" nil (p eh1 r ih0 sh t))
("peristyle" nil (p eh1 r ax s t ay1 l))
("perito" nil (p eh1 r iy1 t ow0))
("perjured" nil (p er1 jh er0 d))
("perjury" nil (p er1 jh er0 iy0))
("perk" nil (p er1 k))
("perked" nil (p er1 k t))
("perkey" nil (p er1 k iy0))
("perkin" nil (p er1 k ih0 n))
("perking" nil (p er1 k ih0 ng))
("perkins" nil (p er1 k ax n z))
("perkinson" nil (p er1 k ih0 n s ax n))
("perko" nil (p er1 k ow0))
("perkovic" nil (p er1 k ax v ih0 ch))
("perkovich" nil (p er1 k ax v ih0 ch))
("perkowski" nil (p er0 k ao1 f s k iy0))
("perks" nil (p er1 k s))
("perky" nil (p er1 k iy0))
("perl" nil (p er1 l))
("perla" nil (p er1 l ax))
("perlberg" nil (p er1 l b er0 g))
("perle" nil (p er1 l))
("perley" nil (p er1 l iy0))
("perlin" nil (p er1 l ih0 n))
("perlis" nil (p er1 l ih1 s))
("perlite" nil (p er1 l ay0 t))
("perlman" nil (p er1 l m ax n))
("perlmutter" nil (p er1 l m ah1 t er0))
("perlow" nil (p er1 l ow0))
("perls" nil (p er1 l z))
("perlstein" nil (p er1 l s t ay0 n))
("perma" nil (p er1 m ax))
("perman" nil (p er1 m ax n))
("permanence" nil (p er1 m ax n ax n s))
("permanent" nil (p er1 m ax n ax n t))
("permanente" nil (p er1 m ax n eh1 n t ey0))
("permanently" nil (p er1 m ax n ax n t l iy0))
("permeability" nil (p er0 m iy0 ax b ih1 l ih0 t iy0))
("permeate" nil (p er1 m iy0 ey1 t))
("permeated" nil (p er1 m iy0 ey1 t ih0 d))
("permeates" nil (p er1 m iy0 ey1 t s))
("permenter" nil (p er1 m ax n t er0))
("permiam" nil (p er0 m ay1 ax m))
("permian" nil (p er1 m iy0 ax n))
("permissible" nil (p er0 m ih1 s ax b ax l))
("permission" nil (p er0 m ih1 sh ax n))
("permissions" nil (p er0 m ih1 sh ax n z))
("permissive" nil (p er0 m ih1 s ih0 v))
("permissiveness" nil (p er0 m ih1 s ih0 v n ax s))
("permit" v (p er0 m ih1 t))
("permit" n (p er1 m ih0 t))
("permits" v (p er0 m ih1 t s))
("permits" n (p er1 m ih0 t s))
("permitted" nil (p er0 m ih1 t ax d))
("permitting" nil (p er0 m ih1 t ih0 ng))
("permut" nil (p er1 m ax t))
("permutations" nil (p er1 m y uw0 t ey1 sh ax n z))
("perna" nil (p er1 n ax))
("pernell" nil (p er1 n ax l))
("pernella" nil (p er0 n eh1 l ax))
("pernice" nil (p er1 n ih0 s))
("perniciaro" nil (p er0 n iy1 ch er0 ow0))
("pernicious" nil (p er0 n ih1 sh ax s))
("perno" nil (p er1 n ow0))
("pernod" nil (p eh0 r n ow1))
("pero" nil (p er1 ow0))
("peron" nil (p er0 ao1 n))
("perona" nil (p er0 ow1 n ax))
("perone" nil (p er0 ow1 n))
("peroni" nil (p er0 ow1 n iy0))
("peronism" nil (p eh1 r ax n ih1 z ax m))
("peronist" nil (p eh1 r ax n ih0 s t))
("peronists" nil (p eh1 r ax n ih0 s t s))
("perot" nil (p er0 ow1))
("perotti" nil (p er0 ow1 t iy0))
("peroutka" nil (p er0 uw1 t k ax))
("perovich" nil (p er1 ax v ih0 ch))
("peroxide" nil (p er0 aa1 k s ay1 d))
("perpendicular" nil (p er1 p ax n d ih1 k y ax l er0))
("perpetrate" nil (p er1 p ax t r ey1 t))
("perpetrated" nil (p er1 p ax t r ey1 t ih0 d))
("perpetrating" nil (p er1 p ax t r ey1 t ih0 ng))
("perpetrator" nil (p er1 p ax t r ey1 t er0))
("perpetrators" nil (p er1 p ax t r ey1 t er0 z))
("perpetual" nil (p er0 p eh1 ch uw0 ax l))
("perpetually" nil (p er0 p eh1 ch uw0 ax l iy0))
("perpetuals" nil (p er0 p eh1 ch uw0 ax l z))
("perpetuate" nil (p er0 p eh1 ch ax w ey1 t))
("perpetuated" nil (p er0 p eh1 ch ax w ey1 t ih0 d))
("perpetuates" nil (p er0 p eh1 ch ax w ey1 t s))
("perpetuating" nil (p er0 p eh1 ch ax w ey1 t ih0 ng))
("perpetuation" nil (p er0 p eh1 ch ax w ey1 sh ax n))
("perpetuity" nil (p er1 p ih0 t y uw1 ih0 t iy0))
("perpich" nil (p er1 p ih0 k))
("perplexed" nil (p er0 p l eh1 k s t))
("perplexes" nil (p er0 p l eh1 k s ih0 z))
("perplexing" nil (p er0 p l eh1 k s ih0 ng))
("perplexity" nil (p er0 p l eh1 k s ax t iy0))
("perquisites" nil (p er1 k w ax z ih0 t s))
("perra" nil (p er1 r ax))
("perras" nil (p er1 r ax z))
("perrault" nil (p eh1 r ao0 l t))
("perreault" nil (p er0 r ow1))
("perreira" nil (p er0 r eh1 r ax))
("perrell" nil (p er0 r ey1 l))
("perrella" nil (p er0 r eh1 l ax))
("perrelli" nil (p er0 r eh1 l iy0))
("perren" nil (p er1 r ax n))
("perret" nil (p er1 r ih0 t))
("perrett" nil (p er1 r ih0 t))
("perretta" nil (p er0 r eh1 t ax))
("perretti" nil (p er0 r eh1 t iy0))
("perri" nil (p er1 r iy0))
("perricone" nil (p er0 r iy0 k ow1 n iy0))
("perriello" nil (p er0 r iy0 eh1 l ow0))
("perrier" nil (p eh1 r iy0 er0))
("perrigo" nil (p er0 r iy1 g ow0))
("perrin" nil (p eh1 r ih0 n))
("perrine" nil (p er0 r iy1 n iy0))
("perrino" nil (p er0 r iy1 n ow0))
("perrins" nil (p eh1 r ih0 n z))
("perritt" nil (p er1 r ih0 t))
("perro" nil (p er1 r ow0))
("perrodin" nil (p er1 r ax d ih0 n))
("perron" nil (p eh1 r ax n))
("perrone" nil (p er0 r ow1 n iy0))
("perrot" nil (p er1 r ax t))
("perrott" nil (p er0 r aa1 t))
("perrotta" nil (p er0 r ow1 t ax))
("perrotti" nil (p er0 r ow1 t iy0))
("perrotto" nil (p er0 r ow1 t ow0))
("perrow" nil (p er1 r ow0))
("perry" nil (p eh1 r iy0))
("perryman" nil (p er1 r iy0 m ax n))
("perryn" nil (p er1 r ih0 n))
("persall" nil (p er1 s ax l))
("persaud" nil (p er1 s ao0 d))
("perschbacher" nil (p er1 sh b aa0 k er0))
("perse" nil (p er1 s))
("persecute" nil (p er1 s ax k y uw1 t))
("persecuted" nil (p er1 s ax k y uw1 t ax d))
("persecuting" nil (p er1 s ih0 k y uw1 t ih0 ng))
("persecution" nil (p er1 s ax k y uw1 sh ax n))
("persepolis" nil (p er0 s eh1 p ax l ax s))
("perseptive" nil (p er0 s eh1 p t ih0 v))
("perses" nil (p er1 s ih0 z))
("perseus" nil (p er1 s iy0 ax s))
("perseverance" nil (p er1 s ax v ih1 r ax n s))
("persevere" nil (p er0 s ax v ih1 r))
("persevered" nil (p er1 s ax v ih1 r d))
("pershall" nil (p er1 sh ax l))
("pershare" nil (p er0 sh eh1 r))
("pershing" nil (p er1 sh ih0 ng))
("pershings" nil (p er1 sh ih0 ng z))
("persia" nil (p er1 zh ax))
("persian" nil (p er1 zh ax n))
("persians" nil (p er1 zh ax n z))
("persico" nil (p er0 s iy1 k ow0))
("persing" nil (p er1 s ih0 ng))
("persinger" nil (p er1 s ih0 n jh er0))
("persis" nil (p er1 s ih0 s))
("persist" nil (p er0 s ih1 s t))
("persisted" nil (p er0 s ih1 s t ax d))
("persistence" nil (p er0 s ih1 s t ax n s))
("persistent" nil (p er0 s ih1 s t ax n t))
("persistently" nil (p er0 s ih1 s t ax n t l iy0))
("persisting" nil (p er0 s ih1 s t ih0 ng))
("persists" nil (p er0 s ih1 s t s))
("persky" nil (p er1 s k iy0))
("persley" nil (p er1 s l iy0))
("person" nil (p er1 s ax n))
("persona" nil (p er0 s ow1 n ax))
("personable" nil (p er1 s ax n ax b ax l))
("personably" nil (p er1 s ax n ax b l iy0))
("personal" nil (p er1 s ax n ax l))
("personalink" nil (p er1 s ax n ax l ih1 ng k))
("personalities" nil (p er1 s ax n ae1 l ih0 t iy0 z))
("personality" nil (p er1 s ax n ae1 l ih0 t iy0))
("personalize" nil (p er1 s ax n ax l ay1 z))
("personalized" nil (p er1 s ax n ax l ay1 z d))
("personally" nil (p er1 s ax n ax l iy0))
("personics" nil (p er0 s aa1 n ih0 k s))
("personification" nil (p er0 s aa1 n ax f ax k ey1 sh ax n))
("personified" nil (p er0 s aa1 n ax f ay1 d))
("personifies" nil (p er0 s aa1 n ax f ay1 z))
("personify" nil (p er0 s aa1 n ax f ay1))
("personnel" nil (p er1 s ax n eh1 l))
("persons" nil (p er1 s ax n z))
("perspective" nil (p er0 s p eh1 k t ih0 v))
("perspectives" nil (p er0 s p eh1 k t ih0 v z))
("perspiration" nil (p er1 s p er0 ey1 sh ax n))
("perspire" nil (p er0 s p ay1 r))
("persson" nil (p er1 s ax n))
("persuadable" nil (p er0 s w ey1 d ax b ax l))
("persuade" nil (p er0 s w ey1 d))
("persuaded" nil (p er0 s w ey1 d ax d))
("persuades" nil (p er0 s w ey1 d z))
("persuading" nil (p er0 s w ey1 d ih0 ng))
("persuasion" nil (p er0 s w ey1 zh ax n))
("persuasions" nil (p er0 s w ey1 zh ax n z))
("persuasive" nil (p er0 s w ey1 s ih0 v))
("persuasively" nil (p er0 s w ey1 s ih0 v l iy0))
("persuasiveness" nil (p er0 s w ey1 s ih0 v n ax s))
("persyn" nil (p er1 s ih0 n))
("pert" nil (p er1 t))
("pertain" nil (p er0 t ey1 n))
("pertained" nil (p er0 t ey1 n d))
("pertaining" nil (p er0 t ey1 n ih0 ng))
("pertains" nil (p er0 t ey1 n z))
("pertamina" nil (p er0 t ax m iy1 n ax))
("perth" nil (p er1 th))
("pertinent" nil (p er1 t ax n ax n t))
("pertschuk" nil (p er1 t ch ax k))
("pertuit" nil (p er1 t ih0 t))
("perturbation" nil (p er1 t er0 b ey1 sh ax n))
("perturbations" nil (p er1 t er0 b ey1 sh ax n z))
("perturbed" nil (p er0 t er1 b d))
("perturbs" nil (p er0 t er1 b z))
("pertussis" nil (p er0 t ah1 s ih0 s))
("peru" nil (p er0 uw1))
("perugina" nil (p eh1 r uw0 jh iy1 n ax))
("perugini" nil (p er0 uw0 jh iy1 n iy0))
("perulac" nil (p eh1 r uw0 l ae0 k))
("perusal" nil (p er0 uw1 z ax l))
("peruse" nil (p er0 uw1 z))
("perused" nil (p er0 uw1 z d))
("perusing" nil (p er0 uw1 z ih0 ng))
("perusse" nil (p er0 uw1 s iy0))
("peruvian" nil (p er0 uw1 v iy0 ax n))
("peruvians" nil (p er0 uw1 v iy0 ax n z))
("peruzzi" nil (p er0 uw1 t s iy0))
("pervade" nil (p er0 v ey1 d))
("pervaded" nil (p er0 v ey1 d ih0 d))
("pervades" nil (p er0 v ey1 d z))
("pervading" nil (p er0 v ey1 d ih0 ng))
("pervasive" nil (p er0 v ey1 s ih0 v))
("pervasiveness" nil (p er0 v ey1 s ih0 v n ih0 s))
("perverse" nil (p er0 v er1 s))
("perversely" nil (p er0 v er1 s l iy0))
("perversion" nil (p er0 v er1 zh ax n))
("perversity" nil (p er0 v er1 s ax t iy0))
("pervert" v (p er0 v er1 t))
("pervert" n (p er1 v er0 t))
("perverted" nil (p er0 v er1 t ih0 d))
("perverting" nil (p er0 v er1 t ih0 ng))
("perverts" v (p er0 v er1 t s))
("perverts" n (p er1 v er0 t s))
("perz" nil (p er1 z))
("pesatori" nil (p eh1 s ax t ao1 r iy0))
("pesavento" nil (p eh1 s ax v eh1 n t ow0))
("pescatore" nil (p eh0 s k aa0 t ao1 r iy0))
("pesce" nil (p eh1 s))
("pesch" nil (p eh1 sh))
("peschel" nil (p eh1 sh ax l))
("peschke" nil (p eh1 sh k))
("pesek" nil (p eh1 s eh0 k))
("peseta" nil (p ax s ey1 t ax))
("pesetas" nil (p ax s ey1 t ax z))
("pesh" nil (p eh1 sh))
("peshawar" nil (p eh1 sh ax w ao0 r))
("peshek" nil (p eh1 sh eh0 k))
("pesina" nil (p eh0 s iy1 n ax))
("peskin" nil (p eh1 s k ih0 n))
("pesky" nil (p eh1 s k iy0))
("pesnell" nil (p ih0 s n eh1 l))
("peso" nil (p ey1 s ow0))
("pesola" nil (p eh0 s ow1 l ax))
("pesos" nil (p ey1 s ow0 z))
("pessimism" nil (p eh1 s ax m ih1 z ax m))
("pessimist" nil (p eh1 s ax m ax s t))
("pessimistic" nil (p eh1 s ax m ih1 s t ih0 k))
("pessimists" nil (p eh1 s ax m ih0 s t s))
("pessin" nil (p eh1 s ih0 n))
("pest" nil (p eh1 s t))
("pesta" nil (p eh1 s t ax))
("pestana" nil (p eh0 s t ae1 n ax))
("pester" nil (p eh1 s t er0))
("pestered" nil (p eh1 s t er0 d))
("pestering" nil (p eh1 s t er0 ih0 ng))
("pesticide" nil (p eh1 s t ax s ay1 d))
("pesticides" nil (p eh1 s t ax s ay1 d z))
("pestilence" nil (p eh1 s t ax l ax n s))
("pestillo" nil (p eh1 s t ih1 l ow0))
("pestka" nil (p eh1 s t k ax))
("pests" nil (p eh1 s t s))
("pet" nil (p eh1 t))
("peta" nil (p iy1 t ax))
("petak" nil (p eh1 t ax k))
("petals" nil (p eh1 t ax l z))
("petaluma" nil (p eh1 t ax l uw1 m ax))
("petard" nil (p eh1 t er0 d))
("pete" nil (p iy1 t))
("peter" nil (p iy1 t er0))
("peterbilt" nil (p iy1 t er0 b ih1 l t))
("peterborough" nil (p iy1 t er0 b er0 ow0))
("petered" nil (p iy1 t er0 d))
("petering" nil (p iy1 t er0 ih0 ng))
("peterka" nil (p ih0 t er1 k ax))
("peterkin" nil (p ih0 t er1 k ih0 n))
("peterman" nil (p iy1 t er0 m ax n))
("petermann" nil (p iy1 t er0 m ax n))
("peterpaul" nil (p iy1 t er0 p ao1 l))
("peters" nil (p iy1 t er0 z))
("petersburg" nil (p iy1 t er0 z b er0 g))
("petersen" nil (p iy1 t er0 s ax n))
("peterson" nil (p iy1 t er0 s ax n))
("petery" nil (p iy1 t er0 iy0))
("petey" nil (p eh1 t iy0))
("peth" nil (p eh1 th))
("pethel" nil (p eh1 th ax l))
("pethtel" nil (p eh1 th ax l))
("petie" nil (p eh1 t iy0))
("petillo" nil (p eh0 t ih1 l ow0))
("petipa" nil (p eh1 t iy1 p ax))
("petit" nil (p eh1 t iy0))
("petite" nil (p ax t iy1 t))
("petites" nil (p eh1 t iy1 t s))
("petition" nil (p ax t ih1 sh ax n))
("petitioned" nil (p ax t ih1 sh ax n d))
("petitioner" nil (p ax t ih1 sh ax n er0))
("petitioners" nil (p ax t ih1 sh ax n er0 z))
("petitioning" nil (p ax t ih1 sh ax n ih0 ng))
("petitions" nil (p ax t ih1 sh ax n z))
("petitjean" nil (p ih0 t ih1 t jh ax n))
("petito" nil (p eh0 t iy1 t ow0))
("petitt" nil (p eh1 t ih0 t))
("petitti" nil (p eh0 t iy1 t iy0))
("petitto" nil (p eh0 t iy1 t ow0))
("petko" nil (p eh1 t k ow0))
("petkus" nil (p eh1 t k ih0 s))
("peto" nil (p iy1 t ow0))
("petra" nil (p eh1 t r ax))
("petracca" nil (p eh0 t r aa1 k ax))
("petraglia" nil (p eh0 t r aa1 g l iy0 ax))
("petraitis" nil (p eh0 t r ay1 t ih0 s))
("petrak" nil (p eh1 t r ax k))
("petrakis" nil (p eh1 t r ax k ih0 s))
("petralia" nil (p eh0 t r aa1 l iy0 ax))
("petrarca" nil (p eh1 t r aa1 r k ax))
("petras" nil (p eh1 t r ax z))
("petrasek" nil (p ih0 t r aa1 s eh0 k))
("petrash" nil (p eh1 t r ax sh))
("petrauskas" nil (p eh1 t r aw0 s k ax z))
("petre" nil (p eh1 t er0))
("petrea" nil (p eh1 t r iy0 ax))
("petree" nil (p eh1 t r iy1))
("petrella" nil (p eh1 t r eh1 l ax))
("petrelli" nil (p eh0 t r eh1 l iy0))
("petrels" nil (p eh1 t r ax l z))
("petrey" nil (p eh1 t r iy0))
("petri" nil (p eh1 t r iy0))
("petrich" nil (p eh1 t r ih0 k))
("petricioli" nil (p eh1 t r ih0 s iy0 ow1 l iy0))
("petrick" nil (p eh1 t r ih0 k))
("petrides" nil (p eh1 t r ay0 d z))
("petrie" nil (p eh1 t r iy0))
("petrified" nil (p eh1 t r ax f ay1 d))
("petrik" nil (p eh1 t r ih0 k))
("petrilla" nil (p eh1 t r ih1 l ax))
("petrilli" nil (p eh0 t r iy1 l iy0))
("petrillo" nil (p eh1 t r ih1 l ow0))
("petrin" nil (p eh1 t r ih0 n))
("petrina" nil (p eh0 t r iy1 n ax))
("petrini" nil (p eh0 t r iy1 n iy0))
("petrino" nil (p eh0 t r iy1 n ow0))
("petrizzo" nil (p eh0 t r iy1 z ow0))
("petro" nil (p eh1 t r ow0))
("petrobras" nil (p eh1 t r aa1 b r ax s))
("petrocelli" nil (p eh0 t r ow0 ch eh1 l iy0))
("petrochemical" nil (p eh1 t r ow0 k eh1 m ih0 k ax l))
("petrochemicals" nil (p eh1 t r ow0 k eh1 m ih0 k ax l z))
("petrocorp" nil (p eh1 t r ow0 k ao1 r p))
("petrodollars" nil (p eh1 t r ow0 d aa1 l er0 z))
("petroff" nil (p eh1 t r ao0 f))
("petrofina" nil (p eh1 t r ax f iy1 n ax))
("petrolane" nil (p eh1 t r ow0 l ey1 n))
("petrolear" nil (p eh1 t r ow0 l ih1 r))
("petroleos" nil (p ax t r ow1 l iy0 ow0 s))
("petroles" nil (p eh1 t r ow1 l z))
("petroleum" nil (p ax t r ow1 l iy0 ax m))
("petrology" nil (p ax t r aa1 l ax jh iy0))
("petromin" nil (p eh1 t r ax m ih0 n))
("petrominerals" nil (p eh1 t r ow0 m ih1 n er0 ax l z))
("petronas" nil (p eh1 t r ow1 n ax s))
("petrone" nil (p eh0 t r ow1 n iy0))
("petronella" nil (p eh1 t r ow0 n eh1 l ax))
("petroni" nil (p eh0 t r ow1 n iy0))
("petronia" nil (p eh0 t r ow1 n iy0 ax))
("petronilla" nil (p eh1 t r ax n ih1 l ax))
("petronio" nil (p eh1 t r ow1 n iy0 ow0))
("petropoulos" nil (p ih0 t r aa1 p ax l ih0 s))
("petros" nil (p eh1 t r ow0 z))
("petrosino" nil (p eh0 t r ow0 s iy1 n ow0))
("petroski" nil (p ih0 t r aw1 s k iy0))
("petrosky" nil (p ih0 t r ow1 s k iy0))
("petrossian" nil (p ax t r ao1 z ax n))
("petrosynthese" nil (p ax t r ow1 s ih0 n th iy1 s))
("petrovic" nil (p ih0 t r aa1 v ih0 k))
("petrovich" nil (p eh1 t r ax v ih0 ch))
("petrovietnam" nil (p eh1 t r ow0 v iy1 eh0 t n aa1 m))
("petrow" nil (p eh1 t r ow1))
("petrowski" nil (p ih0 t r ao1 f s k iy0))
("petrowsky" nil (p ih0 t r aw1 s k iy0))
("petru" nil (p eh1 t r uw0))
("petruccelli" nil (p eh1 t r uw0 ch eh1 l iy0))
("petrucci" nil (p eh0 t r uw1 ch iy0))
("petrucelli" nil (p eh0 t r uw0 ch eh1 l iy0))
("petrus" nil (p eh1 t r ax s))
("petruska" nil (p ih0 t r ah1 s k ax))
("petruzzelli" nil (p eh0 t r uw0 t s eh1 l iy0))
("petruzzi" nil (p eh0 t r uw1 t s iy0))
("petry" nil (p eh1 t r iy0))
("pets" nil (p eh1 t s))
("petsch" nil (p eh1 ch))
("petsche" nil (p eh1 ch))
("petsmart" nil (p eh1 t s m aa1 r t))
("petstuff" nil (p eh1 t s t ah1 f))
("pett" nil (p eh1 t))
("petta" nil (p eh1 t ax))
("pettaway" nil (p eh1 t ax w ey0))
("petted" nil (p eh1 t ax d))
("pettee" nil (p eh1 t iy1))
("pettengill" nil (p eh1 t ih0 ng g ax l))
("petter" nil (p eh1 t er0))
("pettersen" nil (p eh1 t er0 s ax n))
("petterson" nil (p eh1 t er0 s ax n))
("pettersson" nil (p iy1 t er0 s ax n))
("pettet" nil (p eh1 t ih0 t))
("petteway" nil (p eh1 t w ey0))
("pettey" nil (p eh1 t iy0))
("petteys" nil (p eh1 t iy0 z))
("petti" nil (p eh1 t iy0))
("petticoats" nil (p eh1 t iy0 k ow1 t s))
("pettie" nil (p eh1 t iy0))
("petties" nil (p eh1 t iy0 z))
("pettigrew" nil (p eh1 t ih0 g r uw0))
("pettijohn" nil (p eh1 t ih0 jh aa0 n))
("pettinato" nil (p eh0 t iy0 n aa1 t ow0))
("pettine" nil (p eh0 t iy1 n iy0))
("pettiness" nil (p eh1 t iy0 n ax s))
("petting" nil (p eh1 t ih0 ng))
("pettinger" nil (p eh1 t ih0 ng er0))
("pettingill" nil (p ih0 t ih1 ng g ax l))
("pettis" nil (p eh1 t ih0 s))
("pettit" nil (p eh1 t ih0 t))
("pettitt" nil (p eh1 t ih0 t))
("pettry" nil (p eh1 t r iy0))
("pettus" nil (p eh1 t ax s))
("pettway" nil (p eh1 t w ey1))
("petty" nil (p eh1 t iy0))
("petulance" nil (p eh1 ch ax l ax n s))
("petulant" nil (p eh1 ch ax l ax n t))
("petunia" nil (p ax t uw1 n iy0 ax))
("petunias" nil (p ax t uw1 n iy0 ax s))
("petway" nil (p eh1 t w ey1))
("petz" nil (p eh1 t s))
("petzel" nil (p eh1 t z ax l))
("petzinger" nil (p eh1 t s ih1 ng er0))
("petzold" nil (p eh1 t z ow0 l d))
("petzoldt" nil (p eh1 t z ow0 l t))
("peugeot" nil (p y uw0 zh ow1))
("peugh" nil (p y uw1))
("pevehouse" nil (p iy1 v hh aw1 s))
("peveler" nil (p eh1 v ax l er0))
("peverell" nil (p eh1 v er0 eh1 l))
("peveto" nil (p eh0 v eh1 t ow0))
("pevey" nil (p ih0 v ey1))
("pew" nil (p y uw1))
("pewabic" nil (p y uw1 aa1 b ih0 k))
("pewex" nil (p eh1 w eh1 k s))
("pewitt" nil (p y uw1 ih0 t))
("pews" nil (p y uw1 z))
("pewter" nil (p y uw1 t er0))
("peyman" nil (p ey1 m ax n))
("peyrelevade" nil (p eh1 r eh1 l ax v ey1 d))
("peyser" nil (p ey1 z er0))
("peyton" nil (p ey1 t ax n))
("pez" nil (p eh1 z))
("pezeshkan" nil (p ax z eh1 sh k ax n))
("pezim" nil (p eh1 z iy1 m))
("pezza" nil (p eh1 z ax))
("pezzella" nil (p eh0 t s eh1 l ax))
("pezzullo" nil (p eh0 t s uw1 l ow0))
("pfaff" nil (f ae1 f))
("pfahl" nil (f aa1 l))
("pfahler" nil (f aa1 l er0))
("pfalzgraf" nil (f ae1 l z g r ae0 f))
("pfannenstiel" nil (f ae1 n ax n s t iy0 l))
("pfarr" nil (f aa1 r))
("pfau" nil (f aw1))
("pfeffer" nil (f eh1 f er0))
("pfefferkorn" nil (f eh1 f er0 k ao1 r n))
("pfefferle" nil (f eh1 f er0 ax l))
("pfeifer" nil (f ay1 f er0))
("pfeiffer" nil (f ay1 f er0))
("pfeifle" nil (f ay1 f ax l))
("pfeil" nil (f ay1 l))
("pfennig" nil (f eh1 n ih0 g))
("pfennigs" nil (f eh1 n ih0 g z))
("pfenning" nil (f eh1 n ih0 ng))
("pfenninger" nil (f eh1 n ih0 ng er0))
("pfeuffer" nil (f y uw1 f er0))
("pfiester" nil (f ay1 s t er0))
("pfiffner" nil (f ih1 f n er0))
("pfingsten" nil (f ih1 ng s t ax n))
("pfister" nil (f ih1 s t er0))
("pfisterer" nil (f ih1 s t er0 er0))
("pfizer" nil (f ay1 z er0))
("pflaum" nil (f l aw1 m))
("pfleger" nil (f l iy1 g er0))
("pfleiderer" nil (f l ay1 d er0 er0))
("pflieger" nil (f l iy1 g er0))
("pflueger" nil (f l uw1 g er0))
("pflug" nil (f l uw1 g))
("pfluger" nil (f l uw1 g er0))
("pflum" nil (f l ah1 m))
("pfohl" nil (f ow1 l))
("pforzheimer" nil (f ao1 r z hh ay1 m er0))
("pfost" nil (f aa1 s t))
("pfund" nil (f ah1 n d))
("pfundstein" nil (f ah1 n d s t iy1 n))
("ph" nil (p iy1 ey1 ch))
("phagan" nil (f ey1 g ax n))
("phair" nil (f eh1 r))
("phalanges" nil (f ax l ae1 n jh iy0 z))
("phalanx" nil (f ey1 l ae0 ng k s))
("phalen" nil (f ae1 l ax n))
("phallic" nil (f ae1 l ih0 k))
("pham" nil (f ae1 m))
("phan" nil (f ae1 n))
("phaneuf" nil (f ae1 n uw0 f))
("phantasm" nil (f ae1 n t ae1 z ax m))
("phantom" nil (f ae1 n t ax m))
("phantoms" nil (f ae1 n t ax m z))
("phar" nil (f aa1 r))
("pharaon" nil (f eh1 r ow0 n))
("pharaonic" nil (f eh1 r ax aa1 n ih0 k))
("phares" nil (f eh1 r z))
("pharis" nil (f ae1 r ax s))
("pharisaism" nil (f ae1 r ih0 s ey1 ih1 z ax m))
("phariss" nil (f er0 ih1 s))
("pharma" nil (f aa1 r m ax))
("pharmaceutical" nil (f aa1 r m ax s uw1 t ih0 k ax l))
("pharmaceuticals" nil (f aa1 r m ax s uw1 t ih0 k ax l z))
("pharmacia" nil (f aa0 r m ey1 s iy0 ax))
("pharmacies" nil (f aa1 r m ax s iy0 z))
("pharmacist" nil (f aa1 r m ax s ih0 s t))
("pharmacists" nil (f aa1 r m ax s ih0 s t s))
("pharmacological" nil (f aa1 r m ax k ax l aa1 jh ih0 k ax l))
("pharmacologist" nil (f aa1 r m ax k aa1 l ax jh ih0 s t))
("pharmacology" nil (f aa1 r m ax k aa1 l ax jh iy0))
("pharmacy" nil (f aa1 r m ax s iy0))
("pharmakinetics" nil (f aa1 r m ax k ax n eh1 t ih0 k s))
("pharo" nil (f aa1 r ow0))
("pharoah" nil (f ey1 r ow0))
("pharos" nil (f eh1 r ow1 z))
("pharr" nil (f aa1 r))
("pharris" nil (f ae1 r ih0 s))
("pharynx" nil (f eh1 r ih0 ng k s))
("phase" nil (f ey1 z))
("phased" nil (f ey1 z d))
("phaseout" nil (f ey1 z aw1 t))
("phases" nil (f ey1 z ax z))
("phasing" nil (f ey1 z ih0 ng))
("pheasant" nil (f eh1 z ax n t))
("pheasants" nil (f eh1 z ax n t s))
("phebe" nil (f iy1 b iy0))
("phebus" nil (f iy1 b ax s))
("phedra" nil (f eh1 d r ax))
("phegley" nil (f eh1 g l iy0))
("phelan" nil (f eh1 l ax n))
("phelps" nil (f eh1 l p s))
("phenicie" nil (f eh1 n ax k iy0))
("phenix" nil (f eh1 n ih0 k s))
("phenol" nil (f iy1 n aa0 l))
("phenolic" nil (f ax n aa1 l ih0 k))
("phenom" nil (f eh1 n ax m))
("phenomena" nil (f ax n aa1 m ax n ax))
("phenomenal" nil (f ax n aa1 m ax n ax l))
("phenomenally" nil (f ax n aa1 m ax n ax l iy0))
("phenomenon" nil (f ax n aa1 m ax n aa1 n))
("phenotypic" nil (f iy1 n ax t ih1 p ih0 k))
("pherigo" nil (f eh0 r iy1 g ow0))
("pheromones" nil (f eh1 r ax m ow1 n z))
("pherson" nil (f er1 s ax n))
("phetteplace" nil (f eh1 t p l ey0 s))
("phew" nil (f y uw1))
("phi" nil (f ay1))
("phibbs" nil (f ih1 b z))
("phibro" nil (f ih1 b r ow0))
("phifer" nil (f ay1 f er0))
("phil" nil (f ih1 l))
("philadelphia" nil (f ih1 l ax d eh1 l f iy0 ax))
("philadelphians" nil (f ih1 l ax d eh1 l f iy0 ax n z))
("philadephia" nil (f ih1 l ax d eh1 f iy0 ax))
("philana" nil (f ih0 l ae1 n ax))
("philandering" nil (f ay1 l ae1 n d er0 ih0 ng))
("philantha" nil (f ih0 l ae1 n th ax))
("philanthropic" nil (f ih1 l ax n th r aa1 p ih0 k))
("philanthropies" nil (f ax l ae1 n th r ax p iy0 z))
("philanthropist" nil (f ax l ae1 n th r ax p ax s t))
("philanthropists" nil (f ih0 l ae1 n th r ax p ih0 s t s))
("philanthropy" nil (f ih0 l ae1 n th r ax p iy0))
("philbeck" nil (f ih1 l b eh1 k))
("philbert" nil (f ih1 l b er0 t))
("philberta" nil (f ih0 l b er1 t ax))
("philbin" nil (f ih1 l b ih0 n))
("philbrick" nil (f ih1 l b r ih0 k))
("philbrook" nil (f ih1 l b r uh1 k))
("philene" nil (f ih0 l iy1 n))
("phileo" nil (f ih1 l iy0 ow0))
("philharmonia" nil (f ih1 l hh aa0 r m ow1 n iy0 ax))
("philharmonic" nil (f ih1 l hh aa1 r m aa1 n ih0 k))
("philhower" nil (f ih1 l aw0 er0))
("philibert" nil (f ih1 l ih0 b er0 t))
("philibosian" nil (f ih1 l ih0 b ow1 z iy0 ax n))
("philida" nil (f ax l iy1 d ax))
("philina" nil (f ax l iy1 n ax))
("philip" nil (f ih1 l ax p))
("philipp" nil (f ih1 l ih0 p))
("philippa" nil (f ih0 l ih1 p ax))
("philippe" nil (f ih0 l iy1 p ey0))
("philippi" nil (f ih0 l ih1 p iy0))
("philippic" nil (f ih0 l ih1 p ih0 k))
("philippine" nil (f ih1 l ax p iy1 n))
("philippines" nil (f ih1 l ax p iy1 n z))
("philipps" nil (f ih1 l ih0 p s))
("philips" nil (f ih1 l ax p s))
("philipson" nil (f ih1 l ax p s ax n))
("philistia" nil (f ih0 l ih1 s t iy0 ax))
("philistine" nil (f ih1 l ax s t iy1 n))
("philistines" nil (f ih1 l ax s t iy1 n z))
("phillabaum" nil (f ih1 l ax b aw1 m))
("philley" nil (f ih1 l iy0))
("phillie" nil (f ih1 l iy0))
("phillies" nil (f ih1 l iy0 z))
("phillip" nil (f ih1 l ih0 p))
("phillipe" nil (f ih0 l iy1 p))
("phillipines" nil (f ih1 l ax p iy1 n z))
("phillipp" nil (f ih1 l ih0 p))
("phillippe" nil (f ih1 l ax p))
("phillippi" nil (f ax l iy1 p iy0))
("phillipps" nil (f ih1 l ih0 p s))
("phillips" nil (f ih1 l ih0 p s))
("phillipson" nil (f ih1 l ax p s ax n))
("phillis" nil (f ih1 l ih0 s))
("philly" nil (f ih1 l iy0))
("philo" nil (f iy1 l ow0))
("philodendron" nil (f ih1 l ax d eh1 n d r ax n))
("philomela" nil (f ih1 l ax m iy1 l ax))
("philomena" nil (f ih0 l ax m iy1 n ax))
("philon" nil (f ih1 l ax n))
("philosopher" nil (f ax l aa1 s ax f er0))
("philosophers" nil (f ax l aa1 s ax f er0 z))
("philosophic" nil (f ih1 l ax s aa1 f ih0 k))
("philosophical" nil (f ih1 l ax s aa1 f ih0 k ax l))
("philosophically" nil (f ih1 l ax s aa1 f ih0 k ax l iy0))
("philosophies" nil (f ax l aa1 s ax f iy0 z))
("philosophy" nil (f ax l aa1 s ax f iy0))
("philp" nil (f ih1 l p))
("philpot" nil (f ih1 l p ax t))
("philpott" nil (f ih1 l p ax t))
("philson" nil (f ih1 l s ax n))
("philyaw" nil (f ih1 l y aa1))
("phineas" nil (f ih1 n iy0 ax s))
("phinney" nil (f ih1 n iy0))
("phippen" nil (f ih1 p ax n))
("phipps" nil (f ih1 p s))
("phlcorp" nil (p iy1 ey1 ch eh1 l k ao1 r p))
("phlogopite" nil (f l aa1 g ax p ay1 t))
("phnom" nil (f n aa1 m))
("phobia" nil (f ow1 b iy0 ax))
("phobias" nil (f ow1 b iy0 ax z))
("phobos" nil (f ow1 b ow0 s))
("phoebe" nil (f iy1 b iy0))
("phoebus" nil (f iy1 b ax s))
("phoenicia" nil (f ax n iy1 sh ax))
("phoenician" nil (f ax n iy1 sh ax n))
("phoenicians" nil (f ax n iy1 sh ax n z))
("phoenix" nil (f iy1 n ih0 k s))
("phone" nil (f ow1 n))
("phoned" nil (f ow1 n d))
("phonemate" nil (f ow1 n m ey1 t))
("phones" nil (f ow1 n z))
("phonetic" nil (f ax n eh1 t ih0 k))
("phonetically" nil (f ax n eh1 t ih0 k l iy0))
("phonetics" nil (f ax n eh1 t ih0 k s))
("phoney" nil (f ow1 n iy0))
("phonic" nil (f aa1 n ih0 k))
("phonics" nil (f aa1 n ih0 k s))
("phoning" nil (f ow1 n ih0 ng))
("phonograph" nil (f ow1 n ax g r ae1 f))
("phonological" nil (f ow1 n ax l aa1 jh ih0 k ax l))
("phony" nil (f ow1 n iy0))
("phosphate" nil (f aa1 s f ey0 t))
("phosphates" nil (f aa1 s f ey0 t s))
("phosphatic" nil (f aa0 s f ae1 t ih0 k))
("phosphor" nil (f aa1 s f ao1 r))
("phosphorescence" nil (f aa1 s f er0 eh1 s ax n s))
("phosphoric" nil (f aa1 s f er0 ih0 k))
("phosphorites" nil (f aa1 s f er0 ay1 t s))
("phosphorous" nil (f aa1 s f er0 ax s))
("phosphors" nil (f aa1 s f ao1 r z))
("phosphorus" nil (f aa1 s f er0 ax s))
("photo" nil (f ow1 t ow1))
("photochemical" nil (f ow1 t ow0 k eh1 m ax k ax l))
("photoconductive" nil (f ow1 t ow0 k ax n d ah1 k t ih0 v))
("photocopied" nil (f ow1 t ow0 k aa1 p iy0 d))
("photocopier" nil (f ow1 t ow0 k aa1 p iy0 er0))
("photocopiers" nil (f ow1 t ow0 k aa1 p iy0 er0 z))
("photocopies" nil (f ow1 t ow0 k aa1 p iy0 z))
("photocopy" nil (f ow1 t ow0 k aa1 p iy0))
("photocopying" nil (f ow1 t ow0 k aa1 p iy0 ih0 ng))
("photodynamic" nil (f ow1 t ow0 d ay0 n ae1 m ih0 k))
("photoelectric" nil (f ow1 t ow0 ih0 l eh1 k t r ih0 k))
("photofinisher" nil (f ow1 t ow0 f ih1 n ih1 sh er0))
("photofinishers" nil (f ow1 t ow0 f ih1 n ih1 sh er0 z))
("photofinishing" nil (f ow1 t ow0 f ih1 n ih0 sh ih0 ng))
("photogenic" nil (f ow1 t ax jh eh1 n ih0 k))
("photograph" nil (f ow1 t ax g r ae1 f))
("photographed" nil (f ow1 t ax g r ae1 f t))
("photographer" nil (f ax t aa1 g r ax f er0))
("photographers" nil (f ax t aa1 g r ax f er0 z))
("photographic" nil (f ow1 t ax g r ae1 f ih0 k))
("photographing" nil (f ow1 t ax g r ae1 f ih0 ng))
("photographs" nil (f ow1 t ax g r ae1 f s))
("photography" nil (f ax t aa1 g r ax f iy0))
("photon" nil (f ow1 t aa1 n))
("photons" nil (f ow1 t aa1 n z))
("photorefractive" nil (f ow1 t ow0 r ih0 f r ae1 k t ih0 v))
("photos" nil (f ow1 t ow1 z))
("photosynthesis" nil (f ow1 t ow0 s ih1 n th ax s ih0 s))
("phototaxis" nil (f ow1 t ax t ae1 k s ih0 s))
("phototron" nil (f ow1 t ax t r aa0 n))
("photovoltaic" nil (f ow1 t ax v ow1 l t ey1 ih0 k))
("photovoltaics" nil (f ow1 t ow0 v ow0 l t ey1 ih0 k s))
("photronics" nil (f ow1 t r aa1 n ih0 k s))
("phrase" nil (f r ey1 z))
("phrased" nil (f r ey1 z d))
("phrases" nil (f r ey1 z ax z))
("phrasing" nil (f r ey1 z ih0 ng))
("phrygian" nil (f r ih1 jh iy0 ax n))
("phu" nil (f uw1))
("phua" nil (f y uw1 ax))
("phung" nil (f ah1 ng))
("phuong" nil (f uw0 ao1 ng))
("phut" nil (f ah1 t))
("phy" nil (f ay1))
("phyla" nil (f ay1 l ax))
("phyle" nil (f ay1 l))
("phylis" nil (f ay1 l ax s))
("phyllis" nil (f ih1 l ih0 s))
("phyllys" nil (f ih1 l iy0 z))
("phylogeny" nil (f ay0 l aa1 jh ax n iy0))
("phylum" nil (f ay1 l ax m))
("physical" nil (f ih1 z ih0 k ax l))
("physically" nil (f ih1 z ih0 k ax l iy0))
("physicals" nil (f ih1 z ih0 k ax l z))
("physician" nil (f ax z ih1 sh ax n))
("physicians" nil (f ax z ih1 sh ax n z))
("physicist" nil (f ih1 z ih0 s ih0 s t))
("physicists" nil (f ih1 z ih0 s ih0 s t s))
("physics" nil (f ih1 z ih0 k s))
("physio" nil (f ih1 z iy0 ow0))
("physiological" nil (f ih1 z iy0 ax l aa1 jh ih0 k ax l))
("physiologist" nil (f ih1 z iy0 aa1 l ax jh ih0 s t))
("physiology" nil (f ih1 z iy0 aa1 l ax jh iy0))
("physique" nil (f ax z iy1 k))
("phytogeography" nil (f ay0 t ow0 jh iy0 aa1 g r ax f iy0))
("phytoplankton" nil (f ay1 t ow0 p l ae1 ng t ax n))
("pi" nil (p ay1))
("pia" nil (p iy1 ax))
("piacente" nil (p iy0 aa0 ch eh1 n t iy0))
("piacentini" nil (p iy0 aa0 ch eh0 n t iy1 n iy0))
("piaget" nil (p iy1 ax z ey1))
("pianist" nil (p iy0 ae1 n ax s t))
("pianists" nil (p iy0 ae1 n ax s t s))
("pianka" nil (p iy0 aa1 ng k ax))
("piano" nil (p iy0 ae1 n ow0))
("pianos" nil (p iy0 ae1 n ow0 z))
("piascik" nil (p iy1 ax s ch ih0 k))
("piasecki" nil (p iy0 ax s eh1 t s k iy0))
("piasio" nil (p iy0 ae1 s iy0 ow0))
("piatek" nil (p iy0 aa1 t eh0 k))
("piatkowski" nil (p iy0 ax t k ao1 f s k iy0))
("piatt" nil (p ay1 ax t))
("piazza" nil (p iy0 ae1 z ax))
("piazzolla" nil (p iy1 ax z aa1 l ax))
("pic" nil (p ih1 k))
("pica" nil (p ay1 k ax))
("picante" nil (p iy0 k aa1 n t ey0))
("picard" nil (p ih0 k aa1 r d))
("picariello" nil (p iy0 k aa0 r iy0 eh1 l ow0))
("picaro" nil (p iy1 k aa0 r ow1))
("picas" nil (p ay1 k ax z))
("picasso" nil (p ih0 k aa1 s ow0))
("picassos" nil (p ih0 k aa1 s ow0 s))
("picayune" nil (p ih1 k iy0 y uw1 n))
("picazo" nil (p iy0 k aa1 z ow0))
("piccadilly" nil (p ih1 k ax d ih1 l iy0))
("picchi" nil (p ih1 k iy0))
("picciano" nil (p iy1 ch iy0 aa1 n ow0))
("piccinini" nil (p iy0 ch iy0 n iy1 n iy0))
("piccione" nil (p iy0 k ch ow1 n iy0))
("piccirilli" nil (p iy0 ch ih0 r iy1 l iy0))
("piccirillo" nil (p iy0 ch ih0 r ih1 l ow0))
("picco" nil (p ih1 k ow0))
("piccola" nil (p iy0 k ow1 l ax))
("piccoli" nil (p iy0 k ow1 l iy0))
("piccolo" nil (p ih1 k ax l ow1))
("piccone" nil (p iy0 k ow1 n iy0))
("picha" nil (p ih1 ch ax))
("piche" nil (p ih1 ch))
("picheny" nil (p ax ch ey1 n iy0))
("picher" nil (p ih1 ch er0))
("pichette" nil (p ax sh eh1 t))
("pichler" nil (p ih1 k ax l er0))
("pichon" nil (p ih1 ch ax n))
("picht" nil (p ih1 k t))
("picinich" nil (p ih1 s ih0 n ih0 ch))
("pick" nil (p ih1 k))
("pickands" nil (p ih1 k ax n d z))
("pickar" nil (p ih0 k aa1 r))
("pickard" nil (p ih0 k aa1 r d))
("pickart" nil (p ih1 k aa1 r t))
("picked" nil (p ih1 k t))
("pickel" nil (p ih1 k ax l))
("pickell" nil (p ih1 k ax l))
("pickelsimer" nil (p ih1 k ih0 l s ih0 m er0))
("picken" nil (p ih1 k ax n))
("pickens" nil (p ih1 k ax n z))
("picker" nil (p ih1 k er0))
("pickeral" nil (p ih1 k er0 ax l))
("pickerel" nil (p ih1 k er0 ax l))
("pickerell" nil (p ih1 k er0 ax l))
("pickerill" nil (p ih1 k er0 ih1 l))
("pickering" nil (p ih1 k er0 ih0 ng))
("pickers" nil (p ih1 k er0 z))
("pickert" nil (p ih1 k er0 t))
("picket" nil (p ih1 k ax t))
("picketed" nil (p ih1 k ax t ih0 d))
("picketers" nil (p ih1 k ax t er0 z))
("picketing" nil (p ih1 k ax t ih0 ng))
("pickets" nil (p ih1 k ax t s))
("pickett" nil (p ih1 k ih0 t))
("pickford" nil (p ih1 k f er0 d))
("pickier" nil (p ih1 k iy0 er0))
("picking" nil (p ih1 k ih0 ng))
("pickings" nil (p ih1 k ih0 ng z))
("pickle" nil (p ih1 k ax l))
("pickled" nil (p ih1 k ax l d))
("pickler" nil (p ih1 k ax l er0))
("pickles" nil (p ih1 k ax l z))
("picklesimer" nil (p ih1 k l ih0 s ih0 m er0))
("pickling" nil (p ih1 k l ih0 ng))
("pickney" nil (p ih1 k n iy0))
("pickpocket" nil (p ih1 k p aa1 k ax t))
("pickpockets" nil (p ih1 k p aa1 k ax t s))
("pickrel" nil (p ih1 k r ax l))
("pickrell" nil (p ih1 k r ax l))
("pickren" nil (p ih1 k er0 ax n))
("pickron" nil (p ih1 k r ax n))
("picks" nil (p ih1 k s))
("pickup" nil (p ih1 k ah1 p))
("pickups" nil (p ih1 k ah1 p s))
("pickus" nil (p ih1 k ax s))
("pickwick" nil (p ih1 k w ih1 k))
("pickworth" nil (p ih1 k w er1 th))
("picky" nil (p ih1 k iy0))
("picnic" nil (p ih1 k n ih1 k))
("picnics" nil (p ih1 k n ih1 k s))
("pico" nil (p iy1 k ow0))
("picon" nil (p ih1 k ax n))
("picone" nil (p ih0 k ow1 n))
("picop" nil (p ih1 k aa1 p))
("picotte" nil (p ih0 k ao1 t))
("picou" nil (p iy1 k uw0))
("picoult" nil (p ih0 k ow1 l t))
("picower" nil (p ih1 k aw1 r))
("pictet" nil (p ih1 k t ih0 t))
("picton" nil (p ih1 k t ax n))
("pictorial" nil (p ih0 k t ao1 r iy0 ax l))
("picts" nil (p ih1 k t s))
("picture" nil (p ih1 k ch er0))
("pictured" nil (p ih1 k ch er0 d))
("pictures" nil (p ih1 k ch er0 z))
("picturesque" nil (p ih1 k ch er0 ax s k))
("picturetel" nil (p ih1 k ch er1 t eh1 l))
("picturing" nil (p ih1 k ch er0 ih0 ng))
("pidcock" nil (p ih1 d k ax k))
("piddle" nil (p ih1 d ax l))
("piddles" nil (p ih1 d ax l z))
("piddling" nil (p ih1 d ax l ih0 ng))
("piddock" nil (p ih1 d ax k))
("pidgeon" nil (p ih1 d jh ih0 n))
("pie" nil (p ay1))
("piebald" nil (p ay1 b ao1 l d))
("piece" nil (p iy1 s))
("pieced" nil (p iy1 s t))
("piecemeal" nil (p iy1 s m iy1 l))
("pieces" nil (p iy1 s ax z))
("piecework" nil (p iy1 s w er1 k))
("piech" nil (p iy1 ch))
("piechocki" nil (p iy0 hh ow1 t s k iy0))
("piechota" nil (p iy0 hh ow1 t ax))
("piechowski" nil (p iy0 hh ao1 f s k iy0))
("piecing" nil (p iy1 s ih0 ng))
("piecuch" nil (p iy1 k ax k))
("pieczynski" nil (p iy0 ch ih1 n s k iy0))
("pied" nil (p ay1 d))
("piedboeuf" nil (p iy1 d b ah1 f))
("piedmont" nil (p iy1 d m aa1 n t))
("piedra" nil (p iy1 d r ax))
("piehl" nil (p iy1 l))
("piekarski" nil (p iy0 k aa1 r s k iy0))
("piel" nil (p iy1 l))
("piela" nil (p iy1 l ax))
("pienta" nil (p iy1 n t ax))
("pieper" nil (p iy1 p er0))
("piepgras" nil (p iy1 p g r ae1 s))
("piepho" nil (p iy1 f ow0))
("pier" nil (p ih1 r))
("pieraccini" nil (p ih1 r ax ch iy1 n iy0))
("pieratt" nil (p iy1 r ax t))
("pierce" nil (p ih1 r s))
("pierceall" nil (p ih0 r s iy1 l))
("pierced" nil (p ih1 r s t))
("piercey" nil (p ih0 r s iy1))
("piercing" nil (p ih1 r s ih0 ng))
("piercy" nil (p ih1 r k iy0))
("pierette" nil (p ih0 r eh1 t))
("pieri" nil (p iy1 r iy0))
("pierini" nil (p ih0 r iy1 n iy0))
("pierman" nil (p ih1 r m ax n))
("piero" nil (p iy1 r ow0))
("pieroni" nil (p ih0 r ow1 n iy0))
("pierotti" nil (p ih0 r ow1 t iy0))
("pierpoint" nil (p ih0 r p oy1 n t))
("pierpont" nil (p ih1 r p aa1 n t))
("pierre" nil (p iy0 eh1 r))
("pierrelouis" nil (p ih1 r ih0 l w iy0 z))
("pierrepont" nil (p ih0 r ey1 p ow0 n t))
("pierro" nil (p ih1 r ow0))
("pierron" nil (p ih0 r ao1 n))
("piers" nil (p ih1 r z))
("piersall" nil (p ih1 r s ax l))
("piersol" nil (p ih1 r s ao0 l))
("pierson" nil (p ih1 r s ax n))
("pies" nil (p ay1 z))
("piet" nil (p ay1 ih0 t))
("pieter" nil (p iy1 t er0))
("pietermaritzburg" nil (p iy1 t er0 m eh1 r ih0 t s b er0 g))
("pieters" nil (p iy1 t er0 z))
("pieties" nil (p ay1 ax t iy0 z))
("pietila" nil (p iy0 t iy1 l ax))
("pietism" nil (p iy1 t ih0 z ax m))
("pietrangelo" nil (p iy0 t r aa0 ng g eh1 l ow0))
("pietras" nil (p iy1 t r ax z))
("pietro" nil (p iy0 eh1 t r ow0))
("pietrowski" nil (p iy0 t r ao1 f s k iy0))
("pietruski" nil (p iy1 eh0 t r ah1 s k iy0))
("pietruszka" nil (p iy0 t r ah1 sh k ax))
("pietrzak" nil (p iy1 t er0 z ae0 k))
("pietrzyk" nil (p iy1 t er0 z ih0 k))
("pietsch" nil (p iy1 ch))
("piette" nil (p iy1 t))
("piety" nil (p ay1 ax t iy0))
("pietz" nil (p iy1 t s))
("pifer" nil (p ay1 f er0))
("pig" nil (p ih1 g))
("pigeon" nil (p ih1 jh ax n))
("pigeonholed" nil (p ih1 jh ax n hh ow1 l d))
("pigeons" nil (p ih1 jh ax n z))
("pigford" nil (p ih1 g f er0 d))
("pigg" nil (p ih1 g))
("piggee" nil (p ih1 g iy1))
("piggly" nil (p ih1 g l iy0))
("piggott" nil (p ih1 g ax t))
("piggy" nil (p ih1 g iy0))
("piggyback" nil (p ih1 g iy0 b ae1 k))
("piggybacked" nil (p ih1 g iy0 b ae1 k t))
("piggybacking" nil (p ih1 g iy0 b ae1 k ih0 ng))
("pigman" nil (p ih1 g m ax n))
("pigment" nil (p ih1 g m ax n t))
("pigmentation" nil (p ih1 g m ax n t ey1 sh ax n))
("pigments" nil (p ih1 g m ax n t s))
("pigmied" nil (p ih1 g m iy0 d))
("pigmy" nil (p ih1 g m iy0))
("pignataro" nil (p iy0 g n aa0 t aa1 r ow0))
("pignatelli" nil (p iy0 g n aa0 t eh1 l iy0))
("pignato" nil (p iy0 g n aa1 t ow0))
("pignone" nil (p iy0 g n ow1 n iy0))
("pigott" nil (p ih1 g ax t))
("pigs" nil (p ih1 g z))
("pigskin" nil (p ih1 g s k ih1 n))
("pigue" nil (p iy1 g))
("pih" nil (p ih1))
("pihl" nil (p ih1 l))
("pik" nil (p ih1 k))
("pike" nil (p ay1 k))
("piker" nil (p ay1 k er0))
("pikes" nil (p ay1 k s))
("pikul" nil (p ih1 k ax l))
("pikula" nil (p ih0 k uw1 l ax))
("pikus" nil (p ay1 k ax s))
("pil" nil (p ih1 l))
("piland" nil (p ih1 l ax n d))
("pilant" nil (p iy1 l ax n t))
("pilar" nil (p ay1 l er0))
("pilarski" nil (p ih0 l aa1 r s k iy0))
("pilasters" nil (p ax l ae1 s t er0 z))
("pilat" nil (p iy1 l aa0 t))
("pilato" nil (p iy0 l aa1 t ow0))
("pilch" nil (p ih1 l ch))
("pilcher" nil (p ih1 l ch er0))
("pile" nil (p ay1 l))
("pilecki" nil (p ih0 l eh1 k iy0))
("piled" nil (p ay1 l d))
("pileggi" nil (p ih0 l eh1 jh iy0))
("piles" nil (p ay1 l z))
("pileup" nil (p ay1 l ah1 p))
("pilevsky" nil (p ih0 l eh1 v s k iy0))
("pilferage" nil (p ih1 l f er0 ih0 jh))
("pilfering" nil (p ih1 l f er0 ih0 ng))
("pilger" nil (p ih1 l g er0))
("pilgram" nil (p ih1 l g r ax m))
("pilgreen" nil (p ih0 l g r iy1 n))
("pilgrim" nil (p ih1 l g r ax m))
("pilgrimage" nil (p ih1 l g r ax m ax jh))
("pilgrimages" nil (p ih1 l g r ax m ih0 jh ih0 z))
("pilgrims" nil (p ih1 l g r ax m z))
("piling" nil (p ay1 l ih0 ng))
("pilings" nil (p ay1 l ih0 ng z))
("pilkenton" nil (p ih0 l k eh1 n t ax n))
("pilkerton" nil (p ih0 l k er1 t ax n))
("pilkington" nil (p ih1 l k ih0 ng t ax n))
("pilkinton" nil (p ih1 l k ih0 n t ax n))
("pill" nil (p ih1 l))
("pilla" nil (p ih1 l ax))
("pillage" nil (p ih1 l ih0 jh))
("pillaged" nil (p ih1 l ih0 jh d))
("pillager" nil (p ih1 l ih0 jh er0))
("pillagers" nil (p ih1 l ih0 jh er0 z))
("pillages" nil (p ih1 l ih0 jh ih0 z))
("pillaging" nil (p ih1 l ih0 jh ih0 ng))
("pillar" nil (p ih1 l er0))
("pillard" nil (p ih1 l er0 d))
("pillars" nil (p ih1 l er0 z))
("pille" nil (p ih1 l))
("piller" nil (p ih1 l er0))
("pilley" nil (p ih1 l iy0))
("pilling" nil (p ih1 l ih0 ng))
("pillion" nil (p ih1 l y ax n))
("pilloried" nil (p ih1 l er0 iy0 d))
("pillow" nil (p ih1 l ow0))
("pillows" nil (p ih1 l ow0 z))
("pillowtex" nil (p ih1 l ow0 t eh1 k s))
("pills" nil (p ih1 l z))
("pillsbury" nil (p ih1 l z b eh1 r iy0))
("pilon" nil (p iy0 l ao1 n))
("pilot" nil (p ay1 l ax t))
("piloted" nil (p ay1 l ax t ih0 d))
("piloting" nil (p ay1 l ax t ih0 ng))
("pilotless" nil (p ay1 l ax t l ax s))
("pilots" nil (p ay1 l ax t s))
("pilotte" nil (p ih0 l ao1 t))
("pilson" nil (p ih1 l s ax n))
("piltz" nil (p ih1 l t s))
("pilz" nil (p ih1 l z))
("pima" nil (p iy1 m ax))
("pimco" nil (p ih1 m k ow0))
("pimental" nil (p ih0 m eh1 n t ax l))
("pimentel" nil (p ih1 m ih0 n t ax l))
("pimm" nil (p ih1 m))
("pimp" nil (p ih1 m p))
("pimple" nil (p ih1 m p ax l))
("pimples" nil (p ih1 m p ax l z))
("pimps" nil (p ih1 m p s))
("pin" nil (p ih1 n))
("pina" nil (p iy1 n ax))
("pinard" nil (p ih1 n er0 d))
("pinault" nil (p ih1 n ao1 l t))
("pinball" nil (p ih1 n b ao1 l))
("pincavage" nil (p ih1 ng k ax v ih0 jh))
("pincerlike" nil (p ih1 n s er0 l ay1 k))
("pincers" nil (p ih1 n s er0 z))
("pinch" nil (p ih1 n ch))
("pinched" nil (p ih1 n ch t))
("pincher" nil (p ih1 n ch er0))
("pinchers" nil (p ih1 n ch er0 z))
("pinching" nil (p ih1 n ch ih0 ng))
("pinckard" nil (p ih1 ng k er0 d))
("pinckney" nil (p ih1 ng k n iy0))
("pincus" nil (p ih1 ng k ax s))
("pindaric" nil (p ih0 n d ae1 r ih0 k))
("pindell" nil (p ih1 n d ax l))
("pinder" nil (p ay1 n d er0))
("pindling" nil (p ih1 n d l ih0 ng))
("pine" nil (p ay1 n))
("pineapple" nil (p ay1 n ae1 p ax l))
("pineapples" nil (p ay1 n ae1 p ax l z))
("pineau" nil (p ih0 n ow1))
("pineda" nil (p iy0 n eh1 d ax))
("pinedo" nil (p iy0 n ey1 d ow0))
("pinegar" nil (p ih1 n ih0 g er0))
("pineiro" nil (p iy0 n eh1 r ow0))
("pinellas" nil (p ih0 n eh1 l ax s))
("pinelli" nil (p ih0 n eh1 l iy0))
("pineo" nil (p ih1 n iy0 ow0))
("piner" nil (p ay1 n er0))
("pinero" nil (p ih0 n eh1 r ow0))
("pines" nil (p ay1 n z))
("pinetta" nil (p ax n eh1 t ax))
("pinette" nil (p ih0 n eh1 t))
("ping" nil (p ih1 ng))
("pingel" nil (p ih1 ng g ax l))
("pingitore" nil (p ih0 ng g iy0 t ao1 r iy0))
("pingleton" nil (p ih1 ng g l ih0 t aa0 n))
("pingley" nil (p ih1 ng g l iy0))
("pingpong" nil (p ih1 ng p ao0 ng))
("pingree" nil (p ih0 ng g r iy1))
("pinheiro" nil (p iy0 n hh eh1 r ow0))
("pinho" nil (p ih1 n hh ow0))
("pinholes" nil (p ih1 n hh ow1 l z))
("pini" nil (p iy1 n iy0))
("pining" nil (p ay1 n ih0 ng))
("pinion" nil (p ih1 n y ax n))
("pink" nil (p ih1 ng k))
("pinkard" nil (p ih1 ng k er0 d))
("pinkelman" nil (p ih1 ng k ax l m ax n))
("pinkerman" nil (p ih1 ng k er0 m ax n))
("pinkerton" nil (p ih1 ng k er0 t ax n))
("pinkett" nil (p ih1 ng k ih0 t))
("pinkham" nil (p ih1 ng k hh ax m))
("pinkie" nil (p ih1 ng k iy0))
("pinkish" nil (p ih1 ng k ih0 sh))
("pinkley" nil (p ih1 ng k l iy0))
("pinkney" nil (p ih1 ng k n iy0))
("pinkos" nil (p ih1 ng k ow0 z))
("pinkowski" nil (p ih0 ng k ao1 f s k iy0))
("pinks" nil (p ih1 ng k s))
("pinkstaff" nil (p ih1 ng k s t ae1 f))
("pinkston" nil (p ih1 ng k s t ax n))
("pinkus" nil (p ih1 ng k ax s))
("pinkwater" nil (p ih1 ng k w ao1 t er0))
("pinn" nil (p ih1 n))
("pinnacle" nil (p ih1 n ax k ax l))
("pinned" nil (p ih1 n d))
("pinnell" nil (p ih1 n ax l))
("pinneo" nil (p ih1 n iy0 ow0))
("pinner" nil (p ih1 n er0))
("pinney" nil (p ih1 n iy0))
("pinnick" nil (p ih1 n ih0 k))
("pinning" nil (p ih1 n ih0 ng))
("pinnix" nil (p ih1 n ih0 k s))
("pinnock" nil (p ih1 n ax k))
("pinnow" nil (p ih1 n ow0))
("pino" nil (p iy1 n ow0))
("pinocchio" nil (p ih0 n ow1 k iy0 ow0))
("pinochet" nil (p ih1 n ax sh ey1))
("pinola" nil (p ih0 n ow1 l ax))
("pinot" nil (p ih1 n ax t))
("pinpoint" nil (p ih1 n p oy1 n t))
("pinpointed" nil (p ih1 n p oy1 n t ih0 d))
("pinpointing" nil (p ih1 n p oy1 n t ih0 ng))
("pinpoints" nil (p ih1 n p oy1 n t s))
("pins" nil (p ih1 n z))
("pinsker" nil (p ih1 n s k er0))
("pinsky" nil (p ih1 n s k iy0))
("pinson" nil (p ih1 n s ax n))
("pinsoneault" nil (p ih1 n s ax n ao1 l t))
("pinstripe" nil (p ih1 n s t r ay1 p))
("pinstriped" nil (p ih1 n s t r ay1 p t))
("pinstripes" nil (p ih1 n s t r ay1 p s))
("pint" nil (p ay1 n t))
("pintar" nil (p iy0 n t aa1 r))
("pinter" nil (p ay1 n t er0))
("pinto" nil (p ih1 n t ow1))
("pints" nil (p ay1 n t s))
("piny" nil (p ay1 n iy0))
("pinyan" nil (p ih1 n y ax n))
("pinzon" nil (p ih1 n z ao1 n))
("pio" nil (p ay1 ow0))
("pion" nil (p ay1 aa1 n))
("pioneer" nil (p ay1 ax n ih1 r))
("pioneered" nil (p ay1 ax n ih1 r d))
("pioneering" nil (p ay1 ax n ih1 r ih0 ng))
("pioneers" nil (p ay1 ax n ih1 r z))
("piontek" nil (p iy0 ow1 n t eh0 k))
("piontkowski" nil (p iy0 ow0 n t k ao1 f s k iy0))
("piorkowski" nil (p iy0 ao0 r k ao1 f s k iy0))
("piotrowski" nil (p iy0 ow0 t r ao1 f s k iy0))
("piotter" nil (p iy0 aa1 t er0))
("pious" nil (p ay1 ax s))
("piously" nil (p ay1 ax s l iy0))
("pipe" nil (p ay1 p))
("piped" nil (p ay1 p t))
("pipefishes" nil (p ay1 p f ih1 sh ih0 z))
("pipeline" nil (p ay1 p l ay1 n))
("pipelines" nil (p ay1 p l ay1 n z))
("piper" nil (p ay1 p er0))
("pipers" nil (p ay1 p er0 z))
("pipes" nil (p ay1 p s))
("pipetec" nil (p ay1 p t eh1 k))
("pipher" nil (p ih1 f er0))
("piping" nil (p ay1 p ih0 ng))
("pipitone" nil (p ih1 p ih0 t ow1 n))
("pipkin" nil (p ih1 p k ih0 n))
("pipkins" nil (p ih1 p k ih0 n z))
("pipp" nil (p ih1 p))
("pippen" nil (p ih1 p ax n))
("pippenger" nil (p ih1 p ih0 n jh er0))
("pippert" nil (p ih1 p er0 t))
("pippin" nil (p ih1 p ih0 n))
("pippins" nil (p ih1 p ih0 n z))
("pips" nil (p ih1 p s))
("piquant" nil (p iy1 k ax n t))
("pique" nil (p iy1 k))
("piqued" nil (p iy1 k t))
("piquette" nil (p ih0 k eh1 t))
("piracy" nil (p ay1 r ax s iy0))
("piraino" nil (p ih0 r ey1 n ow0))
("pirandello" nil (p ih1 r ax n d eh1 l ow0))
("piranha" nil (p ih0 r ae1 n hh ax))
("pirate" nil (p ay1 r ax t))
("pirated" nil (p ay1 r ax t ih0 d))
("pirates" nil (p ay1 r ax t s))
("pirating" nil (p ay1 r ax t ih0 ng))
("pirelli" nil (p ih0 r eh1 l iy0))
("pires" nil (p ay1 r z))
("pirie" nil (p ih1 r iy0))
("pirkey" nil (p er1 k iy0))
("pirkl" nil (p er1 k ax l))
("pirkle" nil (p er1 k ax l))
("pirko" nil (p er1 k ow0))
("piro" nil (p ih1 r ow0))
("pirog" nil (p er0 aa1 g))
("pirone" nil (p ih0 r ow1 n))
("pirouettes" nil (p ih1 r uw0 eh1 t s))
("pirozzi" nil (p ih0 r aa1 z iy0))
("pirrello" nil (p ih0 r eh1 l ow0))
("pirro" nil (p ih1 r ow0))
("pirrone" nil (p ih0 r ow1 n iy0))
("pirtle" nil (p er1 t ax l))
("pisa" nil (p iy1 s ax))
("pisani" nil (p iy0 s aa1 n iy0))
("pisano" nil (p iy0 s aa1 n ow0))
("pisarek" nil (p ih1 s er0 eh0 k))
("pisarski" nil (p ih0 s aa1 r s k iy0))
("piscataway" nil (p ih0 s k ae1 t ax w ey1))
("pisces" nil (p ay1 s iy0 z))
("pischel" nil (p ih1 sh ax l))
("pischke" nil (p ih1 sh k))
("pisciotta" nil (p iy0 s ch ow1 t ax))
("piscitelli" nil (p iy0 s ch iy0 t eh1 l iy0))
("piscitello" nil (p ih1 s ih0 t eh1 l ow0))
("piscopo" nil (p iy0 s k ow1 p ow0))
("pisello" nil (p ih0 s eh1 l ow0))
("pisoni" nil (p ih0 s ow1 n iy0))
("piss" nil (p ih1 s))
("pistachio" nil (p ax s t ae1 sh iy0 ow1))
("pistachios" nil (p ax s t ae1 sh iy0 ow1 z))
("pistil" nil (p ih1 s t ax l))
("pistilli" nil (p iy0 s t iy1 l iy0))
("pistol" nil (p ih1 s t ax l))
("pistole" nil (p ih0 s t ow1 l))
("pistols" nil (p ih1 s t ax l z))
("piston" nil (p ih1 s t ax n))
("pistone" nil (p iy1 s t ow0 n))
("pistons" nil (p ih1 s t ax n z))
("pistor" nil (p ih1 s t er0))
("pistorio" nil (p ih1 s t ao1 r iy0 ow0))
("pit" nil (p ih1 t))
("pita" nil (p iy1 t ax))
("pitblado" nil (p ih0 t b l aa1 d ow0))
("pitch" nil (p ih1 ch))
("pitchblende" nil (p ih1 ch b l eh1 n d))
("pitched" nil (p ih1 ch t))
("pitcher" nil (p ih1 ch er0))
("pitchers" nil (p ih1 ch er0 z))
("pitches" nil (p ih1 ch ih0 z))
("pitchford" nil (p ih1 ch f er0 d))
("pitchfork" nil (p ih1 ch f ao1 r k))
("pitching" nil (p ih1 ch ih0 ng))
("pitchman" nil (p ih1 ch m ax n))
("pitchmen" nil (p ih1 ch m eh1 n))
("pitcock" nil (p ih1 t k aa1 k))
("pitfall" nil (p ih1 t f ao1 l))
("pitfalls" nil (p ih1 t f ao1 l z))
("pith" nil (p ih1 th))
("pithy" nil (p ih1 th iy0))
("pitiable" nil (p ih1 t iy0 ax b ax l))
("pitiful" nil (p ih1 t ax f ax l))
("pitifully" nil (p ih1 t ih0 f ax l iy0))
("pitiless" nil (p ih1 t iy0 l ax s))
("pitkin" nil (p ih1 t k ih0 n))
("pitman" nil (p ih1 t m ax n))
("pitner" nil (p ih1 t n er0))
("pitney" nil (p ih1 t n iy0))
("pitofsky" nil (p ax t aa1 f s k iy0))
("pitre" nil (p ay1 t er0))
("pits" nil (p ih1 t s))
("pitsch" nil (p ih1 ch))
("pitsenbarger" nil (p ih1 t s ih0 n b aa0 r g er0))
("pitstick" nil (p ih1 t s t ih1 k))
("pitt" nil (p ih1 t))
("pittance" nil (p ih1 t ax n s))
("pittard" nil (p ih1 t er0 d))
("pitted" nil (p ih1 t ax d))
("pittencrieff" nil (p ih1 t ih0 n k r iy1 f))
("pittenger" nil (p ih1 t ih0 n jh er0))
("pitting" nil (p ih1 t ih0 ng))
("pittinger" nil (p ih1 t ih0 ng er0))
("pittle" nil (p ih1 t ax l))
("pittman" nil (p ih1 t m ax n))
("pittner" nil (p ih1 t n er0))
("pitts" nil (p ih1 t s))
("pittsboro" nil (p ih1 t s b er0 ow0))
("pittsburg" nil (p ih1 t s b er0 g))
("pittsburgh" nil (p ih1 t s b er0 g))
("pittsburghers" nil (p ih1 t s b er0 g er0 z))
("pittsfield" nil (p ih1 t s f iy0 l d))
("pittsford" nil (p ih1 t s f er0 d))
("pittsley" nil (p ih1 t s l iy0))
("pittston" nil (p ih1 t s t ax n))
("pituitary" nil (p ax t uw1 ax t eh1 r iy0))
("pity" nil (p ih1 t iy0))
("pitying" nil (p ih1 t iy0 ih0 ng))
("pitz" nil (p ih1 t s))
("pitzen" nil (p ih1 t z ax n))
("pitzer" nil (p ih1 t z er0))
("piver" nil (p ay1 v er0))
("pivonka" nil (p ih0 v aa1 ng k ax))
("pivot" nil (p ih1 v ax t))
("pivotal" nil (p ih1 v ax t ax l))
("pivoted" nil (p ih1 v ax t ax d))
("pixels" nil (p ih1 k s ax l z))
("pixie" nil (p ih1 k s iy0))
("pixler" nil (p ih1 k s l er0))
("pixley" nil (p ih1 k s l iy0))
("pizana" nil (p iy0 z ae1 n ax))
("pizano" nil (p iy0 z aa1 n ow0))
("pizarro" nil (p ih0 z aa1 r ow0))
("pizazz" nil (p ih0 z ae1 z))
("pizer" nil (p ay1 z er0))
("pizza" nil (p iy1 t s ax))
("pizzano" nil (p iy0 t s aa1 n ow0))
("pizzas" nil (p iy1 t s ax z))
("pizzazz" nil (p ih1 z ae1 z))
("pizzeria" nil (p iy1 t s er0 iy1 ax))
("pizzerias" nil (p iy1 t s er0 iy1 ax z))
("pizzi" nil (p ih1 z iy0))
("pizzimenti" nil (p iy0 t s iy0 m eh1 n t iy0))
("pizzino" nil (p iy0 t s iy1 n ow0))
("pizzitola" nil (p iy0 t s iy0 t ow1 l ax))
("pizzo" nil (p ih1 z ow0))
("pizzolato" nil (p iy0 t s ow0 l aa1 t ow0))
("pizzuti" nil (p iy0 t s uw1 t iy0))
("pizzuto" nil (p iy0 t s uw1 t ow0))
("pla" nil (p l aa1))
("placard" nil (p l ae1 k er0 d))
("placards" nil (p l ae1 k er0 d z))
("placate" nil (p l ey1 k ey0 t))
("placating" nil (p l ey1 k ey1 t ih0 ng))
("place" nil (p l ey1 s))
("placebo" nil (p l ax s iy1 b ow0))
("placebos" nil (p l ax s iy1 b ow0 z))
("placed" nil (p l ey1 s t))
("placek" nil (p l aa1 ch eh1 k))
("placement" nil (p l ey1 s m ax n t))
("placements" nil (p l ey1 s m ax n t s))
("placencia" nil (p l aa0 ch eh1 n ch ax))
("placenta" nil (p l ax s eh1 n t ax))
("placentia" nil (p l ax s eh1 n sh ax))
("placer" nil (p l ae1 s er0))
("places" nil (p l ey1 s ax z))
("placeway" nil (p l ey1 s w ey1))
("placid" nil (p l ae1 s ax d))
("placida" nil (p l aa0 ch iy1 d ax))
("placido" nil (p l aa1 ch ih0 d ow0))
("placing" nil (p l ey1 s ih0 ng))
("plack" nil (p l ae1 k))
("placke" nil (p l ae1 k))
("placko" nil (p l ae1 k ow0))
("placoid" nil (p l ae1 k oy0 d))
("placzek" nil (p l aa1 ch eh0 k))
("plagens" nil (p l ae1 g ax n z))
("plagge" nil (p l ae1 g))
("plagiarism" nil (p l ey1 jh er0 ih1 z ax m))
("plagiarized" nil (p l ey1 jh iy0 er0 ay1 z d))
("plagioclase" nil (p l ey1 jh iy0 ax k l ey1 s))
("plague" nil (p l ey1 g))
("plagued" nil (p l ey1 g d))
("plagues" nil (p l ey1 g z))
("plaguing" nil (p l ey1 g ih0 ng))
("plaia" nil (p l aa1 y ax))
("plaid" nil (p l ae1 d))
("plain" nil (p l ey1 n))
("plainclothes" nil (p l ey1 n k l ow1 z))
("plainer" nil (p l ey1 n er0))
("plaines" nil (p l ey1 n z))
("plainfield" nil (p l ey1 n f iy1 l d))
("plainly" nil (p l ey1 n l iy0))
("plains" nil (p l ey1 n z))
("plainsong" nil (p l ey1 n s ao1 ng))
("plaintiff" nil (p l ey1 n t ax f))
("plaintiffs" nil (p l ey1 n t ih0 f s))
("plaintive" nil (p l ey1 n t ih0 v))
("plaintively" nil (p l ey1 n t ay1 v l iy0))
("plainview" nil (p l ey1 n v y uw1))
("plair" nil (p l eh1 r))
("plaisance" nil (p l ey1 s ax n s))
("plaisted" nil (p l aa1 ih0 s t ih0 d))
("plake" nil (p l ey1 k))
("plam" nil (p l ae1 m))
("plamann" nil (p l aa1 m ax n))
("plambeck" nil (p l ae1 m b eh1 k))
("plamondon" nil (p l aa0 m ow0 n d ao1 n))
("plan" nil (p l ae1 n))
("planar" nil (p l ey1 n er0))
("planarian" nil (p l ax n eh1 r iy0 ax n))
("planas" nil (p l ae1 n ax z))
("planck" nil (p l ae1 ng k))
("plane" nil (p l ey1 n))
("planecon" nil (p l ae1 n ax k aa1 n))
("planed" nil (p l ey1 n d))
("planeload" nil (p l ae1 n ax l ow1 d))
("planeloads" nil (p l ae1 n ax l ow1 d z))
("planer" nil (p l ey1 n er0))
("planers" nil (p l ey1 n er0 z))
("planes" nil (p l ey1 n z))
("planet" nil (p l ae1 n ax t))
("planetarium" nil (p l ae1 n ax t eh1 r iy0 ax m))
("planetary" nil (p l ae1 n ax t eh1 r iy0))
("planets" nil (p l ae1 n ax t s))
("planitzer" nil (p l ae1 n ih0 t s er0))
("plank" nil (p l ae1 ng k))
("planking" nil (p l ae1 ng k ih0 ng))
("planks" nil (p l ae1 ng k s))
("plankton" nil (p l ae1 ng k t ax n))
("planktonic" nil (p l ae0 ng k t aa1 n ih0 k))
("planned" nil (p l ae1 n d))
("planner" nil (p l ae1 n er0))
("planners" nil (p l ae1 n er0 z))
("planning" nil (p l ae1 n ih0 ng))
("plano" nil (p l ey1 n ow0))
("plans" nil (p l ae1 n z))
("plant" nil (p l ae1 n t))
("plantain" nil (p l ae1 n t ax n))
("plantains" nil (p l ae1 n t ax n z))
("plantation" nil (p l ae1 n t ey1 sh ax n))
("plantations" nil (p l ae1 n t ey1 sh ax n z))
("plante" nil (p l ae1 n t))
("planted" nil (p l ae1 n t ax d))
("planter" nil (p l ae1 n t er0))
("planters" nil (p l ae1 n t er0 z))
("plantiffs" nil (p l ae1 n t ih0 f s))
("planting" nil (p l ae1 n t ih0 ng))
("plantings" nil (p l ae1 n t ih0 ng z))
("plantlike" nil (p l ae1 n t l ay1 k))
("plantronics" nil (p l ae1 n t r aa1 n ih0 k s))
("plants" nil (p l ae1 n t s))
("plantz" nil (p l ae1 n t s))
("plaque" nil (p l ae1 k))
("plaques" nil (p l ae1 k s))
("plas" nil (p l ae1 s))
("plascencia" nil (p l aa0 s ch eh1 n ch ax))
("plasencia" nil (p l aa0 s eh1 n ch ax))
("plaskett" nil (p l ae1 s k ih0 t))
("plasma" nil (p l ae1 z m ax))
("plasminogen" nil (p l ae1 z m ih1 n ax jh ih0 n))
("plasmodia" nil (p l ae0 z m ow1 d iy0 ax))
("plasmodium" nil (p l ae0 z m ow1 d iy0 ax m))
("plass" nil (p l ae1 s))
("plassard" nil (p l ae1 s er0 d))
("plasse" nil (p l ae1 s))
("plaster" nil (p l ae1 s t er0))
("plasterboard" nil (p l ae1 s t er0 b ao1 r d))
("plastered" nil (p l ae1 s t er0 d))
("plasterer" nil (p l ae1 s t er0 er0))
("plastering" nil (p l ae1 s t er0 ih0 ng))
("plasters" nil (p l ae1 s t er0 z))
("plasterwork" nil (p l ae1 s t er0 w er1 k))
("plastic" nil (p l ae1 s t ih0 k))
("plasticine" nil (p l ae1 s t ih0 s iy1 n))
("plasticizer" nil (p l ae1 s t ax s ay1 z er0))
("plastics" nil (p l ae1 s t ih0 k s))
("plata" nil (p l aa1 t ax))
("plate" nil (p l ey1 t))
("plateau" nil (p l ae0 t ow1))
("plateaued" nil (p l ae0 t ow1 d))
("plateauing" nil (p l ax t ow1 ih0 ng))
("plated" nil (p l ey1 t ax d))
("platek" nil (p l aa1 t eh0 k))
("platelets" nil (p l ey1 t l ax t s))
("platelike" nil (p l ey1 t l ay1 k))
("platen" nil (p l ae1 t ax n))
("plater" nil (p l ey1 t er0))
("plates" nil (p l ey1 t s))
("platform" nil (p l ae1 t f ao1 r m))
("platforms" nil (p l ae1 t f ao1 r m z))
("plath" nil (p l ae1 th))
("plating" nil (p l ey1 t ih0 ng))
("platinum" nil (p l ae1 t n ax m))
("platitudes" nil (p l ae1 t ih0 t uw1 d z))
("platner" nil (p l ae1 t n er0))
("plato" nil (p l ey1 t ow0))
("platona" nil (p l aa0 t ow1 n ax))
("platonic" nil (p l ax t aa1 n ih0 k))
("platonists" nil (p l ey1 t ax n ax s t s))
("platoon" nil (p l ax t uw1 n))
("platoons" nil (p l ax t uw1 n z))
("platt" nil (p l ae1 t))
("platte" nil (p l ae1 t))
("platten" nil (p l ae1 t ax n))
("platter" nil (p l ae1 t er0))
("platters" nil (p l ae1 t er0 z))
("plattner" nil (p l ae1 t n er0))
("platts" nil (p l ae1 t s))
("platy" nil (p l ey1 t iy0))
("platypus" nil (p l ae1 t ax p uh1 s))
("platz" nil (p l ae1 t s))
("platzer" nil (p l ey1 t z er0))
("plauche" nil (p l ao1 ch))
("plaudits" nil (p l ao1 d ih0 t s))
("plaugher" nil (p l ao1 er0))
("plausibility" nil (p l ao1 z ih0 b ih1 l ih0 t iy0))
("plausible" nil (p l ao1 z ax b ax l))
("plausibly" nil (p l ao1 z ax b l iy0))
("plaut" nil (p l ao1 t))
("plautz" nil (p l ao1 t s))
("plax" nil (p l ae1 k s))
("play" nil (p l ey1))
("playa" nil (p l ay1 ax))
("playback" nil (p l ey1 b ae1 k))
("playboy" nil (p l ey1 b oy1))
("playcount" nil (p l ey1 k aw1 n t))
("played" nil (p l ey1 d))
("player" nil (p l ey1 er0))
("players" nil (p l ey1 er0 z))
("playford" nil (p l ey1 f er0 d))
("playful" nil (p l ey1 f ax l))
("playfully" nil (p l ey1 f ax l iy0))
("playfulness" nil (p l ey1 f ax l n ax s))
("playground" nil (p l ey1 g r aw1 n d))
("playgrounds" nil (p l ey1 g r aw1 n d z))
("playhouse" nil (p l ey1 hh aw1 s))
("playing" nil (p l ey1 ih0 ng))
("playmate" nil (p l ey1 m ey1 t))
("playmates" nil (p l ey1 m ey1 t s))
("playoff" nil (p l ey1 ao1 f))
("playoffs" nil (p l ey1 ao1 f s))
("playpen" nil (p l ey1 p eh1 n))
("playroom" nil (p l ey1 r uw1 m))
("plays" nil (p l ey1 z))
("playstation" nil (p l ey1 s t ey1 sh ax n))
("playtex" nil (p l ey1 t eh1 k s))
("plaything" nil (p l ey1 th ih1 ng))
("playthings" nil (p l ey1 th ih1 ng z))
("playwright" nil (p l ey1 r ay1 t))
("playwrights" nil (p l ey1 r ay1 t s))
("plaza" nil (p l aa1 z ax))
("plazas" nil (p l aa1 z ax z))
("plea" nil (p l iy1))
("plead" nil (p l iy1 d))
("pleaded" nil (p l iy1 d ax d))
("pleading" nil (p l iy1 d ih0 ng))
("pleadings" nil (p l iy1 d ih0 ng z))
("pleads" nil (p l iy1 d z))
("pleas" nil (p l iy1 z))
("pleasant" nil (p l eh1 z ax n t))
("pleasantly" nil (p l eh1 z ax n t l iy0))
("pleasanton" nil (p l eh1 z ax n t ax n))
("pleasantries" nil (p l eh1 z ax n t r iy0 z))
("pleasants" nil (p l eh1 z ax n t s))
("pleasantville" nil (p l eh1 z ax n t v ih1 l))
("please" nil (p l iy1 z))
("pleased" nil (p l iy1 z d))
("pleaser" nil (p l iy1 z er0))
("pleases" nil (p l iy1 z ih0 z))
("pleasing" nil (p l iy1 z ih0 ng))
("pleasingly" nil (p l iy1 z ih0 ng l iy0))
("pleasurable" nil (p l eh1 zh er0 ax b ax l))
("pleasurama" nil (p l eh1 zh uh0 r ae1 m ax))
("pleasure" nil (p l eh1 zh er0))
("pleasures" nil (p l eh1 zh er0 z))
("pleats" nil (p l iy1 t s))
("plebeians" nil (p l ax b iy1 ax n z))
("plebiscite" nil (p l eh1 b ax s ay1 t))
("pled" nil (p l eh1 d))
("pledge" nil (p l eh1 jh))
("pledged" nil (p l eh1 jh d))
("pledger" nil (p l eh1 jh er0))
("pledges" nil (p l eh1 jh ih0 z))
("pledging" nil (p l eh1 jh ih0 ng))
("pleiad" nil (p l iy1 ax d))
("pleiades" nil (p l iy1 ax d iy1 z))
("pleiads" nil (p l iy1 ax d z))
("plein" nil (p l iy1 n))
("pleistocene" nil (p l ay1 s t ax s iy1 n))
("plemmons" nil (p l eh1 m ax n z))
("plemons" nil (p l eh1 m ax n z))
("plenary" nil (p l eh1 n er0 iy0))
("plenmeer" nil (p l eh0 n m ih1 r))
("plentiful" nil (p l eh1 n t ax f ax l))
("plenty" nil (p l eh1 n t iy0))
("plenum" nil (p l eh1 n ax m))
("plescia" nil (p l ey1 s iy0 ax))
("plese" nil (p l iy1 z))
("pleshar" nil (p l eh1 sh aa1 r))
("pleskow" nil (p l eh1 s k aw0))
("pless" nil (p l eh1 s))
("plessey" nil (p l eh1 s iy0))
("plessinger" nil (p l eh1 s ih0 ng er0))
("plessis" nil (p l eh1 s ih0 s))
("pletcher" nil (p l eh1 ch er0))
("plethora" nil (p l eh1 th er0 ax))
("pletscher" nil (p l eh1 ch er0))
("plett" nil (p l eh1 t))
("pletz" nil (p l eh1 t s))
("pleurisy" nil (p l uh1 r ax s iy0))
("pleva" nil (p l ey1 v ax))
("plew" nil (p l uw1))
("plewa" nil (p l uw1 ax))
("plexus" nil (p l eh1 k s ax s))
("pliable" nil (p l ay1 ax b ax l))
("pliant" nil (p l ay1 ax n t))
("plichta" nil (p l ih1 ch t ax))
("plied" nil (p l ay1 d))
("pliers" nil (p l ay1 er0 z))
("plies" nil (p l ay1 z))
("plight" nil (p l ay1 t))
("pliler" nil (p l ay1 l er0))
("plimpton" nil (p l ih1 m p t ax n))
("plinius" nil (p l ih1 n iy0 ax s))
("plink" nil (p l ih1 ng k))
("plinking" nil (p l ih1 ng k ih0 ng))
("plinth" nil (p l ih1 n th))
("pliny" nil (p l ih1 n iy0))
("pliocene" nil (p l ay1 ax s iy1 n))
("plisetskaya" nil (p l ih1 s ax t s k ay1 ax))
("pliska" nil (p l ih1 s k ax))
("plitt" nil (p l ih1 t))
("plocar" nil (p l ow1 k aa0 r))
("plocek" nil (p l ow1 ch eh1 k))
("ploch" nil (p l aa1 k))
("plocher" nil (p l aa1 k er0))
("plock" nil (p l aa1 k))
("plod" nil (p l aa1 d))
("plodded" nil (p l aa1 d ax d))
("plodding" nil (p l aa1 d ih0 ng))
("ploeger" nil (p l ow1 g er0))
("ploetz" nil (p l ow1 t s))
("plog" nil (p l aa1 g))
("plohn" nil (p l ow1 n))
("plonka" nil (p l ow1 ng k ax))
("plonski" nil (p l aa1 n s k iy0))
("ploof" nil (p l uw1 f))
("plop" nil (p l aa1 p))
("plopped" nil (p l aa1 p t))
("ploss" nil (p l ao1 s))
("plot" nil (p l aa1 t))
("plotkin" nil (p l aa1 t k ih0 n))
("plotner" nil (p l aa1 t n er0))
("plotnick" nil (p l aa1 t n ih0 k))
("plots" nil (p l aa1 t s))
("plott" nil (p l aa1 t))
("plotted" nil (p l aa1 t ax d))
("plotters" nil (p l aa1 t er0 z))
("plotting" nil (p l aa1 t ih0 ng))
("plotts" nil (p l aa1 t s))
("plotz" nil (p l aa1 t s))
("plouff" nil (p l ow1 f))
("plouffe" nil (p l ow1 f))
("plough" nil (p l aw1))
("ploughed" nil (p l aw1 d))
("plourde" nil (p l ao1 r d))
("plovers" nil (p l ah1 v er0 z))
("plow" nil (p l aw1))
("plowden" nil (p l aw1 d ax n))
("plowed" nil (p l aw1 d))
("plowing" nil (p l aw1 ih0 ng))
("plowman" nil (p l aw1 m ax n))
("plows" nil (p l aw1 z))
("ploy" nil (p l oy1))
("ploys" nil (p l oy1 z))
("plucinski" nil (p l ax ch ih1 n s k iy0))
("pluck" nil (p l ah1 k))
("plucked" nil (p l ah1 k t))
("plucker" nil (p l ah1 k er0))
("plucking" nil (p l ah1 k ih0 ng))
("plucks" nil (p l ah1 k s))
("plucky" nil (p l ah1 k iy0))
("plude" nil (p l uw1 d))
("plue" nil (p l uw1))
("pluff" nil (p l ah1 f))
("plug" nil (p l ah1 g))
("plugged" nil (p l ah1 g d))
("plugging" nil (p l ah1 g ih0 ng))
("plugs" nil (p l ah1 g z))
("plum" nil (p l ah1 m))
("plumage" nil (p l uw1 m ax jh))
("plumages" nil (p l uw1 m ax jh ax z))
("plumb" nil (p l ah1 m))
("plumbed" nil (p l ah1 m d))
("plumber" nil (p l ah1 m er0))
("plumbers" nil (p l ah1 m er0 z))
("plumbing" nil (p l ah1 m ih0 ng))
("plume" nil (p l uw1 m))
("plumer" nil (p l uw1 m er0))
("plumeri" nil (p l uw1 m eh1 r iy0))
("plumes" nil (p l uw1 m z))
("plumlee" nil (p l ah1 m l iy1))
("plumley" nil (p l ah1 m l iy0))
("plummer" nil (p l ah1 m er0))
("plummet" nil (p l ah1 m ax t))
("plummeted" nil (p l ah1 m ax t ih0 d))
("plummeting" nil (p l ah1 m ax t ih0 ng))
("plummets" nil (p l ah1 m ax t s))
("plump" nil (p l ah1 m p))
("plumped" nil (p l ah1 m p t))
("plums" nil (p l ah1 m z))
("plunder" nil (p l ah1 n d er0))
("plundered" nil (p l ah1 n d er0 d))
("plundering" nil (p l ah1 n d er0 ih0 ng))
("plunge" nil (p l ah1 n jh))
("plunged" nil (p l ah1 n jh d))
("plunger" nil (p l ah1 n jh er0))
("plunges" nil (p l ah1 n jh ih0 z))
("plunging" nil (p l ah1 n jh ih0 ng))
("plunk" nil (p l ah1 ng k))
("plunked" nil (p l ah1 ng k t))
("plunkers" nil (p l ah1 ng k er0 z))
("plunkett" nil (p l ah1 ng k ih0 t))
("plunking" nil (p l ah1 ng k ih0 ng))
("plunkitt" nil (p l ah1 ng k ih0 t))
("plural" nil (p l uh1 r ax l))
("pluralism" nil (p l uh1 r ax l ih1 z ax m))
("pluralistic" nil (p l uh1 r ax l ih1 s t ih0 k))
("plurality" nil (p l er0 ae1 l ih0 t iy0))
("plus" nil (p l ah1 s))
("pluses" nil (p l ah1 s ih0 z))
("plush" nil (p l ah1 sh))
("pluta" nil (p l uw1 t ax))
("pluth" nil (p l uw1 th))
("pluto" nil (p l uw1 t ow0))
("plutonian" nil (p l uw0 t ow1 n iy0 ax n))
("plutonic" nil (p l uw0 t aa1 n ih0 k))
("plutonium" nil (p l uw0 t ow1 n iy0 ax m))
("pluvial" nil (p l uw1 v iy0 ax l))
("ply" nil (p l ay1))
("plybon" nil (p l ih1 b ax n))
("plying" nil (p l ay1 ih0 ng))
("plyler" nil (p l ay1 l er0))
("plymale" nil (p l ay1 m ey1 l))
("plymouth" nil (p l ih1 m ax th))
("plympton" nil (p l ih1 m p t ax n))
("plywood" nil (p l ay1 w uh1 d))
("pm" nil (p iy1 eh1 m))
("pneumatic" nil (n uw0 m ae1 t ih0 k))
("pneumatics" nil (n uh0 m ae1 t ih0 k s))
("pneumo" nil (n uw1 m ow0))
("pneumocystis" nil (n uw0 m ow1 s ih0 s t ih0 s))
("pneumonia" nil (n uw0 m ow1 n y ax))
("pneumonic" nil (n uw0 m aa1 n ih0 k))
("po" nil (p ow1))
("poach" nil (p ow1 ch))
("poached" nil (p ow1 ch t))
("poachers" nil (p ow1 ch er0 z))
("poaching" nil (p ow1 ch ih0 ng))
("poag" nil (p ow1 g))
("poage" nil (p ow1 ih0 jh))
("poarch" nil (p ao1 r hh))
("pobanz" nil (p ow1 b aa0 n z))
("pobla" nil (p ow1 b l ax))
("pobst" nil (p aa1 b s t))
("pocahontas" nil (p ow1 k ax hh aa1 n t ax s))
("poch" nil (p aa1 k))
("poche" nil (p aa1 ch))
("pochiluk" nil (p ax ch iy1 l uw0 k))
("pociask" nil (p ax ch iy1 ax s k))
("pocius" nil (p ow1 s iy0 ih0 s))
("pocked" nil (p aa1 k t))
("pocket" nil (p aa1 k ax t))
("pocketbook" nil (p aa1 k ax t b uh1 k))
("pocketbooks" nil (p aa1 k ax t b uh1 k s))
("pocketed" nil (p aa1 k ax t ih0 d))
("pocketful" nil (p aa1 k ax t f uh1 l))
("pocketing" nil (p aa1 k ax t ih0 ng))
("pockets" nil (p aa1 k ax t s))
("pocklington" nil (p aa1 k l ih0 ng t ax n))
("pockmarked" nil (p aa1 k m aa1 r k t))
("poclain" nil (p aa1 k l ih0 n))
("poco" nil (p ow1 k ow0))
("pocock" nil (p aa1 k ax k))
("pocono" nil (p ow1 k ax n ow1))
("poconos" nil (p ow1 k ax n ow0 z))
("pocus" nil (p ow1 k ax s))
("pod" nil (p aa1 d))
("podany" nil (p ax d ao1 n iy0))
("podell" nil (p ow0 d ey1 l))
("podesta" nil (p ow0 d eh1 s t ax))
("podge" nil (p aa1 jh))
("podgorski" nil (p aa0 jh ao1 r s k iy0))
("podgurski" nil (p aa0 jh er1 s k iy0))
("podhoretz" nil (p aa1 d hh er0 eh0 t s))
("podium" nil (p ow1 d iy0 ax m))
("podiums" nil (p ow1 d iy0 ax m z))
("podleska" nil (p aa1 d l eh1 s k ax))
("podnar" nil (p aa1 d n aa1 r))
("podolak" nil (p ax d ow1 l ax k))
("podoll" nil (p aa1 d ax l))
("podolski" nil (p ax d ow1 l s k iy0))
("podolsky" nil (p ax d ow1 l s k iy0))
("podrasky" nil (p ax d r ae1 s k iy0))
("podraza" nil (p ow0 d r aa1 z ax))
("pods" nil (p aa1 d z))
("podunk" nil (p ow1 t ax ng k))
("poduska" nil (p ow0 d ah1 s k ax))
("poe" nil (p ow1))
("poehl" nil (p ow1 l))
("poehler" nil (p ow1 l er0))
("poehlman" nil (p ow1 l m ax n))
("poel" nil (p ow1 ax l))
("poelman" nil (p ow1 l m ax n))
("poem" nil (p ow1 ax m))
("poems" nil (p ow1 ax m z))
("poer" nil (p ow1 er0))
("poeschel" nil (p ow1 sh ax l))
("poeschl" nil (p ow1 s k ax l))
("poet" nil (p ow1 ax t))
("poetic" nil (p ow0 eh1 t ih0 k))
("poetry" nil (p ow1 ax t r iy0))
("poets" nil (p ow1 ax t s))
("pofahl" nil (p aa1 f aa0 l))
("poff" nil (p ao1 f))
("poffenbarger" nil (p aa1 f ih0 n b aa0 r g er0))
("poffenberger" nil (p ao1 f ax n b er0 g er0))
("poggi" nil (p aa1 jh iy0))
("poggioli" nil (p ow1 jh ow1 l iy0))
("pogo" nil (p ow1 g ow1))
("pogorzelski" nil (p ax g er0 z eh1 l s k iy0))
("pogroms" nil (p ax g r aa1 m z))
("pogue" nil (p ow1 g))
("poh" nil (p ow1))
("pohang" nil (p ow1 hh ae0 ng))
("pohjola" nil (p ow1 jh ow1 l ax))
("pohl" nil (p ow1 l))
("pohlad" nil (p ow1 l ae0 d))
("pohle" nil (p ow1 hh ax l))
("pohlman" nil (p ow1 l m ax n))
("pohlmann" nil (p ow1 l m ax n))
("pohnpei" nil (p ow1 n p ey1))
("poignancy" nil (p oy1 n y ax n s iy0))
("poignant" nil (p oy1 n y ax n t))
("poignantly" nil (p oy1 n y ax n t l iy0))
("poindexter" nil (p oy1 n d eh1 k s t er0))
("poinot" nil (p oy1 n aa1 t))
("poinsett" nil (p oy1 n s ih0 t))
("poinsettia" nil (p oy0 n s eh1 t iy0 ax))
("poinsettias" nil (p oy0 n s eh1 t iy0 ax z))
("point" nil (p oy1 n t))
("pointe" nil (p oy1 n t))
("pointed" nil (p oy1 n t ax d))
("pointedly" nil (p oy1 n t ih0 d l iy0))
("pointer" nil (p oy1 n t er0))
("pointers" nil (p oy1 n t er0 z))
("pointing" nil (p oy1 n t ih0 ng))
("pointless" nil (p oy1 n t l ax s))
("points" nil (p oy1 n t s))
("pointy" nil (p oy1 n t iy0))
("poirier" nil (p oy1 r iy0 er0))
("poirrier" nil (p oy1 r iy0 er0))
("poise" nil (p oy1 z))
("poised" nil (p oy1 z d))
("poison" nil (p oy1 z ax n))
("poisoned" nil (p oy1 z ax n d))
("poisoning" nil (p oy1 z ax n ih0 ng))
("poisonings" nil (p oy1 z ax n ih0 ng z))
("poisonous" nil (p oy1 z ax n ax s))
("poisons" nil (p oy1 z ax n z))
("poissant" nil (p oy0 z s ao1 n t))
("poisson" nil (p oy1 z s ax n))
("poitier" nil (p oy1 t y er0))
("poitra" nil (p oy1 t r ax))
("poitras" nil (p oy0 t r aa1 z))
("pokazukha" nil (p aa1 k ax z uw1 k hh ax))
("poke" nil (p ow1 k))
("poked" nil (p ow1 k t))
("poker" nil (p ow1 k er0))
("pokes" nil (p ow1 k s))
("pokeweed" nil (p ow1 k w iy1 d))
("pokey" nil (p ow1 k iy0))
("poking" nil (p ow1 k ih0 ng))
("pokorney" nil (p aa1 k er0 n iy0))
("pokorny" nil (p ax k ao1 r n iy0))
("pokorski" nil (p ax k ao1 r s k iy0))
("poky" nil (p ow1 k iy0))
("pol" nil (p ao1 l))
("polacek" nil (p aa1 l ax ch eh0 k))
("polachek" nil (p aa1 l ax k ih0 k))
("polack" nil (p ow1 l ae0 k))
("polak" nil (p ow1 l ax k))
("polakoff" nil (p aa1 l ax k ao0 f))
("polakowski" nil (p ax l ax k ao1 f s k iy0))
("polan" nil (p ow1 l ax n))
("polanco" nil (p ow0 l aa1 n k ow0))
("poland" nil (p ow1 l ax n d))
("polanski" nil (p ax l ae1 n s k iy0))
("polansky" nil (p ax l ae1 n s k iy0))
("polar" nil (p ow1 l er0))
("polarimeter" nil (p ow1 l er0 ih1 m ax t er0))
("polaris" nil (p ow0 l eh1 r ax s))
("polariscope" nil (p ow0 l eh1 r ax s k ow1 p))
("polarity" nil (p ow0 l eh1 r ax t iy0))
("polarization" nil (p ow1 l er0 ax z ey1 sh ax n))
("polarize" nil (p ow1 l er0 ay1 z))
("polarized" nil (p ow1 l er0 ay1 z d))
("polarizer" nil (p ow1 l er0 ay1 z er0))
("polarizing" nil (p ow1 l er0 ay1 z ih0 ng))
("polarography" nil (p ow1 l er0 aa1 g r ax f iy0))
("polaroid" nil (p ow1 l er0 oy1 d))
("polasek" nil (p ax l aa1 s eh0 k))
("polaski" nil (p ax l aa1 s k iy0))
("polasky" nil (p ax l aa1 s k iy0))
("polce" nil (p ow1 l s))
("polcyn" nil (p ow1 l s ih0 n))
("polder" nil (p ow1 l d er0))
("pole" nil (p ow1 l))
("polecat" nil (p ow1 l k ae1 t))
("polecats" nil (p ow1 l k ae1 t s))
("polek" nil (p ow1 l eh0 k))
("polemic" nil (p ax l eh1 m ih0 k))
("polemical" nil (p ax l eh1 m ax k ax l))
("polemicist" nil (p ax l eh1 m ax s ax s t))
("polemics" nil (p ow0 l eh1 m ih0 k s))
("polen" nil (p ow1 l ax n))
("poles" nil (p ow1 l z))
("poletti" nil (p ow0 l eh1 t iy0))
("polevanov" nil (p ax l eh1 v ax n aa0 v))
("polevoi" nil (p ow1 l ax v oy1))
("poleward" nil (p ow1 l w er0 d))
("poley" nil (p ow1 l iy0))
("polgar" nil (p ow1 l g er0))
("polhamus" nil (p ow1 l hh ax m ih0 s))
("polhemus" nil (p ow1 l hh ih0 m ax s))
("polhill" nil (p ow1 l hh ih1 l))
("poli" nil (p ow1 l iy0))
("policastro" nil (p ow0 l iy0 k ae1 s t r ow0))
("police" nil (p ax l iy1 s))
("policed" nil (p ax l iy1 s t))
("policeman" nil (p ax l iy1 s m ax n))
("policemen" nil (p ax l iy1 s m ih0 n))
("polices" nil (p ax l iy1 s ih0 z))
("policewoman" nil (p ax l iy1 s w uh1 m ax n))
("polich" nil (p aa1 l ih0 k))
("policies" nil (p aa1 l ax s iy0 z))
("policing" nil (p ax l iy1 s ih0 ng))
("policy" nil (p aa1 l ax s iy0))
("policyholder" nil (p aa1 l ax s iy0 hh ow1 l d er0))
("policyholders" nil (p aa1 l ax s iy0 hh ow1 l d er0 z))
("policymakers" nil (p aa1 l ax s iy0 m ey1 k er0 z))
("policymaking" nil (p aa1 l ax s iy0 m ey1 k ih0 ng))
("polidori" nil (p ow0 l iy0 d ao1 r iy0))
("polidoro" nil (p ow0 l iy0 d ao1 r ow0))
("polifroni" nil (p aa0 l ax f r ow1 n iy0))
("polimeni" nil (p ow0 l iy0 m eh1 n iy0))
("polin" nil (p ow1 l ih0 n))
("poling" nil (p ow1 l ih0 ng))
("polino" nil (p ow0 l iy1 n ow0))
("polinski" nil (p ax l ih1 n s k iy0))
("polinsky" nil (p ax l ih1 n s k iy0))
("polio" nil (p ow1 l iy0 ow1))
("poliquin" nil (p ow0 l iy0 k w iy1 n))
("polis" nil (p ow1 l ax s))
("polisario" nil (p ow1 l ih0 s eh1 r iy0 ow0))
("polish" nil (p aa1 l ih0 sh))
("polished" nil (p aa1 l ih0 sh t))
("polishing" nil (p aa1 l ih0 sh ih0 ng))
("politano" nil (p ow0 l iy0 t aa1 n ow0))
("politburo" nil (p aa1 l ax t b y uh1 r ow0))
("polite" nil (p ax l ay1 t))
("politely" nil (p ax l ay1 t l iy0))
("politeness" nil (p ax l ay1 t n ax s))
("polites" nil (p ax l ay1 t s))
("politi" nil (p ow0 l iy1 t iy0))
("politic" nil (p aa1 l ax t ih1 k))
("political" nil (p ax l ih1 t ax k ax l))
("politically" nil (p ax l ih1 t ih0 k ax l iy0))
("politician" nil (p aa1 l ax t ih1 sh ax n))
("politicians" nil (p aa1 l ax t ih1 sh ax n z))
("politicization" nil (p ax l ih1 t ax s ax z ey1 sh ax n))
("politicize" nil (p ax l ih1 t ih0 s ay1 z))
("politicized" nil (p ax l ih1 t ih0 s ay1 z d))
("politicizing" nil (p ax l ih1 t ih0 s ay1 z ih0 ng))
("politicking" nil (p aa1 l ax t ih1 k ih0 ng))
("politico" nil (p ax l ih1 t ih0 k ow1))
("politicos" nil (p ax l ih1 t ih0 k ow1 z))
("politics" nil (p aa1 l ax t ih1 k s))
("politis" nil (p aa1 l ay0 t ih0 s))
("polito" nil (p ow0 l iy1 t ow0))
("politte" nil (p ax l ih1 t))
("polity" nil (p aa1 l ax t iy0))
("politz" nil (p aa1 l ih0 t s))
("polivka" nil (p ow0 l iy1 v k ax))
("polizzi" nil (p ow0 l iy1 t s iy0))
("polk" nil (p ow1 k))
("polka" nil (p ow1 l k ax))
("polkas" nil (p ow1 l k ax z))
("polkinghorn" nil (p ow1 l k ih0 ng hh ao1 r n))
("poll" nil (p ow1 l))
("pollack" nil (p aa1 l ax k))
("pollak" nil (p aa1 l ax k))
("pollan" nil (p aa1 l ax n))
("polland" nil (p aa1 l ax n d))
("pollard" nil (p aa1 l er0 d))
("polled" nil (p ow1 l d))
("pollen" nil (p aa1 l ax n))
("pollet" nil (p aa1 l ih0 t))
("pollett" nil (p aa1 l ih0 t))
("polley" nil (p aa1 l iy0))
("pollick" nil (p aa1 l ih0 k))
("pollina" nil (p ow0 l iy1 n ax))
("pollinate" nil (p aa1 l ax n ey1 t))
("pollinated" nil (p aa1 l ih0 n ey1 t ih0 d))
("pollination" nil (p aa1 l ax n ey1 sh ax n))
("polling" nil (p ow1 l ih0 ng))
("pollinger" nil (p ow1 l ih0 ng er0))
("pollini" nil (p ax l iy1 n iy0))
("pollinia" nil (p aa0 l ih1 n iy0 ax))
("pollino" nil (p ow0 l iy1 n ow0))
("pollio" nil (p aa1 l iy0 ow0))
("pollitt" nil (p aa1 l ih0 t))
("pollman" nil (p ow1 l m ax n))
("pollner" nil (p ow1 l n er0))
("pollo" nil (p aa1 l ow0))
("pollock" nil (p aa1 l ax k))
("polloi" nil (p aa1 l oy1))
("pollok" nil (p aa1 l ax k))
("polls" nil (p ow1 l z))
("pollster" nil (p ow1 l s t er0))
("pollsters" nil (p ow1 l s t er0 z))
("pollutant" nil (p ax l uw1 t ax n t))
("pollutants" nil (p ax l uw1 t ax n t s))
("pollute" nil (p ax l uw1 t))
("polluted" nil (p ax l uw1 t ax d))
("polluter" nil (p ax l uw1 t er0))
("polluters" nil (p ax l uw1 t er0 z))
("polluting" nil (p ax l uw1 t ih0 ng))
("pollution" nil (p ax l uw1 sh ax n))
("pollux" nil (p aa1 l ax k s))
("polly" nil (p aa1 l iy0))
("pollyanna" nil (p aa1 l iy0 ae1 n ax))
("polo" nil (p ow1 l ow0))
("polonaise" nil (p aa1 l ax n ey1 z))
("polonium" nil (p ax l ow1 n iy0 ax m))
("polonsky" nil (p ax l aa1 n s k iy0))
("pols" nil (p ow1 l z))
("polsby" nil (p ow1 l z b iy0))
("polsinelli" nil (p ow0 l s iy0 n eh1 l iy0))
("polsky" nil (p ow1 l s k iy0))
("polson" nil (p ow1 l s ax n))
("polster" nil (p ow1 l s t er0))
("polston" nil (p ow1 l s t ax n))
("poltergeists" nil (p ow1 l t er0 g ay1 s t s))
("poltrack" nil (p ow1 l t r ae1 k))
("polus" nil (p ow1 l ax s))
("poly" nil (p aa1 l iy0))
("polyacetylene" nil (p aa1 l iy0 ax s eh1 t ax l iy1 n))
("polyak" nil (p aa1 l iy0 ae0 k))
("polyamide" nil (p aa1 l iy0 ae1 m ay1 d))
("polyandrous" nil (p aa1 l iy0 ae1 n d r ax s))
("polyandry" nil (p aa1 l iy0 ae1 n d r iy0))
("polybutylene" nil (p aa1 l iy0 b y uw1 t ax l iy1 n))
("polycarpic" nil (p aa1 l ih0 k aa1 r p ih0 k))
("polycast" nil (p aa1 l iy0 k ae1 s t))
("polychlorinated" nil (p aa1 l iy0 k l ao1 r ih0 n ey1 t ih0 d))
("polychrome" nil (p aa1 l ih0 k r ow1 m))
("polyconomics" nil (p aa1 l ih0 k ax n aa1 m ih0 k s))
("polyester" nil (p aa1 l iy0 eh1 s t er0))
("polyesters" nil (p aa1 l iy0 eh1 s t er0 z))
("polyethylene" nil (p aa1 l iy0 eh1 th ax l iy1 n))
("polygamous" nil (p ax l ih1 g ax m ax s))
("polygamy" nil (p ax l ih1 g ax m iy0))
("polyglot" nil (p aa1 l iy0 g l aa0 t))
("polygon" nil (p aa1 l ih0 g aa1 n))
("polygonal" nil (p ax l ih1 g ax n ax l))
("polygram" nil (p aa1 l iy0 g r ae1 m))
("polygraph" nil (p aa1 l ih0 g r ae1 f))
("polygraphs" nil (p aa1 l iy0 g r ae1 f s))
("polygynous" nil (p ax l ih1 jh ax n ax s))
("polygyny" nil (p ax l ih1 jh ax n iy0))
("polyhedron" nil (p aa1 l ih0 hh iy1 d r ax n))
("polyhedrons" nil (p aa1 l ih0 hh iy1 d r ax n z))
("polyhemoglobin" nil (p aa1 l ih0 hh ax m aa1 g l ax b ih0 n))
("polymer" nil (p aa1 l ax m er0))
("polymerase" nil (p aa1 l ih0 m er0 ey1 s))
("polymerized" nil (p aa1 l ih0 m er0 ay1 z d))
("polymerizes" nil (p aa1 l ih0 m er0 ay1 z ax z))
("polymers" nil (p aa1 l ih0 m er0 z))
("polynesia" nil (p aa1 l ih0 n iy1 zh ax))
("polynesian" nil (p aa1 l ih0 n iy1 zh ax n))
("polynomial" nil (p aa1 l ih0 n ow1 m iy0 ax l))
("polyphase" nil (p aa1 l ih0 f ey1 z))
("polyphonic" nil (p aa1 l ih0 f aa1 n ih0 k))
("polyphony" nil (p ax l ih1 f ax n iy0))
("polypropylene" nil (p aa1 l iy0 p r ow1 p ax l iy1 n))
("polyps" nil (p aa1 l ih0 p s))
("polysaccharide" nil (p aa1 l ih0 s ae1 k er0 ay1 d))
("polysaccharides" nil (p aa1 l ih0 s ae1 k er0 ay1 d z))
("polysar" nil (p aa1 l ih0 s aa0 r))
("polysilicon" nil (p aa1 l iy0 s ih1 l ih0 k aa1 n))
("polystyrene" nil (p aa1 l ih0 s t ay1 r iy1 n))
("polytechnic" nil (p aa1 l ih0 t eh1 k n ih0 k))
("polytheism" nil (p aa1 l ih0 th iy0 ih0 z ax m))
("polytheistic" nil (p aa1 l ih0 th iy0 ih1 s t ih0 k))
("polyurethane" nil (p aa1 l iy0 uh1 r ax th ey1 n))
("polyvinyl" nil (p aa1 l iy0 v ay1 n ax l))
("polzer" nil (p ow1 l z er0))
("polzin" nil (p ow1 l z ih0 n))
("poma" nil (p ow1 m ax))
("pombo" nil (p aa1 m b ow0))
("pomerance" nil (p aa1 m er0 ae1 n s))
("pomerania" nil (p aa1 m er0 ey1 n iy0 ax))
("pomeranian" nil (p aa1 m er0 ey1 n iy0 ax n))
("pomerantz" nil (p aa1 m er0 ae1 n t s))
("pomeranz" nil (p aa1 m er0 ae1 n s))
("pomerleau" nil (p aa1 m er0 l ow0))
("pomeroy" nil (p aa1 m er0 oy1))
("pomicino" nil (p ow1 m ih0 s iy1 n ow0))
("pommel" nil (p aa1 m ax l))
("pommer" nil (p aa1 m er0))
("pommier" nil (p aa1 m iy0 er0))
("pomodoro" nil (p aa1 m ax d ao1 r ow0))
("pomology" nil (p ow0 m aa1 l ax jh iy0))
("pomona" nil (p ow0 m ow1 n ax))
("pomp" nil (p aa1 m p))
("pompa" nil (p aa1 m p ax))
("pompadur" nil (p aa1 m p ax d er0))
("pompano" nil (p aa1 m p ax n ow1))
("pompeo" nil (p ow1 m p iy0 ow0))
("pomper" nil (p aa1 m p er0))
("pompey" nil (p aa1 m p iy0))
("pompidou" nil (p aa1 m p ih0 d uw1))
("pompilio" nil (p ow0 m p iy1 l iy0 ow0))
("pomplun" nil (p aa1 m p l ax n))
("pompon" nil (p aa1 m p aa1 n))
("pomponio" nil (p ow0 m p ow1 n iy0 ow0))
("pompons" nil (p aa1 m p aa1 n z))
("pomposity" nil (p aa1 m p aa1 s ih0 t iy0))
("pompous" nil (p aa1 m p ax s))
("pompousness" nil (p aa1 m p ax s n ax s))
("pomrenze" nil (p aa1 m r ax n z))
("pomroy" nil (p aa1 m r oy1))
("pon" nil (p aa1 n))
("ponce" nil (p ow1 n s ey0))
("poncet" nil (p aa1 n s ax t))
("pond" nil (p aa1 n d))
("ponder" nil (p aa1 n d er0))
("pondered" nil (p aa1 n d er0 d))
("pondering" nil (p aa1 n d er0 ih0 ng))
("ponderosa" nil (p aa1 n d er0 ow1 s ax))
("ponderous" nil (p aa1 n d er0 ax s))
("ponders" nil (p aa1 n d er0 z))
("ponds" nil (p aa1 n d z))
("pong" nil (p ao1 ng))
("pongratz" nil (p aa1 ng g r ax t s))
("poniatowski" nil (p ax n iy0 ax t ao1 f s k iy0))
("ponied" nil (p ow1 n iy0 d))
("ponies" nil (p ow1 n iy0 z))
("pons" nil (p aa1 n z))
("ponsolle" nil (p aa1 n s ow0 l))
("pont" nil (p aa1 n t))
("pontarelli" nil (p ow0 n t aa0 r eh1 l iy0))
("pontbriand" nil (p aa1 n t b r iy0 ax n d))
("ponte" nil (p aa1 n t))
("pontes" nil (p ow1 n t eh0 s))
("ponti" nil (p aa1 n t iy0))
("pontiac" nil (p aa1 n t iy0 ae1 k))
("pontiacs" nil (p aa1 n t iy0 ae1 k s))
("pontiff" nil (p aa1 n t ax f))
("pontifical" nil (p aa0 n t ih1 f ax k ax l))
("pontificate" nil (p aa0 n t ih1 f ax k ey1 t))
("pontificated" nil (p aa0 n t ih1 f ax k ey1 t ih0 d))
("pontificater" nil (p aa0 n t ih1 f ax k ey1 t er0))
("pontificates" nil (p aa0 n t ih1 f ax k ey1 t s))
("pontificating" nil (p aa0 n t ih1 f ax k ey1 t ih0 ng))
("pontification" nil (p aa0 n t ih1 f ax k ey1 sh ax n))
("pontifications" nil (p aa0 n t ih1 f ax k ey1 sh ax n z))
("pontikes" nil (p aa1 n t iy1 k eh1 z))
("pontillo" nil (p ow0 n t ih1 l ow0))
("pontious" nil (p ow1 n sh ih0 s))
("pontius" nil (p aa1 n t iy0 ih0 s))
("ponto" nil (p aa1 n t ow0))
("ponton" nil (p aa1 n t ax n))
("pontoon" nil (p aa0 n t uw1 n))
("pontoons" nil (p aa1 n t uw1 n z))
("ponts" nil (p aa1 n t s))
("pony" nil (p ow1 n iy0))
("ponzetti" nil (p aa0 n z eh1 t iy0))
("ponzi" nil (p aa1 n z iy0))
("ponzio" nil (p aa1 n z iy0 ow0))
("ponzo" nil (p aa1 n z ow0))
("poo" nil (p uw1))
("pooch" nil (p uw1 ch))
("pooches" nil (p uw1 ch ih0 z))
("poodle" nil (p uw1 d ax l))
("poodles" nil (p uw1 d ax l z))
("poof" nil (p uw1 f))
("pooh" nil (p uw1))
("poohed" nil (p uw1 d))
("pool" nil (p uw1 l))
("poole" nil (p uw1 l))
("pooled" nil (p uw1 l d))
("pooler" nil (p uw1 l er0))
("pooley" nil (p uw1 l iy0))
("pooling" nil (p uw1 l ih0 ng))
("pools" nil (p uw1 l z))
("poolside" nil (p uw1 l s ay1 d))
("poon" nil (p uw1 n))
("poop" nil (p uw1 p))
("pooped" nil (p uw1 p t))
("poops" nil (p uw1 p s))
("poor" nil (p uh1 r))
("poorbaugh" nil (p uh1 r b ao0))
("poore" nil (p uh1 r))
("poorer" nil (p uh1 r er0))
("poorest" nil (p uh1 r ih0 s t))
("poorhouse" nil (p uh1 r hh aw1 s))
("poorly" nil (p uh1 r l iy0))
("poorman" nil (p uh1 r m ax n))
("poors" nil (p uh1 r z))
("pooser" nil (p uw1 z er0))
("poovey" nil (p uw1 v iy0))
("pop" nil (p aa1 p))
("popa" nil (p ow1 p ax))
("popcorn" nil (p aa1 p k ao1 r n))
("pope" nil (p ow1 p))
("popeil" nil (p ow1 p iy1 l))
("popejoy" nil (p ow1 p jh oy1))
("popek" nil (p ow1 p ih0 k))
("popelka" nil (p ax p eh1 l k ax))
("popeye" nil (p aa1 p ay1))
("popeyes" nil (p ao1 p ay1 z))
("popham" nil (p aa1 f ax m))
("popick" nil (p aa1 p ih0 k))
("popiel" nil (p aa1 p iy0 l))
("popieluszko" nil (p ow0 p iy1 eh0 l ah1 s k ow0))
("popik" nil (p ow1 p ih0 k))
("popken" nil (p aa1 p k ax n))
("popkin" nil (p aa1 p k ih0 n))
("popko" nil (p ow1 p k ow0))
("poplar" nil (p aa1 p l er0))
("poplawski" nil (p ax p l aa1 f s k iy0))
("poplin" nil (p aa1 p l ih0 n))
("popoff" nil (p aa1 p ao1 f))
("popolare" nil (p aa1 p ow0 l aa1 r iy0))
("popov" nil (p ow1 p ax v))
("popovic" nil (p aa1 p ax v ih0 k))
("popovich" nil (p aa1 p ax v ih0 ch))
("popowski" nil (p ax p ao1 f s k iy0))
("popp" nil (p aa1 p))
("poppa" nil (p aa1 p ax))
("poppe" nil (p aa1 p))
("poppea" nil (p aa1 p iy0 ax))
("popped" nil (p aa1 p t))
("poppell" nil (p aa1 p ax l))
("poppen" nil (p aa1 p ax n))
("popper" nil (p aa1 p er0))
("poppers" nil (p aa1 p er0 z))
("poppies" nil (p aa1 p iy0 z))
("popping" nil (p aa1 p ih0 ng))
("popple" nil (p aa1 p ax l))
("poppleton" nil (p aa1 p l ih0 t aa0 n))
("popplewell" nil (p aa1 p l ih0 w eh0 l))
("poppy" nil (p aa1 p iy0))
("pops" nil (p aa1 p s))
("popsicle" nil (p aa1 p s ih0 k ax l))
("populace" nil (p aa1 p y ax l ax s))
("popular" nil (p aa1 p y ax l er0))
("popularity" nil (p aa1 p y ax l eh1 r ax t iy0))
("popularization" nil (p aa1 p y ax l er0 ax z ey1 sh ax n))
("popularize" nil (p aa1 p y ax l er0 ay1 z))
("popularized" nil (p aa1 p y ax l er0 ay1 z d))
("popularizer" nil (p aa1 p y ax l er0 ay1 z er0))
("popularizing" nil (p aa1 p y ax l er0 ay1 z ih0 ng))
("popularly" nil (p aa1 p y ax l er0 l iy0))
("populate" nil (p aa1 p y ax l ey1 t))
("populated" nil (p aa1 p y ax l ey1 t ax d))
("populating" nil (p aa1 p y ax l ey1 t ih0 ng))
("population" nil (p aa1 p y ax l ey1 sh ax n))
("populations" nil (p aa1 p y ax l ey1 sh ax n z))
("populism" nil (p aa1 p y ax l ih1 z ax m))
("populist" nil (p aa1 p y ax l ax s t))
("populists" nil (p aa1 p y ax l ih0 s t s))
("populous" nil (p aa1 p y ax l ax s))
("popwell" nil (p aa1 p w eh1 l))
("poquette" nil (p ax k eh1 t))
("por" nil (p ao1 r))
("porada" nil (p ao0 r aa1 d ax))
("porath" nil (p ao1 r ax th))
("porcaro" nil (p ao0 r k aa1 r ow0))
("porcelain" nil (p ao1 r s ax l ax n))
("porcelains" nil (p ao1 r s ax l ax n z))
("porcella" nil (p ao0 r ch eh1 l ax))
("porcelli" nil (p ao0 r ch eh1 l iy0))
("porcello" nil (p ao0 r ch eh1 l ow0))
("porch" nil (p ao1 r ch))
("porche" nil (p ao1 r ch))
("porcher" nil (p ao1 r ch er0))
("porches" nil (p ao1 r ch ax z))
("porchia" nil (p ao1 r k iy0 ax))
("porco" nil (p ao1 r k ow0))
("porcupine" nil (p ao1 r k y ax p ay1 n))
("porcupines" nil (p ao1 r k y ax p ay1 n z))
("pordy" nil (p ao1 r d iy0))
("pore" nil (p ao1 r))
("pored" nil (p ao1 r d))
("poremba" nil (p ao0 r eh1 m b ax))
("pores" nil (p ao1 r z))
("porges" nil (p ao1 r jh ih0 z))
("porgy" nil (p ao1 r g iy0))
("poring" nil (p ao1 r ih0 ng))
("pork" nil (p ao1 r k))
("porky" nil (p ao1 r k iy0))
("porn" nil (p ao1 r n))
("porno" nil (p ao1 r n ow0))
("pornographic" nil (p ao1 r n ax g r ae1 f ih0 k))
("pornography" nil (p ao0 r n aa1 g r ax f iy0))
("porous" nil (p ao1 r ax s))
("porphyritic" nil (p ao1 r f er0 ih1 t ih0 k))
("porphyry" nil (p ao1 r f er0 iy0))
("porpoises" nil (p ao1 r p ax s ax z))
("porr" nil (p ao1 r))
("porras" nil (p ao1 r aa0 z))
("porrazzo" nil (p ao0 r aa1 z ow0))
("porreca" nil (p ao0 r eh1 k ax))
("porretta" nil (p ao0 r eh1 t ax))
("porridge" nil (p ao1 r ax jh))
("porritt" nil (p ao1 r ih0 t))
("porro" nil (p ao1 r ow0))
("porsche" nil (p ao1 r sh ax))
("porsches" nil (p ao1 r sh ih0 z))
("port" nil (p ao1 r t))
("porta" nil (p ao1 r t ax))
("portability" nil (p ao1 r t ax b ih1 l ih0 t iy0))
("portable" nil (p ao1 r t ax b ax l))
("portables" nil (p ao1 r t ax b ax l z))
("portage" nil (p ao1 r t ax jh))
("portal" nil (p ao1 r t ax l))
("portales" nil (p ao0 r t aa1 l eh0 s))
("portals" nil (p ao1 r t ax l z))
("portanova" nil (p ao0 r t aa0 n ow1 v ax))
("porte" nil (p ao1 r t))
("portec" nil (p ao1 r t eh1 k))
("portee" nil (p ao1 r t iy1))
("portela" nil (p ao0 r t eh1 l ax))
("portell" nil (p ao0 r t ey1 l))
("portelli" nil (p ao0 r t eh1 l iy0))
("portend" nil (p ao0 r t eh1 n d))
("portending" nil (p ao0 r t eh1 n d ih0 ng))
("portends" nil (p ao0 r t eh1 n d z))
("portent" nil (p ao1 r t eh0 n t))
("portentous" nil (p ao0 r t eh1 n t ax s))
("portents" nil (p ao1 r t eh1 n t s))
("porteous" nil (p ao1 r t iy0 ih0 s))
("porter" nil (p ao1 r t er0))
("portera" nil (p ao0 r t eh1 r ax))
("porterages" nil (p ao1 r t er0 ih0 jh ih0 z))
("porterfield" nil (p ao1 r t er0 f iy1 l d))
("portfolio" nil (p ao0 r t f ow1 l iy0 ow1))
("portfolios" nil (p ao0 r t f ow1 l iy0 ow1 z))
("porth" nil (p ao1 r th))
("portia" nil (p ao1 r sh ax))
("portico" nil (p ao1 r t ax k ow1))
("portier" nil (p ao1 r t iy0 er0))
("portilla" nil (p ao1 r t ih1 l ax))
("portillo" nil (p ao1 r t ih1 l ow0))
("porting" nil (p ao1 r t ih0 ng))
("portion" nil (p ao1 r sh ax n))
("portions" nil (p ao1 r sh ax n z))
("portis" nil (p ao1 r t ih0 s))
("portland" nil (p ao1 r t l ax n d))
("portlock" nil (p ao1 r t l aa1 k))
("portly" nil (p ao1 r t l iy0))
("portman" nil (p ao1 r t m ax n))
("portner" nil (p ao1 r t n er0))
("portney" nil (p ao1 r t n iy0))
("portnoy" nil (p ao1 r t n oy0))
("porto" nil (p ao1 r t ow0))
("portrait" nil (p ao1 r t r ax t))
("portraits" nil (p ao1 r t r ax t s))
("portray" nil (p ao0 r t r ey1))
("portrayal" nil (p ao0 r t r ey1 ax l))
("portrayals" nil (p ao0 r t r ey1 ax l z))
("portrayed" nil (p ao0 r t r ey1 d))
("portraying" nil (p ao0 r t r ey1 ih0 ng))
("portrays" nil (p ao0 r t r ey1 z))
("ports" nil (p ao1 r t s))
("portsmouth" nil (p ao1 r t s m ax th))
("portugal" nil (p ao1 r ch ax g ax l))
("portugalia" nil (p ao1 r ch uw1 g ey1 l iy0 ax))
("portuguese" nil (p ao1 r ch ax g iy1 z))
("portwood" nil (p ao1 r t w uh1 d))
("portz" nil (p ao1 r t s))
("porzio" nil (p ao1 r z iy0 ow0))
("posa" nil (p ow1 s ax))
("posada" nil (p ow0 s aa1 d ax))
("posch" nil (p ao1 sh))
("posco" nil (p ao1 s k ow0))
("pose" nil (p ow1 z))
("posed" nil (p ow1 z d))
("poseidon" nil (p ax s ay1 d ax n))
("posen" nil (p ow1 z ax n))
("poser" nil (p ow1 z er0))
("poses" nil (p ow1 z ax z))
("posey" nil (p ow1 z iy0))
("posh" nil (p aa1 sh))
("posing" nil (p ow1 z ih0 ng))
("posit" nil (p aa1 z ax t))
("posited" nil (p aa1 z ax t ax d))
("position" nil (p ax z ih1 sh ax n))
("positioned" nil (p ax z ih1 sh ax n d))
("positioning" nil (p ax z ih1 sh ax n ih0 ng))
("positions" nil (p ax z ih1 sh ax n z))
("positive" nil (p aa1 z ax t ih0 v))
("positively" nil (p aa1 z ax t ih0 v l iy0))
("positives" nil (p aa1 z ax t ih0 v z))
("positivism" nil (p aa1 z ax t ih0 v ih1 z ax m))
("positron" nil (p aa1 z ax t r aa1 n))
("positrons" nil (p aa1 z ax t r aa1 n z))
("posits" nil (p aa1 z ax t s))
("posluns" nil (p ao1 z l ax n z))
("posluszny" nil (p ax s l ah1 sh n iy0))
("posner" nil (p ow1 z n er0))
("posners" nil (p ow1 z n er0 z))
("posnick" nil (p ao1 s n ih0 k))
("pospisil" nil (p ow0 s p iy0 s iy1 l))
("poss" nil (p ao1 s))
("posse" nil (p aa1 s iy0))
("possehl" nil (p aa1 s ax l))
("possess" nil (p ax z eh1 s))
("possessed" nil (p ax z eh1 s t))
("possesses" nil (p ax z eh1 s ax z))
("possessing" nil (p ax z eh1 s ih0 ng))
("possession" nil (p ax z eh1 sh ax n))
("possessions" nil (p ax z eh1 sh ax n z))
("possessive" nil (p ax z eh1 s ih0 v))
("possessor" nil (p ax z eh1 s er0))
("possibilities" nil (p aa1 s ax b ih1 l ax t iy0 z))
("possibility" nil (p aa1 s ax b ih1 l ax t iy0))
("possible" nil (p aa1 s ax b ax l))
("possiblity" nil (p aa1 s ax b ih1 l ax t iy0))
("possibly" nil (p aa1 s ax b l iy0))
("possum" nil (p aa1 s ax m))
("possums" nil (p aa1 s ax m z))
("post" nil (p ow1 s t))
("postage" nil (p ow1 s t ax jh))
("postal" nil (p ow1 s t ax l))
("postcard" nil (p ow1 s t k aa1 r d))
("postcards" nil (p ow1 s t k aa1 r d z))
("postcrash" nil (p ow1 s t k r ae1 sh))
("postdate" nil (p ow1 s t d ey1 t))
("postdoctoral" nil (p ow1 s t d aa1 k t er0 ax l))
("posted" nil (p ow1 s t ih0 d))
("postel" nil (p aa1 s t ax l))
("postell" nil (p aa1 s t ax l))
("postema" nil (p aa0 s t eh1 m ax))
("posten" nil (p ow1 s t ax n))
("poster" nil (p ow1 s t er0))
("posteraro" nil (p ao1 s t eh0 r aa1 r ow0))
("posterity" nil (p aa0 s t eh1 r ax t iy0))
("posters" nil (p ow1 s t er0 z))
("postholes" nil (p ow1 s t hh ow1 l z))
("posthumous" nil (p aa1 s ch uh0 m ax s))
("posthumously" nil (p aa1 s ch uh0 m ax s l iy0))
("posting" nil (p ow1 s t ih0 ng))
("postings" nil (p ow1 s t ih0 ng z))
("postipankki" nil (p ao1 s t ih0 p ae1 ng k iy0))
("postle" nil (p aa1 s ax l))
("postlethwait" nil (p ow1 s t l ih0 th w ey0 t))
("postlewait" nil (p ow1 s t l uw0 ey0 t))
("postlewaite" nil (p ow1 s t l uw0 ey0 t))
("postma" nil (p ow1 s t m ax))
("postman" nil (p ow1 s t m ax n))
("postmarked" nil (p ow1 s t m aa1 r k t))
("postmaster" nil (p ow1 s t m ae1 s t er0))
("postmasters" nil (p ow1 s t m ae1 s t er0 z))
("postmodern" nil (p ow1 s t m aa1 d er0 n))
("postnatal" nil (p ow1 s t n ey1 t ax l))
("poston" nil (p ow1 s t ax n))
("postpone" nil (p ow0 s t p ow1 n))
("postponed" nil (p ow0 s t p ow1 n d))
("postponement" nil (p ow0 s t p ow1 n m ax n t))
("postponements" nil (p ow0 s t p ow1 n m ax n t s))
("postpones" nil (p ow0 s t p ow1 n z))
("postponing" nil (p ow0 s t p ow1 n ih0 ng))
("postrelle" nil (p ow1 s t r eh1 l))
("postretirement" nil (p ow1 s t r iy0 t ay1 er0 m ax n t))
("posts" nil (p ow1 s t s))
("postscript" nil (p ow1 s k r ih1 p t))
("postscripts" nil (p ow1 s k r ih1 p t s))
("postulate" nil (p aa1 s ch ax l ey1 t))
("postulates" nil (p aa1 s ch ax l ey1 t s))
("posture" nil (p aa1 s ch er0))
("postures" nil (p aa1 s ch er0 z))
("posturing" nil (p aa1 s ch er0 ih0 ng))
("postwar" nil (p ow1 s t w ao1 r))
("pot" nil (p aa1 t))
("potable" nil (p ow1 t ax b ax l))
("potamkin" nil (p ow1 t ae1 m k ih0 n))
("potapov" nil (p aa1 t ax p aa1 v))
("potash" nil (p aa1 t ae1 sh))
("potassium" nil (p ax t ae1 s iy0 ax m))
("potato" nil (p ax t ey1 t ow1))
("potatoes" nil (p ax t ey1 t ow0 z))
("pote" nil (p ow1 t))
("poteat" nil (p ow0 t iy1 t))
("poteet" nil (p aa1 t iy0 t))
("poteete" nil (p aa1 t iy0 t))
("potemkin" nil (p ax t eh1 m k ih0 n))
("potempa" nil (p ow0 t eh1 m p ax))
("potency" nil (p ow1 t ax n s iy0))
("potent" nil (p ow1 t ax n t))
("potentates" nil (p ow1 t ax n t ey1 t s))
("potential" nil (p ax t eh1 n sh ax l))
("potentially" nil (p ax t eh1 n sh ax l iy0))
("potentials" nil (p ax t eh1 n ch ax l z))
("potenza" nil (p ow0 t eh1 n z ax))
("poth" nil (p aa1 th))
("pothier" nil (p ow1 th iy0 er0))
("pothitos" nil (p ax th iy1 t ow0 s))
("pothole" nil (p aa1 t hh ow1 l))
("potholes" nil (p aa1 t hh ow1 l z))
("potier" nil (p ow1 t iy0 er0))
("potiker" nil (p ow1 t ih0 k er0))
("potion" nil (p ow1 sh ax n))
("potions" nil (p ow1 sh ax n z))
("potlatch" nil (p aa1 t l ae1 ch))
("potluck" nil (p aa1 t l ah1 k))
("potocki" nil (p ax t ow1 t s k iy0))
("potomac" nil (p ax t ow1 m ax k))
("potpie" nil (p aa1 t p ay1))
("potpourri" nil (p ow1 p uh0 r iy1))
("potratz" nil (p aa1 t r ax t s))
("pots" nil (p aa1 t s))
("potsdam" nil (p aa1 t s d ae1 m))
("potshots" nil (p aa1 ch aa1 t s))
("pott" nil (p aa1 t))
("pottebaum" nil (p aa1 t b aw0 m))
("potted" nil (p aa1 t ih0 d))
("potteiger" nil (p aa1 t ay0 g er0))
("pottenger" nil (p aa1 t ih0 n jh er0))
("potter" nil (p aa1 t er0))
("potters" nil (p aa1 t er0 z))
("pottery" nil (p aa1 t er0 iy0))
("potthast" nil (p aa1 th ax s t))
("potthoff" nil (p aa1 t hh ao1 f))
("pottinger" nil (p aa1 t ih0 ng er0))
("pottle" nil (p aa1 t ax l))
("pottorff" nil (p aa1 t er0 f))
("potts" nil (p aa1 t s))
("potty" nil (p aa1 t iy0))
("potucek" nil (p aa1 t ax ch eh0 k))
("potvin" nil (p aa1 t v ih0 n))
("pou" nil (p uw1))
("pouch" nil (p aw1 ch))
("poucher" nil (p aw1 ch er0))
("pouches" nil (p aw1 ch ax z))
("poudrier" nil (p aw1 d er0 iy0 er0))
("pough" nil (p aw1))
("poughkeepsie" nil (p ax k ih1 p s iy0))
("poul" nil (p uw1 l))
("poulenc" nil (p uw1 l ax ng k))
("poulin" nil (p uw0 l ae1 n))
("pouliot" nil (p uw1 l iy0 ow0))
("poulos" nil (p ax l iy1 s))
("poulsen" nil (p aw1 l s ax n))
("poulson" nil (p aw1 l s ax n))
("poulter" nil (p ow1 l t er0))
("poultices" nil (p ow1 l t ax s ax z))
("poulton" nil (p aw1 l t ax n))
("poultry" nil (p ow1 l t r iy0))
("pounce" nil (p aw1 n s))
("pounced" nil (p aw1 n s t))
("pouncey" nil (p aw1 n s iy0))
("pouncing" nil (p aw1 n s ih0 ng))
("pouncy" nil (p uw0 ng k iy1))
("pound" nil (p aw1 n d))
("poundage" nil (p aw1 n d ih0 jh))
("pounded" nil (p aw1 n d ih0 d))
("pounder" nil (p aw1 n d er0))
("pounders" nil (p aw1 n d er0 z))
("pounding" nil (p aw1 n d ih0 ng))
("pounds" nil (p aw1 n d z))
("poundstone" nil (p aw1 n d s t ow1 n))
("poupard" nil (p uw1 p er0 d))
("pour" nil (p ao1 r))
("pourciau" nil (p ao1 r s iy0 ow0))
("poured" nil (p ao1 r d))
("pouring" nil (p ao1 r ih0 ng))
("pours" nil (p ao1 r z))
("poussaint" nil (p uw1 s aa1 n t))
("poussin" nil (p uw1 s ih0 n))
("pousson" nil (p uw1 s aa1 n))
("poust" nil (p aw1 s t))
("pout" nil (p aw1 t))
("pouted" nil (p aw1 t ih0 d))
("pouter" nil (p aw1 t er0))
("pouting" nil (p aw1 t ih0 ng))
("poutre" nil (p aw1 t er0))
("pouts" nil (p aw1 t s))
("pouty" nil (p aw1 t iy0))
("poveromo" nil (p ow1 v eh0 r ow1 m ow0))
("poverty" nil (p aa1 v er0 t iy0))
("povich" nil (p ow1 v ih0 ch))
("pow" nil (p aw1))
("powder" nil (p aw1 d er0))
("powdered" nil (p aw1 d er0 d))
("powderly" nil (p aw1 d er0 l iy0))
("powders" nil (p aw1 d er0 z))
("powdery" nil (p aw1 d er0 iy0))
("powe" nil (p aw1))
("powell" nil (p aw1 ax l))
("powelson" nil (p aw1 ax l s ax n))
("power" nil (p aw1 er0))
("powerboat" nil (p aw1 er0 b ow1 t))
("powerbook" nil (p aw1 er0 b uh1 k))
("powercise" nil (p aw1 r s ay1 z))
("powered" nil (p aw1 er0 d))
("powerful" nil (p aw1 er0 f ax l))
("powerfully" nil (p aw1 er0 f l iy0))
("powergen" nil (p aw1 er0 jh eh1 n))
("powerhouse" nil (p aw1 er0 hh aw1 s))
("powerhouses" nil (p aw1 er0 hh aw1 s ih0 z))
("powering" nil (p aw1 er0 ih0 ng))
("powerless" nil (p aw1 er0 l ax s))
("powerlessness" nil (p aw1 er0 l ax s n ax s))
("powerpc" nil (p aw1 er0 p iy1 s iy1))
("powerpcs" nil (p aw1 er0 p iy1 s iy1 z))
("powers" nil (p aw1 er0 z))
("powersoft" nil (p aw1 er0 s aa1 f t))
("powertrain" nil (p aw1 r t r ey1 n))
("powis" nil (p aw1 ih0 s))
("powles" nil (p aw1 ax l z))
("powless" nil (p aw1 l ih0 s))
("powley" nil (p aw1 l iy0))
("pownall" nil (p aw1 n ax l))
("pows" nil (p ow1 z))
("powwow" nil (p aw1 w aw1))
("pox" nil (p aa1 k s))
("poyer" nil (p oy1 er0))
("poyner" nil (p oy1 n er0))
("poynor" nil (p oy1 n er0))
("poynter" nil (p oy1 n t er0))
("poynton" nil (p oy1 n t ah1 n))
("poyser" nil (p oy1 s er0))
("poythress" nil (p oy1 th r ih0 s))
("poznan" nil (p aa1 z n ax n))
("poznanski" nil (p ax z n ae1 n s k iy0))
("pozner" nil (p aa1 z n er0))
("pozniak" nil (p aa1 z n iy0 ae0 k))
("poznikov" nil (p aa1 z n ih0 k aa1 v))
("pozo" nil (p ow1 z ow0))
("pozos" nil (p ow1 z ow0 s))
("pozzi" nil (p aa1 z iy0))
("ppm" nil (p iy1 p iy1 eh1 m))
("prab" nil (p r ae1 b))
("pracht" nil (p r ae1 k t))
("practicable" nil (p r ae1 k t ax k ax b ax l))
("practical" nil (p r ae1 k t ax k ax l))
("practicality" nil (p r ae1 k t ih0 k ae1 l ax t iy0))
("practically" nil (p r ae1 k t ax k l iy0))
("practice" nil (p r ae1 k t ax s))
("practiced" nil (p r ae1 k t ax s t))
("practices" nil (p r ae1 k t ax s ax z))
("practicing" nil (p r ae1 k t ax s ih0 ng))
("practitioner" nil (p r ae0 k t ih1 sh ax n er0))
("practitioners" nil (p r ae0 k t ih1 sh ax n er0 z))
("prada" nil (p r aa1 d ax))
("pradesh" nil (p r ax d eh1 sh))
("pradetto" nil (p r ax d eh1 t ow0))
("prado" nil (p r aa1 d ow0))
("praetor" nil (p r iy1 t er0))
("praetorian" nil (p r iy0 t ao1 r iy0 ax n))
("prager" nil (p r ey1 g er0))
("pragmatic" nil (p r ae0 g m ae1 t ih0 k))
("pragmatically" nil (p r ae0 g m ae1 t ih0 k ax l iy0))
("pragmatism" nil (p r ae1 g m ax t ih1 z ax m))
("pragmatist" nil (p r ae1 g m ax t ih0 s t))
("pragmatists" nil (p r ae1 g m ax t ih0 s t s))
("prague" nil (p r aa1 g))
("prahl" nil (p r aa1 l))
("prairie" nil (p r eh1 r iy0))
("prairies" nil (p r ey1 r iy0 z))
("praise" nil (p r ey1 z))
("praised" nil (p r ey1 z d))
("praises" nil (p r ey1 z ax z))
("praiseworthy" nil (p r ey1 z w er1 dh iy0))
("praising" nil (p r ey1 z ih0 ng))
("prall" nil (p r ao1 l))
("pralle" nil (p r ey1 l))
("prances" nil (p r ae1 n s ih0 z))
("prancing" nil (p r ae1 n s ih0 ng))
("prange" nil (p r ey1 n jh))
("pranger" nil (p r ey1 n jh er0))
("prank" nil (p r ae1 ng k))
("pranks" nil (p r ae1 ng k s))
("prankster" nil (p r ae1 ng k s t er0))
("prapas" nil (p r aa1 p ax s))
("prasad" nil (p r ae1 s ax d))
("prasek" nil (p r aa1 s eh0 k))
("prashant" nil (p r aa1 sh aa1 n t))
("prater" nil (p r ey1 t er0))
("pratfalls" nil (p r ae1 t f ao1 l z))
("prather" nil (p r ae1 dh er0))
("prato" nil (p r aa1 t ow0))
("prats" nil (p r ae1 t s))
("pratt" nil (p r ae1 t))
("pratte" nil (p r ae1 t))
("prattle" nil (p r ae1 t ax l))
("prattville" nil (p r ae1 t v ih0 l))
("praun" nil (p r ao1 n))
("prause" nil (p r ao1 z))
("pravda" nil (p r ae1 v d ax))
("prawn" nil (p r ao1 n))
("prawns" nil (p r ao1 n z))
("praxair" nil (p r ae1 k s eh1 r))
("praxis" nil (p r ae1 k s ih0 s))
("pray" nil (p r ey1))
("prayed" nil (p r ey1 d))
("prayer" nil (p r eh1 r))
("prayers" nil (p r eh1 r z))
("praying" nil (p r ey1 ih0 ng))
("prays" nil (p r ey1 z))
("praytor" nil (p r ey1 t er0))
("prazak" nil (p r aa1 z ax k))
("prchal" nil (p r sh ae1 l))
("pre" nil (p r iy1))
("preach" nil (p r iy1 ch))
("preached" nil (p r iy1 ch t))
("preacher" nil (p r iy1 ch er0))
("preachers" nil (p r iy1 ch er0 z))
("preaches" nil (p r iy1 ch ih0 z))
("preaching" nil (p r iy1 ch ih0 ng))
("preakness" nil (p r iy1 k n ax s))
("preamble" nil (p r iy0 ae1 m b ax l))
("prearranged" nil (p r iy1 er0 ey1 n jh d))
("preas" nil (p r iy1 z))
("prebble" nil (p r eh1 b ax l))
("prebe" nil (p r iy1 b))
("preble" nil (p r eh1 b ax l))
("prebon" nil (p r iy1 b aa0 n))
("precambrian" nil (p r iy0 k ae1 m b r iy0 ax n))
("precarious" nil (p r iy0 k eh1 r iy0 ax s))
("precariously" nil (p r ih0 k eh1 r iy0 ax s l iy0))
("precast" nil (p r iy0 k ae1 s t))
("precaution" nil (p r iy0 k ao1 sh ax n))
("precautionary" nil (p r ih0 k ao1 sh ax n eh0 r iy0))
("precautions" nil (p r iy0 k ao1 sh ax n z))
("precede" nil (p r ih0 s iy1 d))
("preceded" nil (p r ih0 s iy1 d ih0 d))
("precedence" nil (p r eh1 s ax d ax n s))
("precedent" nil (p r eh1 s ih0 d ax n t))
("precedents" nil (p r eh1 s ax d ax n t s))
("precedes" nil (p r ih0 s iy1 d z))
("preceding" nil (p r iy0 s iy1 d ih0 ng))
("preceeding" nil (p r ih0 s iy1 d ih0 ng))
("precept" nil (p r iy1 s eh1 p t))
("precepts" nil (p r iy1 s eh1 p t s))
("precession" nil (p r iy0 s eh1 sh ax n))
("precht" nil (p r eh1 k t))
("prechter" nil (p r eh1 k t er0))
("prechtl" nil (p r eh1 k t ax l))
("preciado" nil (p r eh0 ch aa1 d ow0))
("precinct" nil (p r iy1 s ih1 ng k t))
("precincts" nil (p r iy1 s ih1 ng k t s))
("precious" nil (p r eh1 sh ax s))
("precipice" nil (p r eh1 s ax p ax s))
("precipitate" nil (p r ih0 s ih1 p ih0 t ey1 t))
("precipitated" nil (p r ih0 s ih1 p ih0 t ey1 t ih0 d))
("precipitating" nil (p r ih0 s ih1 p ax t ey1 t ih0 ng))
("precipitation" nil (p r ih0 s ih1 p ih0 t ey1 sh ax n))
("precipitous" nil (p r ih0 s ih1 p ih0 t ax s))
("precipitously" nil (p r iy1 s ih1 p ih0 t ax s l iy0))
("precis" nil (p r ey1 s iy1))
("precise" nil (p r ih0 s ay1 s))
("precisely" nil (p r ih0 s ay1 s l iy0))
("precision" nil (p r iy0 s ih1 zh ax n))
("preclinical" nil (p r iy0 k l ih1 n ih0 k ax l))
("preclude" nil (p r ih0 k l uw1 d))
("precluded" nil (p r ih0 k l uw1 d ih0 d))
("precludes" nil (p r ih0 k l uw1 d z))
("precluding" nil (p r ih0 k l uw1 d ih0 ng))
("precocious" nil (p r ih0 k ow1 sh ax s))
("preconceived" nil (p r iy1 k ax n s iy1 v d))
("preconceptions" nil (p r iy0 k ax n s eh1 p sh ax n z))
("precondition" nil (p r iy1 k ax n d ih1 sh ax n))
("preconditions" nil (p r iy1 k ax n d ih1 sh ax n z))
("precook" nil (p r iy1 k uh1 k))
("precooked" nil (p r iy0 k uh1 k t))
("precourt" nil (p r ih0 k ao1 r t))
("precrash" nil (p r iy0 k r ae1 sh))
("precursor" nil (p r iy0 k er1 s er0))
("precursors" nil (p r iy0 k er1 s er0 z))
("predaceous" nil (p r iy0 d ey1 sh ax s))
("predate" nil (p r iy0 d ey1 t))
("predated" nil (p r iy0 d ey1 t ih0 d))
("predates" nil (p r iy1 d ey1 t s))
("predator" nil (p r eh1 d ax t er0))
("predators" nil (p r eh1 d ax t er0 z))
("predatory" nil (p r eh1 d ax t ao1 r iy0))
("predawn" nil (p r iy0 d ao1 n))
("preddy" nil (p r eh1 d iy0))
("predecessor" nil (p r eh1 d ax s eh1 s er0))
("predecessors" nil (p r eh1 d ax s eh1 s er0 z))
("predestination" nil (p r iy1 d eh1 s t ax n ey1 sh ax n))
("predetermined" nil (p r iy1 d iy0 t er1 m ih0 n d))
("predicament" nil (p r ih0 d ih1 k ax m ax n t))
("predicaments" nil (p r ih0 d ih1 k ax m ax n t s))
("predicate" v (p r eh1 d ih0 k ey1 t))
("predicate" n (p r eh1 d ih0 k ax t))
("predicated" nil (p r eh1 d ax k ey1 t ih0 d))
("predicates" v (p r eh1 d ax k ey1 t s))
("predicates" n (p r eh1 d ih0 k ax t s))
("predict" nil (p r ih0 d ih1 k t))
("predictability" nil (p r ih0 d ih1 k t ax b ih1 l ih0 t iy0))
("predictable" nil (p r ih0 d ih1 k t ax b ax l))
("predictably" nil (p r ih0 d ih1 k t ax b l iy0))
("predicted" nil (p r ih0 d ih1 k t ih0 d))
("predicting" nil (p r ih0 d ih1 k t ih0 ng))
("prediction" nil (p r iy0 d ih1 k sh ax n))
("predictions" nil (p r iy0 d ih1 k sh ax n z))
("predictive" nil (p r ih0 d ih1 k t ih0 v))
("predictor" nil (p r ih0 d ih1 k t er0))
("predictors" nil (p r ih0 d ih1 k t er0 z))
("predicts" nil (p r ih0 d ih1 k t s))
("predilection" nil (p r eh1 d ax l eh1 k sh ax n))
("predilections" nil (p r eh1 d ax l eh1 k sh ax n z))
("predispose" nil (p r iy1 d ih0 s p ow1 z))
("predisposed" nil (p r iy1 d ih0 s p ow1 z d))
("predisposing" nil (p r iy1 d ih0 s p ow1 z ih0 ng))
("predisposition" nil (p r iy1 d ih0 s p ax z ih1 sh ax n))
("predispositions" nil (p r iy1 d ih0 s p ax z ih1 sh ax n z))
("predmore" nil (p r eh1 d m ao0 r))
("predominance" nil (p r ih0 d aa1 m ax n ax n s))
("predominant" nil (p r ih0 d aa1 m ax n ax n t))
("predominantly" nil (p r ih0 d aa1 m ax n ax n t l iy0))
("predominate" nil (p r ih0 d aa1 m ax n ey1 t))
("predominated" nil (p r ih0 d aa1 m ax n ey1 t ih0 d))
("predominately" nil (p r ih0 d aa1 m ax n ax t l iy0))
("predominates" nil (p r ih0 d aa1 m ax n ey1 t s))
("predominating" nil (p r ih0 d aa1 m ax n ey1 t ih0 ng))
("pree" nil (p r iy1))
("preece" nil (p r iy1 s))
("preeminence" nil (p r iy0 eh1 m ax n ax n s))
("preeminent" nil (p r iy0 eh1 m ax n ax n t))
("preempt" nil (p r iy1 eh1 m p t))
("preempted" nil (p r iy0 eh1 m p t ih0 d))
("preemption" nil (p r iy1 eh1 m p sh ax n))
("preemptive" nil (p r iy0 eh1 m p t ih0 v))
("preened" nil (p r iy1 n d))
("preening" nil (p r iy1 n ih0 ng))
("preexist" nil (p r iy1 ih0 g z ih1 s t))
("preexisted" nil (p r iy1 ih0 g z ih1 s t ih0 d))
("preexisting" nil (p r iy1 ih0 g z ih1 s t ih0 ng))
("preexists" nil (p r iy1 ih0 g z ih1 s t s))
("prefab" nil (p r iy1 f ae1 b))
("prefabricated" nil (p r iy0 f ae1 b r ih0 k ey1 t ih0 d))
("prefabrication" nil (p r iy1 f ae1 b r ax k ey1 sh ax n))
("prefabs" nil (p r iy1 f ae1 b z))
("preface" nil (p r eh1 f ax s))
("prefaced" nil (p r eh1 f ax s t))
("prefect" nil (p r iy1 f eh1 k t))
("prefectural" nil (p r iy0 f eh1 k ch er0 ax l))
("prefecture" nil (p r iy1 f eh1 k ch er0))
("prefer" nil (p r ax f er1))
("preferable" nil (p r eh1 f er0 ax b ax l))
("preferably" nil (p r eh1 f er0 ax b l iy0))
("prefered" nil (p r ih0 f er1 d))
("preference" nil (p r eh1 f er0 ax n s))
("preferenced" nil (p r eh1 f er0 ax n s t))
("preferences" nil (p r eh1 f er0 ax n s ih0 z))
("preferential" nil (p r eh1 f er0 eh1 n ch ax l))
("preferentially" nil (p r eh1 f er0 eh1 n ch ax l iy0))
("preferred" nil (p r ax f er1 d))
("preferreds" nil (p r iy0 f er1 ax d z))
("preferring" nil (p r ih0 f er1 ih0 ng))
("prefers" nil (p r ax f er1 z))
("prefix" nil (p r iy1 f ih0 k s))
("prefrontal" nil (p r iy0 f r ah1 n t ax l))
("pregler" nil (p r eh1 g l er0))
("pregnancies" nil (p r eh1 g n ax n s iy0 z))
("pregnancy" nil (p r eh1 g n ax n s iy0))
("pregnant" nil (p r eh1 g n ax n t))
("prego" nil (p r ey1 g ow0))
("preheim" nil (p r eh1 hh ay0 m))
("prehistoric" nil (p r iy1 hh ih0 s t ao1 r ih0 k))
("prehn" nil (p r eh1 n))
("preignition" nil (p r iy1 ax g n ih1 sh ax n))
("preis" nil (p r iy1 z))
("preiser" nil (p r ay1 s er0))
("preisig" nil (p r ay1 z ih0 g))
("preisler" nil (p r ay1 s ax l er0))
("preiss" nil (p r ay1 s))
("prejean" nil (p r iy1 jh iy1 n))
("prejudged" nil (p r iy0 jh ah1 jh d))
("prejudgment" nil (p r iy0 jh ah1 jh m ax n t))
("prejudice" nil (p r eh1 jh ax d ih0 s))
("prejudiced" nil (p r eh1 jh ax d ax s t))
("prejudices" nil (p r eh1 jh ax d ih0 s ih0 z))
("prejudicial" nil (p r eh1 jh ax d ih1 sh ax l))
("prelate" nil (p r eh1 l ih0 t))
("prelates" nil (p r eh1 l ih0 t s))
("preliminarily" nil (p r ih0 l ih1 m ax n eh1 r ih0 l iy0))
("preliminary" nil (p r ih0 l ih1 m ax n eh1 r iy0))
("prelims" nil (p r iy1 l ih1 m z))
("prell" nil (p r eh1 l))
("prellwitz" nil (p r eh1 l w ih0 t s))
("prelude" nil (p r ey1 l uw1 d))
("preludes" nil (p r ey1 l uw1 d z))
("prem" nil (p r eh1 m))
("premadasa" nil (p r iy1 m ax d aa1 s ax))
("premarin" nil (p r eh1 m er0 ih0 n))
("premarital" nil (p r iy0 m eh1 r ax t ax l))
("premark" nil (p r iy0 m aa1 r k))
("premarket" nil (p r iy1 m aa1 r k ax t))
("premature" nil (p r iy1 m ax ch uh1 r))
("prematurely" nil (p r iy1 m ax ch uh1 r l iy0))
("premeditate" nil (p r iy0 m eh1 d ax t ey1 t))
("premeditated" nil (p r iy0 m eh1 d ax t ey1 t ih0 d))
("premeditation" nil (p r iy0 m eh1 d ax t ey1 sh ax n))
("premier" nil (p r eh0 m ih1 r))
("premiere" nil (p r eh0 m ih1 r))
("premiered" nil (p r eh0 m ih1 r d))
("premieres" nil (p r eh0 m ih1 r z))
("premiering" nil (p r eh0 m ih1 r ih0 ng))
("premiers" nil (p r eh0 m ih1 r z))
("premiership" nil (p r eh0 m ih1 r sh ih1 p))
("premise" nil (p r eh1 m ih0 s))
("premised" nil (p r eh1 m ax s t))
("premises" nil (p r eh1 m ax s ax z))
("premium" nil (p r iy1 m iy0 ax m))
("premiums" nil (p r iy1 m iy0 ax m z))
("premo" nil (p r eh1 m ow0))
("prenatal" nil (p r iy0 n ey1 t ax l))
("prendergast" nil (p r eh1 n d er0 g ae1 s t))
("prenger" nil (p r eh1 n jh er0))
("prenn" nil (p r eh1 n))
("prensa" nil (p r eh1 n s ax))
("prentice" nil (p r eh1 n t ih0 s))
("prentiss" nil (p r eh1 n t ih0 s))
("prentnieks" nil (p r eh1 n t n iy0 eh1 k s))
("preoccupation" nil (p r iy0 aa1 k y ax p ey1 sh ax n))
("preoccupations" nil (p r iy0 aa1 k y ax p ey1 sh ax n z))
("preoccupied" nil (p r iy0 aa1 k y ax p ay1 d))
("preoccupies" nil (p r iy0 aa1 k y ax p ay1 z))
("preoccupy" nil (p r iy0 aa1 k y ax p ay1))
("preordained" nil (p r iy1 ao0 r d ey1 n d))
("prep" nil (p r eh1 p))
("prepackaged" nil (p r iy0 p ae1 k ax jh d))
("prepaid" nil (p r iy0 p ey1 d))
("preparation" nil (p r eh1 p er0 ey1 sh ax n))
("preparations" nil (p r eh1 p er0 ey1 sh ax n z))
("preparatory" nil (p r ih0 p eh1 r ax t ao1 r iy0))
("prepare" nil (p r iy0 p eh1 r))
("prepared" nil (p r iy0 p eh1 r d))
("preparedness" nil (p r iy0 p eh1 r ax d n ax s))
("preparer" nil (p r iy0 p eh1 r er0))
("preparers" nil (p r iy0 p eh1 r er0 z))
("prepares" nil (p r iy0 p eh1 r z))
("preparing" nil (p r iy0 p eh1 r ih0 ng))
("prepay" nil (p r iy0 p ey1))
("prepaying" nil (p r iy0 p ey1 ih0 ng))
("prepayment" nil (p r iy0 p ey1 m ax n t))
("prepayments" nil (p r iy0 p ey1 m ax n t s))
("preplanning" nil (p r iy0 p l ae1 n ih0 ng))
("preponderance" nil (p r iy0 p aa1 n d r ax n s))
("preposterous" nil (p r ih0 p aa1 s t er0 ax s))
("prepped" nil (p r eh1 p t))
("preppie" nil (p r eh1 p iy0))
("preppy" nil (p r eh1 p iy0))
("prerecorded" nil (p r iy1 r iy0 k ao1 r d ih0 d))
("prerequisite" nil (p r iy0 r eh1 k w ax z ax t))
("prerequisites" nil (p r iy0 r eh1 k w ax z ax t s))
("prerogative" nil (p r ih0 r aa1 g ax t ih0 v))
("prerogatives" nil (p r ih0 r aa1 g ax t ih0 v z))
("pres" nil (p r eh1 z))
("presage" nil (p r eh1 s ih0 jh))
("presaged" nil (p r eh1 s ih0 jh d))
("presages" nil (p r eh1 s ih0 jh ih0 z))
("presaging" nil (p r eh1 s ih0 jh ih0 ng))
("presale" nil (p r iy1 s ey1 l))
("presario" nil (p r ih0 s aa1 r iy0 ow1))
("presby" nil (p r eh1 s b iy0))
("presbyterian" nil (p r eh1 s b ih0 t ih1 r iy0 ax n))
("preschel" nil (p r eh1 sh ax l))
("prescher" nil (p r eh1 sh er0))
("preschool" nil (p r iy0 s k uw1 l))
("preschoolers" nil (p r iy1 s k uw1 l er0 z))
("prescience" nil (p r iy1 sh iy0 ax n s))
("prescient" nil (p r eh1 s iy0 ax n t))
("prescot" nil (p r eh1 s k ax t))
("prescott" nil (p r eh1 s k aa0 t))
("prescribe" nil (p r ax s k r ay1 b))
("prescribed" nil (p r iy0 s k r ay1 b d))
("prescribers" nil (p r iy0 s k r ay1 b er0 z))
("prescribes" nil (p r iy0 s k r ay1 b z))
("prescribing" nil (p r iy0 s k r ay1 b ih0 ng))
("prescription" nil (p r ax s k r ih1 p sh ax n))
("prescriptions" nil (p r ax s k r ih1 p sh ax n z))
("presence" nil (p r eh1 z ax n s))
("presences" nil (p r eh1 z ax n s ih0 z))
("present" n (p r eh1 z ax n t))
("present" v (p r iy0 z eh1 n t))
("presentable" nil (p r ax z eh1 n t ax b ax l))
("presentation" nil (p r eh1 z ax n t ey1 sh ax n))
("presentations" nil (p r eh1 z ax n t ey1 sh ax n z))
("presented" nil (p r iy0 z eh1 n t ax d))
("presenter" nil (p r eh1 z ax n t er0))
("presenters" nil (p r eh1 z ax n t er0 z))
("presenting" nil (p r ih0 z eh1 n t ih0 ng))
("presently" nil (p r eh1 z ax n t l iy0))
("presentment" nil (p r iy0 z eh1 n t m ax n t))
("presents" n (p r eh1 z ax n t s))
("presents" v (p r iy0 z eh1 n t s))
("preservation" nil (p r eh1 z er0 v ey1 sh ax n))
("preservationist" nil (p r eh1 z er0 v ey1 sh ax n ih0 s t))
("preservationists" nil (p r eh1 z er0 v ey1 sh ax n ih0 s t s))
("preservative" nil (p r iy0 z er1 v ax t ih0 v))
("preservatives" nil (p r ax z er1 v ax t ih0 v z))
("preserve" nil (p r ax z er1 v))
("preserved" nil (p r ax z er1 v d))
("preserves" nil (p r ax z er1 v z))
("preserving" nil (p r ax z er1 v ih0 ng))
("preset" nil (p r iy0 s eh1 t))
("preside" nil (p r ih0 z ay1 d))
("presided" nil (p r ih0 z ay1 d ih0 d))
("presidencies" nil (p r eh1 z ax d ax n s iy0 z))
("presidency" nil (p r eh1 z ax d ax n s iy0))
("president" nil (p r eh1 z ax d eh1 n t))
("presidential" nil (p r eh1 z ax d eh1 n ch ax l))
("presidentialist" nil (p r eh1 z ax d eh1 n ch ax l ih0 s t))
("presidentially" nil (p r eh1 s ih0 d eh1 n ch ax l iy0))
("presidents" nil (p r eh1 z ax d eh1 n t s))
("presides" nil (p r ih0 z ay1 d z))
("presiding" nil (p r ih0 z ay1 d ih0 ng))
("presidio" nil (p r ih0 s ih1 d iy0 ow1))
("presidium" nil (p r ih0 s ih1 d iy0 ax m))
("preslar" nil (p r ih0 s l aa1 r))
("presler" nil (p r eh1 s ax l er0))
("presley" nil (p r eh1 s l iy0))
("presnall" nil (p r eh1 s n ax l))
("presnell" nil (p r eh1 s n ax l))
("presplit" nil (p r iy1 s p l ih1 t))
("press" nil (p r eh1 s))
("pressboard" nil (p r eh1 s b ao1 r d))
("pressburger" nil (p r eh1 s b er0 g er0))
("presse" nil (p r eh1 s))
("pressed" nil (p r eh1 s t))
("pressel" nil (p r eh1 s ax l))
("presser" nil (p r eh1 s er0))
("presses" nil (p r eh1 s ax z))
("pressey" nil (p r eh1 s iy0))
("pressing" nil (p r eh1 s ih0 ng))
("pressler" nil (p r eh1 s l er0))
("pressley" nil (p r eh1 s l iy0))
("pressly" nil (p r eh1 s l iy0))
("pressman" nil (p r eh1 s m ax n))
("pressnell" nil (p r eh1 s n ax l))
("presson" nil (p r eh1 s ax n))
("presstek" nil (p r eh1 s t eh1 k))
("pressure" nil (p r eh1 sh er0))
("pressured" nil (p r eh1 sh er0 d))
("pressures" nil (p r eh1 sh er0 z))
("pressuring" nil (p r eh1 sh er0 ih0 ng))
("pressurization" nil (p r eh1 sh er0 ih0 z ey1 sh ax n))
("pressurized" nil (p r eh1 sh er0 ay1 z d))
("presswood" nil (p r eh1 s w uh1 d))
("prest" nil (p r eh1 s t))
("presta" nil (p r eh1 s t ax))
("prestage" nil (p r eh1 s t ih0 jh))
("presti" nil (p r eh1 s t iy0))
("prestia" nil (p r eh1 s t y ax))
("prestidge" nil (p r eh1 s t ih0 jh))
("prestige" nil (p r eh0 s t iy1 zh))
("prestigiacomo" nil (p r eh0 s t iy1 jh ax k ow0 m ow0))
("prestigious" nil (p r eh0 s t ih1 jh ax s))
("presto" nil (p r eh1 s t ow1))
("preston" nil (p r eh1 s t ax n))
("prestowitz" nil (p r eh1 s t ax w ih0 t s))
("prestridge" nil (p r eh1 s t r ih0 jh))
("prestwich" nil (p r eh1 s t w ih0 ch))
("prestwood" nil (p r eh1 s t w uh1 d))
("presumably" nil (p r ax z uw1 m ax b l iy0))
("presume" nil (p r ih0 z uw1 m))
("presumed" nil (p r ax z uw1 m d))
("presumes" nil (p r ax z uw1 m z))
("presuming" nil (p r ih0 z uw1 m ih0 ng))
("presumption" nil (p r ih0 z ah1 m p sh ax n))
("presumptive" nil (p r iy0 z ah1 m p t ih0 v))
("presumptuous" nil (p r ih0 z ah1 m p ch ax w ax s))
("presupposes" nil (p r iy1 s ax p ow1 z ih0 z))
("presupposition" nil (p r iy1 s ax p ax z ih1 sh ax n))
("presutti" nil (p r eh0 s uw1 t iy0))
("preszler" nil (p r eh1 s l er0))
("pretax" nil (p r iy1 t ae1 k s))
("prete" nil (p r iy1 t))
("pretend" nil (p r iy0 t eh1 n d))
("pretended" nil (p r iy0 t eh1 n d ax d))
("pretending" nil (p r iy0 t eh1 n d ih0 ng))
("pretends" nil (p r iy0 t eh1 n d z))
("pretense" nil (p r iy0 t eh1 n s))
("pretenses" nil (p r iy0 t eh1 n s ih0 z))
("pretension" nil (p r iy0 t eh1 n sh ax n))
("pretensions" nil (p r ax t eh1 n sh ax n z))
("pretentious" nil (p r iy0 t eh1 n sh ax s))
("pretext" nil (p r iy1 t eh1 k s t))
("pretoria" nil (p r iy0 t ao1 r iy0 ax))
("pretrial" nil (p r iy0 t r ay1 ax l))
("pretti" nil (p r eh1 t iy0))
("prettier" nil (p r ih1 t iy0 er0))
("pretties" nil (p r ih1 t iy0 z))
("prettiest" nil (p r ih1 t iy0 ax s t))
("pretty" nil (p r ih1 t iy0))
("prettyman" nil (p r eh1 t iy0 m ax n))
("pretzel" nil (p r eh1 t z ax l))
("pretzels" nil (p r eh1 t z ax l z))
("pretzer" nil (p r eh1 t z er0))
("preuss" nil (p r uw1 s))
("preussag" nil (p r uw1 s ae0 g))
("preusser" nil (p r oy1 s er0))
("prevail" nil (p r ih0 v ey1 l))
("prevailed" nil (p r ih0 v ey1 l d))
("prevailing" nil (p r ih0 v ey1 l ih0 ng))
("prevails" nil (p r ih0 v ey1 l z))
("prevalence" nil (p r eh1 v ax l ax n s))
("prevalent" nil (p r eh1 v ax l ax n t))
("prevatt" nil (p r ih0 v ae1 t))
("prevatte" nil (p r ih0 v ae1 t))
("prevent" nil (p r ih0 v eh1 n t))
("preventable" nil (p r ih0 v eh1 n t ax b ax l))
("preventative" nil (p r iy0 v eh1 n t ax t ih0 v))
("preventatives" nil (p r iy0 v eh1 n t ax t ih0 v z))
("prevented" nil (p r ih0 v eh1 n t ih0 d))
("preventing" nil (p r ih0 v eh1 n t ih0 ng))
("prevention" nil (p r iy0 v eh1 n sh ax n))
("preventive" nil (p r ih0 v eh1 n t ih0 v))
("prevents" nil (p r ih0 v eh1 n t s))
("prevette" nil (p r ih0 v eh1 t))
("preview" nil (p r iy1 v y uw1))
("previewed" nil (p r iy1 v y uw1 d))
("previewing" nil (p r iy1 v y uw1 ih0 ng))
("previews" nil (p r iy1 v y uw1 z))
("previous" nil (p r iy1 v iy0 ax s))
("previously" nil (p r iy1 v iy0 ax s l iy0))
("previte" nil (p r eh1 v ay0 t))
("previti" nil (p r eh0 v iy1 t iy0))
("prevo" nil (p r eh1 v ow0))
("prevost" nil (p r ey1 v ow0 s t))
("prew" nil (p r uw1))
("prewar" nil (p r iy0 w ao1 r))
("preway" nil (p r iy1 w ey1))
("prewett" nil (p r uw1 ih0 t))
("prewitt" nil (p r uw1 ih0 t))
("prey" nil (p r ey1))
("preyed" nil (p r ey1 d))
("preyer" nil (p r ey1 er0))
("preying" nil (p r ey1 ih0 ng))
("preys" nil (p r ey1 z))
("preziosi" nil (p r eh0 z iy0 ow1 s iy0))
("prezioso" nil (p r eh0 z iy0 ow1 s ow0))
("prezzano" nil (p r eh0 z aa1 n ow0))
("priam" nil (p r ay1 ae1 m))
("pribble" nil (p r ih1 b ax l))
("pribula" nil (p r iy0 b uw1 l ax))
("pribyl" nil (p r ih1 b ax l))
("price" nil (p r ay1 s))
("priced" nil (p r ay1 s t))
("priceless" nil (p r ay1 s l ax s))
("pricer" nil (p r ay1 s er0))
("prices" nil (p r ay1 s ax z))
("pricey" nil (p r ay1 s iy0))
("prichard" nil (p r ih1 ch er0 d))
("prichett" nil (p r ih1 ch ih0 t))
("pricier" nil (p r ay1 s iy0 er0))
("priciest" nil (p r ay1 s iy0 ih0 s t))
("pricing" nil (p r ay1 s ih0 ng))
("pricings" nil (p r ay1 s ih0 ng z))
("prick" nil (p r ih1 k))
("pricked" nil (p r ih1 k t))
("prickett" nil (p r ih1 k ih0 t))
("pricklebacks" nil (p r ih1 k ax l b ae1 k s))
("prickly" nil (p r ih1 k l iy0))
("pricks" nil (p r ih1 k s))
("pricor" nil (p r ay1 k ao1 r))
("priddy" nil (p r ih1 d iy0))
("pride" nil (p r ay1 d))
("prideaux" nil (p r ih1 d ow0))
("prided" nil (p r ay1 d ih0 d))
("pridemore" nil (p r ay1 d m ao0 r))
("prides" nil (p r ay1 d z))
("pridgen" nil (p r ih1 jh ax n))
("pridgeon" nil (p r ih1 d jh ih0 n))
("pridmore" nil (p r ih1 d m ao0 r))
("priebe" nil (p r iy1 b))
("pried" nil (p r ay1 d))
("priem" nil (p r iy1 m))
("prien" nil (p r iy1 n))
("prier" nil (p r ay1 er0))
("pries" nil (p r ay1 z))
("priess" nil (p r iy1 s))
("priest" nil (p r iy1 s t))
("priester" nil (p r iy1 s t er0))
("priestess" nil (p r iy1 s t ax s))
("priesthood" nil (p r iy1 s t hh uh0 d))
("priestley" nil (p r iy1 s t l iy0))
("priestly" nil (p r iy1 s t l iy0))
("priests" nil (p r iy1 s t s))
("prieta" nil (p r iy1 t ax))
("prieto" nil (p r iy1 t ow0))
("prieur" nil (p r iy1 er0))
("prig" nil (p r ih1 g))
("prigge" nil (p r ih1 g))
("prigmore" nil (p r ih1 g m ao0 r))
("prihoda" nil (p r iy0 hh ow1 d ax))
("prill" nil (p r ih1 l))
("prillaman" nil (p r ih1 l ax m ax n))
("prim" nil (p r ih1 m))
("prima" nil (p r iy1 m ax))
("primack" nil (p r ih1 m ax k))
("primacy" nil (p r ay1 m ax s iy0))
("primal" nil (p r ay1 m ax l))
("primaries" nil (p r ay1 m eh1 r iy0 z))
("primarily" nil (p r ay0 m eh1 r ax l iy0))
("primark" nil (p r ih1 m aa0 r k))
("primary" nil (p r ay1 m eh1 r iy0))
("primate" nil (p r ay1 m ey1 t))
("primates" nil (p r ay1 m ey1 t s))
("primavera" nil (p r iy0 m aa0 v eh1 r ax))
("prime" nil (p r ay1 m))
("primeau" nil (p r ih0 m ow1))
("primeaux" nil (p r ih0 m ow1))
("primebank" nil (p r ay1 m b ae1 ng k))
("primeco" nil (p r ay1 m k ow1))
("primed" nil (p r ay1 m d))
("primenews" nil (p r ay1 m n uw1 z))
("primer" nil (p r ay1 m er0))
("primerica" nil (p r ih0 m eh1 r ih0 k ax))
("primes" nil (p r ay1 m z))
("primestar" nil (p r ay1 m s t aa1 r))
("primetime" nil (p r ay1 m t ay1 m))
("primeval" nil (p r ay0 m iy1 v ax l))
("primiano" nil (p r iy1 m iy0 aa1 n ow0))
("priming" nil (p r ay1 m ih0 ng))
("primitive" nil (p r ih1 m ax t ih0 v))
("primitivistic" nil (p r ih1 m ih0 t ih0 v ih1 s t ih0 k))
("primly" nil (p r ih1 m l iy0))
("primm" nil (p r ih1 m))
("primmer" nil (p r ih1 m er0))
("primo" nil (p r iy1 m ow1))
("primordial" nil (p r ih0 m ao1 r d iy0 ax l))
("primrose" nil (p r ih1 m r ow1 z))
("primroses" nil (p r ih1 m r ow1 z ax z))
("primus" nil (p r ay1 m ax s))
("prince" nil (p r ih1 n s))
("princely" nil (p r ih1 n s l iy0))
("princes" nil (p r ih1 n s ax z))
("princess" nil (p r ih1 n s eh0 s))
("princeton" nil (p r ih1 n s t ax n))
("princeville" nil (p r ih1 n s v ih1 l))
("principal" nil (p r ih1 n s ax p ax l))
("principality" nil (p r ih1 n s ih0 p ae1 l ih0 t iy0))
("principally" nil (p r ih1 n s ih0 p l iy0))
("principals" nil (p r ih1 n s ax p ax l z))
("principato" nil (p r iy0 n ch iy0 p aa1 t ow0))
("principe" nil (p r iy0 n ch iy1 p iy0))
("principle" nil (p r ih1 n s ax p ax l))
("principled" nil (p r ih1 n s ax p ax l d))
("principles" nil (p r ih1 n s ax p ax l z))
("prindiville" nil (p r iy0 n d iy1 v ih0 l))
("prindl" nil (p r ih1 n d ax l))
("prindle" nil (p r ih1 n d ax l))
("prine" nil (p r ay1 n))
("pring" nil (p r ih1 ng))
("pringle" nil (p r ih1 ng g ax l))
("prinkey" nil (p r ih1 ng k iy0))
("prinos" nil (p r iy1 n ow0 s))
("prins" nil (p r ih1 n z))
("prinsen" nil (p r ih1 n s ax n))
("print" nil (p r ih1 n t))
("printable" nil (p r ih1 n t ax b ax l))
("printed" nil (p r ih1 n t ax d))
("printemps" nil (p r ih1 n t eh0 m p s))
("printer" nil (p r ih1 n t er0))
("printers" nil (p r ih1 n t er0 z))
("printing" nil (p r ih1 n t ih0 ng))
("printings" nil (p r ih1 n t ih0 ng z))
("printmaker" nil (p r ih1 n t m ey1 k er0))
("printmakers" nil (p r ih1 n t m ey1 k er0 z))
("printon" nil (p r ih1 n t ax n))
("printout" nil (p r ih1 n t aw1 t))
("printouts" nil (p r ih1 n t aw1 t s))
("prints" nil (p r ih1 n t s))
("printup" nil (p r ih1 n t ah1 p))
("printy" nil (p r ih1 n t iy0))
("printz" nil (p r ih1 n t s))
("prinz" nil (p r ih1 n z))
("priola" nil (p r iy0 ow1 l ax))
("prioleau" nil (p r iy0 ow1 l ow0))
("priolo" nil (p r iy0 ow1 l ow0))
("prior" nil (p r ay1 er0))
("priore" nil (p r iy0 ao1 r iy0))
("priorities" nil (p r ay0 ao1 r ax t iy0 z))
("prioritize" nil (p r ay0 ao1 r ax t ay1 z))
("prioritized" nil (p r ay0 ao1 r ax t ay1 z d))
("prioritizes" nil (p r ay0 ao1 r ax t ay1 z ih0 z))
("prioritizing" nil (p r ay0 ao1 r ax t ay1 z ih0 ng))
("priority" nil (p r ay0 ao1 r ax t iy0))
("priors" nil (p r ay1 er0 z))
("priory" nil (p r ay1 er0 iy0))
("pris" nil (p r ih1 s))
("priscilla" nil (p r ih0 s ih1 l ax))
("prisco" nil (p r iy1 s k ow0))
("prisk" nil (p r ih1 s k))
("prism" nil (p r ih1 z ax m))
("prismatic" nil (p r ih0 z m ae1 t ih0 k))
("prison" nil (p r ih1 z ax n))
("prisoner" nil (p r ih1 z ax n er0))
("prisoners" nil (p r ih1 z ax n er0 z))
("prisons" nil (p r ih1 z ax n z))
("prissie" nil (p r ih1 s iy0))
("prissy" nil (p r ih1 s iy0))
("pristine" nil (p r ih1 s t iy0 n))
("pritchard" nil (p r ih1 ch er0 d))
("pritchett" nil (p r ih1 ch ih0 t))
("pritt" nil (p r ih1 t))
("pritts" nil (p r ih1 t s))
("pritz" nil (p r ih1 t s))
("pritzker" nil (p r ih1 t s k er0))
("pritzkers" nil (p r ih1 t s k er0 z))
("pritzl" nil (p r ih1 t z ax l))
("privacies" nil (p r ay1 v ax s iy0 z))
("privacy" nil (p r ay1 v ax s iy0))
("private" nil (p r ay1 v ax t))
("privately" nil (p r ay1 v ax t l iy0))
("privates" nil (p r ay1 v ax t s))
("privations" nil (p r ay0 v ey1 sh ax n z))
("privatisation" nil (p r ay1 v ax t ax z ey1 sh ax n))
("privatization" nil (p r ay1 v ax t ax z ey1 sh ax n))
("privatizations" nil (p r ay1 v ax t ax z ey1 sh ax n z))
("privatize" nil (p r ih1 v ax t ay1 z))
("privatized" nil (p r ay1 v ax t ay1 z d))
("privatizing" nil (p r ay1 v ax t ay1 z ih0 ng))
("privett" nil (p r ih1 v ih0 t))
("privette" nil (p r ih0 v eh1 t))
("privilege" nil (p r ih1 v l ax jh))
("privileged" nil (p r ih1 v l ax jh d))
("privileges" nil (p r ih1 v l ax jh ax z))
("privitera" nil (p r iy0 v iy0 t eh1 r ax))
("privott" nil (p r ih1 v ax t))
("privy" nil (p r ih1 v iy0))
("prix" nil (p r iy1))
("prize" nil (p r ay1 z))
("prized" nil (p r ay1 z d))
("prizes" nil (p r ay1 z ax z))
("prizm" nil (p r ih1 z ax m))
("pro" nil (p r ow1))
("proactive" nil (p r ow1 ae1 k t ih0 v))
("probabilities" nil (p r aa1 b ax b ih1 l ax t iy0 z))
("probability" nil (p r aa1 b ax b ih1 l ax t iy0))
("probable" nil (p r aa1 b ax b ax l))
("probably" nil (p r aa1 b ax b l iy0))
("probasco" nil (p r ow0 b aa1 s k ow0))
("probate" nil (p r ow1 b ey1 t))
("probation" nil (p r ow0 b ey1 sh ax n))
("probationary" nil (p r ow0 b ey1 sh ax n eh1 r iy0))
("probationers" nil (p r ow0 b ey1 sh ax n er0 z))
("probative" nil (p r ow0 b ey1 t ih0 v))
("probe" nil (p r ow1 b))
("probed" nil (p r ow1 b d))
("probenecid" nil (p r aa1 b eh1 n ax s ih0 d))
("probert" nil (p r aa1 b er0 t))
("probes" nil (p r ow1 b z))
("probing" nil (p r ow1 b ih0 ng))
("probity" nil (p r ow1 b ax t iy0))
("problem" nil (p r aa1 b l ax m))
("problematic" nil (p r aa1 b l ax m ae1 t ih0 k))
("problematical" nil (p r aa1 b l ax m ae1 t ih0 k ax l))
("problems" nil (p r aa1 b l ax m z))
("proboscis" nil (p r ow0 b aa1 s ax s))
("probst" nil (p r aa1 b s t))
("probursa" nil (p r ow1 b er1 s ax))
("probus" nil (p r ow1 b ih0 s))
("procaccini" nil (p r ow0 k aa0 ch iy1 n iy0))
("procaine" nil (p r ow1 k ey1 n))
("procardia" nil (p r ow0 k aa1 r d iy0 ax))
("procassini" nil (p r aa1 k ax s iy1 n iy0))
("procedural" nil (p r ax s iy1 jh er0 ax l))
("procedure" nil (p r ax s iy1 jh er0))
("procedures" nil (p r ax s iy1 jh er0 z))
("proceed" v (p r ax s iy1 d))
("proceed" n (p r ow0 s iy1 d))
("proceeded" nil (p r ax s iy1 d ax d))
("proceeding" nil (p r ax s iy1 d ih0 ng))
("proceedings" nil (p r ow0 s iy1 d ih0 ng z))
("proceeds" v (p r ax s iy1 d z))
("proceeds" n (p r ow0 s iy1 d z))
("procell" nil (p r ow0 s eh1 l))
("proceso" nil (p r ow1 s eh1 s ow0))
("process" nil (p r aa1 s eh1 s))
("processed" nil (p r aa1 s eh0 s t))
("processes" nil (p r aa1 s eh0 s ax z))
("processing" nil (p r aa1 s eh0 s ih0 ng))
("procession" nil (p r ax s eh1 sh ax n))
("processional" nil (p r ax s eh1 sh ax n ax l))
("processions" nil (p r ow0 s eh1 sh ax n z))
("processor" nil (p r aa1 s eh1 s er0))
("processors" nil (p r aa1 s eh1 s er0 z))
("proch" nil (p r aa1 k))
("prochaska" nil (p r ax hh aa1 s k ax))
("prochazka" nil (p r ax hh aa1 z k ax))
("prochnow" nil (p r aa1 n aw0))
("procida" nil (p r ow0 ch iy1 d ax))
("prock" nil (p r aa1 k))
("prockter" nil (p r aa1 k t er0))
("proclaim" nil (p r ow0 k l ey1 m))
("proclaimed" nil (p r ow0 k l ey1 m d))
("proclaiming" nil (p r ow0 k l ey1 m ih0 ng))
("proclaims" nil (p r ow0 k l ey1 m z))
("proclamation" nil (p r aa1 k l ax m ey1 sh ax n))
("proclamations" nil (p r aa1 k l ax m ey1 sh ax n z))
("proclivities" nil (p r ow0 k l ih1 v ax t iy0 z))
("proclivity" nil (p r ow0 k l ih1 v ax t iy0))
("proconsul" nil (p r ow0 k aa1 n s ax l))
("procopio" nil (p r ow0 k ow1 p iy0 ow0))
("procordia" nil (p r aa0 k ao1 r d iy0 ax))
("procrastinate" nil (p r ax k r ae1 s t ax n ey1 t))
("procrastinating" nil (p r ax k r ae1 s t ax n ey1 t ih0 ng))
("procrastination" nil (p r ax k r ae1 s t ax n ey1 sh ax n))
("procreate" nil (p r ow1 k r iy0 ey1 t))
("procreates" nil (p r ow1 k r iy0 ey1 t s))
("procreation" nil (p r ow1 k r iy0 ey1 sh ax n))
("procter" nil (p r aa1 k t er0))
("proctor" nil (p r aa1 k t er0))
("proctors" nil (p r aa1 k t er0 z))
("proctorsilex" nil (p r ao1 k t er0 s ay1 l eh0 k s))
("procure" nil (p r ow0 k y uh1 r))
("procured" nil (p r ow0 k y uh1 r d))
("procurement" nil (p r ow0 k y uh1 r m ax n t))
("procurements" nil (p r ow0 k y uh1 r m ax n t s))
("procurer" nil (p r ow0 k y uh1 r er0))
("procuring" nil (p r ow0 k y uh1 r ih0 ng))
("procyte" nil (p r ow1 s ay1 t))
("prod" nil (p r aa1 d))
("prodded" nil (p r aa1 d ih0 d))
("prodding" nil (p r aa1 d ih0 ng))
("prodi" nil (p r ow1 d iy0))
("prodigal" nil (p r aa1 d ih0 g ax l))
("prodigies" nil (p r aa1 d ax jh iy0 z))
("prodigious" nil (p r ax d ih1 jh ax s))
("prodigiously" nil (p r ow0 d ih1 jh ih0 s l iy0))
("prodigy" nil (p r aa1 d ax jh iy0))
("prods" nil (p r aa1 d z))
("produce" v (p r ax d uw1 s))
("produce" n (p r ow1 d uw0 s))
("produced" nil (p r ax d uw1 s t))
("producer" nil (p r ax d uw1 s er0))
("producers" nil (p r ax d uw1 s er0 z))
("produces" v (p r ax d uw1 s ax z))
("produces" n (p r ax d uw1 s ih0 z))
("producing" nil (p r ax d uw1 s ih0 ng))
("product" nil (p r aa1 d ax k t))
("production" nil (p r ax d ah1 k sh ax n))
("productions" nil (p r ax d ah1 k sh ax n z))
("productive" nil (p r ax d ah1 k t ih0 v))
("productively" nil (p r ax d ah1 k t ih0 v l iy0))
("productivity" nil (p r ow1 d ax k t ih1 v ax t iy0))
("products" nil (p r aa1 d ax k t s))
("proehl" nil (p r ow1 l))
("profane" nil (p r ow0 f ey1 n))
("profanity" nil (p r ow0 f ae1 n ax t iy0))
("profess" nil (p r ax f eh1 s))
("professed" nil (p r ax f eh1 s t))
("professes" nil (p r ax f eh1 s ih0 z))
("professing" nil (p r ax f eh1 s ih0 ng))
("profession" nil (p r ax f eh1 sh ax n))
("professional" nil (p r ax f eh1 sh ax n ax l))
("professionalism" nil (p r ax f eh1 sh ax n ax l ih1 z ax m))
("professionally" nil (p r ax f eh1 sh ax n ax l iy0))
("professionals" nil (p r ax f eh1 sh ax n ax l z))
("professions" nil (p r ax f eh1 sh ax n z))
("professor" nil (p r ax f eh1 s er0))
("professorial" nil (p r ow1 f ax s ao1 r iy0 ax l))
("professors" nil (p r ax f eh1 s er0 z))
("professorship" nil (p r ax f eh1 s er0 sh ih1 p))
("profeta" nil (p r ow0 f eh1 t ax))
("proffer" nil (p r aa1 f er0))
("proffered" nil (p r aa1 f er0 d))
("proffering" nil (p r aa1 f er0 ih0 ng))
("proffit" nil (p r aa1 f ih0 t))
("proffitt" nil (p r aa1 f ih0 t))
("proficiency" nil (p r ax f ih1 sh ax n s iy0))
("proficient" nil (p r aa0 f ih1 sh ax n t))
("profile" nil (p r ow1 f ay1 l))
("profiled" nil (p r ow1 f ay1 l d))
("profiles" nil (p r ow1 f ay1 l z))
("profiling" nil (p r ow1 f ay1 l ih0 ng))
("profit" nil (p r aa1 f ax t))
("profitability" nil (p r aa1 f ih0 t ax b ih1 l ih0 t iy0))
("profitable" nil (p r aa1 f ax t ax b ax l))
("profitablity" nil (p r aa1 f ax t ax b l ih1 t iy0))
("profitably" nil (p r aa1 f ax t ax b l iy0))
("profited" nil (p r aa1 f ax t ax d))
("profiteering" nil (p r aa1 f ax t ih1 r ih0 ng))
("profiteers" nil (p r aa1 f ax t ih1 r z))
("profiting" nil (p r aa1 f ax t ih0 ng))
("profits" nil (p r aa1 f ax t s))
("profitt" nil (p r aa1 f ih0 t))
("profittaking" nil (p r aa1 f ih0 t t ey1 k ih0 ng))
("profligacy" nil (p r ao1 f l ih0 g ae1 s iy0))
("profligate" nil (p r ao1 f l ih0 g ey1 t))
("profound" nil (p r ow0 f aw1 n d))
("profoundly" nil (p r ow0 f aw1 n d l iy0))
("profs" nil (p r aa1 f s))
("profuse" nil (p r ax f y uw1 s))
("profusely" nil (p r ax f y uw1 s l iy0))
("profusion" nil (p r ax f y uw1 zh ax n))
("progams" nil (p r ow1 g ae0 m z))
("progenitor" nil (p r ow0 jh eh1 n ih0 t er0))
("progeny" nil (p r aa1 jh ax n iy0))
("progesterone" nil (p r ow0 jh eh1 s t er0 ow1 n))
("proglacial" nil (p r ow0 g l ey1 sh ax l))
("prognoses" nil (p r aa0 g n ow1 s iy0 z))
("prognosis" nil (p r aa0 g n ow1 s ax s))
("prognosticate" nil (p r aa1 g n aa1 s t ax k ey1 t))
("prognosticated" nil (p r aa1 g n aa1 s t ax k ey1 t ih0 d))
("prognosticater" nil (p r aa1 g n aa1 s t ax k ey1 t er0))
("prognosticates" nil (p r aa1 g n aa1 s t ax k ey1 t s))
("prognosticating" nil (p r aa1 g n aa1 s t ax k ey1 t ih0 ng))
("prognosticative" nil (p r aa1 g n aa1 s t ax k ey1 t ih0 v))
("program" nil (p r ow1 g r ae1 m))
("programing" nil (p r ow1 g r ae1 m ih0 ng))
("programmable" nil (p r ow1 g r ae1 m ax b ax l))
("programmatic" nil (p r aa1 g r ax m ae1 t ih0 k))
("programme" nil (p r ow1 g r ae1 m))
("programmed" nil (p r ow1 g r ae1 m d))
("programmer" nil (p r ow1 g r ae1 m er0))
("programmers" nil (p r ow1 g r ae1 m er0 z))
("programming" nil (p r ow1 g r ae1 m ih0 ng))
("programs" nil (p r ow1 g r ae1 m z))
("progress" n (p r aa1 g r eh1 s))
("progress" v (p r ax g r eh1 s))
("progressed" nil (p r ax g r eh1 s t))
("progresses" nil (p r aa1 g r eh1 s ax z))
("progressing" nil (p r ax g r eh1 s ih0 ng))
("progression" nil (p r ax g r eh1 sh ax n))
("progressive" nil (p r ax g r eh1 s ih0 v))
("progressively" nil (p r aa0 g r eh1 s ih0 v l iy0))
("progressives" nil (p r aa0 g r eh1 s ih0 v z))
("progressivity" nil (p r aa1 g r eh0 s ih1 v ax t iy0))
("prohaska" nil (p r ax hh aa1 s k ax))
("prohibit" nil (p r ow0 hh ih1 b ax t))
("prohibited" nil (p r ow0 hh ih1 b ax t ax d))
("prohibiting" nil (p r ow0 hh ih1 b ax t ih0 ng))
("prohibition" nil (p r ow1 ax b ih1 sh ax n))
("prohibitions" nil (p r ow1 ax b ih1 sh ax n z))
("prohibitive" nil (p r ow0 hh ih1 b ax t ih0 v))
("prohibitively" nil (p r ow0 hh ih1 b ax t ih0 v l iy0))
("prohibitory" nil (p r ow0 hh ih1 b ax t ao1 r iy0))
("prohibits" nil (p r ow0 hh ih1 b ax t s))
("proia" nil (p r ow1 y ax))
("proietti" nil (p r oy0 eh1 t iy0))
("project" n (p r aa1 jh eh0 k t))
("project" v (p r ax jh eh1 k t))
("projected" nil (p r ax jh eh1 k t ax d))
("projectile" nil (p r ax jh eh1 k t ax l))
("projectiles" nil (p r ax jh eh1 k t ax l z))
("projecting" nil (p r ax jh eh1 k t ih0 ng))
("projection" nil (p r ax jh eh1 k sh ax n))
("projections" nil (p r ax jh eh1 k sh ax n z))
("projective" nil (p r ax jh eh1 k t ih0 v))
("projector" nil (p r ax jh eh1 k t er0))
("projectors" nil (p r ax jh eh1 k t er0 z))
("projects" n (p r aa1 jh eh0 k t s))
("projects" v (p r ax jh eh1 k t s))
("prokofiev" nil (p r aa1 k ow0 f iy1 v))
("prokop" nil (p r ow1 k ax p))
("prolactin" nil (p r ow0 l ae1 k t ax n))
("proler" nil (p r ow1 l er0))
("proletarian" nil (p r ow1 l ax t eh1 r iy0 ax n))
("proletariat" nil (p r ow1 l ax t eh1 r iy0 ax t))
("proleukin" nil (p r ow1 l uw0 k ih0 n))
("proliferate" nil (p r ow0 l ih1 f er0 ey1 t))
("proliferated" nil (p r ax l ih1 f er0 ey1 t ih0 d))
("proliferating" nil (p r ow0 l ih1 f er0 ey1 t ih0 ng))
("proliferation" nil (p r ow1 l ih0 f er0 ey1 sh ax n))
("prolific" nil (p r ow0 l ih1 f ih0 k))
("prolifically" nil (p r ow0 l ih1 f ih0 k l iy0))
("prolinea" nil (p r ow1 l ih1 n iy0 ax))
("prolog" nil (p r ow0 l aa1 g))
("prologue" nil (p r ow1 l aa0 g))
("prolong" nil (p r ax l ao1 ng))
("prolonged" nil (p r ax l ao1 ng d))
("prolonging" nil (p r ow0 l ao1 ng ih0 ng))
("prolongs" nil (p r ax l ao1 ng z))
("prom" nil (p r aa1 m))
("promenade" nil (p r aa1 m ax n ey1 d))
("promethium" nil (p r ax m iy1 th iy0 ax m))
("prominence" nil (p r aa1 m ax n ax n s))
("prominences" nil (p r aa1 m ax n ax n s ax z))
("prominent" nil (p r aa1 m ax n ax n t))
("prominently" nil (p r aa1 m ax n ax n t l iy0))
("promiscuity" nil (p r ow1 m ih0 s k y uw1 ax t iy0))
("promiscuous" nil (p r ow1 m ih0 s k w ax s))
("promise" nil (p r aa1 m ax s))
("promised" nil (p r aa1 m ax s t))
("promises" nil (p r aa1 m ax s ax z))
("promising" nil (p r aa1 m ax s ih0 ng))
("promissory" nil (p r aa1 m ax s ao1 r iy0))
("promote" nil (p r ax m ow1 t))
("promoted" nil (p r ax m ow1 t ax d))
("promoter" nil (p r ax m ow1 t er0))
("promoters" nil (p r ax m ow1 t er0 z))
("promotes" nil (p r ax m ow1 t s))
("promoting" nil (p r ax m ow1 t ih0 ng))
("promotion" nil (p r ax m ow1 sh ax n))
("promotional" nil (p r ax m ow1 sh ax n ax l))
("promotions" nil (p r ax m ow1 sh ax n z))
("prompt" nil (p r aa1 m p t))
("prompted" nil (p r aa1 m p t ax d))
("prompting" nil (p r aa1 m p t ih0 ng))
("promptly" nil (p r aa1 m p t l iy0))
("prompts" nil (p r aa1 m p t s))
("promulgate" nil (p r ow0 m ah1 l g ey0 t))
("promulgated" nil (p r aa1 m ax l g ey1 t ax d))
("promulgating" nil (p r aa1 m ax l g ey1 t ih0 ng))
("promus" nil (p r ow1 m ax s))
("prone" nil (p r ow1 n))
("pronged" nil (p r ao1 ng d))
("pronghorn" nil (p r ao1 ng hh ao1 r n))
("pronoun" nil (p r ow1 n aw0 n))
("pronounce" nil (p r ax n aw1 n s))
("pronounced" nil (p r ax n aw1 n s t))
("pronouncement" nil (p r ax n aw1 n s m ax n t))
("pronouncements" nil (p r ax n aw1 n s m ax n t s))
("pronounces" nil (p r ax n aw1 n s ih0 z))
("pronouncing" nil (p r ax n aw1 n s ih0 ng))
("pronouns" nil (p r ow1 n aw0 n z))
("pronovost" nil (p r ow0 n ow1 v ow0 s t))
("pronto" nil (p r aa1 n t ow0))
("pronunciation" nil (p r ax n ah1 n s iy0 ey1 sh ax n))
("proof" nil (p r uw1 f))
("proofed" nil (p r uw1 f t))
("proofing" nil (p r uw1 f ih0 ng))
("proofreading" nil (p r uw1 f r iy1 d ih0 ng))
("proofs" nil (p r uw1 f s))
("prop" nil (p r aa1 p))
("propaganda" nil (p r aa1 p ax g ae1 n d ax))
("propagandist" nil (p r aa1 p ax g ae1 n d ax s t))
("propagandistic" nil (p r aa1 p ax g ax n d ih1 s t ih0 k))
("propagandists" nil (p r aa1 p ax g ae1 n d ax s t s))
("propagandize" nil (p r aa1 p ax g ae1 n d ay1 z))
("propagate" nil (p r aa1 p ax g ey1 t))
("propagated" nil (p r aa1 p ax g ey1 t ih0 d))
("propagating" nil (p r aa1 p ax g ey1 t ih0 ng))
("propagation" nil (p r aa1 p ax g ey1 sh ax n))
("propane" nil (p r ow1 p ey1 n))
("propel" nil (p r ax p eh1 l))
("propellant" nil (p r ax p eh1 l ax n t))
("propellants" nil (p r ow0 p eh1 l ax n t s))
("propelled" nil (p r ax p eh1 l d))
("propeller" nil (p r ax p eh1 l er0))
("propellers" nil (p r ax p eh1 l er0 z))
("propelling" nil (p r ax p eh1 l ih0 ng))
("propels" nil (p r ax p eh1 l z))
("propensities" nil (p r ax p eh1 n s ax t iy0 z))
("propensity" nil (p r ax p eh1 n s ih0 t iy0))
("proper" nil (p r aa1 p er0))
("properly" nil (p r aa1 p er0 l iy0))
("properties" nil (p r aa1 p er0 t iy0 z))
("property" nil (p r aa1 p er0 t iy0))
("propes" nil (p r ow1 p s))
("propfan" nil (p r aa1 p f ae1 n))
("prophecies" nil (p r aa1 f ax s iy0 z))
("prophecy" nil (p r aa1 f ax s iy0))
("prophesied" nil (p r aa1 f ax s ay1 d))
("prophesies" nil (p r aa1 f ax s ay0 z))
("prophet" nil (p r aa1 f ax t))
("prophetess" nil (p r aa1 f ax t ax s))
("prophetic" nil (p r ax f eh1 t ih0 k))
("prophets" nil (p r aa1 f ax t s))
("propionic" nil (p r ow1 p iy0 aa1 n ih0 k))
("propitious" nil (p r ax p ih1 sh ax s))
("proponent" nil (p r ax p ow1 n ax n t))
("proponents" nil (p r ax p ow1 n ax n t s))
("proportion" nil (p r ax p ao1 r sh ax n))
("proportional" nil (p r ax p ao1 r sh ax n ax l))
("proportionally" nil (p r ax p ao1 r sh ax n ax l iy0))
("proportionate" nil (p r ax p ao1 r sh ax n ax t))
("proportionately" nil (p r ax p ao1 r sh ax n ax t l iy0))
("proportioned" nil (p r ax p ao1 r sh ax n d))
("proportions" nil (p r ax p ao1 r sh ax n z))
("proposal" nil (p r ax p ow1 z ax l))
("proposals" nil (p r ax p ow1 z ax l z))
("propose" nil (p r ax p ow1 z))
("proposed" nil (p r ax p ow1 z d))
("proposes" nil (p r ax p ow1 z ih0 z))
("proposing" nil (p r ax p ow1 z ih0 ng))
("proposition" nil (p r aa1 p ax z ih1 sh ax n))
("propositions" nil (p r aa1 p ax z ih1 sh ax n z))
("propounded" nil (p r ax p aw1 n d ax d))
("propp" nil (p r aa1 p))
("propped" nil (p r aa1 p t))
("propper" nil (p r aa1 p er0))
("propping" nil (p r aa1 p ih0 ng))
("propps" nil (p r aa1 p s))
("proprietaries" nil (p r ax p r ay1 ax t eh1 r iy0 z))
("proprietary" nil (p r ax p r ay1 ax t eh1 r iy0))
("proprietor" nil (p r ax p r ay1 ax t er0))
("proprietors" nil (p r ax p r ay1 ax t er0 z))
("proprietorship" nil (p r ax p r ay1 ax t er0 sh ih1 p))
("proprietorships" nil (p r ax p r ay1 ax t er0 sh ih1 p s))
("propriety" nil (p r ax p r ay1 ax t iy0))
("props" nil (p r aa1 p s))
("propst" nil (p r aa1 p s t))
("propulsion" nil (p r ax p ah1 l sh ax n))
("propylene" nil (p r ow0 p ax l iy1 n))
("prorated" nil (p r ow1 r ey1 t ih0 d))
("proration" nil (p r ow1 r ey1 sh ax n))
("prorok" nil (p r ao1 r ax k))
("pros" nil (p r ow1 z))
("prosaic" nil (p r ow0 z ey1 ih0 k))
("proscar" nil (p r ao1 s k aa1 r))
("prosch" nil (p r ao1 sh))
("proscia" nil (p r ow1 s ch ax))
("proscribe" nil (p r ow0 s k r ay1 b))
("proscribed" nil (p r ow0 s k r ay1 b d))
("proscribes" nil (p r ow0 s k r ay1 b z))
("proscription" nil (p r ow0 s k r ih1 p sh ax n))
("prose" nil (p r ow1 z))
("prosecutable" nil (p r aa1 s ih0 k y uw1 t ax b ax l))
("prosecute" nil (p r aa1 s ax k y uw1 t))
("prosecuted" nil (p r aa1 s ih0 k y uw1 t ih0 d))
("prosecutes" nil (p r aa1 s ih0 k y uw1 t s))
("prosecuting" nil (p r aa1 s ih0 k y uw1 t ih0 ng))
("prosecution" nil (p r aa1 s ax k y uw1 sh ax n))
("prosecutions" nil (p r aa1 s ax k y uw1 sh ax n z))
("prosecutor" nil (p r aa1 s ih0 k y uw1 t er0))
("prosecutorial" nil (p r aa1 s ih0 k y uw0 t ao1 r iy0 ax l))
("prosecutors" nil (p r aa1 s ih0 k y uw1 t er0 z))
("prosek" nil (p r ow1 s eh0 k))
("proselytize" nil (p r aa1 s ax l ax t ay1 z))
("proselytized" nil (p r aa1 s ax l ax t ay1 z d))
("proselytizing" nil (p r aa1 s ax l ax t ay1 z ih0 ng))
("proserpina" nil (p r ow0 s er1 p ax n ax))
("proserv" nil (p r ow1 s er1 v))
("proshare" nil (p r ow1 sh eh1 r))
("prosise" nil (p r ow1 s ay0 z))
("proskauer" nil (p r ao1 s k aw0 r))
("proske" nil (p r ow1 s k))
("prosody" nil (p r aa1 s ax d iy0))
("prosorba" nil (p r aa0 s ao1 r b ax))
("prospect" nil (p r aa1 s p eh0 k t))
("prospecting" nil (p r ao1 s p eh1 k t ih0 ng))
("prospective" nil (p r ax s p eh1 k t ih0 v))
("prospectively" nil (p r ax s p eh1 k t ih0 v l iy0))
("prospector" nil (p r ao1 s p eh1 k t er0))
("prospectors" nil (p r ao1 s p eh1 k t er0 z))
("prospects" nil (p r aa1 s p eh0 k t s))
("prospectus" nil (p r ax s p eh1 k t ax s))
("prospectuses" nil (p r ax s p eh1 k t ax s ih0 z))
("prosper" nil (p r aa1 s p er0))
("prospera" nil (p r ow0 s p eh1 r ax))
("prospered" nil (p r aa1 s p er0 d))
("prosperi" nil (p r ow0 s p eh1 r iy0))
("prospering" nil (p r aa1 s p er0 ih0 ng))
("prosperity" nil (p r aa0 s p eh1 r ax t iy0))
("prosperous" nil (p r aa1 s p er0 ax s))
("prospers" nil (p r aa1 s p er0 z))
("pross" nil (p r ao1 s))
("prosser" nil (p r ao1 s er0))
("prost" nil (p r aa1 s t))
("prostaglandin" nil (p r ow0 s t ae0 g l ae1 n d ih0 n))
("prostaglandins" nil (p r ow0 s t ae0 g l ae1 n d ih0 n z))
("prostate" nil (p r aa1 s t ey1 t))
("prostatic" nil (p r ow0 s t ae1 t ih0 k))
("prostheses" nil (p r aa0 s th iy1 s iy0 z))
("prosthesis" nil (p r ao1 s th eh1 s ih0 s))
("prosthetics" nil (p r aa0 s th eh1 t ih0 k s))
("prostitute" nil (p r aa1 s t ax t uw1 t))
("prostitutes" nil (p r aa1 s t ax t uw1 t s))
("prostitution" nil (p r aa1 s t ax t uw1 sh ax n))
("prostrate" nil (p r aa1 s t r ey0 t))
("prostration" nil (p r aa0 s t r ey1 sh ax n))
("protagonist" nil (p r ow0 t ae1 g ax n ax s t))
("protagonists" nil (p r ow0 t ae1 g ax n ax s t s))
("protean" nil (p r ow0 t iy1 ax n))
("protease" nil (p r ow1 t iy0 ey1 z))
("protect" nil (p r ax t eh1 k t))
("protected" nil (p r ax t eh1 k t ax d))
("protecting" nil (p r ax t eh1 k t ih0 ng))
("protection" nil (p r ax t eh1 k sh ax n))
("protectionism" nil (p r ax t eh1 k sh ax n ih1 z ax m))
("protectionist" nil (p r ax t eh1 k sh ax n ih0 s t))
("protectionists" nil (p r ax t eh1 k sh ax n ih0 s t s))
("protections" nil (p r ax t eh1 k sh ax n z))
("protective" nil (p r ax t eh1 k t ih0 v))
("protectively" nil (p r ow0 t eh1 k t ih0 v l iy0))
("protector" nil (p r ax t eh1 k t er0))
("protectorate" nil (p r ax t eh1 k t er0 ax t))
("protectors" nil (p r ax t eh1 k t er0 z))
("protects" nil (p r ax t eh1 k t s))
("protege" nil (p r ow1 t ax zh ey1))
("proteges" nil (p r ow1 t ih0 z ey1 z))
("protein" nil (p r ow1 t iy1 n))
("proteins" nil (p r ow1 t iy1 n z))
("protest" n (p r ow1 t eh0 s t))
("protest" v (p r ow1 t eh1 s t))
("protestant" nil (p r aa1 t ax s t ax n t))
("protestantism" nil (p r aa1 t ax s t ax n t ih1 z ax m))
("protestants" nil (p r aa1 t ax s t ax n t s))
("protestations" nil (p r ow1 t eh1 s t ey1 sh ax n z))
("protested" nil (p r ax t eh1 s t ax d))
("protester" nil (p r ow1 t eh1 s t er0))
("protesters" nil (p r ow1 t eh1 s t er0 z))
("protesting" nil (p r ax t eh1 s t ih0 ng))
("protestors" nil (p r ow1 t eh1 s t er0 z))
("protests" n (p r ow1 t eh0 s t s))
("protests" v (p r ow1 t eh1 s t s))
("prothallus" nil (p r ow1 th ae1 l ax s))
("prothero" nil (p r aa1 dh er0 ow1))
("prothorax" nil (p r ow0 th ao1 r ae0 k s))
("prothro" nil (p r ow1 th r ow0))
("protigal" nil (p r aa1 t ih0 g ax l))
("protium" nil (p r ow1 t iy0 ax m))
("proto" nil (p r ow1 t ax))
("protocol" nil (p r ow1 t ax k aa1 l))
("protocols" nil (p r ow1 t ax k ao1 l z))
("protohistory" nil (p r ow1 t ow0 hh ih1 s t er0 iy0))
("proton" nil (p r ow1 t aa1 n))
("protons" nil (p r ow1 t aa1 n z))
("prototype" nil (p r ow1 t ax t ay1 p))
("prototypes" nil (p r ow1 t ax t ay1 p s))
("prototypical" nil (p r ow1 t ax t ih1 p ih0 k ax l))
("protozoa" nil (p r ow1 t ax z ow1 ax))
("protozoan" nil (p r ow1 t ax z ow1 ax n))
("protozoans" nil (p r ow1 t ax z ow1 ax n z))
("protracted" nil (p r ow0 t r ae1 k t ih0 d))
("protropin" nil (p r aa1 t r ax p ih0 n))
("protrude" nil (p r ow0 t r uw1 d))
("protruding" nil (p r ow0 t r uw1 d ih0 ng))
("protuberances" nil (p r ow0 t uw1 b er0 ax n s ax z))
("protz" nil (p r aa1 t s))
("protzman" nil (p r aa1 t s m ax n))
("proud" nil (p r aw1 d))
("prouder" nil (p r aw1 d er0))
("proudest" nil (p r aw1 d ax s t))
("proudfit" nil (p r aw1 d f ih1 t))
("proudfoot" nil (p r aw1 d f uh1 t))
("proudly" nil (p r aw1 d l iy0))
("prough" nil (p r aw1))
("proulx" nil (p r aw1 l k s))
("prouse" nil (p r aw1 s))
("proust" nil (p r aw1 s t))
("prout" nil (p r aw1 t))
("prouty" nil (p r aw1 t iy0))
("provable" nil (p r uw1 v ax b ax l))
("provance" nil (p r ow1 v ax n s))
("provant" nil (p r ow1 v ax n t))
("prove" nil (p r uw1 v))
("proved" nil (p r uw1 v d))
("proven" nil (p r uw1 v ax n))
("provenance" nil (p r aa1 v ax n ax n s))
("provence" nil (p r ow1 v ax n s))
("provencher" nil (p r aa1 v ih0 n ch er0))
("provencio" nil (p r ow1 v eh1 n s iy0 ow0))
("proventus" nil (p r ow1 v eh1 n t ax s))
("provenza" nil (p r ow1 v eh1 n z ax))
("provenzano" nil (p r ow1 v eh0 n z aa1 n ow0))
("provera" nil (p r ow1 v eh1 r ax))
("proverb" nil (p r ow1 v er0 b))
("proverbial" nil (p r ax v er1 b iy0 ax l))
("proves" nil (p r uw1 v z))
("provide" nil (p r ax v ay1 d))
("provided" nil (p r ax v ay1 d ax d))
("providence" nil (p r aa1 v ax d ax n s))
("provideniya" nil (p r ow1 v ih0 d eh1 n ih0 y ax))
("provident" nil (p r aa1 v ih0 d ax n t))
("providential" nil (p r aa1 v ax d eh1 n ch ax l))
("provider" nil (p r ax v ay1 d er0))
("providers" nil (p r ax v ay1 d er0 z))
("provides" nil (p r ax v ay1 d z))
("providian" nil (p r ow1 v ih1 d iy0 ax n))
("providing" nil (p r ax v ay1 d ih0 ng))
("provigo" nil (p r ow1 v ih1 g ow0))
("province" nil (p r aa1 v ax n s))
("provinces" nil (p r aa1 v ax n s ax z))
("provincetown" nil (p r aa1 v ax n s t aw1 n))
("provincial" nil (p r ax v ih1 n sh ax l))
("provincialism" nil (p r ax v ih1 n ch ax l ih1 z ax m))
("provincially" nil (p r ax v ih1 n sh ax l iy0))
("provine" nil (p r ow0 v iy1 n iy0))
("proving" nil (p r uw1 v ih0 ng))
("provision" nil (p r ax v ih1 zh ax n))
("provisional" nil (p r ax v ih1 zh ax n ax l))
("provisionally" nil (p r ax v ih1 zh ax n ax l iy0))
("provisioning" nil (p r ax v ih1 zh ax n ih0 ng))
("provisions" nil (p r ax v ih1 zh ax n z))
("proviso" nil (p r ax v ay1 z ow0))
("provo" nil (p r ow1 v ow0))
("provocation" nil (p r aa1 v ax k ey1 sh ax n))
("provocations" nil (p r aa1 v ax k ey1 sh ax n z))
("provocative" nil (p r ow0 v aa1 k ax t ih0 v))
("provocatively" nil (p r ow0 v aa1 k ax t ih0 v l iy0))
("provoke" nil (p r ax v ow1 k))
("provoked" nil (p r ax v ow1 k t))
("provokes" nil (p r ax v ow1 k s))
("provoking" nil (p r ax v ow1 k ih0 ng))
("provost" nil (p r ow1 v ow0 s t))
("provosts" nil (p r ow1 v ow0 s t s))
("provus" nil (p r ow1 v ax s))
("prow" nil (p r aw1))
("prowant" nil (p r aw1 ax n t))
("prowell" nil (p r aa1 w eh0 l))
("prowess" nil (p r aw1 ax s))
("prowl" nil (p r aw1 l))
("prowler" nil (p r aw1 l er0))
("prowling" nil (p r aw1 l ih0 ng))
("prows" nil (p r aw1 z))
("prowse" nil (p r aw1 z))
("proxies" nil (p r aa1 k s iy0 z))
("proximal" nil (p r aa1 k s ax m ax l))
("proximate" nil (p r aa1 k s ax m ax t))
("proximity" nil (p r aa0 k s ih1 m ax t iy0))
("proxmire" nil (p r aa1 k s m ay1 r))
("proxy" nil (p r aa1 k s iy0))
("prozac" nil (p r ow1 z ae1 k))
("pru" nil (p r uw1))
("prucapital" nil (p r uw1 k ae1 p ih0 t ax l))
("prucha" nil (p r ah1 ch ax))
("prude" nil (p r uw1 d))
("pruden" nil (p r uw1 d ax n))
("prudence" nil (p r uw1 d ax n s))
("prudency" nil (p r uw1 d ax n s iy0))
("prudent" nil (p r uw1 d ax n t))
("prudente" nil (p r ah1 d ih0 n t))
("prudential" nil (p r uw0 d eh1 n ch ax l))
("prudentialbache" nil (p r uw0 d eh1 n t iy0 ae1 l b ae0 ch))
("prudently" nil (p r uw1 d ax n t l iy0))
("prudentrust" nil (p r uw1 d ax n t r ah1 s t))
("prudhoe" nil (p r ah1 d hh ow1))
("prudhomme" nil (p r ah1 d hh ax m))
("prudish" nil (p r uw1 d ih0 sh))
("prue" nil (p r uw1))
("pruess" nil (p r uw1 s))
("prueter" nil (p r uh1 t er0))
("pruett" nil (p r uw1 t))
("pruette" nil (p r uw0 eh1 t))
("prugh" nil (p r uw1))
("pruiett" nil (p r uw1 ih0 t))
("pruitt" nil (p r uw1 ih0 t))
("prune" nil (p r uw1 n))
("pruneau" nil (p r ax n ow1))
("pruned" nil (p r uw1 n d))
("pruneda" nil (p r uw0 n eh1 d ax))
("prunedale" nil (p r uw1 n d ey1 l))
("prunella" nil (p r uw0 n eh1 l ax))
("pruner" nil (p r uw1 n er0))
("prunes" nil (p r uw1 n z))
("prunier" nil (p r uw1 n iy0 er0))
("pruning" nil (p r uw1 n ih0 ng))
("prunskiene" nil (p r uw1 n s k iy1 n))
("prunty" nil (p r ah1 n t iy0))
("prurient" nil (p r uh1 r iy0 ax n t))
("prus" nil (p r ah1 s))
("prusak" nil (p r uw1 s ax k))
("prusha" nil (p r ah1 sh ax))
("prusinski" nil (p r ax s ih1 n s k iy0))
("pruski" nil (p r ah1 s k iy0))
("pruss" nil (p r ah1 s))
("prussia" nil (p r ah1 sh ax))
("prussian" nil (p r ah1 sh ax n))
("prust" nil (p r ah1 s t))
("prutzman" nil (p r ah1 t s m ax n))
("pruyn" nil (p r ay1 n))
("pry" nil (p r ay1))
("pryce" nil (p r ay1 s))
("pryde" nil (p r ay1 d))
("pryer" nil (p r ay1 er0))
("prying" nil (p r ay1 ih0 ng))
("pryor" nil (p r ay1 er0))
("przybyl" nil (p er0 z ih1 b ax l))
("przybyla" nil (p er1 z ax b ih1 l ax))
("przybylski" nil (p er1 z ax b ih1 l s k iy0))
("przybysz" nil (p er0 z ih1 b ih0 sh))
("przywara" nil (p er0 z ax v aa1 r ax))
("psalm" nil (s aa1 l m))
("psalms" nil (s aa1 l m z))
("psalter" nil (s ao1 l t er0))
("psalters" nil (s ao1 l t er0 z))
("psarouthakis" nil (s eh1 r uw0 th aa1 k ih0 s))
("psencik" nil (s eh1 n s ih0 k))
("pseudo" nil (s uw1 d ow0))
("pseudomonas" nil (s uw1 d ax m ow1 n ax z))
("pseudonym" nil (s uw1 d ax n ih1 m))
("pseudonyms" nil (s uw1 d ax n ih1 m z))
("pseudopodial" nil (s uw1 d ax p ow1 d iy0 ax l))
("pseudoscience" nil (s uw1 d ow0 s ay1 ax n s))
("pseudoscientific" nil (s uw1 d ow0 s ay1 ax n t ih1 f ih0 k))
("psi" nil (s ay1))
("psilocybin" nil (s ay1 l ax s ay1 b ax n))
("psoriasis" nil (s er0 ay1 ax s ax s))
("psych" nil (s ay1 k))
("psyche" nil (s ay1 k iy0))
("psychedelic" nil (s ay1 k ax d eh1 l ih0 k))
("psyches" nil (s ay1 k iy0 z))
("psychiatric" nil (s ay1 k iy0 ae1 t r ih0 k))
("psychiatrist" nil (s ax k ay1 ax t r ax s t))
("psychiatrists" nil (s ax k ay1 ax t r ax s t s))
("psychiatry" nil (s ay0 k ay1 ax t r iy0))
("psychic" nil (s ay1 k ih0 k))
("psychical" nil (s ay1 k ih0 k ax l))
("psychics" nil (s ay1 k ih0 k s))
("psycho" nil (s ay1 k ow0))
("psychoanalysis" nil (s ay1 k ow0 ax n ae1 l ax s ax s))
("psychoanalyst" nil (s ay1 k ow0 ae1 n ax l ax s t))
("psychoanalytic" nil (s ay1 k ow0 ae1 n ax l ih1 t ih0 k))
("psychobabble" nil (s ay1 k ow0 b ae1 b ax l))
("psychodynamic" nil (s ay1 k ow0 d ay0 n ae1 m ih0 k))
("psychogenic" nil (s ay1 k ow0 jh eh1 n ih0 k))
("psychokinesis" nil (s ay1 k ow0 k ax n iy1 s ax s))
("psychological" nil (s ay1 k ax l aa1 jh ih0 k ax l))
("psychologically" nil (s ay1 k ax l aa1 jh ih0 k l iy0))
("psychologist" nil (s ay0 k aa1 l ax jh ax s t))
("psychologists" nil (s ay0 k aa1 l ax jh ax s t s))
("psychology" nil (s ay0 k aa1 l ax jh iy0))
("psychopath" nil (s ay1 k ow0 p ae1 th))
("psychopathic" nil (s ay1 k ax p ae1 th ih0 k))
("psychopaths" nil (s ay1 k ow0 p ae1 th s))
("psychopathy" nil (s ay0 k aa1 p ax th iy0))
("psychosis" nil (s ay0 k ow1 s ax s))
("psychotherapeutic" nil (s ay1 k ow0 th eh1 r ax p y uw1 t ih0 k))
("psychotherapist" nil (s ay1 k ax th eh1 r ax p ih0 s t))
("psychotherapists" nil (s ay1 k ax th eh1 r ax p ih0 s t s))
("psychotherapy" nil (s ay1 k ow0 th eh1 r ax p iy0))
("psychotic" nil (s ay1 k aa1 t ih0 k))
("psyllium" nil (s ih1 l iy0 ax m))
("ptacek" nil (t aa1 ch eh1 k))
("ptak" nil (t ae1 k))
("ptolemaic" nil (t aa1 l ax m ey1 ih0 k))
("ptolemy" nil (t aa1 l ax m iy0))
("ptomaines" nil (t ow1 m ey0 n z))
("pty" nil (t ay1))
("pu" nil (p uw1))
("puat" nil (p y uw1 ae0 t))
("pub" nil (p ah1 b))
("pubco" nil (p ah1 b k ow0))
("puberty" nil (p y uw1 b er0 t iy0))
("public" nil (p ah1 b l ih0 k))
("publication" nil (p ah1 b l ax k ey1 sh ax n))
("publications" nil (p ah1 b l ax k ey1 sh ax n z))
("publicis" nil (p ax b l iy1 s ih0 s))
("publicist" nil (p ah1 b l ih0 s ih0 s t))
("publicists" nil (p ah1 b l ih0 s ih0 s t s))
("publicity" nil (p ax b l ih1 s ax t iy0))
("publicize" nil (p ah1 b l ih0 s ay1 z))
("publicized" nil (p ah1 b l ih0 s ay1 z d))
("publicizing" nil (p ah1 b l ax s ay1 z ih0 ng))
("publicker" nil (p ah1 b l ih0 k er0))
("publicly" nil (p ah1 b l ih0 k l iy0))
("publics" nil (p ah1 b l ih0 k s))
("publish" nil (p ah1 b l ih0 sh))
("published" nil (p ah1 b l ih0 sh t))
("publisher" nil (p ah1 b l ih0 sh er0))
("publishers" nil (p ah1 b l ih0 sh er0 z))
("publishes" nil (p ah1 b l ih0 sh ax z))
("publishing" nil (p ah1 b l ih0 sh ih0 ng))
("publishings" nil (p ah1 b l ih0 sh ih0 ng z))
("publitech" nil (p ah1 b l ih0 t eh1 k))
("publix" nil (p ah1 b l ih0 k s))
("pubs" nil (p ah1 b z))
("pucci" nil (p uw1 ch iy0))
("pucciarelli" nil (p uw0 ch er0 eh1 l iy0))
("puccinelli" nil (p uw0 ch iy0 n eh1 l iy0))
("puccini" nil (p uw0 ch iy1 n iy0))
("puccio" nil (p uw1 ch iy0 ow0))
("puchalski" nil (p ax hh aa1 l s k iy0))
("puche" nil (p uw1 ch))
("pucillo" nil (p uw0 ch ih1 l ow0))
("puck" nil (p ah1 k))
("puckett" nil (p ah1 k ih0 t))
("puckish" nil (p ah1 k ih0 sh))
("pudding" nil (p uh1 d ih0 ng))
("puddings" nil (p uh1 d ih0 ng z))
("puddle" nil (p ah1 d ax l))
("puddles" nil (p ah1 d ax l z))
("puder" nil (p y uw1 d er0))
("pudgies" nil (p ah1 jh iy0 z))
("pudgy" nil (p ah1 jh iy0))
("pudlo" nil (p ah1 d l ow0))
("pudong" nil (p y uw1 d ao1 ng))
("pudwill" nil (p ax d w ih1 l))
("puebla" nil (p w ey1 b l ax))
("pueblo" nil (p w eh1 b l ow0))
("puente" nil (p uw1 n t))
("puentes" nil (p w eh1 n t eh0 s))
("puerile" nil (p y uw0 r iy1 l))
("puerto" nil (p w eh1 r t ow1))
("puett" nil (p uw1 t))
("puetz" nil (p uw1 t s))
("pufahl" nil (p y uw1 f aa0 l))
("puff" nil (p ah1 f))
("puffed" nil (p ah1 f t))
("puffenbarger" nil (p ah1 f ih0 n b aa0 r g er0))
("puffer" nil (p ah1 f er0))
("puffery" nil (p ah1 f er0 iy0))
("puffing" nil (p ah1 f ih0 ng))
("puffs" nil (p ah1 f s))
("puffy" nil (p ah1 f iy0))
("puga" nil (p y uw1 g ax))
("puget" nil (p y uw1 jh ih0 t))
("pugh" nil (p y uw1))
("puglia" nil (p ah1 g l iy0 ax))
("pugliese" nil (p uw0 g l iy0 ey1 z iy0))
("puglisi" nil (p uw0 g l iy1 s iy0))
("pugmire" nil (p ah1 g m ay1 r))
("pugnacious" nil (p ax g n ae1 sh ih0 s))
("pugsley" nil (p ah1 g s l iy0))
("puhl" nil (p ah1 l))
("puig" nil (p uw1 g))
("pujol" nil (p y uw1 jh ax l))
("pulaski" nil (p y uw0 l ae1 s k iy0))
("pulcini" nil (p uw0 l ch iy1 n iy0))
("puleo" nil (p uw1 l iy0 ow0))
("pulford" nil (p uh1 l f er0 d))
("pulice" nil (p uw1 l ih0 s))
("pulido" nil (p uw0 l iy1 d ow0))
("pulitzer" nil (p uh1 l ax t s er0))
("pulkrabek" nil (p ah1 l k r ax b ih0 k))
("pull" nil (p uh1 l))
("pullam" nil (p uh1 l ax m))
("pullan" nil (p uh1 l ax n))
("pullano" nil (p uw0 l aa1 n ow0))
("pullara" nil (p uw0 l aa1 r ax))
("pullback" nil (p uh1 l b ae1 k))
("pullbacks" nil (p uh1 l b ae1 k s))
("pulled" nil (p uh1 l d))
("pullen" nil (p uh1 l ax n))
("puller" nil (p uh1 l er0))
("pulley" nil (p uh1 l iy0))
("pulleys" nil (p uh1 l iy0 z))
("pulliam" nil (p uh1 l iy0 ax m))
("pullin" nil (p uh1 l ih0 n))
("pulling" nil (p uh1 l ih0 ng))
("pullins" nil (p uh1 l ih0 n z))
("pullman" nil (p uh1 l m ax n))
("pullout" nil (p uh1 l aw1 t))
("pullouts" nil (p uh1 l aw1 t s))
("pulls" nil (p uh1 l z))
("pullum" nil (p uh1 l ax m))
("pulmazyme" nil (p uh1 l m ax z ay1 m))
("pulmonary" nil (p uh1 l m ax n eh1 r iy0))
("pulmozyme" nil (p uh1 l m ow0 z ay1 m))
("pulos" nil (p uw1 l ow0 z))
("pulp" nil (p ah1 l p))
("pulping" nil (p ah1 l p ih0 ng))
("pulpit" nil (p uh1 l p ih0 t))
("pulpits" nil (p uh1 l p ih0 t s))
("pulpwood" nil (p ah1 l p w uh1 d))
("puls" nil (p uh1 l z))
("pulsar" nil (p uh1 l s er0))
("pulsating" nil (p ah1 l s ey1 t ih0 ng))
("pulse" nil (p ah1 l s))
("pulses" nil (p ah1 l s ih0 z))
("pulsifer" nil (p ah1 l s ih0 f er0))
("pulsing" nil (p ah1 l s ih0 ng))
("pulsipher" nil (p uh1 l s ih0 f er0))
("pulte" nil (p uh1 l t iy0))
("pultz" nil (p uh1 l t s))
("pulver" nil (p uh1 l v er0))
("pulverize" nil (p ah1 l v er0 ay1 z))
("pulverized" nil (p ah1 l v er0 ay1 z d))
("pulvermacher" nil (p ah1 l v er0 m ax k er0))
("puma" nil (p uw1 m ax))
("pumice" nil (p ah1 m ax s))
("pumilia" nil (p uw0 m iy1 l iy0 ax))
("puming" nil (p uw1 m ih0 ng))
("pummel" nil (p ah1 m ax l))
("pummeled" nil (p ah1 m ax l d))
("pummeling" nil (p ah1 m ax l ih0 ng))
("pummill" nil (p ah1 m ih0 l))
("pump" nil (p ah1 m p))
("pumped" nil (p ah1 m p t))
("pumpernickel" nil (p ah1 m p er0 n ih1 k ax l))
("pumphrey" nil (p ah1 m f r iy0))
("pumping" nil (p ah1 m p ih0 ng))
("pumpkin" nil (p ah1 m p k ih0 n))
("pumpkins" nil (p ah1 m p k ax n z))
("pumps" nil (p ah1 m p s))
("pun" nil (p ah1 n))
("punch" nil (p ah1 n ch))
("punched" nil (p ah1 n ch t))
("punches" nil (p ah1 n ch ih0 z))
("punching" nil (p ah1 n ch ih0 ng))
("punchline" nil (p ah1 n ch l ay1 n))
("punchy" nil (p ah1 n ch iy0))
("punctilious" nil (p ax ng k t ih1 l iy0 ax s))
("punctuality" nil (p ah1 ng k ch uw0 ae1 l ih0 t iy0))
("punctuate" nil (p ah1 ng k ch uw0 ey1 t))
("punctuated" nil (p ah1 ng k ch uw0 ey1 t ih0 d))
("punctuating" nil (p ah1 ng k ch uw0 ey1 d ih0 ng))
("punctuation" nil (p ah1 ng k ch uw0 ey1 sh ax n))
("puncture" nil (p ah1 ng k ch er0))
("punctured" nil (p ah1 ng k ch er0 d))
("punctures" nil (p ah1 ng k ch er0 z))
("puncturing" nil (p ah1 ng k ch er0 ih0 ng))
("pundit" nil (p ah1 n d ax t))
("pundits" nil (p ah1 n d ih0 t s))
("pundt" nil (p ah1 n t))
("puneet" nil (p uw1 n iy1 t))
("pung" nil (p ah1 ng))
("pungent" nil (p ah1 n jh ax n t))
("pungently" nil (p ah1 n jh ax n t l iy0))
("punic" nil (p y uw1 n ih0 k))
("punish" nil (p ah1 n ih0 sh))
("punishable" nil (p ah1 n ih0 sh ax b ax l))
("punished" nil (p ah1 n ih0 sh t))
("punishes" nil (p ah1 n ih0 sh ax z))
("punishing" nil (p ah1 n ih0 sh ih0 ng))
("punishment" nil (p ah1 n ih0 sh m ax n t))
("punishments" nil (p ah1 n ih0 sh m ax n t s))
("punitive" nil (p y uw1 n ax t ih0 v))
("punjab" nil (p ax n jh aa1 b))
("punk" nil (p ah1 ng k))
("punkin" nil (p ah1 ng k ih0 n))
("punks" nil (p ah1 ng k s))
("punky" nil (p ah1 ng k iy0))
("puns" nil (p ah1 n z))
("punt" nil (p ah1 n t))
("punta" nil (p uh1 n t er0))
("punter" nil (p ah1 n t er0))
("punto" nil (p uw1 n t ow1))
("punts" nil (p ah1 n t s))
("puny" nil (p y uw1 n iy0))
("puopolo" nil (p w ow0 p ow1 l ow0))
("pup" nil (p ah1 p))
("pupil" nil (p y uw1 p ax l))
("pupils" nil (p y uw1 p ax l z))
("pupo" nil (p uw1 p ow0))
("puppet" nil (p ah1 p ax t))
("puppets" nil (p ah1 p ax t s))
("puppies" nil (p ah1 p iy0 z))
("puppy" nil (p ah1 p iy0))
("pups" nil (p ah1 p s))
("pur" nil (p er1))
("pura" nil (p uh1 r ax))
("purcell" nil (p er0 s eh1 l))
("purchase" nil (p er1 ch ax s))
("purchased" nil (p er1 ch ax s t))
("purchaser" nil (p er1 ch ax s er0))
("purchasers" nil (p er1 ch ax s er0 z))
("purchases" nil (p er1 ch ax s ax z))
("purchasing" nil (p er1 ch ax s ih0 ng))
("purdie" nil (p er1 d iy0))
("purdin" nil (p er1 d ih0 n))
("purdom" nil (p er1 d ax m))
("purdon" nil (p er1 d ax n))
("purdue" nil (p er0 d uw1))
("purdum" nil (p er1 d ax m))
("purdy" nil (p er1 d iy0))
("pure" nil (p y uh1 r))
("purebred" nil (p y uh1 r b r eh1 d))
("purebreds" nil (p y uh1 r b r eh1 d z))
("puree" nil (p y uh0 r ey1))
("purely" nil (p y uh1 r l iy0))
("purer" nil (p y uh1 r er0))
("purest" nil (p y uh1 r ax s t))
("purex" nil (p y uh1 r eh0 k s))
("purgason" nil (p er1 g ax s ax n))
("purgatory" nil (p er1 g ax t ao1 r iy0))
("purge" nil (p er1 jh))
("purged" nil (p er1 jh d))
("purges" nil (p er1 jh ih0 z))
("purging" nil (p er1 jh ih0 ng))
("puri" nil (p uh1 r iy0))
("puricelli" nil (p uh0 r iy0 ch eh1 l iy0))
("purification" nil (p y uh1 r ax f ax k ey1 sh ax n))
("purified" nil (p y uh1 r ax f ay1 d))
("purifiers" nil (p y uh1 r ax f ay1 er0 z))
("purifoy" nil (p y uw1 r ih0 f oy0))
("purify" nil (p y uh1 r ax f ay1))
("purifying" nil (p y uh1 r ax f ay1 ih0 ng))
("purina" nil (p y uh0 r ih1 n ax))
("purington" nil (p y uh1 r ih0 ng t ax n))
("purinton" nil (p y uw1 r ih0 n t ax n))
("puris" nil (p y uh1 r ih0 s))
("purist" nil (p y uh1 r ih0 s t))
("puristic" nil (p y uh0 r ih1 s t ih0 k))
("purists" nil (p y uh1 r ax s t s))
("puritan" nil (p y uh1 r ax t ax n))
("puritanical" nil (p y uh1 r ax t ae1 n ih0 k ax l))
("puritanism" nil (p y uh1 r ax t ax n ih1 z ax m))
("puritanisms" nil (p y uh1 r ax t ax n ih1 z ax m z))
("puritans" nil (p y uh1 r ax t ax n z))
("purity" nil (p y uh1 r ax t iy0))
("purk" nil (p er1 k))
("purkey" nil (p er1 k iy0))
("purloined" nil (p er0 l oy1 n d))
("purnell" nil (p er1 n ax l))
("purolator" nil (p y uh1 r ax l ey0 t er0))
("purpa" nil (p er1 p ax))
("purple" nil (p er1 p ax l))
("purplish" nil (p er1 p l ih0 sh))
("purport" nil (p er1 p ao1 r t))
("purported" nil (p er0 p ao1 r t ih0 d))
("purportedly" nil (p er0 p ao1 r t ih0 d l iy0))
("purporting" nil (p er0 p ao1 r t ih0 ng))
("purports" nil (p er1 p ao1 r t s))
("purpose" nil (p er1 p ax s))
("purposeful" nil (p er1 p ax s f ax l))
("purposefully" nil (p er1 p ax s f ax l iy0))
("purposeless" nil (p er1 p ax s l ax s))
("purposely" nil (p er1 p ax s l iy0))
("purposes" nil (p er1 p ax s ax z))
("purring" nil (p er1 ih0 ng))
("purrington" nil (p er1 ih0 ng t ax n))
("purrs" nil (p er1 z))
("purse" nil (p er1 s))
("pursed" nil (p er1 s t))
("pursel" nil (p er1 s ax l))
("pursell" nil (p er1 s ax l))
("purser" nil (p er1 s er0))
("purses" nil (p er1 s ih0 z))
("pursifull" nil (p er1 s ih0 f ax l))
("pursley" nil (p er1 s l iy0))
("pursuant" nil (p er0 s uw1 ax n t))
("pursue" nil (p er0 s uw1))
("pursued" nil (p er0 s uw1 d))
("pursuers" nil (p er0 s uw1 er0 z))
("pursues" nil (p er0 s uw1 z))
("pursuing" nil (p er0 s uw1 ih0 ng))
("pursuit" nil (p er0 s uw1 t))
("pursuits" nil (p er0 s uw1 t s))
("purtee" nil (p er1 t iy0))
("purtell" nil (p er1 t ax l))
("purtle" nil (p er1 t ax l))
("purves" nil (p er1 v z))
("purvey" nil (p er0 v ey1))
("purveyor" nil (p er0 v ey1 er0))
("purveyors" nil (p er0 v ey1 er0 z))
("purviance" nil (p uh0 r v iy1 ax n s))
("purview" nil (p er1 v y uw1))
("purvin" nil (p er1 v ih0 n))
("purvis" nil (p er1 v ih0 s))
("puryear" nil (p er1 y ih1 r))
("pusan" nil (p uw1 s aa0 n))
("pusateri" nil (p uw0 s aa0 t eh1 r iy0))
("pusch" nil (p ah1 sh))
("pusey" nil (p y uw1 z iy0))
("push" nil (p uh1 sh))
("pushed" nil (p uh1 sh t))
("pusher" nil (p uh1 sh er0))
("pushers" nil (p uh1 sh er0 z))
("pushes" nil (p uh1 sh ax z))
("pushing" nil (p uh1 sh ih0 ng))
("pushkin" nil (p uh1 sh k ih0 n))
("pushover" nil (p uh1 sh ow1 v er0))
("pushups" nil (p uh1 sh ah1 p s))
("pushy" nil (p uh1 sh iy0))
("puskar" nil (p ah1 s k er0))
("puskarich" nil (p ah1 s k er0 ih0 k))
("puskas" nil (p ah1 s k ax z))
("pussy" nil (p uh1 s iy0))
("pustejovsky" nil (p ax s t ey0 aa1 v s k iy0))
("put" nil (p uh1 t))
("putative" nil (p y uw1 t ax t ih0 v))
("putco" nil (p ah1 t k ow0))
("puterbaugh" nil (p y uw0 t er1 b ao0))
("puth" nil (p uw1 th))
("puthoff" nil (p ah1 t hh ao1 f))
("putman" nil (p ah1 t m ax n))
("putnam" nil (p ah1 t n ax m))
("putney" nil (p ah1 t n iy0))
("putrid" nil (p y uw1 t r ih0 d))
("puts" nil (p uh1 t s))
("putt" nil (p ah1 t))
("puttable" nil (p ah1 t ax b ax l))
("putted" nil (p ah1 t ih0 d))
("putter" nil (p ah1 t er0))
("putterer" nil (p ah1 t er0 er0))
("putterman" nil (p ah1 t er0 m ax n))
("putters" nil (p ah1 t er0 z))
("putting" nil (p ah1 t ih0 ng))
("puttnam" nil (p ah1 t n ax m))
("putts" nil (p ah1 t s))
("putty" nil (p ah1 t iy0))
("putz" nil (p ah1 t s))
("putzier" nil (p ah1 t z iy0 er0))
("puusepp" nil (p uw1 z ax p))
("puza" nil (p uw1 z ax))
("puzio" nil (p uw1 z iy0 ow0))
("puzo" nil (p uw1 z ow0))
("puzzle" nil (p ah1 z ax l))
("puzzled" nil (p ah1 z ax l d))
("puzzlement" nil (p ah1 z ax l m ax n t))
("puzzler" nil (p ah1 z l er0))
("puzzles" nil (p ah1 z ax l z))
("puzzling" nil (p ah1 z ax l ih0 ng))
("puzzo" nil (p uw1 z ow0))
("pyatt" nil (p ay1 ax t))
("pyburn" nil (p ih1 b er0 n))
("pye" nil (p ay1))
("pyeatt" nil (p ay1 ax t))
("pygmalion" nil (p ih1 g m ey1 l y ax n))
("pygmy" nil (p ih1 g m iy0))
("pyka" nil (p ih1 k ax))
("pyke" nil (p ay1 k))
("pyland" nil (p ay1 l ax n d))
("pylant" nil (p y l ao1 n t))
("pyle" nil (p ay1 l))
("pyles" nil (p ay1 l z))
("pylons" nil (p ay1 l aa1 n z))
("pylori" nil (p ay1 l ao1 r iy0))
("pylos" nil (p ay1 l ow0 s))
("pymm" nil (p ih1 m))
("pyne" nil (p ay1 n))
("pynes" nil (p ay1 n z))
("pyongyang" nil (p y ao1 ng y ae1 ng))
("pyper" nil (p ay1 p er0))
("pyramid" nil (p ih1 r ax m ih0 d))
("pyramidal" nil (p er0 ae1 m ax d ax l))
("pyramids" nil (p ih1 r ax m ih0 d z))
("pyrena" nil (p ih0 r iy1 n ax))
("pyrenees" nil (p ih1 r ax iy0 z))
("pyrite" nil (p ay1 r ay0 t))
("pyro" nil (p ay1 r ow0))
("pyron" nil (p ih1 r ax n))
("pyrotechnics" nil (p ay1 r ow0 t eh1 k n ih0 k s))
("pyroxene" nil (p ay0 r aa1 k s iy0 n))
("pyrrhic" nil (p ih1 r ih0 k))
("pysher" nil (p ih1 sh er0))
("pytel" nil (p ih1 t ax l))
("pythia" nil (p ih1 th iy0 ax))
("python" nil (p ay1 th aa0 n))
("pyxis" nil (p ih1 k s ih0 s))
("q" nil (k y uw1))
("qantas" nil (k ae1 n t ax s))
("qatar" nil (k ah1 t aa1 r))
("qawi" nil (k aa1 w iy0))
("qi" nil (k iy1))
("qian" nil (k iy1 aa1 n))
("qiao" nil (k iy0 aw1))
("qiaotou" nil (ch aw1 t uw1))
("qichen" nil (k ih1 ch ih0 n))
("qin" nil (k ih1 n))
("qing" nil (k ih1 ng))
("qingdao" nil (ch ih1 ng d aw1))
("qingming" nil (k ih1 ng m ih1 ng))
("qintex" nil (k ih1 n t eh1 k s))
("qmax" nil (k y uw1 m ae1 k s))
("qom" nil (k aa1 m))
("qua" nil (k w aa1))
("quach" nil (k w aa1 ch))
("quack" nil (k w ae1 k))
("quackenbush" nil (k w ae1 k ax n b uh1 sh))
("quackery" nil (k w ae1 k er0 iy0))
("quacks" nil (k w ae1 k s))
("quad" nil (k w aa1 d))
("quade" nil (k w ey1 d))
("quadra" nil (k w ae1 d r ax))
("quadrant" nil (k w aa1 d r ax n t))
("quadrennial" nil (k w aa0 d r eh1 n iy0 ax l))
("quadrex" nil (k w aa1 d r eh0 k s))
("quadriceps" nil (k w aa1 d r ax s eh1 p s))
("quadriplegic" nil (k w aa1 d r ax p l iy1 jh ih0 k))
("quadruple" nil (k w aa0 d r uw1 p ax l))
("quadrupled" nil (k w aa0 d r uw1 p ax l d))
("quadrupling" nil (k w aa0 d r uw1 p ax l ih0 ng))
("quaglia" nil (k w ae1 g l iy0 ax))
("quagmire" nil (k w ae1 g m ay1 er0))
("quai" nil (k iy1))
("quaid" nil (k w ey1 d))
("quail" nil (k w ey1 l))
("quails" nil (k w ey1 l z))
("quain" nil (k w ey1 n))
("quaint" nil (k w ey1 n t))
("quaintance" nil (k w ey1 n t ax n s))
("quaintly" nil (k w ey1 n t l iy0))
("quake" nil (k w ey1 k))
("quakenbush" nil (k w ax k eh1 n b ax sh))
("quaker" nil (k w ey1 k er0))
("quakes" nil (k w ey1 k s))
("quaking" nil (k w ey1 k ih0 ng))
("qual" nil (k w aa1 l))
("qualcast" nil (k w aa1 l k ae1 s t))
("qualcomm" nil (k w aa1 l k aa1 m))
("quale" nil (k w ey1 l))
("qualex" nil (k w aa1 l eh0 k s))
("qualey" nil (k w ey1 l iy0))
("qualification" nil (k w aa1 l ax f ax k ey1 sh ax n))
("qualifications" nil (k w aa1 l ax f ax k ey1 sh ax n z))
("qualified" nil (k w aa1 l ax f ay1 d))
("qualifier" nil (k w aa1 l ax f ay1 er0))
("qualifies" nil (k w aa1 l ax f ay1 z))
("qualify" nil (k w aa1 l ax f ay1))
("qualifying" nil (k w aa1 l ax f ay1 ih0 ng))
("qualitative" nil (k w aa1 l ax t ey1 t ih0 v))
("qualities" nil (k w aa1 l ax t iy0 z))
("quality" nil (k w aa1 l ax t iy0))
("qualley" nil (k w ao1 l iy0))
("qualls" nil (k w aa1 l z))
("qualms" nil (k w aa1 m z))
("quam" nil (k w aa1 m))
("quamme" nil (k w ae1 m))
("quan" nil (k w ao1 n))
("quandaries" nil (k w aa1 n d er0 iy0 z))
("quandary" nil (k w aa1 n d er0 iy0))
("quandt" nil (k w ao1 n d t))
("quanex" nil (k w aa1 n eh0 k s))
("quang" nil (k w ao1 ng))
("quant" nil (k w ae1 n t))
("quantico" nil (k w aa1 n t ih0 k ow0))
("quantified" nil (k w aa1 n t ih0 f ay1 d))
("quantify" nil (k w aa1 n t ih0 f ay1))
("quantifying" nil (k w aa1 n t ih0 f ay1 ih0 ng))
("quantitative" nil (k w aa1 n t ih0 t ey1 t ih0 v))
("quantities" nil (k w aa1 n t ax t iy0 z))
("quantity" nil (k w aa1 n t ax t iy0))
("quantum" nil (k w aa1 n t ax m))
("quaquil" nil (k w ae1 k w ih0 l))
("quaranta" nil (k w er0 ae1 n t ax))
("quarante" nil (k w er0 aa1 n t ey0))
("quarantine" nil (k w ao1 r ax n t iy1 n))
("quarantined" nil (k w ao1 r ax n t iy1 n d))
("quarantines" nil (k w ao1 r ax n t iy1 n z))
("quarantining" nil (k w ao1 r ax n t iy1 n ih0 ng))
("quark" nil (k w aa1 r k))
("quarks" nil (k w aa1 r k s))
("quarles" nil (k w ao1 r l z))
("quarnstrom" nil (k w ao1 r n s t r ax m))
("quarre" nil (k w aa1 r))
("quarrel" nil (k w ao1 r ax l))
("quarreled" nil (k w ao1 r ax l d))
("quarreling" nil (k w aa1 r ax l ih0 ng))
("quarrels" nil (k w ao1 r ax l z))
("quarrelsome" nil (k w ao1 r ax l s ax m))
("quarries" nil (k w ao1 r iy0 z))
("quarry" nil (k w ao1 r iy0))
("quarrying" nil (k w ao1 r iy0 ih0 ng))
("quart" nil (k w ao1 r t))
("quartararo" nil (k w aa0 r t aa0 r aa1 r ow0))
("quarter" nil (k w ao1 r t er0))
("quarterback" nil (k w ao1 r t er0 b ae1 k))
("quarterbacking" nil (k w ao1 r t er0 b ae1 k ih0 ng))
("quarterbacks" nil (k w ao1 r t er0 b ae1 k s))
("quarterdeck" nil (k w ao1 r t er0 d eh1 k))
("quarterly" nil (k w ao1 r t er0 l iy0))
("quarterman" nil (k w ao1 r t er0 m ax n))
("quarters" nil (k w ao1 r t er0 z))
("quartet" nil (k w ao0 r t eh1 t))
("quartets" nil (k w ao0 r t eh1 t s))
("quartile" nil (k w ao1 r t ih0 l))
("quarto" nil (k w ao1 r t ow0))
("quarts" nil (k w ao1 r t s))
("quartz" nil (k w ao1 r t s))
("quasar" nil (k w ey1 z aa0 r))
("quash" nil (k w ao1 sh))
("quashed" nil (k w ao1 sh t))
("quashing" nil (k w ao1 sh ih0 ng))
("quasi" nil (k w aa1 s iy0))
("quassia" nil (k w aa1 sh iy0 ax))
("quast" nil (k w aa1 s t))
("quaternary" nil (k w aa1 t er0 n eh1 r iy0))
("quattlebaum" nil (k w ao1 t ax l b aw0 m))
("quattro" nil (k w ao1 t r ow0))
("quattrocchi" nil (k w aa0 t r ow1 k iy0))
("quattrochi" nil (k w aa0 t r ow1 k iy0))
("quattrone" nil (k w aa0 t r ow1 n iy0))
("quave" nil (k w ey1 v))
("quay" nil (k iy1))
("quayle" nil (k w ey1 l))
("quazulu" nil (k w aa1 z uw1 l uw0))
("quddus" nil (k uw1 d uw1 s))
("que" nil (k y uw1))
("quealy" nil (k w iy1 l iy0))
("queasiness" nil (k w iy1 z iy0 n ax s))
("queasy" nil (k w iy1 z iy0))
("queau" nil (k w ow1))
("quebec" nil (k w ax b eh1 k))
("quebeckers" nil (k w eh1 b eh1 k er0 z))
("quebecois" nil (k w eh1 b ax s w aa1))
("quebecoise" nil (k eh1 b eh0 k w aa1))
("quebecor" nil (k w eh1 b ih0 k ao0 r))
("quebedeaux" nil (k w eh1 b ih0 d ow0))
("quechee" nil (k w eh1 ch iy0))
("queen" nil (k w iy1 n))
("queena" nil (k w iy1 n ax))
("queenan" nil (k w iy1 n ax n))
("queener" nil (k w iy1 n er0))
("queenfish" nil (k w iy1 n f ih1 sh))
("queenie" nil (k w iy1 n iy0))
("queens" nil (k w iy1 n z))
("queensberry" nil (k w iy1 n z b eh1 r iy0))
("queensland" nil (k w iy1 n z l ae1 n d))
("queensway" nil (k w iy1 n z w ey1))
("queer" nil (k w ih1 r))
("queerer" nil (k w ih1 r er0))
("quell" nil (k w eh1 l))
("quelled" nil (k w eh1 l d))
("quellette" nil (k w ih0 l eh1 t))
("quelling" nil (k w eh1 l ih0 ng))
("quello" nil (k w eh1 l ow0))
("quemoy" nil (k w ih0 m oy1))
("quenby" nil (k w eh1 n b iy0))
("quench" nil (k w eh1 n ch))
("quenching" nil (k w eh1 n ch ih0 ng))
("quennel" nil (k w eh1 n ax l))
("quenneville" nil (k w ey0 n ey0 v ih1 l))
("quentin" nil (k w eh1 n t ih0 n))
("quenzer" nil (k w eh1 n z er0))
("querida" nil (k w eh0 r iy1 d ax))
("queried" nil (k w ih1 r iy0 d))
("queries" nil (k w ih1 r iy0 z))
("querns" nil (k w er1 n z))
("querry" nil (k w eh1 r iy0))
("querulous" nil (k w eh1 r ax l ax s))
("query" nil (k w iy1 r iy0))
("quesada" nil (k w ey0 s aa1 d ax))
("quesenberry" nil (k w iy1 z ax n b eh0 r iy0))
("quesinberry" nil (k w eh1 s ih0 n b eh0 r iy0))
("quesnel" nil (k w ey0 s n eh1 l))
("quesnell" nil (k w ey0 s n ey1 l))
("quest" nil (k w eh1 s t))
("questar" nil (k w eh1 s t er0))
("questech" nil (k w eh1 s t eh1 k))
("question" nil (k w eh1 s ch ax n))
("questionable" nil (k w eh1 s ch ax n ax b ax l))
("questioned" nil (k w eh1 s ch ax n d))
("questioner" nil (k w eh1 s ch ax n er0))
("questioners" nil (k w eh1 s ch ax n er0 z))
("questioning" nil (k w eh1 s ch ax n ih0 ng))
("questionnaire" nil (k w eh1 s ch ax n eh1 r))
("questionnaires" nil (k w eh1 s ch ax n eh1 r z))
("questions" nil (k w eh1 s ch ax n z))
("questran" nil (k w eh1 s t r ae1 n))
("questrom" nil (k w eh1 s t r ax m))
("quests" nil (k w eh1 s t s))
("queue" nil (k y uw1))
("queues" nil (k y uw1 z))
("queuing" nil (k y uw1 ih0 ng))
("quevedo" nil (k w ey0 v ey1 d ow0))
("quezada" nil (k w ey0 z aa1 d ax))
("qui" nil (k w iy1))
("quibble" nil (k w ih1 b ax l))
("quibbles" nil (k w ih1 b ax l z))
("quibbling" nil (k w ih1 b ax l ih0 ng))
("quiche" nil (k iy1 sh))
("quick" nil (k w ih1 k))
("quickel" nil (k w ih1 k ax l))
("quicken" nil (k w ih1 k ax n))
("quickened" nil (k w ih1 k ax n d))
("quickening" nil (k w ih1 k ax n ih0 ng))
("quickens" nil (k w ih1 k ax n z))
("quicker" nil (k w ih1 k er0))
("quickest" nil (k w ih1 k ih0 s t))
("quickie" nil (k w ih1 k iy0))
("quickly" nil (k w ih1 k l iy0))
("quickness" nil (k w ih1 k n ax s))
("quicksand" nil (k w ih1 k s ae1 n d))
("quicksands" nil (k w ih1 k s ae1 n d z))
("quicksilver" nil (k w ih1 k s ih1 l v er0))
("quicktime" nil (k w ih1 k t ay1 m))
("quid" nil (k w ih1 d))
("quiescent" nil (k w ay0 eh1 s ax n t))
("quiet" nil (k w ay1 ax t))
("quieted" nil (k w ay1 ax t ax d))
("quieter" nil (k w ay1 ax t er0))
("quietest" nil (k w ay1 ax t ax s t))
("quieting" nil (k w ay1 ax t ih0 ng))
("quietist" nil (k w ay1 ax t ax s t))
("quietly" nil (k w ay1 ax t l iy0))
("quietness" nil (k w ay1 ax t n ax s))
("quiett" nil (k w iy1 t))
("quigg" nil (k w ih1 g))
("quiggle" nil (k w ih1 g ax l))
("quigley" nil (k w ih1 g l iy0))
("quijada" nil (k w iy0 y aa1 d ax))
("quijano" nil (k w iy0 y aa1 n ow0))
("quik" nil (k w ih1 k))
("quiksilver" nil (k w ih1 k s ih1 l v er0))
("quilali" nil (k w ih0 l aa1 l iy0))
("quiles" nil (k w iy1 l eh0 s))
("quilici" nil (k w iy0 l iy1 ch iy0))
("quill" nil (k w ih1 l))
("quillan" nil (k w ih1 l ax n))
("quillen" nil (k w ih1 l ax n))
("quillian" nil (k w ih1 l y ax n))
("quillin" nil (k w ih1 l ih0 n))
("quilling" nil (k w ih1 l ih0 ng))
("quillman" nil (k w iy0 l m ae1 n))
("quillon" nil (k w ih1 l ax n))
("quilmes" nil (k w ih1 l m z))
("quilt" nil (k w ih1 l t))
("quilted" nil (k w ih1 l t ih0 d))
("quilter" nil (k w ih1 l t er0))
("quilters" nil (k w ih1 l t er0 z))
("quilting" nil (k w ih1 l t ih0 ng))
("quilts" nil (k w ih1 l t s))
("quilty" nil (k w ih1 l t iy0))
("quimby" nil (k w ih1 m b iy0))
("quin" nil (k w ih1 n))
("quina" nil (k w iy1 n ax))
("quinby" nil (k w ih1 n b iy0))
("quince" nil (k w ih1 n s))
("quincy" nil (k w ih1 n s iy0))
("quindlen" nil (k w ih1 n d l ax n))
("quine" nil (k w ay1 n))
("quinine" nil (k w ay1 n ay1 n))
("quinlan" nil (k w ih1 n l ax n))
("quinley" nil (k w ih1 n l iy0))
("quinlin" nil (k w ih1 n l ih0 n))
("quinlivan" nil (k w ih1 n l ih0 v ax n))
("quinn" nil (k w ih1 n))
("quinnell" nil (k w ih1 n ax l))
("quinney" nil (k w ih1 n iy0))
("quinobequin" nil (k w ih1 n ow1 b ih0 k w ih0 n))
("quinoco" nil (k w ih0 n ow1 k ow0))
("quinon" nil (k w ih1 n ax n))
("quinones" nil (k w ih0 n ow1 n z))
("quinonez" nil (k w iy0 n ow1 n eh0 z))
("quint" nil (k w ih1 n t))
("quinta" nil (k w ih1 n t ax))
("quintal" nil (k w ih1 n t ax l))
("quintana" nil (k w iy0 n t ae1 n ax))
("quintanar" nil (k w ih1 n t ax n er0))
("quintanilla" nil (k w ih1 n t ax n ih1 l ax))
("quintela" nil (k w iy0 n t ey1 l ax))
("quinter" nil (k w ih1 n t er0))
("quintero" nil (k w iy0 n t eh1 r ow0))
("quinteros" nil (k w iy0 n t eh1 r ow0 z))
("quintessence" nil (k w ih0 n t eh1 s ax n s))
("quintessential" nil (k w ih1 n t ih0 s eh1 n sh ax l))
("quintessentially" nil (k w ih1 n t ax s eh1 n ch ax l iy0))
("quintet" nil (k w ih0 n t eh1 t))
("quintile" nil (k w ih1 n t ih0 l))
("quintin" nil (k w ih1 n t ih0 n))
("quintina" nil (k w iy0 n t iy1 n ax))
("quinto" nil (k w ih1 n t ow0))
("quinton" nil (k w ih0 n t ao1 n))
("quints" nil (k w ih1 n t s))
("quintuple" nil (k w ih1 n t uw0 p ax l))
("quintupled" nil (k w ih0 n t y uw1 p ax l d))
("quip" nil (k w ih1 p))
("quipp" nil (k w ih1 p))
("quipped" nil (k w ih1 p t))
("quips" nil (k w ih1 p s))
("quiram" nil (k w ao1 r ax m))
("quire" nil (k w ay1 r))
("quirin" nil (k w ih1 r ih0 n))
("quiring" nil (k w ay1 r ih0 ng))
("quirion" nil (k w ih1 r iy0 ax n))
("quirk" nil (k w er1 k))
("quirke" nil (k w er1 k))
("quirks" nil (k w er1 k s))
("quirky" nil (k w er1 k iy0))
("quiroga" nil (k w ih0 r ow1 g ax))
("quiros" nil (k w ih1 r ow0 z))
("quiroz" nil (k w ih1 r ow0 z))
("quisenberry" nil (k w ay1 z ax n b eh0 r iy0))
("quisling" nil (k w ih1 z l ih0 ng))
("quist" nil (k w ih1 s t))
("quit" nil (k w ih1 t))
("quite" nil (k w ay1 t))
("quito" nil (k w iy1 t ow0))
("quits" nil (k w ih1 t s))
("quitter" nil (k w ih1 t er0))
("quitters" nil (k w ih1 t er0 z))
("quitting" nil (k w ih1 t ih0 ng))
("quiver" nil (k w ih1 v er0))
("quivering" nil (k w ih1 v er0 ih0 ng))
("quixote" nil (k iy0 hh ow1 t iy0))
("quixotic" nil (k w ih0 k s aa1 t ih0 k))
("quiz" nil (k w ih1 z))
("quizzed" nil (k w ih1 z d))
("quizzes" nil (k w ih1 z ih0 z))
("quo" nil (k w ow1))
("quon" nil (k w aa1 n))
("quorum" nil (k w ao1 r ax m))
("quorums" nil (k w ao1 r ax m z))
("quota" nil (k w ow1 t ax))
("quotable" nil (k w ow1 t ax b ax l))
("quotas" nil (k w ow1 t ax z))
("quotation" nil (k w ow0 t ey1 sh ax n))
("quotations" nil (k w ow0 t ey1 sh ax n z))
("quote" nil (k w ow1 t))
("quoted" nil (k w ow1 t ax d))
("quotes" nil (k w ow1 t s))
("quotient" nil (k w ow1 sh ax n t))
("quoting" nil (k w ow1 t ih0 ng))
("quotron" nil (k w aa1 t r ax n))
("qureshey" nil (k uh0 r ey1 sh ey0))
("qureshi" nil (k uh0 r ey1 sh iy0))
("r" nil (aa1 r))
("raab" nil (r aa1 b))
("raabe" nil (r aa1 b))
("raad" nil (r aa1 d))
("raap" nil (r aa1 p))
("raasch" nil (r aa1 sh))
("raatz" nil (r aa1 t s))
("rab" nil (r ae1 b))
("rabago" nil (r aa0 b aa1 g ow0))
("rabalais" nil (r ae1 b ax l ey1))
("rabb" nil (r ae1 b))
("rabbi" nil (r ae1 b ay1))
("rabbinical" nil (r ax b ih1 n ih0 k ax l))
("rabbis" nil (r ae1 b ay1 z))
("rabbit" nil (r ae1 b ax t))
("rabbitlike" nil (r ae1 b ax t l ay1 k))
("rabbits" nil (r ae1 b ax t s))
("rabbitt" nil (r ae1 b ih0 t))
("rabble" nil (r ae1 b ax l))
("rabe" nil (r ey1 b))
("rabel" nil (r ae1 b ax l))
("raben" nil (r ae1 b ax n))
("rabenold" nil (r ae1 b ih0 n ow1 l d))
("raber" nil (r ey1 b er0))
("rabey" nil (r ey1 b iy0))
("rabi" nil (r aa1 b iy0))
("rabid" nil (r ae1 b ih0 d))
("rabideau" nil (r ae1 b ih0 d ow1))
("rabies" nil (r ey1 b iy0 z))
("rabin" nil (r aa1 b iy1 n))
("rabine" nil (r ax b iy1 n))
("rabiner" nil (r ax b iy1 n er0))
("rabinovich" nil (r ax b ih1 n ax v ih0 ch))
("rabinovitz" nil (r ax b ih1 n ax v ih0 t s))
("rabinowitz" nil (r ax b ih1 n ax w ih0 t s))
("rabkin" nil (r ae1 b k ih0 n))
("rabobank" nil (r aa1 b ow0 b ae1 ng k))
("raboin" nil (r ax b oy1 n))
("rabold" nil (r ae1 b ow0 l d))
("rabon" nil (r aa0 b ao1 n))
("raborn" nil (r ae1 b er0 n))
("rabourn" nil (r ax b uh1 r n))
("raboy" nil (r ae1 b oy0))
("rabuck" nil (r ae1 b ax k))
("rabuka" nil (r ax b uw1 k ax))
("rabun" nil (r ae1 b ax n))
("raburn" nil (r ae1 b er0 n))
("raby" nil (r ey1 b iy0))
("racal" nil (r ae1 k ax l))
("racamier" nil (r ae1 k ax m ay1 r))
("racanelli" nil (r aa0 k aa0 n eh1 l iy0))
("racca" nil (r ae1 k ax))
("raccoon" nil (r ae0 k uw1 n))
("raccoons" nil (r ae0 k uw1 n z))
("race" nil (r ey1 s))
("raced" nil (r ey1 s t))
("racehorse" nil (r ey1 s hh ao1 r s))
("racehorses" nil (r ae1 s hh ao1 r s ih0 z))
("racemes" nil (r ey0 s iy1 m z))
("racer" nil (r ey1 s er0))
("racers" nil (r ey1 s er0 z))
("races" nil (r ey1 s ax z))
("racetrack" nil (r ey1 s t r ae1 k))
("racetracks" nil (r ey1 s t r ae1 k s))
("racette" nil (r ax s eh1 t))
("raceway" nil (r ey1 s w ey1))
("racey" nil (r ey1 s iy0))
("rach" nil (r ae1 ch))
("rachad" nil (r ax sh aa1 d))
("rachal" nil (r ae1 k ax l))
("rachel" nil (r ey1 ch ax l))
("rachelle" nil (r ax sh eh1 l))
("rachels" nil (r ey1 ch ih0 l z))
("rachford" nil (r ae1 ch f er0 d))
("rachlin" nil (r ae1 k l ih0 n))
("rachmaninoff" nil (r ae0 k m ae1 n ih0 n ao0 f))
("racial" nil (r ey1 sh ax l))
("racialism" nil (r ey1 sh ax l ih1 z ax m))
("racially" nil (r ey1 sh ax l iy0))
("racicot" nil (r ae1 s ih0 k aa0 t))
("racine" nil (r ax s iy1 n))
("raciness" nil (r ey1 s iy0 n ax s))
("racing" nil (r ey1 s ih0 ng))
("racioppi" nil (r aa0 ch ow1 p iy0))
("racism" nil (r ey1 s ih1 z ax m))
("racist" nil (r ey1 s ih0 s t))
("racists" nil (r ey1 s ih0 s t s))
("rack" nil (r ae1 k))
("racked" nil (r ae1 k t))
("rackers" nil (r ae1 k er0 z))
("racket" nil (r ae1 k ih0 t))
("racketeer" nil (r ae1 k ih0 t ih1 r))
("racketeering" nil (r ae1 k ih0 t ih1 r ih0 ng))
("racketeers" nil (r ae1 k ax t ih1 r z))
("rackets" nil (r ae1 k ax t s))
("racking" nil (r ae1 k ih0 ng))
("rackley" nil (r ae1 k l iy0))
("rackliff" nil (r ae1 k l ih0 f))
("rackliffe" nil (r ae1 k l ih0 f))
("rackmil" nil (r ae1 k m ih0 l))
("rackow" nil (r aa1 s k ow0))
("racks" nil (r ae1 k s))
("raconteur" nil (r ae1 k aa0 n t uw1 r))
("racquet" nil (r ae1 k ih0 t))
("racquetball" nil (r ae1 k ax t b ao1 l))
("racquets" nil (r ae1 k ih0 t s))
("racy" nil (r ey1 s iy0))
("racz" nil (r aa1 ch))
("raczka" nil (r aa1 ch k ax))
("raczkowski" nil (r ax ch k ao1 f s k iy0))
("raczynski" nil (r ax ch ih1 n s k iy0))
("rad" nil (r ae1 d))
("rada" nil (r aa1 d ax))
("radabaugh" nil (r ae1 d ax b ao0))
("radakovich" nil (r ax d ae1 k ax v ih0 ch))
("radant" nil (r aa1 d ax n t))
("radar" nil (r ey1 d aa1 r))
("radars" nil (r ey1 d aa1 r z))
("radatz" nil (r ae1 d ax t s))
("radbert" nil (r ae1 d b er0 t))
("radborne" nil (r ax d b ao1 r n))
("radbourn" nil (r ax d b uh1 r n))
("radbourne" nil (r ax d b uh1 r n))
("radburn" nil (r ae1 d b er0 n))
("radcliff" nil (r ae1 d k l ih0 f))
("radcliffe" nil (r ae1 d k l ih0 f))
("raddatz" nil (r ae1 d ax t s))
("radde" nil (r ae1 d))
("radder" nil (r ae1 d er0))
("radebaugh" nil (r ae1 d ih0 b ao0))
("radecki" nil (r ax d eh1 k iy0))
("radel" nil (r ae1 d ax l))
("radell" nil (r aa0 d ey1 l))
("radella" nil (r ax d eh1 l ax))
("rademacher" nil (r ae1 d ih0 m ax k er0))
("rademaker" nil (r ey1 d m ey0 k er0))
("raden" nil (r ey1 d ax n))
("rader" nil (r ey1 d er0))
("radermacher" nil (r ae1 d er0 m ax k er0))
("radford" nil (r ae1 d f er0 d))
("radha" nil (r aa1 d ax))
("radi" nil (r aa1 d iy0))
("radial" nil (r ey1 d iy0 ax l))
("radially" nil (r ey1 d iy0 ax l iy0))
("radials" nil (r ey1 d iy0 ax l z))
("radiant" nil (r ey1 d iy0 ax n t))
("radiate" nil (r ey1 d iy0 ax t))
("radiated" nil (r ey1 d iy0 ey1 t ax d))
("radiates" nil (r ey1 d iy0 ey1 t s))
("radiating" nil (r ey1 d iy0 ey1 t ih0 ng))
("radiation" nil (r ey1 d iy0 ey1 sh ax n))
("radiator" nil (r ey1 d iy0 ey1 t er0))
("radiators" nil (r ey1 d iy0 ey1 t er0 z))
("radical" nil (r ae1 d ax k ax l))
("radicalism" nil (r ae1 d ih0 k ax l ih1 z ax m))
("radicalization" nil (r ae1 d ih0 k ax l ih0 z ey1 sh ax n))
("radicalized" nil (r ae1 d ax k ax l ay1 z d))
("radically" nil (r ae1 d ih0 k l iy0))
("radicals" nil (r ae1 d ax k ax l z))
("radice" nil (r ae1 d ih0 s))
("radich" nil (r ae1 d ih0 k))
("radick" nil (r ae1 d ih0 k))
("radin" nil (r ae1 d ih0 n))
("radinka" nil (r ax d ih1 ng k ax))
("radio" nil (r ey1 d iy0 ow1))
("radioactive" nil (r ey1 d iy0 ow0 ae1 k t ih0 v))
("radioactivity" nil (r ey1 d iy0 ow0 ae0 k t ih1 v ax t iy0))
("radioed" nil (r ey1 d iy0 ow1 d))
("radiography" nil (r ey1 d iy0 aa1 g r ax f iy0))
("radiological" nil (r ey1 d iy0 ax l aa1 jh ih0 k ax l))
("radiologist" nil (r ey1 d iy0 aa1 l ax jh ih0 s t))
("radiologists" nil (r ey1 d iy0 aa1 l ax jh ih0 s t s))
("radiology" nil (r ey1 d iy0 aa1 l ax jh iy0))
("radios" nil (r ey1 d iy0 ow1 z))
("radish" nil (r ae1 d ih0 sh))
("radishes" nil (r ae1 d ih0 sh ih0 z))
("radisson" nil (r ae1 d ax s ax n))
("radium" nil (r ey1 d iy0 ax m))
("radius" nil (r ey1 d iy0 ax s))
("radke" nil (r ey1 d k iy0))
("radle" nil (r ey1 d ax l))
("radler" nil (r ey1 d ax l er0))
("radley" nil (r ae1 d l iy0))
("radliff" nil (r ae1 d l ih0 f))
("radloff" nil (r ae1 d l ao0 f))
("radman" nil (r ae1 d m ax n))
("radmilla" nil (r ae1 d m ih1 l ax))
("radmund" nil (r ae1 d m ax n d))
("radney" nil (r ae1 d n iy0))
("radnor" nil (r ae1 d n er0))
("rado" nil (r aa1 d ow0))
("radolf" nil (r ae1 d ow0 l f))
("radomski" nil (r ax d aa1 m s k iy0))
("radon" nil (r ey1 d aa1 n))
("rados" nil (r aa1 d ow0 z))
("radosevich" nil (r ax d aa1 s ih0 v ih0 ch))
("radosh" nil (r ax d ao1 sh))
("radovan" nil (r aa1 d ow1 v aa1 n))
("radowski" nil (r ax d ow1 s k iy0))
("radtke" nil (r ae1 d k iy0))
("radu" nil (r aa1 d uw0))
("radwan" nil (r ae1 d w ax n))
("rady" nil (r ey1 d iy0))
("radziewicz" nil (r aa1 jh ax v ih0 ch))
("radzik" nil (r ae1 d z ih0 k))
("rae" nil (r ey1))
("raeburn" nil (r ey1 b er0 n))
("raeder" nil (r eh1 d er0))
("rael" nil (r ey1 l))
("raether" nil (r eh1 dh er0))
("raetz" nil (r iy1 t s))
("raf" nil (r ae1 f))
("rafael" nil (r aa1 f ay0 eh1 l))
("rafale" nil (r ax f ey1 l))
("rafalski" nil (r ax f aa1 l s k iy0))
("rafe" nil (r ey1 f))
("rafelghem" nil (r ax f eh1 l g ax m))
("raff" nil (r ae1 f))
("raffa" nil (r ae1 f ax))
("raffaele" nil (r aa0 f ay0 eh1 l ey0))
("raffaelli" nil (r aa0 f aa0 eh1 l iy0))
("raffel" nil (r ae1 f ax l))
("raffensperger" nil (r ae1 f ih0 n s p er0 g er0))
("rafferty" nil (r ae1 f er0 t iy0))
("raffetto" nil (r aa0 f eh1 t ow0))
("raffety" nil (r ae1 f t iy0))
("raffi" nil (r ae1 f iy0))
("raffield" nil (r ae1 f iy1 l d))
("raffish" nil (r ae1 f ih0 sh))
("raffle" nil (r ae1 f ax l))
("raffles" nil (r ae1 f ax l z))
("raffo" nil (r ae1 f ow0))
("rafi" nil (r ae1 f iy0))
("rafsanjani" nil (r ae1 f s ax n jh aa1 n iy0))
("raft" nil (r ae1 f t))
("rafted" nil (r ae1 f t ax d))
("rafter" nil (r ae1 f t er0))
("rafters" nil (r ae1 f t er0 z))
("raftery" nil (r ae1 f t er0 iy0))
("rafting" nil (r ae1 f t ih0 ng))
("rafts" nil (r ae1 f t s))
("rafuse" nil (r aa0 f uw1 s iy0))
("rag" nil (r ae1 g))
("ragain" nil (r ae1 g ax n))
("ragan" nil (r ey1 g ax n))
("ragas" nil (r aa1 g ax z))
("ragavan" nil (r aa1 g ax v aa1 n))
("rage" nil (r ey1 jh))
("raged" nil (r ey1 jh d))
("ragen" nil (r ae1 g ax n))
("rager" nil (r ey1 g er0))
("rages" nil (r ey1 jh ih0 z))
("ragged" nil (r ae1 g ax d))
("raggedy" nil (r ae1 g ax d iy0))
("raggio" nil (r aa1 jh iy0 ow0))
("ragin" nil (r ae1 jh ih0 n))
("raging" nil (r ey1 jh ih0 ng))
("ragland" nil (r ae1 g l ax n d))
("ragle" nil (r ey1 g ax l))
("raglin" nil (r ae1 g l ih0 n))
("ragmen" nil (r ae1 g m ax n))
("rago" nil (r aa1 g ow0))
("ragon" nil (r aa0 g ao1 n))
("ragona" nil (r aa0 g ow1 n ax))
("ragone" nil (r aa0 g ow1 n iy0))
("ragonese" nil (r aa0 g ow0 n ey1 z iy0))
("rags" nil (r ae1 g z))
("ragsdale" nil (r ae1 g z d ey1 l))
("ragtag" nil (r ae1 g t ae1 g))
("ragtime" nil (r ae1 g t ay1 m))
("ragu" nil (r ae0 g uw1))
("ragucci" nil (r aa0 g uw1 ch iy0))
("ragunathan" nil (r aa0 g uw1 n ax th aa0 n))
("ragusa" nil (r aa0 g uw1 s ax))
("ragweed" nil (r ae1 g w iy1 d))
("rah" nil (r aa1))
("rahe" nil (r ey1 hh iy0))
("raheem" nil (r ax hh iy1 m))
("rahill" nil (r aa1 hh ih0 l))
("rahilly" nil (r ae1 hh ax l iy0))
("rahim" nil (r ax hh iy1 m))
("rahl" nil (r aa1 l))
("rahm" nil (r ae1 m))
("rahman" nil (r aa1 m ax n))
("rahming" nil (r aa1 m ih0 ng))
("rahn" nil (r ae1 n))
("rahrig" nil (r ae1 r ih0 g))
("rahul" nil (r ax hh uw1 l))
("rahway" nil (r aa1 w ey1))
("rai" nil (r aa1 iy0))
("raia" nil (r aa1 y ax))
("raible" nil (r ey1 b ax l))
("raiche" nil (r ey1 ch))
("raid" nil (r ey1 d))
("raided" nil (r ey1 d ih0 d))
("raider" nil (r ey1 d er0))
("raiders" nil (r ey1 d er0 z))
("raiding" nil (r ey1 d ih0 ng))
("raids" nil (r ey1 d z))
("raiff" nil (r ey1 f))
("raiford" nil (r ey1 f er0 d))
("raikes" nil (r ey1 k s))
("rail" nil (r ey1 l))
("railbike" nil (r ey1 l b ay1 k))
("railbikers" nil (r ey1 l b ih0 k er0 z))
("railcar" nil (r ey1 l k aa1 r))
("railcars" nil (r ey1 l k aa1 r z))
("raile" nil (r ey1 l))
("railed" nil (r ey1 l d))
("railey" nil (r ey1 l iy0))
("railing" nil (r ey1 l ih0 ng))
("railings" nil (r ey1 l ih0 ng z))
("railroad" nil (r ey1 l r ow1 d))
("railroaded" nil (r ey1 l r ow1 d ih0 d))
("railroading" nil (r ey1 l r ow1 d ih0 ng))
("railroads" nil (r ey1 l r ow1 d z))
("rails" nil (r ey1 l z))
("railsback" nil (r ey1 l z b ae1 k))
("railtex" nil (r ey1 l t eh1 k s))
("railway" nil (r ey1 l w ey1))
("railways" nil (r ey1 l w ey1 z))
("raimer" nil (r ey1 m er0))
("raimo" nil (r ey1 m ow0))
("raimond" nil (r ey1 m ax n d))
("raimondi" nil (r ax m ow1 n d iy0))
("raimondo" nil (r ey1 m aa1 n d ow0))
("rain" nil (r ey1 n))
("raina" nil (r ey1 n ax))
("rainbolt" nil (r ey1 n b ow1 l t))
("rainbow" nil (r ey1 n b ow1))
("rainbows" nil (r ey1 n b ow1 z))
("raincoat" nil (r ey1 n k ow1 t))
("raincoats" nil (r ey1 n k ow1 t s))
("raindancer" nil (r ey1 n d ae1 n s er0))
("raindrops" nil (r ey1 n d r aa1 p s))
("raine" nil (r ey1 n))
("rained" nil (r ey1 n d))
("rainer" nil (r ey1 n er0))
("raineri" nil (r ax n eh1 r iy0))
("raines" nil (r ey1 n z))
("rainey" nil (r ey1 n iy0))
("rainfall" nil (r ey1 n f ao1 l))
("rainford" nil (r ay1 n f er0 d))
("rainforest" nil (r ay1 n f ao1 r ax s t))
("rainforests" nil (r ay1 n f ao1 r ax s t s))
("rainger" nil (r aa1 ih0 ng er0))
("rainier" nil (r ey0 n ih1 r))
("rainiest" nil (r ey1 n iy0 ax s t))
("raining" nil (r ey1 n ih0 ng))
("rainmaker" nil (r ey1 n m ey1 k er0))
("rainman" nil (r ey1 n m ax n))
("rainone" nil (r ey1 n ow1 n))
("rains" nil (r ey1 n z))
("rainstorm" nil (r ey1 n s t ao1 r m))
("rainville" nil (r ey1 n v ih1 l))
("rainwater" nil (r ey1 n w ao1 t er0))
("rainy" nil (r ey1 n iy0))
("raisa" nil (r ey1 s ax))
("raisanen" nil (r ay1 s ax n ax n))
("raisbeck" nil (r ey1 z b eh1 k))
("raisch" nil (r ay1 sh))
("raise" nil (r ey1 z))
("raised" nil (r ey1 z d))
("raiser" nil (r ey1 z er0))
("raisers" nil (r ey1 z er0 z))
("raises" nil (r ey1 z ax z))
("raisin" nil (r ey1 z ih0 n))
("raising" nil (r ey1 z ih0 ng))
("raisins" nil (r ey1 z ax n z))
("raisler" nil (r ey1 z l er0))
("raison" nil (r ey1 s aa0 n))
("raisor" nil (r ey1 z er0))
("raissa" nil (r ey1 s ax))
("raith" nil (r ey1 th))
("raithel" nil (r ey1 th ax l))
("raitt" nil (r ey1 t))
("raj" nil (r aa1 zh))
("rajala" nil (r aa0 y aa1 l ax))
("rajaratnam" nil (r aa0 jh er0 aa1 t n aa0 m))
("rajewski" nil (r ay0 eh1 f s k iy0))
("rajiv" nil (r aa0 jh iy1 v))
("rajkumar" nil (r aa1 zh k uw0 m aa1 r))
("raju" nil (r aa1 y uw0))
("rak" nil (r ae1 k))
("rake" nil (r ey1 k))
("raked" nil (r ey1 k t))
("raker" nil (r ey1 k er0))
("rakers" nil (r ey1 k er0 z))
("rakes" nil (r ey1 k s))
("rakestraw" nil (r ey1 k s t r ao1))
("rakich" nil (r ae1 k ih0 ch))
("raking" nil (r ey1 k ih0 ng))
("rakish" nil (r ey1 k ih0 sh))
("rakoczy" nil (r ax k ow1 ch iy0))
("rakoff" nil (r ae1 k ao1 f))
("rakolta" nil (r ax k aa1 l t ax))
("rakovica" nil (r ax k ow1 v ih0 k ax))
("rakow" nil (r ae1 k aw0))
("rakowski" nil (r ax k aw1 s k iy0))
("raleigh" nil (r ao1 l iy0))
("rales" nil (r ey1 l z))
("raleses" nil (r ax l iy1 s ih0 z))
("raley" nil (r ae1 l iy0))
("ralf" nil (r aa1 l f))
("rall" nil (r ao1 l))
("rallied" nil (r ae1 l iy0 d))
("rallies" nil (r ae1 l iy0 z))
("rallis" nil (r ae1 l ih0 s))
("rallo" nil (r ae1 l ow0))
("ralls" nil (r ao1 l z))
("rally" nil (r ae1 l iy0))
("rallying" nil (r ae1 l iy0 ih0 ng))
("ralph" nil (r ae1 l f))
("ralphs" nil (r ae1 l f s))
("ralston" nil (r ao1 l s t ax n))
("ram" nil (r ae1 m))
("rama" nil (r aa1 m ax))
("ramada" nil (r ax m aa1 d ax))
("ramadan" nil (r ae1 m ax d ax n))
("ramage" nil (r ae1 m ih0 jh))
("ramaker" nil (r aa1 m ey0 k er0))
("ramallah" nil (r ax m ae1 l ax))
("raman" nil (r ey1 m ax n))
("ramaphosa" nil (r ae1 m ax f ow1 s ax))
("ramat" nil (r ae1 m ae0 t))
("rambeau" nil (r ax m b ow1))
("ramberg" nil (r ae1 m b er0 g))
("rambert" nil (r ae1 m b er0 t))
("rambin" nil (r ae1 m b ih0 n))
("ramble" nil (r ae1 m b ax l))
("rambled" nil (r ae1 m b ax l d))
("rambler" nil (r ae1 m b l er0))
("ramblers" nil (r ae1 m b l er0 z))
("rambling" nil (r ae1 m b l ih0 ng))
("rambo" nil (r ae1 m b ow0))
("rambow" nil (r ae1 m b ow0))
("rambunctious" nil (r ae0 m b ah1 ng k sh ax s))
("ramel" nil (r ae1 m ax l))
("ramella" nil (r ax m eh1 l ax))
("ramer" nil (r ey1 m er0))
("rameriz" nil (r aa0 m eh1 r iy0 z))
("rameses" nil (r ae1 m ax s iy1 z))
("ramesh" nil (r aa1 m eh1 sh))
("ramesses" nil (r ae1 m ax s ih0 z))
("ramey" nil (r ae1 m iy0))
("ramifications" nil (r ae1 m ax f ax k ey1 sh ax n z))
("ramires" nil (r aa0 m ih1 r eh0 s))
("ramirez" nil (r ax m ih1 r eh0 z))
("ramiro" nil (r ax m ih1 r ow0))
("ramlawi" nil (r ae1 m l aw1 iy0))
("ramler" nil (r ae1 m l er0))
("ramlow" nil (r ae1 m l ow1))
("ramm" nil (r ae1 m))
("rammed" nil (r ae1 m d))
("rammel" nil (r ae1 m ax l))
("rammer" nil (r ae1 m er0))
("ramming" nil (r ae1 m ih0 ng))
("ramo" nil (r ey1 m ow0))
("ramon" nil (r ax m ow1 n))
("ramona" nil (r ax m ow1 n ax))
("ramonda" nil (r ax m aa1 n d ax))
("ramos" nil (r aa1 m ow0 s))
("ramp" nil (r ae1 m p))
("rampage" n (r ae1 m p ey1 jh))
("rampage" v (r ae0 m p ey1 jh))
("rampaged" nil (r ae0 m p ey1 jh d))
("rampages" n (r ae1 m p ey1 jh ih0 z))
("rampages" v (r ae0 m p ey1 jh ih0 z))
("rampant" nil (r ae1 m p ax n t))
("ramparts" nil (r ae1 m p aa1 r t s))
("rampell" nil (r ae0 m p eh1 l))
("rampey" nil (r ae1 m p iy0))
("ramping" nil (r ae1 m p ih0 ng))
("rampley" nil (r ae1 m p l iy0))
("ramps" nil (r ae1 m p s))
("rampy" nil (r ae1 m p iy0))
("ramqvist" nil (r ae1 m k v ih1 s t))
("ramrod" nil (r ae1 m r aa1 d))
("rams" nil (r ae1 m z))
("ramsay" nil (r ae1 m z iy0))
("ramsburg" nil (r ae1 m s b er0 g))
("ramsdell" nil (r ae1 m s d ax l))
("ramsden" nil (r ae1 m s d ax n))
("ramser" nil (r ae1 m z er0))
("ramses" nil (r ae1 m s iy1 z))
("ramseur" nil (r ax m s er1))
("ramsey" nil (r ae1 m z iy0))
("ramseyer" nil (r ae1 m z iy0 er0))
("ramshackle" nil (r ae1 m sh ae1 k ax l))
("ramstad" nil (r ae1 m s t ax d))
("ramstein" nil (r ae1 m s t ay1 n))
("ramtek" nil (r ae1 m t eh1 k))
("ramthun" nil (r ae1 m th ax n))
("ramune" nil (r ae1 m y uw1 n))
("ramus" nil (r ey1 m ax s))
("ran" nil (r ae1 n))
("rana" nil (r ae1 n ax))
("ranalli" nil (r ax n ae1 l iy0))
("ranallo" nil (r ax n ae1 l ow0))
("rance" nil (r ae1 n s))
("ranch" nil (r ae1 n ch))
("rancher" nil (r ae1 n ch er0))
("ranchers" nil (r ae1 n ch er0 z))
("ranches" nil (r ae1 n ch ax z))
("ranching" nil (r ae1 n ch ih0 ng))
("ranchland" nil (r ae1 n ch l ax n d))
("rancho" nil (r ae1 n ch ow0))
("rancid" nil (r ae1 n s ih0 d))
("ranck" nil (r ae1 ng k))
("ranco" nil (r ae1 ng k ow0))
("rancor" nil (r ae1 ng k er0))
("rancorous" nil (r ae1 ng k er0 ax s))
("rancourt" nil (r ax n k ao1 r t))
("rand" nil (r ae1 n d))
("randa" nil (r aa1 n d ax))
("randal" nil (r ae1 n d ax l))
("randall" nil (r ae1 n d ax l))
("randazzo" nil (r aa0 n d aa1 z ow0))
("randel" nil (r ae1 n d ax l))
("randell" nil (r ae1 n d eh1 l))
("rander" nil (r ae1 n d er0))
("randi" nil (r ae1 n d iy0))
("randle" nil (r ae1 n d ax l))
("randleman" nil (r ae1 n d ax l m ax n))
("randles" nil (r ae1 n d ax l z))
("randlett" nil (r ae1 n d l ih0 t))
("rando" nil (r aa1 n d ow0))
("randol" nil (r ae1 n d ax l))
("randolf" nil (r ae1 n d ow1 l f))
("randolph" nil (r ae1 n d aa0 l f))
("random" nil (r ae1 n d ax m))
("randomized" nil (r ae1 n d ax m ay1 z d))
("randomly" nil (r ae1 n d ax m l iy0))
("randomness" nil (r ae1 n d ax m n ax s))
("rands" nil (r ae1 n d z))
("randy" nil (r ae1 n d iy0))
("ranee" nil (r ae1 n iy1))
("raneri" nil (r aa0 n eh1 r iy0))
("ranes" nil (r ey1 n z))
("raney" nil (r ey1 n iy0))
("ranft" nil (r ae1 n f t))
("rang" nil (r ae1 ng))
("range" nil (r ey1 n jh))
("ranged" nil (r ey1 n jh d))
("rangel" nil (r ae1 n jh eh1 l))
("ranger" nil (r ey1 n jh er0))
("rangers" nil (r ey1 n jh er0 z))
("ranges" nil (r ey1 n jh ax z))
("ranging" nil (r ey1 n jh ih0 ng))
("rangoon" nil (r ae0 ng g uw1 n))
("rangy" nil (r ey1 n jh iy0))
("ranh" nil (r ae1 n))
("rani" nil (r aa1 n iy0))
("rania" nil (r aa1 n iy0 ax))
("ranieri" nil (r ae1 n iy0 eh1 r iy0))
("rank" nil (r ae1 ng k))
("ranke" nil (r ae1 ng k))
("ranked" nil (r ae1 ng k t))
("ranker" nil (r ae1 ng k er0))
("rankin" nil (r ae1 ng k ih0 n))
("rankine" nil (r ae1 ng k ay1 n))
("ranking" nil (r ae1 ng k ih0 ng))
("rankings" nil (r ae1 ng k ih0 ng z))
("rankins" nil (r ae1 ng k ih0 n z))
("rankle" nil (r ae1 ng k ax l))
("rankled" nil (r ae1 ng k ax l d))
("rankles" nil (r ae1 ng k ax l z))
("rankling" nil (r ae1 ng k l ih0 ng))
("ranks" nil (r ae1 ng k s))
("ranley" nil (r ae1 n l iy0))
("rann" nil (r ae1 n))
("ranney" nil (r ae1 n iy0))
("rannow" nil (r ae1 n ow0))
("rans" nil (r ae1 n z))
("ransack" nil (r ae1 n s ae1 k))
("ransacked" nil (r ae1 n s ae1 k t))
("ransbottom" nil (r ae1 n s b ax t aa0 m))
("ransburg" nil (r ae1 n z b er0 g))
("ransdell" nil (r ae1 n z d eh1 l))
("ransford" nil (r ae1 n s f er0 d))
("ransier" nil (r ae1 n s iy0 er0))
("ransley" nil (r ae1 n s l iy0))
("ransom" nil (r ae1 n s ax m))
("ransome" nil (r ae1 n s ax m))
("ransoms" nil (r ae1 n s ax m z))
("ranson" nil (r ae1 n s ax n))
("ransone" nil (r ae1 n s ax n))
("ranta" nil (r ae1 n t ax))
("rantala" nil (r aa0 n t aa1 l ax))
("ranted" nil (r ae1 n t ax d))
("ranting" nil (r ae1 n t ih0 ng))
("rantoul" nil (r ae1 n t uw1 l))
("rantz" nil (r ae1 n t s))
("ranum" nil (r ae1 n ax m))
("ranz" nil (r ae1 n z))
("rao" nil (r aw1))
("raoul" nil (r aa0 uw1 l))
("rap" nil (r ae1 p))
("rapacious" nil (r ax p ae1 sh ih0 s))
("rapacz" nil (r aa1 p ax ch))
("rapanelli" nil (r ae1 p ax n eh1 l iy0))
("rapaport" nil (r ae1 p ax p ao1 r t))
("rape" nil (r ey1 p))
("raped" nil (r ey1 p t))
("raper" nil (r ey1 p er0))
("rapers" nil (r ey1 p er0 z))
("rapes" nil (r ey1 p s))
("rapeseed" nil (r ey1 p s iy1 d))
("raphael" nil (r aa1 f ay0 eh1 l))
("raphaela" nil (r ae1 f ey0 l ax))
("raphalian" nil (r ax f ey1 l iy0 ax n))
("raphel" nil (r ae1 f ax l))
("rapid" nil (r ae1 p ax d))
("rapidity" nil (r ax p ih1 d ax t iy0))
("rapidly" nil (r ae1 p ax d l iy0))
("rapids" nil (r ae1 p ax d z))
("rapier" nil (r ey1 p iy0 er0))
("raping" nil (r ey1 p ih0 ng))
("rapist" nil (r ey1 p ih0 s t))
("rapists" nil (r ey1 p ih0 s t s))
("rapkin" nil (r ae1 p k ih0 n))
("rapley" nil (r ae1 p l iy0))
("rapoca" nil (r ax p ow1 k er0))
("rapoport" nil (r ax p aa1 p ao0 r t))
("raposa" nil (r aa0 p ow1 s ax))
("raposo" nil (r aa0 p ow1 s ow0))
("rapoza" nil (r aa0 p ow1 z ax))
("rapozo" nil (r aa0 p ow1 z ow0))
("rapp" nil (r ae1 p))
("rappa" nil (r ae1 p ax))
("rappahannock" nil (r ae1 p ax hh ae1 n ax k))
("rappaport" nil (r ae1 p ax p ao0 r t))
("rappe" nil (r ae1 p))
("rapped" nil (r ae1 p t))
("rappelling" nil (r ax p eh1 l ih0 ng))
("rapper" nil (r ae1 p er0))
("rappers" nil (r ae1 p er0 z))
("rappold" nil (r ae1 p ow1 l d))
("rappoport" nil (r ae1 p ax p ao0 r t))
("rapport" nil (r ae0 p ao1 r))
("rapprochement" nil (r ae1 p r ow1 sh m aa1 n))
("rapson" nil (r ae1 p s ax n))
("rapt" nil (r ae1 p t))
("raptis" nil (r ax p t iy1 s))
("raptopoulos" nil (r ae0 p t aa1 p ow0 l ax s))
("raptor" nil (r ae1 p t er0))
("raptorial" nil (r ae1 p t ao1 r iy0 ax l))
("raptors" nil (r ae1 p t er0 z))
("rapture" nil (r ae1 p ch er0))
("rapturous" nil (r ae1 p ch er0 ax s))
("rapuano" nil (r aa0 p uw0 aa1 n ow0))
("raquel" nil (r ax k eh1 l))
("rardon" nil (r aa1 r d ax n))
("rare" nil (r eh1 r))
("rarefied" nil (r eh1 r ax f ay0 d))
("rarely" nil (r eh1 r l iy0))
("rarer" nil (r eh1 r er0))
("rarest" nil (r eh1 r ax s t))
("rarick" nil (r ae1 r ih0 k))
("raritan" nil (r eh1 r ih0 t ax n))
("rarities" nil (r eh1 r ih0 t iy0 z))
("rarity" nil (r eh1 r ax t iy0))
("ras" nil (r ae1 s))
("rasberry" nil (r ae1 s b eh0 r iy0))
("rasbury" nil (r ae1 s b eh0 r iy0))
("rascal" nil (r ae1 s k ax l))
("rascals" nil (r ae1 s k ax l z))
("rasch" nil (r ae1 sh))
("rasche" nil (r ae1 sh))
("raschke" nil (r ae1 sh k))
("rasco" nil (r aa1 s k ow0))
("rascoe" nil (r ae1 s k ow0))
("rascon" nil (r ae1 s k ax n))
("rase" nil (r ey1 z))
("raser" nil (r ey1 z er0))
("rasey" nil (r ae1 s iy0))
("rash" nil (r ae1 sh))
("rasheed" nil (r ax sh iy1 d))
("rashes" nil (r ae1 sh ih0 z))
("rashid" nil (r ax sh iy1 d))
("rasia" nil (r aa1 s iy0 ax))
("rask" nil (r ae1 s k))
("raska" nil (r aa1 s k ax))
("raske" nil (r ey1 s k))
("raskin" nil (r ae1 s k ih0 n))
("rasmus" nil (r ae1 z m ih0 s))
("rasmuson" nil (r ae1 z m ax s ax n))
("rasmussen" nil (r ae1 s m ax s ax n))
("rasmusson" nil (r ae1 z m ax s ax n))
("rasnake" nil (r ae1 s n ax k))
("rasner" nil (r ae1 s n er0))
("rasnick" nil (r ae1 s n ih0 k))
("raso" nil (r aa1 s ow0))
("rasor" nil (r ey1 z er0))
("rasorite" nil (r ae1 s er0 ay1 t))
("rasp" nil (r ae1 s p))
("raspberries" nil (r ae1 z b eh1 r iy0 z))
("raspberry" nil (r ae1 z b eh1 r iy0))
("rasped" nil (r ae1 s p t))
("rasps" nil (r ae1 s p s))
("rasputin" nil (r ax s p y uw1 t ih0 n))
("raspy" nil (r ae1 s p iy0))
("rast" nil (r ae1 s t))
("raster" nil (r ae1 s t er0))
("rasterops" nil (r ae1 s t er0 aa1 p s))
("rastetter" nil (r ey1 s t ih0 t er0))
("rastus" nil (r ae1 s t ax s))
("rat" nil (r ae1 t))
("rata" nil (r ae1 t ax))
("ratajczak" nil (r ae1 t ax ch eh0 k))
("ratay" nil (r ae1 t ey0))
("ratchet" nil (r ae1 ch ax t))
("ratcheted" nil (r ae1 ch ax t ih0 d))
("ratcheting" nil (r ae1 ch ax t ih0 ng))
("ratchford" nil (r ae1 ch f er0 d))
("ratcliff" nil (r ae1 t k l ih0 f))
("ratcliffe" nil (r ae1 t k l ih0 f))
("rate" nil (r ey1 t))
("rated" nil (r ey1 t ax d))
("rateliff" nil (r ae1 t l ih0 f))
("ratepayer" nil (r ey1 t p ey1 er0))
("ratepayers" nil (r ey1 t p ey1 er0 z))
("raters" nil (r ey1 t er0 z))
("rates" nil (r ey1 t s))
("rath" nil (r ae1 th))
("rathbone" nil (r ae1 th b ow1 n))
("rathbun" nil (r ae1 th b ax n))
("rathburn" nil (r ae1 th b er0 n))
("rathburne" nil (r ae1 th b er0 n))
("rathe" nil (r ey1 dh))
("rathel" nil (r ae1 th ax l))
("rather" nil (r ae1 dh er0))
("rathert" nil (r ae1 th er0 t))
("rathgeber" nil (r ae1 th g ih0 b er0))
("rathje" nil (r ae1 th jh))
("rathjen" nil (r ae1 th jh ax n))
("rathke" nil (r ae1 th k))
("rathman" nil (r ae1 th m ax n))
("rathmann" nil (r ae1 th m ax n))
("rathskeller" nil (r ae1 th s k eh1 l er0))
("ratican" nil (r ae1 t ih0 k ax n))
("ratieni" nil (r ax t iy1 n iy0))
("ratier" nil (r ey1 t y er0))
("ratification" nil (r ae1 t ax f ax k ey1 sh ax n))
("ratified" nil (r ae1 t ax f ay1 d))
("ratifies" nil (r ae1 t ax f ay1 z))
("ratify" nil (r ae1 t ax f ay1))
("ratifying" nil (r ae1 t ax f ay1 ih0 ng))
("rating" nil (r ey1 t ih0 ng))
("ratings" nil (r ey1 t ih0 ng z))
("ratio" nil (r ey1 sh iy0 ow1))
("ration" nil (r ae1 sh ax n))
("rational" nil (r ae1 sh ax n ax l))
("rationale" nil (r ae1 sh ax n ae1 l))
("rationales" nil (r ae1 sh ax n ae1 l z))
("rationality" nil (r ae1 sh ax n ae1 l ih0 t iy0))
("rationalization" nil (r ae1 sh ax n ax l ih0 z ey1 sh ax n))
("rationalizations" nil (r ae1 sh ax n ax l ih0 z ey1 sh ax n z))
("rationalize" nil (r ae1 sh ax n ax l ay1 z))
("rationalized" nil (r ae1 sh ax n ax l ay1 z d))
("rationalizing" nil (r ae1 sh ax n ax l ay1 z ih0 ng))
("rationally" nil (r ae1 sh ax n ax l iy0))
("rationed" nil (r ae1 sh ax n d))
("rationing" nil (r ae1 sh ax n ih0 ng))
("rations" nil (r ae1 sh ax n z))
("ratios" nil (r ey1 sh iy0 ow1 z))
("ratkovich" nil (r aa1 t k ax v ih0 ch))
("ratkowski" nil (r ax t k ao1 f s k iy0))
("ratledge" nil (r ae1 t l ih0 jh))
("ratley" nil (r ae1 t l iy0))
("ratliff" nil (r ae1 t l ih0 f))
("ratliffe" nil (r ae1 t l ih0 f))
("ratner" nil (r ae1 t n er0))
("ratners" nil (r ae1 t n er0 z))
("raton" nil (r ax t ow1 n))
("rats" nil (r ae1 t s))
("rattan" nil (r ae0 t ae1 n))
("ratte" nil (r ae1 t))
("ratterman" nil (r ae1 t er0 m ax n))
("ratterree" nil (r ae1 t er0 iy1))
("rattigan" nil (r ae1 t ih0 g ax n))
("rattle" nil (r ae1 t ax l))
("rattled" nil (r ae1 t ax l d))
("rattler" nil (r ae1 t ax l er0))
("rattles" nil (r ae1 t ax l z))
("rattlesnake" nil (r ae1 t ax l s n ey1 k))
("rattlesnakes" nil (r ae1 t ax l s n ey1 k s))
("rattling" nil (r ae1 t l ih0 ng))
("rattner" nil (r ae1 t n er0))
("rattray" nil (r ae1 t r ey0))
("ratts" nil (r ae1 t s))
("ratty" nil (r ae1 t iy0))
("ratu" nil (r aa0 t uw1))
("ratz" nil (r ae1 t s))
("ratzinger" nil (r ae1 t s ih1 ng er0))
("ratzlaff" nil (r ae1 t z l ax f))
("rau" nil (r aw1))
("raub" nil (r ao1 b))
("rauber" nil (r aw1 b er0))
("raucci" nil (r ao1 ch iy0))
("rauch" nil (r ao1 ch))
("raucous" nil (r ao1 k ax s))
("raudabaugh" nil (r ao1 d ax b ao0))
("raudenbush" nil (r aw1 d ih0 n b ax sh))
("rauen" nil (r aw1 ax n))
("rauer" nil (r aw1 er0))
("rauh" nil (r ao1))
("raul" nil (r ao1 l))
("raulerson" nil (r ao1 l er0 s ax n))
("rauls" nil (r aa0 uw1 l z))
("raulston" nil (r ao1 l s t ax n))
("raum" nil (r ao1 m))
("raun" nil (r ao1 n))
("raunchy" nil (r ao1 n ch iy0))
("raup" nil (r ao1 p))
("raupp" nil (r ao1 p))
("rausch" nil (r aw1 sh))
("rauschenberg" nil (r aw1 sh ax n b er0 g))
("rauscher" nil (r aw1 sh er0))
("rauser" nil (r aw1 s er0))
("rautenberg" nil (r aw1 t ax n b er0 g))
("rauth" nil (r ao1 th))
("rautio" nil (r aw1 t iy0 ow0))
("ravaged" nil (r ae1 v ih0 jh d))
("ravages" nil (r ae1 v ih0 jh ih0 z))
("ravaging" nil (r ae1 v ih0 jh ih0 ng))
("ravan" nil (r ey1 v ax n))
("rave" nil (r ey1 v))
("raved" nil (r ey1 v d))
("raveled" nil (r ae1 v ax l d))
("ravelo" nil (r aa0 v eh1 l ow0))
("raven" nil (r ey1 v ax n))
("ravenel" nil (r ae1 v ih0 n eh0 l))
("ravenell" nil (r ae1 v ih0 n eh0 l))
("ravenna" nil (r ax v eh1 n ax))
("ravenous" nil (r ae1 v ax n ax s))
("ravens" nil (r ey1 v ax n z))
("ravenscraft" nil (r ey1 v ax n z k r ae1 f t))
("ravenscroft" nil (r ey1 v ax n z k r ao1 f t))
("ravenswood" nil (r ey1 v ax n z w uh1 d))
("raver" nil (r ey1 v er0))
("raves" nil (r ey1 v z))
("ravi" nil (r aa1 v iy1))
("ravin" nil (r ae1 v ih0 n))
("ravindran" nil (r aa1 v ih1 n d r ax n))
("ravine" nil (r ax v iy1 n))
("raving" nil (r ey1 v ih0 ng))
("ravioli" nil (r ae1 v iy0 ow1 l iy0))
("ravishankar" nil (r aa1 v iy0 sh aa1 ng k aa1 r))
("ravishing" nil (r ae1 v ih0 sh ih0 ng))
("ravitch" nil (r ae1 v ih0 ch))
("ravitz" nil (r ae1 v ih0 t s))
("raw" nil (r ao1))
("rawalpindi" nil (r ae1 w ao0 l p ih1 n d iy0))
("rawding" nil (r ao1 d ih0 ng))
("rawdon" nil (r ao1 d ax n))
("rawe" nil (r ao1))
("rawhide" nil (r ao1 hh ay1 d))
("rawl" nil (r ao1 l))
("rawles" nil (r ao1 l z))
("rawley" nil (r ao1 l iy0))
("rawling" nil (r ao1 l ih0 ng))
("rawlings" nil (r ao1 l ih0 ng z))
("rawlins" nil (r ao1 l ih0 n z))
("rawlinson" nil (r ao1 l ih0 n s ax n))
("rawls" nil (r ao1 l z))
("rawski" nil (r ao1 s k iy1))
("rawson" nil (r ao1 s ax n))
("rax" nil (r ae1 k s))
("raxter" nil (r ae1 k s t er0))
("ray" nil (r ey1))
("raya" nil (r ey1 ax))
("raybon" nil (r ey1 b ax n))
("rayborn" nil (r ey1 b er0 n))
("raybould" nil (r ey1 b ow1 l d))
("raybourn" nil (r ey0 b uh1 r n))
("raybuck" nil (r ey1 b ah1 k))
("rayburn" nil (r ey1 b er0 n))
("raychem" nil (r ey1 k eh1 m))
("raycon" nil (r ey1 k ax n))
("raycraft" nil (r ey1 k r ae1 f t))
("raye" nil (r ey1))
("rayed" nil (r ey1 d))
("rayfield" nil (r ey1 f iy1 l d))
("rayford" nil (r ey1 f er0 d))
("raygoza" nil (r ey1 g ow1 z ax))
("rayl" nil (r ey1 l))
("rayle" nil (r ey1 l))
("rayman" nil (r ey1 m ax n))
("raymark" nil (r ey1 m aa1 r k))
("rayment" nil (r ey1 m ax n t))
("raymer" nil (r ey1 m er0))
("raymo" nil (r ey1 m ow0))
("raymond" nil (r ey1 m ax n d))
("raymonda" nil (r ey1 m aa1 n d ax))
("raymund" nil (r ey1 m ax n d))
("raymundo" nil (r ey1 m ah1 n d ow0))
("raynard" nil (r ey1 n er0 d))
("rayne" nil (r ey1 n))
("rayner" nil (r ey1 n er0))
("raynes" nil (r ey1 n z))
("raynet" nil (r ey1 n eh1 t))
("rayno" nil (r ey1 n ow0))
("raynor" nil (r ey1 n er0))
("rayon" nil (r ey1 ax n))
("rayonier" nil (r ey1 ow0 n ih1 r))
("rayos" nil (r ey1 ow0 z))
("rayovac" nil (r ey1 ax v ae0 k))
("rayrock" nil (r ey1 r aa1 k))
("rays" nil (r ey1 z))
("rayson" nil (r ey1 z ax n))
("raysor" nil (r ey1 s er0))
("raytech" nil (r ey1 t eh1 k))
("raytheon" nil (r ey1 th iy0 aa0 n))
("raz" nil (r ae1 z))
("razaleigh" nil (r ae1 z ax l iy1))
("raze" nil (r ey1 z))
("razed" nil (r ey1 z d))
("razing" nil (r ey1 z ih0 ng))
("razo" nil (r aa1 z ow0))
("razor" nil (r ey1 z er0))
("razorback" nil (r ey1 z er0 b ae1 k))
("razorbacks" nil (r ey1 z er0 b ae1 k s))
("razors" nil (r ey1 z er0 z))
("razzano" nil (r aa0 t s aa1 n ow0))
("razzle" nil (r ae1 z ax l))
("razzmatazz" nil (r ae1 z m ax t ae1 z))
("rb" nil (aa1 r b iy1))
("re" nil (r ey1))
("rea" nil (r iy1))
("reabsorbed" nil (r iy1 ax b z ao1 r b d))
("reach" nil (r iy1 ch))
("reachable" nil (r iy1 ch ax b ax l))
("reached" nil (r iy1 ch t))
("reaches" nil (r iy1 ch ax z))
("reaching" nil (r iy1 ch ih0 ng))
("reacquire" nil (r iy0 ax k w ay1 r))
("reacquired" nil (r iy0 ax k w ay1 r d))
("react" nil (r iy0 ae1 k t))
("reacted" nil (r iy0 ae1 k t ax d))
("reacting" nil (r iy0 ae1 k t ih0 ng))
("reaction" nil (r iy0 ae1 k sh ax n))
("reactionaries" nil (r iy0 ae1 k sh ax n eh1 r iy0 z))
("reactionary" nil (r iy0 ae1 k sh ax n eh1 r iy0))
("reactions" nil (r iy0 ae1 k sh ax n z))
("reactivate" nil (r iy1 ae1 k t ax v ey1 t))
("reactivated" nil (r iy1 ae1 k t ax v ey1 t ih0 d))
("reactivating" nil (r iy1 ae1 k t ih0 v ey1 t ih0 ng))
("reactive" nil (r iy0 ae1 k t ih0 v))
("reactivity" nil (r iy1 ae1 k t ih1 v ax t iy0))
("reactor" nil (r iy0 ae1 k t er0))
("reactors" nil (r iy0 ae1 k t er0 z))
("reacts" nil (r iy0 ae1 k t s))
("read" nil (r eh1 d))
("readability" nil (r iy1 d ax b ih1 l ih0 t iy0))
("readable" nil (r iy1 d ax b ax l))
("reade" nil (r eh1 d))
("reader" nil (r iy1 d er0))
("readerman" nil (r iy1 d er0 m ax n))
("readers" nil (r iy1 d er0 z))
("readership" nil (r iy1 d er0 sh ih1 p))
("readied" nil (r eh1 d iy0 d))
("readier" nil (r eh1 d iy0 er0))
("readies" nil (r eh1 d iy0 z))
("readily" nil (r eh1 d ax l iy0))
("readiness" nil (r eh1 d iy0 n ax s))
("reading" n (r eh1 d ih0 ng))
("reading" v (r iy1 d ih0 ng))
("readinger" nil (r eh1 d ih0 ng er0))
("readings" nil (r eh1 d ih0 ng z))
("readjust" nil (r iy1 ax jh ah1 s t))
("readjusted" nil (r iy1 ax jh ah1 s t ih0 d))
("readjusting" nil (r iy1 ax jh ah1 s t ih0 ng))
("readjustment" nil (r iy0 ax jh ah1 s t m ax n t))
("readjustments" nil (r iy0 ax jh ah1 s t m ax n t s))
("readmitted" nil (r iy1 ax d m ih1 t ih0 d))
("readout" nil (r iy1 d aw1 t))
("reads" nil (r iy1 d z))
("readus" nil (r eh1 d ih0 s))
("ready" nil (r eh1 d iy0))
("readying" nil (r eh1 d iy0 ih0 ng))
("reaffiliation" nil (r iy0 ax f ih1 l iy0 ey1 sh ax n))
("reaffirm" nil (r iy1 ax f er1 m))
("reaffirmation" nil (r iy1 ae0 f er0 m ey1 sh ax n))
("reaffirmed" nil (r iy1 ax f er1 m d))
("reaffirming" nil (r iy1 ax f er1 m ih0 ng))
("reaffirms" nil (r iy1 ax f er1 m z))
("reagan" nil (r ey1 g ax n))
("reaganesque" nil (r ey1 g ax n eh1 s k))
("reaganism" nil (r ey1 g ax n ih1 z ax m))
("reaganite" nil (r ey1 g ax n ay1 t))
("reaganites" nil (r ey1 g ax n ay1 t s))
("reaganomics" nil (r ey1 g ax n aa1 m ih0 k s))
("reagans" nil (r ey1 g ax n z))
("reagen" nil (r iy1 g ax n))
("reagents" nil (r iy0 ey1 jh ax n t s))
("reagle" nil (r iy1 g ax l))
("reagor" nil (r iy1 g er0))
("real" nil (r iy1 l))
("realamerica" nil (r iy1 l ax m eh1 r ih0 k ax))
("realcap" nil (r iy1 l k ae1 p))
("reale" nil (r iy1 l))
("realestate" nil (r iy1 l ih0 s t ey1 t))
("reali" nil (r iy1 l iy0))
("realign" nil (r iy0 ax l ay1 n))
("realigned" nil (r iy0 ax l ay1 n d))
("realigning" nil (r iy0 ax l ay1 n ih0 ng))
("realignment" nil (r iy0 ax l ay1 n m ax n t))
("realignments" nil (r iy0 ax l ay1 n m ax n t s))
("realism" nil (r iy1 l ih0 z ax m))
("realist" nil (r iy1 l ih0 s t))
("realistic" nil (r iy1 ax l ih1 s t ih0 k))
("realistically" nil (r iy1 ax l ih1 s t ih0 k l iy0))
("realists" nil (r iy1 l ih0 s t s))
("realities" nil (r iy0 ae1 l ax t iy0 z))
("reality" nil (r iy0 ae1 l ax t iy0))
("realizable" nil (r iy1 ax l ay1 z ax b ax l))
("realization" nil (r iy1 l ax z ey1 sh ax n))
("realizations" nil (r iy1 l ax z ey1 sh ax n z))
("realize" nil (r iy1 l ay1 z))
("realized" nil (r iy1 l ay1 z d))
("realizes" nil (r iy1 ax l ay1 z ih0 z))
("realizing" nil (r iy1 l ay1 z ih0 ng))
("reallocate" nil (r iy0 ae1 l ax k ey0 t))
("reallocated" nil (r iy1 ae1 l ax k ey1 t ih0 d))
("reallocating" nil (r iy0 ae1 l ax k ey1 t ih0 ng))
("reallocation" nil (r iy0 ae1 l ax k ey1 sh ax n))
("reallowance" nil (r iy1 ax l aw1 ax n s))
("really" nil (r ih1 l iy0))
("realm" nil (r eh1 l m))
("realms" nil (r eh1 l m z))
("realmuto" nil (r ax l m uw1 t ow0))
("realpolitik" nil (r iy1 l p ao1 l ih0 t ih1 k))
("realtor" nil (r iy1 ax l t er0))
("realtors" nil (r iy1 ax l t er0 z))
("realty" nil (r iy1 ax l t iy0))
("ream" nil (r iy1 m))
("reamer" nil (r iy1 m er0))
("reamers" nil (r iy1 m er0 z))
("reames" nil (r iy1 m z))
("reams" nil (r iy1 m z))
("reamy" nil (r iy1 m iy0))
("reaney" nil (r iy1 n iy0))
("reap" nil (r iy1 p))
("reaped" nil (r iy1 p t))
("reaper" nil (r iy1 p er0))
("reaping" nil (r iy1 p ih0 ng))
("reappear" nil (r iy1 ax p ih1 r))
("reappearance" nil (r iy1 ax p ih1 r ax n s))
("reappeared" nil (r iy1 ax p ih1 r d))
("reappears" nil (r iy1 ax p ih1 r z))
("reapply" nil (r iy1 ax p l iy0))
("reappoint" nil (r iy1 ax p oy1 n t))
("reappointed" nil (r iy1 ax p oy1 n t ih0 d))
("reappointment" nil (r iy1 ax p oy1 n t m ax n t))
("reapportionment" nil (r iy1 ax p ao1 r sh ax n m ax n t))
("reappraisal" nil (r iy1 ax p r ey1 z ax l))
("reappraise" nil (r iy1 ax p r ey1 z))
("reappraised" nil (r iy0 ax p r ey1 z d))
("reaps" nil (r iy1 p s))
("rear" nil (r ih1 r))
("rearden" nil (r ih1 r d ax n))
("reardon" nil (r ih1 r d ax n))
("reared" nil (r ih1 r d))
("rearick" nil (r ih1 r ih0 k))
("rearing" nil (r ih1 r ih0 ng))
("rearm" nil (r iy1 aa1 r m))
("rearmament" nil (r iy0 aa1 r m ax m ax n t))
("rearrange" nil (r iy1 er0 ey1 n jh))
("rearranged" nil (r iy1 er0 ey1 n jh d))
("rearrangement" nil (r iy0 er0 ey1 n jh m ax n t))
("rearranging" nil (r iy1 er0 ey1 n jh ih0 ng))
("rearrested" nil (r iy0 er0 eh1 s t ih0 d))
("rears" nil (r ih1 r z))
("reas" nil (r iy1 z))
("rease" nil (r iy1 s))
("reaser" nil (r iy1 s er0))
("reasner" nil (r iy1 s n er0))
("reason" nil (r iy1 z ax n))
("reasonable" nil (r iy1 z ax n ax b ax l))
("reasonableness" nil (r iy1 z ax n ax b ax l n ax s))
("reasonably" nil (r iy1 z ax n ax b l iy0))
("reasoned" nil (r iy1 z ax n d))
("reasoner" nil (r iy1 z ax n er0))
("reasoning" nil (r iy1 z ax n ih0 ng))
("reasonover" nil (r iy1 z ax n ow1 v er0))
("reasons" nil (r iy1 z ax n z))
("reasor" nil (r iy1 s er0))
("reassemble" nil (r iy1 ax s eh1 m b ax l))
("reassembled" nil (r iy1 ax s eh1 m b ax l d))
("reassert" nil (r iy1 ax s er1 t))
("reasserted" nil (r iy1 ax s er1 t ax d))
("reasserting" nil (r iy1 ax s er1 t ih0 ng))
("reassertion" nil (r iy1 ax s er1 sh ax n))
("reasserts" nil (r iy1 ax s er1 t s))
("reassess" nil (r iy0 ax s eh1 s))
("reassessed" nil (r iy0 ax s eh1 s t))
("reassessing" nil (r iy0 ax s eh1 s ih0 ng))
("reassessment" nil (r iy1 ax s eh1 s m ax n t))
("reassign" nil (r iy1 ax s ay1 n))
("reassigned" nil (r iy1 ax s ay1 n d))
("reassigning" nil (r iy1 ax s ay1 n ih0 ng))
("reassignment" nil (r iy0 ax s ay1 n m ax n t))
("reassignments" nil (r iy1 ax s ay1 n m ax n t s))
("reassume" nil (r iy0 ax s uw1 m))
("reassumed" nil (r iy0 ax s uw1 m d))
("reassurance" nil (r iy1 ax sh uh1 r ax n s))
("reassurances" nil (r iy1 ax sh er1 ax n s ih0 z))
("reassure" nil (r iy1 ax sh uh1 r))
("reassured" nil (r iy1 ax sh uh1 r d))
("reassures" nil (r iy1 ax sh uh1 r z))
("reassuring" nil (r iy1 ax sh uh1 r ih0 ng))
("reassuringly" nil (r iy0 ax sh er1 ih0 ng l iy0))
("reatta" nil (r iy1 aa1 t ax))
("reaume" nil (r uw1 m))
("reauthorization" nil (r iy0 ao1 th er0 ih0 z ey1 sh ax n))
("reauthorize" nil (r iy0 ao1 th er0 ay1 z))
("reauthorized" nil (r iy0 ao1 th er0 ay1 z d))
("reauthorizing" nil (r iy0 ao1 th er0 ay1 z ih0 ng))
("reaux" nil (r ow1))
("reave" nil (r iy1 v))
("reaver" nil (r iy1 v er0))
("reaves" nil (r iy1 v z))
("reavis" nil (r iy1 v ih0 s))
("reawakened" nil (r iy1 ax w ey1 k ax n d))
("reawakening" nil (r iy1 ax w ey1 k ax n ih0 ng))
("reay" nil (r ey1))
("reba" nil (r iy1 b ax))
("reback" nil (r iy1 b ae0 k))
("rebalancing" nil (r iy0 b ae1 l ax n s ih0 ng))
("rebar" nil (r iy1 b aa1 r))
("rebate" nil (r iy1 b ey1 t))
("rebated" nil (r iy1 b ey1 t ih0 d))
("rebates" nil (r iy1 b ey1 t s))
("rebbe" nil (r iy1 b ax))
("rebecca" nil (r ax b eh1 k ax))
("rebeck" nil (r iy1 b eh0 k))
("rebeka" nil (r ih0 b iy1 k ax))
("rebekka" nil (r ih0 b eh1 k ax))
("rebel" n (r eh1 b ax l))
("rebel" v (r ax b eh1 l))
("rebelled" nil (r ih0 b eh1 l d))
("rebelling" nil (r ih0 b eh1 l ih0 ng))
("rebellion" nil (r ih0 b eh1 l y ax n))
("rebellions" nil (r ih0 b eh1 l y ax n z))
("rebellious" nil (r ih0 b eh1 l y ax s))
("rebelliousness" nil (r ax b eh1 l iy0 ax s n ax s))
("rebello" nil (r eh0 b eh1 l ow0))
("rebelo" nil (r eh0 b eh1 l ow0))
("rebels" n (r eh1 b ax l z))
("rebels" v (r ax b eh1 l z))
("reber" nil (r eh1 b er0))
("rebert" nil (r eh1 b er0 t))
("rebholz" nil (r eh1 b hh ow0 l z))
("rebid" nil (r iy0 b ih1 d))
("rebirth" nil (r iy0 b er1 th))
("rebirths" nil (r iy0 b er1 th s))
("rebman" nil (r eh1 b m ax n))
("rebmann" nil (r eh1 b m ax n))
("rebo" nil (r iy1 b ow0))
("reborn" nil (r iy1 b ao1 r n))
("rebound" v (r iy0 b aw1 n d))
("rebound" n (r iy1 b aw1 n d))
("rebounded" nil (r ih0 b aw1 n d ih0 d))
("rebounding" nil (r ih0 b aw1 n d ih0 ng))
("rebounds" n (r iy1 b aw1 n d z))
("rebounds" v (r iy0 b aw1 n d z))
("rebroadcast" nil (r iy0 b r ao1 d k ae1 s t))
("rebstock" nil (r eh1 b s t aa1 k))
("rebuck" nil (r eh1 b ax k))
("rebuff" nil (r ih0 b ah1 f))
("rebuffed" nil (r ih0 b ah1 f t))
("rebuffing" nil (r iy0 b ah1 f ih0 ng))
("rebuffs" nil (r iy0 b ah1 f s))
("rebuild" nil (r iy0 b ih1 l d))
("rebuilder" nil (r iy0 b ih1 l d er0))
("rebuilders" nil (r iy0 b ih1 l d er0 z))
("rebuilding" nil (r iy0 b ih1 l d ih0 ng))
("rebuilds" nil (r iy0 b ih1 l d z))
("rebuilt" nil (r iy0 b ih1 l t))
("rebuke" nil (r iy0 b y uw1 k))
("rebuked" nil (r ih0 b y uw1 k t))
("rebukes" nil (r iy0 b y uw1 k s))
("rebuking" nil (r iy0 b y uw1 k ih0 ng))
("rebus" nil (r iy1 b ax s))
("rebut" nil (r ih0 b ah1 t))
("rebuttal" nil (r ih0 b ah1 t ax l))
("rebuttals" nil (r ih0 b ah1 t ax l z))
("rebutted" nil (r ih0 b ah1 t ih0 d))
("rebutting" nil (r ih0 b ah1 t ih0 ng))
("recadi" nil (r ih0 k ae1 d iy0))
("recalcitrance" nil (r ih0 k ae1 l s ax t r ax n s))
("recalcitrant" nil (r ih0 k ae1 l s ih0 t r ax n t))
("recalculate" nil (r iy0 k ae1 l k y ax l ey1 t))
("recalculated" nil (r iy0 k ae1 l k y ax l ey1 t ih0 d))
("recalculating" nil (r iy0 k ae1 l k y ax l ey1 t ih0 ng))
("recalculation" nil (r iy0 k ae1 l k y ax l ey1 sh ax n))
("recall" nil (r iy1 k ao1 l))
("recalled" nil (r ih0 k ao1 l d))
("recalling" nil (r ih0 k ao1 l ih0 ng))
("recalls" nil (r iy1 k ao1 l z))
("recant" nil (r iy0 k ae1 n t))
("recantation" nil (r eh1 k ax n t ey1 sh ax n))
("recanted" nil (r iy0 k ae1 n t ih0 d))
("recanting" nil (r ax k ae1 n t ih0 ng))
("recap" nil (r iy1 k ae1 p))
("recapitalization" nil (r iy1 k ae1 p ih0 t ax l ih0 z ey1 sh ax n))
("recapitalizations" nil (r iy0 k ae1 p ax t ax l ax z ey1 sh ax n z))
("recapitalize" nil (r iy1 k ae1 p ih0 t ax l ay1 z))
("recapitalized" nil (r iy1 k ae1 p ih0 t ax l ay1 z d))
("recapitalizing" nil (r iy1 k ae1 p ih0 t ax l ay1 z ih0 ng))
("recapitulates" nil (r iy1 k ax p ih1 ch ax l ey1 t s))
("recapped" nil (r iy0 k ae1 p t))
("recapping" nil (r iy0 k ae1 p ih0 ng))
("recapture" nil (r iy0 k ae1 p ch er0))
("recaptured" nil (r iy0 k ae1 p ch er0 d))
("recapturing" nil (r iy0 k ae1 p ch er0 ih0 ng))
("recarey" nil (r iy0 k eh1 r iy0))
("recast" nil (r iy0 k ae1 s t))
("recasting" nil (r iy0 k ae1 s t ih0 ng))
("recchia" nil (r eh1 k iy0 ax))
("recede" nil (r ih0 s iy1 d))
("receded" nil (r ax s iy1 d ax d))
("recedes" nil (r iy0 s iy1 d z))
("receding" nil (r ih0 s iy1 d ih0 ng))
("receipt" nil (r ih0 s iy1 t))
("receipts" nil (r ih0 s iy1 t s))
("receivable" nil (r ih0 s iy1 v ax b ax l))
("receivables" nil (r ih0 s iy1 v ax b ax l z))
("receive" nil (r ax s iy1 v))
("received" nil (r ax s iy1 v d))
("receiver" nil (r ax s iy1 v er0))
("receivers" nil (r ax s iy1 v er0 z))
("receivership" nil (r ih0 s iy1 v er0 sh ih1 p))
("receiverships" nil (r ih0 s iy1 v er0 sh ih1 p s))
("receives" nil (r ax s iy1 v z))
("receiving" nil (r ax s iy1 v ih0 ng))
("recent" nil (r iy1 s ax n t))
("recently" nil (r iy1 s ax n t l iy0))
("receptacle" nil (r ax s eh1 p t ax k ax l))
("receptacles" nil (r ih0 s eh1 p t ih0 k ax l z))
("receptech" nil (r iy1 s eh1 p t eh1 k))
("reception" nil (r ih0 s eh1 p sh ax n))
("receptionist" nil (r ih0 s eh1 p sh ax n ih0 s t))
("receptionists" nil (r ih0 s eh1 p sh ax n ih0 s t s))
("receptions" nil (r ih0 s eh1 p sh ax n z))
("receptive" nil (r ih0 s eh1 p t ih0 v))
("receptivity" nil (r iy1 s eh1 p t ih1 v ih0 t iy0))
("receptor" nil (r iy0 s eh1 p t er0))
("receptors" nil (r ax s eh1 p t er0 z))
("recess" nil (r ih0 s eh1 s))
("recessed" nil (r ih0 s eh1 s t))
("recesses" nil (r iy1 s eh0 s ax z))
("recession" nil (r ih0 s eh1 sh ax n))
("recessionary" nil (r iy0 s eh1 sh ax n eh1 r iy0))
("recessions" nil (r ih0 s eh1 sh ax n z))
("recessive" nil (r ax s eh1 s ih0 v))
("rech" nil (r eh1 k))
("recharge" nil (r iy0 ch aa1 r jh))
("rechargeable" nil (r iy0 ch aa1 r jh ax b ax l))
("recharged" nil (r iy0 ch aa1 r jh d))
("recharging" nil (r iy0 ch aa1 r jh ih0 ng))
("recheck" nil (r iy1 ch eh1 k))
("rechecked" nil (r iy0 ch eh1 k t))
("rechristened" nil (r iy1 k r ih1 s ax n d))
("recht" nil (r eh1 k t))
("recidivism" nil (r ax s ih1 d ih0 v ih1 z ax m))
("recine" nil (r eh0 ch iy1 n iy0))
("recio" nil (r eh1 ch iy0 ow0))
("recipe" nil (r eh1 s ax p iy0))
("recipes" nil (r eh1 s ax p iy0 z))
("recipient" nil (r ax s ih1 p iy0 ax n t))
("recipients" nil (r ih0 s ih1 p iy0 ax n t s))
("reciprocal" nil (r ih0 s ih1 p r ax k ax l))
("reciprocate" nil (r ih0 s ih1 p r ax k ey1 t))
("reciprocated" nil (r ih0 s ih1 p r ax k ey1 t ih0 d))
("reciprocating" nil (r ih0 s ih1 p r ax k ey1 t ih0 ng))
("reciprocity" nil (r eh1 s ih0 p r aa1 s ih0 t iy0))
("recision" nil (r ih0 s ih1 zh ax n))
("recission" nil (r ax s ih1 sh ax n))
("recital" nil (r ax s ay1 t ax l))
("recitals" nil (r ih0 s ay1 t ax l z))
("recitation" nil (r eh1 s ax t ey1 sh ax n))
("recitations" nil (r eh1 s ih0 t ey1 sh ax n z))
("recitatives" nil (r eh1 s ax t ax t iy1 v z))
("recite" nil (r ax s ay1 t))
("recited" nil (r ax s ay1 t ax d))
("recites" nil (r iy0 s ay1 t s))
("reciting" nil (r iy0 s ay1 t ih0 ng))
("reck" nil (r eh1 k))
("reckard" nil (r eh1 k er0 d))
("recker" nil (r eh1 k er0))
("reckitt" nil (r eh1 k ih0 t))
("reckless" nil (r eh1 k l ax s))
("recklessly" nil (r eh1 k l ax s l iy0))
("recklessness" nil (r eh1 k l ax s n ax s))
("reckner" nil (r eh1 k n er0))
("reckon" nil (r eh1 k ax n))
("reckoned" nil (r eh1 k ax n d))
("reckoning" nil (r eh1 k ax n ih0 ng))
("reckons" nil (r eh1 k ax n z))
("recktenwald" nil (r ih0 k t eh1 n w ax l d))
("reclaim" nil (r iy0 k l ey1 m))
("reclaimed" nil (r iy0 k l ey1 m d))
("reclaimer" nil (r iy0 k l ey1 m er0))
("reclaiming" nil (r iy0 k l ey1 m ih0 ng))
("reclamation" nil (r eh1 k l ax m ey1 sh ax n))
("reclassification" nil (r iy0 k l ae1 s ax f ax k ey1 sh ax n))
("reclassified" nil (r iy0 k l ae1 s ax f ay1 d))
("reclassify" nil (r iy0 k l ae1 s ih0 f ay1))
("reclassifying" nil (r iy0 k l ae1 s ih0 f ay1 ih0 ng))
("recliner" nil (r ih0 k l ay1 n er0))
("reclining" nil (r ih0 k l ay1 n ih0 ng))
("recluse" nil (r ih0 k l uw1 s))
("reclusive" nil (r ih0 k l uw1 s ih0 v))
("reco" nil (r iy1 k ow0))
("recognition" nil (r eh1 k ax g n ih1 sh ax n))
("recognizable" nil (r eh1 k ax g n ay1 z ax b ax l))
("recognizably" nil (r eh1 k ax g n ay1 z ax b l iy0))
("recognizance" nil (r ih0 k aa1 n ax z ax n s))
("recognize" nil (r eh1 k ax g n ay1 z))
("recognized" nil (r eh1 k ax g n ay1 z d))
("recognizes" nil (r eh1 k ax g n ay1 z ax z))
("recognizing" nil (r eh1 k ax g n ay1 z ih0 ng))
("recoil" nil (r iy0 k oy1 l))
("recoiled" nil (r iy0 k oy1 l d))
("recoils" nil (r iy0 k oy1 l z))
("recollect" nil (r eh1 k ax l eh1 k t))
("recollected" nil (r eh1 k ax l eh1 k t ih0 d))
("recollecting" nil (r eh1 k ax l eh1 k t ih0 ng))
("recollection" nil (r eh1 k ax l eh1 k sh ax n))
("recollections" nil (r eh1 k ax l eh1 k sh ax n z))
("recollects" nil (r eh1 k ax l eh1 k t s))
("recombinant" nil (r ih0 k aa1 m b ih0 n ax n t))
("recombine" nil (r iy1 k ax m b ay1 n))
("recommend" nil (r eh1 k ax m eh1 n d))
("recommendation" nil (r eh1 k ax m ax n d ey1 sh ax n))
("recommendations" nil (r eh1 k ax m ax n d ey1 sh ax n z))
("recommended" nil (r eh1 k ax m eh1 n d ax d))
("recommending" nil (r eh1 k ax m eh1 n d ih0 ng))
("recommends" nil (r eh1 k ax m eh1 n d z))
("recommit" nil (r ih0 k aa1 m ih0 t))
("recompense" nil (r eh1 k ax m p eh1 n s))
("reconcile" nil (r eh1 k ax n s ay1 l))
("reconciled" nil (r eh1 k ax n s ay1 l d))
("reconciles" nil (r eh1 k ax n s ay1 l z))
("reconciliation" nil (r eh1 k ax n s ih1 l iy0 ey1 sh ax n))
("reconciliations" nil (r eh1 k ax n s ih1 l iy0 ey1 sh ax n z))
("reconciling" nil (r eh1 k ax n s ay1 l ih0 ng))
("recondite" nil (r eh1 k ax n d ay1 t))
("reconditioned" nil (r iy1 k ax n d ih1 sh ax n d))
("reconditioning" nil (r iy1 k ax n d ih1 sh ax n ih0 ng))
("reconfiguration" nil (r iy0 k ax n f ih1 g y er0 ey1 sh ax n))
("reconfigure" nil (r iy1 k ax n f ih1 g y er0))
("reconfigured" nil (r iy1 k ax n f ih1 g y er0 d))
("reconfirm" nil (r iy1 k ax n f er1 m))
("reconfirmation" nil (r iy1 k aa1 n f er0 m ey1 sh ax n))
("reconfirmed" nil (r iy1 k ax n f er1 m d))
("reconnaissance" nil (r iy0 k aa1 n ax s ax n s))
("reconnect" nil (r iy0 k ax n eh1 k t))
("reconquered" nil (r iy0 k ao1 ng k er0 d))
("reconsider" nil (r iy1 k ax n s ih1 d er0))
("reconsideration" nil (r iy0 k ax n s ih1 d er0 ey1 sh ax n))
("reconsidered" nil (r iy1 k ax n s ih1 d er0 d))
("reconsidering" nil (r iy1 k ax n s ih1 d er0 ih0 ng))
("reconstitute" nil (r iy0 k aa1 n s t ax t uw1 t))
("reconstituted" nil (r iy0 k aa1 n s t ax t uw1 t ih0 d))
("reconstruct" nil (r iy1 k ax n s t r ah1 k t))
("reconstructed" nil (r iy1 k ax n s t r ah1 k t ih0 d))
("reconstructing" nil (r iy1 k ax n s t r ah1 k t ih0 ng))
("reconstruction" nil (r iy1 k ax n s t r ah1 k sh ax n))
("reconstructions" nil (r iy1 k ax n s t r ah1 k sh ax n z))
("reconstructive" nil (r iy1 k ax n s t r ah1 k t ih0 v))
("reconvene" nil (r iy0 k ax n v iy1 n))
("reconvened" nil (r iy0 k ax n v iy1 n d))
("reconvenes" nil (r iy0 k ax n v iy1 n z))
("record" v (r ax k ao1 r d))
("record" n (r eh1 k er0 d))
("recordable" nil (r ih0 k ao1 r d ax b ax l))
("recorded" nil (r ax k ao1 r d ax d))
("recorder" nil (r ih0 k ao1 r d er0))
("recorders" nil (r ih0 k ao1 r d er0 z))
("recording" nil (r ax k ao1 r d ih0 ng))
("recordings" nil (r ih0 k ao1 r d ih0 ng z))
("recordkeeping" nil (r eh1 k er0 d k iy1 p ih0 ng))
("records" v (r ax k ao1 r d z))
("records" n (r eh1 k er0 d z))
("recore" nil (r eh0 k ao1 r iy0))
("recount" v (r iy0 k aw1 n t))
("recount" n (r iy1 k aw0 n t))
("recounted" nil (r ih0 k aw1 n t ih0 d))
("recounting" nil (r ih0 k aw1 n t ih0 ng))
("recounts" n (r iy1 k aw0 n t s))
("recounts" v (r iy0 k aw1 n t s))
("recoup" nil (r ih0 k uw1 p))
("recouped" nil (r ih0 k uw1 p t))
("recouping" nil (r ih0 k uw1 p ih0 ng))
("recoups" nil (r ih0 k uw1 p s))
("recourse" nil (r iy1 k ao0 r s))
("recover" nil (r ih0 k ah1 v er0))
("recoverable" nil (r ih0 k ah1 v er0 ax b ax l))
("recovered" nil (r ax k ah1 v er0 d))
("recoveries" nil (r ih0 k ah1 v er0 iy0 z))
("recovering" nil (r ax k ah1 v er0 ih0 ng))
("recovers" nil (r ih0 k ah1 v er0 z))
("recovery" nil (r ax k ah1 v er0 iy0))
("recreate" nil (r eh1 k r iy0 ey1 t))
("recreated" nil (r eh1 k r iy0 ey1 t ih0 d))
("recreates" nil (r eh1 k r iy0 ey1 t s))
("recreating" nil (r eh1 k r iy0 ey1 t ih0 ng))
("recreation" nil (r eh1 k r iy0 ey1 sh ax n))
("recreational" nil (r eh1 k r iy0 ey1 sh ax n ax l))
("recrimination" nil (r ih0 k r ih1 m ih0 n ey1 sh ax n))
("recriminations" nil (r ih0 k r ih1 m ih0 n ey1 sh ax n z))
("recruit" nil (r ax k r uw1 t))
("recruited" nil (r ih0 k r uw1 t ih0 d))
("recruiter" nil (r ih0 k r uw1 t er0))
("recruiters" nil (r ih0 k r uw1 t er0 z))
("recruiting" nil (r ax k r uw1 t ih0 ng))
("recruitment" nil (r ax k r uw1 t m ax n t))
("recruits" nil (r ax k r uw1 t s))
("rectal" nil (r eh1 k t ax l))
("rectangle" nil (r eh1 k t ae0 ng g ax l))
("rectangles" nil (r eh1 k t ae1 ng g ax l z))
("rectangular" nil (r eh0 k t ae1 ng g y ax l er0))
("rectenwald" nil (r eh1 k t ih0 n w ao0 l d))
("rectification" nil (r eh1 k t ax f ax k ey1 sh ax n))
("rectified" nil (r eh1 k t ax f ay1 d))
("rectifier" nil (r eh1 k t ax f ay1 er0))
("rectifiers" nil (r eh1 k t ax f ay1 er0 z))
("rectify" nil (r eh1 k t ax f ay1))
("rectifying" nil (r eh1 k t ax f ay1 ih0 ng))
("rectitude" nil (r eh1 k t ih0 t uw1 d))
("rector" nil (r eh1 k t er0))
("rectum" nil (r eh1 k t ax m))
("recuperate" nil (r ih0 k uw1 p er0 ey1 t))
("recuperating" nil (r ih0 k uw1 p er0 ey1 t ih0 ng))
("recuperation" nil (r ih0 k uw1 p er0 ey1 sh ax n))
("recupero" nil (r ey0 k uw0 p eh1 r ow0))
("recur" nil (r ih0 k er1))
("recurred" nil (r iy0 k er1 d))
("recurrence" nil (r ih0 k er1 ax n s))
("recurrent" nil (r ih0 k er1 ax n t))
("recurring" nil (r ih0 k er1 ih0 ng))
("recusal" nil (r ih0 k y uw1 z ax l))
("recuse" nil (r ih1 k y uw1 z))
("recused" nil (r ih0 k y uw1 z d))
("recusing" nil (r ih1 k y uw1 z ih0 ng))
("recyclable" nil (r iy0 s ay1 k l ax b ax l))
("recycle" nil (r iy0 s ay1 k ax l))
("recycled" nil (r iy0 s ay1 k ax l d))
("recycler" nil (r iy0 s ay1 k l er0))
("recyclers" nil (r iy0 s ay1 k l er0 z))
("recycles" nil (r iy0 s ay1 k ax l z))
("recycling" nil (r iy0 s ay1 k ax l ih0 ng))
("reczek" nil (r eh1 ch eh0 k))
("red" nil (r eh1 d))
("reda" nil (r ey1 d ax))
("redact" nil (r ih0 d ae1 k t))
("redacted" nil (r ih0 d ae1 k t ih0 d))
("redactes" nil (r ih0 d ae1 k t s))
("redacting" nil (r ih0 d ae1 k t ih0 ng))
("redbird" nil (r eh1 d b er1 d))
("redbook" nil (r eh1 d b uh1 k))
("redbud" nil (r eh1 d b ah1 d))
("redburn" nil (r eh1 d b er1 n))
("redcay" nil (r eh1 d k ey0))
("redco" nil (r eh1 d k ow0))
("redcoats" nil (r eh1 d k ow1 t s))
("redd" nil (r eh1 d))
("reddell" nil (r eh1 d ax l))
("redden" nil (r eh1 d ax n))
("redder" nil (r eh1 d er0))
("reddick" nil (r eh1 d ih0 k))
("reddig" nil (r eh1 d ih0 g))
("reddin" nil (r eh1 d ih0 n))
("redding" nil (r eh1 d ih0 ng))
("reddinger" nil (r eh1 d ih0 ng er0))
("reddington" nil (r eh1 d ih0 ng t ax n))
("reddish" nil (r eh1 d ih0 sh))
("redditt" nil (r eh1 d ih0 t))
("reddoch" nil (r eh1 d ax k))
("reddy" nil (r eh1 d iy0))
("redecorate" nil (r iy0 d eh1 k er0 ey1 t))
("redecorated" nil (r iy0 d eh1 k er0 ey1 t ih0 d))
("redecorating" nil (r iy0 d eh1 k er0 ey1 t ih0 ng))
("redecoration" nil (r iy1 d eh0 k er0 ey1 sh ax n))
("redeem" nil (r ih0 d iy1 m))
("redeemable" nil (r ih0 d iy1 m ax b ax l))
("redeemed" nil (r ih0 d iy1 m d))
("redeeming" nil (r ih0 d iy1 m ih0 ng))
("redeems" nil (r ax d iy1 m z))
("redefine" nil (r iy0 d ih0 f ay1 n))
("redefined" nil (r iy0 d ih0 f ay1 n d))
("redefining" nil (r iy0 d ih0 f ay1 n ih0 ng))
("redefinition" nil (r iy0 d eh1 f ax n ih1 sh ax n))
("redeker" nil (r eh1 d ih0 k er0))
("redel" nil (r eh1 d ax l))
("redell" nil (r eh1 d ax l))
("redemption" nil (r ih0 d eh1 m p sh ax n))
("redemptions" nil (r ih0 d eh1 m p sh ax n z))
("redemptive" nil (r ih0 d eh1 m p t ih0 v))
("redenbaugh" nil (r eh1 d ih0 n b aw0))
("redenius" nil (r eh1 d iy0 n iy0 ih0 s))
("redeploy" nil (r iy1 d ih0 p l oy1))
("redeployed" nil (r iy1 d ih0 p l oy1 d))
("redeploying" nil (r iy1 d ih0 p l oy1 ih0 ng))
("redeployment" nil (r iy1 d ih0 p l oy1 m ax n t))
("redeployments" nil (r iy1 d ih0 p l oy1 m ax n t s))
("redeposition" nil (r iy0 d eh1 p ax z ih1 sh ax n))
("reder" nil (r eh1 d er0))
("redesign" nil (r iy1 d ih0 z ay1 n))
("redesigned" nil (r iy1 d ih0 z ay1 n d))
("redesigning" nil (r iy1 d ih0 z ay1 n ih0 ng))
("redevelop" nil (r iy0 d ih0 v eh1 l ax p))
("redevelopment" nil (r iy1 d ih0 v eh1 l ax p m ax n t))
("redfearn" nil (r eh1 d f er0 n))
("redfern" nil (r eh1 d f er1 n))
("redfield" nil (r eh1 d f iy1 l d))
("redfish" nil (r eh1 d f ih1 sh))
("redford" nil (r eh1 d f er0 d))
("redgrave" nil (r eh1 d g r ey1 v))
("redhanded" nil (r eh1 d hh ae1 n d ih0 d))
("redhead" nil (r eh1 d hh eh1 d))
("redheaded" nil (r eh1 d hh eh1 d ih0 d))
("redheads" nil (r eh1 d hh eh1 d z))
("redi" nil (r eh1 d iy0))
("redic" nil (r eh1 d ih0 k))
("redick" nil (r eh1 d ih0 k))
("redid" nil (r iy0 d ih1 d))
("redifer" nil (r eh1 d ih0 f er0))
("rediffusion" nil (r iy0 d ih0 f y uw1 zh ax n))
("rediger" nil (r eh1 d ih0 g er0))
("rediker" nil (r eh1 d ih0 k er0))
("reding" nil (r iy1 d ih0 ng))
("redinger" nil (r eh1 d ih0 ng er0))
("redington" nil (r eh1 d ih0 ng t ax n))
("redirect" nil (r iy1 d er0 eh1 k t))
("redirected" nil (r iy1 d er0 eh1 k t ih0 d))
("redirecting" nil (r iy1 d er0 eh1 k t ih0 ng))
("redirection" nil (r iy1 d er0 eh1 k sh ax n))
("rediscover" nil (r iy0 d ih0 s k ah1 v er0))
("rediscovered" nil (r iy0 d ih0 s k ah1 v er0 d))
("rediscovering" nil (r iy0 d ih0 s k ah1 v er0 ih0 ng))
("rediscovery" nil (r iy0 d ih0 s k ah1 v r iy0))
("redish" nil (r eh1 d ih0 sh))
("redisplay" nil (r iy0 d ih0 s p l ey1))
("redistribute" nil (r iy1 d ih0 s t r ih1 b y uw0 t))
("redistributed" nil (r iy1 d ih0 s t r ih1 b y uw0 t ih0 d))
("redistributing" nil (r iy1 d ih0 s t r ih1 b y uw0 t ih0 ng))
("redistribution" nil (r iy1 d ih0 s t r ax b y uw1 sh ax n))
("redistributive" nil (r iy0 d ih0 s t r ih1 b y uw0 t ih0 v))
("redistricting" nil (r iy0 d ih1 s t r ih0 k t ih0 ng))
("redken" nil (r eh1 d k ax n))
("redland" nil (r eh1 d l ax n d))
("redler" nil (r eh1 d l er0))
("redley" nil (r eh1 d l iy0))
("redlich" nil (r eh1 d l ih0 k))
("redlin" nil (r eh1 d l ih0 n))
("redlinger" nil (r eh1 d ax l ih0 ng er0))
("redlining" nil (r eh1 d l ay1 n ih0 ng))
("redman" nil (r eh1 d m ae1 n))
("redmann" nil (r eh1 d m ax n))
("redmon" nil (r eh1 d m ax n))
("redmond" nil (r eh1 d m ax n d))
("redmund" nil (r eh1 d m ax n d))
("redneck" nil (r eh1 d n eh1 k))
("rednecks" nil (r eh1 d n eh1 k s))
("redner" nil (r eh1 d n er0))
("redness" nil (r eh1 d n ax s))
("rednour" nil (r eh0 d n uh1 r))
("redo" nil (r iy0 d uw1))
("redoglia" nil (r ax d aa1 g l iy0 ax))
("redoing" nil (r iy0 d uw1 ih0 ng))
("redondo" nil (r ih0 d aa1 n d ow0))
("redone" nil (r iy0 d ah1 n))
("redouble" nil (r iy0 d ah1 b ax l))
("redoubled" nil (r iy0 d ah1 b ax l d))
("redoubling" nil (r iy0 d ah1 b ax l ih0 ng))
("redoubt" nil (r ax d aw1 t))
("redoubtable" nil (r ax d aw1 t ax b ax l))
("redound" nil (r ih0 d aw1 n d))
("redpath" nil (r eh1 d p ae1 th))
("redpolls" nil (r eh1 d p ow1 l z))
("redraft" nil (r iy0 d r ae1 f t))
("redrafted" nil (r iy0 d r ae1 f t ih0 d))
("redraw" nil (r iy0 d r ao1))
("redrawn" nil (r iy0 d r ao1 n))
("redress" nil (r ih0 d r eh1 s))
("redressing" nil (r iy0 d r eh1 s ih0 ng))
("reds" nil (r eh1 d z))
("redshaw" nil (r eh1 d sh ao1))
("redskin" nil (r eh1 d s k ih1 n))
("redskins" nil (r eh1 d s k ih1 n z))
("redstone" nil (r eh1 d s t ow1 n))
("reduce" nil (r ax d uw1 s))
("reduced" nil (r ax d uw1 s t))
("reduces" nil (r ax d uw1 s ax z))
("reducible" nil (r ax d uw1 s ax b ax l))
("reducing" nil (r ax d uw1 s ih0 ng))
("reduction" nil (r ax d ah1 k sh ax n))
("reductions" nil (r ih0 d ah1 k sh ax n z))
("redundancies" nil (r ih0 d ah1 n d ax n s iy0 z))
("redundancy" nil (r ih0 d ah1 n d ax n s iy0))
("redundant" nil (r ih0 d ah1 n d ax n t))
("redus" nil (r iy1 d ax s))
("redwald" nil (r eh1 d w ax l d))
("redwine" nil (r eh1 d w ay1 n))
("redwood" nil (r eh1 d w uh1 d))
("redwoods" nil (r eh1 d w uh1 d z))
("ree" nil (r iy1))
("reeb" nil (r iy1 b))
("reebok" nil (r iy1 b aa0 k))
("reeboks" nil (r iy1 b aa0 k s))
("reebs" nil (r iy1 b z))
("reece" nil (r iy1 s))
("reeck" nil (r iy1 k))
("reed" nil (r iy1 d))
("reeder" nil (r iy1 d er0))
("reeds" nil (r iy1 d z))
("reedy" nil (r iy1 d iy0))
("reef" nil (r iy1 f))
("reefs" nil (r iy1 f s))
("reeg" nil (r iy1 g))
("reeh" nil (r iy1))
("reek" nil (r iy1 k))
("reeking" nil (r iy1 k ih0 ng))
("reeks" nil (r iy1 k s))
("reel" nil (r iy1 l))
("reelected" nil (r iy0 ih0 l eh1 k t ax d))
("reelection" nil (r iy0 ih0 l eh1 k sh ax n))
("reeled" nil (r iy1 l d))
("reeling" nil (r iy1 l ih0 ng))
("reels" nil (r iy1 l z))
("reemerge" nil (r iy0 ih0 m er1 jh))
("reemerged" nil (r iy0 ih0 m er1 jh d))
("reemergence" nil (r iy0 ih0 m er1 jh ax n s))
("reemployment" nil (r iy0 ih0 m p l oy1 m ax n t))
("reen" nil (r iy1 n))
("reenacted" nil (r iy0 ih0 n ae1 k t ih0 d))
("reenactment" nil (r iy0 ih0 n ae1 k t m ax n t))
("reenactments" nil (r iy0 ih0 n ae1 k t m ax n t s))
("reenacts" nil (r iy0 ih0 n ae1 k t s))
("reengineering" nil (r iy0 eh1 n jh ax n ih1 r ih0 ng))
("reenter" nil (r iy0 ih1 n t er0))
("reentering" nil (r iy0 ih1 n t er0 ih0 ng))
("reentry" nil (r iy0 ih1 n t r iy0))
("reents" nil (r iy1 n t s))
("reep" nil (r iy1 p))
("rees" nil (r iy1 z))
("reese" nil (r iy1 s))
("reeser" nil (r iy1 s er0))
("reesman" nil (r iy1 s m ax n))
("reesor" nil (r iy1 s er0))
("reestablish" nil (r iy0 ih0 s t ae1 b l ih0 sh))
("reestablished" nil (r iy1 ih0 s t ae1 b l ih0 sh t))
("reestablishing" nil (r iy0 ih0 s t ae1 b l ih0 sh ih0 ng))
("reetz" nil (r iy1 t s))
("reevaluate" nil (r iy1 ih0 v ae1 l uw0 ey1 t))
("reevaluating" nil (r iy0 ih0 v ae1 l y uw0 ey1 t ih0 ng))
("reevaluation" nil (r iy0 ih0 v ae1 l y uw0 ey1 sh ax n))
("reeve" nil (r iy1 v))
("reever" nil (r iy1 v er0))
("reeves" nil (r iy1 v z))
("reexamination" nil (r iy0 ih0 g z ae1 m ax n ey1 sh ax n))
("reexamine" nil (r iy0 eh0 g z ae1 m ax n))
("reexamined" nil (r iy0 ih0 g z ae1 m ax n d))
("reexamining" nil (r iy0 ih0 g z ae1 m ax n ih0 ng))
("reexport" nil (r iy0 eh1 k s p ao1 r t))
("reexports" nil (r iy0 eh1 k s p ao1 r t s))
("ref" nil (r eh1 f))
("refaat" nil (r ax f aa1 t))
("refah" nil (r ax f aa1))
("refco" nil (r eh1 f k ow0))
("refcorp" nil (r eh1 f k ao0 r p))
("refenes" nil (r eh1 f ih0 n eh1 s))
("refer" nil (r ax f er1))
("referee" nil (r eh1 f er0 iy1))
("referees" nil (r eh1 f er0 iy1 z))
("reference" nil (r eh1 f er0 ax n s))
("referenced" nil (r eh1 f er0 ax n s t))
("references" nil (r eh1 f er0 ax n s ih0 z))
("referenda" nil (r eh1 f er0 eh1 n d ax))
("referendum" nil (r eh1 f er0 eh1 n d ax m))
("referendums" nil (r eh1 f er0 eh1 n d ax m z))
("refering" nil (r ih0 f er1 ih0 ng))
("referral" nil (r ih0 f er1 ax l))
("referrals" nil (r ih0 f er1 ax l z))
("referred" nil (r ax f er1 d))
("referring" nil (r ih0 f er1 ih0 ng))
("refers" nil (r ax f er1 z))
("reff" nil (r eh1 f))
("reffett" nil (r eh1 f ih0 t))
("reffitt" nil (r eh1 f ih0 t))
("reffner" nil (r eh1 f n er0))
("refile" nil (r iy0 f ay1 l))
("refiled" nil (r iy0 f ay1 l d))
("refill" nil (r iy1 f ih1 l))
("refills" nil (r iy1 f ih1 l z))
("refinance" nil (r iy1 f ax n ae1 n s))
("refinanced" nil (r iy1 f ax n ae1 n s t))
("refinances" nil (r iy1 f ax n ae1 n s ih0 z))
("refinancing" nil (r iy1 f ax n ae1 n s ih0 ng))
("refinancings" nil (r iy1 f ax n ae1 n s ih0 ng z))
("refine" nil (r ax f ay1 n))
("refined" nil (r ax f ay1 n d))
("refinement" nil (r ax f ay1 n m ax n t))
("refinements" nil (r ih0 f ay1 n m ax n t s))
("refiner" nil (r ih0 f ay1 n er0))
("refineries" nil (r ih0 f ay1 n er0 iy0 z))
("refiners" nil (r ih0 f ay1 n er0 z))
("refinery" nil (r ih0 f ay1 n er0 iy0))
("refines" nil (r ih0 f ay1 n z))
("refining" nil (r ax f ay1 n ih0 ng))
("refinish" nil (r iy0 f ih1 n ih0 sh))
("refinished" nil (r iy0 f ih1 n ih0 sh t))
("refinishing" nil (r iy0 f ih1 n ih0 sh ih0 ng))
("refit" nil (r iy0 f ih1 t))
("refitted" nil (r iy0 f ih1 t ih0 d))
("refitting" nil (r iy0 f ih1 t ih0 ng))
("reflag" nil (r iy0 f l ae1 g))
("reflagged" nil (r iy0 f l ae1 g d))
("reflagging" nil (r iy0 f l ae1 g ih0 ng))
("reflate" nil (r iy0 f l ey1 t))
("reflation" nil (r iy0 f l ey1 sh ax n))
("reflect" nil (r ax f l eh1 k t))
("reflected" nil (r ax f l eh1 k t ax d))
("reflecting" nil (r ax f l eh1 k t ih0 ng))
("reflection" nil (r ax f l eh1 k sh ax n))
("reflections" nil (r ih0 f l eh1 k sh ax n z))
("reflective" nil (r ih0 f l eh1 k t ih0 v))
("reflectone" nil (r iy0 f l eh1 k t ow1 n))
("reflects" nil (r ih0 f l eh1 k t s))
("reflex" nil (r iy1 f l eh0 k s))
("reflexes" nil (r iy1 f l eh0 k s ax z))
("reflexive" nil (r ax f l eh1 k s ih0 v))
("reflexively" nil (r iy0 f l eh1 k s ih0 v l iy0))
("reflexivity" nil (r iy1 f l eh1 k s ih1 v ih0 t iy0))
("refocus" nil (r iy0 f ow1 k ax s))
("refocused" nil (r iy0 f ow1 k ax s t))
("refocuses" nil (r iy0 f ow1 k ax s ih0 z))
("refocusing" nil (r iy0 f ow1 k ax s ih0 ng))
("reforest" nil (r iy0 f ao1 r ax s t))
("reforestation" nil (r iy1 f ao0 r ax s t ey1 sh ax n))
("reform" nil (r ax f ao1 r m))
("reforma" nil (r ih0 f ao1 r m ax))
("reformation" nil (r eh1 f er0 m ey1 sh ax n))
("reformatories" nil (r ih0 f ao1 r m ax t ao1 r iy0 z))
("reformatory" nil (r ih0 f ao1 r m ax t ao1 r iy0))
("reformed" nil (r ih0 f ao1 r m d))
("reformer" nil (r ih0 f ao1 r m er0))
("reformers" nil (r ih0 f ao1 r m er0 z))
("reforming" nil (r ih0 f ao1 r m ih0 ng))
("reformist" nil (r ih0 f ao1 r m ih0 s t))
("reformists" nil (r ih0 f ao1 r m ih0 s t s))
("reforms" nil (r ax f ao1 r m z))
("reformulate" nil (r iy0 f ao1 r m y ax l ey1 t))
("reformulated" nil (r iy0 f ao1 r m y ax l ey1 t ih0 d))
("refractive" nil (r ax f r ae1 k t ih0 v))
("refractories" nil (r ih0 f r ae1 k t er0 iy0 z))
("refractors" nil (r ax f r ae1 k t er0 z))
("refractory" nil (r ax f r ae1 k t er0 iy0))
("refrain" nil (r ih0 f r ey1 n))
("refrained" nil (r ih0 f r ey1 n d))
("refraining" nil (r ih0 f r ey1 n ih0 ng))
("refrains" nil (r ih0 f r ey1 n z))
("refresh" nil (r ih0 f r eh1 sh))
("refreshed" nil (r iy0 f r eh1 sh t))
("refresher" nil (r ih0 f r eh1 sh er0))
("refreshes" nil (r ih0 f r eh1 sh ih0 z))
("refreshing" nil (r ih0 f r eh1 sh ih0 ng))
("refreshingly" nil (r iy0 f r eh1 sh ih0 ng l iy0))
("refreshment" nil (r ax f r eh1 sh m ax n t))
("refreshments" nil (r ax f r eh1 sh m ax n t s))
("refrigerant" nil (r ih0 f r ih1 jh er0 ax n t))
("refrigerants" nil (r ih0 f r ih1 jh er0 ax n t s))
("refrigerated" nil (r ih0 f r ih1 jh er0 ey1 t ih0 d))
("refrigeration" nil (r ih0 f r ih1 jh er0 ey1 sh ax n))
("refrigerator" nil (r ax f r ih1 jh er0 ey1 t er0))
("refrigerators" nil (r ih0 f r ih1 jh er0 ey1 t er0 z))
("refsnes" nil (r eh1 f s n iy0 z))
("refuel" nil (r iy0 f y uw1 ax l))
("refueled" nil (r iy0 f y uw1 ax l d))
("refueling" nil (r iy0 f y uw1 ax l ih0 ng))
("refuge" nil (r eh1 f y uw0 jh))
("refugee" nil (r eh1 f y uh0 jh iy1))
("refugees" nil (r eh1 f y uw1 jh iy1 z))
("refuges" nil (r eh1 f y uw0 jh ih0 z))
("refund" nil (r ih0 f ah1 n d))
("refundable" nil (r ih0 f ah1 n d ax b ax l))
("refunded" nil (r ih0 f ah1 n d ih0 d))
("refunding" nil (r ih0 f ah1 n d ih0 ng))
("refundings" nil (r iy1 f ah1 n d ih0 ng z))
("refunds" nil (r ih0 f ah1 n d z))
("refurbish" nil (r iy0 f er1 b ih0 sh))
("refurbished" nil (r iy0 f er1 b ih0 sh t))
("refurbishing" nil (r iy0 f er1 b ih0 sh ih0 ng))
("refurbishment" nil (r iy0 f er1 b ih0 sh m ax n t))
("refusal" nil (r ax f y uw1 z ax l))
("refusals" nil (r ih0 f y uw1 z ax l z))
("refuse" v (r ax f y uw1 z))
("refuse" n (r eh1 f y uw1 s))
("refused" nil (r ax f y uw1 z d))
("refuseniks" nil (r ih0 f y uw1 z n ih0 k s))
("refuses" nil (r ax f y uw1 z ax z))
("refusing" nil (r ax f y uw1 z ih0 ng))
("refutation" nil (r eh1 f y uw0 t ey1 sh ax n))
("refute" nil (r ih0 f y uw1 t))
("refuted" nil (r ih0 f y uw1 t ih0 d))
("refutes" nil (r ih0 f y uw1 t s))
("refuting" nil (r ih0 f y uw1 t ih0 ng))
("reg" nil (r eh1 g))
("rega" nil (r iy1 g ax))
("regain" nil (r ih0 g ey1 n))
("regained" nil (r iy0 g ey1 n d))
("regaining" nil (r ih0 g ey1 n ih0 ng))
("regains" nil (r iy0 g ey1 n z))
("regal" nil (r iy1 g ax l))
("regalado" nil (r ey0 g aa0 l aa1 d ow0))
("regaled" nil (r ih0 g ey1 l d))
("regalia" nil (r ih0 g ey1 l y ax))
("regaling" nil (r ih0 g ey1 l ih0 ng))
("regally" nil (r iy1 g ax l iy0))
("regan" nil (r iy1 g ax n))
("regard" nil (r ax g aa1 r d))
("regarded" nil (r ax g aa1 r d ax d))
("regarding" nil (r ax g aa1 r d ih0 ng))
("regardless" nil (r ax g aa1 r d l ax s))
("regards" nil (r ih0 g aa1 r d z))
("regas" nil (r iy1 g ax z))
("regatta" nil (r ih0 g aa1 t ax))
("regattas" nil (r ax g aa1 t ax z))
("regehr" nil (r eh1 g ih0 r))
("regel" nil (r eh1 g ax l))
("regen" nil (r eh1 g ax n))
("regency" nil (r iy1 jh ax n s iy0))
("regenerate" nil (r iy0 jh eh1 n er0 ey1 t))
("regenerated" nil (r iy0 jh eh1 n er0 ey1 t ih0 d))
("regenerating" nil (r iy0 jh eh1 n er0 ey1 t ih0 ng))
("regeneration" nil (r iy0 jh eh1 n er0 ey1 sh ax n))
("regeneron" nil (r iy0 jh eh1 n er0 ao1 n))
("regensburg" nil (r iy1 g ax n z b er0 g))
("regensteiner" nil (r eh1 g ax n s t ay1 n er0))
("regent" nil (r iy1 jh ax n t))
("regents" nil (r iy1 jh ax n t s))
("reger" nil (r iy1 g er0))
("regester" nil (r eh1 jh ih0 s t er0))
("reggae" nil (r eh1 g ey1))
("reggie" nil (r eh1 jh iy0))
("reggio" nil (r eh1 jh iy0 ow0))
("reggy" nil (r eh1 g iy0))
("regie" nil (r eh1 g iy0))
("regier" nil (r iy1 g iy0 er0))
("regime" nil (r ax zh iy1 m))
("regimen" nil (r eh1 jh ax m ax n))
("regimens" nil (r eh1 jh ax m ax n z))
("regiment" nil (r eh1 jh ax m ax n t))
("regimental" nil (r eh1 jh ax m eh1 n t ax l))
("regimentation" nil (r eh1 jh ax m eh0 n t ey1 sh ax n))
("regimented" nil (r eh1 jh ax m eh1 n t ih0 d))
("regiments" nil (r eh1 jh ax m ax n t s))
("regimes" nil (r ax zh iy1 m z))
("regina" nil (r ih0 jh iy1 n ax))
("reginald" nil (r eh1 jh ax n ax l d))
("region" nil (r iy1 jh ax n))
("regional" nil (r iy1 jh ax n ax l))
("regionally" nil (r iy1 jh ax n ax l iy0))
("regionals" nil (r iy1 jh ax n ax l z))
("regions" nil (r iy1 jh ax n z))
("regis" nil (r iy1 jh ih0 s))
("register" nil (r eh1 jh ih0 s t er0))
("registered" nil (r eh1 jh ax s t er0 d))
("registering" nil (r eh1 jh ih0 s t er0 ih0 ng))
("registers" nil (r eh1 jh ax s t er0 z))
("registrants" nil (r eh1 jh ax s t r ax n t s))
("registrar" nil (r eh1 jh ih0 s t r aa1 r))
("registrars" nil (r eh1 jh ih0 s t r aa1 r z))
("registration" nil (r eh1 jh ax s t r ey1 sh ax n))
("registrations" nil (r eh1 jh ih0 s t r ey1 sh ax n z))
("registries" nil (r eh1 jh ih0 s t r iy0 z))
("registry" nil (r eh1 jh ih0 s t r iy0))
("regner" nil (r eh1 g n er0))
("regnery" nil (r eh1 g n er0 iy0))
("regnier" nil (r eh1 g n iy0 er0))
("rego" nil (r eh1 g ow0))
("regress" nil (r iy1 g r eh0 s))
("regression" nil (r ax g r eh1 sh ax n))
("regressions" nil (r ax g r eh1 sh ax n z))
("regressive" nil (r ax g r eh1 s ih0 v))
("regret" nil (r ax g r eh1 t))
("regretful" nil (r ih0 g r eh1 t f ax l))
("regretfully" nil (r ih0 g r eh1 t f ax l iy0))
("regrets" nil (r ih0 g r eh1 t s))
("regrettable" nil (r ih0 g r eh1 t ax b ax l))
("regrettably" nil (r ih0 g r eh1 t ax b l iy0))
("regretted" nil (r ih0 g r eh1 t ih0 d))
("regretting" nil (r ih0 g r eh1 t ih0 ng))
("regroup" nil (r iy0 g r uw1 p))
("regrouped" nil (r iy0 g r uw1 p t))
("regrouping" nil (r iy0 g r uw1 p ih0 ng))
("regula" nil (r eh0 g uw1 l ax))
("regular" nil (r eh1 g y ax l er0))
("regularity" nil (r eh1 g y ax l eh1 r ax t iy0))
("regularly" nil (r eh1 g y ax l er0 l iy0))
("regulars" nil (r eh1 g y ax l er0 z))
("regulate" nil (r eh1 g y ax l ey1 t))
("regulated" nil (r eh1 g y ax l ey1 t ax d))
("regulates" nil (r eh1 g y ax l ey1 t s))
("regulating" nil (r eh1 g y ax l ey1 t ih0 ng))
("regulation" nil (r eh1 g y ax l ey1 sh ax n))
("regulations" nil (r eh1 g y ax l ey1 sh ax n z))
("regulative" nil (r eh1 g y ax l ey1 t ih0 v))
("regulator" nil (r eh1 g y ax l ey1 t er0))
("regulators" nil (r eh1 g y ax l ey1 t er0 z))
("regulatory" nil (r eh1 g y ax l ax t ao1 r iy0))
("reh" nil (r eh1))
("reha" nil (r iy1 hh ax))
("rehab" nil (r iy0 hh ae0 b))
("rehabilitate" nil (r iy1 hh ax b ih1 l ax t ey1 t))
("rehabilitated" nil (r iy1 hh ax b ih1 l ax t ey1 t ih0 d))
("rehabilitating" nil (r iy1 hh ax b ih1 l ih0 t ey1 t ih0 ng))
("rehabilitation" nil (r iy1 hh ax b ih1 l ih0 t ey1 sh ax n))
("rehabilitations" nil (r iy1 hh ax b ih1 l ax t ey1 sh ax n z))
("rehabilitative" nil (r iy1 hh ax b ih1 l ax t ey1 t ih0 v))
("rehability" nil (r iy1 hh ax b ih1 l ax t iy0))
("rehak" nil (r eh1 hh ax k))
("rehash" nil (r iy0 hh ae1 sh))
("rehashing" nil (r iy0 hh ae1 sh ih0 ng))
("rehbein" nil (r eh1 b ay0 n))
("rehberg" nil (r eh1 b er0 g))
("rehberger" nil (r eh1 b er0 g er0))
("rehder" nil (r eh1 d er0))
("rehear" nil (r iy0 hh ih1 r))
("rehearing" nil (r iy0 hh ih1 r ih0 ng))
("rehearings" nil (r iy0 hh ih1 r ih0 ng z))
("rehearsal" nil (r ih0 hh er1 s ax l))
("rehearsals" nil (r ih0 hh er1 s ax l z))
("rehearse" nil (r iy0 hh er1 s))
("rehearsed" nil (r iy0 hh er1 s t))
("rehearsing" nil (r ih0 hh er1 s ih0 ng))
("reher" nil (r eh1 r))
("rehfeld" nil (r eh1 f eh1 l d))
("rehfeldt" nil (r eh1 f eh1 l t))
("rehg" nil (r eh1 jh))
("rehire" nil (r iy0 hh ay1 r))
("rehired" nil (r iy0 hh ay1 er0 d))
("rehiring" nil (r iy0 hh ay1 r ih0 ng))
("rehkopf" nil (r eh1 k ao0 p f))
("rehling" nil (r eh1 l ih0 ng))
("rehm" nil (r eh1 m))
("rehman" nil (r eh1 m ax n))
("rehmann" nil (r eh1 m ax n))
("rehmer" nil (r eh1 m er0))
("rehn" nil (r eh1 n))
("rehnquist" nil (r eh1 n k w ih1 s t))
("rehor" nil (r eh1 hh er0))
("rehrig" nil (r eh1 r ih0 g))
("rehydration" nil (r iy1 hh ay0 d r ey1 sh ax n))
("reibel" nil (r ay1 b ax l))
("reiber" nil (r ay1 b er0))
("reich" nil (r ay1 k))
("reichard" nil (r ay1 k er0 d))
("reichardt" nil (r ay1 k aa0 r t))
("reichart" nil (r iy1 ih0 k hh aa0 r t))
("reiche" nil (r ay1 k))
("reichel" nil (r ay1 k ax l))
("reichelderfer" nil (r ay1 k ih0 l d er0 f er0))
("reichelt" nil (r ay1 k ih0 l t))
("reichenbach" nil (r ay1 k ax n b aa1 k))
("reichenberg" nil (r ay1 k ax n b er0 g))
("reichenberger" nil (r ay1 k ax n b er0 g er0))
("reicher" nil (r ay1 k er0))
("reichert" nil (r ay1 k er0 t))
("reichhold" nil (r ay1 k hh ow1 l d))
("reichl" nil (r ay1 k ax l))
("reichle" nil (r ay1 k ax l))
("reichler" nil (r ay1 k l er0))
("reichley" nil (r ay1 k l iy0))
("reichlin" nil (r ay1 k l ih0 n))
("reichling" nil (r ay1 k ax l ih0 ng))
("reichman" nil (r ay1 k m ax n))
("reichmann" nil (r ay1 k m ax n))
("reichmanns" nil (r ay1 k m ax n z))
("reichmuth" nil (r ay1 k m uw1 th))
("reichow" nil (r ay1 k ow0))
("reichstuhl" nil (r ay1 k s t uw0 l))
("reichwein" nil (r ay1 k w ay1 n))
("reick" nil (r iy1 k))
("reicks" nil (r ay1 k s))
("reid" nil (r iy1 d))
("reidel" nil (r ay1 d ax l))
("reidenbach" nil (r ay1 d ih0 n b aa0 k))
("reider" nil (r iy1 d er0))
("reidinger" nil (r ay1 d ih0 ng er0))
("reidt" nil (r iy1 d t))
("reidy" nil (r iy1 d iy0))
("reier" nil (r ey1 er0))
("reierson" nil (r ay1 er0 s ax n))
("reif" nil (r iy1 f))
("reifel" nil (r ay1 f ax l))
("reiff" nil (r iy1 f))
("reifschneider" nil (r ay1 f sh n ay0 d er0))
("reifsnyder" nil (r ay1 f s n ay0 d er0))
("reifsteck" nil (r ay1 f s t ih0 k))
("reigel" nil (r ay1 g ax l))
("reiger" nil (r ay1 g er0))
("reighard" nil (r ay1 g hh er0 d))
("reigle" nil (r iy1 g ax l))
("reign" nil (r ey1 n))
("reigned" nil (r ey1 n d))
("reigning" nil (r ey1 n ih0 ng))
("reignite" nil (r iy0 ih0 g n ay1 t))
("reignited" nil (r iy0 ih0 g n ay1 t ih0 d))
("reigniting" nil (r iy0 ih0 g n ay1 t ih0 ng))
("reigns" nil (r ey1 n z))
("reiher" nil (r ay1 hh er0))
("reihl" nil (r ay1 l))
("reil" nil (r ey1 l))
("reiland" nil (r ay1 l ax n d))
("reiley" nil (r ey1 l iy0))
("reiling" nil (r ay1 l ih0 ng))
("reilley" nil (r ey1 l iy0))
("reilly" nil (r ay1 l iy0))
("reily" nil (r iy1 l iy0))
("reim" nil (r iy1 m))
("reiman" nil (r ay1 m ax n))
("reimann" nil (r ay1 m ax n))
("reimburse" nil (r iy1 ih0 m b er1 s))
("reimbursed" nil (r iy1 ih0 m b er1 s t))
("reimbursement" nil (r iy1 ih0 m b er1 s m ax n t))
("reimbursements" nil (r iy1 ih0 m b er1 s m ax n t s))
("reimburses" nil (r iy1 ih0 m b er1 s ih0 z))
("reimbursing" nil (r iy1 ih0 m b er1 s ih0 ng))
("reimer" nil (r ay1 m er0))
("reimers" nil (r ay1 m er0 z))
("reimpose" nil (r iy0 ih0 m p ow1 z))
("reimposed" nil (r iy0 ih0 m p ow1 z d))
("reimposing" nil (r iy0 ih0 m p ow1 z ih0 ng))
("reimposition" nil (r iy0 ih0 m p ax z ih1 sh ax n))
("rein" nil (r ey1 n))
("reina" nil (r ey0 iy1 n ax))
("reinaldo" nil (r ey1 n aa1 l d ow0))
("reinard" nil (r ay1 n er0 d))
("reinbold" nil (r ay1 n b ow1 l d))
("reinbolt" nil (r ay1 n b ow1 l t))
("reincarnated" nil (r iy1 ih0 n k aa1 r n ey1 t ih0 d))
("reincarnation" nil (r iy0 ih0 n k aa0 r n ey1 sh ax n))
("reincke" nil (r ay1 ng k))
("reincorporate" nil (r iy0 ih0 n k ao1 r p er0 ey1 t))
("reincorporating" nil (r iy0 ih0 n k ao1 r p er0 ey1 t ih0 ng))
("reincorporation" nil (r iy0 ih0 n k ao1 r p er0 ey1 sh ax n))
("reindeer" nil (r ey1 n d ih1 r))
("reindel" nil (r ay1 n d ax l))
("reinders" nil (r ay1 n d er0 z))
("reindl" nil (r ay1 n d ax l))
("reine" nil (r ey1 n))
("reineck" nil (r ay1 n eh0 k))
("reinecke" nil (r ay1 n ih0 k))
("reined" nil (r ey1 n d))
("reineke" nil (r ay1 n ih0 k))
("reiner" nil (r ay1 n er0))
("reiners" nil (r ay1 n er0 z))
("reinert" nil (r ay1 n er0 t))
("reinertsen" nil (r ay1 n er0 t s ax n))
("reinertson" nil (r ay1 n er0 t s ax n))
("reines" nil (r ey1 n z))
("reinforce" nil (r iy1 ih0 n f ao1 r s))
("reinforced" nil (r iy1 ih0 n f ao1 r s t))
("reinforcement" nil (r iy1 ih0 n f ao1 r s m ax n t))
("reinforcements" nil (r iy1 ih0 n f ao1 r s m ax n t s))
("reinforces" nil (r iy1 ih0 n f ao1 r s ih0 z))
("reinforcing" nil (r iy1 ih0 n f ao1 r s ih0 ng))
("reingold" nil (r ay1 ng g ow0 l d))
("reinhard" nil (r ay1 n hh aa1 r d))
("reinhardt" nil (r ay1 n hh aa1 r t))
("reinhart" nil (r ay1 n hh aa0 r t))
("reinheimer" nil (r ay1 n hh ay1 m er0))
("reinhold" nil (r ay1 n hh ow0 l t))
("reinholz" nil (r ay1 n hh ow0 l z))
("reinig" nil (r ay1 n ih0 g))
("reining" nil (r ey1 n ih0 ng))
("reininger" nil (r ay1 n ih0 ng er0))
("reinitz" nil (r ay1 n ih0 t s))
("reinke" nil (r ey1 ng k))
("reinking" nil (r ay1 ng k ih0 ng))
("reinoehl" nil (r ay1 n ow0 l))
("reins" nil (r ey1 n z))
("reinsch" nil (r ay1 n sh))
("reinsdorf" nil (r ay1 n z d ao1 r f))
("reinsel" nil (r ay1 n s ax l))
("reinspection" nil (r iy0 ih0 n s p eh1 k sh ax n))
("reinstate" nil (r iy1 ih0 n s t ey1 t))
("reinstated" nil (r iy1 ih0 n s t ey1 t ih0 d))
("reinstatement" nil (r iy1 ih0 n s t ey1 t m ax n t))
("reinstating" nil (r iy1 ih0 n s t ey1 t ih0 ng))
("reinstein" nil (r ay1 n s t ay1 n))
("reinstitute" nil (r iy0 ih1 n s t ih0 t uw0 t))
("reinstituted" nil (r iy0 ih1 n s t ih0 t uw0 t ih0 d))
("reinstituting" nil (r iy0 ih1 n s t ax t uw1 t ih0 ng))
("reinsurance" nil (r iy1 ih0 n sh uh1 r ax n s))
("reinsure" nil (r iy1 ih0 n sh uh1 r))
("reinsured" nil (r iy1 ih0 n sh uh1 r d))
("reinsurer" nil (r iy1 ih0 n sh uh1 r er0))
("reinsurers" nil (r iy1 ih0 n sh uh1 r er0 z))
("reintegrated" nil (r iy0 ih1 n t ax g r ey1 t ih0 d))
("reinterpret" nil (r iy1 ih0 n t er1 p r ax t))
("reinterpretation" nil (r iy0 ih0 n t er1 p r ax t ey1 sh ax n))
("reinterpreted" nil (r iy0 ih0 n t er1 p r ax t ih0 d))
("reinterpreting" nil (r iy0 ih0 n t er1 p r ax t ih0 ng))
("reintroduce" nil (r iy0 ih0 n t r ax d uw1 s))
("reintroduced" nil (r iy0 ih0 n t r ax d uw1 s t))
("reintroduces" nil (r iy0 ih0 n t r ax d uw1 s ih0 z))
("reintroducing" nil (r iy0 ih0 n t r ax d uw1 s ih0 ng))
("reintroduction" nil (r iy0 ih0 n t r ax d ah1 k sh ax n))
("reints" nil (r ay1 n t s))
("reinvent" nil (r iy0 ih0 n v eh1 n t))
("reinvented" nil (r iy0 ih0 n v eh1 n t ih0 d))
("reinventing" nil (r iy0 ih0 n v eh1 n t ih0 ng))
("reinvest" nil (r iy1 ih0 n v eh1 s t))
("reinvested" nil (r iy1 ih0 n v eh1 s t ih0 d))
("reinvesting" nil (r iy1 ih0 n v eh1 s t ih0 ng))
("reinvestment" nil (r iy1 ih0 n v eh1 s t m ax n t))
("reinvests" nil (r iy1 ih0 n v eh1 s t s))
("reinvigorate" nil (r iy1 ih0 n v ih1 g er0 ey1 t))
("reinvigorated" nil (r iy1 ih0 n v ih1 g er0 ey1 t ih0 d))
("reinvigorating" nil (r iy1 ih0 n v ih1 g er0 ey1 t ih0 ng))
("reinvigoration" nil (r iy0 ih0 n v ih1 g er0 ey1 sh ax n))
("reinwald" nil (r ay1 n w ao1 l d))
("reis" nil (r iy1 z))
("reisch" nil (r ay1 sh))
("reischauer" nil (r ay1 sh aw0 r))
("reischman" nil (r ay1 sh m ax n))
("reisdorf" nil (r ay1 s d ao0 r f))
("reise" nil (r iy1 s))
("reisen" nil (r ay1 s ax n))
("reisenauer" nil (r ay1 s ih0 n aw0 er0))
("reiser" nil (r ay1 s er0))
("reisert" nil (r ay1 s er0 t))
("reish" nil (r iy1 ih0 sh))
("reisig" nil (r ay1 s ih0 g))
("reising" nil (r ay1 s ih0 ng))
("reisinger" nil (r ay1 s ih0 n jh er0))
("reisman" nil (r ay1 s m ax n))
("reisner" nil (r ay1 s n er0))
("reiss" nil (r ay1 s))
("reissig" nil (r ay1 s ih0 g))
("reissman" nil (r ay1 s m ax n))
("reissue" nil (r iy0 ih1 sh uw0))
("reissued" nil (r iy0 ih1 sh uw0 d))
("reissuing" nil (r iy0 ih1 sh uw0 ih0 ng))
("reist" nil (r iy1 ih0 s t))
("reister" nil (r iy1 ih0 s t er0))
("reiswig" nil (r ay1 s w ih0 g))
("reisz" nil (r ay1 sh))
("reit" nil (r ay1 t))
("reitan" nil (r ay1 t ax n))
("reitano" nil (r ey0 iy0 t aa1 n ow0))
("reiten" nil (r ay1 t ax n))
("reiter" nil (r ay1 t er0))
("reiterate" nil (r iy0 ih1 t er0 ey1 t))
("reiterated" nil (r iy0 ih1 t er0 ey1 t ih0 d))
("reiterates" nil (r iy0 ih1 t er0 ey1 s))
("reiterating" nil (r iy0 ih1 t er0 ey1 t ih0 ng))
("reiteration" nil (r iy0 ih1 t er0 ey1 sh ax n))
("reith" nil (r iy1 th))
("reither" nil (r ay1 dh er0))
("reitman" nil (r ay1 t m ax n))
("reitmeier" nil (r ay1 t m ay0 er0))
("reitmeyer" nil (r ay1 t m ay0 er0))
("reits" nil (r ay1 t s))
("reitsma" nil (r ay1 t s m ax))
("reitter" nil (r ay1 t er0))
("reitz" nil (r ay1 t s))
("reitzel" nil (r ay1 t s ax l))
("reitzes" nil (r ay1 t s ih0 z))
("reitzfeld" nil (r iy1 t s f eh0 l d))
("reject" v (r ih0 jh eh1 k t))
("reject" n (r iy1 jh eh0 k t))
("rejected" nil (r ih0 jh eh1 k t ih0 d))
("rejecting" nil (r ih0 jh eh1 k t ih0 ng))
("rejection" nil (r ih0 jh eh1 k sh ax n))
("rejections" nil (r ih0 jh eh1 k sh ax n z))
("rejects" v (r ih0 jh eh1 k t s))
("rejects" n (r iy1 jh eh0 k t s))
("rejiggering" nil (r iy0 jh ih1 g er0 ih0 ng))
("rejoice" nil (r ih0 jh oy1 s))
("rejoiced" nil (r ih0 jh oy1 s t))
("rejoicing" nil (r ih0 jh oy1 s ih0 ng))
("rejoin" nil (r iy0 jh oy1 n))
("rejoinder" nil (r ih0 jh oy1 n d er0))
("rejoined" nil (r iy0 jh oy1 n d))
("rejoining" nil (r iy0 jh oy1 n ih0 ng))
("rejoins" nil (r iy0 jh oy1 n z))
("rejuvenate" nil (r ih0 jh uw1 v ax n ey1 t))
("rejuvenated" nil (r ih0 jh uw1 v ax n ey1 t ih0 d))
("rejuvenates" nil (r ih0 jh uw1 v ih0 n ey1 t s))
("rejuvenating" nil (r iy0 jh uw1 v ax n ey1 t ih0 ng))
("rejuvenation" nil (r ih0 jh uw1 v ax n ey1 sh ax n))
("rekenthaler" nil (r eh1 k ax n t aa1 l er0))
("reker" nil (r iy1 k er0))
("rekindle" nil (r iy0 k ih1 n d ax l))
("rekindled" nil (r iy0 k ih1 n d ax l d))
("rekindling" nil (r iy0 k ih1 n d l ih0 ng))
("relabel" nil (r iy0 l ey1 b ax l))
("relapse" nil (r iy0 l ae1 p s))
("relate" nil (r ih0 l ey1 t))
("related" nil (r ih0 l ey1 t ih0 d))
("relates" nil (r ih0 l ey1 t s))
("relating" nil (r ih0 l ey1 t ih0 ng))
("relation" nil (r iy0 l ey1 sh ax n))
("relational" nil (r iy0 l ey1 sh ax n ax l))
("relations" nil (r iy0 l ey1 sh ax n z))
("relationship" nil (r iy0 l ey1 sh ax n sh ih1 p))
("relationships" nil (r iy0 l ey1 sh ax n sh ih1 p s))
("relative" nil (r eh1 l ax t ih0 v))
("relatively" nil (r eh1 l ax t ih0 v l iy0))
("relatives" nil (r eh1 l ax t ih0 v z))
("relativism" nil (r eh1 l ax t ih0 v ih1 z ax m))
("relativistic" nil (r eh1 l ax t ih0 v ih1 s t ih0 k))
("relativity" nil (r eh1 l ax t ih1 v ax t iy0))
("relaunch" nil (r iy0 l ao1 n ch))
("relaunched" nil (r iy0 l ao1 n ch t))
("relax" nil (r ih0 l ae1 k s))
("relaxation" nil (r iy1 l ae0 k s ey1 sh ax n))
("relaxed" nil (r ih0 l ae1 k s t))
("relaxes" nil (r ih0 l ae1 k s ih0 z))
("relaxing" nil (r ih0 l ae1 k s ih0 ng))
("relay" nil (r iy1 l ey1))
("relayed" nil (r iy1 l ey1 d))
("relaying" nil (r iy1 l ey1 ih0 ng))
("relays" nil (r iy1 l ey1 z))
("release" nil (r iy0 l iy1 s))
("released" nil (r iy0 l iy1 s t))
("releases" nil (r ih0 l iy1 s ih0 z))
("releasing" nil (r iy0 l iy1 s ih0 ng))
("releford" nil (r eh1 l ih0 f er0 d))
("relegate" nil (r eh1 l ax g ey1 t))
("relegated" nil (r eh1 l ax g ey1 t ih0 d))
("relegating" nil (r eh1 l ax g ey1 t ih0 ng))
("relend" nil (r iy0 l eh1 n d))
("relending" nil (r iy0 l eh1 n d ih0 ng))
("relent" nil (r ih0 l eh1 n t))
("relented" nil (r ih0 l eh1 n t ih0 d))
("relenting" nil (r ih0 l eh1 n t ih0 ng))
("relentless" nil (r ih0 l eh1 n t l ih0 s))
("relentlessly" nil (r ih0 l eh1 n t l ax s l iy0))
("relevance" nil (r eh1 l ax v ax n s))
("relevancy" nil (r eh1 l ax v ax n s iy0))
("relevant" nil (r eh1 l ax v ax n t))
("relf" nil (r eh1 l f))
("relford" nil (r eh1 l f er0 d))
("reliability" nil (r iy0 l ay1 ax b ih1 l ax t iy0))
("reliable" nil (r ih0 l ay1 ax b ax l))
("reliably" nil (r ih0 l ay1 ax b l iy0))
("reliance" nil (r ih0 l ay1 ax n s))
("reliant" nil (r ih0 l ay1 ax n t))
("relic" nil (r eh1 l ih0 k))
("relics" nil (r eh1 l ih0 k s))
("relied" nil (r ih0 l ay1 d))
("relief" nil (r ih0 l iy1 f))
("reliefs" nil (r iy0 l iy1 f s))
("relies" nil (r ih0 l ay1 z))
("relieve" nil (r ih0 l iy1 v))
("relieved" nil (r ih0 l iy1 v d))
("reliever" nil (r iy0 l iy1 v er0))
("relievers" nil (r iy0 l iy1 v er0 z))
("relieves" nil (r iy0 l iy1 v z))
("relieving" nil (r ih0 l iy1 v ih0 ng))
("reliford" nil (r eh1 l ih0 f ao0 r d))
("religion" nil (r ih0 l ih1 jh ax n))
("religione" nil (r ih0 l ih1 jh iy0 ow1 n iy0))
("religionist" nil (r iy0 l ih1 jh ax n ih0 s t))
("religions" nil (r iy0 l ih1 jh ax n z))
("religiosity" nil (r ih0 l ih1 jh iy0 aa1 s ax t iy0))
("religious" nil (r ih0 l ih1 jh ax s))
("religiously" nil (r ih0 l ih1 jh ax s l iy0))
("relinquish" nil (r ih0 l ih1 ng k w ih0 sh))
("relinquished" nil (r ih0 l ih1 ng k w ih0 sh t))
("relinquishes" nil (r ih0 l ih1 ng k w ih0 sh ih0 z))
("relinquishing" nil (r iy0 l ih1 ng k w ih0 sh ih0 ng))
("relish" nil (r eh1 l ih0 sh))
("relished" nil (r eh1 l ih0 sh t))
("relishes" nil (r eh1 l ih0 sh ax z))
("relishing" nil (r eh1 l ih0 sh ih0 ng))
("relive" nil (r iy0 l ih1 v))
("reliving" nil (r iy0 l ih1 v ih0 ng))
("rella" nil (r eh1 l ax))
("reller" nil (r eh1 l er0))
("relman" nil (r eh1 l m ax n))
("reload" nil (r iy0 l ow1 d))
("reloadable" nil (r iy0 l ow1 d ax b ax l))
("reloads" nil (r iy0 l ow1 d z))
("relocate" nil (r iy0 l ow1 k ey0 t))
("relocated" nil (r iy0 l ow1 k ey0 t ih0 d))
("relocating" nil (r iy0 l ow1 k ey0 t ih0 ng))
("relocation" nil (r iy0 l ow1 k ey1 sh ax n))
("relocations" nil (r iy0 l ow1 k ey1 sh ax n z))
("relph" nil (r eh1 l f))
("reluctance" nil (r ih0 l ah1 k t ax n s))
("reluctant" nil (r ih0 l ah1 k t ax n t))
("reluctantly" nil (r ih0 l ah1 k t ax n t l iy0))
("rely" nil (r ih0 l ay1))
("relyea" nil (r eh1 l y ey1))
("relying" nil (r iy0 l ay1 ih0 ng))
("rem" nil (r eh1 m))
("remade" nil (r iy0 m ey1 d))
("remain" nil (r ih0 m ey1 n))
("remainder" nil (r ih0 m ey1 n d er0))
("remained" nil (r ih0 m ey1 n d))
("remaining" nil (r ih0 m ey1 n ih0 ng))
("remains" nil (r ih0 m ey1 n z))
("remake" nil (r iy1 m ey1 k))
("remakes" nil (r iy1 m ey1 k s))
("remaking" nil (r iy1 m ey1 k ih0 ng))
("remaley" nil (r eh1 m ax l iy0))
("remaly" nil (r iy1 m ax l iy0))
("remand" nil (r ih0 m ae1 n d))
("remanded" nil (r ih0 m ae1 n d ih0 d))
("remanufacture" nil (r iy1 m ae1 n uw0 f ae1 k ch er0))
("remark" nil (r ih0 m aa1 r k))
("remarkable" nil (r ih0 m aa1 r k ax b ax l))
("remarkably" nil (r ih0 m aa1 r k ax b l iy0))
("remarked" nil (r ih0 m aa1 r k t))
("remarketed" nil (r iy0 m aa1 r k ax t ih0 d))
("remarketing" nil (r iy0 m aa1 r k ax t ih0 ng))
("remarking" nil (r ih0 m aa1 r k ih0 ng))
("remarks" nil (r ih0 m aa1 r k s))
("remarriage" nil (r iy0 m eh1 r ih0 jh))
("remarried" nil (r iy0 m eh1 r iy0 d))
("remarry" nil (r iy0 m eh1 r iy0))
("remarrying" nil (r iy0 m eh1 r iy0 ih0 ng))
("rematch" nil (r iy1 m ae1 ch))
("rembert" nil (r eh1 m b er0 t))
("rembold" nil (r eh1 m b ow1 l d))
("rembrandt" nil (r eh1 m b r ae1 n t))
("rembrandts" nil (r eh1 m b r ae1 n t s))
("remedial" nil (r ih0 m iy1 d iy0 ax l))
("remediate" nil (r iy0 m iy1 d iy0 ax t))
("remediation" nil (r ih0 m iy1 d iy0 ey1 sh ax n))
("remedied" nil (r eh1 m ax d iy0 d))
("remedies" nil (r eh1 m ax d iy0 z))
("remedy" nil (r eh1 m ax d iy0))
("remedying" nil (r eh1 m ax d iy0 ih0 ng))
("remember" nil (r ih0 m eh1 m b er0))
("remembered" nil (r ih0 m eh1 m b er0 d))
("remembering" nil (r ih0 m eh1 m b er0 ih0 ng))
("remembers" nil (r ih0 m eh1 m b er0 z))
("remembrance" nil (r iy0 m eh1 m b r ax n s))
("remembrances" nil (r ih0 m eh1 m b r ax n s ih0 z))
("remer" nil (r iy1 m er0))
("remic" nil (r eh1 m ih0 k))
("remick" nil (r eh1 m ih0 k))
("remics" nil (r eh1 m ih0 k s))
("remigio" nil (r ih0 m ih1 jh iy0 ow0))
("remillard" nil (r eh1 m ih0 l er0 d))
("remind" nil (r iy0 m ay1 n d))
("reminded" nil (r iy0 m ay1 n d ax d))
("reminder" nil (r iy0 m ay1 n d er0))
("reminders" nil (r iy0 m ay1 n d er0 z))
("reminding" nil (r iy0 m ay1 n d ih0 ng))
("reminds" nil (r iy0 m ay1 n d z))
("remington" nil (r eh1 m ih0 ng t ax n))
("remingtons" nil (r eh1 m ih0 ng t ax n z))
("reminisce" nil (r eh1 m ax n ih1 s))
("reminisced" nil (r eh1 m ax n ih1 s t))
("reminiscence" nil (r eh1 m ax n ih1 s ax n s))
("reminiscences" nil (r eh1 m ax n ih1 s ax n s ih0 z))
("reminiscent" nil (r eh1 m ax n ih1 s ax n t))
("reminisces" nil (r eh1 m ax n ih1 s ih0 z))
("reminiscing" nil (r eh1 m ax n ih1 s ih0 ng))
("remis" nil (r ey1 m iy0 z))
("remiss" nil (r iy0 m ih1 s))
("remisses" nil (r iy0 m ih1 s ih0 z))
("remission" nil (r iy0 m ih1 sh ax n))
("remissions" nil (r iy0 m ih1 sh ax n z))
("remit" nil (r iy0 m ih1 t))
("remittances" nil (r iy0 m ih1 t ax n s ih0 z))
("remitted" nil (r iy0 m ih1 t ih0 d))
("remley" nil (r eh1 m l iy0))
("remlinger" nil (r eh1 m ax l ih0 ng er0))
("remme" nil (r eh1 m))
("remmel" nil (r eh1 m ax l))
("remmers" nil (r eh1 m er0 z))
("remmert" nil (r eh1 m er0 t))
("remnant" nil (r eh1 m n ax n t))
("remnants" nil (r eh1 m n ax n t s))
("remo" nil (r eh1 m ow0))
("remodel" nil (r iy0 m aa1 d ax l))
("remodeled" nil (r iy0 m aa1 d ax l d))
("remodeling" nil (r ih0 m aa1 d ax l ih0 ng))
("remolded" nil (r iy0 m ow1 l d ih0 d))
("remorse" nil (r ih0 m ao1 r s))
("remorseless" nil (r ih0 m ao1 r s l ax s))
("remote" nil (r ih0 m ow1 t))
("remotely" nil (r iy0 m ow1 t l iy0))
("remoteness" nil (r iy0 m ow1 t n ax s))
("remotest" nil (r ih0 m ow1 t ax s t))
("remoulded" nil (r iy0 m ow1 l d ih0 d))
("removable" nil (r ih0 m uw1 v ax b ax l))
("removal" nil (r ih0 m uw1 v ax l))
("removals" nil (r ih0 m uw1 v ax l z))
("remove" nil (r iy0 m uw1 v))
("removed" nil (r iy0 m uw1 v d))
("remover" nil (r ih0 m uw1 v er0))
("removes" nil (r iy0 m uw1 v z))
("removing" nil (r iy0 m uw1 v ih0 ng))
("rempe" nil (r eh1 m p))
("rempel" nil (r eh1 m p ax l))
("rempfer" nil (r eh1 m p f er0))
("remsberg" nil (r eh1 m s b er0 g))
("remsburg" nil (r eh1 m s b er0 g))
("remsen" nil (r eh1 m s ax n))
("remson" nil (r eh1 m s ax n))
("remuneration" nil (r ih0 m y uw1 n er0 ey1 sh ax n))
("remunerative" nil (r iy0 m y uw1 n er0 ax t ih0 v))
("remus" nil (r iy1 m ax s))
("remy" nil (r eh1 m iy0))
("ren" nil (r eh1 n))
("rena" nil (r iy1 n ax))
("renaissance" nil (r eh1 n ax s aa1 n s))
("renal" nil (r iy1 n ax l))
("renaldo" nil (r eh0 n aa1 l d ow0))
("rename" nil (r iy0 n ey1 m))
("renamed" nil (r iy0 n ey1 m d))
("renaming" nil (r iy0 n ey1 m ih0 ng))
("renamo" nil (r eh0 n aa1 m ow0))
("renard" nil (r ih0 n aa1 r d))
("renata" nil (r ax n aa1 t ax))
("renate" nil (r ax n aa1 t ax))
("renationalize" nil (r iy0 n ae1 sh ax n ax l ay1 z))
("renato" nil (r eh0 n aa1 t ow0))
("renaud" nil (r ih0 n ow1))
("renault" nil (r ax n ow1))
("renbarger" nil (r eh1 n b aa1 r g er0))
("rencen" nil (r eh1 n s ax n))
("rench" nil (r eh1 n ch))
("rencher" nil (r eh1 n ch er0))
("renco" nil (r eh1 n k ow0))
("renda" nil (r eh1 n d ax))
("rendall" nil (r eh1 n d ax l))
("rende" nil (r eh1 n d))
("rendell" nil (r eh1 n d ax l))
("render" nil (r eh1 n d er0))
("rendered" nil (r eh1 n d er0 d))
("rendering" nil (r eh1 n d er0 ih0 ng))
("renderings" nil (r eh1 n d er0 ih0 ng z))
("renders" nil (r eh1 n d er0 z))
("rendezvous" nil (r aa1 n d ih0 v uw1))
("rendina" nil (r eh0 n d iy1 n ax))
("rending" nil (r eh1 n d ih0 ng))
("rendition" nil (r eh0 n d ih1 sh ax n))
("renditions" nil (r eh0 n d ih1 sh ax n z))
("rendleman" nil (r eh1 n d ax l m ax n))
("rendon" nil (r eh1 n d ow0 n))
("rene" nil (r ax n ey1))
("reneau" nil (r ih0 n ow1))
("renee" nil (r ax n ey1))
("renegade" nil (r eh1 n ax g ey1 d))
("renegades" nil (r eh1 n ax g ey1 d z))
("renegar" nil (r eh1 n ih0 g er0))
("renege" nil (r ih0 n ih1 g))
("reneged" nil (r ih0 n ih1 g d))
("reneging" nil (r ih0 n ih1 g ih0 ng))
("renegotiate" nil (r iy1 n ih0 g ow1 sh iy0 ey1 t))
("renegotiated" nil (r iy1 n ih0 g ow1 sh iy0 ey1 t ih0 d))
("renegotiating" nil (r iy1 n ih0 g ow1 sh iy0 ey1 t ih0 ng))
("renegotiation" nil (r iy1 n ih0 g ow1 sh iy0 ey1 sh ax n))
("renegotiations" nil (r iy1 n ih0 g ow1 sh iy0 ey1 sh ax n z))
("renehan" nil (r eh1 n ih0 hh ae0 n))
("renew" nil (r ih0 n uw1))
("renewable" nil (r iy0 n uw1 ax b ax l))
("renewal" nil (r ih0 n uw1 ax l))
("renewals" nil (r ih0 n uw1 ax l z))
("renewed" nil (r ih0 n uw1 d))
("renewing" nil (r ih0 n uw1 ih0 ng))
("renews" nil (r ih0 n uw1 z))
("renfred" nil (r eh1 n f er0 d))
("renfrew" nil (r eh1 n f r uw0))
("renfro" nil (r eh1 n f r ow0))
("renfroe" nil (r eh1 n f r ow0))
("renfrow" nil (r eh1 n f r aw0))
("rengel" nil (r eh1 ng g ax l))
("rengo" nil (r eh1 ng g ow0))
("renick" nil (r eh1 n ih0 k))
("renier" nil (r iy1 n iy0 er0))
("renin" nil (r iy1 n ax n))
("renison" nil (r eh1 n ih0 s ax n))
("renita" nil (r eh0 n iy1 t ax))
("renk" nil (r eh1 ng k))
("renken" nil (r eh1 ng k ax n))
("renko" nil (r eh1 ng k ow0))
("renminbi" nil (r eh0 n m ih1 n b iy0))
("renn" nil (r eh1 n))
("renna" nil (r eh1 n ax))
("rennard" nil (r ih0 n aa1 r d))
("renne" nil (r eh1 n))
("rennels" nil (r eh1 n ax l z))
("renner" nil (r eh1 n er0))
("rennert" nil (r eh1 n er0 t))
("rennet" nil (r eh1 n ax t))
("rennick" nil (r eh1 n ih0 k))
("rennie" nil (r eh1 n iy0))
("renninger" nil (r eh1 n ih0 ng er0))
("renno" nil (r eh1 n ow0))
("renny" nil (r eh1 n iy0))
("reno" nil (r iy1 n ow0))
("renoir" nil (r eh0 n w aa1 r))
("renominate" nil (r iy0 n ao1 m ih0 n ey1 t))
("renominated" nil (r iy0 n aa1 m ax n ey1 t ih0 d))
("renomination" nil (r iy1 n aa1 m ax n ey1 sh ax n))
("renouf" nil (r ax n uw1 f))
("renounce" nil (r ih0 n aw1 n s))
("renounced" nil (r ih0 n aw1 n s t))
("renouncing" nil (r ih0 n aw1 n s ih0 ng))
("renovate" nil (r eh1 n ax v ey1 t))
("renovated" nil (r eh1 n ax v ey1 t ih0 d))
("renovating" nil (r eh1 n ax v ey1 t ih0 ng))
("renovation" nil (r eh1 n ax v ey1 sh ax n))
("renovations" nil (r eh1 n ax v ey1 sh ax n z))
("renovator" nil (r eh1 n ax v ey1 t er0))
("renovators" nil (r eh1 n ax v ey1 t er0 z))
("renown" nil (r ih0 n aw1 n))
("renowned" nil (r ih0 n aw1 n d))
("rens" nil (r eh1 n z))
("rensberger" nil (r eh1 n s b er0 g er0))
("rensch" nil (r eh1 n sh))
("renschler" nil (r eh1 n sh ax l er0))
("renshaw" nil (r eh1 n sh ao1))
("rensing" nil (r eh1 n s ih0 ng))
("rensselaer" nil (r eh1 n s ax l ih1 r))
("renstrom" nil (r eh1 n s t r ax m))
("rent" nil (r eh1 n t))
("renta" nil (r eh1 n t ax))
("rentable" nil (r eh1 n t ax b ax l))
("rental" nil (r eh1 n t ax l))
("rentals" nil (r eh1 n t ax l z))
("rentas" nil (r eh1 n t ax z))
("rented" nil (r eh1 n t ax d))
("renter" nil (r eh1 n t er0))
("renteria" nil (r eh0 n t eh1 r iy0 ax))
("renters" nil (r eh1 n t er0 z))
("rentfro" nil (r eh1 n t f r ow0))
("rentfrow" nil (r eh1 n t f r aw0))
("renting" nil (r eh1 n t ih0 ng))
("rentmeester" nil (r eh1 n t m iy1 s t er0))
("renton" nil (r eh1 n t ax n))
("rents" nil (r eh1 n t s))
("rentsch" nil (r eh1 n ch))
("rentschler" nil (r eh1 n ch l er0))
("rentz" nil (r eh1 n t s))
("renunciation" nil (r ih0 n ah1 n s iy0 ey1 sh ax n))
("renwick" nil (r eh1 n w ih0 k))
("reny" nil (r iy1 n iy0))
("renz" nil (r eh1 n z))
("renzi" nil (r eh1 n z iy0))
("renzulli" nil (r eh0 n z uw1 l iy0))
("reo" nil (r iy1 ow0))
("reoffer" nil (r iy0 ao1 f er0))
("reoffered" nil (r iy0 ao1 f er0 d))
("reoffering" nil (r iy0 ao1 f er0 ih0 ng))
("reopen" nil (r iy0 ow1 p ax n))
("reopened" nil (r iy0 ow1 p ax n d))
("reopening" nil (r iy0 ow1 p ax n ih0 ng))
("reopens" nil (r iy0 ow1 p ax n z))
("reorder" nil (r iy0 ao1 r d er0))
("reordering" nil (r iy0 ao1 r d er0 ih0 ng))
("reorganization" nil (r iy1 ao0 r g ax n ax z ey1 sh ax n))
("reorganizations" nil (r iy0 ao1 r g ax n ih0 z ey1 sh ax n z))
("reorganize" nil (r iy0 ao1 r g ax n ay1 z))
("reorganized" nil (r iy0 ao1 r g ax n ay1 z d))
("reorganizes" nil (r iy0 ao1 r g ax n ay1 z ih0 z))
("reorganizing" nil (r iy0 ao1 r g ax n ay1 z ih0 ng))
("reorient" nil (r iy0 ao1 r iy0 eh0 n t))
("rep" nil (r eh1 p))
("repack" nil (r iy0 p ae1 k))
("repackage" nil (r iy1 p ae1 k ih0 jh))
("repackaged" nil (r iy1 p ae0 k ih0 jh d))
("repackages" nil (r iy1 p ae1 k ih0 jh ih0 z))
("repackaging" nil (r iy1 p ae0 k ih0 jh ih0 ng))
("repacked" nil (r iy0 p ae1 k t))
("repaid" nil (r iy0 p ey1 d))
("repaint" nil (r iy0 p ey1 n t))
("repainted" nil (r iy0 p ey1 n t ax d))
("repainting" nil (r iy0 p ey1 n t ih0 ng))
("repair" nil (r ih0 p eh1 r))
("repairable" nil (r ih0 p eh1 r ax b ax l))
("repaired" nil (r ih0 p eh1 r d))
("repairing" nil (r ih0 p eh1 r ih0 ng))
("repairman" nil (r ih0 p eh1 r m ae1 n))
("repairmen" nil (r ih0 p eh1 r m eh1 n))
("repairs" nil (r ih0 p eh1 r z))
("repap" nil (r iy1 p ae0 p))
("reparations" nil (r eh1 p er0 ey1 sh ax n z))
("repartee" nil (r eh1 p er0 t iy1))
("repasky" nil (r ih0 p aa1 s k iy0))
("repass" nil (r iy0 p ae1 s))
("repatriate" nil (r iy0 p ey1 t r iy0 ey1 t))
("repatriated" nil (r iy0 p ey1 t r iy0 ey1 t ih0 d))
("repatriating" nil (r iy0 p ey1 t r iy0 ey1 t ih0 ng))
("repatriation" nil (r iy0 p ey1 t r iy0 ey1 sh ax n))
("repay" nil (r iy0 p ey1))
("repayable" nil (r iy0 p ey1 ax b ax l))
("repaying" nil (r iy0 p ey1 ih0 ng))
("repayment" nil (r iy0 p ey1 m ax n t))
("repayments" nil (r iy0 p ey1 m ax n t s))
("repays" nil (r iy1 p ey1 z))
("repeal" nil (r ih0 p iy1 l))
("repealed" nil (r ih0 p iy1 l d))
("repealing" nil (r iy0 p iy1 l ih0 ng))
("repeals" nil (r iy0 p iy1 l z))
("repeat" nil (r ih0 p iy1 t))
("repeatable" nil (r ih0 p iy1 t ax b ax l))
("repeated" nil (r ih0 p iy1 t ih0 d))
("repeatedly" nil (r ih0 p iy1 t ih0 d l iy0))
("repeater" nil (r ih0 p iy1 t er0))
("repeaters" nil (r ih0 p iy1 t er0 z))
("repeating" nil (r ih0 p iy1 t ih0 ng))
("repeats" nil (r ih0 p iy1 t s))
("repel" nil (r ih0 p eh1 l))
("repelled" nil (r ax p eh1 l d))
("repellent" nil (r ih0 p eh1 l ax n t))
("repelling" nil (r ax p eh1 l ih0 ng))
("repels" nil (r ih0 p eh1 l z))
("repent" nil (r ih0 p eh1 n t))
("repentance" nil (r ih0 p eh1 n t ax n s))
("repented" nil (r ih0 p eh1 n t ih0 d))
("repercussion" nil (r iy1 p er0 k ah1 sh ax n))
("repercussions" nil (r iy1 p er0 k ah1 sh ax n z))
("reperfusion" nil (r eh1 p er0 f y uw1 zh ax n))
("repertoire" nil (r eh1 p er0 t w aa1 r))
("repertory" nil (r eh1 p er0 t ao1 r iy0))
("repetition" nil (r eh1 p ax t ih1 sh ax n))
("repetitions" nil (r eh1 p ax t ih1 sh ax n z))
("repetitious" nil (r eh1 p ax t ih1 sh ax s))
("repetitive" nil (r ih0 p eh1 t ih0 t ih0 v))
("repetti" nil (r eh0 p eh1 t iy0))
("repetto" nil (r eh0 p eh1 t ow0))
("reph" nil (r eh1 f))
("rephrase" nil (r iy0 f r ey1 z))
("repinski" nil (r ih0 p ih1 n s k iy0))
("repka" nil (r eh1 p k ax))
("repko" nil (r eh1 p k ow0))
("replace" nil (r iy1 p l ey1 s))
("replaceable" nil (r iy1 p l ey1 s ax b ax l))
("replaced" nil (r iy1 p l ey1 s t))
("replacement" nil (r ih0 p l ey1 s m ax n t))
("replacements" nil (r iy0 p l ey1 s m ax n t s))
("replaces" nil (r ih0 p l ey1 s ih0 z))
("replacing" nil (r ih0 p l ey1 s ih0 ng))
("replant" nil (r iy0 p l ae1 n t))
("replanted" nil (r iy0 p l ae1 n t ih0 d))
("replanting" nil (r iy0 p l ae1 n t ih0 ng))
("replay" nil (r iy0 p l ey1))
("replayed" nil (r iy0 p l ey1 d))
("replaying" nil (r iy0 p l ey1 ih0 ng))
("replays" nil (r iy0 p l ey1 z))
("replenish" nil (r iy0 p l eh1 n ih0 sh))
("replenished" nil (r iy0 p l eh1 n ih0 sh t))
("replenishing" nil (r iy0 p l eh1 n ih0 sh ih0 ng))
("replenishment" nil (r ih0 p l eh1 n ih0 sh m ax n t))
("replete" nil (r iy0 p l iy1 t))
("replica" nil (r eh1 p l ih0 k ax))
("replicas" nil (r eh1 p l ax k ax z))
("replicase" nil (r eh1 p l ih0 k ey1 z))
("replicate" nil (r eh1 p l ax k ey1 t))
("replicated" nil (r eh1 p l ih0 k ey1 t ih0 d))
("replicates" nil (r eh1 p l ih0 k ey1 t s))
("replicating" nil (r eh1 p l ih0 k ey1 t ih0 ng))
("replication" nil (r eh1 p l ax k ey1 sh ax n))
("replied" nil (r ih0 p l ay1 d))
("replies" nil (r ih0 p l ay1 z))
("repligen" nil (r ih1 p l ay1 jh ih0 n))
("replogle" nil (r eh1 p l ow0 g ax l))
("reply" nil (r ih0 p l ay1))
("replying" nil (r ih0 p l ay1 ih0 ng))
("repo" nil (r iy1 p ow0))
("reponse" nil (r ih0 p aa1 n s))
("report" nil (r iy0 p ao1 r t))
("reportable" nil (r ih0 p ao1 r t ax b ax l))
("reportage" nil (r ih0 p ao1 r t ih0 jh))
("reported" nil (r iy1 p ao1 r t ax d))
("reportedly" nil (r ih0 p ao1 r t ax d l iy0))
("reporter" nil (r ih0 p ao1 r t er0))
("reporters" nil (r ih0 p ao1 r t er0 z))
("reporting" nil (r iy0 p ao1 r t ih0 ng))
("reportorial" nil (r eh1 p er0 t ao1 r iy0 ax l))
("reports" nil (r ih0 p ao1 r t s))
("repos" nil (r iy1 p ow1 z))
("reposa" nil (r eh0 p ow1 s ax))
("repose" nil (r iy0 p ow1 z))
("reposition" nil (r iy1 p ax z ih1 sh ax n))
("repositioned" nil (r iy1 p ax z ih1 sh ax n d))
("repositioning" nil (r iy1 p ax z ih1 sh ax n ih0 ng))
("repositories" nil (r iy0 p aa1 z ax t ao1 r iy0 z))
("repository" nil (r iy0 p aa1 z ax t ao1 r iy0))
("repossess" nil (r iy1 p ax z eh1 s))
("repossessed" nil (r iy1 p ax z eh1 s t))
("repossessions" nil (r iy1 p ax z eh1 sh ax n z))
("repp" nil (r eh1 p))
("reppert" nil (r eh1 p er0 t))
("reppond" nil (r eh1 p ax n d))
("reppucci" nil (r eh0 p uw1 ch iy0))
("reprehensible" nil (r eh1 p r ih0 hh eh1 n s ax b ax l))
("represent" nil (r eh1 p r ax z eh1 n t))
("representation" nil (r eh1 p r ax z eh0 n t ey1 sh ax n))
("representational" nil (r eh1 p r ax z ax n t ey1 sh ax n ax l))
("representations" nil (r eh1 p r ax z ax n t ey1 sh ax n z))
("representative" nil (r eh1 p r ax z eh1 n t ax t ih0 v))
("representatives" nil (r eh1 p r ax z eh1 n t ax t ih0 v z))
("represented" nil (r eh1 p r ax z eh1 n t ax d))
("representing" nil (r eh1 p r ih0 z eh1 n t ih0 ng))
("represents" nil (r eh1 p r ax z eh1 n t s))
("repress" nil (r iy0 p r eh1 s))
("repressed" nil (r iy0 p r eh1 s t))
("repression" nil (r iy0 p r eh1 sh ax n))
("repressions" nil (r iy0 p r eh1 sh ax n z))
("repressive" nil (r iy0 p r eh1 s ih0 v))
("reprice" nil (r iy0 p r ay1 s))
("repriced" nil (r iy0 p r ay1 s t))
("repricing" nil (r iy0 p r ay1 s ih0 ng))
("reprieve" nil (r iy0 p r iy1 v))
("reprimand" nil (r eh1 p r ax m ae1 n d))
("reprimanded" nil (r eh1 p r ax m ae1 n d ih0 d))
("reprimands" nil (r eh1 p r ax m ae1 n d z))
("reprint" nil (r iy0 p r ih1 n t))
("reprinted" nil (r iy0 p r ih1 n t ih0 d))
("reprinting" nil (r iy0 p r ih1 n t ih0 ng))
("reprints" nil (r iy0 p r ih1 n t s))
("reprisal" nil (r iy0 p r ay1 z ax l))
("reprisals" nil (r ih0 p r ay1 z ax l z))
("reprise" nil (r ih0 p r ay1 z))
("reproach" nil (r iy0 p r ow1 ch))
("reprocess" nil (r iy0 p r ao1 s eh0 s))
("reprocessing" nil (r iy0 p r ao1 s eh0 s ih0 ng))
("reproduce" nil (r iy1 p r ax d uw1 s))
("reproduced" nil (r iy1 p r ax d uw1 s t))
("reproduces" nil (r iy1 p r ax d uw1 s ih0 z))
("reproducing" nil (r iy1 p r ax d uw1 s ih0 ng))
("reproduction" nil (r iy1 p r ax d ah1 k sh ax n))
("reproductions" nil (r iy1 p r ax d ah1 k sh ax n z))
("reproductive" nil (r iy1 p r ax d ah1 k t ih0 v))
("reprogrammed" nil (r iy0 p r ow1 g r ae1 m d))
("reprogramming" nil (r iy0 p r ow1 g r ae1 m ih0 ng))
("reprographics" nil (r eh1 p r ow0 g r ae1 f ih0 k s))
("reps" nil (r eh1 p s))
("repsher" nil (r eh1 p sh er0))
("repsol" nil (r eh1 p s aa0 l))
("reptile" nil (r eh1 p t ay0 l))
("reptiles" nil (r eh1 p t ay0 l z))
("republic" nil (r iy0 p ah1 b l ax k))
("republican" nil (r ih0 p ah1 b l ih0 k ax n))
("republicanism" nil (r ih0 p ah1 b l ih0 k ax n ih1 z ax m))
("republicans" nil (r ih0 p ah1 b l ih0 k ax n z))
("republicbank" nil (r iy0 p ah1 b l ih0 k b ae1 ng k))
("republics" nil (r iy0 p ah1 b l ih0 k s))
("repudiate" nil (r iy0 p y uw1 d iy0 ey1 t))
("repudiated" nil (r iy0 p y uw1 d iy0 ey1 t ax d))
("repudiating" nil (r iy0 p y uw1 d iy0 ey1 t ih0 ng))
("repudiation" nil (r ih0 p y uw1 d iy0 ey1 sh ax n))
("repugnant" nil (r ih0 p ah1 g n ax n t))
("repulse" nil (r iy0 p ah1 l s))
("repulsed" nil (r iy0 p ah1 l s t))
("repulsing" nil (r iy0 p ah1 l s ih0 ng))
("repulsive" nil (r iy0 p ah1 l s ih0 v))
("repurchase" nil (r iy0 p er1 ch ax s))
("repurchased" nil (r iy0 p er1 ch ax s t))
("repurchases" nil (r iy0 p er1 ch ax s ih0 z))
("repurchasing" nil (r iy0 p er1 ch ax s ih0 ng))
("reputable" nil (r eh1 p y ax t ax b ax l))
("reputation" nil (r eh1 p y ax t ey1 sh ax n))
("reputations" nil (r eh1 p y ax t ey1 sh ax n z))
("repute" nil (r iy0 p y uw1 t))
("reputed" nil (r ih0 p y uw1 t ih0 d))
("reputedly" nil (r ih0 p y uw1 t ih0 d l iy0))
("requa" nil (r ey1 k w ax))
("requalify" nil (r iy1 k w aa1 l ax f ay1))
("request" nil (r ih0 k w eh1 s t))
("requested" nil (r ih0 k w eh1 s t ih0 d))
("requester" nil (r ih0 k w eh1 s t er0))
("requesting" nil (r ih0 k w eh1 s t ih0 ng))
("requests" nil (r ih0 k w eh1 s t s))
("requiem" nil (r eh1 k w iy0 ax m))
("require" nil (r iy1 k w ay1 er0))
("required" nil (r iy0 k w ay1 er0 d))
("requirement" nil (r ih0 k w ay1 r m ax n t))
("requirements" nil (r ih0 k w ay1 r m ax n t s))
("requires" nil (r iy0 k w ay1 er0 z))
("requiring" nil (r iy0 k w ay1 er0 ih0 ng))
("requisite" nil (r eh1 k w ax z ax t))
("requisites" nil (r eh1 k w ax z ax t s))
("requisitioned" nil (r eh1 k w ax z ih1 sh ax n d))
("reread" nil (r iy1 r iy1 d))
("rereading" nil (r iy1 r iy1 d ih0 ng))
("reregulation" nil (r iy0 r eh0 g y ax l ey1 sh ax n))
("reroute" nil (r iy0 r uw1 t))
("rerouted" nil (r iy0 r uw1 t ih0 d))
("rerouting" nil (r iy0 r uw1 t ih0 ng))
("rerun" nil (r iy1 r ah1 n))
("rerunning" nil (r iy1 r ah1 n ih0 ng))
("reruns" nil (r iy1 r ah1 n z))
("res" nil (r ey1 z))
("resale" nil (r iy1 s ey1 l))
("resales" nil (r iy1 s ey1 l z))
("resch" nil (r eh1 sh))
("reschedule" nil (r iy0 s k eh1 jh uw0 l))
("rescheduled" nil (r iy0 s k eh1 jh uw0 l d))
("rescheduling" nil (r iy0 s k eh1 jh uw0 l ih0 ng))
("reschedulings" nil (r iy0 sh k eh1 jh uw0 l ih0 ng z))
("reschke" nil (r eh1 sh k))
("rescigno" nil (r eh0 s ch iy1 g n ow0))
("rescind" nil (r ih0 s ih1 n d))
("rescinded" nil (r ih0 s ih1 n d ih0 d))
("rescinding" nil (r ih0 s ih1 n d ih0 ng))
("rescission" nil (r ih0 s ih1 zh ax n))
("rescissions" nil (r ih0 s ih1 zh ax n z))
("rescue" nil (r eh1 s k y uw0))
("rescued" nil (r eh1 s k y uw0 d))
("rescuer" nil (r eh1 s k y uw1 er0))
("rescuers" nil (r eh1 s k y uw1 er0 z))
("rescues" nil (r eh1 s k y uw1 z))
("rescuing" nil (r eh1 s k y uw0 ih0 ng))
("resdel" nil (r eh1 z d eh1 l))
("reseachers" nil (r iy0 s er1 ch er0 z))
("resealable" nil (r iy0 s iy1 l ax b ax l))
("research" nil (r iy0 s er1 ch))
("researched" nil (r iy0 s er1 ch t))
("researcher" nil (r iy1 s er0 ch er0))
("researchers" nil (r iy1 s er0 ch er0 z))
("researches" nil (r iy0 s er1 ch ih0 z))
("researching" nil (r iy0 s er1 ch ih0 ng))
("reseda" nil (r eh0 s ey1 d ax))
("reseed" nil (r iy0 s iy1 d))
("resell" nil (r iy0 s eh1 l))
("reseller" nil (r iy0 s eh1 l er0))
("resellers" nil (r iy0 s eh1 l er0 z))
("reselling" nil (r iy0 s eh1 l ih0 ng))
("resells" nil (r iy0 s eh1 l z))
("resemblance" nil (r ih0 z eh1 m b l ax n s))
("resemblances" nil (r iy0 z eh1 m b l ax n s ax z))
("resemble" nil (r ih0 z eh1 m b ax l))
("resembled" nil (r ih0 z eh1 m b ax l d))
("resembles" nil (r ih0 z eh1 m b ax l z))
("resembling" nil (r ih0 z eh1 m b ax l ih0 ng))
("resende" nil (r ih0 s eh1 n d ey0))
("resendes" nil (r eh1 s ih0 n d z))
("resendez" nil (r ey0 s ey1 n d eh0 z))
("resendiz" nil (r ih0 s eh1 n d ih0 z))
("resent" nil (r ih0 z eh1 n t))
("resented" nil (r iy0 z eh1 n t ax d))
("resentful" nil (r ih0 z eh1 n t f ax l))
("resentment" nil (r ih0 z eh1 n t m ax n t))
("resentments" nil (r ih0 z eh1 n t m ax n t s))
("resents" nil (r ih0 z eh1 n t s))
("reser" nil (r iy1 z er0))
("reservation" nil (r eh1 z er0 v ey1 sh ax n))
("reservationists" nil (r eh1 z er0 v ey1 sh ax n ih0 s t s))
("reservations" nil (r eh1 z er0 v ey1 sh ax n z))
("reserve" nil (r ih0 z er1 v))
("reserved" nil (r ih0 z er1 v d))
("reserveese" nil (r eh1 z er0 v iy1 s))
("reserves" nil (r ih0 z er1 v z))
("reserving" nil (r ih0 z er1 v ih0 ng))
("reservist" nil (r ih0 z er1 v ih0 s t))
("reservists" nil (r ih0 z er1 v ih0 s t s))
("reservoir" nil (r eh1 z ax v w aa1 r))
("reservoirs" nil (r eh1 z er0 v w aa1 r z))
("reset" nil (r iy0 s eh1 t))
("resetar" nil (r eh1 s ih0 t er0))
("reseting" nil (r iy0 s eh1 t ih0 ng))
("resets" nil (r iy0 s eh1 t s))
("resettable" nil (r iy0 s eh1 t ax b ax l))
("resettle" nil (r iy0 s eh1 t ax l))
("resettled" nil (r iy0 s eh1 t ax l d))
("resettlement" nil (r iy0 s eh1 t ax l m ax n t))
("resh" nil (r eh1 sh))
("reshape" nil (r iy0 sh ey1 p))
("reshaped" nil (r iy0 sh ey1 p t))
("reshaping" nil (r iy0 sh ey1 p ih0 ng))
("reshuffle" nil (r iy0 sh ah1 f ax l))
("reshuffled" nil (r iy0 sh ah1 f ax l d))
("reshuffling" nil (r iy0 sh ah1 f ax l ih0 ng))
("reside" nil (r ih0 z ay1 d))
("resided" nil (r ih0 z ay1 d ih0 d))
("residence" nil (r eh1 z ax d ax n s))
("residences" nil (r eh1 z ih0 d ax n s ih0 z))
("residencies" nil (r eh1 s ih0 d eh1 n s iy0 z))
("residency" nil (r eh1 z ih0 d ax n s iy0))
("resident" nil (r eh1 z ax d ax n t))
("residential" nil (r eh1 z ax d eh1 n ch ax l))
("residents" nil (r eh1 z ax d ax n t s))
("resides" nil (r ih0 z ay1 d z))
("residing" nil (r ih0 z ay1 d ih0 ng))
("residual" nil (r ih0 z ih1 jh uw0 ax l))
("residuals" nil (r ih0 z ih1 jh uw0 ax l z))
("residue" nil (r eh1 z ax d uw1))
("residues" nil (r eh1 z ax d uw1 z))
("resign" nil (r ih0 z ay1 n))
("resignation" nil (r eh1 z ax g n ey1 sh ax n))
("resignations" nil (r eh1 z ih0 g n ey1 sh ax n z))
("resigned" nil (r ih0 z ay1 n d))
("resignees" nil (r eh1 z ih0 g n iy1 z))
("resigning" nil (r ih0 z ay1 n ih0 ng))
("resigns" nil (r ih0 z ay1 n z))
("resilience" nil (r ih0 z ih1 l iy0 ax n s))
("resiliency" nil (r ih0 z ih1 l y ax n s iy0))
("resilient" nil (r ih0 z ih1 l y ax n t))
("resin" nil (r eh1 z ax n))
("resins" nil (r eh1 z ax n z))
("resist" nil (r ih0 z ih1 s t))
("resistance" nil (r ih0 z ih1 s t ax n s))
("resistant" nil (r ih0 z ih1 s t ax n t))
("resisted" nil (r ih0 z ih1 s t ih0 d))
("resistence" nil (r ih0 z ih1 s t ax n s))
("resisting" nil (r ih0 z ih1 s t ih0 ng))
("resistiveness" nil (r ih1 z ih1 s t ih1 v n ax s))
("resistors" nil (r ih0 z ih1 s t er0 z))
("resists" nil (r ih0 z ih1 s t s))
("reske" nil (r eh1 s k))
("resler" nil (r eh1 z l er0))
("resnais" nil (r eh0 s n ey1))
("resner" nil (r eh1 s n er0))
("resnick" nil (r eh1 z n ih0 k))
("resnik" nil (r eh1 s n ih0 k))
("resold" nil (r iy0 s ow1 l d))
("resolute" nil (r eh1 z ax l uw1 t))
("resolutely" nil (r eh1 s ax l uw1 t l iy0))
("resolution" nil (r eh1 z ax l uw1 sh ax n))
("resolutions" nil (r eh1 z ax l uw1 sh ax n z))
("resolve" nil (r iy0 z aa1 l v))
("resolved" nil (r iy0 z aa1 l v d))
("resolves" nil (r iy0 z aa1 l v z))
("resolving" nil (r iy0 z aa1 l v ih0 ng))
("resonance" nil (r eh1 z ax n ax n s))
("resonant" nil (r eh1 z ax n ax n t))
("resonate" nil (r eh1 z ax n ey1 t))
("resonates" nil (r eh1 z ax n ey1 t s))
("resort" nil (r ih0 z ao1 r t))
("resorted" nil (r ih0 z ao1 r t ih0 d))
("resorting" nil (r ih0 z ao1 r t ih0 ng))
("resorts" nil (r ih0 z ao1 r t s))
("resound" nil (r iy1 s aw1 n d))
("resounding" nil (r iy0 s aw1 n d ih0 ng))
("resoundingly" nil (r ih0 z aw1 n d ih0 ng l iy0))
("resource" nil (r iy1 s ao0 r s))
("resourceful" nil (r iy0 s ao1 r s f ax l))
("resourcefulness" nil (r iy0 s ao1 r s f ax l n ax s))
("resources" nil (r iy1 s ao0 r s ih0 z))
("respect" nil (r ih0 s p eh1 k t))
("respectability" nil (r iy0 s p eh1 k t ax b ih1 l ih0 t iy0))
("respectable" nil (r ih0 s p eh1 k t ax b ax l))
("respectably" nil (r iy0 s p eh1 k t ax b l iy0))
("respected" nil (r ih0 s p eh1 k t ih0 d))
("respectful" nil (r ih0 s p eh1 k t f ax l))
("respectfully" nil (r ih0 s p eh1 k t f ax l iy0))
("respecting" nil (r iy0 s p eh1 k t ih0 ng))
("respective" nil (r ih0 s p eh1 k t ih0 v))
("respectively" nil (r ih0 s p eh1 k t ih0 v l iy0))
("respects" nil (r ih0 s p eh1 k t s))
("respess" nil (r ey1 s p ih0 s))
("respiration" nil (r eh1 s p er0 ey1 sh ax n))
("respirator" nil (r eh1 s p er0 ey1 t er0))
("respirators" nil (r eh1 s p er0 ey1 t er0 z))
("respiratory" nil (r eh1 s p er0 ax t ao1 r iy0))
("respironics" nil (r eh1 s er0 aa1 n ih0 k s))
("respite" nil (r eh1 s p ih0 t))
("resplendent" nil (r iy0 s p l eh1 n d ax n t))
("respond" nil (r ih0 s p aa1 n d))
("responded" nil (r ih0 s p aa1 n d ih0 d))
("respondent" nil (r ih0 s p aa1 n d ax n t))
("respondents" nil (r ih0 s p aa1 n d ax n t s))
("responding" nil (r ih0 s p aa1 n d ih0 ng))
("responds" nil (r ih0 s p aa1 n d z))
("response" nil (r ih0 s p aa1 n s))
("responses" nil (r ih0 s p aa1 n s ih0 z))
("responsibilities" nil (r iy0 s p aa1 n s ax b ih1 l ax t iy0 z))
("responsibility" nil (r iy0 s p aa1 n s ax b ih1 l ax t iy0))
("responsible" nil (r iy0 s p aa1 n s ax b ax l))
("responsiblity" nil (r ih0 s p aa1 n s ax b l ih1 t iy0))
("responsibly" nil (r ih0 s p aa1 n s ax b l iy0))
("responsive" nil (r ih0 s p aa1 n s ih0 v))
("responsiveness" nil (r ih0 s p aa1 n s ih0 v n ax s))
("respress" nil (r eh1 s p r ih0 s))
("ress" nil (r eh1 s))
("ressa" nil (r eh1 s ax))
("resseguie" nil (r eh1 s ih0 g w iy0))
("ressel" nil (r eh1 s ax l))
("ressler" nil (r eh1 s l er0))
("rest" nil (r eh1 s t))
("restaged" nil (r iy0 s t ey1 jh d))
("restaino" nil (r eh0 s t aa0 iy1 n ow0))
("restart" nil (r iy0 s t aa1 r t))
("restarted" nil (r iy0 s t aa1 r t ih0 d))
("restarting" nil (r iy0 s t aa1 r t ih0 ng))
("restate" nil (r iy0 s t ey1 t))
("restated" nil (r iy0 s t ey1 t ih0 d))
("restatement" nil (r iy0 s t ey1 t m ax n t))
("restatements" nil (r iy0 s t ey1 t m ax n t s))
("restates" nil (r iy0 s t ey1 t s))
("restating" nil (r iy0 s t ey1 t ih0 ng))
("restaurant" nil (r eh1 s t er0 aa1 n t))
("restaurants" nil (r eh1 s t er0 aa1 n t s))
("restaurateur" nil (r eh1 s t er0 ax t er1))
("restaurateurs" nil (r eh1 s t er0 ax t er1 z))
("rested" nil (r eh1 s t ax d))
("rester" nil (r eh1 s t er0))
("restful" nil (r eh1 s t f ax l))
("resting" nil (r eh1 s t ih0 ng))
("restitution" nil (r eh1 s t ih0 t uw1 sh ax n))
("restive" nil (r eh1 s t ih0 v))
("restiveness" nil (r eh1 s t ih0 v n ax s))
("restivo" nil (r eh0 s t iy1 v ow0))
("restless" nil (r eh1 s t l ax s))
("restlessly" nil (r eh1 s t l ax s l iy0))
("restlessness" nil (r eh1 s t l ax s n ax s))
("resto" nil (r eh1 s t ow0))
("restock" nil (r iy0 s t aa1 k))
("restocked" nil (r iy0 s t aa1 k t))
("restocking" nil (r iy0 s t aa1 k ih0 ng))
("reston" nil (r eh1 s t ax n))
("restoration" nil (r eh1 s t er0 ey1 sh ax n))
("restorations" nil (r eh1 s t er0 ey1 sh ax n z))
("restore" nil (r ih0 s t ao1 r))
("restored" nil (r ih0 s t ao1 r d))
("restorer" nil (r ih0 s t ao1 r er0))
("restores" nil (r ih0 s t ao1 r z))
("restoring" nil (r ih0 s t ao1 r ih0 ng))
("restrain" nil (r iy0 s t r ey1 n))
("restrained" nil (r iy0 s t r ey1 n d))
("restraining" nil (r iy0 s t r ey1 n ih0 ng))
("restrains" nil (r iy0 s t r ey1 n z))
("restraint" nil (r ih0 s t r ey1 n t))
("restraints" nil (r ih0 s t r ey1 n t s))
("restrepo" nil (r eh0 s t r eh1 p ow0))
("restrict" nil (r iy0 s t r ih1 k t))
("restricted" nil (r iy0 s t r ih1 k t ax d))
("restricting" nil (r iy0 s t r ih1 k t ih0 ng))
("restriction" nil (r iy0 s t r ih1 k sh ax n))
("restrictions" nil (r iy0 s t r ih1 k sh ax n z))
("restrictive" nil (r iy0 s t r ih1 k t ih0 v))
("restrictiveness" nil (r ax s t r ih1 k t ih0 v n ax s))
("restricts" nil (r iy0 s t r ih1 k t s))
("restroom" nil (r eh1 s t r uw1 m))
("restrooms" nil (r eh1 s t r uw1 m z))
("restructure" nil (r iy0 s t r ah1 k ch er0))
("restructured" nil (r iy0 s t r ah1 k ch er0 d))
("restructures" nil (r iy0 s t r ah1 k ch er0 z))
("restructuring" nil (r iy0 s t r ah1 k ch er0 ih0 ng))
("restructurings" nil (r iy0 s t r ah1 k ch er0 ih0 ng z))
("rests" nil (r eh1 s t s))
("restyled" nil (r iy0 s t ay1 l d))
("resubmit" nil (r iy1 s ax b m ih1 t))
("resubmitted" nil (r iy1 s ax b m ih1 t ih0 d))
("resubmitting" nil (r iy1 s ax b m ih1 t ih0 ng))
("result" nil (r ih0 z ah1 l t))
("resultant" nil (r iy0 z ah1 l t ax n t))
("resulted" nil (r ih0 z ah1 l t ih0 d))
("resulting" nil (r ih0 z ah1 l t ih0 ng))
("results" nil (r ih0 z ah1 l t s))
("resume" nil (r ih0 z uw1 m))
("resumed" nil (r ih0 z uw1 m d))
("resumes" nil (r ih0 z uw1 m z))
("resuming" nil (r ih0 z uw1 m ih0 ng))
("resumption" nil (r ih0 z ah1 m p sh ax n))
("resupply" nil (r iy0 s ax p l ay1))
("resurface" nil (r iy0 s er1 f ax s))
("resurfaced" nil (r iy0 s er1 f ax s t))
("resurfacing" nil (r iy0 s er1 f ax s ih0 ng))
("resurgence" nil (r iy0 s er1 jh ax n s))
("resurgency" nil (r ih0 s er1 jh ax n s iy0))
("resurgent" nil (r ih0 s er1 jh ax n t))
("resurging" nil (r iy0 s er1 jh ih0 ng))
("resurrect" nil (r eh1 z er0 eh1 k t))
("resurrected" nil (r eh1 z er0 eh1 k t ih0 d))
("resurrecting" nil (r eh1 z er0 eh1 k t ih0 ng))
("resurrection" nil (r eh1 z er0 eh1 k sh ax n))
("resuscitate" nil (r ih0 s ah1 s ih0 t ey1 t))
("resuscitated" nil (r ih0 s ah1 s ih0 t ey1 t ih0 d))
("resuscitating" nil (r ih0 s ah1 s ih0 t ey1 t ih0 ng))
("resuscitation" nil (r ih0 s ah1 s ih0 t ey1 sh ax n))
("ret" nil (r eh1 t))
("reta" nil (r eh1 t ax))
("retablos" nil (r iy0 t ae1 b l ow0 s))
("retail" nil (r iy1 t ey1 l))
("retailed" nil (r iy1 t ey1 l d))
("retailer" nil (r iy1 t ey1 l er0))
("retailers" nil (r iy1 t ey1 l er0 z))
("retailing" nil (r iy1 t ey1 l ih0 ng))
("retails" nil (r iy1 t ey1 l z))
("retain" nil (r ih0 t ey1 n))
("retained" nil (r ih0 t ey1 n d))
("retainer" nil (r ih0 t ey1 n er0))
("retainers" nil (r ih0 t ey1 n er0 z))
("retaining" nil (r ih0 t ey1 n ih0 ng))
("retains" nil (r ih0 t ey1 n z))
("retake" nil (r iy1 t ey1 k))
("retaliate" nil (r ih0 t ae1 l iy0 ey1 t))
("retaliated" nil (r ih0 t ae1 l iy0 ey1 t ih0 d))
("retaliating" nil (r ih0 t ae1 l iy0 ey1 t ih0 ng))
("retaliation" nil (r iy0 t ae1 l iy0 ey1 sh ax n))
("retaliatory" nil (r iy0 t ae1 l y ax t ao1 r iy0))
("retana" nil (r eh0 t ae1 n ax))
("retard" nil (r ih0 t aa1 r d))
("retardant" nil (r iy0 t aa1 r d ax n t))
("retardation" nil (r iy0 t aa0 r d ey1 sh ax n))
("retarded" nil (r ih0 t aa1 r d ih0 d))
("retarding" nil (r ih0 t aa1 r d ih0 ng))
("retards" nil (r ih0 t aa1 r d z))
("retelling" nil (r iy0 t eh1 l ih0 ng))
("retemeyer" nil (r eh1 t ax m ay1 r))
("retention" nil (r iy0 t eh1 n sh ax n))
("retest" nil (r iy1 t eh1 s t))
("retested" nil (r iy0 t eh1 s t ih0 d))
("retesting" nil (r iy0 t eh1 s t ih0 ng))
("retests" nil (r iy1 t eh1 s t s))
("retherford" nil (r ih0 th er1 f er0 d))
("rethink" nil (r iy0 th ih1 ng k))
("rethinking" nil (r iy0 th ih1 ng k ih0 ng))
("reticence" nil (r eh1 t ih0 s ax n s))
("reticent" nil (r eh1 t ih0 s ax n t))
("retin" nil (r eh1 t ih0 n))
("retina" nil (r eh1 t ax n ax))
("retinal" nil (r eh1 t ax n ax l))
("retinoblastoma" nil (r eh1 t ih0 n ow1 b l ae1 s t ow1 m ax))
("retinue" nil (r eh1 t ax n uw1))
("retinyl" nil (r eh1 t ih0 n ax l))
("retire" nil (r ih0 t ay1 r))
("retired" nil (r ih0 t ay1 r d))
("retiree" nil (r iy0 t ay1 r iy1))
("retirees" nil (r iy0 t ay1 r iy1 z))
("retirement" nil (r iy0 t ay1 er0 m ax n t))
("retirements" nil (r ih0 t ay1 r m ax n t s))
("retires" nil (r ih0 t ay1 r z))
("retiring" nil (r ih0 t ay1 r ih0 ng))
("retlin" nil (r eh1 t l ih0 n))
("reto" nil (r iy1 t uw1))
("retook" nil (r iy0 t uh1 k))
("retool" nil (r iy0 t uw1 l))
("retooled" nil (r iy0 t uw1 l d))
("retooling" nil (r iy0 t uw1 l ih0 ng))
("retort" nil (r iy1 t ao1 r t))
("retorted" nil (r iy0 t ao1 r t ih0 d))
("retorts" nil (r ih0 t ao1 r t s))
("retouching" nil (r iy0 t ah1 ch ih0 ng))
("retrace" nil (r iy0 t r ey1 s))
("retraced" nil (r iy0 t r ey1 s t))
("retracement" nil (r iy0 t r ey1 s m ax n t))
("retracing" nil (r iy0 t r ey1 s ih0 ng))
("retract" nil (r iy0 t r ae1 k t))
("retractable" nil (r iy0 t r ae1 k t ax b ax l))
("retracted" nil (r iy0 t r ae1 k t ax d))
("retracting" nil (r iy0 t r ae1 k t ih0 ng))
("retraction" nil (r iy0 t r ae1 k sh ax n))
("retracts" nil (r iy0 t r ae1 k t s))
("retrain" nil (r iy0 t r ey1 n))
("retrained" nil (r iy0 t r ey1 n d))
("retraining" nil (r iy0 t r ey1 n ih0 ng))
("retransmission" nil (r iy1 t r ae0 n z m ih1 sh ax n))
("retread" nil (r iy0 t r eh1 d))
("retreading" nil (r iy0 t r eh1 d ih0 ng))
("retreads" nil (r iy0 t r eh1 d z))
("retreat" nil (r iy0 t r iy1 t))
("retreated" nil (r iy0 t r iy1 t ax d))
("retreating" nil (r iy0 t r iy1 t ih0 ng))
("retreats" nil (r iy0 t r iy1 t s))
("retrench" nil (r iy0 t r eh1 n ch))
("retrenched" nil (r iy0 t r eh1 n ch t))
("retrenching" nil (r iy0 t r eh1 n ch ih0 ng))
("retrenchment" nil (r iy0 t r eh1 n ch m ax n t))
("retrenchments" nil (r iy0 t r eh1 n ch m ax n t s))
("retrial" nil (r iy0 t r ay1 ax l))
("retribution" nil (r eh1 t r ax b y uw1 sh ax n))
("retried" nil (r iy0 t r ay1 d))
("retrieval" nil (r ih0 t r iy1 v ax l))
("retrieve" nil (r ih0 t r iy1 v))
("retrieved" nil (r iy0 t r iy1 v d))
("retriever" nil (r iy0 t r iy1 v er0))
("retrievers" nil (r iy0 t r iy1 v er0 z))
("retrieving" nil (r iy0 t r iy1 v ih0 ng))
("retro" nil (r eh1 t r ow0))
("retroactive" nil (r eh1 t r ow0 ae1 k t ih0 v))
("retroactively" nil (r eh1 t r ow0 ae1 k t ih0 v l iy0))
("retroactivity" nil (r eh1 t r ow0 ae0 k t ih1 v ih0 t iy0))
("retrofit" nil (r eh1 t r ow0 f ih1 t))
("retrofits" nil (r eh1 t r ow0 f ih1 t s))
("retrofitted" nil (r eh1 t r ow1 f ih1 t ih0 d))
("retrofitter" nil (r eh1 t r ow1 f ih1 t er0))
("retrofitting" nil (r eh1 t r ow0 f ih1 t ih0 ng))
("retrograde" nil (r eh1 t r ax g r ey1 d))
("retrospect" nil (r eh1 t r ax s p eh1 k t))
("retrospective" nil (r eh1 t r ax s p eh1 k t ih0 v))
("retrospectively" nil (r eh1 t r ow0 s p eh1 k t ih0 v l iy0))
("retrovir" nil (r eh1 t r ow0 v ih1 r))
("retroviruses" nil (r eh1 t r ow0 v ay1 r ax s ih0 z))
("retry" nil (r iy0 t r ay1))
("retrying" nil (r iy0 t r ay1 ih0 ng))
("rettberg" nil (r eh1 t b er0 g))
("retter" nil (r eh1 t er0))
("rettew" nil (r eh1 ch uw0))
("rettig" nil (r eh1 t ih0 g))
("rettinger" nil (r eh1 t ih0 n jh er0))
("rettke" nil (r eh1 t k iy0))
("retton" nil (r eh1 t ax n))
("return" nil (r ih0 t er1 n))
("returnable" nil (r iy0 t er1 n ax b ax l))
("returned" nil (r ih0 t er1 n d))
("returnees" nil (r ih0 t er0 n iy1 z))
("returning" nil (r ih0 t er1 n ih0 ng))
("returns" nil (r ih0 t er1 n z))
("retz" nil (r eh1 t s))
("retzer" nil (r eh1 t z er0))
("retzlaff" nil (r eh1 t z l ax f))
("retzloff" nil (r eh1 t z l ao0 f))
("reuben" nil (r uw1 b ax n))
("reuber" nil (r oy1 b er0))
("reul" nil (r uw1 l))
("reuland" nil (r oy1 l ax n d))
("reule" nil (r uw1 l))
("reum" nil (r iy1 ax m))
("reunification" nil (r iy0 uw1 n ax f ax k ey1 sh ax n))
("reunified" nil (r iy0 uw1 n ax f ay1 d))
("reunify" nil (r iy0 uw1 n ax f ay1))
("reunion" nil (r iy0 uw1 n y ax n))
("reunions" nil (r iy0 uw1 n y ax n z))
("reunite" nil (r iy1 uw0 n ay1 t))
("reunited" nil (r iy1 uw0 n ay1 t ih0 d))
("reuniting" nil (r iy1 uw0 n ay1 t ih0 ng))
("reusable" nil (r iy0 uw1 z ax b ax l))
("reusch" nil (r oy1 sh))
("reuse" nil (r iy0 y uw1 s))
("reused" nil (r iy0 uw1 z d))
("reuss" nil (r uw1 s))
("reusser" nil (r oy1 s er0))
("reust" nil (r uw1 s t))
("reuter" nil (r oy1 t er0))
("reuters" nil (r oy1 t er0 z))
("reuther" nil (r oy1 dh er0))
("reutter" nil (r oy1 t er0))
("reutzel" nil (r oy1 t z ax l))
("rev" nil (r eh1 v))
("reva" nil (r ey1 v ax))
("revak" nil (r eh1 v ax k))
("revaluation" nil (r iy0 v ae1 l y uw0 ey1 sh ax n))
("revaluations" nil (r iy0 v ae1 l y uw0 ey1 sh ax n z))
("revalue" nil (r iy0 v ae1 l y uw1))
("revalued" nil (r iy0 v ae1 l y uw0 d))
("revaluing" nil (r iy0 v ae1 l y uw0 ih0 ng))
("revamp" nil (r iy0 v ae1 m p))
("revamped" nil (r iy0 v ae1 m p t))
("revamping" nil (r iy0 v ae1 m p ih0 ng))
("revamps" nil (r iy0 v ae1 m p s))
("revard" nil (r ih0 v aa1 r d))
("revco" nil (r eh1 v k ow0))
("reveal" nil (r ih0 v iy1 l))
("revealed" nil (r ih0 v iy1 l d))
("revealing" nil (r ih0 v iy1 l ih0 ng))
("reveals" nil (r ih0 v iy1 l z))
("revel" nil (r eh1 v ax l))
("revelation" nil (r eh1 v ax l ey1 sh ax n))
("revelations" nil (r eh1 v ax l ey1 sh ax n z))
("revelatory" nil (r ih0 v eh1 l ax t ao1 r iy0))
("reveled" nil (r eh1 v ax l d))
("revelers" nil (r eh1 v ax l er0 z))
("reveles" nil (r ey0 v ey1 l eh0 s))
("reveling" nil (r eh1 v ax l ih0 ng))
("revell" nil (r ey0 v ey1 l))
("revelle" nil (r ih0 v eh1 l))
("revello" nil (r eh0 v eh1 l ow0))
("revelry" nil (r eh1 v ax l r iy0))
("revels" nil (r eh1 v ax l z))
("revenge" nil (r iy0 v eh1 n jh))
("revenue" nil (r eh1 v ax n uw1))
("revenues" nil (r eh1 v ax n uw1 z))
("reverberate" nil (r ih0 v er1 b er0 ax t))
("reverberated" nil (r ih0 v er1 b er0 ey1 t ih0 d))
("reverberates" nil (r ih0 v er1 b er0 ey1 t s))
("reverberating" nil (r ih0 v er1 b er0 ey1 t ih0 ng))
("reverberation" nil (r iy0 v er1 b er0 ey1 sh ax n))
("reverberations" nil (r ih0 v er1 b er0 ey1 sh ax n z))
("revercomb" nil (r eh1 v er0 k ax m))
("revere" nil (r ih0 v ih1 r))
("revered" nil (r ih0 v ih1 r d))
("reverence" nil (r eh1 v er0 ax n s))
("reverend" nil (r eh1 v er0 ax n d))
("reverential" nil (r eh1 v er0 eh1 n ch ax l))
("reverently" nil (r eh1 v er0 ax n t l iy0))
("reverie" nil (r eh1 v er0 iy0))
("reveries" nil (r eh1 v er0 iy0 z))
("reveron" nil (r eh1 v er0 aa1 n))
("reversal" nil (r ih0 v er1 s ax l))
("reversals" nil (r ih0 v er1 s ax l z))
("reverse" nil (r ih0 v er1 s))
("reversed" nil (r ih0 v er1 s t))
("reverser" nil (r ih0 v er1 s er0))
("reversers" nil (r ih0 v er1 s er0 z))
("reverses" nil (r ih0 v er1 s ih0 z))
("reversible" nil (r ih0 v er1 s ax b ax l))
("reversing" nil (r ih0 v er1 s ih0 ng))
("reversion" nil (r ih0 v er1 zh ax n))
("revert" nil (r ih0 v er1 t))
("reverted" nil (r ih0 v er1 t ih0 d))
("reverting" nil (r ih0 v er1 t ih0 ng))
("reverts" nil (r ih0 v er1 t s))
("reves" nil (r iy1 v z))
("revette" nil (r ih0 v eh1 t))
("revier" nil (r iy1 v iy0 er0))
("review" nil (r iy1 v y uw1))
("reviewed" nil (r iy0 v y uw1 d))
("reviewer" nil (r iy0 v y uw1 er0))
("reviewers" nil (r iy0 v y uw1 er0 z))
("reviewing" nil (r iy0 v y uw1 ih0 ng))
("reviews" nil (r iy0 v y uw1 z))
("reviglio" nil (r ax v ih1 g l iy0 ow0))
("reviled" nil (r iy0 v ay1 l d))
("revilla" nil (r eh0 v ih1 l ax))
("reville" nil (r eh1 v ih1 l))
("revis" nil (r eh1 v ih0 s))
("revise" nil (r ih0 v ay1 z))
("revised" nil (r ih0 v ay1 z d))
("revises" nil (r ih0 v ay1 z ih0 z))
("revising" nil (r ih0 v ay1 z ih0 ng))
("revision" nil (r iy0 v ih1 zh ax n))
("revisionism" nil (r iy0 v ih1 zh ax n ih1 z ax m))
("revisionist" nil (r iy0 v ih1 zh ax n ih0 s t))
("revisionists" nil (r iy0 v ih1 zh ax n ih0 s t s))
("revisions" nil (r iy0 v ih1 zh ax n z))
("revisit" nil (r iy0 v ih1 z ih0 t))
("revisited" nil (r iy0 v ih1 z ax t ih0 d))
("revisiting" nil (r iy0 v ih1 z ih0 t ih0 ng))
("revisits" nil (r iy0 v ih1 z ih0 t s))
("revitalization" nil (r iy1 v ay1 t ax l ax z ey1 sh ax n))
("revitalize" nil (r iy0 v ay1 t ax l ay1 z))
("revitalized" nil (r iy0 v ay1 t ax l ay1 z d))
("revitalizing" nil (r iy0 v ay1 t ax l ay1 z ih0 ng))
("revival" nil (r ih0 v ay1 v ax l))
("revivalist" nil (r iy0 v ay1 v ax l ih0 s t))
("revivals" nil (r iy0 v ay1 v ax l z))
("revive" nil (r ih0 v ay1 v))
("revived" nil (r ih0 v ay1 v d))
("revives" nil (r iy0 v ay1 v z))
("reviving" nil (r ih0 v ay1 v ih0 ng))
("revlon" nil (r eh1 v l aa0 n))
("revocable" nil (r eh1 v ax k ax b ax l))
("revocation" nil (r eh1 v ax k ey1 sh ax n))
("revocations" nil (r eh1 v ax k ey1 sh ax n z))
("revoir" nil (r ih0 v w aa1 r))
("revoke" nil (r ih0 v ow1 k))
("revoked" nil (r ih0 v ow1 k t))
("revoking" nil (r ih0 v ow1 k ih0 ng))
("revolt" nil (r ih0 v ow1 l t))
("revolted" nil (r ih0 v ow1 l t ih0 d))
("revolting" nil (r iy0 v ow1 l t ih0 ng))
("revolts" nil (r iy0 v ow1 l t s))
("revolucionario" nil (r eh1 v ow0 l uw1 s iy0 ax n eh1 r iy0 ow0))
("revolution" nil (r eh1 v ax l uw1 sh ax n))
("revolutionaries" nil (r eh1 v ax l uw1 sh ax n eh1 r iy0 z))
("revolutionary" nil (r eh1 v ax l uw1 sh ax n eh1 r iy0))
("revolutionists" nil (r eh1 v ax l uw1 sh ax n ax s t s))
("revolutionize" nil (r eh1 v ax l uw1 sh ax n ay1 z))
("revolutionized" nil (r eh1 v ax l uw1 sh ax n ay1 z d))
("revolutionizing" nil (r eh1 v ax l uw1 sh ax n ay1 z ih0 ng))
("revolutions" nil (r eh1 v ax l uw1 sh ax n z))
("revolve" nil (r iy0 v aa1 l v))
("revolved" nil (r iy0 v aa1 l v d))
("revolver" nil (r ih0 v aa1 l v er0))
("revolvers" nil (r ih0 v aa1 l v er0 z))
("revolves" nil (r iy0 v aa1 l v z))
("revolving" nil (r iy0 v aa1 l v ih0 ng))
("revord" nil (r ih0 v ao1 r d))
("revs" nil (r eh1 v z))
("revson" nil (r eh1 v s ax n))
("revue" nil (r ih0 v y uw1))
("revues" nil (r ih0 v y uw1 z))
("revulsion" nil (r ih0 v ah1 l sh ax n))
("revved" nil (r eh1 v d))
("revving" nil (r eh1 v ih0 ng))
("revzin" nil (r eh1 v z ih0 n))
("rew" nil (r uw1))
("reward" nil (r ih0 w ao1 r d))
("rewarded" nil (r ih0 w ao1 r d ih0 d))
("rewarding" nil (r ih0 w ao1 r d ih0 ng))
("rewards" nil (r ih0 w ao1 r d z))
("rewerts" nil (r uw1 er0 t s))
("rewey" nil (r uw1 iy0))
("rewire" nil (r iy0 w ay1 er0))
("rewired" nil (r iy0 w ay1 er0 d))
("rewiring" nil (r iy0 w ay1 er0 ih0 ng))
("rewis" nil (r uw1 ih0 s))
("reword" nil (r iy0 w er1 d))
("rework" nil (r iy0 w er1 k))
("reworked" nil (r iy0 w er1 k t))
("reworking" nil (r iy0 w er1 k ih0 ng))
("rewrite" nil (r iy0 r ay1 t))
("rewrites" nil (r iy0 r ay1 t s))
("rewriting" nil (r iy0 r ay1 t ih0 ng))
("rewritten" nil (r iy0 r ih1 t ax n))
("rewrote" nil (r iy0 r ow1 t))
("rex" nil (r eh1 k s))
("rexall" nil (r eh1 k s ao1 l))
("rexana" nil (r ih0 g z ae1 n ax))
("rexene" nil (r eh1 k s iy0 n))
("rexford" nil (r eh1 k s f er0 d))
("rexham" nil (r eh1 k s ax m))
("rexnord" nil (r eh1 k s n ao0 r d))
("rexon" nil (r eh1 k s ax n))
("rexroad" nil (r eh1 k s r ow1 d))
("rexroat" nil (r eh1 k s r ow0 t))
("rexrode" nil (r eh1 k s r ow0 d))
("rexrodt" nil (r eh1 k s r ow0 d))
("rexroth" nil (r eh1 k s r aa0 th))
("rey" nil (r ey1))
("reyburn" nil (r ey1 b er0 n))
("reycraft" nil (r ey1 k r ae1 f t))
("reyer" nil (r ey1 er0))
("reyes" nil (r ey1 ih0 s))
("reykjavik" nil (r eh1 k y ax v ih0 k))
("reyman" nil (r ey1 m ax n))
("reyna" nil (r ey1 n ax))
("reynaga" nil (r iy0 n aa1 g ax))
("reynaldo" nil (r ey1 n aa1 l d ow0))
("reynard" nil (r ey1 n er0 d))
("reynaud" nil (r ey1 n ao0 d))
("reynold" nil (r eh1 n ax l d))
("reynolds" nil (r eh1 n ax l d z))
("reynoldson" nil (r ey1 n ow0 l d s ax n))
("reynosa" nil (r iy0 n ow1 s ax))
("reynoso" nil (r iy0 n ow1 s ow0))
("reza" nil (r eh1 z ax))
("rezabek" nil (r ih0 z aa1 b eh0 k))
("rezac" nil (r eh1 z ax k))
("rezek" nil (r eh1 z eh0 k))
("rezendes" nil (r ey0 z ey1 n d eh0 s))
("reznicek" nil (r eh1 z n ih0 ch eh1 k))
("reznick" nil (r eh1 z n ih0 k))
("reznik" nil (r eh1 z n ih0 k))
("rezoning" nil (r iy0 z ow1 n ih0 ng))
("rha" nil (r aa1))
("rhame" nil (r ey1 m))
("rhames" nil (r ey1 m z))
("rhapsodic" nil (r ae0 p s aa1 d ih0 k))
("rhapsodizes" nil (r ae1 p s ax d ay1 z ih0 z))
("rhapsody" nil (r ae1 p s ax d iy0))
("rhea" nil (r iy1 ax))
("rheault" nil (r ow1))
("rheaume" nil (r uw1 m))
("rhee" nil (r iy1))
("rheem" nil (r iy1 m))
("rhein" nil (r ay1 n))
("rheingold" nil (r ay1 n g ow1 l d))
("rheinisch" nil (r ay1 n ih0 sh))
("rheinstein" nil (r ay1 n s t ay1 n))
("rhem" nil (r eh1 m))
("rhesus" nil (r iy1 s ax s))
("rheta" nil (r iy1 t ax))
("rhetoric" nil (r eh1 t er0 ih0 k))
("rhetorical" nil (r ih0 t ao1 r ih0 k ax l))
("rhetorically" nil (r ax t ao1 r ih0 k l iy0))
("rhetoricians" nil (r eh1 t er0 ih1 sh ax n z))
("rhett" nil (r eh1 t))
("rheumatic" nil (r uw0 m ae1 t ih0 k))
("rheumatism" nil (r uw1 m ax t ih1 z ax m))
("rheumatoid" nil (r uw1 m ax t oy1 d))
("rhew" nil (r uw1))
("rhin" nil (r ih1 n))
("rhine" nil (r ay1 n))
("rhinehardt" nil (r ay1 n hh aa1 r t))
("rhinehart" nil (r ay1 n hh aa1 r t))
("rhineland" nil (r ay1 n l ae1 n d))
("rhineman" nil (r ay1 n m ax n))
("rhines" nil (r ay1 n z))
("rhinesmith" nil (r ay1 n s m ih1 th))
("rhinestone" nil (r ay1 n s t ow1 n))
("rhinestones" nil (r ay1 n s t ow1 n z))
("rhino" nil (r ay1 n ow1))
("rhinoceros" nil (r ay0 n aa1 s er0 ax s))
("rhinos" nil (r ay1 n ow1 z))
("rhizome" nil (r ay1 z ow1 m))
("rhizomes" nil (r ay1 z ow1 m z))
("rho" nil (r ow1))
("rhoad" nil (r ow1 d))
("rhoades" nil (r ow1 d z))
("rhoads" nil (r ow1 d z))
("rhoda" nil (r ow1 d ax))
("rhodanthe" nil (r ow1 d ae1 n th))
("rhode" nil (r ow1 d))
("rhoden" nil (r ow1 d ax n))
("rhodes" nil (r ow1 d z))
("rhodesia" nil (r ow0 d iy1 zh ax))
("rhodesian" nil (r ow0 d iy1 zh ax n))
("rhodia" nil (r ow1 d iy0 ax))
("rhodies" nil (r ow1 d iy0 z))
("rhodium" nil (r ow1 d iy0 ax m))
("rhododendron" nil (r ow1 d ax d eh1 n d r ax n))
("rhodopsin" nil (r ow0 d aa1 p s ax n))
("rhodus" nil (r ow1 d ax s))
("rhody" nil (r ow1 d iy0))
("rhome" nil (r ow1 m))
("rhonda" nil (r aa1 n d ax))
("rhone" nil (r ow1 n))
("rhoney" nil (r ow1 n iy0))
("rhorer" nil (r ao1 r er0))
("rhoten" nil (r ow1 t ax n))
("rhoto" nil (r ow1 t ow0))
("rhoton" nil (r ow1 t ax n))
("rhubarb" nil (r uw1 b aa1 r b))
("rhude" nil (r uw1 d))
("rhudy" nil (r uw1 d iy0))
("rhue" nil (r uw1))
("rhule" nil (r uw1 l))
("rhyme" nil (r ay1 m))
("rhymed" nil (r ay1 m d))
("rhymer" nil (r ay1 m er0))
("rhymes" nil (r ay1 m z))
("rhyming" nil (r ay1 m ih0 ng))
("rhyne" nil (r ay1 n))
("rhyner" nil (r ay1 n er0))
("rhynes" nil (r ay1 n z))
("rhythm" nil (r ih1 dh ax m))
("rhythmic" nil (r ih1 dh m ih0 k))
("rhythmically" nil (r ih1 dh m ih0 k l iy0))
("rhythms" nil (r ih1 dh ax m z))
("ria" nil (r iy1 ax))
("riady" nil (r iy0 aa1 d iy0))
("riadys" nil (r iy0 aa1 d iy0 z))
("rial" nil (r ay1 ax l))
("rials" nil (r ay1 l z))
("rib" nil (r ih1 b))
("riba" nil (r iy1 b ax))
("ribar" nil (r ih0 b aa1 r))
("ribas" nil (r ay1 b ax z))
("ribaudo" nil (r iy0 b ao1 d ow0))
("ribavirin" nil (r ih0 b ae1 v ih0 r ih0 n))
("ribbed" nil (r ih1 b d))
("ribbing" nil (r ih1 b ih0 ng))
("ribble" nil (r ih1 b ax l))
("ribbon" nil (r ih1 b ax n))
("ribbons" nil (r ih1 b ax n z))
("ribeiro" nil (r iy0 b eh1 r ow0))
("ribera" nil (r iy0 b eh1 r ax))
("ribi" nil (r iy1 b iy0))
("ribicoff" nil (r ih1 b ih0 k ao1 f))
("ribs" nil (r ih1 b z))
("ric" nil (r ih1 k))
("rica" nil (r iy1 k ax))
("ricadonna" nil (r iy0 k aa0 d ow1 n ax))
("rican" nil (r iy1 k ax n))
("ricans" nil (r ih1 k ax n z))
("ricard" nil (r ih0 k aa1 r d))
("ricardo" nil (r ih0 k aa1 r d ow0))
("ricca" nil (r iy1 k ax))
("riccardi" nil (r iy0 k aa1 r d iy0))
("riccardo" nil (r ih0 k aa1 r d ow0))
("ricci" nil (r iy1 ch iy0))
("ricciardelli" nil (r iy0 ch er0 d eh1 l iy0))
("ricciardi" nil (r iy0 ch aa1 r d iy0))
("riccio" nil (r iy1 ch iy0 ow0))
("riccitelli" nil (r iy0 ch iy0 t eh1 l iy0))
("ricciuti" nil (r iy0 k ch uw1 t iy0))
("ricco" nil (r ih1 k ow0))
("riccobono" nil (r iy0 k ow0 b ow1 n ow0))
("rice" nil (r ay1 s))
("rich" nil (r ih1 ch))
("richa" nil (r ih1 ch ax))
("richard" nil (r ih1 ch er0 d))
("richards" nil (r ih1 ch er0 d z))
("richardson" nil (r ih1 ch er0 d s ax n))
("richardsonsmith" nil (r ih1 ch er0 d s ax n s m ih1 th))
("richardt" nil (r ih1 k aa0 r t))
("richarson" nil (r ih1 ch aa1 r s ax n))
("richart" nil (r ih1 k hh aa0 r t))
("richbourg" nil (r ih1 ch b ao1 r g))
("richburg" nil (r ih1 ch b er0 g))
("richco" nil (r ih1 k ow0))
("richcreek" nil (r ih1 ch k r iy1 k))
("riche" nil (r ih1 ch))
("richelieu" nil (r ih1 sh ax l uw1))
("richelson" nil (r ih1 ch ax l s ax n))
("richemont" nil (r ih1 ch ax m aa1 n t))
("richenthal" nil (r ih1 k ax n th ao1 l))
("richer" nil (r ih1 ch er0))
("richerd" nil (r ih1 ch er0 d))
("richerson" nil (r ih1 ch er0 s ax n))
("richert" nil (r ih1 k er0 t))
("riches" nil (r ih1 ch ax z))
("richesin" nil (r ih1 k ih0 s ih0 n))
("richeson" nil (r ih1 k ih0 s ax n))
("richest" nil (r ih1 ch ax s t))
("richey" nil (r ih1 ch iy0))
("richfield" nil (r ih1 ch f iy1 l d))
("richford" nil (r ih1 ch f er0 d))
("richie" nil (r ih1 ch iy0))
("richins" nil (r ih1 k ih0 n z))
("richison" nil (r ih1 ch ih0 s ax n))
("richland" nil (r ih1 ch l ax n d))
("richly" nil (r ih1 ch l iy0))
("richman" nil (r ih1 ch m ae1 n))
("richmond" nil (r ih1 ch m ax n d))
("richner" nil (r ih1 k n er0))
("richness" nil (r ih1 ch n ax s))
("richoux" nil (r ih0 sh uw1))
("richstone" nil (r ih1 ch s t ow1 n))
("richter" nil (r ih1 k t er0))
("richton" nil (r ih1 ch t ax n))
("richwhite" nil (r ih1 ch w ay1 t))
("richwine" nil (r ih1 ch w ay1 n))
("ricigliano" nil (r ih0 s ih1 g l iy0 aa1 n ow0))
("rick" nil (r ih1 k))
("rickabaugh" nil (r ih1 k ax b ao1))
("rickard" nil (r ih1 k er0 d))
("rickards" nil (r ih1 k er0 d z))
("ricke" nil (r ih1 k))
("rickel" nil (r ih1 k ax l))
("rickels" nil (r ih1 k ax l z))
("ricken" nil (r ih1 k ax n))
("rickenbach" nil (r ih1 k ih0 n b aa0 k))
("rickenbaker" nil (r ih1 k ih0 n b ax k er0))
("ricker" nil (r ih1 k er0))
("rickers" nil (r ih1 k er0 z))
("rickerson" nil (r ih1 k er0 s ax n))
("rickert" nil (r ih1 k er0 t))
("ricketson" nil (r ih1 k ih0 t s ax n))
("rickett" nil (r ih1 k ih0 t))
("ricketts" nil (r ih1 k ih0 t s))
("rickety" nil (r ih1 k ax t iy0))
("rickey" nil (r ih1 k iy0))
("ricki" nil (r ih1 k iy0))
("rickie" nil (r ih1 k iy0))
("ricklefs" nil (r ih1 k l ih0 f s))
("rickles" nil (r ih1 k ax l z))
("rickman" nil (r ih1 k m ax n))
("rickner" nil (r ih1 k n er0))
("ricks" nil (r ih1 k s))
("rickson" nil (r ih1 k s ax n))
("rickward" nil (r ih1 k w er0 d))
("rickwood" nil (r ih1 k w uh1 d))
("ricky" nil (r ih1 k iy0))
("rico" nil (r iy1 k ow0))
("ricochet" nil (r ih1 k ax sh ey1))
("ricocheted" nil (r ih1 k ax sh ey1 d))
("ricoh" nil (r iy1 k ow0))
("ricostruzione" nil (r iy0 k ow0 s t r uw0 z iy0 ow1 n iy0))
("rid" nil (r ih1 d))
("riddance" nil (r ih1 d ax n s))
("riddell" nil (r ih1 d ax l))
("ridden" nil (r ih1 d ax n))
("ridder" nil (r ih1 d er0))
("riddick" nil (r ih1 d ih0 k))
("ridding" nil (r ih1 d ih0 ng))
("riddle" nil (r ih1 d ax l))
("riddled" nil (r ih1 d ax l d))
("riddles" nil (r ih1 d ax l z))
("riddock" nil (r ih1 d ax k))
("ride" nil (r ay1 d))
("rideau" nil (r ih0 d ow1))
("rideaux" nil (r ih0 d ow1))
("riden" nil (r ay1 d ax n))
("ridener" nil (r ih1 d iy0 n er0))
("ridenhour" nil (r ih1 d ih0 n aw0 r))
("ridenour" nil (r ih1 d ih0 n uh0 r))
("rideout" nil (r ay1 d aw1 t))
("rider" nil (r ay1 d er0))
("riders" nil (r ay1 d er0 z))
("ridership" nil (r ay1 d er0 sh ih1 p))
("rides" nil (r ay1 d z))
("ridge" nil (r ih1 jh))
("ridgecrest" nil (r ih1 jh k r eh1 s t))
("ridged" nil (r ih1 jh d))
("ridgefield" nil (r ih1 jh f iy1 l d))
("ridgell" nil (r ih1 jh ax l))
("ridgely" nil (r ih1 jh l iy0))
("ridges" nil (r ih1 jh ax z))
("ridgeway" nil (r ih1 jh w ey1))
("ridgley" nil (r ih1 jh l iy0))
("ridgway" nil (r ih1 jh w ey1))
("ridicule" nil (r ih1 d ax k y uw1 l))
("ridiculed" nil (r ih1 d ax k y uw1 l d))
("ridicules" nil (r ih1 d ax k y uw1 l z))
("ridiculing" nil (r ih1 d ax k y uw1 l ih0 ng))
("ridiculous" nil (r ih0 d ih1 k y ax l ax s))
("ridiculously" nil (r ax d ih1 k y ax l ax s l iy0))
("riding" nil (r ay1 d ih0 ng))
("ridinger" nil (r ay1 d ih0 ng er0))
("ridings" nil (r ay1 d ih0 ng z))
("ridley" nil (r ih1 d l iy0))
("ridling" nil (r ih1 d l ih0 ng))
("ridlon" nil (r ih1 d l ax n))
("ridner" nil (r ih1 d n er0))
("ridolfi" nil (r iy0 d ow1 l f iy0))
("ridout" nil (r ih1 d aw1 t))
("ridpath" nil (r ih1 d p ae1 th))
("rids" nil (r ih1 d z))
("riebe" nil (r iy1 b))
("riebel" nil (r iy1 b ax l))
("rieben" nil (r iy1 b ax n))
("rieber" nil (r iy1 b er0))
("riebman" nil (r iy1 b m ax n))
("riechers" nil (r iy1 k er0 z))
("rieck" nil (r iy1 k))
("riecke" nil (r iy1 k))
("ried" nil (r iy1 d))
("riede" nil (r iy1 d))
("riedel" nil (r iy1 d ax l))
("rieder" nil (r iy1 d er0))
("riederer" nil (r iy1 d er0 er0))
("riedesel" nil (r iy1 d ih0 s ax l))
("riedl" nil (r iy1 d ax l))
("riedlinger" nil (r iy1 d ax l ih0 ng er0))
("riedy" nil (r iy1 d iy0))
("rief" nil (r iy1 f))
("rieg" nil (r iy1 g))
("riege" nil (r iy1 jh))
("riegel" nil (r iy1 g ax l))
("rieger" nil (r iy1 g er0))
("riegert" nil (r iy1 g er0 t))
("riegle" nil (r iy1 g ax l))
("riegler" nil (r iy1 g l er0))
("riehl" nil (r iy1 l))
("riehle" nil (r iy1 hh ax l))
("riehm" nil (r iy1 m))
("riek" nil (r iy1 k))
("rieke" nil (r iy1 k))
("rieken" nil (r iy1 k ax n))
("rieker" nil (r iy1 k er0))
("riel" nil (r iy1 l))
("rielly" nil (r iy1 l iy0))
("riely" nil (r iy1 l iy0))
("rieman" nil (r iy1 m ax n))
("riemann" nil (r iy1 m ax n))
("riemenschneider" nil (r iy1 m ih0 n sh n ay0 d er0))
("riemer" nil (r iy1 m er0))
("riemersma" nil (r iy0 m eh1 r s m ax))
("riendeau" nil (r ax n d ow1))
("rienstra" nil (r iy1 n s t r ax))
("riepe" nil (r iy1 p))
("riera" nil (r iy1 r ax))
("rierson" nil (r iy1 r s ax n))
("ries" nil (r ay1 z))
("riesberg" nil (r iy1 s b er0 g))
("riese" nil (r iy1 z))
("riesen" nil (r iy1 s ax n))
("riesenberg" nil (r iy1 s ax n b er0 g))
("rieser" nil (r iy1 s er0))
("riesh" nil (r ay1 sh))
("riesling" nil (r iy1 z l ih0 ng))
("riess" nil (r iy1 s))
("riester" nil (r ay1 ih0 s t er0))
("riesterer" nil (r iy1 s t er0 er0))
("rieth" nil (r ay1 ih0 th))
("rietveld" nil (r iy1 t v ih0 l d))
("rietz" nil (r iy1 t s))
("rieves" nil (r iy1 v z))
("rife" nil (r ay1 f))
("rifenburgh" nil (r ih1 f ax n b er0 g))
("riff" nil (r ih1 f))
("riffe" nil (r ih1 f))
("riffel" nil (r ih1 f ax l))
("riffey" nil (r ih1 f iy0))
("riffle" nil (r ih1 f ax l))
("riffraff" nil (r ih1 f r ae1 f))
("rifkin" nil (r ih1 f k ih0 n))
("rifkind" nil (r ih1 f k ih0 n d))
("rifle" nil (r ay1 f ax l))
("rifled" nil (r ay1 f ax l d))
("rifleman" nil (r ay1 f ax l m ax n))
("riflery" nil (r ay1 f ax l r iy0))
("rifles" nil (r ay1 f ax l z))
("rifling" nil (r ay1 f l ih0 ng))
("rift" nil (r ih1 f t))
("rifts" nil (r ih1 f t s))
("rig" nil (r ih1 g))
("riga" nil (r iy1 g ax))
("rigas" nil (r ay1 g ax z))
("rigby" nil (r ih1 g b iy0))
("rigdon" nil (r ih1 g d ax n))
("rigel" nil (r ay1 jh ax l))
("rigg" nil (r ih1 g))
("riggan" nil (r ih1 g ax n))
("rigged" nil (r ih1 g d))
("riggen" nil (r ih1 g ax n))
("riggenbach" nil (r ih1 g ih0 n b aa0 k))
("riggi" nil (r ih1 jh iy0))
("riggin" nil (r ih1 g ih0 n))
("rigging" nil (r ih1 g ih0 ng))
("riggings" nil (r ih1 g ih0 ng z))
("riggins" nil (r ih1 g ih0 n z))
("riggio" nil (r iy1 jh iy0 ow0))
("riggle" nil (r ih1 g ax l))
("riggleman" nil (r ih1 g ax l m ax n))
("riggs" nil (r ih1 g z))
("riggsbee" nil (r ih1 g z b iy0))
("righetti" nil (r iy0 g eh1 t iy0))
("righi" nil (r iy1 g iy0))
("right" nil (r ay1 t))
("righted" nil (r ay1 t ih0 d))
("righteous" nil (r ay1 ch ax s))
("righteously" nil (r ay1 ch ax s l iy0))
("righteousness" nil (r ay1 ch ax s n ax s))
("righter" nil (r ay1 t er0))
("rightful" nil (r ay1 t f ax l))
("rightfully" nil (r ay1 t f ax l iy0))
("rightist" nil (r ay1 t ih0 s t))
("rightists" nil (r ay1 t ih0 s t s))
("rightly" nil (r ay1 t l iy0))
("rightmire" nil (r ay1 t m ay1 r))
("rightmyer" nil (r ay1 t m iy0 er0))
("rightness" nil (r ay1 t n ax s))
("rights" nil (r ay1 t s))
("rightward" nil (r ay1 t w er0 d))
("rightwing" nil (r ay1 t w ih1 ng))
("rigid" nil (r ih1 jh ax d))
("rigidities" nil (r ih0 jh ih1 d ih0 t iy0 z))
("rigidity" nil (r ih0 jh ih1 d ax t iy0))
("rigidly" nil (r ih1 jh ih0 d l iy0))
("rigler" nil (r ay1 g ax l er0))
("rigney" nil (r ih1 g n iy0))
("rigo" nil (r iy1 g ow0))
("rigoletto" nil (r ih1 g ax l eh1 t ow0))
("rigoni" nil (r iy0 g ow1 n iy0))
("rigor" nil (r ih1 g er0))
("rigorous" nil (r ih1 g er0 ax s))
("rigorously" nil (r ih1 g er0 ax s l iy0))
("rigors" nil (r ih1 g er0 z))
("rigs" nil (r ih1 g z))
("rigsbee" nil (r ih1 g z b iy1))
("rigsby" nil (r ih1 g z b iy0))
("rigueur" nil (r ih0 g y uw1 r))
("riha" nil (r ay1 hh ax))
("rihn" nil (r ih1 n))
("rijn" nil (r iy1 ax n))
("rikard" nil (r ih1 k er0 d))
("rike" nil (r ay1 k))
("riker" nil (r ay1 k er0))
("riklis" nil (r ih1 k l ih0 s))
("riksbank" nil (r ih1 k s b ae1 ng k))
("riksdag" nil (r ih1 k s d ae0 g))
("rile" nil (r ay1 l))
("riled" nil (r ay1 l d))
("riles" nil (r ay1 l z))
("riley" nil (r ay1 l iy0))
("rill" nil (r ih1 l))
("rilla" nil (r ih1 l ax))
("rillette" nil (r ih0 l eh1 t))
("rilling" nil (r ih1 l ih0 ng))
("rilwanu" nil (r ih0 l w aa1 n uw0))
("rim" nil (r ih1 m))
("rima" nil (r iy1 m ax))
("rimbey" nil (r ih1 m b iy0))
("rimel" nil (r ih1 m ax l))
("rimer" nil (r ay1 m er0))
("rimes" nil (r ay1 m z))
("rimkus" nil (r ih1 m k ax s))
("rimmed" nil (r ih1 m d))
("rimmer" nil (r ih1 m er0))
("rims" nil (r ih1 m z))
("rimson" nil (r ih1 m s ax n))
("rina" nil (r iy1 n ax))
("rinaldi" nil (r iy0 n aa1 l d iy0))
("rinaldo" nil (r ih0 n aa1 l d ow0))
("rinard" nil (r ih1 n er0 d))
("rinas" nil (r ay1 n ax z))
("rinaudo" nil (r iy0 n ao1 d ow0))
("rinck" nil (r ih1 ng k))
("rincon" nil (r ih1 ng k ax n))
("rind" nil (r ay1 n d))
("rindahl" nil (r ih1 n d aa1 l))
("rinderer" nil (r ih1 n d er0 er0))
("rinderknecht" nil (r ih1 n d er0 k n ih0 k t))
("rindfleisch" nil (r ih1 n d f l ay1 sh))
("rinds" nil (r ay1 n d z))
("rindt" nil (r ih1 n t))
("rine" nil (r ay1 n))
("rineer" nil (r ih1 n ih0 r))
("rinehardt" nil (r ay1 n hh aa1 r t))
("rinehart" nil (r ay1 n hh aa1 r t))
("rinehimer" nil (r ay1 n hh ay1 m er0))
("rinella" nil (r ih0 n eh1 l ax))
("riner" nil (r ay1 n er0))
("rines" nil (r ay1 n z))
("riney" nil (r ay1 n iy0))
("ring" nil (r ih1 ng))
("ringberg" nil (r ih1 ng b er0 g))
("ringdahl" nil (r ih1 ng d aa0 l))
("ringe" nil (r ih1 ng))
("ringed" nil (r ih1 ng d))
("ringeisen" nil (r ih1 ng g ay0 s ax n))
("ringel" nil (r ih1 ng g ax l))
("ringen" nil (r ih1 ng ax n))
("ringenberg" nil (r ih1 ng ax n b er0 g))
("ringer" nil (r ih1 ng er0))
("ringers" nil (r ih1 ng er0 z))
("ringgenberg" nil (r ih1 ng g ax n b er0 g))
("ringgit" nil (r ih1 ng ih0 t))
("ringgold" nil (r ih1 ng g ow1 l d))
("ringing" nil (r ih1 ng ih0 ng))
("ringland" nil (r ih1 ng g l ax n d))
("ringle" nil (r ih1 ng g ax l))
("ringleader" nil (r ih1 ng l iy1 d er0))
("ringleaders" nil (r ih1 ng l iy1 d er0 z))
("ringler" nil (r ih1 ng g ax l er0))
("ringley" nil (r ih1 ng g l iy0))
("ringling" nil (r ih1 ng l ih0 ng))
("ringmaster" nil (r ih1 ng m ae1 s t er0))
("ringneck" nil (r ih1 ng n eh1 k))
("ringo" nil (r iy1 ng g ow0))
("ringold" nil (r ih1 ng g ow0 l d))
("ringquist" nil (r ih1 ng k w ih0 s t))
("rings" nil (r ih1 ng z))
("ringside" nil (r ih1 ng s ay1 d))
("ringstad" nil (r ih1 ng s t ax d))
("ringuette" nil (r ih0 ng eh1 t))
("ringwald" nil (r ih1 ng w ao0 l d))
("ringwood" nil (r ih1 ng w uh1 d))
("rini" nil (r iy1 n iy0))
("rininger" nil (r ih1 n ih0 ng er0))
("rink" nil (r ih1 ng k))
("rinke" nil (r ih1 ng k))
("rinkenberger" nil (r ih1 ng k ax n b er0 g er0))
("rinker" nil (r ih1 ng k er0))
("rinks" nil (r ih1 ng k s))
("rinn" nil (r ih1 n))
("rinne" nil (r ih1 n))
("rinse" nil (r ih1 n s))
("rinsing" nil (r ih1 n s ih0 ng))
("rintala" nil (r iy0 n t aa1 l ax))
("rintel" nil (r ih0 n t eh1 l))
("rintels" nil (r ih0 n t eh1 l z))
("rio" nil (r iy1 ow0))
("riojas" nil (r iy0 ow1 y aa0 z))
("riolo" nil (r iy0 ow1 l ow0))
("rion" nil (r ay1 ax n))
("riopel" nil (r iy0 ax p eh1 l))
("riopelle" nil (r iy0 ax p eh1 l iy0))
("riordan" nil (r ih1 r d ax n))
("rios" nil (r iy1 ow0 s))
("riot" nil (r ay1 ax t))
("rioted" nil (r ay1 ax t ih0 d))
("rioters" nil (r ay1 ax t er0 z))
("rioting" nil (r ay1 ax t ih0 ng))
("riotous" nil (r ay1 ax t ax s))
("riots" nil (r ay1 ax t s))
("rioux" nil (r uw1))
("rip" nil (r ih1 p))
("ripa" nil (r iy1 p ax))
("ripe" nil (r ay1 p))
("ripen" nil (r ay1 p ax n))
("ripened" nil (r ay1 p ax n d))
("ripening" nil (r ay1 p ax n ih0 ng))
("riphagen" nil (r ih1 p aa1 g ax n))
("ripka" nil (r iy1 p k ax))
("ripke" nil (r ih1 p k))
("ripley" nil (r ih1 p l iy0))
("riposte" nil (r ih0 p ow1 s t))
("ripostes" nil (r ih0 p ao1 s t s))
("ripp" nil (r ih1 p))
("rippe" nil (r ih1 p))
("ripped" nil (r ih1 p t))
("rippee" nil (r ih1 p iy1))
("rippel" nil (r ih1 p ax l))
("rippeon" nil (r ih1 p iy0 ax n))
("ripper" nil (r ih1 p er0))
("ripperger" nil (r ih1 p er0 g er0))
("rippetoe" nil (r ih1 p ih0 t ow0))
("rippey" nil (r ih1 p iy0))
("ripping" nil (r ih1 p ih0 ng))
("ripple" nil (r ih1 p ax l))
("rippled" nil (r ih1 p ax l d))
("ripples" nil (r ih1 p ax l z))
("rippling" nil (r ih1 p ax l ih0 ng))
("ripplinger" nil (r ih1 p ax l ih0 ng er0))
("rippon" nil (r ih1 p ax n))
("rippy" nil (r ih1 p iy0))
("rips" nil (r ih1 p s))
("risa" nil (r iy1 s ax))
("risberg" nil (r ih1 s b er0 g))
("risc" nil (r ih1 s k))
("risch" nil (r ih1 sh))
("risden" nil (r ih1 s d ax n))
("risdon" nil (r ih1 s d ax n))
("rise" nil (r ay1 z))
("risen" nil (r ih1 z ax n))
("risenhoover" nil (r ih1 s ih0 n hh uw0 v er0))
("riser" nil (r ay1 z er0))
("rises" nil (r ay1 z ax z))
("rish" nil (r ih1 sh))
("rishel" nil (r ih1 sh ax l))
("rishell" nil (r ih1 sh ax l))
("risher" nil (r ih1 sh er0))
("risi" nil (r iy1 s iy0))
("rising" nil (r ay1 z ih0 ng))
("risinger" nil (r ay1 z ih0 ng er0))
("risk" nil (r ih1 s k))
("riske" nil (r ih1 s k))
("risked" nil (r ih1 s k t))
("riskier" nil (r ih1 s k iy0 er0))
("riskiest" nil (r ih1 s k iy0 ax s t))
("riskin" nil (r ih1 s k ih0 n))
("riskiness" nil (r ih1 s k iy0 n ax s))
("risking" nil (r ih1 s k ih0 ng))
("riskless" nil (r ih1 s k l ax s))
("risko" nil (r ih1 s k ow0))
("risks" nil (r ih1 s k s))
("risky" nil (r ih1 s k iy0))
("risley" nil (r ih1 z l iy0))
("risner" nil (r ih1 s n er0))
("riso" nil (r iy1 s ow0))
("rison" nil (r ih1 s ax n))
("rispoli" nil (r iy0 s p ow1 l iy0))
("risque" nil (r ih1 s k ey1))
("riss" nil (r ih1 s))
("risse" nil (r ih1 s))
("risser" nil (r ih1 s er0))
("rissler" nil (r ih1 s l er0))
("rissman" nil (r ih1 s m ax n))
("rissmiller" nil (r ih1 s m ax l er0))
("risso" nil (r ih1 s ow0))
("rist" nil (r ih1 s t))
("ristau" nil (r ih1 s t aw0))
("rister" nil (r ih1 s t er0))
("riston" nil (r ih1 s t ax n))
("ristorante" nil (r ih0 s t er0 aa1 n t ey0))
("ristow" nil (r ih1 s t ow0))
("rita" nil (r iy1 t ax))
("ritacco" nil (r iy0 t aa1 k ow0))
("ritalin" nil (r ih1 t ax l ih0 n))
("ritch" nil (r ih1 ch))
("ritchey" nil (r ih1 ch iy0))
("ritchie" nil (r ih1 ch iy0))
("rite" nil (r ay1 t))
("ritenour" nil (r ih1 t ih0 n uh0 r))
("riter" nil (r ay1 t er0))
("rites" nil (r ay1 t s))
("ritger" nil (r ih1 t g er0))
("ritner" nil (r ih1 t n er0))
("ritsema" nil (r ih0 t s iy1 m ax))
("ritson" nil (r ih1 t s ax n))
("ritt" nil (r ih1 t))
("rittenberg" nil (r ih1 t ax n b er0 g))
("rittenberry" nil (r ih1 t ax n b eh0 r iy0))
("rittenhouse" nil (r ih1 t ax n hh aw1 s))
("rittenour" nil (r ih1 t ih0 n er0))
("ritter" nil (r ih1 t er0))
("ritterbusch" nil (r ih1 t er0 b uh1 sh))
("rittereiser" nil (r ih1 t er0 ay1 z er0))
("rittgers" nil (r ih1 t g er0 z))
("ritthaler" nil (r ih1 th ax l er0))
("rittle" nil (r ih1 t ax l))
("rittman" nil (r ih1 t m ax n))
("rittner" nil (r ih1 t n er0))
("ritts" nil (r ih1 t s))
("ritual" nil (r ih1 ch uw0 ax l))
("ritualistic" nil (r ih1 ch uw0 ax l ih1 s t ih0 k))
("ritualistically" nil (r ih1 ch uw0 ax l ih1 s t ih0 k ax l iy0))
("rituals" nil (r ih1 ch uw0 ax l z))
("ritz" nil (r ih1 t s))
("ritzel" nil (r ih1 t z ax l))
("ritzenthaler" nil (r ih1 t z ih0 n th ax l er0))
("ritzer" nil (r ih1 t z er0))
("ritzert" nil (r ih1 t z er0 t))
("ritzman" nil (r ih1 t s m ax n))
("ritzy" nil (r ih1 t s iy0))
("riunite" nil (r iy0 uw0 n ay1 t))
("riva" nil (r iy1 v ax))
("rival" nil (r ay1 v ax l))
("rivaled" nil (r ay1 v ax l d))
("rivaling" nil (r ay1 v ax l ih0 ng))
("rivalries" nil (r ay1 v ax l r iy0 z))
("rivalry" nil (r ay1 v ax l r iy0))
("rivals" nil (r ay1 v ax l z))
("rivard" nil (r ih1 v er0 d))
("rivas" nil (r iy1 v aa0 z))
("riven" nil (r ih1 v ax n))
("rivenbark" nil (r ih1 v ih0 n b aa0 r k))
("rivenburg" nil (r ih1 v ax n b er0 g))
("rivenburgh" nil (r ih1 v ih0 n b er0 g))
("river" nil (r ih1 v er0))
("rivera" nil (r ih0 v eh1 r ax))
("riverbank" nil (r ih1 v er0 b ae1 ng k))
("riverbed" nil (r ih1 v er0 b eh1 d))
("riverbend" nil (r ih1 v er0 b eh1 n d))
("riverboat" nil (r ih1 v er0 b ow1 t))
("riverboats" nil (r ih1 v er0 b ow1 t s))
("riverdale" nil (r ih1 v er0 d ey1 l))
("riverfront" nil (r ih1 v er0 f r ah1 n t))
("riverhead" nil (r ih1 v er0 hh eh1 d))
("rivero" nil (r iy0 v eh1 r ow0))
("rivers" nil (r ih1 v er0 z))
("riverside" nil (r ih1 v er0 s ay1 d))
("riverso" nil (r ih1 v er0 s ow0))
("riverway" nil (r ih1 v er0 w ey1))
("riverwood" nil (r ih1 v er0 w uh1 d))
("rives" nil (r ay1 v z))
("rivest" nil (r ih1 v ih0 s t))
("rivet" nil (r ih1 v ax t))
("riveted" nil (r ih1 v ax t ih0 d))
("riveting" nil (r ih1 v ax t ih0 ng))
("rivets" nil (r ih1 v ax t s))
("rivett" nil (r ih1 v ih0 t))
("rivette" nil (r ih0 v eh1 t))
("riviello" nil (r iy1 v iy0 eh1 l ow0))
("riviera" nil (r ih1 v iy0 eh1 r ax))
("riviere" nil (r ih1 v iy0 eh0 r))
("rivieres" nil (r ih0 v y ih1 r eh0 z))
("rivkin" nil (r ih1 v k ih0 n))
("rivkind" nil (r ih1 v k ih0 n d))
("rivlin" nil (r ih1 v l ih0 n))
("rix" nil (r ih1 k s))
("riyad" nil (r iy0 aa1 d))
("riyadh" nil (r iy0 ae1 d))
("riyals" nil (r iy0 y aa1 l z))
("rizer" nil (r ay1 z er0))
("rizk" nil (r ih1 z k))
("rizo" nil (r iy1 z ow0))
("rizor" nil (r ay1 z er0))
("rizza" nil (r ih1 z ax))
("rizzi" nil (r ih1 z iy0))
("rizzo" nil (r ih1 z ow0))
("rizzoli" nil (r ih0 z ow1 l iy0))
("rizzolo" nil (r iy0 t s ow1 l ow0))
("rizzuti" nil (r iy0 t s uw1 t iy0))
("rizzuto" nil (r iy0 t s uw1 t ow0))
("ro" nil (r ow1))
("roa" nil (r ow1 ax))
("roach" nil (r ow1 ch))
("roache" nil (r ow1 ch))
("roaches" nil (r ow1 ch ih0 z))
("road" nil (r ow1 d))
("roadblock" nil (r ow1 d b l aa1 k))
("roadblocks" nil (r ow1 d b l aa1 k s))
("roadcap" nil (r ow1 d k ae1 p))
("roades" nil (r ow1 d z))
("roadmaster" nil (r ow1 d m ae1 s t er0))
("roadrailer" nil (r ow1 d r ey1 l er0))
("roadrunner" nil (r ow1 d r ah1 n er0))
("roads" nil (r ow1 d z))
("roadshow" nil (r ow1 d sh ow1))
("roadside" nil (r ow1 d s ay1 d))
("roadsides" nil (r ow1 d s ay1 d z))
("roadster" nil (r ow1 d s t er0))
("roadway" nil (r ow1 d w ey1))
("roadways" nil (r ow1 d w ey1 z))
("roadwork" nil (r ow1 d w er1 k))
("roady" nil (r ow1 d iy0))
("roald" nil (r ow1 l d))
("roam" nil (r ow1 m))
("roamed" nil (r ow1 m d))
("roaming" nil (r ow1 m ih0 ng))
("roams" nil (r ow1 m z))
("roan" nil (r ow1 n))
("roane" nil (r ow1 n))
("roanna" nil (r ow1 n ax))
("roanoke" nil (r ow1 ax n ow1 k))
("roar" nil (r ao1 r))
("roared" nil (r ao1 r d))
("roaring" nil (r ao1 r ih0 ng))
("roark" nil (r ao1 r k))
("roarke" nil (r ao1 r k))
("roars" nil (r ao1 r z))
("roarty" nil (r ao1 r t iy0))
("roast" nil (r ow1 s t))
("roasted" nil (r ow1 s t ax d))
("roaster" nil (r ow1 s t er0))
("roasters" nil (r ow1 s t er0 z))
("roasting" nil (r ow1 s t ih0 ng))
("roasts" nil (r ow1 s t s))
("roat" nil (r ow1 t))
("roath" nil (r ow1 th))
("rob" nil (r aa1 b))
("roback" nil (r ow1 b ae1 k))
("robaina" nil (r ow0 b ey1 n ax))
("robak" nil (r ow1 b ax k))
("robar" nil (r ax b aa1 r))
("robards" nil (r aa1 b er0 d z))
("robare" nil (r ow0 b aa1 r ey0))
("robarge" nil (r aa1 b aa0 r g))
("robart" nil (r aa1 b aa1 r t))
("robarts" nil (r aa1 b aa1 r t s))
("robb" nil (r aa1 b))
("robbe" nil (r aa1 b))
("robbed" nil (r aa1 b d))
("robben" nil (r aa1 b ax n))
("robber" nil (r aa1 b er0))
("robberies" nil (r aa1 b er0 iy0 z))
("robbers" nil (r aa1 b er0 z))
("robberson" nil (r aa1 b er0 s ax n))
("robbery" nil (r aa1 b er0 iy0))
("robbie" nil (r aa1 b iy0))
("robbin" nil (r aa1 b ih0 n))
("robbing" nil (r aa1 b ih0 ng))
("robbins" nil (r aa1 b ih0 n z))
("robbs" nil (r aa1 b z))
("robby" nil (r aa1 b iy0))
("robe" nil (r ow1 b))
("robeck" nil (r ow1 b eh1 k))
("robed" nil (r ow1 b d))
("robel" nil (r ow1 b ax l))
("robello" nil (r ow0 b eh1 l ow0))
("robelo" nil (r aa1 b eh1 l ow0))
("rober" nil (r ow1 b er0))
("roberds" nil (r aa1 b er0 d z))
("roberg" nil (r ow1 b er0 g))
("roberge" nil (r aa1 b er0 jh))
("robers" nil (r ow1 b er0 z))
("roberson" nil (r aa1 b er0 s ax n))
("robert" nil (r aa1 b er0 t))
("roberta" nil (r ax b er1 t ax))
("roberti" nil (r ax b er1 t iy0))
("roberto" nil (r ax b eh1 r t ow0))
("roberts" nil (r aa1 b er0 t s))
("robertshaw" nil (r aa1 b er1 t sh ao1))
("robertson" nil (r aa1 b er0 t s ax n))
("robes" nil (r ow1 b z))
("robeson" nil (r ow1 b ih0 s ax n))
("robey" nil (r ow1 b iy0))
("robia" nil (r ow1 b iy0 ax))
("robichaud" nil (r aa1 b ih0 sh ow1))
("robichaux" nil (r aa1 b ih0 sh ow1))
("robicheaux" nil (r aa1 b ih0 sh ow1))
("robideau" nil (r aa1 b ih0 d ow1))
("robidoux" nil (r aa1 b ih0 d uw0))
("robie" nil (r ow1 b iy0))
("robillard" nil (r aa1 b ih0 l er0 d))
("robin" nil (r aa1 b ax n))
("robina" nil (r ax b iy1 n ax))
("robinett" nil (r aa1 b ih0 n eh0 t))
("robinette" nil (r aa1 b ih0 n eh1 t))
("robinia" nil (r ow0 b iy1 n iy0 ax))
("robins" nil (r aa1 b ax n z))
("robinson" nil (r aa1 b ax n s ax n))
("robinsons" nil (r aa1 b ax n s ax n z))
("robishaw" nil (r aa1 b ih0 sh ao1))
("robison" nil (r aa1 b ih0 s ax n))
("robitaille" nil (r aa1 b ih0 t ey1 l))
("robitussin" nil (r aa1 b ih0 t ah1 s ih0 n))
("robl" nil (r aa1 b ax l))
("roble" nil (r ow1 b ax l))
("robledo" nil (r ow0 b l ey1 d ow0))
("roblee" nil (r aa1 b l iy1))
("robles" nil (r ow1 b ax l z))
("robley" nil (r aa1 b l iy0))
("roblin" nil (r aa1 b l ih0 n))
("robling" nil (r ow1 b ax l ih0 ng))
("robnett" nil (r aa1 b n ih0 t))
("robocop" nil (r ow1 b ow0 k aa1 p))
("robot" nil (r ow1 b aa1 t))
("robotic" nil (r ow1 b aa1 t ih0 k))
("robotics" nil (r ow1 b aa1 t ih0 k s))
("robots" nil (r ow1 b aa1 t s))
("robs" nil (r aa1 b z))
("robson" nil (r aa1 b s ax n))
("robuck" nil (r aa1 b ax k))
("robust" nil (r ow0 b ah1 s t))
("robusta" nil (r aa1 b ah1 s t ax))
("robustas" nil (r aa1 b ah1 s t ax s))
("robustelli" nil (r ow1 b ax s t eh1 l iy0))
("robustly" nil (r ow1 b ah1 s t l iy0))
("robustness" nil (r ow0 b ah1 s t n ax s))
("roby" nil (r ow1 b iy0))
("robyn" nil (r aa1 b ih0 n))
("roca" nil (r ow1 k ax))
("rocap" nil (r ow1 k ae1 p))
("rocard" nil (r ow1 k er0 d))
("rocca" nil (r aa1 k ax))
("roccaforte" nil (r ow0 k ax f ao1 r t iy0))
("roccas" nil (r aa1 k ax s))
("rocchi" nil (r ow1 k iy0))
("rocchio" nil (r ow1 k iy0 ow0))
("rocco" nil (r aa1 k ow0))
("roch" nil (r aa1 k))
("rocha" nil (r ow1 k ax))
("rochat" nil (r aa1 ch ax t))
("rochberg" nil (r ow1 ch b er0 g))
("roche" nil (r ow1 ch))
("rochefort" nil (r aa1 k ih0 f er0 t))
("rocheleau" nil (r aa1 sh ih0 l ow0))
("rochella" nil (r ax ch eh1 l ax))
("rochelle" nil (r ow0 sh eh1 l))
("rochester" nil (r aa1 ch eh1 s t er0))
("rochette" nil (r ax sh eh1 t))
("rochford" nil (r aa1 ch f er0 d))
("rochlin" nil (r aa1 k l ih0 n))
("rochon" nil (r aa1 ch ax n))
("rock" nil (r aa1 k))
("rockafellow" nil (r aa1 k ax f eh1 l ow0))
("rockaway" nil (r aa1 k ax w ey1))
("rocke" nil (r aa1 k))
("rocked" nil (r aa1 k t))
("rockefeller" nil (r aa1 k ax f eh1 l er0))
("rockefellers" nil (r aa1 k ax f eh1 l er0 z))
("rockel" nil (r aa1 k ax l))
("rockenbach" nil (r aa1 k ih0 n b aa0 k))
("rocker" nil (r aa1 k er0))
("rockers" nil (r aa1 k er0 z))
("rocket" nil (r aa1 k ax t))
("rocketdyne" nil (r aa1 k ih0 t d ay1 n))
("rocketed" nil (r aa1 k ax t ih0 d))
("rocketing" nil (r aa1 k ax t ih0 ng))
("rocketry" nil (r aa1 k ax t r iy0))
("rockets" nil (r aa1 k ax t s))
("rockett" nil (r aa1 k ih0 t))
("rockey" nil (r aa1 k iy0))
("rockford" nil (r aa1 k f er0 d))
("rockhill" nil (r aa1 k hh ih1 l))
("rockhold" nil (r aa1 k hh ow1 l d))
("rockholt" nil (r aa1 k hh ow1 l t))
("rockies" nil (r aa1 k iy0 z))
("rocking" nil (r aa1 k ih0 ng))
("rockingham" nil (r aa1 k ih0 ng hh ae1 m))
("rockland" nil (r aa1 k l ax n d))
("rockley" nil (r aa1 k l iy0))
("rocklin" nil (r aa1 k l ih0 n))
("rockman" nil (r aa1 k m ax n))
("rockmore" nil (r aa1 k m ao0 r))
("rockne" nil (r aa1 k n iy0))
("rockoff" nil (r aa1 k ao1 f))
("rockow" nil (r aa1 s k ow0))
("rockport" nil (r aa1 k p ao1 r t))
("rockresorts" nil (r aa1 k r ih0 z ao1 r t s))
("rockrose" nil (r aa1 k r ow1 z))
("rocks" nil (r aa1 k s))
("rockville" nil (r aa1 k v ih1 l))
("rockwell" nil (r aa1 k w eh1 l))
("rockwood" nil (r aa1 k w uh1 d))
("rocky" nil (r aa1 k iy0))
("rococo" nil (r ax k ow1 k ow1))
("rocque" nil (r aa1 k))
("rod" nil (r aa1 d))
("roda" nil (r ow1 d ax))
("rodabaugh" nil (r aa1 d ax b ao0))
("rodak" nil (r ow1 d ax k))
("rodale" nil (r ow1 d ey1 l))
("rodarte" nil (r aa1 d aa0 r t))
("rodas" nil (r ow1 d ax z))
("rodd" nil (r aa1 d))
("rodden" nil (r aa1 d ax n))
("roddenberry" nil (r aa1 d ax n b eh0 r iy0))
("roddey" nil (r aa1 d iy0))
("roddick" nil (r aa1 d ih1 k))
("roddie" nil (r aa1 d iy0))
("rodding" nil (r aa1 d ih0 ng))
("roddy" nil (r aa1 d iy0))
("rode" nil (r ow1 d))
("rodebaugh" nil (r aa1 d ih0 b ao0))
("rodefer" nil (r aa1 d ih0 f er0))
("rodeffer" nil (r aa1 d ih0 f er0))
("rodeheaver" nil (r aa1 d ih0 hh iy0 v er0))
("rodela" nil (r ow0 d eh1 l ax))
("rodell" nil (r aa1 d ax l))
("roden" nil (r ow1 d ax n))
("rodenbaugh" nil (r aa1 d ih0 n b aw0))
("rodenbeck" nil (r ow1 d ax n b eh0 k))
("rodenberg" nil (r ow1 d ax n b er0 g))
("rodenberger" nil (r ow1 d ax n b er0 g er0))
("rodenburg" nil (r ow1 d ax n b er0 g))
("rodent" nil (r ow1 d ax n t))
("rodents" nil (r ow1 d ax n t s))
("rodeo" nil (r ow1 d iy0 ow1))
("rodeos" nil (r ow1 d iy0 ow1 z))
("roder" nil (r ow1 d er0))
("roderica" nil (r ow0 d er0 iy1 k ax))
("roderick" nil (r aa1 d r ih0 k))
("rodes" nil (r ow1 d z))
("rodewald" nil (r aa1 d uw0 ao0 l d))
("rodeway" nil (r ow1 d w ey1))
("rodge" nil (r aa1 jh))
("rodger" nil (r aa1 jh er0))
("rodgers" nil (r aa1 jh er0 z))
("rodgin" nil (r aa1 jh ih0 n))
("rodham" nil (r ao1 d ax m))
("rodi" nil (r ow1 d iy0))
("rodick" nil (r aa1 d ih0 k))
("rodier" nil (r ow1 d iy0 er0))
("rodime" nil (r ow0 d iy1 m))
("rodin" nil (r ow1 d ih0 n))
("rodine" nil (r ow0 d iy1 n iy0))
("rodino" nil (r ow0 d iy1 n ow0))
("rodkey" nil (r aa1 d k iy1))
("rodman" nil (r aa1 d m ax n))
("rodmond" nil (r aa1 d m ax n d))
("rodmund" nil (r aa1 d m ax n d))
("rodney" nil (r aa1 d n iy0))
("rodocker" nil (r aa1 d ax k er0))
("rodolf" nil (r aa1 d ow0 l f))
("rodolfo" nil (r ow0 d aa1 l f ow0))
("rodolph" nil (r ow1 d aa1 l f))
("rodrick" nil (r aa1 d r ih0 k))
("rodrigeuz" nil (r ow0 d r iy1 jh uw0 z))
("rodrigo" nil (r ax d r iy1 g ow0))
("rodrigue" nil (r ow1 d r iy0 g))
("rodrigues" nil (r aa0 d r iy1 g ih0 s))
("rodriguez" nil (r aa0 d r iy1 g eh0 z))
("rodriques" nil (r aa0 d r iy1 g eh0 s))
("rodriquez" nil (r aa0 d r iy1 k eh0 z))
("rods" nil (r aa1 d z))
("rodwell" nil (r aa1 d w eh1 l))
("rody" nil (r ow1 d iy0))
("roe" nil (r ow1))
("roeber" nil (r ow1 b er0))
("roebke" nil (r ow1 b k))
("roebling" nil (r ow1 b l ih0 ng))
("roebuck" nil (r ow1 b ah1 k))
("roecker" nil (r ow1 k er0))
("roed" nil (r ow1 d))
("roedel" nil (r ow1 d ax l))
("roeder" nil (r ow1 d er0))
("roederer" nil (r ow1 d er0 er0))
("roediger" nil (r ow1 d ih0 g er0))
("roedl" nil (r ow1 d ax l))
("roegner" nil (r ow1 g n er0))
("roehl" nil (r ow1 l))
("roehling" nil (r ow1 l ih0 ng))
("roehm" nil (r ow1 m))
("roehr" nil (r ao1 r))
("roehrich" nil (r ao1 r ih0 k))
("roehrig" nil (r ao1 r ih0 g))
("roehrs" nil (r ao1 r z))
("roeland" nil (r ow1 l ax n d))
("roelke" nil (r ow1 l k))
("roell" nil (r ow1 l))
("roelle" nil (r ow1 l))
("roeller" nil (r ow1 l er0))
("roelofs" nil (r ow1 l ax f s))
("roemer" nil (r ow1 m er0))
("roemmich" nil (r ow1 m ih0 k))
("roen" nil (r ow1 n))
("roeper" nil (r ow1 p er0))
("roepke" nil (r ow1 p k))
("roes" nil (r ow1 z))
("roesch" nil (r ow1 sh))
("roese" nil (r ow1 s))
("roesel" nil (r ow1 s ax l))
("roeser" nil (r ow1 z er0))
("roeske" nil (r ow1 s k))
("roesler" nil (r ow1 s ax l er0))
("roesner" nil (r ow1 s n er0))
("roessler" nil (r ow1 s ax l er0))
("roessner" nil (r ow1 s n er0))
("roether" nil (r ow1 dh er0))
("roethler" nil (r ow1 th ax l er0))
("roettger" nil (r ow1 t g er0))
("roever" nil (r aa1 eh0 v er0))
("roff" nil (r ao1 f))
("roffe" nil (r aa1 f))
("roffman" nil (r ao1 f m ax n))
("rofin" nil (r ow1 f ih0 n))
("rog" nil (r aa1 g))
("rogachev" nil (r ow1 g ax ch ax v))
("rogacki" nil (r ax g aa1 t s k iy0))
("rogaine" nil (r ow0 g ey1 n))
("rogal" nil (r ow1 g ax l))
("rogala" nil (r ow0 g aa1 l ax))
("rogalla" nil (r ow0 g aa1 l ax))
("rogalski" nil (r ax g aa1 l s k iy0))
("rogan" nil (r ow1 g ax n))
("rogel" nil (r ow1 g ax l))
("rogelio" nil (r ow0 g iy1 l iy0 ow0))
("roger" nil (r aa1 jh er0))
("rogernomics" nil (r ow1 g er0 n aa1 m ih0 k s))
("rogers" nil (r aa1 jh er0 z))
("rogerson" nil (r aa1 g er0 s ax n))
("rogge" nil (r aa1 g))
("roggenbuck" nil (r aa1 g ih0 n b ax k))
("roggenkamp" nil (r aa1 g ih0 n k ae0 m p))
("roggio" nil (r aa1 z iy0 ow0))
("roggow" nil (r aa1 g ow0))
("rogier" nil (r ow1 g iy0 er0))
("rogin" nil (r ow1 g ih0 n))
("roginski" nil (r ax g ih1 n s k iy0))
("rogness" nil (r aa1 g n ih0 s))
("rogoff" nil (r aa1 g ao0 f))
("rogowski" nil (r ax g ao1 f s k iy0))
("rogozinski" nil (r ax g ax z ih1 n s k iy0))
("rogstad" nil (r aa1 g s t ax d))
("rogue" nil (r ow1 g))
("rogues" nil (r ow1 g z))
("rogus" nil (r ow1 g ax s))
("roh" nil (r ow1))
("rohan" nil (r ow1 ax n))
("rohana" nil (r ax hh ae1 n ax))
("rohatyn" nil (r aa1 hh ax t ih0 n))
("rohde" nil (r ow1 d))
("rohe" nil (r ow1))
("roher" nil (r ow1 er0))
("rohl" nil (r ow1 l))
("rohland" nil (r ow1 l ax n d))
("rohleder" nil (r ow1 l ih0 d er0))
("rohlf" nil (r ow1 l f))
("rohlfing" nil (r ow1 l f ih0 ng))
("rohlfs" nil (r ow1 l f s))
("rohlicek" nil (r aa1 l ax ch eh0 k))
("rohling" nil (r ow1 l ih0 ng))
("rohlman" nil (r ow1 l m ax n))
("rohloff" nil (r ow1 l ao0 f))
("rohm" nil (r ow1 m))
("rohman" nil (r ow1 m ax n))
("rohmer" nil (r ow1 m er0))
("rohn" nil (r aa1 n))
("rohner" nil (r ow1 n er0))
("rohr" nil (r ao1 r))
("rohrbach" nil (r ao1 r b aa0 k))
("rohrbacher" nil (r ao1 r b aa0 k er0))
("rohrback" nil (r ao1 r b ae0 k))
("rohrbaugh" nil (r ao1 r b aw0))
("rohrbough" nil (r ao1 r b aw0))
("rohrer" nil (r ao1 r er0))
("rohrich" nil (r ao1 r ih0 k))
("rohrig" nil (r ao1 r ih0 g))
("rohrman" nil (r ao1 r m ax n))
("rohrs" nil (r ao1 r z))
("rohs" nil (r ow1 z))
("rohstoff" nil (r ow1 s t ao0 f))
("rohwedder" nil (r ow1 w ih0 d er0))
("rohweder" nil (r ow1 w ih0 d er0))
("rohwer" nil (r ow1 w er0))
("roi" nil (r oy1))
("roig" nil (r oy1 g))
("roil" nil (r oy1 l))
("roiled" nil (r oy1 l d))
("roiling" nil (r oy1 l ih0 ng))
("rojas" nil (r ow1 hh aa0 s))
("rojek" nil (r ow1 y eh0 k))
("rojo" nil (r ow1 jh ow0))
("roka" nil (r ow1 k ax))
("rokicki" nil (r ax k ih1 k iy0))
("rokos" nil (r ow1 k ow0 z))
("rokosz" nil (r aa1 k ax sh))
("rolan" nil (r ow1 l ax n))
("roland" nil (r ow1 l ax n d))
("rolanda" nil (r ow0 l aa1 n d ax))
("rolando" nil (r ow0 l aa1 n d ow0))
("roldan" nil (r ow1 l d ax n))
("role" nil (r ow1 l))
("rolen" nil (r ow1 l ax n))
("roleplaying" nil (r ow1 l p l ey1 ih0 ng))
("roles" nil (r ow1 l z))
("rolex" nil (r ow1 l eh0 k s))
("roley" nil (r ow1 l iy0))
("rolf" nil (r ow1 l f))
("rolfe" nil (r ow1 l f))
("rolfes" nil (r ow1 l f s))
("rolfs" nil (r ow1 l f s))
("rolfson" nil (r ow1 l f s ax n))
("rolin" nil (r ow1 l ih0 n))
("roling" nil (r ow1 l ih0 ng))
("rolison" nil (r aa1 l ih0 s ax n))
("roll" nil (r ow1 l))
("rolla" nil (r aa1 l ax))
("rolland" nil (r aa1 l ax n d))
("rollback" nil (r ow1 l b ae1 k))
("rollbacks" nil (r ow1 l b ae1 k s))
("rolle" nil (r ow1 l))
("rolled" nil (r ow1 l d))
("roller" nil (r ow1 l er0))
("rollercoaster" nil (r ow1 l er0 k ow1 s t er0))
("rollers" nil (r ow1 l er0 z))
("rolley" nil (r aa1 l iy0))
("rolli" nil (r aa1 l iy0))
("rollicking" nil (r aa1 l ih0 k ih0 ng))
("rollin" nil (r aa1 l ih0 n))
("rolling" nil (r ow1 l ih0 ng))
("rollinger" nil (r ow1 l ih0 ng er0))
("rollings" nil (r ow1 l ih0 ng z))
("rollins" nil (r aa1 l ih0 n z))
("rollinson" nil (r aa1 l ih0 n s ax n))
("rollison" nil (r aa1 l ih0 s ax n))
("rollman" nil (r ow1 l m ax n))
("rollo" nil (r aa1 l ow0))
("rollout" nil (r ow1 l aw1 t))
("rollover" nil (r ow1 l ow1 v er0))
("rollovers" nil (r ow1 l ow1 v er0 z))
("rolls" nil (r ow1 l z))
("rolltop" nil (r ow1 l t aa1 p))
("rollwagen" nil (r ow1 l w ae1 g ax n))
("rolly" nil (r ow1 l iy0))
("rollyson" nil (r aa1 l ih0 s ax n))
("rolm" nil (r ow1 m))
("rolnick" nil (r ow1 l n ih0 k))
("rolodex" nil (r ow1 l ax d eh1 k s))
("roloff" nil (r aa1 l ao0 f))
("rolon" nil (r ow1 l ax n))
("rolph" nil (r ow1 l f))
("rolston" nil (r ow1 l s t ax n))
("rolt" nil (r ow1 l t))
("rom" nil (r aa1 m))
("roma" nil (r ow1 m ax))
("romack" nil (r aa1 m ax k))
("romagnoli" nil (r ow0 m aa0 g n ow1 l iy0))
("romagnolo" nil (r ow1 m aa0 n y ow1 l ow0))
("romain" nil (r ow0 m ey1 n))
("romaine" nil (r ow0 m ey1 n))
("roman" nil (r ow1 m ax n))
("romance" nil (r ow0 m ae1 n s))
("romances" nil (r ow0 m ae1 n s ih0 z))
("romancing" nil (r ow0 m ae1 n s ih0 ng))
("romanek" nil (r aa1 m ax n ih0 k))
("romanelli" nil (r ow0 m aa0 n eh1 l iy0))
("romanello" nil (r ow0 m aa0 n eh1 l ow0))
("romanesque" nil (r ow1 m ax n eh1 s k))
("romani" nil (r ow0 m aa1 n iy0))
("romania" nil (r ow0 m ey1 n iy0 ax))
("romanian" nil (r ow0 m ey1 n iy0 ax n))
("romanians" nil (r ow0 m ey1 n iy0 ax n z))
("romaniello" nil (r ow0 m aa0 n iy0 eh1 l ow0))
("romanik" nil (r ax m ae1 n ih0 k))
("romann" nil (r ow1 m ax n))
("romano" nil (r ow0 m aa1 n ow1))
("romanoff" nil (r ow1 m ax n ao1 f))
("romanoski" nil (r ax m ax n aw1 s k iy0))
("romanow" nil (r ax m ae1 n ow0))
("romanowski" nil (r ax m ax n ao1 f s k iy0))
("romans" nil (r ow1 m ax n z))
("romanski" nil (r ax m ae1 n s k iy0))
("romantic" nil (r ow0 m ae1 n t ih0 k))
("romantically" nil (r ow0 m ae1 n t ih0 k ax l iy0))
("romanticism" nil (r ow0 m ae1 n t ax s ih1 z ax m))
("romanticize" nil (r ow0 m ae1 n t ax s ay1 z))
("romanticized" nil (r ow0 m ae1 n t ax s ay1 z d))
("romanticizing" nil (r ow0 m ae1 n t ax s ay1 z ih0 ng))
("romantics" nil (r ow0 m ae1 n t ih0 k s))
("romany" nil (r aa1 m ax n iy0))
("romas" nil (r ow1 m ax z))
("rombach" nil (r aa1 m b aa1 k))
("romberg" nil (r aa1 m b er0 g))
("romberger" nil (r aa1 m b er0 g er0))
("rome" nil (r ow1 m))
("romella" nil (r ow0 m eh1 l ax))
("romelle" nil (r ax m eh1 l))
("romeo" nil (r ow1 m iy0 ow1))
("romer" nil (r ow1 m er0))
("romero" nil (r ow0 m eh1 r ow0))
("romesburg" nil (r ow1 m z b er0 g))
("romey" nil (r ow1 m iy0))
("romick" nil (r aa1 m ih0 k))
("romig" nil (r ow1 m ih0 g))
("romilda" nil (r ax m iy1 l d ax))
("romina" nil (r ax m iy1 n ax))
("romine" nil (r ax m iy1 n iy0))
("romines" nil (r aa1 m ih0 n z))
("rominger" nil (r ow1 m ih0 ng er0))
("romito" nil (r ow0 m iy1 t ow0))
("romm" nil (r aa1 m))
("rommel" nil (r aa1 m ax l))
("romney" nil (r aa1 m n iy0))
("romo" nil (r ow1 m ow0))
("romola" nil (r aa1 m ax l ax))
("romp" nil (r aa1 m p))
("romping" nil (r aa1 m p ih0 ng))
("roms" nil (r aa1 m z))
("romulus" nil (r aa1 m y ax l ax s))
("ron" nil (r aa1 n))
("rona" nil (r ow1 n ax))
("ronald" nil (r aa1 n ax l d))
("ronalda" nil (r ow0 n aa1 l d ax))
("ronan" nil (r ow1 n ax n))
("ronayne" nil (r aa1 n ey1 n))
("ronca" nil (r ow1 n k ax))
("ronco" nil (r ow1 n k ow0))
("ronda" nil (r aa1 n d ax))
("rondeau" nil (r aa0 n d ow1))
("rondinelli" nil (r ow0 n d iy0 n eh1 l iy0))
("rondon" nil (r aa1 n d ax n))
("rondonia" nil (r aa0 n d ow1 n iy0 ax))
("rone" nil (r ow1 n))
("ronen" nil (r ow1 n ax n))
("roney" nil (r ow1 n iy0))
("rong" nil (r ao1 ng))
("rongji" nil (r ao1 ng jh iy0))
("ronin" nil (r ow1 n ih0 n))
("ronk" nil (r aa1 ng k))
("ronne" nil (r aa1 n))
("ronnie" nil (r aa1 n iy0))
("ronning" nil (r aa1 n ih0 ng))
("ronny" nil (r aa1 n iy0))
("ronquillo" nil (r ow0 n k w ih1 l ow0))
("ronson" nil (r aa1 n s ax n))
("ronstadt" nil (r aa1 n s t ae1 t))
("rood" nil (r uw1 d))
("roode" nil (r uw1 d))
("roof" nil (r uw1 f))
("roofed" nil (r uw1 f t))
("roofer" nil (r uw1 f er0))
("roofers" nil (r uw1 f er0 z))
("roofing" nil (r uw1 f ih0 ng))
("roofs" nil (r uw1 f s))
("rooftop" nil (r uw1 f t aa1 p))
("rooftops" nil (r uw1 f t aa1 p s))
("rooftree" nil (r uw1 f t r iy1))
("rook" nil (r uh1 k))
("rookard" nil (r uh1 k er0 d))
("rooke" nil (r uh1 k))
("rooker" nil (r uh1 k er0))
("rookie" nil (r uh1 k iy0))
("rookies" nil (r uh1 k iy0 z))
("rooks" nil (r uh1 k s))
("rookstool" nil (r uh1 k s t uw1 l))
("room" nil (r uw1 m))
("roome" nil (r uw1 m))
("roomful" nil (r uw1 m f uh1 l))
("roomier" nil (r uw1 m iy0 er0))
("roominess" nil (r uw1 m iy0 n ax s))
("rooming" nil (r uw1 m ih0 ng))
("roommate" nil (r uw1 m ey1 t))
("roommates" nil (r uw1 m ey1 t s))
("rooms" nil (r uw1 m z))
("roomy" nil (r uw1 m iy0))
("roone" nil (r uw1 n))
("rooney" nil (r uw1 n iy0))
("roop" nil (r uw1 p))
("roope" nil (r uw1 p))
("roorda" nil (r uh1 r d ax))
("roos" nil (r uw1 z))
("roosa" nil (r uw1 s ax))
("roose" nil (r uw1 z))
("roosevelt" nil (r ow1 z ax v eh1 l t))
("roost" nil (r uw1 s t))
("roosted" nil (r uw1 s t ih0 d))
("rooster" nil (r uw1 s t er0))
("root" nil (r uw1 t))
("rooted" nil (r uw1 t ax d))
("rooting" nil (r uw1 t ih0 ng))
("rootless" nil (r uw1 t l ax s))
("roots" nil (r uw1 t s))
("rootstein" nil (r uw1 t s t ay1 n))
("ropak" nil (r ow1 p ae0 k))
("rope" nil (r ow1 p))
("roped" nil (r ow1 p t))
("roper" nil (r ow1 p er0))
("ropers" nil (r ow1 p er0 z))
("ropes" nil (r ow1 p s))
("roping" nil (r ow1 p ih0 ng))
("ropp" nil (r aa1 p))
("roppolo" nil (r ow0 p ow1 l ow0))
("rops" nil (r aa1 p s))
("roque" nil (r ow1 k))
("roquemore" nil (r ow0 k m ao1 r))
("rorabaugh" nil (r ao1 r ax b ao0))
("rorer" nil (r ao1 r er0))
("rorex" nil (r ao1 r eh0 k s))
("rorick" nil (r ao1 r ih0 k))
("rorie" nil (r ao1 r iy0))
("rork" nil (r ao1 r k))
("rorke" nil (r ao1 r k))
("rorrer" nil (r ao1 er0 r))
("rory" nil (r ao1 r iy0))
("ros" nil (r ao1 s))
("rosa" nil (r ow1 z ax))
("rosabel" nil (r ow1 z ax b eh1 l))
("rosabelle" nil (r aa1 s ax b ax l))
("rosado" nil (r ax s aa1 d ow0))
("rosales" nil (r ow1 s ey0 l z))
("rosalez" nil (r ow0 s aa1 l eh0 z))
("rosalia" nil (r ow0 z ax l iy1 ax))
("rosalie" nil (r ow1 z ax l iy0))
("rosalind" nil (r aa1 z ax l ih0 n d))
("rosalinda" nil (r ow0 s aa0 l iy1 n d ax))
("rosalyn" nil (r aa1 z ax l ih0 n))
("rosalynd" nil (r aa1 z ax l ih0 n d))
("rosalynn" nil (r ow1 z ax l ih1 n))
("rosamilia" nil (r ow0 z aa0 m iy1 l iy0 ax))
("rosamond" nil (r ow1 z ax m ax n d))
("rosamund" nil (r ow0 z ax m uh1 n d))
("rosan" nil (r ow1 z ax n))
("rosander" nil (r ow0 z ae1 n d er0))
("rosanna" nil (r ow0 z ae1 n ax))
("rosanne" nil (r ow0 z ae1 n))
("rosano" nil (r ow0 z aa1 n ow0))
("rosaria" nil (r ow0 z aa1 r iy0 ax))
("rosaries" nil (r ow1 z er0 iy0 z))
("rosario" nil (r ow0 z aa1 r iy0 ow0))
("rosas" nil (r ow1 z ax z))
("rosasco" nil (r ow0 z aa1 s k ow0))
("rosati" nil (r ow0 z aa1 t iy0))
("rosato" nil (r ow0 z aa1 t ow0))
("rosberg" nil (r aa1 s b er0 g))
("rosborough" nil (r aa1 s b er0 ow0))
("rosch" nil (r ao1 sh))
("roscher" nil (r ao1 sh er0))
("roscoe" nil (r aa1 s k ow0))
("rose" nil (r ow1 z))
("roseanne" nil (r ow1 z ae1 n))
("roseberry" nil (r ow1 z b eh1 r iy0))
("roseboom" nil (r ow1 z b uw1 m))
("roseboro" nil (r ow0 z b er1 ow0))
("roseborough" nil (r ow1 z b er1 ow0))
("rosebrock" nil (r ow1 z b r ax k))
("rosebrook" nil (r ow1 z b r uh1 k))
("rosebrough" nil (r ow1 z b r aw0))
("rosebush" nil (r ow1 z b uh1 sh))
("rosecrans" nil (r ow1 z k r ae1 n z))
("rosekrans" nil (r ow1 z k r ax n z))
("rosel" nil (r ow1 z ax l))
("roseland" nil (r ow1 z l ax n d))
("roselawn" nil (r ow1 z l ao1 n))
("roseline" nil (r ow1 z l ay1 n))
("rosell" nil (r ow1 z ax l))
("rosella" nil (r ow0 z eh1 l ax))
("rosellen" nil (r ow1 z ax l ax n))
("roselli" nil (r ow0 s eh1 l iy0))
("rosello" nil (r ow0 s eh1 l ow0))
("roseman" nil (r ow1 z m ax n))
("rosemarie" nil (r ow1 z m er0 iy1))
("rosemary" nil (r ow1 z m eh1 r iy0))
("rosemead" nil (r ow1 z m iy1 d))
("rosemeyer" nil (r ow1 z m ay0 er0))
("rosemond" nil (r ow1 z m ax n d))
("rosemont" nil (r ow1 z m aa1 n t))
("rosen" nil (r ow1 z ax n))
("rosenau" nil (r ow1 z n aw0))
("rosenbach" nil (r ow1 z ax n b aa1 k))
("rosenbalm" nil (r ow1 z ax n b aa1 m))
("rosenbaum" nil (r ow1 z ax n b aw1 m))
("rosenbeck" nil (r ow1 z ax n b eh1 k))
("rosenberg" nil (r ow1 z ax n b er0 g))
("rosenberger" nil (r ow1 z ax n b er0 g er0))
("rosenberry" nil (r ow1 z ax n b eh1 r iy0))
("rosenblatt" nil (r ow1 z ax n b l ae1 t))
("rosenbloom" nil (r ow1 z ax n b l uw1 m))
("rosenblum" nil (r ow1 z ax n b l uw1 m))
("rosenbluth" nil (r ow1 z ax n b l uw0 th))
("rosenboom" nil (r ow1 z ax n b uw1 m))
("rosenburg" nil (r ow1 z ax n b er0 g))
("rosencrans" nil (r ow1 z ax n k r ax n z))
("rosendahl" nil (r ow1 z ax n d aa1 l))
("rosendale" nil (r ow1 z ax n d ey1 l))
("rosene" nil (r aa1 s iy0 n))
("rosener" nil (r ow1 z ax n er0))
("rosenfeld" nil (r ow1 z ax n f eh1 l d))
("rosenfield" nil (r ow1 z ax n f iy1 l d))
("rosengarten" nil (r ow1 z ax n g aa1 r t ax n))
("rosengrant" nil (r ow1 z ax n g r ae1 n t))
("rosengren" nil (r ow1 z ax n g r eh0 n))
("rosenhaus" nil (r ow1 z ax n hh aw1 s))
("rosenkrans" nil (r ow1 z ax n k r ae1 n z))
("rosenkrantz" nil (r ow1 z ax n k r ae1 n t s))
("rosenkranz" nil (r ow1 z ax n k r ae1 n t s))
("rosenlund" nil (r ow1 z ax n l ax n d))
("rosenman" nil (r ow1 z ax n m ax n))
("rosenow" nil (r ow1 z ax n aw0))
("rosenquist" nil (r ow1 z ax n k w ih0 s t))
("rosenshine" nil (r ow1 z ax n sh ay1 n))
("rosensteel" nil (r ow1 z ax n s t iy1 l))
("rosenstein" nil (r ow1 z ax n s t ay1 n))
("rosenstock" nil (r ow1 z ax n s t aa1 k))
("rosensweig" nil (r ow1 z ax n s w ay0 g))
("rosenthal" nil (r ow1 z ax n th ao1 l))
("rosentreter" nil (r ow1 z ax n t r eh1 t er0))
("rosenwald" nil (r ow1 z ax n w ao1 l d))
("rosenwasser" nil (r ow1 z ax n w ao0 s er0))
("rosenzweig" nil (r ow1 z ax n z w ay0 g))
("roser" nil (r ow1 z er0))
("roses" nil (r ow1 z ih0 z))
("rosete" nil (r aa1 s iy0 t))
("rosett" nil (r ow1 z ax t))
("rosetta" nil (r ow0 z eh1 t ax))
("rosette" nil (r ax s eh1 t))
("rosettes" nil (r ow1 z eh1 t s))
("rosetti" nil (r ow0 s eh1 t iy0))
("rosevear" nil (r ow0 z v ih1 r))
("roseville" nil (r ow1 z v ih1 l))
("rosewicz" nil (r ow1 z w ih1 t s))
("rosewood" nil (r ow1 z w uh1 d))
("rosh" nil (r ao1 sh))
("roshier" nil (r ow1 sh y er0))
("rosiak" nil (r aa1 s iy0 ae0 k))
("rosie" nil (r ow1 z iy0))
("rosier" nil (r ow1 z iy0 er0))
("rosin" nil (r aa1 z ax n))
("rosina" nil (r ow0 z iy1 n ax))
("rosine" nil (r ow0 s iy1 n iy0))
("rosing" nil (r ow1 z ih0 ng))
("rosinski" nil (r ax s ih1 n s k iy0))
("rosinsky" nil (r ax s ih1 n s k iy0))
("rosita" nil (r ow0 z iy1 t ax))
("roskam" nil (r aa1 s k ax m))
("roske" nil (r ow1 s k))
("rosko" nil (r ow1 s k ow0))
("roskos" nil (r ow1 s k ow0 z))
("roslin" nil (r aa1 s l ih0 n))
("roslund" nil (r ao1 z l ax n d))
("roslyn" nil (r aa1 z l ih0 n))
("rosman" nil (r aa1 s m ax n))
("rosmunda" nil (r ow0 s m uw1 n d ax))
("rosneft" nil (r aa1 z n ih0 f t))
("rosner" nil (r aa1 s n er0))
("rosoff" nil (r aa1 s ao0 f))
("rosol" nil (r ow1 s ao0 l))
("rospatch" nil (r ao1 s p ae0 ch))
("ross" nil (r aa1 s))
("rossa" nil (r ow1 s ax))
("rossano" nil (r ow0 s aa1 n ow0))
("rossbach" nil (r aa1 s b aa0 k))
("rossborough" nil (r ao1 s b er0 ow0))
("rosse" nil (r aa1 s))
("rosselin" nil (r aa1 s ih0 l ih0 n))
("rossell" nil (r aa1 s ax l))
("rosselli" nil (r ow0 s eh1 l iy0))
("rosselot" nil (r aa1 s ih0 l ax t))
("rossen" nil (r ao1 s ax n))
("rosser" nil (r ao1 s er0))
("rossetti" nil (r ow0 s eh1 t iy0))
("rossetto" nil (r ow0 s eh1 t ow0))
("rossi" nil (r ao1 s iy0))
("rossie" nil (r ao1 s iy0))
("rossignol" nil (r aa1 s ih0 g n ao0 l))
("rossin" nil (r aa1 s ih0 n))
("rossing" nil (r ao1 s ih0 ng))
("rossini" nil (r ow0 s iy1 n iy0))
("rossiter" nil (r ao1 s ih0 t er0))
("rossiya" nil (r ao1 s iy0 ax))
("rossler" nil (r aa1 s ax l er0))
("rosslyn" nil (r aa1 s l ih0 n))
("rossman" nil (r ao1 s m ax n))
("rossmann" nil (r ao1 s m ax n))
("rossmiller" nil (r aa1 s m ax l er0))
("rossmore" nil (r aa1 s m ao1 r))
("rossner" nil (r aa1 s n er0))
("rosso" nil (r ow1 s ow0))
("rosson" nil (r aa1 s ax n))
("rossow" nil (r aa1 s ow0))
("rost" nil (r aa1 s t))
("rostad" nil (r aa1 s t ax d))
("rosten" nil (r aa1 s ax n))
("rostenkowski" nil (r ao1 s t eh0 ng k aw1 s k iy0))
("roster" nil (r aa1 s t er0))
("rosters" nil (r aa1 s t er0 z))
("roston" nil (r aa1 s t ax n))
("rostron" nil (r aa1 s t r ax n))
("rostropovich" nil (r ao1 s t r ax p ow1 v ih0 ch))
("rostrum" nil (r aa1 s t r ax m))
("rostuca" nil (r ao1 s t uw1 k ax))
("rosty" nil (r ao1 s t iy0))
("roswald" nil (r aa1 s w ax l d))
("roswell" nil (r aa1 s w eh0 l))
("rosy" nil (r ow1 z iy0))
("roszak" nil (r aa1 sh ax k))
("roszell" nil (r aa1 sh ax l))
("roszkowski" nil (r ax sh k ao1 f s k iy0))
("rot" nil (r aa1 t))
("rota" nil (r ow1 t ax))
("rotan" nil (r ow1 t ax n))
("rotary" nil (r ow1 t er0 iy0))
("rotate" nil (r ow1 t ey1 t))
("rotated" nil (r ow1 t ey1 t ih0 d))
("rotates" nil (r ow1 t ey1 t s))
("rotating" nil (r ow1 t ey1 t ih0 ng))
("rotation" nil (r ow0 t ey1 sh ax n))
("rotational" nil (r ow0 t ey1 sh ax n ax l))
("rotberg" nil (r aa1 t b er0 g))
("rote" nil (r ow1 t))
("rotella" nil (r ow0 t eh1 l ax))
("roten" nil (r ow1 t ax n))
("rotenberg" nil (r ow1 t ax n b er0 g))
("rotenberry" nil (r ow1 t ax n b eh1 r iy0))
("rotert" nil (r aa1 t er0 t))
("roth" nil (r ao1 th))
("rothacker" nil (r ao1 th ax k er0))
("rothbard" nil (r ao1 th b er0 d))
("rothbart" nil (r ao1 th b aa1 r t))
("rothbauer" nil (r ao1 th b aw0 er0))
("rothberg" nil (r ao1 th b er0 g))
("rothchild" nil (r ao1 th ch ay1 l d))
("rothe" nil (r ow1 dh))
("rothenberg" nil (r ao1 th ax n b er0 g))
("rothenberger" nil (r ao1 th ax n b er0 g er0))
("rother" nil (r ao1 th er0))
("rothermel" nil (r aa1 th er0 m ax l))
("rothert" nil (r aa1 th er0 t))
("rothery" nil (r aa1 th er0 iy0))
("rothfuss" nil (r ao1 th f ax s))
("rothgeb" nil (r ao1 th g ih0 b))
("rothkopf" nil (r ao1 th k ao0 p f))
("rothlisberger" nil (r ao1 th l ih0 s b er0 g er0))
("rothman" nil (r ao1 th m ax n))
("rothmans" nil (r ao1 th m ax n z))
("rothmeier" nil (r ao1 th m ay1 r))
("rothrock" nil (r ao1 th r ax k))
("roths" nil (r ao1 th s))
("rothschild" nil (r ao1 th s ch ay1 l d))
("rothschilds" nil (r ao1 th ch ay1 l d z))
("rothstein" nil (r ao1 th s t ay0 n))
("rothweiler" nil (r ao1 th w ay1 l er0))
("rothwell" nil (r ao1 th w eh1 l))
("rothwells" nil (r ao1 th w eh1 l z))
("roti" nil (r ow1 t iy0))
("rotisserie" nil (r ow1 t ih1 s er0 iy0))
("rotman" nil (r aa1 t m ax n))
("rotolo" nil (r ow0 t ow1 l ow0))
("rotondi" nil (r ow0 t ow1 n d iy0))
("rotondo" nil (r ow0 t ow1 n d ow0))
("rotor" nil (r ow1 t er0))
("rotors" nil (r ow1 t er0 z))
("rototill" nil (r ow1 t ax t ih1 l))
("rototilled" nil (r ow1 t ax t ih1 l d))
("rototiller" nil (r ow1 t ax t ih1 l er0))
("rototilles" nil (r ow1 t ax t ih1 l z))
("rototilling" nil (r ow1 t ax t ih1 l ih0 ng))
("rotramel" nil (r aa1 t r ax m ax l))
("rotruck" nil (r aa1 t r ax k))
("rots" nil (r aa1 t s))
("rott" nil (r aa1 t))
("rotted" nil (r aa1 t ih0 d))
("rotten" nil (r aa1 t ax n))
("rottenberg" nil (r aa1 t ax n b er0 g))
("rotter" nil (r aa1 t er0))
("rotterdam" nil (r aa1 t er0 d ae1 m))
("rotting" nil (r aa1 t ih0 ng))
("rottinghaus" nil (r aa1 t ih0 ng hh aw1 s))
("rottler" nil (r aa1 t l er0))
("rottman" nil (r aa1 t m ax n))
("rottmann" nil (r aa1 t m ax n))
("rotund" nil (r ow0 t ah1 n d))
("rotunda" nil (r ow0 t ah1 n d ax))
("rotundo" nil (r ow0 t uw1 n d ow0))
("rotunno" nil (r ow0 t uw1 n ow0))
("rotz" nil (r aa1 t s))
("roubal" nil (r uw0 b ae1 l))
("rouble" nil (r uw1 b ax l))
("rouch" nil (r aw1 ch))
("roudabush" nil (r aw1 d ax b ax sh))
("roudebush" nil (r aw1 d ih0 b ax sh))
("rouge" nil (r uw1 zh))
("rougeau" nil (r uw0 zh ow1))
("rough" nil (r ah1 f))
("roughed" nil (r ah1 f t))
("rougher" nil (r ah1 f er0))
("roughest" nil (r ah1 f ax s t))
("roughing" nil (r ah1 f ih0 ng))
("roughly" nil (r ah1 f l iy0))
("roughnecks" nil (r ah1 f n eh1 k s))
("roughshod" nil (r ah1 f sh aa1 d))
("rought" nil (r ao1 t))
("roughton" nil (r ah1 f t ax n))
("rouillard" nil (r w iy0 l aa1 r d))
("roukema" nil (r uw1 k m aa0))
("roukos" nil (r uw1 k ow0 s))
("roulac" nil (r uw1 l ae0 k))
("rouleau" nil (r uw0 l ow1))
("roulette" nil (r uw0 l eh1 t))
("roulettes" nil (r uw0 l eh1 t s))
("roulhac" nil (r uw1 l hh ax k))
("roulston" nil (r aw1 l s t ax n))
("round" nil (r aw1 n d))
("roundabout" nil (r aw1 n d ax b aw1 t))
("rounded" nil (r aw1 n d ax d))
("rounder" nil (r aw1 n d er0))
("roundhead" nil (r aw1 n d hh eh1 d))
("roundhouse" nil (r aw1 n d hh aw1 s))
("rounding" nil (r aw1 n d ih0 ng))
("roundly" nil (r aw1 n d l iy0))
("rounds" nil (r aw1 n d z))
("roundtable" nil (r aw1 n d t ey1 b ax l))
("roundtree" nil (r aw1 n d t r iy1))
("roundtrip" nil (r aw1 n d t r ih1 p))
("roundup" nil (r aw1 n d ah1 p))
("roundworms" nil (r aw1 n d w er0 m z))
("roundy" nil (r aw1 n d iy0))
("rounsaville" nil (r uw1 n s ax v ih0 l))
("rountree" nil (r aw0 n t r iy1))
("roupe" nil (r uw1 p))
("rourk" nil (r ao1 r k))
("rourke" nil (r ao1 r k))
("rouse" nil (r aw1 s))
("roused" nil (r aw1 z d))
("rouser" nil (r aw1 z er0))
("rousey" nil (r aw1 s iy0))
("roush" nil (r aw1 sh))
("rousing" nil (r aw1 z ih0 ng))
("rousse" nil (r aw1 s))
("rousseau" nil (r uw0 s ow1))
("roussel" nil (r uw0 s eh1 l))
("rousselet" nil (r uw1 s ax l eh0 t))
("roussell" nil (r uw0 s eh1 l))
("rousselle" nil (r uw1 s eh1 l))
("roussin" nil (r uw0 s ae1 n))
("roussos" nil (r uw0 s ow1 z))
("rout" nil (r aw1 t))
("route" nil (r uw1 t))
("routed" nil (r uw1 t ax d))
("routers" nil (r uw1 t er0 z))
("routes" nil (r uw1 t s))
("routh" nil (r aw1 th))
("routhier" nil (r aw1 th iy0 er0))
("routine" nil (r uw0 t iy1 n))
("routinely" nil (r uw0 t iy1 n l iy0))
("routines" nil (r uw0 t iy1 n z))
("routing" nil (r aw1 t ih0 ng))
("routledge" nil (r aw1 t l ih0 jh))
("routon" nil (r aw1 t ax n))
("routson" nil (r aw1 t s ax n))
("routt" nil (r aw1 t))
("routzahn" nil (r aw1 t z ax n))
("roux" nil (r uw1))
("rouyn" nil (r uw1 ih0 n))
("rouzer" nil (r aw1 z er0))
("rovaniemi" nil (r ow0 v ae1 n iy0 eh1 m iy0))
("rove" nil (r ow1 v))
("roven" nil (r ow1 v ax n))
("rover" nil (r ow1 v er0))
("rovers" nil (r ow1 v er0 z))
("roving" nil (r ow1 v ih0 ng))
("rovira" nil (r ow0 v ih1 r ax))
("rovito" nil (r ow0 v iy1 t ow0))
("rovner" nil (r aa1 v n er0))
("row" nil (r ow1))
("rowan" nil (r ow1 ax n))
("rowand" nil (r ow1 ax n d))
("rowbotham" nil (r ow1 b ax th ae0 m))
("rowden" nil (r ow1 d ax n))
("rowdies" nil (r aw1 d iy0 z))
("rowdiness" nil (r aw1 d iy0 n ax s))
("rowdy" nil (r aw1 d iy0))
("rowe" nil (r ow1))
("rowed" nil (r ow1 d))
("rowell" nil (r ow1 ih0 l))
("rowen" nil (r ow1 ax n))
("rowena" nil (r ow0 iy1 n ax))
("rowenta" nil (r ow1 eh1 n t ax))
("rower" nil (r ow1 er0))
("rowes" nil (r ow1 z))
("rowin" nil (r ow1 ih0 n))
("rowing" nil (r ow1 ih0 ng))
("rowinski" nil (r ow0 ih1 n s k iy0))
("rowland" nil (r ow1 l ax n d))
("rowlands" nil (r ow1 l ax n d z))
("rowles" nil (r ow1 l z))
("rowlett" nil (r ow1 l ih0 t))
("rowlette" nil (r ow1 l eh1 t))
("rowley" nil (r ow1 l iy0))
("rowly" nil (r ow1 l iy0))
("rowney" nil (r ow1 n iy0))
("rowntree" nil (r ow1 n t r iy1))
("rowny" nil (r ow1 n iy0))
("rows" nil (r ow1 z))
("rowse" nil (r ow1 s))
("rowser" nil (r ow1 z er0))
("rowsey" nil (r ow1 s iy0))
("rowson" nil (r ow1 s ax n))
("rowton" nil (r ow1 t ax n))
("rox" nil (r aa1 k s))
("roxana" nil (r aa0 k s ae1 n ax))
("roxani" nil (r aa0 k s aa1 n iy0))
("roxanna" nil (r aa1 k s ae1 n ax))
("roxanne" nil (r aa1 k s ih0 n))
("roxbury" nil (r aa1 k s b eh1 r iy0))
("roxie" nil (r aa1 k s iy0))
("roxine" nil (r aa1 k s ay0 n))
("roxy" nil (r aa1 k s iy0))
("roy" nil (r oy1))
("royal" nil (r oy1 ax l))
("royale" nil (r oy0 ae1 l))
("royalist" nil (r oy1 ax l ih0 s t))
("royall" nil (r oy1 ax l))
("royals" nil (r oy1 ax l z))
("royalties" nil (r oy1 ax l t iy0 z))
("royalty" nil (r oy1 ax l t iy0))
("roybal" nil (r oy1 b ax l))
("royce" nil (r oy1 s))
("royces" nil (r oy1 s ih0 z))
("roycroft" nil (r oy1 k r ax f t))
("royd" nil (r oy1 d))
("royden" nil (r oy1 d ax n))
("roydon" nil (r oy1 d ax n))
("roye" nil (r oy1))
("royer" nil (r oy1 er0))
("royex" nil (r oy1 eh0 k s))
("royko" nil (r oy1 k ow0))
("roylance" nil (r oy1 l ax n s))
("royle" nil (r oy1 l))
("roys" nil (r oy1 z))
("royse" nil (r oy1 s))
("royster" nil (r oy1 s t er0))
("royston" nil (r oy1 s t ax n))
("roza" nil (r ow1 z ax))
("rozak" nil (r ow1 z ax k))
("rozanski" nil (r ax z ae1 n s k iy0))
("rozar" nil (r ow1 z er0))
("rozas" nil (r ow1 z aa0 z))
("rozeboom" nil (r aa1 z ih0 b uw1 m))
("rozek" nil (r ow1 z eh0 k))
("rozell" nil (r aa1 z ax l))
("rozella" nil (r ow0 z eh1 l ax))
("rozelle" nil (r ax z eh1 l))
("rozema" nil (r ax z iy1 m ax))
("rozen" nil (r aa1 z ax n))
("rozier" nil (r ow1 z iy0 er0))
("rozman" nil (r aa1 z m ax n))
("roznowski" nil (r ax z n ao1 f s k iy0))
("rozsa" nil (r ow1 z s ax))
("rozycki" nil (r ax z ih1 t s k iy0))
("rozzell" nil (r aa1 z ax l))
("rozzi" nil (r aa1 z iy0))
("rpm" nil (aa1 r p iy1 eh1 m))
("rua" nil (r uw1 ax))
("ruam" nil (r uw1 ax m))
("ruane" nil (r uw0 ey1 n))
("ruano" nil (r uw0 aa1 n ow0))
("ruark" nil (r uw1 aa0 r k))
("rub" nil (r ah1 b))
("ruback" nil (r uw1 b ae1 k))
("rubalcaba" nil (r uw0 b aa0 l k aa1 b ax))
("rubalcava" nil (r uw0 b aa0 l k aa1 v ax))
("rubano" nil (r uw0 b aa1 n ow0))
("rubbed" nil (r ah1 b d))
("rubber" nil (r ah1 b er0))
("rubberized" nil (r ah1 b er0 ay1 z d))
("rubbermaid" nil (r ah1 b er0 m ey1 d))
("rubbers" nil (r ah1 b er0 z))
("rubbery" nil (r ah1 b er0 iy0))
("rubbia" nil (r ah1 b iy0 ax))
("rubbing" nil (r ah1 b ih0 ng))
("rubbish" nil (r ah1 b ih0 sh))
("rubble" nil (r ah1 b ax l))
("rubbo" nil (r uw1 b ow0))
("rube" nil (r uw1 b))
("rubeck" nil (r uw1 b eh1 k))
("rubel" nil (r uw1 b ax l))
("rubell" nil (r uw1 b ax l))
("rubella" nil (r uw0 b eh1 l ax))
("ruben" nil (r uw1 b ax n))
("rubendall" nil (r ah1 b ih0 n d ax l))
("rubenfeld" nil (r ah1 b ih0 n f eh0 l d))
("rubens" nil (r uw1 b ax n z))
("rubenstein" nil (r uw1 b ax n s t ay0 n))
("ruberg" nil (r uw1 b er0 g))
("rubert" nil (r uw1 b er0 t))
("ruberto" nil (r uw0 b eh1 r t ow0))
("rubey" nil (r uw1 b iy0))
("rubi" nil (r uw1 b iy0))
("rubia" nil (r uw1 b iy0 ax))
("rubicam" nil (r uw1 b ih0 k ax m))
("rubie" nil (r uw1 b iy0))
("rubies" nil (r uw1 b iy0 z))
("rubin" nil (r uw1 b ih0 n))
("rubina" nil (r uw0 b iy1 n ax))
("rubino" nil (r uw0 b iy1 n ow0))
("rubinson" nil (r ah1 b ih0 n s ax n))
("rubinstein" nil (r uw1 b ih0 n s t iy1 n))
("rubio" nil (r uw1 b iy0 ow0))
("rubis" nil (r uw1 b ih0 s))
("ruble" nil (r uw1 b ax l))
("rubles" nil (r uw1 b ax l z))
("rubley" nil (r ah1 b l iy0))
("rubloff" nil (r ah1 b l ao0 f))
("rubottom" nil (r ah1 b ax t aa0 m))
("rubout" nil (r ah1 b aw1 t))
("rubric" nil (r uw1 b r ih0 k))
("rubright" nil (r ah1 b r ay1 t))
("rubs" nil (r ah1 b z))
("ruby" nil (r uw1 b iy0))
("rucci" nil (r uw1 ch iy0))
("ruch" nil (r ah1 ch))
("ruchlamer" nil (r uw1 k l ax m er0))
("ruchti" nil (r uw1 k t iy0))
("rucinski" nil (r ax ch ih1 n s k iy0))
("ruck" nil (r ah1 k))
("ruckel" nil (r ah1 k ax l))
("ruckelshaus" nil (r ah1 k ax l z hh aw1 s))
("rucker" nil (r ah1 k er0))
("ruckert" nil (r ah1 k er0 t))
("rucki" nil (r ah1 k iy0))
("ruckle" nil (r ah1 k ax l))
("ruckman" nil (r ah1 k m ax n))
("rucks" nil (r ah1 k s))
("ruckus" nil (r ah1 k ax s))
("rud" nil (r ah1 d))
("ruda" nil (r uw1 d ax))
("rudani" nil (r uw0 d aa1 n iy0))
("rudasill" nil (r ah1 d ax s ih0 l))
("rudd" nil (r ah1 d))
("ruddell" nil (r ah1 d ax l))
("rudden" nil (r ah1 d ax n))
("rudder" nil (r ah1 d er0))
("rudderless" nil (r ah1 d er0 l ax s))
("ruddick" nil (r ah1 d ih0 k))
("ruddle" nil (r ah1 d ax l))
("ruddock" nil (r ah1 d ax k))
("ruddy" nil (r ah1 d iy0))
("rude" nil (r uw1 d))
("rudeen" nil (r ah1 d iy0 n))
("rudel" nil (r uw1 d ax l))
("rudell" nil (r ah1 d ax l))
("rudelle" nil (r ax d eh1 l))
("rudely" nil (r uw1 d l iy0))
("ruden" nil (r uw1 d ax n))
("rudeness" nil (r uw1 d n ax s))
("rudenstine" nil (r uw1 d ax n z t ay1 n))
("ruder" nil (r uw1 d er0))
("ruderman" nil (r uw1 d er0 m ax n))
("rudesill" nil (r ah1 d ih0 s ax l))
("rudest" nil (r uw1 d ax s t))
("rudge" nil (r ah1 jh))
("rudi" nil (r uw1 d iy0))
("rudich" nil (r ah1 d ih0 k))
("rudie" nil (r uw1 d iy0))
("rudiger" nil (r ah1 d ih0 g er0))
("rudimentary" nil (r uw1 d ax m eh1 n t er0 iy0))
("rudiments" nil (r uw1 d ih0 m ax n t s))
("rudin" nil (r uw1 d ih0 n))
("ruding" nil (r uw1 d ih0 ng))
("rudis" nil (r uw1 d iy0 z))
("rudisill" nil (r ah1 d ih0 s ih0 l))
("rudkin" nil (r ah1 d k ih0 n))
("rudloff" nil (r ah1 d l ao0 f))
("rudman" nil (r ah1 d m ax n))
("rudner" nil (r ah1 d n er0))
("rudnet" nil (r ah1 d n eh1 t))
("rudnick" nil (r ah1 d n ih0 k))
("rudnicki" nil (r ax d n ih1 k iy0))
("rudnicky" nil (r ax d n ih1 k iy0))
("rudnik" nil (r ah1 d n ih0 k))
("rudolf" nil (r uw1 d aa1 l f))
("rudolph" nil (r uw1 d ao0 l f))
("rudow" nil (r uw1 d ow0))
("rudy" nil (r uw1 d iy0))
("rudyard" nil (r ah1 d y er0 d))
("rudzinski" nil (r ax jh ih1 n s k iy0))
("rue" nil (r uw1))
("rueb" nil (r uw1 b))
("ruebel" nil (r uw1 b ax l))
("rueckert" nil (r uw1 k er0 t))
("rueda" nil (r uw0 eh1 d ax))
("ruediger" nil (r uw1 d ih0 g er0))
("ruedlinger" nil (r uw1 d l ih0 ng er0))
("ruedy" nil (r uw1 d iy0))
("ruef" nil (r uw1 f))
("rueff" nil (r uw1 f))
("rueful" nil (r uw1 f ax l))
("ruefully" nil (r uw1 f ax l iy0))
("rueger" nil (r uh1 g er0))
("ruegg" nil (r uw1 g))
("ruegsegger" nil (r uh1 g s ih0 g er0))
("ruehl" nil (r uh1 l))
("ruehle" nil (r uw1 hh ax l))
("ruel" nil (r uw1 l))
("ruelas" nil (r uw1 l ax z))
("ruella" nil (r uw1 eh1 l ax))
("ruelle" nil (r uw1 eh1 l))
("ruesch" nil (r uw1 sh))
("ruess" nil (r uw1 s))
("ruest" nil (r uw1 s t))
("rueter" nil (r uw1 t er0))
("rueth" nil (r uw1 th))
("ruether" nil (r uw1 th er0))
("ruetz" nil (r uw1 t s))
("ruf" nil (r ah1 f))
("rufe" nil (r uw1 f))
("rufenacht" nil (r uw1 f ax n aa0 k t))
("rufener" nil (r ah1 f iy0 n er0))
("rufer" nil (r uw1 f er0))
("ruff" nil (r ah1 f))
("ruffalo" nil (r uw0 f aa1 l ow0))
("ruffcorn" nil (r ah1 f k er0 n))
("ruffer" nil (r ah1 f er0))
("ruffin" nil (r ah1 f ih0 n))
("ruffing" nil (r ah1 f ih0 ng))
("ruffini" nil (r uw0 f iy1 n iy0))
("ruffino" nil (r uw0 f iy1 n ow0))
("ruffle" nil (r ah1 f ax l))
("ruffled" nil (r ah1 f ax l d))
("ruffles" nil (r ah1 f ax l z))
("ruffling" nil (r ah1 f ax l ih0 ng))
("ruffner" nil (r ah1 f n er0))
("ruffo" nil (r ah1 f ow0))
("ruffolo" nil (r uw0 f ow1 l ow0))
("rufford" nil (r ah1 f er0 d))
("rufina" nil (r uw0 f iy1 n ax))
("rufo" nil (r uw1 f ow0))
("ruford" nil (r uw1 f er0 d))
("rufty" nil (r ah1 f t iy0))
("rufus" nil (r uw1 f ax s))
("rug" nil (r ah1 g))
("rugby" nil (r ah1 g b iy0))
("ruge" nil (r uw1 jh))
("rugen" nil (r ah1 g ax n))
("ruger" nil (r uw1 g er0))
("rugeroni" nil (r uw1 jh eh0 r ow1 n iy0))
("rugg" nil (r ah1 g))
("rugged" nil (r ah1 g ax d))
("ruggedly" nil (r ah1 g ax d l iy0))
("ruggeri" nil (r uw0 jh eh1 r iy0))
("ruggerio" nil (r uw0 jh eh1 r iy0 ow0))
("ruggieri" nil (r uw0 jh ih1 r iy0))
("ruggiero" nil (r uw0 jh ih1 r ow0))
("ruggirello" nil (r uw0 jh ih0 r eh1 l ow0))
("ruggles" nil (r ah1 g ax l z))
("rugh" nil (r ah1))
("rugs" nil (r ah1 g z))
("ruh" nil (r ah1))
("ruhe" nil (r uw1 hh))
("ruhl" nil (r ah1 l))
("ruhland" nil (r ah1 l ax n d))
("ruhle" nil (r uw1 ax l))
("ruhlman" nil (r uw1 l m ax n))
("ruhnau" nil (r uw1 n aw0))
("ruhnke" nil (r ah1 ng k))
("ruhollah" nil (r uw0 hh aa1 l ax))
("ruhr" nil (r uh1 r))
("ruhrgas" nil (r uh1 r g ax s))
("ruhul" nil (r uw1 hh uw1 l))
("ruin" nil (r uw1 ax n))
("ruined" nil (r uw1 ax n d))
("ruining" nil (r uw1 ih0 n ih0 ng))
("ruinous" nil (r uw1 ax n ax s))
("ruins" nil (r uw1 ax n z))
("ruis" nil (r uw1 ih0 z))
("ruiter" nil (r uw1 t er0))
("ruiz" nil (r uw0 w iy1 z))
("rujith" nil (r uw1 jh iy1 th))
("rukavina" nil (r ax k ax v ay1 n ax))
("rukeyser" nil (r uw1 k ey1 z er0))
("rula" nil (r uw1 l ax))
("ruland" nil (r uw1 l ax n d))
("rule" nil (r uw1 l))
("rulebook" nil (r uw1 l b uh1 k))
("ruled" nil (r uw1 l d))
("rulemaking" nil (r uw1 l m ey1 k ih0 ng))
("ruler" nil (r uw1 l er0))
("rulers" nil (r uw1 l er0 z))
("rules" nil (r uw1 l z))
("ruley" nil (r uw1 l iy0))
("ruling" nil (r uw1 l ih0 ng))
("rulings" nil (r uw1 l ih0 ng z))
("rulison" nil (r ah1 l ih0 s ax n))
("rulli" nil (r uw1 l iy0))
("rullman" nil (r ah1 l m ax n))
("rullo" nil (r uw1 l ow0))
("rulon" nil (r uw1 l ax n))
("rum" nil (r ah1 m))
("rumack" nil (r uw1 m ae0 k))
("rumage" nil (r ah1 m ih0 jh))
("ruman" nil (r uw1 m ax n))
("rumania" nil (r uw0 m ey1 n iy0 ax))
("rumbaugh" nil (r ah1 m b ao1))
("rumberger" nil (r ah1 m b er0 g er0))
("rumble" nil (r ah1 m b ax l))
("rumbled" nil (r ah1 m b ax l d))
("rumbles" nil (r ah1 m b ax l z))
("rumbley" nil (r ah1 m b l iy0))
("rumbling" nil (r ah1 m b ax l ih0 ng))
("rumblings" nil (r ah1 m b ax l ih0 ng z))
("rumbold" nil (r ah1 m b ow1 l d))
("rumen" nil (r uw1 m ax n))
("rumer" nil (r uw1 m er0))
("rumery" nil (r ah1 m er0 iy0))
("rumfelt" nil (r ah1 m f eh1 l t))
("rumford" nil (r ah1 m f er0 d))
("rumina" nil (r uw1 m ih0 n ax))
("ruminant" nil (r uw1 m ax n ax n t))
("ruminated" nil (r uw1 m ax n ey1 t ih0 d))
("rumination" nil (r uw1 m ih0 n ey1 sh ax n))
("ruminations" nil (r uw1 m ih0 n ey1 sh ax n z))
("ruminski" nil (r ax m ih1 n s k iy0))
("rumler" nil (r ah1 m l er0))
("rumley" nil (r ah1 m l iy0))
("rummage" nil (r ah1 m ih0 jh))
("rummaging" nil (r ah1 m ih0 jh ih0 ng))
("rummel" nil (r ah1 m ax l))
("rummell" nil (r ah1 m ax l))
("rummy" nil (r ah1 m iy0))
("rumney" nil (r ah1 m n iy0))
("rumor" nil (r uw1 m er0))
("rumore" nil (r uw0 m ao1 r))
("rumored" nil (r uw1 m er0 d))
("rumors" nil (r uw1 m er0 z))
("rumours" nil (r uw1 m er0 z))
("rump" nil (r ah1 m p))
("rumpel" nil (r ah1 m p ax l))
("rumpf" nil (r ah1 m p f))
("rumph" nil (r ah1 m f))
("rumple" nil (r ah1 m p ax l))
("rumpled" nil (r ah1 m p ax l d))
("rumrill" nil (r ah1 m r ih0 l))
("rums" nil (r ah1 m z))
("rumsey" nil (r ah1 m z iy0))
("rumsfeld" nil (r ah1 m z f eh1 l d))
("rumson" nil (r ah1 m s ax n))
("run" nil (r ah1 n))
("runabout" nil (r ah1 n ax b aw1 t))
("runaway" nil (r ah1 n ax w ey1))
("runcie" nil (r ah1 n s iy0))
("runck" nil (r ah1 ng k))
("runckel" nil (r ah1 ng k ax l))
("runco" nil (r ah1 ng k ow0))
("rund" nil (r ah1 n d))
("rundall" nil (r ah1 n d ax l))
("runde" nil (r ah1 n d))
("rundell" nil (r ah1 n d ax l))
("rundle" nil (r ah1 n d ax l))
("rundown" nil (r ah1 n d aw1 n))
("rundquist" nil (r ah1 n d k w ih1 s t))
("rung" nil (r ah1 ng))
("runge" nil (r ah1 n jh))
("rungs" nil (r ah1 ng z))
("runion" nil (r uw1 n y ax n))
("runions" nil (r uw1 n y ax n z))
("runk" nil (r ah1 ng k))
("runkel" nil (r ah1 ng k ax l))
("runkle" nil (r ah1 ng k ax l))
("runkles" nil (r ah1 ng k ax l z))
("runnells" nil (r ah1 n ax l z))
("runnels" nil (r ah1 n ax l z))
("runner" nil (r ah1 n er0))
("runners" nil (r ah1 n er0 z))
("running" nil (r ah1 n ih0 ng))
("runnion" nil (r ah1 n y ax n))
("runoff" nil (r ah1 n ao1 f))
("runquist" nil (r ah1 n k w ih1 s t))
("runs" nil (r ah1 n z))
("runte" nil (r ah1 n t))
("runup" nil (r ah1 n ah1 p))
("runups" nil (r ah1 n ah1 p s))
("runway" nil (r ah1 n w ey1))
("runways" nil (r ah1 n w ey1 z))
("runyan" nil (r ah1 n y ax n))
("runyon" nil (r ah1 n y ax n))
("runzheimer" nil (r ah1 n z hh ay1 m er0))
("ruocco" nil (r uw0 ow1 k ow0))
("ruoff" nil (r uw1 ao0 f))
("ruotolo" nil (r uw0 ow0 t ow1 l ow0))
("rupard" nil (r uw1 p er0 d))
("rupe" nil (r uw1 p))
("rupee" nil (r uw0 p iy1))
("rupees" nil (r uw0 p iy1 z))
("rupert" nil (r uw1 p er0 t))
("ruperta" nil (r uw0 p eh1 r t ax))
("rupiah" nil (r uw0 p ay1 ax))
("ruple" nil (r uw1 p ax l))
("rupley" nil (r ah1 p l iy0))
("rupnow" nil (r ah1 p n ow0))
("rupp" nil (r ah1 p))
("ruppe" nil (r ah1 p))
("ruppel" nil (r ah1 p ax l))
("ruppenthal" nil (r ah1 p ih0 n th ax l))
("ruppert" nil (r ah1 p er0 t))
("rupprecht" nil (r uw1 p r eh1 k t))
("ruprecht" nil (r ah1 p r ih0 k t))
("rupture" nil (r ah1 p ch er0))
("ruptured" nil (r ah1 p ch er0 d))
("rupturing" nil (r ah1 p ch er0 ih0 ng))
("rural" nil (r uh1 r ax l))
("rurik" nil (r uh1 r ih0 k))
("ruritania" nil (r uh1 r ax t ey1 n iy0 ax))
("rus" nil (r ah1 s))
("rusalka" nil (r uw1 s ae1 l k ax))
("rusch" nil (r ah1 sh))
("rusche" nil (r ah1 sh))
("ruscitti" nil (r uw0 s ch iy1 t iy0))
("rusconi" nil (r uw0 s k ow1 n iy0))
("ruse" nil (r uw1 z))
("ruses" nil (r uw1 z ih0 z))
("rush" nil (r ah1 sh))
("rushdie" nil (r ah1 sh d iy0))
("rushed" nil (r ah1 sh t))
("rusher" nil (r ah1 sh er0))
("rushes" nil (r ah1 sh ax z))
("rushford" nil (r ah1 sh f er0 d))
("rushin" nil (r ah1 sh ih0 n))
("rushing" nil (r ah1 sh ih0 ng))
("rushlow" nil (r ah1 sh l ow1))
("rushmore" nil (r ah1 sh m ao0 r))
("rushton" nil (r ah1 sh t ax n))
("rusin" nil (r uw1 s ih0 n))
("rusinko" nil (r uw1 s ih1 ng k ow0))
("rusk" nil (r ah1 s k))
("ruskin" nil (r ah1 s k ih0 n))
("ruslan" nil (r ah1 s l ax n))
("rusnak" nil (r ah1 s n ax k))
("rusnock" nil (r ah1 s n ax k))
("russ" nil (r ah1 s))
("russa" nil (r uw1 s ax))
("russak" nil (r ah1 s ax k))
("russe" nil (r ah1 s))
("russel" nil (r ah1 s ax l))
("russell" nil (r ah1 s ax l))
("russellville" nil (r ah1 s ax l v ih1 l))
("russert" nil (r ah1 s er0 t))
("russett" nil (r ah1 s ih0 t))
("russey" nil (r ah1 s iy0))
("russi" nil (r uw1 s iy0))
("russia" nil (r ah1 sh ax))
("russian" nil (r ah1 sh ax n))
("russians" nil (r ah1 sh ax n z))
("russification" nil (r ah1 s ax f ax k ey1 sh ax n))
("russin" nil (r ah1 s ih0 n))
("russman" nil (r ah1 s m ax n))
("russo" nil (r uw1 s ow0))
("russolillo" nil (r uw1 s ow0 l ih1 l ow0))
("russom" nil (r ah1 s ax m))
("russomanno" nil (r ah1 s ax m ih0 n ow0))
("russow" nil (r ah1 s ow0))
("russum" nil (r ah1 s ax m))
("rust" nil (r ah1 s t))
("rustad" nil (r ah1 s t ax d))
("rusted" nil (r ah1 s t ax d))
("rustic" nil (r ah1 s t ih0 k))
("rustin" nil (r ah1 s t ih0 n))
("rusting" nil (r ah1 s t ih0 ng))
("rustler" nil (r ah1 s l er0))
("rustlers" nil (r ah1 s ax l er0 z))
("rustling" nil (r ah1 s l ih0 ng))
("ruston" nil (r ah1 s t ax n))
("rustproof" nil (r ah1 s t p r uw1 f))
("rustproofing" nil (r ah1 s t p r uw1 f ih0 ng))
("rusts" nil (r ah1 s t s))
("rusty" nil (r ah1 s t iy0))
("ruszkowski" nil (r ax sh k ao1 f s k iy0))
("rut" nil (r ah1 t))
("ruta" nil (r uw1 t ax))
("rutan" nil (r uw1 t ax n))
("ruter" nil (r uw1 t er0))
("rutgers" nil (r ah1 t g er0 z))
("ruth" nil (r uw1 th))
("ruthann" nil (r uw1 th ae1 n))
("ruthenberg" nil (r ah1 th ax n b er0 g))
("ruther" nil (r uw1 th er0))
("rutherford" nil (r ah1 th er0 f er0 d))
("ruthless" nil (r uw1 th l ax s))
("ruthlessly" nil (r uw1 th l ax s l iy0))
("ruthlessness" nil (r uw1 th l ax s n ax s))
("ruthven" nil (r ah1 th v ax n))
("rutigliano" nil (r uw0 t iy0 g l iy0 aa1 n ow0))
("rutile" nil (r uw1 t iy0 l))
("rutkoski" nil (r ax t k aw1 s k iy0))
("rutkowski" nil (r ax t k ao1 f s k iy0))
("rutland" nil (r ah1 t l ax n d))
("rutledge" nil (r ah1 t l ih0 jh))
("rutley" nil (r ah1 t l iy0))
("rutman" nil (r ah1 t m ax n))
("ruts" nil (r ah1 t s))
("rutskoi" nil (r uw1 t s k oy0))
("rutstein" nil (r ah1 t s t ay1 n))
("rutt" nil (r ah1 t))
("ruttan" nil (r ah1 t ax n))
("rutted" nil (r ah1 t ih0 d))
("rutten" nil (r ah1 t ax n))
("ruttenberg" nil (r ah1 t ax n b er0 g))
("rutter" nil (r ah1 t er0))
("rutz" nil (r ah1 t s))
("rutzen" nil (r ah1 t z ax n))
("ruud" nil (r uw1 d))
("ruvalcaba" nil (r uw0 v aa0 l k aa1 b ax))
("ruvolo" nil (r uw0 v ow1 l ow0))
("ruwe" nil (r uw1 w))
("rux" nil (r ah1 k s))
("ruxpin" nil (r ah1 k s p ih0 n))
("ruybal" nil (r ay1 b ax l))
("ruyle" nil (r ay1 l))
("ruzek" nil (r uw1 z eh0 k))
("ruzich" nil (r ah1 z ih0 hh))
("ruzicka" nil (r uw1 z ih1 k ax))
("ruzzo" nil (r uw1 z ow0))
("rwanda" nil (r ax w aa1 n d ax))
("rwandan" nil (r ax w aa1 n d ax n))
("rwandans" nil (r ax w aa1 n d ax n z))
("rwandese" nil (r ax w aa1 d iy1 z))
("ryal" nil (r iy0 aa1 l))
("ryall" nil (r iy0 aa1 l))
("ryals" nil (r iy0 aa1 l z))
("ryan" nil (r ay1 ax n))
("ryanair" nil (r ay1 ax n eh1 r))
("ryans" nil (r ay1 ax n z))
("ryba" nil (r ih1 b ax))
("ryback" nil (r ay1 b ae1 k))
("rybacki" nil (r ih0 b aa1 t s k iy0))
("rybak" nil (r ih1 b ax k))
("rybakov" nil (r ay1 b ax k aa1 v))
("rybarczyk" nil (r ih1 b er0 ch ih0 k))
("ryberg" nil (r ay1 b er0 g))
("rybicki" nil (r ih0 b ih1 t s k iy0))
("rybinski" nil (r ih0 b ih1 n s k iy0))
("rybka" nil (r ih1 b k ax))
("rybolt" nil (r ih1 b ow0 l t))
("ryburn" nil (r ih1 b er0 n))
("rychlik" nil (r ih1 ch l ih0 k))
("ryckman" nil (r ih1 k m ax n))
("rycroft" nil (r ih1 k r ax f t))
("ryczek" nil (r ih1 ch eh0 k))
("rydberg" nil (r ih1 d b er0 g))
("rydell" nil (r ih1 d ax l))
("ryden" nil (r ay1 d ax n))
("ryder" nil (r ay1 d er0))
("rydman" nil (r ih1 d m ax n))
("rydzewski" nil (r ih0 jh eh1 f s k iy0))
("rye" nil (r ay1))
("ryen" nil (r ay1 ax n))
("ryer" nil (r ay1 er0))
("ryerson" nil (r ay1 r s ax n))
("rygiel" nil (r ih1 g iy0 l))
("ryka" nil (r ay1 k ax))
("ryker" nil (r ay1 k er0))
("rykoff" nil (r ay1 k ao1 f))
("rykowski" nil (r ih0 k ao1 f s k iy0))
("rylan" nil (r ay1 l ax n))
("ryland" nil (r ih1 l ax n d))
("rylander" nil (r ih1 l ax n d er0))
("ryle" nil (r ay1 l))
("rylee" nil (r ih1 l iy0))
("ryles" nil (r ay1 l z))
("ryley" nil (r ay1 l iy0))
("ryman" nil (r ay1 m ax n))
("rymer" nil (r ay1 m er0))
("rynders" nil (r ih1 n d er0 z))
("rynearson" nil (r ih1 n er0 s ax n))
("ryner" nil (r ay1 n er0))
("rynes" nil (r ay1 n z))
("ryno" nil (r ih1 n ow0))
("ryobi" nil (r ay1 ow1 b iy0))
("ryohei" nil (r ay1 ow0 hh ey1))
("ryon" nil (r ay1 ax n))
("ryrie" nil (r ay1 r iy0))
("rys" nil (r ih1 s))
("rysavy" nil (r ih0 s aa1 v iy0))
("ryser" nil (r ay1 s er0))
("ryskamp" nil (r ay1 s k ae1 m p))
("ryther" nil (r ih1 dh er0))
("ryton" nil (r ih1 t ax n))
("ryutaro" nil (r y uw0 t aa1 r ow0))
("ryuzo" nil (r ay1 uw1 z ow0))
("ryzhkov" nil (r ay1 z k aa0 v))
("rzasa" nil (er0 z aa1 s ax))
("rzepka" nil (er0 z eh1 p k ax))
("s" nil (eh1 s))
("sa" nil (s aa1))
("saab" nil (s aa1 b))
("saad" nil (s aa1 d))
("saal" nil (s aa1 l))
("saam" nil (s aa1 m))
("saamstaan" nil (s aa1 m s t aa1 n))
("saar" nil (s aa1 r))
("saari" nil (s aa1 r iy0))
("saarinen" nil (s aa1 r ih0 n ax n))
("saatchi" nil (s aa1 ch iy0))
("saathoff" nil (s aa1 t hh ao1 f))
("saavedra" nil (s aa0 v ey1 d r ax))
("saba" nil (s aa1 b ax))
("sabah" nil (s aa1 b ax))
("sabala" nil (s aa0 b aa1 l ax))
("sabalan" nil (s ae1 b ax l ax n))
("saban" nil (s ey1 b ax n))
("sabat" nil (s aa1 b aa0 t))
("sabatine" nil (s aa0 b aa0 t iy1 n iy0))
("sabatini" nil (s ae1 b ax t iy1 n iy0))
("sabatino" nil (s aa0 b aa0 t iy1 n ow0))
("sabatista" nil (s aa1 b ax t iy1 s t ax))
("sabatistas" nil (s aa1 b ax t iy1 s t ax z))
("sabatka" nil (s aa0 b aa1 t k ax))
("sabato" nil (s aa0 b aa1 t ow0))
("sabaudia" nil (s ax b ao1 d iy0 ax))
("sabb" nil (s ae1 b))
("sabbagh" nil (s ae1 b ax g))
("sabbath" nil (s ae1 b ax th))
("sabbatical" nil (s ax b ae1 t ih0 k ax l))
("sabedra" nil (s ax b eh1 d r ax))
("sabel" nil (s aa0 b eh1 l))
("sabella" nil (s ax b eh1 l ax))
("sabena" nil (s ax b iy1 n ax))
("saber" nil (s ey1 b er0))
("saberbein" nil (s ey1 b er0 b ay1 n))
("sabey" nil (s ey1 b iy0))
("sabia" nil (s aa1 b iy0 ax))
("sabin" nil (s ey1 b ih0 n))
("sabina" nil (s ax b iy1 n ax))
("sabine" nil (s ax b iy1 n))
("sabino" nil (s aa0 b iy1 n ow0))
("sabins" nil (s ey1 b ih0 n z))
("sablan" nil (s ae1 b l ax n))
("sable" nil (s ey1 b ax l))
("sables" nil (s ey1 b ax l z))
("sabo" nil (s aa1 b ow0))
("sabol" nil (s aa0 b ao1 l))
("sabotage" nil (s ae1 b ax t aa1 zh))
("sabotaged" nil (s ae1 b ax t aa1 zh d))
("sabotaging" nil (s ae1 b ax t aa1 zh ih0 ng))
("saboteur" nil (s ae1 b ax t er1))
("saboteurs" nil (s ae1 b ax t er1 z))
("sabourin" nil (s ae1 b uh0 r ae0 n))
("sabra" nil (s aa1 b r ax))
("sabre" nil (s ey1 b er0))
("sabres" nil (s ey1 b er0 z))
("sabrina" nil (s ax b r iy1 n ax))
("sabta" nil (s ae1 b t ax))
("sac" nil (s ae1 k))
("sacca" nil (s ae1 k ax))
("saccharin" nil (s ae1 k er0 ax n))
("saccharine" nil (s ae1 k er0 ay1 n))
("sacco" nil (s ae1 k ow0))
("saccoccio" nil (s aa0 k ow1 ch iy0 ow0))
("saccomanno" nil (s ae1 k ax m ih0 n ow0))
("saccone" nil (s aa0 k ow1 n iy0))
("saccule" nil (s ae1 k y uw1 l))
("sacha" nil (s ae1 ch ax))
("sacher" nil (s ae1 k er0))
("sachs" nil (s ae1 k s))
("sachse" nil (s ae1 ch s))
("sacilor" nil (s ae1 s ax l er0))
("sacirbey" nil (s aa1 k er0 b iy0))
("sack" nil (s ae1 k))
("sacked" nil (s ae1 k t))
("sackett" nil (s ae1 k ih0 t))
("sacking" nil (s ae1 k ih0 ng))
("sackler" nil (s ae1 k l er0))
("sackman" nil (s ae1 k m ax n))
("sackmann" nil (s ae1 k m ax n))
("sackrider" nil (s ae1 k r ay1 d er0))
("sacks" nil (s ae1 k s))
("sacksteder" nil (s ae1 k s t eh1 d er0))
("sackville" nil (s ae1 k v ih1 l))
("saco" nil (s ey1 k ow0))
("sacra" nil (s ae1 k r ax))
("sacrament" nil (s ae1 k r ax m ax n t))
("sacramento" nil (s ae1 k r ax m eh1 n t ow0))
("sacraments" nil (s ae1 k r ax m ax n t s))
("sacred" nil (s ey1 k r ax d))
("sacrifice" nil (s ae1 k r ax f ay1 s))
("sacrificed" nil (s ae1 k r ax f ay1 s t))
("sacrifices" nil (s ae1 k r ax f ay1 s ih0 z))
("sacrificial" nil (s ae1 k r ax f ih1 sh ax l))
("sacrificing" nil (s ae1 k r ax f ay1 s ih0 ng))
("sacrilege" nil (s ae1 k r ax l ax jh))
("sacrilegious" nil (s ae1 k r ax l eh1 jh ih0 s))
("sacrosanct" nil (s ae1 k r ow0 s ae0 ng k t))
("sacs" nil (s ae1 k s))
("sad" nil (s ae1 d))
("sada" nil (s aa1 d ax))
("sadao" nil (s ax d aw1))
("sadat" nil (s aa0 d aa1 t))
("saddam" nil (s aa1 d ax m))
("saddened" nil (s ae1 d ax n d))
("sadder" nil (s ae1 d er0))
("saddest" nil (s ae1 d ax s t))
("saddle" nil (s ae1 d ax l))
("saddlebags" nil (s ae1 d ax l b ae1 g z))
("saddled" nil (s ae1 d ax l d))
("saddler" nil (s ae1 d ax l er0))
("saddles" nil (s ae1 d ax l z))
("saddling" nil (s ae1 d ax l ih0 ng))
("saddoris" nil (s ae1 d er0 ih0 s))
("sadeghi" nil (s aa0 d eh1 g iy0))
("sadeh" nil (s aa1 d eh1))
("sadek" nil (s ae1 d ih0 k))
("sadella" nil (s ax d eh1 l ax))
("sader" nil (s ey1 d er0))
("sadie" nil (s ae1 d iy0))
("sadira" nil (s aa0 d ih1 r ax))
("sadism" nil (s ey1 d ih0 z ax m))
("sadist" nil (s ey1 d ax s t))
("sadistic" nil (s ax d ih1 s t ih0 k))
("sadler" nil (s ae1 d l er0))
("sadlier" nil (s ae1 d l iy0 er0))
("sadlon" nil (s ae1 d l ax n))
("sadlowski" nil (s ae0 d l aw1 s k iy0))
("sadly" nil (s ae1 d l iy0))
("sadness" nil (s ae1 d n ax s))
("sado" nil (s ey1 d ow0))
("sadoff" nil (s ae1 d ao1 f))
("sadowski" nil (s ax d ao1 f s k iy0))
("sadowsky" nil (s ax d aw1 s k iy0))
("sadye" nil (s ae1 d ay0))
("saechao" nil (s iy1 ch aw0))
("saeed" nil (s aa0 iy1 d))
("saeger" nil (s eh1 g er0))
("saehan" nil (s ey1 ax n))
("saelens" nil (s eh1 l ax n z))
("saemaul" nil (s ey1 m ao0 l))
("saenger" nil (s eh1 n jh er0))
("saenz" nil (s iy1 n z))
("saez" nil (s iy1 z))
("safari" nil (s ax f aa1 r iy0))
("safarik" nil (s ae1 f er0 ih0 k))
("safdie" nil (s ae1 f d iy0))
("safe" nil (s ey1 f))
("safecard" nil (s ey1 f k aa1 r d))
("safeco" nil (s ey1 f k ow0))
("safeguard" nil (s ey1 f g aa1 r d))
("safeguarded" nil (s ey1 f g aa1 r d ih0 d))
("safeguarding" nil (s ey1 f g aa1 r d ih0 ng))
("safeguards" nil (s ey1 f g aa1 r d z))
("safekeeping" nil (s ey1 f k iy1 p ih0 ng))
("safely" nil (s ey1 f l iy0))
("safer" nil (s ey1 f er0))
("safes" nil (s ey1 f s))
("safest" nil (s ey1 f ax s t))
("safety" nil (s ey1 f t iy0))
("safeway" nil (s ey1 f w ey1))
("saffell" nil (s ae1 f ax l))
("saffer" nil (s ae1 f er0))
("saffle" nil (s ae1 f ax l))
("safflower" nil (s ae1 f l aw1 er0))
("saffold" nil (s ae1 f ow1 l d))
("safford" nil (s ae1 f er0 d))
("saffran" nil (s ae1 f r ax n))
("safian" nil (s ey1 f iy0 ax n))
("safier" nil (s ax f ih1 r))
("safing" nil (s ey1 f ih0 ng))
("safire" nil (s ax f ay1 r))
("safko" nil (s aa1 f k ow0))
("safley" nil (s ae1 f l iy0))
("safra" nil (s ae1 f r ax))
("safran" nil (s ae1 f r ax n))
("safranek" nil (s ae1 f r ax n ih0 k))
("safrit" nil (s ae1 f r ih0 t))
("sag" nil (s ae1 g))
("saga" nil (s aa1 g ax))
("sagan" nil (s ey1 g ax n))
("sagansky" nil (s ax g ae1 n s k iy0))
("sagar" nil (s aa0 g aa1 r))
("sagas" nil (s aa1 g ax z))
("sage" nil (s ey1 jh))
("sagebrush" nil (s ey1 jh b r ah1 sh))
("sagen" nil (s ae1 g ax n))
("sager" nil (s ey1 g er0))
("sagers" nil (s ey1 g er0 z))
("sages" nil (s ey1 jh ih0 z))
("sagged" nil (s ae1 g d))
("saggese" nil (s ae1 g iy0 z))
("sagging" nil (s ae1 g ih0 ng))
("saginaw" nil (s ae1 g ih0 n ao1))
("sago" nil (s ey1 g ow0))
("sagona" nil (s aa0 g ow1 n ax))
("sagraves" nil (s aa0 g r aa1 v eh0 s))
("sags" nil (s ae1 g z))
("saha" nil (s aa1 hh ax))
("sahagian" nil (s ax hh iy1 jh iy0 ax n))
("sahagun" nil (s ae1 hh ax g ax n))
("sahara" nil (s ax hh eh1 r ax))
("saharan" nil (s ae1 hh er0 ax n))
("sahgal" nil (s aa1 g ax l))
("sahl" nil (s aa1 l))
("sahlen" nil (s aa1 l ax n))
("sahli" nil (s aa1 l iy0))
("sahlin" nil (s aa1 l ih0 n))
("sahlman" nil (s aa1 l m ax n))
("sahm" nil (s ae1 m))
("sahr" nil (s ae1 r))
("saia" nil (s aa1 y ax))
("said" nil (s eh1 d))
("saidiner" nil (s ey1 d ih0 n er0))
("saif" nil (s aa1 iy1 f))
("saigon" nil (s ay1 g aa1 n))
("saiki" nil (s ey1 k iy0))
("sail" nil (s ey1 l))
("sailboat" nil (s ey1 l b ow1 t))
("sailboats" nil (s ey1 l b ow1 t s))
("sailed" nil (s ey1 l d))
("sailer" nil (s ey1 l er0))
("sailfish" nil (s ey1 l f ih1 sh))
("sailing" nil (s ey1 l ih0 ng))
("sailor" nil (s ey1 l er0))
("sailors" nil (s ey1 l er0 z))
("sails" nil (s ey1 l z))
("sain" nil (s ey1 n))
("sainato" nil (s ax n aa1 t ow0))
("saindon" nil (s ey1 n d ax n))
("saine" nil (s ey1 n))
("sainsbury" nil (s ey1 n s b eh0 r iy0))
("saint" nil (s ey1 n t))
("sainted" nil (s ey1 n t ih0 d))
("sainthood" nil (s ey1 n t hh uh1 d))
("saintly" nil (s ey1 n t l iy0))
("saints" nil (s ey1 n t s))
("sainz" nil (s ey1 n z))
("saipan" nil (s ey1 p ax n))
("sais" nil (s ey1 z))
("saison" nil (s ey1 s aa0 n))
("saitama" nil (s aa1 ih0 t aa1 m ax))
("saito" nil (s ey1 t ow1))
("saitta" nil (s ey1 t ax))
("saiz" nil (s ey1 z))
("sajak" nil (s ae1 jh ae0 k))
("sajdak" nil (s ay1 d ax k))
("sak" nil (s ae1 k))
("sakaguchi" nil (s aa0 k aa0 g uw1 ch iy0))
("sakai" nil (s aa0 k aa1 iy0))
("sakal" nil (s ey1 k ax l))
("sakamoto" nil (s aa0 k aa0 m ow1 t ow0))
("sakata" nil (s aa0 k aa1 t ax))
("sakau" nil (s aa0 k aa1 uw0))
("sake" nil (s ey1 k))
("saker" nil (s ey1 k er0))
("sakes" nil (s ey1 k s))
("sakhalin" nil (s ae1 k hh ax l ih0 n))
("sakharov" nil (s aa1 k er0 aa0 v))
("saki" nil (s aa1 k iy0))
("sakigake" nil (s aa1 k iy0 g aa1 k iy0))
("sako" nil (s aa1 k ow0))
("sakowitz" nil (s ae1 k ax w ih0 t s))
("sakowski" nil (s ax k ao1 f s k iy0))
("saks" nil (s ae1 k s))
("saksa" nil (s ae1 k s ax))
("saksida" nil (s aa1 k s iy1 d ax))
("sakuma" nil (s aa0 k uw1 m ax))
("sakura" nil (s ae0 k uh1 r ax))
("sakurai" nil (s aa0 k uh0 r aa1 iy0))
("sal" nil (s ae1 l))
("sala" nil (s aa1 l ax))
("salaam" nil (s ax l aa1 m))
("salable" nil (s ey1 l ax b ax l))
("salables" nil (s ey1 l ax b ax l z))
("salacious" nil (s ax l ey1 sh ax s))
("salad" nil (s ae1 l ax d))
("salada" nil (s ax l aa1 d ax))
("saladin" nil (s ae1 l ax d ih0 n))
("saladino" nil (s aa0 l aa0 d iy1 n ow0))
("salads" nil (s ae1 l ax d z))
("salafia" nil (s ax l ey1 f iy0 ax))
("salah" nil (s ae1 l ax))
("salak" nil (s ae1 l ax k))
("salama" nil (s aa0 l aa1 m ax))
("salamanca" nil (s ae1 l ax m ae1 ng k ax))
("salamander" nil (s ae1 l ax m ae1 n d er0))
("salamanders" nil (s ae1 l ax m ae1 n d er0 z))
("salameh" nil (s aa1 l aa0 m eh0))
("salami" nil (s ax l aa1 m iy0))
("salamis" nil (s ax l aa1 m iy0 z))
("salamon" nil (s ae1 l ax m ax n))
("salamone" nil (s ae1 l ax m ow1 n))
("salant" nil (s ae1 l ax n t))
("salaried" nil (s ae1 l er0 iy0 d))
("salaries" nil (s ae1 l er0 iy0 z))
("salary" nil (s ae1 l er0 iy0))
("salarymen" nil (s ae1 l er0 iy0 m ax n))
("salas" nil (s aa1 l aa0 z))
("salata" nil (s aa0 l aa1 t ax))
("salatino" nil (s aa0 l aa0 t iy1 n ow0))
("salay" nil (s ae1 l ey0))
("salaz" nil (s aa1 l aa0 z))
("salazar" nil (s ae1 l ax z aa0 r))
("salberg" nil (s ae1 l b er0 g))
("salce" nil (s ey1 l s))
("salcedo" nil (s aa0 l ch ey1 d ow0))
("salcido" nil (s aa0 l ch iy1 d ow0))
("saldana" nil (s aa0 l d ae1 n ax))
("saldivar" nil (s aa0 l d iy0 v aa1 r))
("sale" nil (s ey1 l))
("saleable" nil (s ey1 l ax b ax l))
("saleeby" nil (s ae1 l iy0 b iy0))
("saleem" nil (s ae1 l iy0 m))
("saleh" nil (s aa1 l eh0 hh))
("salek" nil (s aa1 l eh0 k))
("salem" nil (s ey1 l ax m))
("salema" nil (s ax l iy1 m ax))
("salemi" nil (s aa0 l eh1 m iy0))
("salemme" nil (s ae1 l ih0 m))
("salerno" nil (s ax l eh1 r n ow0))
("sales" nil (s ey1 l z))
("salesman" nil (s ey1 l z m ax n))
("salesmanship" nil (s ey1 l z m ax n sh ih1 p))
("salesmen" nil (s ey1 l z m ih0 n))
("salespeople" nil (s ey1 l z p iy1 p ax l))
("salesperson" nil (s ey1 l z p er1 s ax n))
("saleswoman" nil (s ey1 l z w uh1 m ax n))
("saleswomen" nil (s ey1 l z w ih1 m ax n))
("salgado" nil (s aa0 l g aa1 d ow0))
("salguero" nil (s aa0 l g eh1 r ow0))
("salhany" nil (s ae1 l hh ey1 n iy0))
("saliba" nil (s aa0 l iy1 b ax))
("salick" nil (s ae1 l ih0 k))
("salient" nil (s ey1 l iy0 ax n t))
("saligman" nil (s ae1 l ih0 g m ax n))
("salim" nil (s ae1 l ih0 m))
("salin" nil (s aa0 l iy1 n))
("salina" nil (s ax l iy1 n ax))
("salinas" nil (s ax l iy1 n ax s))
("saline" nil (s ax l iy1 n))
("saling" nil (s ey1 l ih0 ng))
("salinger" nil (s ey1 l ih0 ng er0))
("salinity" nil (s ax l ih1 n ax t iy0))
("salis" nil (s aa1 l ih0 s))
("salisbury" nil (s ae1 l z b er0 iy0))
("saliva" nil (s ax l ay1 v ax))
("salivated" nil (s ae1 l ax v ey1 t ax d))
("salivating" nil (s ae1 l ax v ey1 t ih0 ng))
("salizzoni" nil (s ae1 l ih0 z ow1 n iy0))
("salk" nil (s ao1 k))
("salkeld" nil (s ae1 l k ih0 l d))
("salkin" nil (s ae1 l k ih0 n))
("sall" nil (s ao1 l))
("sallade" nil (s ae1 l ey1 d))
("sallas" nil (s aa1 l aa0 z))
("salle" nil (s ey1 l))
("sallee" nil (s ae1 l iy0))
("saller" nil (s ao1 l er0))
("salles" nil (s aa1 l eh0 s))
("salley" nil (s ae1 l iy0))
("sallie" nil (s ae1 l iy0))
("sallies" nil (s ae1 l iy0 z))
("salling" nil (s ao1 l ih0 ng))
("sallis" nil (s ae1 l ih0 s))
("salls" nil (s ao1 l z))
("sally" nil (s ae1 l iy0))
("salm" nil (s aa1 m))
("salman" nil (s ae1 m ax n))
("salmans" nil (s ae1 m ax n z))
("salmela" nil (s aa0 l m ey1 l ax))
("salmen" nil (s ae0 l m eh1 n))
("salmeron" nil (s aa0 l m eh0 r ao1 n))
("salmi" nil (s aa1 l m iy0))
("salminen" nil (s ae1 l m ih0 n ax n))
("salmon" nil (s ae1 m ax n))
("salmond" nil (s ae1 l m ax n d))
("salmonella" nil (s ae1 l m ax n eh1 l ax))
("salmons" nil (s ae1 m ax n z))
("salmonson" nil (s aa0 l m ow1 n s ax n))
("salo" nil (s aa1 l ow0))
("salois" nil (s ax l w aa1))
("saloma" nil (s aa0 l ow1 m ax))
("salome" nil (s ax l ow1 m iy0))
("salomi" nil (s aa0 l ow1 m iy0))
("salomon" nil (s ae1 l ax m ax n))
("salomone" nil (s aa0 l ow0 m ow1 n iy0))
("salon" nil (s ax l aa1 n))
("salone" nil (s ax l ow1 n))
("salonga" nil (s ax l ao1 ng g ax))
("salons" nil (s ax l aa1 n z))
("saloojee" nil (s ax l uw1 jh iy0))
("saloon" nil (s ax l uw1 n))
("saloons" nil (s ax l uw1 n z))
("salopek" nil (s ae1 l ax p ih0 k))
("salow" nil (s ae1 l ow0))
("salsa" nil (s ao1 l s ax))
("salsberry" nil (s ao1 l s b eh0 r iy0))
("salsbery" nil (s ae1 l s b er0 iy0))
("salsbury" nil (s ae1 l z b er0 iy0))
("salser" nil (s ey1 l s er0))
("salsgiver" nil (s ae1 l s g ih0 v er0))
("salsman" nil (s ao1 l s m ax n))
("salt" nil (s ao1 l t))
("saltarelli" nil (s ao0 l t aa0 r eh1 l iy0))
("saltbox" nil (s ao1 l t b aa1 k s))
("salted" nil (s ao1 l t ax d))
("salter" nil (s ao1 l t er0))
("salters" nil (s ao1 l t er0 z))
("saltier" nil (s ao1 l t iy0 er0))
("salting" nil (s ao1 l t ih0 ng))
("saltlike" nil (s ao1 l t l ay1 k))
("saltmarsh" nil (s ao1 l t m aa1 r sh))
("salton" nil (s ao1 l t ax n))
("salts" nil (s ao1 l t s))
("saltsman" nil (s ao1 l t s m ax n))
("saltwater" nil (s ao1 l t w aa1 t er0))
("salty" nil (s ao1 l t iy0))
("saltz" nil (s ae1 l t s))
("saltzburg" nil (s ao1 l t s b er0 g))
("saltzman" nil (s ao1 l t s m ax n))
("salus" nil (s ae1 l ih0 s))
("salutary" nil (s ae1 l y ax t eh1 r iy0))
("salutatorian" nil (s ax l uw1 t ax t ao1 r iy0 ax n))
("salute" nil (s ax l uw1 t))
("saluted" nil (s ax l uw1 t ax d))
("salutes" nil (s ax l uw1 t s))
("saluting" nil (s ax l uw1 t ih0 ng))
("salva" nil (s aa1 l v ax))
("salvador" nil (s ae1 l v ax d ao1 r))
("salvadoran" nil (s ae1 l v ax d ao1 r ax n))
("salvadorans" nil (s ae0 l v ax d ao1 r ax n z))
("salvadore" nil (s aa0 l v aa0 d ao1 r ey0))
("salvage" nil (s ae1 l v ax jh))
("salvageable" nil (s ae1 l v ih0 jh ax b ax l))
("salvaged" nil (s ae1 l v ih0 jh d))
("salvagers" nil (s ae1 l v ih0 jh er0 z))
("salvaggio" nil (s aa0 l v aa1 jh iy0 ow0))
("salvaging" nil (s ae1 l v ih0 jh ih0 ng))
("salvas" nil (s aa1 l v aa0 z))
("salvati" nil (s aa0 l v aa1 t iy0))
("salvatierra" nil (s aa0 l v aa0 t ih1 r ax))
("salvation" nil (s ae0 l v ey1 sh ax n))
("salvato" nil (s aa0 l v aa1 t ow0))
("salvatore" nil (s ae0 l v ax t ao1 r iy0))
("salvatori" nil (s aa0 l v aa0 t ao1 r iy0))
("salve" nil (s aa1 v))
("salves" nil (s aa1 v z))
("salvesen" nil (s aa0 l v iy1 z ax n))
("salveson" nil (s aa0 l v ey1 s ax n))
("salvetti" nil (s aa0 l v eh1 t iy0))
("salvi" nil (s aa1 l v iy0))
("salvia" nil (s ae1 l v iy0 ax))
("salvigsen" nil (s ae1 l v ih0 g s ax n))
("salvigsten" nil (s ae1 l v ih0 g s t ax n))
("salvina" nil (s aa0 l v iy1 n ax))
("salvino" nil (s ae0 l v iy1 n ow0))
("salvo" nil (s ae1 l v ow0))
("salvos" nil (s ae1 l v ow0 z))
("salvucci" nil (s aa0 l v uw1 ch iy0))
("salway" nil (s ae1 l w ey0))
("salwen" nil (s ae1 l w ax n))
("salyard" nil (s ao1 l y er0 d))
("salyards" nil (s ae1 l y aa0 r d z))
("salyer" nil (s aa1 l iy0 er0))
("salyers" nil (s aa1 l iy0 er0 z))
("salz" nil (s ao1 l z))
("salzano" nil (s aa0 l z aa1 n ow0))
("salzberg" nil (s ao1 l z b er0 g))
("salzburg" nil (s ao1 l t s b er0 g))
("salzer" nil (s ey1 l z er0))
("salzgitter" nil (s ao1 l t s g ih1 t er0))
("salzhauer" nil (s ao1 l t s hh aw1 r))
("salzman" nil (s ao1 l z m ax n))
("salzmann" nil (s ao1 l z m ax n))
("salzwedel" nil (s ae1 l z w ih0 d ax l))
("sam" nil (s ae1 m))
("sama" nil (s aa1 m ax))
("samaha" nil (s ae1 m ax hh ax))
("samaniego" nil (s aa0 m aa0 n iy1 g ow0))
("samano" nil (s aa0 m aa1 n ow0))
("samantha" nil (s ax m ae1 n th ax))
("samar" nil (s ae1 m aa0 r))
("samara" nil (s ae1 m er0 ax))
("samaranch" nil (s ae1 m er0 ae0 n ch))
("samaras" nil (s ae1 m er0 ax z))
("samaria" nil (s ey1 m er0 iy1 ax))
("samarin" nil (s aa0 m aa1 r iy0 n))
("samaritan" nil (s ax m eh1 r ih0 t ax n))
("samba" nil (s aa1 m b ax))
("samberg" nil (s ae1 m b er0 g))
("sambo" nil (s ae1 m b ow0))
("samborski" nil (s ax m b ao1 r s k iy0))
("sambrano" nil (s aa0 m b r aa1 n ow0))
("sambre" nil (s ae1 m b r ax))
("sambuca" nil (s ae1 m b y uw1 k ax))
("samcor" nil (s ae1 m k ao1 r))
("same" nil (s ey1 m))
("samedan" nil (s ae1 m ax d ax n))
("samek" nil (s ae1 m ih0 k))
("samella" nil (s ax m eh1 l ax))
("samelle" nil (s ax m eh1 l))
("samelson" nil (s ae1 m ih0 l s ax n))
("sameness" nil (s ey1 m n ax s))
("sames" nil (s ey1 m z))
("samet" nil (s ae1 m ih0 t))
("samford" nil (s ae1 m f er0 d))
("sami" nil (s ae1 m iy0))
("samir" nil (s ax m ih1 r))
("sammarco" nil (s aa0 m aa1 r k ow0))
("sammartino" nil (s aa0 m aa0 r t iy1 n ow0))
("sammet" nil (s ae1 m ih0 t))
("sammie" nil (s ae1 m iy0))
("sammis" nil (s ae1 m ih0 s))
("sammon" nil (s ae1 m ax n))
("sammons" nil (s ae1 m ax n z))
("samms" nil (s ae1 m z))
("sammut" nil (s ae1 m ax t))
("sammy" nil (s ae1 m iy0))
("samoa" nil (s ax m ow1 ax))
("samojlik" nil (s ax m oy1 l ih0 k))
("samons" nil (s aa1 m ow0 n z))
("samora" nil (s aa0 m ao1 r ax))
("samos" nil (s ey1 m aa0 s))
("samoth" nil (s ae1 m ax th))
("samp" nil (s ae1 m p))
("sampans" nil (s ae1 m p ae0 n z))
("samper" nil (s ae1 m p er0))
("sampey" nil (s ae1 m p iy0))
("sample" nil (s ae1 m p ax l))
("sampled" nil (s ae1 m p ax l d))
("sampler" nil (s ae1 m p l er0))
("samplers" nil (s ae1 m p l er0 z))
("samples" nil (s ae1 m p ax l z))
("sampley" nil (s ae1 m p l iy0))
("sampling" nil (s ae1 m p l ih0 ng))
("samplings" nil (s ae1 m p l ih0 ng z))
("sampras" nil (s ae1 m p r ax s))
("sampsel" nil (s ae1 m p s ax l))
("sampsell" nil (s ae1 m p s ax l))
("sampson" nil (s ae1 m p s ax n))
("samra" nil (s ae1 m r ax))
("sams" nil (s ae1 m z))
("samsara" nil (s ax m s aa1 r ax))
("samsel" nil (s ae1 m s ax l))
("samsom" nil (s ae1 m s ax m))
("samson" nil (s ae1 m s ax n))
("samsonite" nil (s ae1 m s ax n ay1 t))
("samsung" nil (s ae1 m s ah1 ng))
("samudio" nil (s aa0 m uw1 d iy0 ow0))
("samuel" nil (s ae1 m y uw0 l))
("samuela" nil (s ae1 m y uw0 eh1 l ax))
("samuelle" nil (s ae1 m y uw0 eh1 l))
("samuels" nil (s ae1 m y uw0 ax l z))
("samuelsen" nil (s ae1 m uh0 l s ax n))
("samuelson" nil (s ae1 m y uw0 ax l s ax n))
("samurai" nil (s ae1 m uh0 r ay1))
("samurais" nil (s ae1 m er0 ih0 z))
("san" nil (s ae1 n))
("sanaa" nil (s ax n aa1))
("sanabria" nil (s ax n ae1 b r iy0 ax))
("sanada" nil (s aa0 n aa1 d ax))
("sanborn" nil (s ae1 n b ao1 r n))
("sanches" nil (s aa1 n ch eh0 s))
("sanchez" nil (s ae1 n ch eh0 z))
("sancho" nil (s aa1 n k ow0))
("sancia" nil (s aa1 n ch ax))
("sanctification" nil (s ae1 ng k t ax f ax k ey1 sh ax n))
("sanctimonious" nil (s ae1 ng k t ax m ow1 n iy0 ax s))
("sanction" nil (s ae1 ng k sh ax n))
("sanctioned" nil (s ae1 ng k sh ax n d))
("sanctioning" nil (s ae1 ng k sh ax n ih0 ng))
("sanctions" nil (s ae1 ng k sh ax n z))
("sanctity" nil (s ae1 ng k t ih0 t iy0))
("sanctuaries" nil (s ae1 ng k ch uw0 eh1 r iy0 z))
("sanctuary" nil (s ae1 ng k ch uw0 eh1 r iy0))
("sanctum" nil (s ae1 ng k t ax m))
("sand" nil (s ae1 n d))
("sanda" nil (s ae1 n d ax))
("sandage" nil (s ae1 n d ih0 jh))
("sandahl" nil (s ae1 n d aa1 l))
("sandal" nil (s ae1 n d ax l))
("sandall" nil (s ae1 n d aa0 l))
("sandals" nil (s ae1 n d ax l z))
("sandau" nil (s ae1 n d aw0))
("sandbagged" nil (s ae1 n d b ae1 g d))
("sandbags" nil (s ae1 n d b ae1 g z))
("sandbar" nil (s ae1 n d b aa1 r))
("sandberg" nil (s ae1 n d b er0 g))
("sandblasted" nil (s ae1 n d b l ae1 s t ih0 d))
("sandbox" nil (s ae1 n d b aa1 k s))
("sande" nil (s ae1 n d))
("sanded" nil (s ae1 n d ih0 d))
("sandeen" nil (s ae1 n d iy0 n))
("sandefer" nil (s ae1 n d iy0 f er0))
("sandefur" nil (s ae1 n d eh0 f er0))
("sandel" nil (s ae1 n d eh0 l))
("sandell" nil (s ae1 n d eh0 l))
("sanden" nil (s ae1 n d ax n))
("sander" nil (s ae1 n d er0))
("sanderfer" nil (s ae1 n d er0 f er0))
("sanderford" nil (s ae1 n d er0 f ao0 r d))
("sanderlin" nil (s ae1 n d er0 l ih0 n))
("sanders" nil (s ae1 n d er0 z))
("sanderson" nil (s ae1 n d er0 s ax n))
("sandford" nil (s ae1 n d f er0 d))
("sandgren" nil (s ae1 n d g r eh0 n))
("sandhu" nil (s ae1 n d hh uw0))
("sandi" nil (s ae1 n d iy0))
("sandia" nil (s ae1 n d iy0 ax))
("sandidge" nil (s ae1 n d ih0 jh))
("sandie" nil (s ae1 n d iy0))
("sandifer" nil (s ae1 n d ay0 f er0))
("sandiford" nil (s ae1 n d ih0 f er0 d))
("sandin" nil (s ae1 n d ih1 n))
("sanding" nil (s ae1 n d ih0 ng))
("sandinista" nil (s ae1 n d ih0 n ih1 s t ax))
("sandinistas" nil (s ae1 n d ih0 n iy1 s t ax z))
("sandino" nil (s ae0 n d iy1 n ow0))
("sandler" nil (s ae1 n d l er0))
("sandlin" nil (s ae1 n d l ih0 n))
("sandling" nil (s ae1 d l ih0 ng))
("sandman" nil (s ae1 n d m ae1 n))
("sandmann" nil (s ae1 n d m ax n))
("sandmeyer" nil (s ae1 n d m ay0 er0))
("sandner" nil (s ae1 n d n er0))
("sandness" nil (s ae1 n d n ax s))
("sando" nil (s ae1 n d ow0))
("sandom" nil (s ae1 n d ax m))
("sandon" nil (s ae1 n d ao1 n))
("sandor" nil (s ae1 n d er0))
("sandos" nil (s ae1 n d ow0 z))
("sandoval" nil (s ae1 n d ow1 v ax l))
("sandow" nil (s ae1 n d ow0))
("sandoz" nil (s ae1 n d ow0 z))
("sandpaper" nil (s ae1 n d p ey1 p er0))
("sandquist" nil (s ae1 n d k w ih1 s t))
("sandra" nil (s ae1 n d r ax))
("sandridge" nil (s ae1 n d r ih1 jh))
("sandro" nil (s ae1 n d r ow0))
("sandrock" nil (s ae1 n d r aa1 k))
("sands" nil (s ae1 n d z))
("sandstone" nil (s ae1 n d s t ow1 n))
("sandstorm" nil (s ae1 n d s t ao1 r m))
("sandstorms" nil (s ae1 n d s t ao1 r m z))
("sandstrom" nil (s ae1 n d s t r ax m))
("sandt" nil (s ae1 n t))
("sandusky" nil (s ae0 n d ah1 s k iy0))
("sandvig" nil (s ae1 n d v ih1 g))
("sandvik" nil (s ae1 n d v ih0 k))
("sandwich" nil (s ae1 n d w ih0 ch))
("sandwiched" nil (s ae1 n d w ih1 ch t))
("sandwiches" nil (s ae1 n d w ih0 ch ih0 z))
("sandwick" nil (s ae1 n d w ih1 k))
("sandy" nil (s ae1 n d iy0))
("sane" nil (s ey1 n))
("saner" nil (s ey1 n er0))
("sanfilippo" nil (s ae1 n f ih0 l ih1 p ow0))
("sanford" nil (s ae1 n f er0 d))
("sang" nil (s ae1 ng))
("sanger" nil (s ae1 ng er0))
("sangiovese" nil (s ae1 n jh iy1 ow0 v iy1 s))
("sangster" nil (s ae1 ng s t er0))
("sanguine" nil (s ae1 ng g w ih0 n))
("sanguinetti" nil (s aa0 n g iy0 n eh1 t iy0))
("sangyo" nil (s ae1 n jh y ow0))
("sanitary" nil (s ae1 n ih0 t eh1 r iy0))
("sanitation" nil (s ae1 n ax t ey1 sh ax n))
("sanitized" nil (s ae1 n ih0 t ay1 z d))
("sanitizing" nil (s ae1 n ax t ay1 z ih0 ng))
("sanity" nil (s ae1 n ax t iy0))
("sanjay" nil (s ae1 n jh ey0))
("sanjiv" nil (s aa1 n jh iy1 v))
("sanjuan" nil (s aa0 n y uw0 aa1 n))
("sank" nil (s ae1 ng k))
("sanka" nil (s ae1 ng k ax))
("sankei" nil (s ae1 ng k iy0))
("sanker" nil (s ae1 ng k er0))
("sankey" nil (s ae1 ng k iy0))
("sanko" nil (s ae1 ng k ow0))
("sanks" nil (s ae1 ng k s))
("sankyo" nil (s ae1 ng k y ow0))
("sanmark" nil (s ae1 n m aa1 r k))
("sanmartin" nil (s ae1 n m aa0 r t ih0 n))
("sanmiguel" nil (s aa0 n m iy0 g eh1 l))
("sanna" nil (s ae1 n ax))
("sanner" nil (s ae1 n er0))
("sannes" nil (s ae1 n z))
("sano" nil (s aa1 n ow0))
("sanofi" nil (s ax n ow1 f iy0))
("sanrio" nil (s ae1 n r iy0 ow0))
("sans" nil (s ae1 n z))
("sansbury" nil (s ae1 n s b eh0 r iy0))
("sanseverino" nil (s aa0 n s eh0 v er0 iy1 n ow0))
("sansing" nil (s ae1 n s ih0 ng))
("sanskrit" nil (s ae1 n s k r ih0 t))
("sanso" nil (s ae1 n s ow0))
("sansom" nil (s ae1 n s ax m))
("sanson" nil (s ae1 n s ax n))
("sansone" nil (s ae1 n s ax n))
("sansoucie" nil (s ae1 n s ow0 k iy0))
("sansui" nil (s ae0 n s uw1 iy0))
("sant" nil (s ae1 n t))
("santa" nil (s ae1 n t ax))
("santacroce" nil (s aa0 n t aa0 k r ow1 ch iy0))
("santacruz" nil (s aa0 n t aa1 k r uw0 z))
("santagata" nil (s aa0 n t aa0 g aa1 t ax))
("santala" nil (s ae0 n t aa1 l ax))
("santamaria" nil (s aa0 n t aa0 m aa1 r iy0 ax))
("santana" nil (s ae0 n t ae1 n ax))
("santander" nil (s ae1 n t ae1 n d er0))
("santangelo" nil (s aa0 n t aa0 ng g eh1 l ow0))
("santaniello" nil (s aa0 n t aa0 n iy0 eh1 l ow0))
("santarelli" nil (s aa0 n t aa0 r eh1 l iy0))
("santarsiero" nil (s aa0 n t aa0 r s ih1 r ow0))
("santas" nil (s ae1 n t ax s))
("santayana" nil (s ae0 n t ay1 aa1 n ax))
("sante" nil (s ae1 n t ax))
("santella" nil (s aa0 n t eh1 l ax))
("santelli" nil (s aa0 n t eh1 l iy0))
("santer" nil (s ae1 n t er0))
("santerre" nil (s ae0 n t eh1 r))
("santi" nil (s ae1 n t iy0))
("santiago" nil (s ae1 n t iy0 aa1 g ow0))
("santiesteban" nil (s ae1 n t iy0 s t ih0 b ax n))
("santillan" nil (s ae1 n t ih1 l ax n))
("santillanes" nil (s ae1 n t ih0 l ey1 n z))
("santilli" nil (s aa0 n t iy1 l iy0))
("santillo" nil (s ae1 n t ih1 l ow0))
("santin" nil (s ae1 n t ih0 n))
("santini" nil (s ae0 n t iy1 n iy0))
("santino" nil (s aa0 n t iy1 n ow0))
("santistevan" nil (s aa0 n t iy0 s t ey0 v aa1 n))
("santo" nil (s ae1 n t ow0))
("santon" nil (s ae1 n t ax n))
("santone" nil (s aa0 n t ow1 n iy0))
("santoni" nil (s aa0 n t ow1 n iy0))
("santopietro" nil (s aa0 n t ow0 p iy1 t r ow0))
("santor" nil (s ae1 n t er0))
("santora" nil (s aa0 n t ao1 r ax))
("santore" nil (s aa0 n t ao1 r iy0))
("santorelli" nil (s aa0 n t ao0 r eh1 l iy0))
("santoro" nil (s ae0 n t ao1 r ow0))
("santorum" nil (s ae0 n t ao1 r ax m))
("santos" nil (s ae1 n t ow0 s))
("santosh" nil (s aa1 n t aa1 sh))
("santosuosso" nil (s aa0 n t ow0 s w ow1 s ow0))
("santow" nil (s ae1 n t ow0))
("santoyo" nil (s aa0 n t ow1 y ow0))
("santry" nil (s ae1 n t r iy0))
("santucci" nil (s aa0 n t uw1 ch iy0))
("santulli" nil (s aa0 n t uw1 l iy0))
("santy" nil (s ae1 n t iy0))
("sanville" nil (s aa1 n v ih0 l))
("sanwa" nil (s ae1 n w ax))
("sanyo" nil (s aa1 n y ow0))
("sanz" nil (s ae1 n z))
("sanzo" nil (s ae1 n z ow0))
("sanzone" nil (s aa0 n z ow1 n iy0))
("sao" nil (s aw1))
("saone" nil (s ey1 ow1 n))
("saouma" nil (s aw1 y uw1 m ax))
("sap" nil (s ae1 p))
("sapanski" nil (s ax p ae1 n s k iy0))
("saperstein" nil (s ey1 p er0 s t ay0 n))
("saphira" nil (s aa0 f ih1 r ax))
("sapia" nil (s aa1 p iy0 ax))
("sapien" nil (s ae1 p iy0 n))
("sapiens" nil (s ey1 p iy0 ax n z))
("sapienza" nil (s aa0 p iy1 n z ax))
("sapiro" nil (s ax p ih1 r ow0))
("saplings" nil (s ae1 p l ih0 ng z))
("sapoa" nil (s ax p ow1 ax))
("sapone" nil (s aa0 p ow1 n iy0))
("saponification" nil (s ax p aa1 n ax f ax k ey1 sh ax n))
("saporito" nil (s aa0 p ao0 r iy1 t ow0))
("sapp" nil (s ae1 p))
("sapped" nil (s ae1 p t))
("sappenfield" nil (s ax p eh1 n f iy0 l d))
("sapper" nil (s ae1 p er0))
("sapphire" nil (s ae1 f ay0 er0))
("sapping" nil (s ae1 p ih0 ng))
("sappington" nil (s ae1 p ih0 ng t ax n))
("sapporo" nil (s ax p ao1 r ow0))
("saps" nil (s ae1 p s))
("saputo" nil (s aa0 p uw1 t ow0))
("sar" nil (s aa1 r))
("sara" nil (s eh1 r ax))
("sarabia" nil (s er0 ey1 b iy0 ax))
("saracco" nil (s aa0 r aa1 k ow0))
("saraceni" nil (s aa0 r aa0 ch eh1 n iy0))
("saraceno" nil (s aa0 r aa0 ch eh1 n ow0))
("saracens" nil (s eh1 r ax s ax n z))
("saracino" nil (s aa0 r aa0 ch iy1 n ow0))
("sarafian" nil (s er0 ey1 f iy0 ax n))
("sarafin" nil (s ae1 r ax f ih0 n))
("sarafina" nil (s ae1 r ax f iy1 n ax))
("sarah" nil (s eh1 r ax))
("sarajevan" nil (s aa1 r ax y ey1 v ax n))
("sarajevans" nil (s aa1 r ax y ey1 v ax n z))
("sarajevo" nil (s aa1 r ax y ey1 v ow0))
("saran" nil (s er0 ae1 n))
("sarandon" nil (s eh1 r ax n d ih0 n))
("sarasin" nil (s eh1 r ih0 s ih0 n))
("sarasota" nil (s eh1 r ax s ow1 t ax))
("saratoga" nil (s eh1 r ax t ow1 g ax))
("saravia" nil (s er0 ey1 v iy0 ax))
("sarawak" nil (s eh1 r ax w aa0 k))
("sarazen" nil (s eh1 r ax z ax n))
("sarazin" nil (s aa0 r aa0 z iy1 n))
("sarbanes" nil (s aa1 r b ey1 n z))
("sarber" nil (s aa1 r b er0))
("sarcasm" nil (s aa1 r k ae1 z ax m))
("sarcastic" nil (s aa0 r k ae1 s t ih0 k))
("sarcastically" nil (s aa0 r k ae1 s t ih0 k l iy0))
("sarchet" nil (s aa0 r sh eh1 t))
("sarcoma" nil (s aa0 r k ow1 m er0))
("sarcophagus" nil (s aa0 r k aa1 f ax g ax s))
("sardas" nil (s aa1 r d ax s))
("sardella" nil (s aa1 r d eh1 l ax))
("sardina" nil (s aa0 r d iy1 n ax))
("sardinas" nil (s aa1 r d ih0 n ax z))
("sardine" nil (s aa0 r d iy1 n))
("sardines" nil (s aa0 r d iy1 n z))
("sardinha" nil (s aa1 r d ih1 n hh ax))
("sardinia" nil (s aa0 r d iy1 n iy0 ax))
("sardo" nil (s aa1 r d ow0))
("sardonic" nil (s aa0 r d aa1 n ih0 k))
("sardonically" nil (s aa0 r d aa1 n ih0 k ax l iy0))
("sare" nil (s eh1 r))
("sarejevo" nil (s aa1 r ax y ey1 v ow0))
("sarene" nil (s er0 iy1 n))
("sarette" nil (s er0 eh1 t))
("sarff" nil (s aa1 r f))
("sargasso" nil (s aa0 r g ae1 s ow0))
("sarge" nil (s aa1 r jh))
("sargeant" nil (s aa1 r jh ax n t))
("sargen" nil (s aa1 r g ax n))
("sargent" nil (s aa1 r jh ax n t))
("sargis" nil (s aa1 r g ih0 s))
("sari" nil (s aa1 r iy0))
("sarich" nil (s ae1 r ih0 k))
("sarine" nil (s aa0 r iy1 n iy0))
("sark" nil (s aa1 r k))
("sarka" nil (s aa1 r k ax))
("sarkar" nil (s aa1 r k aa1 r))
("sarkis" nil (s aa1 r k ih0 s))
("sarkisian" nil (s aa0 r k ih1 zh ih0 n))
("sarkissian" nil (s aa0 r k ih1 s zh ih0 n))
("sarles" nil (s aa1 r l z))
("sarli" nil (s aa1 r l iy0))
("sarlo" nil (s aa1 r l ow0))
("sarlos" nil (s aa1 r l ow0 s))
("sarmatian" nil (s aa0 r m ey1 sh ax n))
("sarmento" nil (s aa0 r m eh1 n t ow0))
("sarmiento" nil (s aa0 r m iy1 n t ow0))
("sarna" nil (s aa1 r n ax))
("sarney" nil (s aa1 r n iy0))
("sarni" nil (s aa1 r n iy0))
("sarno" nil (s aa1 r n ow0))
("sarnoff" nil (s aa1 r n ao0 f))
("sarnowski" nil (s er0 n ao1 f s k iy0))
("sarofim" nil (s eh1 r ax f iy1 m))
("sarokin" nil (s eh1 r ax k ih0 n))
("sarongs" nil (s er0 ao1 ng z))
("saros" nil (s aa1 r ow0 s))
("sarra" nil (s aa1 r ax))
("sarratt" nil (s er0 ae1 t))
("sarrazin" nil (s aa1 r ax z ih0 n))
("sarrett" nil (s ae1 r ih0 t))
("sarris" nil (s ae1 r ih0 s))
("sarro" nil (s aa1 r ow0))
("sars" nil (s aa1 r z))
("sarsfield" nil (s aa1 r s f iy0 l d))
("sartain" nil (s aa0 r t ey1 n))
("sarti" nil (s aa1 r t iy0))
("sartin" nil (s aa1 r t ih0 n))
("sartor" nil (s aa1 r t er0))
("sartori" nil (s aa0 r t ao1 r iy0))
("sartorial" nil (s aa0 r t ao1 r iy0 ax l))
("sartorius" nil (s aa0 r t ao1 r iy0 ih0 s))
("sartrain" nil (s aa1 r t r ey1 n))
("sartre" nil (s aa1 r t r ax))
("sartwell" nil (s aa1 r t w eh1 l))
("sarum" nil (s ae1 r ax m))
("sarver" nil (s aa1 r v er0))
("sarvis" nil (s aa1 r v ih0 s))
("sas" nil (s ae1 s))
("sasaki" nil (s aa0 s aa1 k iy0))
("sasek" nil (s ae1 s ih0 k))
("sash" nil (s ae1 sh))
("sasha" nil (s ae1 sh ax))
("sashes" nil (s ae1 sh ih0 z))
("saskatchewan" nil (s ae0 s k ae1 ch ax w ao1 n))
("saskatoon" nil (s ae1 s k ax t uw1 n))
("saslow" nil (s ae1 s l ow0))
("sasnett" nil (s ae1 s n ih0 t))
("sass" nil (s ae1 s))
("sassafras" nil (s ae1 s ax f r ae1 s))
("sassaman" nil (s ae1 s ax m ax n))
("sassano" nil (s aa0 s aa1 n ow0))
("sasse" nil (s ae1 s))
("sasseen" nil (s ae1 s iy0 n))
("sasser" nil (s ae1 s er0))
("sassi" nil (s ae1 s iy0))
("sassman" nil (s ae1 s m ax n))
("sassnet" nil (s ae1 s n eh1 t))
("sassnets" nil (s ae1 s n eh1 t s))
("sasso" nil (s ae1 s ow0))
("sasson" nil (s ae1 s ax n))
("sassone" nil (s aa0 s ow1 n iy0))
("sassoon" nil (s ax s uw1 n))
("sassy" nil (s ae1 s iy0))
("sat" nil (s ae1 t))
("satagaj" nil (s ae1 t ax g ay1))
("satan" nil (s ey1 t ax n))
("satanic" nil (s ax t ae1 n ih0 k))
("satanism" nil (s ey1 t ax n ih1 z ax m))
("satanist" nil (s ey1 t ax n ih0 s t))
("satchell" nil (s ae1 ch ax l))
("satcher" nil (s ae1 ch er0))
("sated" nil (s ey1 t ih0 d))
("satellite" nil (s ae1 t ax l ay1 t))
("satellites" nil (s ae1 t ax l ay1 t s))
("sater" nil (s ae1 t er0))
("sather" nil (s ae1 dh er0))
("sathre" nil (s ae1 th er0))
("satin" nil (s ae1 t ax n))
("satire" nil (s ae1 t ay1 er0))
("satires" nil (s ae1 t ay1 er0 z))
("satiric" nil (s ax t ih1 r ih0 k))
("satirical" nil (s ax t ih1 r ax k ax l))
("satirist" nil (s ae1 t er0 ax s t))
("satirists" nil (s ae1 t er0 ax s t s))
("satirizes" nil (s ae1 t er0 ay1 z ih0 z))
("satisfaction" nil (s ae1 t ax s f ae1 k sh ax n))
("satisfactions" nil (s ae1 t ax s f ae1 k sh ax n z))
("satisfactorily" nil (s ae1 t ih0 s f ae1 k t r ax l iy0))
("satisfactory" nil (s ae1 t ax s f ae1 k t r iy0))
("satisfied" nil (s ae1 t ax s f ay1 d))
("satisfies" nil (s ae1 t ih0 s f ay1 z))
("satisfy" nil (s ae1 t ax s f ay1))
("satisfying" nil (s ae1 t ih0 s f ay1 ih0 ng))
("sato" nil (s aa1 t ow0))
("satoshi" nil (s aa0 t ow1 sh iy0))
("satre" nil (s ey1 t er0))
("satriani" nil (s ae1 t r iy0 aa1 n iy0))
("satter" nil (s ae1 t er0))
("satterfield" nil (s ae1 t er0 f iy1 l d))
("satterlee" nil (s ae1 t er0 l iy1))
("satterly" nil (s ae1 t er0 l iy0))
("satterwhite" nil (s ae1 t er0 w ay1 t))
("sattler" nil (s ae1 t l er0))
("saturate" nil (s ae1 ch er0 ey1 t))
("saturated" nil (s ae1 ch er0 ey1 t ax d))
("saturating" nil (s ae1 ch er0 ey1 t ih0 ng))
("saturation" nil (s ae1 ch er0 ey1 sh ax n))
("saturday" nil (s ae1 t er0 d iy0))
("saturdays" nil (s ae1 t er0 d iy0 z))
("saturn" nil (s ae1 t er0 n))
("saturns" nil (s ae1 t er0 n z))
("satya" nil (s aa1 t y ax))
("satyandra" nil (s aa1 t y aa1 n d r ax))
("satz" nil (s ae1 t s))
("sauber" nil (s ao1 b er0))
("sauce" nil (s ao1 s))
("sauceda" nil (s aw0 s ey1 d ax))
("saucedo" nil (s aw0 s ey1 d ow0))
("saucepan" nil (s ao1 s p ae1 n))
("saucer" nil (s ao1 s er0))
("saucers" nil (s ao1 s er0 z))
("sauces" nil (s ao1 s ax z))
("saucier" nil (s ao1 s iy0 er0))
("saucy" nil (s ao1 s iy0))
("saud" nil (s ao1 d))
("sauder" nil (s ao1 d er0))
("saudi" nil (s ao1 d iy0))
("saudia" nil (s ao1 d iy0 ax))
("saudis" nil (s ao1 d iy0 z))
("sauer" nil (s aw1 er0))
("sauerkraut" nil (s aw1 er0 k r aw1 t))
("sauers" nil (s aw1 er0 z))
("sauerteig" nil (s aw1 er0 t ey1 g))
("sauerwein" nil (s aw1 er0 w ay0 n))
("sauey" nil (s ao1 iy0))
("saugerties" nil (s ao1 g er0 t iy0 z))
("sauk" nil (s ao1 k))
("saul" nil (s ao1 l))
("saulnier" nil (s aw1 l n iy0 er0))
("sauls" nil (s ao1 l z))
("saulsberry" nil (s ao1 l s b eh1 r iy0))
("saulsbury" nil (s ao1 l s b eh0 r iy0))
("sault" nil (s ao1 l t))
("saulter" nil (s ao1 l t er0))
("saulters" nil (s aw1 l t er0 z))
("saum" nil (s ao1 m))
("sauna" nil (s ao1 n ax))
("saunas" nil (s ao1 n ax z))
("saunders" nil (s ao1 n d er0 z))
("saunderson" nil (s ao1 n d er0 s ax n))
("saundra" nil (s ao1 n d r ax))
("saunier" nil (s ao1 n iy0 er0))
("saunter" nil (s ao1 n t er0))
("saur" nil (s ao1 r))
("saurer" nil (s aw1 er0 r))
("sauro" nil (s ao1 r ow0))
("saus" nil (s ao1 z))
("sausage" nil (s ao1 s ax jh))
("sausages" nil (s ao1 s ih0 jh ih0 z))
("sausalito" nil (s ao1 s ax l iy1 t ow0))
("sause" nil (s ao1 z))
("sauseda" nil (s aw0 s ey1 d ax))
("sauser" nil (s aw1 s er0))
("sausser" nil (s ao1 s er0))
("saute" nil (s ao0 t ey1))
("sauteed" nil (s ao0 t ey1 d))
("sauter" nil (s ao0 t ey1 er0))
("sauternes" nil (s ow0 t er1 n z))
("sautter" nil (s ao1 t er0))
("sauvage" nil (s ao1 v ih0 jh))
("sauvageau" nil (s ow1 v ax zh ow0))
("sauve" nil (s ao1 v))
("sauvignon" nil (s ao1 v ih1 n y aa0 n))
("sava" nil (s aa1 v ax))
("savage" nil (s ae1 v ax jh))
("savaged" nil (s ae1 v ih0 jh d))
("savagely" nil (s ae1 v ih0 jh l iy0))
("savagery" nil (s ae1 v ih0 jh eh1 r iy0))
("savages" nil (s ae1 v ax jh ax z))
("savaging" nil (s ae1 v ih0 jh ih0 ng))
("savaiko" nil (s ax v ey1 k ow0))
("savala" nil (s aa0 v aa1 l ax))
("savanna" nil (s ax v ae1 n ax))
("savannah" nil (s ax v ae1 n ax))
("savannas" nil (s ax v ae1 n ax z))
("savant" nil (s ax v aa1 n t))
("savants" nil (s ae1 v ax n t s))
("savard" nil (s ae1 v er0 d))
("savarese" nil (s aa0 v aa0 r ey1 z iy0))
("savarino" nil (s aa0 v aa0 r iy1 n ow0))
("savary" nil (s ae1 v eh0 r iy0))
("savas" nil (s aa1 v aa0 z))
("savasta" nil (s ax v ae1 s t ax))
("savastano" nil (s aa0 v aa0 s t aa1 n ow0))
("save" nil (s ey1 v))
("saved" nil (s ey1 v d))
("savedra" nil (s ax v eh1 d r ax))
("savel" nil (s aa0 v eh1 l))
("savell" nil (s aa0 v ey1 l))
("savely" nil (s ey1 v l iy0))
("saver" nil (s ey1 v er0))
("savers" nil (s ey1 v er0 z))
("savery" nil (s ey1 v er0 iy0))
("saves" nil (s ey1 v z))
("saviano" nil (s aa0 v iy0 aa1 n ow0))
("savich" nil (s ae1 v ih0 ch))
("savickas" nil (s ae1 v ih0 k ax z))
("savidge" nil (s ae1 v ih0 jh))
("savikas" nil (s ax v iy1 k ax s))
("savill" nil (s aa0 v iy1 l))
("saville" nil (s aa1 v ih0 l))
("savimbi" nil (s ax v ih1 m b iy0))
("savin" nil (s ae1 v ih0 n))
("savina" nil (s ax v iy1 n ax))
("saving" nil (s ey1 v ih0 ng))
("savings" nil (s ey1 v ih0 ng z))
("savini" nil (s aa0 v iy1 n iy0))
("savino" nil (s aa0 v iy1 n ow0))
("savio" nil (s aa1 v iy0 ow0))
("savior" nil (s ey1 v y er0))
("savitch" nil (s ae1 v ih0 ch))
("savitsky" nil (s ax v ih1 t s k iy0))
("savitt" nil (s ax v ih1 t))
("savitz" nil (s ae1 v ih0 t s))
("savko" nil (s ae1 v k ow0))
("savo" nil (s aa1 v ow0))
("savoca" nil (s aa0 v ow1 k ax))
("savoia" nil (s aa0 v ow1 y ax))
("savoie" nil (s aa1 v w aa0))
("savona" nil (s aa0 v ow1 n ax))
("savor" nil (s ey1 v er0))
("savored" nil (s ey1 v er0 d))
("savoring" nil (s ey1 v er0 ih0 ng))
("savors" nil (s ey1 v er0 z))
("savory" nil (s ey1 v er0 iy0))
("savoy" nil (s ax v oy1))
("savr" nil (s ey1 v er0))
("savviest" nil (s ae1 v iy0 ih0 s t))
("savvy" nil (s ae1 v iy0))
("saw" nil (s ao1))
("sawa" nil (s ao1 w ax))
("sawall" nil (s ao1 w ao0 l))
("sawasdee" nil (s ax w aa1 z d iy0))
("sawatzky" nil (s ax w aa1 t s k iy0))
("sawaya" nil (s ao0 w aa1 y ax))
("sawchuk" nil (s ao1 ch ax k))
("sawdey" nil (s ao1 d iy0))
("sawdon" nil (s ao1 d ax n))
("sawdust" nil (s ao1 d ah1 s t))
("sawdy" nil (s ao1 d iy0))
("sawed" nil (s ao1 d))
("sawhill" nil (s ao1 hh ih1 l))
("sawicki" nil (s ao0 ih1 t s k iy0))
("sawicz" nil (s aw1 ih0 t s))
("sawin" nil (s ao1 iy0 n))
("sawing" nil (s ao1 ih0 ng))
("sawka" nil (s ao1 k ax))
("sawmill" nil (s ao1 m ih1 l))
("sawmills" nil (s ao1 m ih1 l z))
("saws" nil (s ao1 z))
("sawtell" nil (s ao1 t eh1 l))
("sawtelle" nil (s ao1 t eh1 l))
("sawyer" nil (s ao1 y er0))
("sawyers" nil (s ao1 y er0 z))
("sax" nil (s ae1 k s))
("saxby" nil (s ae1 k s b iy0))
("saxe" nil (s ae1 k s))
("saxena" nil (s ae1 k s ih0 n ax))
("saxer" nil (s ae1 k s er0))
("saxman" nil (s ae1 k s m ax n))
("saxon" nil (s ae1 k s ax n))
("saxona" nil (s ae1 k s ax n ax))
("saxons" nil (s ae1 k s ax n z))
("saxony" nil (s ae1 k s ax n iy0))
("saxophone" nil (s ae1 k s ax f ow1 n))
("saxophones" nil (s ae1 k s ax f ow1 n z))
("saxophonist" nil (s ae1 k s ax f ow1 n ih0 s t))
("saxton" nil (s ae1 k s t ax n))
("say" nil (s ey1))
("sayad" nil (s ay1 ae0 d))
("saybrook" nil (s ey1 b r uh1 k))
("saye" nil (s ey1))
("sayed" nil (s aa1 y eh1 d))
("sayegh" nil (s ey1 ih0 g))
("sayer" nil (s ey1 er0))
("sayers" nil (s ey1 er0 z))
("saying" nil (s ey1 ih0 ng))
("sayings" nil (s ey1 ih0 ng z))
("sayle" nil (s ey1 l))
("sayler" nil (s ey1 l er0))
("sayles" nil (s ey1 l z))
("saylor" nil (s ey1 l er0))
("saylors" nil (s ey1 l er0 z))
("sayre" nil (s eh1 r))
("sayres" nil (s eh1 r z))
("says" nil (s eh1 z))
("sazama" nil (s aa0 z aa1 m ax))
("sbarro" nil (s b aa1 r ow0))
("sbf" nil (eh1 s b iy1 eh1 f))
("scab" nil (s k ae1 b))
("scabbard" nil (s k ae1 b er0 d))
("scabs" nil (s k ae1 b z))
("scaccia" nil (s k aa1 ch ax))
("scadden" nil (s k ae1 d ax n))
("scads" nil (s k ae1 d z))
("scaduto" nil (s k aa0 d uw1 t ow0))
("scaff" nil (s k ae1 f))
("scaffidi" nil (s k aa0 f iy1 d iy0))
("scaffold" nil (s k ae1 f ax l d))
("scaffolding" nil (s k ae1 f ax l d ih0 ng))
("scaffolds" nil (s k ae1 f ax l d z))
("scafidi" nil (s k aa0 f iy1 d iy0))
("scaggs" nil (s k ae1 g z))
("scaglione" nil (s k ae1 g l iy0 ow1 n iy0))
("scahill" nil (s k ey1 hh ih1 l))
("scaife" nil (s k ey1 f))
("scala" nil (s k aa1 l ax))
("scalar" nil (s k ey1 l er0))
("scalding" nil (s k ao1 l d ih0 ng))
("scalds" nil (s k ao1 l d z))
("scale" nil (s k ey1 l))
("scaled" nil (s k ey1 l d))
("scalera" nil (s k aa0 l eh1 r ax))
("scales" nil (s k ey1 l z))
("scalese" nil (s k aa0 l ey1 z iy0))
("scalf" nil (s k ae1 l f))
("scalfaro" nil (s k ae1 l f aa1 r ow0))
("scali" nil (s k aa1 l iy0))
("scalia" nil (s k aa1 l iy0 ax))
("scaling" nil (s k ey1 l ih0 ng))
("scalise" nil (s k aa1 l ay0 z))
("scalisi" nil (s k aa0 l iy1 s iy0))
("scallan" nil (s k ae1 l ax n))
("scallion" nil (s k ae1 l y ax n))
("scallions" nil (s k ae1 l y ax n z))
("scallon" nil (s k ae1 l ax n))
("scallop" nil (s k ae1 l ax p))
("scalloped" nil (s k aa1 l ax p t))
("scallops" nil (s k ae1 l ax p s))
("scally" nil (s k ae1 l iy0))
("scalp" nil (s k ae1 l p))
("scalped" nil (s k ae1 l p t))
("scalpel" nil (s k ae1 l p ax l))
("scalpels" nil (s k ae1 l p ax l z))
("scalpers" nil (s k ae1 l p er0 z))
("scalping" nil (s k ae1 l p ih0 ng))
("scalps" nil (s k ae1 l p s))
("scalzi" nil (s k aa1 l z iy0))
("scalzitti" nil (s k aa0 l z iy1 t iy0))
("scalzo" nil (s k aa1 l z ow0))
("scam" nil (s k ae1 m))
("scammell" nil (s k ae1 m ax l))
("scammers" nil (s k ae1 m er0 z))
("scammon" nil (s k ae1 m ax n))
("scamper" nil (s k ae1 m p er0))
("scampered" nil (s k ae1 m p er0 d))
("scampering" nil (s k ae1 m p er0 ih0 ng))
("scams" nil (s k ae1 m z))
("scan" nil (s k ae1 n))
("scandal" nil (s k ae1 n d ax l))
("scandalized" nil (s k ae1 n d ax l ay1 z d))
("scandalous" nil (s k ae1 n d ax l ax s))
("scandals" nil (s k ae1 n d ax l z))
("scandia" nil (s k ae1 n d iy0 ax))
("scandinavia" nil (s k ae1 n d ih0 n ey1 v iy0 ax))
("scandinavian" nil (s k ae1 n d ih0 n ey1 v iy0 ax n))
("scandinavians" nil (s k ae1 n d ih0 n ey1 v iy0 ax n z))
("scania" nil (s k aa1 n iy0 ax))
("scanlan" nil (s k ae1 n l ax n))
("scanland" nil (s k ae1 n l ax n d))
("scanlin" nil (s k ae1 n l ih0 n))
("scanlon" nil (s k ae1 n l ax n))
("scanned" nil (s k ae1 n d))
("scannell" nil (s k ae1 n ax l))
("scanner" nil (s k ae1 n er0))
("scanners" nil (s k ae1 n er0 z))
("scanning" nil (s k ae1 n ih0 ng))
("scans" nil (s k ae1 n z))
("scant" nil (s k ae1 n t))
("scantily" nil (s k ae1 n t ax l iy0))
("scantiness" nil (s k ae1 n t iy0 n ax s))
("scantlin" nil (s k ae1 n t l ih0 n))
("scanty" nil (s k ae1 n t iy0))
("scapa" nil (s k aa1 p ax))
("scapegoat" nil (s k ey1 p g ow1 t))
("scapegoating" nil (s k ey1 p g ow1 t ih0 ng))
("scapegoats" nil (s k ey1 p g ow1 t s))
("scapula" nil (s k ae1 p y ax l ax))
("scar" nil (s k aa1 r))
("scaramouch" nil (s k ae1 r ax m aw1 ch))
("scarano" nil (s k aa0 r aa1 n ow0))
("scarberry" nil (s k aa1 r b eh1 r iy0))
("scarborough" nil (s k aa1 r b er1 ow0))
("scarbro" nil (s k aa1 r b r ow0))
("scarbrough" nil (s k aa1 r b r aw0))
("scarce" nil (s k eh1 r s))
("scarcella" nil (s k aa1 r s eh1 l ax))
("scarcely" nil (s k eh1 r s l iy0))
("scarcer" nil (s k aa1 r s er0))
("scarcity" nil (s k eh1 r s ih0 t iy0))
("scardina" nil (s k aa0 r d iy1 n ax))
("scardino" nil (s k aa0 r d iy1 n ow0))
("scare" nil (s k eh1 r))
("scared" nil (s k eh1 r d))
("scares" nil (s k eh1 r z))
("scarf" nil (s k aa1 r f))
("scarface" nil (s k aa1 r f ey1 s))
("scarff" nil (s k aa1 r f))
("scarfo" nil (s k aa1 r f ow0))
("scarfs" nil (s k aa1 r f s))
("scargill" nil (s k aa1 r g ih1 l))
("scarier" nil (s k eh1 r iy0 er0))
("scariest" nil (s k eh1 r iy0 ax s t))
("scaring" nil (s k eh1 r ih0 ng))
("scarlata" nil (s k aa0 r l aa1 t ax))
("scarlet" nil (s k aa1 r l ax t))
("scarlett" nil (s k aa1 r l ih0 t))
("scarola" nil (s k aa0 r ow1 l ax))
("scarp" nil (s k aa1 r p))
("scarpa" nil (s k aa1 r p ax))
("scarpati" nil (s k aa0 r p aa1 t iy0))
("scarpelli" nil (s k aa0 r p eh1 l iy0))
("scarpello" nil (s k aa1 r p eh1 l ow0))
("scarpino" nil (s k aa0 r p iy1 n ow0))
("scarpone" nil (s k aa0 r p ow1 n iy0))
("scarpulla" nil (s k aa1 r p uh1 l ax))
("scarred" nil (s k aa1 r d))
("scarring" nil (s k aa1 r ih0 ng))
("scarry" nil (s k ae1 r iy0))
("scars" nil (s k aa1 r z))
("scarsdale" nil (s k aa1 r z d ey1 l))
("scarsella" nil (s k aa1 r s eh1 l ax))
("scarves" nil (s k aa1 r v z))
("scary" nil (s k eh1 r iy0))
("scat" nil (s k ae1 t))
("scatena" nil (s k aa0 t eh1 n ax))
("scates" nil (s k ey1 t s))
("scathing" nil (s k ey1 dh ih0 ng))
("scatter" nil (s k ae1 t er0))
("scattered" nil (s k ae1 t er0 d))
("scattergood" nil (s k ae1 t er0 g uh1 d))
("scattering" nil (s k ae1 t er0 ih0 ng))
("scattershot" nil (s k ae1 t er0 sh aa1 t))
("scaturro" nil (s k aa0 t uh1 r ow0))
("scavenge" nil (s k ae1 v ax n jh))
("scavenger" nil (s k ae1 v ax n jh er0))
("scavengers" nil (s k ae1 v ax n jh er0 z))
("scavo" nil (s k aa1 v ow0))
("scavone" nil (s k ax v ow1 n))
("scavuzzo" nil (s k aa0 v uw1 z ow0))
("scearce" nil (s er1 s))
("scenario" nil (s ih0 n eh1 r iy0 ow0))
("scenarios" nil (s ih0 n eh1 r iy0 ow0 z))
("scene" nil (s iy1 n))
("sceneries" nil (s iy1 n er0 iy0 z))
("scenery" nil (s iy1 n er0 iy0))
("scenes" nil (s iy1 n z))
("scenic" nil (s iy1 n ih0 k))
("scent" nil (s eh1 n t))
("scented" nil (s eh1 n t ih0 d))
("scents" nil (s eh1 n t s))
("sceptre" nil (s eh1 p t er0))
("scerbo" nil (s k eh1 r b ow0))
("schaab" nil (sh aa1 b))
("schaack" nil (sh aa1 k))
("schaad" nil (sh aa1 d))
("schaadt" nil (sh aa1 t))
("schaaf" nil (sh aa1 f))
("schaafsma" nil (sh aa1 f s m ax))
("schaal" nil (sh aa1 l))
("schaap" nil (sh aa1 p))
("schaar" nil (sh aa1 r))
("schab" nil (sh ae1 b))
("schabacker" nil (sh aa1 b ae1 k er0))
("schabel" nil (sh ae1 b ax l))
("schaben" nil (sh ae1 b ax n))
("schaber" nil (sh ey1 b er0))
("schaberg" nil (sh aa1 b er0 g))
("schabes" nil (sh ey1 b z))
("schacher" nil (sh ae1 k er0))
("schachner" nil (sh ae1 k n er0))
("schacht" nil (sh ae1 k t))
("schachter" nil (sh ae1 k t er0))
("schack" nil (sh ae1 k))
("schad" nil (sh ae1 d))
("schade" nil (sh ey1 d))
("schadel" nil (sh ae1 d ax l))
("schaden" nil (sh ae1 d ax n))
("schader" nil (sh ey1 d er0))
("schadler" nil (sh ey1 d ax l er0))
("schadt" nil (sh ae1 t))
("schaecher" nil (sh eh1 k er0))
("schaedel" nil (sh eh1 d ax l))
("schaedler" nil (sh eh1 d ax l er0))
("schaefer" nil (sh ey1 f er0))
("schaefers" nil (sh ey1 f r z))
("schaeffer" nil (sh eh1 f er0))
("schaer" nil (sh aa1 er0))
("schafer" nil (sh ey1 f er0))
("schaff" nil (sh ae1 f))
("schaffer" nil (sh ey1 f er0))
("schaffert" nil (sh ae1 f er0 t))
("schaffner" nil (sh ae1 f n er0))
("schaible" nil (s k ey1 b ax l))
("schairer" nil (sh ay1 er0 r))
("schaja" nil (sh aa1 jh ax))
("schake" nil (sh ey1 k))
("schalk" nil (sh ao1 k))
("schall" nil (sh ao1 l))
("schaller" nil (sh ao1 l er0))
("schallock" nil (sh ae1 l ax k))
("schalow" nil (sh ae1 l ow0))
("schama" nil (sh aa1 m ax))
("schamber" nil (sh ae1 m b er0))
("schamberger" nil (sh ae1 m b er0 g er0))
("schamel" nil (sh ae1 m ax l))
("schamp" nil (sh ae1 m p))
("schanbacher" nil (sh ae1 n b aa1 k er0))
("schanck" nil (sh ae1 ng k))
("schaner" nil (sh ey1 n er0))
("schank" nil (sh ae1 ng k))
("schantz" nil (sh ae1 n t s))
("schanz" nil (sh ae1 n s))
("schaper" nil (sh ey1 p er0))
("schapiro" nil (sh ae1 p ay0 r ow0))
("schappell" nil (sh ae1 p ax l))
("schappert" nil (sh ae1 p er0 t))
("schar" nil (sh aa1 r))
("schara" nil (s k ae1 r ax))
("schardt" nil (sh aa1 r t))
("scharenberg" nil (sh eh1 r ax n b er0 g))
("scharer" nil (sh eh1 r er0))
("scharf" nil (sh aa1 r f))
("scharfe" nil (s k aa1 r f))
("scharfenberg" nil (sh aa1 r f ax n b er0 g))
("scharff" nil (sh aa1 r f))
("scharffenberger" nil (sh aa1 r f ax n b er0 g er0))
("scharlau" nil (sh aa1 r l aw0))
("scharnhorst" nil (sh aa1 r n hh ao0 r s t))
("scharp" nil (sh aa1 r p))
("scharpf" nil (sh aa1 r p f))
("scharr" nil (sh aa1 r))
("scharrer" nil (sh aa1 r er0))
("schartz" nil (sh aa1 r t s))
("schattner" nil (sh ae1 t n er0))
("schatz" nil (sh ae1 t s))
("schatzberg" nil (sh ae1 t s b er0 g))
("schatzel" nil (sh ae1 t z ax l))
("schatzman" nil (sh ae1 t z m ax n))
("schau" nil (sh ow1))
("schaub" nil (sh ao1 b))
("schauble" nil (sh ow1 b ax l))
("schauer" nil (sh aw1 er0))
("schauf" nil (sh aw1 f))
("schaufler" nil (sh aw1 f ax l er0))
("schaul" nil (sh ow1 l))
("schaum" nil (sh ow1 m))
("schaumberg" nil (sh ow1 m b er0 g))
("schaumburg" nil (sh aw1 m b er0 g))
("schaus" nil (s k hh aw1 s))
("schaut" nil (sh ow1 t))
("schave" nil (sh ey1 v))
("schear" nil (sh ih1 r))
("schechter" nil (sh eh1 k t er0))
("schechtman" nil (sh eh1 k t m ax n))
("scheck" nil (sh eh1 k))
("scheckel" nil (sh eh1 k ax l))
("schecter" nil (sh eh1 k t er0))
("schedler" nil (sh eh1 d ax l er0))
("schedule" nil (s k eh1 jh uh0 l))
("scheduled" nil (s k eh1 jh uh0 l d))
("schedulers" nil (s k eh1 d jh uh0 l er0 z))
("schedules" nil (s k eh1 jh uh0 l z))
("scheduling" nil (s k eh1 jh uh0 l ih0 ng))
("scheel" nil (sh iy1 l))
("scheele" nil (sh iy1 l))
("scheeler" nil (sh iy1 l er0))
("scheer" nil (sh ih1 r))
("scheerer" nil (sh ih1 r er0))
("scheets" nil (sh iy1 t s))
("scheetz" nil (sh iy1 t s))
("scheff" nil (sh eh1 f))
("scheffel" nil (sh eh1 f ax l))
("scheffer" nil (sh eh1 f er0))
("scheffler" nil (sh eh1 f ax l er0))
("schehr" nil (sh eh1 r))
("scheib" nil (sh ay1 b))
("scheibe" nil (sh ay1 b))
("scheibel" nil (sh ay1 b ax l))
("scheiber" nil (sh ay1 b er0))
("scheible" nil (s k ay1 b ax l))
("scheibner" nil (sh ay1 b n er0))
("scheid" nil (sh ay1 d))
("scheidecker" nil (sh ay1 d ih0 k er0))
("scheidegger" nil (sh ay1 d ih0 g er0))
("scheidel" nil (sh ay1 d ax l))
("scheider" nil (sh ay1 d er0))
("scheiderer" nil (sh ay1 d er0 er0))
("scheidler" nil (sh ay1 d ax l er0))
("scheidt" nil (sh ay1 t))
("scheier" nil (sh ay1 er0))
("scheiman" nil (sh ay1 m ax n))
("schein" nil (sh ay1 n))
("scheinberg" nil (sh ay1 n b er0 g))
("scheiner" nil (sh ay1 n er0))
("scheirer" nil (sh ay1 er0 r))
("scheler" nil (sh iy1 l er0))
("schelin" nil (sh eh1 l ih0 n))
("schell" nil (s k eh1 l))
("schellenberg" nil (sh eh1 l ax n b er0 g))
("schellenberger" nil (sh eh1 l ax n b er0 g er0))
("schellenger" nil (sh eh1 l ih0 n jh er0))
("scheller" nil (s k eh1 l er0))
("schellhammer" nil (sh eh1 l hh ax m er0))
("schellhase" nil (sh eh1 l hh ax s))
("schellhorn" nil (sh eh1 l hh er0 n))
("schelling" nil (s k eh1 l ih0 ng))
("schellinger" nil (sh eh1 l ih0 ng er0))
("schemata" nil (s k ih0 m ae1 t ax))
("schematic" nil (s k ih0 m ae1 t ih0 k))
("schembri" nil (sh eh1 m b r iy0))
("scheme" nil (s k iy1 m))
("schemed" nil (s k iy1 m d))
("schemel" nil (sh eh1 m ax l))
("schemer" nil (s k iy1 m er0))
("schemes" nil (s k iy1 m z))
("scheming" nil (s k iy1 m ih0 ng))
("schemm" nil (sh eh1 m))
("schemmel" nil (sh eh1 m ax l))
("schempf" nil (sh eh1 m f))
("schempp" nil (sh eh1 m p))
("schena" nil (sh iy1 n ax))
("schenck" nil (sh eh1 ng k))
("schendel" nil (sh eh1 n d ax l))
("schenectady" nil (s k ax n eh1 k t ax d iy0))
("schenk" nil (sh eh1 ng k))
("schenkel" nil (sh eh1 ng k ax l))
("schenker" nil (sh eh1 ng k er0))
("schenley" nil (sh eh1 n l iy0))
("scheper" nil (sh iy1 p er0))
("schepers" nil (sh iy1 p er0 z))
("schepis" nil (sh eh1 p ih0 s))
("schepp" nil (sh eh1 p))
("scher" nil (sh er1))
("scherb" nil (sh er1 b))
("scherbarth" nil (sh er1 b aa0 r th))
("scherer" nil (sh ih1 r er0))
("scherf" nil (sh er1 f))
("scherff" nil (sh er1 f))
("scherger" nil (sh er1 g er0))
("schering" nil (sh er1 ih0 ng))
("scherlis" nil (sh er1 l ih0 s))
("scherman" nil (sh er1 m ax n))
("schermer" nil (sh er1 m er0))
("schermerhorn" nil (sh er1 m er0 hh er0 n))
("scherr" nil (sh eh1 r))
("scherrer" nil (sh eh1 r er0))
("schertz" nil (sh er1 t s))
("schertzer" nil (sh er1 t s er0))
("scherz" nil (sh er1 z))
("scherzer" nil (sh er1 z er0))
("scherzinger" nil (sh er1 z ih0 ng er0))
("schetter" nil (sh eh1 t er0))
("schettino" nil (sh eh1 t iy0 n ow0))
("schettler" nil (sh eh1 t ax l er0))
("scheu" nil (sh oy1))
("scheuer" nil (sh oy1 er0))
("scheuerman" nil (sh oy1 er0 m ax n))
("scheuermann" nil (sh oy1 er0 m ax n))
("scheufler" nil (sh oy1 f ax l er0))
("scheunemann" nil (sh oy1 n m ax n))
("scheurer" nil (sh er1 er0))
("scheurich" nil (sh oy1 r ih0 k))
("scheuring" nil (sh er1 ih0 ng))
("scheve" nil (sh iy1 v))
("schewe" nil (sh y uw1))
("schexnayder" nil (sh eh1 k s n ey0 d er0))
("schexnider" nil (sh eh1 k s n ay0 d er0))
("schey" nil (sh ey1))
("schiano" nil (s k iy0 aa1 n ow0))
("schiappa" nil (s k iy0 aa1 p ax))
("schiavi" nil (s k iy0 aa1 v iy0))
("schiavo" nil (s k iy0 aa1 v ow0))
("schiavone" nil (s k iy0 aa0 v ow1 n iy0))
("schiavoni" nil (s k iy0 aa0 v ow1 n iy0))
("schick" nil (sh ih1 k))
("schicker" nil (sh ih1 k er0))
("schickler" nil (sh ih1 k ax l er0))
("schickling" nil (sh ih1 k ax l ih0 ng))
("schiebel" nil (sh iy1 b ax l))
("schieber" nil (sh iy1 b er0))
("schiefelbein" nil (sh iy1 f ih0 l b ay0 n))
("schiefer" nil (sh iy1 f er0))
("schieffelin" nil (sh iy1 f ax l ih0 n))
("schieffer" nil (sh iy1 f er0))
("schiel" nil (sh iy1 l))
("schield" nil (sh iy1 l d))
("schiele" nil (sh iy1 l))
("schielke" nil (sh iy1 l k))
("schiemann" nil (sh iy1 m ax n))
("schier" nil (sh ay1 er0))
("schieren" nil (sh ih1 r ax n))
("schierl" nil (sh ih1 r l))
("schiess" nil (sh iy1 s))
("schiesser" nil (sh iy1 s er0))
("schiewe" nil (sh iy1 w iy0))
("schifano" nil (s k iy0 f aa1 n ow0))
("schiff" nil (sh ih1 f))
("schiffbauer" nil (sh ih1 f b aw0 er0))
("schiffer" nil (sh ih1 f er0))
("schiffler" nil (sh ih1 f ax l er0))
("schiffman" nil (sh ih1 f m ax n))
("schiffner" nil (sh ih1 f n er0))
("schild" nil (sh ay1 l d))
("schilder" nil (sh ay1 l d er0))
("schildknecht" nil (sh ay1 l d k ax n eh1 k t))
("schildt" nil (sh ih1 l t))
("schilke" nil (sh ih1 l k))
("schill" nil (sh ih1 l))
("schillaci" nil (s k iy0 l aa1 ch iy0))
("schiller" nil (sh ih1 l er0))
("schilling" nil (sh ih1 l ih0 ng))
("schillinger" nil (sh ih1 l ih0 ng er0))
("schillings" nil (sh ih1 l ih0 ng z))
("schillo" nil (s k ih1 l ow0))
("schilt" nil (sh ih1 l t))
("schiltknecht" nil (sh ih1 l t n eh1 k t))
("schiltz" nil (sh ih1 l t s))
("schilz" nil (sh ih1 l z))
("schimberni" nil (sh ih0 m b er1 n iy0))
("schimek" nil (sh ih1 m ih0 k))
("schimke" nil (s k ih1 m k))
("schimmel" nil (sh ih1 m ax l))
("schimmelbusch" nil (sh ih1 m ax l b uh1 sh))
("schimming" nil (sh ih1 m ih0 ng))
("schimpf" nil (sh ih1 m p f))
("schindel" nil (sh ih1 n d ax l))
("schindler" nil (sh ih1 n d l er0))
("schinke" nil (s k ih1 ng k))
("schinkel" nil (sh ih1 ng k ax l))
("schipani" nil (s k iy0 p aa1 n iy0))
("schipke" nil (sh ih1 p k iy0))
("schipper" nil (sh ih1 p er0))
("schippers" nil (sh ih1 p er0 z))
("schiraldi" nil (s k ih0 r aa1 l d iy0))
("schirm" nil (sh er1 m))
("schirmer" nil (sh er1 m er0))
("schiro" nil (s k ih1 r ow0))
("schirtzinger" nil (sh er1 t z ih0 ng er0))
("schisler" nil (sh ih1 s ax l er0))
("schism" nil (s k ih1 z ax m))
("schisms" nil (s k ih1 z ax m z))
("schissel" nil (sh ih1 s ax l))
("schissler" nil (sh ih1 s ax l er0))
("schists" nil (sh ih1 s t s))
("schizophrenia" nil (s k ih1 t s ax f r iy1 n iy0 ax))
("schizophrenic" nil (sh ih1 z ax f r eh1 n ih0 k))
("schlabach" nil (sh l ae1 b aa0 k))
("schlachter" nil (sh l ae1 k t er0))
("schlack" nil (sh l ae1 k))
("schlafer" nil (sh l ey1 f er0))
("schlafly" nil (sh l ae1 f l iy0))
("schlag" nil (sh l ae1 g))
("schlagel" nil (sh l ae1 g ax l))
("schlager" nil (sh l ey1 g er0))
("schlageter" nil (sh l ae1 g iy0 t er0))
("schlake" nil (sh l ey1 k))
("schlang" nil (sh l ae1 ng))
("schlangen" nil (sh l ae1 ng ax n))
("schlanger" nil (sh l ae1 ng er0))
("schlarb" nil (sh l aa1 r b))
("schlater" nil (sh l ey1 t er0))
("schlatter" nil (sh l ae1 t er0))
("schlauch" nil (sh l aw1 k))
("schlecht" nil (sh l eh1 k t))
("schlechter" nil (sh l eh1 k t er0))
("schlee" nil (sh l iy1))
("schleeter" nil (sh l iy1 t er0))
("schlegel" nil (sh l ey1 g ax l))
("schlegelmilch" nil (sh l eh1 g ih0 l m ih0 l k))
("schleich" nil (sh l ay1 k))
("schleicher" nil (sh l ay1 k er0))
("schleif" nil (sh l ay1 f))
("schleifer" nil (sh l ay1 f er0))
("schleiger" nil (sh l ay1 g er0))
("schleimer" nil (sh l ay1 m er0))
("schlein" nil (sh l ay1 n))
("schleis" nil (sh l ay1 z))
("schlemmer" nil (sh l eh1 m er0))
("schlender" nil (sh l eh1 n d er0))
("schlenker" nil (sh l eh1 ng k er0))
("schlepp" nil (sh l eh1 p))
("schlereth" nil (sh l eh1 r ih0 th))
("schlesinger" nil (sh l eh1 s ih0 n jh er0))
("schlesser" nil (sh l eh1 s er0))
("schlessinger" nil (sh l eh1 s ih0 n jh er0))
("schleswig" nil (sh l eh1 s w ih0 g))
("schley" nil (sh l ey1))
("schleyer" nil (sh l ey1 er0))
("schlicher" nil (sh l ih1 k er0))
("schlicht" nil (sh l ih1 k t))
("schlichter" nil (sh l ih1 k t er0))
("schlichting" nil (sh l ih1 k t ih0 ng))
("schlick" nil (sh l ih1 k))
("schlicker" nil (sh l ih1 k er0))
("schlie" nil (sh l iy1))
("schliep" nil (sh l iy1 p))
("schlieper" nil (sh l iy1 p er0))
("schlink" nil (sh l ih1 ng k))
("schlitt" nil (sh l ih1 t))
("schlitter" nil (sh l ih1 t er0))
("schlitz" nil (sh l ih1 t s))
("schlobohm" nil (sh l aa1 b ow0 m))
("schlock" nil (sh l aa1 k))
("schloemer" nil (sh l ow1 m er0))
("schloesser" nil (sh l aa1 ih0 s er0))
("schlosberg" nil (sh l aa1 s b er0 g))
("schloss" nil (sh l ao1 s))
("schlossberg" nil (sh l ao1 s b er0 g))
("schlosser" nil (sh l ao1 s er0))
("schlossman" nil (sh l ao1 s m ax n))
("schlott" nil (sh l aa1 t))
("schlotter" nil (sh l aa1 t er0))
("schlotterbeck" nil (sh l aa1 t er0 b eh0 k))
("schlottman" nil (sh l aa1 t m ax n))
("schlotzhauer" nil (sh l aa1 t s hh aw0 er0))
("schlough" nil (sh l aw1))
("schlueter" nil (sh l uh1 t er0))
("schlumberger" nil (sh l ah1 m b er0 zh ey1))
("schlund" nil (sh l ah1 n d))
("schlup" nil (sh l ah1 p))
("schluter" nil (sh l uw1 t er0))
("schmader" nil (sh m ey1 d er0))
("schmahl" nil (sh m aa1 l))
("schmal" nil (sh m ae1 l))
("schmale" nil (sh m ey1 l))
("schmall" nil (sh m ao1 l))
("schmaltz" nil (sh m aa1 l t s))
("schmalz" nil (sh m ao1 l z))
("schmaus" nil (sh m aw1 z))
("schmechel" nil (sh m eh1 k ax l))
("schmeck" nil (sh m eh1 k))
("schmehl" nil (sh m eh1 l))
("schmeichel" nil (sh m ay1 k ax l))
("schmeiser" nil (sh m ay1 s er0))
("schmeling" nil (sh m eh1 l ih0 ng))
("schmelter" nil (sh m eh1 l t er0))
("schmeltz" nil (sh m eh1 l t s))
("schmeltzer" nil (sh m eh1 l t z er0))
("schmelz" nil (sh m eh1 l z))
("schmelzer" nil (sh m eh1 l z er0))
("schmelzle" nil (sh m eh1 l z ax l))
("schmergel" nil (sh m er1 g ax l))
("schmertz" nil (sh m er1 t s))
("schmetterer" nil (sh m eh1 t er0 er0))
("schmick" nil (sh m ih1 k))
("schmid" nil (sh m ih1 d))
("schmidgall" nil (sh m ih1 jh ax l))
("schmidl" nil (sh m ih1 d ax l))
("schmidlin" nil (sh m ih1 d l ih0 n))
("schmidt" nil (sh m ih1 t))
("schmidtke" nil (sh m ih1 t k iy0))
("schmied" nil (sh m ay1 d))
("schmieder" nil (sh m ay1 d er0))
("schmieg" nil (sh m iy1 g))
("schmierer" nil (sh m ay1 er0 er0))
("schmiesing" nil (sh m iy1 s ih0 ng))
("schmit" nil (sh m ih1 t))
("schmitt" nil (sh m ih1 t))
("schmitter" nil (sh m ih1 t er0))
("schmittou" nil (sh m ih1 ch uw0))
("schmitz" nil (sh m ih1 t s))
("schmitzer" nil (sh m ih1 t s er0))
("schmoker" nil (sh m ow1 k er0))
("schmoldt" nil (sh m ow1 l t))
("schmoll" nil (sh m aa1 l))
("schmoozing" nil (sh m uw1 z ih0 ng))
("schmotzer" nil (sh m ow1 t z er0))
("schmoyer" nil (sh m oy1 er0))
("schmuck" nil (sh m ah1 k))
("schmucker" nil (sh m ah1 k er0))
("schmuckler" nil (sh m ah1 k l er0))
("schmuhl" nil (sh m ah1 l))
("schmults" nil (sh m ah1 l t s))
("schmunk" nil (sh m ah1 ng k))
("schmutz" nil (sh m ah1 t s))
("schmutzler" nil (sh m ah1 t z ax l er0))
("schnabel" nil (sh n ae1 b ax l))
("schnack" nil (sh n ae1 k))
("schnackenberg" nil (sh n ae1 k ax n b er0 g))
("schnake" nil (sh n ey1 k))
("schnakenberg" nil (sh n ey1 k ax n b er0 g))
("schnall" nil (sh n ao1 l))
("schnapp" nil (sh n ae1 p))
("schnapps" nil (sh n ae1 p s))
("schnarr" nil (sh n ae1 r))
("schnauzer" nil (sh n aw1 z er0))
("schneberger" nil (sh n iy1 b er0 g er0))
("schnebly" nil (sh n eh1 b l iy0))
("schneck" nil (sh n eh1 k))
("schneckloth" nil (sh n eh1 k l ax th))
("schnee" nil (sh n iy1))
("schneeberger" nil (sh n iy1 b er0 g er0))
("schneeman" nil (sh n iy1 m ax n))
("schneerson" nil (sh n ih1 r s ax n))
("schneid" nil (sh n ay1 d))
("schneider" nil (sh n ay1 d er0))
("schneiderman" nil (sh n ay1 d er0 m ax n))
("schneiders" nil (sh n ay1 d er0 z))
("schneidewind" nil (sh n ay1 d ax w ih1 n d))
("schneier" nil (sh n ay1 er0))
("schneiter" nil (sh n ay1 t er0))
("schnell" nil (sh n eh1 l))
("schnelle" nil (sh n eh1 l))
("schneller" nil (sh n eh1 l er0))
("schnepf" nil (sh n eh1 p f))
("schnepp" nil (sh n eh1 p))
("schnettler" nil (sh n eh1 t ax l er0))
("schnetzer" nil (sh n eh1 t z er0))
("schnick" nil (sh n ih1 k))
("schnider" nil (sh n ay1 d er0))
("schnieder" nil (sh n ay1 d er0))
("schnieders" nil (sh n ay1 d er0 z))
("schnier" nil (sh n ay1 er0))
("schnitker" nil (sh n ih1 t k er0))
("schnittke" nil (sh n ih1 t k iy0))
("schnitz" nil (sh n ih1 t s))
("schnitzer" nil (sh n ih1 t z er0))
("schnitzler" nil (sh n ih1 t s l er0))
("schnoebelen" nil (sh n ow1 b ax l ax n))
("schnook" nil (sh n uh1 k))
("schnooks" nil (sh n uh1 k s))
("schnoor" nil (sh n uh1 r))
("schnorr" nil (sh n ao1 r))
("schnur" nil (sh n er1))
("schnurr" nil (sh n er1))
("schnyder" nil (sh n ay1 d er0))
("schobel" nil (sh ow1 b ax l))
("schober" nil (sh ow1 b er0))
("schobert" nil (sh aa1 b er0 t))
("schoch" nil (sh aa1 k))
("schock" nil (sh aa1 k))
("schoeck" nil (sh ow1 k))
("schoeffler" nil (sh ow1 f ax l er0))
("schoeller" nil (sh ow1 l er0))
("schoellhorn" nil (sh ow1 l hh ao1 r n))
("schoemaker" nil (sh ow1 m ey1 k er0))
("schoen" nil (sh ow1 n))
("schoenbaum" nil (sh ow1 n b aw1 m))
("schoenbeck" nil (sh ow1 n b eh1 k))
("schoenberg" nil (sh ow1 n b er0 g))
("schoenberger" nil (sh ow1 n b er0 g er0))
("schoenborn" nil (sh ow1 n b er0 n))
("schoendorf" nil (sh ow1 n d ao0 r f))
("schoene" nil (sh aa1 iy0 n))
("schoeneck" nil (sh ow1 n eh0 k))
("schoenecker" nil (sh ow1 n eh0 k er0))
("schoeneman" nil (sh aa1 iy0 n m ax n))
("schoenemann" nil (sh aa1 iy0 n m ax n))
("schoener" nil (sh ow1 n er0))
("schoenfeld" nil (sh ow1 n f eh1 l d))
("schoenfelder" nil (sh ow1 n f eh0 l d er0))
("schoenfeldt" nil (sh ow1 n f ih0 l t))
("schoenhals" nil (sh ow1 n hh ax l z))
("schoenherr" nil (sh ow1 n hh er0))
("schoenholtz" nil (sh ow1 n hh ow1 l t s))
("schoenig" nil (sh ow1 n ih0 g))
("schoening" nil (sh aa1 ax n ih0 ng))
("schoenrock" nil (sh ow1 n r ax k))
("schoenthal" nil (sh ow1 n th ao1 l))
("schoenwald" nil (sh ow1 n w ao1 l d))
("schoepe" nil (sh ow1 p))
("schoepf" nil (sh ow1 p))
("schoepke" nil (sh ow1 p k iy0))
("schoepp" nil (sh ow1 p))
("schoeppner" nil (sh ow1 p n er0))
("schoettle" nil (sh ow1 t ax l))
("schoff" nil (sh ao1 f))
("schoffstall" nil (sh ao1 f s t ax l))
("schofield" nil (s k ow1 f iy0 l d))
("scholar" nil (s k aa1 l er0))
("scholarly" nil (s k aa1 l er0 l iy0))
("scholars" nil (s k aa1 l er0 z))
("scholarship" nil (s k aa1 l er0 sh ih1 p))
("scholarships" nil (s k aa1 l er0 sh ih1 p s))
("scholastic" nil (s k ax l ae1 s t ih0 k))
("scholer" nil (sh ow1 l er0))
("scholes" nil (s k ow1 l z))
("scholey" nil (sh ow1 l iy0))
("scholfield" nil (s k ow1 l f iy1 l d))
("scholl" nil (sh aa1 l))
("scholle" nil (sh ow1 l))
("scholler" nil (sh aa1 l er0))
("schollmeyer" nil (sh aa1 l m ay0 er0))
("scholten" nil (sh ow1 l t ax n))
("scholtes" nil (sh ow1 l t s))
("scholtz" nil (sh ow1 l t s))
("scholz" nil (sh ow1 l z))
("scholze" nil (sh ow1 l z))
("schomaker" nil (sh ow1 m ey1 k er0))
("schomberg" nil (sh aa1 m b er0 g))
("schomburg" nil (sh aa1 m b er0 g))
("schomer" nil (sh ow1 m er0))
("schommer" nil (sh aa1 m er0))
("schon" nil (sh aa1 n))
("schonberg" nil (sh aa1 n b er0 g))
("schonberger" nil (sh aa1 n b er0 g er0))
("schone" nil (sh ow1 n))
("schoneman" nil (sh ow1 n m ax n))
("schonfeld" nil (sh aa1 n f eh1 l d))
("schons" nil (sh aa1 n z))
("schoo" nil (sh uw1))
("schoof" nil (sh uh1 f))
("schook" nil (sh uh1 k))
("school" nil (s k uw1 l))
("schoolbooks" nil (s k uw1 l b uh1 k s))
("schoolboy" nil (s k uw1 l b oy1))
("schoolboys" nil (s k uw1 l b oy1 z))
("schoolchildren" nil (s k uw1 l ch ih1 l d r ax n))
("schoolcraft" nil (s k uw1 l k r ae1 f t))
("schooled" nil (s k uw1 l d))
("schooler" nil (s k uw1 l er0))
("schoolers" nil (s k uw1 l er0 z))
("schooley" nil (s k uw1 l iy0))
("schoolfield" nil (s k uw1 l f iy1 l d))
("schoolhouse" nil (s k uw1 l hh aw1 s))
("schooling" nil (s k uw1 l ih0 ng))
("schoolmaster" nil (s k uw1 l m ae1 s t er0))
("schoolmate" nil (s k uw1 l m ey1 t))
("schoolmates" nil (s k uw1 l m ey1 t s))
("schoolroom" nil (s k uw1 l r uw1 m))
("schools" nil (s k uw1 l z))
("schoolteacher" nil (s k uw1 l t iy1 ch er0))
("schoolteachers" nil (s k uw1 l t iy1 ch er0 z))
("schooltime" nil (s k uw1 l t ay1 m))
("schoolwork" nil (s k uw1 l w er1 k))
("schoolyard" nil (s k uw1 l y aa1 r d))
("schoon" nil (s k uw1 n))
("schooner" nil (s k uw1 n er0))
("schooners" nil (s k uw1 n er0 z))
("schoonmaker" nil (sh uw1 n m ey0 k er0))
("schoonover" nil (sh uw1 n ax v er0))
("schopf" nil (sh aa1 p f))
("schopfer" nil (sh aa1 p f er0))
("schopp" nil (sh aa1 p))
("schoppe" nil (sh aa1 p))
("schor" nil (sh ao1 r))
("schork" nil (sh ao1 r k))
("schorn" nil (sh ao1 r n))
("schorr" nil (sh ao1 r))
("schorsch" nil (sh ao1 r sh))
("schott" nil (sh aa1 t))
("schottenstein" nil (sh aa1 t ax n s t iy1 n))
("schou" nil (sh uw1))
("schouten" nil (sh aa1 uw0 t ax n))
("schow" nil (sh aw1))
("schowalter" nil (sh aw1 ax l t er0))
("schrack" nil (sh r ae1 k))
("schrade" nil (sh r ey1 d))
("schrader" nil (sh r ey1 d er0))
("schraeder" nil (sh r eh1 d er0))
("schrag" nil (sh r ae1 g))
("schrage" nil (sh r ey1 jh))
("schrager" nil (sh r ey1 g er0))
("schram" nil (sh r ae1 m))
("schramm" nil (sh r ae1 m))
("schrandt" nil (sh r ae1 n t))
("schrank" nil (sh r ae1 ng k))
("schrantz" nil (sh r ae1 n t s))
("schranz" nil (sh r ae1 n s))
("schraufnagel" nil (sh r aw1 f n ax g ax l))
("schrecengost" nil (sh r eh1 s ih0 ng g ax s t))
("schreck" nil (sh r eh1 k))
("schreckengost" nil (sh r eh1 k ih0 ng g ax s t))
("schrecongost" nil (sh r eh1 k ax ng g ax s t))
("schreder" nil (sh r iy1 d er0))
("schreffler" nil (sh r eh1 f ax l er0))
("schreger" nil (sh r eh1 g er0))
("schreiber" nil (sh r ay1 b er0))
("schreibman" nil (sh r ay1 b m ax n))
("schreier" nil (sh r ay1 er0))
("schreifels" nil (sh r ay1 f ax l z))
("schreiner" nil (sh r ay1 n er0))
("schreiter" nil (sh r ay1 t er0))
("schremp" nil (sh r eh1 m p))
("schrempf" nil (sh r eh1 m p f))
("schrempp" nil (sh r eh1 m p))
("schrenk" nil (sh r eh1 ng k))
("schreur" nil (sh r er1))
("schreurs" nil (sh r er1 z))
("schreyer" nil (sh r ay1 r))
("schriber" nil (sh r ay1 b er0))
("schrick" nil (sh r ih1 k))
("schricker" nil (sh r ih1 k er0))
("schrieber" nil (sh r iy1 b er0))
("schriefer" nil (sh r iy1 f er0))
("schrier" nil (sh r ay1 er0))
("schriever" nil (sh r iy1 v er0))
("schrimpf" nil (sh r ih1 m p f))
("schrimsher" nil (sh r ih1 m sh er0))
("schriner" nil (sh r ay1 n er0))
("schriver" nil (sh r ay1 v er0))
("schroader" nil (sh r ow1 d er0))
("schrock" nil (sh r aa1 k))
("schroder" nil (sh r ow1 d er0))
("schroders" nil (sh r ow1 d er0 z))
("schrodt" nil (sh r aa1 t))
("schroeck" nil (sh r ow1 k))
("schroeder" nil (sh r ow1 d er0))
("schroedl" nil (sh r ow1 d ax l))
("schroepfer" nil (sh r ow1 p f er0))
("schroer" nil (sh r ow1 er0))
("schroeter" nil (sh r ow1 t er0))
("schroff" nil (sh r ao1 f))
("schroll" nil (sh r ow1 l))
("schrom" nil (sh r aa1 m))
("schronce" nil (sh r aa1 n s))
("schropp" nil (sh r aa1 p))
("schroth" nil (sh r ao1 th))
("schroyer" nil (sh r oy1 er0))
("schrum" nil (sh r ah1 m))
("schrumpf" nil (sh r ah1 m p f))
("schrupp" nil (sh r ah1 p))
("schryer" nil (sh r ay1 er0))
("schryver" nil (sh r ay1 v er0))
("schubach" nil (sh ah1 b aa0 k))
("schubel" nil (sh uw1 b ax l))
("schubert" nil (sh uw1 b er0 t))
("schubring" nil (sh ah1 b er0 ih0 ng))
("schuch" nil (sh ah1 k))
("schuchard" nil (sh ah1 k er0 d))
("schuchardt" nil (sh ah1 k aa0 r t))
("schuchart" nil (sh ah1 k hh aa0 r t))
("schuchert" nil (s k ah1 ch er0 t))
("schuchman" nil (sh ah1 k m ax n))
("schuchmann" nil (sh ah1 k m ax n))
("schuck" nil (sh ah1 k))
("schucker" nil (sh ah1 k er0))
("schuckman" nil (sh ah1 k m ax n))
("schue" nil (sh uw1))
("schuele" nil (sh uw1 l))
("schueler" nil (sh uw1 l er0))
("schuelke" nil (sh uw1 l k))
("schueller" nil (sh uw1 l er0))
("schueneman" nil (sh uw1 n ax m ax n))
("schuenemann" nil (sh uw1 n ax m ax n))
("schuerman" nil (sh uw1 er0 m ax n))
("schuermann" nil (sh uw1 er0 m ax n))
("schuessler" nil (sh uw1 s l er0))
("schuett" nil (s k uw1 t))
("schuette" nil (s k uw1 t))
("schuetz" nil (sh uw1 t s))
("schuetze" nil (sh uw1 t s))
("schuff" nil (sh ah1 f))
("schug" nil (sh ah1 g))
("schuh" nil (sh uw1))
("schuhmacher" nil (sh uw1 m aa0 k er0))
("schuhmann" nil (sh uw1 m ax n))
("schuitema" nil (sh uw1 t ih0 m ax))
("schuknecht" nil (sh ah1 k n ih0 k t))
("schul" nil (sh uh1 l))
("schuld" nil (sh uh1 d))
("schuldener" nil (sh uw1 l d ax n er0))
("schuldt" nil (sh uh1 l t))
("schulenberg" nil (sh uw1 l ax n b er0 g))
("schulenburg" nil (sh uw1 l ax n b er0 g))
("schuler" nil (sh uw1 l er0))
("schulhof" nil (sh uw1 l hh ao0 f))
("schulke" nil (sh uh1 l k iy0))
("schull" nil (sh uh1 l))
("schuller" nil (sh uw1 l er0))
("schulman" nil (sh uw1 l m ax n))
("schulof" nil (sh uw1 l ao0 f))
("schult" nil (sh ah1 l t))
("schulte" nil (sh ah1 l t))
("schulten" nil (sh ah1 l t ax n))
("schultes" nil (sh ah1 l t s))
("schultheis" nil (sh ah1 l dh ay0 z))
("schultheiss" nil (sh ah1 l th ay0 s))
("schultz" nil (sh uh1 l t s))
("schultze" nil (sh ah1 l t z))
("schulz" nil (sh uh1 l t s))
("schulze" nil (sh uh1 l t s))
("schum" nil (sh ah1 m))
("schumacher" nil (sh uw1 m aa1 k er0))
("schumacker" nil (sh uw1 m aa1 k er0))
("schumaker" nil (sh uw1 m ey1 k er0))
("schuman" nil (sh uw1 m ax n))
("schumann" nil (sh uw1 m ax n))
("schumer" nil (sh uw1 m er0))
("schumm" nil (sh ah1 m))
("schumpert" nil (sh ah1 m p er0 t))
("schundler" nil (sh ah1 n d l er0))
("schuneman" nil (sh uw1 n m ax n))
("schunk" nil (sh ah1 ng k))
("schupak" nil (sh uw1 p ae1 k))
("schupbach" nil (sh ah1 p b aa1 k))
("schupp" nil (sh ah1 p))
("schur" nil (sh er1))
("schuring" nil (sh er1 ih0 ng))
("schurman" nil (sh er1 m ax n))
("schurr" nil (sh er1))
("schussler" nil (sh ah1 s l er0))
("schuster" nil (sh uw1 s t er0))
("schut" nil (sh ah1 t))
("schuth" nil (sh uw1 th))
("schutt" nil (sh ah1 t))
("schutte" nil (s k ah1 t))
("schutter" nil (sh ah1 t er0))
("schuttler" nil (sh ah1 t l er0))
("schutz" nil (sh ah1 t s))
("schutzman" nil (sh ah1 t z m ax n))
("schuur" nil (sh uh1 r))
("schuyler" nil (s k ay1 l er0))
("schuylkill" nil (s k y uw1 l k ih1 l))
("schwab" nil (sh w aa1 b))
("schwabe" nil (sh w ao1 b))
("schwaderer" nil (sh w ao1 d er0 er0))
("schwager" nil (sh w ey1 g er0))
("schwahn" nil (sh w ao1 n))
("schwaiger" nil (sh w ay1 g er0))
("schwake" nil (sh w ey1 k))
("schwalb" nil (sh w ao1 l b))
("schwalbach" nil (sh w ao1 l b aa1 k))
("schwalbe" nil (sh w ao1 l b))
("schwall" nil (sh w ao1 l))
("schwaller" nil (sh w ao1 l er0))
("schwalm" nil (sh w aa1 l m))
("schwam" nil (sh w ao1 m))
("schwan" nil (sh w ao1 n))
("schwandt" nil (sh w ao1 n t))
("schwanke" nil (sh w ao1 ng k))
("schwantes" nil (sh w ao1 n t s))
("schwanz" nil (sh w ao1 n s))
("schwark" nil (sh w ao1 r k))
("schwarm" nil (sh w ao1 r m))
("schwartz" nil (sh w ao1 r t s))
("schwartzberg" nil (sh w ao1 r t s b er0 g))
("schwartzkopf" nil (sh w ao1 r t s k ao0 p f))
("schwartzman" nil (sh w ao1 r t s m ax n))
("schwarz" nil (sh w ao1 r t s))
("schwarze" nil (sh w ao1 r t s))
("schwarzenegger" nil (sh w ao1 r z ax n ey1 g er0))
("schwarzer" nil (sh w ao1 r t s er0))
("schwarzkopf" nil (sh w ao1 r t s k ao0 p f))
("schwarzman" nil (sh w ao1 r t s m ax n))
("schwebach" nil (sh w eh1 b aa1 k))
("schwebel" nil (sh w eh1 b ax l))
("schwebke" nil (sh w eh1 b k))
("schwed" nil (sh w eh1 d))
("schweda" nil (sh w iy1 d ax))
("schwede" nil (sh w iy1 d))
("schweder" nil (sh w iy1 d er0))
("schweer" nil (sh w ih1 r))
("schweers" nil (sh w ih1 r z))
("schwegel" nil (sh w eh1 g ax l))
("schwegler" nil (sh w eh1 g ax l er0))
("schwegman" nil (sh w eh1 g m ax n))
("schweich" nil (sh w ay1 k))
("schweickert" nil (sh w ay1 k er0 t))
("schweiger" nil (sh w ay1 g er0))
("schweigert" nil (sh w ay1 g er0 t))
("schweighardt" nil (sh w ay1 g hh aa1 r t))
("schweikert" nil (sh w ay1 k er0 t))
("schweinsberg" nil (sh w ay1 n s b er0 g))
("schweiss" nil (sh w ay1 s))
("schweitzer" nil (sh w ay1 t s er0))
("schweizer" nil (sh w ay1 z er0))
("schweizerische" nil (sh w ay1 z er0 ih0 sh))
("schwemm" nil (sh w eh1 m))
("schwemmer" nil (sh w eh1 m er0))
("schwendeman" nil (sh w eh1 n d m ax n))
("schwenk" nil (sh w eh1 ng k))
("schwenke" nil (sh w eh1 ng k))
("schwenker" nil (sh w eh1 ng k er0))
("schwenn" nil (sh w eh1 n))
("schwent" nil (sh w eh1 n t))
("schweppe" nil (sh w eh1 p))
("schweppes" nil (sh w eh1 p s))
("schwer" nil (sh w er1))
("schwerdloff" nil (sh w er1 d l ao0 f))
("schwerdt" nil (sh w er1 t))
("schwerdtfeger" nil (sh w er1 t f ey1 g er0))
("schwerin" nil (sh w eh1 r ih0 n))
("schwering" nil (sh w ih1 r ih0 ng))
("schwerner" nil (sh w er1 n er0))
("schwertfeger" nil (sh w er1 t f ih0 g er0))
("schwertner" nil (sh w er1 t n er0))
("schwichtenberg" nil (sh w ih1 k t ax n b er0 g))
("schwieger" nil (sh w iy1 g er0))
("schwier" nil (sh w ay1 er0))
("schwiesow" nil (sh w iy1 s ow0))
("schwieterman" nil (sh w iy1 t er0 m ax n))
("schwimmer" nil (sh w ih1 m er0))
("schwind" nil (sh w ih1 n d))
("schwindt" nil (sh w ih1 n t))
("schwing" nil (sh w ih1 ng))
("schwinger" nil (sh w ih1 ng er0))
("schwinn" nil (sh w ih1 n))
("schwisow" nil (sh w ih1 s ow0))
("schwoerer" nil (sh w ow1 er0 er0))
("sci" nil (s iy1))
("sciacca" nil (s k ao1 k aa0))
("scialabba" nil (s k ao1 l aa0 b ax))
("scialdone" nil (sh ao1 l d ow0 n iy0))
("sciandra" nil (sh ao1 n d r ax))
("scianna" nil (sh ao1 n ax))
("sciara" nil (sh aa1 r ax))
("sciaroni" nil (s iy1 er0 ow1 n iy0))
("sciarra" nil (s iy0 aa1 r ax))
("sciarrino" nil (sh er0 iy1 n ow0))
("sciascia" nil (sh ao1 s ch ax))
("scibelli" nil (s ih0 b eh1 l iy0))
("scibilia" nil (s ih0 b iy1 l iy0 ax))
("scicchitano" nil (s ih0 k ih0 t aa1 n ow0))
("sciclone" nil (s ih1 k l ow1 n iy0))
("scicom" nil (s ih1 k ax m))
("science" nil (s ay1 ax n s))
("sciences" nil (s ay1 ax n s ax z))
("scientific" nil (s ay1 ax n t ih1 f ih0 k))
("scientifically" nil (s ay1 ax n t ih1 f ih0 k ax l iy0))
("scientifics" nil (s ay1 ax n t ih1 f ih0 k s))
("scientist" nil (s ay1 ax n t ih0 s t))
("scientists" nil (s ay1 ax n t ih0 s t s))
("scientology" nil (s iy1 ax n t aa1 l ax jh iy0))
("scifres" nil (s ay1 f er0 z))
("scilla" nil (s ih1 l ax))
("scillas" nil (s ih1 l ax z))
("scimeca" nil (s ih0 m eh1 k ax))
("scimed" nil (s ay1 m d))
("scimone" nil (s ih0 m ow1 n))
("scinta" nil (s ih1 n t ax))
("scintilla" nil (s ih0 n t ih1 l ax))
("scintilore" nil (s ih1 n t ax l ao1 r iy0))
("scinto" nil (s ih1 n t ow0))
("scioli" nil (s iy0 ow1 l iy0))
("scion" nil (s ay1 ax n))
("sciortino" nil (s iy0 ao0 r t iy1 n ow0))
("scios" nil (s k ay1 ow0 s))
("scipio" nil (s ih1 p iy0 ow0))
("scipione" nil (s ih0 p iy0 ow1 n iy0))
("scire" nil (s ay1 r))
("scism" nil (s k ih1 z ax m))
("scissors" nil (s ih1 z er0 z))
("scitex" nil (s iy1 t eh1 k s))
("sciulli" nil (s iy0 uw1 l iy0))
("sciullo" nil (s iy0 uw1 l ow0))
("sciuto" nil (s iy0 uw1 t ow0))
("sclafani" nil (s k l aa0 f aa1 n iy0))
("scleroderma" nil (s k l ih1 r ax d er1 m ax))
("sclerosis" nil (s k l er0 ow1 s ax s))
("scobee" nil (s k aa1 b iy0))
("scobey" nil (s k ow1 b iy0))
("scobie" nil (s k aa1 b iy0))
("scoby" nil (s k ow1 b iy0))
("scocozza" nil (s k aa0 k aa1 z ax))
("scoff" nil (s k ao1 f))
("scoffed" nil (s k ao1 f t))
("scofflaws" nil (s k aa1 f l ao1 z))
("scoffs" nil (s k ao1 f s))
("scofield" nil (s k ow1 f iy1 l d))
("scoggin" nil (s k aa1 g ih0 n))
("scoggins" nil (s k aa1 g ih0 n z))
("scogin" nil (s k ow1 g ih0 n))
("scohier" nil (s k ow1 y er0))
("scola" nil (s k ow1 l ax))
("scolari" nil (s k ow0 l aa1 r iy0))
("scolaro" nil (s k ow0 l aa1 r ow0))
("scold" nil (s k ow1 l d))
("scolded" nil (s k ow1 l d ax d))
("scolding" nil (s k ow1 l d ih0 ng))
("scolds" nil (s k ow1 l d z))
("scoles" nil (s k ow1 l z))
("scoma" nil (s k ow1 m ax))
("sconces" nil (s k aa1 n s ih0 z))
("scone" nil (s k ow1 n))
("sconiers" nil (s k ao1 n iy0 er0 z))
("sconnix" nil (s k aa1 n ih0 k s))
("sconyers" nil (s k ao1 n iy0 er0 z))
("scoop" nil (s k uw1 p))
("scooped" nil (s k uw1 p t))
("scooper" nil (s k uw1 p er0))
("scooping" nil (s k uw1 p ih0 ng))
("scoops" nil (s k uw1 p s))
("scoot" nil (s k uw1 t))
("scooter" nil (s k uw1 t er0))
("scooters" nil (s k uw1 t er0 z))
("scoots" nil (s k uw1 t s))
("scope" nil (s k ow1 p))
("scopes" nil (s k ow1 p s))
("scor" nil (s k ao1 r))
("scorched" nil (s k ao1 r ch t))
("scorching" nil (s k ao1 r ch ih0 ng))
("score" nil (s k ao1 r))
("scoreboard" nil (s k ao1 r b ao1 r d))
("scorecard" nil (s k ao1 r k aa1 r d))
("scored" nil (s k ao1 r d))
("scorekeepers" nil (s k ao1 r k iy1 p er0 z))
("scoreless" nil (s k ao1 r l ax s))
("scorer" nil (s k ao1 r er0))
("scorers" nil (s k ao1 r er0 z))
("scores" nil (s k ao1 r z))
("scoring" nil (s k ao1 r ih0 ng))
("scorn" nil (s k ao1 r n))
("scorned" nil (s k ao1 r n d))
("scornful" nil (s k ao1 r n f ax l))
("scorns" nil (s k ao1 r n z))
("scorpio" nil (s k ao1 r p iy0 ow1))
("scorpion" nil (s k ao1 r p iy0 ax n))
("scorpions" nil (s k ao1 r p iy0 ax n z))
("scorsese" nil (s k ao1 r s iy1 z))
("scorsone" nil (s k ao1 r s ax n))
("scorza" nil (s k ao1 r z ax))
("scot" nil (s k aa1 t))
("scotch" nil (s k aa1 ch))
("scotched" nil (s k aa1 ch t))
("scotches" nil (s k aa1 ch ih0 z))
("scotchgard" nil (s k aa1 ch g aa1 r d))
("scotia" nil (s k ow1 sh ax))
("scotland" nil (s k aa1 t l ax n d))
("scots" nil (s k aa1 t s))
("scotsman" nil (s k aa1 t s m ax n))
("scott" nil (s k aa1 t))
("scotten" nil (s k aa1 t ax n))
("scottie" nil (s k aa1 t iy0))
("scottish" nil (s k aa1 t ih0 sh))
("scotto" nil (s k ow1 t ow0))
("scotton" nil (s k aa1 t ax n))
("scotts" nil (s k aa1 t s))
("scottsbluff" nil (s k aa1 t s b l ah1 f))
("scottsdale" nil (s k aa1 t s d ey1 l))
("scotty" nil (s k aa1 t iy0))
("scoundrel" nil (s k aw1 n d r ax l))
("scoundrels" nil (s k aw1 n d r ax l z))
("scour" nil (s k aw1 er0))
("scoured" nil (s k aw1 er0 d))
("scourge" nil (s k er1 jh))
("scourges" nil (s k er1 jh ih0 z))
("scouring" nil (s k aw1 er0 ih0 ng))
("scours" nil (s k aw1 er0 z))
("scout" nil (s k aw1 t))
("scouted" nil (s k aw1 t ax d))
("scouten" nil (s k aw1 t ax n))
("scouters" nil (s k aw1 t er0 z))
("scouting" nil (s k aw1 t ih0 ng))
("scoutmaster" nil (s k aw1 t m ae1 s t er0))
("scouts" nil (s k aw1 t s))
("scovel" nil (s k ow1 v ax l))
("scovell" nil (s k aa1 v ax l))
("scovil" nil (s k ow1 v ax l))
("scovill" nil (s k aa1 v ax l))
("scoville" nil (s k ow1 v ih1 l))
("scow" nil (s k aw1))
("scowcroft" nil (s k ow1 k r ao1 f t))
("scowden" nil (s k aw1 d ax n))
("scowled" nil (s k aw1 l d))
("scowling" nil (s k aw1 l ih0 ng))
("scozzafava" nil (s k ow0 t s aa0 f aa1 v ax))
("scrabble" nil (s k r ae1 b ax l))
("scragg" nil (s k r ae1 g))
("scraggle" nil (s k r ae1 g ax l))
("scraggly" nil (s k r ae1 g l iy0))
("scramble" nil (s k r ae1 m b ax l))
("scrambled" nil (s k r ae1 m b ax l d))
("scrambles" nil (s k r ae1 m b ax l z))
("scrambling" nil (s k r ae1 m b ax l ih0 ng))
("scranton" nil (s k r ae1 n t ax n))
("scrap" nil (s k r ae1 p))
("scrapbook" nil (s k r ae1 p b uh1 k))
("scrape" nil (s k r ey1 p))
("scraped" nil (s k r ey1 p t))
("scraper" nil (s k r ey1 p er0))
("scrapers" nil (s k r ey1 p er0 z))
("scrapes" nil (s k r ey1 p s))
("scraping" nil (s k r ey1 p ih0 ng))
("scrapiron" nil (s k r ae1 p ay1 er0 n))
("scrapped" nil (s k r ae1 p t))
("scrapping" nil (s k r ae1 p ih0 ng))
("scrappy" nil (s k r ae1 p iy0))
("scraps" nil (s k r ae1 p s))
("scratch" nil (s k r ae1 ch))
("scratched" nil (s k r ae1 ch t))
("scratches" nil (s k r ae1 ch ax z))
("scratching" nil (s k r ae1 ch ih0 ng))
("scratchy" nil (s k r ae1 ch iy0))
("scrawl" nil (s k r ao1 l))
("scrawled" nil (s k r ao1 l d))
("scrawny" nil (s k r ao1 n iy0))
("scream" nil (s k r iy1 m))
("screamed" nil (s k r iy1 m d))
("screamer" nil (s k r iy1 m er0))
("screamers" nil (s k r iy1 m er0 z))
("screaming" nil (s k r iy1 m ih0 ng))
("screamingly" nil (s k r iy1 m ih0 ng l iy0))
("screams" nil (s k r iy1 m z))
("screech" nil (s k r iy1 ch))
("screeched" nil (s k r iy1 ch t))
("screeches" nil (s k r iy1 ch ih0 z))
("screeching" nil (s k r iy1 ch ih0 ng))
("screed" nil (s k r iy1 d))
("screen" nil (s k r iy1 n))
("screened" nil (s k r iy1 n d))
("screener" nil (s k r iy1 n er0))
("screeners" nil (s k r iy1 n er0 z))
("screening" nil (s k r iy1 n ih0 ng))
("screenings" nil (s k r iy1 n ih0 ng z))
("screenplay" nil (s k r iy1 n p l ey1))
("screenplays" nil (s k r iy1 n p l ey1 z))
("screens" nil (s k r iy1 n z))
("screenwriter" nil (s k r iy1 n r ay1 t er0))
("screenwriters" nil (s k r iy1 n r ay1 t er0 z))
("screw" nil (s k r uw1))
("screwball" nil (s k r uw1 b ao1 l))
("screwdriver" nil (s k r uw1 d r ay1 v er0))
("screwdrivers" nil (s k r uw1 d r ay1 v er0 z))
("screwed" nil (s k r uw1 d))
("screwing" nil (s k r uw1 ih0 ng))
("screws" nil (s k r uw1 z))
("screwy" nil (s k r uw1 iy0))
("scribble" nil (s k r ih1 b ax l))
("scribbled" nil (s k r ih1 b ax l d))
("scribblers" nil (s k r ih1 b l er0 z))
("scribbles" nil (s k r ih1 b ax l z))
("scribbling" nil (s k r ih1 b ax l ih0 ng))
("scribe" nil (s k r ay1 b))
("scriber" nil (s k r ay1 b er0))
("scribes" nil (s k r ay1 b z))
("scribner" nil (s k r ih1 b n er0))
("scrimgeour" nil (s k r ih1 m g ao0 r))
("scrimmage" nil (s k r ih1 m ih0 jh))
("scrimp" nil (s k r ih1 m p))
("scrimping" nil (s k r ih1 m p ih0 ng))
("scrimshaw" nil (s k r ih1 m sh ao1))
("scrip" nil (s k r ih1 p))
("scripps" nil (s k r ih1 p s))
("script" nil (s k r ih1 p t))
("scripted" nil (s k r ih1 p t ih0 d))
("scripts" nil (s k r ih1 p t s))
("scriptural" nil (s k r ih1 p ch er0 ax l))
("scripture" nil (s k r ih1 p ch er0))
("scriptures" nil (s k r ih1 p ch er0 z))
("scriptwriter" nil (s k r ih1 p t r ay1 t er0))
("scriptwriters" nil (s k r ih1 p t r ay1 t er0 z))
("scriptwriting" nil (s k r ih1 p t r ay1 t ih0 ng))
("scritchfield" nil (s k r ih1 ch f iy0 l d))
("scriven" nil (s k r ih1 v ih0 n))
("scrivener" nil (s k r ih1 v n er0))
("scrivens" nil (s k r ay1 v ax n z))
("scriver" nil (s k r ay1 v er0))
("scrivner" nil (s k r ih1 v n er0))
("scrod" nil (s k r aa1 d))
("scroggin" nil (s k r aa1 g ih0 n))
("scroggins" nil (s k r aa1 g ih0 n z))
("scroggs" nil (s k r aa1 g z))
("scrogham" nil (s k r aa1 g hh ax m))
("scroll" nil (s k r ow1 l))
("scrolls" nil (s k r ow1 l z))
("scronce" nil (s k r aa1 n s))
("scrooge" nil (s k r uw1 jh))
("scrounge" nil (s k r aw1 n jh))
("scrounged" nil (s k r aw1 n jh d))
("scrounging" nil (s k r aw1 n jh ih0 ng))
("scrub" nil (s k r ah1 b))
("scrubbed" nil (s k r ah1 b d))
("scrubber" nil (s k r ah1 b er0))
("scrubbers" nil (s k r ah1 b er0 z))
("scrubbing" nil (s k r ah1 b ih0 ng))
("scrubby" nil (s k r ah1 b iy0))
("scruffy" nil (s k r ah1 f iy0))
("scruggs" nil (s k r ah1 g z))
("scrunch" nil (s k r ah1 n ch))
("scruples" nil (s k r uw1 p ax l z))
("scrupulous" nil (s k r uw1 p y ax l ax s))
("scrupulously" nil (s k r uw1 p y ax l ax s l iy0))
("scrutinize" nil (s k r uw1 t ax n ay1 z))
("scrutinized" nil (s k r uw1 t ax n ay1 z d))
("scrutinizes" nil (s k r uw1 t ax n ay1 z ih0 z))
("scrutinizing" nil (s k r uw1 t ax n ay1 z ih0 ng))
("scrutiny" nil (s k r uw1 t ax n iy0))
("scruton" nil (s k r uw1 t ax n))
("scuba" nil (s k uw1 b ax))
("scud" nil (s k ah1 d))
("scudder" nil (s k ah1 d er0))
("scuderi" nil (s k uw0 d eh1 r iy0))
("scuds" nil (s k ah1 d z))
("scuff" nil (s k ah1 f))
("scuffed" nil (s k ah1 f t))
("scuffle" nil (s k ah1 f ax l))
("scuffled" nil (s k ah1 f ax l d))
("scuffles" nil (s k ah1 f ax l z))
("scull" nil (s k ah1 l))
("sculley" nil (s k ah1 l iy0))
("scullin" nil (s k ah1 l ih0 n))
("scullion" nil (s k ah1 l y ax n))
("scully" nil (s k ah1 l iy0))
("sculpt" nil (s k ah1 l p t))
("sculpted" nil (s k ah1 l p t ih0 d))
("sculpting" nil (s k ah1 l p t ih0 ng))
("sculptor" nil (s k ah1 l p t er0))
("sculptors" nil (s k ah1 l p t er0 z))
("sculpts" nil (s k ah1 l p t s))
("sculptural" nil (s k ah1 l p ch er0 ax l))
("sculpture" nil (s k ah1 l p ch er0))
("sculptured" nil (s k ah1 l p ch er0 d))
("sculptures" nil (s k ah1 l p ch er0 z))
("sculpturing" nil (s k ah1 l p t er0 ih0 ng))
("scum" nil (s k ah1 m))
("scupper" nil (s k ah1 p er0))
("scurlock" nil (s k er1 l ax k))
("scurried" nil (s k er1 iy0 d))
("scurrilous" nil (s k er1 ax l ax s))
("scurry" nil (s k er1 iy0))
("scurrying" nil (s k er1 iy0 ih0 ng))
("scutt" nil (s k ah1 t))
("scuttle" nil (s k ah1 t ax l))
("scuttled" nil (s k ah1 t ax l d))
("scuttling" nil (s k ah1 t ax l ih0 ng))
("scythian" nil (s ih1 th iy0 ax n))
("sczepanski" nil (s ih0 p ae1 n s k iy0))
("sdn" nil (eh1 s d iy1 eh1 n))
("se" nil (s aw1 th iy1 s t))
("sea" nil (s iy1))
("seabaugh" nil (s iy1 b ao1))
("seabeach" nil (s iy1 b iy1 ch))
("seabed" nil (s iy1 b eh1 d))
("seaberg" nil (s iy1 b er0 g))
("seaberry" nil (s iy1 b eh1 r iy0))
("seabert" nil (s iy1 b er0 t))
("seaboard" nil (s iy1 b ao1 r d))
("seabold" nil (s iy1 b ow1 l d))
("seabolt" nil (s iy1 b ow1 l t))
("seaborn" nil (s iy1 b er0 n))
("seaborne" nil (s iy1 b ao1 r n))
("seabright" nil (s iy1 b r ay1 t))
("seabrook" nil (s iy1 b r uh1 k))
("seabrooks" nil (s iy1 b r uh1 k s))
("seaburg" nil (s iy1 b er0 g))
("seabury" nil (s iy1 b eh1 r iy0))
("seaco" nil (s iy1 k ow0))
("seacoast" nil (s iy1 k ow1 s t))
("seader" nil (s iy1 d er0))
("seadrift" nil (s iy1 d r ih1 f t))
("seafarers" nil (s iy1 f eh1 r er0 z))
("seafirst" nil (s iy1 f er1 s t))
("seafood" nil (s iy1 f uw1 d))
("seafoods" nil (s iy1 f uw1 d z))
("seaford" nil (s iy1 f er0 d))
("seaga" nil (s iy1 g ax))
("seagal" nil (s ih0 g aa1 l))
("seagate" nil (s iy1 g ey1 t))
("seager" nil (s iy1 g er0))
("seagle" nil (s iy1 g ax l))
("seago" nil (s iy1 g ow1))
("seagoing" nil (s iy1 g ow1 ih0 ng))
("seagram" nil (s iy1 g r ax m))
("seagrave" nil (s iy1 g r ey1 v))
("seagraves" nil (s iy1 g r ey1 v z))
("seagren" nil (s iy1 g r eh0 n))
("seagroves" nil (s iy1 g r ow1 v z))
("seagull" nil (s iy1 g ah1 l))
("seahawk" nil (s iy1 hh ao1 k))
("seahawks" nil (s iy1 hh ao1 k s))
("seaholm" nil (s iy1 hh ow1 m))
("seal" nil (s iy1 l))
("sealand" nil (s iy1 l ae1 n d))
("sealant" nil (s iy1 l ax n t))
("sealants" nil (s iy1 l ax n t s))
("seale" nil (s iy1 l))
("sealed" nil (s iy1 l d))
("sealer" nil (s iy1 l er0))
("seales" nil (s iy1 l z))
("sealey" nil (s iy1 l iy0))
("sealift" nil (s iy1 l ih1 f t))
("sealing" nil (s iy1 l ih0 ng))
("sealock" nil (s iy1 l aa1 k))
("seals" nil (s iy1 l z))
("sealtest" nil (s iy1 l t eh1 s t))
("sealy" nil (s iy1 l iy0))
("seam" nil (s iy1 m))
("seaman" nil (s iy1 m ax n))
("seamans" nil (s iy1 m ax n z))
("seamen" nil (s iy1 m ax n))
("seamless" nil (s iy1 m l ax s))
("seamlessly" nil (s iy1 m l ax s l iy0))
("seamon" nil (s iy1 m ax n))
("seamons" nil (s iy1 m ax n z))
("seams" nil (s iy1 m z))
("seamster" nil (s iy1 m s t er0))
("seamstress" nil (s iy1 m s t r ih0 s))
("seamstresses" nil (s iy1 m s t r ax s ax z))
("seamus" nil (sh ey1 m ax s))
("seamy" nil (s iy1 m iy0))
("sean" nil (sh ao1 n))
("seanor" nil (s iy1 n er0))
("seaport" nil (s iy1 p ao1 r t))
("seaports" nil (s iy1 p ao1 r t s))
("seapower" nil (s iy1 p aw1 er0))
("seaq" nil (s iy1 k))
("seaquist" nil (s iy1 k w ih1 s t))
("sear" nil (s ih1 r))
("search" nil (s er1 ch))
("searched" nil (s er1 ch t))
("searchers" nil (s er1 ch er0 z))
("searches" nil (s er1 ch ih0 z))
("searching" nil (s er1 ch ih0 ng))
("seared" nil (s ih1 r d))
("searfoss" nil (s er1 f ax s))
("searight" nil (s ih1 r ay0 t))
("searing" nil (s ih1 r ih0 ng))
("searl" nil (s er1 l))
("searle" nil (s er1 l))
("searles" nil (s ao1 r ax l z))
("searls" nil (s er1 l z))
("sears" nil (s ih1 r z))
("searson" nil (s er1 s ax n))
("seas" nil (s iy1 z))
("sease" nil (s iy1 z))
("seashell" nil (s iy1 sh eh1 l))
("seashells" nil (s iy1 sh eh1 l z))
("seashore" nil (s iy1 sh ao1 r))
("seasick" nil (s iy1 s ih1 k))
("seaside" nil (s iy1 s ay1 d))
("season" nil (s iy1 z ax n))
("seasonal" nil (s iy1 z ax n ax l))
("seasonality" nil (s iy1 z ax n ae1 l ih0 t iy0))
("seasonally" nil (s iy1 z ax n ax l iy0))
("seasoned" nil (s iy1 z ax n d))
("seasoning" nil (s iy1 z ax n ih0 ng))
("seasonings" nil (s iy1 z ax n ih0 ng z))
("seasons" nil (s iy1 z ax n z))
("seastrom" nil (s iy1 s t r ax m))
("seat" nil (s iy1 t))
("seated" nil (s iy1 t ax d))
("seater" nil (s iy1 t er0))
("seating" nil (s iy1 t ih0 ng))
("seatings" nil (s iy1 t ih0 ng z))
("seaton" nil (s iy1 t ax n))
("seatrain" nil (s iy1 t r ey1 n))
("seats" nil (s iy1 t s))
("seattle" nil (s iy0 ae1 t ax l))
("seaver" nil (s iy1 v er0))
("seavers" nil (s iy1 v er0 z))
("seavey" nil (s iy1 v iy0))
("seaward" nil (s iy1 w er0 d))
("seawater" nil (s iy1 w aa1 t er0))
("seaway" nil (s iy1 w ey1))
("seaweed" nil (s iy1 w iy1 d))
("seaweeds" nil (s iy1 w iy1 d z))
("seawell" nil (s iy1 w eh1 l))
("seawolf" nil (s iy1 w uh1 l f))
("seawright" nil (s iy1 r ay1 t))
("seay" nil (s ey1))
("sebaceous" nil (s ax b ey1 sh ax s))
("sebald" nil (s iy1 b ao0 l d))
("sebastian" nil (s ax b ae1 s ch ax n))
("sebastiana" nil (s ax b ae1 s t iy0 aa1 n ax))
("sebastiane" nil (s ax b ae1 s t iy0 eh1 n))
("sebastiani" nil (s ax b ae1 s t iy0 aa1 n iy0))
("sebby" nil (s eh1 b iy0))
("sebek" nil (s eh1 b ih0 k))
("seber" nil (s iy1 b er0))
("sebert" nil (s eh1 b er0 t))
("sebesta" nil (s eh0 b eh1 s t ax))
("sebi" nil (s eh1 b iy0))
("sebo" nil (s eh1 b ow0))
("sebold" nil (s eh1 b ow0 l d))
("sebree" nil (s ih0 b r iy1))
("sebring" nil (s iy1 b r ih0 ng))
("sec" nil (s eh1 k))
("secaucus" nil (s ih0 k ao1 k ax s))
("secchia" nil (s eh1 k iy0 ax))
("secede" nil (s ih0 s iy1 d))
("seceded" nil (s ih0 s iy1 d ih0 d))
("secession" nil (s ih0 s eh1 sh ax n))
("secessionist" nil (s ih0 s eh1 sh ax n ih0 s t))
("secessionists" nil (s ih0 s eh1 sh ax n ih0 s t s))
("sechler" nil (s eh1 k l er0))
("sechrest" nil (s eh1 k er0 ih0 s t))
("sechrist" nil (s eh1 k er0 ih0 s t))
("seck" nil (s eh1 k))
("seckel" nil (s eh1 k ax l))
("seckinger" nil (s eh1 k ih0 ng er0))
("seckler" nil (s eh1 k l er0))
("seckman" nil (s eh1 k m ax n))
("secluded" nil (s ih0 k l uw1 d ih0 d))
("seclusion" nil (s ih0 k l uw1 zh ax n))
("seco" nil (s eh1 k ow0))
("secom" nil (s eh1 k ax m))
("secomerica" nil (s eh1 k ow0 m eh1 r ih0 k ax))
("second" nil (s eh1 k ax n d))
("secondaries" nil (s eh1 k ax n d eh1 r iy0 z))
("secondarily" nil (s eh1 k ax n d eh1 r ax l iy0))
("secondary" nil (s eh1 k ax n d eh1 r iy0))
("seconded" nil (s eh1 k ax n d ih0 d))
("secondhand" nil (s eh1 k ax n d hh ae1 n d))
("secondly" nil (s eh1 k ax n d l iy0))
("seconds" nil (s eh1 k ax n d z))
("secor" nil (s eh1 k er0))
("secord" nil (s iy1 k ao0 r d))
("secoy" nil (s eh1 k oy0))
("secrecy" nil (s iy1 k r ax s iy0))
("secrest" nil (s eh1 k er0 ih0 s t))
("secret" nil (s iy1 k r ax t))
("secretarial" nil (s eh1 k r ax t eh1 r iy0 ax l))
("secretariat" nil (s eh1 k r ih0 t eh1 r iy0 ax t))
("secretaries" nil (s eh1 k r ax t eh1 r iy0 z))
("secretary" nil (s eh1 k r ax t eh1 r iy0))
("secrete" nil (s ih0 k r iy1 t))
("secreted" nil (s ax k r iy1 t ax d))
("secretion" nil (s ax k r iy1 sh ax n))
("secretions" nil (s ax k r iy1 sh ax n z))
("secretive" nil (s iy1 k r ax t ih0 v))
("secretiveness" nil (s iy1 k r ax t ih0 v n ax s))
("secretly" nil (s iy1 k r ih0 t l iy0))
("secrets" nil (s iy1 k r ax t s))
("secrist" nil (s eh1 k er0 ih0 s t))
("sect" nil (s eh1 k t))
("sectarian" nil (s eh0 k t eh1 r iy0 ax n))
("sectarianism" nil (s eh0 k t eh1 r iy0 ax n ih1 z ax m))
("secteur" nil (s eh0 k t uw1 r))
("section" nil (s eh1 k sh ax n))
("sectional" nil (s eh1 k sh ax n ax l))
("sectioned" nil (s eh1 k sh ax n d))
("sectioning" nil (s eh1 k sh ax n ih0 ng))
("sections" nil (s eh1 k sh ax n z))
("sector" nil (s eh1 k t er0))
("sectoral" nil (s eh1 k t er0 ax l))
("sectors" nil (s eh1 k t er0 z))
("sects" nil (s eh1 k t s))
("secular" nil (s eh1 k y ax l er0))
("secularism" nil (s eh1 k y ax l er0 ih1 z ax m))
("secularist" nil (s eh1 k y ax l er0 ih0 s t))
("secunda" nil (s ih0 k ah1 n d ax))
("secure" nil (s ih0 k y uh1 r))
("secured" nil (s ih0 k y uh1 r d))
("securely" nil (s ih0 k y uh1 r l iy0))
("secures" nil (s ih0 k y uh1 r z))
("securing" nil (s ih0 k y uh1 r ih0 ng))
("securites" nil (s ih0 k y uh1 r ax t iy0 z))
("securities" nil (s ih0 k y uh1 r ax t iy0 z))
("securitization" nil (s ih0 k y uh1 r ax t ax z ey1 sh ax n))
("securitize" nil (s ih0 k y uh1 r ax t ay1 z))
("securitized" nil (s ih0 k y uh1 r ax t ay1 z d))
("securitizing" nil (s ih0 k y uh1 r ax t ay1 z ih0 ng))
("security" nil (s ih0 k y uh1 r ax t iy0))
("seda" nil (s ey1 d ax))
("sedam" nil (s eh1 d ax m))
("sedan" nil (s ih0 d ae1 n))
("sedano" nil (s ey0 d aa1 n ow0))
("sedans" nil (s ih0 d ae1 n z))
("sedate" nil (s ih0 d ey1 t))
("sedation" nil (s ax d ey1 sh ax n))
("sedative" nil (s eh1 d ax t ih0 v))
("sedberry" nil (s eh1 d b eh1 r iy0))
("sedco" nil (s eh1 d k ow0))
("seddon" nil (s eh1 d ax n))
("sedentary" nil (s eh1 d ax n t eh1 r iy0))
("seder" nil (s iy1 d er0))
("sedge" nil (s eh1 jh))
("sedgley" nil (s eh1 jh l iy0))
("sedgwick" nil (s eh1 jh w ih0 k))
("sedillo" nil (s eh0 d ih1 l ow0))
("sediment" nil (s eh1 d ax m ax n t))
("sedimentary" nil (s eh1 d ax m eh1 n t er0 iy0))
("sedimentation" nil (s eh1 d ax m ax n t ey1 sh ax n))
("sediments" nil (s eh1 d ax m ax n t s))
("sedita" nil (s eh0 d iy1 t ax))
("sedition" nil (s ih0 d ih1 sh ax n))
("seditious" nil (s ih0 d ih1 sh ax s))
("sedivy" nil (s eh1 d ih0 v iy0))
("sedlacek" nil (s eh1 d l ax s ih0 k))
("sedlack" nil (s eh1 d l ax k))
("sedlak" nil (s eh1 d l ax k))
("sedlar" nil (s eh1 d l er0))
("sedler" nil (s eh1 d l er0))
("sedlock" nil (s eh1 d l ax k))
("sedona" nil (s ax d ow1 n ax))
("sedor" nil (s eh0 d ao1 r))
("sedore" nil (s eh0 d ao1 r iy0))
("seduce" nil (s ih0 d uw1 s))
("seduced" nil (s ih0 d uw1 s t))
("seducer" nil (s ih0 d uw1 s er0))
("seducing" nil (s ih0 d uw1 s ih0 ng))
("seduction" nil (s ih0 d ah1 k sh ax n))
("seductive" nil (s ih0 d ah1 k t ih0 v))
("seductively" nil (s ax d ah1 k t ih0 v l iy0))
("sedum" nil (s eh1 d ax m))
("sedums" nil (s eh1 d ax m z))
("sedwick" nil (s eh1 d w ih0 k))
("see" nil (s iy1))
("seebach" nil (s iy1 b aa1 k))
("seebeck" nil (s iy1 b eh1 k))
("seeber" nil (s iy1 b er0))
("seeberger" nil (s iy1 b er0 g er0))
("seebold" nil (s iy1 b ow1 l d))
("seeburger" nil (s iy1 b er0 g er0))
("seed" nil (s iy1 d))
("seedeaters" nil (s iy1 d iy1 t er0 z))
("seeded" nil (s iy1 d ax d))
("seeding" nil (s iy1 d ih0 ng))
("seedling" nil (s iy1 d l ih0 ng))
("seedlings" nil (s iy1 d l ih0 ng z))
("seedorf" nil (s iy1 d ao0 r f))
("seedpod" nil (s iy1 d p aa1 d))
("seeds" nil (s iy1 d z))
("seedsman" nil (s iy1 d z m ax n))
("seedy" nil (s iy1 d iy0))
("seefeld" nil (s iy1 f eh1 l d))
("seefeldt" nil (s iy1 f ih0 l t))
("seegars" nil (s iy1 g er0 z))
("seeger" nil (s iy1 g er0))
("seegers" nil (s iy1 g er0 z))
("seegert" nil (s iy1 g er0 t))
("seegmiller" nil (s iy1 g m ax l er0))
("seehafer" nil (s iy1 hh ax f er0))
("seehusen" nil (s iy1 hh uw0 s ax n))
("seeing" nil (s iy1 ih0 ng))
("seek" nil (s iy1 k))
("seekamp" nil (s iy1 k ae1 m p))
("seeker" nil (s iy1 k er0))
("seekers" nil (s iy1 k er0 z))
("seeking" nil (s iy1 k ih0 ng))
("seekins" nil (s iy1 k ih0 n z))
("seekonk" nil (s iy1 k aa0 ng k))
("seeks" nil (s iy1 k s))
("seel" nil (s iy1 l))
("seelbach" nil (s iy1 l b aa1 k))
("seelert" nil (s iy1 l er0 t))
("seeley" nil (s iy1 l iy0))
("seelig" nil (s iy1 l ih0 g))
("seeling" nil (s iy1 l ih0 ng))
("seelinger" nil (s iy1 l ih0 ng er0))
("seelman" nil (s iy1 l m ax n))
("seely" nil (s iy1 l iy0))
("seelye" nil (s iy1 l ay1))
("seem" nil (s iy1 m))
("seema" nil (s iy1 m aa0))
("seemala" nil (s iy1 m ax l ax))
("seeman" nil (s iy1 m ax n))
("seemann" nil (s iy1 m ax n))
("seemed" nil (s iy1 m d))
("seeming" nil (s iy1 m ih0 ng))
("seemingly" nil (s iy1 m ih0 ng l iy0))
("seems" nil (s iy1 m z))
("seen" nil (s iy1 n))
("seep" nil (s iy1 p))
("seepage" nil (s iy1 p ih0 jh))
("seeped" nil (s iy1 p t))
("seeping" nil (s iy1 p ih0 ng))
("seeps" nil (s iy1 p s))
("seer" nil (s iy1 r))
("seers" nil (s iy1 r z))
("seery" nil (s iy1 r iy0))
("sees" nil (s iy1 z))
("seesaw" nil (s iy1 s ao1))
("seesawed" nil (s iy1 s ao1 d))
("seesawing" nil (s iy1 s ao1 ih0 ng))
("seese" nil (s iy1 z))
("seethe" nil (s iy1 dh))
("seething" nil (s iy1 th ih0 ng))
("seetin" nil (s iy1 t ih0 n))
("seeton" nil (s iy1 t ax n))
("seever" nil (s iy1 v er0))
("seevers" nil (s iy1 v er0 z))
("seewald" nil (s iy1 w ao1 l d))
("sefcik" nil (s eh1 f s ih0 k))
("seff" nil (s eh1 f))
("sefton" nil (s eh1 f t ax n))
("sega" nil (s iy1 g ax))
("segal" nil (s iy1 g ax l))
("segalas" nil (s eh1 g ax l ax s))
("segall" nil (s ey0 g aa1 l))
("segar" nil (s iy1 g er0))
("segarra" nil (s eh0 g aa1 r ax))
("segars" nil (s eh1 g er0 z))
("sege" nil (s eh1 jh))
("segel" nil (s iy1 g ax l))
("seger" nil (s iy1 g er0))
("segers" nil (s iy1 g er0 z))
("segerstrom" nil (s eh1 g er0 s t r ax m))
("segler" nil (s eh1 g l er0))
("segment" n (s eh1 g m ax n t))
("segment" v (s ax g m eh1 n t))
("segmentation" nil (s eh1 g m ax n t ey1 sh ax n))
("segmented" nil (s eh1 g m eh1 n t ih0 d))
("segments" n (s eh1 g m ax n t s))
("segments" v (s ax g m eh1 n t s))
("segner" nil (s eh1 g n er0))
("sego" nil (s iy1 g ow1))
("segovia" nil (s eh0 g ow1 v iy0 ax))
("segraves" nil (s ey0 g r aa1 v eh0 s))
("segregate" nil (s eh1 g r ax g ey1 t))
("segregated" nil (s eh1 g r ax g ey1 t ih0 d))
("segregating" nil (s eh1 g r ih0 g ey1 t ih0 ng))
("segregation" nil (s eh1 g r ax g ey1 sh ax n))
("segregationist" nil (s eh1 g r ax g ey1 sh ax n ih0 s t))
("segrest" nil (s eh1 g er0 ih0 s t))
("segreto" nil (s eh0 g r eh1 t ow0))
("segue" nil (s eh1 g))
("seguin" nil (s ax g iy1 n))
("segundo" nil (s eh1 g uh1 n d ow1))
("segur" nil (s ey0 g uh1 r))
("segura" nil (s ey0 g uh1 r ax))
("seguros" nil (s ey1 g y er1 ow0 z))
("seher" nil (s eh1 hh er0))
("sehnert" nil (s eh1 n er0 t))
("sehorn" nil (s eh1 hh er0 n))
("sehr" nil (s eh1 r))
("seib" nil (s iy1 b))
("seibel" nil (s ay1 b ax l))
("seiber" nil (s ay1 b er0))
("seiberlich" nil (s ay1 b er0 l ih0 k))
("seiberling" nil (s ay1 b er0 l ih0 ng))
("seibert" nil (s ay1 b er0 t))
("seibold" nil (s ay1 b ow1 l d))
("seibu" nil (s ay1 b uw0))
("seid" nil (s ay1 d))
("seide" nil (s ay1 d))
("seidel" nil (s ay1 d ax l))
("seidell" nil (s ay1 d ax l))
("seidelman" nil (s ay1 d ax l m ax n))
("seiden" nil (s ay1 d ax n))
("seidenberg" nil (s ay1 d ax n b er0 g))
("seider" nil (s ay1 d er0))
("seiders" nil (s ay1 d er0 z))
("seidl" nil (s ay1 d ax l))
("seidler" nil (s ay1 d l er0))
("seidman" nil (s ay1 d m ax n))
("seidner" nil (s ay1 d n er0))
("seier" nil (s ay1 er0))
("seif" nil (s iy1 f))
("seife" nil (s iy1 f))
("seifer" nil (s ay1 f er0))
("seifert" nil (s ay1 f er0 t))
("seiffert" nil (s ay1 f er0 t))
("seifried" nil (s ay1 f er0 iy0 d))
("seige" nil (s iy1 jh))
("seigel" nil (s ay1 g ax l))
("seiger" nil (s ay1 g er0))
("seigle" nil (s iy1 g ax l))
("seigler" nil (s ay1 g ax l er0))
("seigniorage" nil (s iy1 g n iy1 er0 ih0 jh))
("seiji" nil (s ey1 jh iy0))
("seiki" nil (s ey1 k iy0))
("seiko" nil (s ey1 k ow0))
("seil" nil (s ay1 l))
("seiler" nil (s ay1 l er0))
("seils" nil (s ay1 l z))
("seim" nil (s ay1 m))
("sein" nil (s ay1 n))
("seine" nil (s ey1 n iy0))
("seinfeld" nil (s ay1 n f eh0 l d))
("seip" nil (s iy1 p))
("seipel" nil (s ay1 p ax l))
("seiple" nil (s iy1 p ax l))
("seipp" nil (s iy1 p))
("seis" nil (s iy1 s))
("seiser" nil (s ay1 s er0))
("seismic" nil (s ay1 z m ih0 k))
("seismologist" nil (s ay1 z m aa1 l ax jh ax s t))
("seismologists" nil (s ay1 z m aa1 l ax jh ax s t s))
("seismology" nil (s ay1 z m aa1 l ax jh iy0))
("seita" nil (s ey1 t ax))
("seitel" nil (s iy1 t eh1 l))
("seiter" nil (s ay1 t er0))
("seith" nil (s iy1 th))
("seither" nil (s ay1 dh er0))
("seitman" nil (s iy1 t m ax n))
("seitter" nil (s ay1 t er0))
("seitz" nil (s ay1 t s))
("seitzinger" nil (s ay1 t z ih0 ng er0))
("seivert" nil (s ay1 v er0 t))
("seiwert" nil (s ay1 w er0 t))
("seix" nil (s iy1 k s))
("seixas" nil (s iy1 k s ax s))
("seiyaku" nil (s ey1 y aa1 k uw1))
("seiyu" nil (s iy1 y uw0))
("seize" nil (s iy1 z))
("seized" nil (s iy1 z d))
("seizes" nil (s iy1 z ih0 z))
("seizing" nil (s iy1 z ih0 ng))
("seizure" nil (s iy1 zh er0))
("seizures" nil (s iy1 zh er0 z))
("sejm" nil (s ey1 m))
("sekerak" nil (s eh1 k er0 ax k))
("seki" nil (s ey1 k iy0))
("sekisui" nil (s ey1 k ih0 s uw1 iy0))
("sekula" nil (s ih0 k uw1 l ax))
("sela" nil (s eh1 l ax))
("selander" nil (s eh1 l ax n d er0))
("selas" nil (s eh1 l ax s))
("selassie" nil (s ax l ae1 s iy0))
("selbe" nil (s eh1 l b))
("selberg" nil (s eh1 l b er0 g))
("selby" nil (s eh1 l b iy0))
("selchow" nil (s eh1 l ch aw0))
("selda" nil (s eh1 l d ax))
("seldane" nil (s eh1 l d ey1 n))
("selden" nil (s eh1 l d ax n))
("selders" nil (s eh1 l d er0 z))
("seldin" nil (s eh1 l d ih0 n))
("seldom" nil (s eh1 l d ax m))
("seldon" nil (s eh1 l d ax n))
("select" nil (s ax l eh1 k t))
("selected" nil (s ax l eh1 k t ax d))
("selecting" nil (s ax l eh1 k t ih0 ng))
("selection" nil (s ax l eh1 k sh ax n))
("selections" nil (s ax l eh1 k sh ax n z))
("selective" nil (s ax l eh1 k t ih0 v))
("selectively" nil (s ax l eh1 k t ih0 v l iy0))
("selectivity" nil (s ih0 l eh1 k t ih1 v ax t iy0))
("selectnet" nil (s ax l eh1 k t n eh1 t))
("selects" nil (s ax l eh1 k t s))
("selene" nil (s ax l iy1 n))
("selenite" nil (s eh1 l ih0 n ay1 t))
("selenium" nil (s ax l iy1 n iy0 ax m))
("seles" nil (s eh1 l eh0 s))
("seley" nil (s iy1 l iy0))
("self" nil (s eh1 l f))
("selfish" nil (s eh1 l f ih0 sh))
("selfishness" nil (s eh1 l f ih0 sh n ax s))
("selfless" nil (s eh1 l f l ax s))
("selfridge" nil (s eh1 l f r ih0 jh))
("selfs" nil (s eh1 l f s))
("selia" nil (s eh1 l iy0 ax))
("selie" nil (s eh1 l iy0))
("selies" nil (s eh1 l iy0 z))
("selig" nil (s eh1 l ih0 g))
("seliga" nil (s eh1 l ih0 g ax))
("seliger" nil (s eh1 l ih0 g er0))
("seligman" nil (s eh1 l ih0 g m ax n))
("seligmann" nil (s eh1 l ih0 g m ax n))
("seligson" nil (s eh1 l ih0 g s ax n))
("selikoff" nil (s eh1 l ih0 k ao1 f))
("selin" nil (s eh1 l ih0 n))
("selina" nil (s ax l iy1 n ax))
("selinda" nil (s eh0 l iy1 n d ax))
("selinger" nil (s eh1 l ih0 ng er0))
("selk" nil (s eh1 l k))
("selke" nil (s eh1 l k))
("selkin" nil (s eh1 l k ih0 n))
("selkirk" nil (s eh1 l k er0 k))
("sell" nil (s eh1 l))
("sella" nil (s eh1 l ax))
("selland" nil (s eh1 l ax n d))
("sellards" nil (s eh1 l er0 d z))
("sellars" nil (s eh1 l er0 z))
("selle" nil (s eh1 l))
("selleck" nil (s eh1 l ih0 k))
("sellen" nil (s eh1 l ax n))
("seller" nil (s eh1 l er0))
("sellers" nil (s eh1 l er0 z))
("selley" nil (s eh1 l iy0))
("sellick" nil (s eh1 l ih0 k))
("sellier" nil (s eh1 l y er0))
("sellin" nil (s eh1 l ih0 n))
("selling" nil (s eh1 l ih0 ng))
("sellinger" nil (s eh1 l ih0 ng er0))
("sellman" nil (s eh1 l m ax n))
("sellmeyer" nil (s eh1 l m ay0 er0))
("sellner" nil (s eh1 l n er0))
("selloff" nil (s eh1 l ao1 f))
("selloffs" nil (s eh1 l ao1 f s))
("sellon" nil (s eh1 l ax n))
("sellout" nil (s eh1 l aw1 t))
("sellouts" nil (s eh1 l aw1 t s))
("sells" nil (s eh1 l z))
("selma" nil (s eh1 l m ax))
("selman" nil (s eh1 l m ax n))
("selmer" nil (s eh1 l m er0))
("selmon" nil (s eh1 l m ax n))
("selner" nil (s eh1 l n er0))
("selover" nil (s eh1 l ax v er0))
("selowsky" nil (s eh0 l aw1 s k iy0))
("selph" nil (s eh1 l f))
("selsor" nil (s eh1 l s er0))
("seltz" nil (s eh1 l t s))
("seltzer" nil (s eh1 l t s er0))
("selva" nil (s ey1 l v ax))
("selvage" nil (s eh1 l v ih0 jh))
("selvaggio" nil (s eh0 l v aa1 jh iy0 ow0))
("selves" nil (s eh1 l v z))
("selvey" nil (s eh1 l v iy0))
("selvidge" nil (s eh1 l v ih0 jh))
("selvig" nil (s eh1 l v ih0 g))
("selway" nil (s eh1 l w ey1))
("selwin" nil (s eh1 l w ih0 n))
("selwitz" nil (s eh1 l w ih0 t s))
("selwyn" nil (s eh1 l w ih0 n))
("selz" nil (s eh1 l z))
("selzer" nil (s eh1 l z er0))
("sema" nil (s iy1 m ax))
("seman" nil (s iy1 m ax n))
("semans" nil (s iy1 m ax n z))
("semantic" nil (s ih0 m ae1 n t ih0 k))
("semantics" nil (s ih0 m ae1 n t ih0 k s))
("sematech" nil (s eh1 m ax t eh1 k))
("sember" nil (s eh1 m b er0))
("semblance" nil (s eh1 m b l ax n s))
("semegran" nil (s eh1 m ax g r ax n))
("semel" nil (s eh1 m ax l))
("semele" nil (s eh1 m ax l iy1))
("semen" nil (s iy1 m ax n))
("semenza" nil (s eh0 m eh1 n z ax))
("semerad" nil (s eh1 m er0 ae0 d))
("semester" nil (s ax m eh1 s t er0))
("semesters" nil (s ax m eh1 s t er0 z))
("semi" nil (s eh1 m iy0))
("semiannual" nil (s eh1 m iy0 ae1 n y ax w ax l))
("semiannually" nil (s eh1 m iy0 ae1 n uw0 ax l iy0))
("semiautomatic" nil (s eh1 m iy0 ao1 t ax m ae1 t ih0 k))
("semicircular" nil (s eh1 m iy0 s er1 k y ax l er0))
("semiclad" nil (s eh1 m iy0 k l ae1 d))
("semiclassical" nil (s eh1 m iy0 k l ae1 s ih0 k ax l))
("semicon" nil (s eh1 m ih0 k aa1 n))
("semiconductor" nil (s eh1 m iy0 k ax n d ah1 k t er0))
("semiconductors" nil (s eh1 m iy0 k ax n d ah1 k t er0 z))
("semicylindrical" nil (s eh1 m iy0 s ax l ih1 n d r ih0 k ax l))
("semidrying" nil (s eh1 m iy0 d r ay1 ih0 ng))
("semien" nil (s eh1 m iy0 n))
("semifinal" nil (s eh1 m iy0 f ay1 n ax l))
("semifinalists" nil (s eh1 m iy0 f ay1 n ax l ih0 s t s))
("semifinals" nil (s eh1 m iy0 f ay1 n ax l z))
("semifinished" nil (s eh1 m iy0 f ih1 n ih0 sh t))
("semigloss" nil (s eh1 m iy0 g l aa1 s))
("semilegendary" nil (s eh1 m iy0 l eh1 jh ax n d eh1 r iy0))
("seminal" nil (s eh1 m ax n ax l))
("seminar" nil (s eh1 m ax n aa1 r))
("seminara" nil (s eh1 m ih0 n aa1 r ax))
("seminarian" nil (s eh1 m ax n eh1 r iy0 ax n))
("seminarians" nil (s eh1 m ax n eh1 r iy0 ax n z))
("seminaries" nil (s eh1 m ax n eh1 r iy0 z))
("seminario" nil (s eh1 m ih0 n eh1 r iy0 ow0))
("seminars" nil (s eh1 m ax n aa1 r z))
("seminary" nil (s eh1 m ax n eh1 r iy0))
("seminole" nil (s eh1 m ih0 n ow1 l))
("semionenkov" nil (s eh1 m iy0 ow0 n eh1 ng k aa0 v))
("semiotics" nil (s iy1 m iy0 aa1 t ih0 k s))
("semipermanent" nil (s eh1 m iy0 p er1 m ax n ax n t))
("semiprecious" nil (s eh1 m iy0 p r eh1 sh ax s))
("semira" nil (s eh0 m ih1 r ax))
("semireligious" nil (s eh1 m iy0 r ih0 l ih1 jh ax s))
("semis" nil (s eh1 m ay0 z))
("semisecrecy" nil (s eh1 m iy0 s iy1 k r ax s iy0))
("semitic" nil (s ax m ih1 t ih0 k))
("semitism" nil (s eh1 m ih0 t ih1 z ax m))
("semitrailer" nil (s eh1 m iy0 t r ey1 l er0))
("semitropical" nil (s eh1 m iy0 t r aa1 p ih0 k ax l))
("semler" nil (s eh1 m l er0))
("semmel" nil (s eh1 m ax l))
("semmens" nil (s eh1 m ax n z))
("semmes" nil (s eh1 m z))
("semmler" nil (s eh1 m l er0))
("semon" nil (s eh1 m ax n))
("semones" nil (s ey0 m ow1 n eh0 s))
("semple" nil (s eh1 m p ax l))
("semrad" nil (s eh1 m r ax d))
("semrau" nil (s eh1 m r aw0))
("semrow" nil (s eh1 m r ow0))
("sen" nil (s eh1 n))
("sena" nil (s eh1 n ax))
("senate" nil (s eh1 n ax t))
("senator" nil (s eh1 n ax t er0))
("senatore" nil (s eh0 n aa0 t ao1 r iy0))
("senatorial" nil (s eh1 n ax t ao1 r iy0 ax l))
("senators" nil (s eh1 n ax t er0 z))
("send" nil (s eh1 n d))
("sendak" nil (s eh1 n d ae0 k))
("sendelbach" nil (s eh1 n d ih0 l b aa0 k))
("sender" nil (s eh1 n d er0))
("sendero" nil (s eh0 n d eh1 r ow0))
("senders" nil (s eh1 n d er0 z))
("sending" nil (s eh1 n d ih0 ng))
("sends" nil (s eh1 n d z))
("seneca" nil (s eh1 n ax k ax))
("senecal" nil (s eh1 n ih0 k ax l))
("senechal" nil (s eh1 n ih0 k ax l))
("seneff" nil (s eh1 n ax f))
("senegal" nil (s eh1 n ax g ao1 l))
("seneker" nil (s eh1 n ax k er0))
("senesac" nil (s eh1 n ih0 s ae0 k))
("senescence" nil (s ax n eh1 s ax n s))
("senese" nil (s eh1 n iy0 z))
("seney" nil (s eh1 n iy0))
("senf" nil (s eh1 n f))
("senff" nil (s eh1 n f))
("senft" nil (s eh1 n f t))
("seng" nil (s eh1 ng))
("senger" nil (s eh1 ng er0))
("sengers" nil (s eh1 ng er0 z))
("senile" nil (s iy1 n ay1 l))
("senility" nil (s ax n ih1 l ax t iy0))
("senior" nil (s iy1 n y er0))
("seniority" nil (s iy0 n y ao1 r ih0 t iy0))
("seniornet" nil (s iy1 n y er0 n eh1 t))
("seniors" nil (s iy1 n y er0 z))
("senk" nil (s eh1 ng k))
("senkbeil" nil (s eh1 ng k b ay1 l))
("senko" nil (s eh1 ng k ow0))
("senn" nil (s eh1 n))
("senna" nil (s eh1 n ax))
("senne" nil (s eh1 n))
("senner" nil (s eh1 n er0))
("sennet" nil (s eh1 n ax t))
("sennett" nil (s eh1 n ih0 t))
("sennott" nil (s eh1 n ax t))
("sens" nil (s eh1 n z))
("sensabaugh" nil (s eh1 n s ax b ao1))
("sensation" nil (s eh0 n s ey1 sh ax n))
("sensational" nil (s eh0 n s ey1 sh ax n ax l))
("sensationalism" nil (s eh0 n s ey1 sh ax n ax l ih1 z ax m))
("sensationalist" nil (s eh0 n s ey1 sh ax n ax l ih0 s t))
("sensationalistic" nil (s eh0 n s ey1 sh ax n ax l ih1 s t ih0 k))
("sensations" nil (s eh0 n s ey1 sh ax n z))
("sense" nil (s eh1 n s))
("sensed" nil (s eh1 n s t))
("senseless" nil (s eh1 n s l ax s))
("sensenbrenner" nil (s eh1 n s ax n b r eh1 n er0))
("senseney" nil (s eh1 n s ih0 n iy0))
("sensenig" nil (s eh1 n s ih0 n ih0 g))
("senses" nil (s eh1 n s ih0 z))
("sensibilities" nil (s eh1 n s ih0 b ih1 l ih0 t iy0 z))
("sensibility" nil (s eh1 n s ih0 b ih1 l ih0 t iy0))
("sensible" nil (s eh1 n s ax b ax l))
("sensibly" nil (s eh1 n s ax b l iy0))
("sensing" nil (s eh1 n s ih0 ng))
("sensitive" nil (s eh1 n s ax t ih0 v))
("sensitively" nil (s eh1 n s ih0 t ih0 v l iy0))
("sensitiveness" nil (s eh1 n s ax t ih0 v n ax s))
("sensitivities" nil (s eh1 n s ih0 t ih1 v ih0 t iy0 z))
("sensitivity" nil (s eh1 n s ih0 t ih1 v ih0 t iy0))
("sensitized" nil (s eh1 n s ax t ay1 z d))
("senske" nil (s eh1 n s k))
("sensor" nil (s eh1 n s er0))
("sensormatic" nil (s eh1 n s er0 m ae1 t ih0 k))
("sensors" nil (s eh1 n s er0 z))
("sensory" nil (s eh1 n s er0 iy0))
("sensual" nil (s eh1 n ch ax w ax l))
("sensuality" nil (s eh1 n ch ax w ae1 l ax t iy0))
("sensuous" nil (s eh1 n ch ax w ax s))
("sent" nil (s eh1 n t))
("sentell" nil (s eh1 n t ax l))
("sentelle" nil (s eh0 n t eh1 l))
("sentence" nil (s eh1 n t ax n s))
("sentenced" nil (s eh1 n t ax n s t))
("sentences" nil (s eh1 n t ax n s ax z))
("sentencing" nil (s eh1 n t ax n s ih0 ng))
("senter" nil (s eh1 n t er0))
("senters" nil (s eh1 n t er0 z))
("sentiment" nil (s eh1 n t ax m ax n t))
("sentimental" nil (s eh1 n t ax m eh1 n t ax l))
("sentimentality" nil (s eh1 n t ax m eh0 n t ae1 l ih0 t iy0))
("sentiments" nil (s eh1 n t ax m ax n t s))
("sentinel" nil (s eh1 n t ax n ax l))
("sentinels" nil (s eh1 n t ax n ax l z))
("sentman" nil (s eh1 n t m ax n))
("sentra" nil (s eh1 n t r ax))
("sentras" nil (s eh1 n t r ax s))
("sentries" nil (s eh1 n t r iy0 z))
("sentry" nil (s eh1 n t r iy0))
("sentz" nil (s eh1 n t s))
("senzaki" nil (s ey0 n z aa1 k iy0))
("seo" nil (s iy1 ow0))
("seoul" nil (s ow1 l))
("seow" nil (s iy1 ow0))
("separate" v (s eh1 p er0 ey1 t))
("separate" n (s eh1 p r ax t))
("separated" nil (s eh1 p er0 ey1 t ax d))
("separately" nil (s eh1 p er0 ax t l iy0))
("separateness" nil (s eh1 p er0 ax t n ax s))
("separates" v (s eh1 p er0 ey1 t s))
("separates" n (s eh1 p er0 ih0 t s))
("separating" nil (s eh1 p er0 ey1 t ih0 ng))
("separation" nil (s eh1 p er0 ey1 sh ax n))
("separations" nil (s eh1 p er0 ey1 sh ax n z))
("separatism" nil (s eh1 p er0 ax t ih1 z ax m))
("separatist" nil (s eh1 p er0 ax t ih0 s t))
("separatists" nil (s eh1 p er0 ax t ih0 s t s))
("sepe" nil (s iy1 p))
("sepeda" nil (s ey0 p ey1 d ax))
("sephardic" nil (s ax f aa1 r d ih0 k))
("sephardim" nil (s ih0 f aa1 r d ih0 m))
("sepich" nil (s eh1 p ih0 ch))
("sepik" nil (s eh1 p ih0 k))
("sepp" nil (s eh1 p))
("seppala" nil (s eh0 p aa1 l ax))
("seppi" nil (s eh1 p iy0))
("sepsis" nil (s eh1 p s ih0 s))
("sept" nil (s eh1 p t))
("septa" nil (s eh1 p t ax))
("september" nil (s eh0 p t eh1 m b er0))
("septer" nil (s eh1 p t er0))
("septic" nil (s eh1 p t ih0 k))
("septima" nil (s eh0 p t iy1 m ax))
("septimus" nil (s eh1 p t ih0 m ih0 s))
("septuagenarian" nil (s eh0 p ch uw1 ax jh ax n eh1 r iy0 ax n))
("sepulvado" nil (s ey0 p uw0 l v aa1 d ow0))
("sepulveda" nil (s ey0 p uw0 l v ey1 d ax))
("sequa" nil (s eh1 k w ax))
("sequeira" nil (s ey0 k w eh1 r ax))
("sequel" nil (s iy1 k w ax l))
("sequels" nil (s iy1 k w ax l z))
("sequence" nil (s iy1 k w ax n s))
("sequences" nil (s iy1 k w ax n s ax z))
("sequencing" nil (s iy1 k w ax n s ih0 ng))
("sequent" nil (s iy1 k w ax n t))
("sequential" nil (s ax k w eh1 n ch ax l))
("sequentially" nil (s ax k w eh1 n ch ax l iy0))
("sequester" nil (s ih0 k w eh1 s t er0))
("sequestered" nil (s ih0 k w eh1 s t er0 d))
("sequestering" nil (s ih0 k w eh1 s t er0 ih0 ng))
("sequesters" nil (s ih0 k w eh1 s t er0 z))
("sequestration" nil (s eh1 k w ax s t r ey1 sh ax n))
("sequin" nil (s iy1 k w ax n))
("sequined" nil (s iy1 k w ax n d))
("sequitur" nil (s eh1 k w ih0 t er0))
("sequoia" nil (s ih0 k w oy1 ax))
("sequoias" nil (s ih0 k w oy1 ax z))
("sequoyah" nil (s ax k w oy1 ax))
("sera" nil (s ih1 r ax))
("serafin" nil (s eh1 r ax f ih0 n))
("serafina" nil (s er0 aa0 f iy1 n ax))
("serafine" nil (s er0 aa0 f iy1 n iy0))
("serafini" nil (s er0 aa0 f iy1 n iy0))
("serafino" nil (s er0 aa0 f iy1 n ow0))
("seraphina" nil (s er0 aa0 f iy1 n ax))
("seraphine" nil (s er0 aa0 f iy1 n iy0))
("seratti" nil (s eh0 r aa1 t iy0))
("serb" nil (s er1 b))
("serbia" nil (s er1 b iy0 ax))
("serbian" nil (s er1 b iy0 ax n))
("serbians" nil (s er1 b iy0 ax n z))
("serbias" nil (s er1 b iy0 ax z))
("serbin" nil (s er1 b ih0 n))
("serbs" nil (s er1 b z))
("serco" nil (s er1 k ow0))
("serda" nil (s eh1 r d ax))
("sere" nil (s ih1 r))
("serenade" nil (s eh1 r ax n ey1 d))
("serenaded" nil (s eh1 r ax n ey1 d ih0 d))
("serendipitous" nil (s eh1 r ax n d ih1 p ih0 t ax s))
("serendipity" nil (s eh1 r ax n d ih1 p ih0 t iy0))
("serene" nil (s er0 iy1 n))
("serenely" nil (s er0 iy1 n ax l iy0))
("serengeti" nil (s eh1 r ax n g eh1 t iy0))
("serenity" nil (s er0 eh1 n ax t iy0))
("sereno" nil (s eh0 r ey1 n ow0))
("seres" nil (s iy1 r z))
("serex" nil (s eh1 r ax k s))
("serf" nil (s er1 f))
("serfass" nil (s er1 f ae1 s))
("serfin" nil (s er1 f ih0 n))
("serfling" nil (s er1 f l ih0 ng))
("serfs" nil (s er1 f s))
("serge" nil (s er1 jh))
("sergeant" nil (s aa1 r jh ax n t))
("sergeants" nil (s aa1 r jh ax n t s))
("sergei" nil (s er0 g ey1))
("sergent" nil (s er1 jh ax n t))
("sergi" nil (s eh1 r jh iy0))
("sergio" nil (s er1 jh iy0 ow0))
("sergius" nil (s er1 jh iy0 ax s))
("seria" nil (s iy1 r iy0 ax))
("serial" nil (s ih1 r iy0 ax l))
("serialized" nil (s ih1 r iy0 ax l ay1 z d))
("serials" nil (s ih1 r iy0 ax l z))
("series" nil (s ih1 r iy0 z))
("serilda" nil (s er0 iy1 l d ax))
("serino" nil (s er0 iy1 n ow0))
("serio" nil (s iy1 r iy0 ow0))
("serious" nil (s ih1 r iy0 ax s))
("seriously" nil (s ih1 r iy0 ax s l iy0))
("seriousness" nil (s ih1 r iy0 ax s n ax s))
("serle" nil (s ao1 r ax l))
("serlin" nil (s er1 l ih0 n))
("serlo" nil (s eh1 r l ow0))
("sermersheim" nil (s er1 m er0 s hh ay0 m))
("sermon" nil (s er1 m ax n))
("sermons" nil (s er1 m ax n z))
("serna" nil (s eh1 r n ax))
("seroka" nil (s eh0 r ow1 k ax))
("serology" nil (s eh0 r aa1 l ax jh iy0))
("serono" nil (s eh1 r aa1 n ow0))
("serota" nil (s er0 ow1 t ax))
("serotonin" nil (s er0 ow1 t ax n ih0 n))
("serpa" nil (s eh1 r p ax))
("serpas" nil (s er1 p ax z))
("serpe" nil (s er1 p))
("serpent" nil (s er1 p ax n t))
("serpentine" nil (s er1 p ax n t ay1 n))
("serpents" nil (s er1 p ax n t s))
("serpico" nil (s er0 p iy1 k ow0))
("serr" nil (s eh1 r))
("serra" nil (s eh1 r ax))
("serrano" nil (s er0 aa1 n ow0))
("serrao" nil (s eh1 r aw0))
("serrated" nil (s eh1 r ey1 t ih0 d))
("serrato" nil (s er0 aa1 t ow0))
("serratore" nil (s er0 aa0 t ao1 r iy0))
("serres" nil (s eh1 r eh0 s))
("serritella" nil (s eh1 r ih0 t eh1 l ax))
("sertich" nil (s er1 t ih0 k))
("serum" nil (s ih1 r ax m))
("serv" nil (s er1 v))
("servaas" nil (s er1 v aa0 s))
("servais" nil (s er0 v ey1))
("servam" nil (s er1 v ax m))
("servan" nil (s er1 v ax n))
("servant" nil (s er1 v ax n t))
("servants" nil (s er1 v ax n t s))
("serve" nil (s er1 v))
("served" nil (s er1 v d))
("servello" nil (s er0 v eh1 l ow0))
("server" nil (s er1 v er0))
("servers" nil (s er1 v er0 z))
("serves" nil (s er1 v z))
("service" nil (s er1 v ax s))
("serviceable" nil (s er1 v ax s ax b ax l))
("serviced" nil (s er1 v ih0 s t))
("serviceman" nil (s er1 v ax s m ae1 n))
("servicemaster" nil (s er1 v ih0 s m ae1 s t er0))
("servicemen" nil (s er1 v ax s m eh0 n))
("servicer" nil (s er1 v ih0 s er0))
("services" nil (s er1 v ax s ax z))
("servicing" nil (s er1 v ih0 s ih0 ng))
("servico" nil (s er1 v ih0 k ow1))
("servidio" nil (s er0 v iy1 d iy0 ow0))
("servile" nil (s er1 v ax l))
("servin" nil (s er1 v ih0 n))
("serving" nil (s er1 v ih0 ng))
("servings" nil (s er1 v ih0 ng z))
("serviou" nil (s er1 v iy0 uw0))
("servis" nil (s er1 v ih0 s))
("serviss" nil (s er1 v ih0 s))
("servitude" nil (s er1 v ax t uw1 d))
("servo" nil (s er1 v ow0))
("sesame" nil (s eh1 s ax m iy0))
("sesdaq" nil (s eh1 s d ae0 k))
("sesit" nil (s eh1 z ih0 t))
("seskar" nil (s eh1 s k aa1 r))
("sesler" nil (s eh1 s ax l er0))
("sesno" nil (s eh1 s n ow1))
("sessa" nil (s eh1 s ax))
("session" nil (s eh1 sh ax n))
("sessions" nil (s eh1 sh ax n z))
("sessler" nil (s eh1 s l er0))
("sessoms" nil (s eh1 s ax m z))
("sessums" nil (s eh1 s ax m z))
("sestak" nil (s eh1 s t ax k))
("sestanovich" nil (s eh0 s t ae1 n ax v ih0 ch))
("sestito" nil (s eh0 s t iy1 t ow0))
("set" nil (s eh1 t))
("setaro" nil (s eh0 t aa1 r ow0))
("setback" nil (s eh1 t b ae1 k))
("setbacks" nil (s eh1 t b ae1 k s))
("seth" nil (s eh1 th))
("sether" nil (s eh1 dh er0))
("sethi" nil (s eh1 th iy0))
("sethness" nil (s eh1 th n ax s))
("seti" nil (s eh1 t iy1))
("setliff" nil (s eh1 t l ih0 f))
("seto" nil (s eh1 t ow0))
("seton" nil (s iy1 t ax n))
("sets" nil (s eh1 t s))
("setser" nil (s eh1 t s er0))
("settanni" nil (s eh0 t aa1 n iy0))
("sette" nil (s eh1 t))
("setter" nil (s eh1 t er0))
("setterlund" nil (s eh1 t er0 l ax n d))
("setters" nil (s eh1 t er0 z))
("setting" nil (s eh1 t ih0 ng))
("settings" nil (s eh1 t ih0 ng z))
("settle" nil (s eh1 t ax l))
("settled" nil (s eh1 t ax l d))
("settlement" nil (s eh1 t ax l m ax n t))
("settlements" nil (s eh1 t ax l m ax n t s))
("settlemire" nil (s eh1 t l ih0 m ay1 r))
("settlemyre" nil (s eh1 t l ih0 m ay1 r))
("settler" nil (s eh1 t ax l er0))
("settlers" nil (s eh1 t l er0 z))
("settles" nil (s eh1 t ax l z))
("settling" nil (s eh1 t ax l ih0 ng))
("settsu" nil (s eh1 t s uw0))
("setty" nil (s eh1 t iy0))
("setup" nil (s eh1 t ah1 p))
("setups" nil (s eh1 t ah1 p s))
("setzer" nil (s eh1 t z er0))
("setzler" nil (s eh1 t s l er0))
("seubert" nil (s uw1 b er0 t))
("seufert" nil (s uw1 f er0 t))
("sevcik" nil (s eh1 v s ih0 k))
("seve" nil (s eh1 v ey0))
("seven" nil (s eh1 v ax n))
("sevenfold" nil (s eh1 v ax n f ow1 l d))
("sevens" nil (s eh1 v ax n z))
("seventeen" nil (s eh1 v ax n t iy1 n))
("seventeens" nil (s eh1 v ax n t iy1 n z))
("seventeenth" nil (s eh1 v ax n t iy1 n th))
("seventh" nil (s eh1 v ax n th))
("sevenths" nil (s eh1 v ax n th s))
("seventies" nil (s eh1 v ax n t iy0 z))
("seventieth" nil (s eh1 v ax n t iy0 ih0 th))
("seventy" nil (s eh1 v ax n t iy0))
("sever" nil (s eh1 v er0))
("severa" nil (s ey0 v eh1 r ax))
("several" nil (s eh1 v r ax l))
("severally" nil (s eh1 v r ax l iy0))
("severance" nil (s eh1 v er0 ax n s))
("severe" nil (s ax v ih1 r))
("severed" nil (s eh1 v er0 d))
("severely" nil (s ax v ih1 r l iy0))
("severest" nil (s ax v ih1 r ax s t))
("severin" nil (s eh1 v er0 ih0 n))
("severing" nil (s eh1 v er0 ih0 ng))
("severini" nil (s eh0 v er0 iy1 n iy0))
("severino" nil (s eh0 v er0 iy1 n ow0))
("severity" nil (s ih0 v eh1 r ih0 t iy0))
("severn" nil (s eh1 v er0 n))
("severns" nil (s eh1 v er0 n z))
("severs" nil (s eh1 v er0 z))
("severson" nil (s eh1 v er0 s ax n))
("severt" nil (s eh1 v er0 t))
("severtson" nil (s eh1 v er0 t s ax n))
("sevey" nil (s ih0 v ey1))
("sevick" nil (s eh1 v ih0 k))
("sevier" nil (s eh1 v y er0))
("sevigny" nil (s ih0 v ih1 g n iy0))
("sevilla" nil (s eh0 v ih1 l ax))
("seville" nil (s ax v ih1 l))
("sevin" nil (s eh1 v ih0 n))
("sevruga" nil (s eh1 v r uw1 g ax))
("sevy" nil (s iy1 v iy0))
("sew" nil (s ow1))
("sewage" nil (s uw1 ax jh))
("sewald" nil (s uw1 ax l d))
("sewall" nil (s uw1 ax l))
("seward" nil (s uw1 er0 d))
("sewed" nil (s ow1 d))
("sewell" nil (s uw1 ax l))
("sewer" nil (s uw1 er0))
("sewerage" nil (s uw1 er0 ih0 jh))
("sewers" nil (s uw1 er0 z))
("sewing" nil (s ow1 ih0 ng))
("sewn" nil (s ow1 n))
("sex" nil (s eh1 k s))
("sexauer" nil (s eh1 k s aw0 er0))
("sexes" nil (s eh1 k s ax z))
("sexier" nil (s eh1 k s iy0 er0))
("sexiest" nil (s eh1 k s iy0 ax s t))
("sexism" nil (s eh1 k s ih0 z ax m))
("sexist" nil (s eh1 k s ih0 s t))
("sexson" nil (s eh1 k s ax n))
("sextet" nil (s eh0 k s t eh1 t))
("sexton" nil (s eh1 k s t ax n))
("sextus" nil (s eh1 k s t ax s))
("sexual" nil (s eh1 k sh uw0 ax l))
("sexuality" nil (s eh1 k sh uw0 ae1 l ax t iy0))
("sexually" nil (s eh1 k sh uw0 ax l iy0))
("sexy" nil (s eh1 k s iy0))
("seybert" nil (s ey1 b er0 t))
("seybold" nil (s ey1 b ow1 l d))
("seydel" nil (s ey1 d ax l))
("seydoux" nil (s ey1 d uw1))
("seyer" nil (s ey1 er0))
("seyfarth" nil (s ey1 f aa1 r th))
("seyfert" nil (s ey1 f er0 t))
("seyfried" nil (s ey1 f er0 iy0 d))
("seyler" nil (s ey1 l er0))
("seyller" nil (s ey1 l er0))
("seymore" nil (s iy1 m ao0 r))
("seymour" nil (s iy1 m ao0 r))
("sfernice" nil (s f er1 n ih0 s))
("sferra" nil (s f eh1 r ax))
("sferrazza" nil (s f er0 aa1 t s ax))
("sffed" nil (s f eh1 d))
("sforza" nil (s f ao1 r z ax))
("sfuzzi" nil (s f uw1 t z iy0))
("sgambati" nil (s k aa0 m b aa1 t iy0))
("sgro" nil (s k r ow1))
("sgroi" nil (s k r oy1))
("sh" nil (eh1 s ey1 ch))
("shaak" nil (sh aa1 k))
("shaanxi" nil (sh aa1 ng k s iy0))
("shaath" nil (sh aa1 th))
("shabazz" nil (sh aa1 b aa0 z))
("shabby" nil (sh ae1 b iy0))
("shaber" nil (sh ey1 b er0))
("shack" nil (sh ae1 k))
("shackelford" nil (sh ae1 k ih0 l f er0 d))
("shackelton" nil (sh ax k eh1 l t ax n))
("shackett" nil (sh ae1 k ih0 t))
("shackle" nil (sh ae1 k ax l))
("shackled" nil (sh ae1 k ax l d))
("shackleford" nil (sh ae1 k l ih0 f er0 d))
("shackles" nil (sh ae1 k ax l z))
("shackleton" nil (sh ae1 k ax l t ax n))
("shacklett" nil (sh ae1 k l ih0 t))
("shacks" nil (sh ae1 k s))
("shad" nil (sh ae1 d))
("shadbush" nil (sh ae1 d b uh1 sh))
("shadd" nil (sh ae1 d))
("shadden" nil (sh ae1 d ax n))
("shaddix" nil (sh ae1 d ih0 k s))
("shaddock" nil (sh ae1 d ax k))
("shadduck" nil (sh ae1 d ax k))
("shade" nil (sh ey1 d))
("shaded" nil (sh ey1 d ih0 d))
("shadel" nil (sh ae1 d ax l))
("shader" nil (sh ey1 d er0))
("shades" nil (sh ey1 d z))
("shadid" nil (sh ae1 d ih0 d))
("shading" nil (sh ey1 d ih0 ng))
("shadings" nil (sh ey1 d ih0 ng z))
("shadle" nil (sh ey1 d ax l))
("shadley" nil (sh ae1 d l iy0))
("shadoan" nil (sh ae1 d ow0 n))
("shadow" nil (sh ae1 d ow1))
("shadowed" nil (sh ae1 d ow0 d))
("shadowens" nil (sh ae1 d ow0 ax n z))
("shadowing" nil (sh ae1 d ow0 ih0 ng))
("shadowland" nil (sh ae1 d ow0 l ae1 n d))
("shadowlands" nil (sh ae1 d ow0 l ae1 n d z))
("shadows" nil (sh ae1 d ow1 z))
("shadowy" nil (sh ae1 d ow0 iy0))
("shadrick" nil (sh ae1 d r ih0 k))
("shadur" nil (sh ae1 d er0))
("shadwell" nil (sh ae1 d w eh0 l))
("shadwick" nil (sh ae1 d w ih0 k))
("shady" nil (sh ey1 d iy0))
("shaefer" nil (sh ey1 f er0))
("shaeffer" nil (sh eh1 f er0))
("shaer" nil (sh ey1 er0))
("shafer" nil (sh ey1 f er0))
("shaff" nil (sh ae1 f))
("shaffer" nil (sh ey1 f er0))
("shaffner" nil (sh ae1 f n er0))
("shafran" nil (sh ae1 f r ax n))
("shaft" nil (sh ae1 f t))
("shafted" nil (sh ae1 f t ih0 d))
("shafts" nil (sh ae1 f t s))
("shag" nil (sh ae1 g))
("shaggy" nil (sh ae1 g iy0))
("shah" nil (sh aa1))
("shahan" nil (sh ae1 hh ax n))
("shaheen" nil (sh ax hh iy1 n))
("shahin" nil (sh ae1 hh ih0 n))
("shahinian" nil (sh ax hh ih1 n iy0 ax n))
("shahrokh" nil (sh aa1 r aa0 k))
("shahrzad" nil (sh aa1 r z ae1 d))
("shaiken" nil (sh ey1 k ax n))
("shaikh" nil (sh ey1 k))
("shain" nil (sh ey1 n))
("shaine" nil (sh ey1 n))
("shaka" nil (sh aa1 k ax))
("shake" nil (sh ey1 k))
("shakedown" nil (sh ey1 k d aw1 n))
("shakedowns" nil (sh ey1 k d aw1 n z))
("shaken" nil (sh ey1 k ax n))
("shakeout" nil (sh ey1 k aw1 t))
("shaker" nil (sh ey1 k er0))
("shakers" nil (sh ey1 k er0 z))
("shakes" nil (sh ey1 k s))
("shakespeare" nil (sh ey1 k s p iy1 r))
("shakespearean" nil (sh ey1 k s p ih1 r iy0 ax n))
("shakeup" nil (sh ey1 k ah1 p))
("shakeups" nil (sh ey1 k ah1 p s))
("shakier" nil (sh ey1 k iy0 er0))
("shakiest" nil (sh ey1 k iy0 ih0 s t))
("shakiness" nil (sh ey1 k iy0 n ax s))
("shaking" nil (sh ey1 k ih0 ng))
("shakir" nil (sh ae1 k ih0 r))
("shaklee" nil (sh ae1 k l iy0))
("shakunaga" nil (sh aa1 k uw0 n aa1 g ax))
("shakur" nil (sh aa1 k er0))
("shaky" nil (sh ey1 k iy0))
("shalala" nil (sh ax l ey1 l ax))
("shale" nil (sh ey1 l))
("shaler" nil (sh ey1 l er0))
("shales" nil (sh ey1 l z))
("shalikashvili" nil (sh ae1 l iy0 k aa0 sh v iy1 l iy0))
("shall" nil (sh ae1 l))
("shallcross" nil (sh ae1 l k r aa1 s))
("shallenberger" nil (sh ao1 l ax n b er0 g er0))
("shallow" nil (sh ae1 l ow0))
("shallower" nil (sh ae1 l ow0 er0))
("shallowness" nil (sh ae1 l ow0 n ax s))
("shalom" nil (sh ax l ow1 m))
("shalt" nil (sh ae1 l t))
("sham" nil (sh ae1 m))
("shaman" nil (sh ey1 m ax n))
("shamans" nil (sh ey1 m ax n z))
("shambaugh" nil (sh ae1 m b ao1))
("shamberger" nil (sh ae1 m b er0 g er0))
("shambles" nil (sh ae1 m b ax l z))
("shambley" nil (sh ae1 m b l iy0))
("shamblin" nil (sh ae1 m b l ih0 n))
("shambo" nil (sh ae1 m b ow0))
("shamburger" nil (sh ae1 m b er0 g er0))
("shame" nil (sh ey1 m))
("shamed" nil (sh ey1 m d))
("shameful" nil (sh ey1 m f ax l))
("shamel" nil (sh ae1 m ax l))
("shameless" nil (sh ey1 m l ax s))
("shamelessly" nil (sh ey1 m l ax s l iy0))
("shames" nil (sh ey1 m z))
("shamir" nil (sh ax m ih1 r))
("shamp" nil (sh ae1 m p))
("shampine" nil (sh ae1 m p ay1 n))
("shampoo" nil (sh ae0 m p uw1))
("shampooed" nil (sh ae0 m p uw1 d))
("shampoos" nil (sh ae0 m p uw1 z))
("shamrock" nil (sh ae1 m r aa1 k))
("shamu" nil (sh aa0 m uw1))
("shamus" nil (sh ey1 m ax s))
("shan" nil (sh ae1 n))
("shanafelt" nil (sh ae1 n ax f eh1 l t))
("shanahan" nil (sh ae1 n ax hh ae0 n))
("shand" nil (sh ae1 n d))
("shandling" nil (sh ae1 n d l ih0 ng))
("shandong" nil (sh aa1 n d ow1 ng))
("shands" nil (sh ae1 n d z))
("shandwick" nil (sh ae1 n d w ih1 k))
("shandy" nil (sh ae1 n d iy0))
("shane" nil (sh ey1 n))
("shaner" nil (sh ey1 n er0))
("shaneyfelt" nil (sh ae1 n iy0 f eh0 l t))
("shanghai" nil (sh ae1 ng hh ay1))
("shangkun" nil (sh ae1 ng k ax n))
("shangri" nil (sh ae1 ng g r iy0))
("shanholtz" nil (sh ae1 n hh ow1 l t s))
("shanholtzer" nil (sh ae1 n hh ow0 l t z er0))
("shank" nil (sh ae1 ng k))
("shankel" nil (sh ae1 ng k ax l))
("shanken" nil (sh ae1 ng k ax n))
("shanker" nil (sh ae1 ng k er0))
("shankland" nil (sh ae1 ng k l ax n d))
("shankle" nil (sh ae1 ng k ax l))
("shankles" nil (sh ae1 ng k ax l z))
("shanklin" nil (sh ae1 ng k l ih0 n))
("shankman" nil (sh ae1 ng k m ax n))
("shanks" nil (sh ae1 ng k s))
("shanley" nil (sh ae1 n l iy0))
("shannahan" nil (sh ae1 n ax hh ae0 n))
("shannon" nil (sh ae1 n ax n))
("shanties" nil (sh ae1 n t iy0 z))
("shanty" nil (sh ae1 n t iy0))
("shantytown" nil (sh ae1 n t iy0 t aw1 n))
("shantytowns" nil (sh ae1 n t iy0 t aw1 n z))
("shantz" nil (sh ae1 n t s))
("shao" nil (sh aw1))
("shape" nil (sh ey1 p))
("shaped" nil (sh ey1 p t))
("shapeless" nil (sh ey1 p l ax s))
("shapely" nil (sh ey1 p l iy0))
("shapero" nil (sh ax p ih1 r ow0))
("shapes" nil (sh ey1 p s))
("shaping" nil (sh ey1 p ih0 ng))
("shapira" nil (sh ax p ih1 r ax))
("shapiro" nil (sh ax p ih1 r ow0))
("shapiros" nil (sh ax p ih1 r ow0 z))
("shapley" nil (sh ae1 p l iy0))
("shappell" nil (sh ae1 p ax l))
("shaq" nil (sh ae1 k))
("shaquille" nil (sh ax k iy1 l))
("sharansky" nil (sh er0 ae1 n s k iy0))
("sharar" nil (sh er0 aa1 r))
("sharber" nil (sh aa1 r b er0))
("sharbono" nil (sh aa0 r b ow1 n ow0))
("shard" nil (sh aa1 r d))
("shards" nil (sh aa1 r d z))
("share" nil (sh eh1 r))
("sharecroppers" nil (sh eh1 r k r aa1 p er0 z))
("shared" nil (sh eh1 r d))
("shareholder" nil (sh eh1 r hh ow1 l d er0))
("shareholders" nil (sh eh1 r hh ow1 l d er0 z))
("shareholding" nil (sh eh1 r hh ow1 l d ih0 ng))
("shareholdings" nil (sh eh1 r hh ow1 l d ih0 ng z))
("shareowner" nil (sh eh1 r ow1 n er0))
("shareowners" nil (sh eh1 r ow1 n er0 z))
("sharer" nil (sh eh1 r er0))
("shares" nil (sh eh1 r z))
("shareware" nil (sh eh1 r w eh1 r))
("sharf" nil (sh aa1 r f))
("sharfman" nil (sh aa1 r f m ax n))
("shari" nil (sh aa1 r iy0))
("sharia" nil (sh eh1 r iy0 ax))
("sharif" nil (sh er0 iy1 f))
("sharing" nil (sh eh1 r ih0 ng))
("shark" nil (sh aa1 r k))
("sharkey" nil (sh aa1 r k iy0))
("sharking" nil (sh aa1 r k ih0 ng))
("sharklike" nil (sh aa1 r k l ay1 k))
("sharks" nil (sh aa1 r k s))
("sharleen" nil (sh aa0 r l iy1 n))
("sharlene" nil (sh aa1 r l iy1 n))
("sharline" nil (sh aa1 r l ay1 n))
("sharlow" nil (sh aa1 r l ow0))
("sharma" nil (sh aa1 r m ax))
("sharman" nil (sh aa1 r m ax n))
("sharon" nil (sh ae1 r ax n))
("sharp" nil (sh aa1 r p))
("sharpe" nil (sh aa1 r p))
("sharpen" nil (sh aa1 r p ax n))
("sharpened" nil (sh aa1 r p ax n d))
("sharpening" nil (sh aa1 r p ax n ih0 ng))
("sharpens" nil (sh aa1 r p ax n z))
("sharper" nil (sh aa1 r p er0))
("sharpest" nil (sh aa1 r p ax s t))
("sharpeville" nil (sh aa1 r p ax v ih1 l))
("sharples" nil (sh aa1 r p ax l z))
("sharpless" nil (sh aa1 r p l ax s))
("sharpley" nil (sh aa1 r p l iy0))
("sharply" nil (sh aa1 r p l iy0))
("sharpnack" nil (sh aa1 r p n ax k))
("sharpness" nil (sh aa1 r p n ax s))
("sharps" nil (sh aa1 r p s))
("sharpshooter" nil (sh aa1 r p sh uw1 t er0))
("sharpton" nil (sh aa1 r p t ax n))
("sharrar" nil (sh er0 aa1 r))
("sharrer" nil (sh aa1 r er0))
("sharrett" nil (sh ae1 r ih0 t))
("sharrock" nil (sh ae1 r ax k))
("sharron" nil (sh ae1 r ax n))
("sharrow" nil (sh ae1 r ow0))
("sharry" nil (sh aa1 r iy0))
("shartzer" nil (sh aa1 r t z er0))
("sharum" nil (sh er0 ah1 m))
("shashlik" nil (sh ae1 sh l ih0 k))
("shashoua" nil (sh ax sh uw1 ax))
("shasta" nil (sh ae1 s t ax))
("shasteen" nil (sh ax s t iy1 n))
("shatkin" nil (sh ae1 t k ih0 n))
("shatley" nil (sh ae1 t l iy0))
("shatrov" nil (sh ae1 t r aa0 v))
("shatt" nil (sh ae1 t))
("shatter" nil (sh ae1 t er0))
("shattered" nil (sh ae1 t er0 d))
("shattering" nil (sh ae1 t er0 ih0 ng))
("shatterproof" nil (sh ae1 t er0 p r uw1 f))
("shatters" nil (sh ae1 t er0 z))
("shattuck" nil (sh ae1 t ax k))
("shatz" nil (sh ae1 t s))
("shatzer" nil (sh ey1 t z er0))
("shaub" nil (sh ao1 b))
("shauger" nil (sh aw1 g er0))
("shaughnessy" nil (sh ao1 n ih0 s iy0))
("shaul" nil (sh ao1 l))
("shaulis" nil (sh aw1 l ih0 s))
("shaull" nil (sh ao1 l))
("shaun" nil (sh ao1 n))
("shave" nil (sh ey1 v))
("shaved" nil (sh ey1 v d))
("shaven" nil (sh ey1 v ax n))
("shaver" nil (sh ey1 v er0))
("shavers" nil (sh ey1 v er0 z))
("shaves" nil (sh ey1 v z))
("shaving" nil (sh ey1 v ih0 ng))
("shavings" nil (sh ey1 v ih0 ng z))
("shaw" nil (sh ao1))
("shawcross" nil (sh ao1 k r ao1 s))
("shawgo" nil (sh ao1 g ow1))
("shawhan" nil (sh ae1 w ax n))
("shawinigan" nil (sh ax w ih1 n ih0 g ax n))
("shawl" nil (sh ao1 l))
("shawler" nil (sh ao1 l er0))
("shawley" nil (sh ao1 l iy0))
("shawls" nil (sh ao1 l z))
("shawmut" nil (sh ao1 m ax t))
("shawn" nil (sh ao1 n))
("shawnee" nil (sh ao1 n iy0))
("shawnut" nil (sh ao1 n ax t))
("shawver" nil (sh ao1 v er0))
("shay" nil (sh ey1))
("shaykin" nil (sh ey1 k ih0 n))
("shayne" nil (sh ey1 n))
("shcharansky" nil (sh er0 ae1 n s k iy0))
("shchedrin" nil (sh eh1 d r ih0 n))
("shcherbitsky" nil (sh er0 b ih1 t s k iy0))
("she" nil (sh iy1))
("shea" nil (sh ey1))
("shead" nil (s hh eh1 d))
("sheaf" nil (sh iy1 f))
("sheaffer" nil (sh iy1 f er0))
("sheahan" nil (sh iy1 ax n))
("shealey" nil (sh iy1 l iy0))
("shealy" nil (sh iy1 l iy0))
("shean" nil (sh iy1 n))
("shear" nil (sh ih1 r))
("sheard" nil (sh ih1 r d))
("sheared" nil (sh ih1 r d))
("shearer" nil (sh ih1 r er0))
("shearin" nil (sh ih1 r ih0 n))
("shearing" nil (sh ih1 r ih0 ng))
("shearman" nil (sh iy1 r m ax n))
("shearn" nil (sh ih1 r n))
("shearon" nil (sh ih1 r ax n))
("shearouse" nil (sh ih1 r aw0 s))
("shears" nil (sh iy1 r z))
("shearson" nil (sh ih1 r s ax n))
("sheath" nil (sh iy1 th))
("sheathe" nil (sh iy1 dh))
("sheathed" nil (sh iy1 dh d))
("sheathing" nil (sh iy1 dh ih0 ng))
("sheats" nil (sh iy1 t s))
("sheaves" nil (sh iy1 v z))
("sheba" nil (sh iy1 b ax))
("sheboygan" nil (sh ax b oy1 g ax n))
("sheck" nil (sh eh1 k))
("sheckler" nil (sh eh1 k l er0))
("shed" nil (sh eh1 d))
("shedd" nil (sh eh1 d))
("shedden" nil (sh eh1 d ax n))
("shedding" nil (sh eh1 d ih0 ng))
("shedlock" nil (sh eh1 d l aa1 k))
("shedrick" nil (sh eh1 d r ih0 k))
("sheds" nil (sh eh1 d z))
("sheeder" nil (sh iy1 d er0))
("sheedy" nil (sh iy1 d iy0))
("sheehan" nil (sh iy1 ax n))
("sheehy" nil (sh iy1 hh iy0))
("sheek" nil (sh iy1 k))
("sheeks" nil (sh iy1 k s))
("sheela" nil (sh iy1 l ax))
("sheelah" nil (sh iy1 l ax))
("sheelen" nil (sh iy1 l ax n))
("sheeler" nil (sh iy1 l er0))
("sheeley" nil (sh iy1 l iy0))
("sheely" nil (sh iy1 l iy0))
("sheen" nil (sh iy1 n))
("sheena" nil (sh iy1 n ax))
("sheep" nil (sh iy1 p))
("sheepish" nil (sh iy1 p ih0 sh))
("sheepishly" nil (sh iy1 p ih0 sh l iy0))
("sheeps" nil (sh iy1 p s))
("sheepskin" nil (sh iy1 p s k ih1 n))
("sheer" nil (sh ih1 r))
("sheeran" nil (sh ih1 r ax n))
("sheerer" nil (sh iy1 r er0))
("sheerin" nil (sh ih1 r ih0 n))
("sheesley" nil (sh iy1 z l iy0))
("sheet" nil (sh iy1 t))
("sheeting" nil (sh iy1 t ih0 ng))
("sheets" nil (sh iy1 t s))
("sheetz" nil (sh iy1 t s))
("sheff" nil (sh eh1 f))
("sheffer" nil (sh eh1 f er0))
("sheffey" nil (sh eh1 f iy0))
("sheffield" nil (sh eh1 f iy0 l d))
("sheffler" nil (sh eh1 f l er0))
("sheftel" nil (sh eh1 f t eh1 l))
("shegog" nil (sh eh1 g aa0 g))
("shehan" nil (sh eh1 hh ax n))
("shehane" nil (sh eh1 hh ax n))
("shehorn" nil (sh eh1 hh er0 n))
("sheik" nil (sh iy1 k))
("sheikdom" nil (sh iy1 k d ax m))
("sheikdoms" nil (sh iy1 k d ax m z))
("sheikh" nil (sh iy1 k))
("sheiks" nil (sh iy1 k s))
("sheil" nil (sh ay1 l))
("sheila" nil (sh iy1 l ax))
("sheilah" nil (sh iy1 l ax))
("sheilds" nil (sh ay1 l d z))
("sheils" nil (sh ay1 l z))
("shein" nil (sh ay1 n))
("sheinberg" nil (sh ay1 n b er0 g))
("shek" nil (sh eh1 k))
("shekel" nil (sh eh1 k ax l))
("shekels" nil (sh eh1 k ax l z))
("shelburne" nil (sh eh1 l b er0 n))
("shelby" nil (sh eh1 l b iy0))
("shelbyville" nil (sh eh1 l b iy0 v ih1 l))
("sheldahl" nil (sh eh1 l d aa1 l))
("shelden" nil (sh eh1 l d ax n))
("sheldon" nil (sh eh1 l d ax n))
("sheley" nil (sh iy1 l iy0))
("shelf" nil (sh eh1 l f))
("shelhamer" nil (sh eh1 l hh ax m er0))
("sheline" nil (sh eh1 l ay0 n))
("shell" nil (sh eh1 l))
("shellabarger" nil (sh eh1 l ax b aa1 r g er0))
("shelled" nil (sh eh1 l d))
("shellenbarger" nil (sh eh1 l ih0 n b aa0 r g er0))
("shellenberger" nil (sh eh1 l ax n b er0 g er0))
("sheller" nil (sh eh1 l er0))
("shelley" nil (sh eh1 l iy0))
("shellfish" nil (sh eh1 l f ih1 sh))
("shellhammer" nil (sh eh1 l hh ae1 m er0))
("shellhorn" nil (sh eh1 l hh er0 n))
("shelling" nil (sh eh1 l ih0 ng))
("shellito" nil (sh ey0 l iy1 t ow0))
("shellman" nil (sh eh1 l m ax n))
("shells" nil (sh eh1 l z))
("shellshocked" nil (sh eh1 l sh aa1 k t))
("shelly" nil (sh eh1 l iy0))
("shelman" nil (sh eh1 l m ax n))
("shelnutt" nil (sh eh1 l n ax t))
("shelor" nil (sh eh1 l er0))
("shelp" nil (sh eh1 l p))
("shelstad" nil (sh eh1 l s t ax d))
("shelter" nil (sh eh1 l t er0))
("sheltered" nil (sh eh1 l t er0 d))
("sheltering" nil (sh eh1 l t er0 ih0 ng))
("shelters" nil (sh eh1 l t er0 z))
("shelton" nil (sh eh1 l t ax n))
("shelve" nil (sh eh1 l v))
("shelved" nil (sh eh1 l v d))
("shelves" nil (sh eh1 l v z))
("shelving" nil (sh eh1 l v ih0 ng))
("shemanski" nil (sh ih0 m ae1 n s k iy0))
("shemwell" nil (sh eh1 m w eh1 l))
("shen" nil (sh eh1 n))
("shena" nil (sh iy1 n ax))
("shenandoah" nil (sh eh1 n ax n d ow1 ax))
("shenanigans" nil (sh ax n ae1 n ih0 g ax n z))
("shenberger" nil (sh eh1 n b er0 g er0))
("shenefield" nil (sh eh1 n ih0 f iy1 l d))
("sheneman" nil (sh iy1 n m ax n))
("sheng" nil (sh eh1 ng))
("shenk" nil (sh eh1 ng k))
("shenker" nil (sh eh1 ng k er0))
("shenkman" nil (sh eh1 ng k m ax n))
("shenton" nil (sh eh1 n t ax n))
("shenyang" nil (sh eh0 n y ae1 ng))
("shenzhen" nil (sh eh1 n zh eh1 n))
("shep" nil (sh eh1 p))
("shepard" nil (sh eh1 p er0 d))
("shepardson" nil (sh eh1 p aa0 r d s ax n))
("sheperd" nil (sh eh1 p er0 d))
("shephard" nil (sh eh1 f er0 d))
("shepheard" nil (sh eh1 f er0 d))
("shepherd" nil (sh eh1 p er0 d))
("shepherded" nil (sh eh1 p er0 d ih0 d))
("shepherding" nil (sh eh1 p er0 d ih0 ng))
("shepherds" nil (sh eh1 p er0 d z))
("shepler" nil (sh eh1 p l er0))
("shepley" nil (sh eh1 p l iy0))
("shepp" nil (sh eh1 p))
("sheppard" nil (sh eh1 p er0 d))
("sheppards" nil (sh eh1 p er0 d z))
("shepperd" nil (sh eh1 p er0 d))
("shepperson" nil (sh eh1 p er0 s ax n))
("sheppy" nil (sh eh1 p iy0))
("sher" nil (sh er1))
("sherak" nil (sh eh1 r ae0 k))
("sherard" nil (sh eh1 r er0 d))
("sheraton" nil (sh eh1 r ax t ax n))
("sherbert" nil (sh er1 b er0 t))
("sherbet" nil (sh er1 b ax t))
("sherbondy" nil (sh er0 b aa1 n d iy0))
("sherborne" nil (sh er1 b er0 n))
("sherbourn" nil (sh er0 b uh1 r n))
("sherbourne" nil (sh er0 b uh1 r n))
("sherburn" nil (sh er1 b er0 n))
("sherburne" nil (sh er1 b er0 n))
("shere" nil (sh ih1 r))
("sheree" nil (sh er0 iy1))
("sherer" nil (sh ih1 r er0))
("sherfey" nil (sh er1 f iy0))
("sherfield" nil (sh er1 f iy0 l d))
("sheri" nil (sh eh1 r iy0))
("sherick" nil (sh eh1 r ih0 k))
("sheridan" nil (sh eh1 r ih0 d ax n))
("sheriff" nil (sh eh1 r ax f))
("sheriffs" nil (sh eh1 r ax f s))
("sherin" nil (sh eh1 r ih0 n))
("sherk" nil (sh er1 k))
("sherley" nil (sh er1 l iy0))
("sherlin" nil (sh er1 l ih0 n))
("sherlock" nil (sh er1 l aa1 k))
("sherlund" nil (sh er1 l ax n d))
("sherman" nil (sh er1 m ax n))
("shermer" nil (sh er1 m er0))
("shernoff" nil (sh er1 n ao1 f))
("sherod" nil (sh eh1 r ax d))
("sherr" nil (sh eh1 r))
("sherrard" nil (sh eh1 r er0 d))
("sherrell" nil (sh eh1 r ax l))
("sherrer" nil (sh eh1 r er0))
("sherrgold" nil (sh eh1 r g ow1 l d))
("sherri" nil (sh eh1 r iy0))
("sherrick" nil (sh eh1 r ih0 k))
("sherrie" nil (sh eh1 r iy0))
("sherriff" nil (sh eh1 r ih0 f))
("sherrill" nil (sh eh1 r ih0 l))
("sherrin" nil (sh eh1 r ih0 n))
("sherritt" nil (sh eh1 r ih0 t))
("sherrod" nil (sh eh1 r ax d))
("sherron" nil (sh eh1 r ax n))
("sherrow" nil (sh eh1 r ow0))
("sherry" nil (sh eh1 r iy0))
("shertzer" nil (sh er1 t z er0))
("sherva" nil (sh er1 v ax))
("sherwin" nil (sh er1 w ih0 n))
("sherwood" nil (sh er1 w uh1 d))
("sheryl" nil (sh eh1 r ax l))
("sheshunoff" nil (sh eh1 sh uw0 n ao0 f))
("sheth" nil (sh eh1 th))
("shetland" nil (sh eh1 t l ax n d))
("shetler" nil (sh eh1 t l er0))
("shetley" nil (sh eh1 t l iy0))
("shetter" nil (sh eh1 t er0))
("shetterly" nil (sh eh1 t er0 l iy0))
("shevardnadze" nil (sh eh1 v er0 d n aa1 d z iy0))
("shevlin" nil (sh eh1 v l ih0 n))
("shew" nil (sh uw1))
("sheward" nil (sh uw1 er0 d))
("shewchuk" nil (sh uw1 ch ax k))
("shewell" nil (sh eh1 w eh0 l))
("shewmake" nil (sh uw1 m ey1 k))
("shewmaker" nil (sh uw1 m ey0 k er0))
("shi" nil (sh iy1))
("shibata" nil (sh iy0 b aa1 t ax))
("shibboleth" nil (sh ih1 b ax l eh1 th))
("shibley" nil (sh ih1 b l iy0))
("shick" nil (sh ih1 k))
("shicoff" nil (sh ih1 k ao1 f))
("shideler" nil (sh ih1 d ax l er0))
("shidler" nil (sh ay1 d ax l er0))
("shied" nil (sh ay1 d))
("shieh" nil (sh iy1))
("shiel" nil (sh iy1 l))
("shield" nil (sh iy1 l d))
("shielded" nil (sh iy1 l d ih0 d))
("shielding" nil (sh iy1 l d ih0 ng))
("shields" nil (sh iy1 l d z))
("shiels" nil (sh iy1 l z))
("shier" nil (sh ay1 er0))
("shies" nil (sh ay1 z))
("shiff" nil (sh ih1 f))
("shiffer" nil (sh ih1 f er0))
("shiffler" nil (sh ih1 f l er0))
("shifflet" nil (sh ih1 f l ih0 t))
("shifflett" nil (sh ih1 f l ih0 t))
("shiffman" nil (sh ih1 f m ax n))
("shiflet" nil (sh ih1 f l ih0 t))
("shiflett" nil (sh ih1 f l ih0 t))
("shifrin" nil (sh ih1 f r ih0 n))
("shift" nil (sh ih1 f t))
("shifted" nil (sh ih1 f t ax d))
("shifter" nil (sh ih1 f t er0))
("shifting" nil (sh ih1 f t ih0 ng))
("shifts" nil (sh ih1 f t s))
("shifty" nil (sh ih1 f t iy0))
("shigeki" nil (sh ih0 g ey1 k iy0))
("shigekuni" nil (sh iy1 g ih0 k uw1 n iy0))
("shigeo" nil (sh ih0 g ey1 ow0))
("shigeru" nil (sh ih0 g ey1 r uw0))
("shigley" nil (sh ih1 g l iy0))
("shih" nil (sh iy1))
("shiina" nil (sh iy1 n ax))
("shiite" nil (sh iy1 ay1 t))
("shiites" nil (sh iy1 ay1 t s))
("shiley" nil (sh ih1 l iy0))
("shill" nil (sh ih1 l))
("shiller" nil (sh ih1 l er0))
("shilling" nil (sh ih1 l ih0 ng))
("shillingburg" nil (sh ih1 l ih0 ng b er0 g))
("shillinglaw" nil (sh ih1 l ih0 ng l ao1))
("shillings" nil (sh ih1 l ih0 ng z))
("shillington" nil (sh ih1 l ih0 ng t ax n))
("shiloh" nil (sh ay1 l ow0))
("shilts" nil (sh ih1 l t s))
("shim" nil (sh ih1 m))
("shima" nil (sh iy1 m ax))
("shimabukuro" nil (sh iy0 m aa0 b uw0 k uh1 r ow0))
("shimada" nil (sh iy0 m aa1 d ax))
("shimbun" nil (sh ih1 m b ah1 n))
("shimek" nil (sh ih1 m ih0 k))
("shimel" nil (sh ih1 m ax l))
("shimer" nil (sh ay1 m er0))
("shimizu" nil (sh ih0 m iy1 z uw0))
("shimko" nil (sh ih1 m k ow0))
("shimkus" nil (sh ih1 m k ax s))
("shimmel" nil (sh ih1 m ax l))
("shimmer" nil (sh ih1 m er0))
("shimmered" nil (sh ih1 m er0 d))
("shimmering" nil (sh ih1 m er0 ih0 ng))
("shimmerlik" nil (sh ih1 m er0 l ih0 k))
("shimmers" nil (sh ih1 m er0 z))
("shimmin" nil (sh ih1 m ih0 n))
("shimmy" nil (sh ih1 m iy0))
("shimoda" nil (sh ih0 m ow1 d ax))
("shimoga" nil (sh ih0 m ow1 g ax))
("shimokawa" nil (sh ih1 m ow0 k aa1 w ax))
("shimon" nil (sh iy1 m ow0 n))
("shimp" nil (sh ih1 m p))
("shin" nil (sh ih1 n))
("shinall" nil (sh ih1 n ax l))
("shinault" nil (sh ih1 n aw0 l t))
("shinbein" nil (sh ih1 n b ay1 n))
("shinbone" nil (sh ih1 n b ow1 n))
("shindig" nil (sh ih1 n d ih0 g))
("shindle" nil (sh ih1 n d ax l))
("shindledecker" nil (sh ih1 n d l ih0 d ih0 k er0))
("shindler" nil (sh ih1 n d l er0))
("shindou" nil (sh ih1 n d ow1))
("shine" nil (sh ay1 n))
("shined" nil (sh ay1 n d))
("shiner" nil (sh ay1 n er0))
("shines" nil (sh ay1 n z))
("shing" nil (sh ih1 ng))
("shingle" nil (sh ih1 ng g ax l))
("shingledecker" nil (sh ih1 ng g l ih0 d ih0 k er0))
("shingler" nil (sh ih1 ng l er0))
("shingles" nil (sh ih1 ng g ax l z))
("shingleton" nil (sh ih1 ng g ax l t ax n))
("shinichi" nil (sh ih0 n iy1 ch iy0))
("shining" nil (sh ay1 n ih0 ng))
("shinkle" nil (sh ih1 ng k ax l))
("shinko" nil (sh ih1 ng k ow0))
("shinn" nil (sh ih1 n))
("shinners" nil (sh ih1 n er0 z))
("shinnick" nil (sh ih1 n ih0 k))
("shinsei" nil (sh ih0 n s ey1))
("shinsky" nil (sh ih1 n s k iy0))
("shintaro" nil (sh ih0 n t aa1 r ow0))
("shinto" nil (sh ih1 n t ow1))
("shiny" nil (sh ay1 n iy0))
("shionogi" nil (sh iy1 ax n ow1 g iy0))
("ship" nil (sh ih1 p))
("shipboard" nil (sh ih1 p b ao1 r d))
("shipbuilder" nil (sh ih1 p b ih1 l d er0))
("shipbuilders" nil (sh ih1 p b ih1 l d er0 z))
("shipbuilding" nil (sh ih1 p b ih1 l d ih0 ng))
("shipbuildings" nil (sh ih1 p b ih1 l d ih0 ng z))
("shipe" nil (sh ay1 p))
("shipes" nil (sh ay1 p s))
("shipholding" nil (sh ih1 p hh ow1 l d ih0 ng))
("shiplett" nil (sh ih1 p l ih0 t))
("shipley" nil (sh ih1 p l iy0))
("shipload" nil (sh ih1 p l ow1 d))
("shiploads" nil (sh ih1 p l ow1 d z))
("shipman" nil (sh ih1 p m ax n))
("shipmate" nil (sh ih1 p m ey1 t))
("shipment" nil (sh ih1 p m ax n t))
("shipments" nil (sh ih1 p m ax n t s))
("shipowners" nil (sh ih1 p ow1 n er0 z))
("shipp" nil (sh ih1 p))
("shipped" nil (sh ih1 p t))
("shippee" nil (sh ih1 p iy1))
("shipper" nil (sh ih1 p er0))
("shippers" nil (sh ih1 p er0 z))
("shippey" nil (sh ih1 p iy0))
("shipping" nil (sh ih1 p ih0 ng))
("shippingport" nil (sh ih1 p ih0 ng p ao1 r t))
("shipps" nil (sh ih1 p s))
("shippy" nil (sh ih1 p iy0))
("ships" nil (sh ih1 p s))
("shipshape" nil (sh ih1 p sh ey1 p))
("shipton" nil (sh ih1 p t ax n))
("shipwash" nil (sh ih1 p w aa1 sh))
("shipwrights" nil (sh ih1 p r ay1 t s))
("shipyard" nil (sh ih1 p y aa1 r d))
("shipyards" nil (sh ih1 p y aa1 r d z))
("shir" nil (sh er1))
("shira" nil (sh ih1 r ax))
("shirah" nil (sh ih1 r ax))
("shiraishi" nil (sh ih0 r aa0 iy1 sh iy0))
("shirazi" nil (sh ih0 r aa1 z iy0))
("shire" nil (sh ay1 r))
("shireman" nil (sh ih0 r ey1 m ax n))
("shiremanstown" nil (sh ay1 r m ax n z t aw1 n))
("shirer" nil (sh ay1 er0 r))
("shires" nil (sh ay1 r z))
("shirey" nil (sh ay1 r iy0))
("shirk" nil (sh er1 k))
("shirked" nil (sh er1 k t))
("shirkey" nil (sh er1 k iy0))
("shirking" nil (sh er1 k ih0 ng))
("shirl" nil (sh er1 l))
("shirlee" nil (sh er1 l iy0))
("shirleen" nil (sh er0 l iy1 n))
("shirlene" nil (sh er1 l iy0 n))
("shirley" nil (sh er1 l iy0))
("shirlie" nil (sh er1 l iy0))
("shiroma" nil (sh ih0 r ow1 m ax))
("shirrell" nil (sh ao1 r ax l))
("shirt" nil (sh er1 t))
("shirts" nil (sh er1 t s))
("shirtsleeve" nil (sh er1 t s l iy1 v))
("shiseido" nil (sh ih0 s ey1 d ow0))
("shishido" nil (sh iy0 sh iy1 d ow0))
("shisler" nil (sh ih1 s ax l er0))
("shissler" nil (sh ih1 s l er0))
("shit" nil (sh iy1 t))
("shiu" nil (sh uw1))
("shiva" nil (sh iy1 v ax))
("shive" nil (sh ay1 v))
("shiveley" nil (sh ih1 v ih0 l iy0))
("shively" nil (sh ay1 v l iy0))
("shiver" nil (sh ih1 v er0))
("shiverdecker" nil (sh ih1 v er0 d ih0 k er0))
("shivering" nil (sh ih1 v er0 ih0 ng))
("shivers" nil (sh ih1 v er0 z))
("shives" nil (sh ay1 v z))
("shivley" nil (sh ih1 v l iy0))
("shiyuan" nil (sh iy1 uw0 aa0 n))
("shizuka" nil (sh ih0 z uw1 k ax))
("shizuoka" nil (sh iy0 z w ow1 k ax))
("shlaes" nil (sh l ey1 z))
("shlenker" nil (sh l eh1 ng k er0))
("shoaf" nil (sh ow1 f))
("shoaff" nil (sh ow1 f))
("shoals" nil (sh ow1 l z))
("shobana" nil (sh ow1 b aa1 n ax))
("shobanna" nil (sh ow1 b aa1 n ax))
("shobe" nil (sh ow1 b))
("shober" nil (sh ow1 b er0))
("shoberg" nil (sh ow1 b er0 g))
("shobert" nil (sh aa1 b er0 t))
("shock" nil (sh aa1 k))
("shocked" nil (sh aa1 k t))
("shocker" nil (sh aa1 k er0))
("shockey" nil (sh aa1 k iy0))
("shocking" nil (sh aa1 k ih0 ng))
("shockingly" nil (sh aa1 k ih0 ng l iy0))
("shockley" nil (sh aa1 k l iy0))
("shocks" nil (sh aa1 k s))
("shod" nil (sh aa1 d))
("shoddy" nil (sh aa1 d iy0))
("shoe" nil (sh uw1))
("shoebox" nil (sh ow1 b aa0 k s))
("shoehorn" nil (sh uw1 hh ao1 r n))
("shoehorned" nil (sh uw1 hh ao0 r n d))
("shoelace" nil (sh uw1 l ey1 s))
("shoelaces" nil (sh uw1 l ey1 s ax z))
("shoemake" nil (sh uw1 m ey1 k))
("shoemaker" nil (sh uw1 m ey1 k er0))
("shoemate" nil (sh uw1 m ey1 t))
("shoen" nil (sh uw1 n))
("shoes" nil (sh uw1 z))
("shoeshine" nil (sh uw1 sh ay1 n))
("shoestring" nil (sh uw1 s t r ih1 ng))
("shoff" nil (sh ao1 f))
("shoffner" nil (sh ao1 f n er0))
("shofner" nil (sh aa1 f n er0))
("shogren" nil (sh aa1 g r eh0 n))
("shogun" nil (sh ow1 g ax n))
("shohat" nil (sh ow1 hh ae0 t))
("shoichi" nil (sh ow0 iy1 ch iy0))
("shoichiro" nil (sh ow1 ih0 ch ih1 r ow0))
("shoji" nil (sh ow1 jh iy0))
("shokhin" nil (sh ow1 k ih1 n))
("shoko" nil (sh ow1 k ow0))
("sholar" nil (sh ow1 l er0))
("sholem" nil (sh ow1 l ax m))
("sholes" nil (sh ow1 l z))
("sholl" nil (sh aa1 l))
("shollenberger" nil (sh aa1 l ax n b er0 g er0))
("sholly" nil (sh aa1 l iy0))
("sholtis" nil (sh ow1 l t ih0 s))
("sholto" nil (sh ow1 l t ow0))
("shomaker" nil (sh ow1 m ey1 k er0))
("shomo" nil (sh ow1 m ow0))
("shon" nil (sh aa1 n))
("shone" nil (sh ow1 n))
("shonk" nil (sh aa1 ng k))
("shonka" nil (sh aa1 ng k ax))
("shonkwiler" nil (sh aa1 ng k w ay1 l er0))
("shontz" nil (sh aa1 n t s))
("shoo" nil (sh uw1))
("shoob" nil (sh uw1 b))
("shooed" nil (sh uw1 d))
("shoofly" nil (sh uw1 f l ay1))
("shook" nil (sh uh1 k))
("shoop" nil (sh uw1 p))
("shoopman" nil (sh uw1 p m ax n))
("shooshan" nil (sh uw1 sh ax n))
("shoot" nil (sh uw1 t))
("shooter" nil (sh uw1 t er0))
("shooters" nil (sh uw1 t er0 z))
("shooting" nil (sh uw1 t ih0 ng))
("shootings" nil (sh uw1 t ih0 ng z))
("shootout" nil (sh uw1 t aw0 t))
("shoots" nil (sh uw1 t s))
("shop" nil (sh aa1 p))
("shope" nil (sh ow1 p))
("shopkeeper" nil (sh aa1 p k iy1 p er0))
("shopkeepers" nil (sh aa1 p k iy1 p er0 z))
("shopko" nil (sh aa1 p k ow0))
("shopkorn" nil (sh aa1 p k ao1 r n))
("shoplifting" nil (sh aa1 p l ih1 f t ih0 ng))
("shoppe" nil (sh aa1 p))
("shopped" nil (sh aa1 p t))
("shopper" nil (sh aa1 p er0))
("shoppers" nil (sh aa1 p er0 z))
("shoppes" nil (sh aa1 p s))
("shopping" nil (sh aa1 p ih0 ng))
("shops" nil (sh aa1 p s))
("shoptaw" nil (sh aa1 p t ao0))
("shopwell" nil (sh aa1 p w eh1 l))
("shopworn" nil (sh aa1 p w ao1 r n))
("shor" nil (sh iy0 er1))
("shorb" nil (sh ao1 r b))
("shore" nil (sh ao1 r))
("shorebird" nil (sh ao1 r b er1 d))
("shored" nil (sh ao1 r d))
("shoreham" nil (sh ao1 r ax m))
("shoreline" nil (sh ao1 r l ay1 n))
("shorenstein" nil (sh ao1 r ax n s t iy0 n))
("shores" nil (sh ao1 r z))
("shoreward" nil (sh ao1 r w er0 d))
("shorey" nil (sh ao1 r iy0))
("shorin" nil (sh ao1 r ih0 n))
("shoring" nil (sh ao1 r ih0 ng))
("shorkey" nil (sh ao1 r k iy0))
("shorn" nil (sh ao1 r n))
("shorr" nil (sh ao1 r))
("short" nil (sh ao1 r t))
("shortage" nil (sh ao1 r t ax jh))
("shortages" nil (sh ao1 r t ax jh ax z))
("shortall" nil (sh ao1 r t ax l))
("shortcake" nil (sh ao1 r t k ey1 k))
("shortchange" nil (sh ao1 r t ch ey1 n jh))
("shortchanged" nil (sh ao1 r t ch ey1 n jh d))
("shortchanging" nil (sh ao1 r t ch ey1 n jh ih0 ng))
("shortcoming" nil (sh ao1 r t k ah1 m ih0 ng))
("shortcomings" nil (sh ao1 r t k ah1 m ih0 ng z))
("shortcut" nil (sh ao1 r t k ah1 t))
("shortcuts" nil (sh ao1 r t k ah1 t s))
("shorted" nil (sh ao1 r t ih0 d))
("shortell" nil (sh ao0 r t eh1 l))
("shorten" nil (sh ao1 r t ax n))
("shortened" nil (sh ao1 r t ax n d))
("shortening" nil (sh ao1 r t ax n ih0 ng))
("shortens" nil (sh ao1 r t ax n z))
("shorter" nil (sh ao1 r t er0))
("shortest" nil (sh ao1 r t ih0 s t))
("shortfall" nil (sh ao1 r t f ao1 l))
("shortfalls" nil (sh ao1 r t f ao1 l z))
("shortgrass" nil (sh ao1 r t g r ae1 s))
("shorthair" nil (sh ao1 r t hh eh1 r))
("shorthaired" nil (sh ao1 r t hh eh1 r d))
("shorthand" nil (sh ao1 r t hh ae1 n d))
("shorting" nil (sh ao1 r t ih0 ng))
("shortlived" nil (sh ao1 r t l ih1 v d))
("shortly" nil (sh ao1 r t l iy0))
("shortness" nil (sh ao1 r t n ax s))
("shortridge" nil (sh ao1 r t r ih0 jh))
("shorts" nil (sh ao1 r t s))
("shortsighted" nil (sh ao1 r t s ay1 t ih0 d))
("shortsightedness" nil (sh ao1 r t s ay1 t ih0 d n ih0 s))
("shortstop" nil (sh ao1 r t s t aa1 p))
("shortt" nil (sh ao1 r t))
("shortterm" nil (sh ao1 r t t er1 m))
("shortwave" nil (sh ao1 r t w ey1 v))
("shorty" nil (sh ao1 r t iy0))
("shortz" nil (sh ao1 r t s))
("shoshone" nil (sh ow0 sh ow1 n iy0))
("shostak" nil (sh aa1 s t ax k))
("shostakovich" nil (sh ao1 s t ax k ow1 v ih0 ch))
("shot" nil (sh aa1 t))
("shotgun" nil (sh aa1 t g ah1 n))
("shotguns" nil (sh aa1 t g ah1 n z))
("shots" nil (sh aa1 t s))
("shott" nil (sh aa1 t))
("shotts" nil (sh aa1 t s))
("shotwell" nil (sh aa1 t w eh1 l))
("shougang" nil (sh aw1 g aa1 ng))
("shough" nil (sh aw1))
("should" nil (sh uh1 d))
("shoulder" nil (sh ow1 l d er0))
("shouldered" nil (sh ow1 l d er0 d))
("shouldering" nil (sh ow1 l d er0 ih0 ng))
("shoulders" nil (sh ow1 l d er0 z))
("shoults" nil (sh ow1 l t s))
("shoultz" nil (sh ow1 l t s))
("shoumaker" nil (sh uw1 m ey1 k er0))
("shoun" nil (sh aw1 n))
("shoup" nil (sh uw1 p))
("shoupe" nil (sh uw1 p))
("shouse" nil (s hh aw1 s))
("shout" nil (sh aw1 t))
("shouted" nil (sh aw1 t ax d))
("shouting" nil (sh aw1 t ih0 ng))
("shouts" nil (sh aw1 t s))
("shove" nil (sh ah1 v))
("shoved" nil (sh ah1 v d))
("shovel" nil (sh ah1 v ax l))
("shoveled" nil (sh ah1 v ax l d))
("shoveling" nil (sh ah1 v l ih0 ng))
("shovels" nil (sh ah1 v ax l z))
("shover" nil (sh ah1 v er0))
("shoves" nil (sh ah1 v z))
("shoving" nil (sh ah1 v ih0 ng))
("shovlin" nil (sh aa1 v l ih0 n))
("show" nil (sh ow1))
("showa" nil (sh ow1 ax))
("showalter" nil (sh ow1 ax l t er0))
("showbiz" nil (sh ow1 b ih0 z))
("showboat" nil (sh ow1 b ow1 t))
("showcase" nil (sh ow1 k ey1 s))
("showcased" nil (sh ow1 k ey1 s t))
("showcases" nil (sh ow1 k ey1 s ih0 z))
("showcasing" nil (sh ow1 k ey0 s ih0 ng))
("showdown" nil (sh ow1 d aw1 n))
("showed" nil (sh ow1 d))
("showell" nil (sh aa1 w eh0 l))
("shower" nil (sh aw1 er0))
("showered" nil (sh aw1 er0 d))
("showering" nil (sh aw1 er0 ih0 ng))
("showers" nil (sh aw1 er0 z))
("showgirl" nil (sh ow1 g er1 l))
("showiest" nil (sh ow1 iy0 ax s t))
("showing" nil (sh ow1 ih0 ng))
("showings" nil (sh ow1 ih0 ng z))
("showman" nil (sh ow1 m ax n))
("showmanship" nil (sh ow1 m ax n sh ih1 p))
("shown" nil (sh ow1 n))
("showpiece" nil (sh ow1 p iy1 s))
("showplace" nil (sh ow1 p l ey1 s))
("showroom" nil (sh ow1 r uh1 m))
("showrooms" nil (sh ow1 r uw1 m z))
("shows" nil (sh ow1 z))
("showscan" nil (sh ow1 s k ae1 n))
("showtime" nil (sh ow1 t ay1 m))
("showy" nil (sh ow1 iy0))
("shrader" nil (sh r ey1 d er0))
("shrake" nil (sh r ey1 k))
("shramek" nil (sh r ae1 m ih0 k))
("shrank" nil (sh r ae1 ng k))
("shrapnel" nil (sh r ae1 p n ax l))
("shreck" nil (sh r eh1 k))
("shred" nil (sh r eh1 d))
("shredded" nil (sh r eh1 d ax d))
("shredder" nil (sh r eh1 d er0))
("shredders" nil (sh r eh1 d er0 z))
("shredding" nil (sh r eh1 d ih0 ng))
("shreds" nil (sh r eh1 d z))
("shreeve" nil (sh r iy1 v))
("shreffler" nil (sh r eh1 f l er0))
("shreiber" nil (sh r ay1 b er0))
("shreiner" nil (sh r ay1 n er0))
("shreve" nil (sh r iy1 v))
("shreveport" nil (sh r iy1 v p ao1 r t))
("shreves" nil (sh r iy1 v z))
("shrewd" nil (sh r uw1 d))
("shrewdest" nil (sh r uw1 d ax s t))
("shrewdly" nil (sh r uw1 d l iy0))
("shrewdness" nil (sh r uw1 d n ax s))
("shrewsberry" nil (sh r uw1 z b eh1 r iy0))
("shrewsbury" nil (sh r uw1 z b eh1 r iy0))
("shri" nil (sh r iy1))
("shriber" nil (sh r ay1 b er0))
("shribman" nil (sh r ih1 b m ax n))
("shrider" nil (sh r ay1 d er0))
("shriek" nil (sh r iy1 k))
("shrieked" nil (sh r iy1 k t))
("shrieking" nil (sh r iy1 k ih0 ng))
("shrieks" nil (sh r iy1 k s))
("shrift" nil (sh r ih1 f t))
("shrikelike" nil (sh r ay1 k l ay1 k))
("shrill" nil (sh r ih1 l))
("shrimp" nil (sh r ih1 m p))
("shrine" nil (sh r ay1 n))
("shriner" nil (sh r ay1 n er0))
("shriners" nil (sh r ay1 n er0 z))
("shrines" nil (sh r ay1 n z))
("shrink" nil (sh r ih1 ng k))
("shrinkage" nil (sh r ih1 ng k ih0 jh))
("shrinking" nil (sh r ih1 ng k ih0 ng))
("shrinks" nil (sh r ih1 ng k s))
("shrivel" nil (sh r ih1 v ax l))
("shriveled" nil (sh r ih1 v ax l d))
("shriveling" nil (sh r ih1 v ax l ih0 ng))
("shriver" nil (sh r ay1 v er0))
("shrock" nil (sh r aa1 k))
("shrode" nil (sh r ow1 d))
("shroff" nil (sh r ao1 f))
("shrontz" nil (sh r aa1 n t s))
("shropshire" nil (sh r aa1 p sh ay1 r))
("shroud" nil (sh r aw1 d))
("shrouded" nil (sh r aw1 d ih0 d))
("shrouding" nil (sh r aw1 d ih0 ng))
("shrout" nil (sh r aw1 t))
("shroyer" nil (sh r oy1 er0))
("shrub" nil (sh r ah1 b))
("shrubbery" nil (sh r ah1 b er0 iy0))
("shrubby" nil (sh r ah1 b iy0))
("shrublike" nil (sh r ah1 b l ay1 k))
("shrubs" nil (sh r ah1 b z))
("shrug" nil (sh r ah1 g))
("shrugged" nil (sh r ah1 g d))
("shrugging" nil (sh r ah1 g ih0 ng))
("shrugs" nil (sh r ah1 g z))
("shrum" nil (sh r ah1 m))
("shrunk" nil (sh r ah1 ng k))
("shrunken" nil (sh r ah1 ng k ax n))
("shryock" nil (sh r ay1 aa0 k))
("shtick" nil (sh t ih1 k))
("shu" nil (sh uw1))
("shuart" nil (sh uw1 er0 t))
("shubert" nil (sh uw1 b er0 t))
("shubin" nil (sh uw1 b ih0 n))
("shuchman" nil (sh ah1 k m ax n))
("shuck" nil (sh ah1 k))
("shucked" nil (sh ah1 k t))
("shucking" nil (sh ah1 k ih0 ng))
("shucks" nil (sh ah1 k s))
("shuda" nil (sh uw1 d ax))
("shudder" nil (sh ah1 d er0))
("shuddering" nil (sh ah1 d er0 ih0 ng))
("shudders" nil (sh ah1 d er0 z))
("shue" nil (sh uw1))
("shuey" nil (sh uw1 iy0))
("shufelt" nil (sh uw1 f eh0 l t))
("shuff" nil (sh ah1 f))
("shuffield" nil (sh ah1 f iy1 l d))
("shuffle" nil (sh ah1 f ax l))
("shuffled" nil (sh ah1 f ax l d))
("shuffler" nil (sh ah1 f ax l er0))
("shufflers" nil (sh ah1 f ax l er0 z))
("shuffles" nil (sh ah1 f ax l z))
("shuffling" nil (sh ah1 f l ih0 ng))
("shuford" nil (sh uw1 f er0 d))
("shufro" nil (sh ah1 f r ow0))
("shugars" nil (sh uw1 g er0 z))
("shugart" nil (sh ah1 g aa0 r t))
("shughart" nil (sh ah1 g hh aa1 r t))
("shugrue" nil (sh ah1 g r uw0))
("shui" nil (sh uw1 iy0))
("shukla" nil (sh ah1 k l ax))
("shular" nil (sh uw1 l er0))
("shuler" nil (sh uw1 l er0))
("shull" nil (sh ah1 l))
("shulman" nil (sh uh1 l m ax n))
("shultis" nil (sh ah1 l t ih0 s))
("shults" nil (sh ah1 l t s))
("shultz" nil (sh uh1 l t s))
("shum" nil (sh ah1 m))
("shumake" nil (sh uw1 m ey1 k))
("shumaker" nil (sh uw1 m ey1 k er0))
("shuman" nil (sh uw1 m ax n))
("shumard" nil (sh uw1 m er0 d))
("shumate" nil (sh uw1 m ey1 t))
("shumeet" nil (sh uw0 m iy1 t))
("shumpert" nil (sh ah1 m p er0 t))
("shumsky" nil (sh ah1 m s k iy0))
("shumway" nil (sh ah1 m w ey1))
("shun" nil (sh ah1 n))
("shunk" nil (sh ah1 ng k))
("shunned" nil (sh ah1 n d))
("shunning" nil (sh ah1 n ih0 ng))
("shuns" nil (sh ah1 n z))
("shunted" nil (sh ah1 n t ih0 d))
("shunting" nil (sh ah1 n t ih0 ng))
("shunto" nil (sh ah1 n t ow0))
("shupe" nil (sh uw1 p))
("shupert" nil (sh uw1 p er0 t))
("shuping" nil (sh uw1 p ih0 ng))
("shupp" nil (sh ah1 p))
("shur" nil (sh er1))
("shure" nil (sh uh1 r))
("shurgard" nil (sh uh1 r g aa1 r d))
("shurley" nil (sh er1 l iy0))
("shurr" nil (sh er1))
("shurtleff" nil (sh er1 t l ih0 f))
("shurtliff" nil (sh er1 t l ih0 f))
("shurtz" nil (sh er1 t s))
("shuster" nil (sh ah1 s t er0))
("shusterman" nil (sh ah1 s t er0 m ax n))
("shut" nil (sh ah1 t))
("shutdown" nil (sh ah1 t d aw1 n))
("shutdowns" nil (sh ah1 t d aw1 n z))
("shute" nil (sh uw1 t))
("shutes" nil (sh uw1 t s))
("shutout" nil (sh ah1 t aw1 t))
("shuts" nil (sh ah1 t s))
("shutt" nil (sh ah1 t))
("shutter" nil (sh ah1 t er0))
("shuttered" nil (sh ah1 t er0 d))
("shuttering" nil (sh ah1 t er0 ih0 ng))
("shutters" nil (sh ah1 t er0 z))
("shutting" nil (sh ah1 t ih0 ng))
("shuttle" nil (sh ah1 t ax l))
("shuttled" nil (sh ah1 t ax l d))
("shuttles" nil (sh ah1 t ax l z))
("shuttlesworth" nil (sh ah1 t ax l z w er1 th))
("shuttleworth" nil (sh ah1 t ax l w er1 th))
("shuttling" nil (sh ah1 t ax l ih0 ng))
("shutts" nil (sh ah1 t s))
("shuwa" nil (sh uw1 w ax))
("shy" nil (sh ay1))
("shying" nil (sh ay1 ih0 ng))
("shylock" nil (sh ay1 l aa1 k))
("shyly" nil (sh ay1 l iy0))
("shymanski" nil (sh ax m ae1 n s k iy0))
("shyne" nil (sh ay1 n))
("shyness" nil (sh ay1 n ax s))
("shysters" nil (sh ay1 s t er0 z))
("si" nil (s iy1))
("siam" nil (s ay0 ae1 m))
("siamese" nil (s ay1 ax m iy1 z))
("siang" nil (s y ae1 ng))
("siano" nil (s iy0 aa1 n ow0))
("sias" nil (zh ao1 z))
("sib" nil (s ih1 b))
("sibbie" nil (s ih1 b iy0))
("sibby" nil (s ih1 b iy0))
("sibelle" nil (s ih0 b eh1 l))
("siberia" nil (s ay0 b ih1 r iy0 ax))
("siberian" nil (s ay0 b ih1 r iy0 ax n))
("sibert" nil (s ih1 b er0 t))
("sibil" nil (s ih1 b ax l))
("sibilia" nil (s iy0 b iy1 l iy0 ax))
("sibilla" nil (s ih0 b ih1 l ax))
("sibille" nil (s ih1 b ih0 l))
("sible" nil (s ay1 b ax l))
("sibley" nil (s ih1 b l iy0))
("sibling" nil (s ih1 b l ih0 ng))
("siblings" nil (s ih1 b l ih0 ng z))
("sibson" nil (s ih1 b s ax n))
("sibyl" nil (s ih1 b ax l))
("sibyll" nil (s ih1 b ih0 l))
("sic" nil (s ih1 k))
("sica" nil (s iy1 k ax))
("sicard" nil (s ih1 k er0 d))
("sichel" nil (s ih1 k ax l))
("sichuan" nil (s ih1 ch w aa1 n))
("sicilia" nil (s ih0 s ih1 l y ax))
("sicilian" nil (s ih0 s ih1 l iy0 ax n))
("siciliano" nil (s iy0 ch iy0 l iy0 aa1 n ow0))
("sicily" nil (s ih1 s ax l iy0))
("sick" nil (s ih1 k))
("sickel" nil (s ih1 k ax l))
("sickels" nil (s ih1 k ax l z))
("sickened" nil (s ih1 k ax n d))
("sickening" nil (s ih1 k ax n ih0 ng))
("sicker" nil (s ih1 k er0))
("sickest" nil (s ih1 k ax s t))
("sickinger" nil (s ih1 k ih0 ng er0))
("sickle" nil (s ih1 k ax l))
("sickler" nil (s ih1 k l er0))
("sickles" nil (s ih1 k ax l z))
("sickly" nil (s ih1 k l iy0))
("sickman" nil (s ih1 k m ax n))
("sickness" nil (s ih1 k n ax s))
("siconolfi" nil (s iy0 k ow0 n ow1 l f iy0))
("sicotte" nil (s ih0 k ao1 t))
("sid" nil (s ih1 d))
("sidak" nil (s ih1 d ae0 k))
("siddall" nil (s ih1 d ax l))
("siddeley" nil (s ih1 d ax l iy0))
("siddell" nil (s ih1 d ax l))
("sidden" nil (s ih1 d ax n))
("siddens" nil (s ih1 d ax n z))
("siddhartha" nil (s ih0 d aa1 r t ah1))
("siddiqi" nil (s iy0 d iy1 k iy0))
("siddiqui" nil (s iy0 d iy1 k w iy0))
("siddle" nil (s ih1 d ax l))
("siddons" nil (s ih1 d ax n z))
("side" nil (s ay1 d))
("sided" nil (s ay1 d ax d))
("sidek" nil (s ay1 d eh1 k))
("sidekick" nil (s ay1 d k ih1 k))
("sideline" nil (s ay1 d l ay1 n))
("sidelined" nil (s ay1 d l ay1 n d))
("sidelines" nil (s ay1 d l ay1 n z))
("sidell" nil (s ay1 d ax l))
("sideman" nil (s ay1 d m ae1 n))
("sidener" nil (s ih1 d iy0 n er0))
("sider" nil (s ay1 d er0))
("siderca" nil (s ih0 d er1 k ax))
("sideris" nil (s ih1 d er0 ih0 s))
("siderographers" nil (s ay1 d er0 ao1 g r ax f er0 z))
("siders" nil (s ay1 d er0 z))
("sides" nil (s ay1 d z))
("sideshow" nil (s ay1 d sh ow1))
("sidestep" nil (s ay1 d s t eh1 p))
("sidestepped" nil (s ay1 d s t eh1 p t))
("sidestepping" nil (s ay1 d s t eh1 p ih0 ng))
("sidesteps" nil (s ay1 d s t eh1 p s))
("sidestream" nil (s ay1 d s t r iy1 m))
("sidetrack" nil (s ay1 d t r ae1 k))
("sidetracked" nil (s ay1 d t r ae1 k t))
("sidewalk" nil (s ay1 d w ao1 k))
("sidewalks" nil (s ay1 d w ao1 k s))
("sidewater" nil (s ay1 d w ao1 t er0))
("sideways" nil (s ay1 d w ey1 z))
("sidewinder" nil (s ay1 d w ay1 n d er0))
("sidewise" nil (s ay1 d w ay1 z))
("sidhu" nil (s ih1 d hh uw0))
("siding" nil (s ay1 d ih0 ng))
("sidle" nil (s ay1 d ax l))
("sidler" nil (s ay1 d ax l er0))
("sidley" nil (s ih1 d l iy0))
("sidman" nil (s ih1 d m ax n))
("sidney" nil (s ih1 d n iy0))
("sidon" nil (s ay1 d ax n))
("sidonia" nil (s iy0 d ow1 n iy0 ax))
("sidor" nil (s ih1 d er0))
("sidoti" nil (s iy0 d ow1 t iy0))
("sidra" nil (s ih1 d r ax))
("sids" nil (s ih1 d z))
("sidwell" nil (s ih1 d w eh1 l))
("sie" nil (s iy1))
("siebe" nil (s iy1 b))
("siebel" nil (s iy1 b ax l))
("siebels" nil (s iy1 b ax l z))
("sieben" nil (s iy1 b ax n))
("siebenaler" nil (s iy1 b ih0 n ax l er0))
("sieber" nil (s iy1 b er0))
("siebers" nil (s iy1 b er0 z))
("siebert" nil (s iy1 b er0 t))
("siebold" nil (s iy1 b ow1 l d))
("siebrecht" nil (s iy1 b r ih0 k t))
("sieck" nil (s iy1 k))
("siecle" nil (s iy1 k ax l))
("siedenburg" nil (s iy1 d ax n b er0 g))
("siedlecki" nil (s iy0 d l eh1 t s k iy0))
("siedschlag" nil (s iy1 d sh l ax g))
("siefert" nil (s iy1 f er0 t))
("siefken" nil (s iy1 f k ax n))
("siefker" nil (s iy1 f k er0))
("sieg" nil (s iy1 g))
("siegal" nil (s iy1 g ax l))
("siegan" nil (s iy1 g ax n))
("siege" nil (s iy1 jh))
("siegecraft" nil (s iy1 jh k r ae1 f t))
("siegel" nil (s iy1 g ax l))
("siegell" nil (s iy1 g ax l))
("siegelman" nil (s iy1 g ax l m ax n))
("siegenthaler" nil (s iy1 g ih0 n th ax l er0))
("sieger" nil (s iy1 g er0))
("siegert" nil (s iy1 g er0 t))
("siegfried" nil (s iy1 g f r iy1 d))
("siegle" nil (s iy1 g ax l))
("siegler" nil (s iy1 g l er0))
("siegman" nil (s iy1 g m ax n))
("siegmann" nil (s iy1 g m ax n))
("siegmund" nil (s iy1 g m ax n d))
("siegrist" nil (s iy1 g r ih0 s t))
("sieh" nil (s iy1))
("sieja" nil (s ey1 ax))
("siek" nil (s iy1 k))
("siekierski" nil (s iy0 k ih1 r s k iy0))
("siekman" nil (s iy1 k m ax n))
("sielaff" nil (s iy0 l ae1 f))
("sieler" nil (s iy1 l er0))
("sieling" nil (s iy1 l ih0 ng))
("sieloff" nil (s iy1 l ao0 f))
("sielski" nil (s iy1 l s k iy0))
("siem" nil (s iy1 m))
("siemel" nil (s iy1 m ax l))
("siemens" nil (s iy1 m ax n z))
("siemer" nil (s iy1 m er0))
("siemers" nil (s iy1 m er0 z))
("sieminski" nil (s iy0 m ih1 n s k iy0))
("siemon" nil (s iy1 m ax n))
("siems" nil (s iy1 m z))
("siemsen" nil (s iy1 m s ax n))
("sienko" nil (s iy0 eh1 ng k ow0))
("sieracki" nil (s ih0 r aa1 t s k iy0))
("sierchio" nil (s iy1 r ch iy0 ow0))
("sierra" nil (s iy0 eh1 r ax))
("sierracin" nil (s iy0 eh1 r ax s ih0 n))
("sierras" nil (s iy0 eh1 r ax z))
("siers" nil (s iy1 r z))
("sies" nil (s iy1 z))
("siess" nil (s iy1 s))
("sietsema" nil (s iy0 t s iy1 m ax))
("sieve" nil (s ih1 v))
("siever" nil (s iy1 v er0))
("sievers" nil (s iy1 v er0 z))
("sievert" nil (s iy1 v er0 t))
("sieves" nil (s ih1 v z))
("sieving" nil (s ih1 v ih0 ng))
("siew" nil (s uw1))
("siewert" nil (s iy1 w er0 t))
("siewiorek" nil (s ax w ao1 r ih0 k))
("sifco" nil (s ih1 f k ow0))
("sifers" nil (s ay1 f er0 z))
("sifford" nil (s ih1 f er0 d))
("sift" nil (s ih1 f t))
("sifted" nil (s ih1 f t ih0 d))
("sifting" nil (s ih1 f t ih0 ng))
("sifts" nil (s ih1 f t s))
("sifuentes" nil (s iy0 f w eh1 n t eh0 s))
("sigafoos" nil (s ih1 g ax f uw1 z))
("sigal" nil (s ay1 jh ax l))
("sigala" nil (s iy0 g aa1 l ax))
("sigel" nil (s ih1 g ax l))
("sigfreda" nil (s iy0 g f r eh1 d ax))
("sigg" nil (s ih1 g))
("siggers" nil (s ih1 g er0 z))
("siggins" nil (s ih1 g ih0 n z))
("sigh" nil (s ay1))
("sighed" nil (s ay1 d))
("sighing" nil (s ay1 ih0 ng))
("sighs" nil (s ay1 z))
("sight" nil (s ay1 t))
("sighted" nil (s ay1 t ax d))
("sighting" nil (s ay1 t ih0 ng))
("sightings" nil (s ay1 t ih0 ng z))
("sights" nil (s ay1 t s))
("sightseeing" nil (s ay1 t s iy1 ih0 ng))
("sigl" nil (s ih1 g ax l))
("sigle" nil (s ay1 g ax l))
("sigler" nil (s ay1 g ax l er0))
("sigley" nil (s ih1 g l iy0))
("siglin" nil (s ih1 g l ih0 n))
("sigma" nil (s ih1 g m ax))
("sigmaform" nil (s ih1 g m ax f ao0 r m))
("sigman" nil (s ih1 g m ax n))
("sigmon" nil (s ih1 g m ax n))
("sigmund" nil (s ih1 g m ax n d))
("sign" nil (s ay1 n))
("signa" nil (s ih1 g n ax))
("signage" nil (s ay1 n ih0 jh))
("signal" nil (s ih1 g n ax l))
("signaled" nil (s ih1 g n ax l d))
("signaling" nil (s ih1 g n ax l ih0 ng))
("signalled" nil (s ih1 g n ax l d))
("signalling" nil (s ih1 g n ax l ih0 ng))
("signals" nil (s ih1 g n ax l z))
("signatories" nil (s ih1 g n ax t ao1 r iy0 z))
("signatory" nil (s ih1 g n ax t ao1 r iy0))
("signature" nil (s ih1 g n ax ch er0))
("signatures" nil (s ih1 g n ax ch er0 z))
("signed" nil (s ay1 n d))
("signer" nil (s ay1 n er0))
("signers" nil (s ay1 n er0 z))
("signet" nil (s ih1 g n ih0 t))
("signficant" nil (s ay1 n f ih0 k ax n t))
("significance" nil (s ax g n ih1 f ih0 k ax n s))
("significant" nil (s ax g n ih1 f ih0 k ax n t))
("significantly" nil (s ih0 g n ih1 f ih0 k ax n t l iy0))
("signified" nil (s ih1 g n ax f ay1 d))
("signifies" nil (s ih1 g n ax f ay1 z))
("signify" nil (s ih1 g n ax f ay1))
("signifying" nil (s ih1 g n ax f ay1 ih0 ng))
("signing" nil (s ay1 n ih0 ng))
("signings" nil (s ay1 n ih0 ng z))
("signor" nil (s iy1 n y ao0 r))
("signore" nil (s iy0 n y ao1 r ey0))
("signorelli" nil (s iy0 g n ao0 r eh1 l iy0))
("signposts" nil (s ay1 n p ow1 s t s))
("signs" nil (s ay1 n z))
("sigoloff" nil (s ih1 g ax l ao0 f))
("sigourney" nil (s ih0 g ao1 r n iy0))
("sigrid" nil (s ih1 g r ih0 d))
("sigrist" nil (s ih1 g r ih0 s t))
("sigur" nil (s ih1 g er0))
("sigurd" nil (s ih1 g er0 d))
("sigurdson" nil (s ih1 g er0 d s ax n))
("sigwald" nil (s ih1 g w ax l d))
("sihanouk" nil (s iy1 ax n uh1 k))
("sikes" nil (s ay1 k s))
("sikh" nil (s iy1 k))
("sikhs" nil (s iy1 k s))
("sikkema" nil (s ih0 k iy1 m ax))
("sikora" nil (s ih0 k ao1 r ax))
("sikorski" nil (s ih0 k ao1 r s k iy0))
("sikorsky" nil (s ih0 k ao1 r s k iy0))
("sil" nil (s ih1 l))
("silajdzic" nil (s ih1 l aa1 jh ih0 k))
("silas" nil (s ay1 l ax s))
("silbaugh" nil (s ih1 l b ao1))
("silber" nil (s ih1 l b er0))
("silberberg" nil (s ih1 l b er0 b er0 g))
("silberg" nil (s ih1 l b er0 g))
("silbergeld" nil (s ih1 l b er0 g eh1 l d))
("silberman" nil (s ih1 l b er0 m ax n))
("silbernagel" nil (s ih1 l b er0 n ax g ax l))
("silberner" nil (s ih0 l b er1 n er0))
("silberstein" nil (s ih1 l b er0 s t iy1 n))
("silbert" nil (s ih1 l b er0 t))
("silcott" nil (s ih1 l k ax t))
("silcox" nil (s ih1 l k aa0 k s))
("silence" nil (s ay1 l ax n s))
("silenced" nil (s ay1 l ax n s t))
("silences" nil (s ay1 l ax n s ih0 z))
("silencing" nil (s ay1 l ax n s ih0 ng))
("silent" nil (s ay1 l ax n t))
("silently" nil (s ay1 l ax n t l iy0))
("sileo" nil (s ih1 l iy0 ow0))
("siler" nil (s ay1 l er0))
("silesia" nil (s ih0 l iy1 z ax))
("silex" nil (s ih1 l ax k s))
("silfies" nil (s ih1 l f iy0 z))
("silguero" nil (s iy0 l g eh1 r ow0))
("silhouette" nil (s ih1 l ax w eh1 t))
("silhouettes" nil (s ih1 l ax w eh1 t s))
("silica" nil (s ih1 l ax k ax))
("silicate" nil (s ih1 l ax k ax t))
("silicates" nil (s ih1 l ax k ax t s))
("silicon" nil (s ih1 l ax k ax n))
("silicone" nil (s ih1 l ax k ow1 n))
("silicones" nil (s ih1 l ax k ow1 n z))
("siliconix" nil (s ih1 l ih0 k aa1 n ih0 k s))
("silk" nil (s ih1 l k))
("silkey" nil (s ih1 l k iy0))
("silks" nil (s ih1 l k s))
("silkwood" nil (s ih1 l k w uh1 d))
("silkworm" nil (s ih1 l k w er0 m))
("silkworms" nil (s ih1 l k w er1 m z))
("silky" nil (s ih1 l k iy0))
("sill" nil (s ih1 l))
("siller" nil (s ih1 l er0))
("sillerman" nil (s ih1 l er0 m ax n))
("sillier" nil (s ih1 l iy0 er0))
("silliest" nil (s ih1 l iy0 ax s t))
("silliman" nil (s ih1 l ih0 m ax n))
("silliness" nil (s ih1 l iy0 n ax s))
("sillman" nil (s ih1 l m ax n))
("sills" nil (s ih1 l z))
("silly" nil (s ih1 l iy0))
("silman" nil (s ih1 l m ax n))
("silmon" nil (s iy0 l m ao1 n))
("silo" nil (s ay1 l ow0))
("silos" nil (s ay1 l ow1 z))
("silsby" nil (s ih1 l s b iy0))
("silt" nil (s ih1 l t))
("siltec" nil (s ih1 l t eh1 k))
("silva" nil (s ih1 l v ax))
("silvadio" nil (s ih0 l v aa1 d iy0 ow0))
("silvana" nil (s ih0 l v aa1 n ax))
("silvar" nil (s ih1 l v aa0 r))
("silvas" nil (s ih1 l v ax z))
("silveira" nil (s iy0 l v eh1 r ax))
("silver" nil (s ih1 l v er0))
("silvera" nil (s iy0 l v eh1 r ax))
("silverado" nil (s ih1 l v er0 aa1 d ow0))
("silverberg" nil (s ih1 l v er0 b er0 g))
("silvercrest" nil (s ih1 l v er0 k r eh1 s t))
("silverdome" nil (s ih1 l v er0 d ow1 m))
("silveri" nil (s iy0 l v eh1 r iy0))
("silveria" nil (s iy0 l v eh1 r iy0 ax))
("silverio" nil (s iy0 l v eh1 r iy0 ow0))
("silverlake" nil (s ih1 l v er0 l ey1 k))
("silverman" nil (s ih1 l v er0 m ae1 n))
("silvernail" nil (s ih1 l v er0 n ey1 l))
("silverplate" nil (s ih1 l v er0 p l ey1 t))
("silvers" nil (s ih1 l v er0 z))
("silvershoe" nil (s ih1 l v er0 sh uw1))
("silversides" nil (s ih1 l v er0 s ay1 d z))
("silverstein" nil (s ih1 l v er0 s t iy1 n))
("silverstone" nil (s ih1 l v er0 s t ow1 n))
("silverthorn" nil (s ih0 l v er1 th er0 n))
("silverthorne" nil (s ih0 l v er1 th er0 n))
("silverware" nil (s ih1 l v er0 w eh1 r))
("silvery" nil (s ih1 l v er0 iy0))
("silvester" nil (s ih0 l v eh1 s t er0))
("silvestre" nil (s iy0 l v ey1 s t r ey0))
("silvestri" nil (s iy0 l v eh1 s t r iy0))
("silvestro" nil (s iy0 l v ey1 s t r ow0))
("silvey" nil (s ih1 l v iy0))
("silvi" nil (s ih1 l v iy0))
("silvia" nil (s ih1 l v iy0 ax))
("silvie" nil (s ih1 l v iy0))
("silvio" nil (s ih1 l v iy0 ow0))
("silvis" nil (s ih1 l v ih0 s))
("silvius" nil (s ih1 l v iy0 ih0 s))
("sim" nil (s ih1 m))
("sima" nil (s iy1 m ax))
("simard" nil (s ih1 m er0 d))
("simas" nil (s ay1 m ax z))
("simba" nil (s ih1 m b ax))
("simcha" nil (s ih1 m hh ax))
("simco" nil (s ih1 m k ow0))
("simcoe" nil (s ih1 m k ow1))
("simcox" nil (s ih1 m k aa1 k s))
("sime" nil (s ay1 m))
("simek" nil (s ih1 m ih0 k))
("simenson" nil (s ih1 m ih0 n s ax n))
("simental" nil (s ih0 m eh1 n t ax l))
("simeon" nil (s ih1 m iy0 ax n))
("simeone" nil (s ih1 m iy0 ah1 n))
("simer" nil (s ay1 m er0))
("simerly" nil (s ay1 m er0 l iy0))
("simerson" nil (s ih1 m er0 s ax n))
("simes" nil (s ay1 m z))
("simex" nil (s ih1 m eh0 k s))
("simi" nil (s iy1 m iy0))
("simian" nil (s ih1 m iy0 ax n))
("simich" nil (s ih1 m ih0 ch))
("simien" nil (s ih1 m iy0 n))
("similar" nil (s ih1 m ax l er0))
("similarities" nil (s ih1 m ax l eh1 r ax t iy0 z))
("similarity" nil (s ih1 m ax l eh1 r ax t iy0))
("similarly" nil (s ih1 m ax l er0 l iy0))
("similiar" nil (s ih0 m ih1 l iy0 aa0 r))
("simington" nil (s ih1 m ih0 ng t ax n))
("siminski" nil (s ih0 m ih1 n s k iy0))
("simione" nil (s iy1 m iy0 ow1 n iy0))
("simison" nil (s ih1 m ih0 s ax n))
("simkin" nil (s ih1 m k ih0 n))
("simkins" nil (s ih1 m k ih0 n z))
("simko" nil (s ih1 m k ow0))
("simler" nil (s ih1 m ax l er0))
("simm" nil (s ih1 m))
("simmer" nil (s ih1 m er0))
("simmered" nil (s ih1 m er0 d))
("simmering" nil (s ih1 m er0 ih0 ng))
("simmerman" nil (s ih1 m er0 m ax n))
("simmers" nil (s ih1 m er0 z))
("simmert" nil (s ih1 m er0 t))
("simmon" nil (s ih1 m ax n))
("simmonds" nil (s ih1 m ax n d z))
("simmons" nil (s ih1 m ax n z))
("simms" nil (s ih1 m z))
("simo" nil (s iy1 m ow0))
("simoes" nil (s ay1 m ow0 z))
("simon" nil (s ay1 m ax n))
("simona" nil (s ih0 m ow1 n ax))
("simonds" nil (s ay1 m ax n d z))
("simone" nil (s ih0 m ow1 n))
("simoneau" nil (s ih1 m ax n ow0))
("simoneaux" nil (s ih1 m ax n ow0))
("simonelli" nil (s iy1 m ow0 n eh1 l iy0))
("simonet" nil (s ih1 m ax n eh0 t))
("simonetta" nil (s iy0 m ax n eh1 t ax))
("simonette" nil (s ih1 m ax n eh0 t))
("simonetti" nil (s iy0 m ow0 n eh1 t iy0))
("simoni" nil (s iy0 m ow1 n iy0))
("simonian" nil (s ih0 m ow1 n iy0 ax n))
("simonich" nil (s ih1 m ax n ih0 k))
("simonin" nil (s ih1 m ax n ih0 n))
("simonis" nil (s ih1 m ax n ih0 s))
("simons" nil (s ay1 m ax n z))
("simonsen" nil (s ih1 m ax n s ax n))
("simonson" nil (s ih1 m ax n s ax n))
("simonsson" nil (s ay1 m ax n s ax n))
("simonton" nil (s ih0 m aa1 n t ax n))
("simper" nil (s ih1 m p er0))
("simpkins" nil (s ih1 m p k ih0 n z))
("simple" nil (s ih1 m p ax l))
("simpler" nil (s ih1 m p ax l er0))
("simplesse" nil (s ih0 m p l eh1 s))
("simplest" nil (s ih1 m p l ax s t))
("simplex" nil (s ih1 m p l eh1 k s))
("simplicity" nil (s ih0 m p l ih1 s ax t iy0))
("simplification" nil (s ih1 m p l ax f ih0 k ey1 sh ax n))
("simplified" nil (s ih1 m p l ax f ay1 d))
("simplifies" nil (s ih1 m p l ax f ay1 z))
("simplify" nil (s ih1 m p l ax f ay1))
("simplifying" nil (s ih1 m p l ax f ay1 ih0 ng))
("simplistic" nil (s ih0 m p l ih1 s t ih0 k))
("simplot" nil (s ih1 m p l aa0 t))
("simply" nil (s ih1 m p l iy0))
("simpson" nil (s ih1 m p s ax n))
("simpsons" nil (s ih1 m p s ax n z))
("sims" nil (s ih1 m z))
("simsbury" nil (s ih1 m z b eh1 r iy0))
("simson" nil (s ih1 m s ax n))
("simulate" nil (s ih1 m y ax l ax t))
("simulated" nil (s ih1 m y ax l ey1 t ih0 d))
("simulates" nil (s ih1 m y ax l ey1 t s))
("simulating" nil (s ih1 m y ax l ey1 t ih0 ng))
("simulation" nil (s ih1 m y ax l ey1 sh ax n))
("simulations" nil (s ih1 m y uw0 l ey1 sh ax n z))
("simulator" nil (s ih1 m y ax l ey1 t er0))
("simulators" nil (s ih1 m y ax l ey1 t er0 z))
("simultaneous" nil (s ay1 m ax l t ey1 n iy0 ax s))
("simultaneously" nil (s ay1 m ax l t ey1 n iy0 ax s l iy0))
("sin" nil (s ih1 n))
("sinagra" nil (s ih0 n ae1 g r ax))
("sinai" nil (s ay1 n ay1))
("sinar" nil (s ay1 n er0))
("sinatra" nil (s ax n aa1 t r ax))
("sinay" nil (s ih0 n ey1))
("sincavage" nil (s iy0 n k aa1 v ih0 jh))
("since" nil (s ih1 n s))
("sincere" nil (s ax n s ih1 r))
("sincerely" nil (s ih0 n s ih1 r l iy0))
("sincerity" nil (s ax n s eh1 r ax t iy0))
("sinclair" nil (s ih0 n k l eh1 r))
("sind" nil (s ih1 n d))
("sindelar" nil (s ih1 n d ih0 l er0))
("sindlinger" nil (s ih1 n d l ih1 ng er0))
("sindona" nil (s ih0 n d ow1 n ax))
("sindoni" nil (s iy0 n d ow1 n iy0))
("sindt" nil (s ih1 n t))
("sine" nil (s ay1 n))
("sineath" nil (s ih1 n eh0 th))
("sinegal" nil (s ih1 n ih0 g ax l))
("siner" nil (s ay1 n er0))
("sines" nil (s ay1 n z))
("sinewy" nil (s ih1 n y uw0 iy0))
("sinfonia" nil (s ih0 n f ow1 n iy0 ax))
("sinful" nil (s ih1 n f ax l))
("sing" nil (s ih1 ng))
("singapore" nil (s ih1 ng ax p ao1 r))
("singaporean" nil (s ih0 ng g ax p ao1 r iy0 ax n))
("singaporeans" nil (s ih0 ng g ax p ao1 r iy0 ax n z))
("singer" nil (s ih1 ng er0))
("singerman" nil (s ih1 n jh er1 m ax n))
("singers" nil (s ih1 ng er0 z))
("singh" nil (s ih1 ng))
("singing" nil (s ih1 ng ih0 ng))
("singlaub" nil (s ih1 ng l aw1 b))
("single" nil (s ih1 ng g ax l))
("singled" nil (s ih1 ng g ax l d))
("singlehandedly" nil (s ih1 ng g ax l hh ae1 n d ih0 d l iy0))
("singler" nil (s ih1 ng g ax l er0))
("singles" nil (s ih1 ng g ax l z))
("singletary" nil (s ih1 ng g l ih0 t eh0 r iy0))
("singleterry" nil (s ih1 ng g l ih0 t eh0 r iy0))
("singleton" nil (s ih1 ng g ax l t ax n))
("singley" nil (s ih1 ng g l iy0))
("singling" nil (s ih1 ng g ax l ih0 ng))
("singly" nil (s ih1 ng g l iy0))
("sings" nil (s ih1 ng z))
("singular" nil (s ih1 ng g y ax l er0))
("singularization" nil (s ih1 ng g y ax l er0 ih0 z ey1 sh ax n))
("singularly" nil (s ih1 ng g y ax l er0 l iy0))
("sinha" nil (s ih1 n hh ax))
("sinhalese" nil (s ih1 n ax l iy1 z))
("siniard" nil (s ih1 n iy0 er0 d))
("sinibaldi" nil (s ih0 n ih0 b aa1 l d iy0))
("siniscalchi" nil (s ih0 n ih0 s k aa1 l k iy0))
("sinisi" nil (s ih0 n iy1 s iy0))
("sinister" nil (s ih1 n ih0 s t er0))
("sink" nil (s ih1 ng k))
("sinker" nil (s ih1 ng k er0))
("sinkfield" nil (s ih1 ng k f iy1 l d))
("sinkhole" nil (s ih1 ng k hh ow1 l))
("sinkhorn" nil (s ih1 ng k hh er0 n))
("sinking" nil (s ih1 ng k ih0 ng))
("sinkler" nil (s ih1 ng k l er0))
("sinko" nil (s ih1 ng k ow0))
("sinks" nil (s ih1 ng k s))
("sinn" nil (s ih1 n))
("sinned" nil (s ih1 n d))
("sinner" nil (s ih1 n er0))
("sinners" nil (s ih1 n er0 z))
("sinnett" nil (s ih1 n ih0 t))
("sinning" nil (s ih1 n ih0 ng))
("sinnott" nil (s ih1 n ax t))
("sino" nil (s ay1 n ow0))
("sinopec" nil (s ay1 n ow0 p eh1 k))
("sinopoli" nil (s iy0 n ow0 p ow1 l iy0))
("sinor" nil (s ay1 n er0))
("sinquefield" nil (s ih1 n k w ih0 f iy0 l d))
("sins" nil (s ih1 n z))
("sinsabaugh" nil (s ih1 n s ax b ao1))
("sinsel" nil (s ih1 n s ax l))
("sinton" nil (s ih1 n t ax n))
("sinuous" nil (s ih1 n w ax s))
("sinus" nil (s ay1 n ax s))
("sinuses" nil (s ay1 n ax s ax z))
("sinyard" nil (s ih1 n y aa1 r d))
("siobhan" nil (sh aw1 b aa1 n))
("sioux" nil (s uw1))
("sip" nil (s ih1 p))
("sipe" nil (s ay1 p))
("sipelstein" nil (s ih1 p ax l s t ay0 n))
("sipes" nil (s ay1 p s))
("siphon" nil (s ay1 f ax n))
("siphoned" nil (s ay1 f ax n d))
("siphoning" nil (s ay1 f ax n ih0 ng))
("siphons" nil (s ay1 f ax n z))
("siple" nil (s ay1 p ax l))
("sipos" nil (s ay1 p ow0 z))
("sipp" nil (s ih1 p))
("sipped" nil (s ih1 p t))
("sippel" nil (s ih1 p ax l))
("sippican" nil (s ih1 p ih0 k ax n))
("sipping" nil (s ih1 p ih0 ng))
("sipple" nil (s ih1 p ax l))
("sips" nil (s ih1 p s))
("sir" nil (s er1))
("siracusa" nil (s ih0 r aa0 k uw1 s ax))
("siracuse" nil (s ih0 r aa0 k uw1 s iy0))
("siragusa" nil (s ih0 r aa0 g uw1 s ax))
("siravo" nil (s ih0 r aa1 v ow0))
("sircy" nil (s er1 k iy0))
("sire" nil (s ay1 er0))
("sirek" nil (s ao1 r ih0 k))
("siren" nil (s ay1 r ax n))
("sirena" nil (s ih0 r eh1 n ax))
("sirens" nil (s ay1 r ax n z))
("sires" nil (s ay1 r z))
("siri" nil (s ih1 r iy0))
("sirianni" nil (s ih0 r iy0 aa1 n iy0))
("sirignano" nil (s ih1 r iy0 n y aa1 n ow0))
("siris" nil (s ay1 r ih0 s))
("sirk" nil (s er1 k))
("sirkin" nil (s er1 k ih0 n))
("sirko" nil (s er1 k ow0))
("sirles" nil (s ao1 r ax l z))
("sirman" nil (s er1 m ax n))
("sirmans" nil (s er1 m ax n z))
("sirmon" nil (s er1 m ax n))
("sirmons" nil (s er1 m ax n z))
("sirna" nil (s er1 n ax))
("sirois" nil (s ay0 r w aa1))
("siroky" nil (s ih1 r ax k iy0))
("siron" nil (s ao1 r ax n))
("sirota" nil (s ih0 r ow1 t ax))
("sirowitz" nil (s ih1 r ax w ih0 t s))
("sirri" nil (s ih1 r iy0))
("sirrine" nil (s ih0 r iy1 n iy0))
("sis" nil (s ih1 s))
("sisco" nil (s iy1 s k ow0))
("siscoe" nil (s ih1 s k ow0))
("sisemore" nil (s ay1 z m ao0 r))
("sisk" nil (s ih1 s k))
("siska" nil (s iy1 s k ax))
("siskel" nil (s ih1 s k ax l))
("siskin" nil (s ih1 s k ih0 n))
("siskind" nil (s ih1 s k ih0 n d))
("sisko" nil (s ih1 s k ow0))
("sisler" nil (s ih1 s ax l er0))
("sisley" nil (s ih1 z l iy0))
("sisneros" nil (s ih1 s n er0 ow0 z))
("sisney" nil (s ih1 z n iy0))
("sison" nil (s ih1 s ax n))
("sissel" nil (s ih1 s ax l))
("sissie" nil (s ih1 s iy0))
("sissom" nil (s ih1 s ax m))
("sisson" nil (s ih1 s ax n))
("sissy" nil (s ih1 s iy0))
("sistare" nil (s iy0 s t aa1 r iy0))
("sistek" nil (s ih1 s t ih0 k))
("sister" nil (s ih1 s t er0))
("sisters" nil (s ih1 s t er0 z))
("sisti" nil (s ih1 s t iy0))
("sistine" nil (s ih0 s t iy1 n))
("sisto" nil (s ih1 s t ow0))
("sistrunk" nil (s ih1 s t r ax ng k))
("sisulu" nil (s ih0 s uw1 l uw0))
("sit" nil (s ih1 t))
("sitar" nil (s ih1 t er0))
("sitcom" nil (s ih1 t k aa1 m))
("sitcoms" nil (s ih1 t k aa0 m z))
("site" nil (s ay1 t))
("sited" nil (s ay1 t ih0 d))
("sitek" nil (s ih1 t eh0 k))
("sites" nil (s ay1 t s))
("sithe" nil (s ay1 dh))
("siting" nil (s ay1 t ih0 ng))
("sitiveni" nil (s iy1 t ih0 v iy1 n iy0))
("sitka" nil (s ih1 t k ax))
("sitko" nil (s ih1 t k ow0))
("sitler" nil (s ay1 t ax l er0))
("sitmar" nil (s ih1 t m aa1 r))
("sitrick" nil (s ih1 t r ih0 k))
("sits" nil (s ih1 t s))
("sitter" nil (s ih1 t er0))
("sitterly" nil (s ih1 t er0 l iy0))
("sitters" nil (s ih1 t er0 z))
("sittig" nil (s ih1 t ih0 g))
("sitting" nil (s ih1 t ih0 ng))
("sittler" nil (s ih1 t l er0))
("sittner" nil (s ih1 t n er0))
("sitton" nil (s ih1 t ax n))
("sitts" nil (s ih1 t s))
("situated" nil (s ih1 ch uw0 ey1 t ih0 d))
("situation" nil (s ih1 ch uw0 ey1 sh ax n))
("situations" nil (s ih1 ch uw0 ey1 sh ax n z))
("situs" nil (s ay1 t ax s))
("sitz" nil (s ih1 t s))
("sitze" nil (s ih1 t z))
("sitzer" nil (s ih1 t z er0))
("sitzes" nil (s ih1 t s ih0 z))
("sitzman" nil (s ih1 t s m ax n))
("sitzmann" nil (s ih1 t s m ax n))
("siu" nil (s iy1 uw0))
("siuda" nil (s iy0 uw1 d ax))
("sivak" nil (s ih1 v ax k))
("siver" nil (s ay1 v er0))
("siverling" nil (s ih1 v er0 l ih0 ng))
("siverson" nil (s ih1 v er0 s ax n))
("sivertsen" nil (s ih1 v er0 t s ax n))
("sivertson" nil (s ih1 v er0 t s ax n))
("sivils" nil (s ih1 v ax l z))
("sivley" nil (s ih1 v l iy0))
("siwek" nil (s ih1 w ih0 k))
("six" nil (s ih1 k s))
("sixed" nil (s ih1 k s t))
("sixes" nil (s ih1 k s ih0 z))
("sixfold" nil (s ih1 k s f ow1 l d))
("sixteen" nil (s ih0 k s t iy1 n))
("sixteens" nil (s ih1 k s t iy1 n z))
("sixteenth" nil (s ih0 k s t iy1 n th))
("sixteenths" nil (s ih1 k s t iy1 n th s))
("sixth" nil (s ih1 k s th))
("sixths" nil (s ih1 k s th s))
("sixties" nil (s ih1 k s t iy0 z))
("sixtieth" nil (s ih1 k s t iy0 ih0 th))
("sixty" nil (s ih1 k s t iy0))
("sizable" nil (s ay1 z ax b ax l))
("size" nil (s ay1 z))
("sizeable" nil (s ay1 z ax b ax l))
("sized" nil (s ay1 z d))
("sizeler" nil (s ay1 z l er0))
("sizelove" nil (s ay1 z l ah1 v))
("sizemore" nil (s ay1 z m ao0 r))
("sizer" nil (s ay1 z er0))
("sizes" nil (s ay1 z ax z))
("sizing" nil (s ay1 z ih0 ng))
("sizzle" nil (s ih1 z ax l))
("sizzled" nil (s ih1 z ax l d))
("sizzler" nil (s ih1 z l er0))
("sizzlin" nil (s ih1 z l ih0 n))
("sizzling" nil (s ih1 z ax l ih0 ng))
("sjoberg" nil (sh ow1 b er0 g))
("sjoblom" nil (sh ow1 b l aa0 m))
("sjodin" nil (sh ow1 d ih0 n))
("sjogren" nil (sh ow1 g r ax n))
("sjolander" nil (sh ow1 l ae1 n d er0))
("sjoquist" nil (sh ow1 k w ih0 s t))
("sjostrom" nil (sh ow1 s t r ax m))
("skaar" nil (s k aa1 r))
("skadden" nil (s k ae1 d ih0 n))
("skaff" nil (s k ae1 f))
("skaggs" nil (s k ae1 g z))
("skains" nil (s k ey1 n z))
("skala" nil (s k aa1 l ax))
("skalicky" nil (s k ax l ih1 k iy0))
("skalla" nil (s k ae1 l ax))
("skalski" nil (s k aa1 l s k iy0))
("skalsky" nil (s k aa1 l s k iy0))
("skandia" nil (s k ae1 n d iy0 er0))
("skandinaviska" nil (s k ae1 n d ih0 n ax v ih1 s ax))
("skanska" nil (s k ae1 n s k ax))
("skarda" nil (s k aa1 r d ax))
("skare" nil (s k eh1 r))
("skase" nil (s k ey1 z))
("skate" nil (s k ey1 t))
("skateboard" nil (s k ey1 t b ao1 r d))
("skateboarding" nil (s k ey1 t b ao1 r d ih0 ng))
("skated" nil (s k ey1 t ih0 d))
("skater" nil (s k ey1 t er0))
("skaters" nil (s k ey1 t er0 z))
("skates" nil (s k ey1 t s))
("skating" nil (s k ey1 t ih0 ng))
("skeat" nil (s k iy1 t))
("skeel" nil (s k iy1 l))
("skeels" nil (s k iy1 l z))
("skeen" nil (s k iy1 n))
("skeens" nil (s k iy1 n z))
("skees" nil (s k iy1 z))
("skeet" nil (s k iy1 t))
("skeete" nil (s k iy1 t))
("skeeter" nil (s k iy1 t er0))
("skeeters" nil (s k iy1 t er0 z))
("skeets" nil (s k iy1 t s))
("skeffington" nil (s k eh1 f ih0 ng t ax n))
("skehan" nil (s k ey1 ax n))
("skeletal" nil (s k eh1 l ax t ax l))
("skeleton" nil (s k eh1 l ax t ax n))
("skeletons" nil (s k eh1 l ax t ax n z))
("skellenger" nil (s k eh1 l ih0 n jh er0))
("skelley" nil (s k eh1 l iy0))
("skelly" nil (s k eh1 l iy0))
("skelter" nil (s k eh1 l t er0))
("skelton" nil (s k eh1 l t ax n))
("skenandore" nil (s k ih0 n ae1 n d er0))
("skender" nil (s k eh1 n d er0))
("skenderian" nil (s k eh1 n d eh1 r iy0 ax n))
("skene" nil (s k iy1 n))
("skeoch" nil (s k iy1 aa0 ch))
("skeptic" nil (s k eh1 p t ih0 k))
("skeptical" nil (s k eh1 p t ax k ax l))
("skeptically" nil (s k eh1 p t ih0 k ax l iy0))
("skepticism" nil (s k eh1 p t ih0 s ih1 z ax m))
("skeptics" nil (s k eh1 p t ih0 k s))
("skerry" nil (s k eh1 r iy0))
("sketch" nil (s k eh1 ch))
("sketchbook" nil (s k eh1 ch b uh1 k))
("sketchbooks" nil (s k eh1 ch b uh1 k s))
("sketched" nil (s k eh1 ch t))
("sketches" nil (s k eh1 ch ax z))
("sketching" nil (s k eh1 ch ih0 ng))
("sketchy" nil (s k eh1 ch iy0))
("skew" nil (s k y uw1))
("skewed" nil (s k y uw1 d))
("skewer" nil (s k y uw1 er0))
("skewers" nil (s k y uw1 er0 z))
("skewes" nil (s k y uw1 z))
("skewing" nil (s k y uw1 ih0 ng))
("skews" nil (s k y uw1 z))
("ski" nil (s k iy1))
("skiba" nil (s k ay1 b ax))
("skibbe" nil (s k ih1 b))
("skibicki" nil (s k ih0 b ih1 k iy0))
("skibinski" nil (s k ih0 b ih1 n s k iy0))
("skibo" nil (s k ay1 b ow0))
("skid" nil (s k ih1 d))
("skidded" nil (s k ih1 d ax d))
("skidding" nil (s k ih1 d ih0 ng))
("skidgel" nil (s k ih1 jh ax l))
("skidmore" nil (s k ih1 d m ao0 r))
("skids" nil (s k ih1 d z))
("skier" nil (s k ay1 r))
("skiers" nil (s k iy1 er0 z))
("skies" nil (s k ay1 z))
("skiff" nil (s k ih1 f))
("skiffington" nil (s k ih1 f ih0 ng t ax n))
("skiffs" nil (s k ih1 f s))
("skiing" nil (s k iy1 ih0 ng))
("skiles" nil (s k ay1 l z))
("skill" nil (s k ih1 l))
("skilled" nil (s k ih1 l d))
("skillen" nil (s k ih1 l ax n))
("skillern" nil (s k ih1 l er0 n))
("skillet" nil (s k ih1 l ax t))
("skillful" nil (s k ih1 l f ax l))
("skillfully" nil (s k ih1 l f ax l iy0))
("skillin" nil (s k ih1 l ih0 n))
("skilling" nil (s k ih1 l ih0 ng))
("skillings" nil (s k ih1 l ih0 ng z))
("skillman" nil (s k ih1 l m ax n))
("skills" nil (s k ih1 l z))
("skilton" nil (s k ih1 l t ax n))
("skim" nil (s k ih1 m))
("skimmed" nil (s k ih1 m d))
("skimmers" nil (s k ih1 m er0 z))
("skimming" nil (s k ih1 m ih0 ng))
("skimp" nil (s k ih1 m p))
("skimpier" nil (s k ih1 m p iy0 er0))
("skimping" nil (s k ih1 m p ih0 ng))
("skimpy" nil (s k ih1 m p iy0))
("skin" nil (s k ih1 n))
("skinheads" nil (s k ih1 n hh eh1 d z))
("skinks" nil (s k ih1 ng k s))
("skinless" nil (s k ih1 n l ax s))
("skinned" nil (s k ih1 n d))
("skinner" nil (s k ih1 n er0))
("skinny" nil (s k ih1 n iy0))
("skins" nil (s k ih1 n z))
("skip" nil (s k ih1 p))
("skipjack" nil (s k ih1 p jh ae1 k))
("skipjacks" nil (s k ih1 p jh ae1 k s))
("skipp" nil (s k ih1 p))
("skippa" nil (s k ih1 p ax))
("skipped" nil (s k ih1 p t))
("skipper" nil (s k ih1 p er0))
("skippers" nil (s k ih1 p er0 z))
("skipping" nil (s k ih1 p ih0 ng))
("skippy" nil (s k ih1 p iy0))
("skips" nil (s k ih1 p s))
("skipton" nil (s k ih1 p t ax n))
("skipwith" nil (s k ih1 p w ih1 th))
("skipworth" nil (s k ih1 p w er1 th))
("skirmish" nil (s k er1 m ih0 sh))
("skirmishes" nil (s k er1 m ih0 sh ih0 z))
("skirmishing" nil (s k er1 m ih0 sh ih0 ng))
("skirt" nil (s k er1 t))
("skirted" nil (s k er1 t ih0 d))
("skirting" nil (s k er1 t ih0 ng))
("skirts" nil (s k er1 t s))
("skirvin" nil (s k er1 v ih0 n))
("skis" nil (s k iy1 z))
("skit" nil (s k ih1 t))
("skits" nil (s k ih1 t s))
("skittish" nil (s k ih1 t ih0 sh))
("skittishness" nil (s k ih1 t ih0 sh n ax s))
("skiver" nil (s k ay1 v er0))
("sklar" nil (s k l aa1 r))
("sklenar" nil (s k l eh1 n er0))
("skoal" nil (s k ow1 l))
("skoczylas" nil (s k ax ch ih1 l ax z))
("skoda" nil (s k ow1 d ax))
("skoff" nil (s k ao1 f))
("skog" nil (s k aa1 g))
("skogen" nil (s k aa1 g ax n))
("skoglund" nil (s k aa1 g l ax n d))
("skok" nil (s k aa1 k))
("skokie" nil (s k ow1 k iy0))
("skoler" nil (s k ow1 l er0))
("skolnick" nil (s k ow1 l n ih0 k))
("skolnik" nil (s k ow1 l n ih0 k))
("skolniks" nil (s k ow1 l n ih0 k s))
("skonieczny" nil (s k ax n iy1 ch n iy0))
("skoog" nil (s k uw1 g))
("skop" nil (s k aa1 p))
("skora" nil (s k ao1 r ax))
("skorupa" nil (s k er0 uw1 p ax))
("skorupski" nil (s k er0 ah1 p s k iy0))
("skousen" nil (s k uw1 s ax n))
("skov" nil (s k aa1 v))
("skow" nil (s k aw1))
("skowron" nil (s k aw1 r ax n))
("skowronek" nil (s k aw0 r ow1 n eh0 k))
("skowronski" nil (s k aw0 r aa1 n s k iy0))
("skramstad" nil (s k r ae1 m s t ae0 d))
("skroch" nil (s k r aa1 k))
("skrocki" nil (s k r aa1 k iy0))
("skrzypek" nil (s k er0 z ih1 p eh0 k))
("skufca" nil (s k ah1 f k ax))
("skull" nil (s k ah1 l))
("skulls" nil (s k ah1 l z))
("skunk" nil (s k ah1 ng k))
("skunks" nil (s k ah1 ng k s))
("skura" nil (s k uh1 r ax))
("skurka" nil (s k er1 k ax))
("skutt" nil (s k ah1 t))
("sky" nil (s k ay1))
("skybox" nil (s k ay1 b aa1 k s))
("skyboxes" nil (s k ay1 b aa1 k s ih0 z))
("skydome" nil (s k ay1 d ow1 m))
("skyhawk" nil (s k ay1 hh ao1 k))
("skyhigh" nil (s k ay1 hh ay1))
("skylark" nil (s k ay1 l aa1 r k))
("skylawn" nil (s k ay1 l ao1 n))
("skyles" nil (s k ay1 l z))
("skylight" nil (s k ay1 l ay1 t))
("skylights" nil (s k ay1 l ay1 t s))
("skyline" nil (s k ay1 l ay1 n))
("skylite" nil (s k ay1 l ay1 t))
("skyrocket" nil (s k ay1 r aa1 k ax t))
("skyrocketed" nil (s k ay1 r aa1 k ax t ih0 d))
("skyrocketing" nil (s k ay1 r aa1 k ax t ih0 ng))
("skyscraper" nil (s k ay1 s k r ey1 p er0))
("skyscrapers" nil (s k ay1 s k r ey1 p er0 z))
("skytel" nil (s k ay1 t eh1 l))
("skyward" nil (s k ay1 w er0 d))
("skywave" nil (s k ay1 w ey1 v))
("skywest" nil (s k ay1 w eh1 s t))
("skywriters" nil (s k ay1 r ay1 t er0 z))
("skywriting" nil (s k ay1 r ay1 t ih0 ng))
("slab" nil (s l ae1 b))
("slabaugh" nil (s l ae1 b ao0))
("slabs" nil (s l ae1 b z))
("slaby" nil (s l ey1 b iy0))
("slack" nil (s l ae1 k))
("slacked" nil (s l ae1 k t))
("slacken" nil (s l ae1 k ax n))
("slackened" nil (s l ae1 k ax n d))
("slackening" nil (s l ae1 k ax n ih0 ng))
("slackens" nil (s l ae1 k ax n z))
("slackness" nil (s l ae1 k n ax s))
("slacks" nil (s l ae1 k s))
("slade" nil (s l ey1 d))
("sladek" nil (s l ae1 d ih0 k))
("sladky" nil (s l ae1 d k iy0))
("slag" nil (s l ae1 g))
("slagel" nil (s l ae1 g ax l))
("slager" nil (s l ey1 g er0))
("slaght" nil (s l ae1 t))
("slagle" nil (s l ey1 g ax l))
("slagter" nil (s l ae1 g t er0))
("slain" nil (s l ey1 n))
("slaine" nil (s l ey1 n))
("slalom" nil (s l aa1 l ax m))
("slam" nil (s l ae1 m))
("slama" nil (s l aa1 m ax))
("slammed" nil (s l ae1 m d))
("slammer" nil (s l ae1 m er0))
("slamming" nil (s l ae1 m ih0 ng))
("slams" nil (s l ae1 m z))
("slander" nil (s l ae1 n d er0))
("slanderous" nil (s l ae1 n d er0 ax s))
("slane" nil (s l ey1 n))
("slaney" nil (s l ey1 n iy0))
("slang" nil (s l ae1 ng))
("slanina" nil (s l aa0 n iy1 n ax))
("slankard" nil (s l ae1 ng k er0 d))
("slant" nil (s l ae1 n t))
("slanted" nil (s l ae1 n t ax d))
("slanting" nil (s l ae1 n t ih0 ng))
("slap" nil (s l ae1 p))
("slapdash" nil (s l ae1 p d ae1 sh))
("slape" nil (s l ey1 p))
("slapped" nil (s l ae1 p t))
("slappey" nil (s l ae1 p iy0))
("slapping" nil (s l ae1 p ih0 ng))
("slaps" nil (s l ae1 p s))
("slapstick" nil (s l ae1 p s t ih1 k))
("slash" nil (s l ae1 sh))
("slashed" nil (s l ae1 sh t))
("slasher" nil (s l ae1 sh er0))
("slashes" nil (s l ae1 sh ih0 z))
("slashing" nil (s l ae1 sh ih0 ng))
("slate" nil (s l ey1 t))
("slated" nil (s l ey1 t ih0 d))
("slaten" nil (s l ey1 t ax n))
("slater" nil (s l ey1 t er0))
("slates" nil (s l ey1 t s))
("slatkin" nil (s l ae1 t k ih0 n))
("slaton" nil (s l ae1 t ax n))
("slats" nil (s l ae1 t s))
("slatten" nil (s l ae1 t ax n))
("slatter" nil (s l ae1 t er0))
("slattery" nil (s l ae1 t er0 iy0))
("slatton" nil (s l ae1 t ax n))
("slaugh" nil (s l ao1))
("slaughter" nil (s l ao1 t er0))
("slaughtered" nil (s l ao1 t er0 d))
("slaughterhouse" nil (s l ao1 t er0 hh aw1 s))
("slaughterhouses" nil (s l ao1 t er0 hh aw1 s ih0 z))
("slaughtering" nil (s l ao1 t er0 ih0 ng))
("slaughterings" nil (s l ao1 t er0 ih0 ng z))
("slaughters" nil (s l ao1 t er0 z))
("slauson" nil (s l aw1 z ax n))
("slave" nil (s l ey1 v))
("slaven" nil (s l ey1 v ax n))
("slavens" nil (s l ey1 v ax n z))
("slavery" nil (s l ey1 v er0 iy0))
("slaves" nil (s l ey1 v z))
("slavic" nil (s l aa1 v ih0 k))
("slavick" nil (s l ae1 v ih0 k))
("slavik" nil (s l aa1 v ih0 k))
("slavin" nil (s l ae1 v ih0 n))
("slavinski" nil (s l ax v ih1 n s k iy0))
("slavish" nil (s l ey1 v ih0 sh))
("slavishly" nil (s l ae1 v ih0 sh l iy0))
("slaw" nil (s l ao1))
("slawinski" nil (s l aa0 v ih1 n s k iy0))
("slawson" nil (s l ao1 s ax n))
("slay" nil (s l ey1))
("slayback" nil (s l ey1 b ae1 k))
("slaybaugh" nil (s l ey1 b ao1))
("slayden" nil (s l ey1 d ax n))
("slaydon" nil (s l ey1 d ax n))
("slaying" nil (s l ey1 ih0 ng))
("slayings" nil (s l ey1 ih0 ng z))
("slaymaker" nil (s l ey1 m ey1 k er0))
("slayter" nil (s l ey1 t er0))
("slayton" nil (s l ey1 t ax n))
("sleasman" nil (s l iy1 z m ax n))
("sleator" nil (s l ey1 t er0))
("sleaze" nil (s l iy1 z))
("sleazy" nil (s l iy1 z iy0))
("slechta" nil (s l eh1 ch t ax))
("sled" nil (s l eh1 d))
("sledd" nil (s l eh1 d))
("sledding" nil (s l eh1 d ih0 ng))
("sledge" nil (s l eh1 jh))
("sledgehammer" nil (s l eh1 jh hh ae1 m er0))
("sleds" nil (s l eh1 d z))
("sledz" nil (s l eh1 d z))
("slee" nil (s l iy1))
("sleek" nil (s l iy1 k))
("sleeker" nil (s l iy1 k er0))
("sleekly" nil (s l iy1 k l iy0))
("sleeman" nil (s l iy1 m ax n))
("sleep" nil (s l iy1 p))
("sleeper" nil (s l iy1 p er0))
("sleepers" nil (s l iy1 p er0 z))
("sleepily" nil (s l iy1 p ax l iy0))
("sleepiness" nil (s l iy1 p iy0 n ax s))
("sleeping" nil (s l iy1 p ih0 ng))
("sleepless" nil (s l iy1 p l ax s))
("sleeps" nil (s l iy1 p s))
("sleepwalkers" nil (s l iy1 p w ao1 k er0 z))
("sleepwalking" nil (s l iy1 p w ao1 k ih0 ng))
("sleepwear" nil (s l iy1 p w eh1 r))
("sleepy" nil (s l iy1 p iy0))
("sleet" nil (s l iy1 t))
("sleeter" nil (s l iy1 t er0))
("sleeth" nil (s l iy1 th))
("sleeve" nil (s l iy1 v))
("sleeved" nil (s l iy1 v d))
("sleeves" nil (s l iy1 v z))
("sleigh" nil (s l ey1))
("sleight" nil (s l ay1 t))
("sleipner" nil (s l ay1 p n er0))
("slemmer" nil (s l eh1 m er0))
("slemp" nil (s l eh1 m p))
("slender" nil (s l eh1 n d er0))
("slentz" nil (s l eh1 n t s))
("slept" nil (s l eh1 p t))
("sletten" nil (s l eh1 t ax n))
("sleuth" nil (s l uw1 th))
("sleuthing" nil (s l uw1 th ih0 ng))
("sleuths" nil (s l uw1 th s))
("sleven" nil (s l iy1 v ax n))
("slevin" nil (s l eh1 v ih0 n))
("slew" nil (s l uw1))
("slezak" nil (s l eh1 z ax k))
("slice" nil (s l ay1 s))
("sliced" nil (s l ay1 s t))
("slices" nil (s l ay1 s ax z))
("slicing" nil (s l ay1 s ih0 ng))
("slick" nil (s l ih1 k))
("slicked" nil (s l ih1 k t))
("slicker" nil (s l ih1 k er0))
("slickers" nil (s l ih1 k er0 z))
("slickly" nil (s l ih1 k l iy0))
("slid" nil (s l ih1 d))
("slide" nil (s l ay1 d))
("slider" nil (s l ay1 d er0))
("slides" nil (s l ay1 d z))
("sliding" nil (s l ay1 d ih0 ng))
("slife" nil (s l ay1 f))
("slifer" nil (s l ay1 f er0))
("slifka" nil (s l ih1 f k ax))
("slifko" nil (s l ih1 f k ow0))
("sligar" nil (s l ih1 g er0))
("sliger" nil (s l ay1 g er0))
("sligh" nil (s l ay1))
("slight" nil (s l ay1 t))
("slighted" nil (s l ay1 t ih0 d))
("slightest" nil (s l ay1 t ax s t))
("slighting" nil (s l ay1 t ih0 ng))
("slightly" nil (s l ay1 t l iy0))
("slights" nil (s l ay1 t s))
("sliker" nil (s l ay1 k er0))
("slim" nil (s l ih1 m))
("slimak" nil (s l ih1 m ax k))
("slime" nil (s l ay1 m))
("slimmed" nil (s l ih1 m d))
("slimmer" nil (s l ih1 m er0))
("slimmest" nil (s l ih1 m ax s t))
("slimming" nil (s l ih1 m ih0 ng))
("slimp" nil (s l ih1 m p))
("slims" nil (s l ih1 m z))
("slimy" nil (s l ay1 m iy0))
("sliney" nil (s l ih1 n iy0))
("sling" nil (s l ih1 ng))
("slinger" nil (s l ih1 ng er0))
("slingerland" nil (s l ih1 ng g er0 l ax n d))
("slinging" nil (s l ih1 ng ih0 ng))
("slings" nil (s l ih1 ng z))
("slingshot" nil (s l ih1 ng sh aa1 t))
("slingshots" nil (s l ih1 ng sh aa1 t s))
("slinkard" nil (s l ih1 ng k er0 d))
("slinker" nil (s l ih1 ng k er0))
("slinky" nil (s l ih1 ng k iy0))
("slip" nil (s l ih1 p))
("slippage" nil (s l ih1 p ih0 jh))
("slipped" nil (s l ih1 p t))
("slipper" nil (s l ih1 p er0))
("slippers" nil (s l ih1 p er0 z))
("slippery" nil (s l ih1 p er0 iy0))
("slipping" nil (s l ih1 p ih0 ng))
("slips" nil (s l ih1 p s))
("slipshod" nil (s l ih1 p sh aa1 d))
("slit" nil (s l ih1 t))
("sliter" nil (s l iy1 t er0))
("slithering" nil (s l ih1 dh er0 ih0 ng))
("slits" nil (s l ih1 t s))
("slitting" nil (s l ih1 t ih0 ng))
("sliva" nil (s l iy1 v ax))
("sliver" nil (s l ih1 v er0))
("slivers" nil (s l ih1 v er0 z))
("slivinski" nil (s l ih0 v ih1 n s k iy0))
("slivka" nil (s l ih1 v k ax))
("slivovitz" nil (s l ih1 v ax v ih0 t s))
("sliwa" nil (s l ay1 v ax))
("sliwinski" nil (s l ih0 v ih1 n s k iy0))
("sloan" nil (s l ow1 n))
("sloane" nil (s l ow1 n))
("sloat" nil (s l ow1 t))
("sloate" nil (s l ow1 t))
("slob" nil (s l aa1 b))
("slobbering" nil (s l aa1 b er0 ih0 ng))
("sloboda" nil (s l ax b ow1 d ax))
("slobodan" nil (s l ow1 b ow0 d aa1 n))
("slobogin" nil (s l ax b ow1 g ax n))
("slobs" nil (s l aa1 b z))
("slocan" nil (s l ow1 k ax n))
("slocomb" nil (s l ow1 k ax m))
("slocum" nil (s l ow1 k ax m))
("slocumb" nil (s l ow1 k ax m))
("sloe" nil (s l ow1))
("slog" nil (s l aa1 g))
("slogan" nil (s l ow1 g ax n))
("slogans" nil (s l ow1 g ax n z))
("slogged" nil (s l aa1 g d))
("slogging" nil (s l aa1 g ih0 ng))
("sloma" nil (s l ow1 m ax))
("sloman" nil (s l ow1 m ax n))
("slominski" nil (s l ax m ih1 n s k iy0))
("slomski" nil (s l aa1 m s k iy0))
("slonaker" nil (s l aa1 n ax k er0))
("slone" nil (s l ow1 n))
("sloniker" nil (s l aa1 n ih0 k er0))
("sloop" nil (s l uw1 p))
("slop" nil (s l aa1 p))
("slope" nil (s l ow1 p))
("sloper" nil (s l ow1 p er0))
("slopes" nil (s l ow1 p s))
("sloping" nil (s l ow1 p ih0 ng))
("sloppily" nil (s l aa1 p ax l iy0))
("sloppiness" nil (s l aa1 p iy0 n ax s))
("sloppy" nil (s l aa1 p iy0))
("slorc" nil (s l ao1 r k))
("slosh" nil (s l aa1 sh))
("sloshing" nil (s l aa1 sh ih0 ng))
("sloss" nil (s l ao1 s))
("slot" nil (s l aa1 t))
("slota" nil (s l ow1 t ax))
("sloth" nil (s l ow1 th))
("slothower" nil (s l aa1 th ow0 er0))
("slotnick" nil (s l aa1 t n ih0 k))
("slots" nil (s l aa1 t s))
("slott" nil (s l aa1 t))
("slotted" nil (s l aa1 t ih0 d))
("slotting" nil (s l aa1 t ih0 ng))
("slouch" nil (s l aw1 ch))
("slouches" nil (s l aw1 ch ih0 z))
("slouching" nil (s l aw1 ch ih0 ng))
("slough" nil (s l ah1 f))
("sloughs" nil (s l ah1 f s))
("slovacek" nil (s l aa1 v ax ch eh0 k))
("slovak" nil (s l ow1 v ae0 k))
("slovakia" nil (s l ow1 v aa1 k iy0 ax))
("slovaks" nil (s l ow1 v ae0 k s))
("slovene" nil (s l ow1 v iy1 n))
("slovenes" nil (s l ow1 v iy1 n z))
("slovenia" nil (s l ow1 v iy1 n iy0 ax))
("slovenian" nil (s l ow1 v iy1 n iy0 ax n))
("slovenliness" nil (s l ah1 v ax n l iy0 n ax s))
("slovenly" nil (s l ah1 v ax n l iy0))
("slover" nil (s l ow1 v er0))
("sloves" nil (s l ow1 v z))
("slovo" nil (s l ow1 v ow0))
("slow" nil (s l ow1))
("slowdown" nil (s l ow1 d aw1 n))
("slowdowns" nil (s l ow1 d aw1 n z))
("slowed" nil (s l ow1 d))
("slower" nil (s l ow1 er0))
("slowest" nil (s l ow1 ax s t))
("slowey" nil (s l ow1 iy0))
("slowik" nil (s l ow1 ih0 k))
("slowing" nil (s l ow1 ih0 ng))
("slowinski" nil (s l ow0 ih1 n s k iy0))
("slowly" nil (s l ow1 l iy0))
("slowness" nil (s l ow1 n ax s))
("slowpoke" nil (s l ow1 p ow1 k))
("slows" nil (s l ow1 z))
("sluder" nil (s l uw1 d er0))
("sludge" nil (s l ah1 jh))
("slug" nil (s l ah1 g))
("sluga" nil (s l uw1 g ax))
("slugfest" nil (s l ah1 g f eh1 s t))
("slugger" nil (s l ah1 g er0))
("slugging" nil (s l ah1 g ih0 ng))
("sluggish" nil (s l ah1 g ih0 sh))
("sluggishly" nil (s l ah1 g ih0 sh l iy0))
("sluggishness" nil (s l ah1 g ih0 sh n ax s))
("slugs" nil (s l ah1 g z))
("sluice" nil (s l uw1 s))
("sluicing" nil (s l uw1 s ih0 ng))
("sluiter" nil (s l uw1 t er0))
("sluka" nil (s l uw1 k ax))
("slum" nil (s l ah1 m))
("slumber" nil (s l ah1 m b er0))
("slumbering" nil (s l ah1 m b er0 ih0 ng))
("slump" nil (s l ah1 m p))
("slumped" nil (s l ah1 m p t))
("slumping" nil (s l ah1 m p ih0 ng))
("slumps" nil (s l ah1 m p s))
("slums" nil (s l ah1 m z))
("slung" nil (s l ah1 ng))
("slur" nil (s l er1))
("slurp" nil (s l er1 p))
("slurry" nil (s l er1 iy0))
("slurs" nil (s l er1 z))
("slush" nil (s l ah1 sh))
("slusher" nil (s l ah1 sh er0))
("sluss" nil (s l ah1 s))
("slusser" nil (s l ah1 s er0))
("slutsky" nil (s l ah1 t s k iy0))
("slutz" nil (s l ah1 t s))
("slutzky" nil (s l ah1 t s k iy0))
("sluyter" nil (s l ay1 t er0))
("sly" nil (s l ay1))
("slye" nil (s l ay1))
("slyly" nil (s l ay1 l iy0))
("slyness" nil (s l ay1 n ax s))
("slyter" nil (s l ay1 t er0))
("smaby" nil (s m ey1 b iy0))
("smack" nil (s m ae1 k))
("smacked" nil (s m ae1 k t))
("smacking" nil (s m ae1 k ih0 ng))
("smacks" nil (s m ae1 k s))
("smail" nil (s m ey1 l))
("smaldone" nil (s m ae1 l d ax n))
("smale" nil (s m ey1 l))
("small" nil (s m ao1 l))
("smallcap" nil (s m ao1 l k ae1 p))
("smallen" nil (s m ao1 l ax n))
("smaller" nil (s m ao1 l er0))
("smallest" nil (s m ao1 l ax s t))
("smalley" nil (s m ae1 l iy0))
("smalling" nil (s m ao1 l ih0 ng))
("smallish" nil (s m ao1 l ih0 sh))
("smallman" nil (s m ao1 l m ax n))
("smallness" nil (s m ao1 l n ax s))
("smallpox" nil (s m ao1 l p aa1 k s))
("smallridge" nil (s m ao1 l r ih1 jh))
("smalls" nil (s m ao1 l z))
("smalltalk" nil (s m ao1 l t ao1 k))
("smalltown" nil (s m ao1 l t aw1 n))
("smallwood" nil (s m ao1 l w uh1 d))
("smaltz" nil (s m ae1 l t s))
("smarmy" nil (s m aa1 r m iy0))
("smarr" nil (s m ae1 r))
("smart" nil (s m aa1 r t))
("smartcard" nil (s m aa1 r t k aa1 r d))
("smarter" nil (s m aa1 r t er0))
("smartest" nil (s m aa1 r t ax s t))
("smarting" nil (s m aa1 r t ih0 ng))
("smartly" nil (s m aa1 r t l iy0))
("smartmoney" nil (s m aa1 r t m ah1 n iy0))
("smarts" nil (s m aa1 r t s))
("smartt" nil (s m aa1 r t))
("smash" nil (s m ae1 sh))
("smashed" nil (s m ae1 sh t))
("smasher" nil (s m ae1 sh er0))
("smashes" nil (s m ae1 sh ih0 z))
("smashing" nil (s m ae1 sh ih0 ng))
("smathers" nil (s m ae1 dh er0 z))
("smatter" nil (s m ae1 t er0))
("smattering" nil (s m ae1 t er0 ih0 ng))
("smay" nil (s m ey1))
("smead" nil (s m iy1 d))
("smeal" nil (s m iy1 l))
("smear" nil (s m ih1 r))
("smeared" nil (s m ih1 r d))
("smearing" nil (s m ih1 r ih0 ng))
("smears" nil (s m ih1 r z))
("smeby" nil (s m iy1 b iy0))
("smedberg" nil (s m eh1 d b er0 g))
("smedley" nil (s m eh1 d l iy0))
("smee" nil (s m iy1))
("smejkal" nil (s m eh1 jh k ax l))
("smelcer" nil (s m eh1 l s er0))
("smell" nil (s m eh1 l))
("smelled" nil (s m eh1 l d))
("smelley" nil (s m eh1 l iy0))
("smelling" nil (s m eh1 l ih0 ng))
("smells" nil (s m eh1 l z))
("smelly" nil (s m eh1 l iy0))
("smelser" nil (s m eh1 l s er0))
("smelter" nil (s m eh1 l t er0))
("smelters" nil (s m eh1 l t er0 z))
("smelting" nil (s m eh1 l t ih0 ng))
("smeltz" nil (s m eh1 l t s))
("smeltzer" nil (s m eh1 l t z er0))
("smestad" nil (s m eh1 s t ax d))
("smet" nil (s m eh1 t))
("smetana" nil (s m eh1 t ax n ax))
("smethers" nil (s m eh1 dh er0 z))
("smethurst" nil (s m eh1 th er0 s t))
("smetzer" nil (s m eh1 t z er0))
("smick" nil (s m ih1 k))
("smid" nil (s m ih1 d))
("smiddy" nil (s m ih1 d iy0))
("smidt" nil (s m ih1 d t))
("smigel" nil (s m ih1 jh ax l))
("smigelski" nil (s m ih0 g eh1 l s k iy0))
("smigiel" nil (s m ih1 jh iy0 l))
("smigielski" nil (s m ih0 g iy1 l s k iy0))
("smile" nil (s m ay1 l))
("smiled" nil (s m ay1 l d))
("smiles" nil (s m ay1 l z))
("smiley" nil (s m ay1 l iy0))
("smilie" nil (s m ih1 l iy0))
("smiling" nil (s m ay1 l ih0 ng))
("smilingly" nil (s m ay1 l ih0 ng l iy0))
("smillie" nil (s m ih1 l iy0))
("smilow" nil (s m ih1 l ow0))
("smink" nil (s m ih1 ng k))
("smirk" nil (s m er1 k))
("smirking" nil (s m er1 k ih0 ng))
("smirks" nil (s m er1 k s))
("smirnoff" nil (s m er1 n ao0 f))
("smirnov" nil (s m er1 n aa0 f))
("smisek" nil (s m ih1 s eh0 k))
("smit" nil (s m ih1 t))
("smith" nil (s m ih1 th))
("smithburg" nil (s m ih1 th b er0 g))
("smithee" nil (s m ih1 th iy1))
("smither" nil (s m ih1 dh er0))
("smithereens" nil (s m ih1 dh er0 iy1 n z))
("smitherman" nil (s m ih1 dh er0 m ax n))
("smithers" nil (s m ih1 dh er0 z))
("smithey" nil (s m ih1 th iy0))
("smithfield" nil (s m ih1 th f iy0 l d))
("smithhart" nil (s m ih1 th hh aa1 r t))
("smithkline" nil (s m ih1 th k l ay1 n))
("smiths" nil (s m ih1 th s))
("smithson" nil (s m ih1 th s ax n))
("smithsonian" nil (s m ih0 th s ow1 n iy0 ax n))
("smithtown" nil (s m ih1 th t aw1 n))
("smithwick" nil (s m ih1 th w ih1 k))
("smitley" nil (s m ih1 t l iy0))
("smits" nil (s m ih1 t s))
("smitten" nil (s m ih1 t ax n))
("smittle" nil (s m ih1 t ax l))
("smitty" nil (s m ih1 t iy0))
("smoak" nil (s m ow1 k))
("smock" nil (s m aa1 k))
("smog" nil (s m aa1 g))
("smoggiest" nil (s m aa1 g iy0 ax s t))
("smoke" nil (s m ow1 k))
("smoked" nil (s m ow1 k t))
("smokejumper" nil (s m ow1 k jh ah1 m p er0))
("smokejumpers" nil (s m ow1 k jh ah1 m p er0 z))
("smokeless" nil (s m ow1 k l ax s))
("smoker" nil (s m ow1 k er0))
("smokers" nil (s m ow1 k er0 z))
("smokes" nil (s m ow1 k s))
("smokescreen" nil (s m ow1 k s k r iy1 n))
("smokestack" nil (s m ow1 k s t ae1 k))
("smokestacks" nil (s m ow1 k s t ae1 k s))
("smokey" nil (s m ow1 k iy0))
("smoking" nil (s m ow1 k ih0 ng))
("smoky" nil (s m ow1 k iy0))
("smola" nil (s m ow1 l ax))
("smolak" nil (s m ow1 l ax k))
("smolder" nil (s m ow1 l d er0))
("smoldering" nil (s m ow1 l d er0 ih0 ng))
("smolen" nil (s m aa1 l ax n))
("smolensk" nil (s m ow0 l eh1 n s k))
("smolenski" nil (s m ax l eh1 n s k iy0))
("smoley" nil (s m ow1 l iy0))
("smolik" nil (s m ow1 l ih0 k))
("smolin" nil (s m ow1 l ih0 n))
("smolinski" nil (s m ax l ih1 n s k iy0))
("smolinsky" nil (s m ax l ih1 n s k iy0))
("smolka" nil (s m ow1 l k ax))
("smoot" nil (s m uw1 t))
("smooth" nil (s m uw1 dh))
("smoothed" nil (s m uw1 dh d))
("smoother" nil (s m uw1 dh er0))
("smoothest" nil (s m uw1 dh ax s t))
("smoothing" nil (s m uw1 dh ih0 ng))
("smoothly" nil (s m uw1 dh l iy0))
("smoothness" nil (s m uw1 dh n ax s))
("smooths" nil (s m uw1 dh z))
("smoots" nil (s m uw1 t s))
("smorgasbord" nil (s m ao1 r g ax s b ao1 r d))
("smother" nil (s m ah1 dh er0))
("smothered" nil (s m ah1 dh er0 d))
("smothering" nil (s m ah1 dh er0 ih0 ng))
("smotherman" nil (s m ah1 dh er0 m ax n))
("smothermon" nil (s m aa1 th er0 m ow0 n))
("smothers" nil (s m ah1 dh er0 z))
("smotrich" nil (s m aa1 t r ih0 k))
("smouse" nil (s m aw1 s))
("smoyer" nil (s m oy1 er0))
("smuck" nil (s m ah1 k))
("smucker" nil (s m ah1 k er0))
("smudge" nil (s m ah1 jh))
("smug" nil (s m ah1 g))
("smuggle" nil (s m ah1 g ax l))
("smuggled" nil (s m ah1 g ax l d))
("smuggler" nil (s m ah1 g ax l er0))
("smugglers" nil (s m ah1 g ax l er0 z))
("smuggling" nil (s m ah1 g ax l ih0 ng))
("smugly" nil (s m ah1 g l iy0))
("smugness" nil (s m ah1 g n ax s))
("smull" nil (s m ah1 l))
("smullen" nil (s m ah1 l ax n))
("smurfit" nil (s m er1 f ih0 t))
("smurfs" nil (s m er1 f s))
("smut" nil (s m ah1 t))
("smyers" nil (s m ay1 er0 z))
("smylie" nil (s m ih1 l iy0))
("smyly" nil (s m ih1 l iy0))
("smyre" nil (s m ay1 r))
("smyrna" nil (s m er1 n ax))
("smyser" nil (s m ay1 s er0))
("smyth" nil (s m ay1 th))
("smythe" nil (s m ay1 dh))
("snack" nil (s n ae1 k))
("snacks" nil (s n ae1 k s))
("snader" nil (s n ey1 d er0))
("snafu" nil (s n ae1 f uw0))
("snafus" nil (s n ae1 f ax s))
("snag" nil (s n ae1 g))
("snagged" nil (s n ae1 g d))
("snaggers" nil (s n ae1 g er0 z))
("snagging" nil (s n ae1 g ih0 ng))
("snags" nil (s n ae1 g z))
("snail" nil (s n ey1 l))
("snails" nil (s n ey1 l z))
("snake" nil (s n ey1 k))
("snakebites" nil (s n ey1 k b ay1 t s))
("snakelike" nil (s n ey1 k l ay1 k))
("snakes" nil (s n ey1 k s))
("snaking" nil (s n ey1 k ih0 ng))
("snap" nil (s n ae1 p))
("snapdragons" nil (s n ae1 p d r ae1 g ax n z))
("snape" nil (s n ey1 p))
("snapp" nil (s n ae1 p))
("snapped" nil (s n ae1 p t))
("snapper" nil (s n ae1 p er0))
("snapping" nil (s n ae1 p ih0 ng))
("snapple" nil (s n ae1 p ax l))
("snapples" nil (s n ae1 p ax l z))
("snappy" nil (s n ae1 p iy0))
("snaps" nil (s n ae1 p s))
("snapshot" nil (s n ae1 p sh aa1 t))
("snapshots" nil (s n ae1 p sh aa1 t s))
("snare" nil (s n eh1 r))
("snared" nil (s n eh1 r d))
("snares" nil (s n eh1 r z))
("snaring" nil (s n eh1 r ih0 ng))
("snarl" nil (s n aa1 r l))
("snarled" nil (s n aa1 r ax l d))
("snarling" nil (s n aa1 r l ih0 ng))
("snarls" nil (s n aa1 r l z))
("snarr" nil (s n ae1 r))
("snarski" nil (s n aa1 r s k iy0))
("snatch" nil (s n ae1 ch))
("snatched" nil (s n ae1 ch t))
("snatcher" nil (s n ae1 ch er0))
("snatchers" nil (s n ae1 ch er0 z))
("snatches" nil (s n ae1 ch ih0 z))
("snatching" nil (s n ae1 ch ih0 ng))
("snavely" nil (s n ey1 v l iy0))
("snay" nil (s n ey1))
("snazzier" nil (s n ae1 z iy0 er0))
("snazzy" nil (s n ae1 z iy0))
("snead" nil (s n iy1 d))
("sneak" nil (s n iy1 k))
("sneaked" nil (s n iy1 k t))
("sneaker" nil (s n iy1 k er0))
("sneakers" nil (s n iy1 k er0 z))
("sneaking" nil (s n iy1 k ih0 ng))
("sneaks" nil (s n iy1 k s))
("sneaky" nil (s n iy1 k iy0))
("sneary" nil (s n ih1 r iy0))
("sneath" nil (s n eh1 th))
("snecma" nil (s n eh1 k m aa0))
("snedaker" nil (s n eh1 d ax k er0))
("snedden" nil (s n eh1 d ax n))
("sneddon" nil (s n eh1 d ax n))
("snedegar" nil (s n eh1 d ih0 g er0))
("snedeker" nil (s n eh1 d ih0 k er0))
("snee" nil (s n iy1))
("sneed" nil (s n iy1 d))
("sneer" nil (s n ih1 r))
("sneered" nil (s n ih1 r d))
("sneering" nil (s n ih1 r ih0 ng))
("sneeringer" nil (s n ih1 r ih0 n jh er0))
("sneers" nil (s n ih1 r z))
("sneeze" nil (s n iy1 z))
("sneezed" nil (s n iy1 z d))
("sneezing" nil (s n iy1 z ih0 ng))
("sneider" nil (s n ay1 d er0))
("snelgrove" nil (s n eh1 l g r ow1 v))
("snell" nil (s n eh1 l))
("snellen" nil (s n eh1 l ax n))
("snellenberger" nil (s n eh1 l ax n b er0 g er0))
("sneller" nil (s n eh1 l er0))
("snellgrove" nil (s n eh1 l g r ow1 v))
("snelling" nil (s n eh1 l ih0 ng))
("snellings" nil (s n eh1 l ih0 ng z))
("snelson" nil (s n eh1 l s ax n))
("snethen" nil (s n eh1 th ax n))
("snia" nil (s n iy1 ax))
("snicker" nil (s n ih1 k er0))
("snickered" nil (s n ih1 k er0 d))
("snickering" nil (s n ih1 k er0 ih0 ng))
("snickers" nil (s n ih1 k er0 z))
("snide" nil (s n ay1 d))
("snider" nil (s n ay1 d er0))
("snidow" nil (s n ih1 d ow0))
("sniegowski" nil (s n iy0 g ao1 f s k iy0))
("sniff" nil (s n ih1 f))
("sniffed" nil (s n ih1 f t))
("sniffen" nil (s n ih1 f ax n))
("sniffer" nil (s n ih1 f er0))
("sniffing" nil (s n ih1 f ih0 ng))
("sniffs" nil (s n ih1 f s))
("snip" nil (s n ih1 p))
("snipe" nil (s n ay1 p))
("sniped" nil (s n ay1 p t))
("sniper" nil (s n ay1 p er0))
("snipers" nil (s n ay1 p er0 z))
("snipes" nil (s n ay1 p s))
("sniping" nil (s n ay1 p ih0 ng))
("snippet" nil (s n ih1 p ax t))
("snippets" nil (s n ih1 p ax t s))
("snips" nil (s n ih1 p s))
("snitker" nil (s n ih1 t k er0))
("snitzer" nil (s n ih1 t z er0))
("snively" nil (s n ay1 v l iy0))
("snob" nil (s n aa1 b))
("snobbery" nil (s n aa1 b er0 iy0))
("snobbish" nil (s n aa1 b ih0 sh))
("snobby" nil (s n aa1 b iy0))
("snobs" nil (s n aa1 b z))
("snodderly" nil (s n aa1 d er0 l iy0))
("snoddy" nil (s n aa1 d iy0))
("snodgrass" nil (s n aa1 d g r ae1 s))
("snohomish" nil (s n aa1 hh ax m ih0 sh))
("snoke" nil (s n ow1 k))
("snook" nil (s n uh1 k))
("snooker" nil (s n uh1 k er0))
("snookered" nil (s n uh1 k er0 d))
("snooks" nil (s n uh1 k s))
("snoop" nil (s n uw1 p))
("snooping" nil (s n uw1 p ih0 ng))
("snoopy" nil (s n uw1 p iy0))
("snooty" nil (s n uw1 t iy0))
("snooze" nil (s n uw1 z))
("snoozing" nil (s n uw1 z ih0 ng))
("snore" nil (s n ao1 r))
("snoring" nil (s n ao1 r ih0 ng))
("snort" nil (s n ao1 r t))
("snorted" nil (s n ao1 r t ih0 d))
("snorting" nil (s n ao1 r t ih0 ng))
("snorts" nil (s n ao1 r t s))
("snouffer" nil (s n ow1 f er0))
("snout" nil (s n aw1 t))
("snover" nil (s n ow1 v er0))
("snow" nil (s n ow1))
("snowball" nil (s n ow1 b ao1 l))
("snowballed" nil (s n ow1 b ao1 l d))
("snowballing" nil (s n ow1 b ao1 l ih0 ng))
("snowberger" nil (s n ow1 b er0 g er0))
("snowbirds" nil (s n ow1 b er1 d z))
("snowboarders" nil (s n ow1 b ao1 r d er0 z))
("snowden" nil (s n ow1 d ax n))
("snowdon" nil (s n ow1 d ax n))
("snowe" nil (s n ow1))
("snowed" nil (s n ow1 d))
("snowfall" nil (s n ow1 f ao1 l))
("snowflake" nil (s n ow1 f l ey1 k))
("snowflakes" nil (s n ow1 f l ey1 k s))
("snowing" nil (s n ow1 ih0 ng))
("snowman" nil (s n ow1 m ae1 n))
("snowmobile" nil (s n ow1 m ow0 b iy1 l))
("snowmobiles" nil (s n ow1 m ow0 b iy1 l z))
("snows" nil (s n ow1 z))
("snowshoe" nil (s n ow1 sh uw1))
("snowstorm" nil (s n ow1 s t ao1 r m))
("snowstorms" nil (s n ow1 s t ao1 r m z))
("snowy" nil (s n ow1 iy0))
("snub" nil (s n ah1 b))
("snubbed" nil (s n ah1 b d))
("snubbing" nil (s n ah1 b ih0 ng))
("snuck" nil (s n ah1 k))
("snuff" nil (s n ah1 f))
("snuffed" nil (s n ah1 f t))
("snuffer" nil (s n ah1 f er0))
("snuffing" nil (s n ah1 f ih0 ng))
("snug" nil (s n ah1 g))
("snugging" nil (s n ah1 g ih0 ng))
("snuggle" nil (s n ah1 g ax l))
("snuggs" nil (s n ah1 g z))
("snyder" nil (s n ay1 d er0))
("snydergeneral" nil (s n ay1 d er0 jh eh1 n er0 ax l))
("so" nil (s ow1))
("soady" nil (s ow1 d iy0))
("soak" nil (s ow1 k))
("soaked" nil (s ow1 k t))
("soaking" nil (s ow1 k ih0 ng))
("soaks" nil (s ow1 k s))
("soap" nil (s ow1 p))
("soapbox" nil (s ow1 p b aa1 k s))
("soaps" nil (s ow1 p s))
("soapy" nil (s ow1 p iy0))
("soar" nil (s ao1 r))
("soard" nil (s ao1 r d))
("soared" nil (s ao1 r d))
("soares" nil (s ao1 r eh0 s))
("soaring" nil (s ao1 r ih0 ng))
("soars" nil (s ao1 r z))
("soave" nil (s ow1 v))
("sob" nil (s aa1 b))
("sobbing" nil (s aa1 b ih0 ng))
("sobbingly" nil (s aa1 b ih0 ng l iy0))
("sobczak" nil (s aa1 b ch ae0 k))
("sobczyk" nil (s aa1 b ch ih0 k))
("sobeck" nil (s ow1 b eh1 k))
("sobecki" nil (s ax b eh1 t s k iy0))
("sobek" nil (s ow1 b ih0 k))
("sobel" nil (s ow1 b ax l))
("sobelman" nil (s ow1 b ax l m ax n))
("sober" nil (s ow1 b er0))
("sobered" nil (s ow1 b er0 d))
("sobering" nil (s ow1 b er0 ih0 ng))
("soberly" nil (s ow1 b er0 l iy0))
("sobers" nil (s ow1 b er0 z))
("sobey" nil (s ow1 b iy0))
("sobiech" nil (s aa1 b iy0 hh))
("sobieski" nil (s ow1 b iy0 eh1 s k iy0))
("sobin" nil (s ow1 b ih0 n))
("sobkowiak" nil (s ax b k aw1 iy0 ae0 k))
("soble" nil (s ow1 b ax l))
("sobocinski" nil (s ax b ax ch ih1 n s k iy0))
("sobol" nil (s ow1 b ax l))
("soboleski" nil (s ax b ax l eh1 s k iy0))
("sobolewski" nil (s ax b ax l eh1 f s k iy0))
("sobolik" nil (s ax b ow1 l ih0 k))
("sobon" nil (s ow0 b ao1 n))
("sobota" nil (s ax b ow1 t ax))
("sobotka" nil (s ax b ow1 t k ax))
("sobotta" nil (s ow0 b ow1 t ax))
("sobriety" nil (s ax b r ay1 ax t iy0))
("sobriquet" nil (s ow1 b r ax k ey1))
("sobs" nil (s aa1 b z))
("socalled" nil (s ow1 k aa1 l d))
("socanav" nil (s aa1 k ax n aa0 v))
("socarras" nil (s ow0 k aa1 r aa0 z))
("soccer" nil (s aa1 k er0))
("socci" nil (s ow1 ch iy0))
("socha" nil (s ow1 ch ax))
("sochacki" nil (s ax ch ae1 k iy0))
("sochet" nil (s ow0 sh ey1))
("sochor" nil (s aa1 k er0))
("socia" nil (s ow1 ch ax))
("sociable" nil (s ow1 sh ax b ax l))
("social" nil (s ow1 sh ax l))
("socialism" nil (s ow1 sh ax l ih1 z ax m))
("socialist" nil (s ow1 sh ax l ax s t))
("socialistic" nil (s ow1 sh ax l ih1 s t ih0 k))
("socialists" nil (s ow1 sh ax l ax s t s))
("socialite" nil (s ow1 sh ax l ay1 t))
("socialites" nil (s ow1 sh ax l ay1 t s))
("socialization" nil (s ow1 sh ax l ih0 z ey1 sh ax n))
("socialize" nil (s ow1 sh ax l ay1 z))
("socialized" nil (s ow1 sh ax l ay1 z d))
("socializing" nil (s ow1 sh ax l ay1 z ih0 ng))
("socially" nil (s ow1 sh ax l iy0))
("sociedad" nil (s ow1 s iy0 d ae1 d))
("societa" nil (s ow1 s iy0 eh1 t ax))
("societal" nil (s ax s ay1 ih0 t ax l))
("societe" nil (s ow1 s iy0 eh0 t ey1))
("societies" nil (s ax s ay1 ax t iy0 z))
("society" nil (s ax s ay1 ax t iy0))
("socio" nil (s ow1 s iy0 ow0))
("socioeconomic" nil (s ow0 s iy1 ow1 eh1 k ax n aa1 m ih0 k))
("sociological" nil (s ow1 s iy0 ax l aa1 jh ih0 k ax l))
("sociologist" nil (s ow1 s iy0 aa1 l ax jh ih0 s t))
("sociologists" nil (s ow1 s iy0 aa1 l ax jh ih0 s t s))
("sociology" nil (s ow1 s iy0 aa1 l ax jh iy0))
("sock" nil (s aa1 k))
("socked" nil (s aa1 k t))
("socket" nil (s aa1 k ax t))
("sockets" nil (s aa1 k ax t s))
("socks" nil (s aa1 k s))
("sockwell" nil (s aa1 k w eh1 l))
("socol" nil (s ow1 k aa1 l))
("socrates" nil (s aa1 k r ax t iy1 z))
("socratic" nil (s ax k r ae1 t ih0 k))
("sod" nil (s aa1 d))
("soda" nil (s ow1 d ax))
("sodano" nil (s ow0 d aa1 n ow0))
("sodaro" nil (s ow0 d aa1 r ow0))
("sodas" nil (s ow1 d ax z))
("sodden" nil (s aa1 d ax n))
("sodders" nil (s aa1 d er0 z))
("sodecom" nil (s ow1 d ax k ax m))
("soden" nil (s ow1 d ax n))
("soder" nil (s ow1 d er0))
("soderberg" nil (s ow1 d er0 b er0 g))
("soderbergh" nil (s ow1 d er0 b er0 g))
("soderblom" nil (s ow1 d er0 b l aa1 m))
("sodergren" nil (s aa1 d er0 g r eh0 n))
("soderholm" nil (s ow1 d er0 hh ow0 l m))
("soderlund" nil (s aa1 d er0 l ax n d))
("soderman" nil (s ow1 d er0 m ax n))
("soderquist" nil (s aa1 d er0 k w ih0 s t))
("soderstrom" nil (s aa1 d er0 s t r ax m))
("soditic" nil (s ow0 d ih1 t ih0 k))
("sodium" nil (s ow1 d iy0 ax m))
("sodomy" nil (s aa1 d ax m iy0))
("soeder" nil (s ow1 d er0))
("soens" nil (s ow1 n z))
("soerensen" nil (s ao1 r ax n s ax n))
("sofa" nil (s ow1 f ax))
("sofaer" nil (s ow0 f ey1 r))
("sofamor" nil (s ow1 f ax m ao1 r))
("sofas" nil (s ow1 f ax z))
("sofer" nil (s ow1 f er0))
("soffel" nil (s ao1 f ax l))
("soffer" nil (s ao1 f er0))
("sofia" nil (s ow0 f iy1 ax))
("sofie" nil (s aa1 f iy0))
("sofranko" nil (s ax f r ae1 ng k ow0))
("soft" nil (s aa1 f t))
("softball" nil (s ao1 f t b ao1 l))
("softbank" nil (s ao1 f t b ae1 ng k))
("softdrink" nil (s ao1 f t d r ih1 ng k))
("softech" nil (s ao1 f t eh1 k))
("soften" nil (s aa1 f ax n))
("softened" nil (s ao1 f ax n d))
("softener" nil (s ao1 f ax n er0))
("softening" nil (s ao1 f ax n ih0 ng))
("softens" nil (s ao1 f ax n z))
("softer" nil (s aa1 f t er0))
("softest" nil (s ao1 f t ax s t))
("softimage" nil (s ao1 f t ih1 m ih0 jh))
("softkey" nil (s aa1 f t k ey1))
("softletter" nil (s ao1 f t l eh1 t er0))
("softly" nil (s ao1 f t l iy0))
("softness" nil (s ao1 f t n ax s))
("softsoap" nil (s ao1 f t s ow1 p))
("softspoken" nil (s ao1 f t s p ow1 k ax n))
("software" nil (s ao1 f t w eh1 r))
("softwood" nil (s ao1 f t w uh1 d))
("sogang" nil (s ow1 g ae0 ng))
("soggy" nil (s aa1 g iy0))
("sogo" nil (s ow1 g ow0))
("sohio" nil (s ow0 hh ay1 ow0))
("sohl" nil (s ow1 l))
("sohm" nil (s ow1 m))
("sohmer" nil (s ow1 m er0))
("sohn" nil (s aa1 n))
("sohns" nil (s aa1 n z))
("soho" nil (s ow1 hh ow0))
("soifer" nil (s oy1 f er0))
("soiffer" nil (s oy1 f er0))
("soil" nil (s oy1 l))
("soileau" nil (s oy1 l ow1))
("soiled" nil (s oy1 l d))
("soils" nil (s oy1 l z))
("soir" nil (s oy1 r))
("soiree" nil (s w aa0 r ey1))
("soisson" nil (s oy1 z s ax n))
("soja" nil (s ow1 jh ax))
("sojka" nil (s oy1 k ax))
("sojourn" nil (s ow1 jh er0 n))
("sok" nil (s aa1 k))
("sokaiya" nil (s ax k ay1 y ax))
("sokol" nil (s ow1 k ax l))
("sokolik" nil (s ax k ow1 l ih0 k))
("sokolin" nil (s aa1 k ax l ih0 n))
("sokoloff" nil (s aa1 k ax l ao0 f))
("sokoloski" nil (s ax k ax l aw1 s k iy0))
("sokolov" nil (s aa1 k ax l aa0 v))
("sokolow" nil (s aa1 k ax l ow0))
("sokolowski" nil (s ax k ax l ao1 f s k iy0))
("sol" nil (s aa1 l))
("sola" nil (s ow1 l ax))
("solace" nil (s aa1 l ax s))
("solak" nil (s ow1 l ax k))
("solaman" nil (s aa1 l ax m ax n))
("solan" nil (s ow1 l ax n))
("solana" nil (s ow0 l ae1 n ax))
("soland" nil (s aa1 l ax n d))
("solano" nil (s ow0 l aa1 n ow0))
("solar" nil (s ow1 l er0))
("solares" nil (s ow0 l aa1 r eh0 s))
("solari" nil (s ow0 l aa1 r iy0))
("solaris" nil (s ow0 l eh1 r ih0 s))
("solarz" nil (s ow1 l aa0 r z))
("solazzo" nil (s ow0 l aa1 z ow0))
("solberg" nil (s ow1 l b er0 g))
("solchaga" nil (s ow0 l ch aa1 g ax))
("sold" nil (s ow1 l d))
("soldan" nil (s ow1 l d ax n))
("soldano" nil (s ow0 l d aa1 n ow0))
("solder" nil (s aa1 d er0))
("soldering" nil (s aa1 d er0 ih0 ng))
("soldier" nil (s ow1 l jh er0))
("soldiering" nil (s ow1 l jh er0 ih0 ng))
("soldiers" nil (s ow1 l jh er0 z))
("soldner" nil (s ow1 l d n er0))
("soldo" nil (s ow1 l d ow0))
("sole" nil (s ow1 l))
("solecki" nil (s ax l eh1 t s k iy0))
("soledad" nil (s ow1 l d ae1 d))
("soleil" nil (s ow0 l ay1 l))
("solely" nil (s ow1 l l iy0))
("solem" nil (s ow1 l ih0 m))
("solemn" nil (s aa1 l ax m))
("solemnity" nil (s ax l eh1 m n ax t iy0))
("solemnly" nil (s ao1 l ax m l iy0))
("soler" nil (s ow1 l er0))
("soleri" nil (s ow0 l eh1 r iy0))
("soles" nil (s ow1 l z))
("solesbee" nil (s ow1 l z b iy1))
("soley" nil (s ow1 l iy0))
("solheim" nil (s ow1 l hh ay1 m))
("soli" nil (s ow1 l iy1))
("solicit" nil (s ax l ih1 s ih0 t))
("solicitation" nil (s ax l ih1 s ih0 t ey1 sh ax n))
("solicitations" nil (s ax l ih1 s ih0 t ey1 sh ax n z))
("solicited" nil (s ax l ih1 s ih0 t ih0 d))
("soliciting" nil (s ax l ih1 s ax t ih0 ng))
("solicitor" nil (s ax l ih1 s ax t er0))
("solicitors" nil (s ax l ih1 s ax t er0 z))
("solicitous" nil (s ax l ih1 s ax t ax s))
("solicits" nil (s ax l ih1 s ax t s))
("solicitude" nil (s ax l ih1 s ih0 t uw1 d))
("solid" nil (s aa1 l ax d))
("solidarity" nil (s aa1 l ax d eh1 r ax t iy0))
("soliday" nil (s ow1 l iy0 d ey0))
("soliders" nil (s aa1 l ih0 d er0 z))
("solidified" nil (s ax l ih1 d ax f ay1 d))
("solidifies" nil (s ax l ih1 d ax f ay1 z))
("solidify" nil (s ax l ih1 d ax f ay1))
("solidifying" nil (s ax l ih1 d ax f ay1 ih0 ng))
("solidity" nil (s ax l ih1 d ax t iy0))
("solidly" nil (s aa1 l ax d l iy0))
("solids" nil (s aa1 l ax d z))
("solie" nil (s ow1 l iy0))
("soliloquy" nil (s ow1 l ax l aa1 k w iy0))
("soliman" nil (s aa1 l ih0 m ax n))
("solimine" nil (s ow0 l iy0 m iy1 n iy0))
("solimon" nil (s ow1 l ih0 m ax n))
("solin" nil (s ow1 l ih0 n))
("solinger" nil (s ow1 l ih0 ng er0))
("solis" nil (s ow1 l ih0 s))
("solita" nil (s ow0 l iy1 t ax))
("solitaire" nil (s aa1 l ax t eh1 r))
("solitary" nil (s aa1 l ax t eh1 r iy0))
("solitec" nil (s aa1 l ih0 t eh1 k))
("solitron" nil (s ow1 l ih0 t r aa0 n))
("solitude" nil (s aa1 l ax t uw1 d))
("soliz" nil (s ow1 l iy0 z))
("soll" nil (s aa1 l))
("sollars" nil (s aa1 l er0 z))
("sollenberger" nil (s aa1 l ax n b er0 g er0))
("soller" nil (s aa1 l er0))
("sollers" nil (s aa1 l er0 z))
("solley" nil (s aa1 l iy0))
("solliday" nil (s aa1 l iy0 d ey0))
("sollie" nil (s aa1 l iy0))
("sollinger" nil (s aa1 l ih0 ng er0))
("sollish" nil (s aa1 l ih0 sh))
("solloway" nil (s aa1 l ow0 w ey1))
("solly" nil (s aa1 l iy0))
("solo" nil (s ow1 l ow1))
("solodar" nil (s aa1 l ax d er0))
("soloff" nil (s aa1 l ao0 f))
("soloist" nil (s ow1 l ow1 ax s t))
("soloists" nil (s ow1 l ow1 ax s t s))
("soloman" nil (s ow0 l ow0 m ae1 n))
("solomon" nil (s aa1 l ax m ax n))
("solomos" nil (s aa1 l ax m ow0 s))
("solon" nil (s ow1 l ax n))
("solorio" nil (s ow0 l ao1 r iy0 ow0))
("solorzano" nil (s ow0 l ao0 r z aa1 n ow0))
("solos" nil (s ow1 l ow0 z))
("soloviev" nil (s aa1 l ow0 v iy1 v))
("solow" nil (s aa1 l ow0))
("soloway" nil (s ow1 l ow0 w ey1))
("solt" nil (s ow1 l t))
("soltau" nil (s ow1 l t aw0))
("soltero" nil (s ow0 l t eh1 r ow0))
("soltes" nil (s ow1 l t s))
("soltesz" nil (s ow1 l t ih0 sh))
("solti" nil (s ow1 l t iy0))
("soltis" nil (s ow1 l t ih0 s))
("soltys" nil (s ow1 l t iy0 z))
("soltysiak" nil (s ow0 l t ih1 s iy0 ae0 k))
("soluble" nil (s aa1 l y ax b ax l))
("solum" nil (s ow1 l ax m))
("solution" nil (s ax l uw1 sh ax n))
("solutions" nil (s ax l uw1 sh ax n z))
("solvable" nil (s aa1 l v ax b ax l))
("solvay" nil (s ow1 l v ey0))
("solve" nil (s aa1 l v))
("solved" nil (s aa1 l v d))
("solvency" nil (s ao1 l v ax n s iy0))
("solvent" nil (s aa1 l v ax n t))
("solvents" nil (s ao1 l v ax n t s))
("solver" nil (s aa1 l v er0))
("solvers" nil (s aa1 l v er0 z))
("solves" nil (s aa1 l v z))
("solvig" nil (s ow1 l v ih0 g))
("solving" nil (s aa1 l v ih0 ng))
("solwin" nil (s ow1 l w ih1 n))
("solzhenitsyn" nil (s ow1 l z ax n ih1 t s ih1 n))
("som" nil (s aa1 m))
("soma" nil (s ow1 m ax))
("somali" nil (s ax m aa1 l iy0))
("somalia" nil (s ax m aa1 l iy0 ax))
("somalis" nil (s ax m aa1 l ih0 z))
("somatogen" nil (s ow1 m ae1 t ax jh eh0 n))
("somatotropin" nil (s ow1 m ax t aa1 t r ax p ih0 n))
("somber" nil (s aa1 m b er0))
("somberly" nil (s aa1 m b er0 l iy0))
("sombrero" nil (s aa0 m b r eh1 r ow0))
("some" nil (s ah1 m))
("somebody" nil (s ah1 m b aa1 d iy0))
("someday" nil (s ah1 m d ey1))
("somehow" nil (s ah1 m hh aw1))
("someone" nil (s ah1 m w ah1 n))
("someplace" nil (s ah1 m p l ey1 s))
("somer" nil (s ah1 m er0))
("somers" nil (s ah1 m er0 z))
("somersaulting" nil (s ah1 m er0 s ao1 l t ih0 ng))
("somersaults" nil (s ah1 m er0 s ao1 l t s))
("somerset" nil (s ah1 m er0 s eh1 t))
("somerton" nil (s ah1 m er0 t ax n))
("somerville" nil (s ah1 m er0 v ih1 l))
("somes" nil (s ah1 m z))
("somesh" nil (s ow1 m eh1 sh))
("something" nil (s ah1 m th ih0 ng))
("somethings" nil (s ah1 m th ih1 ng z))
("sometime" nil (s ah1 m t ay1 m))
("sometimes" nil (s ax m t ay1 m z))
("somewhat" nil (s ah1 m w ah1 t))
("somewhere" nil (s ah1 m w eh1 r))
("somma" nil (s aa1 m ax))
("sommer" nil (s ah1 m er0))
("sommerfeld" nil (s aa1 m er0 f eh0 l d))
("sommerfeldt" nil (s aa1 m er0 f ih0 l t))
("sommerfield" nil (s ax m er1 f iy0 l d))
("sommers" nil (s ah1 m er0 r z))
("sommerville" nil (s aa1 m er0 v ih0 l))
("somnolent" nil (s aa1 m n ax l ax n t))
("somogyi" nil (s ow0 m ow1 g y iy0))
("somoza" nil (s ax m ow1 z ax))
("son" nil (s ah1 n))
("sonar" nil (s ow1 n aa0 r))
("sonat" nil (s aa1 n ax t))
("sonata" nil (s ax n aa1 t ax))
("sonatas" nil (s aa1 n aa1 t ax z))
("sonatrach" nil (s aa1 n ax t r ae0 k))
("sondag" nil (s aa1 n d ax g))
("sonderman" nil (s aa1 n d er0 m ax n))
("sondgeroth" nil (s aa1 n jh er0 aa0 th))
("sondheim" nil (s aa1 n d hh ay1 m))
("sondheimer" nil (s aa1 n d hh ay1 m er0))
("sondra" nil (s aa1 n d r ax))
("sones" nil (s ow1 n z))
("sonesta" nil (s ax n eh1 s t ax))
("sonet" nil (s ow1 n ax t))
("sonex" nil (s ow1 n ax k s))
("song" nil (s ao1 ng))
("songbirds" nil (s ao1 ng b er1 d z))
("songer" nil (s ao1 ng er0))
("songs" nil (s ao1 ng z))
("songwriter" nil (s ao1 ng r ay1 t er0))
("songwriters" nil (s ao1 ng r ay1 t er0 z))
("songwriting" nil (s ao1 ng r ay1 t ih0 ng))
("songy" nil (s aa1 n jh iy0))
("soni" nil (s ow1 n iy0))
("sonia" nil (s ow1 n y ax))
("sonic" nil (s aa1 n ih0 k))
("sonier" nil (s ow1 n iy0 er0))
("sonja" nil (s ow1 n y ax))
("sonn" nil (s aa1 n))
("sonne" nil (s aa1 n))
("sonneborn" nil (s aa1 n ih0 b ao0 r n))
("sonnen" nil (s aa1 n ax n))
("sonnenberg" nil (s aa1 n ax n b er0 g))
("sonnenblick" nil (s ax n eh1 n b l ih0 k))
("sonnenburg" nil (s aa1 n ax n b er0 g))
("sonnenfeld" nil (s aa1 n ih0 n f eh0 l d))
("sonnenschein" nil (s aa1 n ih0 n sh ay0 n))
("sonner" nil (s aa1 n er0))
("sonnets" nil (s aa1 n ax t s))
("sonnett" nil (s aa1 n ax t))
("sonnier" nil (s ah1 n iy0 er0))
("sonntag" nil (s aa1 n t ax g))
("sonny" nil (s ah1 n iy0))
("sonoco" nil (s ax n ow1 k ow0))
("sonoda" nil (s ow0 n ow1 d ax))
("sonoma" nil (s ax n ow1 m ax))
("sonora" nil (s ax n ao1 r ax))
("sonorous" nil (s aa1 n er0 ax s))
("sonrise" nil (s ah1 n r ay1 z))
("sons" nil (s ah1 n z))
("sontag" nil (s aa1 n t ae1 g))
("sony" nil (s ow1 n iy0))
("sonya" nil (s ow1 n y ax))
("soo" nil (s uw1))
("sood" nil (s uw1 d))
("soohoo" nil (s uw1 hh uw1))
("soon" nil (s uw1 n))
("sooner" nil (s uw1 n er0))
("sooners" nil (s uw1 n er0 z))
("soonest" nil (s uw1 n ax s t))
("soong" nil (s uw1 ng))
("soos" nil (s uw1 z))
("soot" nil (s uh1 t))
("sooter" nil (s uh1 t er0))
("soothe" nil (s uw1 dh))
("soothed" nil (s uw1 dh d))
("soothes" nil (s uw1 dh z))
("soothing" nil (s uw1 dh ih0 ng))
("soothingly" nil (s uw1 dh ih0 ng l iy0))
("soothsayers" nil (s uw1 th s ey1 er0 z))
("soots" nil (s uh1 t s))
("sooty" nil (s uw1 t iy0))
("sooy" nil (s uw1 iy0))
("sop" nil (s aa1 p))
("sope" nil (s ow1 p))
("soper" nil (s ow1 p er0))
("sopher" nil (s aa1 f er0))
("sophia" nil (s ow0 f iy1 ax))
("sophie" nil (s ow1 f iy0))
("sophisticated" nil (s ax f ih1 s t ax k ey1 t ih0 d))
("sophisticates" nil (s ax f ih1 s t ax k ih1 t s))
("sophistication" nil (s ax f ih1 s t ax k ey1 sh ax n))
("sophocles" nil (s ow0 f ow1 k ax l z))
("sophomore" nil (s aa1 f m ao1 r))
("sophomores" nil (s aa1 f m ao1 r z))
("sophomoric" nil (s aa1 f ow0 m aa1 r ih0 k))
("sophronia" nil (s ow0 f r ow1 n iy0 ax))
("sophy" nil (s ow1 f iy0))
("sopko" nil (s ow1 p k ow0))
("sopp" nil (s aa1 p))
("sopping" nil (s aa1 p ih0 ng))
("soprano" nil (s ax p r aa1 n ow0))
("sopranos" nil (s ax p r ae1 n ow0 z))
("sops" nil (s aa1 p s))
("soquip" nil (s ow1 k w ih0 p))
("sor" nil (s ao1 r))
("soranno" nil (s ao0 r aa1 n ow0))
("sorbello" nil (s ao1 r b eh1 l ow0))
("sorber" nil (s ao1 r b er0))
("sorbo" nil (s ao1 r b ow0))
("sorbonne" nil (s ao0 r b aa1 n))
("sorbus" nil (s ao1 r b ax s))
("sorce" nil (s ao1 r s))
("sorcerers" nil (s ao1 r s er0 er0 z))
("sorcery" nil (s ao1 r s er0 iy0))
("sorcha" nil (s ao1 r k ax))
("sorci" nil (s ao1 r ch iy0))
("sordid" nil (s ao1 r d ax d))
("sordoni" nil (s ao0 r d ow1 n iy0))
("sore" nil (s ao1 r))
("sorel" nil (s ao1 r ax l))
("sorell" nil (s ao1 r ax l))
("sorely" nil (s ao1 r l iy0))
("sorensen" nil (s ao1 r ih0 n s ax n))
("sorenson" nil (s ao1 r ax n s ax n))
("sores" nil (s ao1 r z))
("sorey" nil (s ao1 r iy0))
("sorg" nil (s ao1 r g))
("sorge" nil (s ao1 r jh))
("sorgen" nil (s ao1 r g ax n))
("sorghum" nil (s ao1 r g ax m))
("soria" nil (s ao1 r iy0 ax))
("soriano" nil (s ao0 r iy0 aa1 n ow0))
("sorice" nil (s ao1 r ih0 s))
("sorkin" nil (s ao1 r k ih0 n))
("sorkow" nil (s ao1 r k aw1))
("sorlie" nil (s ao1 r l iy0))
("soroka" nil (s ao0 r ow1 k ax))
("sororities" nil (s er0 ao1 r ax t iy0 z))
("sorority" nil (s er0 ao1 r ax t iy0))
("soros" nil (s ao1 r ow0 s))
("sorpasso" nil (s ao0 r p aa1 s ow0))
("sorrel" nil (s ao1 r ax l))
("sorrell" nil (s ao1 r ax l))
("sorrells" nil (s ao1 r ax l z))
("sorrels" nil (s ao1 r ax l z))
("sorrenti" nil (s ao0 r eh1 n t iy0))
("sorrow" nil (s aa1 r ow0))
("sorrowful" nil (s aa1 r ow0 f ax l))
("sorry" nil (s aa1 r iy0))
("sort" nil (s ao1 r t))
("sorted" nil (s ao1 r t ih0 d))
("sorter" nil (s ao1 r t er0))
("sorters" nil (s ao1 r t er0 z))
("sorties" nil (s ao1 r t iy0 z))
("sorting" nil (s ao1 r t ih0 ng))
("sortino" nil (s ao0 r t iy1 n ow0))
("sortor" nil (s ao1 r t er0))
("sorts" nil (s ao1 r t s))
("sorum" nil (s ao1 r ax m))
("sosa" nil (s ow1 s ax))
("sosebee" nil (s aa1 s ih0 b iy0))
("sosin" nil (s ow1 s ih1 n))
("sosinski" nil (s ax s ih1 n s k iy0))
("soskin" nil (s aa1 s k ih0 n))
("sosna" nil (s ow1 s n ax))
("sosnoff" nil (s ao1 s n ao0 f))
("sosnowski" nil (s ax s n ao1 f s k iy0))
("soss" nil (s ao1 s))
("sossamon" nil (s ow0 s aa0 m ao1 n))
("sosuke" nil (s ow0 s uw1 k ey1))
("sotak" nil (s ow1 t ax k))
("sotello" nil (s ow0 t eh1 l ow0))
("sotelo" nil (s ow0 t eh1 l ow0))
("soter" nil (s ow1 t er0))
("soto" nil (s ow1 t ow0))
("sotolongo" nil (s ow0 t ow0 l ow1 ng g ow0))
("sotomayor" nil (s ow0 t ow0 m ey0 ao1 r))
("sottile" nil (s ow1 t ax l))
("souccar" nil (s uw1 k aa1 r))
("soucek" nil (s ow1 s ih0 k))
("soucie" nil (s ow1 k iy0))
("soucy" nil (s ow1 s iy0))
("souder" nil (s aw1 d er0))
("souders" nil (s aw1 d er0 z))
("souers" nil (s aw1 er0 z))
("sought" nil (s ao1 t))
("souk" nil (s uw1 k))
("soukup" nil (s aw1 k ax p))
("soul" nil (s ow1 l))
("soule" nil (s aw1 l))
("soules" nil (s uw1 l z))
("soulful" nil (s ow1 l f ax l))
("soulier" nil (s uw1 l iy0 er0))
("soulless" nil (s ow1 l l ax s))
("soulliere" nil (s uw1 l iy0 eh0 r))
("souls" nil (s ow1 l z))
("sound" nil (s aw1 n d))
("soundbites" nil (s aw1 n d b ay1 t s))
("sounded" nil (s aw1 n d ax d))
("sounder" nil (s aw1 n d er0))
("soundest" nil (s aw1 n d ax s t))
("sounding" nil (s aw1 n d ih0 ng))
("soundings" nil (s aw1 n d ih0 ng z))
("soundly" nil (s aw1 n d l iy0))
("soundness" nil (s aw1 n d n ax s))
("sounds" nil (s aw1 n d z))
("soundscan" nil (s aw1 n d s k ae1 n))
("soundtrack" nil (s aw1 n d t r ae1 k))
("soundview" nil (s aw1 n d v y uw1))
("soup" nil (s uw1 p))
("souped" nil (s uw1 p t))
("soups" nil (s uw1 p s))
("soupy" nil (s uw1 p iy0))
("sour" nil (s aw1 er0))
("source" nil (s ao1 r s))
("sourcebook" nil (s ao1 r s b uh1 k))
("sources" nil (s ao1 r s ax z))
("sourcing" nil (s ao1 r s ih0 ng))
("soured" nil (s aw1 er0 d))
("souring" nil (s aw1 er0 ih0 ng))
("souris" nil (s uw1 r ih0 s))
("sourrouille" nil (s ao0 r uw1 iy0))
("sours" nil (s aw1 er0 z))
("sous" nil (s uw1 z))
("sousa" nil (s uw1 z ax))
("sousaphone" nil (s uw1 z ax f ow1 n))
("sousley" nil (s aw1 s l iy0))
("soutar" nil (s uw0 t aa1 r))
("souter" nil (s uw1 t er0))
("south" nil (s aw1 th))
("southall" nil (s aw1 th ax l))
("southam" nil (s aw1 th ax m))
("southampton" nil (s aw0 th hh ae1 m p t ax n))
("southard" nil (s aw1 th er0 d))
("southbound" nil (s aw1 th b aw1 n d))
("southdown" nil (s aw1 th d aw1 n))
("southeast" nil (s aw1 th iy1 s t))
("southeastern" nil (s aw1 th iy1 s t er0 n))
("souther" nil (s ah1 dh er0))
("southerland" nil (s ah1 dh er0 l ax n d))
("southern" nil (s ah1 dh er0 n))
("southerner" nil (s ah1 dh er0 n er0))
("southerners" nil (s ah1 dh er0 n er0 z))
("southernmost" nil (s ah1 dh er0 n m ow1 s t))
("southernnet" nil (s ah1 dh er0 n eh0 t))
("southers" nil (s ah1 dh er0 z))
("southfield" nil (s aw1 th f iy1 l d))
("southgate" nil (s aw1 th g ey1 t))
("southin" nil (s aw1 th ih1 n))
("southland" nil (s aw1 th l ae1 n d))
("southlife" nil (s aw1 th l ay1 f))
("southmark" nil (s aw1 th m aa1 r k))
("southold" nil (s aw1 th ow1 l d))
("southport" nil (s aw1 th p ao1 r t))
("souths" nil (s aw1 th s))
("southside" nil (s aw1 th s ay1 d))
("southstate" nil (s aw1 th s t ey1 t))
("southwall" nil (s aw1 th w ao1 l))
("southward" nil (s aw1 th w er0 d))
("southway" nil (s aw1 th w ey1))
("southwell" nil (s aw1 th w eh1 l))
("southwest" nil (s aw1 th w eh1 s t))
("southwestern" nil (s aw1 th w eh1 s t er0 n))
("southwick" nil (s aw1 th w ih0 k))
("southwood" nil (s aw1 th w uh1 d))
("southworth" nil (s aw1 th w er0 th))
("souto" nil (s uw1 t ow0))
("souvenir" nil (s uw1 v ax n ih1 r))
("souvenirs" nil (s uw1 v ax n ih1 r z))
("souveroff" nil (s uw1 v er0 ao0 f))
("souza" nil (s uw1 z ax))
("sova" nil (s ow1 v ax))
("sovereign" nil (s aa1 v r ax n))
("sovereigns" nil (s aa1 v r ax n z))
("sovereignty" nil (s aa1 v r ax n t iy0))
("sovetskaya" nil (s ow1 v eh0 t s k ay1 ax))
("sovexportfilm" nil (s ow1 v eh1 k s p ao0 r t f ih1 l m))
("sovey" nil (s ow1 v iy0))
("soviet" nil (s ow1 v iy0 ax t))
("sovietologist" nil (s ow1 v iy0 ax t aa1 l ax jh ih0 s t))
("sovietologists" nil (s ow1 v iy0 ax t aa1 l ax jh ih0 s t s))
("soviets" nil (s ow1 v iy0 eh1 t s))
("sovine" nil (s aa1 v ay0 n))
("sovran" nil (s aa1 v r ax n))
("sovrans" nil (s aa1 v r ax n z))
("sow" nil (s aw1))
("sowa" nil (s ow1 ax))
("soward" nil (s ow1 er0 d))
("sowards" nil (s ow1 er0 d z))
("sowash" nil (s ow1 ax sh))
("sowata" nil (s ow1 aa1 t ax))
("sowden" nil (s ow1 d ax n))
("sowder" nil (s ow1 d er0))
("sowders" nil (s ow1 d er0 z))
("sowed" nil (s aw1 d))
("sowell" nil (s aa1 w eh0 l))
("sower" nil (s ow1 er0))
("sowers" nil (s ow1 er0 z))
("soweto" nil (s ow0 ey1 t ow0))
("sowing" nil (s ow1 ih0 ng))
("sowinski" nil (s ow0 ih1 n s k iy0))
("sowle" nil (s ow1 l))
("sowles" nil (s ow1 l z))
("sown" nil (s ow1 n))
("sows" nil (s ow1 z))
("sox" nil (s aa1 k s))
("soy" nil (s oy1))
("soya" nil (s oy1 ax))
("soyars" nil (s oy1 er0 z))
("soybean" nil (s oy1 b iy1 n))
("soybeans" nil (s oy1 b iy1 n z))
("soyka" nil (s oy1 k ax))
("soysauce" nil (s oy1 s ao1 s))
("soyuz" nil (s oy1 ax z))
("soza" nil (s ow1 z ax))
("sozio" nil (s ow1 z iy0 ow0))
("spa" nil (s p aa1))
("space" nil (s p ey1 s))
("spaceballs" nil (s p ey1 s b ao1 l z))
("spacebands" nil (s p ey1 s b ae1 n d z))
("spacecraft" nil (s p ey1 s k r ae1 f t))
("spaced" nil (s p ey1 s t))
("spacek" nil (s p aa1 ch eh0 k))
("spacelink" nil (s p ey1 s l ih1 ng k))
("spacenet" nil (s p ey1 s n eh1 t))
("spaceport" nil (s p ey1 s p ao1 r t))
("spacers" nil (s p ey1 s er0 z))
("spaces" nil (s p ey1 s ax z))
("spaceship" nil (s p ey1 s sh ih1 p))
("spaceships" nil (s p ey1 s sh ih1 p s))
("spacesuit" nil (s p ey1 s uw1 t))
("spacey" nil (s p ey1 s iy0))
("spacing" nil (s p ey1 s ih0 ng))
("spacious" nil (s p ey1 sh ax s))
("spack" nil (s p ae1 k))
("spackman" nil (s p ae1 k m ax n))
("spada" nil (s p aa1 d ax))
("spadaccini" nil (s p aa0 d aa0 ch iy1 n iy0))
("spadafora" nil (s p aa0 d aa0 f ao1 r ax))
("spadafore" nil (s p aa0 d ao1 f ao0 r))
("spadaro" nil (s p aa0 d aa1 r ow0))
("spade" nil (s p ey1 d))
("spader" nil (s p ey1 d er0))
("spades" nil (s p ey1 d z))
("spadework" nil (s p ey1 d w er1 k))
("spadoni" nil (s p aa0 d ow1 n iy0))
("spady" nil (s p ey1 d iy0))
("spaeth" nil (s p iy1 th))
("spafford" nil (s p ae1 f er0 d))
("spaghetti" nil (s p ax g eh1 t iy0))
("spagna" nil (s p ae1 g n ax))
("spagnola" nil (s p aa0 g n ow1 l ax))
("spagnoli" nil (s p aa0 g n ow1 l iy0))
("spagnolo" nil (s p aa0 g n ow1 l ow0))
("spagnuolo" nil (s p aa0 g n y uw0 ow1 l ow0))
("spago" nil (s p ey1 g ow0))
("spahn" nil (s p aa1 n))
("spahr" nil (s p aa1 r))
("spaid" nil (s p ey1 d))
("spain" nil (s p ey1 n))
("spainhour" nil (s p ay1 n aw0 r))
("spainhower" nil (s p ay1 n hh ow0 er0))
("spak" nil (s p ae1 k))
("spake" nil (s p ey1 k))
("spalding" nil (s p ao1 l d ih0 ng))
("spall" nil (s p ao1 l))
("spalla" nil (s p ae1 l ax))
("spallone" nil (s p ae1 l ow1 n))
("spalvins" nil (s p ae1 l v ih0 n z))
("spam" nil (s p ae1 m))
("spampinato" nil (s p aa0 m p iy0 n aa1 t ow0))
("span" nil (s p ae1 n))
("spanbauer" nil (s p ae1 n b aw0 er0))
("spandex" nil (s p ae1 n d ax k s))
("spang" nil (s p ae1 ng))
("spangenberg" nil (s p ae1 ng ax n b er0 g))
("spangle" nil (s p ae1 ng g ax l))
("spangled" nil (s p ae1 ng g ax l d))
("spangler" nil (s p ae1 ng g ax l er0))
("spaniard" nil (s p ae1 n y er0 d))
("spaniards" nil (s p ae1 n y er0 d z))
("spaniel" nil (s p ae1 n y ax l))
("spanier" nil (s p ae1 n y er0))
("spaniol" nil (s p ae1 n y ax l))
("spanish" nil (s p ae1 n ih0 sh))
("spanked" nil (s p ae1 ng k t))
("spanking" nil (s p ae1 ng k ih0 ng))
("spann" nil (s p ae1 n))
("spanned" nil (s p ae1 n d))
("spanner" nil (s p ae1 n er0))
("spanning" nil (s p ae1 n ih0 ng))
("spanninger" nil (s p ae1 n ih0 ng er0))
("spano" nil (s p aa1 n ow0))
("spanos" nil (s p ey1 n ow0 z))
("spans" nil (s p ae1 n z))
("spanton" nil (s p ae1 n t ax n))
("spar" nil (s p aa1 r))
("sparacino" nil (s p er0 ax ch iy1 n ow0))
("sparacio" nil (s p er0 ey1 s iy0 ow0))
("sparaco" nil (s p er0 ae1 k ow0))
("sparano" nil (s p er0 ae1 n ow0))
("sparc" nil (s p aa1 r k))
("spare" nil (s p eh1 r))
("spared" nil (s p eh1 r d))
("spares" nil (s p eh1 r z))
("sparger" nil (s p aa1 r jh er0))
("spargo" nil (s p aa1 r g ow0))
("spargur" nil (s p aa1 r g er0))
("sparing" nil (s p eh1 r ih0 ng))
("sparingly" nil (s p eh1 r ih0 ng l iy0))
("spark" nil (s p aa1 r k))
("sparked" nil (s p aa1 r k t))
("sparkes" nil (s p aa1 r k s))
("sparking" nil (s p aa1 r k ih0 ng))
("sparkle" nil (s p aa1 r k ax l))
("sparkled" nil (s p aa1 r k ax l d))
("sparkles" nil (s p aa1 r k ax l z))
("sparkling" nil (s p aa1 r k l ih0 ng))
("sparkman" nil (s p aa1 r k m ax n))
("sparks" nil (s p aa1 r k s))
("sparky" nil (s p aa1 r k iy0))
("sparlin" nil (s p aa1 r l ih0 n))
("sparling" nil (s p aa1 r l ih0 ng))
("sparr" nil (s p ae1 r))
("sparred" nil (s p aa1 r d))
("sparring" nil (s p aa1 r ih0 ng))
("sparrow" nil (s p eh1 r ow0))
("sparrows" nil (s p eh1 r ow0 z))
("sparse" nil (s p aa1 r s))
("sparsely" nil (s p aa1 r s l iy0))
("sparta" nil (s p aa1 r t ax))
("spartacus" nil (s p aa1 r t ax k ax s))
("spartan" nil (s p aa1 r t ax n))
("spartanburg" nil (s p aa1 r t ax n b er0 g))
("spartech" nil (s p aa1 r t eh1 k))
("spartz" nil (s p aa1 r t s))
("spas" nil (s p aa1 z))
("spasm" nil (s p ae1 z ax m))
("spasms" nil (s p ae1 z ax m z))
("spasso" nil (s p ae1 s ow0))
("spat" nil (s p ae1 t))
("spatafora" nil (s p aa0 t aa0 f ao1 r ax))
("spatafore" nil (s p ae1 t ax f ao1 r))
("spataro" nil (s p aa0 t aa1 r ow0))
("spate" nil (s p ey1 t))
("spates" nil (s p ey1 t s))
("spath" nil (s p ae1 th))
("spatial" nil (s p ey1 sh ax l))
("spatola" nil (s p aa0 t ow1 l ax))
("spats" nil (s p ae1 t s))
("spattered" nil (s p ae1 t er0 d))
("spattering" nil (s p ae1 t er0 ih0 ng))
("spatula" nil (s p ae1 ch uh0 l ax))
("spatz" nil (s p ae1 t s))
("spaugh" nil (s p ao1))
("spaulding" nil (s p ao1 l d ih0 ng))
("spaur" nil (s p ao1 r))
("spavo" nil (s p aa1 v ow0))
("spaw" nil (s p ao1))
("spawn" nil (s p aa1 n))
("spawned" nil (s p ao1 n d))
("spawning" nil (s p aa1 n ih0 ng))
("spawns" nil (s p aa1 n z))
("spayd" nil (s p ey1 d))
("spayde" nil (s p ey1 d))
("spaziani" nil (s p aa0 z iy0 aa1 n iy0))
("speagle" nil (s p iy1 g ax l))
("speak" nil (s p iy1 k))
("speake" nil (s p iy1 k))
("speaker" nil (s p iy1 k er0))
("speakers" nil (s p iy1 k er0 z))
("speakership" nil (s p iy1 k er0 sh ih1 p))
("speakes" nil (s p iy1 k s))
("speaking" nil (s p iy1 k ih0 ng))
("speakman" nil (s p iy1 k m ax n))
("speaks" nil (s p iy1 k s))
("spear" nil (s p ih1 r))
("speare" nil (s p iy1 r))
("spearhead" nil (s p ih1 r hh eh1 d))
("spearheaded" nil (s p ih1 r hh eh1 d ih0 d))
("spearheading" nil (s p ih1 r hh eh1 d ih0 ng))
("spearing" nil (s p ih1 r ih0 ng))
("spearman" nil (s p ih1 r m ax n))
("spears" nil (s p ih1 r z))
("speas" nil (s p iy1 z))
("spease" nil (s p iy1 z))
("spec" nil (s p eh1 k))
("specht" nil (s p eh1 k t))
("special" nil (s p eh1 sh ax l))
("speciale" nil (s p eh1 ch ax l iy0))
("specialist" nil (s p eh1 sh ax l ax s t))
("specialists" nil (s p eh1 sh ax l ax s t s))
("specialities" nil (s p eh1 sh ax l t iy0 z))
("speciality" nil (s p eh1 sh iy0 ae1 l ih0 t iy0))
("specialization" nil (s p eh1 sh ax l ax z ey1 sh ax n))
("specialize" nil (s p eh1 sh ax l ay1 z))
("specialized" nil (s p eh1 sh ax l ay1 z d))
("specializes" nil (s p eh1 sh ax l ay1 z ax z))
("specializing" nil (s p eh1 sh ax l ay1 z ih0 ng))
("specially" nil (s p eh1 sh ax l iy0))
("specials" nil (s p eh1 sh ax l z))
("specialties" nil (s p eh1 sh ax l t iy0 z))
("specialty" nil (s p eh1 sh ax l t iy0))
("species" nil (s p iy1 sh iy0 z))
("specific" nil (s p ax s ih1 f ih0 k))
("specifically" nil (s p ax s ih1 f ih0 k l iy0))
("specification" nil (s p eh1 s ih0 f ih0 k ey1 sh ax n))
("specifications" nil (s p eh1 s ax f ax k ey1 sh ax n z))
("specificity" nil (s p eh1 s ax f ih1 s ax t iy0))
("specifics" nil (s p ih0 s ih1 f ih0 k s))
("specified" nil (s p eh1 s ax f ay1 d))
("specifies" nil (s p eh1 s ax f ay1 z))
("specify" nil (s p eh1 s ax f ay1))
("specifying" nil (s p eh1 s ax f ay1 ih0 ng))
("specimen" nil (s p eh1 s ax m ax n))
("specimens" nil (s p eh1 s ax m ax n z))
("specious" nil (s p iy1 sh ax s))
("speck" nil (s p eh1 k))
("specker" nil (s p eh1 k er0))
("speckled" nil (s p eh1 k ax l d))
("speckman" nil (s p eh1 k m ax n))
("specks" nil (s p eh1 k s))
("specs" nil (s p eh1 k s))
("spectacle" nil (s p eh1 k t ax k ax l))
("spectacles" nil (s p eh1 k t ax k ax l z))
("spectacular" nil (s p eh0 k t ae1 k y ax l er0))
("spectacularly" nil (s p eh0 k t ae1 k y ax l er0 l iy0))
("spectator" nil (s p eh1 k t ey0 t er0))
("spectators" nil (s p eh1 k t ey0 t er0 z))
("specter" nil (s p eh1 k t er0))
("specthrie" nil (s p eh1 k th r iy0))
("spector" nil (s p eh1 k t er0))
("spectra" nil (s p eh1 k t r ax))
("spectradyne" nil (s p eh1 k t r ax d ay1 n))
("spectramed" nil (s p eh1 k t r ax m d))
("spectran" nil (s p eh1 k t r ae1 n))
("spectravision" nil (s p eh1 k t r ax v ih1 zh ax n))
("spectre" nil (s p eh1 k t er0))
("spectrometer" nil (s p eh0 k t r aa1 m ax t er0))
("spectroscopy" nil (s p eh0 k t r aa1 s k ax p iy0))
("spectrum" nil (s p eh1 k t r ax m))
("speculate" nil (s p eh1 k y ax l ey1 t))
("speculated" nil (s p eh1 k y ax l ey1 t ax d))
("speculates" nil (s p eh1 k y ax l ey1 t s))
("speculating" nil (s p eh1 k y ax l ey1 t ih0 ng))
("speculation" nil (s p eh1 k y ax l ey1 sh ax n))
("speculations" nil (s p eh1 k y ax l ey1 sh ax n z))
("speculative" nil (s p eh1 k y ax l ax t ih0 v))
("speculator" nil (s p eh1 k y ax l ey1 t er0))
("speculators" nil (s p eh1 k y ax l ey1 t er0 z))
("sped" nil (s p eh1 d))
("speece" nil (s p iy1 s))
("speech" nil (s p iy1 ch))
("speeches" nil (s p iy1 ch ax z))
("speechless" nil (s p iy1 ch l ax s))
("speechwriter" nil (s p iy1 ch r ay1 t er0))
("speechwriters" nil (s p iy1 ch r ay1 t er0 z))
("speed" nil (s p iy1 d))
("speedboat" nil (s p iy1 d b ow1 t))
("speedboats" nil (s p iy1 d b ow1 t s))
("speeded" nil (s p iy1 d ih0 d))
("speedier" nil (s p iy1 d iy0 er0))
("speedily" nil (s p iy1 d ax l iy0))
("speeding" nil (s p iy1 d ih0 ng))
("speedometer" nil (s p iy0 d aa1 m ax t er0))
("speedring" nil (s p iy1 d r ih1 ng))
("speeds" nil (s p iy1 d z))
("speedskating" nil (s p iy1 d s k ey1 t ih0 ng))
("speedup" nil (s p iy1 d ah1 p))
("speedway" nil (s p iy1 d w ey1))
("speedy" nil (s p iy1 d iy0))
("speegle" nil (s p iy1 g ax l))
("speelman" nil (s p iy1 l m ax n))
("speer" nil (s p ih1 r))
("spees" nil (s p iy1 z))
("spegal" nil (s p iy1 g ax l))
("spehar" nil (s p eh1 hh er0))
("speich" nil (s p ay1 k))
("speicher" nil (s p ay1 k er0))
("speidel" nil (s p ay1 d ax l))
("speier" nil (s p ay1 er0))
("speight" nil (s p ey1 t))
("speights" nil (s p ey1 t s))
("speigner" nil (s p ay1 g n er0))
("speir" nil (s p ih1 r))
("speirs" nil (s p ih1 r z))
("speiser" nil (s p ay1 z er0))
("spell" nil (s p eh1 l))
("spellacy" nil (s p eh1 l ax s iy0))
("spellbinding" nil (s p eh1 l b ay1 n d ih0 ng))
("spellbound" nil (s p eh1 l b aw1 n d))
("spelled" nil (s p eh1 l d))
("speller" nil (s p eh1 l er0))
("spelling" nil (s p eh1 l ih0 ng))
("spellings" nil (s p eh1 l ih0 ng z))
("spellman" nil (s p eh1 l m ax n))
("spellmeyer" nil (s p eh1 l m ay0 er0))
("spells" nil (s p eh1 l z))
("spelman" nil (s p eh1 l m ax n))
("speltz" nil (s p eh1 l t s))
("spence" nil (s p eh1 n s))
("spencer" nil (s p eh1 n s er0))
("spend" nil (s p eh1 n d))
("spendable" nil (s p eh1 n d ax b ax l))
("spender" nil (s p eh1 n d er0))
("spenders" nil (s p eh1 n d er0 z))
("spending" nil (s p eh1 n d ih0 ng))
("spendley" nil (s p eh1 n d l iy0))
("spendlove" nil (s p eh1 n d l ah1 v))
("spends" nil (s p eh1 n d z))
("spendthrift" nil (s p eh1 n d th r ih1 f t))
("spengler" nil (s p ih1 ng ax l er0))
("spenner" nil (s p eh1 n er0))
("speno" nil (s p eh1 n ow0))
("spens" nil (s p eh1 n s))
("spenser" nil (s p eh1 n s er0))
("spensers" nil (s p eh1 n s er0 z))
("spent" nil (s p eh1 n t))
("spera" nil (s p eh1 r ax))
("speranza" nil (s p er0 aa1 n z ax))
("sperbeck" nil (s p er1 b eh0 k))
("sperber" nil (s p er1 b er0))
("sperduto" nil (s p er0 d uw1 t ow0))
("sperl" nil (s p er1 l))
("sperle" nil (s p ao1 r ax l))
("sperlich" nil (s p er1 l ih0 k))
("sperling" nil (s p er1 l ih0 ng))
("sperm" nil (s p er1 m))
("sperms" nil (s p er1 m z))
("spero" nil (s p eh1 r ow0))
("speros" nil (s p eh1 r ow0 z))
("sperrazza" nil (s p er0 aa1 t s ax))
("sperrfrist" nil (s p eh1 r f r ih0 s t))
("sperry" nil (s p eh1 r iy0))
("spessard" nil (s p eh1 s er0 d))
("speth" nil (s p eh1 th))
("spethmann" nil (s p eh1 th m ax n))
("spetsnaz" nil (s p eh1 t s n ae0 z))
("spevak" nil (s p eh1 v ax k))
("spew" nil (s p y uw1))
("spewed" nil (s p y uw1 d))
("spewing" nil (s p y uw1 ih0 ng))
("spews" nil (s p y uw1 z))
("spey" nil (s p ey1))
("speyer" nil (s p ey1 er0))
("speziale" nil (s p eh0 z iy0 aa1 l iy0))
("spezzano" nil (s p eh0 t s aa1 n ow0))
("sphar" nil (s f aa1 r))
("sphere" nil (s f ih1 r))
("spheres" nil (s f ih1 r z))
("spherical" nil (s f eh1 r ih0 k ax l))
("spheroid" nil (s f ih1 r oy1 d))
("sphinx" nil (s f ih1 ng k s))
("spic" nil (s p ih1 k))
("spice" nil (s p ay1 s))
("spiced" nil (s p ay1 s t))
("spicer" nil (s p ay1 s er0))
("spices" nil (s p ay1 s ax z))
("spicey" nil (s p ay1 s iy0))
("spicher" nil (s p ih1 ch er0))
("spicing" nil (s p ay1 s ih0 ng))
("spickard" nil (s p ih1 k er0 d))
("spickler" nil (s p ih1 k l er0))
("spicuzza" nil (s p iy0 k uw1 t s ax))
("spicy" nil (s p ay1 s iy0))
("spidel" nil (s p ih1 d ax l))
("spidell" nil (s p ih1 d ax l))
("spider" nil (s p ay1 d er0))
("spiders" nil (s p ay1 d er0 z))
("spidle" nil (s p ay1 d ax l))
("spie" nil (s p iy1))
("spied" nil (s p ay1 d))
("spiegel" nil (s p iy1 g ax l))
("spiegelberg" nil (s p iy1 g ax l b er0 g))
("spiegelman" nil (s p iy1 g ax l m ax n))
("spiegler" nil (s p iy1 g l er0))
("spieker" nil (s p iy1 k er0))
("spiel" nil (s p iy1 l))
("spielberg" nil (s p iy1 l b er0 g))
("spielberger" nil (s p iy1 l b er0 g er0))
("spieler" nil (s p iy1 l er0))
("spielman" nil (s p iy1 l m ax n))
("spielmann" nil (s p iy1 l m ax n))
("spielvogel" nil (s p iy1 l v ow1 g ax l))
("spier" nil (s p ay1 er0))
("spiering" nil (s p ay1 er0 ih0 ng))
("spiers" nil (s p ay1 er0 z))
("spies" nil (s p ay1 z))
("spiess" nil (s p iy1 z))
("spieth" nil (s p ay1 ax th))
("spiewak" nil (s p iy1 w ae1 k))
("spiffing" nil (s p ih1 f ih0 ng))
("spiffy" nil (s p ih1 f iy0))
("spight" nil (s p ay1 t))
("spigner" nil (s p ay1 g n er0))
("spigot" nil (s p ih1 g ax t))
("spigots" nil (s p ih1 g ax t s))
("spike" nil (s p ay1 k))
("spiked" nil (s p ay1 k t))
("spiker" nil (s p ay1 k er0))
("spikes" nil (s p ay1 k s))
("spiking" nil (s p ay1 k ih0 ng))
("spiky" nil (s p ay1 k iy0))
("spilde" nil (s p ih1 l d))
("spilker" nil (s p ih1 l k er0))
("spill" nil (s p ih1 l))
("spillane" nil (s p ih1 l ax n))
("spilled" nil (s p ih1 l d))
("spiller" nil (s p ih1 l er0))
("spillers" nil (s p ih1 l er0 z))
("spilling" nil (s p ih1 l ih0 ng))
("spillman" nil (s p ih1 l m ax n))
("spillover" nil (s p ih1 l ow1 v er0))
("spills" nil (s p ih1 l z))
("spilman" nil (s p ih1 l m ax n))
("spilotro" nil (s p ih0 l aa1 t r ow0))
("spilt" nil (s p ih1 l t))
("spin" nil (s p ih1 n))
("spina" nil (s p iy1 n ax))
("spinach" nil (s p ih1 n ax ch))
("spinal" nil (s p ay1 n ax l))
("spinale" nil (s p iy0 n aa1 l iy0))
("spindel" nil (s p ih1 n d ax l))
("spindle" nil (s p ih1 n d ax l))
("spindler" nil (s p ih1 n d ax l er0))
("spine" nil (s p ay1 n))
("spineless" nil (s p ay1 n l ax s))
("spinella" nil (s p ih0 n eh1 l ax))
("spinelli" nil (s p ih0 n eh1 l iy0))
("spinello" nil (s p ih0 n eh1 l ow0))
("spines" nil (s p ay1 n z))
("spinfizz" nil (s p ih1 n f ih0 z))
("spink" nil (s p ih1 ng k))
("spinks" nil (s p ih1 ng k s))
("spinner" nil (s p ih1 n er0))
("spinners" nil (s p ih1 n er0 z))
("spinney" nil (s p ih1 n iy0))
("spinning" nil (s p ih1 n ih0 ng))
("spino" nil (s p iy1 n ow0))
("spinoff" nil (s p ih1 n ao1 f))
("spinoffs" nil (s p ih1 n ao1 f s))
("spinola" nil (s p iy0 n ow1 l ax))
("spinosa" nil (s p iy0 n ow1 s ax))
("spinoza" nil (s p ih0 n ow1 z ax))
("spins" nil (s p ih1 n z))
("spinster" nil (s p ih1 n s t er0))
("spiny" nil (s p ay1 n iy0))
("spira" nil (s p ih1 r ax))
("spiral" nil (s p ay1 r ax l))
("spiraled" nil (s p ay1 r ax l d))
("spiraling" nil (s p ay1 r ax l ih0 ng))
("spiralling" nil (s p ay1 r ax l ih0 ng))
("spirals" nil (s p ay1 r ax l z))
("spire" nil (s p ay1 r))
("spires" nil (s p ay1 r z))
("spirit" nil (s p ih1 r ax t))
("spirited" nil (s p ih1 r ax t ax d))
("spiritedness" nil (s p ih1 r ih0 t ih0 d n ax s))
("spirito" nil (s p ih0 r iy1 t ow0))
("spirits" nil (s p ih1 r ax t s))
("spiritual" nil (s p ih1 r ih0 ch ax w ax l))
("spiritualism" nil (s p ih1 r ih0 ch ax w ax l ih0 z m))
("spirituality" nil (s p ih1 r ih0 ch ax w ae1 l ax t iy0))
("spiritually" nil (s p ih1 r ih0 ch ax w ax l iy0))
("spiro" nil (s p ih1 r ow0))
("spisak" nil (s p ih1 s ax k))
("spit" nil (s p ih1 t))
("spitale" nil (s p iy0 t aa1 l iy0))
("spitball" nil (s p ih1 t b ao1 l))
("spite" nil (s p ay1 t))
("spiteful" nil (s p ay1 t f ax l))
("spiteri" nil (s p iy0 t eh1 r iy0))
("spitler" nil (s p ih1 t l er0))
("spits" nil (s p ih1 t s))
("spitting" nil (s p ih1 t ih0 ng))
("spittle" nil (s p ih1 t ax l))
("spittler" nil (s p ih1 t l er0))
("spitz" nil (s p ih1 t s))
("spitzer" nil (s p ih1 t z er0))
("spitzley" nil (s p ih1 t s l iy0))
("spitznagel" nil (s p ih1 t s n ey1 g ax l))
("spiva" nil (s p iy1 v ax))
("spivack" nil (s p ih1 v ax k))
("spivak" nil (s p ih1 v ax k))
("spivey" nil (s p ih1 v iy0))
("spiwak" nil (s p ih1 v ax k))
("spizzirri" nil (s p iy0 t s ih1 r iy0))
("splain" nil (s p l ey1 n))
("splaine" nil (s p l ey1 n))
("splash" nil (s p l ae1 sh))
("splashed" nil (s p l ae1 sh t))
("splashes" nil (s p l ae1 sh ax z))
("splashing" nil (s p l ae1 sh ih0 ng))
("splashy" nil (s p l ae1 sh iy0))
("splat" nil (s p l ae1 t))
("splattered" nil (s p l ae1 t er0 d))
("splawn" nil (s p l ao1 n))
("spleen" nil (s p l iy1 n))
("splendid" nil (s p l eh1 n d ax d))
("splendidly" nil (s p l eh1 n d ax d l iy0))
("splendor" nil (s p l eh1 n d er0))
("splenic" nil (s p l eh1 n ih0 k))
("splice" nil (s p l ay1 s))
("spliced" nil (s p l ay1 s t))
("splices" nil (s p l ay1 s ih0 z))
("splichal" nil (s p l ih1 ch ax l))
("splicing" nil (s p l ay1 s ih0 ng))
("splint" nil (s p l ih1 n t))
("splinter" nil (s p l ih1 n t er0))
("splintered" nil (s p l ih1 n t er0 d))
("splintering" nil (s p l ih1 n t er0 ih0 ng))
("splintery" nil (s p l ih1 n t er0 iy0))
("splints" nil (s p l ih1 n t s))
("split" nil (s p l ih1 t))
("splits" nil (s p l ih1 t s))
("splitt" nil (s p l ih1 t))
("splitting" nil (s p l ih1 t ih0 ng))
("splurge" nil (s p l er1 jh))
("splurged" nil (s p l er1 jh d))
("spock" nil (s p aa1 k))
("spoden" nil (s p ow1 d ax n))
("spoelstra" nil (s p ow1 l s t r ax))
("spoerl" nil (s p ao1 r l))
("spofford" nil (s p aa1 f er0 d))
("spogli" nil (s p aa1 g l iy0))
("spohn" nil (s p aa1 n))
("spohr" nil (s p aa1 r))
("spoil" nil (s p oy1 l))
("spoilage" nil (s p oy1 l ax jh))
("spoiled" nil (s p oy1 l d))
("spoiler" nil (s p oy1 l er0))
("spoiling" nil (s p oy1 l ih0 ng))
("spoils" nil (s p oy1 l z))
("spokane" nil (s p ow0 k ae1 n))
("spoke" nil (s p ow1 k))
("spokeman" nil (s p ow1 k m ax n))
("spoken" nil (s p ow1 k ax n))
("spokes" nil (s p ow1 k s))
("spokesman" nil (s p ow1 k s m ax n))
("spokesmen" nil (s p ow1 k s m ih0 n))
("spokespeople" nil (s p ow1 k s p iy1 p ax l))
("spokesperson" nil (s p ow1 k s p er0 s ax n))
("spokespersons" nil (s p ow1 k ax s p er0 s ax n z))
("spokeswoman" nil (s p ow1 k s w uh1 m ax n))
("spokeswomen" nil (s p ow1 k ax s w ow1 m ax n))
("spokewoman" nil (s p ow1 k w uh1 m ax n))
("spoleto" nil (s p ow0 l ey1 t ow0))
("spomer" nil (s p ow1 m er0))
("sponaugle" nil (s p aa1 n ao0 g ax l))
("spong" nil (s p ao1 ng))
("sponge" nil (s p ah1 n jh))
("sponged" nil (s p ah1 n jh d))
("sponges" nil (s p ah1 n jh ax z))
("spongy" nil (s p ah1 n jh iy0))
("sponsel" nil (s p aa1 n s ax l))
("sponseller" nil (s p aa1 n s ax l er0))
("sponsler" nil (s p aa1 n s ax l er0))
("sponsor" nil (s p aa1 n s er0))
("sponsored" nil (s p aa1 n s er0 d))
("sponsoring" nil (s p aa1 n s er0 ih0 ng))
("sponsors" nil (s p aa1 n s er0 z))
("sponsorship" nil (s p aa1 n s er0 sh ih1 p))
("sponsorships" nil (s p aa1 n s er0 sh ih1 p s))
("spontaneity" nil (s p aa1 n t ax n iy1 ax t iy0))
("spontaneous" nil (s p aa0 n t ey1 n iy0 ax s))
("spontaneously" nil (s p aa0 n t ey1 n iy0 ax s l iy0))
("spoof" nil (s p uw1 f))
("spoofing" nil (s p uw1 f ih0 ng))
("spoofs" nil (s p uw1 f s))
("spook" nil (s p uw1 k))
("spooked" nil (s p uw1 k t))
("spooks" nil (s p uw1 k s))
("spooky" nil (s p uw1 k iy0))
("spool" nil (s p uw1 l))
("spoon" nil (s p uw1 n))
("spoonemore" nil (s p uw1 n m ao0 r))
("spooner" nil (s p uw1 n er0))
("spoonful" nil (s p uw1 n f uh1 l))
("spoons" nil (s p uw1 n z))
("spoor" nil (s p uh1 r))
("sporadic" nil (s p er0 ae1 d ih0 k))
("sporadically" nil (s p er0 ae1 d ih0 k l iy0))
("sporck" nil (s p ao1 r k))
("spore" nil (s p ao1 r))
("sporer" nil (s p ao1 r er0))
("spores" nil (s p ao1 r z))
("sporkin" nil (s p ao1 r k ih0 n))
("sporleder" nil (s p ao1 r l ih0 d er0))
("sporn" nil (s p ao1 r n))
("sporophyte" nil (s p ao1 r f ay1 t))
("sporophytes" nil (s p ao1 r f ay1 t s))
("sporrer" nil (s p ao1 er0 r))
("sport" nil (s p ao1 r t))
("sported" nil (s p ao1 r t ih0 d))
("sportier" nil (s p ao1 r t iy0 er0))
("sporting" nil (s p ao1 r t ih0 ng))
("sportingly" nil (s p ao1 r t ih0 ng l iy0))
("sports" nil (s p ao1 r t s))
("sportsbar" nil (s p ao1 r t s b aa1 r))
("sportscaster" nil (s p ao1 r t s k ae1 s t er0))
("sportschannel" nil (s p ao1 r t s ch ae1 n ax l))
("sportsclub" nil (s p ao1 r t s k l ah1 b))
("sportsman" nil (s p ao1 r t s m ax n))
("sportsmanship" nil (s p ao1 r t s m ax n sh ih1 p))
("sportsmen" nil (s p ao1 r t s m ih0 n))
("sportswear" nil (s p ao1 r t s w eh1 r))
("sportswriter" nil (s p ao1 r t s r ay1 t er0))
("sportswriters" nil (s p ao1 r t s r ay1 t er0 z))
("sporty" nil (s p ao1 r t iy0))
("sposato" nil (s p ow0 s aa1 t ow0))
("sposito" nil (s p ow0 s iy1 t ow0))
("spot" nil (s p aa1 t))
("spotless" nil (s p aa1 t l ax s))
("spotlight" nil (s p aa1 t l ay1 t))
("spotlighted" nil (s p aa1 t l ay1 t ih0 d))
("spotlighting" nil (s p aa1 t l ay1 t ih0 ng))
("spotlights" nil (s p aa1 t l ay1 t s))
("spoto" nil (s p ow1 t ow0))
("spots" nil (s p aa1 t s))
("spotted" nil (s p aa1 t ax d))
("spotter" nil (s p aa1 t er0))
("spotters" nil (s p aa1 t er0 z))
("spotting" nil (s p aa1 t ih0 ng))
("spotts" nil (s p aa1 t s))
("spotty" nil (s p aa1 t iy0))
("spousal" nil (s p aw1 z ax l))
("spouse" nil (s p aw1 s))
("spouses" nil (s p aw1 s ax z))
("spout" nil (s p aw1 t))
("spouted" nil (s p aw1 t ax d))
("spouting" nil (s p aw1 t ih0 ng))
("spracklen" nil (s p r ae1 k ax l ax n))
("spradley" nil (s p r ae1 d l iy0))
("spradlin" nil (s p r ae1 d l ih0 n))
("spradling" nil (s p r ae1 d l ih0 ng))
("spragg" nil (s p r ae1 g))
("spraggins" nil (s p r ae1 g ih0 n z))
("sprague" nil (s p r ey1 g))
("sprained" nil (s p r ey1 n d))
("sprains" nil (s p r ey1 n z))
("spraker" nil (s p r ey1 k er0))
("sprang" nil (s p r ae1 ng))
("spranger" nil (s p r ae1 ng er0))
("sprankle" nil (s p r ae1 ng k ax l))
("spratley" nil (s p r ae1 t l iy0))
("spratlin" nil (s p r ae1 t l ih0 n))
("spratling" nil (s p r ae1 t l ih0 ng))
("spratt" nil (s p r ae1 t))
("sprawl" nil (s p r ao1 l))
("sprawled" nil (s p r ao1 l d))
("sprawling" nil (s p r ao1 l ih0 ng))
("sprawls" nil (s p r ao1 l z))
("spray" nil (s p r ey1))
("sprayberry" nil (s p r ey1 b eh1 r iy0))
("sprayed" nil (s p r ey1 d))
("sprayer" nil (s p r ey1 er0))
("sprayers" nil (s p r ey1 er0 z))
("spraying" nil (s p r ey1 ih0 ng))
("sprays" nil (s p r ey1 z))
("spread" nil (s p r eh1 d))
("spreader" nil (s p r eh1 d er0))
("spreading" nil (s p r eh1 d ih0 ng))
("spreads" nil (s p r eh1 d z))
("spreadsheet" nil (s p r eh1 d sh iy1 t))
("spreadsheets" nil (s p r eh1 d sh iy1 t s))
("sprecher" nil (s p r eh1 k er0))
("spreckels" nil (s p r eh1 k ax l z))
("spree" nil (s p r iy1))
("spreeman" nil (s p r iy1 m ax n))
("spreen" nil (s p r iy1 n))
("sprees" nil (s p r iy1 z))
("sprehe" nil (s p r iy1 hh))
("spreitzer" nil (s p r ay1 t z er0))
("spreng" nil (s p r eh1 ng))
("sprenger" nil (s p r eh1 n jh er0))
("sprenkle" nil (s p r eh1 ng k ax l))
("sprick" nil (s p r ih1 k))
("sprigg" nil (s p r ih1 g))
("sprigged" nil (s p r ih1 g d))
("spriggs" nil (s p r ih1 g z))
("sprightly" nil (s p r ay1 t l iy0))
("spring" nil (s p r ih1 ng))
("springboard" nil (s p r ih1 ng b ao1 r d))
("springborn" nil (s p r ih1 ng g b er0 n))
("springdale" nil (s p r ih1 ng d ey1 l))
("springer" nil (s p r ih1 ng er0))
("springerville" nil (s p r ih1 ng er0 v ih1 l))
("springfield" nil (s p r ih1 ng f iy1 l d))
("springing" nil (s p r ih1 ng ih0 ng))
("springman" nil (s p r ih1 ng m ax n))
("springs" nil (s p r ih1 ng z))
("springstead" nil (s p r ih1 ng s t eh1 d))
("springsteen" nil (s p r ih1 ng s t iy1 n))
("springston" nil (s p r ih1 ng s t ax n))
("springtime" nil (s p r ih1 ng t ay1 m))
("sprinkel" nil (s p r ih1 ng k ax l))
("sprinkle" nil (s p r ih1 ng k ax l))
("sprinkled" nil (s p r ih1 ng k ax l d))
("sprinkler" nil (s p r ih1 ng k l er0))
("sprinklers" nil (s p r ih1 ng k l er0 z))
("sprinkles" nil (s p r ih1 ng k ax l z))
("sprinkling" nil (s p r ih1 ng k l ih0 ng))
("sprint" nil (s p r ih1 n t))
("sprinted" nil (s p r ih1 n t ih0 d))
("sprinter" nil (s p r ih1 n t er0))
("sprinters" nil (s p r ih1 n t er0 z))
("sprinting" nil (s p r ih1 n t ih0 ng))
("sprints" nil (s p r ih1 n t s))
("sprite" nil (s p r ay1 t))
("sprizzo" nil (s p r ih1 z ow0))
("sproat" nil (s p r ow1 t))
("sprock" nil (s p r aa1 k))
("sproles" nil (s p r ow1 l z))
("sprong" nil (s p r ao1 ng))
("sprott" nil (s p r aa1 t))
("sproul" nil (s p r aw1 l))
("sproule" nil (s p r uw1 l))
("sproull" nil (s p r uw1 l))
("sprouls" nil (s p r aw1 l z))
("sprouse" nil (s p r aw1 s))
("sprout" nil (s p r aw1 t))
("sprouted" nil (s p r aw1 t ax d))
("sprouting" nil (s p r aw1 t ih0 ng))
("sprouts" nil (s p r aw1 t s))
("sprow" nil (s p r aw1))
("sprowl" nil (s p r aw1 l))
("sprowle" nil (s p r aw1 ax l))
("sprowls" nil (s p r aw1 l z))
("spruce" nil (s p r uw1 s))
("spruced" nil (s p r uw1 s t))
("sprucing" nil (s p r uw1 s ih0 ng))
("spruell" nil (s p r uw1 l))
("spruiell" nil (s p r uw1 l))
("spruill" nil (s p r uw1 l))
("sprung" nil (s p r ah1 ng))
("sprunger" nil (s p r ah1 ng er0))
("spry" nil (s p r ay1))
("spud" nil (s p ah1 d))
("spuds" nil (s p ah1 d z))
("spuhler" nil (s p uw1 l er0))
("spun" nil (s p ah1 n))
("spunky" nil (s p ah1 ng k iy0))
("spur" nil (s p er1))
("spurdle" nil (s p er1 d ax l))
("spurge" nil (s p er1 jh))
("spurgeon" nil (s p er1 jh ih0 n))
("spurgin" nil (s p er1 jh ih0 n))
("spurious" nil (s p y uh1 r iy0 ax s))
("spurlin" nil (s p er1 l ih0 n))
("spurling" nil (s p er1 l ih0 ng))
("spurlock" nil (s p er1 l aa1 k))
("spurn" nil (s p er1 n))
("spurned" nil (s p er1 n d))
("spurning" nil (s p er1 n ih0 ng))
("spurns" nil (s p er1 n z))
("spurr" nil (s p er1))
("spurred" nil (s p er1 d))
("spurrier" nil (s p er1 iy0 er0))
("spurring" nil (s p er1 ih0 ng))
("spurs" nil (s p er1 z))
("spurt" nil (s p er1 t))
("spurted" nil (s p er1 t ih0 d))
("spurting" nil (s p er1 t ih0 ng))
("spurts" nil (s p er1 t s))
("sputnik" nil (s p ah1 t n ih0 k))
("sputniks" nil (s p ah1 t n ih0 k s))
("sputter" nil (s p ah1 t er0))
("sputtered" nil (s p ah1 t er0 d))
("sputtering" nil (s p ah1 t er0 ih0 ng))
("sputters" nil (s p ah1 t er0 z))
("spy" nil (s p ay1))
("spycatcher" nil (s p ay1 k ae1 ch er0))
("spychalski" nil (s p ih0 hh aa1 l s k iy0))
("spyglass" nil (s p ay1 g l ae1 s))
("spying" nil (s p ay1 ih0 ng))
("spyker" nil (s p ay1 k er0))
("squabble" nil (s k w aa1 b ax l))
("squabbled" nil (s k w aa1 b ax l d))
("squabbles" nil (s k w aa1 b ax l z))
("squabbling" nil (s k w aa1 b ax l ih0 ng))
("squad" nil (s k w aa1 d))
("squadron" nil (s k w aa1 d r ax n))
("squadrons" nil (s k w aa1 d r ax n z))
("squads" nil (s k w aa1 d z))
("squalid" nil (s k w aa1 l ax d))
("squall" nil (s k w ao1 l))
("squalls" nil (s k w ao1 l z))
("squalor" nil (s k w aa1 l er0))
("squander" nil (s k w aa1 n d er0))
("squandered" nil (s k w aa1 n d er0 d))
("squandering" nil (s k w aa1 n d er0 ih0 ng))
("square" nil (s k w eh1 r))
("squared" nil (s k w eh1 r d))
("squarely" nil (s k w eh1 r l iy0))
("squares" nil (s k w eh1 r z))
("squaring" nil (s k w eh1 r ih0 ng))
("squash" nil (s k w aa1 sh))
("squashed" nil (s k w aa1 sh t))
("squashing" nil (s k w aa1 sh ih0 ng))
("squashy" nil (s k w aa1 sh iy0))
("squat" nil (s k w aa1 t))
("squats" nil (s k w aa1 t s))
("squatter" nil (s k w aa1 t er0))
("squatters" nil (s k w aa1 t er0 z))
("squatting" nil (s k w aa1 t ih0 ng))
("squawk" nil (s k w ao1 k))
("squawking" nil (s k w ao1 k ih0 ng))
("squawks" nil (s k w ao1 k s))
("squeak" nil (s k w iy1 k))
("squeaked" nil (s k w iy1 k t))
("squeaker" nil (s k w iy1 k er0))
("squeakers" nil (s k w iy1 k er0 z))
("squeaking" nil (s k w iy1 k ih0 ng))
("squeaks" nil (s k w iy1 k s))
("squeaky" nil (s k w iy1 k iy0))
("squeal" nil (s k w iy1 l))
("squealing" nil (s k w iy1 l ih0 ng))
("squeals" nil (s k w iy1 l z))
("squeamish" nil (s k w iy1 m ih0 sh))
("squeamishness" nil (s k w iy1 m ih0 sh n ax s))
("squeegee" nil (s k w iy1 jh iy0))
("squeeze" nil (s k w iy1 z))
("squeezed" nil (s k w iy1 z d))
("squeezes" nil (s k w iy1 z ih0 z))
("squeezing" nil (s k w iy1 z ih0 ng))
("squelch" nil (s k w eh1 l ch))
("squelched" nil (s k w eh1 l ch t))
("squelching" nil (s k w eh1 l ch ih0 ng))
("squibb" nil (s k w ih1 b))
("squid" nil (s k w ih1 d))
("squier" nil (s k w ay1 er0))
("squiers" nil (s k w y er1 z))
("squillace" nil (s k w ih1 l ax s))
("squillante" nil (s k w iy0 l aa1 n t ey0))
("squint" nil (s k w ih1 n t))
("squinting" nil (s k w ih1 n t ih0 ng))
("squire" nil (s k w ay1 r))
("squires" nil (s k w ay1 er0 z))
("squirm" nil (s k w er1 m))
("squirming" nil (s k w er1 m ih0 ng))
("squirms" nil (s k w er1 m z))
("squirrel" nil (s k w er1 ax l))
("squirreled" nil (s k w er1 ax l d))
("squirrels" nil (s k w er1 ax l z))
("squirt" nil (s k w er1 t))
("squirted" nil (s k w er1 t ax d))
("squirts" nil (s k w er1 t s))
("squishy" nil (s k w ih1 sh iy0))
("squitieri" nil (s k w iy0 t ih1 r iy0))
("squyres" nil (s k w ay1 r z))
("sr" nil (s iy1 n y er0))
("srader" nil (sh r ey1 d er0))
("sram" nil (sh r ae1 m))
("sramek" nil (sh r ae1 m ih0 k))
("srebrenica" nil (s r ey1 b r eh0 n iy1 t s ax))
("sri" nil (sh r iy1))
("srinagar" nil (s r iy1 n ax g aa1 r))
("srinivas" nil (s r iy1 n ih0 v aa1 s))
("srinivasan" nil (s r iy0 n iy0 v aa0 s aa1 n))
("srivastava" nil (s r iy0 v aa0 s t aa1 v ax))
("srock" nil (sh r aa1 k))
("sroge" nil (sh r ow1 g))
("sroka" nil (sh r ow1 k ax))
("sroufe" nil (sh r ow1 f))
("ssangyong" nil (s ae1 ng y ao1 ng))
("st" nil (s t r iy1 t))
("staab" nil (s t aa1 b))
("staack" nil (s t aa1 k))
("staal" nil (s t aa1 l))
("staar" nil (s t aa1 r))
("staas" nil (s t aa1 z))
("staat" nil (s t aa1 t))
("staats" nil (s t aa1 t s))
("stab" nil (s t ae1 b))
("stabbed" nil (s t ae1 b d))
("stabbing" nil (s t ae1 b ih0 ng))
("stabbings" nil (s t ae1 b ih0 ng z))
("stabenow" nil (s t ae1 b ih0 n ow0))
("stabile" nil (s t ey1 b ih0 l))
("stability" nil (s t ax b ih1 l ih0 t iy0))
("stabilization" nil (s t ey1 b ax l ax z ey1 sh ax n))
("stabilize" nil (s t ey1 b ax l ay1 z))
("stabilized" nil (s t ey1 b ax l ay1 z d))
("stabilizer" nil (s t ey1 b ax l ay1 z er0))
("stabilizers" nil (s t ey1 b ax l ay1 z er0 z))
("stabilizes" nil (s t ey1 b ax l ay1 z ax z))
("stabilizing" nil (s t ey1 b ax l ay1 z ih0 ng))
("stable" nil (s t ey1 b ax l))
("stabler" nil (s t ey1 b ax l er0))
("stables" nil (s t ey1 b ax l z))
("stablest" nil (s t ey1 b ax l ax s t))
("stabley" nil (s t ae1 b l iy0))
("stabs" nil (s t ae1 b z))
("stac" nil (s t ae1 k))
("staccato" nil (s t ax k aa1 t ow1))
("stacey" nil (s t ey1 s iy0))
("stach" nil (s t ae1 ch))
("stachnik" nil (s t ae1 k n ih0 k))
("stachowiak" nil (s t ax hh aw1 iy0 ae0 k))
("stachowicz" nil (s t aa1 hh ax v ih0 ch))
("stachowski" nil (s t ax hh ao1 f s k iy0))
("stachura" nil (s t aa0 k uh1 r ax))
("stachurski" nil (s t ax hh er1 s k iy0))
("stacia" nil (s t aa1 ch ax))
("stacie" nil (s t ae1 k iy0))
("stack" nil (s t ae1 k))
("stacked" nil (s t ae1 k t))
("stacker" nil (s t ae1 k er0))
("stackhouse" nil (s t ae1 k hh aw1 s))
("stacking" nil (s t ae1 k ih0 ng))
("stackpole" nil (s t ae1 k p ow1 l))
("stacks" nil (s t ae1 k s))
("stacy" nil (s t ey1 s iy0))
("stade" nil (s t ey1 d))
("stadel" nil (s t ae1 d ax l))
("stadelman" nil (s t ae1 d ax l m ax n))
("stader" nil (s t ey1 d er0))
("stadia" nil (s t ey1 d iy0 ax))
("stadium" nil (s t ey1 d iy0 ax m))
("stadiums" nil (s t ey1 d iy0 ax m z))
("stadler" nil (s t ae1 d l er0))
("stadnik" nil (s t ae1 d n ih0 k))
("stadt" nil (s t ae1 d t))
("stadtlander" nil (sh t ae1 t l ae1 n d er0))
("stadtmiller" nil (sh t ae1 t m ih1 l er0))
("staebell" nil (s t eh1 b eh0 l))
("staebler" nil (s t ey1 l er0))
("staehle" nil (s t ey1 l))
("staff" nil (s t ae1 f))
("staffa" nil (s t aa1 f ax))
("staffed" nil (s t ae1 f t))
("staffer" nil (s t ae1 f er0))
("staffers" nil (s t ae1 f er0 z))
("staffieri" nil (s t aa0 f ih1 r iy0))
("staffing" nil (s t ae1 f ih0 ng))
("stafford" nil (s t ae1 f er0 d))
("staffordshire" nil (s t ae1 f er0 d sh er0))
("staffs" nil (s t ae1 f s))
("stag" nil (s t ae1 g))
("stage" nil (s t ey1 jh))
("stagecoach" nil (s t ey1 jh k ow1 ch))
("stagecraft" nil (s t ey1 jh k r ae1 f t))
("staged" nil (s t ey1 jh d))
("stagehands" nil (s t ey1 jh hh ae1 n d z))
("stager" nil (s t ey1 jh er0))
("stages" nil (s t ey1 jh ax z))
("stagflation" nil (s t ae0 g f l ey1 sh ax n))
("stagg" nil (s t ae1 g))
("stagger" nil (s t ae1 g er0))
("staggered" nil (s t ae1 g er0 d))
("staggering" nil (s t ae1 g er0 ih0 ng))
("staggers" nil (s t ae1 g er0 z))
("staggs" nil (s t ae1 g z))
("staging" nil (s t ey1 jh ih0 ng))
("stagliano" nil (s t aa0 g l iy0 aa1 n ow0))
("stagnant" nil (s t ae1 g n ax n t))
("stagnaro" nil (s t aa0 g n aa1 r ow0))
("stagnate" nil (s t ae1 g n ey1 t))
("stagnated" nil (s t ae1 g n ey1 t ih0 d))
("stagnating" nil (s t ae1 g n ey1 t ih0 ng))
("stagnation" nil (s t ae0 g n ey1 sh ax n))
("stagner" nil (s t ae1 g n er0))
("stahl" nil (s t aa1 l))
("stahlberg" nil (s t aa1 l b er0 g))
("stahle" nil (s t ey1 hh ax l))
("stahlecker" nil (s t aa1 l ih0 k er0))
("stahler" nil (s t aa1 l er0))
("stahley" nil (s t ae1 l iy0))
("stahlhut" nil (s t aa1 l hh ax t))
("stahlman" nil (s t aa1 l m ax n))
("stahly" nil (s t ae1 l iy0))
("stahmer" nil (s t aa1 m er0))
("stahnke" nil (s t ae1 ng k))
("stahr" nil (s t ae1 r))
("stai" nil (s t aa1 iy0))
("staib" nil (s t ey1 b))
("staid" nil (s t ey1 d))
("staiger" nil (s t ay1 g er0))
("stailey" nil (s t ey1 l iy0))
("stain" nil (s t ey1 n))
("stainback" nil (s t ey1 n b ae1 k))
("stainbrook" nil (s t ey1 n b r uh1 k))
("stained" nil (s t ey1 n d))
("staines" nil (s t ey1 n z))
("staining" nil (s t ey1 n ih0 ng))
("stainless" nil (s t ey1 n l ax s))
("stainmaster" nil (s t ey1 n m ae1 s t er0))
("stains" nil (s t ey1 n z))
("stair" nil (s t eh1 r))
("staircase" nil (s t eh1 r k ey1 s))
("staircases" nil (s t eh1 r k ey1 s ih0 z))
("stairs" nil (s t eh1 r z))
("stairway" nil (s t eh1 r w ey1))
("stairways" nil (s t eh1 r w ey1 z))
("stairwell" nil (s t eh1 r w eh1 l))
("stake" nil (s t ey1 k))
("staked" nil (s t ey1 k t))
("stakeholder" nil (s t ey1 k hh ow1 l d er0))
("stakeholders" nil (s t ey1 k hh ow1 l d er0 z))
("stakeout" nil (s t ey1 k aw1 t))
("stakeouts" nil (s t ey1 k aw1 t s))
("staker" nil (s t ey1 k er0))
("stakes" nil (s t ey1 k s))
("staking" nil (s t ey1 k ih0 ng))
("stalcup" nil (s t ao1 l k ah1 p))
("stalder" nil (s t ao1 l d er0))
("stale" nil (s t ey1 l))
("stalemate" nil (s t ey1 l m ey1 t))
("stalemated" nil (s t ey1 l m ey1 t ih0 d))
("staley" nil (s t ey1 l iy0))
("stalin" nil (s t aa1 l ax n))
("stalingrad" nil (s t ae1 l ih0 n g r ae1 d))
("stalinism" nil (s t ae1 l ih0 n ih1 z ax m))
("stalinist" nil (s t aa1 l ih0 n ih0 s t))
("stalinists" nil (s t ae1 l ih0 n ih0 s t s))
("stalinization" nil (s t ae1 l ih0 n ih0 z ey1 sh ax n))
("stalk" nil (s t ao1 k))
("stalked" nil (s t ao1 k t))
("stalker" nil (s t ao1 k er0))
("stalking" nil (s t ao1 k ih0 ng))
("stalks" nil (s t ao1 k s))
("stalky" nil (s t ao1 k iy0))
("stall" nil (s t ao1 l))
("stallard" nil (s t ae1 l er0 d))
("stallcup" nil (s t ao1 l k ah1 p))
("stalled" nil (s t ao1 l d))
("staller" nil (s t ao1 l er0))
("stallibrass" nil (s t ae1 l ih0 b r ae0 s))
("stalling" nil (s t ao1 l ih0 ng))
("stallings" nil (s t ao1 l ih0 ng z))
("stallion" nil (s t ae1 l y ax n))
("stallions" nil (s t ae1 l y ax n z))
("stallkamp" nil (s t ao1 l k ae1 m p))
("stallman" nil (s t ao1 l m ax n))
("stallone" nil (s t ax l ow1 n))
("stalls" nil (s t ao1 l z))
("stallsmith" nil (s t ao1 l s m ih1 th))
("stallworth" nil (s t ao1 l w er1 th))
("stalnaker" nil (s t ae1 l n ax k er0))
("staloff" nil (s t ae1 l ao0 f))
("stalon" nil (s t ey1 l ax n))
("stalowa" nil (s t ax l ow1 ax))
("stalter" nil (s t ao1 l t er0))
("stalvey" nil (s t ae1 l v iy0))
("stalwart" nil (s t ao1 l w er0 t))
("stalwarts" nil (s t ao1 l w er0 t s))
("stalzer" nil (s t ey1 l z er0))
("stam" nil (s t ae1 m))
("stamand" nil (s t ae1 m ax n d))
("stamant" nil (s t ae1 m ax n t))
("stamas" nil (s t aa1 m ax z))
("stamaty" nil (s t ae1 m ax t iy0))
("stambaugh" nil (s t ae1 m b ao0))
("stamberg" nil (s t ae1 m b er0 g))
("stamenson" nil (s t ey1 m ax n s ax n))
("stamer" nil (s t ey1 m er0))
("stamey" nil (s t ey1 m iy0))
("stamford" nil (s t ae1 m f er0 d))
("stamina" nil (s t ae1 m ax n ax))
("stamler" nil (s t ae1 m l er0))
("stamm" nil (s t ae1 m))
("stammen" nil (s t ae1 m ax n))
("stammer" nil (s t ae1 m er0))
("stamos" nil (s t ey1 m ow0 z))
("stamour" nil (s t ax m uh1 r))
("stamp" nil (s t ae1 m p))
("stamped" nil (s t ae1 m p t))
("stampede" nil (s t ae0 m p iy1 d))
("stampeded" nil (s t ae0 m p iy1 d ax d))
("stampeding" nil (s t ae0 m p iy1 d ih0 ng))
("stamper" nil (s t ae1 m p er0))
("stampfli" nil (s t ae1 m p f l iy0))
("stamping" nil (s t ae1 m p ih0 ng))
("stampings" nil (s t ae1 m p ih0 ng z))
("stampley" nil (s t ae1 m p l iy0))
("stamps" nil (s t ae1 m p s))
("stan" nil (s t ae1 n))
("stanadyne" nil (s t ae1 n ax d ay1 n))
("stanaland" nil (s t ae1 n ax l ax n d))
("stanard" nil (s t ae1 n er0 d))
("stanaway" nil (s t ae1 n ax w ey0))
("stanback" nil (s t ae1 n b ae1 k))
("stanberry" nil (s t ae1 n b eh1 r iy0))
("stanbery" nil (s t ae1 n b er0 iy0))
("stanbic" nil (s t ae1 n b ih0 k))
("stanbrough" nil (s t ae1 n b r aw0))
("stanbury" nil (s t ae1 n b eh1 r iy0))
("stance" nil (s t ae1 n s))
("stances" nil (s t ae1 n s ih0 z))
("stanch" nil (s t ae1 n ch))
("stanched" nil (s t ae1 n ch t))
("stanchfield" nil (s t ae1 n ch f iy0 l d))
("stanching" nil (s t ae1 n ch ih0 ng))
("stancik" nil (s t ae1 n s ih0 k))
("stancil" nil (s t ae1 n s ih0 l))
("stancill" nil (s t ae1 n s ih0 l))
("stancliff" nil (s t ae1 n k l ih0 f))
("stanco" nil (s t ae1 ng k ow0))
("stanczak" nil (s t ae1 n ch ae0 k))
("stanczyk" nil (s t ae1 n ch ih0 k))
("stand" nil (s t ae1 n d))
("standa" nil (s t ae1 n d ax))
("standard" nil (s t ae1 n d er0 d))
("standardization" nil (s t ae0 n d er0 d ih0 z ey1 sh ax n))
("standardize" nil (s t ae1 n d er0 d ay1 z))
("standardized" nil (s t ae1 n d er0 d ay1 z d))
("standardizing" nil (s t ae1 n d er0 d ay1 z ih0 ng))
("standards" nil (s t ae1 n d er0 d z))
("standby" nil (s t ae1 n d b ay1))
("standbys" nil (s t ae1 n d b ay1 z))
("standefer" nil (s t ae1 n d ih0 f er0))
("standen" nil (s t ae1 n d ax n))
("stander" nil (s t ae1 n d er0))
("standerfer" nil (s t ae1 n d er0 f er0))
("standex" nil (s t ae1 n d ax k s))
("standfield" nil (s t ae1 n d f iy1 l d))
("standifer" nil (s t ae1 n d ih0 f er0))
("standiford" nil (s t ae1 n d ih0 f er0 d))
("standing" nil (s t ae1 n d ih0 ng))
("standings" nil (s t ae1 n d ih0 ng z))
("standish" nil (s t ae1 n d ih0 sh))
("standlee" nil (s t ae1 n d l iy1))
("standley" nil (s t ae1 n d l iy0))
("standoff" nil (s t ae1 n d ao1 f))
("standout" nil (s t ae1 n d aw1 t))
("standouts" nil (s t ae1 n d aw1 t s))
("standpoint" nil (s t ae1 n d p oy1 n t))
("standpoints" nil (s t ae1 n d p oy1 n t s))
("standre" nil (s t ae1 n d er0))
("standridge" nil (s t ae1 n d r ih0 jh))
("stands" nil (s t ae1 n d z))
("standstill" nil (s t ae1 n d s t ih1 l))
("standup" nil (s t ae1 n d ah1 p))
("stanek" nil (s t ae1 n ih0 k))
("stanfield" nil (s t ae1 n f iy1 l d))
("stanfill" nil (s t ae1 n f ax l))
("stanford" nil (s t ae1 n f er0 d))
("stanforth" nil (s t ae1 n f er0 th))
("stang" nil (s t ae1 ng))
("stanga" nil (s t aa1 ng g ax))
("stange" nil (s t ae1 n jh))
("stangel" nil (s t ey1 ng g ax l))
("stangeland" nil (s t ey1 ng g ih0 l ax n d))
("stangelo" nil (s t aa0 ng g eh1 l ow0))
("stanger" nil (s t ae1 ng er0))
("stangl" nil (s t ae1 ng g ax l))
("stangland" nil (s t ae1 ng g l ax n d))
("stangle" nil (s t ae1 ng g ax l))
("stangler" nil (s t ae1 ng g ax l er0))
("stango" nil (s t aa1 ng g ow0))
("stanhope" nil (s t ae1 n ax p))
("staniar" nil (s t ey1 n iy0 aa0 r))
("stanich" nil (s t ae1 n ih0 ch))
("stanifer" nil (s t ae1 n ih0 f er0))
("stanis" nil (s t ae1 n ih0 s))
("stanish" nil (s t ae1 n ih0 sh))
("stanislas" nil (s t ax n ih1 s l ax s))
("stanislav" nil (s t ae1 n ih0 s l aa0 v))
("stanislaw" nil (s t ae1 n ih0 s l ao1))
("stanislawski" nil (s t ax n ih0 s l aa1 f s k iy0))
("staniszewski" nil (s t ax n ih0 sh eh1 f s k iy0))
("stank" nil (s t ae1 ng k))
("stanke" nil (s t ae1 ng k))
("stankey" nil (s t ae1 ng k iy0))
("stankiewicz" nil (s t ae1 n k ax v ih0 ch))
("stanko" nil (s t ae1 ng k ow0))
("stankovich" nil (s t ae1 ng k ax v ih0 ch))
("stankowski" nil (s t ax ng k ao1 f s k iy0))
("stankus" nil (s t ae1 ng k ax s))
("stanleigh" nil (s t ae1 n l ax))
("stanley" nil (s t ae1 n l iy0))
("stanleytown" nil (s t ae1 n l iy0 t aw1 n))
("stanline" nil (s t ae1 n l ay1 n))
("stanly" nil (s t ae1 n l iy0))
("stanmore" nil (s t ae1 n m ao0 r))
("stann" nil (s t ae1 n))
("stannard" nil (s t ae1 n er0 d))
("stannie" nil (s t ae1 n iy0))
("stano" nil (s t aa1 n ow0))
("stanphill" nil (s t ae1 n p hh ih1 l))
("stansberry" nil (s t ae1 n s b eh1 r iy0))
("stansbury" nil (s t ae1 n s b eh0 r iy0))
("stansel" nil (s t ae1 n s ax l))
("stansell" nil (s t ae1 n s ax l))
("stansfield" nil (s t ae1 n s f iy0 l d))
("stant" nil (s t ae1 n t))
("stanton" nil (s t ae1 n t ax n))
("stanway" nil (s t ae1 n w ey1))
("stanwick" nil (s t ae1 n w ih0 k))
("stanwood" nil (s t ae1 n w uh1 d))
("stanza" nil (s t ae1 n z ax))
("stanzas" nil (s t ae1 n z ax z))
("stanzione" nil (s t aa0 n z iy0 ow1 n iy0))
("stapel" nil (s t ae1 p ax l))
("stapf" nil (s t ae1 p f))
("staple" nil (s t ey1 p ax l))
("stapled" nil (s t ey1 p ax l d))
("stapler" nil (s t ey1 p ax l er0))
("staplers" nil (s t ey1 p ax l er0 z))
("staples" nil (s t ey1 p ax l z))
("stapleton" nil (s t ey1 p ax l t ax n))
("stapley" nil (s t ae1 p l iy0))
("stapling" nil (s t ey1 p ax l ih0 ng))
("stapp" nil (s t ae1 p))
("star" nil (s t aa1 r))
("starace" nil (s t aa0 r aa1 ch iy0))
("starbird" nil (s t aa1 r b er1 d))
("starboard" nil (s t aa1 r b er0 d))
("starboards" nil (s t aa1 r b er0 r d z))
("starbuck" nil (s t aa1 r b ah1 k))
("starbucks" nil (s t aa1 r b ah1 k s))
("starcevich" nil (s t aa1 r s ih0 v ih0 ch))
("starch" nil (s t aa1 r ch))
("starched" nil (s t aa1 r ch t))
("starcher" nil (s t aa1 r ch er0))
("starches" nil (s t aa1 r ch ih0 z))
("starchlike" nil (s t aa1 r ch l ay1 k))
("starchy" nil (s t aa1 r ch iy0))
("starck" nil (s t aa1 r k))
("starckmann" nil (s t aa1 r k m ax n))
("starcraft" nil (s t aa1 r k r ae1 f t))
("starcross" nil (s t aa1 r k r ao1 s))
("stardel" nil (s t aa1 r d eh1 l))
("stardent" nil (s t aa1 r d eh1 n t))
("stardom" nil (s t aa1 r d ax m))
("stardust" nil (s t aa1 r d ah1 s t))
("stare" nil (s t eh1 r))
("stared" nil (s t eh1 r d))
("starer" nil (s t eh1 r er0))
("stares" nil (s t eh1 r z))
("starfish" nil (s t aa1 r f ih1 sh))
("stargate" nil (s t aa1 r g ey1 t))
("stargates" nil (s t aa1 r g ey1 t s))
("starin" nil (s t ae1 r ih0 n))
("staring" nil (s t eh1 r ih0 ng))
("stark" nil (s t aa1 r k))
("starke" nil (s t aa1 r k))
("starkel" nil (s t aa1 r k ax l))
("starker" nil (s t aa1 r k er0))
("starkes" nil (s t aa1 r k s))
("starkey" nil (s t aa1 r k iy1))
("starkist" nil (s t aa1 r k ih0 s t))
("starkly" nil (s t aa1 r k l iy0))
("starkman" nil (s t aa1 r k m ax n))
("starkovich" nil (s t aa1 r k ax v ih0 ch))
("starks" nil (s t aa1 r k s))
("starkweather" nil (s t aa1 r k w eh1 dh er0))
("starlet" nil (s t aa1 r l ax t))
("starlets" nil (s t aa1 r l ax t s))
("starlight" nil (s t aa1 r l ay1 t))
("starlike" nil (s t aa1 r l ay1 k))
("starlin" nil (s t aa1 r l ih0 n))
("starling" nil (s t aa1 r l ih0 ng))
("starliper" nil (s t aa1 r l ih0 p er0))
("starman" nil (s t aa1 r m ax n))
("starmer" nil (s t aa1 r m er0))
("starn" nil (s t aa1 r n))
("starner" nil (s t aa1 r n er0))
("starnes" nil (s t aa1 r n z))
("starns" nil (s t aa1 r n z))
("staron" nil (s t ae1 r ax n))
("starplex" nil (s t aa1 r p l eh1 k s))
("starpointe" nil (s t aa1 r p oy1 n t))
("starr" nil (s t aa1 r))
("starred" nil (s t aa1 r d))
("starrett" nil (s t ae1 r ih0 t))
("starring" nil (s t aa1 r ih0 ng))
("starrs" nil (s t aa1 r z))
("starry" nil (s t aa1 r iy0))
("stars" nil (s t aa1 r z))
("starship" nil (s t aa1 r sh ih1 p))
("starsight" nil (s t aa1 r s ay1 t))
("starstream" nil (s t aa1 r s t r iy1 m))
("start" nil (s t aa1 r t))
("started" nil (s t aa1 r t ax d))
("starter" nil (s t aa1 r t er0))
("starters" nil (s t aa1 r t er0 z))
("starting" nil (s t aa1 r t ih0 ng))
("startle" nil (s t aa1 r t ax l))
("startled" nil (s t aa1 r t ax l d))
("startles" nil (s t aa1 r t ax l z))
("startling" nil (s t aa1 r t l ih0 ng))
("startlingly" nil (s t aa1 r t l ih0 ng l iy0))
("starts" nil (s t aa1 r t s))
("startup" nil (s t aa1 r t ah1 p))
("startups" nil (s t aa1 r t ah1 p s))
("starvation" nil (s t aa0 r v ey1 sh ax n))
("starve" nil (s t aa1 r v))
("starved" nil (s t aa1 r v d))
("starving" nil (s t aa1 r v ih0 ng))
("starwalt" nil (s t aa1 r w ax l t))
("starwave" nil (s t aa1 r w ey1 v))
("stary" nil (s t eh1 r iy0))
("starzyk" nil (s t aa1 r z ih0 k))
("stash" nil (s t ae1 sh))
("stashed" nil (s t ae1 sh t))
("stashing" nil (s t ae1 sh ih0 ng))
("stasi" nil (s t aa1 s iy0))
("stasiak" nil (s t aa1 s iy0 ae0 k))
("stasik" nil (s t aa1 s ih0 k))
("stasio" nil (s t aa1 s iy0 ow0))
("stasko" nil (s t aa1 s k ow0))
("stasney" nil (s t ae1 s n iy0))
("stasny" nil (s t ae1 s n iy0))
("stassen" nil (s t ae1 s ax n))
("stassi" nil (s t ae1 s iy0))
("stastny" nil (s t ae1 s t n iy0))
("staszak" nil (s t aa1 sh ax k))
("staszewski" nil (s t ax sh eh1 f s k iy0))
("stat" nil (s t ae1 t))
("state" nil (s t ey1 t))
("statecraft" nil (s t ey1 t k r ae1 f t))
("stated" nil (s t ey1 t ax d))
("statehood" nil (s t ey1 t hh uh1 d))
("statehouse" nil (s t ey1 t hh aw1 s))
("statehouses" nil (s t ey1 t hh aw1 s ih0 z))
("stately" nil (s t ey1 t l iy0))
("statement" nil (s t ey1 t m ax n t))
("statements" nil (s t ey1 t m ax n t s))
("staten" nil (s t ae1 t ax n))
("stater" nil (s t ey1 t er0))
("staters" nil (s t ey1 t er0 z))
("states" nil (s t ey1 t s))
("stateside" nil (s t ey1 t s ay1 d))
("statesman" nil (s t ey1 t s m ax n))
("statesmanlike" nil (s t ax t eh1 s m ax n l ay1 k))
("statesmanship" nil (s t ey1 t s m ax n sh ih1 p))
("statesmen" nil (s t ey1 t s m ih0 n))
("stateswest" nil (s t ey1 t s w eh1 s t))
("statewide" nil (s t ey1 t w ay1 d))
("statham" nil (s t ae1 th ax m))
("stathis" nil (s t ae1 th ih0 s))
("stathopoulos" nil (s t ax th aa1 p ax l ih0 s))
("static" nil (s t ae1 t ih0 k))
("stating" nil (s t ey1 t ih0 ng))
("station" nil (s t ey1 sh ax n))
("stationary" nil (s t ey1 sh ax n eh1 r iy0))
("stationed" nil (s t ey1 sh ax n d))
("stationers" nil (s t ey1 sh ax n er0 z))
("stationery" nil (s t ey1 sh ax n eh1 r iy0))
("stationing" nil (s t ey1 sh ax n ih0 ng))
("stations" nil (s t ey1 sh ax n z))
("statism" nil (s t ey1 t ih1 z ax m))
("statist" nil (s t ey1 t ih0 s t))
("statistic" nil (s t ax t ih1 s t ih0 k))
("statistical" nil (s t ax t ih1 s t ih0 k ax l))
("statistically" nil (s t ax t ih1 s t ih0 k ax l iy0))
("statistician" nil (s t ae1 t ax s t ih1 sh ax n))
("statisticians" nil (s t ae1 t ih0 s t ih1 sh ax n z))
("statistics" nil (s t ax t ih1 s t ih0 k s))
("statists" nil (s t ey1 t ih0 s t s))
("statler" nil (s t ae1 t l er0))
("statoil" nil (s t ax t oy1 l))
("staton" nil (s t ae1 t ax n))
("stats" nil (s t ae1 t s))
("stattin" nil (s t ae1 t ih0 n))
("statton" nil (s t ae1 t ax n))
("statuary" nil (s t ae1 ch uw0 eh1 r iy0))
("statue" nil (s t ae1 ch uw1))
("statues" nil (s t ae1 ch uw1 z))
("statuesque" nil (s t ae1 ch uw0 eh1 s k))
("statuettes" nil (s t ae1 ch uw0 eh1 t s))
("statum" nil (s t ae1 t ax m))
("stature" nil (s t ae1 ch er0))
("status" nil (s t ae1 t ax s))
("statute" nil (s t ae1 ch uw0 t))
("statutes" nil (s t ae1 ch uw0 t s))
("statutory" nil (s t ae1 ch ax t ao1 r iy0))
("statz" nil (s t ae1 t s))
("statzer" nil (s t ae1 t z er0))
("staub" nil (s t ao1 b))
("stauber" nil (s t aw1 b er0))
("staubin" nil (s t aw1 b ih0 n))
("staubs" nil (s t ao1 b z))
("stauch" nil (s t ao1 ch))
("staudacher" nil (s t aw1 d ax k er0))
("staude" nil (s t ao1 d))
("stauder" nil (s t aw1 d er0))
("staudinger" nil (s t aw1 d ih0 ng er0))
("staudt" nil (s t ao1 d t))
("stauff" nil (s t ao1 f))
("stauffacher" nil (s t aw1 f ax k er0))
("stauffer" nil (s t ao1 f er0))
("staunch" nil (s t ao1 n ch))
("staunchest" nil (s t ao1 n ch ax s t))
("staunchly" nil (s t ao1 n ch l iy0))
("staunton" nil (s t ao1 n t ax n))
("staup" nil (s t ao1 p))
("stauss" nil (s t ao1 s))
("stauter" nil (s t aw1 t er0))
("stave" nil (s t ey1 v))
("staved" nil (s t ey1 v d))
("stavely" nil (s t ey1 v l iy0))
("staver" nil (s t ey1 v er0))
("staves" nil (s t ey1 v z))
("staving" nil (s t ey1 v ih0 ng))
("stavinoha" nil (s t ae0 v ih0 n ow1 hh ax))
("stavola" nil (s t aa0 v ow1 l ax))
("stavropoulos" nil (s t ax v r aa1 p ax l ih0 s))
("stavros" nil (s t ax v r ow1 z))
("stawicki" nil (s t aa0 v ih1 t s k iy0))
("stawski" nil (s t aa1 f s k iy0))
("stay" nil (s t ey1))
("stayed" nil (s t ey1 d))
("stayer" nil (s t ey1 er0))
("staying" nil (s t ey1 ih0 ng))
("stayner" nil (s t ey1 n er0))
("stayover" nil (s t ey1 ow1 v er0))
("stayovers" nil (s t ey1 ow1 v er0 z))
("stays" nil (s t ey1 z))
("stayton" nil (s t ey1 t ax n))
("stead" nil (s t eh1 d))
("steadfast" nil (s t eh1 d f ae1 s t))
("steadfastly" nil (s t eh1 d f ae1 s t l iy0))
("steadfastness" nil (s t eh1 d f ae1 s t n ax s))
("steadham" nil (s t eh1 d hh ax m))
("steadied" nil (s t eh1 d iy0 d))
("steadier" nil (s t eh1 d iy0 er0))
("steadily" nil (s t eh1 d ax l iy0))
("steadiness" nil (s t eh1 d iy0 n ih0 s))
("steadman" nil (s t eh1 d m ax n))
("steady" nil (s t eh1 d iy0))
("steagall" nil (s t iy1 g ax l))
("steak" nil (s t ey1 k))
("steakhouse" nil (s t ey1 k hh aw1 s))
("steakhouses" nil (s t ey1 k hh aw1 s ih0 z))
("steakley" nil (s t iy1 k l iy0))
("steaks" nil (s t ey1 k s))
("steal" nil (s t iy1 l))
("stealey" nil (s t iy1 l iy0))
("stealing" nil (s t iy1 l ih0 ng))
("steals" nil (s t iy1 l z))
("stealth" nil (s t eh1 l th))
("stealthies" nil (s t eh1 l th iy0 z))
("steam" nil (s t iy1 m))
("steamboat" nil (s t iy1 m b ow1 t))
("steamboats" nil (s t iy1 m b ow1 t s))
("steamed" nil (s t iy1 m d))
("steamer" nil (s t iy1 m er0))
("steamers" nil (s t iy1 m er0 z))
("steamily" nil (s t iy1 m ax l iy0))
("steaming" nil (s t iy1 m ih0 ng))
("steamroller" nil (s t iy1 m r ow1 l er0))
("steamrollered" nil (s t iy1 m r ow1 l er0 d))
("steams" nil (s t iy1 m z))
("steamship" nil (s t iy1 m sh ih1 p))
("steamy" nil (s t iy1 m iy0))
("stear" nil (s t ih1 r))
("stearic" nil (s t iy1 r ih0 k))
("stearman" nil (s t ih1 r m ax n))
("stearn" nil (s t er1 n))
("stearne" nil (s t er1 n))
("stearns" nil (s t er1 n z))
("stears" nil (s t iy1 r z))
("stebbins" nil (s t eh1 b ih0 n z))
("steber" nil (s t iy1 b er0))
("stebner" nil (s t eh1 b n er0))
("stec" nil (s t eh1 k))
("stech" nil (s t eh1 k))
("stecher" nil (s t eh1 k er0))
("stechler" nil (s t eh1 k l er0))
("stechschulte" nil (s t eh1 k sh ax l t))
("steck" nil (s t eh1 k))
("steckel" nil (s t eh1 k ax l))
("steckelberg" nil (s t eh1 k ax l b er0 g))
("stecker" nil (s t eh1 k er0))
("stecklein" nil (s t eh1 k l ay1 n))
("steckler" nil (s t eh1 k l er0))
("steckley" nil (s t eh1 k l iy0))
("steckman" nil (s t eh1 k m ax n))
("stedman" nil (s t eh1 d m ax n))
("steeb" nil (s t iy1 b))
("steeber" nil (s t iy1 b er0))
("steed" nil (s t iy1 d))
("steedley" nil (s t iy1 d l iy0))
("steedman" nil (s t iy1 d m ax n))
("steeg" nil (s t iy1 g))
("steege" nil (s t iy1 jh))
("steego" nil (s t iy1 g ow0))
("steel" nil (s t iy1 l))
("steelcase" nil (s t iy1 l k ey1 s))
("steele" nil (s t iy1 l))
("steeler" nil (s t iy1 l er0))
("steelers" nil (s t iy1 l er0 z))
("steeley" nil (s t iy1 l iy0))
("steelmaker" nil (s t iy1 l m ey1 k er0))
("steelmakers" nil (s t iy1 l m ey1 k er0 z))
("steelmaking" nil (s t iy1 l m ey1 k ih0 ng))
("steelman" nil (s t iy1 l m ax n))
("steels" nil (s t iy1 l z))
("steelworker" nil (s t iy1 l w er1 k er0))
("steelworkers" nil (s t iy1 l w er1 k er0 z))
("steelworks" nil (s t iy1 l w er1 k s))
("steely" nil (s t iy1 l iy0))
("steen" nil (s t iy1 n))
("steenbergen" nil (s t iy1 n b er0 g ax n))
("steenkamp" nil (s t iy1 n k ae1 m p))
("steenkiste" nil (s t iy1 n k ih1 s t))
("steenrod" nil (s t iy1 n r ax d))
("steensma" nil (s t iy1 n z m ax))
("steenson" nil (s t iy1 n s ax n))
("steep" nil (s t iy1 p))
("steeped" nil (s t iy1 p t))
("steepened" nil (s t iy1 p ax n d))
("steeper" nil (s t iy1 p er0))
("steepest" nil (s t iy1 p ax s t))
("steeplechase" nil (s t iy1 p ax l ch ey1 s))
("steeply" nil (s t iy1 p l iy0))
("steepness" nil (s t iy1 p n ax s))
("steer" nil (s t ih1 r))
("steere" nil (s t ih1 r))
("steered" nil (s t ih1 r d))
("steering" nil (s t ih1 r ih0 ng))
("steers" nil (s t ih1 r z))
("stees" nil (s t iy1 z))
("steever" nil (s t iy1 v er0))
("steeves" nil (s t iy1 v z))
("stefa" nil (s t eh1 f ax))
("stefan" nil (s t eh1 f aa0 n))
("stefanelli" nil (s t eh0 f aa0 n eh1 l iy0))
("stefani" nil (s t eh1 f ax n iy0))
("stefaniak" nil (s t ih0 f ae1 n iy0 ae0 k))
("stefanic" nil (s t ih0 f ae1 n ih0 k))
("stefanich" nil (s t eh1 f ax n ih0 ch))
("stefanick" nil (s t eh1 f ax n ih0 k))
("stefanik" nil (s t ih0 f ae1 n ih0 k))
("stefanko" nil (s t ih0 f ae1 ng k ow0))
("stefano" nil (s t eh1 f ax n ow0))
("stefanowicz" nil (s t ih0 f ae1 n ax v ih0 ch))
("stefanski" nil (s t ih0 f ae1 n s k iy0))
("steffan" nil (s t eh1 f ax n))
("steffanci" nil (s t eh1 f aa1 n s iy0))
("steffe" nil (s t eh1 f))
("steffek" nil (s t eh1 f ih0 k))
("steffel" nil (s t eh1 f ax l))
("steffen" nil (s t eh1 f ax n))
("steffenhagen" nil (s t eh1 f ih0 n hh ax g ax n))
("steffens" nil (s t eh1 f ax n z))
("steffensen" nil (s t eh1 f ih0 n s ax n))
("steffensmeier" nil (s t eh1 f ih0 n s m ay0 er0))
("steffenson" nil (s t eh1 f ih0 n s ax n))
("steffes" nil (s t eh1 f s))
("steffey" nil (s t eh1 f iy0))
("steffi" nil (s t eh1 f iy0))
("steffie" nil (s t eh1 f iy0))
("steffler" nil (s t eh1 f l er0))
("steffy" nil (s t eh1 f iy0))
("stefko" nil (s t eh1 f k ow0))
("stefl" nil (s t eh1 f ax l))
("stegall" nil (s t eh1 g ax l))
("stege" nil (s t iy1 jh))
("stegeman" nil (s t iy1 g m ax n))
("stegemann" nil (s t iy1 g m ax n))
("stegemeier" nil (s t eh1 g m ay1 r))
("stegenga" nil (s t eh0 jh eh1 ng g ax))
("steger" nil (s t iy1 g er0))
("stegmaier" nil (s t eh1 g m ay0 er0))
("stegman" nil (s t eh1 g m ax n))
("stegmann" nil (s t eh1 g m ax n))
("stegner" nil (s t eh1 g n er0))
("stehle" nil (s t eh1 hh ax l))
("stehlik" nil (s t eh1 l ih0 k))
("stehlin" nil (s t eh1 l ih0 n))
("stehling" nil (s t eh1 l ih0 ng))
("stehman" nil (s t eh1 m ax n))
("stehr" nil (s t eh1 r))
("steib" nil (s t iy1 b))
("steichen" nil (s t ay1 k ax n))
("steidel" nil (s t ay1 d ax l))
("steidinger" nil (s t ay1 d ih0 ng er0))
("steidl" nil (s t iy1 d ax l))
("steidle" nil (s t iy1 d ax l))
("steidtmann" nil (s t ay1 t m ax n))
("steier" nil (s t ay1 er0))
("steig" nil (s t iy1 g))
("steiger" nil (s t ay1 g er0))
("steigerwald" nil (s t ay1 g er0 w ao1 l d))
("steigerwalt" nil (s t ay1 g er0 w ax l t))
("steil" nil (s t iy1 l))
("steiman" nil (s t ay1 m ax n))
("steimel" nil (s t ay1 m ax l))
("steimer" nil (s t ay1 m er0))
("steimle" nil (s t iy1 m ax l))
("stein" nil (s t ay1 n))
("steinacker" nil (s t ay1 n ae0 k er0))
("steinbach" nil (s t ay1 n b aa1 k))
("steinbacher" nil (s t ay1 n b aa1 k er0))
("steinback" nil (s t ay1 n b ae1 k))
("steinbauer" nil (s t ay1 n b aw0 er0))
("steinbaugh" nil (s t ay1 n b aw0))
("steinbeck" nil (s t ay1 n b eh1 k))
("steinberg" nil (s t ay1 n b er0 g))
("steinberger" nil (s t ay1 n b er0 g er0))
("steinbock" nil (s t ay1 n b aa1 k))
("steinborn" nil (s t ay1 n b ao1 r n))
("steinbrecher" nil (s t ay1 n b r eh1 k er0))
("steinbrenner" nil (s t ay1 n b r eh1 n er0))
("steinbrink" nil (s t ay1 n b r ih1 ng k))
("steinem" nil (s t ay1 n ax m))
("steiner" nil (s t ay1 n er0))
("steinert" nil (s t ay1 n er0 t))
("steines" nil (s t ay1 n z))
("steinfeld" nil (s t ay1 n f eh1 l d))
("steinfeldt" nil (s t ay1 n f eh1 l t))
("steingut" nil (s t ay1 n g ah1 t))
("steinhagen" nil (s t ay1 n hh ae0 g ax n))
("steinhardt" nil (s t ay1 n hh aa1 r t))
("steinhart" nil (s t ay1 n hh aa1 r t))
("steinhauer" nil (s t ay1 n hh aw0 er0))
("steinhaus" nil (s t ay1 n hh aw1 s))
("steinhauser" nil (s t ay1 n hh aw1 z er0))
("steinhilber" nil (s t ay1 n hh ih1 l b er0))
("steinhoff" nil (s t ay1 n hh ao1 f))
("steinhorst" nil (s t ay1 n hh ao0 r s t))
("steininger" nil (s t ay1 n ih0 ng er0))
("steinkamp" nil (s t ay1 ng k ae0 m p))
("steinke" nil (s t ay1 ng k))
("steinkraus" nil (s t ay1 ng k r aw0 z))
("steinkrauss" nil (s t ay1 n k r aw1 s))
("steinkuehler" nil (s t ay1 n k y uw1 l er0))
("steinle" nil (s t ay1 n ax l))
("steinman" nil (s t ay1 n m ax n))
("steinmann" nil (s t ay1 n m ax n))
("steinmetz" nil (s t ay1 n m eh0 t s))
("steinmeyer" nil (s t ay1 n m ay0 er0))
("steinmiller" nil (s t ay1 n m ih1 l er0))
("steinroe" nil (s t ay1 n r ow1))
("steinway" nil (s t ay1 n w ey1))
("steitz" nil (s t iy1 t s))
("stejskal" nil (s t eh1 jh s k ax l))
("steketee" nil (s t eh1 k ih0 t iy0))
("stekly" nil (s t eh1 k l iy0))
("stelco" nil (s t eh1 l k ow0))
("stelian" nil (s t iy1 l iy0 ax n))
("stell" nil (s t eh1 l))
("stella" nil (s t eh1 l ax))
("stellar" nil (s t eh1 l er0))
("stellarton" nil (s t eh1 l er0 t ax n))
("stellato" nil (s t eh0 l aa1 t ow0))
("stelle" nil (s t eh1 l))
("stellenbosch" nil (s t eh1 l ax n b ao1 sh))
("steller" nil (s t eh1 l er0))
("stellhorn" nil (s t eh1 l hh er0 n))
("stelling" nil (s t eh1 l ih0 ng))
("stelljes" nil (s t ey1 l y eh0 s))
("stellmach" nil (s t eh1 l m ax k))
("stellmacher" nil (s t eh1 l m ax k er0))
("stellman" nil (s t eh1 l m ax n))
("stello" nil (s t eh1 l ow0))
("stelly" nil (s t eh1 l iy0))
("stelmach" nil (s t eh1 l m ax k))
("stelmack" nil (s t eh1 l m ax k))
("steloff" nil (s t eh1 l ao0 f))
("stelter" nil (s t eh1 l t er0))
("steltz" nil (s t eh1 l t s))
("stelzer" nil (s t eh1 l z er0))
("stelzner" nil (s t eh1 l z n er0))
("stem" nil (s t eh1 m))
("stemberg" nil (s t eh1 m b er0 g))
("stembridge" nil (s t eh1 m b r ih1 jh))
("stemen" nil (s t eh1 m ax n))
("stemler" nil (s t eh1 m l er0))
("stemlike" nil (s t eh1 m l ay1 k))
("stemm" nil (s t eh1 m))
("stemmed" nil (s t eh1 m d))
("stemmer" nil (s t eh1 m er0))
("stemming" nil (s t eh1 m ih0 ng))
("stemmler" nil (s t eh1 m l er0))
("stempel" nil (s t eh1 m p ax l))
("stemper" nil (s t eh1 m p er0))
("stempien" nil (s t eh1 m p iy0 n))
("stemple" nil (s t eh1 m p ax l))
("stempler" nil (s t eh1 m p l er0))
("stemposts" nil (s t eh1 m p ow1 s t s))
("stems" nil (s t eh1 m z))
("sten" nil (s t eh1 n))
("stena" nil (s t iy1 n ax))
("stenberg" nil (s t eh1 n b er0 g))
("stencel" nil (s t eh1 n s ax l))
("stench" nil (s t eh1 n ch))
("stencil" nil (s t eh1 n s ih0 l))
("stenciled" nil (s t eh1 n s ih0 l d))
("stenciling" nil (s t eh1 n s ih0 l ih0 ng))
("stendal" nil (s t eh1 n d ax l))
("stender" nil (s t eh1 n d er0))
("stendig" nil (s t eh1 n d ih0 g))
("stene" nil (s t iy1 n))
("stenerson" nil (s t eh1 n er0 s ax n))
("stengel" nil (s t eh1 ng g ax l))
("stenger" nil (s t eh1 n jh er0))
("stenglein" nil (s t ih1 ng l ay0 n))
("stenholm" nil (s t eh1 n hh ow1 l m))
("stenhouse" nil (s t eh1 n hh aw1 s))
("stenner" nil (s t eh1 n er0))
("stennett" nil (s t eh1 n ih0 t))
("stennis" nil (s t eh1 n ih0 s))
("steno" nil (s t eh1 n ow0))
("stenographic" nil (s t eh1 n ax g r ae1 f ih0 k))
("stenquist" nil (s t eh1 n k w ih1 s t))
("stenseth" nil (s t eh1 n s ih0 th))
("stensland" nil (s t eh1 n s l ax n d))
("stenson" nil (s t eh1 n s ax n))
("stensrud" nil (s t eh1 n s r ax d))
("stenstrom" nil (s t eh1 n s t r ax m))
("stent" nil (s t eh1 n t))
("stentor" nil (s t eh1 n t er0))
("stentors" nil (s t eh1 n t er0 z))
("stentz" nil (s t eh1 n t s))
("stenz" nil (s t eh1 n z))
("stenzel" nil (s t eh1 n z ax l))
("step" nil (s t eh1 p))
("stepan" nil (s t eh1 p ax n))
("stepanakert" nil (s t ax p ae1 n ax k er0 t))
("stepanek" nil (s t eh1 p ax n ih0 k))
("stepanian" nil (s t ih0 p ey1 n iy0 ax n))
("stepanski" nil (s t ih0 p ae1 n s k iy0))
("stepchild" nil (s t eh1 p ch ay1 l d))
("stepchildren" nil (s t eh1 p ch ih1 l d r ax n))
("stepdaughter" nil (s t eh1 p d ao1 t er0))
("stepdaughters" nil (s t eh1 p d ao1 t er0 z))
("stepfather" nil (s t eh1 p f aa1 dh er0))
("steph" nil (s t eh1 f))
("stepha" nil (s t eh1 f ax))
("stephan" nil (s t eh1 f ax n))
("stephana" nil (s t eh0 f aa1 n ax))
("stephani" nil (s t eh1 f ax n iy0))
("stephania" nil (s t ih0 f ae1 n iy0 ax))
("stephanie" nil (s t eh1 f ax n iy0))
("stephanopoulos" nil (s t eh1 f ax n aa1 p ax l ax s))
("stephans" nil (s t eh1 f ax n z))
("stephany" nil (s t eh1 f ax n iy0))
("stephen" nil (s t iy1 v ax n))
("stephens" nil (s t iy1 v ax n z))
("stephenson" nil (s t iy1 v ax n s ax n))
("stephenville" nil (s t iy1 v ih0 n v ih1 l))
("stepien" nil (s t eh1 p iy0 ax n))
("stepka" nil (s t eh1 p k ax))
("stepladders" nil (s t eh1 p l ae1 d er0 z))
("stepmother" nil (s t eh1 p m ah1 dh er0))
("stepney" nil (s t eh1 p n iy0))
("stepp" nil (s t eh1 p))
("steppe" nil (s t eh1 p))
("stepped" nil (s t eh1 p t))
("steppel" nil (s t eh1 p ax l))
("steppenwolf" nil (s t eh1 p ax n w uh1 l f))
("stepper" nil (s t eh1 p er0))
("steppers" nil (s t eh1 p er0 z))
("steppes" nil (s t eh1 p s))
("stepping" nil (s t eh1 p ih0 ng))
("steppingstone" nil (s t eh1 p ih0 ng s t ow1 n))
("steps" nil (s t eh1 p s))
("stepsisters" nil (s t eh1 p s ih1 s t er0 z))
("stepson" nil (s t eh1 p s ah1 n))
("steptoe" nil (s t eh1 p t ow1))
("sterba" nil (s t eh1 r b ax))
("sterbenz" nil (s t er1 b ih0 n s))
("sterchi" nil (s t eh1 r k iy0))
("stereo" nil (s t eh1 r iy0 ow1))
("stereographic" nil (s t eh1 r iy0 ax g r ae1 f ih0 k))
("stereos" nil (s t eh1 r iy0 ow1 z))
("stereotype" nil (s t eh1 r iy0 ax t ay1 p))
("stereotyped" nil (s t eh1 r iy0 ax t ay1 p t))
("stereotypes" nil (s t eh1 r iy0 ax t ay1 p s))
("stereotypical" nil (s t eh1 r iy0 ow0 t ih1 p ih0 k ax l))
("stereotyping" nil (s t eh1 r iy0 ax t ay1 p ih0 ng))
("sterett" nil (s t eh1 r ih0 t))
("sterile" nil (s t eh1 r ax l))
("sterility" nil (s t er0 ih1 l ih0 t iy0))
("sterilization" nil (s t eh1 r ax l ax z ey1 sh ax n))
("sterilize" nil (s t eh1 r ax l ay1 z))
("sterilized" nil (s t eh1 r ax l ay1 z d))
("sterilizer" nil (s t eh1 r ax l ay1 z er0))
("sterilizers" nil (s t eh1 r ax l ay1 z er0 z))
("sterilizes" nil (s t eh1 r ax l ay1 z ih0 z))
("sterilizing" nil (s t eh1 r ax l ay1 z ih0 ng))
("sterk" nil (s t er1 k))
("sterkel" nil (s t er1 k ax l))
("sterle" nil (s t ao1 r ax l))
("sterling" nil (s t er1 l ih0 ng))
("sterman" nil (s t er1 m ax n))
("stermer" nil (s t er1 m er0))
("stern" nil (s t er1 n))
("sternbach" nil (s t er1 n b aa0 k))
("sternberg" nil (s t er1 n b er0 g))
("sternberger" nil (s t er1 n b er0 g er0))
("sterne" nil (s t er1 n))
("sterner" nil (s t er1 n er0))
("sternest" nil (s t er1 n ax s t))
("sternfeld" nil (s t er1 n f eh0 l d))
("sternhagen" nil (s t er1 n hh ax g ax n))
("sternly" nil (s t er1 n l iy0))
("sternness" nil (s t er1 n n ax s))
("sternposts" nil (s t er1 n p ow1 s t s))
("sterns" nil (s t er1 n z))
("steroid" nil (s t er0 oy1 d))
("steroids" nil (s t eh1 r oy0 d z))
("sterr" nil (s t eh1 r))
("sterrett" nil (s t eh1 r ih0 t))
("sterry" nil (s t eh1 r iy0))
("stet" nil (s t eh1 t))
("stethem" nil (s t eh1 th ax m))
("stethoscope" nil (s t eh1 th ax s k ow1 p))
("stethoscopes" nil (s t eh1 th ax s k ow1 p s))
("stetler" nil (s t eh1 t l er0))
("stetson" nil (s t eh1 t s ax n))
("stetter" nil (s t eh1 t er0))
("stettler" nil (s t eh1 t l er0))
("stettner" nil (s t eh1 t n er0))
("stetz" nil (s t eh1 t s))
("stetzel" nil (s t eh1 t z ax l))
("stetzer" nil (s t eh1 t z er0))
("steuart" nil (s t oy1 aa0 r t))
("steuben" nil (s t y uw1 b ih0 n))
("steubenville" nil (s t uw1 b ax n v ih1 l))
("steuber" nil (s t oy1 b er0))
("steuck" nil (s t uw1 k))
("steuer" nil (s t oy1 er0))
("steuerwald" nil (s t oy1 er0 w ao0 l d))
("steurer" nil (s t er1 er0))
("steuver" nil (s t uw1 v er0))
("stevana" nil (s t ih0 v ae1 n ax))
("steve" nil (s t iy1 v))
("stevedore" nil (s t iy1 v ax d ao1 r))
("stevedoring" nil (s t iy1 v ax d ao1 r ih0 ng))
("steven" nil (s t iy1 v ax n))
("stevena" nil (s t eh1 v ih0 n ax))
("stevens" nil (s t iy1 v ax n z))
("stevenson" nil (s t iy1 v ax n s ax n))
("stever" nil (s t iy1 v er0))
("steverson" nil (s t eh1 v er0 s ax n))
("steves" nil (s t iy1 v z))
("steveson" nil (s t eh1 v ih0 s ax n))
("stevick" nil (s t eh1 v ih0 k))
("stevie" nil (s t iy1 v iy0))
("stevison" nil (s t eh1 v ih0 s ax n))
("stew" nil (s t uw1))
("steward" nil (s t uw1 er0 d))
("stewardess" nil (s t uw1 er0 d ax s))
("stewardesses" nil (s t uw1 er0 d ax s ih0 z))
("stewards" nil (s t uw1 er0 d z))
("stewardship" nil (s t uw1 er0 d sh ih1 p))
("stewart" nil (s t uw1 er0 t))
("stewed" nil (s t uw1 d))
("stewing" nil (s t uw1 ih0 ng))
("stews" nil (s t uw1 z))
("steyer" nil (s t ey1 er0))
("sthilaire" nil (s th ax l eh1 r))
("stibel" nil (s t ih1 b ax l))
("stice" nil (s t ay1 s))
("stich" nil (s t ih1 ch))
("sticha" nil (s t ih1 ch ax))
("stichnoth" nil (s t ih1 k n aa1 th))
("sticht" nil (s t ih1 k t))
("stichter" nil (s t ih1 k t er0))
("stick" nil (s t ih1 k))
("sticka" nil (s t ih1 k ax))
("stickball" nil (s t ih1 k b ao1 l))
("stickel" nil (s t ih1 k ax l))
("stickels" nil (s t ih1 k ax l z))
("sticker" nil (s t ih1 k er0))
("stickers" nil (s t ih1 k er0 z))
("stickier" nil (s t ih1 k iy0 er0))
("stickiest" nil (s t ih1 k iy0 ax s t))
("sticking" nil (s t ih1 k ih0 ng))
("stickland" nil (s t ih1 k l ax n d))
("stickle" nil (s t ih1 k ax l))
("stickler" nil (s t ih1 k ax l er0))
("stickles" nil (s t ih1 k ax l z))
("stickley" nil (s t ih1 k l iy0))
("stickney" nil (s t ih1 k n iy0))
("sticks" nil (s t ih1 k s))
("sticktights" nil (s t ih1 k t ay1 t s))
("sticky" nil (s t ih1 k iy0))
("stidd" nil (s t ih1 d))
("stidham" nil (s t ih1 d hh ax m))
("stiebel" nil (s t iy1 b ax l))
("stieber" nil (s t iy1 b er0))
("stief" nil (s t iy1 f))
("stiefel" nil (s t iy1 f ax l))
("stiefvater" nil (s t iy1 f v ax t er0))
("stieg" nil (s t iy1 g))
("stiegemeier" nil (s t iy1 jh ax m ay1 r))
("stiegler" nil (s t iy1 g l er0))
("stieglitz" nil (s t iy1 g l ih0 t s))
("stiehl" nil (s t iy1 l))
("stiens" nil (s t iy1 n z))
("stier" nil (s t ay1 er0))
("stiers" nil (s t ay1 er0 z))
("stierwalt" nil (s t ih1 r w ax l t))
("stieve" nil (s t iy1 v))
("stifel" nil (s t ih1 f ax l))
("stiff" nil (s t ih1 f))
("stiffed" nil (s t ih1 f t))
("stiffel" nil (s t ih1 f ax l))
("stiffen" nil (s t ih1 f ax n))
("stiffened" nil (s t ih1 f ax n d))
("stiffener" nil (s t ih1 f ax n er0))
("stiffening" nil (s t ih1 f ax n ih0 ng))
("stiffens" nil (s t ih1 f ax n z))
("stiffer" nil (s t ih1 f er0))
("stiffest" nil (s t ih1 f ax s t))
("stiffler" nil (s t ih1 f l er0))
("stiffly" nil (s t ih1 f l iy0))
("stiffness" nil (s t ih1 f n ax s))
("stiffs" nil (s t ih1 f s))
("stifle" nil (s t ay1 f ax l))
("stifled" nil (s t ay1 f ax l d))
("stifles" nil (s t ay1 f ax l z))
("stifling" nil (s t ay1 f l ih0 ng))
("stifter" nil (s t ih1 f t er0))
("stig" nil (s t ih1 g))
("stigall" nil (s t ih1 g ax l))
("stiger" nil (s t ay1 g er0))
("stigers" nil (s t ay1 g er0 z))
("stigler" nil (s t ih1 g l er0))
("stiglitz" nil (s t ih1 g l ih0 t s))
("stigma" nil (s t ih1 g m ax))
("stigmatism" nil (s t ih1 g m ax t ih1 z ax m))
("stihl" nil (s t ih1 l))
("stika" nil (s t ih1 k ax))
("stikeleather" nil (s t ih1 k ih0 l eh0 dh er0))
("stile" nil (s t ay1 l))
("stiles" nil (s t ay1 l z))
("stiletto" nil (s t ax l eh1 t ow0))
("stilettos" nil (s t ax l eh1 t ow0 z))
("still" nil (s t ih1 l))
("stillborn" nil (s t ih1 l b ao1 r n))
("stille" nil (s t ih1 l))
("stilled" nil (s t ih1 l d))
("stiller" nil (s t ih1 l er0))
("stilley" nil (s t ih1 l iy0))
("stillinger" nil (s t ih1 l ih0 ng er0))
("stillings" nil (s t ih1 l ih0 ng z))
("stillion" nil (s t ih1 l y ax n))
("stillman" nil (s t ih1 l m ax n))
("stillness" nil (s t ih1 l n ax s))
("stills" nil (s t ih1 l z))
("stillson" nil (s t ih1 l s ax n))
("stillwagon" nil (s t ih1 l w ae1 g ax n))
("stillwater" nil (s t ih1 l w ao1 t er0))
("stillwell" nil (s t ih1 l w eh1 l))
("stilson" nil (s t ih1 l s ax n))
("stilted" nil (s t ih1 l t ih0 d))
("stiltner" nil (s t ih1 l t n er0))
("stilts" nil (s t ih1 l t s))
("stilwell" nil (s t ih1 l w eh1 l))
("stimac" nil (s t ih1 m ax k))
("stimmel" nil (s t ih1 m ax l))
("stimpert" nil (s t ih1 m p er0 t))
("stimpson" nil (s t ih1 m p s ax n))
("stimson" nil (s t ih1 m s ax n))
("stimulant" nil (s t ih1 m y ax l ax n t))
("stimulants" nil (s t ih1 m y ax l ax n t s))
("stimulate" nil (s t ih1 m y ax l ey1 t))
("stimulated" nil (s t ih1 m y ax l ey1 t ax d))
("stimulates" nil (s t ih1 m y ax l ey1 t s))
("stimulating" nil (s t ih1 m y ax l ey1 t ih0 ng))
("stimulation" nil (s t ih1 m y ax l ey1 sh ax n))
("stimulative" nil (s t ih1 m y ax l ey1 t ih0 v))
("stimulator" nil (s t ih1 m y ax l ey1 t er0))
("stimulators" nil (s t ih1 m y ax l ey1 t er0 z))
("stimuli" nil (s t ih1 m y ax l ay1))
("stimulus" nil (s t ih1 m y ax l ax s))
("stinchcomb" nil (s t ih1 n ch k ax m))
("stinchfield" nil (s t ih1 n ch f iy0 l d))
("stine" nil (s t ay1 n))
("stinebaugh" nil (s t ih1 n ih0 b ao0))
("stinel" nil (s t ih1 n ax l))
("stineman" nil (s t ay1 n m ax n))
("stiner" nil (s t ay1 n er0))
("stines" nil (s t ay1 n z))
("sting" nil (s t ih1 ng))
("stinger" nil (s t ih1 ng er0))
("stingers" nil (s t ih1 ng er0 z))
("stingier" nil (s t ih1 n jh iy0 er0))
("stinginess" nil (s t ih1 n jh iy0 n ih0 s))
("stinging" nil (s t ih1 ng ih0 ng))
("stingley" nil (s t ih1 ng g l iy0))
("stings" nil (s t ih1 ng z))
("stingy" nil (s t ih1 n jh iy0))
("stink" nil (s t ih1 ng k))
("stinkbugs" nil (s t ih1 ng k b ax g z))
("stinkers" nil (s t ih1 ng k er0 z))
("stinking" nil (s t ih1 ng k ih0 ng))
("stinks" nil (s t ih1 ng k s))
("stinnett" nil (s t ih1 n ih0 t))
("stinnette" nil (s t ih0 n eh1 t))
("stinson" nil (s t ih1 n s ax n))
("stint" nil (s t ih1 n t))
("stints" nil (s t ih1 n t s))
("stipanovich" nil (s t ih0 p ae1 n ax v ih0 ch))
("stipe" nil (s t ay1 p))
("stipend" nil (s t ih1 p ax n d))
("stipends" nil (s t ih1 p ax n d z))
("stipes" nil (s t ay1 p s))
("stipp" nil (s t ih1 p))
("stipulate" nil (s t ih1 p y ax l ey1 t))
("stipulated" nil (s t ih1 p y ax l ey1 t ih0 d))
("stipulates" nil (s t ih1 p y ax l ey1 t s))
("stipulating" nil (s t ih1 p y ax l ey1 t ih0 ng))
("stipulation" nil (s t ih1 p y ax l ey1 sh ax n))
("stipulations" nil (s t ih1 p y ax l ey1 sh ax n z))
("stir" nil (s t er1))
("stires" nil (s t ay1 r z))
("stirewalt" nil (s t ao1 r uw0 ax l t))
("stirling" nil (s t er1 l ih0 ng))
("stirn" nil (s t er1 n))
("stirred" nil (s t er1 d))
("stirring" nil (s t er1 ih0 ng))
("stirrings" nil (s t er1 ih0 ng z))
("stirrup" nil (s t er1 ax p))
("stirrups" nil (s t er1 ax p s))
("stirs" nil (s t er1 z))
("stitch" nil (s t ih1 ch))
("stitched" nil (s t ih1 ch t))
("stitches" nil (s t ih1 ch ih0 z))
("stitching" nil (s t ih1 ch ih0 ng))
("stiteler" nil (s t ay1 t l er0))
("stitely" nil (s t ay1 t l iy0))
("stites" nil (s t ay1 t s))
("stith" nil (s t ih1 th))
("stitt" nil (s t ih1 t))
("stitz" nil (s t ih1 t s))
("stitzel" nil (s t ih1 t z ax l))
("stitzer" nil (s t ih1 t z er0))
("stiver" nil (s t ay1 v er0))
("stivers" nil (s t ay1 v er0 z))
("stiverson" nil (s t ih1 v er0 s ax n))
("stivoric" nil (s t ih0 v ao1 r ih0 k))
("stobaugh" nil (s t aa1 b ao0))
("stobbe" nil (s t aa1 b))
("stober" nil (s t ow1 b er0))
("stobie" nil (s t aa1 b iy0))
("stock" nil (s t aa1 k))
("stockard" nil (s t aa1 k er0 d))
("stockbridge" nil (s t aa1 k b r ih0 jh))
("stockbroker" nil (s t aa1 k b r ow1 k er0))
("stockbrokerage" nil (s t aa1 k b r ow1 k er0 ih0 jh))
("stockbrokerages" nil (s t aa1 k b r ow1 k er0 ih0 jh ih0 z))
("stockbrokers" nil (s t aa1 k b r ow1 k er0 z))
("stockbroking" nil (s t aa1 k b r ow1 k ih0 ng))
("stockburger" nil (s t aa1 k b er0 g er0))
("stockdale" nil (s t aa1 k d ey1 l))
("stocke" nil (s t aa1 k))
("stocked" nil (s t aa1 k t))
("stockel" nil (s t aa1 k ax l))
("stocker" nil (s t aa1 k er0))
("stockert" nil (s t aa1 k er0 t))
("stockett" nil (s t aa1 k ih0 t))
("stockford" nil (s t aa1 k f er0 d))
("stockham" nil (s t aa1 k hh ax m))
("stockhausen" nil (s t aa1 k hh aw1 z ax n))
("stockholder" nil (s t aa1 k hh ow1 l d er0))
("stockholders" nil (s t aa1 k hh ow1 l d er0 z))
("stockholding" nil (s t aa1 k hh ow1 l d ih0 ng))
("stockholdings" nil (s t aa1 k hh ow1 l d ih0 ng z))
("stockholm" nil (s t aa1 k hh ow1 l m))
("stocking" nil (s t aa1 k ih0 ng))
("stockinger" nil (s t aa1 k ih0 ng er0))
("stockings" nil (s t aa1 k ih0 ng z))
("stockley" nil (s t aa1 k l iy0))
("stockman" nil (s t aa1 k m ax n))
("stockmarket" nil (s t aa1 k m aa1 r k ih0 t))
("stockpile" nil (s t aa1 k p ay1 l))
("stockpiled" nil (s t aa1 k p ay1 l d))
("stockpiles" nil (s t aa1 k p ay1 l z))
("stockpiling" nil (s t aa1 k p ay1 l ih0 ng))
("stockroom" nil (s t aa1 k r uw1 m))
("stocks" nil (s t aa1 k s))
("stocksdale" nil (s t aa1 k s d ey1 l))
("stockslager" nil (s t aa1 k s l ey0 g er0))
("stockstill" nil (s t aa1 k s t ih1 l))
("stockton" nil (s t aa1 k t ax n))
("stockwell" nil (s t aa1 k w eh1 l))
("stocky" nil (s t aa1 k iy0))
("stockyard" nil (s t aa1 k y aa1 r d))
("stockyards" nil (s t aa1 k y aa1 r d z))
("stocum" nil (s t ow1 k ax m))
("stoddard" nil (s t aa1 d er0 d))
("stodghill" nil (s t aa1 jh hh ih1 l))
("stodgy" nil (s t aa1 jh iy0))
("stodola" nil (s t ow0 d ow1 l ax))
("stodolsky" nil (s t ax d ao1 l s k iy0))
("stoeber" nil (s t ow1 b er0))
("stoeckel" nil (s t ow1 k ax l))
("stoecker" nil (s t ow1 k er0))
("stoeckle" nil (s t ow1 k ax l))
("stoecklein" nil (s t ow1 k l ay1 n))
("stoeger" nil (s t ow1 g er0))
("stoehr" nil (s t ao1 r))
("stoelting" nil (s t ow1 l t ih0 ng))
("stoeltze" nil (s t ow1 l t s))
("stoermer" nil (s t ao1 r m er0))
("stoessel" nil (s t ow1 s ax l))
("stoever" nil (s t aa1 eh0 v er0))
("stofer" nil (s t ow1 f er0))
("stoff" nil (s t ao1 f))
("stoffel" nil (s t aa1 f ax l))
("stoffels" nil (s t aa1 f ax l z))
("stoffer" nil (s t ao1 f er0))
("stoffers" nil (s t ao1 f er0 z))
("stoffregen" nil (s t aa1 f r ih0 g ax n))
("stofko" nil (s t ow1 f k ow0))
("stoga" nil (s t ow1 g ax))
("stogdill" nil (s t aa1 g d ih0 l))
("stogner" nil (s t aa1 g n er0))
("stogsdill" nil (s t aa1 g z d ih0 l))
("stohl" nil (s t ow1 l))
("stohler" nil (s t ow1 l er0))
("stohr" nil (s t ao1 r))
("stoia" nil (s t ow1 y ax))
("stoiber" nil (s t oy1 b er0))
("stoic" nil (s t ow1 ih0 k))
("stoically" nil (s t ow1 ih0 k l iy0))
("stoicism" nil (s t ow1 ax s ih1 z ax m))
("stoics" nil (s t ow1 ih0 k s))
("stoke" nil (s t ow1 k))
("stoked" nil (s t ow1 k t))
("stokely" nil (s t ow1 k l iy0))
("stoker" nil (s t ow1 k er0))
("stokes" nil (s t ow1 k s))
("stoking" nil (s t ow1 k ih0 ng))
("stokke" nil (s t aa1 k))
("stokley" nil (s t aa1 k l iy0))
("stoklosa" nil (s t ax k l ow1 s ax))
("stol" nil (s t ow1 l))
("stolar" nil (s t ow1 l er0))
("stolarski" nil (s t ax l aa1 r s k iy0))
("stolarz" nil (s t ow1 l aa0 r z))
("stolberg" nil (s t ow1 l b er0 g))
("stoldt" nil (s t ow1 l t))
("stole" nil (s t ow1 l))
("stolen" nil (s t ow1 l ax n))
("stoler" nil (s t ow1 l er0))
("stolfi" nil (s t ow1 l f iy0))
("stolichnaya" nil (s t ow1 l ih0 k n ay1 ax))
("stolid" nil (s t aa1 l ax d))
("stoll" nil (s t ow1 l))
("stollar" nil (s t aa1 l er0))
("stolle" nil (s t aa1 l))
("stollen" nil (s t aa1 l ax n))
("stoller" nil (s t ow1 l er0))
("stolley" nil (s t aa1 l iy0))
("stollings" nil (s t ow1 l ih0 ng z))
("stolp" nil (s t ow1 l p))
("stolpe" nil (s t ow1 l p))
("stolper" nil (s t ow1 l p er0))
("stolt" nil (s t ow1 l t))
("stolte" nil (s t ow1 l t))
("stoltenberg" nil (s t ow1 l t ax n b er0 g))
("stoltman" nil (s t ow1 l t m ax n))
("stoltz" nil (s t ow1 l t s))
("stoltzfus" nil (s t ow1 l t s f ax s))
("stoltzman" nil (s t ow1 l t s m ax n))
("stolz" nil (s t ow1 l z))
("stolze" nil (s t ow1 l z))
("stomach" nil (s t ah1 m ax k))
("stomachs" nil (s t ah1 m ax k s))
("stomata" nil (s t ow1 m ax t ax))
("stombaugh" nil (s t aa1 m b ao1))
("stomp" nil (s t aa1 m p))
("stomped" nil (s t aa1 m p t))
("stomping" nil (s t aa1 m p ih0 ng))
("stone" nil (s t ow1 n))
("stoneback" nil (s t ow1 n b ae1 k))
("stoneberg" nil (s t ow1 n b er0 g))
("stoneberger" nil (s t ow1 n b er0 g er0))
("stonebraker" nil (s t aa1 n ih0 b r ax k er0))
("stoneburner" nil (s t ow1 n b er1 n er0))
("stonecipher" nil (s t ow1 n s ay1 f er0))
("stonecutters" nil (s t ow1 n k ah1 t er0 z))
("stonecypher" nil (s t aa1 n ih0 s ih0 f er0))
("stoned" nil (s t ow1 n d))
("stoneham" nil (s t ow1 n ax m))
("stonehenge" nil (s t ow1 n hh eh1 n jh))
("stonehill" nil (s t ow1 n hh ih1 l))
("stonehocker" nil (s t ow1 n hh aa1 k er0))
("stonehouse" nil (s t ow1 n hh aw1 s))
("stoneking" nil (s t ow1 n k ih1 ng))
("stoneman" nil (s t ow1 n m ax n))
("stoner" nil (s t ow1 n er0))
("stoneridge" nil (s t ow1 n r ih1 jh))
("stonerock" nil (s t ow1 n r aa1 k))
("stones" nil (s t ow1 n z))
("stonesifer" nil (s t aa1 n ih0 s ih0 f er0))
("stonestreet" nil (s t ow1 n s t r iy1 t))
("stonewall" nil (s t ow1 n w ao1 l))
("stonewalled" nil (s t ow1 n w ao1 l d))
("stonewalling" nil (s t ow1 n w ao1 l ih0 ng))
("stoneware" nil (s t ow1 n w eh1 r))
("stoney" nil (s t ow1 n iy0))
("stong" nil (s t ao1 ng))
("stonge" nil (s t aa1 n jh))
("stoning" nil (s t ow1 n ih0 ng))
("stony" nil (s t ow1 n iy0))
("stonyfield" nil (s t ow1 n iy0 f iy1 l d))
("stood" nil (s t uh1 d))
("stooges" nil (s t uw1 jh ih0 z))
("stookey" nil (s t uw1 k iy0))
("stooksbury" nil (s t uw1 k s b eh0 r iy0))
("stool" nil (s t uw1 l))
("stools" nil (s t uw1 l z))
("stoop" nil (s t uw1 p))
("stooped" nil (s t uw1 p t))
("stoops" nil (s t uw1 p s))
("stoots" nil (s t uw1 t s))
("stop" nil (s t aa1 p))
("stopa" nil (s t ow1 p ax))
("stopgap" nil (s t aa1 p g ae1 p))
("stopher" nil (s t aa1 f er0))
("stopka" nil (s t ow1 p k ax))
("stoplight" nil (s t aa1 p l ay1 t))
("stoplights" nil (s t aa1 p l ay1 t s))
("stopover" nil (s t aa1 p ow1 v er0))
("stopovers" nil (s t aa1 p ow1 v er0 z))
("stoppage" nil (s t aa1 p ih0 jh))
("stoppages" nil (s t aa1 p ih0 jh ih0 z))
("stopped" nil (s t aa1 p t))
("stoppel" nil (s t aa1 p ax l))
("stopper" nil (s t aa1 p er0))
("stoppers" nil (s t aa1 p er0 z))
("stopping" nil (s t aa1 p ih0 ng))
("stops" nil (s t aa1 p s))
("stopwatch" nil (s t aa1 p w aa1 ch))
("stopwatches" nil (s t aa1 p w aa1 ch ih0 z))
("stora" nil (s t ao1 r ax))
("storage" nil (s t ao1 r ax jh))
("storagetek" nil (s t ao1 r ax jh t eh1 k))
("storaska" nil (s t ao1 r ae1 s ax))
("storbeck" nil (s t ao1 r b eh0 k))
("storch" nil (s t ao1 r k))
("storck" nil (s t ao1 r k))
("stordahl" nil (s t ao1 r d aa0 l))
("store" nil (s t ao1 r))
("stored" nil (s t ao1 r d))
("storefront" nil (s t ao1 r f r aa1 n t))
("storefronts" nil (s t ao1 r f r aa1 n t s))
("storehouse" nil (s t ao1 r hh aw1 s))
("storehouses" nil (s t ao1 r hh aw1 z ax z))
("storekeepers" nil (s t ao1 r k iy1 p er0 z))
("storer" nil (s t ao1 r er0))
("storeroom" nil (s t ao1 r r uw1 m))
("stores" nil (s t ao1 r z))
("storey" nil (s t ao1 r iy0))
("storie" nil (s t ao1 r iy0))
("storied" nil (s t ao1 r iy0 d))
("stories" nil (s t ao1 r iy0 z))
("storing" nil (s t ao1 r ih0 ng))
("stork" nil (s t ao1 r k))
("storlie" nil (s t ao1 r l iy0))
("storm" nil (s t ao1 r m))
("stormed" nil (s t ao1 r m d))
("storment" nil (s t ao1 r m ax n t))
("stormer" nil (s t ao1 r m er0))
("stormes" nil (s t ao1 r m z))
("storming" nil (s t ao1 r m ih0 ng))
("stormont" nil (s t ao1 r m ow0 n t))
("storms" nil (s t ao1 r m z))
("stormy" nil (s t ao1 r m iy0))
("storr" nil (s t ao1 r))
("storrow" nil (s t aa1 r ow0))
("storrs" nil (s t ao1 r z))
("storti" nil (s t ao1 r t iy0))
("storting" nil (s t ao1 r t ih0 ng))
("storts" nil (s t ao1 r t s))
("stortz" nil (s t ao1 r t s))
("story" nil (s t ao1 r iy0))
("storyboard" nil (s t ao1 r iy0 b ao1 r d))
("storybook" nil (s t ao1 r iy0 b uh1 k))
("storybooks" nil (s t ao1 r iy0 b uh1 k s))
("storyline" nil (s t ao1 r iy0 l ay1 n))
("storyteller" nil (s t ao1 r iy0 t eh1 l er0))
("storytellers" nil (s t ao1 r iy0 t eh1 l er0 z))
("storytelling" nil (s t ao1 r iy0 t eh1 l ih0 ng))
("storz" nil (s t ao1 r z))
("stossel" nil (s t aa1 s ax l))
("stotler" nil (s t aa1 t l er0))
("stott" nil (s t aa1 t))
("stottlemyer" nil (s t aa1 t l ih0 m iy0 er0))
("stotts" nil (s t aa1 t s))
("stotz" nil (s t aa1 t s))
("stoudemire" nil (s t uw1 d ax m ay1 r))
("stoudenmire" nil (s t uw1 d ax n m ay1 r))
("stouder" nil (s t ah1 d er0))
("stoudt" nil (s t ah1 d t))
("stouffer" nil (s t ah1 f er0))
("stough" nil (s t ah1 f))
("stoughton" nil (s t ow1 t ax n))
("stoup" nil (s t uw1 p))
("stout" nil (s t aw1 t))
("stoute" nil (s t aw1 t))
("stoutenburg" nil (s t aw1 t ax n b er0 g))
("stoutly" nil (s t aw1 t l iy0))
("stoutness" nil (s t aw1 t n ax s))
("stovall" nil (s t ow1 v aa0 l))
("stove" nil (s t ow1 v))
("stover" nil (s t ow1 v er0))
("stoves" nil (s t ow1 v z))
("stow" nil (s t ow1))
("stowe" nil (s t ow1))
("stowed" nil (s t ow1 d))
("stowell" nil (s t aa1 w eh0 l))
("stowers" nil (s t ow1 er0 z))
("stoy" nil (s t oy1))
("stoyer" nil (s t oy1 er0))
("stracener" nil (s t r ae1 s iy0 n er0))
("strachan" nil (s t r ae1 ch ax n))
("strack" nil (s t r ae1 k))
("stracke" nil (s t r ae1 k))
("strada" nil (s t r aa1 d ax))
("straddle" nil (s t r ae1 d ax l))
("straddled" nil (s t r ae1 d ax l d))
("straddles" nil (s t r ae1 d ax l z))
("straddling" nil (s t r ae1 d l ih0 ng))
("strader" nil (s t r ey1 d er0))
("stradford" nil (s t r ae1 d f er0 d))
("stradivarius" nil (s t r ae1 d ih0 v eh1 r iy0 ax s))
("stradley" nil (s t r ae1 d l iy0))
("stradling" nil (s t r ae1 d l ih0 ng))
("straggled" nil (s t r ae1 g ax l d))
("stragglers" nil (s t r ae1 g l er0 z))
("strahan" nil (s t r ae1 hh ax n))
("strahl" nil (s t r aa1 l))
("strahle" nil (s t r ey1 hh ax l))
("strahler" nil (s t r aa1 l er0))
("strahm" nil (s t r aa1 m))
("straight" nil (s t r ey1 t))
("straightedge" nil (s t r ey1 t eh1 jh))
("straighten" nil (s t r ey1 t ax n))
("straightened" nil (s t r ey1 t ax n d))
("straightening" nil (s t r ey1 t ax n ih0 ng))
("straightens" nil (s t r ey1 t ax n z))
("straighter" nil (s t r ey1 t er0))
("straightforward" nil (s t r ey1 t f ao1 r w er0 d))
("straightforwardly" nil (s t r ey1 t f ao1 r w er0 d l iy0))
("straights" nil (s t r ey1 t s))
("strain" nil (s t r ey1 n))
("strained" nil (s t r ey1 n d))
("straining" nil (s t r ey1 n ih0 ng))
("strains" nil (s t r ey1 n z))
("strait" nil (s t r ey1 t))
("straitjacket" nil (s t r ey1 t jh ae1 k ax t))
("straits" nil (s t r ey1 t s))
("straka" nil (s t r aa1 k ax))
("straker" nil (s t r ey1 k er0))
("straley" nil (s t r ae1 l iy0))
("stram" nil (s t r ae1 m))
("stranahan" nil (s t r ae1 n ax hh ae0 n))
("strand" nil (s t r ae1 n d))
("strandberg" nil (s t r ae1 n d b er0 g))
("strande" nil (s t r ae1 n d))
("stranded" nil (s t r ae1 n d ax d))
("stranding" nil (s t r ae1 n d ih0 ng))
("strandlines" nil (s t r ae1 n d l ay1 n z))
("strands" nil (s t r ae1 n d z))
("strang" nil (s t r ae1 ng))
("strange" nil (s t r ey1 n jh))
("strangely" nil (s t r ey1 n jh l iy0))
("strangeness" nil (s t r ey1 n jh n ax s))
("stranger" nil (s t r ey1 n jh er0))
("strangers" nil (s t r ey1 n jh er0 z))
("strangest" nil (s t r ey1 n jh ih0 s t))
("strangis" nil (s t r ae1 n jh ih0 s))
("strangle" nil (s t r ae1 ng g ax l))
("strangled" nil (s t r ae1 ng g ax l d))
("stranglehold" nil (s t r ae1 ng g ax l hh ow1 l d))
("strangling" nil (s t r ae1 ng g ax l ih0 ng))
("strangulation" nil (s t r ae1 ng g y ax l ey1 sh ax n))
("strangulations" nil (s t r ae1 ng g y ax l ey1 sh ax n z))
("strangwayes" nil (s t r ae1 ng w ey1 z))
("strano" nil (s t r aa1 n ow0))
("stransky" nil (s t r ae1 n s k iy0))
("strap" nil (s t r ae1 p))
("strapped" nil (s t r ae1 p t))
("strapping" nil (s t r ae1 p ih0 ng))
("straps" nil (s t r ae1 p s))
("strasbourg" nil (s t r ae1 s b ao1 r g))
("strasburg" nil (s t r ae1 s b er0 g))
("strasburger" nil (s t r ae1 s b er0 g er0))
("strassburg" nil (s t r ae1 s b er0 g))
("strassburger" nil (s t r ae1 s b er0 g er0))
("strasser" nil (s t r ae1 s er0))
("strassman" nil (s t r ae1 s m ax n))
("strassner" nil (s t r ae1 s n er0))
("straszheim" nil (s t r ae1 s hh ay1 m))
("strata" nil (s t r ae1 t ax))
("stratacom" nil (s t r ae1 t ax k aa0 m))
("stratagems" nil (s t r ae1 t ax jh ax m z))
("stratas" nil (s t r ae1 t ax z))
("strate" nil (s t r ey1 t))
("strategem" nil (s t r ae1 t ih0 jh eh0 m))
("strategic" nil (s t r ax t iy1 jh ih0 k))
("strategical" nil (s t r ax t iy1 jh ih0 k ax l))
("strategically" nil (s t r ax t iy1 jh ih0 k l iy0))
("strategies" nil (s t r ae1 t ax jh iy0 z))
("strategist" nil (s t r ae1 t ih0 jh ih0 s t))
("strategists" nil (s t r ae1 t ih0 jh ih0 s t s))
("strategy" nil (s t r ae1 t ax jh iy0))
("strater" nil (s t r ey1 t er0))
("stratford" nil (s t r ae1 t f er0 d))
("strathman" nil (s t r ae1 th m ax n))
("stratified" nil (s t r ae1 t ax f ay1 d))
("stratigraphic" nil (s t r ae1 t ax g r ae1 f ih0 k))
("stratman" nil (s t r ae1 t m ax n))
("stratmann" nil (s t r ae1 t m ax n))
("stratoflex" nil (s t r ae1 t ow0 f l eh1 k s))
("stratosphere" nil (s t r ae1 t ax s f ih1 r))
("stratospheric" nil (s t r ae1 t ax s f ih1 r ih0 k))
("stratton" nil (s t r ae1 t ax n))
("stratum" nil (s t r ae1 t ax m))
("stratus" nil (s t r ae1 t ax s))
("straub" nil (s t r aw1 b))
("straube" nil (s t r aw1 b))
("strauch" nil (s t r aw1 k))
("straughan" nil (s t r ao1 ax n))
("straughn" nil (s t r ao1 n))
("straughter" nil (s t r ao1 t er0))
("straus" nil (s t r aw1 s))
("strausbaugh" nil (s t r aw1 s b aw0))
("strause" nil (s t r aw1 s))
("strauser" nil (s t r aw1 s er0))
("strauss" nil (s t r aw1 s))
("strausser" nil (s t r aw1 s er0))
("stravinsky" nil (s t r ax v ih1 n s k iy0))
("straw" nil (s t r ao1))
("strawberries" nil (s t r ao1 b eh1 r iy0 z))
("strawberry" nil (s t r ao1 b eh1 r iy0))
("strawbridge" nil (s t r ao1 b r ih1 jh))
("strawder" nil (s t r ao1 d er0))
("strawderman" nil (s t r ao1 d er0 m ax n))
("strawn" nil (s t r ao1 n))
("straws" nil (s t r ao1 z))
("strawser" nil (s t r ao1 z er0))
("stray" nil (s t r ey1))
("strayed" nil (s t r ey1 d))
("strayer" nil (s t r ey1 er0))
("strayhorn" nil (s t r ey1 hh er0 n))
("straying" nil (s t r ey1 ih0 ng))
("strays" nil (s t r ey1 z))
("streak" nil (s t r iy1 k))
("streaked" nil (s t r iy1 k t))
("streaker" nil (s t r iy1 k er0))
("streaking" nil (s t r iy1 k ih0 ng))
("streaks" nil (s t r iy1 k s))
("stream" nil (s t r iy1 m))
("streamed" nil (s t r iy1 m d))
("streamers" nil (s t r iy1 m er0 z))
("streaming" nil (s t r iy1 m ih0 ng))
("streamline" nil (s t r iy1 m l ay1 n))
("streamlined" nil (s t r iy1 m l ay1 n d))
("streamlining" nil (s t r iy1 m l ay1 n ih0 ng))
("streams" nil (s t r iy1 m z))
("streater" nil (s t r iy1 t er0))
("streb" nil (s t r eh1 b))
("strebe" nil (s t r iy1 b))
("strebeck" nil (s t r iy1 b eh0 k))
("strebel" nil (s t r eh1 b ax l))
("streck" nil (s t r eh1 k))
("strecker" nil (s t r eh1 k er0))
("streed" nil (s t r iy1 d))
("streep" nil (s t r iy1 p))
("streeper" nil (s t r iy1 p er0))
("street" nil (s t r iy1 t))
("streetcar" nil (s t r iy1 t k aa1 r))
("streeter" nil (s t r iy1 t er0))
("streeters" nil (s t r iy1 t er0 z))
("streetman" nil (s t r iy1 t m ax n))
("streets" nil (s t r iy1 t s))
("streett" nil (s t r iy1 t))
("streety" nil (s t r iy1 t iy0))
("streff" nil (s t r eh1 f))
("strege" nil (s t r iy1 jh))
("strehl" nil (s t r eh1 l))
("strehle" nil (s t r eh1 l))
("strehlow" nil (s t r eh1 l ow0))
("streib" nil (s t r ay1 b))
("streich" nil (s t r ay1 k))
("streicher" nil (s t r ay1 k er0))
("streiff" nil (s t r ay1 f))
("streight" nil (s t r ey1 t))
("streisand" nil (s t r ay1 z ax n d))
("streit" nil (s t r ay1 t))
("streitmatter" nil (s t r ay1 t m ax t er0))
("strelow" nil (s t r eh1 l ow0))
("streng" nil (s t r eh1 ng))
("strenger" nil (s t r eh1 ng g er0))
("strength" nil (s t r eh1 ng k th))
("strengthen" nil (s t r eh1 ng th ax n))
("strengthened" nil (s t r eh1 ng th ax n d))
("strengthening" nil (s t r eh1 ng th ax n ih0 ng))
("strengthens" nil (s t r eh1 ng th ax n z))
("strengths" nil (s t r eh1 ng k th s))
("strenio" nil (s t r iy1 n iy0 ow0))
("strenuous" nil (s t r eh1 n y uw0 ax s))
("strenuously" nil (s t r eh1 n y uw0 ax s l iy0))
("strep" nil (s t r eh1 p))
("streptokinase" nil (s t r eh1 p t ax k ay1 n ey1 s))
("stress" nil (s t r eh1 s))
("stressed" nil (s t r eh1 s t))
("stresses" nil (s t r eh1 s ax z))
("stressful" nil (s t r eh1 s f ax l))
("stressing" nil (s t r eh1 s ih0 ng))
("stretch" nil (s t r eh1 ch))
("stretched" nil (s t r eh1 ch t))
("stretcher" nil (s t r eh1 ch er0))
("stretches" nil (s t r eh1 ch ax z))
("stretching" nil (s t r eh1 ch ih0 ng))
("stretto" nil (s t r eh1 t ow1))
("strevig" nil (s t r eh1 v ih0 g))
("strew" nil (s t r uw1))
("strewn" nil (s t r uw1 n))
("strey" nil (s t r ey1))
("striar" nil (s t r ay1 r))
("stribling" nil (s t er1 ax b ax l ih0 ng))
("strick" nil (s t r ih1 k))
("stricken" nil (s t r ih1 k ax n))
("stricker" nil (s t r ih1 k er0))
("strickland" nil (s t r ih1 k l ax n d))
("stricklen" nil (s t r ih1 k ax l ax n))
("strickler" nil (s t r ih1 k l er0))
("stricklin" nil (s t r ih1 k l ih0 n))
("strickling" nil (s t r ih1 k l ih0 ng))
("strict" nil (s t r ih1 k t))
("stricter" nil (s t r ih1 k t er0))
("strictest" nil (s t r ih1 k t ax s t))
("strictly" nil (s t r ih1 k t l iy0))
("strictures" nil (s t r ih1 k ch er0 z))
("stride" nil (s t r ay1 d))
("stridency" nil (s t r ay1 d ax n s iy0))
("strident" nil (s t r ay1 d ax n t))
("stridently" nil (s t r ay1 d ax n t l iy0))
("strider" nil (s t r ay1 d er0))
("strides" nil (s t r ay1 d z))
("striding" nil (s t r ay1 d ih0 ng))
("striegel" nil (s t r iy1 g ax l))
("strieker" nil (s t r iy1 k er0))
("strieter" nil (s t r iy1 t er0))
("strife" nil (s t r ay1 f))
("strifes" nil (s t r ay1 f s))
("strike" nil (s t r ay1 k))
("strikebreakers" nil (s t r ay1 k b r ey1 k er0 z))
("strikeout" nil (s t r ay1 k aw1 t))
("strikeouts" nil (s t r ay1 k aw1 t s))
("striker" nil (s t r ay1 k er0))
("strikers" nil (s t r ay1 k er0 z))
("strikes" nil (s t r ay1 k s))
("striking" nil (s t r ay1 k ih0 ng))
("strikingly" nil (s t r ay1 k ih0 ng l iy0))
("strimple" nil (s t r ih1 m p ax l))
("strinden" nil (s t r ih1 n d ax n))
("strine" nil (s t r ay1 n))
("string" nil (s t r ih1 ng))
("stringed" nil (s t r ih1 ng d))
("stringency" nil (s t r ih1 n jh ax n s iy0))
("stringent" nil (s t r ih1 n jh ax n t))
("stringently" nil (s t r ih1 n jh ax n t l iy0))
("stringer" nil (s t r ih1 ng er0))
("stringers" nil (s t r ih1 ng er0 z))
("stringfellow" nil (s t r ih1 ng f eh1 l ow0))
("stringfield" nil (s t r ih1 ng f iy1 l d))
("stringham" nil (s t r ih1 ng hh ae1 m))
("stringing" nil (s t r ih1 ng ih0 ng))
("strings" nil (s t r ih1 ng z))
("stringy" nil (s t r ih1 ng iy0))
("strip" nil (s t r ih1 p))
("stripe" nil (s t r ay1 p))
("striped" nil (s t r ay1 p t))
("striper" nil (s t r ay1 p er0))
("stripers" nil (s t r ay1 p er0 z))
("stripes" nil (s t r ay1 p s))
("striplin" nil (s t r ih1 p l ih0 n))
("stripling" nil (s t r ih1 p l ih0 ng))
("stripped" nil (s t r ih1 p t))
("stripper" nil (s t r ih1 p er0))
("strippers" nil (s t r ih1 p er0 z))
("stripping" nil (s t r ih1 p ih0 ng))
("strips" nil (s t r ih1 p s))
("striptease" nil (s t r ih1 p t iy1 z))
("strite" nil (s t r ay1 t))
("strittmatter" nil (s t r ih1 t m ax t er0))
("strive" nil (s t r ay1 v))
("striven" nil (s t r ih1 v ax n))
("strives" nil (s t r ay1 v z))
("striving" nil (s t r ay1 v ih0 ng))
("strivings" nil (s t r ay1 v ih0 ng z))
("strnad" nil (s t r n ae1 d))
("strobe" nil (s t r ow1 b))
("strobel" nil (s t r ow1 b ax l))
("strober" nil (s t r ow1 b er0))
("strobl" nil (s t r aa1 b ax l))
("stroble" nil (s t r ow1 b ax l))
("strobridge" nil (s t r aa1 b r ih0 jh))
("strock" nil (s t r aa1 k))
("strode" nil (s t r ow1 d))
("stroder" nil (s t r ow1 d er0))
("stroebel" nil (s t r ow1 b ax l))
("stroessner" nil (s t r ow1 s n er0))
("stroganoff" nil (s t r ow1 g ax n ao1 f))
("stroh" nil (s t r ow1))
("strohecker" nil (s t r ow1 ih0 k er0))
("strohl" nil (s t r ow1 l))
("strohm" nil (s t r ow1 m))
("strohmaier" nil (s t r ow1 m ay0 er0))
("strohman" nil (s t r ow1 m ax n))
("strohmeier" nil (s t r ow1 m ay0 er0))
("strohmeyer" nil (s t r ow1 m ay0 er0))
("stroik" nil (s t r oy1 k))
("strojny" nil (s t r ow1 y n iy0))
("stroke" nil (s t r ow1 k))
("stroked" nil (s t r ow1 k t))
("strokes" nil (s t r ow1 k s))
("stroking" nil (s t r ow1 k ih0 ng))
("strole" nil (s t r ow1 l))
("stroll" nil (s t r ow1 l))
("strolled" nil (s t r ow1 l d))
("stroller" nil (s t r ow1 l er0))
("strollers" nil (s t r ow1 l er0 z))
("strolling" nil (s t r ow1 l ih0 ng))
("strollo" nil (s t r aa1 l ow0))
("strolls" nil (s t r ow1 l z))
("strom" nil (s t r aa1 m))
("stroma" nil (s t r ow1 m ax))
("stromain" nil (s t r aa1 m ay0 n))
("stroman" nil (s t r ow1 m ax n))
("strombeck" nil (s t r aa1 m b eh1 k))
("stromberg" nil (s t r aa1 m b er0 g))
("strome" nil (s t r ow1 m))
("stromer" nil (s t r ow1 m er0))
("stromgren" nil (s t r aa1 m g r eh0 n))
("stromme" nil (s t r aa1 m))
("strommen" nil (s t r aa1 m ax n))
("stromquist" nil (s t r aa1 m k w ih1 s t))
("stronach" nil (s t r aa1 n ax k))
("strong" nil (s t r ao1 ng))
("stronger" nil (s t r ao1 ng er0))
("strongest" nil (s t r ao1 ng g ax s t))
("stronghold" nil (s t r ao1 ng hh ow1 l d))
("strongholds" nil (s t r ao1 ng hh ow1 l d z))
("strongly" nil (s t r ao1 ng l iy0))
("strongman" nil (s t r ao1 ng m ae1 n))
("strontium" nil (s t r aa1 n t iy0 ax m))
("stroock" nil (s t r uh1 k))
("stroop" nil (s t r uw1 p))
("stroope" nil (s t r uw1 p))
("stroot" nil (s t r uw1 t))
("strope" nil (s t r ow1 p))
("stroschein" nil (s t r ao1 sh ay0 n))
("strosnider" nil (s t r aa1 s n ay0 d er0))
("strother" nil (s t r aa1 dh er0))
("strothers" nil (s t r ah1 dh er0 z))
("strothman" nil (s t r aa1 th m ax n))
("stroud" nil (s t r aw1 d))
("strough" nil (s t r aw1))
("stroup" nil (s t r uw1 p))
("stroupe" nil (s t r uw1 p))
("strouse" nil (s t r aw1 s))
("strout" nil (s t r aw1 t))
("strouth" nil (s t r aw1 th))
("strove" nil (s t r ow1 v))
("strow" nil (s t r ow1))
("strozier" nil (s t r ow1 z iy0 er0))
("strub" nil (s t r ah1 b))
("strubbe" nil (s t r ah1 b))
("strube" nil (s t r uw1 b))
("strubel" nil (s t r uw1 b ax l))
("struble" nil (s t r uw1 b ax l))
("struchen" nil (s t r ah1 k ax n))
("struck" nil (s t r ah1 k))
("struckman" nil (s t r ah1 k m ax n))
("structural" nil (s t r ah1 k ch er0 ax l))
("structurally" nil (s t r ah1 k ch er0 ax l iy0))
("structure" nil (s t r ah1 k ch er0))
("structured" nil (s t r ah1 k ch er0 d))
("structures" nil (s t r ah1 k ch er0 z))
("structuring" nil (s t r ah1 k ch er0 ih0 ng))
("struebing" nil (s t r uh1 b ih0 ng))
("struggle" nil (s t r ah1 g ax l))
("struggled" nil (s t r ah1 g ax l d))
("struggles" nil (s t r ah1 g ax l z))
("struggling" nil (s t r ah1 g ax l ih0 ng))
("strum" nil (s t r ah1 m))
("strung" nil (s t r ah1 ng))
("strunk" nil (s t r ah1 ng k))
("strupp" nil (s t r ah1 p))
("struss" nil (s t r ah1 s))
("strut" nil (s t r ah1 t))
("struthers" nil (s t r ah1 dh er0 z))
("struts" nil (s t r ah1 t s))
("strutting" nil (s t r ah1 t ih0 ng))
("strutz" nil (s t r ah1 t s))
("struve" nil (s t r uw1 v))
("strycharz" nil (s t r ih1 hh er0 z))
("strychnine" nil (s t r ih1 k n ay1 n))
("stryker" nil (s t r ay1 k er0))
("strzelecki" nil (s t r eh1 z ih0 l eh1 t s k iy0))
("stu" nil (s t uw1))
("stuard" nil (s t uw1 er0 d))
("stuart" nil (s t uw1 er0 t))
("stuarts" nil (s t uw1 er0 t s))
("stub" nil (s t ah1 b))
("stubbe" nil (s t ah1 b))
("stubbed" nil (s t ah1 b d))
("stubbins" nil (s t ah1 b ih0 n z))
("stubble" nil (s t ah1 b ax l))
("stubblefield" nil (s t ah1 b ax l f iy1 l d))
("stubborn" nil (s t ah1 b er0 n))
("stubbornly" nil (s t ah1 b er0 n l iy0))
("stubbornness" nil (s t ah1 b er0 n n ax s))
("stubbs" nil (s t ah1 b z))
("stubby" nil (s t ah1 b iy0))
("stuber" nil (s t uw1 b er0))
("stubs" nil (s t ah1 b z))
("stucco" nil (s t ah1 k ow0))
("stuchell" nil (s t ah1 k ax l))
("stuck" nil (s t ah1 k))
("stucke" nil (s t ah1 k))
("stucker" nil (s t ah1 k er0))
("stuckert" nil (s t ah1 k er0 t))
("stuckey" nil (s t ah1 k iy0))
("stucki" nil (s t ah1 k iy0))
("stuckman" nil (s t ah1 k m ax n))
("stucky" nil (s t ah1 k iy0))
("stud" nil (s t ah1 d))
("studdard" nil (s t ah1 d er0 d))
("studded" nil (s t ah1 d ih0 d))
("studds" nil (s t ah1 d z))
("stude" nil (s t uw1 d))
("studebaker" nil (s t uw1 d ax b ey1 k er0))
("student" nil (s t uw1 d ax n t))
("students" nil (s t uw1 d ax n t s))
("studer" nil (s t uw1 d er0))
("studied" nil (s t ah1 d iy0 d))
("studies" nil (s t ah1 d iy0 z))
("studio" nil (s t uw1 d iy0 ow1))
("studios" nil (s t uw1 d iy0 ow1 z))
("studious" nil (s t uw1 d iy0 ax s))
("studiously" nil (s t uw1 d iy0 ax s l iy0))
("studley" nil (s t ah1 d l iy0))
("studnicka" nil (s t ax d n ih1 s k ax))
("studs" nil (s t ah1 d z))
("studstill" nil (s t ah1 d s t ih1 l))
("studt" nil (s t ah1 d t))
("study" nil (s t ah1 d iy0))
("studying" nil (s t ah1 d iy0 ih0 ng))
("studzinski" nil (s t ax jh ih1 n s k iy0))
("stuebe" nil (s t uw1 b))
("stueber" nil (s t uh1 b er0))
("stueck" nil (s t uw1 k))
("stueve" nil (s t uw1 v))
("stuewe" nil (s t uw1))
("stuff" nil (s t ah1 f))
("stuffed" nil (s t ah1 f t))
("stuffing" nil (s t ah1 f ih0 ng))
("stufflebeam" nil (s t ah1 f ax l b iy1 m))
("stufflebean" nil (s t ah1 f ax l b iy1 n))
("stuffs" nil (s t ah1 f s))
("stuffy" nil (s t ah1 f iy0))
("stuhler" nil (s t uw1 l er0))
("stuhr" nil (s t er1))
("stuka" nil (s t uw1 k ax))
("stukel" nil (s t uw1 k ax l))
("stukes" nil (s t uw1 k s))
("stukey" nil (s t ah1 k iy0))
("stull" nil (s t ah1 l))
("stuller" nil (s t ah1 l er0))
("stultifying" nil (s t ah1 l t ax f ay1 ih0 ng))
("stults" nil (s t ah1 l t s))
("stultz" nil (s t ah1 l t s))
("stum" nil (s t ah1 m))
("stumbaugh" nil (s t ah1 m b ao1))
("stumble" nil (s t ah1 m b ax l))
("stumbled" nil (s t ah1 m b ax l d))
("stumbles" nil (s t ah1 m b ax l z))
("stumbling" nil (s t ah1 m b ax l ih0 ng))
("stumbo" nil (s t ah1 m b ow0))
("stumm" nil (s t ah1 m))
("stump" nil (s t ah1 m p))
("stumpage" nil (s t ah1 m p ih0 jh))
("stumpe" nil (s t ah1 m p))
("stumped" nil (s t ah1 m p t))
("stumpf" nil (s t ah1 m p f))
("stumpff" nil (s t ah1 m p f))
("stumph" nil (s t ah1 m f))
("stumping" nil (s t ah1 m p ih0 ng))
("stumpo" nil (s t ah1 m p ow0))
("stumpp" nil (s t ah1 m p))
("stumps" nil (s t ah1 m p s))
("stun" nil (s t ah1 n))
("stung" nil (s t ah1 ng))
("stunk" nil (s t ah1 ng k))
("stunned" nil (s t ah1 n d))
("stunning" nil (s t ah1 n ih0 ng))
("stunningly" nil (s t ah1 n ih0 ng l iy0))
("stunt" nil (s t ah1 n t))
("stunted" nil (s t ah1 n t ih0 d))
("stunts" nil (s t ah1 n t s))
("stuntz" nil (s t ah1 n t s))
("stupa" nil (s t uw1 p ax))
("stupak" nil (s t uw1 p ax k))
("stupar" nil (s t uw1 p er0))
("stupay" nil (s t uw0 p ey1))
("stupendous" nil (s t uw0 p eh1 n d ax s))
("stupid" nil (s t uw1 p ax d))
("stupidest" nil (s t uw1 p ih0 d ax s t))
("stupidity" nil (s t uw0 p ih1 d ih0 t iy0))
("stupka" nil (s t ah1 p k ax))
("stupor" nil (s t uw1 p er0))
("stupp" nil (s t ah1 p))
("stupski" nil (s t ah1 p s k iy0))
("sturbridge" nil (s t er1 b r ih1 jh))
("sturc" nil (s t er1 k))
("sturdevant" nil (s t er1 d ih0 v ax n t))
("sturdier" nil (s t er1 d iy0 er0))
("sturdiness" nil (s t er1 d iy0 n ax s))
("sturdivant" nil (s t er1 d ih0 v ax n t))
("sturdy" nil (s t er1 d iy0))
("sturgell" nil (s t er1 g ax l))
("sturgeon" nil (s t er1 jh ax n))
("sturgeons" nil (s t er1 jh ax n z))
("sturges" nil (s t er1 jh ih0 z))
("sturgess" nil (s t er1 g ih0 s))
("sturgill" nil (s t er1 g ax l))
("sturgis" nil (s t er1 jh ih0 s))
("sturkie" nil (s t er1 k iy0))
("sturm" nil (s t er1 m))
("sturman" nil (s t er1 m ax n))
("sturmans" nil (s t er1 m ax n z))
("sturmer" nil (s t er1 m er0))
("sturn" nil (s t er1 n))
("sturrock" nil (s t ao1 r ax k))
("sturtevant" nil (s t er1 t ax v ax n t))
("sturtz" nil (s t er1 t s))
("sturza" nil (s t er1 z ax))
("stutes" nil (s t uw1 t s))
("stutesman" nil (s t uw1 t s m ax n))
("stuteville" nil (s t uw1 t v ih1 l))
("stutheit" nil (s t ah1 th ay0 t))
("stutler" nil (s t ah1 t l er0))
("stutsman" nil (s t ah1 t s m ax n))
("stutter" nil (s t ah1 t er0))
("stuttering" nil (s t ah1 t er0 ih0 ng))
("stutters" nil (s t ah1 t er0 z))
("stuttgart" nil (s t ah1 t g er0 t))
("stutts" nil (s t ah1 t s))
("stutz" nil (s t ah1 t s))
("stutzman" nil (s t ah1 t s m ax n))
("stuve" nil (s t uw1 v))
("stuver" nil (s t uw1 v er0))
("stuyvesant" nil (s t ay1 v ax s ax n t))
("sty" nil (s t ay1))
("styer" nil (s t ay1 er0))
("styers" nil (s t ay1 er0 z))
("style" nil (s t ay1 l))
("styled" nil (s t ay1 l d))
("styles" nil (s t ay1 l z))
("styling" nil (s t ay1 l ih0 ng))
("stylish" nil (s t ay1 l ih0 sh))
("stylishly" nil (s t ay1 l ih0 sh l iy0))
("stylist" nil (s t ay1 l ih0 s t))
("stylistic" nil (s t ay0 l ih1 s t ih0 k))
("stylistically" nil (s t ay0 l ih1 s t ih0 k l iy0))
("stylists" nil (s t ay1 l ih0 s t s))
("stylites" nil (s t ih0 l ay1 t s))
("stylized" nil (s t ay1 l ay1 z d))
("stylus" nil (s t ay1 l ax s))
("stymie" nil (s t ay1 m iy0))
("stymied" nil (s t ay1 m iy0 d))
("stymies" nil (s t ay1 m iy0 z))
("styrene" nil (s t ay1 r iy1 n))
("styrofoam" nil (s t ay1 r ax f ow1 m))
("styron" nil (s t y r ao1 n))
("stys" nil (s t ih1 s))
("styx" nil (s t ih1 k s))
("su" nil (s uw1))
("suard" nil (s uw1 aa0 r d))
("suarez" nil (s w aa0 r eh1 z))
("suasion" nil (s w ey1 zh ax n))
("suata" nil (s uw0 aa1 t ax))
("suave" nil (s w aa1 v))
("suazo" nil (s w aa1 z ow0))
("sub" nil (s ah1 b))
("suba" nil (s uw1 b ax))
("subacute" nil (s ah1 b ax k y uw1 t))
("subandrio" nil (s ax b ae1 n d r iy0 ow0))
("subaru" nil (s uw1 b er0 uw0))
("subassemblies" nil (s ah1 b ax s eh1 m b l iy0 z))
("subatomic" nil (s ah1 b ax t aa1 m ih0 k))
("subbed" nil (s ah1 b d))
("subbing" nil (s ah1 b ih0 ng))
("subchapter" nil (s ah1 b ch ae1 p t er0))
("subcomittee" nil (s ah1 b k ax m ih1 t iy0))
("subcommander" nil (s ah1 b k ax m ae1 n d er0))
("subcommanders" nil (s ah1 b k ax m ae1 n d er0 z))
("subcommittee" nil (s ah1 b k ax m ih1 t iy0))
("subcommittees" nil (s ah1 b k ax m ih1 t iy0 z))
("subcompact" nil (s ax b k aa1 m p ae0 k t))
("subcompacts" nil (s ax b k aa1 m p ae0 k t s))
("subconscious" nil (s ax b k aa1 n sh ax s))
("subconsciously" nil (s ax b k aa1 n sh ax s l iy0))
("subcontinent" nil (s ax b k aa1 n t ih0 n ax n t))
("subcontinents" nil (s ax b k aa1 n t ih0 n ax n t s))
("subcontract" nil (s ax b k aa1 n t r ae1 k t))
("subcontracted" nil (s ax b k aa1 n t r ae0 k t ih0 d))
("subcontracting" nil (s ah1 b k ax n t r ae1 k t ih0 ng))
("subcontractor" nil (s ax b k aa1 n t r ae1 k t er0))
("subcontractors" nil (s ax b k aa1 n t r ae0 k t er0 z))
("subcontracts" nil (s ax b k aa1 n t r ae1 k t s))
("subculture" nil (s ah1 b k ah1 l ch er0))
("subcultures" nil (s ah1 b k ah1 l ch er0 z))
("subdivide" nil (s ah1 b d ax v ay1 d))
("subdivided" nil (s ah1 b d ih0 v ay1 d ih0 d))
("subdivision" nil (s ah1 b d ih0 v ih1 zh ax n))
("subdivisions" nil (s ah1 b d ih0 v ih1 zh ax n z))
("subdue" nil (s ax b d uw1))
("subdued" nil (s ax b d uw1 d))
("subduing" nil (s ax b d uw1 ih0 ng))
("suber" nil (s uw1 b er0))
("subfamilies" nil (s ah1 b f ae1 m ax l iy0 z))
("subfamily" nil (s ah1 b f ae1 m ax l iy0))
("subgroup" nil (s ah1 b g r uw1 p))
("subgroups" nil (s ah1 b g r uw1 p s))
("subhlok" nil (s ah1 b l aa1 k))
("subia" nil (s uw0 b iy1 ax))
("subic" nil (s uw1 b ih0 k))
("subject" v (s ax b jh eh1 k t))
("subject" n (s ah1 b jh ih0 k t))
("subjected" nil (s ax b jh eh1 k t ih0 d))
("subjecting" nil (s ax b jh eh1 k t ih0 ng))
("subjective" nil (s ax b jh eh1 k t ih0 v))
("subjects" v (s ax b jh eh1 k t s))
("subjects" n (s ah1 b jh ih0 k t s))
("subkingdom" nil (s ax b k ih1 ng d ax m))
("sublease" nil (s ah1 b l iy1 s))
("subleasing" nil (s ax b l iy1 s ih0 ng))
("sublet" nil (s ah1 b l eh1 t))
("sublett" nil (s uw1 b l ih0 t))
("sublime" nil (s ax b l ay1 m))
("subliminal" nil (s ax b l ih1 m ih0 n ax l))
("submachine" nil (s ah1 b m ax sh iy1 n))
("submarine" nil (s ah1 b m er0 iy1 n))
("submarines" nil (s ah1 b m er0 iy1 n z))
("submerge" nil (s ax b m er1 jh))
("submerged" nil (s ax b m er1 jh d))
("submergence" nil (s ax b m er1 jh ax n s))
("submersed" nil (s ax b m er1 s t))
("submersible" nil (s ax b m er1 s ih0 b ax l))
("submersion" nil (s ax b m er1 zh ax n))
("subminimum" nil (s ax b m ih1 n ih0 m ax m))
("submission" nil (s ax b m ih1 sh ax n))
("submissions" nil (s ax b m ih1 sh ax n z))
("submissive" nil (s ax b m ih1 s ih0 v))
("submit" nil (s ax b m ih1 t))
("submits" nil (s ah1 b m ih1 t s))
("submitted" nil (s ax b m ih1 t ax d))
("submitting" nil (s ax b m ih1 t ih0 ng))
("subnotebook" nil (s ah1 b n ow1 t b uh1 k))
("subordinate" nil (s ax b ao1 r d ax n ey1 t))
("subordinated" nil (s ax b ao1 r d ax n ey1 t ih0 d))
("subordinates" nil (s ax b ao1 r d ax n ey1 t s))
("subordinating" nil (s ax b ao1 r d ax n ey1 t ih0 ng))
("subordination" nil (s ax b ao1 r d ax n ey1 sh ax n))
("subotnick" nil (s ax b aa1 t n ih0 k))
("subpar" nil (s ax b p aa1 r))
("subplot" nil (s ah1 b p l aa0 t))
("subplots" nil (s ah1 b p l aa0 t s))
("subpoena" nil (s ax p iy1 n ax))
("subpoenaed" nil (s ax p iy1 n ax d))
("subpoenaing" nil (s ax p iy1 n ax ih0 ng))
("subpoenas" nil (s ax p iy1 n ax z))
("subprincipals" nil (s ax b p r ih1 n s ax p ax l z))
("subramanian" nil (s uw1 b r ax m aa1 n iy0 ax n))
("subroto" nil (s uw0 b r ow1 t ow0))
("subs" nil (s ah1 b z))
("subsaharan" nil (s ah1 b s ax hh eh1 r ax n))
("subscribe" nil (s ax b s k r ay1 b))
("subscribed" nil (s ax b s k r ay1 b d))
("subscriber" nil (s ax b s k r ay1 b er0))
("subscribers" nil (s ax b s k r ay1 b er0 z))
("subscribes" nil (s ax b s k r ay1 b z))
("subscribing" nil (s ax b s k r ay1 b ih0 ng))
("subscription" nil (s ax b s k r ih1 p sh ax n))
("subscriptions" nil (s ax b s k r ih1 p sh ax n z))
("subsequent" nil (s ah1 b s ax k w ax n t))
("subsequently" nil (s ah1 b s ax k w ax n t l iy0))
("subservience" nil (s ax b s er1 v iy0 ax n s))
("subservient" nil (s ax b s er1 v iy0 ax n t))
("subset" nil (s ah1 b s eh1 t))
("subside" nil (s ax b s ay1 d))
("subsided" nil (s ax b s ay1 d ih0 d))
("subsidence" nil (s ax b s ay1 d ax n s))
("subsides" nil (s ax b s ay1 d z))
("subsidiaries" nil (s ax b s ih1 d iy0 eh1 r iy0 z))
("subsidiary" nil (s ax b s ih1 d iy0 eh1 r iy0))
("subsidies" nil (s ah1 b s ax d iy0 z))
("subsiding" nil (s ax b s ay1 d ih0 ng))
("subsidization" nil (s ah1 b s ih0 d ih0 z ey1 sh ax n))
("subsidize" nil (s ah1 b s ih0 d ay1 z))
("subsidized" nil (s ah1 b s ih0 d ay1 z d))
("subsidizes" nil (s ah1 b s ih0 d ay1 z ih0 z))
("subsidizing" nil (s ah1 b s ih0 d ay1 z ih0 ng))
("subsidy" nil (s ah1 b s ih0 d iy0))
("subsist" nil (s ax b s ih1 s t))
("subsistence" nil (s ax b s ih1 s t ax n s))
("subsoil" nil (s ah1 b s oy1 l))
("substance" nil (s ah1 b s t ax n s))
("substances" nil (s ah1 b s t ax n s ax z))
("substandard" nil (s ax b s t ae1 n d er0 d))
("substantial" nil (s ax b s t ae1 n ch ax l))
("substantially" nil (s ax b s t ae1 n sh ax l iy0))
("substantiate" nil (s ax b s t ae1 n ch iy0 ey1 t))
("substantiated" nil (s ax b s t ae1 n sh iy0 ey1 t ih0 d))
("substantiates" nil (s ax b s t ae1 n ch iy0 ey1 t s))
("substantiation" nil (s ax b s t ae1 n ch iy0 ey1 sh ax n))
("substantive" nil (s ah1 b s t ax n t ih0 v))
("substantively" nil (s ah1 b s t ax n t ih0 v l iy0))
("substation" nil (s ah1 b s t ey1 sh ax n))
("substitute" nil (s ah1 b s t ax t uw1 t))
("substituted" nil (s ah1 b s t ax t uw1 t ax d))
("substitutes" nil (s ah1 b s t ax t uw1 t s))
("substituting" nil (s ah1 b s t ih0 t uw1 t ih0 ng))
("substitution" nil (s ah1 b s t ih0 t uw1 sh ax n))
("substitutions" nil (s ah1 b s t ih0 t y uw1 sh ax n z))
("substrate" nil (s ah1 b s t r ey1 t))
("substrates" nil (s ah1 b s t r ey1 t s))
("subsurface" nil (s ah1 b s er1 f ax s))
("subsystems" nil (s ah1 b s ih1 s t ax m z))
("subterfuge" nil (s ah1 b t er0 f y uw1 jh))
("subterranean" nil (s ax b t er0 ey1 n iy0 ax n))
("subtext" nil (s ah1 b t eh1 k s t))
("subtitle" nil (s ah1 b t ay1 t ax l))
("subtitled" nil (s ah1 b t ay1 t ax l d))
("subtitles" nil (s ah1 b t ay1 t ax l z))
("subtle" nil (s ah1 t ax l))
("subtler" nil (s ah1 t ax l er0))
("subtleties" nil (s ah1 t ax l t iy0 z))
("subtlety" nil (s ah1 t ax l t iy0))
("subtly" nil (s ah1 t ax l iy0))
("subtract" nil (s ax b t r ae1 k t))
("subtracted" nil (s ax b t r ae1 k t ih0 d))
("subtracting" nil (s ax b t r ae1 k t ih0 ng))
("subtraction" nil (s ax b t r ae1 k sh ax n))
("suburb" nil (s ah1 b er0 b))
("suburban" nil (s ax b er1 b ax n))
("suburbanite" nil (s ax b er1 b ax n ay1 t))
("suburbanites" nil (s ax b er1 b ax n ay1 t s))
("suburbanization" nil (s ax b er1 b ax n ih0 z ey1 sh ax n))
("suburbans" nil (s ax b er1 b ax n z))
("suburbia" nil (s ax b er1 b iy0 ax))
("suburbs" nil (s ah1 b er0 b z))
("subversion" nil (s ax b v er1 zh ax n))
("subversive" nil (s ax b v er1 s ih0 v))
("subversives" nil (s ax b v er1 s ih0 v z))
("subvert" nil (s ax b v er1 t))
("subverted" nil (s ax b v er1 t ih0 d))
("subverting" nil (s ax b v er1 t ih0 ng))
("subverts" nil (s ax b v er1 t s))
("subvolcanic" nil (s ah1 b v aa0 l k ae1 n ih0 k))
("subway" nil (s ah1 b w ey1))
("subways" nil (s ah1 b w ey1 z))
("subzero" nil (s ah1 b z ih1 r ow0))
("succeed" nil (s ax k s iy1 d))
("succeeded" nil (s ax k s iy1 d ax d))
("succeeding" nil (s ax k s iy1 d ih0 ng))
("succeeds" nil (s ax k s iy1 d z))
("success" nil (s ax k s eh1 s))
("successes" nil (s ax k s eh1 s ax z))
("successful" nil (s ax k s eh1 s f ax l))
("successfully" nil (s ax k s eh1 s f ax l iy0))
("succession" nil (s ax k s eh1 sh ax n))
("successive" nil (s ax k s eh1 s ih0 v))
("successively" nil (s ax k s eh1 s ih0 v l iy0))
("successor" nil (s ax k s eh1 s er0))
("successors" nil (s ax k s eh1 s er0 z))
("succinct" nil (s ax k s ih1 ng k t))
("succinctly" nil (s ax k s ih1 ng k t l iy0))
("succor" nil (s ah1 k er0))
("succulent" nil (s ah1 k y ax l ih0 n t))
("succulents" nil (s ah1 k y ax l ax n t s))
("succumb" nil (s ax k ah1 m))
("succumbed" nil (s ax k ah1 m d))
("succumbing" nil (s ax k ah1 m ih0 ng))
("succumbs" nil (s ax k ah1 m z))
("such" nil (s ah1 ch))
("suchan" nil (s ah1 ch ax n))
("suchanek" nil (s ah1 k ax n ih0 k))
("suchard" nil (s uw0 sh aa1 r d))
("sucharski" nil (s ax k aa1 r s k iy0))
("suchecki" nil (s ax k eh1 k iy0))
("sucher" nil (s ah1 ch er0))
("suchinda" nil (s uw1 ch ih1 n d ax))
("suchocki" nil (s ax k aa1 k iy0))
("suchomel" nil (s ah1 k ow0 m eh1 l))
("suchy" nil (s ah1 ch iy0))
("suck" nil (s ah1 k))
("sucked" nil (s ah1 k t))
("sucker" nil (s ah1 k er0))
("suckered" nil (s ah1 k er0 d))
("suckers" nil (s ah1 k er0 z))
("sucking" nil (s ah1 k ih0 ng))
("suckling" nil (s ah1 k l ih0 ng))
("suckow" nil (s ah1 k aw0))
("suckrow" nil (s ah1 k r ow0))
("sucks" nil (s ah1 k s))
("sucralose" nil (s uw1 k r ax l ow1 s))
("sucre" nil (s uw1 k er0))
("sucrose" nil (s uw1 k r ow0 s))
("suction" nil (s ah1 k sh ax n))
("suda" nil (s uw1 d ax))
("sudafed" nil (s uw1 d ax f eh1 d))
("sudan" nil (s uw0 d ae1 n))
("sudanese" nil (s uw1 d ax n iy1 z))
("sudano" nil (s uw0 d aa1 n ow0))
("sudbeck" nil (s ah1 d b eh1 k))
("sudberry" nil (s ah1 d b eh1 r iy0))
("sudbury" nil (s ah1 d b eh1 r iy0))
("sudd" nil (s ah1 d))
("suddam" nil (s ah1 d aa1 m))
("suddarth" nil (s ah1 d aa0 r th))
("suddath" nil (s ah1 d ax th))
("sudden" nil (s ah1 d ax n))
("suddenly" nil (s ah1 d ax n l iy0))
("suddenness" nil (s ah1 d ax n n ax s))
("sudderth" nil (s ah1 d er0 th))
("suddeth" nil (s ah1 d ih0 th))
("suddreth" nil (s ah1 d r ih0 th))
("sudduth" nil (s ah1 d ax th))
("suder" nil (s uw1 d er0))
("suderman" nil (s uw1 d er0 m ax n))
("sudler" nil (s uw1 d ax l er0))
("sudol" nil (s uw1 d ax l))
("suds" nil (s ah1 d z))
("sue" nil (s uw1))
("sued" nil (s uw1 d))
("suede" nil (s w ey1 d))
("suedes" nil (s w ey1 d z))
("suen" nil (s uw1 n))
("sues" nil (s uw1 z))
("suess" nil (s w ih1 s))
("suey" nil (s uw1 iy0))
("suez" nil (s uw1 eh0 z))
("suffer" nil (s ah1 f er0))
("suffered" nil (s ah1 f er0 d))
("sufferer" nil (s ah1 f er0 er0))
("sufferers" nil (s ah1 f er0 er0 z))
("suffering" nil (s ah1 f er0 ih0 ng))
("sufferings" nil (s ah1 f er0 ih0 ng z))
("suffern" nil (s ah1 f er0 n))
("suffers" nil (s ah1 f er0 z))
("suffice" nil (s ax f ay1 s))
("sufficed" nil (s ax f ay1 s t))
("sufficent" nil (s ax f ih1 sh ax n t))
("suffices" nil (s ax f ay1 s ih0 z))
("sufficiency" nil (s ax f ih1 sh ax n s iy0))
("sufficient" nil (s ax f ih1 sh ax n t))
("sufficiently" nil (s ax f ih1 sh ax n t l iy0))
("suffield" nil (s ah1 f iy0 l d))
("suffix" nil (s ah1 f ih0 k s))
("suffocate" nil (s ah1 f ax k ey1 t))
("suffocating" nil (s ah1 f ax k ey1 t ih0 ng))
("suffocation" nil (s ah1 f ax k ey1 sh ax n))
("suffolk" nil (s ah1 f ax k))
("suffrage" nil (s ah1 f r ih0 jh))
("suffragettes" nil (s ah1 f r ax jh eh1 t s))
("suffused" nil (s ax f y uw1 z d))
("sugar" nil (sh uh1 g er0))
("sugared" nil (sh uh1 g er0 d))
("sugarman" nil (sh uh1 g er0 m ax n))
("sugars" nil (sh uh1 g er0 z))
("sugary" nil (sh uh1 g er0 iy0))
("sugden" nil (s ah1 g d ax n))
("sugerman" nil (s uw1 g er0 m ax n))
("sugg" nil (s ah1 g))
("suggest" nil (s ax g jh eh1 s t))
("suggested" nil (s ax g jh eh1 s t ax d))
("suggesting" nil (s ax g jh eh1 s t ih0 ng))
("suggestion" nil (s ax g jh eh1 s ch ax n))
("suggestions" nil (s ax g jh eh1 s ch ax n z))
("suggestive" nil (s ax g jh eh1 s t ih0 v))
("suggestiveness" nil (s ax g jh eh1 s t ih0 v n ax s))
("suggests" nil (s ax g jh eh1 s t s))
("suggs" nil (s ah1 g z))
("sugimoto" nil (s uw0 g iy0 m ow1 t ow0))
("sugiyama" nil (s uw0 g iy0 y aa1 m ax))
("sugrue" nil (s ah1 g r uw0))
("suh" nil (s ah1))
("suharto" nil (s uw0 hh aa1 r t ow0))
("suhler" nil (s uw1 l er0))
("suhm" nil (s uw1 m))
("suhr" nil (s uh1 r))
("suhre" nil (s uh1 r))
("suhud" nil (s uw0 hh uh1 d))
("sui" nil (s uw1 iy0))
("suicidal" nil (s uw1 ax s ay1 d ax l))
("suicide" nil (s uw1 ax s ay1 d))
("suicides" nil (s uw1 ax s ay1 d z))
("suing" nil (s uw1 ih0 ng))
("suire" nil (s uh1 r))
("suisse" nil (s w ih1 s))
("suit" nil (s uw1 t))
("suitability" nil (s uw1 t ax b ih1 l ih0 t iy0))
("suitable" nil (s uw1 t ax b ax l))
("suitably" nil (s uw1 t ax b l iy0))
("suitcase" nil (s uw1 t k ey1 s))
("suitcases" nil (s uw1 t k ey1 s ih0 z))
("suite" nil (s w iy1 t))
("suited" nil (s uw1 t ax d))
("suiter" nil (s uw1 t er0))
("suites" nil (s w iy1 t s))
("suitor" nil (s uw1 t er0))
("suitors" nil (s uw1 t er0 z))
("suits" nil (s uw1 t s))
("suitt" nil (s uw1 t))
("suk" nil (s ah1 k))
("sukarno" nil (s uw0 k aa1 r n ow0))
("sukey" nil (s uw1 k iy0))
("sukharev" nil (s ah1 k hh er0 ax v))
("suki" nil (s uw1 k iy0))
("sukiyaki" nil (s uw0 k iy0 aa1 k iy0))
("sukthankar" nil (s uw0 k t aa1 ng k aa1 r))
("sukup" nil (s uw1 k ax p))
("sul" nil (s ah1 l))
("sulak" nil (s uw1 l ax k))
("sulawesi" nil (s uw1 l ax w eh1 s iy0))
("sulcer" nil (s ah1 l s er0))
("sulek" nil (s uw1 l ih0 k))
("suleski" nil (s y uw0 l eh1 s k iy0))
("sulewski" nil (s y uw0 l eh1 f s k iy0))
("suleyman" nil (s uw1 l iy0 m ax n))
("sulfa" nil (s ah1 l f ax))
("sulfate" nil (s ah1 l f ey1 t))
("sulfide" nil (s ah1 l f ay1 d))
("sulfite" nil (s ah1 l f ay1 t))
("sulfites" nil (s ah1 l f ay1 t s))
("sulfur" nil (s ah1 l f er0))
("sulfuric" nil (s ax l f y uh1 r ih0 k))
("sulgrave" nil (s ah1 l g r ey1 v))
("sulik" nil (s uw1 l ih0 k))
("sulked" nil (s ah1 l k t))
("sulking" nil (s ah1 l k ih0 ng))
("sulkowski" nil (s ax l k ao1 f s k iy0))
("sullen" nil (s ah1 l ax n))
("sullenberger" nil (s ah1 l ax n b er0 g er0))
("sullenger" nil (s uw1 l ih0 n jh er0))
("sullens" nil (s ah1 l ax n z))
("sullie" nil (s ah1 l iy0))
("sullied" nil (s ah1 l iy0 d))
("sullinger" nil (s ah1 l ih0 ng er0))
("sullins" nil (s ah1 l ih0 n z))
("sullivan" nil (s ah1 l ax v ax n))
("sullivans" nil (s ah1 l ih0 v ax n z))
("sullivant" nil (s ah1 l ih0 v ax n t))
("sullo" nil (s uw1 l ow0))
("sully" nil (s ah1 l iy0))
("sulpetro" nil (s uw0 l p eh1 t r ow0))
("sulphate" nil (s ah1 l f ey1 t))
("sulphates" nil (s ah1 l f ey1 t s))
("sulphur" nil (s ah1 l f er0))
("sulser" nil (s ah1 l s er0))
("sult" nil (s ah1 l t))
("sultan" nil (s ah1 l t ax n))
("sultanate" nil (s ah1 l t ax n ax t))
("sultans" nil (s ah1 l t ax n z))
("sulton" nil (s ah1 l t ax n))
("sultry" nil (s ah1 l t r iy0))
("sulya" nil (s uw1 l y ax))
("sulzbach" nil (s ah1 l z b aa0 k))
("sulzberger" nil (s ah1 l t s b er0 g er0))
("sulzer" nil (s ah1 l z er0))
("sum" nil (s ah1 m))
("sumac" nil (s uw1 m ae0 k))
("suman" nil (s uw1 m ax n))
("sumarlin" nil (s uw1 m aa1 r l ih0 n))
("sumatra" nil (s uw1 m ae1 t r ax))
("sumerlin" nil (s ah1 m er0 l ih0 n))
("sumgait" nil (s ax m g ey1 t))
("sumida" nil (s uw0 m iy1 d ax))
("suminski" nil (s ax m ih1 n s k iy0))
("sumita" nil (s uw0 m iy1 t ax))
("sumitomo" nil (s uw1 m ih0 t ow1 m ow0))
("sumler" nil (s ah1 m l er0))
("sumlin" nil (s ah1 m l ih0 n))
("summa" nil (s uw1 m ax))
("summagraphics" nil (s ah1 m ax g r ae1 f ih0 k s))
("summar" nil (s ah1 m er0))
("summaries" nil (s ah1 m er0 iy0 z))
("summarily" nil (s ax m eh1 r ih0 l iy0))
("summarize" nil (s ah1 m er0 ay1 z))
("summarized" nil (s ah1 m er0 ay1 z d))
("summarizes" nil (s ah1 m er0 ay1 z ih0 z))
("summarizing" nil (s ah1 m er0 ay1 z ih0 ng))
("summary" nil (s ah1 m er0 iy0))
("summation" nil (s ax m ey1 sh ax n))
("summations" nil (s ax m ey1 sh ax n z))
("summcorp" nil (s ah1 m k ao0 r p))
("summed" nil (s ah1 m d))
("summer" nil (s ah1 m er0))
("summerall" nil (s ah1 m er0 ao1 l))
("summerfield" nil (s ah1 m er0 f iy1 l d))
("summerford" nil (s ah1 m er0 f er0 d))
("summerhill" nil (s ah1 m er0 hh ih1 l))
("summerlin" nil (s ah1 m er0 l ih0 n))
("summerour" nil (s ah1 m er0 er0))
("summers" nil (s ah1 m er0 z))
("summerson" nil (s ah1 m er0 s ax n))
("summertime" nil (s ah1 m er0 t ay1 m))
("summerville" nil (s ah1 m er0 v ih1 l))
("summey" nil (s ah1 m iy0))
("summing" nil (s ah1 m ih0 ng))
("summit" nil (s ah1 m ax t))
("summiteers" nil (s ah1 m ih0 t ih1 r z))
("summitry" nil (s ah1 m ih0 t r iy0))
("summits" nil (s ah1 m ih0 t s))
("summitt" nil (s ah1 m ih0 t))
("summon" nil (s ah1 m ax n))
("summoned" nil (s ah1 m ax n d))
("summoning" nil (s ah1 m ax n ih0 ng))
("summons" nil (s ah1 m ax n z))
("summonsed" nil (s ah1 m ax n z d))
("summonses" nil (s ah1 m ax n z ih0 z))
("summum" nil (s ah1 m ax m))
("summy" nil (s ah1 m iy0))
("sumner" nil (s ah1 m n er0))
("sumners" nil (s ah1 m n er0 z))
("sumo" nil (s uw1 m ow0))
("sump" nil (s ah1 m p))
("sumpter" nil (s ah1 m p t er0))
("sumptuous" nil (s ah1 m p ch w ax s))
("sumrall" nil (s ah1 m r ax l))
("sumrell" nil (s uw0 m r ey1 l))
("sumrow" nil (s ah1 m r ow1))
("sums" nil (s ah1 m z))
("sumter" nil (s ah1 m t er0))
("sun" nil (s ah1 n))
("sunamerica" nil (s ah1 n ax m eh1 r ax k ax))
("sunbathing" nil (s ah1 n b ey1 dh ih0 ng))
("sunbeam" nil (s ah1 n b iy1 m))
("sunbelt" nil (s ah1 n b eh1 l t))
("sunbird" nil (s ah1 n b er1 d))
("sunbirds" nil (s ah1 n b er1 d z))
("sunburn" nil (s ah1 n b er1 n))
("sunburned" nil (s ah1 n b er1 n d))
("suncoast" nil (s ah1 n k ow1 s t))
("suncook" nil (s ah1 n k uh1 k))
("suncor" nil (s ah1 n k ao1 r))
("sund" nil (s ah1 n d))
("sundae" nil (s ah1 n d ey0))
("sundahl" nil (s ah1 n d aa1 l))
("sundai" nil (s ax n d ay1))
("sundance" nil (s ah1 n d ae1 n s))
("sundararajan" nil (s uw1 n d aa0 r aa1 r aa0 jh ax n))
("sunday" nil (s ah1 n d ey1))
("sundays" nil (s ah1 n d ey1 z))
("sundberg" nil (s ah1 n d b er0 g))
("sundby" nil (s ah1 n d b iy0))
("sunde" nil (s ah1 n d))
("sundeen" nil (s ah1 n d iy0 n))
("sundell" nil (s ah1 n d ax l))
("sunder" nil (s ah1 n d er0))
("sunderland" nil (s ah1 n d er0 l ax n d))
("sunderlin" nil (s ah1 n d er0 l ih0 n))
("sunderman" nil (s ah1 n d er0 m ax n))
("sundermeyer" nil (s ah1 n d er0 m ay0 er0))
("sundews" nil (s ah1 n d uw1 z))
("sundheim" nil (s ah1 n d hh ay1 m))
("sundial" nil (s ah1 n d ay1 l))
("sundin" nil (s ah1 n d ax n))
("sundlun" nil (s ah1 n d l ax n))
("sundown" nil (s ah1 n d aw1 n))
("sundquist" nil (s ah1 n d k w ih1 s t))
("sundry" nil (s ah1 n d r iy0))
("sundstrand" nil (s ah1 n d s t r ae1 n d))
("sundstrom" nil (s ah1 n d s t r ax m))
("sundt" nil (s ah1 n t))
("sundy" nil (s ah1 n d iy0))
("sunfire" nil (s ah1 n f ay1 r))
("sunfish" nil (s ah1 n f ih1 sh))
("sunflower" nil (s ah1 n f l aw1 er0))
("sunflowers" nil (s ah1 n f l aw1 er0 z))
("sung" nil (s ah1 ng))
("sungard" nil (s ah1 n g aa1 r d))
("sunglass" nil (s ah1 n g l ae1 s))
("sunglasses" nil (s ah1 n g l ae1 s ih0 z))
("sungroup" nil (s ah1 n g r uw1 p))
("sunia" nil (s uw1 n iy0 ax))
("suniga" nil (s uw0 n iy1 g ax))
("sunil" nil (s uw0 n ih1 l))
("sunk" nil (s ah1 ng k))
("sunken" nil (s ah1 ng k ax n))
("sunkist" nil (s ah1 n k ih1 s t))
("sunland" nil (s ah1 n l ae1 n d))
("sunlight" nil (s ah1 n l ay1 t))
("sunlit" nil (s ah1 n l ih1 t))
("sunlite" nil (s ah1 n l ay1 t))
("sunni" nil (s uw1 n iy0))
("sunning" nil (s ah1 n ih0 ng))
("sunnis" nil (s uh1 n iy1 z))
("sunny" nil (s ah1 n iy0))
("sunnyside" nil (s ah1 n iy0 s ay1 d))
("sunnyvale" nil (s ah1 n iy0 v ey1 l))
("sunobe" nil (s uw0 n ow1 b iy0))
("sunoco" nil (s ih0 n ow1 k ow0))
("sunpoint" nil (s ah1 n p oy1 n t))
("sunrise" nil (s ah1 n r ay1 z))
("sunroof" nil (s ah1 n r uw1 f))
("sunroom" nil (s ah1 n r uw1 m))
("suns" nil (s ah1 n z))
("sunscreen" nil (s ax n s k r iy1 n))
("sunseri" nil (s ax n s eh1 r iy0))
("sunset" nil (s ah1 n s eh1 t))
("sunsets" nil (s ah1 n s eh1 t s))
("sunshine" nil (s ah1 n sh ay1 n))
("sunshiny" nil (s ah1 n sh ay1 n iy0))
("sunspots" nil (s ah1 n s p aa1 t s))
("sunstar" nil (s ah1 n s t aa1 r))
("sunstate" nil (s ah1 n s t ey1 t))
("sunstates" nil (s ah1 n s t ey1 t s))
("sunsweet" nil (s ah1 n s w iy1 t))
("suntan" nil (s ah1 n t ae1 n))
("sunter" nil (s ah1 n t er0))
("suntory" nil (s ah1 n t ao1 r iy0))
("suntrust" nil (s ah1 n t r ah1 s t))
("sununu" nil (s ax n uw1 n uw0))
("sunward" nil (s ah1 n w er0 d))
("sunworld" nil (s ah1 n w er1 l d))
("suny" nil (s uw1 n iy1))
("sup" nil (s ah1 p))
("supak" nil (s uw1 p ax k))
("supan" nil (s uw1 p ax n))
("super" nil (s uw1 p er0))
("superabrasives" nil (s uw1 p er0 ax b r ey1 s ih0 v z))
("superamerica" nil (s uw1 p er0 ax m eh1 r ih0 k ax))
("superb" nil (s uh0 p er1 b))
("superbar" nil (s uw1 p er0 b aa1 r))
("superbly" nil (s uw1 p er0 b l iy0))
("superbowl" nil (s uw1 p er0 b ow1 l))
("superbowls" nil (s uw1 p er0 b ow1 l z))
("supercenter" nil (s uw1 p er0 s eh1 n t er0))
("supercenters" nil (s uw1 p er0 s eh1 n t er0 s))
("supercharged" nil (s uw1 p er0 ch aa1 r jh d))
("supercilious" nil (s uw1 p er0 s ih1 l iy0 ax s))
("supercollider" nil (s uw0 p er0 k ax l ay1 d er0))
("supercomputer" nil (s uw1 p er0 k ax m p y uw1 t er0))
("supercomputers" nil (s uw1 p er0 k ax m p y uw1 t er0 z))
("supercomputing" nil (s uw0 p er0 k ax m p y uw1 t ih0 ng))
("superconducting" nil (s uw1 p er0 k ax n d ah1 k t ih0 ng))
("superconductive" nil (s uw0 p er0 k ax n d ah1 k t ih0 v))
("superconductivity" nil (s uw1 p er0 k aa1 n d ah1 k t ih1 v ax t iy0))
("superconductor" nil (s uw1 p er0 k ax n d ah1 k t er0))
("superconductors" nil (s uw1 p er0 k ax n d ah1 k t er0 z))
("supercooled" nil (s uw1 p er0 k uw1 l d))
("supercuts" nil (s uw1 p er0 k ah1 t s))
("superdelegates" nil (s uw0 p er0 d eh1 l ax g ax t s))
("superdome" nil (s uw1 p er0 d ow1 m))
("superdot" nil (s uw1 p er0 d aa1 t))
("superdrug" nil (s uw1 p er0 d r ah1 g))
("superfamily" nil (s uw1 p er0 f ae1 m l iy0))
("superfan" nil (s uw1 p er0 f ae1 n))
("superfast" nil (s uw1 p er0 f ae1 s t))
("superficial" nil (s uw1 p er0 f ih1 sh ax l))
("superficially" nil (s uw1 p er0 f ih1 sh ax l l iy0))
("superfluidity" nil (s uw1 p er0 f l uw1 ih1 d ax t iy0))
("superfluous" nil (s uw1 p er0 f l w ah1 s))
("superfon" nil (s uw1 p er0 f ih0 n))
("superfreighters" nil (s uw1 p er0 f r ey1 t er0 z))
("superfund" nil (s uw1 p er0 f ah1 n d))
("supergiant" nil (s uw1 p er0 jh ay1 ax n t))
("supergiants" nil (s uw1 p er0 jh ay1 ax n t s))
("superheated" nil (s uw1 p er0 hh iy1 t ih0 d))
("superheterodyne" nil (s uw1 p er0 hh eh1 t er0 ax d ay1 n))
("superhighway" nil (s uw1 p er0 hh ay1 w ey1))
("superhuman" nil (s uw1 p er0 hh y uw1 m ax n))
("superimposed" nil (s uw1 p er0 ax m p ow1 z d))
("superintendent" nil (s uw1 p er0 ax n t eh1 n d ax n t))
("superintendents" nil (s uw1 p er0 ax n t eh1 n d ax n t s))
("superior" nil (s uw0 p ih1 r iy0 er0))
("superiority" nil (s uw1 p ih0 r iy0 ao1 r ih0 t iy0))
("superiors" nil (s uw0 p ih1 r iy0 er0 z))
("superlative" nil (s uh0 p er1 l ax t ih0 v))
("superlatives" nil (s uh0 p er1 l ax t ih0 v z))
("supermac" nil (s uw1 p er0 m ae1 k))
("supermajority" nil (s uw1 p er0 m ax jh ao1 r ih0 t iy0))
("superman" nil (s uw1 p er0 m ax n))
("supermarket" nil (s uw1 p er0 m aa1 r k ih0 t))
("supermarkets" nil (s uw1 p er0 m aa1 r k ih0 t s))
("superminicomputer" nil (s uw1 p er0 m ih1 n ih0 k ax m p y uw1 t er0))
("superminicomputers" nil (s uw1 p er0 m ih1 n ih0 k ax m p y uw1 t er0 z))
("supernatural" nil (s uw1 p er0 n ae1 ch er0 ax l))
("supernaturalism" nil (s uw1 p er0 n ae1 ch er0 ax l ih1 z ax m))
("supernova" nil (s uw1 p er0 n ow1 v ax))
("superoxide" nil (s uw1 p er0 aa1 k s ay1 d))
("superpower" nil (s uw1 p er0 p aw1 er0))
("superpowers" nil (s uw1 p er0 p aw1 er0 z))
("superpremium" nil (s uw1 p er0 p r iy1 m iy0 ax m))
("superregional" nil (s uw1 p er0 r iy1 jh ax n ax l))
("superregionals" nil (s uw1 p er0 r iy1 jh ax n ax l z))
("supers" nil (s uw1 p er0 z))
("supersaver" nil (s uw1 p er0 s ey1 v er0))
("supersecret" nil (s uw1 p er0 s iy1 k r ih0 t))
("supersede" nil (s uw1 p er0 s iy1 d))
("superseded" nil (s uw1 p er0 s iy1 d ax d))
("supersedes" nil (s uw1 p er0 s iy1 d z))
("superseding" nil (s uw1 p er0 s iy1 d ih0 ng))
("supersonic" nil (s uw1 p er0 s aa1 n ih0 k))
("superstar" nil (s uw1 p er0 s t aa1 r))
("superstars" nil (s uw1 p er0 s t aa1 r z))
("superstation" nil (s uw1 p er0 s t ey1 sh ax n))
("superstition" nil (s uw1 p er0 s t ih1 sh ax n))
("superstitions" nil (s uw1 p er0 s t ih1 sh ax n z))
("superstitious" nil (s uw1 p er0 s t ih1 sh ax s))
("superstore" nil (s uw1 p er0 s t ao1 r))
("superstores" nil (s uw1 p er0 s t ao1 r z))
("superstructure" nil (s uw1 p er0 s t r ah1 k ch er0))
("superstructures" nil (s uw1 p er0 s t r ah1 k ch er0 z))
("supertanker" nil (s uw1 p er0 t ae1 ng k er0))
("supervalu" nil (s uw1 p er0 v ae1 l y uw0))
("supervise" nil (s uw1 p er0 v ay1 z))
("supervised" nil (s uw1 p er0 v ay1 z d))
("supervises" nil (s uw1 p er0 v ay1 z ih0 z))
("supervising" nil (s uw1 p er0 v ay1 z ih0 ng))
("supervision" nil (s uw1 p er0 v ih1 zh ax n))
("supervisor" nil (s uw1 p er0 v ay1 z er0))
("supervisors" nil (s uw1 p er0 v ay1 z er0 z))
("supervisory" nil (s uw1 p er0 v ay1 z er0 iy0))
("supervoting" nil (s uw1 p er0 v ow1 t ih0 ng))
("superx" nil (s uw1 p er0 eh1 k s))
("supine" nil (s ax p ay1 n))
("supinski" nil (s ax p ih1 n s k iy0))
("suppa" nil (s uw1 p ax))
("supper" nil (s ah1 p er0))
("suppers" nil (s ah1 p er0 z))
("suppes" nil (s ah1 p s))
("supplant" nil (s ax p l ae1 n t))
("supplanted" nil (s ax p l ae1 n t ax d))
("supplanting" nil (s ax p l ae1 n t ih0 ng))
("supple" nil (s ah1 p ax l))
("supplee" nil (s ah1 p l iy0))
("supplement" nil (s ah1 p l ax m ax n t))
("supplemental" nil (s ah1 p l ax m eh1 n t ax l))
("supplementary" nil (s ah1 p l ax m eh1 n t er0 iy0))
("supplemented" nil (s ah1 p l ax m eh1 n t ax d))
("supplementing" nil (s ah1 p l ax m ax n t ih0 ng))
("supplements" nil (s ah1 p l ax m ax n t s))
("supplicant" nil (s ah1 p l ax k ax n t))
("supplied" nil (s ax p l ay1 d))
("supplier" nil (s ax p l ay1 er0))
("suppliers" nil (s ax p l ay1 er0 z))
("supplies" nil (s ax p l ay1 z))
("supply" nil (s ax p l ay1))
("supplying" nil (s ax p l ay1 ih0 ng))
("support" nil (s ax p ao1 r t))
("supportable" nil (s ax p ao1 r t ax b ax l))
("supported" nil (s ax p ao1 r t ax d))
("supporter" nil (s ax p ao1 r t er0))
("supporters" nil (s ax p ao1 r t er0 z))
("supporting" nil (s ax p ao1 r t ih0 ng))
("supportive" nil (s ax p ao1 r t ih0 v))
("supports" nil (s ax p ao1 r t s))
("suppose" nil (s ax p ow1 z))
("supposed" nil (s ax p ow1 z d))
("supposedly" nil (s ax p ow1 z ax d l iy0))
("supposes" nil (s ax p ow1 z ih0 z))
("supposing" nil (s ax p ow1 z ih0 ng))
("supposition" nil (s ah1 p ax z ih1 sh ax n))
("suppress" nil (s ax p r eh1 s))
("suppressant" nil (s ax p r eh1 s ax n t))
("suppressed" nil (s ax p r eh1 s t))
("suppresses" nil (s ax p r eh1 s ih0 z))
("suppressing" nil (s ax p r eh1 s ih0 ng))
("suppression" nil (s ax p r eh1 sh ax n))
("suppressor" nil (s ax p r eh1 s er0))
("supra" nil (s uw1 p r ax))
("supranational" nil (s uw1 p r ax n ae1 sh ax n ax l))
("supremacist" nil (s uw0 p r eh1 m ax s ih0 s t))
("supremacists" nil (s uw0 p r eh1 m ax s ih0 s t s))
("supremacy" nil (s ax p r eh1 m ax s iy0))
("supreme" nil (s ax p r iy1 m))
("supremely" nil (s uw0 p r iy1 m ax l iy0))
("supremes" nil (s uw0 p r eh1 m eh0 z))
("suprenant" nil (s uw0 p r ey1 n ax n t))
("suprisingly" nil (s uw1 p r ay1 z ih0 ng l iy0))
("sur" nil (s er1))
("sura" nil (s uh1 r ax))
("surace" nil (s uh0 r aa1 s ey0))
("surat" nil (s er0 aa1 t))
("surbaugh" nil (s er1 b ao0))
("surber" nil (s er1 b er0))
("surcharge" nil (s er0 ch aa1 r jh))
("surcharges" nil (s er1 ch aa1 r jh ih0 z))
("sure" nil (sh uh1 r))
("surely" nil (sh uh1 r l iy0))
("suren" nil (s uh1 r ax n))
("surer" nil (sh uh1 r er0))
("suresh" nil (s er0 eh1 sh))
("surest" nil (sh uh1 r ih0 s t))
("surette" nil (s er0 eh1 t))
("surety" nil (sh uh1 r ax t iy0))
("surf" nil (s er1 f))
("surface" nil (s er1 f ax s))
("surfaced" nil (s er1 f ih0 s t))
("surfaceness" nil (s er1 f ax s n ax s))
("surfaces" nil (s er1 f ax s ax z))
("surfacing" nil (s er1 f ax s ih0 ng))
("surfactant" nil (s er0 f ae1 k t ax n t))
("surfboard" nil (s er1 f b ao1 r d))
("surfboards" nil (s er1 f b ao1 r d z))
("surfeit" nil (s er1 f ax t))
("surfer" nil (s er1 f er0))
("surfers" nil (s er1 f er0 z))
("surfing" nil (s er1 f ih0 ng))
("surge" nil (s er1 jh))
("surged" nil (s er1 jh d))
("surgeon" nil (s er1 jh ax n))
("surgeons" nil (s er1 jh ax n z))
("surgeries" nil (s er1 jh er0 iy0 z))
("surgery" nil (s er1 jh er0 iy0))
("surges" nil (s er1 jh ax z))
("surgical" nil (s er1 jh ih0 k ax l))
("surgically" nil (s er1 jh ih0 k ax l iy0))
("surging" nil (s er1 jh ih0 ng))
("suriano" nil (s uh1 r iy0 aa1 n ow0))
("surinam" nil (s uh1 r ax n aa1 m))
("suriname" nil (s er1 ih0 n aa0 m))
("surles" nil (s ao1 r ax l z))
("surly" nil (s er1 l iy0))
("surma" nil (s er1 m ax))
("surman" nil (s er1 m ax n))
("surmise" nil (s er0 m ay1 z))
("surmised" nil (s er0 m ay1 z d))
("surmises" nil (s er0 m ay1 z ih0 z))
("surmount" nil (s er0 m aw1 n t))
("surmounted" nil (s er0 m aw1 n t ih0 d))
("surmounting" nil (s er0 m aw1 n t ih0 ng))
("surname" nil (s er1 n ey1 m))
("surnames" nil (s er1 n ey1 m z))
("surowiec" nil (s er0 aw1 iy0 k))
("surpass" nil (s er0 p ae1 s))
("surpassed" nil (s er0 p ae1 s t))
("surpasses" nil (s er0 p ae1 s ih0 z))
("surpassing" nil (s er0 p ae1 s ih0 ng))
("surplus" nil (s er1 p l ax s))
("surpluses" nil (s er1 p l ax s ih0 z))
("surprenant" nil (s er1 p r ih0 n ax n t))
("surprise" nil (s er0 p r ay1 z))
("surprised" nil (s er0 p r ay1 z d))
("surprises" nil (s er0 p r ay1 z ax z))
("surprising" nil (s er0 p r ay1 z ih0 ng))
("surprisingly" nil (s er0 p r ay1 z ih0 ng l iy0))
("surratt" nil (s ao1 r ax t))
("surreal" nil (s er0 iy1 l))
("surrealism" nil (s er0 iy1 l ih0 z ax m))
("surrealisms" nil (s er0 iy1 l ih0 z ax m z))
("surrealistic" nil (s er0 iy1 l ih1 s t ih0 k))
("surrell" nil (s ao1 r ax l))
("surrency" nil (s ao1 r ax n s iy0))
("surrender" nil (s er0 eh1 n d er0))
("surrendered" nil (s er0 eh1 n d er0 d))
("surrendering" nil (s er0 eh1 n d er0 ih0 ng))
("surrenders" nil (s er0 eh1 n d er0 z))
("surreptitious" nil (s er1 ax p t ih1 sh ax s))
("surreptitiously" nil (s er1 ax p t ih1 sh ax s l iy0))
("surrett" nil (s ao1 r ih0 t))
("surrette" nil (s er0 eh1 t))
("surrey" nil (s er1 iy0))
("surrogacy" nil (s er1 ax g ax s iy0))
("surrogate" nil (s er1 ax g ax t))
("surrogates" nil (s er1 ax g ax t s))
("surround" nil (s er0 aw1 n d))
("surrounded" nil (s er0 aw1 n d ax d))
("surrounding" nil (s er0 aw1 n d ih0 ng))
("surroundings" nil (s er0 aw1 n d ih0 ng z))
("surrounds" nil (s er0 aw1 n d z))
("surry" nil (s er1 iy0))
("surtax" nil (s er1 t ae1 k s))
("surtaxes" nil (s er1 t ae1 k s ih0 z))
("surveillance" nil (s er0 v ey1 l ax n s))
("survey" v (s er0 v ey1))
("survey" n (s er1 v ey1))
("surveyed" nil (s er0 v ey1 d))
("surveying" nil (s er0 v ey1 ih0 ng))
("surveyor" nil (s er0 v ey1 er0))
("surveyors" nil (s er0 v ey1 er0 z))
("surveys" v (s er0 v ey1 z))
("surveys" n (s er1 v ey1 z))
("survivability" nil (s er0 v ay1 v ax b ih1 l ih0 t iy0))
("survivable" nil (s er0 v ay1 v ax b ax l))
("survival" nil (s er0 v ay1 v ax l))
("survivalist" nil (s er0 v ay1 v ax l ih0 s t))
("survivalists" nil (s er0 v ay1 v ax l ih0 s t s))
("survive" nil (s er0 v ay1 v))
("survived" nil (s er0 v ay1 v d))
("survives" nil (s er0 v ay1 v z))
("surviving" nil (s er0 v ay1 v ih0 ng))
("survivor" nil (s er0 v ay1 v er0))
("survivors" nil (s er0 v ay1 v er0 z))
("sus" nil (s ah1 s))
("susa" nil (s uw1 s ax))
("susan" nil (s uw1 z ax n))
("susana" nil (s uw0 s aa1 n ax))
("susann" nil (s uw1 z ae1 n))
("susanna" nil (s uw0 z ae1 n ax))
("susannah" nil (s uw1 s ae1 n ax))
("susanne" nil (s uw1 z ae1 n))
("susceptibility" nil (s ax s eh1 p t ax b ih1 l ax t iy0))
("susceptible" nil (s ax s eh1 p t ax b ax l))
("susette" nil (s uw1 z eh1 t))
("sushi" nil (s uw1 sh iy0))
("susi" nil (s uw1 s iy0))
("susie" nil (s uw1 z iy0))
("suski" nil (s ah1 s k iy0))
("suskind" nil (s ah1 s k ih0 n d))
("susko" nil (s ah1 s k ow0))
("susman" nil (s ah1 s m ax n))
("susong" nil (s ah1 s ao0 ng))
("suspect" v (s ax s p eh1 k t))
("suspect" n (s ah1 s p eh1 k t))
("suspected" nil (s ax s p eh1 k t ax d))
("suspecting" nil (s ax s p eh1 k t ih0 ng))
("suspects" v (s ax s p eh1 k t s))
("suspects" n (s ah1 s p eh1 k t s))
("suspend" nil (s ax s p eh1 n d))
("suspended" nil (s ax s p eh1 n d ax d))
("suspenders" nil (s ax s p eh1 n d er0 z))
("suspending" nil (s ax s p eh1 n d ih0 ng))
("suspends" nil (s ax s p eh1 n d z))
("suspense" nil (s ax s p eh1 n s))
("suspenseful" nil (s ax s p eh1 n s f ax l))
("suspension" nil (s ax s p eh1 n sh ax n))
("suspensions" nil (s ax s p eh1 n sh ax n z))
("suspicion" nil (s ax s p ih1 sh ax n))
("suspicions" nil (s ax s p ih1 sh ax n z))
("suspicious" nil (s ax s p ih1 sh ax s))
("suspiciously" nil (s ax s p ih1 sh ax s l iy0))
("susquehanna" nil (s uw1 s k w eh0 hh ae1 n ax))
("suss" nil (s ah1 s))
("susser" nil (s ah1 s er0))
("sussex" nil (s ah1 s ih0 k s))
("susskind" nil (s ah1 s k ih0 n d))
("sussman" nil (s ah1 s m ax n))
("susswein" nil (s ah1 s w iy1 n))
("sustain" nil (s ax s t ey1 n))
("sustainability" nil (s ax s t ey1 n ax b ih1 l ih0 t iy0))
("sustainable" nil (s ax s t ey1 n ax b ax l))
("sustained" nil (s ax s t ey1 n d))
("sustaining" nil (s ax s t ey1 n ih0 ng))
("sustains" nil (s ax s t ey1 n z))
("sustaita" nil (s uw0 s t ey1 t ax))
("sustenance" nil (s ah1 s t ax n ax n s))
("susteren" nil (s ax s t eh1 r ax n))
("susumu" nil (s uw1 s uw1 m uw0))
("susy" nil (s uw1 z iy0))
("sutch" nil (s ah1 ch))
("sutcliff" nil (s ah1 t k l ih0 f))
("sutcliffe" nil (s ah1 t k l ih0 f))
("suter" nil (s uw1 t er0))
("sutera" nil (s uw0 t eh1 r ax))
("sutfin" nil (s ah1 t f ih0 n))
("suthard" nil (s ah1 th er0 d))
("suther" nil (s ah1 dh er0))
("sutherland" nil (s ah1 dh er0 l ax n d))
("sutherlin" nil (s ah1 th er0 l ih0 n))
("suthers" nil (s ah1 dh er0 z))
("sutley" nil (s ah1 t l iy0))
("sutliff" nil (s ah1 t l ih0 f))
("sutnick" nil (s ah1 t n ih0 k))
("suto" nil (s uw1 t ow0))
("sutor" nil (s uw1 t er0))
("sutphen" nil (s ah1 t f ax n))
("sutphin" nil (s ah1 t f ih0 n))
("sutro" nil (s uw1 t r ow0))
("sutter" nil (s ah1 t er0))
("suttle" nil (s ah1 t ax l))
("suttles" nil (s ah1 t ax l z))
("suttmeier" nil (s ah1 t m ay1 r))
("sutton" nil (s ah1 t ax n))
("suture" nil (s uw1 ch er0))
("sutures" nil (s uw1 ch er0 z))
("suu" nil (eh1 s y uw1 y uw1))
("suva" nil (s uw1 v ax))
("suydam" nil (s ay1 d ax m))
("suzanna" nil (s uw1 z ae1 n ax))
("suzanne" nil (s uw0 z ae1 n))
("suzette" nil (s uw1 z eh1 t))
("suzhou" nil (s uw1 z hh ow1))
("suzie" nil (s ah1 z iy0))
("suzuki" nil (s ax z uw1 k iy0))
("suzy" nil (s uw1 z iy0))
("svec" nil (s v eh1 k))
("svehla" nil (s v eh1 l ax))
("svelte" nil (s v eh1 l t))
("sven" nil (s v eh1 n))
("svendsen" nil (s v eh1 n d s ax n))
("svenska" nil (s v eh1 n s k ax))
("svenson" nil (s v eh1 n s ax n))
("svensson" nil (s v eh1 n s ax n))
("sverdlovsk" nil (s v er1 d l aa0 v s k))
("sverige" nil (s v eh1 r ih0 jh))
("svetlik" nil (s v eh1 t l ih0 k))
("svitak" nil (s v ih1 t ax k))
("svizzera" nil (s v ih0 z eh1 r ax))
("svoboda" nil (s v ow0 b ow1 d ax))
("swab" nil (s w aa1 b))
("swaby" nil (s w aa1 b iy0))
("swackhamer" nil (s w ao1 k ae1 m er0))
("swader" nil (s w ey1 d er0))
("swadley" nil (s w aa1 d l iy0))
("swafford" nil (s w aa1 f er0 d))
("swager" nil (s w ey1 g er0))
("swagerty" nil (s w ae1 jh er0 t iy0))
("swaggart" nil (s w ae1 g er0 t))
("swagger" nil (s w ae1 g er0))
("swaggering" nil (s w ae1 g er0 ih0 ng))
("swaggerty" nil (s w ae1 g er0 t iy0))
("swailes" nil (s w ey1 l z))
("swails" nil (s w ey1 l z))
("swaim" nil (s w ey1 m))
("swain" nil (s w ey1 n))
("swaine" nil (s w ey1 n))
("swainston" nil (s w ey1 n s t ax n))
("swales" nil (s w ey1 l z))
("swalley" nil (s w ao1 l iy0))
("swallow" nil (s w aa1 l ow0))
("swallowed" nil (s w aa1 l ow0 d))
("swallowing" nil (s w aa1 l ow0 ih0 ng))
("swallows" nil (s w aa1 l ow0 z))
("swam" nil (s w ae1 m))
("swami" nil (s w aa1 m iy0))
("swaminathan" nil (s w aa1 m ih0 n aa1 th ax n))
("swamp" nil (s w aa1 m p))
("swampbuster" nil (s w aa1 m p b ah1 s t er0))
("swamped" nil (s w ao1 m p t))
("swampers" nil (s w aa1 m p er0 z))
("swamping" nil (s w aa1 m p ih0 ng))
("swamps" nil (s w aa1 m p s))
("swampy" nil (s w aa1 m p iy0))
("swan" nil (s w aa1 n))
("swanberg" nil (s w aa1 n b er0 g))
("swanda" nil (s w aa1 n d ax))
("swander" nil (s w aa1 n d er0))
("swaner" nil (s w ao1 n er0))
("swaney" nil (s w ao1 n iy0))
("swanger" nil (s w ao1 ng er0))
("swango" nil (s w aa1 ng g ow0))
("swanigan" nil (s w aa1 n ih0 g ax n))
("swank" nil (s w ae1 ng k))
("swanke" nil (s w ao1 ng k))
("swanky" nil (s w aa1 ng k iy0))
("swann" nil (s w aa1 n))
("swanner" nil (s w aa1 n er0))
("swans" nil (s w aa1 n z))
("swanson" nil (s w aa1 n s ax n))
("swanstrom" nil (s w aa1 n s t r ax m))
("swantek" nil (s w ao1 n t ih0 k))
("swanton" nil (s w aa1 n t ax n))
("swanzy" nil (s w aa1 n z iy0))
("swap" nil (s w aa1 p))
("swape" nil (s w ey1 p))
("swapes" nil (s w ey1 p s))
("swapo" nil (s w aa1 p ow0))
("swapp" nil (s w aa1 p))
("swapped" nil (s w aa1 p t))
("swapping" nil (s w aa1 p ih0 ng))
("swaps" nil (s w aa1 p s))
("sward" nil (s w ao1 r d))
("swaringen" nil (s w eh1 r ih0 ng ax n))
("swarm" nil (s w ao1 r m))
("swarmed" nil (s w ao1 r m d))
("swarming" nil (s w ao1 r m ih0 ng))
("swarms" nil (s w ao1 r m z))
("swarner" nil (s w ao1 r n er0))
("swarovski" nil (s w aa0 r aa1 v s k iy0))
("swart" nil (s w ao1 r t))
("swarthmore" nil (s w ao1 r th m ao1 r))
("swarthout" nil (s w ao1 r th aw1 t))
("swarthy" nil (s w ao1 r dh iy0))
("swartley" nil (s w ao1 r t l iy0))
("swartout" nil (s w ao1 r t ax t))
("swarts" nil (s w ao1 r t s))
("swartwood" nil (s w ao1 r t w uh1 d))
("swartwout" nil (s w ao1 r t w aw0 t))
("swartz" nil (s w ao1 r t s))
("swartzbaugh" nil (s w ao1 r t s b aa0))
("swartzendruber" nil (s w ao1 r t s ax n d r uw0 b er0))
("swartzentruber" nil (s w ao1 r t s ax n t r uw0 b er0))
("swartzlander" nil (s w ao1 r t s l ae1 n d er0))
("swartzwelder" nil (s w ao1 r t s w eh1 l d er0))
("swasey" nil (s w aa1 z iy0))
("swashbuckling" nil (s w aa1 sh b ah1 k l ih0 ng))
("swat" nil (s w aa1 t))
("swatch" nil (s w aa1 ch))
("swatches" nil (s w aa1 ch ax z))
("swatek" nil (s w ao1 t ih0 k))
("swath" nil (s w aa1 th))
("swathed" nil (s w aa1 dh d))
("swatow" nil (s w aa1 t ow1))
("swatted" nil (s w aa1 t ih0 d))
("swatzell" nil (s w ao1 t z ax l))
("swauger" nil (s w aw1 g er0))
("swavely" nil (s w ey1 v l iy0))
("sway" nil (s w ey1))
("swayed" nil (s w ey1 d))
("swaying" nil (s w ey1 ih0 ng))
("swayne" nil (s w ey1 n))
("sways" nil (s w ey1 z))
("swayze" nil (s w ey1 z))
("swazi" nil (s w aa1 z iy0))
("swaziland" nil (s w aa1 z ih0 l ax n d))
("sweaney" nil (s w iy1 n iy0))
("sweany" nil (s w iy1 n iy0))
("swear" nil (s w eh1 r))
("swearengen" nil (s w ih1 r ih0 ng ax n))
("swearengin" nil (s w eh1 r ih0 ng ax n))
("swearing" nil (s w eh1 r ih0 ng))
("swearingen" nil (s w eh1 r ih0 ng ax n))
("swearingin" nil (s w eh1 r ih0 ng ax n))
("swears" nil (s w eh1 r z))
("sweat" nil (s w eh1 t))
("sweated" nil (s w eh1 t ih0 d))
("sweater" nil (s w eh1 t er0))
("sweaters" nil (s w eh1 t er0 z))
("sweating" nil (s w eh1 t ih0 ng))
("sweatman" nil (s w iy1 t m ax n))
("sweats" nil (s w eh1 t s))
("sweatshirt" nil (s w eh1 t sh er1 t))
("sweatshirts" nil (s w eh1 t sh er1 t s))
("sweatshop" nil (s w eh1 t sh aa1 p))
("sweatshops" nil (s w eh1 t sh aa1 p s))
("sweatt" nil (s w iy1 t))
("sweaty" nil (s w eh1 t iy0))
("sweazy" nil (s w iy1 z iy0))
("swecker" nil (s w eh1 k er0))
("swed" nil (s w eh1 d))
("sweda" nil (s w iy1 d ax))
("swedberg" nil (s w eh1 d b er0 g))
("swede" nil (s w iy1 d))
("sweden" nil (s w iy1 d ax n))
("swedes" nil (s w iy1 d z))
("swedish" nil (s w iy1 d ih0 sh))
("swedlund" nil (s w eh1 d l ax n d))
("sweeden" nil (s w iy1 d ax n))
("sween" nil (s w iy1 n))
("sweeney" nil (s w iy1 n iy0))
("sweeny" nil (s w iy1 n iy0))
("sweep" nil (s w iy1 p))
("sweeper" nil (s w iy1 p er0))
("sweepers" nil (s w iy1 p er0 z))
("sweeping" nil (s w iy1 p ih0 ng))
("sweeps" nil (s w iy1 p s))
("sweepstake" nil (s w iy1 p s t ey1 k))
("sweepstakes" nil (s w iy1 p s t ey1 k s))
("sweers" nil (s w ih1 r z))
("sweet" nil (s w iy1 t))
("sweeten" nil (s w iy1 t ax n))
("sweetened" nil (s w iy1 t ax n d))
("sweetener" nil (s w iy1 t ax n er0))
("sweeteners" nil (s w iy1 t ax n er0 z))
("sweetening" nil (s w iy1 t ax n ih0 ng))
("sweetens" nil (s w iy1 t ax n z))
("sweeter" nil (s w iy1 t er0))
("sweetest" nil (s w iy1 t ax s t))
("sweetheart" nil (s w iy1 t hh aa1 r t))
("sweethearts" nil (s w iy1 t hh aa1 r t s))
("sweetie" nil (s w iy1 t iy0))
("sweeting" nil (s w iy1 t ih0 ng))
("sweetland" nil (s w iy1 t l ax n d))
("sweetly" nil (s w iy1 t l iy0))
("sweetman" nil (s w iy1 t m ax n))
("sweetness" nil (s w iy1 t n ax s))
("sweeton" nil (s w iy1 t ax n))
("sweets" nil (s w iy1 t s))
("sweetser" nil (s w iy1 t s er0))
("sweetwater" nil (s w iy1 t w aa1 t er0))
("sweezey" nil (s w iy1 z iy0))
("sweezy" nil (s w iy1 z iy0))
("sweger" nil (s w iy1 g er0))
("swehla" nil (s w eh1 l ax))
("sweig" nil (s w ay1 g))
("sweigard" nil (s w ay1 g er0 d))
("sweigart" nil (s w iy1 g aa0 r t))
("sweigert" nil (s w ay1 g er0 t))
("sweitzer" nil (s w ay1 t z er0))
("swell" nil (s w eh1 l))
("swelled" nil (s w eh1 l d))
("swelling" nil (s w eh1 l ih0 ng))
("swells" nil (s w eh1 l z))
("swelter" nil (s w eh1 l t er0))
("sweltering" nil (s w eh1 l t er0 ih0 ng))
("swem" nil (s w eh1 m))
("swendsen" nil (s w eh1 n d s ax n))
("swenet" nil (s w eh1 n ax t))
("sweney" nil (s w eh1 n iy0))
("swensen" nil (s w eh1 n s ax n))
("swenson" nil (s w eh1 n s ax n))
("swept" nil (s w eh1 p t))
("swerdlow" nil (s w er1 d l aw0))
("swergold" nil (s w er1 g ow1 l d))
("swerve" nil (s w er1 v))
("swerved" nil (s w er1 v d))
("swete" nil (s w iy1 t))
("swetland" nil (s w eh1 t l ax n d))
("swetnam" nil (s w eh1 t n ax m))
("swett" nil (s w eh1 t))
("swetz" nil (s w eh1 t s))
("swezey" nil (s w eh1 z iy0))
("swiatek" nil (s v iy0 aa1 t eh0 k))
("swiatkowski" nil (s v iy0 ax t k ao1 f s k iy0))
("swicegood" nil (s w ih1 s ih0 g uh0 d))
("swick" nil (s w ih1 k))
("swickard" nil (s w ih1 k er0 d))
("swider" nil (s w ay1 d er0))
("swiderski" nil (s v ih0 d er1 s k iy0))
("swiech" nil (s w iy1 ch))
("swier" nil (s w ay1 er0))
("swift" nil (s w ih1 f t))
("swifter" nil (s w ih1 f t er0))
("swiftest" nil (s w ih1 f t ax s t))
("swiftly" nil (s w ih1 f t l iy0))
("swiftness" nil (s w ih1 f t n ax s))
("swifts" nil (s w ih1 f t s))
("swig" nil (s w ih1 g))
("swigart" nil (s w ih1 g aa1 r t))
("swiger" nil (s w ay1 g er0))
("swigert" nil (s w ih1 g er0 t))
("swiggum" nil (s w ih1 g ax m))
("swihart" nil (s w ih1 hh aa0 r t))
("swilley" nil (s w ih1 l iy0))
("swilling" nil (s w ih1 l ih0 ng))
("swillinger" nil (s w ih1 l ih0 ng er0))
("swim" nil (s w ih1 m))
("swimmer" nil (s w ih1 m er0))
("swimmers" nil (s w ih1 m er0 z))
("swimming" nil (s w ih1 m ih0 ng))
("swims" nil (s w ih1 m z))
("swimsuit" nil (s w ih1 m s uw1 t))
("swimsuits" nil (s w ih1 m s uw1 t s))
("swimwear" nil (s w ih1 m w eh1 r))
("swinburnes" nil (s w ih1 n b er0 n z))
("swindall" nil (s w ih1 n d ax l))
("swindell" nil (s w ih1 n d ax l))
("swindle" nil (s w ih1 n d ax l))
("swindled" nil (s w ih1 n d ax l d))
("swindlehurst" nil (s w ih1 n d l ih0 hh er0 s t))
("swindler" nil (s w ih1 n d ax l er0))
("swindlers" nil (s w ih1 n d l er0 z))
("swindles" nil (s w ih1 n d ax l z))
("swindling" nil (s w ih1 n d l ih0 ng))
("swine" nil (s w ay1 n))
("swineford" nil (s w ih1 n ih0 f er0 d))
("swinehart" nil (s w ay1 n hh aa1 r t))
("swiney" nil (s w ay1 n iy0))
("swinford" nil (s w ih1 n f er0 d))
("swing" nil (s w ih1 ng))
("swinger" nil (s w ih1 ng er0))
("swingers" nil (s w ih1 ng er0 z))
("swinging" nil (s w ih1 ng ih0 ng))
("swingle" nil (s w ih1 ng g ax l))
("swingler" nil (s w ih1 ng g ax l er0))
("swingley" nil (s w ih1 ng g l iy0))
("swings" nil (s w ih1 ng z))
("swink" nil (s w ih1 ng k))
("swinney" nil (s w ih1 n iy0))
("swinson" nil (s w ih1 n s ax n))
("swint" nil (s w ih1 n t))
("swinton" nil (s w ih1 n t ax n))
("swipe" nil (s w ay1 p))
("swiped" nil (s w ay1 p t))
("swipes" nil (s w ay1 p s))
("swiping" nil (s w ay1 p ih0 ng))
("swire" nil (s w ay1 r))
("swirl" nil (s w er1 l))
("swirled" nil (s w er1 l d))
("swirling" nil (s w er1 l ih0 ng))
("swirls" nil (s w er1 l z))
("swirsky" nil (s w er1 s k iy0))
("swished" nil (s w ih1 sh t))
("swisher" nil (s w ih1 sh er0))
("swishes" nil (s w ih1 sh ih0 z))
("swiss" nil (s w ih1 s))
("swissair" nil (s w ih0 s eh1 r))
("swisshelm" nil (s w ih1 sh ih0 l m))
("swistak" nil (s w ih1 s t ax k))
("switaj" nil (s w ih0 t ay1))
("switala" nil (s w ih0 t aa1 l ax))
("switalski" nil (s w ih0 t aa1 l s k iy0))
("switch" nil (s w ih1 ch))
("switchblades" nil (s w ih1 ch b l ey1 d z))
("switchboard" nil (s w ih1 ch b ao1 r d))
("switchboards" nil (s w ih1 ch b ao1 r d z))
("switched" nil (s w ih1 ch t))
("switchers" nil (s w ih1 ch er0 z))
("switches" nil (s w ih1 ch ax z))
("switching" nil (s w ih1 ch ih0 ng))
("switzer" nil (s w ih1 t s er0))
("switzerland" nil (s w ih1 t s er0 l ax n d))
("swivel" nil (s w ih1 v ax l))
("swiveling" nil (s w ih1 v ax l ih0 ng))
("swivels" nil (s w ih1 v ax l z))
("swoboda" nil (s w ow0 b ow1 d ax))
("swofford" nil (s w aa1 f er0 d))
("swogger" nil (s w aa1 g er0))
("swollen" nil (s w ow1 l ax n))
("swonger" nil (s w ao1 ng er0))
("swoon" nil (s w uw1 n))
("swooned" nil (s w uw1 n d))
("swooning" nil (s w uw1 n ih0 ng))
("swoop" nil (s w uw1 p))
("swoope" nil (s w uw1 p))
("swooped" nil (s w uw1 p t))
("swooping" nil (s w uw1 p ih0 ng))
("swoops" nil (s w uw1 p s))
("swoosh" nil (s w uw1 sh))
("swooshed" nil (s w uw1 sh t))
("swope" nil (s w ow1 p))
("swopes" nil (s w ow1 p s))
("swor" nil (s w er1))
("sword" nil (s ao1 r d))
("swordfish" nil (s ao1 r d f ih1 sh))
("swordlike" nil (s ao1 r d l ay1 k))
("swordplay" nil (s ao1 r d p l ey1))
("swordplays" nil (s ao1 r d p l ey1 z))
("swords" nil (s ao1 r d z))
("swore" nil (s w ao1 r))
("sworn" nil (s w ao1 r n))
("swoveland" nil (s w ow1 v l ax n d))
("swoyer" nil (s w oy1 er0))
("swum" nil (s w ah1 m))
("swung" nil (s w ah1 ng))
("swyers" nil (s w ay1 er0 z))
("swygert" nil (s w ih1 g er0 t))
("sy" nil (s ay1))
("sybase" nil (s ay1 b ey1 s))
("sybert" nil (s ih1 b er0 t))
("sybil" nil (s ih1 b ih0 l))
("sybilla" nil (s ih0 b ih1 l ax))
("sybille" nil (s ih1 b ih0 l))
("sybron" nil (s ih1 b r ax n))
("sycamore" nil (s ih1 k ax m ao1 r))
("sycara" nil (s ih0 k aa1 r ax))
("sycophantic" nil (s ih1 k ax f ae1 n t ih0 k))
("syders" nil (s ay1 d er0 z))
("sydney" nil (s ih1 d n iy0))
("sydnor" nil (s ih1 d n er0))
("sydow" nil (s ih1 d ow0))
("syed" nil (s ay1 d))
("syers" nil (s ay1 er0 z))
("syfert" nil (s ih1 f er0 t))
("sykes" nil (s ay1 k s))
("sykora" nil (s ih0 k ao1 r ax))
("syler" nil (s ay1 l er0))
("sylla" nil (s ih1 l ax))
("syllabic" nil (s ax l ae1 b ih0 k))
("syllable" nil (s ih1 l ax b ax l))
("syllables" nil (s ih1 l ax b ax l z))
("syllabus" nil (s ih1 l ax b ax s))
("sylmar" nil (s ih1 l m aa0 r))
("sylva" nil (s ih1 l v ax))
("sylvain" nil (s ih0 l v ey1 n))
("sylvan" nil (s ih1 l v ax n))
("sylvana" nil (s ih0 l v ae1 n ax))
("sylvania" nil (s ih0 l v ey1 n iy0 ax))
("sylvanus" nil (s ih1 l v ax n ih0 s))
("sylvest" nil (s y l v ey1 ih0 s t))
("sylvester" nil (s ih0 l v eh1 s t er0))
("sylvestre" nil (s ih0 l v eh1 s t er0))
("sylvia" nil (s ih1 l v iy0 ax))
("sylvie" nil (s ih1 l v iy0))
("sylvio" nil (s ih1 l v iy0 ow0))
("sylvite" nil (s ih1 l v ay1 t))
("sym" nil (s ih1 m))
("symanski" nil (s ih0 m ae1 n s k iy0))
("symantec" nil (s ih0 m ae1 n t eh1 k))
("symbion" nil (s ih1 m b iy0 ax n))
("symbiosis" nil (s ih1 m b ay0 ow1 s ax s))
("symbiotic" nil (s ih1 m b iy0 aa1 t ih0 k))
("symbol" nil (s ih1 m b ax l))
("symbolic" nil (s ih0 m b aa1 l ih0 k))
("symbolically" nil (s ih0 m b aa1 l ih0 k ax l iy0))
("symbolics" nil (s ih0 m b aa1 l ih0 k s))
("symbolism" nil (s ih1 m b ax l ih1 z ax m))
("symbolists" nil (s ih1 m b ax l ax s t s))
("symbolize" nil (s ih1 m b ax l ay1 z))
("symbolized" nil (s ih1 m b ax l ay1 z d))
("symbolizes" nil (s ih1 m b ax l ay1 z ax z))
("symbolizing" nil (s ih1 m b ax l ay1 z ih0 ng))
("symbols" nil (s ih1 m b ax l z))
("syme" nil (s ay1 m))
("symes" nil (s ay1 m z))
("symington" nil (s ih1 m ih0 ng t ax n))
("symmes" nil (s ih1 m z))
("symmetrical" nil (s ax m eh1 t r ih0 k ax l))
("symmetry" nil (s ih1 m ax t r iy0))
("symmonds" nil (s ih1 m ax n d z))
("symms" nil (s ih1 m z))
("symon" nil (s ih1 m ax n))
("symonds" nil (s ih1 m ax n d z))
("symons" nil (s ih1 m ax n z))
("sympathetic" nil (s ih1 m p ax th eh1 t ih0 k))
("sympathetically" nil (s ih1 m p ax th eh1 t ih0 k ax l iy0))
("sympathies" nil (s ih1 m p ax th iy0 z))
("sympathize" nil (s ih1 m p ax th ay1 z))
("sympathized" nil (s ih1 m p ax th ay1 z d))
("sympathizer" nil (s ih1 m p ax th ay1 z er0))
("sympathizers" nil (s ih1 m p ax th ay1 z er0 z))
("sympathizes" nil (s ih1 m p ax th ay1 z ih0 z))
("sympathy" nil (s ih1 m p ax th iy0))
("symphonic" nil (s ih0 m f aa1 n ih0 k))
("symphonies" nil (s ih1 m f ax n iy0 z))
("symphony" nil (s ih1 m f ax n iy0))
("symposium" nil (s ih0 m p ow1 z iy0 ax m))
("symposiums" nil (s ih0 m p ow1 z iy0 ax m z))
("sympson" nil (s ih1 m p s ax n))
("symptom" nil (s ih1 m p t ax m))
("symptomatic" nil (s ih1 m p t ax m ae1 t ih0 k))
("symptoms" nil (s ih1 m p t ax m z))
("syms" nil (s ih1 m z))
("symtron" nil (s ih1 m t r aa1 n))
("syn" nil (s ih1 n))
("syna" nil (s ih1 n ax))
("synagogue" nil (s ih1 n ax g ao1 g))
("synagogues" nil (s ih1 n ax g ao1 g z))
("synalloy" nil (s ih0 n ae1 l oy0))
("synan" nil (s ay1 n ax n))
("synar" nil (s ih1 n aa0 r))
("synbiotics" nil (s ih1 n b iy0 aa1 t ih0 k s))
("sync" nil (s ih1 ng k))
("synch" nil (s ih1 n ch))
("synchro" nil (s ih1 ng k r ow0))
("synchronic" nil (s ih0 ng k r aa1 n ih0 k))
("synchronized" nil (s ih1 ng k r ax n ay1 z d))
("syncom" nil (s ih1 ng k ax m))
("syncopated" nil (s ih1 ng k ax p ey1 t ih0 d))
("syncor" nil (s ih1 n k ao1 r))
("synder" nil (s ih1 n d er0))
("syndicate" nil (s ih1 n d ih0 k ax t))
("syndicated" nil (s ih1 n d ih0 k ey1 t ih0 d))
("syndicates" nil (s ih1 n d ih0 k ey1 t s))
("syndicating" nil (s ih1 n d ih0 k ey1 t ih0 ng))
("syndication" nil (s ih1 n d ih0 k ey1 sh ax n))
("syndications" nil (s ih1 n d ih0 k ey1 sh ax n z))
("syndicator" nil (s ih1 n d ih0 k ey1 t er0))
("syndicators" nil (s ih1 n d ih0 k ey1 t er0 z))
("syndicats" nil (s ih1 n d ih0 k ih1 t s))
("syndrome" nil (s ih1 n d r ow1 m))
("syndromes" nil (s ih1 n d r ow1 m z))
("synergen" nil (s ih1 n er0 jh eh1 n))
("synergies" nil (s ih1 n er0 jh iy0 z))
("synergism" nil (s ih1 n er0 jh ih1 z ax m))
("synergistic" nil (s ih1 n er0 jh ih1 s t ih0 k))
("synergy" nil (s ih1 n er0 jh iy0))
("syngman" nil (s ih1 ng m ax n))
("synhorst" nil (s ih1 n hh ao1 r s t))
("synnott" nil (s ih1 n ax t))
("synod" nil (s ih1 n ax d))
("synonym" nil (s ih1 n ax n ih1 m))
("synonymous" nil (s ax n aa1 n ax m ax s))
("synonymously" nil (s ax n aa1 n ax m ax s l iy0))
("synopsis" nil (s ih0 n aa1 p s ih0 s))
("synoptics" nil (s ih0 n aa1 p t ih0 k s))
("synovus" nil (s ax n ow1 v ax s))
("syntech" nil (s ih1 n t eh1 k))
("syntex" nil (s ih1 n t eh1 k s))
("synthesis" nil (s ih1 n th ax s ax s))
("synthesize" nil (s ih1 n th ax s ay1 z))
("synthesized" nil (s ih1 n th ih0 s ay1 z d))
("synthesizer" nil (s ih1 n th ax s ay1 z er0))
("synthesizers" nil (s ih1 n th ax s ay1 z er0 z))
("synthesizing" nil (s ih1 n th ax s ay1 z ih0 ng))
("synthetic" nil (s ih0 n th eh1 t ih0 k))
("synthetically" nil (s ih0 n th eh1 t ih0 k l iy0))
("synthetics" nil (s ih0 n th eh1 t ih0 k s))
("syntrex" nil (s ih1 n t r ax k s))
("syp" nil (s ay1 p))
("sypher" nil (s ih1 f er0))
("syphers" nil (s ih1 f er0 z))
("syphilis" nil (s ih1 f ax l ih0 s))
("syphon" nil (s ay1 f ax n))
("sypniewski" nil (s ih0 p n iy0 eh1 f s k iy0))
("sypolt" nil (s ih1 p ow0 l t))
("syracuse" nil (s ih1 r ax k y uw1 z))
("syrek" nil (s ih1 r ih0 k))
("syria" nil (s ih1 r iy0 ax))
("syrian" nil (s ih1 r iy0 ax n))
("syrians" nil (s ih1 r iy0 ax n z))
("syring" nil (s ay1 r ih0 ng))
("syringe" nil (s er0 ih1 n jh))
("syringes" nil (s er0 ih1 n jh ax z))
("syron" nil (s ay1 r ax n))
("syrup" nil (s er1 ax p))
("syrups" nil (s er1 ax p s))
("sysco" nil (s ih1 s k ow0))
("syscon" nil (s ay1 s k ax n))
("system" nil (s ih1 s t ax m))
("systematic" nil (s ih1 s t ax m ae1 t ih0 k))
("systematically" nil (s ih1 s t ax m ae1 t ih0 k l iy0))
("systematics" nil (s ih1 s t ax m ae1 t ih0 k s))
("systemhouse" nil (s ih1 s t ax m hh aw1 s))
("systemic" nil (s ih0 s t eh1 m ih0 k))
("systemically" nil (s ax s t eh1 m ih0 k l iy0))
("systemix" nil (s ih1 s t ih0 m ih0 k s))
("systemone" nil (s ih1 s t ax m ow1 n))
("systems" nil (s ih1 s t ax m z))
("systemwide" nil (s ih1 s t ax m w ay1 d))
("systran" nil (s ay1 s t r ae1 n))
("sytsma" nil (s ih1 t s m ax))
("syverson" nil (s ih1 v er0 s ax n))
("syvertsen" nil (s ih1 v er0 t s ax n))
("szabo" nil (sh aa1 b ow0))
("szafran" nil (sh aa1 f r ax n))
("szafranski" nil (sh ax f r ae1 n s k iy0))
("szalay" nil (sh aa1 l ay0))
("szalkowski" nil (sh ax l k ao1 f s k iy0))
("szanto" nil (sh ae1 n t ow0))
("szarek" nil (sh aa1 r eh0 k))
("szatkowski" nil (sh ax t k ao1 f s k iy0))
("szczech" nil (sh eh1 k))
("szczepaniak" nil (sh ih0 p ae1 n iy0 ae0 k))
("szczepanik" nil (sh ih0 p ae1 n ih0 k))
("szczepanski" nil (sh ih0 p ae1 n s k iy0))
("szczerba" nil (sh er1 b ax))
("szczesniak" nil (sh eh1 z n iy0 ae0 k))
("szczesny" nil (sh eh1 z n iy0))
("szczygiel" nil (sh ih0 g iy1 l))
("sze" nil (sh iy1))
("szekely" nil (sh iy1 k l iy0))
("szeliga" nil (sh ih0 l ay1 g ax))
("szeto" nil (sh iy1 t ow0))
("szewczyk" nil (sh uw1 ch ih0 k))
("szilagyi" nil (sh ih0 l aa1 g iy0))
("szoke" nil (sh ow1 k))
("szostak" nil (sh aa1 s t ax k))
("szostek" nil (sh aa1 s t eh0 k))
("szot" nil (sh aa1 t))
("szott" nil (sh aa1 t))
("szuba" nil (sh uw1 b ax))
("szuch" nil (sh ah1 ch))
("szucs" nil (sh ah1 k s))
("szumski" nil (sh ah1 m s k iy0))
("szwed" nil (sh v eh1 d))
("szydlowski" nil (sh ih0 d l ao1 f s k iy0))
("szymanowski" nil (sh ih0 m ax n ao1 f s k iy0))
("szymanski" nil (sh ih0 m ae1 n s k iy0))
("szymborski" nil (sh ih0 m b ao1 r s k iy0))
("szymczak" nil (sh ih1 m ch ae0 k))
("t" nil (t iy1))
("ta" nil (t aa1))
("taaffe" nil (t aa1 f))
("tab" nil (t ae1 b))
("tabacalera" nil (t ax b ae1 k ax l eh1 r ax))
("tabachneck" nil (t ax b aa1 ch n eh1 k))
("tabak" nil (t ae1 b ae0 k))
("tabaka" nil (t aa0 b aa1 k ax))
("tabar" nil (t aa0 b aa1 r))
("tabares" nil (t aa0 b aa1 r eh0 s))
("tabasco" nil (t ax b ae1 s k ow0))
("tabb" nil (t ae1 b))
("tabbert" nil (t ae1 b er0 t))
("tabbing" nil (t ae1 b ih0 ng))
("tabby" nil (t ae1 b iy0))
("taber" nil (t ey1 b er0))
("tabernacle" nil (t ae1 b er0 n ae1 k ax l))
("tabitha" nil (t ae1 b ih0 th ax))
("tablature" nil (t ae1 b l ax ch er0))
("table" nil (t ey1 b ax l))
("tableau" nil (t ax b l ow1))
("tableaux" nil (t ax b l ow1))
("tablecloth" nil (t ey1 b ax l k l ao1 th))
("tablecloths" nil (t ey1 b ax l k l ao1 th s))
("tabled" nil (t ey1 b ax l d))
("tabler" nil (t ey1 b ax l er0))
("tables" nil (t ey1 b ax l z))
("tablespoon" nil (t ey1 b ax l s p uw1 n))
("tablespoons" nil (t ey1 b ax l s p uw1 n z))
("tablet" nil (t ae1 b l ax t))
("tabletop" nil (t ey1 b ax l t aa1 p))
("tablets" nil (t ae1 b l ax t s))
("tableware" nil (t ey1 b ax l w eh1 r))
("tabling" nil (t ey1 b ax l ih0 ng))
("tabloid" nil (t ae1 b l oy0 d))
("tabloids" nil (t ae1 b l oy0 d z))
("tabone" nil (t aa1 b ow0 n))
("taboo" nil (t ae0 b uw1))
("taboos" nil (t ae0 b uw1 z))
("tabor" nil (t ey1 b er0))
("taborn" nil (t ae1 b er0 n))
("tabron" nil (t ae1 b r ax n))
("tabs" nil (t ae1 b z))
("tabuchi" nil (t aa1 b uw1 ch iy0))
("tabulate" nil (t ae1 b y ax l ey1 t))
("tabulated" nil (t ae1 b y ax l ey1 t ih0 d))
("tabulating" nil (t ae1 b y ax l ey1 t ih0 ng))
("tabulation" nil (t ae1 b y ax l ey1 sh ax n))
("tabulations" nil (t ae1 b y ax l ey1 sh ax n z))
("tabulature" nil (t ae1 b y ax l ax ch er0))
("tac" nil (t ae1 k))
("tacey" nil (t ey1 s iy0))
("tacit" nil (t ae1 s ih0 t))
("tacita" nil (t aa0 ch iy1 t ax))
("tacitly" nil (t ae1 s ih0 t l iy0))
("taciturn" nil (t ae1 s ih0 t er1 n))
("tack" nil (t ae1 k))
("tacke" nil (t ae1 k))
("tacked" nil (t ae1 k t))
("tacker" nil (t ae1 k er0))
("tackett" nil (t ae1 k ih0 t))
("tacking" nil (t ae1 k ih0 ng))
("tackitt" nil (t ae1 k ih0 t))
("tackle" nil (t ae1 k ax l))
("tackled" nil (t ae1 k ax l d))
("tackles" nil (t ae1 k ax l z))
("tackling" nil (t ae1 k l ih0 ng))
("tacks" nil (t ae1 k s))
("tacky" nil (t ae1 k iy0))
("taco" nil (t aa1 k ow0))
("tacoma" nil (t ax k ow1 m ax))
("tacos" nil (t aa1 k ow0 z))
("tact" nil (t ae1 k t))
("tactful" nil (t ae1 k t f ax l))
("tactfully" nil (t ae1 k t f ax l iy0))
("tactic" nil (t ae1 k t ih0 k))
("tactical" nil (t ae1 k t ih0 k ax l))
("tactically" nil (t ae1 k t ih0 k ax l iy0))
("tactician" nil (t ae0 k t ih1 sh ax n))
("tacticians" nil (t ae0 k t ih1 sh ax n z))
("tactics" nil (t ae1 k t ih0 k s))
("tacy" nil (t ey1 s iy0))
("tad" nil (t ae1 d))
("tada" nil (t aa1 d ax))
("tadashi" nil (t aa1 d aa1 sh iy0))
("tadd" nil (t ae1 d))
("taddei" nil (t ae1 d ay0))
("taddeo" nil (t aa1 d iy0 ow0))
("taddy" nil (t ae1 d iy0))
("tadeusz" nil (t ae1 d iy0 uw0 z))
("tadlock" nil (t ae1 d l ax k))
("tadpole" nil (t ae1 d p ow1 l))
("tadpoles" nil (t ae1 d p ow1 l z))
("tadros" nil (t ey1 d r ow0 z))
("tadych" nil (t aa1 d ih0 hh))
("tae" nil (t ey1))
("taegu" nil (t ey1 g uw0))
("taekwondo" nil (t ae1 k w aa1 n d ow1))
("tafaro" nil (t ax f aa1 r ow0))
("tafel" nil (t ae1 f ax l))
("taff" nil (t ae1 f))
("taffe" nil (t ae1 f))
("taffeta" nil (t ae1 f ax t ax))
("taffy" nil (t ae1 f iy0))
("taflinger" nil (t ey1 f ax l ih0 ng er0))
("tafolla" nil (t ax f aa1 l ax))
("tafoya" nil (t aa0 f oy1 ax))
("taft" nil (t ae1 f t))
("tag" nil (t ae1 g))
("tagalog" nil (t ae1 g ax l aa1 g))
("tagamet" nil (t ae1 g ax m eh1 t))
("taganka" nil (t ax g aa1 ng k ax))
("tager" nil (t ey1 g er0))
("tagg" nil (t ae1 g))
("taggart" nil (t ae1 g er0 t))
("tagge" nil (t ae1 g))
("tagged" nil (t ae1 g d))
("taggert" nil (t ae1 g er0 t))
("tagging" nil (t ae1 g ih0 ng))
("tagle" nil (t ey1 g ax l))
("tagliaferri" nil (t aa0 g l y aa0 f eh1 r iy0))
("taglieri" nil (t aa0 g l ih1 r iy0))
("tagline" nil (t ae1 g l ay1 n))
("tagout" nil (t ae1 g aw1 t))
("tags" nil (t ae1 g z))
("tague" nil (t aa1 g))
("taher" nil (t ey1 er0))
("tahiti" nil (t ax hh iy1 t iy0))
("tahmassebi" nil (t aa1 m ax s iy1 b iy0))
("tahoe" nil (t ae1 hh ow0))
("tai" nil (t ay1))
("taibi" nil (t ey1 b iy0))
("taiko" nil (t ey1 k ow0))
("tail" nil (t ey1 l))
("tailed" nil (t ey1 l d))
("tailgate" nil (t ey1 l g ey1 t))
("tailhook" nil (t ey1 l hh uh1 k))
("tailing" nil (t ey1 l ih0 ng))
("tailings" nil (t ey1 l ih0 ng z))
("taillon" nil (t ey1 l ax n))
("tailor" nil (t ey1 l er0))
("tailored" nil (t ey1 l er0 d))
("tailoring" nil (t ey1 l er0 ih0 ng))
("tailors" nil (t ey1 l er0 z))
("tailpipe" nil (t ey1 l p iy1 p))
("tails" nil (t ey1 l z))
("tailspin" nil (t ey1 l s p ih1 n))
("tainer" nil (t ey1 n er0))
("taing" nil (t aa1 ih0 ng))
("taint" nil (t ey1 n t))
("tainted" nil (t ey1 n t ih0 d))
("tainter" nil (t ey1 n t er0))
("tainting" nil (t ey1 n t ih0 ng))
("taints" nil (t ey1 n t s))
("taipei" nil (t ay1 p ey1))
("taira" nil (t aa0 ih1 r ax))
("taisei" nil (t ey1 s ey1))
("taisho" nil (t ey1 sh ow0))
("tait" nil (t ey1 t))
("taite" nil (t ey1 t))
("taitt" nil (t ey1 t))
("taiwan" nil (t ay1 w aa1 n))
("taiwanese" nil (t ay1 w aa0 n iy1 z))
("taiyo" nil (t ay1 y ow0))
("taj" nil (t aa1 zh))
("tajikistan" nil (t aa1 jh iy1 k ih0 s t ae1 n))
("tajima" nil (t aa1 jh iy1 m ax))
("tak" nil (t ae1 k))
("takach" nil (t ae1 k ax k))
("takacs" nil (t ae1 k ax k s))
("takagi" nil (t aa0 k aa1 g iy0))
("takahashi" nil (t aa0 k aa0 hh aa1 sh iy0))
("takaki" nil (t aa0 k aa1 k iy0))
("takako" nil (t aa1 k aa1 k ow0))
("takanashi" nil (t aa1 k aa1 n aa1 sh iy0))
("takao" nil (t aa1 k aa1 ow0))
("takara" nil (t aa0 k aa1 r ax))
("takasago" nil (t aa1 k aa0 s aa1 g ow0))
("takashi" nil (t aa1 k aa1 sh iy0))
("takashima" nil (t aa1 k aa0 sh iy1 m ax))
("takashimaya" nil (t aa1 k aa1 sh ih0 m aa1 y ax))
("takata" nil (t aa0 k aa1 t ax))
("takayama" nil (t aa0 k aa0 y aa1 m ax))
("take" nil (t ey1 k))
("takecare" nil (t ey1 k eh1 r))
("takeda" nil (t aa0 k ey1 d ax))
("takei" nil (t aa1 k ey1))
("takemoto" nil (t aa0 k ey0 m ow1 t ow0))
("takemura" nil (t aa1 k ey0 m uh1 r ax))
("taken" nil (t ey1 k ax n))
("takeo" nil (t aa1 k ey1 ow0))
("takeoff" nil (t ey1 k ao1 f))
("takeoffs" nil (t ey1 k ao1 f s))
("takeout" nil (t ey1 k aw1 t))
("takeover" nil (t ey1 k ow1 v er0))
("takeovers" nil (t ey1 k ow1 v er0 z))
("taker" nil (t ey1 k er0))
("takers" nil (t ey1 k er0 z))
("takes" nil (t ey1 k s))
("takeshi" nil (t ax k eh1 sh iy0))
("takeshima" nil (t ae1 k ih0 sh iy1 m ax))
("takeshita" nil (t aa1 k ax sh iy1 t ax))
("taketa" nil (t aa0 k ey1 t ax))
("taketomi" nil (t aa1 k ih0 t ow1 m iy0))
("takeuchi" nil (t aa1 k ey0 uw1 ch iy0))
("taki" nil (t ae1 k iy0))
("takihyo" nil (t ax k iy1 y ow0))
("taking" nil (t ey1 k ih0 ng))
("takings" nil (t ey1 k ih0 ng z))
("takover" nil (t ae1 k ow0 v er0))
("taku" nil (t aa1 k uw1))
("takushoku" nil (t aa1 k ax sh ow1 k uw0))
("talaga" nil (t aa0 l aa1 g ax))
("talal" nil (t ax l aa1 l))
("talamantes" nil (t aa0 l aa0 m aa1 n t eh0 s))
("talamantez" nil (t aa0 l aa0 m aa1 n t eh0 z))
("talamo" nil (t aa0 l aa1 m ow0))
("talarico" nil (t aa0 l aa0 r iy1 k ow0))
("talavera" nil (t aa0 l aa0 v eh1 r ax))
("talbert" nil (t ae1 l b er0 t))
("talbot" nil (t ae1 l b ax t))
("talbots" nil (t ae1 l b ax t s))
("talbott" nil (t ae1 l b ax t))
("talc" nil (t ae1 l k))
("talcott" nil (t ae1 l k ax t))
("talcs" nil (t ae1 l k s))
("talcum" nil (t ae1 l k ax m))
("tale" nil (t ey1 l))
("talent" nil (t ae1 l ax n t))
("talented" nil (t ae1 l ax n t ax d))
("talents" nil (t ae1 l ax n t s))
("talerico" nil (t aa0 l er0 iy1 k ow0))
("tales" nil (t ey1 l z))
("talford" nil (t ae1 l f er0 d))
("taligent" nil (t ae1 l ih0 jh eh0 n t))
("talisman" nil (t ae1 l ih0 s m ax n))
("talitha" nil (t ae1 l ih0 dh ax))
("talk" nil (t ao1 k))
("talkative" nil (t ao1 k ax t ih0 v))
("talkback" nil (t ao1 k b ae1 k))
("talked" nil (t ao1 k t))
("talker" nil (t ao1 k er0))
("talkers" nil (t ao1 k er0 z))
("talkie" nil (t ao1 k iy0))
("talkies" nil (t ao1 k iy0 z))
("talkin" nil (t aa1 k ax n))
("talking" nil (t ao1 k ih0 ng))
("talkington" nil (t ao1 k ih0 ng t ax n))
("talks" nil (t ao1 k s))
("talky" nil (t ao1 k iy0))
("tall" nil (t ao1 l))
("tallahassee" nil (t ae1 l ax hh ae1 s iy0))
("tallant" nil (t aa1 l ax n t))
("tallarico" nil (t aa0 l aa0 r iy1 k ow0))
("tallent" nil (t aa1 l ax n t))
("taller" nil (t ao1 l er0))
("tallerico" nil (t aa0 l er0 iy1 k ow0))
("tallest" nil (t ao1 l ih0 s t))
("talley" nil (t ae1 l iy0))
("tallie" nil (t ao1 l iy0))
("tallied" nil (t ae1 l iy0 d))
("tallies" nil (t ae1 l iy0 z))
("tallmadge" nil (t ae1 l m ae0 jh))
("tallman" nil (t ao1 l m ax n))
("tallo" nil (t ae1 l ow0))
("tallon" nil (t ae1 l ax n))
("tallow" nil (t ae1 l ow0))
("tallula" nil (t aa0 l uw1 l ax))
("tallulah" nil (t ae1 l uw1 l ax))
("tally" nil (t ae1 l iy0))
("tallyho" nil (t ae1 l iy0 hh ow1))
("tallying" nil (t ae1 l iy0 ih0 ng))
("talmadge" nil (t ae1 l m ae0 jh))
("talmage" nil (t ae1 l m ih0 jh))
("talman" nil (t ae1 l m ax n))
("talmor" nil (t ae1 l m ao1 r))
("talmud" nil (t ae1 l m ax d))
("talon" nil (t ae1 l ax n))
("talsma" nil (t aa1 l s m ax))
("talton" nil (t ae1 l t ax n))
("talty" nil (t ao1 l t iy0))
("tam" nil (t ae1 m))
("tama" nil (t aa1 m ax))
("tamales" nil (t ax m aa1 l iy0 z))
("tamanaha" nil (t aa0 m aa0 n aa1 hh ax))
("tamara" nil (t ax m aa1 r ax))
("tamarac" nil (t ae1 m er0 ae0 k))
("tamargo" nil (t aa0 m aa1 r g ow0))
("tamarin" nil (t ae1 m er0 ax n))
("tamarisks" nil (t ae1 m er0 ih0 s k s))
("tamas" nil (t aa1 m ax z))
("tamashiro" nil (t aa0 m aa0 sh ih1 r ow0))
("tamayo" nil (t aa0 m aa1 y ow0))
("tamblyn" nil (t ae1 m b l ih0 n))
("tambo" nil (t ae1 m b ow0))
("tambourine" nil (t ae1 m b er0 iy1 n))
("tambrands" nil (t ae1 m b r ae1 n d z))
("tambs" nil (t ae1 m z))
("tamburello" nil (t aa0 m b uh0 r eh1 l ow0))
("tamburo" nil (t aa0 m b uh1 r ow0))
("tamburri" nil (t aa0 m b uh1 r iy0))
("tamburrino" nil (t aa0 m b uh0 r iy1 n ow0))
("tamburro" nil (t aa0 m b uh1 r ow0))
("tame" nil (t ey1 m))
("tamed" nil (t ey1 m d))
("tamer" nil (t ey1 m er0))
("tames" nil (t ey1 m z))
("tamez" nil (t aa0 m eh1 z))
("tamil" nil (t ae1 m ax l))
("tamils" nil (t ae1 m ax l z))
("taming" nil (t ey1 m ih0 ng))
("tamke" nil (t ae1 m k iy0))
("tamm" nil (t ae1 m))
("tammany" nil (t ae1 m ax n iy0))
("tammaro" nil (t aa0 m aa1 r ow0))
("tammen" nil (t ae1 m ax n))
("tammie" nil (t ae1 m iy0))
("tamminga" nil (t aa0 m iy1 ng g ax))
("tammy" nil (t ae1 m iy0))
("tamoxifen" nil (t ae1 m aa1 k s ax f ax n))
("tampa" nil (t ae1 m p ax))
("tampax" nil (t ae1 m p ae1 k s))
("tamper" nil (t ae1 m p er0))
("tampered" nil (t ae1 m p er0 d))
("tampering" nil (t ae1 m p er0 ih0 ng))
("tamplin" nil (t ae1 m p l ih0 n))
("tampon" nil (t ae1 m p aa0 n))
("tampons" nil (t ae1 m p aa0 n z))
("tampopo" nil (t ae1 m p ow1 p ow0))
("tamposi" nil (t ae1 m p ow1 s iy0))
("tamura" nil (t aa0 m uh1 r ax))
("tan" nil (t ae1 n))
("tanabe" nil (t aa0 n aa1 b ey0))
("tanaka" nil (t aa0 n aa1 k ax))
("tancredi" nil (t aa0 n k r eh1 d iy0))
("tancredo" nil (t ae1 n k r ey1 d ow0))
("tandem" nil (t ae1 n d ax m))
("tandon" nil (t ae1 n d ax n))
("tandy" nil (t ae1 n d iy0))
("tandycrafts" nil (t ae1 n d iy0 k r ae1 f t s))
("taneja" nil (t ax n ey1 hh ax))
("tanen" nil (t ae1 n ax n))
("tanenbaum" nil (t ae1 n ax n b aw1 m))
("taney" nil (t ey1 n iy0))
("tang" nil (t ae1 ng))
("tange" nil (t ae1 n jh))
("tangeman" nil (t ey1 n jh m ax n))
("tangen" nil (t ae1 ng ax n))
("tangent" nil (t ae1 n jh ax n t))
("tangential" nil (t ae0 n jh eh1 n ch ax l))
("tanger" nil (t ae1 ng er0))
("tangerine" nil (t ae1 n jh er0 iy1 n))
("tangerines" nil (t ae1 n jh er0 iy1 n z))
("tangible" nil (t ae1 n jh ax b ax l))
("tangibles" nil (t ae1 n jh ax b ax l z))
("tangiers" nil (t ae1 n jh ih1 r z))
("tangle" nil (t ae1 ng g ax l))
("tangled" nil (t ae1 ng g ax l d))
("tangles" nil (t ae1 ng g ax l z))
("tanglewood" nil (t ae1 ng g ax l w uh1 d))
("tangney" nil (t ae1 ng g n iy0))
("tango" nil (t ae1 ng g ow0))
("tangos" nil (t ae1 ng g ow0 z))
("tangredi" nil (t aa0 ng g r eh1 d iy0))
("tangs" nil (t ae1 ng z))
("tanguay" nil (t ae1 n g ey0))
("tanguma" nil (t aa0 ng g uw1 m ax))
("tani" nil (t aa1 n iy0))
("tania" nil (t aa1 n y ax))
("taniguchi" nil (t aa1 n ih0 g uw1 ch iy0))
("tanimoto" nil (t aa0 n iy0 m ow1 t ow0))
("tanimura" nil (t aa1 n ih0 m uw1 r ax))
("tanis" nil (t ae1 n ih0 s))
("tanjug" nil (t ae1 n jh ah1 g))
("tank" nil (t ae1 ng k))
("tankan" nil (t ae1 ng k ax n))
("tankard" nil (t ae1 ng k er0 d))
("tanke" nil (t ae1 ng k))
("tanked" nil (t ae1 ng k t))
("tanker" nil (t ae1 ng k er0))
("tankers" nil (t ae1 ng k er0 z))
("tankersley" nil (t ae1 ng k er0 s l iy0))
("tanks" nil (t ae1 ng k s))
("tanksley" nil (t ae1 ng k s l iy0))
("tann" nil (t ae1 n))
("tannahill" nil (t ae1 n ax hh ih1 l))
("tanned" nil (t ae1 n d))
("tannehill" nil (t ae1 n hh ih0 l))
("tannen" nil (t ae1 n ax n))
("tannenbaum" nil (t ae1 n ax n b aw1 m))
("tanner" nil (t ae1 n er0))
("tanney" nil (t ae1 n iy0))
("tannhauser" nil (t ae1 n hh aw1 z er0))
("tanning" nil (t ae1 n ih0 ng))
("tanqueray" nil (t ae1 ng k w er0 ey1))
("tans" nil (t ae1 n z))
("tansey" nil (t ae1 n z iy0))
("tanski" nil (t ae1 n s k iy1))
("tansy" nil (t ae1 n z iy0))
("tant" nil (t ae1 n t))
("tantalized" nil (t ae1 n t ax l ay1 z d))
("tantalizing" nil (t ae1 n t ax l ay1 z ih0 ng))
("tantalizingly" nil (t ae1 n t ax l ay1 z ih0 ng l iy0))
("tantalum" nil (t ae1 n t ax l ax m))
("tantalus" nil (t ae1 n t ax l ax s))
("tantamount" nil (t ae1 n t ax m aw1 n t))
("tantillo" nil (t aa0 n t ih1 l ow0))
("tanton" nil (t ae1 n t ax n))
("tantric" nil (t ae1 n t r ih0 k))
("tantrum" nil (t ae1 n t r ax m))
("tantrums" nil (t ae1 n t r ax m z))
("tanya" nil (t aa1 n y ax))
("tanzania" nil (t ae0 n z ey1 n iy0 ax))
("tanzanian" nil (t ae1 n z ey1 n iy0 ax n))
("tanzer" nil (t ae1 n z er0))
("tanzi" nil (t ae1 n z iy0))
("tao" nil (t aw1))
("taoism" nil (d aw1 ih0 z ax m))
("taoists" nil (d aw1 ax s t s))
("taormina" nil (t aa0 ao0 r m iy1 n ax))
("taos" nil (t aw1 z))
("tap" nil (t ae1 p))
("tapas" nil (t ah1 p ax s))
("tape" nil (t ey1 p))
("taped" nil (t ey1 p t))
("tapeie" nil (t ey1 p iy0 ey0))
("taper" nil (t ey1 p er0))
("tapered" nil (t ey1 p er0 d))
("tapering" nil (t ey1 p er0 ih0 ng))
("tapers" nil (t ey1 p er0 z))
("tapes" nil (t ey1 p s))
("tapestries" nil (t ae1 p ax s t r iy0 z))
("tapestry" nil (t ae1 p ax s t r iy0))
("tapia" nil (t aa1 p iy0 ax))
("tapias" nil (t ax p iy1 ax s))
("tapie" nil (t ae1 p iy0))
("taping" nil (t ey1 p ih0 ng))
("tapley" nil (t ae1 p l iy0))
("taplin" nil (t ae1 p l ih0 n))
("tapp" nil (t ae1 p))
("tappan" nil (t ae1 p ax n))
("tappe" nil (t ae1 p))
("tapped" nil (t ae1 p t))
("tappen" nil (t ae1 p ax n))
("tapper" nil (t ae1 p er0))
("tappet" nil (t ae1 p ax t))
("tappin" nil (t ae1 p ih0 n))
("tapping" nil (t ae1 p ih0 ng))
("taps" nil (t ae1 p s))
("tapscott" nil (t ae1 p s k ax t))
("tar" nil (t aa1 r))
("tara" nil (t eh1 r ax))
("tarallo" nil (t er0 ae1 l ow0))
("tarango" nil (t aa0 r aa1 ng g ow0))
("tarantino" nil (t aa0 r aa0 n t iy1 n ow0))
("taranto" nil (t er0 ae1 n t ow0))
("taras" nil (t aa1 r ax z))
("tarasoff" nil (t eh1 r ih0 s ao0 f))
("taravella" nil (t ae1 r ax v eh1 l ax))
("tarbell" nil (t aa1 r b eh1 l))
("tarbert" nil (t aa1 r b er0 t))
("tarbet" nil (t aa1 r b ih0 t))
("tarbox" nil (t aa1 r b aa1 k s))
("tarbutton" nil (t aa1 r b ah1 t ax n))
("tardif" nil (t aa1 r d ih0 f))
("tardiff" nil (t aa1 r d ih0 f))
("tardiness" nil (t aa1 r d iy0 n ax s))
("tarditi" nil (t aa1 r d ih1 t iy0))
("tardy" nil (t aa1 r d iy0))
("target" nil (t aa1 r g ax t))
("targeted" nil (t aa1 r g ax t ih0 d))
("targeting" nil (t aa1 r g ax t ih0 ng))
("targets" nil (t aa1 r g ax t s))
("tarheels" nil (t aa1 r hh iy1 l z))
("tariff" nil (t eh1 r ax f))
("tariffs" nil (t eh1 r ax f s))
("tarin" nil (t aa1 r iy0 n))
("tariq" nil (t aa1 r iy1 k))
("tarkington" nil (t aa1 r k ih0 ng t ax n))
("tarkowski" nil (t er0 k ao1 f s k iy0))
("tarleton" nil (t aa1 r l t ax n))
("tarlow" nil (t aa1 r l ow1))
("tarlton" nil (t aa1 r l t ax n))
("tarmac" nil (t aa1 r m ae1 k))
("tarmacs" nil (t aa1 r m ae1 k s))
("tarman" nil (t aa1 r m ax n))
("tarnish" nil (t aa1 r n ih0 sh))
("tarnished" nil (t aa1 r n ih0 sh t))
("tarnishing" nil (t aa1 r n ih0 sh ih0 ng))
("tarnoff" nil (t aa1 r n ao1 f))
("tarnow" nil (t aa1 r n ow0))
("tarnowski" nil (t er0 n ao1 f s k iy0))
("taro" nil (t eh1 r ow0))
("tarom" nil (t eh1 r ax m))
("tarp" nil (t aa1 r p))
("tarpey" nil (t aa1 r p iy0))
("tarpley" nil (t aa1 r p l iy0))
("tarpon" nil (t aa1 r p ax n))
("tarps" nil (t aa1 r p s))
("tarquinio" nil (t aa0 r k w iy1 n iy0 ow0))
("tarr" nil (t aa1 r))
("tarrance" nil (t aa1 r ax n s))
("tarrant" nil (t aa1 r ax n t))
("tarred" nil (t aa1 r d))
("tarring" nil (t aa1 r ih0 ng))
("tarry" nil (t eh1 r iy0))
("tarrytown" nil (t eh1 r iy0 t aw1 n))
("tarses" nil (t aa1 r s ih0 s))
("tarsus" nil (t aa1 r s ax s))
("tart" nil (t aa1 r t))
("tartaglia" nil (t aa0 r t ae1 g l iy0 ax))
("tartaglione" nil (t aa0 r t ae0 g l iy0 ow1 n iy0))
("tartan" nil (t aa1 r t ax n))
("tartar" nil (t aa1 r t er0))
("tarte" nil (t aa1 r t))
("tarter" nil (t aa1 r t er0))
("tartikoff" nil (t aa1 r t ih0 k ao1 f))
("tartness" nil (t aa1 r t n ax s))
("tarts" nil (t aa1 r t s))
("tartt" nil (t aa1 r t))
("tarver" nil (t aa1 r v er0))
("tarvin" nil (t aa1 r v ih0 n))
("tarzan" nil (t aa1 r z ae1 n))
("tasca" nil (t aa1 s k ax))
("tasch" nil (t ae1 sh))
("taschner" nil (t ae1 sh n er0))
("tash" nil (t ae1 sh))
("tashiro" nil (t aa0 sh ih1 r ow0))
("tashjian" nil (t ae1 sh jh iy0 ax n))
("tashkent" nil (t ae1 sh k eh1 n t))
("task" nil (t ae1 s k))
("tasker" nil (t ae1 s k er0))
("tasking" nil (t ae1 s k ih0 ng))
("taskmaster" nil (t ae1 s k m ae1 s t er0))
("tasks" nil (t ae1 s k s))
("tasm" nil (t ae1 s ax m))
("tasman" nil (t ae1 z m ax n))
("tasmania" nil (t ae1 z m ey1 n iy0 ax))
("tasmanian" nil (t ae1 z m ey1 n iy0 ax n))
("tass" nil (t ae1 s))
("tassel" nil (t ae1 s ax l))
("tassi" nil (t ae1 s iy0))
("tassin" nil (t ae1 s ih0 n))
("tassinari" nil (t aa0 s iy0 n aa1 r iy0))
("tasso" nil (t ae1 s ow1))
("tassone" nil (t aa0 s ow1 n iy0))
("taste" nil (t ey1 s t))
("tasted" nil (t ey1 s t ax d))
("tasteful" nil (t ey1 s t f ax l))
("tastefully" nil (t ey1 s t f ax l iy0))
("tasteless" nil (t ey1 s t l ax s))
("tasters" nil (t ey1 s t er0 z))
("tastes" nil (t ey1 s t s))
("tastier" nil (t ey1 s t iy0 er0))
("tastiness" nil (t ey1 s t iy0 n ax s))
("tasting" nil (t ey1 s t ih0 ng))
("tastings" nil (t ey1 s t ih0 ng z))
("tasty" nil (t ey1 s t iy0))
("tat" nil (t ae1 t))
("tata" nil (t aa1 t ax))
("tatar" nil (t ae1 t er0))
("tatars" nil (t ae1 t er0 z))
("tatarstan" nil (t aa1 t er0 s t ae1 n))
("tate" nil (t ey1 t))
("tateho" nil (t ax t ey1 hh ow0))
("tatem" nil (t ae1 t ih0 m))
("tatge" nil (t ey1 t jh))
("tatham" nil (t ae1 th ax m))
("tatiana" nil (t ae1 t iy1 ae1 n ax))
("tatlock" nil (t ae1 t l aa1 k))
("tatman" nil (t ae1 t m ax n))
("tatom" nil (t ae1 t ax m))
("tatra" nil (t ae1 t r ax))
("tatro" nil (t ae1 t r ow0))
("tatsch" nil (t ae1 ch))
("tatsukichi" nil (t ax t s uw0 k iy1 ch iy0))
("tatsuno" nil (t ae1 t s uw1 n ow0))
("tattered" nil (t ae1 t er0 d))
("tatters" nil (t ae1 t er0 z))
("tattersall" nil (t ae1 t er0 s ax l))
("tattled" nil (t ae1 t ax l d))
("tattler" nil (t ae1 t l er0))
("tattoo" nil (t ae0 t uw1))
("tattooed" nil (t ae0 t uw1 d))
("tattooing" nil (t ae0 t uw1 ih0 ng))
("tattoos" nil (t ae0 t uw1 z))
("tatty" nil (t ae1 t iy0))
("tatu" nil (t aa0 t uw1))
("tatum" nil (t ey1 t ax m))
("taub" nil (t aw1 b))
("taube" nil (t ao1 b))
("tauber" nil (t aw1 b er0))
("taubert" nil (t aw1 b er0 t))
("taubes" nil (t aw1 b z))
("taubman" nil (t aw1 b m ax n))
("taucher" nil (t aw1 k er0))
("tauer" nil (t aw1 er0))
("taught" nil (t ao1 t))
("tauke" nil (t aw1 k))
("taul" nil (t ao1 l))
("taulbee" nil (t ao1 l b iy1))
("taunt" nil (t ao1 n t))
("taunted" nil (t ao1 n t ih0 d))
("taunting" nil (t ao1 n t ih0 ng))
("taunton" nil (t ao1 n t ax n))
("taunts" nil (t ao1 n t s))
("taurus" nil (t ao1 r ax s))
("tauruses" nil (t ao1 r ax s ax z))
("tausch" nil (t aw1 sh))
("tauscher" nil (t aw1 sh er0))
("taussig" nil (t aw1 s ih0 g))
("taut" nil (t ao1 t))
("tauzin" nil (t aw1 z ih0 n))
("tavano" nil (t aa0 v aa1 n ow0))
("tavares" nil (t aa0 v aa1 r eh0 s))
("tavarez" nil (t aa0 v aa1 r eh0 z))
("tavel" nil (t aa0 v eh1 l))
("tavenner" nil (t ae1 v ih0 n er0))
("tavera" nil (t aa0 v eh1 r ax))
("taveras" nil (t aa0 v eh1 r aa0 z))
("tavern" nil (t ae1 v er0 n))
("taverna" nil (t aa0 v eh1 r n ax))
("tavernier" nil (t ae1 v er0 n iy0 er0))
("taverns" nil (t ae1 v er0 n z))
("taves" nil (t ey1 v z))
("tavie" nil (t ey1 v iy0))
("tavis" nil (t aa1 v ih0 s))
("tavish" nil (t ey1 v ih0 sh))
("tavist" nil (t ae1 v ih0 s t))
("tavoulareas" nil (t aa0 v uw0 l eh1 r iy0 ax s))
("tavy" nil (t ey1 v iy0))
("tawana" nil (t aa1 w aa1 n ax))
("tawdry" nil (t ao1 d r iy0))
("tawes" nil (t ao1 z))
("tawil" nil (t ao1 ax l))
("tawney" nil (t ao1 n iy0))
("tax" nil (t ae1 k s))
("taxability" nil (t ae1 k s ax b ih1 l ih0 t iy0))
("taxable" nil (t ae1 k s ax b ax l))
("taxables" nil (t ae1 k s ax b ax l z))
("taxation" nil (t ae0 k s ey1 sh ax n))
("taxcut" nil (t ae1 k s k ah1 t))
("taxed" nil (t ae1 k s t))
("taxer" nil (t ae1 k s er0))
("taxes" nil (t ae1 k s ax z))
("taxi" nil (t ae1 k s iy0))
("taxicab" nil (t ae1 k s iy0 k ae1 b))
("taxied" nil (t ae1 k s iy0 d))
("taxiing" nil (t ae1 k s iy0 ih0 ng))
("taxing" nil (t ae1 k s ih0 ng))
("taxis" nil (t ae1 k s iy0 z))
("taxol" nil (t ae1 k s aa1 l))
("taxpayer" nil (t ae1 k s p ey1 er0))
("taxpayers" nil (t ae1 k s p ey1 er0 z))
("taxpaying" nil (t ae1 k s p ey1 ih0 ng))
("tay" nil (t ey1))
("tayler" nil (t ey1 l er0))
("tayloe" nil (t ey1 l ow0))
("taylor" nil (t ey1 l er0))
("taylors" nil (t ey1 l er0 z))
("tayman" nil (t ey1 m ax n))
("tays" nil (t ey1 z))
("tb" nil (t iy1 b iy1))
("tbilisi" nil (t ax b ih0 l iy1 s iy0))
("tcas" nil (t iy1 s iy1 ey1 eh1 s))
("tchaikovsky" nil (ch ey1 k aa1 v s k iy0))
("tchuruk" nil (ch uh1 r ih0 k))
("te" nil (t iy1))
("tea" nil (t iy1))
("teac" nil (t iy1 k))
("teach" nil (t iy1 ch))
("teachable" nil (t iy1 ch ax b ax l))
("teacher" nil (t iy1 ch er0))
("teachers" nil (t iy1 ch er0 z))
("teaches" nil (t iy1 ch ax z))
("teachey" nil (t iy1 ch iy0))
("teaching" nil (t iy1 ch ih0 ng))
("teachings" nil (t iy1 ch ih0 ng z))
("teachout" nil (t iy1 ch aw1 t))
("teacup" nil (t iy1 k ah1 p))
("teaford" nil (t iy1 f ao1 r d))
("teagarden" nil (t iy1 g aa1 r d ax n))
("teagle" nil (t iy1 g ax l))
("teague" nil (t iy1 g))
("teahan" nil (t iy1 ax n))
("teak" nil (t iy1 k))
("teakwood" nil (t iy1 k w uh1 d))
("teal" nil (t iy1 l))
("teale" nil (t iy1 l))
("teall" nil (t iy1 l))
("team" nil (t iy1 m))
("teamed" nil (t iy1 m d))
("teamer" nil (t iy1 m er0))
("teaming" nil (t iy1 m ih0 ng))
("teammate" nil (t iy1 m ey1 t))
("teammates" nil (t iy1 m m ey1 t s))
("teams" nil (t iy1 m z))
("teamster" nil (t iy1 m s t er0))
("teamsters" nil (t iy1 m s t er0 z))
("teamwork" nil (t iy1 m w er1 k))
("teaneck" nil (t iy1 n eh1 k))
("teaney" nil (t iy1 n iy0))
("teapot" nil (t iy1 p aa1 t))
("tear" v (t eh1 r))
("tear" n (t iy1 r))
("teare" nil (t iy1 r))
("tearful" nil (t ih1 r f ax l))
("tearfully" nil (t ih1 r f ax l iy0))
("tearing" nil (t eh1 r ih0 ng))
("tearle" nil (t ao1 r ax l))
("tears" v (t eh1 r z))
("tears" n (t ih1 r z))
("teary" nil (t ih1 r iy0))
("teas" nil (t iy1 z))
("teasdale" nil (t iy1 z d ey1 l))
("tease" nil (t iy1 z))
("teased" nil (t iy1 z d))
("teaser" nil (t iy1 z er0))
("teases" nil (t iy1 z ih0 z))
("teasing" nil (t iy1 z ih0 ng))
("teasley" nil (t iy1 z l iy0))
("teaspoon" nil (t iy1 s p uw1 n))
("teaspoons" nil (t iy1 s p uw1 n z))
("teaster" nil (t iy1 s t er0))
("teat" nil (t iy1 t))
("teate" nil (t iy1 ey1 t))
("teater" nil (t iy1 t er0))
("teatro" nil (t iy1 t r ow0))
("teats" nil (t iy1 t s))
("tebbe" nil (t eh1 b))
("tebben" nil (t eh1 b ax n))
("tebbetts" nil (t eh1 b ih0 t s))
("tebbit" nil (t eh1 b ih0 t))
("tebbs" nil (t eh1 b z))
("tebeau" nil (t ih0 b ow1))
("tebelskis" nil (t ax b eh1 l s k ih0 s))
("tebo" nil (t eh1 b ow0))
("tebuthiuron" nil (t eh1 b ax th y uw1 r aa0 n))
("tec" nil (t eh1 k))
("tech" nil (t eh1 k))
("techies" nil (t eh1 k iy0 z))
("techint" nil (t eh1 ch ih0 n t))
("technic" nil (t eh1 k n ih0 k))
("technical" nil (t eh1 k n ih0 k ax l))
("technicalities" nil (t eh1 k n ax k ae1 l ax t iy0 z))
("technicality" nil (t eh1 k n ax k ae1 l ih0 t iy0))
("technically" nil (t eh1 k n ih0 k ax l iy0))
("technician" nil (t eh0 k n ih1 sh ax n))
("technicians" nil (t eh0 k n ih1 sh ax n z))
("technicolor" nil (t eh1 k n ih0 k ah1 l er0))
("technicon" nil (t eh1 k n ih0 k aa1 n))
("technics" nil (t eh1 k n ih0 k s))
("technique" nil (t eh0 k n iy1 k))
("techniques" nil (t eh0 k n iy1 k s))
("technitrol" nil (t eh1 k n ih0 t r ow1 l))
("techno" nil (t eh1 k n ow0))
("technocrat" nil (t eh1 k n ax k r ae1 t))
("technocratic" nil (t eh1 k n ax k r ae1 t ih0 k))
("technocrats" nil (t eh1 k n ax k r ae1 t s))
("technodyne" nil (t eh1 k n ow0 d ay1 n))
("technologic" nil (t eh1 k n ax l aa1 jh ih0 k))
("technological" nil (t eh1 k n ax l aa1 jh ih0 k ax l))
("technologically" nil (t eh1 k n ax l aa1 jh ih0 k ax l iy0))
("technologies" nil (t eh0 k n aa1 l ax jh iy0 z))
("technologist" nil (t eh1 k n aa1 l ax jh ih0 s t))
("technologists" nil (t eh1 k n aa1 l ax jh ih0 s t s))
("technology" nil (t eh0 k n aa1 l ax jh iy0))
("technomic" nil (t eh1 k n aa1 m ih0 k))
("technophobe" nil (t eh1 k n ow0 f ow1 b))
("technophobes" nil (t eh1 k n ow0 f ow1 b z))
("techs" nil (t eh1 k s))
("techsystems" nil (t eh1 k s ih1 s t ax m z))
("techy" nil (t eh1 ch iy0))
("teck" nil (t eh1 k))
("tecla" nil (t eh1 k l ax))
("teco" nil (t iy1 k ow0))
("tecogen" nil (t eh1 k ow0 g ax n))
("tecos" nil (t iy1 k ow0 s))
("tectonics" nil (t eh0 k t aa1 n ih0 k s))
("tecumseh" nil (t ax k ah1 m s ax))
("ted" nil (t eh1 d))
("tedder" nil (t eh1 d er0))
("teddie" nil (t eh1 d iy0))
("teddy" nil (t eh1 d iy0))
("tedeschi" nil (t eh0 d eh1 s k iy0))
("tedesco" nil (t eh0 d eh1 s k ow0))
("tedford" nil (t eh1 d f er0 d))
("tedious" nil (t iy1 d iy0 ax s))
("tediously" nil (t iy1 d iy0 ax s l iy0))
("tedium" nil (t iy1 d iy0 ax m))
("tedman" nil (t eh1 d m ax n))
("tedmond" nil (t eh1 d m ax n d))
("tedmund" nil (t eh1 d m ax n d))
("tedrick" nil (t eh1 d r ih0 k))
("tedrow" nil (t eh1 d r ow1))
("tee" nil (t iy1))
("teed" nil (t iy1 d))
("teegarden" nil (t iy1 g aa1 r d ax n))
("teegardin" nil (t iy0 g aa1 r d ih0 n))
("teehan" nil (t iy1 ax n))
("teeing" nil (t iy1 ih0 ng))
("teel" nil (t iy1 l))
("teele" nil (t iy1 l))
("teeley" nil (t iy1 l iy0))
("teeling" nil (t iy1 l ih0 ng))
("teeming" nil (t iy1 m ih0 ng))
("teems" nil (t iy1 m z))
("teen" nil (t iy1 n))
("teena" nil (t iy1 n ax))
("teenage" nil (t iy1 n ey1 jh))
("teenaged" nil (t iy1 n ey1 jh d))
("teenager" nil (t iy1 n ey1 jh er0))
("teenagers" nil (t iy1 n ey1 jh er0 z))
("teenie" nil (t iy1 n iy0))
("teens" nil (t iy1 n z))
("teensy" nil (t iy1 n s iy0))
("teeny" nil (t iy1 n iy0))
("teeple" nil (t iy1 p ax l))
("teeples" nil (t iy1 p ax l z))
("teer" nil (t ih1 r))
("tees" nil (t iy1 z))
("teeter" nil (t iy1 t er0))
("teetered" nil (t iy1 t er0 d))
("teetering" nil (t iy1 t er0 ih0 ng))
("teeters" nil (t iy1 t er0 z))
("teeth" nil (t iy1 th))
("teething" nil (t iy1 dh ih0 ng))
("teets" nil (t iy1 t s))
("teffeteller" nil (t eh1 f ih0 t eh1 l er0))
("tefft" nil (t eh1 f t))
("teflon" nil (t eh1 f l ax n))
("tefra" nil (t eh1 f r ax))
("tegeler" nil (t eh1 g ax l er0))
("tegethoff" nil (t eh1 g ih0 t ao1 f))
("tegge" nil (t eh1 g))
("tegtmeier" nil (t eh1 t m ay0 er0))
("tegtmeyer" nil (t eh1 t m ay0 er0))
("tegucigalpa" nil (t eh0 g uw0 s iy0 g ae1 l p ax))
("tehan" nil (t ey1 ax n))
("teheran" nil (t eh1 hh er0 aa1 n))
("teherani" nil (t eh1 hh er0 aa1 n iy0))
("teheranis" nil (t eh1 hh er0 aa1 n iy0 z))
("teherans" nil (t eh1 hh er0 aa1 n z))
("tehran" nil (t ey1 r aa1 n))
("tehrani" nil (t ey1 r aa1 n iy0))
("tehranis" nil (t ey1 r aa1 n iy0 z))
("tehrans" nil (t ey1 r aa1 n z))
("teich" nil (t ay1 k))
("teicher" nil (t ay1 k er0))
("teichert" nil (t ay1 k er0 t))
("teichman" nil (t ay1 k m ax n))
("teichmann" nil (t ay1 k m ax n))
("teig" nil (t iy1 g))
("teige" nil (t ay1 jh))
("teigen" nil (t ay1 g ax n))
("teikoku" nil (t ey1 k ow1 k uw1))
("teitel" nil (t ay1 t ax l))
("teitelbaum" nil (t ay1 t ax l b aw0 m))
("teitell" nil (t ay1 t eh1 l))
("teixeira" nil (t ax k s eh1 r ax))
("tejada" nil (t ey0 y aa1 d ax))
("tejas" nil (t iy1 jh ax s))
("tejeda" nil (t ey0 y ey1 d ax))
("tejera" nil (t ey0 ih1 r ax))
("tejon" nil (t eh1 jh aa0 n))
("tek" nil (t eh1 k))
("teknowledge" nil (t eh1 k n aa1 l eh0 jh))
("tektronix" nil (t eh1 k t r aa1 n ih0 k s))
("tel" nil (t eh1 l))
("telaction" nil (t eh0 l ae1 k sh ax n))
("telamon" nil (t eh1 l ax m ax n))
("telco" nil (t eh1 l k ow0))
("tele" nil (t eh1 l iy0))
("telecable" nil (t eh1 l ax k ey1 b ax l))
("telecard" nil (t eh1 l ax k aa1 r d))
("telecast" nil (t eh1 l ax k ae1 s t))
("telecasts" nil (t eh1 l ax k ae1 s t s))
("telecharge" nil (t eh1 l ax ch aa1 r jh))
("telecheck" nil (t eh1 l ax ch eh1 k))
("telecom" nil (t eh1 l ax k aa0 m))
("telecommunication" nil (t eh1 l ax k ax m y uw1 n ih0 k ey1 sh ax n))
("telecommunications" nil (t eh1 l ax k ax m y uw1 n ax k ey1 sh ax n z))
("telecommuter" nil (t eh1 l ax k ax m y uw1 t er0))
("telecommuters" nil (t eh1 l ax k ax m y uw1 t er0 z))
("telecommuting" nil (t eh0 l ax k ax m y uw1 t ih0 ng))
("telecoms" nil (t eh1 l ax k aa0 m z))
("teleconference" nil (t eh0 l ax k aa1 n f r ax n s))
("teleconferencing" nil (t eh1 l ax k aa1 n f r ax n s ih0 ng))
("teleconnect" nil (t eh0 l ax k ax n eh1 k t))
("telecrafter" nil (t eh1 l ax k r ae1 f t er0))
("telecredit" nil (t eh1 l ax k r eh1 t ih0 t))
("telectron" nil (t eh1 l ax k t r aa0 n))
("telectronics" nil (t eh1 l ax k t r aa1 n ih0 k s))
("teledesic" nil (t eh1 l ax d eh1 s ih0 k))
("teledyne" nil (t eh1 l ax d ay1 n))
("teleflex" nil (t eh1 l ax f l eh1 k s))
("telefon" nil (t eh1 l ax f aa1 n))
("telefonica" nil (t eh1 l ax f aa1 n ih0 k ax))
("telefonos" nil (t eh1 l eh0 f ow1 n ow0 s))
("telefunken" nil (t eh1 l ax f ah1 ng k ax n))
("telegenic" nil (t eh1 l ax jh eh1 n ih0 k))
("teleglobe" nil (t eh1 l ax g l ow1 b))
("telegram" nil (t eh1 l ax g r ae1 m))
("telegrams" nil (t eh1 l ax g r ae1 m z))
("telegraph" nil (t eh1 l ax g r ae1 f))
("telegraphed" nil (t eh1 l ax g r ae1 f t))
("telekom" nil (t eh1 l ax k aa0 m))
("telemanagement" nil (t eh1 l ax m ae1 n ih0 jh m ax n t))
("telemarketer" nil (t eh1 l ax m aa1 r k ih0 t er0))
("telemarketers" nil (t eh1 l ax m aa1 r k ih0 t er0 z))
("telemarketing" nil (t eh1 l ax m aa1 r k ax t ih0 ng))
("telematics" nil (t eh1 l ax m ae1 t ih0 k s))
("telemecanique" nil (t eh1 l ax m eh1 k ax n iy1 k))
("telemedia" nil (t eh1 l ax m iy1 t iy0 ax))
("telemetry" nil (t eh1 l ax m eh1 t r iy0))
("telemundo" nil (t eh1 l ax m uw1 n d ow0))
("telenet" nil (t eh1 l ax n eh1 t))
("telepathy" nil (t eh1 l ax p ae1 th iy0))
("telephone" nil (t eh1 l ax f ow1 n))
("telephoned" nil (t eh1 l ax f ow1 n d))
("telephones" nil (t eh1 l ax f ow1 n z))
("telephonic" nil (t eh1 l ax f aa1 n ih0 k))
("telephonics" nil (t eh1 l ax f aa1 n ih0 k s))
("telephoning" nil (t eh1 l ax f ow1 n ih0 ng))
("telephoniques" nil (t eh1 l ax f aa0 n iy1 k s))
("telephony" nil (t eh1 l ax f ow1 n iy0))
("telepictures" nil (t eh1 l ax p ih1 k ch er0 z))
("teleport" nil (t eh1 l ax p ao1 r t))
("teleprobe" nil (t eh1 l ax p r ow1 b))
("teleprompter" nil (t eh1 l ax p r aa1 m p t er0))
("telequest" nil (t eh1 l ax k w eh1 s t))
("telerate" nil (t eh1 l er0 ey1 t))
("telesat" nil (t eh1 l ax s ae0 t))
("telesciences" nil (t eh1 l ax s ay1 ax n s ih0 z))
("telesco" nil (t eh0 l eh1 s k ow0))
("telescope" nil (t eh1 l ax s k ow1 p))
("telescopes" nil (t eh1 l ax s k ow1 p s))
("telescript" nil (t eh1 l ax s k r ih1 p t))
("telesis" nil (t eh1 l ax s ih0 s))
("telesphere" nil (t eh1 l ax s f ih1 r))
("telesystem" nil (t eh1 l ax s ih1 s t ax m))
("teletext" nil (t eh1 l ax t eh1 k s t))
("telethon" nil (t eh1 l ax th ih0 n))
("teletron" nil (t eh1 l ax t r aa1 n))
("telettra" nil (t eh1 l eh0 t r ax))
("teletype" nil (t eh1 l ax t ay1 p))
("teletypes" nil (t eh1 l ax t ay1 p s))
("televangelists" nil (t eh1 l ax v ae1 n jh ax l ax s t s))
("televideo" nil (t eh1 l ax v ih1 t iy0 ow0))
("televisa" nil (t eh1 l ax v iy1 z ax))
("televise" nil (t eh1 l ax v ay1 z))
("televised" nil (t eh1 l ax v ay1 z d))
("televising" nil (t eh1 l ax v ay1 z ih0 ng))
("television" nil (t eh1 l ax v ih1 zh ax n))
("televisions" nil (t eh1 l ax v ih1 zh ax n z))
("televison" nil (t eh1 l ih0 v ih0 zh ax n))
("telewest" nil (t eh1 l ax w eh1 s t))
("telex" nil (t eh1 l eh1 k s))
("telexed" nil (t eh1 l eh1 s k t))
("telexes" nil (t eh1 l eh1 k s ih0 s))
("telfair" nil (t eh1 l f ay0 r))
("telfer" nil (t eh1 l f er0))
("telfor" nil (t eh1 l f er0))
("telford" nil (t eh1 l f er0 d))
("telfour" nil (t eh1 l f er0))
("telit" nil (t eh1 l ih1 t))
("tell" nil (t eh1 l))
("tellabs" nil (t eh1 l ae1 b z))
("tellefsen" nil (t eh1 l ih0 f s ax n))
("tellep" nil (t eh1 l ax p))
("teller" nil (t eh1 l er0))
("tellers" nil (t eh1 l er0 z))
("telles" nil (t eh1 l z))
("tellez" nil (t ey0 l eh1 z))
("tellier" nil (t eh1 l iy0 er0))
("telling" nil (t eh1 l ih0 ng))
("tellingly" nil (t eh1 l ih0 ng l iy0))
("tellis" nil (t eh1 l ih0 s))
("tello" nil (t eh1 l ow0))
("tells" nil (t eh1 l z))
("telltale" nil (t eh1 l t ey1 l))
("telltales" nil (t eh1 l t ey1 l z))
("telluride" nil (t eh1 l y er0 ay1 d))
("tellurides" nil (t eh1 l y er0 ay1 d z))
("tellurium" nil (t eh0 l uh1 r iy0 ax m))
("telly" nil (t eh1 l iy0))
("telmex" nil (t eh1 l m eh1 k s))
("telos" nil (t eh1 l ow0 s))
("telsource" nil (t eh1 l s ao1 r s))
("telstar" nil (t eh1 l s t aa1 r))
("telstra" nil (t eh1 l s t r ax))
("teltschik" nil (t eh1 l ch ih0 k))
("telxon" nil (t eh1 l z aa0 n))
("telzrow" nil (t eh1 l z r ow0))
("temblor" nil (t eh1 m b l er0))
("temblors" nil (t eh1 m b l er0 z))
("temco" nil (t eh1 m k ow0))
("temerity" nil (t ax m eh1 r ax t iy0))
("temerlin" nil (t eh1 m er0 l ih0 n))
("temkin" nil (t eh1 m k ih0 n))
("temme" nil (t eh1 m))
("temp" nil (t eh1 m p))
("tempe" nil (t eh1 m p iy0))
("tempel" nil (t eh1 m p ax l))
("temper" nil (t eh1 m p er0))
("tempera" nil (t eh1 m p er0 ax))
("temperament" nil (t eh1 m p r ax m ax n t))
("temperamental" nil (t eh1 m p r ax m eh1 n t ax l))
("temperamentally" nil (t eh1 m p r ax m eh1 n t ax l iy0))
("temperaments" nil (t eh1 m p r ax m ax n t s))
("temperance" nil (t eh1 m p er0 ax n s))
("temperate" nil (t eh1 m p r ax t))
("temperature" nil (t eh1 m p r ax ch er0))
("temperatures" nil (t eh1 m p r ax ch er0 z))
("tempered" nil (t eh1 m p er0 d))
("tempering" nil (t eh1 m p er0 ih0 ng))
("tempers" nil (t eh1 m p er0 z))
("tempest" nil (t eh1 m p ax s t))
("tempesta" nil (t eh1 m p eh1 s t ax))
("tempestuous" nil (t eh1 m p eh1 s ch uw0 ax s))
("templars" nil (t eh1 m p l er0 z))
("template" nil (t eh1 m p l ax t))
("temple" nil (t eh1 m p ax l))
("templeman" nil (t eh1 m p ax l m ax n))
("templer" nil (t eh1 m p ax l er0))
("temples" nil (t eh1 m p ax l z))
("templet" nil (t eh1 m p l ih0 t))
("templeton" nil (t eh1 m p ax l t ax n))
("templin" nil (t eh1 m p l ih0 n))
("tempo" nil (t eh1 m p ow1))
("temporal" nil (t eh1 m p er0 ax l))
("temporally" nil (t eh1 m p er0 ax l iy0))
("temporaries" nil (t eh1 m p er0 eh1 r iy0 z))
("temporarily" nil (t eh1 m p er0 eh1 r ax l iy0))
("temporary" nil (t eh1 m p er0 eh1 r iy0))
("tempore" nil (t eh1 m p ao0 r))
("temporizing" nil (t eh1 m p er0 ay1 z ih0 ng))
("tempos" nil (t eh1 m p ow1 z))
("temps" nil (t eh1 m p s))
("tempt" nil (t eh1 m p t))
("temptation" nil (t eh0 m t ey1 sh ax n))
("temptations" nil (t eh0 m t ey1 sh ax n z))
("tempted" nil (t eh1 m p t ax d))
("tempting" nil (t eh1 m p t ih0 ng))
("temptress" nil (t eh1 m p t r ih0 s))
("tempts" nil (t eh1 m p t s))
("ten" nil (t eh1 n))
("tena" nil (t eh1 n ax))
("tenable" nil (t eh1 n ax b ax l))
("tenacious" nil (t ax n ey1 sh ax s))
("tenaciously" nil (t ax n ey1 sh ax s l iy0))
("tenacity" nil (t ax n ae1 s ih0 t iy0))
("tenaglia" nil (t eh0 n aa1 g l iy0 ax))
("tenancy" nil (t eh1 n ax n s iy0))
("tenant" nil (t eh1 n ax n t))
("tenants" nil (t eh1 n ax n t s))
("tenbrink" nil (t eh1 n b r ih1 ng k))
("tench" nil (t eh1 n ch))
("tencza" nil (t eh1 n ch ax))
("tend" nil (t eh1 n d))
("tended" nil (t eh1 n d ax d))
("tendencies" nil (t eh1 n d ax n s iy0 z))
("tendency" nil (t eh1 n d ax n s iy0))
("tendentious" nil (t eh1 n d eh1 n sh ax s))
("tender" nil (t eh1 n d er0))
("tendered" nil (t eh1 n d er0 d))
("tendering" nil (t eh1 n d er0 ih0 ng))
("tenderloin" nil (t eh1 n d er0 l oy1 n))
("tenderly" nil (t eh1 n d er0 l iy0))
("tenderness" nil (t eh1 n d er0 n ax s))
("tenders" nil (t eh1 n d er0 z))
("tending" nil (t eh1 n d ih0 ng))
("tendler" nil (t eh1 n d l er0))
("tendon" nil (t eh1 n d ax n))
("tendons" nil (t eh1 n d ax n z))
("tends" nil (t eh1 n d z))
("tenement" nil (t eh1 n ax m ax n t))
("tenements" nil (t eh1 n ax m ax n t s))
("tenenbaum" nil (t eh1 n ax n b aw1 m))
("tener" nil (t eh1 n er0))
("tenera" nil (t eh1 n eh1 r ax))
("tenet" nil (t eh1 n ax t))
("tenets" nil (t eh1 n ax t s))
("teneyck" nil (t eh1 n iy0 k))
("tenfold" nil (t eh1 n f ow1 l d))
("teng" nil (t eh1 ng))
("tengelmann" nil (t eh1 ng g ax l m ax n))
("tengiz" nil (t eh1 ng g ih0 z))
("tenglemann" nil (t eh1 ng g ax l m ax n))
("tenn" nil (t eh1 n ax s iy1))
("tennant" nil (t eh1 n ax n t))
("tenncare" nil (t eh1 n k eh1 r))
("tenneco" nil (t eh1 n ax k ow0))
("tennell" nil (t eh1 n ax l))
("tennenbaum" nil (t eh1 n ax n b aw1 m))
("tennent" nil (t eh1 n ax n t))
("tenner" nil (t eh1 n er0))
("tennessean" nil (t eh1 n ih0 s iy1 ax n))
("tennessee" nil (t eh1 n ax s iy1))
("tenney" nil (t eh1 n iy0))
("tennis" nil (t eh1 n ax s))
("tennison" nil (t eh1 n ih0 s ax n))
("tenny" nil (t eh1 n iy0))
("tennyson" nil (t eh1 n ih0 s ax n))
("tenor" nil (t eh1 n er0))
("tenore" nil (t eh1 n ao1 r))
("tenorio" nil (t eh0 n ao1 r iy0 ow0))
("tenors" nil (t eh1 n er0 z))
("tenpas" nil (t eh1 n p ax z))
("tenpenny" nil (t eh1 n p eh1 n iy0))
("tenrecs" nil (t eh1 n r eh1 k s))
("tens" nil (t eh1 n z))
("tense" nil (t eh1 n s))
("tensely" nil (t eh1 n s l iy0))
("tensile" nil (t eh1 n s ax l))
("tensiometer" nil (t eh1 n s iy0 aa1 m ih0 t er0))
("tension" nil (t eh1 n sh ax n))
("tensions" nil (t eh1 n ch ax n z))
("tent" nil (t eh1 n t))
("tentacle" nil (t eh1 n t ax k ax l))
("tentacles" nil (t eh1 n t ax k ax l z))
("tentative" nil (t eh1 n t ax t ih0 v))
("tentatively" nil (t eh1 n t ax t ih0 v l iy0))
("tenth" nil (t eh1 n th))
("tenths" nil (t eh1 n th s))
("tenting" nil (t eh1 n t ih0 ng))
("tents" nil (t eh1 n t s))
("tenuous" nil (t eh1 n y ax w ax s))
("tenure" nil (t eh1 n y er0))
("tenured" nil (t eh1 n y er0 d))
("tenures" nil (t eh1 n y er0 z))
("tenuta" nil (t eh0 n uw1 t ax))
("tenzer" nil (t eh1 n z er0))
("teo" nil (t ey1 ow0))
("teodoro" nil (t iy1 ow0 d ao1 r ow0))
("tepe" nil (t iy1 p))
("teper" nil (t iy1 p er0))
("tepid" nil (t eh1 p ih0 d))
("teply" nil (t eh1 p l iy0))
("tepper" nil (t eh1 p er0))
("tepperman" nil (t eh1 p er0 m ax n))
("tequila" nil (t ax k iy1 l ax))
("ter" nil (t er1))
("terada" nil (t er0 aa1 d ax))
("teradata" nil (t eh1 r ax d aa1 t ax))
("teradyne" nil (t eh1 r ax d ay1 n))
("teran" nil (t eh1 r ax n))
("terasawa" nil (t eh1 r ax s aa1 w ax))
("terbush" nil (t er1 b ax sh))
("tercel" nil (t er1 s ax l))
("tercero" nil (t er0 ch eh1 r ow0))
("terence" nil (t eh1 r ax n s))
("terentia" nil (t er0 eh1 n sh ax))
("teresa" nil (t er0 iy1 s ax))
("terese" nil (t eh1 r iy0 z))
("teresi" nil (t er0 eh1 s iy0))
("teresita" nil (t er0 eh0 s iy1 t ax))
("teressa" nil (t er0 eh1 s ax))
("terex" nil (t eh1 r ax k s))
("tereza" nil (t er0 ey1 z ax))
("terhaar" nil (t er1 hh aa0 r))
("terhorst" nil (t er1 hh ao0 r s t))
("terhune" nil (t er0 hh y uw1 n))
("teri" nil (t eh1 r iy0))
("teriyaki" nil (t eh1 r ih0 y aa1 k iy0))
("terkhorn" nil (t er1 k ao1 r n))
("terlecki" nil (t er0 l eh1 t s k iy0))
("terlizzi" nil (t er0 l iy1 t s iy0))
("term" nil (t er1 m))
("terman" nil (t er1 m ax n))
("termed" nil (t er1 m d))
("termeer" nil (t er0 m ih1 r))
("terminal" nil (t er1 m ax n ax l))
("terminally" nil (t er1 m ax n ax l iy0))
("terminals" nil (t er1 m ax n ax l z))
("terminate" nil (t er1 m ax n ey1 t))
("terminated" nil (t er1 m ax n ey1 t ax d))
("terminates" nil (t er1 m ih0 n ey1 t s))
("terminating" nil (t er1 m ax n ey1 t ih0 ng))
("termination" nil (t er0 m ax n ey1 sh ax n))
("terminations" nil (t er1 m ax n ey1 sh ax n z))
("terminator" nil (t er1 m ax n ey1 t er0))
("termine" nil (t er1 m ih0 n))
("terming" nil (t er1 m ih0 ng))
("termini" nil (t er1 m ih0 n ay1))
("terminology" nil (t er1 m ih0 n aa1 l ax jh iy0))
("termite" nil (t er1 m ay0 t))
("termites" nil (t er1 m ay0 t s))
("terms" nil (t er1 m z))
("ternes" nil (t er1 n z))
("terns" nil (t er1 n z))
("terpening" nil (t er1 p ax n ih0 ng))
("terpstra" nil (t eh1 r p s t r ax))
("terra" nil (t eh1 r ax))
("terracciano" nil (t er0 aa0 ch iy0 aa1 n ow0))
("terrace" nil (t eh1 r ax s))
("terraced" nil (t eh1 r ax s t))
("terraces" nil (t eh1 r ax s ax z))
("terrain" nil (t er0 ey1 n))
("terrains" nil (t er0 ey1 n z))
("terral" nil (t eh1 r ax l))
("terrana" nil (t er0 ae1 n ax))
("terrance" nil (t eh1 r ax n s))
("terrano" nil (t er0 aa1 n ow0))
("terranova" nil (t er0 aa0 n ow1 v ax))
("terrasi" nil (t er0 aa1 s iy0))
("terrazas" nil (t eh0 r aa1 z aa0 z))
("terre" nil (t eh1 r ax))
("terrebonne" nil (t er0 eh0 b ow1 n iy0))
("terrel" nil (t eh1 r ax l))
("terrell" nil (t eh1 r ax l))
("terrence" nil (t eh1 r ax n s))
("terrestrial" nil (t er0 eh1 s t r iy0 ax l))
("terri" nil (t eh1 r iy0))
("terrible" nil (t eh1 r ax b ax l))
("terribly" nil (t eh1 r ax b l iy0))
("terrie" nil (t eh1 r iy0))
("terrien" nil (t eh1 r iy0 ax n))
("terrier" nil (t eh1 r iy0 er0))
("terriers" nil (t eh1 r iy0 er0 z))
("terrific" nil (t er0 ih1 f ih0 k))
("terrifically" nil (t er0 ih1 f ih0 k l iy0))
("terrified" nil (t eh1 r ax f ay1 d))
("terrifies" nil (t eh1 r ax f ay1 z))
("terrify" nil (t eh1 r ax f ay1))
("terrifying" nil (t eh1 r ax f ay1 ih0 ng))
("terrile" nil (t eh1 r iy1 l))
("terrill" nil (t eh1 r ax l))
("terrington" nil (t eh1 r ih0 ng t ax n))
("terrio" nil (t eh1 r iy0 ow0))
("terris" nil (t eh1 r ih0 s))
("territo" nil (t er0 iy1 t ow0))
("territorial" nil (t eh1 r ih0 t ao1 r iy0 ax l))
("territorialism" nil (t eh1 r ax t ao1 r iy0 ax l ih1 z ax m))
("territories" nil (t eh1 r ax t ao1 r iy0 z))
("territory" nil (t eh1 r ih0 t ao1 r iy0))
("terrizzi" nil (t eh1 r ih1 t s iy0))
("terrones" nil (t eh1 r ax n z))
("terror" nil (t eh1 r er0))
("terrorism" nil (t eh1 r er0 ih1 z ax m))
("terrorist" nil (t eh1 r er0 ih0 s t))
("terroristic" nil (t eh1 r er0 ih1 s t ih0 k))
("terrorists" nil (t eh1 r er0 ax s t s))
("terrorize" nil (t eh1 r er0 ay1 z))
("terrorized" nil (t eh1 r er0 ay1 z d))
("terrorizes" nil (t eh1 r er0 ay1 z ih0 z))
("terrorizing" nil (t eh1 r er0 ay1 z ih0 ng))
("terry" nil (t eh1 r iy0))
("terse" nil (t er1 s))
("tersely" nil (t er1 s l iy0))
("tertia" nil (t eh1 r sh ax))
("tertiary" nil (t er1 sh er0 iy0))
("tertius" nil (t er1 t iy0 ih0 s))
("teruko" nil (t eh1 r uw0 k ow0))
("teruya" nil (t eh0 r uw1 y ax))
("tervo" nil (t eh1 r v ow0))
("terwilliger" nil (t er1 w ih0 l ih0 g er0))
("terzian" nil (t er1 z iy0 ax n))
("tesar" nil (t ih0 s aa1 r))
("tesch" nil (t eh1 sh))
("teschner" nil (t eh1 sh n er0))
("tesco" nil (t eh1 s k ow0))
("tese" nil (t iy1 s))
("tesh" nil (t eh1 sh))
("teska" nil (t eh1 s k ax))
("teske" nil (t eh1 s k))
("tesler" nil (t eh1 s ax l er0))
("teslik" nil (t eh1 s l ih0 k))
("tesmer" nil (t eh1 s ax m er0))
("tesobono" nil (t eh1 s ax b ow1 n ow0))
("tesobonos" nil (t eh1 s ax b ow1 n ow0 s))
("tesoriero" nil (t eh0 s ao0 r ih1 r ow0))
("tesoro" nil (t eh0 s ao1 r ow0))
("tess" nil (t eh1 s))
("tessa" nil (t eh1 s ax))
("tesselated" nil (t eh1 s ax l ey1 t ih0 d))
("tessie" nil (t eh1 s iy0))
("tessier" nil (t eh1 s iy0 er0))
("tessitore" nil (t eh0 s iy0 t ao1 r iy0))
("tessler" nil (t eh1 s l er0))
("tessman" nil (t eh1 s m ax n))
("tessmer" nil (t eh1 s m er0))
("tessy" nil (t eh1 s iy0))
("test" nil (t eh1 s t))
("testa" nil (t eh1 s t ax))
("testament" nil (t eh1 s t ax m ax n t))
("testamentary" nil (t eh1 s t ax m eh1 n t er0 iy0))
("tested" nil (t eh1 s t ax d))
("tester" nil (t eh1 s t er0))
("testerman" nil (t eh1 s t er0 m ax n))
("testers" nil (t eh1 s t er0 z))
("testicle" nil (t eh1 s t ih0 k ax l))
("testicles" nil (t eh1 s t ih0 k ax l z))
("testicular" nil (t eh1 s t ih1 k y ax l er0))
("testified" nil (t eh1 s t ax f ay1 d))
("testifies" nil (t eh1 s t ax f ay1 z))
("testify" nil (t eh1 s t ax f ay1))
("testifying" nil (t eh1 s t ax f ay1 ih0 ng))
("testily" nil (t eh1 s t ax l iy0))
("testimonial" nil (t eh1 s t ax m ow1 n iy0 ax l))
("testimonials" nil (t eh1 s t ax m ow1 n iy0 ax l z))
("testimonies" nil (t eh1 s t ax m ow1 n iy0 z))
("testimony" nil (t eh1 s t ax m ow1 n iy0))
("testing" nil (t eh1 s t ih0 ng))
("teston" nil (t eh1 s t ax n))
("testor" nil (t eh1 s t er0))
("testosterone" nil (t eh1 s t aa1 s t er0 ow1 n))
("testrake" nil (t eh1 s t r ey1 k))
("tests" nil (t eh1 s t s))
("testuro" nil (t eh1 s t uh1 r ow0))
("testy" nil (t eh1 s t iy0))
("tet" nil (t eh1 t))
("teta" nil (t eh1 t ax))
("tetanus" nil (t eh1 t ax n ax s))
("tete" nil (t eh1 t))
("teter" nil (t iy1 t er0))
("teters" nil (t iy1 t er0 z))
("tether" nil (t eh1 dh er0))
("tethered" nil (t eh1 dh er0 d))
("teti" nil (t eh1 t iy0))
("tetley" nil (t eh1 t l iy0))
("tetlow" nil (t eh1 t l ow0))
("teton" nil (t iy1 t ax n))
("tetra" nil (t eh1 t r ax))
("tetrads" nil (t eh1 t r ae1 d z))
("tetrahedral" nil (t eh1 t r ax hh iy1 d r ax l))
("tetrahedron" nil (t eh1 t r ax hh iy1 d r ax n))
("tetrault" nil (t eh1 t r aw0 l t))
("tetravalent" nil (t eh1 t r ax v ey1 l ax n t))
("tetreault" nil (t ih0 t r ow1))
("tetrick" nil (t eh1 t r ih0 k))
("tetro" nil (t eh1 t r ow0))
("tetsuji" nil (t eh1 t s uw1 jh iy0))
("tetsuo" nil (t eh1 t s uw1 ow0))
("tettamanti" nil (t eh1 t ax m ae1 n t iy0))
("tetterton" nil (t eh1 t er0 t ax n))
("tetzlaff" nil (t eh1 t z l ax f))
("tetzloff" nil (t eh1 t z l ao0 f))
("teuber" nil (t oy1 b er0))
("teubner" nil (t oy1 b n er0))
("teufel" nil (t oy1 f ax l))
("teuscher" nil (t oy1 sh er0))
("teutsch" nil (t oy1 ch))
("teva" nil (t ey1 v ax))
("tevatron" nil (t eh1 v ax t r aa0 n))
("teves" nil (t iy1 v z))
("tevis" nil (t eh1 v ih0 s))
("tevlin" nil (t eh1 v l ih0 n))
("tevye" nil (t eh1 v iy0))
("tew" nil (ch uw1))
("tewell" nil (t eh1 w eh0 l))
("tewes" nil (ch uw1 z))
("tewksbury" nil (t uw1 k s b er0 iy0))
("tews" nil (ch uw1 z))
("tex" nil (t eh1 k s))
("texaco" nil (t eh1 k s ax k ow0))
("texan" nil (t eh1 k s ax n))
("texans" nil (t eh1 k s ax n z))
("texarkana" nil (t eh1 k s aa0 r k ae1 n ax))
("texas" nil (t eh1 k s ax s))
("texasgulf" nil (t eh1 k s ax s g ah1 l f))
("texeira" nil (t ey0 k s eh1 r ax))
("texfi" nil (t eh1 k s f iy0))
("texpool" nil (t eh1 k s p uw1 l))
("texscan" nil (t eh1 k s k ax n))
("texstar" nil (t eh1 k s t aa1 r))
("texstyrene" nil (t eh1 k s t ay1 r iy1 n))
("text" nil (t eh1 k s t))
("textbook" nil (t eh1 k s t b uh1 k))
("textbooks" nil (t eh1 k s t b uh1 k s))
("texter" nil (t eh1 k s t er0))
("textile" nil (t eh1 k s t ay1 l))
("textiles" nil (t eh1 k s t ay1 l z))
("textor" nil (t eh1 k s t er0))
("textron" nil (t eh1 k s t r aa0 n))
("texts" nil (t eh1 k s t s))
("textual" nil (t eh1 k s ch ax w ax l))
("texture" nil (t eh1 k s ch er0))
("textured" nil (t eh1 k s ch er0 d))
("textures" nil (t eh1 k s ch er0 z))
("tezak" nil (t eh1 z ax k))
("tezeno" nil (t ey0 z ey1 n ow0))
("th" nil (t iy1 ey1 ch))
("thabo" nil (th ey1 b ow0))
("thach" nil (th ae1 ch))
("thacher" nil (th ae1 k er0))
("thacker" nil (th ae1 k er0))
("thackeray" nil (th ae1 k er0 iy0))
("thackston" nil (th ae1 k s t ax n))
("thad" nil (th ae1 d))
("thada" nil (th aa1 d ax))
("thadda" nil (th ae1 d ax))
("thaddea" nil (th ae1 d iy0 ax))
("thaddeus" nil (th ae1 d iy0 ax s))
("thaden" nil (th ey1 d ax n))
("thagard" nil (th ae1 g er0 d))
("thaggard" nil (th ae1 g er0 d))
("thai" nil (t ay1))
("thailand" nil (t ay1 l ae1 n d))
("thain" nil (th ey1 n))
("thaine" nil (th ey1 n))
("thais" nil (t ay1 z))
("thakkar" nil (th ae1 k er0))
("thal" nil (th ae1 l))
("thalacker" nil (th ae1 l ax k er0))
("thalamus" nil (th ae1 l ax m ax s))
("thalassa" nil (th aa0 l aa1 s ax))
("thaler" nil (th ey1 l er0))
("thalheimer" nil (th ae1 l hh ay0 m er0))
("thalia" nil (th ey1 l y ax))
("thalidomide" nil (th ax l ih1 d ax m ay1 d))
("thall" nil (th ao1 l))
("thallium" nil (th ae1 l iy0 ax m))
("thalman" nil (th ae1 l m ax n))
("thalmann" nil (th ao1 l m ax n))
("tham" nil (th ae1 m))
("thames" nil (t eh1 m z))
("than" nil (dh ae1 n))
("thane" nil (th ey1 n))
("thanh" nil (th ae1 n))
("thank" nil (th ae1 ng k))
("thanked" nil (th ae1 ng k t))
("thankful" nil (th ae1 ng k f ax l))
("thankfully" nil (th ae1 ng k f ax l iy0))
("thanking" nil (th ae1 ng k ih0 ng))
("thankless" nil (th ae1 ng k l ax s))
("thanks" nil (th ae1 ng k s))
("thanksgiving" nil (th ae1 ng k s g ih1 v ih0 ng))
("thanos" nil (th aa1 n ow0 s))
("thao" nil (dh aw1))
("thar" nil (th aa1 r))
("tharp" nil (th aa1 r p))
("tharpe" nil (th aa1 r p))
("tharrington" nil (th ae1 r ih0 ng t ax n))
("thassos" nil (th ae1 s ow0 s))
("that" nil (dh ae1 t))
("thatch" nil (th ae1 ch))
("thatched" nil (th ae1 ch t))
("thatcher" nil (th ae1 ch er0))
("thatcherism" nil (th ae1 ch er0 ih1 z ax m))
("thatcherite" nil (th ae1 ch er0 ay1 t))
("thatches" nil (th ae1 ch ih0 z))
("thau" nil (dh aw1))
("thaw" nil (th ao1))
("thawed" nil (th ao1 d))
("thawing" nil (th ao1 ih0 ng))
("thaws" nil (th ao1 z))
("thaxter" nil (th ae1 k s t er0))
("thaxton" nil (th ae1 k s t ax n))
("thayer" nil (th ey1 er0))
("thayne" nil (th ey1 n))
("the" nil (dh ax))
("thea" nil (th iy1 ax))
("theall" nil (th iy1 l))
("theano" nil (th iy1 n ow0))
("theater" nil (th iy1 ax t er0))
("theatergoers" nil (th iy1 t er0 g ow0 er0 z))
("theaters" nil (th iy1 ax t er0 z))
("theatre" nil (th iy1 ax t er0))
("theatres" nil (th iy1 ax t er0 z))
("theatrical" nil (th iy0 ae1 t r ih0 k ax l))
("theatricality" nil (th iy0 ae1 t r ax k ae1 l ax t iy0))
("theatrically" nil (th iy0 ae1 t r ih0 k ax l iy0))
("theatrics" nil (th iy1 t r ih0 k s))
("thebeau" nil (th ih0 b ow1))
("theberge" nil (th eh1 b er0 jh))
("thecla" nil (th eh1 k l ax))
("theda" nil (th iy1 d ax))
("thede" nil (th iy1 d))
("thedford" nil (th eh1 d f er0 d))
("thee" nil (dh iy1))
("theel" nil (th iy1 l))
("theft" nil (th eh1 f t))
("thefts" nil (th eh1 f t s))
("theil" nil (th ay1 l))
("theile" nil (th ay1 l))
("theilen" nil (th ay1 l ax n))
("theiler" nil (th ay1 l er0))
("thein" nil (th ay1 n))
("their" nil (dh eh1 r))
("theirs" nil (dh eh1 r z))
("theis" nil (dh ay1 z))
("theisen" nil (th ay1 s ax n))
("theism" nil (th iy1 ih0 z ax m))
("theiss" nil (th ay1 s))
("theissen" nil (th ay1 s ax n))
("thekla" nil (th eh1 k l ax))
("thelander" nil (th eh1 l ax n d er0))
("thelen" nil (th eh1 l ax n))
("thelin" nil (th eh1 l ih0 n))
("thelma" nil (th eh1 l m ax))
("them" nil (dh eh1 m))
("thematic" nil (th iy0 m ae1 t ih0 k))
("thematically" nil (th ax m ae1 t ih0 k l iy0))
("theme" nil (th iy1 m))
("themed" nil (th iy1 m d))
("themes" nil (th iy1 m z))
("themself" nil (dh eh0 m s eh1 l f))
("themselves" nil (dh eh0 m s eh1 l v z))
("then" nil (dh eh1 n))
("thence" nil (dh eh1 n s))
("thenceforth" nil (dh eh1 n s f ao1 r th))
("theo" nil (th iy1 ow0))
("theobald" nil (th iy1 ax b ax l d))
("theocracy" nil (th iy0 aa1 k r ax s iy0))
("theocratic" nil (th iy1 ax k r ae1 t ih0 k))
("theodor" nil (th iy1 ax d er0))
("theodora" nil (th iy1 ax d ao1 r ax))
("theodore" nil (th iy1 ax d ao1 r))
("theodorou" nil (th iy1 ax d er0 uw0))
("theodric" nil (th iy1 ax d r ih0 k))
("theola" nil (th iy1 ax l ax))
("theologian" nil (th iy1 ax l ow1 jh iy0 ax n))
("theologians" nil (th iy1 ax l ow1 jh ax n z))
("theological" nil (th iy1 ax l aa1 jh ih0 k ax l))
("theology" nil (th iy0 aa1 l ax jh iy0))
("theon" nil (th iy1 ax n))
("theone" nil (th iy1 aa0 n))
("theophania" nil (th iy0 ax f ae1 n iy0 ax))
("theophila" nil (th ey0 ax f iy1 l ax))
("theora" nil (th iy1 er0 ax))
("theorem" nil (th ih1 r ax m))
("theoretical" nil (th iy1 er0 eh1 t ih0 k ax l))
("theoretically" nil (th iy1 er0 eh1 t ih0 k ax l iy0))
("theoretician" nil (th iy1 er0 ax t ih1 sh ax n))
("theoreticians" nil (th iy1 er0 ax t ih1 sh ax n z))
("theories" nil (th ih1 r iy0 z))
("theorist" nil (th iy1 er0 ih0 s t))
("theorists" nil (th iy1 er0 ih0 s t s))
("theorize" nil (th iy1 er0 ay1 z))
("theorized" nil (th iy1 er0 ay1 z d))
("theorizes" nil (th iy1 er0 ay1 z ih0 z))
("theorizing" nil (th iy1 er0 ay1 z ih0 ng))
("theory" nil (th ih1 r iy0))
("thera" nil (th eh1 r ax))
("therafectin" nil (th eh1 r ax f eh1 k t ih0 n))
("therani" nil (t eh1 r aa1 n iy0))
("therapeutic" nil (th eh1 r ax p y uw1 t ih0 k))
("therapeutically" nil (th eh1 r ax p y uw1 t ih0 k ax l iy0))
("therapeutics" nil (th eh1 r ax p y uw1 t ih0 k s))
("therapies" nil (th eh1 r ax p iy0 z))
("therapist" nil (th eh1 r ax p ax s t))
("therapists" nil (th eh1 r ax p ax s t s))
("therapy" nil (th eh1 r ax p iy0))
("there" nil (dh eh1 r))
("thereabouts" nil (dh eh1 r ax b aw1 t s))
("thereafter" nil (dh eh0 r ae1 f t er0))
("thereby" nil (dh eh1 r b ay1))
("therefore" nil (dh eh1 r f ao1 r))
("therein" nil (dh eh0 r ih1 n))
("thereof" nil (th eh1 r ah1 v))
("theresa" nil (t er0 iy1 s ax))
("therese" nil (th eh1 r iy0 s))
("thereupon" nil (dh eh1 r ax p aa1 n))
("theriault" nil (th eh1 r iy0 ow1))
("therien" nil (th ih1 r iy0 n))
("theriot" nil (th ih1 r iy0 aa0 t))
("therm" nil (th er1 m))
("thermae" nil (th er1 m iy1))
("thermal" nil (th er1 m ax l))
("thermco" nil (th er1 m k ow0))
("thermedics" nil (th er0 m eh1 d ih0 k s))
("thermits" nil (th er1 m ih0 t s))
("thermo" nil (th er1 m ow0))
("thermometer" nil (th er0 m aa1 m ax t er0))
("thermometers" nil (th er0 m aa1 m ax t er0 z))
("thermonuclear" nil (th er1 m ow0 n uw1 k l iy0 er0))
("thermopatch" nil (th er1 m ow0 p ae1 ch))
("thermoplastic" nil (th er1 m ax p l ae1 s t ih0 k))
("thermoplastics" nil (th er1 m ax p l ae1 s t ih0 k s))
("thermos" nil (th er1 m ax s))
("thermosetting" nil (th er1 m ow0 s eh1 t ih0 ng))
("thermostat" nil (th er1 m ax s t ae1 t))
("thermostats" nil (th er1 m ax s t ae1 t s))
("theron" nil (th eh1 r ax n))
("theroux" nil (th er0 uw1))
("therrell" nil (th eh1 r ax l))
("therriault" nil (th eh1 r iy0 ow1))
("therrien" nil (th eh1 r iy0 n))
("thesaurus" nil (th ax s ao1 r ax s))
("these" nil (dh iy1 z))
("theses" nil (th iy1 s iy0 z))
("thesing" nil (dh iy1 z ih0 ng))
("thesis" nil (th iy1 s ax s))
("thespian" nil (th eh1 s p iy0 ax n))
("theta" nil (th ey1 t ax))
("thetford" nil (th eh1 t f er0 d))
("thetis" nil (th iy1 t ax s))
("theurer" nil (th er1 er0))
("theus" nil (dh y uw1 z))
("thew" nil (th uw1))
("they" nil (dh ey1))
("thi" nil (th iy1))
("thiamin" nil (th ay1 ax m ax n))
("thibadeau" nil (t ih1 b ax d ow0))
("thibault" nil (th ih0 b ow1))
("thibeau" nil (th ih0 b ow1))
("thibeault" nil (t iy1 b ow0))
("thibeaux" nil (th ih0 b ow1))
("thibedeau" nil (th ih1 b ih0 d ow0))
("thibert" nil (th ih0 b er1 t))
("thibodeau" nil (th ih1 b ax d ow0))
("thibodeaux" nil (th ih1 b ax d ow0))
("thick" nil (th ih1 k))
("thicken" nil (th ih1 k ax n))
("thickened" nil (th ih1 k ax n d))
("thickener" nil (th ih1 k ax n er0))
("thickening" nil (th ih1 k ax n ih0 ng))
("thickens" nil (th ih1 k ax n z))
("thicker" nil (th ih1 k er0))
("thicket" nil (th ih1 k ih0 t))
("thickets" nil (th ih1 k ax t s))
("thickhead" nil (th ih1 k hh eh1 d))
("thickly" nil (th ih1 k l iy0))
("thickness" nil (th ih1 k n ax s))
("thiebaud" nil (th iy1 b ow1))
("thiede" nil (th iy1 d))
("thief" nil (th iy1 f))
("thiel" nil (th iy1 l))
("thiele" nil (th iy1 l))
("thielemann" nil (th iy1 l m ax n))
("thielen" nil (th iy1 l ax n))
("thielke" nil (th iy1 l k))
("thielman" nil (th iy1 l m ax n))
("thielsch" nil (th iy1 l sh))
("thiem" nil (th iy1 m))
("thieman" nil (th iy1 m ax n))
("thiemann" nil (th iy1 m ax n))
("thieme" nil (th iy1 m))
("thien" nil (th iy1 n))
("thier" nil (th iy1 er0))
("thierry" nil (th ih0 r iy1))
("thiery" nil (th ih1 r iy0))
("thies" nil (th iy1 z))
("thiesen" nil (th iy1 s ax n))
("thiessen" nil (th iy1 s ax n))
("thievery" nil (th iy1 v er0 iy0))
("thieves" nil (th iy1 v z))
("thieving" nil (th iy1 v ih0 ng))
("thigh" nil (th ay1))
("thighs" nil (th ay1 z))
("thigpen" nil (th ih1 g p ax n))
("thill" nil (th ih1 l))
("thimble" nil (th ih1 m b ax l))
("thin" nil (th ih1 n))
("thine" nil (dh ay1 n))
("thing" nil (th ih1 ng))
("things" nil (th ih1 ng z))
("think" nil (th ih1 ng k))
("thinker" nil (th ih1 ng k er0))
("thinkers" nil (th ih1 ng k er0 z))
("thinkin" nil (th ih1 ng k ih0 n))
("thinking" nil (th ih1 ng k ih0 ng))
("thinkpad" nil (th ih1 ng k p ae1 d))
("thinks" nil (th ih1 ng k s))
("thinly" nil (th ih1 n l iy0))
("thinned" nil (th ih1 n d))
("thinner" nil (th ih1 n er0))
("thinnes" nil (th ih1 n z))
("thinness" nil (th ih1 n n ih0 s))
("thinnest" nil (th ih1 n ax s t))
("thinning" nil (th ih1 n ih0 ng))
("thins" nil (th ih1 n z))
("thiodiglycol" nil (th ay1 ow0 d ih1 g l ih0 k aa1 l))
("thiokol" nil (th ay1 ax k ao1 l))
("third" nil (th er1 d))
("thirdly" nil (th er1 d l iy0))
("thirdquarter" nil (th er1 d k w ao1 r t er0))
("thirds" nil (th er1 d z))
("thirst" nil (th er1 s t))
("thirsty" nil (th er1 s t iy0))
("thirteen" nil (th er1 t iy1 n))
("thirteenth" nil (th er1 t iy1 n th))
("thirteenths" nil (th er1 t iy1 n th s))
("thirties" nil (th er1 t iy0 z))
("thirtieth" nil (th er1 t iy0 ax th))
("thirty" nil (th er1 d iy0))
("thirtysomething" nil (th er1 t iy0 s ah1 m th ih0 ng))
("thiry" nil (th ih1 r iy0))
("thirza" nil (th er1 z ax))
("this" nil (dh ih1 s))
("thissen" nil (th ih1 s ax n))
("thistle" nil (th ih1 s ax l))
("thistles" nil (th ih1 s ax l z))
("thivierge" nil (th ih0 v iy1 r jh))
("thobe" nil (th ow1 b))
("thode" nil (th ow1 d))
("thoele" nil (th ow1 l))
("thoen" nil (th ow1 n))
("thoene" nil (th aa1 iy0 n))
("thoennes" nil (th ow1 n z))
("thole" nil (th ow1 l))
("tholen" nil (th aa1 l ax n))
("tholl" nil (th aa1 l))
("thom" nil (t aa1 m))
("thoma" nil (th ow1 m ax))
("thoman" nil (th ow1 m ax n))
("thomann" nil (dh ow1 m ax n))
("thomas" nil (t aa1 m ax s))
("thomases" nil (t aa1 m ax s ih0 z))
("thomasina" nil (th ow0 m aa0 s iy1 n ax))
("thomasine" nil (th ow0 m aa0 s iy1 n iy0))
("thomason" nil (th aa1 m ax s ax n))
("thomassen" nil (th aa1 m ax s ax n))
("thomasson" nil (th aa1 m ax s ax n))
("thomasville" nil (t aa1 m ax s v ih1 l))
("thome" nil (th ow1 m))
("thomes" nil (th ow1 m z))
("thometz" nil (th ow1 m eh0 t s))
("thomison" nil (th aa1 m ih0 s ax n))
("thomley" nil (th aa1 m l iy0))
("thommen" nil (th aa1 m ax n))
("thomp" nil (th aa1 m p))
("thompkins" nil (th aa1 m p k ih0 n z))
("thompsen" nil (th aa1 m p s ax n))
("thompson" nil (t aa1 m p s ax n))
("thompsons" nil (t aa1 m p s ax n z))
("thoms" nil (th aa1 m z))
("thomsen" nil (th aa1 m s ax n))
("thomson" nil (t aa1 m s ax n))
("thomspon" nil (th aa1 m s p ax n))
("thomure" nil (th aa1 m y uw0 r))
("thon" nil (th aa1 n))
("thone" nil (th ow1 n))
("thong" nil (th ao1 ng))
("thor" nil (th ao1 r))
("thora" nil (th ao1 r ax))
("thoracic" nil (th ao0 r ae1 s ih0 k))
("thorald" nil (th ao1 r ax l d))
("thorax" nil (th ao1 r ae0 k s))
("thorbert" nil (th ao1 r b er0 t))
("thorberta" nil (th ao0 r b eh1 r t ax))
("thorburn" nil (th ao1 r b er0 n))
("thordia" nil (th ao1 r d iy0 ax))
("thordis" nil (th ao1 r d ih0 s))
("thoreau" nil (th er0 ow1))
("thorell" nil (th ao1 r ax l))
("thoren" nil (th ao1 r ax n))
("thoresen" nil (th ao1 r iy0 z ax n))
("thoreson" nil (th ao1 r ih0 s ax n))
("thorington" nil (th ao1 r ih0 ng t ax n))
("thorium" nil (th aa1 r iy0 ax m))
("thorley" nil (th ao1 r l iy0))
("thorman" nil (th ao1 r m ax n))
("thormond" nil (th ao1 r m ax n d))
("thormund" nil (th ao1 r m ax n d))
("thorn" nil (th ao1 r n))
("thornberg" nil (th ao1 r n b er0 g))
("thornberry" nil (th ao1 r n b eh1 r iy0))
("thornburg" nil (th ao1 r n b er0 g))
("thornburgh" nil (th ao1 r n b er0 g))
("thornbury" nil (th ao1 r n b eh1 r iy0))
("thorndike" nil (th ao1 r n d ih0 k))
("thorndyke" nil (th ao1 r n d ay1 k))
("thorne" nil (th ao1 r n))
("thornell" nil (th ao1 r n ax l))
("thorner" nil (th ao1 r n er0))
("thornhill" nil (th ao1 r n hh ih1 l))
("thorniest" nil (th ao1 r n iy0 ax s t))
("thornley" nil (th ao1 r n l iy0))
("thornock" nil (th ao1 r n ax k))
("thorns" nil (th ao1 r n z))
("thornsberry" nil (th ao1 r n z b eh0 r iy0))
("thornsbury" nil (th ao1 r n z b eh0 r iy0))
("thornton" nil (th ao1 r n t ax n))
("thorny" nil (th ao1 r n iy0))
("thorough" nil (th er1 ow0))
("thoroughbred" nil (th er1 ow0 b r eh1 d))
("thoroughbreds" nil (th er1 ow0 b r eh1 d z))
("thoroughfare" nil (th er1 ow0 f eh1 r))
("thoroughfares" nil (th er1 ow0 f eh1 r z))
("thoroughly" nil (th er1 ow0 l iy0))
("thoroughness" nil (th er1 ow0 n ax s))
("thorp" nil (th ao1 r p))
("thorpe" nil (th ao1 r p))
("thorsell" nil (th ao1 r s ax l))
("thorsen" nil (th ao1 r s ax n))
("thorson" nil (th ao1 r s ax n))
("thorstad" nil (th ao1 r s t ax d))
("thorsten" nil (t ao1 r s t ax n))
("thorstenson" nil (th ao1 r s t ax n s ax n))
("thortec" nil (th ao1 r t eh1 k))
("thorton" nil (th ao1 r t ax n))
("thorup" nil (th ao1 r ax p))
("those" nil (dh ow1 z))
("thou" nil (dh aw1))
("though" nil (dh ow1))
("thought" nil (th ao1 t))
("thoughtful" nil (th ao1 t f ax l))
("thoughtfully" nil (th ao1 t f ax l iy0))
("thoughtless" nil (th ao1 t l ax s))
("thoughts" nil (th ao1 t s))
("thous" nil (dh aw1 z))
("thousand" nil (th aw1 z ax n d))
("thousands" nil (th aw1 z ax n d z))
("thousandth" nil (th aw1 z ax n d th))
("thousandths" nil (th aw1 z ax n d th s))
("thrailkill" nil (th r ey1 l k ih1 l))
("thrall" nil (th r ao1 l))
("thrash" nil (th r ae1 sh))
("thrashed" nil (th r ae1 sh t))
("thrasher" nil (th r ae1 sh er0))
("thrashes" nil (th r ae1 sh ih0 z))
("thrashing" nil (th r ae1 sh ih0 ng))
("thread" nil (th r eh1 d))
("threadbare" nil (th r eh1 d b eh1 r))
("threaded" nil (th r eh1 d ax d))
("threadfin" nil (th r eh1 d f ih0 n))
("threadgill" nil (th r eh1 d g ih1 l))
("threading" nil (th r eh1 d ih0 ng))
("threads" nil (th r eh1 d z))
("threat" nil (th r eh1 t))
("threaten" nil (th r eh1 t ax n))
("threatened" nil (th r eh1 t ax n d))
("threatening" nil (th r eh1 t ax n ih0 ng))
("threateningly" nil (th r eh1 t ax n ih0 ng l iy0))
("threatens" nil (th r eh1 t ax n z))
("threats" nil (th r eh1 t s))
("threatt" nil (th r iy1 t))
("three" nil (th r iy1))
("threefold" nil (th r iy1 f ow1 l d))
("threemonth" nil (th r iy1 m aa0 n th))
("threes" nil (th r iy1 z))
("threesome" nil (th r iy1 s ax m))
("threet" nil (th r iy1 t))
("threlkeld" nil (th r eh1 l k eh1 l d))
("thresh" nil (th r eh1 sh))
("thresher" nil (th r eh1 sh er0))
("threshhold" nil (th r eh1 sh hh ow1 l d))
("threshold" nil (th r eh1 sh ow1 l d))
("thresholds" nil (th r eh1 sh hh ow1 l d z))
("threw" nil (th r uw1))
("thrice" nil (th r ay1 s))
("thrift" nil (th r ih1 f t))
("thriftier" nil (th r ih1 f t iy0 er0))
("thrifts" nil (th r ih1 f t s))
("thrifty" nil (th r ih1 f t iy0))
("thrill" nil (th r ih1 l))
("thrilled" nil (th r ih1 l d))
("thriller" nil (th r ih1 l er0))
("thrillers" nil (th r ih1 l er0 z))
("thrilling" nil (th r ih1 l ih0 ng))
("thrills" nil (th r ih1 l z))
("thrips" nil (th r ih1 p s))
("thrive" nil (th r ay1 v))
("thrived" nil (th r ay1 v d))
("thrives" nil (th r ay1 v z))
("thriving" nil (th r ay1 v ih0 ng))
("throat" nil (th r ow1 t))
("throated" nil (th r ow1 t ih0 d))
("throats" nil (th r ow1 t s))
("throbbing" nil (th r aa1 b ih0 ng))
("throes" nil (th r ow1 z))
("throgmorton" nil (th r ax g m ao1 r t ax n))
("throm" nil (th r aa1 m))
("thrombolysis" nil (th r aa0 m b ow1 l ih0 s ih0 s))
("thrombolytic" nil (th r aa1 m b ow0 l ih1 t ih0 k))
("thrombosis" nil (th r aa0 m b ow1 s ax s))
("throne" nil (th r ow1 n))
("throneberry" nil (th r ow1 n b eh1 r iy0))
("throng" nil (th r ao1 ng))
("thronged" nil (th r ao1 ng d))
("throngs" nil (th r ao1 ng z))
("thronson" nil (th r aa1 n s ax n))
("throop" nil (th r uw1 p))
("throttle" nil (th r aa1 t ax l))
("throttled" nil (th r aa1 t ax l d))
("throttles" nil (th r aa1 t ax l z))
("throttling" nil (th r aa1 t ax l ih0 ng))
("through" nil (th r uw1))
("throughout" nil (th r uw0 aw1 t))
("throughput" nil (th r uw1 p uh1 t))
("throughs" nil (th r uw1 z))
("throve" nil (th r ow1 v))
("throw" nil (th r ow1))
("throwaway" nil (th r ow1 ax w ey1))
("throwback" nil (th r ow1 b ae1 k))
("thrower" nil (th r ow1 er0))
("throwers" nil (th r ow1 er0 z))
("throwing" nil (th r ow1 ih0 ng))
("thrown" nil (th r ow1 n))
("throws" nil (th r ow1 z))
("thru" nil (th r uw1))
("thrun" nil (th r ah1 n))
("thrush" nil (th r ah1 sh))
("thrushes" nil (th r ah1 sh ax z))
("thrust" nil (th r ah1 s t))
("thrusting" nil (th r ah1 s t ih0 ng))
("thrusts" nil (th r ah1 s t s))
("thruway" nil (th r uw1 w ey1))
("ths" nil (th s))
("thud" nil (th ah1 d))
("thug" nil (th ah1 g))
("thugs" nil (th ah1 g z))
("thul" nil (th ah1 l))
("thulin" nil (th uw1 l ih0 n))
("thull" nil (th ah1 l))
("thum" nil (th ah1 m))
("thuma" nil (th uw1 m ax))
("thuman" nil (th uw1 m ax n))
("thumann" nil (th uw1 m ax n))
("thumb" nil (th ah1 m))
("thumbed" nil (th ah1 m d))
("thumbing" nil (th ah1 m ih0 ng))
("thumbnail" nil (th ah1 m n ey1 l))
("thumbs" nil (th ah1 m z))
("thumm" nil (th ah1 m))
("thumma" nil (th ah1 m ax))
("thump" nil (th ah1 m p))
("thumped" nil (th ah1 m p t))
("thumper" nil (th ah1 m p er0))
("thumping" nil (th ah1 m p ih0 ng))
("thumps" nil (th ah1 m p s))
("thun" nil (th ah1 n))
("thunberg" nil (th ah1 n b er0 g))
("thunder" nil (th ah1 n d er0))
("thunderbird" nil (th ah1 n d er0 b er1 d))
("thunderbirds" nil (th ah1 n d er0 b er1 d z))
("thundercats" nil (th ah1 n d er0 k ae1 t s))
("thundered" nil (th ah1 n d er0 d))
("thundering" nil (th ah1 n d er0 ih0 ng))
("thunderous" nil (th ah1 n d er0 ax s))
("thunders" nil (th ah1 n d er0 z))
("thunderstorm" nil (th ah1 n d er0 s t ao1 r m))
("thunderstorms" nil (th ah1 n d er0 s t ao1 r m z))
("thunderstruck" nil (th ah1 n d er0 s t r ah1 k))
("thune" nil (th uw1 n))
("thur" nil (dh er1))
("thurber" nil (th er1 b er0))
("thurgood" nil (th er1 g uh0 d))
("thurlow" nil (th er1 l ow0))
("thurm" nil (th er1 m))
("thurman" nil (th er1 m ax n))
("thurmon" nil (th er1 m ax n))
("thurmond" nil (th er1 m ax n d))
("thurn" nil (th er1 n))
("thurnau" nil (th er0 n ow1))
("thurner" nil (th er1 n er0))
("thurnher" nil (th er1 n er0))
("thurow" nil (th uh1 r ow0))
("thursby" nil (th er1 s b iy0))
("thursday" nil (th er1 z d iy0))
("thursdays" nil (th er1 z d iy0 z))
("thurstan" nil (th er1 s t ax n))
("thurston" nil (th er1 s t ax n))
("thus" nil (dh ah1 s))
("thusly" nil (dh ah1 s l iy0))
("thuy" nil (t uw1))
("thwart" nil (th w ao1 r t))
("thwarted" nil (th w ao1 r t ax d))
("thwarting" nil (th w ao1 r t ih0 ng))
("thwarts" nil (th w ao1 r t s))
("thweatt" nil (th w iy1 t))
("thwing" nil (th w ih1 ng))
("thy" nil (dh ay1))
("thygerson" nil (th ay1 g er0 s ax n))
("thyme" nil (th ay1 m))
("thymidine" nil (th ih1 m ih0 d iy1 n))
("thyra" nil (th ay1 r ax))
("thyroid" nil (th ay1 r oy0 d))
("thyssen" nil (t ay1 s ax n))
("ti" nil (t iy1))
("tiaacref" nil (t iy1 ax k r eh1 f))
("tian" nil (t y aa1 n))
("tiananmen" nil (t iy0 eh1 n ae0 n m eh1 n))
("tianjin" nil (t iy0 ae1 n jh ih0 n))
("tiano" nil (t iy0 aa1 n ow0))
("tiara" nil (t iy0 aa1 r ax))
("tibbals" nil (t ih1 b ax l z))
("tibbets" nil (t ih1 b ih0 t s))
("tibbett" nil (t ih1 b ih0 t))
("tibbetts" nil (t ih1 b ih0 t s))
("tibbie" nil (t ih1 b iy0))
("tibbits" nil (t ih1 b ih0 t s))
("tibbitts" nil (t ih1 b ih0 t s))
("tibbs" nil (t ih1 b z))
("tibby" nil (t ih1 b iy0))
("tiber" nil (t ay1 b er0))
("tiberi" nil (t iy0 b eh1 r iy0))
("tiberia" nil (t ih0 b iy1 r iy0 ax))
("tiberio" nil (t ih0 b iy1 r iy0 ow0))
("tibet" nil (t ax b eh1 t))
("tibetan" nil (t ih0 b eh1 t ax n))
("tibetans" nil (t ax b eh1 t ax n z))
("tibia" nil (t ih1 b iy0 ax))
("tic" nil (t ih1 k))
("tice" nil (t ay1 s))
("ticer" nil (t ay1 s er0))
("tichenor" nil (t ih1 k ax n er0))
("tichy" nil (t ih1 ch iy0))
("ticinus" nil (t ih0 s iy1 n ax s))
("tick" nil (t ih1 k))
("ticked" nil (t ih1 k t))
("ticker" nil (t ih1 k er0))
("ticket" nil (t ih1 k ax t))
("ticketed" nil (t ih1 k ax t ih0 d))
("ticketing" nil (t ih1 k ax t ih0 ng))
("ticketless" nil (t ih1 k ax t l ax s))
("ticketmaster" nil (t ih1 k ih0 t m ae1 s t er0))
("ticketron" nil (t ih1 k ax t r aa0 n))
("tickets" nil (t ih1 k ax t s))
("ticking" nil (t ih1 k ih0 ng))
("tickle" nil (t ih1 k ax l))
("tickled" nil (t ih1 k ax l d))
("ticklish" nil (t ih1 k ax l ih0 sh))
("tickner" nil (t ih1 k n er0))
("ticknor" nil (t ih1 k n er0))
("ticks" nil (t ih1 k s))
("ticonderoga" nil (t ay0 k aa1 n d er0 ow1 g ax))
("ticor" nil (t ay1 k ao1 r))
("tidal" nil (t ay1 d ax l))
("tidball" nil (t ih1 d b ao1 l))
("tidbit" nil (t ih1 d b ih1 t))
("tidbits" nil (t ih1 d b ih0 t s))
("tidd" nil (t ih1 d))
("tiddlywinks" nil (t ih1 d l ih0 w ih0 ng k s))
("tide" nil (t ay1 d))
("tides" nil (t ay1 d z))
("tidewater" nil (t ay1 d w ao1 t er0))
("tidings" nil (t ay1 d ih0 ng z))
("tidmore" nil (t ih1 d m ao0 r))
("tidrick" nil (t ih1 d r ih0 k))
("tidwell" nil (t ih1 d w ax l))
("tidy" nil (t ay1 d iy0))
("tie" nil (t ay1))
("tied" nil (t ay1 d))
("tiede" nil (t iy1 d))
("tiedeman" nil (t iy1 d m ax n))
("tiedemann" nil (t iy1 d m ax n))
("tiedt" nil (t iy1 d t))
("tiegs" nil (t iy1 g z))
("tieing" nil (t ay1 ih0 ng))
("tieken" nil (t iy1 k ax n))
("tieless" nil (t ay1 l ax s))
("tieman" nil (t iy1 m ax n))
("tiemann" nil (t iy1 m ax n))
("tiemeyer" nil (t iy1 m ay0 er0))
("tien" nil (t y eh1 n))
("tier" nil (t iy1 r))
("tierce" nil (t iy1 r s))
("tierco" nil (t iy1 r k ow0))
("tiered" nil (t iy1 r d))
("tiernan" nil (t ih1 r n ax n))
("tierney" nil (t ih1 r n iy0))
("tierno" nil (t ih1 r n ow0))
("tiers" nil (t iy1 r z))
("ties" nil (t ay1 z))
("tieszen" nil (t iy1 sh ax n))
("tietje" nil (t iy1 t jh))
("tietjen" nil (t iy1 t jh ax n))
("tietmeyer" nil (t iy1 t m ay1 r))
("tietz" nil (t iy1 t s))
("tietze" nil (t iy1 t z))
("tieu" nil (t iy0 uw1))
("tiff" nil (t ih1 f))
("tiffany" nil (t ih1 f ax n iy0))
("tiffin" nil (t ih1 f ih0 n))
("tifft" nil (t ih1 f t))
("tift" nil (t ih1 f t))
("tig" nil (t ih1 g))
("tiger" nil (t ay1 g er0))
("tigera" nil (t ih0 jh eh1 r ax))
("tigers" nil (t ay1 g er0 z))
("tigert" nil (t ay1 g er0 t))
("tigges" nil (t ih1 g z))
("tiggs" nil (t ih1 g z))
("tighe" nil (t ay1 g))
("tight" nil (t ay1 t))
("tighten" nil (t ay1 t ax n))
("tightened" nil (t ay1 t ax n d))
("tightening" nil (t ay1 t ax n ih0 ng))
("tightenings" nil (t ay1 t ax n ih0 ng z))
("tightens" nil (t ay1 t ax n z))
("tighter" nil (t ay1 t er0))
("tightest" nil (t ay1 t ax s t))
("tightfisted" nil (t ay1 t f ih1 s t ih0 d))
("tightly" nil (t ay1 t l iy0))
("tightness" nil (t ay1 t n ax s))
("tightrope" nil (t ay1 t r ow1 p))
("tights" nil (t ay1 t s))
("tigner" nil (t ay1 g n er0))
("tignor" nil (t ih1 g n er0))
("tigon" nil (t ih1 g ax n))
("tigrean" nil (t ih0 g r iy1 n))
("tigris" nil (t ay1 g r ax s))
("tigue" nil (t iy1 g))
("tijerina" nil (t iy0 y eh0 r iy1 n ax))
("tijuana" nil (t ih0 w aa1 n ax))
("tike" nil (t ay1 k))
("tikes" nil (t ay1 k s))
("til" nil (t ih1 l))
("tilbury" nil (t ih1 l b eh1 r iy0))
("tilda" nil (t ih1 l d ax))
("tilden" nil (t ih1 l d ax n))
("tile" nil (t ay1 l))
("tiled" nil (t ay1 l d))
("tiles" nil (t ay1 l z))
("tiley" nil (t ay1 l iy0))
("tilford" nil (t ih1 l f er0 d))
("tilghman" nil (t ih1 l m ax n))
("till" nil (t ih1 l))
("tiller" nil (t ih1 l er0))
("tillery" nil (t ih1 l er0 iy0))
("tilles" nil (t ay1 l z))
("tillett" nil (t ih1 l ih0 t))
("tilley" nil (t ih1 l iy0))
("tillie" nil (t ih1 l iy0))
("tilling" nil (t ih1 l ih0 ng))
("tillinghast" nil (t ih1 l ih0 ng hh ae1 s t))
("tillis" nil (t ih1 l ih0 s))
("tillison" nil (t ih1 l ih0 s ax n))
("tillman" nil (t ih1 l m ax n))
("tillmon" nil (t ih1 l m ax n))
("tillotson" nil (t ih1 l ax t s ax n))
("tillson" nil (t ih1 l s ax n))
("tilly" nil (t ih1 l iy0))
("tilman" nil (t ih1 l m ax n))
("tilney" nil (t ih1 l n iy0))
("tilson" nil (t ih1 l s ax n))
("tilt" nil (t ih1 l t))
("tilted" nil (t ih1 l t ax d))
("tilth" nil (t ih1 l th))
("tilting" nil (t ih1 l t ih0 ng))
("tilton" nil (t ih1 l t ax n))
("tilts" nil (t ih1 l t s))
("tim" nil (t ih1 m))
("timan" nil (t ay1 m ax n))
("timber" nil (t ih1 m b er0))
("timberlake" nil (t ih1 m b er0 l ey1 k))
("timberland" nil (t ih1 m b er0 l ae1 n d))
("timberlands" nil (t ih1 m b er0 l ae1 n d z))
("timberline" nil (t ih1 m b er0 l ay1 n))
("timberman" nil (t ih1 m b er0 m ax n))
("timbers" nil (t ih1 m b er0 z))
("timblin" nil (t ih1 m b l ih0 n))
("timbre" nil (t ih1 m b er0))
("timbrook" nil (t ih1 m b r uh1 k))
("timbs" nil (t ih1 m z))
("time" nil (t ay1 m))
("timed" nil (t ay1 m d))
("timeframe" nil (t ay1 m f r ey1 m))
("timeless" nil (t ay1 m l ax s))
("timeliness" nil (t ay1 m l iy0 n ax s))
("timely" nil (t ay1 m l iy0))
("timepiece" nil (t ay1 m p iy1 s))
("timeplex" nil (t ay1 m p l eh1 k s))
("timer" nil (t ay1 m er0))
("timers" nil (t ay1 m er0 z))
("times" nil (t ay1 m z))
("timeshare" nil (t ay1 m sh eh1 r))
("timetable" nil (t ay1 m t ey1 b ax l))
("timetables" nil (t ay1 m t ey1 b ax l z))
("timex" nil (t ih1 m eh0 k s))
("timid" nil (t ih1 m ih0 d))
("timidity" nil (t ax m ih1 d ax t iy0))
("timidly" nil (t ih1 m ax d l iy0))
("timing" nil (t ay1 m ih0 ng))
("timisoara" nil (t ih1 m ax s ow0 aa1 r ax))
("timken" nil (t ih1 m k ax n))
("timko" nil (t ih1 m k ow0))
("timlen" nil (t ih1 m l ax n))
("timlin" nil (t ih1 m l ih0 n))
("timm" nil (t ih1 m))
("timme" nil (t ih1 m))
("timmeny" nil (t ih1 m ax n iy0))
("timmer" nil (t ih1 m er0))
("timmerman" nil (t ih1 m er0 m ax n))
("timmermann" nil (t ih1 m er0 m ax n))
("timmers" nil (t ih1 m er0 z))
("timmie" nil (t ih1 m iy0))
("timmins" nil (t ih1 m ih0 n z))
("timmons" nil (t ih1 m ax n z))
("timms" nil (t ih1 m z))
("timmy" nil (t ih1 m iy0))
("timon" nil (t ay1 m ax n))
("timoney" nil (t ih1 m ax n iy0))
("timor" nil (t iy0 m ao1 r))
("timorous" nil (t ih1 m er0 ax s))
("timothea" nil (t ih0 m ax dh iy1 ax))
("timothy" nil (t ih1 m ax th iy0))
("timpani" nil (t ih1 m p ax n iy1))
("timpe" nil (t ih1 m p))
("timpone" nil (t iy0 m p ow1 n iy0))
("timpson" nil (t ih1 m p s ax n))
("tims" nil (t ih1 m z))
("timson" nil (t ih1 m s ax n))
("tin" nil (t ih1 n))
("tina" nil (t iy1 n ax))
("tinajero" nil (t iy0 n aa0 y eh1 r ow0))
("tinch" nil (t ih1 n ch))
("tincher" nil (t ih1 n ch er0))
("tincture" nil (t ih1 ng k ch er0))
("tinctures" nil (t ih1 ng k ch er0 z))
("tindal" nil (t ih1 n d ax l))
("tindall" nil (t ih1 n d ax l))
("tindel" nil (t ih1 n d ax l))
("tindell" nil (t ih1 n d ax l))
("tinder" nil (t ih1 n d er0))
("tinderbox" nil (t ih1 n d er0 b aa1 k s))
("tindle" nil (t ih1 n d ax l))
("tindol" nil (t ih1 n d ax l))
("tine" nil (t ay1 n))
("tiner" nil (t ay1 n er0))
("tines" nil (t ay1 n z))
("ting" nil (t ih1 ng))
("tinge" nil (t ih1 n jh))
("tinged" nil (t ih1 ng d))
("tingen" nil (t ih1 ng ax n))
("tingey" nil (t ih1 ng g iy0))
("tingle" nil (t ih1 ng g ax l))
("tingler" nil (t ih1 ng g ax l er0))
("tingley" nil (t ih1 ng g l iy0))
("tingling" nil (t ih1 ng g ax l ih0 ng))
("tinier" nil (t ay1 n iy0 er0))
("tiniest" nil (t ay1 n iy0 ax s t))
("tinker" nil (t ih1 ng k er0))
("tinkered" nil (t ih1 ng k er0 d))
("tinkerer" nil (t ih1 ng k er0 er0))
("tinkerers" nil (t ih1 ng k er0 er0 z))
("tinkering" nil (t ih1 ng k er0 ih0 ng))
("tinkey" nil (t ih1 n k iy1))
("tinkham" nil (t ih1 ng k ax m))
("tinkle" nil (t ih1 ng k ax l))
("tinkled" nil (t ih1 ng k ax l d))
("tinkler" nil (t ih1 ng k ax l er0))
("tinkling" nil (t ih1 ng k ax l ih0 ng))
("tinley" nil (t ih1 n l iy0))
("tinman" nil (t ih1 n m ae1 n))
("tinnell" nil (t ih1 n ax l))
("tinney" nil (t ih1 n iy0))
("tinnin" nil (t ih1 n ih0 n))
("tinnon" nil (t ih1 n ax n))
("tinny" nil (t ih1 n iy0))
("tino" nil (t iy1 n ow0))
("tinoco" nil (t iy0 n ow1 k ow0))
("tins" nil (t ih1 n z))
("tinsel" nil (t ih1 n s ax l))
("tinseltown" nil (t ih1 n s ax l t aw1 n))
("tinsley" nil (t ih1 n s l iy0))
("tinsman" nil (t ih1 n s m ax n))
("tint" nil (t ih1 n t))
("tinted" nil (t ih1 n t ih0 d))
("tinto" nil (t ih1 n t ow0))
("tinton" nil (t ih1 n t ax n))
("tints" nil (t ih1 n t s))
("tintype" nil (t ih1 n t ay1 p))
("tiny" nil (t ay1 n iy0))
("tip" nil (t ih1 p))
("tipa" nil (t ih1 p ax))
("tiphook" nil (t ih1 p hh uh1 k))
("tipler" nil (t ay1 p ax l er0))
("tipo" nil (t iy1 p ow0))
("tipoff" nil (t ih1 p ao1 f))
("tipp" nil (t ih1 p))
("tipped" nil (t ih1 p t))
("tippen" nil (t ih1 p ax n))
("tippens" nil (t ih1 p ax n z))
("tipper" nil (t ih1 p er0))
("tipperary" nil (t ih1 p er0 eh1 r iy0))
("tippers" nil (t ih1 p er0 z))
("tippet" nil (t ih1 p ax t))
("tippets" nil (t ih1 p ih0 t s))
("tippett" nil (t ih1 p ih0 t))
("tippetts" nil (t ih1 p ih0 t s))
("tippie" nil (t ih1 p iy0))
("tippin" nil (t ih1 p ih0 n))
("tipping" nil (t ih1 p ih0 ng))
("tippins" nil (t ih1 p ih0 n z))
("tippit" nil (t ih1 p ih0 t))
("tippitt" nil (t ih1 p ih0 t))
("tipple" nil (t ih1 p ax l))
("tipps" nil (t ih1 p s))
("tippy" nil (t ih1 p iy0))
("tips" nil (t ih1 p s))
("tipster" nil (t ih1 p s t er0))
("tipsword" nil (t ih1 p s ao1 r d))
("tipsy" nil (t ih1 p s iy0))
("tiptoe" nil (t ih1 p t ow1))
("tiptoed" nil (t ih1 p t ow1 d))
("tiptoeing" nil (t ih1 p t ow1 ih0 ng))
("tipton" nil (t ih1 p t ax n))
("tirade" nil (t ay0 r ey1 d))
("tirades" nil (t ay0 r ey1 d z))
("tirado" nil (t ih0 r aa1 d ow0))
("tire" nil (t ay1 er0))
("tired" nil (t ay1 er0 d))
("tireless" nil (t ay1 er0 l ax s))
("tirelessly" nil (t ay1 r l ax s l iy0))
("tirello" nil (t ih0 r eh1 l ow0))
("tiremaker" nil (t ay1 r m ey1 k er0))
("tires" nil (t ay1 er0 z))
("tiresome" nil (t ay1 er0 s ax m))
("tirey" nil (t ay1 r iy0))
("tiring" nil (t ay1 r ih0 ng))
("tirone" nil (t ih0 r ow1 n))
("tiros" nil (t ay1 r ow0 z))
("tirpak" nil (t er1 p ax k))
("tirrell" nil (t ih0 r ey1 l))
("tis" nil (t iy1 z))
("tisch" nil (t ih1 sh))
("tischer" nil (t ih1 sh er0))
("tischler" nil (t ih1 sh ax l er0))
("tisdale" nil (t ih1 s d ey0 l))
("tisdel" nil (t ih1 s d ax l))
("tisdell" nil (t ih1 s d ax l))
("tish" nil (t ih1 sh))
("tisher" nil (t ih1 sh er0))
("tishler" nil (t ih1 sh l er0))
("tishman" nil (t ih1 sh m ax n))
("tison" nil (t ih1 s ax n))
("tissue" nil (t ih1 s y uw1))
("tissues" nil (t ih1 s y uw1 z))
("tit" nil (t ih1 t))
("tita" nil (t iy1 t ax))
("titan" nil (t ay1 t ax n))
("titanate" nil (t ay1 t ax n ey1 t))
("titania" nil (t ax t aa1 n y ax))
("titanic" nil (t ay0 t ae1 n ih0 k))
("titanium" nil (t ay0 t ey1 n iy0 ax m))
("titans" nil (t ay1 t ax n z))
("titcomb" nil (t ih1 t k ax m))
("tithe" nil (t ay1 dh))
("tithing" nil (t ay1 dh ih0 ng))
("titian" nil (t ih1 sh ax n))
("titillated" nil (t ih1 t ax l ey1 t ih0 d))
("titillating" nil (t ih1 t ax l ey1 t ih0 ng))
("titillation" nil (t ih1 t ih0 l ey1 sh ax n))
("title" nil (t ay1 t ax l))
("titled" nil (t ay1 t ax l d))
("titleholder" nil (t ay1 t ax l hh ow1 l d er0))
("titles" nil (t ay1 t ax l z))
("titlow" nil (t ih1 t l ow1))
("titman" nil (t ih1 t m ax n))
("tito" nil (t iy1 t ow0))
("titone" nil (t ih1 t ax n))
("titsworth" nil (t ih1 t s w er1 th))
("tittel" nil (t ih1 t ax l))
("titter" nil (t ih1 t er0))
("titterington" nil (t ih1 t er0 ih0 ng t ax n))
("tittle" nil (t ih1 t ax l))
("tittsworth" nil (t ih1 t s w er0 th))
("titular" nil (t ih1 ch ax l er0))
("titus" nil (t ay1 t ax s))
("titusville" nil (t ay1 t ax s v ih1 l))
("titzer" nil (t ih1 t z er0))
("tivoli" nil (t ih1 v ax l iy0))
("tizzy" nil (t ih1 z iy0))
("tjaden" nil (jh ey1 d ax n))
("tjarks" nil (jh aa1 r k s))
("tkach" nil (k ae1 ch))
("tkacz" nil (k aa1 ch))
("to" nil (t uw1))
("toa" nil (t ow1 ax))
("toad" nil (t ow1 d))
("toads" nil (t ow1 d z))
("toal" nil (t ow1 l))
("toalson" nil (t ow1 l s ax n))
("toalster" nil (t ow1 l s t er0))
("toast" nil (t ow1 s t))
("toasted" nil (t ow1 s t ih0 d))
("toaster" nil (t ow1 s t er0))
("toasters" nil (t ow1 s t er0 z))
("toasting" nil (t ow1 s t ih0 ng))
("toastmaster" nil (t ow1 s t m ae1 s t er0))
("toasts" nil (t ow1 s t s))
("tobacco" nil (t ax b ae1 k ow1))
("toback" nil (ch uw1 b ae0 k))
("tobago" nil (t ax b ey1 g ow0))
("tobar" nil (t ow1 b er0))
("tobe" nil (t ow1 b))
("toben" nil (t ow1 b ax n))
("tober" nil (t ow1 b er0))
("tobey" nil (t ow1 b iy0))
("tobia" nil (t ow1 b iy0 ax))
("tobias" nil (t ax b ay1 ax s))
("tobiason" nil (t ax b ay1 ax s ax n))
("tobie" nil (t ow1 b iy0))
("tobin" nil (t ow1 b ih0 n))
("tobler" nil (t ow1 b ax l er0))
("toboggans" nil (t ax b aa1 g ax n z))
("tobolski" nil (t ax b ow1 l s k iy0))
("tobu" nil (t ow0 b uw1))
("toby" nil (t ow1 b iy0))
("tocci" nil (t ow1 ch iy0))
("tocco" nil (t aa1 k ow0))
("tock" nil (t aa1 k))
("tocqueville" nil (t ow1 k v ih0 l))
("toczek" nil (t aa1 ch eh0 k))
("tod" nil (t aa1 d))
("toda" nil (t ow1 d ax))
("todaro" nil (t ow0 d aa1 r ow0))
("today" nil (t ax d ey1))
("todd" nil (t aa1 d))
("toddler" nil (t aa1 d l er0))
("toddlers" nil (t aa1 d l er0 z))
("toddling" nil (t aa1 d l ih0 ng))
("todhunter" nil (t aa1 d hh ah1 n t er0))
("todisco" nil (t ow0 d iy1 s k ow0))
("todoroff" nil (t aa1 d er0 ao0 f))
("todt" nil (t aa1 d t))
("toe" nil (t ow1))
("toed" nil (t ow1 d))
("toehold" nil (t ow1 hh ow1 l d))
("toeing" nil (t ow1 ih0 ng))
("toelle" nil (t ow1 l))
("toenails" nil (t ow1 n ey1 l z))
("toenjes" nil (t ow1 n jh ih0 z))
("toensing" nil (t ow1 n s ih0 ng))
("toepfer" nil (t ow1 p f er0))
("toes" nil (t ow1 z))
("toews" nil (t aa1 uw0 z))
("toffee" nil (t aa1 f iy0))
("toffler" nil (t ao1 f l er0))
("toft" nil (t aa1 f t))
("tofte" nil (t ow1 f t))
("tofu" nil (t ow1 f uw0))
("toga" nil (t ow1 g ax))
("together" nil (t ax g eh1 dh er0))
("togetherness" nil (t ax g eh1 dh er0 n ax s))
("togethers" nil (t ax g eh1 dh er0 z))
("toggle" nil (t aa1 g ax l))
("toggled" nil (t aa1 g ax l d))
("toggling" nil (t aa1 g l ih0 ng))
("tognino" nil (t aa1 g n iy1 n ow0))
("togo" nil (t ow1 g ow0))
("togs" nil (t aa1 g z))
("toil" nil (t oy1 l))
("toiled" nil (t oy1 l d))
("toilet" nil (t oy1 l ax t))
("toiletries" nil (t oy1 l ax t r iy0 z))
("toiletry" nil (t oy1 l ax t r iy0))
("toilets" nil (t oy1 l ax t s))
("toiling" nil (t oy1 l ih0 ng))
("toils" nil (t oy1 l z))
("toivonen" nil (t oy1 v ax n ax n))
("tokai" nil (t ow0 k ay1))
("tokar" nil (t ow0 k aa1 r))
("tokarczyk" nil (t aa1 k er0 ch ih0 k))
("tokarski" nil (t ax k aa1 r s k iy0))
("tokarz" nil (t ow1 k aa0 r z))
("token" nil (t ow1 k ax n))
("tokenism" nil (t ow1 k ax n ih1 z ax m))
("tokens" nil (t ow1 k ax n z))
("tokio" nil (t ow0 k iy1 ow0))
("tokkin" nil (t aa1 k ih0 n))
("tokos" nil (t ow1 k ow0 s))
("tokunaga" nil (t ow0 k uw0 n aa1 g ax))
("tokuo" nil (t aa1 k y uw0 ow0))
("tokuyama" nil (t ow1 k uw1 y aa1 m ax))
("tokyo" nil (t ow1 k iy0 ow1))
("tokyu" nil (t ow1 k y uw0))
("tolan" nil (t ow1 l ax n))
("toland" nil (t ow1 l ax n d))
("tolanthe" nil (t ow0 l aa1 n dh iy0))
("tolar" nil (t ow1 l er0))
("tolbert" nil (t ow1 l b er0 t))
("told" nil (t ow1 l d))
("toldrian" nil (t ow1 l d r iy0 ax n))
("tole" nil (t ow1 l))
("toledo" nil (t ax l iy1 d ow0))
("tolen" nil (t ow1 l ax n))
("tolentino" nil (t ow1 l eh0 n t iy1 n ow0))
("toler" nil (t ow1 l er0))
("tolerable" nil (t aa1 l er0 ax b ax l))
("tolerance" nil (t aa1 l er0 ax n s))
("tolerances" nil (t aa1 l er0 ax n s ih0 z))
("tolerant" nil (t aa1 l er0 ax n t))
("tolerate" nil (t aa1 l er0 ey1 t))
("tolerated" nil (t aa1 l er0 ey1 t ax d))
("tolerates" nil (t ao1 l er0 ey1 t s))
("tolerating" nil (t aa1 l er0 ey1 t ih0 ng))
("toleration" nil (t aa1 l er0 ey1 sh ax n))
("toles" nil (t ow1 l z))
("tolhurst" nil (t ow1 l hh er0 s t))
("tolin" nil (t ow1 l ih0 n))
("toliver" nil (t ow1 l ih0 v er0))
("toll" nil (t ow1 l))
("tolland" nil (t aa1 l ax n d))
("tollbooth" nil (t ow1 l b uw1 th))
("tolle" nil (t aa1 l))
("tollefsen" nil (t aa1 l ih0 f s ax n))
("tollefson" nil (t aa1 l ih0 f s ax n))
("toller" nil (t ow1 l er0))
("tolles" nil (t ow1 l z))
("tolleson" nil (t aa1 l ih0 s ax n))
("tollett" nil (t aa1 l ih0 t))
("tolley" nil (t aa1 l iy0))
("tolling" nil (t ow1 l ih0 ng))
("tollison" nil (t aa1 l ih0 s ax n))
("tolliver" nil (t ow1 l ih0 v er0))
("tolls" nil (t ow1 l z))
("tolly" nil (t ow1 l iy0))
("tolman" nil (t aa1 l m ax n))
("tolsma" nil (t ow1 l s m ax))
("tolson" nil (t ow1 l s ax n))
("tolstoy" nil (t ow1 l s t oy1))
("tom" nil (t aa1 m))
("toma" nil (t ow1 m ax))
("tomahawk" nil (t aa1 m ax hh ao1 k))
("tomaino" nil (t ow0 m aa0 iy1 n ow0))
("tomako" nil (t ow0 m aa1 k ow0))
("toman" nil (t ow1 m ax n))
("tomanek" nil (t aa1 m ax n ih0 k))
("tomaro" nil (t ow0 m aa1 r ow0))
("tomas" nil (t ow0 m aa1 s))
("tomasek" nil (t ax m aa1 s eh0 k))
("tomaselli" nil (t ow0 m aa0 s eh1 l iy0))
("tomasello" nil (t ow0 m aa0 s eh1 l ow0))
("tomasetti" nil (t ow0 m aa0 s eh1 t iy0))
("tomasi" nil (t ow0 m aa1 s iy0))
("tomasic" nil (t ax m aa1 s ih0 k))
("tomasik" nil (t ax m aa1 s ih0 k))
("tomasina" nil (t ao1 m ax s iy1 n ax))
("tomasine" nil (t ow0 m aa0 s iy1 n iy0))
("tomasini" nil (t ow0 m aa0 s iy1 n iy0))
("tomasino" nil (t ow0 m aa0 s iy1 n ow0))
("tomasko" nil (t ax m aa1 s k ow0))
("tomaso" nil (t ow0 m aa1 s ow0))
("tomassetti" nil (t ow0 m aa0 s eh1 t iy0))
("tomassi" nil (t ow0 m aa1 s iy0))
("tomasso" nil (t ow0 m aa1 s ow0))
("tomasulo" nil (t ow0 m aa0 s uw1 l ow0))
("tomaszewski" nil (t ax m ax sh eh1 f s k iy0))
("tomato" nil (t ax m ey1 t ow1))
("tomatoes" nil (t ax m ey1 t ow0 z))
("tomatos" nil (t ax m ey1 t ow1 z))
("tomayko" nil (t ax m ay1 k ow0))
("tomb" nil (t uw1 m))
("tomberlin" nil (t aa1 m b er0 l ih0 n))
("tomblike" nil (t uw1 m l ay1 k))
("tomblin" nil (t aa1 m b l ih0 n))
("tomboy" nil (t aa1 m b oy1))
("tombs" nil (t uw1 m z))
("tombstone" nil (t uw1 m s t ow1 n))
("tombstones" nil (t uw1 m s t ow1 n z))
("tomcat" nil (t aa1 m k ae1 t))
("tomczak" nil (t aa1 m ch ae0 k))
("tomczyk" nil (t aa1 m ch ih0 k))
("tome" nil (t ow1 m))
("tomei" nil (t aa1 m ay0))
("tomek" nil (t ow1 m eh0 k))
("tomeo" nil (t ow1 m iy0 ow0))
("tomer" nil (t ow1 m er0))
("tomerlin" nil (t aa1 m er0 l ih0 n))
("tomes" nil (t ow1 m z))
("tomey" nil (t ow1 m iy0))
("tomich" nil (t aa1 m ih0 k))
("tomiichi" nil (t ow1 m iy0 iy1 ch iy0))
("tomilson" nil (t aa1 m ax l s ax n))
("tomita" nil (t ow0 m iy1 t ax))
("tomkiewicz" nil (t aa1 m k ax v ih0 ch))
("tomkin" nil (t aa1 m k ih0 n))
("tomkins" nil (t aa1 m k ih0 n z))
("tomkinson" nil (t aa1 m k ih0 n s ax n))
("tomko" nil (t aa1 m k ow0))
("tomlin" nil (t aa1 m l ih0 n))
("tomlinson" nil (t aa1 m l ih0 n s ax n))
("tommie" nil (t aa1 m iy0))
("tommy" nil (t aa1 m iy0))
("tomorrow" nil (t ax m aa1 r ow1))
("tompane" nil (t aa1 m p ey1 n))
("tompkins" nil (t aa1 m p k ih0 n z))
("tompkinses" nil (t aa1 m p k ih0 n s ih0 z))
("tompson" nil (t aa1 m p s ax n))
("toms" nil (t aa1 m z))
("tomsic" nil (t aa1 m s ih0 k))
("tomson" nil (t aa1 m s ax n))
("ton" nil (t ah1 n))
("tonal" nil (t ow1 n ax l))
("tonalities" nil (t ow0 n ae1 l ax t iy0 z))
("tonality" nil (t ow0 n ae1 l ax t iy0))
("tonawanda" nil (t aa1 n ax w aa1 n d ax))
("tonda" nil (t aa1 n d ax))
("tondreau" nil (t ax n d r ow1))
("tone" nil (t ow1 n))
("toned" nil (t ow1 n d))
("tonegawa" nil (t ow1 n ih0 g aa1 w ax))
("tonelli" nil (t ow0 n eh1 l iy0))
("toner" nil (t ow1 n er0))
("tones" nil (t ow1 n z))
("toney" nil (t ow1 n iy0))
("tong" nil (t ao1 ng))
("tonge" nil (t aa1 n jh))
("tongs" nil (t aa1 ng z))
("tongue" nil (t ah1 ng))
("tongued" nil (t ah1 ng d))
("tongues" nil (t ah1 ng z))
("toni" nil (t ow1 n iy0))
("tonia" nil (t ow1 n iy0 ax))
("tonic" nil (t aa1 n ih0 k))
("tonics" nil (t aa1 n ih0 k s))
("tonie" nil (t ow1 n iy0))
("toniest" nil (t ow0 n iy1 s t))
("tonight" nil (t ax n ay1 t))
("toning" nil (t ow1 n ih0 ng))
("tonini" nil (t ow0 n iy1 n iy0))
("tonite" nil (t ax n ay1 t))
("tonjes" nil (t ow1 n y eh0 s))
("tonk" nil (t ao1 ng k))
("tonka" nil (t aa1 ng k ax))
("tonkin" nil (t aa1 ng k ih0 n))
("tonkovich" nil (t aa1 ng k ax v ih0 ch))
("tonks" nil (t aa1 ng k s))
("tonn" nil (t aa1 n))
("tonnage" nil (t ah1 n ax jh))
("tonnages" nil (t ah1 n ax jh ax z))
("tonne" nil (t ah1 n))
("tonner" nil (t ah1 n er0))
("tonnes" nil (t ah1 n z))
("tonnesen" nil (t ah1 n s ax n))
("tons" nil (t ah1 n z))
("tonsillectomies" nil (t aa1 n s ih0 l eh1 k t ax m iy0 z))
("tonsillectomy" nil (t aa1 n s ih0 l eh1 k t ax m iy0))
("tonsils" nil (t aa1 n s ax l z))
("tonti" nil (t aa1 n t iy0))
("tonton" nil (t aa1 n t ax n))
("tontons" nil (t aa1 n t ax n z))
("tony" nil (t ow1 n iy0))
("tonya" nil (t aa1 n y ax))
("tonyes" nil (t ow1 n y ax z))
("too" nil (t uw1))
("toobin" nil (t uw1 b ax n))
("toogood" nil (t uw1 g uh1 d))
("toohey" nil (t uw1 iy0))
("took" nil (t uh1 k))
("tooke" nil (t uh1 k))
("tooker" nil (t uh1 k er0))
("tookes" nil (t uh1 k s))
("tool" nil (t uw1 l))
("toolan" nil (t uw1 l ax n))
("toole" nil (t uw1 l))
("tooled" nil (t uw1 l d))
("tooley" nil (t uw1 l iy0))
("tooling" nil (t uw1 l ih0 ng))
("toolmakers" nil (t uw1 l m ey1 k er0 z))
("toolroom" nil (t uw1 l r uw1 m))
("tools" nil (t uw1 l z))
("toolworks" nil (t uw1 l w er1 k s))
("tooman" nil (t uw1 m ax n))
("toombs" nil (t uw1 m z))
("toomer" nil (t uw1 m er0))
("toomey" nil (t uw1 m iy0))
("toon" nil (t uw1 n))
("toone" nil (t uw1 n))
("toops" nil (t uw1 p s))
("toot" nil (t uw1 t))
("tootal" nil (t uw1 t ax l))
("tooth" nil (t uw1 th))
("toothaker" nil (t uw1 th ax k er0))
("toothbrush" nil (t uw1 th b r ax sh))
("toothbrushes" nil (t uw1 th b r ah1 sh ih0 z))
("toothed" nil (t uw1 th t))
("toothless" nil (t uw1 th l ax s))
("toothlike" nil (t uw1 th l ay1 k))
("toothman" nil (t uw1 th m ax n))
("toothpaste" nil (t uw1 th p ey1 s t))
("toothpastes" nil (t uw1 th p ey1 s t s))
("toothpick" nil (t uw1 th p ih1 k))
("toothpicks" nil (t uw1 th p ih1 k s))
("toothy" nil (t uw1 th iy0))
("tootle" nil (t uw1 t ax l))
("toots" nil (t uw1 t s))
("tootsie" nil (t uw1 t s iy1))
("top" nil (t aa1 p))
("topalian" nil (t ax p ey1 l iy0 ax n))
("topaz" nil (t ow1 p ae1 z))
("tope" nil (t ow1 p))
("topeka" nil (t ax p iy1 k ax))
("topel" nil (t ow1 p ax l))
("topete" nil (t aa1 p iy0 t))
("topham" nil (t aa1 f ax m))
("topiary" nil (t ow1 p iy0 eh1 r iy0))
("topic" nil (t aa1 p ih0 k))
("topical" nil (t aa1 p ax k ax l))
("topics" nil (t aa1 p ih0 k s))
("topix" nil (t ow1 p ih1 k s))
("topkapi" nil (t aa1 p k aa1 p iy0))
("topknot" nil (t aa1 p n aa1 t))
("topless" nil (t aa1 p l ax s))
("topliff" nil (t aa1 p l ih0 f))
("topographic" nil (t aa1 p ax g r ae1 f ih0 k))
("topography" nil (t ax p aa1 g r ax f iy0))
("topol" nil (t ow1 p aa0 l))
("topolski" nil (t ax p ow1 l s k iy0))
("topor" nil (t aa1 p er0))
("topp" nil (t aa1 p))
("topped" nil (t aa1 p t))
("topper" nil (t aa1 p er0))
("toppers" nil (t aa1 p er0 z))
("toppin" nil (t aa1 p ih0 n))
("topping" nil (t aa1 p ih0 ng))
("toppings" nil (t aa1 p ih0 ng z))
("toppins" nil (t aa1 p ih0 n z))
("topple" nil (t aa1 p ax l))
("toppled" nil (t aa1 p ax l d))
("topples" nil (t aa1 p ax l z))
("toppling" nil (t aa1 p ax l ih0 ng))
("topps" nil (t aa1 p s))
("tops" nil (t aa1 p s))
("topsoil" nil (t aa1 p s oy1 l))
("topsy" nil (t aa1 p s iy0))
("toquepala" nil (t aa1 k w eh0 p aa1 l ax))
("tor" nil (t ao1 r))
("torah" nil (t ao1 r ax))
("torain" nil (t er0 ey1 n))
("torald" nil (t ao1 r ax l d))
("toran" nil (t ao0 r aa1 n))
("toray" nil (t ao0 r ey1))
("torbeck" nil (t ao1 r b eh0 k))
("torbert" nil (t ao1 r b er0 t))
("torbett" nil (t ao1 r b ih0 t))
("torch" nil (t ao1 r ch))
("torched" nil (t ao1 r ch t))
("torches" nil (t ao1 r ch ih0 z))
("torchia" nil (t ao1 r k iy0 ax))
("torchmark" nil (t ao1 r ch m aa1 r k))
("tordella" nil (t ao1 r d eh1 l ax))
("tore" nil (t ao1 r))
("torell" nil (t ao0 r eh1 l))
("torelli" nil (t ao0 r eh1 l iy0))
("torello" nil (t ao0 r eh1 l ow0))
("toren" nil (t ao1 r ax n))
("torey" nil (t ao1 r iy0))
("torgersen" nil (t ao1 r g er0 s ax n))
("torgerson" nil (t ao1 r g er0 s ax n))
("torgeson" nil (t ao1 r g ih0 s ax n))
("torian" nil (t ao1 r iy0 ax n))
("toribio" nil (t ao0 r iy1 b iy0 ow0))
("tories" nil (t ao1 r iy0 z))
("torino" nil (t ao0 r iy1 n ow0))
("torkelson" nil (t ao1 r k ih0 l s ax n))
("torley" nil (t ao1 r l iy0))
("torma" nil (t ao1 r m ax))
("torme" nil (t ao1 r m))
("torment" n (t ao1 r m eh1 n t))
("torment" v (t ao0 r m eh1 n t))
("tormenta" nil (t ao1 r m eh1 n t ax))
("tormented" nil (t ao1 r m eh1 n t ih0 d))
("tormentors" nil (t ao1 r m eh1 n t er0 z))
("torments" n (t ao1 r m eh1 n t s))
("torments" v (t ao0 r m eh1 n t s))
("tormey" nil (t ao1 r m iy0))
("torn" nil (t ao1 r n))
("tornabene" nil (t ao0 r n aa0 b eh1 n ax))
("tornado" nil (t ao0 r n ey1 d ow1))
("tornadoes" nil (t ao0 r n ey1 d ow0 z))
("tornados" nil (t ao0 r n ey1 d ow1 z))
("tornatore" nil (t ao0 r n aa0 t ao1 r iy0))
("torney" nil (t ao1 r n iy0))
("torno" nil (t ao1 r n ow0))
("tornow" nil (t ao1 r n ow0))
("tornquist" nil (t ao1 r n k w ih0 s t))
("toro" nil (t ao1 r ow0))
("torok" nil (t ao1 r ax k))
("toronado" nil (t ao1 r ax n aa1 d ow0))
("toronto" nil (t er0 aa1 n t ow0))
("torosian" nil (t er0 aa1 zh ih0 n))
("torp" nil (t ao1 r p))
("torpedo" nil (t ao0 r p iy1 d ow1))
("torpedoed" nil (t ao0 r p iy1 d ow1 d))
("torpedoes" nil (t ao0 r p iy1 d ow0 z))
("torpedoing" nil (t ao0 r p iy1 d ow1 ih0 ng))
("torpedos" nil (t ao0 r p iy1 d ow1 z))
("torpey" nil (t ao1 r p iy0))
("torpid" nil (t ao1 r p ax d))
("torpor" nil (t ao1 r p er0))
("torque" nil (t ao1 r k))
("torr" nil (t ao1 r))
("torrance" nil (t ao1 r ax n s))
("torras" nil (t ao1 r ax s))
("torray" nil (t ao1 r ey0))
("torre" nil (t ao1 r))
("torregrossa" nil (t ao0 r eh0 g r ow1 s ax))
("torrejon" nil (t ao1 r ax jh aa0 n))
("torrence" nil (t ao1 r ax n s))
("torrens" nil (t ao1 r ax n z))
("torrent" nil (t ao1 r ax n t))
("torrential" nil (t ao0 r eh1 n ch ax l))
("torrents" nil (t ao1 r ax n t s))
("torrenzano" nil (t ao1 r eh0 n z aa1 n ow0))
("torreon" nil (t ao1 r iy0 ax n))
("torres" nil (t ao1 r eh1 z))
("torrey" nil (t ao1 r iy0))
("torrez" nil (t ao0 r eh1 z))
("torricelli" nil (t ao1 r ih0 s eh1 l iy0))
("torrid" nil (t ao1 r ax d))
("torrijos" nil (t ao0 r iy1 ow0 s))
("torrington" nil (t ao1 r ih0 ng t ax n))
("torrisi" nil (t ao0 r iy1 s iy0))
("torry" nil (t ao1 r iy0))
("torsiello" nil (t ao0 r s iy0 eh1 l ow0))
("torso" nil (t ao1 r s ow1))
("torsos" nil (t ao1 r s ow1 z))
("torstar" nil (t ao1 r s t aa1 r))
("torsten" nil (t ao1 r s t ax n))
("tort" nil (t ao1 r t))
("torti" nil (t ao1 r t iy0))
("tortilla" nil (t ao0 r t iy1 ax))
("tortillas" nil (t ao1 r t iy1 ax z))
("tortoise" nil (t ao1 r t ax s))
("tortoises" nil (t ao1 r t ax s ax z))
("tortora" nil (t ao0 r t ao1 r ax))
("tortorella" nil (t ao0 r t ao0 r eh1 l ax))
("tortorelli" nil (t ao0 r t ao0 r eh1 l iy0))
("tortorello" nil (t ao0 r t ao0 r eh1 l ow0))
("tortorice" nil (t ao0 r t ao1 r ih0 s))
("tortorici" nil (t ao0 r t ao0 r iy1 ch iy0))
("tortoriello" nil (t ao0 r t ao0 r iy0 eh1 l ow0))
("torts" nil (t ao1 r t s))
("tortuous" nil (t ao1 r ch ax w ax s))
("torture" nil (t ao1 r ch er0))
("tortured" nil (t ao1 r ch er0 d))
("tortures" nil (t ao1 r ch er0 z))
("torturing" nil (t ao1 r ch er0 ih0 ng))
("toru" nil (t ao1 r uw0))
("torumi" nil (t ao1 r uw1 m iy0))
("tory" nil (t ao1 r iy0))
("tosca" nil (t ao1 s k ax))
("toscanini" nil (t ao1 s k ax n iy1 n iy0))
("toscano" nil (t ow0 s k aa1 n ow0))
("tosch" nil (t ao1 sh))
("tosco" nil (t ao1 s k ow0))
("tosh" nil (t aa1 sh))
("toshiba" nil (t ow0 sh iy1 b ax))
("toshiharu" nil (t ow1 sh ih0 hh aa1 r uw0))
("toshihiko" nil (t ow1 sh ih0 hh iy1 k ow0))
("toshiki" nil (t ow0 sh iy1 k iy0))
("toshio" nil (t ow0 sh iy1 ow0))
("toshiyuki" nil (t ow1 sh ih0 y uw1 k iy0))
("tosi" nil (t ow1 s iy0))
("toso" nil (t ow1 s ow0))
("toss" nil (t ao1 s))
("tossed" nil (t ao1 s t))
("tosses" nil (t ao1 s ih0 z))
("tossing" nil (t ao1 s ih0 ng))
("tostado" nil (t ow0 s t aa1 d ow0))
("toste" nil (t ow1 s t))
("tostenson" nil (t aa1 s t ih0 n s ax n))
("tosti" nil (t ao1 s t iy0))
("tosto" nil (t ow1 s t ow0))
("tot" nil (t aa1 t))
("tota" nil (t ow1 t ax))
("total" nil (t ow1 t ax l))
("totaled" nil (t ow1 t ax l d))
("totaling" nil (t ow1 t ax l ih0 ng))
("totalitarian" nil (t ow1 t ae1 l ih0 t eh1 r iy0 ax n))
("totalitarianism" nil (t ow1 t ae1 l ax t eh1 r iy0 ax n ih1 z ax m))
("totalitarians" nil (t ow0 t ae1 l ax t eh1 r iy0 ax n z))
("totality" nil (t ow0 t ae1 l ax t iy0))
("totalled" nil (t ow1 t ax l d))
("totalling" nil (t ow1 t ax l ih0 ng))
("totally" nil (t ow1 t ax l iy0))
("totals" nil (t ow1 t ax l z))
("totaro" nil (t ow0 t aa1 r ow0))
("tote" nil (t ow1 t))
("toted" nil (t ow1 t ih0 d))
("totem" nil (t ow1 t ax m))
("totems" nil (t ow1 t ax m z))
("totenberg" nil (t ow1 t ax n b er0 g))
("totes" nil (t ow1 t s))
("toth" nil (t aa1 th))
("totherow" nil (t ah1 dh er0 ow0))
("toti" nil (t ow1 t iy0))
("toting" nil (t ow1 t ih0 ng))
("totman" nil (t aa1 t m ax n))
("toto" nil (t ow1 t ow0))
("tots" nil (t aa1 t s))
("totten" nil (t aa1 t ax n))
("tottering" nil (t aa1 t er0 ih0 ng))
("totton" nil (t aa1 t ax n))
("totty" nil (t aa1 t iy0))
("totzke" nil (t aa1 t z k))
("touareg" nil (t uw1 er0 ax g))
("touch" nil (t ah1 ch))
("touchdown" nil (t ah1 ch d aw1 n))
("touchdowns" nil (t ah1 ch d aw1 n z))
("touche" nil (t uw1 sh))
("touched" nil (t ah1 ch t))
("touches" nil (t ah1 ch ax z))
("touchet" nil (t uw0 sh eh1 t))
("touchette" nil (t uw1 sh eh1 t))
("touching" nil (t ah1 ch ih0 ng))
("touchstone" nil (t ah1 ch s t ow1 n))
("touchton" nil (t ah1 ch t ax n))
("touchy" nil (t ah1 ch iy0))
("toufexis" nil (t uw1 f eh1 k s ih0 s))
("tougaloo" nil (t uw1 g ax l uw1))
("tougas" nil (t ah1 g ax z))
("tough" nil (t ah1 f))
("toughed" nil (t ah1 f t))
("toughen" nil (t ah1 f ax n))
("toughened" nil (t ah1 f ax n d))
("toughening" nil (t ah1 f ax n ih0 ng))
("toughens" nil (t ah1 f ax n z))
("tougher" nil (t ah1 f er0))
("toughest" nil (t ah1 f ax s t))
("toughness" nil (t ah1 f n ax s))
("toughs" nil (t ah1 f s))
("touhey" nil (t ah1 hh iy0))
("toulouse" nil (t uw0 l uw1 z))
("toupee" nil (t uw1 p ey1))
("toupin" nil (t uw1 p ih0 n))
("toups" nil (t uw1 p s))
("tour" nil (t uh1 r))
("tourangeau" nil (t uh1 r ey0 ng g ow0))
("toured" nil (t uh1 r d))
("touretzky" nil (t er0 eh1 t s k iy0))
("tourigny" nil (t uh1 r ay0 n iy0))
("touring" nil (t uh1 r ih0 ng))
("tourism" nil (t uh1 r ih1 z ax m))
("tourist" nil (t uh1 r ax s t))
("tourists" nil (t uh1 r ax s t s))
("tourmaline" nil (t uh1 r m ax l iy1 n))
("tournament" nil (t uh1 r n ax m ax n t))
("tournaments" nil (t er1 n ax m ax n t s))
("tourney" nil (t er1 n iy0))
("tourneys" nil (t uw1 r n iy0 z))
("touro" nil (t uw1 r ow0))
("tours" nil (t uh1 r z))
("tourtelot" nil (t uh1 r t ax l aa0 t))
("tourville" nil (t uh1 r v ih0 l))
("tousey" nil (t ah1 s iy0))
("tousignant" nil (t uw1 s ih0 g n ax n t))
("tousley" nil (t ah1 s l iy0))
("toussaint" nil (t uw0 s ae1 n))
("toussie" nil (t uw1 s iy0))
("tout" nil (t aw1 t))
("toutant" nil (t uw0 t ao1 n t))
("touted" nil (t aw1 t ih0 d))
("touting" nil (t aw1 t ih0 ng))
("touts" nil (t aw1 t s))
("touvier" nil (t uw1 v iy0 ey1))
("tovar" nil (t ow0 v aa1 r))
("tovaz" nil (t ow1 v aa1 z))
("tovey" nil (t ow1 v iy0))
("tow" nil (t ow1))
("toward" nil (t ax w ao1 r d))
("towards" nil (t ax w ao1 r d z))
("towbin" nil (t ow1 b ih1 n))
("towboat" nil (t ow1 b ow1 t))
("towe" nil (t ow1))
("towed" nil (t ow1 d))
("towel" nil (t aw1 ax l))
("toweling" nil (t aw1 ax l ih0 ng))
("towell" nil (t aa1 w eh0 l))
("towels" nil (t aw1 ax l z))
("tower" nil (t aw1 er0))
("towered" nil (t aw1 er0 d))
("towering" nil (t aw1 er0 ih0 ng))
("towers" nil (t aw1 er0 z))
("towery" nil (t ow0 er1 iy0))
("towey" nil (t ow1 iy0))
("towing" nil (t ow1 ih0 ng))
("towle" nil (t aw1 l))
("towler" nil (t ow1 l er0))
("towles" nil (t ow1 ax l z))
("town" nil (t aw1 n))
("towne" nil (t aw1 n))
("towner" nil (t aw1 n er0))
("towners" nil (t aw1 n er0 z))
("townes" nil (t aw1 n z))
("townhouse" nil (t aw1 n hh aw1 s))
("townhouses" nil (t aw1 n hh aw1 s ih0 z))
("townley" nil (t aw1 n l iy0))
("towns" nil (t aw1 n z))
("townsel" nil (t aw1 n s ax l))
("townsell" nil (t aw1 n s ax l))
("townsend" nil (t aw1 n z ax n d))
("townsfolk" nil (t aw1 n z f ow1 k))
("township" nil (t aw1 n sh ih0 p))
("townships" nil (t aw1 n sh ih0 p s))
("townsley" nil (t aw1 n s l iy0))
("townsman" nil (t aw1 n z m ax n))
("townson" nil (t aw1 n s ax n))
("townspeople" nil (t aw1 n z p iy1 p ax l))
("towry" nil (t ao1 r iy0))
("tows" nil (t ow1 z))
("towsley" nil (t ow1 s l iy0))
("towson" nil (t ow1 s ax n))
("toxic" nil (t aa1 k s ih0 k))
("toxicity" nil (t aa0 k s ih1 s ax t iy0))
("toxicological" nil (t aa1 k s ax k ax l aa1 jh ih0 k ax l))
("toxicologist" nil (t aa1 k s ih0 k aa1 l ax jh ih0 s t))
("toxicologists" nil (t aa1 k s ih0 k aa1 l ax jh ih0 s t s))
("toxicology" nil (t aa1 k s ih0 k aa1 l ax jh iy0))
("toxics" nil (t aa1 k s ih0 k s))
("toxin" nil (t aa1 k s ax n))
("toxins" nil (t aa1 k s ax n z))
("toy" nil (t oy1))
("toyama" nil (t ow0 y aa1 m ax))
("toye" nil (t oy1))
("toyed" nil (t oy1 d))
("toying" nil (t oy1 ih0 ng))
("toyko" nil (t oy1 k ow0))
("toymaker" nil (t oy1 m ey1 k er0))
("toyo" nil (t ow1 y ow0))
("toyobo" nil (t ow0 y ow1 b ow0))
("toyoda" nil (t ow0 y ow1 d ax))
("toyoo" nil (t oy0 uw1))
("toyota" nil (t ow0 y ow1 t ax))
("toyotas" nil (t oy1 ow1 t ax z))
("toys" nil (t oy1 z))
("toystores" nil (t oy1 s t ao1 r z))
("tozer" nil (t ow1 z er0))
("tozier" nil (t ow1 z iy0 er0))
("tozzi" nil (t aa1 z iy0))
("traber" nil (t r ey1 b er0))
("trabert" nil (t r ae1 b er0 t))
("trabucco" nil (t r aa0 b uw1 k ow0))
("trabue" nil (t r aa1 b w eh0))
("trac" nil (t r ae1 k))
("trace" nil (t r ey1 s))
("traceable" nil (t r ey1 s ax b ax l))
("traced" nil (t r ey1 s t))
("tracer" nil (t r ey1 s er0))
("tracers" nil (t r ey1 s er0 z))
("traces" nil (t r ey1 s ax z))
("tracey" nil (t r ey1 s iy0))
("trachea" nil (t r ey1 k iy0 ax))
("tracheophytes" nil (t r ey1 k iy0 ax f ay0 t s))
("trachsel" nil (t r ae1 k s ax l))
("tracht" nil (t r ae1 k t))
("trachtenberg" nil (t r ae1 k t ax n b er0 g))
("tracie" nil (t r ey1 s iy0))
("tracinda" nil (t r ax s ih1 n d ax))
("tracing" nil (t r ey1 s ih0 ng))
("track" nil (t r ae1 k))
("trackage" nil (t r ae1 k ih0 jh))
("trackballs" nil (t r ae1 k b ao1 l z))
("tracked" nil (t r ae1 k t))
("tracker" nil (t r ae1 k er0))
("trackers" nil (t r ae1 k er0 z))
("tracking" nil (t r ae1 k ih0 ng))
("tracks" nil (t r ae1 k s))
("tracor" nil (t r ey1 s er0))
("tract" nil (t r ae1 k t))
("tractable" nil (t r ae1 k t ax b ax l))
("tractebel" nil (t r ae1 k t ax b ax l))
("traction" nil (t r ae1 k sh ax n))
("tractor" nil (t r ae1 k t er0))
("tractors" nil (t r ae1 k t er0 z))
("tracts" nil (t r ae1 k t s))
("tracy" nil (t r ey1 s iy0))
("tracz" nil (t r aa1 ch))
("traczyk" nil (t r aa1 ch ih1 k))
("tradable" nil (t r ey1 d ax b ax l))
("trade" nil (t r ey1 d))
("tradeable" nil (t r ey1 d ax b ax l))
("traded" nil (t r ey1 d ih0 d))
("trademark" nil (t r ey1 d m aa1 r k))
("trademarked" nil (t r ey1 d m aa1 r k t))
("trademarks" nil (t r ey1 d m aa1 r k s))
("tradeoff" nil (t r ey1 d ao1 f))
("tradeoffs" nil (t r ey1 d ao1 f s))
("trader" nil (t r ey1 d er0))
("traders" nil (t r ey1 d er0 z))
("trades" nil (t r ey1 d z))
("tradesmen" nil (t r ey1 d z m ax n))
("trading" nil (t r ey1 d ih0 ng))
("tradings" nil (t r ey1 d ih0 ng z))
("tradition" nil (t r ax d ih1 sh ax n))
("traditional" nil (t r ax d ih1 sh ax n ax l))
("traditionalist" nil (t r ax d ih1 sh ax n ax l ih0 s t))
("traditionalists" nil (t r ax d ih1 sh n ax l ax s t s))
("traditionally" nil (t r ax d ih1 sh ax n ax l iy0))
("traditions" nil (t r ax d ih1 sh ax n z))
("traeger" nil (t r eh1 g er0))
("traer" nil (t r eh1 r))
("trafalgar" nil (t r ax f ae1 l g er0))
("traffic" nil (t r ae1 f ih0 k))
("trafficker" nil (t r ae1 f ih0 k er0))
("traffickers" nil (t r ae1 f ih0 k er0 z))
("trafficking" nil (t r ae1 f ih0 k ih0 ng))
("trafford" nil (t r ae1 f er0 d))
("traficant" nil (t r ae1 f ih0 k ax n t))
("traficante" nil (t r aa0 f iy0 k aa1 n t iy0))
("trafillio" nil (t r ax f iy1 l iy0 ow0))
("trafton" nil (t r ae1 f t ax n))
("tragedies" nil (t r ae1 jh ax d iy0 z))
("tragedy" nil (t r ae1 jh ax d iy0))
("trager" nil (t r ey1 g er0))
("trageser" nil (t r ae1 g iy0 z er0))
("tragic" nil (t r ae1 jh ih0 k))
("tragically" nil (t r ae1 jh ih0 k l iy0))
("tragicomic" nil (t r ae1 jh ih0 k aa1 m ih0 k))
("tragos" nil (t r ae1 g ow0 z))
("trahan" nil (t r ae1 hh ax n))
("trahern" nil (t r ae1 hh er0 n))
("trail" nil (t r ey1 l))
("trailblazer" nil (t r ey1 l b l ey1 z er0))
("trailed" nil (t r ey1 l d))
("trailer" nil (t r ey1 l er0))
("trailers" nil (t r ey1 l er0 z))
("trailing" nil (t r ey1 l ih0 ng))
("trails" nil (t r ey1 l z))
("trailways" nil (t r ey1 l w ey1 z))
("train" nil (t r ey1 n))
("traina" nil (t r ey1 n ax))
("trainable" nil (t r ey1 n ax b ax l))
("trained" nil (t r ey1 n d))
("trainee" nil (t r ey1 n iy1))
("trainees" nil (t r ey1 n iy1 z))
("trainer" nil (t r ey1 n er0))
("trainers" nil (t r ey1 n er0 z))
("training" nil (t r ey1 n ih0 ng))
("trainmen" nil (t r ey1 n m ax n))
("trainor" nil (t r ey1 n er0))
("trains" nil (t r ey1 n z))
("traipse" nil (t r ey1 p s))
("traister" nil (t r ey1 s t er0))
("trait" nil (t r ey1 t))
("traitor" nil (t r ey1 t er0))
("traitorous" nil (t r ey1 t er0 ax s))
("traitors" nil (t r ey1 t er0 z))
("traits" nil (t r ey1 t s))
("trajan" nil (t r ey1 jh ax n))
("trajectory" nil (t r ax jh eh1 k t er0 iy0))
("trak" nil (t r ae1 k))
("trakas" nil (t r aa1 k ax z))
("tram" nil (t r ae1 m))
("tramble" nil (t r ae1 m b ax l))
("tramco" nil (t r ae1 m k ow0))
("tramel" nil (t r ae1 m ax l))
("tramell" nil (t r aa0 m ey1 l))
("tramiel" nil (t r ae1 m iy0 ax l))
("trammel" nil (t r ae1 m ax l))
("trammell" nil (t r ae1 m ax l))
("tramontana" nil (t r aa0 m ow0 n t ae1 n ax))
("tramontano" nil (t r aa0 m ow0 n t aa1 n ow0))
("tramonte" nil (t r aa0 m ow1 n t iy0))
("tramontin" nil (t r ax m aa1 n t ih0 n))
("tramp" nil (t r ae1 m p))
("trampe" nil (t r ae1 m p))
("tramping" nil (t r ae1 m p ih0 ng))
("trample" nil (t r ae1 m p ax l))
("trampled" nil (t r ae1 m p ax l d))
("tramples" nil (t r ae1 m p ax l z))
("trampling" nil (t r ae1 m p l ih0 ng))
("trampoline" nil (t r ae1 m p ax l iy1 n))
("tran" nil (t r ae1 n))
("tranberg" nil (t r ae1 n b er0 g))
("trance" nil (t r ae1 n s))
("tranche" nil (t r ae1 n ch))
("tranches" nil (t r ae1 n ch eh0 z))
("tranchina" nil (t r aa0 n k iy1 n ax))
("trane" nil (t r ey1 n))
("trang" nil (t r ae1 ng))
("trani" nil (t r aa1 n iy0))
("tranquil" nil (t r ae1 ng k w ax l))
("tranquility" nil (t r ae0 ng k w ih1 l ih0 t iy0))
("tranquilizer" nil (t r ae1 ng k w ax l ay1 z er0))
("tranquilizers" nil (t r ae1 ng k w ax l ay1 z er0 z))
("tranquilizing" nil (t r ae1 ng k w ax l ay1 z ih0 ng))
("tranquillity" nil (t r ae0 n k w ih1 l ax t iy0))
("trans" nil (t r ae1 n z))
("transact" nil (t r ae0 n z ae1 k t))
("transacted" nil (t r ae0 n s ae1 k t ih0 d))
("transaction" nil (t r ae0 n z ae1 k sh ax n))
("transactions" nil (t r ae0 n z ae1 k sh ax n z))
("transafrica" nil (t r ae1 n z ae1 f r ih0 k ax))
("transalaska" nil (t r ae1 n z ax l ae1 s k ax))
("transalta" nil (t r ae1 n z ao1 l t ax))
("transamerica" nil (t r ae1 n s ax m eh1 r ih0 k ax))
("transamerican" nil (t r ae1 n z ax m eh1 r ih0 k ax n))
("transatlantic" nil (t r ae1 n z ax t l ae1 n t ih0 k))
("transcanada" nil (t r ae1 n z k ae1 n ax t ax))
("transcapital" nil (t r ae1 n z k ae1 p ih0 t ax l))
("transcend" nil (t r ae0 n s eh1 n d))
("transcended" nil (t r ae0 n s eh1 n d ih0 d))
("transcendence" nil (t r ae0 n s eh1 n d ax n s))
("transcendent" nil (t r ae0 n s eh1 n d ax n t))
("transcendental" nil (t r ae1 n s ax n d eh1 n t ax l))
("transcending" nil (t r ae0 n s eh1 n d ih0 ng))
("transcends" nil (t r ae0 n s eh1 n d z))
("transchannel" nil (t r ah1 n s ch ae1 n ax l))
("transcisco" nil (t r ae1 n s ih1 s k ow0))
("transco" nil (t r ae1 n s k ow0))
("transcon" nil (t r ae1 n z k aa0 n))
("transcontinental" nil (t r ae1 n z k aa1 n t ih0 n eh1 n t ax l))
("transcribe" nil (t r ae0 n s k r ay1 b))
("transcribed" nil (t r ae0 n s k r ay1 b d))
("transcribers" nil (t r ae0 n s k r ay1 b er0 z))
("transcribes" nil (t r ae0 n s k r ay1 b z))
("transcribing" nil (t r ae0 n s k r ay1 b ih0 ng))
("transcript" nil (t r ae1 n s k r ih1 p t))
("transcription" nil (t r ae1 n s k r ih1 p sh ax n))
("transcriptions" nil (t r ae1 n s k r ih1 p sh ax n z))
("transcripts" nil (t r ae1 n s k r ih1 p t s))
("transducer" nil (t r ae0 n s d uw1 s er0))
("transducers" nil (t r ae0 n s d uw1 s er0 z))
("transfer" v (t r ae0 n s f er1))
("transfer" n (t r ae1 n s f er0))
("transferability" nil (t r ae1 n s f er0 ax b ih1 l ih0 t iy0))
("transferable" nil (t r ae0 n s f er1 ax b ax l))
("transfered" nil (t r ae0 n s f er1 d))
("transfering" nil (t r ae0 n s f er1 ih0 ng))
("transferrable" nil (t r ae1 n s f er1 ax b ax l))
("transferred" nil (t r ae0 n s f er1 d))
("transferring" nil (t r ae0 n s f er1 ih0 ng))
("transfers" v (t r ae0 n s f er1 z))
("transfers" n (t r ae1 n s f er0 z))
("transfixed" nil (t r ae0 n s f ih1 k s t))
("transform" nil (t r ae0 n s f ao1 r m))
("transformation" nil (t r ae1 n s f er0 m ey1 sh ax n))
("transformational" nil (t r ae1 n s f er0 m ey1 sh ax n ax l))
("transformations" nil (t r ae1 n s f er0 m ey1 sh ax n z))
("transformed" nil (t r ae0 n s f ao1 r m d))
("transformer" nil (t r ae0 n s f ao1 r m er0))
("transformers" nil (t r ae0 n s f ao1 r m er0 z))
("transforming" nil (t r ae0 n s f ao1 r m ih0 ng))
("transforms" nil (t r ae0 n s f ao1 r m z))
("transfused" nil (t r ae0 n s f y uw1 z d))
("transfusion" nil (t r ae0 n s f y uw1 zh ax n))
("transfusions" nil (t r ae0 n s f y uw1 zh ax n z))
("transgenic" nil (t r ae1 n z jh eh1 n ih0 k))
("transgression" nil (t r ae0 n z g r eh1 sh ax n))
("transgressions" nil (t r ae0 n z g r eh1 sh ax n z))
("transgressor" nil (t r ae0 n z g r eh1 s er0))
("transience" nil (t r ae1 n z iy0 ax n s))
("transient" nil (t r ae1 n zh ax n t))
("transients" nil (t r ae1 n z iy0 ax n t s))
("transillumination" nil (t r ae1 n z ax l uw1 m ax n ey1 sh ax n))
("transimage" nil (t r ae1 n z ih1 m ih0 jh))
("transistor" nil (t r ae0 n z ih1 s t er0))
("transistors" nil (t r ae0 n z ih1 s t er0 z))
("transit" nil (t r ae1 n z ax t))
("transition" nil (t r ae0 n z ih1 sh ax n))
("transitional" nil (t r ae0 n s ih1 sh ax n ax l))
("transitions" nil (t r ae0 n z ih1 sh ax n z))
("transitory" nil (t r ae1 n z ax t ao1 r iy0))
("transkei" nil (t r ae1 n z k ey1))
("translate" nil (t r ae0 n z l ey1 t))
("translated" nil (t r ae0 n z l ey1 t ax d))
("translates" nil (t r ae0 n z l ey1 t s))
("translating" nil (t r ae0 n z l ey1 t ih0 ng))
("translation" nil (t r ae0 n z l ey1 sh ax n))
("translations" nil (t r ae0 n z l ey1 sh ax n z))
("translator" nil (t r ae0 n s l ey1 t er0))
("translators" nil (t r ae0 n s l ey1 t er0 z))
("translogic" nil (t r ae1 n z l aa1 jh ih0 k))
("translucent" nil (t r ae0 n s l uw1 s ax n t))
("transmark" nil (t r ae1 n z m aa1 r k))
("transmedia" nil (t r ae1 n z m iy1 d iy0 ax))
("transmission" nil (t r ae0 n s m ih1 sh ax n))
("transmissions" nil (t r ae0 n z m ih1 sh ax n z))
("transmit" nil (t r ae0 n z m ih1 t))
("transmits" nil (t r ae0 n z m ih1 t s))
("transmittable" nil (t r ae0 n z m ih1 t ax b ax l))
("transmittal" nil (t r ae0 n s m ih1 t ax l))
("transmitted" nil (t r ae0 n s m ih1 t ih0 d))
("transmitter" nil (t r ae0 n s m ih1 t er0))
("transmitters" nil (t r ae0 n s m ih1 t er0 z))
("transmitting" nil (t r ae0 n s m ih1 t ih0 ng))
("transnational" nil (t r ae0 n s n ae1 sh ax n ax l))
("transoceanic" nil (t r ae1 n z ow0 sh iy0 ae1 n ih0 k))
("transohio" nil (t r ae1 n z ow0 hh ay1 ow0))
("transou" nil (t r ae1 n z uw1))
("transpac" nil (t r ae1 n z p ae1 k))
("transpacific" nil (t r ae1 n s p ax s ih1 f ih0 k))
("transparencies" nil (t r ae0 n s p eh1 r ax n s iy0 z))
("transparency" nil (t r ae0 n s p eh1 r ax n s iy0))
("transparent" nil (t r ae0 n s p eh1 r ax n t))
("transparently" nil (t r ae0 n s p eh1 r ax n t l iy0))
("transpark" nil (t r ae1 n s p aa1 r k))
("transpired" nil (t r ae0 n s p ay1 er0 d))
("transpires" nil (t r ae0 n s p ay1 er0 z))
("transplant" nil (t r ae0 n s p l ae1 n t))
("transplantation" nil (t r ae1 n z p l ae0 n t ey1 sh ax n))
("transplanted" nil (t r ae0 n s p l ae1 n t ih0 d))
("transplanting" nil (t r ae0 n s p l ae1 n t ih0 ng))
("transplants" nil (t r ae0 n s p l ae1 n t s))
("transponder" nil (t r ae0 n s p aa1 n d er0))
("transponders" nil (t r ae0 n s p aa1 n d er0 z))
("transporation" nil (t r ae1 n z p er0 ey1 sh ax n))
("transport" v (t r ae0 n s p ao1 r t))
("transport" n (t r ae1 n s p ao0 r t))
("transportable" nil (t r ae0 n s p ao1 r t ax b ax l))
("transportation" nil (t r ae1 n s p er0 t ey1 sh ax n))
("transported" nil (t r ae0 n s p ao1 r t ax d))
("transporter" nil (t r ae0 n s p ao1 r t er0))
("transporters" nil (t r ae0 n s p ao1 r t er0 z))
("transporting" nil (t r ae0 n s p ao1 r t ih0 ng))
("transports" v (t r ae0 n s p ao1 r t s))
("transports" n (t r ae1 n s p ao0 r t s))
("transpose" nil (t r ae0 n s p ow1 z))
("transposed" nil (t r ae0 n s p ow1 z d))
("transracial" nil (t r ae1 n z r ey1 sh ax l))
("transrapid" nil (t r ae1 n z r ae1 p ih0 d))
("transtar" nil (t r ae0 n s t aa1 r))
("transtechnology" nil (t r ae1 n z t ax k n aa1 l ax jh iy0))
("transtector" nil (t r ae1 n z t eh1 k t er0))
("transue" nil (t r ae1 n z uw0))
("transvaal" nil (t r ae0 n z v aa1 l))
("transverse" nil (t r ae0 n z v er1 s))
("transway" nil (t r ae1 n z w ey1))
("transwestern" nil (t r ae1 n z w eh1 s t er0 n))
("transworld" nil (t r ae0 n s w er1 l d))
("transylvania" nil (t r ae1 n z iy0 l v ey1 n iy0 ax))
("trant" nil (t r ae1 n t))
("tranter" nil (t r ae1 n t er0))
("trantham" nil (t r ae1 n th ax m))
("tranum" nil (t r ae1 n ax m))
("tranzonic" nil (t r ae0 n z aa1 n ih0 k))
("trap" nil (t r ae1 p))
("trapani" nil (t r aa0 p aa1 n iy0))
("trapasso" nil (t r aa0 p aa1 s ow0))
("trapelo" nil (t r ax p eh1 l ow0))
("trapeze" nil (t r ax p iy1 z))
("traphagen" nil (t r ae1 f ax g ax n))
("trapnell" nil (t r ae1 p n ax l))
("trapp" nil (t r ae1 p))
("trappe" nil (t r ae1 p))
("trapped" nil (t r ae1 p t))
("trappers" nil (t r ae1 p er0 z))
("trapping" nil (t r ae1 p ih0 ng))
("trappings" nil (t r ae1 p ih0 ng z))
("trappist" nil (t r ae1 p ih0 s t))
("traps" nil (t r ae1 p s))
("trash" nil (t r ae1 sh))
("trashed" nil (t r ae1 sh t))
("trashes" nil (t r ae1 sh ih0 z))
("trashing" nil (t r ae1 sh ih0 ng))
("trashy" nil (t r ae1 sh iy0))
("trask" nil (t r ae1 s k))
("traub" nil (t r ao1 b))
("traudt" nil (t r ao1 d t))
("trauger" nil (t r aw1 g er0))
("traugh" nil (t r ao1))
("traughber" nil (t r ao1 b er0))
("traugott" nil (t r aw1 g ax t))
("traum" nil (t r ao1 m))
("trauma" nil (t r ao1 m ax))
("traumas" nil (t r ao1 m ax z))
("traumatic" nil (t r ao0 m ae1 t ih0 k))
("traumatized" nil (t r ao1 m ax t ay1 z d))
("trausch" nil (t r aw1 sh))
("traut" nil (t r ao1 t))
("trauth" nil (t r ao1 th))
("trautman" nil (t r aw1 t m ax n))
("trautmann" nil (t r aw1 t m ax n))
("trautner" nil (t r aw1 t n er0))
("trautwein" nil (t r aw1 t w ay1 n))
("travaglini" nil (t r aa0 v aa0 g l iy1 n iy0))
("travail" nil (t r ax v ey1 l))
("travails" nil (t r ax v ey1 l z))
("travel" nil (t r ae1 v ax l))
("traveldays" nil (t r ae1 v ax l d ey1 z))
("traveled" nil (t r ae1 v ax l d))
("traveler" nil (t r ae1 v ax l er0))
("travelers" nil (t r ae1 v ax l er0 z))
("traveling" nil (t r ae1 v ax l ih0 ng))
("travelled" nil (t r ae1 v ax l d))
("travellers" nil (t r ae1 v ax l er0 z))
("travelling" nil (t r ae1 v ax l ih0 ng))
("travelodge" nil (t r ae1 v ax l aa1 jh))
("travelogue" nil (t r ae1 v ax l ao1 g))
("travels" nil (t r ae1 v ax l z))
("travelstead" nil (t r ae1 v ax l s t eh1 d))
("travenol" nil (t r ae1 v ax n ax l))
("traver" nil (t r ey1 v er0))
("travers" nil (t r ae1 v er0 z))
("traverse" nil (t r ae1 v er0 s))
("traversed" nil (t r ae1 v er0 s t))
("traversing" nil (t r ax v er1 s ih0 ng))
("traverso" nil (t r aa0 v eh1 r s ow0))
("travesties" nil (t r ae1 v ax s t iy0 z))
("travesty" nil (t r ae1 v ax s t iy0))
("traviata" nil (t r aa0 v iy0 aa1 t ax))
("travieso" nil (t r aa0 v iy1 s ow0))
("travis" nil (t r ae1 v ih0 s))
("travisano" nil (t r ae1 v ih0 s aa1 n ow0))
("travolta" nil (t r ax v ow1 l t ax))
("travoltas" nil (t r ax v ow1 l t ax z))
("traweek" nil (t r ao1 iy1 k))
("trawick" nil (t r ao1 ih0 k))
("trawler" nil (t r ao1 l er0))
("trawlers" nil (t r ao1 l er0 z))
("trax" nil (t r ae1 k s))
("traxler" nil (t r ae1 k s l er0))
("tray" nil (t r ey1))
("trayer" nil (t r ey1 er0))
("traylor" nil (t r ey1 l er0))
("traynham" nil (t r ey1 n hh ax m))
("traynor" nil (t r ey1 n er0))
("trays" nil (t r ey1 z))
("traywick" nil (t r ey1 w ih1 k))
("trbovich" nil (t er0 b aa1 v ih0 ch))
("treacherous" nil (t r eh1 ch er0 ax s))
("treachery" nil (t r eh1 ch er0 iy0))
("treacy" nil (t r ey1 s iy0))
("tread" nil (t r eh1 d))
("treadaway" nil (t r eh1 d ax w ey1))
("treading" nil (t r eh1 d ih0 ng))
("treadmill" nil (t r eh1 d m ih1 l))
("treadmills" nil (t r eh1 d m ih1 l z))
("treads" nil (t r eh1 d z))
("treadway" nil (t r eh1 d w ey1))
("treadwell" nil (t r eh1 d w eh1 l))
("treadwheel" nil (t r eh1 d w iy1 l))
("treanor" nil (t r iy1 n er0))
("trease" nil (t r iy1 z))
("treason" nil (t r iy1 z ax n))
("treaster" nil (t r iy1 s t er0))
("treasure" nil (t r eh1 zh er0))
("treasured" nil (t r eh1 zh er0 d))
("treasurer" nil (t r eh1 zh er0 er0))
("treasurers" nil (t r eh1 zh er0 er0 z))
("treasures" nil (t r eh1 zh er0 z))
("treasuries" nil (t r eh1 zh er0 iy0 z))
("treasury" nil (t r eh1 zh er0 iy0))
("treasurys" nil (t r eh1 zh er0 iy0 z))
("treat" nil (t r iy1 t))
("treatable" nil (t r iy1 t ax b ax l))
("treated" nil (t r iy1 t ax d))
("treaters" nil (t r iy1 t er0 z))
("treaties" nil (t r iy1 t iy0 z))
("treating" nil (t r iy1 t ih0 ng))
("treatise" nil (t r iy1 t ax s))
("treatises" nil (t r iy1 t ax s ax z))
("treatment" nil (t r iy1 t m ax n t))
("treatments" nil (t r iy1 t m ax n t s))
("treats" nil (t r iy1 t s))
("treaty" nil (t r iy1 t iy0))
("trebilcock" nil (t r ih0 b ih1 l k ax k))
("treble" nil (t r eh1 b ax l))
("trebled" nil (t r eh1 b ax l d))
("treblinka" nil (t r eh0 b l ih1 ng k er0))
("trecker" nil (t r eh1 k er0))
("treder" nil (t r iy1 d er0))
("tredway" nil (t r eh1 d w ey1))
("tree" nil (t r iy1))
("treece" nil (t r iy1 s))
("treeless" nil (t r iy1 l ax s))
("treen" nil (t r iy1 n))
("trees" nil (t r iy1 z))
("treese" nil (t r iy1 z))
("treesh" nil (t r iy1 sh))
("treesweet" nil (t r iy1 s w iy1 t))
("trefethen" nil (t r eh1 f ih0 th ax n))
("trefgarne" nil (t r eh1 f g aa0 r n))
("trefry" nil (t r eh1 f r iy0))
("trefz" nil (t r eh1 f z))
("treglia" nil (t r eh1 g l iy0 ax))
("trego" nil (t r eh1 g ow0))
("tregoning" nil (t r eh1 g ax n ih0 ng))
("tregre" nil (t r eh1 g er0))
("tregurtha" nil (t r eh0 g er1 th ax))
("treharne" nil (t r eh1 hh aa0 r n))
("treiber" nil (t r ay1 b er0))
("treichel" nil (t r ay1 k ax l))
("treichler" nil (t r ay1 k ax l er0))
("treinen" nil (t r ay1 n ax n))
("trejo" nil (t r ey1 y ow0))
("trek" nil (t r eh1 k))
("trekked" nil (t r eh1 k t))
("trekking" nil (t r eh1 k ih0 ng))
("treks" nil (t r eh1 k s))
("trela" nil (t r eh1 l ax))
("trella" nil (t r eh1 l ax))
("trelleborg" nil (t r eh1 l ax b ao0 r g))
("trellis" nil (t r eh1 l ax s))
("treloar" nil (t r eh1 l ao0 r))
("tremain" nil (t r eh1 m ay0 n))
("tremaine" nil (t r ih0 m ey1 n))
("tremayne" nil (t r eh1 m ey0 n))
("trembath" nil (t r eh1 m b ax th))
("tremblay" nil (t r eh1 m b l ey0))
("tremble" nil (t r eh1 m b ax l))
("trembled" nil (t r eh1 m b ax l d))
("trembley" nil (t r eh1 m b l iy0))
("trembling" nil (t r eh1 m b ax l ih0 ng))
("trembly" nil (t r eh1 m b l iy0))
("tremel" nil (t r eh1 m ax l))
("tremendous" nil (t r ax m eh1 n d ax s))
("tremendously" nil (t r ax m eh1 n d ax s l iy0))
("tremens" nil (t r eh1 m ax n z))
("treml" nil (t r eh1 m ax l))
("tremmel" nil (t r eh1 m ax l))
("tremont" nil (t r eh1 m ax n t))
("tremor" nil (t r eh1 m er0))
("tremors" nil (t r eh1 m er0 z))
("tremper" nil (t r eh1 m p er0))
("tremulous" nil (t r eh1 m y ax l ax s))
("tremulously" nil (t r eh1 m y ax l ax s l iy0))
("trenary" nil (t r eh1 n eh0 r iy0))
("trench" nil (t r eh1 n ch))
("trenchant" nil (t r eh1 n ch ax n t))
("trenchard" nil (t r eh1 ng k er0 d))
("trencher" nil (t r eh1 n ch er0))
("trenches" nil (t r eh1 n ch ih0 z))
("trend" nil (t r eh1 n d))
("trended" nil (t r eh1 n d ih0 d))
("trendier" nil (t r eh1 n d y er0))
("trendiest" nil (t r eh0 n d iy1 s t))
("trending" nil (t r eh1 n d ih0 ng))
("trendless" nil (t r eh1 n d l ax s))
("trendline" nil (t r eh1 n d l ay1 n))
("trends" nil (t r eh1 n d z))
("trendsetter" nil (t r eh1 n d s eh1 t er0))
("trendy" nil (t r eh1 n d iy0))
("trenholm" nil (t r eh1 n hh ow1 l m))
("trenkamp" nil (t r eh1 n k ae1 m p))
("trenkle" nil (t r eh1 ng k ax l))
("trent" nil (t r eh1 n t))
("trentham" nil (t r eh1 n th ax m))
("trentman" nil (t r eh1 n t m ax n))
("trenton" nil (t r eh1 n t ax n))
("treon" nil (t r iy1 ax n))
("trepagnier" nil (t r eh1 p ax g n iy0 er0))
("trepanier" nil (t r eh1 p ax n iy0 er0))
("trepidation" nil (t r eh1 p ih0 d ey1 sh ax n))
("treppel" nil (t r eh1 p ax l))
("treptow" nil (t r eh1 p t ow0))
("tresch" nil (t r eh1 sh))
("trescott" nil (t r eh1 s k ax t))
("trespass" nil (t r eh1 s p ae1 s))
("trespassing" nil (t r eh1 s p ae1 s ih0 ng))
("tress" nil (t r eh1 s))
("tressel" nil (t r eh1 s ax l))
("tresses" nil (t r eh1 s ih0 z))
("tressler" nil (t r eh1 s l er0))
("trest" nil (t r eh1 s t))
("trester" nil (t r eh1 s t er0))
("trestle" nil (t r eh1 s ax l))
("trethewey" nil (t r eh1 th y uw0 iy0))
("tretinoin" nil (t r eh1 t ih0 n oy1 n))
("trettel" nil (t r eh1 t ax l))
("tretter" nil (t r eh1 t er0))
("trettin" nil (t r eh1 t ih0 n))
("treu" nil (t r uw1))
("treuhand" nil (t r uw1 hh ae1 n d))
("treuhandanstalt" nil (t r uw1 hh ae1 n d ax n s t aa1 l t))
("treurnicht" nil (t r uw1 r n ih0 k t))
("trevathan" nil (t r eh1 v ax th ax n))
("trevelyan" nil (t r ax v eh1 l y ax n))
("trevino" nil (t r ax v iy1 n ow0))
("treviso" nil (t r eh0 v iy1 s ow0))
("trevizo" nil (t r eh0 v iy1 z ow0))
("trevor" nil (t r eh1 v er0))
("trew" nil (t r uw1))
("trexler" nil (t r eh1 k s l er0))
("trey" nil (t r ey1))
("treybig" nil (t r ey1 b ih0 g))
("trezise" nil (t r eh1 z ay0 z))
("trezza" nil (t r eh1 z ax))
("tri" nil (t r ay1))
("triad" nil (t r ay1 ae1 d))
("triads" nil (t r ay1 ae1 d z))
("triage" nil (t r ay1 ih0 jh))
("trial" nil (t r ay1 ax l))
("trials" nil (t r ay1 ax l z))
("triana" nil (t r iy0 ae1 n ax))
("triangle" nil (t r ay1 ae1 ng g ax l))
("triangles" nil (t r ay1 ae1 ng g ax l z))
("triangular" nil (t r ay0 ae1 ng g y ax l er0))
("triano" nil (t r iy0 aa1 n ow0))
("triarc" nil (t r ay1 aa1 r k))
("triathlon" nil (t r ay1 ae1 th l ax n))
("trib" nil (t r ih1 b))
("tribal" nil (t r ay1 b ax l))
("tribasa" nil (t r ay1 b ae1 s ax))
("tribbett" nil (t r ih1 b ih0 t))
("tribbey" nil (t r ih1 b iy0))
("tribble" nil (t r ih1 b ax l))
("tribbles" nil (t r ih1 b ax l z))
("tribby" nil (t r ih1 b iy0))
("tribe" nil (t r ay1 b))
("tribeca" nil (t r ih0 b eh1 k ax))
("tribes" nil (t r ay1 b z))
("tribesmen" nil (t r ay1 b z m ih0 n))
("trible" nil (t r ih1 b ax l))
("triborough" nil (t r ay1 b er0 ow0))
("tribulations" nil (t r ih1 b y ax l ey1 sh ax n z))
("tribull" nil (t r ih1 b ax l))
("tribunal" nil (t r ax b y uw1 n ax l))
("tribunals" nil (t r ay1 b y uw1 n ax l z))
("tribune" nil (t r ih1 b y uw0 n))
("tributaries" nil (t r ih1 b y ax t eh1 r iy0 z))
("tributary" nil (t r ih1 b y ax t eh1 r iy0))
("tribute" nil (t r ih1 b y uw0 t))
("tributes" nil (t r ih1 b y uw0 t s))
("tricarico" nil (t r iy0 k aa0 r iy1 k ow0))
("trice" nil (t r ay1 s))
("tricentrol" nil (t r ay1 s eh1 n t r aa0 l))
("triche" nil (t r ih1 ch))
("trichet" nil (t r ih1 ch ih0 t))
("tricia" nil (t r ih1 sh ax))
("tricil" nil (t r ih1 s ih0 l))
("trick" nil (t r ih1 k))
("tricked" nil (t r ih1 k t))
("trickel" nil (t r ih1 k ax l))
("trickery" nil (t r ih1 k er0 iy0))
("trickett" nil (t r ih1 k ih0 t))
("trickey" nil (t r ih1 k iy0))
("trickier" nil (t r ih1 k iy0 er0))
("trickiest" nil (t r ih1 k iy0 ax s t))
("tricking" nil (t r ih1 k ih0 ng))
("trickle" nil (t r ih1 k ax l))
("trickled" nil (t r ih1 k ax l d))
("trickles" nil (t r ih1 k ax l z))
("trickling" nil (t r ih1 k l ih0 ng))
("tricks" nil (t r ih1 k s))
("tricky" nil (t r ih1 k iy0))
("trico" nil (t r iy1 k ow0))
("tricuspid" nil (t r ay0 k ah1 s p ax d))
("tricycle" nil (t r ih1 s ih0 k ax l))
("trident" nil (t r ay1 d ax n t))
("tridex" nil (t r ih1 d eh1 k s))
("tried" nil (t r ay1 d))
("triennial" nil (t r ay0 eh1 n iy0 ax l))
("trier" nil (t r ay1 er0))
("trierweiler" nil (t r ih1 r w ay0 l er0))
("tries" nil (t r ay1 z))
("trieste" nil (t r iy1 s t))
("trieu" nil (t r uw1))
("trifari" nil (t r ih0 f aa1 r iy0))
("trifle" nil (t r ay1 f ax l))
("trifles" nil (t r ay1 f ax l z))
("trifling" nil (t r ay1 f l ih0 ng))
("trig" nil (t r ih1 g))
("trigg" nil (t r ih1 g))
("trigger" nil (t r ih1 g er0))
("triggered" nil (t r ih1 g er0 d))
("triggering" nil (t r ih1 g er0 ih0 ng))
("triggers" nil (t r ih1 g er0 z))
("triggs" nil (t r ih1 g z))
("triglycerides" nil (t r ay0 g l ih1 s er0 ay1 d z))
("trigo" nil (t r iy1 g ow0))
("trilateral" nil (t r ay0 l ae1 t er0 ax l))
("trilby" nil (t r ih1 l b iy0))
("trillin" nil (t r ih1 l ih0 n))
("trilling" nil (t r ih1 l ih0 ng))
("trillion" nil (t r ih1 l y ax n))
("trillions" nil (t r ih1 l y ax n z))
("trillo" nil (t r ih1 l ow0))
("trilogy" nil (t r ih1 l ax jh iy0))
("trilon" nil (t r ih1 l ax n))
("trim" nil (t r ih1 m))
("trimac" nil (t r ih1 m ae0 k))
("trimarchi" nil (t r iy0 m aa1 r k iy0))
("trimarco" nil (t r ih0 m aa1 r k ow0))
("trimas" nil (t r iy1 m ax s))
("trimble" nil (t r ih1 m b ax l))
("trimboli" nil (t r iy0 m b ow1 l iy0))
("trimedyne" nil (t r ay1 m d ay1 n))
("trimester" nil (t r ay0 m eh1 s t er0))
("trimetrexate" nil (t r ih0 m eh1 t r ax k s ey1 t))
("trimm" nil (t r ih1 m))
("trimmed" nil (t r ih1 m d))
("trimmer" nil (t r ih1 m er0))
("trimmers" nil (t r ih1 m er0 z))
("trimming" nil (t r ih1 m ih0 ng))
("trimmings" nil (t r ih1 m ih0 ng z))
("trimpe" nil (t r ih1 m p))
("trims" nil (t r ih1 m z))
("trincomalee" nil (t r ih0 ng k ow1 m ax l iy0))
("trine" nil (t r ay1 n))
("tringali" nil (t r ih0 ng g aa1 l iy0))
("trinh" nil (t r ih1 n))
("trinidad" nil (t r ih1 n ih0 d ae1 d))
("trinity" nil (t r ih1 n ax t iy0))
("trinka" nil (t r ih1 ng k ax))
("trinket" nil (t r ih1 ng k ax t))
("trinkets" nil (t r ih1 ng k ax t s))
("trinkle" nil (t r ih1 ng k ax l))
("trinova" nil (t r ay1 n ow1 v ax))
("trintex" nil (t r ih1 n t eh1 k s))
("trio" nil (t r iy1 ow1))
("triola" nil (t r iy0 ow1 l ax))
("triolo" nil (t r iy0 ow1 l ow0))
("triomphe" nil (t r iy0 ow1 m f))
("trip" nil (t r ih1 p))
("tripartite" nil (t r ay0 p aa1 r t ay1 t))
("tripe" nil (t r ay1 p))
("tripi" nil (t r iy1 p iy0))
("triple" nil (t r ih1 p ax l))
("triplecast" nil (t r ih1 p ax l k ae1 s t))
("tripled" nil (t r ih1 p ax l d))
("triples" nil (t r ih1 p ax l z))
("triplets" nil (t r ih1 p l ax t s))
("triplett" nil (t r ih1 p l ih0 t))
("triplicate" nil (t r ih1 p l ih0 k ax t))
("tripling" nil (t r ih1 p ax l ih0 ng))
("tripod" nil (t r ay1 p aa1 d))
("tripodi" nil (t r iy0 p ow1 d iy0))
("tripods" nil (t r ay1 p aa1 d z))
("tripoli" nil (t r ih1 p ax l iy0))
("tripp" nil (t r ih1 p))
("trippe" nil (t r ih1 p))
("tripped" nil (t r ih1 p t))
("trippel" nil (t r ih1 p ax l))
("trippers" nil (t r ih1 p er0 z))
("trippett" nil (t r ih1 p ih0 t))
("tripping" nil (t r ih1 p ih0 ng))
("tripple" nil (t r ih1 p ax l))
("trips" nil (t r ih1 p s))
("triptych" nil (t r ih1 p t ih0 k))
("triremes" nil (t r ay1 r iy1 m z))
("trish" nil (t r ih1 sh))
("trisha" nil (t r ih1 sh ax))
("triska" nil (t r ih1 s k ax))
("trisler" nil (t r ih1 s ax l er0))
("trism" nil (t r ih1 z ax m))
("trista" nil (t r ih1 s t ax))
("tristan" nil (t r ih1 s t ae1 n))
("tristar" nil (t r ay1 s t aa1 r))
("tristate" nil (t r ay1 s t ey1 t))
("tristram" nil (t r ih1 s t r ax m))
("tritch" nil (t r ih1 ch))
("trite" nil (t r ay1 t))
("trites" nil (t r ay1 t s))
("tritium" nil (t r ih1 t iy0 ax m))
("triton" nil (t r ay1 t ax n))
("tritsch" nil (t r ih1 ch))
("tritschler" nil (t r ih1 ch l er0))
("tritt" nil (t r ih1 t))
("tritz" nil (t r ih1 t s))
("triumph" nil (t r ay1 ax m f))
("triumphal" nil (t r iy0 uw1 m f ax l))
("triumphant" nil (t r ay0 ah1 m f ax n t))
("triumphantly" nil (t r ay0 ah1 m f ax n t l iy0))
("triumphed" nil (t r ay1 ax m f t))
("triumphs" nil (t r ay1 ax m f s))
("triumvirate" nil (t r ay0 ah1 m v er0 ax t))
("trivedi" nil (t r iy0 v eh1 d iy0))
("trivest" nil (t r ih1 v ax s t))
("trivett" nil (t r ih1 v ih0 t))
("trivette" nil (t r ih0 v eh1 t))
("trivia" nil (t r ih1 v iy0 ax))
("trivial" nil (t r ih1 v iy0 ax l))
("triviality" nil (t r ih1 v iy0 ae1 l ax t iy0))
("trivialize" nil (t r ih1 v iy0 ax l ay1 z))
("trivializes" nil (t r ih1 v iy0 ax l ay1 z ih0 z))
("trixie" nil (t r ih1 k s iy0))
("trixy" nil (t r ih1 k s iy0))
("trizec" nil (t r ih1 z ax k))
("trnka" nil (t r ng k aa1))
("trobaugh" nil (t r aa1 b ao0))
("troche" nil (t r aa1 ch))
("trocki" nil (t r aa1 k iy0))
("trod" nil (t r aa1 d))
("troeger" nil (t r ow1 g er0))
("troendle" nil (t r ow1 n d ax l))
("troester" nil (t r ow1 s t er0))
("trogdon" nil (t r aa1 g d ax n))
("troha" nil (t r ow1 hh ax))
("troia" nil (t r ow1 y ax))
("troiani" nil (t r ow0 y aa1 n iy0))
("troiano" nil (t r ow0 iy0 aa1 n ow0))
("troika" nil (t r oy1 k ax))
("trois" nil (t w aa1))
("troise" nil (t r oy1 z))
("troisi" nil (t r oy1 s iy0))
("trojan" nil (t r ow1 jh ax n))
("trojanowski" nil (t r ax y ax n ao1 f s k iy0))
("trojans" nil (t r ow1 jh ax n z))
("trokel" nil (t r ow1 k ax l))
("trolinger" nil (t r ow1 l ih0 ng er0))
("troll" nil (t r ow1 l))
("trolley" nil (t r aa1 l iy0))
("trolleys" nil (t r aa1 l iy0 z))
("trolling" nil (t r ow1 l ih0 ng))
("trollinger" nil (t r ow1 l ih0 ng er0))
("troma" nil (t r ow1 m ax))
("trombino" nil (t r ow0 m b iy1 n ow0))
("trombley" nil (t r aa1 m b l iy0))
("trombly" nil (t r aa1 m b l iy0))
("trombone" nil (t r aa0 m b ow1 n))
("trombones" nil (t r aa0 m b ow1 n z))
("trombonist" nil (t r aa1 m b ow1 n ih0 s t))
("tromp" nil (t r aa1 m p))
("trompeter" nil (t r aa1 m p iy0 t er0))
("tron" nil (t r aa1 n))
("troncoso" nil (t r ow0 n k ow1 s ow0))
("trone" nil (t r ow1 n))
("tronic" nil (t r aa1 n ih0 k))
("troon" nil (t r uw1 n))
("troop" nil (t r uw1 p))
("trooped" nil (t r uw1 p t))
("trooper" nil (t r uw1 p er0))
("troopers" nil (t r uw1 p er0 z))
("trooping" nil (t r uw1 p ih0 ng))
("troops" nil (t r uw1 p s))
("troost" nil (t r uw1 s t))
("tropea" nil (t r ow1 p iy0 ax))
("tropeano" nil (t r ow1 p iy1 n ow0))
("trophies" nil (t r ow1 f iy0 z))
("trophy" nil (t r ow1 f iy0))
("tropic" nil (t r aa1 p ih0 k))
("tropical" nil (t r aa1 p ih0 k ax l))
("tropicana" nil (t r aa1 p ih0 k ae1 n ax))
("tropics" nil (t r aa1 p ih0 k s))
("tropopause" nil (t r aa1 p ax p ao1 z))
("tropp" nil (t r aa1 p))
("tropworld" nil (t r aa1 p w er0 l d))
("trosclair" nil (t r ax s k l eh1 r))
("trosper" nil (t r aa1 s p er0))
("trost" nil (t r aa1 s t))
("trostel" nil (t r aa1 s t ax l))
("trostle" nil (t r aa1 s ax l))
("trot" nil (t r aa1 t))
("troth" nil (t r ow1 th))
("trotman" nil (t r aa1 t m ax n))
("trots" nil (t r aa1 t s))
("trotsky" nil (t r aa1 t s k iy1))
("trott" nil (t r aa1 t))
("trotted" nil (t r aa1 t ax d))
("trotter" nil (t r aa1 t er0))
("trottier" nil (t r aa1 t iy0 er0))
("trotting" nil (t r aa1 t ih0 ng))
("troubadour" nil (t r uw1 b ax d ao1 r))
("troubh" nil (t r uw1 b))
("trouble" nil (t r ah1 b ax l))
("troubled" nil (t r ah1 b ax l d))
("troublefield" nil (t r ah1 b ax l f iy1 l d))
("troublemaker" nil (t r ah1 b ax l m ey1 k er0))
("troublemakers" nil (t r ah1 b ax l m ey1 k er0 z))
("troubles" nil (t r ah1 b ax l z))
("troubleshooter" nil (t r ah1 b ax l sh uw1 t er0))
("troublesome" nil (t r ah1 b ax l s ax m))
("troubling" nil (t r ah1 b ax l ih0 ng))
("troudt" nil (t r aw1 d t))
("trough" nil (t r ao1 f))
("troughs" nil (t r ao1 f s))
("trounced" nil (t r aw1 n s t))
("trouncing" nil (t r aw1 n s ih0 ng))
("troung" nil (t r aw1 ng))
("troup" nil (t r uw1 p))
("troupe" nil (t r uw1 p))
("troupes" nil (t r uw1 p s))
("trousdale" nil (t er1 ax s d ey0 l))
("trouser" nil (t r aw1 z er0))
("trousers" nil (t r aw1 z er0 z))
("trout" nil (t r aw1 t))
("troutman" nil (t r aw1 t m ax n))
("troutner" nil (t r aw1 t n er0))
("troutt" nil (t r aw1 t))
("troutwine" nil (t r aw1 t w ay1 n))
("trovato" nil (t r ow0 v aa1 t ow0))
("trovatore" nil (t r ow1 v ax t ao1 r))
("trove" nil (t r ow1 v))
("trow" nil (t r ow1))
("trowbridge" nil (t r ow1 b r ih0 jh))
("trowell" nil (t r aa1 w eh0 l))
("trower" nil (t r aw1 er0))
("troxel" nil (t r aa1 k s ax l))
("troxell" nil (t r aa1 k s ax l))
("troxler" nil (t r aa1 k s l er0))
("troy" nil (t r oy1))
("troyan" nil (t r oy1 ax n))
("troyanos" nil (t r oy1 aa1 n ow0 s))
("troyat" nil (t r oy1 ae0 t))
("troyer" nil (t r oy1 er0))
("troyka" nil (t r oy1 k ax))
("troyu" nil (t r oy0 uw1))
("truancy" nil (t r uw1 ax n s iy0))
("truax" nil (t r uw1 ae0 k s))
("trubey" nil (t r uw1 b iy0))
("truby" nil (t r uw1 b iy0))
("trucco" nil (t r uw1 k ow0))
("truce" nil (t r uw1 s))
("truchan" nil (t r ah1 ch ax n))
("truck" nil (t r ah1 k))
("trucked" nil (t r ah1 k t))
("trucker" nil (t r ah1 k er0))
("truckers" nil (t r ah1 k er0 z))
("trucking" nil (t r ah1 k ih0 ng))
("truckload" nil (t r ah1 k l ow1 d))
("truckloads" nil (t r ah1 k l ow1 d z))
("trucks" nil (t r ah1 k s))
("truculent" nil (t r ah1 k y ax l ax n t))
("truda" nil (t r uw1 d ax))
("trude" nil (t r uw1 d))
("trudeau" nil (t r uw0 d ow1))
("trudel" nil (t r uw1 d ax l))
("trudell" nil (t r ah1 d ax l))
("trudge" nil (t r ah1 jh))
("trudged" nil (t r ah1 jh d))
("trudgen" nil (t r ah1 jh ax n))
("trudges" nil (t r ah1 jh ih0 z))
("trudging" nil (t r ah1 jh ih0 ng))
("trudia" nil (t r uw1 d iy0 ax))
("trudie" nil (t r uw1 d iy0))
("trudo" nil (t r uw1 d ow0))
("trudy" nil (t r uw1 d iy0))
("true" nil (t r uw1))
("trueblood" nil (t r uw1 b l ah1 d))
("truell" nil (t r uw1 ax l))
("truelove" nil (t r uw1 l ah1 v))
("trueman" nil (t r uh1 m ax n))
("truer" nil (t r uw1 er0))
("truesdale" nil (t r uw1 z d ey1 l))
("truesdell" nil (t r uw1 z d eh1 l))
("truest" nil (t r uw1 ih0 s t))
("truett" nil (t r uw1 t))
("truex" nil (t r uw1 eh1 k s))
("truffaut" nil (t r uw0 f ow1))
("truffle" nil (t r ah1 f ax l))
("truffles" nil (t r ah1 f ax l z))
("truglio" nil (t r ah1 g l iy0 ow0))
("truicko" nil (t r uw1 k ow0))
("truism" nil (t r uw1 ih0 z ax m))
("truitt" nil (t r uw1 t))
("trujillo" nil (t r uw0 hh iy1 ow0))
("trull" nil (t r ah1 l))
("trullinger" nil (t r ah1 l ih0 ng er0))
("trulock" nil (t r ah1 l ax k))
("trulove" nil (t r uw0 l ow1 v iy0))
("trulson" nil (t r ah1 l s ax n))
("truluck" nil (t r ah1 l ax k))
("truly" nil (t r uw1 l iy0))
("truman" nil (t r uw1 m ax n))
("trumble" nil (t r ah1 m b ax l))
("trumbo" nil (t r ah1 m b ow0))
("trumbull" nil (t r ah1 m b ax l))
("trumka" nil (t r ah1 m k ax))
("trump" nil (t r ah1 m p))
("trumped" nil (t r ah1 m p t))
("trumpet" nil (t r ah1 m p ax t))
("trumpeted" nil (t r ah1 m p ax t ih0 d))
("trumpeter" nil (t r ah1 m p ax t er0))
("trumpeting" nil (t r ah1 m p ax t ih0 ng))
("trumpets" nil (t r ah1 m p ax t s))
("trumpower" nil (t r ah1 m p ow1 er0))
("trumps" nil (t r ah1 m p s))
("truncated" nil (t r ah1 ng k ey1 t ih0 d))
("truncheons" nil (t r ah1 n ch ih0 n z))
("trundle" nil (t r ah1 n d ax l))
("trundled" nil (t r ah1 n d ax l d))
("trunk" nil (t r ah1 ng k))
("trunkline" nil (t r ah1 ng k l ay1 n))
("trunks" nil (t r ah1 ng k s))
("trunnell" nil (t r ah1 n ax l))
("trunzo" nil (t r ah1 n z ow0))
("truong" nil (t r ao1 ng))
("trupiano" nil (t r uw0 p iy0 aa1 n ow0))
("trupin" nil (t r uw1 p ih0 n))
("trupp" nil (t r ah1 p))
("truro" nil (t r uw1 r ow0))
("trus" nil (t r ah1 s))
("truscott" nil (t r ah1 s k ax t))
("truskowski" nil (t r ax s k ao1 f s k iy0))
("trusler" nil (t r ah1 s ax l er0))
("truslow" nil (t r ah1 s l ow0))
("truss" nil (t r ah1 s))
("trussell" nil (t r ah1 s ax l))
("trusses" nil (t r ah1 s ih0 z))
("trust" nil (t r ah1 s t))
("trustco" nil (t r ah1 s t k ow0))
("trustcorp" nil (t r ah1 s t k ao0 r p))
("trusted" nil (t r ah1 s t ih0 d))
("trustee" nil (t r ah1 s t iy1))
("trustees" nil (t r ah1 s t iy1 z))
("trusteeship" nil (t r ah1 s t iy1 sh ih1 p))
("trusthouse" nil (t r ah1 s t hh aw1 s))
("trusting" nil (t r ah1 s t ih0 ng))
("trusts" nil (t r ah1 s t s))
("trustworthiness" nil (t r ah1 s t w er1 dh iy0 n ax s))
("trustworthy" nil (t r ah1 s t w er1 dh iy0))
("trusty" nil (t r ah1 s t iy0))
("truszkowski" nil (t r ax sh k ao1 f s k iy0))
("truth" nil (t r uw1 th))
("truthful" nil (t r uw1 th f ax l))
("truthfully" nil (t r uw1 th f ax l iy0))
("truthfulness" nil (t r uw1 th f ax l n ax s))
("truths" nil (t r uw1 th s))
("truxal" nil (t r ah1 k s ax l))
("truxillo" nil (t r ah1 k s ih1 l ow0))
("try" nil (t r ay1))
("tryart" nil (t r ay1 aa1 r t))
("tryba" nil (t r ih1 b ax))
("trybus" nil (t r ih1 b ih0 s))
("trygg" nil (t r ih1 g))
("trygstad" nil (t r ih1 g s t ax d))
("trygve" nil (t r ih1 g v iy0))
("trying" nil (t r ay1 ih0 ng))
("tryon" nil (t r ay1 ax n))
("tryout" nil (t r ay1 aw1 t))
("tryouts" nil (t r ay1 aw1 t s))
("tryphena" nil (t r ih1 f ih0 n ax))
("tryst" nil (t r ay1 s t))
("trzaska" nil (t er0 z aa1 s k ax))
("trzcinski" nil (t er0 zh ih1 n s k iy0))
("trzeciak" nil (t er0 z eh1 ch iy0 ae0 k))
("ts" nil (t iy1 eh1 s))
("tsai" nil (t s ay1))
("tsakos" nil (t s aa1 k ow0 s))
("tsang" nil (t s ae1 ng))
("tsao" nil (t s aw1))
("tsar" nil (z aa1 r))
("tschantz" nil (ch ae1 n t s))
("tschetter" nil (ch eh1 t er0))
("tschida" nil (ch ay1 d ax))
("tschirhart" nil (ch er1 hh aa0 r t))
("tschopp" nil (ch aa1 p))
("tschudy" nil (ch uw1 d iy0))
("tschumi" nil (ch uw1 m iy0))
("tse" nil (t s iy1))
("tsemel" nil (t s eh1 m ax l))
("tseng" nil (t s eh1 ng))
("tsetse" nil (t s iy1 t s iy0))
("tsiang" nil (t s y ae1 ng))
("tsingtao" nil (s ih1 ng t aw1))
("tso" nil (t s ow1))
("tsongas" nil (t s ao1 ng g ax s))
("tsuda" nil (t s uw1 d ax))
("tsui" nil (t s uw1 iy0))
("tsuji" nil (t s uw1 jh iy0))
("tsukamoto" nil (t s uw0 k aa0 m ow1 t ow0))
("tsukuba" nil (t s uw1 k y uw1 b ax))
("tsunami" nil (t s uw0 n aa1 m iy0))
("tsuneo" nil (t s uw1 n iy0 ow0))
("tsutomu" nil (t s uw0 t ow1 m uw0))
("tu" nil (t uw1))
("tub" nil (t ah1 b))
("tuba" nil (t uw1 b ax))
("tubb" nil (t ah1 b))
("tubbs" nil (t ah1 b z))
("tubby" nil (t ah1 b iy0))
("tube" nil (t uw1 b))
("tuberculosis" nil (t ax b er1 k y ax l ow1 s ih0 s))
("tuberville" nil (t uw1 b er0 v ih1 l))
("tubes" nil (t uw1 b z))
("tubing" nil (t uw1 b ih0 ng))
("tubman" nil (t ah1 b m ax n))
("tubs" nil (t ah1 b z))
("tubular" nil (t uw1 b y ax l er0))
("tubule" nil (t uw1 b y uw0 l))
("tucci" nil (t uw1 ch iy0))
("tucciarone" nil (t uw0 ch er0 ow1 n iy0))
("tuccillo" nil (t uw0 ch ih1 l ow0))
("tucek" nil (t uw1 ch eh1 k))
("tuch" nil (t ah1 ch))
("tuchman" nil (t ah1 k m ax n))
("tucholski" nil (t ax hh ow1 l s k iy0))
("tuck" nil (t ah1 k))
("tucked" nil (t ah1 k t))
("tucker" nil (t ah1 k er0))
("tuckerman" nil (t ah1 k er0 m ax n))
("tuckett" nil (t ah1 k ih0 t))
("tuckey" nil (t ah1 k iy0))
("tucson" nil (t uw1 s aa1 n))
("tucuman" nil (t uw1 k y uw0 m ax n))
("tudisco" nil (t uw0 d iy1 s k ow0))
("tudjman" nil (t uh1 jh m ax n))
("tudor" nil (t uw1 d er0))
("tuel" nil (t uw1 l))
("tuell" nil (t uw1 l))
("tuesday" nil (t uw1 z d iy0))
("tuesdays" nil (t uw1 z d ey1 z))
("tufa" nil (t uw1 f ax))
("tufano" nil (t uw0 f aa1 n ow0))
("tuff" nil (t ah1 f))
("tufo" nil (t uw1 f ow0))
("tuft" nil (t ah1 f t))
("tufte" nil (t ah1 f t))
("tufted" nil (t ah1 f t ax d))
("tuftlike" nil (t ah1 f t l ay1 k))
("tufts" nil (t ah1 f t s))
("tug" nil (t ah1 g))
("tugboat" nil (t ah1 g b ow1 t))
("tugboats" nil (t ah1 g b ow1 t s))
("tugged" nil (t ah1 g d))
("tugging" nil (t ah1 g ih0 ng))
("tuggle" nil (t ah1 g ax l))
("tugman" nil (t ah1 g m ax n))
("tugs" nil (t ah1 g z))
("tugwell" nil (t ah1 g w eh1 l))
("tuinstra" nil (t uw0 ih1 n s t r ax))
("tuite" nil (t uw1 t))
("tuition" nil (t y uw0 ih1 sh ax n))
("tuitions" nil (t uw0 ih1 sh ax n z))
("tuk" nil (t uw1 k))
("tuks" nil (t ah1 k s))
("tulane" nil (t uw0 l ey1 n))
("tuley" nil (t y uw1 l iy0))
("tulip" nil (t uw1 l ax p))
("tulips" nil (t uw1 l ax p s))
("tull" nil (t ah1 l))
("tullar" nil (t ah1 l er0))
("tuller" nil (t ah1 l er0))
("tulley" nil (t uw1 l iy0))
("tullia" nil (t ah1 l iy0 ax))
("tullier" nil (t ah1 l iy0 er0))
("tullio" nil (t uw1 l iy0 ow0))
("tullis" nil (t ah1 l ih0 s))
("tullius" nil (t ah1 l iy0 ih0 s))
("tullo" nil (t uw1 l ow0))
("tulloch" nil (t ah1 l ax k))
("tullock" nil (t ah1 l ax k))
("tullos" nil (t ah1 l ow0 z))
("tully" nil (t ah1 l iy0))
("tulsa" nil (t ah1 l s ax))
("tuma" nil (t uw1 m ax))
("tuman" nil (ch uw1 m ax n))
("tumazos" nil (t uw0 m aa1 z ow0 s))
("tumble" nil (t ah1 m b ax l))
("tumbled" nil (t ah1 m b ax l d))
("tumbler" nil (t ah1 m b l er0))
("tumblers" nil (t ah1 m b l er0 z))
("tumbles" nil (t ah1 m b ax l z))
("tumbleson" nil (t ah1 m b l ih0 s ax n))
("tumblin" nil (t ah1 m b l ih0 n))
("tumbling" nil (t ah1 m b ax l ih0 ng))
("tumey" nil (t ah1 m iy0))
("tuminello" nil (t uw0 m iy0 n eh1 l ow0))
("tumlin" nil (t ah1 m l ih0 n))
("tumlinson" nil (t ah1 m l ih0 n s ax n))
("tummies" nil (t ah1 m iy0 z))
("tumminello" nil (t uw0 m iy0 n eh1 l ow0))
("tummond" nil (t ah1 m ax n d))
("tummy" nil (t ah1 m iy0))
("tumolo" nil (t uw0 m ow1 l ow0))
("tumor" nil (t uw1 m er0))
("tumors" nil (t uw1 m er0 z))
("tums" nil (t ah1 m z))
("tumult" nil (t uw1 m ax l t))
("tumultuous" nil (t uw1 m ah1 l ch uw0 ax s))
("tuna" nil (t uw1 n ax))
("tunable" nil (t uw1 n ax b ax l))
("tunafish" nil (t uw1 n ax f ih0 sh))
("tunas" nil (t uw1 n ax z))
("tunde" nil (t ah1 n d))
("tundra" nil (t ah1 n d r ax))
("tune" nil (t uw1 n))
("tuned" nil (t uw1 n d))
("tuneful" nil (t uw1 n f ax l))
("tuner" nil (t uw1 n er0))
("tunes" nil (t uw1 n z))
("tuneup" nil (t uw1 n ah1 p))
("tung" nil (t ah1 ng))
("tungate" nil (t uw1 ng ey0 t))
("tungsram" nil (t ah1 ng z r ae1 m))
("tungsrom" nil (t ah1 ng z r ax m))
("tungsten" nil (t ah1 ng s t ax n))
("tunic" nil (t uw1 n ih0 k))
("tunica" nil (t uw1 n ih0 k ax))
("tunicate" nil (t uw1 n ih0 k ax t))
("tunick" nil (t ah1 n ih0 k))
("tuning" nil (t uw1 n ih0 ng))
("tunis" nil (t uw1 n ih0 s))
("tunisia" nil (t uw1 n iy1 zh ax))
("tunisian" nil (t uw1 n iy1 zh ax n))
("tunison" nil (t ah1 n ih0 s ax n))
("tunkelang" nil (t ah1 ng k l ae1 ng))
("tunks" nil (t ah1 ng k s))
("tunku" nil (t ah1 ng k uw0))
("tunnel" nil (t ah1 n ax l))
("tunneling" nil (t ah1 n ax l ih0 ng))
("tunnell" nil (t ah1 n ax l))
("tunnels" nil (t ah1 n ax l z))
("tunney" nil (t ah1 n iy0))
("tunny" nil (t ah1 n iy0))
("tunstall" nil (t ah1 n s t ax l))
("tuohey" nil (t uw1 hh ey0))
("tuohy" nil (t uw1 iy0))
("tuomi" nil (t w ow1 m iy0))
("tupa" nil (t uw1 p ax))
("tupelo" nil (t y uw1 p ax l ow1))
("tupelov" nil (t uw1 p ax l aa0 v))
("tupper" nil (t ah1 p er0))
("tupperware" nil (t ah1 p er0 w eh1 r))
("tupy" nil (t uw1 p iy0))
("turandot" nil (t er0 ae1 n d ax t))
("turano" nil (t uh0 r aa1 n ow0))
("turban" nil (t er1 b ax n))
("turbans" nil (t er1 b ax n z))
("turbas" nil (t er1 b ax s))
("turben" nil (t er1 b ax n))
("turberville" nil (t er1 b er0 v ih1 l))
("turbett" nil (t er1 b ih0 t))
("turbeville" nil (t er1 b v ih0 l))
("turbidity" nil (t er0 b ih1 d ax t iy0))
("turbin" nil (t er1 b ih0 n))
("turbine" nil (t er1 b ay0 n))
("turbinen" nil (t er1 b ih0 n ax n))
("turbines" nil (t er1 b ay0 n z))
("turbo" nil (t er1 b ow0))
("turbocharge" nil (t er1 b ow0 ch aa1 r jh))
("turbocharged" nil (t er1 b ow0 ch aa1 r jh d))
("turbocharges" nil (t er1 b ow0 ch aa1 r jh ih0 z))
("turbocharging" nil (t er1 b ow0 ch aa1 r jh ih0 ng))
("turbofan" nil (t er1 b ow0 f ae1 n))
("turbojet" nil (t er1 b ow0 jh eh1 t))
("turboprop" nil (t er1 b ow0 p r aa1 p))
("turboprops" nil (t er1 b ow0 p r aa1 p s))
("turbos" nil (t er1 b ow0 z))
("turbot" nil (t er1 b ow1))
("turbotax" nil (t er1 b ow0 t ae1 k s))
("turbulence" nil (t er1 b y ax l ax n s))
("turbulent" nil (t er1 b y ax l ax n t))
("turbyfill" nil (t er1 b ih0 f ih0 l))
("turchi" nil (t uh1 r k iy0))
("turchin" nil (t er1 ch ih0 n))
("turchyn" nil (t er1 ch ih0 n))
("turck" nil (t er1 k))
("turco" nil (t er1 k ow0))
("turcott" nil (t er1 k ax t))
("turcotte" nil (t er0 k ao1 t))
("ture" nil (t uh1 r))
("turek" nil (t y uw1 r ih0 k))
("turenne" nil (t y uw1 r ih0 n))
("turf" nil (t er1 f))
("turgeon" nil (t er1 jh ih0 n))
("turgid" nil (t er1 jh ih0 d))
("turgut" nil (t er1 g ax t))
("turi" nil (t uh1 r iy0))
("turin" nil (t y uh1 r ih0 n))
("turing" nil (t uh1 r ih0 ng))
("turismo" nil (t uh1 r ih0 s m ow0))
("turk" nil (t er1 k))
("turkel" nil (t er1 k ax l))
("turkey" nil (t er1 k iy0))
("turkeys" nil (t er1 k iy0 z))
("turkic" nil (t er1 k ih0 k))
("turkington" nil (t er1 k ih0 ng t ax n))
("turkish" nil (t er1 k ih0 sh))
("turkmenistan" nil (t er1 k m eh1 n ih0 s t ae1 n))
("turko" nil (t er1 k ow0))
("turkovich" nil (t er1 k ax v ih0 ch))
("turks" nil (t er1 k s))
("turley" nil (t er1 l iy0))
("turlington" nil (t er1 l ih0 ng t ax n))
("turman" nil (t er1 m ax n))
("turmel" nil (t er1 m ax l))
("turmoil" nil (t er1 m oy1 l))
("turn" nil (t er1 n))
("turnabout" nil (t er1 n ax b aw1 t))
("turnage" nil (t er1 n ih0 jh))
("turnaround" nil (t er1 n er0 aw1 n d))
("turnarounds" nil (t er1 n er0 aw1 n d z))
("turnbaugh" nil (t er1 n b ao0))
("turnbo" nil (t uh1 r n b ow0))
("turnbough" nil (t er1 n b aw1))
("turnbow" nil (t er1 n b ow0))
("turnbridge" nil (t er1 n b r ih1 jh))
("turnbull" nil (t er1 n b uh1 l))
("turndown" nil (t er1 n d aw1 n))
("turned" nil (t er1 n d))
("turner" nil (t er1 n er0))
("turney" nil (t er1 n iy0))
("turnham" nil (t er1 n hh ax m))
("turning" nil (t er1 n ih0 ng))
("turnip" nil (t er1 n ax p))
("turnips" nil (t er1 n ax p s))
("turnipseed" nil (t er0 n ih1 p s iy0 d))
("turnkey" nil (t er1 n k iy1))
("turnley" nil (t er1 n l iy0))
("turnmire" nil (t er1 n m ay0 r))
("turnoff" nil (t er1 n ao1 f))
("turnout" nil (t er1 n aw1 t))
("turnouts" nil (t er1 n aw1 t s))
("turnover" nil (t er1 n ow1 v er0))
("turnovers" nil (t er1 n ow1 v er0 z))
("turnpike" nil (t er1 n p ay1 k))
("turnpikes" nil (t er1 n p ay1 k s))
("turnquist" nil (t er1 n k w ih0 s t))
("turns" nil (t er1 n z))
("turnstile" nil (t er1 n s t ay1 l))
("turnstiles" nil (t er1 n s t ay1 l z))
("turntable" nil (t er1 n t ey1 b ax l))
("turntables" nil (t er1 n t ey1 b ax l z))
("turo" nil (t uh1 r ow0))
("turow" nil (t uh1 r ow0))
("turowski" nil (t er0 ao1 f s k iy0))
("turpen" nil (t er1 p ax n))
("turpentine" nil (t er1 p ax n t ay1 n))
("turpin" nil (t er1 p ih0 n))
("turpitude" nil (t er1 p ih0 t uw1 d))
("turquoise" nil (t er1 k w oy0 z))
("turrell" nil (t ao1 r ax l))
("turrentine" nil (t uh0 r eh0 n t iy1 n iy0))
("turret" nil (t er1 ax t))
("turrets" nil (t er1 ax t s))
("turri" nil (t uh1 r iy0))
("turrill" nil (t ao1 r ax l))
("tursi" nil (t uh1 r s iy0))
("turski" nil (t er1 s k iy0))
("turtle" nil (t er1 t ax l))
("turtleneck" nil (t er1 t ax l n eh1 k))
("turtlenecks" nil (t er1 t ax l n eh1 k s))
("turtles" nil (t er1 t ax l z))
("turton" nil (t er1 t ax n))
("turvey" nil (t er0 v ey1))
("turville" nil (t er1 v ih1 l))
("turvy" nil (t er1 v iy0))
("tusa" nil (t uw1 s ax))
("tuscaloosa" nil (t ah1 s k ax l uw1 s ax))
("tuscan" nil (t ah1 s k ax n))
("tuscany" nil (t ah1 s k ax n iy0))
("tush" nil (t uh1 sh))
("tushes" nil (t uh1 sh ih0 z))
("tusing" nil (t uw1 s ih0 ng))
("tusk" nil (t ah1 s k))
("tuskegee" nil (t ah1 s k ax g iy1))
("tusks" nil (t ah1 s k s))
("tussey" nil (t ah1 s iy0))
("tussing" nil (t ah1 s ih0 ng))
("tussle" nil (t ah1 s ax l))
("tussled" nil (t ah1 s ax l d))
("tussles" nil (t ah1 s ax l z))
("tustin" nil (t ah1 s t ih0 n))
("tut" nil (t ah1 t))
("tutelage" nil (t y uw1 t ih0 l ih0 jh))
("tuten" nil (t y uw1 t ax n))
("tuthill" nil (t ah1 t hh ih1 l))
("tutino" nil (t uw0 t iy1 n ow0))
("tutko" nil (t ah1 t k ow0))
("tutor" nil (t uw1 t er0))
("tutored" nil (t uw1 t er0 d))
("tutorials" nil (t uw0 t ao1 r iy0 ax l z))
("tutoring" nil (t uw1 t er0 ih0 ng))
("tutors" nil (t uw1 t er0 z))
("tutsi" nil (t uw1 t s iy0))
("tutsis" nil (t uw1 t s iy0 z))
("tutt" nil (t ah1 t))
("tutterow" nil (t ah1 t er0 ow0))
("tuttle" nil (t ah1 t ax l))
("tutton" nil (t ah1 t ax n))
("tutu" nil (t uw1 t uw1))
("tutwiler" nil (t ah1 t w ay1 l er0))
("tux" nil (t ah1 k s))
("tuxedo" nil (t ah1 k s iy1 d ow0))
("tuxedos" nil (t ax k s iy1 d ow1 z))
("tuxford" nil (t ah1 k s f er0 d))
("tuxhorn" nil (t ah1 k s hh er0 n))
("tuyle" nil (t uw1 l))
("tuzla" nil (t uw1 z l ax))
("tuzzolino" nil (t uw0 t s ow0 l iy1 n ow0))
("tvedt" nil (t v eh1 d t))
("tveit" nil (t v iy1 t))
("tvsat" nil (t ax v s ae1 t))
("twaddell" nil (t w aa0 d eh1 l))
("twaddle" nil (t w aa1 d ax l))
("twain" nil (t w ey1 n))
("twang" nil (t w aa1 ng))
("twangy" nil (t w aa1 n jh iy0))
("twardowski" nil (t w er0 d aw1 s k iy0))
("twardy" nil (t w ao1 r d iy0))
("twarog" nil (t w ao1 r ao0 g))
("tweak" nil (t w iy1 k))
("tweaked" nil (t w iy1 k t))
("tweaking" nil (t w iy1 k ih0 ng))
("tweaks" nil (t w iy1 k s))
("twedt" nil (t w eh1 d t))
("tweed" nil (t w iy1 d))
("tweedie" nil (t w iy1 d iy0))
("tweedle" nil (t w iy1 d ax l))
("tweedy" nil (t w iy1 d iy0))
("tweet" nil (t w iy1 t))
("tweezers" nil (t w iy1 z er0 z))
("twelfth" nil (t w eh1 l f th))
("twelve" nil (t w eh1 l v))
("twelvth" nil (t w eh1 l v th))
("twenties" nil (t w eh1 n t iy0 z))
("twentieth" nil (t w eh1 n t iy0 ax th))
("twenty" nil (t w eh1 n t iy0))
("twersky" nil (t w er1 s k iy0))
("tweten" nil (t w iy1 t ax n))
("twice" nil (t w ay1 s))
("twichell" nil (t w ih1 ch ax l))
("twiddle" nil (t w ih1 d ax l))
("twiddy" nil (t w ih1 d iy0))
("twiford" nil (t w ih1 f er0 d))
("twigg" nil (t w ih1 g))
("twigged" nil (t w ih1 g d))
("twiggs" nil (t w ih1 g z))
("twiggy" nil (t w ih1 g iy0))
("twigs" nil (t w ih1 g z))
("twilight" nil (t w ay1 l ay1 t))
("twill" nil (t w ih1 l))
("twilley" nil (t w ih1 l iy0))
("twin" nil (t w ih1 n))
("twine" nil (t w ay1 n))
("twined" nil (t w ay1 n d))
("twinge" nil (t w ih1 n jh))
("twining" nil (t w ay1 n ih0 ng))
("twinjets" nil (t w ih1 n jh eh1 t s))
("twinkies" nil (t w ih1 ng k iy0 z))
("twinkle" nil (t w ih1 ng k ax l))
("twinkles" nil (t w ih1 ng k ax l z))
("twinkling" nil (t w ih1 ng k ax l ih0 ng))
("twins" nil (t w ih1 n z))
("twinsburg" nil (t w ih1 n z b er0 g))
("twirl" nil (t w er1 l))
("twirled" nil (t w er1 l d))
("twirler" nil (t w er1 l er0))
("twirling" nil (t w er1 l ih0 ng))
("twirls" nil (t w er1 l z))
("twiss" nil (t w ih1 s))
("twist" nil (t w ih1 s t))
("twisted" nil (t w ih1 s t ax d))
("twister" nil (t w ih1 s t er0))
("twisters" nil (t w ih1 s t er0 z))
("twisting" nil (t w ih1 s t ih0 ng))
("twists" nil (t w ih1 s t s))
("twisty" nil (t w ih1 s t iy0))
("twitch" nil (t w ih1 ch))
("twitched" nil (t w ih1 ch t))
("twitchell" nil (t w ih1 ch ax l))
("twitches" nil (t w ih1 ch ih0 z))
("twitching" nil (t w ih1 ch ih0 ng))
("twite" nil (t w ay1 t))
("twitty" nil (t w ih1 t iy0))
("two" nil (t uw1))
("twofold" nil (t uw1 f ow1 l d))
("twohig" nil (t w ow1 hh ih0 g))
("twombly" nil (t w uw1 m b l iy0))
("twomey" nil (t w aa1 m iy0))
("twonshein" nil (t w aa1 n sh ay1 n))
("tworek" nil (t w er1 ih0 k))
("twos" nil (t uw1 z))
("twosome" nil (t uw1 s ax m))
("twothirds" nil (t uw1 th er1 d z))
("twyford" nil (t w ay1 f er0 d))
("twyla" nil (t w ay1 l ax))
("twyman" nil (t w ay1 m ax n))
("ty" nil (t ay1))
("tybalt" nil (t ih1 b ax l t))
("tyburski" nil (t ay0 b er1 s k iy0))
("tyce" nil (t ay1 s))
("tyco" nil (t ay1 k ow0))
("tycoon" nil (t ay0 k uw1 n))
("tycoons" nil (t ay0 k uw1 n z))
("tydings" nil (t ay1 d ih0 ng z))
("tye" nil (t ay1))
("tyer" nil (t ay1 er0))
("tygar" nil (t ay1 g aa1 r))
("tygart" nil (t ay1 g aa1 r t))
("tyger" nil (t ay1 g er0))
("tying" nil (t ay1 ih0 ng))
("tyke" nil (t ay1 k))
("tylan" nil (t ay1 l ax n))
("tylee" nil (t ay1 l iy0))
("tylenol" nil (t ay1 l ax n ao0 l))
("tyler" nil (t ay1 l er0))
("tylka" nil (t ih1 l k ax))
("tyminski" nil (t ih0 m ih1 n s k iy0))
("tymnet" nil (t ay1 m n eh1 t))
("tympanic" nil (t ih0 m p ae1 n ih0 k))
("tympanum" nil (t ih1 m p ax n ax m))
("tynan" nil (t ay1 n ax n))
("tyndall" nil (t ih1 n d ax l))
("tyner" nil (t ay1 n er0))
("tynes" nil (t ay1 n z))
("tyo" nil (t ay1 ow0))
("type" nil (t ay1 p))
("typecasting" nil (t ay1 p k ae1 s t ih0 ng))
("typed" nil (t ay1 p t))
("typeface" nil (t ay1 p f ey1 s))
("typefaces" nil (t ay1 p f ey1 s ax z))
("types" nil (t ay1 p s))
("typeset" nil (t ay1 p s eh1 t))
("typesetting" nil (t ay1 p s eh1 t ih0 ng))
("typewriter" nil (t ay1 p r ay1 t er0))
("typewriters" nil (t ay1 p r ay1 t er0 z))
("typewriting" nil (t ay1 p r ay1 t ih0 ng))
("typewritten" nil (t ay1 p r ih1 t ax n))
("typhoid" nil (t ay1 f oy0 d))
("typhoon" nil (t ay1 f uw1 n))
("typhoons" nil (t ay1 f uw1 n z))
("typhus" nil (t ay1 f ax s))
("typical" nil (t ih1 p ax k ax l))
("typically" nil (t ih1 p ax k l iy0))
("typified" nil (t ih1 p ax f ay1 d))
("typifies" nil (t ih1 p ax f ay1 z))
("typify" nil (t ih1 p ax f ay1))
("typing" nil (t ay1 p ih0 ng))
("typist" nil (t ay1 p ih0 s t))
("typists" nil (t ay1 p ih0 s t s))
("typo" nil (t ay1 p ow0))
("typographical" nil (t ay1 p ax g r ae1 f ih0 k ax l))
("typography" nil (t ax p aa1 g r ax f iy0))
("tyra" nil (t ay1 r ax))
("tyrannical" nil (t er0 ae1 n ih0 k ax l))
("tyrannies" nil (t ih1 r ax n iy0 z))
("tyranny" nil (t ih1 r ax n iy0))
("tyrant" nil (t ay1 r ax n t))
("tyrants" nil (t ay1 r ax n t s))
("tyre" nil (t ay1 r))
("tyree" nil (t ay0 r iy1))
("tyrell" nil (t ay1 r ax l))
("tyrone" nil (t ay1 r ow1 n))
("tyrrell" nil (t ay1 r eh1 l))
("tyrrhenian" nil (t er0 iy1 n iy0 ax n))
("tysinger" nil (t ay1 s ih0 n jh er0))
("tyson" nil (t ay1 s ax n))
("tysons" nil (t ay1 s ax n z))
("tyus" nil (t y uw1 s))
("u" nil (y uw1))
("uarco" nil (y uw1 aa1 r k ow0))
("uart" nil (y uw1 aa0 r t))
("ubben" nil (ah1 b ax n))
("uber" nil (y uw1 b er0))
("ubiquitous" nil (y uw0 b ih1 k w ih0 t ax s))
("ubiquity" nil (y uw0 b ih1 k w ih0 t iy0))
("uccel" nil (uw1 ch ax l))
("ucci" nil (uw1 ch iy0))
("uchida" nil (uw0 k iy1 d ax))
("uclaf" nil (ah1 k l ae0 f))
("udagawa" nil (uw1 d aa0 g aa1 w ax))
("udale" nil (y uw1 d ey1 l))
("udall" nil (y uw1 d ax l))
("uddin" nil (ah1 d ih0 n))
("ude" nil (uw1 d))
("udell" nil (ah1 d ax l))
("udelle" nil (ax d eh1 l))
("uden" nil (uw1 d ax n))
("udo" nil (uw1 d ow1))
("udolf" nil (ah1 d ow0 l f))
("udvar" nil (ah1 d v aa0 r))
("udverhye" nil (uw1 d v eh1 r hh iy0))
("udy" nil (y uw1 d iy0))
("uebel" nil (uh1 b ax l))
("ueberroth" nil (y uw1 b er0 r ao1 th))
("uecker" nil (uh1 k er0))
("ueckert" nil (uh1 k er0 t))
("ueda" nil (w eh1 d ax))
("uehara" nil (uw0 ey0 hh aa1 r ax))
("uehling" nil (uh1 l ih0 ng))
("ueki" nil (y uw1 eh1 k iy0))
("ueland" nil (uh1 l ax n d))
("uelman" nil (uh1 l m ax n))
("uelmen" nil (uh1 l m eh0 n))
("uemura" nil (uw1 ih0 m uw1 r ax))
("ueno" nil (w eh1 n ow0))
("uffelman" nil (ah1 f ax l m ax n))
("ugalde" nil (uw0 g aa1 l d iy0))
("uganda" nil (y uw0 g ae1 n d ax))
("ugandan" nil (y uw1 g ae1 n d ax n))
("ugarte" nil (uw0 g aa1 r t iy0))
("ugh" nil (ah1 g))
("uglier" nil (ah1 g l iy0 er0))
("ugliest" nil (ah1 g l iy0 ax s t))
("ugliness" nil (ah1 g l iy0 n ax s))
("ugly" nil (ah1 g l iy0))
("ugolin" nil (y uw1 g ax l ih0 n))
("ugolini" nil (uw0 g ow0 l iy1 n iy0))
("ugric" nil (y uw1 g r ih0 k))
("uh" nil (ah1))
("uhde" nil (ah1 d))
("uher" nil (uw1 er0))
("uhl" nil (ah1 l))
("uhland" nil (uw1 l ax n d))
("uhle" nil (uw1 ax l))
("uhler" nil (uw1 l er0))
("uhlich" nil (uw1 l ih0 k))
("uhlig" nil (uw1 l ih0 g))
("uhlir" nil (uw1 l er0))
("uhlman" nil (uw1 l m ax n))
("uhlmann" nil (uw1 l m ax n))
("uhls" nil (uw1 l z))
("uhr" nil (uh1 r))
("uhrich" nil (uh1 r ih0 k))
("uhrig" nil (uh1 r ih0 g))
("uhrin" nil (uh1 r ih0 n))
("ukase" nil (y uw1 k ey1 z))
("ukman" nil (ah1 k m ax n))
("ukraine" nil (y uw0 k r ey1 n))
("ukrainian" nil (y uw0 k r ey1 n iy0 ax n))
("ukrainians" nil (y uw0 k r ey1 n iy0 ax n z))
("ukulele" nil (y uw1 k ax l ey1 l iy0))
("ul" nil (ah1 l))
("ula" nil (y uw1 l ax))
("ulam" nil (y uw1 l ae0 m))
("uland" nil (y uw1 l ax n d))
("ulatowski" nil (ax l ax t ao1 f s k iy0))
("ulbrich" nil (ah1 l b r ih0 k))
("ulbricht" nil (ah1 l b r ih0 k t))
("ulcer" nil (ah1 l s er0))
("ulcers" nil (ah1 l s er0 z))
("ulch" nil (ah1 l ch))
("ulery" nil (ax l er1 iy0))
("ulf" nil (ah1 l f))
("ulfred" nil (ah1 l f er0 d))
("ulger" nil (ah1 l g er0))
("uliano" nil (uw0 l iy0 aa1 n ow0))
("ulibarri" nil (uw0 l iy0 b aa1 r iy0))
("ulicny" nil (ax l ih1 k n iy0))
("ulima" nil (uw0 l iy1 m ax))
("ulin" nil (y uw1 l ih0 n))
("ulitsa" nil (y uw0 l ih1 t s ax))
("ulland" nil (ah1 l ax n d))
("ullenberg" nil (y uw1 l ax n b er0 g))
("ullery" nil (uw1 l er0 iy0))
("ullman" nil (ah1 l m ax n))
("ullmann" nil (ah1 l m ax n))
("ullmer" nil (ah1 l m er0))
("ulloa" nil (uw0 l ow1 ax))
("ullock" nil (ah1 l ax k))
("ullom" nil (ah1 l ax m))
("ullrich" nil (uh1 l r ih0 k))
("ulm" nil (ah1 l m))
("ulman" nil (ah1 l m ax n))
("ulmar" nil (ah1 l m er0))
("ulmer" nil (ah1 l m er0))
("ulmus" nil (ah1 l m ax s))
("ulrey" nil (ah1 l r iy0))
("ulric" nil (ah1 l r ih0 k))
("ulrica" nil (ah1 l r ih0 k ax))
("ulrich" nil (ah1 l r ih0 ch))
("ulrick" nil (ah1 l r ih0 k))
("ulsan" nil (ah1 l s ax n))
("ulsh" nil (ah1 l sh))
("ulshafer" nil (ah1 l sh ax f er0))
("ulster" nil (ah1 l s t er0))
("ulterior" nil (ax l t ih1 r iy0 er0))
("ultima" nil (ah1 l t ih0 m ax))
("ultimate" nil (ah1 l t ax m ax t))
("ultimately" nil (ah1 l t ax m ax t l iy0))
("ultimatum" nil (ah1 l t ax m ey1 t ax m))
("ultimatums" nil (ah1 l t ax m ey1 t ax m z))
("ultra" nil (ah1 l t r ax))
("ultraconservative" nil (ah1 l t r ax k ax n s er1 v ax t ih0 v))
("ultrair" nil (ah1 l t r eh1 r))
("ultramar" nil (ah1 l t r ax m aa0 r))
("ultramodern" nil (ah1 l t r ax m aa1 d er0 n))
("ultranationalist" nil (ah1 l t r ax n ae1 sh ax n ax l ax s t))
("ultranationalists" nil (ah1 l t r ax n ae1 sh ax n ax l ax s t s))
("ultrasonic" nil (ah1 l t r ax s aa1 n ih0 k))
("ultrasound" nil (ah1 l t r ax s aw1 n d))
("ultrasystems" nil (ah1 l t r ax s ih1 s t ax m z))
("ultraviolet" nil (ah1 l t r ax v ay1 ax l ih0 t))
("ulundi" nil (y uw0 l ah1 n d iy0))
("ulva" nil (uw1 l v ax))
("ulvert" nil (ah1 l v er0 t))
("ulysses" nil (y uw0 l ih1 s iy0 z))
("um" nil (ah1 m))
("umana" nil (uw0 m aa1 n ax))
("umansky" nil (ax m ae1 n s k iy0))
("umbach" nil (ah1 m b aa1 k))
("umbarger" nil (ah1 m b aa1 r g er0))
("umbaugh" nil (ah1 m b aw0))
("umbel" nil (ah1 m b ax l))
("umberger" nil (ah1 m b er0 g er0))
("umberto" nil (ax m b eh1 r t ow1))
("umbilical" nil (ax m b ih1 l ih0 k ax l))
("umble" nil (ah1 m b ax l))
("umbra" nil (ah1 m b r ax))
("umbrage" nil (ah1 m b r ih0 jh))
("umbrella" nil (ax m b r eh1 l ax))
("umbrellas" nil (ax m b r eh1 l ax z))
("umetsu" nil (y uw1 m eh1 t s uw0))
("umfleet" nil (ah1 m f l iy1 t))
("umholtz" nil (ah1 m hh ow1 l t s))
("umland" nil (ah1 m l ax n d))
("umlasi" nil (uw1 m l aa1 s iy0))
("umlauf" nil (ah1 m l aw0 f))
("umm" nil (ah1 m))
("ummel" nil (ah1 m ax l))
("umphlett" nil (ax m f l eh1 t))
("umphress" nil (ah1 m f r ih0 s))
("umphrey" nil (ah1 m f r iy0))
("umpire" nil (ah1 m p ay1 er0))
("umpires" nil (ah1 m p ay1 er0 z))
("umpteen" nil (ah1 m p t iy1 n))
("umpteenth" nil (ah1 m p t iy1 n th))
("umscheid" nil (ah1 m sh ay1 d))
("umstead" nil (ah1 m s t eh1 d))
("un" nil (ah1 n))
("una" nil (uw1 n ax))
("unabashed" nil (ah1 n ax b ae1 sh t))
("unabashedly" nil (ah1 n ax b ae1 sh ih0 d l iy0))
("unabated" nil (ah1 n ax b ey1 t ih0 d))
("unable" nil (ax n ey1 b ax l))
("unabridged" nil (ah1 n ax b r ih1 jh d))
("unacceptable" nil (ah1 n ae0 k s eh1 p t ax b ax l))
("unacceptably" nil (ah1 n ae0 k s eh1 p t ax b l iy0))
("unaccompanied" nil (ah1 n ax k ah1 m p ax n iy0 d))
("unaccountable" nil (ah1 n ax k aw1 n t ax b ax l))
("unaccountably" nil (ah1 n ax k aw1 n t ax b l iy0))
("unaccounted" nil (ah1 n ax k aw1 n t ih0 d))
("unaccredited" nil (ah1 n ax k r eh1 d ih1 t ih0 d))
("unaccustomed" nil (ah1 n ax k ah1 s t ax m d))
("unaddressed" nil (ah1 n ax d r eh1 s t))
("unadited" nil (ax n ae1 d ih1 t ih0 d))
("unadjusted" nil (ah1 n ax jh ah1 s t ih0 d))
("unadorned" nil (ah1 n ax d ao1 r n d))
("unadulterated" nil (ah1 n ax d ah1 l t er0 ey1 t ih0 d))
("unaffected" nil (ah1 n ax f eh1 k t ih0 d))
("unaffiliated" nil (ax n ax f ih1 l iy0 ey1 t ih0 d))
("unaffordable" nil (ax n ax f ao1 r d ax b ax l))
("unafraid" nil (ah1 n ax f r ey1 d))
("unaided" nil (ax n ey1 d ih0 d))
("unallocated" nil (ax n ae1 l ax k ey1 t ih0 d))
("unallowable" nil (ah1 n ax l aw1 ax b ax l))
("unalloyed" nil (ax n ae1 l oy1 d))
("unalterable" nil (ax n ao1 l t er0 ax b ax l))
("unaltered" nil (ax n ao1 l t er0 d))
("unambiguous" nil (ah1 n ae0 m b ih1 g y ax w ax s))
("unambiguously" nil (ax n ae0 m b ih1 g y uw0 ax s l iy0))
("unamended" nil (ax n ax m eh1 n d ih0 d))
("unamortized" nil (ax n ae1 m er0 t ay1 z d))
("unangst" nil (ao1 n ax ng s t))
("unanimity" nil (y uw1 n ax n ih1 m ax t iy0))
("unanimous" nil (y uw0 n ae1 n ax m ax s))
("unanimously" nil (y uw0 n ae1 n ax m ax s l iy0))
("unannounced" nil (ah1 n ax n aw1 n s t))
("unanswerable" nil (ah1 n ae1 n s er0 ax b ax l))
("unanswered" nil (ah1 n ae1 n s er0 d))
("unanticipated" nil (ah1 n ae0 n t ih1 s ax p ey1 t ih0 d))
("unapologetic" nil (ah1 n ax p aa1 l ax jh eh1 t ih0 k))
("unappealing" nil (ah1 n ax p iy1 l ih0 ng))
("unappetizing" nil (ah1 n ae1 p ax t ay1 z ih0 ng))
("unappreciated" nil (ah1 n ax p r iy1 sh iy0 ey1 t ih0 d))
("unapproved" nil (ah1 n ax p r uw1 v d))
("unarmed" nil (ax n aa1 r m d))
("unashamed" nil (ah1 n ax sh ey1 m d))
("unashamedly" nil (ah1 n ax sh ey1 m ax d l iy0))
("unassailable" nil (ah1 n ax s ey1 l ax b ax l))
("unassisted" nil (ah1 n ax s ih1 s t ih0 d))
("unassuming" nil (ah1 n ax s uw1 m ih0 ng))
("unattached" nil (ah1 n ax t ae1 ch t))
("unattainable" nil (ah1 n ax t ey1 n ax b ax l))
("unattended" nil (ax n ax t eh1 n d ih0 d))
("unattractive" nil (ah1 n ax t r ae1 k t ih0 v))
("unaudited" nil (ax n ao1 d ax t ih0 d))
("unauthentic" nil (ah1 n ao0 th eh1 n t ih0 k))
("unauthorized" nil (ax n ao1 th er0 ay1 z d))
("unavailability" nil (ax n ax v ey1 l ax b ih1 l ih0 t iy0))
("unavailable" nil (ah1 n ax v ey1 l ax b ax l))
("unavailing" nil (ah1 n ax v ey1 l ih0 ng))
("unavoidable" nil (ah1 n ax v oy1 d ax b ax l))
("unavoidably" nil (ah1 n ax v oy1 d ax b l iy0))
("unaware" nil (ah1 n ax w eh1 r))
("unbalanced" nil (ax n b ae1 l ax n s t))
("unbearable" nil (ax n b eh1 r ax b ax l))
("unbearably" nil (ax n b eh1 r ax b l iy0))
("unbeatable" nil (ah1 n b iy1 t ax b ax l))
("unbeaten" nil (ax n b iy1 t ax n))
("unbeknown" nil (ah1 n b ih0 n ow1 n))
("unbeknownst" nil (ax n b iy0 n ow1 n s t))
("unbelievable" nil (ah1 n b ax l iy1 v ax b ax l))
("unbelievably" nil (ah1 n b ax l iy1 v ax b l iy0))
("unbelieving" nil (ah1 n b ax l iy1 v ih0 ng))
("unbending" nil (ah1 n b eh1 n d ih0 ng))
("unbiased" nil (ah1 n b ay1 ax s t))
("unblemished" nil (ax n b l eh1 m ih0 sh t))
("unbolted" nil (ah1 n b ow1 l t ih0 d))
("unbook" nil (ax n b uh1 k))
("unborn" nil (ah1 n b ao1 r n))
("unbound" nil (ax n b aw1 n d))
("unbounded" nil (ax n b aw1 n d ih0 d))
("unbranded" nil (ax n b r ae1 n d ih0 d))
("unbridled" nil (ax n b r ay1 d ax l d))
("unbroken" nil (ax n b r ow1 k ax n))
("unbuildable" nil (ax n b ih1 l d ax b ax l))
("unbuilt" nil (ah1 n b ih1 l t))
("unbundling" nil (ax n b ah1 n d l ih0 ng))
("unburdened" nil (ax n b er1 d ax n d))
("unburned" nil (ax n b er1 n d))
("unbuttoned" nil (ax n b ah1 t ax n d))
("uncalled" nil (ax n k ao1 l d))
("uncannily" nil (ax n k ae1 n ax l iy0))
("uncanny" nil (ax n k ae1 n iy0))
("uncapher" nil (ah1 n k ax f er0))
("uncaring" nil (ax n k eh1 r ih0 ng))
("unceasing" nil (ax n s iy1 s ih0 ng))
("uncensored" nil (ax n s eh1 n s er0 d))
("unceremonious" nil (ah1 n s eh1 r ax m ow1 n iy0 ax s))
("unceremoniously" nil (ah1 n s eh1 r ax m ow1 n iy0 ax s l iy0))
("uncertain" nil (ax n s er1 t ax n))
("uncertainties" nil (ax n s er1 t ax n t iy0 z))
("uncertainty" nil (ax n s er1 t ax n t iy0))
("unchallenged" nil (ax n ch ae1 l ih0 n jh d))
("unchanged" nil (ax n ch ey1 n jh d))
("unchanging" nil (ax n ch ey1 n jh ih0 ng))
("uncharacteristic" nil (ah1 n k eh1 r ih0 k t er0 ih1 s t ih0 k))
("uncharacteristically" nil (ah1 n k eh1 r ih0 k t er0 ih1 s t ih0 k l iy0))
("uncharted" nil (ax n ch aa1 r t ih0 d))
("unchecked" nil (ax n ch eh1 k t))
("uncivilized" nil (ax n s ih1 v ax l ay0 z d))
("unclaimed" nil (ax n k l ey1 m d))
("unclassified" nil (ax n k l ae1 s ih0 f ay1 d))
("uncle" nil (ah1 ng k ax l))
("unclean" nil (ax n k l iy1 n))
("unclear" nil (ax n k l ih1 r))
("uncles" nil (ah1 ng k ax l z))
("uncluttered" nil (ax n k l ah1 t er0 d))
("uncoated" nil (ax n k ow1 t ih0 d))
("uncoil" nil (ah1 n k oy1 l))
("uncoiled" nil (ah1 n k oy1 l d))
("uncollectable" nil (ax n k ax l eh1 k t ax b ax l))
("uncollected" nil (ax n k ax l eh1 k t ih0 d))
("uncollectible" nil (ax n k ax l eh1 k t ih0 b ax l))
("uncomfortable" nil (ax n k ah1 m f er0 t ax b ax l))
("uncomfortably" nil (ax n k ah1 m f t ax b l iy0))
("uncommitted" nil (ah1 n k ax m ih1 t ih0 d))
("uncommon" nil (ax n k aa1 m ax n))
("uncommonly" nil (ah1 n k aa1 m ax n l iy0))
("uncompensated" nil (ax n k aa1 m p ax n s ey1 t ih0 d))
("uncompetitive" nil (ax n k ax m p eh1 t ax t ih0 v))
("uncompleted" nil (ah1 n k ax m p l iy1 t ih0 d))
("uncomplicated" nil (ax n k aa1 m p l ax k ey1 t ih0 d))
("uncompromising" nil (ax n k aa1 m p r ax m ay0 z ih0 ng))
("unconcealed" nil (ah1 n k ax n s iy1 l d))
("unconcern" nil (ah1 n k ax n s er1 n))
("unconcerned" nil (ah1 n k ax n s er1 n d))
("unconditional" nil (ah1 n k ax n d ih1 sh ax n ax l))
("unconditionally" nil (ah1 n k ax n d ih1 sh ax n ax l iy0))
("unconfined" nil (ah1 n k ax n f ay1 n d))
("unconfirmed" nil (ah1 n k ax n f er1 m d))
("unconnected" nil (ah1 n k ax n eh1 k t ih0 d))
("unconscionable" nil (ax n k aa1 n sh ax n ax b ax l))
("unconscious" nil (ah1 n k aa1 n sh ax s))
("unconsciously" nil (ah1 n k aa1 n sh ax s l iy0))
("unconsolidated" nil (ax n k ax n s aa1 l ax d ey1 t ih0 d))
("unconstitutional" nil (ah1 n k aa1 n s t ax t uw1 sh ax n ax l))
("unconstitutionally" nil (ah1 n k aa1 n s t ax t uw1 sh ax n ax l iy0))
("unconstrained" nil (ah1 n k ax n s t r ey1 n d))
("uncontested" nil (ah1 n k ax n t eh1 s t ih0 d))
("uncontrollable" nil (ah1 n k ax n t r ow1 l ax b ax l))
("uncontrollably" nil (ah1 n k ax n t r ow1 l ax b l iy0))
("uncontrolled" nil (ah1 n k ax n t r ow1 l d))
("uncontroversial" nil (ah1 n k aa1 n t r ax v er1 sh ax l))
("unconventional" nil (ah1 n k ax n v eh1 n sh ax n ax l))
("unconverted" nil (ah1 n k ax n v er1 t ih0 d))
("unconvinced" nil (ah1 n k ax n v ih1 n s t))
("unconvincing" nil (ah1 n k ax n v ih1 n s ih0 ng))
("uncooperative" nil (ax n k ow0 aa1 p er0 ax t ih0 v))
("uncoordinated" nil (ax n k ow0 ao1 r d ax n ey0 t ih0 d))
("uncorrected" nil (ah1 n k er0 eh1 k t ih0 d))
("uncounted" nil (ah1 n k aw1 n t ih0 d))
("uncouth" nil (ah1 n k uw1 th))
("uncover" nil (ax n k ah1 v er0))
("uncovered" nil (ax n k ah1 v er0 d))
("uncovering" nil (ax n k ah1 v er0 ih0 ng))
("uncovers" nil (ah1 n k ah1 v er0 z))
("uncritical" nil (ax n k r ih1 t ih0 k ax l))
("uncritically" nil (ax n k r ih1 t ih0 k ax l iy0))
("unctuous" nil (ah1 ng ch w ax s))
("uncured" nil (ah1 n k y er1 d))
("uncut" nil (ah1 n k ah1 t))
("und" nil (ah1 n d))
("undamaged" nil (ah1 n d ae1 m ax jh d))
("undated" nil (ah1 n d ey1 t ih0 d))
("undaunted" nil (ax n d ao1 n t ih0 d))
("undecided" nil (ah1 n d ih0 s ay1 d ih0 d))
("undeclared" nil (ax n d ih0 k l eh1 r d))
("undefeated" nil (ah1 n d ih0 f iy1 t ih0 d))
("undefined" nil (ah1 n d ih0 f ay1 n d))
("undelivered" nil (ah1 n d ih0 l ih1 v er0 d))
("undemocratic" nil (ah1 n d eh0 m ax k r ae1 t ih0 k))
("undeniable" nil (ah1 n d ih0 n ay1 ax b ax l))
("undeniably" nil (ah1 n d ih0 n ay1 ax b l iy0))
("under" nil (ah1 n d er0))
("underachievers" nil (ah1 n d er0 ax ch iy1 v er0 z))
("underage" nil (ah1 n d er0 ih0 jh))
("underarm" nil (ah1 n d er0 aa1 r m))
("underbelly" nil (ah1 n d er0 b eh1 l iy0))
("underberg" nil (ah1 n d er0 b er0 g))
("underbid" nil (ah1 n d er0 b ih1 d))
("underbrush" nil (ah1 n d er0 b r ah1 sh))
("undercapitalized" nil (ax n d er0 k ae1 p ax t ax l ay1 z d))
("undercarriage" nil (ah1 n d er0 k ae1 r ih0 jh))
("underclass" nil (ah1 n d er0 k l ae1 s))
("undercoat" nil (ah1 n d er0 k ow1 t))
("undercooked" nil (ax n d er0 k uh1 k t))
("undercount" nil (ah1 n d er0 k aw1 n t))
("undercounted" nil (ah1 n d er0 k aw1 n t ih0 d))
("undercover" nil (ah1 n d er0 k ah1 v er0))
("undercurrent" nil (ah1 n d er0 k er1 ax n t))
("undercurrents" nil (ah1 n d er0 k er1 ax n t s))
("undercut" nil (ah1 n d er0 k ah1 t))
("undercuts" nil (ah1 n d er0 k ah1 t s))
("undercutting" nil (ah1 n d er0 k ah1 t ih0 ng))
("underdahl" nil (ah1 n d er0 d aa1 l))
("underdeveloped" nil (ah1 n d er0 d ih0 v eh1 l ax p t))
("underdevelopment" nil (ax n d er0 d ax v eh1 l ax p m ax n t))
("underdog" nil (ah1 n d er0 d ao1 g))
("underdogs" nil (ah1 n d er0 d ao1 g z))
("underdown" nil (ah1 n d er0 d aw1 n))
("underemployed" nil (ah1 n d er0 ih0 m p l oy1 d))
("underemployment" nil (ax n d er0 ih0 m p l oy1 m ax n t))
("underestimate" nil (ah1 n d er0 eh1 s t ax m ey1 t))
("underestimated" nil (ah1 n d er0 eh1 s t ax m ey1 t ih0 d))
("underestimates" nil (ah1 n d er0 eh1 s t ih0 m ih0 t s))
("underestimating" nil (ah1 n d er0 eh1 s t ih0 m ey1 t ih0 ng))
("underfinanced" nil (ax n d er0 f ih0 n ae1 n s t))
("underfoot" nil (ah1 n d er0 f uh1 t))
("underfunded" nil (ah1 n d er0 f ah1 n d ih0 d))
("underfunding" nil (ah1 n d er0 f ah1 n d ih0 ng))
("undergarment" nil (ah1 n d er0 g aa1 r m ax n t))
("undergarments" nil (ah1 n d er0 g aa1 r m ax n t s))
("undergo" nil (ah1 n d er0 g ow1))
("undergoes" nil (ah1 n d er0 g ow1 z))
("undergoing" nil (ah1 n d er0 g ow1 ih0 ng))
("undergone" nil (ah1 n d er0 g ao1 n))
("undergrad" nil (ah1 n d er0 g r ae1 d))
("undergraduate" nil (ah1 n d er0 g r ae1 jh ax w ax t))
("undergraduates" nil (ah1 n d er0 g r ae1 jh ax w ax t s))
("underground" nil (ah1 n d er0 g r aw1 n d))
("undergrowth" nil (ah1 n d er0 g r ow1 th))
("underhanded" nil (ah1 n d er0 hh ae1 n d ih0 d))
("underhill" nil (ah1 n d er0 hh ih1 l))
("underkoffler" nil (ah1 n d er0 k ax f ax l er0))
("underlie" nil (ah1 n d er0 l ay1))
("underlies" nil (ah1 n d er0 l ay1 z))
("underline" nil (ah1 n d er0 l ay1 n))
("underlined" nil (ah1 n d er0 l ay1 n d))
("underlines" nil (ah1 n d er0 l ay1 n z))
("underling" nil (ah1 n d er0 l ih0 ng))
("underlings" nil (ah1 n d er0 l ih0 ng z))
("underlining" nil (ah1 n d er0 l ay1 n ih0 ng))
("underlying" nil (ah1 n d er0 l ay1 ih0 ng))
("undermanned" nil (ah1 n d er0 m ae1 n d))
("undermine" nil (ah1 n d er0 m ay1 n))
("undermined" nil (ah1 n d er0 m ay1 n d))
("undermines" nil (ah1 n d er0 m ay1 n z))
("undermining" nil (ah1 n d er0 m ay1 n ih0 ng))
("underneath" nil (ah1 n d er0 n iy1 th))
("underpaid" nil (ah1 n d er0 p ey1 d))
("underpass" nil (ah1 n d er0 p ae1 s))
("underpaying" nil (ah1 n d er0 p ey1 ih0 ng))
("underpayment" nil (ah1 n d er0 p ey1 m ax n t))
("underpayments" nil (ah1 n d er0 p ey1 m ax n t s))
("underperform" nil (ah1 n d er0 p er0 f ao1 r m))
("underperformance" nil (ax n d er0 p er0 f ao1 r m ax n s))
("underperformed" nil (ah1 n d er0 p er0 f ao1 r m d))
("underperformer" nil (ah1 n d er0 p er0 f ao1 r m er0))
("underperforming" nil (ah1 n d er0 p er0 f ao1 r m ih0 ng))
("underpin" nil (ah1 n d er0 p ih1 n))
("underpinned" nil (ah1 n d er0 p ih1 n d))
("underpinning" nil (ah1 n d er0 p ih1 n ih0 ng))
("underpinnings" nil (ah1 n d er0 p ih1 n ih0 ng z))
("underpowered" nil (ax n d er0 p aw1 er0 d))
("underprice" nil (ah1 n d er0 p r ay1 s))
("underpriced" nil (ah1 n d er0 p r ay1 s t))
("underpricing" nil (ah1 n d er0 p r ay1 s ih0 ng))
("underprivileged" nil (ax n d er0 p r ih1 v l ih0 jh d))
("underrated" nil (ax n d er0 r ey1 t ih0 d))
("underreport" nil (ax n d er0 r ih0 p ao1 r t))
("underreported" nil (ax n d er0 r ih0 p ao1 r t ih0 d))
("underreporting" nil (ah1 n d er0 r ih0 p ao1 r t ih0 ng))
("underrepresented" nil (ax n d er0 r eh1 p r ih0 z eh1 n t ih0 d))
("underscore" nil (ah1 n d er0 s k ao1 r))
("underscored" nil (ah1 n d er0 s k ao1 r d))
("underscores" nil (ah1 n d er0 s k ao1 r z))
("underscoring" nil (ah1 n d er0 s k ao1 r ih0 ng))
("undersea" nil (ah1 n d er0 s iy1))
("underseas" nil (ax n d er0 s iy1 z))
("undersecretary" nil (ah1 n d er0 s eh1 k r ih0 t eh1 r iy0))
("undersell" nil (ah1 n d er0 s eh1 l))
("underselling" nil (ah1 n d er0 s eh1 l ih0 ng))
("underserved" nil (ax n d er0 s er1 v d))
("undershirts" nil (ah1 n d er0 sh er1 t s))
("underside" nil (ah1 n d er0 s ay1 d))
("undersized" nil (ah1 n d er0 s ay1 z d))
("undersold" nil (ax n d er0 s ow1 l d))
("underspin" nil (ah1 n d er0 s p ih1 n))
("understaffed" nil (ah1 n d er0 s t ae1 f t))
("understand" nil (ah1 n d er0 s t ae1 n d))
("understandable" nil (ah1 n d er0 s t ae1 n d ax b ax l))
("understandably" nil (ah1 n d er0 s t ae1 n d ax b l iy0))
("understanding" nil (ah1 n d er0 s t ae1 n d ih0 ng))
("understandings" nil (ax n d er0 s t ae1 n d ih0 ng z))
("understands" nil (ah1 n d er0 s t ae1 n d z))
("understate" nil (ah1 n d er0 s t ey1 t))
("understated" nil (ah1 n d er0 s t ey1 t ih0 d))
("understatement" nil (ah1 n d er0 s t ey1 t m ax n t))
("understates" nil (ah1 n d er0 s t ey1 t s))
("understating" nil (ah1 n d er0 s t ey1 t ih0 ng))
("understood" nil (ah1 n d er0 s t uh1 d))
("understory" nil (ah1 n d er0 s t ao1 r iy0))
("understudy" nil (ah1 n d er0 s t ah1 d iy0))
("undersubscribed" nil (ax n d er0 s ax b s k r ay1 b d))
("undertake" nil (ah1 n d er0 t ey1 k))
("undertaken" nil (ah1 n d er0 t ey1 k ax n))
("undertaker" nil (ah1 n d er0 t ey1 k er0))
("undertakes" nil (ah1 n d er0 t ey1 k s))
("undertaking" nil (ah1 n d er0 t ey1 k ih0 ng))
("undertakings" nil (ah1 n d er0 t ey1 k ih0 ng z))
("undertone" nil (ah1 n d er0 t ow1 n))
("undertones" nil (ah1 n d er0 t ow1 n z))
("undertook" nil (ah1 n d er0 t uh1 k))
("undertow" nil (ah1 n d er0 t ow1))
("underused" nil (ah1 n d er0 y uw1 z d))
("underutilized" nil (ah1 n d er0 y uw1 t ax l ay1 z d))
("undervaluation" nil (ah1 n d er0 v ae1 l y uw0 ey1 sh ax n))
("undervalue" nil (ah1 n d er0 v ae1 l y uw0))
("undervalued" nil (ah1 n d er0 v ae1 l y uw0 d))
("undervalues" nil (ah1 n d er0 v ae1 l y uw0 z))
("undervaluing" nil (ax n d er0 v ae1 l y uw0 ih0 ng))
("underwater" nil (ah1 n d er0 w ao1 t er0))
("underway" nil (ah1 n d er0 w ey1))
("underwear" nil (ah1 n d er0 w eh1 r))
("underweight" nil (ah1 n d er0 w ey1 t))
("underweighted" nil (ax n d er0 w ey1 t ih0 d))
("underwent" nil (ah1 n d er0 w eh1 n t))
("underwithheld" nil (ah1 n d er0 w ih1 th hh eh1 l d))
("underwithholding" nil (ah1 n d er0 w ih1 th hh ow1 l d ih0 ng))
("underwood" nil (ah1 n d er0 w uh1 d))
("underwoods" nil (ah1 n d er0 w uh1 d z))
("underworld" nil (ah1 n d er0 w er1 l d))
("underwrite" nil (ah1 n d er0 r ay1 t))
("underwriter" nil (ah1 n d er0 r ay1 t er0))
("underwriters" nil (ah1 n d er0 r ay1 t er0 z))
("underwrites" nil (ah1 n d er0 r ay1 t s))
("underwriting" nil (ah1 n d er0 r ay1 t ih0 ng))
("underwritings" nil (ah1 n d er0 r ay1 t ih0 ng z))
("underwritten" nil (ah1 n d er0 r ih1 t ax n))
("underwrote" nil (ah1 n d er0 r ow1 t))
("undeserved" nil (ah1 n d ih0 z er1 v d))
("undesirable" nil (ah1 n d ih0 z ay1 r ax b ax l))
("undetectable" nil (ax n d ih0 t eh1 k t ax b ax l))
("undetected" nil (ah1 n d ih0 t eh1 k t ih0 d))
("undetermined" nil (ah1 n d ih0 t er1 m ih0 n d))
("undetermining" nil (ah1 n d ih0 t er1 m ih0 n ih0 ng))
("undeterred" nil (ah1 n d ih0 t er1 d))
("undeveloped" nil (ah1 n d ih0 v eh1 l ax p t))
("undid" nil (ax n d ih1 d))
("undifferentiated" nil (ax n d ih0 f er0 eh1 n sh iy0 ey1 t ih0 d))
("undignified" nil (ax n d ih1 g n ih0 f ay1 d))
("undiluted" nil (ah1 n d ax l uw1 t ih0 d))
("undiminished" nil (ah1 n d ax m ih1 n ih0 sh t))
("undine" nil (ah1 n d iy1 n))
("undisciplined" nil (ax n d ih1 s ih0 p l ih0 n d))
("undisclosed" nil (ah1 n d ih0 s k l ow1 z d))
("undiscovered" nil (ah1 n d ih0 s k ah1 v er0 d))
("undisguised" nil (ah1 n d ih0 s g ay1 z d))
("undisputed" nil (ah1 n d ih0 s p y uw1 t ih0 d))
("undistinguished" nil (ah1 n d ih0 s t ih1 ng g w ih0 sh t))
("undistributed" nil (ah1 n d ih0 s t r ih1 b y uw0 t ih0 d))
("undisturbed" nil (ah1 n d ih0 s t er1 b d))
("undiversified" nil (ah1 n d ih0 v er1 s ih0 f ay1 d))
("undivided" nil (ah1 n d ax v ay1 d ih0 d))
("undo" nil (ax n d uw1))
("undocumented" nil (ax n d aa1 k y ax m eh0 n t ih0 d))
("undoing" nil (ax n d uw1 ih0 ng))
("undone" nil (ax n d ah1 n))
("undoubted" nil (ax n d aw1 t ih0 d))
("undoubtedly" nil (ax n d aw1 t ih0 d l iy0))
("undreamed" nil (ax n d r iy1 m d))
("undress" nil (ax n d r eh1 s))
("undue" nil (ax n d uw1))
("undulating" nil (ah1 n jh ax l ey1 t ih0 ng))
("unduly" nil (ax n d uw1 l iy0))
("undying" nil (ax n d ay1 ih0 ng))
("unearned" nil (ax n er1 n d))
("unearth" nil (ax n er1 th))
("unearthed" nil (ax n er1 th t))
("unearthly" nil (ax n er1 th l iy0))
("unease" nil (ax n iy1 z))
("uneasily" nil (ax n iy1 z ax l iy0))
("uneasiness" nil (ah1 n iy1 z iy0 n ax s))
("uneasy" nil (ax n iy1 z iy0))
("uneconomic" nil (ax n eh1 k ax n aa1 m ih0 k))
("uneconomical" nil (ax n eh1 k ax n aa1 m ih0 k ax l))
("unedited" nil (ax n eh1 d ax t ih0 d))
("uneducated" nil (ax n eh1 jh uh0 k ey1 t ih0 d))
("unelected" nil (ah1 n ih0 l eh1 k t ih0 d))
("unemployable" nil (ax n ih0 m p l oy1 ax b ax l))
("unemployed" nil (ah1 n eh0 m p l oy1 d))
("unemployment" nil (ah1 n ih0 m p l oy1 m ax n t))
("unencumbered" nil (ah1 n eh0 n k ah1 m b er0 d))
("unending" nil (ax n eh1 n d ih0 ng))
("unenforceable" nil (ah1 n eh0 n f ao1 r s ax b ax l))
("unenlightened" nil (ah1 n eh0 n l ay1 t ax n d))
("unenthusiastic" nil (ax n ih0 n th uw0 z iy0 ae1 s t ih0 k))
("unenumerated" nil (ax n ih0 n uw1 m er0 ey1 t ih0 d))
("unenviable" nil (ax n eh1 n v iy1 ax b ax l))
("unequal" nil (ax n iy1 k w ax l))
("unequaled" nil (ax n iy1 k w ah1 l d))
("unequivocably" nil (ax n ih0 k w ih1 v ax k ax b l iy0))
("unequivocal" nil (ah1 n ih0 k w ih1 v ax k ax l))
("unequivocally" nil (ah1 n iy0 k w ih1 v ax k ax l iy0))
("unesco" nil (y uw0 n eh1 s k ow1))
("unethical" nil (ax n eh1 th ih0 k ax l))
("uneven" nil (ax n iy1 v ax n))
("unevenly" nil (ax n iy1 v ax n l iy0))
("uneventful" nil (ah1 n iy0 v eh1 n t f ax l))
("unexciting" nil (ax n ih0 k s ay1 t ih0 ng))
("unexercised" nil (ax n eh1 k s er0 s ay1 z d))
("unexpected" nil (ah1 n ih0 k s p eh1 k t ih0 d))
("unexpectedly" nil (ah1 n ih0 k s p eh1 k t ih0 d l iy0))
("unexpired" nil (ah1 n ih0 k s p ay1 er0 d))
("unexplainable" nil (ah1 n ih0 k s p l ey1 n ax b ax l))
("unexplained" nil (ah1 n ih0 k s p l ey1 n d))
("unexploded" nil (ah1 n ih0 k s p l ow1 d ih0 d))
("unexploited" nil (ax n ih0 k s p l oy1 t ih0 d))
("unexplored" nil (ah1 n ih0 k s p l ao1 r d))
("unfailing" nil (ax n f ey1 l ih0 ng))
("unfailingly" nil (ax n f ey1 l ih0 ng l iy0))
("unfair" nil (ax n f eh1 r))
("unfairly" nil (ax n f eh1 r l iy0))
("unfairness" nil (ax n f eh1 r n ax s))
("unfaithful" nil (ax n f ey1 th f ax l))
("unfamiliar" nil (ah1 n f ax m ih1 l y er0))
("unfamiliarities" nil (ah1 n f ax m ih1 l y eh1 r ax t iy0 z))
("unfamiliarity" nil (ah1 n f ax m ih1 l y eh1 r ax t iy0))
("unfashionable" nil (ax n f ae1 sh ax n ax b ax l))
("unfathomable" nil (ax n f ae1 th ax m ax b ax l))
("unfavorable" nil (ah1 n f ey1 v er0 ax b ax l))
("unfavorably" nil (ax n f ey1 v er0 ax b l iy0))
("unfazed" nil (ax n f ey1 z d))
("unfeasible" nil (ax n f iy1 z ax b ax l))
("unfenced" nil (ax n f eh1 n s t))
("unfettered" nil (ax n f eh1 t er0 d))
("unfilled" nil (ax n f ih1 l d))
("unfiltered" nil (ax n f ih1 l t er0 d))
("unfinished" nil (ax n f ih1 n ih0 sh t))
("unfit" nil (ax n f ih1 t))
("unflagging" nil (ax n f l ae1 g ih0 ng))
("unflappable" nil (ax n f l ae1 p ax b ax l))
("unflattering" nil (ax n f l ae1 t er0 ih0 ng))
("unflinching" nil (ax n f l ih1 n ch ih0 ng))
("unfocused" nil (ax n f ow1 k ax s t))
("unfold" nil (ax n f ow1 l d))
("unfolded" nil (ax n f ow1 l d ax d))
("unfolding" nil (ax n f ow1 l d ih0 ng))
("unfolds" nil (ax n f ow1 l d z))
("unforeseeable" nil (ah1 n f ao0 r s iy1 ax b ax l))
("unforeseen" nil (ah1 n f ao0 r s iy1 n))
("unforgettable" nil (ah1 n f er0 g eh1 t ax b ax l))
("unforgivable" nil (ah1 n f ao0 r g ih1 v ax b ax l))
("unforgiving" nil (ax n f er0 g ih1 v ih0 ng))
("unformed" nil (ax n f ao1 r m d))
("unfortunate" nil (ax n f ao1 r ch ax n ax t))
("unfortunately" nil (ax n f ao1 r ch ax n ax t l iy0))
("unfortunates" nil (ax n f ao1 r ch ax n ax t s))
("unfounded" nil (ax n f aw1 n d ih0 d))
("unfriendly" nil (ax n f r eh1 n d l iy0))
("unfulfilled" nil (ah1 n f uh0 l f ih1 l d))
("unfunded" nil (ax n f ah1 n d ih0 d))
("unfunnily" nil (ax n f ah1 n ax l iy0))
("unfurled" nil (ax n f er1 l d))
("unfurling" nil (ax n f er1 l ih0 ng))
("ung" nil (ah1 ng))
("ungainly" nil (ax n g ey1 n l iy0))
("ungar" nil (ah1 ng g er0))
("ungaro" nil (uw0 ng g aa1 r ow0))
("unger" nil (ah1 ng g er0))
("ungerer" nil (ah1 ng er0 er0))
("ungermann" nil (ax n jh er1 m ax n))
("unglamorous" nil (ax n g l ae1 m er0 ax s))
("unglued" nil (ax n g l uw1 d))
("ungo" nil (ax n g ow1))
("ungovernable" nil (ax n g ah1 v er0 n ax b ax l))
("ungrateful" nil (ax n g r ey1 t f ax l))
("unguarded" nil (ax n g aa1 r d ih0 d))
("unhampered" nil (ax n hh ae1 m p er0 d))
("unhappily" nil (ax n hh ae1 p ax l iy0))
("unhappiness" nil (ax n hh ae1 p iy0 n ih0 s))
("unhappy" nil (ax n hh ae1 p iy0))
("unharmed" nil (ax n hh aa1 r m d))
("unhealthful" nil (ax n hh eh1 l th f ax l))
("unhealthy" nil (ax n hh eh1 l th iy0))
("unheard" nil (ax n hh er1 d))
("unhedged" nil (ax n hh eh1 jh d))
("unheeded" nil (ax n hh iy1 d ih0 d))
("unheralded" nil (ax n hh eh1 r ax l d ih0 d))
("unhesitatingly" nil (ax n hh eh1 z ih0 t ey1 t ih0 ng l iy0))
("unhindered" nil (ax n hh ih1 n d er0 d))
("unhinged" nil (ax n hh ih1 n jh d))
("unholy" nil (ax n hh ow1 l iy0))
("unhorsed" nil (ax n hh ao1 r s t))
("unhurried" nil (ax n hh er1 iy0 d))
("unhurt" nil (ax n hh er1 t))
("uni" nil (y uw1 n iy0))
("unibancorp" nil (y uw1 n ih0 b ae1 n k ao1 r p))
("unicef" nil (y uw1 n ax s eh1 f))
("unicellular" nil (y uw1 n ih0 s eh1 l y ax l er0))
("unicenter" nil (y uw1 n ih0 s eh1 n t er0))
("unicorn" nil (y uw1 n ih0 k ao1 r n))
("unicorp" nil (y uw1 n ih0 k ao1 r p))
("uniden" nil (y uw1 n ih0 d eh1 n))
("unidentified" nil (ah1 n ay0 d eh1 n t ax f ay1 d))
("unification" nil (y uw1 n ax f ax k ey1 sh ax n))
("unified" nil (y uw1 n ax f ay1 d))
("uniforce" nil (y uw1 n ih0 f ao1 r s))
("uniform" nil (y uw1 n ax f ao1 r m))
("uniformed" nil (y uw1 n ax f ao1 r m d))
("uniformity" nil (y uw1 n ax f ao1 r m ax t iy0))
("uniformly" nil (y uw1 n ax f ao1 r m l iy0))
("uniforms" nil (y uw1 n ax f ao1 r m z))
("unify" nil (y uw1 n ax f ay1))
("unifying" nil (y uw1 n ax f ay1 ih0 ng))
("unigesco" nil (y uw1 n ih0 g eh1 s k ow0))
("unilab" nil (y uw1 n ih0 l ae1 b))
("unilateral" nil (y uw1 n ax l ae1 t er0 ax l))
("unilateralism" nil (y uw1 n ih0 l ae1 t er0 ax l ih1 z ax m))
("unilaterally" nil (y uw1 n ax l ae1 t er0 ax l iy0))
("unilever" nil (y uw1 n ih0 l iy1 v er0))
("unimaginable" nil (ah1 n ih0 m ae1 jh ih0 n ax b ax l))
("unimaginative" nil (ax n ih0 m ae1 jh ih1 n ax t ih0 v))
("unimagined" nil (ah1 n ih0 m ae1 jh ax n d))
("unimate" nil (y uw1 n ih0 m ey1 t))
("unimation" nil (y uw1 n ih0 m ey1 sh ax n))
("unimedia" nil (y uw1 n ih0 m iy1 d iy0 ax))
("unimpaired" nil (ah1 n ih0 m p eh1 r d))
("unimpeachable" nil (ah1 n ih0 m p iy1 ch ax b ax l))
("unimpeded" nil (ah1 n ih0 m p iy1 d ih0 d))
("unimportant" nil (ax n ih0 m p ao1 r t ax n t))
("unimpressed" nil (ah1 n ih0 m p r eh1 s t))
("unimpressive" nil (ah1 n ih0 m p r eh1 s ih0 v))
("unimproved" nil (ah1 n ih0 m p r uw1 v d))
("unincorporated" nil (ah1 n ih0 n k ao1 r p er0 ey1 t ih0 d))
("unindicted" nil (ah1 n ih0 n d ay1 t ih0 d))
("uninfected" nil (ah1 n ih0 n f eh1 k t ih0 d))
("uninformative" nil (ax n ih0 n f ao1 r m ax t ih0 v))
("uninformed" nil (ah1 n ih0 n f ao1 r m d))
("uninhabitable" nil (ah1 n ih0 n hh ae1 b ax t ax b ax l))
("uninhabited" nil (ah1 n ih0 n hh ae1 b ax t ih0 d))
("uninhibited" nil (ah1 n ih0 n hh ih1 b ih0 t ih0 d))
("uninitiated" nil (ah1 n ih0 n ih1 sh iy0 ey1 t ih0 d))
("uninjured" nil (ax n ih1 n jh er0 d))
("uninspired" nil (ah1 n ih0 n s p ay1 er0 d))
("uninspiring" nil (ax n ih0 n s p ay1 r ih0 ng))
("uninsurable" nil (ax n ih0 n sh uh1 r ax b ax l))
("uninsured" nil (ah1 n ih0 n sh uh1 r d))
("unintelligible" nil (ah1 n ih0 n t eh1 l ax jh ax b ax l))
("unintended" nil (ah1 n ih0 n t eh1 n d ih0 d))
("unintentional" nil (ah1 n ih0 n t eh1 n sh ax n ax l))
("unintentionally" nil (ah1 n ih0 n t eh1 n sh ax n ax l iy0))
("uninterested" nil (ax n ih1 n t r ax s t ax d))
("uninteresting" nil (ax n ih1 n t r ax s t ih0 ng))
("uninterrupted" nil (ah1 n ih0 n t er0 ah1 p t ih0 d))
("uninterruptible" nil (ax n ih0 n t er0 ah1 p t ax b ax l))
("uninvited" nil (ah1 n ih0 n v ay1 t ih0 d))
("uninviting" nil (ah1 n ih0 n v ay1 t ih0 ng))
("uninvolved" nil (ax n ih0 n v ao1 l v d))
("union" nil (y uw1 n y ax n))
("uniondale" nil (y uw1 n y ax n d ey1 l))
("unionfed" nil (y uw1 n y ax n f eh1 d))
("unionism" nil (y uw1 n y ax n ih1 z ax m))
("unionist" nil (y uw1 n y ax n ax s t))
("unionists" nil (y uw1 n y ax n ih0 s t s))
("unionization" nil (y uw1 n y ax n ax z ey1 sh ax n))
("unionize" nil (y uw1 n y ax n ay1 z))
("unionized" nil (y uw1 n y ax n ay1 z d))
("unionizing" nil (y uw1 n y ax n ay1 z ih0 ng))
("unions" nil (y uw1 n y ax n z))
("unique" nil (y uw0 n iy1 k))
("uniquely" nil (y uw0 n iy1 k l iy0))
("uniqueness" nil (y uw0 n iy1 k n ax s))
("uniroyal" nil (y uw1 n ih0 r oy1 ax l))
("unisex" nil (y uw1 n ih0 s eh1 k s))
("unison" nil (y uw1 n ax s ax n))
("unisource" nil (y uw1 n ih0 s ao1 r s))
("unissued" nil (ax n ih1 sh uw1 d))
("unisys" nil (y uw1 n ih0 s ih1 s))
("unit" nil (y uw1 n ax t))
("unita" nil (y uw1 n iy1 t ax))
("unitarian" nil (y uw1 n ax t eh1 r iy0 ax n))
("unitary" nil (y uw1 n ih0 t eh1 r iy0))
("unitas" nil (y uw0 n ay1 t ax s))
("unite" nil (y uw1 n ay1 t))
("united" nil (y uw0 n ay1 t ax d))
("unitedbank" nil (y uw1 n ay1 t ih0 d b ae1 ng k))
("unitek" nil (y uw1 n ih0 t eh1 k))
("unitel" nil (y uw1 n ih0 t eh1 l))
("unites" nil (y uw0 n ay1 t s))
("unitholder" nil (y uw1 n ih0 t hh ow1 l d er0))
("unitholders" nil (y uw1 n ih0 t hh ow1 l d er0 z))
("uniting" nil (y uw0 n ay1 t ih0 ng))
("unitrin" nil (y uw1 n ih0 t r ih1 n))
("unitrode" nil (y uw1 n ih0 t r ow1 d))
("units" nil (y uw1 n ax t s))
("unity" nil (y uw1 n ax t iy0))
("univa" nil (y uw1 n ih0 v ax))
("univar" nil (y uw1 n ih0 v aa0 r))
("univation" nil (y uw1 n ih0 v ey1 sh ax n))
("universal" nil (y uw1 n ax v er1 s ax l))
("universality" nil (y uw1 n ax v er0 s ae1 l ax t iy0))
("universally" nil (y uw1 n ax v er1 s ax l iy0))
("universe" nil (y uw1 n ax v er1 s))
("universities" nil (y uw1 n ax v er1 s ax t iy0 z))
("university" nil (y uw1 n ax v er1 s ax t iy0))
("univisa" nil (y uw1 n ih0 v iy1 s ax))
("univision" nil (y uw1 n ih0 v ih1 zh ax n))
("unix" nil (y uw1 n ih0 k s))
("unjust" nil (ax n jh ah1 s t))
("unjustifiable" nil (ah1 n jh ah1 s t ih0 f ay1 ax b ax l))
("unjustifiably" nil (ah1 n jh ah1 s t ih0 f ay1 ax b l iy0))
("unjustified" nil (ax n jh ah1 s t ax f ay1 d))
("unjustly" nil (ax n jh ah1 s t l iy0))
("unkefer" nil (ah1 ng k ih0 f er0))
("unkempt" nil (ax n k eh1 m p t))
("unkind" nil (ax n k ay1 n d))
("unknowable" nil (ax n n ow1 ax b ax l))
("unknowingly" nil (ax n n ow1 ih0 ng l iy0))
("unknown" nil (ax n n ow1 n))
("unknowns" nil (ax n ow1 n z))
("unlabeled" nil (ax n l ey1 b ax l d))
("unland" nil (ah1 n l ax n d))
("unlawful" nil (ax n l ao1 f ax l))
("unlawfully" nil (ax n l ao1 f ax l iy0))
("unleaded" nil (ax n l eh1 d ax d))
("unleash" nil (ax n l iy1 sh))
("unleashed" nil (ax n l iy1 sh t))
("unleashes" nil (ax n l iy1 sh ih0 z))
("unleashing" nil (ax n l iy1 sh ih0 ng))
("unless" nil (ax n l eh1 s))
("unleveraged" nil (ax n l eh1 v r ih0 jh d))
("unlicensed" nil (ax n l ay1 s ax n s t))
("unlike" nil (ax n l ay1 k))
("unlikely" nil (ax n l ay1 k l iy0))
("unlimited" nil (ax n l ih1 m ax t ax d))
("unlined" nil (ax n l ay1 n d))
("unlisted" nil (ax n l ih1 s t ih0 d))
("unload" nil (ax n l ow1 d))
("unloaded" nil (ax n l ow1 d ax d))
("unloading" nil (ax n l ow1 d ih0 ng))
("unloads" nil (ax n l ow1 d z))
("unlock" nil (ax n l aa1 k))
("unlocked" nil (ax n l aa1 k t))
("unlocking" nil (ax n l aa1 k ih0 ng))
("unloved" nil (ax n l ah1 v d))
("unlucky" nil (ax n l ah1 k iy0))
("unmade" nil (ax n m ey1 d))
("unmanage" nil (ax n m ae1 n ih0 jh))
("unmanageable" nil (ax n m ae1 n ih0 jh ax b ax l))
("unmanaged" nil (ax n m ae1 n ih0 jh d))
("unmanned" nil (ax n m ae1 n d))
("unmarked" nil (ax n m aa1 r k t))
("unmarried" nil (ax n m eh1 r iy0 d))
("unmask" nil (ax n m ae1 s k))
("unmasked" nil (ax n m ae1 s k t))
("unmatched" nil (ax n m ae1 ch t))
("unmentionable" nil (ax n m eh1 n sh ax n ax b ax l))
("unmentioned" nil (ax n m eh1 n ch ax n d))
("unmet" nil (ax n m eh1 t))
("unmistakable" nil (ah1 n m ih0 s t ey1 k ax b ax l))
("unmistakably" nil (ah1 n m ih0 s t ey1 k ax b l iy0))
("unmitigated" nil (ax n m ih1 t ax g ey1 t ih0 d))
("unmixed" nil (ax n m ih1 k s t))
("unmolested" nil (ah1 n m ax l eh1 s t ih0 d))
("unmoved" nil (ax n m uw1 v d))
("unnamed" nil (ax n n ey1 m d))
("unnatural" nil (ax n n ae1 ch er0 ax l))
("unnecessarily" nil (ax n n eh1 s ax s eh1 r ax l iy0))
("unnecessary" nil (ax n n eh1 s ax s eh1 r iy0))
("unneeded" nil (ax n n iy1 d ih0 d))
("unnerve" nil (ax n er1 v))
("unnerved" nil (ax n n er1 v d))
("unnerving" nil (ax n er1 v ih0 ng))
("unnoticed" nil (ax n n ow1 t ih0 s t))
("uno" nil (ax n ow1))
("unobstructed" nil (ah1 n ax b s t r ah1 k t ih0 d))
("unobtainable" nil (ah1 n ax b t ey1 n ax b ax l))
("unobtrusive" nil (ah1 n ax b t r uw1 s ih0 v))
("unocal" nil (y uw1 n ax k ae1 l))
("unoccupied" nil (ax n aa1 k y ax p ay1 d))
("unofficial" nil (ah1 n ax f ih1 sh ax l))
("unofficially" nil (ax n ax f ih1 sh ax l iy0))
("unopened" nil (ax n ow1 p ax n d))
("unopposed" nil (ah1 n ax p ow1 z d))
("unorganized" nil (ax n ao1 r g ax n ay1 z d))
("unorthodox" nil (ax n ao1 r th ax d aa1 k s))
("unos" nil (uw1 n ow0 z))
("unpack" nil (ax n p ae1 k))
("unpaid" nil (ax n p ey1 d))
("unpainted" nil (ax n p ey1 n t ih0 d))
("unpalatable" nil (ax n p ae1 l ax t ax b ax l))
("unparalleled" nil (ax n p eh1 r ax l eh1 l d))
("unpatriotic" nil (ax n p ey1 t r iy0 aa1 t ih0 k))
("unpayable" nil (ax n p ey1 ax b ax l))
("unperturbed" nil (ah1 n p er0 t er1 b d))
("unplaced" nil (ax n p l ey1 s t))
("unplanned" nil (ax n p l ae1 n d))
("unpleasant" nil (ax n p l eh1 z ax n t))
("unpleasantly" nil (ax n p l eh1 z ax n t l iy0))
("unpleasantness" nil (ax n p l eh1 z ax n t n ax s))
("unplugged" nil (ax n p l ah1 g d))
("unpopular" nil (ah1 n p aa1 p y ax l er0))
("unpopularity" nil (ax n p aa1 p y ax l eh1 r ih0 t iy0))
("unprecedented" nil (ax n p r eh1 s ih0 d eh1 n t ih0 d))
("unprecedentedly" nil (ax n p r eh1 s ax d eh1 n t ih0 d l iy0))
("unpredictability" nil (ah1 n p r ax d ih1 k t ax b ih1 l ih0 t iy0))
("unpredictable" nil (ah1 n p r ih0 d ih1 k t ax b ax l))
("unpredictably" nil (ah1 n p r ih0 d ih1 k t ax b l iy0))
("unprepared" nil (ah1 n p r iy0 p eh1 r d))
("unpretentious" nil (ah1 n p r iy0 t eh1 n sh ax s))
("unprincipled" nil (ax n p r ih1 n s ax p ax l d))
("unprintable" nil (ax n p r ih1 n t ax b ax l))
("unprocessed" nil (ax n p r ao1 s eh1 s t))
("unproductive" nil (ah1 n p r ax d ah1 k t ih0 v))
("unprofessional" nil (ah1 n p r ax f eh1 sh ax n ax l))
("unprofitability" nil (ax n p r aa1 f ih0 t ax b ih1 l ih0 t iy0))
("unprofitable" nil (ax n p r aa1 f ih0 t ax b ax l))
("unprofor" nil (ah1 p r ow0 f ao1 r))
("unpromising" nil (ax n p r ao1 m ih0 s ih0 ng))
("unprotected" nil (ah1 n p r ax t eh1 k t ih0 d))
("unproved" nil (ax n p r uw1 v d))
("unproven" nil (ax n p r uw1 v ax n))
("unprovoked" nil (ah1 n p r ax v ow1 k t))
("unpublicized" nil (ax n p ah1 b l ih0 s ay0 z d))
("unpublished" nil (ax n p ah1 b l ih0 sh t))
("unpunished" nil (ax n p ah1 n ih0 sh t))
("unqualified" nil (ax n k w aa1 l ih0 f ay1 d))
("unquestionable" nil (ax n k w eh1 s ch ax n ax b ax l))
("unquestionably" nil (ax n k w eh1 s ch ax n ax b l iy0))
("unquestioned" nil (ax n k w eh1 s ch ax n d))
("unquestioning" nil (ax n k w eh1 s ch ax n ih0 ng))
("unquote" nil (ah1 n k w ow1 t))
("unrated" nil (ax n r ey1 t ih0 d))
("unrath" nil (ah1 n r ax th))
("unratified" nil (ax n r ae1 t ih0 f ay1 d))
("unravel" nil (ax n r ae1 v ax l))
("unraveled" nil (ax n r ae1 v ax l d))
("unraveling" nil (ax n r ae1 v ax l ih0 ng))
("unreachable" nil (ax n r iy1 ch ax b ax l))
("unread" nil (ax n r eh1 d))
("unreadable" nil (ax n r iy1 d ax b ax l))
("unreal" nil (ax n r iy1 l))
("unrealistic" nil (ax n r iy1 l ih1 s t ih0 k))
("unrealistically" nil (ax n r iy1 ax l ih1 s t ih0 k l iy0))
("unreality" nil (ah1 n r iy0 ae1 l ax t iy0))
("unrealized" nil (ax n r iy1 ax l ay1 z d))
("unreasonable" nil (ax n r iy1 z n ax b ax l))
("unreasonably" nil (ax n r iy1 z ax n ax b l iy0))
("unreasoning" nil (ax n r iy1 z n ih0 ng))
("unreceptive" nil (ax n r ih0 s eh1 p t ih0 v))
("unrecognizable" nil (ax n r eh1 k ax g n ay1 z ax b ax l))
("unrecognized" nil (ax n r eh1 k ax g n ay1 z d))
("unreconciled" nil (ax n r eh1 k ax n s ay1 l d))
("unreconstructed" nil (ah1 n r iy0 k ax n s t r ah1 k t ih0 d))
("unrecorded" nil (ah1 n r ih0 k ao1 r d ih0 d))
("unredeemed" nil (ah1 n r iy0 d iy1 m d))
("unrefined" nil (ah1 n r iy0 f ay1 n d))
("unregistered" nil (ax n r eh1 jh ih0 s t er0 d))
("unregulated" nil (ax n r eh1 g y ax l ey1 t ih0 d))
("unreimbursed" nil (ax n r iy0 ih0 m b er1 s t))
("unrein" nil (ao1 n r ay0 n))
("unrelated" nil (ah1 n r ih0 l ey1 t ih0 d))
("unreleased" nil (ax n r ih0 l iy1 s t))
("unrelenting" nil (ah1 n r iy0 l eh1 n t ih0 ng))
("unreliability" nil (ah1 n r iy0 l ay1 ax b ih1 l ih0 t iy0))
("unreliable" nil (ah1 n r ih0 l ay1 ax b ax l))
("unrelieved" nil (ah1 n r iy0 l iy1 v d))
("unremarkable" nil (ax n r ih0 m aa1 r k ax b ax l))
("unremarked" nil (ax n r ih0 m aa1 r k t))
("unremitted" nil (ax n r ih0 m ih1 t ih0 d))
("unremitting" nil (ah1 n r ih0 m ih1 t ih0 ng))
("unrepentant" nil (ah1 n r ih0 p eh1 n t ax n t))
("unreported" nil (ah1 n r iy0 p ao1 r t ih0 d))
("unrepresentative" nil (ax n r eh1 p r ih0 z eh1 n t ax t ih0 v))
("unrepresented" nil (ax n r eh1 p r ih0 z eh1 n t ih0 d))
("unrequited" nil (ah1 n r iy0 k w ay1 t ih0 d))
("unresolved" nil (ax n r ih0 z aa1 l v d))
("unresponsive" nil (ah1 n r iy0 s p aa1 n s ih0 v))
("unrest" nil (ax n r eh1 s t))
("unrestrained" nil (ah1 n r iy0 s t r ey1 n d))
("unrestricted" nil (ah1 n r iy0 s t r ih1 k t ih0 d))
("unrivaled" nil (ax n r ay1 v ax l d))
("unrue" nil (ah1 n r uw0))
("unruffled" nil (ax n r ah1 f ax l d))
("unruh" nil (ah1 n r uw0))
("unruly" nil (ax n r uw1 l iy0))
("unsafe" nil (ax n s ey1 f))
("unsalable" nil (ax n s ey1 l ax b ax l))
("unsaleable" nil (ax n s ey1 l ax b ax l))
("unsanctioned" nil (ax n s ae1 ng k sh ax n d))
("unsanitary" nil (ax n s ae1 n ax t eh1 r iy0))
("unsatisfactory" nil (ah1 n s ax t ih0 s f ae1 k t er0 iy0))
("unsatisfied" nil (ax n s ae1 t ih0 s f ay1 d))
("unsatisfying" nil (ax n s ae1 t ih0 s f ay1 ih0 ng))
("unsaturated" nil (ax n s ae1 ch er0 ey1 t ih0 d))
("unsavory" nil (ax n s ey1 v er0 iy0))
("unscathed" nil (ax n s k ey1 dh d))
("unscheduled" nil (ax n s k eh1 jh uw0 l d))
("unscientific" nil (ax n s ay1 ax n t ih1 f ih0 k))
("unscramble" nil (ax n s k r ae1 m b ax l))
("unscrupulous" nil (ax n s k r uw1 p y ax l ax s))
("unseal" nil (ax n s iy1 l))
("unsealed" nil (ax n s iy1 l d))
("unseasonably" nil (ax n s iy1 z ax n ax b l iy0))
("unseat" nil (ax n s iy1 t))
("unseated" nil (ax n s iy1 t ih0 d))
("unseating" nil (ax n s iy1 t ih0 ng))
("unsecured" nil (ah1 n s ih0 k y uh1 r d))
("unseemly" nil (ax n s iy1 m l iy0))
("unseen" nil (ax n s iy1 n))
("unsell" nil (ao1 n s ax l))
("unsentimental" nil (ax n s eh1 n t ih0 m eh1 n t ax l))
("unser" nil (ah1 n s er0))
("unservicable" nil (ax n s er1 v ax s ax b ax l))
("unset" nil (ax n s eh1 t))
("unsettle" nil (ax n s eh1 t ax l))
("unsettled" nil (ax n s eh1 t ax l d))
("unsettling" nil (ax n s eh1 t ax l ih0 ng))
("unshackled" nil (ax n sh ae1 k ax l d))
("unshakable" nil (ax n sh ey1 k ax b ax l))
("unshaken" nil (ax n sh ey1 k ax n))
("unsharpened" nil (ax n sh aa1 r p ax n d))
("unshaven" nil (ax n sh ey1 v ax n))
("unsightly" nil (ax n s ay1 t l iy0))
("unsigned" nil (ax n s ay1 n d))
("unsinkable" nil (ax n s ih1 ng k ax b ax l))
("unskilled" nil (ax n s k ih1 l d))
("unsmiling" nil (ax n s m ay1 l ih0 ng))
("unsold" nil (ax n s ow1 l d))
("unsolicited" nil (ah1 n s ax l ih1 s ih0 t ih0 d))
("unsolved" nil (ax n s aa1 l v d))
("unsophisticated" nil (ah1 n s ax f ih1 s t ih0 k ey1 t ih0 d))
("unsound" nil (ax n s aw1 n d))
("unsparing" nil (ax n s p eh1 r ih0 ng))
("unspeakable" nil (ax n s p iy1 k ax b ax l))
("unspecified" nil (ax n s p eh1 s ax f ay1 d))
("unspectacular" nil (ah1 n s p eh0 k t ae1 k y ax l er0))
("unspent" nil (ax n s p eh1 n t))
("unspoiled" nil (ax n s p oy1 l d))
("unspoken" nil (ax n s p ow1 k ax n))
("unstable" nil (ax n s t ey1 b ax l))
("unstated" nil (ax n s t ey1 t ih0 d))
("unsteady" nil (ax n s t eh1 d iy0))
("unstinting" nil (ax n s t ih1 n t ih0 ng))
("unstoppable" nil (ax n s t aa1 p ax b ax l))
("unstructured" nil (ax n s t r ah1 k sh er0 d))
("unstuck" nil (ax n s t ah1 k))
("unsubscribed" nil (ax n s ax b s k r ay1 b d))
("unsubsidized" nil (ax n s ah1 b s ax d ay1 z d))
("unsubstantiated" nil (ah1 n s ax b s t ae1 n sh iy0 ey1 t ih0 d))
("unsubtle" nil (ax n s ah1 t ax l))
("unsuccessful" nil (ah1 n s ax k s eh1 s f ax l))
("unsuccessfully" nil (ah1 n s ax k s eh1 s f ax l iy0))
("unsuitable" nil (ax n s uw1 t ax b ax l))
("unsuited" nil (ax n s uw1 t ih0 d))
("unsullied" nil (ax n s ah1 l iy0 d))
("unsung" nil (ax n s ah1 ng))
("unsupervised" nil (ax n s uw1 p er0 v ay1 z d))
("unsupportable" nil (ah1 n s ax p ao1 r t ax b ax l))
("unsupported" nil (ax n s ax p ao1 r t ih0 d))
("unsure" nil (ax n sh uh1 r))
("unsurpassed" nil (ah1 n s er0 p ae1 s t))
("unsurprising" nil (ax n s er0 p r ay1 z ih0 ng))
("unsurprisingly" nil (ah1 n s er0 p r ay1 z ih0 ng l iy0))
("unsuspected" nil (ah1 n s ax s p eh1 k t ih0 d))
("unsuspecting" nil (ah1 n s ax s p eh1 k t ih0 ng))
("unsustainable" nil (ah1 n s ax s t ey1 n ax b ax l))
("unsustainably" nil (ah1 n s ax s t ey1 n ax b l iy0))
("unswayed" nil (ax n s w ey1 d))
("unswerving" nil (ax n s w er1 v ih0 ng))
("unsworth" nil (ah1 n s w er0 th))
("unsympathetic" nil (ax n s ih1 m p ax th eh1 t ih0 k))
("untainted" nil (ax n t ey1 n t ih0 d))
("untangle" nil (ax n t ae1 ng g ax l))
("untangling" nil (ax n t ae1 ng l ih0 ng))
("untapped" nil (ax n t ae1 p t))
("untaxed" nil (ax n t ae1 k s t))
("untenable" nil (ax n t eh1 n ax b ax l))
("untendered" nil (ax n t eh1 n d er0 d))
("unterberg" nil (ah1 n t er0 b er0 g))
("unterman" nil (ah1 n t er0 m ax n))
("untermeyer" nil (ah1 n t er0 m ay1 r))
("unterreiner" nil (ao1 n t er0 ay0 n er0))
("untested" nil (ax n t eh1 s t ih0 d))
("unthinkable" nil (ax n th ih1 ng k ax b ax l))
("unthinking" nil (ax n th ih1 ng k ih0 ng))
("unthinkingly" nil (ax n th ih1 ng k ih0 ng l iy0))
("untidy" nil (ax n t ay1 d iy0))
("untied" nil (ax n t ay1 d))
("untiedt" nil (ao1 n t iy0 t))
("until" nil (ax n t ih1 l))
("untimely" nil (ax n t ay1 m l iy0))
("unto" nil (ah1 n t uw0))
("untold" nil (ax n t ow1 l d))
("untouchable" nil (ax n t ah1 ch ax b ax l))
("untouchables" nil (ax n t ah1 ch ax b ax l z))
("untouched" nil (ax n t ah1 ch t))
("untoward" nil (ax n t uw0 ao1 r d))
("untraceable" nil (ax n t r ey1 s ax b ax l))
("untraditional" nil (ah1 n t r ax d ih1 sh ax n ax l))
("untrained" nil (ax n t r ey1 n d))
("untrammeled" nil (ax n t r ae1 m ax l d))
("untreatable" nil (ax n t r iy1 t ax b ax l))
("untreated" nil (ax n t r iy1 t ih0 d))
("untried" nil (ax n t r ay1 d))
("untroubled" nil (ax n t r ah1 b ax l d))
("untrue" nil (ax n t r uw1))
("untrustworthy" nil (ax n t r ah1 s t w er1 dh iy0))
("unturned" nil (ax n t er1 n d))
("untypical" nil (ax n t ih1 p ih0 k ax l))
("unum" nil (y uw1 n ax m))
("unusable" nil (ax n y uw1 z ax b ax l))
("unused" nil (ax n y uw1 z d))
("unusual" nil (ax n y uw1 zh ax w ax l))
("unusually" nil (ax n y uw1 zh ax w ax l iy0))
("unvarnished" nil (ax n v aa1 r n ih0 sh t))
("unveil" nil (ax n v ey1 l))
("unveiled" nil (ax n v ey1 l d))
("unveiling" nil (ax n v ey1 l ih0 ng))
("unveils" nil (ax n v ey1 l z))
("unverifiable" nil (ax n v eh1 r ih0 f ay1 ax b ax l))
("unverified" nil (ax n v eh1 r ih0 f ay1 d))
("unverzagt" nil (ao1 n v er0 z ao0 g t))
("unwanted" nil (ax n w ao1 n t ih0 d))
("unwarranted" nil (ax n w ao1 r ax n t ih0 d))
("unwary" nil (ax n w eh1 r iy0))
("unwashed" nil (ax n w aa1 sh t))
("unwavering" nil (ax n w ey1 v er0 ih0 ng))
("unwed" nil (ax n w eh1 d))
("unwelcome" nil (ax n w eh1 l k ax m))
("unwieldiness" nil (ax n w iy1 l d iy0 n ax s))
("unwieldy" nil (ax n w iy1 l d iy0))
("unwilling" nil (ax n w ih1 l ih0 ng))
("unwillingness" nil (ax n w ih1 l ih0 ng n ih0 s))
("unwin" nil (ao1 n w ih0 n))
("unwind" nil (ax n w ay1 n d))
("unwinding" nil (ax n w ay1 n d ih0 ng))
("unwise" nil (ax n w ay1 z))
("unwisely" nil (ax n w ay1 z l iy0))
("unwitting" nil (ax n w ih1 t ih0 ng))
("unwittingly" nil (ax n w ih1 t ih0 ng l iy0))
("unworkable" nil (ax n w er1 k ax b ax l))
("unworried" nil (ax n w er1 iy0 d))
("unworthy" nil (ax n w er1 dh iy0))
("unwound" nil (ax n w aw1 n d))
("unwrapping" nil (ax n r ae1 p ih0 ng))
("unwritten" nil (ax n r ih1 t ax n))
("unyielding" nil (ax n y iy1 l d ih0 ng))
("unzicker" nil (ao1 n z ih0 k er0))
("up" nil (ah1 p))
("upbeat" nil (ah1 p b iy1 t))
("upbraided" nil (ax p b r ey1 d ih0 d))
("upbringing" nil (ah1 p b r ih1 ng ih0 ng))
("upchurch" nil (ah1 p ch er1 ch))
("upcoming" nil (ah1 p k ah1 m ih0 ng))
("update" nil (ax p d ey1 t))
("updated" nil (ax p d ey1 t ax d))
("updates" nil (ax p d ey1 t s))
("updating" nil (ax p d ey1 t ih0 ng))
("updegraff" nil (ah1 p d ih0 g r ax f))
("updegrove" nil (uw0 p d eh0 g r ow1 v iy0))
("updike" nil (ah1 p d ay1 k))
("updraft" nil (ah1 p d r ae1 f t))
("updyke" nil (ah1 p d ay1 k))
("upfront" nil (ah1 p f r ah1 n t))
("upgrade" nil (ax p g r ey1 d))
("upgraded" nil (ax p g r ey1 d ax d))
("upgrader" nil (ah1 p g r ey1 d er0))
("upgrades" nil (ax p g r ey1 d z))
("upgrading" nil (ax p g r ey1 d ih0 ng))
("upham" nil (ah1 p ax m))
("upheaval" nil (ax p hh iy1 v ax l))
("upheavals" nil (ax p hh iy1 v ax l z))
("upheld" nil (ax p hh eh1 l d))
("uphill" nil (ah1 p hh ih1 l))
("uphoff" nil (ah1 p hh ao1 f))
("uphold" nil (ax p hh ow1 l d))
("upholding" nil (ax p hh ow1 l d ih0 ng))
("upholds" nil (ax p hh ow1 l d z))
("upholstered" nil (ax p ow1 l s t er0 d))
("upholstery" nil (ax p ow1 l s t er0 iy0))
("upjohn" nil (ah1 p jh aa1 n))
("upkeep" nil (ah1 p k iy1 p))
("upland" nil (ah1 p l ax n d))
("uplands" nil (ah1 p l ax n d z))
("uplift" nil (ah1 p l ih0 f t))
("uplifting" nil (ah1 p l ih1 f t ih0 ng))
("uplinger" nil (uw1 p ax l ih0 ng er0))
("upmanship" nil (ah1 p m ax n sh ih1 p))
("upmarket" nil (ah1 p m aa1 r k ax t))
("upon" nil (ax p aa1 n))
("upp" nil (ah1 p))
("upped" nil (ah1 p t))
("upper" nil (ah1 p er0))
("upperclass" nil (ah1 p er0 k l ae1 s))
("upperman" nil (ah1 p er0 m ax n))
("uppermost" nil (ah1 p er0 m ow1 s t))
("upping" nil (ah1 p ih0 ng))
("upright" nil (ax p r ay1 t))
("uprights" nil (ah1 p r ay1 t s))
("uprising" nil (ax p r ay1 z ih0 ng))
("uprisings" nil (ah1 p r ay1 z ih0 ng z))
("upriver" nil (ah1 p r ih1 v er0))
("uproar" nil (ah1 p r ao1 r))
("uproot" nil (ax p r uw1 t))
("uprooted" nil (ax p r uw1 t ih0 d))
("uprooting" nil (ax p r uw1 t ih0 ng))
("ups" nil (ah1 p s))
("upscale" nil (ah1 p s k ey1 l))
("upset" v (ax p s eh1 t))
("upset" n (ah1 p s eh1 t))
("upsets" v (ax p s eh1 t s))
("upsets" n (ah1 p s eh1 t s))
("upsetting" nil (ax p s eh1 t ih0 ng))
("upshaw" nil (ah1 p sh ao1))
("upshot" nil (ah1 p sh aa1 t))
("upshur" nil (ah1 p sh er0))
("upside" nil (ah1 p s ay1 d))
("upsize" nil (ah1 p s ay1 z))
("upsizing" nil (ah1 p s ay1 z ih0 ng))
("upson" nil (ah1 p s ax n))
("upstage" nil (ax p s t ey1 jh))
("upstaged" nil (ah1 p s t ey1 jh d))
("upstairs" nil (ax p s t eh1 r z))
("upstanding" nil (ah1 p s t ae1 n d ih0 ng))
("upstart" nil (ah1 p s t aa1 r t))
("upstarts" nil (ax p s t aa1 r t s))
("upstate" nil (ah1 p s t ey1 t))
("upstream" nil (ah1 p s t r iy1 m))
("upsurge" nil (ah1 p s er1 jh))
("upswing" nil (ax p s w ih1 ng))
("upswings" nil (ah1 p s w ih1 ng z))
("uptain" nil (ax p t ey1 n))
("uptegrove" nil (ah1 p t ih0 g r ax v))
("uptick" nil (ah1 p t ih1 k))
("upticks" nil (ah1 p t ih1 k s))
("uptight" nil (ax p t ay1 t))
("upton" nil (ah1 p t ax n))
("uptown" nil (ah1 p t aw1 n))
("uptrend" nil (ah1 p t r eh1 n d))
("upturn" nil (ax p t er1 n))
("upturns" nil (ah1 p t er1 n z))
("upward" nil (ah1 p w er0 d))
("upwardly" nil (ah1 p w er0 d l iy0))
("upwards" nil (ah1 p w er0 d z))
("upwood" nil (ah1 p w uh1 d))
("ur" nil (er1))
("urals" nil (y uh1 r ax l z))
("uram" nil (y uw1 r ax m))
("uranga" nil (er0 aa1 ng g ax))
("uranium" nil (y er0 ey1 n iy0 ax m))
("uranus" nil (y uw1 r ax n ax s))
("urbach" nil (er1 b aa0 k))
("urbain" nil (er0 b ey1 n))
("urban" nil (er1 b ax n))
("urbana" nil (er0 b ae1 n ax))
("urbanchuk" nil (er0 b ah1 n ch ax k))
("urbanczyk" nil (er0 b ah1 n ch ih0 k))
("urbane" nil (er0 b ey1 n))
("urbanek" nil (er0 b ae1 n eh0 k))
("urbani" nil (er0 b aa1 n iy0))
("urbaniak" nil (er0 b ae1 n iy0 ae0 k))
("urbanik" nil (er0 b ae1 n ih0 k))
("urbanites" nil (er1 b ax n ay1 t s))
("urbanization" nil (er1 b ax n ax z ey1 sh ax n))
("urbanized" nil (er1 b ax n ay1 z d))
("urbano" nil (er0 b aa1 n ow0))
("urbanowicz" nil (er0 b ae1 n ax v ih0 ch))
("urbanski" nil (er0 b ae1 n s k iy0))
("urbas" nil (er1 b ax z))
("urbaszewski" nil (er1 b ax s uw1 s k iy0))
("urbina" nil (er0 b iy1 n ax))
("urchin" nil (er1 ch ax n))
("urchins" nil (er1 ch ax n z))
("urda" nil (er1 d ax))
("urdiales" nil (er1 d iy0 ey0 l z))
("urdu" nil (er0 d uw1))
("ure" nil (y uw1 r))
("urea" nil (y er0 iy1 ax))
("uremia" nil (y er0 iy1 m iy0 ax))
("uren" nil (er0 eh1 n))
("urena" nil (er0 ey1 n ax))
("uresti" nil (er0 eh1 s t iy0))
("urethane" nil (y er1 ax th ey1 n))
("urey" nil (y er1 iy0))
("urfer" nil (er1 f er0))
("urge" nil (er1 jh))
("urged" nil (er1 jh d))
("urgen" nil (er1 jh eh1 n))
("urgency" nil (er1 jh ax n s iy0))
("urgent" nil (er1 jh ax n t))
("urgently" nil (er1 jh ax n t l iy0))
("urges" nil (er1 jh ax z))
("urging" nil (er1 jh ih0 ng))
("urgings" nil (er1 jh ih0 ng z))
("urgo" nil (er1 g ow0))
("uri" nil (y er1 iy0))
("urian" nil (y er1 iy0 ax n))
("uriarte" nil (y er1 iy0 aa1 r t iy0))
("urias" nil (y er0 iy1 ax z))
("uribe" nil (y er0 iy1 b iy0))
("uric" nil (y er1 ih0 k))
("urich" nil (y er1 ih0 k))
("urick" nil (y er1 r ih0 k))
("urie" nil (y er1 iy0))
("uriegas" nil (y er0 iy1 g ax z))
("urinalysis" nil (y er1 ax n ae1 l ax s ih0 s))
("urinary" nil (y er1 ax n eh1 r iy0))
("urinating" nil (y er1 ax n ey1 t ih0 ng))
("urine" nil (y er1 ax n))
("urioste" nil (y er0 iy0 ow1 s t iy0))
("urman" nil (er1 m ax n))
("urn" nil (er1 n))
("urness" nil (er1 n ax s))
("urokinase" nil (y er1 ax k ax n ey1 z))
("urologist" nil (y er0 aa1 l ax jh ax s t))
("urologists" nil (y er0 aa1 l ax jh ax s t s))
("urology" nil (y er0 aa1 l ax jh iy0))
("urquhart" nil (er1 k aa1 r t))
("urquidez" nil (er0 k w iy1 d eh0 z))
("urquidi" nil (er0 k w iy1 d iy0))
("urquiza" nil (er0 k w iy1 z ax))
("urrea" nil (er0 iy1 ax))
("urrutia" nil (er0 uw1 sh ax))
("ursa" nil (er1 s ax))
("urschel" nil (er1 sh ax l))
("ursery" nil (er1 s er0 iy0))
("ursie" nil (er1 s iy0))
("ursin" nil (er1 s ih0 n))
("ursini" nil (er0 s iy1 n iy0))
("urso" nil (er1 s ow0))
("ursola" nil (er0 s ow1 l ax))
("urson" nil (er1 s ax n))
("urstadt" nil (er1 s t ae1 t))
("ursula" nil (er1 s ax l ax))
("ursus" nil (er1 s ax s))
("ursy" nil (er1 s iy0))
("urton" nil (er1 t ax n))
("uruguay" nil (y er1 ax g w ey1))
("uruguayan" nil (y er1 ax g w ey1 ax n))
("urwin" nil (er1 w ax n))
("ury" nil (y uw1 r iy0))
("us" nil (ah1 s))
("usa" nil (y uw1 eh1 s ey1))
("usable" nil (y uw1 z ax b ax l))
("usage" nil (y uw1 s ax jh))
("usages" nil (y uw1 s ax jh ax z))
("usair" nil (y uw1 eh1 s eh1 r))
("usameribancs" nil (y uw1 eh1 s ax m eh1 r ih0 b ae1 n k s))
("usbancorp" nil (y uw1 eh1 s b ae1 ng k ao1 r p))
("use" n (y uw0 s))
("use" v (y uw1 z))
("usec" nil (y uw1 s eh0 k))
("used" nil (y uw1 z d))
("useful" nil (y uw1 s f ax l))
("usefully" nil (y uw1 s f ax l iy0))
("usefulness" nil (y uw1 s f ax l n ax s))
("useless" nil (y uw1 s l ax s))
("uselman" nil (y uw1 s ax l m ax n))
("uselton" nil (y uw1 s ax l t ax n))
("usenet" nil (y uw1 z n eh1 t))
("user" nil (y uw1 z er0))
("users" nil (y uw1 z er0 z))
("usery" nil (y uw1 z er0 iy0))
("uses" n (y uw1 s ax z))
("uses" v (y uw1 z ih0 z))
("usher" nil (ah1 sh er0))
("ushered" nil (ah1 sh er0 d))
("ushering" nil (ah1 sh er0 ih0 ng))
("ushers" nil (ah1 sh er0 z))
("usines" nil (y uw1 z iy1 n z))
("using" nil (y uw1 z ih0 ng))
("usinor" nil (y uw1 s ax n er0))
("usrey" nil (ah1 s r iy0))
("usry" nil (ah1 s r iy0))
("ussery" nil (ah1 s er0 iy0))
("ustinov" nil (uw1 s t ih0 n aa0 v))
("ustrust" nil (y uw1 eh1 s t r ah1 s t))
("usual" nil (y uw1 zh ax w ax l))
("usually" nil (y uw1 zh ax w ax l iy0))
("usurp" nil (y uw1 s er1 p))
("usurpation" nil (y uw1 s er0 p ey1 sh ax n))
("usurped" nil (y uw1 s er1 p t))
("usurping" nil (y uw1 s er1 p ih0 ng))
("usurps" nil (y uw1 s er1 p s))
("usury" nil (y uw1 zh er0 iy0))
("utah" nil (y uw1 t ao1))
("utamaro" nil (y uw1 t ax m aa1 r ow0))
("utech" nil (y uw1 t eh1 k))
("utecht" nil (y uw1 t eh1 k t))
("utensils" nil (y uw0 t eh1 n s ax l z))
("uterine" nil (y uw1 t er0 ax n))
("uterus" nil (y uw1 t er0 ax s))
("utgoff" nil (ah1 t g ao0 f))
("uthe" nil (ah1 dh))
("utica" nil (y uw1 t ax k ax))
("utilicorp" nil (y uw0 t ih1 l ax k ao0 r p))
("utilitarian" nil (y uw0 t ih1 l ax t eh1 r iy0 ax n))
("utilities" nil (y uw0 t ih1 l ax t iy0 z))
("utility" nil (y uw0 t ih1 l ax t iy0))
("utilization" nil (y uw1 t ax l ax z ey1 sh ax n))
("utilize" nil (y uw1 t ax l ay1 z))
("utilized" nil (y uw1 t ax l ay1 z d))
("utilizes" nil (y uw1 t ax l ay1 z ih0 z))
("utilizing" nil (y uw1 t ax l ay1 z ih0 ng))
("utke" nil (ah1 t k iy0))
("utley" nil (ah1 t l iy0))
("utmost" nil (ah1 t m ow1 s t))
("utopia" nil (y uw0 t ow1 p iy0 ax))
("utopian" nil (y uw0 t ow1 p iy0 ax n))
("utopians" nil (y uw0 t ow1 p iy0 ax n z))
("utopias" nil (y uw0 t ow1 p iy0 ax z))
("utsey" nil (ah1 t s iy0))
("utsumi" nil (uw0 t s uw1 m iy0))
("utt" nil (ah1 t))
("uttech" nil (ah1 t eh1 k))
("utter" nil (ah1 t er0))
("utterance" nil (ah1 t er0 ax n s))
("utterances" nil (ah1 t er0 ax n s ih0 z))
("uttered" nil (ah1 t er0 d))
("uttering" nil (ah1 t er0 ih0 ng))
("utterly" nil (ah1 t er0 l iy0))
("utters" nil (ah1 t er0 z))
("utz" nil (ah1 t s))
("utzinger" nil (ah1 t z ih0 ng er0))
("uva" nil (y uw1 v ax))
("uwe" nil (y uw1))
("uy" nil (uw1 iy0))
("uyeda" nil (ay0 iy1 d ax))
("uyehara" nil (uw0 y ey0 hh aa1 r ax))
("uyeno" nil (ay0 ey1 n ow0))
("uzbek" nil (uw1 z b eh0 k))
("uzbekistan" nil (uw0 z b eh1 k ih0 s t ae1 n))
("uzbeks" nil (uw1 z b eh0 k s))
("uzelac" nil (ax z eh1 l ax k))
("uzi" nil (uw1 z iy0))
("uzzell" nil (uw0 z eh1 l))
("uzzle" nil (ah1 z ax l))
("v" nil (v iy1))
("vaal" nil (v aa1 l))
("vaapenfabrikk" nil (v aa1 p eh1 n f ax b r ih0 k))
("vaca" nil (v ae1 k ax))
("vacancies" nil (v ey1 k ax n s iy0 z))
("vacancy" nil (v ey1 k ax n s iy0))
("vacant" nil (v ey1 k ax n t))
("vacanti" nil (v ax k ae1 n t iy0))
("vacate" nil (v ey1 k ey0 t))
("vacated" nil (v ey0 k ey1 t ax d))
("vacating" nil (v ey1 k ey0 t ih0 ng))
("vacation" nil (v ey0 k ey1 sh ax n))
("vacationed" nil (v ey0 k ey1 sh ax n d))
("vacationer" nil (v ey0 k ey1 sh ax n er0))
("vacationers" nil (v ey0 k ey1 sh ax n er0 z))
("vacationing" nil (v ey0 k ey1 sh ax n ih0 ng))
("vacations" nil (v ey0 k ey1 sh ax n z))
("vacaville" nil (v ae1 k ax v ih1 l))
("vacca" nil (v ae1 k ax))
("vaccarella" nil (v aa0 k er0 eh1 l ax))
("vaccaro" nil (v ax k aa1 r ow0))
("vaccinated" nil (v ae1 k s ax n ey0 t ih0 d))
("vaccination" nil (v ae0 k s ax n ey1 sh ax n))
("vaccinations" nil (v ae1 k s ax n ey1 sh ax n z))
("vaccine" nil (v ae1 k s iy1 n))
("vaccines" nil (v ae1 k s iy1 n z))
("vaccinia" nil (v ax ch iy1 n iy0 ax))
("vacek" nil (v aa1 ch eh1 k))
("vacha" nil (v ae1 ch ax))
("vachon" nil (v ae1 ch ax n))
("vacillate" nil (v ae1 s ax l ey1 t))
("vacillated" nil (v ae1 s ax l ey0 t ih0 d))
("vacillating" nil (v ae1 s ax l ey1 t ih0 ng))
("vacillation" nil (v ae1 s ax l ey1 sh ax n))
("vaclav" nil (v ae1 k l aa1 v))
("vaclavik" nil (v ae1 k l ax v ih0 k))
("vacuous" nil (v ae1 k y uw0 ax s))
("vacuum" nil (v ae1 k y uw0 m))
("vacuuming" nil (v ae1 k y uw0 m ih0 ng))
("vacuums" nil (v ae1 k y uw0 m z))
("vadala" nil (v aa0 d aa1 l ax))
("vadas" nil (v aa1 d aa0 z))
("vadehra" nil (v ax d ey1 r ax))
("vaden" nil (v ey1 d ax n))
("vader" nil (v ey1 d er0))
("vadim" nil (v aa0 d iy1 m))
("vadnais" nil (v ax d n ey1))
("vadney" nil (v ae1 d n iy0))
("vaeth" nil (v iy1 th))
("vagabond" nil (v ae1 g ax b aa0 n d))
("vagabonds" nil (v ae1 g ax b aa0 n d z))
("vagaries" nil (v ey1 g er0 iy0 z))
("vagelos" nil (v ax jh eh1 l ow0 s))
("vagina" nil (v ax jh ay1 n ax))
("vaginal" nil (v ax jh ay1 n ax l))
("vagrancy" nil (v ey1 g r ax n s iy0))
("vagrants" nil (v ey1 g r ax n t s))
("vague" nil (v ey1 g))
("vaguely" nil (v ey1 g l iy0))
("vagueness" nil (v ey1 g n ih0 s))
("vaguer" nil (v ey1 g er0))
("vaguest" nil (v ey1 g ih0 s t))
("vaguine" nil (v ax g w iy1 n))
("vahey" nil (v ae1 hh iy0))
("vahid" nil (v aa0 hh iy1 d))
("vahl" nil (v aa1 l))
("vahle" nil (v ey1 hh ax l))
("vail" nil (v ey1 l))
("vaile" nil (v ey1 l))
("vaillancourt" nil (v ey1 l ax n k ao1 r t))
("vain" nil (v ey1 n))
("vainly" nil (v ey1 n l iy0))
("vajda" nil (v ay1 d ax))
("vajna" nil (v ay1 n ax))
("val" nil (v ae1 l))
("vala" nil (v aa1 l ax))
("valade" nil (v aa0 l aa1 d ey0))
("valadez" nil (v aa0 l aa1 d eh0 z))
("valasek" nil (v ax l aa1 s eh0 k))
("valborga" nil (v aa0 l b ao1 r g ax))
("valcourt" nil (v ax l k ao1 r t))
("valda" nil (v aa1 l d ax))
("valdemar" nil (v aa0 l d ey0 m aa1 r))
("valderrama" nil (v aa0 l d eh0 r aa1 m ax))
("valdes" nil (v ae0 l d eh1 z))
("valdez" nil (v ae0 l d eh1 z))
("valdis" nil (v ae1 l d ih0 s))
("valdivia" nil (v aa0 l d iy1 v iy0 ax))
("valdovinos" nil (v aa0 l d ow0 v iy1 n ow0 z))
("vale" nil (v ey1 l))
("valeda" nil (v aa0 l ey1 d ax))
("valedictory" nil (v ae1 l ax d ih1 k t er0 iy0))
("valek" nil (v aa1 l eh0 k))
("valence" nil (v ey1 l ax n s))
("valencia" nil (v ax l eh1 n s iy0 ax))
("valencienne" nil (v ax l eh1 n s iy0 eh1 n))
("valens" nil (v ae1 l ax n z))
("valensuela" nil (v ae1 l ax n z w ey1 l ax))
("valent" nil (v aa0 l ey1 ax n t))
("valenta" nil (v ax l eh1 n t ax))
("valente" nil (v aa0 l eh1 n t iy0))
("valenti" nil (v ax l eh1 n t iy0))
("valentia" nil (v aa0 l eh1 n sh ax))
("valentin" nil (v aa0 l ey0 n t iy1 n))
("valentina" nil (v ae0 l ih0 n t iy1 n ax))
("valentine" nil (v ae1 l ax n t ay1 n))
("valentini" nil (v ae1 l ax n t iy1 n iy0))
("valentino" nil (v ae1 l ax n t iy1 n ow0))
("valenza" nil (v ax l eh1 n z ax))
("valenzano" nil (v ae1 l eh0 n z aa1 n ow0))
("valenzuela" nil (v aa0 l eh0 n z uw1 l ax))
("valeo" nil (v ae1 l iy0 ow0))
("valera" nil (v ax l eh1 r ax))
("valeri" nil (v aa0 l eh1 r iy0))
("valeria" nil (v ax l iy1 r iy0 ax))
("valerian" nil (v ax l ih1 r iy0 ax n))
("valeriano" nil (v aa0 l er0 iy0 aa1 n ow0))
("valerie" nil (v ae1 l er0 iy0))
("valerio" nil (v ax l iy1 r iy0 ow0))
("valero" nil (v ax l eh1 r ow0))
("valery" nil (v ae1 l er0 iy0))
("vales" nil (v ey1 l z))
("valeska" nil (v aa0 l ey1 s k ax))
("valet" nil (v ae0 l ey1))
("valhalla" nil (v ae1 l hh ae1 l ax))
("valhi" nil (v ae1 l hh iy0))
("valiant" nil (v ae1 l y ax n t))
("valiantly" nil (v ae1 l y ax n t l iy0))
("valid" nil (v ae1 l ax d))
("valida" nil (v aa0 l iy1 d ax))
("validate" nil (v ae1 l ax d ey0 t))
("validated" nil (v ae1 l ax d ey0 t ax d))
("validates" nil (v ae1 l ax d ey1 t s))
("validating" nil (v ae1 l ax d ey1 t ih0 ng))
("validation" nil (v ae1 l ax d ey1 sh ax n))
("validity" nil (v ax l ih1 d ax t iy0))
("validly" nil (v ae1 l ih0 d l iy0))
("valiente" nil (v aa0 l iy1 n t iy0))
("valin" nil (v ae1 l ih0 n))
("valiquette" nil (v ae1 l ih0 k eh1 t))
("valis" nil (v ae1 l ih0 s))
("valium" nil (v ey1 l iy0 ax m))
("valk" nil (v ao1 k))
("valko" nil (v ae1 l k ow0))
("valla" nil (v ae1 l ax))
("valladares" nil (v aa0 l aa0 d aa1 r eh0 s))
("vallance" nil (v ae1 l ax n s))
("vallandingham" nil (v ae0 l ah1 n d ih0 ng hh ae1 m))
("vallario" nil (v aa0 l aa1 r iy0 ow0))
("vallarta" nil (v ae1 l aa1 r t ax))
("vallas" nil (v ae1 l ax z))
("valle" nil (v ey1 l))
("valleau" nil (v ax l ow1))
("vallee" nil (v ae1 l iy0))
("vallejo" nil (v aa0 l ey1 y ow0))
("vallejos" nil (v aa0 l ey1 y ow0 z))
("vallely" nil (v ey1 l iy0))
("vallery" nil (v ae1 l er0 iy0))
("valles" nil (v ey1 l z))
("vallette" nil (v ae1 l eh1 t))
("valley" nil (v ae1 l iy0))
("valleys" nil (v ae1 l iy0 z))
("vallez" nil (v aa0 l eh1 z))
("valli" nil (v ae1 l iy0))
("valliant" nil (v aa0 l iy1 ax n t))
("vallie" nil (v ao1 l iy0))
("vallier" nil (v ae1 l iy0 er0))
("valliere" nil (v ae1 l iy0 eh0 r))
("vallis" nil (v ae1 l ih0 s))
("vallo" nil (v ae1 l ow0))
("vallone" nil (v aa0 l ow1 n iy0))
("vallot" nil (v ae1 l ax t))
("valls" nil (v ao1 l z))
("valmont" nil (v ao1 l m aa1 n t))
("valois" nil (v ae0 l w aa1))
("valone" nil (v ax l ow1 n))
("valonia" nil (v ax l ow1 n y ax))
("valor" nil (v ae1 l er0))
("valoree" nil (v ae1 l er0 iy1))
("valores" nil (v ax l ao1 r eh0 z))
("valparaiso" nil (v ae1 l p er0 ey1 s ow0))
("valsella" nil (v ao0 l s eh1 l ax))
("valspar" nil (v ae1 l z p aa0 r))
("valtek" nil (v ao1 l t eh1 k))
("valtierra" nil (v aa0 l t ih1 r ax))
("valu" nil (v ae1 l y uw0))
("valuable" nil (v ae1 l y ax b ax l))
("valuables" nil (v ae1 l y ax b ax l z))
("valuation" nil (v ae0 l y uw0 ey1 sh ax n))
("valuations" nil (v ae0 l y uw0 ey1 sh ax n z))
("value" nil (v ae1 l y uw0))
("valued" nil (v ae1 l y uw0 d))
("valueless" nil (v ae1 l y uw0 l ax s))
("valuepoint" nil (v ae1 l y uw0 p oy1 n t))
("values" nil (v ae1 l y uw0 z))
("valuevision" nil (v ae1 l y uw0 v ih1 zh ax n))
("valuing" nil (v ae1 l y uw0 ih0 ng))
("valujet" nil (v ae1 l y uw0 jh eh1 t))
("valukas" nil (v ax l uw1 k ax s))
("valvano" nil (v aa0 l v aa1 n ow0))
("valve" nil (v ae1 l v))
("valverde" nil (v aa0 l v eh1 r d ey0))
("valves" nil (v ae1 l v z))
("valvo" nil (v aa1 l v ow0))
("valvoline" nil (v ae1 l v ax l iy1 n))
("vamos" nil (v aa1 m ow0 z))
("vamp" nil (v ae1 m p))
("vampire" nil (v ae1 m p ay0 r))
("vampires" nil (v ae1 m p ay0 r z))
("vampiric" nil (v ae0 m p ih1 r ih0 k))
("van" nil (v ae1 n))
("vana" nil (v ae1 n ax))
("vanacker" nil (v ae0 n ae1 k er0))
("vanacore" nil (v aa0 n aa0 k ao1 r iy0))
("vanadium" nil (v ax n ey1 d iy0 ax m))
("vanaken" nil (v ae1 n ax k ax n))
("vanallen" nil (v ax n ao1 l ax n))
("vanalstine" nil (v ae0 n ae1 l s t iy0 n))
("vanalstyne" nil (v ae1 n ax l s t ay1 n))
("vanaman" nil (v ae0 n ao1 m ax n))
("vanamburg" nil (v ae0 n ae1 m b er0 g))
("vanamburgh" nil (v ae0 n ae1 m b er0 g))
("vanantwerp" nil (v ax n ae1 n t w er0 p))
("vanarsdale" nil (v ae1 n er0 z d ey1 l))
("vanarsdall" nil (v ax n aa1 r s d ax l))
("vanasse" nil (v ae0 n ae1 s))
("vanatta" nil (v aa0 n aa1 t ax))
("vanauken" nil (v ae0 n aw1 k ax n))
("vanausdall" nil (v ae0 n aw1 s d ax l))
("vanbebber" nil (v ae1 n b eh1 b er0))
("vanbeek" nil (v ae1 n b iy1 k))
("vanbenschoten" nil (v ae0 n b eh1 n sh ax t ax n))
("vanbergen" nil (v ae0 n b er1 g ax n))
("vanbibber" nil (v ae1 n b ih1 b er0))
("vanblarcom" nil (v ae1 n b l aa1 r k ax m))
("vanblaricom" nil (v ae1 n b l ae1 r ih0 k ax m))
("vanbrocklin" nil (v ae1 n b r aa1 k l ih0 n))
("vanbrunt" nil (v ae1 n b r ah1 n t))
("vanburen" nil (v ae0 n b y uw1 r ax n))
("vanbuskirk" nil (v ae1 n b ah1 s k er0 k))
("vancamp" nil (v ae1 n k ae1 m p))
("vancampen" nil (v ae1 n k ae1 m p ax n))
("vance" nil (v ae1 n s))
("vancil" nil (v ae1 n s ax l))
("vancise" nil (v aa1 n ch ay0 s))
("vancleave" nil (v ae1 n k l ax v))
("vancleef" nil (v ae1 n k l iy1 f))
("vancleve" nil (v ae1 n k l iy1 v))
("vancott" nil (v ax n k aa1 t))
("vancourt" nil (v ax n k ao1 r t))
("vancouver" nil (v ae0 n k uw1 v er0))
("vancura" nil (v aa0 n k uh1 r ax))
("vancuren" nil (v ae0 n k y uw1 r ax n))
("vandaele" nil (v ae0 n d eh1 l))
("vandagriff" nil (v ae1 n d ae1 g r ih0 f))
("vandal" nil (v ae1 n d ax l))
("vandalen" nil (v ae1 n d ae1 l ax n))
("vandalism" nil (v ae1 n d ax l ih0 z ax m))
("vandalized" nil (v ae1 n d ax l ay1 z d))
("vandall" nil (v ae1 n d ao1 l))
("vandals" nil (v ae1 n d ax l z))
("vandam" nil (v ae1 n d ae1 m))
("vandamme" nil (v ae1 n d ae1 m))
("vande" nil (v ae1 n d))
("vandeberg" nil (v ae1 n d ax b er0 g))
("vandecar" nil (v ae1 n d ax k aa1 r))
("vandegrift" nil (v ae1 n d ax g r ih0 f t))
("vandehei" nil (v ae1 n d ax hh ay0))
("vandehey" nil (v ae1 n d ih0 hh iy0))
("vandeman" nil (v ae1 n d ax m ax n))
("vandemark" nil (v ae1 n d ax m aa1 r k))
("vanden" nil (v ae1 n d ax n))
("vandenberg" nil (v ae1 n d ax n b er0 g))
("vandenbergh" nil (v ae1 n d ax n b er0 g))
("vandenberghe" nil (v ae1 n d ax n b er0 g))
("vandenboom" nil (v ae1 n d ax n b uw1 m))
("vandenbos" nil (v ae1 n d ax n b ow0 z))
("vandenbosch" nil (v ae1 n d ax n b ao1 sh))
("vandenbrink" nil (v ae1 n d ax n b r ih0 ng k))
("vandenburg" nil (v ae1 n d ax n b er0 g))
("vandenburgh" nil (v ae1 n d ax n b er0 g))
("vandenheuvel" nil (v ae1 n d ax n hh oy0 v ax l))
("vander" nil (v ae1 n d er0))
("vanderbeck" nil (v ae1 n d er0 b eh1 k))
("vanderbeek" nil (v ae1 n d er0 b iy1 k))
("vanderberg" nil (v ae1 n d er0 b er0 g))
("vanderbilt" nil (v ae1 n d er0 b ih0 l t))
("vanderburg" nil (v ae1 n d er0 b er0 g))
("vanderbush" nil (v ae1 n d er0 b ax sh))
("vandercook" nil (v ae1 n d er0 k uh1 k))
("vanderford" nil (v ae1 n d er0 f er0 d))
("vandergriff" nil (v ae1 n d er0 g r ih0 f))
("vandergrift" nil (v ae1 n d er0 g r ih0 f t))
("vanderheide" nil (v ae1 n d er0 hh ay1 d))
("vanderheiden" nil (v ae1 n d er0 hh ay0 d ax n))
("vanderheyden" nil (v ae1 n d er0 hh ey0 d ax n))
("vanderhoef" nil (v ae1 n d er0 hh ow1 f))
("vanderhoff" nil (v ae1 n d er0 hh ao1 f))
("vanderhoof" nil (v ae1 n d er0 hh uh1 f))
("vanderhorst" nil (v ae1 n d er0 hh ao1 r s t))
("vanderkolk" nil (v ae1 n d er0 k ow1 k))
("vanderkooi" nil (v ae1 n d er0 k uw1 iy0))
("vanderlaan" nil (v ae1 n d er0 l aa1 n))
("vanderleest" nil (v ae1 n d er0 ax l iy1 s t))
("vanderlinde" nil (v ae1 n d er0 l ih1 n d))
("vanderlinden" nil (v ae1 n d er0 l ih1 n d ax n))
("vanderlip" nil (v ae1 n d er0 l ih1 p))
("vandermark" nil (v ae1 n d er0 m aa1 r k))
("vandermeer" nil (v ae1 n d er0 m ih1 r))
("vandermeulen" nil (v ae1 n d er0 m oy1 l ax n))
("vandermolen" nil (v ae1 n d er0 m aa1 l ax n))
("vanderploeg" nil (v ae1 n d er0 p l ow1 g))
("vanderpoel" nil (v ae1 n d er0 p ow1 l))
("vanderpol" nil (v ae1 n d er0 p ao1 l))
("vanderpool" nil (v ae1 n d er0 p uw1 l))
("vanderschaaf" nil (v ae1 n d er0 sh aa1 f))
("vanderslice" nil (v ae1 n d er0 s l ay1 s))
("vandersluis" nil (v ae1 n d er0 s l uw1 ih0 s))
("vanderveen" nil (v ae1 n d er0 v iy1 n))
("vanderveer" nil (v ae1 n d er0 v ih1 r))
("vandervelde" nil (v ae1 n d er0 v eh1 l d))
("vandervelden" nil (v ae1 n d er0 v eh1 l d ax n))
("vandervliet" nil (v ae1 n d er0 v l iy1 t))
("vandervoort" nil (v ae1 n d er0 v uh1 r t))
("vandervort" nil (v ae1 n d er0 v ao1 r t))
("vanderwal" nil (v ae1 n d er0 w aa1 l))
("vanderwall" nil (v ae1 n d er0 w aa1 l))
("vanderweele" nil (v ae1 n d er0 w iy1 l))
("vanderweide" nil (v ae1 n d er0 w ay1 d))
("vanderwerf" nil (v ae1 n d er0 w er1 f))
("vanderwerff" nil (v ae1 n d er0 w er1 f))
("vanderwilt" nil (v ae1 n d er0 w ih1 l t))
("vanderwoude" nil (v ae1 n d er0 w aw1 d))
("vanderzanden" nil (v ae1 n d er0 z ae1 n d ax n))
("vanderzee" nil (v ae1 n d er0 z iy1))
("vandeusen" nil (v ae0 n d oy1 s ax n))
("vandevander" nil (v ae1 n d ax v ae1 n d er0))
("vandeveer" nil (v ae1 n d ax v ih1 r))
("vandevelde" nil (v ae1 n d ax v eh1 l d))
("vandeven" nil (v ae1 n d ax v ax n))
("vandevender" nil (v ae1 n d ax v eh1 n d er0))
("vandeventer" nil (v ae1 n d ax v ax n t er0))
("vandever" nil (v ae1 n d iy1 v er0))
("vandevoorde" nil (v ae1 n d ax v uh0 r d))
("vandevoort" nil (v ae1 n d ax v uh0 r t))
("vandewalker" nil (v ae1 n d ax w ao1 k er0))
("vandewalle" nil (v ae0 n d uw1 ey0 l))
("vandewater" nil (v ae1 n d ax w ao0 t er0))
("vandezande" nil (v ae1 n d ax z ih0 n d))
("vandine" nil (v ae1 n d ay1 n))
("vandiver" nil (v ae1 n d ay1 v er0))
("vandivier" nil (v ae0 n d ay1 v iy0 er0))
("vandoren" nil (v ae0 n d ao1 r ax n))
("vandorn" nil (v ae0 n d ao1 r n))
("vandriel" nil (v ae1 n d r iy1 l))
("vandunk" nil (v ae1 n d ah1 ng k))
("vandusen" nil (v ae0 n d uw1 s ax n))
("vanduyn" nil (v ae1 n d ay1 n))
("vanduyne" nil (v ae1 n d ay1 n))
("vanduzer" nil (v ae1 n d uw1 z er0))
("vandyck" nil (v ae0 n d ih1 k))
("vandyk" nil (v ae1 n d ih0 k))
("vandyke" nil (v ae1 n d ay1 k))
("vandyken" nil (v ae1 n d ay1 k ax n))
("vandyne" nil (v ae1 n d ay1 n))
("vane" nil (v ey1 n))
("vaneaton" nil (v ae1 n ax t aa0 n))
("vanecek" nil (v ae0 n eh1 s ih0 k))
("vaneck" nil (v aa1 n eh0 k))
("vanegas" nil (v ae0 n iy1 g ax z))
("vanek" nil (v ae0 n eh1 k))
("vanella" nil (v ax n eh1 l ax))
("vanengen" nil (v ae0 n eh1 ng ax n))
("vanepps" nil (v ae0 n eh1 p s))
("vanes" nil (v ey1 n z))
("vaness" nil (v ae1 n ax s))
("vanessa" nil (v ax n eh1 s ax))
("vanessen" nil (v aa1 n ih0 s ax n))
("vanetten" nil (v ae1 n eh1 t ax n))
("vanevery" nil (v ax n eh1 v r iy0))
("vanfleet" nil (v ae1 n f l iy1 t))
("vanfossan" nil (v ae0 n f aa1 s ax n))
("vanfossen" nil (v ae0 n f aa1 s ax n))
("vang" nil (v ae1 ng))
("vangel" nil (v ey1 ng g ax l))
("vangelder" nil (v ey1 ng g ih0 l d er0))
("vangie" nil (v ae1 ng iy0))
("vangieson" nil (v ae1 ng g iy0 z ax n))
("vangilder" nil (v ae1 ng g ih0 l d er0))
("vangorden" nil (v ae1 ng g er0 d ax n))
("vangorder" nil (v ae1 ng g er0 d er0))
("vangorp" nil (v ae1 ng g er0 p))
("vanguard" nil (v ae1 n g aa1 r d))
("vanguilder" nil (v ae0 n g ay1 l d er0))
("vangundy" nil (v ax ng g ah1 n d iy0))
("vangy" nil (v ae1 n jh iy0))
("vanhall" nil (v ae1 n hh ao1 l))
("vanhandel" nil (v ae1 n hh ae1 n d ax l))
("vanhecke" nil (v ae1 n hh eh1 k))
("vanheel" nil (v ae1 n hh iy1 l))
("vanhise" nil (v ae0 n hh ay1 z))
("vanhoesen" nil (v ae1 n hh ow1 s ax n))
("vanhook" nil (v ae1 n hh uh1 k))
("vanhoose" nil (v ae1 n hh uw1 s))
("vanhooser" nil (v ae1 n hh uw1 z er0))
("vanhoozer" nil (v ae1 n hh uw1 z er0))
("vanhorn" nil (v ae0 n hh ao1 r n))
("vanhorne" nil (v ae0 n hh ao1 r n))
("vanhousen" nil (v ae1 n hh aw1 s ax n))
("vanhouten" nil (v ae0 n hh aa1 uw0 t ax n))
("vanhove" nil (v ae0 n hh ah1 v))
("vanhoy" nil (v ae1 n hh oy0))
("vanhuss" nil (v ae1 n hh ah1 s))
("vanhyning" nil (v ae1 n hh ay1 n ih0 ng))
("vania" nil (v aa1 n iy0 ax))
("vanier" nil (v ae0 n iy1 er0))
("vanik" nil (v aa1 n ih0 k))
("vanilla" nil (v ax n ih1 l ax))
("vanillin" nil (v ax n ih1 l ih0 n))
("vanish" nil (v ae1 n ih0 sh))
("vanished" nil (v ae1 n ih0 sh t))
("vanishes" nil (v ae1 n ih0 sh ih0 z))
("vanishing" nil (v ae1 n ih0 sh ih0 ng))
("vanities" nil (v ae1 n ax t iy0 z))
("vanity" nil (v ae1 n ax t iy0))
("vankampen" nil (v ae1 n k ae1 m p ax n))
("vankeuren" nil (v ae1 ng k oy0 r ax n))
("vankirk" nil (v ae1 ng k er0 k))
("vankleeck" nil (v ae1 ng k l iy1 k))
("vankuren" nil (v ae1 ng k y uw0 r ax n))
("vanlandingham" nil (v ae1 n l ae1 n d ih0 ng hh ae1 m))
("vanlanen" nil (v ae1 n l ae1 n ax n))
("vanlaningham" nil (v ae1 n l ae1 n ih0 ng hh ae1 m))
("vanleer" nil (v ae0 n l ih1 r))
("vanleeuwen" nil (v ae0 n l uw1 ax n))
("vanleuven" nil (v ae0 n l oy1 v ax n))
("vanliere" nil (v ae0 n l ih1 r))
("vanliew" nil (v ae1 n l iy0 uw0))
("vanloan" nil (v ae1 n l ow1 n))
("vanloo" nil (v ae1 n l uw1))
("vanlue" nil (v ae1 n l uw1))
("vanluven" nil (v ae1 n l uw1 v ax n))
("vanmaanen" nil (v ae1 n m aa1 n ax n))
("vanmarter" nil (v ae0 n m aa1 r t er0))
("vanmatre" nil (v ae0 n m ey1 t er0))
("vanmeter" nil (v ae0 n m iy1 t er0))
("vanmetre" nil (v ae0 n m iy1 t er0))
("vann" nil (v ae1 n))
("vanna" nil (v ae1 n ax))
("vanname" nil (v ae1 n ax m))
("vannatta" nil (v aa0 n aa1 t ax))
("vannatter" nil (v ae0 n ae1 t er0))
("vannelli" nil (v aa0 n eh1 l iy0))
("vanness" nil (v ae0 n iy1 s))
("vannest" nil (v ae0 n iy1 s t))
("vannguyen" nil (v ae0 n g iy1 ax n))
("vanni" nil (v ae1 n iy0))
("vannice" nil (v ae1 n ih0 s))
("vannie" nil (v ae1 n iy0))
("vannorman" nil (v ae0 n ao1 r m ax n))
("vannortwick" nil (v ax n ao1 r t w ih0 k))
("vannostrand" nil (v ae0 n aa1 s t r ax n d))
("vannote" nil (v ae0 n ow1 t))
("vannoy" nil (v ae1 n oy0))
("vannucci" nil (v aa0 n uw1 ch iy0))
("vanny" nil (v ae1 n iy0))
("vano" nil (v aa1 n ow0))
("vanora" nil (v ae1 n er0 ax))
("vanorden" nil (v ae0 n ao1 r d ax n))
("vanorder" nil (v ae0 n ao1 r d er0))
("vanorman" nil (v ae0 n ao1 r m ax n))
("vanornum" nil (v ae0 n ao1 r n ax m))
("vanosdol" nil (v ae0 n aa1 s d ax l))
("vanoss" nil (v ae0 n aa1 s))
("vanostrand" nil (v ae0 n aa1 s t r ax n d))
("vanous" nil (v ae1 n ax s))
("vanover" nil (v ae1 n ow1 v er0))
("vanpatten" nil (v ae0 n p ae1 t ax n))
("vanpelt" nil (v ae1 n p eh1 l t))
("vanputten" nil (v ae0 n p ah1 t ax n))
("vanquished" nil (v ae1 ng k w ih0 sh t))
("vanriper" nil (v ae1 n r ay1 p er0))
("vanroekel" nil (v ae1 n r ow1 k ax l))
("vanrossum" nil (v ae1 n r aa1 s ax m))
("vanryn" nil (v ae1 n r ih1 n))
("vans" nil (v ae1 n z))
("vansandt" nil (v ae1 n s ae1 n t))
("vansant" nil (v aa1 n s ax n t))
("vanschaick" nil (v ae0 n sh ay1 k))
("vanschoick" nil (v ae1 n sh oy1 k))
("vansciver" nil (v ae0 n s k ay1 v er0))
("vanscoy" nil (v ae1 n s k oy1))
("vanscoyoc" nil (v ae1 n s k oy1 aa0 k))
("vanscyoc" nil (v ae1 n s iy0 aa1 k))
("vanselow" nil (v ae1 n s ih0 l ow0))
("vansickel" nil (v ae1 n s ih1 k ax l))
("vansickle" nil (v ae1 n s ih1 k ax l))
("vanskike" nil (v ae1 n s k ay1 k))
("vanskiver" nil (v ae1 n s k ay1 v er0))
("vanslooten" nil (v ae1 n s l uw1 t ax n))
("vanslyke" nil (v ae1 n s l ay1 k))
("vanstone" nil (v ae1 n s t ow1 n))
("vanstory" nil (v ae1 n s t ao1 r iy0))
("vanstraten" nil (v ae1 n s t r ey1 t ax n))
("vansyckle" nil (v ae1 n s ay0 k ax l))
("vantage" nil (v ae1 n t ax jh))
("vantages" nil (v ae1 n t ih0 jh ih0 z))
("vantassel" nil (v ae1 n t ae1 s ax l))
("vantassell" nil (v ae1 n t ae1 s ax l))
("vantil" nil (v aa0 n t iy1 l))
("vantilburg" nil (v ae1 n t ih1 l b er0 g))
("vantine" nil (v aa0 n t iy1 n iy0))
("vantol" nil (v ae0 n t ao1 l))
("vantran" nil (v ae1 n t r ae1 n))
("vantrease" nil (v ax n t r iy1 s))
("vantreese" nil (v ae1 n t r iy1 s))
("vantuyl" nil (v ae0 n t ay1 l))
("vanuaaku" nil (v ae1 n w ax aa1 k uw1))
("vanuatu" nil (v ae0 n uw0 aa1 t uw0))
("vanunu" nil (v ax n uw1 n uw0))
("vanvalkenburg" nil (v ae0 n v ao1 k ax n b er0 g))
("vanvalkenburgh" nil (v ae0 n v ae1 l k ih0 n b er0 g))
("vanveen" nil (v ax n v iy1 n))
("vanvlack" nil (v ae1 n v l ae1 k))
("vanvleck" nil (v ae1 n v l eh1 k))
("vanvleet" nil (v ae1 n v l iy1 t))
("vanvliet" nil (v ae1 n v l iy1 t))
("vanvooren" nil (v ae0 n v uh1 r ax n))
("vanvoorhis" nil (v ae0 n v uh1 r hh ih0 s))
("vanvoorst" nil (v ae0 n v uh1 r s t))
("vanvorst" nil (v ae0 n v ao1 r s t))
("vanvranken" nil (v ae1 n v r ae1 ng k ax n))
("vanwagenen" nil (v ae0 n w ae1 g ax n ax n))
("vanwagner" nil (v ae1 n w ae1 g n er0))
("vanwagoner" nil (v ae1 n w ae1 g ax n er0))
("vanwart" nil (v ae1 n w ao1 r t))
("vanwert" nil (v ae0 n w er1 t))
("vanwey" nil (v ae1 n w iy0))
("vanwhy" nil (v ae1 n w iy0))
("vanwie" nil (v ae1 n w iy0))
("vanwieren" nil (v ae0 n w ih1 r ax n))
("vanwinkle" nil (v ae1 n w ih1 ng k ax l))
("vanwormer" nil (v ae0 n w er1 m er0))
("vanwyck" nil (v ae0 n w ih1 k))
("vanwyhe" nil (v ae1 n w ay0 hh))
("vanwyk" nil (v ae0 n w ih1 k))
("vanya" nil (v aa1 n y ax))
("vanyo" nil (v aa1 n y ow0))
("vanzandt" nil (v ae1 n z ae1 n t))
("vanzant" nil (v aa1 n z ax n t))
("vanzanten" nil (v ae1 n z ae1 n t ax n))
("vanzee" nil (v aa1 n z iy0))
("vanzile" nil (v aa1 n z ay0 l))
("vapid" nil (v ae1 p ih0 d))
("vapor" nil (v ey1 p er0))
("vaporization" nil (v ey0 p er0 ax z ey1 sh ax n))
("vaporize" nil (v ey1 p er0 ay1 z))
("vaporized" nil (v ey1 p er0 ay1 z d))
("vapors" nil (v ey1 p er0 z))
("vaporware" nil (v ey1 p er0 w eh1 r))
("vaquera" nil (v aa0 k w eh1 r ax))
("vara" nil (v aa1 r ax))
("varady" nil (v er0 aa1 d iy0))
("varani" nil (v er0 aa1 n iy0))
("varano" nil (v aa0 r aa1 n ow0))
("varble" nil (v aa1 r b ax l))
("varco" nil (v aa1 r k ow0))
("vardeman" nil (v aa1 r d m ax n))
("varden" nil (v aa1 r d ax n))
("vardon" nil (v aa0 r d ao1 n))
("varela" nil (v aa0 r ey1 l ax))
("varga" nil (v aa1 r g ax))
("vargas" nil (v aa1 r g ax s))
("vargason" nil (v aa1 r g ax s ax n))
("varghese" nil (v aa1 r g hh iy0 z))
("vargo" nil (v aa1 r g ow0))
("vari" nil (v aa1 r iy0))
("variability" nil (v eh0 r iy0 ax b ih1 l ih0 t iy0))
("variable" nil (v eh1 r iy0 ax b ax l))
("variables" nil (v eh1 r iy0 ax b ax l z))
("varian" nil (v eh1 r iy0 ax n))
("variance" nil (v eh1 r iy0 ax n s))
("variances" nil (v eh1 r iy0 ax n s ih0 z))
("variant" nil (v eh1 r iy0 ax n t))
("variants" nil (v eh1 r iy0 ax n t s))
("variation" nil (v eh1 r iy0 ey1 sh ax n))
("variations" nil (v eh1 r iy0 ey1 sh ax n z))
("varick" nil (v eh1 r ih0 k))
("varied" nil (v eh1 r iy0 d))
("varies" nil (v eh1 r iy0 z))
("varietal" nil (v er0 iy1 t ax l))
("varietals" nil (v er0 ay1 ax t ax l z))
("varieties" nil (v er0 ay1 ax t iy0 z))
("variety" nil (v er0 ay1 ax t iy0))
("varig" nil (v eh1 r ih0 g))
("varin" nil (v aa0 r iy1 n))
("varina" nil (v aa0 r iy1 n ax))
("various" nil (v eh1 r iy0 ax s))
("variously" nil (v eh1 r iy0 ax s l iy0))
("varisco" nil (v aa0 r iy1 s k ow0))
("varitronic" nil (v eh1 r ih0 t r aa1 n ih0 k))
("varity" nil (v eh1 r ih0 t iy0))
("varityper" nil (v eh1 r ih0 t ay1 p er0))
("varlam" nil (v aa1 r l ae0 m))
("varlen" nil (v aa1 r l ax n))
("varley" nil (v aa1 r l iy0))
("varma" nil (v aa1 r m ax))
("varmint" nil (v aa1 r m ax n t))
("varmus" nil (v aa1 r m ax s))
("varn" nil (v aa1 r n))
("varnado" nil (v aa0 r n aa1 d ow0))
("varnadoe" nil (v aa0 r n aa1 d ow0))
("varnadore" nil (v aa0 r n aa0 d ao1 r ey0))
("varnell" nil (v aa1 r n ax l))
("varner" nil (v aa1 r n er0))
("varnes" nil (v aa1 r n z))
("varney" nil (v aa1 r n iy0))
("varni" nil (v aa1 r n iy0))
("varnish" nil (v aa1 r n ih0 sh))
("varnished" nil (v aa1 r n ih0 sh t))
("varnishes" nil (v aa1 r n ih0 sh ax z))
("varnon" nil (v aa0 r n ao1 n))
("varnum" nil (v aa1 r n ax m))
("varo" nil (v eh1 r ow0))
("varon" nil (v aa0 r ao1 n))
("varona" nil (v aa0 r ow1 n ax))
("varone" nil (v er0 ow1 n))
("varricchio" nil (v aa0 r iy1 k iy0 ow0))
("varrone" nil (v aa0 r ow1 n ey0))
("vars" nil (v aa1 r z))
("varsity" nil (v aa1 r s ih0 t iy0))
("vartanian" nil (v aa0 r t ey1 n iy0 ax n))
("varvaro" nil (v aa0 r v aa1 r ow0))
("varvel" nil (v aa0 r v eh1 l))
("varves" nil (v aa1 r v z))
("vary" nil (v eh1 r iy0))
("varying" nil (v eh1 r iy0 ih0 ng))
("varzi" nil (v aa1 r z iy0))
("vasbinder" nil (v ae1 s b ih0 n d er0))
("vasco" nil (v ae1 s k ow0))
("vasconcellos" nil (v ae0 s k ax n s eh1 l ow0 z))
("vasconcelos" nil (v aa0 s k ow0 n s ey1 l ow0 z))
("vascular" nil (v ae1 s k y ax l er0))
("vase" nil (v ey1 s))
("vasek" nil (v aa1 s eh0 k))
("vaseline" nil (v ae1 s ax l iy1 n))
("vases" nil (v ey1 s ax z))
("vasey" nil (v ae1 s iy0))
("vashti" nil (v ae1 sh t iy0))
("vasicek" nil (v aa1 s ih0 ch eh0 k))
("vasil" nil (v aa0 s iy1 l))
("vasile" nil (v aa1 s ax l))
("vasily" nil (v ae1 s ax l iy0))
("vaske" nil (v ey1 s k))
("vaskevitch" nil (v ae1 s k ax v ih0 ch))
("vasko" nil (v aa1 s k ow0))
("vaslov" nil (v ae1 s l aa1 v))
("vasotec" nil (v ey1 z ow0 t eh1 k))
("vasques" nil (v aa1 s k w eh0 s))
("vasquez" nil (v ae0 s k eh1 z))
("vass" nil (v ae1 s))
("vassal" nil (v ae1 s ax l))
("vassallo" nil (v aa0 s aa1 l ow0))
("vassals" nil (v ae1 s ax l z))
("vassar" nil (v ae1 s er0))
("vassel" nil (v ae1 s ax l))
("vasser" nil (v ae1 s er0))
("vasseur" nil (v ae1 s er0))
("vassey" nil (v ae1 s iy0))
("vassilios" nil (v ax s ih1 l y ax s))
("vassiliou" nil (v ae1 s ih1 l iy0 uw0))
("vast" nil (v ae1 s t))
("vasta" nil (v ae1 s t ax))
("vastine" nil (v aa0 s t iy1 n iy0))
("vastly" nil (v ae1 s t l iy0))
("vastness" nil (v ae1 s t n ax s))
("vastola" nil (v aa0 s t ow1 l ax))
("vat" nil (v ae1 t))
("vater" nil (v ey1 t er0))
("vath" nil (v ae1 th))
("vatican" nil (v ae1 t ih0 k ax n))
("vats" nil (v ae1 t s))
("vatted" nil (v ae1 t ih0 d))
("vatter" nil (v ae1 t er0))
("vaudeville" nil (v aa1 d v ih0 l))
("vaudevillian" nil (v aa0 d v ih1 l y ax n))
("vaugh" nil (v ao1))
("vaughan" nil (v ao1 n))
("vaughn" nil (v ao1 n))
("vaughns" nil (v ao1 n z))
("vaught" nil (v ao1 t))
("vault" nil (v ao1 l t))
("vaulted" nil (v ao1 l t ax d))
("vaulting" nil (v ao1 l t ih0 ng))
("vaults" nil (v ao1 l t s))
("vaunted" nil (v ao1 n t ih0 d))
("vaupel" nil (v aw0 p eh1 l))
("vause" nil (v ao1 s))
("vaux" nil (v ao1 k s))
("vauxhall" nil (v aa1 k s hh ao1 l))
("vavra" nil (v ae1 v r ax))
("vavrek" nil (v aa1 v r eh0 k))
("vawter" nil (v ao1 t er0))
("vax" nil (v ae1 k s))
("vaxes" nil (v ae1 k s ih0 z))
("vaxstation" nil (v ae1 k s t ey1 sh ax n))
("vayda" nil (v ey1 d ax))
("vaz" nil (v ae1 z))
("vazquez" nil (v ae0 s k eh1 z))
("veach" nil (v iy1 ch))
("veal" nil (v iy1 l))
("veale" nil (v iy1 l))
("veals" nil (v iy1 l z))
("veasey" nil (v iy1 z iy0))
("veasley" nil (v iy1 z l iy0))
("veatch" nil (v iy1 ch))
("veazey" nil (v iy1 z iy0))
("veazie" nil (v iy1 z iy0))
("veba" nil (v iy1 b ax))
("vecchiarelli" nil (v eh0 k iy0 aa0 r eh1 l iy0))
("vecchio" nil (v eh1 k iy0 ow0))
("vecchione" nil (v eh1 k iy0 ow1 n iy0))
("vecci" nil (v eh1 ch iy0))
("vecellio" nil (v eh0 ch eh1 l iy0 ow0))
("vector" nil (v eh1 k t er0))
("vectra" nil (v eh1 k t r er0))
("veda" nil (v ey1 d ax))
("vedder" nil (v eh1 d er0))
("vedette" nil (v ih0 d eh1 t))
("vedis" nil (v ey1 d ih0 s))
("vee" nil (v iy1))
("veeco" nil (v iy1 k ow0))
("veeder" nil (v iy1 d er0))
("veen" nil (v iy1 n))
("veenstra" nil (v iy1 n s t r ax))
("veer" nil (v ih1 r))
("veered" nil (v ih1 r d))
("veering" nil (v ih1 r ih0 ng))
("veers" nil (v ih1 r z))
("vees" nil (v iy1 z))
("vega" nil (v ey1 g ax))
("vegas" nil (v ey1 g ax s))
("vegesna" nil (v eh0 g eh1 s n ax))
("vegetable" nil (v eh1 jh t ax b ax l))
("vegetables" nil (v eh1 jh t ax b ax l z))
("vegetal" nil (v eh1 jh ax t ax l))
("vegetarian" nil (v eh1 jh ax t eh1 r iy0 ax n))
("vegetarianism" nil (v eh1 jh ax t eh1 r iy0 ax n ih0 z ax m))
("vegetarians" nil (v eh1 jh ax t eh1 r iy0 ax n z))
("vegetation" nil (v eh1 jh ax t ey1 sh ax n))
("vegh" nil (v eh1 g))
("vegisnax" nil (v ax g ih1 s n ae0 k s))
("vehemence" nil (v iy1 ax m ax n s))
("vehement" nil (v iy1 ax m ax n t))
("vehemently" nil (v iy1 ax m ax n t l iy0))
("vehicle" nil (v iy1 hh ih0 k ax l))
("vehicles" nil (v iy1 hh ih0 k ax l z))
("vehicular" nil (v iy0 hh ih1 k y ax l er0))
("veiga" nil (v ey1 g ax))
("veigel" nil (v ay1 g ax l))
("veil" nil (v ey1 l))
("veiled" nil (v ey1 l d))
("veiling" nil (v ey1 l ih0 ng))
("veillette" nil (v ax l eh1 t))
("veilleux" nil (v ax l ow1))
("veillon" nil (v ey1 l ax n))
("veils" nil (v ey1 l z))
("vein" nil (v ey1 n))
("veins" nil (v ey1 n z))
("veit" nil (v iy1 t))
("veitch" nil (v ay1 ch))
("veith" nil (v iy1 th))
("vela" nil (v eh1 l ax))
("velagrande" nil (v eh1 l ax g r aa1 n d ey1))
("velarde" nil (v eh0 l aa1 r d iy0))
("velardi" nil (v eh0 l aa1 r d iy0))
("velardo" nil (v ey0 l aa1 r d ow0))
("velasco" nil (v eh0 l aa1 s k ow0))
("velayati" nil (v eh0 l ay1 aa1 t iy0))
("velazco" nil (v ax l ae1 s k ow0))
("velazquez" nil (v eh0 l ae1 s k eh0 z))
("velcro" nil (v eh1 l k r ow0))
("velda" nil (v eh1 l d ax))
("veldhuizen" nil (v eh1 l d hh ih0 z ax n))
("veldman" nil (v eh1 l d m ax n))
("veley" nil (v eh1 l iy0))
("velez" nil (v eh0 l eh1 z))
("velie" nil (v eh1 l iy0))
("velika" nil (v eh1 l ih0 k ax))
("veliotis" nil (v eh0 l iy0 ow1 t ih0 s))
("veliz" nil (v eh1 l ih0 z))
("vella" nil (v eh1 l ax))
("vellucci" nil (v eh0 l uw1 ch iy0))
("velma" nil (v eh1 l m ax))
("velo" nil (v eh1 l ow0))
("velobind" nil (v eh1 l ax b ih0 n d))
("velocities" nil (v ax l aa1 s ax t iy0 z))
("velocity" nil (v ax l aa1 s ax t iy0))
("veloso" nil (v eh1 l ow1 s ow0))
("velotta" nil (v eh0 l ow1 t ax))
("veloz" nil (v ey1 l ow0 z))
("velsicol" nil (v eh1 l s ih0 k aa1 l))
("velte" nil (v eh1 l t))
("velten" nil (v eh1 l t ax n))
("veltman" nil (v eh1 l t m ax n))
("veltre" nil (v eh1 l t er0))
("veltri" nil (v eh1 l t r iy0))
("velveeta" nil (v eh0 l v iy1 t ax))
("velvet" nil (v eh1 l v ax t))
("velvety" nil (v eh1 l v ax t iy0))
("vempala" nil (v eh1 m p aa1 l ax))
("vena" nil (v iy1 n ax))
("venable" nil (v eh1 n ax b ax l))
("venables" nil (v eh1 n ax b ax l z))
("venal" nil (v iy1 n ax l))
("venality" nil (v ih0 n ae1 l ih0 t iy0))
("venalum" nil (v eh1 n ax l ax m))
("venango" nil (v eh0 n ae1 ng g ow0))
("venard" nil (v eh1 n er0 d))
("vencill" nil (v eh1 n s ih0 l))
("vencor" nil (v eh1 n k ao1 r))
("vendee" nil (v eh1 n d iy1))
("vendetta" nil (v eh0 n d eh1 t ax))
("vendetti" nil (v eh0 n d eh1 t iy0))
("vending" nil (v eh1 n d ih0 ng))
("venditti" nil (v eh0 n d iy1 t iy0))
("vendo" nil (v eh1 n d ow0))
("vendome" nil (v eh0 n d ow1 m))
("vendor" nil (v eh1 n d er0))
("vendors" nil (v eh1 n d er0 z))
("veneer" nil (v ax n ih1 r))
("venegas" nil (v eh1 n ih0 g ax z))
("venema" nil (v eh1 n ih0 m ax))
("venerable" nil (v eh1 n er0 ax b ax l))
("venerated" nil (v eh1 n er0 ey1 t ih0 d))
("venereal" nil (v ax n ih1 r iy0 ax l))
("venetian" nil (v ax n iy1 sh ax n))
("veney" nil (v eh1 n iy0))
("venezia" nil (v eh0 n eh1 z iy0 ax))
("veneziano" nil (v eh0 n eh0 z iy0 aa1 n ow0))
("venezuela" nil (v eh1 n ih0 z w ey1 l ax))
("venezuelan" nil (v eh1 n ih0 z w ey1 l ax n))
("venezuelans" nil (v eh1 n ih0 z w ey1 l ax n z))
("vengeance" nil (v eh1 n jh ax n s))
("vengeful" nil (v eh1 n jh f ax l))
("venice" nil (v eh1 n ax s))
("venier" nil (v iy1 n iy0 er0))
("venison" nil (v eh1 n ax s ax n))
("venita" nil (v eh0 n iy1 t ax))
("venkatesh" nil (v eh1 n k aa0 t eh1 sh))
("venn" nil (v eh1 n))
("vennard" nil (v eh1 n er0 d))
("venne" nil (v eh1 n))
("venneman" nil (v eh1 n m ax n))
("venner" nil (v eh1 n er0))
("venning" nil (v eh1 n ih0 ng))
("veno" nil (v ey1 n ow0))
("venom" nil (v eh1 n ax m))
("venomous" nil (v eh1 n ax m ax s))
("venous" nil (v iy1 n ax s))
("vensel" nil (v eh1 n s ax l))
("venson" nil (v eh1 n s ax n))
("vent" nil (v eh1 n t))
("ventech" nil (v eh1 n t eh1 k))
("vented" nil (v eh1 n t ih0 d))
("venter" nil (v eh1 n t er0))
("venters" nil (v eh1 n t er0 z))
("venti" nil (v eh1 n t iy0))
("ventilated" nil (v eh1 n t ax l ey1 t ih0 d))
("ventilating" nil (v eh1 n t ax l ey1 t ih0 ng))
("ventilation" nil (v eh1 n t ax l ey1 sh ax n))
("ventilator" nil (v eh1 n t ax l ey1 t er0))
("ventimiglia" nil (v eh1 n t ih0 m ih1 g l iy0 ax))
("venting" nil (v eh1 n t ih0 ng))
("vento" nil (v eh1 n t ow0))
("ventola" nil (v eh0 n t ow1 l ax))
("ventral" nil (v eh1 n t r ax l))
("ventre" nil (v eh1 n t er0))
("ventrella" nil (v eh1 n t r eh1 l ax))
("ventres" nil (v eh1 n t er0 z))
("ventresca" nil (v eh0 n t r eh1 s k ax))
("ventress" nil (v eh1 n t r ih0 s))
("ventricular" nil (v eh0 n t r ih1 k y uw0 l er0))
("ventritex" nil (v eh1 n t r ih0 t eh1 k s))
("vents" nil (v eh1 n t s))
("ventura" nil (v eh0 n ch er1 ax))
("venture" nil (v eh1 n ch er0))
("ventured" nil (v eh1 n ch er0 d))
("venturella" nil (v eh0 n t uh0 r eh1 l ax))
("ventures" nil (v eh1 n ch er0 z))
("venturesome" nil (v eh1 n ch er0 s ax m))
("venturi" nil (v eh0 n t uh1 r iy0))
("venturian" nil (v eh0 n t uh1 r iy0 ax n))
("venturing" nil (v eh1 n ch er0 ih0 ng))
("venturini" nil (v eh0 n t uh0 r iy1 n iy0))
("venturino" nil (v eh0 n t uh0 r iy1 n ow0))
("venue" nil (v eh1 n y uw0))
("venues" nil (v eh1 n uw0 z))
("venus" nil (v iy1 n ax s))
("venuti" nil (v eh0 n uw1 t iy0))
("venuto" nil (v eh0 n uw1 t ow0))
("venzke" nil (v eh1 n z k))
("vera" nil (v eh1 r ax))
("veracity" nil (v er0 ae1 s ih0 t iy0))
("veranda" nil (v er0 ae1 n d ax))
("verandas" nil (v er0 ae1 n d ax z))
("veras" nil (v eh1 r ax z))
("verb" nil (v er1 b))
("verba" nil (v eh1 r b ax))
("verbal" nil (v er1 b ax l))
("verbalize" nil (v er1 b ax l ay1 z))
("verbalizing" nil (v er1 b ax l ay1 z ih0 ng))
("verbally" nil (v er0 b ae1 l iy0))
("verbatim" nil (v er0 b ey1 t ax m))
("verbeck" nil (v er1 b eh0 k))
("verbeek" nil (v er1 b iy0 k))
("verbeke" nil (v er1 b ih0 k))
("verbena" nil (v er0 b iy1 n ax))
("verbiage" nil (v er1 b iy0 ih0 jh))
("verble" nil (v er1 b ax l))
("verboten" nil (v er0 b ow1 t ax n))
("verbrugge" nil (v er1 b r ax g))
("verbs" nil (v er1 b z))
("verburg" nil (v er1 b er0 g))
("vercammen" nil (v er1 k ae1 m ax n))
("vercher" nil (v er1 k er0))
("verda" nil (v eh1 r d ax))
("verdant" nil (v er1 d ax n t))
("verde" nil (v er1 d iy0))
("verderame" nil (v er1 d er0 ax m))
("verderber" nil (v er1 d er0 b er0))
("verderosa" nil (v er0 d er0 ow1 s ax))
("verdes" nil (v er1 d iy0))
("verdi" nil (v eh1 r d iy0))
("verdict" nil (v er1 d ih0 k t))
("verdicts" nil (v er1 d ih0 k t s))
("verdier" nil (v er1 d iy0 er0))
("verdin" nil (v er1 d ih0 n))
("verdon" nil (v eh0 r d ao1 n))
("verdone" nil (v eh0 r d ow1 n ey0))
("verdugo" nil (v er0 d uw1 g ow0))
("verduin" nil (v eh0 r d uw0 ih1 n))
("verdun" nil (v er1 d ax n))
("verduzco" nil (v er0 d uw1 z k ow0))
("vere" nil (v ih1 r))
("vereb" nil (v eh1 r ih0 b))
("vereen" nil (v ih1 r iy0 n))
("vereinsbank" nil (v er0 ay1 n z b ae1 ng k))
("verena" nil (v er0 ey1 n ax))
("verene" nil (v eh1 r iy0 n))
("veres" nil (v iy1 r z))
("verex" nil (v eh1 r ax k s))
("verga" nil (v eh1 r g ax))
("vergara" nil (v er0 g aa1 r ax))
("verge" nil (v er1 jh))
("verges" nil (v er1 jh ih0 z))
("vergesh" nil (v er1 g eh1 sh))
("vergil" nil (v er1 jh ax l))
("verging" nil (v er1 jh ih0 ng))
("vergresh" nil (v er1 g r eh1 sh))
("verhage" nil (v er1 hh ih0 jh))
("verhagen" nil (v er1 hh ax g ax n))
("verhey" nil (v er1 hh iy0))
("verhoef" nil (v er1 hh ow0 f))
("verhoeven" nil (v er1 hh ow0 v ax n))
("verhoff" nil (v er1 hh ao0 f))
("verhofstadt" nil (v er0 hh ao1 f s t ae1 t))
("verhulst" nil (v er1 hh ax l s t))
("veribanc" nil (v eh1 r ih0 b ae1 ng k))
("verifiable" nil (v eh1 r ax f ay1 ax b ax l))
("verification" nil (v eh1 r ax f ax k ey1 sh ax n))
("verified" nil (v eh1 r ax f ay1 d))
("verifies" nil (v eh1 r ax f ay1 z))
("verify" nil (v eh1 r ax f ay1))
("verifying" nil (v eh1 r ax f ay1 ih0 ng))
("verina" nil (v er0 iy1 n ax))
("verine" nil (v er0 iy1 n iy0))
("verisimilitude" nil (v eh1 r ax s ax m ih1 l ax t uw1 d))
("verit" nil (v eh1 r ih0 t))
("veritable" nil (v eh1 r ih0 t ax b ax l))
("verities" nil (v eh1 r ax t iy0 z))
("verity" nil (v eh1 r ax t iy0))
("verkuilen" nil (v er1 k ax l ax n))
("verla" nil (v eh1 r l ax))
("verlag" nil (v er1 l ae0 g))
("verleger" nil (v er1 l ax g er0))
("verley" nil (v er1 l iy0))
("verma" nil (v eh1 r m ax))
("vermeer" nil (v er1 m ih0 r))
("vermette" nil (v er0 m eh1 t))
("vermeulen" nil (v er0 m oy1 l ax n))
("vermilion" nil (v er0 m ih1 l y ax n))
("vermillion" nil (v er0 m ih1 l y ax n))
("vermilya" nil (v er0 m iy1 l y ax))
("vermilyea" nil (v er0 m iy1 l y ax))
("vermin" nil (v er1 m ih0 n))
("vermont" nil (v er0 m aa1 n t))
("vermouth" nil (v er0 m uw1 th))
("vern" nil (v er1 n))
("verna" nil (v er1 n ax))
("vernacular" nil (v er0 n ae1 k y ax l er0))
("vernal" nil (v er1 n ax l))
("verne" nil (v er1 n))
("verner" nil (v er1 n er0))
("vernes" nil (v er1 n z))
("verneta" nil (v er0 n eh1 t ax))
("verney" nil (v er1 n iy0))
("vernick" nil (v er1 n ih0 k))
("vernier" nil (v er1 n iy0 er0))
("vernis" nil (v er1 n ih0 s))
("vernita" nil (v er0 n iy1 t ax))
("vernitron" nil (v er1 n ih0 t r aa0 n))
("verno" nil (v eh1 r n ow0))
("vernon" nil (v er1 n ax n))
("vernor" nil (v er1 n er0))
("vero" nil (v eh1 r ow0))
("veron" nil (v eh1 r ax n))
("verona" nil (v er0 ow1 n ax))
("veronda" nil (v eh0 r aa1 n d ax))
("veronica" nil (v er0 aa1 n ih0 k ax))
("veronique" nil (v eh1 r aa0 n iy1 k))
("veronis" nil (v eh0 r ow1 n ih0 s))
("verrall" nil (v eh1 r ax l))
("verrastro" nil (v eh1 r ae1 s t r ow0))
("verrell" nil (v eh0 r ey1 l))
("verret" nil (v eh1 r ey0 t))
("verrett" nil (v eh1 r ih0 t))
("verrette" nil (v er0 eh1 t))
("verri" nil (v eh1 r iy0))
("verrier" nil (v eh1 r iy0 er0))
("verrill" nil (v eh0 r iy1 l))
("verrilli" nil (v er0 iy1 l iy0))
("verry" nil (v eh1 r iy0))
("versa" nil (v er1 s ax))
("versace" nil (v er0 s aa1 s))
("versailles" nil (v eh0 r s ay1))
("versatile" nil (v er1 s ax t ax l))
("versatility" nil (v er1 s ax t ih1 l ax t iy0))
("verse" nil (v er1 s))
("versed" nil (v er1 s t))
("verser" nil (v er1 s er0))
("verses" nil (v er1 s ax z))
("versicherung" nil (v er0 s ih1 ch er0 ax ng))
("versicherungs" nil (v er0 s ih1 ch er0 ax ng z))
("version" nil (v er1 zh ax n))
("versions" nil (v er1 zh ax n z))
("versluis" nil (v er1 s l uw0 ih0 z))
("versteeg" nil (v er1 s t iy0 g))
("verstraete" nil (v er1 s t r eh0 t))
("versus" nil (v er1 s ax s))
("vert" nil (v er1 t))
("vertebral" nil (v er1 t ax b r ax l))
("vertebrates" nil (v er1 t ax b r ey1 t s))
("vertel" nil (v er1 t eh1 l))
("vertex" nil (v er1 t eh1 k s))
("vertical" nil (v er1 t ih0 k ax l))
("vertically" nil (v er1 t ih0 k l iy0))
("verticom" nil (v er1 t ih0 k aa0 m))
("vertigo" nil (v er1 t ih0 g ow1))
("vertol" nil (v er1 t aa0 l))
("vertrees" nil (v er0 t r iy1 z))
("vertucci" nil (v er0 t uw1 ch iy0))
("vertz" nil (v er1 t s))
("verve" nil (v er1 v))
("verville" nil (v er1 v ih0 l))
("verwey" nil (v er1 w iy0))
("verwoerd" nil (v er0 w ao1 r d))
("very" nil (v eh1 r iy0))
("vescio" nil (v eh1 s iy0 ow0))
("vesco" nil (v eh1 s k ow0))
("vesel" nil (v eh1 s ax l))
("veselka" nil (v ey0 s ey1 l k ax))
("vesely" nil (v iy1 z l iy0))
("vesey" nil (v eh1 s iy0))
("vesmen" nil (v eh1 s m ax n))
("vespa" nil (v ey1 s p ax))
("vesper" nil (v eh1 s p er0))
("vespera" nil (v ey0 s p eh1 r ax))
("vess" nil (v eh1 s))
("vessel" nil (v eh1 s ax l))
("vessell" nil (v eh1 s ax l))
("vessels" nil (v eh1 s ax l z))
("vesser" nil (v eh1 s er0))
("vessey" nil (v eh1 s iy0))
("vest" nil (v eh1 s t))
("vesta" nil (v eh1 s t ax))
("vestal" nil (v eh1 s t ax l))
("vestar" nil (v eh1 s t er0))
("vested" nil (v eh1 s t ax d))
("vester" nil (v eh1 s t er0))
("vestige" nil (v eh1 s t ih0 jh))
("vestiges" nil (v eh1 s t ih0 jh ih0 z))
("vestigial" nil (v ax s t ih1 jh iy0 ax l))
("vesting" nil (v eh1 s t ih0 ng))
("vestron" nil (v eh1 s t r ax n))
("vests" nil (v eh1 s t s))
("vesuvius" nil (v ax s uw1 v iy0 ax s))
("vet" nil (v eh1 t))
("veta" nil (v eh1 t ax))
("vetco" nil (v eh1 t k ow0))
("veteran" nil (v eh1 t er0 ax n))
("veterans" nil (v eh1 t er0 ax n z))
("vetere" nil (v eh1 t er0))
("veterinarian" nil (v eh1 t r ax n eh1 r iy0 ax n))
("veterinarians" nil (v eh1 t r ax n eh1 r iy0 ax n z))
("veterinary" nil (v eh1 t r ax n eh1 r iy0))
("veteto" nil (v eh0 t eh1 t ow0))
("veto" nil (v iy1 t ow0))
("vetoed" nil (v iy1 t ow0 d))
("vetoes" nil (v iy1 t ow0 z))
("vetoing" nil (v iy1 t ow0 ih0 ng))
("vetos" nil (v iy1 t ow0 z))
("vetrano" nil (v eh0 t r aa1 n ow0))
("vets" nil (v eh1 t s))
("vetsch" nil (v eh1 ch))
("vette" nil (v eh1 t))
("vetted" nil (v eh1 t ih0 d))
("vetter" nil (v eh1 t er0))
("veuve" nil (v y uw1 v))
("vevay" nil (v eh1 v ey0))
("veverka" nil (v ey0 v eh1 r k ax))
("vevey" nil (v eh1 v iy0))
("vevila" nil (v ey0 v iy1 l ax))
("vex" nil (v eh1 k s))
("vexatious" nil (v eh0 k s ey1 sh ax s))
("vexed" nil (v eh1 k s t))
("vexing" nil (v eh1 k s ih0 ng))
("vey" nil (v ey1))
("veyne" nil (v ey1 n))
("vezina" nil (v eh0 z iy1 n ax))
("via" nil (v ay1 ax))
("viability" nil (v ay0 ax b ih1 l ax t iy0))
("viable" nil (v ay1 ax b ax l))
("viacom" nil (v ay1 ax k aa0 m))
("viaducts" nil (v ay1 ax d ax k t s))
("viag" nil (v ay1 ae0 g))
("viagene" nil (v ay1 ax jh iy1 n))
("vial" nil (v ay1 ax l))
("viall" nil (v ay1 ax l))
("vialpando" nil (v iy0 aa0 l p aa1 n d ow0))
("vials" nil (v ay1 ax l z))
("vian" nil (v iy1 ax n))
("viana" nil (v iy0 ae1 n ax))
("viani" nil (v iy0 aa1 n iy0))
("viano" nil (v iy0 aa1 n ow0))
("viar" nil (v ay1 er0))
("viars" nil (v ay1 er0 z))
("viatical" nil (v ay1 ae1 t ih0 k ax l))
("viator" nil (v ay0 ey1 t er0))
("viau" nil (v iy0 ow1))
("vibbert" nil (v ih1 b er0 t))
("vibes" nil (v ay1 b z))
("vibrancy" nil (v ay1 b r ax n s iy0))
("vibrant" nil (v ay1 b r ax n t))
("vibrate" nil (v ay1 b r ey0 t))
("vibrating" nil (v ay1 b r ey0 t ih0 ng))
("vibration" nil (v ay0 b r ey1 sh ax n))
("vibrations" nil (v ay0 b r ey1 sh ax n z))
("vibrato" nil (v iy0 b r aa1 t ow0))
("vic" nil (v ih1 k))
("vicar" nil (v ih1 k er0))
("vicari" nil (v iy0 k aa1 r iy0))
("vicario" nil (v iy0 k aa1 r iy0 ow0))
("vicarious" nil (v ay0 k eh1 r iy0 ax s))
("vicariously" nil (v ay0 k eh1 r iy0 ax s l iy0))
("vicars" nil (v ih1 k er0 z))
("vice" nil (v ay1 s))
("vicencio" nil (v ih0 s eh1 n s iy0 ow0))
("vicente" nil (v iy0 ch eh1 n t iy0))
("vicenzi" nil (v ih0 s eh1 n z iy0))
("viceroy" nil (v ay1 s r oy0))
("vices" nil (v ay1 s ax z))
("vichy" nil (v iy1 sh iy0))
("vicinity" nil (v ax s ih1 n ax t iy0))
("vicino" nil (v iy0 ch iy1 n ow0))
("vicious" nil (v ih1 sh ax s))
("viciously" nil (v ih1 sh ax s l iy0))
("vicissitudes" nil (v ih0 s ih1 s ih0 t uw0 d z))
("vick" nil (v ih1 k))
("vicker" nil (v ih1 k er0))
("vickerman" nil (v ih1 k er0 m ax n))
("vickers" nil (v ih1 k er0 z))
("vickery" nil (v ih0 s k er1 iy0))
("vickey" nil (v ih1 k iy0))
("vicki" nil (v ih1 k iy0))
("vickie" nil (v ih1 k iy0))
("vicknair" nil (v ih1 k n er0))
("vickrey" nil (v ih1 k r iy0))
("vickroy" nil (v ih1 k r oy1))
("vicks" nil (v ih1 k s))
("vicksburg" nil (v ih1 k s b er0 g))
("vicky" nil (v ih1 k iy0))
("vicomte" nil (v ih0 k aa1 m t iy0))
("vicon" nil (v ih1 k ax n))
("vicorp" nil (v ay1 k ao0 r p))
("victim" nil (v ih1 k t ax m))
("victimization" nil (v ih1 k t ax m ax z ey1 sh ax n))
("victimized" nil (v ih1 k t ax m ay0 z d))
("victimless" nil (v ih1 k t ax m l ax s))
("victims" nil (v ih1 k t ax m z))
("victoire" nil (v ih0 k t w aa1 r))
("victor" nil (v ih1 k t er0))
("victoria" nil (v ih0 k t ao1 r iy0 ax))
("victorian" nil (v ih0 k t ao1 r iy0 ax n))
("victorians" nil (v ih0 k t ao1 r iy0 ax n z))
("victorias" nil (v ih0 k t ao1 r iy0 ax z))
("victories" nil (v ih1 k t er0 iy0 z))
("victorine" nil (v iy0 k t ao0 r iy1 n iy0))
("victorino" nil (v iy0 k t ao0 r iy1 n ow0))
("victorious" nil (v ih0 k t ao1 r iy0 ax s))
("victors" nil (v ih1 k t er0 z))
("victory" nil (v ih1 k t er0 iy0))
("vicuna" nil (v ih0 k y uw1 n ax))
("vida" nil (v ay1 d ax))
("vidal" nil (v ay0 d ae1 l))
("vidales" nil (v iy0 d aa1 l eh0 s))
("vidalia" nil (v ih0 d ey1 l y ax))
("vidaurri" nil (v ih0 d ao1 r iy0))
("vide" nil (v ay1 d))
("video" nil (v ih1 d iy0 ow0))
("videocassette" nil (v ih1 d iy0 ow0 k ae0 s eh1 t))
("videocassettes" nil (v ih1 d iy0 ow0 k ae0 s eh1 t s))
("videoconferencing" nil (v iy1 d iy0 ow0 k aa1 n f r ax n s ih0 ng))
("videodisk" nil (v ih1 d iy0 ow0 d ih1 s k))
("videodisks" nil (v ih1 d iy0 ow0 d ih1 s k s))
("videogame" nil (v ih1 d iy0 ow0 g ey1 m))
("videophone" nil (v ih1 d iy0 ow0 f ow1 n))
("videos" nil (v ih1 d iy0 ow0 z))
("videotape" nil (v ih1 d iy0 ow0 t ey1 p))
("videotaped" nil (v ih1 d iy0 ow0 t ey1 p t))
("videotapes" nil (v ih1 d iy0 ow0 t ey1 p s))
("videotaping" nil (v ih1 d iy0 ow0 t ey1 p ih0 ng))
("videotex" nil (v ih1 d iy0 ow0 t eh1 k s))
("videotext" nil (v ih1 d iy0 ow0 t eh1 k s t))
("videotron" nil (v ih1 d iy0 ow0 t r aa1 n))
("vidigo" nil (v ih1 d iy0 g ow1))
("vidmar" nil (v ih1 d m er0))
("vidonia" nil (v iy0 d ow1 n iy0 ax))
("vidrine" nil (v iy0 d r iy1 n))
("vie" nil (v ay1))
("vieau" nil (v iy0 ow1))
("viebrock" nil (v iy1 b r ax k))
("vied" nil (v ay1 d))
("viedma" nil (v iy1 d m ax))
("viegas" nil (v iy1 g ax z))
("vieira" nil (v iy0 eh1 r ax))
("viejo" nil (v iy0 eh1 jh ow0))
("viel" nil (v iy1 l))
("viele" nil (v iy1 l))
("vielma" nil (v iy1 l m ax))
("vien" nil (v iy1 n))
("vienna" nil (v iy0 eh1 n ax))
("vienneau" nil (v ax n ow1))
("viennese" nil (v iy0 eh1 n iy1 s))
("vienot" nil (v iy1 n aa0 t))
("viens" nil (v iy1 n z))
("viera" nil (v iy1 r ax))
("viereck" nil (v ih1 r ih0 k))
("vierling" nil (v ih1 r l ih0 ng))
("vierra" nil (v ih1 r ax))
("viers" nil (v ih1 r z))
("viertel" nil (v ih1 r t eh0 l))
("vies" nil (v ay1 z))
("viet" nil (v iy0 eh1 t))
("vietcong" nil (v iy0 eh1 t k ao1 ng))
("vieth" nil (v ay1 ih0 th))
("vietnam" nil (v iy0 eh1 t n aa1 m))
("vietnamese" nil (v iy0 eh0 t n aa0 m iy1 s))
("vietor" nil (v iy1 t er0))
("viets" nil (v iy1 t s))
("vietti" nil (v iy0 eh1 t iy0))
("view" nil (v y uw1))
("viewed" nil (v y uw1 d))
("viewer" nil (v y uw1 er0))
("viewers" nil (v y uw1 er0 z))
("viewership" nil (v y uw1 er0 sh ih1 p))
("viewing" nil (v y uw1 ih0 ng))
("viewpoint" nil (v y uw1 p oy1 n t))
("viewpoints" nil (v y uw1 p oy1 n t s))
("views" nil (v y uw1 z))
("vieyra" nil (v iy0 eh1 r ax))
("vig" nil (v ih1 g))
("vigdor" nil (v ih1 g d er0))
("vigeant" nil (v ih1 jh ax n t))
("vigeland" nil (v ay1 g l ax n d))
("vigen" nil (v ih1 g ax n))
("viger" nil (v ay1 g er0))
("viggiano" nil (v ih0 jh iy0 aa1 n ow0))
("vigil" nil (v ih1 jh ax l))
("vigilance" nil (v ih1 jh ax l ax n s))
("vigilant" nil (v ih1 jh ax l ax n t))
("vigilante" nil (v ih1 jh ax l ae1 n t iy0))
("vigilantes" nil (v ih1 jh ax l ae1 n t iy0 z))
("vigilantism" nil (v ih1 jh ax l ae1 n t ih0 z ax m))
("vigilia" nil (v iy0 jh iy1 l iy0 ax))
("viglione" nil (v ih0 g l iy0 ow1 n iy0))
("vigliotti" nil (v ih0 g l iy0 ao1 t iy0))
("viglund" nil (v ih1 g l ax n d))
("vigna" nil (v ih1 g n ax))
("vigneau" nil (v ih0 g n ow1))
("vigneault" nil (v ih0 g n ow1))
("vignette" nil (v ih0 n y eh1 t))
("vignettes" nil (v ih0 n y eh1 t s))
("vignola" nil (v iy0 g n ow1 l ax))
("vigo" nil (v iy1 g ow0))
("vigor" nil (v ih1 g er0))
("vigorito" nil (v iy0 g ao0 r iy1 t ow0))
("vigorous" nil (v ih1 g er0 ax s))
("vigorously" nil (v ih1 g er0 ax s l iy0))
("vigue" nil (v iy1 g))
("viguerie" nil (v ih1 g er0 iy0))
("vigus" nil (v ay1 g ax s))
("vik" nil (v ih1 k))
("viking" nil (v ay1 k ih0 ng))
("vikings" nil (v ay1 k ih0 ng z))
("viktor" nil (v ih1 k t er0))
("vila" nil (v iy1 l ax))
("vilardo" nil (v iy0 l aa1 r d ow0))
("vilas" nil (v iy1 l aa0 z))
("vile" nil (v ay1 l))
("viles" nil (v ay1 l z))
("vilhauer" nil (v ih1 l hh aw0 er0))
("vilification" nil (v ih1 l ax f ax k ey1 sh ax n))
("vilified" nil (v ih1 l ax f ay0 d))
("vilify" nil (v ih1 l ih0 f ay1))
("villa" nil (v ih1 l ax))
("villafana" nil (v iy0 l aa0 f ae1 n ax))
("villafane" nil (v iy0 l aa0 f aa1 n ey0))
("villafranca" nil (v ih1 l ax f r ae1 ng k ax))
("villafuerte" nil (v iy0 l aa0 f w eh1 r t ey0))
("village" nil (v ih1 l ax jh))
("villager" nil (v ih1 l ih0 jh er0))
("villagers" nil (v ih1 l ih0 jh er0 z))
("villages" nil (v ih1 l ax jh ax z))
("villagomez" nil (v iy0 l aa0 g ow1 m eh0 z))
("villagran" nil (v iy0 l aa0 g r aa1 n))
("villagrana" nil (v iy0 l aa0 g r ae1 n ax))
("villain" nil (v ih1 l ax n))
("villainous" nil (v ih1 l ax n ax s))
("villains" nil (v ih1 l ax n z))
("villalba" nil (v iy0 l aa1 l b ax))
("villalobos" nil (v iy0 l aa0 l ow1 b ow0 z))
("villalon" nil (v iy0 l aa0 l ao1 n))
("villalovos" nil (v iy0 l aa0 l ow1 v ow0 z))
("villalpando" nil (v iy0 l aa0 l p aa1 n d ow0))
("villalva" nil (v iy0 l aa1 l v ax))
("villamil" nil (v iy0 l aa0 m iy1 l))
("villani" nil (v iy0 l aa1 n iy0))
("villano" nil (v iy0 l aa1 n ow0))
("villanova" nil (v ih1 l ax n ow1 v ax))
("villanueva" nil (v iy0 l aa0 n w eh1 v ax))
("villar" nil (v ih1 l er0))
("villard" nil (v ih1 l er0 d))
("villareal" nil (v ih0 l er0 iy0 aa1 l))
("villari" nil (v iy0 l aa1 r iy0))
("villarreal" nil (v iy0 l aa0 r iy1 l))
("villars" nil (v ih1 l er0 z))
("villas" nil (v ih1 l ax z))
("villasana" nil (v iy0 l aa0 s ae1 n ax))
("villasenor" nil (v iy0 l aa0 s ey0 ax n ao1 r))
("villaverde" nil (v iy0 l aa0 v eh1 r d iy0))
("villavicencio" nil (v ih1 l ax v ih0 s eh1 n s iy0 ow0))
("ville" nil (v ih1 l))
("villegas" nil (v ih1 l ih0 g ax z))
("villela" nil (v ih1 l ih0 l ax))
("villella" nil (v iy0 l eh1 l ax))
("villeneuve" nil (v ih1 l ih0 n uw0 v))
("villers" nil (v ih1 l er0 z))
("villette" nil (v ih0 l eh1 t))
("villiers" nil (v ih1 l y er0 z))
("villines" nil (v ih1 l ay1 n z))
("villwock" nil (v ih1 l w ax k))
("vilma" nil (v ih1 l m ax))
("vilnius" nil (v ih1 l n iy0 ax s))
("viloria" nil (v iy0 l ao1 r iy0 ax))
("vim" nil (v ih1 m))
("vin" nil (v ih1 n))
("vina" nil (v iy1 n ax))
("vinal" nil (v ay1 n ax l))
("vinca" nil (v ih1 ng k ax))
("vince" nil (v ih1 n s))
("vincelette" nil (v ih1 n s ih0 l eh0 t))
("vincennes" nil (v ih1 n s eh1 n ax s))
("vincent" nil (v ih1 n s ax n t))
("vincente" nil (v ih0 n ch eh1 n t iy0))
("vincenti" nil (v iy0 n ch eh1 n t iy0))
("vincentia" nil (v iy0 n ch eh1 n sh ax))
("vincenzo" nil (v ih0 n s eh1 n z ow0))
("vinci" nil (v ih1 n s iy0))
("vinciguerra" nil (v iy0 n ch iy0 g eh1 r ax))
("vindicate" nil (v ih1 n d ax k ey0 t))
("vindicated" nil (v ih1 n d ax k ey0 t ih0 d))
("vindicates" nil (v ih1 n d ih0 k ey1 t s))
("vindication" nil (v ih0 n d ax k ey1 sh ax n))
("vindictive" nil (v ih0 n d ih1 k t ih0 v))
("vindictiveness" nil (v ih0 n d ih1 k t ih0 v n ax s))
("vine" nil (v ay1 n))
("vinegar" nil (v ih1 n ax g er0))
("vinegars" nil (v ih1 n ih0 g er0 z))
("vineland" nil (v ay1 n l ax n d))
("viner" nil (v ay1 n er0))
("vines" nil (v ay1 n z))
("vinet" nil (v ih1 n ih0 t))
("viney" nil (v ay1 n iy0))
("vineyard" nil (v ih1 n y er0 d))
("vineyards" nil (v ih1 n y er0 d z))
("vingmed" nil (v ih1 ng m eh0 d))
("vingo" nil (v ih1 ng g ow0))
("vinicio" nil (v ih0 n ih1 s iy0 ow0))
("vinik" nil (v ih1 n ih0 k))
("vining" nil (v ay1 n ih0 ng))
("vinita" nil (v iy0 n iy1 t ax))
("vinje" nil (v ih1 n jh))
("vink" nil (v ih1 ng k))
("vinnie" nil (v ih1 n iy0))
("vinny" nil (v ih1 n iy0))
("vinocur" nil (v ih1 n ax k er0))
("vinsant" nil (v iy1 n s ax n t))
("vinso" nil (v ih1 n s ow0))
("vinson" nil (v ih1 n s ax n))
("vint" nil (v ih1 n t))
("vintage" nil (v ih1 n t ih0 jh))
("vintages" nil (v ih1 n t ih0 jh ih0 z))
("vintner" nil (v ih1 n t n er0))
("vintners" nil (v ih1 n t n er0 z))
("vinton" nil (v ih1 n t ax n))
("vinyard" nil (v ih1 n y er0 d))
("vinyl" nil (v ay1 n ax l))
("vinzant" nil (v ih1 n z ax n t))
("viola" nil (v ay0 ow1 l ax))
("violante" nil (v iy0 ow0 l aa1 n t iy0))
("violas" nil (v iy0 ow1 l ax z))
("violate" nil (v ay1 ax l ey0 t))
("violated" nil (v ay1 ax l ey0 t ih0 d))
("violates" nil (v ay1 ax l ey0 t s))
("violating" nil (v ay1 ax l ey0 t ih0 ng))
("violation" nil (v ay0 ax l ey1 sh ax n))
("violations" nil (v ay0 ax l ey1 sh ax n z))
("violator" nil (v ay1 ax l ey1 t er0))
("violators" nil (v ay1 ax l ey0 t er0 z))
("viole" nil (v ay1 ow1 l))
("violence" nil (v ay1 ax l ax n s))
("violent" nil (v ay1 ax l ax n t))
("violently" nil (v ay1 ax l ax n t l iy0))
("violet" nil (v ay1 ax l ih0 t))
("violeta" nil (v iy1 ow0 l eh1 t ax))
("violets" nil (v ay1 ax l ax t s))
("violett" nil (v ay1 ax l ax t))
("violetta" nil (v iy0 ow0 l eh1 t ax))
("violette" nil (v iy1 ow0 l eh1 t))
("violin" nil (v ay0 ax l ih1 n))
("violinist" nil (v ay0 ax l ih1 n ax s t))
("violins" nil (v ay0 ax l ih1 n z))
("violist" nil (v iy1 ow1 l ax s t))
("viper" nil (v ay1 p er0))
("vipers" nil (v ay1 p er0 z))
("vipond" nil (v ih1 p ax n d))
("vipont" nil (v ih1 p aa1 n t))
("vipperman" nil (v ih1 p er0 m ax n))
("virag" nil (v ao1 r ax g))
("virago" nil (v ih1 r ax g ow0))
("viral" nil (v ay1 r ax l))
("viramontes" nil (v ih0 r aa0 m ow1 n t eh0 s))
("viratek" nil (v ih1 r ax t eh1 k))
("viray" nil (v ay1 r ey0))
("virazole" nil (v ih1 r ax z ow1 l))
("virco" nil (v er1 k ow0))
("virden" nil (v er1 d ax n))
("viren" nil (v ih1 r ax n))
("vires" nil (v ay1 r z))
("virga" nil (v er1 g ax))
("virge" nil (v er1 jh))
("virgen" nil (v er1 jh ax n))
("virgie" nil (v er1 g iy0))
("virgil" nil (v er1 jh ax l))
("virgilio" nil (v er0 jh ih1 l iy0 ow0))
("virgin" nil (v er1 jh ih0 n))
("virginal" nil (v er1 jh ax n ax l))
("virginia" nil (v er0 jh ih1 n y ax))
("virginian" nil (v er0 jh ih1 n y ax n))
("virginians" nil (v er0 jh ih1 n y ax n z))
("virginity" nil (v er0 jh ih1 n ih0 t iy0))
("virgins" nil (v er1 jh ax n z))
("virgo" nil (v er1 g ow0))
("virgy" nil (v er1 jh iy0))
("viridis" nil (v ao1 r ih0 d ih0 s))
("virile" nil (v ih1 r ax l))
("virility" nil (v er0 ih1 l ax t iy0))
("virina" nil (v ih0 r iy1 n ax))
("virkler" nil (v er1 k ax l er0))
("virna" nil (v er1 n ax))
("virnig" nil (v er1 n ih0 g))
("virological" nil (v ay0 r ax l aa1 jh ih0 k ax l))
("virology" nil (v ih0 r aa1 l ax jh iy0))
("virts" nil (v er1 t s))
("virtual" nil (v er1 ch uw0 ax l))
("virtually" nil (v er1 ch uw0 ax l iy0))
("virtue" nil (v er1 ch uw0))
("virtues" nil (v er1 ch uw0 z))
("virtuosity" nil (v er0 ch uw0 aa1 s ih0 t iy0))
("virtuoso" nil (v er0 ch uw0 ow1 s ow0))
("virtuous" nil (v er1 ch uw0 ax s))
("virulence" nil (v ih1 r ax l ax n s))
("virulent" nil (v ih1 r ax l ax n t))
("virulently" nil (v ih1 r ax l ax n t l iy0))
("virus" nil (v ay1 r ax s))
("viruses" nil (v ay1 r ax s ax z))
("virzi" nil (v ih1 r z iy0))
("vis" nil (v ih1 s))
("visa" nil (v iy1 z ax))
("visage" nil (v ih1 z ax jh))
("visas" nil (v iy1 z ax z))
("viscardi" nil (v iy0 s k aa1 r d iy0))
("visceral" nil (v ih1 s er0 ax l))
("viscerally" nil (v ih1 s er0 ax l iy0))
("visclosky" nil (v ih0 s k l ao1 s k iy0))
("visco" nil (v iy1 s k ow0))
("viscomi" nil (v iy0 s k ow1 m iy0))
("visconti" nil (v iy0 s k ow1 n t iy0))
("viscosity" nil (v ih0 s k aa1 s ax t iy0))
("viscous" nil (v ih1 s k ax s))
("viscusi" nil (v iy0 s k uw1 s iy0))
("viscuso" nil (v iy0 s k uw1 s ow0))
("vise" nil (v ay1 s))
("vishay" nil (v ih1 sh ey0))
("visibility" nil (v ih1 z ax b ih1 l ih0 t iy0))
("visible" nil (v ih1 z ax b ax l))
("visibly" nil (v ih1 z ax b l iy0))
("visicalc" nil (v ih1 z ih0 k ae1 l k))
("visigoths" nil (v ih1 z ax g aa0 th s))
("vision" nil (v ih1 zh ax n))
("visionaries" nil (v ih1 zh ax n eh0 r iy0 z))
("visionary" nil (v ih1 zh ax n eh1 r iy0))
("visions" nil (v ih1 zh ax n z))
("visit" nil (v ih1 z ax t))
("visitation" nil (v ih1 z ax t ey1 sh ax n))
("visited" nil (v ih1 z ax t ax d))
("visiting" nil (v ih1 z ax t ih0 ng))
("visitor" nil (v ih1 z ax t er0))
("visitors" nil (v ih1 z ax t er0 z))
("visits" nil (v ih1 z ax t s))
("visnic" nil (v ih1 s n ih0 ch))
("visors" nil (v ih1 z er0 z))
("visscher" nil (v ih1 sh er0))
("visser" nil (v ih1 s er0))
("vista" nil (v ih1 s t ax))
("vistas" nil (v ih1 s t ax z))
("visual" nil (v ih1 zh ax w ax l))
("visualization" nil (v ih1 zh w ax l ax z ey1 sh ax n))
("visualize" nil (v ih1 zh w ax l ay1 z))
("visualizing" nil (v ih1 zh w ax l ay1 z ih0 ng))
("visually" nil (v ih1 zh w ax l iy0))
("visuals" nil (v ih1 zh w ax l z))
("visx" nil (v ih1 s eh1 k s))
("vita" nil (v ay1 t ax))
("vitagliano" nil (v iy0 t aa0 g l iy0 aa1 n ow0))
("vital" nil (v ay1 t ax l))
("vitale" nil (v ih0 t ae1 l iy0))
("vitali" nil (v iy0 t aa1 l iy0))
("vitalink" nil (v ay1 t ax l ih1 ng k))
("vitality" nil (v ay0 t ae1 l ax t iy0))
("vitally" nil (v ay1 t ax l iy0))
("vitaly" nil (v ax t ae1 l iy0))
("vitamin" nil (v ay1 t ax m ax n))
("vitamins" nil (v ay1 t ax m ax n z))
("vitanza" nil (v ih0 t ae1 n z ax))
("vitarine" nil (v ih1 t er0 iy1 n))
("vitek" nil (v ih1 t eh0 k))
("vitelli" nil (v ih0 t eh1 l iy0))
("vitello" nil (v ih0 t eh1 l ow0))
("viti" nil (v iy1 t iy0))
("vitia" nil (v iy1 sh ax))
("vitiello" nil (v iy0 t iy0 eh1 l ow0))
("vitner" nil (v ih1 t n er0))
("vito" nil (v iy1 t ow0))
("vitolo" nil (v iy0 t ow1 l ow0))
("vitramon" nil (v ih1 t r ax m aa0 n))
("vitrano" nil (v iy0 t r aa1 n ow0))
("vitreous" nil (v ih1 t r iy0 ax s))
("vitric" nil (v ih1 t r ih0 k))
("vitriol" nil (v ih1 t r iy0 ax l))
("vitriolic" nil (v ih1 t r iy0 aa1 l ih0 k))
("vitro" nil (v ih1 t r ow0))
("vitronics" nil (v ay1 t r aa1 n ih0 k s))
("vitruvius" nil (v ih0 t r uw1 v iy0 ax s))
("vitry" nil (v ih1 t r iy0))
("vitt" nil (v ih1 t))
("vittetoe" nil (v ih1 t ih0 t ow0))
("vittitow" nil (v ih1 t ih0 t ow0))
("vittles" nil (v ih1 t ax l z))
("vittoria" nil (v ih0 t ao1 r iy0 ax))
("vittorio" nil (v iy0 t ao1 r iy0 ow0))
("vitucci" nil (v iy0 t uw1 ch iy0))
("vitulli" nil (v iy0 t uw1 l iy0))
("vitullo" nil (v iy0 t uw1 l ow0))
("viv" nil (v ih1 v))
("viva" nil (v iy1 v ax))
("vivacious" nil (v ax v ey1 sh ax s))
("vivaldi" nil (v ih0 v aa1 l d iy0))
("vivas" nil (v iy1 v ax z))
("viveiros" nil (v iy0 v ih1 r ow0 z))
("vivendi" nil (v ih0 v eh1 n d iy0))
("viverette" nil (v ih1 v er0 eh1 t))
("viveros" nil (v iy0 v eh1 r ow0 z))
("vives" nil (v ay1 v z))
("vivian" nil (v ih1 v iy0 ax n))
("viviana" nil (v ih0 v iy0 ae1 n ax))
("viviani" nil (v iy1 v iy0 aa1 n iy0))
("viviano" nil (v iy1 v iy0 aa1 n ow0))
("vivid" nil (v ih1 v ax d))
("vividly" nil (v ih1 v ax d l iy0))
("vividness" nil (v ih1 v ax d n ax s))
("vivie" nil (v ih1 v iy0))
("vivien" nil (v ih1 v iy0 ax n))
("vivienne" nil (v ih1 v iy0 ax n))
("vivier" nil (v ay1 v iy0 er0))
("viviparous" nil (v ay0 v ih1 p er0 ax s))
("vivo" nil (v iy1 v ow0))
("vivona" nil (v iy0 v ow1 n ax))
("vivra" nil (v iy1 v r ax))
("vivyan" nil (v iy0 v y aa1 n))
("vixen" nil (v ih1 k s ih0 n))
("viyella" nil (v ih0 y eh1 l ax))
("vizcaino" nil (v iy0 z k aa0 iy1 n ow0))
("vizcarra" nil (v iy0 z k aa1 r ax))
("vizcaya" nil (v ih0 z k ay1 ax))
("vizzini" nil (v iy0 t s iy1 n iy0))
("vlach" nil (v l ae1 ch))
("vlachos" nil (v l ey1 k ow0 s))
("vlad" nil (v l ae1 d))
("vladeck" nil (v l ae1 d eh0 k))
("vladimir" nil (v l ae1 d ax m ih0 r))
("vladislav" nil (v l ae1 d ax s l aa0 v))
("vladivostok" nil (v l ae1 d ih0 v ao1 s t aa0 k))
("vlahakis" nil (v l ah1 hh ao1 k ih0 s))
("vlahos" nil (v l aa1 hh ow0 s))
("vlasak" nil (v l aa1 s ax k))
("vlasic" nil (v l ae1 z ih0 k))
("vlcek" nil (v l eh1 s ih0 k))
("vliet" nil (v l iy1 t))
("vlok" nil (v l aa1 k))
("vnesheconombank" nil (v ax n eh1 sh ax k aa1 n ax m b ae1 ng k))
("vo" nil (v ow1))
("vobis" nil (v ow1 b ax s))
("vocabulary" nil (v ow0 k ae1 b y ax l eh1 r iy0))
("vocal" nil (v ow1 k ax l))
("vocalist" nil (v ow1 k ax l ih0 s t))
("vocally" nil (v ow1 k ax l iy0))
("vocals" nil (v ow1 k ax l z))
("vocaltec" nil (v ow1 k ax l t eh1 k))
("vocation" nil (v ow0 k ey1 sh ax n))
("vocational" nil (v ow0 k ey1 sh ax n ax l))
("vocations" nil (v ow0 k ey1 sh ax n z))
("vociferous" nil (v ow0 s ih1 f er0 ax s))
("vociferously" nil (v ax s ih1 f er0 ax s l iy0))
("vocke" nil (v aa1 k))
("vocs" nil (v aa1 k s))
("vodafone" nil (v ow1 d ax f ow1 n))
("vodavi" nil (v ow0 d aa1 v iy0))
("vodicka" nil (v aa1 d ih0 k ax))
("vodka" nil (v aa1 d k ax))
("vodkas" nil (v aa1 d k ax z))
("voegele" nil (v ow1 g ax l))
("voegeli" nil (v ow1 g ih0 l iy0))
("voelkel" nil (v ow1 l k ax l))
("voelker" nil (v ow1 l k er0))
("voell" nil (v ow1 l))
("voeller" nil (v ow1 l er0))
("voeltz" nil (v ow1 l t s))
("voelz" nil (v ow1 l z))
("voest" nil (v ow1 s t))
("vogan" nil (v ow1 g ax n))
("voge" nil (v ow1 jh))
("vogel" nil (v ow1 g ax l))
("vogeler" nil (v ow1 g ax l er0))
("vogelgesang" nil (v aa1 g ih0 l g ih0 s ax ng))
("vogelpohl" nil (v aa1 g ih0 l p ow0 l))
("vogels" nil (v ow1 g ax l z))
("vogelsang" nil (v aa1 g ih0 l s ax ng))
("vogelsong" nil (v aa1 g ih0 l s ao0 ng))
("vogelstein" nil (v ow1 g ax l s t iy1 n))
("voges" nil (v ow1 jh ih0 z))
("voght" nil (v aa1 t))
("vogl" nil (v aa1 g ax l))
("vogler" nil (v ow1 g l er0))
("vogt" nil (v ow1 t))
("vogtle" nil (v aa1 g t ax l))
("vogue" nil (v ow1 g))
("vohs" nil (v aa1 s))
("voice" nil (v oy1 s))
("voiced" nil (v oy1 s t))
("voiceless" nil (v oy1 s l ax s))
("voicemail" nil (v oy1 s m ey1 l))
("voiceover" nil (v oy1 s ow1 v er0))
("voices" nil (v oy1 s ax z))
("voiceworks" nil (v oy1 s w er1 k s))
("voicing" nil (v oy1 s ih0 ng))
("void" nil (v oy1 d))
("voided" nil (v oy1 d ih0 d))
("voiding" nil (v oy1 d ih0 ng))
("voids" nil (v oy1 d z))
("voight" nil (v oy1 t))
("voigt" nil (v oy1 g t))
("voigts" nil (v oy1 g t s))
("voila" nil (v w aa1 l aa1))
("voiles" nil (v oy1 l z))
("voinovich" nil (v oy1 n ax v ih0 ch))
("voir" nil (v w aa1 r))
("voisey" nil (v w aa1 s ey1))
("voisin" nil (v oy0 z ae1 n))
("voisine" nil (v oy0 z iy1 n))
("voit" nil (v oy1 t))
("vojta" nil (v oy1 t ah1))
("vokes" nil (v ow1 k s))
("voland" nil (v aa1 l ax n d))
("volante" nil (v ow0 l aa1 n t ey0))
("volatile" nil (v aa1 l ax t ax l))
("volatility" nil (v aa1 l ax t ih1 l ax t iy0))
("volberding" nil (v ow1 l b er0 d ih0 ng))
("volcanic" nil (v aa0 l k ae1 n ih0 k))
("volcanically" nil (v aa0 l k ae1 n ih0 k l iy0))
("volcano" nil (v aa0 l k ey1 n ow0))
("volcanoes" nil (v aa0 l k ey1 n ow0 z))
("volcanos" nil (v aa0 l k ey1 n ow0 z))
("volcker" nil (v ow1 l k er0))
("vold" nil (v ow1 l d))
("volden" nil (v ow1 l d ax n))
("volentine" nil (v ow0 l eh0 n t iy1 n iy0))
("voles" nil (v ow1 l z))
("voleta" nil (v ow0 l eh1 t ax))
("volga" nil (v aa1 l g ax))
("volgograd" nil (v ow1 l g ow0 g r ae1 d))
("volin" nil (v ow1 l ih0 n))
("volio" nil (v ow1 l iy0 ow0))
("volition" nil (v ow0 l ih1 sh ax n))
("volk" nil (v ow1 l k))
("volker" nil (v ow1 l k er0))
("volkers" nil (v ow1 l k er0 z))
("volkert" nil (v ow1 l k er0 t))
("volkman" nil (v ow1 l k m ax n))
("volkmann" nil (v ow1 l k m ax n))
("volkmar" nil (v ow1 l k m er0))
("volkmer" nil (v ow1 l k m er0))
("volksbank" nil (v ow1 l k s b ae1 ng k))
("volksfuersorge" nil (v ow0 l k s f y uw1 r s ao0 r jh))
("volkswagen" nil (v ow1 l k s w ae1 g ax n))
("volkswagens" nil (v ow1 l k s w ae1 g ax n z))
("voll" nil (v aa1 l))
("volland" nil (v aa1 l ax n d))
("vollbrecht" nil (v aa1 l b r ih0 k t))
("volle" nil (v aa1 l))
("vollenweider" nil (v aa1 l ih0 n w ay0 d er0))
("voller" nil (v aa1 l er0))
("volley" nil (v aa1 l iy0))
("volleyball" nil (v aa1 l iy0 b ao1 l))
("volleys" nil (v aa1 l iy0 z))
("vollman" nil (v aa1 l m ax n))
("vollmar" nil (v aa1 l m er0))
("vollmer" nil (v aa1 l m er0))
("vollrath" nil (v aa1 l r ax th))
("volmer" nil (v ow1 l m er0))
("volner" nil (v ow1 l n er0))
("volney" nil (v ow1 l n iy0))
("volokh" nil (v aa1 l aa0 k))
("volpe" nil (v ow1 l p))
("volpi" nil (v ow1 l p iy0))
("volpicella" nil (v ow1 l p ih0 s eh1 l ax))
("volt" nil (v ow1 l t))
("volta" nil (v ow1 l t ax))
("voltage" nil (v ow1 l t ax jh))
("voltages" nil (v ow1 l t ax jh ax z))
("voltaire" nil (v ow0 l t eh1 r))
("voltaren" nil (v ow1 l t er0 ax n))
("volts" nil (v ow1 l t s))
("voltz" nil (v ow1 l t s))
("voluble" nil (v aa1 l y ax b ax l))
("volume" nil (v aa1 l y uw0 m))
("volumes" nil (v aa1 l y uw0 m z))
("voluminous" nil (v ax l uw1 m ax n ax s))
("voluntarily" nil (v aa1 l ax n t eh1 r ax l iy0))
("voluntarism" nil (v ow0 l ah1 n t er0 ih1 z ax m))
("voluntary" nil (v aa1 l ax n t eh0 r iy0))
("volunteer" nil (v aa1 l ax n t ih1 r))
("volunteered" nil (v aa1 l ax n t ih1 r d))
("volunteering" nil (v ao1 l ax n t ih1 r ih0 ng))
("volunteers" nil (v aa1 l ax n t ih1 r z))
("voluptuous" nil (v ax l ah1 p ch ax w ax s))
("volvo" nil (v ow1 l v ow0))
("volvos" nil (v ao1 l v ow0 z))
("volvovitz" nil (v ow1 l v ax v ih0 t s))
("volvox" nil (v aa1 l v aa0 k s))
("volz" nil (v ow1 l z))
("vomit" nil (v aa1 m ax t))
("vomiting" nil (v aa1 m ax t ih0 ng))
("von" nil (v ao1 n))
("vona" nil (v ow1 n ax))
("vonada" nil (v ow0 n aa1 d ax))
("vonallmen" nil (v aa1 n ax l m eh0 n))
("vonarx" nil (v ax n aa1 r k s))
("vonbargen" nil (v aa1 n b aa1 r g ax n))
("vonbehren" nil (v aa1 n b ih1 r ax n))
("vonbergen" nil (v aa1 n b er1 g ax n))
("voncannon" nil (v aa1 n k ae1 n ax n))
("vonder" nil (v aa1 n d er0))
("vonderhaar" nil (v aa1 n d er0 hh aa1 r))
("vonderheide" nil (v aa1 n d er0 hh ay1 d))
("vondra" nil (v aa1 n d r ax))
("vondracek" nil (v aa1 n d r ax s ih0 k))
("vondrak" nil (v aa1 n d r ax k))
("vondrasek" nil (v ax n d r aa1 s eh0 k))
("vonfeldt" nil (v aa1 n f ih0 l t))
("vong" nil (v ao1 ng))
("vongunten" nil (v aa1 ng g ax n t ax n))
("vonk" nil (v aa1 ng k))
("vonnegut" nil (v aa1 n ax g ax t))
("vonnie" nil (v aa1 n iy0))
("vonny" nil (v aa1 n iy0))
("vonruden" nil (v aa1 n r uw0 d ax n))
("vons" nil (v aa1 n z))
("vonseggern" nil (v aa1 n s ih0 g er0 n))
("vonstein" nil (v aa1 n s t ay0 n))
("vontobel" nil (v aa1 n t ow1 b ax l))
("voodoo" nil (v uw1 d uw1))
("voorhees" nil (v uh1 r hh iy0 z))
("voorheis" nil (v uh1 r hh ay0 z))
("voorhies" nil (v uh1 r hh iy0 z))
("voorhis" nil (v uh1 r hh ih0 s))
("vora" nil (v ao1 r ax))
("voracious" nil (v ao0 r ey1 sh ax s))
("vorce" nil (v ao1 r s))
("vore" nil (v ao1 r))
("vorhauer" nil (v ao1 r hh aw1 r))
("vorhees" nil (v ao1 r hh iy0 z))
("vorhies" nil (v ao1 r hh iy0 z))
("voris" nil (v ao1 r ih0 s))
("vornado" nil (v ao1 r n aa1 d ow0))
("vorndran" nil (v ao1 r n d r ax n))
("vorontsov" nil (v ao0 r aa1 n t s aa1 v))
("voros" nil (v ao1 r ow0 z))
("vorpahl" nil (v ao1 r p aa0 l))
("vortec" nil (v ao1 r t eh1 k))
("vortex" nil (v ao1 r t eh0 k s))
("vorwald" nil (v ao1 r w ao0 l d))
("vorwerk" nil (v ao1 r w er0 k))
("vos" nil (v aa1 s))
("vosberg" nil (v aa1 s b er0 g))
("vosburg" nil (v aa1 s b er0 g))
("vosburgh" nil (v aa1 s b er0 g))
("vose" nil (v ow1 z))
("voshell" nil (v aa1 sh ax l))
("vosler" nil (v aa1 s ax l er0))
("voss" nil (v ao1 s))
("vossen" nil (v ao1 s ax n))
("vossler" nil (v aa1 s ax l er0))
("votava" nil (v ow0 t aa1 v ax))
("votaw" nil (v ow1 t ao0))
("vote" nil (v ow1 t))
("voted" nil (v ow1 t ax d))
("voter" nil (v ow1 t er0))
("voters" nil (v ow1 t er0 z))
("votes" nil (v ow1 t s))
("voth" nil (v aa1 th))
("voting" nil (v ow1 t ih0 ng))
("votrax" nil (v aa1 t r ae0 k s))
("votruba" nil (v ax t r uw1 b ax))
("vouch" nil (v aw1 ch))
("vouched" nil (v aw1 ch t))
("voucher" nil (v aw1 ch er0))
("vouchers" nil (v aw1 ch er0 z))
("vought" nil (v ao1 t))
("voute" nil (v uw1 t))
("vow" nil (v aw1))
("vowed" nil (v aw1 d))
("vowel" nil (v aw1 ax l))
("vowell" nil (v aa1 w eh0 l))
("vowels" nil (v aw1 ax l z))
("vowing" nil (v aw1 ih0 ng))
("vowles" nil (v aw1 ax l z))
("vows" nil (v aw1 z))
("vox" nil (v aa1 k s))
("voyage" nil (v oy1 ax jh))
("voyaged" nil (v oy1 ax jh d))
("voyager" nil (v oy1 ax jh er0))
("voyagers" nil (v oy1 ih0 jh er0 z))
("voyages" nil (v oy1 ax jh ax z))
("voyer" nil (v oy1 er0))
("voyeur" nil (v oy1 y uw1 r))
("voyeurism" nil (v oy1 y uw1 r ih0 z ax m))
("voyles" nil (v oy1 l z))
("voytek" nil (v oy1 t ih0 k))
("voytko" nil (v oy1 t k ow0))
("vrabel" nil (v r ae1 b ax l))
("vradenburg" nil (v r ey1 d ax n b er0 g))
("vrain" nil (v r ey1 n))
("vrana" nil (v r ae1 n ax))
("vranesevic" nil (v r aa1 n ax s eh1 v ih0 ch))
("vranitzky" nil (v r ax n ih1 t s k iy1))
("vranos" nil (v r aa1 n ow0 s))
("vrba" nil (v er1 b aa1))
("vrdolyak" nil (v er0 d ow1 l iy0 ae0 k))
("vredenburg" nil (v r iy1 d ax n b er0 g))
("vreeland" nil (v r iy1 l ax n d))
("vremya" nil (v r eh1 m iy0 ax))
("vries" nil (v r iy1 z))
("vroman" nil (v r ow1 m ax n))
("vroom" nil (v r uw1 m))
("vrooman" nil (v r uw1 m ax n))
("vs" nil (v er1 s ax z))
("vsel" nil (v iy1 s eh1 l))
("vu" nil (v uw1))
("vue" nil (v y uw1))
("vuitton" nil (v uw1 t ax n))
("vukelich" nil (v ax k eh1 l ih0 hh))
("vukovich" nil (v y uw1 k ax v ih0 ch))
("vulcan" nil (v ah1 l k ax n))
("vulcans" nil (v ah1 l k ax n z))
("vulgamore" nil (v ah1 l g ax m ao0 r))
("vulgar" nil (v ah1 l g er0))
("vulgarity" nil (v ax l g eh1 r ih0 t iy0))
("vulgarization" nil (v ah1 l g er0 ih0 z ey1 sh ax n))
("vullo" nil (v uw1 l ow0))
("vulnerabilities" nil (v ah1 l n er0 ax b ih1 l ih0 t iy0 z))
("vulnerability" nil (v ah1 l n er0 ax b ih1 l ih0 t iy0))
("vulnerable" nil (v ah1 l n er0 ax b ax l))
("vultaggio" nil (v uw0 l t aa1 jh iy0 ow0))
("vulture" nil (v ah1 l ch er0))
("vultures" nil (v ah1 l ch er0 z))
("vuncannon" nil (v ah1 n k ax n aa0 n))
("vuolo" nil (v uw0 ow1 l ow0))
("vuong" nil (v uw0 ao1 ng))
("vyacheslav" nil (v y aa1 ch ax s l aa0 v))
("vyas" nil (v y aa1 s))
("vying" nil (v ay1 ih0 ng))
("w" nil (d ah1 b ax l y uw0))
("wa" nil (w aa1))
("waack" nil (w aa1 k))
("waag" nil (w aa1 g))
("waage" nil (w aa1 ih0 jh))
("waal" nil (w aa1 l))
("waara" nil (w aa1 r ax))
("waas" nil (w aa1 z))
("wabash" nil (w ao1 b ae0 sh))
("wace" nil (w ey1 s))
("wach" nil (w ao1 ch))
("wacha" nil (w aa1 ch ax))
("wachholz" nil (w ao1 k hh ow0 l z))
("wachner" nil (w ae1 k n er0))
("wachob" nil (w ao1 k ax b))
("wachovia" nil (w aa0 ch ow1 v iy0 ax))
("wachowiak" nil (v ax hh aw1 iy0 ae0 k))
("wachowski" nil (v ax hh ao1 f s k iy0))
("wachs" nil (w ao1 k s))
("wachsman" nil (w ao1 k s m ax n))
("wachsmuth" nil (w ao1 k s m uh0 th))
("wachtel" nil (w ao1 ch ax l))
("wachtell" nil (w aa0 k t eh1 l))
("wachter" nil (w ao1 k t er0))
("wachtler" nil (w ae1 k t l er0))
("wack" nil (w ae1 k))
("wackenhut" nil (w aa1 k ax n hh ah1 t))
("wacker" nil (w ae1 k er0))
("wackerle" nil (w ae1 k er0 ax l))
("wacko" nil (w aa1 k ow0))
("wacksman" nil (w ae1 k s m ax n))
("wacky" nil (w ae1 k iy0))
("waco" nil (w ey1 k ow0))
("wactlar" nil (w aa1 k t l er0))
("wad" nil (w aa1 d))
("wada" nil (w aa1 d ax))
("wadas" nil (w aa1 d ax z))
("wadded" nil (w aa1 d ax d))
("waddell" nil (w aa0 d eh1 l))
("waddie" nil (w aa1 d iy0))
("waddill" nil (w ao1 d ih1 l))
("waddington" nil (w aa1 d ih0 ng t ax n))
("waddle" nil (w aa1 d ax l))
("waddy" nil (w aa1 d iy0))
("wade" nil (w ey1 d))
("waded" nil (w ey1 d ih0 d))
("wadel" nil (w ey1 d ax l))
("waders" nil (w ey1 d er0 z))
("wades" nil (w ey1 d z))
("wadford" nil (w ao1 d f er0 d))
("wadhams" nil (w ao1 d ax m z))
("wading" nil (w ey1 d ih0 ng))
("wadkins" nil (w ao1 d k ih0 n z))
("wadle" nil (w ao1 d ax l))
("wadleigh" nil (w ao1 d l iy0))
("wadley" nil (w aa1 d l iy0))
("wadlington" nil (w aa1 d l ih0 ng t ax n))
("wadlow" nil (w aa1 d l ow1))
("wadman" nil (w ao1 d m ax n))
("wads" nil (w aa1 d z))
("wadsworth" nil (w aa1 d z w er0 th))
("waechter" nil (w eh1 k t er0))
("waertsilae" nil (w eh1 r t s ax l ey1))
("wafer" nil (w ey1 f er0))
("wafers" nil (w ey1 f er0 z))
("waffle" nil (w aa1 f ax l))
("waffled" nil (w aa1 f ax l d))
("waffles" nil (w aa1 f ax l z))
("waffling" nil (w aa1 f l ih0 ng))
("wafford" nil (w aa1 f er0 d))
("waft" nil (w aa1 f t))
("wafted" nil (w aa1 f t ih0 d))
("wafting" nil (w aa1 f t ih0 ng))
("wag" nil (w ae1 g))
("wagaman" nil (w ae1 g ah1 m ax n))
("wagar" nil (w ae1 g er0))
("wage" nil (w ey1 jh))
("waged" nil (w ey1 jh d))
("wageman" nil (w ey1 jh m ax n))
("wagenaar" nil (w ae1 g ax n aa0 r))
("wagener" nil (w ae1 g ax n er0))
("wagenknecht" nil (w ae1 g ax n ih0 k t))
("wager" nil (w ey1 jh er0))
("wagered" nil (w ey1 jh er0 d))
("wagering" nil (w ey1 jh er0 ih0 ng))
("wagers" nil (w ey1 jh er0 z))
("wages" nil (w ey1 jh ax z))
("wagg" nil (w ae1 g))
("wagged" nil (w ae1 g d))
("waggener" nil (w ae1 g ax n er0))
("wagging" nil (w ae1 g ih0 ng))
("waggling" nil (w ae1 g ax l ih0 ng))
("waggoner" nil (w ae1 g ax n er0))
("waggy" nil (w ae1 g iy0))
("waging" nil (w ey1 jh ih0 ng))
("wagle" nil (w ae1 g ax l))
("wagler" nil (w ae1 g l er0))
("wagley" nil (w ae1 g l iy0))
("wagman" nil (w ae1 g m ax n))
("wagner" nil (w ae1 g n er0))
("wagnerian" nil (w ae1 g n eh1 r iy0 ax n))
("wagnon" nil (w ae1 g n ax n))
("wagon" nil (w ae1 g ax n))
("wagoneer" nil (w ae1 g ax n ih1 r))
("wagoneers" nil (w ae1 g ax n ih1 r z))
("wagoner" nil (w ae1 g ax n er0))
("wagons" nil (w ae1 g ax n z))
("wags" nil (w ae1 g z))
("wagstaff" nil (w ae1 g s t ae1 f))
("wagster" nil (w ae1 g s t er0))
("waguespack" nil (w ae1 g ih0 s p ae1 k))
("wagy" nil (w ae1 g iy0))
("wah" nil (w aa1))
("wahl" nil (w aa1 l))
("wahlberg" nil (w aa1 l b er0 g))
("wahle" nil (w ao1 l))
("wahlen" nil (w aa1 l ax n))
("wahler" nil (w aa1 l er0))
("wahlers" nil (w aa1 l er0 z))
("wahlert" nil (w aa1 l er0 t))
("wahlgren" nil (w aa1 l g r ax n))
("wahlquist" nil (w aa1 l k w ih1 s t))
("wahlstrom" nil (w aa1 l s t r ax m))
("wai" nil (w ay1))
("waid" nil (w ey1 d))
("waide" nil (w ey1 d))
("waidelich" nil (w ay1 d l ih0 k))
("waif" nil (w ey1 f))
("waigel" nil (w ay1 g ax l))
("waikiki" nil (w ay1 k iy0 k iy1))
("wail" nil (w ey1 l))
("wailed" nil (w ey1 l d))
("wailes" nil (w ey1 l z))
("wailing" nil (w ey1 l ih0 ng))
("wails" nil (w ey1 l z))
("wain" nil (w ey1 n))
("wainer" nil (w ey1 n er0))
("wainio" nil (w ey1 n iy0 ow0))
("wainman" nil (w ey1 n m ax n))
("wainoco" nil (w ey1 n ow1 k ow0))
("wainright" nil (w ey1 n r ay1 t))
("wainscott" nil (w ey1 n s k ax t))
("wainwright" nil (w ey1 n r ay1 t))
("wais" nil (w ey1 z))
("waisanen" nil (w ay1 s ax n ax n))
("waisner" nil (w ey1 z n er0))
("waist" nil (w ey1 s t))
("waistline" nil (w ey1 s t l ay1 n))
("waists" nil (w ey1 s t s))
("wait" nil (w ey1 t))
("waite" nil (w ey1 t))
("waited" nil (w ey1 t ax d))
("waiter" nil (w ey1 t er0))
("waiters" nil (w ey1 t er0 z))
("waites" nil (w ey1 t s))
("waiting" nil (w ey1 t ih0 ng))
("waitkus" nil (w ey1 t k ax s))
("waitman" nil (w ay1 t m ax n))
("waitress" nil (w ey1 t r ax s))
("waitresses" nil (w ey1 t r ax s ih0 z))
("waits" nil (w ey1 t s))
("waitt" nil (w ey1 t))
("waitzkin" nil (w ey1 t s k ih1 n))
("waive" nil (w ey1 v))
("waived" nil (w ey1 v d))
("waiver" nil (w ey1 v er0))
("waivers" nil (w ey1 v er0 z))
("waives" nil (w ey1 v z))
("waiving" nil (w ey1 v ih0 ng))
("wajda" nil (v ay1 d ax))
("wakabayashi" nil (w aa0 k aa1 b aa0 y aa1 sh iy0))
("wake" nil (w ey1 k))
("wakefield" nil (w ey1 k f iy1 l d))
("wakeham" nil (w ae1 k ax m))
("wakeland" nil (w ey1 k l ax n d))
("wakeley" nil (w ae1 k l iy0))
("wakely" nil (w ey1 k l iy0))
("wakeman" nil (w ey1 k m ax n))
("waken" nil (w ey1 k ax n))
("wakes" nil (w ey1 k s))
("waking" nil (w ey1 k ih0 ng))
("wakley" nil (w ae1 k l iy0))
("wako" nil (w ae1 k ow0))
("wal" nil (w ao1 l))
("walas" nil (w ao1 l ax z))
("walberg" nil (w ao1 l b er0 g))
("walbert" nil (w ao1 l b er0 t))
("walborn" nil (w ao1 l b er0 n))
("walbridge" nil (w ao1 l b r ih0 jh))
("walbro" nil (w ao1 l b r ow0))
("walburn" nil (w ao1 l b er0 n))
("walby" nil (w ao1 l b iy0))
("walch" nil (w ao1 l ch))
("walcher" nil (w ao1 l ch er0))
("walck" nil (w ao1 l k))
("walcot" nil (w ao1 l k aa0 t))
("walcott" nil (w ao1 l k aa0 t))
("walczak" nil (v aa1 l ch ae0 k))
("walczyk" nil (v aa1 l ch ih0 k))
("wald" nil (w ao1 l d))
("walda" nil (v aa1 l d ax))
("waldbaum" nil (w ao1 l d b aw1 m))
("walde" nil (w ao1 l d))
("waldeck" nil (w ao1 l d eh0 k))
("waldecker" nil (w ao1 l d eh1 k er0))
("waldegard" nil (w ao1 l d ax g aa1 r d))
("waldemar" nil (v aa1 l d ax m aa0 r))
("walden" nil (w ao1 l d ax n))
("waldenbooks" nil (w ao1 l d ax n b uh1 k s))
("walder" nil (w ao1 l d er0))
("waldheim" nil (w ao1 l d hh ay1 m))
("waldholz" nil (w ao1 l d hh ow1 l t s))
("waldhorn" nil (w ao1 l d hh ao1 r n))
("waldie" nil (w ao1 l d iy0))
("walding" nil (w ao1 l d ih0 ng))
("waldinger" nil (w ao1 l d ih0 ng er0))
("waldman" nil (w aa1 l d m ax n))
("waldmann" nil (w ao1 l d m ax n))
("waldner" nil (w ao1 l d n er0))
("waldo" nil (w aa1 l d ow0))
("waldoch" nil (w ao1 l d aa0 k))
("waldock" nil (w ao1 l d aa0 k))
("waldon" nil (w ao1 l d ax n))
("waldorf" nil (w ao1 l d ao0 r f))
("waldow" nil (w ao1 l d ow0))
("waldren" nil (w ao1 l d r ax n))
("waldrep" nil (w ao1 l d r ax p))
("waldridge" nil (w ao1 l d r ih0 jh))
("waldrip" nil (w ao1 l d r ax p))
("waldron" nil (w ao1 l d r ax n))
("waldroop" nil (w ao1 l d r uw1 p))
("waldrop" nil (w ao1 l d r aa0 p))
("waldroup" nil (w ao1 l d r uw1 p))
("waldrum" nil (w ao1 l d r ax m))
("waldrup" nil (w ao1 l d r ax p))
("waldschmidt" nil (w ao1 l d sh m ih1 t))
("waldvogel" nil (w ao1 l d v ow1 g ax l))
("wale" nil (w ey1 l))
("waleed" nil (w aa1 l iy1 d))
("walek" nil (v aa1 l eh0 k))
("walen" nil (w ey1 l ax n))
("walenta" nil (w ax l eh1 n t ax))
("wales" nil (w ey1 l z))
("walesa" nil (w ax l eh1 s ax))
("waleson" nil (w ey1 l s ax n))
("walford" nil (w ao1 l f er0 d))
("walfred" nil (w ao1 l f r eh0 d))
("walgreen" nil (w ao1 l g r iy1 n))
("walgren" nil (w ao1 l g r ax n))
("walicki" nil (w ax l ih1 k iy0))
("walid" nil (w aa0 l iy1 d))
("waligora" nil (w ao0 l ih0 g ao1 r ax))
("walinsky" nil (w ax l ih1 n s k iy0))
("walizer" nil (w ao1 l ay0 z er0))
("walk" nil (w ao1 k))
("walke" nil (w ao1 k))
("walked" nil (w ao1 k t))
("walkenhorst" nil (w ao1 k ax n hh ao1 r s t))
("walker" nil (w ao1 k er0))
("walkers" nil (w ao1 k er0 z))
("walkie" nil (w ao1 k iy0))
("walking" nil (w ao1 k ih0 ng))
("walkington" nil (w ao1 k ih0 ng t ax n))
("walkinshaw" nil (w ao1 k ax n sh ao0))
("walkley" nil (w ao1 k l iy0))
("walkman" nil (w ao1 k m ae1 n))
("walko" nil (w ao1 k ow0))
("walkout" nil (w ao1 k aw1 t))
("walkouts" nil (w ao1 k aw1 t s))
("walkowiak" nil (w ao0 k aw1 iy0 ae0 k))
("walkowski" nil (w ao0 k ao1 f s k iy0))
("walks" nil (w ao1 k s))
("walkup" nil (w ao1 k ah1 p))
("walkure" nil (w ao1 k y er0))
("walkway" nil (w ao1 k w ey1))
("walkways" nil (w ao1 k w ey1 z))
("wall" nil (w ao1 l))
("walla" nil (w ao1 l ax))
("wallace" nil (w ao1 l ax s))
("wallach" nil (w ao1 l ax k))
("wallack" nil (w ao1 l ax k))
("wallander" nil (w ao1 l ax n d er0))
("wallar" nil (w ao1 l er0))
("wallboard" nil (w ao1 l b ao1 r d))
("walle" nil (w ao1 l))
("walled" nil (w ao1 l d))
("wallen" nil (w ao1 l ax n))
("wallenberg" nil (w ao1 l ax n b er0 g))
("wallenstein" nil (w ao1 l ax n s t ay1 n))
("waller" nil (w ao1 l er0))
("wallerstein" nil (w ao1 l er0 s t ay1 n))
("walles" nil (w ao1 l z))
("wallet" nil (w ao1 l ax t))
("wallets" nil (w ao1 l ax t s))
("walley" nil (w ao1 l iy0))
("walleye" nil (w ao1 l ay1))
("wallflower" nil (w ao1 l f l aw1 er0))
("wallgren" nil (w ao1 l g r ax n))
("wallich" nil (w ao1 l ih0 k))
("wallick" nil (w ao1 l ih0 k))
("wallie" nil (w ao1 l iy0))
("wallin" nil (w ao1 l ih0 n))
("walling" nil (w ao1 l ih0 ng))
("wallingford" nil (w ao1 l ih0 ng f er0 d))
("wallington" nil (w ao1 l ih0 ng t ax n))
("wallis" nil (w ao1 l ih0 s))
("wallison" nil (w ao1 l ih0 s ax n))
("wallman" nil (w ao1 l m ax n))
("wallner" nil (w ao1 l n er0))
("wallo" nil (w aa1 l ow0))
("walloch" nil (w ao1 l ax k))
("wallop" nil (w aa1 l ax p))
("walloping" nil (w ao1 l ax p ih0 ng))
("wallops" nil (w aa1 l ax p s))
("wallow" nil (w aa1 l ow0))
("wallowed" nil (w aa1 l ow0 d))
("wallowing" nil (w aa1 l ow0 ih0 ng))
("wallpaper" nil (w ao1 l p ey1 p er0))
("wallpapered" nil (w ao1 l p ey1 p er0 d))
("wallpapering" nil (w ao1 l p ey1 p er0 ih0 ng))
("wallpapers" nil (w ao1 l p ey1 p er0 z))
("walls" nil (w ao1 l z))
("wallsend" nil (w ao1 l s eh1 n d))
("wallstreet" nil (w ao1 l s t r iy1 t))
("wallwork" nil (w ao1 l w er1 k))
("wally" nil (w ao1 l iy0))
("walmart" nil (w ao1 l m aa1 r t))
("walmer" nil (w ao1 l m er0))
("walmond" nil (w ao1 l m ax n d))
("walmsley" nil (w ao1 l m s l iy0))
("waln" nil (w ao1 l n))
("walnut" nil (w ao1 l n ah1 t))
("walnuts" nil (w ao1 l n ah1 t s))
("walp" nil (w ae1 l p))
("walpole" nil (w ao1 l p ow1 l))
("walrath" nil (w ao1 l r ax th))
("walraven" nil (w ao1 l r ey1 v ax n))
("walrod" nil (w ao1 l r aa0 d))
("walrus" nil (w ao1 l r ax s))
("walruses" nil (w aa1 l r ax s ih0 z))
("walser" nil (w ao1 l z er0))
("walsh" nil (w ao1 l sh))
("walski" nil (v aa1 l s k iy0))
("walstad" nil (w ao1 l s t ae0 d))
("walston" nil (w ao1 l s t ax n))
("walstrom" nil (w ao1 l s t r ax m))
("walsworth" nil (w ao1 l s w er0 th))
("walt" nil (w ao1 l t))
("walter" nil (w ao1 l t er0))
("waltermire" nil (w ao1 l t er0 m ay0 r))
("walters" nil (w ao1 l t er0 z))
("walth" nil (w ao1 l th))
("walthall" nil (w ao1 l th ax l))
("waltham" nil (w ao1 l th ax m))
("walther" nil (w ao1 l th er0))
("walthers" nil (w ao1 l th er0 z))
("walthour" nil (w ao1 l t aw0 r))
("waltman" nil (w ao1 l t m ax n))
("waltner" nil (w ao1 l t n er0))
("walton" nil (w ao1 l t ax n))
("waltrip" nil (w ao1 l t r ih0 p))
("walts" nil (w ao1 l t s))
("waltz" nil (w ao1 l t s))
("waltzer" nil (w ao1 l t s er0))
("waltzes" nil (w ao1 l t s ax z))
("waltzing" nil (w ao1 l t s ih0 ng))
("walworth" nil (w ao1 l w er0 th))
("walwyn" nil (w ao1 l w ih0 n))
("walz" nil (w ao1 l z))
("walzer" nil (w ao1 l z er0))
("wambach" nil (w ao1 m b aa0 k))
("wamble" nil (w aa1 m b ax l))
("wambold" nil (w aa1 m b ow1 l d))
("wamboldt" nil (w aa1 m b ow0 l t))
("wambolt" nil (w aa1 m b ow1 l t))
("wampler" nil (w aa1 m p l er0))
("wampole" nil (w aa1 m p ow1 l))
("wampum" nil (w aa1 m p ax m))
("wamser" nil (w aa1 m z er0))
("wamsley" nil (w aa1 m z l iy0))
("wan" nil (w aa1 n))
("wanamaker" nil (w aa1 n ax m ey1 k er0))
("wanat" nil (w ey1 n ax t))
("wand" nil (w aa1 n d))
("wanda" nil (w aa1 n d ax))
("wandel" nil (w aa1 n d ax l))
("wandell" nil (w aa1 n d ax l))
("wander" nil (w aa1 n d er0))
("wandered" nil (w aa1 n d er0 d))
("wanderer" nil (w aa1 n d er0 er0))
("wanderers" nil (w aa1 n d er0 er0 z))
("wandering" nil (w aa1 n d er0 ih0 ng))
("wanderings" nil (w aa1 n d er0 ih0 ng z))
("wanderlust" nil (w aa1 n d er0 l ah1 s t))
("wanders" nil (w aa1 n d er0 z))
("wandersee" nil (w aa1 n d er0 s iy1))
("wandie" nil (w aa1 n d iy0))
("wandis" nil (w aa1 n d ih0 s))
("wandler" nil (w aa1 n d l er0))
("wandling" nil (w aa1 n d l ih0 ng))
("wandrey" nil (w aa1 n d r iy0))
("wands" nil (w aa1 n d z))
("wane" nil (w ey1 n))
("waned" nil (w ey1 n d))
("wanek" nil (w ey1 n ih0 k))
("waner" nil (w ey1 n er0))
("wanes" nil (w ey1 n z))
("wang" nil (w ae1 ng))
("wangen" nil (w ae1 ng ax n))
("wanger" nil (w ae1 ng er0))
("wangerin" nil (w ao1 ng g er0 ih0 n))
("wangled" nil (w ae1 ng g ax l d))
("wangler" nil (w ae1 ng g l er0))
("waning" nil (w ey1 n ih0 ng))
("wank" nil (w aa1 ng k))
("wanke" nil (w aa1 ng k))
("wanko" nil (w aa1 ng k ow0))
("wanless" nil (w aa1 n l ax s))
("wann" nil (w aa1 n))
("wanna" nil (w aa1 n ax))
("wannabe" nil (w aa1 n ax b iy1))
("wannabes" nil (w aa1 n ax b iy1 z))
("wannamaker" nil (w aa1 n ax m ey1 k er0))
("wannemacher" nil (w aa1 n ax m aa0 k er0))
("wanner" nil (w aa1 n er0))
("wanninger" nil (w aa1 n ih0 ng er0))
("wanniski" nil (w ax n ih1 s k iy0))
("wanser" nil (w ao1 n z er0))
("wansley" nil (w ao1 n z l iy0))
("want" nil (w aa1 n t))
("wanta" nil (w aa1 n t ax))
("wantage" nil (w aa1 n t ih0 jh))
("wantages" nil (w aa1 n t ih0 jh ih0 z))
("wanted" nil (w aa1 n t ax d))
("wanting" nil (w aa1 n t ih0 ng))
("wantland" nil (w aa1 n t l ax n d))
("wanton" nil (w ao1 n t ax n))
("wantonly" nil (w ao1 n t ax n l iy0))
("wants" nil (w aa1 n t s))
("wantz" nil (w aa1 n t s))
("wanzer" nil (w aa1 n z er0))
("waples" nil (w ao1 p ax l z))
("wapner" nil (w aa1 p n er0))
("wapping" nil (w aa1 p ih0 ng))
("war" nil (w ao1 r))
("warbington" nil (w ao1 r b ih0 ng t ax n))
("warbling" nil (w ao1 r b ax l ih0 ng))
("warburg" nil (w ao1 r b er0 g))
("warburton" nil (w ao1 r b er0 t ax n))
("warchol" nil (w ao1 r k ao0 l))
("ward" nil (w ao1 r d))
("warda" nil (w ao1 r d ax))
("wardair" nil (w ao1 r d eh1 r))
("warde" nil (w ao1 r d))
("warded" nil (w ao1 r d ih0 d))
("wardell" nil (w ao1 r d eh0 l))
("warden" nil (w ao1 r d ax n))
("wardens" nil (w ao1 r d ax n z))
("warder" nil (w ao1 r d er0))
("warding" nil (w ao1 r d ih0 ng))
("wardlaw" nil (w ao1 r d l ao1))
("wardle" nil (w ao1 r d ax l))
("wardley" nil (w ao1 r d l iy0))
("wardlow" nil (w ao1 r d l ow1))
("wardrip" nil (w ao1 r d r ih0 p))
("wardrobe" nil (w ao1 r d r ow1 b))
("wardrobes" nil (w ao1 r d r ow1 b z))
("wardrop" nil (w ao1 r d r aa1 p))
("wards" nil (w ao1 r d z))
("wardwell" nil (w ao1 r d w eh1 l))
("ware" nil (w eh1 r))
("warehime" nil (w eh1 r hh ay1 m))
("warehouse" nil (w eh1 r hh aw1 s))
("warehoused" nil (w eh1 r hh aw1 z d))
("warehouses" nil (w eh1 r hh aw1 z ih0 z))
("warehousing" nil (w eh1 r hh aw1 z ih0 ng))
("wareing" nil (w eh1 r ih0 ng))
("waren" nil (w eh1 r ax n))
("wares" nil (w eh1 r z))
("warf" nil (w ao1 r f))
("warfare" nil (w ao1 r f eh1 r))
("warfel" nil (w ao1 r f ax l))
("warfield" nil (w ao1 r f iy0 l d))
("warford" nil (w ao1 r f er0 d))
("warga" nil (w ao1 r g ax))
("wargo" nil (w ao1 r g ow1))
("warhead" nil (w ao1 r hh eh1 d))
("warheads" nil (w ao1 r hh eh1 d z))
("warhol" nil (w ao1 r hh ao0 l))
("warhurst" nil (w ao1 r hh er0 s t))
("warily" nil (w eh1 r ax l iy0))
("wariness" nil (w eh1 r iy0 n ih0 s))
("waring" nil (w eh1 r ih0 ng))
("wark" nil (w ao1 r k))
("warkentin" nil (w ao0 r k eh1 n t ih0 n))
("warley" nil (w ao1 r l iy0))
("warlick" nil (w ao1 r l ih0 k))
("warlike" nil (w ao1 r l ay1 k))
("warloeck" nil (w ao1 r l aa1 k))
("warlord" nil (w ao1 r l ao1 r d))
("warlords" nil (w ao1 r l ao1 r d z))
("warm" nil (w ao1 r m))
("warmack" nil (w ao1 r m ae0 k))
("warman" nil (w ao1 r m ax n))
("warmed" nil (w ao1 r m d))
("warmer" nil (w ao1 r m er0))
("warmest" nil (w ao1 r m ax s t))
("warming" nil (w ao1 r m ih0 ng))
("warmington" nil (w ao1 r m ih0 ng t ax n))
("warminster" nil (w ao1 r m ih1 n s t er0))
("warmish" nil (w ao1 r m ih0 sh))
("warmly" nil (w ao1 r m l iy0))
("warmond" nil (w ao1 r m ax n d))
("warmongering" nil (w ao1 r m aa1 ng g er0 ih0 ng))
("warmoth" nil (w ao1 r m ax th))
("warms" nil (w ao1 r m z))
("warmth" nil (w ao1 r m th))
("warmup" nil (w ao1 r m ah1 p))
("warmuth" nil (w ao1 r m uh1 th))
("warn" nil (w ao1 r n))
("warnaco" nil (w ao1 r n ax k ow1))
("warncke" nil (w ao1 r ng k iy0))
("warne" nil (w ao1 r n))
("warnecke" nil (w ao1 r n ih0 k))
("warned" nil (w ao1 r n d))
("warneke" nil (w ao1 r n ih0 k))
("warnell" nil (w ao1 r n ax l))
("warner" nil (w ao1 r n er0))
("warnes" nil (w ao1 r n z))
("warnick" nil (w ao1 r n ih0 k))
("warning" nil (w ao1 r n ih0 ng))
("warningly" nil (w ao1 r n ih0 ng l iy0))
("warnings" nil (w ao1 r n ih0 ng z))
("warnke" nil (w ao1 r ng k iy0))
("warnken" nil (w ao1 r ng k ax n))
("warnock" nil (w ao1 r n aa0 k))
("warns" nil (w ao1 r n z))
("waronker" nil (w ao0 r aa1 ng k er0))
("warp" nil (w ao1 r p))
("warpath" nil (w ao1 r p ae1 th))
("warped" nil (w ao1 r p t))
("warping" nil (w ao1 r p ih0 ng))
("warplane" nil (w ao1 r p l ey1 n))
("warplanes" nil (w ao1 r p l ey1 n z))
("warps" nil (w ao1 r p s))
("warr" nil (w ao1 r))
("warrant" nil (w ao1 r ax n t))
("warranted" nil (w ao1 r ax n t ih0 d))
("warranties" nil (w ao1 r ax n t iy0 z))
("warrantless" nil (w ao1 r ax n t l ax s))
("warrants" nil (w ao1 r ax n t s))
("warranty" nil (w ao1 r ax n t iy0))
("warrell" nil (w ao1 r eh1 l))
("warren" nil (w ao1 r ax n))
("warrens" nil (w ao1 r ax n z))
("warrick" nil (w ao1 r ih0 k))
("warriner" nil (w ao1 r ax n er0))
("warring" nil (w ao1 r ih0 ng))
("warrington" nil (w ao1 r ih0 ng t ax n))
("warrior" nil (w ao1 r iy0 er0))
("warriors" nil (w ao1 r iy0 er0 z))
("wars" nil (w ao1 r z))
("warsaw" nil (w ao1 r s ao1))
("warsaws" nil (w ao1 r s ao1 z))
("warshauer" nil (w ao1 r sh aw0 er0))
("warshaw" nil (w ao1 r sh ao1))
("warshawsky" nil (w ao1 r sh ax w s k iy0))
("warship" nil (w ao1 r sh ih1 p))
("warships" nil (w ao1 r sh ih1 p s))
("warstler" nil (w ao1 r s t l er0))
("wart" nil (w ao1 r t))
("wartenberg" nil (w ao1 r t ax n b er0 g))
("warth" nil (w ao1 r th))
("warthen" nil (w ao1 r th ax n))
("wartime" nil (w ao1 r t ay1 m))
("wartimes" nil (w ao1 r t ay1 m z))
("wartman" nil (w ao1 r t m ax n))
("warton" nil (w ao1 r t ax n))
("warts" nil (w ao1 r t s))
("wartzman" nil (w ao1 r t s m ax n))
("warwick" nil (w ao1 r w ih0 k))
("wary" nil (w eh1 r iy0))
("warzecha" nil (w ao0 r z eh1 hh ax))
("was" nil (w aa1 z))
("wasatch" nil (w aa1 s ae0 ch))
("wasch" nil (w ao1 sh))
("wascher" nil (w ao1 sh er0))
("wasco" nil (w ao1 s k ow0))
("wascom" nil (w ao1 s k ax m))
("wasden" nil (w ao1 s d ax n))
("wasem" nil (w ao1 s ih0 m))
("wash" nil (w aa1 sh))
("washabaugh" nil (w ao1 sh ax b ao0))
("washable" nil (w aa1 sh ax b ax l))
("washam" nil (w aa1 sh ax m))
("washbasin" nil (w aa1 sh b ey1 s ax n))
("washboard" nil (w aa1 sh b ao1 r d))
("washburn" nil (w aa1 sh b er1 n))
("washburne" nil (w ao1 sh b er1 n))
("washed" nil (w aa1 sh t))
("washer" nil (w aa1 sh er0))
("washers" nil (w aa1 sh er0 z))
("washes" nil (w aa1 sh ih0 z))
("washing" nil (w aa1 sh ih0 ng))
("washington" nil (w aa1 sh ih0 ng t ax n))
("washingtonian" nil (w aa1 sh ih0 ng t ow1 n iy0 ax n))
("washingtonians" nil (w aa1 sh ih0 ng t ow1 n iy0 ax n z))
("washko" nil (v aa1 sh k ow0))
("washout" nil (w aa1 sh aw1 t))
("washouts" nil (w aa1 sh aw1 t s))
("washroom" nil (w aa1 sh r uw1 m))
("washtub" nil (w aa1 sh t ah1 b))
("washy" nil (w aa1 sh iy0))
("wasielewski" nil (v ax s iy0 l eh1 f s k iy0))
("wasik" nil (v aa1 s ih0 k))
("wasil" nil (w ao1 s ax l))
("wasilewski" nil (v ax s ih0 l eh1 f s k iy0))
("wasinger" nil (w ao1 s ih0 n jh er0))
("waskiewicz" nil (v aa1 s k ax v ih0 ch))
("wasko" nil (v aa1 s k ow0))
("waskow" nil (v aa1 s k ow0))
("wasley" nil (w ao1 s l iy0))
("wasmer" nil (w ao1 s ax m er0))
("wasmund" nil (w ao1 z m ax n d))
("wasmuth" nil (w ao1 z m uw0 th))
("wason" nil (w aa1 s ax n))
("wasp" nil (w aa1 s p))
("wasps" nil (w aa1 s p s))
("wassel" nil (w ao1 s ax l))
("wassell" nil (w ao1 s ax l))
("wassenaar" nil (w ao1 s ih0 n aa0 r))
("wasser" nil (w ao1 s er0))
("wasserman" nil (w aa1 s er0 m ax n))
("wasserstein" nil (w aa1 s er0 s t iy1 n))
("wassink" nil (w ao1 s ih0 ng k))
("wassman" nil (w ao1 s m ax n))
("wassmer" nil (w aa1 s m er0))
("wassom" nil (w aa1 s ax m))
("wasson" nil (w ao1 s ax n))
("wassum" nil (w aa1 s ax m))
("waste" nil (w ey1 s t))
("wastebasket" nil (w ey1 s t b ae1 s k ax t))
("wastebaskets" nil (w ey1 s t b ae1 s k ax t s))
("wasted" nil (w ey1 s t ax d))
("wasteful" nil (w ey1 s t f ax l))
("wastefulness" nil (w ey1 s t f ax l n ax s))
("wasteland" nil (w ey1 s t l ae1 n d))
("wastepaper" nil (w ey1 s t p ey1 p er0))
("wasters" nil (w ey1 s t er0 z))
("wastes" nil (w ey1 s t s))
("wastewater" nil (w ey1 s t w ao1 t er0))
("wasting" nil (w ey1 s t ih0 ng))
("waszak" nil (v aa1 sh ax k))
("waszkiewicz" nil (v aa1 sh k ax v ih0 ch))
("wat" nil (w ao1 t))
("watanabe" nil (w aa1 t aa0 n aa1 b ey0))
("watch" nil (w aa1 ch))
("watchdog" nil (w aa1 ch d ao1 g))
("watchdogs" nil (w aa1 ch d ao1 g z))
("watched" nil (w aa1 ch t))
("watcher" nil (w aa1 ch er0))
("watchers" nil (w aa1 ch er0 z))
("watches" nil (w aa1 ch ax z))
("watchful" nil (w aa1 ch f ax l))
("watching" nil (w aa1 ch ih0 ng))
("watchman" nil (w aa1 ch m ax n))
("watchorn" nil (w ao1 ch er0 n))
("watchword" nil (w aa1 ch w er1 d))
("water" nil (w ao1 t er0))
("waterbed" nil (w ao1 t er0 b eh1 d))
("waterbeds" nil (w ao1 t er0 b eh1 d z))
("waterborne" nil (w ao1 t er0 b ao1 r n))
("waterbury" nil (w ao1 t er0 b eh1 r iy0))
("watercolor" nil (w ao1 t er0 k ah1 l er0))
("watercolors" nil (w ao1 t er0 k ah1 l er0 z))
("watercress" nil (w ao1 t er0 k r eh1 s))
("watered" nil (w ao1 t er0 d))
("waterer" nil (w ao1 t er0 er0))
("waterfall" nil (w ao1 t er0 f ao1 l))
("waterfalls" nil (w ao1 t er0 f ao1 l z))
("waterfield" nil (w ao1 t er0 f iy0 l d))
("waterford" nil (w ao1 t er0 f er0 d))
("waterfowl" nil (w ao1 t er0 f aw1 l))
("waterfront" nil (w ao1 t er0 f r ah1 n t))
("watergate" nil (w ao1 t er0 g ey1 t))
("waterhouse" nil (w ao1 t er0 hh aw1 s))
("watering" nil (w ao1 t er0 ih0 ng))
("waterishness" nil (w ao1 t er0 ih0 sh n ax s))
("waterlogged" nil (w ao1 t er0 l aa1 g d))
("waterloo" nil (w ao1 t er0 l uw1))
("waterman" nil (w ao1 t er0 m ax n))
("watermelon" nil (w ao1 t er0 m eh1 l ax n))
("watermelons" nil (w ao1 t er0 m eh1 l ax n z))
("watermen" nil (w ao1 t er0 m ax n))
("waterproof" nil (w ao1 t er0 p r uw1 f))
("waterproofing" nil (w ao1 t er0 p r uw1 f ih0 ng))
("waters" nil (w ao1 t er0 z))
("watershed" nil (w ao1 t er0 sh eh1 d))
("waterson" nil (w ao1 t er0 s ax n))
("waterstein" nil (w ao1 t er0 s t iy0 n))
("waterston" nil (w ao1 t er0 s t ax n))
("watertown" nil (w ao1 t er0 t aw1 n))
("waterway" nil (w ao1 t er0 w ey1))
("waterways" nil (w ao1 t er0 w ey1 z))
("waterworks" nil (w ao1 t er0 w er1 k s))
("waterworld" nil (w ao1 t er0 w er1 l d))
("waterworth" nil (w ao1 t er0 w er0 th))
("watery" nil (w ao1 t er0 iy0))
("watford" nil (w ao1 t f er0 d))
("wathen" nil (w ao1 th ax n))
("watkin" nil (w ao1 t k ih0 n))
("watkins" nil (w aa1 t k ih0 n z))
("watkinson" nil (w ao1 t k ih0 n s ax n))
("watland" nil (w aa1 t l ax n d))
("watley" nil (w aa1 t l iy0))
("watling" nil (w ao1 t ax l ih0 ng))
("watlington" nil (w ao1 t ax l ih0 ng t ax n))
("watne" nil (w ao1 t n))
("watrous" nil (w aa1 t r ax s))
("watry" nil (w aa1 t r iy0))
("watsco" nil (w aa1 t s k ow1))
("watson" nil (w aa1 t s ax n))
("watsonville" nil (w aa1 t s ax n v ih1 l))
("watt" nil (w aa1 t))
("wattenbarger" nil (w ao1 t ih0 n b aa0 r g er0))
("wattenberg" nil (w aa1 t ax n b er0 g))
("watters" nil (w aa1 t er0 z))
("wattie" nil (w aa1 t iy0))
("wattles" nil (w aa1 t ax l z))
("watton" nil (w ao1 t ax n))
("watts" nil (w aa1 t s))
("wattwil" nil (w aa1 t w ih0 l))
("watwood" nil (w ao1 t w uh0 d))
("watz" nil (w aa1 t s))
("watzman" nil (w aa1 t s m ax n))
("waugaman" nil (w ao1 g ax m ax n))
("waugh" nil (w ao1))
("waukegan" nil (w ao0 k iy1 g ax n))
("waukesha" nil (w ao0 k iy1 sh ax))
("wausau" nil (w ao1 s ao0))
("wauters" nil (w aw1 t er0 z))
("wave" nil (w ey1 v))
("waved" nil (w ey1 v d))
("wavelength" nil (w ey1 v l eh1 ng th))
("wavelengths" nil (w ey1 v l eh1 ng th s))
("waver" nil (w ey1 v er0))
("wavered" nil (w ey1 v er0 d))
("wavering" nil (w ey1 v er0 ih0 ng))
("waverley" nil (w ey1 v er0 l iy0))
("waverly" nil (w ey1 v er0 l iy0))
("waves" nil (w ey1 v z))
("wavetek" nil (w ey1 v t eh1 k))
("waving" nil (w ey1 v ih0 ng))
("wavra" nil (w aa1 v r ax))
("wavy" nil (w ey1 v iy0))
("wawrzyniak" nil (w aa0 r z ih1 n iy0 ae0 k))
("wax" nil (w ae1 k s))
("waxed" nil (w ae1 k s t))
("waxes" nil (w ae1 k s ih0 z))
("waxing" nil (w ae1 k s ih0 ng))
("waxler" nil (w ae1 k s l er0))
("waxman" nil (w ae1 k s m ax n))
("waxy" nil (w ae1 k s iy0))
("way" nil (w ey1))
("waybright" nil (w ey1 b r ay1 t))
("waycaster" nil (w ey1 k ae1 s t er0))
("waye" nil (w ey1))
("waylaid" nil (w ey1 l ey1 d))
("wayland" nil (w ey1 l ax n d))
("wayman" nil (w ey1 m ax n))
("waymire" nil (w ey1 m ay1 r))
("wayne" nil (w ey1 n))
("waynesboro" nil (w ey1 n z b er0 ow0))
("waynesville" nil (w ey1 n z v ih1 l))
("waynick" nil (w ey1 n ih0 k))
("ways" nil (w ey1 z))
("wayside" nil (w ey1 s ay1 d))
("wayson" nil (w ey1 z ax n))
("wayt" nil (w ey1 t))
("wayward" nil (w ey1 w er0 d))
("wazir" nil (w aa0 z ih1 r))
("we" nil (w iy1))
("weak" nil (w iy1 k))
("weaken" nil (w iy1 k ax n))
("weakened" nil (w iy1 k ax n d))
("weakening" nil (w iy1 k ax n ih0 ng))
("weakens" nil (w iy1 k ax n z))
("weaker" nil (w iy1 k er0))
("weakest" nil (w iy1 k ax s t))
("weakland" nil (w iy1 k l ax n d))
("weakley" nil (w iy1 k l iy0))
("weakly" nil (w iy1 k l iy0))
("weakness" nil (w iy1 k n ax s))
("weaknesses" nil (w iy1 k n ax s ax z))
("weaks" nil (w iy1 k s))
("wealth" nil (w eh1 l th))
("wealthier" nil (w eh1 l th iy0 er0))
("wealthiest" nil (w eh1 l th iy0 ax s t))
("wealths" nil (w eh1 l th s))
("wealthy" nil (w eh1 l th iy0))
("wean" nil (w iy1 n))
("weaned" nil (w iy1 n d))
("weaning" nil (w iy1 n ih0 ng))
("weant" nil (w iy1 n t))
("weapon" nil (w eh1 p ax n))
("weaponry" nil (w eh1 p ax n r iy0))
("weapons" nil (w eh1 p ax n z))
("wear" nil (w eh1 r))
("wearer" nil (w eh1 r er0))
("wearers" nil (w eh1 r er0 z))
("wearever" nil (w eh1 r eh1 v er0))
("wearhouse" nil (w eh1 r hh aw1 s))
("wearied" nil (w ih1 r iy0 d))
("wearily" nil (w eh1 r ax l iy0))
("weariness" nil (w ih1 r iy0 n ax s))
("wearing" nil (w eh1 r ih0 ng))
("wearisome" nil (w ih1 r iy0 s ax m))
("wears" nil (w eh1 r z))
("weary" nil (w ih1 r iy0))
("wearying" nil (w iy1 r iy0 ih0 ng))
("wease" nil (w iy1 z))
("weasel" nil (w iy1 z ax l))
("weasellike" nil (w iy1 z ax l l ay1 k))
("weast" nil (w iy1 s t))
("weather" nil (w eh1 dh er0))
("weatherall" nil (w eh1 dh er0 ao1 l))
("weatherbee" nil (w eh1 dh er0 b iy1))
("weatherby" nil (w eh1 dh er0 b iy0))
("weathered" nil (w eh1 dh er0 d))
("weatherford" nil (w eh1 dh er0 f er0 d))
("weatherhead" nil (w eh1 dh er0 hh eh1 d))
("weatherholt" nil (w eh1 dh er0 hh ow1 l t))
("weatherholtz" nil (w eh1 dh er0 hh ow1 l t s))
("weathering" nil (w eh1 dh er0 ih0 ng))
("weatherington" nil (w eh1 dh er0 ih0 ng t ax n))
("weatherly" nil (w eh1 dh er0 l iy0))
("weatherman" nil (w eh1 dh er0 m ae1 n))
("weathermen" nil (w eh1 dh er0 m eh1 n))
("weatherperson" nil (w eh1 th er0 p er1 s ax n))
("weatherproof" nil (w eh1 dh er0 p r uw1 f))
("weathers" nil (w eh1 dh er0 z))
("weathersbee" nil (w eh1 dh er0 z b iy1))
("weathersby" nil (w eh1 dh er0 s b iy0))
("weatherspoon" nil (w eh1 dh er0 s p uw1 n))
("weatherstone" nil (w eh1 dh er0 s t ow1 n))
("weatherwax" nil (w eh1 dh er0 w ae1 k s))
("weathington" nil (w eh1 th ih0 ng t ax n))
("weave" nil (w iy1 v))
("weaved" nil (w iy1 v d))
("weaver" nil (w iy1 v er0))
("weaverling" nil (w iy1 v er0 l ih0 ng))
("weavers" nil (w iy1 v er0 z))
("weaves" nil (w iy1 v z))
("weavil" nil (w iy1 v ax l))
("weaving" nil (w iy1 v ih0 ng))
("weavings" nil (w iy1 v ih0 ng z))
("web" nil (w eh1 b))
("webb" nil (w eh1 b))
("webbed" nil (w eh1 b d))
("webber" nil (w eh1 b er0))
("webbing" nil (w eh1 b ih0 ng))
("webbs" nil (w eh1 b z))
("weber" nil (w eh1 b er0))
("weberg" nil (w eh1 b er0 g))
("webers" nil (w eh1 b er0 z))
("webley" nil (w eh1 b l iy0))
("webre" nil (w eh1 b er0))
("webs" nil (w eh1 b z))
("webster" nil (w eh1 b s t er0))
("wechsel" nil (w eh1 k s ax l))
("wechsler" nil (w eh1 k s l er0))
("wecht" nil (w eh1 k t))
("wechter" nil (w eh1 k t er0))
("weck" nil (w eh1 k))
("wecker" nil (w eh1 k er0))
("weckerly" nil (w eh1 k er0 l iy0))
("weckesser" nil (w eh1 k ih0 s er0))
("weckwerth" nil (w eh1 k w er0 th))
("wed" nil (w eh1 d))
("wedbush" nil (w eh1 d b uh1 sh))
("wedd" nil (w eh1 d))
("wedded" nil (w eh1 d ax d))
("weddell" nil (w eh1 d ax l))
("wedding" nil (w eh1 d ih0 ng))
("weddings" nil (w eh1 d ih0 ng z))
("weddington" nil (w eh1 d ih0 ng t ax n))
("weddle" nil (w eh1 d ax l))
("wedekind" nil (w eh1 d ih0 k ih0 n d))
("wedeking" nil (w eh1 d ih0 k ih0 ng))
("wedel" nil (w eh1 d ax l))
("wedell" nil (w eh1 d ax l))
("wedemeyer" nil (w eh1 d ih0 m ay1 er0))
("wedge" nil (w eh1 jh))
("wedged" nil (w eh1 jh d))
("wedges" nil (w eh1 jh ax z))
("wedgestone" nil (w eh1 jh s t ow1 n))
("wedgewood" nil (w eh1 jh w uh1 d))
("wedgeworth" nil (w eh1 jh w er1 th))
("wedgwood" nil (w eh1 jh w uh1 d))
("wedig" nil (w eh1 d ih0 g))
("wedin" nil (w eh1 d ih0 n))
("wedlock" nil (w eh1 d l aa1 k))
("wednesday" nil (w eh1 n z d iy0))
("wednesdays" nil (w eh1 n z d ey1 z))
("wedowee" nil (w eh0 d aw1 w iy1))
("wedtech" nil (w eh1 d t eh1 k))
("wee" nil (w iy1))
("weeber" nil (w iy1 b er0))
("weech" nil (w iy1 ch))
("weed" nil (w iy1 d))
("weeded" nil (w iy1 d ih0 d))
("weeden" nil (w iy1 d ax n))
("weeding" nil (w iy1 d ih0 ng))
("weedman" nil (w iy1 d m ax n))
("weedon" nil (w iy1 d ax n))
("weeds" nil (w iy1 d z))
("weedy" nil (w iy1 d iy0))
("weehawken" nil (w iy1 hh ao1 k ax n))
("week" nil (w iy1 k))
("weekday" nil (w iy1 k d ey1))
("weekdays" nil (w iy1 k d ey1 z))
("weekend" nil (w iy1 k eh1 n d))
("weekender" nil (w iy1 k eh1 n d er0))
("weekends" nil (w iy1 k eh1 n d z))
("weekes" nil (w iy1 k s))
("weekley" nil (w iy1 k l iy0))
("weeklies" nil (w iy1 k l iy0 z))
("weeklong" nil (w iy1 k l ao1 ng))
("weekly" nil (w iy1 k l iy0))
("weeks" nil (w iy1 k s))
("weems" nil (w iy1 m z))
("weeny" nil (w iy1 n iy0))
("weep" nil (w iy1 p))
("weeping" nil (w iy1 p ih0 ng))
("weepy" nil (w iy1 p iy0))
("weers" nil (w iy1 er0 z))
("weerts" nil (w ih1 r t s))
("weese" nil (w iy1 z))
("weesner" nil (w iy1 z n er0))
("weevil" nil (w iy1 v ax l))
("weevils" nil (w iy1 v ax l z))
("wegbreit" nil (w eh1 g b r ay1 t))
("wege" nil (w iy1 jh))
("wegener" nil (w eh1 g iy0 n er0))
("weger" nil (w iy1 g er0))
("weglarz" nil (w eh1 g l aa0 r z))
("wegman" nil (w eh1 g m ax n))
("wegmann" nil (w eh1 g m ax n))
("wegner" nil (w eh1 g n er0))
("wegrzyn" nil (w eh1 g er0 z ih0 n))
("wehde" nil (w eh1 d))
("wehe" nil (w iy1 hh))
("wehle" nil (w eh1 l iy0))
("wehler" nil (w eh1 l er0))
("wehling" nil (w eh1 l ih0 ng))
("wehman" nil (w eh1 m ax n))
("wehmeier" nil (w eh1 m ay0 er0))
("wehmeyer" nil (w eh1 m ay0 er0))
("wehner" nil (w eh1 n er0))
("wehr" nil (w eh1 r))
("wehrenberg" nil (w eh1 r ax n b er0 g))
("wehrheim" nil (w eh1 r hh ay0 m))
("wehrle" nil (w eh1 r ax l))
("wehrli" nil (w eh1 r l iy0))
("wehrly" nil (w eh1 r l iy0))
("wehrman" nil (w eh1 r m ax n))
("wehrmann" nil (w eh1 r m ax n))
("wehunt" nil (w eh1 hh ax n t))
("wei" nil (w ey1))
("weiand" nil (w ay1 ax n d))
("weibel" nil (w ay1 b ax l))
("weible" nil (w iy1 b ax l))
("weich" nil (w ay1 k))
("weichel" nil (w ay1 k ax l))
("weichert" nil (w ay1 k er0 t))
("weichman" nil (w ay1 k m ax n))
("weicht" nil (w ay1 k t))
("weick" nil (w iy1 k))
("weicker" nil (w ay1 k er0))
("weida" nil (v ay1 d ax))
("weide" nil (w ay1 d iy0))
("weidel" nil (w ay1 d ax l))
("weideman" nil (w ay1 d m ax n))
("weidemann" nil (w ay1 d m ax n))
("weidenbach" nil (w ay1 d ih0 n b aa0 k))
("weidenbaum" nil (w ay1 d ax n b aw1 m))
("weidenfeld" nil (w ay1 d ax n f eh1 l d))
("weider" nil (w ay1 d er0))
("weidig" nil (w ay1 d ih0 g))
("weidinger" nil (w ay1 d ih0 ng er0))
("weidler" nil (w ay1 d ax l er0))
("weidlich" nil (w ay1 d l ih0 k))
("weidman" nil (w ay1 d m ax n))
("weidmann" nil (w ay1 d m ax n))
("weidner" nil (w ay1 d n er0))
("weier" nil (w ey1 er0))
("weigand" nil (w ay1 g ax n d))
("weigandt" nil (w ay1 g ax n t))
("weigel" nil (w ay1 g ax l))
("weigelt" nil (w ay1 g ih0 l t))
("weigert" nil (w ay1 g er0 t))
("weigh" nil (w ey1))
("weighed" nil (w ey1 d))
("weighing" nil (w ey1 ih0 ng))
("weighs" nil (w ey1 z))
("weight" nil (w ey1 t))
("weighted" nil (w ey1 t ih0 d))
("weighting" nil (w ey1 t ih0 ng))
("weightings" nil (w ey1 t ih0 ng z))
("weightless" nil (w ey1 t l ax s))
("weightlessness" nil (w ey1 t l ax s n ax s))
("weightlifting" nil (w ey1 t l ih1 f t ih0 ng))
("weightman" nil (w ay1 t m ax n))
("weights" nil (w ey1 t s))
("weighty" nil (w ey1 t iy0))
("weigl" nil (w iy1 g ax l))
("weigle" nil (w iy1 g ax l))
("weigold" nil (w ay1 g ow1 l d))
("weihe" nil (w iy1 hh))
("weiher" nil (w ay1 hh er0))
("weik" nil (w iy1 k))
("weikel" nil (w ay1 k ax l))
("weiker" nil (w ay1 k er0))
("weikert" nil (w ay1 k er0 t))
("weikle" nil (w iy1 k ax l))
("weil" nil (w ay1 l))
("weiland" nil (w ay1 l ax n d))
("weilbacher" nil (w ay1 l b aa1 k er0))
("weild" nil (w ay1 l d))
("weiler" nil (w ay1 l er0))
("weill" nil (w ay1 l))
("weiman" nil (w ay1 m ax n))
("weimann" nil (w ay1 m ax n))
("weimar" nil (v ay1 m aa0 r))
("weimer" nil (w ay1 m er0))
("wein" nil (w iy1 n))
("weinand" nil (w ay1 n ax n d))
("weinbach" nil (w ay1 n b aa1 k))
("weinbaum" nil (w ay1 n b aw1 m))
("weinberg" nil (w ay1 n b er0 g))
("weinberger" nil (w ay1 n b er0 g er0))
("weindel" nil (w ay1 n d ax l))
("weinel" nil (w ay1 n ax l))
("weiner" nil (w ay1 n er0))
("weinert" nil (w ay1 n er0 t))
("weinfeld" nil (w ay1 n f eh1 l d))
("weingart" nil (w ay1 ng g aa0 r t))
("weingarten" nil (w ay1 n g aa1 r t ax n))
("weingartner" nil (w ay1 ng g aa0 r t n er0))
("weinger" nil (w ay1 n g er0))
("weinheimer" nil (w ay1 n hh ay1 m er0))
("weinhold" nil (w ay1 n hh ow1 l d))
("weininger" nil (w ay1 n ih0 ng er0))
("weinkauf" nil (w ay1 ng k aw0 f))
("weinman" nil (w ay1 n m ax n))
("weinmann" nil (w ay1 n m ax n))
("weinreb" nil (w ay1 n r ih0 b))
("weinreich" nil (w ay1 n r ay1 k))
("weinrich" nil (w ay1 n r ih0 k))
("weinroth" nil (w ay1 n r ao1 th))
("weins" nil (w iy1 n z))
("weinstein" nil (w ay1 n s t ay1 n))
("weinstock" nil (w ay1 n s t aa1 k))
("weintraub" nil (w ay1 n t r aw1 b))
("weintz" nil (w ay1 n t s))
("weinzierl" nil (w ay1 n z ih0 r l))
("weipert" nil (w ay1 p er0 t))
("weir" nil (w iy1 r))
("weirauch" nil (w ay1 r aw0 k))
("weird" nil (w ih1 r d))
("weirder" nil (w ih1 r d er0))
("weirdest" nil (w ih1 r d ax s t))
("weirdly" nil (w ih1 r d l iy0))
("weirdness" nil (w ih1 r d n ax s))
("weirdo" nil (w ih1 r d ow0))
("weirdos" nil (w ih1 r d ow0 z))
("weired" nil (w ay1 r d))
("weirich" nil (w ay1 r ih0 k))
("weirick" nil (w ay1 r ih0 k))
("weirton" nil (w ih1 r t ax n))
("weis" nil (w ay1 s))
("weisbach" nil (w ay1 s b aa1 k))
("weisbecker" nil (w ay1 s b eh1 k er0))
("weisberg" nil (w ay1 s b er0 g))
("weisberger" nil (w ay1 s b er0 g er0))
("weisbrod" nil (w ay1 s b r aa1 d))
("weisbrot" nil (w ay1 s b r ax t))
("weischedel" nil (w ay1 sh ax d ax l))
("weise" nil (w iy1 s))
("weisel" nil (w ay1 s ax l))
("weisenbach" nil (w ay1 s ih0 n b aa0 k))
("weisenberg" nil (w ay1 s ax n b er0 g))
("weisenberger" nil (w ay1 s ax n b er0 g er0))
("weisenborn" nil (w ay1 s ih0 n b er0 n))
("weisenburger" nil (w ay1 s ax n b er0 g er0))
("weisensel" nil (w ay1 s ih0 n s ax l))
("weiser" nil (w ay1 s er0))
("weisfeld" nil (w ay1 s f eh1 l d))
("weisgerber" nil (w ay1 s g er0 b er0))
("weishaar" nil (w ay1 sh aa0 r))
("weishaupt" nil (w ay1 sh aw0 p t))
("weisheit" nil (w ay1 sh ay0 t))
("weisinger" nil (w ay1 s ih0 n jh er0))
("weiskopf" nil (w ay1 s k ao0 f))
("weisman" nil (w ay1 s m ax n))
("weismann" nil (w ay1 s m ax n))
("weisner" nil (w ay1 s n er0))
("weiss" nil (w ay1 s))
("weissberg" nil (w ay1 s b er0 g))
("weisse" nil (w ay1 s))
("weissenborn" nil (w ay1 s ih0 n b er0 n))
("weisser" nil (w ay1 s er0))
("weissert" nil (w ay1 s er0 t))
("weissinger" nil (w ay1 s ih0 n jh er0))
("weissman" nil (w ay1 s m ax n))
("weissmann" nil (w ay1 s m ax n))
("weist" nil (w iy1 ih0 s t))
("weisz" nil (v ay1 sh))
("weitek" nil (w ey1 t eh1 k))
("weith" nil (w iy1 th))
("weitkamp" nil (w ay1 t k ae1 m p))
("weitler" nil (w ay1 t l er0))
("weitman" nil (w ay1 t m ax n))
("weitz" nil (w iy1 t s))
("weitzel" nil (w ay1 t z ax l))
("weitzen" nil (w ay1 t s ax n))
("weitzman" nil (w ay1 t s m ax n))
("weitzner" nil (w ay1 t s n er0))
("weizsaecker" nil (w ay1 s ae1 k er0))
("weksel" nil (w eh1 k s ax l))
("welbilt" nil (w eh1 l b ih1 l t))
("welborn" nil (w eh1 l b er0 n))
("welborne" nil (w eh1 l b er0 n))
("welby" nil (w eh1 l b iy0))
("welch" nil (w eh1 l ch))
("welchel" nil (w eh1 l ch ax l))
("welcher" nil (w eh1 l ch er0))
("welcome" nil (w eh1 l k ax m))
("welcomed" nil (w eh1 l k ax m d))
("welcomes" nil (w eh1 l k ax m z))
("welcoming" nil (w eh1 l k ax m ih0 ng))
("weld" nil (w eh1 l d))
("welded" nil (w eh1 l d ih0 d))
("welden" nil (w eh1 l d ax n))
("welder" nil (w eh1 l d er0))
("welders" nil (w eh1 l d er0 z))
("weldin" nil (w eh1 l d ih0 n))
("welding" nil (w eh1 l d ih0 ng))
("weldon" nil (w eh1 l d ax n))
("welds" nil (w eh1 l d z))
("weldwood" nil (w eh1 l d w uh1 d))
("weldy" nil (w eh1 l d iy0))
("welfare" nil (w eh1 l f eh1 r))
("welford" nil (w eh1 l f er0 d))
("welge" nil (w eh1 l jh))
("welinder" nil (w eh1 l ih1 n d er0))
("welk" nil (w eh1 l k))
("welke" nil (w eh1 l k))
("welker" nil (w eh1 l k er0))
("well" nil (w eh1 l))
("welland" nil (w eh1 l ax n d))
("wellborn" nil (w eh1 l b ao1 r n))
("wellbrock" nil (w eh1 l b r ax k))
("wellco" nil (w eh1 l k ow0))
("wellcome" nil (w eh1 l k ah1 m))
("welle" nil (w eh1 l))
("wellek" nil (w eh1 l eh0 k))
("wellen" nil (w eh1 l ax n))
("wellendorf" nil (w eh1 l ih0 n d ao0 r f))
("wellens" nil (w eh1 l ax n z))
("weller" nil (w eh1 l er0))
("welles" nil (w eh1 l z))
("wellesley" nil (w eh1 l z l iy0))
("wellfleet" nil (w eh1 l f l iy1 t))
("wellhead" nil (w eh1 l hh eh1 d))
("welling" nil (w eh1 l ih0 ng))
("wellington" nil (w eh1 l ih0 ng t ax n))
("welliver" nil (w eh1 l ih0 v er0))
("wellman" nil (w eh1 l m ax n))
("wellner" nil (w eh1 l n er0))
("wellness" nil (w eh1 l n ax s))
("wellnitz" nil (w eh1 l n ih0 t s))
("wellons" nil (w eh1 l ax n z))
("wellpoint" nil (w eh1 l p oy1 n t))
("wells" nil (w eh1 l z))
("wellspring" nil (w eh1 l s p r ih1 ng))
("wellstone" nil (w eh1 l s ow1 n))
("welltech" nil (w eh1 l t eh1 k))
("welna" nil (w eh1 l n ax))
("welp" nil (w eh1 l p))
("welsch" nil (w eh1 l sh))
("welser" nil (w eh1 l s er0))
("welsh" nil (w eh1 l ch))
("welshans" nil (w eh1 l sh ax n z))
("welt" nil (w eh1 l t))
("welte" nil (w eh1 l t))
("welter" nil (w eh1 l t er0))
("welterweight" nil (w eh1 l t er0 w ey1 t))
("weltman" nil (w eh1 l t m ax n))
("welton" nil (w eh1 l t ax n))
("welty" nil (w eh1 l t iy0))
("weltz" nil (w eh1 l t s))
("welz" nil (w eh1 l z))
("wembley" nil (w eh1 m b l iy0))
("wemhoff" nil (w eh1 m hh ao1 f))
("wemmer" nil (w eh1 m er0))
("wempe" nil (w eh1 m p))
("wemple" nil (w eh1 m p ax l))
("wen" nil (w eh1 n))
("wenatchee" nil (w ax n ae1 ch iy0))
("wenberg" nil (w eh1 n b er0 g))
("wenceslaus" nil (w eh1 n s ax s l ao1 s))
("wenches" nil (w eh1 n ch ih0 z))
("wenchester" nil (w eh1 n ch eh1 s t er0))
("wenck" nil (w eh1 ng k))
("wend" nil (w eh1 n d))
("wenda" nil (w eh1 n d ax))
("wende" nil (w eh1 n d))
("wendel" nil (w eh1 n d ax l))
("wendeline" nil (w eh1 n d ih0 l ay1 n))
("wendelken" nil (w eh1 n d ih0 l k ax n))
("wendell" nil (w eh1 n d ax l))
("wender" nil (w eh1 n d er0))
("wenders" nil (w eh1 n d er0 z))
("wending" nil (w eh1 n d ih0 ng))
("wendland" nil (w eh1 n d l ax n d))
("wendlandt" nil (w eh1 n d l ax n t))
("wendler" nil (w eh1 n d l er0))
("wendling" nil (w eh1 n d l ih0 ng))
("wendorf" nil (w eh1 n d ao0 r f))
("wendorff" nil (w eh1 n d ao0 r f))
("wends" nil (w eh1 n d z))
("wendt" nil (w eh1 n t))
("wendy" nil (w eh1 n d iy0))
("wenfan" nil (w eh1 n f ae1 n))
("weng" nil (w eh1 ng))
("wenge" nil (w eh1 n jh))
("wenger" nil (w eh1 ng er0))
("wengerd" nil (w eh1 ng g er0 d))
("wengert" nil (w eh1 ng g er0 t))
("wengler" nil (w ih1 ng ax l er0))
("wenig" nil (w eh1 n ih0 g))
("weniger" nil (w eh1 n ih0 g er0))
("weninger" nil (w eh1 n ih0 ng er0))
("wenk" nil (w eh1 ng k))
("wenke" nil (w eh1 ng k))
("wenker" nil (w eh1 ng k er0))
("wennberg" nil (w eh1 n b er0 g))
("wenner" nil (w eh1 n er0))
("wennerstrom" nil (w eh1 n er0 s t r ax m))
("wenning" nil (w eh1 n ih0 ng))
("wenninger" nil (w eh1 n ih0 ng er0))
("wenona" nil (w eh1 n ax n ax))
("wenrich" nil (w eh1 n r ih0 k))
("wenrick" nil (w eh1 n r ih0 k))
("wensberg" nil (w eh1 n z b er0 g))
("wensel" nil (w eh1 n s ax l))
("wenstrom" nil (w eh1 n s t r ax m))
("went" nil (w eh1 n t))
("wente" nil (w eh1 n t))
("wentland" nil (w eh1 n t l ax n d))
("wentling" nil (w eh1 n t l ih0 ng))
("wentworth" nil (w eh1 n t w er1 th))
("wentz" nil (w eh1 n t s))
("wentzel" nil (w eh1 n t z ax l))
("wentzell" nil (w eh1 n t z ax l))
("wentzville" nil (w eh1 n t s v ih1 l))
("wenz" nil (w eh1 n z))
("wenzel" nil (w eh1 n z ax l))
("wenzhou" nil (w eh0 n z uw1))
("wenzl" nil (w eh1 n z ax l))
("wenzler" nil (w eh1 n z l er0))
("wenzlick" nil (w eh1 n z l ih0 k))
("weppler" nil (w eh1 p l er0))
("wept" nil (w eh1 p t))
("werber" nil (w er1 b er0))
("werden" nil (w er1 d ax n))
("werder" nil (w er1 d er0))
("werdesheim" nil (w er1 d z hh ay1 m))
("were" nil (w er0))
("wereldhave" nil (w eh1 r ax l d hh aa1 v ey0))
("werewolf" nil (w eh1 r w uh1 l f))
("wergin" nil (w er1 g ih0 n))
("werk" nil (w er1 k))
("werke" nil (w er1 k))
("werkheiser" nil (w er1 k hh ay0 s er0))
("werking" nil (w er1 k ih0 ng))
("werkmeister" nil (w er1 k m ay0 s t er0))
("werle" nil (w ao1 r ax l))
("werley" nil (w er1 l iy0))
("werling" nil (w er1 l ih0 ng))
("werman" nil (w er1 m ax n))
("wermiel" nil (w er0 m iy0 ax l))
("wermuth" nil (w er0 m uw1 th))
("wernecke" nil (w er1 n ih0 k))
("werner" nil (w er1 n er0))
("wernert" nil (w er1 n er0 t))
("wernet" nil (w er1 n ih0 t))
("wernette" nil (w er0 n eh1 t))
("wernick" nil (w er1 n ih0 k))
("wernicke" nil (w er1 n ih0 k))
("wernimont" nil (w er1 n ih0 m ax n t))
("werning" nil (w er1 n ih0 ng))
("wernli" nil (w er1 n l iy0))
("werntz" nil (w er1 n t s))
("werre" nil (w eh1 r))
("werry" nil (w eh1 r iy0))
("werst" nil (w er1 s t))
("wert" nil (w er1 t))
("wertenberger" nil (w er1 t ax n b er0 g er0))
("werth" nil (w er1 th))
("wertheim" nil (w er1 t hh ay0 m))
("wertheimer" nil (w er1 t hh ay0 m er0))
("werther" nil (w er1 dh er0))
("wertman" nil (w er1 t m ax n))
("werts" nil (w er1 t s))
("wertz" nil (w er1 t s))
("wery" nil (w eh1 r iy0))
("wes" nil (w eh1 s))
("wesch" nil (w eh1 sh))
("wesche" nil (w eh1 sh))
("wesco" nil (w eh1 s k ow0))
("wescoat" nil (w eh1 s k ow1 t))
("wescott" nil (w eh1 s k aa0 t))
("weseloh" nil (w ey0 s ey1 l ow0))
("wesely" nil (w iy1 z l iy0))
("weseman" nil (w iy1 z m ax n))
("wesemann" nil (w iy1 z m ax n))
("wesenberg" nil (w iy1 z ax n b er0 g))
("wesker" nil (w eh1 s k er0))
("wesler" nil (w eh1 s ax l er0))
("wesley" nil (w eh1 s l iy0))
("wesleyan" nil (w eh1 z l iy0 ax n))
("wesner" nil (w eh1 s n er0))
("wesolek" nil (w eh1 s ax l ih0 k))
("wesolowski" nil (v ih0 s ax l ao1 f s k iy0))
("wesp" nil (w eh1 s p))
("wespac" nil (w eh1 s p ae1 k))
("wespercorp" nil (w eh1 s p er0 k ao1 r p))
("wesray" nil (w eh1 s r ey0))
("wessel" nil (w eh1 s ax l))
("wessell" nil (w eh1 s ax l))
("wessells" nil (w eh1 s ax l z))
("wesselman" nil (w eh1 s ax l m ax n))
("wessels" nil (w eh1 s ax l z))
("wessex" nil (w eh1 s ax k s))
("wessinger" nil (w eh1 s ih0 n jh er0))
("wessler" nil (w eh1 s l er0))
("wessling" nil (w eh1 s ax l ih0 ng))
("wessman" nil (w eh1 s m ax n))
("wessner" nil (w eh1 s n er0))
("wesson" nil (w eh1 s ax n))
("west" nil (w eh1 s t))
("westair" nil (w eh1 s t eh1 r))
("westall" nil (w eh1 s t ax l))
("westamerica" nil (w eh1 s t ax m eh1 r ih0 k ax))
("westar" nil (w eh1 s t er0))
("westbay" nil (w eh1 s t b ey1))
("westberg" nil (w eh1 s t b er0 g))
("westberry" nil (w eh1 s t b eh1 r iy0))
("westboro" nil (w eh1 s t b er0 ow0))
("westborough" nil (w eh1 s t b er0 ow0))
("westbound" nil (w eh1 s t b aw1 n d))
("westbridge" nil (w eh1 s t b r ih1 jh))
("westbrook" nil (w eh1 s t b r uh1 k))
("westbrooks" nil (w eh1 s t b r uh1 k s))
("westburne" nil (w eh1 s t b er0 n))
("westbury" nil (w eh1 s t b eh1 r iy0))
("westby" nil (w eh1 s t b iy0))
("westcap" nil (w eh1 s t k ae1 p))
("westchester" nil (w eh1 s t ch eh1 s t er0))
("westcoast" nil (w eh1 s t k ow1 s t))
("westcorp" nil (w eh1 s t k ao1 r p))
("westcott" nil (w eh1 s t k aa1 t))
("westdeutsche" nil (w eh1 s t d oy1 ch))
("westen" nil (w eh1 s t ax n))
("westenberger" nil (w eh1 s t ax n b er0 g er0))
("westendorf" nil (w eh1 s t ih0 n d ao0 r f))
("wester" nil (w eh1 s t er0))
("westerbeck" nil (w eh1 s t er0 b eh1 k))
("westerberg" nil (w eh1 s t er0 b er0 g))
("westerfeld" nil (w eh1 s t er0 f eh1 l d))
("westerfield" nil (w eh1 s t er0 f iy1 l d))
("westergaard" nil (w eh1 s t er0 g aa1 r d))
("westergard" nil (w eh1 s t er0 g er0 d))
("westergren" nil (w eh1 s t er0 g r eh0 n))
("westerhoff" nil (w eh1 s t er0 hh ao1 f))
("westerhold" nil (w eh1 s t er0 hh ow1 l d))
("westerlund" nil (w eh1 s t er0 l ax n d))
("westerly" nil (w eh1 s t er0 l iy0))
("westerman" nil (w eh1 s t er0 m ax n))
("westermann" nil (w eh1 s t er0 m ax n))
("westermeyer" nil (w eh1 s t er0 m ay0 er0))
("western" nil (w eh1 s t er0 n))
("westerner" nil (w eh1 s t er0 n er0))
("westerners" nil (w eh1 s t er0 n er0 z))
("westernization" nil (w eh1 s t er0 n ih0 z ey1 sh ax n))
("westernize" nil (w eh1 s t er0 n ay1 z))
("westernized" nil (w eh1 s t er0 n ay1 z d))
("westernmost" nil (w eh1 s t er0 n m ow1 s t))
("westerns" nil (w eh1 s t er0 n z))
("westervelt" nil (w eh1 s t er0 v ih0 l t))
("westfaelisches" nil (w eh1 s t f ey1 l ih0 sh ih0 z))
("westfahl" nil (w eh1 s t f aa1 l))
("westfall" nil (w eh1 s t f ao1 l))
("westfed" nil (w eh1 s t f eh1 d))
("westfield" nil (w eh1 s t f iy0 l d))
("westford" nil (w eh1 s t f er0 d))
("westgate" nil (w eh1 s t g ey1 t))
("westheimer" nil (w eh1 s t hh ay1 m er0))
("westhoff" nil (w eh1 s t hh ao1 f))
("westin" nil (w eh1 s t ih0 n))
("westinghouse" nil (w eh1 s t ih0 ng hh aw1 s))
("westlake" nil (w eh1 s t l ey1 k))
("westland" nil (w eh1 s t l ax n d))
("westleigh" nil (w eh1 s t l ay0))
("westley" nil (w eh1 s t l iy0))
("westling" nil (w eh1 s t l ih0 ng))
("westlund" nil (w eh1 s t l ax n d))
("westman" nil (w eh1 s t m ax n))
("westmarc" nil (w eh1 s t m aa1 r k))
("westmark" nil (w eh1 s t m aa1 r k))
("westmin" nil (w eh1 s t m ih0 n))
("westminster" nil (w eh1 s t m ih1 n s t er0))
("westmoreland" nil (w eh0 s t m ao1 r l ax n d))
("weston" nil (w eh1 s t ax n))
("westover" nil (w eh1 s t ow1 v er0))
("westpac" nil (w eh1 s t p ae1 k))
("westphal" nil (w eh1 s t f ax l))
("westphalen" nil (w eh1 s t f ax l ax n))
("westphalia" nil (w eh1 s t f ey1 l iy0 ax))
("westport" nil (w eh1 s t p ao1 r t))
("westpride" nil (w eh1 s t p r ay1 d))
("westra" nil (w eh1 s t r ax))
("westrich" nil (w eh1 s t r ih0 k))
("westrick" nil (w eh1 s t r ih0 k))
("westridge" nil (w eh1 s t r ih1 jh))
("westrom" nil (w eh1 s t r ax m))
("westrum" nil (w eh1 s t r ax m))
("westrup" nil (w eh1 s t r ax p))
("westside" nil (w eh1 s t s ay1 d))
("westtech" nil (w eh1 s t eh1 k))
("westvaco" nil (w eh1 s t v ae1 k ow0))
("westward" nil (w eh1 s t w er0 d))
("westwards" nil (w eh1 s t w er0 d z))
("westwood" nil (w eh1 s t w uh1 d))
("westworld" nil (w eh1 s t w er1 l d))
("wesun" nil (w iy1 s ah1 n))
("wet" nil (w eh1 t))
("wetherbee" nil (w eh1 dh er0 b iy1))
("wetherby" nil (w eh1 th er0 b iy0))
("wetherell" nil (w eh1 th er0 ax l))
("wetherill" nil (w eh1 th er0 ax l))
("wetherington" nil (w eh1 dh er0 ih0 ng t ax n))
("wetherly" nil (w eh1 dh er0 l iy0))
("wethington" nil (w eh1 th ih0 ng t ax n))
("wetland" nil (w eh1 t l ae1 n d))
("wetlands" nil (w eh1 t l ae1 n d z))
("wetmore" nil (w eh1 t m ao0 r))
("wetness" nil (w eh1 t n ax s))
("wetsel" nil (w eh1 t s ax l))
("wetstein" nil (w eh1 t s t iy1 n))
("wettengel" nil (w eh1 t ih0 ng g ax l))
("wetter" nil (w eh1 t er0))
("wetterau" nil (w eh1 t er0 aw0))
("wettergreen" nil (w eh1 t er0 g r iy1 n))
("wettest" nil (w eh1 t ax s t))
("wettlaufer" nil (w eh1 t l aw0 f er0))
("wettstein" nil (w eh1 t s t ay0 n))
("wetz" nil (w eh1 t s))
("wetzel" nil (w eh1 t z ax l))
("wetzler" nil (w eh1 t s l er0))
("wetzstein" nil (w eh1 t s t ay0 n))
("wever" nil (w iy1 v er0))
("wexler" nil (w eh1 k s l er0))
("wexner" nil (w eh1 k s n er0))
("wey" nil (w ey1))
("weyand" nil (w ey1 ax n d))
("weyandt" nil (w ey1 ax n t))
("weyant" nil (w ey1 ax n t))
("weyenberg" nil (w ay1 n b er0 g))
("weyer" nil (w ey1 er0))
("weyerhaeuser" nil (w eh1 r hh aw1 z er0))
("weyers" nil (w ey1 er0 z))
("weyforth" nil (w ey1 f ao1 r th))
("weygandt" nil (w ey1 g ax n t))
("weyhrauch" nil (w ey1 r aw1 k))
("weyl" nil (w ey1 l))
("weyland" nil (w ey1 l ax n d))
("weylin" nil (w ey1 l ih0 n))
("weyman" nil (w ey1 m ax n))
("weymouth" nil (w ey1 m ax th))
("weyrauch" nil (w eh1 r aw0 k))
("weyrich" nil (w eh1 r ih0 k))
("weyrick" nil (w ey1 r ih0 k))
("whack" nil (w ae1 k))
("whacked" nil (w ae1 k t))
("whacking" nil (w ae1 k ih0 ng))
("whacks" nil (w ae1 k s))
("whale" nil (w ey1 l))
("whalen" nil (w ey1 l ax n))
("whaler" nil (w ey1 l er0))
("whales" nil (w ey1 l z))
("whaley" nil (w ey1 l iy0))
("whalin" nil (w ae1 l ih0 n))
("whaling" nil (w ey1 l ih0 ng))
("whalley" nil (w ae1 l iy0))
("wham" nil (w ae1 m))
("whammy" nil (w ae1 m iy0))
("whampoa" nil (w ae0 m p ow1 ax))
("whan" nil (w ae1 n))
("whang" nil (w ae1 ng))
("wharf" nil (w ao1 r f))
("wharff" nil (w aa1 r f))
("wharry" nil (w ae1 r iy0))
("wharton" nil (w ao1 r t ax n))
("what" nil (w ah1 t))
("whatever" nil (w ah1 t eh1 v er0))
("whatley" nil (w ah1 t l iy0))
("whatnot" nil (w ah1 t n aa1 t))
("whats" nil (w ax t s))
("whatsoever" nil (w ah1 t s ow0 eh1 v er0))
("wheat" nil (w iy1 t))
("wheaten" nil (w iy1 t ax n))
("wheaties" nil (w iy1 t iy0 z))
("wheatley" nil (w iy1 t l iy0))
("wheaton" nil (w iy1 t ax n))
("whedbee" nil (w eh1 d b iy1))
("whedon" nil (w eh1 d ax n))
("wheel" nil (w iy1 l))
("wheelabrator" nil (w iy1 l ax b r ey1 t er0))
("wheelan" nil (w iy1 l ax n))
("wheeland" nil (w iy1 l ax n d))
("wheelbarrow" nil (w iy1 l b eh1 r ow0))
("wheelbarrows" nil (w iy1 l b eh1 r ow0 z))
("wheelbase" nil (w iy1 l b ey1 s))
("wheelchair" nil (w iy1 l ch eh1 r))
("wheelchairs" nil (w iy1 l ch eh1 r z))
("wheeldon" nil (w iy1 l d ax n))
("wheeled" nil (w iy1 l d))
("wheeler" nil (w iy1 l er0))
("wheelers" nil (w iy1 l er0 z))
("wheeless" nil (w iy1 l ax s))
("wheeling" nil (w iy1 l ih0 ng))
("wheelis" nil (w iy1 l ih0 s))
("wheelock" nil (w iy1 l aa1 k))
("wheelon" nil (w iy1 l ax n))
("wheels" nil (w iy1 l z))
("wheelus" nil (w iy1 l ax s))
("wheelwright" nil (w iy1 l r ay1 t))
("wheelwriter" nil (w iy1 l r ay1 t er0))
("wheezes" nil (w iy1 z ax z))
("wheezing" nil (w iy1 z ih0 ng))
("whelan" nil (w eh1 l ax n))
("whelchel" nil (w eh1 l ch ax l))
("wheless" nil (w iy1 l ih0 s))
("whelpley" nil (w eh1 l p l iy0))
("whelton" nil (w eh1 l t ax n))
("when" nil (w eh1 n))
("whence" nil (w eh1 n s))
("whenever" nil (w eh0 n eh1 v er0))
("where" nil (w eh1 r))
("whereabouts" nil (w eh1 r ax b aw1 t s))
("whereas" nil (w eh0 r ae1 z))
("whereby" nil (w eh0 r b ay1))
("wherehouse" nil (w eh1 r hh aw1 s))
("wherein" nil (w eh0 r ih1 n))
("whereupon" nil (w eh1 r ax p aa1 n))
("wherever" nil (w eh0 r eh1 v er0))
("wherewithal" nil (w eh1 r w ih0 dh ao1 l))
("wherley" nil (w er1 l iy0))
("wherry" nil (w eh1 r iy0))
("whet" nil (w eh1 t))
("whether" nil (w eh1 dh er0))
("whetsel" nil (w eh1 t s ax l))
("whetsell" nil (w eh1 t s ax l))
("whetstine" nil (w eh1 t s t iy0 n))
("whetstone" nil (w eh1 t s t ow1 n))
("whetted" nil (w eh1 t ih0 d))
("whetzel" nil (w eh1 t z ax l))
("whew" nil (w uw1))
("whey" nil (w ey1))
("which" nil (w ih1 ch))
("whichard" nil (w ih1 ch er0 d))
("whichever" nil (w ih0 ch eh1 v er0))
("whicker" nil (w ih1 k er0))
("whidby" nil (w ih1 d b iy0))
("whidden" nil (w ih1 d ax n))
("whiddon" nil (w ih1 d ax n))
("whiff" nil (w ih1 f))
("whig" nil (w ih1 g))
("whigham" nil (w ih1 g ax m))
("whigs" nil (w ih1 g z))
("whilden" nil (w ay1 l d ax n))
("while" nil (w ay1 l))
("whiles" nil (w ay1 l z))
("whim" nil (w ih1 m))
("whimper" nil (w ih1 m p er0))
("whimpering" nil (w ih1 m p er0 ih0 ng))
("whims" nil (w ih1 m z))
("whimsical" nil (w ih1 m z ih0 k ax l))
("whimsy" nil (w ih1 m s iy0))
("whine" nil (w ay1 n))
("whinery" nil (w ay1 n er0 iy0))
("whines" nil (w ay1 n z))
("whining" nil (w ay1 n ih0 ng))
("whinnery" nil (w ih1 n er0 iy0))
("whinney" nil (w ih1 n iy0))
("whiny" nil (w ay1 n iy0))
("whip" nil (w ih1 p))
("whipkey" nil (w ih1 p k iy1))
("whiplash" nil (w ih1 p l ae1 sh))
("whiplashes" nil (w ih1 p l ae1 sh ax z))
("whipp" nil (w ih1 p))
("whippany" nil (w ih1 p ax n iy0))
("whipped" nil (w ih1 p t))
("whipping" nil (w ih1 p ih0 ng))
("whipple" nil (w ih1 p ax l))
("whippoorwills" nil (w ih1 p er0 w ih1 l z))
("whipps" nil (w ih1 p s))
("whips" nil (w ih1 p s))
("whipsaw" nil (w ih1 p s ao1))
("whipsawed" nil (w ih1 p s ao1 d))
("whipsawing" nil (w ih1 p s ao1 ih0 ng))
("whirl" nil (w er1 l))
("whirled" nil (w er1 l d))
("whirley" nil (w er1 l iy0))
("whirling" nil (w er1 l ih0 ng))
("whirlpool" nil (w er1 l p uw1 l))
("whirlpools" nil (w er1 l p uw1 l z))
("whirlwind" nil (w er1 l w ih1 n d))
("whirring" nil (w er1 ih0 ng))
("whisenand" nil (w ih1 s ih0 n ae0 n d))
("whisenant" nil (w ih1 s ih0 n ax n t))
("whisenhunt" nil (w ay1 z ax n hh ah1 n t))
("whisk" nil (w ih1 s k))
("whisked" nil (w ih1 s k t))
("whiskers" nil (w ih1 s k er0 z))
("whiskey" nil (w ih1 s k iy0))
("whiskeys" nil (w ih1 s k iy0 z))
("whisks" nil (w ih1 s k s))
("whisky" nil (w ih1 s k iy0))
("whisler" nil (w ih1 s l er0))
("whisman" nil (w ih1 s m ax n))
("whisnant" nil (w ih1 s n ax n t))
("whisner" nil (w ih1 s n er0))
("whisonant" nil (w ih1 s ax n ax n t))
("whisper" nil (w ih1 s p er0))
("whispered" nil (w ih1 s p er0 d))
("whispering" nil (w ih1 s p er0 ih0 ng))
("whispers" nil (w ih1 s p er0 z))
("whistle" nil (w ih1 s ax l))
("whistleblower" nil (w ih1 s l ax b l ow1 er0))
("whistleblowers" nil (w ih1 s l ax b l ow1 er0 z))
("whistled" nil (w ih1 s ax l d))
("whistler" nil (w ih1 s l er0))
("whistlers" nil (w ih1 s l er0 z))
("whistles" nil (w ih1 s ax l z))
("whistling" nil (w ih1 s l ih0 ng))
("whiston" nil (w ih1 s t ax n))
("whit" nil (w ih1 t))
("whitacre" nil (w ih1 t ax k er0))
("whitaker" nil (w ih1 t ax k er0))
("whitbeck" nil (w ih1 t b eh1 k))
("whitbread" nil (w ih1 t b r eh1 d))
("whitby" nil (w ih1 t b iy0))
("whitcher" nil (w ih1 ch er0))
("whitchurch" nil (w ih1 t ch er1 ch))
("whitcomb" nil (w ih1 t k ax m))
("whitcraft" nil (w ih1 t k r ae1 f t))
("white" nil (w ay1 t))
("whiteaker" nil (w ih1 t ax k er0))
("whitebread" nil (w ay1 t b r eh1 d))
("whitecotton" nil (w ay1 t k aa1 t ax n))
("whited" nil (w ay1 t ih0 d))
("whitefield" nil (w ay1 t f iy1 l d))
("whitefish" nil (w ay1 t f ih1 sh))
("whiteford" nil (w ay1 t f ao0 r d))
("whitehair" nil (w ay1 t hh eh1 r))
("whitehall" nil (w ay1 t hh ao1 l))
("whitehead" nil (w ay1 t hh eh1 d))
("whitehill" nil (w ay1 t hh ih1 l))
("whitehorn" nil (w ay1 t hh ao1 r n))
("whitehorse" nil (w ay1 t hh ao1 r s))
("whitehouse" nil (w ay1 t hh aw1 s))
("whitehurst" nil (w ay1 t hh er1 s t))
("whitelaw" nil (w ay1 t l ao1))
("whiteley" nil (w ay1 t l iy0))
("whitelock" nil (w ay1 t l aa1 k))
("whitely" nil (w ay1 t l iy0))
("whiteman" nil (w ay1 t m ax n))
("whitemont" nil (w ay1 t m aa1 n t))
("whiten" nil (w ay1 t ax n))
("whitenack" nil (w ay1 t n ae1 k))
("whiteneir" nil (w ay1 t n ih1 r))
("whitener" nil (w ay1 t n er0))
("whiteness" nil (w ay1 t n ax s))
("whitenight" nil (w ay1 t n ay1 t))
("whiter" nil (w ay1 t er0))
("whites" nil (w ay1 t s))
("whitescarver" nil (w ay1 t s k aa1 r v er0))
("whitesel" nil (w ay1 t s eh1 l))
("whitesell" nil (w ay1 t s eh1 l))
("whiteside" nil (w ay1 t s ay1 d))
("whitesides" nil (w ay1 t s ay1 d z))
("whitetail" nil (w ay1 t t ey1 l))
("whitewash" nil (w ay1 t w aa1 sh))
("whitewashed" nil (w ay1 t w aa1 sh t))
("whitewater" nil (w ay1 t w ao1 t er0))
("whitey" nil (w ay1 t iy1))
("whitfield" nil (w ih1 t f iy0 l d))
("whitfill" nil (w ih1 t f ih1 l))
("whitford" nil (w ih1 t f er0 d))
("whitham" nil (w ih1 th ax m))
("whither" nil (w ih1 dh er0))
("whiting" nil (w ay1 t ih0 ng))
("whitings" nil (w ay1 t ih0 ng z))
("whitis" nil (w ay1 t ih0 s))
("whitish" nil (w ay1 t ih0 sh))
("whitlatch" nil (w ih1 t l ae1 ch))
("whitledge" nil (w ih1 t l eh1 jh))
("whitley" nil (w ih1 t l iy0))
("whitling" nil (w ih1 t l ih0 ng))
("whitlock" nil (w ih1 t l aa1 k))
("whitlow" nil (w ih1 t l ow1))
("whitman" nil (w ih1 t m ax n))
("whitmarsh" nil (w ih1 t m aa1 r sh))
("whitmer" nil (w ih1 t m er0))
("whitmill" nil (w ih1 t m ih1 l))
("whitmire" nil (w ih1 t m ay1 r))
("whitmore" nil (w ih1 t m ao0 r))
("whitmoyer" nil (w ih1 t m oy1 er0))
("whitmyer" nil (w ih1 t m ay1 er0))
("whitner" nil (w ih1 t n er0))
("whitney" nil (w ih1 t n iy0))
("whiton" nil (w ih1 t ax n))
("whitrow" nil (w ih1 t r ow1))
("whitsel" nil (w ih1 t s ax l))
("whitsell" nil (w ih1 t s ax l))
("whitsett" nil (w ih1 t s ih0 t))
("whitsitt" nil (w ih1 t s ih0 t))
("whitson" nil (w ih1 t s ax n))
("whitt" nil (w ih1 t))
("whittaker" nil (w ih1 t ax k er0))
("whitted" nil (w ih1 t ih0 d))
("whittemore" nil (w ih1 t m ao0 r))
("whitten" nil (w ih1 t ax n))
("whittenberg" nil (w ih1 t ax n b er0 g))
("whittenburg" nil (w ih1 t ax n b er0 g))
("whittet" nil (w ih1 t ih0 t))
("whittier" nil (w ih1 t iy0 er0))
("whitting" nil (w ih1 t ih0 ng))
("whittingham" nil (w ih1 t ih0 ng hh ae1 m))
("whittinghill" nil (w ih1 t ih0 ng hh ih1 l))
("whittington" nil (w ih1 t ih0 ng t ax n))
("whittle" nil (w ih1 t ax l))
("whittled" nil (w ih1 t ax l d))
("whittlesey" nil (w ih1 t ax l s iy0))
("whittling" nil (w ih1 t l ih0 ng))
("whitton" nil (w ih1 t ax n))
("whitty" nil (w ih1 t iy0))
("whitwam" nil (w ih1 t w aa0 m))
("whitwell" nil (w ih1 t w eh1 l))
("whitworth" nil (w ih1 t w er1 th))
("whiz" nil (w ih1 z))
("whizzed" nil (w ih1 z d))
("whizzes" nil (w ih1 z ih0 z))
("whizzing" nil (w ih1 z ih0 ng))
("who" nil (hh uw1))
("whoa" nil (w ow1))
("whobrey" nil (w aa1 b r iy0))
("whodunit" nil (hh uw0 d ah1 n ih0 t))
("whodunits" nil (hh uw0 d ah1 n ih0 t s))
("whoever" nil (hh uw0 eh1 v er0))
("whole" nil (hh ow1 l))
("wholehearted" nil (hh ow1 l hh aa1 r t ih0 d))
("wholeheartedly" nil (hh ow1 l hh aa0 r t ih0 d l iy0))
("wholesale" nil (hh ow1 l s ey1 l))
("wholesaler" nil (hh ow1 l s ey1 l er0))
("wholesalers" nil (hh ow1 l s ey1 l er0 z))
("wholesales" nil (hh ow1 l s ey1 l z))
("wholesaling" nil (hh ow1 l s ey1 l ih0 ng))
("wholesome" nil (hh ow1 l s ax m))
("wholesomeness" nil (hh ow1 l s ax m n ax s))
("wholey" nil (hh aa1 l iy0))
("wholly" nil (hh ow1 l iy0))
("whom" nil (hh uw1 m))
("whomever" nil (hh uw0 m eh1 v er0))
("whoop" nil (w uw1 p))
("whooped" nil (w uw1 p t))
("whoopee" nil (w uw1 p iy1))
("whoopi" nil (w uw1 p iy0))
("whooping" nil (w uw1 p ih0 ng))
("whoops" nil (w uw1 p s))
("whoosh" nil (w uw1 sh))
("whopper" nil (w aa1 p er0))
("whoppers" nil (w aa1 p er0 z))
("whopping" nil (w aa1 p ih0 ng))
("whore" nil (hh ao1 r))
("whores" nil (hh ao1 r z))
("whorl" nil (w er1 l))
("whorley" nil (hh ao1 r l iy0))
("whorls" nil (w er1 l z))
("whorton" nil (hh ao1 r t ax n))
("whose" nil (hh uw1 z))
("whosoever" nil (hh uw1 s ow0 eh1 v er0))
("why" nil (w ay1))
("whyte" nil (w ay1 t))
("wiacek" nil (v ay1 ax ch eh0 k))
("wian" nil (w ay1 ax n))
("wiant" nil (w ay1 ax n t))
("wiard" nil (w ay1 aa0 r d))
("wiater" nil (w ay1 ax t er0))
("wiatrowski" nil (v iy0 ax t r ao1 f s k iy0))
("wiatt" nil (w ay1 ax t))
("wibbelsman" nil (w ih1 b ax l z m ax n))
("wibbenmeyer" nil (w ih1 b ih0 n m ay0 er0))
("wiberg" nil (w ay1 b er0 g))
("wible" nil (w ay1 b ax l))
("wicat" nil (w ih1 k ae0 t))
("wicca" nil (w ih1 k ax))
("wiccan" nil (w ih1 k ax n))
("wice" nil (w ay1 s))
("wich" nil (w ih1 ch))
("wichern" nil (w ih1 k er0 n))
("wichert" nil (w ih1 ch er0 t))
("wichita" nil (w ih1 ch ax t ao1))
("wichman" nil (w ih1 ch m ax n))
("wichmann" nil (w ih1 ch m ax n))
("wick" nil (w ih1 k))
("wickard" nil (w ih1 k er0 d))
("wicke" nil (w ih1 k))
("wicked" nil (w ih1 k ax d))
("wickedly" nil (w ih1 k ih0 d l iy0))
("wickedness" nil (w ih1 k ax d n ax s))
("wickens" nil (w ih1 k ax n z))
("wicker" nil (w ih1 k er0))
("wickerham" nil (w ih1 k er0 hh ae1 m))
("wickersham" nil (w ih1 k er0 sh ae1 m))
("wickert" nil (w ih1 k er0 t))
("wickes" nil (w ih1 k s))
("wickets" nil (w ih1 k ax t s))
("wickett" nil (w ih1 k ih0 t))
("wickey" nil (w ih1 k iy0))
("wickham" nil (w ih1 k ax m))
("wickizer" nil (w ih1 k ay1 z er0))
("wickland" nil (w ih1 k l ax n d))
("wicklander" nil (w ih1 k l ae1 n d er0))
("wickley" nil (w ih1 k l iy0))
("wickliff" nil (w ih1 k l ih0 f))
("wickliffe" nil (w ih1 k l ih0 f))
("wickline" nil (w ih1 k l ay1 n))
("wicklund" nil (w ih1 k l ax n d))
("wickman" nil (w ih1 k m ax n))
("wicks" nil (w ih1 k s))
("wickstrom" nil (w ih1 k s t r ax m))
("wickware" nil (w ih1 k w eh1 r))
("wickwire" nil (v ih1 s k v ay0 r))
("wicor" nil (w ay1 k ao1 r))
("widder" nil (w ih1 d er0))
("widdows" nil (w ih1 d ow1 z))
("widdowson" nil (w ih1 d aw0 s ax n))
("wide" nil (w ay1 d))
("widebody" nil (w ay1 d b aa1 d iy0))
("widell" nil (w ih1 d ax l))
("widely" nil (w ay1 d l iy0))
("wideman" nil (w ay1 d m ax n))
("widen" nil (w ay1 d ax n))
("widened" nil (w ay1 d ax n d))
("widener" nil (w ih1 d iy0 n er0))
("widening" nil (w ay1 d ax n ih0 ng))
("widens" nil (w ay1 d ax n z))
("wider" nil (w ay1 d er0))
("widespread" nil (w ay1 d s p r eh1 d))
("widest" nil (w ay1 d ax s t))
("widger" nil (w ih1 jh er0))
("widget" nil (w ih1 jh ih0 t))
("widgets" nil (w ih1 jh ih0 t s))
("widhalm" nil (w ih1 d hh aa0 m))
("widick" nil (w ih1 d ih0 k))
("widing" nil (w ay1 d ih0 ng))
("widmaier" nil (w ih1 d m ay0 er0))
("widman" nil (w ih1 d m ax n))
("widmann" nil (w ih1 d m ax n))
("widmar" nil (w ih1 d m er0))
("widmayer" nil (w ih1 d m ey1 er0))
("widmer" nil (w ih1 d m er0))
("widner" nil (w ih1 d n er0))
("widow" nil (w ih1 d ow0))
("widowed" nil (w ih1 d ow0 d))
("widower" nil (w ih1 d ow0 er0))
("widowers" nil (w ih1 d ow0 er0 z))
("widows" nil (w ih1 d ow0 z))
("widrick" nil (w ih1 d r ih0 k))
("widrig" nil (w ih1 d r ih0 g))
("width" nil (w ih1 d th))
("widths" nil (w ih1 d th s))
("widuri" nil (w ih0 d uh1 r iy0))
("wieand" nil (w iy1 ax n d))
("wiebe" nil (w iy1 b))
("wieber" nil (w iy1 b er0))
("wieberg" nil (w iy1 b er0 g))
("wiebke" nil (w iy1 b k iy0))
("wiebold" nil (w iy1 b ow1 l d))
("wiebusch" nil (w iy1 b ax sh))
("wiech" nil (w iy1 ch))
("wiechert" nil (w iy1 k er0 t))
("wiechman" nil (w iy1 k m ax n))
("wiechmann" nil (w iy1 k m ax n))
("wieck" nil (w iy1 k))
("wieczorek" nil (v iy0 ch ao1 r eh0 k))
("wied" nil (w iy1 d))
("wiedel" nil (w iy1 d ax l))
("wiedeman" nil (w iy1 d m ax n))
("wiedemann" nil (w iy1 d m ax n))
("wieden" nil (w iy1 d ax n))
("wieder" nil (w iy1 d er0))
("wiederaufbau" nil (w iy1 d er0 ao1 f b aw0))
("wiederhold" nil (w iy1 d er0 hh ow0 l d))
("wiederholt" nil (w iy1 d er0 hh ow0 l t))
("wiederkehr" nil (w iy1 d er0 k ih0 r))
("wiedman" nil (w iy1 d m ax n))
("wiedmann" nil (w iy1 d m ax n))
("wiedmeyer" nil (w iy1 d m ay0 er0))
("wiedner" nil (w iy1 d n er0))
("wiedrich" nil (w iy1 d r ih0 k))
("wiegand" nil (w iy1 g ax n d))
("wiegel" nil (w iy1 g ax l))
("wiegers" nil (w iy1 g er0 z))
("wiegert" nil (w iy1 g er0 t))
("wiegman" nil (w iy1 g m ax n))
("wiegmann" nil (w iy1 g m ax n))
("wiehe" nil (w iy1))
("wieland" nil (w iy1 l ax n d))
("wield" nil (w iy1 l d))
("wielded" nil (w iy1 l d ih0 d))
("wielding" nil (w iy1 l d ih0 ng))
("wields" nil (w iy1 l d z))
("wielgus" nil (w iy1 l g ax s))
("wieman" nil (w iy1 m ax n))
("wiemann" nil (w iy1 m ax n))
("wiemer" nil (w iy1 m er0))
("wiemers" nil (w iy1 m er0 z))
("wien" nil (w iy1 n))
("wiencek" nil (v iy1 n ch eh0 k))
("wienecke" nil (w iy1 n ih0 k))
("wieneke" nil (w iy1 n ih0 k))
("wiener" nil (w iy1 n er0))
("wieners" nil (w iy1 n er0 z))
("wienerschnitzel" nil (w iy1 n er0 sh n ih1 t s ax l))
("wienke" nil (w iy1 ng k))
("wiens" nil (w iy1 n z))
("wier" nil (w ih1 r))
("wierenga" nil (v ih0 r eh1 ng g ax))
("wierman" nil (w ih1 r m ax n))
("wiers" nil (w iy1 r z))
("wiersema" nil (v ih0 r s iy1 m ax))
("wiersma" nil (v ih1 r s m ax))
("wierzba" nil (v ih1 r z b ax))
("wierzbicki" nil (v ih0 r z b ih1 t s k iy0))
("wies" nil (w ay1 z))
("wiesbaden" nil (w iy1 s b aa1 d ax n))
("wiese" nil (w iy1 z))
("wiesel" nil (w iy1 s ax l))
("wieseler" nil (w iy1 s ax l er0))
("wiesemann" nil (w iy1 s m ax n))
("wiesen" nil (w iy1 s ax n))
("wieser" nil (w iy1 s er0))
("wiesman" nil (w iy1 z m ax n))
("wiesner" nil (w iy1 z n er0))
("wiess" nil (w iy1 s))
("wiessner" nil (w iy1 s n er0))
("wiest" nil (w ay1 ih0 s t))
("wieting" nil (w iy1 t ih0 ng))
("wife" nil (w ay1 f))
("wig" nil (w ih1 g))
("wigal" nil (w ih1 g ax l))
("wigand" nil (w ih1 g ax n d))
("wigen" nil (w ih1 g ax n))
("wigfall" nil (w ih1 g f ao1 l))
("wigfield" nil (w ih1 g f iy1 l d))
("wigg" nil (w ih1 g))
("wiggans" nil (w ih1 g ax n z))
("wigger" nil (w ih1 g er0))
("wiggers" nil (w ih1 g er0 z))
("wiggin" nil (w ih1 g ih0 n))
("wiggington" nil (w ih1 g ih0 ng t ax n))
("wiggins" nil (w ih1 g ih0 n z))
("wigginton" nil (w ih1 g ih0 n t ax n))
("wiggle" nil (w ih1 g ax l))
("wigglesworth" nil (w ih1 g ax l z w er1 th))
("wiggling" nil (w ih1 g ax l ih0 ng))
("wiggly" nil (w ih1 g ax l iy0))
("wiggs" nil (w ih1 g z))
("wight" nil (w ay1 t))
("wightman" nil (w ay1 t m ax n))
("wigington" nil (w ih1 g ih0 ng t ax n))
("wiginton" nil (w ih1 jh ih0 n t ax n))
("wigle" nil (w ay1 g ax l))
("wigley" nil (w ih1 g l iy0))
("wigmore" nil (w ih1 g m ao0 r))
("wignall" nil (w ih1 g n ax l))
("wigs" nil (w ih1 g z))
("wigton" nil (w ih1 g t ax n))
("wigwam" nil (w ih1 g w aa0 m))
("wiitala" nil (v iy0 t aa1 l ax))
("wik" nil (w ih1 k))
("wike" nil (w ay1 k))
("wikel" nil (w ih1 k ax l))
("wiker" nil (w ay1 k er0))
("wikle" nil (w ay1 k ax l))
("wiklund" nil (w ih1 k l ax n d))
("wikoff" nil (w ih1 k ao0 f))
("wikstrom" nil (w ih1 k s t r ax m))
("wil" nil (w ih1 l))
("wiland" nil (w ay1 l ax n d))
("wilander" nil (w ay1 l ax n d er0))
("wilbanks" nil (w ih1 l b ax ng k s))
("wilber" nil (w ih1 l b er0))
("wilberforce" nil (w ih1 l b er0 f ao1 r s))
("wilberg" nil (w ih1 l b er0 g))
("wilbert" nil (w ih1 l b er0 t))
("wilbon" nil (w ih1 l b ax n))
("wilborn" nil (w ih1 l b er0 n))
("wilbourn" nil (w ih1 l b er0 n))
("wilbourne" nil (w ih1 l b er0 n))
("wilbur" nil (w ih1 l b er0))
("wilburn" nil (w ih1 l b er0 n))
("wilcher" nil (w ih1 l ch er0))
("wilcock" nil (w ih1 l k aa0 k))
("wilcox" nil (w ih1 l k aa0 k s))
("wilcoxen" nil (w ih0 l k aa1 k s ax n))
("wilcoxon" nil (w ih0 l k aa1 k s ax n))
("wilcoxson" nil (w ih1 l k aa0 k s ax n))
("wilczak" nil (v ih1 l ch ae0 k))
("wilczek" nil (v ih1 l ch eh0 k))
("wilczewski" nil (v ih0 l ch eh1 f s k iy0))
("wilczynski" nil (v ih0 l ch ih1 n s k iy0))
("wild" nil (w ay1 l d))
("wildasin" nil (w ay1 l d ax s ih1 n))
("wildavsky" nil (w ih0 l d ae1 v s k iy0))
("wildcat" nil (w ay1 l d k ae1 t))
("wildcats" nil (w ay1 l d k ae1 t s))
("wildcatter" nil (w ay1 l d k ae1 t er0))
("wildcatters" nil (w ay1 l d k ae1 t er0 z))
("wildcatting" nil (w ay1 l d k ae1 t ih0 ng))
("wilde" nil (w ay1 l d))
("wildeman" nil (w ay1 l d m ax n))
("wilden" nil (w ay1 l d ax n))
("wilder" nil (w ay1 l d er0))
("wilderman" nil (w ay1 l d er0 m ax n))
("wildermuth" nil (w ay1 l d er0 m uw0 th))
("wilderness" nil (w ih1 l d er0 n ax s))
("wildes" nil (w ay1 l d z))
("wildest" nil (w ay1 l d ih0 s t))
("wildey" nil (w ih1 l d iy0))
("wildfire" nil (w ay1 l d f ay1 er0))
("wildfires" nil (w ay1 l d f ay1 er0 z))
("wildflower" nil (w ay1 l d f l aw1 er0))
("wildflowers" nil (w ay1 l d f l aw1 r z))
("wilding" nil (w ay1 l d ih0 ng))
("wildlife" nil (w ay1 l d l ay1 f))
("wildly" nil (w ay1 l d l iy0))
("wildman" nil (w ay1 l d m ax n))
("wildmon" nil (w ay1 l d m ax n))
("wildness" nil (w ay1 l d n ax s))
("wildon" nil (w ay1 l d ax n))
("wildrick" nil (w ay1 l d r ih0 k))
("wilds" nil (w ay1 l d z))
("wildt" nil (w ih1 l t))
("wildwood" nil (w ay1 l d w uh1 d))
("wile" nil (w ay1 l))
("wileen" nil (w ih0 l iy1 n))
("wileman" nil (w ay1 l m ax n))
("wilemon" nil (w ih1 l ih0 m aa0 n))
("wilen" nil (w ay1 l ax n))
("wilensky" nil (w ax l ih1 n s k iy0))
("wiler" nil (w ay1 l er0))
("wiles" nil (w ay1 l z))
("wiley" nil (w ay1 l iy0))
("wilfert" nil (w ih1 l f er0 t))
("wilfong" nil (w ih1 l f ao0 ng))
("wilford" nil (w ih1 l f er0 d))
("wilfred" nil (w ih1 l f r ih0 d))
("wilfreda" nil (w ih1 l f r ih0 d ax))
("wilfredo" nil (w ih1 l f ey1 d ow0))
("wilfrid" nil (w ih1 l f r ih0 d))
("wilfried" nil (w ih1 l f r iy0 d))
("wilgus" nil (w ih1 l g ax s))
("wilham" nil (w ih1 l hh ax m))
("wilhelm" nil (w ih1 l hh eh1 l m))
("wilhelma" nil (w ih0 l hh eh1 l m ax))
("wilhelmi" nil (w ih0 l hh eh1 l m iy0))
("wilhelmina" nil (w ih1 l hh eh0 l m iy1 n ax))
("wilhelmine" nil (w ih1 l hh ih0 l m ih0 n))
("wilhelms" nil (w ih1 l hh eh1 l m z))
("wilhelmsen" nil (w ih1 l hh ih0 l m s ax n))
("wilhelmy" nil (w ih1 l hh ih0 l m iy0))
("wilhide" nil (w ih1 l hh ay1 d))
("wilhite" nil (w ih1 l hh ay1 t))
("wilhoit" nil (w ih1 l hh oy1 t))
("wilhoite" nil (w ih1 l hh oy1 t))
("wiliams" nil (w ih1 l iy0 ax m z))
("wilinski" nil (v ih0 l ih1 n s k iy0))
("wilk" nil (w ih1 l k))
("wilke" nil (w ih1 l k))
("wilken" nil (w ih1 l k ax n))
("wilkening" nil (w ih1 l k ax n ih0 ng))
("wilkens" nil (w ih1 l k ax n z))
("wilkenson" nil (w ih1 l k ih0 n s ax n))
("wilker" nil (w ih1 l k er0))
("wilkerson" nil (w ih1 l k er0 s ax n))
("wilkes" nil (w ih1 l k s))
("wilkesboro" nil (w ih1 l k s b er0 ow0))
("wilkey" nil (w ih1 l k iy0))
("wilkie" nil (w ih1 l k iy0))
("wilkin" nil (w ih1 l k ih0 n))
("wilking" nil (w ih1 l k ih0 ng))
("wilkins" nil (w ih1 l k ih0 n z))
("wilkinson" nil (w ih1 l k ax n s ax n))
("wilkis" nil (w ih1 l k ih0 s))
("wilkison" nil (w ih1 l k ih0 s ax n))
("wilkowski" nil (v ih0 l k ao1 f s k iy0))
("wilks" nil (w ih1 l k s))
("will" nil (w ih1 l))
("willa" nil (w ih1 l ax))
("willabelle" nil (w ih1 l ax b eh1 l))
("willadsen" nil (w ih0 l ae1 d s ax n))
("willaims" nil (w ih1 l ax m z))
("willam" nil (w ih1 l ax m))
("willamette" nil (w ax l ae1 m ax t))
("willamina" nil (w ih0 l ax m ay1 n ax))
("willapa" nil (w ih0 l aa1 p ax))
("willard" nil (w ih1 l er0 d))
("willbanks" nil (w ih1 l b ae1 ng k s))
("willcox" nil (w ih1 l k aa1 k s))
("willcutt" nil (w ih1 l k ax t))
("willden" nil (w ih1 l d ax n))
("wille" nil (w ih1 l))
("willed" nil (w ih1 l d))
("willeford" nil (w ih1 l ih0 f er0 d))
("willem" nil (w ih1 l ax m))
("willems" nil (w ih1 l ax m z))
("willemsen" nil (w ih0 l eh1 m s ax n))
("willen" nil (w ih1 l ax n))
("willenborg" nil (w ih1 l ih0 n b ao0 r g))
("willenbring" nil (w ih1 l ax n b r ih1 ng))
("willens" nil (w ih1 l ax n z))
("willer" nil (w ih1 l er0))
("willers" nil (w ih1 l er0 z))
("willert" nil (w ih1 l er0 t))
("willes" nil (w ay1 l z))
("willet" nil (w ih1 l ih0 t))
("willets" nil (w ih1 l ih0 t s))
("willett" nil (w ih1 l ih0 t))
("willette" nil (w ih0 l eh1 t))
("willetts" nil (w ih1 l ih0 t s))
("willey" nil (w ih1 l iy0))
("willford" nil (w ih1 l f er0 d))
("willful" nil (w ih1 l f ax l))
("willfully" nil (w ih1 l f ax l iy0))
("willhelm" nil (w ih1 l hh eh1 l m))
("willhite" nil (w ih1 l hh ay1 t))
("willhoit" nil (w ih1 l hh oy1 t))
("willhoite" nil (w ih1 l hh oy1 t))
("willi" nil (w ih1 l iy0))
("william" nil (w ih1 l y ax m))
("williams" nil (w ih1 l y ax m z))
("williamsburg" nil (w ih1 l y ax m z b er0 g))
("williamsburgh" nil (w ih1 l y ax m z b er0 g))
("williamsen" nil (w ih1 l y ax m s ax n))
("williamson" nil (w ih1 l y ax m s ax n))
("williamsport" nil (w ih1 l y ax m s p ao1 r t))
("williamstown" nil (w ih1 l y ax m s t aw1 n))
("williard" nil (w ih1 l y aa0 r d))
("willie" nil (w ih1 l iy0))
("williford" nil (w ih1 l ih0 f er0 d))
("willig" nil (w ih1 l ih0 g))
("willing" nil (w ih1 l ih0 ng))
("willinger" nil (w ih1 l ih0 ng er0))
("willingham" nil (w ih1 l ih0 ng hh ae1 m))
("willingly" nil (w ih1 l ih0 ng l iy0))
("willingness" nil (w ih1 l ih0 ng n ax s))
("willis" nil (w ih1 l ih0 s))
("willison" nil (w ih1 l ih0 s ax n))
("williston" nil (w ih1 l ih0 s t aa0 n))
("willits" nil (w ih1 l ih0 t s))
("willke" nil (w ih1 l k iy0))
("willkie" nil (w ih1 l k iy0))
("willman" nil (w ih1 l m ax n))
("willmann" nil (w ih1 l m ax n))
("willmon" nil (w ih1 l m ax n))
("willmore" nil (w ih1 l m ao0 r))
("willmott" nil (w ih1 l m ax t))
("willms" nil (w ih1 l m z))
("willner" nil (w ih1 l n er0))
("willock" nil (w ih1 l ax k))
("willoughby" nil (w ih1 l ax b iy0))
("willow" nil (w ih1 l ow1))
("willowbrook" nil (w ih1 l ow0 b r uh1 k))
("willowes" nil (w ih1 l ow1 z))
("willows" nil (w ih1 l ow1 z))
("willowy" nil (w ih1 l ax w iy0))
("willpower" nil (w ih1 l p aw1 er0))
("wills" nil (w ih1 l z))
("willse" nil (w ih1 l s iy0))
("willsey" nil (w ih1 l s iy0))
("willson" nil (w ih1 l s ax n))
("willwerth" nil (w ih1 l w er0 th))
("willy" nil (w ih1 l iy0))
("willyard" nil (w ih1 l y aa1 r d))
("wilma" nil (w ih1 l m ax))
("wilmar" nil (w ih1 l m er0))
("wilmarth" nil (w ih1 l m aa0 r th))
("wilmer" nil (w ih1 l m er0))
("wilmes" nil (w ih1 l m z))
("wilmet" nil (w ih1 l m ih0 t))
("wilmeth" nil (w ih1 l m ih0 th))
("wilmette" nil (w ih0 l m eh1 t))
("wilmington" nil (w ih1 l m ih0 ng t ax n))
("wilmore" nil (w ih1 l m ao0 r))
("wilmot" nil (w ih1 l m ax t))
("wilmoth" nil (w ih1 l m ax th))
("wilmott" nil (w ih1 l m ax t))
("wilmouth" nil (w ih1 l m aw0 th))
("wilms" nil (w ih1 l m z))
("wilner" nil (w ih1 l n er0))
("wilpon" nil (w ih1 l p aa0 n))
("wilsey" nil (w ih1 l s iy0))
("wilshire" nil (w ih1 l sh ay1 r))
("wilshusen" nil (w ih1 l sh uw0 s ax n))
("wilson" nil (w ih1 l s ax n))
("wilt" nil (w ih1 l t))
("wilted" nil (w ih1 l t ih0 d))
("wiltel" nil (w ih1 l t eh1 l))
("wiltfong" nil (w ih1 l t f ao0 ng))
("wiltgen" nil (w ih1 l t g ax n))
("wilthew" nil (w ih1 l th y uw0))
("wilting" nil (w ih1 l t ih0 ng))
("wilton" nil (w ih1 l t ax n))
("wiltron" nil (w ih1 l t r ax n))
("wiltrout" nil (w ih1 l t r aw1 t))
("wilts" nil (w ih1 l t s))
("wiltse" nil (w ih1 l t s))
("wiltsey" nil (w ih1 l t s iy0))
("wiltshire" nil (w ih1 l ch ay1 r))
("wiltsie" nil (w ih1 l t s iy0))
("wiltz" nil (w ih1 l t s))
("wily" nil (w ay1 l iy0))
("wiman" nil (w ay1 m ax n))
("wimberley" nil (w ih1 m b er0 l iy0))
("wimberly" nil (w ih1 m b er0 l iy0))
("wimbish" nil (w ih1 m b ih0 sh))
("wimbledon" nil (w ih1 m b ax l d ax n))
("wimbley" nil (w ih1 m b l iy0))
("wimbush" nil (w ih1 m b ax sh))
("wimer" nil (w ay1 m er0))
("wimmer" nil (w ih1 m er0))
("wimp" nil (w ih1 m p))
("wimpee" nil (w ih1 m p iy0))
("wimps" nil (w ih1 m p s))
("wimpy" nil (w ih1 m p iy0))
("wims" nil (w ih1 m z))
("wimsatt" nil (w ih1 m s ax t))
("win" nil (w ih1 n))
("winamac" nil (w ih1 n ax m ae0 k))
("winans" nil (w ih1 n ax n z))
("winant" nil (w ay1 n ax n t))
("winberg" nil (w ih1 n b er0 g))
("winberry" nil (w ih1 n b eh1 r iy0))
("winborn" nil (w ih1 n b er0 n))
("winborne" nil (w ih1 n b er0 n))
("winburn" nil (w ih1 n b er1 n))
("winbush" nil (w ih1 n b uh1 sh))
("wince" nil (w ih1 n s))
("winced" nil (w ih1 n s t))
("wincek" nil (w ih1 n s ih0 k))
("winch" nil (w ih1 n ch))
("winchel" nil (w ih1 n k ax l))
("winchell" nil (w ih1 n k ax l))
("winchester" nil (w ih1 n ch eh1 s t er0))
("wincing" nil (w ih1 n s ih0 ng))
("winckler" nil (w ih1 ng k l er0))
("wind" v (w ay1 n d))
("wind" n (w ih1 n d))
("windchill" nil (w ih1 n d ch ih1 l))
("windchime" nil (w ih1 n d ch ay1 m))
("windchimes" nil (w ih1 n d ch ay1 m z))
("windecker" nil (w ih1 n d eh1 k er0))
("winded" nil (w ih1 n d ih0 d))
("windell" nil (w ih1 n d ax l))
("windels" nil (w ih1 n d ax l z))
("winder" nil (w ih1 n d er0))
("winders" nil (w ih1 n d er0 z))
("windes" nil (w ih1 n d z))
("windfall" nil (w ih1 n d f ao1 l))
("windfalls" nil (w ih1 n d f ao1 l z))
("windham" nil (w ih1 n d ax m))
("windheim" nil (w ih1 n d hh ay1 m))
("windhoek" nil (w ih1 n d hh ow1 k))
("windholz" nil (w ih1 n d hh ow1 l z))
("windhorst" nil (w ih1 n d hh ao0 r s t))
("winding" nil (w ay1 n d ih0 ng))
("windisch" nil (w ih1 n d ih0 sh))
("windish" nil (w ih1 n d ih0 sh))
("windlass" nil (w ih1 n d l ax s))
("windle" nil (w ih1 n d ax l))
("windler" nil (w ih1 n d l er0))
("windley" nil (w ih1 n d l iy0))
("windmere" nil (w ih1 n d m ih1 r))
("windmill" nil (w ih1 n d m ih1 l))
("windmiller" nil (w ih1 n d m ih1 l er0))
("windmills" nil (w ih1 n d m ih1 l z))
("windom" nil (w ih1 n d ax m))
("windon" nil (w ih1 n d ax n))
("window" nil (w ih1 n d ow0))
("windowed" nil (w ih1 n d ow0 d))
("windowless" nil (w ih1 n d ow0 l ax s))
("windowpanes" nil (w ih1 n d ow0 p ey1 n z))
("windows" nil (w ih1 n d ow0 z))
("winds" nil (w ih1 n d z))
("windshield" nil (w ih1 n d sh iy1 l d))
("windshields" nil (w ih1 n d sh iy1 l d z))
("windsor" nil (w ih1 n z er0))
("windspeed" nil (w ih1 n d s p iy1 d))
("windstar" nil (w ih1 n d s t aa1 r))
("windstorm" nil (w ih1 n d s t ao1 r m))
("windswept" nil (w ih1 n s w eh1 p t))
("windt" nil (w ih1 n t))
("windup" nil (w ay1 n d ah1 p))
("windward" nil (w ih1 n d w er0 d))
("windy" nil (w ih1 n d iy0))
("wine" nil (w ay1 n))
("winebarger" nil (w ih1 n ih0 b aa0 r g er0))
("wineberg" nil (w ay1 n b er0 g))
("winebrenner" nil (w ih1 n ih0 b r ih0 n er0))
("winecoff" nil (w ih1 n ih0 k ao0 f))
("wined" nil (w ay1 n d))
("winegar" nil (w ih1 n ih0 g er0))
("winegarden" nil (w ay1 n g aa1 r d ax n))
("winegardner" nil (w ih1 n ih0 g aa0 r d n er0))
("wineheim" nil (w ay1 n hh ay1 m))
("wineinger" nil (w ay1 n ih0 ng er0))
("wineland" nil (w ih1 n ih0 l ax n d))
("winema" nil (w ih1 n ih0 m ax))
("wineman" nil (w ay1 n m ax n))
("winemiller" nil (w ay1 n m ih1 l er0))
("winer" nil (w ay1 n er0))
("wineries" nil (w ay1 n er0 iy0 z))
("winery" nil (w ay1 n er0 iy0))
("wines" nil (w ay1 n z))
("winey" nil (w ay1 n iy0))
("winfield" nil (w ih1 n f iy1 l d))
("winford" nil (w ih1 n f er0 d))
("winfred" nil (w ih1 n f r ih0 d))
("winfree" nil (w ih1 n f r iy1))
("winfrey" nil (w ih1 n f r iy0))
("winfrid" nil (w ih1 n f r ih0 d))
("wing" nil (w ih1 ng))
("wingard" nil (w ih1 ng g er0 d))
("wingate" nil (w ih1 n g ey1 t))
("wingback" nil (w ih1 ng b ae1 k))
("winge" nil (w ih1 n jh))
("winged" nil (w ih1 ng d))
("winger" nil (w ih1 ng er0))
("wingers" nil (w ih1 ng er0 z))
("wingert" nil (w ih1 ng g er0 t))
("wingerter" nil (w ih1 ng g er0 t er0))
("winget" nil (w ih1 ng g ih0 t))
("wingett" nil (w ih1 ng g ih0 t))
("wingfield" nil (w ih1 ng f iy1 l d))
("winging" nil (w ih1 ng ih0 ng))
("wingler" nil (w ih1 ng g ax l er0))
("winglike" nil (w ih1 ng l ay1 k))
("wingman" nil (w ih1 ng m ax n))
("wingo" nil (w iy1 ng g ow0))
("wingrove" nil (w ih1 n g r ow1 v))
("wings" nil (w ih1 ng z))
("wingspan" nil (w ih1 ng s p ae1 n))
("winiarski" nil (v ih0 n iy0 aa1 r s k iy0))
("winick" nil (w ih1 n ih0 k))
("winiecki" nil (v ih0 n iy1 t s k iy0))
("winifred" nil (w ih1 n ih0 f r ih0 d))
("wining" nil (w ay1 n ih0 ng))
("wininger" nil (w ay1 n ih0 ng er0))
("winings" nil (w ay1 n ih0 ng z))
("wink" nil (w ih1 ng k))
("winked" nil (w ih1 ng k t))
("winkel" nil (w ih1 ng k ax l))
("winkelman" nil (w ih1 ng k ax l m ax n))
("winkelmann" nil (w ih1 ng k ax l m ax n))
("winkels" nil (w ih1 ng k ax l z))
("winker" nil (w ih1 ng k er0))
("winkfield" nil (w ih1 ng k f iy1 l d))
("winking" nil (w ih1 ng k ih0 ng))
("winkle" nil (w ih1 ng k ax l))
("winkleman" nil (w ih1 ng k ax l m ax n))
("winkler" nil (w ih1 ng k l er0))
("winkles" nil (w ih1 ng k ax l z))
("winkley" nil (w ih1 ng k l iy0))
("winkowski" nil (v ih0 ng k ao1 f s k iy0))
("winks" nil (w ih1 ng k s))
("winland" nil (w ih1 n l ax n d))
("winn" nil (w ih1 n))
("winnable" nil (w ih1 n ax b ax l))
("winne" nil (w ih1 n))
("winnebago" nil (w ih1 n ax b ey1 g ow0))
("winner" nil (w ih1 n er0))
("winners" nil (w ih1 n er0 z))
("winnetka" nil (w ih0 n eh1 t k ax))
("winnett" nil (w ih1 n ih0 t))
("winney" nil (w ih1 n iy0))
("winnick" nil (w ih1 n ih0 k))
("winnicki" nil (v ih0 n ih1 t s k iy0))
("winnie" nil (w ih1 n iy0))
("winning" nil (w ih1 n ih0 ng))
("winningest" nil (w ih1 n ih0 ng ax s t))
("winningham" nil (w ih1 n ih0 ng hh ae1 m))
("winnings" nil (w ih1 n ih0 ng z))
("winnipeg" nil (w ih1 n ih0 p ax g))
("winnow" nil (w ih1 n ow1))
("winnowed" nil (w ih1 n ow1 d))
("winnowing" nil (w ih1 n ow1 ih0 ng))
("winny" nil (w ih1 n iy0))
("wino" nil (w iy1 n ow0))
("winograd" nil (w ih1 n ax g r ae0 d))
("winokur" nil (w ih1 n ax k er0))
("winola" nil (v ih0 n ow1 l ax))
("winona" nil (w ih1 n ax n ax))
("winonah" nil (w ih1 n ax n ax))
("winooski" nil (w ih0 n uw1 s k iy0))
("winos" nil (w iy1 n ow0 s))
("winquist" nil (w ih1 n k w ih1 s t))
("wins" nil (w ih1 n z))
("winsett" nil (w ih1 n s ih0 t))
("winship" nil (w ih1 n sh ih1 p))
("winski" nil (w ih1 n s k iy1))
("winslett" nil (w ih1 n s l ih0 t))
("winslow" nil (w ih1 n z l ow0))
("winsome" nil (w ih1 n s ax m))
("winson" nil (w ih1 n s ax n))
("winsor" nil (w ih1 n z er0))
("winstanley" nil (w ih1 n s t ax n l iy0))
("winstead" nil (w ih1 n s t eh1 d))
("winston" nil (w ih1 n s t ax n))
("wint" nil (w ih1 n t))
("winter" nil (w ih1 n t er0))
("winterberg" nil (w ih1 n t er0 b er0 g))
("winterbourne" nil (w ih1 n t er0 b ao1 r n))
("wintered" nil (w ih1 n t er0 d))
("winterhalter" nil (w ih1 n t er0 hh ao1 l t er0))
("winterized" nil (w ih1 n t er0 ay1 z d))
("wintermute" nil (w ih1 n t er0 m y uw1 t))
("winterrowd" nil (w ih1 n t er0 aw0 d))
("winters" nil (w ih1 n t er0 z))
("wintershall" nil (w ih1 n t er0 sh ae1 l))
("wintersteen" nil (w ih1 n t er0 s t iy1 n))
("winterstein" nil (w ih1 n t er0 s t ay1 n))
("winterthur" nil (w ih1 n t er0 th er0))
("wintertime" nil (w ih1 n t er0 t ay1 m))
("winterton" nil (w ih1 n t er0 t ax n))
("winther" nil (w ih1 n th er0))
("winthrop" nil (w ih1 n th r ax p))
("wintle" nil (w ih1 n t ax l))
("winton" nil (w ih1 n t ax n))
("wintour" nil (w ih1 n t uh1 r))
("wintry" nil (w ih1 n t r iy0))
("wintz" nil (w ih1 n t s))
("wintzer" nil (w ih1 n t s er0))
("winward" nil (w ih1 n w er0 d))
("winwood" nil (w ih1 n w uh1 d))
("winzeler" nil (w ih1 n z ax l er0))
("winzer" nil (w ih1 n z er0))
("wion" nil (w ay1 ax n))
("wipe" nil (w ay1 p))
("wiped" nil (w ay1 p t))
("wipeout" nil (w ay1 p aw1 t))
("wiper" nil (w ay1 p er0))
("wipers" nil (w ay1 p er0 z))
("wipes" nil (w ay1 p s))
("wipf" nil (w ih1 p f))
("wiping" nil (w ay1 p ih0 ng))
("wipperfurth" nil (w ih1 p er0 f er0 th))
("wire" nil (w ay1 er0))
("wired" nil (w ay1 er0 d))
("wireless" nil (w ay1 r l ih0 s))
("wireline" nil (w ay1 r l ay1 n))
("wireman" nil (w ay1 r m ax n))
("wires" nil (w ay1 er0 z))
("wiretap" nil (w ay1 er0 t ae1 p))
("wiretapped" nil (w ay1 er0 t ae1 p t))
("wiretapping" nil (w ay1 er0 t ae1 p ih0 ng))
("wiretaps" nil (w ay1 r t ae1 p s))
("wirick" nil (w ih1 r ih0 k))
("wiring" nil (w ay1 r ih0 ng))
("wirkkala" nil (v er0 k aa1 l ax))
("wirkus" nil (w er1 k ih0 s))
("wirsing" nil (w er1 s ih0 ng))
("wirt" nil (w er1 t))
("wirtanen" nil (w er1 t ax n ax n))
("wirth" nil (w er1 th))
("wirthlin" nil (w er1 th l ih0 n))
("wirtz" nil (w er1 t s))
("wiry" nil (w ih1 r iy0))
("wirz" nil (w er1 z))
("wis" nil (w ih1 s))
("wisby" nil (w ih1 s b iy0))
("wisch" nil (w ih1 sh))
("wischmeyer" nil (w ih1 sh m ay0 er0))
("wisconsin" nil (w ih0 s k aa1 n s ax n))
("wisdom" nil (w ih1 z d ax m))
("wise" nil (w ay1 z))
("wisecarver" nil (w ay1 z k aa1 r v er0))
("wisecracking" nil (w ay1 z k r ae1 k ih0 ng))
("wisecracks" nil (w ay1 z k r ae1 k s))
("wisecup" nil (w ay1 z k ah1 p))
("wised" nil (w ay1 z d))
("wiseguy" nil (w ay1 z g ay1))
("wisehart" nil (w ay1 z hh aa1 r t))
("wisel" nil (w ay1 z ax l))
("wiseley" nil (w ih1 s ih0 l iy0))
("wisely" nil (w ay1 z l iy0))
("wiseman" nil (w ay1 z m ax n))
("wisenbaker" nil (w ay1 z ax n b ey1 k er0))
("wisener" nil (w ih1 s iy0 n er0))
("wiser" nil (w ay1 z er0))
("wisest" nil (w ay1 z ax s t))
("wish" nil (w ih1 sh))
("wisham" nil (w ih1 sh ax m))
("wishard" nil (w ih1 sh er0 d))
("wishart" nil (w ih1 sh aa1 r t))
("wishbone" nil (w ih1 sh b ow1 n))
("wished" nil (w ih1 sh t))
("wisher" nil (w ih1 sh er0))
("wishers" nil (w ih1 sh er0 z))
("wishes" nil (w ih1 sh ih0 z))
("wishful" nil (w ih1 sh f ax l))
("wishfully" nil (w ih1 sh f ax l iy0))
("wishing" nil (w ih1 sh ih0 ng))
("wishner" nil (w ih1 sh n er0))
("wishnick" nil (w ih1 sh n ih0 k))
("wishon" nil (w ih1 sh ax n))
("wishy" nil (w ih1 sh iy0))
("wisinski" nil (v ih0 s ih1 n s k iy0))
("wisler" nil (w ih1 s ax l er0))
("wisley" nil (w ih1 z l iy0))
("wisman" nil (w ih1 z m ax n))
("wismer" nil (w ih1 z ax m er0))
("wisner" nil (w ih1 s n er0))
("wisneski" nil (v ih0 s n eh1 s k iy0))
("wisnewski" nil (v ih0 s n eh1 f s k iy0))
("wisnieski" nil (v ih0 s n iy1 s k iy0))
("wisniewski" nil (w ih0 z n iy0 eh1 f s k iy0))
("wisor" nil (w ay1 z er0))
("wisp" nil (w ih1 s p))
("wispy" nil (w ih1 s p iy0))
("wiss" nil (w ih1 s))
("wissa" nil (w ih1 s ax))
("wissel" nil (w ih1 s ax l))
("wisser" nil (w ih1 s er0))
("wissing" nil (w ih1 s ih0 ng))
("wissinger" nil (w ih1 s ih0 ng er0))
("wissink" nil (w ih1 s ih0 ng k))
("wissler" nil (w ih1 s l er0))
("wissman" nil (w ih1 s m ax n))
("wissmann" nil (w ih1 s m ax n))
("wissner" nil (w ih1 s n er0))
("wist" nil (w ih1 s t))
("wistful" nil (w ih1 s t f ax l))
("wistfully" nil (w ih1 s t f ax l iy0))
("wistfulness" nil (w ih1 s t f ax l n ax s))
("wiswell" nil (w ih1 s w eh0 l))
("wisz" nil (v ih1 sh))
("wit" nil (w ih1 t))
("witbeck" nil (w ih1 t b eh1 k))
("witbrock" nil (w ih1 t b r ao1 k))
("witch" nil (w ih1 ch))
("witchcraft" nil (w ih1 ch k r ae1 f t))
("witcher" nil (w ih1 ch er0))
("witches" nil (w ih1 ch ax z))
("witchey" nil (w ih1 ch iy0))
("witching" nil (w ih1 ch ih0 ng))
("witco" nil (w ih1 t k ow0))
("witczak" nil (v ih1 t ch ae0 k))
("witek" nil (v ih1 t eh0 k))
("with" nil (w ih1 dh))
("witham" nil (w ih1 th ax m))
("withdraw" nil (w ih0 dh d r ao1))
("withdrawal" nil (w ih0 dh d r ao1 ax l))
("withdrawals" nil (w ih0 th d r ao1 ax l z))
("withdrawing" nil (w ih0 th d r ao1 ih0 ng))
("withdrawn" nil (w ih0 th d r ao1 n))
("withdraws" nil (w ih0 dh d r ao1 z))
("withdrew" nil (w ih0 th d r uw1))
("withee" nil (w ih1 th iy1))
("withem" nil (w ih1 th ih0 m))
("wither" nil (w ih1 dh er0))
("withered" nil (w ih1 dh er0 d))
("witherell" nil (w ih1 th er0 ax l))
("withering" nil (w ih1 dh er0 ih0 ng))
("witherington" nil (w ih1 th er0 ih0 ng t ax n))
("witherow" nil (w ih1 th er0 ow0))
("withers" nil (w ih1 dh er0 z))
("witherspoon" nil (w ih1 dh er0 s p uw1 n))
("withey" nil (w ih1 th iy0))
("withheld" nil (w ih0 th hh eh1 l d))
("withhold" nil (w ih0 th hh ow1 l d))
("withholding" nil (w ih0 th hh ow1 l d ih0 ng))
("withholds" nil (w ih1 th hh ow1 l d z))
("within" nil (w ih0 dh ih1 n))
("withington" nil (w ih1 th ih0 ng t ax n))
("withnail" nil (w ih0 th n ey1 l))
("without" nil (w ih0 th aw1 t))
("withrow" nil (w ih1 th r ow1))
("withstand" nil (w ih0 th s t ae1 n d))
("withstanding" nil (w ih0 th s t ae1 n d ih0 ng))
("withstands" nil (w ih0 th s t ae1 n d z))
("withstood" nil (w ih0 th s t uh1 d))
("witkin" nil (w ih1 t k ih1 n))
("witkop" nil (w ih1 t k ax p))
("witkowski" nil (v ih0 t k ao1 f s k iy0))
("witless" nil (w ih1 t l ax s))
("witman" nil (w ih1 t m ax n))
("witmer" nil (w ih1 t m er0))
("witness" nil (w ih1 t n ax s))
("witnessed" nil (w ih1 t n ax s t))
("witnesses" nil (w ih1 t n ax s ax z))
("witnessing" nil (w ih1 t n ax s ih0 ng))
("witowski" nil (v ih0 t ao1 f s k iy0))
("wits" nil (w ih1 t s))
("witt" nil (w ih1 t))
("witte" nil (w ih1 t))
("witted" nil (w ih1 t ih0 d))
("wittekind" nil (w ih1 t ih0 k ih0 n d))
("witteman" nil (w ih1 t m ax n))
("witten" nil (w ih1 t ax n))
("wittenauer" nil (w ih1 t ih0 n aw0 er0))
("wittenberg" nil (w ih1 t ax n b er0 g))
("wittenborn" nil (w ih1 t ih0 n b er0 n))
("wittenburg" nil (w ih1 t ax n b er0 g))
("wittenmyer" nil (w ih1 t ih0 n m iy0 er0))
("witter" nil (w ih1 t er0))
("witters" nil (w ih1 t er0 z))
("witthuhn" nil (w ih1 th ax n))
("wittig" nil (w ih1 t ih0 g))
("wittily" nil (w ih1 t ax l iy0))
("witting" nil (w ih1 t ih0 ng))
("wittingly" nil (w ih1 t ih0 ng l iy0))
("wittke" nil (w ih1 t k iy0))
("wittkopp" nil (w ih1 t k ax p))
("wittler" nil (w ih1 t l er0))
("wittman" nil (w ih1 t m ax n))
("wittmann" nil (w ih1 t m ax n))
("wittmer" nil (w ih1 t m er0))
("wittmeyer" nil (w ih1 t m ay0 er0))
("wittner" nil (w ih1 t n er0))
("witton" nil (w ih1 t ax n))
("wittrock" nil (w ih1 t r ax k))
("witts" nil (w ih1 t s))
("wittwer" nil (w ih1 t w er0))
("witty" nil (w ih1 t iy0))
("witucki" nil (w ih0 t ah1 k iy0))
("witwatersrand" nil (w ih1 t w ao1 t er0 s r ae0 n d))
("witwer" nil (w ih1 t w er0))
("witz" nil (w ih1 t s))
("witzel" nil (w ih1 t z ax l))
("witzig" nil (w ih1 t z ih0 g))
("witzke" nil (w ih1 t z k))
("wives" nil (w ay1 v z))
("wix" nil (w ih1 k s))
("wixom" nil (w ih1 k s ax m))
("wixon" nil (w ih1 k s ax n))
("wixson" nil (w ih1 k s ax n))
("wixted" nil (w ih1 k s t ih0 d))
("wiz" nil (w ih1 z))
("wizard" nil (w ih1 z er0 d))
("wizardry" nil (w ih1 z er0 d r iy0))
("wizards" nil (w ih1 z er0 d z))
("wizened" nil (w ay1 z ax n d))
("wlodarczyk" nil (w ax l aa1 d er0 ch ih0 k))
("wlodarski" nil (w ax l ax d aa1 r s k iy0))
("wm" nil (w ih1 l y ax m))
("wnek" nil (w n eh1 k))
("wnuk" nil (w n ah1 k))
("wobbe" nil (w aa1 b))
("wobble" nil (w aa1 b ax l))
("wobbled" nil (w aa1 b ax l d))
("wobbling" nil (w aa1 b ax l ih0 ng))
("wobbly" nil (w aa1 b ax l iy0))
("wobegon" nil (w ow1 b ax g aa0 n))
("wobig" nil (w ow1 b ih0 g))
("wobst" nil (w aa1 b s t))
("woburn" nil (w uw1 b er0 n))
("woe" nil (w ow1))
("woebegone" nil (w ow1 b ih0 g ao1 n))
("woeful" nil (w ow1 f ax l))
("woefully" nil (w ow1 f ax l iy0))
("woehl" nil (w ow1 l))
("woehler" nil (w ow1 l er0))
("woehr" nil (w ao1 r))
("woehrle" nil (w ao1 r ax l))
("woelfel" nil (w ow1 l f ax l))
("woelfle" nil (w ow1 l f ax l))
("woerner" nil (w ao1 r n er0))
("woes" nil (w ow1 z))
("woessner" nil (w ow1 s n er0))
("woeste" nil (w ow1 s t))
("wofford" nil (w aa1 f er0 d))
("wogan" nil (w ow1 g ax n))
("wogoman" nil (w ow1 g ow0 m ax n))
("wohl" nil (w ow1 l))
("wohler" nil (w ow1 l er0))
("wohlers" nil (w ow1 l er0 z))
("wohlfarth" nil (w ow1 l f aa1 r th))
("wohlfeil" nil (w ow1 l f ay1 l))
("wohlford" nil (w ow1 l f er0 d))
("wohlgemuth" nil (w ow1 l g ax m uw0 th))
("wohlstetter" nil (w ow1 l s t eh1 t er0))
("wohlwend" nil (w ow1 l w eh0 n d))
("woitschatzke" nil (w oy1 ch ae1 t s k iy1))
("wojahn" nil (w ow1 hh aa0 n))
("wojciak" nil (w oy1 ch iy0 ae0 k))
("wojcicki" nil (w oy1 ch ih1 t s k iy0))
("wojciech" nil (w oy1 ch eh0 k))
("wojciechowski" nil (w oy0 ch ih0 hh ao1 f s k iy0))
("wojcik" nil (w oy1 ch ih0 k))
("wojdyla" nil (w oy1 d ih1 l ax))
("wojick" nil (w oy1 ch ih0 k))
("wojnar" nil (w oy1 n er0))
("wojnarowski" nil (w oy1 n er0 aw1 s k iy0))
("wojnilower" nil (w oy1 n ax l ow1 er0))
("wojnowski" nil (w oy1 n aw1 s k iy0))
("wojtas" nil (w oy1 t ax s))
("wojtaszek" nil (w oy1 t aa1 sh eh0 k))
("wojtkiewicz" nil (w oy1 t k ax v ih0 ch))
("wojtkowski" nil (w oy1 t k aw1 s k iy0))
("wojtowicz" nil (w oy1 t ax v ih0 ch))
("wok" nil (w aa1 k))
("woke" nil (w ow1 k))
("woken" nil (w ow1 k ax n))
("wola" nil (w ow1 l ax))
("wolak" nil (w ow1 l ax k))
("wolanin" nil (w aa1 l ax n ih0 n))
("wolanski" nil (v ax l ae1 n s k iy0))
("wolaver" nil (w aa1 l ax v er0))
("wolber" nil (w ow1 l b er0))
("wolbert" nil (w ow1 l b er0 t))
("wolcott" nil (w ow1 l k ax t))
("wold" nil (w ow1 l d))
("wolden" nil (w ow1 l d ax n))
("woldt" nil (w ow1 l t))
("wolenzac" nil (w ow1 l ax n z ae1 k))
("wolf" nil (w uh1 l f))
("wolfarth" nil (w uh1 l f aa0 r th))
("wolfe" nil (w uh1 l f))
("wolfenbarger" nil (w uh1 l f ih0 n b aa0 r g er0))
("wolfenden" nil (w uh1 l f eh1 n d ax n))
("wolfensohn" nil (w uh1 l f ax n s ax n))
("wolfer" nil (w uh1 l f er0))
("wolfert" nil (w uh1 l f er0 t))
("wolff" nil (w uh1 l f))
("wolfgang" nil (w uh1 l f g ae1 ng))
("wolfgram" nil (w uh1 l f g r ae1 m))
("wolfinbarger" nil (w uh1 l f ih0 n b aa1 r g er0))
("wolfinger" nil (w uh1 l f ih0 ng er0))
("wolfley" nil (w uh1 l f l iy0))
("wolfman" nil (w uh1 l f m ax n))
("wolford" nil (w ow1 l f er0 d))
("wolfram" nil (w uh1 l f r ae1 m))
("wolfrey" nil (w uh1 l f r iy0))
("wolfrom" nil (w uh1 l f r ax m))
("wolfrum" nil (w uh1 l f r ax m))
("wolfsburg" nil (w uh1 l f s b er0 g))
("wolfson" nil (w uh1 l f s ax n))
("wolgamott" nil (w ow1 l g ax m aa0 t))
("wolgast" nil (w ow1 l g ax s t))
("wolgemuth" nil (w ow1 l g ih0 m uw0 th))
("wolin" nil (w ow1 l ih0 n))
("wolinski" nil (v ax l ih1 n s k iy0))
("wolinsky" nil (v ax l ih1 n s k iy0))
("wolitarsky" nil (w ow1 l ih0 t aa1 r s k iy0))
("wolk" nil (w ow1 k))
("wolke" nil (w ow1 l k))
("wolken" nil (w ow1 k ax n))
("wolkoff" nil (w ow1 l k ao0 f))
("woll" nil (w aa1 l))
("wollack" nil (w aa1 l ax k))
("wollaeger" nil (w ax l ey1 g er0))
("wollam" nil (w aa1 l ax m))
("wollard" nil (w aa1 l er0 d))
("wolle" nil (w aa1 l))
("wollen" nil (w aa1 l ax n))
("wollenberg" nil (w aa1 l ax n b er0 g))
("wollenweber" nil (w aa1 l ih0 n w ih0 b er0))
("woller" nil (w aa1 l er0))
("wollin" nil (w aa1 l ih0 n))
("wollman" nil (w aa1 l m ax n))
("wollner" nil (w aa1 l n er0))
("wollschlager" nil (w aa1 l sh l ey0 g er0))
("wolman" nil (w aa1 l m ax n))
("wolohan" nil (w aa1 l ax hh ae0 n))
("woloszyn" nil (v ax l aa1 sh ih0 n))
("wolpe" nil (w ow1 l p iy0))
("wolper" nil (w ow1 l p er0))
("wolpert" nil (w ow1 l p er0 t))
("wolrath" nil (w ow1 l r ae1 th))
("wolsey" nil (w ow1 l s iy0))
("wolsfeld" nil (w ow1 l z f eh1 l d))
("wolske" nil (w ow1 l s k))
("wolski" nil (v ow1 l s k iy0))
("wolsky" nil (v ow1 l s k iy0))
("wolstenholme" nil (w ow1 l s t ih0 n hh ow0 l m))
("wolter" nil (w ow1 l t er0))
("wolters" nil (w ow1 l t er0 z))
("woltman" nil (w ow1 l t m ax n))
("woltz" nil (w ow1 l t s))
("wolven" nil (w uh1 l v ax n))
("wolverine" nil (w uh1 l v er0 iy1 n))
("wolverton" nil (w uh0 l v er1 t ax n))
("wolves" nil (w uh1 l v z))
("wolz" nil (w ow1 l z))
("womac" nil (w ow1 m ax k))
("womack" nil (w ow1 m ae0 k))
("woman" nil (w uh1 m ax n))
("womanhood" nil (w uh1 m ax n hh uh1 d))
("womanizer" nil (w uh1 m ax n ay1 z er0))
("womanizing" nil (w uh1 m ax n ay1 z ih0 ng))
("womb" nil (w uw1 m))
("wombacher" nil (w aa1 m b aa1 k er0))
("womble" nil (w aa1 m b ax l))
("wombles" nil (w aa1 m b ax l z))
("women" nil (w ih1 m ax n))
("womer" nil (w ow1 m er0))
("wometco" nil (w ow0 m eh1 t k ow0))
("wommack" nil (w aa1 m ax k))
("won" nil (w ah1 n))
("wonda" nil (w aa1 n d ax))
("wonder" nil (w ah1 n d er0))
("wondered" nil (w ah1 n d er0 d))
("wonderful" nil (w ah1 n d er0 f ax l))
("wonderfully" nil (w ah1 n d er0 f ax l iy0))
("wonderfulness" nil (w ah1 n d er0 f ax l n ax s))
("wondering" nil (w ah1 n d er0 ih0 ng))
("wonderland" nil (w ah1 n d er0 l ae1 n d))
("wonderly" nil (w ah1 n d er0 l iy0))
("wonderment" nil (w ah1 n d er0 m ax n t))
("wonders" nil (w ah1 n d er0 z))
("wondra" nil (w aa1 n d r ax))
("wondrous" nil (w ah1 n d r ax s))
("wong" nil (w ao1 ng))
("wonnacott" nil (w ah1 n ax k aa0 t))
("wont" nil (w ow1 n t))
("woo" nil (w uw1))
("wood" nil (w uh1 d))
("woodall" nil (w uh1 d ao1 l))
("woodard" nil (w uh1 d er0 d))
("woodbeck" nil (w uh1 d b eh1 k))
("woodberry" nil (w uh1 d b eh1 r iy0))
("woodbridge" nil (w uh1 d b r ih1 jh))
("woodburn" nil (w uh1 d b er1 n))
("woodbury" nil (w uh1 d b eh1 r iy0))
("woodby" nil (w uh1 d b iy0))
("woodcarvers" nil (w uh1 d k aa1 r v er0 z))
("woodchuck" nil (w uh1 d ch ah1 k))
("woodcliff" nil (w uh1 d k l ih1 f))
("woodcock" nil (w uh1 d k aa1 k))
("woodcox" nil (w uh1 d k aa1 k s))
("wooddell" nil (w uh1 d ax l))
("wooded" nil (w uh1 d ih0 d))
("woodell" nil (w uh1 d ax l))
("wooden" nil (w uh1 d ax n))
("woodfield" nil (w uh1 d f iy1 l d))
("woodfill" nil (w uh1 d f ih1 l))
("woodfin" nil (w uh1 d f ih0 n))
("woodford" nil (w uh1 d f er0 d))
("woodfork" nil (w uh1 d f er0 k))
("woodhall" nil (w uh1 d hh ao1 l))
("woodham" nil (w uh1 d hh ax m))
("woodhams" nil (w uh1 d hh ax m z))
("woodhead" nil (w uh1 d hh eh1 d))
("woodhouse" nil (w uh1 d hh aw1 s))
("woodhull" nil (w uh1 d hh ah1 l))
("woodie" nil (w uh1 d iy0))
("woodin" nil (w uh1 d ih0 n))
("wooding" nil (w uh1 d ih0 ng))
("woodington" nil (w uh1 d ih0 ng t ax n))
("woodis" nil (w uh1 d ih0 s))
("woodke" nil (w uh1 d k iy0))
("woodland" nil (w uh1 d l ae1 n d))
("woodlands" nil (w uh1 d l ae1 n d z))
("woodle" nil (w uh1 d ax l))
("woodlee" nil (w uh1 d l iy1))
("woodley" nil (w uh1 d l iy0))
("woodlief" nil (w uh1 d l iy1 f))
("woodliff" nil (w uh1 d l ih0 f))
("woodling" nil (w uh1 d l ih0 ng))
("woodlock" nil (w uh1 d l aa1 k))
("woodlot" nil (w uh1 d l aa1 t))
("woodmac" nil (w uh1 d m ae0 k))
("woodman" nil (w uh1 d m ax n))
("woodmansee" nil (w uh0 d m ax n s iy1))
("woodpecker" nil (w uh1 d p eh1 k er0))
("woodpeckers" nil (w uh1 d p eh1 k er0 z))
("woodrich" nil (w uh1 d r ih1 ch))
("woodrick" nil (w uh1 d r ih0 k))
("woodring" nil (w uh1 d r ih1 ng))
("woodrome" nil (w uh1 d r ax m))
("woodroof" nil (w uh1 d r uw1 f))
("woodrow" nil (w uh1 d r ow1))
("woodruff" nil (w uh1 d r ax f))
("woodrum" nil (w uh1 d r ax m))
("woods" nil (w uh1 d z))
("woodside" nil (w uh1 d s ay1 d))
("woodsmall" nil (w uh1 d s m ao1 l))
("woodson" nil (w uh1 d s ax n))
("woodstock" nil (w uh1 d s t aa1 k))
("woodstream" nil (w uh1 d s t r iy1 m))
("woodward" nil (w uh1 d w ao1 r d))
("woodwind" nil (w uh1 d w ih1 n d))
("woodwinds" nil (w uh1 d w ih1 n d z))
("woodwork" nil (w uh1 d w er1 k))
("woodworker" nil (w uh1 d w er1 k er0))
("woodworkers" nil (w uh1 d w er1 k er0 z))
("woodworking" nil (w uh1 d w er1 k ih0 ng))
("woodworth" nil (w uh1 d w er1 th))
("woody" nil (w uh1 d iy0))
("woodyard" nil (w uh1 d y aa1 r d))
("wooed" nil (w uw1 d))
("woof" nil (w uw1 f))
("woofter" nil (w uw1 f t er0))
("woogie" nil (w uw1 g iy0))
("wooing" nil (w uw1 ih0 ng))
("wool" nil (w uh1 l))
("woolard" nil (w uh1 l er0 d))
("woolbright" nil (w uh1 l b r ay1 t))
("woolco" nil (w uh1 l k ow1))
("woolcock" nil (w uh1 l k aa1 k))
("wooldridge" nil (w uh1 l d r ih0 jh))
("woolen" nil (w uh1 l ax n))
("woolens" nil (w uh1 l ax n z))
("woolery" nil (w uh1 l er0 iy0))
("woolever" nil (w uh1 l eh1 v er0))
("wooley" nil (w uh1 l iy0))
("woolf" nil (w uh1 l f))
("woolfolk" nil (w uh1 l f ow1 k))
("woolford" nil (w uh1 l f er0 d))
("woolfork" nil (w uh1 l f ao1 r k))
("woollard" nil (w uh1 l er0 d))
("woollcott" nil (w uh1 l k aa0 t))
("woollen" nil (w uh1 l ax n))
("woolley" nil (w uh1 l iy0))
("woolly" nil (w uh1 l iy0))
("woolman" nil (w uh1 l m ax n))
("woolridge" nil (w uh1 l r ih1 jh))
("wools" nil (w uh1 l z))
("woolsey" nil (w uh1 l z iy0))
("woolson" nil (w uh1 l s ax n))
("woolston" nil (w uh1 l s t ax n))
("woolum" nil (w uh1 l ax m))
("woolums" nil (w uh1 l ax m z))
("woolverton" nil (w uh1 l v er0 t ax n))
("woolwine" nil (w uh1 l w ay1 n))
("woolworth" nil (w uh1 l w er1 th))
("wooly" nil (w uh1 l iy0))
("woomer" nil (w uw1 m er0))
("woong" nil (w uw1 ng))
("woos" nil (w uw1 z))
("woosley" nil (w uw1 z l iy0))
("wooster" nil (w uw1 s t er0))
("wootan" nil (w uw1 t ax n))
("wooten" nil (w uw1 t ax n))
("wooters" nil (w uw1 t er0 z))
("wooton" nil (w uw1 t ax n))
("wootten" nil (w uw1 t ax n))
("wootton" nil (w uw1 t ax n))
("woozy" nil (w uw1 z iy0))
("wop" nil (w aa1 p))
("wops" nil (w aa1 p s))
("worcester" nil (w uh1 s t er0))
("word" nil (w er1 d))
("worded" nil (w er1 d ih0 d))
("wordell" nil (w er1 d ax l))
("worden" nil (w er1 d ax n))
("wording" nil (w er1 d ih0 ng))
("wordless" nil (w er1 d l ax s))
("wordperfect" nil (w er1 d p er1 f eh0 k t))
("words" nil (w er1 d z))
("wordsmith" nil (w er1 d s m ih1 th))
("wordstar" nil (w er1 d s t aa1 r))
("wordsworth" nil (w er1 d z w er0 th))
("wore" nil (w ao1 r))
("work" nil (w er1 k))
("workable" nil (w er1 k ax b ax l))
("workaday" nil (w er1 k ax d ey1))
("workaholic" nil (w er1 k ax hh aa1 l ih0 k))
("workaholics" nil (w er1 k ax hh aa1 l ih0 k s))
("workbench" nil (w er1 k b eh1 n ch))
("workday" nil (w er1 k d ey1))
("workdays" nil (w er1 k d ey1 z))
("worked" nil (w er1 k t))
("worker" nil (w er1 k er0))
("workers" nil (w er1 k er0 z))
("workfare" nil (w er1 k f eh1 r))
("workforce" nil (w er1 k f ao1 r s))
("workgroup" nil (w er1 k g r uw1 p))
("workgroups" nil (w er1 k g r uw1 p s))
("workhorse" nil (w er1 k hh ao1 r s))
("workhorses" nil (w er1 k hh ao1 r s ih0 z))
("working" nil (w er1 k ih0 ng))
("workinger" nil (w er1 k ih0 ng er0))
("workings" nil (w er1 k ih0 ng z))
("workload" nil (w er1 k l ow1 d))
("workloads" nil (w er1 k l ow1 d z))
("workman" nil (w er1 k m ax n))
("workmanlike" nil (w er1 k m ae1 n l ay1 k))
("workmanship" nil (w er1 k m ax n sh ih1 p))
("workmen" nil (w er1 k m ih0 n))
("workout" nil (w er1 k aw1 t))
("workouts" nil (w er1 k aw1 t s))
("workplace" nil (w er1 k p l ey1 s))
("workplaces" nil (w er1 k p l ey1 s ih0 z))
("workroom" nil (w er1 k r uw1 m))
("works" nil (w er1 k s))
("worksheet" nil (w er1 k sh iy1 t))
("worksheets" nil (w er1 k sh iy1 t s))
("workshop" nil (w er1 k sh aa1 p))
("workshops" nil (w er1 k sh aa1 p s))
("workstation" nil (w er1 k s t ey1 sh ax n))
("workstations" nil (w er1 k s t ey1 sh ax n z))
("workweek" nil (w er1 k w iy1 k))
("workweeks" nil (w er1 k w iy1 k s))
("worland" nil (w er1 l ax n d))
("worlco" nil (w er1 l k ow0))
("world" nil (w er1 l d))
("worldcom" nil (w er1 l d k aa1 m))
("worldcorp" nil (w er1 l d k ao1 r p))
("worldliness" nil (w er1 l d l iy0 n ax s))
("worldly" nil (w er1 l d l iy0))
("worldnews" nil (w er1 l d n uw1 z))
("worldpass" nil (w er1 l d p ae1 s))
("worlds" nil (w er1 l d z))
("worldscope" nil (w er1 l d s k ow1 p))
("worldspan" nil (w er1 l d s p ae1 n))
("worldvision" nil (w er1 l d v ih1 zh ax n))
("worldwide" nil (w er1 l d w ay1 d))
("worley" nil (w er1 l iy0))
("worm" nil (w er1 m))
("worman" nil (w er1 m ax n))
("wormington" nil (w er1 m ih0 ng t ax n))
("wormley" nil (w er1 m l iy0))
("wormlike" nil (w er1 m l ay1 k))
("worms" nil (w er1 m z))
("wormser" nil (w er1 m z er0))
("wormuth" nil (w er1 m uw0 th))
("wormwood" nil (w er1 m w uh1 d))
("worn" nil (w ao1 r n))
("worner" nil (w ao1 r n er0))
("woronoff" nil (w ao1 r ax n ao0 f))
("worrall" nil (w er1 r ax l))
("worrel" nil (w er1 r ax l))
("worrell" nil (w er1 r ax l))
("worried" nil (w er1 iy0 d))
("worriers" nil (w er1 iy0 er0 z))
("worries" nil (w er1 iy0 z))
("worrisome" nil (w er1 iy0 s ax m))
("worry" nil (w er1 iy0))
("worrying" nil (w er1 iy0 ih0 ng))
("worse" nil (w er1 s))
("worseck" nil (w er1 s eh0 k))
("worsen" nil (w er1 s ax n))
("worsened" nil (w er1 s ax n d))
("worsening" nil (w er1 s ax n ih0 ng))
("worsens" nil (w er1 s ax n z))
("worsham" nil (w er1 sh ax m))
("worship" nil (w er1 sh ax p))
("worshiped" nil (w er1 sh ih0 p t))
("worshiper" nil (w er1 sh ax p er0))
("worshipers" nil (w er1 sh ih0 p er0 z))
("worshipful" nil (w er1 sh ax p f ax l))
("worshipped" nil (w er1 sh ax p t))
("worshippers" nil (w er1 sh ih0 p er0 z))
("worshipping" nil (w er1 sh ax p ih0 ng))
("worships" nil (w er1 sh ax p s))
("worsley" nil (w er1 s l iy0))
("worst" nil (w er1 s t))
("worsted" nil (w er1 s t ih0 d))
("worstell" nil (w er1 s t ax l))
("worster" nil (w er1 s t er0))
("worth" nil (w er1 th))
("wortham" nil (w er1 th ax m))
("worthen" nil (w er1 th ax n))
("worthey" nil (w er1 dh iy0))
("worthier" nil (w er1 dh iy0 er0))
("worthies" nil (w er1 dh iy0 z))
("worthiest" nil (w er1 dh iy0 ax s t))
("worthiness" nil (w er1 dh iy0 n ih0 s))
("worthing" nil (w er1 th ih0 ng))
("worthington" nil (w er1 dh ih0 ng t ax n))
("worthless" nil (w er1 th l ax s))
("worthley" nil (w er1 th l iy0))
("worthman" nil (w er1 th m ax n))
("worths" nil (w er1 th s))
("worthwhile" nil (w er1 th w ay1 l))
("worthy" nil (w er1 dh iy0))
("wortley" nil (w er1 t l iy0))
("wortman" nil (w er1 t m ax n))
("wortmann" nil (w er1 t m ax n))
("worton" nil (w er1 t ax n))
("worts" nil (w er1 t s))
("wotan" nil (w ow1 t ax n))
("wotherspoon" nil (w ax dh er0 s p uw1 n))
("wotring" nil (w aa1 t er0 ih0 ng))
("wotton" nil (w aa1 t ax n))
("wouk" nil (w uw1 k))
("would" nil (w uh1 d))
("woulfe" nil (w aw1 l f))
("wound" nil (w aw1 n d))
("wounded" nil (w uw1 n d ax d))
("wounding" nil (w uw1 n d ih0 ng))
("wounds" nil (w uw1 n d z))
("wouters" nil (w aw1 t er0 z))
("wove" nil (w ow1 v))
("woven" nil (w ow1 v ax n))
("wow" nil (w aw1))
("wowed" nil (w aw1 d))
("wows" nil (w aw1 z))
("woy" nil (w oy1))
("wozniak" nil (v aa1 z n iy0 ae0 k))
("woznick" nil (w aa1 z n ih0 k))
("woznicki" nil (v ax z n ih1 t s k iy0))
("wozny" nil (w aa1 z n iy0))
("wrack" nil (r ae1 k))
("wracked" nil (r ae1 k t))
("wracking" nil (r ae1 k ih0 ng))
("wrage" nil (r ey1 jh))
("wragg" nil (r ae1 g))
("wrangell" nil (r ae1 ng g ax l))
("wrangle" nil (r ae1 ng g ax l))
("wrangled" nil (r ae1 ng g ax l d))
("wrangler" nil (r ae1 ng g ax l er0))
("wranglers" nil (r ae1 ng g ax l er0 z))
("wrangles" nil (r ae1 ng g ax l z))
("wrangling" nil (r ae1 ng g ax l ih0 ng))
("wrap" nil (r ae1 p))
("wrapped" nil (r ae1 p t))
("wrapper" nil (r ae1 p er0))
("wrappers" nil (r ae1 p er0 z))
("wrapping" nil (r ae1 p ih0 ng))
("wraps" nil (r ae1 p s))
("wrath" nil (r ae1 th))
("wrather" nil (r ae1 dh er0))
("wray" nil (r ey1))
("wreak" nil (r iy1 k))
("wreaked" nil (r iy1 k t))
("wreaking" nil (r iy1 k ih0 ng))
("wreaks" nil (r iy1 k s))
("wreath" nil (r iy1 th))
("wreaths" nil (r iy1 th s))
("wreck" nil (r eh1 k))
("wreckage" nil (r eh1 k ax jh))
("wrecked" nil (r eh1 k t))
("wrecker" nil (r eh1 k er0))
("wrecking" nil (r eh1 k ih0 ng))
("wrecks" nil (r eh1 k s))
("wrede" nil (r iy1 d))
("wren" nil (r eh1 n))
("wrench" nil (r eh1 n ch))
("wrenched" nil (r eh1 n ch t))
("wrenches" nil (r eh1 n ch ih0 z))
("wrenching" nil (r eh1 n ch ih0 ng))
("wrenn" nil (r eh1 n))
("wrest" nil (r eh1 s t))
("wrested" nil (r eh1 s t ih0 d))
("wresting" nil (r eh1 s t ih0 ng))
("wrestle" nil (r eh1 s ax l))
("wrestled" nil (r eh1 s ax l d))
("wrestler" nil (r eh1 s ax l er0))
("wrestlers" nil (r eh1 s t l er0 z))
("wrestles" nil (r eh1 s ax l z))
("wrestling" nil (r eh1 s l ih0 ng))
("wretched" nil (r eh1 ch ih0 d))
("wretchedly" nil (r eh1 ch ih0 d l iy0))
("wriggle" nil (r ih1 g ax l))
("wright" nil (r ay1 t))
("wrights" nil (r ay1 t s))
("wrightsman" nil (r ay1 t s m ax n))
("wrightson" nil (r ay1 t s ax n))
("wrigley" nil (r ih1 g l iy0))
("wring" nil (r ih1 ng))
("wringer" nil (r ih1 ng er0))
("wringing" nil (r ih1 ng ih0 ng))
("wrinkle" nil (r ih1 ng k ax l))
("wrinkled" nil (r ih1 ng k ax l d))
("wrinkles" nil (r ih1 ng k ax l z))
("wrinkling" nil (r ih1 ng k ax l ih0 ng))
("wrisley" nil (r ih1 z l iy0))
("wrist" nil (r ih1 s t))
("wriston" nil (r ih1 s t ax n))
("wrists" nil (r ih1 s t s))
("wristwatch" nil (r ih1 s t w aa1 ch))
("wristwatches" nil (r ih1 s t w aa1 ch ih0 z))
("writ" nil (r ih1 t))
("write" nil (r ay1 t))
("writedown" nil (r ay1 t d aw1 n))
("writedowns" nil (r ay1 t d aw1 n z))
("writeoff" nil (r ih1 t ao0 f))
("writeoffs" nil (r ih1 t ao0 f s))
("writer" nil (r ay1 t er0))
("writers" nil (r ay1 t er0 z))
("writes" nil (r ay1 t s))
("writhing" nil (r ay1 dh ih0 ng))
("writing" nil (r ay1 t ih0 ng))
("writings" nil (r ay1 t ih0 ng z))
("writs" nil (r ih1 t s))
("written" nil (r ih1 t ax n))
("wrobel" nil (r ow1 b ax l))
("wrobleski" nil (r ax b l eh1 s k iy0))
("wroblewski" nil (r ax b l uw1 f s k iy0))
("wroclaw" nil (r aa1 k l ao1))
("wroe" nil (r ow1))
("wrona" nil (r ow1 n ax))
("wrong" nil (r ao1 ng))
("wrongdoers" nil (r ao1 ng d uw1 er0 z))
("wrongdoing" nil (r ao1 ng d uw0 ih0 ng))
("wrongdoings" nil (r ao0 ng d uw1 ih0 ng z))
("wronged" nil (r ao1 ng d))
("wrongful" nil (r ao1 ng f ax l))
("wrongfully" nil (r ao1 ng f ax l iy0))
("wrongheaded" nil (r ao1 ng hh eh1 d ih0 d))
("wrongly" nil (r ao1 ng l iy0))
("wrongs" nil (r ao1 ng z))
("wronski" nil (r aa1 n s k iy0))
("wrote" nil (r ow1 t))
("wroten" nil (r ow1 t ax n))
("wrought" nil (r ao1 t))
("wrubel" nil (r uw1 b ax l))
("wruck" nil (r ah1 k))
("wrung" nil (r ah1 ng))
("wry" nil (r ay1))
("wrye" nil (r ay1))
("wryly" nil (r ay1 l iy0))
("wryness" nil (r ay1 n ax s))
("wrzesinski" nil (r ax z ih0 s ih1 n s k iy0))
("ws" nil (d ah1 b ax l y uw0 eh1 s))
("wu" nil (w uw1))
("wubben" nil (w ah1 b ax n))
("wuellner" nil (w uh1 l n er0))
("wuensch" nil (w uh1 n sh))
("wuerth" nil (w uh1 r th))
("wuerttemberg" nil (w er1 t ax m b er0 g))
("wuertz" nil (w uh1 r t s))
("wuest" nil (w uw1 ih0 s t))
("wuhan" nil (w uw1 ax n))
("wujcik" nil (w ah1 jh s ih0 k))
("wulf" nil (w uh1 l f))
("wulff" nil (w uh1 l f))
("wuliger" nil (w uw1 l ih0 g er0))
("wunder" nil (w ah1 n d er0))
("wunderkind" nil (w ah1 n d er0 k ay1 n d))
("wunderle" nil (w ah1 n d er0 l iy0))
("wunderlich" nil (w ah1 n d er0 l ih0 k))
("wunderlin" nil (w ah1 n d er0 l ih0 n))
("wunsch" nil (w ah1 n sh))
("wunschel" nil (w ah1 n sh ax l))
("wurdeman" nil (w er1 d m ax n))
("wurl" nil (w er1 l))
("wurlitzer" nil (w er1 l ih0 t s er0))
("wurm" nil (w er1 m))
("wurst" nil (w er1 s t))
("wurster" nil (w er1 s t er0))
("wursthaus" nil (w er1 s t hh aw0 s))
("wurth" nil (w er1 th))
("wurtman" nil (w er1 t m ax n))
("wurtz" nil (w er1 t s))
("wurtzel" nil (w er1 t z ax l))
("wurz" nil (w er1 z))
("wurzel" nil (w er1 z ax l))
("wurzer" nil (w er1 z er0))
("wussler" nil (w ah1 s l er0))
("wussow" nil (w ah1 s ow0))
("wuthering" nil (w ah1 dh er0 ih0 ng))
("wuthrich" nil (w ah1 th r ih0 k))
("wuttke" nil (w ah1 t k iy0))
("wuxi" nil (w ah1 k s iy0))
("wy" nil (w ey1))
("wyand" nil (w ay1 ax n d))
("wyant" nil (w ay1 ax n t))
("wyatt" nil (w ay1 ax t))
("wyble" nil (w ay1 b ax l))
("wyborn" nil (v ih1 b er0 n))
("wyche" nil (w ay1 ch))
("wyckoff" nil (w ih1 k ao0 f))
("wycliff" nil (w ih1 k l ih0 f))
("wycoff" nil (w ih1 k ao0 f))
("wyden" nil (w ay1 d ax n))
("wydra" nil (w ay1 d r ax))
("wye" nil (w ay1))
("wyer" nil (w ay1 er0))
("wyers" nil (w ay1 er0 z))
("wyeth" nil (w ay1 eh0 th))
("wyffels" nil (w ih1 f ax l z))
("wygal" nil (w ay1 g ax l))
("wygant" nil (w ay1 g ax n t))
("wygod" nil (w ay1 g aa1 d))
("wyk" nil (w ih1 k))
("wyka" nil (v ih1 k ax))
("wyke" nil (w ay1 k))
("wykes" nil (w ay1 k s))
("wykle" nil (w ay1 k ax l))
("wykoff" nil (w ih1 k ao0 f))
("wyland" nil (w ay1 l ax n d))
("wylde" nil (w ay1 l d))
("wyle" nil (w ay1 l))
("wyler" nil (w ay1 l er0))
("wyles" nil (w ay1 l z))
("wylie" nil (w ay1 l iy0))
("wyllie" nil (w ih1 l iy0))
("wylma" nil (w ay1 l m ax))
("wyly" nil (w ay1 l iy0))
("wyman" nil (w ay1 m ax n))
("wymer" nil (w ay1 m er0))
("wymore" nil (w ay1 m ao0 r))
("wyndham" nil (w ih1 n d ax m))
("wyne" nil (w ay1 n))
("wyner" nil (w ay1 n er0))
("wynes" nil (w ay1 n z))
("wyngaarden" nil (w ih1 n g aa1 r d ax n))
("wynia" nil (w ih1 n iy0 ax))
("wynkoop" nil (w ih1 ng k uw1 p))
("wynn" nil (w ih1 n))
("wynne" nil (w ih1 n))
("wynns" nil (w ih1 n z))
("wynter" nil (w ih1 n t er0))
("wynton" nil (w ih1 n t ax n))
("wynyard" nil (w ih1 n y er0 d))
("wyoming" nil (w ay1 ow1 m ih0 ng))
("wyre" nil (w ay1 r))
("wyrick" nil (w ih1 r ih0 k))
("wyse" nil (w ay1 z))
("wyser" nil (w ay1 z er0))
("wysocki" nil (v ih0 s ow1 t s k iy0))
("wysong" nil (w ih1 s ao0 ng))
("wyss" nil (w ih1 s))
("wyszynski" nil (v ih0 sh ih1 n s k iy0))
("wythe" nil (w ay1 dh))
("x" nil (eh1 k s))
("xanax" nil (z ae1 n ae0 k s))
("xanthe" nil (z ae1 n dh))
("xantippe" nil (z ae1 n t ih0 p))
("xaver" nil (z ey1 v er0))
("xavier" nil (z ey1 v y er0))
("xaviera" nil (ih0 g z ey0 v y eh1 r ax))
("xebec" nil (z eh1 b ax k))
("xena" nil (z iy1 n ax))
("xene" nil (z iy1 n))
("xenia" nil (z iy1 n iy0 ax))
("xenon" nil (z iy1 n aa0 n))
("xenophobia" nil (z eh1 n ax f ow1 b iy0 ax))
("xenophobic" nil (z eh1 n ax f aa1 b ih0 k))
("xenophon" nil (z eh1 n ax f ax n))
("xenos" nil (z iy1 n ow0 z))
("xeriscape" nil (z eh1 r ih0 s k ey1 p))
("xerox" nil (z ih1 r aa0 k s))
("xerxes" nil (z er1 k s iy0 z))
("xiamen" nil (z iy0 ey1 m ax n))
("xian" nil (z y aa1 n))
("xiaoping" nil (zh aw1 p ih1 ng))
("xiaoyun" nil (sh aw1 y uw1 n))
("xicor" nil (z ay1 k ao1 r))
("xidex" nil (z ih1 d ax k s))
("xie" nil (z iy1))
("xilinx" nil (z ay1 l ih1 ng k s))
("xing" nil (sh ih1 ng))
("xinhua" nil (z ih1 n w aa1))
("xinjiang" nil (z ih1 n jh iy0 ae1 ng))
("xiong" nil (zh ao1 ng))
("xoma" nil (z ow1 m ax))
("xscribe" nil (eh0 k s k r ay1 b))
("xtra" nil (eh1 k s t r ax))
("xu" nil (z uw1))
("xuan" nil (zh w aa1 n))
("xudong" nil (z uw1 d ao1 ng))
("xue" nil (z uw1))
("xuedong" nil (z uw0 d ao1 ng))
("xuemei" nil (z uw1 m ay1))
("xus" nil (z ah1 s))
("xylia" nil (z ay1 l iy0 ax))
("xylina" nil (z ay1 l ih0 n ax))
("xylogics" nil (z ay1 l aa1 jh ih0 k s))
("xylon" nil (z ay1 l ax n))
("xylona" nil (z ay1 l ax n ax))
("xylophone" nil (z ay1 l ax f ow1 n))
("xyvision" nil (z ay1 v ih1 zh ax n))
("y" nil (w ay1))
("ya" nil (y aa1))
("yaacov" nil (y aa1 k ow1 v))
("yablon" nil (y ae1 b l ax n))
("yablonski" nil (y ax b l aa1 n s k iy0))
("yablonsky" nil (y ax b l aa1 n s k iy0))
("yacht" nil (y aa1 t))
("yachters" nil (y aa1 t er0 z))
("yachting" nil (y aa1 t ih0 ng))
("yachts" nil (y aa1 t s))
("yachtsman" nil (y aa1 t s m ax n))
("yack" nil (y ae1 k))
("yackel" nil (y ae1 k ax l))
("yackley" nil (y ae1 k l iy0))
("yacko" nil (y ae1 k ow0))
("yacktman" nil (y ae1 k t m ax n))
("yacono" nil (y ae1 k ax n ow0))
("yacos" nil (y aa1 k ow0 s))
("yacoub" nil (y ae1 k aw0 b))
("yaden" nil (y ey1 d ax n))
("yadon" nil (y ae1 d ax n))
("yaeger" nil (y iy1 jh er0))
("yaffe" nil (y ae1 f))
("yager" nil (y ey1 jh er0))
("yagi" nil (y aa1 g iy0))
("yagoda" nil (y ax g ow1 d ax))
("yahi" nil (y aa1 hh iy0))
("yahn" nil (y ae1 n))
("yahnke" nil (y ae1 ng k))
("yahoo" nil (y aa1 hh uw1))
("yahoos" nil (y aa1 hh uw1 z))
("yahr" nil (y aa1 r))
("yak" nil (y ae1 k))
("yake" nil (y ey1 k))
("yakel" nil (y ae1 k ax l))
("yakima" nil (y ae1 k ih0 m ax))
("yaklin" nil (y ae1 k l ih0 n))
("yakov" nil (y aa1 k ao1 v))
("yakovlev" nil (y ax k aa1 v l ax v))
("yale" nil (y ey1 l))
("yalin" nil (y aa1 l ih1 n))
("yalta" nil (y ao1 l t ax))
("yam" nil (y ae1 m))
("yamada" nil (y ax m aa1 d ax))
("yamagata" nil (y aa0 m aa0 g aa1 t ax))
("yamaguchi" nil (y aa1 m aa0 g uw1 ch iy0))
("yamaha" nil (y aa1 m aa0 hh aa1))
("yamaichi" nil (y aa1 m aa0 iy1 ch iy0))
("yamamoto" nil (y aa0 m aa0 m ow1 t ow0))
("yamanaka" nil (y aa1 m aa0 n aa1 k ax))
("yamane" nil (y aa0 m aa1 n ey0))
("yamani" nil (y aa0 m aa1 n iy0))
("yamanouchi" nil (y aa0 m aa0 n uw1 ch iy0))
("yamasaki" nil (y aa1 m aa0 s aa1 k iy0))
("yamashiro" nil (y aa0 m aa0 sh ih1 r ow0))
("yamashita" nil (y aa0 m aa0 sh iy1 t ax))
("yamatake" nil (y aa1 m aa0 t aa1 k ey1))
("yamato" nil (y aa0 m aa1 t ow0))
("yamauchi" nil (y aa1 m aa0 uw1 ch iy0))
("yamazaki" nil (y aa1 m aa0 z aa1 k iy0))
("yamin" nil (y ae1 m ih0 n))
("yams" nil (y ae1 m z))
("yan" nil (y ae1 n))
("yanbu" nil (y ae1 n b uw0))
("yancey" nil (y ae1 n s iy0))
("yancy" nil (y ae1 n s iy0))
("yanda" nil (y ae1 n d ax))
("yandell" nil (y ae1 n d ax l))
("yandle" nil (y ae1 n d ax l))
("yandow" nil (y ae1 n d ow0))
("yanes" nil (y ey1 n z))
("yaney" nil (y ey1 n iy0))
("yanez" nil (y ae1 n ih0 z))
("yang" nil (y ae1 ng))
("yangsheng" nil (y aa1 ng sh eh1 ng))
("yangtze" nil (y ae1 ng k t s iy1))
("yani" nil (y aa1 n iy0))
("yanick" nil (y ae1 n ih0 k))
("yanik" nil (y ae1 n ih0 k))
("yanis" nil (y ae1 n ax s))
("yanish" nil (y ae1 n ih0 sh))
("yank" nil (y ae1 ng k))
("yanke" nil (y ae1 ng k))
("yanked" nil (y ae1 ng k t))
("yankee" nil (y ae1 ng k iy0))
("yankees" nil (y ae1 ng k iy0 z))
("yankelovich" nil (y ae1 ng k eh1 l ax v ih0 ch))
("yankey" nil (y ae1 ng k iy0))
("yanking" nil (y ae1 ng k ih0 ng))
("yanko" nil (y ae1 ng k ow0))
("yankovich" nil (y ae1 ng k ax v ih0 ch))
("yankowski" nil (y ax ng k ao1 f s k iy0))
("yanks" nil (y ae1 ng k s))
("yanni" nil (y ae1 n iy0))
("yannick" nil (y ae1 n ih0 k))
("yannone" nil (y ae1 n ax n))
("yannuzzi" nil (y aa0 n uw1 t s iy0))
("yano" nil (y aa1 n ow0))
("yanofsky" nil (y ax n aa1 f s k iy0))
("yant" nil (y ae1 n t))
("yantis" nil (y ae1 n t ih0 s))
("yantz" nil (y ae1 n t s))
("yao" nil (y aw1))
("yaobang" nil (y aw1 b ae0 ng))
("yaohan" nil (y aw1 hh aa1 n))
("yap" nil (y ae1 p))
("yaple" nil (y ey1 p ax l))
("yapp" nil (y ae1 p))
("yaps" nil (y ae1 p s))
("yarber" nil (y aa1 r b er0))
("yarberry" nil (y aa1 r b eh1 r iy0))
("yarboro" nil (y aa1 r b er0 ow0))
("yarborough" nil (y aa1 r b er1 ow0))
("yarbough" nil (y aa1 r b aw1))
("yarbro" nil (y aa1 r b r ow0))
("yarbrough" nil (y aa1 r b er0 ow0))
("yarchoan" nil (y aa0 r ch ow1 n))
("yard" nil (y aa1 r d))
("yardage" nil (y aa1 r d ih0 jh))
("yarde" nil (y aa1 r d))
("yardeni" nil (y aa0 r d iy1 n iy0))
("yarder" nil (y aa1 r d er0))
("yardley" nil (y aa1 r d l iy0))
("yards" nil (y aa1 r d z))
("yardstick" nil (y aa1 r d s t ih1 k))
("yardsticks" nil (y aa1 r d s t ih1 k s))
("yarger" nil (y aa1 r jh er0))
("yarling" nil (y aa1 r l ih0 ng))
("yarmulkes" nil (y aa1 r m ax l k s))
("yarn" nil (y aa1 r n))
("yarnall" nil (y aa1 r n ax l))
("yarnell" nil (y aa1 r n ax l))
("yarns" nil (y aa1 r n z))
("yaros" nil (y eh1 r ow0 z))
("yarosh" nil (y ae1 r ax sh))
("yarrington" nil (y ae1 r ih0 ng t ax n))
("yarrow" nil (y eh1 r ow0))
("yarwood" nil (y aa1 r w uh1 d))
("yasir" nil (y aa1 s er0))
("yasmine" nil (y ae1 z m ih0 n))
("yass" nil (y ae1 s))
("yasser" nil (y ae1 s er0))
("yassir" nil (y ae1 s ih0 r))
("yassukovich" nil (y ax s uw1 k ax v ih0 ch))
("yastrow" nil (y ae1 s t r ow0))
("yasuda" nil (y aa0 s uw1 d ax))
("yasuhiko" nil (y aa1 s uw0 hh iy1 k ow0))
("yasuhiro" nil (y aa1 s uw0 hh iy1 r ow0))
("yasunori" nil (y aa1 s uw0 n ao1 r iy0))
("yasuo" nil (y aa0 s uw1 ow0))
("yasushi" nil (y aa0 s uw1 sh iy0))
("yasutaka" nil (y aa1 s uw0 t aa1 k ax))
("yasuyoshi" nil (y aa1 s uw0 y ow1 sh iy0))
("yater" nil (y ey1 t er0))
("yates" nil (y ey1 t s))
("yatsko" nil (y aa1 t s k ow0))
("yau" nil (y ow1))
("yauch" nil (y ao1 ch))
("yauger" nil (y ao1 jh er0))
("yaun" nil (y ao1 n))
("yaupon" nil (y ao1 p ax n))
("yaupons" nil (y ao1 p ax n z))
("yavlinsky" nil (y ae1 v l ih1 n s k iy0))
("yaw" nil (y ao1))
("yawn" nil (y ao1 n))
("yawned" nil (y ao1 n d))
("yawner" nil (y ao1 n er0))
("yawning" nil (y ao1 n ih0 ng))
("yawns" nil (y ao1 n z))
("yaworski" nil (y ax w er1 s k iy0))
("yax" nil (y ae1 k s))
("yazov" nil (y ae1 z aa0 v))
("yazzie" nil (y ae1 z iy0))
("ybanez" nil (iy0 b ae1 n ih0 z))
("ybarbo" nil (iy0 b aa1 r b ow0))
("ybarra" nil (iy0 b aa1 r ax))
("ydstie" nil (y ih1 d s t iy0))
("ye" nil (y iy1))
("yea" nil (y ey1))
("yeadon" nil (y eh1 d ax n))
("yeager" nil (y ey1 g er0))
("yeagle" nil (y iy1 g ax l))
("yeagley" nil (y iy1 g l iy0))
("yeah" nil (y ae1))
("yeakel" nil (y iy1 k ax l))
("yeakey" nil (y iy1 k iy0))
("yeakle" nil (y iy1 k ax l))
("yeakley" nil (y iy1 k l iy0))
("yeaman" nil (y iy1 m ax n))
("yeamans" nil (y iy1 m ax n z))
("year" nil (y ih1 r))
("yearago" nil (y ih1 r ax g ow0))
("yearbook" nil (y ih1 r b uh1 k))
("yearbooks" nil (y ih1 r b uh1 k s))
("yearby" nil (y er1 b iy0))
("yearearlier" nil (y ih1 r er1 l y er0))
("yearend" nil (y ih1 r eh1 n d))
("yeargain" nil (y er1 g ax n))
("yeargan" nil (y er1 g ax n))
("yeargin" nil (y er1 jh ih0 n))
("yearick" nil (y iy1 r ih0 k))
("yearling" nil (y er1 l ih0 ng))
("yearlings" nil (y er1 l ih0 ng z))
("yearlong" nil (y ih1 r l ao1 ng))
("yearly" nil (y ih1 r l iy0))
("yearn" nil (y er1 n))
("yearned" nil (y er1 n d))
("yearning" nil (y er1 n ih0 ng))
("yearnings" nil (y er1 n ih0 ng z))
("yearns" nil (y er1 n z))
("yearout" nil (y ih1 r aw0 t))
("years" nil (y ih1 r z))
("yearsley" nil (y er1 s l iy0))
("yearwood" nil (y ih1 r w uh0 d))
("yeary" nil (y ih1 r iy0))
("yeast" nil (y iy1 s t))
("yeasts" nil (y iy1 s t s))
("yeater" nil (y iy1 t er0))
("yeates" nil (y iy1 ey1 t s))
("yeatman" nil (y iy1 t m ax n))
("yeaton" nil (y iy1 t ax n))
("yeats" nil (y ey1 t s))
("yeatts" nil (y iy1 t s))
("yeazel" nil (y iy1 z ax l))
("yeck" nil (y eh1 k))
("yeckley" nil (y eh1 k l iy0))
("yedda" nil (y eh1 d ax))
("yedinak" nil (y eh1 d ih0 n ae0 k))
("yee" nil (y iy1))
("yegor" nil (y ey1 g er0))
("yeh" nil (y eh1))
("yehle" nil (y eh1 hh ax l))
("yehudi" nil (y ey0 hh uw1 d iy0))
("yeiri" nil (y eh1 r iy0))
("yeiser" nil (y iy1 s er0))
("yeisley" nil (y iy1 s l iy0))
("yeldell" nil (y eh1 l d ax l))
("yelena" nil (y eh1 l ih0 n ax))
("yell" nil (y eh1 l))
("yelland" nil (y eh1 l ax n d))
("yelle" nil (y eh1 l))
("yelled" nil (y eh1 l d))
("yellen" nil (y eh1 l ax n))
("yellin" nil (y eh1 l ih0 n))
("yelling" nil (y eh1 l ih0 ng))
("yellow" nil (y eh1 l ow0))
("yellowing" nil (y eh1 l ow0 ih0 ng))
("yellowish" nil (y eh1 l ow0 ih0 sh))
("yellowknife" nil (y eh1 l ow0 n ay1 f))
("yellows" nil (y eh1 l ow0 z))
("yellowstone" nil (y eh1 l ow0 s t ow1 n))
("yells" nil (y eh1 l z))
("yelton" nil (y eh1 l t ax n))
("yeltsin" nil (y eh1 l t s ax n))
("yelverton" nil (y eh1 l v er0 t ax n))
("yelvington" nil (y eh1 l v ih0 ng t ax n))
("yemen" nil (y eh1 m ax n))
("yemeni" nil (y eh1 m ax n iy0))
("yemenidjian" nil (y eh1 m ax n ih1 jh iy0 ax n))
("yen" nil (y eh1 n))
("yenching" nil (y eh1 n ch ih1 ng))
("yenlu" nil (y eh1 n l uw0))
("yeo" nil (y iy1 ow0))
("yeoman" nil (y ow1 m ax n))
("yeomans" nil (y ow1 m ax n z))
("yep" nil (y eh1 p))
("yepez" nil (y eh1 p ih0 z))
("yerby" nil (y er1 b iy0))
("yerdon" nil (y er1 d ax n))
("yerena" nil (y eh1 r ih0 n ax))
("yerevan" nil (y eh1 r ax v ax n))
("yerger" nil (y er1 jh er0))
("yerian" nil (y iy1 r iy0 ax n))
("yerke" nil (y er1 k))
("yerkes" nil (y er1 k iy0 z))
("yerkey" nil (y er1 k iy0))
("yerman" nil (y er1 m ax n))
("yes" nil (y eh1 s))
("yeses" nil (y eh1 s ih0 z))
("yeske" nil (y eh1 s k))
("yesterday" nil (y eh1 s t er0 d ey1))
("yesteryear" nil (y eh1 s t er0 y ih1 r))
("yet" nil (y eh1 t))
("yetman" nil (y eh1 t m ax n))
("yetnikoff" nil (y eh1 t n ih0 k ao0 f))
("yett" nil (y eh1 t))
("yetta" nil (y eh1 t ax))
("yetter" nil (y eh1 t er0))
("yeung" nil (y uw1 ng))
("yeutter" nil (y uw1 t er0))
("yevette" nil (y ih0 v eh1 t))
("yevgeny" nil (y eh1 v g eh1 n iy0))
("yevtushenko" nil (y eh1 v t ax sh eh1 ng k ow0))
("yew" nil (y uw1))
("yglesias" nil (iy0 g l ey1 s iy0 ax s))
("yi" nil (y iy1))
("yiddish" nil (y ih1 d ih0 sh))
("yield" nil (y iy1 l d))
("yielded" nil (y iy1 l d ax d))
("yielding" nil (y iy1 l d ih0 ng))
("yields" nil (y iy1 l d z))
("yilin" nil (y ih1 l ih0 n))
("yim" nil (y ih1 m))
("yin" nil (y ih1 n))
("ying" nil (y ih1 ng))
("yinger" nil (y ih1 ng er0))
("yingling" nil (y ih1 ng g ax l ih0 ng))
("yingst" nil (y ih1 ng g s t))
("yip" nil (y ih1 p))
("yitzhak" nil (y ih1 t s aa0 k))
("ylvisaker" nil (y ih1 l v ih0 s aa1 k er0))
("ynez" nil (y n eh1 z))
("yniguez" nil (iy0 n iy1 g eh0 z))
("ynjiun" nil (y ih1 n jh ih0 n))
("yo" nil (y ow1))
("yoak" nil (y ow1 k))
("yoakam" nil (y ow1 k ax m))
("yoakum" nil (y ow1 k ax m))
("yoast" nil (y ow1 s t))
("yocam" nil (y ow1 k ax m))
("yocham" nil (y aa1 ch ax m))
("yochim" nil (y aa1 ch ih0 m))
("yochum" nil (y aa1 ch ax m))
("yockey" nil (y aa1 k iy0))
("yocom" nil (y ow1 k ax m))
("yocum" nil (y ow1 k ax m))
("yodel" nil (y ow1 d ax l))
("yodeling" nil (y ow1 d ax l ih0 ng))
("yoder" nil (y ow1 d er0))
("yodice" nil (y ow1 d ih0 s))
("yoe" nil (y ow1))
("yoes" nil (y ow1 z))
("yoffie" nil (y ao1 f iy0))
("yoga" nil (y ow1 g ax))
("yogi" nil (y ow1 g iy0))
("yogiisms" nil (y ow0 g iy1 s ax m z))
("yogurt" nil (y ow1 g er0 t))
("yoh" nil (y ow1))
("yohe" nil (y ow1 hh ey1))
("yohei" nil (y ow1 hh ey1))
("yohn" nil (y aa1 n))
("yoho" nil (y ow1 hh ow0))
("yoichi" nil (y ow0 iy1 ch iy0))
("yokado" nil (y ax k aa1 d ow0))
("yoke" nil (y ow1 k))
("yoked" nil (y ow1 k t))
("yokich" nil (y ow1 k ih0 ch))
("yokley" nil (y aa1 k l iy0))
("yoko" nil (y ow1 k ow0))
("yokohama" nil (y ow1 k ax hh aa1 m ax))
("yokokohji" nil (y ow1 k ow0 k ow1 jh iy0))
("yokota" nil (y ow0 k ow1 t ax))
("yokoyama" nil (y ow1 k ow0 y aa1 m ax))
("yokum" nil (y ow1 k ax m))
("yola" nil (y ow1 l ax))
("yolanda" nil (y ow0 l aa1 n d ax))
("yolande" nil (y ow1 l ax n d))
("yolk" nil (y ow1 k))
("yolks" nil (y ow1 k s))
("yom" nil (y aa1 m))
("yomiuri" nil (y ow1 m iy1 er0 iy0))
("yon" nil (y aa1 n))
("yona" nil (y ow1 n ax))
("yonan" nil (y ow1 n ax n))
("yonce" nil (y aa1 n s))
("yonder" nil (y aa1 n d er0))
("yoneyama" nil (y ow1 n iy0 aa1 m ax))
("yong" nil (y ao1 ng))
("yongbyon" nil (y ao1 ng b y ao1 n))
("yongue" nil (y aa1 ng g))
("yonhap" nil (y aa1 n hh ae1 p))
("yonke" nil (y aa1 ng k))
("yonker" nil (y aa1 ng k er0))
("yonkers" nil (y aa1 ng k er0 z))
("yonsei" nil (y ah1 n s ey1))
("yonts" nil (y aa1 n t s))
("yontz" nil (y aa1 n t s))
("yoo" nil (y uw1))
("yoon" nil (y uw1 n))
("yoor" nil (y uw1 r))
("yoos" nil (y uw1 z))
("yopp" nil (y aa1 p))
("yoram" nil (y ao1 r ax m))
("yorba" nil (y ao1 r b ax))
("yordy" nil (y ao1 r d iy0))
("yore" nil (y ao1 r))
("yorich" nil (y ao1 r ih0 ch))
("yorio" nil (y ao1 r y ow0))
("york" nil (y ao1 r k))
("yorkbased" nil (y ao1 r k b ey1 s t))
("yorke" nil (y ao1 r k))
("yorker" nil (y ao1 r k er0))
("yorkers" nil (y ao1 r k er0 z))
("yorks" nil (y ao1 r k s))
("yorkshire" nil (y ao1 r k sh er0))
("yorktown" nil (y ao1 r k t aw1 n))
("yorrick" nil (y ao1 r ih0 k))
("yosef" nil (y ow1 s ax f))
("yosemite" nil (y ow0 s eh1 m ih0 t iy0))
("yoshi" nil (y ow1 sh iy0))
("yoshiaki" nil (y ow1 sh iy0 aa1 k iy0))
("yoshida" nil (y ow0 sh iy1 d ax))
("yoshihara" nil (y ow1 sh ih0 hh aa1 r ax))
("yoshihashi" nil (y ow1 sh ih0 hh aa1 sh iy0))
("yoshihiro" nil (y ow1 sh ih0 hh iy1 r ow1))
("yoshihisa" nil (y ow1 sh ih0 hh iy1 s ax))
("yoshikawa" nil (y ow0 sh iy0 k aa1 w ax))
("yoshikazu" nil (y ow1 sh ih0 k aa1 z uw0))
("yoshimoto" nil (y ow0 sh iy0 m ow1 t ow0))
("yoshimura" nil (y ow1 sh ih0 m uw1 r ax))
("yoshino" nil (y ow0 sh iy1 n ow0))
("yoshio" nil (y ow0 sh iy1 ow0))
("yoshioka" nil (y ow1 sh iy0 ow1 k ax))
("yoshiro" nil (y ow0 sh ih1 r ow0))
("yoss" nil (y ao1 s))
("yossi" nil (y ow1 s iy0))
("yost" nil (y ow1 s t))
("yother" nil (y ah1 dh er0))
("yott" nil (y aa1 t))
("you" nil (y uw1))
("youell" nil (y aa1 uw0 l))
("youghal" nil (y ow1 ax l))
("youker" nil (y aw1 k er0))
("youman" nil (y uw1 m ax n))
("youmans" nil (y uw1 m ax n z))
("youn" nil (y ah1 n))
("younan" nil (y ah1 n ax n))
("younce" nil (y aw1 n s))
("younes" nil (y ah1 n z))
("young" nil (y ah1 ng))
("youngberg" nil (y ah1 ng b er0 g))
("youngblood" nil (y ah1 ng b l ah1 d))
("youngblut" nil (y ah1 ng b l ax t))
("youngdahl" nil (y ah1 ng d aa1 l))
("younge" nil (y ah1 n jh))
("younger" nil (y ah1 ng g er0))
("youngerman" nil (y ah1 ng er0 m ax n))
("youngers" nil (y ah1 ng g er0 z))
("youngest" nil (y ah1 ng g ax s t))
("younglove" nil (y ah1 ng l ah1 v))
("youngman" nil (y ah1 ng m ae1 n))
("youngquist" nil (y ah1 ng k w ih1 s t))
("youngren" nil (y ah1 n g r eh0 n))
("youngs" nil (y ah1 ng z))
("youngster" nil (y ah1 ng s t er0))
("youngsters" nil (y ah1 ng s t er0 z))
("youngstown" nil (y ah1 ng z t aw1 n))
("youngstrom" nil (y ah1 ng s t r ax m))
("younis" nil (y ah1 n ih0 s))
("younker" nil (y ah1 ng k er0))
("younkers" nil (y ah1 ng k er0 z))
("younkin" nil (y ah1 ng k ih0 n))
("younkins" nil (y ah1 ng k ih0 n z))
("yount" nil (y aw1 n t))
("younts" nil (y aw1 n t s))
("your" nil (y ao1 r))
("youree" nil (y ao0 r iy1))
("yours" nil (y uh1 r z))
("yourself" nil (y er0 s eh1 l f))
("yourselfers" nil (y ao1 r s eh1 l f er0 z))
("yourselves" nil (y uh0 r s eh1 l v z))
("youse" nil (y aw1 s))
("yousef" nil (y uw1 s eh0 f))
("yousif" nil (y aw1 s ih0 f))
("youssef" nil (y uw1 s eh0 f))
("youth" nil (y uw1 th))
("youthful" nil (y uw1 th f ax l))
("youths" nil (y uw1 dh z))
("youtsey" nil (y aw1 t s iy0))
("youtz" nil (y aw1 t s))
("youville" nil (y uw1 v ih1 l))
("yow" nil (y aw1))
("yowell" nil (y aa1 w eh0 l))
("ypsilanti" nil (ih1 p s ax l ae1 n t iy0))
("yquem" nil (iy1 k w ax m))
("ysleta" nil (iy0 s l ey1 t ax))
("yttrium" nil (ih1 t r iy0 ax m))
("yu" nil (y uw1))
("yuan" nil (y uw0 aa1 n))
("yuba" nil (y uw1 b ax))
("yucaipa" nil (y uw1 k ay1 p ax))
("yucatan" nil (y uw1 k ax t ae1 n))
("yucca" nil (y ah1 k ax))
("yue" nil (y uw1))
("yuen" nil (y w eh1 n))
("yugo" nil (y uw1 g ow0))
("yugos" nil (y uw1 g ow0 s))
("yugoslav" nil (y uw1 g ow0 s l aa1 v))
("yugoslavia" nil (y uw1 g ow0 s l aa1 v iy0 ax))
("yugoslavian" nil (y uw1 g ow0 s l aa1 v iy0 ax n))
("yugoslavs" nil (y uw1 g ow0 s l ae1 v z))
("yuhas" nil (y uw1 hh ax z))
("yuhasz" nil (y ah1 hh ax sh))
("yuill" nil (y uw1 l))
("yuille" nil (y uw1 l))
("yuk" nil (y ah1 k))
("yukio" nil (y uw0 k iy1 ow0))
("yuko" nil (y uw1 k ow0))
("yukon" nil (y uw1 k aa1 n))
("yuks" nil (y ah1 k s))
("yul" nil (y ah1 l))
("yule" nil (y uw1 l))
("yuletide" nil (y uw1 l t ay1 d))
("yuli" nil (y uw1 l iy0))
("yum" nil (y ah1 m))
("yuma" nil (y uw1 m ax))
("yummies" nil (y ah1 m iy0 z))
("yummy" nil (y ah1 m iy0))
("yun" nil (y ah1 n))
("yunde" nil (y uw1 n d ey1))
("yundt" nil (y ah1 n d t))
("yung" nil (y ah1 ng))
("yunich" nil (y uw1 n ih0 ch))
("yunis" nil (y uw1 n ih0 s))
("yunk" nil (y ah1 ng k))
("yunker" nil (y ah1 ng k er0))
("yup" nil (y ah1 p))
("yuppie" nil (y ah1 p iy0))
("yuppies" nil (y ah1 p iy0 z))
("yuppified" nil (y ah1 p ih0 f ay1 d))
("yurachek" nil (y er0 ae1 ch eh0 k))
("yurchak" nil (y er1 ch ax k))
("yurchenko" nil (y er0 ch eh1 ng k ow0))
("yurek" nil (y uh1 r eh0 k))
("yuri" nil (y uh1 r iy0))
("yurick" nil (y uh1 r ih0 k))
("yurko" nil (y er1 k ow0))
("yurkovich" nil (y er1 k ax v ih0 ch))
("yurt" nil (y uh1 r t))
("yury" nil (y uh1 r iy0))
("yusef" nil (y uw1 s eh0 f))
("yusen" nil (y uw1 s ax n))
("yusko" nil (y ah1 s k ow0))
("yust" nil (y ah1 s t))
("yutaka" nil (y uw0 t aa1 k ax))
("yutzy" nil (y ah1 t z iy0))
("yuval" nil (y uw1 v ax l))
("yuzong" nil (y uw1 z ao1 ng))
("yves" nil (iy1 v))
("yvette" nil (ih0 v eh1 t))
("yvonne" nil (ih0 v aa1 n))
("yzaguirre" nil (iy0 z ax g w ay1 er0))
("z" nil (z iy1))
("zabala" nil (z aa0 b aa1 l ax))
("zabawa" nil (z aa0 b aa1 w ax))
("zabel" nil (z ey1 b ax l))
("zabielski" nil (z ax b iy1 l s k iy0))
("zabinski" nil (z ax b ih1 n s k iy0))
("zablocki" nil (z ax b l ow1 t s k iy0))
("zaborowski" nil (z ax b er0 ao1 f s k iy0))
("zabowski" nil (z ax b aw1 s k iy0))
("zabriskie" nil (z ae1 b r ih0 s k iy0))
("zacarias" nil (z aa0 k aa0 r iy1 ax z))
("zaccagnini" nil (z aa0 k ax g n iy1 n iy0))
("zaccagnino" nil (z aa0 k ax g n iy1 n ow0))
("zaccardi" nil (z aa0 k aa1 r d iy0))
("zaccaria" nil (z aa0 k aa1 r iy0 ax))
("zaccaro" nil (z aa0 k aa1 r ow0))
("zaccheus" nil (z ae1 k oy0 z))
("zaccone" nil (z aa0 k ow1 n iy0))
("zacek" nil (z aa1 ch eh1 k))
("zach" nil (z ae1 k))
("zachar" nil (z ax k aa1 r))
("zachariah" nil (z ae1 k er0 ay1 ax))
("zacharias" nil (z ae1 k er0 ay1 ax s))
("zachary" nil (z ae1 k er0 iy0))
("zacher" nil (z ae1 k er0))
("zachery" nil (z ae1 k er0 iy0))
("zachman" nil (z ae1 k m ax n))
("zachmann" nil (z aa1 k m ax n))
("zachow" nil (z aa1 k ow0))
("zachry" nil (z aa1 k r iy0))
("zack" nil (z ae1 k))
("zackery" nil (z ae1 k er0 iy0))
("zacks" nil (z ae1 k s))
("zada" nil (z aa1 d ax))
("zadeh" nil (z aa1 d eh1))
("zadoc" nil (z aa1 d ax k))
("zadok" nil (z aa1 d ax k))
("zadrozny" nil (z ax d r ow1 z n iy0))
("zaentz" nil (z ae1 n t s))
("zaffino" nil (z aa0 f iy1 n ow0))
("zaffuto" nil (z ax f uw1 t ow0))
("zaftig" nil (z ae1 f t ih0 g))
("zag" nil (z ae1 g))
("zagar" nil (z aa0 g aa1 r))
("zagat" nil (z ae1 g ax t))
("zagel" nil (z ey1 g ax l))
("zager" nil (z ey1 g er0))
("zagha" nil (z aa1 g ax))
("zagladin" nil (z ae1 g l ax d ih0 n))
("zagorski" nil (z ax g ao1 r s k iy0))
("zagreb" nil (z aa1 g r eh1 b))
("zagury" nil (z ae1 g y er0 iy0))
("zahir" nil (z ax hh ih1 r))
("zahler" nil (z aa1 l er0))
("zahm" nil (z ae1 m))
("zahn" nil (z ae1 n))
("zahner" nil (z aa1 n er0))
("zahniser" nil (z aa1 n ay0 z er0))
("zahnow" nil (z aa1 n ow0))
("zahradnik" nil (z er0 ae1 d n ih0 k))
("zaibatsu" nil (z ey1 b ae1 t s uw1))
("zaid" nil (z ey1 d))
("zaino" nil (z ey1 n ow0))
("zaire" nil (z ay0 ih1 r))
("zairean" nil (z ay0 ih1 r ax n))
("zaireans" nil (z ay0 ih1 r ax n z))
("zairian" nil (z ay0 ih1 r iy0 ax n))
("zairians" nil (z ay0 ih1 r iy0 ax n z))
("zaiser" nil (z ay1 s er0))
("zaitech" nil (z ey1 t eh1 k))
("zaitsev" nil (z ey1 t s eh0 v))
("zaitz" nil (z ey1 t s))
("zajac" nil (z ay1 ax k))
("zajdel" nil (z ae1 jh d ax l))
("zajic" nil (z ae1 jh ih0 k))
("zajicek" nil (z ay1 ih0 ch eh0 k))
("zajkowski" nil (z ay0 k ao1 f s k iy0))
("zak" nil (z ae1 k))
("zakarian" nil (z ax k eh1 r iy0 ax n))
("zakharov" nil (z ae1 k er0 aa0 v))
("zaki" nil (z aa1 k iy0))
("zakrajsek" nil (z ax k r ay1 s eh0 k))
("zakrzewski" nil (z aa0 k er0 z uw1 s k iy0))
("zalar" nil (z aa0 l aa1 r))
("zalay" nil (z ax l ey1))
("zaldivar" nil (z aa0 l d iy0 v aa1 r))
("zale" nil (z ey1 l))
("zalenski" nil (z ax l eh1 n s k iy0))
("zaleski" nil (z ax l eh1 s k iy0))
("zalesky" nil (z ax l eh1 s k iy0))
("zalewski" nil (z ax l eh1 f s k iy0))
("zaley" nil (z ey1 l iy0))
("zalm" nil (z aa1 m))
("zaloudek" nil (z ae1 l aw0 d ih0 k))
("zalygin" nil (z ae1 l ih0 g ih0 n))
("zaman" nil (z ey1 m ax n))
("zamarello" nil (z ae1 m er0 eh1 l ow0))
("zamarripa" nil (z aa0 m aa0 r iy1 p ax))
("zamarron" nil (z aa0 m aa0 r ao1 n))
("zambelli" nil (z aa0 m b eh1 l iy0))
("zambia" nil (z ae1 m b iy0 ax))
("zambian" nil (z ae1 m b iy0 ax n))
("zambito" nil (z aa0 m b iy1 t ow0))
("zambo" nil (z ae1 m b ow0))
("zamboanga" nil (z ae0 m b ow1 ng g ax))
("zambrana" nil (z aa0 m b r ae1 n ax))
("zambrano" nil (z aa0 m b r aa1 n ow0))
("zamfir" nil (z ae0 m f ih1 r))
("zammit" nil (z ae1 m ih0 t))
("zamora" nil (z ax m ao1 r ax))
("zamorano" nil (z aa0 m ao0 r aa1 n ow0))
("zampino" nil (z aa0 m p iy1 n ow0))
("zamudio" nil (z aa0 m uw1 d iy0 ow0))
("zamzow" nil (z ae1 m z ow0))
("zanca" nil (z ae1 ng k ax))
("zander" nil (z ae1 n d er0))
("zanders" nil (z ae1 n d er0 z))
("zandi" nil (z aa1 n d iy0))
("zandra" nil (z ae1 n d r ax))
("zandstra" nil (z ae1 n d s t r ax))
("zandy" nil (z ae1 n d iy0))
("zane" nil (z ey1 n))
("zanella" nil (z ax n eh1 l ax))
("zaneta" nil (z aa0 n eh1 t ax))
("zang" nil (z ae1 ng))
("zangara" nil (z aa0 ng g aa1 r ax))
("zangari" nil (z aa0 ng g aa1 r iy0))
("zanger" nil (z ae1 ng er0))
("zanghi" nil (z aa1 n g iy0))
("zani" nil (z aa1 n iy0))
("zaniest" nil (z ey1 n iy0 ax s t))
("zaniewski" nil (z ae0 n iy0 eh1 f s k iy0))
("zank" nil (z ae1 ng k))
("zanni" nil (z ae1 n iy0))
("zanoni" nil (z aa0 n ow1 n iy0))
("zanoyan" nil (z ax n oy1 ax n))
("zant" nil (z ae1 n t))
("zantac" nil (z ae1 n t ae0 k))
("zanu" nil (z aa0 n uw1))
("zanuck" nil (z ae1 n ax k))
("zanussi" nil (z ax n uw1 s iy0))
("zany" nil (z ey1 n iy0))
("zanzibar" nil (z ae1 n z ax b aa1 r))
("zap" nil (z ae1 p))
("zapalac" nil (z ax p aa1 l ax k))
("zapata" nil (z aa0 p aa1 t ax))
("zapatista" nil (z aa1 p ax t iy1 s t ax))
("zapatistas" nil (z aa1 p ax t iy1 s t ax z))
("zapf" nil (z ae1 p f))
("zapien" nil (z ae1 p iy0 n))
("zapmail" nil (z ae1 p m ey1 l))
("zapp" nil (z ae1 p))
("zappa" nil (z ae1 p ax))
("zappala" nil (z aa0 p aa1 l ax))
("zapped" nil (z ae1 p t))
("zappia" nil (z ae1 p iy0 ax))
("zapping" nil (z ae1 p ih0 ng))
("zappone" nil (z aa0 p ow1 n iy0))
("zappulla" nil (z aa0 p uw1 l ax))
("zaps" nil (z ae1 p s))
("zara" nil (z ae1 r ax))
("zaragoza" nil (z aa0 r aa0 g ow1 z ax))
("zarah" nil (z ae1 r ax))
("zarate" nil (z aa1 r ey0 t))
("zarb" nil (z aa1 r b))
("zarcone" nil (z aa0 r k ow1 n iy0))
("zared" nil (z eh1 r d))
("zarek" nil (z aa1 r eh0 k))
("zaremba" nil (z aa0 r eh1 m b ax))
("zaremski" nil (z er0 eh1 s k iy0))
("zaretsky" nil (z er0 eh1 t s k iy0))
("zarett" nil (z eh1 r ax t))
("zaria" nil (z aa1 r iy0 ax))
("zaring" nil (z eh1 r ih0 ng))
("zarlengo" nil (z aa0 r l ey1 ng g ow0))
("zarling" nil (z aa1 r l ih0 ng))
("zaro" nil (z ae1 r ow0))
("zarr" nil (z ae1 r))
("zarrella" nil (z aa1 r eh1 l ax))
("zarro" nil (z aa1 r ow0))
("zarroli" nil (z er0 ow1 l iy0))
("zartman" nil (z aa1 r t m ax n))
("zaruba" nil (z er0 uw1 b ax))
("zary" nil (z aa1 r iy0))
("zarzecki" nil (z aa0 r z eh1 k iy0))
("zarzycki" nil (z er0 z ih1 t s k iy0))
("zasada" nil (z aa0 s aa1 d ax))
("zaske" nil (z ey1 s k))
("zaslavskaya" nil (z ae1 s l ax v s k ay1 ax))
("zaslow" nil (z aa1 s l ow0))
("zastrow" nil (z ae1 s t r ow0))
("zaucha" nil (z aw1 hh ax))
("zaugg" nil (z ao1 g))
("zaun" nil (z ao1 n))
("zauner" nil (z ao1 n er0))
("zavada" nil (z aa0 v aa1 d ax))
("zavadil" nil (z aa0 v aa0 d iy1 l))
("zavala" nil (z aa0 v aa1 l ax))
("zavodnik" nil (z ax v aa1 d n ih0 k))
("zavodny" nil (z ax v aa1 d n iy0))
("zawacki" nil (z aa0 v aa1 t s k iy0))
("zawada" nil (z aa0 w aa1 d ax))
("zawadzki" nil (z aa0 v aa1 jh k iy0))
("zawislak" nil (z aa0 v ih1 s l ax k))
("zawistowski" nil (z aa0 v ih0 s t ao1 f s k iy0))
("zayac" nil (z ay1 ax k))
("zayas" nil (z ey1 aa0 z))
("zayed" nil (z ey1 ax d))
("zayre" nil (z ey1 r))
("zazueta" nil (z aa0 z w eh1 t ax))
("zbigniew" nil (z b ih1 g n uw0))
("zbikowski" nil (z b ih0 k aw1 s k iy0))
("zbinden" nil (z b ay1 n d ax n))
("zdrojewski" nil (z d r oy1 uw1 s k iy0))
("ze" nil (z iy1))
("zea" nil (z iy1))
("zeagler" nil (z iy1 g l er0))
("zeal" nil (z iy1 l))
("zealand" nil (z iy1 l ax n d))
("zealanders" nil (z iy1 l ax n d er0 z))
("zealot" nil (z eh1 l ax t))
("zealotry" nil (z eh1 l ax t r iy0))
("zealots" nil (z eh1 l ax t s))
("zealous" nil (z eh1 l ax s))
("zealously" nil (z iy1 l ax s l iy0))
("zebell" nil (z iy1 b eh0 l))
("zebley" nil (z eh1 b l iy0))
("zebra" nil (z iy1 b r ax))
("zebras" nil (z iy1 b r ax z))
("zebrowski" nil (z ax b r ao1 f s k iy0))
("zebulon" nil (z iy1 b ax l ax n))
("zecca" nil (z eh1 k ax))
("zech" nil (z eh1 k))
("zecher" nil (z eh1 k er0))
("zechman" nil (z eh1 k m ax n))
("zeck" nil (z eh1 k))
("zeckendorf" nil (z eh1 k ax n d ao1 r f))
("zed" nil (z eh1 d))
("zedekiah" nil (z eh1 d ax k ay1 ax))
("zedillo" nil (z ey1 d iy1 ow0))
("zee" nil (z iy1))
("zeeb" nil (z iy1 b))
("zeebrugge" nil (z iy1 b r uw1 g iy0))
("zeek" nil (z iy1 k))
("zeeland" nil (z iy1 l ax n d))
("zeeman" nil (z iy1 m ax n))
("zeff" nil (z eh1 f))
("zeffirelli" nil (z eh1 f ih0 r eh1 l iy0))
("zegeer" nil (z ih0 g ih1 r))
("zeger" nil (z iy1 g er0))
("zegers" nil (z iy1 g er0 z))
("zeglin" nil (z eh1 g l ih0 n))
("zeh" nil (z eh1))
("zehnder" nil (z eh1 n d er0))
("zehner" nil (z eh1 n er0))
("zehntel" nil (z eh1 n t eh1 l))
("zehr" nil (z eh1 r))
("zehren" nil (z eh1 r ax n))
("zehring" nil (z eh1 r ih0 ng))
("zeibel" nil (z ay1 b ax l))
("zeichner" nil (z ay1 k n er0))
("zeiders" nil (z ay1 d er0 z))
("zeidler" nil (z ay1 d ax l er0))
("zeidman" nil (z ay1 d m ax n))
("zeien" nil (z ay1 ax n))
("zeiger" nil (z ay1 g er0))
("zeigler" nil (z ay1 g ax l er0))
("zeiler" nil (z ay1 l er0))
("zeilinger" nil (z ay1 l ih0 ng er0))
("zeimet" nil (z ay1 m ih0 t))
("zeinab" nil (z ay1 n ae0 b))
("zeiner" nil (z ay1 n er0))
("zeis" nil (z iy1 z))
("zeiser" nil (z ay1 s er0))
("zeisler" nil (z ay1 s ax l er0))
("zeiss" nil (z ay1 s))
("zeit" nil (z ay1 t))
("zeiter" nil (z ay1 t er0))
("zeitgeist" nil (t s ay1 t g ay1 s t))
("zeitler" nil (z ay1 t ax l er0))
("zeitlin" nil (z ay1 t l ih0 n))
("zeitung" nil (z ay1 t ax ng))
("zeitz" nil (z iy1 t s))
("zekauskas" nil (z ax k aw1 s k ax s))
("zeke" nil (z iy1 k))
("zel" nil (z eh1 l))
("zelasko" nil (z ih0 l aa1 s k ow0))
("zelaya" nil (z ey0 l ay1 ax))
("zelazny" nil (z ih0 l aa1 z n iy0))
("zelda" nil (z eh1 l d ax))
("zeldin" nil (z eh1 l d ih0 n))
("zele" nil (z iy1 l))
("zelek" nil (z eh1 l ih0 k))
("zelenak" nil (z eh1 l ih0 n ax k))
("zelenka" nil (z ih0 l eh1 ng k ax))
("zeleny" nil (z ih0 l iy1 n iy0))
("zelesnik" nil (z ax l eh1 s n ih0 k))
("zeleznik" nil (z ih0 l eh1 z n ih0 k))
("zelia" nil (z eh1 l iy0 ax))
("zelie" nil (z eh1 l iy0))
("zelina" nil (z eh0 l iy1 n ax))
("zelinka" nil (z ih0 l ih1 ng k ax))
("zelinski" nil (z ih0 l ih1 n s k iy0))
("zelinsky" nil (z ih0 l ih1 n s k iy0))
("zelko" nil (z eh1 l k ow0))
("zell" nil (z eh1 l))
("zellars" nil (z eh1 l er0 z))
("zelle" nil (z eh1 l))
("zeller" nil (z eh1 l er0))
("zellerbach" nil (z eh1 l er0 b aa1 k))
("zellers" nil (z eh1 l er0 z))
("zellman" nil (z eh1 l m ax n))
("zellmer" nil (z eh1 l m er0))
("zellner" nil (z eh1 l n er0))
("zelma" nil (z eh1 l m ax))
("zelman" nil (z eh1 l m ax n))
("zelnick" nil (z eh1 l n ih0 k))
("zelos" nil (z iy1 l ow0 z))
("zelotes" nil (z eh1 l ax t s))
("zeltner" nil (z eh1 l t n er0))
("zeltzer" nil (z eh1 l t s er0))
("zelvin" nil (z eh1 l v ih0 n))
("zemaitis" nil (z eh1 m ay1 t ax s))
("zeman" nil (z iy1 m ax n))
("zemanek" nil (z eh1 m ax n ih0 k))
("zemba" nil (z eh1 m b ax))
("zembower" nil (z eh1 m b aw1 er0))
("zembriski" nil (z eh0 m b r ih1 s k iy0))
("zemel" nil (z eh1 m ax l))
("zemin" nil (z iy1 m ih0 n))
("zemke" nil (z eh1 m k))
("zempel" nil (z eh1 m p ax l))
("zen" nil (z eh1 n))
("zena" nil (z iy1 n ax))
("zenaida" nil (z ih0 n ey1 d ax))
("zenas" nil (z iy1 n ax z))
("zenchu" nil (z eh1 n ch uw1))
("zenda" nil (z eh1 n d ax))
("zendejas" nil (z iy1 n d ih0 jh ax z))
("zender" nil (z eh1 n d er0))
("zeneca" nil (z eh1 n ax k ax))
("zeng" nil (z eh1 ng))
("zenger" nil (z iy1 n jh er0))
("zenia" nil (z ey1 n iy0 ax))
("zenina" nil (z eh0 n iy1 n ax))
("zenith" nil (z iy1 n ax th))
("zenk" nil (z eh1 ng k))
("zenker" nil (z eh1 ng k er0))
("zenna" nil (z eh1 n ax))
("zenner" nil (z eh1 n er0))
("zeno" nil (z ey1 n ow0))
("zenobia" nil (z ax n ow1 b iy0 ax))
("zenon" nil (z iy1 n ax n))
("zenor" nil (z eh1 n er0))
("zens" nil (z eh1 n z))
("zent" nil (z eh1 n t))
("zentec" nil (z eh1 n t eh1 k))
("zentner" nil (z eh1 n t n er0))
("zentralsparkasse" nil (z eh1 n t r ax l s p aa1 r k aa1 z ih0))
("zentz" nil (z eh1 n t s))
("zenz" nil (z eh1 n z))
("zeoli" nil (z iy0 aa1 l iy0))
("zeos" nil (z iy1 ow1 s))
("zepeda" nil (z ey0 p ey1 d ax))
("zepp" nil (z eh1 p))
("zeppelin" nil (z eh1 p ih0 l ih0 n))
("zeppenfeld" nil (z eh1 p ax f eh1 l d))
("zera" nil (z eh1 r ax))
("zerbe" nil (z ih1 r b))
("zerby" nil (z ih1 r b iy0))
("zerilli" nil (z er0 iy1 l iy0))
("zeringue" nil (z eh1 r ih0 ng))
("zerkle" nil (z ih1 r k ax l))
("zerlina" nil (z er0 l iy1 n ax))
("zerlinda" nil (z er0 l iy1 n d ax))
("zermeno" nil (z er0 m eh1 n ow0))
("zern" nil (z ih1 r n))
("zero" nil (z ih1 r ow0))
("zeroed" nil (z ih1 r ow0 d))
("zeroes" nil (z ih1 r ow0 z))
("zeroing" nil (z iy1 r ow0 ih0 ng))
("zeros" nil (z ih1 r ow0 z))
("zerr" nil (z ih1 r))
("zertuche" nil (z ih1 r t ax k))
("zervas" nil (z ih1 r v ax z))
("zervos" nil (z ih1 r v ow0 z))
("zest" nil (z eh1 s t))
("zestful" nil (z eh1 s t f ax l))
("zeta" nil (z ey1 t ax))
("zettel" nil (z eh1 t ax l))
("zettle" nil (z eh1 t ax l))
("zettlemoyer" nil (z eh1 t l ih0 m oy0 er0))
("zettler" nil (z eh1 t l er0))
("zeus" nil (z uw1 s))
("zev" nil (z eh1 v))
("zeva" nil (z ey1 v ax))
("zhan" nil (z aa1 n))
("zhang" nil (zh ae1 ng))
("zhao" nil (zh aw1))
("zhejiang" nil (zh iy1 jh aa1 ng))
("zheng" nil (z eh1 ng))
("zhirinovsky" nil (zh ih1 r ax n aa1 f s k iy0))
("zhivago" nil (zh ih0 v aa1 g ow0))
("zhivkov" nil (zh ih1 v k aa0 v))
("zhou" nil (z uw1))
("zhu" nil (z uw1))
("zhuhai" nil (z uw1 hh ay1))
("zia" nil (z iy1 ax))
("zibell" nil (z ay1 b eh0 l))
("ziccardi" nil (z iy0 k aa1 r d iy0))
("zich" nil (z ih1 ch))
("zick" nil (z ih1 k))
("zickefoose" nil (z ih1 k ih0 f uw0 s))
("zico" nil (z iy1 k ow0))
("zidek" nil (z ih1 d ih0 k))
("zieba" nil (z iy1 b ax))
("ziebarth" nil (z iy1 b aa0 r th))
("ziebell" nil (z iy1 b eh0 l))
("zieg" nil (z iy1 g))
("ziegenbein" nil (z iy1 g ih0 n b ay0 n))
("ziegenfuss" nil (z iy1 g ih0 n f ax s))
("ziegenhorn" nil (z iy1 g ih0 n hh er0 n))
("zieger" nil (z iy1 g er0))
("ziegler" nil (z iy1 g l er0))
("ziehl" nil (z iy1 l))
("ziehm" nil (z iy1 m))
("ziel" nil (z iy1 l))
("zielinski" nil (z iy0 l ih1 n s k iy0))
("zielke" nil (z iy1 l k))
("zieman" nil (z iy1 m ax n))
("ziemann" nil (z iy1 m ax n))
("ziemba" nil (z iy1 m b ax))
("ziemer" nil (z iy1 m er0))
("ziemian" nil (z iy1 m iy0 ax n))
("ziemke" nil (z iy1 m k))
("zientek" nil (z iy1 n t ih0 k))
("zier" nil (z ih1 r))
("zierke" nil (z iy1 r k))
("ziesemer" nil (z iy1 s iy0 m er0))
("zieske" nil (z iy1 s k))
("ziesmer" nil (z iy1 s ax m er0))
("zietlow" nil (z iy1 t l ow0))
("zietz" nil (z iy1 t s))
("ziff" nil (z ih1 f))
("zig" nil (z ih1 g))
("ziggurat" nil (z ih1 g er0 ae1 t))
("ziggurats" nil (z ih1 g er0 ae1 t s))
("ziglar" nil (z ih1 g l er0))
("zigler" nil (z ay1 g ax l er0))
("zigman" nil (z ih1 g m ax n))
("zigzag" nil (z ih1 g z ae0 g))
("zigzagged" nil (z ih1 g z ae0 g d))
("zigzagging" nil (z ih1 g z ae0 g ih0 ng))
("zigzags" nil (z ih1 g z ae0 g z))
("zika" nil (z ih1 k ax))
("zike" nil (z ay1 k))
("zil" nil (z ih1 l))
("zilber" nil (z ih1 l b er0))
("zilch" nil (z ih1 l ch))
("zilka" nil (z ih1 l k ax))
("zilkha" nil (z ih1 l k hh ax))
("zill" nil (z ih1 l))
("zilla" nil (z ih1 l ax))
("zillah" nil (z ih1 l ax))
("ziller" nil (z ih1 l er0))
("zillion" nil (z ih1 l y ax n))
("zillions" nil (z ih1 l y ax n z))
("zillmer" nil (z ih1 l m er0))
("zilvia" nil (z ih1 l v iy0 ax))
("zim" nil (z ih1 m))
("zima" nil (z iy1 m ax))
("zimbabwe" nil (z ih0 m b aa1 b w ey0))
("zimbabwean" nil (z ih0 m b aa1 b w iy0 ax n))
("zimbalist" nil (z ih1 m b ax l ih0 s t))
("zimbelman" nil (z ih1 m b ax l m ax n))
("zimdars" nil (z ih1 m d er0 z))
("zimerman" nil (z ay1 m er0 m ax n))
("zimmer" nil (z ih1 m er0))
("zimmerer" nil (z ih1 m er0 er0))
("zimmerle" nil (z ih1 m er0 ax l))
("zimmerly" nil (z ih1 m er0 l iy0))
("zimmerman" nil (z ih1 m er0 m ax n))
("zimmermann" nil (z ih1 m er0 m ax n))
("zimmers" nil (z ih1 m er0 z))
("zimny" nil (z ih1 m n iy0))
("zimpfer" nil (z ih1 m p f er0))
("zinc" nil (z ih1 ng k))
("zinck" nil (z ih1 ng k))
("zinda" nil (z ih1 n d ax))
("zinder" nil (z ih1 n d er0))
("zinfandel" nil (z ih1 n f ax n d eh1 l))
("zing" nil (z ih1 ng))
("zingale" nil (z ih0 ng g aa1 l iy0))
("zingaro" nil (z ih0 ng g aa1 r ow0))
("zinger" nil (z ih1 ng er0))
("zingg" nil (z ih1 ng g))
("zingler" nil (z ih1 ng g ax l er0))
("zinia" nil (z iy1 n iy0 ax))
("zink" nil (z ih1 ng k))
("zinke" nil (z ih1 ng k))
("zinman" nil (z ih1 n m ax n))
("zinn" nil (z ih1 n))
("zinner" nil (z ih1 n er0))
("zinni" nil (z ih1 n iy0))
("zinnia" nil (z ih1 n iy0 ax))
("zinnias" nil (z ih1 n iy0 ax z))
("zinno" nil (z ih1 n ow0))
("zins" nil (z ih1 n z))
("zinser" nil (z ih1 n s er0))
("zinsmeister" nil (z ih1 n s m ay0 s t er0))
("zinter" nil (z ih1 n t er0))
("ziobro" nil (z iy0 ow1 b r ow0))
("ziolkowski" nil (z iy0 ow0 l k ao1 f s k iy0))
("ziomek" nil (z iy0 ow1 m eh0 k))
("zion" nil (z ay1 ax n))
("zionism" nil (z ay1 ax n ih1 z ax m))
("zionist" nil (z ay1 ax n ih0 s t))
("zionists" nil (z ay1 ax n ih0 s t s))
("zions" nil (z ay1 ax n z))
("zip" nil (z ih1 p))
("zipf" nil (z ih1 p f))
("zipfel" nil (z ih1 p f ax l))
("zipkin" nil (z ih1 p k ih0 n))
("zipless" nil (z ih1 p l ax s))
("zipp" nil (z ih1 p))
("zipped" nil (z ih1 p t))
("zipper" nil (z ih1 p er0))
("zippered" nil (z ih1 p er0 d))
("zipperer" nil (z ih1 p er0 er0))
("zippering" nil (z ih1 p er0 ih0 ng))
("zippers" nil (z ih1 p er0 z))
("zipping" nil (z ih1 p ih0 ng))
("zippo" nil (z ih1 p ow0))
("zippora" nil (z iy0 p ao1 r ax))
("zippy" nil (z ih1 p iy0))
("zirbel" nil (z er1 b ax l))
("zirbes" nil (z er1 b z))
("zircon" nil (z er1 k ax n))
("zirconium" nil (z er0 k ow1 n iy0 ax m))
("zirconiums" nil (z er0 k ow1 n iy0 ax m z))
("zircons" nil (z er1 k aa0 n z))
("zirk" nil (z er1 k))
("zirkel" nil (z er1 k ax l))
("zirkelbach" nil (z er1 k ih0 l b aa0 k))
("zirkle" nil (z er1 k ax l))
("zisa" nil (z ay1 s ax))
("zises" nil (z ay1 z ih0 z))
("zisk" nil (z ih1 s k))
("ziska" nil (z ih1 s k ax))
("zisson" nil (z ih1 s ax n))
("zita" nil (z iy1 t ax))
("zitek" nil (z ih1 t ih0 k))
("zither" nil (z ih1 dh er0))
("zithers" nil (z ih1 dh er0 z))
("zito" nil (z iy1 t ow0))
("zittel" nil (z ih1 t ax l))
("ziv" nil (z ih1 v))
("ziyad" nil (z iy1 ae0 d))
("ziyang" nil (zh iy0 ae1 ng))
("zizzo" nil (z ih1 z ow0))
("zlata" nil (z l aa1 t ah1))
("zlogar" nil (z l ow1 g er0))
("zlotnick" nil (z l aa1 t n ih0 k))
("zloty" nil (z l ao1 t iy0))
("zlotys" nil (z l ao1 t iy0 z))
("zmijewski" nil (z m iy0 uw1 s k iy0))
("zobel" nil (z ow1 b ax l))
("zobrist" nil (z aa1 b r ih0 s t))
("zoch" nil (z aa1 k))
("zocor" nil (z ow1 k er0))
("zodiac" nil (z ow1 d iy0 ae1 k))
("zody" nil (z ow1 d iy0))
("zoe" nil (z ow1 iy0))
("zoeller" nil (z ow1 l er0))
("zoellick" nil (z ow1 l ih0 k))
("zoellner" nil (z ow1 l n er0))
("zoete" nil (z ow1 t iy0))
("zofran" nil (z ow1 f r ae1 n))
("zogg" nil (z aa1 g))
("zoghby" nil (z ow1 b iy0))
("zola" nil (z ow1 l ax))
("zoll" nil (z aa1 l))
("zollars" nil (z aa1 l er0 z))
("zoller" nil (z aa1 l er0))
("zollinger" nil (z aa1 l ih0 ng er0))
("zollman" nil (z aa1 l m ax n))
("zollner" nil (z aa1 l n er0))
("zollo" nil (z aa1 l ow0))
("zolman" nil (z aa1 l m ax n))
("zoloft" nil (z aa1 l aa0 f t))
("zolp" nil (z ow1 l p))
("zoltan" nil (z ow1 l t ax n))
("zombie" nil (z aa1 m b iy0))
("zombies" nil (z aa1 m b iy0 z))
("zon" nil (z ao1 n))
("zona" nil (z ow1 n ax))
("zondervan" nil (z aa1 n d er0 v ae0 n))
("zone" nil (z ow1 n))
("zoned" nil (z ow1 n d))
("zones" nil (z ow1 n z))
("zoning" nil (z ow1 n ih0 ng))
("zoo" nil (z uw1))
("zook" nil (z uh1 k))
("zoological" nil (z uw1 l aa1 jh ih0 k ax l))
("zoologist" nil (z ow0 aa1 l ax jh ax s t))
("zoology" nil (z ow0 aa1 l ax jh iy0))
("zoom" nil (z uw1 m))
("zoomed" nil (z uw1 m d))
("zooming" nil (z uw1 m ih0 ng))
("zooms" nil (z uw1 m z))
("zoos" nil (z uw1 z))
("zopfi" nil (z ow1 p f iy0))
("zora" nil (z ao1 r ax))
("zorah" nil (z ao1 r ax))
("zoran" nil (z ao1 r ax n))
("zorana" nil (z ao0 r ae1 n ax))
("zorich" nil (z ao1 r ih0 k))
("zorina" nil (z ao0 r iy1 n ax))
("zorine" nil (z ao0 r iy1 n iy0))
("zorinsky" nil (z ao0 r ih1 n s k iy0))
("zorn" nil (z ao1 r n))
("zornes" nil (z ao1 r n z))
("zornow" nil (z ao1 r n aw0))
("zortman" nil (z ao1 r t m ax n))
("zorzi" nil (z ao1 r z iy0))
("zosen" nil (z ow1 z ax n))
("zoss" nil (z ao1 s))
("zoster" nil (z aa1 s t er0))
("zotos" nil (z ow1 t ow0 z))
("zotter" nil (z aa1 t er0))
("zoucha" nil (z uw1 hh ax))
("zovirax" nil (z ow1 v ih0 r ae0 k s))
("zsa" nil (zh aa1))
("zsazsa" nil (zh aa1 zh aa1))
("zschau" nil (zh aw1))
("zuba" nil (z uw1 b ax))
("zube" nil (z uw1 b))
("zuber" nil (z uw1 b er0))
("zubia" nil (z uw1 b iy0 ax))
("zubillaga" nil (z uw1 b ih0 l aa1 g ax))
("zubin" nil (z uw1 b ih0 n))
("zuboff" nil (z uw1 b ao0 f))
("zubrod" nil (z ah1 b r ax d))
("zuccarelli" nil (z uw0 k er0 eh1 l iy0))
("zuccaro" nil (z uw0 k aa1 r ow0))
("zucchini" nil (z uw0 k iy1 n iy0))
("zucco" nil (z uw1 k ow0))
("zuch" nil (z ah1 ch))
("zuchowski" nil (z ax hh ao1 f s k iy0))
("zuck" nil (z ah1 k))
("zucker" nil (z ah1 k er0))
("zuckerman" nil (z ah1 k er0 m ax n))
("zue" nil (z uw1))
("zuege" nil (z uw1 jh))
("zuehlke" nil (z uh1 l k))
("zuelke" nil (z uw1 l k))
("zuercher" nil (z uh1 r k er0))
("zufall" nil (z uw1 f ax l))
("zufelt" nil (z ah1 f ih0 l t))
("zug" nil (z ah1 g))
("zuhlke" nil (z uw1 l k))
("zuidema" nil (z uw0 ih0 d eh1 m ax))
("zuk" nil (z ah1 k))
("zukas" nil (z uw1 k ax z))
("zuker" nil (z uw1 k er0))
("zukerman" nil (z uw1 k er0 m ax n))
("zukin" nil (z uw1 k ih0 n))
("zukowski" nil (z ax k ao1 f s k iy0))
("zulauf" nil (z uw1 l aw0 f))
("zuleger" nil (z uw1 l ih0 g er0))
("zulfikar" nil (z ah1 l f ih0 k aa1 r))
("zulli" nil (z uw1 l iy0))
("zullo" nil (z uw1 l ow0))
("zulu" nil (z uw1 l uw1))
("zuluaga" nil (z uw1 l uw0 aa1 g ax))
("zulus" nil (z uw1 l uw0 z))
("zumbro" nil (z ah1 m b r ow0))
("zumbrun" nil (z ah1 m b r ax n))
("zumel" nil (z uw1 m eh1 l))
("zummo" nil (z uw1 m ow0))
("zumpano" nil (z uw0 m p aa1 n ow0))
("zumstein" nil (z ah1 m s t iy1 n))
("zumwalt" nil (z ah1 m w ax l t))
("zundel" nil (z ah1 n d ax l))
("zuniga" nil (z uw0 n iy1 g ax))
("zunino" nil (z uw0 n iy1 n ow0))
("zunker" nil (z ah1 ng k er0))
("zupan" nil (z uw1 p ax n))
("zupancic" nil (z ax p ae1 ng k ih0 k))
("zupko" nil (z ah1 p k ow0))
("zurawski" nil (z er0 aa1 f s k iy0))
("zurcher" nil (z er1 k er0))
("zurek" nil (z uh1 r eh0 k))
("zurfluh" nil (z er1 f l uw0))
("zurich" nil (z uh1 r ih0 k))
("zurita" nil (z er0 ay1 t ax))
("zurkuhlen" nil (z er0 k y uw1 l ax n))
("zurn" nil (z er1 n))
("zurowski" nil (z er0 ao1 f s k iy0))
("zuver" nil (z uw1 v er0))
("zwack" nil (z w ao1 k))
("zwahlen" nil (z w aa1 l ax n))
("zwart" nil (z w ao1 r t))
("zweber" nil (z w iy1 b er0))
("zweibel" nil (z w ay1 b ax l))
("zweifel" nil (z w ay1 f ax l))
("zweig" nil (z w ay1 g))
("zwerdling" nil (z w er1 d l ih0 ng))
("zwetchkenbaum" nil (z w eh1 ch k ax n b aa0 m))
("zwick" nil (z w ih1 k))
("zwicker" nil (z w ih1 k er0))
("zwicky" nil (z w ih1 k iy0))
("zwiebel" nil (z w iy1 b ax l))
("zwiefelhofer" nil (z w iy1 f ax l hh aa1 f er0))
("zwieg" nil (z w iy1 g))
("zwilling" nil (z w ih1 l ih0 ng))
("zwolinski" nil (z v ax l ih1 n s k iy0))
("zycad" nil (z ih1 k ae1 d))
("zych" nil (z ay1 ch))
("zycher" nil (z ih1 k er0))
("zydeco" nil (z ay1 d eh1 k ow1))
("zygmunt" nil (z ih1 g m ax n t))
("zygote" nil (z ay1 g ow0 t))
("zyla" nil (z ih1 l ax))
("zylka" nil (z ih1 l k ax))
("zylstra" nil (z ih1 l s t r ax))
("zyman" nil (z ay1 m ax n))
("zynda" nil (z ih1 n d ax))
("zysk" nil (z ay1 s k))
("zyskowski" nil (z ih0 s k ao1 f s k iy0))
("zywicki" nil (z ih0 w ih1 k iy0))
("zzzz" nil (z iy1 z))
