( "'em"  prp ( @ m ) ((pos "Qx$" ) ))
( "'neath"  in ( n ii th ) ((pos "T-$" ) ))
( "'shun"  uh ( sh uh1 n ) ((pos "W-$" ) ))
( "'twas"  v ( t w o z ) ((pos "Gf$" ) ))
( "'tween"  a ( t w ii n ) ((pos "Pu$" "T-$" ) ))
( "'tween-decks"  a ( t w ii1 n - d e k s ) ((pos "Pu$" ) ))
( "'twere"  v ( t w @@ r ) ((pos "Gf$" ) ))
( "'twill"  v ( t w i l ) ((pos "Gf$" ) ))
( "'twixt"  in ( t w i k s t ) ((pos "T-$" ) ))
( "'twould"  v ( t w u d ) ((pos "Gf$" ) ))
( "'un"  prp ( @ n ) ((pos "Qx$" ) ))
( "A"  n ( ei1 ) ((pos "Ki$" ) ))
( "A's"  n ( ei1 z ) ((pos "Kj$" ) ))
( "A-bomb"  n ( ei1 - b o m ) ((pos "K6$" ) ))
( "A-bombs"  n ( ei1 - b o m z ) ((pos "Kj$" ) ))
( "A-level"  n ( ei1 - l e v @ l ) ((pos "K6%" ) ))
( "A-levels"  n ( ei1 - l e v @ l z ) ((pos "Kj%" ) ))
( "AA"  n ( ei2 ei1 ) ((pos "Y>%" ) ))
( "ABC"  n ( ei2 b ii2 s ii1 ) ((pos "Y>%" ) ))
( "ABCs"  n ( ei2 b ii2 s ii1 z ) ((pos "Y)$" ) ))
( "AD"  n ( ei2 d ii1 ) ((pos "Y~%" ) ))
( "AGM"  n ( ei2 jh i e1 m ) ((pos "Y>%" ) ))
( "AIDS"  n ( ei1 d z ) ((pos "Y}%" ) ))
( "AWOL"  n ( ei1 w o l ) ((pos "Y~%" ) ))
( "Aachen"  n ( aa1 k @ n ) ((pos "Nn$" ) ))
( "Aarhus"  n ( aa1 h uu s ) ((pos "Nn$" ) ))
( "Abe"  n ( ei1 b ) ((pos "Nl$" ) ))
( "Abercarn"  n ( a1 b @ k aa n ) ((pos "Nn$" ) ))
( "Aberdare"  n ( a2 b @ d e@1 r ) ((pos "Nn$" ) ))
( "Aberdeen"  n ( a2 b @ d ii1 n ) ((pos "Nn%" ) ))
( "Abergavenny"  n ( a2 b @ g @ v e1 n ii ) ((pos "Nn%" ) ))
( "Abergele"  n ( a2 b @ g e1 l ii ) ((pos "Nn$" ) ))
( "Abertillery"  n ( a2 b @ t i l e@1 r ii ) ((pos "Nn%" ) ))
( "Aberystwyth"  n ( a2 b @ r i1 s t w i th ) ((pos "Nn%" ) ))
( "Abingdon"  n ( a1 b i ng d @ n ) ((pos "Nn%" ) ))
( "Abo"  n ( a1 b ou ) ((pos "K6$" ) ))
( "Aborigine"  n ( a2 b @ r i1 jh @ n ii ) ((pos "K6%" ) ))
( "Aborigines"  n ( a2 b @ r i1 jh @ n i z ) ((pos "Kj%" ) ))
( "Abos"  n ( a1 b ou z ) ((pos "Kj$" ) ))
( "Abraham"  n ( ei1 b r @ h a m ) ((pos "Nl%" ) ))
( "Accra"  n ( @1 k r aa1 ) ((pos "Nn%" ) ))
( "Accrington"  n ( a1 k r i ng t @ n ) ((pos "Nn%" ) ))
( "Achilles"  n ( @1 k i1 l ii z ) ((pos "Nl%" ) ))
( "Ada"  n ( ei1 d @ ) ((pos "Nl%" ) ))
( "Adam"  n ( a1 d @ m ) ((pos "Nl%" ) ))
( "Addis Ababa"  n ( a2 d i s - a1 b @ b @ ) ((pos "Nn%" ) ))
( "Addressograph"  n ( @1 d r e1 s ou g r aa f ) ((pos "K6$" ) ))
( "Addressographs"  n ( @1 d r e1 s ou g r aa f s ) ((pos "Kj$" ) ))
( "Adelaide"  n ( a1 d @ l ei d ) ((pos "Nn%" ) ))
( "Adrian"  n ( ei1 d r i@ n ) ((pos "Nl%" ) ))
( "Adventist"  n ( a1 d v @ n t i s t ) ((pos "K6$" ) ))
( "Adventists"  n ( a1 d v @ n t i s t s ) ((pos "Kj$" ) ))
( "Afghan"  n ( a1 f g a n ) ((pos "M6%" "OA%" ) ))
( "Afghanistan"  n ( a1 f g a2 n i s t aa1 n ) ((pos "Nm%" ) ))
( "Afghanistani"  n ( a1 f g a2 n i s t aa1 n ii ) ((pos "K6$" "OA$" ) ))
( "Afghanistanis"  n ( a1 f g a2 n i s t aa1 n i z ) ((pos "Kj$" ) ))
( "Afghans"  n ( a1 f g a n z ) ((pos "Mj%" ) ))
( "Africa"  n ( a1 f r i k @ ) ((pos "Nm%" ) ))
( "African"  n ( a1 f r i k @ n ) ((pos "K6%" "OA%" ) ))
( "Africans"  n ( a1 f r i k @ n z ) ((pos "Kj%" ) ))
( "Afrikaans"  n ( a2 f r i k aa1 n s ) ((pos "L@%" ) ))
( "Afrikaner"  n ( a2 f r i k aa1 n @ r ) ((pos "K6%" "OA%" ) ))
( "Afrikaners"  n ( a2 f r i k aa1 n @ z ) ((pos "Kj%" ) ))
( "Afro-"  j ( a2 f r ou - ) ((pos "U-%" ) ))
( "Afro-American"  n ( a2 f r ou - @ m e1 r i k @ n ) ((pos "K6%" ) ))
( "Afro-Americans"  n ( a2 f r ou - @ m e1 r i k @ n z ) ((pos "Kj%" ) ))
( "Afro-Asian"  j ( a2 f r ou - ei1 sh @ n ) ((pos "OA%" ) ))
( "Afro-wig"  n ( a2 f r ou - w i1 g ) ((pos "K6$" ) ))
( "Afro-wigs"  n ( a2 f r ou - w i1 g z ) ((pos "Kj$" ) ))
( "Agatha"  n ( a1 g @ th @ ) ((pos "Nl%" ) ))
( "Aggie"  n ( a1 g ii ) ((pos "Nl$" ) ))
( "Agnes"  n ( a1 g n i s ) ((pos "Nl%" ) ))
( "Agra"  n ( aa1 g r @ ) ((pos "Nn$" ) ))
( "Ahmedabad"  n ( aa1 m e1 d @ b a d ) ((pos "Nn$" ) ))
( "Airdrie"  n ( e@1 d r ii ) ((pos "Nn%" ) ))
( "Aireborough"  n ( e@1 b @ r @ ) ((pos "Nn$" ) ))
( "Airedale"  n ( e@1 d ei l ) ((pos "K6$" ) ))
( "Airedales"  n ( e@1 d ei l z ) ((pos "Kj$" ) ))
( "Akron"  n ( a1 k r o n ) ((pos "Nn$" ) ))
( "Al"  n ( a1 l ) ((pos "Nl%" ) ))
( "Alabama"  n ( a2 l @ b a1 m @ ) ((pos "Nm%" ) ))
( "Alan"  n ( a1 l @ n ) ((pos "Nl%" ) ))
( "Alaska"  n ( @1 l a1 s k @ ) ((pos "Nm%" ) ))
( "Albania"  n ( a1 l b ei1 n i@ ) ((pos "Nm%" ) ))
( "Albanian"  n ( a1 l b ei1 n i@ n ) ((pos "M6%" "OA%" ) ))
( "Albanians"  n ( a1 l b ei1 n i@ n z ) ((pos "Mj%" ) ))
( "Albert"  n ( a1 l b @ t ) ((pos "Nl%" ) ))
( "Alberta"  n ( a1 l b @@1 t @ ) ((pos "Nm%" ) ))
( "Aldershot"  n ( oo1 l d @ sh o t ) ((pos "Nn%" ) ))
( "Aleppo"  n ( @1 l e1 p ou ) ((pos "Nn$" ) ))
( "Alessandria"  n ( a2 l i s a1 n d r i@ ) ((pos "Nn$" ) ))
( "Alex"  n ( a1 l i k s ) ((pos "Nl%" ) ))
( "Alexander"  n ( a2 l i g z aa1 n d @ r ) ((pos "Nl%" ) ))
( "Alexandra"  n ( a2 l i g z aa1 n d r @ ) ((pos "Nl%" ) ))
( "Alexandria"  n ( a2 l i g z aa1 n d r i@ ) ((pos "Nn%" ) ))
( "Alf"  n ( a1 l f ) ((pos "Nl%" ) ))
( "Alfred"  n ( a1 l f r i d ) ((pos "Nl%" ) ))
( "Alfreton"  n ( oo1 l f r i t @ n ) ((pos "Nn$" ) ))
( "Alger"  n ( a1 l jh @ r ) ((pos "Nn$" ) ))
( "Algeria"  n ( a1 l jh i@1 r i@ ) ((pos "Nm%" ) ))
( "Algerian"  n ( a1 l jh i@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Algerians"  n ( a1 l jh i@1 r i@ n z ) ((pos "Kj%" ) ))
( "Ali"  n ( a1 l ii ) ((pos "Nl%" ) ))
( "Alicante"  n ( a1 l i k a1 n t ii ) ((pos "Nn$" ) ))
( "Alice"  n ( a1 l i s ) ((pos "Nl%" ) ))
( "Alison"  n ( a1 l i s n ) ((pos "Nl%" ) ))
( "All right"  uh ( oo2 l - r ai1 t ) ((pos "W-%" ) ))
( "Allah"  n ( a1 l @ ) ((pos "No%" ) ))
( "Allahabad"  n ( a2 l @ h @ b aa1 d ) ((pos "Nn$" ) ))
( "Allan"  n ( a1 l @ n ) ((pos "Nl%" ) ))
( "Allen"  n ( a1 l @ n ) ((pos "Nl%" ) ))
( "Alloa"  n ( a1 l ou @ ) ((pos "Nn$" ) ))
( "Alma Mater"  n ( a2 l m @ - m aa1 t @ r ) ((pos "K6$" ) ))
( "Alma Maters"  n ( a2 l m @ - m aa1 t @ z ) ((pos "Kj$" ) ))
( "Alma-Ata"  n ( a2 l m @ - aa1 t @ ) ((pos "Nn$" ) ))
( "Almeria"  n ( a2 l m @ r i@1 ) ((pos "Nn$" ) ))
( "Alnwick"  n ( a1 n i k ) ((pos "Nn$" ) ))
( "Alright"  uh ( oo2 l r ai1 t ) ((pos "W-%" ) ))
( "Alsatian"  n ( a1 l s ei1 sh @ n ) ((pos "K6%" ) ))
( "Alsatians"  n ( a1 l s ei1 sh @ n z ) ((pos "Kj%" ) ))
( "Alton"  n ( oo1 l t @ n ) ((pos "Nn%" ) ))
( "Altrincham"  n ( oo1 l t r i ng @ m ) ((pos "Nn%" ) ))
( "Alvechurch"  n ( oo1 l v ch @@ ch ) ((pos "Nn$" ) ))
( "Amanda"  n ( @1 m a1 n d @ ) ((pos "Nl%" ) ))
( "Amazon"  n ( a1 m @ z @ n ) ((pos "K6%" ) ))
( "Amazons"  n ( a1 m @ z @ n z ) ((pos "Kj%" ) ))
( "America"  n ( @1 m e1 r i k @ ) ((pos "Nm%" ) ))
( "American"  n ( @1 m e1 r i k @ n ) ((pos "K6*" "OA*" ) ))
( "Americanism"  n ( @1 m e1 r i k @ n i z @ m ) ((pos "M6%" ) ))
( "Americanisms"  n ( @1 m e1 r i k @ n i z @ m z ) ((pos "Mj%" ) ))
( "Americans"  n ( @1 m e1 r i k @ n z ) ((pos "Kj%" ) ))
( "Amesbury"  n ( ei1 m z b @ r ii ) ((pos "Nn$" ) ))
( "Amharic"  n ( a1 m h a1 r i k ) ((pos "L@%" ) ))
( "Amiens"  n ( a1 m i@ n z ) ((pos "Nn%" ) ))
( "Ammanford"  n ( a1 m @ n f @ d ) ((pos "Nn$" ) ))
( "Amsterdam"  n ( a2 m s t @ d a1 m ) ((pos "Nn%" ) ))
( "Amy"  n ( ei1 m ii ) ((pos "Nl%" ) ))
( "Ancona"  n ( a1 n k ou1 n @ ) ((pos "Nn$" ) ))
( "Andorra"  n ( a1 n d oo1 r @ ) ((pos "Nm%" ) ))
( "Andorran"  n ( a1 n d oo1 r @ n ) ((pos "K6%" "OA%" ) ))
( "Andorrans"  n ( a1 n d oo1 r @ n z ) ((pos "Kj%" ) ))
( "Andover"  n ( a1 n d ou v @ r ) ((pos "Nn%" ) ))
( "Andrew"  n ( a1 n d r uu ) ((pos "Nl%" ) ))
( "Andy"  n ( a1 n d ii ) ((pos "Nl%" ) ))
( "Angela"  n ( a1 n jh @ l @ ) ((pos "Nl%" ) ))
( "Angers"  n ( a1 n zh e@1 z ) ((pos "Nn$" ) ))
( "Angie"  n ( a1 n jh ii ) ((pos "Nl%" ) ))
( "Anglican"  n ( a1 ng g l i k @ n ) ((pos "K6%" "OA%" ) ))
( "Anglicans"  n ( a1 ng g l i k @ n z ) ((pos "Kj%" ) ))
( "Anglo-"  j ( a2 ng g l ou - ) ((pos "U-%" ) ))
( "Anglo-Catholic"  n ( a2 ng g l ou - k a1 th l i k ) ((pos "K6%" "OA%" ) ))
( "Anglo-Catholics"  n ( a2 ng g l ou - k a1 th l i k s ) ((pos "Kj%" ) ))
( "Anglo-Indian"  n ( a2 ng g l ou - i1 n d i@ n ) ((pos "K6%" "OA%" ) ))
( "Anglo-Indians"  n ( a2 ng g l ou - i1 n d i@ n z ) ((pos "Kj%" ) ))
( "Anglo-Saxon"  n ( a2 ng g l ou - s a1 k s @ n ) ((pos "K6%" "OA%" ) ))
( "Anglo-Saxons"  n ( a2 ng g l ou - s a1 k s @ n z ) ((pos "Kj%" ) ))
( "Anglomania"  n ( a2 ng g l ou m ei1 n i@ ) ((pos "L@$" ) ))
( "Anglophil"  n ( a1 ng g l ou f i l ) ((pos "K6$" ) ))
( "Anglophile"  n ( a1 ng g l ou f ai l ) ((pos "K6%" ) ))
( "Anglophiles"  n ( a1 ng g l ou f ai l z ) ((pos "Kj%" ) ))
( "Anglophils"  n ( a1 ng g l ou f i l z ) ((pos "Kj$" ) ))
( "Anglophobe"  n ( a1 ng g l ou f ou b ) ((pos "K6$" ) ))
( "Anglophobes"  n ( a1 ng g l ou f ou b z ) ((pos "Kj$" ) ))
( "Anglophobia"  n ( a2 ng g l ou f ou1 b i@ ) ((pos "L@$" ) ))
( "Angola"  n ( a1 ng g ou1 l @ ) ((pos "Nm%" ) ))
( "Angolan"  n ( a1 ng g ou1 l @ n ) ((pos "K6%" "OA%" ) ))
( "Angolans"  n ( a1 ng g ou1 l @ n z ) ((pos "Kj%" ) ))
( "Anguilla"  n ( a1 ng g w i1 l @ ) ((pos "Nm$" ) ))
( "Anguillan"  n ( a1 ng g w i1 l @ n ) ((pos "K6$" "OA$" ) ))
( "Anguillans"  n ( a1 ng g w i1 l @ n z ) ((pos "Kj$" ) ))
( "Angus"  n ( a1 ng g @ s ) ((pos "Nl%" ) ))
( "Anita"  n ( @1 n ii1 t @ ) ((pos "Nl%" ) ))
( "Ankara"  n ( a1 ng k @ r @ ) ((pos "Nn%" ) ))
( "Ann"  n ( a1 n ) ((pos "Nl%" ) ))
( "Annabel"  n ( a1 n @ b e l ) ((pos "Nl%" ) ))
( "Annan"  n ( a1 n @ n ) ((pos "Nn$" ) ))
( "Anne"  n ( a1 n ) ((pos "Nl%" ) ))
( "Annfield"  n ( a1 n f ii l d ) ((pos "Nn$" ) ))
( "Annfield Plain"  n ( a2 n f ii l d - p l ei1 n ) ((pos "Nn$" ) ))
( "Annie"  n ( a1 n ii ) ((pos "Nl%" ) ))
( "Anno Domini"  nil ( a2 n ou - d o1 m i n ai ) ((pos "Z-%" ) ))
( "Anon"  n ( @1 n o1 n ) ((pos "Y=%" ) ))
( "Anshan"  n ( a1 n sh @ n ) ((pos "Nn$" ) ))
( "Anthea"  n ( a1 n th i@ ) ((pos "Nl%" ) ))
( "Anthony"  n ( a1 n t @ n ii ) ((pos "Nl%" ) ))
( "Antigua"  n ( a1 n t ii1 g @ ) ((pos "Nm%" ) ))
( "Antiguan"  n ( a1 n t ii1 g @ n ) ((pos "K6%" "OA%" ) ))
( "Antiguans"  n ( a1 n t ii1 g @ n z ) ((pos "Kj%" ) ))
( "Antony"  n ( a1 n t @ n ii ) ((pos "Nl%" ) ))
( "Antrim"  n ( a1 n t r i m ) ((pos "Nm%" ) ))
( "Antwerp"  n ( a1 n t w @@ p ) ((pos "Nn%" ) ))
( "Apeldoorn"  n ( a1 p l d oo n ) ((pos "Nn$" ) ))
( "Apocrypha"  n ( @1 p o1 k r i f @ ) ((pos "Lk%" ) ))
( "Apr"  n ( ei1 p r i l ) ((pos "Y=%" ) ))
( "April"  n ( ei1 p r @ l ) ((pos "K6%" "Nl%" ) ))
( "Aprils"  n ( ei1 p r @ l z ) ((pos "Kj$" ) ))
( "Aquarius"  n ( @1 k w e@1 r i@ s ) ((pos "No%" ) ))
( "Arab"  n ( a1 r @ b ) ((pos "K6%" ) ))
( "Arabian"  n ( @1 r ei1 b i@ n ) ((pos "K6%" "OA%" ) ))
( "Arabians"  n ( @1 r ei1 b i@ n z ) ((pos "Kj%" ) ))
( "Arabic"  n ( a1 r @ b i k ) ((pos "L@%" "OA%" ) ))
( "Arabist"  n ( a1 r @ b i s t ) ((pos "K6$" ) ))
( "Arabists"  n ( a1 r @ b i s t s ) ((pos "Kj$" ) ))
( "Arabs"  n ( a1 r @ b z ) ((pos "Kj%" ) ))
( "Arbroath"  n ( aa1 b r ou1 th ) ((pos "Nn%" ) ))
( "Arcadian"  n ( aa1 k ei1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Arcadians"  n ( aa1 k ei1 d i@ n z ) ((pos "Kj%" ) ))
( "Ardrossan"  n ( aa1 d r o1 s @ n ) ((pos "Nn$" ) ))
( "Argentina"  n ( aa2 jh @ n t ii1 n @ ) ((pos "Nm%" ) ))
( "Argentine"  n ( aa1 jh @ n t ai n ) ((pos "Nm%" ) ))
( "Argentinian"  n ( aa2 jh @ n t ii1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Argentinians"  n ( aa2 jh @ n t ii1 n i@ n z ) ((pos "Kj%" ) ))
( "Argonaut"  n ( aa1 g @ n oo t ) ((pos "K6%" ) ))
( "Argonauts"  n ( aa1 g @ n oo t s ) ((pos "Kj%" ) ))
( "Argus"  n ( aa1 g @ s ) ((pos "K7$" ) ))
( "Argus-eyed"  j ( aa1 g @ s - ai d ) ((pos "OA$" ) ))
( "Arguses"  n ( aa1 g @ s i z ) ((pos "Kj$" ) ))
( "Aries"  n ( e@1 r ii z ) ((pos "No%" ) ))
( "Arizona"  n ( a2 r i z ou1 n @ ) ((pos "Nm%" ) ))
( "Arkansas"  n ( aa1 k @ n s oo ) ((pos "Nm%" ) ))
( "Armadale"  n ( aa1 m @ d ei l ) ((pos "Nn$" ) ))
( "Armageddon"  n ( aa2 m @ g e1 d n ) ((pos "No%" ) ))
( "Armagh"  n ( aa1 m aa1 ) ((pos "Nm%" ) ))
( "Arnhem"  n ( aa1 n @ m ) ((pos "Nn%" ) ))
( "Arnold"  n ( aa1 n l d ) ((pos "Nl%" ) ))
( "Art"  n ( aa1 t ) ((pos "Nl%" ) ))
( "Arthur"  n ( aa1 th @ r ) ((pos "Nl%" ) ))
( "Artie"  n ( aa1 t ii ) ((pos "Nl%" ) ))
( "Aryan"  n ( e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Aryans"  n ( e@1 r i@ n z ) ((pos "Kj%" ) ))
( "Ascot"  n ( a1 s k @ t ) ((pos "Nn%" ) ))
( "Ashbourne"  n ( a1 sh b oo n ) ((pos "Nn%" ) ))
( "Ashby"  n ( a1 sh b ii ) ((pos "Nn$" ) ))
( "Ashby Woulds"  n ( a2 sh b i - w ou1 l d z ) ((pos "Nn$" ) ))
( "Ashby de la Zouch"  n ( a2 sh b i - d @ - l @ - z uu1 sh ) ((pos "Nn$" ) ))
( "Ashford"  n ( a1 sh f @ d ) ((pos "Nn%" ) ))
( "Ashington"  n ( a1 sh i ng t @ n ) ((pos "Nn$" ) ))
( "Ashton-in-Makerfield"  n ( a2 sh t @ n - i n - m ei1 k @ f ii l d ) ((pos "Nn$" ) ))
( "Ashton-under-Lyne"  n ( a2 sh t @ n - uh n d @ - l ai1 n ) ((pos "Nn$" ) ))
( "Asian"  n ( ei1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Asians"  n ( ei1 sh @ n z ) ((pos "Kj%" ) ))
( "Asiatic"  n ( ei2 sh i a1 t i k ) ((pos "K6%" "OA%" ) ))
( "Asiatics"  n ( ei2 sh i a1 t i k s ) ((pos "Kj%" ) ))
( "Askern"  n ( a1 s k @ n ) ((pos "Nn$" ) ))
( "Aspull"  n ( a1 s p @ l ) ((pos "Nn$" ) ))
( "Aston"  n ( a1 s t @ n ) ((pos "Nn%" ) ))
( "Athenian"  n ( @1 th ii1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Athenians"  n ( @1 th ii1 n i@ n z ) ((pos "Kj%" ) ))
( "Athens"  n ( a1 th i n z ) ((pos "Nn%" ) ))
( "Atherstone"  n ( a1 th @ s t @ n ) ((pos "Nn$" ) ))
( "Athlone"  n ( a1 th l ou1 n ) ((pos "Nn%" ) ))
( "Atlanta"  n ( a1 t l a1 n t @ ) ((pos "Nn%" ) ))
( "Atlantic"  n ( @1 t l a1 n t i k ) ((pos "No%" ) ))
( "Attic"  j ( a1 t i k ) ((pos "OA$" ) ))
( "Auckland"  n ( oo1 k l @ n d ) ((pos "Nm%" ) ))
( "Audley"  n ( oo1 d l ii ) ((pos "Nn$" ) ))
( "Audrey"  n ( oo1 d r ii ) ((pos "Nl%" ) ))
( "Aug"  n ( oo1 g @ s t ) ((pos "Y=%" ) ))
( "Augsburg"  n ( au1 g z b @@ g ) ((pos "Nn$" ) ))
( "August"  n ( oo1 g @ s t ) ((pos "K6%" "No%" ) ))
( "Augustan"  j ( oo1 g uh1 s t @ n ) ((pos "OA%" ) ))
( "Augusts"  n ( oo1 g @ s t s ) ((pos "Kj$" ) ))
( "Aussie"  n ( o1 z ii ) ((pos "K6%" ) ))
( "Aussies"  n ( o1 z i z ) ((pos "Kj%" ) ))
( "Australia"  n ( o1 s t r ei1 l i@ ) ((pos "Nm%" ) ))
( "Australian"  n ( o1 s t r ei1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "Australians"  n ( o1 s t r ei1 l i@ n z ) ((pos "Kj%" ) ))
( "Austria"  n ( o1 s t r i@ ) ((pos "Nm%" ) ))
( "Austrian"  n ( o1 s t r i@ n ) ((pos "K6%" "OA%" ) ))
( "Austrians"  n ( o1 s t r i@ n z ) ((pos "Kj%" ) ))
( "Ave"  n ( a1 v ) ((pos "Y=%" ) ))
( "Aveley"  n ( ei1 v l ii ) ((pos "Nn$" ) ))
( "Avignon"  n ( a1 v i ng y o n ) ((pos "Nn%" ) ))
( "Avon"  n ( ei1 v @ n ) ((pos "Nm%" ) ))
( "Aycliffe"  n ( ei1 k l i f ) ((pos "Nn$" ) ))
( "Aylesbury"  n ( ei1 l z b @ r ii ) ((pos "Nn%" ) ))
( "Aylesford"  n ( ei1 l z f @ d ) ((pos "Nn$" ) ))
( "Ayr"  n ( e@1 r ) ((pos "Nn%" ) ))
( "B"  n ( b ii1 ) ((pos "Ki$" ) ))
( "B Ed"  n ( b ii2 - e1 d ) ((pos "Y>%" ) ))
( "B's"  n ( b ii1 z ) ((pos "Kj$" ) ))
( "BA"  n ( b ii2 ei1 ) ((pos "Y>%" ) ))
( "BBC"  n ( b ii2 b ii2 s ii1 ) ((pos "Y>%" ) ))
( "BC"  n ( b ii2 s ii1 ) ((pos "Y~%" ) ))
( "BMA"  n ( b ii2 e2 m ei1 ) ((pos "Y>%" ) ))
( "BMus"  n ( b ii2 m uh1 z ) ((pos "Y>%" ) ))
( "BST"  n ( b ii2 e2 s t ii1 ) ((pos "Y~%" ) ))
( "BSc"  n ( b ii2 e2 s s ii1 ) ((pos "Y>%" ) ))
( "BVM"  n ( b ii2 v ii2 e1 m ) ((pos "Y>%" ) ))
( "Babs"  n ( b a1 b z ) ((pos "Nl%" ) ))
( "Backworth"  n ( b a1 k w @ th ) ((pos "Nn$" ) ))
( "Bacup"  n ( b ei1 k @ p ) ((pos "Nn$" ) ))
( "Badajoz"  n ( b a1 d @ h o z ) ((pos "Nn$" ) ))
( "Badalona"  n ( b a2 d @ l ou1 n @ ) ((pos "Nn$" ) ))
( "Baghdad"  n ( b a1 g d a1 d ) ((pos "Nn%" ) ))
( "Bahamas"  n ( b @1 h aa1 m @ z ) ((pos "Nm%" ) ))
( "Bahamian"  n ( b @1 h aa1 m i@ n ) ((pos "K6%" "OA%" ) ))
( "Bahamians"  n ( b @1 h aa1 m i@ n z ) ((pos "Kj%" ) ))
( "Bahasa"  n ( b @1 h aa1 z @ ) ((pos "L@$" ) ))
( "Bahrain"  n ( b aa1 r ei1 n ) ((pos "Nm%" ) ))
( "Bahraini"  n ( b aa1 r ei1 n ii ) ((pos "K6$" "OA$" ) ))
( "Bahrainis"  n ( b aa1 r ei1 n i z ) ((pos "Kj$" ) ))
( "Bailey"  n ( b ei1 l ii ) ((pos "No%" ) ))
( "Baku"  n ( b a1 k uu ) ((pos "Nn$" ) ))
( "Baldock"  n ( b oo1 l d o k ) ((pos "Nn$" ) ))
( "Baltimore"  n ( b oo1 l t i m oo r ) ((pos "Nn%" ) ))
( "Banbury"  n ( b a1 n b @ r ii ) ((pos "Nn%" ) ))
( "Bandung"  n ( b a1 n d @ ng ) ((pos "Nn$" ) ))
( "Bangalore"  n ( b a2 ng g @ l oo1 r ) ((pos "Nn$" ) ))
( "Bangkok"  n ( b a1 ng k o1 k ) ((pos "Nn%" ) ))
( "Bangladesh"  n ( b a2 ng g l @ d e1 sh ) ((pos "Nm%" ) ))
( "Bangladeshi"  n ( b a2 ng g l @ d e1 sh ii ) ((pos "K6%" "OA%" ) ))
( "Bangladeshis"  n ( b a2 ng g l @ d e1 sh i z ) ((pos "Kj%" ) ))
( "Bangor"  n ( b a1 ng g @ r ) ((pos "Nn%" ) ))
( "Bannockburn"  n ( b a1 n @ k b @@ n ) ((pos "Nn%" ) ))
( "Bantry"  n ( b a1 n t r ii ) ((pos "Nn$" ) ))
( "Bantu"  n ( b a1 n t uu1 ) ((pos "K9%" "OA%" ) ))
( "Bantus"  n ( b a1 n t uu1 z ) ((pos "Kj%" ) ))
( "Baptist"  n ( b a1 p t i s t ) ((pos "K6%" "OA%" ) ))
( "Baptists"  n ( b a1 p t i s t s ) ((pos "Kj%" ) ))
( "Baracaldo"  n ( b a2 r @ k a1 l d ou ) ((pos "Nn$" ) ))
( "Barbadian"  n ( b aa1 b ei1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Barbadians"  n ( b aa1 b ei1 d i@ n z ) ((pos "Kj%" ) ))
( "Barbados"  n ( b aa1 b ei1 d @ s ) ((pos "Nm%" ) ))
( "Barbara"  n ( b aa1 b r @ ) ((pos "Nl%" ) ))
( "Barcelona"  n ( b aa2 s i l ou1 n @ ) ((pos "Nn%" ) ))
( "Bargoed"  n ( b aa1 g ou1 i d ) ((pos "Nn$" ) ))
( "Bari"  n ( b aa1 r ii ) ((pos "Nn$" ) ))
( "Barking"  n ( b aa1 k i ng ) ((pos "Nn%" ) ))
( "Barnard Castle"  n ( b aa2 n @ d - k aa1 s l ) ((pos "Nn%" ) ))
( "Barnet"  n ( b aa1 n i t ) ((pos "Nn%" ) ))
( "Barnoldswick"  n ( b aa1 n @ l d z w i k ) ((pos "Nn$" ) ))
( "Barnsley"  n ( b aa1 n z l ii ) ((pos "Nn%" ) ))
( "Barnstaple"  n ( b aa1 n s t @ p l ) ((pos "Nn%" ) ))
( "Barranquilla"  n ( b a2 r @ n k ii1 @ ) ((pos "Nn$" ) ))
( "Barrhead"  n ( b aa1 h e d ) ((pos "Nn$" ) ))
( "Barrow-in-Furness"  n ( b a2 r ou - i n - f @@1 n i s ) ((pos "Nn%" ) ))
( "Barry"  n ( b a1 r ii ) ((pos "Nl%" ) ))
( "Bart"  n ( b aa1 t ) ((pos "Nl$" "Y:$" ) ))
( "Bartholomew"  n ( b aa1 th o1 l @ m y uu ) ((pos "Nl%" ) ))
( "Barton-upon-Humber"  n ( b aa2 t @ n - @ p o n - h uh1 m b @ r ) ((pos "Nn$" ) ))
( "Basel"  n ( b aa1 z l ) ((pos "Nn%" ) ))
( "Basil"  n ( b a1 z l ) ((pos "Nl%" ) ))
( "Basildon"  n ( b a1 z i l d @ n ) ((pos "Nn%" ) ))
( "Basingstoke"  n ( b ei1 z ii ng s t ou k ) ((pos "Nn%" ) ))
( "Bass"  n ( b a1 s ) ((pos "L@$" ) ))
( "Bath"  n ( b aa1 th ) ((pos "Nn%" ) ))
( "Bath-chair"  n ( b aa1 th - ch e@ r ) ((pos "K6%" ) ))
( "Bath-chairs"  n ( b aa1 th - ch e@ z ) ((pos "Kj%" ) ))
( "Bathgate"  n ( b aa1 th g ei t ) ((pos "Nn$" ) ))
( "Batley"  n ( b a1 t l ii ) ((pos "Nn%" ) ))
( "Beaconsfield"  n ( b e1 k @ n z f ii l d ) ((pos "Nn%" ) ))
( "Bearsden"  n ( b e@1 z d @ n ) ((pos "Nn$" ) ))
( "Beatrice"  n ( b i@1 t r i s ) ((pos "Nl%" ) ))
( "Beaujolais"  n ( b ou1 zh @ l ei ) ((pos "L@%" ) ))
( "Beaumaris"  n ( b ou1 m a1 r i s ) ((pos "Nn%" ) ))
( "Bebington"  n ( b e1 b i ng t @ n ) ((pos "Nn$" ) ))
( "Beccles"  n ( b e1 k l z ) ((pos "Nn%" ) ))
( "Beddau"  n ( b e1 dh au ) ((pos "Nn$" ) ))
( "Bedford"  n ( b e1 d f @ d ) ((pos "Nn%" ) ))
( "Bedfordshire"  n ( b e1 d f @ d sh @ r ) ((pos "Nm%" ) ))
( "Bedlington"  n ( b e1 d l i ng t @ n ) ((pos "Nn$" ) ))
( "Bedouin"  n ( b e1 d u i n ) ((pos "K9%" ) ))
( "Beduin"  n ( b e1 d u i n ) ((pos "K9$" ) ))
( "Bedwas"  n ( b e1 d w o z ) ((pos "Nn$" ) ))
( "Beirut"  n ( b ei2 r uu1 t ) ((pos "Nn%" ) ))
( "Beith"  n ( b ii1 th ) ((pos "Nn$" ) ))
( "Belem"  n ( b e1 l @ m ) ((pos "Nn$" ) ))
( "Belfast"  n ( b e1 l f aa s t ) ((pos "Nn%" ) ))
( "Belgian"  n ( b e1 l jh @ n ) ((pos "K6%" "OA%" ) ))
( "Belgians"  n ( b e1 l jh @ n z ) ((pos "Kj%" ) ))
( "Belgium"  n ( b e1 l jh @ m ) ((pos "Nm%" ) ))
( "Belgrade"  n ( b e2 l g r ei1 d ) ((pos "Nn%" ) ))
( "Belinda"  n ( b @1 l i1 n d @ ) ((pos "Nl%" ) ))
( "Bella"  n ( b e1 l @ ) ((pos "Nl%" ) ))
( "Belle"  n ( b e1 l ) ((pos "Nl%" ) ))
( "Bellshill"  n ( b e1 l z h i l ) ((pos "Nn$" ) ))
( "Belo Horizonte"  n ( b e2 l ou - h o r i z o1 n t ii ) ((pos "Nn$" ) ))
( "Belper"  n ( b e1 l p @ r ) ((pos "Nn%" ) ))
( "Ben"  n ( b e1 n ) ((pos "Nl%" ) ))
( "Benedictine"  n ( b e2 n i d i1 k t i n ) ((pos "M6%" "OA%" ) ))
( "Benedictines"  n ( b e2 n i d i1 k t i n z ) ((pos "Mj%" ) ))
( "Bengali"  n ( b e1 ng g oo1 l ii ) ((pos "M6%" "OA%" ) ))
( "Bengalis"  n ( b e1 ng g oo1 l i z ) ((pos "Mj%" ) ))
( "Benin"  n ( b e1 n i1 n ) ((pos "Nm%" ) ))
( "Beninese"  n ( b e1 n i n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Benjamin"  n ( b e1 n jh @ m i n ) ((pos "Nl%" ) ))
( "Benny"  n ( b e1 n ii ) ((pos "Nl%" ) ))
( "Benzedrine"  n ( b e1 n z @ d r ii n ) ((pos "K6%" ) ))
( "Benzedrines"  n ( b e1 n z @ d r ii n z ) ((pos "Kj$" ) ))
( "Beograd"  n ( b ei1 ou g r a d ) ((pos "Nn$" ) ))
( "Bergamo"  n ( b @@1 g aa1 m ou ) ((pos "Nn$" ) ))
( "Bergen"  n ( b @@1 g @ n ) ((pos "Nn%" ) ))
( "Berkhamsted"  n ( b @@1 k @ m s t i d ) ((pos "Nn%" ) ))
( "Berkshire"  n ( b aa1 k sh @ r ) ((pos "Nm%" ) ))
( "Berlin"  n ( b @@1 l i1 n ) ((pos "Nn%" ) ))
( "Bermuda"  n ( b @1 m y uu1 d @ ) ((pos "Nm%" ) ))
( "Bermudan"  n ( b @1 m y uu1 d @ n ) ((pos "K6$" "OA$" ) ))
( "Bermudans"  n ( b @1 m y uu1 d @ n z ) ((pos "Kj$" ) ))
( "Bern"  n ( b @@1 n ) ((pos "Nn$" ) ))
( "Bernard"  n ( b @@1 n @ d ) ((pos "Nl%" ) ))
( "Bernie"  n ( b @@1 n ii ) ((pos "Nl%" ) ))
( "Bert"  n ( b @@1 t ) ((pos "Nl%" ) ))
( "Bertha"  n ( b @@1 th @ ) ((pos "Nl%" ) ))
( "Bertie"  n ( b @@1 t ii ) ((pos "Nl%" ) ))
( "Berwick-upon-Tweed"  n ( b e2 r i k - @ p o n - t w ii1 d ) ((pos "Nn%" ) ))
( "Beryl"  n ( b e1 r @ l ) ((pos "Nl%" ) ))
( "Bess"  n ( b e1 s ) ((pos "Nl%" ) ))
( "Bessie"  n ( b e1 s ii ) ((pos "Nl%" ) ))
( "Bethune"  n ( b e1 th uu1 n ) ((pos "Nn$" ) ))
( "Betsy"  n ( b e1 t s ii ) ((pos "Nl%" ) ))
( "Betty"  n ( b e1 t ii ) ((pos "Nl%" ) ))
( "Beverley"  n ( b e1 v @ l ii ) ((pos "Nn%" ) ))
( "Bewdley"  n ( b y uu1 d l ii ) ((pos "Nn$" ) ))
( "Bexhill"  n ( b e2 k s h i1 l ) ((pos "Nn%" ) ))
( "Bexley"  n ( b e1 k s l ii ) ((pos "Nn%" ) ))
( "Bhutan"  n ( b uu1 t aa1 n ) ((pos "Nm%" ) ))
( "Bhutani"  n ( b uu1 t aa1 n ii ) ((pos "K6$" "OA$" ) ))
( "Bhutanis"  n ( b uu1 t aa1 n i z ) ((pos "Kj$" ) ))
( "Bible"  n ( b ai1 b l ) ((pos "K6%" ) ))
( "Bibles"  n ( b ai1 b l z ) ((pos "Kj%" ) ))
( "Bicester"  n ( b i1 s t @ r ) ((pos "Nn%" ) ))
( "Biddulph"  n ( b i1 d uh l f ) ((pos "Nn$" ) ))
( "Bideford"  n ( b i1 d i f @ d ) ((pos "Nn%" ) ))
( "Bielefeld"  n ( b ii1 l @ f e l t ) ((pos "Nn$" ) ))
( "Biggleswade"  n ( b i1 g l z w ei d ) ((pos "Nn%" ) ))
( "Bilbao"  n ( b i1 l b aa1 ou ) ((pos "Nn%" ) ))
( "Bill"  n ( b i1 l ) ((pos "Nl%" ) ))
( "Billericay"  n ( b i1 l @ r i1 k ii ) ((pos "Nn%" ) ))
( "Billington"  n ( b i1 l i ng t @ n ) ((pos "Nn$" ) ))
( "Billy"  n ( b i1 l ii ) ((pos "Nl%" ) ))
( "Bingley"  n ( b i1 ng l ii ) ((pos "Nn%" ) ))
( "Birkenhead"  n ( b @@1 k @ n h e d ) ((pos "Nn%" ) ))
( "Birstall"  n ( b @@1 s t oo l ) ((pos "Nn$" ) ))
( "Birtley"  n ( b @@1 t l ii ) ((pos "Nn$" ) ))
( "Bishop Auckland"  n ( b i2 sh @ p - oo1 k l @ n d ) ((pos "Nn%" ) ))
( "Bishop's Cleeve"  n ( b i2 sh @ p s - k l ii1 v ) ((pos "Nn$" ) ))
( "Bishop's Stortford"  n ( b i2 sh @ p - s t oo1 f @ d ) ((pos "Nn%" ) ))
( "Bishopbriggs"  n ( b i1 sh @ p b r i g z ) ((pos "Nn$" ) ))
( "Blackburn"  n ( b l a1 k b @@ n ) ((pos "Nn%" ) ))
( "Blackhall"  n ( b l a1 k h oo l ) ((pos "Nn$" ) ))
( "Blackpool"  n ( b l a1 k p uu l ) ((pos "Nn%" ) ))
( "Blackshirt"  n ( b l a1 k sh @@ t ) ((pos "K6$" ) ))
( "Blackshirts"  n ( b l a1 k sh @@ t s ) ((pos "Kj$" ) ))
( "Blackwood"  n ( b l a1 k w u d ) ((pos "Nn$" ) ))
( "Blaenau-Ffestiniog"  n ( b l ei2 n @ - f e s t i1 n i o g ) ((pos "Nn%" ) ))
( "Blaenavon"  n ( b l ei1 n a1 v @ n ) ((pos "Nn$" ) ))
( "Blaengwrach"  n ( b l ei1 ng g r aa1 k ) ((pos "Nn$" ) ))
( "Blaina"  n ( b l ai1 n @ ) ((pos "Nn$" ) ))
( "Blairgowrie"  n ( b l e@1 g au1 r ii ) ((pos "Nn$" ) ))
( "Blantyre"  n ( b l a1 n t ei1 @ r ) ((pos "Nn%" ) ))
( "Bldg"  n ( b i1 l d i ng ) ((pos "Y=%" ) ))
( "Bldgs"  n ( b i1 l d i ng z ) ((pos "Y=%" ) ))
( "Bletchley"  n ( b l e1 ch l ii ) ((pos "Nn%" ) ))
( "Blidworth"  n ( b l i1 d w @ th ) ((pos "Nn$" ) ))
( "Blighty"  n ( b l ai1 t ii ) ((pos "Ki%" ) ))
( "Blvd"  n ( b uu1 l @ v aa d ) ((pos "Y>%" ) ))
( "Blyth"  n ( b l ai1 ) ((pos "Nn$" ) ))
( "Bo'ness"  n ( b ou2 n e1 s ) ((pos "Nn$" ) ))
( "Bob"  n ( b o1 b ) ((pos "Nl%" ) ))
( "Bobby"  n ( b o1 b ii ) ((pos "Nl%" ) ))
( "Boche"  n ( b o1 sh ) ((pos "Kj$" "OA$" ) ))
( "Bochum"  n ( b o1 k @ m ) ((pos "Nn$" ) ))
( "Bodmin"  n ( b o1 d m i n ) ((pos "Nn%" ) ))
( "Boer"  n ( b ou1 @ r ) ((pos "K6%" "OA%" ) ))
( "Boers"  n ( b ou1 @ z ) ((pos "Kj%" ) ))
( "Bognor"  n ( b o1 g n @ r ) ((pos "Nn%" ) ))
( "Bognor Regis"  n ( b o2 g n @ - r ii1 jh i s ) ((pos "Nn%" ) ))
( "Bogota"  n ( b o2 g @ t aa1 ) ((pos "Nn%" ) ))
( "Boldon"  n ( b o1 l d @ n ) ((pos "Nn$" ) ))
( "Bolivia"  n ( b @1 l i1 v i@ ) ((pos "Nm%" ) ))
( "Bolivian"  n ( b @1 l i1 v i@ n ) ((pos "K6%" "OA%" ) ))
( "Bolivians"  n ( b @1 l i1 v i@ n z ) ((pos "Kj%" ) ))
( "Bollington"  n ( b o1 l i ng t @ n ) ((pos "Nn$" ) ))
( "Bologna"  n ( b @1 l o1 n y @ ) ((pos "Nn%" ) ))
( "Bolshevik"  n ( b o1 l sh @ v i k ) ((pos "K6%" ) ))
( "Bolsheviks"  n ( b o1 l sh @ v i k s ) ((pos "Kj%" ) ))
( "Bolsover"  n ( b o1 l s ou v @ r ) ((pos "Nn%" ) ))
( "Bolton"  n ( b ou1 l t @ n ) ((pos "Nn%" ) ))
( "Bolton-le-Sands"  n ( b ou2 l t @ n - l i - s a1 n d z ) ((pos "Nn%" ) ))
( "Bolzano"  n ( b o1 l z aa1 n ou ) ((pos "Nn$" ) ))
( "Bombay"  n ( b o1 m b ei1 ) ((pos "Nn%" ) ))
( "Bonn"  n ( b o1 n ) ((pos "Nn%" ) ))
( "Bonnybridge"  n ( b o1 n i b r i jh ) ((pos "Nn$" ) ))
( "Bonnyrigg"  n ( b o1 n i r i g ) ((pos "Nn$" ) ))
( "Bootle"  n ( b uu1 t l ) ((pos "Nn%" ) ))
( "Bordeaux"  n ( b oo1 d ou1 ) ((pos "L@%" "Nn%" ) ))
( "Borders"  n ( b oo1 d @ z ) ((pos "Nm%" ) ))
( "Bordon"  n ( b oo1 d @ n ) ((pos "Nn$" ) ))
( "Boris"  n ( b o1 r i s ) ((pos "Nl%" ) ))
( "Boston"  n ( b o1 s t @ n ) ((pos "Nn%" ) ))
( "Botswana"  n ( b o1 t s w aa1 n @ ) ((pos "Nm%" ) ))
( "Bottrop"  n ( b o1 t r @ p ) ((pos "Nn$" ) ))
( "Bourne"  n ( b oo1 n ) ((pos "Nn$" ) ))
( "Bournemouth"  n ( b oo1 n m @ th ) ((pos "Nn%" ) ))
( "Bow Bells"  n ( b ou2 - b e1 l z ) ((pos "Kj%" ) ))
( "Bowburn"  n ( b ou1 b @ n ) ((pos "Nn$" ) ))
( "Boxing Day"  n ( b o1 k s i ng - d ei ) ((pos "K6%" "No%" ) ))
( "Boxing Days"  n ( b o1 k s i ng - d ei z ) ((pos "Kj$" ) ))
( "Bracknell"  n ( b r a1 k n @ l ) ((pos "Nn%" ) ))
( "Bradford"  n ( b r a1 d f @ d ) ((pos "Nn%" ) ))
( "Bradford-on-Avon"  n ( b r a2 d f @ d - o n - ei1 v @ n ) ((pos "Nn%" ) ))
( "Brahmin"  n ( b r aa1 m i n ) ((pos "K6%" ) ))
( "Brahmins"  n ( b r aa1 m i n z ) ((pos "Kj%" ) ))
( "Braintree"  n ( b r ei1 n t r ii ) ((pos "Nn%" ) ))
( "Branderburgh"  n ( b r a1 n d @ b @ r @ ) ((pos "Nn$" ) ))
( "Brasilia"  n ( b r @1 z i1 l i@ ) ((pos "Nn%" ) ))
( "Bratislava"  n ( b r a2 t i s l aa1 v @ ) ((pos "Nn%" ) ))
( "Braunschweig"  n ( b r oo1 n sh v ei g ) ((pos "Nn%" ) ))
( "Braunton"  n ( b r oo1 n t @ n ) ((pos "Nn$" ) ))
( "Brazil"  n ( b r @1 z i1 l ) ((pos "Nm%" ) ))
( "Brazilian"  n ( b r @1 z i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "Brazilians"  n ( b r @1 z i1 l i@ n z ) ((pos "Kj%" ) ))
( "Breaston"  n ( b r ii1 s t @ n ) ((pos "Nn$" ) ))
( "Brecknock"  n ( b r e1 k n o k ) ((pos "Nn%" ) ))
( "Breda"  n ( b r ii1 d @ ) ((pos "Nn$" ) ))
( "Breedsall"  n ( b r ii1 d s oo l ) ((pos "Nn$" ) ))
( "Bremen"  n ( b r ei1 m @ n ) ((pos "Nn%" ) ))
( "Bremerhaven"  n ( b r ei1 m @ h aa v @ n ) ((pos "Nn%" ) ))
( "Bren"  n ( b r e1 n ) ((pos "K6$" ) ))
( "Bren-gun"  n ( b r e1 n - g uh n ) ((pos "K6%" ) ))
( "Bren-guns"  n ( b r e1 n - g uh n z ) ((pos "Kj%" ) ))
( "Brenda"  n ( b r e1 n d @ ) ((pos "Nl%" ) ))
( "Brens"  n ( b r e1 n z ) ((pos "Kj$" ) ))
( "Brent"  n ( b r e1 n t ) ((pos "Nn%" ) ))
( "Brentwood"  n ( b r e1 n t w u d ) ((pos "Nn%" ) ))
( "Brescia"  n ( b r e1 sh i@ ) ((pos "Nn$" ) ))
( "Brest"  n ( b r e1 s t ) ((pos "Nn$" ) ))
( "Brian"  n ( b r ai1 @ n ) ((pos "Nl%" ) ))
( "Bricket Wood"  n ( b r i2 k i t - w u1 d ) ((pos "Nn%" ) ))
( "Bridgend"  n ( b r i1 jh e1 n d ) ((pos "Nn%" ) ))
( "Bridget"  n ( b r i1 jh i t ) ((pos "Nl%" ) ))
( "Bridgnorth"  n ( b r i1 jh n oo th ) ((pos "Nn%" ) ))
( "Bridgwater"  n ( b r i1 jh w oo t @ r ) ((pos "Nn%" ) ))
( "Bridlington"  n ( b r i1 d l i ng t @ n ) ((pos "Nn%" ) ))
( "Bridport"  n ( b r i1 d p oo t ) ((pos "Nn%" ) ))
( "Brig"  n ( b r i1 g ) ((pos "Y:%" ) ))
( "Brigadier"  n ( b r i2 g @ d i@1 r ) ((pos "K6%" ) ))
( "Brigadiers"  n ( b r i2 g @ d i@1 z ) ((pos "Kj%" ) ))
( "Brighouse"  n ( b r i1 g h au s ) ((pos "Nn%" ) ))
( "Brightlingsea"  n ( b r ai1 t l i ng s ii ) ((pos "Nn%" ) ))
( "Brighton"  n ( b r ai1 t n ) ((pos "Nn%" ) ))
( "Brisbane"  n ( b r i1 z b @ n ) ((pos "Nn%" ) ))
( "Bristol"  n ( b r i1 s t l ) ((pos "Nn%" ) ))
( "Brit"  n ( b r i1 t ) ((pos "K6%" ) ))
( "Britain"  n ( b r i1 t n ) ((pos "Nm%" ) ))
( "Britannic"  j ( b r i1 t a1 n i k ) ((pos "OA%" ) ))
( "British"  j ( b r i1 t i sh ) ((pos "OA%" ) ))
( "Britisher"  n ( b r i1 t i sh @ r ) ((pos "K6%" ) ))
( "Britishers"  n ( b r i1 t i sh @ z ) ((pos "Kj%" ) ))
( "Briton"  n ( b r i1 t n ) ((pos "K6%" "OA%" ) ))
( "Britons"  n ( b r i1 t n z ) ((pos "Kj%" ) ))
( "Brits"  n ( b r i1 t s ) ((pos "Kj%" ) ))
( "Brixham"  n ( b r i1 k s @ m ) ((pos "Nn%" ) ))
( "Brno"  n ( b @@1 n ou ) ((pos "Nn%" ) ))
( "Bromley"  n ( b r o1 m l ii ) ((pos "Nn%" ) ))
( "Bromsgrove"  n ( b r o1 m z g r ou v ) ((pos "Nn%" ) ))
( "Bros"  n ( b r o1 s ) ((pos "Y=%" ) ))
( "Broughton"  n ( b r oo1 t n ) ((pos "Nn$" ) ))
( "Broxburn"  n ( b r o1 k s b @@ n ) ((pos "Nn$" ) ))
( "Bruce"  n ( b r uu1 s ) ((pos "Nl%" ) ))
( "Bruges"  n ( b r uu1 zh ) ((pos "Nn%" ) ))
( "Brunei"  n ( b r uu1 n ai ) ((pos "Nm%" ) ))
( "Bruneian"  n ( b r uu1 n ai @ n ) ((pos "K6$" "OA$" ) ))
( "Bruneians"  n ( b r uu1 n ai @ n z ) ((pos "Kj$" ) ))
( "Brunswick"  n ( b r uh1 n z w i k ) ((pos "Nm%" ) ))
( "Brussels"  n ( b r uh1 s l z ) ((pos "Nn%" ) ))
( "Bryan"  n ( b r ai1 @ n ) ((pos "Nl%" ) ))
( "Bt"  n ( b aa1 t ) ((pos "Y:%" ) ))
( "Bucarest"  n ( b y uu2 k @ r e1 s t ) ((pos "Nn%" ) ))
( "Buckhaven"  n ( b uh1 k h ei v n ) ((pos "Nn$" ) ))
( "Buckie"  n ( b uh1 k ii ) ((pos "Nn$" ) ))
( "Buckingham"  n ( b uh1 k i ng @ m ) ((pos "Nn%" ) ))
( "Buckinghamshire"  n ( b uh1 k i ng @ m sh @ r ) ((pos "Nm%" ) ))
( "Buckley"  n ( b uh1 k l ii ) ((pos "Nn$" ) ))
( "Bucksburn"  n ( b uh1 k s b @@ n ) ((pos "Nn$" ) ))
( "Budapest"  n ( b y uu1 d @ p e s t ) ((pos "Nn%" ) ))
( "Buddhism"  n ( b u1 d i z @ m ) ((pos "L@%" ) ))
( "Buddhist"  n ( b u1 d i s t ) ((pos "K6%" ) ))
( "Buddhists"  n ( b u1 d i s t s ) ((pos "Kj%" ) ))
( "Buenos Aires"  n ( b w e2 n @ s - ei1 @ r i z ) ((pos "Nn%" ) ))
( "Buffalo"  n ( b uh1 f @ l ou ) ((pos "Nn%" ) ))
( "Bulgaria"  n ( b uh1 l g e@1 r i@ ) ((pos "Nm%" ) ))
( "Bulgarian"  n ( b uh1 l g e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Bulgarians"  n ( b uh1 l g e@1 r i@ n z ) ((pos "Kj%" ) ))
( "Bulkington"  n ( b uh1 l k i ng t n ) ((pos "Nn$" ) ))
( "Bunsen"  n ( b uh1 n s n ) ((pos "K6%" ) ))
( "Bunsens"  n ( b uh1 n s n z ) ((pos "Kj$" ) ))
( "Burberries"  n ( b @@1 b @ r i z ) ((pos "Kj$" ) ))
( "Burberry"  n ( b @@1 b @ r ii ) ((pos "K8%" ) ))
( "Burgess"  n ( b @@1 jh i s ) ((pos "Nn%" ) ))
( "Burgos"  n ( b @@1 g o s ) ((pos "Nn$" ) ))
( "Burgundy"  n ( b @@1 g @ n d ii ) ((pos "L@%" ) ))
( "Burley"  n ( b @@1 l ii ) ((pos "Nn$" ) ))
( "Burma"  n ( b @@1 m @ ) ((pos "Nm%" ) ))
( "Burmese"  n ( b @@1 m ii1 z ) ((pos "M9%" "OA%" ) ))
( "Burnham-on-sea"  n ( b @@2 n @ m - o n - s ii1 ) ((pos "Nn%" ) ))
( "Burnley"  n ( b @@1 n l ii ) ((pos "Nn%" ) ))
( "Burntisland"  n ( b @@1 n t ai1 l @ n d ) ((pos "Nn$" ) ))
( "Burntwood"  n ( b @@1 n t w u d ) ((pos "Nn$" ) ))
( "Burry Port"  n ( b uh1 r i - p oo t ) ((pos "Nn$" ) ))
( "Burscough"  n ( b @@1 s k ou ) ((pos "Nn$" ) ))
( "Burton-upon-Trent"  n ( b @@2 t n - @ p o n - t r e1 n t ) ((pos "Nn%" ) ))
( "Burundi"  n ( b u1 r u1 n d ii ) ((pos "Nm%" ) ))
( "Burundian"  n ( b u1 r u1 n d i@ n ) ((pos "K6$" "OA$" ) ))
( "Burundians"  n ( b u1 r u1 n d i@ n z ) ((pos "Kj$" ) ))
( "Bury"  n ( b e1 r ii ) ((pos "Nn%" ) ))
( "Bury St. Edmunds"  n ( b e2 r i - s n t - e1 d m @ n z ) ((pos "Nn%" ) ))
( "Bushman"  n ( b uh1 sh m @ n ) ((pos "Ki%" ) ))
( "Bushmen"  n ( b uh1 sh m @ n ) ((pos "Kj%" ) ))
( "Buxton"  n ( b uh1 k s t @ n ) ((pos "Nn%" ) ))
( "C"  n ( s ii1 ) ((pos "Ki$" ) ))
( "C of E"  n ( s ii2 - @ v - ii1 ) ((pos "Y>%" ) ))
( "C's"  n ( s ii1 z ) ((pos "Kj$" ) ))
( "C-in-C"  n ( s ii2 i n s ii1 ) ((pos "Y:%" ) ))
( "CBI"  n ( s ii2 b ii2 ai1 ) ((pos "Y>%" ) ))
( "CD"  n ( s ii2 d ii1 ) ((pos "Y>%" ) ))
( "CIA"  n ( s i2 ai2 ei1 ) ((pos "Y>%" ) ))
( "CID"  n ( s ii2 ai2 d ii1 ) ((pos "Y>%" ) ))
( "CO"  n ( s ii2 ou1 ) ((pos "Y>%" ) ))
( "COD"  n ( s ii2 ou d ii1 ) ((pos "Y~$" ) ))
( "Cadiz"  n ( k @1 d i1 z ) ((pos "Nn%" ) ))
( "Caen"  n ( k ei1 n ) ((pos "Nn%" ) ))
( "Caerleon"  n ( k aa1 l ii1 @ n ) ((pos "Nn%" ) ))
( "Caernarfon"  n ( k @1 n aa1 v @ n ) ((pos "Nn%" ) ))
( "Caerphilly"  n ( k @1 f i1 l ii ) ((pos "Nn%" ) ))
( "Caesar"  n ( s ii1 z @ r ) ((pos "K6%" ) ))
( "Caesarian"  j ( s i1 z e@1 r i@ n ) ((pos "OA%" ) ))
( "Caesars"  n ( s ii1 z @ z ) ((pos "Kj%" ) ))
( "Cagliari"  n ( k a1 l y aa1 r ii ) ((pos "Nn$" ) ))
( "Cairo"  n ( k ai1 r ou ) ((pos "Nn%" ) ))
( "Calcutta"  n ( k a1 l k uh1 t @ ) ((pos "Nn%" ) ))
( "Caldicot"  n ( k oo1 l d i k o t ) ((pos "Nn$" ) ))
( "Cali"  n ( k aa1 l ii ) ((pos "Nn$" ) ))
( "California"  n ( k a2 l i f oo1 n i@ ) ((pos "Nm%" ) ))
( "Californian"  n ( k a2 l i f oo1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Californians"  n ( k a2 l i f oo1 n i@ n z ) ((pos "Kj%" ) ))
( "Calne"  n ( k aa1 n ) ((pos "Nn$" ) ))
( "Calor"  n ( k a1 l @ ) ((pos "No%" ) ))
( "Calvaries"  n ( k a1 l v @ r i z ) ((pos "Kj$" ) ))
( "Calvary"  n ( k a1 l v @ r ii ) ((pos "K8%" ) ))
( "Calverton"  n ( k a1 l v @ t @ n ) ((pos "Nn$" ) ))
( "Calvinism"  n ( k a1 l v i n i z @ m ) ((pos "L@%" ) ))
( "Calvinist"  n ( k a1 l v i n i s t ) ((pos "K6%" ) ))
( "Calvinists"  n ( k a1 l v i n i s t s ) ((pos "Kj%" ) ))
( "Cambodia"  n ( k a1 m b ou1 d i@ ) ((pos "Nm%" ) ))
( "Cambodian"  n ( k a1 m b ou1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Cambodians"  n ( k a1 m b ou1 d i@ n z ) ((pos "Kj%" ) ))
( "Cambourne"  n ( k a1 m b oo n ) ((pos "Nn$" ) ))
( "Cambridge"  n ( k ei1 m b r i jh ) ((pos "Nn%" ) ))
( "Cambridgeshire"  n ( k ei1 m b r i jh - sh @ r ) ((pos "Nm%" ) ))
( "Cambuslang"  n ( k a1 m b @ s l a ng ) ((pos "Nn$" ) ))
( "Camden"  n ( k a1 m d @ n ) ((pos "Nn%" ) ))
( "Camembert"  n ( k a1 m @ m b e@ r ) ((pos "K6%" ) ))
( "Camemberts"  n ( k a1 m @ m b e@ z ) ((pos "Kj$" ) ))
( "Cameroon"  n ( k a2 m @ r uu1 n ) ((pos "Nm%" ) ))
( "Cameroonian"  n ( k a2 m @ r uu1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Cameroonians"  n ( k a2 m @ r uu1 n i@ n z ) ((pos "Kj%" ) ))
( "Campbeltown"  n ( k a1 m b @ l t au n ) ((pos "Nn%" ) ))
( "Canada"  n ( k a1 n @ d @ ) ((pos "Nm%" ) ))
( "Canadian"  n ( k @1 n ei1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Canadians"  n ( k @1 n ei1 d i@ n z ) ((pos "Kj%" ) ))
( "Canberra"  n ( k a1 n b @ r @ ) ((pos "Nn%" ) ))
( "Cancer"  n ( k a1 n s @ r ) ((pos "No%" ) ))
( "Cannes"  n ( k a1 n ) ((pos "Nn%" ) ))
( "Cannock"  n ( k a1 n @ k ) ((pos "Nn%" ) ))
( "Cantab"  n ( k a1 n t a b ) ((pos "Y~$" ) ))
( "Canterbury"  n ( k a1 n t @ b r ii ) ((pos "Nn%" ) ))
( "Canton"  n ( k a2 n t o1 n ) ((pos "Nn%" ) ))
( "Canuck"  n ( k @1 n u1 k ) ((pos "K6$" ) ))
( "Canucks"  n ( k @1 n u1 k s ) ((pos "Kj$" ) ))
( "Canvey"  n ( k a1 n v ii ) ((pos "Nn%" ) ))
( "Canvey Island"  n ( k a2 n v i - ai1 l @ n d ) ((pos "Nn%" ) ))
( "Cape Town"  n ( k ei1 p - t au n ) ((pos "Nn%" ) ))
( "Capitol"  n ( k a1 p i t l ) ((pos "K6$" ) ))
( "Capitols"  n ( k a1 p i t l z ) ((pos "Kj$" ) ))
( "Capricorn"  n ( k a1 p r i k oo n ) ((pos "No%" ) ))
( "Capt"  n ( k a1 p t ) ((pos "Y:%" ) ))
( "Caracas"  n ( k @1 r a1 k @ s ) ((pos "Nn%" ) ))
( "Cardenden"  n ( k aa1 d n d @ n ) ((pos "Nn$" ) ))
( "Cardiff"  n ( k aa1 d i f ) ((pos "Nn%" ) ))
( "Cardigan"  n ( k aa1 d i g @ n ) ((pos "Nn%" ) ))
( "Carl"  n ( k aa1 l ) ((pos "Nl%" ) ))
( "Carlisle"  n ( k aa1 l ai1 l ) ((pos "Nn%" ) ))
( "Carlow"  n ( k aa1 l ou ) ((pos "Nm$" ) ))
( "Carluke"  n ( k aa1 l uu1 k ) ((pos "Nn$" ) ))
( "Carmarthen"  n ( k @1 m aa1 dh @ n ) ((pos "Nn%" ) ))
( "Carmelite"  n ( k aa1 m @ l ai t ) ((pos "K6%" "OA%" ) ))
( "Carmelites"  n ( k aa1 m @ l ai t s ) ((pos "Kj%" ) ))
( "Carnforth"  n ( k aa1 n f @ th ) ((pos "Nn%" ) ))
( "Carnoustie"  n ( k aa1 n uu1 s t ii ) ((pos "Nn%" ) ))
( "Carol"  n ( k a1 r @ l ) ((pos "Nl%" ) ))
( "Carole"  n ( k a1 r @ l ) ((pos "Nl%" ) ))
( "Carolina"  n ( k a2 r @ l ai1 n @ ) ((pos "Nm%" ) ))
( "Caroline"  n ( k a1 r @ l ai n ) ((pos "Nl%" ) ))
( "Carolyn"  n ( k a1 r @ l i n ) ((pos "Nl%" ) ))
( "Carrie"  n ( k a1 r ii ) ((pos "Nl%" ) ))
( "Carron"  n ( k a1 r @ n ) ((pos "Nn$" ) ))
( "Cartagena"  n ( k aa2 t @ jh ii1 n @ ) ((pos "Nn$" ) ))
( "Casablanca"  n ( k a2 s @ b l a1 ng k @ ) ((pos "Nn%" ) ))
( "Castleford"  n ( k aa1 s l f @ d ) ((pos "Nn%" ) ))
( "Catalan"  n ( k a1 t @ l a n ) ((pos "L@%" "OA%" ) ))
( "Catania"  n ( k @1 t ei1 n i@ ) ((pos "Nn$" ) ))
( "Cath"  n ( k a1 th ) ((pos "Nl%" ) ))
( "Catherine"  n ( k a1 th r i n ) ((pos "Nl%" ) ))
( "Catholic"  n ( k a1 th l i k ) ((pos "K6%" "OA%" ) ))
( "Catholicism"  n ( k @1 th o1 l i s ii z m ) ((pos "L@%" ) ))
( "Catholics"  n ( k a1 th l i k s ) ((pos "Kj%" ) ))
( "Cathy"  n ( k a1 th ii ) ((pos "Nl%" ) ))
( "Caucasian"  n ( k oo1 k ei1 z i@ n ) ((pos "K6%" "OA%" ) ))
( "Caucasians"  n ( k oo1 k ei1 z i@ n z ) ((pos "Kj%" ) ))
( "Cavan"  n ( k a1 v @ n ) ((pos "Nm$" ) ))
( "Cdr"  n ( k @1 m aa1 n d @ r ) ((pos "Y:%" ) ))
( "Cdre"  n ( k o1 m @ d oo r ) ((pos "Y:%" ) ))
( "Cecil"  n ( s e1 s l ) ((pos "Nl%" ) ))
( "Cecilia"  n ( s i1 s ii1 l i@ ) ((pos "Nl%" ) ))
( "Cecily"  n ( s e1 s @ l ii ) ((pos "Nl%" ) ))
( "Cedric"  n ( s e1 d r i k ) ((pos "Nl%" ) ))
( "Cefn-mawr"  n ( k e2 f n - m au1 r ) ((pos "Nn$" ) ))
( "Celia"  n ( s ii1 l i@ ) ((pos "Nl%" ) ))
( "Celsius"  nil ( s e1 l s i@ s ) ((pos "Z-%" ) ))
( "Celt"  n ( k e1 l t ) ((pos "K6%" ) ))
( "Celtic"  n ( k e1 l t i k ) ((pos "K6%" "OA%" ) ))
( "Celtics"  n ( k e1 l t i k s ) ((pos "Kj$" ) ))
( "Celts"  n ( k e1 l t s ) ((pos "Kj%" ) ))
( "Cesarean"  j ( s i1 z e@1 r i@ n ) ((pos "OA%" ) ))
( "Ceylon"  n ( s i1 l o1 n ) ((pos "Nm%" ) ))
( "Chad"  n ( ch a1 d ) ((pos "Nm%" ) ))
( "Chadian"  n ( ch a1 d i@ n ) ((pos "K6$" "OA$" ) ))
( "Chadians"  n ( ch a1 d i@ n z ) ((pos "Kj$" ) ))
( "Chalfont"  n ( ch a1 l f @ n t ) ((pos "Nn%" ) ))
( "Chalfont St Giles"  n ( ch a2 l f @ n t - s n t - jh ai1 l z ) ((pos "Nn%" ) ))
( "Chalfont St Peter"  n ( ch a2 l f @ n t - s n t - p ii1 t @ r ) ((pos "Nn%" ) ))
( "Changchun"  n ( ch a2 ng ch u1 n ) ((pos "Nn$" ) ))
( "Changsha"  n ( ch a2 ng sh aa1 ) ((pos "Nn$" ) ))
( "Chard"  n ( ch aa1 d ) ((pos "Nn$" ) ))
( "Charles"  n ( ch aa1 l z ) ((pos "Nl%" ) ))
( "Charleston"  n ( ch aa1 l s t @ n ) ((pos "K6%" ) ))
( "Charlestons"  n ( ch aa1 l s t @ n z ) ((pos "Kj$" ) ))
( "Charlie"  n ( ch aa1 l ii ) ((pos "Nl%" ) ))
( "Charlotte"  n ( sh aa1 l @ t ) ((pos "Nl%" ) ))
( "Chartism"  n ( ch aa1 t i z @ m ) ((pos "L@%" ) ))
( "Chartist"  n ( ch aa1 t i s t ) ((pos "K6%" ) ))
( "Chartists"  n ( ch aa1 t i s t s ) ((pos "Kj%" ) ))
( "Charybdis"  n ( k @1 r i1 b d i s ) ((pos "No%" ) ))
( "Chas"  n ( ch a1 z ) ((pos "Nl%" ) ))
( "Chatham"  n ( ch a1 t @ m ) ((pos "Nn%" ) ))
( "Cheadle"  n ( ch ii1 d l ) ((pos "Nn%" ) ))
( "Cheddar"  n ( ch e1 d @ r ) ((pos "L@%" ) ))
( "Cheers"  uh ( ch i@1 z ) ((pos "W-%" ) ))
( "Chelmsford"  n ( ch e1 l m z f @ d ) ((pos "Nn%" ) ))
( "Chelsea"  n ( ch e1 l s ii ) ((pos "Nn%" ) ))
( "Cheltenham"  n ( ch e1 l t n @ m ) ((pos "Nn%" ) ))
( "Chelyabinsk"  n ( ch i2 l i@ b i1 n s k ) ((pos "Nn$" ) ))
( "Chengchow"  n ( ch e2 ng ch au1 ) ((pos "Nn$" ) ))
( "Chengtu"  n ( ch e2 ng t uu1 ) ((pos "Nn$" ) ))
( "Chepstow"  n ( ch e1 p s t ou ) ((pos "Nn%" ) ))
( "Chesham"  n ( ch e1 sh @ m ) ((pos "Nn%" ) ))
( "Cheshire"  n ( ch e1 sh @ r ) ((pos "Nm%" ) ))
( "Chester"  n ( ch e1 s t @ r ) ((pos "Nn%" ) ))
( "Chester-le-Street"  n ( ch e2 s t @ - l i - s t r ii1 t ) ((pos "Nn%" ) ))
( "Chesterfield"  n ( ch e1 s t @ f ii l d ) ((pos "Nn%" ) ))
( "Chianti"  n ( k i1 a1 n t ii ) ((pos "L@%" ) ))
( "Chicago"  n ( sh i1 k aa1 g ou ) ((pos "Nn%" ) ))
( "Chichester"  n ( ch i1 ch i s t @ r ) ((pos "Nn%" ) ))
( "Chile"  n ( ch i1 l ii ) ((pos "Nm%" ) ))
( "Chilean"  n ( ch i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "Chileans"  n ( ch i1 l i@ n z ) ((pos "Kj%" ) ))
( "China"  n ( ch ai1 n @ ) ((pos "Nm%" ) ))
( "Chinchow"  n ( ch i2 n ch au1 ) ((pos "Nn$" ) ))
( "Chinese"  n ( ch ai1 n ii1 z ) ((pos "M9%" "OA%" ) ))
( "Chippendale"  n ( ch i1 p @ n d ei l ) ((pos "K6%" ) ))
( "Chippendales"  n ( ch i1 p @ n d ei l z ) ((pos "Kj%" ) ))
( "Chippenham"  n ( ch i1 p n @ m ) ((pos "Nn%" ) ))
( "Chloe"  n ( k l ou1 ii ) ((pos "Nl%" ) ))
( "Chopwell"  n ( ch o1 p w i l ) ((pos "Nn$" ) ))
( "Chorley"  n ( ch oo1 l ii ) ((pos "Nn%" ) ))
( "Chris"  n ( k r i1 s ) ((pos "Nl%" ) ))
( "Chrissie"  n ( k r i1 s ii ) ((pos "Nl%" ) ))
( "Chrissy"  n ( k r i1 s ii ) ((pos "Nl%" ) ))
( "Christ"  n ( k r ai1 s t ) ((pos "No%" ) ))
( "Christchurch"  n ( k r ai1 s ch @@ ch ) ((pos "Nn%" ) ))
( "Christendom"  n ( k r i1 s n d @ m ) ((pos "K6%" ) ))
( "Christendoms"  n ( k r i1 s n d @ m z ) ((pos "Kj$" ) ))
( "Christian"  n ( k r i1 s ch @ n ) ((pos "K6%" "Nl%" "OA%" ) ))
( "Christianity"  n ( k r i2 s t i a1 n i t ii ) ((pos "L@%" ) ))
( "Christians"  n ( k r i1 s ch @ n z ) ((pos "Kj%" ) ))
( "Christina"  n ( k r i1 s t ii1 n @ ) ((pos "Nl%" ) ))
( "Christine"  n ( k r i1 s t ii n ) ((pos "Nl%" ) ))
( "Christlike"  j ( k r ai1 s l ai k ) ((pos "OA%" ) ))
( "Christmas"  n ( k r i1 s m @ s ) ((pos "K7%" "No%" ) ))
( "Christmas-box"  n ( k r i1 s m @ s - b o k s ) ((pos "K7%" ) ))
( "Christmas-boxes"  n ( k r i1 s m @ s - b o k s i z ) ((pos "Kj%" ) ))
( "Christmas-tree"  n ( k r i1 s m @ s - t r ii ) ((pos "K6%" ) ))
( "Christmas-trees"  n ( k r i1 s m @ s - t r ii z ) ((pos "Kj%" ) ))
( "Christmases"  n ( k r i1 s m @ s i z ) ((pos "Kj%" ) ))
( "Christmastide"  n ( k r i1 s m @ s t ai d ) ((pos "K6%" ) ))
( "Christmastides"  n ( k r i1 s m @ s t ai d z ) ((pos "Kj$" ) ))
( "Christmastime"  n ( k r i1 s m @ s t ai m ) ((pos "K6%" ) ))
( "Christmastimes"  n ( k r i1 s m @ s t ai m z ) ((pos "Kj$" ) ))
( "Christopher"  n ( k r i1 s t @ f @ r ) ((pos "Nl%" ) ))
( "Chungking"  n ( ch u2 ng k i1 ng ) ((pos "Nn$" ) ))
( "Cincinnati"  n ( s i2 n s i n a1 t ii ) ((pos "Nn%" ) ))
( "Cinderella"  n ( s i2 n d @ r e1 l @ ) ((pos "K6%" ) ))
( "Cinderellas"  n ( s i2 n d @ r e1 l @ z ) ((pos "Kj$" ) ))
( "Cinderford"  n ( s i1 n d @ f @ d ) ((pos "Nn$" ) ))
( "Cirencester"  n ( s ai1 r @ n s e s t @ r ) ((pos "Nn%" ) ))
( "Civvy Street"  n ( s i1 v i - s t r ii t ) ((pos "No$" ) ))
( "Clackmannon"  n ( k l a1 k m a1 n @ n ) ((pos "Nn$" ) ))
( "Clacton"  n ( k l a1 k t @ n ) ((pos "Nn%" ) ))
( "Clare"  n ( k l e@1 r ) ((pos "Nl%" ) ))
( "Clarkston"  n ( k l aa1 k s t @ n ) ((pos "Nn$" ) ))
( "Clarrie"  n ( k l a1 r ii ) ((pos "Nl$" ) ))
( "Classics"  n ( k l a1 s i k s ) ((pos "Lk%" ) ))
( "Claud"  n ( k l oo1 d ) ((pos "Nl%" ) ))
( "Claude"  n ( k l oo1 d ) ((pos "Nl%" ) ))
( "Clay Cross"  n ( k l ei2 - k r o1 s ) ((pos "Nn$" ) ))
( "Cleator Moor"  n ( k l ii2 t @ - m u@1 r ) ((pos "Nn$" ) ))
( "Cleethorpes"  n ( k l ii1 th oo p s ) ((pos "Nn%" ) ))
( "Clem"  n ( k l e1 m ) ((pos "Nl%" ) ))
( "Clement"  n ( k l e1 m @ n t ) ((pos "Nl%" ) ))
( "Clermont-Ferrand"  n ( k l e@2 m o ng - f e r o1 ng ) ((pos "Nn%" ) ))
( "Clevedon"  n ( k l ii1 v d @ n ) ((pos "Nn%" ) ))
( "Cleveland"  n ( k l ii1 v l @ n d ) ((pos "Nm%" ) ))
( "Cliff"  n ( k l i1 f ) ((pos "Nl%" ) ))
( "Clifford"  n ( k l i1 f @ d ) ((pos "Nl%" ) ))
( "Clitheroe"  n ( k l i1 dh @ r ou ) ((pos "Nn%" ) ))
( "Clive"  n ( k l ai1 v ) ((pos "Nl%" ) ))
( "Clowne"  n ( k l au1 n ) ((pos "Nn$" ) ))
( "Clwyd"  n ( k l uu1 i d ) ((pos "Nm%" ) ))
( "Clydach"  n ( k l i1 d a k ) ((pos "Nn$" ) ))
( "Clydebank"  n ( k l ai1 d b a ng k ) ((pos "Nn%" ) ))
( "Co"  n ( k ou1 ) ((pos "Y>%" ) ))
( "Co-op"  n ( k ou1 - o1 p ) ((pos "Y>%" ) ))
( "Coalville"  n ( k ou1 l v i l ) ((pos "Nn$" ) ))
( "Coatbridge"  n ( k ou1 t b r i jh ) ((pos "Nn$" ) ))
( "Cobham"  n ( k o1 b @ m ) ((pos "Nn%" ) ))
( "Coca-Cola"  n ( k ou2 k @ - k ou1 l @ ) ((pos "M6%" ) ))
( "Coca-Colas"  n ( k ou2 k @ - k ou1 l @ z ) ((pos "Mj%" ) ))
( "Cockermouth"  n ( k o1 k @ m @ th ) ((pos "Nn%" ) ))
( "Codsall"  n ( k o1 d s @ l ) ((pos "Nn$" ) ))
( "Coimbatore"  n ( k oo2 i m b @ t oo1 r ) ((pos "Nn$" ) ))
( "Col"  n ( k @@1 n l ) ((pos "Y:%" ) ))
( "Colchester"  n ( k o1 l ch e s t @ r ) ((pos "Nn%" ) ))
( "Coleshill"  n ( k ou1 l z h i l ) ((pos "Nn$" ) ))
( "Colin"  n ( k o1 l i n ) ((pos "Nl%" ) ))
( "Coll"  n ( k o1 l i jh ) ((pos "Y=%" ) ))
( "Colne"  n ( k ou1 n ) ((pos "Nn$" ) ))
( "Cologne"  n ( k @1 l ou1 n ) ((pos "Nn%" ) ))
( "Colombia"  n ( k @1 l o1 m b i@ ) ((pos "Nm%" ) ))
( "Colombian"  n ( k @1 l o1 m b i@ n ) ((pos "K6%" "OA%" ) ))
( "Colombians"  n ( k @1 l o1 m b i@ n z ) ((pos "Kj%" ) ))
( "Colombo"  n ( k @1 l o1 m b ou ) ((pos "Nn%" ) ))
( "Colorado"  n ( k o2 l @ r aa1 d ou ) ((pos "Nm%" ) ))
( "Columbia"  n ( k @1 l uh1 m b i@ ) ((pos "Nm%" ) ))
( "Columbus"  n ( k @1 l uh1 m b @ s ) ((pos "Nn%" ) ))
( "Colwyn Bay"  n ( k o2 l w i n - b ei1 ) ((pos "Nn%" ) ))
( "Confucian"  n ( k @1 n f y uu1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Confucians"  n ( k @1 n f y uu1 sh @ n z ) ((pos "Kj%" ) ))
( "Congleton"  n ( k o1 ng g l t @ n ) ((pos "Nn%" ) ))
( "Congo"  n ( k o1 ng g ou ) ((pos "Nm%" ) ))
( "Congolese"  n ( k o2 ng g @ l ii1 z ) ((pos "K9%" "OA%" ) ))
( "Congregational"  j ( k o2 ng g r i g ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "Connah's Quay"  n ( k o2 n @ z - k ii1 ) ((pos "Nn$" ) ))
( "Connaught"  n ( k o1 n oo t ) ((pos "Nm%" ) ))
( "Connecticut"  n ( k @1 n e1 t i k @ t ) ((pos "Nm%" ) ))
( "Connie"  n ( k o1 n ii ) ((pos "Nl%" ) ))
( "Cons"  n ( k o1 n s ) ((pos "Y~%" ) ))
( "Consett"  n ( k o1 n s i t ) ((pos "Nn$" ) ))
( "Constance"  n ( k o1 n s t @ n s ) ((pos "Nl%" ) ))
( "Conwy"  n ( k o1 n w ei ) ((pos "Nn%" ) ))
( "Cookham"  n ( k u1 k @ m ) ((pos "Nn%" ) ))
( "Copenhagen"  n ( k ou2 p @ n h ei1 g @ n ) ((pos "Nn%" ) ))
( "Copernican"  j ( k @1 p @@1 n i k @ n ) ((pos "OA%" ) ))
( "Coppull"  n ( k o1 p @ l ) ((pos "Nn$" ) ))
( "Copt"  n ( k o1 p t ) ((pos "K6$" "OA$" ) ))
( "Coptic"  n ( k o1 p t i k ) ((pos "K6%" ) ))
( "Coptics"  n ( k o1 p t i k s ) ((pos "Kj%" ) ))
( "Copts"  n ( k o1 p t s ) ((pos "Kj$" ) ))
( "Corby"  n ( k oo1 b ii ) ((pos "Nn%" ) ))
( "Cordoba"  n ( k oo1 d @ b @ ) ((pos "Nn%" ) ))
( "Corinthian"  n ( k @1 r i1 n th i@ n ) ((pos "K6%" "OA%" ) ))
( "Corinthians"  n ( k @1 r i1 n th i@ n z ) ((pos "Kj%" ) ))
( "Cork"  n ( k oo1 k ) ((pos "Nn%" ) ))
( "Cornwall"  n ( k oo1 n w oo l ) ((pos "Nm%" ) ))
( "Corp"  n ( k oo1 p ) ((pos "Y=%" ) ))
( "Corps Diplomatique"  n ( k oo2 - d i2 p l @ m a t ii1 k ) ((pos "Ki%" ) ))
( "Corsham"  n ( k oo1 sh @ m ) ((pos "Nn$" ) ))
( "Corunna"  n ( k @1 r uh1 n @ ) ((pos "Nn%" ) ))
( "Cosenza"  n ( k @1 s e1 n z @ ) ((pos "Nn$" ) ))
( "Costa Rica"  n ( k o2 s t @ - r ii1 k @ ) ((pos "Nm%" ) ))
( "Costa Rican"  n ( k o2 s t @ - r ii1 k @ n ) ((pos "K6%" "OA%" ) ))
( "Costa Ricans"  n ( k o2 s t @ - r ii1 k @ n z ) ((pos "Kj%" ) ))
( "Coventry"  n ( k o1 v n t r ii ) ((pos "Nn%" ) ))
( "Cowdenbeath"  n ( k au2 d @ n b ii1 th ) ((pos "Nn%" ) ))
( "Cowes"  n ( k au1 z ) ((pos "Nn%" ) ))
( "Cpl"  n ( k oo1 p r @ l ) ((pos "Y:%" ) ))
( "Cramlington"  n ( k r a1 m l i ng t @ n ) ((pos "Nn$" ) ))
( "Crawley"  n ( k r oo1 l ii ) ((pos "Nn%" ) ))
( "Creole"  n ( k r ii1 ou l ) ((pos "M6%" "OA%" ) ))
( "Creoles"  n ( k r ii1 ou l z ) ((pos "Mj$" ) ))
( "Cres"  n ( k r e1 z ) ((pos "Y=%" ) ))
( "Creswell"  n ( k r e1 s w i l ) ((pos "Nn$" ) ))
( "Crewe"  n ( k r uu1 ) ((pos "Nn%" ) ))
( "Crieff"  n ( k r ii1 f ) ((pos "Nn$" ) ))
( "Croesus"  n ( k r ii1 s @ s ) ((pos "Nl%" ) ))
( "Cromer"  n ( k r ou1 m @ r ) ((pos "Nn%" ) ))
( "Crook"  n ( k r u1 k ) ((pos "Nn$" ) ))
( "Crosby"  n ( k r o1 s b ii ) ((pos "Nn%" ) ))
( "Crowborough"  n ( k r ou1 b @ r @ ) ((pos "Nn%" ) ))
( "Crowthorne"  n ( k r ou1 th oo n ) ((pos "Nn%" ) ))
( "Croydon"  n ( k r oi1 d @ n ) ((pos "Nn%" ) ))
( "Cuba"  n ( k y uu1 b @ ) ((pos "Nm%" ) ))
( "Cuban"  n ( k y uu1 b @ n ) ((pos "K6%" "OA%" ) ))
( "Cubans"  n ( k y uu1 b @ n z ) ((pos "Kj%" ) ))
( "Cudworth"  n ( k uh1 d w @ th ) ((pos "Nn$" ) ))
( "Cuffley"  n ( k uh1 f l ii ) ((pos "Nn$" ) ))
( "Culcheth"  n ( k uh1 l ch @ th ) ((pos "Nn$" ) ))
( "Cumberland"  n ( k uh1 m b @ l @ n d ) ((pos "Nm%" ) ))
( "Cumbernauld"  n ( k uh1 m b @ n oo l d ) ((pos "Nn%" ) ))
( "Cumbria"  n ( k uh1 m b r i@ ) ((pos "Nm%" ) ))
( "Cumnock"  n ( k uh1 m n @ k ) ((pos "Nn$" ) ))
( "Cupar"  n ( k uu1 p @ r ) ((pos "Nn$" ) ))
( "Cupid"  n ( k y uu1 p i d ) ((pos "Nl%" ) ))
( "Curitiba"  n ( k u@2 r i t ii1 b @ ) ((pos "Nn$" ) ))
( "Currie"  n ( k uh1 r ii ) ((pos "Nn$" ) ))
( "Cwmbran"  n ( k uu1 m b r aa1 n ) ((pos "Nn%" ) ))
( "Cyclopean"  j ( s ai1 k l ou1 p i@ n ) ((pos "OA%" ) ))
( "Cynthia"  n ( s i1 n th i@ ) ((pos "Nl%" ) ))
( "Cyprian"  j ( s i1 p r i@ n ) ((pos "OA$" ) ))
( "Cypriot"  n ( s i1 p r i@ t ) ((pos "K6%" "OA%" ) ))
( "Cypriots"  n ( s i1 p r i@ t s ) ((pos "Kj%" ) ))
( "Cyprus"  n ( s ai1 p r @ s ) ((pos "Nm%" ) ))
( "Cyril"  n ( s i1 r @ l ) ((pos "Nl%" ) ))
( "Cyrillic"  j ( s i1 r i1 l i k ) ((pos "OA%" ) ))
( "Czech"  n ( ch e1 k ) ((pos "M6%" "OA%" ) ))
( "Czechoslovak"  n ( ch e2 k ou s l ou1 v a k ) ((pos "K6%" "OA%" ) ))
( "Czechoslovakia"  n ( ch e2 k ou s l @ v a1 k i@ ) ((pos "Nm%" ) ))
( "Czechoslovakian"  n ( ch e2 k ou s l @ v a1 k i@ n ) ((pos "K6%" "OA%" ) ))
( "Czechoslovakians"  n ( ch e2 k ou s l @ v a1 k i@ n z ) ((pos "Kj%" ) ))
( "Czechoslovaks"  n ( ch e2 k ou s l ou1 v a k s ) ((pos "Kj%" ) ))
( "Czechs"  n ( ch e1 k s ) ((pos "Mj%" ) ))
( "D"  n ( d ii1 ) ((pos "Ki$" ) ))
( "D's"  n ( d ii1 z ) ((pos "Kj$" ) ))
( "D-day"  n ( d ii1 - d ei ) ((pos "No%" ) ))
( "DDT"  n ( d ii2 d ii2 t ii1 ) ((pos "Y}%" ) ))
( "DES"  n ( d ii2 ii2 e1 s ) ((pos "Y>%" ) ))
( "DG"  n ( d ii2 jh ii1 ) ((pos "Y:$" ) ))
( "DIY"  n ( d ii2 ai2 w ai1 ) ((pos "Y}%" ) ))
( "DJ"  n ( d ii2 jh ei1 ) ((pos "Y>%" ) ))
( "DJs"  n ( d ii2 jh ei1 z ) ((pos "Y)%" ) ))
( "DLitt"  n ( d ii2 l i1 t ) ((pos "Y>$" ) ))
( "DM"  n ( d oi1 ch m aa k ) ((pos "Y]%" ) ))
( "DNA"  n ( d ii2 e2 n ei1 ) ((pos "Y}%" ) ))
( "DPhil"  n ( d ii2 f i1 l ) ((pos "Y>%" ) ))
( "DSS"  n ( d ii2 e2 s e1 s ) ((pos "Y>%" ) ))
( "DSc"  n ( d ii2 e2 s s ii1 ) ((pos "Y>%" ) ))
( "DTI"  n ( d ii2 t ii2 ai1 ) ((pos "Y>%" ) ))
( "DTs"  n ( d ii2 t ii1 z ) ((pos "Y)$" ) ))
( "Dacca"  n ( d a1 k @ ) ((pos "Nn%" ) ))
( "Dail Eireann"  n ( d oi2 l - e@1 r @ n ) ((pos "Ki%" ) ))
( "Daisy"  n ( d ei1 z ii ) ((pos "Nl%" ) ))
( "Dakar"  n ( d a1 k @ r ) ((pos "Nn$" ) ))
( "Dakota"  n ( d @1 k ou1 t @ ) ((pos "Nm%" ) ))
( "Dalkeith"  n ( d a1 l k ii1 th ) ((pos "Nn%" ) ))
( "Dallas"  n ( d a1 l @ s ) ((pos "Nn%" ) ))
( "Dalry"  n ( d a1 l r ai1 ) ((pos "Nn$" ) ))
( "Dalton"  n ( d o1 l t n ) ((pos "Nn$" ) ))
( "Damascus"  n ( d @1 m a1 s k @ s ) ((pos "Nn%" ) ))
( "Damocles"  n ( d a1 m @ k l ii z ) ((pos "Nl%" ) ))
( "Dan"  n ( d a1 n ) ((pos "Nl%" ) ))
( "Dane"  n ( d ei1 n ) ((pos "K6%" ) ))
( "Danes"  n ( d ei1 n z ) ((pos "Kj%" ) ))
( "Daniel"  n ( d aa1 n i@ l ) ((pos "K6%" "Nl%" ) ))
( "Daniels"  n ( d aa1 n i@ l z ) ((pos "Kj%" ) ))
( "Danish"  n ( d ei1 n i sh ) ((pos "L@%" "OA%" ) ))
( "Danny"  n ( d a1 n ii ) ((pos "Nl%" ) ))
( "Daphne"  n ( d a1 f n ii ) ((pos "Nl%" ) ))
( "Darby"  n ( d aa1 b ii ) ((pos "Nl$" ) ))
( "Darenth"  n ( d a1 r @ n th ) ((pos "Nn$" ) ))
( "Darfield"  n ( d aa1 f ii l d ) ((pos "Nn$" ) ))
( "Darkey"  n ( d aa1 k ii ) ((pos "K6$" ) ))
( "Darkeys"  n ( d aa1 k i z ) ((pos "Kj$" ) ))
( "Darkie"  n ( d aa1 k ii ) ((pos "K6%" ) ))
( "Darkies"  n ( d aa1 k i z ) ((pos "Kj%" ) ))
( "Darky"  n ( d aa1 k ii ) ((pos "K8%" ) ))
( "Darlington"  n ( d aa1 l i ng t @ n ) ((pos "Nn%" ) ))
( "Darmstadt"  n ( d aa1 m s t a t ) ((pos "Nn%" ) ))
( "Dartford"  n ( d aa1 t f @ d ) ((pos "Nn%" ) ))
( "Dartmouth"  n ( d aa1 t m @ th ) ((pos "Nn%" ) ))
( "Darwen"  n ( d aa1 w i n ) ((pos "Nn%" ) ))
( "Dave"  n ( d ei1 v ) ((pos "Nl%" ) ))
( "Daventry"  n ( d a1 v i n t r ii ) ((pos "Nn%" ) ))
( "Davey"  n ( d ei1 v ii ) ((pos "Nl%" ) ))
( "David"  n ( d ei1 v i d ) ((pos "Nl%" ) ))
( "Dawley"  n ( d oo1 l ii ) ((pos "Nn$" ) ))
( "Dawlish"  n ( d oo1 l i sh ) ((pos "Nn%" ) ))
( "Dawn"  n ( d oo1 n ) ((pos "Nl%" ) ))
( "Dayton"  n ( d ei1 t n ) ((pos "Nn$" ) ))
( "Deal"  n ( d ii1 l ) ((pos "Nn$" ) ))
( "Dean"  n ( d ii1 n ) ((pos "Nl%" ) ))
( "Debbie"  n ( d e1 b ii ) ((pos "Nl%" ) ))
( "Debby"  n ( d e1 b ii ) ((pos "Nl%" ) ))
( "Deborah"  n ( d e1 b @ r @ ) ((pos "Nl%" ) ))
( "Dec"  n ( d e1 k ) ((pos "Y=%" ) ))
( "Decalogue"  n ( d e1 k @ l o g ) ((pos "K6%" ) ))
( "Decalogues"  n ( d e1 k @ l o g z ) ((pos "Kj$" ) ))
( "December"  n ( d i1 s e1 m b @ r ) ((pos "K6%" "No%" ) ))
( "Decembers"  n ( d i1 s e1 m b @ z ) ((pos "Kj$" ) ))
( "Deirdre"  n ( d i@1 d r ii ) ((pos "Nl%" ) ))
( "Delaware"  n ( d e1 l @ w e@ r ) ((pos "Nm%" ) ))
( "Delhi"  n ( d e1 l ii ) ((pos "Nn%" ) ))
( "Delphic"  j ( d e1 l f i k ) ((pos "OA%" ) ))
( "Denain"  n ( d @1 n ei1 n ) ((pos "Nn$" ) ))
( "Denbigh"  n ( d e1 n b ii ) ((pos "Nn$" ) ))
( "Denis"  n ( d e1 n i s ) ((pos "Nl%" ) ))
( "Denise"  n ( d @1 n ii1 z ) ((pos "Nl%" ) ))
( "Denmark"  n ( d e1 n m aa k ) ((pos "Nm%" ) ))
( "Dennis"  n ( d e1 n i s ) ((pos "Nl%" ) ))
( "Denny"  n ( d e1 n ii ) ((pos "Nn%" ) ))
( "Denver"  n ( d e1 n v @ r ) ((pos "Nn%" ) ))
( "Dept"  n ( d i1 p aa1 t m @ n t ) ((pos "Y=%" ) ))
( "Derby"  n ( d aa1 b ii ) ((pos "Nn%" ) ))
( "Derbyshire"  n ( d aa1 b i sh @ r ) ((pos "Nm%" ) ))
( "Dereham"  n ( d i@1 r @ m ) ((pos "Nn$" ) ))
( "Derek"  n ( d e1 r i k ) ((pos "Nl%" ) ))
( "Des"  n ( d e1 z ) ((pos "Nl%" ) ))
( "Desmond"  n ( d e1 z m @ n d ) ((pos "Nl%" ) ))
( "Dessau"  n ( d e1 s au ) ((pos "Nn$" ) ))
( "Detroit"  n ( d i1 t r oi1 t ) ((pos "Nn%" ) ))
( "Deutschmark"  n ( d oi1 ch m aa k ) ((pos "K6%" ) ))
( "Deutschmarks"  n ( d oi1 ch m aa k s ) ((pos "Kj%" ) ))
( "Devizes"  n ( d i1 v ai1 z i z ) ((pos "Nn%" ) ))
( "Devon"  n ( d e1 v n ) ((pos "Nm%" ) ))
( "Dewsbury"  n ( d y uu1 z b @ r ii ) ((pos "Nn%" ) ))
( "Di"  n ( d ai1 ) ((pos "Nl%" ) ))
( "Diana"  n ( d ai1 a1 n @ ) ((pos "Nl%" ) ))
( "Diaspora"  n ( d ai1 a1 s p @ r @ ) ((pos "K6%" ) ))
( "Diasporas"  n ( d ai1 a1 s p @ r @ z ) ((pos "Kj$" ) ))
( "Dick"  n ( d i1 k ) ((pos "Nl%" ) ))
( "Dicky"  n ( d i1 k ii ) ((pos "Nl%" ) ))
( "Dictaphone"  n ( d i1 k t @ f ou n ) ((pos "K6%" ) ))
( "Dictaphones"  n ( d i1 k t @ f ou n z ) ((pos "Kj$" ) ))
( "Didcot"  n ( d i1 d k @ t ) ((pos "Nn%" ) ))
( "Dijon"  n ( d ii1 zh o ng ) ((pos "Nn%" ) ))
( "Dingle"  n ( d i1 ng g l ) ((pos "Nn%" ) ))
( "Dinnington"  n ( d i1 n i ng t @ n ) ((pos "Nn$" ) ))
( "Dip"  n ( d i1 p ) ((pos "Y>%" ) ))
( "Dip Ed"  n ( d i2 p - e1 d ) ((pos "Y>%" ) ))
( "Dir"  n ( d i1 r e1 k t @ r ) ((pos "Y>%" ) ))
( "Dives"  n ( d ai1 v ii z ) ((pos "Nl$" ) ))
( "Djibouti"  n ( jh i1 b uu1 t ii ) ((pos "Nm%" ) ))
( "Djiboutian"  n ( jh i1 b uu1 t i@ n ) ((pos "K6$" "OA$" ) ))
( "Djiboutians"  n ( jh i1 b uu1 t i@ n z ) ((pos "Kj$" ) ))
( "Dnepropetrovsk"  n ( n i1 p r o2 p i t r o1 f s k ) ((pos "Nn$" ) ))
( "DoE"  n ( d ii2 ou2 ii1 ) ((pos "Y>%" ) ))
( "Dolly"  n ( d o1 l ii ) ((pos "Nl%" ) ))
( "Domesday"  n ( d uu1 m z d ei ) ((pos "No%" ) ))
( "Dominic"  n ( d o1 m i n ii k ) ((pos "Nl%" ) ))
( "Dominica"  n ( d @1 m i1 n i k @ ) ((pos "Nm%" ) ))
( "Dominican"  n ( d @1 m i1 n i k @ n ) ((pos "K6%" "OA%" ) ))
( "Dominicans"  n ( d @1 m i1 n i k @ n z ) ((pos "Kj%" ) ))
( "Don"  n ( d o1 n ) ((pos "Nl%" ) ))
( "Donald"  n ( d o1 n l d ) ((pos "Nl%" ) ))
( "Doncaster"  n ( d o1 ng k a s t @ r ) ((pos "Nn%" ) ))
( "Donegal"  n ( d o1 n i g oo l ) ((pos "Nm%" ) ))
( "Donetsk"  n ( d @1 n e1 t s k ) ((pos "Nn$" ) ))
( "Doomsday"  n ( d uu1 m z d ei ) ((pos "No%" ) ))
( "Dora"  n ( d oo1 r @ ) ((pos "Nl%" ) ))
( "Dorchester"  n ( d oo1 ch i s t @ r ) ((pos "Nn%" ) ))
( "Dordrecht"  n ( d oo1 d r i k t ) ((pos "Nn$" ) ))
( "Doreen"  n ( d oo1 r ii n ) ((pos "Nl%" ) ))
( "Doric"  j ( d o1 r i k ) ((pos "OA$" ) ))
( "Doris"  n ( d o1 r i s ) ((pos "Nl%" ) ))
( "Dorking"  n ( d oo1 k i ng ) ((pos "Nn%" ) ))
( "Dorothy"  n ( d o1 r @ th ii ) ((pos "Nl%" ) ))
( "Dorset"  n ( d oo1 s i t ) ((pos "Nm%" ) ))
( "Dortmund"  n ( d oo1 t m @ n d ) ((pos "Nn%" ) ))
( "Douai"  n ( d uu1 ei ) ((pos "Nn$" ) ))
( "Doug"  n ( d uh1 g ) ((pos "Nl%" ) ))
( "Douglas"  n ( d uh1 g l @ s ) ((pos "Nl%" ) ))
( "Dover"  n ( d ou1 v @ r ) ((pos "Nn%" ) ))
( "Down"  n ( d au1 n ) ((pos "Nm$" ) ))
( "Downing Street"  n ( d au1 n i ng - s t r ii t ) ((pos "No%" ) ))
( "Dr"  n ( d o1 k t @ r ) ((pos "Y:%" ) ))
( "Drayton"  n ( d r ei1 t n ) ((pos "Nn$" ) ))
( "Dresden"  n ( d r e1 z d @ n ) ((pos "Nn%" ) ))
( "Driffield"  n ( d r i1 f ii l d ) ((pos "Nn$" ) ))
( "Drogheda"  n ( d r oi1 i d @ ) ((pos "Nn%" ) ))
( "Droitwich"  n ( d r oi1 t w i ch ) ((pos "Nn%" ) ))
( "Dronfield"  n ( d r o1 n f ii l d ) ((pos "Nn$" ) ))
( "Druid"  n ( d r uu1 i d ) ((pos "K6%" ) ))
( "Druids"  n ( d r uu1 i d z ) ((pos "Kj%" ) ))
( "Dublin"  n ( d uh1 b l i n ) ((pos "Nn%" ) ))
( "Dubliner"  n ( d uh1 b l i n @ r ) ((pos "K6%" ) ))
( "Dubliners"  n ( d uh1 b l i n @ z ) ((pos "Kj%" ) ))
( "Duce"  n ( d uu1 ch ei ) ((pos "Ki$" ) ))
( "Dudley"  n ( d uh1 d l ii ) ((pos "Nn%" ) ))
( "Duisburg"  n ( d uu1 s b @@ g ) ((pos "Nn$" ) ))
( "Dukinfield"  n ( d uh1 k i n f ii l d ) ((pos "Nn$" ) ))
( "Dumbarton"  n ( d uh1 m b aa1 t n ) ((pos "Nn%" ) ))
( "Dumfries"  n ( d uh1 m f r ii1 s ) ((pos "Nn%" ) ))
( "Dun Laoghaire"  n ( d uh1 n - l i@1 r ii ) ((pos "Nn%" ) ))
( "Duncan"  n ( d uh1 n k @ n ) ((pos "Nl%" ) ))
( "Dundalk"  n ( d uh1 n d oo1 k ) ((pos "Nn$" ) ))
( "Dundee"  n ( d uh1 n d ii1 ) ((pos "Nn%" ) ))
( "Dunfermline"  n ( d uh1 n f @@1 m l i n ) ((pos "Nn%" ) ))
( "Dunkirk"  n ( d uh1 n k @@1 k ) ((pos "Nn%" ) ))
( "Dunoon"  n ( d uh1 n uu1 n ) ((pos "Nn$" ) ))
( "Dunstable"  n ( d uh1 n s t @ b l ) ((pos "Nn%" ) ))
( "Durban"  n ( d @@1 b @ n ) ((pos "Nn%" ) ))
( "Durham"  n ( d uh1 r @ m ) ((pos "Nn%" ) ))
( "Durrington"  n ( d uh1 r i ng t @ n ) ((pos "Nn$" ) ))
( "Dursley"  n ( d @@1 z l ii ) ((pos "Nn$" ) ))
( "Dusseldorf"  n ( d uh1 s l d oo f ) ((pos "Nn%" ) ))
( "Dutch"  n ( d uh1 ch ) ((pos "L@%" "OA%" ) ))
( "Dutchman"  n ( d uh1 ch m @ n ) ((pos "Ki%" ) ))
( "Dutchmen"  n ( d uh1 ch m @ n ) ((pos "Kj%" ) ))
( "Dyfed"  n ( d uh1 v i d ) ((pos "Nm%" ) ))
( "E"  n ( ii1 ) ((pos "Ki$" ) ))
( "E's"  n ( ii1 z ) ((pos "Kj$" ) ))
( "EC"  n ( ii2 s ii1 ) ((pos "Y>%" ) ))
( "EDP"  n ( ii2 d ii2 p ii1 ) ((pos "Y}$" ) ))
( "EEC"  n ( ii2 ii2 s ii1 ) ((pos "Y>%" ) ))
( "EEG"  n ( ii2 ii2 jh ii1 ) ((pos "Y>%" ) ))
( "EFTA"  n ( e1 f t @ ) ((pos "Y>%" ) ))
( "ESP"  n ( ii2 e2 s p ii1 ) ((pos "Y}%" ) ))
( "Eaglescliffe"  n ( ii1 g l z k l i f ) ((pos "Nn$" ) ))
( "Ealing"  n ( ii1 l i ng ) ((pos "Nn%" ) ))
( "Earl Shilton"  n ( @@2 l - sh i1 l t n ) ((pos "Nn$" ) ))
( "Easington"  n ( ii1 z i ng t @ n ) ((pos "Nn$" ) ))
( "East Dereham"  n ( ii2 s t - d i@1 r @ m ) ((pos "Nn$" ) ))
( "East Grinstead"  n ( ii2 s t - g r i1 n s t i d ) ((pos "Nn%" ) ))
( "East Kilbride"  n ( ii2 s t - k i l b r ai1 d ) ((pos "Nn%" ) ))
( "East Retford"  n ( ii2 s t - r e1 t f @ d ) ((pos "Nn$" ) ))
( "Eastbourne"  n ( ii1 s b oo n ) ((pos "Nn%" ) ))
( "Easter"  n ( ii1 s t @ r ) ((pos "K6%" "No%" ) ))
( "Easters"  n ( ii1 s t @ z ) ((pos "Kj$" ) ))
( "Eastleigh"  n ( ii1 s t l ii ) ((pos "Nn$" ) ))
( "Ebbw Vale"  n ( e2 b u - v ei1 l ) ((pos "Nn%" ) ))
( "Eccles"  n ( e1 k l z ) ((pos "Nn%" ) ))
( "Ecuador"  n ( e1 k w @ d oo r ) ((pos "Nm%" ) ))
( "Ecuadorian"  n ( e2 k w @ d oo1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Ecuadorians"  n ( e2 k w @ d oo1 r i@ n z ) ((pos "Kj%" ) ))
( "Ed"  n ( e1 d ) ((pos "Nl%" ) ))
( "Eddie"  n ( e1 d ii ) ((pos "Nl%" ) ))
( "Eddy"  n ( e1 d ii ) ((pos "Nl%" ) ))
( "Eden"  n ( ii1 d n ) ((pos "No%" ) ))
( "Edenbridge"  n ( ii1 d n b r i jh ) ((pos "Nn%" ) ))
( "Edgar"  n ( e1 d g @ r ) ((pos "Nl%" ) ))
( "Edinburgh"  n ( e1 d i n b r @ ) ((pos "Nn%" ) ))
( "Edith"  n ( ii1 d i th ) ((pos "Nl%" ) ))
( "Edmund"  n ( e1 d m @ n d ) ((pos "Nl%" ) ))
( "Edward"  n ( e1 d w @ d ) ((pos "Nl%" ) ))
( "Edwardian"  n ( i1 d w oo1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Edwardians"  n ( i1 d w oo1 d i@ n z ) ((pos "Kj%" ) ))
( "Egremont"  n ( e1 g r i m o n t ) ((pos "Nn$" ) ))
( "Egypt"  n ( ii1 jh i p t ) ((pos "Nm%" ) ))
( "Egyptian"  n ( i1 jh i1 p sh @ n ) ((pos "K6%" "OA%" ) ))
( "Egyptians"  n ( i1 jh i1 p sh @ n z ) ((pos "Kj%" ) ))
( "Eiche"  n ( ai1 k @ ) ((pos "Nn$" ) ))
( "Eileen"  n ( ai1 l ii n ) ((pos "Nl%" ) ))
( "Eindhoven"  n ( ai1 n d h o v n ) ((pos "Nn%" ) ))
( "El Dorado"  n ( e2 l - d @ r aa1 d ou ) ((pos "Ki%" ) ))
( "El Dorados"  n ( e2 l - d @ r aa1 d ou z ) ((pos "Kj$" ) ))
( "El Salvador"  n ( e1 l - s a1 l v @ d oo r ) ((pos "Nm%" ) ))
( "Elaine"  n ( i1 l ei1 n ) ((pos "Nl%" ) ))
( "Elastoplast"  n ( i1 l a1 s t @ p l aa2 s t ) ((pos "L@%" ) ))
( "Elderslie"  n ( e1 l d @ z l ii ) ((pos "Nn$" ) ))
( "Eleanor"  n ( e1 l @ n @ r ) ((pos "Nl%" ) ))
( "Elgin"  n ( e1 l g i n ) ((pos "Nn$" ) ))
( "Eliza"  n ( i1 l ai1 z @ ) ((pos "Nl%" ) ))
( "Elizabeth"  n ( i1 l i1 z @ b @ th ) ((pos "Nl%" ) ))
( "Elizabethan"  n ( i1 l i2 z @ b ii1 th n ) ((pos "K6%" "OA%" ) ))
( "Elizabethans"  n ( i1 l i2 z @ b ii1 th n z ) ((pos "Kj%" ) ))
( "Elland"  n ( e1 l @ n d ) ((pos "Nn$" ) ))
( "Ellen"  n ( e1 l @ n ) ((pos "Nl%" ) ))
( "Ellesmere"  n ( e1 l z m i@ r ) ((pos "Nn%" ) ))
( "Ellesmere Port"  n ( e2 l z m i@ - p oo1 t ) ((pos "Nn%" ) ))
( "Ellie"  n ( e1 l ii ) ((pos "Nl%" ) ))
( "Elloughton"  n ( e1 l @ t @ n ) ((pos "Nn$" ) ))
( "Elsie"  n ( e1 l s ii ) ((pos "Nl%" ) ))
( "Elstree"  n ( e1 l s t r ii ) ((pos "Nn%" ) ))
( "Ely"  n ( ii1 l ii ) ((pos "Nn%" ) ))
( "Elysian"  j ( i1 l i1 z i@ n ) ((pos "OA%" ) ))
( "Elysium"  n ( i1 l i1 z i@ m ) ((pos "No%" ) ))
( "Emily"  n ( e1 m l ii ) ((pos "Nl%" ) ))
( "Emma"  n ( e1 m @ ) ((pos "Nl%" ) ))
( "Emsworth"  n ( e1 m z w @ th ) ((pos "Nn$" ) ))
( "Enfield"  n ( e1 n f ii l d ) ((pos "Nn%" ) ))
( "England"  n ( i1 ng g l @ n d ) ((pos "Nm%" ) ))
( "English"  n ( i1 ng g l i sh ) ((pos "L@*" "OA*" ) ))
( "Englishman"  n ( i1 ng g l i sh m @ n ) ((pos "Ki%" ) ))
( "Englishmen"  n ( i1 ng g l i sh m @ n ) ((pos "Kj%" ) ))
( "Englishwoman"  n ( i1 ng g l i sh w u m @ n ) ((pos "Ki%" ) ))
( "Englishwomen"  n ( i1 ng g l i sh w i m i n ) ((pos "Kj%" ) ))
( "Enoch"  n ( ii1 n o k ) ((pos "Nl%" ) ))
( "Enschede"  n ( e1 n s k @ d ii ) ((pos "Nn$" ) ))
( "Epiphany"  n ( i1 p i1 f @ n ii ) ((pos "No%" ) ))
( "Epping"  n ( e1 p i ng ) ((pos "Nn%" ) ))
( "Epsom"  n ( e1 p s @ m ) ((pos "Nn%" ) ))
( "Erfurt"  n ( e@1 f @@ t ) ((pos "Nn$" ) ))
( "Eric"  n ( e1 r ii k ) ((pos "Nl%" ) ))
( "Erica"  n ( e1 r i k @ ) ((pos "Nl%" ) ))
( "Erin"  n ( e1 r i n ) ((pos "Nm%" ) ))
( "Eritrea"  n ( e2 r i t r ei1 @ ) ((pos "Nm%" ) ))
( "Eritrean"  n ( e2 r i t r ei1 @ n ) ((pos "K6%" "OA%" ) ))
( "Eritreans"  n ( e2 r i t r ei1 @ n z ) ((pos "Kj%" ) ))
( "Erlangen"  n ( @@1 l a ng @ n ) ((pos "Nn$" ) ))
( "Ernest"  n ( @@1 n i s t ) ((pos "Nl%" ) ))
( "Ernie"  n ( @@1 n ii ) ((pos "Nl%" ) ))
( "Erse"  n ( @@1 s ) ((pos "L@$" ) ))
( "Esfahan"  n ( i2 s f @ h aa1 n ) ((pos "Nn$" ) ))
( "Eskimo"  n ( e1 s k i m ou ) ((pos "K6%" ) ))
( "Eskimoes"  n ( e1 s k i m ou z ) ((pos "Kj%" ) ))
( "Eskimos"  n ( e1 s k i m ou z ) ((pos "Kj%" ) ))
( "Esperanto"  n ( e2 s p @ r a1 n t ou ) ((pos "L@%" ) ))
( "Esq"  n ( e1 s k ) ((pos "Y:%" ) ))
( "Esquire"  n ( i1 s k w ai1 @ r ) ((pos "K6%" ) ))
( "Esquires"  n ( i1 s k w ai1 @ z ) ((pos "Kj%" ) ))
( "Essen"  n ( e1 s @ n ) ((pos "Nn$" ) ))
( "Essex"  n ( e1 s i k s ) ((pos "Nm%" ) ))
( "Esther"  n ( e1 s t @ r ) ((pos "Nl%" ) ))
( "Ethel"  n ( e1 th l ) ((pos "Nl%" ) ))
( "Ethiopia"  n ( ii2 th i ou1 p i@ ) ((pos "Nm%" ) ))
( "Ethiopian"  n ( ii2 th i ou1 p i@ n ) ((pos "K6%" "OA%" ) ))
( "Ethiopians"  n ( ii2 th i ou1 p i@ n z ) ((pos "Kj%" ) ))
( "Eucharist"  n ( y uu1 k @ r i s t ) ((pos "K6%" ) ))
( "Eucharists"  n ( y uu1 k @ r i s t s ) ((pos "Kj$" ) ))
( "Euclidean"  j ( y uu1 k l i1 d i@ n ) ((pos "OA%" ) ))
( "Eugene"  n ( y uu1 jh ii1 n ) ((pos "Nl%" ) ))
( "Eunice"  n ( y uu1 n i s ) ((pos "Nl%" ) ))
( "Eurasia"  n ( y u@1 r ei1 zh @ ) ((pos "Nm$" ) ))
( "Eurasian"  n ( y u@1 r ei1 zh n ) ((pos "K6%" "OA%" ) ))
( "Eurasians"  n ( y u@1 r ei1 zh n z ) ((pos "Kj%" ) ))
( "Eurodollar"  n ( y u@1 r ou d o l @ r ) ((pos "K6%" ) ))
( "Eurodollars"  n ( y u@1 r ou d o l @ z ) ((pos "Kj%" ) ))
( "Europe"  n ( y u@1 r @ p ) ((pos "Nm%" ) ))
( "European"  n ( y u@2 r @ p i@1 n ) ((pos "K6%" "OA%" ) ))
( "Europeans"  n ( y u@2 r @ p i@1 n z ) ((pos "Kj%" ) ))
( "Eurovision"  n ( y u@1 r @ v i zh n ) ((pos "No%" ) ))
( "Eustachian"  j ( y uu1 s t ei1 sh @ n ) ((pos "Oq%" ) ))
( "Eva"  n ( ii1 v @ ) ((pos "Nl%" ) ))
( "Eve"  n ( ii1 v ) ((pos "Nl%" ) ))
( "Evelyn"  n ( ii1 v l i n ) ((pos "Nl%" ) ))
( "Evesham"  n ( ii1 v sh @ m ) ((pos "Nn%" ) ))
( "Ewell"  n ( y uu1 @ l ) ((pos "Nn$" ) ))
( "Excellencies"  n ( e1 k s @ l @ n s i z ) ((pos "Kj%" ) ))
( "Excellency"  n ( e1 k s @ l @ n s ii ) ((pos "K8%" ) ))
( "Exeter"  n ( e1 k s i t @ r ) ((pos "Nn%" ) ))
( "Exmouth"  n ( e1 k s m @ th ) ((pos "Nn%" ) ))
( "F"  n ( e1 f ) ((pos "Ki$" ) ))
( "F's"  n ( e1 f s ) ((pos "Kj$" ) ))
( "FA"  n ( e2 f ei1 ) ((pos "Y>%" ) ))
( "FAO"  n ( e2 f ei2 ou1 ) ((pos "Y>%" ) ))
( "FBI"  n ( e2 f b i ai1 ) ((pos "Y>%" ) ))
( "FM"  n ( e2 f e1 m ) ((pos "Y}%" ) ))
( "FO"  n ( e2 f ou1 ) ((pos "Y>%" ) ))
( "FRS"  n ( e2 f aa r e1 s ) ((pos "Y>%" ) ))
( "Fabian"  n ( f ei1 b i@ n ) ((pos "K6%" "OA%" ) ))
( "Fabians"  n ( f ei1 b i@ n z ) ((pos "Kj%" ) ))
( "Fahrenheit"  nil ( f a1 r @ n h ai t ) ((pos "Z-%" ) ))
( "Falkirk"  n ( f oo1 l k @@ k ) ((pos "Nn%" ) ))
( "Fallopian"  j ( f @1 l ou1 p i@ n ) ((pos "Oq%" ) ))
( "Falmouth"  n ( f a1 l m @ th ) ((pos "Nn%" ) ))
( "Fanny"  n ( f a1 n ii ) ((pos "Nl%" ) ))
( "Farnham"  n ( f aa1 n @ m ) ((pos "Nn%" ) ))
( "Farnworth"  n ( f aa1 n w @ th ) ((pos "Nn$" ) ))
( "Farsi"  n ( f aa1 s ii ) ((pos "L@$" ) ))
( "Fauldhouse"  n ( f oo1 l d h au s ) ((pos "Nn$" ) ))
( "Faversham"  n ( f a1 v @ sh @ m ) ((pos "Nn%" ) ))
( "Fawley"  n ( f oo1 l ii ) ((pos "Nn%" ) ))
( "Featherstone"  n ( f e1 dh @ s t @ n ) ((pos "Nn%" ) ))
( "Feb"  n ( f e1 b ) ((pos "Y=%" ) ))
( "Februaries"  n ( f e1 b y u@ r i z ) ((pos "Kj$" ) ))
( "February"  n ( f e1 b y u@ r ii ) ((pos "K8%" "No%" ) ))
( "Fed"  n ( f e1 d ) ((pos "Y>%" ) ))
( "Felicity"  n ( f @1 l i1 s i t ii ) ((pos "Nl%" ) ))
( "Felix"  n ( f ii1 l i k s ) ((pos "Nl%" ) ))
( "Felixstowe"  n ( f ii1 l i k s t ou ) ((pos "Nn%" ) ))
( "Fermanagh"  n ( f @1 m a1 n @ ) ((pos "Nm%" ) ))
( "Ferrara"  n ( f @1 r aa1 r @ ) ((pos "Nn$" ) ))
( "Ferryhill"  n ( f e1 r i h i l ) ((pos "Nn$" ) ))
( "Fife"  n ( f ai1 f ) ((pos "Nm%" ) ))
( "Fiji"  n ( f ii2 jh ii1 ) ((pos "Nm%" ) ))
( "Fijian"  n ( f ii2 jh i@1 n ) ((pos "K6%" "OA%" ) ))
( "Fijians"  n ( f ii2 jh i@1 n z ) ((pos "Kj%" ) ))
( "Filipino"  n ( f i2 l i p ii1 n ou ) ((pos "K6%" "OA%" ) ))
( "Filipinos"  n ( f i2 l i p ii1 n ou z ) ((pos "Kj%" ) ))
( "Finland"  n ( f i1 n l @ n d ) ((pos "Nm%" ) ))
( "Finn"  n ( f i1 n ) ((pos "K6%" ) ))
( "Finnish"  n ( f i1 n i sh ) ((pos "L@%" "OA%" ) ))
( "Finns"  n ( f i1 n z ) ((pos "Kj%" ) ))
( "Fiona"  n ( f i1 ou1 n @ ) ((pos "Nl%" ) ))
( "Firenze"  n ( f i1 r e1 n z ii ) ((pos "Nn%" ) ))
( "Fleet"  n ( f l ii1 t ) ((pos "Nn%" ) ))
( "Fleet Street"  n ( f l ii1 t - s t r ii t ) ((pos "No%" ) ))
( "Fleetwood"  n ( f l ii1 t w u d ) ((pos "Nn%" ) ))
( "Flemish"  n ( f l e1 m i sh ) ((pos "L@%" "OA%" ) ))
( "Flint"  n ( f l i1 n t ) ((pos "Nn$" ) ))
( "Flora"  n ( f l oo1 r @ ) ((pos "Nl%" ) ))
( "Florence"  n ( f l o1 r @ n s ) ((pos "Nl%" ) ))
( "Florida"  n ( f l o1 r i d @ ) ((pos "Nm%" ) ))
( "Florrie"  n ( f l oo1 r ii ) ((pos "Nl%" ) ))
( "Foggia"  n ( f o1 jh i@ ) ((pos "Nn$" ) ))
( "Folkestone"  n ( f ou1 k s t @ n ) ((pos "Nn%" ) ))
( "Foochow"  n ( f uu2 ch au1 ) ((pos "Nn$" ) ))
( "Forfar"  n ( f oo1 f @ r ) ((pos "Nn%" ) ))
( "Forli"  n ( f oo1 l ii ) ((pos "Nn$" ) ))
( "Formby"  n ( f oo1 m b ii ) ((pos "Nn%" ) ))
( "Formica"  n ( f oo1 m ai1 k @ ) ((pos "L@%" ) ))
( "Formosa"  n ( f oo1 m ou1 s @ ) ((pos "Nm%" ) ))
( "Fort Lauderdale"  n ( f oo2 t - l oo1 d @ d ei l ) ((pos "Nn%" ) ))
( "Fort William"  n ( f oo2 t - w i1 l i@ m ) ((pos "Nn%" ) ))
( "Fort Worth"  n ( f oo2 t - w @@1 th ) ((pos "Nn%" ) ))
( "Fortaleza"  n ( f oo2 t @ l ei1 z @ ) ((pos "Nn$" ) ))
( "Fowey"  n ( f oi1 ) ((pos "Nn%" ) ))
( "Fr"  n ( f aa1 dh @ r ) ((pos "Y:%" ) ))
( "Fr\"aulein"  n ( f r oi1 l ai n ) ((pos "K6%" ) ))
( "Fr\"auleins"  n ( f r oi1 l ai n z ) ((pos "Kj%" ) ))
( "Frampton Cotterell"  n ( f r a2 m p t @ n - k o1 t r i l ) ((pos "Nn$" ) ))
( "Fran"  n ( f r a1 n ) ((pos "Nl%" ) ))
( "France"  n ( f r aa1 n s ) ((pos "Nm%" ) ))
( "Frances"  n ( f r aa1 n s i s ) ((pos "Nl%" ) ))
( "Francis"  n ( f r aa1 n s i s ) ((pos "Nl%" ) ))
( "Franciscan"  n ( f r a1 n s i1 s k @ n ) ((pos "K6%" "OA%" ) ))
( "Franciscans"  n ( f r a1 n s i1 s k @ n z ) ((pos "Kj%" ) ))
( "Franco-"  j ( f r a1 ng k ou - ) ((pos "U-%" ) ))
( "Frank"  n ( f r a1 ng k ) ((pos "K6%" "Nl%" ) ))
( "Frankfurt"  n ( f r a1 n k f @ t ) ((pos "Nn%" ) ))
( "Frankie"  n ( f r a1 n k ii ) ((pos "Nl%" ) ))
( "Franks"  n ( f r a1 ng k s ) ((pos "Kj%" ) ))
( "Fraserburgh"  n ( f r ei1 z @ b @ r @ ) ((pos "Nn$" ) ))
( "Frau"  n ( f r au1 ) ((pos "Ki%" ) ))
( "Frauen"  n ( f r au1 @ n ) ((pos "Kj$" ) ))
( "Freckleton"  n ( f r e1 k l t @ n ) ((pos "Nn$" ) ))
( "Fred"  n ( f r e1 d ) ((pos "Nl%" ) ))
( "Freda"  n ( f r ii1 d @ ) ((pos "Nl%" ) ))
( "Freddie"  n ( f r e1 d ii ) ((pos "Nl%" ) ))
( "Freddy"  n ( f r e1 d ii ) ((pos "Nl%" ) ))
( "Frederick"  n ( f r e1 d r i k ) ((pos "Nl%" ) ))
( "Frederiksberg"  n ( f r e1 d r i k s b @@ g ) ((pos "Nn%" ) ))
( "Freemason"  n ( f r ii1 m ei s n ) ((pos "K6%" ) ))
( "Freemasonry"  n ( f r ii1 m ei s @ n r ii ) ((pos "L@%" ) ))
( "Freemasons"  n ( f r ii1 m ei s n z ) ((pos "Kj%" ) ))
( "Freiburg"  n ( f r ai1 b @@ g ) ((pos "Nn%" ) ))
( "French"  n ( f r e1 n ch ) ((pos "L@%" "OA%" ) ))
( "Frenchman"  n ( f r e1 n ch m @ n ) ((pos "Ki%" ) ))
( "Frenchmen"  n ( f r e1 n ch m @ n ) ((pos "Kj%" ) ))
( "Frenchwoman"  n ( f r e1 n ch w u m @ n ) ((pos "Ki%" ) ))
( "Frenchwomen"  n ( f r e1 n ch w i m i n ) ((pos "Kj%" ) ))
( "Freshwater"  n ( f r e1 sh w oo t @ r ) ((pos "Nn$" ) ))
( "Freudian"  j ( f r oi1 d i@ n ) ((pos "OA%" ) ))
( "Fri"  n ( f r ai1 d ei ) ((pos "Y=%" ) ))
( "Friday"  n ( f r ai1 d ii ) ((pos "K6%" "No%" ) ))
( "Fridays"  n ( f r ai1 d i z ) ((pos "Kj%" ) ))
( "Frinton"  n ( f r i1 n t n ) ((pos "Nn$" ) ))
( "Frodsham"  n ( f r o1 d sh @ m ) ((pos "Nn$" ) ))
( "Frome"  n ( f r uu1 m ) ((pos "Nn$" ) ))
( "Furth"  n ( f @@1 th ) ((pos "Nn$" ) ))
( "Fushun"  n ( f uu2 sh u1 n ) ((pos "Nn$" ) ))
( "G"  n ( jh ii1 ) ((pos "Ki$" ) ))
( "G's"  n ( jh ii1 z ) ((pos "Kj$" ) ))
( "G-man"  n ( jh ii1 - m a n ) ((pos "Ki$" ) ))
( "G-men"  n ( jh ii1 - m e n ) ((pos "Kj$" ) ))
( "GATT"  n ( g a1 t ) ((pos "Y>%" ) ))
( "GB"  n ( jh ii2 b ii1 ) ((pos "Y=%" ) ))
( "GCSE"  n ( jh ii2 s i e s ii1 ) ((pos "Y>%" ) ))
( "GCSEs"  n ( jh ii2 s i e s ii1 z ) ((pos "Y)%" ) ))
( "GHQ"  n ( jh ii2 ei2 ch k y uu1 ) ((pos "Y=%" ) ))
( "GI"  n ( jh ii2 ai1 ) ((pos "Y>%" ) ))
( "GIs"  n ( jh ii2 ai1 z ) ((pos "Y)%" ) ))
( "GMT"  n ( jh ii2 e2 m t ii1 ) ((pos "Y~%" ) ))
( "GNP"  n ( jh i2 e2 n p ii1 ) ((pos "Y>%" ) ))
( "GP"  n ( jh ii2 p ii1 ) ((pos "Y>%" ) ))
( "GP's"  n ( jh ii2 p ii1 z ) ((pos "Y)%" ) ))
( "Gabon"  n ( g a1 b o1 n ) ((pos "Nm%" ) ))
( "Gabonese"  n ( g a2 b @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Gael"  n ( g ei1 l ) ((pos "K6$" ) ))
( "Gaelic"  n ( g ei1 l i k ) ((pos "K6%" "OA%" ) ))
( "Gaelics"  n ( g ei1 l i k s ) ((pos "Kj%" ) ))
( "Gaels"  n ( g ei1 l z ) ((pos "Kj$" ) ))
( "Gainsborough"  n ( g ei1 n z b r @ ) ((pos "K6%" "Nn%" ) ))
( "Gainsboroughs"  n ( g ei1 n z b r @ z ) ((pos "Kj%" ) ))
( "Galashiels"  n ( g a1 l @ sh i@ l z ) ((pos "Nn%" ) ))
( "Gallic"  j ( g a1 l i k ) ((pos "OA%" ) ))
( "Galloway"  n ( g a1 l @ w ei ) ((pos "Nm%" ) ))
( "Gallup"  n ( g a1 l @ p ) ((pos "No%" ) ))
( "Galway"  n ( g oo1 l w ei ) ((pos "Nm%" ) ))
( "Gambia"  n ( g a1 m b i@ ) ((pos "Nm%" ) ))
( "Gambian"  n ( g a1 m b i@ n ) ((pos "K6%" "OA%" ) ))
( "Gambians"  n ( g a1 m b i@ n z ) ((pos "Kj%" ) ))
( "Gareth"  n ( g a1 r @ th ) ((pos "Nl%" ) ))
( "Garforth"  n ( g aa1 f @ th ) ((pos "Nn$" ) ))
( "Garrowhill"  n ( g a1 r @ h i l ) ((pos "Nn$" ) ))
( "Gary"  n ( g a1 r ii ) ((pos "Nl%" ) ))
( "Gateshead"  n ( g ei1 t s h e d ) ((pos "Nn%" ) ))
( "Gaul"  n ( g oo1 l ) ((pos "K6%" ) ))
( "Gauls"  n ( g oo1 l z ) ((pos "Kj%" ) ))
( "Gavin"  n ( g a1 v i n ) ((pos "Nl%" ) ))
( "Gdansk"  n ( g d a1 n s k ) ((pos "Nn%" ) ))
( "Gdn"  n ( g aa1 d n ) ((pos "Y=%" ) ))
( "Gdns"  n ( g aa1 d n z ) ((pos "Y=%" ) ))
( "Geiger"  n ( g ai1 g @ r ) ((pos "K6%" ) ))
( "Geigers"  n ( g ai1 g @ z ) ((pos "Kj$" ) ))
( "Gelligaer"  n ( g e1 l i g @ r ) ((pos "Nn$" ) ))
( "Gelsenkirchen"  n ( g e1 l z @ n k @@ k n ) ((pos "Nn$" ) ))
( "Gemini"  n ( jh e1 m i n ii ) ((pos "No%" ) ))
( "Gen"  n ( jh e1 n r @ l ) ((pos "Y:%" ) ))
( "Gene"  n ( jh ii1 n ) ((pos "Nl%" ) ))
( "Geneva"  n ( jh i1 n ii1 v @ ) ((pos "Nn%" ) ))
( "Genoa"  n ( jh e1 n ou @ ) ((pos "Nn%" ) ))
( "Genova"  n ( jh e1 n @ v @ ) ((pos "Nn%" ) ))
( "Geoff"  n ( jh e1 f ) ((pos "Nl%" ) ))
( "Geoffrey"  n ( jh e1 f r ii ) ((pos "Nl%" ) ))
( "George"  n ( jh oo1 jh ) ((pos "Nl%" ) ))
( "Georges"  n ( jh oo1 jh i z ) ((pos "Kj$" ) ))
( "Georgia"  n ( jh oo1 jh @ ) ((pos "Nm%" ) ))
( "Georgian"  j ( jh oo1 jh @ n ) ((pos "OA%" ) ))
( "Georgian"  n ( jh oo1 jh y @ n ) ((pos "K6%" "OA%" ) ))
( "Georgians"  n ( jh oo1 jh y @ n z ) ((pos "Kj%" ) ))
( "Georgie"  n ( jh oo1 jh ii ) ((pos "Nl%" ) ))
( "Gerald"  n ( jh e1 r @ l d ) ((pos "Nl%" ) ))
( "Geraldine"  n ( jh e1 r @ l d ii n ) ((pos "Nl%" ) ))
( "Gerard"  n ( jh e1 r @ d ) ((pos "Nl%" ) ))
( "German"  n ( jh @@1 m @ n ) ((pos "M6%" "OA%" ) ))
( "Germanic"  j ( jh @1 m a1 n i k ) ((pos "OA%" ) ))
( "Germans"  n ( jh @@1 m @ n z ) ((pos "Mj%" ) ))
( "Germany"  n ( jh @@1 m @ n ii ) ((pos "Nm%" ) ))
( "Gerry"  n ( jh e1 r ii ) ((pos "Nl%" ) ))
( "Gertie"  n ( g @@1 t ii ) ((pos "Nl%" ) ))
( "Gertrude"  n ( g @@1 t r uu d ) ((pos "Nl%" ) ))
( "Gestapo"  n ( g e1 s t aa1 p ou ) ((pos "K6%" ) ))
( "Gestapos"  n ( g e1 s t aa1 p ou z ) ((pos "Kj$" ) ))
( "Ghana"  n ( g aa1 n @ ) ((pos "Nm%" ) ))
( "Ghanaian"  n ( g aa1 n ei1 @ n ) ((pos "K6%" "OA%" ) ))
( "Ghanaians"  n ( g aa1 n ei1 @ n z ) ((pos "Kj%" ) ))
( "Ghent"  n ( g e1 n t ) ((pos "Nn%" ) ))
( "Gibraltar"  n ( jh i1 b r oo1 l t @ r ) ((pos "Nm%" ) ))
( "Gibraltarian"  n ( jh i2 b r oo l t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Gibraltarians"  n ( jh i2 b r oo l t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "Giffnock"  n ( g i1 f n o k ) ((pos "Nn$" ) ))
( "Gilbert"  n ( g i1 l b @ t ) ((pos "Nl%" ) ))
( "Gilbertian"  j ( g i1 l b @@1 t i@ n ) ((pos "OA$" ) ))
( "Giles"  n ( jh ai1 l z ) ((pos "Nl%" ) ))
( "Gilfach Goch"  n ( g ii2 l f a k - g o1 k ) ((pos "Nn$" ) ))
( "Gill"  n ( jh i1 l ) ((pos "Nl%" ) ))
( "Gillian"  n ( jh i1 l i@ n ) ((pos "Nl%" ) ))
( "Gillingham"  n ( jh i1 l i ng @ m ) ((pos "Nn%" ) ))
( "Gina"  n ( jh ii1 n @ ) ((pos "Nl%" ) ))
( "Girvan"  n ( g @@1 v n ) ((pos "Nn$" ) ))
( "Gladys"  n ( g l a1 d i s ) ((pos "Nl%" ) ))
( "Glamorgan"  n ( g l @1 m oo1 g @ n ) ((pos "Nm%" ) ))
( "Glasgow"  n ( g l a1 z g ou ) ((pos "Nn%" ) ))
( "Glastonbury"  n ( g l a1 s t @ n b r ii ) ((pos "Nn%" ) ))
( "Glaswegian"  n ( g l a1 z w ii1 jh @ n ) ((pos "K6%" "OA%" ) ))
( "Glaswegians"  n ( g l a1 z w ii1 jh @ n z ) ((pos "Kj%" ) ))
( "Glen"  n ( g l e1 n ) ((pos "Nl%" ) ))
( "Glengarries"  n ( g l e2 n g a1 r i z ) ((pos "Kj$" ) ))
( "Glengarry"  n ( g l e2 n g a1 r ii ) ((pos "K8$" ) ))
( "Glenrothes"  n ( g l e1 n r o1 th i s ) ((pos "Nn$" ) ))
( "Gloria"  n ( g l oo1 r i@ ) ((pos "Nl%" ) ))
( "Glossop"  n ( g l o1 s @ p ) ((pos "Nn%" ) ))
( "Gloucester"  n ( g l o1 s t @ r ) ((pos "Nn%" ) ))
( "Gloucestershire"  n ( g l o1 s t @ sh @ r ) ((pos "Nm%" ) ))
( "Glusburn"  n ( g l uh1 z b @@ n ) ((pos "Nn$" ) ))
( "God"  n ( g o1 d ) ((pos "No%" ) ))
( "Godalming"  n ( g o1 d l m i ng ) ((pos "Nn%" ) ))
( "Godfrey"  n ( g o1 d f r ii ) ((pos "Nl%" ) ))
( "Godmanchester"  n ( g o1 d m @ n ch e2 s t @ r ) ((pos "Nn%" ) ))
( "Goffs Oak"  n ( g o2 f s - ou1 k ) ((pos "Nn$" ) ))
( "Golbourne"  n ( g o1 l b oo n ) ((pos "Nn$" ) ))
( "Goliath"  n ( g @1 l ai1 @ th ) ((pos "K6%" ) ))
( "Goliaths"  n ( g @1 l ai1 @ th s ) ((pos "Kj$" ) ))
( "Goole"  n ( g uu1 l ) ((pos "Nn$" ) ))
( "Gordian"  j ( g oo1 d i@ n ) ((pos "OA%" ) ))
( "Gordon"  n ( g oo1 d n ) ((pos "Nl%" ) ))
( "Gorgon"  n ( g oo1 g @ n ) ((pos "K6%" ) ))
( "Gorgons"  n ( g oo1 g @ n z ) ((pos "Kj$" ) ))
( "Gorgonzola"  n ( g oo2 g @ n z ou1 l @ ) ((pos "L@%" ) ))
( "Gorky"  n ( g oo1 k ii ) ((pos "Nn$" ) ))
( "Gorseinon"  n ( g oo1 s ei1 n @ n ) ((pos "Nn$" ) ))
( "Gosport"  n ( g o1 s p oo t ) ((pos "Nn%" ) ))
( "Goteborg"  n ( g o1 t @ b oo g ) ((pos "Nn%" ) ))
( "Goth"  n ( g o1 th ) ((pos "K6%" ) ))
( "Gothenburg"  n ( g o1 th @ n b @@ g ) ((pos "Nn%" ) ))
( "Gothic"  n ( g o1 th i k ) ((pos "L@%" "OA%" ) ))
( "Goths"  n ( g o1 th s ) ((pos "Kj%" ) ))
( "Gottingen"  n ( g o1 t i ng @ n ) ((pos "Nn%" ) ))
( "Gourock"  n ( g uh1 r @ k ) ((pos "Nn$" ) ))
( "Gov"  n ( g uh1 v @ n @ r ) ((pos "Y:%" ) ))
( "Governor-General"  n ( g uh2 v n @ - jh e1 n r @ l ) ((pos "K6%" ) ))
( "Governor-Generals"  n ( g uh2 v n @ - jh e1 n r @ l z ) ((pos "Kj$" ) ))
( "Grampian"  n ( g r a1 m p i@ n ) ((pos "Nm%" ) ))
( "Granada"  n ( g r @1 n aa1 d @ ) ((pos "Nn%" ) ))
( "Grand Prix"  n ( g r aa2 n - p r ii1 ) ((pos "Ki%" ) ))
( "Grands Prix"  n ( g r aa2 n - p r ii1 ) ((pos "Kj$" ) ))
( "Grangemouth"  n ( g r ei1 n jh m @ th ) ((pos "Nn$" ) ))
( "Grantham"  n ( g r a1 n th @ m ) ((pos "Nn%" ) ))
( "Gravenhage"  n ( g r aa2 v n h aa1 g @ ) ((pos "Nn%" ) ))
( "Gravesend"  n ( g r ei2 v z e1 n d ) ((pos "Nn%" ) ))
( "Graz"  n ( g r aa1 t s ) ((pos "Nn%" ) ))
( "Great Harwood"  n ( g r ei2 t - h aa1 w u d ) ((pos "Nn$" ) ))
( "Great Malvern"  n ( g r ei2 t - m o1 l v @ n ) ((pos "Nn$" ) ))
( "Great Shelford"  n ( g r ei2 t - sh e1 l f @ d ) ((pos "Nn$" ) ))
( "Great Yarmouth"  n ( g r ei2 t - y aa1 m @ th ) ((pos "Nn%" ) ))
( "Grecian"  j ( g r ii1 sh @ n ) ((pos "OA%" ) ))
( "Greece"  n ( g r i1 s ) ((pos "Nm%" ) ))
( "Greek"  n ( g r ii1 k ) ((pos "M6%" "OA%" ) ))
( "Greeks"  n ( g r ii1 k s ) ((pos "Mj%" ) ))
( "Greenock"  n ( g r ii1 n @ k ) ((pos "Nn%" ) ))
( "Greenwich"  n ( g r e1 n i ch ) ((pos "Nn%" ) ))
( "Greg"  n ( g r e1 g ) ((pos "Nl%" ) ))
( "Gregorian"  j ( g r i1 g oo1 r i@ n ) ((pos "OA%" ) ))
( "Gregory"  n ( g r e1 g @ r ii ) ((pos "Nl%" ) ))
( "Grenada"  n ( g r i1 n ei1 d @ ) ((pos "Nm%" ) ))
( "Grenadian"  n ( g r i1 n ei1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Grenadians"  n ( g r i1 n ei1 d i@ n z ) ((pos "Kj%" ) ))
( "Grimethorpe"  n ( g r ai1 m th oo p ) ((pos "Nn%" ) ))
( "Grimsby"  n ( g r i1 m z b ii ) ((pos "Nn%" ) ))
( "Grinstead"  n ( g r i1 n s t i d ) ((pos "Nn$" ) ))
( "Groningen"  n ( g r ou1 n i ng @ n ) ((pos "Nn$" ) ))
( "Grundyism"  n ( g r uh1 n d i i z @ m ) ((pos "L@$" ) ))
( "Guadalajara"  n ( g w aa2 d l @ h aa1 r @ ) ((pos "Nn%" ) ))
( "Guatemala"  n ( g w aa2 t @ m aa1 l @ ) ((pos "Nm%" ) ))
( "Guatemala City"  n ( g w a2 t i m aa2 l @ - s i1 t ii ) ((pos "Nn%" ) ))
( "Guatemalan"  n ( g w aa2 t @ m aa1 l @ n ) ((pos "K6%" "OA%" ) ))
( "Guatemalans"  n ( g w aa2 t @ m aa1 l @ n z ) ((pos "Kj%" ) ))
( "Guayaquil"  n ( g w ai2 @ k ii1 l ) ((pos "Nn%" ) ))
( "Guild-hall"  n ( g i1 l d - h oo l ) ((pos "K6%" ) ))
( "Guild-halls"  n ( g i1 l d - h oo l z ) ((pos "Kj%" ) ))
( "Guildford"  n ( g i1 l f @ d ) ((pos "Nn%" ) ))
( "Guildhall"  n ( g i1 l d h oo l ) ((pos "K6%" ) ))
( "Guildhalls"  n ( g i1 l d h oo l z ) ((pos "Kj%" ) ))
( "Guinea"  n ( g i1 n ii ) ((pos "Nm%" ) ))
( "Guinean"  n ( g i1 n i@ n ) ((pos "K6$" "OA$" ) ))
( "Guineans"  n ( g i1 n i@ n z ) ((pos "Kj$" ) ))
( "Guinness"  n ( g i1 n i s ) ((pos "K7%" ) ))
( "Guinnesses"  n ( g i1 n i s i z ) ((pos "Kj%" ) ))
( "Guisborough"  n ( g i1 z b r @ ) ((pos "Nn$" ) ))
( "Gurkha"  n ( g @@1 k @ ) ((pos "K6%" ) ))
( "Gurkhas"  n ( g @@1 k @ z ) ((pos "Kj%" ) ))
( "Guy"  n ( g ai1 ) ((pos "Nl%" ) ))
( "Guy's"  n ( g ai1 z ) ((pos "No$" ) ))
( "Guyana"  n ( g ai1 a1 n @ ) ((pos "Nm%" ) ))
( "Guyanese"  n ( g ai2 @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Gwen"  n ( g w e1 n ) ((pos "Nl%" ) ))
( "Gwendoline"  n ( g w e1 n d @ l i n ) ((pos "Nl%" ) ))
( "Gwent"  n ( g w e1 n t ) ((pos "Nm%" ) ))
( "Gwersyllt"  n ( g w @@1 s i l t ) ((pos "Nn$" ) ))
( "Gwynedd"  n ( g w i1 n @ dh ) ((pos "Nm%" ) ))
( "Gypsies"  n ( jh i1 p s i z ) ((pos "Kj%" ) ))
( "Gypsy"  n ( jh i1 p s ii ) ((pos "K8%" ) ))
( "H"  n ( ei1 ch ) ((pos "Ki$" ) ))
( "H's"  n ( ei1 ch i z ) ((pos "Kj$" ) ))
( "H-bomb"  n ( ei1 ch - b o m ) ((pos "K6%" ) ))
( "H-bombs"  n ( ei1 ch - b o m z ) ((pos "Kj%" ) ))
( "HM"  n ( ei2 ch e1 m ) ((pos "Y:%" ) ))
( "HMSO"  n ( ei2 ch e2 m e2 s ou1 ) ((pos "Y=%" ) ))
( "HP"  n ( ei2 ch p ii1 ) ((pos "Y}%" ) ))
( "HQ"  n ( ei2 ch k y uu1 ) ((pos "Y=%" ) ))
( "HRH"  n ( ei2 ch aa r ei1 ch ) ((pos "Y:%" ) ))
( "Haarlem"  n ( h aa1 l @ m ) ((pos "Nn%" ) ))
( "Hackney"  n ( h a1 k n ii ) ((pos "Nn%" ) ))
( "Haddington"  n ( h a1 d i ng t @ n ) ((pos "Nn$" ) ))
( "Hades"  n ( h ei1 d ii z ) ((pos "No%" ) ))
( "Hadji"  n ( h a1 jh ii ) ((pos "K6$" ) ))
( "Hadjis"  n ( h a1 jh i z ) ((pos "Kj$" ) ))
( "Hagen"  n ( h aa1 g @ n ) ((pos "Nn$" ) ))
( "Hagley"  n ( h a1 g l ii ) ((pos "Nn$" ) ))
( "Hailsham"  n ( h ei1 l sh @ m ) ((pos "Nn$" ) ))
( "Haiphong"  n ( h ai2 f o1 ng ) ((pos "Nn$" ) ))
( "Haiti"  n ( h ei1 t ii ) ((pos "Nm%" ) ))
( "Haitian"  n ( h ei1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Haitians"  n ( h ei1 sh @ n z ) ((pos "Kj%" ) ))
( "Hal"  n ( h a1 l ) ((pos "Nl%" ) ))
( "Halesowen"  n ( h ei1 l z ou1 i n ) ((pos "Nn$" ) ))
( "Halifax"  n ( h a1 l i f a k s ) ((pos "Nn%" ) ))
( "Halle"  n ( h a1 l ii ) ((pos "Nn%" ) ))
( "Hallowe'en"  n ( h a2 l ou ii1 n ) ((pos "K6%" "No%" ) ))
( "Hallowe'ens"  n ( h a2 l ou ii1 n z ) ((pos "Kj$" ) ))
( "Halstead"  n ( h o1 l s t i d ) ((pos "Nn$" ) ))
( "Hamburg"  n ( h a1 m b @@ g ) ((pos "Nn%" ) ))
( "Hamhung"  n ( h a2 m h u1 ng ) ((pos "Nn$" ) ))
( "Hamilton"  n ( h a1 m i l t n ) ((pos "Nn%" ) ))
( "Hammersmith"  n ( h a1 m @ s m i th ) ((pos "Nn%" ) ))
( "Hampreston"  n ( h a1 m p r i s t n ) ((pos "Nn$" ) ))
( "Hampshire"  n ( h a1 m p sh @ r ) ((pos "Nm%" ) ))
( "Hangchon"  n ( h a2 ng ch o1 n ) ((pos "Nn$" ) ))
( "Hanoi"  n ( h a1 n oi1 ) ((pos "Nn%" ) ))
( "Hanover"  n ( h a1 n ou v @ r ) ((pos "Nn%" ) ))
( "Hansard"  n ( h a1 n s aa d ) ((pos "K6%" ) ))
( "Hansards"  n ( h a1 n s aa d z ) ((pos "Kj$" ) ))
( "Harbin"  n ( h aa1 b i n ) ((pos "Nn$" ) ))
( "Haringey"  n ( h a1 r i ng g ei ) ((pos "Nn%" ) ))
( "Harley Street"  n ( h aa1 l i - s t r ii t ) ((pos "No%" ) ))
( "Harlow"  n ( h aa1 l ou ) ((pos "Nn%" ) ))
( "Harold"  n ( h a1 r @ l d ) ((pos "Nl%" ) ))
( "Harpenden"  n ( h aa1 p i n d @ n ) ((pos "Nn%" ) ))
( "Harriet"  n ( h a1 r i@ t ) ((pos "Nl%" ) ))
( "Harrogate"  n ( h a1 r @ g i t ) ((pos "Nn%" ) ))
( "Harrow"  n ( h a1 r ou ) ((pos "Nn%" ) ))
( "Harry"  n ( h a1 r ii ) ((pos "Nl%" ) ))
( "Hartlepool"  n ( h aa1 t l i p uu l ) ((pos "Nn%" ) ))
( "Hartley"  n ( h aa1 t l ii ) ((pos "Nn$" ) ))
( "Harvey"  n ( h aa1 v ii ) ((pos "Nl%" ) ))
( "Harwich"  n ( h a1 r i ch ) ((pos "Nn%" ) ))
( "Harwood"  n ( h aa1 w u d ) ((pos "Nn$" ) ))
( "Harworth"  n ( h aa1 w @ th ) ((pos "Nn$" ) ))
( "Haslemere"  n ( h ei1 z l m i@ r ) ((pos "Nn$" ) ))
( "Haslingden"  n ( h a1 z l i ng d @ n ) ((pos "Nn$" ) ))
( "Hastings"  n ( h ei1 s t i ng z ) ((pos "Nn%" ) ))
( "Hatfield"  n ( h a1 t f ii l d ) ((pos "Nn%" ) ))
( "Havana"  n ( h @1 v a1 n @ ) ((pos "K6%" "Nn%" ) ))
( "Havanas"  n ( h @1 v a1 n @ z ) ((pos "Kj%" ) ))
( "Haverfordwest"  n ( h a2 v @ f @ d w e1 s t ) ((pos "Nn%" ) ))
( "Haverhill"  n ( h ei1 v @ r i l ) ((pos "Nn$" ) ))
( "Havering"  n ( h ei1 v @ r i ng ) ((pos "Nn%" ) ))
( "Hawaii"  n ( h @1 w ai1 ii ) ((pos "Nm%" ) ))
( "Hawick"  n ( h oo1 i k ) ((pos "Nn$" ) ))
( "Hawke's Bay"  n ( h oo2 k s - b ei1 ) ((pos "Nm$" ) ))
( "Hayling"  n ( h ei1 l i ng ) ((pos "Nn%" ) ))
( "Haywards Heath"  n ( h ei2 w @ d z - h ii1 th ) ((pos "Nn%" ) ))
( "Hazel"  n ( h ei1 z l ) ((pos "Nl%" ) ))
( "Heanor"  n ( h ii1 n @ r ) ((pos "Nn$" ) ))
( "Heather"  n ( h e1 dh @ r ) ((pos "Nl%" ) ))
( "Heaviside"  n ( h e1 v i s ai d ) ((pos "No$" ) ))
( "Hebden Royal"  n ( h e2 b d @ n - r oi1 @ l ) ((pos "Nn$" ) ))
( "Hebraic"  j ( h ii1 b r ei1 i k ) ((pos "OA$" ) ))
( "Hebrew"  n ( h ii1 b r uu ) ((pos "M6%" "OA%" ) ))
( "Hebrews"  n ( h ii1 b r uu z ) ((pos "Mj%" ) ))
( "Hedge End"  n ( h e2 jh - e1 n d ) ((pos "Nn$" ) ))
( "Hegira"  n ( h e1 jh i r @ ) ((pos "K6$" ) ))
( "Hegiras"  n ( h e1 jh i r @ z ) ((pos "Kj$" ) ))
( "Heidelburg"  n ( h ai1 d l b @@ g ) ((pos "Nn%" ) ))
( "Hejira"  n ( h e1 jh i r @ ) ((pos "K6$" ) ))
( "Hejiras"  n ( h e1 jh i r @ z ) ((pos "Kj$" ) ))
( "Helen"  n ( h e1 l @ n ) ((pos "Nl%" ) ))
( "Helensburgh"  n ( h e1 l i n z b r @ ) ((pos "Nn%" ) ))
( "Hellene"  n ( h e1 l ii n ) ((pos "K6%" ) ))
( "Hellenes"  n ( h e1 l ii n z ) ((pos "Kj%" ) ))
( "Hellenic"  j ( h e1 l ii1 n i k ) ((pos "OA%" ) ))
( "Helsinki"  n ( h e1 l s i ng k ii ) ((pos "Nn%" ) ))
( "Helston"  n ( h e1 l s t @ n ) ((pos "Nn$" ) ))
( "Hemel Hempstead"  n ( h e2 m l - h e1 m p s t i d ) ((pos "Nn%" ) ))
( "Hemsworth"  n ( h e1 m z w @ th ) ((pos "Nn$" ) ))
( "Henley-on-Thames"  n ( h e2 n l i - o n - t e1 m z ) ((pos "Nn%" ) ))
( "Henry"  n ( h e1 n r ii ) ((pos "Nl%" ) ))
( "Herb"  n ( h @@1 b ) ((pos "Nl%" ) ))
( "Herbert"  n ( h @@1 b @ t ) ((pos "Nl%" ) ))
( "Hereford"  n ( h e1 r i f @ d ) ((pos "Nm%" ) ))
( "Herne"  n ( h @@1 n ) ((pos "Nn$" ) ))
( "Herne Bay"  n ( h @@2 n - b ei1 ) ((pos "Nn%" ) ))
( "Herr"  n ( h e@1 r ) ((pos "Ki%" ) ))
( "Herren"  n ( h e1 r @ n ) ((pos "Kj$" ) ))
( "Hertford"  n ( h aa1 t f @ d ) ((pos "Nn%" ) ))
( "Hertfordshire"  n ( h aa1 t f @ d sh @ r ) ((pos "Nm%" ) ))
( "Hertzian"  j ( h @@1 t s i@ n ) ((pos "OA$" ) ))
( "Hetton"  n ( h e1 t n ) ((pos "Nn$" ) ))
( "Hexham"  n ( h e1 k s @ m ) ((pos "Nn$" ) ))
( "Hey presto"  uh ( h ei2 - p r e1 s t ou ) ((pos "W-$" ) ))
( "Heysham"  n ( h ii1 sh @ m ) ((pos "Nn$" ) ))
( "Heywood"  n ( h ei1 w u d ) ((pos "Nn$" ) ))
( "High Spen"  n ( h ai2 - s p e1 n ) ((pos "Nn$" ) ))
( "High Wycombe"  n ( h ai2 - w i1 k @ m ) ((pos "Nn%" ) ))
( "Highland"  n ( h ai1 l @ n d ) ((pos "Nm%" ) ))
( "Highlander"  n ( h ai1 l @ n d @ r ) ((pos "K6%" ) ))
( "Highlanders"  n ( h ai1 l @ n d @ z ) ((pos "Kj%" ) ))
( "Hilary"  n ( h i1 l @ r ii ) ((pos "Nl%" ) ))
( "Hilda"  n ( h i1 l d @ ) ((pos "Nl%" ) ))
( "Hillingdon"  n ( h i1 l i ng d @ n ) ((pos "Nn%" ) ))
( "Hinckley"  n ( h i1 ng k l ii ) ((pos "Nn$" ) ))
( "Hindi"  n ( h i1 n d ii ) ((pos "L@%" "OA%" ) ))
( "Hindu"  n ( h i2 n d uu1 ) ((pos "K6%" "OA%" ) ))
( "Hinduism"  n ( h i1 n d uu i z @ m ) ((pos "L@%" ) ))
( "Hindus"  n ( h i2 n d uu1 z ) ((pos "Kj%" ) ))
( "Hindustani"  n ( h i2 n d u s t aa1 n ii ) ((pos "K6%" "OA%" ) ))
( "Hindustanis"  n ( h i2 n d u s t aa1 n i z ) ((pos "Kj%" ) ))
( "Hippocratic"  j ( h i2 p @ k r a1 t i k ) ((pos "OA%" ) ))
( "Hiroshima"  n ( h i1 r o1 sh i m @ ) ((pos "Nn%" ) ))
( "Hitchin"  n ( h i1 ch i n ) ((pos "Nn%" ) ))
( "Hobson"  n ( h o1 b s n ) ((pos "No$" ) ))
( "Hockley"  n ( h o1 k l ii ) ((pos "Nn$" ) ))
( "Hoddesdon"  n ( h o1 d z d @ n ) ((pos "Nn$" ) ))
( "Hofei"  n ( h ou2 f ei1 ) ((pos "Nn$" ) ))
( "Hogmanay"  n ( h o1 g m @ n ei ) ((pos "K6%" "No%" ) ))
( "Hogmanays"  n ( h o1 g m @ n ei z ) ((pos "Kj$" ) ))
( "Holland"  n ( h o1 l @ n d ) ((pos "Nm%" ) ))
( "Hollander"  n ( h o1 l @ n d @ r ) ((pos "K6$" "OA$" ) ))
( "Hollanders"  n ( h o1 l @ n d @ z ) ((pos "Kj$" ) ))
( "Hollywood"  n ( h o1 l i w u d ) ((pos "No%" ) ))
( "Holmfirth"  n ( h ou1 m f @ th ) ((pos "Nn%" ) ))
( "Holyhead"  n ( h o1 l i h e d ) ((pos "Nn%" ) ))
( "Homeric"  j ( h ou1 m e1 r i k ) ((pos "OA%" ) ))
( "Hon"  n ( h o1 n ) ((pos "Y:%" ) ))
( "Honduran"  n ( h o1 n d y u@1 r @ n ) ((pos "K6%" "OA%" ) ))
( "Hondurans"  n ( h o1 n d y u@1 r @ n z ) ((pos "Kj%" ) ))
( "Honduras"  n ( h o1 n d y u@1 r @ s ) ((pos "Nm%" ) ))
( "Hong Kong"  n ( h o2 ng - k o1 ng ) ((pos "Nm%" ) ))
( "Honiton"  n ( h o1 n i t n ) ((pos "Nn%" ) ))
( "Hoo"  n ( h uu1 ) ((pos "Nn$" ) ))
( "Hoover"  v ( h uu1 v @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "Hoovered"  v ( h uu1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "Hoovering"  v ( h uu1 v @ r i ng ) ((pos "Hb%" ) (subcat )))
( "Hoovers"  v ( h uu1 v @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "Hope"  n ( h ou1 p ) ((pos "Nn%" ) ))
( "Horace"  n ( h o1 r i s ) ((pos "Nl%" ) ))
( "Horley"  n ( h oo1 l ii ) ((pos "Nn$" ) ))
( "Hornsea"  n ( h oo1 n z ii ) ((pos "Nn$" ) ))
( "Horsham"  n ( h oo1 sh @ m ) ((pos "Nn$" ) ))
( "Horsley"  n ( h oo1 z l ii ) ((pos "Nn$" ) ))
( "Horwich"  n ( h o1 r i ch ) ((pos "Nn%" ) ))
( "Hosp"  n ( h o1 s p ) ((pos "Y=%" ) ))
( "Host"  n ( h ou1 s t ) ((pos "Ki$" ) ))
( "Houghton"  n ( h oo1 t n ) ((pos "Nn$" ) ))
( "Hounslow"  n ( h au1 n z l ou ) ((pos "Nn%" ) ))
( "Houston"  n ( h y uu1 s t n ) ((pos "Nn%" ) ))
( "Hove"  n ( h ou1 v ) ((pos "Nn%" ) ))
( "Howard"  n ( h au1 @ d ) ((pos "Nl%" ) ))
( "Hoylake"  n ( h oi1 l ei k ) ((pos "Nn%" ) ))
( "Hoyland Nether"  n ( h oi2 l n d - n e1 dh @ r ) ((pos "Nn$" ) ))
( "Hubert"  n ( h y uu1 b @ t ) ((pos "Nl%" ) ))
( "Hucknall"  n ( h uh1 k n l ) ((pos "Nn$" ) ))
( "Huddersfield"  n ( h uh1 d @ s f ii l d ) ((pos "Nn%" ) ))
( "Hugh"  n ( h y uu1 ) ((pos "Nl%" ) ))
( "Hughie"  n ( h y uu1 ii ) ((pos "Nl%" ) ))
( "Huguenot"  n ( h y uu1 g @ n ou ) ((pos "K6%" ) ))
( "Huguenots"  n ( h y uu1 g @ n ou z ) ((pos "Kj%" ) ))
( "Huhehot"  n ( h uu2 h i h o1 t ) ((pos "Nn$" ) ))
( "Humberside"  n ( h uh1 m b @ s ai d ) ((pos "Nm%" ) ))
( "Humberston"  n ( h uh1 m b @ s t @ n ) ((pos "Nn$" ) ))
( "Humphrey"  n ( h uh1 m f r ii ) ((pos "Nl%" ) ))
( "Hun"  n ( h uh1 n ) ((pos "K6$" ) ))
( "Hungarian"  n ( h uh2 ng g e@1 r i@ n ) ((pos "M6%" "OA%" ) ))
( "Hungarians"  n ( h uh2 ng g e@1 r i@ n z ) ((pos "Mj%" ) ))
( "Hungary"  n ( h uh1 ng g @ r ii ) ((pos "Nm%" ) ))
( "Huns"  n ( h uh1 n z ) ((pos "Kj$" ) ))
( "Huntingdon"  n ( h uh1 n t i ng d @ n ) ((pos "Nn%" ) ))
( "Hurstpierpoint"  n ( h @@2 s p ii @ p oi1 n t ) ((pos "Nn$" ) ))
( "Hwainan"  n ( h w ai2 n a1 n ) ((pos "Nn$" ) ))
( "Hyde"  n ( h ai1 d ) ((pos "Nn%" ) ))
( "Hyderabad"  n ( h ai1 d r @ b a d ) ((pos "Nn%" ) ))
( "Hythe"  n ( h ai1 dh ) ((pos "Nn%" ) ))
( "I"  n ( ai ) ((pos "Ki$" "Qx*" ) ))
( "I'd"  v ( ai d ) ((pos "Gf%" ) ))
( "I'll"  v ( ai l ) ((pos "Gf*" ) ))
( "I'm"  v ( ai m ) ((pos "Gf*" ) ))
( "I's"  n ( ai1 z ) ((pos "Kj$" ) ))
( "I've"  v ( ai v ) ((pos "Gf%" ) ))
( "IBA"  n ( ai2 b ii2 ei1 ) ((pos "Y>%" ) ))
( "ICBM"  n ( ai2 s ii2 b ii2 e1 m ) ((pos "Y>%" ) ))
( "ILO"  n ( ai2 e2 l ou1 ) ((pos "Y>%" ) ))
( "IMF"  n ( ai2 e2 m e1 f ) ((pos "Y>%" ) ))
( "IOU"  n ( ai2 ou y uu1 ) ((pos "Y>%" ) ))
( "IOUs"  n ( ai2 ou y uu1 z ) ((pos "Y)%" ) ))
( "IQ"  n ( ai2 k y uu1 ) ((pos "Y}%" ) ))
( "IRA"  n ( ai2 aa2 r ei1 ) ((pos "Y>%" ) ))
( "ITV"  n ( ai2 t ii2 v ii1 ) ((pos "Y>%" ) ))
( "IUD"  n ( ai2 y uu2 d ii1 ) ((pos "Y>%" ) ))
( "Ian"  n ( i@1 n ) ((pos "Nl%" ) ))
( "Ibadan"  n ( i1 b a1 d n ) ((pos "Nn%" ) ))
( "Iceland"  n ( ai1 s l @ n d ) ((pos "Nm%" ) ))
( "Icelander"  n ( ai1 s l @ n d @ r ) ((pos "K6%" "OA%" ) ))
( "Icelanders"  n ( ai1 s l @ n d @ z ) ((pos "Kj%" ) ))
( "Icelandic"  n ( ai1 s l @1 n d i k ) ((pos "L@%" "OA%" ) ))
( "Ida"  n ( ai1 d @ ) ((pos "Nl%" ) ))
( "Idaho"  n ( ai1 d @ h ou ) ((pos "Nm%" ) ))
( "Ilfracombe"  n ( i1 l f r @ k uu m ) ((pos "Nn%" ) ))
( "Ilkeston"  n ( i1 l k s t @ n ) ((pos "Nn%" ) ))
( "Ilkley"  n ( i1 l k l ii ) ((pos "Nn%" ) ))
( "Illinois"  n ( i2 l @ n oi1 ) ((pos "Nm%" ) ))
( "Immingham"  n ( i1 m i ng @ m ) ((pos "Nn$" ) ))
( "Inc"  n ( i1 ng k ) ((pos "Y~%" ) ))
( "Inchon"  n ( i2 n ch o1 n ) ((pos "Nn$" ) ))
( "India"  n ( i1 n d i@ ) ((pos "Nm%" ) ))
( "India-rubber"  n ( i1 n d i@ - r uh b @ r ) ((pos "M6%" ) ))
( "India-rubbers"  n ( i1 n d i@ - r uh b @ z ) ((pos "Mj$" ) ))
( "Indiaman"  n ( i1 n d i@ m @ n ) ((pos "Ki$" ) ))
( "Indiamen"  n ( i1 n d i@ m @ n ) ((pos "Kj$" ) ))
( "Indian"  n ( i1 n d i@ n ) ((pos "K6%" "OA%" ) ))
( "Indiana"  n ( i2 n d i a1 n @ ) ((pos "Nm%" ) ))
( "Indianapolis"  n ( i2 n d ii @ n a1 p @ l i s ) ((pos "Nn%" ) ))
( "Indians"  n ( i1 n d i@ n z ) ((pos "Kj%" ) ))
( "Indies"  n ( i1 n d i z ) ((pos "Nm%" ) ))
( "Indo-"  j ( i2 n d ou - ) ((pos "U-%" ) ))
( "Indo-European"  j ( i2 n d ou - y u@2 r @ p ii1 @ n ) ((pos "OA%" ) ))
( "Indonesia"  n ( i2 n d @ n ii1 z i@ ) ((pos "Nm%" ) ))
( "Indonesian"  n ( i2 n d @ n ii1 z i@ n ) ((pos "M6%" "OA%" ) ))
( "Indonesians"  n ( i2 n d @ n ii1 z i@ n z ) ((pos "Mj%" ) ))
( "Indore"  n ( i1 n d oo1 r ) ((pos "Nn$" ) ))
( "Ingatestone"  n ( i1 ng g ei t s t ou n ) ((pos "Nn$" ) ))
( "Ingrid"  n ( i1 ng g r i d ) ((pos "Nl%" ) ))
( "Innsbruck"  n ( i1 n z b r u k ) ((pos "Nn%" ) ))
( "Inst"  n ( i1 n s t ) ((pos "Y=%" ) ))
( "Interpol"  n ( i1 n t @ p o l ) ((pos "No%" ) ))
( "Inverkeithing"  n ( i2 n v @ k ii1 dh i ng ) ((pos "Nn$" ) ))
( "Inverness"  n ( i2 n v @ n e1 s ) ((pos "Nn%" ) ))
( "Inverurie"  n ( i2 n v @ r uu1 r ii ) ((pos "Nn$" ) ))
( "Ionic"  j ( ai1 o1 n i k ) ((pos "OA%" ) ))
( "Iowa"  n ( ai1 @ w @ ) ((pos "Nm%" ) ))
( "Ipswich"  n ( i1 p s w i ch ) ((pos "Nn%" ) ))
( "Iran"  n ( i1 r aa1 n ) ((pos "Nm%" ) ))
( "Iranian"  n ( i1 r ei1 n i@ n ) ((pos "M6%" "OA%" ) ))
( "Iranians"  n ( i1 r ei1 n i@ n z ) ((pos "Mj%" ) ))
( "Iraq"  n ( i1 r aa1 k ) ((pos "Nm%" ) ))
( "Iraqi"  n ( i1 r aa1 k ii ) ((pos "K6%" "OA%" ) ))
( "Iraqis"  n ( i1 r aa1 k i z ) ((pos "Kj%" ) ))
( "Ireland"  n ( ai1 @ l @ n d ) ((pos "Nm%" ) ))
( "Irene"  n ( ai1 r ii n ) ((pos "Nl%" ) ))
( "Iris"  n ( ai1 r i s ) ((pos "Nl%" ) ))
( "Irish"  n ( ai1 r i sh ) ((pos "L@%" "OA%" ) ))
( "Irishman"  n ( ai1 r i sh m @ n ) ((pos "Ki%" ) ))
( "Irishmen"  n ( ai1 r i sh m @ n ) ((pos "Kj%" ) ))
( "Irishwoman"  n ( ai1 r i sh w u m @ n ) ((pos "Ki%" ) ))
( "Irishwomen"  n ( ai1 r i sh w i m i n ) ((pos "Kj%" ) ))
( "Irthlingborough"  n ( @@1 th l i ng b r @ ) ((pos "Nn$" ) ))
( "Irvine"  n ( @@1 v ai n ) ((pos "Nn%" ) ))
( "Isaac"  n ( ai1 z @ k ) ((pos "Nl%" ) ))
( "Isabel"  n ( i1 z @ b e l ) ((pos "Nl%" ) ))
( "Isabella"  n ( i2 z @ b e1 l @ ) ((pos "Nl%" ) ))
( "Islam"  n ( i1 z l aa1 m ) ((pos "L@%" ) ))
( "Islamic"  j ( i1 z l a1 m i k ) ((pos "OA%" ) ))
( "Islington"  n ( i1 z l i ng t @ n ) ((pos "Nn%" ) ))
( "Isobel"  n ( i1 z @ b e l ) ((pos "Nl%" ) ))
( "Israel"  n ( i1 z r ei l ) ((pos "Nm%" ) ))
( "Israeli"  n ( i1 z r ei1 l ii ) ((pos "K6%" "OA%" ) ))
( "Israelis"  n ( i1 z r ei1 l i z ) ((pos "Kj%" ) ))
( "Istanbul"  n ( i2 s t a n b u1 l ) ((pos "Nn%" ) ))
( "Italian"  n ( i1 t a1 l i@ n ) ((pos "M6%" "OA%" ) ))
( "Italians"  n ( i1 t a1 l i@ n z ) ((pos "Mj%" ) ))
( "Italy"  n ( i1 t @ l ii ) ((pos "Nm%" ) ))
( "Ivan"  n ( ai1 v @ n ) ((pos "Nl%" ) ))
( "Iver"  n ( ai1 v @ r ) ((pos "Nn$" ) ))
( "Ivor"  n ( ai1 v @ r ) ((pos "Nl%" ) ))
( "Ivy"  n ( ai1 v ii ) ((pos "Nl%" ) ))
( "Izmir"  n ( i1 z m i@1 r ) ((pos "Nn$" ) ))
( "J"  n ( jh ei1 ) ((pos "Ki$" ) ))
( "J's"  n ( jh ei1 z ) ((pos "Kj$" ) ))
( "JC"  n ( jh ei2 s ii1 ) ((pos "Y=$" ) ))
( "JP"  n ( jh ei2 p ii1 ) ((pos "Y>%" ) ))
( "Jabalpur"  n ( jh aa2 b l p u@1 r ) ((pos "Nn$" ) ))
( "Jack"  n ( jh a1 k ) ((pos "Nl%" ) ))
( "Jackie"  n ( jh a1 k ii ) ((pos "Nl%" ) ))
( "Jacksonville"  n ( jh a1 k s @ n v i l ) ((pos "Nn$" ) ))
( "Jacob"  n ( jh ei1 k @ b ) ((pos "Nl%" ) ))
( "Jacobean"  j ( jh a2 k @ b i@1 n ) ((pos "OA%" ) ))
( "Jacobin"  n ( jh a1 k @ b i n ) ((pos "K6$" "OA$" ) ))
( "Jacobinism"  n ( jh a1 k @ b i n i z @ m ) ((pos "L@$" ) ))
( "Jacobins"  n ( jh a1 k @ b i n z ) ((pos "Kj$" ) ))
( "Jacobite"  n ( jh a1 k @ b ai t ) ((pos "K6%" ) ))
( "Jacobites"  n ( jh a1 k @ b ai t s ) ((pos "Kj%" ) ))
( "Jacqueline"  n ( jh a1 k @ l ii n ) ((pos "Nl%" ) ))
( "Jaipur"  n ( jh ai2 p u@1 r ) ((pos "Nn%" ) ))
( "Jakarta"  n ( jh @1 k aa1 t @ ) ((pos "Nn%" ) ))
( "Jake"  n ( jh ei1 k ) ((pos "Nl%" ) ))
( "Jamaica"  n ( jh @1 m ei1 k @ ) ((pos "Nm%" ) ))
( "Jamaican"  n ( jh @1 m ei1 k @ n ) ((pos "K6%" "OA%" ) ))
( "Jamaicans"  n ( jh @1 m ei1 k @ n z ) ((pos "Kj%" ) ))
( "James"  n ( jh ei1 m z ) ((pos "Nl%" ) ))
( "Jan"  n ( jh a1 n ) ((pos "Nl%" "Y=%" ) ))
( "Jane"  n ( jh ei1 n ) ((pos "Nl%" ) ))
( "Janet"  n ( jh a1 n i t ) ((pos "Nl%" ) ))
( "Janice"  n ( jh a1 n i s ) ((pos "Nl%" ) ))
( "Janie"  n ( jh ei1 n ii ) ((pos "Nl%" ) ))
( "Januaries"  n ( jh a1 n y u r i z ) ((pos "Kj$" ) ))
( "January"  n ( jh a1 n y u r ii ) ((pos "K8%" "No%" ) ))
( "Janus"  n ( jh ei1 n @ s ) ((pos "No$" ) ))
( "Japan"  n ( jh @1 p a1 n ) ((pos "Nm%" ) ))
( "Japanese"  n ( jh a2 p @ n ii1 z ) ((pos "M9%" "OA%" ) ))
( "Jarrow"  n ( jh a1 r ou ) ((pos "Nn%" ) ))
( "Jason"  n ( jh ei1 s @ n ) ((pos "Nl%" ) ))
( "Java"  n ( jh aa1 v @ ) ((pos "Nm%" ) ))
( "Javanese"  n ( jh aa2 v @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Jean"  n ( jh ii1 n ) ((pos "Nl%" ) ))
( "Jeanie"  n ( jh ii1 n ii ) ((pos "Nl%" ) ))
( "Jedburgh"  n ( jh e1 d b r @ ) ((pos "Nn%" ) ))
( "Jeff"  n ( jh e1 f ) ((pos "Nl%" ) ))
( "Jeffrey"  n ( jh e1 f r ii ) ((pos "Nl%" ) ))
( "Jehovah"  n ( jh i1 h ou1 v @ ) ((pos "No%" ) ))
( "Jekyll-and-Hyde"  n ( jh e2 k l - @ n - h ai1 d ) ((pos "No%" ) ))
( "Jennifer"  n ( jh e1 n i f @ r ) ((pos "Nl%" ) ))
( "Jenny"  n ( jh e1 n ii ) ((pos "Nl%" ) ))
( "Jeremy"  n ( jh e1 r @ m ii ) ((pos "Nl%" ) ))
( "Jerome"  n ( jh e1 r ou1 m ) ((pos "Nl%" ) ))
( "Jerry"  n ( jh e1 r ii ) ((pos "Nl%" ) ))
( "Jersey"  n ( jh @@1 z ii ) ((pos "Nm%" ) ))
( "Jess"  n ( jh e1 s ) ((pos "Nl%" ) ))
( "Jessica"  n ( jh e1 s i k @ ) ((pos "Nl%" ) ))
( "Jessie"  n ( jh e1 s ii ) ((pos "Nl%" ) ))
( "Jesuit"  n ( jh e1 z y u i t ) ((pos "K6%" ) ))
( "Jesuitical"  j ( jh e2 z y u i1 t i k l ) ((pos "OA%" ) ))
( "Jesuits"  n ( jh e1 z y u i t s ) ((pos "Kj%" ) ))
( "Jesus"  n ( jh ii1 z @ z ) ((pos "Nl%" ) ))
( "Jew"  n ( jh uu1 ) ((pos "K6%" ) ))
( "Jewess"  n ( jh uu1 i s ) ((pos "K7%" ) ))
( "Jewesses"  n ( jh uu1 i s i z ) ((pos "Kj%" ) ))
( "Jewish"  j ( jh uu1 i sh ) ((pos "OA%" ) ))
( "Jews"  n ( jh uu1 z ) ((pos "Kj%" ) ))
( "Jezebel"  n ( jh e1 z @ b l ) ((pos "K6%" ) ))
( "Jezebels"  n ( jh e1 z @ b l z ) ((pos "Kj$" ) ))
( "Jill"  n ( jh i1 l ) ((pos "Nl%" ) ))
( "Jim"  n ( jh i1 m ) ((pos "Nl%" ) ))
( "Jim Crow"  n ( jh i2 m - k r ou1 ) ((pos "Ki$" ) ))
( "Jimmy"  n ( jh i1 m ii ) ((pos "Nl%" ) ))
( "Jnr"  n ( jh uu1 n i@ r ) ((pos "Y~%" ) ))
( "Jo"  n ( jh ou1 ) ((pos "Nl%" ) ))
( "Joan"  n ( jh ou1 n ) ((pos "Nl%" ) ))
( "Joanie"  n ( jh ou1 n ii ) ((pos "Nl%" ) ))
( "Joann"  n ( jh ou1 a1 n ) ((pos "Nl%" ) ))
( "Joanna"  n ( jh ou1 a1 n @ ) ((pos "Nl%" ) ))
( "Joanne"  n ( jh ou1 a1 n ) ((pos "Nl%" ) ))
( "Job"  n ( jh ou1 b ) ((pos "Nl$" ) ))
( "Jobs"  n ( jh ou1 b z ) ((pos "Kj$" ) ))
( "Jocelyn"  n ( jh o1 s l i n ) ((pos "Nl%" ) ))
( "Joe"  n ( jh ou1 ) ((pos "Nl%" ) ))
( "Joey"  n ( jh ou1 ii ) ((pos "Nl%" ) ))
( "Johannesburg"  n ( jh @1 h a1 n @ z b @@ g ) ((pos "Nn%" ) ))
( "John"  n ( jh o1 n ) ((pos "Nl%" ) ))
( "John Bull"  n ( jh o2 n - b u1 l ) ((pos "Ki$" ) ))
( "Johnny"  n ( jh o1 n ii ) ((pos "Nl%" ) ))
( "Johnstone"  n ( jh o1 n s t @ n ) ((pos "Nn$" ) ))
( "Jonah"  n ( jh ou1 n @ ) ((pos "Nl%" ) ))
( "Jonathan"  n ( jh o1 n @ th @ n ) ((pos "Nl%" ) ))
( "Jordan"  n ( jh oo1 d n ) ((pos "Nm%" ) ))
( "Jordanian"  n ( jh oo1 d ei1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Jordanians"  n ( jh oo1 d ei1 n i@ n z ) ((pos "Kj%" ) ))
( "Joseph"  n ( jh ou1 z i f ) ((pos "Nl%" ) ))
( "Josephine"  n ( jh ou1 z @ f ii n ) ((pos "Nl%" ) ))
( "Josh"  n ( jh o1 sh ) ((pos "Nl%" ) ))
( "Joshua"  n ( jh o1 sh u@ ) ((pos "Nl%" ) ))
( "Jove"  n ( jh ou1 v ) ((pos "No%" ) ))
( "Joy"  n ( jh oi1 ) ((pos "Nl%" ) ))
( "Joyce"  n ( jh oi1 s ) ((pos "Nl%" ) ))
( "Judaic"  j ( jh uu1 d ei1 i k ) ((pos "OA%" ) ))
( "Judaism"  n ( jh uu1 d ei i z @ m ) ((pos "L@%" ) ))
( "Judas"  n ( jh uu1 d @ s ) ((pos "K7%" "Nl%" ) ))
( "Judases"  n ( jh uu1 d @ s i z ) ((pos "Kj$" ) ))
( "Judith"  n ( jh uu1 d ii th ) ((pos "Nl%" ) ))
( "Judy"  n ( jh uu1 d ii ) ((pos "Nl%" ) ))
( "Jul"  n ( jh @1 l ai1 ) ((pos "Y=%" ) ))
( "Julia"  n ( jh uu1 l i@ ) ((pos "Nl%" ) ))
( "Julian"  n ( jh uu1 l i@ n ) ((pos "Nl%" "OA%" ) ))
( "Julie"  n ( jh uu1 l ii ) ((pos "Nl%" ) ))
( "Juliet"  n ( jh u1 l i e t ) ((pos "Nl%" ) ))
( "July"  n ( jh uu1 l ai1 ) ((pos "Ki%" ) ))
( "Jun"  n ( jh uu1 n ) ((pos "Y=%" ) ))
( "June"  n ( jh uu1 n ) ((pos "K6%" "Nl%" ) ))
( "Junes"  n ( jh uu1 n z ) ((pos "Kj%" ) ))
( "Junoesque"  j ( jh uu2 n ou e1 s k ) ((pos "OA$" ) ))
( "Jupiter"  n ( jh uu1 p i t @ r ) ((pos "No%" ) ))
( "Justin"  n ( jh uh1 s t i n ) ((pos "Nl%" ) ))
( "K"  n ( k ei1 ) ((pos "Ki$" ) ))
( "K's"  n ( k ei1 z ) ((pos "Kj$" ) ))
( "KGB"  n ( k ei2 jh ii2 b ii1 ) ((pos "Y>%" ) ))
( "KO"  n ( k ei2 ou1 ) ((pos "Y>%" ) ))
( "Kaiser"  n ( k ai1 z @ r ) ((pos "K6%" ) ))
( "Kaisers"  n ( k ai1 z @ z ) ((pos "Kj%" ) ))
( "Kalgan"  n ( k aa2 l g aa1 n ) ((pos "Nn$" ) ))
( "Kampuchea"  n ( k a1 m p u ch i@1 ) ((pos "Nm%" ) ))
( "Kampuchean"  n ( k a1 m p u ch i@1 n ) ((pos "K6%" "OA%" ) ))
( "Kampucheans"  n ( k a1 m p u ch i@1 n z ) ((pos "Kj%" ) ))
( "Kanpur"  n ( k aa2 n p u@1 r ) ((pos "Nn%" ) ))
( "Kansas"  n ( k a1 n z @ s ) ((pos "Nm%" ) ))
( "Kansas City"  n ( k a2 n z @ s - s i1 t ii ) ((pos "Nn%" ) ))
( "Kaohsiung"  n ( k au2 sh y u1 ng ) ((pos "Nn$" ) ))
( "Karachi"  n ( k @1 r aa1 ch ii ) ((pos "Nn%" ) ))
( "Karaganda"  n ( k a2 r @ g a1 n d @ ) ((pos "Nn$" ) ))
( "Karen"  n ( k a1 r @ n ) ((pos "Nl%" ) ))
( "Karlsruhe"  n ( k aa1 l z r u@ ) ((pos "Nn%" ) ))
( "Kashmir"  n ( k a1 sh m i@1 r ) ((pos "Nm%" ) ))
( "Kashmiri"  n ( k a1 sh m i@1 r ii ) ((pos "K6%" "OA%" ) ))
( "Kashmiris"  n ( k a1 sh m i@1 r i z ) ((pos "Kj%" ) ))
( "Kassel"  n ( k a1 s @ l ) ((pos "Nn$" ) ))
( "Kate"  n ( k ei1 t ) ((pos "Nl%" ) ))
( "Kath"  n ( k a1 th ) ((pos "Nl%" ) ))
( "Katherine"  n ( k a1 th r i n ) ((pos "Nl%" ) ))
( "Kathy"  n ( k a1 th ii ) ((pos "Nl%" ) ))
( "Katie"  n ( k ei1 t ii ) ((pos "Nl%" ) ))
( "Katowice"  n ( k a2 t @ v ii1 ch @ ) ((pos "Nn$" ) ))
( "Kay"  n ( k ei1 ) ((pos "Nl%" ) ))
( "Kazan"  n ( k @1 z a1 n ) ((pos "Nn$" ) ))
( "Keighley"  n ( k ii1 th l ii ) ((pos "Nn%" ) ))
( "Keith"  n ( k ii1 th ) ((pos "Nl%" ) ))
( "Kelso"  n ( k e1 l s ou ) ((pos "Nn%" ) ))
( "Kelt"  n ( k e1 l t ) ((pos "K6$" ) ))
( "Kelts"  n ( k e1 l t s ) ((pos "Kj$" ) ))
( "Kelty"  n ( k e1 l t ii ) ((pos "Nn$" ) ))
( "Kemsing"  n ( k e1 m z i ng ) ((pos "Nn$" ) ))
( "Ken"  n ( k e1 n ) ((pos "Nl%" ) ))
( "Kendal"  n ( k e1 n d l ) ((pos "Nn%" ) ))
( "Kenilworth"  n ( k e1 n l w @ th ) ((pos "Nn%" ) ))
( "Kenneth"  n ( k e1 n i th ) ((pos "Nl%" ) ))
( "Kenny"  n ( k e1 n ii ) ((pos "Nl%" ) ))
( "Kensington"  n ( k e1 n z i ng t @ n ) ((pos "Nn%" ) ))
( "Kent"  n ( k e1 n t ) ((pos "Nm%" ) ))
( "Kentucky"  n ( k e1 n t uh1 k ii ) ((pos "Nm%" ) ))
( "Kenya"  n ( k e1 n y @ ) ((pos "Nm%" ) ))
( "Kenyan"  n ( k e1 n y @ n ) ((pos "K6%" "OA%" ) ))
( "Kenyans"  n ( k e1 n y @ n z ) ((pos "Kj%" ) ))
( "Kerry"  n ( k e1 r ii ) ((pos "Nm%" ) ))
( "Kettering"  n ( k e1 t @ r i ng ) ((pos "Nn%" ) ))
( "Kevin"  n ( k e1 v i n ) ((pos "Nl%" ) ))
( "Keynsham"  n ( k ei1 n sh @ m ) ((pos "Nn$" ) ))
( "Keyworth"  n ( k ii1 w @ th ) ((pos "Nn$" ) ))
( "Khalka"  n ( k a1 l k @ ) ((pos "L@$" ) ))
( "Kharkov"  n ( k aa1 k o f ) ((pos "Nn$" ) ))
( "Khartoum"  n ( k aa2 t uu1 m ) ((pos "Nn%" ) ))
( "Khmer"  n ( k m e@1 r ) ((pos "L@$" ) ))
( "Kidderminster"  n ( k i1 d @ m i n s t @ r ) ((pos "Nn%" ) ))
( "Kidlington"  n ( k i1 d l i ng t @ n ) ((pos "Nn%" ) ))
( "Kidsgrove"  n ( k i1 d z g r ou v ) ((pos "Nn%" ) ))
( "Kiev"  n ( k ii1 e f ) ((pos "Nn%" ) ))
( "Kilbirnie"  n ( k i1 l b @@1 n ii ) ((pos "Nn$" ) ))
( "Kildare"  n ( k i1 l d e@1 r ) ((pos "Nm$" ) ))
( "Kilkenny"  n ( k i1 l k e1 n ii ) ((pos "Nm%" ) ))
( "Killarney"  n ( k i1 l aa1 n ii ) ((pos "Nn%" ) ))
( "Kilmarnock"  n ( k i1 l m aa1 n @ k ) ((pos "Nn%" ) ))
( "Kilsyth"  n ( k i1 l s ai1 th ) ((pos "Nn$" ) ))
( "Kilwinning"  n ( k i1 l w i1 n i ng ) ((pos "Nn$" ) ))
( "Kimberley"  n ( k i1 m b @ l ii ) ((pos "Nn%" ) ))
( "King's Lynn"  n ( k i2 ng z - l i1 n ) ((pos "Nn%" ) ))
( "Kingsbury"  n ( k i1 ng z b @ r ii ) ((pos "Nn$" ) ))
( "Kingsteignton"  n ( k i1 ng s t ei1 n t n ) ((pos "Nn$" ) ))
( "Kingston"  n ( k i1 ng s t @ n ) ((pos "Nn%" ) ))
( "Kingston upon Hull"  n ( k i2 ng s t @ n - uh p o n - h uh1 l ) ((pos "Nn%" ) ))
( "Kingston upon Thames"  n ( k i2 ng s t @ n - uh p o n - t e1 m z ) ((pos "Nn%" ) ))
( "Kinshasa"  n ( k i1 n sh aa1 s @ ) ((pos "Nn%" ) ))
( "Kippax"  n ( k i1 p a k s ) ((pos "Nn$" ) ))
( "Kirin"  n ( k i2 r i1 n ) ((pos "Nn$" ) ))
( "Kirkby"  n ( k @@1 b ii ) ((pos "Nn%" ) ))
( "Kirkby in Ashfield"  n ( k @@2 b i - i n - a1 sh f ii l d ) ((pos "Nn$" ) ))
( "Kirkcaldy"  n ( k @1 k oo1 d ii ) ((pos "Nn%" ) ))
( "Kirkham"  n ( k @@1 k @ m ) ((pos "Nn$" ) ))
( "Kirkintilloch"  n ( k @@2 k i n t i1 l @ k ) ((pos "Nn$" ) ))
( "Kirkwall"  n ( k @@1 k w oo l ) ((pos "Nn$" ) ))
( "Kitakyushu"  n ( k i2 t @ k y uu1 sh uu ) ((pos "Nn$" ) ))
( "Kitty"  n ( k i1 t ii ) ((pos "Nl%" ) ))
( "Knaresborough"  n ( n e@1 z b r @ ) ((pos "Nn%" ) ))
( "Knesset"  n ( k n e1 s e t ) ((pos "K6%" ) ))
( "Knessets"  n ( k n e1 s e t s ) ((pos "Kj$" ) ))
( "Knottingley"  n ( n o1 t i ng l ii ) ((pos "Nn$" ) ))
( "Knutsford"  n ( n uh1 t s f @ d ) ((pos "Nn%" ) ))
( "Kobe"  n ( k ou1 b ii ) ((pos "Nn$" ) ))
( "Koblenz"  n ( k @1 b l e1 n t s ) ((pos "Nn%" ) ))
( "Koln"  n ( k @@1 l n ) ((pos "Nn%" ) ))
( "Koran"  n ( k @1 r aa1 n ) ((pos "K6%" ) ))
( "Koranic"  j ( k @1 r a1 n i k ) ((pos "OA%" ) ))
( "Korans"  n ( k @1 r aa1 n z ) ((pos "Kj$" ) ))
( "Korea"  n ( k @1 r i@1 ) ((pos "Nm%" ) ))
( "Korean"  n ( k @1 r i@1 n ) ((pos "M6%" "OA%" ) ))
( "Koreans"  n ( k @1 r i@1 n z ) ((pos "Mj%" ) ))
( "Krakow"  n ( k r aa1 k o f ) ((pos "Nn%" ) ))
( "Krasnoyarsk"  n ( k r a2 z n @ y aa1 s k ) ((pos "Nn$" ) ))
( "Krefeld"  n ( k r ei1 f e l t ) ((pos "Nn%" ) ))
( "Kremlin"  n ( k r e1 m l i n ) ((pos "K6%" ) ))
( "Kremlins"  n ( k r e1 m l i n z ) ((pos "Kj$" ) ))
( "Krivoi Rog"  n ( k r i1 v oi2 - r o1 k ) ((pos "Nn$" ) ))
( "Kunming"  n ( k u2 n m i1 ng ) ((pos "Nn$" ) ))
( "Kuwait"  n ( k u1 w ei1 t ) ((pos "Nm%" ) ))
( "Kuwait City"  n ( k u2 w ei t - s i1 t ii ) ((pos "Nn%" ) ))
( "Kuwaiti"  n ( k u1 w ei1 t ii ) ((pos "K6%" "OA%" ) ))
( "Kuwaitis"  n ( k u1 w ei1 t i z ) ((pos "Kj%" ) ))
( "Kuybyshev"  n ( k u1 b i sh e f ) ((pos "Nn$" ) ))
( "Kweiyang"  n ( k w ei2 y aa1 ng ) ((pos "Nn$" ) ))
( "Kyoto"  n ( k i1 ou1 t ou ) ((pos "Nn%" ) ))
( "L"  n ( e1 l ) ((pos "Ki$" ) ))
( "L's"  n ( e1 l z ) ((pos "Kj$" ) ))
( "L-plate"  n ( e1 l - p l ei t ) ((pos "K6%" ) ))
( "L-plates"  n ( e1 l - p l ei t s ) ((pos "Kj%" ) ))
( "LA"  n ( e2 l ei1 ) ((pos "Y=%" ) ))
( "LEA"  n ( e2 l ii2 ei1 ) ((pos "Y>%" ) ))
( "LLB"  n ( e2 l e2 l b ii1 ) ((pos "Y>%" ) ))
( "LP"  n ( e2 l p ii1 ) ((pos "Y>%" ) ))
( "LPs"  n ( e2 l p ii1 z ) ((pos "Y)%" ) ))
( "LSD"  n ( e2 l e2 s d ii1 ) ((pos "Y}%" ) ))
( "La Habana"  n ( l a1 - h @ b aa1 n @ ) ((pos "Nn$" ) ))
( "La Paz"  n ( l a1 - p a1 z ) ((pos "Nn%" ) ))
( "La Plata"  n ( l a1 - p l aa1 t @ ) ((pos "Nn%" ) ))
( "La Spezia"  n ( l a1 - s p e1 t s i@ ) ((pos "Nn%" ) ))
( "Lab"  n ( l a1 b ) ((pos "Y~%" ) ))
( "Labourite"  n ( l ei1 b @ r ai t ) ((pos "K6%" ) ))
( "Labourites"  n ( l ei1 b @ r ai t s ) ((pos "Kj%" ) ))
( "Labrador"  n ( l a1 b r @ d oo r ) ((pos "Nm%" ) ))
( "Lady-chapel"  n ( l ei1 d i - ch a p l ) ((pos "K6%" ) ))
( "Lady-chapels"  n ( l ei1 d i - ch a p l z ) ((pos "Kj%" ) ))
( "Lagos"  n ( l ei1 g o s ) ((pos "Nn%" ) ))
( "Lahore"  n ( l @1 h oo1 r ) ((pos "Nn%" ) ))
( "Lakenheath"  n ( l ei1 k n h ii th ) ((pos "Nn$" ) ))
( "Lambeth"  n ( l a1 m b @ th ) ((pos "Nn%" ) ))
( "Lammas"  n ( l a1 m @ s ) ((pos "K7$" "No$" ) ))
( "Lammases"  n ( l a1 m @ s i z ) ((pos "Kj$" ) ))
( "Lanark"  n ( l a1 n @ k ) ((pos "Nn%" ) ))
( "Lancashire"  n ( l a1 ng k @ sh @ r ) ((pos "Nm%" ) ))
( "Lancaster"  n ( l a1 ng k i s t @ r ) ((pos "Nn%" ) ))
( "Lancastrian"  n ( l a1 ng k a1 s t r i@ n ) ((pos "K6%" "OA%" ) ))
( "Lancastrians"  n ( l a1 ng k a1 s t r i@ n z ) ((pos "Kj%" ) ))
( "Lanchow"  n ( l a2 n ch au1 ) ((pos "Nn$" ) ))
( "Lancing"  n ( l aa1 n s i ng ) ((pos "Nn%" ) ))
( "Landrover"  n ( l a1 n d r ou v @ r ) ((pos "K6%" ) ))
( "Landrovers"  n ( l a1 n d r ou v @ z ) ((pos "Kj%" ) ))
( "Lao"  n ( l a1 ou ) ((pos "L@$" ) ))
( "Laos"  n ( l aa1 o s ) ((pos "Nm%" ) ))
( "Laotian"  n ( l aa1 o sh @ n ) ((pos "M6%" "OA%" ) ))
( "Laotians"  n ( l aa1 o sh @ n z ) ((pos "Mj%" ) ))
( "Largs"  n ( l aa1 g z ) ((pos "Nn$" ) ))
( "Larkhall"  n ( l aa1 k h oo l ) ((pos "Nn$" ) ))
( "Larne"  n ( l aa1 n ) ((pos "Nn$" ) ))
( "Larry"  n ( l a1 r ii ) ((pos "Nl%" ) ))
( "Las Palmas"  n ( l a1 s - p a1 l m @ s ) ((pos "Nn%" ) ))
( "Latin"  n ( l a1 t i n ) ((pos "K6%" "OA%" ) ))
( "Latinist"  n ( l a1 t i n i s t ) ((pos "K6%" ) ))
( "Latinists"  n ( l a1 t i n i s t s ) ((pos "Kj$" ) ))
( "Latinize"  v ( l a1 t i n ai z ) ((pos "H2%" ) ))
( "Latinized"  v ( l a1 t i n ai z d ) ((pos "Hc%" "Hd%" ) ))
( "Latinizes"  v ( l a1 t i n ai z i z ) ((pos "Ha%" ) ))
( "Latinizing"  v ( l a1 t i n ai z i ng ) ((pos "Hb%" ) ))
( "Latins"  n ( l a1 t i n z ) ((pos "Kj%" ) ))
( "Laura"  n ( l oo1 r @ ) ((pos "Nl%" ) ))
( "Laurence"  n ( l o1 r @ n s ) ((pos "Nl%" ) ))
( "Lausanne"  n ( l ou1 z a1 n ) ((pos "Nn%" ) ))
( "Lawrence"  n ( l o1 r @ n s ) ((pos "Nl%" ) ))
( "Lazarus"  n ( l a1 z @ r @ s ) ((pos "Ki%" "Nl%" ) ))
( "Lazaruses"  n ( l a1 z @ r @ s i z ) ((pos "Kj$" ) ))
( "Le Havre"  n ( l @1 - h aa1 v r @ ) ((pos "Nn%" ) ))
( "Le Mans"  n ( l @1 - m o1 ng ) ((pos "Nn%" ) ))
( "Leamington Spa"  n ( l e2 m i ng t @ n - s p aa1 ) ((pos "Nn%" ) ))
( "Leatherhead"  n ( l e1 dh @ h e d ) ((pos "Nn%" ) ))
( "Lebanese"  n ( l e2 b @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Lebanon"  n ( l e1 b @ n @ n ) ((pos "Nm%" ) ))
( "Leeds"  n ( l ii1 d z ) ((pos "Nn%" ) ))
( "Leek"  n ( l ii1 k ) ((pos "Nn$" ) ))
( "Leicester"  n ( l e1 s t @ r ) ((pos "Nn%" ) ))
( "Leicestershire"  n ( l e1 s t @ sh @ r ) ((pos "Nm%" ) ))
( "Leiden"  n ( l ai1 d n ) ((pos "Nn%" ) ))
( "Leigh"  n ( l ii1 ) ((pos "Nn%" ) ))
( "Leighton-Linslade"  n ( l ei2 t n - l i1 n z l ei d ) ((pos "Nn$" ) ))
( "Leinster"  n ( l e1 n s t @ r ) ((pos "Nm%" ) ))
( "Leipzig"  n ( l ai1 p z i g ) ((pos "Nn%" ) ))
( "Leitrim"  n ( l ii1 t r i m ) ((pos "Nm$" ) ))
( "Leix"  n ( l ii1 sh ) ((pos "Nm$" ) ))
( "Len"  n ( l e1 n ) ((pos "Nl%" ) ))
( "Leningrad"  n ( l e1 n i n g r a d ) ((pos "Nn%" ) ))
( "Lenny"  n ( l e1 n ii ) ((pos "Nl%" ) ))
( "Lens"  n ( l o1 n s ) ((pos "Nn$" ) ))
( "Lent"  n ( l e1 n t ) ((pos "K6%" "No%" ) ))
( "Lenten"  j ( l e1 n t @ n ) ((pos "OA%" ) ))
( "Lents"  n ( l e1 n t s ) ((pos "Kj$" ) ))
( "Leo"  n ( l ii1 ou ) ((pos "Nl%" ) ))
( "Leominster"  n ( l e1 m s t @ r ) ((pos "Nn$" ) ))
( "Leon"  n ( l ii1 @ n ) ((pos "Nn%" ) ))
( "Leonard"  n ( l e1 n @ d ) ((pos "Nl%" ) ))
( "Lerwick"  n ( l @@1 w i k ) ((pos "Nn%" ) ))
( "Les"  n ( l e1 z ) ((pos "Nl%" ) ))
( "Lesley"  n ( l e1 z l ii ) ((pos "Nl%" ) ))
( "Leslie"  n ( l e1 z l ii ) ((pos "Nl%" ) ))
( "Lesotho"  n ( l @1 s uu1 t uu ) ((pos "Nm%" ) ))
( "Letchworth"  n ( l e1 ch w @ th ) ((pos "Nn%" ) ))
( "Lethe"  n ( l ii1 th ii ) ((pos "No$" ) ))
( "Levant"  n ( l i1 v a1 n t ) ((pos "No%" ) ))
( "Levantine"  n ( l @1 v a1 n t ai n ) ((pos "K6%" "OA%" ) ))
( "Levantines"  n ( l @1 v a1 n t ai n z ) ((pos "Kj%" ) ))
( "Leven"  n ( l e1 v n ) ((pos "Nn$" ) ))
( "Leverkusen"  n ( l ei1 v @ k uu z n ) ((pos "Nn$" ) ))
( "Lewes"  n ( l uu1 i s ) ((pos "Nn%" ) ))
( "Lewis"  n ( l uu1 i s ) ((pos "Nl%" ) ))
( "Lewisham"  n ( l uu1 i sh @ m ) ((pos "Nn%" ) ))
( "Leyland"  n ( l ei1 l @ n d ) ((pos "Nn%" ) ))
( "Lib"  n ( l i1 b ) ((pos "Y~%" ) ))
( "Liberia"  n ( l ai1 b i@1 r i@ ) ((pos "Nm%" ) ))
( "Liberian"  n ( l ai1 b i@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Liberians"  n ( l ai1 b i@1 r i@ n z ) ((pos "Kj%" ) ))
( "Libra"  n ( l ii1 b r @ ) ((pos "No%" ) ))
( "Libya"  n ( l i1 b i@ ) ((pos "Nm%" ) ))
( "Libyan"  n ( l i1 b i@ n ) ((pos "K6%" "OA%" ) ))
( "Libyans"  n ( l i1 b i@ n z ) ((pos "Kj%" ) ))
( "Lichfield"  n ( l i1 ch f ii l d ) ((pos "Nn%" ) ))
( "Liechtenstein"  n ( l i1 k t @ n s t ai n ) ((pos "Nm%" ) ))
( "Liechtensteiner"  n ( l i1 k t @ n s t ai n @ r ) ((pos "K6%" "OA%" ) ))
( "Liechtensteiners"  n ( l i1 k t @ n s t ai n @ z ) ((pos "Kj%" ) ))
( "Liege"  n ( l i1 ei1 zh ) ((pos "Nn%" ) ))
( "Lilian"  n ( l i1 l i@ n ) ((pos "Nl%" ) ))
( "Lille"  n ( l ii1 l ) ((pos "Nn%" ) ))
( "Lilliputian"  n ( l i2 l i p y uu1 sh @ n ) ((pos "K6$" "OA$" ) ))
( "Lilliputians"  n ( l i2 l i p y uu1 sh @ n z ) ((pos "Kj$" ) ))
( "Lily"  n ( l i1 l ii ) ((pos "Nl%" ) ))
( "Lima"  n ( l ii1 m @ ) ((pos "Nn%" ) ))
( "Limerick"  n ( l i1 m @ r i k ) ((pos "Nn%" ) ))
( "Limoges"  n ( l i1 m ou1 zh ) ((pos "Nn%" ) ))
( "Lincoln"  n ( l i1 ng k @ n ) ((pos "Nn%" ) ))
( "Lincolnshire"  n ( l i1 ng k @ n sh @ r ) ((pos "Nm%" ) ))
( "Linda"  n ( l i1 n d @ ) ((pos "Nl%" ) ))
( "Linlithgow"  n ( l i1 n l i1 th g ou ) ((pos "Nn$" ) ))
( "Linwood"  n ( l i1 n w u d ) ((pos "Nn$" ) ))
( "Linz"  n ( l i1 n t s ) ((pos "Nn$" ) ))
( "Lionel"  n ( l ai1 @ n l ) ((pos "Nl%" ) ))
( "Lisa"  n ( l ii1 s @ ) ((pos "Nl%" ) ))
( "Lisbon"  n ( l i1 z b @ n ) ((pos "Nn%" ) ))
( "Liskeard"  n ( l i1 s k aa1 d ) ((pos "Nn$" ) ))
( "Littlehampton"  n ( l i1 t l h a m t @ n ) ((pos "Nn%" ) ))
( "Liverpool"  n ( l i1 v @ p uu l ) ((pos "Nn%" ) ))
( "Liverpudlian"  n ( l i2 v @ p uh1 d l i@ n ) ((pos "K6%" "OA%" ) ))
( "Liverpudlians"  n ( l i2 v @ p uh1 d l i@ n z ) ((pos "Kj%" ) ))
( "Livingston"  n ( l i1 v i ng s t @ n ) ((pos "Nn%" ) ))
( "Livorno"  n ( l i1 v oo1 n ou ) ((pos "Nn%" ) ))
( "Liz"  n ( l i1 z ) ((pos "Nl%" ) ))
( "Liza"  n ( l ai1 z @ ) ((pos "Nl%" ) ))
( "Lizzy"  n ( l i1 z ii ) ((pos "Nl%" ) ))
( "Ljubljana"  n ( l y uu2 b l i aa1 n @ ) ((pos "Nn%" ) ))
( "Llandudno"  n ( l a1 n d i1 d n ou ) ((pos "Nn%" ) ))
( "Llanelli"  n ( l a1 n e1 th l ii ) ((pos "Nn%" ) ))
( "Llangollen"  n ( l a1 n g o1 th l @ n ) ((pos "Nn%" ) ))
( "Llantrisant"  n ( l a1 n t r i1 z n t ) ((pos "Nn$" ) ))
( "Llantwit Major"  n ( l a2 n t w i t - m ei1 jh @ r ) ((pos "Nn$" ) ))
( "Lloyd's"  n ( l oi1 d z ) ((pos "No%" ) ))
( "Loanhead"  n ( l ou1 n h e d ) ((pos "Nn$" ) ))
( "Lochgelly"  n ( l o1 k g e1 l ii ) ((pos "Nn$" ) ))
( "Lodz"  n ( l o1 d z ) ((pos "Nn$" ) ))
( "Loftus"  n ( l o1 f t @ s ) ((pos "Nn$" ) ))
( "Lois"  n ( l ou1 i s ) ((pos "Nl%" ) ))
( "London"  n ( l uh1 n d @ n ) ((pos "Nn%" ) ))
( "Londonderry"  n ( l uh1 n d @ n d e r ii ) ((pos "Nn%" ) ))
( "Londoner"  n ( l uh1 n d @ n @ r ) ((pos "K6%" ) ))
( "Londoners"  n ( l uh1 n d @ n @ z ) ((pos "Kj%" ) ))
( "Long Eaton"  n ( l o2 ng - ii1 t n ) ((pos "Nn$" ) ))
( "Longridge"  n ( l o1 ng r i jh ) ((pos "Nn$" ) ))
( "Longton"  n ( l o1 ng t @ n ) ((pos "Nn$" ) ))
( "Lorna"  n ( l oo1 n @ ) ((pos "Nl%" ) ))
( "Los Angeles"  n ( l o1 s - a1 n jh i l ii z ) ((pos "Nn%" ) ))
( "Lossiemouth"  n ( l o1 s i m @ th ) ((pos "Nn%" ) ))
( "Lothian"  n ( l ou1 dh i@ n ) ((pos "Nm%" ) ))
( "Loughborough"  n ( l uh1 f b r @ ) ((pos "Nn%" ) ))
( "Louis"  n ( l uu1 ii ) ((pos "Nl%" ) ))
( "Louise"  n ( l uu1 ii1 z ) ((pos "Nl%" ) ))
( "Louisiana"  n ( l uu1 ii2 z i a1 n @ ) ((pos "Nm%" ) ))
( "Louisville"  n ( l uu1 i v i l ) ((pos "Nn%" ) ))
( "Louth"  n ( l au1 dh ) ((pos "Nm$" ) ))
( "Lowestoft"  n ( l ou1 i s t o f t ) ((pos "Nn%" ) ))
( "Loyang"  n ( l ou2 y aa1 ng ) ((pos "Nn$" ) ))
( "Lt"  n ( l e2 f t e1 n @ n t ) ((pos "Y:%" ) ))
( "Ltd"  n ( l i1 m i t i d ) ((pos "Y=%" ) ))
( "Lubeck"  n ( l uu1 b e k ) ((pos "Nn%" ) ))
( "Lucifer"  n ( l uu1 s i f @ r ) ((pos "K6%" ) ))
( "Lucifers"  n ( l uu1 s i f @ z ) ((pos "Kj%" ) ))
( "Lucknow"  n ( l uh1 k n au ) ((pos "Nn%" ) ))
( "Lucy"  n ( l uu1 s ii ) ((pos "Nl%" ) ))
( "Luddite"  n ( l uh1 d ai t ) ((pos "K6%" ) ))
( "Luddites"  n ( l uh1 d ai t s ) ((pos "Kj%" ) ))
( "Ludlow"  n ( l uh1 d l ou ) ((pos "Nn%" ) ))
( "Ludwigshafen"  n ( l uh1 d v i g z h aa f n ) ((pos "Nn$" ) ))
( "Luke"  n ( l uu1 k ) ((pos "Nl%" ) ))
( "Luta"  n ( l uu1 t @ ) ((pos "Nn$" ) ))
( "Lutheran"  n ( l uu1 th @ r @ n ) ((pos "K6%" "OA%" ) ))
( "Lutherans"  n ( l uu1 th @ r @ n z ) ((pos "Kj%" ) ))
( "Luton"  n ( l uu1 t n ) ((pos "Nn%" ) ))
( "Luxembourg"  n ( l uh1 k s @ m b @@ g ) ((pos "Nn%" ) ))
( "Luxemburg"  n ( l uh1 k s @ m b @@ g ) ((pos "Nm%" ) ))
( "Luxemburger"  n ( l uh1 k s @ m b @@ g @ r ) ((pos "K6%" "OA%" ) ))
( "Luxemburgers"  n ( l uh1 k s @ m b @@ g @ z ) ((pos "Kj%" ) ))
( "Lvov"  n ( l @1 v o1 f ) ((pos "Nn$" ) ))
( "Lyallpur"  n ( l ai2 @ l p uu1 @ r ) ((pos "Nn$" ) ))
( "Lydia"  n ( l i1 d i@ ) ((pos "Nl%" ) ))
( "Lydney"  n ( l i1 d n ii ) ((pos "Nn$" ) ))
( "Lymington"  n ( l i1 m i ng t @ n ) ((pos "Nn%" ) ))
( "Lymm"  n ( l i1 m ) ((pos "Nn$" ) ))
( "Lynn"  n ( l i1 n ) ((pos "Nl%" ) ))
( "Lynne"  n ( l i1 n ) ((pos "Nl%" ) ))
( "Lyons"  n ( l ai1 @ n z ) ((pos "Nn%" ) ))
( "Lytham"  n ( l i1 dh @ m ) ((pos "Nn%" ) ))
( "Lytham St Annes"  n ( l i2 dh m - s n t - a1 n z ) ((pos "Nn%" ) ))
( "M"  n ( e1 m ) ((pos "Ki$" ) ))
( "M's"  n ( e1 m z ) ((pos "Kj$" ) ))
( "MA"  n ( e2 m ei1 ) ((pos "Y>%" ) ))
( "MB"  n ( e2 m b ii1 ) ((pos "Y>%" ) ))
( "MC"  n ( e2 m s ii1 ) ((pos "Y>%" ) ))
( "MCC"  n ( e2 m s ii2 s ii1 ) ((pos "Y>%" ) ))
( "MD"  n ( e2 m d ii1 ) ((pos "Y>%" ) ))
( "MI5"  n ( e2 m ai2 f ai1 v ) ((pos "Y=%" ) ))
( "MI6"  n ( e2 m ai2 s i1 k s ) ((pos "Y=%" ) ))
( "MP"  n ( e2 m p ii1 ) ((pos "Y>%" ) ))
( "MP's"  n ( e2 m p ii1 z ) ((pos "Y)%" ) ))
( "MS"  n ( m a1 n y @ s k r i p t ) ((pos "Y>%" ) ))
( "MSc"  n ( e2 m e2 s s ii1 ) ((pos "Y>%" ) ))
( "Maastricht"  n ( m aa1 s t r i k t ) ((pos "Nn%" ) ))
( "Mabel"  n ( m ei1 b l ) ((pos "Nl%" ) ))
( "Macclesfield"  n ( m a1 k l z f ii l d ) ((pos "Nn%" ) ))
( "Macedonian"  n ( m a2 s i d ou1 n i@ n ) ((pos "M6%" "OA%" ) ))
( "Macedonians"  n ( m a2 s i d ou1 n i@ n z ) ((pos "Mj%" ) ))
( "Mach"  n ( m aa1 k ) ((pos "No%" ) ))
( "Madagascan"  n ( m a2 d @ g a1 s k @ n ) ((pos "K6%" "OA%" ) ))
( "Madagascans"  n ( m a2 d @ g a1 s k @ n z ) ((pos "Kj%" ) ))
( "Madagascar"  n ( m a2 d @ g a1 s k @ r ) ((pos "Nm%" ) ))
( "Madame"  n ( m @1 d aa1 m ) ((pos "K6%" ) ))
( "Madames"  n ( m @1 d aa1 m z ) ((pos "Kj$" ) ))
( "Maddy"  n ( m a1 d ii ) ((pos "Nl%" ) ))
( "Madeira"  n ( m @1 d i@1 r @ ) ((pos "K6%" ) ))
( "Madeiras"  n ( m @1 d i@1 r @ z ) ((pos "Kj%" ) ))
( "Madeleine"  n ( m a1 d @ l i n ) ((pos "Nl%" ) ))
( "Mademoiselle"  n ( m a2 d m w @ z e1 l ) ((pos "K6%" ) ))
( "Mademoiselles"  n ( m a2 d m w @ z e1 l z ) ((pos "Kj%" ) ))
( "Madge"  n ( m a1 jh ) ((pos "Nl%" ) ))
( "Madonna"  n ( m @1 d o1 n @ ) ((pos "K6%" ) ))
( "Madonnas"  n ( m @1 d o1 n @ z ) ((pos "Kj%" ) ))
( "Madras"  n ( m @1 d r aa1 s ) ((pos "Nn%" ) ))
( "Madrid"  n ( m @1 d r i1 d ) ((pos "Nn%" ) ))
( "Madurai"  n ( m a1 d y uh r ai ) ((pos "Nn$" ) ))
( "Maecenas"  n ( m ai1 s ii1 n @ s ) ((pos "No$" ) ))
( "Maesteg"  n ( m ai1 s t e g ) ((pos "Nn%" ) ))
( "Mafia"  n ( m a1 f i@ ) ((pos "K6%" ) ))
( "Mafias"  n ( m a1 f i@ z ) ((pos "Kj$" ) ))
( "Magdeburg"  n ( m a1 g d i b @@ g ) ((pos "Nn$" ) ))
( "Maggie"  n ( m a1 g ii ) ((pos "Nl%" ) ))
( "Maghull"  n ( m @1 g uh1 l ) ((pos "Nn%" ) ))
( "Magi"  n ( m ei1 jh ai ) ((pos "Kj%" ) ))
( "Magnificat"  n ( m a1 g n i1 f i k a t ) ((pos "K6%" ) ))
( "Magnificats"  n ( m a1 g n i1 f i k a t s ) ((pos "Kj$" ) ))
( "Magyar"  n ( m a1 g y aa r ) ((pos "M6%" "OA%" ) ))
( "Magyars"  n ( m a1 g y aa z ) ((pos "Mj%" ) ))
( "Maharaja"  n ( m aa2 h @ r aa1 jh @ ) ((pos "K6%" ) ))
( "Maharajah"  n ( m aa2 h @ r aa1 jh @ ) ((pos "K6%" ) ))
( "Maharajahs"  n ( m aa2 h @ r aa1 jh @ z ) ((pos "Kj%" ) ))
( "Maharajas"  n ( m aa2 h @ r aa1 jh @ z ) ((pos "Kj%" ) ))
( "Maharanee"  n ( m aa2 h @ r aa1 n ii ) ((pos "K6$" ) ))
( "Maharanees"  n ( m aa2 h @ r aa1 n ii z ) ((pos "Kj$" ) ))
( "Mahatma"  n ( m @1 h a1 t m @ ) ((pos "K6%" ) ))
( "Mahatmas"  n ( m @1 h a1 t m @ z ) ((pos "Kj$" ) ))
( "Maidenhead"  n ( m ei1 d n h e d ) ((pos "Nn%" ) ))
( "Maidstone"  n ( m ei1 d s t @ n ) ((pos "Nn%" ) ))
( "Maine"  n ( m ei1 n ) ((pos "Nm%" ) ))
( "Mainz"  n ( m ei1 n t s ) ((pos "Nn%" ) ))
( "Maj"  n ( m ei1 jh @ r ) ((pos "Y:%" ) ))
( "Malaga"  n ( m a1 l @ g @ ) ((pos "Nn%" ) ))
( "Malawi"  n ( m @1 l aa1 w ii ) ((pos "Nm%" ) ))
( "Malawian"  n ( m @1 l aa1 w i@ n ) ((pos "K6%" "OA%" ) ))
( "Malawians"  n ( m @1 l aa1 w i@ n z ) ((pos "Kj%" ) ))
( "Malay"  n ( m @1 l ei1 ) ((pos "M6%" "OA%" ) ))
( "Malaya"  n ( m @1 l ei1 @ ) ((pos "Nm%" ) ))
( "Malayan"  n ( m @1 l ei1 @ n ) ((pos "K6%" "OA%" ) ))
( "Malayans"  n ( m @1 l ei1 @ n z ) ((pos "Kj%" ) ))
( "Malays"  n ( m @1 l ei1 z ) ((pos "Mj%" ) ))
( "Malaysia"  n ( m @1 l ei1 z i@ ) ((pos "Nm%" ) ))
( "Malaysian"  n ( m @1 l ei1 z i@ n ) ((pos "K6%" "OA%" ) ))
( "Malaysians"  n ( m @1 l ei1 z i@ n z ) ((pos "Kj%" ) ))
( "Malcolm"  n ( m a1 l k @ m ) ((pos "Nl%" ) ))
( "Maldon"  n ( m o1 l d n ) ((pos "Nn%" ) ))
( "Mali"  n ( m aa1 l ii ) ((pos "Nm%" ) ))
( "Malian"  n ( m aa1 l i@ n ) ((pos "K6$" "OA$" ) ))
( "Malians"  n ( m aa1 l i@ n z ) ((pos "Kj$" ) ))
( "Mallow"  n ( m a1 l ou ) ((pos "Nn$" ) ))
( "Malmo"  n ( m a1 l m ou ) ((pos "Nn%" ) ))
( "Malta"  n ( m oo1 l t @ ) ((pos "Nm%" ) ))
( "Maltby"  n ( m o1 l t b ii ) ((pos "Nn$" ) ))
( "Maltese"  n ( m oo1 l t ii1 z ) ((pos "M9%" "OA%" ) ))
( "Malthusian"  j ( m a1 l th y uu1 z i@ n ) ((pos "OA$" ) ))
( "Malvern"  n ( m o1 l v n ) ((pos "Nn%" ) ))
( "Mamie"  n ( m ei1 m ii ) ((pos "Nl$" ) ))
( "Manchester"  n ( m a1 n ch i s t @ r ) ((pos "Nn%" ) ))
( "Mancunian"  n ( m a1 n k y uu1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Mancunians"  n ( m a1 n k y uu1 n i@ n z ) ((pos "Kj%" ) ))
( "Mandy"  n ( m a1 n d ii ) ((pos "Nl%" ) ))
( "Manila"  n ( m @1 n i1 l @ ) ((pos "Nn%" ) ))
( "Manilla"  n ( m @1 n i1 l @ ) ((pos "K6%" ) ))
( "Manillas"  n ( m @1 n i1 l @ z ) ((pos "Kj$" ) ))
( "Manitoba"  n ( m a2 n i t ou1 b @ ) ((pos "Nm%" ) ))
( "Mannheim"  n ( m a1 n h ai m ) ((pos "Nn%" ) ))
( "Mansfield"  n ( m a1 n s f ii l d ) ((pos "Nn%" ) ))
( "Manx"  n ( m a1 ng k s ) ((pos "L@%" "OA%" ) ))
( "Maoism"  n ( m au1 i z @ m ) ((pos "L@%" ) ))
( "Maoist"  n ( m au1 i s t ) ((pos "K6%" ) ))
( "Maoists"  n ( m au1 i s t s ) ((pos "Kj%" ) ))
( "Maori"  n ( m au1 r ii ) ((pos "K6%" ) ))
( "Maoris"  n ( m au1 r i z ) ((pos "Kj%" ) ))
( "Mar"  n ( m aa1 ch ) ((pos "Y=%" ) ))
( "Maracaibo"  n ( m a2 r @ k ai1 b ou ) ((pos "Nn$" ) ))
( "March"  n ( m aa1 ch ) ((pos "K7%" "Nn%" ) ))
( "Marches"  n ( m aa1 ch i z ) ((pos "Kj$" ) ))
( "Mardi Gras"  n ( m aa2 d i - g r aa1 ) ((pos "K9%" ) ))
( "Margaret"  n ( m aa1 g r i t ) ((pos "Nl%" ) ))
( "Margate"  n ( m aa1 g ei t ) ((pos "Nn%" ) ))
( "Marge"  n ( m aa1 jh ) ((pos "Nl%" ) ))
( "Margery"  n ( m aa1 jh @ r ii ) ((pos "Nl%" ) ))
( "Margie"  n ( m aa1 jh ii ) ((pos "Nl%" ) ))
( "Margrave"  n ( m aa1 g r ei v ) ((pos "K6$" ) ))
( "Margraves"  n ( m aa1 g r ei v z ) ((pos "Kj$" ) ))
( "Maria"  n ( m @1 r i@1 ) ((pos "Nl%" ) ))
( "Marian"  n ( m a1 r i@ n ) ((pos "Nl%" ) ))
( "Marie"  n ( m @1 r ii1 ) ((pos "Nl%" ) ))
( "Marilyn"  n ( m a1 r i l i n ) ((pos "Nl%" ) ))
( "Marion"  n ( m a1 r i@ n ) ((pos "Nl%" ) ))
( "Marjorie"  n ( m aa1 jh @ r ii ) ((pos "Nl%" ) ))
( "Mark"  n ( m aa1 k ) ((pos "Nl%" ) ))
( "Market Drayton"  n ( m aa2 k i t - d r ei1 t n ) ((pos "Nn%" ) ))
( "Market Harborough"  n ( m aa2 k i t - h aa1 b r @ ) ((pos "Nn%" ) ))
( "Marlborough"  n ( m aa1 l b r @ ) ((pos "Nn%" ) ))
( "Marlene"  n ( m aa1 l ii1 n ) ((pos "Nl%" ) ))
( "Marlow"  n ( m aa1 l ou ) ((pos "Nn%" ) ))
( "Marple"  n ( m aa1 p l ) ((pos "Nn$" ) ))
( "Mars"  n ( m aa1 z ) ((pos "No%" ) ))
( "Marsala"  n ( m aa1 s aa1 l @ ) ((pos "L@$" ) ))
( "Marseillaise"  n ( m aa2 s @ l ei1 z ) ((pos "K6%" ) ))
( "Marseillaises"  n ( m aa2 s @ l ei1 z i z ) ((pos "Kj$" ) ))
( "Marseilles"  n ( m aa1 s ei1 l z ) ((pos "Nn%" ) ))
( "Marske"  n ( m aa1 s k ) ((pos "Nn$" ) ))
( "Martha"  n ( m aa1 th @ ) ((pos "Nl%" ) ))
( "Martian"  n ( m aa1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Martians"  n ( m aa1 sh @ n z ) ((pos "Kj%" ) ))
( "Martin"  n ( m aa1 t i n ) ((pos "Nl%" ) ))
( "Marxism"  n ( m aa1 k s i z @ m ) ((pos "L@%" ) ))
( "Marxist"  n ( m aa1 k s i s t ) ((pos "K6%" ) ))
( "Marxists"  n ( m aa1 k s i s t s ) ((pos "Kj%" ) ))
( "Mary"  n ( m e@1 r ii ) ((pos "Nl%" ) ))
( "Maryland"  n ( m e@1 r i l @ n d ) ((pos "Nm%" ) ))
( "Maryport"  n ( m e@1 r i p oo t ) ((pos "Nn$" ) ))
( "Mason-Dixon"  n ( m ei2 s n - d i1 k s n ) ((pos "No$" ) ))
( "Mass"  n ( m a1 s ) ((pos "M7%" ) ))
( "Massachusetts"  n ( m a2 s @ ch uu1 s i t s ) ((pos "Nm%" ) ))
( "Masses"  n ( m a1 s i z ) ((pos "Mj%" ) ))
( "Matlock"  n ( m a1 t l o k ) ((pos "Nn%" ) ))
( "Matt"  n ( m a1 t ) ((pos "Nl%" ) ))
( "Matthew"  n ( m a1 th y uu ) ((pos "Nl%" ) ))
( "Maud"  n ( m oo1 d ) ((pos "Nl%" ) ))
( "Maudie"  n ( m oo1 d ii ) ((pos "Nl%" ) ))
( "Maundy Thursday"  n ( m oo2 n d i - th @@1 z d ii ) ((pos "K6%" "No%" ) ))
( "Maundy Thursdays"  n ( m oo2 n d i - th @@1 z d i z ) ((pos "Kj$" ) ))
( "Maureen"  n ( m oo1 r ii n ) ((pos "Nl%" ) ))
( "Maurice"  n ( m o1 r i s ) ((pos "Nl%" ) ))
( "Mauritania"  n ( m o2 r i t ei1 n i@ ) ((pos "Nm%" ) ))
( "Mauritanian"  n ( m o2 r i t ei1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Mauritanians"  n ( m o2 r i t ei1 n i@ n z ) ((pos "Kj%" ) ))
( "Mauritian"  n ( m @1 r i1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Mauritians"  n ( m @1 r i1 sh @ n z ) ((pos "Kj%" ) ))
( "Mauritius"  n ( m @1 r i1 sh @ s ) ((pos "Nm%" ) ))
( "Mavis"  n ( m ei1 v i s ) ((pos "Nl%" ) ))
( "Max"  n ( m a1 k s ) ((pos "Nl%" ) ))
( "Maxine"  n ( m a1 k s ii n ) ((pos "Nl%" ) ))
( "May"  n ( m ei1 ) ((pos "K6%" "Nl%" ) ))
( "Mayfair"  n ( m ei1 f e@ r ) ((pos "No%" ) ))
( "Mayfield"  n ( m ei1 f ii l d ) ((pos "Nn$" ) ))
( "Mayo"  n ( m ei1 ou ) ((pos "Nm%" ) ))
( "Mays"  n ( m ei1 z ) ((pos "Kj$" ) ))
( "McCarthyism"  n ( m @1 k aa1 th i i z @ m ) ((pos "L@%" ) ))
( "Meath"  n ( m ii1 th ) ((pos "Nm$" ) ))
( "Mecca"  n ( m e1 k @ ) ((pos "Nn%" ) ))
( "Med"  n ( m e1 d ) ((pos "Y>%" ) ))
( "Medan"  n ( m @1 d aa1 n ) ((pos "Nn$" ) ))
( "Medellin"  n ( m e2 d @ l ii1 n ) ((pos "Nn$" ) ))
( "Medicare"  n ( m e1 d i k e@ r ) ((pos "L@%" ) ))
( "Mediterranean"  j ( m e2 d i t @ r ei1 n i@ n ) ((pos "OA%" ) ))
( "Meg"  n ( m e1 g ) ((pos "Nl%" ) ))
( "Melbourne"  n ( m e1 l b @ n ) ((pos "Nn%" ) ))
( "Melcombe"  n ( m e1 l k @ m ) ((pos "Nn$" ) ))
( "Melcombe Regis"  n ( m e2 l k @ m - r ii1 jh i s ) ((pos "Nn$" ) ))
( "Melksham"  n ( m e1 l k sh @ m ) ((pos "Nn$" ) ))
( "Meltham"  n ( m e1 l th @ m ) ((pos "Nn$" ) ))
( "Melton Mowbray"  n ( m e2 l t n - m ou1 b r ei ) ((pos "Nn%" ) ))
( "Memphis"  n ( m e1 m f i s ) ((pos "Nn%" ) ))
( "Mendelian"  j ( m e1 n d ii1 l i@ n ) ((pos "OA$" ) ))
( "Meopham"  n ( m e1 p @ m ) ((pos "Nn$" ) ))
( "Mephistophelian"  j ( m e2 f i s t @ f ii1 l i@ n ) ((pos "OA%" ) ))
( "Mercator"  n ( m @1 k ei1 t @ r ) ((pos "No%" ) ))
( "Mercury"  n ( m @@1 k y u r ii ) ((pos "No%" ) ))
( "Merthyr Tydfil"  n ( m @@2 th @ - t i1 d v i l ) ((pos "Nn%" ) ))
( "Merton"  n ( m @@1 t n ) ((pos "Nn%" ) ))
( "Mesdames"  n ( m ei1 d aa1 m ) ((pos "Kj%" ) ))
( "Mesdemoiselles"  n ( m ei2 d m w @ z e1 l ) ((pos "Kj%" ) ))
( "Messiah"  n ( m i1 s ai1 @ ) ((pos "K6%" ) ))
( "Messiahs"  n ( m i1 s ai1 @ z ) ((pos "Kj%" ) ))
( "Messieurs"  n ( m ei1 s y @@1 r ) ((pos "Kj%" ) ))
( "Messina"  n ( m e1 s ii1 n @ ) ((pos "Nn%" ) ))
( "Messrs"  n ( m e1 s @ z ) ((pos "Kj%" ) ))
( "Met"  n ( m e1 t ) ((pos "No$" ) ))
( "Methodism"  n ( m e1 th @ d i z @ m ) ((pos "L@%" ) ))
( "Methodist"  n ( m e1 th @ d i s t ) ((pos "K6%" "OA%" ) ))
( "Methodists"  n ( m e1 th @ d i s t s ) ((pos "Kj%" ) ))
( "Methuselah"  n ( m i1 th y uu1 z @ l @ ) ((pos "No%" ) ))
( "Metro"  n ( m e1 t r ou ) ((pos "K6%" ) ))
( "Metros"  n ( m e1 t r ou z ) ((pos "Kj%" ) ))
( "Metz"  n ( m e1 t s ) ((pos "Nn%" ) ))
( "Mexican"  n ( m e1 k s i k @ n ) ((pos "K6%" "OA%" ) ))
( "Mexicans"  n ( m e1 k s i k @ n z ) ((pos "Kj%" ) ))
( "Mexico"  n ( m e1 k s i k ou ) ((pos "Nm%" ) ))
( "Mexico City"  n ( m e2 k s i k ou - s i1 t ii ) ((pos "Nn%" ) ))
( "Mgr"  n ( m o2 n s ii1 n y @ r ) ((pos "Y:%" ) ))
( "Miami"  n ( m ai1 a1 m ii ) ((pos "Nn%" ) ))
( "Michael"  n ( m ai1 k l ) ((pos "Nl%" ) ))
( "Michaelmas"  n ( m i1 k l m @ s ) ((pos "K7%" "No%" ) ))
( "Michaelmases"  n ( m i1 k l m @ s i z ) ((pos "Kj$" ) ))
( "Michelle"  n ( m i1 sh e1 l ) ((pos "Nl%" ) ))
( "Michigan"  n ( m i1 sh i g @ n ) ((pos "Nm%" ) ))
( "Mick"  n ( m i1 k ) ((pos "Nl%" ) ))
( "Mickey"  n ( m i1 k ii ) ((pos "Nl%" ) ))
( "Middlesex"  n ( m i1 d l s e k s ) ((pos "Nm%" ) ))
( "Middleton"  n ( m i1 d l t @ n ) ((pos "Nn%" ) ))
( "Middlewich"  n ( m i1 d l w i ch ) ((pos "Nn$" ) ))
( "Midlands"  n ( m i1 d l @ n d z ) ((pos "Nm%" ) ))
( "Midwest"  n ( m i1 d w e1 s t ) ((pos "No%" ) ))
( "Midwestern"  j ( m i1 d w e1 s t n ) ((pos "OA%" ) ))
( "Mike"  n ( m ai1 k ) ((pos "Nl%" ) ))
( "Milan"  n ( m i1 l a1 n ) ((pos "Nn%" ) ))
( "Milano"  n ( m i1 l aa1 n ou ) ((pos "Nn%" ) ))
( "Mildenhall"  n ( m i1 l d @ n h aa l ) ((pos "Nn$" ) ))
( "Mildred"  n ( m i1 l d r i d ) ((pos "Nl%" ) ))
( "Miles"  n ( m ai1 l z ) ((pos "Nl%" ) ))
( "Milford"  n ( m i1 l f @ d ) ((pos "Nn$" ) ))
( "Milford Haven"  n ( m i2 l f @ d - h ei1 v n ) ((pos "Nn%" ) ))
( "Millicent"  n ( m i1 l i s n t ) ((pos "Nl%" ) ))
( "Millie"  n ( m i1 l ii ) ((pos "Nl%" ) ))
( "Millom"  n ( m i1 l @ m ) ((pos "Nn$" ) ))
( "Milly"  n ( m i1 l ii ) ((pos "Nl%" ) ))
( "Milngavie"  n ( m i1 l g ai1 ) ((pos "Nn%" ) ))
( "Milnrow"  n ( m i1 l n r ou1 ) ((pos "Nn$" ) ))
( "Milton Keynes"  n ( m i2 l t n - k ii1 n z ) ((pos "Nn%" ) ))
( "Milwaukee"  n ( m i1 l w oo1 k ii ) ((pos "Nn%" ) ))
( "Minehead"  n ( m ai1 n h e d ) ((pos "Nn%" ) ))
( "Minneapolis"  n ( m i2 n i a1 p @ l i s ) ((pos "Nn%" ) ))
( "Minnesota"  n ( m i2 n i s ou1 t @ ) ((pos "Nm%" ) ))
( "Minotaur"  n ( m ai1 n @ t oo r ) ((pos "K6%" ) ))
( "Minotaurs"  n ( m ai1 n @ t oo z ) ((pos "Kj$" ) ))
( "Minsk"  n ( m i1 n s k ) ((pos "Nn%" ) ))
( "Minster-in-Sheppey"  n ( m i2 n s t @ r - i n - sh e1 p ii ) ((pos "Nn$" ) ))
( "Miranda"  n ( m i1 r a1 n d @ ) ((pos "Nl%" ) ))
( "Miriam"  n ( m i1 r i@ m ) ((pos "Nl%" ) ))
( "Miss"  n ( m i1 s ) ((pos "K7%" ) ))
( "Misses"  n ( m i1 s i z ) ((pos "Kj%" ) ))
( "Mississippi"  n ( m i2 s i s i1 p ii ) ((pos "Nm%" ) ))
( "Missouri"  n ( m i1 z u@1 r ii ) ((pos "Nm%" ) ))
( "Mlle"  n ( m a1 m z e1 l ) ((pos "Y:$" ) ))
( "Mme"  n ( m a1 d a1 m ) ((pos "Y:$" ) ))
( "Mo"  n ( m ou1 ) ((pos "Nl%" ) ))
( "Mod"  n ( m o1 d ) ((pos "K6$" ) ))
( "Modena"  n ( m @1 d ii1 n @ ) ((pos "Nn%" ) ))
( "Mods"  n ( m o1 d z ) ((pos "Kj$" ) ))
( "Mohammedan"  n ( m @1 h a1 m i d @ n ) ((pos "K6%" ) ))
( "Mohammedans"  n ( m @1 h a1 m i d @ n z ) ((pos "Kj%" ) ))
( "Moira"  n ( m oi1 r @ ) ((pos "Nl%" ) ))
( "Mold"  n ( m ou1 l d ) ((pos "Nn$" ) ))
( "Molly"  n ( m o1 l ii ) ((pos "Nl%" ) ))
( "Moloch"  n ( m ou1 l o k ) ((pos "No$" ) ))
( "Mon"  n ( m uh1 n d ei ) ((pos "Y=%" ) ))
( "Monaco"  n ( m o1 n @ k ou ) ((pos "Nm%" ) ))
( "Monaghan"  n ( m o1 n @ h @ n ) ((pos "Nm%" ) ))
( "Monchengladbach"  n ( m uh2 n ch i n g l a1 d b a k ) ((pos "Nn%" ) ))
( "Monday"  n ( m uh1 n d ii ) ((pos "K6%" "No%" ) ))
( "Mondays"  n ( m uh1 n d i z ) ((pos "Kj%" ) ))
( "Monegasque"  n ( m o2 n @ g a1 s k ) ((pos "K6$" "OA$" ) ))
( "Monegasques"  n ( m o2 n @ g a1 s k s ) ((pos "Kj$" ) ))
( "Mongol"  n ( m o1 ng g l ) ((pos "M6%" "OA%" ) ))
( "Mongolia"  n ( m o1 ng g ou1 l i@ ) ((pos "Nm%" ) ))
( "Mongolian"  n ( m o1 ng g ou1 l i@ n ) ((pos "M6%" "OA%" ) ))
( "Mongolians"  n ( m o1 ng g ou1 l i@ n z ) ((pos "Mj%" ) ))
( "Mongols"  n ( m o1 ng g l z ) ((pos "Mj%" ) ))
( "Monica"  n ( m o1 n i k @ ) ((pos "Nl%" ) ))
( "Monifieth"  n ( m o1 n i f ii th ) ((pos "Nn$" ) ))
( "Monmouth"  n ( m o1 n m @ th ) ((pos "Nn%" ) ))
( "Monsieur"  n ( m @1 s y @@1 r ) ((pos "Ki%" ) ))
( "Monsignor"  n ( m o1 n s ii1 n y @ r ) ((pos "K6%" ) ))
( "Monsignors"  n ( m o1 n s ii1 n y @ z ) ((pos "Kj%" ) ))
( "Montana"  n ( m o1 n t a1 n @ ) ((pos "Nm%" ) ))
( "Monterrey"  n ( m o2 n t @ r ei1 ) ((pos "Nn%" ) ))
( "Montevideo"  n ( m o2 n t i v i d ei1 ou ) ((pos "Nn%" ) ))
( "Montreal"  n ( m o2 n t r i oo1 l ) ((pos "Nn%" ) ))
( "Montrose"  n ( m @1 n t r ou1 z ) ((pos "Nn%" ) ))
( "Montserrat"  n ( m o2 n t s @ r a1 t ) ((pos "Nm$" ) ))
( "Montserratian"  n ( m o2 n t s @ r a1 sh @ n ) ((pos "K6$" "OA$" ) ))
( "Montserratians"  n ( m o2 n t s @ r a1 sh @ n z ) ((pos "Kj$" ) ))
( "Monza"  n ( m o1 n z @ ) ((pos "Nn%" ) ))
( "Moor"  n ( m u@1 r ) ((pos "K6$" ) ))
( "Moorish"  j ( m u@1 r i sh ) ((pos "OA$" ) ))
( "Moors"  n ( m u@1 z ) ((pos "Kj$" ) ))
( "Morecambe"  n ( m oo1 k @ m ) ((pos "Nn%" ) ))
( "Moresque"  j ( m oo1 r e1 s k ) ((pos "OA$" ) ))
( "Morley"  n ( m oo1 l ii ) ((pos "Nn$" ) ))
( "Mormon"  n ( m oo1 m @ n ) ((pos "K6%" "OA%" ) ))
( "Mormonism"  n ( m oo1 m @ n i z @ m ) ((pos "L@%" ) ))
( "Mormons"  n ( m oo1 m @ n z ) ((pos "Kj%" ) ))
( "Moroccan"  n ( m @1 r o1 k @ n ) ((pos "K6%" "OA%" ) ))
( "Moroccans"  n ( m @1 r o1 k @ n z ) ((pos "Kj%" ) ))
( "Morocco"  n ( m @1 r o1 k ou ) ((pos "Nm%" ) ))
( "Morpeth"  n ( m oo1 p i th ) ((pos "Nn$" ) ))
( "Morpheus"  n ( m oo1 f i@ s ) ((pos "No$" ) ))
( "Morse"  n ( m oo1 s ) ((pos "L@%" ) ))
( "Mosaic"  j ( m ou1 z ei1 i k ) ((pos "OA$" ) ))
( "Moscow"  n ( m o1 s k ou ) ((pos "Nn%" ) ))
( "Moslem"  n ( m o1 z l @ m ) ((pos "K6%" "OA%" ) ))
( "Moslems"  n ( m o1 z l @ m z ) ((pos "Kj%" ) ))
( "Mossley"  n ( m o1 s l ii ) ((pos "Nn$" ) ))
( "Motherwell"  n ( m uh1 dh @ w e l ) ((pos "Nn%" ) ))
( "Mountain Ash"  n ( m au2 n t i n - a1 sh ) ((pos "Nn$" ) ))
( "Mountie"  n ( m au1 n t ii ) ((pos "K6%" ) ))
( "Mounties"  n ( m au1 n t i z ) ((pos "Kj%" ) ))
( "Mozambican"  n ( m ou2 z a m b ii1 k @ n ) ((pos "K6%" "OA%" ) ))
( "Mozambicans"  n ( m ou2 z a m b ii1 k @ n z ) ((pos "Kj%" ) ))
( "Mozambique"  n ( m ou2 z a m b ii1 k ) ((pos "Nm%" ) ))
( "Mr"  n ( m i1 s t @ r ) ((pos "Y:*" ) ))
( "Mrs"  n ( m i1 s i z ) ((pos "Y:*" ) ))
( "Ms"  n ( m i1 z ) ((pos "Y:%" ) ))
( "Mt"  n ( m au1 n t ) ((pos "Y=%" ) ))
( "Muhammad"  n ( m @1 h a1 m i d ) ((pos "No%" ) ))
( "Muhammadan"  n ( m @1 h a1 m i d @ n ) ((pos "K6%" "OA%" ) ))
( "Muhammadanism"  n ( m @1 h a1 m i d @ n i z @ m ) ((pos "L@%" ) ))
( "Muhammadans"  n ( m @1 h a1 m i d @ n z ) ((pos "Kj%" ) ))
( "Mukden"  n ( m u1 k d @ n ) ((pos "Nn$" ) ))
( "Mulheim"  n ( m u1 l h ai m ) ((pos "Nn$" ) ))
( "Mulhouse"  n ( m u1 l h au s ) ((pos "Nn$" ) ))
( "Multan"  n ( m u2 l t aa1 n ) ((pos "Nn$" ) ))
( "Munich"  n ( m y uu1 n i k ) ((pos "Nn%" ) ))
( "Munster"  n ( m uh1 n s t @ r ) ((pos "Nm%" ) ))
( "Muriel"  n ( m y uu1 r i@ l ) ((pos "Nl%" ) ))
( "Murton"  n ( m @@1 t n ) ((pos "Nn$" ) ))
( "Muscovies"  n ( m uh1 s k @ v i z ) ((pos "Kj$" ) ))
( "Muscovite"  n ( m uh1 s k @ v ai t ) ((pos "K6%" "OA%" ) ))
( "Muscovites"  n ( m uh1 s k @ v ai t s ) ((pos "Kj%" ) ))
( "Muscovy"  n ( m uh1 s k @ v ii ) ((pos "K8$" ) ))
( "Muslim"  n ( m u1 z l i m ) ((pos "K6%" ) ))
( "Muslims"  n ( m u1 z l i m z ) ((pos "Kj%" ) ))
( "Musselburgh"  n ( m uh1 s l b r @ ) ((pos "Nn%" ) ))
( "Myra"  n ( m ai1 @ r @ ) ((pos "Nl%" ) ))
( "N"  n ( e1 n ) ((pos "Ki$" ) ))
( "N's"  n ( e1 n z ) ((pos "Kj$" ) ))
( "NAAFI"  n ( n a1 f ii ) ((pos "Y>%" ) ))
( "NASA"  n ( n aa1 s @ ) ((pos "Y=%" ) ))
( "NATO"  n ( n ei1 t ou ) ((pos "Y=%" ) ))
( "NB"  n ( e2 n b ii1 ) ((pos "Y~%" ) ))
( "NCO"  n ( e2 n s i ou1 ) ((pos "Y>%" ) ))
( "NHS"  n ( e2 n ei2 ch e1 s ) ((pos "Y>%" ) ))
( "NSPCC"  n ( e2 n e2 s p ii2 s ii2 s ii1 ) ((pos "Y>%" ) ))
( "NT"  n ( e2 n t ii1 ) ((pos "Y>%" ) ))
( "Nagoya"  n ( n aa1 g ou y @ ) ((pos "Nn$" ) ))
( "Nagpur"  n ( n a1 g p u@1 r ) ((pos "Nn$" ) ))
( "Nailsea"  n ( n ei1 l z ii ) ((pos "Nn$" ) ))
( "Nairn"  n ( n e@1 n ) ((pos "Nn$" ) ))
( "Nairobi"  n ( n ai1 r ou1 b ii ) ((pos "Nn%" ) ))
( "Namibia"  n ( n @1 m i1 b i@ ) ((pos "Nm%" ) ))
( "Namibian"  n ( n @1 m i1 b i@ n ) ((pos "K6%" "OA%" ) ))
( "Namibians"  n ( n @1 m i1 b i@ n z ) ((pos "Kj%" ) ))
( "Nanchang"  n ( n a2 n ch a1 ng ) ((pos "Nn$" ) ))
( "Nancy"  n ( n a1 n s ii ) ((pos "Nl%" ) ))
( "Nanking"  n ( n a2 n k i1 ng ) ((pos "Nn%" ) ))
( "Nanning"  n ( n a2 n n i1 ng ) ((pos "Nn$" ) ))
( "Nantwich"  n ( n a1 n t w i ch ) ((pos "Nn%" ) ))
( "Naomi"  n ( n ei1 @ m ii ) ((pos "Nl%" ) ))
( "Naples"  n ( n ei1 p l z ) ((pos "Nn%" ) ))
( "Napoleonic"  j ( n @1 p ou2 l i o1 n i k ) ((pos "OA%" ) ))
( "Napoli"  n ( n aa1 p @ l ii ) ((pos "Nn%" ) ))
( "Narborough"  n ( n aa1 b r @ ) ((pos "Nn$" ) ))
( "Nat"  n ( n a1 t ) ((pos "Nl%" ) ))
( "Natalie"  n ( n a1 t @ l ii ) ((pos "Nl%" ) ))
( "Nathaniel"  n ( n @1 th a1 n i@ l ) ((pos "Nl%" ) ))
( "Nauru"  n ( n au1 r uu ) ((pos "Nm$" ) ))
( "Nauruan"  n ( n au1 r uu @ n ) ((pos "M6$" "OA$" ) ))
( "Nauruans"  n ( n au1 r uu @ n z ) ((pos "Mj$" ) ))
( "Nazi"  n ( n aa1 t s ii ) ((pos "K6%" "OA%" ) ))
( "Nazis"  n ( n aa1 t s i z ) ((pos "Kj%" ) ))
( "Nazism"  n ( n aa1 t s i z @ m ) ((pos "L@%" ) ))
( "Neanderthal"  j ( n ii1 a1 n d @ t aa l ) ((pos "OA%" ) ))
( "Neapolitan"  n ( n i@1 p o1 l i t @ n ) ((pos "K6%" "OA%" ) ))
( "Neapolitans"  n ( n i@1 p o1 l i t @ n z ) ((pos "Kj%" ) ))
( "Neath"  n ( n ii1 th ) ((pos "Nn$" ) ))
( "Nebraska"  n ( n @1 b r a1 s k @ ) ((pos "Nm%" ) ))
( "Ned"  n ( n e1 d ) ((pos "Nl%" ) ))
( "Neddy"  n ( n e1 d ii ) ((pos "Nl%" ) ))
( "Negress"  n ( n ii1 g r e s ) ((pos "K7%" ) ))
( "Negresses"  n ( n ii1 g r e s i z ) ((pos "Kj%" ) ))
( "Negro"  n ( n ii1 g r ou ) ((pos "K7%" ) ))
( "Negroes"  n ( n ii1 g r ou z ) ((pos "Kj%" ) ))
( "Negroid"  n ( n ii1 g r oi d ) ((pos "K6$" "OA%" ) ))
( "Negroids"  n ( n ii1 g r oi d z ) ((pos "Kj$" ) ))
( "Negus"  n ( n ii1 g @ s ) ((pos "K7$" ) ))
( "Neguses"  n ( n ii1 g @ s i z ) ((pos "Kj$" ) ))
( "Neil"  n ( n ii1 l ) ((pos "Nl%" ) ))
( "Nell"  n ( n e1 l ) ((pos "Nl%" ) ))
( "Nelly"  n ( n e1 l ii ) ((pos "Ki%" "Nl%" ) ))
( "Nelson"  n ( n e1 l s n ) ((pos "Nn%" ) ))
( "Nepal"  n ( n i1 p oo1 l ) ((pos "Nm%" ) ))
( "Nepalese"  n ( n e2 p @ l ii1 z ) ((pos "K9%" "OA%" ) ))
( "Nepali"  n ( n i1 p oo1 l ii ) ((pos "M6%" "OA%" ) ))
( "Nepalis"  n ( n i1 p oo1 l i z ) ((pos "Mj%" ) ))
( "Neptune"  n ( n e1 p t y uu n ) ((pos "No%" ) ))
( "Neston"  n ( n e1 s t @ n ) ((pos "Nn$" ) ))
( "Nestor"  n ( n e1 s t @ r ) ((pos "No$" ) ))
( "Netherlander"  n ( n e1 dh @ l @ n d @ r ) ((pos "K6%" ) ))
( "Netherlanders"  n ( n e1 dh @ l @ n d @ z ) ((pos "Kj%" ) ))
( "Netherlands"  n ( n e1 dh @ l @ n d z ) ((pos "Nm%" ) ))
( "Netley"  n ( n e1 t l ii ) ((pos "Nn$" ) ))
( "Neuss"  n ( n oi1 s ) ((pos "Nn$" ) ))
( "Nevada"  n ( n @1 v aa1 d @ ) ((pos "Nm%" ) ))
( "Neville"  n ( n e1 v @ l ) ((pos "Nl%" ) ))
( "New Cumnock"  n ( n y uu2 - k uh1 m n @ k ) ((pos "Nn$" ) ))
( "New Mills"  n ( n y uu2 - m i1 l z ) ((pos "Nn$" ) ))
( "New Orleans"  n ( n y uu2 - oo1 l i@ n z ) ((pos "Nn%" ) ))
( "New Tredegar"  n ( n y uu2 - t r i d ii1 g @ r ) ((pos "Nn$" ) ))
( "New Windsor"  n ( n y uu2 - w i1 n z @ r ) ((pos "Nn$" ) ))
( "New York"  n ( n y uu2 - y oo1 k ) ((pos "Nn%" ) ))
( "Newark"  n ( n y uu1 @ k ) ((pos "Nn%" ) ))
( "Newarthill"  n ( n y uu2 @ t h i1 l ) ((pos "Nn$" ) ))
( "Newbiggin"  n ( n y uu1 b i g i n ) ((pos "Nn%" ) ))
( "Newbury"  n ( n y uu1 b @ r ii ) ((pos "Nn%" ) ))
( "Newcastle"  n ( n y uu1 k a s l ) ((pos "Nn%" ) ))
( "Newcastle upon Tyne"  n ( n y uu2 k a s l - uh p o n - t ai1 n ) ((pos "Nn%" ) ))
( "Newcastle-under-Lyme"  n ( n y uu2 k a s l - uh2 n d @ - l ai1 m ) ((pos "Nn%" ) ))
( "Newfoundland"  n ( n y uu1 f @ n l @ n d ) ((pos "Nm%" ) ))
( "Newham"  n ( n y uu1 @ m ) ((pos "Nn%" ) ))
( "Newhaven"  n ( n y uu1 h ei v n ) ((pos "Nn%" ) ))
( "Newmains"  n ( n y uu1 m ei n z ) ((pos "Nn$" ) ))
( "Newmarket"  n ( n y uu1 m aa k i t ) ((pos "L@%" "Nn%" ) ))
( "Newport"  n ( n y uu1 p oo t ) ((pos "Nn%" ) ))
( "Newport Pagnell"  n ( n y uu2 p oo t - p a1 g n i l ) ((pos "Nn%" ) ))
( "Newquay"  n ( n y uu1 k ii ) ((pos "Nn%" ) ))
( "Newry"  n ( n y u@1 r ii ) ((pos "Nn%" ) ))
( "Newton Abbot"  n ( n y uu2 t n - a1 b @ t ) ((pos "Nn%" ) ))
( "Newton Aycliffe"  n ( n y uu2 t n - ei1 k l i f ) ((pos "Nn$" ) ))
( "Newton Mearns"  n ( n y uu2 t n - m i@1 n z ) ((pos "Nn$" ) ))
( "Newton-le-Willows"  n ( n y uu2 t n - l i - w i1 l ou z ) ((pos "Nn$" ) ))
( "Newtonian"  n ( n y uu1 t ou1 n i@ n ) ((pos "K6$" "OA%" ) ))
( "Newtonians"  n ( n y uu1 t ou1 n i@ n z ) ((pos "Kj$" ) ))
( "Newtown"  n ( n y uu1 t ou n ) ((pos "Nn$" ) ))
( "Nicaragua"  n ( n i2 k @ r a1 g y u@ ) ((pos "Nm%" ) ))
( "Nicaraguan"  n ( n i2 k @ r a1 g y u@ n ) ((pos "K6%" "OA%" ) ))
( "Nicaraguans"  n ( n i2 k @ r a1 g y u@ n z ) ((pos "Kj%" ) ))
( "Nice"  n ( n ii1 s ) ((pos "Nn%" ) ))
( "Nicholas"  n ( n i1 k @ l @ s ) ((pos "Nl%" ) ))
( "Nick"  n ( n i1 k ) ((pos "Nl%" ) ))
( "Nicola"  n ( n i1 k @ l @ ) ((pos "Nl%" ) ))
( "Nicole"  n ( n i1 k o1 l ) ((pos "Nl%" ) ))
( "Nigel"  n ( n ai1 jh l ) ((pos "Nl%" ) ))
( "Niger"  n ( n ii1 zh e@1 r ) ((pos "Nm%" ) ))
( "Nigeria"  n ( n ai1 jh i@1 r i@ ) ((pos "Nm%" ) ))
( "Nigerian"  n ( n ai1 jh i@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Nigerians"  n ( n ai1 jh i@1 r i@ n z ) ((pos "Kj%" ) ))
( "Nigerien"  n ( n ii1 zh e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "Nigeriens"  n ( n ii1 zh e@1 r i@ n z ) ((pos "Kj$" ) ))
( "Nijmegen"  n ( n ai1 m ei g n ) ((pos "Nn%" ) ))
( "Nilotic"  j ( n ai1 l o1 t i k ) ((pos "OA$" ) ))
( "Nimrod"  n ( n i1 m r o d ) ((pos "No$" ) ))
( "Niobe"  n ( n ai1 @ b ii ) ((pos "No$" ) ))
( "Nipponese"  j ( n i2 p @ n ii1 z ) ((pos "OA$" ) ))
( "Nissen"  n ( n i1 s n ) ((pos "No$" ) ))
( "Noah"  n ( n ou1 @ ) ((pos "No$" ) ))
( "Nobel"  n ( n ou1 b e1 l ) ((pos "No$" ) ))
( "Noel"  n ( n ou1 @ l ) ((pos "K6$" "Nl%" ) ))
( "Noels"  n ( n ou1 @ l z ) ((pos "Kj$" ) ))
( "Nora"  n ( n oo1 r @ ) ((pos "Nl%" ) ))
( "Nordic"  n ( n oo1 d i k ) ((pos "K6$" "OA%" ) ))
( "Nordics"  n ( n oo1 d i k s ) ((pos "Kj$" ) ))
( "Norfolk"  n ( n oo1 f @ k ) ((pos "Nm%" ) ))
( "Norman"  n ( n oo1 m @ n ) ((pos "K6%" "Nl%" "OA%" ) ))
( "Normans"  n ( n oo1 m @ n z ) ((pos "Kj%" ) ))
( "Normanton"  n ( n oo1 m @ n t @ n ) ((pos "Nn$" ) ))
( "Normantown"  n ( n oo1 m @ n t au n ) ((pos "Nn$" ) ))
( "Norse"  n ( n oo1 s ) ((pos "L@%" "OA%" ) ))
( "North Walsham"  n ( n oo2 th - w o1 l sh @ m ) ((pos "Nn$" ) ))
( "Northallerton"  n ( n oo1 th a1 l @ t @ n ) ((pos "Nn%" ) ))
( "Northam"  n ( n oo1 th @ m ) ((pos "Nn$" ) ))
( "Northampton"  n ( n oo1 th a1 m p t @ n ) ((pos "Nn%" ) ))
( "Northamptonshire"  n ( n oo1 th a1 m p t @ n sh @ r ) ((pos "Nm%" ) ))
( "Northman"  n ( n oo1 th m @ n ) ((pos "Ki$" ) ))
( "Northmen"  n ( n oo1 th m @ n ) ((pos "Kj$" ) ))
( "Northumberland"  n ( n oo1 th uh1 m b @ l @ n d ) ((pos "Nm%" ) ))
( "Northwich"  n ( n oo1 th w i ch ) ((pos "Nn$" ) ))
( "Norton"  n ( n oo1 t n ) ((pos "Nn%" ) ))
( "Norway"  n ( n oo1 w ei ) ((pos "Nm%" ) ))
( "Norwegian"  n ( n oo1 w ii1 jh @ n ) ((pos "M6%" "OA%" ) ))
( "Norwegians"  n ( n oo1 w ii1 jh @ n z ) ((pos "Mj%" ) ))
( "Norwich"  n ( n o1 r i ch ) ((pos "Nn%" ) ))
( "Nottingham"  n ( n o1 t i ng @ m ) ((pos "Nn%" ) ))
( "Nottinghamshire"  n ( n o1 t i ng @ m sh @ r ) ((pos "Nm%" ) ))
( "Nov"  n ( n o1 v ) ((pos "Y=%" ) ))
( "Nova Scotia"  n ( n ou2 v @ - s k ou1 sh @ ) ((pos "Nm%" ) ))
( "Novara"  n ( n @1 v aa1 r @ ) ((pos "Nn$" ) ))
( "November"  n ( n ou1 v e1 m b @ r ) ((pos "K6%" "No%" ) ))
( "Novembers"  n ( n ou1 v e1 m b @ z ) ((pos "Kj$" ) ))
( "Novokuznetsk"  n ( n o2 v @ k u z n y e1 t s k ) ((pos "Nn$" ) ))
( "Novosibirsk"  n ( n o2 v @ s i b i@1 s k ) ((pos "Nn$" ) ))
( "Nuneaton"  n ( n uh1 n ii1 t n ) ((pos "Nn%" ) ))
( "Nurenburg"  n ( n y u@1 r @ n b @@ g ) ((pos "Nn%" ) ))
( "O"  n ( ou1 ) ((pos "Ki$" "W-$" ) ))
( "O's"  n ( ou1 z ) ((pos "Kj$" ) ))
( "O-level"  n ( ou1 - l e v @ l ) ((pos "K6%" ) ))
( "O-levels"  n ( ou1 - l e v @ l z ) ((pos "Kj%" ) ))
( "OAP"  n ( ou2 ei2 p ii1 ) ((pos "Y>%" ) ))
( "OAP's"  n ( ou2 ei2 p ii1 z ) ((pos "Y)%" ) ))
( "OAU"  n ( ou2 ei2 y uu1 ) ((pos "Y>%" ) ))
( "OECD"  n ( ou2 ii2 s ii2 d ii1 ) ((pos "Y>%" ) ))
( "OED"  n ( ou2 ii2 d ii1 ) ((pos "Y>%" ) ))
( "OK"  uh ( ou2 k ei1 ) ((pos "W-%" ) ))
( "OPEC"  n ( ou1 p e k ) ((pos "Y=%" ) ))
( "OT"  n ( ou2 t ii1 ) ((pos "Y>%" ) ))
( "Oakengates"  n ( ou1 k n g ei t s ) ((pos "Nn$" ) ))
( "Oban"  n ( ou1 b @ n ) ((pos "Nn%" ) ))
( "Oberhausen"  n ( ou1 b @ h au z n ) ((pos "Nn$" ) ))
( "Occident"  n ( o1 k s i d @ n t ) ((pos "No%" ) ))
( "Occidental"  n ( o2 k s i d e1 n t l ) ((pos "K6$" "OA%" ) ))
( "Occidentals"  n ( o2 k s i d e1 n t l z ) ((pos "Kj$" ) ))
( "Ockbrook"  n ( o1 k b r u k ) ((pos "Nn$" ) ))
( "Ockendon"  n ( o1 k n d @ n ) ((pos "Nn$" ) ))
( "Oct"  n ( o1 k t ) ((pos "Y=%" ) ))
( "October"  n ( o1 k t ou1 b @ r ) ((pos "K6%" "No%" ) ))
( "Octobers"  n ( o1 k t ou1 b @ z ) ((pos "Kj$" ) ))
( "Odessa"  n ( ou1 d e1 s @ ) ((pos "Nn%" ) ))
( "Oedipus"  n ( ii1 d i p @ s ) ((pos "No%" ) ))
( "Offaly"  n ( o1 f @ l ii ) ((pos "Nm%" ) ))
( "Offenbach"  n ( o1 f n b aa k ) ((pos "Nn%" ) ))
( "Ogmore Valley"  n ( o2 g m oo - v a1 l ii ) ((pos "Nn$" ) ))
( "Ohio"  n ( ou1 h ai1 ou ) ((pos "Nm%" ) ))
( "Oklahoma"  n ( ou2 k l @ h ou1 m @ ) ((pos "Nm%" ) ))
( "Oklahoma City"  n ( ou2 k l @ h ou2 m @ - s i1 t ii ) ((pos "Nn%" ) ))
( "Old Windsor"  n ( ou2 l d - w i1 n z @ r ) ((pos "Nn$" ) ))
( "Oldenburg"  n ( ou1 l d n b @@ g ) ((pos "Nn$" ) ))
( "Oldham"  n ( ou1 l d @ m ) ((pos "Nn%" ) ))
( "Oldland"  n ( ou1 l d l a n d ) ((pos "Nn$" ) ))
( "Olive"  n ( o1 l i v ) ((pos "Nl%" ) ))
( "Oliver"  n ( o1 l i v @ r ) ((pos "Nl%" ) ))
( "Olivia"  n ( @1 l i1 v i@ ) ((pos "Nl%" ) ))
( "Ollerton"  n ( o1 l @ t @ n ) ((pos "Nn$" ) ))
( "Ollie"  n ( o1 l ii ) ((pos "Nl%" ) ))
( "Olympiad"  n ( @1 l i1 m p i a d ) ((pos "K6%" ) ))
( "Olympiads"  n ( @1 l i1 m p i a d z ) ((pos "Kj$" ) ))
( "Olympian"  n ( @1 l i1 m p i@ n ) ((pos "K6%" "OA%" ) ))
( "Olympians"  n ( @1 l i1 m p i@ n z ) ((pos "Kj%" ) ))
( "Olympic"  j ( @1 l i1 m p i k ) ((pos "OA%" ) ))
( "Olympics"  n ( @1 l i1 m p i k s ) ((pos "No%" ) ))
( "Oman"  n ( ou1 m aa1 n ) ((pos "Nm%" ) ))
( "Omani"  n ( ou1 m aa1 n ii ) ((pos "K6%" "OA%" ) ))
( "Omanis"  n ( ou1 m aa1 n i z ) ((pos "Kj%" ) ))
( "Omsk"  n ( o1 m s k ) ((pos "Nn%" ) ))
( "Ongar"  n ( o1 ng g @ r ) ((pos "Nn%" ) ))
( "Ontario"  n ( o1 n t e@1 r i@ @ ) ((pos "Nm%" ) ))
( "Orangeman"  n ( o1 r i n jh m a n ) ((pos "Ki%" ) ))
( "Orangemen"  n ( o1 r i n jh m e n ) ((pos "Kj%" ) ))
( "Oregon"  n ( o1 r i g @ n ) ((pos "Nm%" ) ))
( "Orkney"  n ( oo1 k n ii ) ((pos "Nm%" ) ))
( "Orleans"  n ( oo1 l i@1 n z ) ((pos "Nn%" ) ))
( "Ormskirk"  n ( oo1 m z k @@ k ) ((pos "Nn%" ) ))
( "Osaka"  n ( ou1 s aa1 k @ ) ((pos "Nn%" ) ))
( "Oscar"  n ( o1 s k @ r ) ((pos "K6%" "Nl%" ) ))
( "Oscars"  n ( o1 s k @ z ) ((pos "Kj%" ) ))
( "Oslo"  n ( o1 z l ou ) ((pos "Nn%" ) ))
( "Osnabruck"  n ( o1 z n @ b r u k ) ((pos "Nn$" ) ))
( "Ossett"  n ( o1 s i t ) ((pos "Nn$" ) ))
( "Oswald"  n ( o1 z w @ l d ) ((pos "Nl%" ) ))
( "Oswestry"  n ( o1 z w i s t r ii ) ((pos "Nn%" ) ))
( "Otago"  n ( ou1 t aa1 g ou ) ((pos "Nm$" ) ))
( "Otley"  n ( o1 t l ii ) ((pos "Nn%" ) ))
( "Ottawa"  n ( o1 t @ w @ ) ((pos "Nn%" ) ))
( "Oviedo"  n ( o2 v i ei1 d ou ) ((pos "Nn$" ) ))
( "Oxbridge"  n ( o1 k s b r i jh ) ((pos "No%" ) ))
( "Oxford"  n ( o1 k s f @ d ) ((pos "Nn%" ) ))
( "Oxfordshire"  n ( o1 k s f @ d sh @ r ) ((pos "Nm%" ) ))
( "Oxon"  n ( o1 k s n ) ((pos "Y~$" ) ))
( "Oxonian"  n ( o1 k s ou1 n i@ n ) ((pos "K6$" "OA$" ) ))
( "Oxonians"  n ( o1 k s ou1 n i@ n z ) ((pos "Kj$" ) ))
( "Oxted"  n ( o1 k s t i d ) ((pos "Nn$" ) ))
( "Ozzie"  n ( o1 z ii ) ((pos "Nl%" ) ))
( "P"  n ( p ii1 ) ((pos "Ki$" ) ))
( "P's"  n ( p ii1 z ) ((pos "Kj$" ) ))
( "PA"  n ( p ii2 ei1 ) ((pos "Y>%" ) ))
( "PAYE"  n ( p ii2 ei2 w ai2 ii1 ) ((pos "Y=%" ) ))
( "PC"  n ( p ii2 s ii1 ) ((pos "Y:%" ) ))
( "PDSA"  n ( p ii2 d ii2 e2 s ei1 ) ((pos "Y>%" ) ))
( "PE"  n ( p ii2 ii1 ) ((pos "Y}%" ) ))
( "PLC"  n ( p ii2 e2 l s ii1 ) ((pos "Y=%" ) ))
( "PM"  n ( p ii2 e1 m ) ((pos "Y>%" ) ))
( "PO"  n ( p ii2 ou1 ) ((pos "Y>%" ) ))
( "POW"  n ( p ii2 ou2 d uh1 b l y uu ) ((pos "Y>%" ) ))
( "PR"  n ( p ii2 aa1 r ) ((pos "Y}%" ) ))
( "PT"  n ( p ii2 t ii1 ) ((pos "Y}%" ) ))
( "PTA"  n ( p ii2 t ii2 ei1 ) ((pos "Y>%" ) ))
( "PTO"  n ( p ii2 t ii2 ou1 ) ((pos "Y~%" ) ))
( "Pacific"  n ( p @1 s i1 f i k ) ((pos "No%" ) ))
( "Paddies"  n ( p a1 d i z ) ((pos "Kj$" ) ))
( "Paddy"  n ( p a1 d ii ) ((pos "K8$" "Nl%" ) ))
( "Paddy-wagon"  n ( p a1 d i - w a g n ) ((pos "K6$" ) ))
( "Paddy-wagons"  n ( p a1 d i - w a g n z ) ((pos "Kj$" ) ))
( "Padova"  n ( p aa1 d @ v @ ) ((pos "Nn$" ) ))
( "Padua"  n ( p a1 jh u@ ) ((pos "Nn%" ) ))
( "Paisley"  n ( p ei1 z l ii ) ((pos "Nn%" ) ))
( "Pakistan"  n ( p aa2 k i s t aa1 n ) ((pos "Nm%" ) ))
( "Pakistani"  n ( p aa2 k i s t aa1 n ii ) ((pos "K6%" "OA%" ) ))
( "Pakistanis"  n ( p aa2 k i s t aa1 n i z ) ((pos "Kj%" ) ))
( "Palembang"  n ( p aa1 l e1 m b aa ng ) ((pos "Nn$" ) ))
( "Palermo"  n ( p @1 l @@1 m ou ) ((pos "Nn%" ) ))
( "Palestine"  n ( p a1 l @ s t ai n ) ((pos "Nm%" ) ))
( "Palestinian"  n ( p a2 l @ s t i1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Palestinians"  n ( p a2 l @ s t i1 n i@ n z ) ((pos "Kj%" ) ))
( "Pam"  n ( p a1 m ) ((pos "Nl%" ) ))
( "Pamela"  n ( p a1 m @ l @ ) ((pos "Nl%" ) ))
( "Pamplona"  n ( p a1 m p l ou1 n @ ) ((pos "Nn%" ) ))
( "Panama"  n ( p a2 n @ m aa1 ) ((pos "Nm%" ) ))
( "Panamanian"  n ( p a2 n @ m ei1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Panamanians"  n ( p a2 n @ m ei1 n i@ n z ) ((pos "Kj%" ) ))
( "Paotow"  n ( p au2 t au1 ) ((pos "Nn$" ) ))
( "Papua"  n ( p a1 p y u@ ) ((pos "Nm%" ) ))
( "Papuan"  n ( p a1 p y u@ n ) ((pos "K6%" "OA%" ) ))
( "Papuans"  n ( p a1 p y u@ n z ) ((pos "Kj%" ) ))
( "Paraguay"  n ( p a1 r @ g w ai ) ((pos "Nm%" ) ))
( "Paraguayan"  n ( p a2 r @ g w ai1 @ n ) ((pos "K6%" "OA%" ) ))
( "Paraguayans"  n ( p a2 r @ g w ai1 @ n z ) ((pos "Kj%" ) ))
( "Paris"  n ( p a1 r i s ) ((pos "Nn%" ) ))
( "Parisian"  n ( p @1 r i1 z i@ n ) ((pos "K6%" "OA%" ) ))
( "Parisians"  n ( p @1 r i1 z i@ n z ) ((pos "Kj%" ) ))
( "Parkinson's"  j ( p aa1 k i n s n z ) ((pos "OA$" ) ))
( "Parma"  n ( p aa1 m @ ) ((pos "Nn%" ) ))
( "Parmesan"  n ( p aa1 m i z a n ) ((pos "L@%" ) ))
( "Parsee"  n ( p aa1 s ii1 ) ((pos "K6$" ) ))
( "Parsees"  n ( p aa1 s ii1 z ) ((pos "Kj$" ) ))
( "Parthian"  j ( p aa1 th i@ n ) ((pos "OA$" ) ))
( "Pashto"  n ( p uh1 sh t ou ) ((pos "L@$" ) ))
( "Passover"  n ( p aa1 s ou v @ r ) ((pos "K6%" ) ))
( "Passovers"  n ( p aa1 s ou v @ z ) ((pos "Kj$" ) ))
( "Pat"  n ( p a1 t ) ((pos "Nl%" ) ))
( "Patience"  n ( p ei1 sh @ n s ) ((pos "Nl%" ) ))
( "Patrai"  n ( p a1 t r ii ) ((pos "Nn$" ) ))
( "Patricia"  n ( p @1 t r i1 sh @ ) ((pos "Nl%" ) ))
( "Patrick"  n ( p a1 t r i k ) ((pos "Nl%" ) ))
( "Patty"  n ( p a1 t ii ) ((pos "Nl%" ) ))
( "Paul"  n ( p oo1 l ) ((pos "Nl%" ) ))
( "Paula"  n ( p oo1 l @ ) ((pos "Nl%" ) ))
( "Pauline"  n ( p oo1 l ii n ) ((pos "Nl%" ) ))
( "Pax Romana"  n ( p a2 k s - r ou m aa1 n @ ) ((pos "Ki$" ) ))
( "Peacehaven"  n ( p ii1 s h ei v n ) ((pos "Nn$" ) ))
( "Pearl"  n ( p @@1 l ) ((pos "Nl%" ) ))
( "Peebles"  n ( p ii1 b l z ) ((pos "Nn%" ) ))
( "Peg"  n ( p e1 g ) ((pos "Nl%" ) ))
( "Peggy"  n ( p e1 g ii ) ((pos "Nl%" ) ))
( "Peking"  n ( p ii2 k i1 ng ) ((pos "Nn%" ) ))
( "Pelton"  n ( p e1 l t n ) ((pos "Nn$" ) ))
( "Pembroke"  n ( p e1 m b r @ k ) ((pos "Nn%" ) ))
( "Penarth"  n ( p i1 n aa1 th ) ((pos "Nn%" ) ))
( "Pendlebury"  n ( p e1 n d l b r ii ) ((pos "Nn$" ) ))
( "Penelope"  n ( p @1 n e1 l @ p ii ) ((pos "Nl%" ) ))
( "Penicuik"  n ( p e1 n i k u k ) ((pos "Nn$" ) ))
( "Penki"  n ( p e2 n ch ii1 ) ((pos "Nn$" ) ))
( "Pennsylvania"  n ( p e2 n s l v ei1 n i@ ) ((pos "Nm%" ) ))
( "Penny"  n ( p e1 n ii ) ((pos "Nl%" ) ))
( "Penrhyn"  n ( p e1 n r i n ) ((pos "Nn%" ) ))
( "Penrith"  n ( p e1 n r i th ) ((pos "Nn%" ) ))
( "Penryn"  n ( p e1 n r i n ) ((pos "Nn%" ) ))
( "Pentateuch"  n ( p e1 n t @ t y uu k ) ((pos "K6%" ) ))
( "Pentateuchs"  n ( p e1 n t @ t y uu k s ) ((pos "Kj$" ) ))
( "Pentecost"  n ( p e1 n t i k o s t ) ((pos "L@%" ) ))
( "Pentecostal"  j ( p e2 n t i k o1 s t l ) ((pos "OA%" ) ))
( "Penzance"  n ( p e1 n z a1 n s ) ((pos "Nn%" ) ))
( "Percy"  n ( p @@1 s ii ) ((pos "Nl%" ) ))
( "Perm"  n ( p y e1 r m ) ((pos "Nn$" ) ))
( "Pershore"  n ( p @@1 sh oo r ) ((pos "Nn%" ) ))
( "Persia"  n ( p @@1 sh @ ) ((pos "Nm%" ) ))
( "Persian"  n ( p @@1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Persians"  n ( p @@1 sh @ n z ) ((pos "Kj%" ) ))
( "Perth"  n ( p @@1 th ) ((pos "Nn%" ) ))
( "Peru"  n ( p @1 r uu1 ) ((pos "Nm%" ) ))
( "Perugia"  n ( p @1 r uu1 jh i@ ) ((pos "Nn%" ) ))
( "Peruvian"  n ( p @1 r u1 v i@ n ) ((pos "K6%" "OA%" ) ))
( "Peruvians"  n ( p @1 r u1 v i@ n z ) ((pos "Kj%" ) ))
( "Pescara"  n ( p e1 s k aa1 r @ ) ((pos "Nn$" ) ))
( "Pete"  n ( p ii1 t ) ((pos "Nl%" ) ))
( "Peter"  n ( p ii1 t @ r ) ((pos "Nl%" ) ))
( "Peterborough"  n ( p ii1 t @ b r @ ) ((pos "Nn%" ) ))
( "Peterhead"  n ( p ii1 t @ h e d ) ((pos "Nn%" ) ))
( "Peterlee"  n ( p ii1 t @ l ii ) ((pos "Nn%" ) ))
( "Petersfield"  n ( p ii1 t @ z f ii l d ) ((pos "Nn$" ) ))
( "PhD"  n ( p ii2 ei2 ch d ii1 ) ((pos "Y>%" ) ))
( "Pharaoh"  n ( f e@1 r ou ) ((pos "K6%" ) ))
( "Pharaohs"  n ( f e@1 r ou z ) ((pos "Kj%" ) ))
( "Pharisee"  n ( f a1 r i s ii ) ((pos "K6%" ) ))
( "Pharisees"  n ( f a1 r i s ii z ) ((pos "Kj%" ) ))
( "Phil"  n ( f i1 l ) ((pos "Nl%" ) ))
( "Philadelphia"  n ( f i2 l @ d e1 l f i@ ) ((pos "Nn%" ) ))
( "Philip"  n ( f i1 l i p ) ((pos "Nl%" ) ))
( "Philippa"  n ( f i1 l i p @ ) ((pos "Nl%" ) ))
( "Philippine"  n ( f i1 l i p ii n ) ((pos "K6%" "OA%" ) ))
( "Philippines"  n ( f i1 l i p ii n z ) ((pos "Nm%" ) ))
( "Philistine"  n ( f i1 l i s t ai n ) ((pos "K6%" ) ))
( "Philistines"  n ( f i1 l i s t ai n z ) ((pos "Kj%" ) ))
( "Philistinism"  n ( f i1 l i s t i n i z m ) ((pos "L@%" ) ))
( "Phnom-Penh"  n ( n o2 m - p e1 n ) ((pos "Nn%" ) ))
( "Phoebe"  n ( f ii1 b ii ) ((pos "Nl%" ) ))
( "Phoenix"  n ( f ii1 n i k s ) ((pos "Nn%" ) ))
( "Phyllis"  n ( f i1 l i s ) ((pos "Nl%" ) ))
( "Piacenza"  n ( p i@2 ch e1 n z @ ) ((pos "Nn$" ) ))
( "Pill"  n ( p i1 l ) ((pos "Nn$" ) ))
( "Piraeus"  n ( p ai1 r ii1 @ s ) ((pos "Nn%" ) ))
( "Pisa"  n ( p ii1 z @ ) ((pos "Nn%" ) ))
( "Pisces"  n ( p ai1 s ii z ) ((pos "No%" ) ))
( "Pittsburgh"  n ( p i1 t s b @@ g ) ((pos "Nn%" ) ))
( "Platonic"  j ( p l @1 t o1 n i k ) ((pos "OA%" ) ))
( "Plimsoll"  n ( p l i1 m s @ l ) ((pos "K6$" ) ))
( "Plimsolls"  n ( p l i1 m s @ l z ) ((pos "Kj$" ) ))
( "Pluto"  n ( p l uu1 t ou ) ((pos "No%" ) ))
( "Plutos"  n ( p l uu1 t ou z ) ((pos "Kj$" ) ))
( "Plymouth"  n ( p l i1 m @ th ) ((pos "Nn%" ) ))
( "Poland"  n ( p ou1 l @ n d ) ((pos "Nm%" ) ))
( "Polaris"  n ( p ou1 l a1 r i s ) ((pos "Ki%" ) ))
( "Polaroid"  n ( p ou1 l @ r oi d ) ((pos "L@%" ) ))
( "Pole"  n ( p ou1 l ) ((pos "K6%" ) ))
( "Poles"  n ( p ou1 l z ) ((pos "Kj%" ) ))
( "Polish"  n ( p ou1 l i sh ) ((pos "L@%" "OA%" ) ))
( "Polly"  n ( p o1 l ii ) ((pos "Nl%" ) ))
( "Pont-Llan-Fraith"  n ( p o2 n t - l a n - f r ei1 th ) ((pos "Nn$" ) ))
( "Pontardawe"  n ( p o2 n t @ d ou1 ii ) ((pos "Nn$" ) ))
( "Pontardulais"  n ( p o2 n t @ d uu1 l ei s ) ((pos "Nn$" ) ))
( "Pontefract"  n ( p o1 n t i f r a k t ) ((pos "Nn%" ) ))
( "Pontycymmer"  n ( p o2 n t i k i1 m @ r ) ((pos "Nn$" ) ))
( "Pontypool"  n ( p o2 n t i p uu1 l ) ((pos "Nn%" ) ))
( "Poole"  n ( p uu1 l ) ((pos "Nn%" ) ))
( "Poona"  n ( p uu1 n @ ) ((pos "Nn%" ) ))
( "Pope"  n ( p ou1 p ) ((pos "K6%" ) ))
( "Popes"  n ( p ou1 p s ) ((pos "Kj%" ) ))
( "Port Glasgow"  n ( p oo2 t - g l a1 z g ou ) ((pos "Nn$" ) ))
( "Port Talbot"  n ( p oo2 t - t o1 l b @ t ) ((pos "Nn%" ) ))
( "Porthcawl"  n ( p oo1 th k oo1 l ) ((pos "Nn%" ) ))
( "Porthmadog"  n ( p oo1 t m a1 d @ k ) ((pos "Nn%" ) ))
( "Portishead"  n ( p oo1 t i z h e d ) ((pos "Nn%" ) ))
( "Portland"  n ( p oo1 t l @ n d ) ((pos "Nn%" ) ))
( "Porto Alegre"  n ( p oo2 t ou - @ l ei1 g r ii ) ((pos "Nn$" ) ))
( "Portsmouth"  n ( p oo1 t s m @ th ) ((pos "Nn%" ) ))
( "Portugal"  n ( p oo1 ch u g l ) ((pos "Nm%" ) ))
( "Portuguese"  n ( p oo2 ch u g ii1 z ) ((pos "M9%" "OA%" ) ))
( "Potsdam"  n ( p o1 t s d a m ) ((pos "Nn%" ) ))
( "Potters Bar"  n ( p o2 t @ z - b aa1 r ) ((pos "Nn$" ) ))
( "Powys"  n ( p au1 i s ) ((pos "Nm%" ) ))
( "Poynton"  n ( p oi1 n t @ n ) ((pos "Nn$" ) ))
( "Poznan"  n ( p o1 z n aa1 n ) ((pos "Nn$" ) ))
( "Prague"  n ( p r aa1 g ) ((pos "Nn%" ) ))
( "Prato"  n ( p r aa1 t ou ) ((pos "Nn$" ) ))
( "Pre-Raphaelite"  n ( p r ii2 r a1 f @ l ai t ) ((pos "K6%" "OA%" ) ))
( "Pre-Raphaelites"  n ( p r ii2 r a1 f @ l ai t s ) ((pos "Kj%" ) ))
( "Pres"  n ( p r e1 z ) ((pos "Y>%" ) ))
( "Presbyterian"  n ( p r e2 z b i t i@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Presbyterianism"  n ( p r e2 s b i t i@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "Presbyterians"  n ( p r e2 z b i t i@1 r i@ n z ) ((pos "Kj%" ) ))
( "Prescot"  n ( p r e1 s k @ t ) ((pos "Nn$" ) ))
( "Prestatyn"  n ( p r e1 s t a1 t i n ) ((pos "Nn%" ) ))
( "Preston"  n ( p r e1 s t n ) ((pos "Nn%" ) ))
( "Prestonpans"  n ( p r e1 s t n p a n z ) ((pos "Nn$" ) ))
( "Prestwich"  n ( p r e1 s t w i ch ) ((pos "Nn$" ) ))
( "Prestwick"  n ( p r e1 s t w i k ) ((pos "Nn$" ) ))
( "Pretoria"  n ( p r i1 t oo1 r i@ ) ((pos "Nn%" ) ))
( "Princes Risborough"  n ( p r i2 n s i z - r i1 z b r @ ) ((pos "Nn$" ) ))
( "Priscilla"  n ( p r i1 s i1 l @ ) ((pos "Nl%" ) ))
( "Prof"  n ( p r o1 f ) ((pos "Y:%" ) ))
( "Protestant"  n ( p r o1 t i s t @ n t ) ((pos "K6%" "OA%" ) ))
( "Protestantism"  n ( p r o1 t i s t @ n t i z @ m ) ((pos "L@%" ) ))
( "Protestants"  n ( p r o1 t i s t @ n t s ) ((pos "Kj%" ) ))
( "Providence"  n ( p r o1 v i d n s ) ((pos "Nn%" ) ))
( "Pru"  n ( p r uu1 ) ((pos "Nl%" ) ))
( "Prudence"  n ( p r uu1 d n s ) ((pos "Nl%" ) ))
( "Prudhoe"  n ( p r uu1 d ou ) ((pos "Nn$" ) ))
( "Prussian"  n ( p r uh1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Prussians"  n ( p r uh1 sh @ n z ) ((pos "Kj%" ) ))
( "Pte"  n ( p r ai1 v i t ) ((pos "Y:%" ) ))
( "Pty"  n ( p r @1 p r ai1 @ t @ r ii ) ((pos "Y=%" ) ))
( "Pudsey"  n ( p uh1 d z ii ) ((pos "Nn$" ) ))
( "Pullman"  n ( p u1 l m @ n ) ((pos "K6%" ) ))
( "Pullmans"  n ( p u1 l m @ n z ) ((pos "Kj%" ) ))
( "Punch"  n ( p uh1 n ch ) ((pos "K7%" ) ))
( "Punches"  n ( p uh1 n ch i z ) ((pos "Kj$" ) ))
( "Punic"  j ( p y uu1 n i k ) ((pos "OA$" ) ))
( "Pusan"  n ( p uu2 s a1 n ) ((pos "Nn$" ) ))
( "Pwllheli"  n ( p u1 th e1 l ii ) ((pos "Nn%" ) ))
( "Pyle"  n ( p ai1 l ) ((pos "Nn$" ) ))
( "Pyongyang"  n ( p y o2 ng y a1 ng ) ((pos "Nn$" ) ))
( "Pyrrhic"  j ( p i1 r i k ) ((pos "OA$" ) ))
( "Q"  n ( k y uu1 ) ((pos "Ki$" ) ))
( "Q's"  n ( k y uu1 z ) ((pos "Kj$" ) ))
( "QC"  n ( k y uu2 s ii1 ) ((pos "Y>%" ) ))
( "QED"  n ( k y uu2 ii2 d ii1 ) ((pos "Y~%" ) ))
( "Qatar"  n ( k uh1 t aa1 r ) ((pos "Nm%" ) ))
( "Qatari"  n ( k uh1 t aa1 r ii ) ((pos "K6%" "OA%" ) ))
( "Qataris"  n ( k uh1 t aa1 r i z ) ((pos "Kj%" ) ))
( "Quai d'Orsay"  n ( k ei2 - d oo s ei1 ) ((pos "Ki%" ) ))
( "Quaker"  n ( k w ei1 k @ r ) ((pos "K6%" ) ))
( "Quakers"  n ( k w ei1 k @ z ) ((pos "Kj%" ) ))
( "Quebec"  n ( k w i1 b e1 k ) ((pos "Nm%" ) ))
( "Queenborough-in-Sheppey"  n ( k w ii2 n b r @ - i n - sh e1 p ii ) ((pos "Nn$" ) ))
( "Queensbury"  n ( k w ii1 n z b @ r ii ) ((pos "Nn$" ) ))
( "Queensferry"  n ( k w ii1 n z f e r ii ) ((pos "Nn$" ) ))
( "Queensland"  n ( k w ii1 n z l @ n d ) ((pos "Nm%" ) ))
( "Quentin"  n ( k w e1 n t i n ) ((pos "Nl%" ) ))
( "Quinquagesima"  n ( k w i2 ng k w @ jh e1 s i m @ ) ((pos "K6$" "No$" ) ))
( "Quinquagesimas"  n ( k w i2 ng k w @ jh e1 s i m @ z ) ((pos "Kj$" ) ))
( "Quito"  n ( k ii1 t ou ) ((pos "Nn$" ) ))
( "Quonset"  n ( k w o1 n s i t ) ((pos "K6$" ) ))
( "Quonsets"  n ( k w o1 n s i t s ) ((pos "Kj$" ) ))
( "R"  n ( aa1 r ) ((pos "Ki$" ) ))
( "R\"ontgen"  n ( r o1 n t y @ n ) ((pos "K6%" ) ))
( "R\"ontgens"  n ( r o1 n t y @ n z ) ((pos "Kj$" ) ))
( "R's"  n ( aa1 z ) ((pos "Kj$" ) ))
( "RA"  n ( aa2 r ei1 ) ((pos "Y>%" ) ))
( "RADA"  n ( r aa1 d @ ) ((pos "Y=%" ) ))
( "RAF"  n ( aa2 r ei2 e1 f ) ((pos "Y>%" ) ))
( "RC"  n ( aa2 s ii1 ) ((pos "Y~%" ) ))
( "RIP"  n ( aa2 r ai2 p ii1 ) ((pos "Y~%" ) ))
( "RM"  n ( aa2 r e1 m ) ((pos "Y>$" ) ))
( "RN"  n ( aa2 r e1 n ) ((pos "Y>%" ) ))
( "RSPCA"  n ( aa2 r e2 s p ii2 s ii2 ei1 ) ((pos "Y>%" ) ))
( "RSVP"  n ( aa2 r e2 s v ii2 p ii1 ) ((pos "Y~%" ) ))
( "Rabelaisian"  j ( r a2 b @ l ei1 z i@ n ) ((pos "OA$" ) ))
( "Rachel"  n ( r ei1 ch l ) ((pos "Nl%" ) ))
( "Radcliffe"  n ( r a1 d k l i f ) ((pos "Nn$" ) ))
( "Radcliffe on Trent"  n ( r a2 d k l i f - o n - t r e1 n t ) ((pos "Nn$" ) ))
( "Radlett"  n ( r a1 d l i t ) ((pos "Nn$" ) ))
( "Radstock"  n ( r a1 d s t o k ) ((pos "Nn$" ) ))
( "Rainford"  n ( r ei1 n f @ d ) ((pos "Nn$" ) ))
( "Ralph"  n ( r a1 l f ) ((pos "Nl%" ) ))
( "Ramadan"  n ( r a2 m @ d aa1 n ) ((pos "K6%" "No%" ) ))
( "Ramadans"  n ( r a2 m @ d aa1 n z ) ((pos "Kj$" ) ))
( "Ramsbottom"  n ( r a1 m z b o t @ m ) ((pos "Nn$" ) ))
( "Ramsgate"  n ( r a1 m z g ei t ) ((pos "Nn%" ) ))
( "Randolph"  n ( r a1 n d o l f ) ((pos "Nl%" ) ))
( "Rangoon"  n ( r a1 ng g uu1 n ) ((pos "Nn%" ) ))
( "Ravenna"  n ( r @1 v e1 n @ ) ((pos "Nn%" ) ))
( "Rawtenstall"  n ( r o1 t n s t oo l ) ((pos "Nn$" ) ))
( "Ray"  n ( r ei1 ) ((pos "Nl%" ) ))
( "Raymond"  n ( r ei1 m @ n d ) ((pos "Nl%" ) ))
( "Rd"  n ( r ou1 d ) ((pos "Y=%" ) ))
( "Reading"  n ( r e1 d i ng ) ((pos "Nn%" ) ))
( "Realtor"  n ( r i@1 l t @ r ) ((pos "K6$" ) ))
( "Realtors"  n ( r i@1 l t @ z ) ((pos "Kj$" ) ))
( "Rebecca"  n ( r @1 b e1 k @ ) ((pos "Nl%" ) ))
( "Recife"  n ( r e1 s ii1 f @ ) ((pos "Nn%" ) ))
( "Recklinghausen"  n ( r e1 k l i ng h au z n ) ((pos "Nn$" ) ))
( "Redbrick"  j ( r e1 d b r i k ) ((pos "OA%" ) ))
( "Redbridge"  n ( r e1 d b r i jh ) ((pos "Nn$" ) ))
( "Redditch"  n ( r e1 d i ch ) ((pos "Nn$" ) ))
( "Redruth"  n ( r e1 d r uu th ) ((pos "Nn$" ) ))
( "Reg"  n ( r e1 jh ) ((pos "Nl%" ) ))
( "Regensburg"  n ( r ei1 g @ n z b @@ g ) ((pos "Nn$" ) ))
( "Regina"  n ( r i1 jh ai1 n @ ) ((pos "No%" ) ))
( "Reginald"  n ( r e1 jh i n l d ) ((pos "Nl%" ) ))
( "Reich"  n ( r ai1 k ) ((pos "K6%" ) ))
( "Reichs"  n ( r ai1 k s ) ((pos "Kj$" ) ))
( "Reigate"  n ( r ai1 g ei t ) ((pos "Nn%" ) ))
( "Remscheid"  n ( r e1 m sh ai d ) ((pos "Nn$" ) ))
( "Renfrew"  n ( r e1 n f r uu ) ((pos "Nn%" ) ))
( "Rennes"  n ( r e1 n ) ((pos "Nn%" ) ))
( "Retford"  n ( r e1 t f @ d ) ((pos "Nn$" ) ))
( "Rev"  n ( r e1 v ) ((pos "Y:%" ) ))
( "Revd"  n ( r e1 v @ r @ n d ) ((pos "Y:%" ) ))
( "Rex"  n ( r e1 k s ) ((pos "Nl%" ) ))
( "Reykjavik"  n ( r ei1 k y @ v i k ) ((pos "Nn%" ) ))
( "Rheims"  n ( r ii1 m z ) ((pos "Nn%" ) ))
( "Rhenish"  j ( r e1 n i sh ) ((pos "OA$" ) ))
( "Rheydt"  n ( r ai1 t ) ((pos "Nn$" ) ))
( "Rhine"  n ( r ai1 n ) ((pos "No%" ) ))
( "Rhinestone"  n ( r ai1 n s t ou n ) ((pos "K6%" ) ))
( "Rhinestones"  n ( r ai1 n s t ou n z ) ((pos "Kj%" ) ))
( "Rhode"  n ( r ou1 d ) ((pos "Nm%" ) ))
( "Rhondda"  n ( r o1 n d @ ) ((pos "Nn%" ) ))
( "Rhosllanerchrugog"  n ( r o2 s l a n @ k r uu1 g o g ) ((pos "Nn$" ) ))
( "Rhyl"  n ( r i1 l ) ((pos "Nn%" ) ))
( "Rhymney"  n ( r i1 m n ii ) ((pos "Nn$" ) ))
( "Richard"  n ( r i1 ch @ d ) ((pos "Nl%" ) ))
( "Richmond"  n ( r i1 ch m @ n d ) ((pos "Nn%" ) ))
( "Richmond upon Thames"  n ( r i2 ch m @ n d - uh p o n - t e1 m z ) ((pos "Nn%" ) ))
( "Rick"  n ( r i1 k ) ((pos "Nl%" ) ))
( "Rickmansworth"  n ( r i1 k m @ n z w @ th ) ((pos "Nn%" ) ))
( "Ricky"  n ( r i1 k ii ) ((pos "Nl%" ) ))
( "Riesling"  n ( r ii1 s l i ng ) ((pos "K6%" ) ))
( "Rieslings"  n ( r ii1 s l i ng z ) ((pos "Kj$" ) ))
( "Riga"  n ( r ii1 g @ ) ((pos "Nn$" ) ))
( "Rijeka"  n ( r i1 e1 k @ ) ((pos "Nn$" ) ))
( "Rimini"  n ( r i1 m i n ii ) ((pos "Nn%" ) ))
( "Ringwood"  n ( r i1 ng w u d ) ((pos "Nn$" ) ))
( "Rio de Janeiro"  n ( r i@2 u - d @ - jh @ n i@1 r ou ) ((pos "Nn%" ) ))
( "Ripley"  n ( r i1 p l ii ) ((pos "Nn$" ) ))
( "Ripon"  n ( r i1 p n ) ((pos "Nn%" ) ))
( "Risborough"  n ( r ii1 z b r @ ) ((pos "Nn$" ) ))
( "Risca"  n ( r i1 s k @ ) ((pos "Nn$" ) ))
( "Rishton"  n ( r i1 sh t @ n ) ((pos "Nn$" ) ))
( "Rita"  n ( r ii1 t @ ) ((pos "Nl%" ) ))
( "Riviera"  n ( r i2 v i e@1 r @ ) ((pos "No%" ) ))
( "Rob"  n ( r o1 b ) ((pos "Nl%" ) ))
( "Robert"  n ( r o1 b @ t ) ((pos "Nl%" ) ))
( "Robin"  n ( r o1 b i n ) ((pos "Nl%" ) ))
( "Rochdale"  n ( r o1 ch d ei l ) ((pos "Nn%" ) ))
( "Rochester"  n ( r o1 ch i s t @ r ) ((pos "Nn%" ) ))
( "Rod"  n ( r o1 d ) ((pos "Nl%" ) ))
( "Rodney"  n ( r o1 d n ii ) ((pos "Nl%" ) ))
( "Roentgen"  n ( r o1 n t y @ n ) ((pos "K6%" ) ))
( "Roentgens"  n ( r o1 n t y @ n z ) ((pos "Kj%" ) ))
( "Roger"  n ( r o1 jh @ r ) ((pos "Nl%" ) ))
( "Romaic"  n ( r ou1 m ei1 i k ) ((pos "L@$" "OA$" ) ))
( "Roman"  n ( r ou1 m @ n ) ((pos "K6%" "OA%" ) ))
( "Romance"  j ( r @1 m a1 n s ) ((pos "OA%" ) ))
( "Romanesque"  n ( r ou2 m @ n e1 s k ) ((pos "L@%" ) ))
( "Romania"  n ( r @1 m ei1 n i@ ) ((pos "Nm%" ) ))
( "Romanian"  n ( r @1 m ei1 n i@ n ) ((pos "M6%" "OA%" ) ))
( "Romanians"  n ( r @1 m ei1 n i@ n z ) ((pos "Mj%" ) ))
( "Romanies"  n ( r o1 m @ n i z ) ((pos "Mj%" ) ))
( "Romans"  n ( r ou1 m @ n z ) ((pos "Kj%" ) ))
( "Romany"  n ( r o1 m @ n ii ) ((pos "M8%" "OA%" ) ))
( "Rome"  n ( r ou1 m ) ((pos "Nn%" ) ))
( "Romish"  j ( r ou1 m i sh ) ((pos "OA$" ) ))
( "Romsey"  n ( r uh1 m z ii ) ((pos "Nn$" ) ))
( "Ron"  n ( r o1 n ) ((pos "Nl%" ) ))
( "Ronald"  n ( r o1 n l d ) ((pos "Nl%" ) ))
( "Roneo"  v ( r ou1 n i@ @ ) ((pos "H0%" "K6%" ) ))
( "Roneoed"  v ( r ou1 n i@ u d ) ((pos "Hc%" "Hd%" ) ))
( "Roneoing"  v ( r ou1 n i@ u i ng ) ((pos "Hb$" ) ))
( "Roneos"  v ( r ou1 n i@ u z ) ((pos "Ha$" "Kj$" ) ))
( "Ronnie"  n ( r o1 n ii ) ((pos "Nl%" ) ))
( "Roquefort"  n ( r o1 k f oo r ) ((pos "L@%" ) ))
( "Rosalie"  n ( r ou1 z @ l ii ) ((pos "Nl%" ) ))
( "Rosalind"  n ( r o1 z @ l i n d ) ((pos "Nl%" ) ))
( "Rosamund"  n ( r o1 z @ m @ n d ) ((pos "Nl%" ) ))
( "Rosario"  n ( r @1 z aa1 r i@ @ ) ((pos "Nn$" ) ))
( "Roscommon"  n ( r o1 s k o1 m @ n ) ((pos "Nm$" ) ))
( "Rose"  n ( r ou1 z ) ((pos "Nl%" ) ))
( "Rosemary"  n ( r ou1 z m @ r ii ) ((pos "Nl%" ) ))
( "Rosie"  n ( r ou1 z ii ) ((pos "Nl%" ) ))
( "Roslyn"  n ( r o1 z l i n ) ((pos "Nl%" ) ))
( "Ross-on-Wye"  n ( r o2 s - o n - w ai1 ) ((pos "Nn%" ) ))
( "Rossington"  n ( r o1 s i ng t @ n ) ((pos "Nn$" ) ))
( "Rosslare"  n ( r o1 s l e@ r ) ((pos "Nn%" ) ))
( "Rostock"  n ( r o1 s t o k ) ((pos "Nn$" ) ))
( "Rostov-Na-Donu"  n ( r o2 s t o f - n @ - d ou1 n uu ) ((pos "Nn$" ) ))
( "Rotarian"  n ( r ou1 t e@1 r i@ n ) ((pos "K6%" ) ))
( "Rotarians"  n ( r ou1 t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "Rotherham"  n ( r o1 dh @ r @ m ) ((pos "Nn%" ) ))
( "Rothesay"  n ( r o1 th s ii ) ((pos "Nn$" ) ))
( "Rothwell"  n ( r o1 th w e l ) ((pos "Nn$" ) ))
( "Rotterdam"  n ( r o1 t @ d a m ) ((pos "Nn%" ) ))
( "Roubaix"  n ( r uu1 b ei ) ((pos "Nn$" ) ))
( "Rouen"  n ( r uu1 o ng ) ((pos "Nn%" ) ))
( "Roundhead"  n ( r au1 n d h e d ) ((pos "K6%" ) ))
( "Roundheads"  n ( r au1 n d h e d z ) ((pos "Kj%" ) ))
( "Rowlands Gill"  n ( r ou2 l @ n z - g i1 l ) ((pos "Nn$" ) ))
( "Roy"  n ( r oi1 ) ((pos "Nl%" ) ))
( "Royston"  n ( r oi1 s t @ n ) ((pos "Nn$" ) ))
( "Rt Hon"  n ( r ai1 t - h o2 n ) ((pos "Y~%" ) ))
( "Rubicon"  n ( r uu1 b i k @ n ) ((pos "K6%" ) ))
( "Rubicons"  n ( r uu1 b i k @ n z ) ((pos "Kj$" ) ))
( "Ruddington"  n ( r uh1 d i ng t @ n ) ((pos "Nn$" ) ))
( "Rudolf"  n ( r uu1 d o l f ) ((pos "Nl%" ) ))
( "Rudy"  n ( r uu1 d ii ) ((pos "Nl%" ) ))
( "Rugby"  n ( r uh1 g b ii ) ((pos "Nn%" ) ))
( "Rugeley"  n ( r uu1 jh l ii ) ((pos "Nn%" ) ))
( "Runcorn"  n ( r uh1 ng k oo n ) ((pos "Nn%" ) ))
( "Rupert"  n ( r uu1 p @ t ) ((pos "Nl%" ) ))
( "Ruritanian"  j ( r u@2 r i t ei1 n i@ n ) ((pos "OA%" ) ))
( "Russia"  n ( r uh1 sh @ ) ((pos "Nm%" ) ))
( "Russian"  n ( r uh1 sh @ n ) ((pos "M6%" "OA%" ) ))
( "Russians"  n ( r uh1 sh @ n z ) ((pos "Mj%" ) ))
( "Ruth"  n ( r uu1 th ) ((pos "Nl%" ) ))
( "Rutherglen"  n ( r uh1 dh @ g l e n ) ((pos "Nn$" ) ))
( "Rutland"  n ( r uh1 t l @ n d ) ((pos "Nm%" ) ))
( "Rwanda"  n ( r u1 a1 n d @ ) ((pos "Nm%" ) ))
( "Rwandan"  n ( r u1 a1 n d @ n ) ((pos "K6%" "OA%" ) ))
( "Rwandans"  n ( r u1 a1 n d @ n z ) ((pos "Kj%" ) ))
( "Ryde"  n ( r ai1 d ) ((pos "Nn$" ) ))
( "Ryehill"  n ( r ai1 h i l ) ((pos "Nn$" ) ))
( "S"  n ( e1 s ) ((pos "Ki$" ) ))
( "S's"  n ( e1 s i z ) ((pos "Kj$" ) ))
( "SALT"  n ( s o1 l t ) ((pos "Y~%" ) ))
( "SOS"  n ( e2 s ou e1 s ) ((pos "Y>%" ) ))
( "SOSs"  n ( e2 s ou e1 s i z ) ((pos "Y)%" ) ))
( "SRN"  n ( e2 s aa2 r e1 n ) ((pos "Y>%" ) ))
( "SS"  n ( e2 s e1 s ) ((pos "Y>%" ) ))
( "STD"  n ( e2 s t ii2 d ii1 ) ((pos "Y~%" ) ))
( "Saarbrucken"  n ( z aa1 b r u k @ n ) ((pos "Nn%" ) ))
( "Sabadeli"  n ( s a2 b @ d e1 l ii ) ((pos "Nn$" ) ))
( "Sabah"  n ( s uh1 b @ ) ((pos "Nm$" ) ))
( "Sabahan"  n ( s uh1 b @ h @ n ) ((pos "K6$" "OA$" ) ))
( "Sabahans"  n ( s uh1 b @ h @ n z ) ((pos "Kj$" ) ))
( "Sabbath"  n ( s a1 b @ th ) ((pos "K6%" ) ))
( "Sabbaths"  n ( s a1 b @ th s ) ((pos "Kj$" ) ))
( "Sacramento"  n ( s a2 k r @ m e1 n t ou ) ((pos "Nn%" ) ))
( "Saffron Walden"  n ( s a2 f r @ n - w o1 l d n ) ((pos "Nn%" ) ))
( "Sagittarius"  n ( s a2 jh i t e@1 r i@ s ) ((pos "No%" ) ))
( "Saigon"  n ( s ai1 g o1 n ) ((pos "Nn%" ) ))
( "Salamanca"  n ( s a2 l @ m a1 ng k @ ) ((pos "Nn$" ) ))
( "Sale"  n ( s ei1 l ) ((pos "Nn$" ) ))
( "Salerno"  n ( s @1 l @@1 n ou ) ((pos "Nn%" ) ))
( "Salford"  n ( s o1 l f @ d ) ((pos "Nn%" ) ))
( "Salisbury"  n ( s o1 l z b @ r ii ) ((pos "Nn%" ) ))
( "Sally"  n ( s a1 l ii ) ((pos "Nl%" ) ))
( "Saltash"  n ( s o1 l t a sh ) ((pos "Nn$" ) ))
( "Saltburn"  n ( s o1 l t b @@ n ) ((pos "Nn$" ) ))
( "Saltcoats"  n ( s o1 l t k ou t s ) ((pos "Nn$" ) ))
( "Salvador"  n ( s a1 l v @ d oo r ) ((pos "Nn%" ) ))
( "Salvadorean"  n ( s a2 l v @ d oo1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Salvadoreans"  n ( s a2 l v @ d oo1 r i@ n z ) ((pos "Kj%" ) ))
( "Salzburg"  n ( s o1 l t s b @@ g ) ((pos "Nn%" ) ))
( "Sam"  n ( s a1 m ) ((pos "Nl%" ) ))
( "Samantha"  n ( s @1 m a1 n th @ ) ((pos "Nl%" ) ))
( "Samaritan"  n ( s @1 m a1 r i t @ n ) ((pos "K6%" ) ))
( "Samaritans"  n ( s @1 m a1 r i t @ n z ) ((pos "Kj%" ) ))
( "Sammy"  n ( s a1 m ii ) ((pos "Nl%" ) ))
( "Samoa"  n ( s @1 m ou1 @ ) ((pos "Nm%" ) ))
( "Samoan"  n ( s @1 m ou1 @ n ) ((pos "K6%" "OA%" ) ))
( "Samoans"  n ( s @1 m ou1 @ n z ) ((pos "Kj%" ) ))
( "Samuel"  n ( s a1 m y uu @ l ) ((pos "Nl%" ) ))
( "San Antonio"  n ( s a2 n - a n t ou1 n i@ @ ) ((pos "Nn%" ) ))
( "San Bernardino"  n ( s a2 n - b @@2 n @ d ii1 n ou ) ((pos "Nn$" ) ))
( "San Diego"  n ( s a2 n - d i ei1 g ou ) ((pos "Nn%" ) ))
( "San Francisco"  n ( s a2 n - f r @ n s ii1 s k ou ) ((pos "Nn%" ) ))
( "San Jose"  n ( s a2 n - h @ z ei1 ) ((pos "Nn%" ) ))
( "San Juan"  n ( s a2 n - h w aa1 n ) ((pos "Nn%" ) ))
( "San Marinese"  n ( s a2 n - m a2 r i n ii1 z ) ((pos "K9$" "OA$" ) ))
( "San Marino"  n ( s a2 n - m @ r ii1 n ou ) ((pos "Nm$" ) ))
( "San Sebastian"  n ( s a2 n - s i b a1 s t i@ n ) ((pos "Nn%" ) ))
( "Sandbach"  n ( s a1 n b a ch ) ((pos "Nn$" ) ))
( "Sandown"  n ( s a1 n d au n ) ((pos "Nn%" ) ))
( "Sandra"  n ( s aa1 n d r @ ) ((pos "Nl%" ) ))
( "Sandy"  n ( s a1 n d ii ) ((pos "Nl%" ) ))
( "Sango"  n ( s aa1 ng g ou ) ((pos "L@$" ) ))
( "Sanskrit"  n ( s a1 n s k r i t ) ((pos "L@%" ) ))
( "Santa Claus"  n ( s a1 n t @ - k l oo z ) ((pos "K7%" "No%" ) ))
( "Santa Clauses"  n ( s a1 n t @ - k l oo z i z ) ((pos "Kj$" ) ))
( "Santander"  n ( s a2 n t @ n d e@1 r ) ((pos "Nn%" ) ))
( "Santiago"  n ( s a2 n t i aa1 g ou ) ((pos "Nn%" ) ))
( "Santo Domingo"  n ( s a2 n t ou - d @ m i1 ng g ou ) ((pos "Nn$" ) ))
( "Santos"  n ( s a1 n t o s ) ((pos "Nn$" ) ))
( "Sao Paolo"  n ( s au2 - p au1 l ou ) ((pos "Nn%" ) ))
( "Sapphic"  j ( s a1 f i k ) ((pos "OA$" ) ))
( "Sapporo"  n ( s aa1 p ou r ou ) ((pos "Nn$" ) ))
( "Sara"  n ( s e@1 r @ ) ((pos "Nl%" ) ))
( "Saracen"  n ( s a1 r @ s n ) ((pos "K6%" ) ))
( "Saracens"  n ( s a1 r @ s n z ) ((pos "Kj%" ) ))
( "Sarah"  n ( s e@1 r @ ) ((pos "Nl%" ) ))
( "Sarajevo"  n ( s a2 r @ y ei1 v ou ) ((pos "Nn%" ) ))
( "Saratov"  n ( s i1 r aa1 t o f ) ((pos "Nn$" ) ))
( "Sarawak"  n ( s @1 r aa1 w a k ) ((pos "Nm%" ) ))
( "Sarawakian"  n ( s a2 r @ w a1 k i@ n ) ((pos "K6$" "OA$" ) ))
( "Sarawakians"  n ( s a2 r @ w a1 k i@ n z ) ((pos "Kj$" ) ))
( "Sarisbury"  n ( s e@1 z b @ r ii ) ((pos "Nn$" ) ))
( "Saskatchewan"  n ( s @1 s k a1 ch @ w @ n ) ((pos "Nm%" ) ))
( "Sassari"  n ( s a1 s @ r ii ) ((pos "Nn$" ) ))
( "Sassenach"  n ( s a1 s @ n a k ) ((pos "K6%" ) ))
( "Sassenachs"  n ( s a1 s @ n a k s ) ((pos "Kj%" ) ))
( "Sat"  n ( s a1 t @ d ei ) ((pos "Y=%" ) ))
( "Satan"  n ( s ei1 t n ) ((pos "K6%" ) ))
( "Satanic"  j ( s @1 t a1 n i k ) ((pos "OA%" ) ))
( "Satans"  n ( s ei1 t n z ) ((pos "Kj$" ) ))
( "Saturday"  n ( s a1 t @ d ii ) ((pos "K6%" "No%" ) ))
( "Saturdays"  n ( s a1 t @ d i z ) ((pos "Kj%" ) ))
( "Saturn"  n ( s a1 t @ n ) ((pos "No%" ) ))
( "Sauchie"  n ( s oo1 k ii ) ((pos "Nn$" ) ))
( "Saudi Arabia"  n ( s au2 d i - @ r ei1 b i@ ) ((pos "Nm%" ) ))
( "Saudi Arabian"  n ( s au2 d i - @ r ei1 b i@ n ) ((pos "K6%" "OA%" ) ))
( "Saudi Arabians"  n ( s au2 d i - @ r ei1 b i@ n z ) ((pos "Kj%" ) ))
( "Sawbridgeworth"  n ( s oo1 b r i jh w @ th ) ((pos "Nn$" ) ))
( "Saxon"  n ( s a1 k s n ) ((pos "K6%" "OA%" ) ))
( "Saxons"  n ( s a1 k s n z ) ((pos "Kj%" ) ))
( "Scandinavian"  n ( s k a2 n d i n ei1 v i@ n ) ((pos "K6%" "OA%" ) ))
( "Scandinavians"  n ( s k a2 n d i n ei1 v i@ n z ) ((pos "Kj%" ) ))
( "Scarborough"  n ( s k aa1 b r @ ) ((pos "Nn%" ) ))
( "Schwerin"  n ( sh v e1 r ii1 n ) ((pos "Nn$" ) ))
( "Sci Fi"  n ( s ai2 - f ai1 ) ((pos "Y}%" ) ))
( "Scorpio"  n ( s k oo1 p i@ @ ) ((pos "No%" ) ))
( "Scot"  n ( s k o1 t ) ((pos "K6%" ) ))
( "Scotch"  n ( s k o1 ch ) ((pos "M7%" "OA%" ) ))
( "Scotches"  n ( s k o1 ch i z ) ((pos "Mj%" ) ))
( "Scotchman"  n ( s k o1 ch m @ n ) ((pos "Ki$" ) ))
( "Scotchmen"  n ( s k o1 ch m @ n ) ((pos "Kj$" ) ))
( "Scotchwoman"  n ( s k o1 ch w u m @ n ) ((pos "Ki$" ) ))
( "Scotchwomen"  n ( s k o1 ch w i m i n ) ((pos "Kj$" ) ))
( "Scotland"  n ( s k o1 t l @ n d ) ((pos "Nm%" ) ))
( "Scotland Yard"  n ( s k o2 t l @ n d - y aa1 d ) ((pos "No%" ) ))
( "Scots"  n ( s k o1 t s ) ((pos "Kj%" "OA%" ) ))
( "Scotsman"  n ( s k o1 t s m @ n ) ((pos "Ki%" ) ))
( "Scotsmen"  n ( s k o1 t s m @ n ) ((pos "Kj%" ) ))
( "Scotswoman"  n ( s k o1 t s w u m @ n ) ((pos "Ki%" ) ))
( "Scotswomen"  n ( s k o1 t s w i m i n ) ((pos "Kj%" ) ))
( "Scottish"  j ( s k o1 t i sh ) ((pos "OA%" ) ))
( "Scrooge"  n ( s k r uu1 jh ) ((pos "K6%" ) ))
( "Scrooges"  n ( s k r uu1 jh i z ) ((pos "Kj$" ) ))
( "Scunthorpe"  n ( s k uh1 n th oo p ) ((pos "Nn%" ) ))
( "Scylla"  n ( s i1 l @ ) ((pos "K6$" ) ))
( "Scyllas"  n ( s i1 l @ z ) ((pos "Kj$" ) ))
( "Seaford"  n ( s ii1 f @ d ) ((pos "Nn$" ) ))
( "Seaham"  n ( s ii1 @ m ) ((pos "Nn$" ) ))
( "Sealyham"  n ( s ii1 l i@ m ) ((pos "K6$" ) ))
( "Sealyhams"  n ( s ii1 l i@ m z ) ((pos "Kj$" ) ))
( "Seamus"  n ( sh ei1 m @ s ) ((pos "Nl%" ) ))
( "Sean"  n ( sh oo1 n ) ((pos "Nl%" ) ))
( "Seaton Burn"  n ( s ii2 t n - b @@1 n ) ((pos "Nn$" ) ))
( "Seaton Delaval"  n ( s ii2 t n - d e1 l @ v aa l ) ((pos "Nn$" ) ))
( "Seattle"  n ( s i1 a1 t l ) ((pos "Nn%" ) ))
( "Secretary-General"  n ( s e2 k r i t e r i - jh e1 n r @ l ) ((pos "K6%" ) ))
( "Secretary-Generals"  n ( s e2 k r i t e r i - jh e1 n r @ l z ) ((pos "Kj%" ) ))
( "Securicor"  n ( s i1 k y u@1 r i k oo r ) ((pos "No%" ) ))
( "Selby"  n ( s e1 l b ii ) ((pos "Nn%" ) ))
( "Selkirk"  n ( s e1 l k @@ k ) ((pos "Nn%" ) ))
( "Selsey"  n ( s e1 l s ii ) ((pos "Nn%" ) ))
( "Selston"  n ( s e1 l s t @ n ) ((pos "Nn$" ) ))
( "Semarang"  n ( s @1 m aa1 r aa ng ) ((pos "Nn$" ) ))
( "Semite"  n ( s ii1 m ai t ) ((pos "K6$" "OA$" ) ))
( "Semites"  n ( s ii1 m ai t s ) ((pos "Kj$" ) ))
( "Semitic"  j ( s i1 m i1 t i k ) ((pos "OA%" ) ))
( "Senegal"  n ( s e2 n i g oo1 l ) ((pos "Nm%" ) ))
( "Senegalese"  n ( s e2 n i g @ l ii1 z ) ((pos "K9%" "OA%" ) ))
( "Seoul"  n ( s ou1 l ) ((pos "Nn%" ) ))
( "Sept"  n ( s e1 p t ) ((pos "Y=%" ) ))
( "September"  n ( s e1 p t e1 m b @ r ) ((pos "K6%" "No%" ) ))
( "Septembers"  n ( s e1 p t e1 m b @ z ) ((pos "Kj$" ) ))
( "Septuagint"  n ( s e1 p t y u@ jh i n t ) ((pos "K6$" ) ))
( "Septuagints"  n ( s e1 p t y u@ jh i n t s ) ((pos "Kj$" ) ))
( "Serbo-Croat"  n ( s @@2 b ou - k r ou1 a t ) ((pos "L@%" ) ))
( "Serjeant-at-arms"  n ( s aa2 jh @ n t - @ t - aa1 m z ) ((pos "Ki$" ) ))
( "Serjeants-at-arms"  n ( s aa2 jh @ n t s - @ t - aa1 m z ) ((pos "Kj$" ) ))
( "Sesotho"  n ( s i1 s uu1 t uu ) ((pos "L@$" ) ))
( "Setswana"  n ( s i1 t s w aa1 n @ ) ((pos "L@$" ) ))
( "Seven Sisters"  n ( s e2 v n - s ii1 s t @ z ) ((pos "Nn$" ) ))
( "Sevenoaks"  n ( s e1 v n ou k s ) ((pos "Nn%" ) ))
( "Sevilla"  n ( s @1 v ii1 l y @ ) ((pos "Nn$" ) ))
( "Seville"  n ( s @1 v i1 l ) ((pos "Nn%" ) ))
( "Seychelles"  n ( s ei1 sh e l z ) ((pos "Nm%" ) ))
( "Seychellois"  n ( s ei1 sh e1 l w aa ) ((pos "K9%" "OA%" ) ))
( "Sgt"  n ( s aa1 jh @ n t ) ((pos "Y:%" ) ))
( "Shakespearian"  j ( sh ei1 k s p i@1 r i@ n ) ((pos "OA%" ) ))
( "Shanghai"  n ( sh a2 ng h ai1 ) ((pos "Nn%" ) ))
( "Shanklin"  n ( sh a1 ng k l i n ) ((pos "Nn$" ) ))
( "Sharon"  n ( sh a1 r @ n ) ((pos "Nl%" ) ))
( "Shavian"  n ( sh ei1 v i@ n ) ((pos "K6$" "OA$" ) ))
( "Shavians"  n ( sh ei1 v i@ n z ) ((pos "Kj$" ) ))
( "Sheerness"  n ( sh i@2 n e1 s ) ((pos "Nn%" ) ))
( "Sheffield"  n ( sh e1 f ii l d ) ((pos "Nn%" ) ))
( "Sheila"  n ( sh ii1 l @ ) ((pos "Nl%" ) ))
( "Shelford"  n ( sh e1 l f @ d ) ((pos "Nn$" ) ))
( "Shepshed"  n ( sh e1 p sh e d ) ((pos "Nn$" ) ))
( "Shepton Mallet"  n ( sh e2 p t @ n - m a1 l i t ) ((pos "Nn%" ) ))
( "Sheraton"  n ( sh e1 r @ t @ n ) ((pos "L@$" ) ))
( "Sherbourne"  n ( sh @@1 b oo n ) ((pos "Nn%" ) ))
( "Sheringham"  n ( sh e1 r i ng @ m ) ((pos "Nn$" ) ))
( "Shetland"  n ( sh e1 t l @ n d ) ((pos "Nm%" ) ))
( "Shevington"  n ( sh e1 v i ng t @ n ) ((pos "Nn$" ) ))
( "Shihkiachwang"  n ( sh ii2 ch aa ch w a1 ng ) ((pos "Nn$" ) ))
( "Shildon"  n ( sh i1 l d n ) ((pos "Nn$" ) ))
( "Shirebrook"  n ( sh ai1 @ b r u k ) ((pos "Nn$" ) ))
( "Shiremoor"  n ( sh ai1 @ m u@ r ) ((pos "Nn$" ) ))
( "Shirley"  n ( sh @@1 l ii ) ((pos "Nl%" ) ))
( "Shotton"  n ( sh o1 t n ) ((pos "Nn$" ) ))
( "Shotts"  n ( sh o1 t s ) ((pos "Nn$" ) ))
( "Shrewsbury"  n ( sh r ou1 z b r ii ) ((pos "Nn%" ) ))
( "Shropshire"  n ( sh r o1 p sh @ r ) ((pos "Nm%" ) ))
( "Shrove Tuesday"  n ( sh r ou2 v - t y uu1 z d ii ) ((pos "K6%" "No%" ) ))
( "Shrove Tuesdays"  n ( sh r ou2 v - t y uu1 z d i z ) ((pos "Kj$" ) ))
( "Shucks"  uh ( sh uh1 k s ) ((pos "W-$" ) ))
( "Siam"  n ( s ai1 a1 m ) ((pos "Nm%" ) ))
( "Siamese"  n ( s ai2 @ m ii1 z ) ((pos "K9%" "OA%" ) ))
( "Sian"  n ( sh y aa1 n ) ((pos "Nn$" ) ))
( "Siberian"  j ( s ai1 b i@1 r i@ n ) ((pos "OA%" ) ))
( "Sicilian"  n ( s i1 s i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "Sicilians"  n ( s i1 s i1 l i@ n z ) ((pos "Kj%" ) ))
( "Sid"  n ( s i1 d ) ((pos "Nl%" ) ))
( "Sidmouth"  n ( s i1 d m @ th ) ((pos "Nn%" ) ))
( "Sidney"  n ( s i1 d n ii ) ((pos "Nl%" ) ))
( "Sierra Leone"  n ( s i1 e2 r @ - l i ou1 n ) ((pos "Nm%" ) ))
( "Sierra Leonian"  n ( s i1 e2 r @ - l i ou1 n i@ n ) ((pos "K6%" "OA%" ) ))
( "Sierra Leonians"  n ( s i1 e2 r @ - l i ou1 n i@ n z ) ((pos "Kj%" ) ))
( "Sikh"  n ( s ii1 k ) ((pos "K6%" ) ))
( "Sikhs"  n ( s ii1 k s ) ((pos "Kj%" ) ))
( "Sileby"  n ( s ai1 l b ii ) ((pos "Nn$" ) ))
( "Silsden"  n ( s i1 l z d @ n ) ((pos "Nn$" ) ))
( "Silvia"  n ( s i1 l v i@ ) ((pos "Nl%" ) ))
( "Simon"  n ( s ai1 m @ n ) ((pos "Nl%" ) ))
( "Singapore"  n ( s i2 ng @ p oo1 r ) ((pos "Nm%" ) ))
( "Singaporean"  n ( s i2 ng @ p oo1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Singaporeans"  n ( s i2 ng @ p oo1 r i@ n z ) ((pos "Kj%" ) ))
( "Singhalese"  j ( s i2 ng h @ l ii1 z ) ((pos "OA$" ) ))
( "Sinhala"  n ( s i1 n h aa1 l @ ) ((pos "L@$" ) ))
( "Sinhalese"  n ( s i2 n h @ l ii1 z ) ((pos "K9$" "OA$" ) ))
( "Sining"  n ( sh ii2 n i1 ng ) ((pos "Nn$" ) ))
( "Sinn Fein"  n ( sh i2 n - f ei1 n ) ((pos "No%" ) ))
( "Sinologist"  n ( s ai1 n o1 l @ jh i s t ) ((pos "K6$" ) ))
( "Sinologists"  n ( s ai1 n o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "Sinology"  n ( s ai1 n o1 l @ jh ii ) ((pos "L@$" ) ))
( "Sioux"  n ( s uu1 ) ((pos "K9%" ) ))
( "Sir Roger de Coverley"  n ( s @@1 - r o2 jh @ - d @ - k uh1 v @ l ii ) ((pos "Ki$" ) ))
( "Siracusa"  n ( s i2 r @ k uu1 z @ ) ((pos "Nn$" ) ))
( "Siswati"  n ( s i1 s w aa1 t ii ) ((pos "L@$" ) ))
( "Sittingbourne"  n ( s i1 t i ng b oo n ) ((pos "Nn%" ) ))
( "Skegness"  n ( s k e1 g n e1 s ) ((pos "Nn%" ) ))
( "Skelmanthorpe"  n ( s k e1 l m @ n th oo p ) ((pos "Nn$" ) ))
( "Skelmersdale"  n ( s k e1 l m @ z d ei l ) ((pos "Nn$" ) ))
( "Skewen"  n ( s k y uu1 i n ) ((pos "Nn$" ) ))
( "Skipton"  n ( s k i1 p t @ n ) ((pos "Nn%" ) ))
( "Skopje"  n ( s k oo1 p y ii ) ((pos "Nn$" ) ))
( "Slav"  n ( s l aa1 v ) ((pos "K6%" "OA%" ) ))
( "Slavonic"  j ( s l @1 v o1 n i k ) ((pos "OA%" ) ))
( "Slavs"  n ( s l aa1 v z ) ((pos "Kj%" ) ))
( "Sleaford"  n ( s l ii1 f @ d ) ((pos "Nn$" ) ))
( "Sligo"  n ( s l ai1 g ou ) ((pos "Nm%" ) ))
( "Slough"  n ( s l au1 ) ((pos "Nn%" ) ))
( "Slovak"  n ( s l ou1 v a k ) ((pos "L@%" ) ))
( "Slovenian"  n ( s l @1 v ii1 n i@ n ) ((pos "M6$" "OA$" ) ))
( "Slovenians"  n ( s l @1 v ii1 n i@ n z ) ((pos "Mj$" ) ))
( "Snr"  n ( s ii1 n i@ r ) ((pos "Y~%" ) ))
( "Soc"  n ( s o1 k ) ((pos "Y>%" ) ))
( "Socratic"  j ( s @1 k r a1 t i k ) ((pos "OA%" ) ))
( "Sofia"  n ( s ou1 f i@ ) ((pos "Nn%" ) ))
( "Soho"  n ( s ou1 h ou ) ((pos "No%" ) ))
( "Sol"  n ( s o1 l ) ((pos "No$" ) ))
( "Solicitor-General"  n ( s @1 l i2 s i t @ - jh e1 n r @ l ) ((pos "K6%" ) ))
( "Solicitor-Generals"  n ( s @1 l i2 s i t @ - jh e1 n r @ l z ) ((pos "Kj$" ) ))
( "Solihull"  n ( s ou2 l i h uh1 l ) ((pos "Nn%" ) ))
( "Solingen"  n ( z oo1 l i ng @ n ) ((pos "Nn$" ) ))
( "Solon"  n ( s ou1 l o n ) ((pos "No$" ) ))
( "Somali"  n ( s @1 m aa1 l ii ) ((pos "M6%" "OA%" ) ))
( "Somalia"  n ( s @1 m aa1 l i@ ) ((pos "Nm%" ) ))
( "Somalian"  n ( s @1 m aa1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "Somalians"  n ( s @1 m aa1 l i@ n z ) ((pos "Kj%" ) ))
( "Somalis"  n ( s @1 m aa1 l i z ) ((pos "Mj%" ) ))
( "Somercotes"  n ( s uh1 m @ k ou t s ) ((pos "Nn$" ) ))
( "Somerset"  n ( s uh1 m @ s @ t ) ((pos "Nm%" ) ))
( "Sonia"  n ( s o1 n i@ ) ((pos "Nl%" ) ))
( "Soochow"  n ( s uu2 ch au1 ) ((pos "Nn$" ) ))
( "Sophia"  n ( s @1 f ai1 @ ) ((pos "Nl%" ) ))
( "Sophie"  n ( s ou1 f ii ) ((pos "Nl%" ) ))
( "Sotho"  n ( s uu1 t uu ) ((pos "M6$" "OA$" ) ))
( "Sothos"  n ( s uu1 t uu z ) ((pos "Mj$" ) ))
( "South Kirkby"  n ( s au2 th - k @@1 k b ii ) ((pos "Nn$" ) ))
( "South Normantown"  n ( s au2 th - n oo1 m @ n t au n ) ((pos "Nn$" ) ))
( "South Ockendon"  n ( s au2 th - o1 k n d @ n ) ((pos "Nn$" ) ))
( "South Shields"  n ( s au2 th - sh ii1 l z ) ((pos "Nn%" ) ))
( "Southampton"  n ( s au1 th a1 m t @ n ) ((pos "Nn%" ) ))
( "Southend-on-Sea"  n ( s au2 th e n d - o n - s ii1 ) ((pos "Nn%" ) ))
( "Southport"  n ( s au1 th p oo t ) ((pos "Nn%" ) ))
( "Southwark"  n ( s uh1 dh @ k ) ((pos "Nn%" ) ))
( "Soviet"  n ( s ou1 v i@ t ) ((pos "K6%" "OA%" ) ))
( "Soviets"  n ( s ou1 v i@ t s ) ((pos "Kj%" ) ))
( "Spain"  n ( s p ei1 n ) ((pos "Nm%" ) ))
( "Spalding"  n ( s p o1 l d i ng ) ((pos "Nn$" ) ))
( "Spaniard"  n ( s p a1 n i@ d ) ((pos "K6%" ) ))
( "Spaniards"  n ( s p a1 n i@ d z ) ((pos "Kj%" ) ))
( "Spanish"  n ( s p a1 n i sh ) ((pos "L@%" "OA%" ) ))
( "Spartan"  n ( s p aa1 t n ) ((pos "K6%" "OA%" ) ))
( "Spartans"  n ( s p aa1 t n z ) ((pos "Kj%" ) ))
( "Spenborough"  n ( s p e1 n b @ r @ ) ((pos "Nn$" ) ))
( "Spennymoor"  n ( s p e1 n i m u@ r ) ((pos "Nn$" ) ))
( "Split"  n ( s p l i1 t ) ((pos "Nn$" ) ))
( "Spode"  n ( s p ou1 d ) ((pos "L@$" ) ))
( "Springfield"  n ( s p r i1 ng f ii l d ) ((pos "Nn$" ) ))
( "Sq"  n ( s k w e@1 r ) ((pos "Y=%" ) ))
( "Sr"  n ( s i1 s t @ r ) ((pos "Y:%" ) ))
( "Sri Lanka"  n ( s r i2 - l a1 n k @ ) ((pos "Nm%" ) ))
( "Sri Lankan"  n ( s r i2 - l a1 n k @ n ) ((pos "K6%" "OA%" ) ))
( "Sri Lankans"  n ( s r i2 - l a1 n k @ n z ) ((pos "Kj%" ) ))
( "St"  n ( s ei1 n t ) ((pos "Y=%" ) ))
( "St Albans"  n ( s n t - o1 l b n z ) ((pos "Nn%" ) ))
( "St Andrews"  n ( s n t - a1 n d r uu z ) ((pos "Nn%" ) ))
( "St Andrews Major"  n ( s n t - a2 n d r uu z - m ei1 jh @ r ) ((pos "Nn$" ) ))
( "St Austell"  n ( s n t - o1 s t l ) ((pos "Nn%" ) ))
( "St Blazey"  n ( s n t - b l ei1 z ii ) ((pos "Nn$" ) ))
( "St Helens"  n ( s n t - h e1 l n z ) ((pos "Nn%" ) ))
( "St Ives"  n ( s n t - ai1 v z ) ((pos "Nn%" ) ))
( "St Louis"  n ( s n t - l uu1 ii ) ((pos "Nn%" ) ))
( "St Neots"  n ( s n t - n i@1 t s ) ((pos "Nn$" ) ))
( "St Paul"  n ( s n t - p oo1 l ) ((pos "Nn%" ) ))
( "St Petersburg"  n ( s n t - p ii1 t @ z b @@ g ) ((pos "Nn%" ) ))
( "St Stephen"  n ( s n t - s t ii1 v n ) ((pos "Nn$" ) ))
( "Sta"  n ( s t ei1 sh @ n ) ((pos "Y=%" ) ))
( "Stafford"  n ( s t a1 f @ d ) ((pos "Nn%" ) ))
( "Staffordshire"  n ( s t a1 f @ d sh @ r ) ((pos "Nm%" ) ))
( "Stakeford"  n ( s t ei1 k f @ d ) ((pos "Nn$" ) ))
( "Stalybridge"  n ( s t ei1 l i b r i jh ) ((pos "Nn$" ) ))
( "Stamford"  n ( s t a1 m f @ d ) ((pos "Nn%" ) ))
( "Stan"  n ( s t a1 n ) ((pos "Nl%" ) ))
( "Stanford"  n ( s t a1 n f @ d ) ((pos "Nn%" ) ))
( "Stanley"  n ( s t a1 n l ii ) ((pos "Nl%" ) ))
( "Statehouse"  n ( s t ei1 t h au s ) ((pos "K6$" ) ))
( "Statehouses"  n ( s t ei1 t h au z i z ) ((pos "Kj$" ) ))
( "Stella"  n ( s t e1 l @ ) ((pos "Nl%" ) ))
( "Sten"  n ( s t e1 n ) ((pos "No$" ) ))
( "Stephanie"  n ( s t e1 f @ n ii ) ((pos "Nl%" ) ))
( "Stephen"  n ( s t ii1 v n ) ((pos "Nl%" ) ))
( "Steve"  n ( s t ii1 v ) ((pos "Nl%" ) ))
( "Steven"  n ( s t ii1 v n ) ((pos "Nl%" ) ))
( "Stevenage"  n ( s t ii1 v n i jh ) ((pos "Nn%" ) ))
( "Stevenston"  n ( s t ii1 v n s t @ n ) ((pos "Nn$" ) ))
( "Stewart"  n ( s t y uu1 @ t ) ((pos "Nl%" ) ))
( "Steyning"  n ( s t e1 n i ng ) ((pos "Nn$" ) ))
( "Stilton"  n ( s t i1 l t @ n ) ((pos "M6%" ) ))
( "Stiltons"  n ( s t i1 l t @ n z ) ((pos "Mj$" ) ))
( "Stirling"  n ( s t @@1 l i ng ) ((pos "Nn%" ) ))
( "Stockholm"  n ( s t o1 k h ou m ) ((pos "Nn%" ) ))
( "Stockport"  n ( s t o1 k p oo t ) ((pos "Nn%" ) ))
( "Stocksbridge"  n ( s t o1 k s b r i jh ) ((pos "Nn$" ) ))
( "Stoke-on-Trent"  n ( s t ou2 k - o n - t r e1 n t ) ((pos "Nn%" ) ))
( "Stone"  n ( s t ou1 n ) ((pos "Nn$" ) ))
( "Stonehouse"  n ( s t ou1 n h au s ) ((pos "Nn$" ) ))
( "Stornoway"  n ( s t oo1 n @ w ei ) ((pos "Nn%" ) ))
( "Stotfold"  n ( s t o1 t f ou l d ) ((pos "Nn$" ) ))
( "Stourbridge"  n ( s t au1 @ b r i jh ) ((pos "Nn$" ) ))
( "Stourport-on-Severn"  n ( s t au2 @ p oo t - o n - s e1 v n ) ((pos "Nn$" ) ))
( "Stowmarket"  n ( s t ou1 m aa k i t ) ((pos "Nn$" ) ))
( "Stranraer"  n ( s t r a1 n r aa1 r ) ((pos "Nn%" ) ))
( "Strasbourg"  n ( s t r a1 s b @@ g ) ((pos "Nn%" ) ))
( "Stratford-on-Avon"  n ( s t r a2 t f @ d - o n - ei1 v n ) ((pos "Nn%" ) ))
( "Strathaven"  n ( s t r ei1 v n ) ((pos "Nn$" ) ))
( "Strathclyde"  n ( s t r a1 th k l ai1 d ) ((pos "Nm%" ) ))
( "Street"  n ( s t r ii1 t ) ((pos "Nn$" ) ))
( "Stretford"  n ( s t r e1 t f @ d ) ((pos "Nn$" ) ))
( "Stroud"  n ( s t r au1 d ) ((pos "Nn$" ) ))
( "Stuart"  n ( s t y uu1 @ t ) ((pos "Nl%" ) ))
( "Studley"  n ( s t uh1 d l ii ) ((pos "Nn$" ) ))
( "Stuttgart"  n ( sh t uh1 t g aa t ) ((pos "Nn%" ) ))
( "Stygian"  j ( s t i1 jh i@ n ) ((pos "OA$" ) ))
( "Styx"  n ( s t i1 k s ) ((pos "No$" ) ))
( "Suchow"  n ( s uu2 ch au1 ) ((pos "Nn$" ) ))
( "Sudan"  n ( s uu1 d aa1 n ) ((pos "Nm%" ) ))
( "Sudanese"  n ( s uu2 d @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Sudbury"  n ( s uh1 d b r ii ) ((pos "Nn$" ) ))
( "Sue"  n ( s uu1 ) ((pos "Nl%" ) ))
( "Suffolk"  n ( s uh1 f @ k ) ((pos "Nm%" ) ))
( "Sumatra"  n ( s uu1 m aa1 t r @ ) ((pos "Nm%" ) ))
( "Sumatran"  n ( s uu1 m aa1 t r @ n ) ((pos "K6%" "OA%" ) ))
( "Sumatrans"  n ( s uu1 m aa1 t r @ n z ) ((pos "Kj%" ) ))
( "Sun"  n ( s uh1 n d ei ) ((pos "Y=%" ) ))
( "Sunday"  n ( s uh1 n d ii ) ((pos "K6%" "No%" ) ))
( "Sundays"  n ( s uh1 n d i z ) ((pos "Kj%" ) ))
( "Sunderland"  n ( s uh1 n d @ l @ n d ) ((pos "Nn%" ) ))
( "Supt"  n ( s uu2 p @ r i n t e1 n d @ n t ) ((pos "Y:%" ) ))
( "Surabaja"  n ( s u@2 r @ b ai1 @ ) ((pos "Nn$" ) ))
( "Surrey"  n ( s uh1 r ii ) ((pos "Nm%" ) ))
( "Susan"  n ( s uu1 z n ) ((pos "Nl%" ) ))
( "Susanna"  n ( s uu1 z a1 n @ ) ((pos "Nl%" ) ))
( "Susie"  n ( s u1 z ii ) ((pos "Nl%" ) ))
( "Sussex"  n ( s uh1 s i k s ) ((pos "Nm%" ) ))
( "Sutton"  n ( s uh1 t n ) ((pos "Nn$" ) ))
( "Sutton Coldfield"  n ( s uh2 t n - k ou1 l f ii l d ) ((pos "Nn%" ) ))
( "Sutton in Ashfield"  n ( s uh2 t n - i n - a1 sh f ii l d ) ((pos "Nn$" ) ))
( "Sutton-at-Hone"  n ( s uh2 t n - a t - h ou1 n ) ((pos "Nn$" ) ))
( "Suzanne"  n ( s uu1 z a1 n ) ((pos "Nl%" ) ))
( "Sverdlovsk"  n ( s f @@2 d l o1 f s k ) ((pos "Nn$" ) ))
( "Swadlincote"  n ( s w o1 d l i ng k o t ) ((pos "Nn$" ) ))
( "Swahili"  n ( s w @1 h ii1 l ii ) ((pos "L@%" ) ))
( "Swanage"  n ( s w o1 n i jh ) ((pos "Nn%" ) ))
( "Swanley"  n ( s w o1 n l ii ) ((pos "Nn$" ) ))
( "Swansea"  n ( s w o1 n z ii ) ((pos "Nn%" ) ))
( "Swazi"  n ( s w aa1 z ii ) ((pos "M6%" "OA%" ) ))
( "Swaziland"  n ( s w aa1 z i l a n d ) ((pos "Nm%" ) ))
( "Swazis"  n ( s w aa1 z i z ) ((pos "Mj%" ) ))
( "Swede"  n ( s w ii1 d ) ((pos "K6%" "OA%" ) ))
( "Sweden"  n ( s w ii1 d n ) ((pos "Nm%" ) ))
( "Swedes"  n ( s w ii1 d z ) ((pos "Kj%" ) ))
( "Swedish"  n ( s w ii1 d i sh ) ((pos "L@%" "OA%" ) ))
( "Swindon"  n ( s w i1 n d @ n ) ((pos "Nn%" ) ))
( "Swinton"  n ( s w i1 n t @ n ) ((pos "Nn$" ) ))
( "Swiss"  n ( s w i1 s ) ((pos "K9%" "OA%" ) ))
( "Switzerland"  n ( s w i1 t s @ l @ n d ) ((pos "Nm%" ) ))
( "Sydney"  n ( s i1 d n ii ) ((pos "Nn%" ) ))
( "Sylvia"  n ( s i1 l v i@ ) ((pos "Nl%" ) ))
( "Syracuse"  n ( s ai1 r @ k y uu z ) ((pos "Nn%" ) ))
( "Syria"  n ( s i1 r i@ ) ((pos "Nm%" ) ))
( "Syrian"  n ( s i1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Syrians"  n ( s i1 r i@ n z ) ((pos "Kj%" ) ))
( "Syston"  n ( s i1 s t @ n ) ((pos "Nn$" ) ))
( "T"  n ( t ii1 ) ((pos "Ki$" ) ))
( "T's"  n ( t ii1 z ) ((pos "Kj$" ) ))
( "T-junction"  n ( t ii1 - jh uh ng k sh @ n ) ((pos "K6%" ) ))
( "T-junctions"  n ( t ii1 - jh uh ng k sh @ n z ) ((pos "Kj%" ) ))
( "T-shirt"  n ( t ii1 - sh @@ t ) ((pos "K6%" ) ))
( "T-shirts"  n ( t ii1 - sh @@ t s ) ((pos "Kj%" ) ))
( "T-square"  n ( t ii1 - s k w e@ r ) ((pos "K6%" ) ))
( "T-squares"  n ( t ii1 - s k w e@ z ) ((pos "Kj%" ) ))
( "TB"  n ( t ii2 b ii1 ) ((pos "Y}%" ) ))
( "TNT"  n ( t ii2 e2 n t ii1 ) ((pos "Y}%" ) ))
( "TUC"  n ( t ii2 y uu2 s ii1 ) ((pos "Y>%" ) ))
( "TV"  n ( t ii2 v ii1 ) ((pos "Y>%" ) ))
( "TVs"  n ( t ii2 v ii1 z ) ((pos "Y)%" ) ))
( "Tadley"  n ( t a1 d l ii ) ((pos "Nn$" ) ))
( "Taegu"  n ( t ei2 g uu1 ) ((pos "Nn$" ) ))
( "Taffies"  n ( t a1 f i z ) ((pos "Kj$" ) ))
( "Taffy"  n ( t a1 f ii ) ((pos "K8$" ) ))
( "Tahiti"  n ( t aa1 h ii1 t ii ) ((pos "Nm%" ) ))
( "Tahitian"  n ( t aa1 h ii1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "Tahitians"  n ( t aa1 h ii1 sh @ n z ) ((pos "Kj%" ) ))
( "Taipei"  n ( t ai2 p ei1 ) ((pos "Nn$" ) ))
( "Taiwan"  n ( t ai1 w aa1 n ) ((pos "Nm%" ) ))
( "Taiwanese"  n ( t ai2 w @ n ii1 z ) ((pos "K9%" "OA%" ) ))
( "Taiyuan"  n ( t ai2 y uu aa1 n ) ((pos "Nn$" ) ))
( "Talmud"  n ( t a1 l m u d ) ((pos "K6%" ) ))
( "Talmuds"  n ( t a1 l m u d z ) ((pos "Kj%" ) ))
( "Tamil"  n ( t a1 m i l ) ((pos "M6%" "OA%" ) ))
( "Tamils"  n ( t a1 m i l z ) ((pos "Mj%" ) ))
( "Tammany"  n ( t a1 m @ n ii ) ((pos "Ki$" ) ))
( "Tampere"  n ( t a1 m p @ r @ ) ((pos "Nn$" ) ))
( "Tamworth"  n ( t a1 m w @ th ) ((pos "Nn$" ) ))
( "Tangshan"  n ( t a2 ng sh a1 n ) ((pos "Nn$" ) ))
( "Tanzania"  n ( t a2 n z @ n i@1 ) ((pos "Nm%" ) ))
( "Tanzanian"  n ( t a2 n z @ n i@1 n ) ((pos "K6%" "OA%" ) ))
( "Tanzanians"  n ( t a2 n z @ n i@1 n z ) ((pos "Kj%" ) ))
( "Taranaki"  n ( t a2 r @ n aa1 k ii ) ((pos "Nm$" ) ))
( "Taranto"  n ( t @1 t a1 n t ou ) ((pos "Nn$" ) ))
( "Tarrasa"  n ( t @1 r a1 s @ ) ((pos "Nn$" ) ))
( "Tashkent"  n ( t a2 sh k e1 n t ) ((pos "Nn%" ) ))
( "Tasmania"  n ( t a1 z m ei1 n i@ ) ((pos "Nm%" ) ))
( "Taunton"  n ( t oo1 n t @ n ) ((pos "Nn%" ) ))
( "Taurus"  n ( t oo1 r @ s ) ((pos "No%" ) ))
( "Taverham"  n ( t a1 v @ r @ m ) ((pos "Nn$" ) ))
( "Tavistock"  n ( t a1 v i s t o k ) ((pos "Nn$" ) ))
( "Tayside"  n ( t ei1 s ai d ) ((pos "Nm$" ) ))
( "Tbilisi"  n ( d b i1 l ii1 s ii ) ((pos "Nn$" ) ))
( "Te Deum"  n ( t ii1 - d ii1 @ m ) ((pos "K6$" ) ))
( "Te Deums"  n ( t ii1 - d ii1 @ m z ) ((pos "Kj$" ) ))
( "Tech"  n ( t e1 k ) ((pos "Y>%" ) ))
( "Technicolor"  n ( t e1 k n i k uh l @ r ) ((pos "L@%" ) ))
( "Ted"  n ( t e1 d ) ((pos "K6%" "Nl%" ) ))
( "Teddy"  n ( t e1 d ii ) ((pos "Nl%" ) ))
( "Teddy boy"  n ( t e1 d i - b oo ii ) ((pos "K6%" ) ))
( "Teddy boys"  n ( t e1 d i - b oo i z ) ((pos "Kj%" ) ))
( "Teds"  n ( t e1 d z ) ((pos "Kj$" ) ))
( "Teesside"  n ( t ii1 z - s ai d ) ((pos "Nn%" ) ))
( "Teheran"  n ( t i@2 r aa1 n ) ((pos "Nn%" ) ))
( "Teignmouth"  n ( t i1 n m @ th ) ((pos "Nn%" ) ))
( "Tel-Aviv"  n ( t e2 l - @ v ii1 v ) ((pos "Nn%" ) ))
( "Telford"  n ( t e1 l f @ d ) ((pos "Nn%" ) ))
( "Telstar"  n ( t e1 l s t aa r ) ((pos "No%" ) ))
( "Tenby"  n ( t e1 n b ii ) ((pos "Nn%" ) ))
( "Tennessee"  n ( t e2 n @ s ii1 ) ((pos "Nm%" ) ))
( "Tenterden"  n ( t e1 n t @ d @ n ) ((pos "Nn$" ) ))
( "Terence"  n ( t e1 r @ n s ) ((pos "Nl%" ) ))
( "Teresa"  n ( t @1 r ii1 z @ ) ((pos "Nl%" ) ))
( "Termi"  n ( t e@1 m ii ) ((pos "Nn$" ) ))
( "Terpsichorean"  j ( t @@2 p s i k @ r ii1 @ n ) ((pos "OA%" ) ))
( "Terr"  n ( t e1 r @ s ) ((pos "Y=%" ) ))
( "Terry"  n ( t e1 r ii ) ((pos "Nl%" ) ))
( "Tess"  n ( t e1 s ) ((pos "Nl%" ) ))
( "Tessa"  n ( t e1 s @ ) ((pos "Nl%" ) ))
( "Teuton"  n ( t y uu1 t @ n ) ((pos "K6$" ) ))
( "Teutonic"  j ( t y uu1 t o1 n i k ) ((pos "OA%" ) ))
( "Teutons"  n ( t y uu1 t @ n z ) ((pos "Kj$" ) ))
( "Tewkesbury"  n ( t y uu1 k s b @ r ii ) ((pos "Nn%" ) ))
( "Texan"  n ( t e1 k s @ n ) ((pos "K6%" "OA%" ) ))
( "Texans"  n ( t e1 k s @ n z ) ((pos "Kj%" ) ))
( "Texas"  n ( t e1 k s @ s ) ((pos "Nm%" ) ))
( "Thai"  n ( t ai1 ) ((pos "M6%" "OA%" ) ))
( "Thailand"  n ( t ai1 l a n d ) ((pos "Nm%" ) ))
( "Thais"  n ( t ai1 z ) ((pos "Mj%" ) ))
( "Thame"  n ( t ei1 m ) ((pos "Nn$" ) ))
( "Thanksgiving"  n ( th a1 ng k s g i v i ng ) ((pos "K6%" ) ))
( "Thanksgivings"  n ( th a1 ng k s g i v i ng z ) ((pos "Kj$" ) ))
( "Thatcham"  n ( th a1 ch @ m ) ((pos "Nn$" ) ))
( "Theo"  n ( th ii1 ou ) ((pos "Nl%" ) ))
( "Theodore"  n ( th ii1 @ d oo r ) ((pos "Nl%" ) ))
( "Theresa"  n ( t @1 r ii1 z @ ) ((pos "Nl%" ) ))
( "Thespian"  n ( th e1 s p i@ n ) ((pos "K6%" "OA%" ) ))
( "Thespians"  n ( th e1 s p i@ n z ) ((pos "Kj%" ) ))
( "Thessaloniki"  n ( th e2 s @ l o1 n i k ii ) ((pos "Nn%" ) ))
( "Thetford"  n ( th e1 t f @ d ) ((pos "Nn%" ) ))
( "Thionville"  n ( t i1 o ng v ii l ) ((pos "Nn$" ) ))
( "Thirsk"  n ( th @@1 s k ) ((pos "Nn$" ) ))
( "Thomas"  n ( t o1 m @ s ) ((pos "Nl%" ) ))
( "Thornbury"  n ( th oo1 n b r ii ) ((pos "Nn$" ) ))
( "Thorne"  n ( th oo1 n ) ((pos "Nn$" ) ))
( "Thurcroft"  n ( th @@1 k r o f t ) ((pos "Nn$" ) ))
( "Thurmaston"  n ( th @@1 m i s t n ) ((pos "Nn$" ) ))
( "Thurrock"  n ( th uh1 r @ k ) ((pos "Nn$" ) ))
( "Thurs"  n ( th @@1 z d ei ) ((pos "Y=%" ) ))
( "Thursday"  n ( th @@1 z d ii ) ((pos "K6%" "No%" ) ))
( "Thursdays"  n ( th @@1 z d i z ) ((pos "Kj%" ) ))
( "Thurso"  n ( th @@1 s ou ) ((pos "Nn$" ) ))
( "Tibet"  n ( t i1 b e1 t ) ((pos "Nm%" ) ))
( "Tibetan"  n ( t i1 b e1 t n ) ((pos "M6%" "OA%" ) ))
( "Tibetans"  n ( t i1 b e1 t n z ) ((pos "Mj%" ) ))
( "Tidworth"  n ( t i1 d w @ th ) ((pos "Nn$" ) ))
( "Tientsin"  n ( t i2 e n t s i1 n ) ((pos "Nn$" ) ))
( "Tilburg"  n ( t i1 l b @@ g ) ((pos "Nn$" ) ))
( "Tilbury"  n ( t i1 l b r ii ) ((pos "Nn%" ) ))
( "Tim"  n ( t i1 m ) ((pos "Nl%" ) ))
( "Timmy"  n ( t i1 m ii ) ((pos "Nl%" ) ))
( "Timothy"  n ( t i1 m @ th ii ) ((pos "Nl%" ) ))
( "Tina"  n ( t ii1 n @ ) ((pos "Nl%" ) ))
( "Tipperary"  n ( t i2 p @ r e@1 r ii ) ((pos "Nm%" ) ))
( "Tiverton"  n ( t i1 v @ t @ n ) ((pos "Nn$" ) ))
( "Tobago"  n ( t @1 b ei1 g ou ) ((pos "Nm%" ) ))
( "Tobagonian"  n ( t ou2 b @ g ou1 n i@ n ) ((pos "K6$" "OA$" ) ))
( "Tobagonians"  n ( t ou2 b @ g ou1 n i@ n z ) ((pos "Kj$" ) ))
( "Toby"  n ( t ou1 b ii ) ((pos "Nl%" ) ))
( "Todmorden"  n ( t o1 d m @ d @ n ) ((pos "Nn$" ) ))
( "Togo"  n ( t ou1 g ou ) ((pos "Nm%" ) ))
( "Togolese"  n ( t ou2 g @ l ii1 z ) ((pos "K9$" "OA$" ) ))
( "Tokay"  n ( t ou1 k ei1 ) ((pos "L@$" ) ))
( "Tokyo"  n ( t ou1 k y ou ) ((pos "Nn%" ) ))
( "Tom"  n ( t o1 m ) ((pos "Nl%" ) ))
( "Tommy"  n ( t o1 m ii ) ((pos "Nl%" ) ))
( "Tonbridge"  n ( t o1 n b r i jh ) ((pos "Nn%" ) ))
( "Tonga"  n ( t o1 ng @ ) ((pos "Nm%" ) ))
( "Tongan"  n ( t o1 ng @ n ) ((pos "M6%" "OA%" ) ))
( "Tongans"  n ( t o1 ng @ n z ) ((pos "Mj%" ) ))
( "Tony"  n ( t ou1 n ii ) ((pos "Nl%" ) ))
( "Tonyrefail"  n ( t o2 n i r e1 f ei l ) ((pos "Nn$" ) ))
( "Torbay"  n ( t oo2 b ei1 ) ((pos "Nn%" ) ))
( "Tories"  n ( t oo1 r i z ) ((pos "Kj%" ) ))
( "Torino"  n ( t @1 r ii1 n ou ) ((pos "Nn$" ) ))
( "Toronto"  n ( t @1 r o1 n t ou ) ((pos "Nn%" ) ))
( "Tory"  n ( t oo1 r ii ) ((pos "K8%" ) ))
( "Totnes"  n ( t o1 t n i s ) ((pos "Nn%" ) ))
( "Toulon"  n ( t uu1 l o1 n ) ((pos "Nn%" ) ))
( "Toulouse"  n ( t uu1 l uu1 z ) ((pos "Nn%" ) ))
( "Tours"  n ( t u@1 r ) ((pos "Nn%" ) ))
( "Tower Hamlets"  n ( t au2 @ - h a1 m l i t s ) ((pos "Nn%" ) ))
( "Tracy"  n ( t r ei1 s ii ) ((pos "Nl%" ) ))
( "Tralee"  n ( t r @1 l ii1 ) ((pos "Nn$" ) ))
( "Tranent"  n ( t r @1 n e1 n t ) ((pos "Nn$" ) ))
( "Trappist"  n ( t r a1 p i s t ) ((pos "K6$" ) ))
( "Trappists"  n ( t r a1 p i s t s ) ((pos "Kj$" ) ))
( "Tredegar"  n ( t r i1 d ii1 g @ r ) ((pos "Nn$" ) ))
( "Trevor"  n ( t r e1 v @ r ) ((pos "Nl%" ) ))
( "Trieste"  n ( t r i1 e1 s t ) ((pos "Nn%" ) ))
( "Tring"  n ( t r i1 ng ) ((pos "Nn%" ) ))
( "Trinidad"  n ( t r i1 n i d a d ) ((pos "Nm%" ) ))
( "Trinidadian"  n ( t r i2 n i d a1 d i@ n ) ((pos "K6%" "OA%" ) ))
( "Trinidadians"  n ( t r i2 n i d a1 d i@ n z ) ((pos "Kj%" ) ))
( "Trojan"  n ( t r ou1 jh @ n ) ((pos "K6%" "OA%" ) ))
( "Trojans"  n ( t r ou1 jh @ n z ) ((pos "Kj%" ) ))
( "Troon"  n ( t r uu1 n ) ((pos "Nn$" ) ))
( "Trowbridge"  n ( t r ou1 b r i jh ) ((pos "Nn$" ) ))
( "Truro"  n ( t r u@1 r ou ) ((pos "Nn%" ) ))
( "Tsar"  n ( z aa1 r ) ((pos "K6%" ) ))
( "Tsarina"  n ( z aa1 r ii1 n @ ) ((pos "K6%" ) ))
( "Tsarinas"  n ( z aa1 r ii1 n @ z ) ((pos "Kj$" ) ))
( "Tsars"  n ( z aa1 z ) ((pos "Kj%" ) ))
( "Tsinan"  n ( t s ii2 n a1 n ) ((pos "Nn$" ) ))
( "Tsingtao"  n ( t s i2 ng t au1 ) ((pos "Nn$" ) ))
( "Tsitsihar"  n ( t s i2 t s i h aa1 r ) ((pos "Nn$" ) ))
( "Tswana"  n ( t s w aa1 n @ ) ((pos "M6$" "OA$" ) ))
( "Tswanas"  n ( t s w aa1 n @ z ) ((pos "Mj$" ) ))
( "Tues"  n ( t y uu1 z d ei ) ((pos "Y=%" ) ))
( "Tuesday"  n ( t y uu1 z d ii ) ((pos "K6%" "No%" ) ))
( "Tuesdays"  n ( t y uu1 z d i z ) ((pos "Kj%" ) ))
( "Tullibody"  n ( t uh2 l i b o1 d ii ) ((pos "Nn$" ) ))
( "Tunbridge Wells"  n ( t uh2 n b r i jh - w e1 l z ) ((pos "Nn%" ) ))
( "Tunis"  n ( t y uu1 n i s ) ((pos "Nn%" ) ))
( "Tunisia"  n ( t y uu1 n i1 z i@ ) ((pos "Nm%" ) ))
( "Tunisian"  n ( t y uu1 n i1 z i@ n ) ((pos "K6%" "OA%" ) ))
( "Tunisians"  n ( t y uu1 n i1 z i@ n z ) ((pos "Kj%" ) ))
( "Turin"  n ( t y u1 r i1 n ) ((pos "Nn%" ) ))
( "Turk"  n ( t @@1 k ) ((pos "K6%" ) ))
( "Turkey"  n ( t @@1 k ii ) ((pos "Nm%" ) ))
( "Turkish"  n ( t @@1 k i sh ) ((pos "L@%" "OA%" ) ))
( "Turks"  n ( t @@1 k s ) ((pos "Kj%" ) ))
( "Turku"  n ( t @@1 k uu ) ((pos "Nn$" ) ))
( "Twelfth-night"  n ( t w e2 l f th - n ai1 t ) ((pos "K6%" "No%" ) ))
( "Twelfth-nights"  n ( t w e2 l f th - n ai1 t s ) ((pos "Kj$" ) ))
( "Tyne"  n ( t ai1 n ) ((pos "Nm%" ) ))
( "Tynemouth"  n ( t ai1 n m au th ) ((pos "Nn%" ) ))
( "Tyrone"  n ( t i1 r ou1 n ) ((pos "Nm%" ) ))
( "Tzupo"  n ( t s uu2 p ou1 ) ((pos "Nn$" ) ))
( "U"  n ( y uu1 ) ((pos "Ki$" ) ))
( "U's"  n ( y uu1 z ) ((pos "Kj$" ) ))
( "U-boat"  n ( y uu1 - b ou t ) ((pos "K6%" ) ))
( "U-boats"  n ( y uu1 - b ou t s ) ((pos "Kj%" ) ))
( "U-turn"  n ( y uu1 - t @@ n ) ((pos "K6%" ) ))
( "U-turns"  n ( y uu1 - t @@ n z ) ((pos "Kj%" ) ))
( "UDI"  n ( y uu2 d i ai1 ) ((pos "Y~%" ) ))
( "UFO"  n ( y uu2 e2 f ou1 ) ((pos "Y>%" ) ))
( "UFO's"  n ( y uu2 e2 f ou1 z ) ((pos "Y)%" ) ))
( "UK"  n ( y uu2 k ei1 ) ((pos "Y>%" ) ))
( "UN"  n ( y uu2 e1 n ) ((pos "Y>%" ) ))
( "UNESCO"  n ( y uu1 n e1 s k ou ) ((pos "Y=%" ) ))
( "UNICEF"  n ( y uu1 n i s e f ) ((pos "Y=%" ) ))
( "US"  n ( y uu2 e1 s ) ((pos "Y>%" ) ))
( "USA"  n ( y uu2 e2 s ei1 ) ((pos "Y>%" ) ))
( "USAF"  n ( y uu2 e2 s ei2 e1 f ) ((pos "Y>%" ) ))
( "USN"  n ( y uu2 e2 s e1 n ) ((pos "Y>%" ) ))
( "USS"  n ( y uu2 e2 s e1 s ) ((pos "Y>%" ) ))
( "USSR"  n ( y uu2 e2 s e2 s aa1 r ) ((pos "Y>%" ) ))
( "Uddingston"  n ( uh1 d i ng s t @ n ) ((pos "Nn$" ) ))
( "Udine"  n ( uu1 d ii n @ ) ((pos "Nn$" ) ))
( "Ufa"  n ( uu1 f aa1 ) ((pos "Nn$" ) ))
( "Uganda"  n ( y uu1 g a1 n d @ ) ((pos "Nm%" ) ))
( "Ugandan"  n ( y uu1 g a1 n d @ n ) ((pos "K6%" "OA%" ) ))
( "Ugandans"  n ( y uu1 g a1 n d @ n z ) ((pos "Kj%" ) ))
( "Ulster"  n ( uh1 l s t @ r ) ((pos "Nm%" ) ))
( "Ulverston"  n ( uh1 l v @ s t @ n ) ((pos "Nn%" ) ))
( "Unitarian"  n ( y uu2 n i t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Unitarianism"  n ( y uu2 n i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "Unitarians"  n ( y uu2 n i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "Univ"  n ( y uu2 n i v @@1 s i t ii ) ((pos "Y=%" ) ))
( "Up Holland"  n ( uh1 p - h o1 l @ n d ) ((pos "Nn$" ) ))
( "Upton"  n ( uh1 p t @ n ) ((pos "Nn$" ) ))
( "Uranus"  n ( y u1 r ei1 n @ s ) ((pos "No%" ) ))
( "Urdu"  n ( u@1 d uu ) ((pos "L@%" "OA%" ) ))
( "Ursula"  n ( @@1 s y u l @ ) ((pos "Nl%" ) ))
( "Uruguay"  n ( y u@1 r @ g w ai ) ((pos "Nm%" ) ))
( "Uruguayan"  n ( y u@2 r @ g w ai1 @ n ) ((pos "K6%" "OA%" ) ))
( "Uruguayans"  n ( y u@2 r @ g w ai1 @ n z ) ((pos "Kj%" ) ))
( "Urumchi"  n ( uu1 r uu1 m ch ii ) ((pos "Nn$" ) ))
( "Ushaw Moor"  n ( uh1 sh oo - m u@ r ) ((pos "Nn$" ) ))
( "Utah"  n ( y uu1 t oo ) ((pos "Nm%" ) ))
( "Utopia"  n ( y uu1 t ou1 p i@ ) ((pos "K6%" ) ))
( "Utopian"  j ( y uu1 t ou1 p i@ n ) ((pos "OA%" ) ))
( "Utopias"  n ( y uu1 t ou1 p i@ z ) ((pos "Kj%" ) ))
( "Utrecht"  n ( y uu1 t r e k t ) ((pos "Nn%" ) ))
( "Uttoxeter"  n ( y uu1 t o1 k s i t @ r ) ((pos "Nn%" ) ))
( "V"  n ( v ii1 ) ((pos "Ki$" ) ))
( "V and A"  n ( v ii2 - @ n - ei1 ) ((pos "Y>%" ) ))
( "V's"  n ( v ii1 z ) ((pos "Kj$" ) ))
( "VAT"  n ( v ii2 ei2 t ii1 ) ((pos "Y}%" ) ))
( "VC"  n ( v ii2 s ii1 ) ((pos "Y>%" ) ))
( "VD"  n ( v ii2 d ii1 ) ((pos "Y}%" ) ))
( "VHF"  n ( v ii2 ei2 ch e1 f ) ((pos "Y~%" ) ))
( "VIP"  n ( v ii2 ai2 p ii1 ) ((pos "Y>%" ) ))
( "VIPs"  n ( v ii2 ai2 p ii1 z ) ((pos "Y)%" ) ))
( "VP"  n ( v ii2 p ii1 ) ((pos "Y>%" ) ))
( "VSO"  n ( v ii2 e2 s ou1 ) ((pos "Y=%" ) ))
( "Valencia"  n ( v @1 l e1 n s i@ ) ((pos "Nn%" ) ))
( "Valenciennes"  n ( v @1 l o2 n s i e1 n ) ((pos "Nn%" ) ))
( "Valladolid"  n ( v a2 l @ d o l ii1 d ) ((pos "Nn$" ) ))
( "Vancouver"  n ( v a1 n k uu1 v @ r ) ((pos "Nn%" ) ))
( "Vandal"  n ( v a1 n d l ) ((pos "K6%" ) ))
( "Vandals"  n ( v a1 n d l z ) ((pos "Kj%" ) ))
( "Vanessa"  n ( v @1 n e1 s @ ) ((pos "Nl%" ) ))
( "Varanasi"  n ( v @1 r aa1 n @ s ii ) ((pos "Nn$" ) ))
( "Vasteras"  n ( v e2 s t @ r oo1 s ) ((pos "Nn$" ) ))
( "Vatican"  n ( v a1 t i k @ n ) ((pos "K6%" ) ))
( "Vaticans"  n ( v a1 t i k @ n z ) ((pos "Kj$" ) ))
( "Venetian"  j ( v @1 n ii1 sh @ n ) ((pos "OA%" ) ))
( "Venezia"  n ( v @1 n e1 t s i@ ) ((pos "Nn$" ) ))
( "Venezuela"  n ( v e2 n i z w ei1 l @ ) ((pos "Nm%" ) ))
( "Venezuelan"  n ( v e2 n i z w ei1 l @ n ) ((pos "K6%" "OA%" ) ))
( "Venezuelans"  n ( v e2 n i z w ei1 l @ n z ) ((pos "Kj%" ) ))
( "Venice"  n ( v e1 n i s ) ((pos "Nn%" ) ))
( "Ventnor"  n ( v e1 n t n @ r ) ((pos "Nn$" ) ))
( "Venus"  n ( v ii1 n @ s ) ((pos "No%" ) ))
( "Vera"  n ( v i@1 r @ ) ((pos "Nl%" ) ))
( "Verey"  j ( v e1 r ii ) ((pos "OA$" ) ))
( "Vermont"  n ( v @@1 m o1 n t ) ((pos "Nm%" ) ))
( "Vernon"  n ( v @@1 n @ n ) ((pos "Nl%" ) ))
( "Verona"  n ( v @1 r ou1 n @ ) ((pos "Nn%" ) ))
( "Veronica"  n ( v @1 r o1 n i k @ ) ((pos "Nl%" ) ))
( "Very"  j ( v e1 r ii ) ((pos "OA$" ) ))
( "Vic"  n ( v i1 k ) ((pos "Nl%" ) ))
( "Vicenza"  n ( v i1 ch e1 n z @ ) ((pos "Nn$" ) ))
( "Vicky"  n ( v i1 k ii ) ((pos "Nl%" ) ))
( "Victor"  n ( v i1 k t @ r ) ((pos "Nl%" ) ))
( "Victoria"  n ( v i1 k t oo1 r i@ ) ((pos "Nl%" ) ))
( "Victorian"  n ( v i1 k t oo1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "Victoriana"  n ( v i1 k t oo2 r i aa1 n @ ) ((pos "L@%" ) ))
( "Victorians"  n ( v i1 k t oo1 r i@ n z ) ((pos "Kj%" ) ))
( "Vienna"  n ( v i1 e1 n @ ) ((pos "Nn%" ) ))
( "Vietnam"  n ( v i2 e t n aa1 m ) ((pos "Nm%" ) ))
( "Vietnamese"  n ( v i2 e t n @ m ii1 z ) ((pos "M9%" "OA%" ) ))
( "Viewpark"  n ( v y uu1 p aa k ) ((pos "Nn$" ) ))
( "Vigo"  n ( v ii1 g ou ) ((pos "Nn$" ) ))
( "Viking"  n ( v ai1 k i ng ) ((pos "K6%" ) ))
( "Vikings"  n ( v ai1 k i ng z ) ((pos "Kj%" ) ))
( "Vince"  n ( v i1 n s ) ((pos "Nl%" ) ))
( "Vincent"  n ( v i1 n s n t ) ((pos "Nl%" ) ))
( "Viola"  n ( v ai1 @ l @ ) ((pos "Nl%" ) ))
( "Violet"  n ( v ai1 @ l @ t ) ((pos "Nl%" ) ))
( "Virginia"  n ( v @1 jh i1 n i@ ) ((pos "L@%" "Nl%" ) ))
( "Virginia Water"  n ( v @1 jh i1 n i@ - w oo2 t @ r ) ((pos "Nn%" ) ))
( "Virgo"  n ( v @@1 g ou ) ((pos "No%" ) ))
( "Vitoria"  n ( v i1 t oo1 r i@ ) ((pos "Nn$" ) ))
( "Viv"  n ( v i1 v ) ((pos "Nl%" ) ))
( "Vivian"  n ( v i1 v i@ n ) ((pos "Nl%" ) ))
( "Vivien"  n ( v i1 v i@ n ) ((pos "Nl%" ) ))
( "Vivienne"  n ( v i1 v i@ n ) ((pos "Nl%" ) ))
( "Volgograd"  n ( v o1 l g @ g r a d ) ((pos "Nn%" ) ))
( "Voronezh"  n ( v @1 r ou1 n ii ) ((pos "Nn$" ) ))
( "Vulgate"  n ( v uh1 l g ei t ) ((pos "K6$" ) ))
( "Vulgates"  n ( v uh1 l g ei t s ) ((pos "Kj$" ) ))
( "W"  n ( d uh1 b l y uu ) ((pos "Ki$" ) ))
( "W's"  n ( d uh1 b l y uu z ) ((pos "Kj$" ) ))
( "WC"  n ( d uh2 b l y uu s ii1 ) ((pos "Y>%" ) ))
( "WCs"  n ( d uh2 b l y uu s ii1 z ) ((pos "Y)%" ) ))
( "WHO"  n ( d uh2 b l y uu ei2 ch ou1 ) ((pos "Y>%" ) ))
( "WI"  n ( d uh2 b l y uu ai1 ) ((pos "Y>%" ) ))
( "WRAC"  n ( d uh2 b l y uu aa2 r ei2 s ii1 ) ((pos "Y>%" ) ))
( "WRAF"  n ( d uh2 b l y uu aa2 r ei2 e1 f ) ((pos "Y>%" ) ))
( "WRNS"  n ( r e1 n z ) ((pos "Y>%" ) ))
( "Wakefield"  n ( w ei1 k f ii l d ) ((pos "Nn%" ) ))
( "Wales"  n ( w ei1 l z ) ((pos "Nm%" ) ))
( "Wall Street"  n ( w oo1 l - s t r ii t ) ((pos "No%" ) ))
( "Wallace"  n ( w o1 l @ s ) ((pos "Nl%" ) ))
( "Wallasey"  n ( w o1 l @ s ii ) ((pos "Nn%" ) ))
( "Wallingford"  n ( w o1 l i ng f @ d ) ((pos "Nn$" ) ))
( "Wallsend"  n ( w oo1 l z e n d ) ((pos "Nn$" ) ))
( "Wally"  n ( w o1 l ii ) ((pos "Nl%" ) ))
( "Walsall"  n ( w o1 l s oo l ) ((pos "Nn%" ) ))
( "Walsham"  n ( w o1 l sh @ m ) ((pos "Nn$" ) ))
( "Walter"  n ( w oo1 l t @ r ) ((pos "Nl%" ) ))
( "Waltham Forest"  n ( w o2 l th @ m - f o1 r i s t ) ((pos "Nn%" ) ))
( "Walton"  n ( w o1 l t n ) ((pos "Nn$" ) ))
( "Wandsworth"  n ( w o1 n z w @ th ) ((pos "Nn%" ) ))
( "Wantage"  n ( w o1 n t i jh ) ((pos "Nn$" ) ))
( "Ware"  n ( w e@1 r ) ((pos "Nn$" ) ))
( "Warley"  n ( w oo1 l ii ) ((pos "Nn$" ) ))
( "Warminster"  n ( w oo1 m i n s t @ r ) ((pos "Nn$" ) ))
( "Warrington"  n ( w o1 r i ng t @ n ) ((pos "Nn%" ) ))
( "Warsaw"  n ( w oo1 s oo ) ((pos "Nn%" ) ))
( "Warsop"  n ( w oo1 s o p ) ((pos "Nn$" ) ))
( "Warwick"  n ( w o1 r i k ) ((pos "Nn%" ) ))
( "Warwickshire"  n ( w o1 r i k sh @ r ) ((pos "Nm%" ) ))
( "Washington"  n ( w o1 sh i ng t @ n ) ((pos "Nn%" ) ))
( "Washington DC"  n ( w o2 sh i ng t @ n - d ii2 - s ii1 ) ((pos "Nn%" ) ))
( "Waterford"  n ( w oo1 t @ f @ d ) ((pos "Nn%" ) ))
( "Waterloo"  n ( w oo2 t @ l uu1 ) ((pos "No%" ) ))
( "Watford"  n ( w o1 t f @ d ) ((pos "Nn%" ) ))
( "Wayne"  n ( w ei1 n ) ((pos "Nl%" ) ))
( "Wear"  n ( w i@1 r ) ((pos "Nm$" ) ))
( "Wed"  n ( w e1 n z d ei ) ((pos "Y=%" ) ))
( "Wednesday"  n ( w e1 n z d ei ) ((pos "K6%" "No%" ) ))
( "Wednesdays"  n ( w e1 n z d ei z ) ((pos "Kj%" ) ))
( "Wellingborough"  n ( w e1 l i ng b r @ ) ((pos "Nn$" ) ))
( "Wellington"  n ( w e1 l i ng t @ n ) ((pos "Nm%" ) ))
( "Wells"  n ( w e1 l z ) ((pos "Nn%" ) ))
( "Welsh"  n ( w e1 l sh ) ((pos "L@%" "OA%" ) ))
( "Welshman"  n ( w e1 l sh m @ n ) ((pos "Ki%" ) ))
( "Welshmen"  n ( w e1 l sh m @ n ) ((pos "Kj%" ) ))
( "Welshpool"  n ( w e1 l sh p uu l ) ((pos "Nn$" ) ))
( "Welwyn"  n ( w e1 l i n ) ((pos "Nn%" ) ))
( "Welwyn Garden City"  n ( w e2 l i n - g aa2 d n - s i1 t ii ) ((pos "Nn%" ) ))
( "Wendover"  n ( w e1 n d ou v @ r ) ((pos "Nn$" ) ))
( "Wendy"  n ( w e1 n d ii ) ((pos "Nl%" ) ))
( "Wesleyan"  n ( w e1 z l i@ n ) ((pos "K6%" "OA%" ) ))
( "Wesleyans"  n ( w e1 z l i@ n z ) ((pos "Kj%" ) ))
( "West Bromwich"  n ( w e2 s t - b r o1 m i ch ) ((pos "Nn%" ) ))
( "Westbury"  n ( w e1 s b @ r ii ) ((pos "Nn$" ) ))
( "Westhoughton"  n ( w e1 s t h oo1 t n ) ((pos "Nn$" ) ))
( "Westmeath"  n ( w e1 s t m ii1 dh ) ((pos "Nm$" ) ))
( "Westminster"  n ( w e1 s t m i n s t @ r ) ((pos "Nn%" ) ))
( "Westmoreland"  n ( w e1 s t m @ l @ n d ) ((pos "Nm%" ) ))
( "Weston-super-Mare"  n ( w e2 s t n - s uu2 p @ - m e@1 r ) ((pos "Nn%" ) ))
( "Wetherby"  n ( w e1 dh @ b ii ) ((pos "Nn%" ) ))
( "Wexford"  n ( w e1 k s f @ d ) ((pos "Nm%" ) ))
( "Weymouth"  n ( w ei1 m @ th ) ((pos "Nn%" ) ))
( "Whaley Bridge"  n ( w ei2 l i - b r i1 jh ) ((pos "Nn$" ) ))
( "Wheatley Hill"  n ( w ii2 t l i - h i1 l ) ((pos "Nn$" ) ))
( "Whig"  n ( w i1 g ) ((pos "K6$" ) ))
( "Whigs"  n ( w i1 g z ) ((pos "Kj$" ) ))
( "Whit"  n ( w i1 t ) ((pos "K6$" ) ))
( "Whitburn"  n ( w i1 t b @@ n ) ((pos "Nn$" ) ))
( "Whitby"  n ( w i1 t b ii ) ((pos "Nn%" ) ))
( "Whitchurch"  n ( w i1 ch @@ ch ) ((pos "Nn$" ) ))
( "Whitehall"  n ( w ai1 t h oo1 l ) ((pos "No%" ) ))
( "Whitehaven"  n ( w ai1 t h ei v n ) ((pos "Nn%" ) ))
( "Whitley Bay"  n ( w i2 t l i - b ei1 ) ((pos "Nn$" ) ))
( "Whits"  n ( w i1 t s ) ((pos "Kj$" ) ))
( "Whitstable"  n ( w i1 t s t @ b l ) ((pos "Nn%" ) ))
( "Whitsun"  n ( w i1 t s n ) ((pos "K6%" "No%" ) ))
( "Whitsuns"  n ( w i1 t s n z ) ((pos "Kj$" ) ))
( "Whitsuntide"  n ( w i1 t s @ n t ai d ) ((pos "K6%" "No%" ) ))
( "Whitsuntides"  n ( w i1 t s @ n t ai d z ) ((pos "Kj$" ) ))
( "Whittle-le-Woods"  n ( w i2 t l - i - w u1 d z ) ((pos "Nn$" ) ))
( "Whittlesey"  n ( w i1 t l s ii ) ((pos "Nn$" ) ))
( "Whitworth"  n ( w i1 t w @ th ) ((pos "Nn$" ) ))
( "Wick"  n ( w i1 k ) ((pos "Nn$" ) ))
( "Wickford"  n ( w i1 k f @ d ) ((pos "Nn$" ) ))
( "Wicklow"  n ( w i1 k l ou ) ((pos "Nm%" ) ))
( "Widnes"  n ( w i1 d n i s ) ((pos "Nn%" ) ))
( "Wiesbaden"  n ( v ii1 s b aa d n ) ((pos "Nn%" ) ))
( "Wigan"  n ( w i1 g n ) ((pos "Nn%" ) ))
( "Wight"  n ( w ai1 t ) ((pos "Nm%" ) ))
( "Wilf"  n ( w i1 l f ) ((pos "Nl%" ) ))
( "Wilfrid"  n ( w i1 l f r i d ) ((pos "Nl%" ) ))
( "Wilhelmshaven"  n ( v i1 l h e l m z h aa v n ) ((pos "Nn$" ) ))
( "Will"  n ( w i1 l ) ((pos "Nl%" ) ))
( "William"  n ( w i1 l i@ m ) ((pos "Nl%" ) ))
( "Willie"  n ( w i1 l ii ) ((pos "Nl%" ) ))
( "Willington"  n ( w i1 l i ng t @ n ) ((pos "Nn$" ) ))
( "Willy"  n ( w i1 l ii ) ((pos "Nl%" ) ))
( "Wilmslow"  n ( w i1 l m z l ou ) ((pos "Nn$" ) ))
( "Wilton"  n ( w i1 l t @ n ) ((pos "K6$" ) ))
( "Wiltons"  n ( w i1 l t @ n z ) ((pos "Kj$" ) ))
( "Wiltshire"  n ( w i1 l ch @ r ) ((pos "Nm%" ) ))
( "Wimbourne"  n ( w i1 m b oo n ) ((pos "Nn$" ) ))
( "Wimbourne Minster"  n ( w i2 m b oo n - m i1 n s t @ r ) ((pos "Nn$" ) ))
( "Winchester"  n ( w i1 n ch i s t @ r ) ((pos "Nn%" ) ))
( "Windermere"  n ( w i1 n d @ m i@ r ) ((pos "Nn%" ) ))
( "Windsor"  n ( w i1 n z @ r ) ((pos "Nn%" ) ))
( "Wingate"  n ( w i1 ng g ei t ) ((pos "Nn$" ) ))
( "Winifred"  n ( w i1 n i f r i d ) ((pos "Nl%" ) ))
( "Winnie"  n ( w i1 n ii ) ((pos "Nl%" ) ))
( "Winnipeg"  n ( w i1 n i p e g ) ((pos "Nn%" ) ))
( "Winsford"  n ( w i1 n z f @ d ) ((pos "Nn$" ) ))
( "Wisbech"  n ( w i1 z b ii ch ) ((pos "Nn$" ) ))
( "Wisconsin"  n ( w i1 s k o1 n s i n ) ((pos "Nm%" ) ))
( "Witham"  n ( w i1 t @ m ) ((pos "Nn$" ) ))
( "Withernsea"  n ( w i1 dh @ n s ii ) ((pos "Nn$" ) ))
( "Witney"  n ( w i1 t n ii ) ((pos "Nn$" ) ))
( "Woburn"  n ( w ou1 b @ n ) ((pos "Nn$" ) ))
( "Wokingham"  n ( w ou1 k i ng @ m ) ((pos "Nn$" ) ))
( "Wolverhampton"  n ( w u1 l v @ h a2 m p t @ n ) ((pos "Nn%" ) ))
( "Wolverton"  n ( w u1 l v @ t @ n ) ((pos "Nn$" ) ))
( "Wombourne"  n ( w o1 m b oo n ) ((pos "Nn$" ) ))
( "Woodbridge"  n ( w u1 d b r i jh ) ((pos "Nn$" ) ))
( "Wootton Bassett"  n ( w u2 t n - b a1 s i t ) ((pos "Nn$" ) ))
( "Worcester"  n ( w u1 s t @ r ) ((pos "Nn%" ) ))
( "Workington"  n ( w @@1 k i ng t @ n ) ((pos "Nn%" ) ))
( "Worksop"  n ( w @@1 k s o p ) ((pos "Nn$" ) ))
( "Worthing"  n ( w @@1 dh i ng ) ((pos "Nn%" ) ))
( "Wrexham"  n ( r e1 k s @ m ) ((pos "Nn%" ) ))
( "Writtle"  n ( r i1 t l ) ((pos "Nn$" ) ))
( "Wroclaw"  n ( r o1 t s l aa f ) ((pos "Nn$" ) ))
( "Wroughton"  n ( r oo1 t n ) ((pos "Nn$" ) ))
( "Wuhan"  n ( w u2 h aa1 n ) ((pos "Nn$" ) ))
( "Wuppertal"  n ( v u1 p @ t aa l ) ((pos "Nn$" ) ))
( "Wurzburg"  n ( v @@1 t s b @@ g ) ((pos "Nn$" ) ))
( "Wusih"  n ( w u2 s ii1 ) ((pos "Nn$" ) ))
( "Wycombe"  n ( w i1 k @ m ) ((pos "Nn$" ) ))
( "Wymondham"  n ( w i1 m @ n d @ m ) ((pos "Nn$" ) ))
( "Wyoming"  n ( w ai1 ou1 m i ng ) ((pos "Nm%" ) ))
( "Wythall"  n ( w i1 t l ) ((pos "Nn$" ) ))
( "X"  n ( e1 k s ) ((pos "Ki$" ) ))
( "X's"  n ( e1 k s i z ) ((pos "Kj$" ) ))
( "X-ray"  v ( e1 k s - r ei ) ((pos "H0%" "K6%" ) (subcat )))
( "X-rayed"  v ( e1 k s - r ei d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "X-raying"  v ( e1 k s - r ei i ng ) ((pos "Hb%" ) (subcat )))
( "X-rays"  v ( e1 k s - r ei z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "Xerox"  v ( z i@1 r o k s ) ((pos "H1%" "K7%" ) ))
( "Xeroxed"  v ( z i@1 r o k s t ) ((pos "Hc%" "Hd%" ) ))
( "Xeroxes"  v ( z i@1 r o k s i z ) ((pos "Ha%" "Kj%" ) ))
( "Xeroxing"  v ( z i@1 r o k s i ng ) ((pos "Hb%" ) ))
( "Xmas"  n ( e1 k s m @ s ) ((pos "K7%" "No%" ) ))
( "Xmases"  n ( e1 k s m @ s i z ) ((pos "Kj$" ) ))
( "Y"  n ( w ai1 ) ((pos "Ki$" ) ))
( "Y's"  n ( w ai1 z ) ((pos "Kj$" ) ))
( "YHA"  n ( w ai2 ei2 ch ei1 ) ((pos "Y>%" ) ))
( "YMCA"  n ( w ai2 e2 m s ii2 ei1 ) ((pos "Y>%" ) ))
( "YWCA"  n ( w ai2 d uh2 b l y uu s ii2 ei1 ) ((pos "Y>%" ) ))
( "Yank"  n ( y a1 ng k ) ((pos "K6%" ) ))
( "Yankee"  n ( y a1 ng k ii ) ((pos "K6%" ) ))
( "Yankees"  n ( y a1 ng k i z ) ((pos "Kj%" ) ))
( "Yanks"  n ( y a1 ng k s ) ((pos "Kj%" ) ))
( "Yarmouth"  n ( y aa1 m @ th ) ((pos "Nn%" ) ))
( "Yaroslavi"  n ( y a2 r @ s l aa1 v ii ) ((pos "Nn$" ) ))
( "Yate"  n ( y ei1 t ) ((pos "Nn$" ) ))
( "Yateley"  n ( y ei1 t l ii ) ((pos "Nn$" ) ))
( "Yemen"  n ( y e1 m @ n ) ((pos "Nm%" ) ))
( "Yemeni"  n ( y e1 m @ n ii ) ((pos "K6$" "OA$" ) ))
( "Yemenis"  n ( y e1 m @ n i z ) ((pos "Kj$" ) ))
( "Yeovil"  n ( y ou1 v i l ) ((pos "Nn%" ) ))
( "Yerevan"  n ( y e2 r i v aa1 n ) ((pos "Nn$" ) ))
( "Yiddish"  n ( y i1 d i sh ) ((pos "L@%" ) ))
( "Yokohama"  n ( y ou2 k @ h aa1 m @ ) ((pos "Nn%" ) ))
( "York"  n ( y oo1 k ) ((pos "Nn%" ) ))
( "Yorkley"  n ( y oo1 k l ii ) ((pos "Nn$" ) ))
( "Yorkshire"  n ( y oo1 k sh @ r ) ((pos "Nm%" ) ))
( "Ystradgynlais"  n ( i2 s t r a d g i1 n l ai s ) ((pos "Nn$" ) ))
( "Yugoslav"  n ( y uu1 g ou s l aa v ) ((pos "K6%" "OA%" ) ))
( "Yugoslavia"  n ( y uu2 g ou s l aa1 v i@ ) ((pos "Nm%" ) ))
( "Yugoslavian"  n ( y uu2 g ou s l aa1 v i@ n ) ((pos "K6%" "OA%" ) ))
( "Yugoslavians"  n ( y uu2 g ou s l aa1 v i@ n z ) ((pos "Kj%" ) ))
( "Yugoslavs"  n ( y uu1 g ou s l aa v z ) ((pos "Kj%" ) ))
( "Yukon"  n ( y uu1 k o n ) ((pos "Nm$" ) ))
( "Yvonne"  n ( i1 v o1 n ) ((pos "Nl%" ) ))
( "Z"  n ( z e1 d ) ((pos "Ki$" ) ))
( "Z's"  n ( z e1 d z ) ((pos "Kj$" ) ))
( "Zagreb"  n ( z aa1 g r e b ) ((pos "Nn%" ) ))
( "Zaire"  n ( z aa1 i@1 r ) ((pos "Nm%" ) ))
( "Zairean"  n ( z aa1 i@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "Zaireans"  n ( z aa1 i@1 r i@ n z ) ((pos "Kj$" ) ))
( "Zambia"  n ( z a1 m b i@ ) ((pos "Nm%" ) ))
( "Zambian"  n ( z a1 m b i@ n ) ((pos "K6%" "OA%" ) ))
( "Zambians"  n ( z a1 m b i@ n z ) ((pos "Kj%" ) ))
( "Zaporozhye"  n ( z a2 p @ r oo1 zh ii ) ((pos "Nn$" ) ))
( "Zaragoza"  n ( z a2 r @ g ou1 z @ ) ((pos "Nn$" ) ))
( "Zealand"  n ( z i1 l @ n d ) ((pos "Nm$" ) ))
( "Zealander"  n ( z i1 l @ n d @ r ) ((pos "K6$" "OA$" ) ))
( "Zealanders"  n ( z i1 l @ n d @ z ) ((pos "Kj$" ) ))
( "Zen"  n ( z e1 n ) ((pos "L@%" ) ))
( "Zimbabwe"  n ( z i1 m b aa1 b w ii ) ((pos "Nm%" ) ))
( "Zimbabwean"  n ( z i1 m b aa1 b w i@ n ) ((pos "K6$" "OA$" ) ))
( "Zimbabweans"  n ( z i1 m b aa1 b w i@ n z ) ((pos "Kj$" ) ))
( "Zion"  n ( z ai1 @ n ) ((pos "K6%" ) ))
( "Zionism"  n ( z ai1 @ n i z @ m ) ((pos "L@%" ) ))
( "Zionist"  n ( z ai1 @ n i s t ) ((pos "K6%" "OA%" ) ))
( "Zionists"  n ( z ai1 @ n i s t s ) ((pos "Kj%" ) ))
( "Zions"  n ( z ai1 @ n z ) ((pos "Kj$" ) ))
( "Zoe"  n ( z ou1 ii ) ((pos "Nl%" ) ))
( "Zurich"  n ( z y u@1 r i k ) ((pos "Nn%" ) ))
( "_eclair"  n ( i1 k l e@1 r ) ((pos "K6%" ) ))
( "_eclairs"  n ( i1 k l e@1 z ) ((pos "Kj%" ) ))
( "_eclat"  n ( ei1 k l aa ) ((pos "L@$" ) ))
( "_elan"  n ( ei1 l aa1 n ) ((pos "L@$" ) ))
( "_elite"  n ( ei1 l ii1 t ) ((pos "K6%" ) ))
( "_elites"  n ( ei1 l ii1 t s ) ((pos "Kj$" ) ))
( "_emigr_e"  n ( e1 m i g r ei ) ((pos "K6%" ) ))
( "_emigr_es"  n ( e1 m i g r ei z ) ((pos "Kj%" ) ))
( "_ep_ee"  n ( ei1 p ei1 ) ((pos "K6$" ) ))
( "_ep_ees"  n ( ei1 p ei1 z ) ((pos "Kj$" ) ))
( "`a la carte"  a ( aa2 - l aa - k aa1 t ) ((pos "Pu%" ) ))
( "`a la mode"  a ( aa2 - l aa - m ou1 d ) ((pos "Pu%" ) ))
( "a"  dt ( @ ) ((pos "S-*" ) ))
( "a"  n ( ei1 ) ((pos "Ki$" ) ))
( "a fortiori"  a ( ei2 - f oo2 t i oo1 r ai ) ((pos "Pu$" ) ))
( "a posteriori"  j ( ei2 - p o2 s t e r i oo1 r ai ) ((pos "OA$" "Pu$" ) ))
( "a priori"  j ( ei2 - p r ai2 oo1 r ai ) ((pos "OA$" "Pu$" ) ))
( "a's"  n ( ei1 z ) ((pos "Kj$" ) ))
( "ab initio"  a ( a2 b - i n i1 sh i@ @ ) ((pos "Pu$" ) ))
( "abaci"  n ( a1 b @ s ai ) ((pos "Kj$" ) ))
( "aback"  a ( @1 b a1 k ) ((pos "Pu%" ) ))
( "abacus"  n ( a1 b @ k @ s ) ((pos "K7%" ) ))
( "abacuses"  n ( a1 b @ k @ s i z ) ((pos "Kj%" ) ))
( "abaft"  a ( @1 b aa1 f t ) ((pos "Pu$" "T-$" ) ))
( "abandon"  v ( @1 b a1 n d @ n ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "abandoned"  v ( @1 b a1 n d @ n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "abandoning"  v ( @1 b a1 n d @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "abandonment"  n ( @1 b a1 n d @ n m @ n t ) ((pos "L@%" ) ))
( "abandons"  v ( @1 b a1 n d @ n z ) ((pos "Ha%" ) (subcat "6A" )))
( "abase"  v ( @1 b ei1 s ) ((pos "H2%" ) (subcat )))
( "abased"  v ( @1 b ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abasement"  n ( @1 b ei1 s m @ n t ) ((pos "L@%" ) ))
( "abases"  v ( @1 b ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "abash"  v ( @1 b a1 sh ) ((pos "H1%" ) (subcat )))
( "abashed"  v ( @1 b a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abashes"  v ( @1 b a1 sh i z ) ((pos "Ha$" ) (subcat )))
( "abashing"  v ( @1 b a1 sh i ng ) ((pos "Hb$" ) (subcat )))
( "abasing"  v ( @1 b ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "abate"  v ( @1 b ei1 t ) ((pos "J2%" ) (subcat "2A" )))
( "abated"  v ( @1 b ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "abatement"  n ( @1 b ei1 t m @ n t ) ((pos "L@%" ) ))
( "abates"  v ( @1 b ei1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "abating"  v ( @1 b ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "abattoir"  n ( a1 b @ t w aa r ) ((pos "K6%" ) ))
( "abattoirs"  n ( a1 b @ t w aa z ) ((pos "Kj%" ) ))
( "abb_e"  n ( a1 b ei ) ((pos "K6$" ) ))
( "abb_es"  n ( a1 b ei z ) ((pos "Kj$" ) ))
( "abbess"  n ( a1 b e s ) ((pos "K7%" ) ))
( "abbesses"  n ( a1 b e s i z ) ((pos "Kj%" ) ))
( "abbey"  n ( a1 b ii ) ((pos "K6%" ) ))
( "abbeys"  n ( a1 b i z ) ((pos "Kj%" ) ))
( "abbot"  n ( a1 b @ t ) ((pos "K6%" ) ))
( "abbots"  n ( a1 b @ t s ) ((pos "Kj%" ) ))
( "abbreviate"  v ( @1 b r ii1 v i ei t ) ((pos "H2%" ) (subcat "6A" )))
( "abbreviated"  v ( @1 b r ii1 v i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "abbreviates"  v ( @1 b r ii1 v i ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "abbreviating"  v ( @1 b r ii1 v i ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "abbreviation"  n ( @1 b r ii2 v i ei1 sh @ n ) ((pos "M6%" ) ))
( "abbreviations"  n ( @1 b r ii2 v i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "abdicate"  v ( a1 b d i k ei t ) ((pos "J2%" ) (subcat "2A" )))
( "abdicated"  v ( a1 b d i k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "abdicates"  v ( a1 b d i k ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "abdicating"  v ( a1 b d i k ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "abdication"  n ( a2 b d i k ei1 sh @ n ) ((pos "M6%" ) ))
( "abdications"  n ( a2 b d i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "abdomen"  n ( a1 b d @ m @ n ) ((pos "K6%" ) ))
( "abdomens"  n ( a1 b d @ m @ n z ) ((pos "Kj%" ) ))
( "abdominal"  j ( a1 b d o1 m i n l ) ((pos "OA%" ) ))
( "abduct"  v ( a1 b d uh1 k t ) ((pos "H0%" ) (subcat )))
( "abducted"  v ( a1 b d uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abducting"  v ( a1 b d uh1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "abduction"  n ( a1 b d uh1 k sh @ n ) ((pos "K6%" ) ))
( "abductions"  n ( a1 b d uh1 k sh @ n z ) ((pos "Kj%" ) ))
( "abducts"  v ( a1 b d uh1 k t s ) ((pos "Ha%" ) (subcat )))
( "abeam"  a ( @1 b ii1 m ) ((pos "Pu$" ) ))
( "abed"  a ( @1 b e1 d ) ((pos "Pu%" ) ))
( "aberrant"  j ( a1 b e1 r @ n t ) ((pos "OA%" ) ))
( "aberration"  n ( a2 b @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "aberrations"  n ( a2 b @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "abet"  v ( @1 b e1 t ) ((pos "H4%" ) (subcat "6A" )))
( "abets"  v ( @1 b e1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "abetted"  v ( @1 b e1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "abetting"  v ( @1 b e1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "abeyance"  n ( @1 b ei1 @ n s ) ((pos "L@%" ) ))
( "abhor"  v ( @1 b h oo1 r ) ((pos "H4%" ) (subcat )))
( "abhorred"  v ( @1 b h oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abhorrence"  n ( @1 b h o1 r @ n s ) ((pos "L@%" ) ))
( "abhorrent"  j ( @1 b h o1 r @ n t ) ((pos "OA%" ) ))
( "abhorring"  v ( @1 b h oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "abhors"  v ( @1 b h oo1 z ) ((pos "Ha%" ) (subcat )))
( "abide"  v ( @1 b ai1 d ) ((pos "J2%" ) (subcat "2C" "3A" )))
( "abided"  v ( @1 b ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" )))
( "abides"  v ( @1 b ai1 d z ) ((pos "Ja%" ) (subcat "2C" "3A" )))
( "abiding"  v ( @1 b ai1 d i ng ) ((pos "Jb%" "OA%" ) (subcat "2C" "3A" )))
( "abilities"  n ( @1 b i1 l i t i z ) ((pos "Mj%" ) ))
( "ability"  n ( @1 b i1 l i t ii ) ((pos "M8%" ) ))
( "abject"  j ( a1 b jh e k t ) ((pos "OA%" ) ))
( "abjection"  n ( a1 b jh e1 k sh @ n ) ((pos "K6%" ) ))
( "abjections"  n ( a1 b jh e1 k sh @ n z ) ((pos "Kj$" ) ))
( "abjectly"  a ( a1 b jh e k t l ii ) ((pos "Pu%" ) ))
( "abjuration"  n ( a2 b jh u@ r ei1 sh @ n ) ((pos "M6$" ) ))
( "abjurations"  n ( a2 b jh u@ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "abjure"  v ( @1 b jh u@1 r ) ((pos "H2$" ) (subcat )))
( "abjured"  v ( @1 b jh u@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "abjures"  v ( @1 b jh u@1 z ) ((pos "Ha$" ) (subcat )))
( "abjuring"  v ( @1 b jh u@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "ablative"  n ( a1 b l @ t i v ) ((pos "K6$" "OA$" ) ))
( "ablatives"  n ( a1 b l @ t i v z ) ((pos "Kj$" ) ))
( "ablaut"  n ( a1 b l au t ) ((pos "K6$" ) ))
( "ablauts"  n ( a1 b l au t s ) ((pos "Kj$" ) ))
( "ablaze"  j ( @1 b l ei1 z ) ((pos "Op%" "Pu%" ) ))
( "able"  j ( ei1 b l ) ((pos "OB*" ) ))
( "able-bodied"  j ( ei2 b @ l - b o1 d i d ) ((pos "OA%" ) ))
( "abler"  j ( ei1 b l @ r ) ((pos "Or%" ) ))
( "ablest"  j ( ei1 b l i s t ) ((pos "Os%" ) ))
( "ablution"  n ( @1 b l uu1 sh @ n ) ((pos "K6%" ) ))
( "ablutions"  n ( @1 b l uu1 sh @ n z ) ((pos "Kj%" ) ))
( "ably"  a ( ei1 b l ii ) ((pos "Pu%" ) ))
( "abnegation"  n ( a2 b n i g ei1 sh @ n ) ((pos "L@$" ) ))
( "abnormal"  j ( a1 b n oo1 m l ) ((pos "OA%" ) ))
( "abnormalities"  n ( a2 b n oo m a1 l i t i z ) ((pos "Mj%" ) ))
( "abnormality"  n ( a2 b n oo m a1 l i t ii ) ((pos "M8%" ) ))
( "abnormally"  a ( a1 b n oo1 m @ l ii ) ((pos "Pu%" ) ))
( "aboard"  a ( @1 b oo1 d ) ((pos "Pu%" "T-%" ) ))
( "abode"  v ( @1 b ou1 d ) ((pos "Ic%" "K6%" ) (subcat )))
( "abodes"  n ( @1 b ou1 d z ) ((pos "Kj%" ) ))
( "abolish"  v ( @1 b o1 l i sh ) ((pos "H1%" ) (subcat )))
( "abolished"  v ( @1 b o1 l i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abolishes"  v ( @1 b o1 l i sh i z ) ((pos "Ha%" ) (subcat )))
( "abolishing"  v ( @1 b o1 l i sh i ng ) ((pos "Hb%" ) (subcat )))
( "abolition"  n ( a2 b @ l i1 sh @ n ) ((pos "L@%" ) ))
( "abolitionist"  n ( a2 b @ l i1 sh @ n i s t ) ((pos "K6%" ) ))
( "abolitionists"  n ( a2 b @ l i1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "abominable"  j ( @1 b o1 m i n @ b l ) ((pos "OA%" ) ))
( "abominably"  a ( @1 b o1 m i n @ b l ii ) ((pos "Pu%" ) ))
( "abominate"  v ( @1 b o1 m i n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "abominated"  v ( @1 b o1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "abominates"  v ( @1 b o1 m i n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "abominating"  v ( @1 b o1 m i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "abomination"  n ( @1 b o2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "abominations"  n ( @1 b o2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "aboriginal"  n ( a2 b @ r i1 jh @ n l ) ((pos "K6$" "OA$" ) ))
( "aboriginals"  n ( a2 b @ r i1 jh @ n l z ) ((pos "Kj$" ) ))
( "aborigine"  n ( a2 b @ r i1 jh @ n ii ) ((pos "K6%" ) ))
( "aborigines"  n ( a2 b @ r i1 jh @ n i z ) ((pos "Kj%" ) ))
( "abort"  v ( @1 b oo1 t ) ((pos "J0%" ) (subcat "2A" )))
( "aborted"  v ( @1 b oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "aborting"  v ( @1 b oo1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "abortion"  n ( @1 b oo1 sh @ n ) ((pos "M6%" ) ))
( "abortionist"  n ( @1 b oo1 sh @ n i s t ) ((pos "K6%" ) ))
( "abortionists"  n ( @1 b oo1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "abortions"  n ( @1 b oo1 sh @ n z ) ((pos "Mj%" ) ))
( "abortive"  j ( @1 b oo1 t i v ) ((pos "OA%" ) ))
( "abortively"  a ( @1 b oo1 t i v l ii ) ((pos "Pu%" ) ))
( "aborts"  v ( @1 b oo1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "abound"  v ( @1 b au1 n d ) ((pos "I0%" ) (subcat )))
( "abounded"  v ( @1 b au1 n d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "abounding"  v ( @1 b au1 n d i ng ) ((pos "Ib%" ) (subcat )))
( "abounds"  v ( @1 b au1 n d z ) ((pos "Ia%" ) (subcat )))
( "about"  a ( @1 b au1 t ) ((pos "P+*" "T-*" ) ))
( "about-face"  v ( @1 b au2 t - f ei1 s ) ((pos "I2%" "Ki%" ) ))
( "about-faced"  v ( @1 b au2 t - f ei1 s t ) ((pos "Ic%" "Id%" ) ))
( "about-faces"  v ( @1 b au2 t - f ei1 s i z ) ((pos "Ia%" ) ))
( "about-facing"  v ( @1 b au2 t - f ei1 s i ng ) ((pos "Ib%" ) ))
( "above"  a ( @1 b uh1 v ) ((pos "Pu*" "T-*" ) ))
( "above board"  j ( @1 b uh2 v - b oo1 d ) ((pos "Op%" "Pu%" ) ))
( "above-mentioned"  j ( @2 b uh v - m e1 n sh @ n d ) ((pos "OA%" ) ))
( "above-named"  j ( @2 b uh v - n ei1 m d ) ((pos "OA%" ) ))
( "abracadabra"  n ( a2 b r @ k @ d a1 b r @ ) ((pos "L@$" "W-%" ) ))
( "abrade"  v ( @1 b r ei1 d ) ((pos "H2$" ) (subcat )))
( "abraded"  v ( @1 b r ei1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "abrades"  v ( @1 b r ei1 d z ) ((pos "Ha$" ) (subcat )))
( "abrading"  v ( @1 b r ei1 d i ng ) ((pos "Hb$" ) (subcat )))
( "abrasion"  n ( @1 b r ei1 zh n ) ((pos "M6%" ) ))
( "abrasions"  n ( @1 b r ei1 zh n z ) ((pos "Mj%" ) ))
( "abrasive"  n ( @1 b r ei1 s i v ) ((pos "M6%" "OA%" ) ))
( "abrasives"  n ( @1 b r ei1 s i v z ) ((pos "Mj%" ) ))
( "abreast"  a ( @1 b r e1 s t ) ((pos "Pu%" ) ))
( "abridge"  v ( @1 b r i1 jh ) ((pos "H2%" ) (subcat )))
( "abridged"  v ( @1 b r i1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abridgement"  n ( @1 b r i1 jh m @ n t ) ((pos "M6%" ) ))
( "abridgements"  n ( @1 b r i1 jh m @ n t s ) ((pos "Mj%" ) ))
( "abridges"  v ( @1 b r i1 jh i z ) ((pos "Ha%" ) (subcat )))
( "abridging"  v ( @1 b r i1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "abridgment"  n ( @1 b r i1 jh m @ n t ) ((pos "M6%" ) ))
( "abridgments"  n ( @1 b r i1 jh m @ n t s ) ((pos "Mj%" ) ))
( "abroad"  a ( @1 b r oo1 d ) ((pos "Pu%" ) ))
( "abrogate"  v ( a1 b r @ g ei t ) ((pos "H2$" ) (subcat )))
( "abrogated"  v ( a1 b r @ g ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "abrogates"  v ( a1 b r @ g ei t s ) ((pos "Ha$" ) (subcat )))
( "abrogating"  v ( a1 b r @ g ei t i ng ) ((pos "Hb$" ) (subcat )))
( "abrogation"  n ( a2 b r @ g ei1 sh @ n ) ((pos "K6$" ) ))
( "abrogations"  n ( a2 b r @ g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "abrupt"  j ( @1 b r uh1 p t ) ((pos "OA%" ) ))
( "abruptly"  a ( @1 b r uh1 p t l ii ) ((pos "Pu%" ) ))
( "abruptness"  n ( @1 b r uh1 p t n @ s ) ((pos "L@%" ) ))
( "abscess"  n ( a1 b s e s ) ((pos "K7%" ) ))
( "abscesses"  n ( a1 b s e s i z ) ((pos "Kj%" ) ))
( "abscond"  v ( @1 b s k o1 n d ) ((pos "I0%" ) (subcat "2A" )))
( "absconded"  v ( @1 b s k o1 n d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "absconding"  v ( @1 b s k o1 n d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "absconds"  v ( @1 b s k o1 n d z ) ((pos "Ia%" ) (subcat "2A" )))
( "absence"  n ( a1 b s @ n s ) ((pos "M6%" ) ))
( "absences"  n ( a1 b s @ n s i z ) ((pos "Mj%" ) ))
( "absent"  j ( a1 b s @ n t ) ((pos "OA%" ) ))
( "absent"  v ( @1 b s e1 n t ) ((pos "H0%" ) (subcat "6B" )))
( "absent-minded"  j ( a2 b s @ n t - m ai1 n d i d ) ((pos "OA%" ) ))
( "absent-mindedly"  a ( a2 b s @ n t - m ai1 n d i d l ii ) ((pos "Pu%" ) ))
( "absent-mindedness"  n ( a2 b s @ n t - m ai1 n d i d n @ s ) ((pos "L@%" ) ))
( "absented"  v ( @1 b s e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6B" )))
( "absentee"  n ( a2 b s @ n t ii1 ) ((pos "K6%" ) ))
( "absenteeism"  n ( a2 b s @ n t ii1 i z @ m ) ((pos "L@%" ) ))
( "absentees"  n ( a2 b s @ n t ii1 z ) ((pos "Kj%" ) ))
( "absenting"  v ( @1 b s e1 n t i ng ) ((pos "Hb%" ) (subcat "6B" )))
( "absently"  a ( a1 b s @ n t l ii ) ((pos "Pu%" ) ))
( "absents"  v ( @1 b s e1 n t s ) ((pos "Ha%" ) (subcat "6B" )))
( "absinth"  n ( a1 b s i n th ) ((pos "L@%" ) ))
( "absinthe"  n ( a1 b s i n th ) ((pos "L@%" ) ))
( "absolute"  j ( a1 b s @ l uu t ) ((pos "OA%" ) ))
( "absolutely"  a ( a1 b s @ l uu t l ii ) ((pos "Pu%" ) ))
( "absolution"  n ( a2 b s @ l uu1 sh @ n ) ((pos "L@%" ) ))
( "absolutism"  n ( a2 b s @ l uu1 t i z @ m ) ((pos "L@%" ) ))
( "absolve"  v ( @1 b z o1 l v ) ((pos "H2%" ) (subcat "6A" )))
( "absolved"  v ( @1 b z o1 l v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "absolves"  v ( @1 b z o1 l v z ) ((pos "Ha%" ) (subcat "6A" )))
( "absolving"  v ( @1 b z o1 l v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "absorb"  v ( @1 b s oo1 b ) ((pos "H0%" ) (subcat )))
( "absorbed"  v ( @1 b s oo1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "absorbent"  n ( @1 b s oo1 b @ n t ) ((pos "K6%" "OA%" ) ))
( "absorbents"  n ( @1 b s oo1 b @ n t s ) ((pos "Kj%" ) ))
( "absorbing"  v ( @1 b s oo1 b i ng ) ((pos "Hb%" ) (subcat )))
( "absorbs"  v ( @1 b s oo1 b z ) ((pos "Ha%" ) (subcat )))
( "absorption"  n ( @1 b s oo1 p sh @ n ) ((pos "L@%" ) ))
( "abstain"  v ( @1 b s t ei1 n ) ((pos "I0%" ) (subcat "2A" )))
( "abstained"  v ( @1 b s t ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "abstainer"  n ( @1 b s t ei1 n @ r ) ((pos "K6%" ) ))
( "abstainers"  n ( @1 b s t ei1 n @ z ) ((pos "Kj%" ) ))
( "abstaining"  v ( @1 b s t ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "abstains"  v ( @1 b s t ei1 n z ) ((pos "Ia%" ) (subcat "2A" )))
( "abstemious"  j ( @1 b s t ii1 m i@ s ) ((pos "OA%" ) ))
( "abstemiously"  a ( @1 b s t ii1 m i@ s l ii ) ((pos "Pu%" ) ))
( "abstemiousness"  n ( @1 b s t ii1 m i@ s n @ s ) ((pos "L@%" ) ))
( "abstention"  n ( @1 b s t e1 n sh @ n ) ((pos "M6$" ) ))
( "abstentions"  n ( @1 b s t e1 n sh @ n z ) ((pos "Mj$" ) ))
( "abstinence"  n ( a1 b s t i n @ n s ) ((pos "L@%" ) ))
( "abstract"  n ( a1 b s t r a k t ) ((pos "K6%" "OA%" ) ))
( "abstract"  v ( @1 b s t r a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "abstracted"  v ( @1 b s t r a1 k t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "abstractedly"  a ( @1 b s t r a1 k t i d l ii ) ((pos "Pu%" ) ))
( "abstracting"  v ( @1 b s t r a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "abstraction"  n ( @1 b s t r a1 k sh @ n ) ((pos "M6%" ) ))
( "abstractions"  n ( @1 b s t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "abstracts"  n ( a1 b s t r a k t s ) ((pos "Kj%" ) ))
( "abstracts"  v ( @1 b s t r a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "abstruse"  j ( @1 b s t r uu1 s ) ((pos "OA%" ) ))
( "abstrusely"  a ( @1 b s t r uu1 s l ii ) ((pos "Pu%" ) ))
( "abstruseness"  n ( @1 b s t r uu1 s n @ s ) ((pos "L@%" ) ))
( "absurd"  j ( @1 b s @@1 d ) ((pos "OA%" ) ))
( "absurdities"  n ( @1 b s @@1 d i t i z ) ((pos "Mj%" ) ))
( "absurdity"  n ( @1 b s @@1 d i t ii ) ((pos "M8%" ) ))
( "absurdly"  a ( @1 b s @@1 d l ii ) ((pos "Pu%" ) ))
( "abundance"  n ( @1 b uh1 n d @ n s ) ((pos "L@%" ) ))
( "abundant"  j ( @1 b uh1 n d @ n t ) ((pos "OA%" ) ))
( "abundantly"  a ( @1 b uh1 n d @ n t l ii ) ((pos "Pu%" ) ))
( "abuse"  n ( @1 b y uu1 s ) ((pos "M6%" ) ))
( "abuse"  v ( @1 b y uu1 z ) ((pos "H2%" ) (subcat )))
( "abused"  v ( @1 b y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "abuses"  n ( @1 b y uu1 s i z ) ((pos "Mj%" ) ))
( "abuses"  v ( @1 b y uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "abusing"  v ( @1 b y uu1 z i ng ) ((pos "Hb%" ) (subcat )))
( "abusive"  j ( @1 b y uu1 s i v ) ((pos "OA%" ) ))
( "abusively"  a ( @1 b y uu1 s i v l ii ) ((pos "Pu%" ) ))
( "abut"  v ( @1 b uh1 t ) ((pos "I4%" ) (subcat )))
( "abutment"  n ( @1 b uh1 t m @ n t ) ((pos "K6%" ) ))
( "abutments"  n ( @1 b uh1 t m @ n t s ) ((pos "Kj%" ) ))
( "abuts"  v ( @1 b uh1 t s ) ((pos "Ia%" ) (subcat )))
( "abutted"  v ( @1 b uh1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "abutting"  v ( @1 b uh1 t i ng ) ((pos "Ib%" ) (subcat )))
( "abysm"  n ( @1 b i1 z @ m ) ((pos "K6$" ) ))
( "abysmal"  j ( @1 b i1 z m l ) ((pos "OA%" ) ))
( "abysmally"  a ( @1 b i1 z m @ l ii ) ((pos "Pu%" ) ))
( "abysms"  n ( @1 b i1 z @ m z ) ((pos "Kj$" ) ))
( "abyss"  n ( @1 b i1 s ) ((pos "K7%" ) ))
( "abysses"  n ( @1 b i1 s i z ) ((pos "Kj%" ) ))
( "acacia"  n ( @1 k ei1 sh @ ) ((pos "K6$" ) ))
( "acacias"  n ( @1 k ei1 sh @ z ) ((pos "Kj$" ) ))
( "academic"  n ( a2 k @ d e1 m i k ) ((pos "K6%" "OA%" ) ))
( "academically"  a ( a2 k @ d e1 m i k l ii ) ((pos "Pu%" ) ))
( "academicals"  n ( a2 k @ d e1 m i k l z ) ((pos "Kj$" ) ))
( "academician"  n ( @1 k a2 d @ m i1 sh @ n ) ((pos "K6%" ) ))
( "academicians"  n ( @1 k a2 d @ m i1 sh @ n z ) ((pos "Kj%" ) ))
( "academics"  n ( a2 k @ d e1 m i k s ) ((pos "Kj%" ) ))
( "academies"  n ( @1 k a1 d @ m i z ) ((pos "Kj%" ) ))
( "academy"  n ( @1 k a1 d @ m ii ) ((pos "K8%" ) ))
( "accede"  v ( @1 k s ii1 d ) ((pos "I2%" ) (subcat "2A" )))
( "acceded"  v ( @1 k s ii1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "accedes"  v ( @1 k s ii1 d z ) ((pos "Ia%" ) (subcat "2A" )))
( "acceding"  v ( @1 k s ii1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "accelerando"  n ( a1 k s e2 l @ r a1 n d ou ) ((pos "K6$" "OA$" "Pu$" ) ))
( "accelerandos"  n ( a1 k s e2 l @ r a1 n d ou z ) ((pos "Kj$" ) ))
( "accelerate"  v ( @1 k s e1 l @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "accelerated"  v ( @1 k s e1 l @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "accelerates"  v ( @1 k s e1 l @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "accelerating"  v ( @1 k s e1 l @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "acceleration"  n ( @1 k s e2 l @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "accelerator"  n ( @1 k s e1 l @ r ei t @ r ) ((pos "K6%" ) ))
( "accelerators"  n ( @1 k s e1 l @ r ei t @ z ) ((pos "Kj%" ) ))
( "accent"  v ( a1 k s e1 n t ) ((pos "H0%" ) (subcat )))
( "accent"  n ( a1 k s @ n t ) ((pos "M6%" ) ))
( "accented"  v ( a1 k s e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "accenting"  v ( a1 k s e1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "accents"  v ( a1 k s e1 n t s ) ((pos "Ha%" ) (subcat )))
( "accents"  n ( a1 k s @ n t s ) ((pos "Mj%" ) ))
( "accentuate"  v ( @1 k s e1 n ch u ei t ) ((pos "H2%" ) (subcat )))
( "accentuated"  v ( @1 k s e1 n ch u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "accentuates"  v ( @1 k s e1 n ch u ei t s ) ((pos "Ha%" ) (subcat )))
( "accentuating"  v ( @1 k s e1 n ch u ei t i ng ) ((pos "Hb%" ) (subcat )))
( "accentuation"  n ( @1 k s e2 n ch u ei1 sh @ n ) ((pos "K6%" ) ))
( "accentuations"  n ( @1 k s e2 n ch u ei1 sh @ n z ) ((pos "Kj%" ) ))
( "accept"  v ( @1 k s e1 p t ) ((pos "J0%" ) (subcat "2A" "6A" "9" )))
( "acceptability"  n ( @1 k s e2 p t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "acceptable"  j ( @1 k s e1 p t @ b l ) ((pos "OA%" ) ))
( "acceptably"  a ( @1 k s e1 p t @ b l ii ) ((pos "Pu%" ) ))
( "acceptance"  n ( @1 k s e1 p t @ n s ) ((pos "L@%" ) ))
( "acceptation"  n ( a2 k s e p t ei1 sh @ n ) ((pos "K6$" ) ))
( "acceptations"  n ( a2 k s e p t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "accepted"  v ( @1 k s e1 p t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" )))
( "accepting"  v ( @1 k s e1 p t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "9" )))
( "accepts"  v ( @1 k s e1 p t s ) ((pos "Ja%" ) (subcat "2A" "6A" "9" )))
( "access"  n ( a1 k s e s ) ((pos "L@%" ) ))
( "accessaries"  n ( @1 k s e1 s @ r i z ) ((pos "Kj%" ) ))
( "accessary"  n ( @1 k s e1 s @ r ii ) ((pos "K8%" ) ))
( "accessibility"  n ( @1 k s e2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "accessible"  j ( @1 k s e1 s @ b l ) ((pos "OA%" ) ))
( "accession"  n ( a1 k s e1 sh @ n ) ((pos "M6%" ) ))
( "accessions"  n ( a1 k s e1 sh @ n z ) ((pos "Mj%" ) ))
( "accessories"  n ( @1 k s e1 s @ r i z ) ((pos "Kj%" ) ))
( "accessory"  n ( @1 k s e1 s @ r ii ) ((pos "K8%" ) ))
( "accidence"  n ( a1 k s i d @ n s ) ((pos "L@$" ) ))
( "accident"  n ( a1 k s i d @ n t ) ((pos "M6%" ) ))
( "accident-prone"  j ( a1 k s i d @ n t - p r ou2 n ) ((pos "OA%" ) ))
( "accidental"  j ( a2 k s i d e1 n t l ) ((pos "OA%" ) ))
( "accidentally"  a ( a2 k s i d e1 n t @ l ii ) ((pos "Pu%" ) ))
( "accidents"  n ( a1 k s i d @ n t s ) ((pos "Mj%" ) ))
( "acclaim"  v ( @1 k l ei1 m ) ((pos "H0%" "L@%" ) (subcat "6A" "16B" )))
( "acclaimed"  v ( @1 k l ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "16B" )))
( "acclaiming"  v ( @1 k l ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" "16B" )))
( "acclaims"  v ( @1 k l ei1 m z ) ((pos "Ha%" ) (subcat "6A" "16B" )))
( "acclamation"  n ( a2 k l @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "acclimate"  v ( a1 k l i m ei t ) ((pos "J2$" ) ))
( "acclimated"  v ( a1 k l i m ei t i d ) ((pos "Jc$" "Jd$" ) ))
( "acclimates"  v ( a1 k l i m ei t s ) ((pos "Ja$" ) ))
( "acclimating"  v ( a1 k l i m ei t i ng ) ((pos "Jb$" ) ))
( "acclimation"  n ( a2 k l ai m ei1 sh @ n ) ((pos "L@$" ) ))
( "acclimatization"  n ( @1 k l ai2 m @ t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "acclimatize"  v ( @1 k l ai1 m @ t ai z ) ((pos "J2%" ) (subcat "2A" )))
( "acclimatized"  v ( @1 k l ai1 m @ t ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "acclimatizes"  v ( @1 k l ai1 m @ t ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "acclimatizing"  v ( @1 k l ai1 m @ t ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "acclivities"  n ( @1 k l i1 v i t i z ) ((pos "Kj$" ) ))
( "acclivity"  n ( @1 k l i1 v i t ii ) ((pos "K8$" ) ))
( "accolade"  n ( a1 k @ l ei d ) ((pos "K6%" ) ))
( "accolades"  n ( a1 k @ l ei d z ) ((pos "Kj%" ) ))
( "accommodate"  v ( @1 k o1 m @ d ei t ) ((pos "H2%" ) (subcat "6A" )))
( "accommodated"  v ( @1 k o1 m @ d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "accommodates"  v ( @1 k o1 m @ d ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "accommodating"  v ( @1 k o1 m @ d ei t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "accommodation"  n ( @1 k o2 m @ d ei1 sh @ n ) ((pos "M6%" ) ))
( "accommodations"  n ( @1 k o2 m @ d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "accompanied"  v ( @1 k uh1 m p @ n i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "accompanies"  v ( @1 k uh1 m p @ n i z ) ((pos "Ha%" ) (subcat "6A" )))
( "accompaniment"  n ( @1 k uh1 m p @ n i m @ n t ) ((pos "K6%" ) ))
( "accompaniments"  n ( @1 k uh1 m p @ n i m @ n t s ) ((pos "Kj%" ) ))
( "accompanist"  n ( @1 k uh1 m p @ n i s t ) ((pos "K6%" ) ))
( "accompanists"  n ( @1 k uh1 m p @ n i s t s ) ((pos "Kj%" ) ))
( "accompany"  v ( @1 k uh1 m p @ n ii ) ((pos "H3%" ) (subcat "6A" )))
( "accompanying"  v ( @1 k uh1 m p @ n i i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "accomplice"  n ( @1 k uh1 m p l i s ) ((pos "K6%" ) ))
( "accomplices"  n ( @1 k uh1 m p l i s i z ) ((pos "Kj%" ) ))
( "accomplish"  v ( @1 k uh1 m p l i sh ) ((pos "H1%" ) (subcat )))
( "accomplished"  v ( @1 k uh1 m p l i sh t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "accomplishes"  v ( @1 k uh1 m p l i sh i z ) ((pos "Ha%" ) (subcat )))
( "accomplishing"  v ( @1 k uh1 m p l i sh i ng ) ((pos "Hb%" ) (subcat )))
( "accomplishment"  n ( @1 k uh1 m p l i sh m @ n t ) ((pos "M6%" ) ))
( "accomplishments"  n ( @1 k uh1 m p l i sh m @ n t s ) ((pos "Mj%" ) ))
( "accord"  v ( @1 k oo1 d ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "12A" )))
( "accordance"  n ( @1 k oo1 d @ n s ) ((pos "K6%" ) ))
( "accordances"  n ( @1 k oo1 d @ n s i z ) ((pos "Kj$" ) ))
( "accorded"  v ( @1 k oo1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "12A" )))
( "according"  v ( @1 k oo1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "12A" )))
( "according as"  cc ( @1 k oo1 d i ng - a z ) ((pos "V-%" ) ))
( "according to"  in ( @1 k oo1 d i ng - t @ ) ((pos "T-%" ) ))
( "accordingly"  a ( @1 k oo1 d i ng l ii ) ((pos "Pu%" ) ))
( "accordion"  n ( @1 k oo1 d i@ n ) ((pos "K6%" ) ))
( "accordions"  n ( @1 k oo1 d i@ n z ) ((pos "Kj%" ) ))
( "accords"  v ( @1 k oo1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "12A" )))
( "accost"  v ( @1 k o1 s t ) ((pos "H0%" ) (subcat )))
( "accosted"  v ( @1 k o1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "accosting"  v ( @1 k o1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "accosts"  v ( @1 k o1 s t s ) ((pos "Ha%" ) (subcat )))
( "accouchement"  n ( @1 k uu1 sh m o ng ) ((pos "K6$" ) ))
( "accouchements"  n ( @1 k uu1 sh m o ng z ) ((pos "Kj$" ) ))
( "account"  v ( @1 k au1 n t ) ((pos "J0%" "M6%" ) (subcat "3A" )))
( "accountable"  j ( @1 k ou1 n t @ b l ) ((pos "OA%" ) ))
( "accountancy"  n ( @1 k au1 n t @ n s ii ) ((pos "L@%" ) ))
( "accountant"  n ( @1 k au1 n t @ n t ) ((pos "K6%" ) ))
( "accountants"  n ( @1 k au1 n t @ n t s ) ((pos "Kj%" ) ))
( "accounted"  v ( @1 k au1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "accounting"  v ( @1 k au1 n t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "accounts"  v ( @1 k au1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "accoutrements"  n ( @1 k uu1 t @ m @ n t s ) ((pos "Kj$" ) ))
( "accredit"  v ( @1 k r e1 d i t ) ((pos "H0%" ) (subcat )))
( "accredited"  v ( @1 k r e1 d i t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "accrediting"  v ( @1 k r e1 d i t i ng ) ((pos "Hb%" ) (subcat )))
( "accredits"  v ( @1 k r e1 d i t s ) ((pos "Ha%" ) (subcat )))
( "accretion"  n ( @1 k r ii1 sh @ n ) ((pos "M6%" ) ))
( "accretions"  n ( @1 k r ii1 sh @ n z ) ((pos "Mj%" ) ))
( "accrue"  v ( @1 k r uu1 ) ((pos "I2%" ) (subcat "2A" )))
( "accrued"  v ( @1 k r uu1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "accrues"  v ( @1 k r uu1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "accruing"  v ( @1 k r uu1 i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "accumulate"  v ( @1 k y uu1 m y u l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "accumulated"  v ( @1 k y uu1 m y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "accumulates"  v ( @1 k y uu1 m y u l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "accumulating"  v ( @1 k y uu1 m y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "accumulation"  n ( @1 k y uu2 m y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "accumulations"  n ( @1 k y uu2 m y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "accumulative"  j ( @1 k y uu1 m y u l @ t i v ) ((pos "OA%" ) ))
( "accumulator"  n ( @1 k y uu1 m y u l ei t @ r ) ((pos "K6%" ) ))
( "accumulators"  n ( @1 k y uu1 m y u l ei t @ z ) ((pos "Kj%" ) ))
( "accuracies"  n ( a1 k y @ r @ s i z ) ((pos "Mj%" ) ))
( "accuracy"  n ( a1 k y @ r @ s ii ) ((pos "M8%" ) ))
( "accurate"  j ( a1 k y @ r @ t ) ((pos "OA%" ) ))
( "accurately"  a ( a1 k y @ r @ t l ii ) ((pos "Pu%" ) ))
( "accursed"  j ( @1 k @@1 s i d ) ((pos "OA%" ) ))
( "accurst"  j ( @1 k @@1 s t ) ((pos "OA%" ) ))
( "accusation"  n ( a2 k y uu z ei1 sh @ n ) ((pos "M6%" ) ))
( "accusations"  n ( a2 k y uu z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "accusative"  n ( @1 k y uu1 z @ t i v ) ((pos "K6%" "OA%" ) ))
( "accusatives"  n ( @1 k y uu1 z @ t i v z ) ((pos "Kj%" ) ))
( "accuse"  v ( @1 k y uu1 z ) ((pos "H2%" ) (subcat "6A" )))
( "accused"  v ( @1 k y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "accuser"  n ( @1 k y uu1 z @ r ) ((pos "K6%" ) ))
( "accusers"  n ( @1 k y uu1 z @ z ) ((pos "Kj%" ) ))
( "accuses"  v ( @1 k y uu1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "accusing"  v ( @1 k y uu1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "accusingly"  a ( @1 k y uu1 z i ng l ii ) ((pos "Pu%" ) ))
( "accustom"  v ( @1 k uh1 s t @ m ) ((pos "H0%" ) (subcat )))
( "accustomed"  v ( @1 k uh1 s t @ m d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "accustoming"  v ( @1 k uh1 s t @ m i ng ) ((pos "Hb%" ) (subcat )))
( "accustoms"  v ( @1 k uh1 s t @ m z ) ((pos "Ha%" ) (subcat )))
( "ace"  n ( ei1 s ) ((pos "K6%" ) ))
( "acerbities"  n ( @1 s @@1 b i t i z ) ((pos "Mj$" ) ))
( "acerbity"  n ( @1 s @@1 b i t ii ) ((pos "M8%" ) ))
( "aces"  n ( ei1 s i z ) ((pos "Kj%" ) ))
( "acetate"  n ( a1 s i t ei t ) ((pos "K6$" ) ))
( "acetates"  n ( a1 s i t ei t s ) ((pos "Kj$" ) ))
( "acetic"  j ( @1 s ii1 t i k ) ((pos "OA%" ) ))
( "acetylene"  n ( @1 s e1 t i l ii n ) ((pos "L@%" ) ))
( "ache"  v ( ei1 k ) ((pos "I2%" "K6%" ) (subcat "2A" "3A" )))
( "ached"  v ( ei1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "aches"  v ( ei1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" "3A" )))
( "achievable"  j ( @1 ch ii1 v @ b l ) ((pos "OA%" ) ))
( "achieve"  v ( @1 ch ii1 v ) ((pos "H2%" ) (subcat )))
( "achieved"  v ( @1 ch ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "achievement"  n ( @1 ch ii1 v m @ n t ) ((pos "M6%" ) ))
( "achievements"  n ( @1 ch ii1 v m @ n t s ) ((pos "Mj%" ) ))
( "achieves"  v ( @1 ch ii1 v z ) ((pos "Ha%" ) (subcat )))
( "achieving"  v ( @1 ch ii1 v i ng ) ((pos "Hb%" ) (subcat )))
( "aching"  v ( ei1 k i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "acid"  n ( a1 s i d ) ((pos "M6%" "OA%" ) ))
( "acidic"  j ( @1 s i1 d i k ) ((pos "OA%" ) ))
( "acidified"  v ( @1 s i1 d i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "acidifies"  v ( @1 s i1 d i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "acidify"  v ( @1 s i1 d i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "acidifying"  v ( @1 s i1 d i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "acidity"  n ( @1 s i1 d i t ii ) ((pos "L@%" ) ))
( "acids"  n ( a1 s i d z ) ((pos "Mj%" ) ))
( "acidulated"  j ( @1 s i1 d y u l ei t i d ) ((pos "OA$" ) ))
( "acidulous"  j ( @1 s i1 d y u l @ s ) ((pos "OA$" ) ))
( "ack-ack"  n ( a2 k - a1 k ) ((pos "L@$" ) ))
( "acknowledge"  v ( @1 k n o1 l i jh ) ((pos "H2%" ) (subcat "6A" "6C" "9" "16B" "24A" )))
( "acknowledged"  v ( @1 k n o1 l i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "16B" "24A" )))
( "acknowledgement"  n ( @1 k n o1 l i jh m @ n t ) ((pos "K6%" ) ))
( "acknowledgements"  n ( @1 k n o1 l i jh m @ n t s ) ((pos "Kj%" ) ))
( "acknowledges"  v ( @1 k n o1 l i jh i z ) ((pos "Ha%" ) (subcat "6A" "6C" "9" "16B" "24A" )))
( "acknowledging"  v ( @1 k n o1 l i jh i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "16B" "24A" )))
( "acme"  n ( a1 k m ii ) ((pos "Ki%" ) ))
( "acne"  n ( a1 k n ii ) ((pos "L@%" ) ))
( "acolyte"  n ( a1 k @ l ai t ) ((pos "K6$" ) ))
( "acolytes"  n ( a1 k @ l ai t s ) ((pos "Kj$" ) ))
( "aconite"  n ( a1 k @ n ai t ) ((pos "K6$" ) ))
( "aconites"  n ( a1 k @ n ai t s ) ((pos "Kj$" ) ))
( "acorn"  n ( ei1 k oo n ) ((pos "K6%" ) ))
( "acorn-cup"  n ( ei1 k oo n - k uh p ) ((pos "K6$" ) ))
( "acorn-cups"  n ( ei1 k oo n - k uh p s ) ((pos "Kj$" ) ))
( "acorns"  n ( ei1 k oo n z ) ((pos "Kj%" ) ))
( "acoustic"  n ( @1 k uu1 s t i k ) ((pos "K6%" "OA%" ) ))
( "acoustics"  n ( @1 k uu1 s t i k s ) ((pos "Lk%" ) ))
( "acquaint"  v ( @1 k w ei1 n t ) ((pos "H0%" ) (subcat )))
( "acquaintance"  n ( @1 k w ei1 n t @ n s ) ((pos "M6%" ) ))
( "acquaintances"  n ( @1 k w ei1 n t @ n s i z ) ((pos "Mj%" ) ))
( "acquaintanceship"  n ( @1 k w ei1 n t @ n sh i p ) ((pos "K6%" ) ))
( "acquaintanceships"  n ( @1 k w ei1 n t @ n sh i p s ) ((pos "Kj%" ) ))
( "acquainted"  v ( @1 k w ei1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "acquainting"  v ( @1 k w ei1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "acquaints"  v ( @1 k w ei1 n t s ) ((pos "Ha%" ) (subcat )))
( "acquiesce"  v ( a2 k w i e1 s ) ((pos "I2%" ) (subcat "2A" )))
( "acquiesced"  v ( a2 k w i e1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "acquiescence"  n ( a2 k w i e1 s n s ) ((pos "K6%" ) ))
( "acquiescences"  n ( a2 k w i e1 s n s i z ) ((pos "Kj$" ) ))
( "acquiescent"  j ( a2 k w i e1 s n t ) ((pos "OA%" ) ))
( "acquiesces"  v ( a2 k w i e1 s i z ) ((pos "Ia%" ) (subcat "2A" )))
( "acquiescing"  v ( a2 k w i e1 s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "acquire"  v ( @1 k w ai1 @ r ) ((pos "H2%" ) (subcat )))
( "acquired"  v ( @1 k w ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "acquirement"  n ( @1 k w ai1 @ m @ n t ) ((pos "M6%" ) ))
( "acquirements"  n ( @1 k w ai1 @ m @ n t s ) ((pos "Mj%" ) ))
( "acquires"  v ( @1 k w ai1 @ z ) ((pos "Ha%" ) (subcat )))
( "acquiring"  v ( @1 k w ai1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "acquisition"  n ( a2 k w i z i1 sh @ n ) ((pos "M6%" ) ))
( "acquisitions"  n ( a2 k w i z i1 sh @ n z ) ((pos "Mj%" ) ))
( "acquisitive"  j ( @1 k w i1 z @ t i v ) ((pos "OA%" ) ))
( "acquit"  v ( @1 k w i1 t ) ((pos "H4%" ) (subcat "6A" "14" )))
( "acquits"  v ( @1 k w i1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "acquittal"  n ( @1 k w i1 t l ) ((pos "M6%" ) ))
( "acquittals"  n ( @1 k w i1 t l z ) ((pos "Mj%" ) ))
( "acquitted"  v ( @1 k w i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "acquitting"  v ( @1 k w i1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "acre"  n ( ei1 k @ r ) ((pos "K6%" ) ))
( "acreage"  n ( ei1 k @ r i jh ) ((pos "L@%" ) ))
( "acres"  n ( ei1 k @ z ) ((pos "Kj%" ) ))
( "acrid"  j ( a1 k r i d ) ((pos "OA%" ) ))
( "acrimonious"  j ( a2 k r i m ou1 n i@ s ) ((pos "OA%" ) ))
( "acrimony"  n ( a1 k r i m @ n ii ) ((pos "L@%" ) ))
( "acrobat"  n ( a1 k r @ b a t ) ((pos "K6%" ) ))
( "acrobatic"  j ( a2 k r @ b a1 t i k ) ((pos "OA%" ) ))
( "acrobatics"  n ( a2 k r @ b a1 t i k s ) ((pos "Lk%" ) ))
( "acrobats"  n ( a1 k r @ b a t s ) ((pos "Kj%" ) ))
( "acronym"  n ( a1 k r @ n i m ) ((pos "K6%" ) ))
( "acronyms"  n ( a1 k r @ n i m z ) ((pos "Kj%" ) ))
( "acropolis"  n ( @1 k r o1 p @ l i s ) ((pos "K7$" ) ))
( "acropolises"  n ( @1 k r o1 p @ l i s i z ) ((pos "Kj$" ) ))
( "across"  a ( @1 k r o1 s ) ((pos "Pu*" "T-*" ) ))
( "acrostic"  n ( @1 k r o1 s t i k ) ((pos "K6$" ) ))
( "acrostics"  n ( @1 k r o1 s t i k s ) ((pos "Kj$" ) ))
( "acrylic"  n ( @1 k r i1 l i k ) ((pos "K6%" ) ))
( "acrylics"  n ( @1 k r i1 l i k s ) ((pos "Kj%" ) ))
( "act"  v ( a1 k t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "acted"  v ( a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "acting"  v ( a1 k t i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" "3A" "6A" )))
( "actinic"  j ( a1 k t i1 n i k ) ((pos "OA$" ) ))
( "actinism"  n ( a1 k t i n i z @ m ) ((pos "L@$" ) ))
( "action"  n ( a1 k sh @ n ) ((pos "M6%" ) ))
( "actionable"  j ( a1 k sh @ n @ b l ) ((pos "OA%" ) ))
( "actions"  n ( a1 k sh @ n z ) ((pos "Mj%" ) ))
( "activate"  v ( a1 k t i v ei t ) ((pos "H2%" ) (subcat )))
( "activated"  v ( a1 k t i v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "activates"  v ( a1 k t i v ei t s ) ((pos "Ha%" ) (subcat )))
( "activating"  v ( a1 k t i v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "activation"  n ( a2 k t i v ei1 sh @ n ) ((pos "K6%" ) ))
( "activations"  n ( a2 k t i v ei1 sh @ n z ) ((pos "Kj%" ) ))
( "active"  j ( a1 k t i v ) ((pos "OA%" ) ))
( "actively"  a ( a1 k t i v l ii ) ((pos "Pu%" ) ))
( "activist"  n ( a1 k t i v i s t ) ((pos "K6%" ) ))
( "activists"  n ( a1 k t i v i s t s ) ((pos "Kj%" ) ))
( "activities"  n ( a1 k t i1 v i t i z ) ((pos "Mj%" ) ))
( "activity"  n ( a1 k t i1 v i t ii ) ((pos "M8%" ) ))
( "actor"  n ( a1 k t @ r ) ((pos "K6%" ) ))
( "actors"  n ( a1 k t @ z ) ((pos "Kj%" ) ))
( "actress"  n ( a1 k t r i s ) ((pos "K7%" ) ))
( "actresses"  n ( a1 k t r i s i z ) ((pos "Kj%" ) ))
( "acts"  v ( a1 k t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "actual"  j ( a1 k ch u@ l ) ((pos "OA%" ) ))
( "actualities"  n ( a2 k ch u a1 l i t i z ) ((pos "Mj%" ) ))
( "actuality"  n ( a2 k ch u a1 l i t ii ) ((pos "M8%" ) ))
( "actually"  a ( a1 k ch u l ii ) ((pos "Pu%" ) ))
( "actuarial"  j ( a2 k ch u e@1 r i@ l ) ((pos "OA%" ) ))
( "actuaries"  n ( a1 k ch u@ r i z ) ((pos "Kj%" ) ))
( "actuary"  n ( a1 k ch u@ r ii ) ((pos "K8%" ) ))
( "actuate"  v ( a1 k ch u ei t ) ((pos "H2%" ) (subcat )))
( "actuated"  v ( a1 k ch u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "actuates"  v ( a1 k ch u ei t s ) ((pos "Ha%" ) (subcat )))
( "actuating"  v ( a1 k ch u ei t i ng ) ((pos "Hb%" ) (subcat )))
( "acuity"  n ( @1 k y uu1 i t ii ) ((pos "L@$" ) ))
( "acumen"  n ( a1 k y u m @ n ) ((pos "L@$" ) ))
( "acupuncture"  n ( a1 k y u p uh ng k ch @ r ) ((pos "L@%" ) ))
( "acute"  j ( @1 k y uu1 t ) ((pos "OA%" ) ))
( "acutely"  a ( @1 k y uu1 t l ii ) ((pos "Pu%" ) ))
( "acuteness"  n ( @1 k y uu1 t n @ s ) ((pos "L@%" ) ))
( "ad"  n ( a1 d ) ((pos "K6%" ) ))
( "ad hoc"  j ( a2 d - h o1 k ) ((pos "OA%" "Pu%" ) ))
( "ad infinitum"  a ( a2 d - i2 n f i n ai1 t @ m ) ((pos "Pu%" ) ))
( "ad interim"  a ( a2 d - i1 n t @ r i m ) ((pos "Pu$" ) ))
( "ad lib"  a ( a2 d - l i1 b ) ((pos "Pu%" ) ))
( "ad libitum"  a ( a2 d - l i1 b i t @ m ) ((pos "Pu$" ) ))
( "ad nauseam"  a ( a2 d - n oo1 z i a m ) ((pos "Pu%" ) ))
( "ad valorem"  a ( a2 d - v @ l oo1 r @ m ) ((pos "Pu$" ) ))
( "ad-lib"  v ( a1 d - l i1 b ) ((pos "I4%" "OA%" ) (subcat )))
( "ad-libbed"  v ( a1 d - l i1 b d ) ((pos "Ic%" "Id%" ) (subcat )))
( "ad-libbing"  v ( a1 d - l i1 b i ng ) ((pos "Ib%" ) (subcat )))
( "ad-libs"  v ( a1 d - l i1 b z ) ((pos "Ia%" ) (subcat )))
( "ad-man"  n ( a1 d - m a n ) ((pos "Ki$" ) ))
( "ad-men"  n ( a1 d - m e n ) ((pos "Kj$" ) ))
( "adage"  n ( a1 d i jh ) ((pos "K6%" ) ))
( "adages"  n ( a1 d i jh i z ) ((pos "Kj%" ) ))
( "adagio"  n ( @1 d aa1 jh i@ @ ) ((pos "K6%" "OA%" "Pu%" ) ))
( "adagios"  n ( @1 d aa1 jh i@ u z ) ((pos "Kj$" ) ))
( "adamant"  n ( a1 d @ m @ n t ) ((pos "K6%" "Op%" ) ))
( "adamantine"  j ( a2 d @ m a1 n t ai n ) ((pos "OA$" ) ))
( "adamants"  n ( a1 d @ m @ n t s ) ((pos "Kj$" ) ))
( "adapt"  v ( @1 d a1 p t ) ((pos "H0%" ) (subcat "6A" )))
( "adaptability"  n ( @1 d a2 p t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "adaptable"  j ( @1 d a1 p t @ b l ) ((pos "OA%" ) ))
( "adaptation"  n ( a2 d a p t ei1 sh @ n ) ((pos "M6%" ) ))
( "adaptations"  n ( a2 d a p t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "adapted"  v ( @1 d a1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "adapter"  n ( @1 d a1 p t @ r ) ((pos "K6%" ) ))
( "adapters"  n ( @1 d a1 p t @ z ) ((pos "Kj%" ) ))
( "adapting"  v ( @1 d a1 p t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "adaptor"  n ( @1 d a1 p t @ r ) ((pos "K6%" ) ))
( "adaptors"  n ( @1 d a1 p t @ z ) ((pos "Kj%" ) ))
( "adapts"  v ( @1 d a1 p t s ) ((pos "Ha%" ) (subcat "6A" )))
( "add"  v ( a1 d ) ((pos "J0%" ) (subcat "2C" "3A" "6A" "9" "14" )))
( "added"  v ( a1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "9" "14" )))
( "addenda"  n ( @1 d e1 n d @ ) ((pos "Kj%" ) ))
( "addendum"  n ( @1 d e1 n d @ m ) ((pos "Ki%" ) ))
( "adder"  n ( a1 d @ r ) ((pos "K6%" ) ))
( "adders"  n ( a1 d @ z ) ((pos "Kj%" ) ))
( "addict"  n ( a1 d i k t ) ((pos "K6%" ) ))
( "addict"  v ( @1 d i1 k t ) ((pos "H0%" ) ))
( "addicted"  v ( @1 d i1 k t i d ) ((pos "Hc%" "Hd%" ) ))
( "addicting"  v ( @1 d i1 k t i ng ) ((pos "Hb%" ) ))
( "addiction"  n ( @1 d i1 k sh @ n ) ((pos "M6%" ) ))
( "addictions"  n ( @1 d i1 k sh @ n z ) ((pos "Mj%" ) ))
( "addictive"  j ( @1 d i1 k t i v ) ((pos "OA%" ) ))
( "addicts"  n ( a1 d i k t s ) ((pos "Kj%" ) ))
( "addicts"  v ( @1 d i1 k t s ) ((pos "Ha%" ) ))
( "adding"  v ( a1 d i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "9" "14" )))
( "adding-machine"  n ( a1 d i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "adding-machines"  n ( a1 d i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "addition"  n ( @1 d i1 sh @ n ) ((pos "M6%" ) ))
( "additional"  j ( @1 d i1 sh @ n l ) ((pos "OA%" ) ))
( "additionally"  a ( @1 d i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "additions"  n ( @1 d i1 sh @ n z ) ((pos "Mj%" ) ))
( "additive"  n ( a1 d i t i v ) ((pos "K6%" ) ))
( "additives"  n ( a1 d i t i v z ) ((pos "Kj%" ) ))
( "addle"  v ( a1 d l ) ((pos "J2%" "OA%" ) (subcat "2A" )))
( "addle-brained"  j ( a1 d l - b r ei n d ) ((pos "OA%" ) ))
( "addle-head"  n ( a1 d l - h e d ) ((pos "K6$" ) ))
( "addle-heads"  n ( a1 d l - h e d z ) ((pos "Kj$" ) ))
( "addle-pated"  j ( a1 d l - p ei2 t i d ) ((pos "OA%" ) ))
( "addled"  v ( a1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "addles"  v ( a1 d l z ) ((pos "Ja$" ) (subcat "2A" )))
( "addling"  v ( a1 d l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "address"  v ( @1 d r e1 s ) ((pos "H1%" "M7%" ) (subcat "6A" "14" )))
( "addressed"  v ( @1 d r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "addressee"  n ( a2 d r e s ii1 ) ((pos "K6%" ) ))
( "addressees"  n ( a2 d r e s ii1 z ) ((pos "Kj%" ) ))
( "addresses"  v ( @1 d r e1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "addressing"  v ( @1 d r e1 s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "adds"  v ( a1 d z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "9" "14" )))
( "adduce"  v ( @1 d y uu1 s ) ((pos "H2%" ) (subcat )))
( "adduced"  v ( @1 d y uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "adduces"  v ( @1 d y uu1 s i z ) ((pos "Ha%" ) (subcat )))
( "adducing"  v ( @1 d y uu1 s i ng ) ((pos "Hb%" ) (subcat )))
( "adenoidal"  j ( a2 d i n oi1 d l ) ((pos "OA%" ) ))
( "adenoids"  n ( a1 d i n oi d z ) ((pos "Kj%" ) ))
( "adept"  n ( a1 d e p t ) ((pos "K6$" "OA%" ) ))
( "adepts"  n ( a1 d e p t s ) ((pos "Kj$" ) ))
( "adequacy"  n ( a1 d i k w @ s ii ) ((pos "L@%" ) ))
( "adequate"  j ( a1 d i k w @ t ) ((pos "OA%" ) ))
( "adequately"  a ( a1 d i k w @ t l ii ) ((pos "Pu%" ) ))
( "adhere"  v ( @1 d h i@1 r ) ((pos "I2%" ) (subcat "2A" )))
( "adhered"  v ( @1 d h i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "adherence"  n ( @1 d h i@1 r @ n s ) ((pos "K6%" ) ))
( "adherences"  n ( @1 d h i@1 r @ n s i z ) ((pos "Kj$" ) ))
( "adherent"  n ( @1 d h i@1 r @ n t ) ((pos "K6%" ) ))
( "adherents"  n ( @1 d h i@1 r @ n t s ) ((pos "Kj%" ) ))
( "adheres"  v ( @1 d h i@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "adhering"  v ( @1 d h i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "adhesion"  n ( @1 d h ii1 zh n ) ((pos "M6%" ) ))
( "adhesions"  n ( @1 d h ii1 zh n z ) ((pos "Mj$" ) ))
( "adhesive"  n ( @1 d h ii1 s i v ) ((pos "M6%" "OA%" ) ))
( "adhesives"  n ( @1 d h ii1 s i v z ) ((pos "Mj%" ) ))
( "adieu"  n ( @1 d y uu1 ) ((pos "K6%" "W-%" ) ))
( "adieus"  n ( @1 d y uu1 z ) ((pos "Kj%" ) ))
( "adieux"  n ( @1 d y uu1 z ) ((pos "Kj$" ) ))
( "adipose"  j ( a1 d i p ou s ) ((pos "OA$" ) ))
( "adjacent"  j ( @1 jh ei1 s n t ) ((pos "OA%" ) ))
( "adjectival"  j ( a2 jh i k t ai1 v @ l ) ((pos "OA%" ) ))
( "adjective"  n ( a1 jh i k t i v ) ((pos "K6%" ) ))
( "adjectives"  n ( a1 jh i k t i v z ) ((pos "Kj%" ) ))
( "adjoin"  v ( @1 jh oi1 n ) ((pos "J0%" ) (subcat "2A" )))
( "adjoined"  v ( @1 jh oi1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "adjoining"  v ( @1 jh oi1 n i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "adjoins"  v ( @1 jh oi1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "adjourn"  v ( @1 jh @@1 n ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "adjourned"  v ( @1 jh @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "adjourning"  v ( @1 jh @@1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "adjournment"  n ( @1 jh @@1 n m @ n t ) ((pos "K6%" ) ))
( "adjournments"  n ( @1 jh @@1 n m @ n t s ) ((pos "Kj%" ) ))
( "adjourns"  v ( @1 jh @@1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "adjudge"  v ( @1 jh uh1 jh ) ((pos "H2%" ) (subcat "9" "14" )))
( "adjudged"  v ( @1 jh uh1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "9" "14" )))
( "adjudges"  v ( @1 jh uh1 jh i z ) ((pos "Ha%" ) (subcat "9" "14" )))
( "adjudging"  v ( @1 jh uh1 jh i ng ) ((pos "Hb%" ) (subcat "9" "14" )))
( "adjudicate"  v ( @1 jh uu1 d i k ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "14" )))
( "adjudicated"  v ( @1 jh uu1 d i k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "adjudicates"  v ( @1 jh uu1 d i k ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "14" )))
( "adjudicating"  v ( @1 jh uu1 d i k ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "adjudication"  n ( @1 jh uu2 d i k ei1 sh @ n ) ((pos "K6%" ) ))
( "adjudications"  n ( @1 jh uu2 d i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "adjudicator"  n ( @1 jh uu d i k ei t @ r ) ((pos "K6%" ) ))
( "adjudicators"  n ( @1 jh uu d i k ei t @ z ) ((pos "Kj%" ) ))
( "adjunct"  n ( a1 jh uh ng k t ) ((pos "K6$" ) ))
( "adjuncts"  n ( a1 jh uh ng k t s ) ((pos "Kj$" ) ))
( "adjuration"  n ( a2 jh u@ r ei1 sh @ n ) ((pos "M6$" ) ))
( "adjurations"  n ( a2 jh u@ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "adjure"  v ( @1 jh u@1 r ) ((pos "H2$" ) (subcat )))
( "adjured"  v ( @1 jh u@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "adjures"  v ( @1 jh u@1 z ) ((pos "Ha$" ) (subcat )))
( "adjuring"  v ( @1 jh u@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "adjust"  v ( @1 jh uh1 s t ) ((pos "H0%" ) (subcat "6A" )))
( "adjustable"  j ( @1 jh uh1 s t @ b l ) ((pos "OA%" ) ))
( "adjusted"  v ( @1 jh uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "adjuster"  n ( @1 jh uh1 s t @ r ) ((pos "K6%" ) ))
( "adjusters"  n ( @1 jh uh1 s t @ z ) ((pos "Kj%" ) ))
( "adjusting"  v ( @1 jh uh1 s t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "adjustment"  n ( @1 jh uh1 s t m @ n t ) ((pos "M6%" ) ))
( "adjustments"  n ( @1 jh uh1 s t m @ n t s ) ((pos "Mj%" ) ))
( "adjusts"  v ( @1 jh uh1 s t s ) ((pos "Ha%" ) (subcat "6A" )))
( "adjutant"  n ( a1 jh u t @ n t ) ((pos "K6%" ) ))
( "adjutants"  n ( a1 jh u t @ n t s ) ((pos "Kj%" ) ))
( "admass"  n ( a1 d m a s ) ((pos "L@$" ) ))
( "administer"  v ( @1 d m i1 n i s t @ r ) ((pos "J0%" ) (subcat "6A" )))
( "administered"  v ( @1 d m i1 n i s t @ d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "administering"  v ( @1 d m i1 n i s t @ r i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "administers"  v ( @1 d m i1 n i s t @ z ) ((pos "Ja%" ) (subcat "6A" )))
( "administration"  n ( @1 d m i2 n i s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "administrations"  n ( @1 d m i2 n i s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "administrative"  j ( @1 d m i1 n i s t r @ t i v ) ((pos "OA%" ) ))
( "administratively"  a ( @1 d m i1 n i s t r @ t i v l ii ) ((pos "Pu%" ) ))
( "administrator"  n ( @1 d m i1 n i s t r ei t @ r ) ((pos "K6%" ) ))
( "administrators"  n ( @1 d m i1 n i s t r ei t @ z ) ((pos "Kj%" ) ))
( "admirable"  j ( a1 d m @ r @ b l ) ((pos "OA%" ) ))
( "admirably"  a ( a1 d m i r @ b l ii ) ((pos "Pu%" ) ))
( "admiral"  n ( a1 d m @ r @ l ) ((pos "K6%" ) ))
( "admirals"  n ( a1 d m @ r @ l z ) ((pos "Kj%" ) ))
( "admiralties"  n ( a1 d m @ r @ l t i z ) ((pos "Kj%" ) ))
( "admiralty"  n ( a1 d m @ r @ l t ii ) ((pos "K8%" ) ))
( "admiration"  n ( a2 d m @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "admire"  v ( @1 d m ai1 @ r ) ((pos "H2%" ) (subcat )))
( "admired"  v ( @1 d m ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "admirer"  n ( @1 d m ai1 r @ r ) ((pos "K6%" ) ))
( "admirers"  n ( @1 d m ai1 r @ z ) ((pos "Kj%" ) ))
( "admires"  v ( @1 d m ai1 @ z ) ((pos "Ha%" ) (subcat )))
( "admiring"  v ( @1 d m ai1 @ r i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "admiringly"  a ( @1 d m ai1 @ r i ng l ii ) ((pos "Pu%" ) ))
( "admissibility"  n ( @1 d m i2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "admissible"  j ( @1 d m i1 s @ b l ) ((pos "OA%" ) ))
( "admission"  n ( @1 d m i1 sh @ n ) ((pos "M6%" ) ))
( "admissions"  n ( @1 d m i1 sh @ n z ) ((pos "Mj%" ) ))
( "admit"  v ( @1 d m i1 t ) ((pos "J4%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "admits"  v ( @1 d m i1 t s ) ((pos "Ja%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "admittance"  n ( @1 d m i1 t n s ) ((pos "L@%" ) ))
( "admitted"  v ( @1 d m i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "admittedly"  a ( @1 d m i1 t i d l ii ) ((pos "Pu%" ) ))
( "admitting"  v ( @1 d m i1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "admix"  v ( a1 d m i1 k s ) ((pos "J1$" ) (subcat "2A" )))
( "admixed"  v ( a1 d m i1 k s t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "admixes"  v ( a1 d m i1 k s i z ) ((pos "Ja$" ) (subcat "2A" )))
( "admixing"  v ( a1 d m i1 k s i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "admixture"  n ( a1 d m i1 k s ch @ r ) ((pos "K6%" ) ))
( "admixtures"  n ( a1 d m i1 k s ch @ z ) ((pos "Kj$" ) ))
( "admonish"  v ( @1 d m o1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "admonished"  v ( @1 d m o1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "admonishes"  v ( @1 d m o1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "admonishing"  v ( @1 d m o1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "admonition"  n ( a2 d m @ n i1 sh @ n ) ((pos "M6%" ) ))
( "admonitions"  n ( a2 d m @ n i1 sh @ n z ) ((pos "Mj%" ) ))
( "admonitory"  j ( @1 d m o1 n i t r ii ) ((pos "OA%" ) ))
( "ado"  n ( @1 d uu1 ) ((pos "L@%" ) ))
( "adobe"  n ( @1 d ou1 b ii ) ((pos "L@$" ) ))
( "adolescence"  n ( a2 d @ l e1 s n s ) ((pos "L@%" ) ))
( "adolescent"  n ( a2 d @ l e1 s n t ) ((pos "K6%" "OA%" ) ))
( "adolescents"  n ( a2 d @ l e1 s n t s ) ((pos "Kj%" ) ))
( "adopt"  v ( @1 d o1 p t ) ((pos "H0%" ) (subcat )))
( "adopted"  v ( @1 d o1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "adopting"  v ( @1 d o1 p t i ng ) ((pos "Hb%" ) (subcat )))
( "adoption"  n ( @1 d o1 p sh @ n ) ((pos "K6%" ) ))
( "adoptions"  n ( @1 d o1 p sh @ n z ) ((pos "Kj%" ) ))
( "adoptive"  j ( @1 d o1 p t i v ) ((pos "OA%" ) ))
( "adopts"  v ( @1 d o1 p t s ) ((pos "Ha%" ) (subcat )))
( "adorable"  j ( @1 d oo1 r @ b l ) ((pos "OA%" ) ))
( "adorably"  a ( @1 d oo1 r @ b l ii ) ((pos "Pu%" ) ))
( "adoration"  n ( a2 d @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "adore"  v ( @1 d oo1 r ) ((pos "H2%" ) (subcat "6A" )))
( "adored"  v ( @1 d oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "adorer"  n ( @1 d oo1 r @ r ) ((pos "K6%" ) ))
( "adorers"  n ( @1 d oo1 r @ z ) ((pos "Kj%" ) ))
( "adores"  v ( @1 d oo1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "adoring"  v ( @1 d oo1 r i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "adoringly"  a ( @1 d oo1 r i ng l ii ) ((pos "Pu%" ) ))
( "adorn"  v ( @1 d oo1 n ) ((pos "H0%" ) (subcat "6A" )))
( "adorned"  v ( @1 d oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "adorning"  v ( @1 d oo1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "adornment"  n ( @1 d oo1 n m @ n t ) ((pos "M6%" ) ))
( "adornments"  n ( @1 d oo1 n m @ n t s ) ((pos "Mj%" ) ))
( "adorns"  v ( @1 d oo1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "adrenal"  j ( @1 d r ii1 n l ) ((pos "OA$" ) ))
( "adrenalin"  n ( @1 d r e1 n @ l i n ) ((pos "L@%" ) ))
( "adrift"  j ( @1 d r i1 f t ) ((pos "Op%" "Pu%" ) ))
( "adroit"  j ( @1 d r oi1 t ) ((pos "OA%" ) ))
( "adroitly"  a ( @1 d r oi1 t l ii ) ((pos "Pu%" ) ))
( "adroitness"  n ( @1 d r oi1 t n @ s ) ((pos "L@%" ) ))
( "ads"  n ( a1 d z ) ((pos "Kj%" ) ))
( "adulation"  n ( a2 d y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "adult"  n ( a1 d uh l t ) ((pos "K6%" "OA%" ) ))
( "adulterant"  n ( @1 d uh1 l t @ r @ n t ) ((pos "K6%" ) ))
( "adulterants"  n ( @1 d uh1 l t @ r @ n t s ) ((pos "Kj%" ) ))
( "adulterate"  v ( @1 d uh1 l t @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "adulterated"  v ( @1 d uh1 l t @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "adulterates"  v ( @1 d uh1 l t @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "adulterating"  v ( @1 d uh1 l t @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "adulteration"  n ( @1 d uh2 l t @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "adulterations"  n ( @1 d uh2 l t @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "adulterer"  n ( @1 d uh1 l t @ r @ r ) ((pos "K6%" ) ))
( "adulterers"  n ( @1 d uh1 l t @ r @ z ) ((pos "Kj%" ) ))
( "adulteress"  n ( @1 d uh1 l t @ r i s ) ((pos "K7%" ) ))
( "adulteresses"  n ( @1 d uh1 l t @ r i s i z ) ((pos "Kj%" ) ))
( "adulteries"  n ( @1 d uh1 l t @ r i z ) ((pos "Mj$" ) ))
( "adulterous"  j ( @1 d uh1 l t @ r @ s ) ((pos "OA%" ) ))
( "adultery"  n ( @1 d uh1 l t @ r ii ) ((pos "M8%" ) ))
( "adulthood"  n ( a1 d @ l t h u d ) ((pos "M6%" ) ))
( "adulthoods"  n ( a1 d @ l t h u d z ) ((pos "Mj$" ) ))
( "adults"  n ( a1 d uh l t s ) ((pos "Kj%" ) ))
( "adumbrate"  v ( a1 d uh m b r ei t ) ((pos "H2$" ) (subcat )))
( "adumbrated"  v ( a1 d uh m b r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "adumbrates"  v ( a1 d uh m b r ei t s ) ((pos "Ha$" ) (subcat )))
( "adumbrating"  v ( a1 d uh m b r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "advance"  v ( @1 d v aa1 n s ) ((pos "J2%" "M6%" ) (subcat "2A" "2B" "3A" "6A" "12A" "13A" )))
( "advanced"  v ( @1 d v aa1 n s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "3A" "6A" "12A" "13A" )))
( "advancement"  n ( @1 d v aa1 n s m @ n t ) ((pos "L@%" ) ))
( "advances"  v ( @1 d v aa1 n s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "3A" "6A" "12A" "13A" )))
( "advancing"  v ( @1 d v aa1 n s i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "3A" "6A" "12A" "13A" )))
( "advantage"  v ( @1 d v aa1 n t i jh ) ((pos "H2%" "M6%" ) (subcat )))
( "advantaged"  v ( @1 d v aa1 n t i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "advantageous"  j ( a2 d v @ n t ei1 jh @ s ) ((pos "OA%" ) ))
( "advantageously"  a ( a2 d v @ n t ei1 jh @ s l ii ) ((pos "Pu%" ) ))
( "advantages"  v ( @1 d v aa1 n t i jh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "advantaging"  v ( @1 d v aa1 n t i jh i ng ) ((pos "Hb%" ) (subcat )))
( "advent"  n ( a1 d v @ n t ) ((pos "K6%" ) ))
( "adventitious"  j ( a2 d v e n t i1 sh @ s ) ((pos "OA$" ) ))
( "advents"  n ( a1 d v @ n t s ) ((pos "Kj%" ) ))
( "adventure"  v ( @1 d v e1 n ch @ r ) ((pos "H2%" "M6%" ) ))
( "adventured"  v ( @1 d v e1 n ch @ d ) ((pos "Hc%" "Hd%" ) ))
( "adventurer"  n ( @1 d v e1 n ch @ r @ r ) ((pos "K6%" ) ))
( "adventurers"  n ( @1 d v e1 n ch @ r @ z ) ((pos "Kj%" ) ))
( "adventures"  v ( @1 d v e1 n ch @ z ) ((pos "Ha%" "Mj%" ) ))
( "adventuresome"  j ( @1 d v e1 n ch @ s @ m ) ((pos "OA%" ) ))
( "adventuress"  n ( @1 d v e1 n ch @ r i s ) ((pos "K7%" ) ))
( "adventuresses"  n ( @1 d v e1 n ch @ r i s i z ) ((pos "Kj%" ) ))
( "adventuring"  v ( @1 d v e1 n ch @ r i ng ) ((pos "Hb%" ) ))
( "adventurous"  j ( @1 d v e1 n ch @ r @ s ) ((pos "OA%" ) ))
( "adverb"  n ( a1 d v @@ b ) ((pos "K6%" ) ))
( "adverbial"  j ( a1 d v @@1 b i@ l ) ((pos "OA%" ) ))
( "adverbially"  a ( a1 d v @@1 b i@ l ii ) ((pos "Pu%" ) ))
( "adverbs"  n ( a1 d v @@ b z ) ((pos "Kj%" ) ))
( "adversaries"  n ( a1 d v @ s @ r i z ) ((pos "Kj%" ) ))
( "adversary"  n ( a1 d v @ s @ r ii ) ((pos "K8%" ) ))
( "adverse"  j ( a1 d v @@ s ) ((pos "OA%" ) ))
( "adversely"  a ( a1 d v @@1 s l ii ) ((pos "Pu%" ) ))
( "adversities"  n ( @1 d v @@1 s i t i z ) ((pos "Mj%" ) ))
( "adversity"  n ( @1 d v @@1 s i t ii ) ((pos "M8%" ) ))
( "advert"  n ( a1 d v @@ t ) ((pos "K6%" ) ))
( "advert"  v ( @1 d v @@1 t ) ((pos "I0$" ) (subcat )))
( "adverted"  v ( @1 d v @@1 t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "adverting"  v ( @1 d v @@1 t i ng ) ((pos "Ib$" ) (subcat )))
( "advertise"  v ( a1 d v @ t ai z ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "advertised"  v ( a1 d v @ t ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "advertisement"  n ( @1 d v @@1 t i s m @ n t ) ((pos "M6%" ) ))
( "advertisements"  n ( @1 d v @@1 t i s m @ n t s ) ((pos "Mj%" ) ))
( "advertiser"  n ( a1 d v @ t ai z @ r ) ((pos "K6%" ) ))
( "advertisers"  n ( a1 d v @ t ai z @ z ) ((pos "Kj%" ) ))
( "advertises"  v ( a1 d v @ t ai z i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "advertising"  v ( a1 d v @ t ai z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "adverts"  n ( a1 d v @@ t s ) ((pos "Kj%" ) ))
( "adverts"  v ( @1 d v @@1 t s ) ((pos "Ia$" ) (subcat )))
( "advice"  n ( @1 d v ai1 s ) ((pos "M6%" ) ))
( "advices"  n ( @1 d v ai1 s i z ) ((pos "Mj$" ) ))
( "advisability"  n ( @1 d v ai2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "advisable"  j ( @1 d v ai1 z @ b l ) ((pos "OA%" ) ))
( "advise"  v ( @1 d v ai1 z ) ((pos "J2%" ) (subcat "3A" "6A" "6C" "14" "17" "20" )))
( "advised"  v ( @1 d v ai1 z d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" "6C" "14" "17" "20" )))
( "advisedly"  a ( @1 d v ai1 z i d l ii ) ((pos "Pu%" ) ))
( "adviser"  n ( @1 d v ai1 z @ r ) ((pos "K6%" ) ))
( "advisers"  n ( @1 d v ai1 z @ z ) ((pos "Kj%" ) ))
( "advises"  v ( @1 d v ai1 z i z ) ((pos "Ja%" ) (subcat "3A" "6A" "6C" "14" "17" "20" )))
( "advising"  v ( @1 d v ai1 z i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "6C" "14" "17" "20" )))
( "advisory"  j ( @1 d v ai1 z @ r ii ) ((pos "OA%" ) ))
( "advocacy"  n ( a1 d v @ k @ s ii ) ((pos "L@%" ) ))
( "advocate"  n ( a1 d v @ k @ t ) ((pos "K6%" ) ))
( "advocate"  v ( a1 d v @ k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "advocated"  v ( a1 d v @ k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "advocates"  n ( a1 d v @ k @ t s ) ((pos "Kj%" ) ))
( "advocates"  v ( a1 d v @ k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "advocating"  v ( a1 d v @ k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "advowson"  n ( a2 d v au1 z n ) ((pos "K6$" ) ))
( "advowsons"  n ( a2 d v au1 z n z ) ((pos "Kj$" ) ))
( "adz"  n ( a1 d z ) ((pos "K7$" ) ))
( "adze"  n ( a1 d z ) ((pos "K6$" ) ))
( "adzes"  n ( a1 d z i z ) ((pos "Kj$" ) ))
( "aegis"  n ( ii1 jh i s ) ((pos "K7$" ) ))
( "aegises"  n ( ii1 jh i s i z ) ((pos "Kj$" ) ))
( "aeon"  n ( ii1 @ n ) ((pos "K6$" ) ))
( "aeons"  n ( ii1 @ n z ) ((pos "Kj$" ) ))
( "aerate"  v ( e@1 r ei t ) ((pos "H2%" ) (subcat )))
( "aerated"  v ( e@1 r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "aerates"  v ( e@1 r ei t s ) ((pos "Ha%" ) (subcat )))
( "aerating"  v ( e@1 r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "aeration"  n ( e@1 r ei1 sh @ n ) ((pos "K6%" ) ))
( "aerations"  n ( e@1 r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "aerial"  n ( e@1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "aerials"  n ( e@1 r i@ l z ) ((pos "Kj%" ) ))
( "aerie"  n ( e@1 r ii ) ((pos "K6$" ) ))
( "aeries"  n ( e@1 r i z ) ((pos "Kj$" ) ))
( "aerobatics"  n ( e@2 r @ b a1 t i k s ) ((pos "Lk%" ) ))
( "aerodrome"  n ( e@1 r @ d r ou m ) ((pos "K6%" ) ))
( "aerodromes"  n ( e@1 r @ d r ou m z ) ((pos "Kj%" ) ))
( "aerodynamic"  j ( e@2 r ou d ai n a1 m i k ) ((pos "OA%" ) ))
( "aerodynamics"  n ( e@2 r ou d ai n a1 m i k s ) ((pos "Lk%" ) ))
( "aeronaut"  n ( e@1 r @ n oo t ) ((pos "K6%" ) ))
( "aeronautics"  n ( e@2 r @ n oo1 t i k s ) ((pos "Lk%" ) ))
( "aeronauts"  n ( e@1 r @ n oo t s ) ((pos "Kj%" ) ))
( "aeroplane"  n ( e@1 r @ p l ei n ) ((pos "K6%" ) ))
( "aeroplanes"  n ( e@1 r @ p l ei n z ) ((pos "Kj%" ) ))
( "aerosol"  n ( e@1 r @ s o l ) ((pos "M6%" ) ))
( "aerosols"  n ( e@1 r @ s o l z ) ((pos "Mj%" ) ))
( "aerospace"  n ( e@1 r ou s p ei s ) ((pos "L@%" ) ))
( "aertex"  n ( e@1 t e k s ) ((pos "L@%" ) ))
( "aery"  n ( e@1 r ii ) ((pos "K8$" ) ))
( "aesthete"  n ( ii1 s th ii t ) ((pos "K6%" ) ))
( "aesthetes"  n ( ii1 s th ii t s ) ((pos "Kj%" ) ))
( "aesthetic"  n ( ii1 s th e1 t i k ) ((pos "L@%" "OA%" ) ))
( "aesthetical"  j ( ii1 s th e1 t i k l ) ((pos "OA%" ) ))
( "aesthetically"  a ( ii1 s th e1 t i k l ii ) ((pos "Pu%" ) ))
( "aesthetics"  n ( ii1 s th e1 t i k s ) ((pos "Lk%" ) ))
( "aether"  n ( ii1 th @ r ) ((pos "L@$" ) ))
( "aetiologies"  n ( ii2 t i o1 l @ jh i z ) ((pos "Kj%" ) ))
( "aetiology"  n ( ii2 t i o1 l @ jh ii ) ((pos "K8%" ) ))
( "afar"  a ( @1 f aa1 r ) ((pos "Pu%" ) ))
( "affability"  n ( a2 f @ b i1 l i t ii ) ((pos "L@%" ) ))
( "affable"  j ( a1 f @ b l ) ((pos "OA%" ) ))
( "affably"  a ( a1 f @ b l ii ) ((pos "Pu%" ) ))
( "affair"  n ( @1 f e@1 r ) ((pos "K6%" ) ))
( "affairs"  n ( @1 f e@1 z ) ((pos "Kj%" ) ))
( "affect"  v ( @1 f e1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "affectation"  n ( a2 f e k t ei1 sh @ n ) ((pos "M6%" ) ))
( "affectations"  n ( a2 f e k t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "affected"  v ( @1 f e1 k t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "affecting"  v ( @1 f e1 k t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "affectingly"  a ( @1 f e1 k t i ng l ii ) ((pos "Pu%" ) ))
( "affection"  n ( @1 f e1 k sh @ n ) ((pos "M6%" ) ))
( "affectionate"  j ( @1 f e1 k sh @ n @ t ) ((pos "OA%" ) ))
( "affectionately"  a ( @1 f e1 k sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "affections"  n ( @1 f e1 k sh @ n z ) ((pos "Mj%" ) ))
( "affects"  v ( @1 f e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "affiance"  v ( @1 f ai1 @ n s ) ((pos "H2$" ) (subcat )))
( "affianced"  v ( @1 f ai1 @ n s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "affiances"  v ( @1 f ai1 @ n s i z ) ((pos "Ha$" ) (subcat )))
( "affiancing"  v ( @1 f ai1 @ n s i ng ) ((pos "Hb$" ) (subcat )))
( "affidavit"  n ( a2 f i d ei1 v i t ) ((pos "K6%" ) ))
( "affidavits"  n ( a2 f i d ei1 v i t s ) ((pos "Kj%" ) ))
( "affiliate"  v ( @1 f i1 l i ei t ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "affiliated"  v ( @1 f i1 l i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "affiliates"  v ( @1 f i1 l i ei t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "affiliating"  v ( @1 f i1 l i ei t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "affiliation"  n ( @1 f i2 l i ei1 sh @ n ) ((pos "M6%" ) ))
( "affiliations"  n ( @1 f i2 l i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "affinities"  n ( @1 f i1 n i t i z ) ((pos "Mj%" ) ))
( "affinity"  n ( @1 f i1 n i t ii ) ((pos "M8%" ) ))
( "affirm"  v ( @1 f @@1 m ) ((pos "J0%" ) (subcat "2A" "6A" "9" )))
( "affirmation"  n ( a2 f @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "affirmations"  n ( a2 f @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "affirmative"  n ( @1 f @@1 m @ t i v ) ((pos "K6%" "OA%" ) ))
( "affirmatives"  n ( @1 f @@1 m @ t i v z ) ((pos "Kj%" ) ))
( "affirmed"  v ( @1 f @@1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" )))
( "affirming"  v ( @1 f @@1 m i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "9" )))
( "affirms"  v ( @1 f @@1 m z ) ((pos "Ja%" ) (subcat "2A" "6A" "9" )))
( "affix"  n ( a1 f i k s ) ((pos "K7%" ) ))
( "affix"  v ( @1 f i1 k s ) ((pos "H1%" ) (subcat "6A" )))
( "affixed"  v ( @1 f i1 k s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "affixes"  n ( a1 f i k s i z ) ((pos "Kj%" ) ))
( "affixes"  v ( @1 f i1 k s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "affixing"  v ( @1 f i1 k s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "afflatus"  n ( @1 f l ei1 t @ s ) ((pos "L@$" ) ))
( "afflict"  v ( @1 f l i1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "afflicted"  v ( @1 f l i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "afflicting"  v ( @1 f l i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "affliction"  n ( @1 f l i1 k sh @ n ) ((pos "M6%" ) ))
( "afflictions"  n ( @1 f l i1 k sh @ n z ) ((pos "Mj%" ) ))
( "afflicts"  v ( @1 f l i1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "affluence"  n ( a1 f l u@ n s ) ((pos "L@%" ) ))
( "affluent"  n ( a1 f l u@ n t ) ((pos "K6%" "OA%" ) ))
( "affluents"  n ( a1 f l u@ n t s ) ((pos "Kj%" ) ))
( "afford"  v ( @1 f oo1 d ) ((pos "H0%" ) (subcat "6A" "7A" "12A" )))
( "afforded"  v ( @1 f oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" "12A" )))
( "affording"  v ( @1 f oo1 d i ng ) ((pos "Hb%" ) (subcat "6A" "7A" "12A" )))
( "affords"  v ( @1 f oo1 d z ) ((pos "Ha%" ) (subcat "6A" "7A" "12A" )))
( "afforest"  v ( @1 f o1 r i s t ) ((pos "H0%" ) (subcat )))
( "afforestation"  n ( @1 f o2 r i s t ei1 sh @ n ) ((pos "K6%" ) ))
( "afforestations"  n ( @1 f o2 r i s t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "afforested"  v ( @1 f o1 r i s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "afforesting"  v ( @1 f o1 r i s t i ng ) ((pos "Hb%" ) (subcat )))
( "afforests"  v ( @1 f o1 r i s t s ) ((pos "Ha%" ) (subcat )))
( "affranchise"  v ( @1 f r a1 n ch ai z ) ((pos "H2$" ) (subcat )))
( "affranchised"  v ( @1 f r a1 n ch ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "affranchises"  v ( @1 f r a1 n ch ai z i z ) ((pos "Ha$" ) (subcat )))
( "affranchising"  v ( @1 f r a1 n ch ai z i ng ) ((pos "Hb$" ) (subcat )))
( "affray"  n ( @1 f r ei1 ) ((pos "K6$" ) ))
( "affrays"  n ( @1 f r ei1 z ) ((pos "Kj$" ) ))
( "affront"  v ( @1 f r uh1 n t ) ((pos "H0$" "K6%" ) (subcat )))
( "affronted"  v ( @1 f r uh1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "affronting"  v ( @1 f r uh1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "affronts"  v ( @1 f r uh1 n t s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "afield"  a ( @1 f ii1 l d ) ((pos "Pu%" ) ))
( "afire"  j ( @1 f ai1 @ r ) ((pos "Op%" ) ))
( "aflame"  j ( @1 f l ei1 m ) ((pos "Op%" ) ))
( "afloat"  j ( @1 f l ou1 t ) ((pos "Op%" ) ))
( "afoot"  j ( @1 f u1 t ) ((pos "Op%" ) ))
( "afore"  a ( @1 f oo1 r ) ((pos "Pu%" "T-%" ) ))
( "aforesaid"  j ( @1 f oo1 s e d ) ((pos "OA%" "Qx%" ) ))
( "aforethought"  a ( @1 f oo1 th oo t ) ((pos "Pu%" ) ))
( "afoul"  a ( @1 f au1 l ) ((pos "Pu$" ) ))
( "afraid"  j ( @1 f r ei1 d ) ((pos "Op%" ) ))
( "afresh"  a ( @1 f r e1 sh ) ((pos "Pu%" ) ))
( "aft"  a ( aa1 f t ) ((pos "Pu%" ) ))
( "after"  j ( aa1 f t @ r ) ((pos "OA%" "Pu*" "T-*" "V-*" ) ))
( "after-"  j ( aa1 f t @ r - ) ((pos "U-%" ) ))
( "aftercare"  n ( aa1 f t @ k e@ r ) ((pos "L@%" ) ))
( "afterdamp"  n ( aa1 f t @ d a m p ) ((pos "L@$" ) ))
( "aftereffect"  n ( aa1 f t @ r i f e k t ) ((pos "K6%" ) ))
( "aftereffects"  n ( aa1 f t @ r i f e k t s ) ((pos "Kj%" ) ))
( "afterglow"  n ( aa1 f t @ g l ou ) ((pos "Ki%" ) ))
( "aftermath"  n ( aa1 f t @ m a th ) ((pos "Ki%" ) ))
( "afternoon"  n ( aa2 f t @ n uu1 n ) ((pos "M6%" ) ))
( "afternoons"  n ( aa2 f t @ n uu1 n z ) ((pos "Mj%" ) ))
( "afters"  n ( aa1 f t @ z ) ((pos "Kj%" ) ))
( "afterthought"  n ( aa1 f t @ th oo t ) ((pos "M6%" ) ))
( "afterthoughts"  n ( aa1 f t @ th oo t s ) ((pos "Mj%" ) ))
( "afterwards"  a ( aa1 f t @ w @ d z ) ((pos "Pu%" ) ))
( "again"  a ( @1 g e1 n ) ((pos "Pu*" ) ))
( "against"  in ( @1 g e1 n s t ) ((pos "T-*" ) ))
( "agape"  j ( @1 g ei1 p ) ((pos "Op%" ) ))
( "agar-agar"  n ( ei2 g aa r - ei1 g aa r ) ((pos "L@$" ) ))
( "agate"  n ( a1 g @ t ) ((pos "K6$" ) ))
( "agates"  n ( a1 g @ t s ) ((pos "Kj$" ) ))
( "agave"  n ( @1 g ei1 v ii ) ((pos "K6$" ) ))
( "agaves"  n ( @1 g ei1 v i z ) ((pos "Kj$" ) ))
( "age"  v ( ei1 jh ) ((pos "J2%" "M6*" ) (subcat "2A" )))
( "age-bracket"  n ( ei1 jh - b r a k i t ) ((pos "K6$" ) ))
( "age-brackets"  n ( ei1 jh - b r a k i t s ) ((pos "Kj$" ) ))
( "age-group"  n ( ei1 jh - g r uu p ) ((pos "K6%" ) ))
( "age-groups"  n ( ei1 jh - g r uu p s ) ((pos "Kj%" ) ))
( "age-long"  j ( ei1 jh - l o ng ) ((pos "OA$" ) ))
( "age-old"  j ( ei1 jh - ou l d ) ((pos "OA%" ) ))
( "aged"  j ( ei1 jh i d ) ((pos "Oq%" ) ))
( "aged"  v ( ei1 jh d ) ((pos "Jc%" "Jd%" "Op%" ) (subcat "2A" )))
( "ageing"  v ( ei1 jh i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "ageless"  j ( ei1 jh l @ s ) ((pos "OA%" ) ))
( "agencies"  n ( ei1 jh @ n s i z ) ((pos "Mj%" ) ))
( "agency"  n ( ei1 jh @ n s ii ) ((pos "M8%" ) ))
( "agenda"  n ( @1 jh e1 n d @ ) ((pos "K6%" ) ))
( "agendas"  n ( @1 jh e1 n d @ z ) ((pos "Kj%" ) ))
( "agent"  n ( ei1 jh @ n t ) ((pos "K6%" ) ))
( "agent provocateur"  n ( a2 zh o n - p r @ v o2 k @ t @@1 r ) ((pos "Ki$" ) ))
( "agents"  n ( ei1 jh @ n t s ) ((pos "Kj%" ) ))
( "agents provocateurs"  n ( a2 zh o n - p r @ v o2 k @ t @@1 r ) ((pos "Kj$" ) ))
( "ages"  v ( ei1 jh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "agglomerate"  j ( @1 g l o1 m @ r @ t ) ((pos "OA$" ) ))
( "agglomerate"  v ( @1 g l o1 m @ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "agglomerated"  v ( @1 g l o1 m @ r ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "agglomerates"  v ( @1 g l o1 m @ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "agglomerating"  v ( @1 g l o1 m @ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "agglomeration"  n ( @1 g l o2 m @ r ei1 sh @ n ) ((pos "M6$" ) ))
( "agglomerations"  n ( @1 g l o2 m @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "agglutinate"  v ( @1 g l uu1 t i n ei t ) ((pos "H2$" ) (subcat "2A" )))
( "agglutinated"  v ( @1 g l uu1 t i n ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "2A" )))
( "agglutinates"  v ( @1 g l uu1 t i n ei t s ) ((pos "Ha$" ) (subcat "2A" )))
( "agglutinating"  v ( @1 g l uu1 t i n ei t i ng ) ((pos "Hb$" ) (subcat "2A" )))
( "agglutinative"  j ( @1 g l uu1 t i n @ t i v ) ((pos "OA$" ) ))
( "aggrandize"  v ( @1 g r a1 n d ai z ) ((pos "H2$" ) (subcat )))
( "aggrandized"  v ( @1 g r a1 n d ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "aggrandizement"  n ( @1 g r a1 n d i z m @ n t ) ((pos "K6$" ) ))
( "aggrandizements"  n ( @1 g r a1 n d i z m @ n t s ) ((pos "Kj$" ) ))
( "aggrandizes"  v ( @1 g r a1 n d ai z i z ) ((pos "Ha$" ) (subcat )))
( "aggrandizing"  v ( @1 g r a1 n d ai z i ng ) ((pos "Hb$" ) (subcat )))
( "aggravate"  v ( a1 g r @ v ei t ) ((pos "H2%" ) (subcat )))
( "aggravated"  v ( a1 g r @ v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "aggravates"  v ( a1 g r @ v ei t s ) ((pos "Ha%" ) (subcat )))
( "aggravating"  v ( a1 g r @ v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "aggravation"  n ( a2 g r @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "aggravations"  n ( a2 g r @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "aggregate"  n ( a1 g r i g @ t ) ((pos "K6%" ) ))
( "aggregate"  v ( a1 g r i g ei t ) ((pos "J2$" ) (subcat "2A" "2E" )))
( "aggregated"  v ( a1 g r i g ei t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" "2E" )))
( "aggregates"  n ( a1 g r i g @ t s ) ((pos "Kj$" ) ))
( "aggregates"  v ( a1 g r i g ei t s ) ((pos "Ja$" ) (subcat "2A" "2E" )))
( "aggregating"  v ( a1 g r i g ei t i ng ) ((pos "Jb$" ) (subcat "2A" "2E" )))
( "aggregation"  n ( a2 g r i g ei1 sh @ n ) ((pos "M6$" ) ))
( "aggregations"  n ( a2 g r i g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "aggression"  n ( @1 g r e1 sh @ n ) ((pos "M6%" ) ))
( "aggressions"  n ( @1 g r e1 sh @ n z ) ((pos "Mj$" ) ))
( "aggressive"  j ( @1 g r e1 s i v ) ((pos "OA%" ) ))
( "aggressively"  a ( @1 g r e1 s i v l ii ) ((pos "Pu%" ) ))
( "aggressiveness"  n ( @1 g r e1 s i v n @ s ) ((pos "L@%" ) ))
( "aggressor"  n ( @1 g r e1 s @ r ) ((pos "K6%" ) ))
( "aggressors"  n ( @1 g r e1 s @ z ) ((pos "Kj%" ) ))
( "aggrieve"  v ( @1 g r ii1 v ) ((pos "H2$" ) ))
( "aggrieved"  v ( @1 g r ii1 v d ) ((pos "Hc$" "Hd%" ) ))
( "aggrieves"  v ( @1 g r ii1 v z ) ((pos "Ha$" ) ))
( "aggrieving"  v ( @1 g r ii1 v i ng ) ((pos "Hb$" ) ))
( "aggro"  n ( a1 g r ou ) ((pos "L@%" ) ))
( "aghast"  j ( @1 g aa1 s t ) ((pos "Op%" ) ))
( "agile"  j ( a1 jh ai l ) ((pos "OA%" ) ))
( "agilely"  a ( a1 jh ai l ii ) ((pos "Pu%" ) ))
( "agility"  n ( @1 jh i1 l i t ii ) ((pos "L@%" ) ))
( "agin"  in ( @1 g i1 n ) ((pos "T-$" ) ))
( "aging"  v ( ei1 jh i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "agitate"  v ( a1 jh i t ei t ) ((pos "J2%" ) (subcat "3A" )))
( "agitated"  v ( a1 jh i t ei t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" )))
( "agitates"  v ( a1 jh i t ei t s ) ((pos "Ja%" ) (subcat "3A" )))
( "agitating"  v ( a1 jh i t ei t i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "agitation"  n ( a2 jh i t ei1 sh @ n ) ((pos "M6%" ) ))
( "agitations"  n ( a2 jh i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "agitator"  n ( a1 jh i t ei t @ r ) ((pos "K6%" ) ))
( "agitators"  n ( a1 jh i t ei t @ z ) ((pos "Kj%" ) ))
( "aglow"  j ( @1 g l ou1 ) ((pos "Op%" ) ))
( "agnail"  n ( a1 g n ei l ) ((pos "L@$" ) ))
( "agnostic"  n ( a1 g n o1 s t i k ) ((pos "K6%" "OA%" ) ))
( "agnosticism"  n ( a1 g n o1 s t i s i z @ m ) ((pos "L@%" ) ))
( "agnostics"  n ( a1 g n o1 s t i k s ) ((pos "Kj%" ) ))
( "ago"  a ( @1 g ou1 ) ((pos "Pu*" ) ))
( "agog"  j ( @1 g o1 g ) ((pos "Op%" ) ))
( "agonies"  n ( a1 g @ n i z ) ((pos "Mj%" ) ))
( "agonized"  j ( a1 g @ n ai z d ) ((pos "OA%" ) ))
( "agonizing"  j ( a1 g @ n ai z i ng ) ((pos "OA%" ) ))
( "agonizingly"  a ( a1 g @ n ai z i ng l ii ) ((pos "Pu%" ) ))
( "agony"  n ( a1 g @ n ii ) ((pos "M8%" ) ))
( "agora"  n ( a1 g @ r @ ) ((pos "K6$" ) ))
( "agoraphobia"  n ( a2 g @ r @ f ou1 b i@ ) ((pos "L@%" ) ))
( "agoras"  n ( a1 g @ r @ z ) ((pos "Kj$" ) ))
( "agrarian"  j ( @1 g r e@1 r i@ n ) ((pos "OA$" ) ))
( "agree"  v ( @1 g r ii1 ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "3B" "4C" "6A" )))
( "agreeable"  j ( @1 g r ii1 @ b l ) ((pos "OA%" ) ))
( "agreeably"  a ( @1 g r ii1 @ b l ii ) ((pos "Pu%" ) ))
( "agreed"  v ( @1 g r ii1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "3B" "4C" "6A" )))
( "agreeing"  v ( @1 g r ii1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "3B" "4C" "6A" )))
( "agreement"  n ( @1 g r ii1 m @ n t ) ((pos "M6%" ) ))
( "agreements"  n ( @1 g r ii1 m @ n t s ) ((pos "Mj%" ) ))
( "agrees"  v ( @1 g r ii1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "3B" "4C" "6A" )))
( "agricultural"  j ( a2 g r i k uh1 l ch @ r @ l ) ((pos "OA%" ) ))
( "agriculture"  n ( a1 g r i k uh l ch @ r ) ((pos "L@%" ) ))
( "aground"  j ( @1 g r au1 n d ) ((pos "Op%" "Pu%" ) ))
( "ague"  n ( ei1 g y uu ) ((pos "K6$" ) ))
( "agues"  n ( ei1 g y uu z ) ((pos "Kj$" ) ))
( "ah"  uh ( aa1 ) ((pos "W-%" ) ))
( "aha"  uh ( aa1 h aa1 ) ((pos "W-%" ) ))
( "ahead"  a ( @1 h e1 d ) ((pos "Pu%" ) ))
( "ahem"  uh ( @1 h @1 m ) ((pos "W-%" ) ))
( "ahoy"  uh ( @1 h oi1 ) ((pos "W-%" ) ))
( "aid"  v ( ei1 d ) ((pos "H0%" "M6%" ) (subcat "6A" "14" )))
( "aide-de-camp"  n ( ei2 d - d @ - k o1 m ) ((pos "Ki%" ) ))
( "aide-m_emoire"  n ( ei2 d - m e2 m w aa1 r ) ((pos "K6%" ) ))
( "aide-m_emoires"  n ( ei2 d - m e2 m w aa1 z ) ((pos "Kj%" ) ))
( "aided"  v ( ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "aides-de-camp"  n ( ei2 d - d @ - k o1 m ) ((pos "Kj%" ) ))
( "aiding"  v ( ei1 d i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "aids"  v ( ei1 d z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "aigret"  n ( ei1 g r e t ) ((pos "K6$" ) ))
( "aigrets"  n ( ei1 g r e t s ) ((pos "Kj$" ) ))
( "aigrette"  n ( ei1 g r e t ) ((pos "K6$" ) ))
( "aigrettes"  n ( ei1 g r e t s ) ((pos "Kj$" ) ))
( "ail"  v ( ei1 l ) ((pos "J0%" ) (subcat "2A" "2B" )))
( "ailed"  v ( ei1 l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2B" )))
( "aileron"  n ( ei1 l @ r o n ) ((pos "K6$" ) ))
( "ailerons"  n ( ei1 l @ r o n z ) ((pos "Kj$" ) ))
( "ailing"  v ( ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2B" )))
( "ailment"  n ( ei1 l m @ n t ) ((pos "K6%" ) ))
( "ailments"  n ( ei1 l m @ n t s ) ((pos "Kj%" ) ))
( "ails"  v ( ei1 l z ) ((pos "Ja%" ) (subcat "2A" "2B" )))
( "aim"  v ( ei1 m ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "4A" "6A" )))
( "aimed"  v ( ei1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" "6A" )))
( "aiming"  v ( ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" "6A" )))
( "aimless"  j ( ei1 m l @ s ) ((pos "OA%" ) ))
( "aimlessly"  a ( ei1 m l @ s l ii ) ((pos "Pu%" ) ))
( "aims"  v ( ei1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "4A" "6A" )))
( "ain't"  v ( ei n t ) ((pos "Gg%" ) ))
( "air"  v ( e@1 r ) ((pos "H0%" "M6*" ) (subcat )))
( "air-bladder"  n ( e@1 - b l a d @ r ) ((pos "K6%" ) ))
( "air-bladders"  n ( e@1 - b l a d @ z ) ((pos "Kj%" ) ))
( "air-conditioned"  j ( e@1 - k @ n d i2 sh @ n d ) ((pos "OA%" ) ))
( "air-conditioning"  n ( e@1 - k @ n d i sh @ n i ng ) ((pos "L@%" ) ))
( "air-cooled"  j ( e@1 - k uu l d ) ((pos "OA%" ) ))
( "air-minded"  j ( e@1 - m ai1 n d i d ) ((pos "OA$" ) ))
( "air-pump"  n ( e@1 - p uh m p ) ((pos "K6%" ) ))
( "air-pumps"  n ( e@1 - p uh m p s ) ((pos "Kj%" ) ))
( "air-raid"  n ( e@1 - r ei d ) ((pos "K6%" ) ))
( "air-raids"  n ( e@1 - r ei d z ) ((pos "Kj%" ) ))
( "air-shaft"  n ( e@1 - sh aa f t ) ((pos "K6%" ) ))
( "air-shafts"  n ( e@1 - sh aa f t s ) ((pos "Kj%" ) ))
( "air-sick"  j ( e@1 - s i k ) ((pos "OA%" ) ))
( "air-sickness"  n ( e@1 - s i k n @ s ) ((pos "L@%" ) ))
( "air-to-air"  j ( e@2 - t u - e@1 r ) ((pos "OA%" ) ))
( "air-to-ground"  j ( e@2 - t u - g r au1 n d ) ((pos "OA%" ) ))
( "airbed"  n ( e@1 b e d ) ((pos "K6%" ) ))
( "airbeds"  n ( e@1 b e d z ) ((pos "Kj%" ) ))
( "airborne"  j ( e@1 b oo n ) ((pos "OA%" ) ))
( "airbrake"  n ( e@1 b r ei k ) ((pos "K6%" ) ))
( "airbrakes"  n ( e@1 b r ei k s ) ((pos "Kj%" ) ))
( "aircraft"  n ( e@1 k r aa f t ) ((pos "K9%" ) ))
( "aircraftman"  n ( e@1 k r a f t m @ n ) ((pos "Ki$" ) ))
( "aircraftmen"  n ( e@1 k r a f t m @ n ) ((pos "Kj$" ) ))
( "aircrew"  n ( e@1 k r uu ) ((pos "K6%" ) ))
( "aircrews"  n ( e@1 k r uu z ) ((pos "Kj%" ) ))
( "airdrome"  n ( e@1 d r ou m ) ((pos "K6$" ) ))
( "airdromes"  n ( e@1 d r ou m z ) ((pos "Kj$" ) ))
( "aired"  v ( e@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "airfield"  n ( e@1 f ii l d ) ((pos "K6%" ) ))
( "airfields"  n ( e@1 f ii l d z ) ((pos "Kj%" ) ))
( "airflow"  n ( e@1 f l ou ) ((pos "K6%" ) ))
( "airflows"  n ( e@1 f l ou z ) ((pos "Kj$" ) ))
( "airframe"  n ( e@1 f r ei m ) ((pos "K6$" ) ))
( "airframes"  n ( e@1 f r ei m z ) ((pos "Kj$" ) ))
( "airily"  a ( e@1 r @ l ii ) ((pos "Pu%" ) ))
( "airing"  v ( e@1 r i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "airing-cupboard"  n ( e@1 r i ng - k uh2 b @ d ) ((pos "K6%" ) ))
( "airing-cupboards"  n ( e@1 r i ng - k uh2 b @ d z ) ((pos "Kj%" ) ))
( "airings"  n ( e@1 r i ng z ) ((pos "Kj%" ) ))
( "airless"  j ( e@1 l i s ) ((pos "OA%" ) ))
( "airline"  n ( e@1 l ai n ) ((pos "K6%" ) ))
( "airliner"  n ( e@1 l ai n @ r ) ((pos "K6%" ) ))
( "airliners"  n ( e@1 l ai n @ z ) ((pos "Kj%" ) ))
( "airlines"  n ( e@1 l ai n z ) ((pos "Kj%" ) ))
( "airmail"  n ( e@1 m ei l ) ((pos "L@%" ) ))
( "airman"  n ( e@1 m @ n ) ((pos "Ki%" ) ))
( "airmen"  n ( e@1 m @ n ) ((pos "Kj%" ) ))
( "airplane"  n ( e@1 p l ei n ) ((pos "K6%" ) ))
( "airplanes"  n ( e@1 p l ei n z ) ((pos "Kj%" ) ))
( "airport"  n ( e@1 p oo t ) ((pos "K6%" ) ))
( "airports"  n ( e@1 p oo t s ) ((pos "Kj%" ) ))
( "airs"  v ( e@1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "airscrew"  n ( e@1 s k r uu ) ((pos "K6$" ) ))
( "airscrews"  n ( e@1 s k r uu z ) ((pos "Kj$" ) ))
( "airship"  n ( e@1 sh i p ) ((pos "K6%" ) ))
( "airships"  n ( e@1 sh i p s ) ((pos "Kj%" ) ))
( "airstrip"  n ( e@1 s t r i p ) ((pos "K6%" ) ))
( "airstrips"  n ( e@1 s t r i p s ) ((pos "Kj%" ) ))
( "airtight"  j ( e@1 t ai t ) ((pos "OA%" ) ))
( "airway"  n ( e@1 w ei ) ((pos "K6%" ) ))
( "airways"  n ( e@1 w ei z ) ((pos "Kj%" ) ))
( "airwoman"  n ( e@1 w u m @ n ) ((pos "Ki$" ) ))
( "airwomen"  n ( e@1 w i m i n ) ((pos "Kj$" ) ))
( "airworthiness"  n ( e@1 w @@ dh i n @ s ) ((pos "L@%" ) ))
( "airworthy"  j ( e@1 w @@ dh ii ) ((pos "OA%" ) ))
( "airy"  j ( e@1 r ii ) ((pos "OA%" ) ))
( "aisle"  n ( ai1 l ) ((pos "K6%" ) ))
( "aisles"  n ( ai1 l z ) ((pos "Kj%" ) ))
( "aitch"  n ( ei1 ch ) ((pos "K7%" ) ))
( "aitch-bone"  n ( ei1 ch - b ou n ) ((pos "K6%" ) ))
( "aitch-bones"  n ( ei1 ch - b ou n z ) ((pos "Kj$" ) ))
( "aitches"  n ( ei1 ch i z ) ((pos "Kj%" ) ))
( "ajar"  j ( @1 jh aa1 r ) ((pos "Op%" ) ))
( "akimbo"  a ( @1 k i1 m b ou ) ((pos "Pu%" ) ))
( "akin"  j ( @1 k i1 n ) ((pos "Op%" ) ))
( "alabaster"  n ( a1 l @ b aa s t @ r ) ((pos "L@%" "OA%" ) ))
( "alack"  uh ( @1 l a1 k ) ((pos "W-$" ) ))
( "alacrity"  n ( @1 l a1 k r i t ii ) ((pos "L@%" ) ))
( "alarm"  v ( @1 l aa1 m ) ((pos "H0%" "M6%" ) (subcat )))
( "alarm-clock"  n ( @1 l aa1 m - k l o k ) ((pos "K6%" ) ))
( "alarm-clocks"  n ( @1 l aa1 m - k l o k s ) ((pos "Kj%" ) ))
( "alarmed"  v ( @1 l aa1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "alarming"  v ( @1 l aa1 m i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "alarmingly"  a ( @1 l aa1 m i ng l ii ) ((pos "Pu%" ) ))
( "alarmist"  n ( @1 l aa1 m i s t ) ((pos "K6%" ) ))
( "alarmists"  n ( @1 l aa1 m i s t s ) ((pos "Kj%" ) ))
( "alarms"  v ( @1 l aa1 m z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "alas"  uh ( @1 l a1 s ) ((pos "W-%" ) ))
( "alb"  n ( a1 l b ) ((pos "K6$" ) ))
( "albatross"  n ( a1 l b @ t r o s ) ((pos "K7%" ) ))
( "albatrosses"  n ( a1 l b @ t r o s i z ) ((pos "Kj%" ) ))
( "albeit"  cc ( oo2 l b ii1 i t ) ((pos "V-%" ) ))
( "albino"  n ( a1 l b ii1 n ou ) ((pos "K6%" ) ))
( "albinos"  n ( a1 l b ii1 n ou z ) ((pos "Kj%" ) ))
( "albs"  n ( a1 l b z ) ((pos "Kj$" ) ))
( "album"  n ( a1 l b @ m ) ((pos "K6%" ) ))
( "albumen"  n ( a1 l b y u m @ n ) ((pos "L@%" ) ))
( "albums"  n ( a1 l b @ m z ) ((pos "Kj%" ) ))
( "alchemist"  n ( a1 l k i m i s t ) ((pos "K6%" ) ))
( "alchemists"  n ( a1 l k i m i s t s ) ((pos "Kj%" ) ))
( "alchemy"  n ( a1 l k i m ii ) ((pos "L@%" ) ))
( "alcohol"  n ( a1 l k @ h o l ) ((pos "M6%" ) ))
( "alcoholic"  n ( a2 l k @ h o1 l i k ) ((pos "K6%" "OA%" ) ))
( "alcoholics"  n ( a2 l k @ h o1 l i k s ) ((pos "Kj%" ) ))
( "alcoholism"  n ( a1 l k @ h o l i z @ m ) ((pos "L@%" ) ))
( "alcohols"  n ( a1 l k @ h o l z ) ((pos "Mj%" ) ))
( "alcove"  n ( a1 l k ou v ) ((pos "K6%" ) ))
( "alcoves"  n ( a1 l k ou v z ) ((pos "Kj%" ) ))
( "alder"  n ( oo1 l d @ r ) ((pos "K6%" ) ))
( "alderman"  n ( oo1 l d @ m @ n ) ((pos "Ki%" ) ))
( "aldermanic"  j ( oo2 l d @ m a1 n i k ) ((pos "OA$" ) ))
( "aldermen"  n ( oo1 l d @ m @ n ) ((pos "Kj%" ) ))
( "alders"  n ( oo1 l d @ z ) ((pos "Kj%" ) ))
( "ale"  n ( ei1 l ) ((pos "M6%" ) ))
( "ale-house"  n ( ei1 l - h au s ) ((pos "K6%" ) ))
( "ale-houses"  n ( ei1 l - h au z i z ) ((pos "Kj%" ) ))
( "alee"  j ( @1 l ii1 ) ((pos "Op$" "Pu$" ) ))
( "alert"  v ( @1 l @@1 t ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "alerted"  v ( @1 l @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "alerting"  v ( @1 l @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "alertly"  a ( @1 l @@1 t l ii ) ((pos "Pu%" ) ))
( "alertness"  n ( @1 l @@1 t n @ s ) ((pos "L@%" ) ))
( "alerts"  v ( @1 l @@1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ales"  n ( ei1 l z ) ((pos "Mj%" ) ))
( "alexandrine"  n ( a2 l i g z a1 n d r ai n ) ((pos "K6$" ) ))
( "alexandrines"  n ( a2 l i g z a1 n d r ai n z ) ((pos "Kj$" ) ))
( "alexia"  n ( @1 l e1 k s i@ ) ((pos "L@$" ) ))
( "alexic"  n ( @1 l e1 k s i k ) ((pos "K6$" "OA$" ) ))
( "alexics"  n ( @1 l e1 k s i k s ) ((pos "Kj$" ) ))
( "alfalfa"  n ( a1 l f a1 l f @ ) ((pos "L@$" ) ))
( "alfresco"  j ( a2 l f r e1 s k ou ) ((pos "OA%" "Pu%" ) ))
( "alga"  n ( a1 l g @ ) ((pos "Ki$" ) ))
( "algae"  n ( a1 l jh ii ) ((pos "Kj%" ) ))
( "algebra"  n ( a1 l jh i b r @ ) ((pos "M6%" ) ))
( "algebraic"  j ( a2 l jh i b r ei1 i k ) ((pos "OA%" ) ))
( "algebraical"  j ( a2 l jh @ b r ei1 k l ) ((pos "OA$" ) ))
( "algebraically"  a ( a2 l jh @ b r ei1 k l ii ) ((pos "Pu%" ) ))
( "algebras"  n ( a1 l jh i b r @ z ) ((pos "Mj%" ) ))
( "alias"  n ( ei1 l i@ s ) ((pos "K7%" "Pu%" ) ))
( "aliases"  n ( ei1 l i@ s i z ) ((pos "Kj%" ) ))
( "alibi"  n ( a1 l i b ai ) ((pos "K6%" ) ))
( "alibis"  n ( a1 l i b ai z ) ((pos "Kj%" ) ))
( "alien"  n ( ei1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "alienate"  v ( ei1 l i@ n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "alienated"  v ( ei1 l i@ n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "alienates"  v ( ei1 l i@ n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "alienating"  v ( ei1 l i@ n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "alienation"  n ( ei2 l i@ n ei1 sh @ n ) ((pos "L@%" ) ))
( "alienist"  n ( ei1 l i@ n i s t ) ((pos "K6$" ) ))
( "alienists"  n ( ei1 l i@ n i s t s ) ((pos "Kj$" ) ))
( "aliens"  n ( ei1 l i@ n z ) ((pos "Kj%" ) ))
( "alight"  v ( @1 l ai1 t ) ((pos "I0%" "Op%" ) (subcat "2A" )))
( "alighted"  v ( @1 l ai1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "alighting"  v ( @1 l ai1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "alights"  v ( @1 l ai1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "align"  v ( @1 l ai1 n ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "aligned"  v ( @1 l ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "aligning"  v ( @1 l ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "alignment"  n ( @1 l ai1 n m @ n t ) ((pos "M6%" ) ))
( "alignments"  n ( @1 l ai1 n m @ n t s ) ((pos "Mj%" ) ))
( "aligns"  v ( @1 l ai1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "alike"  j ( @1 l ai1 k ) ((pos "Op%" "Pu%" ) ))
( "alimentary"  j ( a2 l i m e1 n t @ r ii ) ((pos "OA%" ) ))
( "alimony"  n ( a1 l i m @ n ii ) ((pos "L@%" ) ))
( "alive"  j ( @1 l ai1 v ) ((pos "Op%" ) ))
( "alkali"  n ( a1 l k @ l ai ) ((pos "M6%" ) ))
( "alkaline"  j ( a1 l k @ l ai n ) ((pos "OA%" ) ))
( "alkalis"  n ( a1 l k @ l ai z ) ((pos "Mj%" ) ))
( "all"  n ( oo l ) ((pos "Ki%" "OA*" "Pu*" "Qx*" "U-%" ) ))
( "all-mains"  j ( aa1 l - m ei n z ) ((pos "Oq$" ) ))
( "all-round"  j ( oo1 l - r au n d ) ((pos "OA%" ) ))
( "all-rounder"  n ( oo1 l - r au1 n d @ r ) ((pos "K6%" ) ))
( "all-rounders"  n ( oo1 l - r au1 n d @ z ) ((pos "Kj%" ) ))
( "allay"  v ( @1 l ei1 ) ((pos "H0%" ) (subcat )))
( "allayed"  v ( @1 l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "allaying"  v ( @1 l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "allays"  v ( @1 l ei1 z ) ((pos "Ha%" ) (subcat )))
( "allegation"  n ( a2 l i g ei1 sh @ n ) ((pos "M6%" ) ))
( "allegations"  n ( a2 l i g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "allege"  v ( @1 l e1 jh ) ((pos "H2%" ) (subcat "6A" )))
( "alleged"  v ( @1 l e1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "allegedly"  a ( @1 l e1 jh i d l ii ) ((pos "Pu%" ) ))
( "alleges"  v ( @1 l e1 jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "allegiance"  n ( @1 l ii1 jh @ n s ) ((pos "L@%" ) ))
( "alleging"  v ( @1 l e1 jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "allegoric"  j ( a2 l i g o1 r i k ) ((pos "OA$" ) ))
( "allegorical"  j ( a2 l i g o1 r i k @ l ) ((pos "OA%" ) ))
( "allegories"  n ( a1 l i g @ r i z ) ((pos "Kj%" ) ))
( "allegory"  n ( a1 l i g @ r ii ) ((pos "K8%" ) ))
( "allegretto"  n ( a2 l i g r e1 t ou ) ((pos "K6%" "OA%" "Pu%" ) ))
( "allegrettos"  n ( a2 l i g r e1 t ou z ) ((pos "Kj$" ) ))
( "allegro"  n ( @1 l ei1 g r ou ) ((pos "K6%" "OA%" "Pu%" ) ))
( "allegros"  n ( @1 l ei1 g r ou z ) ((pos "Kj%" ) ))
( "alleluia"  uh ( a2 l i l uu1 y @ ) ((pos "W-%" ) ))
( "allergen"  n ( a1 l @ jh e n ) ((pos "K6$" ) ))
( "allergens"  n ( a1 l @ jh e n z ) ((pos "Kj$" ) ))
( "allergic"  j ( @1 l @@1 jh i k ) ((pos "OA%" ) ))
( "allergies"  n ( a1 l @ jh i z ) ((pos "Kj%" ) ))
( "allergy"  n ( a1 l @ jh ii ) ((pos "K8%" ) ))
( "alleviate"  v ( @1 l ii1 v i ei t ) ((pos "H2%" ) (subcat )))
( "alleviated"  v ( @1 l ii1 v i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "alleviates"  v ( @1 l ii1 v i ei t s ) ((pos "Ha%" ) (subcat )))
( "alleviating"  v ( @1 l ii1 v i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "alleviation"  n ( @1 l ii2 v i ei1 sh @ n ) ((pos "K6%" ) ))
( "alleviations"  n ( @1 l ii2 v i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "alley"  n ( a1 l ii ) ((pos "K6%" ) ))
( "alleys"  n ( a1 l i z ) ((pos "Kj%" ) ))
( "alleyway"  n ( a1 l i w ei ) ((pos "K6%" ) ))
( "alleyways"  n ( a1 l i w ei z ) ((pos "Kj%" ) ))
( "alliance"  n ( @1 l ai1 @ n s ) ((pos "M6%" ) ))
( "alliances"  n ( @1 l ai1 @ n s i z ) ((pos "Mj%" ) ))
( "allied"  v ( @1 l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "allies"  n ( a1 l ai z ) ((pos "Kj%" ) ))
( "allies"  v ( @1 l ai1 z ) ((pos "Ha%" ) (subcat )))
( "alligator"  n ( a1 l i g ei t @ r ) ((pos "K6%" ) ))
( "alligators"  n ( a1 l i g ei t @ z ) ((pos "Kj%" ) ))
( "alliteration"  n ( @1 l i2 t @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "alliterative"  j ( @1 l i1 t r @ t i v ) ((pos "OA%" ) ))
( "alliteratively"  a ( @1 l i1 t r @ t i v l ii ) ((pos "Pu%" ) ))
( "allocate"  v ( a1 l @ k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "allocated"  v ( a1 l @ k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "allocates"  v ( a1 l @ k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "allocating"  v ( a1 l @ k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "allocation"  n ( a2 l @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "allocations"  n ( a2 l @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "allot"  v ( @1 l o1 t ) ((pos "H4%" ) (subcat "6A" "12A" "13A" )))
( "allotment"  n ( @1 l o1 t m @ n t ) ((pos "M6%" ) ))
( "allotments"  n ( @1 l o1 t m @ n t s ) ((pos "Mj%" ) ))
( "allots"  v ( @1 l o1 t s ) ((pos "Ha%" ) (subcat "6A" "12A" "13A" )))
( "allotted"  v ( @1 l o1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" )))
( "allotting"  v ( @1 l o1 t i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" )))
( "allow"  v ( @1 l au1 ) ((pos "J0%" ) (subcat "3A" "6A" "6C" "9" "12A" "13A" "14" "15B" "17" )))
( "allowable"  j ( @1 l ou1 @ b l ) ((pos "OA%" ) ))
( "allowance"  n ( @1 l au1 @ n s ) ((pos "K6%" ) ))
( "allowances"  n ( @1 l au1 @ n s i z ) ((pos "Kj%" ) ))
( "allowed"  v ( @1 l au1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "6C" "9" "12A" "13A" "14" "15B" "17" )))
( "allowing"  v ( @1 l au1 i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "6C" "9" "12A" "13A" "14" "15B" "17" )))
( "allows"  v ( @1 l au1 z ) ((pos "Ja%" ) (subcat "3A" "6A" "6C" "9" "12A" "13A" "14" "15B" "17" )))
( "alloy"  n ( a1 l oi ) ((pos "M6%" ) ))
( "alloy"  v ( @1 l oi1 ) ((pos "H0%" ) (subcat )))
( "alloyed"  v ( @1 l oi1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "alloying"  v ( @1 l oi1 i ng ) ((pos "Hb%" ) (subcat )))
( "alloys"  n ( a1 l oi z ) ((pos "Mj%" ) ))
( "alloys"  v ( @1 l oi1 z ) ((pos "Ha%" ) (subcat )))
( "allspice"  n ( oo1 l s p ai s ) ((pos "L@%" ) ))
( "allude"  v ( @1 l uu1 d ) ((pos "I2%" ) (subcat )))
( "alluded"  v ( @1 l uu1 d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "alludes"  v ( @1 l uu1 d z ) ((pos "Ia%" ) (subcat )))
( "alluding"  v ( @1 l uu1 d i ng ) ((pos "Ib%" ) (subcat )))
( "allure"  v ( @1 l u@1 r ) ((pos "H2%" "M6%" ) (subcat "6A" "14" )))
( "allured"  v ( @1 l u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "allurement"  n ( @1 l u@1 m @ n t ) ((pos "M6%" ) ))
( "allurements"  n ( @1 l u@1 m @ n t s ) ((pos "Mj$" ) ))
( "allures"  v ( @1 l u@1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "alluring"  v ( @1 l u@1 r i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "allusion"  n ( @1 l uu1 zh n ) ((pos "K6%" ) ))
( "allusions"  n ( @1 l uu1 zh n z ) ((pos "Kj%" ) ))
( "allusive"  j ( @1 l uu1 s i v ) ((pos "OA$" ) ))
( "alluvial"  j ( @1 l uu1 v i@ l ) ((pos "OA%" ) ))
( "ally"  n ( a1 l ai ) ((pos "K8%" ) ))
( "ally"  v ( @1 l ai1 ) ((pos "H3%" ) (subcat )))
( "allying"  v ( @1 l ai1 i ng ) ((pos "Hb%" ) (subcat )))
( "almanac"  n ( oo1 l m @ n a k ) ((pos "K6%" ) ))
( "almanacs"  n ( oo1 l m @ n a k s ) ((pos "Kj%" ) ))
( "almighty"  n ( oo1 l m ai1 t ii ) ((pos "Ki%" "OA%" ) ))
( "almond"  n ( aa1 m @ n d ) ((pos "K6%" ) ))
( "almond-eyed"  j ( aa2 m @ n d - ai1 d ) ((pos "OA%" ) ))
( "almonds"  n ( aa1 m @ n d z ) ((pos "Kj%" ) ))
( "almoner"  n ( aa1 m @ n @ r ) ((pos "K6%" ) ))
( "almoners"  n ( aa1 m @ n @ z ) ((pos "Kj%" ) ))
( "almost"  a ( oo1 l m ou s t ) ((pos "Pu*" ) ))
( "alms"  n ( aa1 m z ) ((pos "Kj%" ) ))
( "alms-box"  n ( aa1 m z - b o k s ) ((pos "K7%" ) ))
( "alms-boxes"  n ( aa1 m z - b o k s i z ) ((pos "Kj%" ) ))
( "alms-giving"  n ( aa1 m z - g i v i ng ) ((pos "L@%" ) ))
( "alms-house"  n ( aa1 m z - h au s ) ((pos "K6%" ) ))
( "alms-houses"  n ( aa1 m z - h au z i z ) ((pos "Kj%" ) ))
( "aloe"  n ( a1 l ou ) ((pos "K6%" ) ))
( "aloes"  n ( a1 l ou z ) ((pos "Kj%" ) ))
( "aloft"  a ( @1 l o1 f t ) ((pos "Pu%" ) ))
( "alone"  j ( @1 l ou1 n ) ((pos "Op*" "Pu*" ) ))
( "along"  a ( @1 l o1 ng ) ((pos "Pu*" "T-*" ) ))
( "alongside"  a ( @1 l o2 ng s ai1 d ) ((pos "Pu%" "T-%" ) ))
( "aloof"  j ( @1 l uu1 f ) ((pos "OA%" "Pu%" ) ))
( "aloofness"  n ( @1 l uu1 f n @ s ) ((pos "L@%" ) ))
( "aloud"  a ( @1 l au1 d ) ((pos "Pu%" ) ))
( "alp"  n ( a1 l p ) ((pos "K6%" ) ))
( "alpaca"  n ( a1 l p a1 k @ ) ((pos "M6%" ) ))
( "alpacas"  n ( a1 l p a1 k @ z ) ((pos "Mj$" ) ))
( "alpenstock"  n ( a1 l p @ n s t o k ) ((pos "K6%" ) ))
( "alpenstocks"  n ( a1 l p @ n s t o k s ) ((pos "Kj%" ) ))
( "alpha"  n ( a1 l f @ ) ((pos "K6%" ) ))
( "alphabet"  n ( a1 l f @ b e t ) ((pos "K6%" ) ))
( "alphabetical"  j ( a2 l f @ b e1 t i k l ) ((pos "OA%" ) ))
( "alphabetically"  a ( a2 l f @ b e1 t i k l ii ) ((pos "Pu%" ) ))
( "alphabets"  n ( a1 l f @ b e t s ) ((pos "Kj%" ) ))
( "alphas"  n ( a1 l f @ z ) ((pos "Kj$" ) ))
( "alpine"  j ( a1 l p ai n ) ((pos "OA%" ) ))
( "alpinist"  n ( a1 l p i n i s t ) ((pos "K6%" ) ))
( "alpinists"  n ( a1 l p i n i s t s ) ((pos "Kj%" ) ))
( "alps"  n ( a1 l p s ) ((pos "Kj%" ) ))
( "already"  a ( oo1 l r e1 d ii ) ((pos "Pu*" ) ))
( "alright"  j ( oo1 l r ai1 t ) ((pos "OA%" "Pu%" ) ))
( "alsatian"  n ( a1 l s ei1 sh @ n ) ((pos "K6%" ) ))
( "alsatians"  n ( a1 l s ei1 sh @ n z ) ((pos "Kj%" ) ))
( "also"  a ( oo1 l s ou ) ((pos "Pu*" ) ))
( "also-ran"  n ( oo1 l s ou - r a n ) ((pos "K6%" ) ))
( "also-rans"  n ( oo1 l s ou - r a n z ) ((pos "Kj%" ) ))
( "altar"  n ( oo1 l t @ r ) ((pos "K6%" ) ))
( "altar-piece"  n ( oo1 l t @ - p ii s ) ((pos "K6%" ) ))
( "altar-pieces"  n ( oo1 l t @ - p ii s i z ) ((pos "Kj$" ) ))
( "altars"  n ( oo1 l t @ z ) ((pos "Kj%" ) ))
( "alter"  v ( oo1 l t @ r ) ((pos "J0%" ) (subcat "2A" )))
( "alter ego"  n ( a2 l t @ r - e1 g ou ) ((pos "K6%" ) ))
( "alter egos"  n ( a2 l t @ r - e1 g ou z ) ((pos "Kj%" ) ))
( "alterable"  j ( o1 l t @ r @ b l ) ((pos "OA%" ) ))
( "alteration"  n ( oo2 l t @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "alterations"  n ( oo2 l t @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "altercation"  n ( oo2 l t @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "altercations"  n ( oo2 l t @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "altered"  v ( oo1 l t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "altering"  v ( oo1 l t @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "alternate"  v ( oo1 l t @ n ei t ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "alternate"  j ( oo1 l t @@1 n @ t ) ((pos "OA%" ) ))
( "alternated"  v ( oo1 l t @ n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "alternately"  a ( oo1 l t @@1 n @ t l ii ) ((pos "Pu%" ) ))
( "alternates"  v ( oo1 l t @ n ei t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "alternating"  v ( oo1 l t @ n ei t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "alternation"  n ( oo2 l t @ n ei1 sh @ n ) ((pos "K6%" ) ))
( "alternations"  n ( oo2 l t @ n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "alternative"  n ( oo1 l t @@1 n @ t i v ) ((pos "K6%" "OA%" ) ))
( "alternatively"  a ( oo1 l t @@1 n @ t i v l ii ) ((pos "Pu%" ) ))
( "alternatives"  n ( oo1 l t @@1 n @ t i v z ) ((pos "Kj%" ) ))
( "alters"  v ( oo1 l t @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "although"  cc ( oo1 l dh ou1 ) ((pos "V-%" ) ))
( "altimeter"  n ( a1 l t i m ii t @ r ) ((pos "K6%" ) ))
( "altimeters"  n ( a1 l t i m ii t @ z ) ((pos "Kj%" ) ))
( "altitude"  n ( a1 l t i t y uu d ) ((pos "K6%" ) ))
( "altitudes"  n ( a1 l t i t y uu d z ) ((pos "Kj%" ) ))
( "alto"  n ( a1 l t ou ) ((pos "K6%" ) ))
( "altogether"  a ( oo2 l t @ g e1 dh @ r ) ((pos "Pu%" ) ))
( "altos"  n ( a1 l t ou z ) ((pos "Kj%" ) ))
( "altruism"  n ( a1 l t r uu i z @ m ) ((pos "M6%" ) ))
( "altruisms"  n ( a1 l t r uu i z @ m z ) ((pos "Mj$" ) ))
( "altruist"  n ( a1 l t r uu i s t ) ((pos "K6%" ) ))
( "altruistic"  j ( a2 l t r uu i1 s t i k ) ((pos "OA%" ) ))
( "altruistically"  a ( a2 l t r uu i1 s t i k l ii ) ((pos "Pu%" ) ))
( "altruists"  n ( a1 l t r uu i s t s ) ((pos "Kj$" ) ))
( "alum"  n ( a1 l @ m ) ((pos "L@$" ) ))
( "aluminium"  n ( a2 l y u m i1 n i@ m ) ((pos "L@%" ) ))
( "alumna"  n ( @1 l uh1 m n @ ) ((pos "Ki$" ) ))
( "alumnae"  n ( @1 l uh1 m n ii ) ((pos "Kj$" ) ))
( "alumni"  n ( @1 l uh1 m n ai ) ((pos "Kj%" ) ))
( "alumnus"  n ( @1 l uh1 m n @ s ) ((pos "Ki%" ) ))
( "alveolar"  n ( a1 l v i@1 l @ r ) ((pos "K6$" "OA$" ) ))
( "alveolars"  n ( a1 l v i@1 l @ z ) ((pos "Kj$" ) ))
( "always"  a ( oo1 l w ei z ) ((pos "Pu*" ) ))
( "am"  v ( a m ) ((pos "Ge*" "Ie%" ) (subcat )))
( "am"  n ( ei2 e1 m ) ((pos "Y~%" ) ))
( "amah"  n ( aa1 m @ ) ((pos "K6$" ) ))
( "amahs"  n ( aa1 m @ z ) ((pos "Kj$" ) ))
( "amain"  a ( @1 m ei1 n ) ((pos "Pu$" ) ))
( "amalgam"  n ( @1 m a1 l g @ m ) ((pos "K6%" ) ))
( "amalgamate"  v ( @1 m a1 l g @ m ei t ) ((pos "J2%" ) (subcat "2A" )))
( "amalgamated"  v ( @1 m a1 l g @ m ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "amalgamates"  v ( @1 m a1 l g @ m ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "amalgamating"  v ( @1 m a1 l g @ m ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "amalgamation"  n ( @1 m a2 l g @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "amalgamations"  n ( @1 m a2 l g @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "amalgams"  n ( @1 m a1 l g @ m z ) ((pos "Kj%" ) ))
( "amanuenses"  n ( @1 m a2 n y u e1 n s ii z ) ((pos "Kj$" ) ))
( "amanuensis"  n ( @1 m a2 n y u e1 n s i s ) ((pos "Ki$" ) ))
( "amaryllis"  n ( a2 m @ r i1 l i s ) ((pos "K7$" ) ))
( "amaryllises"  n ( a2 m @ r i1 l i s i z ) ((pos "Kj$" ) ))
( "amass"  v ( @1 m a1 s ) ((pos "H1%" ) (subcat )))
( "amassed"  v ( @1 m a1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "amasses"  v ( @1 m a1 s i z ) ((pos "Ha%" ) (subcat )))
( "amassing"  v ( @1 m a1 s i ng ) ((pos "Hb%" ) (subcat )))
( "amateur"  n ( a1 m @ t @ r ) ((pos "K6%" ) ))
( "amateurish"  j ( a1 m @ t @ r i sh ) ((pos "OA%" ) ))
( "amateurism"  n ( a1 m @ t @ r i z @ m ) ((pos "L@%" ) ))
( "amateurs"  n ( a1 m @ t @ z ) ((pos "Kj%" ) ))
( "amatory"  j ( a1 m @ t @ r ii ) ((pos "OA%" ) ))
( "amaze"  v ( @1 m ei1 z ) ((pos "H2%" ) (subcat )))
( "amazed"  v ( @1 m ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "amazement"  n ( @1 m ei1 z m @ n t ) ((pos "L@%" ) ))
( "amazes"  v ( @1 m ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "amazing"  v ( @1 m ei1 z i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "amazingly"  a ( @1 m ei1 z i ng l ii ) ((pos "Pu%" ) ))
( "ambassador"  n ( a1 m b a1 s @ d @ r ) ((pos "K6%" ) ))
( "ambassadorial"  j ( a1 m b a2 s @ d oo1 r i@ l ) ((pos "OA%" ) ))
( "ambassadors"  n ( a1 m b a1 s @ d @ z ) ((pos "Kj%" ) ))
( "ambassadress"  n ( a1 m b a1 s @ d r i s ) ((pos "K7%" ) ))
( "ambassadresses"  n ( a1 m b a1 s @ d r i s i z ) ((pos "Kj$" ) ))
( "amber"  n ( a1 m b @ r ) ((pos "L@%" ) ))
( "ambergris"  n ( a1 m b @ g r ii s ) ((pos "L@$" ) ))
( "ambidextrous"  j ( a2 m b i d e1 k s t r @ s ) ((pos "OA%" ) ))
( "ambience"  n ( a1 m b i@ n s ) ((pos "Ki%" ) ))
( "ambient"  j ( a1 m b i@ n t ) ((pos "OA%" ) ))
( "ambiguities"  n ( a2 m b i g y uu1 i t i z ) ((pos "Mj%" ) ))
( "ambiguity"  n ( a2 m b i g y uu1 i t ii ) ((pos "M8%" ) ))
( "ambiguous"  j ( a1 m b i1 g y u@ s ) ((pos "OA%" ) ))
( "ambiguously"  a ( a1 m b i1 g y u@ s l ii ) ((pos "Pu%" ) ))
( "ambit"  n ( a1 m b i t ) ((pos "K6%" ) ))
( "ambition"  n ( a1 m b i1 sh @ n ) ((pos "M6%" ) ))
( "ambitions"  n ( a1 m b i1 sh @ n z ) ((pos "Mj%" ) ))
( "ambitious"  j ( a1 m b i1 sh @ s ) ((pos "OA%" ) ))
( "ambitiously"  a ( a1 m b i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "ambits"  n ( a1 m b i t s ) ((pos "Kj$" ) ))
( "ambivalence"  n ( a1 m b i1 v @ l @ n s ) ((pos "L@%" ) ))
( "ambivalent"  j ( a1 m b i1 v @ l @ n t ) ((pos "OA%" ) ))
( "amble"  v ( a1 m b l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "ambled"  v ( a1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "ambles"  v ( a1 m b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "ambling"  v ( a1 m b l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "ambrosia"  n ( a1 m b r ou1 z i@ ) ((pos "L@%" ) ))
( "ambulance"  n ( a1 m b y u l @ n s ) ((pos "K6%" ) ))
( "ambulances"  n ( a1 m b y u l @ n s i z ) ((pos "Kj%" ) ))
( "ambuscade"  v ( a2 m b @ s k ei1 d ) ((pos "H2$" "K6$" ) ))
( "ambuscaded"  v ( a2 m b @ s k ei1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "ambuscades"  v ( a2 m b @ s k ei1 d z ) ((pos "Ha$" "Kj$" ) ))
( "ambuscading"  v ( a2 m b @ s k ei1 d i ng ) ((pos "Hb$" ) ))
( "ambush"  v ( a1 m b u sh ) ((pos "H1%" "M7%" ) (subcat )))
( "ambushed"  v ( a1 m b u sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ambushes"  v ( a1 m b u sh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "ambushing"  v ( a1 m b u sh i ng ) ((pos "Hb%" ) (subcat )))
( "ameba"  n ( @1 m ii1 b @ ) ((pos "K6$" ) ))
( "amebas"  n ( @1 m ii1 b @ z ) ((pos "Kj$" ) ))
( "ameer"  n ( @1 m i@1 r ) ((pos "K6$" ) ))
( "ameers"  n ( @1 m i@1 z ) ((pos "Kj$" ) ))
( "ameliorate"  v ( @1 m ii1 l i@ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "ameliorated"  v ( @1 m ii1 l i@ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ameliorates"  v ( @1 m ii1 l i@ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "ameliorating"  v ( @1 m ii1 l i@ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "amelioration"  n ( @1 m ii2 l i@ r ei1 sh @ n ) ((pos "K6%" ) ))
( "ameliorations"  n ( @1 m ii2 l i@ r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "amen"  uh ( aa1 m e1 n ) ((pos "W-%" ) ))
( "amenable"  j ( @1 m ii1 n @ b l ) ((pos "OA%" ) ))
( "amend"  v ( @1 m e1 n d ) ((pos "J0%" ) (subcat "2A" )))
( "amendable"  j ( @1 m e1 n d @ b l ) ((pos "OA$" ) ))
( "amended"  v ( @1 m e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "amending"  v ( @1 m e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "amendment"  n ( @1 m e1 n d m @ n t ) ((pos "M6%" ) ))
( "amendments"  n ( @1 m e1 n d m @ n t s ) ((pos "Mj%" ) ))
( "amends"  v ( @1 m e1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "amenities"  n ( @1 m ii1 n i t i z ) ((pos "Kj%" ) ))
( "amenity"  n ( @1 m ii1 n i t ii ) ((pos "K8%" ) ))
( "amethyst"  n ( a1 m i th i s t ) ((pos "K6%" ) ))
( "amethysts"  n ( a1 m i th i s t s ) ((pos "Kj%" ) ))
( "amiability"  n ( ei2 m i@ b i1 l i t ii ) ((pos "L@%" ) ))
( "amiable"  j ( ei1 m i@ b l ) ((pos "OA%" ) ))
( "amiably"  a ( ei1 m i@ b l ii ) ((pos "Pu%" ) ))
( "amicability"  n ( a2 m i k @ b i1 l i t ii ) ((pos "L@$" ) ))
( "amicable"  j ( a1 m i k @ b l ) ((pos "OA%" ) ))
( "amicably"  a ( a1 m i k @ b l ii ) ((pos "Pu%" ) ))
( "amid"  in ( @1 m i1 d ) ((pos "T-%" ) ))
( "amidships"  a ( @1 m i1 d sh i p s ) ((pos "Pu%" ) ))
( "amidst"  in ( @1 m i1 d s t ) ((pos "T-%" ) ))
( "amir"  n ( @1 m i@1 r ) ((pos "K6$" ) ))
( "amirs"  n ( @1 m i@1 z ) ((pos "Kj$" ) ))
( "amiss"  j ( @1 m i1 s ) ((pos "Op%" "Pu%" ) ))
( "amity"  n ( a1 m i t ii ) ((pos "L@$" ) ))
( "ammeter"  n ( a1 m i t @ r ) ((pos "K6$" ) ))
( "ammeters"  n ( a1 m i t @ z ) ((pos "Kj$" ) ))
( "ammonia"  n ( @1 m ou1 n i@ ) ((pos "L@%" ) ))
( "ammoniated"  j ( @1 m ou1 n i ei t i d ) ((pos "OA$" ) ))
( "ammonite"  n ( a1 m @ n ai t ) ((pos "K6%" ) ))
( "ammonites"  n ( a1 m @ n ai t s ) ((pos "Kj%" ) ))
( "ammunition"  n ( a2 m y u n i1 sh @ n ) ((pos "L@%" ) ))
( "amnesia"  n ( a1 m n ii1 z i@ ) ((pos "L@%" ) ))
( "amnesties"  n ( a1 m n @ s t i z ) ((pos "Kj%" ) ))
( "amnesty"  n ( a1 m n @ s t ii ) ((pos "K8%" ) ))
( "amoeba"  n ( @1 m ii1 b @ ) ((pos "K6%" ) ))
( "amoebae"  n ( @1 m ii1 b ii ) ((pos "Kj%" ) ))
( "amoebas"  n ( @1 m ii1 b @ z ) ((pos "Kj%" ) ))
( "amoebic"  j ( @1 m ii1 b i k ) ((pos "OA%" ) ))
( "amok"  a ( @1 m o1 k ) ((pos "Pu%" ) ))
( "among"  in ( @1 m uh1 ng ) ((pos "T-*" ) ))
( "amongst"  in ( @1 m uh1 ng s t ) ((pos "T-%" ) ))
( "amoral"  j ( ei2 m o1 r @ l ) ((pos "OA%" ) ))
( "amorous"  j ( a1 m @ r @ s ) ((pos "OA%" ) ))
( "amorously"  a ( a1 m @ r @ s l ii ) ((pos "Pu%" ) ))
( "amorphous"  j ( @1 m oo1 f @ s ) ((pos "OA%" ) ))
( "amortization"  n ( @1 m oo2 t i z ei1 sh @ n ) ((pos "K6$" ) ))
( "amortizations"  n ( @1 m oo2 t i z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "amortize"  v ( @1 m oo1 t ai z ) ((pos "H2$" ) (subcat )))
( "amortized"  v ( @1 m oo1 t ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "amortizes"  v ( @1 m oo1 t ai z i z ) ((pos "Ha$" ) (subcat )))
( "amortizing"  v ( @1 m oo1 t ai z i ng ) ((pos "Hb$" ) (subcat )))
( "amount"  v ( @1 m au1 n t ) ((pos "I0%" "K6%" ) (subcat )))
( "amounted"  v ( @1 m au1 n t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "amounting"  v ( @1 m au1 n t i ng ) ((pos "Ib%" ) (subcat )))
( "amounts"  v ( @1 m au1 n t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "amour"  n ( @1 m u@1 r ) ((pos "K6%" ) ))
( "amour-propre"  n ( a2 m u@ - p r o1 p r @ ) ((pos "L@%" ) ))
( "amours"  n ( @1 m u@1 z ) ((pos "Kj%" ) ))
( "amp"  n ( a1 m p ) ((pos "K6%" ) ))
( "ampere"  n ( a1 m p e@ r ) ((pos "K6%" ) ))
( "amperes"  n ( a1 m p e@ z ) ((pos "Kj%" ) ))
( "amphetamine"  n ( a1 m f e1 t @ m ii n ) ((pos "M6%" ) ))
( "amphetamines"  n ( a1 m f e1 t @ m ii n z ) ((pos "Mj%" ) ))
( "amphibian"  n ( a1 m f i1 b i@ n ) ((pos "K6%" ) ))
( "amphibians"  n ( a1 m f i1 b i@ n z ) ((pos "Kj%" ) ))
( "amphibious"  j ( a1 m f i1 b i@ s ) ((pos "OA%" ) ))
( "amphitheatre"  n ( a1 m f i th i@ t @ r ) ((pos "K6%" ) ))
( "amphitheatres"  n ( a1 m f i th i@ t @ z ) ((pos "Kj%" ) ))
( "amphora"  n ( a1 m f @ r @ ) ((pos "K6$" ) ))
( "amphorae"  n ( a1 m f @ r ii ) ((pos "Kj$" ) ))
( "amphoras"  n ( a1 m f @ r @ z ) ((pos "Kj$" ) ))
( "ample"  j ( a1 m p l ) ((pos "OB%" ) ))
( "ampler"  j ( a1 m p l @ r ) ((pos "Or%" ) ))
( "amplest"  j ( a1 m p l i s t ) ((pos "Os%" ) ))
( "amplification"  n ( a2 m p l i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "amplifications"  n ( a2 m p l i f i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "amplified"  v ( a1 m p l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "amplifier"  n ( a1 m p l i f ai @ r ) ((pos "K6%" ) ))
( "amplifiers"  n ( a1 m p l i f ai @ z ) ((pos "Kj%" ) ))
( "amplifies"  v ( a1 m p l i f ai z ) ((pos "Ha%" ) (subcat )))
( "amplify"  v ( a1 m p l i f ai ) ((pos "H3%" ) (subcat )))
( "amplifying"  v ( a1 m p l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "amplitude"  n ( a1 m p l i t y uu d ) ((pos "L@%" ) ))
( "amply"  a ( a1 m p l ii ) ((pos "Pu%" ) ))
( "ampoule"  n ( a1 m p uu l ) ((pos "K6$" ) ))
( "ampoules"  n ( a1 m p uu l z ) ((pos "Kj$" ) ))
( "amps"  n ( a1 m p s ) ((pos "Kj%" ) ))
( "amputate"  v ( a1 m p y u t ei t ) ((pos "H2%" ) (subcat )))
( "amputated"  v ( a1 m p y u t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "amputates"  v ( a1 m p y u t ei t s ) ((pos "Ha%" ) (subcat )))
( "amputating"  v ( a1 m p y u t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "amputation"  n ( a2 m p y u t ei1 sh @ n ) ((pos "K6%" ) ))
( "amputations"  n ( a2 m p y u t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "amuck"  a ( @1 m uh1 k ) ((pos "Pu%" ) ))
( "amulet"  n ( a1 m y u l i t ) ((pos "K6%" ) ))
( "amulets"  n ( a1 m y u l i t s ) ((pos "Kj%" ) ))
( "amuse"  v ( @1 m y uu1 z ) ((pos "H2%" ) (subcat )))
( "amused"  v ( @1 m y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "amusement"  n ( @1 m y uu1 z m @ n t ) ((pos "M6%" ) ))
( "amusements"  n ( @1 m y uu1 z m @ n t s ) ((pos "Mj%" ) ))
( "amuses"  v ( @1 m y uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "amusing"  v ( @1 m y uu1 z i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "amusingly"  a ( @1 m y uu1 z i ng l ii ) ((pos "Pu%" ) ))
( "an"  cc ( a n ) ((pos "V-%" ) ))
( "an"  dt ( @ n ) ((pos "S-*" ) ))
( "anachronism"  n ( @1 n a1 k r @ n i z @ m ) ((pos "K6%" ) ))
( "anachronisms"  n ( @1 n a1 k r @ n i z @ m z ) ((pos "Kj%" ) ))
( "anachronistic"  j ( @1 n a2 k r @ n i1 s t i k ) ((pos "OA%" ) ))
( "anaconda"  n ( a2 n @ k o1 n d @ ) ((pos "K6%" ) ))
( "anacondas"  n ( a2 n @ k o1 n d @ z ) ((pos "Kj%" ) ))
( "anaemia"  n ( @1 n ii1 m i@ ) ((pos "L@%" ) ))
( "anaemic"  j ( @1 n ii1 m i k ) ((pos "OA%" ) ))
( "anaesthesia"  n ( a2 n i s th ii1 z i@ ) ((pos "L@%" ) ))
( "anaesthetic"  n ( a2 n i s th e1 t i k ) ((pos "K6%" "OA%" ) ))
( "anaesthetics"  n ( a2 n i s th e1 t i k s ) ((pos "Kj%" ) ))
( "anaesthetist"  n ( @1 n ii1 s th @ t i s t ) ((pos "K6%" ) ))
( "anaesthetists"  n ( @1 n ii1 s th @ t i s t s ) ((pos "Kj%" ) ))
( "anaesthetize"  v ( @1 n ii1 s th @ t ai z ) ((pos "H2%" ) (subcat )))
( "anaesthetized"  v ( @1 n ii1 s th @ t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "anaesthetizes"  v ( @1 n ii1 s th @ t ai z i z ) ((pos "Ha%" ) (subcat )))
( "anaesthetizing"  v ( @1 n ii1 s th @ t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "anagram"  n ( a1 n @ g r a m ) ((pos "K6%" ) ))
( "anagrams"  n ( a1 n @ g r a m z ) ((pos "Kj%" ) ))
( "anal"  j ( ei1 n l ) ((pos "OA%" ) ))
( "analecta"  n ( a2 n @ l e1 k t @ ) ((pos "Kj$" ) ))
( "analects"  n ( a1 n @ l e k t s ) ((pos "Kj$" ) ))
( "analgesia"  n ( a2 n a l jh ii1 z i@ ) ((pos "L@%" ) ))
( "analgesic"  n ( a2 n a l jh ii1 z i k ) ((pos "K6%" ) ))
( "analgesics"  n ( a2 n a l jh ii1 z i k s ) ((pos "Kj%" ) ))
( "analog"  n ( a1 n @ l o g ) ((pos "K6$" ) ))
( "analogies"  n ( @1 n a1 l @ jh i z ) ((pos "Mj%" ) ))
( "analogous"  j ( @1 n a1 l @ g @ s ) ((pos "OA%" ) ))
( "analogously"  a ( @1 n a1 l @ g @ s l ii ) ((pos "Pu%" ) ))
( "analogs"  n ( a1 n @ l o g z ) ((pos "Kj$" ) ))
( "analogue"  n ( a1 n @ l o g ) ((pos "K6%" ) ))
( "analogues"  n ( a1 n @ l o g z ) ((pos "Kj%" ) ))
( "analogy"  n ( @1 n a1 l @ jh ii ) ((pos "M8%" ) ))
( "analyse"  v ( a1 n @ l ai z ) ((pos "H2%" ) (subcat )))
( "analysed"  v ( a1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "analyses"  v ( a1 n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "analyses"  n ( @1 n a1 l @ s ii z ) ((pos "Kj%" ) ))
( "analysing"  v ( a1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "analysis"  n ( @1 n a1 l @ s i s ) ((pos "Mi%" ) ))
( "analyst"  n ( a1 n @ l i s t ) ((pos "K6%" ) ))
( "analysts"  n ( a1 n @ l i s t s ) ((pos "Kj%" ) ))
( "analytic"  j ( a2 n @ l i1 t i k ) ((pos "OA%" ) ))
( "analytical"  j ( a2 n @ l i1 t i k l ) ((pos "OA%" ) ))
( "analytically"  a ( a2 n @ l i1 t i k l ii ) ((pos "Pu%" ) ))
( "analyze"  v ( a1 n @ l ai z ) ((pos "H2%" ) (subcat )))
( "analyzed"  v ( a1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "analyzes"  v ( a1 n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "analyzing"  v ( a1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "anapaest"  n ( a1 n @ p ii s t ) ((pos "K6$" ) ))
( "anapaestic"  j ( a2 n @ p ii1 s t i k ) ((pos "OA$" ) ))
( "anapaests"  n ( a1 n @ p ii s t s ) ((pos "Kj$" ) ))
( "anarchic"  j ( @1 n aa1 k i k ) ((pos "OA%" ) ))
( "anarchically"  a ( @1 n aa1 k i k l ii ) ((pos "Pu%" ) ))
( "anarchism"  n ( a1 n @ k i z @ m ) ((pos "L@%" ) ))
( "anarchist"  n ( a1 n @ k i s t ) ((pos "K6%" ) ))
( "anarchists"  n ( a1 n @ k i s t s ) ((pos "Kj%" ) ))
( "anarchy"  n ( a1 n @ k ii ) ((pos "L@%" ) ))
( "anathema"  n ( @1 n a1 th @ m @ ) ((pos "K6%" ) ))
( "anathemas"  n ( @1 n a1 th @ m @ z ) ((pos "Kj$" ) ))
( "anathematize"  v ( @1 n a1 th @ m @ t ai z ) ((pos "J2$" ) ))
( "anathematized"  v ( @1 n a1 th @ m @ t ai z d ) ((pos "Jc$" "Jd$" ) ))
( "anathematizes"  v ( @1 n a1 th @ m @ t ai z i z ) ((pos "Ja$" ) ))
( "anathematizing"  v ( @1 n a1 th @ m @ t ai z i ng ) ((pos "Jb$" ) ))
( "anatomical"  j ( a2 n @ t o1 m i k l ) ((pos "OA%" ) ))
( "anatomically"  a ( a2 n @ t o1 m i k l ii ) ((pos "Pu%" ) ))
( "anatomist"  n ( @1 n a1 t @ m i s t ) ((pos "K6%" ) ))
( "anatomists"  n ( @1 n a1 t @ m i s t s ) ((pos "Kj%" ) ))
( "anatomy"  n ( @1 n a1 t @ m ii ) ((pos "L@%" ) ))
( "ancestor"  n ( a1 n s e s t @ r ) ((pos "K6%" ) ))
( "ancestors"  n ( a1 n s e s t @ z ) ((pos "Kj%" ) ))
( "ancestral"  j ( a1 n s e1 s t r @ l ) ((pos "OA%" ) ))
( "ancestress"  n ( a1 n s e s t r i s ) ((pos "K7%" ) ))
( "ancestresses"  n ( a1 n s e s t r i s i z ) ((pos "Kj%" ) ))
( "ancestries"  n ( a1 n s e s t r i z ) ((pos "Kj%" ) ))
( "ancestry"  n ( a1 n s e s t r ii ) ((pos "K8%" ) ))
( "anchor"  v ( a1 ng k @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "anchorage"  n ( a1 n k @ r i jh ) ((pos "K6%" ) ))
( "anchorages"  n ( a1 n k @ r i jh i z ) ((pos "Kj$" ) ))
( "anchored"  v ( a1 ng k @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "anchoring"  v ( a1 ng k @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "anchorite"  n ( a1 ng k @ r ai t ) ((pos "K6$" ) ))
( "anchorites"  n ( a1 ng k @ r ai t s ) ((pos "Kj$" ) ))
( "anchorman"  n ( a1 n k @ m @ n ) ((pos "Ki%" ) ))
( "anchormen"  n ( a1 n k @ m @ n ) ((pos "Kj%" ) ))
( "anchors"  v ( a1 ng k @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "anchovies"  n ( a1 n ch @ v i z ) ((pos "Kj%" ) ))
( "anchovy"  n ( a1 n ch @ v ii ) ((pos "K8%" ) ))
( "ancient"  j ( ei1 n sh @ n t ) ((pos "OA%" ) ))
( "ancillary"  j ( a1 n s i1 l @ r ii ) ((pos "OA%" ) ))
( "and"  cc ( a n d ) ((pos "V-*" ) ))
( "andante"  n ( a1 n d a1 n t ii ) ((pos "K6%" "OA%" "Pu%" ) ))
( "andantes"  n ( a1 n d a1 n t i z ) ((pos "Kj$" ) ))
( "andiron"  n ( a1 n d ai @ n ) ((pos "K6$" ) ))
( "andirons"  n ( a1 n d ai @ n z ) ((pos "Kj$" ) ))
( "anecdotal"  j ( a2 n i k d ou1 t l ) ((pos "OA%" ) ))
( "anecdote"  n ( a1 n i k d ou t ) ((pos "K6%" ) ))
( "anecdotes"  n ( a1 n i k d ou t s ) ((pos "Kj%" ) ))
( "anemometer"  n ( a2 n i m o1 m i t @ r ) ((pos "K6%" ) ))
( "anemometers"  n ( a2 n i m o1 m i t @ z ) ((pos "Kj%" ) ))
( "anemone"  n ( @1 n e1 m @ n ii ) ((pos "K6%" ) ))
( "anemones"  n ( @1 n e1 m @ n i z ) ((pos "Kj%" ) ))
( "anent"  in ( @1 n e1 n t ) ((pos "T-$" ) ))
( "aneroid"  n ( a1 n @ r oi d ) ((pos "K6$" "OA$" ) ))
( "aneroids"  n ( a1 n @ r oi d z ) ((pos "Kj$" ) ))
( "anesthetic"  n ( a2 n i s th e1 t i k ) ((pos "K6$" "OA$" ) ))
( "anesthetics"  n ( a2 n i s th e1 t i k s ) ((pos "Kj$" ) ))
( "anesthetist"  n ( @1 n ii1 s th @ t i s t ) ((pos "K6$" ) ))
( "anesthetists"  n ( @1 n ii1 s th @ t i s t s ) ((pos "Kj$" ) ))
( "anesthetize"  v ( @1 n ii1 s th @ t ai z ) ((pos "H2$" ) (subcat )))
( "anesthetized"  v ( @1 n ii1 s th @ t ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "anesthetizes"  v ( @1 n ii1 s th @ t ai z i z ) ((pos "Ha$" ) (subcat )))
( "anesthetizing"  v ( @1 n ii1 s th @ t ai z i ng ) ((pos "Hb$" ) (subcat )))
( "anew"  a ( @1 n y uu1 ) ((pos "Pu%" ) ))
( "angel"  n ( ei1 n jh l ) ((pos "K6%" ) ))
( "angelic"  j ( a1 n jh e1 l i k ) ((pos "OA%" ) ))
( "angelica"  n ( a1 n jh e1 l i k @ ) ((pos "L@%" ) ))
( "angelically"  a ( @1 n jh e1 l i k l ii ) ((pos "Pu%" ) ))
( "angels"  n ( ei1 n jh l z ) ((pos "Kj%" ) ))
( "angelus"  n ( a1 n jh i l @ s ) ((pos "K7%" ) ))
( "angeluses"  n ( a1 n jh i l @ s i z ) ((pos "Kj$" ) ))
( "anger"  v ( a1 ng g @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "angered"  v ( a1 ng g @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "angering"  v ( a1 ng g @ r i ng ) ((pos "Hb%" ) (subcat )))
( "angers"  v ( a1 ng g @ z ) ((pos "Ha%" ) (subcat )))
( "angina"  n ( a1 n jh ai1 n @ ) ((pos "L@%" ) ))
( "angle"  v ( a1 ng g l ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "angle-dozer"  n ( a1 ng g @ l - d ou z @ r ) ((pos "K6$" ) ))
( "angle-dozers"  n ( a1 ng g @ l - d ou z @ z ) ((pos "Kj$" ) ))
( "angle-iron"  n ( a1 ng g @ l - ai @ n ) ((pos "K6%" ) ))
( "angle-irons"  n ( a1 ng g @ l - ai @ n z ) ((pos "Kj%" ) ))
( "angle-park"  v ( a1 ng g @ l - p aa k ) ((pos "J0$" ) ))
( "angle-parked"  v ( a1 ng g @ l - p aa k t ) ((pos "Jc$" "Jd$" ) ))
( "angle-parking"  v ( a1 ng g @ l - p aa k i ng ) ((pos "Jb$" ) ))
( "angle-parks"  v ( a1 ng g @ l - p aa k s ) ((pos "Ja$" ) ))
( "angled"  v ( a1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "angler"  n ( a1 ng g l @ r ) ((pos "K6%" ) ))
( "anglers"  n ( a1 ng g l @ z ) ((pos "Kj%" ) ))
( "angles"  v ( a1 ng g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "anglicism"  n ( a1 ng g l i s i z @ m ) ((pos "K6%" ) ))
( "anglicisms"  n ( a1 ng g l i s i z @ m z ) ((pos "Kj$" ) ))
( "anglicize"  v ( a1 ng g l i s ai z ) ((pos "H2%" ) (subcat )))
( "anglicized"  v ( a1 ng g l i s ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "anglicizes"  v ( a1 ng g l i s ai z i z ) ((pos "Ha%" ) (subcat )))
( "anglicizing"  v ( a1 ng g l i s ai z i ng ) ((pos "Hb%" ) (subcat )))
( "angling"  v ( a1 ng g l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" )))
( "angora"  n ( a1 ng g oo1 r @ ) ((pos "M6%" ) ))
( "angoras"  n ( a1 ng g oo1 r @ z ) ((pos "Mj$" ) ))
( "angostura"  n ( a2 ng g @ s t y u@1 r @ ) ((pos "L@%" ) ))
( "angrier"  j ( a1 ng g r i@ r ) ((pos "Or%" ) ))
( "angriest"  j ( a1 ng g r i i s t ) ((pos "Os%" ) ))
( "angrily"  a ( a1 n g r @ l ii ) ((pos "Pu%" ) ))
( "angry"  j ( a1 ng g r ii ) ((pos "OD%" ) ))
( "angst"  n ( a1 ng s t ) ((pos "L@%" ) ))
( "anguish"  n ( a1 ng g w i sh ) ((pos "L@%" ) ))
( "anguished"  j ( a1 ng g w i sh t ) ((pos "OA%" ) ))
( "angular"  j ( a1 ng g y u l @ r ) ((pos "OA%" ) ))
( "angularities"  n ( a2 ng g y u l a1 r i t i z ) ((pos "Mj%" ) ))
( "angularity"  n ( a2 ng g y u l a1 r i t ii ) ((pos "M8%" ) ))
( "aniline"  n ( a1 n i l ii n ) ((pos "M6$" ) ))
( "anilines"  n ( a1 n i l ii n z ) ((pos "Mj$" ) ))
( "animadversion"  n ( a2 n i m a d v @@1 sh @ n ) ((pos "K6$" ) ))
( "animadversions"  n ( a2 n i m a d v @@1 sh @ n z ) ((pos "Kj$" ) ))
( "animadvert"  v ( a2 n i m a d v @@1 t ) ((pos "I0$" ) (subcat )))
( "animadverted"  v ( a2 n i m a d v @@1 t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "animadverting"  v ( a2 n i m a d v @@1 t i ng ) ((pos "Ib$" ) (subcat )))
( "animadverts"  v ( a2 n i m a d v @@1 t s ) ((pos "Ia$" ) (subcat )))
( "animal"  n ( a1 n i m @ l ) ((pos "K6%" ) ))
( "animalcule"  n ( a2 n i m a1 l k y uu l ) ((pos "K6$" ) ))
( "animalcules"  n ( a2 n i m a1 l k y uu l z ) ((pos "Kj$" ) ))
( "animals"  n ( a1 n i m @ l z ) ((pos "Kj%" ) ))
( "animate"  j ( a1 n i m @ t ) ((pos "OA%" ) ))
( "animate"  v ( a1 n i m ei t ) ((pos "H2%" ) (subcat "6A" )))
( "animated"  v ( a1 n i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "animatedly"  j ( a1 n i m ei t i d l ii ) ((pos "OA%" ) ))
( "animates"  v ( a1 n i m ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "animating"  v ( a1 n i m ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "animation"  n ( a2 n i m ei1 sh @ n ) ((pos "L@%" ) ))
( "animism"  n ( a1 n i m i z @ m ) ((pos "L@%" ) ))
( "animosities"  n ( a2 n i m o1 s i t i z ) ((pos "Mj%" ) ))
( "animosity"  n ( a2 n i m o1 s i t ii ) ((pos "M8%" ) ))
( "animus"  n ( a1 n i m @ s ) ((pos "L@%" ) ))
( "anise"  n ( a1 n i s ) ((pos "K6%" ) ))
( "aniseed"  n ( a1 n i s ii d ) ((pos "L@%" ) ))
( "anises"  n ( a1 n i s i z ) ((pos "Kj$" ) ))
( "ankle"  n ( a1 ng k l ) ((pos "K6%" ) ))
( "ankles"  n ( a1 ng k l z ) ((pos "Kj%" ) ))
( "anklet"  n ( a1 ng k l i t ) ((pos "K6%" ) ))
( "anklets"  n ( a1 ng k l i t s ) ((pos "Kj%" ) ))
( "anna"  n ( a1 n @ ) ((pos "K6$" ) ))
( "annalist"  n ( a1 n @ l i s t ) ((pos "K6$" ) ))
( "annalists"  n ( a1 n @ l i s t s ) ((pos "Kj$" ) ))
( "annals"  n ( a1 n l z ) ((pos "Kj%" ) ))
( "annas"  n ( a1 n @ z ) ((pos "Kj$" ) ))
( "anneal"  v ( @1 n ii1 l ) ((pos "H0$" ) (subcat )))
( "annealed"  v ( @1 n ii1 l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "annealing"  v ( @1 n ii1 l i ng ) ((pos "Hb$" ) (subcat )))
( "anneals"  v ( @1 n ii1 l z ) ((pos "Ha$" ) (subcat )))
( "annex"  n ( a1 n e k s ) ((pos "K7%" ) ))
( "annex"  v ( @1 n e1 k s ) ((pos "H1%" ) (subcat "6A" )))
( "annexation"  n ( a2 n e k s ei1 sh @ n ) ((pos "L@%" ) ))
( "annexe"  n ( a1 n e k s ) ((pos "K6%" ) ))
( "annexed"  v ( @1 n e1 k s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "annexes"  n ( a1 n e k s i z ) ((pos "Kj%" ) ))
( "annexes"  v ( @1 n e1 k s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "annexing"  v ( @1 n e1 k s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "annihilate"  v ( @1 n ai1 @ l ei t ) ((pos "H2%" ) (subcat )))
( "annihilated"  v ( @1 n ai1 @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "annihilates"  v ( @1 n ai1 @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "annihilating"  v ( @1 n ai1 @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "annihilation"  n ( @1 n ai2 @ l ei1 sh @ n ) ((pos "L@%" ) ))
( "anniversaries"  n ( a2 n i v @@1 s @ r i z ) ((pos "Kj%" ) ))
( "anniversary"  n ( a2 n i v @@1 s @ r ii ) ((pos "K8%" ) ))
( "annotate"  v ( a1 n @ t ei t ) ((pos "H2%" ) (subcat )))
( "annotated"  v ( a1 n @ t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "annotates"  v ( a1 n @ t ei t s ) ((pos "Ha%" ) (subcat )))
( "annotating"  v ( a1 n @ t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "annotation"  n ( a2 n @ t ei1 sh @ n ) ((pos "M6%" ) ))
( "annotations"  n ( a2 n @ t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "announce"  v ( @1 n au1 n s ) ((pos "H2%" ) (subcat "6A" "9" )))
( "announced"  v ( @1 n au1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "announcement"  n ( @1 n au1 n s m @ n t ) ((pos "K6%" ) ))
( "announcements"  n ( @1 n au1 n s m @ n t s ) ((pos "Kj%" ) ))
( "announcer"  n ( @1 n au1 n s @ r ) ((pos "K6%" ) ))
( "announcers"  n ( @1 n au1 n s @ z ) ((pos "Kj%" ) ))
( "announces"  v ( @1 n au1 n s i z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "announcing"  v ( @1 n au1 n s i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "annoy"  v ( @1 n oi1 ) ((pos "H0%" ) (subcat )))
( "annoyance"  n ( @1 n oi1 @ n s ) ((pos "M6%" ) ))
( "annoyances"  n ( @1 n oi1 @ n s i z ) ((pos "Mj%" ) ))
( "annoyed"  v ( @1 n oi1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "annoying"  v ( @1 n oi1 i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "annoys"  v ( @1 n oi1 z ) ((pos "Ha%" ) (subcat )))
( "annual"  n ( a1 n y u@ l ) ((pos "K6%" "OA%" ) ))
( "annually"  a ( a1 n y u@ l ii ) ((pos "Pu%" ) ))
( "annuals"  n ( a1 n y u@ l z ) ((pos "Kj%" ) ))
( "annuitant"  n ( @1 n y uu1 i t @ n t ) ((pos "K6$" ) ))
( "annuitants"  n ( @1 n y uu1 i t @ n t s ) ((pos "Kj$" ) ))
( "annuities"  n ( @1 n y uu1 i t i z ) ((pos "Kj%" ) ))
( "annuity"  n ( @1 n y uu1 i t ii ) ((pos "K8%" ) ))
( "annul"  v ( @1 n uh1 l ) ((pos "H4%" ) (subcat )))
( "annular"  j ( a1 n y u l @ r ) ((pos "OA$" ) ))
( "annulled"  v ( @1 n uh1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "annulling"  v ( @1 n uh1 l i ng ) ((pos "Hb%" ) (subcat )))
( "annulment"  n ( @1 n uh1 l m @ n t ) ((pos "K6%" ) ))
( "annulments"  n ( @1 n uh1 l m @ n t s ) ((pos "Kj%" ) ))
( "annuls"  v ( @1 n uh1 l z ) ((pos "Ha%" ) (subcat )))
( "annunciate"  v ( @1 n uh1 n s i ei t ) ((pos "H2$" ) (subcat )))
( "annunciated"  v ( @1 n uh1 n s i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "annunciates"  v ( @1 n uh1 n s i ei t s ) ((pos "Ha$" ) (subcat )))
( "annunciating"  v ( @1 n uh1 n s i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "annunciation"  n ( @1 n uh2 n s i ei1 sh @ n ) ((pos "K6%" ) ))
( "annunciations"  n ( @1 n uh2 n s i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "anode"  n ( a1 n ou d ) ((pos "K6%" ) ))
( "anodes"  n ( a1 n ou d z ) ((pos "Kj$" ) ))
( "anodyne"  n ( a1 n @ d ai n ) ((pos "M6%" "OA%" ) ))
( "anodynes"  n ( a1 n @ d ai n z ) ((pos "Mj$" ) ))
( "anoint"  v ( @1 n oi1 n t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "anointed"  v ( @1 n oi1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "anointing"  v ( @1 n oi1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "anointment"  n ( @1 n oi1 n t m @ n t ) ((pos "K6%" ) ))
( "anointments"  n ( @1 n oi1 n t m @ n t s ) ((pos "Kj$" ) ))
( "anoints"  v ( @1 n oi1 n t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "anomalies"  n ( @1 n o1 m @ l i z ) ((pos "Kj%" ) ))
( "anomalous"  j ( @1 n o1 m @ l @ s ) ((pos "OA%" ) ))
( "anomalously"  a ( @1 n o1 m @ l @ s l ii ) ((pos "Pu%" ) ))
( "anomaly"  n ( @1 n o1 m @ l ii ) ((pos "K8%" ) ))
( "anon"  a ( @1 n o1 n ) ((pos "Pu%" "Z-%" ) ))
( "anonymity"  n ( a2 n @ n i1 m i t ii ) ((pos "L@%" ) ))
( "anonymous"  j ( @1 n o1 n i m @ s ) ((pos "OA%" ) ))
( "anonymously"  a ( @1 n o1 n i m @ s l ii ) ((pos "Pu%" ) ))
( "anopheles"  n ( @1 n o1 f i l ii z ) ((pos "K9$" ) ))
( "anorak"  n ( a1 n @ r a k ) ((pos "K6%" ) ))
( "anoraks"  n ( a1 n @ r a k s ) ((pos "Kj%" ) ))
( "another"  j ( @1 n uh1 dh @ r ) ((pos "OA*" "Qx*" ) ))
( "answer"  v ( aa1 n s @ r ) ((pos "J0*" "K6*" ) (subcat "2A" "2C" "3A" "6A" "9" "12A" )))
( "answerable"  j ( a1 n s @ r @ b l ) ((pos "OA%" ) ))
( "answered"  v ( aa1 n s @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "9" "12A" )))
( "answering"  v ( aa1 n s @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "9" "12A" )))
( "answers"  v ( aa1 n s @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "9" "12A" )))
( "ant"  n ( a1 n t ) ((pos "K6%" ) ))
( "ant-eater"  n ( a1 n t - ii t @ r ) ((pos "K6%" ) ))
( "ant-eaters"  n ( a1 n t - ii t @ z ) ((pos "Kj%" ) ))
( "ant-hill"  n ( a1 n t - h i l ) ((pos "K6%" ) ))
( "ant-hills"  n ( a1 n t - h i l z ) ((pos "Kj%" ) ))
( "antagonism"  n ( a1 n t a1 g @ n i z @ m ) ((pos "M6%" ) ))
( "antagonisms"  n ( a1 n t a1 g @ n i z @ m z ) ((pos "Mj%" ) ))
( "antagonist"  n ( a1 n t a1 g @ n i s t ) ((pos "K6%" ) ))
( "antagonistic"  j ( a1 n t a2 g @ n i1 s t i k ) ((pos "OA%" ) ))
( "antagonistically"  a ( @1 n t a2 g @ n i1 s t i k l ii ) ((pos "Pu%" ) ))
( "antagonists"  n ( a1 n t a1 g @ n i s t s ) ((pos "Kj%" ) ))
( "antagonize"  v ( a2 n t a1 g @ n ai z ) ((pos "H2%" ) (subcat )))
( "antagonized"  v ( a2 n t a1 g @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "antagonizes"  v ( a2 n t a1 g @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "antagonizing"  v ( a2 n t a1 g @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "antarctic"  j ( a1 n t aa1 k t i k ) ((pos "OA%" ) ))
( "ante"  n ( a1 n t ii ) ((pos "K6%" "U-%" ) ))
( "ante meridiem"  a ( a2 n t i - m @ r i1 d i@ m ) ((pos "Pu%" ) ))
( "antecedence"  n ( a1 n t i s ii d n s ) ((pos "K6$" ) ))
( "antecedences"  n ( a1 n t i s ii d n s i z ) ((pos "Kj$" ) ))
( "antecedent"  n ( a2 n t i s ii1 d n t ) ((pos "K6%" "OA%" ) ))
( "antecedents"  n ( a2 n t i s ii1 d n t s ) ((pos "Kj%" ) ))
( "antechamber"  n ( a1 n t i ch ei m b @ r ) ((pos "K6%" ) ))
( "antechambers"  n ( a1 n t i ch ei m b @ z ) ((pos "Kj%" ) ))
( "antedate"  v ( a2 n t i d ei1 t ) ((pos "H2%" ) (subcat )))
( "antedated"  v ( a2 n t i d ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "antedates"  v ( a2 n t i d ei1 t s ) ((pos "Ha%" ) (subcat )))
( "antedating"  v ( a2 n t i d ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "antediluvian"  n ( a2 n t i d i l uu1 v i@ n ) ((pos "K6%" "OA%" ) ))
( "antediluvians"  n ( a2 n t i d i l uu1 v i@ n z ) ((pos "Kj%" ) ))
( "antelope"  n ( a1 n t i l ou p ) ((pos "K6%" ) ))
( "antelopes"  n ( a1 n t i l ou p s ) ((pos "Kj%" ) ))
( "antenatal"  j ( a2 n t i n ei1 t l ) ((pos "OA%" ) ))
( "antenna"  n ( a1 n t e1 n @ ) ((pos "Ki%" ) ))
( "antennae"  n ( a1 n t e1 n ii ) ((pos "Kj%" ) ))
( "antenuptial"  j ( a2 n t i n uh1 p sh l ) ((pos "OA$" ) ))
( "antepenultimate"  j ( a2 n t i p i n uh1 l t i m @ t ) ((pos "OA$" ) ))
( "anterior"  j ( a1 n t i@1 r i@ r ) ((pos "OA%" ) ))
( "anteroom"  n ( a1 n t i r u m ) ((pos "K6%" ) ))
( "anterooms"  n ( a1 n t i r u m z ) ((pos "Kj%" ) ))
( "antes"  n ( a1 n t i z ) ((pos "Kj%" ) ))
( "anthem"  n ( a1 n th @ m ) ((pos "K6%" ) ))
( "anthems"  n ( a1 n th @ m z ) ((pos "Kj%" ) ))
( "anther"  n ( a1 n th @ r ) ((pos "K6$" ) ))
( "anthers"  n ( a1 n th @ z ) ((pos "Kj$" ) ))
( "anthologies"  n ( a2 n th o1 l @ jh i z ) ((pos "Kj%" ) ))
( "anthology"  n ( a2 n th o1 l @ jh ii ) ((pos "K8%" ) ))
( "anthracite"  n ( a1 n th r @ s ai t ) ((pos "L@%" ) ))
( "anthrax"  n ( a1 n th r a k s ) ((pos "L@%" ) ))
( "anthropoid"  n ( a1 n th r @ p oi d ) ((pos "K6%" "OA%" ) ))
( "anthropoids"  n ( a1 n th r @ p oi d z ) ((pos "Kj%" ) ))
( "anthropological"  j ( a2 n th r @ p @ l o1 jh i k l ) ((pos "OA%" ) ))
( "anthropologist"  n ( a2 n th r @ p o1 l @ jh i s t ) ((pos "K6%" ) ))
( "anthropologists"  n ( a2 n th r @ p o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "anthropology"  n ( a2 n th r @ p o1 l @ jh ii ) ((pos "L@%" ) ))
( "anti-"  j ( a2 n t i - ) ((pos "U-%" ) ))
( "anti-Semite"  n ( a2 n t i - s ii1 m ai t ) ((pos "K6%" "OA%" ) ))
( "anti-Semites"  n ( a2 n t i - s ii1 m ai t s ) ((pos "Kj%" ) ))
( "anti-Semitic"  j ( a2 n t i - s i m i1 t i k ) ((pos "OA%" ) ))
( "anti-Semitism"  n ( a2 n t i - s e1 m i t i z @ m ) ((pos "L@%" ) ))
( "anti-aircraft"  j ( a2 n t i - e@1 k r aa f t ) ((pos "OA%" ) ))
( "anti-hero"  n ( a1 n t i - h i@ r ou ) ((pos "K7%" ) ))
( "anti-heroes"  n ( a1 n t i - h i@ r ou z ) ((pos "Kj%" ) ))
( "anti-personnel"  j ( a2 n t i - p @@2 s @ n e1 l ) ((pos "OA%" ) ))
( "antibiotic"  n ( a2 n t i b ai o1 t i k ) ((pos "K6%" "OA%" ) ))
( "antibiotics"  n ( a2 n t i b ai o1 t i k s ) ((pos "Kj%" ) ))
( "antibodies"  n ( a1 n t i b o d i z ) ((pos "Kj%" ) ))
( "antibody"  n ( a1 n t i b o d ii ) ((pos "K8%" ) ))
( "antic"  n ( a1 n t i k ) ((pos "K6%" ) ))
( "anticipate"  v ( a1 n t i1 s i p ei t ) ((pos "H2%" ) (subcat "6A" "6C" )))
( "anticipated"  v ( a1 n t i1 s i p ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "anticipates"  v ( a1 n t i1 s i p ei t s ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "anticipating"  v ( a1 n t i1 s i p ei t i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "anticipation"  n ( a1 n t i2 s i p ei1 sh @ n ) ((pos "M6%" ) ))
( "anticipations"  n ( a1 n t i2 s i p ei1 sh @ n z ) ((pos "Mj$" ) ))
( "anticipatory"  j ( a1 n t i2 s i p ei1 t @ r ii ) ((pos "OA%" ) ))
( "anticlimax"  n ( a2 n t i k l ai1 m a k s ) ((pos "K7%" ) ))
( "anticlimaxes"  n ( a2 n t i k l ai1 m a k s i z ) ((pos "Kj%" ) ))
( "anticlockwise"  a ( a2 n t i k l o1 k w ai z ) ((pos "Pu%" ) ))
( "antics"  n ( a1 n t i k s ) ((pos "Kj%" ) ))
( "anticyclone"  n ( a2 n t i s ai1 k l ou n ) ((pos "K6%" ) ))
( "anticyclones"  n ( a2 n t i s ai1 k l ou n z ) ((pos "Kj%" ) ))
( "antidote"  n ( a1 n t i d ou t ) ((pos "K6%" ) ))
( "antidotes"  n ( a1 n t i d ou t s ) ((pos "Kj%" ) ))
( "antifreeze"  n ( a1 n t i f r ii z ) ((pos "L@%" ) ))
( "antiknock"  n ( a2 n t i n o1 k ) ((pos "L@$" ) ))
( "antilogarithm"  n ( a2 n t i l o1 g @ r i dh @ m ) ((pos "K6%" ) ))
( "antilogarithms"  n ( a2 n t i l o1 g @ r i dh @ m z ) ((pos "Kj%" ) ))
( "antimacassar"  n ( a2 n t i m @ k a1 s @ r ) ((pos "K6$" ) ))
( "antimacassars"  n ( a2 n t i m @ k a1 s @ z ) ((pos "Kj$" ) ))
( "antimony"  n ( a1 n t i m @ n ii ) ((pos "L@$" ) ))
( "antipathetic"  j ( a2 n t i p @ th e1 t i k ) ((pos "OA%" ) ))
( "antipathies"  n ( a1 n t i1 p @ th i z ) ((pos "Mj%" ) ))
( "antipathy"  n ( a1 n t i1 p @ th ii ) ((pos "M8%" ) ))
( "antipodes"  n ( a1 n t i1 p @ d ii z ) ((pos "Kj%" ) ))
( "antiquarian"  n ( a2 n t i k w e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "antiquarians"  n ( a2 n t i k w e@1 r i@ n z ) ((pos "Kj%" ) ))
( "antiquaries"  n ( a1 n t i k w @ r i z ) ((pos "Kj%" ) ))
( "antiquary"  n ( a1 n t i k w @ r ii ) ((pos "K8%" ) ))
( "antiquated"  j ( a1 n t i k w ei t i d ) ((pos "OA%" ) ))
( "antique"  n ( a1 n t ii1 k ) ((pos "K6%" "OA%" ) ))
( "antiques"  n ( a1 n t ii1 k s ) ((pos "Kj%" ) ))
( "antiquities"  n ( a1 n t i1 k w i t i z ) ((pos "Mj%" ) ))
( "antiquity"  n ( a1 n t i1 k w i t ii ) ((pos "M8%" ) ))
( "antirrhinum"  n ( a2 n t i r ai1 n @ m ) ((pos "K6%" ) ))
( "antirrhinums"  n ( a2 n t i r ai1 n @ m z ) ((pos "Kj%" ) ))
( "antiseptic"  n ( a2 n t i s e1 p t i k ) ((pos "K6%" "OA%" ) ))
( "antiseptics"  n ( a2 n t i s e1 p t i k s ) ((pos "Kj%" ) ))
( "antisocial"  j ( a2 n t i s ou1 sh l ) ((pos "OA%" ) ))
( "antitank"  j ( a2 n t i t a1 ng k ) ((pos "Oq%" ) ))
( "antitheses"  n ( a1 n t i1 th @ s ii z ) ((pos "Kj%" ) ))
( "antithesis"  n ( a1 n t i1 th @ s i s ) ((pos "Mi%" ) ))
( "antithetic"  j ( a2 n t i th e1 t i k ) ((pos "OA$" ) ))
( "antithetical"  j ( a2 n t i th e1 t i k l ) ((pos "OA$" ) ))
( "antithetically"  a ( a2 n t i th e1 t i k l ii ) ((pos "Pu$" ) ))
( "antitoxin"  n ( a2 n t i t o1 k s i n ) ((pos "K6%" ) ))
( "antitoxins"  n ( a2 n t i t o1 k s i n z ) ((pos "Kj%" ) ))
( "antitrade"  n ( a2 n t i t r ei1 d ) ((pos "K6$" "OA$" ) ))
( "antitrades"  n ( a2 n t i t r ei1 d z ) ((pos "Kj$" ) ))
( "antler"  n ( a1 n t l @ r ) ((pos "K6%" ) ))
( "antlers"  n ( a1 n t l @ z ) ((pos "Kj%" ) ))
( "antonym"  n ( a1 n t @ n i m ) ((pos "K6%" ) ))
( "antonyms"  n ( a1 n t @ n i m z ) ((pos "Kj%" ) ))
( "ants"  n ( a1 n t s ) ((pos "Kj%" ) ))
( "anus"  n ( ei1 n @ s ) ((pos "K7%" ) ))
( "anuses"  n ( ei1 n @ s i z ) ((pos "Kj$" ) ))
( "anvil"  n ( a1 n v i l ) ((pos "K6%" ) ))
( "anvils"  n ( a1 n v i l z ) ((pos "Kj%" ) ))
( "anxieties"  n ( a1 ng z ai1 @ t i z ) ((pos "Mj%" ) ))
( "anxiety"  n ( a1 ng z ai1 @ t ii ) ((pos "M8%" ) ))
( "anxious"  j ( a1 ng k sh @ s ) ((pos "OA%" ) ))
( "anxiously"  a ( a1 ng k sh @ s l ii ) ((pos "Pu%" ) ))
( "any"  j ( e1 n ii ) ((pos "OA*" "Pu*" "Qx*" ) ))
( "anybody"  n ( e1 n i b o d ii ) ((pos "Ki%" "Qx%" ) ))
( "anyhow"  a ( e1 n i h au ) ((pos "Pu%" ) ))
( "anyone"  n ( e1 n i w uh n ) ((pos "Ki%" "Qx%" ) ))
( "anyplace"  a ( e1 n i p l ei s ) ((pos "Pu%" ) ))
( "anything"  n ( e1 n i th i ng ) ((pos "Ki*" "Qx*" ) ))
( "anyway"  a ( e1 n i w ei ) ((pos "Pu%" ) ))
( "anywhere"  a ( e1 n i w e@ r ) ((pos "Pu%" ) ))
( "aorta"  n ( ei1 oo1 t @ ) ((pos "K6%" ) ))
( "aortas"  n ( ei1 oo1 t @ z ) ((pos "Kj%" ) ))
( "apace"  a ( @1 p ei1 s ) ((pos "Pu%" ) ))
( "apache"  n ( @1 p a1 ch ii ) ((pos "K6%" ) ))
( "apaches"  n ( @1 p a1 ch i z ) ((pos "Kj%" ) ))
( "apanage"  n ( a1 p @ n i jh ) ((pos "L@$" ) ))
( "apart"  a ( @1 p aa1 t ) ((pos "Pu%" ) ))
( "apartheid"  n ( @1 p aa1 t ai t ) ((pos "L@%" ) ))
( "apartment"  n ( @1 p aa1 t m @ n t ) ((pos "K6%" ) ))
( "apartments"  n ( @1 p aa1 t m @ n t s ) ((pos "Kj%" ) ))
( "apathetic"  j ( a2 p @ th e1 t i k ) ((pos "OA%" ) ))
( "apathetically"  a ( a2 p @ th e1 t i k l ii ) ((pos "Pu%" ) ))
( "apathy"  n ( a1 p @ th ii ) ((pos "L@%" ) ))
( "ape"  v ( ei1 p ) ((pos "H2%" "K6%" ) ))
( "aped"  v ( ei1 p t ) ((pos "Hc%" "Hd%" ) ))
( "aperient"  n ( @1 p i@1 r i@ n t ) ((pos "K6$" "OA$" ) ))
( "aperients"  n ( @1 p i@1 r i@ n t s ) ((pos "Kj$" ) ))
( "aperitif"  n ( @1 p e1 r @ t i f ) ((pos "K6%" ) ))
( "aperitifs"  n ( @1 p e1 r @ t i f s ) ((pos "Kj%" ) ))
( "aperture"  n ( a1 p @ ch @ r ) ((pos "K6%" ) ))
( "apertures"  n ( a1 p @ ch @ z ) ((pos "Kj%" ) ))
( "apes"  v ( ei1 p s ) ((pos "Ha%" "Kj%" ) ))
( "apex"  n ( ei1 p e k s ) ((pos "K7%" ) ))
( "apexes"  n ( ei1 p e k s i z ) ((pos "Kj%" ) ))
( "aphasia"  n ( @1 f ei1 z i@ ) ((pos "L@%" ) ))
( "aphid"  n ( ei1 f i d ) ((pos "K6%" ) ))
( "aphides"  n ( ei1 f i d ii z ) ((pos "Kj$" ) ))
( "aphids"  n ( ei1 f i d z ) ((pos "Kj%" ) ))
( "aphis"  n ( ei1 f i s ) ((pos "Ki%" ) ))
( "aphorism"  n ( a1 f @ r i z @ m ) ((pos "K6%" ) ))
( "aphorisms"  n ( a1 f @ r i z @ m z ) ((pos "Kj%" ) ))
( "aphrodisiac"  n ( a2 f r @ d i1 z i a k ) ((pos "M6%" "OA%" ) ))
( "aphrodisiacs"  n ( a2 f r @ d i1 z i a k s ) ((pos "Mj%" ) ))
( "apiaries"  n ( ei1 p i@ r i z ) ((pos "Kj%" ) ))
( "apiarist"  n ( ei1 p i@ r i s t ) ((pos "K6%" ) ))
( "apiarists"  n ( ei1 p i@ r i s t s ) ((pos "Kj%" ) ))
( "apiary"  n ( ei1 p i@ r ii ) ((pos "K8%" ) ))
( "apices"  n ( ei1 p i s ii z ) ((pos "Kj$" ) ))
( "apiculture"  n ( ei1 p i k uh l ch @ r ) ((pos "K6%" ) ))
( "apicultures"  n ( ei1 p i k uh l ch @ z ) ((pos "Kj$" ) ))
( "apiece"  a ( @1 p ii1 s ) ((pos "Pu%" ) ))
( "aping"  v ( ei1 p i ng ) ((pos "Hb%" ) ))
( "apish"  j ( ei1 p i sh ) ((pos "OA%" ) ))
( "aplomb"  n ( @1 p l o1 m ) ((pos "L@%" ) ))
( "apocalypse"  n ( @1 p o1 k @ l i p s ) ((pos "K6%" ) ))
( "apocalypses"  n ( @1 p o1 k @ l i p s i z ) ((pos "Kj%" ) ))
( "apocalyptic"  j ( @1 p o2 k @ l i1 p t i k ) ((pos "OA%" ) ))
( "apocryphal"  j ( @1 p o1 k r i f @ l ) ((pos "OA%" ) ))
( "apogee"  n ( a1 p @ jh ii ) ((pos "K6%" ) ))
( "apogees"  n ( a1 p @ jh ii z ) ((pos "Kj$" ) ))
( "apologetic"  j ( @1 p o2 l @ jh e1 t i k ) ((pos "OA%" ) ))
( "apologetically"  a ( @1 p o2 l @ jh e1 t i k l ii ) ((pos "Pu%" ) ))
( "apologetics"  n ( @1 p o2 l @ jh e1 t i k s ) ((pos "Lk%" ) ))
( "apologies"  n ( @1 p o1 l @ jh i z ) ((pos "Kj%" ) ))
( "apologist"  n ( @1 p o1 l @ jh i s t ) ((pos "K6%" ) ))
( "apologists"  n ( @1 p o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "apologize"  v ( @1 p o1 l @ jh ai z ) ((pos "I2%" ) (subcat "2A" )))
( "apologized"  v ( @1 p o1 l @ jh ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "apologizes"  v ( @1 p o1 l @ jh ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "apologizing"  v ( @1 p o1 l @ jh ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "apology"  n ( @1 p o1 l @ jh ii ) ((pos "K8%" ) ))
( "apophthegm"  n ( a1 p @ th e m ) ((pos "K6%" ) ))
( "apophthegms"  n ( a1 p @ th e m z ) ((pos "Kj$" ) ))
( "apoplectic"  j ( a2 p @ p l e1 k t i k ) ((pos "OA%" ) ))
( "apoplexy"  n ( a1 p @ p l e k s ii ) ((pos "L@%" ) ))
( "apostasies"  n ( @1 p o1 s t @ s i z ) ((pos "Mj$" ) ))
( "apostasy"  n ( @1 p o1 s t @ s ii ) ((pos "M8%" ) ))
( "apostate"  n ( @1 p o1 s t ei t ) ((pos "K6%" "OA%" ) ))
( "apostates"  n ( @1 p o1 s t ei t s ) ((pos "Kj%" ) ))
( "apostle"  n ( @1 p o1 s l ) ((pos "K6%" ) ))
( "apostles"  n ( @1 p o1 s l z ) ((pos "Kj%" ) ))
( "apostolic"  j ( a2 p @ s t o1 l i k ) ((pos "OA%" ) ))
( "apostrophe"  n ( @1 p o1 s t r @ f ii ) ((pos "K6%" ) ))
( "apostrophes"  n ( @1 p o1 s t r @ f i z ) ((pos "Kj%" ) ))
( "apostrophize"  v ( @1 p o1 s t r @ f ai z ) ((pos "H2$" ) ))
( "apostrophized"  v ( @1 p o1 s t r @ f ai z d ) ((pos "Hc$" "Hd$" ) ))
( "apostrophizes"  v ( @1 p o1 s t r @ f ai z i z ) ((pos "Ha$" ) ))
( "apostrophizing"  v ( @1 p o1 s t r @ f ai z i ng ) ((pos "Hb$" ) ))
( "apothecaries"  n ( @1 p o1 th i k @ r i z ) ((pos "Kj%" ) ))
( "apothecary"  n ( @1 p o1 th i k @ r ii ) ((pos "K8%" ) ))
( "apothegm"  n ( a1 p @ th e m ) ((pos "K6$" ) ))
( "apothegms"  n ( a1 p @ th e m z ) ((pos "Kj$" ) ))
( "apotheoses"  n ( @1 p o2 th i ou1 s ii z ) ((pos "Kj$" ) ))
( "apotheosis"  n ( @1 p o2 th i ou1 s i s ) ((pos "Ki%" ) ))
( "appal"  v ( @1 p oo1 l ) ((pos "H4%" ) (subcat )))
( "appalled"  v ( @1 p oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "appalling"  v ( @1 p oo1 l i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "appallingly"  a ( @1 p oo1 l i ng l ii ) ((pos "Pu%" ) ))
( "appals"  v ( @1 p oo1 l z ) ((pos "Ha%" ) (subcat )))
( "appanage"  n ( a1 p @ n i jh ) ((pos "L@$" ) ))
( "apparatus"  n ( a2 p @ r ei1 t @ s ) ((pos "M7%" ) ))
( "apparatuses"  n ( a2 p @ r ei1 t @ s i z ) ((pos "Mj%" ) ))
( "apparel"  v ( @1 p a1 r @ l ) ((pos "H4$" "L@%" ) ))
( "apparelled"  v ( @1 p a1 r @ l d ) ((pos "Hc$" "Hd$" ) ))
( "apparelling"  v ( @1 p a1 r @ l i ng ) ((pos "Hb$" ) ))
( "apparels"  v ( @1 p a1 r @ l z ) ((pos "Ha$" ) ))
( "apparent"  j ( @1 p a1 r @ n t ) ((pos "OA%" ) ))
( "apparently"  a ( @1 p a1 r @ n t l ii ) ((pos "Pu%" ) ))
( "apparition"  n ( a2 p @ r i1 sh @ n ) ((pos "K6%" ) ))
( "apparitions"  n ( a2 p @ r i1 sh @ n z ) ((pos "Kj%" ) ))
( "appeal"  v ( @1 p ii1 l ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "appealed"  v ( @1 p ii1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "appealing"  v ( @1 p ii1 l i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "appealingly"  a ( @1 p ii1 l i ng l ii ) ((pos "Pu%" ) ))
( "appeals"  v ( @1 p ii1 l z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "appear"  v ( @1 p i@1 r ) ((pos "I0%" ) (subcat "2A" "2C" "4D" )))
( "appearance"  n ( @1 p i@1 r @ n s ) ((pos "K6%" ) ))
( "appearances"  n ( @1 p i@1 r @ n s i z ) ((pos "Kj%" ) ))
( "appeared"  v ( @1 p i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "4D" )))
( "appearing"  v ( @1 p i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "4D" )))
( "appears"  v ( @1 p i@1 z ) ((pos "Ia%" ) (subcat "2A" "2C" "4D" )))
( "appease"  v ( @1 p ii1 z ) ((pos "H2%" ) (subcat )))
( "appeased"  v ( @1 p ii1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "appeasement"  n ( @1 p ii1 z m @ n t ) ((pos "L@%" ) ))
( "appeases"  v ( @1 p ii1 z i z ) ((pos "Ha%" ) (subcat )))
( "appeasing"  v ( @1 p ii1 z i ng ) ((pos "Hb%" ) (subcat )))
( "appellant"  n ( @1 p e1 l @ n t ) ((pos "K6$" "OA$" ) ))
( "appellants"  n ( @1 p e1 l @ n t s ) ((pos "Kj$" ) ))
( "appellation"  n ( a2 p @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "appellations"  n ( a2 p @ l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "append"  v ( @1 p e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "appendage"  n ( @1 p e1 n d i jh ) ((pos "K6%" ) ))
( "appendages"  n ( @1 p e1 n d i jh i z ) ((pos "Kj%" ) ))
( "appendectomies"  n ( a2 p e n d e1 k t @ m i z ) ((pos "Kj%" ) ))
( "appendectomy"  n ( a2 p e n d e1 k t @ m ii ) ((pos "K8%" ) ))
( "appended"  v ( @1 p e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "appendices"  n ( @1 p e1 n d i s ii z ) ((pos "Kj%" ) ))
( "appendicitis"  n ( @1 p e2 n d i s ai1 t i s ) ((pos "L@%" ) ))
( "appending"  v ( @1 p e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "appendix"  n ( @1 p e1 n d i k s ) ((pos "K7%" ) ))
( "appendixes"  n ( @1 p e1 n d i k s i z ) ((pos "Kj%" ) ))
( "appends"  v ( @1 p e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "appertain"  v ( a2 p @ t ei1 n ) ((pos "I0%" ) (subcat )))
( "appertained"  v ( a2 p @ t ei1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "appertaining"  v ( a2 p @ t ei1 n i ng ) ((pos "Ib%" ) (subcat )))
( "appertains"  v ( a2 p @ t ei1 n z ) ((pos "Ia%" ) (subcat )))
( "appetite"  n ( a1 p i t ai t ) ((pos "M6%" ) ))
( "appetites"  n ( a1 p i t ai t s ) ((pos "Mj%" ) ))
( "appetizer"  n ( a1 p i t ai z @ r ) ((pos "K6%" ) ))
( "appetizers"  n ( a1 p i t ai z @ z ) ((pos "Kj%" ) ))
( "appetizing"  j ( a1 p i t ai z i ng ) ((pos "OA%" ) ))
( "applaud"  v ( @1 p l oo1 d ) ((pos "J0%" ) (subcat "2A" "2B" )))
( "applauded"  v ( @1 p l oo1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" )))
( "applauding"  v ( @1 p l oo1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2B" )))
( "applauds"  v ( @1 p l oo1 d z ) ((pos "Ja%" ) (subcat "2A" "2B" )))
( "applause"  n ( @1 p l oo1 z ) ((pos "L@%" ) ))
( "apple"  n ( a1 p l ) ((pos "M6%" ) ))
( "applejack"  n ( a1 p l jh a k ) ((pos "L@$" ) ))
( "apples"  n ( a1 p l z ) ((pos "Mj%" ) ))
( "appliance"  n ( @1 p l ai1 @ n s ) ((pos "K6%" ) ))
( "appliances"  n ( @1 p l ai1 @ n s i z ) ((pos "Kj%" ) ))
( "applicability"  n ( @1 p l i2 k @ b i1 l i t ii ) ((pos "L@%" ) ))
( "applicable"  j ( @1 p l i1 k @ b l ) ((pos "OA%" ) ))
( "applicant"  n ( a1 p l i k @ n t ) ((pos "K6%" ) ))
( "applicants"  n ( a1 p l i k @ n t s ) ((pos "Kj%" ) ))
( "application"  n ( a2 p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "applications"  n ( a2 p l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "applied"  v ( @1 p l ai1 d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "3A" "6A" )))
( "applies"  v ( @1 p l ai1 z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" )))
( "appliqu_e"  v ( a1 p l ii1 k ei ) ((pos "H0%" "L@%" ) ))
( "appliqu_eed"  v ( a1 p l ii1 k ei d ) ((pos "Hc%" "Hd%" ) ))
( "appliqu_eing"  v ( a1 p l ii1 k ei i ng ) ((pos "Hb%" ) ))
( "appliqu_es"  v ( a1 p l ii1 k ei z ) ((pos "Ha%" ) ))
( "apply"  v ( @1 p l ai1 ) ((pos "J3%" ) (subcat "2C" "3A" "6A" )))
( "applying"  v ( @1 p l ai1 i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "appoint"  v ( @1 p oi1 n t ) ((pos "H0%" ) (subcat "6A" "9" "14" "16A" "23" )))
( "appointed"  v ( @1 p oi1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" "16A" "23" )))
( "appointee"  n ( @1 p oi n t ii1 ) ((pos "K6%" ) ))
( "appointees"  n ( @1 p oi n t ii1 z ) ((pos "Kj%" ) ))
( "appointing"  v ( @1 p oi1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" "16A" "23" )))
( "appointment"  n ( @1 p oi1 n t m @ n t ) ((pos "M6%" ) ))
( "appointments"  n ( @1 p oi1 n t m @ n t s ) ((pos "Mj%" ) ))
( "appoints"  v ( @1 p oi1 n t s ) ((pos "Ha%" ) (subcat "6A" "9" "14" "16A" "23" )))
( "apportion"  v ( @1 p oo1 sh @ n ) ((pos "H0%" ) (subcat "6A" "12B" "13B" )))
( "apportioned"  v ( @1 p oo1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12B" "13B" )))
( "apportioning"  v ( @1 p oo1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" "12B" "13B" )))
( "apportionment"  n ( @1 p oo1 sh @ n m @ n t ) ((pos "L@%" ) ))
( "apportions"  v ( @1 p oo1 sh @ n z ) ((pos "Ha%" ) (subcat "6A" "12B" "13B" )))
( "apposite"  j ( a1 p @ z i t ) ((pos "OA%" ) ))
( "appositely"  a ( a1 p @ z i t l ii ) ((pos "Pu%" ) ))
( "apposition"  n ( a2 p @ z i1 sh @ n ) ((pos "L@$" ) ))
( "appraisal"  n ( @1 p r ei1 z l ) ((pos "K6%" ) ))
( "appraisals"  n ( @1 p r ei1 z l z ) ((pos "Kj$" ) ))
( "appraise"  v ( @1 p r ei1 z ) ((pos "H2%" ) (subcat )))
( "appraised"  v ( @1 p r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "appraises"  v ( @1 p r ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "appraising"  v ( @1 p r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "appreciable"  j ( @1 p r ii1 sh @ b l ) ((pos "OA%" ) ))
( "appreciably"  a ( @1 p r ii1 sh @ b l ii ) ((pos "Pu%" ) ))
( "appreciate"  v ( @1 p r ii1 sh i ei t ) ((pos "J2%" ) (subcat "2B" "6A" "9" )))
( "appreciated"  v ( @1 p r ii1 sh i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2B" "6A" "9" )))
( "appreciates"  v ( @1 p r ii1 sh i ei t s ) ((pos "Ja%" ) (subcat "2B" "6A" "9" )))
( "appreciating"  v ( @1 p r ii1 sh i ei t i ng ) ((pos "Jb%" ) (subcat "2B" "6A" "9" )))
( "appreciation"  n ( @1 p r ii2 sh i ei1 sh @ n ) ((pos "M6%" ) ))
( "appreciations"  n ( @1 p r ii2 sh i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "appreciative"  j ( @1 p r ii1 sh @ t i v ) ((pos "OA%" ) ))
( "appreciatively"  a ( @1 p r ii1 sh @ t i v l ii ) ((pos "Pu%" ) ))
( "apprehend"  v ( a2 p r i h e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "apprehended"  v ( a2 p r i h e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "apprehending"  v ( a2 p r i h e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "apprehends"  v ( a2 p r i h e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "apprehensible"  j ( a2 p r i h e1 n s @ b l ) ((pos "OA$" ) ))
( "apprehension"  n ( a2 p r i h e1 n sh @ n ) ((pos "M6%" ) ))
( "apprehensions"  n ( a2 p r i h e1 n sh @ n z ) ((pos "Mj$" ) ))
( "apprehensive"  j ( a2 p r i h e1 n s i v ) ((pos "OA%" ) ))
( "apprentice"  v ( @1 p r e1 n t i s ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "apprenticed"  v ( @1 p r e1 n t i s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "apprentices"  v ( @1 p r e1 n t i s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "apprenticeship"  n ( @1 p r e1 n t i sh i p ) ((pos "K6%" ) ))
( "apprenticeships"  n ( @1 p r e1 n t i sh i p s ) ((pos "Kj%" ) ))
( "apprenticing"  v ( @1 p r e1 n t i s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "apprise"  v ( @1 p r ai1 z ) ((pos "H2%" ) (subcat )))
( "apprised"  v ( @1 p r ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "apprises"  v ( @1 p r ai1 z i z ) ((pos "Ha$" ) (subcat )))
( "apprising"  v ( @1 p r ai1 z i ng ) ((pos "Hb$" ) (subcat )))
( "appro"  n ( a1 p r ou ) ((pos "L@$" ) ))
( "approach"  v ( @1 p r ou1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" )))
( "approachable"  j ( @1 p r ou1 ch @ b l ) ((pos "OA%" ) ))
( "approached"  v ( @1 p r ou1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "approaches"  v ( @1 p r ou1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "approaching"  v ( @1 p r ou1 ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "approbation"  n ( a2 p r @ b ei1 sh @ n ) ((pos "L@%" ) ))
( "appropriate"  j ( @1 p r ou1 p r i@ t ) ((pos "OA%" ) ))
( "appropriate"  v ( @1 p r ou1 p r i ei t ) ((pos "H2%" ) (subcat "6A" )))
( "appropriated"  v ( @1 p r ou1 p r i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "appropriately"  a ( @1 p r ou1 p r i@ t l ii ) ((pos "Pu%" ) ))
( "appropriates"  v ( @1 p r ou1 p r i ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "appropriating"  v ( @1 p r ou1 p r i ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "appropriation"  n ( @1 p r ou2 p r i ei1 sh @ n ) ((pos "M6%" ) ))
( "appropriations"  n ( @1 p r ou2 p r i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "approval"  n ( @1 p r uu1 v @ l ) ((pos "L@%" ) ))
( "approve"  v ( @1 p r uu1 v ) ((pos "J2%" ) (subcat "3A" )))
( "approved"  v ( @1 p r uu1 v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "approves"  v ( @1 p r uu1 v z ) ((pos "Ja%" ) (subcat "3A" )))
( "approving"  v ( @1 p r uu1 v i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "approvingly"  a ( @1 p r uu1 v i ng l ii ) ((pos "Pu%" ) ))
( "approx"  n ( @1 p r o1 k s ) ((pos "Y~%" ) ))
( "approximate"  j ( @1 p r o1 k s i m @ t ) ((pos "OA%" ) ))
( "approximate"  v ( @1 p r o1 k s i m ei t ) ((pos "J2%" ) (subcat "3A" )))
( "approximated"  v ( @1 p r o1 k s i m ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "approximately"  a ( @1 p r o1 k s i m @ t l ii ) ((pos "Pu%" ) ))
( "approximates"  v ( @1 p r o1 k s i m ei t s ) ((pos "Ja%" ) (subcat "3A" )))
( "approximating"  v ( @1 p r o1 k s i m ei t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "approximation"  n ( @1 p r o2 k s i m ei1 sh @ n ) ((pos "M6%" ) ))
( "approximations"  n ( @1 p r o2 k s i m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "appurtenance"  n ( @1 p @@1 t i n @ n s ) ((pos "K6%" ) ))
( "appurtenances"  n ( @1 p @@1 t i n @ n s i z ) ((pos "Kj%" ) ))
( "apr`es-ski"  j ( a2 p r ei - s k ii1 ) ((pos "Oq%" ) ))
( "apricot"  n ( ei1 p r i k o t ) ((pos "K6%" ) ))
( "apricots"  n ( ei1 p r i k o t s ) ((pos "Kj%" ) ))
( "apron"  n ( ei1 p r @ n ) ((pos "K6%" ) ))
( "aprons"  n ( ei1 p r @ n z ) ((pos "Kj%" ) ))
( "apropos"  j ( a2 p r @ p ou1 ) ((pos "Op%" "Pu%" ) ))
( "apse"  n ( a1 p s ) ((pos "K6$" ) ))
( "apses"  n ( a1 p s i z ) ((pos "Kj$" ) ))
( "apt"  j ( a1 p t ) ((pos "OC%" ) ))
( "apter"  j ( a1 p t @ r ) ((pos "Or$" ) ))
( "aptest"  j ( a1 p t i s t ) ((pos "Os$" ) ))
( "aptitude"  n ( a1 p t i t y uu d ) ((pos "M6%" ) ))
( "aptitudes"  n ( a1 p t i t y uu d z ) ((pos "Mj%" ) ))
( "aptly"  a ( a1 p t l ii ) ((pos "Pu%" ) ))
( "aptness"  n ( a1 p t n @ s ) ((pos "L@%" ) ))
( "aqualung"  n ( a1 k w @ l uh ng ) ((pos "K6%" ) ))
( "aqualungs"  n ( a1 k w @ l uh ng z ) ((pos "Kj%" ) ))
( "aquamarine"  n ( a2 k w @ m @ r ii1 n ) ((pos "M6%" ) ))
( "aquamarines"  n ( a2 k w @ m @ r ii1 n z ) ((pos "Mj%" ) ))
( "aquanaut"  n ( a1 k w @ n oo t ) ((pos "K6$" ) ))
( "aquanauts"  n ( a1 k w @ n oo t s ) ((pos "Kj$" ) ))
( "aquaplane"  v ( a1 k w @ p l ei n ) ((pos "I2%" "K6%" ) ))
( "aquaplaned"  v ( a1 k w @ p l ei n d ) ((pos "Ic%" "Id%" ) ))
( "aquaplanes"  v ( a1 k w @ p l ei n z ) ((pos "Ia%" "Kj%" ) ))
( "aquaplaning"  v ( a1 k w @ p l ei n i ng ) ((pos "Ib%" ) ))
( "aquaria"  n ( @1 k w e@1 r i@ ) ((pos "Kj$" ) ))
( "aquarium"  n ( @1 k w e@1 r i@ m ) ((pos "K6%" ) ))
( "aquariums"  n ( @1 k w e@1 r i@ m z ) ((pos "Kj%" ) ))
( "aquatic"  j ( @1 k w a1 t i k ) ((pos "OA%" ) ))
( "aquatint"  n ( a1 k w @ t i n t ) ((pos "M6%" ) ))
( "aquatints"  n ( a1 k w @ t i n t s ) ((pos "Mj%" ) ))
( "aquavit"  n ( a1 k w @ v i t ) ((pos "L@$" ) ))
( "aqueduct"  n ( a1 k w i d uh k t ) ((pos "K6%" ) ))
( "aqueducts"  n ( a1 k w i d uh k t s ) ((pos "Kj%" ) ))
( "aqueous"  j ( ei1 k w i@ s ) ((pos "OA$" ) ))
( "aquiline"  j ( a1 k w i l ai n ) ((pos "OA%" ) ))
( "ar^ete"  n ( a1 r ei1 t ) ((pos "K6$" ) ))
( "ar^etes"  n ( a1 r ei1 t s ) ((pos "Kj$" ) ))
( "arabesque"  n ( a2 r @ b e1 s k ) ((pos "K6%" ) ))
( "arabesques"  n ( a2 r @ b e1 s k s ) ((pos "Kj%" ) ))
( "arable"  j ( a1 r @ b l ) ((pos "OA%" ) ))
( "arachnid"  n ( @1 r a1 k n i d ) ((pos "K6%" ) ))
( "arachnids"  n ( @1 r a1 k n i d z ) ((pos "Kj%" ) ))
( "arbiter"  n ( aa1 b i t @ r ) ((pos "K6%" ) ))
( "arbiters"  n ( aa1 b i t @ z ) ((pos "Kj%" ) ))
( "arbitrament"  n ( aa1 b i1 t r @ m @ n t ) ((pos "M6$" ) ))
( "arbitraments"  n ( aa1 b i1 t r @ m @ n t s ) ((pos "Mj$" ) ))
( "arbitrarily"  a ( aa2 b i t r e@1 r i l ii ) ((pos "Pu%" ) ))
( "arbitrary"  j ( aa1 b i t r @ r ii ) ((pos "OA%" ) ))
( "arbitrate"  v ( aa1 b i t r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "arbitrated"  v ( aa1 b i t r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "arbitrates"  v ( aa1 b i t r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "arbitrating"  v ( aa1 b i t r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "arbitration"  n ( aa2 b i t r ei1 sh @ n ) ((pos "M6%" ) ))
( "arbitrations"  n ( aa2 b i t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "arbitrator"  n ( aa1 b i t r ei t @ r ) ((pos "K6%" ) ))
( "arbitrators"  n ( aa1 b i t r ei t @ z ) ((pos "Kj%" ) ))
( "arboreal"  j ( aa1 b oo1 r i@ l ) ((pos "OA$" ) ))
( "arbour"  n ( aa1 b @ r ) ((pos "K6%" ) ))
( "arbours"  n ( aa1 b @ z ) ((pos "Kj%" ) ))
( "arc"  n ( aa1 k ) ((pos "K6%" ) ))
( "arc-lamp"  n ( aa1 k - l a m p ) ((pos "K6%" ) ))
( "arc-lamps"  n ( aa1 k - l a m p s ) ((pos "Kj%" ) ))
( "arc-light"  n ( aa1 k - l ai t ) ((pos "K6%" ) ))
( "arc-lights"  n ( aa1 k - l ai t s ) ((pos "Kj%" ) ))
( "arcade"  n ( aa1 k ei1 d ) ((pos "K6%" ) ))
( "arcades"  n ( aa1 k ei1 d z ) ((pos "Kj%" ) ))
( "arcane"  j ( aa1 k ei1 n ) ((pos "OA%" ) ))
( "arch"  v ( aa1 ch ) ((pos "J1%" "K7%" "Oq%" ) (subcat "2C" )))
( "arch-"  j ( aa ch - ) ((pos "U-%" ) ))
( "archaeological"  j ( aa2 k i@ l o1 jh i k l ) ((pos "OA%" ) ))
( "archaeologist"  n ( aa2 k i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "archaeologists"  n ( aa2 k i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "archaeology"  n ( aa2 k i o1 l @ jh ii ) ((pos "L@%" ) ))
( "archaic"  j ( aa1 k ei1 i k ) ((pos "OA%" ) ))
( "archaism"  n ( aa1 k ei i z @ m ) ((pos "M6%" ) ))
( "archaisms"  n ( aa1 k ei i z @ m z ) ((pos "Mj%" ) ))
( "archangel"  n ( aa1 k ei n jh l ) ((pos "K6%" ) ))
( "archangels"  n ( aa1 k ei n jh l z ) ((pos "Kj%" ) ))
( "archbishop"  n ( aa2 ch b i1 sh @ p ) ((pos "K6%" ) ))
( "archbishopric"  n ( aa1 ch b i1 sh @ p r i k ) ((pos "K6%" ) ))
( "archbishoprics"  n ( aa1 ch b i1 sh @ p r i k s ) ((pos "Kj$" ) ))
( "archbishops"  n ( aa2 ch b i1 sh @ p s ) ((pos "Kj%" ) ))
( "archdeacon"  n ( aa2 ch d ii1 k @ n ) ((pos "K6%" ) ))
( "archdeaconries"  n ( aa1 ch d ii1 k @ n r i z ) ((pos "Kj$" ) ))
( "archdeaconry"  n ( aa1 ch d ii1 k @ n r ii ) ((pos "K8%" ) ))
( "archdeacons"  n ( aa2 ch d ii1 k @ n z ) ((pos "Kj%" ) ))
( "archdiocese"  n ( aa2 ch d ai1 @ s i s ) ((pos "K6%" ) ))
( "archdioceses"  n ( aa2 ch d ai1 @ s i s i z ) ((pos "Kj$" ) ))
( "archduke"  n ( aa2 ch d y uu1 k ) ((pos "K6%" ) ))
( "archdukes"  n ( aa2 ch d y uu1 k s ) ((pos "Kj%" ) ))
( "arched"  v ( aa1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "archeology"  n ( aa2 k i o1 l @ jh ii ) ((pos "L@%" ) ))
( "archer"  n ( aa1 ch @ r ) ((pos "K6%" ) ))
( "archers"  n ( aa1 ch @ z ) ((pos "Kj%" ) ))
( "archery"  n ( aa1 ch @ r ii ) ((pos "L@%" ) ))
( "arches"  v ( aa1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "archetypal"  j ( aa2 k i t ai1 p l ) ((pos "OA%" ) ))
( "archetype"  n ( aa1 k i t ai p ) ((pos "K6%" ) ))
( "archetypes"  n ( aa1 k i t ai p s ) ((pos "Kj%" ) ))
( "archimandrite"  n ( aa2 k i m a1 n d r ai t ) ((pos "K6$" ) ))
( "archimandrites"  n ( aa2 k i m a1 n d r ai t s ) ((pos "Kj$" ) ))
( "arching"  v ( aa1 ch i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "archipelago"  n ( aa2 k i p e1 l @ g ou ) ((pos "K6%" ) ))
( "archipelagoes"  n ( aa2 k i p e1 l @ g ou z ) ((pos "Kj%" ) ))
( "archipelagos"  n ( aa2 k i p e1 l @ g ou z ) ((pos "Kj%" ) ))
( "architect"  n ( aa1 k i t e k t ) ((pos "K6%" ) ))
( "architects"  n ( aa1 k i t e k t s ) ((pos "Kj%" ) ))
( "architectural"  j ( aa2 k i t e1 k ch @ r @ l ) ((pos "OA%" ) ))
( "architecturally"  a ( aa2 k i t e1 k ch @ r @ l ii ) ((pos "Pu%" ) ))
( "architecture"  n ( aa1 k i t e k ch @ r ) ((pos "L@%" ) ))
( "archives"  n ( aa1 k ai v z ) ((pos "Kj%" ) ))
( "archivist"  n ( aa1 k i v i s t ) ((pos "K6%" ) ))
( "archivists"  n ( aa1 k i v i s t s ) ((pos "Kj%" ) ))
( "archly"  a ( aa1 ch l ii ) ((pos "Pu%" ) ))
( "archway"  n ( aa1 ch w ei ) ((pos "K6%" ) ))
( "archways"  n ( aa1 ch w ei z ) ((pos "Kj%" ) ))
( "arcs"  n ( aa1 k s ) ((pos "Kj%" ) ))
( "arctic"  j ( aa1 k t i k ) ((pos "OA%" ) ))
( "ardent"  j ( aa1 d n t ) ((pos "OA%" ) ))
( "ardently"  a ( aa1 d n t l ii ) ((pos "Pu%" ) ))
( "ardour"  n ( aa1 d @ r ) ((pos "M6%" ) ))
( "ardours"  n ( aa1 d @ z ) ((pos "Mj$" ) ))
( "arduous"  j ( aa1 d y u@ s ) ((pos "OA%" ) ))
( "arduously"  a ( aa1 d y u@ s l ii ) ((pos "Pu%" ) ))
( "are"  v ( aa r ) ((pos "Ge*" "Ie*" "K6$" ) (subcat )))
( "area"  n ( e@1 r i@ ) ((pos "M6*" ) ))
( "areas"  n ( e@1 r i@ z ) ((pos "Mj%" ) ))
( "areca"  n ( a1 r i k @ ) ((pos "K6$" ) ))
( "arecas"  n ( a1 r i k @ z ) ((pos "Kj$" ) ))
( "aren't"  v ( aa n t ) ((pos "Gg%" ) ))
( "arena"  n ( @1 r ii1 n @ ) ((pos "K6%" ) ))
( "arenas"  n ( @1 r ii1 n @ z ) ((pos "Kj%" ) ))
( "ares"  n ( aa1 z ) ((pos "Kj$" ) ))
( "argent"  n ( aa1 jh @ n t ) ((pos "K6$" "OA$" ) ))
( "argents"  n ( aa1 jh @ n t s ) ((pos "Kj$" ) ))
( "argon"  n ( aa1 g o n ) ((pos "L@%" ) ))
( "argosies"  n ( aa1 g @ s i z ) ((pos "Kj$" ) ))
( "argosy"  n ( aa1 g @ s ii ) ((pos "K8$" ) ))
( "argot"  n ( aa1 g ou ) ((pos "L@$" ) ))
( "arguable"  j ( aa1 g y u@ b l ) ((pos "OA%" ) ))
( "arguably"  a ( aa1 g y u@ b l ii ) ((pos "Pu%" ) ))
( "argue"  v ( aa1 g y uu ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "6A" "9" )))
( "argued"  v ( aa1 g y uu d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "9" )))
( "argues"  v ( aa1 g y uu z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "9" )))
( "arguing"  v ( aa1 g y uu i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "9" )))
( "argument"  n ( aa1 g y u m @ n t ) ((pos "M6%" ) ))
( "argumentation"  n ( aa2 g y u m e n t ei1 sh @ n ) ((pos "L@$" ) ))
( "argumentative"  j ( aa2 g y u m e1 n t @ t i v ) ((pos "OA%" ) ))
( "arguments"  n ( aa1 g y u m @ n t s ) ((pos "Mj%" ) ))
( "aria"  n ( aa1 r i@ ) ((pos "K6%" ) ))
( "arias"  n ( aa1 r i@ z ) ((pos "Kj%" ) ))
( "arid"  j ( a1 r i d ) ((pos "OA%" ) ))
( "aridity"  n ( @1 r i1 d i t ii ) ((pos "L@%" ) ))
( "aright"  a ( @1 r ai1 t ) ((pos "Pu%" ) ))
( "arise"  v ( @1 r ai1 z ) ((pos "I5%" ) (subcat "2A" )))
( "arisen"  v ( @1 r i1 z n ) ((pos "Id%" ) (subcat "2A" )))
( "arises"  v ( @1 r ai1 z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "arising"  v ( @1 r ai1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "aristocracies"  n ( a2 r i s t o1 k r @ s i z ) ((pos "Mj%" ) ))
( "aristocracy"  n ( a2 r i s t o1 k r @ s ii ) ((pos "M8%" ) ))
( "aristocrat"  n ( a1 r i s t @ k r a t ) ((pos "K6%" ) ))
( "aristocratic"  j ( a2 r i s t @ k r a1 t i k ) ((pos "OA%" ) ))
( "aristocratically"  a ( a2 r i s t @ k r a1 t i k l ii ) ((pos "Pu%" ) ))
( "aristocrats"  n ( a1 r i s t @ k r a t s ) ((pos "Kj%" ) ))
( "arithmetic"  n ( @1 r i1 th m @ t i k ) ((pos "L@%" ) ))
( "arithmetical"  j ( a2 r i th m e1 t i k l ) ((pos "OA%" ) ))
( "arithmetically"  a ( a2 r i th m e1 t i k l ii ) ((pos "Pu%" ) ))
( "arithmetician"  n ( @1 r i2 th m @ t i1 sh @ n ) ((pos "K6$" ) ))
( "arithmeticians"  n ( @1 r i2 th m @ t i1 sh @ n z ) ((pos "Kj$" ) ))
( "ark"  n ( aa1 k ) ((pos "K6%" ) ))
( "arks"  n ( aa1 k s ) ((pos "Kj%" ) ))
( "arm"  v ( aa1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "arm-hole"  n ( aa1 m - h ou l ) ((pos "K6%" ) ))
( "arm-holes"  n ( aa1 m - h ou l z ) ((pos "Kj%" ) ))
( "armada"  n ( aa1 m aa1 d @ ) ((pos "K6%" ) ))
( "armadas"  n ( aa1 m aa1 d @ z ) ((pos "Kj%" ) ))
( "armadillo"  n ( aa2 m @ d i1 l ou ) ((pos "K6%" ) ))
( "armadillos"  n ( aa2 m @ d i1 l ou z ) ((pos "Kj%" ) ))
( "armament"  n ( aa1 m @ m @ n t ) ((pos "M6%" ) ))
( "armaments"  n ( aa1 m @ m @ n t s ) ((pos "Mj%" ) ))
( "armature"  n ( aa1 m @ ch u@ r ) ((pos "K6$" ) ))
( "armatures"  n ( aa1 m @ ch u@ z ) ((pos "Kj$" ) ))
( "armband"  n ( aa1 m b a n d ) ((pos "K6%" ) ))
( "armbands"  n ( aa1 m b a n d z ) ((pos "Kj%" ) ))
( "armchair"  n ( aa1 m ch e@ r ) ((pos "K6%" ) ))
( "armchairs"  n ( aa1 m ch e@ z ) ((pos "Kj%" ) ))
( "armed"  v ( aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "armful"  n ( aa1 m f u l ) ((pos "K6%" ) ))
( "armfuls"  n ( aa1 m f u l z ) ((pos "Kj%" ) ))
( "armies"  n ( aa1 m i z ) ((pos "Kj%" ) ))
( "arming"  v ( aa1 m i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "armistice"  n ( aa1 m i s t i s ) ((pos "K6%" ) ))
( "armistices"  n ( aa1 m i s t i s i z ) ((pos "Kj%" ) ))
( "armlet"  n ( aa1 m l i t ) ((pos "K6%" ) ))
( "armlets"  n ( aa1 m l i t s ) ((pos "Kj%" ) ))
( "armoire"  n ( aa1 m w aa r ) ((pos "K6$" ) ))
( "armoires"  n ( aa1 m w aa z ) ((pos "Kj$" ) ))
( "armorial"  j ( aa1 m oo1 r i@ l ) ((pos "OA%" ) ))
( "armour"  n ( aa1 m @ r ) ((pos "L@%" ) ))
( "armour-plate"  n ( aa1 m @ - p l ei t ) ((pos "K6%" ) ))
( "armour-plates"  n ( aa1 m @ - p l ei t s ) ((pos "Kj$" ) ))
( "armoured"  j ( aa1 m @ d ) ((pos "OA%" ) ))
( "armourer"  n ( aa1 m @ r @ r ) ((pos "K6%" ) ))
( "armourers"  n ( aa1 m @ r @ z ) ((pos "Kj%" ) ))
( "armouries"  n ( aa1 m @ r i z ) ((pos "Kj%" ) ))
( "armoury"  n ( aa1 m @ r ii ) ((pos "K8%" ) ))
( "armpit"  n ( aa1 m p i t ) ((pos "K6%" ) ))
( "armpits"  n ( aa1 m p i t s ) ((pos "Kj%" ) ))
( "arms"  v ( aa1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "arms-race"  n ( aa1 m z - r ei s ) ((pos "Ki%" ) ))
( "arms-runner"  n ( aa1 m z - r uh n @ r ) ((pos "K6%" ) ))
( "arms-runners"  n ( aa1 m z - r uh n @ z ) ((pos "Kj%" ) ))
( "army"  n ( aa1 m ii ) ((pos "K8%" ) ))
( "arnica"  n ( aa1 n i k @ ) ((pos "L@$" ) ))
( "aroma"  n ( @1 r ou1 m @ ) ((pos "K6%" ) ))
( "aromas"  n ( @1 r ou1 m @ z ) ((pos "Kj%" ) ))
( "aromatic"  j ( a2 r @ m a1 t i k ) ((pos "OA%" ) ))
( "arose"  v ( @1 r ou1 z ) ((pos "Ic%" ) (subcat "2A" )))
( "around"  a ( @1 r au1 n d ) ((pos "Pu*" "T-*" ) ))
( "arouse"  v ( @1 r au1 z ) ((pos "H2%" ) (subcat "6A" )))
( "aroused"  v ( @1 r au1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "arouses"  v ( @1 r au1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "arousing"  v ( @1 r au1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "arpeggio"  n ( aa1 p e1 jh i@ @ ) ((pos "K6%" ) ))
( "arpeggios"  n ( aa1 p e1 jh i@ u z ) ((pos "Kj%" ) ))
( "arquebus"  n ( aa1 k w i b @ s ) ((pos "K7$" ) ))
( "arquebuses"  n ( aa1 k w i b @ s i z ) ((pos "Kj$" ) ))
( "arr"  n ( @1 r ai1 v ) ((pos "Y~%" ) ))
( "arrack"  n ( a1 r a k ) ((pos "L@$" ) ))
( "arraign"  v ( @1 r ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "arraigned"  v ( @1 r ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "arraigning"  v ( @1 r ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "arraignment"  n ( @1 r ei1 n m @ n t ) ((pos "K6$" ) ))
( "arraignments"  n ( @1 r ei1 n m @ n t s ) ((pos "Kj$" ) ))
( "arraigns"  v ( @1 r ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "arrange"  v ( @1 r ei1 n jh ) ((pos "J2%" ) (subcat "3A" "4C" "6A" "14" )))
( "arranged"  v ( @1 r ei1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "4C" "6A" "14" )))
( "arrangement"  n ( @1 r ei1 n jh m @ n t ) ((pos "M6%" ) ))
( "arrangements"  n ( @1 r ei1 n jh m @ n t s ) ((pos "Mj%" ) ))
( "arranges"  v ( @1 r ei1 n jh i z ) ((pos "Ja%" ) (subcat "3A" "4C" "6A" "14" )))
( "arranging"  v ( @1 r ei1 n jh i ng ) ((pos "Jb%" ) (subcat "3A" "4C" "6A" "14" )))
( "arrant"  j ( a1 r @ n t ) ((pos "OA%" ) ))
( "arras"  n ( a1 r @ s ) ((pos "K7$" ) ))
( "arrases"  n ( a1 r @ s i z ) ((pos "Kj$" ) ))
( "array"  v ( @1 r ei1 ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "arrayed"  v ( @1 r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "arraying"  v ( @1 r ei1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "arrays"  v ( @1 r ei1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "arrears"  n ( @1 r i@1 z ) ((pos "Kj%" ) ))
( "arrest"  v ( @1 r e1 s t ) ((pos "H0%" "K6%" ) (subcat )))
( "arrested"  v ( @1 r e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "arrester"  n ( @1 r e1 s t @ r ) ((pos "K6%" ) ))
( "arresters"  n ( @1 r e1 s t @ z ) ((pos "Kj%" ) ))
( "arresting"  v ( @1 r e1 s t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "arrests"  v ( @1 r e1 s t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "arri`ere pens_ee"  n ( a2 r i e@ - p o1 n s ei ) ((pos "K6$" ) ))
( "arri`ere pens_ees"  n ( a2 r i e@ - p o1 n s ei z ) ((pos "Kj$" ) ))
( "arrival"  n ( @1 r ai1 v @ l ) ((pos "M6%" ) ))
( "arrivals"  n ( @1 r ai1 v @ l z ) ((pos "Mj%" ) ))
( "arrive"  v ( @1 r ai1 v ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "arrived"  v ( @1 r ai1 v d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "arrives"  v ( @1 r ai1 v z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "arriving"  v ( @1 r ai1 v i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "arrogance"  n ( a1 r @ g @ n s ) ((pos "L@%" ) ))
( "arrogant"  j ( a1 r @ g @ n t ) ((pos "OA%" ) ))
( "arrogantly"  a ( a1 r @ g @ n t l ii ) ((pos "Pu%" ) ))
( "arrogate"  v ( a1 r @ g ei t ) ((pos "H2$" ) (subcat )))
( "arrogated"  v ( a1 r @ g ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "arrogates"  v ( a1 r @ g ei t s ) ((pos "Ha$" ) (subcat )))
( "arrogating"  v ( a1 r @ g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "arrow"  n ( a1 r ou ) ((pos "K6%" ) ))
( "arrowhead"  n ( a1 r ou h e d ) ((pos "K6%" ) ))
( "arrowheads"  n ( a1 r ou h e d z ) ((pos "Kj%" ) ))
( "arrowroot"  n ( a1 r ou r uu t ) ((pos "L@%" ) ))
( "arrows"  n ( a1 r ou z ) ((pos "Kj%" ) ))
( "arse"  n ( aa1 s ) ((pos "K6%" ) ))
( "arsehole"  n ( aa1 s h ou l ) ((pos "K6%" ) ))
( "arseholes"  n ( aa1 s h ou l z ) ((pos "Kj%" ) ))
( "arsenal"  n ( aa1 s n @ l ) ((pos "K6%" ) ))
( "arsenals"  n ( aa1 s n @ l z ) ((pos "Kj%" ) ))
( "arsenic"  n ( aa1 s n i k ) ((pos "L@%" ) ))
( "arses"  n ( aa1 s i z ) ((pos "Kj%" ) ))
( "arson"  n ( aa1 s n ) ((pos "L@%" ) ))
( "art"  n ( aa1 t ) ((pos "M6%" ) ))
( "artefact"  n ( aa1 t i f a k t ) ((pos "K6%" ) ))
( "artefacts"  n ( aa1 t i f a k t s ) ((pos "Kj%" ) ))
( "arterial"  j ( aa1 t i@1 r i@ l ) ((pos "OA%" ) ))
( "arteries"  n ( aa1 t @ r i z ) ((pos "Kj%" ) ))
( "arteriosclerosis"  n ( aa1 t i@2 r i@ u s k l @ r ou1 s i s ) ((pos "L@%" ) ))
( "artery"  n ( aa1 t @ r ii ) ((pos "K8%" ) ))
( "artesian"  j ( aa1 t ii1 z i@ n ) ((pos "OA%" ) ))
( "artful"  j ( aa1 t f @ l ) ((pos "OA%" ) ))
( "artfully"  a ( aa1 t f @ l ii ) ((pos "Pu%" ) ))
( "artfulness"  n ( aa1 t f @ l n @ s ) ((pos "L@%" ) ))
( "arthritic"  j ( aa1 th r i1 t i k ) ((pos "OA%" ) ))
( "arthritis"  n ( aa2 th r ai1 t i s ) ((pos "L@%" ) ))
( "artichoke"  n ( aa1 t i ch ou k ) ((pos "K6%" ) ))
( "artichokes"  n ( aa1 t i ch ou k s ) ((pos "Kj%" ) ))
( "article"  v ( aa1 t i k l ) ((pos "H2%" "K6%" ) ))
( "articled"  v ( aa1 t i k l d ) ((pos "Hc%" "Hd%" ) ))
( "articles"  v ( aa1 t i k l z ) ((pos "Ha$" "Kj%" ) ))
( "articling"  v ( aa1 t i k l i ng ) ((pos "Hb$" ) ))
( "articulate"  j ( aa1 t i1 k y u l @ t ) ((pos "OA%" ) ))
( "articulate"  v ( aa1 t i1 k y u l ei t ) ((pos "J2%" ) (subcat "2C" "6A" )))
( "articulated"  v ( aa1 t i1 k y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "articulately"  a ( aa1 t i1 k y u l @ t l ii ) ((pos "Pu%" ) ))
( "articulates"  v ( aa1 t i1 k y u l ei t s ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "articulating"  v ( aa1 t i1 k y u l ei t i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "articulation"  n ( aa1 t i2 k y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "artifact"  n ( aa1 t i f a k t ) ((pos "K6%" ) ))
( "artifacts"  n ( aa1 t i f a k t s ) ((pos "Kj%" ) ))
( "artifice"  n ( aa1 t i f i s ) ((pos "M6%" ) ))
( "artificer"  n ( aa1 t i1 f i s @ r ) ((pos "K6$" ) ))
( "artificers"  n ( aa1 t i1 f i s @ z ) ((pos "Kj$" ) ))
( "artifices"  n ( aa1 t i f i s i z ) ((pos "Mj$" ) ))
( "artificial"  j ( aa2 t i f i1 sh l ) ((pos "OA%" ) ))
( "artificially"  a ( aa2 t i f i1 sh @ l ii ) ((pos "Pu%" ) ))
( "artillery"  n ( aa1 t i1 l @ r ii ) ((pos "L@%" ) ))
( "artisan"  n ( aa2 t i z a1 n ) ((pos "K6%" ) ))
( "artisans"  n ( aa2 t i z a1 n z ) ((pos "Kj%" ) ))
( "artist"  n ( aa1 t i s t ) ((pos "K6%" ) ))
( "artiste"  n ( aa1 t ii1 s t ) ((pos "K6%" ) ))
( "artistes"  n ( aa1 t ii1 s t s ) ((pos "Kj%" ) ))
( "artistic"  j ( aa1 t i1 s t i k ) ((pos "OA%" ) ))
( "artistically"  a ( aa1 t i1 s t i k l ii ) ((pos "Pu%" ) ))
( "artistry"  n ( aa1 t i s t r ii ) ((pos "L@%" ) ))
( "artists"  n ( aa1 t i s t s ) ((pos "Kj%" ) ))
( "artless"  j ( aa1 t l i s ) ((pos "OA%" ) ))
( "artlessly"  a ( aa1 t l i s l ii ) ((pos "Pu%" ) ))
( "artlessness"  n ( aa1 t l i s n @ s ) ((pos "L@%" ) ))
( "arts"  n ( aa1 t s ) ((pos "Mj%" ) ))
( "arty"  j ( aa1 t ii ) ((pos "OA%" ) ))
( "arty-crafty"  j ( aa2 t i - k r a1 f t ii ) ((pos "OA%" ) ))
( "arum"  n ( e@1 r @ m ) ((pos "K6$" ) ))
( "arums"  n ( e@1 r @ m z ) ((pos "Kj$" ) ))
( "as"  cc ( a z ) ((pos "V-*" ) ))
( "asap"  n ( ei2 e2 s ei2 p ii1 ) ((pos "Y~%" ) ))
( "asbestos"  n ( a1 z b e1 s t o s ) ((pos "L@%" ) ))
( "ascend"  v ( @1 s e1 n d ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "ascendancy"  n ( @1 s e1 n d @ n s ii ) ((pos "L@%" ) ))
( "ascendant"  n ( @1 s e1 n d @ n t ) ((pos "K6%" ) ))
( "ascendants"  n ( @1 s e1 n d @ n t s ) ((pos "Kj$" ) ))
( "ascended"  v ( @1 s e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "ascendency"  n ( @1 s e1 n d @ n s ii ) ((pos "L@%" ) ))
( "ascendent"  n ( @1 s e1 n d @ n t ) ((pos "K6$" ) ))
( "ascendents"  n ( @1 s e1 n d @ n t s ) ((pos "Kj$" ) ))
( "ascending"  v ( @1 s e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "ascends"  v ( @1 s e1 n d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "ascension"  n ( @1 s e1 n sh @ n ) ((pos "K6%" ) ))
( "ascensions"  n ( @1 s e1 n sh @ n z ) ((pos "Kj$" ) ))
( "ascent"  n ( @1 s e1 n t ) ((pos "K6%" ) ))
( "ascents"  n ( @1 s e1 n t s ) ((pos "Kj%" ) ))
( "ascertain"  v ( a2 s @ t ei1 n ) ((pos "H0%" ) (subcat "6A" "8" "9" "10" )))
( "ascertainable"  j ( a2 s @ t ei1 n @ b l ) ((pos "OA%" ) ))
( "ascertained"  v ( a2 s @ t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" )))
( "ascertaining"  v ( a2 s @ t ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" )))
( "ascertains"  v ( a2 s @ t ei1 n z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" )))
( "ascetic"  n ( @1 s e1 t i k ) ((pos "K6%" "OA%" ) ))
( "ascetically"  a ( @1 s ii1 t i k l ii ) ((pos "Pu%" ) ))
( "asceticism"  n ( @1 s e1 t i s i z @ m ) ((pos "L@%" ) ))
( "ascetics"  n ( @1 s e1 t i k s ) ((pos "Kj%" ) ))
( "ascorbic"  j ( @1 s k oo1 b i k ) ((pos "OA%" ) ))
( "ascribable"  j ( @1 s k r ai1 b @ b l ) ((pos "OA%" ) ))
( "ascribe"  v ( @1 s k r ai1 b ) ((pos "H2%" ) (subcat )))
( "ascribed"  v ( @1 s k r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ascribes"  v ( @1 s k r ai1 b z ) ((pos "Ha%" ) (subcat )))
( "ascribing"  v ( @1 s k r ai1 b i ng ) ((pos "Hb%" ) (subcat )))
( "ascription"  n ( @1 s k r i1 p sh @ n ) ((pos "K6$" ) ))
( "ascriptions"  n ( @1 s k r i1 p sh @ n z ) ((pos "Kj$" ) ))
( "asdic"  n ( a1 z d i k ) ((pos "K6$" ) ))
( "asdics"  n ( a1 z d i k s ) ((pos "Kj$" ) ))
( "asepsis"  n ( ei2 s e1 p s i s ) ((pos "L@$" ) ))
( "aseptic"  j ( ei2 s e1 p t i k ) ((pos "OA%" ) ))
( "asexual"  j ( ei2 s e1 k sh u@ l ) ((pos "OA%" ) ))
( "asexuality"  n ( ei1 s e2 k sh u a1 l i t ii ) ((pos "L@%" ) ))
( "ash"  n ( a1 sh ) ((pos "M7%" ) ))
( "ash-bin"  n ( a1 sh - b i n ) ((pos "K6%" ) ))
( "ash-bins"  n ( a1 sh - b i n z ) ((pos "Kj%" ) ))
( "ash-can"  n ( a1 sh - k a n ) ((pos "K6%" ) ))
( "ash-cans"  n ( a1 sh - k a n z ) ((pos "Kj%" ) ))
( "ash-key"  n ( a1 sh - k ii ) ((pos "K6$" ) ))
( "ash-keys"  n ( a1 sh - k ii z ) ((pos "Kj$" ) ))
( "ash-pan"  n ( a1 sh - p a n ) ((pos "K6%" ) ))
( "ash-pans"  n ( a1 sh - p a n z ) ((pos "Kj%" ) ))
( "ashamed"  j ( @1 sh ei1 m d ) ((pos "Op%" ) ))
( "ashamedly"  a ( @1 sh ei1 m i d l ii ) ((pos "Pu%" ) ))
( "ashen"  j ( a1 sh @ n ) ((pos "OA%" ) ))
( "ashes"  n ( a1 sh i z ) ((pos "Mj%" ) ))
( "ashore"  a ( @1 sh oo1 r ) ((pos "Pu%" ) ))
( "ashtray"  n ( a1 sh t r ei ) ((pos "K6%" ) ))
( "ashtrays"  n ( a1 sh t r ei z ) ((pos "Kj%" ) ))
( "ashy"  j ( a1 sh ii ) ((pos "OA%" ) ))
( "aside"  n ( @1 s ai1 d ) ((pos "K6%" "Pu%" ) ))
( "asides"  n ( @1 s ai1 d z ) ((pos "Kj%" ) ))
( "asinine"  j ( a1 s i n ai n ) ((pos "OA%" ) ))
( "ask"  v ( aa1 s k ) ((pos "J0%" ) (subcat "3A" "6A" "7A" "8" "9" "10" "12C" "14" "15B" "17" "20" )))
( "askance"  a ( @1 s k a1 n s ) ((pos "Pu%" ) ))
( "asked"  v ( aa1 s k t ) ((pos "Jc*" "Jd*" ) (subcat "3A" "6A" "7A" "8" "9" "10" "12C" "14" "15B" "17" "20" )))
( "askew"  j ( @1 s k y uu1 ) ((pos "Op%" "Pu%" ) ))
( "asking"  v ( aa1 s k i ng ) ((pos "Jb%" "Ki%" ) (subcat "3A" "6A" "7A" "8" "9" "10" "12C" "14" "15B" "17" "20" )))
( "asks"  v ( aa1 s k s ) ((pos "Ja%" ) (subcat "3A" "6A" "7A" "8" "9" "10" "12C" "14" "15B" "17" "20" )))
( "aslant"  a ( @1 s l aa1 n t ) ((pos "Pu%" "T-%" ) ))
( "asleep"  j ( @1 s l ii1 p ) ((pos "Op%" "Pu%" ) ))
( "asp"  n ( a1 s p ) ((pos "K6%" ) ))
( "asparagus"  n ( @1 s p a1 r @ g @ s ) ((pos "L@%" ) ))
( "aspect"  n ( a1 s p e k t ) ((pos "K6%" ) ))
( "aspects"  n ( a1 s p e k t s ) ((pos "Kj%" ) ))
( "aspectual"  j ( a1 s p e1 k ch u@ l ) ((pos "OA$" ) ))
( "aspen"  n ( a1 s p @ n ) ((pos "K6$" ) ))
( "aspens"  n ( a1 s p @ n z ) ((pos "Kj$" ) ))
( "asperities"  n ( a1 s p e1 r i t i z ) ((pos "Mj$" ) ))
( "asperity"  n ( a1 s p e1 r i t ii ) ((pos "M8%" ) ))
( "asperse"  v ( @1 s p @@1 s ) ((pos "H2$" ) (subcat )))
( "aspersed"  v ( @1 s p @@1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "asperses"  v ( @1 s p @@1 s i z ) ((pos "Ha$" ) (subcat )))
( "aspersing"  v ( @1 s p @@1 s i ng ) ((pos "Hb$" ) (subcat )))
( "aspersion"  n ( @1 s p @@1 sh @ n ) ((pos "K6%" ) ))
( "aspersions"  n ( @1 s p @@1 sh @ n z ) ((pos "Kj%" ) ))
( "asphalt"  v ( a1 s f a l t ) ((pos "H0%" "L@%" ) (subcat )))
( "asphalted"  v ( a1 s f a l t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "asphalting"  v ( a1 s f a l t i ng ) ((pos "Hb%" ) (subcat )))
( "asphalts"  v ( a1 s f a l t s ) ((pos "Ha%" ) (subcat )))
( "asphodel"  n ( a1 s f @ d e l ) ((pos "K6$" ) ))
( "asphodels"  n ( a1 s f @ d e l z ) ((pos "Kj$" ) ))
( "asphyxia"  n ( a1 s f i1 k s i@ ) ((pos "L@%" ) ))
( "asphyxiate"  v ( @1 s f i1 k s i ei t ) ((pos "H2%" ) (subcat )))
( "asphyxiated"  v ( @1 s f i1 k s i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "asphyxiates"  v ( @1 s f i1 k s i ei t s ) ((pos "Ha%" ) (subcat )))
( "asphyxiating"  v ( @1 s f i1 k s i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "asphyxiation"  n ( @1 s f i2 k s i ei1 sh @ n ) ((pos "L@%" ) ))
( "aspic"  n ( a1 s p i k ) ((pos "L@%" ) ))
( "aspidistra"  n ( a2 s p i d i1 s t r @ ) ((pos "K6%" ) ))
( "aspidistras"  n ( a2 s p i d i1 s t r @ z ) ((pos "Kj%" ) ))
( "aspirant"  n ( a1 s p i r @ n t ) ((pos "K6%" ) ))
( "aspirants"  n ( a1 s p i r @ n t s ) ((pos "Kj%" ) ))
( "aspirate"  n ( a1 s p i r @ t ) ((pos "K6%" ) ))
( "aspirate"  v ( a1 s p i r ei t ) ((pos "H2%" ) ))
( "aspirated"  v ( a1 s p i r ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "aspirates"  n ( a1 s p i r @ t s ) ((pos "Kj%" ) ))
( "aspirates"  v ( a1 s p i r ei t s ) ((pos "Ha%" ) ))
( "aspirating"  v ( a1 s p i r ei t i ng ) ((pos "Hb%" ) ))
( "aspiration"  n ( a2 s p i r ei1 sh @ n ) ((pos "M6%" ) ))
( "aspirations"  n ( a2 s p i r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "aspire"  v ( @1 s p ai1 @ r ) ((pos "I2%" ) (subcat "3A" )))
( "aspired"  v ( @1 s p ai1 @ d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "aspires"  v ( @1 s p ai1 @ z ) ((pos "Ia%" ) (subcat "3A" )))
( "aspirin"  n ( a1 s p r i n ) ((pos "M6%" ) ))
( "aspiring"  v ( @1 s p ai1 @ r i ng ) ((pos "Ib%" ) (subcat "3A" )))
( "aspirins"  n ( a1 s p r i n z ) ((pos "Mj%" ) ))
( "asps"  n ( a1 s p s ) ((pos "Kj%" ) ))
( "ass"  n ( a1 s ) ((pos "K7%" ) ))
( "assagai"  n ( a1 s @ g ai ) ((pos "K6$" ) ))
( "assagais"  n ( a1 s @ g ai z ) ((pos "Kj$" ) ))
( "assail"  v ( @1 s ei1 l ) ((pos "H0%" ) (subcat "6A" )))
( "assailable"  j ( @1 s ei1 l @ b l ) ((pos "OA$" ) ))
( "assailant"  n ( @1 s ei1 l @ n t ) ((pos "K6%" ) ))
( "assailants"  n ( @1 s ei1 l @ n t s ) ((pos "Kj%" ) ))
( "assailed"  v ( @1 s ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "assailing"  v ( @1 s ei1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "assails"  v ( @1 s ei1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "assassin"  n ( @1 s a1 s i n ) ((pos "K6%" ) ))
( "assassinate"  v ( @1 s a1 s i n ei t ) ((pos "H2%" ) (subcat )))
( "assassinated"  v ( @1 s a1 s i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "assassinates"  v ( @1 s a1 s i n ei t s ) ((pos "Ha%" ) (subcat )))
( "assassinating"  v ( @1 s a1 s i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "assassination"  n ( @1 s a2 s i n ei1 sh @ n ) ((pos "M6%" ) ))
( "assassinations"  n ( @1 s a2 s i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "assassins"  n ( @1 s a1 s i n z ) ((pos "Kj%" ) ))
( "assault"  v ( @1 s oo1 l t ) ((pos "H0%" "K6%" ) (subcat )))
( "assaulted"  v ( @1 s oo1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "assaulting"  v ( @1 s oo1 l t i ng ) ((pos "Hb%" ) (subcat )))
( "assaults"  v ( @1 s oo1 l t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "assay"  v ( @1 s ei1 ) ((pos "H0$" "K6$" ) (subcat "6A" )))
( "assayed"  v ( @1 s ei1 d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "assaying"  v ( @1 s ei1 i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "assays"  v ( @1 s ei1 z ) ((pos "Ha$" "Kj$" ) (subcat "6A" )))
( "assegai"  n ( a1 s @ g ai ) ((pos "K6%" ) ))
( "assegais"  n ( a1 s @ g ai z ) ((pos "Kj%" ) ))
( "assemblage"  n ( @1 s e1 m b l i jh ) ((pos "M6%" ) ))
( "assemblages"  n ( @1 s e1 m b l i jh i z ) ((pos "Mj$" ) ))
( "assemble"  v ( @1 s e1 m b l ) ((pos "J2%" ) (subcat "2A" )))
( "assembled"  v ( @1 s e1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "assembles"  v ( @1 s e1 m b l z ) ((pos "Ja%" ) (subcat "2A" )))
( "assemblies"  n ( @1 s e1 m b l i z ) ((pos "Kj%" ) ))
( "assembling"  v ( @1 s e1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "assembly"  n ( @1 s e1 m b l ii ) ((pos "K8%" ) ))
( "assent"  v ( @1 s e1 n t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "assented"  v ( @1 s e1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "assenting"  v ( @1 s e1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "assents"  v ( @1 s e1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "assert"  v ( @1 s @@1 t ) ((pos "H0%" ) (subcat "6A" "9" )))
( "asserted"  v ( @1 s @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "asserting"  v ( @1 s @@1 t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "assertion"  n ( @1 s @@1 sh @ n ) ((pos "M6%" ) ))
( "assertions"  n ( @1 s @@1 sh @ n z ) ((pos "Mj%" ) ))
( "assertive"  j ( @1 s @@1 t i v ) ((pos "OA%" ) ))
( "assertively"  a ( @1 s @@1 t i v l ii ) ((pos "Pu%" ) ))
( "asserts"  v ( @1 s @@1 t s ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "asses"  n ( a1 s i z ) ((pos "Kj%" ) ))
( "assess"  v ( @1 s e1 s ) ((pos "H1%" ) (subcat "6A" )))
( "assessed"  v ( @1 s e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "assesses"  v ( @1 s e1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "assessing"  v ( @1 s e1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "assessment"  n ( @1 s e1 s m @ n t ) ((pos "M6%" ) ))
( "assessments"  n ( @1 s e1 s m @ n t s ) ((pos "Mj%" ) ))
( "assessor"  n ( @1 s e1 s @ r ) ((pos "K6%" ) ))
( "assessors"  n ( @1 s e1 s @ z ) ((pos "Kj%" ) ))
( "asset"  n ( a1 s e t ) ((pos "K6%" ) ))
( "assets"  n ( a1 s e t s ) ((pos "Kj%" ) ))
( "asseverate"  v ( @1 s e1 v @ r ei t ) ((pos "H2$" ) (subcat "6A" )))
( "asseverated"  v ( @1 s e1 v @ r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "asseverates"  v ( @1 s e1 v @ r ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "asseverating"  v ( @1 s e1 v @ r ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "asseveration"  n ( @1 s e2 v @ r ei1 sh @ n ) ((pos "K6$" ) ))
( "asseverations"  n ( @1 s e2 v @ r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "asshole"  n ( a1 s h ou l ) ((pos "K6%" ) ))
( "assholes"  n ( a1 s h ou l z ) ((pos "Kj%" ) ))
( "assiduities"  n ( a2 s i d y uu1 i t i z ) ((pos "Mj$" ) ))
( "assiduity"  n ( a2 s i d y uu1 i t ii ) ((pos "M8%" ) ))
( "assiduous"  j ( @1 s i1 d y u@ s ) ((pos "OA%" ) ))
( "assiduously"  a ( @1 s i1 d y u@ s l ii ) ((pos "Pu%" ) ))
( "assign"  v ( @1 s ai1 n ) ((pos "H0%" ) (subcat "12A" "13A" "13B" "14" )))
( "assignable"  j ( @1 s ai1 n @ b l ) ((pos "OA%" ) ))
( "assignation"  n ( a2 s i g n ei1 sh @ n ) ((pos "K6%" ) ))
( "assignations"  n ( a2 s i g n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "assigned"  v ( @1 s ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "12A" "13A" "13B" "14" )))
( "assigning"  v ( @1 s ai1 n i ng ) ((pos "Hb%" ) (subcat "12A" "13A" "13B" "14" )))
( "assignment"  n ( @1 s ai1 n m @ n t ) ((pos "M6%" ) ))
( "assignments"  n ( @1 s ai1 n m @ n t s ) ((pos "Mj%" ) ))
( "assigns"  v ( @1 s ai1 n z ) ((pos "Ha%" ) (subcat "12A" "13A" "13B" "14" )))
( "assimilate"  v ( @1 s i1 m @ l ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "assimilated"  v ( @1 s i1 m @ l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "assimilates"  v ( @1 s i1 m @ l ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "assimilating"  v ( @1 s i1 m @ l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "assimilation"  n ( @1 s i2 m @ l ei1 sh @ n ) ((pos "L@%" ) ))
( "assist"  v ( @1 s i1 s t ) ((pos "J0%" ) (subcat "2A" "3A" "6A" "14" )))
( "assistance"  n ( @1 s i1 s t @ n s ) ((pos "L@%" ) ))
( "assistant"  n ( @1 s i1 s t @ n t ) ((pos "K6%" ) ))
( "assistants"  n ( @1 s i1 s t @ n t s ) ((pos "Kj%" ) ))
( "assisted"  v ( @1 s i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "assisting"  v ( @1 s i1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "assists"  v ( @1 s i1 s t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "14" )))
( "assize"  n ( @1 s ai1 z ) ((pos "L@$" ) ))
( "assizes"  n ( @1 s ai1 z i z ) ((pos "Kj%" ) ))
( "assoc"  n ( @1 s o1 k ) ((pos "Y>%" ) ))
( "associate"  n ( @1 s ou1 sh i@ t ) ((pos "K6%" "OA%" ) ))
( "associate"  v ( @1 s ou1 sh i ei t ) ((pos "J2%" ) (subcat "3A" )))
( "associated"  v ( @1 s ou1 sh i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "associates"  n ( @1 s ou1 sh i@ t s ) ((pos "Kj%" ) ))
( "associates"  v ( @1 s ou1 sh i ei t s ) ((pos "Ja%" ) (subcat "3A" )))
( "associating"  v ( @1 s ou1 sh i ei t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "association"  n ( @1 s ou2 s i ei1 sh @ n ) ((pos "M6%" ) ))
( "associations"  n ( @1 s ou2 s i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "assonance"  n ( a1 s @ n @ n s ) ((pos "K6%" ) ))
( "assonances"  n ( a1 s @ n @ n s i z ) ((pos "Kj$" ) ))
( "assorted"  j ( @1 s oo1 t i d ) ((pos "OA%" ) ))
( "assortment"  n ( @1 s oo1 t m @ n t ) ((pos "K6%" ) ))
( "assortments"  n ( @1 s oo1 t m @ n t s ) ((pos "Kj%" ) ))
( "asst"  n ( @1 s i1 s t @ n t ) ((pos "Y>%" ) ))
( "assuage"  v ( @1 s w ei1 jh ) ((pos "H2%" ) (subcat )))
( "assuaged"  v ( @1 s w ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "assuages"  v ( @1 s w ei1 jh i z ) ((pos "Ha%" ) (subcat )))
( "assuaging"  v ( @1 s w ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "assume"  v ( @1 s y uu1 m ) ((pos "H2%" ) (subcat "6A" "9" )))
( "assumed"  v ( @1 s y uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "assumes"  v ( @1 s y uu1 m z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "assuming"  v ( @1 s y uu1 m i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "assumption"  n ( @1 s uh1 m p sh @ n ) ((pos "K6%" ) ))
( "assumptions"  n ( @1 s uh1 m p sh @ n z ) ((pos "Kj%" ) ))
( "assurance"  n ( @1 sh u@1 r @ n s ) ((pos "M6%" ) ))
( "assurances"  n ( @1 sh u@1 r @ n s i z ) ((pos "Mj%" ) ))
( "assure"  v ( @1 sh u@1 r ) ((pos "H2%" ) (subcat "6A" "11" )))
( "assured"  v ( @1 sh u@1 d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "11" )))
( "assuredly"  a ( @1 sh u@1 r i d l ii ) ((pos "Pu%" ) ))
( "assuredness"  n ( @1 sh u@1 r i d n i s ) ((pos "L@%" ) ))
( "assures"  v ( @1 sh u@1 z ) ((pos "Ha%" ) (subcat "6A" "11" )))
( "assuring"  v ( @1 sh u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "11" )))
( "aster"  n ( a1 s t @ r ) ((pos "K6%" ) ))
( "asterisk"  n ( a1 s t @ r i s k ) ((pos "K6%" ) ))
( "asterisks"  n ( a1 s t @ r i s k s ) ((pos "Kj%" ) ))
( "astern"  a ( @1 s t @@1 n ) ((pos "Pu%" ) ))
( "asteroid"  n ( a1 s t @ r oi d ) ((pos "K6%" ) ))
( "asteroids"  n ( a1 s t @ r oi d z ) ((pos "Kj%" ) ))
( "asters"  n ( a1 s t @ z ) ((pos "Kj%" ) ))
( "asthma"  n ( a1 s m @ ) ((pos "L@%" ) ))
( "asthmatic"  j ( a1 s m a1 t i k ) ((pos "OA%" ) ))
( "astigmatic"  j ( a2 s t i g m a1 t i k ) ((pos "OA%" ) ))
( "astigmatism"  n ( @1 s t i1 g m @ t i z @ m ) ((pos "L@%" ) ))
( "astir"  j ( @1 s t @@1 r ) ((pos "Op$" "Pu$" ) ))
( "astonish"  v ( @1 s t o1 n i sh ) ((pos "H1%" ) (subcat )))
( "astonished"  v ( @1 s t o1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "astonishes"  v ( @1 s t o1 n i sh i z ) ((pos "Ha%" ) (subcat )))
( "astonishing"  v ( @1 s t o1 n i sh i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "astonishingly"  a ( @1 s t o1 n i sh i ng l ii ) ((pos "Pu%" ) ))
( "astonishment"  n ( @1 s t o1 n i sh m @ n t ) ((pos "L@%" ) ))
( "astound"  v ( @1 s t au1 n d ) ((pos "H0%" ) (subcat )))
( "astounded"  v ( @1 s t au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "astounding"  v ( @1 s t au1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "astounds"  v ( @1 s t au1 n d z ) ((pos "Ha%" ) (subcat )))
( "astrakhan"  n ( a2 s t r @ k a1 n ) ((pos "L@%" ) ))
( "astral"  j ( a1 s t r @ l ) ((pos "OA%" ) ))
( "astray"  j ( @1 s t r ei1 ) ((pos "Op%" "Pu%" ) ))
( "astride"  j ( @1 s t r ai1 d ) ((pos "Op%" "Pu%" "T-%" ) ))
( "astringency"  n ( @1 s t r i1 n jh @ n s ii ) ((pos "L@%" ) ))
( "astringent"  n ( @1 s t r i1 n jh @ n t ) ((pos "K6%" "OA%" ) ))
( "astringents"  n ( @1 s t r i1 n jh @ n t s ) ((pos "Kj%" ) ))
( "astrodome"  n ( a1 s t r @ d ou m ) ((pos "K6$" ) ))
( "astrodomes"  n ( a1 s t r @ d ou m z ) ((pos "Kj$" ) ))
( "astrolabe"  n ( a1 s t r @ l ei b ) ((pos "K6$" ) ))
( "astrolabes"  n ( a1 s t r @ l ei b z ) ((pos "Kj$" ) ))
( "astrologer"  n ( @1 s t r o1 l @ jh @ r ) ((pos "K6%" ) ))
( "astrologers"  n ( @1 s t r o1 l @ jh @ z ) ((pos "Kj%" ) ))
( "astrological"  j ( a2 s t r @ l o1 jh i k l ) ((pos "OA%" ) ))
( "astrology"  n ( @1 s t r o1 l @ jh ii ) ((pos "L@%" ) ))
( "astronaut"  n ( a1 s t r @ n oo t ) ((pos "K6%" ) ))
( "astronautics"  n ( a2 s t r @ n oo1 t i k s ) ((pos "Lk$" ) ))
( "astronauts"  n ( a1 s t r @ n oo t s ) ((pos "Kj%" ) ))
( "astronomer"  n ( @1 s t r o1 n @ m @ r ) ((pos "K6%" ) ))
( "astronomers"  n ( @1 s t r o1 n @ m @ z ) ((pos "Kj%" ) ))
( "astronomical"  j ( a2 s t r @ n o1 m i k l ) ((pos "OA%" ) ))
( "astronomically"  a ( a2 s t r @ n o1 m i k l ii ) ((pos "Pu%" ) ))
( "astronomy"  n ( @1 s t r o1 n @ m ii ) ((pos "L@%" ) ))
( "astrophysics"  n ( a2 s t r ou f i1 z i k s ) ((pos "Lk%" ) ))
( "astute"  j ( @1 s t y uu1 t ) ((pos "OA%" ) ))
( "astutely"  a ( @1 s t y uu1 t l ii ) ((pos "Pu%" ) ))
( "astuteness"  n ( @1 s t y uu1 t n @ s ) ((pos "L@%" ) ))
( "asunder"  a ( @1 s uh1 n d @ r ) ((pos "Pu%" ) ))
( "asylum"  n ( @1 s ai1 l @ m ) ((pos "M6%" ) ))
( "asylums"  n ( @1 s ai1 l @ m z ) ((pos "Mj%" ) ))
( "asymmetric"  j ( ei2 s i m e1 t r i k ) ((pos "OA%" ) ))
( "asymmetrical"  j ( ei2 s i m e1 t r i k l ) ((pos "OA%" ) ))
( "asymmetrically"  a ( ei2 s i m e1 t r i k l ii ) ((pos "Pu%" ) ))
( "asymmetry"  n ( ei1 s i1 m i t r ii ) ((pos "L@%" ) ))
( "asymptote"  n ( a1 s i m t ou t ) ((pos "K6%" ) ))
( "asymptotes"  n ( a1 s i m t ou t s ) ((pos "Kj$" ) ))
( "asymptotic"  j ( a2 s i m t o1 t i k ) ((pos "OA%" ) ))
( "asymptotically"  a ( a2 s i m t o1 t i k l ii ) ((pos "Pu%" ) ))
( "at"  in ( a t ) ((pos "T-*" ) ))
( "at-home"  n ( @1 t - h ou1 m ) ((pos "K6%" ) ))
( "at-homes"  n ( @1 t - h ou1 m z ) ((pos "Kj%" ) ))
( "atabrine"  n ( a1 t @ b r ii n ) ((pos "L@$" ) ))
( "atavism"  n ( a1 t @ v i z @ m ) ((pos "K6%" ) ))
( "atavisms"  n ( a1 t @ v i z @ m z ) ((pos "Kj$" ) ))
( "atavistic"  j ( a2 t @ v i1 s t i k ) ((pos "OA%" ) ))
( "ate"  v ( e1 t ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "atelier"  n ( a1 t e1 l i ei ) ((pos "K6$" ) ))
( "ateliers"  n ( a1 t e1 l i ei z ) ((pos "Kj$" ) ))
( "atheism"  n ( ei1 th i i z @ m ) ((pos "L@%" ) ))
( "atheist"  n ( ei1 th i i s t ) ((pos "K6%" ) ))
( "atheistic"  j ( ei2 th i i1 s t i k ) ((pos "OA%" ) ))
( "atheists"  n ( ei1 th i i s t s ) ((pos "Kj%" ) ))
( "athirst"  j ( @1 th @@1 s t ) ((pos "Op$" ) ))
( "athlete"  n ( a1 th l ii t ) ((pos "K6%" ) ))
( "athletes"  n ( a1 th l ii t s ) ((pos "Kj%" ) ))
( "athletic"  j ( a1 th l e1 t i k ) ((pos "OA%" ) ))
( "athletics"  n ( a1 th l e1 t i k s ) ((pos "Lk%" ) ))
( "athwart"  a ( @1 th w oo1 t ) ((pos "Pu%" "T-%" ) ))
( "atishoo"  uh ( @1 t i1 sh uu ) ((pos "W-%" ) ))
( "atlas"  n ( a1 t l @ s ) ((pos "K7%" ) ))
( "atlases"  n ( a1 t l @ s i z ) ((pos "Kj%" ) ))
( "atmosphere"  n ( a1 t m @ s f i@ r ) ((pos "M6%" ) ))
( "atmospheres"  n ( a1 t m @ s f i@ z ) ((pos "Mj%" ) ))
( "atmospheric"  j ( a2 t m @ s f e1 r i k ) ((pos "OA%" ) ))
( "atmospherics"  n ( a2 t m @ s f e1 r i k s ) ((pos "Lk%" ) ))
( "atoll"  n ( a1 t o l ) ((pos "K6$" ) ))
( "atolls"  n ( a1 t o l z ) ((pos "Kj$" ) ))
( "atom"  n ( a1 t @ m ) ((pos "K6%" ) ))
( "atomic"  j ( @1 t o1 m i k ) ((pos "OA%" ) ))
( "atomize"  v ( a1 t @ m ai z ) ((pos "H2%" ) ))
( "atomized"  v ( a1 t @ m ai z d ) ((pos "Hc%" "Hd%" ) ))
( "atomizer"  n ( a1 t @ m ai z @ r ) ((pos "K6%" ) ))
( "atomizers"  n ( a1 t @ m ai z @ z ) ((pos "Kj%" ) ))
( "atomizes"  v ( a1 t @ m ai z i z ) ((pos "Ha%" ) ))
( "atomizing"  v ( a1 t @ m ai z i ng ) ((pos "Hb%" ) ))
( "atoms"  n ( a1 t @ m z ) ((pos "Kj%" ) ))
( "atonal"  j ( ei1 t ou1 n l ) ((pos "OA%" ) ))
( "atonality"  n ( ei2 t ou n a1 l i t ii ) ((pos "L@$" ) ))
( "atone"  v ( @1 t ou1 n ) ((pos "I2%" ) (subcat "2A" )))
( "atoned"  v ( @1 t ou1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "atonement"  n ( @1 t ou1 n m @ n t ) ((pos "L@%" ) ))
( "atones"  v ( @1 t ou1 n z ) ((pos "Ia%" ) (subcat "2A" )))
( "atoning"  v ( @1 t ou1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "atop"  a ( @1 t o1 p ) ((pos "Pu%" ) ))
( "atrabilious"  j ( a2 t r @ b i1 l i@ s ) ((pos "OA$" ) ))
( "atrocious"  j ( @1 t r ou1 sh @ s ) ((pos "OA%" ) ))
( "atrociously"  a ( @1 t r ou1 sh @ s l ii ) ((pos "Pu%" ) ))
( "atrocities"  n ( @1 t r o1 s i t i z ) ((pos "Mj%" ) ))
( "atrocity"  n ( @1 t r o1 s i t ii ) ((pos "M8%" ) ))
( "atrophied"  v ( a1 t r @ f i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" )))
( "atrophies"  v ( a1 t r @ f i z ) ((pos "Ja%" ) (subcat "2A" "2B" )))
( "atrophy"  v ( a1 t r @ f ii ) ((pos "J3%" "L@%" ) (subcat "2A" "2B" )))
( "atrophying"  v ( a1 t r @ f i i ng ) ((pos "Jb%" ) (subcat "2A" "2B" )))
( "attaboy"  uh ( a2 t @ b oi1 ) ((pos "W-%" ) ))
( "attach"  v ( @1 t a1 ch ) ((pos "J1%" ) (subcat "3A" "6A" )))
( "attach_e"  n ( @1 t a1 sh ei ) ((pos "K6%" ) ))
( "attach_es"  n ( @1 t a1 sh ei z ) ((pos "Kj%" ) ))
( "attached"  v ( @1 t a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "attaches"  v ( @1 t a1 ch i z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "attaching"  v ( @1 t a1 ch i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "attachment"  n ( @1 t a1 ch m @ n t ) ((pos "M6%" ) ))
( "attachments"  n ( @1 t a1 ch m @ n t s ) ((pos "Mj%" ) ))
( "attack"  v ( @1 t a1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "attacked"  v ( @1 t a1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "attacker"  n ( @1 t a1 k @ r ) ((pos "K6%" ) ))
( "attackers"  n ( @1 t a1 k @ z ) ((pos "Kj%" ) ))
( "attacking"  v ( @1 t a1 k i ng ) ((pos "Hb%" ) (subcat )))
( "attacks"  v ( @1 t a1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "attain"  v ( @1 t ei1 n ) ((pos "J0%" ) (subcat "3A" )))
( "attainable"  j ( @1 t ai1 n @ b l ) ((pos "OA%" ) ))
( "attainder"  n ( @1 t ei1 n d @ r ) ((pos "K6$" ) ))
( "attainders"  n ( @1 t ei1 n d @ z ) ((pos "Kj$" ) ))
( "attained"  v ( @1 t ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "attaining"  v ( @1 t ei1 n i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "attainment"  n ( @1 t ei1 n m @ n t ) ((pos "M6%" ) ))
( "attainments"  n ( @1 t ei1 n m @ n t s ) ((pos "Mj%" ) ))
( "attains"  v ( @1 t ei1 n z ) ((pos "Ja%" ) (subcat "3A" )))
( "attar"  n ( a1 t @ r ) ((pos "L@$" ) ))
( "attempt"  v ( @1 t e1 m p t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "attempted"  v ( @1 t e1 m p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "attempting"  v ( @1 t e1 m p t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "attempts"  v ( @1 t e1 m p t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "attend"  v ( @1 t e1 n d ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "attendance"  n ( @1 t e1 n d @ n s ) ((pos "M6%" ) ))
( "attendances"  n ( @1 t e1 n d @ n s i z ) ((pos "Mj%" ) ))
( "attendant"  n ( @1 t e1 n d @ n t ) ((pos "K6%" "OA%" ) ))
( "attendants"  n ( @1 t e1 n d @ n t s ) ((pos "Kj%" ) ))
( "attended"  v ( @1 t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "attending"  v ( @1 t e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "attends"  v ( @1 t e1 n d z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "attention"  n ( @1 t e1 n sh @ n ) ((pos "M6%" ) ))
( "attentions"  n ( @1 t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "attentive"  j ( @1 t e1 n t i v ) ((pos "OA%" ) ))
( "attentively"  a ( @1 t e1 n t i v l ii ) ((pos "Pu%" ) ))
( "attentiveness"  n ( @1 t e1 n t i v n i s ) ((pos "L@%" ) ))
( "attenuate"  v ( @1 t e1 n y u ei t ) ((pos "H2$" ) (subcat )))
( "attenuated"  v ( @1 t e1 n y u ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "attenuates"  v ( @1 t e1 n y u ei t s ) ((pos "Ha$" ) (subcat )))
( "attenuating"  v ( @1 t e1 n y u ei t i ng ) ((pos "Hb$" ) (subcat )))
( "attenuation"  n ( @1 t e2 n y u ei1 sh @ n ) ((pos "L@%" ) ))
( "attest"  v ( @1 t e1 s t ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "attested"  v ( @1 t e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "attesting"  v ( @1 t e1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "attests"  v ( @1 t e1 s t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "attic"  n ( a1 t i k ) ((pos "K6%" ) ))
( "attics"  n ( a1 t i k s ) ((pos "Kj%" ) ))
( "attire"  v ( @1 t ai1 @ r ) ((pos "H2%" "L@%" ) (subcat )))
( "attired"  v ( @1 t ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "attires"  v ( @1 t ai1 @ z ) ((pos "Ha%" ) (subcat )))
( "attiring"  v ( @1 t ai1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "attitude"  n ( a1 t i t y uu d ) ((pos "K6%" ) ))
( "attitudes"  n ( a1 t i t y uu d z ) ((pos "Kj%" ) ))
( "attitudinize"  v ( a2 t i t y uu1 d i n ai z ) ((pos "I2$" ) (subcat )))
( "attitudinized"  v ( a2 t i t y uu1 d i n ai z d ) ((pos "Ic$" "Id$" ) (subcat )))
( "attitudinizes"  v ( a2 t i t y uu1 d i n ai z i z ) ((pos "Ia$" ) (subcat )))
( "attitudinizing"  v ( a2 t i t y uu1 d i n ai z i ng ) ((pos "Ib$" ) (subcat )))
( "attorney"  n ( @1 t @@1 n ii ) ((pos "K6%" ) ))
( "attorneys"  n ( @1 t @@1 n i z ) ((pos "Kj%" ) ))
( "attract"  v ( @1 t r a1 k t ) ((pos "H0%" ) (subcat )))
( "attracted"  v ( @1 t r a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "attracting"  v ( @1 t r a1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "attraction"  n ( @1 t r a1 k sh @ n ) ((pos "M6%" ) ))
( "attractions"  n ( @1 t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "attractive"  j ( @1 t r a1 k t i v ) ((pos "OA%" ) ))
( "attractively"  a ( @1 t r a1 k t i v l ii ) ((pos "Pu%" ) ))
( "attracts"  v ( @1 t r a1 k t s ) ((pos "Ha%" ) (subcat )))
( "attributable"  j ( @1 t r i1 b y u t @ b l ) ((pos "OA%" ) ))
( "attribute"  n ( a1 t r i b y uu t ) ((pos "K6%" ) ))
( "attribute"  v ( @1 t r i1 b y uu t ) ((pos "H2%" ) (subcat )))
( "attributed"  v ( @1 t r i1 b y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "attributes"  n ( a1 t r i b y uu t s ) ((pos "Kj%" ) ))
( "attributes"  v ( @1 t r i1 b y uu t s ) ((pos "Ha%" ) (subcat )))
( "attributing"  v ( @1 t r i1 b y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "attribution"  n ( a2 t r i b y uu1 sh @ n ) ((pos "M6%" ) ))
( "attributions"  n ( a2 t r i b y uu1 sh @ n z ) ((pos "Mj$" ) ))
( "attributive"  j ( @1 t r i1 b y u t i v ) ((pos "OA%" ) ))
( "attributively"  a ( @1 t r i1 b y u t i v l ii ) ((pos "Pu%" ) ))
( "attrition"  n ( @1 t r i1 sh @ n ) ((pos "L@%" ) ))
( "attune"  v ( @1 t y uu1 n ) ((pos "H2%" ) (subcat )))
( "attuned"  v ( @1 t y uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "attunes"  v ( @1 t y uu1 n z ) ((pos "Ha%" ) (subcat )))
( "attuning"  v ( @1 t y uu1 n i ng ) ((pos "Hb%" ) (subcat )))
( "atypical"  j ( ei1 t i1 p i k l ) ((pos "OA%" ) ))
( "atypically"  a ( ei1 t i1 p i k l ii ) ((pos "Pu%" ) ))
( "au fait"  j ( ou2 - f ei1 ) ((pos "Op%" ) ))
( "au fond"  a ( ou2 - f o1 n ) ((pos "Pu$" ) ))
( "au pair"  n ( ou2 - p e@1 r ) ((pos "K6%" ) ))
( "au pairs"  n ( ou2 - p e@1 z ) ((pos "Kj%" ) ))
( "au revoir"  uh ( ou2 - r @ v w aa1 r ) ((pos "W-%" ) ))
( "aubergine"  n ( ou1 b @ zh ii n ) ((pos "K6%" ) ))
( "aubergines"  n ( ou1 b @ zh ii n z ) ((pos "Kj%" ) ))
( "aubrietia"  n ( oo1 b r ii1 sh @ ) ((pos "K6$" ) ))
( "aubrietias"  n ( oo1 b r ii1 sh @ z ) ((pos "Kj$" ) ))
( "auburn"  j ( oo1 b @ n ) ((pos "OA%" ) ))
( "auction"  v ( oo1 k sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "auctioned"  v ( oo1 k sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "auctioneer"  n ( oo2 k sh @ n i@1 r ) ((pos "K6%" ) ))
( "auctioneers"  n ( oo2 k sh @ n i@1 z ) ((pos "Kj%" ) ))
( "auctioning"  v ( oo1 k sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "auctions"  v ( oo1 k sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "audacious"  j ( oo1 d ei1 sh @ s ) ((pos "OA%" ) ))
( "audaciously"  a ( oo1 d ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "audacity"  n ( oo1 d a1 s i t ii ) ((pos "L@%" ) ))
( "audibility"  n ( oo2 d i b i1 l i t ii ) ((pos "L@%" ) ))
( "audible"  j ( oo1 d @ b l ) ((pos "OA%" ) ))
( "audibly"  a ( oo1 d @ b l ii ) ((pos "Pu%" ) ))
( "audience"  n ( oo1 d i@ n s ) ((pos "K6%" ) ))
( "audiences"  n ( oo1 d i@ n s i z ) ((pos "Kj%" ) ))
( "audio-"  j ( oo2 d i@ u - ) ((pos "U-%" ) ))
( "audio-lingual"  j ( oo2 d i@ u - l i1 ng g w @ l ) ((pos "OA$" ) ))
( "audit"  v ( oo1 d i t ) ((pos "H0%" "K6%" ) (subcat )))
( "audited"  v ( oo1 d i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "auditing"  v ( oo1 d i t i ng ) ((pos "Hb%" ) (subcat )))
( "audition"  v ( oo1 d i1 sh @ n ) ((pos "H0%" "M6%" ) (subcat )))
( "auditioned"  v ( oo1 d i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "auditioning"  v ( oo1 d i1 sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "auditions"  v ( oo1 d i1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "auditor"  n ( oo1 d i t @ r ) ((pos "K6%" ) ))
( "auditorium"  n ( oo2 d i t oo1 r i@ m ) ((pos "K6%" ) ))
( "auditoriums"  n ( oo2 d i t oo1 r i@ m z ) ((pos "Kj%" ) ))
( "auditors"  n ( oo1 d i t @ z ) ((pos "Kj%" ) ))
( "auditory"  j ( oo1 d i t r ii ) ((pos "OA%" ) ))
( "audits"  v ( oo1 d i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "auger"  n ( oo1 g @ r ) ((pos "K6$" ) ))
( "augers"  n ( oo1 g @ z ) ((pos "Kj$" ) ))
( "aught"  n ( oo1 t ) ((pos "L@$" ) ))
( "augment"  v ( oo1 g m e1 n t ) ((pos "J0%" ) (subcat "2A" )))
( "augmentation"  n ( oo2 g m e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "augmentations"  n ( oo2 g m e n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "augmented"  v ( oo1 g m e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "augmenting"  v ( oo1 g m e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "augments"  v ( oo1 g m e1 n t s ) ((pos "Ja%" ) (subcat "2A" )))
( "augur"  v ( oo1 g @ r ) ((pos "J0%" "K6$" ) (subcat "2A" )))
( "augured"  v ( oo1 g @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "auguries"  n ( oo1 g y u r i z ) ((pos "Kj$" ) ))
( "auguring"  v ( oo1 g @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "augurs"  v ( oo1 g @ z ) ((pos "Ja%" "Kj$" ) (subcat "2A" )))
( "augury"  n ( oo1 g y u r ii ) ((pos "K8$" ) ))
( "august"  j ( oo1 g uh1 s t ) ((pos "OA%" ) ))
( "auk"  n ( oo1 k ) ((pos "K6%" ) ))
( "auks"  n ( oo1 k s ) ((pos "Kj%" ) ))
( "auld lang syne"  n ( oo2 l d - l a ng - z ai1 n ) ((pos "Ki%" ) ))
( "aunt"  n ( aa1 n t ) ((pos "K6%" ) ))
( "auntie"  n ( aa1 n t ii ) ((pos "K6%" ) ))
( "aunties"  n ( aa1 n t i z ) ((pos "Kj%" ) ))
( "aunts"  n ( aa1 n t s ) ((pos "Kj%" ) ))
( "aunty"  n ( aa1 n t ii ) ((pos "K8%" ) ))
( "aura"  n ( oo1 r @ ) ((pos "K6%" ) ))
( "aural"  j ( oo1 r @ l ) ((pos "OA%" ) ))
( "auras"  n ( oo1 r @ z ) ((pos "Kj%" ) ))
( "aureole"  n ( oo1 r i@ u l ) ((pos "K6$" ) ))
( "aureoles"  n ( oo1 r i@ u l z ) ((pos "Kj$" ) ))
( "auricle"  n ( oo1 r i k l ) ((pos "K6$" ) ))
( "auricles"  n ( oo1 r i k l z ) ((pos "Kj$" ) ))
( "auricular"  j ( oo1 r i1 k y u l @ r ) ((pos "OA$" ) ))
( "auriferous"  j ( oo1 r i1 f @ r @ s ) ((pos "OA$" ) ))
( "aurora"  n ( oo1 r oo1 r @ ) ((pos "K6%" ) ))
( "auroras"  n ( oo1 r oo1 r @ z ) ((pos "Kj%" ) ))
( "auspices"  n ( oo1 s p i s i z ) ((pos "Kj%" ) ))
( "auspicious"  j ( oo1 s p i1 sh @ s ) ((pos "OA%" ) ))
( "auspiciously"  a ( oo1 s p i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "austere"  j ( oo1 s t i@1 r ) ((pos "OA%" ) ))
( "austerely"  a ( oo1 s t i@1 l ii ) ((pos "Pu%" ) ))
( "austerities"  n ( oo1 s t e1 r i t i z ) ((pos "Mj%" ) ))
( "austerity"  n ( oo1 s t e1 r i t ii ) ((pos "M8%" ) ))
( "autarchies"  n ( oo1 t aa k i z ) ((pos "Mj$" ) ))
( "autarchy"  n ( oo1 t aa k ii ) ((pos "M8$" ) ))
( "autarky"  n ( oo1 t aa k ii ) ((pos "L@$" ) ))
( "authentic"  j ( oo1 th e1 n t i k ) ((pos "OA%" ) ))
( "authentically"  a ( oo1 th e1 n t i k l ii ) ((pos "Pu%" ) ))
( "authenticate"  v ( oo1 th e1 n t i k ei t ) ((pos "H2%" ) (subcat )))
( "authenticated"  v ( oo1 th e1 n t i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "authenticates"  v ( oo1 th e1 n t i k ei t s ) ((pos "Ha%" ) (subcat )))
( "authenticating"  v ( oo1 th e1 n t i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "authentication"  n ( oo1 th e2 n t i k ei1 sh @ n ) ((pos "L@%" ) ))
( "authenticity"  n ( oo2 th @ n t i1 s i t ii ) ((pos "L@%" ) ))
( "author"  n ( oo1 th @ r ) ((pos "K6%" ) ))
( "authoress"  n ( oo1 th @ r i s ) ((pos "K7$" ) ))
( "authoresses"  n ( oo1 th @ r i s i z ) ((pos "Kj$" ) ))
( "authoritarian"  n ( oo1 th o2 r i t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "authoritarianism"  n ( oo1 th o2 r i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "authoritarians"  n ( oo1 th o2 r i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "authoritative"  j ( oo1 th o1 r i t @ t i v ) ((pos "OA%" ) ))
( "authoritatively"  a ( oo1 th o1 r i t @ t i v l ii ) ((pos "Pu%" ) ))
( "authorities"  n ( oo1 th o1 r i t i z ) ((pos "Mj%" ) ))
( "authority"  n ( oo1 th o1 r i t ii ) ((pos "M8%" ) ))
( "authorization"  n ( oo2 th @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "authorize"  v ( oo1 th @ r ai z ) ((pos "H2%" ) (subcat "6A" )))
( "authorized"  v ( oo1 th @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "authorizes"  v ( oo1 th @ r ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "authorizing"  v ( oo1 th @ r ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "authors"  n ( oo1 th @ z ) ((pos "Kj%" ) ))
( "authorship"  n ( oo1 th @ sh i p ) ((pos "L@%" ) ))
( "autism"  n ( oo1 t i z @ m ) ((pos "L@%" ) ))
( "autistic"  j ( oo1 t i1 s t i k ) ((pos "OA%" ) ))
( "auto"  n ( oo1 t ou ) ((pos "K6%" ) ))
( "auto-"  j ( oo2 t ou - ) ((pos "U-%" ) ))
( "auto-changer"  n ( oo1 t ou - ch ei n jh @ r ) ((pos "K6$" ) ))
( "auto-changers"  n ( oo1 t ou - ch ei n jh @ z ) ((pos "Kj$" ) ))
( "auto-da-f_e"  n ( oo2 t ou - d aa - f ei1 ) ((pos "Ki$" ) ))
( "autobahn"  n ( oo1 t @ b aa n ) ((pos "K6$" ) ))
( "autobahnen"  n ( oo1 t @ b aa n @ n ) ((pos "Kj$" ) ))
( "autobahns"  n ( oo1 t @ b aa n z ) ((pos "Kj$" ) ))
( "autobiographic"  j ( oo2 t @ b ai @ g r a1 f i k ) ((pos "OA$" ) ))
( "autobiographical"  j ( oo2 t @ b ai2 @ g r a1 f i k l ) ((pos "OA%" ) ))
( "autobiographies"  n ( oo2 t @ b ai o1 g r @ f i z ) ((pos "Mj%" ) ))
( "autobiography"  n ( oo2 t @ b ai o1 g r @ f ii ) ((pos "M8%" ) ))
( "autocracies"  n ( oo1 t o1 k r @ s i z ) ((pos "Mj%" ) ))
( "autocracy"  n ( oo1 t o1 k r @ s ii ) ((pos "M8%" ) ))
( "autocrat"  n ( oo1 t @ k r a t ) ((pos "K6%" ) ))
( "autocratic"  j ( oo2 t @ k r a1 t i k ) ((pos "OA%" ) ))
( "autocratically"  a ( oo2 t @ k r a1 t i k l ii ) ((pos "Pu%" ) ))
( "autocrats"  n ( oo1 t @ k r a t s ) ((pos "Kj%" ) ))
( "autogiro"  n ( oo2 t ou jh ai1 @ r ou ) ((pos "K6$" ) ))
( "autogiros"  n ( oo2 t ou jh ai1 @ r ou z ) ((pos "Kj$" ) ))
( "autograph"  v ( oo1 t @ g r aa f ) ((pos "H0%" "K6%" ) (subcat )))
( "autographed"  v ( oo1 t @ g r aa f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "autographing"  v ( oo1 t @ g r aa f i ng ) ((pos "Hb%" ) (subcat )))
( "autographs"  v ( oo1 t @ g r aa f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "autogyro"  n ( oo2 t ou jh ai1 @ r ou ) ((pos "K6$" ) ))
( "autogyros"  n ( oo2 t ou jh ai1 @ r ou z ) ((pos "Kj$" ) ))
( "automat"  n ( oo1 t @ m a t ) ((pos "K6$" ) ))
( "automata"  n ( oo1 t o1 m @ t @ ) ((pos "Kj%" ) ))
( "automate"  v ( oo1 t @ m ei t ) ((pos "H2%" ) ))
( "automated"  v ( oo1 t @ m ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "automates"  v ( oo1 t @ m ei t s ) ((pos "Ha%" ) ))
( "automatic"  n ( oo2 t @ m a1 t i k ) ((pos "K6%" "OA%" ) ))
( "automatically"  a ( oo2 t @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "automatics"  n ( oo2 t @ m a1 t i k s ) ((pos "Kj%" ) ))
( "automating"  v ( oo1 t @ m ei t i ng ) ((pos "Hb%" ) ))
( "automation"  n ( oo2 t @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "automaton"  n ( oo1 t o1 m @ t @ n ) ((pos "K6%" ) ))
( "automatons"  n ( oo1 t o1 m @ t @ n z ) ((pos "Kj%" ) ))
( "automats"  n ( oo1 t @ m a t s ) ((pos "Kj$" ) ))
( "automobile"  n ( oo1 t @ m @ b ii l ) ((pos "K6%" ) ))
( "automobiles"  n ( oo1 t @ m @ b ii l z ) ((pos "Kj%" ) ))
( "autonomies"  n ( oo1 t o1 n @ m i z ) ((pos "Mj$" ) ))
( "autonomous"  j ( oo1 t o1 n @ m @ s ) ((pos "OA%" ) ))
( "autonomy"  n ( oo1 t o1 n @ m ii ) ((pos "M8%" ) ))
( "autopsies"  n ( oo1 t o p s i z ) ((pos "Kj%" ) ))
( "autopsy"  n ( oo1 t o p s ii ) ((pos "K8%" ) ))
( "autos"  n ( oo1 t ou z ) ((pos "Kj%" ) ))
( "autos-da-f_e"  n ( oo2 t ou z - d aa - f ei1 ) ((pos "Kj$" ) ))
( "autostrada"  n ( oo2 t ou s t r aa1 d @ ) ((pos "K6$" ) ))
( "autostradas"  n ( oo2 t ou s t r aa1 d @ z ) ((pos "Kj%" ) ))
( "autostrade"  n ( au2 t ou s t r aa1 d ei ) ((pos "Kj$" ) ))
( "autumn"  n ( oo1 t @ m ) ((pos "M6%" ) ))
( "autumnal"  j ( oo1 t uh1 m n @ l ) ((pos "OA%" ) ))
( "autumns"  n ( oo1 t @ m z ) ((pos "Mj%" ) ))
( "auxiliaries"  n ( oo1 g z i1 l i@ r i z ) ((pos "Kj%" ) ))
( "auxiliary"  n ( oo1 g z i1 l i@ r ii ) ((pos "K8%" "OA%" ) ))
( "avail"  v ( @1 v ei1 l ) ((pos "J0%" "L@%" ) (subcat "2A" "3A" )))
( "availability"  n ( @1 v ei2 l @ b i1 l i t ii ) ((pos "L@%" ) ))
( "available"  j ( @1 v ei1 l @ b l ) ((pos "OA%" ) ))
( "availed"  v ( @1 v ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "availing"  v ( @1 v ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "avails"  v ( @1 v ei1 l z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "avalanche"  n ( a1 v @ l aa n sh ) ((pos "K6%" ) ))
( "avalanches"  n ( a1 v @ l aa n sh i z ) ((pos "Kj%" ) ))
( "avant-garde"  n ( a2 v o n - g aa1 d ) ((pos "Ki%" ) ))
( "avarice"  n ( a1 v @ r i s ) ((pos "L@%" ) ))
( "avaricious"  j ( a2 v @ r i1 sh @ s ) ((pos "OA%" ) ))
( "avariciously"  a ( a2 v @ r i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "avast"  uh ( @1 v aa1 s t ) ((pos "W-%" ) ))
( "avatar"  n ( a2 v @ t aa1 r ) ((pos "K6$" ) ))
( "avatars"  n ( a2 v @ t aa1 z ) ((pos "Kj$" ) ))
( "avaunt"  uh ( @1 v oo1 n t ) ((pos "W-$" ) ))
( "avenge"  v ( @1 v e1 n jh ) ((pos "H2%" ) (subcat "6A" )))
( "avenged"  v ( @1 v e1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "avenger"  n ( @1 v e1 n jh @ r ) ((pos "K6%" ) ))
( "avengers"  n ( @1 v e1 n jh @ z ) ((pos "Kj%" ) ))
( "avenges"  v ( @1 v e1 n jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "avenging"  v ( @1 v e1 n jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "avenue"  n ( a1 v @ n y uu ) ((pos "K6%" ) ))
( "avenues"  n ( a1 v @ n y uu z ) ((pos "Kj%" ) ))
( "aver"  v ( @1 v @@1 r ) ((pos "H4%" ) (subcat "6A" )))
( "average"  v ( a1 v @ r i jh ) ((pos "J2%" "M6%" "OA%" ) (subcat "2B" )))
( "averaged"  v ( a1 v @ r i jh d ) ((pos "Jc%" "Jd%" ) (subcat "2B" )))
( "averages"  v ( a1 v @ r i jh i z ) ((pos "Ja%" "Mj%" ) (subcat "2B" )))
( "averaging"  v ( a1 v @ r i jh i ng ) ((pos "Jb%" ) (subcat "2B" )))
( "averred"  v ( @1 v @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "averring"  v ( @1 v @@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "avers"  v ( @1 v @@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "averse"  j ( @1 v @@1 s ) ((pos "OA%" ) ))
( "aversion"  n ( @1 v @@1 sh @ n ) ((pos "M6%" ) ))
( "aversions"  n ( @1 v @@1 sh @ n z ) ((pos "Mj%" ) ))
( "avert"  v ( @1 v @@1 t ) ((pos "H0%" ) (subcat "6A" )))
( "averted"  v ( @1 v @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "averting"  v ( @1 v @@1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "averts"  v ( @1 v @@1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "aviaries"  n ( ei1 v i@ r i z ) ((pos "Kj%" ) ))
( "aviary"  n ( ei1 v i@ r ii ) ((pos "K8%" ) ))
( "aviation"  n ( ei2 v i ei1 sh @ n ) ((pos "L@%" ) ))
( "aviator"  n ( ei1 v i ei t @ r ) ((pos "K6%" ) ))
( "aviators"  n ( ei1 v i ei t @ z ) ((pos "Kj%" ) ))
( "avid"  j ( a1 v i d ) ((pos "OA%" ) ))
( "avidity"  n ( @1 v i1 d i t ii ) ((pos "L@%" ) ))
( "avidly"  a ( a1 v i d l ii ) ((pos "Pu%" ) ))
( "avocado"  n ( a2 v @ k aa1 d ou ) ((pos "K6%" ) ))
( "avocados"  n ( a2 v @ k aa1 d ou z ) ((pos "Kj%" ) ))
( "avocation"  n ( a2 v @ k ei1 sh @ n ) ((pos "K6$" ) ))
( "avocations"  n ( a2 v @ k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "avoid"  v ( @1 v oi1 d ) ((pos "H0%" ) (subcat "6A" )))
( "avoidable"  j ( @1 v oi1 d @ b l ) ((pos "OA%" ) ))
( "avoidance"  n ( @1 v oi1 d @ n s ) ((pos "L@%" ) ))
( "avoided"  v ( @1 v oi1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "avoiding"  v ( @1 v oi1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "avoids"  v ( @1 v oi1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "avoirdupois"  n ( a2 v @ d @ p oi1 z ) ((pos "L@$" ) ))
( "avouch"  v ( @1 v au1 ch ) ((pos "J1$" ) (subcat "3A" "6A" )))
( "avouched"  v ( @1 v au1 ch t ) ((pos "Jc$" "Jd$" ) (subcat "3A" "6A" )))
( "avouches"  v ( @1 v au1 ch i z ) ((pos "Ja$" ) (subcat "3A" "6A" )))
( "avouching"  v ( @1 v au1 ch i ng ) ((pos "Jb$" ) (subcat "3A" "6A" )))
( "avow"  v ( @1 v au1 ) ((pos "H0$" ) (subcat "6A" )))
( "avowal"  n ( @1 v ou1 @ l ) ((pos "M6$" ) ))
( "avowals"  n ( @1 v ou1 @ l z ) ((pos "Mj$" ) ))
( "avowed"  v ( @1 v au1 d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "avowedly"  a ( @1 v au1 i d l ii ) ((pos "Pu%" ) ))
( "avowing"  v ( @1 v au1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "avows"  v ( @1 v au1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "avuncular"  j ( @1 v uh1 ng k y u l @ r ) ((pos "OA%" ) ))
( "await"  v ( @1 w ei1 t ) ((pos "H0%" ) (subcat )))
( "awaited"  v ( @1 w ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "awaiting"  v ( @1 w ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "awaits"  v ( @1 w ei1 t s ) ((pos "Ha%" ) (subcat )))
( "awake"  v ( @1 w ei1 k ) ((pos "I5%" "Op%" ) (subcat "2A" "3A" )))
( "awaken"  v ( @1 w ei1 k @ n ) ((pos "H0%" ) (subcat "6A" )))
( "awakened"  v ( @1 w ei1 k @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "awakening"  v ( @1 w ei1 k @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "awakening"  n ( @1 w ei1 k n i ng ) ((pos "K6%" ) ))
( "awakenings"  n ( @1 w ei1 k n i ng z ) ((pos "Kj%" ) ))
( "awakens"  v ( @1 w ei1 k @ n z ) ((pos "Ha%" ) (subcat "6A" )))
( "awakes"  v ( @1 w ei1 k s ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "awaking"  v ( @1 w ei1 k i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "award"  v ( @1 w oo1 d ) ((pos "H0%" "K6%" ) (subcat "6A" "12A" )))
( "awarded"  v ( @1 w oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" )))
( "awarding"  v ( @1 w oo1 d i ng ) ((pos "Hb%" ) (subcat "6A" "12A" )))
( "awards"  v ( @1 w oo1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "12A" )))
( "aware"  j ( @1 w e@1 r ) ((pos "Op%" ) ))
( "awareness"  n ( @1 w e@1 n @ s ) ((pos "L@%" ) ))
( "awash"  j ( @1 w o1 sh ) ((pos "Op%" ) ))
( "away"  j ( @1 w ei1 ) ((pos "OA*" "P+*" ) ))
( "awe"  v ( oo1 ) ((pos "H2%" "L@%" ) (subcat "6A" )))
( "awe-inspiring"  j ( oo1 - i n s p ai r i ng ) ((pos "OA%" ) ))
( "awe-stricken"  j ( oo1 - s t r i k @ n ) ((pos "OA%" ) ))
( "awe-struck"  j ( oo1 - s t r uh k ) ((pos "OA%" ) ))
( "awed"  v ( oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "aweigh"  a ( @1 w ei1 ) ((pos "Pu$" ) ))
( "awes"  v ( oo1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "awesome"  j ( oo1 s @ m ) ((pos "OA%" ) ))
( "awful"  j ( oo1 f @ l ) ((pos "OA%" ) ))
( "awfully"  a ( oo1 f l ii ) ((pos "Pu%" ) ))
( "awhile"  a ( @1 w ai1 l ) ((pos "Pu%" ) ))
( "awing"  v ( oo1 i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "awkward"  j ( oo1 k w @ d ) ((pos "OA%" ) ))
( "awkwardly"  a ( oo1 k w @ d l ii ) ((pos "Pu%" ) ))
( "awkwardness"  n ( oo1 k w @ d n @ s ) ((pos "L@%" ) ))
( "awl"  n ( oo1 l ) ((pos "K6$" ) ))
( "awls"  n ( oo1 l z ) ((pos "Kj$" ) ))
( "awning"  n ( oo1 n i ng ) ((pos "K6%" ) ))
( "awnings"  n ( oo1 n i ng z ) ((pos "Kj%" ) ))
( "awoke"  v ( @1 w ou1 k ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "awoken"  v ( @1 w ou1 k @ n ) ((pos "Id%" ) (subcat "2A" "3A" )))
( "awry"  j ( @1 r ai1 ) ((pos "Op%" "Pu%" ) ))
( "ax"  v ( a1 k s ) ((pos "H1$" "K7$" ) (subcat )))
( "axe"  v ( a1 k s ) ((pos "H2%" "K6%" ) (subcat )))
( "axed"  v ( a1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "axes"  v ( a1 k s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "axes"  n ( a1 k s ii z ) ((pos "Kj%" ) ))
( "axing"  v ( a1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "axiom"  n ( a1 k s i@ m ) ((pos "K6%" ) ))
( "axiomatic"  j ( a2 k s i@ m a1 t i k ) ((pos "OA%" ) ))
( "axiomatically"  a ( a2 k s i@ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "axioms"  n ( a1 k s i@ m z ) ((pos "Kj%" ) ))
( "axis"  n ( a1 k s i s ) ((pos "Ki%" ) ))
( "axle"  n ( a1 k s l ) ((pos "K6%" ) ))
( "axles"  n ( a1 k s l z ) ((pos "Kj%" ) ))
( "ay"  a ( ai1 ) ((pos "Pu$" "W-%" ) ))
( "ayah"  n ( ai1 @ ) ((pos "K6$" ) ))
( "ayahs"  n ( ai1 @ z ) ((pos "Kj$" ) ))
( "aye"  a ( ai1 ) ((pos "Pu$" "W-%" ) ))
( "aye"  a ( ei1 ) ((pos "Pu$" ) ))
( "ayes"  n ( ai1 z ) ((pos "Kj$" ) ))
( "ays"  n ( ai1 z ) ((pos "Kj$" ) ))
( "azalea"  n ( @1 z ei1 l i@ ) ((pos "K6$" ) ))
( "azaleas"  n ( @1 z ei1 l i@ z ) ((pos "Kj$" ) ))
( "azimuth"  n ( a1 z i m @ th ) ((pos "K6$" ) ))
( "azimuths"  n ( a1 z i m @ th s ) ((pos "Kj$" ) ))
( "azure"  n ( a1 zh @ r ) ((pos "K6%" "OA%" ) ))
( "azures"  n ( a1 zh @ z ) ((pos "Kj$" ) ))
( "b"  n ( b ii1 ) ((pos "Ki$" ) ))
( "b's"  n ( b ii1 z ) ((pos "Kj$" ) ))
( "b^ete noire"  n ( b e2 t - n w aa1 r ) ((pos "Ki%" ) ))
( "b^etes noires"  n ( b e2 t - n w aa1 ) ((pos "Kj%" ) ))
( "baa"  v ( b aa1 ) ((pos "I0%" "K6%" ) ))
( "baa'd"  v ( b aa1 d ) ((pos "Ic$" ) ))
( "baa-lamb"  n ( b aa1 - l a m ) ((pos "K6%" ) ))
( "baa-lambs"  n ( b aa1 - l a m z ) ((pos "Kj%" ) ))
( "baaed"  v ( b aa1 d ) ((pos "Ic$" "Id$" ) ))
( "baaing"  v ( b aa1 i ng ) ((pos "Ib%" ) ))
( "baas"  n ( b aa1 s ) ((pos "Ki$" ) ))
( "baas"  v ( b aa1 z ) ((pos "Ia$" "Kj$" ) ))
( "babble"  v ( b a1 b l ) ((pos "J2%" "L@%" ) (subcat "2A" "2B" "2C" "6A" )))
( "babbled"  v ( b a1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" )))
( "babbler"  n ( b a1 b l @ r ) ((pos "K6%" ) ))
( "babblers"  n ( b a1 b l @ z ) ((pos "Kj%" ) ))
( "babbles"  v ( b a1 b l z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" )))
( "babbling"  v ( b a1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" )))
( "babe"  n ( b ei1 b ) ((pos "K6%" ) ))
( "babel"  n ( b ei1 b l ) ((pos "K6%" ) ))
( "babels"  n ( b ei1 b l z ) ((pos "Kj$" ) ))
( "babes"  n ( b ei1 b z ) ((pos "Kj%" ) ))
( "babied"  v ( b ei1 b i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "babies"  v ( b ei1 b i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "baboo"  n ( b aa1 b uu ) ((pos "K6$" ) ))
( "baboon"  n ( b @1 b uu1 n ) ((pos "K6%" ) ))
( "baboons"  n ( b @1 b uu1 n z ) ((pos "Kj%" ) ))
( "baboos"  n ( b aa1 b uu z ) ((pos "Kj$" ) ))
( "babu"  n ( b aa1 b uu ) ((pos "K6$" ) ))
( "babus"  n ( b aa1 b uu z ) ((pos "Kj$" ) ))
( "baby"  v ( b ei1 b ii ) ((pos "H3%" "K8%" ) (subcat )))
( "baby-faced"  j ( b ei1 b i - f ei s t ) ((pos "OA%" ) ))
( "baby-farmer"  n ( b ei1 b i - f aa m @ r ) ((pos "K6$" ) ))
( "baby-farmers"  n ( b ei1 b i - f aa m @ z ) ((pos "Kj$" ) ))
( "baby-minder"  n ( b ei1 b i - m ai n d @ r ) ((pos "K6%" ) ))
( "baby-minders"  n ( b ei1 b i - m ai n d @ z ) ((pos "Kj%" ) ))
( "baby-talk"  n ( b ei1 b i - t oo k ) ((pos "L@%" ) ))
( "babyhood"  n ( b ei1 b i h u d ) ((pos "Ki%" ) ))
( "babying"  v ( b ei1 b i i ng ) ((pos "Hb%" ) (subcat )))
( "babyish"  j ( b ei1 b i i sh ) ((pos "OA%" ) ))
( "babysat"  v ( b ei1 b i s a t ) ((pos "Ic%" "Id%" ) ))
( "babysit"  v ( b ei1 b i s i t ) ((pos "I5%" ) ))
( "babysits"  v ( b ei1 b i s i t s ) ((pos "Ia%" ) ))
( "babysitter"  n ( b ei1 b i s i t @ r ) ((pos "K6%" ) ))
( "babysitters"  n ( b ei1 b i s i t @ z ) ((pos "Kj%" ) ))
( "babysitting"  v ( b ei1 b i s i t i ng ) ((pos "Ib%" "L@%" ) ))
( "baccalaureate"  n ( b a2 k @ l oo1 r i@ t ) ((pos "K6%" ) ))
( "baccalaureates"  n ( b a2 k @ l oo1 r i@ t s ) ((pos "Kj%" ) ))
( "baccarat"  n ( b a1 k @ r aa ) ((pos "L@$" ) ))
( "bacchanal"  n ( b a1 k @ n @ l ) ((pos "K6%" "OA$" ) ))
( "bacchanalian"  j ( b a2 k @ n ei1 l i@ n ) ((pos "OA%" ) ))
( "bacchanals"  n ( b a1 k @ n @ l z ) ((pos "Kj%" ) ))
( "baccy"  n ( b a1 k ii ) ((pos "L@%" ) ))
( "bachelor"  n ( b a1 ch @ l @ r ) ((pos "K6%" ) ))
( "bachelors"  n ( b a1 ch @ l @ z ) ((pos "Kj%" ) ))
( "bacilli"  n ( b @1 s i1 l ai ) ((pos "Kj%" ) ))
( "bacillus"  n ( b @1 s i1 l @ s ) ((pos "Ki%" ) ))
( "back"  v ( b a1 k ) ((pos "J0%" "K6%" "P+*" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "back-blocks"  n ( b a1 k - b l o k s ) ((pos "Kj$" ) ))
( "back-breaking"  j ( b a1 k - b r ei k i ng ) ((pos "OA%" ) ))
( "back-down"  n ( b a1 k - d au n ) ((pos "K6%" ) ))
( "back-downs"  n ( b a1 k - d au n z ) ((pos "Kj$" ) ))
( "back-formation"  n ( b a1 k - f oo m ei2 sh @ n ) ((pos "M6$" ) ))
( "back-formations"  n ( b a1 k - f oo m ei2 sh @ n z ) ((pos "Mj$" ) ))
( "back-strokes"  n ( b a1 k - s t r ou k s ) ((pos "Mj%" ) ))
( "back-up"  n ( b a1 k - uh p ) ((pos "K6%" ) ))
( "back-ups"  n ( b a1 k - uh p s ) ((pos "Kj%" ) ))
( "backache"  n ( b a1 k ei k ) ((pos "M6%" ) ))
( "backaches"  n ( b a1 k ei k s ) ((pos "Mj%" ) ))
( "backband"  n ( b a1 k b a n d ) ((pos "K6$" ) ))
( "backbands"  n ( b a1 k b a n d z ) ((pos "Kj$" ) ))
( "backbench"  n ( b a1 k b e n ch ) ((pos "K7%" ) ))
( "backbencher"  n ( b a1 k b e1 n ch @ r ) ((pos "K6%" ) ))
( "backbenchers"  n ( b a1 k b e1 n ch @ z ) ((pos "Kj%" ) ))
( "backbenches"  n ( b a1 k b e n ch i z ) ((pos "Kj%" ) ))
( "backbit"  v ( b a1 k b i t ) ((pos "Jc$" ) ))
( "backbite"  v ( b a1 k b ai t ) ((pos "J5%" ) ))
( "backbiter"  n ( b a1 k b ai t @ r ) ((pos "K6%" ) ))
( "backbiters"  n ( b a1 k b ai t @ z ) ((pos "Kj%" ) ))
( "backbites"  v ( b a1 k b ai t s ) ((pos "Ja%" ) ))
( "backbiting"  v ( b a1 k b ai t i ng ) ((pos "Jb%" ) ))
( "backbitten"  v ( b a1 k b i t @ n ) ((pos "Jd$" ) ))
( "backboard"  n ( b a1 k b oo d ) ((pos "K6$" ) ))
( "backboards"  n ( b a1 k b oo d z ) ((pos "Kj$" ) ))
( "backbone"  n ( b a1 k b ou n ) ((pos "M6%" ) ))
( "backbones"  n ( b a1 k b ou n z ) ((pos "Mj%" ) ))
( "backchat"  n ( b a1 k ch a t ) ((pos "L@%" ) ))
( "backcloth"  n ( b a1 k - k l o th ) ((pos "K6%" ) ))
( "backcloths"  n ( b a1 k - k l o th s ) ((pos "Kj%" ) ))
( "backdate"  v ( b a1 k d ei1 t ) ((pos "H2%" ) ))
( "backdated"  v ( b a1 k d ei1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "backdates"  v ( b a1 k d ei1 t s ) ((pos "Ha%" ) ))
( "backdating"  v ( b a1 k d ei1 t i ng ) ((pos "Hb%" ) ))
( "backdoor"  n ( b a1 k d oo1 r ) ((pos "K6%" ) ))
( "backdoors"  n ( b a1 k d oo1 z ) ((pos "Kj%" ) ))
( "backdrop"  n ( b a1 k d r o p ) ((pos "K6%" ) ))
( "backdrops"  n ( b a1 k d r o p s ) ((pos "Kj%" ) ))
( "backed"  v ( b a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "backer"  n ( b a1 k @ r ) ((pos "K6%" ) ))
( "backers"  n ( b a1 k @ z ) ((pos "Kj%" ) ))
( "backfire"  v ( b a1 k f ai1 @ r ) ((pos "I2%" "K6%" ) ))
( "backfired"  v ( b a1 k f ai1 @ d ) ((pos "Ic%" "Id%" ) ))
( "backfires"  v ( b a1 k f ai1 @ z ) ((pos "Ia%" "Kj%" ) ))
( "backfiring"  v ( b a1 k f ai1 @ r i ng ) ((pos "Ib%" ) ))
( "backgammon"  n ( b a1 k g a1 m @ n ) ((pos "L@%" ) ))
( "background"  n ( b a1 k g r au n d ) ((pos "K6%" ) ))
( "backgrounds"  n ( b a1 k g r au n d z ) ((pos "Kj%" ) ))
( "backhand"  j ( b a1 k h a n d ) ((pos "OA%" ) ))
( "backhanded"  j ( b a1 k h a n d i d ) ((pos "OA%" ) ))
( "backing"  v ( b a1 k i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "backings"  n ( b a1 k i ng z ) ((pos "Mj%" ) ))
( "backlash"  n ( b a1 k l a sh ) ((pos "L@%" ) ))
( "backless"  j ( b a1 k l @ s ) ((pos "OA%" ) ))
( "backlog"  n ( b a1 k l o g ) ((pos "K6%" ) ))
( "backlogs"  n ( b a1 k l o g z ) ((pos "Kj$" ) ))
( "backmost"  j ( b a1 k m ou s t ) ((pos "OA$" ) ))
( "backpedal"  v ( b a1 k p e1 d @ l ) ((pos "I4%" ) ))
( "backpedalled"  v ( b a1 k p e1 d @ l d ) ((pos "Ic%" "Id%" ) ))
( "backpedalling"  v ( b a1 k p e1 d @ l i ng ) ((pos "Ib%" ) ))
( "backpedals"  v ( b a1 k p e1 d @ l z ) ((pos "Ia%" ) ))
( "backroom"  n ( b a1 k r uu m ) ((pos "K6%" ) ))
( "backrooms"  n ( b a1 k r uu m z ) ((pos "Kj%" ) ))
( "backs"  v ( b a1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "backscratcher"  n ( b a1 k s k r a ch @ r ) ((pos "K6%" ) ))
( "backscratchers"  n ( b a1 k s k r a ch @ z ) ((pos "Kj%" ) ))
( "backseat"  n ( b a1 k s ii1 t ) ((pos "K6%" ) ))
( "backseats"  n ( b a1 k s ii1 t s ) ((pos "Kj%" ) ))
( "backsheesh"  n ( b a1 k sh ii sh ) ((pos "M9%" ) ))
( "backside"  n ( b a1 k s ai d ) ((pos "K6%" ) ))
( "backsides"  n ( b a1 k s ai d z ) ((pos "Kj%" ) ))
( "backslid"  v ( b a1 k s l i1 d ) ((pos "Ic$" "Id$" ) (subcat )))
( "backslide"  v ( b a1 k s l ai1 d ) ((pos "I5%" ) (subcat )))
( "backslides"  v ( b a1 k s l ai1 d z ) ((pos "Ia%" ) (subcat )))
( "backsliding"  v ( b a1 k s l ai1 d i ng ) ((pos "Ib%" ) (subcat )))
( "backspace"  v ( b a1 k s p ei s ) ((pos "I2%" ) ))
( "backspaced"  v ( b a1 k s p ei s t ) ((pos "Ic%" "Id%" ) ))
( "backspaces"  v ( b a1 k s p ei s i z ) ((pos "Ia%" ) ))
( "backspacing"  v ( b a1 k s p ei s i ng ) ((pos "Ib%" ) ))
( "backstage"  a ( b a1 k s t ei jh ) ((pos "Pu%" ) ))
( "backstair"  j ( b a1 k s t e@ r ) ((pos "OA$" ) ))
( "backstairs"  n ( b a1 k s t e@ z ) ((pos "Kj%" ) ))
( "backstays"  n ( b a1 k s t ei z ) ((pos "Kj$" ) ))
( "backstroke"  n ( b a1 k s t r ou k ) ((pos "M6%" ) ))
( "backsword"  n ( b a1 k s oo d ) ((pos "K6$" ) ))
( "backswords"  n ( b a1 k s oo d z ) ((pos "Kj$" ) ))
( "backtalk"  n ( b a1 k t oo k ) ((pos "L@%" ) ))
( "backward"  j ( b a1 k w @ d ) ((pos "OA%" "Pu%" ) ))
( "backwards"  j ( b a1 k w @ d z ) ((pos "OA%" "Pu%" ) ))
( "backwash"  n ( b a1 k w o sh ) ((pos "L@%" ) ))
( "backwater"  n ( b a1 k w oo t @ r ) ((pos "K6%" ) ))
( "backwaters"  n ( b a1 k w oo t @ z ) ((pos "Kj%" ) ))
( "backwoods"  n ( b a1 k w u d z ) ((pos "Kj%" ) ))
( "backwoodsman"  n ( b a1 k w u1 d z m @ n ) ((pos "Ki%" ) ))
( "backwoodsmen"  n ( b a1 k w u1 d z m @ n ) ((pos "Kj%" ) ))
( "bacon"  n ( b ei1 k @ n ) ((pos "L@%" ) ))
( "bacteria"  n ( b a1 k t i@1 r i@ ) ((pos "Kj%" ) ))
( "bacterial"  j ( b a1 k t i@1 r i@ l ) ((pos "OA%" ) ))
( "bacteriologist"  n ( b a1 k t i@2 r i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "bacteriologists"  n ( b a1 k t i@2 r i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "bacteriology"  n ( b a1 k t i@2 r i o1 l @ jh ii ) ((pos "L@%" ) ))
( "bacterium"  n ( b a1 k t i@1 r i@ m ) ((pos "Ki%" ) ))
( "bad"  n ( b a1 d ) ((pos "L@%" "OE%" ) ))
( "bade"  v ( b a1 d ) ((pos "Jc%" ) (subcat "12A" "13A" "17" )))
( "badge"  n ( b a1 jh ) ((pos "K6%" ) ))
( "badger"  v ( b a1 jh @ r ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "badgered"  v ( b a1 jh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "badgering"  v ( b a1 jh @ r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "badgers"  v ( b a1 jh @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "badges"  n ( b a1 jh i z ) ((pos "Kj%" ) ))
( "badinage"  n ( b a1 d i n aa zh ) ((pos "L@%" ) ))
( "badlands"  n ( b a1 d l a n d z ) ((pos "Kj%" ) ))
( "badly"  a ( b a1 d l ii ) ((pos "Pu%" ) ))
( "badly-behaved"  j ( b a2 d l i - b i h ei1 v d ) ((pos "OA%" ) ))
( "badminton"  n ( b a1 d m i n t @ n ) ((pos "L@%" ) ))
( "badness"  n ( b a1 d n @ s ) ((pos "L@%" ) ))
( "baffle"  v ( b a1 f l ) ((pos "H2%" "K6%" ) (subcat )))
( "baffled"  v ( b a1 f l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "baffles"  v ( b a1 f l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "baffling"  v ( b a1 f l i ng ) ((pos "Hb%" ) (subcat )))
( "bag"  v ( b a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "bagatelle"  n ( b a2 g @ t e1 l ) ((pos "M6%" ) ))
( "bagatelles"  n ( b a2 g @ t e1 l z ) ((pos "Mj%" ) ))
( "baggage"  n ( b a1 g i jh ) ((pos "L@%" ) ))
( "bagged"  v ( b a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bagging"  v ( b a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "baggy"  j ( b a1 g ii ) ((pos "OA%" ) ))
( "bagnio"  n ( b aa1 n i@ @ ) ((pos "K6$" ) ))
( "bagnios"  n ( b aa1 n i@ u z ) ((pos "Kj$" ) ))
( "bagpipe"  n ( b a1 g p ai p ) ((pos "K6$" ) ))
( "bagpipes"  n ( b a1 g p ai p s ) ((pos "Kj%" ) ))
( "bags"  v ( b a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "bah"  uh ( b aa1 ) ((pos "W-%" ) ))
( "bail"  v ( b ei1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "bailed"  v ( b ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bailee"  n ( b ei2 l ii1 ) ((pos "K6$" ) ))
( "bailees"  n ( b ei2 l ii1 z ) ((pos "Kj$" ) ))
( "bailey"  n ( b ei1 l ii ) ((pos "K6%" ) ))
( "baileys"  n ( b ei1 l i z ) ((pos "Kj%" ) ))
( "bailiff"  n ( b ei1 l i f ) ((pos "K6%" ) ))
( "bailiffs"  n ( b ei1 l i f s ) ((pos "Kj%" ) ))
( "bailing"  v ( b ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "bailment"  n ( b ei1 l m @ n t ) ((pos "K6$" ) ))
( "bailments"  n ( b ei1 l m @ n t s ) ((pos "Kj$" ) ))
( "bailor"  n ( b ei1 l oo r ) ((pos "K6$" ) ))
( "bailors"  n ( b ei1 l oo z ) ((pos "Kj$" ) ))
( "bails"  v ( b ei1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "bairn"  n ( b e@1 n ) ((pos "K6%" ) ))
( "bairns"  n ( b e@1 n z ) ((pos "Kj%" ) ))
( "bait"  v ( b ei1 t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "baited"  v ( b ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "baiting"  v ( b ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "baits"  v ( b ei1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "baize"  n ( b ei1 z ) ((pos "L@%" ) ))
( "bake"  v ( b ei1 k ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "baked"  v ( b ei1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bakelite"  n ( b ei1 k @ l ai t ) ((pos "L@%" ) ))
( "baker"  n ( b ei1 k @ r ) ((pos "K6%" ) ))
( "bakeries"  n ( b ei1 k @ r i z ) ((pos "Kj%" ) ))
( "bakers"  n ( b ei1 k @ z ) ((pos "Kj%" ) ))
( "bakery"  n ( b ei1 k @ r ii ) ((pos "K8%" ) ))
( "bakes"  v ( b ei1 k s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "baking"  v ( b ei1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "baking-hot"  j ( b ei2 k i ng - h o1 t ) ((pos "OA%" ) ))
( "baking-powder"  n ( b ei1 k i ng - p ou d @ r ) ((pos "K6%" ) ))
( "baking-powders"  n ( b ei1 k i ng - p ou d @ z ) ((pos "Kj%" ) ))
( "baksheesh"  n ( b a1 k sh ii sh ) ((pos "M9%" ) ))
( "balaclava"  n ( b a2 l @ k l aa1 v @ ) ((pos "K6%" ) ))
( "balaclavas"  n ( b a2 l @ k l aa1 v @ z ) ((pos "Kj%" ) ))
( "balalaika"  n ( b a2 l @ l ai1 k @ ) ((pos "K6%" ) ))
( "balalaikas"  n ( b a2 l @ l ai1 k @ z ) ((pos "Kj%" ) ))
( "balance"  v ( b a1 l @ n s ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" "14" )))
( "balance-sheet"  n ( b a1 l @ n s - sh ii t ) ((pos "K6%" ) ))
( "balance-sheets"  n ( b a1 l @ n s - sh ii t s ) ((pos "Kj%" ) ))
( "balance-wheel"  n ( b a1 l @ n s - w ii l ) ((pos "K6%" ) ))
( "balance-wheels"  n ( b a1 l @ n s - w ii l z ) ((pos "Kj%" ) ))
( "balanced"  v ( b a1 l @ n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "balances"  v ( b a1 l @ n s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "14" )))
( "balancing"  v ( b a1 l @ n s i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "balconied"  j ( b a1 l k @ n i d ) ((pos "OA%" ) ))
( "balconies"  n ( b a1 l k @ n i z ) ((pos "Kj%" ) ))
( "balcony"  n ( b a1 l k @ n ii ) ((pos "K8%" ) ))
( "bald"  j ( b oo1 l d ) ((pos "OC%" ) ))
( "bald-head"  n ( b oo1 l d - h e d ) ((pos "K6$" ) ))
( "bald-heads"  n ( b oo1 l d - h e d z ) ((pos "Kj$" ) ))
( "bald-pate"  n ( b oo1 l d - p ei t ) ((pos "K6%" ) ))
( "bald-pates"  n ( b oo1 l d - p ei t s ) ((pos "Kj$" ) ))
( "balder"  j ( b oo1 l d @ r ) ((pos "Or%" ) ))
( "balderdash"  n ( b oo1 l d @ d a sh ) ((pos "L@%" ) ))
( "baldest"  j ( b oo1 l d i s t ) ((pos "Os%" ) ))
( "baldly"  a ( b oo1 l d l ii ) ((pos "Pu%" ) ))
( "baldness"  n ( b oo1 l d n @ s ) ((pos "L@%" ) ))
( "baldric"  n ( b oo1 l d r i k ) ((pos "K6$" ) ))
( "baldrics"  n ( b oo1 l d r i k s ) ((pos "Kj$" ) ))
( "bale"  v ( b ei1 l ) ((pos "H2%" "K6%" ) (subcat )))
( "baled"  v ( b ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "baleful"  j ( b ei1 l f @ l ) ((pos "OA%" ) ))
( "balefully"  a ( b ei1 l f @ l ii ) ((pos "Pu%" ) ))
( "bales"  v ( b ei1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "baling"  v ( b ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "balk"  v ( b oo1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "balked"  v ( b oo1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "balking"  v ( b oo1 k i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "balks"  v ( b oo1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "ball"  v ( b oo1 l ) ((pos "J0$" "K6%" "W-$" ) ))
( "ball-cartridge"  n ( b oo1 l - k aa1 t r i jh ) ((pos "K6$" ) ))
( "ball-cartridges"  n ( b oo1 l - k aa1 t r i jh i z ) ((pos "Kj$" ) ))
( "ball-dress"  n ( b oo1 l - d r e s ) ((pos "K7%" ) ))
( "ball-dresses"  n ( b oo1 l - d r e s i z ) ((pos "Kj%" ) ))
( "ballad"  n ( b a1 l @ d ) ((pos "K6%" ) ))
( "ballade"  n ( b a1 l aa1 d ) ((pos "K6$" ) ))
( "ballades"  n ( b a1 l aa1 d z ) ((pos "Kj$" ) ))
( "ballads"  n ( b a1 l @ d z ) ((pos "Kj%" ) ))
( "ballast"  v ( b a1 l @ s t ) ((pos "H0%" "L@%" ) (subcat )))
( "ballasted"  v ( b a1 l @ s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ballasting"  v ( b a1 l @ s t i ng ) ((pos "Hb%" ) (subcat )))
( "ballasts"  v ( b a1 l @ s t s ) ((pos "Ha%" ) (subcat )))
( "ballbearing"  n ( b oo1 l b e@1 r i ng ) ((pos "K6%" ) ))
( "ballbearings"  n ( b oo1 l b e@1 r i ng z ) ((pos "Kj%" ) ))
( "ballcock"  n ( b oo1 l k o k ) ((pos "K6%" ) ))
( "ballcocks"  n ( b oo1 l k o k s ) ((pos "Kj%" ) ))
( "balled"  v ( b oo1 l d ) ((pos "Jc$" "Jd$" ) ))
( "ballerina"  n ( b a2 l @ r ii1 n @ ) ((pos "K6%" ) ))
( "ballerinas"  n ( b a2 l @ r ii1 n @ z ) ((pos "Kj%" ) ))
( "ballet"  n ( b a1 l ei ) ((pos "M6%" ) ))
( "ballet-dancer"  n ( b a1 l i - d aa n s @ r ) ((pos "K6%" ) ))
( "ballet-dancers"  n ( b a1 l i - d aa n s @ z ) ((pos "Kj%" ) ))
( "ballet-skirt"  n ( b a1 l i - s k @@ t ) ((pos "K6%" ) ))
( "ballet-skirts"  n ( b a1 l i - s k @@ t s ) ((pos "Kj%" ) ))
( "ballets"  n ( b a1 l ei z ) ((pos "Mj%" ) ))
( "balling"  v ( b oo1 l i ng ) ((pos "Jb$" ) ))
( "ballistic"  j ( b @1 l i1 s t i k ) ((pos "OA%" ) ))
( "ballistics"  n ( b @1 l i1 s t i k s ) ((pos "Lk%" ) ))
( "ballock"  n ( b o1 l @ k ) ((pos "K6$" ) ))
( "ballocks"  n ( b o1 l @ k s ) ((pos "Kj$" ) ))
( "balloon"  v ( b @1 l uu1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "ballooned"  v ( b @1 l uu1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "ballooning"  v ( b @1 l uu1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "balloonist"  n ( b @1 l uu1 n i s t ) ((pos "K6%" ) ))
( "balloonists"  n ( b @1 l uu1 n i s t s ) ((pos "Kj%" ) ))
( "balloons"  v ( b @1 l uu1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "ballot"  v ( b a1 l @ t ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "ballot-box"  n ( b a1 l @ t - b o k s ) ((pos "K7%" ) ))
( "ballot-boxes"  n ( b a1 l @ t - b o k s i z ) ((pos "Kj%" ) ))
( "balloted"  v ( b a1 l @ t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "balloting"  v ( b a1 l @ t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "ballots"  v ( b a1 l @ t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "ballpen"  n ( b oo1 l p e n ) ((pos "K6%" ) ))
( "ballpens"  n ( b oo1 l p e n z ) ((pos "Kj%" ) ))
( "ballpoint"  n ( b oo1 l p oi n t ) ((pos "K6%" ) ))
( "ballpoint-pen"  n ( b aa2 l p oi n t - p e1 n ) ((pos "K6%" ) ))
( "ballpoint-pens"  n ( b aa2 l p oi n t - p e1 n z ) ((pos "Kj%" ) ))
( "ballpoints"  n ( b oo1 l p oi n t s ) ((pos "Kj%" ) ))
( "ballroom"  n ( b oo1 l r uu m ) ((pos "K6%" ) ))
( "ballrooms"  n ( b oo1 l r uu m z ) ((pos "Kj%" ) ))
( "balls"  v ( b oo1 l z ) ((pos "Ja$" "Kj%" "W-%" ) ))
( "balls-up"  n ( b aa1 l z - uh p ) ((pos "K6%" ) ))
( "balls-ups"  n ( b aa1 l z - uh p s ) ((pos "Kj%" ) ))
( "bally"  j ( b a1 l ii ) ((pos "OA$" "Pu$" ) ))
( "ballyhoo"  n ( b a2 l i h uu1 ) ((pos "L@%" ) ))
( "balm"  n ( b aa1 m ) ((pos "L@%" ) ))
( "balmier"  j ( b aa1 m i@ r ) ((pos "Or%" ) ))
( "balmiest"  j ( b aa1 m i i s t ) ((pos "Os%" ) ))
( "balmy"  j ( b aa1 m ii ) ((pos "OD%" ) ))
( "baloney"  n ( b @1 l ou1 n ii ) ((pos "L@%" ) ))
( "balsa"  n ( b oo1 l s @ ) ((pos "M6%" ) ))
( "balsam"  n ( b oo1 l s @ m ) ((pos "K6%" ) ))
( "balsams"  n ( b oo1 l s @ m z ) ((pos "Kj$" ) ))
( "balsas"  n ( b oo1 l s @ z ) ((pos "Mj$" ) ))
( "baluster"  n ( b a1 l @ s t @ r ) ((pos "K6$" ) ))
( "balusters"  n ( b a1 l @ s t @ z ) ((pos "Kj$" ) ))
( "balustrade"  n ( b a2 l @ s t r ei1 d ) ((pos "K6%" ) ))
( "balustrades"  n ( b a2 l @ s t r ei1 d z ) ((pos "Kj%" ) ))
( "bambino"  n ( b a1 m b ii1 n ou ) ((pos "K6%" ) ))
( "bambinos"  n ( b a1 m b ii1 n ou z ) ((pos "Kj%" ) ))
( "bamboo"  n ( b a1 m b uu1 ) ((pos "M6%" ) ))
( "bamboos"  n ( b a1 m b uu1 z ) ((pos "Mj%" ) ))
( "bamboozle"  v ( b a1 m b uu1 z l ) ((pos "H2%" ) (subcat "6A" )))
( "bamboozled"  v ( b a1 m b uu1 z l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bamboozles"  v ( b a1 m b uu1 z l z ) ((pos "Ha%" ) (subcat "6A" )))
( "bamboozling"  v ( b a1 m b uu1 z l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ban"  v ( b a1 n ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "banal"  j ( b @1 n aa1 l ) ((pos "OA%" ) ))
( "banalities"  n ( b @1 n a1 l i t i z ) ((pos "Mj%" ) ))
( "banality"  n ( b @1 n a1 l i t ii ) ((pos "M8%" ) ))
( "banana"  n ( b @1 n aa1 n @ ) ((pos "K6%" ) ))
( "bananas"  n ( b @1 n aa1 n @ z ) ((pos "Kj%" ) ))
( "band"  v ( b a1 n d ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "14" )))
( "band-saw"  n ( b a1 n d - s oo ) ((pos "K6$" ) ))
( "band-saws"  n ( b a1 n d - s oo z ) ((pos "Kj$" ) ))
( "bandage"  v ( b a1 n d i jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "bandaged"  v ( b a1 n d i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bandages"  v ( b a1 n d i jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "bandaging"  v ( b a1 n d i jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bandanna"  n ( b a1 n d a1 n @ ) ((pos "K6$" ) ))
( "bandannas"  n ( b a1 n d a1 n @ z ) ((pos "Kj$" ) ))
( "bandbox"  n ( b a1 n d b o k s ) ((pos "K7$" ) ))
( "bandboxes"  n ( b a1 n d b o k s i z ) ((pos "Kj$" ) ))
( "bandeau"  n ( b a1 n d ou ) ((pos "Ki$" ) ))
( "bandeaux"  n ( b a1 n d ou z ) ((pos "Kj$" ) ))
( "banded"  v ( b a1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" )))
( "bandied"  v ( b a1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "bandier"  j ( b a1 n d i@ r ) ((pos "Or$" ) ))
( "bandies"  v ( b a1 n d i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "bandiest"  j ( b a1 n d i i s t ) ((pos "Os%" ) ))
( "banding"  v ( b a1 n d i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" )))
( "bandit"  n ( b a1 n d i t ) ((pos "K6%" ) ))
( "banditry"  n ( b a1 n d i t r ii ) ((pos "L@%" ) ))
( "bandits"  n ( b a1 n d i t s ) ((pos "Kj%" ) ))
( "bandleader"  n ( b a1 n d l ii d @ r ) ((pos "K6%" ) ))
( "bandleaders"  n ( b a1 n d l ii d @ z ) ((pos "Kj%" ) ))
( "bandmaster"  n ( b a1 n d m aa s t @ r ) ((pos "K6%" ) ))
( "bandmasters"  n ( b a1 n d m aa s t @ z ) ((pos "Kj%" ) ))
( "bandoleer"  n ( b a2 n d @ l i@1 r ) ((pos "K6$" ) ))
( "bandoleers"  n ( b a2 n d @ l i@1 z ) ((pos "Kj$" ) ))
( "bandolier"  n ( b a2 n d @ l i@1 r ) ((pos "K6$" ) ))
( "bandoliers"  n ( b a2 n d @ l i@1 z ) ((pos "Kj$" ) ))
( "bands"  v ( b a1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "14" )))
( "bandsman"  n ( b a1 n d z m @ n ) ((pos "Ki%" ) ))
( "bandsmen"  n ( b a1 n d z m @ n ) ((pos "Kj%" ) ))
( "bandstand"  n ( b a1 n d s t a n d ) ((pos "K6%" ) ))
( "bandstands"  n ( b a1 n d s t a n d z ) ((pos "Kj%" ) ))
( "bandwagon"  n ( b a1 n d w a g @ n ) ((pos "K6%" ) ))
( "bandwagons"  n ( b a1 n d w a g @ n z ) ((pos "Kj$" ) ))
( "bandy"  v ( b a1 n d ii ) ((pos "H3%" "OD%" ) (subcat "6A" "14" )))
( "bandy-legged"  j ( b a2 n d i - l e1 g i d ) ((pos "OA%" ) ))
( "bandying"  v ( b a1 n d i i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "bane"  n ( b ei1 n ) ((pos "L@%" ) ))
( "baneful"  j ( b ei1 n f @ l ) ((pos "OA$" ) ))
( "banefully"  a ( b ei1 n f @ l ii ) ((pos "Pu$" ) ))
( "bang"  v ( b a1 ng ) ((pos "J0%" "M6%" "Pu%" "W-%" ) (subcat "2A" "2C" "6A" )))
( "banged"  v ( b a1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "banger"  n ( b a1 ng @ r ) ((pos "K6%" ) ))
( "bangers"  n ( b a1 ng @ z ) ((pos "Kj%" ) ))
( "banging"  v ( b a1 ng i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "bangle"  n ( b a1 ng g l ) ((pos "K6%" ) ))
( "bangles"  n ( b a1 ng g l z ) ((pos "Kj%" ) ))
( "bangs"  v ( b a1 ng z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "banian"  n ( b a1 n y @ n ) ((pos "K6$" ) ))
( "banian-tree"  n ( b a1 n y @ n - t r ii ) ((pos "K6$" ) ))
( "banian-trees"  n ( b a1 n y @ n - t r ii z ) ((pos "Kj$" ) ))
( "banians"  n ( b a1 n y @ n z ) ((pos "Kj$" ) ))
( "banish"  v ( b a1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "banished"  v ( b a1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "banishes"  v ( b a1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "banishing"  v ( b a1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "banishment"  n ( b a1 n i sh m @ n t ) ((pos "L@%" ) ))
( "banister"  n ( b a1 n i s t @ r ) ((pos "K6%" ) ))
( "banisters"  n ( b a1 n i s t @ z ) ((pos "Kj%" ) ))
( "banjo"  n ( b a1 n jh ou ) ((pos "K6%" ) ))
( "banjoes"  n ( b a1 n jh ou z ) ((pos "Kj%" ) ))
( "banjos"  n ( b a1 n jh ou z ) ((pos "Kj%" ) ))
( "bank"  v ( b a1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bank-bill"  n ( b a1 ng k - b i l ) ((pos "K6$" ) ))
( "bank-bills"  n ( b a1 ng k - b i l z ) ((pos "Kj$" ) ))
( "bank-book"  n ( b a1 ng k - b u k ) ((pos "K6%" ) ))
( "bank-books"  n ( b a1 ng k - b u k s ) ((pos "Kj%" ) ))
( "bank-draft"  n ( b a1 ng k - d r aa f t ) ((pos "K6%" ) ))
( "bank-drafts"  n ( b a1 ng k - d r aa f t s ) ((pos "Kj%" ) ))
( "bank-rate"  n ( b a1 ng k - r ei t ) ((pos "K6%" ) ))
( "bank-rates"  n ( b a1 ng k - r ei t s ) ((pos "Kj%" ) ))
( "banked"  v ( b a1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "banker"  n ( b a1 ng k @ r ) ((pos "K6%" ) ))
( "bankers"  n ( b a1 ng k @ z ) ((pos "Kj%" ) ))
( "banking"  v ( b a1 ng k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" )))
( "banknote"  n ( b a1 ng k n ou t ) ((pos "K6%" ) ))
( "banknotes"  n ( b a1 ng k n ou t s ) ((pos "Kj%" ) ))
( "bankroll"  n ( b a1 ng k r ou l ) ((pos "K6%" ) ))
( "bankrolls"  n ( b a1 ng k r ou l z ) ((pos "Kj%" ) ))
( "bankrupt"  v ( b a1 ng k r uh p t ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "bankruptcies"  n ( b a1 ng k r @ p s i z ) ((pos "Mj%" ) ))
( "bankruptcy"  n ( b a1 ng k r @ p s ii ) ((pos "M8%" ) ))
( "bankrupted"  v ( b a1 ng k r uh p t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bankrupting"  v ( b a1 ng k r uh p t i ng ) ((pos "Hb%" ) (subcat )))
( "bankrupts"  v ( b a1 ng k r uh p t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "banks"  v ( b a1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "banned"  v ( b a1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "banner"  n ( b a1 n @ r ) ((pos "K6%" ) ))
( "banners"  n ( b a1 n @ z ) ((pos "Kj%" ) ))
( "banning"  v ( b a1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "banning-order"  n ( b a1 n i ng - oo d @ r ) ((pos "K6%" ) ))
( "banning-orders"  n ( b a1 n i ng - oo d @ z ) ((pos "Kj%" ) ))
( "bannister"  n ( b a1 n i s t @ r ) ((pos "K6$" ) ))
( "bannisters"  n ( b a1 n i s t @ z ) ((pos "Kj$" ) ))
( "bannock"  n ( b a1 n @ k ) ((pos "K6$" ) ))
( "bannocks"  n ( b a1 n @ k s ) ((pos "Kj$" ) ))
( "banns"  n ( b a1 n z ) ((pos "Kj%" ) ))
( "banquet"  v ( b a1 ng k w i t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "banqueted"  v ( b a1 ng k w i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "banqueting"  v ( b a1 ng k w i t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "banquets"  v ( b a1 ng k w i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "bans"  v ( b a1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "banshee"  n ( b a1 n sh ii1 ) ((pos "K6%" ) ))
( "banshees"  n ( b a1 n sh ii1 z ) ((pos "Kj%" ) ))
( "bant"  v ( b a1 n t ) ((pos "I0$" ) (subcat )))
( "bantam"  n ( b a1 n t @ m ) ((pos "K6%" ) ))
( "bantams"  n ( b a1 n t @ m z ) ((pos "Kj%" ) ))
( "banted"  v ( b a1 n t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "banter"  v ( b a1 n t @ r ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "bantered"  v ( b a1 n t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bantering"  v ( b a1 n t @ r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "banteringly"  a ( b a1 n t @ r i ng l ii ) ((pos "Pu%" ) ))
( "banters"  v ( b a1 n t @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "banting"  v ( b a1 n t i ng ) ((pos "Ib$" "L@$" ) (subcat )))
( "bants"  v ( b a1 n t s ) ((pos "Ia$" ) (subcat )))
( "banyan"  n ( b a1 n y @ n ) ((pos "K6%" ) ))
( "banyans"  n ( b a1 n y @ n z ) ((pos "Kj%" ) ))
( "baobab"  n ( b ei1 @ b a b ) ((pos "K6$" ) ))
( "baobabs"  n ( b ei1 @ b a b z ) ((pos "Kj$" ) ))
( "baptism"  n ( b a1 p t i z @ m ) ((pos "M6%" ) ))
( "baptismal"  j ( b a1 p t i1 z m @ l ) ((pos "OA%" ) ))
( "baptisms"  n ( b a1 p t i z @ m z ) ((pos "Mj%" ) ))
( "baptize"  v ( b a1 p t ai1 z ) ((pos "H2%" ) (subcat "6A" )))
( "baptized"  v ( b a1 p t ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "baptizes"  v ( b a1 p t ai1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "baptizing"  v ( b a1 p t ai1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bar"  v ( b aa r ) ((pos "H4%" "K6%" "T-%" ) (subcat "6A" "6C" "12C" "14" )))
( "barb"  n ( b aa1 b ) ((pos "K6%" ) ))
( "barbarian"  n ( b aa1 b e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "barbarians"  n ( b aa1 b e@1 r i@ n z ) ((pos "Kj%" ) ))
( "barbaric"  j ( b aa1 b a1 r i k ) ((pos "OA%" ) ))
( "barbarism"  n ( b aa1 b @ r i z @ m ) ((pos "M6%" ) ))
( "barbarisms"  n ( b aa1 b @ r i z @ m z ) ((pos "Mj$" ) ))
( "barbarities"  n ( b aa1 b a1 r i t i z ) ((pos "Mj%" ) ))
( "barbarity"  n ( b aa1 b a1 r i t ii ) ((pos "M8%" ) ))
( "barbarize"  v ( b aa1 b @ r ai z ) ((pos "H2%" ) (subcat )))
( "barbarized"  v ( b aa1 b @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "barbarizes"  v ( b aa1 b @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "barbarizing"  v ( b aa1 b @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "barbarous"  j ( b aa1 b @ r @ s ) ((pos "OA%" ) ))
( "barbarously"  a ( b aa1 b @ r @ s l ii ) ((pos "Pu%" ) ))
( "barbecue"  v ( b aa1 b i k y uu ) ((pos "H2%" "K6%" ) ))
( "barbecued"  v ( b aa1 b i k y uu d ) ((pos "Hc%" "Hd%" ) ))
( "barbecues"  v ( b aa1 b i k y uu z ) ((pos "Ha%" "Kj%" ) ))
( "barbecuing"  v ( b aa1 b i k y uu i ng ) ((pos "Hb%" ) ))
( "barbed"  j ( b aa1 b d ) ((pos "OA%" ) ))
( "barbel"  n ( b aa1 b l ) ((pos "K6$" ) ))
( "barbels"  n ( b aa1 b l z ) ((pos "Kj$" ) ))
( "barber"  n ( b aa1 b @ r ) ((pos "K6%" ) ))
( "barbers"  n ( b aa1 b @ z ) ((pos "Kj%" ) ))
( "barbican"  n ( b aa1 b i k @ n ) ((pos "K6$" ) ))
( "barbicans"  n ( b aa1 b i k @ n z ) ((pos "Kj$" ) ))
( "barbitone"  n ( b aa1 b i t ou n ) ((pos "L@$" ) ))
( "barbiturate"  n ( b aa1 b i1 t y u r @ t ) ((pos "M6%" ) ))
( "barbiturates"  n ( b aa1 b i1 t y u r @ t s ) ((pos "Mj%" ) ))
( "barbs"  n ( b aa1 b z ) ((pos "Kj%" ) ))
( "barcarole"  n ( b aa2 k @ r ou1 l ) ((pos "K6$" ) ))
( "barcaroles"  n ( b aa2 k @ r ou1 l z ) ((pos "Kj$" ) ))
( "barcarolle"  n ( b aa2 k @ r ou1 l ) ((pos "K6%" ) ))
( "barcarolles"  n ( b aa2 k @ r ou1 l z ) ((pos "Kj%" ) ))
( "bard"  n ( b aa1 d ) ((pos "K6%" ) ))
( "bardic"  j ( b aa1 d i k ) ((pos "OA%" ) ))
( "bardolatry"  n ( b aa1 d o1 l @ t r ii ) ((pos "L@$" ) ))
( "bards"  n ( b aa1 d z ) ((pos "Kj%" ) ))
( "bare"  v ( b e@1 r ) ((pos "H2%" "OB%" ) (subcat )))
( "bareback"  a ( b e@1 b a k ) ((pos "Pu%" ) ))
( "barebacked"  j ( b e@1 b a k t ) ((pos "OA%" "Pu$" ) ))
( "bared"  v ( b e@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "barefaced"  j ( b e@1 f ei s t ) ((pos "OA%" ) ))
( "barefacedly"  a ( b e@1 f ei s t l ii ) ((pos "Pu%" ) ))
( "barefoot"  a ( b e@1 f u1 t ) ((pos "Pu%" ) ))
( "barefooted"  j ( b e@1 f u1 t i d ) ((pos "OA%" "Pu%" ) ))
( "bareheaded"  j ( b e@1 h e1 d i d ) ((pos "OA%" ) ))
( "barelegged"  j ( b e@1 l e1 g i d ) ((pos "OA%" ) ))
( "barely"  a ( b e@1 l ii ) ((pos "Pu%" ) ))
( "bareness"  n ( b e@1 n @ s ) ((pos "L@%" ) ))
( "barer"  j ( b e@1 r @ r ) ((pos "Or$" ) ))
( "bares"  v ( b e@1 z ) ((pos "Ha%" ) (subcat )))
( "barest"  j ( b e@1 r i s t ) ((pos "Os$" ) ))
( "bargain"  v ( b aa1 g i n ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "9" )))
( "bargained"  v ( b aa1 g i n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "9" )))
( "bargaining"  v ( b aa1 g i n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "9" )))
( "bargains"  v ( b aa1 g i n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "9" )))
( "barge"  v ( b aa1 jh ) ((pos "I2%" "K6%" ) (subcat "2C" )))
( "barged"  v ( b aa1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "bargee"  n ( b aa1 jh ii1 ) ((pos "K6%" ) ))
( "bargees"  n ( b aa1 jh ii1 z ) ((pos "Kj%" ) ))
( "bargepole"  n ( b aa1 jh p ou l ) ((pos "K6%" ) ))
( "bargepoles"  n ( b aa1 jh p ou l z ) ((pos "Kj%" ) ))
( "barges"  v ( b aa1 jh i z ) ((pos "Ia%" "Kj%" ) (subcat "2C" )))
( "barging"  v ( b aa1 jh i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "baring"  v ( b e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "baritone"  n ( b a1 r i t ou n ) ((pos "K6%" ) ))
( "baritones"  n ( b a1 r i t ou n z ) ((pos "Kj%" ) ))
( "barium"  n ( b e@1 r i@ m ) ((pos "L@$" ) ))
( "bark"  v ( b aa1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "barked"  v ( b aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "barker"  n ( b aa1 k @ r ) ((pos "K6%" ) ))
( "barkers"  n ( b aa1 k @ z ) ((pos "Kj%" ) ))
( "barking"  v ( b aa1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "barks"  v ( b aa1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "barley"  n ( b aa1 l ii ) ((pos "L@%" ) ))
( "barley-sugar"  n ( b aa1 l i - sh u g @ r ) ((pos "M6%" ) ))
( "barley-sugars"  n ( b aa1 l i - sh u g @ z ) ((pos "Mj%" ) ))
( "barley-water"  n ( b aa1 l i - w oo t @ r ) ((pos "L@%" ) ))
( "barleycorn"  n ( b aa1 l i k oo n ) ((pos "M6%" ) ))
( "barleycorns"  n ( b aa1 l i k oo n z ) ((pos "Mj%" ) ))
( "barm"  n ( b aa1 m ) ((pos "L@$" ) ))
( "barmaid"  n ( b aa1 m ei d ) ((pos "K6%" ) ))
( "barmaids"  n ( b aa1 m ei d z ) ((pos "Kj%" ) ))
( "barman"  n ( b aa1 m a n ) ((pos "Ki%" ) ))
( "barmen"  n ( b aa1 m e n ) ((pos "Kj%" ) ))
( "barmy"  j ( b aa1 m ii ) ((pos "OA%" ) ))
( "barn"  n ( b aa1 n ) ((pos "K6%" ) ))
( "barn-door"  n ( b aa1 n - d oo1 r ) ((pos "K6%" ) ))
( "barn-doors"  n ( b aa1 n - d oo1 z ) ((pos "Kj%" ) ))
( "barnacle"  n ( b aa1 n @ k l ) ((pos "K6%" ) ))
( "barnacles"  n ( b aa1 n @ k l z ) ((pos "Kj%" ) ))
( "barns"  n ( b aa1 n z ) ((pos "Kj%" ) ))
( "barnstorm"  v ( b aa1 n s t oo m ) ((pos "I0%" ) ))
( "barnstormed"  v ( b aa1 n s t oo m d ) ((pos "Ic$" "Id$" ) ))
( "barnstormer"  n ( b aa1 n s t oo m @ r ) ((pos "K6$" ) ))
( "barnstormers"  n ( b aa1 n s t oo m @ z ) ((pos "Kj$" ) ))
( "barnstorming"  v ( b aa1 n s t oo m i ng ) ((pos "Ib$" ) ))
( "barnstorms"  v ( b aa1 n s t oo m z ) ((pos "Ia$" ) ))
( "barnyard"  n ( b aa1 n y aa d ) ((pos "K6%" ) ))
( "barnyards"  n ( b aa1 n y aa d z ) ((pos "Kj%" ) ))
( "barometer"  n ( b @1 r o1 m i t @ r ) ((pos "K6%" ) ))
( "barometers"  n ( b @1 r o1 m i t @ z ) ((pos "Kj%" ) ))
( "barometric"  j ( b a2 r @ m e1 t r i k ) ((pos "OA$" ) ))
( "baron"  n ( b a1 r @ n ) ((pos "K6%" ) ))
( "baronage"  n ( b a1 r @ n i jh ) ((pos "K6$" ) ))
( "baronages"  n ( b a1 r @ n i jh i z ) ((pos "Kj$" ) ))
( "baroness"  n ( b a1 r @ n e s ) ((pos "K7%" ) ))
( "baronesses"  n ( b a1 r @ n e s i z ) ((pos "Kj%" ) ))
( "baronet"  n ( b a1 r @ n e t ) ((pos "K6%" ) ))
( "baronetcies"  n ( b a1 r @ n i t s i z ) ((pos "Kj$" ) ))
( "baronetcy"  n ( b a1 r @ n i t s ii ) ((pos "K8%" ) ))
( "baronets"  n ( b a1 r @ n e t s ) ((pos "Kj%" ) ))
( "baronial"  j ( b @1 r ou1 n i@ l ) ((pos "OA%" ) ))
( "baronies"  n ( b a1 r @ n i z ) ((pos "Kj$" ) ))
( "barons"  n ( b a1 r @ n z ) ((pos "Kj%" ) ))
( "barony"  n ( b a1 r @ n ii ) ((pos "K8$" ) ))
( "baroque"  n ( b @1 r o1 k ) ((pos "Ki%" "OA%" ) ))
( "barouche"  n ( b @1 r uu1 sh ) ((pos "K6$" ) ))
( "barouches"  n ( b @1 r uu1 sh i z ) ((pos "Kj$" ) ))
( "barque"  n ( b aa1 k ) ((pos "K6$" ) ))
( "barques"  n ( b aa1 k s ) ((pos "Kj$" ) ))
( "barrack"  v ( b a1 r @ k ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "barracked"  v ( b a1 r @ k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "barracking"  v ( b a1 r @ k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "barracks"  v ( b a1 r @ k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "barracuda"  n ( b a2 r @ k uu1 d @ ) ((pos "K6%" ) ))
( "barracudas"  n ( b a2 r @ k uu1 d @ z ) ((pos "Kj%" ) ))
( "barrage"  n ( b a1 r aa zh ) ((pos "K6%" ) ))
( "barrages"  n ( b a1 r aa zh i z ) ((pos "Kj%" ) ))
( "barred"  v ( b aa1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "12C" "14" )))
( "barrel"  v ( b a1 r @ l ) ((pos "H4$" "K6%" ) ))
( "barrelled"  v ( b a1 r @ l d ) ((pos "Hc$" "Hd%" "OA%" ) ))
( "barrelling"  v ( b a1 r @ l i ng ) ((pos "Hb$" ) ))
( "barrels"  v ( b a1 r @ l z ) ((pos "Ha$" "Kj%" ) ))
( "barren"  j ( b a1 r @ n ) ((pos "OA%" ) ))
( "barrenness"  n ( b a1 r @ n - n @ s ) ((pos "L@%" ) ))
( "barricade"  v ( b a2 r i k ei1 d ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "barricaded"  v ( b a2 r i k ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "barricades"  v ( b a2 r i k ei1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "barricading"  v ( b a2 r i k ei1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "barrier"  n ( b a1 r i@ r ) ((pos "K6%" ) ))
( "barriers"  n ( b a1 r i@ z ) ((pos "Kj%" ) ))
( "barring"  v ( b aa1 r i ng ) ((pos "Hb%" "T-%" ) (subcat "6A" "6C" "12C" "14" )))
( "barrister"  n ( b a1 r i s t @ r ) ((pos "K6%" ) ))
( "barristers"  n ( b a1 r i s t @ z ) ((pos "Kj%" ) ))
( "barrow"  n ( b a1 r ou ) ((pos "K6%" ) ))
( "barrow-boy"  n ( b a1 r ou - b oi ) ((pos "K6%" ) ))
( "barrow-boys"  n ( b a1 r ou - b oi z ) ((pos "Kj%" ) ))
( "barrow-man"  n ( b a1 r ou - m a n ) ((pos "Ki$" ) ))
( "barrow-men"  n ( b a1 r ou - m e n ) ((pos "Kj$" ) ))
( "barrows"  n ( b a1 r ou z ) ((pos "Kj%" ) ))
( "bars"  v ( b aa1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "6C" "12C" "14" )))
( "bartender"  n ( b aa1 t e n d @ r ) ((pos "K6%" ) ))
( "bartenders"  n ( b aa1 t e n d @ z ) ((pos "Kj%" ) ))
( "barter"  v ( b aa1 t @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "14" )))
( "bartered"  v ( b aa1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "14" )))
( "barterer"  n ( b aa1 t @ r @ r ) ((pos "K6%" ) ))
( "barterers"  n ( b aa1 t @ r @ z ) ((pos "Kj%" ) ))
( "bartering"  v ( b aa1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "14" )))
( "barters"  v ( b aa1 t @ z ) ((pos "Ja%" ) (subcat "2A" "14" )))
( "bas-relief"  n ( b a2 s - r i l ii1 f ) ((pos "M6%" ) ))
( "bas-reliefs"  n ( b a2 s - r i l ii1 f s ) ((pos "Mj%" ) ))
( "basal"  j ( b ei1 s l ) ((pos "OA%" ) ))
( "basalt"  n ( b a1 s oo l t ) ((pos "L@%" ) ))
( "bascule"  n ( b a1 s k y uu l ) ((pos "K6$" ) ))
( "bascules"  n ( b a1 s k y uu l z ) ((pos "Kj$" ) ))
( "base"  v ( b ei1 s ) ((pos "H2%" "K6%" "OB%" ) (subcat )))
( "baseball"  n ( b ei1 s b oo l ) ((pos "M6%" ) ))
( "baseballs"  n ( b ei1 s b oo l z ) ((pos "Mj%" ) ))
( "baseboard"  n ( b ei1 s b oo d ) ((pos "K6%" ) ))
( "baseboards"  n ( b ei1 s b oo d z ) ((pos "Kj%" ) ))
( "based"  v ( b ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "baseless"  j ( b ei1 s l i s ) ((pos "OA%" ) ))
( "basement"  n ( b ei1 s m @ n t ) ((pos "K6%" ) ))
( "basements"  n ( b ei1 s m @ n t s ) ((pos "Kj%" ) ))
( "baser"  j ( b ei1 s @ r ) ((pos "Or$" ) ))
( "bases"  v ( b ei1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bases"  n ( b ei1 s ii z ) ((pos "Kj%" ) ))
( "basest"  j ( b ei1 s i s t ) ((pos "Os%" ) ))
( "bash"  v ( b a1 sh ) ((pos "H1%" "K7%" ) (subcat "6A" "15A" )))
( "bashed"  v ( b a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "bashes"  v ( b a1 sh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "bashful"  j ( b a1 sh f u l ) ((pos "OA%" ) ))
( "bashfully"  a ( b a1 sh f u l ii ) ((pos "Pu%" ) ))
( "bashing"  v ( b a1 sh i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "basic"  j ( b ei1 s i k ) ((pos "OA%" ) ))
( "basically"  a ( b ei1 s i k l ii ) ((pos "Pu%" ) ))
( "basil"  n ( b a1 z l ) ((pos "L@%" ) ))
( "basilica"  n ( b @1 z i1 l i k @ ) ((pos "K6%" ) ))
( "basilicas"  n ( b @1 z i1 l i k @ z ) ((pos "Kj%" ) ))
( "basilisk"  n ( b a1 s i l i s k ) ((pos "K6%" ) ))
( "basilisks"  n ( b a1 s i l i s k s ) ((pos "Kj%" ) ))
( "basin"  n ( b ei1 s n ) ((pos "K6%" ) ))
( "basing"  v ( b ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "basins"  n ( b ei1 s n z ) ((pos "Kj%" ) ))
( "basis"  n ( b ei1 s i s ) ((pos "Ki%" ) ))
( "bask"  v ( b aa1 s k ) ((pos "I0%" ) (subcat )))
( "basked"  v ( b aa1 s k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "basket"  n ( b aa1 s k i t ) ((pos "K6%" ) ))
( "basketball"  n ( b aa1 s k i t b oo l ) ((pos "M6%" ) ))
( "basketballs"  n ( b aa1 s k i t b oo l z ) ((pos "Mj%" ) ))
( "basketry"  n ( b aa1 s k i t r ii ) ((pos "L@%" ) ))
( "baskets"  n ( b aa1 s k i t s ) ((pos "Kj%" ) ))
( "basking"  v ( b aa1 s k i ng ) ((pos "Ib%" ) (subcat )))
( "basks"  v ( b aa1 s k s ) ((pos "Ia%" ) (subcat )))
( "bass"  n ( b a1 s ) ((pos "K9%" ) ))
( "bass"  n ( b ei1 s ) ((pos "K7%" "OA%" ) ))
( "basses"  n ( b ei1 s i z ) ((pos "Kj%" ) ))
( "bassinet"  n ( b a2 s i n e1 t ) ((pos "K6$" ) ))
( "bassinets"  n ( b a2 s i n e1 t s ) ((pos "Kj$" ) ))
( "bassoon"  n ( b @1 s uu1 n ) ((pos "K6%" ) ))
( "bassoons"  n ( b @1 s uu1 n z ) ((pos "Kj%" ) ))
( "bast"  n ( b a1 s t ) ((pos "L@$" ) ))
( "bastard"  n ( b aa1 s t @ d ) ((pos "K6%" ) ))
( "bastardies"  n ( b aa1 s t @ d i z ) ((pos "Kj$" ) ))
( "bastardize"  v ( b a1 s t @ d ai z ) ((pos "H2$" ) ))
( "bastardized"  v ( b a1 s t @ d ai z d ) ((pos "Hc$" "Hd$" ) ))
( "bastardizes"  v ( b a1 s t @ d ai z i z ) ((pos "Ha$" ) ))
( "bastardizing"  v ( b a1 s t @ d ai z i ng ) ((pos "Hb$" ) ))
( "bastards"  n ( b aa1 s t @ d z ) ((pos "Kj%" ) ))
( "bastardy"  n ( b aa1 s t @ d ii ) ((pos "K8%" ) ))
( "baste"  v ( b ei1 s t ) ((pos "H2%" ) (subcat "6A" )))
( "basted"  v ( b ei1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bastes"  v ( b ei1 s t s ) ((pos "Ha%" ) (subcat "6A" )))
( "bastinado"  v ( b a2 s t i n aa1 d ou ) ((pos "H0$" "Ki$" ) (subcat )))
( "bastinadoed"  v ( b a2 s t i n aa1 d ou d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "bastinadoes"  n ( b a2 s t i n aa1 d ou z ) ((pos "Kj$" ) ))
( "bastinadoing"  v ( b a2 s t i n aa1 d ou i ng ) ((pos "Hb$" ) (subcat )))
( "bastinados"  v ( b a2 s t i n aa1 d ou z ) ((pos "Ha$" ) (subcat )))
( "basting"  v ( b ei1 s t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bastion"  n ( b a1 s t i@ n ) ((pos "K6%" ) ))
( "bastions"  n ( b a1 s t i@ n z ) ((pos "Kj%" ) ))
( "bat"  v ( b a1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "batch"  n ( b a1 ch ) ((pos "K7%" ) ))
( "batches"  n ( b a1 ch i z ) ((pos "Kj%" ) ))
( "bate"  v ( b ei1 t ) ((pos "H2$" ) ))
( "bated"  v ( b ei1 t i d ) ((pos "Hc$" "Hd%" ) ))
( "bates"  v ( b ei1 t s ) ((pos "Ha$" ) ))
( "bath"  v ( b aa1 th ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "bathe"  v ( b ei1 dh ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "bathed"  v ( b aa1 th t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bathed"  v ( b ei1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bather"  n ( b ei1 dh @ r ) ((pos "K6%" ) ))
( "bathers"  n ( b ei1 dh @ z ) ((pos "Kj%" ) ))
( "bathes"  v ( b ei1 dh z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "bathing"  v ( b aa1 th i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bathing"  v ( b ei1 dh i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" )))
( "bathing-cap"  n ( b ei1 dh i ng - k a p ) ((pos "K6%" ) ))
( "bathing-caps"  n ( b ei1 dh i ng - k a p s ) ((pos "Kj%" ) ))
( "bathing-costume"  n ( b ei1 dh i ng - k o s t y uu m ) ((pos "K6%" ) ))
( "bathing-costumes"  n ( b ei1 dh i ng - k o s t y uu m z ) ((pos "Kj%" ) ))
( "bathing-machine"  n ( b ei1 dh i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "bathing-machines"  n ( b ei1 dh i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "bathing-suit"  n ( b ei1 dh i ng - s uu t ) ((pos "K6%" ) ))
( "bathing-suits"  n ( b ei1 dh i ng - s uu t s ) ((pos "Kj%" ) ))
( "bathings"  n ( b ei1 dh i ng z ) ((pos "Kj%" ) ))
( "bathos"  n ( b ei1 th o s ) ((pos "L@%" ) ))
( "bathrobe"  n ( b aa1 th r ou b ) ((pos "K6%" ) ))
( "bathrobes"  n ( b aa1 th r ou b z ) ((pos "Kj%" ) ))
( "bathroom"  n ( b aa1 th r uu m ) ((pos "K6%" ) ))
( "bathrooms"  n ( b aa1 th r uu m z ) ((pos "Kj%" ) ))
( "baths"  n ( b aa1 dh z ) ((pos "Kj%" ) ))
( "baths"  v ( b aa1 th s ) ((pos "Ja%" ) (subcat "2A" )))
( "bathtub"  n ( b aa1 th t uh b ) ((pos "K6%" ) ))
( "bathtubs"  n ( b aa1 th t uh b z ) ((pos "Kj%" ) ))
( "bathysphere"  n ( b a1 th i s f i@ r ) ((pos "K6%" ) ))
( "bathyspheres"  n ( b a1 th i s f i@ z ) ((pos "Kj%" ) ))
( "batik"  n ( b @1 t ii1 k ) ((pos "M6%" ) ))
( "batiks"  n ( b @1 t ii1 k s ) ((pos "Mj$" ) ))
( "bating"  v ( b ei1 t i ng ) ((pos "Hb%" ) ))
( "batiste"  n ( b a1 t ii1 s t ) ((pos "L@$" ) ))
( "batman"  n ( b a1 t m @ n ) ((pos "Ki%" ) ))
( "batmen"  n ( b a1 t m @ n ) ((pos "Kj%" ) ))
( "baton"  n ( b a1 t @ n ) ((pos "K6%" ) ))
( "batons"  n ( b a1 t @ n z ) ((pos "Kj%" ) ))
( "bats"  v ( b a1 t s ) ((pos "Ja%" "Kj%" "Op%" ) (subcat "2A" "2B" "2C" )))
( "batsman"  n ( b a1 t s m @ n ) ((pos "Ki%" ) ))
( "batsmen"  n ( b a1 t s m @ n ) ((pos "Kj%" ) ))
( "battalion"  n ( b @1 t a1 l i@ n ) ((pos "K6%" ) ))
( "battalions"  n ( b @1 t a1 l i@ n z ) ((pos "Kj%" ) ))
( "batted"  v ( b a1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "batten"  v ( b a1 t n ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" )))
( "battened"  v ( b a1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "battening"  v ( b a1 t n i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "battens"  v ( b a1 t n z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "batter"  v ( b a1 t @ r ) ((pos "J0%" "L@%" ) (subcat "2C" "6A" "15A" )))
( "battered"  v ( b a1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "batteries"  n ( b a1 t @ r i z ) ((pos "Kj%" ) ))
( "battering"  v ( b a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "batters"  v ( b a1 t @ z ) ((pos "Ja%" ) (subcat "2C" "6A" "15A" )))
( "battery"  n ( b a1 t @ r ii ) ((pos "K8%" ) ))
( "batting"  v ( b a1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" )))
( "battle"  v ( b a1 t l ) ((pos "I2%" "M6%" ) (subcat )))
( "battle-axe"  n ( b a1 t l - a k s ) ((pos "K6%" ) ))
( "battle-axes"  n ( b a1 t l - a k s i z ) ((pos "Kj%" ) ))
( "battle-cruiser"  n ( b a1 t l - k r uu z @ r ) ((pos "K6%" ) ))
( "battle-cruisers"  n ( b a1 t l - k r uu z @ z ) ((pos "Kj%" ) ))
( "battled"  v ( b a1 t l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "battledore"  n ( b a1 t l d oo r ) ((pos "K6$" ) ))
( "battledores"  n ( b a1 t l d oo z ) ((pos "Kj$" ) ))
( "battledress"  n ( b a1 t l d r e s ) ((pos "Ki%" ) ))
( "battlefield"  n ( b a1 t l f ii l d ) ((pos "K6%" ) ))
( "battlefields"  n ( b a1 t l f ii l d z ) ((pos "Kj%" ) ))
( "battleground"  n ( b a1 t l g r au n d ) ((pos "K6%" ) ))
( "battlegrounds"  n ( b a1 t l g r au n d z ) ((pos "Kj%" ) ))
( "battlements"  n ( b a1 t l m @ n t s ) ((pos "Kj%" ) ))
( "battles"  v ( b a1 t l z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "battleship"  n ( b a1 t l sh i p ) ((pos "K6%" ) ))
( "battleships"  n ( b a1 t l sh i p s ) ((pos "Kj%" ) ))
( "battling"  v ( b a1 t l i ng ) ((pos "Ib%" ) (subcat )))
( "battue"  n ( b a1 t uu1 ) ((pos "K6$" ) ))
( "battues"  n ( b a1 t uu1 z ) ((pos "Kj$" ) ))
( "batty"  j ( b a1 t ii ) ((pos "OA%" ) ))
( "bauble"  n ( b oo1 b l ) ((pos "K6%" ) ))
( "baubles"  n ( b oo1 b l z ) ((pos "Kj%" ) ))
( "baulk"  v ( b oo1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "baulked"  v ( b oo1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "baulking"  v ( b oo1 k i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "baulks"  v ( b oo1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "bauxite"  n ( b oo1 k s ai t ) ((pos "L@%" ) ))
( "bawbee"  n ( b oo1 b ii ) ((pos "K6$" ) ))
( "bawbees"  n ( b oo1 b ii z ) ((pos "Kj$" ) ))
( "bawd"  n ( b oo1 d ) ((pos "K6%" ) ))
( "bawdier"  j ( b oo1 d i@ r ) ((pos "Or%" ) ))
( "bawdiest"  j ( b oo1 d i i s t ) ((pos "Os%" ) ))
( "bawdily"  a ( b oo1 d i l ii ) ((pos "Pu%" ) ))
( "bawds"  n ( b oo1 d z ) ((pos "Kj%" ) ))
( "bawdy"  n ( b oo1 d ii ) ((pos "L@$" "OD%" ) ))
( "bawl"  v ( b oo1 l ) ((pos "J0%" ) (subcat "2C" "3A" "6A" )))
( "bawled"  v ( b oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "bawling"  v ( b oo1 l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "bawls"  v ( b oo1 l z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" )))
( "bay"  v ( b ei1 ) ((pos "I0%" "K6%" "OA%" ) (subcat )))
( "bay-wreath"  n ( b ei1 - r ii th ) ((pos "K6$" ) ))
( "bay-wreaths"  n ( b ei1 - r ii th s ) ((pos "Kj$" ) ))
( "bayed"  v ( b ei1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "baying"  v ( b ei1 i ng ) ((pos "Ib%" ) (subcat )))
( "bayonet"  v ( b ei1 @ n e t ) ((pos "H0%" "K6%" ) (subcat )))
( "bayoneted"  v ( b ei1 @ n e t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bayoneting"  v ( b ei1 @ n e t i ng ) ((pos "Hb%" ) (subcat )))
( "bayonets"  v ( b ei1 @ n e t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bayou"  n ( b ai1 uu ) ((pos "K6$" ) ))
( "bayous"  n ( b ai1 uu z ) ((pos "Kj$" ) ))
( "bays"  v ( b ei1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "bazaar"  n ( b @1 z aa1 r ) ((pos "K6%" ) ))
( "bazaars"  n ( b @1 z aa1 z ) ((pos "Kj%" ) ))
( "bazooka"  n ( b @1 z uu1 k @ ) ((pos "K6%" ) ))
( "bazookas"  n ( b @1 z uu1 k @ z ) ((pos "Kj%" ) ))
( "be"  v ( b ii ) ((pos "G5*" "I5%" ) (subcat "1" )))
( "be-"  j ( b i - ) ((pos "U-%" ) ))
( "beach"  v ( b ii1 ch ) ((pos "H1%" "K7%" ) (subcat )))
( "beachcomber"  n ( b ii1 ch k ou m @ r ) ((pos "K6%" ) ))
( "beachcombers"  n ( b ii1 ch k ou m @ z ) ((pos "Kj%" ) ))
( "beached"  v ( b ii1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beaches"  v ( b ii1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "beachhead"  n ( b ii1 ch h e d ) ((pos "K6%" ) ))
( "beachheads"  n ( b ii1 ch h e d z ) ((pos "Kj%" ) ))
( "beaching"  v ( b ii1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "beachwear"  n ( b ii1 ch w e@ r ) ((pos "L@%" ) ))
( "beacon"  n ( b ii1 k @ n ) ((pos "K6%" ) ))
( "beacon-fire"  n ( b ii1 k @ n - f ai @ r ) ((pos "K6$" ) ))
( "beacon-fires"  n ( b ii1 k @ n - f ai @ z ) ((pos "Kj$" ) ))
( "beacon-light"  n ( b ii1 k @ n - l ai t ) ((pos "K6$" ) ))
( "beacon-lights"  n ( b ii1 k @ n - l ai t s ) ((pos "Kj$" ) ))
( "beacons"  n ( b ii1 k @ n z ) ((pos "Kj%" ) ))
( "bead"  n ( b ii1 d ) ((pos "K6%" ) ))
( "beading"  n ( b ii1 d i ng ) ((pos "L@%" ) ))
( "beadle"  n ( b ii1 d l ) ((pos "K6%" ) ))
( "beadles"  n ( b ii1 d l z ) ((pos "Kj%" ) ))
( "beads"  n ( b ii1 d z ) ((pos "Kj%" ) ))
( "beady"  j ( b ii1 d ii ) ((pos "OA%" ) ))
( "beagle"  n ( b ii1 g l ) ((pos "K6%" ) ))
( "beagles"  n ( b ii1 g l z ) ((pos "Kj%" ) ))
( "beagling"  n ( b ii1 g l i ng ) ((pos "L@$" ) ))
( "beak"  n ( b ii1 k ) ((pos "K6%" ) ))
( "beaker"  n ( b ii1 k @ r ) ((pos "K6%" ) ))
( "beakers"  n ( b ii1 k @ z ) ((pos "Kj%" ) ))
( "beaks"  n ( b ii1 k s ) ((pos "Kj%" ) ))
( "beam"  v ( b ii1 m ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "beamed"  v ( b ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "beaming"  v ( b ii1 m i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "beams"  v ( b ii1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "bean"  n ( b ii1 n ) ((pos "K6%" ) ))
( "beanfeast"  n ( b ii1 n f ii s t ) ((pos "K6%" ) ))
( "beanfeasts"  n ( b ii1 n f ii s t s ) ((pos "Kj%" ) ))
( "beano"  n ( b ii1 n ou ) ((pos "K6%" ) ))
( "beanos"  n ( b ii1 n ou z ) ((pos "Kj%" ) ))
( "beans"  n ( b ii1 n z ) ((pos "Kj%" ) ))
( "beanstalk"  n ( b ii1 n s t oo k ) ((pos "K6%" ) ))
( "beanstalks"  n ( b ii1 n s t oo k s ) ((pos "Kj%" ) ))
( "bear"  v ( b e@1 r ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15B" "16B" )))
( "bearable"  j ( b e@1 r @ b l ) ((pos "OA%" ) ))
( "beard"  v ( b i@1 d ) ((pos "H0%" "K6%" ) (subcat )))
( "bearded"  v ( b i@1 d i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "bearding"  v ( b i@1 d i ng ) ((pos "Hb%" ) (subcat )))
( "beardless"  j ( b i@1 d l @ s ) ((pos "OA%" ) ))
( "beards"  v ( b i@1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bearer"  n ( b e@1 r @ r ) ((pos "K6%" ) ))
( "bearers"  n ( b e@1 r @ z ) ((pos "Kj%" ) ))
( "bearing"  v ( b e@1 r i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15B" "16B" )))
( "bearings"  n ( b e@1 r i ng z ) ((pos "Mj%" ) ))
( "bearish"  j ( b e@1 r i sh ) ((pos "OA%" ) ))
( "bears"  v ( b e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15B" "16B" )))
( "bearskin"  n ( b e@1 s k i n ) ((pos "K6%" ) ))
( "bearskins"  n ( b e@1 s k i n z ) ((pos "Kj%" ) ))
( "beast"  n ( b ii1 s t ) ((pos "K6%" ) ))
( "beastlier"  j ( b ii1 s t l i@ r ) ((pos "Or%" ) ))
( "beastliest"  j ( b ii1 s t l i i s t ) ((pos "Os%" ) ))
( "beastliness"  n ( b ii1 s t l i n @ s ) ((pos "L@%" ) ))
( "beastly"  j ( b ii1 s t l ii ) ((pos "OD%" "Pu%" ) ))
( "beasts"  n ( b ii1 s t s ) ((pos "Kj%" ) ))
( "beat"  v ( b ii1 t ) ((pos "J5%" "Jc%" "K6%" "Oq%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "beaten"  v ( b ii1 t n ) ((pos "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "beater"  n ( b ii1 t @ r ) ((pos "K6%" ) ))
( "beaters"  n ( b ii1 t @ z ) ((pos "Kj%" ) ))
( "beatific"  j ( b i@1 t i1 f i k ) ((pos "OA%" ) ))
( "beatification"  n ( b i1 a2 t i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "beatifications"  n ( b i1 a2 t i f i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "beatified"  v ( b i1 a1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beatifies"  v ( b i1 a1 t i f ai z ) ((pos "Ha%" ) (subcat )))
( "beatify"  v ( b i1 a1 t i f ai ) ((pos "H3%" ) (subcat )))
( "beatifying"  v ( b i1 a1 t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "beating"  v ( b ii1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "beatings"  n ( b ii1 t i ng z ) ((pos "Kj%" ) ))
( "beatitude"  n ( b i1 a1 t i t y uu d ) ((pos "M6%" ) ))
( "beatitudes"  n ( b i1 a1 t i t y uu d z ) ((pos "Mj%" ) ))
( "beatnik"  n ( b ii1 t n i k ) ((pos "K6%" ) ))
( "beatniks"  n ( b ii1 t n i k s ) ((pos "Kj%" ) ))
( "beats"  v ( b ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "beau"  n ( b ou1 ) ((pos "Ki%" ) ))
( "beauteous"  j ( b y uu1 t i@ s ) ((pos "OA%" ) ))
( "beautician"  n ( b y uu1 t i1 sh @ n ) ((pos "K6%" ) ))
( "beauticians"  n ( b y uu1 t i1 sh @ n z ) ((pos "Kj%" ) ))
( "beauties"  n ( b y uu1 t i z ) ((pos "Mj%" ) ))
( "beautified"  v ( b y uu1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beautifies"  v ( b y uu1 t i f ai z ) ((pos "Ha%" ) (subcat )))
( "beautiful"  j ( b y uu1 t i f @ l ) ((pos "OA%" ) ))
( "beautifully"  a ( b y uu1 t @ f @ l ii ) ((pos "Pu%" ) ))
( "beautify"  v ( b y uu1 t i f ai ) ((pos "H3%" ) (subcat )))
( "beautifying"  v ( b y uu1 t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "beauty"  n ( b y uu1 t ii ) ((pos "M8%" ) ))
( "beauty-parlour"  n ( b y uu1 t i - p aa l @ r ) ((pos "K6%" ) ))
( "beauty-parlours"  n ( b y uu1 t i - p aa l @ z ) ((pos "Kj%" ) ))
( "beauty-salon"  n ( b y uu1 t i - s a l o n ) ((pos "K6%" ) ))
( "beauty-salons"  n ( b y uu1 t i - s a l o n z ) ((pos "Kj%" ) ))
( "beauty-sleep"  n ( b y uu1 t i - s l ii p ) ((pos "L@%" ) ))
( "beauty-spot"  n ( b y uu1 t i - s p o t ) ((pos "K6%" ) ))
( "beauty-spots"  n ( b y uu1 t i - s p o t s ) ((pos "Kj%" ) ))
( "beaux"  n ( b ou1 z ) ((pos "Kj$" ) ))
( "beaver"  v ( b ii1 v @ r ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "beavered"  v ( b ii1 v @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "beavering"  v ( b ii1 v @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "beavers"  v ( b ii1 v @ z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "becalmed"  j ( b i1 k aa1 m d ) ((pos "Op%" ) ))
( "became"  v ( b i1 k ei1 m ) ((pos "Jc*" ) (subcat "2D" "3A" )))
( "because"  cc ( b i1 k o1 z ) ((pos "V-*" ) ))
( "beck"  n ( b e1 k ) ((pos "K6%" ) ))
( "beckon"  v ( b e1 k @ n ) ((pos "J0%" ) (subcat "2A" "6A" "15B" )))
( "beckoned"  v ( b e1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15B" )))
( "beckoning"  v ( b e1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15B" )))
( "beckons"  v ( b e1 k @ n z ) ((pos "Ja%" ) (subcat "2A" "6A" "15B" )))
( "becks"  n ( b e1 k s ) ((pos "Kj%" ) ))
( "become"  v ( b i1 k uh1 m ) ((pos "J5*" "Jd*" ) (subcat "2D" "3A" )))
( "becomes"  v ( b i1 k uh1 m z ) ((pos "Ja%" ) (subcat "2D" "3A" )))
( "becoming"  v ( b i1 k uh1 m i ng ) ((pos "Jb%" "OA%" ) (subcat "2D" "3A" )))
( "becomingly"  a ( b i1 k uh1 m i ng l ii ) ((pos "Pu%" ) ))
( "bed"  v ( b e1 d ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" )))
( "bedaubed"  j ( b i1 d oo1 b d ) ((pos "Op%" ) ))
( "bedbug"  n ( b e1 d b uh g ) ((pos "K6%" ) ))
( "bedbugs"  n ( b e1 d b uh g z ) ((pos "Kj%" ) ))
( "bedclothes"  n ( b e1 d k l ou dh z ) ((pos "Kj%" ) ))
( "bedded"  v ( b e1 d i d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat "6A" "15A" )))
( "bedding"  v ( b e1 d i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" "15A" )))
( "bedecked"  j ( b i1 d e1 k t ) ((pos "Op%" ) ))
( "bedevil"  v ( b i1 d e1 v @ l ) ((pos "H4%" ) ))
( "bedevilled"  v ( b i1 d e1 v @ l d ) ((pos "Hc%" "Hd%" ) ))
( "bedevilling"  v ( b i1 d e1 v @ l i ng ) ((pos "Hb$" ) ))
( "bedevilment"  n ( b i1 d e1 v @ l m @ n t ) ((pos "K6$" ) ))
( "bedevilments"  n ( b i1 d e1 v @ l m @ n t s ) ((pos "Kj$" ) ))
( "bedevils"  v ( b i1 d e1 v @ l z ) ((pos "Ha$" ) ))
( "bedewed"  j ( b i1 d y uu1 d ) ((pos "Op%" ) ))
( "bedfellow"  n ( b e1 d f e l ou ) ((pos "K6%" ) ))
( "bedfellows"  n ( b e1 d f e l ou z ) ((pos "Kj%" ) ))
( "bedimmed"  j ( b i1 d i1 m d ) ((pos "Op$" ) ))
( "bedlam"  n ( b e1 d l @ m ) ((pos "K6%" ) ))
( "bedlams"  n ( b e1 d l @ m z ) ((pos "Kj$" ) ))
( "bedpan"  n ( b e1 d p a n ) ((pos "K6%" ) ))
( "bedpans"  n ( b e1 d p a n z ) ((pos "Kj%" ) ))
( "bedpost"  n ( b e1 d p ou s t ) ((pos "K6%" ) ))
( "bedposts"  n ( b e1 d p ou s t s ) ((pos "Kj%" ) ))
( "bedraggled"  j ( b i1 d r a1 g l d ) ((pos "Op%" ) ))
( "bedridden"  j ( b e1 d r i d n ) ((pos "OA%" ) ))
( "bedrock"  n ( b e1 d r o k ) ((pos "K6%" ) ))
( "bedrocks"  n ( b e1 d r o k s ) ((pos "Kj$" ) ))
( "bedroll"  n ( b e1 d r ou l ) ((pos "K6%" ) ))
( "bedrolls"  n ( b e1 d r ou l z ) ((pos "Kj%" ) ))
( "bedroom"  n ( b e1 d r uu m ) ((pos "K6%" ) ))
( "bedroomed"  j ( b e1 d r uu m d ) ((pos "OA%" ) ))
( "bedrooms"  n ( b e1 d r uu m z ) ((pos "Kj%" ) ))
( "beds"  v ( b e1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "bedside"  n ( b e1 d s ai d ) ((pos "K6%" ) ))
( "bedsides"  n ( b e1 d s ai d z ) ((pos "Kj$" ) ))
( "bedsit"  n ( b e1 d s i t ) ((pos "K6%" ) ))
( "bedsits"  n ( b e1 d s i t s ) ((pos "Kj%" ) ))
( "bedsitter"  n ( b e1 d s i1 t @ r ) ((pos "K6%" ) ))
( "bedsitters"  n ( b e1 d s i1 t @ z ) ((pos "Kj%" ) ))
( "bedsitting-room"  n ( b e1 d s i1 t i ng - r uu m ) ((pos "K6%" ) ))
( "bedsitting-rooms"  n ( b e1 d s i1 t i ng - r uu m z ) ((pos "Kj%" ) ))
( "bedsore"  n ( b e1 d s oo r ) ((pos "K6%" ) ))
( "bedsores"  n ( b e1 d s oo z ) ((pos "Kj%" ) ))
( "bedspread"  n ( b e1 d s p r e d ) ((pos "K6%" ) ))
( "bedspreads"  n ( b e1 d s p r e d z ) ((pos "Kj%" ) ))
( "bedstead"  n ( b e1 d s t e d ) ((pos "K6%" ) ))
( "bedsteads"  n ( b e1 d s t e d z ) ((pos "Kj%" ) ))
( "bedtime"  n ( b e1 d t ai m ) ((pos "K6%" ) ))
( "bedtimes"  n ( b e1 d t ai m z ) ((pos "Kj%" ) ))
( "bee"  n ( b ii1 ) ((pos "K6%" ) ))
( "beech"  n ( b ii1 ch ) ((pos "M7%" ) ))
( "beeches"  n ( b ii1 ch i z ) ((pos "Mj%" ) ))
( "beef"  v ( b ii1 f ) ((pos "I0%" "Mi%" ) ))
( "beefeater"  n ( b ii1 f ii t @ r ) ((pos "K6%" ) ))
( "beefeaters"  n ( b ii1 f ii t @ z ) ((pos "Kj%" ) ))
( "beefed"  v ( b ii1 f t ) ((pos "Ic%" "Id%" ) ))
( "beefier"  j ( b ii1 f i@ r ) ((pos "Or%" ) ))
( "beefiest"  j ( b ii1 f i i s t ) ((pos "Os%" ) ))
( "beefing"  v ( b ii1 f i ng ) ((pos "Ib%" ) ))
( "beefs"  v ( b ii1 f s ) ((pos "Ia%" ) ))
( "beefsteak"  n ( b ii1 f s t ei k ) ((pos "K6%" ) ))
( "beefsteaks"  n ( b ii1 f s t ei k s ) ((pos "Kj%" ) ))
( "beefy"  j ( b ii1 f ii ) ((pos "OD%" ) ))
( "beehive"  n ( b ii1 h ai v ) ((pos "K6%" ) ))
( "beehives"  n ( b ii1 h ai v z ) ((pos "Kj%" ) ))
( "been"  v ( b ii n ) ((pos "Gd*" "Id%" ) (subcat )))
( "beep"  n ( b ii1 p ) ((pos "K6%" ) ))
( "beeps"  n ( b ii1 p s ) ((pos "Kj%" ) ))
( "beer"  n ( b i@1 r ) ((pos "M6%" ) ))
( "beerier"  j ( b i@1 r i@ r ) ((pos "Or$" ) ))
( "beeriest"  j ( b i@1 r i i s t ) ((pos "Os$" ) ))
( "beers"  n ( b i@1 z ) ((pos "Mj%" ) ))
( "beery"  j ( b i@1 r ii ) ((pos "OD%" ) ))
( "bees"  n ( b ii1 z ) ((pos "Kj%" ) ))
( "beeswax"  v ( b ii1 z w a k s ) ((pos "H1$" "L@%" ) ))
( "beeswaxed"  v ( b ii1 z w a k s t ) ((pos "Hc$" "Hd$" ) ))
( "beeswaxes"  v ( b ii1 z w a k s i z ) ((pos "Ha$" ) ))
( "beeswaxing"  v ( b ii1 z w a k s i ng ) ((pos "Hb$" ) ))
( "beet"  n ( b ii1 t ) ((pos "K6%" ) ))
( "beetle"  v ( b ii1 t l ) ((pos "I2%" "K6%" ) (subcat )))
( "beetle-browed"  j ( b ii1 t l - b r au d ) ((pos "OA%" ) ))
( "beetled"  v ( b ii1 t l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "beetles"  v ( b ii1 t l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "beetling"  v ( b ii1 t l i ng ) ((pos "Ib%" ) (subcat )))
( "beetroot"  n ( b ii1 t r uu t ) ((pos "K6%" ) ))
( "beetroots"  n ( b ii1 t r uu t s ) ((pos "Kj%" ) ))
( "beets"  n ( b ii1 t s ) ((pos "Kj%" ) ))
( "beeves"  n ( b ii1 v z ) ((pos "Kj$" ) ))
( "befall"  v ( b i1 f oo1 l ) ((pos "J5%" ) (subcat )))
( "befallen"  v ( b i1 f oo1 l @ n ) ((pos "Jd%" ) (subcat )))
( "befalling"  v ( b i1 f oo1 l i ng ) ((pos "Jb$" ) (subcat )))
( "befalls"  v ( b i1 f oo1 l z ) ((pos "Ja%" ) (subcat )))
( "befell"  v ( b i1 f e1 l ) ((pos "Jc%" ) (subcat )))
( "befit"  v ( b i1 f i1 t ) ((pos "H4%" ) (subcat )))
( "befits"  v ( b i1 f i1 t s ) ((pos "Ha%" ) (subcat )))
( "befitted"  v ( b i1 f i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "befitting"  v ( b i1 f i1 t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "befittingly"  a ( b i1 f i1 t i ng l ii ) ((pos "Pu%" ) ))
( "befogged"  j ( b i1 f o1 g d ) ((pos "Op$" ) ))
( "before"  a ( b i1 f oo1 r ) ((pos "Pu*" "T-%" "V-*" ) ))
( "beforehand"  j ( b i1 f oo1 h a n d ) ((pos "Op%" "Pu%" ) ))
( "befoul"  v ( b i1 f au1 l ) ((pos "H0%" ) ))
( "befouled"  v ( b i1 f au1 l d ) ((pos "Hc%" "Hd%" ) ))
( "befouling"  v ( b i1 f au1 l i ng ) ((pos "Hb%" ) ))
( "befouls"  v ( b i1 f au1 l z ) ((pos "Ha%" ) ))
( "befriend"  v ( b i1 f r e1 n d ) ((pos "H0%" ) (subcat )))
( "befriended"  v ( b i1 f r e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "befriending"  v ( b i1 f r e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "befriends"  v ( b i1 f r e1 n d z ) ((pos "Ha%" ) (subcat )))
( "beg"  v ( b e1 g ) ((pos "J4%" ) (subcat "2A" "2C" "3A" "6A" "7A" "9" "14" )))
( "begad"  uh ( b i1 g a1 d ) ((pos "W-$" ) ))
( "began"  v ( b i1 g a1 n ) ((pos "Jc*" ) (subcat "2A" "3A" "6A" "6D" )))
( "begat"  v ( b i1 g a1 t ) ((pos "Hc$" ) (subcat )))
( "beget"  v ( b i1 g e1 t ) ((pos "H5%" ) (subcat )))
( "begets"  v ( b i1 g e1 t s ) ((pos "Ha%" ) (subcat )))
( "begetter"  n ( b i1 g e1 t @ r ) ((pos "K6%" ) ))
( "begetters"  n ( b i1 g e1 t @ z ) ((pos "Kj%" ) ))
( "beggar"  v ( b e1 g @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "beggared"  v ( b e1 g @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beggaring"  v ( b e1 g @ r i ng ) ((pos "Hb%" ) (subcat )))
( "beggarly"  j ( b e1 g @ l ii ) ((pos "OA%" ) ))
( "beggarman"  n ( b e1 g @ m a n ) ((pos "Ki%" ) ))
( "beggarmen"  n ( b e1 g @ m e n ) ((pos "Kj%" ) ))
( "beggars"  v ( b e1 g @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "beggarwoman"  n ( b e1 g @ w u m @ n ) ((pos "Ki%" ) ))
( "beggarwomen"  n ( b e1 g @ w i m i n ) ((pos "Kj%" ) ))
( "beggary"  n ( b e1 g @ r ii ) ((pos "L@%" ) ))
( "begged"  v ( b e1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "7A" "9" "14" )))
( "begging"  v ( b e1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "7A" "9" "14" )))
( "begin"  v ( b i1 g i1 n ) ((pos "J5%" ) (subcat "2A" "3A" "6A" "6D" )))
( "beginner"  n ( b i1 g i1 n @ r ) ((pos "K6%" ) ))
( "beginners"  n ( b i1 g i1 n @ z ) ((pos "Kj%" ) ))
( "beginning"  v ( b i1 g i1 n i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "3A" "6A" "6D" )))
( "beginnings"  n ( b i1 g i1 n i ng z ) ((pos "Kj%" ) ))
( "begins"  v ( b i1 g i1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "6D" )))
( "begone"  nil ( b i1 g o1 n ) ((pos "Z-%" ) ))
( "begonia"  n ( b i1 g ou1 n i@ ) ((pos "K6%" ) ))
( "begonias"  n ( b i1 g ou1 n i@ z ) ((pos "Kj%" ) ))
( "begorra"  uh ( b i1 g o1 r @ ) ((pos "W-%" ) ))
( "begot"  v ( b i1 g o1 t ) ((pos "Hc%" ) (subcat )))
( "begotten"  v ( b i1 g o1 t n ) ((pos "Hd%" ) (subcat )))
( "begrimed"  j ( b i1 g r ai1 m d ) ((pos "Op$" ) ))
( "begrudge"  v ( b i1 g r uh1 jh ) ((pos "H2%" ) (subcat "6C" "12A" )))
( "begrudged"  v ( b i1 g r uh1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6C" "12A" )))
( "begrudges"  v ( b i1 g r uh1 jh i z ) ((pos "Ha%" ) (subcat "6C" "12A" )))
( "begrudging"  v ( b i1 g r uh1 jh i ng ) ((pos "Hb%" ) (subcat "6C" "12A" )))
( "begs"  v ( b e1 g z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "7A" "9" "14" )))
( "beguile"  v ( b i1 g ai1 l ) ((pos "H2%" ) (subcat "6A" )))
( "beguiled"  v ( b i1 g ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "beguiles"  v ( b i1 g ai1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "beguiling"  v ( b i1 g ai1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "begum"  n ( b ei1 g @ m ) ((pos "K6$" ) ))
( "begums"  n ( b ei1 g @ m z ) ((pos "Kj$" ) ))
( "begun"  v ( b i1 g uh1 n ) ((pos "Jd%" ) (subcat "2A" "3A" "6A" "6D" )))
( "behalf"  n ( b i1 h aa1 f ) ((pos "Ki%" ) ))
( "behalves"  n ( b i1 h aa1 v z ) ((pos "Kj$" ) ))
( "behave"  v ( b i1 h ei1 v ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "behaved"  v ( b i1 h ei1 v d ) ((pos "Ic%" "Id%" "Ot%" ) (subcat "2A" "2C" )))
( "behaves"  v ( b i1 h ei1 v z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "behaving"  v ( b i1 h ei1 v i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "behaviour"  n ( b i1 h ei1 v i@ r ) ((pos "L@%" ) ))
( "behaviourism"  n ( b i1 h ei1 v i@ r i z @ m ) ((pos "L@%" ) ))
( "behaviourist"  n ( b i1 h ei1 v i@ r i s t ) ((pos "K6%" ) ))
( "behaviourists"  n ( b i1 h ei1 v i@ r i s t s ) ((pos "Kj%" ) ))
( "behead"  v ( b i1 h e1 d ) ((pos "H0%" ) (subcat )))
( "beheaded"  v ( b i1 h e1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beheading"  v ( b i1 h e1 d i ng ) ((pos "Hb%" ) (subcat )))
( "beheads"  v ( b i1 h e1 d z ) ((pos "Ha%" ) (subcat )))
( "beheld"  v ( b i1 h e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "behest"  n ( b i1 h e1 s t ) ((pos "K6%" ) ))
( "behests"  n ( b i1 h e1 s t s ) ((pos "Kj$" ) ))
( "behind"  n ( b i1 h ai1 n d ) ((pos "K6%" "Pu*" "T-*" ) ))
( "behindhand"  j ( b i1 h ai1 n d h a n d ) ((pos "Op%" ) ))
( "behinds"  n ( b i1 h ai1 n d z ) ((pos "Kj%" ) ))
( "behold"  v ( b i1 h ou1 l d ) ((pos "H5%" ) (subcat )))
( "beholden"  j ( b i1 h ou1 l d @ n ) ((pos "Op%" ) ))
( "beholder"  n ( b i1 h ou1 l d @ r ) ((pos "K6%" ) ))
( "beholders"  n ( b i1 h ou1 l d @ z ) ((pos "Kj%" ) ))
( "beholding"  v ( b i1 h ou1 l d i ng ) ((pos "Hb%" ) (subcat )))
( "beholds"  v ( b i1 h ou1 l d z ) ((pos "Ha%" ) (subcat )))
( "behove"  v ( b i1 h ou1 v ) ((pos "H2%" ) ))
( "behoved"  v ( b i1 h ou1 v d ) ((pos "Hc%" "Hd%" ) ))
( "behoves"  v ( b i1 h ou1 v z ) ((pos "Ha%" ) ))
( "behoving"  v ( b i1 h ou1 v i ng ) ((pos "Hb%" ) ))
( "beige"  n ( b ei1 zh ) ((pos "L@%" ) ))
( "being"  v ( b ii1 i ng ) ((pos "Gb*" "Ib%" "M6%" ) (subcat )))
( "beings"  n ( b ii1 i ng z ) ((pos "Mj%" ) ))
( "bejewelled"  j ( b i1 jh uu1 @ l d ) ((pos "OA%" ) ))
( "belabour"  v ( b i1 l ei1 b @ r ) ((pos "H0%" ) (subcat )))
( "belaboured"  v ( b i1 l ei1 b @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "belabouring"  v ( b i1 l ei1 b @ r i ng ) ((pos "Hb%" ) (subcat )))
( "belabours"  v ( b i1 l ei1 b @ z ) ((pos "Ha%" ) (subcat )))
( "belated"  j ( b i1 l ei1 t i d ) ((pos "OA%" ) ))
( "belatedly"  a ( b i1 l ei1 t i d l ii ) ((pos "Pu%" ) ))
( "belay"  v ( b i1 l ei1 ) ((pos "H0$" "K6$" ) (subcat )))
( "belayed"  v ( b i1 l ei1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "belaying"  v ( b i1 l ei1 i ng ) ((pos "Hb$" ) (subcat )))
( "belaying-pin"  n ( b i1 l ei1 i ng - p i n ) ((pos "K6$" ) ))
( "belaying-pins"  n ( b i1 l ei1 i ng - p i n z ) ((pos "Kj$" ) ))
( "belays"  v ( b i1 l ei1 z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "belch"  v ( b e1 l ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" )))
( "belched"  v ( b e1 l ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "belches"  v ( b e1 l ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "belching"  v ( b e1 l ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "beldam"  n ( b e1 l d @ m ) ((pos "K6$" ) ))
( "beldame"  n ( b e1 l d @ m ) ((pos "K6$" ) ))
( "beldames"  n ( b e1 l d @ m z ) ((pos "Kj$" ) ))
( "beldams"  n ( b e1 l d @ m z ) ((pos "Kj$" ) ))
( "beleaguer"  v ( b i1 l ii1 g @ r ) ((pos "H0$" ) (subcat )))
( "beleaguered"  v ( b i1 l ii1 g @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "beleaguering"  v ( b i1 l ii1 g @ r i ng ) ((pos "Hb$" ) (subcat )))
( "beleaguers"  v ( b i1 l ii1 g @ z ) ((pos "Ha$" ) (subcat )))
( "belfries"  n ( b e1 l f r i z ) ((pos "Kj%" ) ))
( "belfry"  n ( b e1 l f r ii ) ((pos "K8%" ) ))
( "belie"  v ( b i1 l ai1 ) ((pos "H5%" ) (subcat )))
( "belied"  v ( b i1 l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "belief"  n ( b i1 l ii1 f ) ((pos "M6%" ) ))
( "beliefs"  n ( b i1 l ii1 f s ) ((pos "Mj%" ) ))
( "belies"  v ( b i1 l ai1 z ) ((pos "Ha%" ) (subcat )))
( "believable"  j ( b i1 l ii1 v @ b l ) ((pos "OA%" ) ))
( "believe"  v ( b i1 l ii1 v ) ((pos "J2*" ) (subcat "3A" "6A" "9" "10" )))
( "believed"  v ( b i1 l ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" "10" )))
( "believer"  n ( b i1 l ii1 v @ r ) ((pos "K6%" ) ))
( "believers"  n ( b i1 l ii1 v @ z ) ((pos "Kj%" ) ))
( "believes"  v ( b i1 l ii1 v z ) ((pos "Ja%" ) (subcat "3A" "6A" "9" "10" )))
( "believing"  v ( b i1 l ii1 v i ng ) ((pos "Jb%" "L@%" ) (subcat "3A" "6A" "9" "10" )))
( "belike"  a ( b i1 l ai1 k ) ((pos "Pu%" ) ))
( "belittle"  v ( b i1 l i1 t l ) ((pos "H2%" ) (subcat )))
( "belittled"  v ( b i1 l i1 t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "belittles"  v ( b i1 l i1 t l z ) ((pos "Ha%" ) (subcat )))
( "belittling"  v ( b i1 l i1 t l i ng ) ((pos "Hb%" ) (subcat )))
( "bell"  v ( b e1 l ) ((pos "H0$" "K6%" ) ))
( "bell-bottomed"  j ( b e1 l - b o t @ m d ) ((pos "OA%" ) ))
( "bell-bottoms"  n ( b e1 l - b o t @ m z ) ((pos "Kj%" ) ))
( "bell-buoy"  n ( b e1 l - b oi ) ((pos "K6$" ) ))
( "bell-buoys"  n ( b e1 l - b oi z ) ((pos "Kj$" ) ))
( "bell-flower"  n ( b e1 l - f l au @ r ) ((pos "K6%" ) ))
( "bell-flowers"  n ( b e1 l - f l au @ z ) ((pos "Kj%" ) ))
( "bell-founder"  n ( b e1 l - f au n d @ r ) ((pos "K6$" ) ))
( "bell-founders"  n ( b e1 l - f au n d @ z ) ((pos "Kj$" ) ))
( "bell-foundries"  n ( b e1 l - f au n d r i z ) ((pos "Kj%" ) ))
( "bell-foundry"  n ( b e1 l - f au n d r ii ) ((pos "K8%" ) ))
( "bell-metal"  n ( b e1 l - m e t l ) ((pos "L@$" ) ))
( "bell-push"  n ( b e1 l - p u sh ) ((pos "K7%" ) ))
( "bell-pushes"  n ( b e1 l - p u sh i z ) ((pos "Kj%" ) ))
( "bell-ringer"  n ( b e1 l - r i ng @ r ) ((pos "K6%" ) ))
( "bell-ringers"  n ( b e1 l - r i ng @ z ) ((pos "Kj%" ) ))
( "bell-tent"  n ( b e1 l - t e n t ) ((pos "K6%" ) ))
( "bell-tents"  n ( b e1 l - t e n t s ) ((pos "Kj%" ) ))
( "belladonna"  n ( b e2 l @ d o1 n @ ) ((pos "M6%" ) ))
( "belladonnas"  n ( b e2 l @ d o1 n @ z ) ((pos "Mj$" ) ))
( "bellboy"  n ( b e1 l b oi ) ((pos "K6%" ) ))
( "bellboys"  n ( b e1 l b oi z ) ((pos "Kj%" ) ))
( "belle"  n ( b e1 l ) ((pos "K6%" ) ))
( "belled"  v ( b e1 l d ) ((pos "Hc$" "Hd$" ) ))
( "belles"  n ( b e1 l z ) ((pos "Kj%" ) ))
( "belles-lettres"  n ( b e2 l - l e1 t r @ ) ((pos "Kj%" ) ))
( "bellhop"  n ( b e1 l h o p ) ((pos "K6%" ) ))
( "bellhops"  n ( b e1 l h o p s ) ((pos "Kj%" ) ))
( "bellicose"  j ( b e1 l i k ou s ) ((pos "OA%" ) ))
( "bellied"  v ( b e1 l i d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2A" "2C" "6A" )))
( "bellies"  v ( b e1 l i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "belligerency"  n ( b i1 l i1 jh @ r @ n s ii ) ((pos "L@%" ) ))
( "belligerent"  n ( b i1 l i1 jh @ r @ n t ) ((pos "K6$" "OA%" ) ))
( "belligerently"  a ( b i1 l i1 jh @ r @ n t l ii ) ((pos "Pu%" ) ))
( "belligerents"  n ( b i1 l i1 jh @ r @ n t s ) ((pos "Kj$" ) ))
( "belling"  v ( b e1 l i ng ) ((pos "Hb$" ) ))
( "bellow"  v ( b e1 l ou ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "bellowed"  v ( b e1 l ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "bellowing"  v ( b e1 l ou i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "bellows"  v ( b e1 l ou z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "bells"  v ( b e1 l z ) ((pos "Ha%" "Kj%" ) ))
( "bellwether"  n ( b e1 l w e dh @ r ) ((pos "K6$" ) ))
( "bellwethers"  n ( b e1 l w e dh @ z ) ((pos "Kj$" ) ))
( "belly"  v ( b e1 l ii ) ((pos "J3$" "K8%" ) (subcat "2A" "2C" "6A" )))
( "bellyache"  v ( b e1 l i ei k ) ((pos "I2%" "K6%" ) ))
( "bellyached"  v ( b e1 l i ei k t ) ((pos "Ic%" "Id%" ) ))
( "bellyaches"  v ( b e1 l i ei k s ) ((pos "Ia%" "Kj%" ) ))
( "bellyaching"  v ( b e1 l i ei k i ng ) ((pos "Ib%" ) ))
( "bellyflop"  n ( b e1 l i f l o p ) ((pos "K6%" ) ))
( "bellyflops"  n ( b e1 l i f l o p s ) ((pos "Kj%" ) ))
( "bellyful"  n ( b e1 l i f u l ) ((pos "K6%" ) ))
( "bellyfuls"  n ( b e1 l i f u l z ) ((pos "Kj%" ) ))
( "bellying"  v ( b e1 l i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "bellyland"  v ( b e1 l i l a n d ) ((pos "I0%" ) ))
( "bellylanded"  v ( b e1 l i l a n d i d ) ((pos "Ic%" "Id%" ) ))
( "bellylanding"  v ( b e1 l i l a n d i ng ) ((pos "Ib%" ) ))
( "bellylands"  v ( b e1 l i l a n d z ) ((pos "Ia$" ) ))
( "bellylaugh"  v ( b e1 l i l a f ) ((pos "I0$" "K6%" ) ))
( "bellylaughed"  v ( b e1 l i l a f t ) ((pos "Ic$" "Id$" ) ))
( "bellylaughing"  v ( b e1 l i l a f i ng ) ((pos "Ib$" ) ))
( "bellylaughs"  v ( b e1 l i l a f s ) ((pos "Ia$" "Kj%" ) ))
( "belong"  v ( b i1 l o1 ng ) ((pos "I0%" ) (subcat "2C" )))
( "belonged"  v ( b i1 l o1 ng d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "belonging"  v ( b i1 l o1 ng i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "belongings"  n ( b i1 l o1 ng i ng z ) ((pos "Kj%" ) ))
( "belongs"  v ( b i1 l o1 ng z ) ((pos "Ia%" ) (subcat "2C" )))
( "beloved"  n ( b i1 l uh1 v i d ) ((pos "K6%" "OA%" ) ))
( "beloveds"  n ( b i1 l uh1 v i d z ) ((pos "Kj%" ) ))
( "below"  a ( b i1 l ou1 ) ((pos "Pu%" "T-%" ) ))
( "belt"  v ( b e1 l t ) ((pos "H0%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "belted"  v ( b e1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat "2C" "3A" "6A" )))
( "belting"  v ( b e1 l t i ng ) ((pos "Hb%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "beltings"  n ( b e1 l t i ng z ) ((pos "Kj%" ) ))
( "belts"  v ( b e1 l t s ) ((pos "Ha%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "belying"  v ( b i1 l ai1 i ng ) ((pos "Hb%" ) (subcat )))
( "bemoan"  v ( b i1 m ou1 n ) ((pos "H0%" ) (subcat )))
( "bemoaned"  v ( b i1 m ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bemoaning"  v ( b i1 m ou1 n i ng ) ((pos "Hb%" ) (subcat )))
( "bemoans"  v ( b i1 m ou1 n z ) ((pos "Ha%" ) (subcat )))
( "bemused"  j ( b i1 m y uu1 z d ) ((pos "Op%" ) ))
( "ben"  n ( b e1 n ) ((pos "K6$" ) ))
( "bench"  n ( b e1 n ch ) ((pos "K7%" ) ))
( "benches"  n ( b e1 n ch i z ) ((pos "Kj%" ) ))
( "bend"  v ( b e1 n d ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "bended"  v ( b e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "bending"  v ( b e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "bends"  v ( b e1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "beneath"  a ( b i1 n ii1 th ) ((pos "Pu%" "T-%" ) ))
( "benedick"  n ( b e1 n i d i k ) ((pos "K6%" ) ))
( "benedicks"  n ( b e1 n i d i k s ) ((pos "Kj$" ) ))
( "benediction"  n ( b e2 n i d i1 k sh @ n ) ((pos "K6$" ) ))
( "benedictions"  n ( b e2 n i d i1 k sh @ n z ) ((pos "Kj%" ) ))
( "benefaction"  n ( b e2 n i f a1 k sh @ n ) ((pos "M6$" ) ))
( "benefactions"  n ( b e2 n i f a1 k sh @ n z ) ((pos "Mj$" ) ))
( "benefactor"  n ( b e1 n i f a k t @ r ) ((pos "K6%" ) ))
( "benefactors"  n ( b e1 n i f a k t @ z ) ((pos "Kj%" ) ))
( "benefactress"  n ( b e1 n i f a k t r i s ) ((pos "K7%" ) ))
( "benefactresses"  n ( b e1 n i f a k t r i s i z ) ((pos "Kj$" ) ))
( "benefice"  n ( b e1 n i f i s ) ((pos "K6$" ) ))
( "beneficed"  j ( b e1 n i f i s t ) ((pos "OA$" ) ))
( "beneficence"  n ( b i1 n e1 f i s n s ) ((pos "L@%" ) ))
( "beneficent"  j ( b i1 n e1 f i s n t ) ((pos "OA%" ) ))
( "benefices"  n ( b e1 n i f i s i z ) ((pos "Kj$" ) ))
( "beneficial"  j ( b e2 n i f i1 sh l ) ((pos "OA%" ) ))
( "beneficially"  a ( b e2 n i f i1 sh @ l ii ) ((pos "Pu%" ) ))
( "beneficiaries"  n ( b e2 n i f i1 sh @ r i z ) ((pos "Kj%" ) ))
( "beneficiary"  n ( b e2 n i f i1 sh @ r ii ) ((pos "K8%" ) ))
( "benefit"  v ( b e1 n i f i t ) ((pos "J0%" "M6%" ) (subcat "3A" )))
( "benefited"  v ( b e1 n i f i t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "benefiting"  v ( b e1 n i f i t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "benefits"  v ( b e1 n i f i t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "benevolence"  n ( b i1 n e1 v @ l @ n s ) ((pos "L@%" ) ))
( "benevolent"  j ( b i1 n e1 v @ l @ n t ) ((pos "OA%" ) ))
( "benevolently"  a ( b i1 n e1 v @ l @ n t l ii ) ((pos "Pu%" ) ))
( "benighted"  j ( b i1 n ai1 t i d ) ((pos "OA%" ) ))
( "benign"  j ( b i1 n ai1 n ) ((pos "OA%" ) ))
( "benignant"  j ( b i1 n i1 g n @ n t ) ((pos "OA$" ) ))
( "benignantly"  a ( b i1 n i1 g n @ n t l ii ) ((pos "Pu$" ) ))
( "benignities"  n ( b i1 n i1 g n i t i z ) ((pos "Mj$" ) ))
( "benignity"  n ( b i1 n i1 g n i t ii ) ((pos "M8$" ) ))
( "benignly"  a ( b i1 n ai1 n l ii ) ((pos "Pu%" ) ))
( "benison"  n ( b e1 n i z n ) ((pos "K6$" ) ))
( "benisons"  n ( b e1 n i z n z ) ((pos "Kj$" ) ))
( "bens"  n ( b e1 n z ) ((pos "Kj$" ) ))
( "bent"  v ( b e1 n t ) ((pos "Jc%" "Jd%" "K6%" "Op%" ) (subcat "2A" "2C" "6A" "15A" )))
( "bents"  n ( b e1 n t s ) ((pos "Kj$" ) ))
( "benumbed"  j ( b i1 n uh1 m d ) ((pos "Op%" ) ))
( "benzene"  n ( b e1 n z ii n ) ((pos "L@%" ) ))
( "benzine"  n ( b e1 n z ii n ) ((pos "L@%" ) ))
( "benzol"  n ( b e1 n z o l ) ((pos "L@$" ) ))
( "bequeath"  v ( b i1 k w ii1 dh ) ((pos "H0%" ) (subcat "6A" "12A" )))
( "bequeathed"  v ( b i1 k w ii1 dh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" )))
( "bequeathing"  v ( b i1 k w ii1 dh i ng ) ((pos "Hb%" ) (subcat "6A" "12A" )))
( "bequeaths"  v ( b i1 k w ii1 dh z ) ((pos "Ha%" ) (subcat "6A" "12A" )))
( "bequest"  n ( b i1 k w e1 s t ) ((pos "M6%" ) ))
( "bequests"  n ( b i1 k w e1 s t s ) ((pos "Mj%" ) ))
( "berate"  v ( b i1 r ei1 t ) ((pos "H2%" ) (subcat )))
( "berated"  v ( b i1 r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "berates"  v ( b i1 r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "berating"  v ( b i1 r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "bereave"  v ( b i1 r ii1 v ) ((pos "H2%" ) (subcat )))
( "bereaved"  v ( b i1 r ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bereavement"  n ( b i1 r ii1 v m @ n t ) ((pos "M6%" ) ))
( "bereavements"  n ( b i1 r ii1 v m @ n t s ) ((pos "Mj%" ) ))
( "bereaves"  v ( b i1 r ii1 v z ) ((pos "Ha%" ) (subcat )))
( "bereaving"  v ( b i1 r ii1 v i ng ) ((pos "Hb%" ) (subcat )))
( "bereft"  v ( b i1 r e1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "beret"  n ( b e1 r ei ) ((pos "K6%" ) ))
( "berets"  n ( b e1 r ei z ) ((pos "Kj%" ) ))
( "berg"  n ( b @@1 g ) ((pos "K6$" ) ))
( "bergs"  n ( b @@1 g z ) ((pos "Kj$" ) ))
( "beri-beri"  n ( b e2 r i - b e1 r ii ) ((pos "L@$" ) ))
( "berries"  n ( b e1 r i z ) ((pos "Kj%" ) ))
( "berry"  n ( b e1 r ii ) ((pos "K8%" ) ))
( "berserk"  j ( b @1 s @@1 k ) ((pos "Op%" ) ))
( "berth"  v ( b @@1 th ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "berthed"  v ( b @@1 th t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "berthing"  v ( b @@1 th i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "berths"  v ( b @@1 th s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "beryl"  n ( b e1 r @ l ) ((pos "K6$" ) ))
( "beryls"  n ( b e1 r @ l z ) ((pos "Kj$" ) ))
( "beseech"  v ( b i1 s ii1 ch ) ((pos "H1%" ) (subcat "6A" "11" "13B" )))
( "beseeched"  v ( b i1 s ii1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "13B" )))
( "beseeches"  v ( b i1 s ii1 ch i z ) ((pos "Ha%" ) (subcat "6A" "11" "13B" )))
( "beseeching"  v ( b i1 s ii1 ch i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "11" "13B" )))
( "beseechingly"  a ( b i1 s ii1 ch i ng l ii ) ((pos "Pu%" ) ))
( "beseem"  v ( b i1 s ii1 m ) ((pos "H0$" ) ))
( "beseemed"  v ( b i1 s ii1 m d ) ((pos "Hc$" "Hd$" ) ))
( "beseeming"  v ( b i1 s ii1 m i ng ) ((pos "Hb$" ) ))
( "beseems"  v ( b i1 s ii1 m z ) ((pos "Ha$" ) ))
( "beset"  v ( b i1 s e1 t ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "besets"  v ( b i1 s e1 t s ) ((pos "Ha%" ) (subcat )))
( "besetting"  v ( b i1 s e1 t i ng ) ((pos "Hb%" ) (subcat )))
( "beshrew"  v ( b i1 sh r uu1 ) ((pos "H0$" ) ))
( "beshrewed"  v ( b i1 sh r uu1 d ) ((pos "Hc$" "Hd$" ) ))
( "beshrewing"  v ( b i1 sh r uu1 i ng ) ((pos "Hb$" ) ))
( "beshrews"  v ( b i1 sh r uu1 z ) ((pos "Ha$" ) ))
( "beside"  in ( b i1 s ai1 d ) ((pos "T-%" ) ))
( "besides"  a ( b i1 s ai1 d z ) ((pos "Pu%" "T-%" ) ))
( "besiege"  v ( b i1 s ii1 jh ) ((pos "H2%" ) (subcat "6A" )))
( "besieged"  v ( b i1 s ii1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "besieger"  n ( b i1 s ii1 jh @ r ) ((pos "K6%" ) ))
( "besiegers"  n ( b i1 s ii1 jh @ z ) ((pos "Kj%" ) ))
( "besieges"  v ( b i1 s ii1 jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "besieging"  v ( b i1 s ii1 jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "besmear"  v ( b i1 s m i@1 r ) ((pos "H0$" ) ))
( "besmeared"  v ( b i1 s m i@1 d ) ((pos "Hc%" "Hd%" ) ))
( "besmearing"  v ( b i1 s m i@1 r i ng ) ((pos "Hb$" ) ))
( "besmears"  v ( b i1 s m i@1 z ) ((pos "Ha$" ) ))
( "besmirch"  v ( b i1 s m @@1 ch ) ((pos "H1$" ) ))
( "besmirched"  v ( b i1 s m @@1 ch t ) ((pos "Hc%" "Hd%" ) ))
( "besmirches"  v ( b i1 s m @@1 ch i z ) ((pos "Ha$" ) ))
( "besmirching"  v ( b i1 s m @@1 ch i ng ) ((pos "Hb$" ) ))
( "besom"  n ( b ii1 z @ m ) ((pos "K6$" ) ))
( "besoms"  n ( b ii1 z @ m z ) ((pos "Kj$" ) ))
( "besotted"  j ( b i1 s o1 t i d ) ((pos "OA%" ) ))
( "besought"  v ( b i1 s oo1 t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "13B" )))
( "bespangled"  j ( b i1 s p a1 ng g l d ) ((pos "Op%" ) ))
( "bespattered"  j ( b i1 s p a1 t @ d ) ((pos "Op%" ) ))
( "bespeak"  v ( b i1 s p ii1 k ) ((pos "H5$" ) (subcat "6A" )))
( "bespeaking"  v ( b i1 s p ii1 k i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "bespeaks"  v ( b i1 s p ii1 k s ) ((pos "Ha$" ) (subcat "6A" )))
( "bespectacled"  j ( b i1 s p e1 k t @ k @ l d ) ((pos "OA%" ) ))
( "bespoke"  v ( b i1 s p ou1 k ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bespoken"  v ( b i1 s p ou1 k @ n ) ((pos "Hd$" ) (subcat "6A" )))
( "best"  v ( b e s t ) ((pos "H0$" "Os*" "Pu*" "Qx%" ) (subcat )))
( "best-seller"  n ( b e1 s t - s e1 l @ r ) ((pos "K6%" ) ))
( "best-sellers"  n ( b e1 s t - s e1 l @ z ) ((pos "Kj%" ) ))
( "bested"  v ( b e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bestial"  j ( b e1 s t i@ l ) ((pos "OA%" ) ))
( "bestialities"  n ( b e2 s t i a1 l i t i z ) ((pos "Mj$" ) ))
( "bestiality"  n ( b e2 s t i a1 l i t ii ) ((pos "M8%" ) ))
( "bestially"  a ( b e1 s t i@ l ii ) ((pos "Pu%" ) ))
( "bestiaries"  n ( b e1 s t i@ r i z ) ((pos "Kj%" ) ))
( "bestiary"  n ( b e1 s t i@ r ii ) ((pos "K8%" ) ))
( "besting"  v ( b e1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "bestir"  v ( b i1 s t @@1 r ) ((pos "H4%" ) (subcat "6A" )))
( "bestirred"  v ( b i1 s t @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bestirring"  v ( b i1 s t @@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bestirs"  v ( b i1 s t @@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "bestow"  v ( b i1 s t ou1 ) ((pos "H0%" ) (subcat "6A" )))
( "bestowal"  n ( b i1 s t ou1 @ l ) ((pos "K6$" ) ))
( "bestowals"  n ( b i1 s t ou1 @ l z ) ((pos "Kj$" ) ))
( "bestowed"  v ( b i1 s t ou1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bestowing"  v ( b i1 s t ou1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bestows"  v ( b i1 s t ou1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "bestrew"  v ( b i1 s t r uu1 ) ((pos "H0%" ) (subcat "6A" )))
( "bestrewed"  v ( b i1 s t r uu1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bestrewing"  v ( b i1 s t r uu1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bestrewn"  v ( b i1 s t r uu1 n ) ((pos "Hd%" ) (subcat "6A" )))
( "bestrews"  v ( b i1 s t r uu1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "bestrid"  v ( b i1 s t r i1 d ) ((pos "Hd$" ) (subcat )))
( "bestridden"  v ( b i1 s t r i1 d n ) ((pos "Hd$" ) (subcat )))
( "bestride"  v ( b i1 s t r ai1 d ) ((pos "H5%" ) (subcat )))
( "bestrides"  v ( b i1 s t r ai1 d z ) ((pos "Ha%" ) (subcat )))
( "bestriding"  v ( b i1 s t r ai1 d i ng ) ((pos "Hb%" ) (subcat )))
( "bestrode"  v ( b i1 s t r ou1 d ) ((pos "Hc$" ) (subcat )))
( "bests"  v ( b e1 s t s ) ((pos "Ha%" ) (subcat )))
( "bet"  v ( b e1 t ) ((pos "J4%" "Jc%" "Jd%" "K6%" ) (subcat "2A" "3A" "9" "11" )))
( "beta"  n ( b ii1 t @ ) ((pos "K6%" ) ))
( "betake"  v ( b i1 t ei1 k ) ((pos "H5$" ) (subcat )))
( "betaken"  v ( b i1 t ei1 k @ n ) ((pos "Hd$" ) (subcat )))
( "betakes"  v ( b i1 t ei1 k s ) ((pos "Ha$" ) (subcat )))
( "betaking"  v ( b i1 t ei1 k i ng ) ((pos "Hb$" ) (subcat )))
( "betas"  n ( b ii1 t @ z ) ((pos "Kj$" ) ))
( "betel"  n ( b ii1 t l ) ((pos "K6$" ) ))
( "betel-nut"  n ( b ii1 t l - n uh t ) ((pos "K6%" ) ))
( "betel-nuts"  n ( b ii1 t l - n uh t s ) ((pos "Kj%" ) ))
( "betels"  n ( b ii1 t l z ) ((pos "Kj$" ) ))
( "bethel"  n ( b e1 th l ) ((pos "K6$" ) ))
( "bethels"  n ( b e1 th l z ) ((pos "Kj$" ) ))
( "bethink"  v ( b i1 th i1 ng k ) ((pos "H5$" ) (subcat "11" "14" "17" "20" )))
( "bethinking"  v ( b i1 th i1 ng k i ng ) ((pos "Hb$" ) (subcat "11" "14" "17" "20" )))
( "bethinks"  v ( b i1 th i1 ng k s ) ((pos "Ha$" ) (subcat "11" "14" "17" "20" )))
( "bethought"  v ( b i1 th oo1 t ) ((pos "Hc%" "Hd%" ) (subcat "11" "14" "17" "20" )))
( "betide"  v ( b i1 t ai1 d ) ((pos "H2%" ) ))
( "betided"  v ( b i1 t ai1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "betides"  v ( b i1 t ai1 d z ) ((pos "Ha$" ) ))
( "betiding"  v ( b i1 t ai1 d i ng ) ((pos "Hb$" ) ))
( "betimes"  a ( b i1 t ai1 m z ) ((pos "Pu%" ) ))
( "betoken"  v ( b i1 t ou1 k @ n ) ((pos "H0%" ) (subcat )))
( "betokened"  v ( b i1 t ou1 k @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "betokening"  v ( b i1 t ou1 k @ n i ng ) ((pos "Hb$" ) (subcat )))
( "betokens"  v ( b i1 t ou1 k @ n z ) ((pos "Ha%" ) (subcat )))
( "betook"  v ( b i1 t u1 k ) ((pos "Hc%" ) (subcat )))
( "betray"  v ( b i1 t r ei1 ) ((pos "H0%" ) (subcat "6A" "14" )))
( "betrayal"  n ( b i1 t r ei1 @ l ) ((pos "M6%" ) ))
( "betrayals"  n ( b i1 t r ei1 @ l z ) ((pos "Mj%" ) ))
( "betrayed"  v ( b i1 t r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "betrayer"  n ( b i1 t r ei1 @ r ) ((pos "K6%" ) ))
( "betrayers"  n ( b i1 t r ei1 @ z ) ((pos "Kj%" ) ))
( "betraying"  v ( b i1 t r ei1 i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "betrays"  v ( b i1 t r ei1 z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "betroth"  v ( b i1 t r ou1 dh ) ((pos "H0$" ) (subcat "6A" )))
( "betrothal"  n ( b i1 t r ou1 dh @ l ) ((pos "K6%" ) ))
( "betrothals"  n ( b i1 t r ou1 dh @ l z ) ((pos "Kj$" ) ))
( "betrothed"  v ( b i1 t r ou1 dh d ) ((pos "Hc$" "Hd%" "K6%" ) (subcat "6A" )))
( "betrotheds"  n ( b i1 t r ou1 dh d z ) ((pos "Kj$" ) ))
( "betrothing"  v ( b i1 t r ou1 dh i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "betroths"  v ( b i1 t r ou1 dh z ) ((pos "Ha$" ) (subcat "6A" )))
( "bets"  v ( b e1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "9" "11" )))
( "betted"  v ( b e1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "9" "11" )))
( "better"  v ( b e1 t @ r ) ((pos "H0%" "K6%" "Or*" "Pu*" ) (subcat )))
( "bettered"  v ( b e1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bettering"  v ( b e1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "betterment"  n ( b e1 t @ m @ n t ) ((pos "L@%" ) ))
( "betters"  v ( b e1 t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "betting"  v ( b e1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "9" "11" )))
( "bettor"  n ( b e1 t @ r ) ((pos "K6$" ) ))
( "bettors"  n ( b e1 t @ z ) ((pos "Kj$" ) ))
( "between"  a ( b i1 t w ii1 n ) ((pos "Pu*" "T-*" ) ))
( "betwixt"  a ( b i1 t w i1 k s t ) ((pos "Pu%" "T-%" ) ))
( "bevel"  v ( b e1 v @ l ) ((pos "H4%" "K6%" ) ))
( "bevelled"  v ( b e1 v @ l d ) ((pos "Hc%" "Hd%" ) ))
( "bevelling"  v ( b e1 v @ l i ng ) ((pos "Hb%" ) ))
( "bevels"  v ( b e1 v @ l z ) ((pos "Ha%" "Kj%" ) ))
( "beverage"  n ( b e1 v @ r i jh ) ((pos "K6%" ) ))
( "beverages"  n ( b e1 v @ r i jh i z ) ((pos "Kj%" ) ))
( "bevies"  n ( b e1 v i z ) ((pos "Kj$" ) ))
( "bevy"  n ( b e1 v ii ) ((pos "K8%" ) ))
( "bewail"  v ( b i1 w ei1 l ) ((pos "H0%" ) (subcat )))
( "bewailed"  v ( b i1 w ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bewailing"  v ( b i1 w ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "bewails"  v ( b i1 w ei1 l z ) ((pos "Ha%" ) (subcat )))
( "beware"  v ( b i1 w e@1 r ) ((pos "J5%" ) (subcat "2A" "3A" )))
( "bewilder"  v ( b i1 w i1 l d @ r ) ((pos "H0%" ) (subcat )))
( "bewildered"  v ( b i1 w i1 l d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bewildering"  v ( b i1 w i1 l d @ r i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "bewilderment"  n ( b i1 w i1 l d @ m @ n t ) ((pos "L@%" ) ))
( "bewilders"  v ( b i1 w i1 l d @ z ) ((pos "Ha%" ) (subcat )))
( "bewitch"  v ( b i1 w i1 ch ) ((pos "H1%" ) (subcat )))
( "bewitched"  v ( b i1 w i1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bewitches"  v ( b i1 w i1 ch i z ) ((pos "Ha%" ) (subcat )))
( "bewitching"  v ( b i1 w i1 ch i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "bewitchingly"  a ( b i1 w i1 ch i ng l ii ) ((pos "Pu%" ) ))
( "bey"  n ( b ei1 ) ((pos "K6$" ) ))
( "beyond"  a ( b i1 y o1 n d ) ((pos "Pu%" "T-%" ) ))
( "beys"  n ( b ei1 z ) ((pos "Kj$" ) ))
( "bezique"  n ( b @1 z ii1 k ) ((pos "L@$" ) ))
( "bhang"  n ( b a1 ng ) ((pos "K6$" ) ))
( "bhangs"  n ( b a1 ng z ) ((pos "Kj$" ) ))
( "bi-"  j ( b ai2 - ) ((pos "U-%" ) ))
( "bias"  v ( b ai1 @ s ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "biased"  v ( b ai1 @ s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "biases"  v ( b ai1 @ s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "biasing"  v ( b ai1 @ s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "biassed"  v ( b ai1 @ s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bib"  v ( b i1 b ) ((pos "I4$" "K6%" ) ))
( "bibbed"  v ( b i1 b d ) ((pos "Ic$" "Id$" ) ))
( "bibbing"  v ( b i1 b i ng ) ((pos "Ib$" ) ))
( "biblical"  j ( b i1 b l i k l ) ((pos "OA%" ) ))
( "bibliographer"  n ( b i2 b l i o1 g r @ f @ r ) ((pos "K6%" ) ))
( "bibliographers"  n ( b i2 b l i o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "bibliographies"  n ( b i2 b l i o1 g r @ f i z ) ((pos "Mj%" ) ))
( "bibliography"  n ( b i2 b l i o1 g r @ f ii ) ((pos "M8%" ) ))
( "bibliophile"  n ( b i1 b l i@ f ai l ) ((pos "K6%" ) ))
( "bibliophiles"  n ( b i1 b l i@ f ai l z ) ((pos "Kj%" ) ))
( "bibs"  v ( b i1 b z ) ((pos "Ia%" "Kj%" ) ))
( "bibulous"  j ( b i1 b y u l @ s ) ((pos "OA$" ) ))
( "bicameral"  j ( b ai2 k a1 m @ r @ l ) ((pos "OA$" ) ))
( "bicarbonate"  n ( b ai2 k aa1 b @ n @ t ) ((pos "L@%" ) ))
( "bicentenaries"  n ( b ai2 s e n t ii1 n @ r i z ) ((pos "Kj%" ) ))
( "bicentenary"  n ( b ai2 s e n t ii1 n @ r ii ) ((pos "K8%" ) ))
( "bicentennial"  n ( b ai2 s e n t e1 n i@ l ) ((pos "K6%" "OA%" ) ))
( "bicentennials"  n ( b ai2 s e n t e1 n i@ l z ) ((pos "Kj%" ) ))
( "biceps"  n ( b ai1 s e p s ) ((pos "K9%" ) ))
( "bicker"  v ( b i1 k @ r ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "bickered"  v ( b i1 k @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "bickering"  v ( b i1 k @ r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "bickers"  v ( b i1 k @ z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "bicycle"  v ( b ai1 s i k l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "bicycled"  v ( b ai1 s i k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "bicycles"  v ( b ai1 s i k l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "bicycling"  v ( b ai1 s i k l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "bid"  v ( b i1 d ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2A" "3A" "6A" "12A" "13A" "14" "15B" "17" )))
( "biddable"  j ( b i1 d @ b l ) ((pos "OA%" ) ))
( "bidden"  v ( b i1 d n ) ((pos "Jd%" ) (subcat "12A" "13A" "17" )))
( "bidder"  n ( b i1 d @ r ) ((pos "K6%" ) ))
( "bidders"  n ( b i1 d @ z ) ((pos "Kj%" ) ))
( "bidding"  v ( b i1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" "6A" "12A" "13A" "14" "15B" "17" )))
( "bide"  v ( b ai1 d ) ((pos "H2%" ) ))
( "bided"  v ( b ai1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "bides"  v ( b ai1 d z ) ((pos "Ha%" ) ))
( "bidet"  n ( b ii1 d ei ) ((pos "K6%" ) ))
( "bidets"  n ( b ii1 d ei z ) ((pos "Kj%" ) ))
( "biding"  v ( b ai1 d i ng ) ((pos "Hb%" ) ))
( "bids"  v ( b i1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "12A" "13A" "14" "15B" "17" )))
( "biennial"  n ( b ai1 e1 n i@ l ) ((pos "K6%" "OA%" ) ))
( "biennially"  a ( b ai1 e1 n i@ l ii ) ((pos "Pu%" ) ))
( "biennials"  n ( b ai1 e1 n i@ l z ) ((pos "Kj%" ) ))
( "bier"  n ( b i@1 r ) ((pos "K6%" ) ))
( "biers"  n ( b i@1 z ) ((pos "Kj%" ) ))
( "biff"  v ( b i1 f ) ((pos "H0%" "K6%" ) ))
( "biffed"  v ( b i1 f t ) ((pos "Hc%" "Hd%" ) ))
( "biffing"  v ( b i1 f i ng ) ((pos "Hb%" ) ))
( "biffs"  v ( b i1 f s ) ((pos "Ha%" "Kj%" ) ))
( "bifocal"  j ( b ai2 f ou1 k l ) ((pos "OA%" ) ))
( "bifocals"  n ( b ai2 f ou1 k l z ) ((pos "Kj%" ) ))
( "bifurcate"  v ( b ai1 f @ k ei t ) ((pos "J2%" "OA$" ) (subcat "2A" )))
( "bifurcated"  v ( b ai1 f @ k ei t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "bifurcates"  v ( b ai1 f @ k ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "bifurcating"  v ( b ai1 f @ k ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bifurcation"  n ( b ai2 f @ k ei1 sh @ n ) ((pos "K6%" ) ))
( "bifurcations"  n ( b ai2 f @ k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "big"  j ( b i1 g ) ((pos "OE%" ) ))
( "big-boned"  j ( b i1 g - b ou1 n d ) ((pos "OA%" ) ))
( "bigamist"  n ( b i1 g @ m i s t ) ((pos "K6%" ) ))
( "bigamists"  n ( b i1 g @ m i s t s ) ((pos "Kj%" ) ))
( "bigamous"  j ( b i1 g @ m @ s ) ((pos "OA%" ) ))
( "bigamy"  n ( b i1 g @ m ii ) ((pos "L@%" ) ))
( "bigger"  j ( b i1 g @ r ) ((pos "Or%" ) ))
( "biggest"  j ( b i1 g i s t ) ((pos "Os%" ) ))
( "bight"  n ( b ai1 t ) ((pos "K6$" ) ))
( "bights"  n ( b ai1 t s ) ((pos "Kj$" ) ))
( "bigot"  n ( b i1 g @ t ) ((pos "K6%" ) ))
( "bigoted"  j ( b i1 g @ t i d ) ((pos "OA%" ) ))
( "bigotries"  n ( b i1 g @ t r i z ) ((pos "Mj$" ) ))
( "bigotry"  n ( b i1 g @ t r ii ) ((pos "M8%" ) ))
( "bigots"  n ( b i1 g @ t s ) ((pos "Kj%" ) ))
( "bigwig"  n ( b i1 g w i g ) ((pos "K6%" ) ))
( "bigwigs"  n ( b i1 g w i g z ) ((pos "Kj%" ) ))
( "bijou"  n ( b ii1 zh uu ) ((pos "Ki$" "OA%" ) ))
( "bike"  v ( b ai1 k ) ((pos "I2%" "K6%" ) ))
( "biked"  v ( b ai1 k t ) ((pos "Ic%" "Id%" ) ))
( "bikes"  v ( b ai1 k s ) ((pos "Ia%" "Kj%" ) ))
( "biking"  v ( b ai1 k i ng ) ((pos "Ib%" ) ))
( "bikini"  n ( b i1 k ii1 n ii ) ((pos "K6%" ) ))
( "bikinis"  n ( b i1 k ii1 n i z ) ((pos "Kj%" ) ))
( "bilabial"  n ( b ai2 l ei1 b i@ l ) ((pos "K6$" "OA$" ) ))
( "bilabials"  n ( b ai2 l ei1 b i@ l z ) ((pos "Kj$" ) ))
( "bilateral"  j ( b ai2 l a1 t @ r @ l ) ((pos "OA%" ) ))
( "bilateralism"  n ( b ai2 l a1 t r @ l i z @ m ) ((pos "M6%" ) ))
( "bilateralisms"  n ( b ai2 l a1 t r @ l i z @ m z ) ((pos "Mj$" ) ))
( "bilaterally"  a ( b ai2 l a1 t @ r @ l ii ) ((pos "Pu%" ) ))
( "bilberries"  n ( b i1 l b @ r i z ) ((pos "Kj%" ) ))
( "bilberry"  n ( b i1 l b @ r ii ) ((pos "K8%" ) ))
( "bile"  n ( b ai1 l ) ((pos "L@%" ) ))
( "bile-duct"  n ( b ai1 l - d uh k t ) ((pos "K6%" ) ))
( "bile-ducts"  n ( b ai1 l - d uh k t s ) ((pos "Kj%" ) ))
( "bilge"  n ( b i1 l jh ) ((pos "L@%" ) ))
( "bilge-water"  n ( b i1 l jh - w oo t @ r ) ((pos "L@%" ) ))
( "bilharzia"  n ( b i2 l h aa1 z i@ ) ((pos "L@%" ) ))
( "bilingual"  n ( b ai1 l i1 ng g w @ l ) ((pos "K6%" "OA%" ) ))
( "bilinguals"  n ( b ai1 l i1 ng g w @ l z ) ((pos "Kj$" ) ))
( "bilious"  j ( b i1 l i@ s ) ((pos "OA%" ) ))
( "biliousness"  n ( b i1 l i@ s n @ s ) ((pos "L@%" ) ))
( "bilk"  v ( b i1 l k ) ((pos "H0%" ) (subcat "6A" )))
( "bilked"  v ( b i1 l k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bilking"  v ( b i1 l k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bilks"  v ( b i1 l k s ) ((pos "Ha%" ) (subcat "6A" )))
( "bill"  v ( b i1 l ) ((pos "J0%" "K6%" ) (subcat "6A" )))
( "bill-poster"  n ( b i1 l - p ou s t @ r ) ((pos "K6%" ) ))
( "bill-posters"  n ( b i1 l - p ou s t @ z ) ((pos "Kj%" ) ))
( "bill-sticker"  n ( b i1 l - s t i k @ r ) ((pos "K6%" ) ))
( "bill-stickers"  n ( b i1 l - s t i k @ z ) ((pos "Kj%" ) ))
( "billboard"  n ( b i1 l b oo d ) ((pos "K6%" ) ))
( "billboards"  n ( b i1 l b oo d z ) ((pos "Kj%" ) ))
( "billed"  v ( b i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "billet"  v ( b i1 l i t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "billet-doux"  n ( b i2 l ei - d uu1 ) ((pos "Ki%" ) ))
( "billeted"  v ( b i1 l i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "billeting"  v ( b i1 l i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "billets"  v ( b i1 l i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "billets-doux"  n ( b i2 l ei - d uu1 ) ((pos "Kj%" ) ))
( "billfold"  n ( b i1 l f ou l d ) ((pos "K6$" ) ))
( "billfolds"  n ( b i1 l f ou l d z ) ((pos "Kj$" ) ))
( "billhook"  n ( b i1 l h u k ) ((pos "K6%" ) ))
( "billhooks"  n ( b i1 l h u k s ) ((pos "Kj%" ) ))
( "billiard-marker"  n ( b i1 l i@ d - m aa k @ r ) ((pos "K6$" ) ))
( "billiard-markers"  n ( b i1 l i@ d - m aa k @ z ) ((pos "Kj$" ) ))
( "billiard-player"  n ( b i1 l i@ d - p l ei @ r ) ((pos "K6%" ) ))
( "billiard-players"  n ( b i1 l i@ d - p l ei @ z ) ((pos "Kj%" ) ))
( "billiard-room"  n ( b i1 l i@ d - r uu m ) ((pos "K6%" ) ))
( "billiard-rooms"  n ( b i1 l i@ d - r uu m z ) ((pos "Kj%" ) ))
( "billiard-table"  n ( b i1 l i@ d - t ei b l ) ((pos "K6%" ) ))
( "billiard-tables"  n ( b i1 l i@ d - t ei b l z ) ((pos "Kj%" ) ))
( "billiards"  n ( b i1 l i@ d z ) ((pos "Lk%" ) ))
( "billies"  n ( b i1 l i z ) ((pos "Kj$" ) ))
( "billing"  v ( b i1 l i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "billingsgate"  n ( b i1 l i ng z g ei t ) ((pos "L@$" ) ))
( "billion"  n ( b i1 l i@ ng ) ((pos "K6%" "OA%" ) ))
( "billions"  n ( b i1 l i@ ng z ) ((pos "Kj%" ) ))
( "billionth"  n ( b i1 l i@ n th ) ((pos "K6%" "OA%" ) ))
( "billionths"  n ( b i1 l i@ n th s ) ((pos "Kj%" ) ))
( "billow"  v ( b i1 l ou ) ((pos "I0%" "K6%" ) (subcat )))
( "billowed"  v ( b i1 l ou d ) ((pos "Ic%" "Id%" ) (subcat )))
( "billowing"  v ( b i1 l ou i ng ) ((pos "Ib%" ) (subcat )))
( "billows"  v ( b i1 l ou z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "billowy"  j ( b i1 l ou ii ) ((pos "OA%" ) ))
( "bills"  v ( b i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "billy"  n ( b i1 l ii ) ((pos "K8%" ) ))
( "billy-goat"  n ( b i1 l i - g ou t ) ((pos "K6%" ) ))
( "billy-goats"  n ( b i1 l i - g ou t s ) ((pos "Kj%" ) ))
( "billy-ho"  n ( b i1 l i - h ou ) ((pos "Ki$" ) ))
( "billy-o"  n ( b i1 l i - ou ) ((pos "L@$" ) ))
( "biltong"  n ( b i1 l t o ng ) ((pos "L@$" ) ))
( "bimetallic"  j ( b ai2 m @ t a1 l i k ) ((pos "OA$" ) ))
( "bimetallism"  n ( b ai2 m e1 t @ l i z @ m ) ((pos "L@$" ) ))
( "bin"  n ( b i1 n ) ((pos "K6%" ) ))
( "binary"  j ( b ai1 n @ r ii ) ((pos "OA%" ) ))
( "bind"  v ( b ai1 n d ) ((pos "J5%" "M6%" ) (subcat "2A" "6A" "14" "15A" "15B" "16B" )))
( "binder"  n ( b ai1 n d @ r ) ((pos "K6%" ) ))
( "binderies"  n ( b ai1 n d @ r i z ) ((pos "Kj$" ) ))
( "binders"  n ( b ai1 n d @ z ) ((pos "Kj%" ) ))
( "bindery"  n ( b ai1 n d @ r ii ) ((pos "K8$" ) ))
( "binding"  v ( b ai1 n d i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "14" "15A" "15B" "16B" )))
( "binds"  v ( b ai1 n d z ) ((pos "Ja%" "Mj$" ) (subcat "2A" "6A" "14" "15A" "15B" "16B" )))
( "bindweed"  n ( b ai1 n d w ii d ) ((pos "L@%" ) ))
( "bine"  n ( b ai1 n ) ((pos "K6$" ) ))
( "bines"  n ( b ai1 n z ) ((pos "Kj$" ) ))
( "binge"  n ( b i1 n jh ) ((pos "K6%" ) ))
( "binges"  n ( b i1 n jh i z ) ((pos "Kj%" ) ))
( "bingo"  n ( b i1 ng g ou ) ((pos "L@%" ) ))
( "binnacle"  n ( b i1 n @ k l ) ((pos "K6$" ) ))
( "binnacles"  n ( b i1 n @ k l z ) ((pos "Kj$" ) ))
( "binoculars"  n ( b i1 n o1 k y u l @ z ) ((pos "Kj%" ) ))
( "binomial"  j ( b ai1 n ou1 m i@ l ) ((pos "OA%" ) ))
( "bins"  n ( b i1 n z ) ((pos "Kj%" ) ))
( "biochemistry"  n ( b ai2 ou k e1 m i s t r ii ) ((pos "L@%" ) ))
( "biodegradable"  j ( b ai2 ou d i g r ei1 d @ b l ) ((pos "OA%" ) ))
( "biograph"  n ( b ai1 @ g r aa f ) ((pos "K6$" ) ))
( "biographer"  n ( b ai1 o1 g r @ f @ r ) ((pos "K6%" ) ))
( "biographers"  n ( b ai1 o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "biographic"  j ( b ai2 @ g r a1 f i k ) ((pos "OA$" ) ))
( "biographical"  j ( b ai2 @ g r a1 f i k l ) ((pos "OA%" ) ))
( "biographies"  n ( b ai1 o1 g r @ f i z ) ((pos "Mj%" ) ))
( "biographs"  n ( b ai1 @ g r aa f s ) ((pos "Kj$" ) ))
( "biography"  n ( b ai1 o1 g r @ f ii ) ((pos "M8%" ) ))
( "biological"  j ( b ai2 @ l o1 jh i k l ) ((pos "OA%" ) ))
( "biologically"  a ( b ai2 @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "biologist"  n ( b ai1 o1 l @ jh i s t ) ((pos "K6%" ) ))
( "biologists"  n ( b ai1 o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "biology"  n ( b ai1 o1 l @ jh ii ) ((pos "L@%" ) ))
( "bioscope"  n ( b ai1 @ s k ou p ) ((pos "K6%" ) ))
( "bioscopes"  n ( b ai1 @ s k ou p s ) ((pos "Kj%" ) ))
( "bipartisan"  j ( b ai2 p aa t i z a1 n ) ((pos "OA%" ) ))
( "biped"  n ( b ai1 p e d ) ((pos "K6%" ) ))
( "bipeds"  n ( b ai1 p e d z ) ((pos "Kj%" ) ))
( "biplane"  n ( b ai1 p l ei n ) ((pos "K6%" ) ))
( "biplanes"  n ( b ai1 p l ei n z ) ((pos "Kj%" ) ))
( "birch"  v ( b @@1 ch ) ((pos "H1%" "M7%" ) (subcat )))
( "birch-rod"  n ( b @@1 ch - r o d ) ((pos "K6%" ) ))
( "birch-rods"  n ( b @@1 ch - r o d z ) ((pos "Kj%" ) ))
( "birched"  v ( b @@1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "birches"  v ( b @@1 ch i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "birching"  v ( b @@1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "bird"  n ( b @@1 d ) ((pos "K6%" ) ))
( "bird's-eye"  j ( b @@1 d z - ei ) ((pos "Oq%" ) ))
( "bird-fancier"  n ( b @@1 d - f a n s i@ r ) ((pos "K6%" ) ))
( "bird-fanciers"  n ( b @@1 d - f a n s i@ z ) ((pos "Kj%" ) ))
( "birdcage"  n ( b @@1 d k ei jh ) ((pos "K6%" ) ))
( "birdcages"  n ( b @@1 d k ei jh i z ) ((pos "Kj%" ) ))
( "birdlime"  n ( b @@1 d l ai m ) ((pos "K6%" ) ))
( "birdlimes"  n ( b @@1 d l ai m z ) ((pos "Kj$" ) ))
( "birdnesting"  n ( b @@1 d n e s t i ng ) ((pos "L@%" ) ))
( "birds"  n ( b @@1 d z ) ((pos "Kj%" ) ))
( "birdwatcher"  n ( b @@1 d w o ch @ r ) ((pos "K6%" ) ))
( "birdwatchers"  n ( b @@1 d w o ch @ z ) ((pos "Kj%" ) ))
( "biretta"  n ( b i1 r e1 t @ ) ((pos "K6$" ) ))
( "birettas"  n ( b i1 r e1 t @ z ) ((pos "Kj$" ) ))
( "biro"  n ( b ai1 @ r ou ) ((pos "K6%" ) ))
( "biros"  n ( b ai1 @ r ou z ) ((pos "Kj%" ) ))
( "birth"  n ( b @@1 th ) ((pos "M6%" ) ))
( "birth-control"  n ( b @@1 th - k @ n t r ou l ) ((pos "L@%" ) ))
( "birthday"  n ( b @@1 th d ei ) ((pos "K6%" ) ))
( "birthdays"  n ( b @@1 th d ei z ) ((pos "Kj%" ) ))
( "birthmark"  n ( b @@1 th m aa k ) ((pos "K6%" ) ))
( "birthmarks"  n ( b @@1 th m aa k s ) ((pos "Kj%" ) ))
( "birthplace"  n ( b @@1 th p l ei s ) ((pos "K6%" ) ))
( "birthplaces"  n ( b @@1 th p l ei s i z ) ((pos "Kj%" ) ))
( "birthrate"  n ( b @@1 th r ei t ) ((pos "K6%" ) ))
( "birthrates"  n ( b @@1 th r ei t s ) ((pos "Kj%" ) ))
( "birthright"  n ( b @@1 th r ai t ) ((pos "K6%" ) ))
( "birthrights"  n ( b @@1 th r ai t s ) ((pos "Kj$" ) ))
( "births"  n ( b @@1 th s ) ((pos "Mj%" ) ))
( "biscuit"  n ( b i1 s k i t ) ((pos "K6%" ) ))
( "biscuits"  n ( b i1 s k i t s ) ((pos "Kj%" ) ))
( "bisect"  v ( b ai1 s e1 k t ) ((pos "H0%" ) (subcat )))
( "bisected"  v ( b ai1 s e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bisecting"  v ( b ai1 s e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "bisection"  n ( b ai1 s e1 k sh @ n ) ((pos "L@$" ) ))
( "bisects"  v ( b ai1 s e1 k t s ) ((pos "Ha%" ) (subcat )))
( "bisexual"  n ( b ai2 s e1 k sh u@ l ) ((pos "K6%" "OA%" ) ))
( "bisexuality"  n ( b ai2 s e k sh u a1 l i t ii ) ((pos "L@%" ) ))
( "bisexuals"  n ( b ai2 s e1 k sh u@ l z ) ((pos "Kj%" ) ))
( "bishop"  n ( b i1 sh @ p ) ((pos "K6%" ) ))
( "bishopric"  n ( b i1 sh @ p r i k ) ((pos "K6%" ) ))
( "bishoprics"  n ( b i1 sh @ p r i k s ) ((pos "Kj%" ) ))
( "bishops"  n ( b i1 sh @ p s ) ((pos "Kj%" ) ))
( "bismuth"  n ( b i1 z m @ th ) ((pos "L@$" ) ))
( "bison"  n ( b ai1 s n ) ((pos "K9%" ) ))
( "bistro"  n ( b ii1 s t r ou ) ((pos "K6%" ) ))
( "bistros"  n ( b ii1 s t r ou z ) ((pos "Kj%" ) ))
( "bit"  v ( b i1 t ) ((pos "Jc%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "bitch"  v ( b i1 ch ) ((pos "I1%" "K7%" ) (subcat )))
( "bitched"  v ( b i1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "bitches"  v ( b i1 ch i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "bitchier"  j ( b i1 ch i@ r ) ((pos "Or%" ) ))
( "bitchiest"  j ( b i1 ch i i s t ) ((pos "Os%" ) ))
( "bitching"  v ( b i1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "bitchy"  j ( b i1 ch ii ) ((pos "OD%" ) ))
( "bite"  v ( b ai1 t ) ((pos "J5%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "bites"  v ( b ai1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "biting"  v ( b ai1 t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" )))
( "bitingly"  a ( b ai1 t i ng l ii ) ((pos "Pu%" ) ))
( "bits"  n ( b i1 t s ) ((pos "Kj%" ) ))
( "bitten"  v ( b i1 t n ) ((pos "Jd%" ) (subcat "2A" "3A" "6A" )))
( "bitter"  n ( b i1 t @ r ) ((pos "M6%" "OA%" ) ))
( "bitter-sweet"  j ( b i2 t @ - s w ii1 t ) ((pos "OA%" ) ))
( "bitterly"  a ( b i1 t @ l ii ) ((pos "Pu%" ) ))
( "bittern"  n ( b i1 t @ n ) ((pos "K6%" ) ))
( "bitterness"  n ( b i1 t @ n @ s ) ((pos "L@%" ) ))
( "bitterns"  n ( b i1 t @ n z ) ((pos "Kj%" ) ))
( "bitters"  n ( b i1 t @ z ) ((pos "Mj%" ) ))
( "bitumen"  n ( b i1 t y u m @ n ) ((pos "L@$" ) ))
( "bituminous"  j ( b i1 t y uu1 m i n @ s ) ((pos "OA$" ) ))
( "bivalve"  n ( b ai1 v a l v ) ((pos "K6%" ) ))
( "bivalves"  n ( b ai1 v a l v z ) ((pos "Kj%" ) ))
( "bivouac"  v ( b i1 v u a k ) ((pos "I5%" "K6%" ) (subcat )))
( "bivouacked"  v ( b i1 v u a k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "bivouacking"  v ( b i1 v u a k i ng ) ((pos "Ib%" ) (subcat )))
( "bivouacs"  v ( b i1 v u a k s ) ((pos "Ia$" "Kj%" ) (subcat )))
( "biz"  n ( b i1 z ) ((pos "L@%" ) ))
( "bizarre"  j ( b i1 z aa1 r ) ((pos "OA%" ) ))
( "bizonal"  j ( b ai2 z ou1 n l ) ((pos "OA$" ) ))
( "blab"  v ( b l a1 b ) ((pos "J4%" ) (subcat "2A" "6A" )))
( "blabbed"  v ( b l a1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "blabber"  v ( b l a1 b @ r ) ((pos "J0%" ) ))
( "blabbered"  v ( b l a1 b @ d ) ((pos "Jc%" "Jd%" ) ))
( "blabbering"  v ( b l a1 b @ r i ng ) ((pos "Jb%" ) ))
( "blabbermouth"  n ( b l a1 b @ m au th ) ((pos "K6%" ) ))
( "blabbermouths"  n ( b l a1 b @ m au dh z ) ((pos "Kj%" ) ))
( "blabbers"  v ( b l a1 b @ z ) ((pos "Ja%" ) ))
( "blabbing"  v ( b l a1 b i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "blabs"  v ( b l a1 b z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "black"  v ( b l a1 k ) ((pos "H0%" "M6%" "OC%" ) (subcat )))
( "black-beetle"  n ( b l a1 k - b ii t l ) ((pos "K6%" ) ))
( "black-beetles"  n ( b l a1 k - b ii t l z ) ((pos "Kj%" ) ))
( "black-lead"  v ( b l a1 k - l e d ) ((pos "H0%" "L@%" ) ))
( "black-leaded"  v ( b l a1 k - l e d i d ) ((pos "Hc%" "Hd%" ) ))
( "black-leading"  v ( b l a1 k - l e d i ng ) ((pos "Hb%" ) ))
( "black-leads"  v ( b l a1 k - l e d z ) ((pos "Ha%" ) ))
( "blackamoor"  n ( b l a1 k @ m oo r ) ((pos "K6$" ) ))
( "blackamoors"  n ( b l a1 k @ m oo z ) ((pos "Kj$" ) ))
( "blackball"  v ( b l a1 k b oo l ) ((pos "H0%" ) ))
( "blackballed"  v ( b l a1 k b oo l d ) ((pos "Hc%" "Hd%" ) ))
( "blackballing"  v ( b l a1 k b oo l i ng ) ((pos "Hb%" ) ))
( "blackballs"  v ( b l a1 k b oo l z ) ((pos "Ha%" ) ))
( "blackberries"  n ( b l a1 k b @ r i z ) ((pos "Kj%" ) ))
( "blackberry"  n ( b l a1 k b @ r ii ) ((pos "K8%" ) ))
( "blackbird"  n ( b l a1 k b @@ d ) ((pos "K6%" ) ))
( "blackbirds"  n ( b l a1 k b @@ d z ) ((pos "Kj%" ) ))
( "blackboard"  n ( b l a1 k b oo d ) ((pos "K6%" ) ))
( "blackboards"  n ( b l a1 k b oo d z ) ((pos "Kj%" ) ))
( "blackcurrant"  n ( b l a1 k k uh1 r @ n t ) ((pos "K6%" ) ))
( "blackcurrants"  n ( b l a1 k k uh1 r @ n t s ) ((pos "Kj%" ) ))
( "blacked"  v ( b l a1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blacken"  v ( b l a1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "blackened"  v ( b l a1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blackening"  v ( b l a1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blackens"  v ( b l a1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "blacker"  j ( b l a1 k @ r ) ((pos "Or%" ) ))
( "blackest"  j ( b l a1 k i s t ) ((pos "Os%" ) ))
( "blackguard"  v ( b l a1 g aa d ) ((pos "H0$" "K6%" ) ))
( "blackguarded"  v ( b l a1 g aa d i d ) ((pos "Hc$" "Hd$" ) ))
( "blackguarding"  v ( b l a1 g aa d i ng ) ((pos "Hb$" ) ))
( "blackguardly"  j ( b l a1 g @ d l ii ) ((pos "OA%" ) ))
( "blackguards"  v ( b l a1 g aa d z ) ((pos "Ha$" "Kj%" ) ))
( "blackhead"  n ( b l a1 k h e d ) ((pos "K6%" ) ))
( "blackheads"  n ( b l a1 k h e d z ) ((pos "Kj%" ) ))
( "blacking"  v ( b l a1 k i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "blackish"  j ( b l a1 k i sh ) ((pos "OA%" ) ))
( "blackleg"  v ( b l a1 k l e g ) ((pos "J4$" "K6%" ) ))
( "blacklegged"  v ( b l a1 k l e g d ) ((pos "Jc$" "Jd$" ) ))
( "blacklegging"  v ( b l a1 k l e g i ng ) ((pos "Jb$" ) ))
( "blacklegs"  v ( b l a1 k l e g z ) ((pos "Ja$" "Kj%" ) ))
( "blacklist"  v ( b l a1 k l i s t ) ((pos "H0%" "K6%" ) ))
( "blacklisted"  v ( b l a1 k l i s t i d ) ((pos "Hc%" "Hd%" ) ))
( "blacklisting"  v ( b l a1 k l i s t i ng ) ((pos "Hb%" ) ))
( "blacklists"  v ( b l a1 k l i s t s ) ((pos "Ha%" "Kj%" ) ))
( "blackmail"  v ( b l a1 k m ei l ) ((pos "H0%" "L@%" ) ))
( "blackmailed"  v ( b l a1 k m ei l d ) ((pos "Hc%" "Hd%" ) ))
( "blackmailer"  n ( b l a1 k m ei l @ r ) ((pos "K6%" ) ))
( "blackmailers"  n ( b l a1 k m ei l @ z ) ((pos "Kj%" ) ))
( "blackmailing"  v ( b l a1 k m ei l i ng ) ((pos "Hb%" ) ))
( "blackmails"  v ( b l a1 k m ei l z ) ((pos "Ha%" ) ))
( "blackness"  n ( b l a1 k n i s ) ((pos "L@%" ) ))
( "blackout"  n ( b l a1 k au t ) ((pos "K6%" ) ))
( "blackouts"  n ( b l a1 k au t s ) ((pos "Kj$" ) ))
( "blacks"  v ( b l a1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "blacksmith"  n ( b l a1 k s m i th ) ((pos "K6%" ) ))
( "blacksmiths"  n ( b l a1 k s m i th s ) ((pos "Kj%" ) ))
( "blackthorn"  n ( b l a1 k th oo n ) ((pos "K6%" ) ))
( "blackthorns"  n ( b l a1 k th oo n z ) ((pos "Kj$" ) ))
( "blackwater"  j ( b l a1 k w oo t @ r ) ((pos "Oq$" ) ))
( "bladder"  n ( b l a1 d @ r ) ((pos "K6%" ) ))
( "bladders"  n ( b l a1 d @ z ) ((pos "Kj%" ) ))
( "blade"  n ( b l ei1 d ) ((pos "K6%" ) ))
( "blades"  n ( b l ei1 d z ) ((pos "Kj%" ) ))
( "blaeberries"  n ( b l ei1 b @ r i z ) ((pos "Kj$" ) ))
( "blaeberry"  n ( b l ei1 b @ r ii ) ((pos "K8$" ) ))
( "blah"  n ( b l aa1 ) ((pos "L@%" ) ))
( "blame"  v ( b l ei1 m ) ((pos "H2%" "L@%" ) (subcat "6A" )))
( "blamed"  v ( b l ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "blameless"  j ( b l ei1 m l @ s ) ((pos "OA%" ) ))
( "blamelessly"  a ( b l ei1 m l @ s l ii ) ((pos "Pu%" ) ))
( "blames"  v ( b l ei1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "blameworthy"  j ( b l ei1 m w @@ dh ii ) ((pos "OA%" ) ))
( "blaming"  v ( b l ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "blanch"  v ( b l aa1 n ch ) ((pos "J1%" ) (subcat "2A" )))
( "blanched"  v ( b l aa1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blanches"  v ( b l aa1 n ch i z ) ((pos "Ja%" ) (subcat "2A" )))
( "blanching"  v ( b l aa1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blancmange"  n ( b l @1 m o1 n zh ) ((pos "M6%" ) ))
( "blancmanges"  n ( b l @1 m o1 n zh i z ) ((pos "Mj$" ) ))
( "bland"  j ( b l a1 n d ) ((pos "OC%" ) ))
( "blander"  j ( b l a1 n d @ r ) ((pos "Or%" ) ))
( "blandest"  j ( b l a1 n d i s t ) ((pos "Os%" ) ))
( "blandishment"  n ( b l a1 n d i sh m @ n t ) ((pos "K6%" ) ))
( "blandishments"  n ( b l a1 n d i sh m @ n t s ) ((pos "Kj%" ) ))
( "blandly"  a ( b l a1 n d l ii ) ((pos "Pu%" ) ))
( "blandness"  n ( b l a1 n d n @ s ) ((pos "L@%" ) ))
( "blank"  n ( b l a1 ng k ) ((pos "M6%" "OA%" ) ))
( "blanket"  v ( b l a1 ng k i t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "blanketed"  v ( b l a1 ng k i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "blanketing"  v ( b l a1 ng k i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "blankets"  v ( b l a1 ng k i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "blankly"  a ( b l a1 ng k l ii ) ((pos "Pu%" ) ))
( "blanks"  n ( b l a1 ng k s ) ((pos "Mj%" ) ))
( "blare"  v ( b l e@1 r ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" )))
( "blared"  v ( b l e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "blares"  v ( b l e@1 z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "blaring"  v ( b l e@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "blarney"  n ( b l aa1 n ii ) ((pos "L@%" ) ))
( "blas_e"  j ( b l aa1 z ei ) ((pos "OA%" ) ))
( "blaspheme"  v ( b l a1 s f ii1 m ) ((pos "J2%" ) (subcat "2A" )))
( "blasphemed"  v ( b l a1 s f ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blasphemer"  n ( b l a1 s f ii1 m @ r ) ((pos "K6%" ) ))
( "blasphemers"  n ( b l a1 s f ii1 m @ z ) ((pos "Kj%" ) ))
( "blasphemes"  v ( b l a1 s f ii1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "blasphemies"  n ( b l a1 s f @ m i z ) ((pos "Mj%" ) ))
( "blaspheming"  v ( b l a1 s f ii1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blasphemous"  j ( b l a1 s f @ m @ s ) ((pos "OA%" ) ))
( "blasphemously"  a ( b l a1 s f @ m @ s l ii ) ((pos "Pu%" ) ))
( "blasphemy"  n ( b l a1 s f @ m ii ) ((pos "M8%" ) ))
( "blast"  v ( b l aa1 s t ) ((pos "H0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "blast-furnace"  n ( b l aa1 s t - f @@ n i s ) ((pos "K6%" ) ))
( "blast-furnaces"  n ( b l aa1 s t - f @@ n i s i z ) ((pos "Kj%" ) ))
( "blast-off"  n ( b l aa1 s t - o f ) ((pos "K6%" ) ))
( "blast-offs"  n ( b l aa1 s t - o f s ) ((pos "Kj%" ) ))
( "blasted"  v ( b l aa1 s t i d ) ((pos "Hc%" "Hd%" "Oq%" ) (subcat "2A" "2C" "6A" )))
( "blasting"  v ( b l aa1 s t i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" )))
( "blasts"  v ( b l aa1 s t s ) ((pos "Ha%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "blatant"  j ( b l ei1 t n t ) ((pos "OA%" ) ))
( "blatantly"  a ( b l ei1 t n t l ii ) ((pos "Pu%" ) ))
( "blather"  v ( b l a1 dh @ r ) ((pos "I0%" "M6$" ) ))
( "blathered"  v ( b l a1 dh @ d ) ((pos "Ic%" "Id%" ) ))
( "blathering"  v ( b l a1 dh @ r i ng ) ((pos "Ib%" ) ))
( "blathers"  v ( b l a1 dh @ z ) ((pos "Ia%" "Mj$" ) ))
( "blaze"  v ( b l ei1 z ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "blazed"  v ( b l ei1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "blazer"  n ( b l ei1 z @ r ) ((pos "K6%" ) ))
( "blazers"  n ( b l ei1 z @ z ) ((pos "Kj%" ) ))
( "blazes"  v ( b l ei1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "blazing"  v ( b l ei1 z i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "blazon"  v ( b l ei1 z n ) ((pos "H0%" "K6%" ) ))
( "blazoned"  v ( b l ei1 z n d ) ((pos "Hc%" "Hd%" ) ))
( "blazoning"  v ( b l ei1 z n i ng ) ((pos "Hb%" ) ))
( "blazonries"  n ( b l ei1 z @ n r i z ) ((pos "Kj$" ) ))
( "blazonry"  n ( b l ei1 z @ n r ii ) ((pos "K8$" ) ))
( "blazons"  v ( b l ei1 z n z ) ((pos "Ha%" "Kj%" ) ))
( "bleach"  v ( b l ii1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "bleached"  v ( b l ii1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bleachers"  n ( b l ii1 ch @ z ) ((pos "Kj$" ) ))
( "bleaches"  v ( b l ii1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "bleaching"  v ( b l ii1 ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bleaching-powder"  n ( b l ii1 ch i ng - p au d @ r ) ((pos "K6%" ) ))
( "bleaching-powders"  n ( b l ii1 ch i ng - p au d @ z ) ((pos "Kj%" ) ))
( "bleak"  j ( b l ii1 k ) ((pos "OC%" ) ))
( "bleaker"  j ( b l ii1 k @ r ) ((pos "Or%" ) ))
( "bleakest"  j ( b l ii1 k i s t ) ((pos "Os%" ) ))
( "bleakly"  a ( b l ii1 k l ii ) ((pos "Pu%" ) ))
( "bleary"  j ( b l i@1 r ii ) ((pos "OA%" ) ))
( "bleary-eyed"  j ( b l i@2 r i - ai1 d ) ((pos "OA%" ) ))
( "bleat"  v ( b l ii1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "bleated"  v ( b l ii1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "bleating"  v ( b l ii1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "bleats"  v ( b l ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "bled"  v ( b l e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bleed"  v ( b l ii1 d ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bleeding"  v ( b l ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bleeds"  v ( b l ii1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bleep"  v ( b l ii1 p ) ((pos "I0%" "K6%" ) ))
( "bleeped"  v ( b l ii1 p t ) ((pos "Ic%" "Id%" ) ))
( "bleeping"  v ( b l ii1 p i ng ) ((pos "Ib%" ) ))
( "bleeps"  v ( b l ii1 p s ) ((pos "Ia%" "Kj%" ) ))
( "blemish"  v ( b l e1 m i sh ) ((pos "H1%" "M7%" ) (subcat )))
( "blemished"  v ( b l e1 m i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blemishes"  v ( b l e1 m i sh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "blemishing"  v ( b l e1 m i sh i ng ) ((pos "Hb%" ) (subcat )))
( "blench"  v ( b l e1 n ch ) ((pos "I1%" ) (subcat )))
( "blenched"  v ( b l e1 n ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "blenches"  v ( b l e1 n ch i z ) ((pos "Ia%" ) (subcat )))
( "blenching"  v ( b l e1 n ch i ng ) ((pos "Ib%" ) (subcat )))
( "blend"  v ( b l e1 n d ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" )))
( "blended"  v ( b l e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "blending"  v ( b l e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "blends"  v ( b l e1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "blent"  v ( b l e1 n t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "3A" )))
( "bless"  v ( b l e1 s ) ((pos "H1%" ) (subcat )))
( "blessed"  j ( b l e1 s i d ) ((pos "OA%" ) ))
( "blessed"  v ( b l e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blessedness"  n ( b l e1 s i d n @ s ) ((pos "L@%" ) ))
( "blesses"  v ( b l e1 s i z ) ((pos "Ha%" ) (subcat )))
( "blessing"  v ( b l e1 s i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "blessings"  n ( b l e1 s i ng z ) ((pos "Kj%" ) ))
( "blest"  v ( b l e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blether"  v ( b l e1 dh @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "blethered"  v ( b l e1 dh @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "blethering"  v ( b l e1 dh @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "blethers"  v ( b l e1 dh @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "blew"  v ( b l uu1 ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2E" "3A" "6A" "12A" "15A" )))
( "blight"  v ( b l ai1 t ) ((pos "H0%" "M6%" ) (subcat )))
( "blighted"  v ( b l ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blighter"  n ( b l ai1 t @ r ) ((pos "K6%" ) ))
( "blighters"  n ( b l ai1 t @ z ) ((pos "Kj%" ) ))
( "blighting"  v ( b l ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "blights"  v ( b l ai1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "blimey"  uh ( b l ai1 m ii ) ((pos "W-%" ) ))
( "blimp"  n ( b l i1 m p ) ((pos "K6$" ) ))
( "blimps"  n ( b l i1 m p s ) ((pos "Kj$" ) ))
( "blind"  v ( b l ai1 n d ) ((pos "H0%" "K6%" "OA%" ) (subcat "6A" )))
( "blinded"  v ( b l ai1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "blinders"  n ( b l ai1 n d @ z ) ((pos "Kj%" ) ))
( "blindfold"  v ( b l ai1 n d f ou l d ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "blindfolded"  v ( b l ai1 n d f ou l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blindfolding"  v ( b l ai1 n d f ou l d i ng ) ((pos "Hb$" ) (subcat )))
( "blindfolds"  v ( b l ai1 n d f ou l d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "blinding"  v ( b l ai1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "blindly"  a ( b l ai1 n d l ii ) ((pos "Pu%" ) ))
( "blindman's buff"  n ( b l ai2 n d m a n z - b uh1 f ) ((pos "L@%" ) ))
( "blindness"  n ( b l ai1 n d n @ s ) ((pos "L@%" ) ))
( "blinds"  v ( b l ai1 n d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "blink"  v ( b l i1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "blinked"  v ( b l i1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "blinkers"  n ( b l i1 ng k @ z ) ((pos "Kj%" ) ))
( "blinking"  v ( b l i1 ng k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "blinks"  v ( b l i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "blip"  n ( b l i1 p ) ((pos "K6%" ) ))
( "blips"  n ( b l i1 p s ) ((pos "Kj%" ) ))
( "bliss"  n ( b l i1 s ) ((pos "L@%" ) ))
( "blissful"  j ( b l i1 s f @ l ) ((pos "OA%" ) ))
( "blissfully"  a ( b l i1 s f @ l ii ) ((pos "Pu%" ) ))
( "blister"  v ( b l i1 s t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "blistered"  v ( b l i1 s t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blistering"  v ( b l i1 s t @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blisters"  v ( b l i1 s t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "blithe"  j ( b l ai1 dh ) ((pos "OA%" ) ))
( "blithely"  a ( b l ai1 dh l ii ) ((pos "Pu%" ) ))
( "blithering"  j ( b l i1 dh @ r i ng ) ((pos "OA%" ) ))
( "blithesome"  j ( b l ai1 dh s @ m ) ((pos "OA$" ) ))
( "blitz"  v ( b l i1 t s ) ((pos "H1%" "K7%" ) (subcat )))
( "blitzed"  v ( b l i1 t s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blitzes"  v ( b l i1 t s i z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "blitzing"  v ( b l i1 t s i ng ) ((pos "Hb$" ) (subcat )))
( "blizzard"  n ( b l i1 z @ d ) ((pos "K6%" ) ))
( "blizzards"  n ( b l i1 z @ d z ) ((pos "Kj%" ) ))
( "bloated"  j ( b l ou1 t i d ) ((pos "OA%" ) ))
( "bloater"  n ( b l ou1 t @ r ) ((pos "K6%" ) ))
( "bloaters"  n ( b l ou1 t @ z ) ((pos "Kj%" ) ))
( "blob"  n ( b l o1 b ) ((pos "K6%" ) ))
( "blobs"  n ( b l o1 b z ) ((pos "Kj%" ) ))
( "bloc"  n ( b l o1 k ) ((pos "K6%" ) ))
( "block"  v ( b l o1 k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "blockade"  v ( b l o1 k ei1 d ) ((pos "H2%" "K6%" ) (subcat )))
( "blockade-runner"  n ( b l o1 k ei1 d - r uh n @ r ) ((pos "K6%" ) ))
( "blockade-runners"  n ( b l o1 k ei1 d - r uh n @ z ) ((pos "Kj%" ) ))
( "blockaded"  v ( b l o1 k ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blockades"  v ( b l o1 k ei1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "blockading"  v ( b l o1 k ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "blockage"  n ( b l o1 k i jh ) ((pos "K6%" ) ))
( "blockages"  n ( b l o1 k i jh i z ) ((pos "Kj%" ) ))
( "blockbuster"  n ( b l o1 k b uh s t @ r ) ((pos "K6%" ) ))
( "blockbusters"  n ( b l o1 k b uh s t @ z ) ((pos "Kj%" ) ))
( "blocked"  v ( b l o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "blockhead"  n ( b l o1 k h e d ) ((pos "K6%" ) ))
( "blockheads"  n ( b l o1 k h e d z ) ((pos "Kj%" ) ))
( "blockhouse"  n ( b l o1 k h au s ) ((pos "K6$" ) ))
( "blockhouses"  n ( b l o1 k h au z i z ) ((pos "Kj$" ) ))
( "blocking"  v ( b l o1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "blocks"  v ( b l o1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "blocs"  n ( b l o1 k s ) ((pos "Kj%" ) ))
( "bloke"  n ( b l ou1 k ) ((pos "K6%" ) ))
( "blokes"  n ( b l ou1 k s ) ((pos "Kj%" ) ))
( "blond"  n ( b l o1 n d ) ((pos "K6$" "OC%" ) ))
( "blonde"  n ( b l o1 n d ) ((pos "K6%" "OA%" ) ))
( "blonder"  j ( b l o1 n d @ r ) ((pos "Or%" ) ))
( "blondes"  n ( b l o1 n d z ) ((pos "Kj%" ) ))
( "blondest"  j ( b l o1 n d i s t ) ((pos "Os%" ) ))
( "blonds"  n ( b l o1 n d z ) ((pos "Kj$" ) ))
( "blood"  v ( b l uh1 d ) ((pos "H0%" "M6%" ) (subcat )))
( "blood-bath"  n ( b l uh1 d - b aa th ) ((pos "K6%" ) ))
( "blood-baths"  n ( b l uh1 d - b aa dh z ) ((pos "Kj%" ) ))
( "blood-donor"  n ( b l uh1 d - d ou n @ r ) ((pos "K6%" ) ))
( "blood-donors"  n ( b l uh1 d - d ou n @ z ) ((pos "Kj%" ) ))
( "blood-group"  n ( b l uh1 d - g r uu p ) ((pos "K6%" ) ))
( "blood-groups"  n ( b l uh1 d - g r uu p s ) ((pos "Kj%" ) ))
( "blood-heat"  n ( b l uh1 d - h ii t ) ((pos "L@%" ) ))
( "blood-letting"  n ( b l uh1 d - l e t i ng ) ((pos "M6%" ) ))
( "blood-lettings"  n ( b l uh1 d - l e t i ng z ) ((pos "Mj$" ) ))
( "blood-money"  n ( b l uh1 d - m uh n ii ) ((pos "L@%" ) ))
( "blood-poisoning"  n ( b l uh1 d - p oi z n i ng ) ((pos "L@%" ) ))
( "blood-pressure"  n ( b l uh1 d - p r e sh @ r ) ((pos "M6%" ) ))
( "blood-pressures"  n ( b l uh1 d - p r e sh @ z ) ((pos "Mj$" ) ))
( "blood-relation"  n ( b l uh2 d - r i l ei1 sh @ n ) ((pos "K6%" ) ))
( "blood-relations"  n ( b l uh2 d - r i l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "blood-sports"  n ( b l uh1 d - s p oo t s ) ((pos "Kj%" ) ))
( "blood-transfusion"  n ( b l uh2 d - t r a n s f y uu1 zh n ) ((pos "K6%" ) ))
( "blood-transfusions"  n ( b l uh2 d - t r a n s f y uu1 zh n z ) ((pos "Kj%" ) ))
( "blood-type"  n ( b l uh1 d - t ai p ) ((pos "K6%" ) ))
( "blood-types"  n ( b l uh1 d - t ai p s ) ((pos "Kj%" ) ))
( "blood-vessel"  n ( b l uh1 d - v e s l ) ((pos "K6%" ) ))
( "blood-vessels"  n ( b l uh1 d - v e s l z ) ((pos "Kj%" ) ))
( "bloodcurdling"  j ( b l uh1 d k @@ d l i ng ) ((pos "OA%" ) ))
( "blooded"  v ( b l uh1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bloodhound"  n ( b l uh1 d h au n d ) ((pos "K6%" ) ))
( "bloodhounds"  n ( b l uh1 d h au n d z ) ((pos "Kj%" ) ))
( "bloodier"  j ( b l uh1 d i@ r ) ((pos "Or%" ) ))
( "bloodiest"  j ( b l uh1 d i i s t ) ((pos "Os%" ) ))
( "blooding"  v ( b l uh1 d i ng ) ((pos "Hb$" ) (subcat )))
( "bloodless"  j ( b l uh1 d l @ s ) ((pos "OA%" ) ))
( "bloodlessly"  a ( b l uh1 d l @ s l ii ) ((pos "Pu%" ) ))
( "bloodlust"  n ( b l uh1 d l uh s t ) ((pos "L@%" ) ))
( "bloods"  v ( b l uh1 d z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "bloodshed"  n ( b l uh1 d sh e d ) ((pos "K6%" ) ))
( "bloodsheds"  n ( b l uh1 d sh e d z ) ((pos "Kj$" ) ))
( "bloodshot"  j ( b l uh1 d sh o t ) ((pos "OA%" ) ))
( "bloodstain"  n ( b l uh1 d s t ei n ) ((pos "K6%" ) ))
( "bloodstained"  j ( b l uh1 d s t ei n d ) ((pos "OA%" ) ))
( "bloodstains"  n ( b l uh1 d s t ei n z ) ((pos "Kj%" ) ))
( "bloodstock"  n ( b l uh1 d s t o k ) ((pos "K6%" ) ))
( "bloodstocks"  n ( b l uh1 d s t o k s ) ((pos "Kj$" ) ))
( "bloodsucker"  n ( b l uh1 d s uh k @ r ) ((pos "K6%" ) ))
( "bloodsuckers"  n ( b l uh1 d s uh k @ z ) ((pos "Kj%" ) ))
( "bloodthirstiness"  n ( b l uh1 d th @@ s t i n @ s ) ((pos "L@%" ) ))
( "bloodthirsty"  j ( b l u1 d th @@ s t ii ) ((pos "OA%" ) ))
( "bloody"  j ( b l uh1 d ii ) ((pos "OD%" "Pu%" ) ))
( "bloody-minded"  j ( b l uh2 d i - m ai1 n d i d ) ((pos "OA%" ) ))
( "bloom"  v ( b l uu1 m ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "bloomed"  v ( b l uu1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "bloomer"  n ( b l uu1 m @ r ) ((pos "K6%" ) ))
( "bloomers"  n ( b l uu1 m @ z ) ((pos "Kj%" ) ))
( "blooming"  v ( b l uu1 m i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "blooms"  v ( b l uu1 m z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "blossom"  v ( b l o1 s @ m ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "blossomed"  v ( b l o1 s @ m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "blossoming"  v ( b l o1 s @ m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "blossoms"  v ( b l o1 s @ m z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "blot"  v ( b l o1 t ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "blotch"  n ( b l o1 ch ) ((pos "K7%" ) ))
( "blotches"  n ( b l o1 ch i z ) ((pos "Kj%" ) ))
( "blots"  v ( b l o1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "blotted"  v ( b l o1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "blotter"  n ( b l o1 t @ r ) ((pos "K6%" ) ))
( "blotters"  n ( b l o1 t @ z ) ((pos "Kj%" ) ))
( "blotting"  v ( b l o1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "blotting-paper"  n ( b l o1 t i ng - p ei p @ r ) ((pos "K6%" ) ))
( "blotting-papers"  n ( b l o1 t i ng - p ei p @ z ) ((pos "Kj$" ) ))
( "blotto"  j ( b l o1 t ou ) ((pos "Op%" ) ))
( "blouse"  n ( b l au1 z ) ((pos "K6%" ) ))
( "blouses"  n ( b l au1 z i z ) ((pos "Kj%" ) ))
( "blow"  v ( b l ou1 ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "2E" "3A" "6A" "12A" "15A" )))
( "blow-dried"  v ( b l ou1 - d r ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blow-dries"  v ( b l ou1 - d r ai z ) ((pos "Ha%" ) (subcat )))
( "blow-dry"  v ( b l ou1 - d r ai ) ((pos "H3%" ) (subcat )))
( "blow-drying"  v ( b l ou1 - d r ai i ng ) ((pos "Hb%" ) (subcat )))
( "blow-up"  n ( b l ou1 - uh p ) ((pos "K6%" ) ))
( "blow-ups"  n ( b l ou1 - uh p s ) ((pos "Kj$" ) ))
( "blowback"  n ( b l ou1 b a k ) ((pos "K6%" ) ))
( "blowbacks"  n ( b l ou1 b a k s ) ((pos "Kj$" ) ))
( "blowed"  v ( b l ou1 d ) ((pos "Jd%" ) (subcat "6A" )))
( "blower"  n ( b l ou1 @ r ) ((pos "K6%" ) ))
( "blowers"  n ( b l ou1 @ z ) ((pos "Kj%" ) ))
( "blowflies"  n ( b l ou1 f l ai z ) ((pos "Kj%" ) ))
( "blowfly"  n ( b l ou1 f l ai ) ((pos "K8%" ) ))
( "blowhole"  n ( b l ou1 h ou l ) ((pos "K6%" ) ))
( "blowholes"  n ( b l ou1 h ou l z ) ((pos "Kj%" ) ))
( "blowing"  v ( b l ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "2E" "3A" "6A" "12A" "15A" )))
( "blowing-up"  n ( b l ou2 i ng - uh1 p ) ((pos "K6%" ) ))
( "blowing-ups"  n ( b l ou2 i ng - uh1 p s ) ((pos "Kj$" ) ))
( "blowlamp"  n ( b l ou1 l a m p ) ((pos "K6%" ) ))
( "blowlamps"  n ( b l ou1 l a m p s ) ((pos "Kj%" ) ))
( "blown"  v ( b l ou1 n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "2E" "3A" "6A" "12A" "15A" )))
( "blowout"  n ( b l ou1 au t ) ((pos "K6%" ) ))
( "blowouts"  n ( b l ou1 au t s ) ((pos "Kj%" ) ))
( "blowpipe"  n ( b l ou1 p ai p ) ((pos "K6%" ) ))
( "blowpipes"  n ( b l ou1 p ai p s ) ((pos "Kj%" ) ))
( "blows"  v ( b l ou1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2E" "3A" "6A" "12A" "15A" )))
( "blowtorch"  n ( b l ou1 t oo ch ) ((pos "K7%" ) ))
( "blowtorches"  n ( b l ou1 t oo ch i z ) ((pos "Kj%" ) ))
( "blowzy"  j ( b l au1 z ii ) ((pos "OA%" ) ))
( "blubber"  v ( b l uh1 b @ r ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "blubbered"  v ( b l uh1 b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blubbering"  v ( b l uh1 b @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blubbers"  v ( b l uh1 b @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "bludgeon"  v ( b l uh1 jh @ n ) ((pos "H0%" "K6$" ) (subcat "6A" )))
( "bludgeoned"  v ( b l uh1 jh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bludgeoning"  v ( b l uh1 jh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bludgeons"  v ( b l uh1 jh @ n z ) ((pos "Ha%" "Kj$" ) (subcat "6A" )))
( "blue"  v ( b l uu1 ) ((pos "H2%" "M6%" "OB%" ) ))
( "blue-collar"  j ( b l uu1 - k o l @ r ) ((pos "OA%" ) ))
( "blue-jacket"  n ( b l uu1 - jh a k i t ) ((pos "K6$" ) ))
( "blue-jackets"  n ( b l uu1 - jh a k i t s ) ((pos "Kj$" ) ))
( "blue-pencil"  v ( b l uu1 - p e1 n s i l ) ((pos "H4%" ) ))
( "blue-pencilled"  v ( b l uu1 - p e1 n s i l d ) ((pos "Hc%" "Hd%" ) ))
( "blue-pencilling"  v ( b l uu1 - p e1 n s i l i ng ) ((pos "Hb%" ) ))
( "blue-pencils"  v ( b l uu1 - p e1 n s i l z ) ((pos "Ha%" ) ))
( "bluebell"  n ( b l uu1 b e l ) ((pos "K6%" ) ))
( "bluebells"  n ( b l uu1 b e l z ) ((pos "Kj%" ) ))
( "bluebottle"  n ( b l uu1 b o t l ) ((pos "K6%" ) ))
( "bluebottles"  n ( b l uu1 b o t l z ) ((pos "Kj%" ) ))
( "blued"  v ( b l uu1 d ) ((pos "Hc%" "Hd%" ) ))
( "blueprint"  n ( b l uu1 p r i n t ) ((pos "K6%" ) ))
( "blueprints"  n ( b l uu1 p r i n t s ) ((pos "Kj%" ) ))
( "bluer"  j ( b l uu1 @ r ) ((pos "Or%" ) ))
( "blues"  v ( b l uu1 z ) ((pos "Ha%" "Mj%" ) ))
( "bluest"  j ( b l uu1 i s t ) ((pos "Os%" ) ))
( "bluestocking"  n ( b l uu1 s t o k i ng ) ((pos "K6%" ) ))
( "bluestockings"  n ( b l uu1 s t o k i ng z ) ((pos "Kj%" ) ))
( "bluff"  v ( b l uh1 f ) ((pos "J0%" "M6%" "OA%" ) (subcat "2A" "6A" "14" )))
( "bluffed"  v ( b l uh1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "bluffer"  n ( b l uh1 f @ r ) ((pos "K6%" ) ))
( "bluffers"  n ( b l uh1 f @ z ) ((pos "Kj%" ) ))
( "bluffing"  v ( b l uh1 f i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "bluffly"  a ( b l uh1 f l ii ) ((pos "Pu%" ) ))
( "bluffness"  n ( b l uh1 f n @ s ) ((pos "L@$" ) ))
( "bluffs"  v ( b l uh1 f s ) ((pos "Ja%" "Mj$" ) (subcat "2A" "6A" "14" )))
( "bluing"  v ( b l uu1 i ng ) ((pos "Hb%" ) ))
( "bluish"  j ( b l uu1 i sh ) ((pos "OA%" ) ))
( "blunder"  v ( b l uh1 n d @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "blunderbuss"  n ( b l uh1 n d @ b uh s ) ((pos "K7%" ) ))
( "blunderbusses"  n ( b l uh1 n d @ b uh s i z ) ((pos "Kj%" ) ))
( "blundered"  v ( b l uh1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "blunderer"  n ( b l uh1 n d @ r @ r ) ((pos "K6%" ) ))
( "blunderers"  n ( b l uh1 n d @ r @ z ) ((pos "Kj%" ) ))
( "blundering"  v ( b l uh1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "blunders"  v ( b l uh1 n d @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "blunt"  v ( b l uh1 n t ) ((pos "H0%" "OC%" ) (subcat )))
( "blunted"  v ( b l uh1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blunter"  j ( b l uh1 n t @ r ) ((pos "Or%" ) ))
( "bluntest"  j ( b l uh1 n t i s t ) ((pos "Os%" ) ))
( "blunting"  v ( b l uh1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "bluntly"  a ( b l uh1 n t l ii ) ((pos "Pu%" ) ))
( "bluntness"  n ( b l uh1 n t n @ s ) ((pos "L@%" ) ))
( "blunts"  v ( b l uh1 n t s ) ((pos "Ha%" ) (subcat )))
( "blur"  v ( b l @@1 r ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "blurb"  n ( b l @@1 b ) ((pos "M6%" ) ))
( "blurbs"  n ( b l @@1 b z ) ((pos "Mj$" ) ))
( "blurred"  v ( b l @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "blurring"  v ( b l @@1 r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "blurs"  v ( b l @@1 z ) ((pos "Ja%" "Kj$" ) (subcat "2A" )))
( "blurt"  v ( b l @@1 t ) ((pos "H0%" ) (subcat )))
( "blurted"  v ( b l @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "blurting"  v ( b l @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "blurts"  v ( b l @@1 t s ) ((pos "Ha%" ) (subcat )))
( "blush"  v ( b l uh1 sh ) ((pos "I1%" "K7%" ) (subcat "2A" "2C" "3A" )))
( "blushed"  v ( b l uh1 sh t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "3A" )))
( "blushes"  v ( b l uh1 sh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "blushing"  v ( b l uh1 sh i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" "2C" "3A" )))
( "blushingly"  a ( b l uh1 sh i ng l ii ) ((pos "Pu%" ) ))
( "bluster"  v ( b l uh1 s t @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "blustered"  v ( b l uh1 s t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "blustering"  v ( b l uh1 s t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "blusters"  v ( b l uh1 s t @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "blustery"  j ( b l uh1 s t @ r ii ) ((pos "OA%" ) ))
( "bo"  uh ( b ou1 ) ((pos "W-%" ) ))
( "bo'sn"  n ( b ou1 s n ) ((pos "K6$" ) ))
( "bo'sns"  n ( b ou1 s n z ) ((pos "Kj$" ) ))
( "bo'sun"  n ( b ou1 s n ) ((pos "K6%" ) ))
( "bo'suns"  n ( b ou1 s n z ) ((pos "Kj$" ) ))
( "boa"  n ( b ou1 @ ) ((pos "K6%" ) ))
( "boa-constrictor"  n ( b ou1 @ - k @ n s t r i2 k t @ r ) ((pos "K6%" ) ))
( "boa-constrictors"  n ( b ou1 @ - k @ n s t r i2 k t @ z ) ((pos "Kj%" ) ))
( "boar"  n ( b oo1 r ) ((pos "K6%" ) ))
( "board"  v ( b oo1 d ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" )))
( "boarded"  v ( b oo1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "boarder"  n ( b oo1 d @ r ) ((pos "K6%" ) ))
( "boarders"  n ( b oo1 d @ z ) ((pos "Kj%" ) ))
( "boarding"  v ( b oo1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "3A" "6A" )))
( "boarding-card"  n ( b oo1 d i ng - k aa d ) ((pos "K6%" ) ))
( "boarding-cards"  n ( b oo1 d i ng - k aa d z ) ((pos "Kj%" ) ))
( "boarding-house"  n ( b oo1 d i ng - h au s ) ((pos "K6%" ) ))
( "boarding-houses"  n ( b oo1 d i ng - h au z i z ) ((pos "Kj%" ) ))
( "boarding-school"  n ( b oo1 d i ng - s k uu l ) ((pos "K6%" ) ))
( "boarding-schools"  n ( b oo1 d i ng - s k uu l z ) ((pos "Kj%" ) ))
( "boardroom"  n ( b oo1 d r uu m ) ((pos "K6%" ) ))
( "boardrooms"  n ( b oo1 d r uu m z ) ((pos "Kj%" ) ))
( "boards"  v ( b oo1 d z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "boardwalk"  n ( b oo1 d w oo k ) ((pos "K6%" ) ))
( "boardwalks"  n ( b oo1 d w oo k s ) ((pos "Kj$" ) ))
( "boars"  n ( b oo1 z ) ((pos "Kj%" ) ))
( "boas"  n ( b ou1 @ z ) ((pos "Kj%" ) ))
( "boast"  v ( b ou1 s t ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "3B" )))
( "boasted"  v ( b ou1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "3B" )))
( "boaster"  n ( b ou1 s t @ r ) ((pos "K6%" ) ))
( "boasters"  n ( b ou1 s t @ z ) ((pos "Kj%" ) ))
( "boastful"  j ( b ou1 s t f @ l ) ((pos "OA%" ) ))
( "boastfully"  a ( b ou1 s t f @ l ii ) ((pos "Pu%" ) ))
( "boasting"  v ( b ou1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "3B" )))
( "boasts"  v ( b ou1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "3B" )))
( "boat"  v ( b ou1 t ) ((pos "I0$" "K6%" ) (subcat "2A" )))
( "boat-hook"  n ( b ou1 t - h u k ) ((pos "K6%" ) ))
( "boat-hooks"  n ( b ou1 t - h u k s ) ((pos "Kj%" ) ))
( "boat-house"  n ( b ou1 t - h au s ) ((pos "K6%" ) ))
( "boat-houses"  n ( b ou1 t - h au z i z ) ((pos "Kj%" ) ))
( "boat-race"  n ( b ou1 t - r ei s ) ((pos "K6%" ) ))
( "boat-races"  n ( b ou1 t - r ei s i z ) ((pos "Kj%" ) ))
( "boat-train"  n ( b ou1 t - t r ei n ) ((pos "K6%" ) ))
( "boat-trains"  n ( b ou1 t - t r ei n z ) ((pos "Kj%" ) ))
( "boated"  v ( b ou1 t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "boater"  n ( b ou1 t @ r ) ((pos "K6%" ) ))
( "boaters"  n ( b ou1 t @ z ) ((pos "Kj%" ) ))
( "boating"  v ( b ou1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "boatman"  n ( b ou1 t m @ n ) ((pos "Ki%" ) ))
( "boatmen"  n ( b ou1 t m @ n ) ((pos "Kj%" ) ))
( "boats"  v ( b ou1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "boatswain"  n ( b ou1 s n ) ((pos "K6$" ) ))
( "boatswains"  n ( b ou1 s n z ) ((pos "Kj$" ) ))
( "bob"  v ( b o1 b ) ((pos "J4%" "K9%" ) (subcat "2C" )))
( "bobbed"  v ( b o1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "bobbies"  n ( b o1 b i z ) ((pos "Kj%" ) ))
( "bobbin"  n ( b o1 b i n ) ((pos "K6%" ) ))
( "bobbing"  v ( b o1 b i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "bobbins"  n ( b o1 b i n z ) ((pos "Kj%" ) ))
( "bobby"  n ( b o1 b ii ) ((pos "K8%" ) ))
( "bobby-socks"  n ( b o1 b i - s o k s ) ((pos "Kj$" ) ))
( "bobby-sox"  n ( b o1 b i - s o k s ) ((pos "Kj$" ) ))
( "bobby-soxer"  n ( b o1 b i - s o k s @ r ) ((pos "K6$" ) ))
( "bobby-soxers"  n ( b o1 b i - s o k s @ z ) ((pos "Kj$" ) ))
( "bobolink"  n ( b o1 b @ l i ng k ) ((pos "K6$" ) ))
( "bobolinks"  n ( b o1 b @ l i ng k s ) ((pos "Kj$" ) ))
( "bobs"  v ( b o1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "bobsled"  n ( b o1 b s l e d ) ((pos "K6$" ) ))
( "bobsleds"  n ( b o1 b s l e d z ) ((pos "Kj$" ) ))
( "bobsleigh"  n ( b o1 b s l ei ) ((pos "K6%" ) ))
( "bobsleighs"  n ( b o1 b s l ei z ) ((pos "Kj%" ) ))
( "bobtail"  n ( b o1 b t ei l ) ((pos "K6%" ) ))
( "bobtails"  n ( b o1 b t ei l z ) ((pos "Kj%" ) ))
( "bode"  v ( b ou1 d ) ((pos "J2%" ) (subcat "12B" )))
( "boded"  v ( b ou1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "12B" )))
( "bodes"  v ( b ou1 d z ) ((pos "Ja%" ) (subcat "12B" )))
( "bodice"  n ( b o1 d i s ) ((pos "K6%" ) ))
( "bodices"  n ( b o1 d i s i z ) ((pos "Kj%" ) ))
( "bodied"  j ( b o1 d i d ) ((pos "Ot%" ) ))
( "bodies"  n ( b o1 d i z ) ((pos "Mj%" ) ))
( "bodily"  j ( b o1 d @ l ii ) ((pos "OA%" "Pu%" ) ))
( "boding"  v ( b ou1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "12B" )))
( "bodkin"  n ( b o1 d k i n ) ((pos "K6$" ) ))
( "bodkins"  n ( b o1 d k i n z ) ((pos "Kj$" ) ))
( "body"  n ( b o1 d ii ) ((pos "M8*" ) ))
( "body-servant"  n ( b o1 d i - s @@ v @ n t ) ((pos "K6$" ) ))
( "body-servants"  n ( b o1 d i - s @@ v @ n t s ) ((pos "Kj$" ) ))
( "body-snatcher"  n ( b o1 d i - s n a ch @ r ) ((pos "K6%" ) ))
( "body-snatchers"  n ( b o1 d i - s n a ch @ z ) ((pos "Kj%" ) ))
( "bodyguard"  n ( b o1 d i g aa d ) ((pos "K6%" ) ))
( "bodyguards"  n ( b o1 d i g aa d z ) ((pos "Kj%" ) ))
( "bodywork"  n ( b o1 d i w @@ k ) ((pos "K6%" ) ))
( "bodyworks"  n ( b o1 d i w @@ k s ) ((pos "Kj$" ) ))
( "boffin"  n ( b o1 f i n ) ((pos "K6%" ) ))
( "boffins"  n ( b o1 f i n z ) ((pos "Kj%" ) ))
( "bog"  v ( b o1 g ) ((pos "J4%" "K6%" ) (subcat "2E" )))
( "bogey"  n ( b ou1 g ii ) ((pos "K6%" ) ))
( "bogeyman"  n ( b ou1 g i m a n ) ((pos "Ki%" ) ))
( "bogeymen"  n ( b ou1 g i m e n ) ((pos "Kj%" ) ))
( "bogeys"  n ( b ou1 g i z ) ((pos "Kj%" ) ))
( "bogged"  v ( b o1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2E" )))
( "boggier"  j ( b o1 g i@ r ) ((pos "Or$" ) ))
( "boggiest"  j ( b o1 g i i s t ) ((pos "Os$" ) ))
( "bogging"  v ( b o1 g i ng ) ((pos "Jb%" ) (subcat "2E" )))
( "boggle"  v ( b o1 g l ) ((pos "I2%" ) (subcat "2A" )))
( "boggled"  v ( b o1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "boggles"  v ( b o1 g l z ) ((pos "Ia%" ) (subcat "2A" )))
( "boggling"  v ( b o1 g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "boggy"  j ( b o1 g ii ) ((pos "OD%" ) ))
( "bogie"  n ( b ou1 g ii ) ((pos "K6$" ) ))
( "bogies"  n ( b ou1 g i z ) ((pos "Kj$" ) ))
( "bogs"  v ( b o1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2E" )))
( "bogus"  j ( b ou1 g @ s ) ((pos "OA%" ) ))
( "bogy"  n ( b ou1 g ii ) ((pos "K8$" ) ))
( "boh"  uh ( b ou1 ) ((pos "W-%" ) ))
( "bohemian"  n ( b ou1 h ii1 m i@ n ) ((pos "K6%" "OA%" ) ))
( "bohemians"  n ( b ou1 h ii1 m i@ n z ) ((pos "Kj%" ) ))
( "boil"  v ( b oi1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15B" )))
( "boiled"  v ( b oi1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15B" )))
( "boiler"  n ( b oi1 l @ r ) ((pos "K6%" ) ))
( "boilers"  n ( b oi1 l @ z ) ((pos "Kj%" ) ))
( "boilersuit"  n ( b oi1 l @ s uu t ) ((pos "K6%" ) ))
( "boilersuits"  n ( b oi1 l @ s uu t s ) ((pos "Kj%" ) ))
( "boiling"  v ( b oi1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15B" )))
( "boiling-point"  n ( b oi1 l i ng - p oi n t ) ((pos "K6%" ) ))
( "boiling-points"  n ( b oi1 l i ng - p oi n t s ) ((pos "Kj%" ) ))
( "boils"  v ( b oi1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15B" )))
( "boisterous"  j ( b oi1 s t @ r @ s ) ((pos "OA%" ) ))
( "boisterously"  a ( b oi1 s t @ r @ s l ii ) ((pos "Pu%" ) ))
( "boisterousness"  n ( b oi1 s t @ r @ s n i s ) ((pos "L@%" ) ))
( "bold"  j ( b ou1 l d ) ((pos "OC%" ) ))
( "bolder"  j ( b ou1 l d @ r ) ((pos "Or%" ) ))
( "boldest"  j ( b ou1 l d i s t ) ((pos "Os%" ) ))
( "boldly"  a ( b ou1 l d l ii ) ((pos "Pu%" ) ))
( "boldness"  n ( b ou1 l d n @ s ) ((pos "L@%" ) ))
( "bole"  n ( b ou1 l ) ((pos "K6$" ) ))
( "bolero"  n ( b @1 l e@1 r ou ) ((pos "K6$" ) ))
( "boleros"  n ( b @1 l e@1 r ou z ) ((pos "Kj$" ) ))
( "boles"  n ( b ou1 l z ) ((pos "Kj$" ) ))
( "boll"  n ( b ou1 l ) ((pos "K6$" ) ))
( "bollard"  n ( b o1 l @ d ) ((pos "K6%" ) ))
( "bollards"  n ( b o1 l @ d z ) ((pos "Kj%" ) ))
( "bollock"  n ( b o1 l @ k ) ((pos "K6$" ) ))
( "bollocks"  n ( b o1 l @ k s ) ((pos "Kj%" ) ))
( "bolls"  n ( b ou1 l z ) ((pos "Kj$" ) ))
( "boloney"  n ( b @1 l ou1 n ii ) ((pos "L@$" ) ))
( "bolshy"  j ( b o1 l sh ii ) ((pos "OA%" ) ))
( "bolster"  v ( b ou1 l s t @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "bolstered"  v ( b ou1 l s t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bolstering"  v ( b ou1 l s t @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bolsters"  v ( b ou1 l s t @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "bolt"  v ( b ou1 l t ) ((pos "J0%" "K6%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "bolt-hole"  n ( b ou1 l t - h ou l ) ((pos "K6%" ) ))
( "bolt-holes"  n ( b ou1 l t - h ou l z ) ((pos "Kj%" ) ))
( "bolted"  v ( b ou1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bolting"  v ( b ou1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "bolts"  v ( b ou1 l t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "bomb"  v ( b o1 m ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "bomb-proof"  j ( b o1 m - p r uu f ) ((pos "OA%" ) ))
( "bomb-sight"  n ( b o1 m - s ai t ) ((pos "K6%" ) ))
( "bomb-sights"  n ( b o1 m - s ai t s ) ((pos "Kj%" ) ))
( "bomb-site"  n ( b o1 m - s ai t ) ((pos "K6%" ) ))
( "bomb-sites"  n ( b o1 m - s ai t s ) ((pos "Kj%" ) ))
( "bombard"  v ( b o1 m b aa1 d ) ((pos "H0%" ) (subcat "6A" )))
( "bombarded"  v ( b o1 m b aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bombardier"  n ( b o2 m b @ d i@1 r ) ((pos "K6%" ) ))
( "bombardiers"  n ( b o2 m b @ d i@1 z ) ((pos "Kj%" ) ))
( "bombarding"  v ( b o1 m b aa1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bombardment"  n ( b o1 m b aa1 d m @ n t ) ((pos "M6%" ) ))
( "bombardments"  n ( b o1 m b aa1 d m @ n t s ) ((pos "Mj%" ) ))
( "bombards"  v ( b o1 m b aa1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "bombast"  n ( b o1 m b a s t ) ((pos "L@%" ) ))
( "bombastic"  j ( b o1 m b a1 s t i k ) ((pos "OA%" ) ))
( "bombastically"  a ( b o1 m b a1 s t i k l ii ) ((pos "Pu%" ) ))
( "bombed"  v ( b o1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "bomber"  n ( b o1 m @ r ) ((pos "K6%" ) ))
( "bombers"  n ( b o1 m @ z ) ((pos "Kj%" ) ))
( "bombing"  v ( b o1 m i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "bombs"  v ( b o1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "bombshell"  n ( b o1 m sh e l ) ((pos "K6%" ) ))
( "bombshells"  n ( b o1 m sh e l z ) ((pos "Kj%" ) ))
( "bon mot"  n ( b o2 n - m ou1 ) ((pos "Ki%" ) ))
( "bona fide"  j ( b ou2 n @ - f ai1 d ii ) ((pos "OA%" "Pu$" ) ))
( "bona fides"  n ( b ou2 n @ - f ai1 d i z ) ((pos "Kj$" ) ))
( "bonanza"  n ( b @1 n a1 n z @ ) ((pos "K6%" ) ))
( "bonanzas"  n ( b @1 n a1 n z @ z ) ((pos "Kj$" ) ))
( "bonbon"  n ( b o1 n b o n ) ((pos "K6%" ) ))
( "bonbons"  n ( b o1 n b o n z ) ((pos "Kj%" ) ))
( "bond"  v ( b o1 n d ) ((pos "H0%" "K6%" ) (subcat )))
( "bond-"  j ( b o n d - ) ((pos "U-%" ) ))
( "bond-holder"  n ( b o1 n d - h ou l d @ r ) ((pos "K6%" ) ))
( "bond-holders"  n ( b o1 n d - h ou l d @ z ) ((pos "Kj%" ) ))
( "bondage"  n ( b o1 n d i jh ) ((pos "L@%" ) ))
( "bonded"  v ( b o1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bonding"  v ( b o1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "bonds"  v ( b o1 n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bone"  v ( b ou1 n ) ((pos "H2%" "M6%" ) (subcat )))
( "bone-dry"  j ( b ou1 n - d r ai1 ) ((pos "OA%" ) ))
( "bone-head"  n ( b ou1 n - h e d ) ((pos "K6%" ) ))
( "bone-heads"  n ( b ou1 n - h e d z ) ((pos "Kj%" ) ))
( "bone-idle"  j ( b ou1 n - ai1 d l ) ((pos "OA%" ) ))
( "bone-lazy"  j ( b ou1 n - l ei1 z ii ) ((pos "OA$" ) ))
( "bone-setter"  n ( b ou1 n - s e t @ r ) ((pos "K6%" ) ))
( "bone-setters"  n ( b ou1 n - s e t @ z ) ((pos "Kj%" ) ))
( "boned"  v ( b ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "boneless"  j ( b ou1 n l i s ) ((pos "OA%" ) ))
( "bonemeal"  n ( b ou1 n m ii l ) ((pos "K6%" ) ))
( "bonemeals"  n ( b ou1 n m ii l z ) ((pos "Kj$" ) ))
( "boner"  n ( b ou1 n @ r ) ((pos "K6$" ) ))
( "boners"  n ( b ou1 n @ z ) ((pos "Kj$" ) ))
( "bones"  v ( b ou1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "boneshaker"  n ( b ou1 n sh ei k @ r ) ((pos "K6%" ) ))
( "boneshakers"  n ( b ou1 n sh ei k @ z ) ((pos "Kj%" ) ))
( "bonfire"  n ( b o1 n f ai @ r ) ((pos "K6%" ) ))
( "bonfires"  n ( b o1 n f ai @ z ) ((pos "Kj%" ) ))
( "bongo"  n ( b o1 ng g ou ) ((pos "K6%" ) ))
( "bongos"  n ( b o1 ng g ou z ) ((pos "Kj%" ) ))
( "bonhomie"  n ( b o1 n @ m ii ) ((pos "L@%" ) ))
( "bonier"  j ( b ou1 n i@ r ) ((pos "Or%" ) ))
( "boniest"  j ( b ou1 n i i s t ) ((pos "Os%" ) ))
( "boning"  v ( b ou1 n i ng ) ((pos "Hb%" ) (subcat )))
( "bonito"  n ( b @1 n ii1 t ou ) ((pos "K6$" ) ))
( "bonitos"  n ( b @1 n ii1 t ou z ) ((pos "Kj$" ) ))
( "bonkers"  j ( b o1 ng k @ z ) ((pos "Op%" ) ))
( "bonnet"  n ( b o1 n i t ) ((pos "K6%" ) ))
( "bonnets"  n ( b o1 n i t s ) ((pos "Kj%" ) ))
( "bonnier"  j ( b o1 n i@ r ) ((pos "Or%" ) ))
( "bonniest"  j ( b o1 n i i s t ) ((pos "Os%" ) ))
( "bonnily"  a ( b o1 n i l ii ) ((pos "Pu%" ) ))
( "bonny"  j ( b o1 n ii ) ((pos "OD%" ) ))
( "bons mots"  n ( b o2 n - m ou1 z ) ((pos "Kj%" ) ))
( "bonus"  n ( b ou1 n @ s ) ((pos "K7%" ) ))
( "bonuses"  n ( b ou1 n @ s i z ) ((pos "Kj%" ) ))
( "bony"  j ( b ou1 n ii ) ((pos "OD%" ) ))
( "boo"  v ( b uu1 ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "boob"  v ( b uu1 b ) ((pos "I0%" "K6%" ) ))
( "boobed"  v ( b uu1 b d ) ((pos "Ic%" "Id%" ) ))
( "boobies"  n ( b uu1 b i z ) ((pos "Kj%" ) ))
( "boobing"  v ( b uu1 b i ng ) ((pos "Ib%" ) ))
( "boobs"  v ( b uu1 b z ) ((pos "Ia%" "Kj%" ) ))
( "booby"  n ( b uu1 b ii ) ((pos "K8%" ) ))
( "booby-trap"  n ( b uu1 b i - t r a p ) ((pos "K6%" ) ))
( "booby-traps"  n ( b uu1 b i - t r a p s ) ((pos "Kj%" ) ))
( "booed"  v ( b uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "boogie"  n ( b uu1 g ii ) ((pos "M6%" ) ))
( "boogie-woogie"  n ( b uu2 g i - w uu1 g ii ) ((pos "M6$" ) ))
( "boogie-woogies"  n ( b uu2 g i - w uu1 g i z ) ((pos "Mj$" ) ))
( "boogies"  n ( b uu1 g i z ) ((pos "Mj%" ) ))
( "booing"  v ( b uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "book"  v ( b u1 k ) ((pos "H0%" "K6*" ) (subcat )))
( "book-end"  n ( b u1 k - e n d ) ((pos "K6%" ) ))
( "book-ends"  n ( b u1 k - e n d z ) ((pos "Kj%" ) ))
( "book-keeper"  n ( b u1 k - k ii p @ r ) ((pos "K6%" ) ))
( "book-keepers"  n ( b u1 k - k ii p @ z ) ((pos "Kj%" ) ))
( "bookable"  j ( b u1 k @ b l ) ((pos "OA%" ) ))
( "bookcase"  n ( b u1 k - k ei s ) ((pos "K6%" ) ))
( "bookcases"  n ( b u1 k - k ei s i z ) ((pos "Kj%" ) ))
( "bookclub"  n ( b u1 k - k l uh b ) ((pos "K6%" ) ))
( "bookclubs"  n ( b u1 k - k l uh b z ) ((pos "Kj%" ) ))
( "booked"  v ( b u1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bookie"  n ( b u1 k ii ) ((pos "K6%" ) ))
( "bookies"  n ( b u1 k i z ) ((pos "Kj%" ) ))
( "booking"  v ( b u1 k i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "bookings"  n ( b u1 k i ng z ) ((pos "Kj%" ) ))
( "bookish"  j ( b u1 k i sh ) ((pos "OA%" ) ))
( "bookishness"  n ( b u1 k i sh n @ s ) ((pos "L@%" ) ))
( "bookkeeper"  n ( b u1 k - k ii p @ r ) ((pos "K6%" ) ))
( "bookkeepers"  n ( b u1 k - k ii p @ z ) ((pos "Kj%" ) ))
( "bookkeeping"  n ( b u1 k - k ii p i ng ) ((pos "L@%" ) ))
( "booklet"  n ( b u1 k l i t ) ((pos "K6%" ) ))
( "booklets"  n ( b u1 k l i t s ) ((pos "Kj%" ) ))
( "bookmaker"  n ( b u1 k m ei k @ r ) ((pos "K6%" ) ))
( "bookmakers"  n ( b u1 k m ei k @ z ) ((pos "Kj%" ) ))
( "bookmark"  n ( b u1 k m aa k ) ((pos "K6%" ) ))
( "bookmarker"  n ( b u1 k m aa k @ r ) ((pos "K6$" ) ))
( "bookmarkers"  n ( b u1 k m aa k @ z ) ((pos "Kj$" ) ))
( "bookmarks"  n ( b u1 k m aa k s ) ((pos "Kj%" ) ))
( "bookmobile"  n ( b u1 k m ou b ii l ) ((pos "K6$" ) ))
( "bookmobiles"  n ( b u1 k m ou b ii l z ) ((pos "Kj$" ) ))
( "books"  v ( b u1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bookseller"  n ( b u1 k s e l @ r ) ((pos "K6%" ) ))
( "booksellers"  n ( b u1 k s e l @ z ) ((pos "Kj%" ) ))
( "bookshop"  n ( b u1 k sh o p ) ((pos "K6%" ) ))
( "bookshops"  n ( b u1 k sh o p s ) ((pos "Kj%" ) ))
( "bookstall"  n ( b u1 k s t oo l ) ((pos "K6%" ) ))
( "bookstalls"  n ( b u1 k s t oo l z ) ((pos "Kj%" ) ))
( "bookworm"  n ( b u1 k w @@ m ) ((pos "K6%" ) ))
( "bookworms"  n ( b u1 k w @@ m z ) ((pos "Kj%" ) ))
( "boom"  v ( b uu1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "boomed"  v ( b uu1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "boomerang"  n ( b uu1 m @ r a ng ) ((pos "K6%" ) ))
( "boomerangs"  n ( b uu1 m @ r a ng z ) ((pos "Kj%" ) ))
( "booming"  v ( b uu1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "booms"  v ( b uu1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "boon"  n ( b uu1 n ) ((pos "K6%" "OA$" ) ))
( "boons"  n ( b uu1 n z ) ((pos "Kj$" ) ))
( "boor"  n ( b u@1 r ) ((pos "K6%" ) ))
( "boorish"  j ( b u@1 r i sh ) ((pos "OA%" ) ))
( "boorishly"  a ( b u@1 r i sh l ii ) ((pos "Pu%" ) ))
( "boorishness"  n ( b u@1 r i sh n @ s ) ((pos "L@%" ) ))
( "boors"  n ( b u@1 z ) ((pos "Kj%" ) ))
( "boos"  v ( b uu1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "boost"  v ( b uu1 s t ) ((pos "H0%" "K6%" ) (subcat )))
( "boosted"  v ( b uu1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "booster"  n ( b uu1 s t @ r ) ((pos "K6%" ) ))
( "boosters"  n ( b uu1 s t @ z ) ((pos "Kj%" ) ))
( "boosting"  v ( b uu1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "boosts"  v ( b uu1 s t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "boot"  v ( b uu1 t ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "booted"  v ( b uu1 t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "15A" )))
( "bootee"  n ( b uu1 t ii1 ) ((pos "K6%" ) ))
( "bootees"  n ( b uu1 t ii1 z ) ((pos "Kj%" ) ))
( "booth"  n ( b uu1 dh ) ((pos "K6%" ) ))
( "booths"  n ( b uu1 dh z ) ((pos "Kj%" ) ))
( "booting"  v ( b uu1 t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "bootlace"  n ( b uu1 t l ei s ) ((pos "K6%" ) ))
( "bootlaces"  n ( b uu1 t l ei s i z ) ((pos "Kj%" ) ))
( "bootleg"  v ( b uu1 t l e g ) ((pos "H4$" ) ))
( "bootlegged"  v ( b uu1 t l e g d ) ((pos "Hc$" "Hd$" ) ))
( "bootlegger"  n ( b uu1 t l e g @ r ) ((pos "K6%" ) ))
( "bootleggers"  n ( b uu1 t l e g @ z ) ((pos "Kj%" ) ))
( "bootlegging"  v ( b uu1 t l e g i ng ) ((pos "Hb%" ) ))
( "bootlegs"  v ( b uu1 t l e g z ) ((pos "Ha$" ) ))
( "bootless"  j ( b uu1 t l i s ) ((pos "OA%" ) ))
( "boots"  v ( b uu1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "booty"  n ( b uu1 t ii ) ((pos "L@%" ) ))
( "booze"  v ( b uu1 z ) ((pos "I2%" "L@%" ) (subcat "2A" )))
( "booze-up"  n ( b uu1 z - uh p ) ((pos "K6%" ) ))
( "booze-ups"  n ( b uu1 z - uh p s ) ((pos "Kj%" ) ))
( "boozed"  v ( b uu1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "boozer"  n ( b uu1 z @ r ) ((pos "K6%" ) ))
( "boozers"  n ( b uu1 z @ z ) ((pos "Kj%" ) ))
( "boozes"  v ( b uu1 z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "boozier"  j ( b uu1 z i@ r ) ((pos "Or%" ) ))
( "booziest"  j ( b uu1 z i i s t ) ((pos "Os%" ) ))
( "boozing"  v ( b uu1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "boozy"  j ( b uu1 z ii ) ((pos "OD%" ) ))
( "bopeep"  n ( b ou2 p ii1 p ) ((pos "L@$" ) ))
( "boracic"  j ( b @1 r a1 s i k ) ((pos "OA%" ) ))
( "borage"  n ( b o1 r i jh ) ((pos "L@$" ) ))
( "borax"  n ( b oo1 r a k s ) ((pos "L@$" ) ))
( "border"  v ( b oo1 d @ r ) ((pos "J0%" "K6%" ) (subcat "3A" )))
( "bordered"  v ( b oo1 d @ d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "borderer"  n ( b oo1 d @ r @ r ) ((pos "K6$" ) ))
( "borderers"  n ( b oo1 d @ r @ z ) ((pos "Kj$" ) ))
( "bordering"  v ( b oo1 d @ r i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "borderland"  n ( b oo1 d @ l a n d ) ((pos "K6%" ) ))
( "borderlands"  n ( b oo1 d @ l a n d z ) ((pos "Kj%" ) ))
( "borderline"  n ( b oo1 d @ l ai n ) ((pos "K6%" ) ))
( "borderlines"  n ( b oo1 d @ l ai n z ) ((pos "Kj%" ) ))
( "borders"  v ( b oo1 d @ z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "bore"  v ( b oo1 r ) ((pos "J2%" "Jc%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15A" "15B" "16B" )))
( "bore-hole"  n ( b oo1 - h ou l ) ((pos "K6%" ) ))
( "bore-holes"  n ( b oo1 - h ou l z ) ((pos "Kj%" ) ))
( "bored"  v ( b oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15A" "15B" "16B" )))
( "boredom"  n ( b oo1 d @ m ) ((pos "L@%" ) ))
( "borer"  n ( b oo1 r @ r ) ((pos "K6%" ) ))
( "borers"  n ( b oo1 r @ z ) ((pos "Kj%" ) ))
( "bores"  v ( b oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15A" "15B" "16B" )))
( "boric"  j ( b oo1 r i k ) ((pos "OA%" ) ))
( "boring"  v ( b oo1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15A" "15B" "16B" )))
( "born"  v ( b oo1 n ) ((pos "Hd%" ) ))
( "borne"  v ( b oo1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "6D" "6E" "7A" "11" "12C" "14" "15B" "16B" )))
( "boron"  n ( b oo1 r o n ) ((pos "L@$" ) ))
( "borough"  n ( b uh1 r @ ) ((pos "K6%" ) ))
( "boroughs"  n ( b uh1 r @ z ) ((pos "Kj%" ) ))
( "borrow"  v ( b o1 r ou ) ((pos "H0%" ) (subcat "6A" )))
( "borrowed"  v ( b o1 r ou d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "borrower"  n ( b o1 r ou @ r ) ((pos "K6%" ) ))
( "borrowers"  n ( b o1 r ou @ z ) ((pos "Kj%" ) ))
( "borrowing"  v ( b o1 r ou i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "borrows"  v ( b o1 r ou z ) ((pos "Ha%" ) (subcat "6A" )))
( "borsch"  n ( b oo1 sh ) ((pos "L@$" ) ))
( "borstal"  n ( b oo1 s t l ) ((pos "K6%" ) ))
( "borstals"  n ( b oo1 s t l z ) ((pos "Kj$" ) ))
( "bortsch"  n ( b oo1 ch ) ((pos "L@$" ) ))
( "borzoi"  n ( b oo1 z oi ) ((pos "K6$" ) ))
( "borzois"  n ( b oo1 z oi z ) ((pos "Kj$" ) ))
( "bosh"  n ( b o1 sh ) ((pos "L@$" "W-$" ) ))
( "bosky"  j ( b o1 s k ii ) ((pos "OA$" ) ))
( "bosom"  n ( b u1 z @ m ) ((pos "K6%" ) ))
( "bosoms"  n ( b u1 z @ m z ) ((pos "Kj%" ) ))
( "boss"  v ( b o1 s ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "boss-eyed"  j ( b o1 s - ai d ) ((pos "OA%" ) ))
( "bossed"  v ( b o1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bosses"  v ( b o1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "bossier"  j ( b o1 s i@ r ) ((pos "Or%" ) ))
( "bossiest"  j ( b o1 s i i s t ) ((pos "Os%" ) ))
( "bossing"  v ( b o1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bossy"  j ( b o1 s ii ) ((pos "OD%" ) ))
( "botanical"  j ( b @1 t a1 n i k l ) ((pos "OA%" ) ))
( "botanist"  n ( b o1 t @ n i s t ) ((pos "K6%" ) ))
( "botanists"  n ( b o1 t @ n i s t s ) ((pos "Kj%" ) ))
( "botanize"  v ( b o1 t @ n ai z ) ((pos "I2$" ) ))
( "botanized"  v ( b o1 t @ n ai z d ) ((pos "Ic$" "Id$" ) ))
( "botanizes"  v ( b o1 t @ n ai z i z ) ((pos "Ia$" ) ))
( "botanizing"  v ( b o1 t @ n ai z i ng ) ((pos "Ib$" ) ))
( "botany"  n ( b o1 t @ n ii ) ((pos "L@%" ) ))
( "botch"  v ( b o1 ch ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "botched"  v ( b o1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "botcher"  n ( b o1 ch @ r ) ((pos "K6%" ) ))
( "botchers"  n ( b o1 ch @ z ) ((pos "Kj%" ) ))
( "botches"  v ( b o1 ch i z ) ((pos "Ha%" "Kj$" ) (subcat "6A" )))
( "botching"  v ( b o1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "both"  j ( b ou th ) ((pos "OA*" "Pu*" "Qx*" ) ))
( "bother"  v ( b o1 dh @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "3A" "4C" "6A" "14" )))
( "botheration"  uh ( b o2 dh @ r ei1 sh @ n ) ((pos "W-%" ) ))
( "bothered"  v ( b o1 dh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4C" "6A" "14" )))
( "bothering"  v ( b o1 dh @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4C" "6A" "14" )))
( "bothers"  v ( b o1 dh @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "4C" "6A" "14" )))
( "bothersome"  j ( b o1 dh @ s @ m ) ((pos "OA%" ) ))
( "bottle"  v ( b o1 t l ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "bottle-fed"  j ( b o1 t l - f e d ) ((pos "OA%" ) ))
( "bottle-green"  j ( b o2 t l - g r ii1 n ) ((pos "OA%" ) ))
( "bottled"  v ( b o1 t l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bottleneck"  n ( b o1 t l n e k ) ((pos "K6%" ) ))
( "bottlenecks"  n ( b o1 t l n e k s ) ((pos "Kj%" ) ))
( "bottles"  v ( b o1 t l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "bottling"  v ( b o1 t l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bottom"  v ( b o1 t @ m ) ((pos "I0%" "K6%" ) ))
( "bottomed"  v ( b o1 t @ m d ) ((pos "Ic%" "Id%" ) ))
( "bottoming"  v ( b o1 t @ m i ng ) ((pos "Ib%" ) ))
( "bottomless"  j ( b o1 t @ m l @ s ) ((pos "OA%" ) ))
( "bottoms"  v ( b o1 t @ m z ) ((pos "Ia%" "Kj%" ) ))
( "botulism"  n ( b o1 t y u l i z @ m ) ((pos "L@$" ) ))
( "boudoir"  n ( b uu1 d w aa r ) ((pos "K6%" ) ))
( "boudoirs"  n ( b uu1 d w aa z ) ((pos "Kj%" ) ))
( "bougainvillea"  n ( b uu2 g @ n v i1 l i@ ) ((pos "K6%" ) ))
( "bougainvilleas"  n ( b uu2 g @ n v i1 l i@ z ) ((pos "Kj%" ) ))
( "bough"  n ( b au1 ) ((pos "K6%" ) ))
( "boughs"  n ( b au1 z ) ((pos "Kj%" ) ))
( "bought"  v ( b oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12B" "13B" )))
( "bouillon"  n ( b uu1 y o n ) ((pos "L@$" ) ))
( "boulder"  n ( b ou1 l d @ r ) ((pos "K6%" ) ))
( "boulders"  n ( b ou1 l d @ z ) ((pos "Kj%" ) ))
( "boulevard"  n ( b uu1 l @ v aa d ) ((pos "K6%" ) ))
( "boulevards"  n ( b uu1 l @ v aa d z ) ((pos "Kj%" ) ))
( "bounce"  v ( b au1 n s ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" )))
( "bounced"  v ( b au1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "bounces"  v ( b au1 n s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "bouncier"  j ( b au1 n s i@ r ) ((pos "Or$" ) ))
( "bounciest"  j ( b au1 n s i@ s t ) ((pos "Os%" ) ))
( "bouncing"  v ( b au1 n s i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" )))
( "bouncy"  j ( b au1 n s ii ) ((pos "OD%" ) ))
( "bound"  v ( b au1 n d ) ((pos "J0%" "Jc%" "Jd%" "K6%" "OA%" ) (subcat "2A" "2C" "4A" "6A" "14" "15A" "15B" "16B" )))
( "boundaries"  n ( b au1 n d r i z ) ((pos "Kj%" ) ))
( "boundary"  n ( b au1 n d r ii ) ((pos "K8%" ) ))
( "bounded"  v ( b au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" "6A" "14" "15A" "15B" "16B" )))
( "bounden"  j ( b au1 n d @ n ) ((pos "OA%" ) ))
( "bounder"  n ( b au1 n d @ r ) ((pos "K6%" ) ))
( "bounders"  n ( b au1 n d @ z ) ((pos "Kj%" ) ))
( "bounding"  v ( b au1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" "6A" "14" "15A" "15B" "16B" )))
( "boundless"  j ( b au1 n d l i s ) ((pos "OA%" ) ))
( "boundlessly"  a ( b au1 n d l i s l ii ) ((pos "Pu%" ) ))
( "bounds"  v ( b au1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4A" "6A" "14" "15A" "15B" "16B" )))
( "bounteous"  j ( b au1 n t i@ s ) ((pos "OA%" ) ))
( "bounteously"  a ( b au1 n t i@ s l ii ) ((pos "Pu%" ) ))
( "bounties"  n ( b au1 n t i z ) ((pos "Mj$" ) ))
( "bountiful"  j ( b au1 n t i f @ l ) ((pos "OA%" ) ))
( "bountifully"  a ( b au1 n t i f @ l ii ) ((pos "Pu%" ) ))
( "bounty"  n ( b au1 n t ii ) ((pos "M8%" ) ))
( "bouquet"  n ( b u1 k ei1 ) ((pos "K6%" ) ))
( "bouquets"  n ( b u1 k ei1 z ) ((pos "Kj%" ) ))
( "bourbon"  n ( b @@1 b @ n ) ((pos "M6%" ) ))
( "bourbons"  n ( b @@1 b @ n z ) ((pos "Mj$" ) ))
( "bourgeois"  n ( b u@1 zh w aa ) ((pos "K9%" "OA%" ) ))
( "bourgeoisie"  n ( b u@2 zh w aa z ii1 ) ((pos "K6%" ) ))
( "bourgeoisies"  n ( b u@2 zh w aa z ii1 z ) ((pos "Kj$" ) ))
( "bourn"  n ( b u@1 n ) ((pos "K6$" ) ))
( "bourne"  n ( b u@1 n ) ((pos "K6$" ) ))
( "bournes"  n ( b u@1 n z ) ((pos "Kj$" ) ))
( "bourns"  n ( b u@1 n z ) ((pos "Kj$" ) ))
( "bourse"  n ( b u@1 s ) ((pos "K6$" ) ))
( "bourses"  n ( b u@1 s i z ) ((pos "Kj$" ) ))
( "bout"  n ( b au1 t ) ((pos "K6%" ) ))
( "boutique"  n ( b uu1 t ii1 k ) ((pos "K6%" ) ))
( "boutiques"  n ( b uu1 t ii1 k s ) ((pos "Kj%" ) ))
( "bouts"  n ( b au1 t s ) ((pos "Kj%" ) ))
( "bovine"  j ( b ou1 v ai n ) ((pos "OA%" ) ))
( "bovril"  n ( b o1 v r i l ) ((pos "L@%" ) ))
( "bow"  v ( b ou1 ) ((pos "H0%" "K6%" ) ))
( "bow"  v ( b au1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "bow-wow"  n ( b au1 - w au ) ((pos "K6%" ) ))
( "bow-wow"  uh ( b au2 - w au1 ) ((pos "W-%" ) ))
( "bow-wows"  n ( b au1 - w au z ) ((pos "Kj%" ) ))
( "bowdlerize"  v ( b au1 d l @ r ai z ) ((pos "H2$" ) ))
( "bowdlerized"  v ( b au1 d l @ r ai z d ) ((pos "Hc$" "Hd$" ) ))
( "bowdlerizes"  v ( b au1 d l @ r ai z i z ) ((pos "Ha$" ) ))
( "bowdlerizing"  v ( b au1 d l @ r ai z i ng ) ((pos "Hb$" ) ))
( "bowed"  v ( b ou1 d ) ((pos "Hc%" "Hd%" ) ))
( "bowed"  v ( b au1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "bowel"  n ( b au1 @ l ) ((pos "K6%" ) ))
( "bowels"  n ( b au1 @ l z ) ((pos "Kj%" ) ))
( "bower"  n ( b au1 @ r ) ((pos "K6%" ) ))
( "bowers"  n ( b au1 @ z ) ((pos "Kj%" ) ))
( "bowie knife"  n ( b ou1 i - n ai f ) ((pos "Ki%" ) ))
( "bowie knives"  n ( b ou1 i - n ai v z ) ((pos "Kj%" ) ))
( "bowing"  v ( b ou1 i ng ) ((pos "Hb%" "L@%" ) ))
( "bowing"  v ( b au1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "bowl"  v ( b ou1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "bowled"  v ( b ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bowler"  n ( b ou1 l @ r ) ((pos "K6%" ) ))
( "bowlers"  n ( b ou1 l @ z ) ((pos "Kj%" ) ))
( "bowline"  n ( b ou1 l i n ) ((pos "K6$" ) ))
( "bowlines"  n ( b ou1 l i n z ) ((pos "Kj$" ) ))
( "bowling"  v ( b ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "bowling-green"  n ( b ou1 l i ng - g r ii n ) ((pos "K6%" ) ))
( "bowling-greens"  n ( b ou1 l i ng - g r ii n z ) ((pos "Kj%" ) ))
( "bowls"  v ( b ou1 l z ) ((pos "Ja%" "Kj%" "Lk%" ) (subcat "2A" "2C" "6A" )))
( "bowman"  n ( b ou1 m @ n ) ((pos "Ki%" ) ))
( "bowmen"  n ( b ou1 m e n ) ((pos "Kj%" ) ))
( "bows"  v ( b ou1 z ) ((pos "Ha%" "Kj%" ) ))
( "bows"  v ( b au1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "bowsprit"  n ( b ou1 s p r i t ) ((pos "K6$" ) ))
( "bowsprits"  n ( b ou1 s p r i t s ) ((pos "Kj$" ) ))
( "box"  v ( b o1 k s ) ((pos "J1%" "M7%" ) (subcat "2A" "3A" "6A" )))
( "box-kite"  n ( b o1 k s - k ai t ) ((pos "K6%" ) ))
( "box-kites"  n ( b o1 k s - k ai t s ) ((pos "Kj%" ) ))
( "box-number"  n ( b o1 k s - n uh m b @ r ) ((pos "K6%" ) ))
( "box-numbers"  n ( b o1 k s - n uh m b @ z ) ((pos "Kj%" ) ))
( "box-office"  n ( b o1 k s - o f i s ) ((pos "K6%" ) ))
( "box-offices"  n ( b o1 k s - o f i s i z ) ((pos "Kj%" ) ))
( "boxed"  v ( b o1 k s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "boxer"  n ( b o1 k s @ r ) ((pos "K6%" ) ))
( "boxers"  n ( b o1 k s @ z ) ((pos "Kj%" ) ))
( "boxes"  v ( b o1 k s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "boxful"  n ( b o1 k s f u l ) ((pos "K6%" ) ))
( "boxfuls"  n ( b o1 k s f u l z ) ((pos "Kj%" ) ))
( "boxing"  v ( b o1 k s i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" "6A" )))
( "boxing-glove"  n ( b o1 k s i ng - g l uh v ) ((pos "K6%" ) ))
( "boxing-gloves"  n ( b o1 k s i ng - g l uh v z ) ((pos "Kj%" ) ))
( "boxing-match"  n ( b o1 k s i ng - m a ch ) ((pos "K7%" ) ))
( "boxing-matches"  n ( b o1 k s i ng - m a ch i z ) ((pos "Kj%" ) ))
( "boxwood"  n ( b o1 k s w u d ) ((pos "L@%" ) ))
( "boy"  n ( b oi1 ) ((pos "K6*" ) ))
( "boycott"  v ( b oi1 k o t ) ((pos "H0%" "K6%" ) (subcat )))
( "boycotted"  v ( b oi1 k o t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "boycotting"  v ( b oi1 k o t i ng ) ((pos "Hb%" ) (subcat )))
( "boycotts"  v ( b oi1 k o t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "boyfriend"  n ( b oi1 f r e n d ) ((pos "K6%" ) ))
( "boyfriends"  n ( b oi1 f r e n d z ) ((pos "Kj%" ) ))
( "boyhood"  n ( b oi1 h u d ) ((pos "L@%" ) ))
( "boyish"  j ( b oi1 i sh ) ((pos "OA%" ) ))
( "boyishly"  a ( b oi1 i sh l ii ) ((pos "Pu%" ) ))
( "boys"  n ( b oi1 z ) ((pos "Kj%" ) ))
( "bra"  n ( b r aa1 ) ((pos "K6%" ) ))
( "brace"  v ( b r ei1 s ) ((pos "J2%" "K9%" ) (subcat "6A" "15B" )))
( "braced"  v ( b r ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15B" )))
( "bracelet"  n ( b r ei1 s l i t ) ((pos "K6%" ) ))
( "bracelets"  n ( b r ei1 s l i t s ) ((pos "Kj%" ) ))
( "braces"  v ( b r ei1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "6A" "15B" )))
( "bracing"  v ( b r ei1 s i ng ) ((pos "Jb%" ) (subcat "6A" "15B" )))
( "bracken"  n ( b r a1 k @ n ) ((pos "L@%" ) ))
( "bracket"  v ( b r a1 k i t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "bracketed"  v ( b r a1 k i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bracketing"  v ( b r a1 k i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "brackets"  v ( b r a1 k i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "brackish"  j ( b r a1 k i sh ) ((pos "OA%" ) ))
( "bract"  n ( b r a1 k t ) ((pos "K6$" ) ))
( "bracts"  n ( b r a1 k t s ) ((pos "Kj$" ) ))
( "brad"  n ( b r a1 d ) ((pos "K6$" ) ))
( "bradawl"  n ( b r a1 d oo l ) ((pos "K6$" ) ))
( "bradawls"  n ( b r a1 d oo l z ) ((pos "Kj$" ) ))
( "brads"  n ( b r a1 d z ) ((pos "Kj$" ) ))
( "brae"  n ( b r ei1 ) ((pos "K6$" ) ))
( "braes"  n ( b r ei1 z ) ((pos "Kj$" ) ))
( "brag"  v ( b r a1 g ) ((pos "I4%" ) (subcat "2A" )))
( "braggart"  n ( b r a1 g @ t ) ((pos "K6%" ) ))
( "braggarts"  n ( b r a1 g @ t s ) ((pos "Kj%" ) ))
( "bragged"  v ( b r a1 g d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "bragging"  v ( b r a1 g i ng ) ((pos "Ib%" "L@%" ) (subcat "2A" )))
( "brags"  v ( b r a1 g z ) ((pos "Ia%" ) (subcat "2A" )))
( "braid"  v ( b r ei1 d ) ((pos "H0%" "M6%" ) (subcat )))
( "braided"  v ( b r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "braiding"  v ( b r ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "braids"  v ( b r ei1 d z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "braille"  n ( b r ei1 l ) ((pos "L@%" ) ))
( "brain"  v ( b r ei1 n ) ((pos "H0%" "K6%" ) (subcat )))
( "brain-fag"  n ( b r ei1 n - f a g ) ((pos "L@$" ) ))
( "brain-teaser"  n ( b r ei1 n - t ii z @ r ) ((pos "K6%" ) ))
( "brain-teasers"  n ( b r ei1 n - t ii z @ z ) ((pos "Kj%" ) ))
( "brainchild"  n ( b r ei1 n ch ai l d ) ((pos "Ki%" ) ))
( "brained"  v ( b r ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "brainier"  j ( b r ei1 n i@ r ) ((pos "Or%" ) ))
( "brainiest"  j ( b r ei1 n i i s t ) ((pos "Os%" ) ))
( "braining"  v ( b r ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "brainless"  j ( b r ei1 n l @ s ) ((pos "OA%" ) ))
( "brains"  v ( b r ei1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "brainstorm"  n ( b r ei1 n s t oo m ) ((pos "K6%" ) ))
( "brainstorms"  n ( b r ei1 n s t oo m z ) ((pos "Kj%" ) ))
( "brainwash"  v ( b r ei1 n w o sh ) ((pos "H1%" ) (subcat )))
( "brainwashed"  v ( b r ei1 n w o sh t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "brainwashes"  v ( b r ei1 n w o sh i z ) ((pos "Ha$" ) (subcat )))
( "brainwashing"  v ( b r ei1 n w o sh i ng ) ((pos "Hb$" "L@%" ) (subcat )))
( "brainwave"  n ( b r ei1 n w ei v ) ((pos "K6%" ) ))
( "brainwaves"  n ( b r ei1 n w ei v z ) ((pos "Kj%" ) ))
( "brainy"  j ( b r ei1 n ii ) ((pos "OD%" ) ))
( "braise"  v ( b r ei1 z ) ((pos "H2%" ) (subcat )))
( "braised"  v ( b r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "braises"  v ( b r ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "braising"  v ( b r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "brake"  v ( b r ei1 k ) ((pos "J2%" "Jc%" "K6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "braked"  v ( b r ei1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "brakeman"  n ( b r ei1 k m @ n ) ((pos "Ki$" ) ))
( "brakemen"  n ( b r ei1 k m @ n ) ((pos "Kj$" ) ))
( "brakes"  v ( b r ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "braking"  v ( b r ei1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "bramble"  n ( b r a1 m b l ) ((pos "K6%" ) ))
( "brambles"  n ( b r a1 m b l z ) ((pos "Kj%" ) ))
( "bran"  n ( b r a1 n ) ((pos "L@%" ) ))
( "bran-new"  j ( b r a2 n - n y uu1 ) ((pos "OA$" ) ))
( "branch"  v ( b r aa1 n ch ) ((pos "I1%" "K7%" ) (subcat "2A" )))
( "branched"  v ( b r aa1 n ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "branches"  v ( b r aa1 n ch i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "branchier"  j ( b r aa1 n ch i@ r ) ((pos "Or$" ) ))
( "branchiest"  j ( b r aa1 n ch i i s t ) ((pos "Os$" ) ))
( "branching"  v ( b r aa1 n ch i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "branchy"  j ( b r aa1 n ch ii ) ((pos "OD$" ) ))
( "brand"  v ( b r a1 n d ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "brand-new"  j ( b r a2 n - n y uu1 ) ((pos "OA%" ) ))
( "branded"  v ( b r a1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "brandies"  n ( b r a1 n d i z ) ((pos "Mj%" ) ))
( "branding"  v ( b r a1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "branding-iron"  n ( b r a1 n d i ng - ai @ n ) ((pos "K6%" ) ))
( "branding-irons"  n ( b r a1 n d i ng - ai @ n z ) ((pos "Kj%" ) ))
( "brandish"  v ( b r a1 n d i sh ) ((pos "H1%" ) (subcat )))
( "brandished"  v ( b r a1 n d i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "brandishes"  v ( b r a1 n d i sh i z ) ((pos "Ha%" ) (subcat )))
( "brandishing"  v ( b r a1 n d i sh i ng ) ((pos "Hb%" ) (subcat )))
( "brands"  v ( b r a1 n d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "brandy"  n ( b r a1 n d ii ) ((pos "M8%" ) ))
( "brandy-ball"  n ( b r a1 n d i - b oo l ) ((pos "K6%" ) ))
( "brandy-balls"  n ( b r a1 n d i - b oo l z ) ((pos "Kj%" ) ))
( "brandy-snap"  n ( b r a1 n d i - s n a p ) ((pos "K6%" ) ))
( "brandy-snaps"  n ( b r a1 n d i - s n a p s ) ((pos "Kj%" ) ))
( "bras"  n ( b r aa1 z ) ((pos "Kj%" ) ))
( "brash"  j ( b r a1 sh ) ((pos "OC%" ) ))
( "brasher"  j ( b r a1 sh @ r ) ((pos "Or%" ) ))
( "brashest"  j ( b r a1 sh i s t ) ((pos "Os%" ) ))
( "brass"  n ( b r aa1 s ) ((pos "M7%" ) ))
( "brassard"  n ( b r a1 s aa d ) ((pos "K6$" ) ))
( "brassards"  n ( b r a1 s aa d z ) ((pos "Kj$" ) ))
( "brasserie"  n ( b r a1 s @ r ii ) ((pos "K6%" ) ))
( "brasseries"  n ( b r a1 s @ r i z ) ((pos "Kj%" ) ))
( "brasses"  n ( b r aa1 s i z ) ((pos "Mj%" ) ))
( "brassi`ere"  n ( b r a1 z i@ r ) ((pos "K6$" ) ))
( "brassi`eres"  n ( b r a1 z i@ z ) ((pos "Kj$" ) ))
( "brassier"  j ( b r aa1 s i@ r ) ((pos "Or%" ) ))
( "brassiere"  n ( b r a1 z i@ r ) ((pos "K6$" ) ))
( "brassieres"  n ( b r a1 z i@ z ) ((pos "Kj$" ) ))
( "brassiest"  j ( b r aa1 s i i s t ) ((pos "Os%" ) ))
( "brassy"  j ( b r aa1 s ii ) ((pos "OD%" ) ))
( "brat"  n ( b r a1 t ) ((pos "K6%" ) ))
( "brats"  n ( b r a1 t s ) ((pos "Kj%" ) ))
( "bravado"  n ( b r @1 v aa1 d ou ) ((pos "M6%" ) ))
( "bravados"  n ( b r @1 v aa1 d ou z ) ((pos "Mj$" ) ))
( "brave"  v ( b r ei1 v ) ((pos "H2%" "K6%" "OB%" ) (subcat "6A" )))
( "braved"  v ( b r ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bravely"  a ( b r ei1 v l ii ) ((pos "Pu%" ) ))
( "braver"  j ( b r ei1 v @ r ) ((pos "Or%" ) ))
( "bravery"  n ( b r ei1 v @ r ii ) ((pos "L@%" ) ))
( "braves"  v ( b r ei1 v z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "bravest"  j ( b r ei1 v i s t ) ((pos "Os%" ) ))
( "braving"  v ( b r ei1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bravo"  n ( b r aa2 v ou1 ) ((pos "K6$" "W-%" ) ))
( "bravoes"  n ( b r aa2 v ou1 z ) ((pos "Kj$" ) ))
( "bravos"  n ( b r aa2 v ou1 z ) ((pos "Kj$" ) ))
( "brawl"  v ( b r oo1 l ) ((pos "I0%" "K6%" ) (subcat )))
( "brawled"  v ( b r oo1 l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "brawler"  n ( b r oo1 l @ r ) ((pos "K6%" ) ))
( "brawlers"  n ( b r oo1 l @ z ) ((pos "Kj%" ) ))
( "brawling"  v ( b r oo1 l i ng ) ((pos "Ib%" ) (subcat )))
( "brawls"  v ( b r oo1 l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "brawn"  n ( b r oo1 n ) ((pos "L@%" ) ))
( "brawnier"  j ( b r oo1 n i@ r ) ((pos "Or%" ) ))
( "brawniest"  j ( b r oo1 n i i s t ) ((pos "Os%" ) ))
( "brawny"  j ( b r oo1 n ii ) ((pos "OD%" ) ))
( "bray"  v ( b r ei1 ) ((pos "H0%" "K6%" ) (subcat )))
( "brayed"  v ( b r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "braying"  v ( b r ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "brays"  v ( b r ei1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "braze"  v ( b r ei1 z ) ((pos "H2$" ) (subcat )))
( "brazed"  v ( b r ei1 z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "brazen"  v ( b r ei1 z n ) ((pos "H0%" "OA%" ) ))
( "brazen-faced"  j ( b r ei1 z n - f ei s t ) ((pos "OA$" ) ))
( "brazened"  v ( b r ei1 z n d ) ((pos "Hc%" "Hd%" ) ))
( "brazening"  v ( b r ei1 z n i ng ) ((pos "Hb%" ) ))
( "brazens"  v ( b r ei1 z n z ) ((pos "Ha%" ) ))
( "brazes"  v ( b r ei1 z i z ) ((pos "Ha$" ) (subcat )))
( "brazier"  n ( b r ei1 z i@ r ) ((pos "K6%" ) ))
( "braziers"  n ( b r ei1 z i@ z ) ((pos "Kj%" ) ))
( "brazing"  v ( b r ei1 z i ng ) ((pos "Hb$" ) (subcat )))
( "breach"  v ( b r ii1 ch ) ((pos "H1%" "K7%" ) (subcat )))
( "breached"  v ( b r ii1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "breaches"  v ( b r ii1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "breaching"  v ( b r ii1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "bread"  n ( b r e1 d ) ((pos "L@%" ) ))
( "breadcrumb"  n ( b r e1 d k r uh m ) ((pos "K6%" ) ))
( "breadcrumbs"  n ( b r e1 d k r uh m z ) ((pos "Kj%" ) ))
( "breadfruit"  n ( b r e1 d f r uu t ) ((pos "K9%" ) ))
( "breadfruits"  n ( b r e1 d f r uu t s ) ((pos "Kj$" ) ))
( "breadline"  n ( b r e1 d l ai n ) ((pos "K6%" ) ))
( "breadlines"  n ( b r e1 d l ai n z ) ((pos "Kj$" ) ))
( "breadstuffs"  n ( b r e1 d s t uh f s ) ((pos "Kj$" ) ))
( "breadth"  n ( b r e1 d th ) ((pos "M6%" ) ))
( "breadths"  n ( b r e1 d th s ) ((pos "Mj%" ) ))
( "breadthways"  a ( b r e1 d th w ei z ) ((pos "Pu%" ) ))
( "breadthwise"  a ( b r e1 d th w ai z ) ((pos "Pu%" ) ))
( "breadwinner"  n ( b r e1 d w i n @ r ) ((pos "K6%" ) ))
( "breadwinners"  n ( b r e1 d w i n @ z ) ((pos "Kj%" ) ))
( "break"  v ( b r ei1 k ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "break-in"  n ( b r ei1 k - i n ) ((pos "K6%" ) ))
( "break-ins"  n ( b r ei1 k - i n z ) ((pos "Kj%" ) ))
( "break-up"  n ( b r ei1 k - uh p ) ((pos "K6%" ) ))
( "break-ups"  n ( b r ei1 k - uh p s ) ((pos "Kj%" ) ))
( "breakable"  j ( b r ei1 k @ b l ) ((pos "OA%" ) ))
( "breakables"  n ( b r ei1 k @ b l z ) ((pos "Kj%" ) ))
( "breakage"  n ( b r ei1 k i jh ) ((pos "K6%" ) ))
( "breakages"  n ( b r ei1 k i jh i z ) ((pos "Kj%" ) ))
( "breakaway"  n ( b r ei1 k @ w ei ) ((pos "K6%" ) ))
( "breakaways"  n ( b r ei1 k @ w ei z ) ((pos "K6$" ) ))
( "breakdown"  n ( b r ei1 k d au n ) ((pos "K6%" ) ))
( "breakdowns"  n ( b r ei1 k d au n z ) ((pos "Kj%" ) ))
( "breaker"  n ( b r ei1 k @ r ) ((pos "K6%" ) ))
( "breakers"  n ( b r ei1 k @ z ) ((pos "Kj%" ) ))
( "breakfast"  v ( b r e1 k f @ s t ) ((pos "I0%" "K6%" ) ))
( "breakfasted"  v ( b r e1 k f @ s t i d ) ((pos "Ic%" "Id%" ) ))
( "breakfasting"  v ( b r e1 k f @ s t i ng ) ((pos "Ib%" ) ))
( "breakfasts"  v ( b r e1 k f @ s t s ) ((pos "Ia%" "Kj%" ) ))
( "breaking"  v ( b r ei1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "breakneck"  j ( b r ei1 k n e k ) ((pos "OA%" ) ))
( "breaks"  v ( b r ei1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "breakthrough"  n ( b r ei1 k th r uu ) ((pos "K6%" ) ))
( "breakthroughs"  n ( b r ei1 k th r uu z ) ((pos "Kj%" ) ))
( "breakwater"  n ( b r ei1 k w oo t @ r ) ((pos "K6%" ) ))
( "breakwaters"  n ( b r ei1 k w oo t @ z ) ((pos "Kj%" ) ))
( "bream"  n ( b r ii1 m ) ((pos "K9%" ) ))
( "breast"  v ( b r e1 s t ) ((pos "H0$" "K6%" ) (subcat )))
( "breast-deep"  a ( b r e1 s t - d ii p ) ((pos "Pu%" ) ))
( "breast-fed"  j ( b r e1 s t - f e d ) ((pos "OA%" ) ))
( "breast-high"  a ( b r e1 s t - h ai ) ((pos "Pu%" ) ))
( "breast-plate"  n ( b r e1 s t - p l ei t ) ((pos "K6%" ) ))
( "breast-plates"  n ( b r e1 s t - p l ei t s ) ((pos "Kj%" ) ))
( "breasted"  v ( b r e1 s t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "breastfed"  v ( b r e1 s t f e d ) ((pos "Jc%" "Jd%" ) ))
( "breastfeed"  v ( b r e1 s t f ii d ) ((pos "J5%" ) ))
( "breastfeeding"  v ( b r e1 s t f ii d i ng ) ((pos "Jb%" ) ))
( "breastfeeds"  v ( b r e1 s t f ii d z ) ((pos "Ja%" ) ))
( "breasting"  v ( b r e1 s t i ng ) ((pos "Hb$" ) (subcat )))
( "breasts"  v ( b r e1 s t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "breaststroke"  n ( b r e1 s - s t r ou k ) ((pos "L@%" ) ))
( "breastwork"  n ( b r e1 s t w @@ k ) ((pos "K6$" ) ))
( "breastworks"  n ( b r e1 s t w @@ k s ) ((pos "Kj$" ) ))
( "breath"  n ( b r e1 th ) ((pos "M6%" ) ))
( "breathalyser"  n ( b r e1 th @ l ai z @ r ) ((pos "K6%" ) ))
( "breathalysers"  n ( b r e1 th @ l ai z @ z ) ((pos "Kj%" ) ))
( "breathe"  v ( b r ii1 dh ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "breathed"  v ( b r ii1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "breather"  n ( b r ii1 dh @ r ) ((pos "K6%" ) ))
( "breathers"  n ( b r ii1 dh @ z ) ((pos "Kj%" ) ))
( "breathes"  v ( b r ii1 dh z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "breathing"  v ( b r ii1 dh i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "breathing-space"  n ( b r ii1 dh i ng - s p ei s ) ((pos "K6%" ) ))
( "breathing-spaces"  n ( b r ii1 dh i ng - s p ei s i z ) ((pos "Kj$" ) ))
( "breathless"  j ( b r e1 th l @ s ) ((pos "OA%" ) ))
( "breathlessly"  a ( b r e1 th l @ s l ii ) ((pos "Pu%" ) ))
( "breaths"  n ( b r e1 th s ) ((pos "Mj%" ) ))
( "breathtaking"  j ( b r e1 th t ei k i ng ) ((pos "OA%" ) ))
( "bred"  v ( b r e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "breech"  n ( b r ii1 ch ) ((pos "K7%" ) ))
( "breech-block"  n ( b r ii1 ch - b l o k ) ((pos "K6$" ) ))
( "breech-blocks"  n ( b r ii1 ch - b l o k s ) ((pos "Kj$" ) ))
( "breeches"  n ( b r i1 ch i z ) ((pos "Kj%" ) ))
( "breeches"  n ( b r ii1 ch i z ) ((pos "Kj%" ) ))
( "breeches-buoy"  n ( b r ii2 ch i z - b oi1 ) ((pos "K6%" ) ))
( "breeches-buoys"  n ( b r ii2 ch i z - b oi1 z ) ((pos "Kj%" ) ))
( "breed"  v ( b r ii1 d ) ((pos "J5%" "K6%" ) (subcat "2A" )))
( "breeder"  n ( b r ii1 d @ r ) ((pos "K6%" ) ))
( "breeders"  n ( b r ii1 d @ z ) ((pos "Kj%" ) ))
( "breeding"  v ( b r ii1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "breeds"  v ( b r ii1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "breeze"  v ( b r ii1 z ) ((pos "I2%" "M6%" ) (subcat )))
( "breezed"  v ( b r ii1 z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "breezes"  v ( b r ii1 z i z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "breezier"  j ( b r ii1 z i@ r ) ((pos "Or%" ) ))
( "breeziest"  j ( b r ii1 z i i s t ) ((pos "Os%" ) ))
( "breezily"  a ( b r ii1 z @ l ii ) ((pos "Pu%" ) ))
( "breeziness"  n ( b r ii1 z i n @ s ) ((pos "L@%" ) ))
( "breezing"  v ( b r ii1 z i ng ) ((pos "Ib%" ) (subcat )))
( "breezy"  j ( b r ii1 z ii ) ((pos "OD%" ) ))
( "brethren"  n ( b r e1 dh r @ n ) ((pos "Kj%" ) ))
( "breve"  n ( b r ii1 v ) ((pos "K6$" ) ))
( "breves"  n ( b r ii1 v z ) ((pos "Kj$" ) ))
( "brevet"  n ( b r e1 v i t ) ((pos "K6$" ) ))
( "brevets"  n ( b r e1 v i t s ) ((pos "Kj$" ) ))
( "breviaries"  n ( b r ii1 v i@ r i z ) ((pos "Kj$" ) ))
( "breviary"  n ( b r ii1 v i@ r ii ) ((pos "K8$" ) ))
( "brevity"  n ( b r e1 v i t ii ) ((pos "L@%" ) ))
( "brew"  v ( b r uu1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "brewed"  v ( b r uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "brewer"  n ( b r uu1 @ r ) ((pos "K6%" ) ))
( "breweries"  n ( b r u@1 r i z ) ((pos "Kj%" ) ))
( "brewers"  n ( b r uu1 @ z ) ((pos "Kj%" ) ))
( "brewery"  n ( b r u@1 r ii ) ((pos "K8%" ) ))
( "brewing"  v ( b r uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "brews"  v ( b r uu1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "briar"  n ( b r ai1 @ r ) ((pos "M6%" ) ))
( "briars"  n ( b r ai1 @ z ) ((pos "Mj%" ) ))
( "bribable"  j ( b r ai1 b @ b l ) ((pos "OA%" ) ))
( "bribe"  v ( b r ai1 b ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "bribed"  v ( b r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "bribery"  n ( b r ai1 b @ r ii ) ((pos "L@%" ) ))
( "bribes"  v ( b r ai1 b z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "bribing"  v ( b r ai1 b i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "bric-a-brac"  n ( b r i1 k - @ - b r a k ) ((pos "L@%" ) ))
( "brick"  v ( b r i1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "brick-field"  n ( b r i1 k - f ii l d ) ((pos "K6$" ) ))
( "brick-fields"  n ( b r i1 k - f ii l d z ) ((pos "Kj$" ) ))
( "brickbat"  n ( b r i1 k b a t ) ((pos "K6%" ) ))
( "brickbats"  n ( b r i1 k b a t s ) ((pos "Kj%" ) ))
( "bricked"  v ( b r i1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bricking"  v ( b r i1 k i ng ) ((pos "Hb%" ) (subcat )))
( "brickkiln"  n ( b r i1 k - k i l n ) ((pos "K6%" ) ))
( "brickkilns"  n ( b r i1 k - k i l n z ) ((pos "Kj%" ) ))
( "bricklayer"  n ( b r i1 k l ei @ r ) ((pos "K6%" ) ))
( "bricklayers"  n ( b r i1 k l ei @ z ) ((pos "Kj%" ) ))
( "bricks"  v ( b r i1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "brickwork"  n ( b r i1 k w @@ k ) ((pos "K6%" ) ))
( "brickworks"  n ( b r i1 k w @@ k s ) ((pos "Kj%" ) ))
( "bridal"  n ( b r ai1 d l ) ((pos "K6$" "Oq%" ) ))
( "bridals"  n ( b r ai1 d l z ) ((pos "Kj$" ) ))
( "bride"  n ( b r ai1 d ) ((pos "K6%" ) ))
( "bridecake"  n ( b r ai1 d k ei k ) ((pos "K6$" ) ))
( "bridecakes"  n ( b r ai1 d k ei k s ) ((pos "Kj$" ) ))
( "bridegroom"  n ( b r ai1 d g r u m ) ((pos "K6%" ) ))
( "bridegrooms"  n ( b r ai1 d g r u m z ) ((pos "Kj%" ) ))
( "brides"  n ( b r ai1 d z ) ((pos "Kj%" ) ))
( "bridesmaid"  n ( b r ai1 d z m ei d ) ((pos "K6%" ) ))
( "bridesmaids"  n ( b r ai1 d z m ei d z ) ((pos "Kj%" ) ))
( "bridge"  v ( b r i1 jh ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "bridged"  v ( b r i1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bridgehead"  n ( b r i1 jh h e d ) ((pos "K6%" ) ))
( "bridgeheads"  n ( b r i1 jh h e d z ) ((pos "Kj%" ) ))
( "bridges"  v ( b r i1 jh i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "bridging"  v ( b r i1 jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bridle"  v ( b r ai1 d l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "bridle-path"  n ( b r ai1 d l - p aa th ) ((pos "K6%" ) ))
( "bridle-paths"  n ( b r ai1 d l - p aa dh z ) ((pos "Kj%" ) ))
( "bridle-road"  n ( b r ai1 d l - r ou d ) ((pos "K6%" ) ))
( "bridle-roads"  n ( b r ai1 d l - r ou d z ) ((pos "Kj%" ) ))
( "bridled"  v ( b r ai1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "bridles"  v ( b r ai1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "bridling"  v ( b r ai1 d l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "brief"  v ( b r ii1 f ) ((pos "H0%" "K6%" "OC%" ) (subcat )))
( "briefcase"  n ( b r ii1 f k ei s ) ((pos "K6%" ) ))
( "briefcases"  n ( b r ii1 f k ei s i z ) ((pos "Kj%" ) ))
( "briefed"  v ( b r ii1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "briefer"  j ( b r ii1 f @ r ) ((pos "Or%" ) ))
( "briefest"  j ( b r ii1 f i s t ) ((pos "Os%" ) ))
( "briefing"  v ( b r ii1 f i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "briefings"  n ( b r ii1 f i ng z ) ((pos "Kj%" ) ))
( "briefly"  a ( b r ii1 f l ii ) ((pos "Pu%" ) ))
( "briefs"  v ( b r ii1 f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "brier"  n ( b r ai1 @ r ) ((pos "K6$" ) ))
( "briers"  n ( b r ai1 @ z ) ((pos "Kj$" ) ))
( "brig"  n ( b r i1 g ) ((pos "K6$" ) ))
( "brigade"  n ( b r i1 g ei1 d ) ((pos "K6%" ) ))
( "brigades"  n ( b r i1 g ei1 d z ) ((pos "Kj%" ) ))
( "brigand"  n ( b r i1 g @ n d ) ((pos "K6%" ) ))
( "brigands"  n ( b r i1 g @ n d z ) ((pos "Kj%" ) ))
( "brigantine"  n ( b r i1 g @ n t ii n ) ((pos "K6$" ) ))
( "brigantines"  n ( b r i1 g @ n t ii n z ) ((pos "Kj$" ) ))
( "bright"  j ( b r ai1 t ) ((pos "OC%" "Pu%" ) ))
( "brighten"  v ( b r ai1 t n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "brightened"  v ( b r ai1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "brightening"  v ( b r ai1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "brightens"  v ( b r ai1 t n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "brighter"  j ( b r ai1 t @ r ) ((pos "Or%" ) ))
( "brightest"  j ( b r ai1 t i s t ) ((pos "Os%" ) ))
( "brightly"  a ( b r ai1 t l ii ) ((pos "Pu%" ) ))
( "brightness"  n ( b r ai1 t n @ s ) ((pos "L@%" ) ))
( "brigs"  n ( b r i1 g z ) ((pos "Kj$" ) ))
( "brill"  n ( b r i1 l ) ((pos "K9$" ) ))
( "brilliance"  n ( b r i1 l i@ n s ) ((pos "L@%" ) ))
( "brilliancy"  n ( b r i1 l i@ n s ii ) ((pos "L@$" ) ))
( "brilliant"  j ( b r i1 l i@ n t ) ((pos "OA%" ) ))
( "brilliantine"  n ( b r i1 l i@ n t ii n ) ((pos "L@$" ) ))
( "brilliantly"  a ( b r i1 l i@ n t l ii ) ((pos "Pu%" ) ))
( "brim"  v ( b r i1 m ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "brimful"  j ( b r i2 m f u1 l ) ((pos "OA%" ) ))
( "brimfull"  j ( b r i2 m f u1 l ) ((pos "OA$" ) ))
( "brimmed"  v ( b r i1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "brimming"  v ( b r i1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "brims"  v ( b r i1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "brimstone"  n ( b r i1 m s t ou n ) ((pos "L@%" ) ))
( "brindled"  j ( b r i1 n d l d ) ((pos "OA$" ) ))
( "brine"  n ( b r ai1 n ) ((pos "M6%" ) ))
( "brines"  n ( b r ai1 n z ) ((pos "Mj$" ) ))
( "bring"  v ( b r i1 ng ) ((pos "H5%" ) (subcat "6A" "12A" "12C" "13A" "14" "15B" "17" )))
( "bringing"  v ( b r i1 ng i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "12C" "13A" "14" "15B" "17" )))
( "brings"  v ( b r i1 ng z ) ((pos "Ha%" ) (subcat "6A" "12A" "12C" "13A" "14" "15B" "17" )))
( "brinier"  j ( b r ai1 n i@ r ) ((pos "Or$" ) ))
( "briniest"  j ( b r ai1 n i i s t ) ((pos "Os$" ) ))
( "brink"  n ( b r i1 ng k ) ((pos "K6%" ) ))
( "brinkmanship"  n ( b r i1 ng k m @ n sh i p ) ((pos "L@%" ) ))
( "brinks"  n ( b r i1 ng k s ) ((pos "Kj%" ) ))
( "briny"  j ( b r ai1 n ii ) ((pos "OD%" ) ))
( "brioche"  n ( b r ii1 o1 sh ) ((pos "K6%" ) ))
( "brioches"  n ( b r ii1 o1 sh i z ) ((pos "Kj$" ) ))
( "briquet"  n ( b r i1 k e1 t ) ((pos "K6$" ) ))
( "briquets"  n ( b r i1 k e1 t s ) ((pos "Kj$" ) ))
( "briquette"  n ( b r i1 k e1 t ) ((pos "K6$" ) ))
( "briquettes"  n ( b r i1 k e1 t s ) ((pos "Kj$" ) ))
( "brisk"  j ( b r i1 s k ) ((pos "OC%" ) ))
( "brisker"  j ( b r i1 s k @ r ) ((pos "Or%" ) ))
( "briskest"  j ( b r i1 s k i s t ) ((pos "Os%" ) ))
( "brisket"  n ( b r i1 s k i t ) ((pos "L@%" ) ))
( "briskly"  a ( b r i1 s k l ii ) ((pos "Pu%" ) ))
( "bristle"  v ( b r i1 s l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "bristled"  v ( b r i1 s l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "bristles"  v ( b r i1 s l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "bristlier"  j ( b r i1 s l i@ r ) ((pos "Or%" ) ))
( "bristliest"  j ( b r i1 s l i i s t ) ((pos "Os$" ) ))
( "bristling"  v ( b r i1 s l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "bristly"  j ( b r i1 s l ii ) ((pos "OD%" ) ))
( "brittle"  j ( b r i1 t l ) ((pos "OA%" ) ))
( "broach"  v ( b r ou1 ch ) ((pos "J1%" ) (subcat "2C" "6A" )))
( "broached"  v ( b r ou1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "broaches"  v ( b r ou1 ch i z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "broaching"  v ( b r ou1 ch i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "broad"  n ( b r oo1 d ) ((pos "K6%" "OC%" ) ))
( "broad-minded"  j ( b r oo1 d - m ai1 n d i d ) ((pos "OA%" ) ))
( "broad-mindedness"  n ( b r oo1 d - m ai1 n d i d n i s ) ((pos "L@%" ) ))
( "broadcast"  v ( b r oo1 d k aa s t ) ((pos "J0%" "Jc%" "Jd%" "K6%" "Pu%" ) (subcat "2A" )))
( "broadcasted"  v ( b r oo1 d k aa s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "broadcasting"  v ( b r oo1 d k aa s t i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" )))
( "broadcasts"  v ( b r oo1 d k aa s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "broadcloth"  n ( b r oo1 d k l o th ) ((pos "L@$" ) ))
( "broaden"  v ( b r oo1 d n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "broadened"  v ( b r oo1 d n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "broadening"  v ( b r oo1 d n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "broadens"  v ( b r oo1 d n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "broader"  j ( b r oo1 d @ r ) ((pos "Or%" ) ))
( "broadest"  j ( b r oo1 d i s t ) ((pos "Os%" ) ))
( "broadly"  a ( b r oo1 d l ii ) ((pos "Pu%" ) ))
( "broadness"  n ( b r oo1 d n @ s ) ((pos "L@%" ) ))
( "broads"  n ( b r oo1 d z ) ((pos "Kj%" ) ))
( "broadsheet"  n ( b r oo1 d sh ii t ) ((pos "K6%" ) ))
( "broadsheets"  n ( b r oo1 d sh ii t s ) ((pos "Kj%" ) ))
( "broadside"  n ( b r oo1 d s ai d ) ((pos "K6%" ) ))
( "broadsides"  n ( b r oo1 d s ai d z ) ((pos "Kj%" ) ))
( "broadways"  a ( b r oo1 d w ei z ) ((pos "Pu$" ) ))
( "broadwise"  a ( b r oo1 d w ai z ) ((pos "Pu$" ) ))
( "brocade"  v ( b r @1 k ei1 d ) ((pos "H2$" "M6$" ) (subcat )))
( "brocaded"  v ( b r @1 k ei1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "brocades"  v ( b r @1 k ei1 d z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "brocading"  v ( b r @1 k ei1 d i ng ) ((pos "Hb$" ) (subcat )))
( "broccoli"  n ( b r o1 k @ l ii ) ((pos "L@%" ) ))
( "brochure"  n ( b r ou1 sh @ r ) ((pos "K6%" ) ))
( "brochures"  n ( b r ou1 sh @ z ) ((pos "Kj%" ) ))
( "brogue"  n ( b r ou1 g ) ((pos "K6%" ) ))
( "brogues"  n ( b r ou1 g z ) ((pos "Kj%" ) ))
( "broil"  v ( b r oi1 l ) ((pos "J0%" ) (subcat "2A" )))
( "broiled"  v ( b r oi1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "broiler"  n ( b r oi1 l @ r ) ((pos "K6%" ) ))
( "broilers"  n ( b r oi1 l @ z ) ((pos "Kj%" ) ))
( "broiling"  v ( b r oi1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "broils"  v ( b r oi1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "broke"  v ( b r ou1 k ) ((pos "Jc%" "Op%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "broken"  v ( b r ou1 k @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15A" "15B" )))
( "broken-hearted"  j ( b r ou2 k @ n - h aa1 t i d ) ((pos "OA%" ) ))
( "broker"  n ( b r ou1 k @ r ) ((pos "K6%" ) ))
( "brokerage"  n ( b r ou1 k @ r i jh ) ((pos "L@%" ) ))
( "brokers"  n ( b r ou1 k @ z ) ((pos "Kj%" ) ))
( "brollies"  n ( b r o1 l i z ) ((pos "Kj%" ) ))
( "brolly"  n ( b r o1 l ii ) ((pos "K8%" ) ))
( "bromide"  n ( b r ou1 m ai d ) ((pos "M6%" ) ))
( "bromides"  n ( b r ou1 m ai d z ) ((pos "Mj%" ) ))
( "bromine"  n ( b r ou1 m ii n ) ((pos "L@$" ) ))
( "bronchi"  n ( b r o1 ng k ai ) ((pos "Kj$" ) ))
( "bronchial"  j ( b r o1 ng k i@ l ) ((pos "OA%" ) ))
( "bronchitic"  j ( b r o1 ng k i1 t i k ) ((pos "OA%" ) ))
( "bronchitis"  n ( b r o1 ng k ai1 t i s ) ((pos "L@%" ) ))
( "bronchus"  n ( b r o1 ng k @ s ) ((pos "Ki$" ) ))
( "bronco"  n ( b r o1 ng k ou ) ((pos "K6%" ) ))
( "broncos"  n ( b r o1 ng k ou z ) ((pos "Kj$" ) ))
( "bronze"  v ( b r o1 n z ) ((pos "J2$" "M6%" ) (subcat "2A" )))
( "bronzed"  v ( b r o1 n z d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "bronzes"  v ( b r o1 n z i z ) ((pos "Ja$" "Mj%" ) (subcat "2A" )))
( "bronzing"  v ( b r o1 n z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "brooch"  n ( b r ou1 ch ) ((pos "K7%" ) ))
( "brooches"  n ( b r ou1 ch i z ) ((pos "Kj%" ) ))
( "brood"  v ( b r uu1 d ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "brood-hen"  n ( b r uu1 d - h e n ) ((pos "K6$" ) ))
( "brood-hens"  n ( b r uu1 d - h e n z ) ((pos "Kj$" ) ))
( "brood-mare"  n ( b r uu1 d - m e@ r ) ((pos "K6%" ) ))
( "brood-mares"  n ( b r uu1 d - m e@ z ) ((pos "Kj%" ) ))
( "brooded"  v ( b r uu1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "broodier"  j ( b r uu1 d i@ r ) ((pos "Or$" ) ))
( "broodiest"  j ( b r uu1 d i i s t ) ((pos "Os$" ) ))
( "brooding"  v ( b r uu1 d i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "broods"  v ( b r uu1 d z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "broody"  j ( b r uu1 d ii ) ((pos "OD%" ) ))
( "brook"  v ( b r u1 k ) ((pos "H0$" "K6%" ) (subcat "6A" )))
( "brooked"  v ( b r u1 k t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "brooking"  v ( b r u1 k i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "brooks"  v ( b r u1 k s ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "broom"  n ( b r uu1 m ) ((pos "M6%" ) ))
( "brooms"  n ( b r uu1 m z ) ((pos "Mj%" ) ))
( "broomstick"  n ( b r uu1 m s t i k ) ((pos "K6%" ) ))
( "broomsticks"  n ( b r uu1 m s t i k s ) ((pos "Kj%" ) ))
( "broth"  n ( b r o1 th ) ((pos "L@%" ) ))
( "brothel"  n ( b r o1 th l ) ((pos "K6%" ) ))
( "brothels"  n ( b r o1 th l z ) ((pos "Kj%" ) ))
( "brother"  n ( b r uh1 dh @ r ) ((pos "K6%" ) ))
( "brother-in-law"  n ( b r uh1 dh @ r - i n - l oo ) ((pos "Ki%" ) ))
( "brotherhood"  n ( b r uh1 dh @ h u d ) ((pos "M6%" ) ))
( "brotherhoods"  n ( b r uh1 dh @ h u d z ) ((pos "Mj$" ) ))
( "brotherly"  j ( b r uh1 dh @ l ii ) ((pos "OA%" ) ))
( "brothers"  n ( b r uh1 dh @ z ) ((pos "Kj%" ) ))
( "brothers-in-law"  n ( b r uh1 dh @ z - i n - l oo ) ((pos "Kj%" ) ))
( "brougham"  n ( b r uu1 @ m ) ((pos "K6$" ) ))
( "broughams"  n ( b r uu1 @ m z ) ((pos "Kj$" ) ))
( "brought"  v ( b r oo1 t ) ((pos "Hc*" "Hd*" ) (subcat "6A" "12A" "12C" "13A" "14" "15B" "17" )))
( "brouhaha"  n ( b r uu1 h aa h aa ) ((pos "K6$" ) ))
( "brouhahas"  n ( b r uu1 h aa h aa z ) ((pos "Kj$" ) ))
( "brow"  n ( b r au1 ) ((pos "K6%" ) ))
( "browbeat"  v ( b r au1 b ii t ) ((pos "H5%" "Hc%" ) (subcat "6A" )))
( "browbeaten"  v ( b r au1 b ii t n ) ((pos "Hd%" ) (subcat "6A" )))
( "browbeating"  v ( b r au1 b ii t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "browbeats"  v ( b r au1 b ii t s ) ((pos "Ha%" ) (subcat "6A" )))
( "brown"  v ( b r au1 n ) ((pos "J0%" "OC%" ) (subcat "2A" )))
( "browned"  v ( b r au1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "browner"  j ( b r au1 n @ r ) ((pos "Or%" ) ))
( "brownest"  j ( b r au1 n i s t ) ((pos "Os%" ) ))
( "brownie"  n ( b r au1 n ii ) ((pos "K6%" ) ))
( "brownies"  n ( b r au1 n i z ) ((pos "Kj%" ) ))
( "browning"  v ( b r au1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "brownish"  j ( b r au1 n i sh ) ((pos "OA%" ) ))
( "browns"  v ( b r au1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "brownstone"  n ( b r au1 n s t ou n ) ((pos "M6$" ) ))
( "brownstones"  n ( b r au1 n s t ou n z ) ((pos "Mj$" ) ))
( "brows"  n ( b r au1 z ) ((pos "Kj%" ) ))
( "browse"  v ( b r au1 z ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "browsed"  v ( b r au1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "browses"  v ( b r au1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "browsing"  v ( b r au1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "bruin"  n ( b r uu1 i n ) ((pos "K6$" ) ))
( "bruins"  n ( b r uu1 i n z ) ((pos "Kj$" ) ))
( "bruise"  v ( b r uu1 z ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "bruised"  v ( b r uu1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bruiser"  n ( b r uu1 z @ r ) ((pos "K6%" ) ))
( "bruisers"  n ( b r uu1 z @ z ) ((pos "Kj%" ) ))
( "bruises"  v ( b r uu1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "bruising"  v ( b r uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bruit"  v ( b r uu1 t ) ((pos "H0$" ) (subcat )))
( "bruited"  v ( b r uu1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "bruiting"  v ( b r uu1 t i ng ) ((pos "Hb$" ) (subcat )))
( "bruits"  v ( b r uu1 t s ) ((pos "Ha$" ) (subcat )))
( "brunch"  n ( b r uh1 n ch ) ((pos "K7%" ) ))
( "brunches"  n ( b r uh1 n ch i z ) ((pos "Kj$" ) ))
( "brunette"  n ( b r uu1 n e1 t ) ((pos "K6%" ) ))
( "brunettes"  n ( b r uu1 n e1 t s ) ((pos "Kj%" ) ))
( "brunt"  n ( b r uh1 n t ) ((pos "K6%" ) ))
( "brunts"  n ( b r uh1 n t s ) ((pos "Kj$" ) ))
( "brush"  v ( b r uh1 sh ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "6A" "15B" )))
( "brush-off"  n ( b r uh1 sh - o f ) ((pos "K6%" ) ))
( "brush-offs"  n ( b r uh1 sh - o f s ) ((pos "Kj$" ) ))
( "brush-up"  n ( b r uh1 sh - uh p ) ((pos "K6%" ) ))
( "brush-ups"  n ( b r uh1 sh - uh p s ) ((pos "Kj%" ) ))
( "brushed"  v ( b r uh1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15B" )))
( "brushes"  v ( b r uh1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15B" )))
( "brushing"  v ( b r uh1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15B" )))
( "brushwood"  n ( b r uh1 sh w u d ) ((pos "L@%" ) ))
( "brushwork"  n ( b r uh1 sh w @@ k ) ((pos "K6%" ) ))
( "brushworks"  n ( b r uh1 sh w @@ k s ) ((pos "Kj$" ) ))
( "brusque"  j ( b r uu1 s k ) ((pos "OA%" ) ))
( "brusquely"  a ( b r uu1 s k l ii ) ((pos "Pu%" ) ))
( "brusqueness"  n ( b r uu1 s k n @ s ) ((pos "L@%" ) ))
( "brutal"  j ( b r uu1 t l ) ((pos "OA%" ) ))
( "brutalities"  n ( b r uu1 t a1 l i t i z ) ((pos "Mj$" ) ))
( "brutality"  n ( b r uu1 t a1 l i t ii ) ((pos "M8%" ) ))
( "brutalize"  v ( b r uu1 t @ l ai z ) ((pos "H2%" ) (subcat )))
( "brutalized"  v ( b r uu1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "brutalizes"  v ( b r uu1 t @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "brutalizing"  v ( b r uu1 t @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "brutally"  a ( b r uu1 t @ l ii ) ((pos "Pu%" ) ))
( "brute"  n ( b r uu1 t ) ((pos "K6%" ) ))
( "brutes"  n ( b r uu1 t s ) ((pos "Kj%" ) ))
( "brutish"  j ( b r uu1 t i sh ) ((pos "OA%" ) ))
( "brutishly"  a ( b r uu1 t i sh l ii ) ((pos "Pu%" ) ))
( "bubble"  v ( b uh1 b l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "bubbled"  v ( b uh1 b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "bubbles"  v ( b uh1 b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "bubblier"  j ( b uh1 b l i@ r ) ((pos "Or%" ) ))
( "bubbliest"  j ( b uh1 b l i i s t ) ((pos "Os%" ) ))
( "bubbling"  v ( b uh1 b l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "bubbly"  n ( b uh1 b l ii ) ((pos "L@%" "OD%" ) ))
( "bubonic"  j ( b y uu1 b o1 n i k ) ((pos "OA$" ) ))
( "buccaneer"  n ( b uh2 k @ n i@1 r ) ((pos "K6%" ) ))
( "buccaneers"  n ( b uh2 k @ n i@1 z ) ((pos "Kj%" ) ))
( "buck"  v ( b uh1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "bucked"  v ( b uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "bucket"  v ( b uh1 k i t ) ((pos "I0%" "K6%" ) ))
( "bucketed"  v ( b uh1 k i t i d ) ((pos "Ic%" "Id%" ) ))
( "bucketful"  n ( b uh1 k i t f u l ) ((pos "K6%" ) ))
( "bucketfuls"  n ( b uh1 k i t f u l z ) ((pos "Kj%" ) ))
( "bucketing"  v ( b uh1 k i t i ng ) ((pos "Ib%" ) ))
( "buckets"  v ( b uh1 k i t s ) ((pos "Ia%" "Kj%" ) ))
( "bucking"  v ( b uh1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "buckle"  v ( b uh1 k l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "buckled"  v ( b uh1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "buckler"  n ( b uh1 k l @ r ) ((pos "K6$" ) ))
( "bucklers"  n ( b uh1 k l @ z ) ((pos "Kj$" ) ))
( "buckles"  v ( b uh1 k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "buckling"  v ( b uh1 k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "buckram"  n ( b uh1 k r @ m ) ((pos "L@$" ) ))
( "bucks"  v ( b uh1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "buckshot"  n ( b uh1 k sh o t ) ((pos "L@%" ) ))
( "buckskin"  n ( b uh1 k s k i n ) ((pos "L@%" ) ))
( "buckteeth"  n ( b uh1 k t ii th ) ((pos "Kj%" ) ))
( "bucktooth"  n ( b uh1 k t uu th ) ((pos "Ki$" ) ))
( "buckwheat"  n ( b uh1 k w ii t ) ((pos "L@%" ) ))
( "bucolic"  j ( b y uu1 k o1 l i k ) ((pos "OA%" ) ))
( "bucolics"  n ( b y uu1 k o1 l i k s ) ((pos "Lk$" ) ))
( "bud"  v ( b uh1 d ) ((pos "I4%" "K6%" ) ))
( "budded"  v ( b uh1 d i d ) ((pos "Ic%" "Id%" ) ))
( "buddies"  n ( b uh1 d i z ) ((pos "Kj%" ) ))
( "budding"  v ( b uh1 d i ng ) ((pos "Ib%" "OA%" ) ))
( "buddy"  n ( b uh1 d ii ) ((pos "K8%" ) ))
( "budge"  v ( b uh1 jh ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "budged"  v ( b uh1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "budgerigar"  n ( b uh1 jh @ r i g aa r ) ((pos "K6%" ) ))
( "budgerigars"  n ( b uh1 jh @ r i g aa z ) ((pos "Kj%" ) ))
( "budges"  v ( b uh1 jh i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "budget"  v ( b uh1 jh @ t ) ((pos "I0%" "K6%" ) (subcat )))
( "budgetary"  j ( b uh1 jh @ t @ r ii ) ((pos "OA%" ) ))
( "budgeted"  v ( b uh1 jh @ t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "budgeting"  v ( b uh1 jh @ t i ng ) ((pos "Ib%" ) (subcat )))
( "budgets"  v ( b uh1 jh @ t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "budgie"  n ( b uh1 jh ii ) ((pos "K6%" ) ))
( "budgies"  n ( b uh1 jh i z ) ((pos "Kj%" ) ))
( "budging"  v ( b uh1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "buds"  v ( b uh1 d z ) ((pos "Ia%" "Kj%" ) ))
( "buff"  v ( b uh1 f ) ((pos "H0$" "M6$" ) ))
( "buffalo"  n ( b uh1 f @ l ou ) ((pos "K9%" ) ))
( "buffalos"  n ( b uh1 f @ l ou z ) ((pos "Kj%" ) ))
( "buffed"  v ( b uh1 f t ) ((pos "Hc$" "Hd$" ) ))
( "buffer"  n ( b uh1 f @ r ) ((pos "K6%" ) ))
( "buffers"  n ( b uh1 f @ z ) ((pos "Kj%" ) ))
( "buffet"  n ( b u1 f ei ) ((pos "K6%" ) ))
( "buffet"  v ( b uh1 f i t ) ((pos "J0%" "K6$" ) (subcat "3A" )))
( "buffeted"  v ( b uh1 f i t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "buffeting"  v ( b uh1 f i t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "buffets"  n ( b u1 f ei z ) ((pos "Kj%" ) ))
( "buffets"  v ( b uh1 f i t s ) ((pos "Ja%" "Kj$" ) (subcat "3A" )))
( "buffing"  v ( b uh1 f i ng ) ((pos "Hb$" ) ))
( "buffoon"  n ( b @1 f uu1 n ) ((pos "K6%" ) ))
( "buffoonery"  n ( b @1 f uu1 n @ r ii ) ((pos "L@%" ) ))
( "buffoons"  n ( b @1 f uu1 n z ) ((pos "Kj%" ) ))
( "buffs"  v ( b uh1 f s ) ((pos "Ha$" "Mj$" ) ))
( "bug"  v ( b uh1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "bug-hunter"  n ( b uh1 g - h uh n t @ r ) ((pos "K6$" ) ))
( "bug-hunters"  n ( b uh1 g - h uh n t @ z ) ((pos "Kj$" ) ))
( "bugaboo"  n ( b uh1 g @ b uu ) ((pos "K6$" ) ))
( "bugaboos"  n ( b uh1 g @ b uu z ) ((pos "Kj$" ) ))
( "bugbear"  n ( b uh1 g b e@ r ) ((pos "K6%" ) ))
( "bugbears"  n ( b uh1 g b e@ z ) ((pos "Kj$" ) ))
( "bugged"  v ( b uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bugger"  v ( b uh1 g @ r ) ((pos "J0%" "K6%" ) (subcat )))
( "bugger-all"  n ( b uh2 g @ r - oo1 l ) ((pos "L@%" ) ))
( "buggered"  v ( b uh1 g @ d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "buggering"  v ( b uh1 g @ r i ng ) ((pos "Jb%" ) (subcat )))
( "buggers"  v ( b uh1 g @ z ) ((pos "Ja%" "Kj%" ) (subcat )))
( "buggery"  n ( b uh1 g @ r ii ) ((pos "L@%" ) ))
( "buggies"  n ( b uh1 g i z ) ((pos "Kj%" ) ))
( "bugging"  v ( b uh1 g i ng ) ((pos "Hb%" ) (subcat )))
( "buggy"  n ( b uh1 g ii ) ((pos "K8%" ) ))
( "bugle"  n ( b y uu1 g l ) ((pos "K6%" ) ))
( "bugler"  n ( b y uu1 g l @ r ) ((pos "K6%" ) ))
( "buglers"  n ( b y uu1 g l @ z ) ((pos "Kj%" ) ))
( "bugles"  n ( b y uu1 g l z ) ((pos "Kj%" ) ))
( "bugs"  v ( b uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "buhl"  n ( b uu1 l ) ((pos "L@$" ) ))
( "build"  v ( b i1 l d ) ((pos "J5%" "L@%" ) (subcat "2C" "3A" "6A" "12B" "13B" "14" )))
( "build-up"  n ( b i1 l d - uh p ) ((pos "K6%" ) ))
( "build-ups"  n ( b i1 l d - uh p s ) ((pos "Kj$" ) ))
( "builder"  n ( b i1 l d @ r ) ((pos "K6%" ) ))
( "builders"  n ( b i1 l d @ z ) ((pos "Kj%" ) ))
( "building"  v ( b i1 l d i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "3A" "6A" "12B" "13B" "14" )))
( "building-societies"  n ( b i1 l d i ng - s @ s ai2 @ t i z ) ((pos "Kj%" ) ))
( "building-society"  n ( b i1 l d i ng - s @ s ai2 @ t ii ) ((pos "K8%" ) ))
( "buildings"  n ( b i1 l d i ng z ) ((pos "Mj%" ) ))
( "builds"  v ( b i1 l d z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "12B" "13B" "14" )))
( "built"  v ( b i1 l t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "12B" "13B" "14" )))
( "built-in"  j ( b i1 l t - i1 n ) ((pos "OA%" ) ))
( "built-up"  j ( b i1 l t - uh1 p ) ((pos "OA%" ) ))
( "bulb"  n ( b uh1 l b ) ((pos "K6%" ) ))
( "bulbous"  j ( b uh1 l b @ s ) ((pos "OA%" ) ))
( "bulbs"  n ( b uh1 l b z ) ((pos "Kj%" ) ))
( "bulbul"  n ( b u1 l b u l ) ((pos "K6$" ) ))
( "bulbuls"  n ( b u1 l b u l z ) ((pos "Kj$" ) ))
( "bulge"  v ( b uh1 l jh ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "bulged"  v ( b uh1 l jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "bulges"  v ( b uh1 l jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "bulging"  v ( b uh1 l jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "bulk"  v ( b uh1 l k ) ((pos "I0%" "L@%" ) ))
( "bulked"  v ( b uh1 l k t ) ((pos "Ic%" "Id%" ) ))
( "bulkhead"  n ( b uh1 l k h e d ) ((pos "K6%" ) ))
( "bulkheads"  n ( b uh1 l k h e d z ) ((pos "Kj%" ) ))
( "bulkier"  j ( b uh1 l k i@ r ) ((pos "Or%" ) ))
( "bulkiest"  j ( b uh1 l k i i s t ) ((pos "Os%" ) ))
( "bulking"  v ( b uh1 l k i ng ) ((pos "Ib%" ) ))
( "bulks"  v ( b uh1 l k s ) ((pos "Ia%" ) ))
( "bulky"  j ( b uh1 l k ii ) ((pos "OD%" ) ))
( "bull"  n ( b u1 l ) ((pos "K6%" ) ))
( "bull's-eye"  n ( b u1 l z - ai ) ((pos "K6%" ) ))
( "bull's-eyes"  n ( b u1 l z - ai z ) ((pos "Kj%" ) ))
( "bull-headed"  j ( b u1 l - h e d i d ) ((pos "OA%" ) ))
( "bull-neck"  n ( b u1 l - n e1 k ) ((pos "K6%" ) ))
( "bull-necks"  n ( b u1 l - n e1 k s ) ((pos "Kj$" ) ))
( "bull-terrier"  n ( b u1 l - t e1 r i@ r ) ((pos "K6%" ) ))
( "bull-terriers"  n ( b u1 l - t e1 r i@ z ) ((pos "Kj%" ) ))
( "bulldog"  n ( b u1 l d o g ) ((pos "K6%" ) ))
( "bulldogs"  n ( b u1 l d o g z ) ((pos "Kj%" ) ))
( "bulldoze"  v ( b u1 l d ou z ) ((pos "H2%" ) (subcat "6A" )))
( "bulldozed"  v ( b u1 l d ou z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bulldozer"  n ( b u1 l d ou z @ r ) ((pos "K6%" ) ))
( "bulldozers"  n ( b u1 l d ou z @ z ) ((pos "Kj%" ) ))
( "bulldozes"  v ( b u1 l d ou z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "bulldozing"  v ( b u1 l d ou z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bullet"  n ( b u1 l i t ) ((pos "K6%" ) ))
( "bullet-headed"  j ( b u2 l i t - h e1 d i d ) ((pos "OA%" ) ))
( "bulletin"  n ( b u1 l @ t i n ) ((pos "K6%" ) ))
( "bulletins"  n ( b u1 l @ t i n z ) ((pos "Kj%" ) ))
( "bulletproof"  j ( b u1 l i t p r uu f ) ((pos "OA%" ) ))
( "bullets"  n ( b u1 l i t s ) ((pos "Kj%" ) ))
( "bullfight"  n ( b u1 l f ai t ) ((pos "K6%" ) ))
( "bullfighter"  n ( b u1 l f ai t @ r ) ((pos "K6%" ) ))
( "bullfighters"  n ( b u1 l f ai t @ z ) ((pos "Kj%" ) ))
( "bullfights"  n ( b u1 l f ai t s ) ((pos "Kj%" ) ))
( "bullfinch"  n ( b u1 l f i n ch ) ((pos "K7%" ) ))
( "bullfinches"  n ( b u1 l f i n ch i z ) ((pos "Kj%" ) ))
( "bullfrog"  n ( b u1 l f r o g ) ((pos "K6%" ) ))
( "bullfrogs"  n ( b u1 l f r o g z ) ((pos "Kj%" ) ))
( "bullied"  v ( b u1 l i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "bullies"  v ( b u1 l i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "bullion"  n ( b u1 l i@ n ) ((pos "L@%" ) ))
( "bullock"  n ( b u1 l @ k ) ((pos "K6%" ) ))
( "bullocks"  n ( b u1 l @ k s ) ((pos "Kj%" ) ))
( "bullring"  n ( b u1 l r i ng ) ((pos "K6%" ) ))
( "bullrings"  n ( b u1 l r i ng z ) ((pos "Kj%" ) ))
( "bulls"  n ( b u1 l z ) ((pos "Kj%" ) ))
( "bullshit"  v ( b u1 l sh i t ) ((pos "I4%" "L@%" ) ))
( "bullshits"  v ( b u1 l sh i t s ) ((pos "Ia%" ) ))
( "bullshitted"  v ( b u1 l sh i t i d ) ((pos "Ic%" "Id%" ) ))
( "bullshitting"  v ( b u1 l sh i t i ng ) ((pos "Ib%" ) ))
( "bully"  v ( b u1 l ii ) ((pos "J3%" "M8%" "OA$" ) (subcat "2C" "6A" )))
( "bullying"  v ( b u1 l i i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "bulrush"  n ( b u1 l r uh sh ) ((pos "K7%" ) ))
( "bulrushes"  n ( b u1 l r uh sh i z ) ((pos "Kj%" ) ))
( "bulwark"  n ( b u1 l w @ k ) ((pos "K6%" ) ))
( "bulwarks"  n ( b u1 l w @ k s ) ((pos "Kj%" ) ))
( "bum"  v ( b uh1 m ) ((pos "J4%" "K6%" "OA%" ) (subcat "2C" "6A" )))
( "bumblebee"  n ( b uh1 m b l b ii ) ((pos "K6%" ) ))
( "bumblebees"  n ( b uh1 m b l b ii z ) ((pos "Kj%" ) ))
( "bumboat"  n ( b uh1 m b ou t ) ((pos "K6$" ) ))
( "bumboats"  n ( b uh1 m b ou t s ) ((pos "Kj$" ) ))
( "bummed"  v ( b uh1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "bumming"  v ( b uh1 m i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "bump"  v ( b uh1 m p ) ((pos "J0%" "K6%" "Pu%" ) (subcat "2C" "3A" "6A" "14" )))
( "bumped"  v ( b uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" )))
( "bumper"  n ( b uh1 m p @ r ) ((pos "K6%" ) ))
( "bumpers"  n ( b uh1 m p @ z ) ((pos "Kj%" ) ))
( "bumpier"  j ( b uh1 m p i@ r ) ((pos "Or%" ) ))
( "bumpiest"  j ( b uh1 m p i i s t ) ((pos "Os%" ) ))
( "bumping"  v ( b uh1 m p i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" )))
( "bumpkin"  n ( b uh1 m p k i n ) ((pos "K6%" ) ))
( "bumpkins"  n ( b uh1 m p k i n z ) ((pos "Kj%" ) ))
( "bumps"  v ( b uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" )))
( "bumptious"  j ( b uh1 m p sh @ s ) ((pos "OA%" ) ))
( "bumptiously"  a ( b uh1 m p sh @ s l ii ) ((pos "Pu%" ) ))
( "bumptiousness"  n ( b uh1 m p sh @ s n @ s ) ((pos "L@%" ) ))
( "bumpy"  j ( b uh1 m p ii ) ((pos "OD%" ) ))
( "bums"  v ( b uh1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "bun"  n ( b uh1 n ) ((pos "K6%" ) ))
( "buna"  n ( b uu1 n @ ) ((pos "L@$" ) ))
( "bunch"  v ( b uh1 n ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" )))
( "bunched"  v ( b uh1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "bunches"  v ( b uh1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "bunching"  v ( b uh1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "bundle"  v ( b uh1 n d l ) ((pos "J2%" "K6%" ) (subcat "2C" "15A" )))
( "bundled"  v ( b uh1 n d l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "15A" )))
( "bundles"  v ( b uh1 n d l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "15A" )))
( "bundling"  v ( b uh1 n d l i ng ) ((pos "Jb%" ) (subcat "2C" "15A" )))
( "bung"  v ( b uh1 ng ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "bung-hole"  n ( b uh1 ng - h ou l ) ((pos "K6%" ) ))
( "bung-holes"  n ( b uh1 ng - h ou l z ) ((pos "Kj%" ) ))
( "bungaloid"  j ( b uh1 ng g @ l oi d ) ((pos "OA$" ) ))
( "bungalow"  n ( b uh1 ng g @ l ou ) ((pos "K6%" ) ))
( "bungalows"  n ( b uh1 ng g @ l ou z ) ((pos "Kj%" ) ))
( "bunged"  v ( b uh1 ng d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "bunging"  v ( b uh1 ng i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "bungle"  v ( b uh1 ng g l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "bungled"  v ( b uh1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "bungler"  n ( b uh1 ng g l @ r ) ((pos "K6%" ) ))
( "bunglers"  n ( b uh1 ng g l @ z ) ((pos "Kj%" ) ))
( "bungles"  v ( b uh1 ng g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "bungling"  v ( b uh1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bungs"  v ( b uh1 ng z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "bunion"  n ( b uh1 n i@ n ) ((pos "K6%" ) ))
( "bunions"  n ( b uh1 n i@ n z ) ((pos "Kj%" ) ))
( "bunk"  v ( b uh1 ng k ) ((pos "I0$" "M6%" ) ))
( "bunked"  v ( b uh1 ng k t ) ((pos "Ic$" "Id$" ) ))
( "bunker"  v ( b uh1 ng k @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "bunkered"  v ( b uh1 ng k @ d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "bunkering"  v ( b uh1 ng k @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "bunkers"  v ( b uh1 ng k @ z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "bunking"  v ( b uh1 ng k i ng ) ((pos "Ib$" ) ))
( "bunks"  v ( b uh1 ng k s ) ((pos "Ia$" "Mj%" ) ))
( "bunkum"  n ( b uh1 ng k @ m ) ((pos "L@%" ) ))
( "bunnies"  n ( b uh1 n i z ) ((pos "Kj%" ) ))
( "bunny"  n ( b uh1 n ii ) ((pos "K8%" ) ))
( "buns"  n ( b uh1 n z ) ((pos "Kj%" ) ))
( "bunting"  n ( b uh1 n t i ng ) ((pos "L@%" ) ))
( "buoy"  v ( b oi1 ) ((pos "I0%" "K6%" ) (subcat "6A" )))
( "buoyancy"  n ( b oi1 @ n s ii ) ((pos "L@%" ) ))
( "buoyant"  j ( b oi1 @ n t ) ((pos "OA%" ) ))
( "buoyantly"  a ( b oi1 @ n t l ii ) ((pos "Pu%" ) ))
( "buoyed"  v ( b oi1 d ) ((pos "Ic%" "Id%" ) (subcat "6A" )))
( "buoying"  v ( b oi1 i ng ) ((pos "Ib%" ) (subcat "6A" )))
( "buoys"  v ( b oi1 z ) ((pos "Ia%" "Kj%" ) (subcat "6A" )))
( "bur"  n ( b @@1 r ) ((pos "K6$" ) ))
( "burble"  v ( b @@1 b l ) ((pos "I2%" ) ))
( "burbled"  v ( b @@1 b l d ) ((pos "Ic%" "Id%" ) ))
( "burbles"  v ( b @@1 b l z ) ((pos "Ia%" ) ))
( "burbling"  v ( b @@1 b l i ng ) ((pos "Ib%" ) ))
( "burden"  v ( b @@1 d n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "burdened"  v ( b @@1 d n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "burdening"  v ( b @@1 d n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "burdens"  v ( b @@1 d n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "burdensome"  j ( b @@1 d n s @ m ) ((pos "OA%" ) ))
( "burdock"  n ( b @@1 d o k ) ((pos "K6%" ) ))
( "burdocks"  n ( b @@1 d o k s ) ((pos "Kj$" ) ))
( "bureau"  n ( b y u@1 r ou ) ((pos "Ki%" ) ))
( "bureaucracies"  n ( b y u@1 r o1 k r @ s i z ) ((pos "Mj%" ) ))
( "bureaucracy"  n ( b y u@1 r o1 k r @ s ii ) ((pos "M8%" ) ))
( "bureaucrat"  n ( b y u@1 r @ k r a t ) ((pos "K6%" ) ))
( "bureaucratic"  j ( b y u@2 r @ k r a1 t i k ) ((pos "OA%" ) ))
( "bureaucratically"  a ( b y u@2 r @ k r a1 t i k l ii ) ((pos "Pu%" ) ))
( "bureaucrats"  n ( b y u@1 r @ k r a t s ) ((pos "Kj%" ) ))
( "bureaux"  n ( b y u@1 r ou z ) ((pos "Kj%" ) ))
( "burette"  n ( b y u@1 r e1 t ) ((pos "K6$" ) ))
( "burettes"  n ( b y u@1 r e1 t s ) ((pos "Kj$" ) ))
( "burg"  n ( b @@1 g ) ((pos "K6$" ) ))
( "burgeon"  v ( b @@1 jh @ n ) ((pos "I0%" ) ))
( "burgeoned"  v ( b @@1 jh @ n d ) ((pos "Ic%" "Id%" ) ))
( "burgeoning"  v ( b @@1 jh @ n i ng ) ((pos "Ib%" ) ))
( "burgeons"  v ( b @@1 jh @ n z ) ((pos "Ia%" ) ))
( "burgess"  n ( b @@1 jh i s ) ((pos "K7$" ) ))
( "burgesses"  n ( b @@1 jh i s i z ) ((pos "Kj$" ) ))
( "burgh"  n ( b uh1 r @ ) ((pos "K6$" ) ))
( "burgher"  n ( b @@1 g @ r ) ((pos "K6%" ) ))
( "burghers"  n ( b @@1 g @ z ) ((pos "Kj%" ) ))
( "burghs"  n ( b uh1 r @ z ) ((pos "Kj$" ) ))
( "burglar"  n ( b @@1 g l @ r ) ((pos "K6%" ) ))
( "burglar-alarm"  n ( b @@1 g l @ r - @ l aa m ) ((pos "K6%" ) ))
( "burglar-alarms"  n ( b @@1 g l @ r - @ l aa m z ) ((pos "Kj%" ) ))
( "burglar-proof"  j ( b @@1 g l @ r - p r uu f ) ((pos "OA%" ) ))
( "burglaries"  n ( b @@1 g l @ r i z ) ((pos "Mj%" ) ))
( "burglarious"  j ( b @@1 g l e@1 r i@ s ) ((pos "OA$" ) ))
( "burglars"  n ( b @@1 g l @ z ) ((pos "Kj%" ) ))
( "burglary"  n ( b @@1 g l @ r ii ) ((pos "M8%" ) ))
( "burgle"  v ( b @@1 g l ) ((pos "J2%" ) (subcat "2A" )))
( "burgled"  v ( b @@1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "burgles"  v ( b @@1 g l z ) ((pos "Ja%" ) (subcat "2A" )))
( "burgling"  v ( b @@1 g l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "burgomaster"  n ( b @@1 g @ m aa s t @ r ) ((pos "K6%" ) ))
( "burgomasters"  n ( b @@1 g @ m aa s t @ z ) ((pos "Kj%" ) ))
( "burgs"  n ( b @@1 g z ) ((pos "Kj$" ) ))
( "burial"  n ( b e1 r i@ l ) ((pos "M6%" ) ))
( "burial-ground"  n ( b e1 r i@ l - g r au n d ) ((pos "K6%" ) ))
( "burial-grounds"  n ( b e1 r i@ l - g r au n d z ) ((pos "Kj%" ) ))
( "burials"  n ( b e1 r i@ l z ) ((pos "Mj%" ) ))
( "buried"  v ( b e1 r i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "buries"  v ( b e1 r i z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "burke"  v ( b @@1 k ) ((pos "H2$" ) (subcat )))
( "burked"  v ( b @@1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "burkes"  v ( b @@1 k s ) ((pos "Ha$" ) (subcat )))
( "burking"  v ( b @@1 k i ng ) ((pos "Hb$" ) (subcat )))
( "burlap"  n ( b @@1 l a p ) ((pos "L@$" ) ))
( "burlesque"  v ( b @@1 l e1 s k ) ((pos "H2$" "M6%" ) (subcat )))
( "burlesqued"  v ( b @@1 l e1 s k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "burlesques"  v ( b @@1 l e1 s k s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "burlesquing"  v ( b @@1 l e1 s k i ng ) ((pos "Hb$" ) (subcat )))
( "burlier"  j ( b @@1 l i@ r ) ((pos "Or%" ) ))
( "burliest"  j ( b @@1 l i i s t ) ((pos "Os%" ) ))
( "burly"  j ( b @@1 l ii ) ((pos "OD%" ) ))
( "burn"  v ( b @@1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "4A" "6A" "14" )))
( "burn-up"  n ( b @@1 n - uh p ) ((pos "K6%" ) ))
( "burn-ups"  n ( b @@1 n - uh p s ) ((pos "Kj$" ) ))
( "burned"  v ( b @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "4A" "6A" "14" )))
( "burner"  n ( b @@1 n @ r ) ((pos "K6%" ) ))
( "burners"  n ( b @@1 n @ z ) ((pos "Kj%" ) ))
( "burning"  v ( b @@1 n i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "4A" "6A" "14" )))
( "burnish"  v ( b @@1 n i sh ) ((pos "J1%" ) (subcat "2A" )))
( "burnished"  v ( b @@1 n i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "burnishes"  v ( b @@1 n i sh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "burnishing"  v ( b @@1 n i sh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "burnouse"  n ( b @@1 n uu1 s ) ((pos "K6$" ) ))
( "burnouses"  n ( b @@1 n uu1 s i z ) ((pos "Kj$" ) ))
( "burns"  v ( b @@1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "4A" "6A" "14" )))
( "burnt"  v ( b @@1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "4A" "6A" "14" )))
( "burp"  v ( b @@1 p ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "burped"  v ( b @@1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "burping"  v ( b @@1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "burps"  v ( b @@1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "burr"  n ( b @@1 r ) ((pos "K6$" ) ))
( "burr-drill"  n ( b @@1 - d r i l ) ((pos "K6$" ) ))
( "burr-drills"  n ( b @@1 - d r i l z ) ((pos "Kj$" ) ))
( "burrow"  v ( b uh1 r ou ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "burrowed"  v ( b uh1 r ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "burrowing"  v ( b uh1 r ou i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "burrows"  v ( b uh1 r ou z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "burrs"  n ( b @@1 z ) ((pos "Kj$" ) ))
( "burs"  n ( b @@1 z ) ((pos "Kj$" ) ))
( "bursar"  n ( b @@1 s @ r ) ((pos "K6%" ) ))
( "bursaries"  n ( b @@1 s @ r i z ) ((pos "Kj%" ) ))
( "bursars"  n ( b @@1 s @ z ) ((pos "Kj%" ) ))
( "bursary"  n ( b @@1 s @ r ii ) ((pos "K8%" ) ))
( "burst"  v ( b @@1 s t ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bursting"  v ( b @@1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "bursts"  v ( b @@1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "burthen"  v ( b @@1 dh @ n ) ((pos "H0$" "K6$" ) ))
( "burthened"  v ( b @@1 dh @ n d ) ((pos "Hc$" "Hd$" ) ))
( "burthening"  v ( b @@1 dh @ n i ng ) ((pos "Hb$" ) ))
( "burthens"  v ( b @@1 dh @ n z ) ((pos "Ha$" "Kj$" ) ))
( "burton"  n ( b @@1 t n ) ((pos "K6$" ) ))
( "burtons"  n ( b @@1 t n z ) ((pos "Kj$" ) ))
( "bury"  v ( b e1 r ii ) ((pos "H3%" ) (subcat "6A" "15A" )))
( "burying"  v ( b e1 r i i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "burying-ground"  n ( b e1 r i i ng - g r au n d ) ((pos "K6%" ) ))
( "burying-grounds"  n ( b e1 r i i ng - g r au n d z ) ((pos "Kj%" ) ))
( "bus"  v ( b uh1 s ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "busbies"  n ( b uh1 z b i z ) ((pos "Kj%" ) ))
( "busby"  n ( b uh1 z b ii ) ((pos "K8%" ) ))
( "bused"  v ( b uh1 s t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "buses"  v ( b uh1 s i z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "bush"  n ( b u1 sh ) ((pos "M7%" ) ))
( "bushel"  n ( b u1 sh l ) ((pos "K6%" ) ))
( "bushels"  n ( b u1 sh l z ) ((pos "Kj%" ) ))
( "bushes"  n ( b u1 sh i z ) ((pos "Mj%" ) ))
( "bushy"  j ( b u1 sh ii ) ((pos "OA%" ) ))
( "busied"  v ( b i1 z i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "busier"  j ( b i1 z i@ r ) ((pos "Or%" ) ))
( "busies"  v ( b i1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "busiest"  j ( b i1 z i i s t ) ((pos "Os%" ) ))
( "busily"  a ( b i1 z i l ii ) ((pos "Pu%" ) ))
( "business"  n ( b i1 z n @ s ) ((pos "M7*" ) ))
( "businesses"  n ( b i1 z n @ s i z ) ((pos "Mj%" ) ))
( "businesslike"  j ( b i1 z n @ s l ai k ) ((pos "OA%" ) ))
( "businessman"  n ( b i1 z n @ s m a n ) ((pos "Ki%" ) ))
( "businessmen"  n ( b i1 z n @ s m e n ) ((pos "Kj%" ) ))
( "busing"  v ( b uh1 s i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "busker"  n ( b uh1 s k @ r ) ((pos "K6%" ) ))
( "buskers"  n ( b uh1 s k @ z ) ((pos "Kj%" ) ))
( "busman"  n ( b uh1 s m @ n ) ((pos "Ki%" ) ))
( "busmen"  n ( b uh1 s m @ n ) ((pos "Kj%" ) ))
( "bussing"  v ( b uh1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "bust"  v ( b uh1 s t ) ((pos "J0%" "Jc%" "Jd%" "K6%" ) ))
( "bust-up"  n ( b uh1 s t - uh p ) ((pos "K6%" ) ))
( "bust-ups"  n ( b uh1 s t - uh p s ) ((pos "Kj$" ) ))
( "bustard"  n ( b uh1 s t @ d ) ((pos "K6$" ) ))
( "bustards"  n ( b uh1 s t @ d z ) ((pos "Kj$" ) ))
( "busted"  v ( b uh1 s t i d ) ((pos "Jc%" "Jd%" ) ))
( "buster"  n ( b uh1 s t @ r ) ((pos "K6%" ) ))
( "busters"  n ( b uh1 s t @ z ) ((pos "Kj$" ) ))
( "busting"  v ( b uh1 s t i ng ) ((pos "Jb%" ) ))
( "bustle"  v ( b uh1 s l ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" )))
( "bustled"  v ( b uh1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "bustles"  v ( b uh1 s l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "bustling"  v ( b uh1 s l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "busts"  v ( b uh1 s t s ) ((pos "Ja%" "Kj%" ) ))
( "busy"  v ( b i1 z ii ) ((pos "H3%" "OD%" ) (subcat "6A" )))
( "busybodies"  n ( b i1 z i b o d i z ) ((pos "Kj%" ) ))
( "busybody"  n ( b i1 z i b o d ii ) ((pos "K8%" ) ))
( "busying"  v ( b i1 z i i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "but"  a ( b uh t ) ((pos "Pu*" "Qz$" "T-*" "V-*" ) ))
( "butane"  n ( b y uu1 t ei n ) ((pos "L@%" ) ))
( "butch"  j ( b u1 ch ) ((pos "OA%" ) ))
( "butcher"  v ( b u1 ch @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "butchered"  v ( b u1 ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "butchering"  v ( b u1 ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "butchers"  v ( b u1 ch @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "butchery"  n ( b u1 ch @ r ii ) ((pos "L@%" ) ))
( "butler"  n ( b uh1 t l @ r ) ((pos "K6%" ) ))
( "butlers"  n ( b uh1 t l @ z ) ((pos "Kj%" ) ))
( "butt"  v ( b uh1 t ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "butted"  v ( b uh1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "butter"  v ( b uh1 t @ r ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "butterbean"  n ( b uh1 t @ b ii n ) ((pos "K6%" ) ))
( "butterbeans"  n ( b uh1 t @ b ii n z ) ((pos "Kj%" ) ))
( "buttercup"  n ( b uh1 t @ k uh p ) ((pos "K6%" ) ))
( "buttercups"  n ( b uh1 t @ k uh p s ) ((pos "Kj%" ) ))
( "buttered"  v ( b uh1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "butterfingers"  n ( b uh1 t @ f i ng g @ z ) ((pos "Ki%" ) ))
( "butterflies"  n ( b uh1 t @ f l ai z ) ((pos "Kj%" ) ))
( "butterfly"  n ( b uh1 t @ f l ai ) ((pos "K8%" ) ))
( "butteries"  n ( b uh1 t @ r i z ) ((pos "Kj$" ) ))
( "buttering"  v ( b uh1 t @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "buttermilk"  n ( b uh1 t @ m i l k ) ((pos "L@%" ) ))
( "butters"  v ( b uh1 t @ z ) ((pos "Ha%" ) (subcat "6A" )))
( "butterscotch"  n ( b uh1 t @ s k o ch ) ((pos "L@%" ) ))
( "buttery"  n ( b uh1 t @ r ii ) ((pos "K8%" ) ))
( "butting"  v ( b uh1 t i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "buttock"  n ( b uh1 t @ k ) ((pos "K6%" ) ))
( "buttocks"  n ( b uh1 t @ k s ) ((pos "Kj%" ) ))
( "button"  v ( b uh1 t n ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "buttoned"  v ( b uh1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "buttoned-up"  j ( b uh2 t n d - uh1 p ) ((pos "OA%" ) ))
( "buttonhole"  v ( b uh1 t n h ou l ) ((pos "H2%" "K6%" ) (subcat )))
( "buttonholed"  v ( b uh1 t n h ou l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "buttonholes"  v ( b uh1 t n h ou l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "buttonholing"  v ( b uh1 t n h ou l i ng ) ((pos "Hb%" ) (subcat )))
( "buttonhook"  n ( b uh1 t n h u k ) ((pos "K6$" ) ))
( "buttonhooks"  n ( b uh1 t n h u k s ) ((pos "Kj$" ) ))
( "buttoning"  v ( b uh1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "buttons"  v ( b uh1 t n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "buttonwood"  n ( b uh1 t n w u d ) ((pos "M6$" ) ))
( "buttonwoods"  n ( b uh1 t n w u d z ) ((pos "Mj$" ) ))
( "buttress"  v ( b uh1 t r @ s ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "buttressed"  v ( b uh1 t r @ s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "buttresses"  v ( b uh1 t r @ s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "buttressing"  v ( b uh1 t r @ s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "butts"  v ( b uh1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "buxom"  j ( b uh1 k s @ m ) ((pos "OA%" ) ))
( "buy"  v ( b ai1 ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "6A" "12B" "13B" )))
( "buyer"  n ( b ai1 @ r ) ((pos "K6%" ) ))
( "buyers"  n ( b ai1 @ z ) ((pos "Kj%" ) ))
( "buying"  v ( b ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "12B" "13B" )))
( "buys"  v ( b ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "12B" "13B" )))
( "buzz"  v ( b uh1 z ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" )))
( "buzzard"  n ( b uh1 z @ d ) ((pos "K6%" ) ))
( "buzzards"  n ( b uh1 z @ d z ) ((pos "Kj%" ) ))
( "buzzed"  v ( b uh1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "buzzer"  n ( b uh1 z @ r ) ((pos "K6%" ) ))
( "buzzers"  n ( b uh1 z @ z ) ((pos "Kj%" ) ))
( "buzzes"  v ( b uh1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "buzzing"  v ( b uh1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "by"  a ( b ai ) ((pos "P+*" "T-*" ) ))
( "by-election"  n ( b ai1 - i l e k sh @ n ) ((pos "K6%" ) ))
( "by-elections"  n ( b ai1 - i l e k sh @ n z ) ((pos "Kj%" ) ))
( "bye"  n ( b ai1 ) ((pos "K6%" ) ))
( "bye-bye"  n ( b ai1 - b ai ) ((pos "K6%" ) ))
( "bye-bye"  uh ( b ai2 - b ai1 ) ((pos "W-%" ) ))
( "bye-byes"  n ( b ai1 - b ai z ) ((pos "Kj%" ) ))
( "bye-law"  n ( b ai1 - l oo ) ((pos "K6%" ) ))
( "bye-laws"  n ( b ai1 - l oo z ) ((pos "Kj%" ) ))
( "byes"  n ( b ai1 z ) ((pos "Kj%" ) ))
( "bygone"  j ( b ai1 g o n ) ((pos "OA%" ) ))
( "bygones"  n ( b ai1 g o n z ) ((pos "Kj%" ) ))
( "bylaw"  n ( b ai1 l oo ) ((pos "K6%" ) ))
( "bylaws"  n ( b ai1 l oo z ) ((pos "Kj%" ) ))
( "bypass"  v ( b ai1 p aa s ) ((pos "H1%" "K7%" ) (subcat )))
( "bypassed"  v ( b ai1 p aa s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "bypasses"  v ( b ai1 p aa s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "bypassing"  v ( b ai1 p aa s i ng ) ((pos "Hb%" ) (subcat )))
( "bypath"  n ( b ai1 p aa th ) ((pos "K6$" ) ))
( "bypaths"  n ( b ai1 p aa dh z ) ((pos "Kj$" ) ))
( "byplay"  n ( b ai1 p l ei ) ((pos "L@%" ) ))
( "byproduct"  n ( b ai1 p r o d uh k t ) ((pos "K6%" ) ))
( "byproducts"  n ( b ai1 p r o d uh k t s ) ((pos "Kj%" ) ))
( "byroad"  n ( b ai1 r ou d ) ((pos "K6%" ) ))
( "byroads"  n ( b ai1 r ou d z ) ((pos "Kj%" ) ))
( "bystander"  n ( b ai1 s t a n d @ r ) ((pos "K6%" ) ))
( "bystanders"  n ( b ai1 s t a n d @ z ) ((pos "Kj%" ) ))
( "byway"  n ( b ai1 w ei ) ((pos "K6%" ) ))
( "byways"  n ( b ai1 w ei z ) ((pos "Kj%" ) ))
( "byword"  n ( b ai1 w @@ d ) ((pos "K6%" ) ))
( "bywords"  n ( b ai1 w @@ d z ) ((pos "Kj%" ) ))
( "c"  n ( s ii1 ) ((pos "Ki$" ) ))
( "c's"  n ( s ii1 z ) ((pos "Kj$" ) ))
( "ca'canny"  n ( k aa2 k a1 n ii ) ((pos "Ki$" ) ))
( "cab"  n ( k a1 b ) ((pos "K6%" ) ))
( "cab-rank"  n ( k a1 b - r a ng k ) ((pos "K6%" ) ))
( "cab-ranks"  n ( k a1 b - r a ng k s ) ((pos "Kj%" ) ))
( "cabal"  n ( k @1 b a1 l ) ((pos "K6$" ) ))
( "cabals"  n ( k @1 b a1 l z ) ((pos "Kj$" ) ))
( "cabaret"  n ( k a1 b @ r ei ) ((pos "K6%" ) ))
( "cabarets"  n ( k a1 b @ r ei z ) ((pos "Kj%" ) ))
( "cabbage"  n ( k a1 b i jh ) ((pos "M6%" ) ))
( "cabbages"  n ( k a1 b i jh i z ) ((pos "Mj%" ) ))
( "cabbies"  n ( k a1 b i z ) ((pos "Kj%" ) ))
( "cabby"  n ( k a1 b ii ) ((pos "K8%" ) ))
( "caber"  n ( k ei1 b @ r ) ((pos "K6$" ) ))
( "cabers"  n ( k ei1 b @ z ) ((pos "Kj$" ) ))
( "cabin"  n ( k a1 b i n ) ((pos "K6%" ) ))
( "cabinet"  n ( k a1 b i n @ t ) ((pos "K6%" ) ))
( "cabinet-maker"  n ( k a1 b i n @ t - m ei2 k @ r ) ((pos "K6%" ) ))
( "cabinet-makers"  n ( k a1 b i n @ t - m ei2 k @ z ) ((pos "Kj%" ) ))
( "cabinets"  n ( k a1 b i n @ t s ) ((pos "Kj%" ) ))
( "cabins"  n ( k a1 b i n z ) ((pos "Kj%" ) ))
( "cable"  v ( k ei1 b l ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "cable's-length"  n ( k ei1 b l z - l e ng th ) ((pos "K6%" ) ))
( "cable's-lengths"  n ( k ei1 b l z - l e ng th s ) ((pos "Kj%" ) ))
( "cable-car"  n ( k ei1 b l - k aa r ) ((pos "K6%" ) ))
( "cable-cars"  n ( k ei1 b l - k aa z ) ((pos "Kj%" ) ))
( "cable-length"  n ( k ei1 b l - l e ng th ) ((pos "K6$" ) ))
( "cable-lengths"  n ( k ei1 b l - l e ng th s ) ((pos "Kj$" ) ))
( "cable-railway"  n ( k ei2 b l - r ei1 l w ei ) ((pos "K6%" ) ))
( "cable-railways"  n ( k ei2 b l - r ei1 l w ei z ) ((pos "Kj%" ) ))
( "cabled"  v ( k ei1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cablegram"  n ( k ei1 b l g r a m ) ((pos "K6$" ) ))
( "cablegrams"  n ( k ei1 b l g r a m z ) ((pos "Kj$" ) ))
( "cables"  v ( k ei1 b l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "cabling"  v ( k ei1 b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cabman"  n ( k a1 b m a n ) ((pos "Ki%" ) ))
( "cabmen"  n ( k a1 b m e n ) ((pos "Kj%" ) ))
( "caboodle"  n ( k @1 b uu1 d l ) ((pos "K6%" ) ))
( "caboodles"  n ( k @1 b uu1 d l z ) ((pos "Kj$" ) ))
( "caboose"  n ( k @1 b uu1 s ) ((pos "K6$" ) ))
( "cabooses"  n ( k @1 b uu1 s i z ) ((pos "Kj$" ) ))
( "cabs"  n ( k a1 b z ) ((pos "Kj%" ) ))
( "cabstand"  n ( k a1 b s t a n d ) ((pos "K6$" ) ))
( "cabstands"  n ( k a1 b s t a n d z ) ((pos "Kj$" ) ))
( "cacao"  n ( k @1 k aa1 ou ) ((pos "K6$" ) ))
( "cacao-bean"  n ( k @1 k aa1 ou - b ii n ) ((pos "K6$" ) ))
( "cacao-beans"  n ( k @1 k aa1 ou - b ii n z ) ((pos "Kj$" ) ))
( "cacao-tree"  n ( k @1 k aa1 ou - t r ii ) ((pos "K6$" ) ))
( "cacao-trees"  n ( k @1 k aa1 ou - t r ii z ) ((pos "Kj$" ) ))
( "cacaos"  n ( k @1 k aa1 ou z ) ((pos "Kj$" ) ))
( "cache"  v ( k a1 sh ) ((pos "H2$" "K6%" ) ))
( "cached"  v ( k a1 sh t ) ((pos "Hc$" "Hd$" ) ))
( "caches"  v ( k a1 sh i z ) ((pos "Ha$" "Kj$" ) ))
( "cachet"  n ( k a1 sh ei ) ((pos "K6%" ) ))
( "cachets"  n ( k a1 sh ei z ) ((pos "Kj$" ) ))
( "caching"  v ( k a1 sh i ng ) ((pos "Hb$" ) ))
( "cachou"  n ( k a1 sh uu ) ((pos "K6$" ) ))
( "cachous"  n ( k a1 sh uu z ) ((pos "Kj$" ) ))
( "cackle"  v ( k a1 k l ) ((pos "I2%" "M6%" ) ))
( "cackled"  v ( k a1 k l d ) ((pos "Ic%" "Id%" ) ))
( "cackler"  n ( k a1 k l @ r ) ((pos "K6%" ) ))
( "cacklers"  n ( k a1 k l @ z ) ((pos "Kj%" ) ))
( "cackles"  v ( k a1 k l z ) ((pos "Ia%" "Mj%" ) ))
( "cackling"  v ( k a1 k l i ng ) ((pos "Ib%" ) ))
( "cacophonies"  n ( k a1 k o1 f @ n i z ) ((pos "Kj$" ) ))
( "cacophonous"  j ( k a1 k o1 f @ n @ s ) ((pos "OA%" ) ))
( "cacophony"  n ( k a1 k o1 f @ n ii ) ((pos "K8%" ) ))
( "cacti"  n ( k a1 k t ai ) ((pos "Kj%" ) ))
( "cactus"  n ( k a1 k t @ s ) ((pos "K7%" ) ))
( "cactuses"  n ( k a1 k t @ s i z ) ((pos "Kj%" ) ))
( "cad"  n ( k a1 d ) ((pos "K6%" ) ))
( "cadaver"  n ( k @1 d aa1 v @ r ) ((pos "K6%" ) ))
( "cadaverous"  j ( k @1 d a1 v @ r @ s ) ((pos "OA%" ) ))
( "cadavers"  n ( k @1 d aa1 v @ z ) ((pos "Kj%" ) ))
( "caddie"  n ( k a1 d ii ) ((pos "K6%" ) ))
( "caddies"  n ( k a1 d i z ) ((pos "Kj%" ) ))
( "caddish"  j ( k a1 d i sh ) ((pos "OA%" ) ))
( "caddy"  n ( k a1 d ii ) ((pos "K8%" ) ))
( "cadence"  n ( k ei1 d @ n s ) ((pos "K6%" ) ))
( "cadences"  n ( k ei1 d @ n s i z ) ((pos "Kj%" ) ))
( "cadenza"  n ( k @1 d e1 n z @ ) ((pos "K6%" ) ))
( "cadenzas"  n ( k @1 d e1 n z @ z ) ((pos "Kj%" ) ))
( "cadet"  n ( k @1 d e1 t ) ((pos "K6%" ) ))
( "cadets"  n ( k @1 d e1 t s ) ((pos "Kj%" ) ))
( "cadge"  v ( k a1 jh ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "cadged"  v ( k a1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "cadger"  n ( k a1 jh @ r ) ((pos "K6%" ) ))
( "cadgers"  n ( k a1 jh @ z ) ((pos "Kj%" ) ))
( "cadges"  v ( k a1 jh i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "cadging"  v ( k a1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "cadmium"  n ( k a1 d m i@ m ) ((pos "L@%" ) ))
( "cadre"  n ( k aa1 d @ r ) ((pos "K6$" ) ))
( "cadres"  n ( k aa1 d @ z ) ((pos "Kj$" ) ))
( "cads"  n ( k a1 d z ) ((pos "Kj%" ) ))
( "caesura"  n ( s i1 z y u@1 r @ ) ((pos "K6$" ) ))
( "caesuras"  n ( s i1 z y u@1 r @ z ) ((pos "Kj$" ) ))
( "caf_e"  n ( k a1 f ei ) ((pos "K6%" ) ))
( "caf_es"  n ( k a1 f ei z ) ((pos "Kj%" ) ))
( "cafe-au-lait"  n ( k a2 f ei - ou - l ei1 ) ((pos "K6%" ) ))
( "cafe-au-laits"  n ( k a2 f ei - ou - l ei1 z ) ((pos "Kj%" ) ))
( "cafeteria"  n ( k a2 f @ t i@1 r i@ ) ((pos "K6%" ) ))
( "cafeterias"  n ( k a2 f @ t i@1 r i@ z ) ((pos "Kj%" ) ))
( "caff"  n ( k a1 f ) ((pos "K6%" ) ))
( "caffeine"  n ( k a1 f ii n ) ((pos "L@%" ) ))
( "caffs"  n ( k a1 f s ) ((pos "Kj%" ) ))
( "caftan"  n ( k a1 f t a n ) ((pos "K6%" ) ))
( "caftans"  n ( k a1 f t a n z ) ((pos "Kj%" ) ))
( "cage"  v ( k ei1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "caged"  v ( k ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cages"  v ( k ei1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cagey"  j ( k ei1 jh ii ) ((pos "OA%" ) ))
( "cagily"  a ( k ei1 jh i l ii ) ((pos "Pu%" ) ))
( "caging"  v ( k ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "cagoule"  n ( k @1 g uu1 l ) ((pos "K6%" ) ))
( "cagoules"  n ( k @1 g uu1 l z ) ((pos "Kj%" ) ))
( "cahoots"  n ( k @1 h uu1 t s ) ((pos "Kj%" ) ))
( "caiman"  n ( k ei1 m @ n ) ((pos "K6$" ) ))
( "caimans"  n ( k ei1 m @ n z ) ((pos "Kj$" ) ))
( "cairn"  n ( k e@1 n ) ((pos "K6%" ) ))
( "cairns"  n ( k e@1 n z ) ((pos "Kj%" ) ))
( "caisson"  n ( k ei1 s n ) ((pos "K6$" ) ))
( "caissons"  n ( k ei1 s n z ) ((pos "Kj$" ) ))
( "caitiff"  n ( k ei1 t i f ) ((pos "K6$" ) ))
( "caitiffs"  n ( k ei1 t i f s ) ((pos "Kj$" ) ))
( "cajole"  v ( k @1 jh ou1 l ) ((pos "H2%" ) (subcat "6A" )))
( "cajoled"  v ( k @1 jh ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cajolery"  n ( k @1 jh ou1 l @ r ii ) ((pos "L@$" ) ))
( "cajoles"  v ( k @1 jh ou1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "cajoling"  v ( k @1 jh ou1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cake"  v ( k ei1 k ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "caked"  v ( k ei1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cakes"  v ( k ei1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "caking"  v ( k ei1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "calabash"  n ( k a1 l @ b a sh ) ((pos "K7$" ) ))
( "calabashes"  n ( k a1 l @ b a sh i z ) ((pos "Kj$" ) ))
( "calamities"  n ( k @1 l a1 m i t i z ) ((pos "Kj%" ) ))
( "calamitous"  j ( k @1 l a1 m i t @ s ) ((pos "OA%" ) ))
( "calamity"  n ( k @1 l a1 m i t ii ) ((pos "K8%" ) ))
( "calcified"  v ( k a1 l s i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "calcifies"  v ( k a1 l s i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "calcify"  v ( k a1 l s i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "calcifying"  v ( k a1 l s i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "calcination"  n ( k a2 l s i n ei1 sh @ n ) ((pos "K6%" ) ))
( "calcinations"  n ( k a2 l s i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "calcine"  v ( k a1 l s ai n ) ((pos "J2$" ) (subcat "2A" )))
( "calcined"  v ( k a1 l s ai n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "calcines"  v ( k a1 l s ai n z ) ((pos "Ja$" ) (subcat "2A" )))
( "calcining"  v ( k a1 l s ai n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "calcium"  n ( k a1 l s i@ m ) ((pos "L@%" ) ))
( "calculable"  j ( k a1 l k y u l @ b l ) ((pos "OA%" ) ))
( "calculate"  v ( k a1 l k y u l ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "8" "9" )))
( "calculated"  v ( k a1 l k y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "9" )))
( "calculates"  v ( k a1 l k y u l ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "9" )))
( "calculating"  v ( k a1 l k y u l ei t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "8" "9" )))
( "calculation"  n ( k a2 l k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "calculations"  n ( k a2 l k y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "calculator"  n ( k a1 l k y u l ei t @ r ) ((pos "K6%" ) ))
( "calculators"  n ( k a1 l k y u l ei t @ z ) ((pos "Kj%" ) ))
( "calculi"  n ( k a1 l k y u l ai ) ((pos "Kj$" ) ))
( "calculus"  n ( k a1 l k y u l @ s ) ((pos "M7%" ) ))
( "calculuses"  n ( k a1 l k y u l @ s i z ) ((pos "Mj$" ) ))
( "caldron"  n ( k oo1 l d r @ n ) ((pos "K6$" ) ))
( "caldrons"  n ( k oo1 l d r @ n z ) ((pos "Kj$" ) ))
( "calendar"  n ( k a1 l i n d @ r ) ((pos "K6%" ) ))
( "calendars"  n ( k a1 l i n d @ z ) ((pos "Kj%" ) ))
( "calender"  v ( k a1 l i n d @ r ) ((pos "H0$" "K6$" ) (subcat )))
( "calendered"  v ( k a1 l i n d @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "calendering"  v ( k a1 l i n d @ r i ng ) ((pos "Hb$" ) (subcat )))
( "calenders"  v ( k a1 l i n d @ z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "calends"  n ( k a1 l @ n d z ) ((pos "Kj$" ) ))
( "calf"  n ( k aa1 f ) ((pos "Mi%" ) ))
( "calf-love"  n ( k aa1 f - l uh v ) ((pos "L@%" ) ))
( "calibrate"  v ( k a1 l i b r ei t ) ((pos "H2%" ) (subcat )))
( "calibrated"  v ( k a1 l i b r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "calibrates"  v ( k a1 l i b r ei t s ) ((pos "Ha%" ) (subcat )))
( "calibrating"  v ( k a1 l i b r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "calibration"  n ( k a2 l i b r ei1 sh @ n ) ((pos "M6%" ) ))
( "calibrations"  n ( k a2 l i b r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "calibre"  n ( k a1 l i b @ r ) ((pos "M6%" ) ))
( "calibres"  n ( k a1 l i b @ z ) ((pos "Mj$" ) ))
( "calico"  n ( k a1 l i k ou ) ((pos "L@%" ) ))
( "calif"  n ( k ei1 l i f ) ((pos "K6$" ) ))
( "califs"  n ( k ei1 l i f s ) ((pos "Kj$" ) ))
( "calipers"  n ( k a1 l i p @ z ) ((pos "Kj%" ) ))
( "caliph"  n ( k ei1 l i f ) ((pos "K6%" ) ))
( "caliphate"  n ( k a1 l i f ei t ) ((pos "K6$" ) ))
( "caliphates"  n ( k a1 l i f ei t s ) ((pos "Kj$" ) ))
( "caliphs"  n ( k ei1 l i f s ) ((pos "Kj$" ) ))
( "calisthenics"  n ( k a2 l i s th e1 n i k s ) ((pos "Lk$" ) ))
( "calk"  v ( k oo1 k ) ((pos "H0$" "K6$" ) ))
( "calked"  v ( k oo1 k t ) ((pos "Hc$" "Hd$" ) ))
( "calking"  v ( k oo1 k i ng ) ((pos "Hb$" ) ))
( "calks"  v ( k oo1 k s ) ((pos "Ha$" "Kj$" ) ))
( "call"  v ( k oo1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "12B" "13B" "15B" "22" )))
( "call-box"  n ( k oo1 l - b o k s ) ((pos "K7%" ) ))
( "call-boxes"  n ( k oo1 l - b o k s i z ) ((pos "Kj%" ) ))
( "call-girl"  n ( k oo1 l - g @@ l ) ((pos "K6%" ) ))
( "call-girls"  n ( k oo1 l - g @@ l z ) ((pos "Kj%" ) ))
( "call-over"  n ( k oo1 l - ou v @ r ) ((pos "K6$" ) ))
( "call-overs"  n ( k oo1 l - ou v @ z ) ((pos "Kj$" ) ))
( "call-up"  n ( k oo1 l - uh p ) ((pos "K6%" ) ))
( "call-ups"  n ( k oo1 l - uh p s ) ((pos "Kj$" ) ))
( "called"  v ( k oo1 l d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "12B" "13B" "15B" "22" )))
( "caller"  n ( k oo1 l @ r ) ((pos "K6%" ) ))
( "callers"  n ( k oo1 l @ z ) ((pos "Kj%" ) ))
( "calligraphy"  n ( k @1 l i1 g r @ f ii ) ((pos "L@%" ) ))
( "calling"  v ( k oo1 l i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "12B" "13B" "15B" "22" )))
( "callings"  n ( k oo1 l i ng z ) ((pos "Kj%" ) ))
( "calliope"  n ( k @1 l ai1 @ p ii ) ((pos "K6$" ) ))
( "calliopes"  n ( k @1 l ai1 @ p i z ) ((pos "Kj$" ) ))
( "callipers"  n ( k a1 l i p @ z ) ((pos "Kj$" ) ))
( "callisthenics"  n ( k a2 l i s th e1 n i k s ) ((pos "Lk$" ) ))
( "callosities"  n ( k a1 l o1 s i t i z ) ((pos "Kj$" ) ))
( "callosity"  n ( k a1 l o1 s i t ii ) ((pos "K8$" ) ))
( "callous"  j ( k a1 l @ s ) ((pos "OA%" ) ))
( "callousness"  n ( k a1 l @ s n @ s ) ((pos "L@%" ) ))
( "callow"  j ( k a1 l ou ) ((pos "OA%" ) ))
( "callowness"  n ( k a1 l ou n @ s ) ((pos "L@$" ) ))
( "calls"  v ( k oo1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "12B" "13B" "15B" "22" )))
( "callus"  n ( k a1 l @ s ) ((pos "K7$" ) ))
( "calluses"  n ( k a1 l @ s i z ) ((pos "Kj$" ) ))
( "calm"  v ( k aa1 m ) ((pos "J0%" "K6%" "OC%" ) (subcat "2C" "6A" )))
( "calmed"  v ( k aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "calmer"  j ( k aa1 m @ r ) ((pos "Or%" ) ))
( "calmest"  j ( k aa1 m i s t ) ((pos "Os%" ) ))
( "calming"  v ( k aa1 m i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "calmly"  a ( k aa1 m l ii ) ((pos "Pu%" ) ))
( "calmness"  n ( k aa1 m n @ s ) ((pos "L@%" ) ))
( "calms"  v ( k aa1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "calomel"  n ( k a1 l @ m e l ) ((pos "L@$" ) ))
( "calorie"  n ( k a1 l @ r ii ) ((pos "K6%" ) ))
( "calories"  n ( k a1 l @ r i z ) ((pos "Kj%" ) ))
( "calorific"  j ( k a2 l @ r i1 f i k ) ((pos "OA%" ) ))
( "calumniate"  v ( k @1 l uh1 m n i ei t ) ((pos "H2$" ) (subcat )))
( "calumniated"  v ( k @1 l uh1 m n i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "calumniates"  v ( k @1 l uh1 m n i ei t s ) ((pos "Ha$" ) (subcat )))
( "calumniating"  v ( k @1 l uh1 m n i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "calumnies"  n ( k a1 l @ m n i z ) ((pos "Mj%" ) ))
( "calumny"  n ( k a1 l @ m n ii ) ((pos "M8%" ) ))
( "calve"  v ( k aa1 v ) ((pos "I2%" ) ))
( "calved"  v ( k aa1 v d ) ((pos "Ic%" "Id%" ) ))
( "calves"  v ( k aa1 v z ) ((pos "Ia%" "Kj%" ) ))
( "calving"  v ( k aa1 v i ng ) ((pos "Ib%" ) ))
( "calyces"  n ( k ei1 l i s ii z ) ((pos "Kj$" ) ))
( "calypso"  n ( k @1 l i1 p s ou ) ((pos "K6%" ) ))
( "calypsos"  n ( k @1 l i1 p s ou z ) ((pos "Kj%" ) ))
( "calyx"  n ( k ei1 l i k s ) ((pos "K7$" ) ))
( "calyxes"  n ( k ei1 l i k s i z ) ((pos "Kj$" ) ))
( "cam"  n ( k a1 m ) ((pos "K6%" ) ))
( "camaraderie"  n ( k a2 m @ r aa1 d @ r ii ) ((pos "L@%" ) ))
( "camber"  v ( k a1 m b @ r ) ((pos "J0$" "K6%" ) ))
( "cambered"  v ( k a1 m b @ d ) ((pos "Jc$" "Jd%" ) ))
( "cambering"  v ( k a1 m b @ r i ng ) ((pos "Jb$" ) ))
( "cambers"  v ( k a1 m b @ z ) ((pos "Ja$" "Kj$" ) ))
( "cambric"  n ( k ei1 m b r i k ) ((pos "L@$" ) ))
( "came"  v ( k ei1 m ) ((pos "Ic*" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" )))
( "camel"  n ( k a1 m @ l ) ((pos "K6%" ) ))
( "camel-hair"  n ( k a1 m @ l - h e@ r ) ((pos "L@%" ) ))
( "camellia"  n ( k @1 m ii1 l i@ ) ((pos "K6%" ) ))
( "camellias"  n ( k @1 m ii1 l i@ z ) ((pos "Kj%" ) ))
( "camels"  n ( k a1 m @ l z ) ((pos "Kj%" ) ))
( "cameo"  n ( k a1 m i@ @ ) ((pos "K6%" ) ))
( "cameos"  n ( k a1 m i@ u z ) ((pos "Kj%" ) ))
( "camera"  n ( k a1 m @ r @ ) ((pos "K6%" ) ))
( "cameraman"  n ( k a1 m @ r @ m a n ) ((pos "Ki%" ) ))
( "cameramen"  n ( k a1 m @ r @ m e n ) ((pos "Kj%" ) ))
( "cameras"  n ( k a1 m @ r @ z ) ((pos "Kj%" ) ))
( "camion"  n ( k a1 m i@ n ) ((pos "K6$" ) ))
( "camions"  n ( k a1 m i@ n z ) ((pos "Kj$" ) ))
( "camomile"  n ( k a1 m @ m ai l ) ((pos "L@%" ) ))
( "camouflage"  v ( k a1 m @ f l aa zh ) ((pos "H2%" "L@%" ) (subcat )))
( "camouflaged"  v ( k a1 m @ f l aa zh d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "camouflages"  v ( k a1 m @ f l aa zh i z ) ((pos "Ha%" ) (subcat )))
( "camouflaging"  v ( k a1 m @ f l aa zh i ng ) ((pos "Hb%" ) (subcat )))
( "camp"  v ( k a1 m p ) ((pos "J0%" "K6%" "OA%" ) (subcat "2A" )))
( "camp-bed"  n ( k a1 m p - b e d ) ((pos "K6%" ) ))
( "camp-beds"  n ( k a1 m p - b e d z ) ((pos "Kj%" ) ))
( "camp-chair"  n ( k a1 m p - ch e@ r ) ((pos "K6$" ) ))
( "camp-chairs"  n ( k a1 m p - ch e@ z ) ((pos "Kj$" ) ))
( "camp-fire"  n ( k a1 m p - f ai @ r ) ((pos "K6%" ) ))
( "camp-fires"  n ( k a1 m p - f ai @ z ) ((pos "Kj%" ) ))
( "camp-follower"  n ( k a1 m p - f o1 l ou @ r ) ((pos "K6%" ) ))
( "camp-followers"  n ( k a1 m p - f o1 l ou @ z ) ((pos "Kj%" ) ))
( "camp-stool"  n ( k a1 m p - s t uu l ) ((pos "K6%" ) ))
( "camp-stools"  n ( k a1 m p - s t uu l z ) ((pos "Kj%" ) ))
( "campaign"  v ( k a1 m p ei1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "campaigned"  v ( k a1 m p ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "campaigner"  n ( k a1 m p ei1 n @ r ) ((pos "K6%" ) ))
( "campaigners"  n ( k a1 m p ei1 n @ z ) ((pos "Kj%" ) ))
( "campaigning"  v ( k a1 m p ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "campaigns"  v ( k a1 m p ei1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "campanile"  n ( k a2 m p @ n ii1 l ii ) ((pos "K6$" ) ))
( "campaniles"  n ( k a2 m p @ n ii1 l i z ) ((pos "Kj$" ) ))
( "campanula"  n ( k @1 m p a1 n y u l @ ) ((pos "K6$" ) ))
( "campanulas"  n ( k @1 m p a1 n y u l @ z ) ((pos "Kj$" ) ))
( "camped"  v ( k a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "camper"  n ( k a1 m p @ r ) ((pos "K6%" ) ))
( "campers"  n ( k a1 m p @ z ) ((pos "Kj%" ) ))
( "camphor"  n ( k a1 m f @ r ) ((pos "L@%" ) ))
( "camphorated"  j ( k a1 m f @ r ei t i d ) ((pos "OA%" ) ))
( "camping"  v ( k a1 m p i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "campion"  n ( k a1 m p i@ n ) ((pos "L@%" ) ))
( "camps"  v ( k a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "campus"  n ( k a1 m p @ s ) ((pos "K7%" ) ))
( "campuses"  n ( k a1 m p @ s i z ) ((pos "Kj%" ) ))
( "cams"  n ( k a1 m z ) ((pos "Kj%" ) ))
( "camshaft"  n ( k a1 m sh aa f t ) ((pos "K6%" ) ))
( "camshafts"  n ( k a1 m sh aa f t s ) ((pos "Kj%" ) ))
( "can"  v ( k a n ) ((pos "G5*" "Ga*" "H4%" "K6%" ) (subcat "5" )))
( "can't"  v ( k aa n t ) ((pos "Gg%" ) ))
( "canal"  n ( k @1 n a1 l ) ((pos "K6%" ) ))
( "canalization"  n ( k a2 n @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "canalizations"  n ( k a2 n @ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "canalize"  v ( k a1 n @ l ai z ) ((pos "H2%" ) (subcat "6A" )))
( "canalized"  v ( k a1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "canalizes"  v ( k a1 n @ l ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "canalizing"  v ( k a1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "canals"  n ( k @1 n a1 l z ) ((pos "Kj%" ) ))
( "canap_e"  n ( k a1 n @ p ei ) ((pos "K6$" ) ))
( "canap_es"  n ( k a1 n @ p ei z ) ((pos "Kj$" ) ))
( "canard"  n ( k a1 n aa1 d ) ((pos "K6$" ) ))
( "canards"  n ( k a1 n aa1 d z ) ((pos "Kj$" ) ))
( "canaries"  n ( k @1 n e@1 r i z ) ((pos "Kj%" ) ))
( "canary"  n ( k @1 n e@1 r ii ) ((pos "K8%" ) ))
( "canary-bird"  n ( k @1 n e@1 r i - b @@ d ) ((pos "K6$" ) ))
( "canary-birds"  n ( k @1 n e@1 r i - b @@ d z ) ((pos "Kj$" ) ))
( "canary-wine"  n ( k @1 n e@1 r i - w ai n ) ((pos "K6$" ) ))
( "canary-wines"  n ( k @1 n e@1 r i - w ai n z ) ((pos "Kj$" ) ))
( "canasta"  n ( k @1 n a1 s t @ ) ((pos "K6$" ) ))
( "canastas"  n ( k @1 n a1 s t @ z ) ((pos "Kj$" ) ))
( "cancan"  n ( k a1 n k a n ) ((pos "K6%" ) ))
( "cancans"  n ( k a1 n k a n z ) ((pos "Kj%" ) ))
( "cancel"  v ( k a1 n s @ l ) ((pos "J4%" ) (subcat "2C" "6A" )))
( "cancellation"  n ( k a2 n s @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "cancellations"  n ( k a2 n s @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "cancelled"  v ( k a1 n s @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "cancelling"  v ( k a1 n s @ l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "cancels"  v ( k a1 n s @ l z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "cancer"  n ( k a1 n s @ r ) ((pos "M6%" ) ))
( "cancerous"  j ( k a1 n s @ r @ s ) ((pos "OA%" ) ))
( "cancers"  n ( k a1 n s @ z ) ((pos "Mj%" ) ))
( "candelabra"  n ( k a2 n d @ l aa1 b r @ ) ((pos "Kj%" ) ))
( "candelabrum"  n ( k a2 n d @ l aa1 b r @ m ) ((pos "Ki$" ) ))
( "candid"  j ( k a1 n d i d ) ((pos "OA%" ) ))
( "candidate"  n ( k a1 n d i d @ t ) ((pos "K6%" ) ))
( "candidates"  n ( k a1 n d i d @ t s ) ((pos "Kj%" ) ))
( "candidature"  n ( k a1 n d i d @ ch @ r ) ((pos "K6%" ) ))
( "candidatures"  n ( k a1 n d i d @ ch @ z ) ((pos "Kj$" ) ))
( "candidly"  a ( k a1 n d i d l ii ) ((pos "Pu%" ) ))
( "candied"  v ( k a1 n d i d ) ((pos "Jc$" "Jd%" "OA%" ) (subcat "2A" )))
( "candies"  v ( k a1 n d i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "candle"  n ( k a1 n d l ) ((pos "K6%" ) ))
( "candle-power"  n ( k a1 n d l - p au @ r ) ((pos "L@%" ) ))
( "candlelight"  n ( k a1 n d l - l ai t ) ((pos "L@%" ) ))
( "candles"  n ( k a1 n d l z ) ((pos "Kj%" ) ))
( "candlestick"  n ( k a1 n d l s t i k ) ((pos "K6%" ) ))
( "candlesticks"  n ( k a1 n d l s t i k s ) ((pos "Kj%" ) ))
( "candlewick"  n ( k a1 n d l w i k ) ((pos "L@%" ) ))
( "candour"  n ( k a1 n d @ r ) ((pos "L@%" ) ))
( "candy"  v ( k a1 n d ii ) ((pos "J3$" "M8%" ) (subcat "2A" )))
( "candy-floss"  n ( k a1 n d i - f l o s ) ((pos "L@%" ) ))
( "candying"  v ( k a1 n d i i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "candytuft"  n ( k a1 n d i t uh f t ) ((pos "K6%" ) ))
( "candytufts"  n ( k a1 n d i t uh f t s ) ((pos "Kj$" ) ))
( "cane"  v ( k ei1 n ) ((pos "H2%" "M6%" ) (subcat )))
( "caned"  v ( k ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "canes"  v ( k ei1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "canine"  j ( k ei1 n ai n ) ((pos "OA%" ) ))
( "caning"  v ( k ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "canister"  n ( k a1 n i s t @ r ) ((pos "K6%" ) ))
( "canisters"  n ( k a1 n i s t @ z ) ((pos "Kj%" ) ))
( "canker"  v ( k a1 ng k @ r ) ((pos "H0$" "L@$" ) ))
( "cankered"  v ( k a1 ng k @ d ) ((pos "Hc$" "Hd$" ) ))
( "cankering"  v ( k a1 ng k @ r i ng ) ((pos "Hb$" ) ))
( "cankerous"  j ( k a1 ng k @ r @ s ) ((pos "OA$" ) ))
( "cankers"  v ( k a1 ng k @ z ) ((pos "Ha$" ) ))
( "canna"  n ( k a1 n @ ) ((pos "K6$" ) ))
( "cannabis"  n ( k a1 n @ b i s ) ((pos "L@%" ) ))
( "cannas"  n ( k a1 n @ z ) ((pos "Kj$" ) ))
( "canned"  v ( k a1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "5" )))
( "canneries"  n ( k a1 n @ r i z ) ((pos "Kj%" ) ))
( "cannery"  n ( k a1 n @ r ii ) ((pos "K8%" ) ))
( "cannibal"  n ( k a1 n i b @ l ) ((pos "K6%" ) ))
( "cannibalism"  n ( k a1 n i b @ l i z @ m ) ((pos "L@%" ) ))
( "cannibalistic"  j ( k a2 n i b @ l l i1 s t i k ) ((pos "OA%" ) ))
( "cannibalize"  v ( k a1 n i b @ l ai z ) ((pos "H2%" ) ))
( "cannibalized"  v ( k a1 n i b @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "cannibalizes"  v ( k a1 n i b @ l ai z i z ) ((pos "Ha%" ) ))
( "cannibalizing"  v ( k a1 n i b @ l ai z i ng ) ((pos "Hb%" ) ))
( "cannibals"  n ( k a1 n i b @ l z ) ((pos "Kj%" ) ))
( "cannier"  j ( k a1 n i@ r ) ((pos "Or%" ) ))
( "canniest"  j ( k a1 n i i s t ) ((pos "Os%" ) ))
( "cannily"  a ( k a1 n i l ii ) ((pos "Pu%" ) ))
( "canning"  v ( k a1 n i ng ) ((pos "Hb%" ) (subcat "5" )))
( "cannon"  n ( k a1 n @ n ) ((pos "K6%" ) ))
( "cannon-fodder"  n ( k a1 n @ n - f o d @ r ) ((pos "L@%" ) ))
( "cannonade"  n ( k a2 n @ n ei1 d ) ((pos "K6%" ) ))
( "cannonades"  n ( k a2 n @ n ei1 d z ) ((pos "Kj%" ) ))
( "cannons"  n ( k a1 n @ n z ) ((pos "Kj%" ) ))
( "cannot"  v ( k a1 n @ t ) ((pos "Gg*" ) ))
( "canny"  j ( k a1 n ii ) ((pos "OD%" ) ))
( "canoe"  v ( k @1 n uu1 ) ((pos "H5%" "K6%" ) (subcat "2A" )))
( "canoed"  v ( k @1 n uu1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "canoeing"  v ( k @1 n uu1 i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "canoeist"  n ( k @1 n uu1 i s t ) ((pos "K6%" ) ))
( "canoeists"  n ( k @1 n uu1 i s t s ) ((pos "Kj%" ) ))
( "canoes"  v ( k @1 n uu1 z ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "canon"  n ( k a1 n @ n ) ((pos "K6%" ) ))
( "canonical"  j ( k @1 n o1 n i k l ) ((pos "OA%" ) ))
( "canonization"  n ( k a2 n @ n ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "canonizations"  n ( k a2 n @ n ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "canonize"  v ( k a1 n @ n ai z ) ((pos "H2%" ) (subcat )))
( "canonized"  v ( k a1 n @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "canonizes"  v ( k a1 n @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "canonizing"  v ( k a1 n @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "canons"  n ( k a1 n @ n z ) ((pos "Kj%" ) ))
( "canopied"  j ( k a1 n @ p i d ) ((pos "OA%" ) ))
( "canopies"  n ( k a1 n @ p i z ) ((pos "Kj%" ) ))
( "canopy"  n ( k a1 n @ p ii ) ((pos "K8%" ) ))
( "cans"  v ( k a1 n z ) ((pos "Ha%" "Kj%" ) (subcat "5" )))
( "cant"  v ( k a1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "cantaloup"  n ( k a1 n t @ l uu p ) ((pos "K6$" ) ))
( "cantaloupe"  n ( k a1 n t @ l uu p ) ((pos "K6%" ) ))
( "cantaloupes"  n ( k a1 n t @ l uu p s ) ((pos "Kj%" ) ))
( "cantaloups"  n ( k a1 n t @ l uu p s ) ((pos "Kj$" ) ))
( "cantankerous"  j ( k a1 n t a1 ng k @ r @ s ) ((pos "OA%" ) ))
( "cantankerously"  a ( k a1 n t a1 ng k @ r @ s l ii ) ((pos "Pu%" ) ))
( "cantata"  n ( k a1 n t aa1 t @ ) ((pos "K6%" ) ))
( "cantatas"  n ( k a1 n t aa1 t @ z ) ((pos "Kj%" ) ))
( "canted"  v ( k a1 n t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" )))
( "canteen"  n ( k a1 n t ii1 n ) ((pos "K6%" ) ))
( "canteens"  n ( k a1 n t ii1 n z ) ((pos "Kj%" ) ))
( "canter"  v ( k a1 n t @ r ) ((pos "J0%" "K6%" ) ))
( "cantered"  v ( k a1 n t @ d ) ((pos "Jc%" "Jd%" ) ))
( "cantering"  v ( k a1 n t @ r i ng ) ((pos "Jb%" ) ))
( "canters"  v ( k a1 n t @ z ) ((pos "Ja%" "Kj%" ) ))
( "canticle"  n ( k a1 n t i k l ) ((pos "K6%" ) ))
( "canticles"  n ( k a1 n t i k l z ) ((pos "Kj%" ) ))
( "cantilever"  n ( k a1 n t i l ii v @ r ) ((pos "K6%" ) ))
( "cantilevers"  n ( k a1 n t i l ii v @ z ) ((pos "Kj%" ) ))
( "canting"  v ( k a1 n t i ng ) ((pos "Jb$" ) (subcat "2A" "2C" "6A" )))
( "canto"  n ( k a1 n t ou ) ((pos "K6%" ) ))
( "canton"  n ( k a1 n t o n ) ((pos "K6%" ) ))
( "cantonment"  n ( k a1 n t uu1 n m @ n t ) ((pos "K6$" ) ))
( "cantonments"  n ( k a1 n t uu1 n m @ n t s ) ((pos "Kj$" ) ))
( "cantons"  n ( k a1 n t o n z ) ((pos "Kj%" ) ))
( "cantor"  n ( k a1 n t oo r ) ((pos "K6$" ) ))
( "cantors"  n ( k a1 n t oo z ) ((pos "Kj$" ) ))
( "cantos"  n ( k a1 n t ou z ) ((pos "Kj$" ) ))
( "cants"  v ( k a1 n t s ) ((pos "Ja$" "Mj$" ) (subcat "2A" "2C" "6A" )))
( "canvas"  n ( k a1 n v @ s ) ((pos "M7%" ) ))
( "canvases"  n ( k a1 n v @ s i z ) ((pos "Mj%" ) ))
( "canvass"  v ( k a1 n v @ s ) ((pos "J1%" "K7%" ) (subcat "2A" "3A" )))
( "canvassed"  v ( k a1 n v @ s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "canvasses"  v ( k a1 n v @ s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "canvassing"  v ( k a1 n v @ s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "canyon"  n ( k a1 n y @ n ) ((pos "K6%" ) ))
( "canyons"  n ( k a1 n y @ n z ) ((pos "Kj%" ) ))
( "cap"  v ( k a1 p ) ((pos "H4%" "K6%" ) (subcat )))
( "cap-`a-pie"  a ( k a2 p - @ - p ii1 ) ((pos "Pu$" ) ))
( "capabilities"  n ( k ei2 p @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "capability"  n ( k ei2 p @ b i1 l i t ii ) ((pos "M8%" ) ))
( "capable"  j ( k ei1 p @ b l ) ((pos "OA%" ) ))
( "capably"  a ( k ei1 p @ b l ii ) ((pos "Pu%" ) ))
( "capacious"  j ( k @1 p ei1 sh @ s ) ((pos "OA%" ) ))
( "capacities"  n ( k @1 p a1 s i t i z ) ((pos "Mj%" ) ))
( "capacity"  n ( k @1 p a1 s i t ii ) ((pos "M8%" ) ))
( "caparison"  v ( k @1 p a1 r i s @ n ) ((pos "H0$" "K6$" ) ))
( "caparisoned"  v ( k @1 p a1 r i s @ n d ) ((pos "Hc$" "Hd$" ) ))
( "caparisoning"  v ( k @1 p a1 r i s @ n i ng ) ((pos "Hb$" ) ))
( "caparisons"  v ( k @1 p a1 r i s @ n z ) ((pos "Ha$" "Kj$" ) ))
( "cape"  n ( k ei1 p ) ((pos "K6%" ) ))
( "caper"  v ( k ei1 p @ r ) ((pos "I0%" "K6%" ) ))
( "capered"  v ( k ei1 p @ d ) ((pos "Ic%" "Id%" ) ))
( "capering"  v ( k ei1 p @ r i ng ) ((pos "Ib%" ) ))
( "capers"  v ( k ei1 p @ z ) ((pos "Ia%" "Kj%" ) ))
( "capes"  n ( k ei1 p s ) ((pos "Kj%" ) ))
( "capillaries"  n ( k @1 p i1 l @ r i z ) ((pos "Kj%" ) ))
( "capillary"  n ( k @1 p i1 l @ r ii ) ((pos "K8%" ) ))
( "capital"  n ( k a1 p i t @ l ) ((pos "M6%" "OA%" ) ))
( "capitalism"  n ( k a1 p i t @ l i z @ m ) ((pos "L@%" "OA%" ) ))
( "capitalist"  n ( k a1 p i t @ l i s t ) ((pos "K6%" ) ))
( "capitalistic"  j ( k a2 p i t @ l i1 s t i k ) ((pos "OA%" ) ))
( "capitalists"  n ( k a1 p i t @ l i s t s ) ((pos "Kj%" ) ))
( "capitalization"  n ( k a2 p i t @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "capitalizations"  n ( k a2 p i t @ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "capitalize"  v ( k a1 p i t @ l ai z ) ((pos "J2%" ) (subcat "3A" )))
( "capitalized"  v ( k a1 p i t @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "capitalizes"  v ( k a1 p i t @ l ai z i z ) ((pos "Ja%" ) (subcat "3A" )))
( "capitalizing"  v ( k a1 p i t @ l ai z i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "capitals"  n ( k a1 p i t @ l z ) ((pos "Mj%" ) ))
( "capitation"  n ( k a2 p i t ei1 sh @ n ) ((pos "K6%" ) ))
( "capitations"  n ( k a2 p i t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "capitulate"  v ( k @1 p i1 ch u l ei t ) ((pos "H2%" ) (subcat )))
( "capitulated"  v ( k @1 p i1 ch u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "capitulates"  v ( k @1 p i1 ch u l ei t s ) ((pos "Ha%" ) (subcat )))
( "capitulating"  v ( k @1 p i1 ch u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "capitulation"  n ( k @1 p i2 ch u l ei1 sh @ n ) ((pos "L@%" ) ))
( "capon"  n ( k ei1 p @ n ) ((pos "K6%" ) ))
( "capons"  n ( k ei1 p @ n z ) ((pos "Kj%" ) ))
( "capped"  v ( k a1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "capping"  v ( k a1 p i ng ) ((pos "Hb%" ) (subcat )))
( "caprice"  n ( k @1 p r ii1 s ) ((pos "K6%" ) ))
( "caprices"  n ( k @1 p r ii1 s i z ) ((pos "Kj%" ) ))
( "capricious"  j ( k @1 p r i1 sh @ s ) ((pos "OA%" ) ))
( "capriciously"  a ( k @1 p r i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "capriciousness"  n ( k @1 p r i1 sh @ s n i s ) ((pos "L@%" ) ))
( "caps"  v ( k a1 p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "capsicum"  n ( k a1 p s i k @ m ) ((pos "K6%" ) ))
( "capsicums"  n ( k a1 p s i k @ m z ) ((pos "Kj%" ) ))
( "capsize"  v ( k a1 p s ai1 z ) ((pos "J2%" ) (subcat "2A" )))
( "capsized"  v ( k a1 p s ai1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "capsizes"  v ( k a1 p s ai1 z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "capsizing"  v ( k a1 p s ai1 z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "capstan"  n ( k a1 p s t @ n ) ((pos "K6%" ) ))
( "capstans"  n ( k a1 p s t @ n z ) ((pos "Kj%" ) ))
( "capsule"  n ( k a1 p s y uu l ) ((pos "K6%" ) ))
( "capsules"  n ( k a1 p s y uu l z ) ((pos "Kj%" ) ))
( "captain"  v ( k a1 p t i n ) ((pos "H0%" "K6%" ) (subcat )))
( "captained"  v ( k a1 p t i n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "captaining"  v ( k a1 p t i n i ng ) ((pos "Hb%" ) (subcat )))
( "captains"  v ( k a1 p t i n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "caption"  n ( k a1 p sh @ n ) ((pos "K6%" ) ))
( "captions"  n ( k a1 p sh @ n z ) ((pos "Kj%" ) ))
( "captious"  j ( k a1 p sh @ s ) ((pos "OA$" ) ))
( "captiously"  a ( k a1 p sh @ s l ii ) ((pos "Pu$" ) ))
( "captivate"  v ( k a1 p t i v ei t ) ((pos "H2%" ) (subcat )))
( "captivated"  v ( k a1 p t i v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "captivates"  v ( k a1 p t i v ei t s ) ((pos "Ha%" ) (subcat )))
( "captivating"  v ( k a1 p t i v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "captive"  n ( k a1 p t i v ) ((pos "K6%" "OA%" ) ))
( "captives"  n ( k a1 p t i v z ) ((pos "Kj%" ) ))
( "captivity"  n ( k a1 p t i1 v i t ii ) ((pos "L@%" ) ))
( "captor"  n ( k a1 p t @ r ) ((pos "K6%" ) ))
( "captors"  n ( k a1 p t @ z ) ((pos "Kj%" ) ))
( "capture"  v ( k a1 p ch @ r ) ((pos "H2%" "M6%" ) (subcat )))
( "captured"  v ( k a1 p ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "captures"  v ( k a1 p ch @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "capturing"  v ( k a1 p ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "car"  n ( k aa1 r ) ((pos "K6*" ) ))
( "car-ferries"  n ( k aa1 - f e r i z ) ((pos "Kj%" ) ))
( "car-ferry"  n ( k aa1 - f e r ii ) ((pos "K8%" ) ))
( "carafe"  n ( k @1 r a1 f ) ((pos "K6%" ) ))
( "carafes"  n ( k @1 r a1 f s ) ((pos "Kj%" ) ))
( "caramel"  n ( k a1 r @ m @ l ) ((pos "M6%" ) ))
( "caramels"  n ( k a1 r @ m @ l z ) ((pos "Mj%" ) ))
( "carapace"  n ( k a1 r @ p ei s ) ((pos "K6$" ) ))
( "carapaces"  n ( k a1 r @ p ei s i z ) ((pos "Kj$" ) ))
( "carat"  n ( k a1 r @ t ) ((pos "K6%" ) ))
( "carats"  n ( k a1 r @ t s ) ((pos "Kj%" ) ))
( "caravan"  n ( k a1 r @ v a n ) ((pos "K6%" ) ))
( "caravanning"  n ( k a2 r @ v a1 n i ng ) ((pos "L@%" ) ))
( "caravans"  n ( k a1 r @ v a n z ) ((pos "Kj%" ) ))
( "caravansaries"  n ( k a2 r @ v a1 n s @ r i z ) ((pos "Kj$" ) ))
( "caravansary"  n ( k a2 r @ v a1 n s @ r ii ) ((pos "K8$" ) ))
( "caravanserai"  n ( k a2 r @ v a1 n s @ r ai ) ((pos "K6$" ) ))
( "caravanserais"  n ( k a2 r @ v a1 n s @ r ai z ) ((pos "Kj$" ) ))
( "caraway"  n ( k a1 r @ w ei ) ((pos "K6%" ) ))
( "caraways"  n ( k a1 r @ w ei z ) ((pos "Kj$" ) ))
( "carbide"  n ( k aa1 b ai d ) ((pos "K6%" ) ))
( "carbides"  n ( k aa1 b ai d z ) ((pos "Kj%" ) ))
( "carbine"  n ( k aa1 b ai n ) ((pos "K6%" ) ))
( "carbines"  n ( k aa1 b ai n z ) ((pos "Kj%" ) ))
( "carbohydrate"  n ( k aa2 b @ h ai1 d r ei t ) ((pos "M6%" ) ))
( "carbohydrates"  n ( k aa2 b @ h ai1 d r ei t s ) ((pos "Mj%" ) ))
( "carbolic"  j ( k aa1 b o1 l i k ) ((pos "OA%" ) ))
( "carbon"  n ( k aa1 b @ n ) ((pos "M6%" ) ))
( "carbon-paper"  n ( k aa1 b @ n - p ei p @ r ) ((pos "L@%" ) ))
( "carbonated"  j ( k aa1 b @ n ei t i d ) ((pos "OA%" ) ))
( "carbonic"  j ( k aa1 b o1 n i k ) ((pos "OA$" ) ))
( "carboniferous"  j ( k aa2 b @ n i1 f @ r @ s ) ((pos "OA%" ) ))
( "carbonization"  n ( k aa2 b @ n ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "carbonizations"  n ( k aa2 b @ n ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "carbonize"  v ( k aa1 b @ n ai z ) ((pos "H2%" ) (subcat )))
( "carbonized"  v ( k aa1 b @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "carbonizes"  v ( k aa1 b @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "carbonizing"  v ( k aa1 b @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "carbons"  n ( k aa1 b @ n z ) ((pos "Mj%" ) ))
( "carborundum"  n ( k aa2 b @ r uh1 n d @ m ) ((pos "L@%" ) ))
( "carboy"  n ( k aa1 b oi ) ((pos "K6$" ) ))
( "carboys"  n ( k aa1 b oi z ) ((pos "Kj$" ) ))
( "carbuncle"  n ( k aa1 b uh ng k l ) ((pos "K6%" ) ))
( "carbuncles"  n ( k aa1 b uh ng k l z ) ((pos "Kj%" ) ))
( "carburettor"  n ( k aa2 b @ r e1 t @ r ) ((pos "K6%" ) ))
( "carburettors"  n ( k aa2 b @ r e1 t @ z ) ((pos "Kj%" ) ))
( "carcase"  n ( k aa1 k @ s ) ((pos "K6$" ) ))
( "carcases"  n ( k aa1 k @ s i z ) ((pos "Kj$" ) ))
( "carcass"  n ( k aa1 k @ s ) ((pos "K7%" ) ))
( "carcasses"  n ( k aa1 k @ s i z ) ((pos "Kj%" ) ))
( "card"  v ( k aa1 d ) ((pos "H0$" "K6%" ) ))
( "card-sharper"  n ( k aa1 d - sh aa p @ r ) ((pos "K6%" ) ))
( "card-sharpers"  n ( k aa1 d - sh aa p @ z ) ((pos "Kj%" ) ))
( "cardamom"  n ( k aa1 d @ m @ m ) ((pos "L@$" ) ))
( "cardboard"  n ( k aa1 d b oo d ) ((pos "L@%" ) ))
( "carded"  v ( k aa1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "cardiac"  j ( k aa1 d i a k ) ((pos "OA%" ) ))
( "cardigan"  n ( k aa1 d i g @ n ) ((pos "K6%" ) ))
( "cardigans"  n ( k aa1 d i g @ n z ) ((pos "Kj%" ) ))
( "cardinal"  n ( k aa1 d i n l ) ((pos "K6%" "OA%" ) ))
( "cardinals"  n ( k aa1 d i n l z ) ((pos "Kj%" ) ))
( "carding"  v ( k aa1 d i ng ) ((pos "Hb$" ) ))
( "cards"  v ( k aa1 d z ) ((pos "Ha$" "Kj%" ) ))
( "care"  v ( k e@1 r ) ((pos "I2%" "M6%" ) (subcat "2A" "3A" "3B" )))
( "cared"  v ( k e@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" "3B" )))
( "careen"  v ( k @1 r ii1 n ) ((pos "J0%" ) (subcat "2A" )))
( "careened"  v ( k @1 r ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "careening"  v ( k @1 r ii1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "careens"  v ( k @1 r ii1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "career"  v ( k @1 r i@1 r ) ((pos "I0%" "M6%" ) (subcat "2C" )))
( "careered"  v ( k @1 r i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "careering"  v ( k @1 r i@1 r i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "careerist"  n ( k @1 r i@1 r i s t ) ((pos "K6%" ) ))
( "careerists"  n ( k @1 r i@1 r i s t s ) ((pos "Kj%" ) ))
( "careers"  v ( k @1 r i@1 z ) ((pos "Ia%" "Mj%" ) (subcat "2C" )))
( "carefree"  j ( k e@1 f r ii ) ((pos "OA%" ) ))
( "careful"  j ( k e@1 f u l ) ((pos "OA%" ) ))
( "carefully"  a ( k e@1 f @ l ii ) ((pos "Pu%" ) ))
( "carefulness"  n ( k e@1 f @ l n @ s ) ((pos "L@%" ) ))
( "careladen"  j ( k e@1 l ei d n ) ((pos "OA$" ) ))
( "careless"  j ( k e@1 l @ s ) ((pos "OA%" ) ))
( "carelessly"  a ( k e@1 l @ s l ii ) ((pos "Pu%" ) ))
( "carelessness"  n ( k e@1 l @ s n @ s ) ((pos "L@%" ) ))
( "cares"  v ( k e@1 z ) ((pos "Ia%" "Mj%" ) (subcat "2A" "3A" "3B" )))
( "caress"  v ( k @1 r e1 s ) ((pos "H1%" "K7%" ) (subcat )))
( "caressed"  v ( k @1 r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "caresses"  v ( k @1 r e1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "caressing"  v ( k @1 r e1 s i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "caressingly"  a ( k @1 r e1 s i ng l ii ) ((pos "Pu%" ) ))
( "caret"  n ( k a1 r e t ) ((pos "K6$" ) ))
( "caretaker"  n ( k e@1 t ei k @ r ) ((pos "K6%" ) ))
( "caretakers"  n ( k e@1 t ei k @ z ) ((pos "Kj%" ) ))
( "carets"  n ( k a1 r e t s ) ((pos "Kj$" ) ))
( "careworn"  j ( k e@1 w oo n ) ((pos "OA%" ) ))
( "cargo"  n ( k aa1 g ou ) ((pos "M7%" ) ))
( "cargoes"  n ( k aa1 g ou z ) ((pos "Mj%" ) ))
( "caribou"  n ( k a1 r i b uu ) ((pos "K9%" ) ))
( "caribous"  n ( k a1 r i b uu z ) ((pos "Kj%" ) ))
( "caricature"  v ( k a1 r i k @ ch u@ r ) ((pos "H2%" "M6%" ) (subcat )))
( "caricatured"  v ( k a1 r i k @ ch u@ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "caricatures"  v ( k a1 r i k @ ch u@ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "caricaturing"  v ( k a1 r i k @ ch u@ r i ng ) ((pos "Hb%" ) (subcat )))
( "caricaturist"  n ( k a1 r i k @ ch uu r i s t ) ((pos "K6%" ) ))
( "caricaturists"  n ( k a1 r i k @ ch uu r i s t s ) ((pos "Kj%" ) ))
( "caries"  n ( k e@1 r ii z ) ((pos "L@%" ) ))
( "carillon"  n ( k @1 r i1 l y @ n ) ((pos "K6$" ) ))
( "carillons"  n ( k @1 r i1 l y @ n z ) ((pos "Kj$" ) ))
( "caring"  v ( k e@1 r i ng ) ((pos "Ib%" ) (subcat "2A" "3A" "3B" )))
( "carious"  j ( k e@1 r i@ s ) ((pos "OA$" ) ))
( "carmine"  n ( k aa1 m ai n ) ((pos "M6%" "OA%" ) ))
( "carmines"  n ( k aa1 m ai n z ) ((pos "Mj$" ) ))
( "carnage"  n ( k aa1 n i jh ) ((pos "L@%" ) ))
( "carnal"  j ( k aa1 n l ) ((pos "OA%" ) ))
( "carnally"  a ( k aa1 n @ l ii ) ((pos "Pu%" ) ))
( "carnation"  n ( k aa1 n ei1 sh @ n ) ((pos "K6%" ) ))
( "carnations"  n ( k aa1 n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "carnival"  n ( k aa1 n i v @ l ) ((pos "M6%" ) ))
( "carnivals"  n ( k aa1 n i v @ l z ) ((pos "Mj%" ) ))
( "carnivore"  n ( k aa1 n i v oo r ) ((pos "K6%" ) ))
( "carnivores"  n ( k aa1 n i v oo z ) ((pos "Kj%" ) ))
( "carnivorous"  j ( k aa1 n i1 v @ r @ s ) ((pos "OA%" ) ))
( "carol"  v ( k a1 r @ l ) ((pos "H4%" "K6%" ) ))
( "carolled"  v ( k a1 r @ l d ) ((pos "Hc%" "Hd%" ) ))
( "caroller"  n ( k a1 r @ l @ r ) ((pos "K6%" ) ))
( "carollers"  n ( k a1 r @ l @ z ) ((pos "Kj%" ) ))
( "carolling"  v ( k a1 r @ l i ng ) ((pos "Hb%" ) ))
( "carols"  v ( k a1 r @ l z ) ((pos "Ha%" "Kj%" ) ))
( "carousal"  n ( k @1 r au1 z l ) ((pos "K6$" ) ))
( "carousals"  n ( k @1 r au1 z l z ) ((pos "Kj$" ) ))
( "carouse"  v ( k @1 r au1 z ) ((pos "H2%" ) (subcat )))
( "caroused"  v ( k @1 r au1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "carousel"  n ( k a2 r @ s e1 l ) ((pos "K6%" ) ))
( "carousels"  n ( k a2 r @ s e1 l z ) ((pos "Kj$" ) ))
( "carouses"  v ( k @1 r au1 z i z ) ((pos "Ha%" ) (subcat )))
( "carousing"  v ( k @1 r au1 z i ng ) ((pos "Hb%" ) (subcat )))
( "carp"  v ( k aa1 p ) ((pos "H0%" "K9%" ) (subcat "2A" )))
( "carpal"  n ( k aa1 p l ) ((pos "K6$" "OA$" ) ))
( "carpals"  n ( k aa1 p l z ) ((pos "Kj$" ) ))
( "carped"  v ( k aa1 p t ) ((pos "Hc$" "Hd$" ) (subcat "2A" )))
( "carpenter"  n ( k aa1 p i n t @ r ) ((pos "K6%" ) ))
( "carpenters"  n ( k aa1 p i n t @ z ) ((pos "Kj%" ) ))
( "carpentry"  n ( k aa1 p i n t r ii ) ((pos "L@%" ) ))
( "carpet"  v ( k aa1 p i t ) ((pos "H0%" "K6%" ) (subcat )))
( "carpet-beater"  n ( k aa1 p i t - b ii t @ r ) ((pos "K6%" ) ))
( "carpet-beaters"  n ( k aa1 p i t - b ii t @ z ) ((pos "Kj%" ) ))
( "carpet-knight"  n ( k aa1 p i t - n ai t ) ((pos "K6$" ) ))
( "carpet-knights"  n ( k aa1 p i t - n ai t s ) ((pos "Kj$" ) ))
( "carpet-slippers"  n ( k aa1 p i t - s l i p @ z ) ((pos "Kj%" ) ))
( "carpet-sweeper"  n ( k aa1 p i t - s w ii p @ r ) ((pos "K6%" ) ))
( "carpet-sweepers"  n ( k aa1 p i t - s w ii p @ z ) ((pos "Kj%" ) ))
( "carpetbag"  n ( k aa1 p i t b a g ) ((pos "K6%" ) ))
( "carpetbagger"  n ( k aa1 p i t b a g @ r ) ((pos "K6%" ) ))
( "carpetbaggers"  n ( k aa1 p i t b a g @ z ) ((pos "Kj%" ) ))
( "carpetbags"  n ( k aa1 p i t b a g z ) ((pos "Kj%" ) ))
( "carpeted"  v ( k aa1 p i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "carpeting"  v ( k aa1 p i t i ng ) ((pos "Hb%" ) (subcat )))
( "carpets"  v ( k aa1 p i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "carping"  v ( k aa1 p i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "carport"  n ( k aa1 p oo t ) ((pos "K6%" ) ))
( "carports"  n ( k aa1 p oo t s ) ((pos "Kj%" ) ))
( "carps"  v ( k aa1 p s ) ((pos "Ha%" ) (subcat "2A" )))
( "carriage"  n ( k a1 r i jh ) ((pos "M6%" ) ))
( "carriages"  n ( k a1 r i jh i z ) ((pos "Mj%" ) ))
( "carriageway"  n ( k a1 r i jh w ei ) ((pos "K6%" ) ))
( "carriageways"  n ( k a1 r i jh w ei z ) ((pos "Kj%" ) ))
( "carried"  v ( k a1 r i d ) ((pos "Jc%" "Jd%" ) (subcat "2B" "2C" "6A" "15A" "15B" )))
( "carrier"  n ( k a1 r i@ r ) ((pos "K6%" ) ))
( "carrier-bag"  n ( k a1 r i@ - b a g ) ((pos "K6%" ) ))
( "carrier-bags"  n ( k a1 r i@ - b a g z ) ((pos "Kj%" ) ))
( "carrier-pigeon"  n ( k a1 r i@ - p i jh @ n ) ((pos "K6%" ) ))
( "carrier-pigeons"  n ( k a1 r i@ - p i jh @ n z ) ((pos "Kj%" ) ))
( "carriers"  n ( k a1 r i@ z ) ((pos "Kj%" ) ))
( "carries"  v ( k a1 r i z ) ((pos "Ja%" "Kj%" ) (subcat "2B" "2C" "6A" "15A" "15B" )))
( "carrion"  n ( k a1 r i@ n ) ((pos "L@%" ) ))
( "carrion-crow"  n ( k a1 r i@ n - k r ou ) ((pos "K6%" ) ))
( "carrion-crows"  n ( k a1 r i@ n - k r ou z ) ((pos "Kj%" ) ))
( "carrot"  n ( k a1 r @ t ) ((pos "K6%" ) ))
( "carrots"  n ( k a1 r @ t s ) ((pos "Kj%" ) ))
( "carroty"  j ( k a1 r @ t ii ) ((pos "OA%" ) ))
( "carry"  v ( k a1 r ii ) ((pos "J3%" "K8%" ) (subcat "2B" "2C" "6A" "15A" "15B" )))
( "carrycot"  n ( k a1 r i k o t ) ((pos "K6%" ) ))
( "carrycots"  n ( k a1 r i k o t s ) ((pos "Kj%" ) ))
( "carrying"  v ( k a1 r i i ng ) ((pos "Jb%" ) (subcat "2B" "2C" "6A" "15A" "15B" )))
( "carryings-on"  n ( k a2 r i i ng z - o1 n ) ((pos "Kj%" ) ))
( "cars"  n ( k aa1 z ) ((pos "Kj%" ) ))
( "carsick"  j ( k aa1 s i k ) ((pos "OA%" ) ))
( "carsickness"  n ( k aa1 s i k n @ s ) ((pos "L@%" ) ))
( "cart"  v ( k aa1 t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "cart-track"  n ( k aa1 t - t r a k ) ((pos "K6%" ) ))
( "cart-tracks"  n ( k aa1 t - t r a k s ) ((pos "Kj%" ) ))
( "cartage"  n ( k aa1 t i jh ) ((pos "L@$" ) ))
( "carte blanche"  n ( k aa2 t - b l o1 n sh ) ((pos "Ki%" ) ))
( "carted"  v ( k aa1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cartel"  n ( k aa1 t e1 l ) ((pos "K6%" ) ))
( "cartels"  n ( k aa1 t e1 l z ) ((pos "Kj%" ) ))
( "carter"  n ( k aa1 t @ r ) ((pos "K6%" ) ))
( "carters"  n ( k aa1 t @ z ) ((pos "Kj%" ) ))
( "carthorse"  n ( k aa1 t h oo s ) ((pos "K6%" ) ))
( "carthorses"  n ( k aa1 t h oo s i z ) ((pos "Kj%" ) ))
( "cartilage"  n ( k aa1 t i l i jh ) ((pos "M6%" ) ))
( "cartilages"  n ( k aa1 t i l i jh i z ) ((pos "Mj$" ) ))
( "cartilaginous"  j ( k aa2 t i l a1 jh i n @ s ) ((pos "OA$" ) ))
( "carting"  v ( k aa1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cartload"  n ( k aa1 t l ou d ) ((pos "K6%" ) ))
( "cartloads"  n ( k aa1 t l ou d z ) ((pos "Kj%" ) ))
( "cartographer"  n ( k aa1 t o1 g r @ f @ r ) ((pos "K6%" ) ))
( "cartographers"  n ( k aa1 t o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "cartography"  n ( k aa1 t o1 g r @ f ii ) ((pos "L@%" ) ))
( "carton"  n ( k aa1 t n ) ((pos "K6%" ) ))
( "cartons"  n ( k aa1 t n z ) ((pos "Kj%" ) ))
( "cartoon"  v ( k aa1 t uu1 n ) ((pos "H0$" "K6%" ) ))
( "cartooned"  v ( k aa1 t uu1 n d ) ((pos "Hc$" "Hd$" ) ))
( "cartooning"  v ( k aa1 t uu1 n i ng ) ((pos "Hb$" ) ))
( "cartoonist"  n ( k aa1 t uu1 n i s t ) ((pos "K6%" ) ))
( "cartoonists"  n ( k aa1 t uu1 n i s t s ) ((pos "Kj%" ) ))
( "cartoons"  v ( k aa1 t uu1 n z ) ((pos "Ha$" "Kj%" ) ))
( "cartridge"  n ( k aa1 t r i jh ) ((pos "K6%" ) ))
( "cartridge-belt"  n ( k aa1 t r i jh - b e l t ) ((pos "K6%" ) ))
( "cartridge-belts"  n ( k aa1 t r i jh - b e l t s ) ((pos "Kj%" ) ))
( "cartridge-paper"  n ( k aa1 t r i jh - p ei p @ r ) ((pos "K6%" ) ))
( "cartridge-papers"  n ( k aa1 t r i jh - p ei p @ z ) ((pos "Kj%" ) ))
( "cartridges"  n ( k aa1 t r i jh i z ) ((pos "Kj%" ) ))
( "cartroad"  n ( k aa1 t r ou d ) ((pos "K6%" ) ))
( "cartroads"  n ( k aa1 t r ou d z ) ((pos "Kj%" ) ))
( "carts"  v ( k aa1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "cartwheel"  n ( k aa1 t w ii l ) ((pos "K6%" ) ))
( "cartwheels"  n ( k aa1 t w ii l z ) ((pos "Kj%" ) ))
( "carve"  v ( k aa1 v ) ((pos "J2%" ) (subcat "6A" "14" "15A" )))
( "carved"  v ( k aa1 v d ) ((pos "Jc%" "Jd%" ) (subcat "6A" "14" "15A" )))
( "carver"  n ( k aa1 v @ r ) ((pos "K6%" ) ))
( "carvers"  n ( k aa1 v @ z ) ((pos "Kj%" ) ))
( "carves"  v ( k aa1 v z ) ((pos "Ja%" ) (subcat "6A" "14" "15A" )))
( "carving"  v ( k aa1 v i ng ) ((pos "Jb%" "M6%" ) (subcat "6A" "14" "15A" )))
( "carving-fork"  n ( k aa1 v i ng - f oo k ) ((pos "K6%" ) ))
( "carving-forks"  n ( k aa1 v i ng - f oo k s ) ((pos "Kj%" ) ))
( "carving-knife"  n ( k aa1 v i ng - n ai f ) ((pos "Ki%" ) ))
( "carving-knives"  n ( k aa1 v i ng - n ai v z ) ((pos "Kj%" ) ))
( "carvings"  n ( k aa1 v i ng z ) ((pos "Mj%" ) ))
( "caryatid"  n ( k a2 r i a1 t i d ) ((pos "K6$" ) ))
( "caryatids"  n ( k a2 r i a1 t i d z ) ((pos "Kj$" ) ))
( "cascade"  v ( k a1 s k ei1 d ) ((pos "I2%" "K6%" ) ))
( "cascaded"  v ( k a1 s k ei1 d i d ) ((pos "Ic%" "Id%" ) ))
( "cascades"  v ( k a1 s k ei1 d z ) ((pos "Ia%" "Kj%" ) ))
( "cascading"  v ( k a1 s k ei1 d i ng ) ((pos "Ib%" ) ))
( "case"  v ( k ei1 s ) ((pos "H2%" "K6%" ) (subcat )))
( "case-hardened"  j ( k ei1 s - h aa d @ n d ) ((pos "OA%" ) ))
( "case-histories"  n ( k ei1 s - h i1 s t r i z ) ((pos "Kj%" ) ))
( "case-history"  n ( k ei1 s - h i1 s t r ii ) ((pos "K8%" ) ))
( "case-law"  n ( k ei1 s - l oo ) ((pos "K6%" ) ))
( "case-laws"  n ( k ei1 s - l oo z ) ((pos "Kj$" ) ))
( "casebook"  n ( k ei1 s b u k ) ((pos "K6%" ) ))
( "casebooks"  n ( k ei1 s b u k s ) ((pos "Kj$" ) ))
( "cased"  v ( k ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "casein"  n ( k ei1 s ii n ) ((pos "L@$" ) ))
( "casement"  n ( k ei1 s m @ n t ) ((pos "K6%" ) ))
( "casements"  n ( k ei1 s m @ n t s ) ((pos "Kj%" ) ))
( "cases"  v ( k ei1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "casework"  n ( k ei1 s w @@ k ) ((pos "K6%" ) ))
( "caseworks"  n ( k ei1 s w @@ k s ) ((pos "Kj$" ) ))
( "cash"  v ( k a1 sh ) ((pos "J1%" "L@%" ) (subcat "2C" "6A" "12B" )))
( "cashable"  j ( k a1 sh @ b l ) ((pos "OA%" ) ))
( "cashed"  v ( k a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "12B" )))
( "cashes"  v ( k a1 sh i z ) ((pos "Ja%" ) (subcat "2C" "6A" "12B" )))
( "cashew"  n ( k a1 sh uu ) ((pos "K6%" ) ))
( "cashews"  n ( k a1 sh uu z ) ((pos "Kj%" ) ))
( "cashier"  v ( k a1 sh i@1 r ) ((pos "H0%" "K6%" ) (subcat )))
( "cashiered"  v ( k a1 sh i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cashiering"  v ( k a1 sh i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "cashiers"  v ( k a1 sh i@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cashing"  v ( k a1 sh i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "12B" )))
( "cashmere"  n ( k a1 sh m i@1 r ) ((pos "L@%" ) ))
( "casing"  v ( k ei1 s i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "casings"  n ( k ei1 s i ng z ) ((pos "Kj%" ) ))
( "casino"  n ( k @1 s ii1 n ou ) ((pos "K6%" ) ))
( "casinos"  n ( k @1 s ii1 n ou z ) ((pos "Kj%" ) ))
( "cask"  n ( k aa1 s k ) ((pos "K6%" ) ))
( "casket"  n ( k aa1 s k i t ) ((pos "K6%" ) ))
( "caskets"  n ( k aa1 s k i t s ) ((pos "Kj%" ) ))
( "casks"  n ( k aa1 s k s ) ((pos "Kj%" ) ))
( "cassava"  n ( k @1 s aa1 v @ ) ((pos "L@%" ) ))
( "casserole"  n ( k a1 s @ r ou l ) ((pos "K6%" ) ))
( "casseroles"  n ( k a1 s @ r ou l z ) ((pos "Kj%" ) ))
( "cassette"  n ( k @1 s e1 t ) ((pos "K6%" ) ))
( "cassettes"  n ( k @1 s e1 t s ) ((pos "Kj%" ) ))
( "cassock"  n ( k a1 s @ k ) ((pos "K6%" ) ))
( "cassocks"  n ( k a1 s @ k s ) ((pos "Kj%" ) ))
( "cassowaries"  n ( k a1 s @ w e@ r i z ) ((pos "Kj$" ) ))
( "cassowary"  n ( k a1 s @ w e@ r ii ) ((pos "K8$" ) ))
( "cast"  v ( k aa1 s t ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "cast-iron"  j ( k aa1 s t - ai1 @ n ) ((pos "OA%" ) ))
( "cast-off"  j ( k aa1 s t - o f ) ((pos "Oq%" ) ))
( "cast-offs"  n ( k aa1 s t - o f s ) ((pos "Kj%" ) ))
( "castanets"  n ( k a2 s t @ n e1 t s ) ((pos "Kj%" ) ))
( "castaway"  n ( k aa1 s t @ w ei ) ((pos "K6%" ) ))
( "castaways"  n ( k aa1 s t @ w ei z ) ((pos "Kj%" ) ))
( "caste"  n ( k aa1 s t ) ((pos "M6%" ) ))
( "castellated"  j ( k a1 s t @ l ei t i d ) ((pos "OA%" ) ))
( "caster"  n ( k aa1 s t @ r ) ((pos "K6%" ) ))
( "casters"  n ( k aa1 s t @ z ) ((pos "Kj%" ) ))
( "castes"  n ( k aa1 s t s ) ((pos "Mj%" ) ))
( "castigate"  v ( k a1 s t i g ei t ) ((pos "H2%" ) (subcat )))
( "castigated"  v ( k a1 s t i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "castigates"  v ( k a1 s t i g ei t s ) ((pos "Ha%" ) (subcat )))
( "castigating"  v ( k a1 s t i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "castigation"  n ( k a2 s t i g ei1 sh @ n ) ((pos "M6%" ) ))
( "castigations"  n ( k a2 s t i g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "casting"  v ( k aa1 s t i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "castings"  n ( k aa1 s t i ng z ) ((pos "Kj%" ) ))
( "castle"  v ( k aa1 s l ) ((pos "I2%" "K6%" ) ))
( "castled"  v ( k aa1 s l d ) ((pos "Ic%" "Id%" ) ))
( "castles"  v ( k aa1 s l z ) ((pos "Ia%" "Kj%" ) ))
( "castling"  v ( k aa1 s l i ng ) ((pos "Ib%" ) ))
( "castor"  n ( k aa1 s t @ r ) ((pos "K6%" ) ))
( "castor oil"  n ( k aa2 s t @ r - oi1 l ) ((pos "L@%" ) ))
( "castors"  n ( k aa1 s t @ z ) ((pos "Kj%" ) ))
( "castrate"  v ( k a1 s t r ei1 t ) ((pos "H2%" ) (subcat )))
( "castrated"  v ( k a1 s t r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "castrates"  v ( k a1 s t r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "castrating"  v ( k a1 s t r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "castration"  n ( k a1 s t r ei1 sh @ n ) ((pos "K6%" ) ))
( "castrations"  n ( k a1 s t r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "casts"  v ( k aa1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "casual"  j ( k a1 zh u@ l ) ((pos "OA%" ) ))
( "casually"  a ( k a1 zh u@ l ii ) ((pos "Pu%" ) ))
( "casuals"  n ( k a1 zh u@ l z ) ((pos "Kj%" ) ))
( "casualties"  n ( k a1 zh u@ l t i z ) ((pos "Kj%" ) ))
( "casualty"  n ( k a1 zh u@ l t ii ) ((pos "K8%" ) ))
( "casuist"  n ( k a1 z y u i s t ) ((pos "K6$" ) ))
( "casuistic"  j ( k a2 z y u i1 s t i k ) ((pos "OA$" ) ))
( "casuistical"  j ( k a2 z y u i1 s t i k l ) ((pos "OA$" ) ))
( "casuistries"  n ( k a1 z y u i s t r i z ) ((pos "Mj$" ) ))
( "casuistry"  n ( k a1 z y u i s t r ii ) ((pos "M8%" ) ))
( "casuists"  n ( k a1 z y u i s t s ) ((pos "Kj$" ) ))
( "casus belli"  n ( k ei2 s @ s - b e1 l ai ) ((pos "Ki$" ) ))
( "cat"  n ( k a1 t ) ((pos "K6%" ) ))
( "cat-nap"  n ( k a1 t - n a p ) ((pos "K6%" ) ))
( "cat-naps"  n ( k a1 t - n a p s ) ((pos "Kj%" ) ))
( "cat-o'-nine-tails"  n ( k a2 t - @ - n ai1 n - t ei l z ) ((pos "Ki%" ) ))
( "cat-sleep"  n ( k a1 t - s l ii p ) ((pos "K6$" ) ))
( "cat-sleeps"  n ( k a1 t - s l ii p s ) ((pos "Kj$" ) ))
( "cataclysm"  n ( k a1 t @ k l i z @ m ) ((pos "K6%" ) ))
( "cataclysmic"  j ( k a2 t @ k l i1 z m i k ) ((pos "OA%" ) ))
( "cataclysms"  n ( k a1 t @ k l i z @ m z ) ((pos "Kj%" ) ))
( "catacombs"  n ( k a1 t @ k uu m z ) ((pos "Kj%" ) ))
( "catafalque"  n ( k a1 t @ f a l k ) ((pos "K6$" ) ))
( "catafalques"  n ( k a1 t @ f a l k s ) ((pos "Kj$" ) ))
( "catalepsy"  n ( k a1 t @ l e p s ii ) ((pos "L@$" ) ))
( "cataleptic"  n ( k a2 t @ l e1 p t i k ) ((pos "K6$" "OA$" ) ))
( "cataleptics"  n ( k a2 t @ l e1 p t i k s ) ((pos "Kj$" ) ))
( "catalogue"  v ( k a1 t @ l o g ) ((pos "H2%" "K6%" ) (subcat )))
( "catalogued"  v ( k a1 t @ l o g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "catalogues"  v ( k a1 t @ l o g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cataloguing"  v ( k a1 t @ l o g i ng ) ((pos "Hb%" ) (subcat )))
( "catalpa"  n ( k @1 t a1 l p @ ) ((pos "K6$" ) ))
( "catalpas"  n ( k @1 t a1 l p @ z ) ((pos "Kj$" ) ))
( "catalysis"  n ( k @1 t a1 l @ s i s ) ((pos "L@$" ) ))
( "catalyst"  n ( k a1 t @ l i s t ) ((pos "K6%" ) ))
( "catalysts"  n ( k a1 t @ l i s t s ) ((pos "Kj%" ) ))
( "catalytic"  j ( k a2 t @ l i1 t i k ) ((pos "OA$" ) ))
( "catamaran"  n ( k a2 t @ m @ r a1 n ) ((pos "K6%" ) ))
( "catamarans"  n ( k a2 t @ m @ r a1 n z ) ((pos "Kj%" ) ))
( "catapult"  v ( k a1 t @ p uh l t ) ((pos "H0%" "K6%" ) ))
( "catapulted"  v ( k a1 t @ p uh l t i d ) ((pos "Hc%" "Hd%" ) ))
( "catapulting"  v ( k a1 t @ p uh l t i ng ) ((pos "Hb%" ) ))
( "catapults"  v ( k a1 t @ p uh l t s ) ((pos "Ha%" "Kj%" ) ))
( "cataract"  n ( k a1 t @ r a k t ) ((pos "K6%" ) ))
( "cataracts"  n ( k a1 t @ r a k t s ) ((pos "Kj%" ) ))
( "catarrh"  n ( k @1 t aa1 r ) ((pos "L@%" ) ))
( "catastrophe"  n ( k @1 t a1 s t r @ f ii ) ((pos "K6%" ) ))
( "catastrophes"  n ( k @1 t a1 s t r @ f i z ) ((pos "Kj%" ) ))
( "catastrophic"  j ( k a2 t @ s t r o1 f i k ) ((pos "OA%" ) ))
( "catastrophically"  a ( k a2 t @ s t r o1 f i k l ii ) ((pos "Pu%" ) ))
( "catcall"  v ( k a1 t k aa l ) ((pos "I0%" "K6%" ) ))
( "catcalled"  v ( k a1 t k aa l d ) ((pos "Ic%" "Id%" ) ))
( "catcalling"  v ( k a1 t k aa l i ng ) ((pos "Ib%" ) ))
( "catcalls"  v ( k a1 t k aa l z ) ((pos "Ia%" "Kj%" ) ))
( "catch"  v ( k a1 ch ) ((pos "J5%" "K7%" ) (subcat "2C" "3A" "6A" "12C" "14" "15A" "15B" "19B" )))
( "catch-crop"  n ( k a1 ch - k r o p ) ((pos "K6$" ) ))
( "catch-crops"  n ( k a1 ch - k r o p s ) ((pos "Kj$" ) ))
( "catcher"  n ( k a1 ch @ r ) ((pos "K6%" ) ))
( "catchers"  n ( k a1 ch @ z ) ((pos "Kj%" ) ))
( "catches"  v ( k a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "12C" "14" "15A" "15B" "19B" )))
( "catchier"  j ( k a1 ch i@ r ) ((pos "Or$" ) ))
( "catchiest"  j ( k a1 ch i i s t ) ((pos "Os%" ) ))
( "catching"  v ( k a1 ch i ng ) ((pos "Jb%" "OA%" ) (subcat "2C" "3A" "6A" "12C" "14" "15A" "15B" "19B" )))
( "catchment"  n ( k a1 ch m @ n t ) ((pos "K6%" ) ))
( "catchment-area"  n ( k a1 ch m @ n t - e@ r i@ ) ((pos "K6%" ) ))
( "catchment-areas"  n ( k a1 ch m @ n t - e@ r i@ z ) ((pos "Kj%" ) ))
( "catchment-basin"  n ( k a1 ch m @ n t - b ei s n ) ((pos "K6%" ) ))
( "catchment-basins"  n ( k a1 ch m @ n t - b ei s n z ) ((pos "Kj%" ) ))
( "catchments"  n ( k a1 ch m @ n t s ) ((pos "Kj%" ) ))
( "catchpenny"  j ( k a1 ch p e n ii ) ((pos "OA%" ) ))
( "catchup"  n ( k a1 ch @ p ) ((pos "K6$" ) ))
( "catchups"  n ( k a1 ch @ p s ) ((pos "Kj$" ) ))
( "catchword"  n ( k a1 ch w @@ d ) ((pos "K6%" ) ))
( "catchwords"  n ( k a1 ch w @@ d z ) ((pos "Kj%" ) ))
( "catchy"  j ( k a1 ch ii ) ((pos "OD%" ) ))
( "catechism"  n ( k a1 t @ k i z @ m ) ((pos "M6%" ) ))
( "catechisms"  n ( k a1 t @ k i z @ m z ) ((pos "Mj$" ) ))
( "catechize"  v ( k a1 t @ k ai z ) ((pos "H2$" ) (subcat )))
( "catechized"  v ( k a1 t @ k ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "catechizes"  v ( k a1 t @ k ai z i z ) ((pos "Ha$" ) (subcat )))
( "catechizing"  v ( k a1 t @ k ai z i ng ) ((pos "Hb$" ) (subcat )))
( "categorical"  j ( k a2 t i g o1 r i k l ) ((pos "OA%" ) ))
( "categorically"  a ( k a2 t i g o1 r i k l ii ) ((pos "Pu%" ) ))
( "categories"  n ( k a1 t i g @ r i z ) ((pos "Kj%" ) ))
( "categorize"  v ( k a1 t i g @ r ai z ) ((pos "H2%" ) (subcat )))
( "categorized"  v ( k a1 t i g @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "categorizes"  v ( k a1 t i g @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "categorizing"  v ( k a1 t i g @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "category"  n ( k a1 t i g @ r ii ) ((pos "K8%" ) ))
( "cater"  v ( k ei1 t @ r ) ((pos "I0%" ) (subcat )))
( "catered"  v ( k ei1 t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "caterer"  n ( k ei1 t @ r @ r ) ((pos "K6%" ) ))
( "caterers"  n ( k ei1 t @ r @ z ) ((pos "Kj%" ) ))
( "catering"  v ( k ei1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "caterpillar"  n ( k a1 t @ p i l @ r ) ((pos "K6%" ) ))
( "caterpillars"  n ( k a1 t @ p i l @ z ) ((pos "Kj%" ) ))
( "caters"  v ( k ei1 t @ z ) ((pos "Ia%" ) (subcat )))
( "caterwaul"  v ( k a1 t @ w oo l ) ((pos "I0%" "K6%" ) ))
( "caterwauled"  v ( k a1 t @ w oo l d ) ((pos "Ic%" "Id%" ) ))
( "caterwauling"  v ( k a1 t @ w oo l i ng ) ((pos "Ib%" ) ))
( "caterwauls"  v ( k a1 t @ w oo l z ) ((pos "Ia%" "Kj%" ) ))
( "catfish"  n ( k a1 t f i sh ) ((pos "K9%" ) ))
( "catfishes"  n ( k a1 t f i sh i z ) ((pos "Kj%" ) ))
( "catgut"  n ( k a1 t g uh t ) ((pos "L@%" ) ))
( "catharses"  n ( k @1 th aa1 s ii z ) ((pos "Kj$" ) ))
( "catharsis"  n ( k @1 th aa1 s i s ) ((pos "Mi%" ) ))
( "cathartic"  n ( k @1 th aa1 t i k ) ((pos "M6%" ) ))
( "cathartics"  n ( k @1 th aa1 t i k s ) ((pos "Mj$" ) ))
( "cathedral"  n ( k @1 th ii1 d r @ l ) ((pos "K6%" ) ))
( "cathedrals"  n ( k @1 th ii1 d r @ l z ) ((pos "Kj%" ) ))
( "cathode"  n ( k a1 th ou d ) ((pos "K6%" ) ))
( "cathodes"  n ( k a1 th ou d z ) ((pos "Kj%" ) ))
( "catholic"  j ( k a1 th l i k ) ((pos "OA%" ) ))
( "catholicity"  n ( k a2 th @ l i1 s i t ii ) ((pos "L@$" ) ))
( "catkin"  n ( k a1 t k i n ) ((pos "K6%" ) ))
( "catkins"  n ( k a1 t k i n z ) ((pos "Kj%" ) ))
( "cats"  n ( k a1 t s ) ((pos "Kj%" ) ))
( "catsup"  n ( k a1 t s @ p ) ((pos "M6$" ) ))
( "catsups"  n ( k a1 t s @ p s ) ((pos "Mj$" ) ))
( "cattier"  j ( k a1 t i@ r ) ((pos "Or%" ) ))
( "cattiest"  j ( k a1 t i i s t ) ((pos "Os%" ) ))
( "cattiness"  n ( k a1 t i n @ s ) ((pos "L@%" ) ))
( "cattish"  j ( k a1 t i sh ) ((pos "OA$" ) ))
( "cattle"  n ( k a1 t l ) ((pos "Kj%" ) ))
( "cattle-cake"  n ( k a1 t l - k ei k ) ((pos "L@%" ) ))
( "cattleman"  n ( k a1 t l m @ n ) ((pos "Ki%" ) ))
( "cattlemen"  n ( k a1 t l m e n ) ((pos "Kj%" ) ))
( "catty"  j ( k a1 t ii ) ((pos "OD%" ) ))
( "catwalk"  n ( k a1 t w oo k ) ((pos "K6%" ) ))
( "catwalks"  n ( k a1 t w oo k s ) ((pos "Kj%" ) ))
( "caucus"  n ( k oo1 k @ s ) ((pos "K7%" ) ))
( "caucuses"  n ( k oo1 k @ s i z ) ((pos "Kj%" ) ))
( "caught"  v ( k oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "12C" "14" "15A" "15B" "19B" )))
( "caul"  n ( k oo1 l ) ((pos "K6$" ) ))
( "cauldron"  n ( k oo1 l d r @ n ) ((pos "K6%" ) ))
( "cauldrons"  n ( k oo1 l d r @ n z ) ((pos "Kj%" ) ))
( "cauliflower"  n ( k o1 l i f l au @ r ) ((pos "M6%" ) ))
( "cauliflowers"  n ( k o1 l i f l au @ z ) ((pos "Mj%" ) ))
( "caulk"  v ( k oo1 k ) ((pos "H0$" ) (subcat )))
( "caulked"  v ( k oo1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "caulking"  v ( k oo1 k i ng ) ((pos "Hb$" ) (subcat )))
( "caulks"  v ( k oo1 k s ) ((pos "Ha$" ) (subcat )))
( "cauls"  n ( k oo1 l z ) ((pos "Kj$" ) ))
( "causal"  j ( k oo1 z l ) ((pos "OA$" ) ))
( "causality"  n ( k oo1 z a1 l i t ii ) ((pos "L@%" ) ))
( "causation"  n ( k oo1 z ei1 sh @ n ) ((pos "L@%" ) ))
( "causative"  j ( k oo1 z @ t i v ) ((pos "OA%" ) ))
( "cause"  v ( k oo1 z ) ((pos "H2%" "M6%" ) (subcat "6A" "12A" "13A" )))
( "caused"  v ( k oo1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" )))
( "causeless"  j ( k oo1 z l @ s ) ((pos "OA$" ) ))
( "causerie"  n ( k ou1 z @ r ii ) ((pos "K6$" ) ))
( "causeries"  n ( k ou1 z @ r i z ) ((pos "Kj$" ) ))
( "causes"  v ( k oo1 z i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "12A" "13A" )))
( "causeway"  n ( k oo1 z w ei ) ((pos "K6%" ) ))
( "causeways"  n ( k oo1 z w ei z ) ((pos "Kj%" ) ))
( "causing"  v ( k oo1 z i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" )))
( "caustic"  j ( k oo1 s t i k ) ((pos "OA%" ) ))
( "caustically"  a ( k oo1 s t i k l ii ) ((pos "Pu%" ) ))
( "cauterize"  v ( k oo1 t @ r ai z ) ((pos "H2%" ) (subcat )))
( "cauterized"  v ( k oo1 t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cauterizes"  v ( k oo1 t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "cauterizing"  v ( k oo1 t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "caution"  v ( k oo1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" "14" )))
( "cautionary"  j ( k oo1 sh @ n r ii ) ((pos "OA%" ) ))
( "cautioned"  v ( k oo1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "cautioning"  v ( k oo1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "cautions"  v ( k oo1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "cautious"  j ( k oo1 sh @ s ) ((pos "OA%" ) ))
( "cautiously"  a ( k oo1 sh @ s l ii ) ((pos "Pu%" ) ))
( "cavalcade"  n ( k a2 v @ l k ei1 d ) ((pos "K6%" ) ))
( "cavalcades"  n ( k a2 v @ l k ei1 d z ) ((pos "Kj%" ) ))
( "cavalier"  n ( k a2 v @ l i@1 r ) ((pos "K6%" "OA%" ) ))
( "cavalierly"  a ( k a2 v @ l i@1 l ii ) ((pos "Pu$" ) ))
( "cavaliers"  n ( k a2 v @ l i@1 z ) ((pos "Kj%" ) ))
( "cavalries"  n ( k a1 v @ l r i z ) ((pos "Kj%" ) ))
( "cavalry"  n ( k a1 v @ l r ii ) ((pos "K8%" ) ))
( "cavalryman"  n ( k a1 v @ l r i m @ n ) ((pos "Ki%" ) ))
( "cavalrymen"  n ( k a1 v @ l r i m e n ) ((pos "Kj%" ) ))
( "cave"  v ( k ei1 v ) ((pos "J2%" "K6%" ) (subcat "2C" )))
( "cave-dweller"  n ( k ei1 v - d w e l @ r ) ((pos "K6%" ) ))
( "cave-dwellers"  n ( k ei1 v - d w e l @ z ) ((pos "Kj%" ) ))
( "cave-in"  n ( k ei1 v - i n ) ((pos "K6%" ) ))
( "cave-ins"  n ( k ei1 v - i n z ) ((pos "Kj%" ) ))
( "caveat"  n ( k ei1 v i a t ) ((pos "K6%" ) ))
( "caveats"  n ( k ei1 v i a t s ) ((pos "Kj%" ) ))
( "caved"  v ( k ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "caveman"  n ( k ei1 v m a n ) ((pos "Ki%" ) ))
( "cavemen"  n ( k ei1 v m e n ) ((pos "Kj%" ) ))
( "cavern"  n ( k a1 v @ n ) ((pos "K6%" ) ))
( "cavernous"  j ( k a1 v @ n @ s ) ((pos "OA%" ) ))
( "caverns"  n ( k a1 v @ n z ) ((pos "Kj%" ) ))
( "caves"  v ( k ei1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "caviar"  n ( k a1 v i aa r ) ((pos "L@%" ) ))
( "caviare"  n ( k a1 v i aa r ) ((pos "L@%" ) ))
( "cavil"  v ( k a1 v @ l ) ((pos "I4%" ) (subcat "2A" )))
( "cavilled"  v ( k a1 v @ l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "cavilling"  v ( k a1 v @ l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "cavils"  v ( k a1 v @ l z ) ((pos "Ia%" ) (subcat "2A" )))
( "caving"  v ( k ei1 v i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "cavities"  n ( k a1 v i t i z ) ((pos "Kj%" ) ))
( "cavity"  n ( k a1 v i t ii ) ((pos "K8%" ) ))
( "cavort"  v ( k @1 v oo1 t ) ((pos "I0%" ) ))
( "cavorted"  v ( k @1 v oo1 t i d ) ((pos "Ic%" "Id%" ) ))
( "cavorting"  v ( k @1 v oo1 t i ng ) ((pos "Ib%" ) ))
( "cavorts"  v ( k @1 v oo1 t s ) ((pos "Ia%" ) ))
( "caw"  v ( k oo1 ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "cawed"  v ( k oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cawing"  v ( k oo1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "caws"  v ( k oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "cayenne"  n ( k ei1 e1 n ) ((pos "L@%" ) ))
( "cayenne pepper"  n ( k ei2 e n - p e1 p @ r ) ((pos "L@%" ) ))
( "cayman"  n ( k ei1 m @ n ) ((pos "K6$" ) ))
( "caymans"  n ( k ei1 m @ n z ) ((pos "Kj$" ) ))
( "ca~non"  n ( k a1 n y @ n ) ((pos "K6$" ) ))
( "ca~nons"  n ( k a1 n y @ n z ) ((pos "Kj$" ) ))
( "cc"  n ( s ii2 s ii1 ) ((pos "Y]%" ) ))
( "cease"  v ( s ii1 s ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "6D" )))
( "cease-fire"  n ( s ii1 s - f ai @ r ) ((pos "K6%" ) ))
( "cease-fires"  n ( s ii1 s - f ai @ z ) ((pos "Kj%" ) ))
( "ceased"  v ( s ii1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "6D" )))
( "ceaseless"  j ( s ii1 s l @ s ) ((pos "OA%" ) ))
( "ceaselessly"  a ( s ii1 s l @ s l ii ) ((pos "Pu%" ) ))
( "ceases"  v ( s ii1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "6D" )))
( "ceasing"  v ( s ii1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "6D" )))
( "cedar"  n ( s ii1 d @ r ) ((pos "M6%" ) ))
( "cedars"  n ( s ii1 d @ z ) ((pos "Mj%" ) ))
( "cede"  v ( s ii1 d ) ((pos "H2%" ) (subcat "6A" )))
( "ceded"  v ( s ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cedes"  v ( s ii1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "cedilla"  n ( s i1 d i1 l @ ) ((pos "K6%" ) ))
( "cedillas"  n ( s i1 d i1 l @ z ) ((pos "Kj%" ) ))
( "ceding"  v ( s ii1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ceiling"  n ( s ii1 l i ng ) ((pos "K6%" ) ))
( "ceilings"  n ( s ii1 l i ng z ) ((pos "Kj%" ) ))
( "celandine"  n ( s e1 l @ n d ai n ) ((pos "K6%" ) ))
( "celandines"  n ( s e1 l @ n d ai n z ) ((pos "Kj%" ) ))
( "celebrant"  n ( s e1 l i b r @ n t ) ((pos "K6%" ) ))
( "celebrants"  n ( s e1 l i b r @ n t s ) ((pos "Kj%" ) ))
( "celebrate"  v ( s e1 l i b r ei t ) ((pos "H2%" ) (subcat )))
( "celebrated"  v ( s e1 l i b r ei t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "celebrates"  v ( s e1 l i b r ei t s ) ((pos "Ha%" ) (subcat )))
( "celebrating"  v ( s e1 l i b r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "celebration"  n ( s e2 l i b r ei1 sh @ n ) ((pos "M6%" ) ))
( "celebrations"  n ( s e2 l i b r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "celebrities"  n ( s i1 l e1 b r i t i z ) ((pos "Mj%" ) ))
( "celebrity"  n ( s i1 l e1 b r i t ii ) ((pos "M8%" ) ))
( "celerity"  n ( s i1 l e1 r i t ii ) ((pos "L@$" ) ))
( "celery"  n ( s e1 l @ r ii ) ((pos "L@%" ) ))
( "celestial"  j ( s i1 l e1 s t i@ l ) ((pos "OA%" ) ))
( "celibacy"  n ( s e1 l i b @ s ii ) ((pos "L@%" ) ))
( "celibate"  n ( s e1 l i b @ t ) ((pos "K6%" ) ))
( "celibates"  n ( s e1 l i b @ t s ) ((pos "Kj%" ) ))
( "cell"  n ( s e1 l ) ((pos "K6%" ) ))
( "cellar"  n ( s e1 l @ r ) ((pos "K6%" ) ))
( "cellarage"  n ( s e1 l @ r i jh ) ((pos "M6$" ) ))
( "cellarages"  n ( s e1 l @ r i jh i z ) ((pos "Mj$" ) ))
( "cellars"  n ( s e1 l @ z ) ((pos "Kj%" ) ))
( "cellist"  n ( ch e1 l i s t ) ((pos "K6%" ) ))
( "cellists"  n ( ch e1 l i s t s ) ((pos "Kj%" ) ))
( "cello"  n ( ch e1 l ou ) ((pos "K6%" ) ))
( "cellophane"  n ( s e1 l @ f ei n ) ((pos "L@%" ) ))
( "cellos"  n ( ch e1 l ou z ) ((pos "Kj%" ) ))
( "cells"  n ( s e1 l z ) ((pos "Kj%" ) ))
( "cellular"  j ( s e1 l y u l @ r ) ((pos "OA%" ) ))
( "celluloid"  n ( s e1 l y u l oi d ) ((pos "L@%" ) ))
( "cellulose"  n ( s e1 l y u l ou s ) ((pos "L@%" ) ))
( "cement"  v ( s i1 m e1 n t ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "cement-mixer"  n ( s i1 m e1 n t - m i k s @ r ) ((pos "K6%" ) ))
( "cement-mixers"  n ( s i1 m e1 n t - m i k s @ z ) ((pos "Kj%" ) ))
( "cemented"  v ( s i1 m e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cementing"  v ( s i1 m e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cements"  v ( s i1 m e1 n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "cemeteries"  n ( s e1 m @ t r i z ) ((pos "Kj%" ) ))
( "cemetery"  n ( s e1 m @ t r ii ) ((pos "K8%" ) ))
( "cenotaph"  n ( s e1 n @ t aa f ) ((pos "K6%" ) ))
( "cenotaphs"  n ( s e1 n @ t aa f s ) ((pos "Kj$" ) ))
( "censer"  n ( s e1 n s @ r ) ((pos "K6$" ) ))
( "censers"  n ( s e1 n s @ z ) ((pos "Kj$" ) ))
( "censor"  v ( s e1 n s @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "censored"  v ( s e1 n s @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "censoring"  v ( s e1 n s @ r i ng ) ((pos "Hb%" ) (subcat )))
( "censorious"  j ( s e1 n s oo1 r i@ s ) ((pos "OA%" ) ))
( "censors"  v ( s e1 n s @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "censorship"  n ( s e1 n s @ sh i p ) ((pos "K6%" ) ))
( "censorships"  n ( s e1 n s @ sh i p s ) ((pos "Kj$" ) ))
( "censure"  v ( s e1 n sh @ r ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "censured"  v ( s e1 n sh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "censures"  v ( s e1 n sh @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "censuring"  v ( s e1 n sh @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "census"  n ( s e1 n s @ s ) ((pos "K7%" ) ))
( "censuses"  n ( s e1 n s @ s i z ) ((pos "Kj%" ) ))
( "cent"  n ( s e1 n t ) ((pos "K6%" ) ))
( "centaur"  n ( s e1 n t oo r ) ((pos "K6$" ) ))
( "centaurs"  n ( s e1 n t oo z ) ((pos "Kj$" ) ))
( "centenarian"  n ( s e2 n t i n e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "centenarians"  n ( s e2 n t i n e@1 r i@ n z ) ((pos "Kj%" ) ))
( "centenaries"  n ( s e1 n t ii1 n @ r i z ) ((pos "Kj%" ) ))
( "centenary"  n ( s e1 n t ii1 n @ r ii ) ((pos "K8%" "OA%" ) ))
( "centennial"  n ( s e1 n t e1 n i@ l ) ((pos "K6%" "OA%" ) ))
( "centennially"  a ( s e1 n t e1 n i@ l ii ) ((pos "Pu$" ) ))
( "centennials"  n ( s e1 n t e1 n i@ l z ) ((pos "Kj$" ) ))
( "centi-"  j ( s e1 n t i - ) ((pos "U-%" ) ))
( "centigrade"  j ( s e1 n t i g r ei d ) ((pos "OA%" ) ))
( "centime"  n ( s o1 n t ii m ) ((pos "K6%" ) ))
( "centimes"  n ( s o1 n t ii m z ) ((pos "Kj%" ) ))
( "centimetre"  n ( s e1 n t i m ii2 t @ r ) ((pos "K6%" ) ))
( "centimetres"  n ( s e1 n t i m ii2 t @ z ) ((pos "Kj%" ) ))
( "centipede"  n ( s e1 n t i p ii d ) ((pos "K6%" ) ))
( "centipedes"  n ( s e1 n t i p ii d z ) ((pos "Kj%" ) ))
( "central"  n ( s e1 n t r @ l ) ((pos "K6$" "OA%" ) ))
( "centralization"  n ( s e2 n t r @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "centralizations"  n ( s e2 n t r @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "centralize"  v ( s e1 n t r @ l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "centralized"  v ( s e1 n t r @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "centralizes"  v ( s e1 n t r @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "centralizing"  v ( s e1 n t r @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "centrally"  a ( s e1 n t r @ l ii ) ((pos "Pu%" ) ))
( "centrals"  n ( s e1 n t r @ l z ) ((pos "Kj$" ) ))
( "centre"  v ( s e1 n t @ r ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "14" )))
( "centre-bit"  n ( s e1 n t @ - b i t ) ((pos "K6$" ) ))
( "centre-bits"  n ( s e1 n t @ - b i t s ) ((pos "Kj$" ) ))
( "centre-board"  n ( s e1 n t @ - b oo d ) ((pos "K6%" ) ))
( "centre-boards"  n ( s e1 n t @ - b oo d z ) ((pos "Kj%" ) ))
( "centred"  v ( s e1 n t @ d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" )))
( "centrepiece"  n ( s e1 n t @ p ii s ) ((pos "K6%" ) ))
( "centrepieces"  n ( s e1 n t @ p ii s i z ) ((pos "Kj$" ) ))
( "centres"  v ( s e1 n t @ z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" )))
( "centrifugal"  j ( s e1 n t r i1 f y u g l ) ((pos "OA%" ) ))
( "centrifuge"  n ( s e1 n t r i f y uu jh ) ((pos "K6%" ) ))
( "centrifuges"  n ( s e1 n t r i f y uu jh i z ) ((pos "Kj%" ) ))
( "centring"  v ( s e1 n t @ r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" )))
( "centripetal"  j ( s e1 n t r i1 p i t l ) ((pos "OA$" ) ))
( "cents"  n ( s e1 n t s ) ((pos "Kj%" ) ))
( "centuries"  n ( s e1 n ch @ r i z ) ((pos "Kj%" ) ))
( "centurion"  n ( s e1 n t y u@1 r i@ n ) ((pos "K6%" ) ))
( "centurions"  n ( s e1 n t y u@1 r i@ n z ) ((pos "Kj%" ) ))
( "century"  n ( s e1 n ch @ r ii ) ((pos "K8%" ) ))
( "ceramic"  j ( s @1 r a1 m i k ) ((pos "OA%" ) ))
( "ceramics"  n ( s @1 r a1 m i k s ) ((pos "Lk%" ) ))
( "cereal"  n ( s i@1 r i@ l ) ((pos "M6%" ) ))
( "cereals"  n ( s i@1 r i@ l z ) ((pos "Mj%" ) ))
( "cerebral"  j ( s e1 r @ b r @ l ) ((pos "OA%" ) ))
( "cerebration"  n ( s e2 r i b r ei1 sh @ n ) ((pos "L@$" ) ))
( "ceremonial"  n ( s e2 r i m ou1 n i@ l ) ((pos "M6%" "OA%" ) ))
( "ceremonially"  a ( s e2 r i m ou1 n i@ l ii ) ((pos "Pu%" ) ))
( "ceremonials"  n ( s e2 r i m ou1 n i@ l z ) ((pos "Mj%" ) ))
( "ceremonies"  n ( s e1 r i m @ n i z ) ((pos "Mj%" ) ))
( "ceremonious"  j ( s e2 r i m ou1 n i@ s ) ((pos "OA%" ) ))
( "ceremoniously"  a ( s e2 r i m ou1 n i@ s l ii ) ((pos "Pu%" ) ))
( "ceremony"  n ( s e1 r i m @ n ii ) ((pos "M8%" ) ))
( "cerise"  n ( s @1 r ii1 z ) ((pos "M6%" "OA%" ) ))
( "cerises"  n ( s @1 r ii1 z i z ) ((pos "Mj$" ) ))
( "cert"  n ( s @@1 t ) ((pos "K6%" ) ))
( "certain"  j ( s @@1 t n ) ((pos "OA*" ) ))
( "certainly"  a ( s @@1 t n l ii ) ((pos "Pu%" ) ))
( "certainties"  n ( s @@1 t n t i z ) ((pos "Mj%" ) ))
( "certainty"  n ( s @@1 t n t ii ) ((pos "M8%" ) ))
( "certifiable"  j ( s @@2 t i f ai1 @ b l ) ((pos "OA%" ) ))
( "certificate"  n ( s @1 t i1 f i k @ t ) ((pos "K6%" ) ))
( "certificate"  v ( s @1 t i1 f i k ei t ) ((pos "H2$" ) ))
( "certificated"  v ( s @1 t i1 f i k ei t i d ) ((pos "Hc$" "Hd$" "OA%" ) ))
( "certificates"  n ( s @1 t i1 f i k @ t s ) ((pos "Kj%" ) ))
( "certificates"  v ( s @1 t i1 f i k ei t s ) ((pos "Ha$" ) ))
( "certificating"  v ( s @1 t i1 f i k ei t i ng ) ((pos "Hb$" ) ))
( "certification"  n ( s @@2 t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "certifications"  n ( s @@2 t i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "certified"  v ( s @@1 t i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" "16B" )))
( "certifies"  v ( s @@1 t i f ai z ) ((pos "Ja%" ) (subcat "3A" "6A" "9" "16B" )))
( "certify"  v ( s @@1 t i f ai ) ((pos "J3%" ) (subcat "3A" "6A" "9" "16B" )))
( "certifying"  v ( s @@1 t i f ai i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "9" "16B" )))
( "certitude"  n ( s @@1 t i t y uu d ) ((pos "L@%" ) ))
( "certs"  n ( s @@1 t s ) ((pos "Kj%" ) ))
( "cerulean"  j ( s i1 r uu1 l i@ n ) ((pos "OA$" ) ))
( "cervical"  j ( s @@1 v ai1 k l ) ((pos "OA%" ) ))
( "cervices"  n ( s @@1 v i s ii z ) ((pos "Kj$" ) ))
( "cervix"  n ( s @@1 v i k s ) ((pos "K7%" ) ))
( "cervixes"  n ( s @@1 v i k s i z ) ((pos "Kj$" ) ))
( "cessation"  n ( s e1 s ei1 sh @ n ) ((pos "L@%" ) ))
( "cession"  n ( s e1 sh @ n ) ((pos "M6$" ) ))
( "cessions"  n ( s e1 sh @ n z ) ((pos "Mj$" ) ))
( "cesspit"  n ( s e1 s p i t ) ((pos "K6%" ) ))
( "cesspits"  n ( s e1 s p i t s ) ((pos "Kj%" ) ))
( "cesspool"  n ( s e1 s p uu l ) ((pos "K6%" ) ))
( "cesspools"  n ( s e1 s p uu l z ) ((pos "Kj%" ) ))
( "cf"  n ( s ii2 e1 f ) ((pos "Y~%" ) ))
( "ch^ateau"  n ( sh a1 t ou ) ((pos "Ki%" ) ))
( "chafe"  v ( ch ei1 f ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "chafed"  v ( ch ei1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "chafes"  v ( ch ei1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "chaff"  v ( ch aa1 f ) ((pos "H0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "chaffed"  v ( ch aa1 f t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "6A" )))
( "chaffinch"  n ( ch a1 f i n ch ) ((pos "K7%" ) ))
( "chaffinches"  n ( ch a1 f i n ch i z ) ((pos "Kj%" ) ))
( "chaffing"  v ( ch aa1 f i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" )))
( "chaffs"  v ( ch aa1 f s ) ((pos "Ha%" ) (subcat "2A" "2C" "6A" )))
( "chafing"  v ( ch ei1 f i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "chafing dish"  n ( ch ei1 f i ng - d i sh ) ((pos "K7$" ) ))
( "chafing dishes"  n ( ch ei1 f i ng - d i sh i z ) ((pos "Kj$" ) ))
( "chagrin"  v ( sh a1 g r i n ) ((pos "H0$" "L@%" ) (subcat )))
( "chagrined"  v ( sh a1 g r i n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "chagrining"  v ( sh a1 g r i n i ng ) ((pos "Hb$" ) (subcat )))
( "chagrins"  v ( sh a1 g r i n z ) ((pos "Ha$" ) (subcat )))
( "chain"  v ( ch ei1 n ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "chain-armour"  n ( ch ei1 n - aa1 m @ r ) ((pos "L@%" ) ))
( "chain-gang"  n ( ch ei1 n - g a ng ) ((pos "K6%" ) ))
( "chain-gangs"  n ( ch ei1 n - g a ng z ) ((pos "Kj%" ) ))
( "chain-letter"  n ( ch ei1 n - l e t @ r ) ((pos "K6%" ) ))
( "chain-letters"  n ( ch ei1 n - l e t @ z ) ((pos "Kj%" ) ))
( "chain-mail"  n ( ch ei1 n - m ei l ) ((pos "L@%" ) ))
( "chain-smoker"  n ( ch ei1 n - s m ou k @ r ) ((pos "K6%" ) ))
( "chain-smokers"  n ( ch ei1 n - s m ou k @ z ) ((pos "Kj%" ) ))
( "chain-stitch"  n ( ch ei1 n - s t i ch ) ((pos "K7%" ) ))
( "chain-stitches"  n ( ch ei1 n - s t i ch i z ) ((pos "Kj$" ) ))
( "chain-store"  n ( ch ei1 n - s t oo r ) ((pos "K6%" ) ))
( "chain-stores"  n ( ch ei1 n - s t oo z ) ((pos "Kj%" ) ))
( "chained"  v ( ch ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "chaining"  v ( ch ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "chains"  v ( ch ei1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "chair"  v ( ch e@1 r ) ((pos "H0%" "K6%" ) (subcat )))
( "chair-lift"  n ( ch e@1 - l i f t ) ((pos "K6%" ) ))
( "chair-lifts"  n ( ch e@1 - l i f t s ) ((pos "Kj%" ) ))
( "chaired"  v ( ch e@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chairing"  v ( ch e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "chairman"  n ( ch e@1 m @ n ) ((pos "Ki%" ) ))
( "chairmanship"  n ( ch e@1 m @ n sh i p ) ((pos "L@%" ) ))
( "chairmen"  n ( ch e@1 m @ n ) ((pos "Kj%" ) ))
( "chairs"  v ( ch e@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "chaise"  n ( sh ei1 z ) ((pos "K6%" ) ))
( "chaise longue"  n ( sh ei2 z - l o1 ng ) ((pos "K6%" ) ))
( "chaise longues"  n ( sh ei2 z - l o1 ng z ) ((pos "Kj%" ) ))
( "chaises"  n ( sh ei1 z i z ) ((pos "Kj%" ) ))
( "chalet"  n ( sh a1 l ei ) ((pos "K6%" ) ))
( "chalets"  n ( sh a1 l ei z ) ((pos "Kj%" ) ))
( "chalice"  n ( ch a1 l i s ) ((pos "K6%" ) ))
( "chalices"  n ( ch a1 l i s i z ) ((pos "Kj%" ) ))
( "chalk"  v ( ch oo1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "chalked"  v ( ch oo1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chalkier"  j ( ch oo1 k i@ r ) ((pos "Or%" ) ))
( "chalkiest"  j ( ch oo1 k i i s t ) ((pos "Os%" ) ))
( "chalking"  v ( ch oo1 k i ng ) ((pos "Hb%" ) (subcat )))
( "chalkpit"  n ( ch oo1 k p i t ) ((pos "K6%" ) ))
( "chalkpits"  n ( ch oo1 k p i t s ) ((pos "Kj%" ) ))
( "chalks"  v ( ch oo1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "chalky"  j ( ch oo1 k ii ) ((pos "OD%" ) ))
( "challenge"  v ( ch a1 l @ n jh ) ((pos "H2%" "K6%" ) (subcat "6A" "14" )))
( "challenged"  v ( ch a1 l @ n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "challenger"  n ( ch a1 l @ n jh @ r ) ((pos "K6%" ) ))
( "challengers"  n ( ch a1 l @ n jh @ z ) ((pos "Kj%" ) ))
( "challenges"  v ( ch a1 l @ n jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "challenging"  v ( ch a1 l @ n jh i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "chamber"  n ( ch ei1 m b @ r ) ((pos "K6%" ) ))
( "chamberlain"  n ( ch ei1 m b @ l i n ) ((pos "K6%" ) ))
( "chamberlains"  n ( ch ei1 m b @ l i n z ) ((pos "Kj%" ) ))
( "chambermaid"  n ( ch ei1 m b @ m ei d ) ((pos "K6%" ) ))
( "chambermaids"  n ( ch ei1 m b @ m ei d z ) ((pos "Kj%" ) ))
( "chamberpot"  n ( ch ei1 m b @ p o t ) ((pos "K6%" ) ))
( "chamberpots"  n ( ch ei1 m b @ p o t s ) ((pos "Kj%" ) ))
( "chambers"  n ( ch ei1 m b @ z ) ((pos "Kj%" ) ))
( "chameleon"  n ( k @1 m ii1 l i@ n ) ((pos "K6%" ) ))
( "chameleons"  n ( k @1 m ii1 l i@ n z ) ((pos "Kj%" ) ))
( "chammy-leather"  n ( sh a1 m i - l e dh @ r ) ((pos "K6%" ) ))
( "chammy-leathers"  n ( sh a1 m i - l e dh @ z ) ((pos "Kj%" ) ))
( "chamois"  n ( sh a1 m w aa ) ((pos "K7%" ) ))
( "chamois-leather"  n ( sh a1 m i - l e dh @ r ) ((pos "K6$" ) ))
( "chamois-leathers"  n ( sh a1 m i - l e dh @ z ) ((pos "Kj$" ) ))
( "chamoises"  n ( sh a1 m w aa z ) ((pos "Kj$" ) ))
( "champ"  v ( ch a1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "4A" )))
( "champagne"  n ( sh a1 m p ei1 n ) ((pos "K6%" ) ))
( "champagnes"  n ( sh a1 m p ei1 n z ) ((pos "Kj%" ) ))
( "champed"  v ( ch a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" )))
( "champing"  v ( ch a1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" )))
( "champion"  v ( ch a1 m p i@ n ) ((pos "H0%" "K6%" "OA%" "Pu%" ) (subcat )))
( "championed"  v ( ch a1 m p i@ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "championing"  v ( ch a1 m p i@ n i ng ) ((pos "Hb%" ) (subcat )))
( "champions"  v ( ch a1 m p i@ n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "championship"  n ( ch a1 m p i@ n sh i p ) ((pos "M6%" ) ))
( "championships"  n ( ch a1 m p i@ n sh i p s ) ((pos "Mj%" ) ))
( "champs"  v ( ch a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4A" )))
( "chance"  v ( ch aa1 n s ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" "4E" "6A" )))
( "chanced"  v ( ch aa1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4E" "6A" )))
( "chancel"  n ( ch aa1 n s l ) ((pos "K6%" ) ))
( "chancelleries"  n ( ch aa1 n s @ l r i z ) ((pos "Kj$" ) ))
( "chancellery"  n ( ch aa1 n s @ l r ii ) ((pos "K8%" ) ))
( "chancellor"  n ( ch aa1 n s @ l @ r ) ((pos "K6%" ) ))
( "chancellors"  n ( ch aa1 n s @ l @ z ) ((pos "Kj%" ) ))
( "chancels"  n ( ch aa1 n s l z ) ((pos "Kj%" ) ))
( "chanceries"  n ( ch aa1 n s @ r i z ) ((pos "Kj$" ) ))
( "chancery"  n ( ch aa1 n s @ r ii ) ((pos "K8%" ) ))
( "chances"  v ( ch aa1 n s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "4E" "6A" )))
( "chancing"  v ( ch aa1 n s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4E" "6A" )))
( "chancy"  j ( ch aa1 n s ii ) ((pos "OA%" ) ))
( "chandelier"  n ( sh a2 n d @ l i@1 r ) ((pos "K6%" ) ))
( "chandeliers"  n ( sh a2 n d @ l i@1 z ) ((pos "Kj%" ) ))
( "chandler"  n ( ch aa1 n d l @ r ) ((pos "K6%" ) ))
( "chandlers"  n ( ch aa1 n d l @ z ) ((pos "Kj%" ) ))
( "change"  v ( ch ei1 n jh ) ((pos "J2*" "M6*" ) (subcat "2A" "2C" "6A" )))
( "changeable"  j ( ch ei1 n jh @ b @ l ) ((pos "OA%" ) ))
( "changeableness"  n ( ch ei1 n jh @ b @ l n @ s ) ((pos "L@%" ) ))
( "changed"  v ( ch ei1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "changeful"  j ( ch ei1 n jh f @ l ) ((pos "OA%" ) ))
( "changeless"  j ( ch ei1 n jh l @ s ) ((pos "OA%" ) ))
( "changeling"  n ( ch ei1 n jh l i ng ) ((pos "K6$" ) ))
( "changelings"  n ( ch ei1 n jh l i ng z ) ((pos "Kj$" ) ))
( "changeover"  n ( ch ei1 n jh ou v @ r ) ((pos "K6%" ) ))
( "changeovers"  n ( ch ei1 n jh ou v @ z ) ((pos "Kj%" ) ))
( "changes"  v ( ch ei1 n jh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "changing"  v ( ch ei1 n jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "channel"  v ( ch a1 n l ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "channelled"  v ( ch a1 n l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "channelling"  v ( ch a1 n @ l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "channels"  v ( ch a1 n l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "chant"  v ( ch aa1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "chanted"  v ( ch aa1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "chanting"  v ( ch aa1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "chants"  v ( ch aa1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "chaos"  n ( k ei1 o s ) ((pos "L@%" ) ))
( "chaotic"  j ( k ei1 o1 t i k ) ((pos "OA%" ) ))
( "chaotically"  a ( k ei1 o1 t i k l ii ) ((pos "Pu%" ) ))
( "chap"  v ( ch a1 p ) ((pos "J4$" "K6%" ) (subcat "2A" )))
( "chapel"  n ( ch a1 p l ) ((pos "K6%" ) ))
( "chapelgoer"  n ( ch a1 p l g ou @ r ) ((pos "K6%" ) ))
( "chapelgoers"  n ( ch a1 p l g ou @ z ) ((pos "Kj%" ) ))
( "chapels"  n ( ch a1 p l z ) ((pos "Kj%" ) ))
( "chaperon"  v ( sh a1 p @ r ou n ) ((pos "H0%" "K6%" ) (subcat )))
( "chaperoned"  v ( sh a1 p @ r ou n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chaperoning"  v ( sh a1 p @ r ou n i ng ) ((pos "Hb%" ) (subcat )))
( "chaperons"  v ( sh a1 p @ r ou n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "chapfallen"  j ( ch a1 p f oo l @ n ) ((pos "OA$" ) ))
( "chaplain"  n ( ch a1 p l i n ) ((pos "K6%" ) ))
( "chaplaincies"  n ( ch a1 p l i n s i z ) ((pos "Kj$" ) ))
( "chaplaincy"  n ( ch a1 p l i n s ii ) ((pos "K8%" ) ))
( "chaplains"  n ( ch a1 p l i n z ) ((pos "Kj%" ) ))
( "chaplet"  n ( ch a1 p l i t ) ((pos "K6$" ) ))
( "chaplets"  n ( ch a1 p l i t s ) ((pos "Kj$" ) ))
( "chapman"  n ( ch a1 p m @ n ) ((pos "Ki$" ) ))
( "chapmen"  n ( ch a1 p m @ n ) ((pos "Kj$" ) ))
( "chapped"  v ( ch a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "chapping"  v ( ch a1 p i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "chaps"  v ( ch a1 p s ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "chapter"  n ( ch a1 p t @ r ) ((pos "K6%" ) ))
( "chapterhouse"  n ( ch a1 p t @ h au s ) ((pos "K6%" ) ))
( "chapterhouses"  n ( ch a1 p t @ h au z i z ) ((pos "Kj%" ) ))
( "chapters"  n ( ch a1 p t @ z ) ((pos "Kj%" ) ))
( "char"  v ( ch aa1 r ) ((pos "J4%" "M6%" ) (subcat "2A" )))
( "char`abanc"  n ( sh a1 r @ b a ng ) ((pos "K6$" ) ))
( "char`abancs"  n ( sh a1 r @ b a ng z ) ((pos "Kj$" ) ))
( "charabanc"  n ( sh a1 r @ b a ng ) ((pos "K6%" ) ))
( "charabancs"  n ( sh a1 r @ b a ng z ) ((pos "Kj%" ) ))
( "character"  n ( k a1 r @ k t @ r ) ((pos "M6%" ) ))
( "characteristic"  n ( k a2 r @ k t @ r i1 s t i k ) ((pos "K6%" "OA%" ) ))
( "characteristically"  a ( k a2 r @ k t @ r i1 s t i k l ii ) ((pos "Pu%" ) ))
( "characteristics"  n ( k a2 r @ k t @ r i1 s t i k s ) ((pos "Kj%" ) ))
( "characterization"  n ( k a2 r @ k t @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "characterize"  v ( k a1 r @ k t @ r ai z ) ((pos "H2%" ) (subcat )))
( "characterized"  v ( k a1 r @ k t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "characterizes"  v ( k a1 r @ k t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "characterizing"  v ( k a1 r @ k t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "characterless"  j ( k a1 r @ k t @ l @ s ) ((pos "OA%" ) ))
( "characters"  n ( k a1 r @ k t @ z ) ((pos "Mj%" ) ))
( "charade"  n ( sh @1 r aa1 d ) ((pos "K6%" ) ))
( "charades"  n ( sh @1 r aa1 d z ) ((pos "Kj%" ) ))
( "charcoal"  n ( ch aa1 k ou l ) ((pos "L@%" ) ))
( "charcoal-burner"  n ( ch aa1 k ou l - b @@2 n @ r ) ((pos "K6%" ) ))
( "charcoal-burners"  n ( ch aa1 k ou l - b @@2 n @ z ) ((pos "Kj%" ) ))
( "chard"  n ( ch aa1 d ) ((pos "K6$" ) ))
( "chards"  n ( ch aa1 d z ) ((pos "Kj$" ) ))
( "charg_e d'affaires"  n ( sh aa2 zh ei - d a f e@1 r ) ((pos "Ki%" ) ))
( "charg_es d'affaires"  n ( sh aa2 zh ei - d a f e@1 r ) ((pos "Kj$" ) ))
( "charge"  v ( ch aa1 jh ) ((pos "J2%" "M6%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" "15B" )))
( "charge-account"  n ( ch aa1 jh - @ k au n t ) ((pos "K6%" ) ))
( "charge-accounts"  n ( ch aa1 jh - @ k au n t s ) ((pos "Kj%" ) ))
( "charge-sheet"  n ( ch aa1 jh - sh ii t ) ((pos "K6%" ) ))
( "charge-sheets"  n ( ch aa1 jh - sh ii t s ) ((pos "Kj%" ) ))
( "chargeable"  j ( ch aa1 jh @ b l ) ((pos "OA%" ) ))
( "charged"  v ( ch aa1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" "15B" )))
( "charger"  n ( ch aa1 jh @ r ) ((pos "K6%" ) ))
( "chargers"  n ( ch aa1 jh @ z ) ((pos "Kj%" ) ))
( "charges"  v ( ch aa1 jh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" "15B" )))
( "charging"  v ( ch aa1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" "15B" )))
( "charily"  a ( ch e@1 r i l ii ) ((pos "Pu%" ) ))
( "chariot"  n ( ch a1 r i@ t ) ((pos "K6%" ) ))
( "charioteer"  n ( ch a2 r i@ t i@1 r ) ((pos "K6%" ) ))
( "charioteers"  n ( ch a2 r i@ t i@1 z ) ((pos "Kj%" ) ))
( "chariots"  n ( ch a1 r i@ t s ) ((pos "Kj%" ) ))
( "charisma"  n ( k @1 r i1 z m @ ) ((pos "K6%" ) ))
( "charismas"  n ( k @1 r i1 z m @ z ) ((pos "Kj$" ) ))
( "charismatic"  j ( k a2 r i z m a1 t i k ) ((pos "OA%" ) ))
( "charitable"  j ( ch a1 r i t @ b l ) ((pos "OA%" ) ))
( "charitably"  a ( ch a1 r i t @ b l ii ) ((pos "Pu%" ) ))
( "charities"  n ( ch a1 r i t i z ) ((pos "Mj%" ) ))
( "charity"  n ( ch a1 r i t ii ) ((pos "M8%" ) ))
( "charivari"  n ( sh aa2 r i v aa1 r ii ) ((pos "L@$" ) ))
( "charladies"  n ( ch aa1 l ei d i z ) ((pos "Kj%" ) ))
( "charlady"  n ( ch aa1 l ei d ii ) ((pos "K8%" ) ))
( "charlatan"  n ( sh aa1 l @ t @ n ) ((pos "K6%" ) ))
( "charlatans"  n ( sh aa1 l @ t @ n z ) ((pos "Kj%" ) ))
( "charlock"  n ( ch aa1 l o k ) ((pos "L@$" ) ))
( "charm"  v ( ch aa1 m ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "charmed"  v ( ch aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "charmer"  n ( ch aa1 m @ r ) ((pos "K6%" ) ))
( "charmers"  n ( ch aa1 m @ z ) ((pos "Kj%" ) ))
( "charming"  v ( ch aa1 m i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" )))
( "charmingly"  a ( ch aa1 m i ng l ii ) ((pos "Pu%" ) ))
( "charms"  v ( ch aa1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "charnel house"  n ( ch aa1 n l - h au s ) ((pos "K6$" ) ))
( "charnel houses"  n ( ch aa1 n l - h au z i z ) ((pos "Kj$" ) ))
( "charred"  v ( ch aa1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "charring"  v ( ch aa1 r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "chars"  v ( ch aa1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "chart"  v ( ch aa1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "charted"  v ( ch aa1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "charter"  v ( ch aa1 t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "charter-parties"  n ( ch aa1 t @ - p aa2 t i z ) ((pos "Kj$" ) ))
( "charter-party"  n ( ch aa1 t @ - p aa2 t ii ) ((pos "K8$" ) ))
( "chartered"  v ( ch aa1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chartering"  v ( ch aa1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "charters"  v ( ch aa1 t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "charting"  v ( ch aa1 t i ng ) ((pos "Hb%" ) (subcat )))
( "chartreuse"  n ( sh aa1 t r @@1 z ) ((pos "L@%" ) ))
( "charts"  v ( ch aa1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "charwoman"  n ( ch aa1 w u m @ n ) ((pos "Ki%" ) ))
( "charwomen"  n ( ch aa1 w i m i n ) ((pos "Kj%" ) ))
( "chary"  j ( ch e@1 r ii ) ((pos "OA%" ) ))
( "chase"  v ( ch ei1 s ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "6A" "15A" )))
( "chased"  v ( ch ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "chaser"  n ( ch ei1 s @ r ) ((pos "K6%" ) ))
( "chasers"  n ( ch ei1 s @ z ) ((pos "Kj%" ) ))
( "chases"  v ( ch ei1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "15A" )))
( "chasing"  v ( ch ei1 s i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "chasm"  n ( k a1 z @ m ) ((pos "K6%" ) ))
( "chasms"  n ( k a1 z @ m z ) ((pos "Kj%" ) ))
( "chassis"  n ( sh a1 s ii ) ((pos "K9%" ) ))
( "chaste"  j ( ch ei1 s t ) ((pos "OA%" ) ))
( "chastely"  a ( ch ei1 s t l ii ) ((pos "Pu%" ) ))
( "chasten"  v ( ch ei1 s n ) ((pos "H0%" ) (subcat )))
( "chastened"  v ( ch ei1 s n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chastening"  v ( ch ei1 s n i ng ) ((pos "Hb%" ) (subcat )))
( "chastens"  v ( ch ei1 s n z ) ((pos "Ha%" ) (subcat )))
( "chastise"  v ( ch a1 s t ai1 z ) ((pos "H2%" ) (subcat )))
( "chastised"  v ( ch a1 s t ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chastisement"  n ( ch a1 s t ai1 z m @ n t ) ((pos "L@%" ) ))
( "chastises"  v ( ch a1 s t ai1 z i z ) ((pos "Ha%" ) (subcat )))
( "chastising"  v ( ch a1 s t ai1 z i ng ) ((pos "Hb%" ) (subcat )))
( "chastity"  n ( ch a1 s t i t ii ) ((pos "L@%" ) ))
( "chasuble"  n ( ch a1 z y u b l ) ((pos "K6$" ) ))
( "chasubles"  n ( ch a1 z y u b l z ) ((pos "Kj$" ) ))
( "chat"  v ( ch a1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "chateaux"  n ( sh a1 t ou z ) ((pos "Kj%" ) ))
( "chatelaine"  n ( sh a1 t @ l ei n ) ((pos "K6$" ) ))
( "chatelaines"  n ( sh a1 t @ l ei n z ) ((pos "Kj$" ) ))
( "chats"  v ( ch a1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "chatted"  v ( ch a1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "chattel"  n ( ch a1 t l ) ((pos "K6$" ) ))
( "chattels"  n ( ch a1 t l z ) ((pos "Kj%" ) ))
( "chatter"  v ( ch a1 t @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "chatterbox"  n ( ch a1 t @ b o k s ) ((pos "K7%" ) ))
( "chatterboxes"  n ( ch a1 t @ b o k s i z ) ((pos "Kj%" ) ))
( "chattered"  v ( ch a1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "chattering"  v ( ch a1 t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "chatters"  v ( ch a1 t @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "chattier"  j ( ch a1 t i@ r ) ((pos "Or%" ) ))
( "chattiest"  j ( ch a1 t i i s t ) ((pos "Os%" ) ))
( "chattily"  a ( ch a1 t i l ii ) ((pos "Pu%" ) ))
( "chatting"  v ( ch a1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "chatty"  j ( ch a1 t ii ) ((pos "OD%" ) ))
( "chauffeur"  n ( sh ou1 f @ r ) ((pos "K6%" ) ))
( "chauffeurs"  n ( sh ou1 f @ z ) ((pos "Kj%" ) ))
( "chauffeuse"  n ( sh ou1 f @@ z ) ((pos "Ki%" ) ))
( "chauffeuses"  n ( sh ou1 f @@ z ) ((pos "Kj%" ) ))
( "chauvinism"  n ( sh ou1 v i n i z @ m ) ((pos "L@%" ) ))
( "chauvinist"  n ( sh ou1 v i n i s t ) ((pos "K6%" ) ))
( "chauvinistic"  j ( sh ou2 v i n i1 s t i k ) ((pos "OA%" ) ))
( "chauvinists"  n ( sh ou1 v i n i s t s ) ((pos "Kj%" ) ))
( "chaw"  v ( ch oo1 ) ((pos "H0$" "K6$" ) ))
( "chaw-bacon"  n ( ch oo1 - b ei k @ n ) ((pos "K6$" ) ))
( "chaw-bacons"  n ( ch oo1 - b ei k @ n z ) ((pos "Kj$" ) ))
( "chawed"  v ( ch oo1 d ) ((pos "Hc$" "Hd$" ) ))
( "chawing"  v ( ch oo1 i ng ) ((pos "Hb$" ) ))
( "chaws"  v ( ch oo1 z ) ((pos "Ha$" "Kj$" ) ))
( "cheap"  j ( ch ii1 p ) ((pos "OC%" ) ))
( "cheapen"  v ( ch ii1 p @ n ) ((pos "J0%" ) (subcat "2A" )))
( "cheapened"  v ( ch ii1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cheapening"  v ( ch ii1 p @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cheapens"  v ( ch ii1 p @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "cheaper"  j ( ch ii1 p @ r ) ((pos "Or%" ) ))
( "cheapest"  j ( ch ii1 p i s t ) ((pos "Os%" ) ))
( "cheapjack"  j ( ch ii1 p jh a k ) ((pos "OA$" ) ))
( "cheaply"  a ( ch ii1 p l ii ) ((pos "Pu%" ) ))
( "cheapness"  n ( ch ii1 p n @ s ) ((pos "L@$" ) ))
( "cheat"  v ( ch ii1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "cheated"  v ( ch ii1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cheating"  v ( ch ii1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "cheats"  v ( ch ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "check"  v ( ch e1 k ) ((pos "J0%" "M6%" ) (subcat "2C" "6A" )))
( "checkbook"  n ( ch e1 k b u k ) ((pos "K6$" ) ))
( "checkbooks"  n ( ch e1 k b u k s ) ((pos "Kj$" ) ))
( "checked"  v ( ch e1 k t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "6A" )))
( "checker"  v ( ch e1 k @ r ) ((pos "H0$" "K6%" ) ))
( "checkered"  v ( ch e1 k @ d ) ((pos "Hc%" "Hd%" ) ))
( "checkering"  v ( ch e1 k @ r i ng ) ((pos "Hb$" ) ))
( "checkers"  v ( ch e1 k @ z ) ((pos "Ha$" "Kj%" "Lk%" ) ))
( "checking"  v ( ch e1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "checklist"  n ( ch e1 k l i s t ) ((pos "K6%" ) ))
( "checklists"  n ( ch e1 k l i s t s ) ((pos "Kj%" ) ))
( "checkmate"  v ( ch e1 k m ei t ) ((pos "H2%" "K6%" ) (subcat )))
( "checkmated"  v ( ch e1 k m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "checkmates"  v ( ch e1 k m ei t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "checkmating"  v ( ch e1 k m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "checkout"  n ( ch e1 k au t ) ((pos "K6%" ) ))
( "checkouts"  n ( ch e1 k au t s ) ((pos "Kj%" ) ))
( "checkpoint"  n ( ch e1 k p oi n t ) ((pos "K6%" ) ))
( "checkpoints"  n ( ch e1 k p oi n t s ) ((pos "Kj%" ) ))
( "checkroom"  n ( ch e1 k r uu m ) ((pos "K6$" ) ))
( "checkrooms"  n ( ch e1 k r uu m z ) ((pos "Kj$" ) ))
( "checks"  v ( ch e1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "checkup"  n ( ch e1 k uh p ) ((pos "K6%" ) ))
( "checkups"  n ( ch e1 k uh p s ) ((pos "Kj%" ) ))
( "cheek"  v ( ch ii1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "cheekbone"  n ( ch ii1 k b ou n ) ((pos "K6%" ) ))
( "cheekbones"  n ( ch ii1 k b ou n z ) ((pos "Kj%" ) ))
( "cheeked"  v ( ch ii1 k t ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat )))
( "cheekier"  j ( ch ii1 k i@ r ) ((pos "Or%" ) ))
( "cheekiest"  j ( ch ii1 k i i s t ) ((pos "Os%" ) ))
( "cheekily"  a ( ch ii1 k i l ii ) ((pos "Pu%" ) ))
( "cheeking"  v ( ch ii1 k i ng ) ((pos "Hb%" ) (subcat )))
( "cheeks"  v ( ch ii1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "cheeky"  j ( ch ii1 k ii ) ((pos "OD%" ) ))
( "cheep"  v ( ch ii1 p ) ((pos "I0%" "K6%" ) ))
( "cheeped"  v ( ch ii1 p t ) ((pos "Ic%" "Id%" ) ))
( "cheeping"  v ( ch ii1 p i ng ) ((pos "Ib%" ) ))
( "cheeps"  v ( ch ii1 p s ) ((pos "Ia%" "Kj%" ) ))
( "cheer"  v ( ch i@1 r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "cheered"  v ( ch i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cheerful"  j ( ch i@1 f @ l ) ((pos "OA%" ) ))
( "cheerfully"  a ( ch i@1 f @ l ii ) ((pos "Pu%" ) ))
( "cheerfulness"  n ( ch i@1 f @ l n @ s ) ((pos "L@%" ) ))
( "cheerily"  a ( ch i@1 r @ l ii ) ((pos "Pu%" ) ))
( "cheering"  v ( ch i@1 r i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "cheerio"  uh ( ch i@2 r i ou1 ) ((pos "W-%" ) ))
( "cheerleader"  n ( ch i@1 l ii d @ r ) ((pos "K6%" ) ))
( "cheerleaders"  n ( ch i@1 l ii d @ z ) ((pos "Kj%" ) ))
( "cheerless"  j ( ch i@1 l @ s ) ((pos "OA%" ) ))
( "cheerlessly"  a ( ch i@1 l @ s l ii ) ((pos "Pu%" ) ))
( "cheerlessness"  n ( ch i@1 l @ s n @ s ) ((pos "L@%" ) ))
( "cheers"  v ( ch i@1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "cheery"  j ( ch i@1 r ii ) ((pos "OA%" ) ))
( "cheese"  n ( ch ii1 z ) ((pos "M6%" ) ))
( "cheesecake"  n ( ch ii1 z k ei k ) ((pos "M6%" ) ))
( "cheesecakes"  n ( ch ii1 z k ei k s ) ((pos "Mj%" ) ))
( "cheesecloth"  n ( ch ii1 z k l o th ) ((pos "M6%" ) ))
( "cheesecloths"  n ( ch ii1 z k l o th s ) ((pos "Mj$" ) ))
( "cheeseparing"  j ( ch ii1 z p e@ r i ng ) ((pos "OA%" ) ))
( "cheeses"  n ( ch ii1 z i z ) ((pos "Mj%" ) ))
( "cheetah"  n ( ch ii1 t @ ) ((pos "K6%" ) ))
( "cheetahs"  n ( ch ii1 t @ z ) ((pos "Kj%" ) ))
( "chef"  n ( sh e1 f ) ((pos "K6%" ) ))
( "chef-d'oeuvre"  n ( sh ei2 - d @@1 v r @ ) ((pos "Ki$" ) ))
( "chefs"  n ( sh e1 f s ) ((pos "Kj%" ) ))
( "chefs-d'oeuvre"  n ( sh ei2 - d @@1 v r @ ) ((pos "Kj$" ) ))
( "chemical"  n ( k e1 m i k l ) ((pos "K6%" "OA%" ) ))
( "chemically"  a ( k e1 m i k l ii ) ((pos "Pu%" ) ))
( "chemicals"  n ( k e1 m i k l z ) ((pos "Kj%" ) ))
( "chemise"  n ( sh @1 m ii1 z ) ((pos "K6%" ) ))
( "chemises"  n ( sh @1 m ii1 z i z ) ((pos "Kj$" ) ))
( "chemist"  n ( k e1 m i s t ) ((pos "K6%" ) ))
( "chemistry"  n ( k e1 m i s t r ii ) ((pos "L@%" ) ))
( "chemists"  n ( k e1 m i s t s ) ((pos "Kj%" ) ))
( "chemotherapy"  n ( k e2 m ou th e1 r @ p ii ) ((pos "L@%" ) ))
( "chenille"  n ( sh @1 n ii1 l ) ((pos "L@$" ) ))
( "cheque"  n ( ch e1 k ) ((pos "K6%" ) ))
( "chequebook"  n ( ch e1 k b u k ) ((pos "K6%" ) ))
( "chequebooks"  n ( ch e1 k b u k s ) ((pos "Kj%" ) ))
( "chequer"  v ( ch e1 k @ r ) ((pos "H0$" ) ))
( "chequered"  v ( ch e1 k @ d ) ((pos "Hc$" "Hd%" ) ))
( "chequering"  v ( ch e1 k @ r i ng ) ((pos "Hb$" ) ))
( "chequers"  v ( ch e1 k @ z ) ((pos "Ha$" ) ))
( "cheques"  n ( ch e1 k s ) ((pos "Kj%" ) ))
( "cherish"  v ( ch e1 r i sh ) ((pos "H1%" ) (subcat )))
( "cherished"  v ( ch e1 r i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cherishes"  v ( ch e1 r i sh i z ) ((pos "Ha%" ) (subcat )))
( "cherishing"  v ( ch e1 r i sh i ng ) ((pos "Hb%" ) (subcat )))
( "cheroot"  n ( sh @1 r uu1 t ) ((pos "K6%" ) ))
( "cheroots"  n ( sh @1 r uu1 t s ) ((pos "Kj%" ) ))
( "cherries"  n ( ch e1 r i z ) ((pos "Kj%" ) ))
( "cherry"  n ( ch e1 r ii ) ((pos "K8%" "OA%" ) ))
( "cherub"  n ( ch e1 r @ b ) ((pos "K6%" ) ))
( "cherubic"  j ( ch i1 r uu1 b i k ) ((pos "OA%" ) ))
( "cherubs"  n ( ch e1 r @ b z ) ((pos "Kj%" ) ))
( "chervil"  n ( ch @@1 v i l ) ((pos "L@%" ) ))
( "chess"  n ( ch e1 s ) ((pos "L@%" ) ))
( "chessboard"  n ( ch e1 s b oo d ) ((pos "K6%" ) ))
( "chessboards"  n ( ch e1 s b oo d z ) ((pos "Kj%" ) ))
( "chessman"  n ( ch e1 s m a n ) ((pos "Ki%" ) ))
( "chessmen"  n ( ch e1 s m e n ) ((pos "Kj%" ) ))
( "chest"  n ( ch e1 s t ) ((pos "K6%" ) ))
( "chesterfield"  n ( ch e1 s t @ f ii l d ) ((pos "K6%" ) ))
( "chesterfields"  n ( ch e1 s t @ f ii l d z ) ((pos "Kj%" ) ))
( "chestnut"  n ( ch e1 s n uh t ) ((pos "M6%" ) ))
( "chestnuts"  n ( ch e1 s n uh t s ) ((pos "Mj%" ) ))
( "chests"  n ( ch e1 s t s ) ((pos "Kj%" ) ))
( "cheval glass"  n ( sh @1 v a1 l - g l aa s ) ((pos "K7$" ) ))
( "cheval glasses"  n ( sh @1 v a1 l - g l aa s i z ) ((pos "Kj$" ) ))
( "chevron"  n ( sh e1 v r @ n ) ((pos "K6$" ) ))
( "chevrons"  n ( sh e1 v r @ n z ) ((pos "Kj$" ) ))
( "chew"  v ( ch uu1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "chewed"  v ( ch uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "chewing"  v ( ch uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "chewing-gum"  n ( ch uu1 i ng - g uh m ) ((pos "L@%" ) ))
( "chews"  v ( ch uu1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "chiaroscuro"  n ( k i1 aa2 r @ s k u@1 r ou ) ((pos "L@$" ) ))
( "chic"  n ( sh ii1 k ) ((pos "L@%" "OA%" ) ))
( "chicaneries"  n ( sh i1 k ei1 n @ r i z ) ((pos "Mj$" ) ))
( "chicanery"  n ( sh i1 k ei1 n @ r ii ) ((pos "M8%" ) ))
( "chichi"  j ( sh ii1 sh ii ) ((pos "OA$" ) ))
( "chick"  n ( ch i1 k ) ((pos "K6%" ) ))
( "chicken"  n ( ch i1 k i n ) ((pos "M6%" ) ))
( "chicken-hearted"  j ( ch i2 k i n - h aa1 t i d ) ((pos "OA%" ) ))
( "chicken-run"  n ( ch i1 k i n - r uh n ) ((pos "K6%" ) ))
( "chicken-runs"  n ( ch i1 k i n - r uh n z ) ((pos "Kj%" ) ))
( "chickenfeed"  n ( ch i1 k i n f ii d ) ((pos "L@%" ) ))
( "chickenpox"  n ( ch i1 k i n p o k s ) ((pos "L@%" ) ))
( "chickens"  n ( ch i1 k i n z ) ((pos "Mj%" ) ))
( "chickpea"  n ( ch i1 k p ii ) ((pos "K6%" ) ))
( "chickpeas"  n ( ch i1 k p ii z ) ((pos "Kj%" ) ))
( "chicks"  n ( ch i1 k s ) ((pos "Kj%" ) ))
( "chickweed"  n ( ch i1 k w ii d ) ((pos "L@%" ) ))
( "chicle"  n ( ch i1 k l ) ((pos "L@$" ) ))
( "chicory"  n ( ch i1 k @ r ii ) ((pos "L@%" ) ))
( "chid"  v ( ch i1 d ) ((pos "Jc$" ) (subcat "6A" )))
( "chidden"  v ( ch i1 d @ n ) ((pos "Jd$" ) (subcat "6A" )))
( "chide"  v ( ch ai1 d ) ((pos "J2%" ) (subcat "6A" )))
( "chided"  v ( ch ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "chides"  v ( ch ai1 d z ) ((pos "Ja%" ) (subcat "6A" )))
( "chiding"  v ( ch ai1 d i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "chief"  n ( ch ii1 f ) ((pos "K6%" "OA%" ) ))
( "chiefly"  a ( ch ii1 f l ii ) ((pos "Pu%" ) ))
( "chiefs"  n ( ch ii1 f s ) ((pos "Kj%" ) ))
( "chieftain"  n ( ch ii1 f t @ n ) ((pos "K6%" ) ))
( "chieftaincies"  n ( ch ii1 f t @ n s i z ) ((pos "Kj$" ) ))
( "chieftaincy"  n ( ch ii1 f t @ n s ii ) ((pos "K8%" ) ))
( "chieftains"  n ( ch ii1 f t @ n z ) ((pos "Kj%" ) ))
( "chiffon"  n ( sh i1 f o n ) ((pos "L@%" ) ))
( "chiffonier"  n ( sh i2 f @ n i@1 r ) ((pos "K6$" ) ))
( "chiffoniers"  n ( sh i2 f @ n i@1 z ) ((pos "Kj$" ) ))
( "chignon"  n ( sh ii1 n y o n ) ((pos "K6%" ) ))
( "chignons"  n ( sh ii1 n y o n z ) ((pos "Kj%" ) ))
( "chilblain"  n ( ch i1 l b l ei n ) ((pos "K6%" ) ))
( "chilblained"  j ( ch i1 l b l ei n d ) ((pos "OA$" ) ))
( "chilblains"  n ( ch i1 l b l ei n z ) ((pos "Kj%" ) ))
( "child"  n ( ch ai1 l d ) ((pos "Ki*" ) ))
( "child's-play"  n ( ch ai1 l z - p l ei ) ((pos "L@%" ) ))
( "child-bearing"  n ( ch ai1 l - b e@ r i ng ) ((pos "L@%" ) ))
( "childbirth"  n ( ch ai1 l d b @@ th ) ((pos "L@%" ) ))
( "childhood"  n ( ch ai1 l d h u d ) ((pos "L@%" ) ))
( "childish"  j ( ch ai1 l d i sh ) ((pos "OA%" ) ))
( "childishly"  a ( ch ai1 l d i sh l ii ) ((pos "Pu%" ) ))
( "childless"  j ( ch ai1 l d l @ s ) ((pos "OA%" ) ))
( "childlike"  j ( ch ai1 l d l ai k ) ((pos "OA%" ) ))
( "children"  n ( ch i1 l d r @ n ) ((pos "Kj*" ) ))
( "chill"  v ( ch i1 l ) ((pos "J0%" "K6%" "OA%" ) (subcat "2A" "2C" )))
( "chilled"  v ( ch i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "chilli"  n ( ch i1 l ii ) ((pos "K7%" ) ))
( "chillier"  j ( ch i1 l i@ r ) ((pos "Or%" ) ))
( "chillies"  n ( ch i1 l i z ) ((pos "Kj%" ) ))
( "chilliest"  j ( ch i1 l i i s t ) ((pos "Os%" ) ))
( "chilling"  v ( ch i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "chills"  v ( ch i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "chilly"  n ( ch i1 l ii ) ((pos "K8$" "OD%" ) ))
( "chimaera"  n ( k ai1 m i@1 r @ ) ((pos "K6$" ) ))
( "chimaeras"  n ( k ai1 m i@1 r @ z ) ((pos "Kj$" ) ))
( "chime"  v ( ch ai1 m ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "chimed"  v ( ch ai1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "chimera"  n ( k ai1 m i@1 r @ ) ((pos "K6%" ) ))
( "chimeras"  n ( k ai1 m i@1 r @ z ) ((pos "Kj%" ) ))
( "chimerical"  j ( k ai1 m e1 r i k l ) ((pos "OA$" ) ))
( "chimes"  v ( ch ai1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "chiming"  v ( ch ai1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "chimney"  n ( ch i1 m n ii ) ((pos "K6%" ) ))
( "chimney-sweep"  n ( ch i1 m n i - s w ii p ) ((pos "K6%" ) ))
( "chimney-sweeps"  n ( ch i1 m n i - s w ii p s ) ((pos "Kj%" ) ))
( "chimneybreast"  n ( ch i1 m n i b r e s t ) ((pos "K6%" ) ))
( "chimneybreasts"  n ( ch i1 m n i b r e s t s ) ((pos "Kj%" ) ))
( "chimneypiece"  n ( ch i1 m n i p ii s ) ((pos "K6%" ) ))
( "chimneypieces"  n ( ch i1 m n i p ii s i z ) ((pos "Kj%" ) ))
( "chimneypot"  n ( ch i1 m n i p o t ) ((pos "K6%" ) ))
( "chimneypots"  n ( ch i1 m n i p o t s ) ((pos "Kj%" ) ))
( "chimneys"  n ( ch i1 m n i z ) ((pos "Kj%" ) ))
( "chimneystack"  n ( ch i1 m n i s t a k ) ((pos "K6%" ) ))
( "chimneystacks"  n ( ch i1 m n i s t a k s ) ((pos "Kj%" ) ))
( "chimneysweep"  n ( ch i1 m n i s w ii p ) ((pos "K6%" ) ))
( "chimneysweeper"  n ( ch i1 m n i s w ii p @ r ) ((pos "K6$" ) ))
( "chimneysweepers"  n ( ch i1 m n i s w ii p @ z ) ((pos "Kj$" ) ))
( "chimneysweeps"  n ( ch i1 m n i s w ii p s ) ((pos "Kj%" ) ))
( "chimp"  n ( ch i1 m p ) ((pos "K6%" ) ))
( "chimpanzee"  n ( ch i2 m p a n z ii1 ) ((pos "K6%" ) ))
( "chimpanzees"  n ( ch i2 m p a n z ii1 z ) ((pos "Kj%" ) ))
( "chimps"  n ( ch i1 m p s ) ((pos "Kj%" ) ))
( "chin"  n ( ch i1 n ) ((pos "K6%" ) ))
( "chin-strap"  n ( ch i1 n - s t r a p ) ((pos "K6%" ) ))
( "chin-straps"  n ( ch i1 n - s t r a p s ) ((pos "Kj%" ) ))
( "chin-wagging"  n ( ch i1 n - w a g i ng ) ((pos "L@%" ) ))
( "china"  n ( ch ai1 n @ ) ((pos "L@%" ) ))
( "china-closet"  n ( ch ai2 n @ - k l o1 z i t ) ((pos "K6$" ) ))
( "china-closets"  n ( ch ai2 n @ - k l o1 z i t s ) ((pos "Kj$" ) ))
( "chinaware"  n ( ch ai1 n @ w e@ r ) ((pos "L@%" ) ))
( "chinchilla"  n ( ch i1 n ch i1 l @ ) ((pos "M6%" ) ))
( "chinchillas"  n ( ch i1 n ch i1 l @ z ) ((pos "Mj%" ) ))
( "chine"  n ( ch ai1 n ) ((pos "K6$" ) ))
( "chines"  n ( ch ai1 n z ) ((pos "Kj$" ) ))
( "chink"  v ( ch i1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "chinked"  v ( ch i1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "chinking"  v ( ch i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "chinks"  v ( ch i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "chins"  n ( ch i1 n z ) ((pos "Kj%" ) ))
( "chintz"  n ( ch i1 n t s ) ((pos "L@%" ) ))
( "chip"  v ( ch i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "chipboard"  n ( ch i1 p b oo d ) ((pos "L@%" ) ))
( "chipmunk"  n ( ch i1 p m uh ng k ) ((pos "K6%" ) ))
( "chipmunks"  n ( ch i1 p m uh ng k s ) ((pos "Kj%" ) ))
( "chipped"  v ( ch i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "chipping"  v ( ch i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "chippings"  n ( ch i1 p i ng z ) ((pos "Kj%" ) ))
( "chips"  v ( ch i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "chiropodist"  n ( k i1 r o1 p @ d i s t ) ((pos "K6%" ) ))
( "chiropodists"  n ( k i1 r o1 p @ d i s t s ) ((pos "Kj%" ) ))
( "chiropody"  n ( k i1 r o1 p @ d ii ) ((pos "L@%" ) ))
( "chiropractor"  n ( k ai1 @ r ou p r a k t @ r ) ((pos "K6%" ) ))
( "chiropractors"  n ( k ai1 @ r ou p r a k t @ z ) ((pos "Kj%" ) ))
( "chirp"  v ( ch @@1 p ) ((pos "J0%" "K6%" ) ))
( "chirped"  v ( ch @@1 p t ) ((pos "Jc%" "Jd%" ) ))
( "chirpily"  a ( ch @@1 p i l ii ) ((pos "Pu%" ) ))
( "chirpiness"  n ( ch @@1 p i n @ s ) ((pos "L@%" ) ))
( "chirping"  v ( ch @@1 p i ng ) ((pos "Jb%" ) ))
( "chirps"  v ( ch @@1 p s ) ((pos "Ja%" "Kj%" ) ))
( "chirpy"  j ( ch @@1 p ii ) ((pos "OA%" ) ))
( "chirrup"  v ( ch i1 r @ p ) ((pos "J0%" "K6%" ) ))
( "chirruped"  v ( ch i1 r @ p t ) ((pos "Jc%" "Jd%" ) ))
( "chirruping"  v ( ch i1 r @ p i ng ) ((pos "Jb%" ) ))
( "chirrups"  v ( ch i1 r @ p s ) ((pos "Ja%" "Kj%" ) ))
( "chisel"  v ( ch i1 z l ) ((pos "H4%" "K6%" ) (subcat "6A" "14" )))
( "chiselled"  v ( ch i1 z l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "chiseller"  n ( ch i1 z @ l @ r ) ((pos "K6%" ) ))
( "chisellers"  n ( ch i1 z @ l @ z ) ((pos "Kj%" ) ))
( "chiselling"  v ( ch i1 z @ l i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "chisels"  v ( ch i1 z l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "chit"  n ( ch i1 t ) ((pos "K6%" ) ))
( "chit-chat"  n ( ch i1 t - ch a t ) ((pos "L@%" ) ))
( "chits"  n ( ch i1 t s ) ((pos "Kj%" ) ))
( "chivalrous"  j ( sh i1 v @ l r @ s ) ((pos "OA%" ) ))
( "chivalrously"  a ( sh i1 v @ l r @ s l ii ) ((pos "Pu%" ) ))
( "chivalry"  n ( sh i1 v @ l r ii ) ((pos "L@%" ) ))
( "chive"  n ( ch ai1 v ) ((pos "M6$" ) ))
( "chives"  n ( ch ai1 v z ) ((pos "Mj%" ) ))
( "chivied"  v ( ch i1 v i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "chivies"  v ( ch i1 v i z ) ((pos "Ha$" ) (subcat )))
( "chivvied"  v ( ch i1 v i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chivvies"  v ( ch i1 v i z ) ((pos "Ha%" ) (subcat )))
( "chivvy"  v ( ch i1 v ii ) ((pos "H3%" ) (subcat )))
( "chivvying"  v ( ch i1 v i i ng ) ((pos "Hb%" ) (subcat )))
( "chivy"  v ( ch i1 v ii ) ((pos "H3$" ) (subcat )))
( "chivying"  v ( ch i1 v i i ng ) ((pos "Hb$" ) (subcat )))
( "chloride"  n ( k l oo1 r ai d ) ((pos "L@%" ) ))
( "chlorinate"  v ( k l o1 r i n ei t ) ((pos "H2%" ) ))
( "chlorinated"  v ( k l o1 r i n ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "chlorinates"  v ( k l o1 r i n ei t s ) ((pos "Ha%" ) ))
( "chlorinating"  v ( k l o1 r i n ei t i ng ) ((pos "Hb%" ) ))
( "chlorination"  n ( k l o2 r i n ei1 sh @ n ) ((pos "L@%" ) ))
( "chlorine"  n ( k l oo1 r ii n ) ((pos "L@%" ) ))
( "chloroform"  n ( k l o1 r @ f oo m ) ((pos "L@%" ) ))
( "chlorophyll"  n ( k l o1 r @ f i l ) ((pos "L@%" ) ))
( "choc"  n ( ch o1 k ) ((pos "K6%" ) ))
( "choc-ice"  n ( ch o1 k - ai s ) ((pos "K6%" ) ))
( "choc-ices"  n ( ch o1 k - ai s i z ) ((pos "Kj%" ) ))
( "chock"  v ( ch o1 k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "chock-a-block"  j ( ch o2 k - @ - b l o1 k ) ((pos "OA%" "Pu%" ) ))
( "chock-full"  j ( ch o1 k - f u1 l ) ((pos "OA%" ) ))
( "chocked"  v ( ch o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "chocking"  v ( ch o1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "chocks"  v ( ch o1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "chocolate"  n ( ch o1 k l @ t ) ((pos "M6%" ) ))
( "chocolates"  n ( ch o1 k l @ t s ) ((pos "Mj%" ) ))
( "chocs"  n ( ch o1 k s ) ((pos "Kj%" ) ))
( "choice"  n ( ch oi1 s ) ((pos "M6%" "OA%" ) ))
( "choices"  n ( ch oi1 s i z ) ((pos "Mj%" ) ))
( "choir"  n ( k w ai1 @ r ) ((pos "K6%" ) ))
( "choir-school"  n ( k w ai1 @ - s k uu l ) ((pos "K6%" ) ))
( "choir-schools"  n ( k w ai1 @ - s k uu l z ) ((pos "Kj%" ) ))
( "choirboy"  n ( k w ai1 @ b oi ) ((pos "K6%" ) ))
( "choirboys"  n ( k w ai1 @ b oi z ) ((pos "Kj%" ) ))
( "choirs"  n ( k w ai1 @ z ) ((pos "Kj%" ) ))
( "choke"  v ( ch ou1 k ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "14" )))
( "choke-damp"  n ( ch ou1 k - d a m p ) ((pos "L@$" ) ))
( "choked"  v ( ch ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "choker"  n ( ch ou1 k @ r ) ((pos "K6%" ) ))
( "chokers"  n ( ch ou1 k @ z ) ((pos "Kj%" ) ))
( "chokes"  v ( ch ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "14" )))
( "chokey"  n ( ch ou1 k ii ) ((pos "K6$" ) ))
( "chokeys"  n ( ch ou1 k i z ) ((pos "Kj$" ) ))
( "chokies"  n ( ch ou1 k i z ) ((pos "Kj$" ) ))
( "choking"  v ( ch ou1 k i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "choky"  n ( ch ou1 k ii ) ((pos "K8$" ) ))
( "choler"  n ( k o1 l @ r ) ((pos "K6$" ) ))
( "cholera"  n ( k o1 l @ r @ ) ((pos "L@%" ) ))
( "choleric"  j ( k o1 l @ r i k ) ((pos "OA%" ) ))
( "cholers"  n ( k o1 l @ z ) ((pos "Kj$" ) ))
( "choose"  v ( ch uu1 z ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "6A" "7A" "16A" "16B" )))
( "chooses"  v ( ch uu1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "7A" "16A" "16B" )))
( "choosey"  j ( ch uu1 z ii ) ((pos "OA%" ) ))
( "choosier"  j ( ch uu1 z i@ r ) ((pos "Or%" ) ))
( "choosiest"  j ( ch uu1 z i i s t ) ((pos "Os%" ) ))
( "choosing"  v ( ch uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "7A" "16A" "16B" )))
( "choosy"  j ( ch uu1 z ii ) ((pos "OD%" ) ))
( "chop"  v ( ch o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "chop suey"  n ( ch o2 p - s uu1 ii ) ((pos "L@%" ) ))
( "chop-chop"  a ( ch o2 p - ch o1 p ) ((pos "Pu%" ) ))
( "chop-house"  n ( ch o1 p - h au s ) ((pos "K6$" ) ))
( "chop-houses"  n ( ch o1 p - h au z i z ) ((pos "Kj$" ) ))
( "chopped"  v ( ch o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "chopper"  n ( ch o1 p @ r ) ((pos "K6%" ) ))
( "choppers"  n ( ch o1 p @ z ) ((pos "Kj%" ) ))
( "choppier"  j ( ch o1 p i@ r ) ((pos "Or%" ) ))
( "choppiest"  j ( ch o1 p i i s t ) ((pos "Os%" ) ))
( "chopping"  v ( ch o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "choppy"  j ( ch o1 p ii ) ((pos "OD%" ) ))
( "chops"  v ( ch o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "chopsticks"  n ( ch o1 p s t i k s ) ((pos "Kj%" ) ))
( "choral"  j ( k oo1 r @ l ) ((pos "OA%" ) ))
( "chorale"  n ( k @1 r aa1 l ) ((pos "K6%" ) ))
( "chorales"  n ( k @1 r aa1 l z ) ((pos "Kj%" ) ))
( "chord"  n ( k oo1 d ) ((pos "K6%" ) ))
( "chords"  n ( k oo1 d z ) ((pos "Kj%" ) ))
( "chore"  n ( ch oo1 r ) ((pos "K6%" ) ))
( "choreographer"  n ( k o2 r i o1 g r @ f @ r ) ((pos "K6%" ) ))
( "choreographers"  n ( k o2 r i o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "choreography"  n ( k o2 r i o1 g r @ f ii ) ((pos "L@%" ) ))
( "chores"  n ( ch oo1 z ) ((pos "Kj%" ) ))
( "chorister"  n ( k o1 r i s t @ r ) ((pos "K6%" ) ))
( "choristers"  n ( k o1 r i s t @ z ) ((pos "Kj%" ) ))
( "chortle"  v ( ch oo1 t l ) ((pos "I2%" "K6%" ) ))
( "chortled"  v ( ch oo1 t l d ) ((pos "Ic%" "Id%" ) ))
( "chortles"  v ( ch oo1 t l z ) ((pos "Ia%" "Kj%" ) ))
( "chortling"  v ( ch oo1 t l i ng ) ((pos "Ib%" ) ))
( "chorus"  v ( k oo1 r @ s ) ((pos "H1%" "K7%" ) ))
( "chorus-girl"  n ( k oo1 r @ s - g @@ l ) ((pos "K6%" ) ))
( "chorus-girls"  n ( k oo1 r @ s - g @@ l z ) ((pos "Kj%" ) ))
( "chorused"  v ( k oo1 r @ s t ) ((pos "Hc%" "Hd%" ) ))
( "choruses"  v ( k oo1 r @ s i z ) ((pos "Ha%" "Kj%" ) ))
( "chorusing"  v ( k oo1 r @ s i ng ) ((pos "Hb%" ) ))
( "chose"  v ( ch ou1 z ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" "7A" "16A" "16B" )))
( "chosen"  v ( ch ou1 z @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "7A" "16A" "16B" )))
( "chow"  n ( ch au1 ) ((pos "K6%" ) ))
( "chowder"  n ( ch au1 d @ r ) ((pos "K6%" ) ))
( "chowders"  n ( ch au1 d @ z ) ((pos "Kj$" ) ))
( "chows"  n ( ch au1 z ) ((pos "Kj%" ) ))
( "christen"  v ( k r i1 s n ) ((pos "H0%" ) (subcat "6A" )))
( "christened"  v ( k r i1 s n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "christening"  v ( k r i1 s n i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" )))
( "christenings"  n ( k r i1 s n i ng z ) ((pos "Kj%" ) ))
( "christens"  v ( k r i1 s n z ) ((pos "Ha%" ) (subcat "6A" )))
( "chromatic"  j ( k r ou1 m a1 t i k ) ((pos "OA%" ) ))
( "chrome"  n ( k r ou1 m ) ((pos "L@%" ) ))
( "chromium"  n ( k r ou1 m i@ m ) ((pos "L@%" ) ))
( "chromosome"  n ( k r ou1 m @ s ou m ) ((pos "K6%" ) ))
( "chromosomes"  n ( k r ou1 m @ s ou m z ) ((pos "Kj%" ) ))
( "chronic"  j ( k r o1 n i k ) ((pos "OA%" ) ))
( "chronically"  a ( k r o1 n i k l ii ) ((pos "Pu%" ) ))
( "chronicle"  v ( k r o1 n i k l ) ((pos "H2%" "K6%" ) (subcat )))
( "chronicled"  v ( k r o1 n i k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "chronicler"  n ( k r o1 n i k l @ r ) ((pos "K6%" ) ))
( "chroniclers"  n ( k r o1 n i k l @ z ) ((pos "Kj%" ) ))
( "chronicles"  v ( k r o1 n i k l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "chronicling"  v ( k r o1 n i k l i ng ) ((pos "Hb%" ) (subcat )))
( "chronological"  j ( k r o2 n @ l o1 jh i k l ) ((pos "OA%" ) ))
( "chronologically"  a ( k r o2 n @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "chronologies"  n ( k r @1 n o1 l @ jh i z ) ((pos "Mj$" ) ))
( "chronology"  n ( k r @1 n o1 l @ jh ii ) ((pos "M8%" ) ))
( "chronometer"  n ( k r @1 n o1 m i t @ r ) ((pos "K6$" ) ))
( "chronometers"  n ( k r @1 n o1 m i t @ z ) ((pos "Kj$" ) ))
( "chrysalis"  n ( k r i1 s @ l i s ) ((pos "K7%" ) ))
( "chrysalises"  n ( k r i1 s @ l i s i z ) ((pos "Kj$" ) ))
( "chrysanthemum"  n ( k r i1 s a1 n th @ m @ m ) ((pos "K6%" ) ))
( "chrysanthemums"  n ( k r i1 s a1 n th @ m @ m z ) ((pos "Kj%" ) ))
( "chubbier"  j ( ch uh1 b i@ r ) ((pos "Or%" ) ))
( "chubbiest"  j ( ch uh1 b i i s t ) ((pos "Os%" ) ))
( "chubby"  j ( ch uh1 b ii ) ((pos "OD%" ) ))
( "chuck"  v ( ch uh1 k ) ((pos "H0%" "K6$" ) (subcat "6A" "12A" "13A" "15A" )))
( "chucked"  v ( ch uh1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" "15A" )))
( "chucker-out"  n ( ch uh2 k @ r - au1 t ) ((pos "Ki%" ) ))
( "chuckers-out"  n ( ch uh2 k @ z - au1 t ) ((pos "Kj%" ) ))
( "chucking"  v ( ch uh1 k i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" "15A" )))
( "chuckle"  v ( ch uh1 k l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "chuckled"  v ( ch uh1 k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "chuckles"  v ( ch uh1 k l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "chuckling"  v ( ch uh1 k l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "chucks"  v ( ch uh1 k s ) ((pos "Ha%" "Kj$" ) (subcat "6A" "12A" "13A" "15A" )))
( "chug"  v ( ch uh1 g ) ((pos "I4%" "K6%" ) (subcat )))
( "chugged"  v ( ch uh1 g d ) ((pos "Ic%" "Id%" ) (subcat )))
( "chugging"  v ( ch uh1 g i ng ) ((pos "Ib%" ) (subcat )))
( "chugs"  v ( ch uh1 g z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "chukker"  n ( ch uh1 k @ r ) ((pos "K6$" ) ))
( "chukkers"  n ( ch uh1 k @ z ) ((pos "Kj$" ) ))
( "chum"  v ( ch uh1 m ) ((pos "I4$" "K6%" ) (subcat )))
( "chummed"  v ( ch uh1 m d ) ((pos "Ic$" "Id$" ) (subcat )))
( "chummier"  j ( ch uh1 m i@ r ) ((pos "Or$" ) ))
( "chummiest"  j ( ch uh1 m i i s t ) ((pos "Os$" ) ))
( "chumming"  v ( ch uh1 m i ng ) ((pos "Ib$" ) (subcat )))
( "chummy"  j ( ch uh1 m ii ) ((pos "OD%" ) ))
( "chump"  n ( ch uh1 m p ) ((pos "K6%" ) ))
( "chumps"  n ( ch uh1 m p s ) ((pos "Kj%" ) ))
( "chums"  v ( ch uh1 m z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "chunk"  n ( ch uh1 ng k ) ((pos "K6%" ) ))
( "chunkier"  j ( ch uh1 ng k i@ r ) ((pos "Or%" ) ))
( "chunkiest"  j ( ch uh1 ng k i i s t ) ((pos "Os%" ) ))
( "chunks"  n ( ch uh1 ng k s ) ((pos "Kj%" ) ))
( "chunky"  j ( ch uh1 ng k ii ) ((pos "OD%" ) ))
( "church"  n ( ch @@1 ch ) ((pos "M7%" ) ))
( "churches"  n ( ch @@1 ch i z ) ((pos "Mj%" ) ))
( "churchgoer"  n ( ch @@1 ch g ou @ r ) ((pos "K6%" ) ))
( "churchgoers"  n ( ch @@1 ch g ou @ z ) ((pos "Kj%" ) ))
( "churchman"  n ( ch @@1 ch m @ n ) ((pos "Ki%" ) ))
( "churchmen"  n ( ch @@1 ch m e n ) ((pos "Kj%" ) ))
( "churchwarden"  n ( ch @@1 ch w oo1 d n ) ((pos "K6%" ) ))
( "churchwardens"  n ( ch @@1 ch w oo1 d n z ) ((pos "Kj%" ) ))
( "churchyard"  n ( ch @@1 ch y aa d ) ((pos "K6%" ) ))
( "churchyards"  n ( ch @@1 ch y aa d z ) ((pos "Kj%" ) ))
( "churl"  n ( ch @@1 l ) ((pos "K6%" ) ))
( "churlish"  j ( ch @@1 l i sh ) ((pos "OA%" ) ))
( "churlishly"  a ( ch @@1 l i sh l ii ) ((pos "Pu%" ) ))
( "churls"  n ( ch @@1 l z ) ((pos "Kj%" ) ))
( "churn"  v ( ch @@1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "churned"  v ( ch @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "churning"  v ( ch @@1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "churns"  v ( ch @@1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "chute"  n ( sh uu1 t ) ((pos "K6%" ) ))
( "chutes"  n ( sh uu1 t s ) ((pos "Kj%" ) ))
( "chutney"  n ( ch uh1 t n ii ) ((pos "M6%" ) ))
( "chutneys"  n ( ch uh1 t n i z ) ((pos "Mj%" ) ))
( "cicada"  n ( s i1 k aa1 d @ ) ((pos "K6%" ) ))
( "cicadas"  n ( s i1 k aa1 d @ z ) ((pos "Kj%" ) ))
( "cicala"  n ( s i1 k aa1 l @ ) ((pos "K6$" ) ))
( "cicalas"  n ( s i1 k aa1 l @ z ) ((pos "Kj$" ) ))
( "cicatrice"  n ( s i1 k @ t r i s ) ((pos "K6$" ) ))
( "cicatrices"  n ( s i1 k @ t r i s i z ) ((pos "Kj$" ) ))
( "cicatrices"  n ( s i2 k @ t r ai1 s ii z ) ((pos "Kj$" ) ))
( "cicatrix"  n ( s i1 k @ t r i k s ) ((pos "Ki$" ) ))
( "cicerone"  n ( ch i2 ch @ r ou1 n ii ) ((pos "Ki$" ) ))
( "ciceroni"  n ( ch i2 ch @ r ou1 n ii ) ((pos "Kj$" ) ))
( "cider"  n ( s ai1 d @ r ) ((pos "M6%" ) ))
( "ciderpress"  n ( s ai1 d @ p r e s ) ((pos "K7%" ) ))
( "ciderpresses"  n ( s ai1 d @ p r e s i z ) ((pos "Kj%" ) ))
( "ciders"  n ( s ai1 d @ z ) ((pos "Mj%" ) ))
( "cif"  n ( s ii2 ai2 e1 f ) ((pos "Y~$" ) ))
( "cigar"  n ( s i1 g aa1 r ) ((pos "K6%" ) ))
( "cigar-shaped"  j ( s i1 g aa1 - sh ei p t ) ((pos "OA%" ) ))
( "cigarette"  n ( s i2 g @ r e1 t ) ((pos "K6%" ) ))
( "cigarette-case"  n ( s i2 g @ r e1 t - k ei s ) ((pos "K6%" ) ))
( "cigarette-cases"  n ( s i2 g @ r e1 t - k ei s i z ) ((pos "Kj%" ) ))
( "cigarette-holder"  n ( s i2 g @ r e1 t - h ou l d @ r ) ((pos "K6%" ) ))
( "cigarette-holders"  n ( s i2 g @ r e1 t - h ou l d @ z ) ((pos "Kj%" ) ))
( "cigarette-paper"  n ( s i2 g @ r e1 t - p ei p @ r ) ((pos "K6%" ) ))
( "cigarette-papers"  n ( s i2 g @ r e1 t - p ei p @ z ) ((pos "Kj%" ) ))
( "cigarettes"  n ( s i2 g @ r e1 t s ) ((pos "Kj%" ) ))
( "cigars"  n ( s i1 g aa1 z ) ((pos "Kj%" ) ))
( "cinch"  n ( s i1 n ch ) ((pos "Ki%" ) ))
( "cinchona"  n ( s i1 ng k ou1 n @ ) ((pos "K6$" ) ))
( "cinchonas"  n ( s i1 ng k ou1 n @ z ) ((pos "Kj$" ) ))
( "cincture"  n ( s i1 ng k ch @ r ) ((pos "K6$" ) ))
( "cinctures"  n ( s i1 ng k ch @ z ) ((pos "Kj$" ) ))
( "cinder"  n ( s i1 n d @ r ) ((pos "K6%" ) ))
( "cinder-track"  n ( s i1 n d @ - t r a k ) ((pos "K6%" ) ))
( "cinder-tracks"  n ( s i1 n d @ - t r a k s ) ((pos "Kj%" ) ))
( "cinders"  n ( s i1 n d @ z ) ((pos "Kj%" ) ))
( "cine-"  j ( s i1 n i - ) ((pos "U-%" ) ))
( "cine-camera"  n ( s i1 n i - k a m r @ ) ((pos "K6%" ) ))
( "cine-cameras"  n ( s i1 n i - k a m r @ z ) ((pos "Kj%" ) ))
( "cine-film"  n ( s i1 n i - f i l m ) ((pos "K6%" ) ))
( "cine-films"  n ( s i1 n i - f i l m z ) ((pos "Kj%" ) ))
( "cine-projector"  n ( s i1 n i - p r @ jh e k t @ r ) ((pos "K6%" ) ))
( "cine-projectors"  n ( s i1 n i - p r @ jh e k t @ z ) ((pos "Kj%" ) ))
( "cinema"  n ( s i1 n @ m @ ) ((pos "K6%" ) ))
( "cinemas"  n ( s i1 n @ m @ z ) ((pos "Kj%" ) ))
( "cinematic"  j ( s i2 n @ m a1 t i k ) ((pos "OA%" ) ))
( "cinematography"  n ( s i2 n @ m @ t o1 g r @ f ii ) ((pos "L@%" ) ))
( "cinnamon"  n ( s i1 n @ m @ n ) ((pos "L@%" ) ))
( "cinquefoil"  n ( s i1 ng k f oi l ) ((pos "K6$" ) ))
( "cinquefoils"  n ( s i1 ng k f oi l z ) ((pos "Kj$" ) ))
( "cipher"  v ( s ai1 f @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "ciphered"  v ( s ai1 f @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ciphering"  v ( s ai1 f @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ciphers"  v ( s ai1 f @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "circa"  in ( s @@1 k @ ) ((pos "T-%" ) ))
( "circle"  v ( s @@1 k l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "circled"  v ( s @@1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "circles"  v ( s @@1 k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "circlet"  n ( s @@1 k l i t ) ((pos "K6$" ) ))
( "circlets"  n ( s @@1 k l i t s ) ((pos "Kj$" ) ))
( "circling"  v ( s @@1 k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "circuit"  n ( s @@1 k i t ) ((pos "K6%" ) ))
( "circuitous"  j ( s @@1 k y uu1 i t @ s ) ((pos "OA%" ) ))
( "circuits"  n ( s @@1 k i t s ) ((pos "Kj%" ) ))
( "circular"  n ( s @@1 k y u l @ r ) ((pos "K6%" "OA%" ) ))
( "circularity"  n ( s @@2 k y u l a1 r i t ii ) ((pos "L@%" ) ))
( "circularize"  v ( s @@1 k y u l @ r ai z ) ((pos "H2%" ) (subcat )))
( "circularized"  v ( s @@1 k y u l @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "circularizes"  v ( s @@1 k y u l @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "circularizing"  v ( s @@1 k y u l @ r ai z i ng ) ((pos "Hb$" ) (subcat )))
( "circulars"  n ( s @@1 k y u l @ z ) ((pos "Kj%" ) ))
( "circulate"  v ( s @@1 k y u l ei t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "circulated"  v ( s @@1 k y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "circulates"  v ( s @@1 k y u l ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "circulating"  v ( s @@1 k y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "circulation"  n ( s @@2 k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "circulations"  n ( s @@2 k y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "circumcise"  v ( s @@1 k @ m s ai z ) ((pos "H2%" ) ))
( "circumcised"  v ( s @@1 k @ m s ai z d ) ((pos "Hc%" "Hd%" ) ))
( "circumcises"  v ( s @@1 k @ m s ai z i z ) ((pos "Ha%" ) ))
( "circumcising"  v ( s @@1 k @ m s ai z i ng ) ((pos "Hb%" ) ))
( "circumcision"  n ( s @@2 k @ m s i1 zh n ) ((pos "K6%" ) ))
( "circumcisions"  n ( s @@2 k @ m s i1 zh n z ) ((pos "Kj$" ) ))
( "circumference"  n ( s @@1 k uh1 m f @ r @ n s ) ((pos "K6%" ) ))
( "circumferences"  n ( s @@1 k uh1 m f @ r @ n s i z ) ((pos "Kj%" ) ))
( "circumflex"  n ( s @@1 k @ m f l e k s ) ((pos "K7%" ) ))
( "circumflexes"  n ( s @@1 k @ m f l e k s i z ) ((pos "Kj$" ) ))
( "circumlocution"  n ( s @@2 k @ m l @ k y uu1 sh @ n ) ((pos "M6%" ) ))
( "circumlocutions"  n ( s @@2 k @ m l @ k y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "circumnavigate"  v ( s @@2 k @ m n a1 v i g ei t ) ((pos "H2%" ) (subcat )))
( "circumnavigated"  v ( s @@2 k @ m n a1 v i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "circumnavigates"  v ( s @@2 k @ m n a1 v i g ei t s ) ((pos "Ha%" ) (subcat )))
( "circumnavigating"  v ( s @@2 k @ m n a1 v i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "circumnavigation"  n ( s @@2 k @ m n a2 v i g ei1 sh @ n ) ((pos "K6%" ) ))
( "circumnavigations"  n ( s @@2 k @ m n a2 v i g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "circumscribe"  v ( s @@1 k @ m s k r ai b ) ((pos "H2%" ) (subcat )))
( "circumscribed"  v ( s @@1 k @ m s k r ai b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "circumscribes"  v ( s @@1 k @ m s k r ai b z ) ((pos "Ha%" ) (subcat )))
( "circumscribing"  v ( s @@1 k @ m s k r ai b i ng ) ((pos "Hb%" ) (subcat )))
( "circumscription"  n ( s @@2 k @ m s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "circumscriptions"  n ( s @@2 k @ m s k r i1 p sh @ n z ) ((pos "Mj$" ) ))
( "circumspect"  j ( s @@1 k @ m s p e k t ) ((pos "OA%" ) ))
( "circumspection"  n ( s @@2 k @ m s p e1 k sh @ n ) ((pos "L@%" ) ))
( "circumspectly"  a ( s @@1 k @ m s p e k t l ii ) ((pos "Pu%" ) ))
( "circumstance"  n ( s @@1 k @ m s t @ n s ) ((pos "K6%" ) ))
( "circumstances"  n ( s @@1 k @ m s t @ n s i z ) ((pos "Kj%" ) ))
( "circumstantial"  j ( s @@2 k @ m s t a1 n sh l ) ((pos "OA%" ) ))
( "circumstantially"  a ( s @@2 k @ m s t a1 n sh @ l ii ) ((pos "Pu%" ) ))
( "circumvent"  v ( s @@2 k @ m v e1 n t ) ((pos "H0%" ) (subcat )))
( "circumvented"  v ( s @@2 k @ m v e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "circumventing"  v ( s @@2 k @ m v e1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "circumvention"  n ( s @@2 k @ m v e1 n sh @ n ) ((pos "K6%" ) ))
( "circumventions"  n ( s @@2 k @ m v e1 n sh @ n z ) ((pos "Kj%" ) ))
( "circumvents"  v ( s @@2 k @ m v e1 n t s ) ((pos "Ha%" ) (subcat )))
( "circus"  n ( s @@1 k @ s ) ((pos "K7%" ) ))
( "circuses"  n ( s @@1 k @ s i z ) ((pos "Kj%" ) ))
( "cirrhosis"  n ( s i1 r ou1 s i s ) ((pos "L@%" ) ))
( "cirrus"  n ( s i1 r @ s ) ((pos "L@%" ) ))
( "cissies"  n ( s i1 s i z ) ((pos "Kj%" ) ))
( "cissy"  n ( s i1 s ii ) ((pos "K8%" "OA%" ) ))
( "cistern"  n ( s i1 s t @ n ) ((pos "K6%" ) ))
( "cisterns"  n ( s i1 s t @ n z ) ((pos "Kj%" ) ))
( "citadel"  n ( s i1 t @ d @ l ) ((pos "K6%" ) ))
( "citadels"  n ( s i1 t @ d @ l z ) ((pos "Kj%" ) ))
( "citation"  n ( s ai1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "citations"  n ( s ai1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "cite"  v ( s ai1 t ) ((pos "H2%" ) (subcat )))
( "cited"  v ( s ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cites"  v ( s ai1 t s ) ((pos "Ha%" ) (subcat )))
( "cities"  n ( s i1 t i z ) ((pos "Kj%" ) ))
( "citing"  v ( s ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "citizen"  n ( s i1 t i z @ n ) ((pos "K6%" ) ))
( "citizens"  n ( s i1 t i z @ n z ) ((pos "Kj%" ) ))
( "citizenship"  n ( s i1 t i z @ n sh i p ) ((pos "K6%" ) ))
( "citizenships"  n ( s i1 t i z @ n sh i p s ) ((pos "Kj$" ) ))
( "citric"  j ( s i1 t r i k ) ((pos "OA%" ) ))
( "citron"  n ( s i1 t r @ n ) ((pos "K6%" ) ))
( "citrons"  n ( s i1 t r @ n z ) ((pos "Kj$" ) ))
( "citrous"  j ( s i1 t r @ s ) ((pos "OA%" ) ))
( "citrus"  n ( s i1 t r @ s ) ((pos "K7%" ) ))
( "citruses"  n ( s i1 t r @ s i z ) ((pos "Kj$" ) ))
( "city"  n ( s i1 t ii ) ((pos "K8*" ) ))
( "civet"  n ( s i1 v i t ) ((pos "M6%" ) ))
( "civet-cat"  n ( s i1 v i t - k a t ) ((pos "K6$" ) ))
( "civet-cats"  n ( s i1 v i t - k a t s ) ((pos "Kj$" ) ))
( "civets"  n ( s i1 v i t s ) ((pos "Mj$" ) ))
( "civic"  j ( s i1 v i k ) ((pos "OA%" ) ))
( "civics"  n ( s i1 v i k s ) ((pos "Lk%" ) ))
( "civies"  n ( s i1 v i z ) ((pos "Kj$" ) ))
( "civil"  j ( s i1 v @ l ) ((pos "OA%" ) ))
( "civilian"  n ( s i1 v i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "civilians"  n ( s i1 v i1 l i@ n z ) ((pos "Kj%" ) ))
( "civilities"  n ( s i1 v i1 l i t i z ) ((pos "Mj%" ) ))
( "civility"  n ( s i1 v i1 l i t ii ) ((pos "M8%" ) ))
( "civilization"  n ( s i2 v i l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "civilizations"  n ( s i2 v i l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "civilize"  v ( s i1 v i l ai z ) ((pos "H2%" ) (subcat )))
( "civilized"  v ( s i1 v i l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "civilizes"  v ( s i1 v i l ai z i z ) ((pos "Ha%" ) (subcat )))
( "civilizing"  v ( s i1 v i l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "civilly"  a ( s i1 v @ l ii ) ((pos "Pu%" ) ))
( "civvies"  n ( s i1 v i z ) ((pos "Kj%" ) ))
( "clack"  v ( k l a1 k ) ((pos "I0%" "K6%" ) ))
( "clacked"  v ( k l a1 k t ) ((pos "Ic%" "Id%" ) ))
( "clacking"  v ( k l a1 k i ng ) ((pos "Ib%" ) ))
( "clacks"  v ( k l a1 k s ) ((pos "Ia%" "Kj%" ) ))
( "clad"  v ( k l a1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "claim"  v ( k l ei1 m ) ((pos "J0%" "M6%" ) (subcat "2A" "4A" "7A" )))
( "claimant"  n ( k l ei1 m @ n t ) ((pos "K6%" ) ))
( "claimants"  n ( k l ei1 m @ n t s ) ((pos "Kj%" ) ))
( "claimed"  v ( k l ei1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "4A" "7A" )))
( "claiming"  v ( k l ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" "4A" "7A" )))
( "claims"  v ( k l ei1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "4A" "7A" )))
( "clairvoyance"  n ( k l e@1 v oi1 @ n s ) ((pos "L@%" ) ))
( "clairvoyant"  n ( k l e@1 v oi1 @ n t ) ((pos "K6%" ) ))
( "clairvoyants"  n ( k l e@1 v oi1 @ n t s ) ((pos "Kj%" ) ))
( "clam"  v ( k l a1 m ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "clambake"  n ( k l a1 m b ei k ) ((pos "K6%" ) ))
( "clambakes"  n ( k l a1 m b ei k s ) ((pos "Kj$" ) ))
( "clamber"  v ( k l a1 m b @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "clambered"  v ( k l a1 m b @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "clambering"  v ( k l a1 m b @ r i ng ) ((pos "Ib%" ) (subcat )))
( "clambers"  v ( k l a1 m b @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "clammed"  v ( k l a1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "clammier"  j ( k l a1 m i@ r ) ((pos "Or%" ) ))
( "clammiest"  j ( k l a1 m i i s t ) ((pos "Os%" ) ))
( "clammily"  a ( k l a1 m i l ii ) ((pos "Pu%" ) ))
( "clamming"  v ( k l a1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "clammy"  j ( k l a1 m ii ) ((pos "OD%" ) ))
( "clamorous"  j ( k l a1 m @ r @ s ) ((pos "OA%" ) ))
( "clamour"  v ( k l a1 m @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "clamoured"  v ( k l a1 m @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "clamouring"  v ( k l a1 m @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "clamours"  v ( k l a1 m @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "clamp"  v ( k l a1 m p ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "clamp-down"  n ( k l a1 m p - d au n ) ((pos "K6%" ) ))
( "clamp-downs"  n ( k l a1 m p - d au n z ) ((pos "Kj%" ) ))
( "clamped"  v ( k l a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "clamping"  v ( k l a1 m p i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "clamps"  v ( k l a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "clams"  v ( k l a1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "clan"  n ( k l a1 n ) ((pos "K6%" ) ))
( "clandestine"  j ( k l a1 n d e1 s t i n ) ((pos "OA%" ) ))
( "clang"  v ( k l a1 ng ) ((pos "J0%" "K6%" ) ))
( "clanged"  v ( k l a1 ng d ) ((pos "Jc%" "Jd%" ) ))
( "clanger"  n ( k l a1 ng @ r ) ((pos "K6%" ) ))
( "clangers"  n ( k l a1 ng @ z ) ((pos "Kj%" ) ))
( "clanging"  v ( k l a1 ng i ng ) ((pos "Jb%" ) ))
( "clangorous"  j ( k l a1 ng @ r @ s ) ((pos "OA$" ) ))
( "clangour"  n ( k l a1 ng @ r ) ((pos "L@%" ) ))
( "clangs"  v ( k l a1 ng z ) ((pos "Ja%" "Kj%" ) ))
( "clank"  v ( k l a1 ng k ) ((pos "J0%" "K6%" ) ))
( "clanked"  v ( k l a1 ng k t ) ((pos "Jc%" "Jd%" ) ))
( "clanking"  v ( k l a1 ng k i ng ) ((pos "Jb%" ) ))
( "clanks"  v ( k l a1 ng k s ) ((pos "Ja%" "Kj%" ) ))
( "clannish"  j ( k l a1 n i sh ) ((pos "OA%" ) ))
( "clannishly"  a ( k l a1 n i sh l ii ) ((pos "Pu%" ) ))
( "clans"  n ( k l a1 n z ) ((pos "Kj%" ) ))
( "clansman"  n ( k l a1 n z m @ n ) ((pos "Ki%" ) ))
( "clansmen"  n ( k l a1 n z m @ n ) ((pos "Kj%" ) ))
( "clap"  v ( k l a1 p ) ((pos "J4%" "M6%" ) (subcat "2A" "2B" "6A" "14" "15A" )))
( "clapboard"  n ( k l a1 p - b oo d ) ((pos "K6%" ) ))
( "clapboards"  n ( k l a1 p - b oo d z ) ((pos "Kj$" ) ))
( "clapped"  v ( k l a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "6A" "14" "15A" )))
( "clapper"  n ( k l a1 p @ r ) ((pos "K6%" ) ))
( "clapperboard"  n ( k l a1 p @ b oo d ) ((pos "K6%" ) ))
( "clapperboards"  n ( k l a1 p @ b oo d z ) ((pos "Kj%" ) ))
( "clappers"  n ( k l a1 p @ z ) ((pos "Kj%" ) ))
( "clapping"  v ( k l a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "6A" "14" "15A" )))
( "claps"  v ( k l a1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "6A" "14" "15A" )))
( "claptrap"  n ( k l a1 p t r a p ) ((pos "L@%" ) ))
( "claque"  n ( k l a1 k ) ((pos "K6$" ) ))
( "claques"  n ( k l a1 k s ) ((pos "Kj$" ) ))
( "claret"  n ( k l a1 r @ t ) ((pos "M6%" ) ))
( "clarets"  n ( k l a1 r @ t s ) ((pos "Mj%" ) ))
( "clarification"  n ( k l a2 r i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "clarified"  v ( k l a1 r i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "clarifies"  v ( k l a1 r i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "clarify"  v ( k l a1 r i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "clarifying"  v ( k l a1 r i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "clarinet"  n ( k l a2 r i n e1 t ) ((pos "K6%" ) ))
( "clarinetist"  n ( k l a2 r i n e1 t i s t ) ((pos "K6$" ) ))
( "clarinetists"  n ( k l a2 r i n e1 t i s t s ) ((pos "Kj$" ) ))
( "clarinets"  n ( k l a2 r i n e1 t s ) ((pos "Kj%" ) ))
( "clarinettist"  n ( k l a2 r i n e1 t i s t ) ((pos "K6%" ) ))
( "clarinettists"  n ( k l a2 r i n e1 t i s t s ) ((pos "Kj%" ) ))
( "clarion"  n ( k l a1 r i@ n ) ((pos "K6%" ) ))
( "clarions"  n ( k l a1 r i@ n z ) ((pos "Kj$" ) ))
( "clarity"  n ( k l a1 r i t ii ) ((pos "L@%" ) ))
( "clash"  v ( k l a1 sh ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" "6A" )))
( "clashed"  v ( k l a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "clashes"  v ( k l a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "clashing"  v ( k l a1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "clasp"  v ( k l aa1 s p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "clasp-knife"  n ( k l aa1 s p - n ai f ) ((pos "Ki%" ) ))
( "clasp-knives"  n ( k l aa1 s p - n ai v z ) ((pos "Kj%" ) ))
( "clasped"  v ( k l aa1 s p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "clasping"  v ( k l aa1 s p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "clasps"  v ( k l aa1 s p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "class"  v ( k l aa1 s ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "class-conscious"  j ( k l aa1 s - k o1 n sh @ s ) ((pos "OA%" ) ))
( "class-feeling"  n ( k l aa1 s - f ii1 l i ng ) ((pos "L@%" ) ))
( "class-fellow"  n ( k l aa1 s - f e l ou ) ((pos "K6%" ) ))
( "class-fellows"  n ( k l aa1 s - f e l ou z ) ((pos "Kj%" ) ))
( "class-list"  n ( k l aa1 s - l i s t ) ((pos "K6%" ) ))
( "class-lists"  n ( k l aa1 s - l i s t s ) ((pos "Kj%" ) ))
( "class-warfare"  n ( k l aa1 s - w oo1 f e@ r ) ((pos "L@%" ) ))
( "classed"  v ( k l aa1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "classes"  v ( k l aa1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "classic"  n ( k l a1 s i k ) ((pos "K6%" "OA%" ) ))
( "classical"  j ( k l a1 s i k l ) ((pos "OA%" ) ))
( "classically"  a ( k l a1 s i k l ii ) ((pos "Pu%" ) ))
( "classicism"  n ( k l a1 s i s i z m ) ((pos "L@%" ) ))
( "classicist"  n ( k l a1 s i s i s t ) ((pos "K6%" ) ))
( "classicists"  n ( k l a1 s i s i s t s ) ((pos "Kj%" ) ))
( "classics"  n ( k l a1 s i k s ) ((pos "Kj%" "Lk%" ) ))
( "classier"  j ( k l aa1 s i@ r ) ((pos "Or%" ) ))
( "classiest"  j ( k l aa1 s i i s t ) ((pos "Os%" ) ))
( "classifiable"  j ( k l a1 s i f ai @ b l ) ((pos "OA%" ) ))
( "classification"  n ( k l a2 s i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "classifications"  n ( k l a2 s i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "classified"  v ( k l a1 s i f ai d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "classifies"  v ( k l a1 s i f ai z ) ((pos "Ha%" ) (subcat )))
( "classify"  v ( k l a1 s i f ai ) ((pos "H3%" ) (subcat )))
( "classifying"  v ( k l a1 s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "classing"  v ( k l aa1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "classless"  j ( k l aa1 s l @ s ) ((pos "OA%" ) ))
( "classmate"  n ( k l aa1 s m ei t ) ((pos "K6%" ) ))
( "classmates"  n ( k l aa1 s m ei t s ) ((pos "Kj%" ) ))
( "classroom"  n ( k l a1 s r uu m ) ((pos "K6%" ) ))
( "classrooms"  n ( k l a1 s r uu m z ) ((pos "Kj%" ) ))
( "classy"  j ( k l aa1 s ii ) ((pos "OD%" ) ))
( "clatter"  v ( k l a1 t @ r ) ((pos "J0%" "Ki%" ) (subcat "2A" "2C" )))
( "clattered"  v ( k l a1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "clattering"  v ( k l a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "clatters"  v ( k l a1 t @ z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "clause"  n ( k l oo1 z ) ((pos "K6%" ) ))
( "clauses"  n ( k l oo1 z i z ) ((pos "Kj%" ) ))
( "claustrophobia"  n ( k l oo2 s t r @ f ou1 b i@ ) ((pos "L@%" ) ))
( "claustrophobic"  j ( k l oo2 s t r @ f ou1 b i k ) ((pos "OA%" ) ))
( "clave"  v ( k l ei1 v ) ((pos "Ic$" ) (subcat )))
( "clavichord"  n ( k l a1 v i k oo d ) ((pos "K6%" ) ))
( "clavichords"  n ( k l a1 v i k oo d z ) ((pos "Kj%" ) ))
( "clavicle"  n ( k l a1 v i k l ) ((pos "K6%" ) ))
( "clavicles"  n ( k l a1 v i k l z ) ((pos "Kj%" ) ))
( "claw"  v ( k l oo1 ) ((pos "H0%" "K6%" ) (subcat "3A" "6A" )))
( "clawback"  n ( k l oo1 b a k ) ((pos "L@%" ) ))
( "clawed"  v ( k l oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" )))
( "clawhammer"  n ( k l oo1 h a m @ r ) ((pos "K6%" ) ))
( "clawhammers"  n ( k l oo1 h a m @ z ) ((pos "Kj%" ) ))
( "clawing"  v ( k l oo1 i ng ) ((pos "Hb%" ) (subcat "3A" "6A" )))
( "claws"  v ( k l oo1 z ) ((pos "Ha%" "Kj%" ) (subcat "3A" "6A" )))
( "clay"  n ( k l ei1 ) ((pos "L@%" ) ))
( "clayey"  j ( k l ei1 ii ) ((pos "OA%" ) ))
( "clean"  v ( k l ii1 n ) ((pos "J0%" "K6%" "OC%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "clean-bowled"  j ( k l ii1 n - b ou1 l d ) ((pos "OA%" ) ))
( "clean-cut"  j ( k l ii1 n - k uh1 t ) ((pos "OA%" ) ))
( "clean-limbed"  j ( k l ii1 n - l i1 m d ) ((pos "OA%" ) ))
( "clean-living"  j ( k l ii1 n - l i1 v i ng ) ((pos "OA%" ) ))
( "clean-shaven"  j ( k l ii1 n - sh ei1 v n ) ((pos "OA%" ) ))
( "clean-up"  n ( k l ii1 n - uh p ) ((pos "K6%" ) ))
( "clean-ups"  n ( k l ii1 n - uh p s ) ((pos "Kj$" ) ))
( "cleaned"  v ( k l ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cleaner"  n ( k l ii1 n @ r ) ((pos "K6%" "Or%" ) ))
( "cleaners"  n ( k l ii1 n @ z ) ((pos "Kj%" ) ))
( "cleanest"  j ( k l ii1 n i s t ) ((pos "Os%" ) ))
( "cleaning"  v ( k l ii1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "cleanlier"  j ( k l e1 n l i@ r ) ((pos "Or$" ) ))
( "cleanliest"  j ( k l e1 n l i i s t ) ((pos "Os$" ) ))
( "cleanliness"  n ( k l e1 n l i n @ s ) ((pos "L@%" ) ))
( "cleanly"  j ( k l e1 n l ii ) ((pos "OD$" ) ))
( "cleanly"  a ( k l ii1 n l ii ) ((pos "Pu%" ) ))
( "cleans"  v ( k l ii1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "cleanse"  v ( k l e1 n z ) ((pos "H2%" ) (subcat )))
( "cleansed"  v ( k l e1 n z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cleanser"  n ( k l e1 n z @ r ) ((pos "M6%" ) ))
( "cleansers"  n ( k l e1 n z @ z ) ((pos "Mj%" ) ))
( "cleanses"  v ( k l e1 n z i z ) ((pos "Ha%" ) (subcat )))
( "cleansing"  v ( k l e1 n z i ng ) ((pos "Hb%" ) (subcat )))
( "clear"  v ( k l i@1 r ) ((pos "J0%" "K6%" "OC%" "Pu%" ) (subcat "2C" "6A" "14" )))
( "clear-cut"  a ( k l i@1 - k uh1 t ) ((pos "Pu%" ) ))
( "clear-headed"  j ( k l i@1 - h e1 d i d ) ((pos "OA%" ) ))
( "clear-sighted"  j ( k l i@1 - s ai1 t i d ) ((pos "OA%" ) ))
( "clearance"  n ( k l i@1 r @ n s ) ((pos "M6%" ) ))
( "clearances"  n ( k l i@1 r @ n s i z ) ((pos "Mj%" ) ))
( "cleared"  v ( k l i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" )))
( "clearer"  j ( k l i@1 r @ r ) ((pos "Or%" ) ))
( "clearest"  j ( k l i@1 r i s t ) ((pos "Os%" ) ))
( "clearing"  v ( k l i@1 r i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "14" )))
( "clearing-house"  n ( k l i@1 r i ng - h au s ) ((pos "K6%" ) ))
( "clearing-houses"  n ( k l i@1 r i ng - h au z i z ) ((pos "Kj%" ) ))
( "clearings"  n ( k l i@1 r i ng z ) ((pos "Kj%" ) ))
( "clearly"  a ( k l i@1 l ii ) ((pos "Pu%" ) ))
( "clearness"  n ( k l i@1 n @ s ) ((pos "L@%" ) ))
( "clears"  v ( k l i@1 z ) ((pos "Ja%" "Kj$" ) (subcat "2C" "6A" "14" )))
( "clearway"  n ( k l i@1 w ei ) ((pos "K6%" ) ))
( "clearways"  n ( k l i@1 w ei z ) ((pos "Kj%" ) ))
( "cleat"  n ( k l ii1 t ) ((pos "K6$" ) ))
( "cleats"  n ( k l ii1 t s ) ((pos "Kj$" ) ))
( "cleavage"  n ( k l ii1 v i jh ) ((pos "K6%" ) ))
( "cleavages"  n ( k l ii1 v i jh i z ) ((pos "Kj$" ) ))
( "cleave"  v ( k l ii1 v ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "14" "15A" "15B" )))
( "cleaved"  v ( k l ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" "15A" "15B" )))
( "cleaver"  n ( k l ii1 v @ r ) ((pos "K6%" ) ))
( "cleavers"  n ( k l ii1 v @ z ) ((pos "Kj%" ) ))
( "cleaves"  v ( k l ii1 v z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "14" "15A" "15B" )))
( "cleaving"  v ( k l ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" "15A" "15B" )))
( "clef"  n ( k l e1 f ) ((pos "K6%" ) ))
( "clefs"  n ( k l e1 f s ) ((pos "Kj%" ) ))
( "cleft"  v ( k l e1 f t ) ((pos "Jc%" "Jd%" "K6%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "clefts"  n ( k l e1 f t s ) ((pos "Kj%" ) ))
( "clematis"  n ( k l e1 m @ t i s ) ((pos "L@%" ) ))
( "clemency"  n ( k l e1 m @ n s ii ) ((pos "L@%" ) ))
( "clement"  j ( k l e1 m @ n t ) ((pos "OA%" ) ))
( "clench"  v ( k l e1 n ch ) ((pos "H1%" ) (subcat "6A" )))
( "clenched"  v ( k l e1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "clenches"  v ( k l e1 n ch i z ) ((pos "Ha%" ) (subcat "6A" )))
( "clenching"  v ( k l e1 n ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "clerestories"  n ( k l i@1 s t oo r i z ) ((pos "Kj$" ) ))
( "clerestory"  n ( k l i@1 s t oo r ii ) ((pos "K8$" ) ))
( "clergies"  n ( k l @@1 jh i z ) ((pos "Kj%" ) ))
( "clergy"  n ( k l @@1 jh ii ) ((pos "K8%" ) ))
( "clergyman"  n ( k l @@1 jh i m @ n ) ((pos "Ki%" ) ))
( "clergymen"  n ( k l @@1 jh i m @ n ) ((pos "Kj%" ) ))
( "cleric"  n ( k l e1 r i k ) ((pos "K6%" ) ))
( "clerical"  j ( k l e1 r i k l ) ((pos "OA%" ) ))
( "clerics"  n ( k l e1 r i k s ) ((pos "Kj%" ) ))
( "clerihew"  n ( k l e1 r i h y uu ) ((pos "K6$" ) ))
( "clerihews"  n ( k l e1 r i h y uu z ) ((pos "Kj$" ) ))
( "clerk"  v ( k l aa1 k ) ((pos "I0$" "K6%" ) ))
( "clerked"  v ( k l aa1 k t ) ((pos "Ic$" "Id$" ) ))
( "clerking"  v ( k l aa1 k i ng ) ((pos "Ib$" ) ))
( "clerks"  v ( k l aa1 k s ) ((pos "Ia$" "Kj%" ) ))
( "clever"  j ( k l e1 v @ r ) ((pos "OC%" ) ))
( "cleverer"  j ( k l e1 v @ r @ r ) ((pos "Or%" ) ))
( "cleverest"  j ( k l e1 v @ r i s t ) ((pos "Os%" ) ))
( "cleverly"  a ( k l e1 v @ l ii ) ((pos "Pu%" ) ))
( "cleverness"  n ( k l e1 v @ n @ s ) ((pos "L@%" ) ))
( "clew"  v ( k l uu1 ) ((pos "H0$" "K6$" ) ))
( "clewed"  v ( k l uu1 d ) ((pos "Hc$" "Hd$" ) ))
( "clewing"  v ( k l uu1 i ng ) ((pos "Hb$" ) ))
( "clews"  v ( k l uu1 z ) ((pos "Ha$" "Kj$" ) ))
( "clich_e"  n ( k l ii1 sh ei ) ((pos "K6%" ) ))
( "clich_es"  n ( k l ii1 sh ei z ) ((pos "Kj%" ) ))
( "click"  v ( k l i1 k ) ((pos "I0%" "K6%" ) (subcat )))
( "clicked"  v ( k l i1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "clicking"  v ( k l i1 k i ng ) ((pos "Ib%" ) (subcat )))
( "clicks"  v ( k l i1 k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "client"  n ( k l ai1 @ n t ) ((pos "K6%" ) ))
( "clientele"  n ( k l ii2 @ n t e1 l ) ((pos "K6%" ) ))
( "clienteles"  n ( k l ii2 @ n t e1 l z ) ((pos "Kj%" ) ))
( "clients"  n ( k l ai1 @ n t s ) ((pos "Kj%" ) ))
( "cliff"  n ( k l i1 f ) ((pos "K6%" ) ))
( "cliff-hanger"  n ( k l i1 f - h a ng @ r ) ((pos "K6%" ) ))
( "cliff-hangers"  n ( k l i1 f - h a ng @ z ) ((pos "Kj%" ) ))
( "cliffs"  n ( k l i1 f s ) ((pos "Kj%" ) ))
( "climacteric"  n ( k l ai1 m a1 k t @ r i k ) ((pos "K6$" ) ))
( "climacterics"  n ( k l ai1 m a1 k t @ r i k s ) ((pos "Kj$" ) ))
( "climactic"  j ( k l ai1 m a1 k t i k ) ((pos "OA%" ) ))
( "climate"  n ( k l ai1 m @ t ) ((pos "K6%" ) ))
( "climates"  n ( k l ai1 m @ t s ) ((pos "Kj%" ) ))
( "climatic"  j ( k l ai1 m a1 t i k ) ((pos "OA%" ) ))
( "climatically"  a ( k l ai1 m a1 t i k l ii ) ((pos "Pu%" ) ))
( "climatology"  n ( k l ai2 m @ t o1 l @ jh ii ) ((pos "L@%" ) ))
( "climax"  v ( k l ai1 m a k s ) ((pos "J1%" "K7%" ) ))
( "climaxed"  v ( k l ai1 m a k s t ) ((pos "Jc%" "Jd%" ) ))
( "climaxes"  v ( k l ai1 m a k s i z ) ((pos "Ja%" "Kj%" ) ))
( "climaxing"  v ( k l ai1 m a k s i ng ) ((pos "Jb%" ) ))
( "climb"  v ( k l ai1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "climb-down"  n ( k l ai1 m - d au n ) ((pos "K6%" ) ))
( "climb-downs"  n ( k l ai1 m - d au n z ) ((pos "Kj$" ) ))
( "climbed"  v ( k l ai1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "climber"  n ( k l ai1 m @ r ) ((pos "K6%" ) ))
( "climbers"  n ( k l ai1 m @ z ) ((pos "Kj%" ) ))
( "climbing"  v ( k l ai1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "climbs"  v ( k l ai1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "clime"  n ( k l ai1 m ) ((pos "K6$" ) ))
( "climes"  n ( k l ai1 m z ) ((pos "Kj%" ) ))
( "clinch"  v ( k l i1 n ch ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "clinched"  v ( k l i1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "clincher"  n ( k l i1 n ch @ r ) ((pos "K6%" ) ))
( "clinchers"  n ( k l i1 n ch @ z ) ((pos "Kj$" ) ))
( "clinches"  v ( k l i1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "clinching"  v ( k l i1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cling"  v ( k l i1 ng ) ((pos "I5%" ) (subcat "2C" )))
( "clinging"  v ( k l i1 ng i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "clings"  v ( k l i1 ng z ) ((pos "Ia%" ) (subcat "2C" )))
( "clinic"  n ( k l i1 n i k ) ((pos "K6%" ) ))
( "clinical"  j ( k l i1 n i k l ) ((pos "OA%" ) ))
( "clinically"  a ( k l i1 n i k l ii ) ((pos "Pu%" ) ))
( "clinics"  n ( k l i1 n i k s ) ((pos "Kj%" ) ))
( "clink"  v ( k l i1 ng k ) ((pos "J0%" "K6%" ) ))
( "clinked"  v ( k l i1 ng k t ) ((pos "Jc%" "Jd%" ) ))
( "clinker"  n ( k l i1 ng k @ r ) ((pos "M6%" ) ))
( "clinker-built"  j ( k l i1 ng k @ - b i l t ) ((pos "OA$" ) ))
( "clinkers"  n ( k l i1 ng k @ z ) ((pos "Mj%" ) ))
( "clinking"  v ( k l i1 ng k i ng ) ((pos "Jb%" ) ))
( "clinks"  v ( k l i1 ng k s ) ((pos "Ja%" "Kj%" ) ))
( "clip"  v ( k l i1 p ) ((pos "H4%" "K6%" ) (subcat "2C" "6A" "15A" "15B" )))
( "clip-joint"  n ( k l i1 p - jh oi n t ) ((pos "K6%" ) ))
( "clip-joints"  n ( k l i1 p - jh oi n t s ) ((pos "Kj%" ) ))
( "clip-on"  j ( k l i1 p - o n ) ((pos "OA%" ) ))
( "clipped"  v ( k l i1 p t ) ((pos "Hc%" "Hd%" ) (subcat "2C" "6A" "15A" "15B" )))
( "clipper"  n ( k l i1 p @ r ) ((pos "K6%" ) ))
( "clippers"  n ( k l i1 p @ z ) ((pos "Kj%" ) ))
( "clipping"  v ( k l i1 p i ng ) ((pos "Hb%" "K6%" ) (subcat "2C" "6A" "15A" "15B" )))
( "clippings"  n ( k l i1 p i ng z ) ((pos "Kj%" ) ))
( "clips"  v ( k l i1 p s ) ((pos "Ha%" "Kj%" ) (subcat "2C" "6A" "15A" "15B" )))
( "clique"  n ( k l ii1 k ) ((pos "K6%" ) ))
( "cliques"  n ( k l ii1 k s ) ((pos "Kj%" ) ))
( "cliquish"  j ( k l ii1 k i sh ) ((pos "OA%" ) ))
( "clitoris"  n ( k l i1 t @ r i s ) ((pos "K7%" ) ))
( "clitorises"  n ( k l i1 t @ r i s i z ) ((pos "Kj$" ) ))
( "cloak"  v ( k l ou1 k ) ((pos "H0%" "K6%" ) (subcat )))
( "cloaked"  v ( k l ou1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cloaking"  v ( k l ou1 k i ng ) ((pos "Hb%" ) (subcat )))
( "cloakroom"  n ( k l ou1 k r uu m ) ((pos "K6%" ) ))
( "cloakrooms"  n ( k l ou1 k r uu m z ) ((pos "Kj%" ) ))
( "cloaks"  v ( k l ou1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "clobber"  v ( k l o1 b @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "clobbered"  v ( k l o1 b @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "clobbering"  v ( k l o1 b @ r i ng ) ((pos "Hb%" ) (subcat )))
( "clobbers"  v ( k l o1 b @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cloche"  n ( k l o1 sh ) ((pos "K6%" ) ))
( "cloches"  n ( k l o1 sh i z ) ((pos "Kj%" ) ))
( "clock"  v ( k l o1 k ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "clock-dial"  n ( k l o1 k - d ai @ l ) ((pos "K6$" ) ))
( "clock-dials"  n ( k l o1 k - d ai @ l z ) ((pos "Kj$" ) ))
( "clock-face"  n ( k l o1 k - f ei s ) ((pos "K6%" ) ))
( "clock-faces"  n ( k l o1 k - f ei s i z ) ((pos "Kj%" ) ))
( "clock-golf"  n ( k l o1 k - g o1 l f ) ((pos "L@$" ) ))
( "clock-tower"  n ( k l o1 k - t au @ r ) ((pos "K6%" ) ))
( "clock-towers"  n ( k l o1 k - t au @ z ) ((pos "Kj%" ) ))
( "clock-watching"  n ( k l o1 k - w o ch i ng ) ((pos "L@%" ) ))
( "clocked"  v ( k l o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "clocking"  v ( k l o1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "clocks"  v ( k l o1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "clockwise"  a ( k l o1 k w ai z ) ((pos "Pu%" ) ))
( "clockwork"  n ( k l o1 k w @@ k ) ((pos "K6%" ) ))
( "clockworks"  n ( k l o1 k w @@ k s ) ((pos "Kj$" ) ))
( "clod"  n ( k l o1 d ) ((pos "K6%" ) ))
( "clodhopper"  n ( k l o1 d h o p @ r ) ((pos "K6%" ) ))
( "clodhoppers"  n ( k l o1 d h o p @ z ) ((pos "Kj%" ) ))
( "clods"  n ( k l o1 d z ) ((pos "Kj%" ) ))
( "clog"  v ( k l o1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "clog-dance"  n ( k l o1 g - d aa n s ) ((pos "K6%" ) ))
( "clog-dances"  n ( k l o1 g - d aa n s i z ) ((pos "Kj%" ) ))
( "clogged"  v ( k l o1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cloggier"  j ( k l o1 g i@ r ) ((pos "Or$" ) ))
( "cloggiest"  j ( k l o1 g i i s t ) ((pos "Os$" ) ))
( "clogging"  v ( k l o1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "cloggy"  j ( k l o1 g ii ) ((pos "OD%" ) ))
( "clogs"  v ( k l o1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "cloisonn_e"  n ( k l w aa1 z o1 n ei ) ((pos "L@$" ) ))
( "cloister"  v ( k l oi1 s t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "cloistered"  v ( k l oi1 s t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cloistering"  v ( k l oi1 s t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "cloisters"  v ( k l oi1 s t @ z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "clone"  n ( k l ou1 n ) ((pos "K6%" ) ))
( "clones"  n ( k l ou1 n z ) ((pos "Kj%" ) ))
( "close"  n ( k l ou1 s ) ((pos "K6%" "OB*" "Pu*" ) ))
( "close"  v ( k l ou1 z ) ((pos "J2*" "Ki%" ) (subcat "2A" "2C" "3A" "6A" )))
( "close-cropped"  j ( k l ou1 s - k r o1 p t ) ((pos "OA%" ) ))
( "close-cut"  j ( k l ou1 s - k uh1 t ) ((pos "OA%" ) ))
( "close-down"  n ( k l ou1 z - d au n ) ((pos "K6%" ) ))
( "close-downs"  n ( k l ou1 z - d au n z ) ((pos "Kj$" ) ))
( "close-fisted"  j ( k l ou1 s - f i1 s t i d ) ((pos "OA%" ) ))
( "close-fitting"  j ( k l ou1 s - f i1 t i ng ) ((pos "OA%" ) ))
( "close-grained"  j ( k l ou1 s - g r ei1 n d ) ((pos "OA%" ) ))
( "close-hauled"  j ( k l ou1 s - h oo1 l d ) ((pos "OA$" ) ))
( "close-set"  j ( k l ou1 s - s e t ) ((pos "OA%" ) ))
( "close-up"  n ( k l ou1 s - uh p ) ((pos "K6%" ) ))
( "close-ups"  n ( k l ou1 s - uh p s ) ((pos "Kj%" ) ))
( "closed"  v ( k l ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "closely"  a ( k l ou1 s l ii ) ((pos "Pu%" ) ))
( "closeness"  n ( k l ou1 s n @ s ) ((pos "L@%" ) ))
( "closer"  j ( k l ou1 s @ r ) ((pos "Or%" ) ))
( "closes"  n ( k l ou1 s i z ) ((pos "Kj$" ) ))
( "closes"  v ( k l ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "closest"  j ( k l ou1 s i s t ) ((pos "Os%" ) ))
( "closet"  v ( k l o1 z i t ) ((pos "H0%" "K6%" "Oq%" ) ))
( "closeted"  v ( k l o1 z i t i d ) ((pos "Hc%" "Hd%" ) ))
( "closeting"  v ( k l o1 z i t i ng ) ((pos "Hb$" ) ))
( "closets"  v ( k l o1 z i t s ) ((pos "Ha$" "Kj%" ) ))
( "closing"  v ( k l ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "closure"  n ( k l ou1 zh @ r ) ((pos "M6%" ) ))
( "closures"  n ( k l ou1 zh @ z ) ((pos "Mj%" ) ))
( "clot"  v ( k l o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "cloth"  n ( k l o1 th ) ((pos "M6%" ) ))
( "clothe"  v ( k l ou1 dh ) ((pos "H2%" ) (subcat )))
( "clothed"  v ( k l ou1 dh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "clothes"  v ( k l ou1 dh z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "clothes-basket"  n ( k l ou1 dh z - b aa s k i t ) ((pos "K6%" ) ))
( "clothes-baskets"  n ( k l ou1 dh z - b aa s k i t s ) ((pos "Kj%" ) ))
( "clothes-hanger"  n ( k l ou1 dh z - h a ng @ r ) ((pos "K6%" ) ))
( "clothes-hangers"  n ( k l ou1 dh z - h a ng @ z ) ((pos "Kj%" ) ))
( "clothes-moth"  n ( k l ou1 dh z - m o th ) ((pos "K6%" ) ))
( "clothes-moths"  n ( k l ou1 dh z - m o th s ) ((pos "Kj%" ) ))
( "clothes-peg"  n ( k l ou1 dh z - p e g ) ((pos "K6%" ) ))
( "clothes-pegs"  n ( k l ou1 dh z - p e g z ) ((pos "Kj%" ) ))
( "clothes-pin"  n ( k l ou1 dh z - p i n ) ((pos "K6$" ) ))
( "clothes-pins"  n ( k l ou1 dh z - p i n z ) ((pos "Kj$" ) ))
( "clotheshorse"  n ( k l ou1 dh z h oo s ) ((pos "K6%" ) ))
( "clotheshorses"  n ( k l ou1 dh z h oo s i z ) ((pos "Kj%" ) ))
( "clothesline"  n ( k l ou1 dh z l ai n ) ((pos "K6%" ) ))
( "clotheslines"  n ( k l ou1 dh z l ai n z ) ((pos "Kj%" ) ))
( "clothier"  n ( k l ou1 dh i@ r ) ((pos "K6%" ) ))
( "clothiers"  n ( k l ou1 dh i@ z ) ((pos "Kj%" ) ))
( "clothing"  v ( k l ou1 dh i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "cloths"  n ( k l o1 th s ) ((pos "Mj%" ) ))
( "clots"  v ( k l o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "clotted"  v ( k l o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "clotting"  v ( k l o1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cloud"  v ( k l au1 d ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" )))
( "cloud-bank"  n ( k l au1 d - b a ng k ) ((pos "K6%" ) ))
( "cloud-banks"  n ( k l au1 d - b a ng k s ) ((pos "Kj%" ) ))
( "cloud-capped"  j ( k l au1 d - k a p t ) ((pos "OA%" ) ))
( "cloud-cuckoo-land"  n ( k l au2 d - k u1 k uu - l a n d ) ((pos "Ki%" ) ))
( "cloudburst"  n ( k l au1 d b @@ s t ) ((pos "K6%" ) ))
( "cloudbursts"  n ( k l au1 d b @@ s t s ) ((pos "Kj%" ) ))
( "clouded"  v ( k l au1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "cloudier"  j ( k l au1 d i@ r ) ((pos "Or%" ) ))
( "cloudiest"  j ( k l au1 d i i s t ) ((pos "Os%" ) ))
( "clouding"  v ( k l au1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "cloudless"  j ( k l au1 d l @ s ) ((pos "OA%" ) ))
( "clouds"  v ( k l au1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" )))
( "cloudy"  j ( k l au1 d ii ) ((pos "OD%" ) ))
( "clout"  v ( k l au1 t ) ((pos "H0%" "K6%" ) ))
( "clouted"  v ( k l au1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "clouting"  v ( k l au1 t i ng ) ((pos "Hb%" ) ))
( "clouts"  v ( k l au1 t s ) ((pos "Ha%" "Kj$" ) ))
( "clove"  v ( k l ou1 v ) ((pos "Jc%" "K6%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "clove hitch"  n ( k l ou1 v - h i ch ) ((pos "K7%" ) ))
( "clove hitches"  n ( k l ou1 v - h i ch i z ) ((pos "Kj%" ) ))
( "cloven"  v ( k l ou1 v n ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "clover"  n ( k l ou1 v @ r ) ((pos "L@%" ) ))
( "cloverleaf"  n ( k l ou1 v @ l ii f ) ((pos "Ki%" ) ))
( "cloverleaves"  n ( k l ou1 v @ l ii v z ) ((pos "Kj%" ) ))
( "cloves"  n ( k l ou1 v z ) ((pos "Kj%" ) ))
( "clown"  v ( k l au1 n ) ((pos "I0%" "K6%" ) (subcat )))
( "clowned"  v ( k l au1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "clowning"  v ( k l au1 n i ng ) ((pos "Ib%" ) (subcat )))
( "clownish"  j ( k l au1 n i sh ) ((pos "OA%" ) ))
( "clowns"  v ( k l au1 n z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "cloy"  v ( k l oi1 ) ((pos "J0%" ) (subcat "2A" )))
( "cloyed"  v ( k l oi1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cloying"  v ( k l oi1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cloys"  v ( k l oi1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "club"  v ( k l uh1 b ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" )))
( "club-footed"  j ( k l uh1 b - f u t i d ) ((pos "OA%" ) ))
( "clubbable"  j ( k l uh1 b @ b @ l ) ((pos "OA%" ) ))
( "clubbed"  v ( k l uh1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "clubbing"  v ( k l uh1 b i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "clubfoot"  n ( k l uh1 b f u t ) ((pos "Ki%" ) ))
( "clubhouse"  n ( k l uh1 b h au s ) ((pos "K6%" ) ))
( "clubhouses"  n ( k l uh1 b h au z i z ) ((pos "Kj%" ) ))
( "clubs"  v ( k l uh1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "cluck"  v ( k l uh1 k ) ((pos "I0%" "K6%" ) ))
( "clucked"  v ( k l uh1 k t ) ((pos "Ic%" "Id%" ) ))
( "clucking"  v ( k l uh1 k i ng ) ((pos "Ib%" ) ))
( "clucks"  v ( k l uh1 k s ) ((pos "Ia%" "Kj%" ) ))
( "clue"  n ( k l uu1 ) ((pos "K6%" ) ))
( "clues"  n ( k l uu1 z ) ((pos "Kj%" ) ))
( "clump"  v ( k l uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "clumped"  v ( k l uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "clumping"  v ( k l uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "clumps"  v ( k l uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "clumsier"  j ( k l uh1 m z i@ r ) ((pos "Or%" ) ))
( "clumsiest"  j ( k l uh1 m z i i s t ) ((pos "Os%" ) ))
( "clumsily"  a ( k l uh1 m z i l ii ) ((pos "Pu%" ) ))
( "clumsiness"  n ( k l uh1 m z i n @ s ) ((pos "L@%" ) ))
( "clumsy"  j ( k l uh1 m z ii ) ((pos "OD%" ) ))
( "clung"  v ( k l uh1 ng ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "clunk"  v ( k l uh1 ng k ) ((pos "I0%" "K6%" ) ))
( "clunked"  v ( k l uh1 ng k t ) ((pos "Ic%" "Id%" ) ))
( "clunking"  v ( k l uh1 ng k i ng ) ((pos "Ib%" ) ))
( "clunks"  v ( k l uh1 ng k s ) ((pos "Ia%" "Kj%" ) ))
( "cluster"  v ( k l uh1 s t @ r ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "clustered"  v ( k l uh1 s t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "clustering"  v ( k l uh1 s t @ r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "clusters"  v ( k l uh1 s t @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "clutch"  v ( k l uh1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "3A" )))
( "clutched"  v ( k l uh1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "clutches"  v ( k l uh1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "clutching"  v ( k l uh1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "clutter"  v ( k l uh1 t @ r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "cluttered"  v ( k l uh1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cluttering"  v ( k l uh1 t @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "clutters"  v ( k l uh1 t @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "cm"  n ( s ii2 e1 m ) ((pos "Y]%" ) ))
( "co-"  j ( k ou - ) ((pos "U-%" ) ))
( "co-ed"  n ( k ou1 - e d ) ((pos "K6$" ) ))
( "co-eds"  n ( k ou1 - e d z ) ((pos "Kj$" ) ))
( "co-op"  n ( k ou1 - o p ) ((pos "K6%" ) ))
( "co-ops"  n ( k ou1 - o p s ) ((pos "Kj%" ) ))
( "co-opt"  v ( k ou1 - o1 p t ) ((pos "H0%" ) (subcat "6A" )))
( "co-opted"  v ( k ou1 - o1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "co-opting"  v ( k ou1 - o1 p t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "co-opts"  v ( k ou1 - o1 p t s ) ((pos "Ha%" ) (subcat "6A" )))
( "co-respondent"  n ( k ou2 - r i s p o1 n d @ n t ) ((pos "K6%" ) ))
( "co-respondents"  n ( k ou2 - r i s p o1 n d @ n t s ) ((pos "Kj%" ) ))
( "co-star"  n ( k ou1 - s t aa r ) ((pos "K6%" ) ))
( "co-star"  v ( k ou2 - s t aa1 r ) ((pos "J4%" ) (subcat "6A" )))
( "co-starred"  v ( k ou2 - s t aa1 d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "co-starring"  v ( k ou2 - s t aa1 r i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "co-stars"  n ( k ou1 - s t aa z ) ((pos "Kj%" ) ))
( "co-stars"  v ( k ou2 - s t aa1 z ) ((pos "Ja%" ) (subcat "6A" )))
( "coach"  v ( k ou1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" )))
( "coach-builder"  n ( k ou1 ch - b i l d @ r ) ((pos "K6%" ) ))
( "coach-builders"  n ( k ou1 ch - b i l d @ z ) ((pos "Kj%" ) ))
( "coached"  v ( k ou1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "coaches"  v ( k ou1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "coaching"  v ( k ou1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "coachman"  n ( k ou1 ch m @ n ) ((pos "Ki%" ) ))
( "coachmen"  n ( k ou1 ch m e n ) ((pos "Kj%" ) ))
( "coagulate"  v ( k ou1 a1 g y u l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "coagulated"  v ( k ou1 a1 g y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "coagulates"  v ( k ou1 a1 g y u l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "coagulating"  v ( k ou1 a1 g y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "coagulation"  n ( k ou1 a2 g y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "coagulations"  n ( k ou1 a2 g y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "coal"  v ( k ou1 l ) ((pos "J0$" "M6%" ) (subcat "2A" )))
( "coal-gas"  n ( k ou1 l - g a s ) ((pos "L@%" ) ))
( "coal-hole"  n ( k ou1 l - h ou l ) ((pos "K6%" ) ))
( "coal-holes"  n ( k ou1 l - h ou l z ) ((pos "Kj%" ) ))
( "coal-house"  n ( k ou1 l - h au s ) ((pos "K6%" ) ))
( "coal-houses"  n ( k ou1 l - h au z i z ) ((pos "Kj%" ) ))
( "coal-scuttle"  n ( k ou1 l - s k uh t l ) ((pos "K6%" ) ))
( "coal-scuttles"  n ( k ou1 l - s k uh t l z ) ((pos "Kj%" ) ))
( "coal-seam"  n ( k ou1 l - s ii m ) ((pos "K6%" ) ))
( "coal-seams"  n ( k ou1 l - s ii m z ) ((pos "Kj%" ) ))
( "coal-tar"  n ( k ou1 l - t aa r ) ((pos "L@%" ) ))
( "coaled"  v ( k ou1 l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "coalesce"  v ( k ou2 @ l e1 s ) ((pos "I2%" ) ))
( "coalesced"  v ( k ou2 @ l e1 s t ) ((pos "Ic%" "Id%" ) ))
( "coalescence"  n ( k ou2 @ l e1 s @ n s ) ((pos "M6%" ) ))
( "coalescences"  n ( k ou2 @ l e1 s @ n s i z ) ((pos "Mj$" ) ))
( "coalesces"  v ( k ou2 @ l e1 s i z ) ((pos "Ia%" ) ))
( "coalescing"  v ( k ou2 @ l e1 s i ng ) ((pos "Ib%" ) ))
( "coalface"  n ( k ou1 l f ei s ) ((pos "K6%" ) ))
( "coalfaces"  n ( k ou1 l f ei s i z ) ((pos "Kj%" ) ))
( "coalfield"  n ( k ou1 l f ii l d ) ((pos "K6%" ) ))
( "coalfields"  n ( k ou1 l f ii l d z ) ((pos "Kj%" ) ))
( "coaling"  v ( k ou1 l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "coaling-station"  n ( k ou1 l i ng - s t ei sh @ n ) ((pos "K6$" ) ))
( "coaling-stations"  n ( k ou1 l i ng - s t ei sh @ n z ) ((pos "Kj$" ) ))
( "coalition"  n ( k ou2 @ l i1 sh @ n ) ((pos "M6%" ) ))
( "coalitions"  n ( k ou2 @ l i1 sh @ n z ) ((pos "Mj%" ) ))
( "coalman"  n ( k ou1 l m @ n ) ((pos "Ki%" ) ))
( "coalmen"  n ( k ou1 l m e n ) ((pos "Kj%" ) ))
( "coalmine"  n ( k ou1 l m ai n ) ((pos "K6%" ) ))
( "coalmines"  n ( k ou1 l m ai n z ) ((pos "Kj%" ) ))
( "coalpit"  n ( k ou1 l p i t ) ((pos "K6%" ) ))
( "coalpits"  n ( k ou1 l p i t s ) ((pos "Kj%" ) ))
( "coals"  v ( k ou1 l z ) ((pos "Ja$" "Mj%" ) (subcat "2A" )))
( "coaming"  n ( k ou1 m i ng ) ((pos "K6$" ) ))
( "coamings"  n ( k ou1 m i ng z ) ((pos "Kj$" ) ))
( "coarse"  j ( k oo1 s ) ((pos "OB%" ) ))
( "coarsely"  a ( k oo1 s l ii ) ((pos "Pu%" ) ))
( "coarsen"  v ( k oo1 s n ) ((pos "J0%" ) (subcat "2A" )))
( "coarsened"  v ( k oo1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "coarseness"  n ( k oo1 s n @ s ) ((pos "L@%" ) ))
( "coarsening"  v ( k oo1 s n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "coarsens"  v ( k oo1 s n z ) ((pos "Ja%" ) (subcat "2A" )))
( "coarser"  j ( k oo1 s @ r ) ((pos "Or%" ) ))
( "coarsest"  j ( k oo1 s i s t ) ((pos "Os%" ) ))
( "coast"  v ( k ou1 s t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "coastal"  j ( k ou1 s t l ) ((pos "OA%" ) ))
( "coasted"  v ( k ou1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "coaster"  n ( k ou1 s t @ r ) ((pos "K6%" ) ))
( "coasters"  n ( k ou1 s t @ z ) ((pos "Kj%" ) ))
( "coastguard"  n ( k ou1 s t g aa d ) ((pos "K6%" ) ))
( "coastguards"  n ( k ou1 s t g aa d z ) ((pos "Kj%" ) ))
( "coasting"  v ( k ou1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "coastline"  n ( k ou1 s t l ai n ) ((pos "K6%" ) ))
( "coastlines"  n ( k ou1 s t l ai n z ) ((pos "Kj%" ) ))
( "coasts"  v ( k ou1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "coastwise"  j ( k ou1 s t w ai z ) ((pos "OA$" "Pu$" ) ))
( "coat"  v ( k ou1 t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "coat-hanger"  n ( k ou1 t - h a ng @ r ) ((pos "K6%" ) ))
( "coat-hangers"  n ( k ou1 t - h a ng @ z ) ((pos "Kj%" ) ))
( "coat-tails"  n ( k ou1 t - t ei l s ) ((pos "Kj%" ) ))
( "coated"  v ( k ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "coatee"  n ( k ou1 t ii1 ) ((pos "K6$" ) ))
( "coatees"  n ( k ou1 t ii1 z ) ((pos "Kj$" ) ))
( "coating"  v ( k ou1 t i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "coatings"  n ( k ou1 t i ng z ) ((pos "Mj%" ) ))
( "coats"  v ( k ou1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "coax"  v ( k ou1 k s ) ((pos "J1%" ) (subcat "2A" "6A" "14" "15B" )))
( "coaxed"  v ( k ou1 k s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15B" )))
( "coaxes"  v ( k ou1 k s i z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" "15B" )))
( "coaxing"  v ( k ou1 k s i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" "14" "15B" )))
( "coaxingly"  a ( k ou1 k s i ng l ii ) ((pos "Pu%" ) ))
( "coaxings"  n ( k ou1 k s i ng z ) ((pos "Mj$" ) ))
( "cob"  n ( k o1 b ) ((pos "K6%" ) ))
( "cob-nut"  n ( k o1 b - n uh t ) ((pos "K6%" ) ))
( "cob-nuts"  n ( k o1 b - n uh t s ) ((pos "Kj%" ) ))
( "cobalt"  n ( k ou1 b oo l t ) ((pos "L@%" ) ))
( "cobber"  n ( k o1 b @ r ) ((pos "K6$" ) ))
( "cobbers"  n ( k o1 b @ z ) ((pos "Kj$" ) ))
( "cobble"  v ( k o1 b l ) ((pos "H2%" "K6%" ) (subcat )))
( "cobbled"  v ( k o1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cobbler"  n ( k o1 b l @ r ) ((pos "K6%" ) ))
( "cobblers"  n ( k o1 b l @ z ) ((pos "Kj%" ) ))
( "cobbles"  v ( k o1 b l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cobblestone"  n ( k o1 b l s t ou n ) ((pos "K6%" ) ))
( "cobblestones"  n ( k o1 b l s t ou n z ) ((pos "Kj%" ) ))
( "cobbling"  v ( k o1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "cobra"  n ( k ou1 b r @ ) ((pos "K6%" ) ))
( "cobras"  n ( k ou1 b r @ z ) ((pos "Kj%" ) ))
( "cobs"  n ( k o1 b z ) ((pos "Kj%" ) ))
( "cobweb"  n ( k o1 b w e b ) ((pos "K6%" ) ))
( "cobwebs"  n ( k o1 b w e b z ) ((pos "Kj%" ) ))
( "cocaine"  n ( k ou1 k ei1 n ) ((pos "L@%" ) ))
( "cochineal"  n ( k o2 ch i n ii1 l ) ((pos "L@$" ) ))
( "cochlea"  n ( k o1 k l i@ ) ((pos "K6$" ) ))
( "cochleas"  n ( k o1 k l i@ z ) ((pos "Kj$" ) ))
( "cock"  v ( k o1 k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "cock-a-doodle-doo"  n ( k o2 k - @ - d uu2 d l - d uu1 ) ((pos "K6%" ) ))
( "cock-a-doodle-doos"  n ( k o2 k - @ - d uu2 d l - d uu1 z ) ((pos "Kj%" ) ))
( "cock-a-hoop"  j ( k o2 k - @ - h uu1 p ) ((pos "OA%" "Pu%" ) ))
( "cock-crow"  n ( k o1 k - k r ou ) ((pos "K6%" ) ))
( "cock-crows"  n ( k o1 k - k r ou z ) ((pos "Kj%" ) ))
( "cockade"  n ( k o1 k ei1 d ) ((pos "K6%" ) ))
( "cockades"  n ( k o1 k ei1 d z ) ((pos "Kj%" ) ))
( "cockatoo"  n ( k o2 k @ t uu1 ) ((pos "K6%" ) ))
( "cockatoos"  n ( k o2 k @ t uu1 z ) ((pos "Kj%" ) ))
( "cockchafer"  n ( k o1 k ch ei f @ r ) ((pos "K6%" ) ))
( "cockchafers"  n ( k o1 k ch ei f @ z ) ((pos "Kj%" ) ))
( "cocked"  v ( k o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cocker"  n ( k o1 k @ r ) ((pos "K6$" ) ))
( "cockerel"  n ( k o1 k @ r @ l ) ((pos "K6%" ) ))
( "cockerels"  n ( k o1 k @ r @ l z ) ((pos "Kj%" ) ))
( "cockers"  n ( k o1 k @ z ) ((pos "Kj$" ) ))
( "cockeyed"  j ( k o1 k ai d ) ((pos "OA%" ) ))
( "cockfighting"  n ( k o1 k f ai t i ng ) ((pos "L@%" ) ))
( "cockhorse"  n ( k o2 k h oo1 s ) ((pos "K6%" ) ))
( "cockhorses"  n ( k o2 k h oo1 s i z ) ((pos "Kj$" ) ))
( "cockier"  j ( k o1 k i@ r ) ((pos "Or%" ) ))
( "cockiest"  j ( k o1 k i i s t ) ((pos "Os%" ) ))
( "cocking"  v ( k o1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cockle"  n ( k o1 k l ) ((pos "K6%" ) ))
( "cockles"  n ( k o1 k l z ) ((pos "Kj%" ) ))
( "cockleshell"  n ( k o1 k l sh e l ) ((pos "K6%" ) ))
( "cockleshells"  n ( k o1 k l sh e l z ) ((pos "Kj%" ) ))
( "cockney"  n ( k o1 k n ii ) ((pos "K6%" "OA%" ) ))
( "cockneys"  n ( k o1 k n i z ) ((pos "Kj%" ) ))
( "cockpit"  n ( k o1 k p i t ) ((pos "K6%" ) ))
( "cockpits"  n ( k o1 k p i t s ) ((pos "Kj%" ) ))
( "cockroach"  n ( k o1 k r ou ch ) ((pos "K7%" ) ))
( "cockroaches"  n ( k o1 k r ou ch i z ) ((pos "Kj%" ) ))
( "cocks"  v ( k o1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "cockscomb"  n ( k o1 k s k ou m ) ((pos "K6%" ) ))
( "cockscombs"  n ( k o1 k s k ou m z ) ((pos "Kj$" ) ))
( "cocksure"  j ( k o2 k sh u@1 r ) ((pos "OA%" ) ))
( "cocktail"  n ( k o1 k t ei l ) ((pos "K6%" ) ))
( "cocktails"  n ( k o1 k t ei l z ) ((pos "Kj%" ) ))
( "cockup"  n ( k o1 k uh p ) ((pos "K6%" ) ))
( "cockups"  n ( k o1 k uh p s ) ((pos "Kj%" ) ))
( "cocky"  j ( k o1 k ii ) ((pos "OD%" ) ))
( "coco"  n ( k ou1 k ou ) ((pos "K6$" ) ))
( "cocoa"  n ( k ou1 k ou ) ((pos "L@%" ) ))
( "coconut"  n ( k ou1 k @ n uh t ) ((pos "K6%" ) ))
( "coconuts"  n ( k ou1 k @ n uh t s ) ((pos "Kj%" ) ))
( "cocoon"  v ( k @1 k uu1 n ) ((pos "H0%" "K6%" ) (subcat )))
( "cocooned"  v ( k @1 k uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cocooning"  v ( k @1 k uu1 n i ng ) ((pos "Hb%" ) (subcat )))
( "cocoons"  v ( k @1 k uu1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cocos"  n ( k ou1 k ou z ) ((pos "Kj$" ) ))
( "cocotte"  n ( k o1 k o1 t ) ((pos "K6$" ) ))
( "cocottes"  n ( k o1 k o1 t s ) ((pos "Kj$" ) ))
( "cod"  v ( k o1 d ) ((pos "J4%" "M6%" ) (subcat "2A" )))
( "cod-liver oil"  n ( k o2 d - l i v @ r - oi1 l ) ((pos "L@%" ) ))
( "coda"  n ( k ou1 d @ ) ((pos "K6%" ) ))
( "codas"  n ( k ou1 d @ z ) ((pos "Kj%" ) ))
( "codded"  v ( k o1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "codding"  v ( k o1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "coddle"  v ( k o1 d l ) ((pos "H2%" ) (subcat )))
( "coddled"  v ( k o1 d l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "coddles"  v ( k o1 d l z ) ((pos "Ha%" ) (subcat )))
( "coddling"  v ( k o1 d l i ng ) ((pos "Hb%" ) (subcat )))
( "code"  v ( k ou1 d ) ((pos "H2%" "M6%" ) ))
( "coded"  v ( k ou1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "codeine"  n ( k ou1 d ii n ) ((pos "L@%" ) ))
( "codes"  v ( k ou1 d z ) ((pos "Ha%" "Mj%" ) ))
( "codex"  n ( k ou1 d e k s ) ((pos "Ki$" ) ))
( "codfish"  n ( k o1 d f i sh ) ((pos "K9%" ) ))
( "codfishes"  n ( k o1 d f i sh i z ) ((pos "Kj$" ) ))
( "codger"  n ( k o1 jh @ r ) ((pos "K6%" ) ))
( "codgers"  n ( k o1 jh @ z ) ((pos "Kj%" ) ))
( "codices"  n ( k ou1 d i s ii z ) ((pos "Kj$" ) ))
( "codicil"  n ( k ou1 d i s i l ) ((pos "K6$" ) ))
( "codicils"  n ( k ou1 d i s i l z ) ((pos "Kj$" ) ))
( "codification"  n ( k ou2 d i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "codifications"  n ( k ou2 d i f i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "codified"  v ( k ou1 d i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "codifies"  v ( k ou1 d i f ai z ) ((pos "Ha%" ) (subcat )))
( "codify"  v ( k ou1 d i f ai ) ((pos "H3%" ) (subcat )))
( "codifying"  v ( k ou1 d i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "coding"  v ( k ou1 d i ng ) ((pos "Hb%" ) ))
( "codling"  n ( k o1 d l i ng ) ((pos "K6$" ) ))
( "codlings"  n ( k o1 d l i ng z ) ((pos "Kj$" ) ))
( "codpiece"  n ( k o1 d p ii s ) ((pos "K6%" ) ))
( "codpieces"  n ( k o1 d p ii s i z ) ((pos "Kj%" ) ))
( "cods"  v ( k o1 d z ) ((pos "Ja%" "Mj$" ) (subcat "2A" )))
( "coeducation"  n ( k ou2 e2 jh u k ei1 sh @ n ) ((pos "L@%" ) ))
( "coeducational"  j ( k ou2 i d y @ k ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "coefficient"  n ( k ou2 i f i1 sh @ n t ) ((pos "K6%" ) ))
( "coefficients"  n ( k ou2 i f i1 sh @ n t s ) ((pos "Kj%" ) ))
( "coerce"  v ( k ou1 @@1 s ) ((pos "H2%" ) (subcat "6A" )))
( "coerced"  v ( k ou1 @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "coerces"  v ( k ou1 @@1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "coercing"  v ( k ou1 @@1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "coercion"  n ( k ou1 @@1 sh @ n ) ((pos "L@%" ) ))
( "coercive"  j ( k ou1 @@1 s i v ) ((pos "OA$" ) ))
( "coeval"  n ( k ou2 ii1 v @ l ) ((pos "K6$" "OA$" ) ))
( "coevals"  n ( k ou2 ii1 v @ l z ) ((pos "Kj$" ) ))
( "coexist"  v ( k ou2 i g z i1 s t ) ((pos "I0%" ) (subcat "2A" )))
( "coexisted"  v ( k ou2 i g z i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "coexistence"  n ( k ou2 i g z i1 s t @ n s ) ((pos "L@%" ) ))
( "coexisting"  v ( k ou2 i g z i1 s t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "coexists"  v ( k ou2 i g z i1 s t s ) ((pos "Ia%" ) (subcat "2A" )))
( "coffee"  n ( k o1 f ii ) ((pos "M6%" ) ))
( "coffee-house"  n ( k o1 f i - h au s ) ((pos "K6%" ) ))
( "coffee-houses"  n ( k o1 f i - h au z i z ) ((pos "Kj%" ) ))
( "coffee-mill"  n ( k o1 f i - m i l ) ((pos "K6%" ) ))
( "coffee-mills"  n ( k o1 f i - m i l z ) ((pos "Kj%" ) ))
( "coffee-stall"  n ( k o1 f i - s t oo l ) ((pos "K6%" ) ))
( "coffee-stalls"  n ( k o1 f i - s t oo l z ) ((pos "Kj%" ) ))
( "coffees"  n ( k o1 f i z ) ((pos "Mj%" ) ))
( "coffer"  n ( k o1 f @ r ) ((pos "K6$" ) ))
( "coffer-dam"  n ( k o1 f @ - d a m ) ((pos "K6$" ) ))
( "coffer-dams"  n ( k o1 f @ - d a m z ) ((pos "Kj$" ) ))
( "coffers"  n ( k o1 f @ z ) ((pos "Kj%" ) ))
( "coffin"  n ( k o1 f i n ) ((pos "K6%" ) ))
( "coffins"  n ( k o1 f i n z ) ((pos "Kj%" ) ))
( "cog"  n ( k o1 g ) ((pos "K6%" ) ))
( "cogency"  n ( k ou1 jh @ n s ii ) ((pos "L@%" ) ))
( "cogent"  j ( k ou1 jh @ n t ) ((pos "OA%" ) ))
( "cogitate"  v ( k o1 jh i t ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "cogitated"  v ( k o1 jh i t ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "cogitates"  v ( k o1 jh i t ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "cogitating"  v ( k o1 jh i t ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "cogitation"  n ( k o2 jh i t ei1 sh @ n ) ((pos "L@%" ) ))
( "cognac"  n ( k o1 n y a k ) ((pos "L@%" ) ))
( "cognate"  n ( k o1 g n ei t ) ((pos "K6$" "OA%" ) ))
( "cognates"  n ( k o1 g n ei t s ) ((pos "Kj$" ) ))
( "cognition"  n ( k o1 g n i1 sh @ n ) ((pos "L@%" ) ))
( "cognizance"  n ( k o1 g n i z @ n s ) ((pos "L@%" ) ))
( "cognizant"  j ( k o1 g n i z @ n t ) ((pos "OA%" ) ))
( "cognomen"  n ( k o1 g n ou1 m @ n ) ((pos "K6$" ) ))
( "cognomens"  n ( k o1 g n ou1 m @ n z ) ((pos "Kj$" ) ))
( "cogs"  n ( k o1 g z ) ((pos "Kj%" ) ))
( "cogwheel"  n ( k o1 g w ii l ) ((pos "K6%" ) ))
( "cogwheels"  n ( k o1 g w ii l z ) ((pos "Kj%" ) ))
( "cohabit"  v ( k ou1 h a1 b i t ) ((pos "I0%" ) ))
( "cohabitation"  n ( k ou2 h a b i t ei1 sh @ n ) ((pos "M6%" ) ))
( "cohabited"  v ( k ou1 h a1 b i t i d ) ((pos "Ic%" "Id%" ) ))
( "cohabiting"  v ( k ou1 h a1 b i t i ng ) ((pos "Ib%" ) ))
( "cohabits"  v ( k ou1 h a1 b i t s ) ((pos "Ia%" ) ))
( "cohere"  v ( k ou1 h i@1 r ) ((pos "I2$" ) (subcat )))
( "cohered"  v ( k ou1 h i@1 d ) ((pos "Ic$" "Id$" ) (subcat )))
( "coherence"  n ( k ou2 h i@1 r @ n s ) ((pos "L@%" ) ))
( "coherency"  n ( k ou2 h i@1 r @ n s ii ) ((pos "L@$" ) ))
( "coherent"  j ( k @1 h i@1 r @ n t ) ((pos "OA%" ) ))
( "coherently"  a ( k @1 h i@1 r @ n t l ii ) ((pos "Pu%" ) ))
( "coheres"  v ( k ou1 h i@1 z ) ((pos "Ia$" ) (subcat )))
( "cohering"  v ( k ou1 h i@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "cohesion"  n ( k ou1 h ii1 zh n ) ((pos "L@%" ) ))
( "cohesive"  j ( k ou1 h ii1 s i v ) ((pos "OA%" ) ))
( "cohort"  n ( k ou1 h oo t ) ((pos "K6%" ) ))
( "cohorts"  n ( k ou1 h oo t s ) ((pos "Kj%" ) ))
( "coif"  n ( k oi1 f ) ((pos "K6$" ) ))
( "coiffeur"  n ( k w aa1 f @@1 r ) ((pos "K6%" ) ))
( "coiffeurs"  n ( k w aa1 f @@1 z ) ((pos "Kj%" ) ))
( "coiffure"  n ( k w aa1 f y u@1 r ) ((pos "K6%" ) ))
( "coiffures"  n ( k w aa1 f y u@1 z ) ((pos "Kj%" ) ))
( "coifs"  n ( k oi1 f s ) ((pos "Kj$" ) ))
( "coign"  n ( k oi1 n ) ((pos "K6$" ) ))
( "coigns"  n ( k oi1 n z ) ((pos "Kj$" ) ))
( "coil"  v ( k oi1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "coiled"  v ( k oi1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "coiling"  v ( k oi1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "coils"  v ( k oi1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "coin"  v ( k oi1 n ) ((pos "H0%" "M6%" ) (subcat )))
( "coinage"  n ( k oi1 n i jh ) ((pos "M6%" ) ))
( "coinages"  n ( k oi1 n i jh i z ) ((pos "Mj%" ) ))
( "coincide"  v ( k ou2 i n s ai1 d ) ((pos "I2%" ) (subcat "2A" )))
( "coincided"  v ( k ou2 i n s ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "coincidence"  n ( k ou1 i1 n s i d @ n s ) ((pos "M6%" ) ))
( "coincidences"  n ( k ou1 i1 n s i d @ n s i z ) ((pos "Mj%" ) ))
( "coincident"  j ( k ou1 i1 n s i d @ n t ) ((pos "OA$" ) ))
( "coincidental"  j ( k ou1 i2 n s i d e1 n t l ) ((pos "OA%" ) ))
( "coincides"  v ( k ou2 i n s ai1 d z ) ((pos "Ia%" ) (subcat "2A" )))
( "coinciding"  v ( k ou2 i n s ai1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "coined"  v ( k oi1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "coiner"  n ( k oi1 n @ r ) ((pos "K6%" ) ))
( "coiners"  n ( k oi1 n @ z ) ((pos "Kj%" ) ))
( "coining"  v ( k oi1 n i ng ) ((pos "Hb%" ) (subcat )))
( "coins"  v ( k oi1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "coir"  n ( k oi1 @ r ) ((pos "L@$" ) ))
( "coition"  n ( k ou1 i1 sh @ n ) ((pos "L@$" ) ))
( "coitus"  n ( k ou1 i t @ s ) ((pos "L@%" ) ))
( "coke"  v ( k ou1 k ) ((pos "H2$" "M6%" ) ))
( "coked"  v ( k ou1 k t ) ((pos "Hc$" "Hd$" ) ))
( "cokes"  v ( k ou1 k s ) ((pos "Ha$" "Mj$" ) ))
( "coking"  v ( k ou1 k i ng ) ((pos "Hb$" ) ))
( "col"  n ( k o1 l ) ((pos "K6$" ) ))
( "cola"  n ( k ou1 l @ ) ((pos "K6$" ) ))
( "colander"  n ( k uh1 l @ n d @ r ) ((pos "K6%" ) ))
( "colanders"  n ( k uh1 l @ n d @ z ) ((pos "Kj%" ) ))
( "colas"  n ( k ou1 l @ z ) ((pos "Kj$" ) ))
( "cold"  n ( k ou1 l d ) ((pos "M6%" "OC%" ) ))
( "cold-blooded"  j ( k ou1 l d - b l uh1 d i d ) ((pos "OA%" ) ))
( "cold-hearted"  j ( k ou1 l d - h aa1 t i d ) ((pos "OA%" ) ))
( "cold-shoulder"  v ( k ou1 l d - sh ou1 l d @ r ) ((pos "H0%" ) (subcat )))
( "cold-shouldered"  v ( k ou1 l d - sh ou1 l d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cold-shouldering"  v ( k ou1 l d - sh ou1 l d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "cold-shoulders"  v ( k ou1 l d - sh ou1 l d @ z ) ((pos "Ha%" ) (subcat )))
( "colder"  j ( k ou1 l d @ r ) ((pos "Or%" ) ))
( "coldest"  j ( k ou1 l d i s t ) ((pos "Os%" ) ))
( "coldly"  a ( k ou1 l d l ii ) ((pos "Pu%" ) ))
( "coldness"  n ( k ou1 l d n @ s ) ((pos "L@%" ) ))
( "colds"  n ( k ou1 l d z ) ((pos "Mj%" ) ))
( "coleslaw"  n ( k ou1 l s l oo ) ((pos "L@%" ) ))
( "colic"  n ( k o1 l i k ) ((pos "L@%" ) ))
( "colitis"  n ( k @1 l ai1 t i s ) ((pos "L@%" ) ))
( "collaborate"  v ( k @1 l a1 b @ r ei t ) ((pos "I2%" ) (subcat "2A" )))
( "collaborated"  v ( k @1 l a1 b @ r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "collaborates"  v ( k @1 l a1 b @ r ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "collaborating"  v ( k @1 l a1 b @ r ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "collaboration"  n ( k @1 l a2 b @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "collaborationist"  n ( k @1 l a2 b @ r ei1 sh @ n i s t ) ((pos "K6$" ) ))
( "collaborationists"  n ( k @1 l a2 b @ r ei1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "collaborator"  n ( k @1 l a1 b @ r ei t @ r ) ((pos "K6%" ) ))
( "collaborators"  n ( k @1 l a1 b @ r ei t @ z ) ((pos "Kj%" ) ))
( "collage"  n ( k o1 l aa zh ) ((pos "M6%" ) ))
( "collages"  n ( k o1 l aa zh i z ) ((pos "Mj%" ) ))
( "collapsable"  j ( k @1 l a1 p s @ b l ) ((pos "OA$" ) ))
( "collapse"  v ( k @1 l a1 p s ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "collapsed"  v ( k @1 l a1 p s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "collapses"  v ( k @1 l a1 p s i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "collapsible"  j ( k @1 l a1 p s @ b l ) ((pos "OA%" ) ))
( "collapsing"  v ( k @1 l a1 p s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "collar"  v ( k o1 l @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "collarbone"  n ( k o1 l @ b ou n ) ((pos "K6%" ) ))
( "collarbones"  n ( k o1 l @ b ou n z ) ((pos "Kj%" ) ))
( "collared"  v ( k o1 l @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "collaring"  v ( k o1 l @ r i ng ) ((pos "Hb%" ) (subcat )))
( "collars"  v ( k o1 l @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "collate"  v ( k @1 l ei1 t ) ((pos "H2%" ) (subcat )))
( "collated"  v ( k @1 l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "collateral"  n ( k @1 l a1 t @ r @ l ) ((pos "L@%" "OA%" ) ))
( "collates"  v ( k @1 l ei1 t s ) ((pos "Ha%" ) (subcat )))
( "collating"  v ( k @1 l ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "collation"  n ( k @1 l ei1 sh @ n ) ((pos "K6$" ) ))
( "collations"  n ( k @1 l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "colleague"  n ( k o1 l ii g ) ((pos "K6%" ) ))
( "colleagues"  n ( k o1 l ii g z ) ((pos "Kj%" ) ))
( "collect"  n ( k o1 l e k t ) ((pos "K6$" ) ))
( "collect"  v ( k @1 l e1 k t ) ((pos "J0%" "OA%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "collected"  v ( k @1 l e1 k t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "collectedly"  a ( k @1 l e1 k t i d l ii ) ((pos "Pu%" ) ))
( "collecting"  v ( k @1 l e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "collection"  n ( k @1 l e1 k sh @ n ) ((pos "M6%" ) ))
( "collections"  n ( k @1 l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "collective"  j ( k @1 l e1 k t i v ) ((pos "OA%" ) ))
( "collectivization"  n ( k @1 l e2 k t i v ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "collectivizations"  n ( k @1 l e2 k t i v ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "collectivize"  v ( k @1 l e1 k t i v ai z ) ((pos "H2$" ) (subcat )))
( "collectivized"  v ( k @1 l e1 k t i v ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "collectivizes"  v ( k @1 l e1 k t i v ai z i z ) ((pos "Ha$" ) (subcat )))
( "collectivizing"  v ( k @1 l e1 k t i v ai z i ng ) ((pos "Hb$" ) (subcat )))
( "collector"  n ( k @1 l e1 k t @ r ) ((pos "K6%" ) ))
( "collectors"  n ( k @1 l e1 k t @ z ) ((pos "Kj%" ) ))
( "collects"  n ( k o1 l e k t s ) ((pos "Kj$" ) ))
( "collects"  v ( k @1 l e1 k t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "colleen"  n ( k o1 l ii n ) ((pos "K6$" ) ))
( "colleens"  n ( k o1 l ii n z ) ((pos "Kj$" ) ))
( "college"  n ( k o1 l i jh ) ((pos "M6%" ) ))
( "colleges"  n ( k o1 l i jh i z ) ((pos "Mj%" ) ))
( "collegiate"  j ( k @1 l ii1 jh i@ t ) ((pos "OA%" ) ))
( "collide"  v ( k @1 l ai1 d ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "collided"  v ( k @1 l ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "collides"  v ( k @1 l ai1 d z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "colliding"  v ( k @1 l ai1 d i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "collie"  n ( k o1 l ii ) ((pos "K6%" ) ))
( "collier"  n ( k o1 l i@ r ) ((pos "K6%" ) ))
( "collieries"  n ( k o1 l y @ r i z ) ((pos "Kj%" ) ))
( "colliers"  n ( k o1 l i@ z ) ((pos "Kj%" ) ))
( "colliery"  n ( k o1 l y @ r ii ) ((pos "K8%" ) ))
( "collies"  n ( k o1 l i z ) ((pos "Kj%" ) ))
( "collision"  n ( k @1 l i1 zh n ) ((pos "M6%" ) ))
( "collisions"  n ( k @1 l i1 zh n z ) ((pos "Mj%" ) ))
( "collocate"  v ( k o1 l @ k ei t ) ((pos "I2$" ) (subcat "2A" )))
( "collocated"  v ( k o1 l @ k ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "collocates"  v ( k o1 l @ k ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "collocating"  v ( k o1 l @ k ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "collocation"  n ( k o2 l @ k ei1 sh @ n ) ((pos "M6$" ) ))
( "collocations"  n ( k o2 l @ k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "colloquial"  j ( k @1 l ou1 k w i@ l ) ((pos "OA%" ) ))
( "colloquialism"  n ( k @1 l ou1 k w i@ l i z @ m ) ((pos "K6%" ) ))
( "colloquialisms"  n ( k @1 l ou1 k w i@ l i z @ m z ) ((pos "Kj%" ) ))
( "colloquially"  a ( k @1 l ou1 k w i@ l ii ) ((pos "Pu%" ) ))
( "colloquies"  n ( k o1 l @ k w i z ) ((pos "Mj$" ) ))
( "colloquy"  n ( k o1 l @ k w ii ) ((pos "M8$" ) ))
( "collusion"  n ( k @1 l uu1 zh n ) ((pos "L@%" ) ))
( "collusive"  j ( k @1 l uu1 s i v ) ((pos "OA$" ) ))
( "collywobbles"  n ( k o1 l i w o b l z ) ((pos "Kj%" ) ))
( "colon"  n ( k ou1 l @ n ) ((pos "K6%" ) ))
( "colonel"  n ( k @@1 n l ) ((pos "K6%" ) ))
( "colonels"  n ( k @@1 n l z ) ((pos "Kj%" ) ))
( "colonial"  n ( k @1 l ou1 n i@ l ) ((pos "K6%" "OA%" ) ))
( "colonialism"  n ( k @1 l ou1 n i@ l i z @ m ) ((pos "L@%" ) ))
( "colonialist"  n ( k @1 l ou1 n i@ l i s t ) ((pos "K6%" ) ))
( "colonialists"  n ( k @1 l ou1 n i@ l i s t s ) ((pos "Kj%" ) ))
( "colonials"  n ( k @1 l ou1 n i@ l z ) ((pos "Kj%" ) ))
( "colonies"  n ( k o1 l @ n i z ) ((pos "Kj%" ) ))
( "colonist"  n ( k o1 l @ n i s t ) ((pos "K6%" ) ))
( "colonists"  n ( k o1 l @ n i s t s ) ((pos "Kj%" ) ))
( "colonization"  n ( k o2 l @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "colonize"  v ( k o1 l @ n ai z ) ((pos "H2%" ) (subcat )))
( "colonized"  v ( k o1 l @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "colonizer"  n ( k o1 l @ n ai z @ r ) ((pos "K6%" ) ))
( "colonizers"  n ( k o1 l @ n ai z @ z ) ((pos "Kj%" ) ))
( "colonizes"  v ( k o1 l @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "colonizing"  v ( k o1 l @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "colonnade"  n ( k o2 l @ n ei1 d ) ((pos "K6%" ) ))
( "colonnaded"  j ( k o2 l @ n ei1 d i d ) ((pos "OA%" ) ))
( "colonnades"  n ( k o2 l @ n ei1 d z ) ((pos "Kj%" ) ))
( "colons"  n ( k ou1 l @ n z ) ((pos "Kj%" ) ))
( "colony"  n ( k o1 l @ n ii ) ((pos "K8%" ) ))
( "coloratura"  n ( k o2 l @ r @ t u@1 r @ ) ((pos "L@$" ) ))
( "colossal"  j ( k @1 l o1 s l ) ((pos "OA%" ) ))
( "colossi"  n ( k @1 l o1 s ai ) ((pos "Kj$" ) ))
( "colossus"  n ( k @1 l o1 s @ s ) ((pos "Ki%" ) ))
( "colour"  v ( k uh1 l @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "colour-bar"  n ( k uh1 l @ - b aa r ) ((pos "K6%" ) ))
( "colour-bars"  n ( k uh1 l @ - b aa z ) ((pos "Kj%" ) ))
( "colour-blind"  j ( k uh1 l @ - b l ai n d ) ((pos "OA%" ) ))
( "colour-wash"  n ( k uh1 l @ - w o sh ) ((pos "K7%" ) ))
( "colour-washes"  n ( k uh1 l @ - w o sh i z ) ((pos "Kj%" ) ))
( "coloured"  v ( k uh1 l @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "colourful"  j ( k uh1 l @ f @ l ) ((pos "OA%" ) ))
( "colouring"  v ( k uh1 l @ r i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "colourless"  j ( k uh1 l @ l @ s ) ((pos "OA%" ) ))
( "colours"  v ( k uh1 l @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "cols"  n ( k o1 l z ) ((pos "Kj$" ) ))
( "colt"  n ( k ou1 l t ) ((pos "K6%" ) ))
( "coltish"  j ( k ou1 l t i sh ) ((pos "OA%" ) ))
( "colts"  n ( k ou1 l t s ) ((pos "Kj%" ) ))
( "columbine"  n ( k o1 l @ m b ai n ) ((pos "K6%" ) ))
( "columbines"  n ( k o1 l @ m b ai n z ) ((pos "Kj$" ) ))
( "column"  n ( k o1 l @ m ) ((pos "K6%" ) ))
( "columnist"  n ( k o1 l @ m n i s t ) ((pos "K6%" ) ))
( "columnists"  n ( k o1 l @ m n i s t s ) ((pos "Kj%" ) ))
( "columns"  n ( k o1 l @ m z ) ((pos "Kj%" ) ))
( "coma"  n ( k ou1 m @ ) ((pos "K6%" ) ))
( "comas"  n ( k ou1 m @ z ) ((pos "Kj%" ) ))
( "comatose"  j ( k ou1 m @ t ou s ) ((pos "OA%" ) ))
( "comb"  v ( k ou1 m ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "comb-out"  n ( k ou1 m - au t ) ((pos "K6%" ) ))
( "comb-outs"  n ( k ou1 m - au t s ) ((pos "Kj$" ) ))
( "combat"  v ( k o1 m b a t ) ((pos "J0%" "K6%" ) (subcat "3A" )))
( "combatant"  n ( k o1 m b @ t @ n t ) ((pos "K6%" "OA%" ) ))
( "combatants"  n ( k o1 m b @ t @ n t s ) ((pos "Kj%" ) ))
( "combated"  v ( k o1 m b a t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "combating"  v ( k o1 m b a t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "combative"  j ( k o1 m b @ t i v ) ((pos "OA%" ) ))
( "combatively"  a ( k o1 m b @ t i v l ii ) ((pos "Pu$" ) ))
( "combats"  v ( k o1 m b a t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "combed"  v ( k ou1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "combination"  n ( k o2 m b i n ei1 sh @ n ) ((pos "M6%" ) ))
( "combination-lock"  n ( k o2 m b i n ei1 sh @ n - l o k ) ((pos "K6%" ) ))
( "combination-locks"  n ( k o2 m b i n ei1 sh @ n - l o k s ) ((pos "Kj%" ) ))
( "combinations"  n ( k o2 m b i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "combine"  n ( k o1 m b ai n ) ((pos "K6%" ) ))
( "combine"  v ( k @1 m b ai1 n ) ((pos "J2%" ) (subcat "2A" "3A" "4A" "6A" )))
( "combined"  v ( k @1 m b ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" "6A" )))
( "combines"  n ( k o1 m b ai n z ) ((pos "Kj%" ) ))
( "combines"  v ( k @1 m b ai1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "4A" "6A" )))
( "combing"  v ( k ou1 m i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "combining"  v ( k @1 m b ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" "6A" )))
( "combs"  v ( k ou1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "combustible"  n ( k @1 m b uh1 s t @ b l ) ((pos "K6$" "OA%" ) ))
( "combustibles"  n ( k @1 m b uh1 s t @ b l z ) ((pos "Kj$" ) ))
( "combustion"  n ( k @1 m b uh1 s ch @ n ) ((pos "L@%" ) ))
( "come"  v ( k uh1 m ) ((pos "I5*" "Id*" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" )))
( "come-at-able"  j ( k uh2 m - a1 t - @ b l ) ((pos "OA$" ) ))
( "come-on"  n ( k uh1 m - o n ) ((pos "K6%" ) ))
( "come-ons"  n ( k uh1 m - o n z ) ((pos "Kj$" ) ))
( "comeback"  n ( k uh1 m b a k ) ((pos "K6%" ) ))
( "comebacks"  n ( k uh1 m b a k s ) ((pos "Kj$" ) ))
( "comedian"  n ( k @1 m ii1 d i@ n ) ((pos "K6%" ) ))
( "comedians"  n ( k @1 m ii1 d i@ n z ) ((pos "Kj%" ) ))
( "comedienne"  n ( k @1 m ii2 d i e1 n ) ((pos "K6%" ) ))
( "comediennes"  n ( k @1 m ii2 d i e1 n z ) ((pos "Kj%" ) ))
( "comedies"  n ( k o1 m @ d i z ) ((pos "Mj%" ) ))
( "comedown"  n ( k uh1 m d au n ) ((pos "K6%" ) ))
( "comedowns"  n ( k uh1 m d au n z ) ((pos "Kj$" ) ))
( "comedy"  n ( k o1 m @ d ii ) ((pos "M8%" ) ))
( "comelier"  j ( k uh1 m l i@ r ) ((pos "Or%" ) ))
( "comeliest"  j ( k uh1 m l i i s t ) ((pos "Os%" ) ))
( "comeliness"  n ( k uh1 m l i n @ s ) ((pos "L@%" ) ))
( "comely"  j ( k uh1 m l ii ) ((pos "OD%" ) ))
( "comer"  n ( k uh1 m @ r ) ((pos "K6%" ) ))
( "comers"  n ( k uh1 m @ z ) ((pos "Kj%" ) ))
( "comes"  v ( k uh1 m z ) ((pos "Ia%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" )))
( "comestible"  n ( k @1 m e1 s t @ b l ) ((pos "K6$" ) ))
( "comestibles"  n ( k @1 m e1 s t @ b l z ) ((pos "Kj%" ) ))
( "comet"  n ( k o1 m i t ) ((pos "K6%" ) ))
( "comets"  n ( k o1 m i t s ) ((pos "Kj%" ) ))
( "comfier"  j ( k uh1 m f i@ r ) ((pos "Or%" ) ))
( "comfiest"  j ( k uh1 m f i i s t ) ((pos "Os%" ) ))
( "comfit"  n ( k uh1 m f i t ) ((pos "K6$" ) ))
( "comfits"  n ( k uh1 m f i t s ) ((pos "Kj$" ) ))
( "comfort"  v ( k uh1 m f @ t ) ((pos "H0%" "M6%" ) (subcat )))
( "comfortable"  j ( k uh1 m f t @ b l ) ((pos "OA%" ) ))
( "comfortably"  a ( k uh1 m f t @ b l ii ) ((pos "Pu%" ) ))
( "comforted"  v ( k uh1 m f @ t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "comforter"  n ( k uh1 m f @ t @ r ) ((pos "K6%" ) ))
( "comforters"  n ( k uh1 m f @ t @ z ) ((pos "Kj%" ) ))
( "comforting"  v ( k uh1 m f @ t i ng ) ((pos "Hb%" ) (subcat )))
( "comfortingly"  a ( k uh1 m f @ t i ng l ii ) ((pos "Pu%" ) ))
( "comfortless"  j ( k uh1 m f @ t l @ s ) ((pos "OA%" ) ))
( "comforts"  v ( k uh1 m f @ t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "comfrey"  n ( k uh1 m f r ii ) ((pos "L@$" ) ))
( "comfy"  j ( k uh1 m f ii ) ((pos "OD%" ) ))
( "comic"  n ( k o1 m i k ) ((pos "K6%" "OA%" ) ))
( "comical"  j ( k o1 m i k l ) ((pos "OA%" ) ))
( "comically"  a ( k o1 m i k l ii ) ((pos "Pu%" ) ))
( "comics"  n ( k o1 m i k s ) ((pos "Kj%" ) ))
( "coming"  v ( k uh1 m i ng ) ((pos "Ib%" "K6%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" )))
( "comings"  n ( k uh1 m i ng z ) ((pos "Kj%" ) ))
( "comity"  n ( k o1 m i t ii ) ((pos "L@$" ) ))
( "comma"  n ( k o1 m @ ) ((pos "K6%" ) ))
( "command"  v ( k @1 m aa1 n d ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "9" )))
( "commandant"  n ( k o2 m @ n d a1 n t ) ((pos "K6%" ) ))
( "commandants"  n ( k o2 m @ n d a1 n t s ) ((pos "Kj%" ) ))
( "commanded"  v ( k @1 m aa1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" )))
( "commandeer"  v ( k o2 m @ n d i@1 r ) ((pos "H0%" ) (subcat )))
( "commandeered"  v ( k o2 m @ n d i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "commandeering"  v ( k o2 m @ n d i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "commandeers"  v ( k o2 m @ n d i@1 z ) ((pos "Ha%" ) (subcat )))
( "commander"  n ( k @1 m aa1 n d @ r ) ((pos "K6%" ) ))
( "commanders"  n ( k @1 m aa1 n d @ z ) ((pos "Kj%" ) ))
( "commanding"  v ( k @1 m aa1 n d i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "9" )))
( "commandment"  n ( k @1 m aa1 n d m @ n t ) ((pos "K6%" ) ))
( "commandments"  n ( k @1 m aa1 n d m @ n t s ) ((pos "Kj%" ) ))
( "commando"  n ( k @1 m aa1 n d ou ) ((pos "K6%" ) ))
( "commandoes"  n ( k @1 m aa1 n d ou z ) ((pos "Kj$" ) ))
( "commandos"  n ( k @1 m aa1 n d ou z ) ((pos "Kj%" ) ))
( "commands"  v ( k @1 m aa1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "9" )))
( "commas"  n ( k o1 m @ z ) ((pos "Kj%" ) ))
( "commemorate"  v ( k @1 m e1 m @ r ei t ) ((pos "H2%" ) (subcat )))
( "commemorated"  v ( k @1 m e1 m @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "commemorates"  v ( k @1 m e1 m @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "commemorating"  v ( k @1 m e1 m @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "commemoration"  n ( k @1 m e2 m @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "commemorations"  n ( k @1 m e2 m @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "commemorative"  j ( k @1 m e1 m @ r @ t i v ) ((pos "OA%" ) ))
( "commence"  v ( k @1 m e1 n s ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "commenced"  v ( k @1 m e1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "commencement"  n ( k @1 m e1 n s m @ n t ) ((pos "K6%" ) ))
( "commencements"  n ( k @1 m e1 n s m @ n t s ) ((pos "Kj$" ) ))
( "commences"  v ( k @1 m e1 n s i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "commencing"  v ( k @1 m e1 n s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "commend"  v ( k @1 m e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "commendable"  j ( k @1 m e1 n d @ b l ) ((pos "OA%" ) ))
( "commendation"  n ( k o2 m e n d ei1 sh @ n ) ((pos "M6%" ) ))
( "commendations"  n ( k o2 m e n d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "commended"  v ( k @1 m e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "commending"  v ( k @1 m e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "commends"  v ( k @1 m e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "commensurable"  j ( k @1 m e1 n sh @ r @ b l ) ((pos "OA$" ) ))
( "commensurate"  j ( k @1 m e1 n sh @ r @ t ) ((pos "OA%" ) ))
( "comment"  v ( k o1 m e n t ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "commentaries"  n ( k o1 m @ n t r i z ) ((pos "Kj%" ) ))
( "commentary"  n ( k o1 m @ n t r ii ) ((pos "K8%" ) ))
( "commentate"  v ( k o1 m @ n t ei t ) ((pos "I2%" ) (subcat )))
( "commentated"  v ( k o1 m @ n t ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "commentates"  v ( k o1 m @ n t ei t s ) ((pos "Ia%" ) (subcat )))
( "commentating"  v ( k o1 m @ n t ei t i ng ) ((pos "Ib%" ) (subcat )))
( "commentator"  n ( k o1 m @ n t ei t @ r ) ((pos "K6%" ) ))
( "commentators"  n ( k o1 m @ n t ei t @ z ) ((pos "Kj%" ) ))
( "commented"  v ( k o1 m e n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "commenting"  v ( k o1 m e n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "comments"  v ( k o1 m e n t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "commerce"  n ( k o1 m @@ s ) ((pos "L@%" ) ))
( "commercial"  n ( k @1 m @@1 sh l ) ((pos "K6%" "OA%" ) ))
( "commercialism"  n ( k @1 m @@1 sh l i z m ) ((pos "L@%" ) ))
( "commercialize"  v ( k @1 m @@1 sh @ l ai z ) ((pos "H2%" ) (subcat )))
( "commercialized"  v ( k @1 m @@1 sh @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "commercializes"  v ( k @1 m @@1 sh @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "commercializing"  v ( k @1 m @@1 sh @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "commercially"  a ( k @1 m @@1 sh l ii ) ((pos "Pu%" ) ))
( "commercials"  n ( k @1 m @@1 sh l z ) ((pos "Kj%" ) ))
( "commination"  n ( k o2 m i n ei1 sh @ n ) ((pos "M6$" ) ))
( "comminations"  n ( k o2 m i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "comminatory"  j ( k o1 m i n @ t r ii ) ((pos "OA$" ) ))
( "commingle"  v ( k @1 m i1 ng g l ) ((pos "J2$" ) (subcat "2A" )))
( "commingled"  v ( k @1 m i1 ng g l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "commingles"  v ( k @1 m i1 ng g l z ) ((pos "Ja$" ) (subcat "2A" )))
( "commingling"  v ( k @1 m i1 ng g l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "commiserate"  v ( k @1 m i1 z @ r ei t ) ((pos "I2%" ) (subcat )))
( "commiserated"  v ( k @1 m i1 z @ r ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "commiserates"  v ( k @1 m i1 z @ r ei t s ) ((pos "Ia%" ) (subcat )))
( "commiserating"  v ( k @1 m i1 z @ r ei t i ng ) ((pos "Ib%" ) (subcat )))
( "commiseration"  n ( k @1 m i2 z @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "commiserations"  n ( k @1 m i2 z @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "commissar"  n ( k o1 m i s aa r ) ((pos "K6%" ) ))
( "commissariat"  n ( k o2 m i s e@1 r i@ t ) ((pos "K6%" ) ))
( "commissariats"  n ( k o2 m i s e@1 r i@ t s ) ((pos "Kj%" ) ))
( "commissaries"  n ( k o1 m i s @ r i z ) ((pos "Kj$" ) ))
( "commissars"  n ( k o1 m i s aa z ) ((pos "Kj%" ) ))
( "commissary"  n ( k o1 m i s @ r ii ) ((pos "K8$" ) ))
( "commission"  v ( k @1 m i1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "commissionaire"  n ( k @1 m i2 sh @ n e@1 r ) ((pos "K6%" ) ))
( "commissionaires"  n ( k @1 m i2 sh @ n e@1 z ) ((pos "Kj%" ) ))
( "commissioned"  v ( k @1 m i1 sh @ n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "commissioner"  n ( k @1 m i1 sh @ n @ r ) ((pos "K6%" ) ))
( "commissioners"  n ( k @1 m i1 sh @ n @ z ) ((pos "Kj%" ) ))
( "commissioning"  v ( k @1 m i1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "commissions"  v ( k @1 m i1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "commit"  v ( k @1 m i1 t ) ((pos "H4%" ) (subcat "6A" "14" )))
( "commital"  n ( k @1 m i1 t l ) ((pos "K6%" ) ))
( "commitals"  n ( k @1 m i1 t l z ) ((pos "Kj%" ) ))
( "commitment"  n ( k @1 m i1 t m @ n t ) ((pos "M6%" ) ))
( "commitments"  n ( k @1 m i1 t m @ n t s ) ((pos "Mj%" ) ))
( "commits"  v ( k @1 m i1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "committed"  v ( k @1 m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "committee"  n ( k @1 m i1 t ii ) ((pos "K6%" ) ))
( "committees"  n ( k @1 m i1 t i z ) ((pos "Kj%" ) ))
( "committing"  v ( k @1 m i1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "commode"  n ( k @1 m ou1 d ) ((pos "K6%" ) ))
( "commodes"  n ( k @1 m ou1 d z ) ((pos "Kj%" ) ))
( "commodious"  j ( k @1 m ou1 d i@ s ) ((pos "OA%" ) ))
( "commodities"  n ( k @1 m o1 d i t i z ) ((pos "Kj%" ) ))
( "commodity"  n ( k @1 m o1 d i t ii ) ((pos "K8%" ) ))
( "commodore"  n ( k o1 m @ d oo r ) ((pos "K6%" ) ))
( "commodores"  n ( k o1 m @ d oo z ) ((pos "Kj%" ) ))
( "common"  n ( k o1 m @ n ) ((pos "K6%" "OC*" ) ))
( "common-room"  n ( k o1 m @ n - r uu m ) ((pos "K6%" ) ))
( "common-rooms"  n ( k o1 m @ n - r uu m z ) ((pos "Kj%" ) ))
( "commonalties"  n ( k o1 m @ n @ l t i z ) ((pos "Kj$" ) ))
( "commonalty"  n ( k o1 m @ n @ l t ii ) ((pos "K8$" ) ))
( "commoner"  n ( k o1 m @ n @ r ) ((pos "K6%" "Or%" ) ))
( "commoners"  n ( k o1 m @ n @ z ) ((pos "Kj%" ) ))
( "commonest"  j ( k o1 m @ n i s t ) ((pos "Os%" ) ))
( "commonly"  a ( k o1 m @ n l ii ) ((pos "Pu%" ) ))
( "commonplace"  n ( k o1 m @ n p l ei s ) ((pos "K6%" "OA%" ) ))
( "commonplaces"  n ( k o1 m @ n p l ei s i z ) ((pos "Kj%" ) ))
( "commons"  n ( k o1 m @ n z ) ((pos "Kj%" ) ))
( "commonwealth"  n ( k o1 m @ n w e l th ) ((pos "K6%" ) ))
( "commonwealths"  n ( k o1 m @ n w e l th s ) ((pos "Kj$" ) ))
( "commotion"  n ( k @1 m ou1 sh @ n ) ((pos "M6%" ) ))
( "commotions"  n ( k @1 m ou1 sh @ n z ) ((pos "Mj$" ) ))
( "communal"  j ( k o1 m y u n l ) ((pos "OA%" ) ))
( "communally"  a ( k o1 m y u n @ l ii ) ((pos "Pu%" ) ))
( "commune"  n ( k o1 m y uu n ) ((pos "K6%" ) ))
( "commune"  v ( k @1 m y uu1 n ) ((pos "I2%" ) (subcat "2C" )))
( "communed"  v ( k @1 m y uu1 n d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "communes"  n ( k o1 m y uu n z ) ((pos "Kj%" ) ))
( "communes"  v ( k @1 m y uu1 n z ) ((pos "Ia%" ) (subcat "2C" )))
( "communicable"  j ( k @1 m y uu1 n i k @ b l ) ((pos "OA%" ) ))
( "communicant"  n ( k @1 m y uu1 n i k @ n t ) ((pos "K6$" ) ))
( "communicants"  n ( k @1 m y uu1 n i k @ n t s ) ((pos "Kj$" ) ))
( "communicate"  v ( k @1 m y uu1 n i k ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "communicated"  v ( k @1 m y uu1 n i k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "communicates"  v ( k @1 m y uu1 n i k ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "communicating"  v ( k @1 m y uu1 n i k ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "communication"  n ( k @1 m y uu2 n i k ei1 sh @ n ) ((pos "M6%" ) ))
( "communications"  n ( k @1 m y uu2 n i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "communicative"  j ( k @1 m y uu1 n i k @ t i v ) ((pos "OA%" ) ))
( "communing"  v ( k @1 m y uu1 n i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "communion"  n ( k @1 m y uu1 n i@ n ) ((pos "M6%" ) ))
( "communions"  n ( k @1 m y uu1 n i@ n z ) ((pos "Mj$" ) ))
( "communiqu_e"  n ( k @1 m y uu1 n i k ei ) ((pos "K6%" ) ))
( "communiqu_es"  n ( k @1 m y uu1 n i k ei z ) ((pos "Kj%" ) ))
( "communism"  n ( k o1 m y u n i z @ m ) ((pos "L@%" ) ))
( "communist"  n ( k o1 m y u n i s t ) ((pos "K6%" "OA%" ) ))
( "communists"  n ( k o1 m y u n i s t s ) ((pos "Kj%" ) ))
( "communities"  n ( k @1 m y uu1 n i t i z ) ((pos "Mj%" ) ))
( "community"  n ( k @1 m y uu1 n i t ii ) ((pos "M8%" ) ))
( "commutable"  j ( k @1 m y uu1 t @ b l ) ((pos "OA$" ) ))
( "commutation"  n ( k o2 m y uu t ei1 sh @ n ) ((pos "M6$" ) ))
( "commutations"  n ( k o2 m y uu t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "commutator"  n ( k o1 m y uu t ei t @ r ) ((pos "K6$" ) ))
( "commutators"  n ( k o1 m y uu t ei t @ z ) ((pos "Kj$" ) ))
( "commute"  v ( k @1 m y uu1 t ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "commuted"  v ( k @1 m y uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "commuter"  n ( k @1 m y uu1 t @ r ) ((pos "K6%" ) ))
( "commuters"  n ( k @1 m y uu1 t @ z ) ((pos "Kj%" ) ))
( "commutes"  v ( k @1 m y uu1 t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "commuting"  v ( k @1 m y uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "comp`ere"  v ( k o1 m p e@ r ) ((pos "H2%" "K6%" ) ))
( "comp`ered"  v ( k o1 m p e@ d ) ((pos "Hc%" "Hd%" ) ))
( "comp`eres"  v ( k o1 m p e@ z ) ((pos "Ha%" "Kj%" ) ))
( "comp`ering"  v ( k o1 m p e@ r i ng ) ((pos "Hb%" ) ))
( "compact"  n ( k o1 m p a k t ) ((pos "K6%" ) ))
( "compact"  v ( k @1 m p a1 k t ) ((pos "H0$" "OA%" ) ))
( "compacted"  v ( k @1 m p a1 k t i d ) ((pos "Hc$" "Hd%" ) ))
( "compacting"  v ( k @1 m p a1 k t i ng ) ((pos "Hb$" ) ))
( "compactly"  a ( k @1 m p a1 k t l ii ) ((pos "Pu%" ) ))
( "compactness"  n ( k @1 m p a1 k t n @ s ) ((pos "L@%" ) ))
( "compacts"  n ( k o1 m p a k t s ) ((pos "Kj%" ) ))
( "compacts"  v ( k @1 m p a1 k t s ) ((pos "Ha$" ) ))
( "companies"  n ( k uh1 m p @ n i z ) ((pos "Mj%" ) ))
( "companion"  n ( k @1 m p a1 n i@ n ) ((pos "K6%" ) ))
( "companionable"  j ( k @1 m p a1 n i@ n @ b l ) ((pos "OA%" ) ))
( "companions"  n ( k @1 m p a1 n i@ n z ) ((pos "Kj%" ) ))
( "companionship"  n ( k @1 m p a1 n i@ n sh i p ) ((pos "L@%" ) ))
( "companionway"  n ( k @1 m p a1 n i@ n w ei ) ((pos "K6%" ) ))
( "companionways"  n ( k @1 m p a1 n i@ n w ei z ) ((pos "Kj$" ) ))
( "company"  n ( k uh1 m p @ n ii ) ((pos "M8%" ) ))
( "comparability"  n ( k o2 m p @ r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "comparable"  j ( k o1 m p @ r @ b l ) ((pos "OA%" ) ))
( "comparative"  n ( k @1 m p a1 r @ t i v ) ((pos "K6%" "OA%" ) ))
( "comparatively"  a ( k @1 m p a1 r @ t i v l ii ) ((pos "Pu%" ) ))
( "comparatives"  n ( k @1 m p a1 r @ t i v z ) ((pos "Kj%" ) ))
( "compare"  v ( k @1 m p e@1 r ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" )))
( "compared"  v ( k @1 m p e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "compares"  v ( k @1 m p e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "comparing"  v ( k @1 m p e@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "comparison"  n ( k @1 m p a1 r i s n ) ((pos "M6%" ) ))
( "comparisons"  n ( k @1 m p a1 r i s n z ) ((pos "Mj%" ) ))
( "compartment"  n ( k @1 m p aa1 t m @ n t ) ((pos "K6%" ) ))
( "compartmentalize"  v ( k o2 m p aa t m e1 n t @ l ai z ) ((pos "H2%" ) ))
( "compartmentalized"  v ( k o2 m p aa t m e1 n t @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "compartmentalizes"  v ( k o2 m p aa t m e1 n t @ l ai z i z ) ((pos "Ha%" ) ))
( "compartmentalizing"  v ( k o2 m p aa t m e1 n t @ l ai z i ng ) ((pos "Hb%" ) ))
( "compartments"  n ( k @1 m p aa1 t m @ n t s ) ((pos "Kj%" ) ))
( "compass"  v ( k uh1 m p @ s ) ((pos "H1$" "K7%" ) ))
( "compassed"  v ( k uh1 m p @ s t ) ((pos "Hc$" "Hd$" ) ))
( "compasses"  v ( k uh1 m p @ s i z ) ((pos "Ha$" "Kj%" ) ))
( "compassing"  v ( k uh1 m p @ s i ng ) ((pos "Hb$" ) ))
( "compassion"  n ( k @1 m p a1 sh @ n ) ((pos "L@%" ) ))
( "compassionate"  j ( k @1 m p a1 sh @ n @ t ) ((pos "OA%" ) ))
( "compassionately"  a ( k @1 m p a1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "compatibility"  n ( k @1 m p a2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "compatible"  j ( k @1 m p a1 t @ b l ) ((pos "OA%" ) ))
( "compatibly"  a ( k @1 m p a1 t @ b l ii ) ((pos "Pu%" ) ))
( "compatriot"  n ( k @1 m p a1 t r i@ t ) ((pos "K6%" ) ))
( "compatriots"  n ( k @1 m p a1 t r i@ t s ) ((pos "Kj%" ) ))
( "compeer"  n ( k o1 m p i@ r ) ((pos "K6$" ) ))
( "compeers"  n ( k o1 m p i@ z ) ((pos "Kj$" ) ))
( "compel"  v ( k @1 m p e1 l ) ((pos "H4%" ) (subcat "6A" "14" )))
( "compelled"  v ( k @1 m p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "compelling"  v ( k @1 m p e1 l i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "compels"  v ( k @1 m p e1 l z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "compendious"  j ( k @1 m p e1 n d i@ s ) ((pos "OA%" ) ))
( "compendium"  n ( k @1 m p e1 n d i@ m ) ((pos "K6%" ) ))
( "compendiums"  n ( k @1 m p e1 n d i@ m z ) ((pos "Kj%" ) ))
( "compensate"  v ( k o1 m p @ n s ei t ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "compensated"  v ( k o1 m p @ n s ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "compensates"  v ( k o1 m p @ n s ei t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "compensating"  v ( k o1 m p @ n s ei t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "compensation"  n ( k o2 m p e n s ei1 sh @ n ) ((pos "M6%" ) ))
( "compensations"  n ( k o2 m p e n s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "compensatory"  j ( k @1 m p e1 n s @ t @ r ii ) ((pos "OA%" ) ))
( "compete"  v ( k @1 m p ii1 t ) ((pos "I2%" ) (subcat "2A" )))
( "competed"  v ( k @1 m p ii1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "competence"  n ( k o1 m p i t @ n s ) ((pos "L@%" ) ))
( "competent"  j ( k o1 m p i t @ n t ) ((pos "OA%" ) ))
( "competently"  a ( k o1 m p i t @ n t l ii ) ((pos "Pu%" ) ))
( "competes"  v ( k @1 m p ii1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "competing"  v ( k @1 m p ii1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "competition"  n ( k o2 m p @ t i1 sh @ n ) ((pos "M6%" ) ))
( "competitions"  n ( k o2 m p @ t i1 sh @ n z ) ((pos "Mj%" ) ))
( "competitive"  j ( k @1 m p e1 t @ t i v ) ((pos "OA%" ) ))
( "competitiveness"  n ( k @1 m p e1 t @ t i v n i s ) ((pos "L@%" ) ))
( "competitor"  n ( k @1 m p e1 t i t @ r ) ((pos "K6%" ) ))
( "competitors"  n ( k @1 m p e1 t i t @ z ) ((pos "Kj%" ) ))
( "compilation"  n ( k o2 m p i l ei1 sh @ n ) ((pos "M6%" ) ))
( "compilations"  n ( k o2 m p i l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "compile"  v ( k @1 m p ai1 l ) ((pos "H2%" ) (subcat )))
( "compiled"  v ( k @1 m p ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "compiler"  n ( k @1 m p ai1 l @ r ) ((pos "K6%" ) ))
( "compilers"  n ( k @1 m p ai1 l @ z ) ((pos "Kj%" ) ))
( "compiles"  v ( k @1 m p ai1 l z ) ((pos "Ha%" ) (subcat )))
( "compiling"  v ( k @1 m p ai1 l i ng ) ((pos "Hb%" ) (subcat )))
( "complacence"  n ( k @1 m p l ei1 s @ n s ) ((pos "L@%" ) ))
( "complacency"  n ( k @1 m p l ei1 s @ n s ii ) ((pos "L@%" ) ))
( "complacent"  j ( k @1 m p l ei1 s @ n t ) ((pos "OA%" ) ))
( "complacently"  a ( k @1 m p l ei1 s @ n t l ii ) ((pos "Pu%" ) ))
( "complain"  v ( k @1 m p l ei1 n ) ((pos "I0%" ) (subcat "2A" "3A" )))
( "complainant"  n ( k @1 m p l ei1 n @ n t ) ((pos "K6%" ) ))
( "complainants"  n ( k @1 m p l ei1 n @ n t s ) ((pos "Kj%" ) ))
( "complained"  v ( k @1 m p l ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "complaining"  v ( k @1 m p l ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "complainingly"  a ( k @1 m p l ei1 n i ng l ii ) ((pos "Pu%" ) ))
( "complains"  v ( k @1 m p l ei1 n z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "complaint"  n ( k @1 m p l ei1 n t ) ((pos "M6%" ) ))
( "complaints"  n ( k @1 m p l ei1 n t s ) ((pos "Mj%" ) ))
( "complaisance"  n ( k @1 m p l ei1 z @ n s ) ((pos "L@$" ) ))
( "complaisant"  j ( k @1 m p l ei1 z @ n t ) ((pos "OA$" ) ))
( "complement"  v ( k o1 m p l i m @ n t ) ((pos "H0%" "K6%" ) (subcat )))
( "complementary"  j ( k o2 m p l i m e1 n t r ii ) ((pos "OA%" ) ))
( "complemented"  v ( k o1 m p l i m @ n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "complementing"  v ( k o1 m p l i m @ n t i ng ) ((pos "Hb%" ) (subcat )))
( "complements"  v ( k o1 m p l i m @ n t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "complete"  v ( k @1 m p l ii1 t ) ((pos "H2%" "OA%" ) (subcat )))
( "completed"  v ( k @1 m p l ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "completely"  a ( k @1 m p l ii1 t l ii ) ((pos "Pu%" ) ))
( "completeness"  n ( k @1 m p l ii1 t n @ s ) ((pos "L@%" ) ))
( "completes"  v ( k @1 m p l ii1 t s ) ((pos "Ha%" ) (subcat )))
( "completing"  v ( k @1 m p l ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "completion"  n ( k @1 m p l ii1 sh @ n ) ((pos "L@%" ) ))
( "complex"  n ( k o1 m p l e k s ) ((pos "K7%" "OA%" ) ))
( "complexes"  n ( k o1 m p l e k s i z ) ((pos "Kj%" ) ))
( "complexion"  n ( k @1 m p l e1 k sh @ n ) ((pos "K6%" ) ))
( "complexions"  n ( k @1 m p l e1 k sh @ n z ) ((pos "Kj%" ) ))
( "complexities"  n ( k @1 m p l e1 k s i t i z ) ((pos "Mj%" ) ))
( "complexity"  n ( k @1 m p l e1 k s i t ii ) ((pos "M8%" ) ))
( "compliance"  n ( k @1 m p l ai1 @ n s ) ((pos "L@%" ) ))
( "compliant"  j ( k @1 m p l ai1 @ n t ) ((pos "OA%" ) ))
( "complicate"  v ( k o1 m p l i k ei t ) ((pos "H2%" ) (subcat )))
( "complicated"  v ( k o1 m p l i k ei t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "complicates"  v ( k o1 m p l i k ei t s ) ((pos "Ha%" ) (subcat )))
( "complicating"  v ( k o1 m p l i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "complication"  n ( k o2 m p l i k ei1 sh @ n ) ((pos "K6%" ) ))
( "complications"  n ( k o2 m p l i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "complicity"  n ( k @1 m p l i1 s i t ii ) ((pos "L@%" ) ))
( "complied"  v ( k @1 m p l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "complies"  v ( k @1 m p l ai1 z ) ((pos "Ha%" ) (subcat "2A" )))
( "compliment"  n ( k o1 m p l i m @ n t ) ((pos "K6%" ) ))
( "compliment"  v ( k o1 m p l i m e n t ) ((pos "H0%" ) (subcat "6A" )))
( "complimentary"  j ( k o2 m p l i m e1 n t r ii ) ((pos "OA%" ) ))
( "complimented"  v ( k o1 m p l i m e n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "complimenting"  v ( k o1 m p l i m e n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "compliments"  n ( k o1 m p l i m @ n t s ) ((pos "Kj%" ) ))
( "compliments"  v ( k o1 m p l i m e n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "complin"  n ( k o1 m p l i n ) ((pos "K6$" ) ))
( "compline"  n ( k o1 m p l i n ) ((pos "K6$" ) ))
( "complines"  n ( k o1 m p l i n z ) ((pos "Kj$" ) ))
( "complins"  n ( k o1 m p l i n z ) ((pos "Kj$" ) ))
( "comply"  v ( k @1 m p l ai1 ) ((pos "H3%" ) (subcat "2A" )))
( "complying"  v ( k @1 m p l ai1 i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "component"  n ( k @1 m p ou1 n @ n t ) ((pos "K6%" "OA%" ) ))
( "components"  n ( k @1 m p ou1 n @ n t s ) ((pos "Kj%" ) ))
( "comport"  v ( k @1 m p oo1 t ) ((pos "J0%" ) (subcat "3A" )))
( "comported"  v ( k @1 m p oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "comporting"  v ( k @1 m p oo1 t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "comportment"  n ( k @1 m p oo1 t m @ n t ) ((pos "K6$" ) ))
( "comportments"  n ( k @1 m p oo1 t m @ n t s ) ((pos "Kj$" ) ))
( "comports"  v ( k @1 m p oo1 t s ) ((pos "Ja%" ) (subcat "3A" )))
( "compos mentis"  j ( k o2 m p @ s - m e1 n t i s ) ((pos "OA%" ) ))
( "compose"  v ( k @1 m p ou1 z ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "composed"  v ( k @1 m p ou1 z d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "6A" )))
( "composedly"  a ( k @1 m p ou1 z i d l ii ) ((pos "Pu%" ) ))
( "composer"  n ( k @1 m p ou1 z @ r ) ((pos "K6%" ) ))
( "composers"  n ( k @1 m p ou1 z @ z ) ((pos "Kj%" ) ))
( "composes"  v ( k @1 m p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "composing"  v ( k @1 m p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "composite"  j ( k o1 m p @ z i t ) ((pos "OA%" ) ))
( "composition"  n ( k o2 m p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "compositions"  n ( k o2 m p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "compositor"  n ( k @1 m p o1 z i t @ r ) ((pos "K6%" ) ))
( "compositors"  n ( k @1 m p o1 z i t @ z ) ((pos "Kj%" ) ))
( "compost"  v ( k o1 m p o s t ) ((pos "H0$" "L@%" ) (subcat )))
( "composted"  v ( k o1 m p o s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "composting"  v ( k o1 m p o s t i ng ) ((pos "Hb$" ) (subcat )))
( "composts"  v ( k o1 m p o s t s ) ((pos "Ha$" ) (subcat )))
( "composure"  n ( k @1 m p ou1 zh @ r ) ((pos "L@%" ) ))
( "compote"  n ( k o1 m p o t ) ((pos "M6$" ) ))
( "compotes"  n ( k o1 m p o t s ) ((pos "Mj$" ) ))
( "compound"  n ( k o1 m p au n d ) ((pos "K6%" "OA%" ) ))
( "compound"  v ( k @1 m p au1 n d ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "compounded"  v ( k @1 m p au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "compounding"  v ( k @1 m p au1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "compounds"  n ( k o1 m p au n d z ) ((pos "Kj%" ) ))
( "compounds"  v ( k @1 m p au1 n d z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "comprehend"  v ( k o2 m p r i h e1 n d ) ((pos "H0%" ) (subcat )))
( "comprehended"  v ( k o2 m p r i h e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "comprehending"  v ( k o2 m p r i h e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "comprehends"  v ( k o2 m p r i h e1 n d z ) ((pos "Ha%" ) (subcat )))
( "comprehensibility"  n ( k o2 m p r i h e2 n s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "comprehensible"  j ( k o2 m p r i h e1 n s @ b l ) ((pos "OA%" ) ))
( "comprehension"  n ( k o2 m p r i h e1 n sh @ n ) ((pos "M6%" ) ))
( "comprehensions"  n ( k o2 m p r i h e1 n sh @ n z ) ((pos "Mj$" ) ))
( "comprehensive"  j ( k o2 m p r i h e1 n s i v ) ((pos "OA%" ) ))
( "comprehensively"  a ( k o2 m p r i h e1 n s i v l ii ) ((pos "Pu%" ) ))
( "comprehensiveness"  n ( k o2 m p r i h e1 n s i v n @ s ) ((pos "L@%" ) ))
( "compress"  n ( k o1 m p r e s ) ((pos "K7%" ) ))
( "compress"  v ( k @1 m p r e1 s ) ((pos "H1%" ) (subcat "6A" )))
( "compressed"  v ( k @1 m p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "compresses"  n ( k o1 m p r e s i z ) ((pos "Kj%" ) ))
( "compresses"  v ( k @1 m p r e1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "compressing"  v ( k @1 m p r e1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "compression"  n ( k @1 m p r e1 sh @ n ) ((pos "L@%" ) ))
( "compressor"  n ( k @1 m p r e1 s @ r ) ((pos "K6%" ) ))
( "compressors"  n ( k @1 m p r e1 s @ z ) ((pos "Kj%" ) ))
( "comprise"  v ( k @1 m p r ai1 z ) ((pos "H2%" ) (subcat )))
( "comprised"  v ( k @1 m p r ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "comprises"  v ( k @1 m p r ai1 z i z ) ((pos "Ha%" ) (subcat )))
( "comprising"  v ( k @1 m p r ai1 z i ng ) ((pos "Hb%" ) (subcat )))
( "compromise"  v ( k o1 m p r @ m ai z ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "compromised"  v ( k o1 m p r @ m ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "compromises"  v ( k o1 m p r @ m ai z i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "compromising"  v ( k o1 m p r @ m ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "comptroller"  n ( k @1 n t r ou1 l @ r ) ((pos "K6$" ) ))
( "comptrollers"  n ( k @1 n t r ou1 l @ z ) ((pos "Kj$" ) ))
( "compulsion"  n ( k @1 m p uh1 l sh @ n ) ((pos "L@%" ) ))
( "compulsive"  j ( k @1 m p uh1 l s i v ) ((pos "OA%" ) ))
( "compulsively"  a ( k @1 m p uh1 l s i v l ii ) ((pos "Pu%" ) ))
( "compulsorily"  a ( k @1 m p uh1 l s @ r @ l ii ) ((pos "Pu%" ) ))
( "compulsory"  j ( k @1 m p uh1 l s @ r ii ) ((pos "OA%" ) ))
( "compunction"  n ( k @1 m p uh1 ng k sh @ n ) ((pos "L@%" ) ))
( "computation"  n ( k o2 m p y u t ei1 sh @ n ) ((pos "M6%" ) ))
( "computational"  j ( k o2 m p y u t ei1 sh @ n l ) ((pos "OA%" ) ))
( "computationally"  a ( k o2 m p y u t ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "computations"  n ( k o2 m p y u t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "compute"  v ( k @1 m p y uu1 t ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "computed"  v ( k @1 m p y uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "computer"  n ( k @1 m p y uu1 t @ r ) ((pos "K6%" ) ))
( "computerize"  v ( k @1 m p y uu1 t @ r ai z ) ((pos "H2%" ) (subcat )))
( "computerized"  v ( k @1 m p y uu1 t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "computerizes"  v ( k @1 m p y uu1 t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "computerizing"  v ( k @1 m p y uu1 t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "computers"  n ( k @1 m p y uu1 t @ z ) ((pos "Kj%" ) ))
( "computes"  v ( k @1 m p y uu1 t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "computing"  v ( k @1 m p y uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "comrade"  n ( k o1 m r ei d ) ((pos "K6%" ) ))
( "comradely"  a ( k o1 m r ei d l ii ) ((pos "Pu%" ) ))
( "comrades"  n ( k o1 m r ei d z ) ((pos "Kj%" ) ))
( "comradeship"  n ( k o1 m r ei d sh i p ) ((pos "K6%" ) ))
( "comradeships"  n ( k o1 m r ei d sh i p s ) ((pos "Kj$" ) ))
( "con"  v ( k o1 n ) ((pos "H4%" "K6%" "Pu%" ) (subcat "6A" )))
( "con-man"  n ( k o1 n - m a n ) ((pos "Ki%" ) ))
( "con-men"  n ( k o1 n - m e n ) ((pos "Kj%" ) ))
( "concatenation"  n ( k o1 n k a2 t @ n ei1 sh @ n ) ((pos "M6$" ) ))
( "concatenations"  n ( k o1 n k a2 t @ n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "concave"  j ( k o1 ng k ei v ) ((pos "OA%" ) ))
( "concavities"  n ( k o2 n k a1 v i t i z ) ((pos "Mj$" ) ))
( "concavity"  n ( k o2 n k a1 v i t ii ) ((pos "M8%" ) ))
( "conceal"  v ( k @1 n s ii1 l ) ((pos "H0%" ) (subcat "6A" )))
( "concealed"  v ( k @1 n s ii1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "concealing"  v ( k @1 n s ii1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "concealment"  n ( k @1 n s ii1 l m @ n t ) ((pos "L@%" ) ))
( "conceals"  v ( k @1 n s ii1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "concede"  v ( k @1 n s ii1 d ) ((pos "H2%" ) (subcat "6A" "9" "12A" )))
( "conceded"  v ( k @1 n s ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "12A" )))
( "concedes"  v ( k @1 n s ii1 d z ) ((pos "Ha%" ) (subcat "6A" "9" "12A" )))
( "conceding"  v ( k @1 n s ii1 d i ng ) ((pos "Hb%" ) (subcat "6A" "9" "12A" )))
( "conceit"  n ( k @1 n s ii1 t ) ((pos "M6%" ) ))
( "conceited"  j ( k @1 n s ii1 t i d ) ((pos "OA%" ) ))
( "conceitedly"  a ( k @1 n s ii1 t i d l ii ) ((pos "Pu%" ) ))
( "conceits"  n ( k @1 n s ii1 t s ) ((pos "Mj%" ) ))
( "conceivable"  j ( k @1 n s ii1 v @ b l ) ((pos "OA%" ) ))
( "conceivably"  a ( k @1 n s ii1 v @ b l ii ) ((pos "Pu%" ) ))
( "conceive"  v ( k @1 n s ii1 v ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "9" "10" )))
( "conceived"  v ( k @1 n s ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "9" "10" )))
( "conceives"  v ( k @1 n s ii1 v z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "9" "10" )))
( "conceiving"  v ( k @1 n s ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "9" "10" )))
( "concentrate"  v ( k o1 n s @ n t r ei t ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "concentrated"  v ( k o1 n s @ n t r ei t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" )))
( "concentrates"  v ( k o1 n s @ n t r ei t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "concentrating"  v ( k o1 n s @ n t r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "concentration"  n ( k o2 n s @ n t r ei1 sh @ n ) ((pos "M6%" ) ))
( "concentrations"  n ( k o2 n s @ n t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "concentric"  j ( k @1 n s e1 n t r i k ) ((pos "OA%" ) ))
( "concept"  n ( k o1 n s e p t ) ((pos "K6%" ) ))
( "conception"  n ( k @1 n s e1 p sh @ n ) ((pos "M6%" ) ))
( "conceptions"  n ( k @1 n s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "concepts"  n ( k o1 n s e p t s ) ((pos "Kj%" ) ))
( "conceptual"  j ( k @1 n s e1 p t y u@ l ) ((pos "OA%" ) ))
( "conceptually"  a ( k @1 n s e1 p t y u@ l ii ) ((pos "Pu%" ) ))
( "concern"  v ( k @1 n s @@1 n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "concerned"  v ( k @1 n s @@1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "concernedly"  a ( k @1 n s @@1 n i d l ii ) ((pos "Pu%" ) ))
( "concerning"  v ( k @1 n s @@1 n i ng ) ((pos "Hb%" "T-%" ) (subcat "6A" )))
( "concerns"  v ( k @1 n s @@1 n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "concert"  n ( k o1 n s @ t ) ((pos "M6%" ) ))
( "concert"  v ( k @1 n s @@1 t ) ((pos "H0$" ) ))
( "concert-hall"  n ( k o1 n s @ t - h oo l ) ((pos "K6%" ) ))
( "concert-halls"  n ( k o1 n s @ t - h oo l z ) ((pos "Kj%" ) ))
( "concerted"  v ( k @1 n s @@1 t i d ) ((pos "Hc$" "Hd%" "OA%" ) ))
( "concertina"  n ( k o2 n s @ t ii1 n @ ) ((pos "K6%" ) ))
( "concertinas"  n ( k o2 n s @ t ii1 n @ z ) ((pos "Kj%" ) ))
( "concerting"  v ( k @1 n s @@1 t i ng ) ((pos "Hb$" ) ))
( "concerto"  n ( k @1 n ch e@1 t ou ) ((pos "K6%" ) ))
( "concertos"  n ( k @1 n ch e@1 t ou z ) ((pos "Kj%" ) ))
( "concerts"  n ( k o1 n s @ t s ) ((pos "Mj%" ) ))
( "concerts"  v ( k @1 n s @@1 t s ) ((pos "Ha$" ) ))
( "concession"  n ( k @1 n s e1 sh @ n ) ((pos "M6%" ) ))
( "concessionaire"  n ( k @1 n s e2 sh @ n e@1 r ) ((pos "K6$" ) ))
( "concessionaires"  n ( k @1 n s e2 sh @ n e@1 z ) ((pos "Kj$" ) ))
( "concessions"  n ( k @1 n s e1 sh @ n z ) ((pos "Mj%" ) ))
( "concessive"  j ( k @1 n s e1 s i v ) ((pos "OA$" ) ))
( "conch"  n ( k o1 n ch ) ((pos "K7%" ) ))
( "conches"  n ( k o1 n ch i z ) ((pos "Kj%" ) ))
( "conchology"  n ( k o1 ng k o1 l @ jh ii ) ((pos "L@$" ) ))
( "concierge"  n ( k o2 n s i e@1 zh ) ((pos "K6%" ) ))
( "concierges"  n ( k o2 n s i e@1 zh i z ) ((pos "Kj%" ) ))
( "conciliate"  v ( k @1 n s i1 l i ei t ) ((pos "H2%" ) (subcat )))
( "conciliated"  v ( k @1 n s i1 l i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "conciliates"  v ( k @1 n s i1 l i ei t s ) ((pos "Ha%" ) (subcat )))
( "conciliating"  v ( k @1 n s i1 l i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "conciliation"  n ( k @1 n s i2 l i ei1 sh @ n ) ((pos "L@%" ) ))
( "conciliatory"  j ( k @1 n s i1 l i@ t @ r ii ) ((pos "OA%" ) ))
( "concise"  j ( k @1 n s ai1 s ) ((pos "OA%" ) ))
( "concisely"  a ( k @1 n s ai1 s l ii ) ((pos "Pu%" ) ))
( "conciseness"  n ( k @1 n s ai1 s n @ s ) ((pos "L@%" ) ))
( "conclave"  n ( k o1 n k l ei v ) ((pos "K6%" ) ))
( "conclaves"  n ( k o1 n k l ei v z ) ((pos "Kj$" ) ))
( "conclude"  v ( k @1 n k l uu1 d ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "7A" "9" )))
( "concluded"  v ( k @1 n k l uu1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "7A" "9" )))
( "concludes"  v ( k @1 n k l uu1 d z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "7A" "9" )))
( "concluding"  v ( k @1 n k l uu1 d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "7A" "9" )))
( "conclusion"  n ( k @1 n k l uu1 zh n ) ((pos "K6%" ) ))
( "conclusions"  n ( k @1 n k l uu1 zh n z ) ((pos "Kj%" ) ))
( "conclusive"  j ( k @1 n k l uu1 s i v ) ((pos "OA%" ) ))
( "conclusively"  a ( k @1 n k l uu1 s i v l ii ) ((pos "Pu%" ) ))
( "concoct"  v ( k @1 n k o1 k t ) ((pos "H0%" ) (subcat )))
( "concocted"  v ( k @1 n k o1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "concocting"  v ( k @1 n k o1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "concoction"  n ( k @1 n k o1 k sh @ n ) ((pos "M6%" ) ))
( "concoctions"  n ( k @1 n k o1 k sh @ n z ) ((pos "Mj%" ) ))
( "concocts"  v ( k @1 n k o1 k t s ) ((pos "Ha%" ) (subcat )))
( "concomitant"  n ( k @1 n k o1 m i t @ n t ) ((pos "K6$" "OA%" ) ))
( "concomitants"  n ( k @1 n k o1 m i t @ n t s ) ((pos "Kj$" ) ))
( "concord"  n ( k o1 ng k oo d ) ((pos "M6%" ) ))
( "concordance"  n ( k @1 n k oo1 d @ n s ) ((pos "M6%" ) ))
( "concordances"  n ( k @1 n k oo1 d @ n s i z ) ((pos "Mj%" ) ))
( "concordant"  j ( k @1 n k oo1 d @ n t ) ((pos "OA%" ) ))
( "concordat"  n ( k @1 n k oo1 d a t ) ((pos "K6%" ) ))
( "concordats"  n ( k @1 n k oo1 d a t s ) ((pos "Kj%" ) ))
( "concords"  n ( k o1 ng k oo d z ) ((pos "Mj$" ) ))
( "concourse"  n ( k o1 ng k oo s ) ((pos "K6%" ) ))
( "concourses"  n ( k o1 ng k oo s i z ) ((pos "Kj$" ) ))
( "concrete"  v ( k o1 ng k r ii t ) ((pos "J2%" "L@%" "OA%" ) (subcat "2A" )))
( "concreted"  v ( k o1 ng k r ii t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "concretely"  a ( k @1 n k r ii1 t l ii ) ((pos "Pu%" ) ))
( "concretes"  v ( k o1 ng k r ii t s ) ((pos "Ja%" ) (subcat "2A" )))
( "concreting"  v ( k o1 ng k r ii t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "concretion"  n ( k @1 n k r ii1 sh @ n ) ((pos "M6%" ) ))
( "concretions"  n ( k @1 n k r ii1 sh @ n z ) ((pos "Mj$" ) ))
( "concubine"  n ( k o1 ng k y u b ai n ) ((pos "K6%" ) ))
( "concubines"  n ( k o1 ng k y u b ai n z ) ((pos "Kj%" ) ))
( "concupiscence"  n ( k @1 n k y uu1 p i s @ n s ) ((pos "L@$" ) ))
( "concur"  v ( k @1 n k @@1 r ) ((pos "I4%" ) (subcat "2A" "3A" )))
( "concurred"  v ( k @1 n k @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "concurrence"  n ( k @1 n k uh1 r @ n s ) ((pos "M6%" ) ))
( "concurrences"  n ( k @1 n k uh1 r @ n s i z ) ((pos "Mj%" ) ))
( "concurrent"  j ( k @1 n k uh1 r @ n t ) ((pos "OA%" ) ))
( "concurrently"  a ( k @1 n k uh1 r @ n t l ii ) ((pos "Pu%" ) ))
( "concurring"  v ( k @1 n k @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "concurs"  v ( k @1 n k @@1 z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "concuss"  v ( k @1 n k uh1 s ) ((pos "H1%" ) (subcat )))
( "concussed"  v ( k @1 n k uh1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "concusses"  v ( k @1 n k uh1 s i z ) ((pos "Ha%" ) (subcat )))
( "concussing"  v ( k @1 n k uh1 s i ng ) ((pos "Hb%" ) (subcat )))
( "concussion"  n ( k @1 n k uh1 sh @ n ) ((pos "M6%" ) ))
( "concussions"  n ( k @1 n k uh1 sh @ n z ) ((pos "Mj$" ) ))
( "condemn"  v ( k @1 n d e1 m ) ((pos "H0%" ) (subcat "6A" "14" "16B" )))
( "condemnation"  n ( k o2 n d e m n ei1 sh @ n ) ((pos "M6%" ) ))
( "condemnations"  n ( k o2 n d e m n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "condemned"  v ( k @1 n d e1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "16B" )))
( "condemning"  v ( k @1 n d e1 m i ng ) ((pos "Hb%" ) (subcat "6A" "14" "16B" )))
( "condemns"  v ( k @1 n d e1 m z ) ((pos "Ha%" ) (subcat "6A" "14" "16B" )))
( "condensation"  n ( k o2 n d e n s ei1 sh @ n ) ((pos "M6%" ) ))
( "condensations"  n ( k o2 n d e n s ei1 sh @ n z ) ((pos "Mj$" ) ))
( "condense"  v ( k @1 n d e1 n s ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "condensed"  v ( k @1 n d e1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "condenser"  n ( k @1 n d e1 n s @ r ) ((pos "K6%" ) ))
( "condensers"  n ( k @1 n d e1 n s @ z ) ((pos "Kj%" ) ))
( "condenses"  v ( k @1 n d e1 n s i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "condensing"  v ( k @1 n d e1 n s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "condescend"  v ( k o2 n d i s e1 n d ) ((pos "I0%" ) (subcat "2A" "3A" )))
( "condescended"  v ( k o2 n d i s e1 n d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "condescending"  v ( k o2 n d i s e1 n d i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" "3A" )))
( "condescendingly"  a ( k o2 n d i s e1 n d i ng l ii ) ((pos "Pu%" ) ))
( "condescends"  v ( k o2 n d i s e1 n d z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "condescension"  n ( k o2 n d i s e1 n sh @ n ) ((pos "M6%" ) ))
( "condescensions"  n ( k o2 n d i s e1 n sh @ n z ) ((pos "Mj$" ) ))
( "condign"  j ( k @1 n d ai1 n ) ((pos "OA$" ) ))
( "condiment"  n ( k o1 n d i m @ n t ) ((pos "M6$" ) ))
( "condiments"  n ( k o1 n d i m @ n t s ) ((pos "Mj%" ) ))
( "condition"  v ( k @1 n d i1 sh @ n ) ((pos "H0%" "K6%" ) (subcat )))
( "conditional"  j ( k @1 n d i1 sh @ n l ) ((pos "OA%" ) ))
( "conditionally"  a ( k @1 n d i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "conditioned"  j ( k @1 n d i1 sh @ n d ) ((pos "OA%" ) ))
( "conditioned"  v ( k @1 n d i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "conditioner"  n ( k @1 n d i1 sh @ n @ r ) ((pos "K6%" ) ))
( "conditioners"  n ( k @1 n d i1 sh @ n @ z ) ((pos "Kj%" ) ))
( "conditioning"  v ( k @1 n d i1 sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "conditions"  v ( k @1 n d i1 sh @ n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "condole"  v ( k @1 n d ou1 l ) ((pos "I2%" ) (subcat )))
( "condoled"  v ( k @1 n d ou1 l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "condolence"  n ( k @1 n d ou1 l @ n s ) ((pos "K6%" ) ))
( "condolences"  n ( k @1 n d ou1 l @ n s i z ) ((pos "Kj%" ) ))
( "condoles"  v ( k @1 n d ou1 l z ) ((pos "Ia%" ) (subcat )))
( "condoling"  v ( k @1 n d ou1 l i ng ) ((pos "Ib%" ) (subcat )))
( "condominium"  n ( k o2 n d @ m i1 n i@ m ) ((pos "K6%" ) ))
( "condominiums"  n ( k o2 n d @ m i1 n i@ m z ) ((pos "Kj%" ) ))
( "condonation"  n ( k o2 n d ou n ei1 sh @ n ) ((pos "K6$" ) ))
( "condonations"  n ( k o2 n d ou n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "condone"  v ( k @1 n d ou1 n ) ((pos "H2%" ) (subcat "6A" )))
( "condoned"  v ( k @1 n d ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "condones"  v ( k @1 n d ou1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "condoning"  v ( k @1 n d ou1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "condor"  n ( k o1 n d oo r ) ((pos "K6%" ) ))
( "condors"  n ( k o1 n d oo z ) ((pos "Kj%" ) ))
( "conduce"  v ( k @1 n d y uu1 s ) ((pos "I2%" ) (subcat )))
( "conduced"  v ( k @1 n d y uu1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "conduces"  v ( k @1 n d y uu1 s i z ) ((pos "Ia%" ) (subcat )))
( "conducing"  v ( k @1 n d y uu1 s i ng ) ((pos "Ib%" ) (subcat )))
( "conducive"  j ( k @1 n d y uu1 s i v ) ((pos "OA%" ) ))
( "conduct"  n ( k o1 n d uh k t ) ((pos "L@%" ) ))
( "conduct"  v ( k @1 n d uh1 k t ) ((pos "J0%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "conducted"  v ( k @1 n d uh1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "conducting"  v ( k @1 n d uh1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "conduction"  n ( k @1 n d uh1 k sh @ n ) ((pos "L@%" ) ))
( "conductive"  j ( k @1 n d uh1 k t i v ) ((pos "OA%" ) ))
( "conductivities"  n ( k o2 n d uh k t i1 v i t i z ) ((pos "Mj$" ) ))
( "conductivity"  n ( k o2 n d uh k t i1 v i t ii ) ((pos "M8%" ) ))
( "conductor"  n ( k @1 n d uh1 k t @ r ) ((pos "K6%" ) ))
( "conductors"  n ( k @1 n d uh1 k t @ z ) ((pos "Kj%" ) ))
( "conductress"  n ( k @1 n d uh1 k t r i s ) ((pos "K7%" ) ))
( "conductresses"  n ( k @1 n d uh1 k t r i s i z ) ((pos "Kj%" ) ))
( "conducts"  v ( k @1 n d uh1 k t s ) ((pos "Ja%" ) (subcat "2A" "6A" "14" "15A" "15B" )))
( "conduit"  n ( k o1 n d w i t ) ((pos "K6%" ) ))
( "conduits"  n ( k o1 n d w i t s ) ((pos "Kj%" ) ))
( "cone"  v ( k ou1 n ) ((pos "H2$" "K6%" ) (subcat )))
( "coned"  v ( k ou1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "cones"  v ( k ou1 n z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "coney"  n ( k ou1 n ii ) ((pos "K6%" ) ))
( "coneys"  n ( k ou1 n i z ) ((pos "Kj%" ) ))
( "confab"  v ( k o1 n f a b ) ((pos "I4$" "K6%" ) ))
( "confabbed"  v ( k o1 n f a b d ) ((pos "Ic$" "Id$" ) ))
( "confabbing"  v ( k o1 n f a b i ng ) ((pos "Ib$" ) ))
( "confabs"  v ( k o1 n f a b z ) ((pos "Ia$" "Kj%" ) ))
( "confabulate"  v ( k @2 n f a1 b y u l ei t ) ((pos "I2$" ) (subcat "2A" )))
( "confabulated"  v ( k @2 n f a1 b y u l ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "confabulates"  v ( k @2 n f a1 b y u l ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "confabulating"  v ( k @2 n f a1 b y u l ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "confabulation"  n ( k @1 n f a2 b y u l ei1 sh @ n ) ((pos "K6$" ) ))
( "confabulations"  n ( k @1 n f a2 b y u l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "confection"  n ( k @1 n f e1 k sh @ n ) ((pos "M6%" ) ))
( "confectioner"  n ( k @1 n f e1 k sh @ n @ r ) ((pos "K6%" ) ))
( "confectioneries"  n ( k @1 n f e1 k sh @ n @ r i z ) ((pos "Mj$" ) ))
( "confectioners"  n ( k @1 n f e1 k sh @ n @ z ) ((pos "Kj%" ) ))
( "confectionery"  n ( k @1 n f e1 k sh @ n @ r ii ) ((pos "M8%" ) ))
( "confections"  n ( k @1 n f e1 k sh @ n z ) ((pos "Mj%" ) ))
( "confederacies"  n ( k @1 n f e1 d @ r @ s i z ) ((pos "Kj$" ) ))
( "confederacy"  n ( k @1 n f e1 d @ r @ s ii ) ((pos "K8%" ) ))
( "confederate"  n ( k @1 n f e1 d @ r @ t ) ((pos "K6%" "OA%" ) ))
( "confederate"  v ( k @1 n f e1 d @ r ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "confederated"  v ( k @1 n f e1 d @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "confederates"  n ( k @1 n f e1 d @ r @ t s ) ((pos "Kj%" ) ))
( "confederates"  v ( k @1 n f e1 d @ r ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "confederating"  v ( k @1 n f e1 d @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "confederation"  n ( k @1 n f e2 d @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "confederations"  n ( k @1 n f e2 d @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "confer"  v ( k @1 n f @@1 r ) ((pos "J4%" ) (subcat "2A" "3A" )))
( "conference"  n ( k o1 n f @ r @ n s ) ((pos "M6%" ) ))
( "conferences"  n ( k o1 n f @ r @ n s i z ) ((pos "Mj%" ) ))
( "conferment"  n ( k @1 n f @@1 m @ n t ) ((pos "K6%" ) ))
( "conferments"  n ( k @1 n f @@1 m @ n t s ) ((pos "Kj$" ) ))
( "conferred"  v ( k @1 n f @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "conferring"  v ( k @1 n f @@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "confers"  v ( k @1 n f @@1 z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "confess"  v ( k @1 n f e1 s ) ((pos "J1%" ) (subcat "2A" "3A" "3B" "6A" "9" "14" )))
( "confessed"  v ( k @1 n f e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "3B" "6A" "9" "14" )))
( "confessedly"  a ( k @1 n f e1 s i d l ii ) ((pos "Pu%" ) ))
( "confesses"  v ( k @1 n f e1 s i z ) ((pos "Ja%" ) (subcat "2A" "3A" "3B" "6A" "9" "14" )))
( "confessing"  v ( k @1 n f e1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "3B" "6A" "9" "14" )))
( "confession"  n ( k @1 n f e1 sh @ n ) ((pos "M6%" ) ))
( "confessional"  n ( k @1 n f e1 sh @ n l ) ((pos "K6%" ) ))
( "confessionals"  n ( k @1 n f e1 sh @ n l z ) ((pos "Kj%" ) ))
( "confessions"  n ( k @1 n f e1 sh @ n z ) ((pos "Mj%" ) ))
( "confessor"  n ( k @1 n f e1 s @ r ) ((pos "K6%" ) ))
( "confessors"  n ( k @1 n f e1 s @ z ) ((pos "Kj%" ) ))
( "confetti"  n ( k @1 n f e1 t ii ) ((pos "Lk%" ) ))
( "confidant"  n ( k o2 n f i d a1 n t ) ((pos "K6%" ) ))
( "confidants"  n ( k o2 n f i d a1 n t s ) ((pos "Kj%" ) ))
( "confide"  v ( k @1 n f ai1 d ) ((pos "J2%" ) (subcat "3A" )))
( "confided"  v ( k @1 n f ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "confidence"  n ( k o1 n f i d @ n s ) ((pos "M6%" ) ))
( "confidences"  n ( k o1 n f i d @ n s i z ) ((pos "Mj%" ) ))
( "confident"  j ( k o1 n f i d @ n t ) ((pos "OA%" ) ))
( "confidential"  j ( k o2 n f i d e1 n sh l ) ((pos "OA%" ) ))
( "confidentialities"  n ( k o2 n f i d e2 n sh i a1 l i t i z ) ((pos "Mj$" ) ))
( "confidentiality"  n ( k o2 n f i d e2 n sh i a1 l i t ii ) ((pos "M8%" ) ))
( "confidentially"  a ( k o2 n f i d e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "confidently"  a ( k o1 n f i d @ n t l ii ) ((pos "Pu%" ) ))
( "confides"  v ( k @1 n f ai1 d z ) ((pos "Ja%" ) (subcat "3A" )))
( "confiding"  v ( k @1 n f ai1 d i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "confidingly"  a ( k @1 n f ai1 d i ng l ii ) ((pos "Pu%" ) ))
( "configuration"  n ( k @1 n f i2 g @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "configurations"  n ( k @1 n f i2 g @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "configure"  v ( k @1 n f i1 g @ r ) ((pos "H2%" ) (subcat )))
( "configured"  v ( k @1 n f i1 g @ d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "configures"  v ( k @1 n f i1 g @ z ) ((pos "Ha%" ) (subcat )))
( "configuring"  v ( k @1 n f i1 g @ r i ng ) ((pos "Hb%" ) (subcat )))
( "confine"  v ( k @1 n f ai1 n ) ((pos "H2%" ) (subcat "6A" )))
( "confined"  v ( k @1 n f ai1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "confinement"  n ( k @1 n f ai1 n m @ n t ) ((pos "M6%" ) ))
( "confinements"  n ( k @1 n f ai1 n m @ n t s ) ((pos "Mj%" ) ))
( "confines"  n ( k o1 n f ai n z ) ((pos "Kj%" ) ))
( "confines"  v ( k @1 n f ai1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "confining"  v ( k @1 n f ai1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "confirm"  v ( k @1 n f @@1 m ) ((pos "H0%" ) (subcat "6A" )))
( "confirmation"  n ( k o2 n f @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "confirmations"  n ( k o2 n f @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "confirmed"  v ( k @1 n f @@1 m d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "confirming"  v ( k @1 n f @@1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "confirms"  v ( k @1 n f @@1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "confiscate"  v ( k o1 n f i s k ei t ) ((pos "H2%" ) (subcat )))
( "confiscated"  v ( k o1 n f i s k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "confiscates"  v ( k o1 n f i s k ei t s ) ((pos "Ha%" ) (subcat )))
( "confiscating"  v ( k o1 n f i s k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "confiscation"  n ( k o2 n f i s k ei1 sh @ n ) ((pos "M6%" ) ))
( "confiscations"  n ( k o2 n f i s k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "conflagration"  n ( k o2 n f l @ g r ei1 sh @ n ) ((pos "K6%" ) ))
( "conflagrations"  n ( k o2 n f l @ g r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "conflict"  n ( k o1 n f l i k t ) ((pos "M6%" ) ))
( "conflict"  v ( k @1 n f l i1 k t ) ((pos "I0%" ) (subcat "2A" )))
( "conflicted"  v ( k @1 n f l i1 k t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "conflicting"  v ( k @1 n f l i1 k t i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "conflicts"  n ( k o1 n f l i k t s ) ((pos "Mj%" ) ))
( "conflicts"  v ( k @1 n f l i1 k t s ) ((pos "Ia%" ) (subcat "2A" )))
( "confluence"  n ( k o1 n f l u@ n s ) ((pos "K6%" ) ))
( "confluences"  n ( k o1 n f l u@ n s i z ) ((pos "Kj$" ) ))
( "confluent"  j ( k o1 n f l u@ n t ) ((pos "OA$" ) ))
( "conform"  v ( k @1 n f oo1 m ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "conformable"  j ( k @1 n f oo1 m @ b l ) ((pos "OA$" ) ))
( "conformation"  n ( k o2 n f oo m ei1 sh @ n ) ((pos "K6$" ) ))
( "conformations"  n ( k o2 n f oo m ei1 sh @ n z ) ((pos "Kj$" ) ))
( "conformed"  v ( k @1 n f oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "conforming"  v ( k @1 n f oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "conformist"  n ( k @1 n f oo1 m i s t ) ((pos "K6%" ) ))
( "conformists"  n ( k @1 n f oo1 m i s t s ) ((pos "Kj%" ) ))
( "conformity"  n ( k @1 n f oo1 m i t ii ) ((pos "L@%" ) ))
( "conforms"  v ( k @1 n f oo1 m z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "confound"  v ( k @1 n f au1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "confounded"  v ( k @1 n f au1 n d i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "confoundedly"  a ( k @1 n f au1 n d i d l ii ) ((pos "Pu$" ) ))
( "confounding"  v ( k @1 n f au1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "confounds"  v ( k @1 n f au1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "confr`ere"  n ( k o1 n f r e@ r ) ((pos "K6$" ) ))
( "confr`eres"  n ( k o1 n f r e@ z ) ((pos "Kj$" ) ))
( "confront"  v ( k @1 n f r uh1 n t ) ((pos "H0%" ) (subcat "6A" )))
( "confrontation"  n ( k o2 n f r uh n t ei1 sh @ n ) ((pos "M6%" ) ))
( "confrontations"  n ( k o2 n f r uh n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "confronted"  v ( k @1 n f r uh1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "confronting"  v ( k @1 n f r uh1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "confronts"  v ( k @1 n f r uh1 n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "confuse"  v ( k @1 n f y uu1 z ) ((pos "H2%" ) (subcat "6A" )))
( "confused"  v ( k @1 n f y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "confusedly"  a ( k @1 n f y uu1 z i d l ii ) ((pos "Pu%" ) ))
( "confuses"  v ( k @1 n f y uu1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "confusing"  v ( k @1 n f y uu1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "confusion"  n ( k @1 n f y uu1 zh n ) ((pos "L@%" ) ))
( "confutation"  n ( k o2 n f y uu t ei1 sh @ n ) ((pos "K6$" ) ))
( "confutations"  n ( k o2 n f y uu t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "confute"  v ( k @1 n f y uu1 t ) ((pos "H2%" ) (subcat )))
( "confuted"  v ( k @1 n f y uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "confutes"  v ( k @1 n f y uu1 t s ) ((pos "Ha%" ) (subcat )))
( "confuting"  v ( k @1 n f y uu1 t i ng ) ((pos "Hb$" ) (subcat )))
( "cong_e"  n ( k o1 n zh ei ) ((pos "K6$" ) ))
( "cong_es"  n ( k o1 n zh ei z ) ((pos "Kj$" ) ))
( "congeal"  v ( k @1 n jh ii1 l ) ((pos "J0%" ) (subcat "2A" )))
( "congealed"  v ( k @1 n jh ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "congealing"  v ( k @1 n jh ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "congeals"  v ( k @1 n jh ii1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "congenial"  j ( k @1 n jh ii1 n i@ l ) ((pos "OA%" ) ))
( "congenially"  a ( k @1 n jh ii1 n i@ l ii ) ((pos "Pu%" ) ))
( "congenital"  j ( k @1 n jh e1 n i t l ) ((pos "OA%" ) ))
( "conger"  n ( k o1 ng g @ r ) ((pos "K6$" ) ))
( "conger-eel"  n ( k o1 ng g @ r - ii l ) ((pos "K6%" ) ))
( "conger-eels"  n ( k o1 ng g @ r - ii l z ) ((pos "Kj%" ) ))
( "congers"  n ( k o1 ng g @ z ) ((pos "Kj$" ) ))
( "congested"  j ( k @1 n jh e1 s t i d ) ((pos "OA%" ) ))
( "congestion"  n ( k @1 n jh e1 s ch @ n ) ((pos "L@%" ) ))
( "conglomerate"  n ( k @1 n g l o1 m @ r @ t ) ((pos "K6%" "OA%" ) ))
( "conglomerate"  v ( k @1 n g l o1 m @ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "conglomerated"  v ( k @1 n g l o1 m @ r ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "conglomerates"  n ( k @1 n g l o1 m @ r @ t s ) ((pos "Kj%" ) ))
( "conglomerates"  v ( k @1 n g l o1 m @ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "conglomerating"  v ( k @1 n g l o1 m @ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "conglomeration"  n ( k @1 n g l o2 m @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "conglomerations"  n ( k @1 n g l o2 m @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "congratulate"  v ( k @1 n g r a1 ch u l ei t ) ((pos "H2%" ) (subcat "6A" )))
( "congratulated"  v ( k @1 n g r a1 ch u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "congratulates"  v ( k @1 n g r a1 ch u l ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "congratulating"  v ( k @1 n g r a1 ch u l ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "congratulation"  n ( k @1 n g r a2 ch u l ei1 sh @ n ) ((pos "K6%" ) ))
( "congratulations"  n ( k @1 n g r a2 ch u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "congratulatory"  j ( k @1 n g r a1 ch u l @ t @ r ii ) ((pos "OA%" ) ))
( "congregate"  v ( k o1 ng g r i g ei t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "congregated"  v ( k o1 ng g r i g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "congregates"  v ( k o1 ng g r i g ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "congregating"  v ( k o1 ng g r i g ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "congregation"  n ( k o2 ng g r i g ei1 sh @ n ) ((pos "M6%" ) ))
( "congregational"  j ( k o2 ng g r i g ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "congregations"  n ( k o2 ng g r i g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "congress"  n ( k o1 ng g r e s ) ((pos "K7%" ) ))
( "congresses"  n ( k o1 ng g r e s i z ) ((pos "Kj%" ) ))
( "congressional"  j ( k @1 n g r e1 sh @ n l ) ((pos "OA%" ) ))
( "congressman"  n ( k o1 ng g r e s m @ n ) ((pos "Ki%" ) ))
( "congressmen"  n ( k o1 ng g r e s m @ n ) ((pos "Kj%" ) ))
( "congresswoman"  n ( k o1 ng g r e s w u m @ n ) ((pos "Ki$" ) ))
( "congresswomen"  n ( k o1 ng g r e s w i m i n ) ((pos "Kj$" ) ))
( "congruent"  j ( k o1 ng g r u@ n t ) ((pos "OA%" ) ))
( "congruous"  j ( k o1 ng g r u@ s ) ((pos "OA$" ) ))
( "conic"  j ( k o1 n i k ) ((pos "OA$" ) ))
( "conical"  j ( k o1 n i k l ) ((pos "OA%" ) ))
( "conies"  n ( k ou1 n i z ) ((pos "Kj$" ) ))
( "conifer"  n ( k o1 n i f @ r ) ((pos "K6%" ) ))
( "coniferous"  j ( k @1 n i1 f @ r @ s ) ((pos "OA%" ) ))
( "conifers"  n ( k o1 n i f @ z ) ((pos "Kj%" ) ))
( "coning"  v ( k ou1 n i ng ) ((pos "Hb$" ) (subcat )))
( "conjectural"  j ( k @1 n jh e1 k ch @ r @ l ) ((pos "OA%" ) ))
( "conjecture"  v ( k @1 n jh e1 k ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" "9" )))
( "conjectured"  v ( k @1 n jh e1 k ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" )))
( "conjectures"  v ( k @1 n jh e1 k ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "9" )))
( "conjecturing"  v ( k @1 n jh e1 k ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "9" )))
( "conjoin"  v ( k @1 n jh oi1 n ) ((pos "J0$" ) (subcat "2A" )))
( "conjoined"  v ( k @1 n jh oi1 n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "conjoining"  v ( k @1 n jh oi1 n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "conjoins"  v ( k @1 n jh oi1 n z ) ((pos "Ja$" ) (subcat "2A" )))
( "conjoint"  j ( k @1 n jh oi1 n t ) ((pos "OA$" ) ))
( "conjointly"  a ( k @1 n jh oi1 n t l ii ) ((pos "Pu%" ) ))
( "conjugal"  j ( k o1 n jh u g @ l ) ((pos "OA%" ) ))
( "conjugally"  a ( k o1 n jh u g @ l ii ) ((pos "Pu%" ) ))
( "conjugate"  v ( k o1 n jh u g ei t ) ((pos "J2%" ) (subcat "2A" )))
( "conjugated"  v ( k o1 n jh u g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "conjugates"  v ( k o1 n jh u g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "conjugating"  v ( k o1 n jh u g ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "conjugation"  n ( k o2 n jh u g ei1 sh @ n ) ((pos "M6%" ) ))
( "conjugations"  n ( k o2 n jh u g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "conjunction"  n ( k @1 n jh uh1 ng k sh @ n ) ((pos "M6%" ) ))
( "conjunctions"  n ( k @1 n jh uh1 ng k sh @ n z ) ((pos "Mj%" ) ))
( "conjunctive"  n ( k @1 n jh uh1 ng k t i v ) ((pos "K6%" "OA%" ) ))
( "conjunctives"  n ( k @1 n jh uh1 ng k t i v z ) ((pos "Kj%" ) ))
( "conjuncture"  n ( k @1 n jh uh1 ng k ch @ r ) ((pos "K6$" ) ))
( "conjunctures"  n ( k @1 n jh uh1 ng k ch @ z ) ((pos "Kj$" ) ))
( "conjuration"  n ( k o2 n jh u r ei1 sh @ n ) ((pos "K6$" ) ))
( "conjurations"  n ( k o2 n jh u r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "conjure"  v ( k uh1 n jh @ r ) ((pos "J2%" ) (subcat "2A" "15A" "15B" )))
( "conjured"  v ( k uh1 n jh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "15A" "15B" )))
( "conjurer"  n ( k uh1 n jh @ r @ r ) ((pos "K6$" ) ))
( "conjurers"  n ( k uh1 n jh @ r @ z ) ((pos "Kj$" ) ))
( "conjures"  v ( k uh1 n jh @ z ) ((pos "Ja%" ) (subcat "2A" "15A" "15B" )))
( "conjuring"  v ( k uh1 n jh @ r i ng ) ((pos "Jb%" ) (subcat "2A" "15A" "15B" )))
( "conjuror"  n ( k uh1 n jh @ r @ r ) ((pos "K6%" ) ))
( "conjurors"  n ( k uh1 n jh @ r @ z ) ((pos "Kj%" ) ))
( "conk"  v ( k o1 ng k ) ((pos "I0%" "K6%" ) ))
( "conked"  v ( k o1 ng k t ) ((pos "Ic%" "Id%" ) ))
( "conker"  n ( k o1 ng k @ r ) ((pos "K6%" ) ))
( "conkers"  n ( k o1 ng k @ z ) ((pos "Kj%" ) ))
( "conking"  v ( k o1 ng k i ng ) ((pos "Ib%" ) ))
( "conks"  v ( k o1 ng k s ) ((pos "Ia%" "Kj%" ) ))
( "connect"  v ( k @1 n e1 k t ) ((pos "J0%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "connected"  v ( k @1 n e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "connecting"  v ( k @1 n e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "connection"  n ( k @1 n e1 k sh @ n ) ((pos "M6%" ) ))
( "connections"  n ( k @1 n e1 k sh @ n z ) ((pos "Mj%" ) ))
( "connective"  n ( k @1 n e1 k t i v ) ((pos "K6%" "OA%" ) ))
( "connectives"  n ( k @1 n e1 k t i v z ) ((pos "Kj%" ) ))
( "connects"  v ( k @1 n e1 k t s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "conned"  v ( k o1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "connexion"  n ( k @1 n e1 k sh @ n ) ((pos "M6$" ) ))
( "connexions"  n ( k @1 n e1 k sh @ n z ) ((pos "Mj$" ) ))
( "conning"  v ( k o1 n i ng ) ((pos "Hb%" "Oq%" ) (subcat "6A" )))
( "connivance"  n ( k @1 n ai1 v @ n s ) ((pos "L@%" ) ))
( "connive"  v ( k @1 n ai1 v ) ((pos "I2%" ) (subcat )))
( "connived"  v ( k @1 n ai1 v d ) ((pos "Ic%" "Id%" ) (subcat )))
( "connives"  v ( k @1 n ai1 v z ) ((pos "Ia%" ) (subcat )))
( "conniving"  v ( k @1 n ai1 v i ng ) ((pos "Ib%" ) (subcat )))
( "connoisseur"  n ( k o2 n @ s @@1 r ) ((pos "K6%" ) ))
( "connoisseurs"  n ( k o2 n @ s @@1 z ) ((pos "Kj%" ) ))
( "connotation"  n ( k o2 n @ t ei1 sh @ n ) ((pos "K6%" ) ))
( "connotations"  n ( k o2 n @ t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "connote"  v ( k @1 n ou1 t ) ((pos "H2$" ) (subcat )))
( "connoted"  v ( k @1 n ou1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "connotes"  v ( k @1 n ou1 t s ) ((pos "Ha$" ) (subcat )))
( "connoting"  v ( k @1 n ou1 t i ng ) ((pos "Hb$" ) (subcat )))
( "connubial"  j ( k @1 n y uu1 b i@ l ) ((pos "OA$" ) ))
( "conquer"  v ( k o1 ng k @ r ) ((pos "H0%" ) (subcat )))
( "conquered"  v ( k o1 ng k @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "conquering"  v ( k o1 ng k @ r i ng ) ((pos "Hb%" ) (subcat )))
( "conqueror"  n ( k o1 ng k @ r @ r ) ((pos "K6%" ) ))
( "conquerors"  n ( k o1 ng k @ r @ z ) ((pos "Kj%" ) ))
( "conquers"  v ( k o1 ng k @ z ) ((pos "Ha%" ) (subcat )))
( "conquest"  n ( k o1 ng k w e s t ) ((pos "M6%" ) ))
( "conquests"  n ( k o1 ng k w e s t s ) ((pos "Mj%" ) ))
( "conquistador"  n ( k o1 n k w i1 s t @ d oo r ) ((pos "K6%" ) ))
( "conquistadors"  n ( k o1 n k w i1 s t @ d oo z ) ((pos "Kj%" ) ))
( "cons"  v ( k o1 n z ) ((pos "Ha%" "Kj$" ) (subcat "6A" )))
( "consanguinity"  n ( k o2 n s a ng g w i1 n i t ii ) ((pos "L@$" ) ))
( "conscience"  n ( k o1 n sh @ n s ) ((pos "M6%" ) ))
( "conscience-smitten"  j ( k o1 n sh @ n s - s m i t n ) ((pos "OA%" ) ))
( "consciences"  n ( k o1 n sh @ n s i z ) ((pos "Mj%" ) ))
( "conscientious"  j ( k o2 n sh i e1 n sh @ s ) ((pos "OA%" ) ))
( "conscientiously"  a ( k o2 n sh i e1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "conscientiousness"  n ( k o2 n sh i e1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "conscious"  j ( k o1 n sh @ s ) ((pos "OA%" ) ))
( "consciously"  a ( k o1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "consciousness"  n ( k o1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "conscript"  n ( k o1 n s k r i p t ) ((pos "K6%" ) ))
( "conscript"  v ( k @1 n s k r i1 p t ) ((pos "H0%" ) (subcat "6A" )))
( "conscripted"  v ( k @1 n s k r i1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "conscripting"  v ( k @1 n s k r i1 p t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "conscription"  n ( k @1 n s k r i1 p sh @ n ) ((pos "L@%" ) ))
( "conscripts"  n ( k o1 n s k r i p t s ) ((pos "Kj%" ) ))
( "conscripts"  v ( k @1 n s k r i1 p t s ) ((pos "Ha%" ) (subcat "6A" )))
( "consecrate"  v ( k o1 n s i k r ei t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "consecrated"  v ( k o1 n s i k r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "consecrates"  v ( k o1 n s i k r ei t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "consecrating"  v ( k o1 n s i k r ei t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "consecration"  n ( k o2 n s i k r ei1 sh @ n ) ((pos "M6%" ) ))
( "consecrations"  n ( k o2 n s i k r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "consecutive"  j ( k @1 n s e1 k y u t i v ) ((pos "OA%" ) ))
( "consecutively"  a ( k @1 n s e1 k y u t i v l ii ) ((pos "Pu%" ) ))
( "consensus"  n ( k @1 n s e1 n s @ s ) ((pos "Mi%" ) ))
( "consent"  v ( k @1 n s e1 n t ) ((pos "I0%" "L@%" ) (subcat "2A" "3A" )))
( "consented"  v ( k @1 n s e1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "consenting"  v ( k @1 n s e1 n t i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "consents"  v ( k @1 n s e1 n t s ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "consequence"  n ( k o1 n s i k w @ n s ) ((pos "M6%" ) ))
( "consequences"  n ( k o1 n s i k w @ n s i z ) ((pos "Mj%" ) ))
( "consequent"  j ( k o1 n s i k w @ n t ) ((pos "OA%" ) ))
( "consequential"  j ( k o2 n s i k w e1 n sh l ) ((pos "OA%" ) ))
( "consequentially"  a ( k o2 n s i k w e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "consequently"  a ( k o1 n s i k w @ n t l ii ) ((pos "Pu%" ) ))
( "conservancies"  n ( k @1 n s @@1 v @ n s i z ) ((pos "Mj$" ) ))
( "conservancy"  n ( k @1 n s @@1 v @ n s ii ) ((pos "M8$" ) ))
( "conservation"  n ( k o2 n s @ v ei1 sh @ n ) ((pos "L@%" ) ))
( "conservatism"  n ( k @1 n s @@1 v @ t i z @ m ) ((pos "L@%" ) ))
( "conservative"  n ( k @1 n s @@1 v @ t i v ) ((pos "K6%" "OA%" ) ))
( "conservatively"  a ( k @1 n s @@1 v @ t i v l ii ) ((pos "Pu%" ) ))
( "conservatives"  n ( k @1 n s @@1 v @ t i v z ) ((pos "Kj%" ) ))
( "conservatoire"  n ( k @1 n s @@1 v @ t w aa r ) ((pos "K6%" ) ))
( "conservatoires"  n ( k @1 n s @@1 v @ t w aa z ) ((pos "Kj%" ) ))
( "conservatories"  n ( k @1 n s @@1 v @ t r i z ) ((pos "Kj%" ) ))
( "conservatory"  n ( k @1 n s @@1 v @ t r ii ) ((pos "K8%" ) ))
( "conserve"  v ( k @1 n s @@1 v ) ((pos "H2%" "K6%" ) (subcat )))
( "conserved"  v ( k @1 n s @@1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "conserves"  v ( k @1 n s @@1 v z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "conserving"  v ( k @1 n s @@1 v i ng ) ((pos "Hb%" ) (subcat )))
( "consider"  v ( k @1 n s i1 d @ r ) ((pos "H0%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "considerable"  j ( k @1 n s i1 d @ r @ b l ) ((pos "OA%" ) ))
( "considerably"  a ( k @1 n s i1 d @ r @ b l ii ) ((pos "Pu%" ) ))
( "considerate"  j ( k @1 n s i1 d @ r @ t ) ((pos "OA%" ) ))
( "considerately"  a ( k @1 n s i1 d @ r @ t l ii ) ((pos "Pu%" ) ))
( "considerateness"  n ( k @1 n s i1 d @ r @ t n @ s ) ((pos "L@%" ) ))
( "consideration"  n ( k @1 n s i2 d @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "considerations"  n ( k @1 n s i2 d @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "considered"  v ( k @1 n s i1 d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "considering"  v ( k @1 n s i1 d @ r i ng ) ((pos "Hb%" "T-%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "considers"  v ( k @1 n s i1 d @ z ) ((pos "Ha%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "consign"  v ( k @1 n s ai1 n ) ((pos "H0%" ) (subcat "6A" )))
( "consigned"  v ( k @1 n s ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "consignee"  n ( k o2 n s ai n ii1 ) ((pos "K6$" ) ))
( "consignees"  n ( k o2 n s ai n ii1 z ) ((pos "Kj$" ) ))
( "consigner"  n ( k @1 n s ai1 n @ r ) ((pos "K6%" ) ))
( "consigners"  n ( k @1 n s ai1 n @ z ) ((pos "Kj%" ) ))
( "consigning"  v ( k @1 n s ai1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "consignment"  n ( k @1 n s ai1 n m @ n t ) ((pos "M6%" ) ))
( "consignments"  n ( k @1 n s ai1 n m @ n t s ) ((pos "Mj%" ) ))
( "consignor"  n ( k @1 n s ai1 n @ r ) ((pos "K6$" ) ))
( "consignors"  n ( k @1 n s ai1 n @ z ) ((pos "Kj$" ) ))
( "consigns"  v ( k @1 n s ai1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "consist"  v ( k @1 n s i1 s t ) ((pos "I0%" ) (subcat )))
( "consisted"  v ( k @1 n s i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "consistence"  n ( k @1 n s i1 s t @ n s ) ((pos "L@%" ) ))
( "consistencies"  n ( k @1 n s i1 s t @ n s i z ) ((pos "Mj%" ) ))
( "consistency"  n ( k @1 n s i1 s t @ n s ii ) ((pos "M8%" ) ))
( "consistent"  j ( k @1 n s i1 s t @ n t ) ((pos "OA%" ) ))
( "consistently"  a ( k @1 n s i1 s t @ n t l ii ) ((pos "Pu%" ) ))
( "consisting"  v ( k @1 n s i1 s t i ng ) ((pos "Ib%" ) (subcat )))
( "consistories"  n ( k @1 n s i1 s t @ r i z ) ((pos "Kj$" ) ))
( "consistory"  n ( k @1 n s i1 s t @ r ii ) ((pos "K8$" ) ))
( "consists"  v ( k @1 n s i1 s t s ) ((pos "Ia%" ) (subcat )))
( "consolable"  j ( k @1 n s ou1 l @ b l ) ((pos "OA%" ) ))
( "consolation"  n ( k o2 n s @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "consolations"  n ( k o2 n s @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "consolatory"  j ( k @1 n s o1 l @ t @ r ii ) ((pos "OA$" ) ))
( "console"  n ( k o1 n s ou l ) ((pos "K6%" ) ))
( "console"  v ( k @1 n s ou1 l ) ((pos "H2%" ) (subcat "6A" )))
( "consoled"  v ( k @1 n s ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "consoles"  n ( k o1 n s ou l z ) ((pos "Kj%" ) ))
( "consoles"  v ( k @1 n s ou1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "consolidate"  v ( k @1 n s o1 l i d ei t ) ((pos "J2%" ) (subcat "2A" )))
( "consolidated"  v ( k @1 n s o1 l i d ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "consolidates"  v ( k @1 n s o1 l i d ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "consolidating"  v ( k @1 n s o1 l i d ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "consolidation"  n ( k @1 n s o2 l i d ei1 sh @ n ) ((pos "M6%" ) ))
( "consolidations"  n ( k @1 n s o2 l i d ei1 sh @ n z ) ((pos "Mj$" ) ))
( "consoling"  v ( k @1 n s ou1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "consols"  n ( k o1 n s o l z ) ((pos "Kj%" ) ))
( "consomm_e"  n ( k @1 n s o1 m ei ) ((pos "M6%" ) ))
( "consomm_es"  n ( k @1 n s o1 m ei z ) ((pos "Mj$" ) ))
( "consonance"  n ( k o1 n s @ n @ n s ) ((pos "L@$" ) ))
( "consonant"  n ( k o1 n s @ n @ n t ) ((pos "K6%" "OA%" ) ))
( "consonants"  n ( k o1 n s @ n @ n t s ) ((pos "Kj%" ) ))
( "consort"  n ( k o1 n s oo t ) ((pos "K6%" ) ))
( "consort"  v ( k @1 n s oo1 t ) ((pos "I0%" ) (subcat "2C" )))
( "consorted"  v ( k @1 n s oo1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "consortia"  n ( k @1 n s oo1 t i@ ) ((pos "Kj%" ) ))
( "consorting"  v ( k @1 n s oo1 t i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "consortium"  n ( k @1 n s oo1 t i@ m ) ((pos "Ki%" ) ))
( "consorts"  n ( k o1 n s oo t s ) ((pos "Kj%" ) ))
( "consorts"  v ( k @1 n s oo1 t s ) ((pos "Ia%" ) (subcat "2C" )))
( "conspectus"  n ( k @1 n s p e1 k t @ s ) ((pos "K7$" ) ))
( "conspectuses"  n ( k @1 n s p e1 k t @ s i z ) ((pos "Kj$" ) ))
( "conspicuous"  j ( k @1 n s p i1 k y u@ s ) ((pos "OA%" ) ))
( "conspicuously"  a ( k @1 n s p i1 k y u@ s l ii ) ((pos "Pu%" ) ))
( "conspicuousness"  n ( k @1 n s p i1 k y u@ s n @ s ) ((pos "L@%" ) ))
( "conspiracies"  n ( k @1 n s p i1 r @ s i z ) ((pos "Mj%" ) ))
( "conspiracy"  n ( k @1 n s p i1 r @ s ii ) ((pos "M8%" ) ))
( "conspirator"  n ( k @1 n s p i1 r @ t @ r ) ((pos "K6%" ) ))
( "conspiratorial"  j ( k @1 n s p i2 r @ t oo1 r i@ l ) ((pos "OA%" ) ))
( "conspirators"  n ( k @1 n s p i1 r @ t @ z ) ((pos "Kj%" ) ))
( "conspire"  v ( k @1 n s p ai1 @ r ) ((pos "J2%" ) (subcat "2A" "3A" "4A" )))
( "conspired"  v ( k @1 n s p ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" )))
( "conspires"  v ( k @1 n s p ai1 @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "4A" )))
( "conspiring"  v ( k @1 n s p ai1 @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" )))
( "constable"  n ( k o1 n s t @ b l ) ((pos "K6%" ) ))
( "constables"  n ( k o1 n s t @ b l z ) ((pos "Kj%" ) ))
( "constabularies"  n ( k @1 n s t a1 b y u l @ r i z ) ((pos "Kj%" ) ))
( "constabulary"  n ( k @1 n s t a1 b y u l @ r ii ) ((pos "K8%" ) ))
( "constancy"  n ( k o1 n s t @ n s ii ) ((pos "L@%" ) ))
( "constant"  n ( k o1 n s t @ n t ) ((pos "K6%" "OA%" ) ))
( "constantly"  a ( k o1 n s t @ n t l ii ) ((pos "Pu%" ) ))
( "constants"  n ( k o1 n s t @ n t s ) ((pos "Kj%" ) ))
( "constellation"  n ( k o2 n s t @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "constellations"  n ( k o2 n s t @ l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "consternation"  n ( k o2 n s t @ n ei1 sh @ n ) ((pos "L@%" ) ))
( "constipate"  v ( k o1 n s t i p ei t ) ((pos "H2%" ) (subcat )))
( "constipated"  v ( k o1 n s t i p ei t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "constipates"  v ( k o1 n s t i p ei t s ) ((pos "Ha%" ) (subcat )))
( "constipating"  v ( k o1 n s t i p ei t i ng ) ((pos "Hb%" ) (subcat )))
( "constipation"  n ( k o2 n s t i p ei1 sh @ n ) ((pos "L@%" ) ))
( "constituencies"  n ( k @1 n s t i1 t y u@ n s i z ) ((pos "Kj%" ) ))
( "constituency"  n ( k @1 n s t i1 t y u@ n s ii ) ((pos "K8%" ) ))
( "constituent"  n ( k @1 n s t i1 t y u@ n t ) ((pos "K6%" "OA%" ) ))
( "constituents"  n ( k @1 n s t i1 t y u@ n t s ) ((pos "Kj%" ) ))
( "constitute"  v ( k o1 n s t i t y uu t ) ((pos "H2%" ) (subcat "6A" )))
( "constituted"  v ( k o1 n s t i t y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "constitutes"  v ( k o1 n s t i t y uu t s ) ((pos "Ha%" ) (subcat "6A" )))
( "constituting"  v ( k o1 n s t i t y uu t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "constitution"  n ( k o2 n s t i t y uu1 sh @ n ) ((pos "M6%" ) ))
( "constitutional"  n ( k o2 n s t i t y uu1 sh @ n l ) ((pos "K6%" "OA%" ) ))
( "constitutionalism"  n ( k o2 n s t i t y uu1 sh @ n @ l i z @ m ) ((pos "L@$" ) ))
( "constitutionalist"  n ( k o2 n s t i t y uu1 sh @ n @ l i s t ) ((pos "K6$" ) ))
( "constitutionalists"  n ( k o2 n s t i t y uu1 sh @ n @ l i s t s ) ((pos "Kj$" ) ))
( "constitutionalize"  v ( k o2 n s t i t y uu1 sh @ n @ l ai z ) ((pos "H2$" ) ))
( "constitutionalized"  v ( k o2 n s t i t y uu1 sh @ n @ l ai z d ) ((pos "Hc$" "Hd$" ) ))
( "constitutionalizes"  v ( k o2 n s t i t y uu1 sh @ n @ l ai z i z ) ((pos "Ha$" ) ))
( "constitutionalizing"  v ( k o2 n s t i t y uu1 sh @ n @ l ai z i ng ) ((pos "Hb$" ) ))
( "constitutionally"  a ( k o2 n s t i t y uu1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "constitutionals"  n ( k o2 n s t i t y uu1 sh @ n l z ) ((pos "Kj$" ) ))
( "constitutions"  n ( k o2 n s t i t y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "constitutive"  j ( k @1 n s t i1 t y u t i v ) ((pos "OA$" ) ))
( "constrain"  v ( k @1 n s t r ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "constrained"  v ( k @1 n s t r ei1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "constrainedly"  a ( k @1 n s t r ei1 n @ d l ii ) ((pos "Pu%" ) ))
( "constraining"  v ( k @1 n s t r ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "constrains"  v ( k @1 n s t r ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "constraint"  n ( k @1 n s t r ei1 n t ) ((pos "M6%" ) ))
( "constraints"  n ( k @1 n s t r ei1 n t s ) ((pos "Mj%" ) ))
( "constrict"  v ( k @1 n s t r i1 k t ) ((pos "H0%" ) (subcat )))
( "constricted"  v ( k @1 n s t r i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "constricting"  v ( k @1 n s t r i1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "constriction"  n ( k @1 n s t r i1 k sh @ n ) ((pos "M6%" ) ))
( "constrictions"  n ( k @1 n s t r i1 k sh @ n z ) ((pos "Mj%" ) ))
( "constricts"  v ( k @1 n s t r i1 k t s ) ((pos "Ha%" ) (subcat )))
( "construct"  v ( k @1 n s t r uh1 k t ) ((pos "H0%" ) (subcat )))
( "constructed"  v ( k @1 n s t r uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "constructing"  v ( k @1 n s t r uh1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "construction"  n ( k @1 n s t r uh1 k sh @ n ) ((pos "M6%" ) ))
( "constructional"  j ( k @1 n s t r uh1 k sh @ n l ) ((pos "OA%" ) ))
( "constructions"  n ( k @1 n s t r uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "constructive"  j ( k @1 n s t r uh1 k t i v ) ((pos "OA%" ) ))
( "constructively"  a ( k @1 n s t r uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "constructor"  n ( k @1 n s t r uh1 k t @ r ) ((pos "K6%" ) ))
( "constructors"  n ( k @1 n s t r uh1 k t @ z ) ((pos "Kj%" ) ))
( "constructs"  v ( k @1 n s t r uh1 k t s ) ((pos "Ha%" ) (subcat )))
( "construe"  v ( k @1 n s t r uu1 ) ((pos "J2%" ) (subcat "2A" )))
( "construed"  v ( k @1 n s t r uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "construes"  v ( k @1 n s t r uu1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "construing"  v ( k @1 n s t r uu1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "consubstantiation"  n ( k o2 n s @ b s t a2 n sh i ei1 sh @ n ) ((pos "L@$" ) ))
( "consul"  n ( k o1 n s l ) ((pos "K6%" ) ))
( "consular"  j ( k o1 n s y u l @ r ) ((pos "OA%" ) ))
( "consulate"  n ( k o1 n s y u l @ t ) ((pos "K6%" ) ))
( "consulates"  n ( k o1 n s y u l @ t s ) ((pos "Kj%" ) ))
( "consuls"  n ( k o1 n s l z ) ((pos "Kj%" ) ))
( "consulship"  n ( k o1 n s @ l sh i p ) ((pos "K6%" ) ))
( "consulships"  n ( k o1 n s @ l sh i p s ) ((pos "Kj$" ) ))
( "consult"  v ( k @1 n s uh1 l t ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "consultant"  n ( k @1 n s uh1 l t @ n t ) ((pos "K6%" ) ))
( "consultants"  n ( k @1 n s uh1 l t @ n t s ) ((pos "Kj%" ) ))
( "consultation"  n ( k o2 n s l t ei1 sh @ n ) ((pos "M6%" ) ))
( "consultations"  n ( k o2 n s l t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "consultative"  j ( k @1 n s uh1 l t @ t i v ) ((pos "OA%" ) ))
( "consulted"  v ( k @1 n s uh1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "consulting"  v ( k @1 n s uh1 l t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "consults"  v ( k @1 n s uh1 l t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "consume"  v ( k @1 n s y uu1 m ) ((pos "J2%" ) (subcat )))
( "consumed"  v ( k @1 n s y uu1 m d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "consumer"  n ( k @1 n s y uu1 m @ r ) ((pos "K6%" ) ))
( "consumers"  n ( k @1 n s y uu1 m @ z ) ((pos "Kj%" ) ))
( "consumes"  v ( k @1 n s y uu1 m z ) ((pos "Ja%" ) (subcat )))
( "consuming"  v ( k @1 n s y uu1 m i ng ) ((pos "Jb%" "OA%" ) (subcat )))
( "consummate"  v ( k o1 n s @ m ei t ) ((pos "H2%" ) (subcat )))
( "consummate"  j ( k @1 n s uh1 m @ t ) ((pos "OA%" ) ))
( "consummated"  v ( k o1 n s @ m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "consummates"  v ( k o1 n s @ m ei t s ) ((pos "Ha%" ) (subcat )))
( "consummating"  v ( k o1 n s @ m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "consummation"  n ( k o2 n s @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "consummations"  n ( k o2 n s @ m ei1 sh @ n z ) ((pos "Mj$" ) ))
( "consumption"  n ( k @1 n s uh1 m p sh @ n ) ((pos "L@%" ) ))
( "consumptive"  n ( k @1 n s uh1 m p t i v ) ((pos "K6%" "OA%" ) ))
( "consumptives"  n ( k @1 n s uh1 m p t i v z ) ((pos "Kj%" ) ))
( "cont"  n ( k o1 n t ) ((pos "Y~%" ) ))
( "contact"  v ( k o1 n t a k t ) ((pos "H0%" "M6%" ) (subcat )))
( "contacted"  v ( k o1 n t a k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "contacting"  v ( k o1 n t a k t i ng ) ((pos "Hb%" ) (subcat )))
( "contacts"  v ( k o1 n t a k t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "contagion"  n ( k @1 n t ei1 jh @ n ) ((pos "M6%" ) ))
( "contagions"  n ( k @1 n t ei1 jh @ n z ) ((pos "Mj$" ) ))
( "contagious"  j ( k @1 n t ei1 jh @ s ) ((pos "OA%" ) ))
( "contagiously"  a ( k @1 n t ei1 jh @ s l ii ) ((pos "Pu%" ) ))
( "contain"  v ( k @1 n t ei1 n ) ((pos "H0%" ) (subcat )))
( "contained"  v ( k @1 n t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "container"  n ( k @1 n t ei1 n @ r ) ((pos "K6%" ) ))
( "containers"  n ( k @1 n t ei1 n @ z ) ((pos "Kj%" ) ))
( "containing"  v ( k @1 n t ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "containment"  n ( k @1 n t ei1 n m @ n t ) ((pos "L@%" ) ))
( "contains"  v ( k @1 n t ei1 n z ) ((pos "Ha%" ) (subcat )))
( "contaminant"  n ( k @1 n t a1 m i n @ n t ) ((pos "K6%" ) ))
( "contaminants"  n ( k @1 n t a1 m i n @ n t s ) ((pos "Kj%" ) ))
( "contaminate"  v ( k @1 n t a1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "contaminated"  v ( k @1 n t a1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "contaminates"  v ( k @1 n t a1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "contaminating"  v ( k @1 n t a1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "contamination"  n ( k @1 n t a2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "contaminations"  n ( k @1 n t a2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "contemn"  v ( k @1 n t e1 m ) ((pos "H0$" ) (subcat )))
( "contemned"  v ( k @1 n t e1 m d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "contemning"  v ( k @1 n t e1 m i ng ) ((pos "Hb$" ) (subcat )))
( "contemns"  v ( k @1 n t e1 m z ) ((pos "Ha$" ) (subcat )))
( "contemplate"  v ( k o1 n t @ m p l ei t ) ((pos "J2%" ) (subcat "2A" "6A" "6C" )))
( "contemplated"  v ( k o1 n t @ m p l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "6C" )))
( "contemplates"  v ( k o1 n t @ m p l ei t s ) ((pos "Ja%" ) (subcat "2A" "6A" "6C" )))
( "contemplating"  v ( k o1 n t @ m p l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "6C" )))
( "contemplation"  n ( k o2 n t @ m p l ei1 sh @ n ) ((pos "L@%" ) ))
( "contemplative"  j ( k @1 n t e1 m p l @ t i v ) ((pos "OA%" ) ))
( "contemporaneous"  j ( k @1 n t e2 m p @ r ei1 n i@ s ) ((pos "OA%" ) ))
( "contemporaneously"  a ( k @1 n t e2 m p @ r ei1 n i@ s l ii ) ((pos "Pu%" ) ))
( "contemporaries"  n ( k @1 n t e1 m p r @ r i z ) ((pos "Kj%" ) ))
( "contemporary"  n ( k @1 n t e1 m p r @ r ii ) ((pos "K8%" "OA%" ) ))
( "contempt"  n ( k @1 n t e1 m p t ) ((pos "L@%" ) ))
( "contemptible"  j ( k @1 n t e1 m p t @ b l ) ((pos "OA%" ) ))
( "contemptuous"  j ( k @1 n t e1 m p ch u@ s ) ((pos "OA%" ) ))
( "contemptuously"  a ( k @1 n t e1 m p ch u@ s l ii ) ((pos "Pu%" ) ))
( "contend"  v ( k @1 n t e1 n d ) ((pos "J0%" ) (subcat "3A" )))
( "contended"  v ( k @1 n t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "contender"  n ( k @1 n t e1 n d @ r ) ((pos "K6%" ) ))
( "contenders"  n ( k @1 n t e1 n d @ z ) ((pos "Kj%" ) ))
( "contending"  v ( k @1 n t e1 n d i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "contends"  v ( k @1 n t e1 n d z ) ((pos "Ja%" ) (subcat "3A" )))
( "content"  n ( k o1 n t e n t ) ((pos "K6%" ) ))
( "content"  v ( k @1 n t e1 n t ) ((pos "H0%" "L@%" "OA%" ) (subcat "6A" )))
( "contented"  v ( k @1 n t e1 n t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "contentedly"  a ( k @1 n t e1 n t i d l ii ) ((pos "Pu%" ) ))
( "contenting"  v ( k @1 n t e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "contention"  n ( k @1 n t e1 n sh @ n ) ((pos "M6%" ) ))
( "contentions"  n ( k @1 n t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "contentious"  j ( k @1 n t e1 n sh @ s ) ((pos "OA%" ) ))
( "contentment"  n ( k @1 n t e1 n t m @ n t ) ((pos "L@%" ) ))
( "contents"  n ( k o1 n t e n t s ) ((pos "Kj%" ) ))
( "contents"  v ( k @1 n t e1 n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "conterminous"  j ( k o1 n t @@1 m i n @ s ) ((pos "OA$" ) ))
( "contest"  n ( k o1 n t e s t ) ((pos "K6%" ) ))
( "contest"  v ( k @1 n t e1 s t ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "contestant"  n ( k @1 n t e1 s t @ n t ) ((pos "K6%" ) ))
( "contestants"  n ( k @1 n t e1 s t @ n t s ) ((pos "Kj%" ) ))
( "contested"  v ( k @1 n t e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "contesting"  v ( k @1 n t e1 s t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "contests"  n ( k o1 n t e s t s ) ((pos "Kj%" ) ))
( "contests"  v ( k @1 n t e1 s t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "context"  n ( k o1 n t e k s t ) ((pos "M6%" ) ))
( "contexts"  n ( k o1 n t e k s t s ) ((pos "Mj%" ) ))
( "contextual"  j ( k @1 n t e1 k s ch u@ l ) ((pos "OA%" ) ))
( "contiguity"  n ( k o2 n t i g y uu1 i t ii ) ((pos "L@%" ) ))
( "contiguous"  j ( k @1 n t i1 g y u@ s ) ((pos "OA%" ) ))
( "contiguously"  a ( k @1 n t i1 g y u@ s l ii ) ((pos "Pu%" ) ))
( "continence"  n ( k o1 n t i n @ n s ) ((pos "L@%" ) ))
( "continent"  n ( k o1 n t i n @ n t ) ((pos "K6%" "OA%" ) ))
( "continental"  j ( k o2 n t i n e1 n t l ) ((pos "OA%" ) ))
( "continents"  n ( k o1 n t i n @ n t s ) ((pos "Kj%" ) ))
( "contingencies"  n ( k @1 n t i1 n jh @ n s i z ) ((pos "Mj%" ) ))
( "contingency"  n ( k @1 n t i1 n jh @ n s ii ) ((pos "M8%" ) ))
( "contingent"  n ( k @1 n t i1 n jh @ n t ) ((pos "K6%" "OA%" ) ))
( "contingents"  n ( k @1 n t i1 n jh @ n t s ) ((pos "Kj$" ) ))
( "continual"  j ( k @1 n t i1 n y u@ l ) ((pos "OA%" ) ))
( "continually"  a ( k @1 n t i1 n y u@ l ii ) ((pos "Pu%" ) ))
( "continuance"  n ( k @1 n t i1 n y u@ n s ) ((pos "Ki%" ) ))
( "continuation"  n ( k @1 n t i2 n y u ei1 sh @ n ) ((pos "M6%" ) ))
( "continuations"  n ( k @1 n t i2 n y u ei1 sh @ n z ) ((pos "Mj$" ) ))
( "continue"  v ( k @1 n t i1 n y uu ) ((pos "J2%" ) (subcat "2A" "2B" "2D" "2E" "6A" "6D" "7A" )))
( "continued"  v ( k @1 n t i1 n y uu d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2D" "2E" "6A" "6D" "7A" )))
( "continues"  v ( k @1 n t i1 n y uu z ) ((pos "Ja%" ) (subcat "2A" "2B" "2D" "2E" "6A" "6D" "7A" )))
( "continuing"  v ( k @1 n t i1 n y uu i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2D" "2E" "6A" "6D" "7A" )))
( "continuity"  n ( k o2 n t i n y uu1 i t ii ) ((pos "L@%" ) ))
( "continuous"  j ( k @1 n t i1 n y u@ s ) ((pos "OA%" ) ))
( "continuously"  a ( k @1 n t i1 n y u@ s l ii ) ((pos "Pu%" ) ))
( "contort"  v ( k @1 n t oo1 t ) ((pos "H0%" ) (subcat "6A" )))
( "contorted"  v ( k @1 n t oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "contorting"  v ( k @1 n t oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "contortion"  n ( k @1 n t oo1 sh @ n ) ((pos "M6%" ) ))
( "contortionist"  n ( k @1 n t oo1 sh @ n i s t ) ((pos "K6%" ) ))
( "contortionists"  n ( k @1 n t oo1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "contortions"  n ( k @1 n t oo1 sh @ n z ) ((pos "Mj%" ) ))
( "contorts"  v ( k @1 n t oo1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "contour"  v ( k o1 n t u@ r ) ((pos "H0%" "K6%" ) (subcat )))
( "contoured"  v ( k o1 n t u@ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "contouring"  v ( k o1 n t u@ r i ng ) ((pos "Hb%" ) (subcat )))
( "contours"  v ( k o1 n t u@ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "contra-"  j ( k o2 n t r @ - ) ((pos "U-%" ) ))
( "contraband"  n ( k o1 n t r @ b a n d ) ((pos "L@%" ) ))
( "contrabass"  n ( k o2 n t r @ b ei1 s ) ((pos "K7%" ) ))
( "contrabasses"  n ( k o2 n t r @ b ei1 s i z ) ((pos "Kj%" ) ))
( "contraception"  n ( k o2 n t r @ s e1 p sh @ n ) ((pos "L@%" ) ))
( "contraceptive"  n ( k o2 n t r @ s e1 p t i v ) ((pos "K6%" "OA%" ) ))
( "contraceptives"  n ( k o2 n t r @ s e1 p t i v z ) ((pos "Kj%" ) ))
( "contract"  n ( k o1 n t r a k t ) ((pos "M6%" ) ))
( "contract"  v ( k @1 n t r a1 k t ) ((pos "J0%" ) (subcat "2A" "2C" "4A" "6A" )))
( "contracted"  v ( k @1 n t r a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" "6A" )))
( "contractible"  j ( k o1 n t r a k t @ b l ) ((pos "OA$" ) ))
( "contractile"  j ( k @1 n t r a1 k t ai l ) ((pos "OA$" ) ))
( "contracting"  v ( k @1 n t r a1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" "6A" )))
( "contraction"  n ( k @1 n t r a1 k sh @ n ) ((pos "M6%" ) ))
( "contractions"  n ( k @1 n t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "contractor"  n ( k o1 n t r a k t @ r ) ((pos "K6%" ) ))
( "contractors"  n ( k o1 n t r a k t @ z ) ((pos "Kj%" ) ))
( "contracts"  n ( k o1 n t r a k t s ) ((pos "Mj%" ) ))
( "contracts"  v ( k @1 n t r a1 k t s ) ((pos "Ja%" ) (subcat "2A" "2C" "4A" "6A" )))
( "contractual"  j ( k @1 n t r a1 k ch u@ l ) ((pos "OA%" ) ))
( "contradict"  v ( k o2 n t r @ d i1 k t ) ((pos "H0%" ) (subcat )))
( "contradicted"  v ( k o2 n t r @ d i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "contradicting"  v ( k o2 n t r @ d i1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "contradiction"  n ( k o2 n t r @ d i1 k sh @ n ) ((pos "M6%" ) ))
( "contradictions"  n ( k o2 n t r @ d i1 k sh @ n z ) ((pos "Mj%" ) ))
( "contradictory"  j ( k o2 n t r @ d i1 k t @ r ii ) ((pos "OA%" ) ))
( "contradicts"  v ( k o2 n t r @ d i1 k t s ) ((pos "Ha%" ) (subcat )))
( "contradistinction"  n ( k o2 n t r @ d i s t i1 ng k sh @ n ) ((pos "K6$" ) ))
( "contradistinctions"  n ( k o2 n t r @ d i s t i1 ng k sh @ n z ) ((pos "Kj$" ) ))
( "contradistinguish"  v ( k o2 n t r @ d i s t i1 ng g w i sh ) ((pos "H1$" ) (subcat )))
( "contradistinguished"  v ( k o2 n t r @ d i s t i1 ng g w i sh t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "contradistinguishes"  v ( k o2 n t r @ d i s t i1 ng g w i sh i z ) ((pos "Ha$" ) (subcat )))
( "contradistinguishing"  v ( k o2 n t r @ d i s t i1 ng g w i sh i ng ) ((pos "Hb$" ) (subcat )))
( "contralto"  n ( k @1 n t r a1 l t ou ) ((pos "K6%" ) ))
( "contraltos"  n ( k @1 n t r a1 l t ou z ) ((pos "Kj%" ) ))
( "contraption"  n ( k @1 n t r a1 p sh @ n ) ((pos "K6%" ) ))
( "contraptions"  n ( k @1 n t r a1 p sh @ n z ) ((pos "Kj%" ) ))
( "contrapuntal"  j ( k o2 n t r @ p uh1 n t l ) ((pos "OA%" ) ))
( "contraries"  n ( k o1 n t r @ r i z ) ((pos "Mj$" ) ))
( "contrarieties"  n ( k o2 n t r @ r ai1 @ t i z ) ((pos "Mj$" ) ))
( "contrariety"  n ( k o2 n t r @ r ai1 @ t ii ) ((pos "M8$" ) ))
( "contrarily"  a ( k o1 n t r @ r @ l ii ) ((pos "Pu%" ) ))
( "contrariness"  n ( k @1 n t r e@1 r i n @ s ) ((pos "L@%" ) ))
( "contrariwise"  a ( k o2 n t r e@1 r i w ai z ) ((pos "Pu%" ) ))
( "contrary"  n ( k o1 n t r @ r ii ) ((pos "M8%" "OA%" ) ))
( "contrary"  j ( k @1 n t r e@1 r ii ) ((pos "OA%" ) ))
( "contrast"  n ( k o1 n t r aa s t ) ((pos "M6%" ) ))
( "contrast"  v ( k @1 n t r aa1 s t ) ((pos "J0%" ) (subcat "2C" "3A" "6A" )))
( "contrasted"  v ( k @1 n t r aa1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "contrasting"  v ( k @1 n t r aa1 s t i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "contrastingly"  a ( k @1 n t r aa1 s t i ng l ii ) ((pos "Pu%" ) ))
( "contrasts"  n ( k o1 n t r aa s t s ) ((pos "Mj%" ) ))
( "contrasts"  v ( k @1 n t r aa1 s t s ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" )))
( "contravene"  v ( k o2 n t r @ v ii1 n ) ((pos "H2%" ) (subcat )))
( "contravened"  v ( k o2 n t r @ v ii1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "contravenes"  v ( k o2 n t r @ v ii1 n z ) ((pos "Ha%" ) (subcat )))
( "contravening"  v ( k o2 n t r @ v ii1 n i ng ) ((pos "Hb%" ) (subcat )))
( "contravention"  n ( k o2 n t r @ v e1 n sh @ n ) ((pos "M6%" ) ))
( "contraventions"  n ( k o2 n t r @ v e1 n sh @ n z ) ((pos "Mj%" ) ))
( "contretemps"  n ( k o1 n t r @ t o m ) ((pos "K9%" ) ))
( "contribute"  v ( k @1 n t r i1 b y uu t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "contributed"  v ( k @1 n t r i1 b y uu t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "contributes"  v ( k @1 n t r i1 b y uu t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "contributing"  v ( k @1 n t r i1 b y uu t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "contribution"  n ( k o2 n t r i b y uu1 sh @ n ) ((pos "M6%" ) ))
( "contributions"  n ( k o2 n t r i b y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "contributor"  n ( k @1 n t r i1 b y uu t @ r ) ((pos "K6%" ) ))
( "contributors"  n ( k @1 n t r i1 b y uu t @ z ) ((pos "Kj%" ) ))
( "contributory"  j ( k @1 n t r i1 b y u t @ r ii ) ((pos "OA%" ) ))
( "contrite"  j ( k @1 n t r ai1 t ) ((pos "OA%" ) ))
( "contritely"  a ( k @1 n t r ai1 t l ii ) ((pos "Pu%" ) ))
( "contrition"  n ( k @1 n t r i1 sh @ n ) ((pos "L@%" ) ))
( "contrivance"  n ( k @1 n t r ai1 v @ n s ) ((pos "M6%" ) ))
( "contrivances"  n ( k @1 n t r ai1 v @ n s i z ) ((pos "Mj%" ) ))
( "contrive"  v ( k @1 n t r ai1 v ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "contrived"  v ( k @1 n t r ai1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "contriver"  n ( k @1 n t r ai1 v @ r ) ((pos "K6$" ) ))
( "contrivers"  n ( k @1 n t r ai1 v @ z ) ((pos "Kj$" ) ))
( "contrives"  v ( k @1 n t r ai1 v z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "contriving"  v ( k @1 n t r ai1 v i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "control"  v ( k @1 n t r ou1 l ) ((pos "H4%" "M6%" ) (subcat )))
( "controllable"  j ( k @1 n t r ou1 l @ b l ) ((pos "OA%" ) ))
( "controlled"  v ( k @1 n t r ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "controller"  n ( k @1 n t r ou1 l @ r ) ((pos "K6%" ) ))
( "controllers"  n ( k @1 n t r ou1 l @ z ) ((pos "Kj%" ) ))
( "controlling"  v ( k @1 n t r ou1 l i ng ) ((pos "Hb%" ) (subcat )))
( "controls"  v ( k @1 n t r ou1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "controversial"  j ( k o2 n t r @ v @@1 sh l ) ((pos "OA%" ) ))
( "controversialist"  n ( k o2 n t r @ v @@1 sh @ l i s t ) ((pos "K6$" ) ))
( "controversialists"  n ( k o2 n t r @ v @@1 sh @ l i s t s ) ((pos "Kj$" ) ))
( "controversially"  a ( k o2 n t r @ v @@1 sh @ l ii ) ((pos "Pu%" ) ))
( "controversies"  n ( k o1 n t r @ v @@ s i z ) ((pos "Mj%" ) ))
( "controversy"  n ( k o1 n t r @ v @@ s ii ) ((pos "M8%" ) ))
( "controvert"  v ( k o2 n t r @ v @@1 t ) ((pos "H0$" ) (subcat )))
( "controverted"  v ( k o2 n t r @ v @@1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "controverting"  v ( k o2 n t r @ v @@1 t i ng ) ((pos "Hb$" ) (subcat )))
( "controverts"  v ( k o2 n t r @ v @@1 t s ) ((pos "Ha$" ) (subcat )))
( "contumacies"  n ( k o1 n t y u m @ s i z ) ((pos "Mj$" ) ))
( "contumacious"  j ( k o2 n t y uu m ei1 sh @ s ) ((pos "OA$" ) ))
( "contumaciously"  a ( k o2 n t y uu m ei1 sh @ s l ii ) ((pos "Pu$" ) ))
( "contumacy"  n ( k o1 n t y u m @ s ii ) ((pos "M8$" ) ))
( "contumelies"  n ( k o1 n t y uu m l i z ) ((pos "Mj$" ) ))
( "contumelious"  j ( k o2 n t y uu m ii1 l i@ s ) ((pos "OA$" ) ))
( "contumeliously"  a ( k o2 n t y uu m ii1 l i@ s l ii ) ((pos "Pu$" ) ))
( "contumely"  n ( k o1 n t y uu m l ii ) ((pos "M8$" ) ))
( "contuse"  v ( k @1 n t y uu1 z ) ((pos "H2$" ) (subcat )))
( "contused"  v ( k @1 n t y uu1 z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "contuses"  v ( k @1 n t y uu1 z i z ) ((pos "Ha$" ) (subcat )))
( "contusing"  v ( k @1 n t y uu1 z i ng ) ((pos "Hb$" ) (subcat )))
( "contusion"  n ( k @1 n t y uu1 zh n ) ((pos "K6%" ) ))
( "contusions"  n ( k @1 n t y uu1 zh n z ) ((pos "Kj%" ) ))
( "conundrum"  n ( k @1 n uh1 n d r @ m ) ((pos "K6%" ) ))
( "conundrums"  n ( k @1 n uh1 n d r @ m z ) ((pos "Kj%" ) ))
( "conurbation"  n ( k o2 n @@ b ei1 sh @ n ) ((pos "K6%" ) ))
( "conurbations"  n ( k o2 n @@ b ei1 sh @ n z ) ((pos "Kj%" ) ))
( "convalesce"  v ( k o2 n v @ l e1 s ) ((pos "I2%" ) (subcat )))
( "convalesced"  v ( k o2 n v @ l e1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "convalescence"  n ( k o2 n v @ l e1 s n s ) ((pos "L@%" ) ))
( "convalescent"  n ( k o2 n v @ l e1 s n t ) ((pos "K6%" "OA%" ) ))
( "convalescents"  n ( k o2 n v @ l e1 s n t s ) ((pos "Kj%" ) ))
( "convalesces"  v ( k o2 n v @ l e1 s i z ) ((pos "Ia%" ) (subcat )))
( "convalescing"  v ( k o2 n v @ l e1 s i ng ) ((pos "Ib%" ) (subcat )))
( "convection"  n ( k @1 n v e1 k sh @ n ) ((pos "L@%" ) ))
( "convector"  n ( k @1 n v e1 k t @ r ) ((pos "K6%" ) ))
( "convectors"  n ( k @1 n v e1 k t @ z ) ((pos "Kj%" ) ))
( "convene"  v ( k @1 n v ii1 n ) ((pos "J2%" ) (subcat "2A" )))
( "convened"  v ( k @1 n v ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "convener"  n ( k @1 n v ii1 n @ r ) ((pos "K6%" ) ))
( "conveners"  n ( k @1 n v ii1 n @ z ) ((pos "Kj%" ) ))
( "convenes"  v ( k @1 n v ii1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "convenience"  n ( k @1 n v ii1 n i@ n s ) ((pos "M6%" ) ))
( "conveniences"  n ( k @1 n v ii1 n i@ n s i z ) ((pos "Mj%" ) ))
( "convenient"  j ( k @1 n v ii1 n i@ n t ) ((pos "OA%" ) ))
( "conveniently"  a ( k @1 n v ii1 n i@ n t l ii ) ((pos "Pu%" ) ))
( "convening"  v ( k @1 n v ii1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "convent"  n ( k o1 n v @ n t ) ((pos "K6%" ) ))
( "conventicle"  n ( k @1 n v e1 n t i k l ) ((pos "K6$" ) ))
( "conventicles"  n ( k @1 n v e1 n t i k l z ) ((pos "Kj$" ) ))
( "convention"  n ( k @1 n v e1 n sh @ n ) ((pos "M6%" ) ))
( "conventional"  j ( k @1 n v e1 n sh @ n l ) ((pos "OA%" ) ))
( "conventionalities"  n ( k @1 n v e2 n sh @ n a1 l i t i z ) ((pos "Mj$" ) ))
( "conventionality"  n ( k @1 n v e2 n sh @ n a1 l i t ii ) ((pos "M8$" ) ))
( "conventionally"  a ( k @1 n v e1 n sh @ n @ l ii ) ((pos "Pu%" ) ))
( "conventions"  n ( k @1 n v e1 n sh @ n z ) ((pos "Mj%" ) ))
( "convents"  n ( k o1 n v @ n t s ) ((pos "Kj%" ) ))
( "converge"  v ( k @1 n v @@1 jh ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "converged"  v ( k @1 n v @@1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "convergence"  n ( k @1 n v @@1 jh @ n s ) ((pos "K6%" ) ))
( "convergences"  n ( k @1 n v @@1 jh @ n s i z ) ((pos "Kj$" ) ))
( "convergent"  j ( k @1 n v @@1 jh @ n t ) ((pos "OA%" ) ))
( "converges"  v ( k @1 n v @@1 jh i z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "converging"  v ( k @1 n v @@1 jh i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "conversant"  j ( k @1 n v @@1 s n t ) ((pos "OA%" ) ))
( "conversation"  n ( k o2 n v @ s ei1 sh @ n ) ((pos "M6%" ) ))
( "conversational"  j ( k o2 n v @ s ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "conversationalist"  n ( k o2 n v @ s ei1 sh @ n @ l i s t ) ((pos "K6%" ) ))
( "conversationalists"  n ( k o2 n v @ s ei1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "conversationally"  a ( k o2 n v @ s ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "conversations"  n ( k o2 n v @ s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "converse"  n ( k o1 n v @@ s ) ((pos "Mi%" "OA%" ) ))
( "converse"  v ( k @1 n v @@1 s ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "conversed"  v ( k @1 n v @@1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "conversely"  a ( k @1 n v @@1 s l ii ) ((pos "Pu%" ) ))
( "converses"  v ( k @1 n v @@1 s i z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "conversing"  v ( k @1 n v @@1 s i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "conversion"  n ( k @1 n v @@1 sh @ n ) ((pos "M6%" ) ))
( "conversions"  n ( k @1 n v @@1 sh @ n z ) ((pos "Mj%" ) ))
( "convert"  n ( k o1 n v @@ t ) ((pos "K6%" ) ))
( "convert"  v ( k @1 n v @@1 t ) ((pos "H0%" ) (subcat "6A" )))
( "converted"  v ( k @1 n v @@1 t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "converter"  n ( k @1 n v @@1 t @ r ) ((pos "K6%" ) ))
( "converters"  n ( k @1 n v @@1 t @ z ) ((pos "Kj%" ) ))
( "convertibility"  n ( k @1 n v @@2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "convertible"  n ( k @1 n v @@1 t @ b l ) ((pos "K6%" "OA%" ) ))
( "convertibles"  n ( k @1 n v @@1 t @ b l z ) ((pos "Kj%" ) ))
( "converting"  v ( k @1 n v @@1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "converts"  n ( k o1 n v @@ t s ) ((pos "Kj%" ) ))
( "converts"  v ( k @1 n v @@1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "convex"  j ( k o1 n v e k s ) ((pos "OA%" ) ))
( "convexity"  n ( k @1 n v e1 k s i t ii ) ((pos "L@$" ) ))
( "convexly"  a ( k o1 n v e k s l ii ) ((pos "Pu$" ) ))
( "convey"  v ( k @1 n v ei1 ) ((pos "H0%" ) (subcat "6A" )))
( "conveyance"  n ( k @1 n v ei1 @ n s ) ((pos "M6%" ) ))
( "conveyancer"  n ( k @1 n v ei1 @ n s @ r ) ((pos "K6%" ) ))
( "conveyancers"  n ( k @1 n v ei1 @ n s @ z ) ((pos "Kj%" ) ))
( "conveyances"  n ( k @1 n v ei1 @ n s i z ) ((pos "Mj%" ) ))
( "conveyed"  v ( k @1 n v ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "conveyer"  n ( k @1 n v ei1 @ r ) ((pos "K6%" ) ))
( "conveyer-belt"  n ( k @1 n v ei1 @ - b e l t ) ((pos "K6%" ) ))
( "conveyer-belts"  n ( k @1 n v ei1 @ - b e l t s ) ((pos "Kj%" ) ))
( "conveyers"  n ( k @1 n v ei1 @ z ) ((pos "Kj%" ) ))
( "conveying"  v ( k @1 n v ei1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "conveyor"  n ( k @1 n v ei1 @ r ) ((pos "K6%" ) ))
( "conveyors"  n ( k @1 n v ei1 @ z ) ((pos "Kj%" ) ))
( "conveys"  v ( k @1 n v ei1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "convict"  n ( k o1 n v i k t ) ((pos "K6%" ) ))
( "convict"  v ( k @1 n v i1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "convicted"  v ( k @1 n v i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "convicting"  v ( k @1 n v i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "conviction"  n ( k @1 n v i1 k sh @ n ) ((pos "M6%" ) ))
( "convictions"  n ( k @1 n v i1 k sh @ n z ) ((pos "Mj%" ) ))
( "convicts"  n ( k o1 n v i k t s ) ((pos "Kj%" ) ))
( "convicts"  v ( k @1 n v i1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "convince"  v ( k @1 n v i1 n s ) ((pos "H2%" ) (subcat "6A" "11" )))
( "convinced"  v ( k @1 n v i1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" )))
( "convinces"  v ( k @1 n v i1 n s i z ) ((pos "Ha%" ) (subcat "6A" "11" )))
( "convincible"  j ( k @1 n v i1 n s @ b l ) ((pos "OA$" ) ))
( "convincing"  v ( k @1 n v i1 n s i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "11" )))
( "convincingly"  a ( k @1 n v i1 n s i ng l ii ) ((pos "Pu%" ) ))
( "convivial"  j ( k @1 n v i1 v i@ l ) ((pos "OA%" ) ))
( "convivialities"  n ( k @1 n v i2 v i a1 l i t i z ) ((pos "Mj$" ) ))
( "conviviality"  n ( k @1 n v i2 v i a1 l i t ii ) ((pos "M8%" ) ))
( "convivially"  a ( k @1 n v i1 v i@ l ii ) ((pos "Pu%" ) ))
( "convocation"  n ( k o2 n v @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "convocations"  n ( k o2 n v @ k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "convoke"  v ( k @1 n v ou1 k ) ((pos "H2$" ) (subcat )))
( "convoked"  v ( k @1 n v ou1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "convokes"  v ( k @1 n v ou1 k s ) ((pos "Ha$" ) (subcat )))
( "convoking"  v ( k @1 n v ou1 k i ng ) ((pos "Hb$" ) (subcat )))
( "convoluted"  j ( k o2 n v @ l uu1 t i d ) ((pos "OA%" ) ))
( "convolution"  n ( k o2 n v @ l uu1 sh @ n ) ((pos "K6%" ) ))
( "convolutions"  n ( k o2 n v @ l uu1 sh @ n z ) ((pos "Kj%" ) ))
( "convolvulus"  n ( k @1 n v o1 l v y u l @ s ) ((pos "K7%" ) ))
( "convolvuluses"  n ( k @1 n v o1 l v y u l @ s i z ) ((pos "Kj$" ) ))
( "convoy"  v ( k o1 n v oi ) ((pos "H0%" "M6%" ) (subcat )))
( "convoyed"  v ( k o1 n v oi d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "convoying"  v ( k o1 n v oi i ng ) ((pos "Hb%" ) (subcat )))
( "convoys"  v ( k o1 n v oi z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "convulse"  v ( k @1 n v uh1 l s ) ((pos "H2%" ) (subcat )))
( "convulsed"  v ( k @1 n v uh1 l s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "convulses"  v ( k @1 n v uh1 l s i z ) ((pos "Ha%" ) (subcat )))
( "convulsing"  v ( k @1 n v uh1 l s i ng ) ((pos "Hb%" ) (subcat )))
( "convulsion"  n ( k @1 n v uh1 l sh @ n ) ((pos "K6%" ) ))
( "convulsions"  n ( k @1 n v uh1 l sh @ n z ) ((pos "Kj%" ) ))
( "convulsive"  j ( k @1 n v uh1 l s i v ) ((pos "OA%" ) ))
( "convulsively"  a ( k @1 n v uh1 l s i v l ii ) ((pos "Pu%" ) ))
( "cony"  n ( k ou1 n ii ) ((pos "K8$" ) ))
( "coo"  v ( k uu1 ) ((pos "J0%" "K6%" ) ))
( "cooed"  v ( k uu1 d ) ((pos "Jc%" "Jd%" ) ))
( "cooing"  v ( k uu1 i ng ) ((pos "Jb%" ) ))
( "cook"  v ( k u1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "12B" "13B" )))
( "cookbook"  n ( k u1 k b u k ) ((pos "K6%" ) ))
( "cookbooks"  n ( k u1 k b u k s ) ((pos "Kj%" ) ))
( "cooked"  v ( k u1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "12B" "13B" )))
( "cooker"  n ( k u1 k @ r ) ((pos "K6%" ) ))
( "cookers"  n ( k u1 k @ z ) ((pos "Kj%" ) ))
( "cookery"  n ( k u1 k @ r ii ) ((pos "L@%" ) ))
( "cookery-book"  n ( k u1 k @ r i - b u k ) ((pos "K6%" ) ))
( "cookery-books"  n ( k u1 k @ r i - b u k s ) ((pos "Kj%" ) ))
( "cookhouse"  n ( k u1 k h au s ) ((pos "K6%" ) ))
( "cookhouses"  n ( k u1 k h au z i z ) ((pos "Kj%" ) ))
( "cookie"  n ( k u1 k ii ) ((pos "K6%" ) ))
( "cookies"  n ( k u1 k i z ) ((pos "Kj%" ) ))
( "cooking"  v ( k u1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "6A" "12B" "13B" )))
( "cooks"  v ( k u1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "12B" "13B" )))
( "cooky"  n ( k u1 k ii ) ((pos "K8$" ) ))
( "cool"  v ( k uu1 l ) ((pos "J0%" "L@%" "OC%" ) (subcat "2A" "2C" )))
( "cool-headed"  j ( k uu1 l - h e d i d ) ((pos "OA%" ) ))
( "coolant"  n ( k uu1 l @ n t ) ((pos "M6%" ) ))
( "coolants"  n ( k uu1 l @ n t s ) ((pos "Mj%" ) ))
( "cooled"  v ( k uu1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "cooler"  n ( k uu1 l @ r ) ((pos "K6%" "Or%" ) ))
( "coolers"  n ( k uu1 l @ z ) ((pos "Kj%" ) ))
( "coolest"  j ( k uu1 l i s t ) ((pos "Os%" ) ))
( "coolie"  n ( k uu1 l ii ) ((pos "K6%" ) ))
( "coolies"  n ( k uu1 l i z ) ((pos "Kj%" ) ))
( "cooling"  v ( k uu1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "cooling-tower"  n ( k uu1 l i ng - t au @ r ) ((pos "K6%" ) ))
( "cooling-towers"  n ( k uu1 l i ng - t au @ z ) ((pos "Kj%" ) ))
( "coolly"  a ( k uu1 l - l ii ) ((pos "Pu%" ) ))
( "coolness"  n ( k uu1 l n @ s ) ((pos "L@%" ) ))
( "cools"  v ( k uu1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "coon"  n ( k uu1 n ) ((pos "K6%" ) ))
( "coons"  n ( k uu1 n z ) ((pos "Kj%" ) ))
( "coop"  v ( k uu1 p ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "cooped"  v ( k uu1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cooper"  n ( k uu1 p @ r ) ((pos "K6$" ) ))
( "cooperate"  v ( k ou1 o1 p @ r ei t ) ((pos "I2%" ) (subcat "2A" "3A" )))
( "cooperated"  v ( k ou1 o1 p @ r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "cooperates"  v ( k ou1 o1 p @ r ei t s ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "cooperating"  v ( k ou1 o1 p @ r ei t i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "cooperation"  n ( k ou1 o2 p @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "cooperative"  n ( k ou1 o1 p @ r @ t i v ) ((pos "K6%" "OA%" ) ))
( "cooperatives"  n ( k ou1 o1 p @ r @ t i v z ) ((pos "Kj%" ) ))
( "cooperator"  n ( k ou1 o1 p @ r ei t @ r ) ((pos "K6%" ) ))
( "cooperators"  n ( k ou1 o1 p @ r ei t @ z ) ((pos "Kj%" ) ))
( "coopers"  n ( k uu1 p @ z ) ((pos "Kj$" ) ))
( "cooping"  v ( k uu1 p i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "coops"  v ( k uu1 p s ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "coordinate"  n ( k ou2 oo1 d @ n @ t ) ((pos "K6%" "OA%" ) ))
( "coordinate"  v ( k ou2 oo1 d i n ei t ) ((pos "H2%" ) (subcat )))
( "coordinated"  v ( k ou2 oo1 d i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "coordinately"  a ( k ou2 oo1 d i n @ t l ii ) ((pos "Pu$" ) ))
( "coordinates"  n ( k ou2 oo1 d @ n @ t s ) ((pos "Kj%" ) ))
( "coordinates"  v ( k ou2 oo1 d i n ei t s ) ((pos "Ha%" ) (subcat )))
( "coordinating"  v ( k ou2 oo1 d i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "coordination"  n ( k ou2 oo2 d i n ei1 sh @ n ) ((pos "M6%" ) ))
( "coordinations"  n ( k ou2 oo2 d i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "coordinator"  n ( k ou1 oo1 d i n ei t @ r ) ((pos "K6%" ) ))
( "coordinators"  n ( k ou1 oo1 d i n ei t @ z ) ((pos "Kj%" ) ))
( "coos"  v ( k uu1 z ) ((pos "Ja%" "Kj%" ) ))
( "coot"  n ( k uu1 t ) ((pos "K6%" ) ))
( "coots"  n ( k uu1 t s ) ((pos "Kj%" ) ))
( "cop"  v ( k o1 p ) ((pos "J4%" "K6%" ) (subcat )))
( "cop-out"  n ( k o1 p - au t ) ((pos "K6%" ) ))
( "cop-outs"  n ( k o1 p - au t s ) ((pos "Kj$" ) ))
( "copartner"  n ( k ou2 p aa1 t n @ r ) ((pos "K6%" ) ))
( "copartners"  n ( k ou2 p aa1 t n @ z ) ((pos "Kj%" ) ))
( "copartnership"  n ( k ou1 p aa1 t n @ sh i p ) ((pos "K6%" ) ))
( "copartnerships"  n ( k ou1 p aa1 t n @ sh i p s ) ((pos "Kj%" ) ))
( "cope"  v ( k ou1 p ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "copeck"  n ( k ou1 p e k ) ((pos "K6$" ) ))
( "copecks"  n ( k ou1 p e k s ) ((pos "Kj$" ) ))
( "coped"  v ( k ou1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "copes"  v ( k ou1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "copied"  v ( k o1 p i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "copies"  v ( k o1 p i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "15A" )))
( "coping"  v ( k ou1 p i ng ) ((pos "Ib%" "K6%" ) (subcat "2A" )))
( "coping-stone"  n ( k ou1 p i ng - s t ou n ) ((pos "K6%" ) ))
( "coping-stones"  n ( k ou1 p i ng - s t ou n z ) ((pos "Kj%" ) ))
( "copings"  n ( k ou1 p i ng z ) ((pos "Kj$" ) ))
( "copious"  j ( k ou1 p i@ s ) ((pos "OA%" ) ))
( "copiously"  a ( k ou1 p i@ s l ii ) ((pos "Pu%" ) ))
( "copped"  v ( k o1 p t ) ((pos "Jc%" "Jd%" ) (subcat )))
( "copper"  v ( k o1 p @ r ) ((pos "H0$" "M6%" ) ))
( "copper-bottom"  v ( k o2 p @ - b o1 t @ m ) ((pos "H0$" ) ))
( "copper-bottomed"  j ( k o1 p @ - b o t @ m d ) ((pos "OA%" ) ))
( "copper-bottomed"  v ( k o2 p @ - b o1 t @ m d ) ((pos "Hc$" "Hd%" ) ))
( "copper-bottoming"  v ( k o2 p @ - b o1 t @ m i ng ) ((pos "Hb$" ) ))
( "copper-bottoms"  v ( k o2 p @ - b o1 t @ m z ) ((pos "Ha$" ) ))
( "coppered"  v ( k o1 p @ d ) ((pos "Hc$" "Hd$" ) ))
( "copperhead"  n ( k o1 p @ h e d ) ((pos "K6$" ) ))
( "copperheads"  n ( k o1 p @ h e d z ) ((pos "Kj$" ) ))
( "coppering"  v ( k o1 p @ r i ng ) ((pos "Hb$" ) ))
( "copperplate"  n ( k o1 p @ p l ei t ) ((pos "L@%" ) ))
( "coppers"  v ( k o1 p @ z ) ((pos "Ha$" "Mj%" ) ))
( "coppersmith"  n ( k o1 p @ s m i th ) ((pos "K6%" ) ))
( "coppersmiths"  n ( k o1 p @ s m i th s ) ((pos "Kj%" ) ))
( "coppice"  n ( k o1 p i s ) ((pos "K6%" ) ))
( "coppices"  n ( k o1 p i s i z ) ((pos "Kj%" ) ))
( "copping"  v ( k o1 p i ng ) ((pos "Jb%" ) (subcat )))
( "copra"  n ( k o1 p r @ ) ((pos "L@$" ) ))
( "cops"  v ( k o1 p s ) ((pos "Ja%" "Kj%" ) (subcat )))
( "copse"  n ( k o1 p s ) ((pos "K6%" ) ))
( "copses"  n ( k o1 p s i z ) ((pos "Kj%" ) ))
( "copula"  n ( k o1 p y u l @ ) ((pos "K6$" ) ))
( "copulas"  n ( k o1 p y u l @ z ) ((pos "Kj$" ) ))
( "copulate"  v ( k o1 p y u l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "copulated"  v ( k o1 p y u l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "copulates"  v ( k o1 p y u l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "copulating"  v ( k o1 p y u l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "copulation"  n ( k o2 p y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "copulations"  n ( k o2 p y u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "copulative"  n ( k o1 p y u l @ t i v ) ((pos "K6$" "OA$" ) ))
( "copulatives"  n ( k o1 p y u l @ t i v z ) ((pos "Kj$" ) ))
( "copy"  v ( k o1 p ii ) ((pos "J3%" "M8%" ) (subcat "2A" "6A" "15A" )))
( "copybook"  n ( k o1 p i b u k ) ((pos "K6%" ) ))
( "copybooks"  n ( k o1 p i b u k s ) ((pos "Kj%" ) ))
( "copycat"  n ( k o1 p i k a t ) ((pos "K6%" ) ))
( "copycats"  n ( k o1 p i k a t s ) ((pos "Kj%" ) ))
( "copyhold"  n ( k o1 p i h ou l d ) ((pos "L@$" ) ))
( "copyholder"  n ( k o1 p i h ou l d @ r ) ((pos "K6$" ) ))
( "copyholders"  n ( k o1 p i h ou l d @ z ) ((pos "Kj$" ) ))
( "copying"  v ( k o1 p i i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "copyist"  n ( k o1 p i i s t ) ((pos "K6$" ) ))
( "copyists"  n ( k o1 p i i s t s ) ((pos "Kj$" ) ))
( "copyright"  v ( k o1 p i r ai t ) ((pos "H0%" "M6%" ) ))
( "copyrighted"  v ( k o1 p i r ai t i d ) ((pos "Hc$" "Hd%" ) ))
( "copyrighting"  v ( k o1 p i r ai t i ng ) ((pos "Hb$" ) ))
( "copyrights"  v ( k o1 p i r ai t s ) ((pos "Ha$" "Mj$" ) ))
( "copywriter"  n ( k o1 p i r ai t @ r ) ((pos "K6%" ) ))
( "copywriters"  n ( k o1 p i r ai t @ z ) ((pos "Kj%" ) ))
( "coquetries"  n ( k o1 k i t r i z ) ((pos "Mj$" ) ))
( "coquetry"  n ( k o1 k i t r ii ) ((pos "M8%" ) ))
( "coquette"  n ( k o1 k e1 t ) ((pos "K6%" ) ))
( "coquettes"  n ( k o1 k e1 t s ) ((pos "Kj%" ) ))
( "coquettish"  j ( k ou1 k e1 t i sh ) ((pos "OA%" ) ))
( "coquettishly"  a ( k ou1 k e1 t i sh l ii ) ((pos "Pu%" ) ))
( "cor anglais"  n ( k oo2 r - o1 ng g l ei ) ((pos "Ki%" ) ))
( "coracle"  n ( k o1 r @ k l ) ((pos "K6%" ) ))
( "coracles"  n ( k o1 r @ k l z ) ((pos "Kj%" ) ))
( "coral"  n ( k o1 r @ l ) ((pos "M6%" "OA%" ) ))
( "coral-reef"  n ( k o1 r @ l - r ii f ) ((pos "K6%" ) ))
( "coral-reefs"  n ( k o1 r @ l - r ii f s ) ((pos "Kj%" ) ))
( "corals"  n ( k o1 r @ l z ) ((pos "Mj%" ) ))
( "corbel"  n ( k oo1 b l ) ((pos "K6$" ) ))
( "corbels"  n ( k oo1 b l z ) ((pos "Kj$" ) ))
( "cord"  v ( k oo1 d ) ((pos "H0%" "M6%" ) ))
( "cordage"  n ( k oo1 d i jh ) ((pos "L@$" ) ))
( "corded"  v ( k oo1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "cordial"  n ( k oo1 d i@ l ) ((pos "M6%" "OA%" ) ))
( "cordialities"  n ( k oo2 d i a1 l i t i z ) ((pos "Mj$" ) ))
( "cordiality"  n ( k oo2 d i a1 l i t ii ) ((pos "M8%" ) ))
( "cordially"  a ( k oo1 d i@ l ii ) ((pos "Pu%" ) ))
( "cordials"  n ( k oo1 d i@ l z ) ((pos "Mj%" ) ))
( "cording"  v ( k oo1 d i ng ) ((pos "Hb$" ) ))
( "cordite"  n ( k oo1 d ai t ) ((pos "L@%" ) ))
( "cordon"  v ( k oo1 d n ) ((pos "H0%" "K6%" ) (subcat )))
( "cordon bleu"  j ( k oo2 d o n - b l @@1 ) ((pos "OA%" ) ))
( "cordoned"  v ( k oo1 d n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cordoning"  v ( k oo1 d n i ng ) ((pos "Hb%" ) (subcat )))
( "cordons"  v ( k oo1 d n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cords"  v ( k oo1 d z ) ((pos "Ha%" "Mj%" ) ))
( "corduroy"  n ( k oo1 d @ r oi ) ((pos "L@%" ) ))
( "core"  v ( k oo1 r ) ((pos "H2$" "K6%" ) (subcat )))
( "cored"  v ( k oo1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "coreligionist"  n ( k ou2 r i l i1 jh @ n i s t ) ((pos "K6%" ) ))
( "coreligionists"  n ( k ou2 r i l i1 jh @ n i s t s ) ((pos "Kj%" ) ))
( "cores"  v ( k oo1 z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "corgi"  n ( k oo1 g ii ) ((pos "K6%" ) ))
( "corgis"  n ( k oo1 g i z ) ((pos "Kj%" ) ))
( "coring"  v ( k oo1 r i ng ) ((pos "Hb$" ) (subcat )))
( "cork"  v ( k oo1 k ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "corkage"  n ( k oo1 k i jh ) ((pos "L@%" ) ))
( "corked"  v ( k oo1 k t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "corker"  n ( k oo1 k @ r ) ((pos "K6%" ) ))
( "corkers"  n ( k oo1 k @ z ) ((pos "Kj%" ) ))
( "corking"  v ( k oo1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "corks"  v ( k oo1 k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "corkscrew"  n ( k oo1 k s k r uu ) ((pos "K6%" ) ))
( "corkscrews"  n ( k oo1 k s k r uu z ) ((pos "Kj%" ) ))
( "corm"  n ( k oo1 m ) ((pos "K6%" ) ))
( "cormorant"  n ( k oo1 m @ r @ n t ) ((pos "K6%" ) ))
( "cormorants"  n ( k oo1 m @ r @ n t s ) ((pos "Kj%" ) ))
( "corms"  n ( k oo1 m z ) ((pos "Kj%" ) ))
( "corn"  v ( k oo1 n ) ((pos "H0$" "M6%" ) ))
( "corn-exchange"  n ( k oo1 n - i k s ch ei n jh ) ((pos "K6%" ) ))
( "corn-exchanges"  n ( k oo1 n - i k s ch ei n jh i z ) ((pos "Kj%" ) ))
( "corncob"  n ( k oo1 n k o b ) ((pos "K6%" ) ))
( "corncobs"  n ( k oo1 n k o b z ) ((pos "Kj%" ) ))
( "corncrake"  n ( k oo1 n k r ei k ) ((pos "K6%" ) ))
( "corncrakes"  n ( k oo1 n k r ei k s ) ((pos "Kj%" ) ))
( "cornea"  n ( k oo1 n i@ ) ((pos "K6%" ) ))
( "corneas"  n ( k oo1 n i@ z ) ((pos "Kj%" ) ))
( "corned"  v ( k oo1 n d ) ((pos "Hc$" "Hd$" ) ))
( "cornelian"  n ( k oo1 n ii1 l i@ n ) ((pos "K6$" ) ))
( "cornelians"  n ( k oo1 n ii1 l i@ n z ) ((pos "Kj$" ) ))
( "corner"  v ( k oo1 n @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "corner-kick"  n ( k oo1 n @ - k i k ) ((pos "K6%" ) ))
( "corner-kicks"  n ( k oo1 n @ - k i k s ) ((pos "Kj%" ) ))
( "cornered"  v ( k oo1 n @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "cornering"  v ( k oo1 n @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "corners"  v ( k oo1 n @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "cornerstone"  n ( k oo1 n @ s t ou n ) ((pos "K6%" ) ))
( "cornerstones"  n ( k oo1 n @ s t ou n z ) ((pos "Kj%" ) ))
( "cornet"  n ( k oo1 n i t ) ((pos "K6%" ) ))
( "cornets"  n ( k oo1 n i t s ) ((pos "Kj%" ) ))
( "cornflake"  n ( k oo1 n f l ei k ) ((pos "K6%" ) ))
( "cornflakes"  n ( k oo1 n f l ei k s ) ((pos "Kj%" ) ))
( "cornflour"  n ( k oo1 n f l au @ r ) ((pos "L@%" ) ))
( "cornflower"  n ( k oo1 n f l au @ r ) ((pos "K6%" ) ))
( "cornflowers"  n ( k oo1 n f l au @ z ) ((pos "Kj%" ) ))
( "cornice"  n ( k oo1 n i s ) ((pos "K6%" ) ))
( "cornices"  n ( k oo1 n i s i z ) ((pos "Kj%" ) ))
( "cornier"  j ( k oo1 n i@ r ) ((pos "Or%" ) ))
( "corniest"  j ( k oo1 n i i s t ) ((pos "Os%" ) ))
( "corning"  v ( k oo1 n i ng ) ((pos "Hb$" ) ))
( "cornpone"  n ( k oo1 n p ou n ) ((pos "L@$" ) ))
( "corns"  v ( k oo1 n z ) ((pos "Ha$" "Mj%" ) ))
( "cornstarch"  n ( k oo1 n s t aa ch ) ((pos "L@%" ) ))
( "cornucopia"  n ( k oo2 n y u k ou1 p i@ ) ((pos "K6%" ) ))
( "cornucopias"  n ( k oo2 n y u k ou1 p i@ z ) ((pos "Kj$" ) ))
( "corny"  j ( k oo1 n ii ) ((pos "OD%" ) ))
( "corolla"  n ( k @1 r o1 l @ ) ((pos "K6$" ) ))
( "corollaries"  n ( k @1 r o1 l @ r i z ) ((pos "Kj%" ) ))
( "corollary"  n ( k @1 r o1 l @ r ii ) ((pos "K8%" ) ))
( "corollas"  n ( k @1 r o1 l @ z ) ((pos "Kj$" ) ))
( "corona"  n ( k @1 r ou1 n @ ) ((pos "K6%" ) ))
( "coronaries"  n ( k o1 r @ n r i z ) ((pos "Kj%" ) ))
( "coronary"  n ( k o1 r @ n r ii ) ((pos "K8%" "OA%" ) ))
( "coronas"  n ( k @1 r ou1 n @ z ) ((pos "Kj$" ) ))
( "coronation"  n ( k o2 r @ n ei1 sh @ n ) ((pos "K6%" ) ))
( "coronations"  n ( k o2 r @ n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "coroner"  n ( k o1 r @ n @ r ) ((pos "K6%" ) ))
( "coroners"  n ( k o1 r @ n @ z ) ((pos "Kj%" ) ))
( "coronet"  n ( k o1 r @ n e t ) ((pos "K6%" ) ))
( "coronets"  n ( k o1 r @ n e t s ) ((pos "Kj%" ) ))
( "corpora"  n ( k oo1 p @ r @ ) ((pos "Kj$" ) ))
( "corporal"  n ( k oo1 p @ r @ l ) ((pos "K6%" "OA%" ) ))
( "corporals"  n ( k oo1 p @ r @ l z ) ((pos "Kj%" ) ))
( "corporate"  j ( k oo1 p @ r @ t ) ((pos "OA%" ) ))
( "corporation"  n ( k oo2 p @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "corporations"  n ( k oo2 p @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "corporeal"  j ( k oo1 p oo1 r i@ l ) ((pos "OA%" ) ))
( "corps"  n ( k oo1 r ) ((pos "Ki$" ) ))
( "corps"  n ( k oo1 z ) ((pos "Kj$" ) ))
( "corps de ballet"  n ( k oo2 - d @ - b a1 l ei ) ((pos "Ki%" ) ))
( "corps de ballets"  n ( k oo2 - d @ - b a1 l ei ) ((pos "Kj$" ) ))
( "corpse"  n ( k oo1 p s ) ((pos "K6%" ) ))
( "corpses"  n ( k oo1 p s i z ) ((pos "Kj%" ) ))
( "corpulence"  n ( k oo1 p y u l @ n s ) ((pos "L@%" ) ))
( "corpulent"  j ( k oo1 p y u l @ n t ) ((pos "OA%" ) ))
( "corpus"  n ( k oo1 p @ s ) ((pos "Ki%" ) ))
( "corpuscle"  n ( k oo1 p uh s l ) ((pos "K6%" ) ))
( "corpuscles"  n ( k oo1 p uh s l z ) ((pos "Kj%" ) ))
( "corral"  v ( k @1 r aa1 l ) ((pos "H4$" "K6%" ) (subcat )))
( "corralled"  v ( k @1 r aa1 l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "corralling"  v ( k @1 r aa1 l i ng ) ((pos "Hb$" ) (subcat )))
( "corrals"  v ( k @1 r aa1 l z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "correct"  v ( k @1 r e1 k t ) ((pos "H0%" "OA%" ) (subcat "6A" )))
( "corrected"  v ( k @1 r e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "correcting"  v ( k @1 r e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "correction"  n ( k @1 r e1 k sh @ n ) ((pos "M6%" ) ))
( "corrections"  n ( k @1 r e1 k sh @ n z ) ((pos "Mj%" ) ))
( "correctitude"  n ( k @1 r e1 k t i t y uu d ) ((pos "L@$" ) ))
( "corrective"  n ( k @1 r e1 k t i v ) ((pos "K6%" "OA%" ) ))
( "correctives"  n ( k @1 r e1 k t i v z ) ((pos "Kj%" ) ))
( "correctly"  a ( k @1 r e1 k t l ii ) ((pos "Pu%" ) ))
( "correctness"  n ( k @1 r e1 k t n @ s ) ((pos "L@%" ) ))
( "corrects"  v ( k @1 r e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "correlate"  v ( k o1 r @ l ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "correlated"  v ( k o1 r @ l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "correlates"  v ( k o1 r @ l ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "correlating"  v ( k o1 r @ l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "correlation"  n ( k o2 r @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "correlations"  n ( k o2 r @ l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "correlative"  n ( k o1 r e1 l @ t i v ) ((pos "K6$" "OA$" ) ))
( "correlatives"  n ( k o1 r e1 l @ t i v z ) ((pos "Kj$" ) ))
( "correspond"  v ( k o2 r i s p o1 n d ) ((pos "I0%" ) (subcat "2A" )))
( "corresponded"  v ( k o2 r i s p o1 n d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "correspondence"  n ( k o2 r i s p o1 n d @ n s ) ((pos "M6%" ) ))
( "correspondences"  n ( k o2 r i s p o1 n d @ n s i z ) ((pos "Mj%" ) ))
( "correspondent"  n ( k o2 r i s p o1 n d @ n t ) ((pos "K6%" ) ))
( "correspondents"  n ( k o2 r i s p o1 n d @ n t s ) ((pos "Kj%" ) ))
( "corresponding"  v ( k o2 r i s p o1 n d i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "correspondingly"  a ( k o2 r i s p o1 n d i ng l ii ) ((pos "Pu%" ) ))
( "corresponds"  v ( k o2 r i s p o1 n d z ) ((pos "Ia%" ) (subcat "2A" )))
( "corridor"  n ( k o1 r i d oo r ) ((pos "K6%" ) ))
( "corridors"  n ( k o1 r i d oo z ) ((pos "Kj%" ) ))
( "corrie"  n ( k o1 r ii ) ((pos "K6$" ) ))
( "corries"  n ( k o1 r i z ) ((pos "Kj$" ) ))
( "corrigenda"  n ( k o2 r i jh e1 n d @ ) ((pos "Kj$" ) ))
( "corrigendum"  n ( k o2 r i jh e1 n d @ m ) ((pos "Ki$" ) ))
( "corrigible"  j ( k o1 r i jh @ b l ) ((pos "OA$" ) ))
( "corroborate"  v ( k @1 r o1 b @ r ei t ) ((pos "H2%" ) (subcat )))
( "corroborated"  v ( k @1 r o1 b @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "corroborates"  v ( k @1 r o1 b @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "corroborating"  v ( k @1 r o1 b @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "corroboration"  n ( k @1 r o2 b @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "corroborative"  j ( k @1 r o1 b @ r @ t i v ) ((pos "OA%" ) ))
( "corrode"  v ( k @1 r ou1 d ) ((pos "J2%" ) (subcat "2A" )))
( "corroded"  v ( k @1 r ou1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "corrodes"  v ( k @1 r ou1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "corroding"  v ( k @1 r ou1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "corrosion"  n ( k @1 r ou1 zh n ) ((pos "L@%" ) ))
( "corrosive"  n ( k @1 r ou1 s i v ) ((pos "K6%" "OA%" ) ))
( "corrosives"  n ( k @1 r ou1 s i v z ) ((pos "Kj%" ) ))
( "corrugate"  v ( k o1 r @ g ei t ) ((pos "J2$" ) (subcat "2A" )))
( "corrugated"  v ( k o1 r @ g ei t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "corrugates"  v ( k o1 r @ g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "corrugating"  v ( k o1 r @ g ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "corrugation"  n ( k o2 r @ g ei1 sh @ n ) ((pos "M6$" ) ))
( "corrugations"  n ( k o2 r @ g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "corrupt"  v ( k @1 r uh1 p t ) ((pos "J0%" "OA%" ) (subcat "2A" )))
( "corrupted"  v ( k @1 r uh1 p t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "corruptibility"  n ( k @1 r uh2 p t @ b i1 l i t ii ) ((pos "L@$" ) ))
( "corruptible"  j ( k @1 r uh1 p t @ b l ) ((pos "OA%" ) ))
( "corrupting"  v ( k @1 r uh1 p t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "corruption"  n ( k @1 r uh1 p sh @ n ) ((pos "L@%" ) ))
( "corruptly"  a ( k @1 r uh1 p t l ii ) ((pos "Pu%" ) ))
( "corruptness"  n ( k @1 r uh1 p t n @ s ) ((pos "L@%" ) ))
( "corrupts"  v ( k @1 r uh1 p t s ) ((pos "Ja%" ) (subcat "2A" )))
( "cors anglais"  n ( k oo2 r - o1 ng g l ei ) ((pos "Kj$" ) ))
( "corsage"  n ( k oo1 s aa1 zh ) ((pos "K6$" ) ))
( "corsages"  n ( k oo1 s aa1 zh i z ) ((pos "Kj$" ) ))
( "corsair"  n ( k oo1 s e@ r ) ((pos "K6$" ) ))
( "corsairs"  n ( k oo1 s e@ z ) ((pos "Kj$" ) ))
( "corse"  n ( k oo1 s ) ((pos "K6$" ) ))
( "corselet"  n ( k oo1 s l i t ) ((pos "K6$" ) ))
( "corselets"  n ( k oo1 s l i t s ) ((pos "Kj$" ) ))
( "corses"  n ( k oo1 s i z ) ((pos "Kj$" ) ))
( "corset"  n ( k oo1 s i t ) ((pos "K6%" ) ))
( "corsets"  n ( k oo1 s i t s ) ((pos "Kj%" ) ))
( "corslet"  n ( k oo1 s l i t ) ((pos "K6$" ) ))
( "corslets"  n ( k oo1 s l i t s ) ((pos "Kj$" ) ))
( "cort`ege"  n ( k oo1 t ei1 zh ) ((pos "K6$" ) ))
( "cort`eges"  n ( k oo1 t ei1 zh i z ) ((pos "Kj$" ) ))
( "cortege"  n ( k oo1 t ei1 zh ) ((pos "K6$" ) ))
( "corteges"  n ( k oo1 t ei1 zh i z ) ((pos "Kj$" ) ))
( "cortex"  n ( k oo1 t e k s ) ((pos "Ki%" ) ))
( "cortical"  j ( k oo1 t i k l ) ((pos "OA%" ) ))
( "cortices"  n ( k oo1 t i s ii z ) ((pos "Kj$" ) ))
( "cortisone"  n ( k oo1 t i z ou n ) ((pos "L@%" ) ))
( "corundum"  n ( k @1 r uh1 n d @ m ) ((pos "L@$" ) ))
( "coruscate"  v ( k o1 r @ s k ei t ) ((pos "I2$" ) (subcat )))
( "coruscated"  v ( k o1 r @ s k ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "coruscates"  v ( k o1 r @ s k ei t s ) ((pos "Ia$" ) (subcat )))
( "coruscating"  v ( k o1 r @ s k ei t i ng ) ((pos "Ib$" ) (subcat )))
( "coruscation"  n ( k o2 r @ s k ei1 sh @ n ) ((pos "K6$" ) ))
( "coruscations"  n ( k o2 r @ s k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "corv_ee"  n ( k oo1 v ei ) ((pos "K6$" ) ))
( "corv_ees"  n ( k oo1 v ei z ) ((pos "Kj$" ) ))
( "corvette"  n ( k oo1 v e1 t ) ((pos "K6$" ) ))
( "corvettes"  n ( k oo1 v e1 t s ) ((pos "Kj$" ) ))
( "cos"  n ( k o1 s ) ((pos "K9$" ) ))
( "cos"  cc ( k @ z ) ((pos "V-$" ) ))
( "cosh"  v ( k o1 sh ) ((pos "H1%" "K7%" ) ))
( "coshed"  v ( k o1 sh t ) ((pos "Hc%" "Hd%" ) ))
( "cosher"  j ( k ou1 sh @ r ) ((pos "OA%" ) ))
( "coshes"  v ( k o1 sh i z ) ((pos "Ha%" "Kj%" ) ))
( "coshing"  v ( k o1 sh i ng ) ((pos "Hb%" ) ))
( "cosier"  j ( k ou1 z i@ r ) ((pos "Or%" ) ))
( "cosies"  n ( k ou1 z i z ) ((pos "Kj%" ) ))
( "cosiest"  j ( k ou1 z i i s t ) ((pos "Os%" ) ))
( "cosignatories"  n ( k ou2 s i1 g n @ t @ r i z ) ((pos "Kj%" ) ))
( "cosignatory"  n ( k ou2 s i1 g n @ t @ r ii ) ((pos "K8%" "OA%" ) ))
( "cosily"  a ( k ou1 z i l ii ) ((pos "Pu%" ) ))
( "cosine"  n ( k ou1 s ai n ) ((pos "K6%" ) ))
( "cosines"  n ( k ou1 s ai n z ) ((pos "Kj%" ) ))
( "cosiness"  n ( k ou1 z i n @ s ) ((pos "L@%" ) ))
( "cosmetic"  n ( k o1 z m e1 t i k ) ((pos "K6%" "OA%" ) ))
( "cosmetician"  n ( k o2 z m @ t i1 sh @ n ) ((pos "K6%" ) ))
( "cosmeticians"  n ( k o2 z m @ t i1 sh @ n z ) ((pos "Kj%" ) ))
( "cosmetics"  n ( k o1 z m e1 t i k s ) ((pos "Kj%" ) ))
( "cosmic"  j ( k o1 z m i k ) ((pos "OA%" ) ))
( "cosmogonies"  n ( k o1 z m o1 g @ n i z ) ((pos "Kj$" ) ))
( "cosmogony"  n ( k o1 z m o1 g @ n ii ) ((pos "K8$" ) ))
( "cosmonaut"  n ( k o1 z m @ n oo t ) ((pos "K6%" ) ))
( "cosmonauts"  n ( k o1 z m @ n oo t s ) ((pos "Kj%" ) ))
( "cosmopolitan"  n ( k o2 z m @ p o1 l i t @ n ) ((pos "K6$" "OA%" ) ))
( "cosmopolitans"  n ( k o2 z m @ p o1 l i t @ n z ) ((pos "Kj$" ) ))
( "cosmos"  n ( k o1 z m o s ) ((pos "Ki%" ) ))
( "cosset"  v ( k o1 s i t ) ((pos "H0%" ) ))
( "cosseted"  v ( k o1 s i t i d ) ((pos "Hc%" "Hd%" ) ))
( "cosseting"  v ( k o1 s i t i ng ) ((pos "Hb%" ) ))
( "cossets"  v ( k o1 s i t s ) ((pos "Ha%" ) ))
( "cost"  v ( k o1 s t ) ((pos "Ic%" "Id%" "J0%" "M6%" ) (subcat "2B" )))
( "costed"  v ( k o1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2B" )))
( "costermonger"  n ( k o1 s t @ m uh ng g @ r ) ((pos "K6%" ) ))
( "costermongers"  n ( k o1 s t @ m uh ng g @ z ) ((pos "Kj%" ) ))
( "costing"  v ( k o1 s t i ng ) ((pos "Jb%" "M6%" ) (subcat "2B" )))
( "costings"  n ( k o1 s t i ng z ) ((pos "Mj%" ) ))
( "costive"  j ( k o1 s t i v ) ((pos "OA$" ) ))
( "costlier"  j ( k o1 s t l i@ r ) ((pos "Or%" ) ))
( "costliest"  j ( k o1 s t l i i s t ) ((pos "Os%" ) ))
( "costliness"  n ( k o1 s t l i n @ s ) ((pos "L@%" ) ))
( "costly"  j ( k o1 s t l ii ) ((pos "OD%" ) ))
( "costs"  v ( k o1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2B" )))
( "costume"  n ( k o1 s t y uu m ) ((pos "M6%" ) ))
( "costumes"  n ( k o1 s t y uu m z ) ((pos "Mj%" ) ))
( "costumier"  n ( k o1 s t y uu1 m i@ r ) ((pos "K6%" ) ))
( "costumiers"  n ( k o1 s t y uu1 m i@ z ) ((pos "Kj%" ) ))
( "cosy"  n ( k ou1 z ii ) ((pos "K8%" "OD%" ) ))
( "cot"  n ( k o1 t ) ((pos "K6%" ) ))
( "cote"  n ( k ou1 t ) ((pos "K6$" ) ))
( "cotenant"  n ( k ou2 t e1 n @ n t ) ((pos "K6%" ) ))
( "cotenants"  n ( k ou2 t e1 n @ n t s ) ((pos "Kj%" ) ))
( "coterie"  n ( k ou1 t @ r ii ) ((pos "K6%" ) ))
( "coteries"  n ( k ou1 t @ r i z ) ((pos "Kj%" ) ))
( "coterminous"  j ( k ou2 t @@1 m i n @ s ) ((pos "OA$" ) ))
( "cotes"  n ( k ou1 t s ) ((pos "Kj$" ) ))
( "cotilion"  n ( k @1 t i1 l i@ n ) ((pos "K6$" ) ))
( "cotilions"  n ( k @1 t i1 l i@ n z ) ((pos "Kj$" ) ))
( "cotillion"  n ( k @1 t i1 l i@ n ) ((pos "K6$" ) ))
( "cotillions"  n ( k @1 t i1 l i@ n z ) ((pos "Kj$" ) ))
( "cots"  n ( k o1 t s ) ((pos "Kj%" ) ))
( "cottage"  n ( k o1 t i jh ) ((pos "K6%" ) ))
( "cottages"  n ( k o1 t i jh i z ) ((pos "Kj%" ) ))
( "cottar"  n ( k o1 t @ r ) ((pos "K6$" ) ))
( "cottars"  n ( k o1 t @ z ) ((pos "Kj$" ) ))
( "cotter"  n ( k o1 t @ r ) ((pos "K6$" ) ))
( "cotters"  n ( k o1 t @ z ) ((pos "Kj$" ) ))
( "cotton"  v ( k o1 t n ) ((pos "I0%" "L@%" ) ))
( "cotton-cake"  n ( k o1 t n - k ei k ) ((pos "L@$" ) ))
( "cotton-plant"  n ( k o1 t n - p l aa n t ) ((pos "K6%" ) ))
( "cotton-plants"  n ( k o1 t n - p l aa n t s ) ((pos "Kj%" ) ))
( "cotton-wool"  n ( k o2 t n - w u1 l ) ((pos "L@%" ) ))
( "cottoned"  v ( k o1 t n d ) ((pos "Ic%" "Id%" ) ))
( "cottoning"  v ( k o1 t n i ng ) ((pos "Ib%" ) ))
( "cottons"  v ( k o1 t n z ) ((pos "Ia%" ) ))
( "cottontail"  n ( k o1 t n t ei l ) ((pos "K6$" ) ))
( "cottontails"  n ( k o1 t n t ei l z ) ((pos "Kj$" ) ))
( "cotyledon"  n ( k o2 t i l ii1 d n ) ((pos "K6$" ) ))
( "cotyledons"  n ( k o2 t i l ii1 d n z ) ((pos "Kj$" ) ))
( "couch"  v ( k au1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "6A" )))
( "couch-grass"  n ( k au1 ch - g r aa s ) ((pos "L@%" ) ))
( "couchant"  j ( k au1 ch @ n t ) ((pos "OA$" ) ))
( "couched"  v ( k au1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "couches"  v ( k au1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "couchette"  n ( k uu1 sh e1 t ) ((pos "K6%" ) ))
( "couchettes"  n ( k uu1 sh e1 t s ) ((pos "Kj%" ) ))
( "couching"  v ( k au1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "cougar"  n ( k uu1 g @ r ) ((pos "K6%" ) ))
( "cougars"  n ( k uu1 g @ z ) ((pos "Kj%" ) ))
( "cough"  v ( k o1 f ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "coughed"  v ( k o1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "coughing"  v ( k o1 f i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "coughs"  v ( k o1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "could"  v ( k u d ) ((pos "Gc*" ) (subcat )))
( "couldn't"  v ( k u1 d n t ) ((pos "Gg%" ) ))
( "couldst"  v ( k u d s t ) ((pos "Gc$" ) ))
( "coulter"  n ( k ou1 l t @ r ) ((pos "K6$" ) ))
( "coulters"  n ( k ou1 l t @ z ) ((pos "Kj$" ) ))
( "council"  n ( k au1 n s l ) ((pos "K6%" ) ))
( "council-board"  n ( k au1 n s l - b oo d ) ((pos "K6$" ) ))
( "council-boards"  n ( k au1 n s l - b oo d z ) ((pos "Kj$" ) ))
( "council-chamber"  n ( k au1 n s l - ch ei m b @ r ) ((pos "K6%" ) ))
( "council-chambers"  n ( k au1 n s l - ch ei m b @ z ) ((pos "Kj%" ) ))
( "councillor"  n ( k au1 n s @ l @ r ) ((pos "K6%" ) ))
( "councillors"  n ( k au1 n s @ l @ z ) ((pos "Kj%" ) ))
( "councils"  n ( k au1 n s l z ) ((pos "Kj%" ) ))
( "counsel"  v ( k au1 n s l ) ((pos "H4%" "L@%" ) (subcat "6A" "6B" )))
( "counselled"  v ( k au1 n s l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6B" )))
( "counselling"  v ( k au1 n s @ l i ng ) ((pos "Hb%" ) (subcat "6A" "6B" )))
( "counsellor"  n ( k au1 n s @ l @ r ) ((pos "K6%" ) ))
( "counsellors"  n ( k au1 n s @ l @ z ) ((pos "Kj%" ) ))
( "counsels"  v ( k au1 n s l z ) ((pos "Ha%" ) (subcat "6A" "6B" )))
( "count"  v ( k au1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "14" "16B" )))
( "countable"  j ( k au1 n t @ b l ) ((pos "OA%" ) ))
( "countdown"  n ( k au1 n t d au n ) ((pos "K6%" ) ))
( "countdowns"  n ( k au1 n t d au n z ) ((pos "Kj$" ) ))
( "counted"  v ( k au1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "16B" )))
( "countenance"  v ( k au1 n t i n @ n s ) ((pos "H2%" "M6%" ) (subcat )))
( "countenanced"  v ( k au1 n t i n @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "countenances"  v ( k au1 n t i n @ n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "countenancing"  v ( k au1 n t i n @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "counter"  v ( k au1 n t @ r ) ((pos "J0%" "K6%" "Pu%" ) (subcat "2A" "3A" "6A" )))
( "counter-"  j ( k au2 n t @ r - ) ((pos "U-%" ) ))
( "counter-example"  n ( k au1 n t @ r - i g z aa2 m p l ) ((pos "K6%" ) ))
( "counter-examples"  n ( k au1 n t @ r - i g z aa2 m p l z ) ((pos "Kj%" ) ))
( "counter-revolution"  n ( k au2 n t @ - r e2 v @ l uu1 sh @ n ) ((pos "M6%" ) ))
( "counter-revolutionaries"  n ( k au2 n t @ - r e2 v @ l uu1 sh @ n @ r ii z ) ((pos "Kj%" ) ))
( "counter-revolutionary"  n ( k au2 n t @ - r e2 v @ l uu1 sh @ n @ r ii ) ((pos "K8%" "OA%" ) ))
( "counter-revolutions"  n ( k au2 n t @ - r e2 v @ l uu1 sh @ n z ) ((pos "Mj%" ) ))
( "counteract"  v ( k au2 n t @ r a1 k t ) ((pos "H0%" ) (subcat )))
( "counteracted"  v ( k au2 n t @ r a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "counteracting"  v ( k au2 n t @ r a1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "counteraction"  n ( k au2 n t @ r a1 k sh @ n ) ((pos "K6%" ) ))
( "counteractions"  n ( k au2 n t @ r a1 k sh @ n z ) ((pos "Kj$" ) ))
( "counteracts"  v ( k au2 n t @ r a1 k t s ) ((pos "Ha%" ) (subcat )))
( "counterattack"  v ( k au1 n t @ r @ t a k ) ((pos "J0%" "K6%" ) ))
( "counterattacked"  v ( k au1 n t @ r @ t a k t ) ((pos "Jc%" "Jd%" ) ))
( "counterattacking"  v ( k au1 n t @ r @ t a k i ng ) ((pos "Jb%" ) ))
( "counterattacks"  v ( k au1 n t @ r @ t a k s ) ((pos "Ja%" "Kj%" ) ))
( "counterattraction"  n ( k au2 n t @ r @ t r a1 k sh @ n ) ((pos "K6%" ) ))
( "counterattractions"  n ( k au2 n t @ r @ t r a1 k sh @ n z ) ((pos "Kj%" ) ))
( "counterbalance"  n ( k au1 n t @ b a l @ n s ) ((pos "K6$" ) ))
( "counterbalance"  v ( k au2 n t @ b a1 l @ n s ) ((pos "H2%" ) (subcat )))
( "counterbalanced"  v ( k au2 n t @ b a1 l @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "counterbalances"  n ( k au1 n t @ b a l @ n s i z ) ((pos "Kj$" ) ))
( "counterbalances"  v ( k au2 n t @ b a1 l @ n s i z ) ((pos "Ha%" ) (subcat )))
( "counterbalancing"  v ( k au2 n t @ b a1 l @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "counterblast"  n ( k au1 n t @ b l aa s t ) ((pos "K6%" ) ))
( "counterblasts"  n ( k au1 n t @ b l aa s t s ) ((pos "Kj$" ) ))
( "counterclaim"  n ( k au1 n t @ k l ei m ) ((pos "K6%" ) ))
( "counterclaims"  n ( k au1 n t @ k l ei m z ) ((pos "Kj%" ) ))
( "counterclockwise"  a ( k au2 n t @ k l o1 k w ai z ) ((pos "Pu$" ) ))
( "countered"  v ( k au1 n t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "counterespionage"  n ( k au2 n t @ r e1 s p i@ n aa zh ) ((pos "L@%" ) ))
( "counterfeit"  v ( k au1 n t @ f i t ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "counterfeited"  v ( k au1 n t @ f i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "counterfeiter"  n ( k au1 n t @ f i t @ r ) ((pos "K6%" ) ))
( "counterfeiters"  n ( k au1 n t @ f i t @ z ) ((pos "Kj%" ) ))
( "counterfeiting"  v ( k au1 n t @ f i t i ng ) ((pos "Hb%" ) (subcat )))
( "counterfeits"  v ( k au1 n t @ f i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "counterfoil"  n ( k au1 n t @ f oi l ) ((pos "K6%" ) ))
( "counterfoils"  n ( k au1 n t @ f oi l z ) ((pos "Kj%" ) ))
( "countering"  v ( k au1 n t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "counterintelligence"  n ( k au2 n t @ r i n t e1 l i jh @ n s ) ((pos "L@%" ) ))
( "counterirritant"  n ( k au2 n t @ r i1 r i t @ n t ) ((pos "K6$" ) ))
( "counterirritants"  n ( k au2 n t @ r i1 r i t @ n t s ) ((pos "Kj$" ) ))
( "countermand"  v ( k au2 n t @ m aa1 n d ) ((pos "H0%" ) (subcat )))
( "countermanded"  v ( k au2 n t @ m aa1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "countermanding"  v ( k au2 n t @ m aa1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "countermands"  v ( k au2 n t @ m aa1 n d z ) ((pos "Ha%" ) (subcat )))
( "countermine"  v ( k au1 n t @ m ai n ) ((pos "J2$" "K6$" ) ))
( "countermined"  v ( k au1 n t @ m ai n d ) ((pos "Jc$" "Jd$" ) ))
( "countermines"  v ( k au1 n t @ m ai n z ) ((pos "Ja$" "Kj$" ) ))
( "countermining"  v ( k au1 n t @ m ai n i ng ) ((pos "Jb$" ) ))
( "counteroffer"  n ( k au1 n t @ r o f @ r ) ((pos "K6%" ) ))
( "counteroffers"  n ( k au1 n t @ r o f @ z ) ((pos "Kj%" ) ))
( "counterpane"  n ( k au1 n t @ p ei n ) ((pos "K6%" ) ))
( "counterpanes"  n ( k au1 n t @ p ei n z ) ((pos "Kj%" ) ))
( "counterpart"  n ( k au1 n t @ p aa t ) ((pos "K6%" ) ))
( "counterparts"  n ( k au1 n t @ p aa t s ) ((pos "Kj%" ) ))
( "counterplot"  v ( k au1 n t @ p l o t ) ((pos "J4$" "K6$" ) ))
( "counterplots"  v ( k au1 n t @ p l o t s ) ((pos "Ja$" "Kj$" ) ))
( "counterplotted"  v ( k au1 n t @ p l o t i d ) ((pos "Jc$" "Jd$" ) ))
( "counterplotting"  v ( k au1 n t @ p l o t i ng ) ((pos "Jb$" ) ))
( "counterpoint"  n ( k au1 n t @ p oi n t ) ((pos "M6%" ) ))
( "counterpoints"  n ( k au1 n t @ p oi n t s ) ((pos "Mj$" ) ))
( "counterpoise"  v ( k au1 n t @ p oi z ) ((pos "H2$" "M6$" ) ))
( "counterpoised"  v ( k au1 n t @ p oi z d ) ((pos "Hc$" "Hd$" ) ))
( "counterpoises"  v ( k au1 n t @ p oi z i z ) ((pos "Ha$" "Mj$" ) ))
( "counterpoising"  v ( k au1 n t @ p oi z i ng ) ((pos "Hb$" ) ))
( "counters"  v ( k au1 n t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "countersank"  v ( k au1 n t @ s a ng k ) ((pos "Hc$" ) ))
( "countersign"  v ( k au1 n t @ s ai n ) ((pos "H0%" "K6%" ) (subcat )))
( "countersigned"  v ( k au1 n t @ s ai n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "countersigning"  v ( k au1 n t @ s ai n i ng ) ((pos "Hb%" ) (subcat )))
( "countersigns"  v ( k au1 n t @ s ai n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "countersink"  v ( k au1 n t @ s i ng k ) ((pos "H5%" ) ))
( "countersinking"  v ( k au1 n t @ s i ng k i ng ) ((pos "Hb$" ) ))
( "countersinks"  v ( k au1 n t @ s i ng k s ) ((pos "Ha$" ) ))
( "countersunk"  v ( k au1 n t @ s uh ng k ) ((pos "Hd%" ) ))
( "countertenor"  n ( k au1 n t @ t e n @ r ) ((pos "K6%" ) ))
( "countertenors"  n ( k au1 n t @ t e n @ z ) ((pos "Kj%" ) ))
( "countervail"  v ( k au1 n t @ v ei l ) ((pos "J0$" ) (subcat "2A" )))
( "countervailed"  v ( k au1 n t @ v ei l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "countervailing"  v ( k au1 n t @ v ei l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "countervails"  v ( k au1 n t @ v ei l z ) ((pos "Ja$" ) (subcat "2A" )))
( "countess"  n ( k au1 n t i s ) ((pos "K7%" ) ))
( "countesses"  n ( k au1 n t i s i z ) ((pos "Kj%" ) ))
( "counties"  n ( k au1 n t i z ) ((pos "Kj%" ) ))
( "counting"  v ( k au1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "16B" )))
( "counting-house"  n ( k au1 n t i ng - h au s ) ((pos "K6%" ) ))
( "counting-houses"  n ( k au1 n t i ng - h au z i z ) ((pos "Kj%" ) ))
( "countless"  j ( k au1 n t l @ s ) ((pos "OA%" ) ))
( "countries"  n ( k uh1 n t r i z ) ((pos "Mj%" ) ))
( "countrified"  j ( k uh1 n t r i f ai d ) ((pos "OA%" ) ))
( "country"  n ( k uh1 n t r ii ) ((pos "M8*" ) ))
( "country-house"  n ( k uh2 n t r i - h au1 s ) ((pos "K6%" ) ))
( "country-houses"  n ( k uh2 n t r i - h au1 z i z ) ((pos "Kj%" ) ))
( "country-seat"  n ( k uh2 n t r i - s ii1 t ) ((pos "K6%" ) ))
( "country-seats"  n ( k uh2 n t r i - s ii1 t s ) ((pos "Kj%" ) ))
( "countryman"  n ( k uh1 n t r i m @ n ) ((pos "Ki%" ) ))
( "countrymen"  n ( k uh1 n t r i m @ n ) ((pos "Kj%" ) ))
( "countryside"  n ( k uh1 n t r i s ai d ) ((pos "L@%" ) ))
( "countrywoman"  n ( k uh1 n t r i w u m @ n ) ((pos "Ki%" ) ))
( "countrywomen"  n ( k uh1 n t r i w i m i n ) ((pos "Kj%" ) ))
( "counts"  v ( k au1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "16B" )))
( "county"  n ( k au1 n t ii ) ((pos "K8%" ) ))
( "coup"  n ( k uu1 ) ((pos "K6%" ) ))
( "coup d'etat"  n ( k uu2 - d ei t aa1 ) ((pos "Ki%" ) ))
( "coup de grace"  n ( k uu2 - d @ - g r aa1 s ) ((pos "Ki%" ) ))
( "coup_e"  n ( k uu1 p ei ) ((pos "K6%" ) ))
( "coup_es"  n ( k uu1 p ei z ) ((pos "Kj%" ) ))
( "couple"  v ( k uh1 p l ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" )))
( "coupled"  v ( k uh1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "couples"  v ( k uh1 p l z ) ((pos "Ja$" "Kj%" ) (subcat "2A" "6A" )))
( "couplet"  n ( k uh1 p l i t ) ((pos "K6%" ) ))
( "couplets"  n ( k uh1 p l i t s ) ((pos "Kj%" ) ))
( "coupling"  v ( k uh1 p l i ng ) ((pos "Jb$" "M6%" ) (subcat "2A" "6A" )))
( "couplings"  n ( k uh1 p l i ng z ) ((pos "Mj%" ) ))
( "coupon"  n ( k uu1 p o n ) ((pos "K6%" ) ))
( "coupons"  n ( k uu1 p o n z ) ((pos "Kj%" ) ))
( "coups"  n ( k uu1 z ) ((pos "Kj%" ) ))
( "coups d'etat"  n ( k uu2 - d ei t aa1 ) ((pos "Kj$" ) ))
( "coups de grace"  n ( k uu2 - d @ - g r aa1 s ) ((pos "Kj$" ) ))
( "courage"  n ( k uh1 r i jh ) ((pos "L@%" ) ))
( "courageous"  j ( k @1 r ei1 jh @ s ) ((pos "OA%" ) ))
( "courageously"  a ( k @1 r ei1 jh @ s l ii ) ((pos "Pu%" ) ))
( "courgette"  n ( k u@1 zh e1 t ) ((pos "K6%" ) ))
( "courgettes"  n ( k u@1 zh e1 t s ) ((pos "Kj%" ) ))
( "courier"  n ( k u1 r i@ r ) ((pos "K6%" ) ))
( "couriers"  n ( k u1 r i@ z ) ((pos "Kj%" ) ))
( "course"  v ( k oo1 s ) ((pos "J2$" "M6*" ) (subcat "2A" "2C" )))
( "coursed"  v ( k oo1 s t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" )))
( "courser"  n ( k oo1 s @ r ) ((pos "K6$" ) ))
( "coursers"  n ( k oo1 s @ z ) ((pos "Kj$" ) ))
( "courses"  v ( k oo1 s i z ) ((pos "Ja$" "Mj%" ) (subcat "2A" "2C" )))
( "coursing"  v ( k oo1 s i ng ) ((pos "Jb$" "K6$" ) (subcat "2A" "2C" )))
( "coursings"  n ( k oo1 s i ng z ) ((pos "Kj$" ) ))
( "court"  v ( k oo1 t ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "court-card"  n ( k oo1 t - k aa d ) ((pos "K6%" ) ))
( "court-cards"  n ( k oo1 t - k aa d z ) ((pos "Kj%" ) ))
( "court-martial"  v ( k oo2 t - m aa1 sh l ) ((pos "H0%" "Ki%" ) (subcat )))
( "court-martialed"  v ( k oo2 t - m aa1 sh l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "court-martialing"  v ( k oo2 t - m aa1 sh @ l i ng ) ((pos "Hb$" ) (subcat )))
( "court-martials"  v ( k oo2 t - m aa1 sh l z ) ((pos "Ha$" ) (subcat )))
( "courted"  v ( k oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "courteous"  j ( k @@1 t i@ s ) ((pos "OA%" ) ))
( "courteously"  a ( k @@1 t i@ s l ii ) ((pos "Pu%" ) ))
( "courtesan"  n ( k oo2 t i z a1 n ) ((pos "K6%" ) ))
( "courtesans"  n ( k oo2 t i z a1 n z ) ((pos "Kj%" ) ))
( "courtesies"  n ( k @@1 t @ s i z ) ((pos "Mj%" ) ))
( "courtesy"  n ( k @@1 t @ s ii ) ((pos "M8%" ) ))
( "courtier"  n ( k oo1 t i@ r ) ((pos "K6%" ) ))
( "courtiers"  n ( k oo1 t i@ z ) ((pos "Kj%" ) ))
( "courting"  v ( k oo1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "courtlier"  j ( k oo1 t l i@ r ) ((pos "Or$" ) ))
( "courtliest"  j ( k oo1 t l i i s t ) ((pos "Os$" ) ))
( "courtliness"  n ( k oo1 t l i n @ s ) ((pos "L@%" ) ))
( "courtly"  j ( k oo1 t l ii ) ((pos "OD%" ) ))
( "courtroom"  n ( k oo1 t r uu m ) ((pos "K6%" ) ))
( "courtrooms"  n ( k oo1 t r uu m z ) ((pos "Kj%" ) ))
( "courts"  v ( k oo1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "courts-martial"  n ( k oo2 t s - m aa1 sh l ) ((pos "Kj$" ) ))
( "courtship"  n ( k oo1 t - sh i p ) ((pos "M6%" ) ))
( "courtships"  n ( k oo1 t - sh i p s ) ((pos "Mj%" ) ))
( "courtyard"  n ( k oo1 t y aa d ) ((pos "K6%" ) ))
( "courtyards"  n ( k oo1 t y aa d z ) ((pos "Kj%" ) ))
( "cousin"  n ( k uh1 z n ) ((pos "K6%" ) ))
( "cousinly"  j ( k uh1 z n l ii ) ((pos "OA$" ) ))
( "cousins"  n ( k uh1 z n z ) ((pos "Kj%" ) ))
( "cove"  n ( k ou1 v ) ((pos "K6%" ) ))
( "coven"  n ( k uh1 v n ) ((pos "K6%" ) ))
( "covenant"  v ( k uh1 v @ n @ n t ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" "7A" "9" )))
( "covenanted"  v ( k uh1 v @ n @ n t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "9" )))
( "covenanting"  v ( k uh1 v @ n @ n t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "7A" "9" )))
( "covenants"  v ( k uh1 v @ n @ n t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "7A" "9" )))
( "covens"  n ( k uh1 v n z ) ((pos "Kj%" ) ))
( "cover"  v ( k uh1 v @ r ) ((pos "H0%" "M6%" ) (subcat "6A" "15A" )))
( "cover-up"  n ( k uh1 v @ r - uh p ) ((pos "K6%" ) ))
( "cover-ups"  n ( k uh1 v @ r - uh p s ) ((pos "Kj%" ) ))
( "coverage"  n ( k uh1 v @ r i jh ) ((pos "L@%" ) ))
( "covered"  v ( k uh1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "covering"  v ( k uh1 v @ r i ng ) ((pos "Hb%" "K6%" "OA%" ) (subcat "6A" "15A" )))
( "coverings"  n ( k uh1 v @ r i ng z ) ((pos "Kj%" ) ))
( "coverlet"  n ( k uh1 v @ l i t ) ((pos "K6%" ) ))
( "coverlets"  n ( k uh1 v @ l i t s ) ((pos "Kj%" ) ))
( "covers"  v ( k uh1 v @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "covert"  n ( k uh1 v @ t ) ((pos "K6%" "OA%" ) ))
( "covertly"  a ( k uh1 v @ t l ii ) ((pos "Pu%" ) ))
( "coverts"  n ( k uh1 v @ t s ) ((pos "Kj%" ) ))
( "coves"  n ( k ou1 v z ) ((pos "Kj%" ) ))
( "covet"  v ( k uh1 v i t ) ((pos "H0%" ) (subcat )))
( "coveted"  v ( k uh1 v i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "coveting"  v ( k uh1 v i t i ng ) ((pos "Hb%" ) (subcat )))
( "covetous"  j ( k uh1 v i t @ s ) ((pos "OA%" ) ))
( "covetously"  a ( k uh1 v i t @ s l ii ) ((pos "Pu%" ) ))
( "covetousness"  n ( k uh1 v i t @ s n @ s ) ((pos "L@%" ) ))
( "covets"  v ( k uh1 v i t s ) ((pos "Ha%" ) (subcat )))
( "covey"  n ( k uh1 v ii ) ((pos "K6%" ) ))
( "coveys"  n ( k uh1 v i z ) ((pos "Kj%" ) ))
( "cow"  v ( k au1 ) ((pos "H0%" "K6%" ) (subcat )))
( "coward"  n ( k au1 @ d ) ((pos "K6%" ) ))
( "cowardice"  n ( k au1 @ d i s ) ((pos "L@%" ) ))
( "cowardly"  j ( k au1 @ d l ii ) ((pos "OA%" ) ))
( "cowards"  n ( k au1 @ d z ) ((pos "Kj%" ) ))
( "cowbell"  n ( k au1 b e l ) ((pos "K6%" ) ))
( "cowbells"  n ( k au1 b e l z ) ((pos "Kj%" ) ))
( "cowboy"  n ( k au1 b oi ) ((pos "K6%" ) ))
( "cowboys"  n ( k au1 b oi z ) ((pos "Kj%" ) ))
( "cowcatcher"  n ( k au1 k a ch @ r ) ((pos "K6$" ) ))
( "cowcatchers"  n ( k au1 k a ch @ z ) ((pos "Kj$" ) ))
( "cowed"  v ( k au1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cower"  v ( k au1 @ r ) ((pos "I0%" ) (subcat "2A" )))
( "cowered"  v ( k au1 @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "cowering"  v ( k au1 @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "cowers"  v ( k au1 @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "cowhand"  n ( k au1 h a n d ) ((pos "K6%" ) ))
( "cowhands"  n ( k au1 h a n d z ) ((pos "Kj%" ) ))
( "cowherd"  n ( k au1 h @@ d ) ((pos "K6%" ) ))
( "cowherds"  n ( k au1 h @@ d z ) ((pos "Kj%" ) ))
( "cowhide"  n ( k au1 h ai d ) ((pos "M6%" ) ))
( "cowhides"  n ( k au1 h ai d z ) ((pos "Kj$" ) ))
( "cowhouse"  n ( k au1 h au s ) ((pos "K6$" ) ))
( "cowhouses"  n ( k au1 h au z i z ) ((pos "Kj$" ) ))
( "cowing"  v ( k au1 i ng ) ((pos "Hb%" ) (subcat )))
( "cowl"  n ( k au1 l ) ((pos "K6%" ) ))
( "cowling"  n ( k au1 l i ng ) ((pos "K6%" ) ))
( "cowlings"  n ( k au1 l i ng z ) ((pos "Kj$" ) ))
( "cowls"  n ( k au1 l z ) ((pos "Kj%" ) ))
( "cowman"  n ( k au1 m a n ) ((pos "Ki%" ) ))
( "cowmen"  n ( k au1 m e n ) ((pos "Kj%" ) ))
( "cowpox"  n ( k au1 p o k s ) ((pos "K7%" ) ))
( "cowpoxes"  n ( k au1 p o k s i z ) ((pos "Kj%" ) ))
( "cowrie"  n ( k au1 r ii ) ((pos "K6%" ) ))
( "cowries"  n ( k au1 r i z ) ((pos "Kj%" ) ))
( "cows"  v ( k au1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cowshed"  n ( k au1 sh e d ) ((pos "K6%" ) ))
( "cowsheds"  n ( k au1 sh e d z ) ((pos "Kj%" ) ))
( "cowskin"  n ( k au1 s k i n ) ((pos "K6$" ) ))
( "cowskins"  n ( k au1 s k i n z ) ((pos "Kj$" ) ))
( "cowslip"  n ( k au1 s l i p ) ((pos "K6%" ) ))
( "cowslips"  n ( k au1 s l i p s ) ((pos "Kj%" ) ))
( "cox"  v ( k o1 k s ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "coxcomb"  n ( k o1 k s k ou m ) ((pos "K6%" ) ))
( "coxcombs"  n ( k o1 k s k ou m z ) ((pos "Kj%" ) ))
( "coxed"  v ( k o1 k s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "coxes"  v ( k o1 k s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "coxing"  v ( k o1 k s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "coxswain"  n ( k o1 k s n ) ((pos "K6$" ) ))
( "coxswains"  n ( k o1 k s n z ) ((pos "Kj$" ) ))
( "coy"  j ( k oi1 ) ((pos "OC%" ) ))
( "coyer"  j ( k oi1 @ r ) ((pos "Or%" ) ))
( "coyest"  j ( k oi1 i s t ) ((pos "Os%" ) ))
( "coyly"  a ( k oi1 l ii ) ((pos "Pu%" ) ))
( "coyness"  n ( k oi1 n @ s ) ((pos "L@%" ) ))
( "coyote"  n ( k oi1 ou1 t ) ((pos "K6%" ) ))
( "coyotes"  n ( k oi1 ou1 t s ) ((pos "Kj%" ) ))
( "coypu"  n ( k oi1 p uu ) ((pos "K6%" ) ))
( "coypus"  n ( k oi1 p uu z ) ((pos "Kj%" ) ))
( "cozen"  v ( k uh1 z n ) ((pos "H0$" ) (subcat "6A" )))
( "cozened"  v ( k uh1 z n d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "cozening"  v ( k uh1 z n i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "cozens"  v ( k uh1 z n z ) ((pos "Ha$" ) (subcat "6A" )))
( "cozier"  j ( k ou1 z i@ r ) ((pos "Or%" ) ))
( "coziest"  j ( k ou1 z i i s t ) ((pos "Os%" ) ))
( "cozy"  j ( k ou1 z ii ) ((pos "OD%" ) ))
( "cp"  n ( s ii2 p ii1 ) ((pos "Y~$" ) ))
( "cr^epe"  n ( k r ei1 p ) ((pos "L@%" ) ))
( "cr`eche"  n ( k r ei1 sh ) ((pos "K6%" ) ))
( "cr`eches"  n ( k r ei1 sh i z ) ((pos "Kj%" ) ))
( "cr`eme de menthe"  n ( k r e2 m - d @ - m o1 n th ) ((pos "L@%" ) ))
( "crab"  v ( k r a1 b ) ((pos "J4%" "M6%" ) ))
( "crab-apple"  n ( k r a1 b - a p l ) ((pos "K6%" ) ))
( "crab-apples"  n ( k r a1 b - a p l z ) ((pos "Kj%" ) ))
( "crabbed"  j ( k r a1 b i d ) ((pos "OA$" ) ))
( "crabbed"  v ( k r a1 b d ) ((pos "Jc$" "Jd$" ) ))
( "crabbing"  v ( k r a1 b i ng ) ((pos "Jb%" ) ))
( "crabs"  v ( k r a1 b z ) ((pos "Ja$" "Mj%" ) ))
( "crack"  v ( k r a1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "crack-down"  n ( k r a1 k - d au n ) ((pos "K6%" ) ))
( "crack-downs"  n ( k r a1 k - d au n z ) ((pos "Kj%" ) ))
( "crack-up"  n ( k r a1 k - uh p ) ((pos "K6%" ) ))
( "crack-ups"  n ( k r a1 k - uh p s ) ((pos "Kj%" ) ))
( "cracked"  v ( k r a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cracker"  n ( k r a1 k @ r ) ((pos "K6%" ) ))
( "crackers"  n ( k r a1 k @ z ) ((pos "Kj%" "Op%" ) ))
( "cracking"  v ( k r a1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "crackle"  v ( k r a1 k l ) ((pos "I2%" "L@%" ) (subcat "2A" )))
( "crackle-china"  n ( k r a1 k l - ch ai n @ ) ((pos "L@$" ) ))
( "crackled"  v ( k r a1 k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "crackles"  v ( k r a1 k l z ) ((pos "Ia%" ) (subcat "2A" )))
( "crackleware"  n ( k r a1 k l w e@ r ) ((pos "L@$" ) ))
( "crackling"  v ( k r a1 k l i ng ) ((pos "Ib%" "L@%" ) (subcat "2A" )))
( "crackpot"  n ( k r a1 k p o t ) ((pos "K6%" ) ))
( "crackpots"  n ( k r a1 k p o t s ) ((pos "Kj%" ) ))
( "cracks"  v ( k r a1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "cracksman"  n ( k r a1 k s m @ n ) ((pos "Ki%" ) ))
( "cracksmen"  n ( k r a1 k s m @ n ) ((pos "Kj%" ) ))
( "cradle"  v ( k r ei1 d l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "cradled"  v ( k r ei1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cradles"  v ( k r ei1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "cradling"  v ( k r ei1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "craft"  n ( k r aa1 f t ) ((pos "M6%" ) ))
( "craftier"  j ( k r aa1 f t i@ r ) ((pos "Or%" ) ))
( "craftiest"  j ( k r aa1 f t i i s t ) ((pos "Os%" ) ))
( "craftily"  a ( k r aa1 f t i l ii ) ((pos "Pu%" ) ))
( "craftiness"  n ( k r aa1 f t i n @ s ) ((pos "L@%" ) ))
( "crafts"  n ( k r aa1 f t s ) ((pos "Mj%" ) ))
( "craftsman"  n ( k r aa1 f t s m @ n ) ((pos "Ki%" ) ))
( "craftsmanship"  n ( k r aa1 f t m @ n sh i p ) ((pos "L@%" ) ))
( "craftsmen"  n ( k r aa1 f t s m @ n ) ((pos "Kj%" ) ))
( "crafty"  j ( k r aa1 f t ii ) ((pos "OD%" ) ))
( "crag"  n ( k r a1 g ) ((pos "K6%" ) ))
( "cragged"  j ( k r a1 g i d ) ((pos "OA$" ) ))
( "craggier"  j ( k r a1 g i@ r ) ((pos "Or$" ) ))
( "craggiest"  j ( k r a1 g i i s t ) ((pos "Os$" ) ))
( "craggy"  j ( k r a1 g ii ) ((pos "OD%" ) ))
( "crags"  n ( k r a1 g z ) ((pos "Kj%" ) ))
( "cragsman"  n ( k r a1 g z m @ n ) ((pos "Ki%" ) ))
( "cragsmen"  n ( k r a1 g z m @ n ) ((pos "Kj%" ) ))
( "crake"  n ( k r ei1 k ) ((pos "K6$" ) ))
( "crakes"  n ( k r ei1 k s ) ((pos "Kj$" ) ))
( "cram"  v ( k r a1 m ) ((pos "J4%" ) (subcat "2A" "6A" "14" )))
( "cram-full"  j ( k r a1 m - f u1 l ) ((pos "OA%" "Pu$" ) ))
( "crammed"  v ( k r a1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "crammer"  n ( k r a1 m @ r ) ((pos "K6%" ) ))
( "crammers"  n ( k r a1 m @ z ) ((pos "Kj%" ) ))
( "cramming"  v ( k r a1 m i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "cramp"  v ( k r a1 m p ) ((pos "H0%" "M6%" ) (subcat )))
( "cramp-iron"  n ( k r a1 m p - ai @ n ) ((pos "K6$" ) ))
( "cramp-irons"  n ( k r a1 m p - ai @ n z ) ((pos "Kj$" ) ))
( "cramped"  v ( k r a1 m p t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "cramping"  v ( k r a1 m p i ng ) ((pos "Hb%" ) (subcat )))
( "crampon"  n ( k r a1 m p o n ) ((pos "K6%" ) ))
( "crampons"  n ( k r a1 m p o n z ) ((pos "Kj%" ) ))
( "cramps"  v ( k r a1 m p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "crams"  v ( k r a1 m z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "cranberries"  n ( k r a1 n b @ r i z ) ((pos "Kj%" ) ))
( "cranberry"  n ( k r a1 n b @ r ii ) ((pos "K8%" ) ))
( "crane"  v ( k r ei1 n ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "crane-flies"  n ( k r ei1 n - f l ai z ) ((pos "Kj$" ) ))
( "crane-fly"  n ( k r ei1 n - f l ai ) ((pos "K8$" ) ))
( "craned"  v ( k r ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "cranes"  v ( k r ei1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "cranial"  j ( k r ei1 n i@ l ) ((pos "OA%" ) ))
( "craning"  v ( k r ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "cranium"  n ( k r ei1 n i@ m ) ((pos "K6%" ) ))
( "craniums"  n ( k r ei1 n i@ m z ) ((pos "Kj$" ) ))
( "crank"  v ( k r a1 ng k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "cranked"  v ( k r a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "crankier"  j ( k r a1 ng k i@ r ) ((pos "Or%" ) ))
( "crankiest"  j ( k r a1 ng k i i s t ) ((pos "Os%" ) ))
( "cranking"  v ( k r a1 ng k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cranks"  v ( k r a1 ng k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "crankshaft"  n ( k r a1 n k sh aa f t ) ((pos "K6%" ) ))
( "crankshafts"  n ( k r a1 n k sh aa f t s ) ((pos "Kj%" ) ))
( "cranky"  j ( k r a1 ng k ii ) ((pos "OD%" ) ))
( "crannied"  j ( k r a1 n i d ) ((pos "OA$" ) ))
( "crannies"  n ( k r a1 n i z ) ((pos "Kj%" ) ))
( "cranny"  n ( k r a1 n ii ) ((pos "K8%" ) ))
( "crap"  v ( k r a1 p ) ((pos "I4%" "M6%" ) ))
( "crap-shooting"  n ( k r a1 p - sh uu t i ng ) ((pos "K6%" ) ))
( "crap-shootings"  n ( k r a1 p - sh uu t i ng z ) ((pos "Kj$" ) ))
( "crape"  n ( k r ei1 p ) ((pos "L@$" ) ))
( "crapped"  v ( k r a1 p t ) ((pos "Ic%" "Id%" ) ))
( "crapping"  v ( k r a1 p i ng ) ((pos "Ib%" ) ))
( "craps"  v ( k r a1 p s ) ((pos "Ia%" "Lk%" "Mj%" ) ))
( "crash"  v ( k r a1 sh ) ((pos "J1%" "M7%" "Pu%" ) (subcat "2A" "2C" )))
( "crash-dive"  v ( k r a1 sh - d ai v ) ((pos "I2%" "K6%" ) ))
( "crash-dived"  v ( k r a1 sh - d ai v d ) ((pos "Ic%" "Id%" ) ))
( "crash-dives"  v ( k r a1 sh - d ai v z ) ((pos "Ia%" "Kj%" ) ))
( "crash-diving"  v ( k r a1 sh - d ai v i ng ) ((pos "Ib%" ) ))
( "crash-helmet"  n ( k r a1 sh - h e l m i t ) ((pos "K6%" ) ))
( "crash-helmets"  n ( k r a1 sh - h e l m i t s ) ((pos "Kj%" ) ))
( "crash-land"  v ( k r a1 sh - l a1 n d ) ((pos "J0%" ) ))
( "crash-landed"  v ( k r a1 sh - l a1 n d i d ) ((pos "Jc%" "Jd%" ) ))
( "crash-landing"  v ( k r a1 sh - l a1 n d i ng ) ((pos "Jb%" "K6%" ) ))
( "crash-landings"  n ( k r a1 sh - l a1 n d i ng z ) ((pos "Kj%" ) ))
( "crash-lands"  v ( k r a1 sh - l a1 n d z ) ((pos "Ja%" ) ))
( "crashed"  v ( k r a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "crashes"  v ( k r a1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "crashing"  v ( k r a1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "crass"  j ( k r a1 s ) ((pos "OA%" ) ))
( "crate"  v ( k r ei1 t ) ((pos "H2$" "K6%" ) ))
( "crated"  v ( k r ei1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "crater"  n ( k r ei1 t @ r ) ((pos "K6%" ) ))
( "craters"  n ( k r ei1 t @ z ) ((pos "Kj%" ) ))
( "crates"  v ( k r ei1 t s ) ((pos "Ha$" "Kj%" ) ))
( "crating"  v ( k r ei1 t i ng ) ((pos "Hb$" ) ))
( "cravat"  n ( k r @1 v a1 t ) ((pos "K6%" ) ))
( "cravats"  n ( k r @1 v a1 t s ) ((pos "Kj%" ) ))
( "crave"  v ( k r ei1 v ) ((pos "J2%" ) (subcat "2A" )))
( "craved"  v ( k r ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "craven"  n ( k r ei1 v n ) ((pos "K6$" "OA%" ) ))
( "cravens"  n ( k r ei1 v n z ) ((pos "Kj$" ) ))
( "craves"  v ( k r ei1 v z ) ((pos "Ja%" ) (subcat "2A" )))
( "craving"  v ( k r ei1 v i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" )))
( "cravings"  n ( k r ei1 v i ng z ) ((pos "Kj%" ) ))
( "crawfish"  n ( k r oo1 f i sh ) ((pos "K7$" ) ))
( "crawfishes"  n ( k r oo1 f i sh i z ) ((pos "Kj$" ) ))
( "crawl"  v ( k r oo1 l ) ((pos "I0%" "Ki%" ) (subcat "2A" )))
( "crawled"  v ( k r oo1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "crawler"  n ( k r oo1 l @ r ) ((pos "K6%" ) ))
( "crawlers"  n ( k r oo1 l @ z ) ((pos "Kj%" ) ))
( "crawling"  v ( k r oo1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "crawls"  v ( k r oo1 l z ) ((pos "Ia%" ) (subcat "2A" )))
( "crayfish"  n ( k r ei1 f i sh ) ((pos "K7%" ) ))
( "crayfishes"  n ( k r ei1 f i sh i z ) ((pos "Kj$" ) ))
( "crayon"  v ( k r ei1 @ n ) ((pos "H0%" "K6%" ) ))
( "crayoned"  v ( k r ei1 @ n d ) ((pos "Hc%" "Hd%" ) ))
( "crayoning"  v ( k r ei1 @ n i ng ) ((pos "Hb%" ) ))
( "crayons"  v ( k r ei1 @ n z ) ((pos "Ha%" "Kj%" ) ))
( "craze"  n ( k r ei1 z ) ((pos "K6%" ) ))
( "crazed"  j ( k r ei1 z d ) ((pos "OA%" ) ))
( "crazes"  n ( k r ei1 z i z ) ((pos "Kj%" ) ))
( "crazier"  j ( k r ei1 z i@ r ) ((pos "Or%" ) ))
( "craziest"  j ( k r ei1 z i i s t ) ((pos "Os%" ) ))
( "crazily"  a ( k r ei1 z i l ii ) ((pos "Pu%" ) ))
( "craziness"  n ( k r ei1 z i n @ s ) ((pos "L@%" ) ))
( "crazy"  j ( k r ei1 z ii ) ((pos "OD%" ) ))
( "creak"  v ( k r ii1 k ) ((pos "I0%" "K6%" ) ))
( "creaked"  v ( k r ii1 k t ) ((pos "Ic%" "Id%" ) ))
( "creakier"  j ( k r ii1 k i@ r ) ((pos "Or%" ) ))
( "creakiest"  j ( k r ii1 k i i s t ) ((pos "Os%" ) ))
( "creakily"  a ( k r ii1 k i l ii ) ((pos "Pu%" ) ))
( "creaking"  v ( k r ii1 k i ng ) ((pos "Ib%" ) ))
( "creaks"  v ( k r ii1 k s ) ((pos "Ia%" "Kj%" ) ))
( "creaky"  j ( k r ii1 k ii ) ((pos "OD%" ) ))
( "cream"  v ( k r ii1 m ) ((pos "H0%" "M6%" ) ))
( "creamed"  v ( k r ii1 m d ) ((pos "Hc%" "Hd%" ) ))
( "creameries"  n ( k r ii1 m @ r i z ) ((pos "Kj%" ) ))
( "creamery"  n ( k r ii1 m @ r ii ) ((pos "K8%" ) ))
( "creamier"  j ( k r ii1 m i@ r ) ((pos "Or%" ) ))
( "creamiest"  j ( k r ii1 m i i s t ) ((pos "Os%" ) ))
( "creaming"  v ( k r ii1 m i ng ) ((pos "Hb%" ) ))
( "creams"  v ( k r ii1 m z ) ((pos "Ha%" "Mj%" ) ))
( "creamy"  j ( k r ii1 m ii ) ((pos "OD%" ) ))
( "crease"  v ( k r ii1 s ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "creased"  v ( k r ii1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "creases"  v ( k r ii1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "creasing"  v ( k r ii1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "create"  v ( k r ii1 ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "created"  v ( k r ii1 ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "creates"  v ( k r ii1 ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "creating"  v ( k r ii1 ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "creation"  n ( k r ii1 ei1 sh @ n ) ((pos "M6%" ) ))
( "creations"  n ( k r ii1 ei1 sh @ n z ) ((pos "Mj%" ) ))
( "creative"  j ( k r ii1 ei1 t i v ) ((pos "OA%" ) ))
( "creatively"  a ( k r ii1 ei1 t i v l ii ) ((pos "Pu%" ) ))
( "creativeness"  n ( k r ii1 ei1 t i v n @ s ) ((pos "L@%" ) ))
( "creator"  n ( k r ii1 ei1 t @ r ) ((pos "K6%" ) ))
( "creators"  n ( k r ii1 ei1 t @ z ) ((pos "Kj%" ) ))
( "creature"  n ( k r ii1 ch @ r ) ((pos "K6%" ) ))
( "creatures"  n ( k r ii1 ch @ z ) ((pos "Kj%" ) ))
( "credence"  n ( k r ii1 d n s ) ((pos "L@%" ) ))
( "credentials"  n ( k r i1 d e1 n sh l z ) ((pos "Kj%" ) ))
( "credibility"  n ( k r e2 d i b i1 l i t ii ) ((pos "L@%" ) ))
( "credible"  j ( k r e1 d @ b l ) ((pos "OA%" ) ))
( "credibly"  a ( k r e1 d @ b l ii ) ((pos "Pu%" ) ))
( "credit"  v ( k r e1 d i t ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "credit-side"  n ( k r e1 d i t - s ai d ) ((pos "K6%" ) ))
( "credit-sides"  n ( k r e1 d i t - s ai d z ) ((pos "Kj$" ) ))
( "credit-worthiness"  n ( k r e1 d i t - w @@ dh i n @ s ) ((pos "L@%" ) ))
( "credit-worthy"  j ( k r e1 d i t - w @@ dh ii ) ((pos "OA%" ) ))
( "creditable"  j ( k r e1 d i t @ b l ) ((pos "OA%" ) ))
( "creditably"  a ( k r e1 d i t @ b l ii ) ((pos "Pu%" ) ))
( "credited"  v ( k r e1 d i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "crediting"  v ( k r e1 d i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "creditor"  n ( k r e1 d i t @ r ) ((pos "K6%" ) ))
( "creditors"  n ( k r e1 d i t @ z ) ((pos "Kj%" ) ))
( "credits"  v ( k r e1 d i t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "credo"  n ( k r ii1 d ou ) ((pos "K6%" ) ))
( "credos"  n ( k r ii1 d ou z ) ((pos "Kj%" ) ))
( "credulities"  n ( k r i1 d y uu1 l i t i z ) ((pos "Mj$" ) ))
( "credulity"  n ( k r i1 d y uu1 l i t ii ) ((pos "M8%" ) ))
( "credulous"  j ( k r e1 d y u l @ s ) ((pos "OA%" ) ))
( "credulously"  a ( k r e1 d y u l @ s l ii ) ((pos "Pu%" ) ))
( "creed"  n ( k r ii1 d ) ((pos "K6%" ) ))
( "creeds"  n ( k r ii1 d z ) ((pos "Kj%" ) ))
( "creek"  n ( k r ii1 k ) ((pos "K6%" ) ))
( "creeks"  n ( k r ii1 k s ) ((pos "Kj%" ) ))
( "creel"  n ( k r ii1 l ) ((pos "K6$" ) ))
( "creels"  n ( k r ii1 l z ) ((pos "Kj$" ) ))
( "creep"  v ( k r ii1 p ) ((pos "I5%" "K6%" ) (subcat "2A" "2B" )))
( "creeper"  n ( k r ii1 p @ r ) ((pos "K6%" ) ))
( "creepers"  n ( k r ii1 p @ z ) ((pos "Kj%" ) ))
( "creepier"  j ( k r ii1 p i@ r ) ((pos "Or%" ) ))
( "creepiest"  j ( k r ii1 p i i s t ) ((pos "Os%" ) ))
( "creeping"  v ( k r ii1 p i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "creeps"  v ( k r ii1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" )))
( "creepy"  j ( k r ii1 p ii ) ((pos "OD%" ) ))
( "cremate"  v ( k r i1 m ei1 t ) ((pos "H2%" ) (subcat )))
( "cremated"  v ( k r i1 m ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cremates"  v ( k r i1 m ei1 t s ) ((pos "Ha%" ) (subcat )))
( "cremating"  v ( k r i1 m ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "cremation"  n ( k r i1 m ei1 sh @ n ) ((pos "M6%" ) ))
( "cremations"  n ( k r i1 m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "crematoria"  n ( k r e2 m @ t oo1 r i@ ) ((pos "Kj%" ) ))
( "crematories"  n ( k r e1 m @ t @ r i z ) ((pos "Kj$" ) ))
( "crematorium"  n ( k r e2 m @ t oo1 r i@ m ) ((pos "K6%" ) ))
( "crematoriums"  n ( k r e2 m @ t oo1 r i@ m z ) ((pos "Kj%" ) ))
( "crematory"  n ( k r e1 m @ t @ r ii ) ((pos "K8$" ) ))
( "crenellated"  j ( k r e1 n @ l ei t i d ) ((pos "OA$" ) ))
( "creosote"  n ( k r i@1 s ou t ) ((pos "L@%" ) ))
( "crepe"  n ( k r ei1 p ) ((pos "L@%" ) ))
( "crepitate"  v ( k r e1 p i t ei t ) ((pos "I2$" ) (subcat )))
( "crepitated"  v ( k r e1 p i t ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "crepitates"  v ( k r e1 p i t ei t s ) ((pos "Ia$" ) (subcat )))
( "crepitating"  v ( k r e1 p i t ei t i ng ) ((pos "Ib$" ) (subcat )))
( "crepitation"  n ( k r e2 p i t ei1 sh @ n ) ((pos "K6$" ) ))
( "crepitations"  n ( k r e2 p i t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "crept"  v ( k r e1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "crepuscular"  j ( k r i1 p uh1 s k y u l @ r ) ((pos "OA$" ) ))
( "crescendo"  n ( k r i1 sh e1 n d ou ) ((pos "K6%" ) ))
( "crescendos"  n ( k r i1 sh e1 n d ou z ) ((pos "Kj%" ) ))
( "crescent"  n ( k r e1 s n t ) ((pos "K6%" ) ))
( "crescents"  n ( k r e1 s n t s ) ((pos "Kj%" ) ))
( "cress"  n ( k r e1 s ) ((pos "L@%" ) ))
( "crest"  v ( k r e1 s t ) ((pos "J0%" "K6%" ) ))
( "crested"  v ( k r e1 s t i d ) ((pos "Jc%" "Jd%" "OA%" ) ))
( "crestfallen"  j ( k r e1 s t f oo l @ n ) ((pos "OA%" ) ))
( "cresting"  v ( k r e1 s t i ng ) ((pos "Jb%" ) ))
( "crests"  v ( k r e1 s t s ) ((pos "Ja%" "Kj%" ) ))
( "cretaceous"  j ( k r i1 t ei1 sh @ s ) ((pos "OA$" ) ))
( "cretin"  n ( k r e1 t i n ) ((pos "K6%" ) ))
( "cretinous"  j ( k r e1 t i n @ s ) ((pos "OA%" ) ))
( "cretins"  n ( k r e1 t i n z ) ((pos "Kj%" ) ))
( "cretonne"  n ( k r e1 t o n ) ((pos "L@$" ) ))
( "crevasse"  n ( k r i1 v a1 s ) ((pos "K6%" ) ))
( "crevasses"  n ( k r i1 v a1 s i z ) ((pos "Kj%" ) ))
( "crevice"  n ( k r e1 v i s ) ((pos "K6%" ) ))
( "crevices"  n ( k r e1 v i s i z ) ((pos "Kj%" ) ))
( "crew"  v ( k r uu1 ) ((pos "I0%" "Ic%" "K6%" ) (subcat "3A" )))
( "crew-cut"  n ( k r uu1 - k uh t ) ((pos "K6%" ) ))
( "crew-cuts"  n ( k r uu1 - k uh t s ) ((pos "Kj%" ) ))
( "crew-neck"  n ( k r uu1 - n e k ) ((pos "K6$" ) ))
( "crew-necks"  n ( k r uu1 - n e k s ) ((pos "Kj$" ) ))
( "crewed"  v ( k r uu1 d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "crewing"  v ( k r uu1 i ng ) ((pos "Ib%" ) (subcat "3A" )))
( "crews"  v ( k r uu1 z ) ((pos "Ia%" "Kj%" ) (subcat "3A" )))
( "crib"  v ( k r i1 b ) ((pos "J4%" "K6%" ) ))
( "cribbage"  n ( k r i1 b i jh ) ((pos "L@%" ) ))
( "cribbage-board"  n ( k r i1 b i jh - b oo d ) ((pos "K6%" ) ))
( "cribbage-boards"  n ( k r i1 b i jh - b oo d z ) ((pos "Kj%" ) ))
( "cribbed"  v ( k r i1 b d ) ((pos "Jc%" "Jd%" ) ))
( "cribbing"  v ( k r i1 b i ng ) ((pos "Jb%" ) ))
( "cribs"  v ( k r i1 b z ) ((pos "Ja%" "Kj%" ) ))
( "crick"  v ( k r i1 k ) ((pos "H0%" "K6%" ) ))
( "cricked"  v ( k r i1 k t ) ((pos "Hc%" "Hd%" ) ))
( "cricket"  n ( k r i1 k i t ) ((pos "M6%" ) ))
( "cricketer"  n ( k r i1 k i t @ r ) ((pos "K6%" ) ))
( "cricketers"  n ( k r i1 k i t @ z ) ((pos "Kj%" ) ))
( "crickets"  n ( k r i1 k i t s ) ((pos "Mj%" ) ))
( "cricking"  v ( k r i1 k i ng ) ((pos "Hb%" ) ))
( "cricks"  v ( k r i1 k s ) ((pos "Ha%" "Kj%" ) ))
( "cried"  v ( k r ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "9" "14" "15A" )))
( "crier"  n ( k r ai1 @ r ) ((pos "K6%" ) ))
( "criers"  n ( k r ai1 @ z ) ((pos "Kj%" ) ))
( "cries"  v ( k r ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "9" "14" "15A" )))
( "crikey"  uh ( k r ai1 k ii ) ((pos "W-%" ) ))
( "crime"  v ( k r ai1 m ) ((pos "H2$" "M6%" ) ))
( "crimed"  v ( k r ai1 m d ) ((pos "Hc$" "Hd$" ) ))
( "crimes"  v ( k r ai1 m z ) ((pos "Ha$" "Mj%" ) ))
( "criminal"  n ( k r i1 m i n l ) ((pos "K6%" "OA%" ) ))
( "criminally"  a ( k r i1 m i n @ l ii ) ((pos "Pu%" ) ))
( "criminals"  n ( k r i1 m i n l z ) ((pos "Kj%" ) ))
( "criming"  v ( k r ai1 m i ng ) ((pos "Hb$" ) ))
( "criminology"  n ( k r i2 m i n o1 l @ jh ii ) ((pos "L@%" ) ))
( "crimp"  v ( k r i1 m p ) ((pos "H0%" ) ))
( "crimped"  v ( k r i1 m p t ) ((pos "Hc%" "Hd%" ) ))
( "crimping"  v ( k r i1 m p i ng ) ((pos "Hb%" ) ))
( "crimps"  v ( k r i1 m p s ) ((pos "Ha%" ) ))
( "crimson"  v ( k r i1 m z n ) ((pos "J0%" "M6%" "OA%" ) ))
( "crimsoned"  v ( k r i1 m z n d ) ((pos "Jc%" "Jd%" ) ))
( "crimsoning"  v ( k r i1 m z n i ng ) ((pos "Jb%" ) ))
( "crimsons"  v ( k r i1 m z n z ) ((pos "Ja%" "Mj%" ) ))
( "cringe"  v ( k r i1 n jh ) ((pos "I2%" ) (subcat "2A" )))
( "cringed"  v ( k r i1 n jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "cringes"  v ( k r i1 n jh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "cringing"  v ( k r i1 n jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "crinkle"  v ( k r i1 ng k l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "crinkled"  v ( k r i1 ng k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "crinkles"  v ( k r i1 ng k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "crinklier"  j ( k r i1 ng k l i@ r ) ((pos "Or%" ) ))
( "crinkliest"  j ( k r i1 ng k l i i s t ) ((pos "Os%" ) ))
( "crinkling"  v ( k r i1 ng k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "crinkly"  j ( k r i1 ng k l ii ) ((pos "OD%" ) ))
( "crinoline"  n ( k r i1 n @ l i n ) ((pos "M6%" ) ))
( "crinolines"  n ( k r i1 n @ l i n z ) ((pos "Mj%" ) ))
( "cripes"  uh ( k r ai1 p s ) ((pos "W-$" ) ))
( "cripple"  v ( k r i1 p l ) ((pos "H2%" "K6%" ) (subcat )))
( "crippled"  v ( k r i1 p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cripples"  v ( k r i1 p l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "crippling"  v ( k r i1 p l i ng ) ((pos "Hb%" ) (subcat )))
( "crises"  n ( k r ai1 s ii z ) ((pos "Kj%" ) ))
( "crisis"  n ( k r ai1 s i s ) ((pos "Ki%" ) ))
( "crisp"  v ( k r i1 s p ) ((pos "J0%" "K6%" "OC%" ) ))
( "crisped"  v ( k r i1 s p t ) ((pos "Jc%" "Jd%" ) ))
( "crisper"  j ( k r i1 s p @ r ) ((pos "Or%" ) ))
( "crispest"  j ( k r i1 s p i s t ) ((pos "Os%" ) ))
( "crisping"  v ( k r i1 s p i ng ) ((pos "Jb%" ) ))
( "crisply"  a ( k r i1 s p l ii ) ((pos "Pu%" ) ))
( "crispness"  n ( k r i1 s p n @ s ) ((pos "L@%" ) ))
( "crisps"  v ( k r i1 s p s ) ((pos "Ja%" "Kj%" ) ))
( "crisscross"  v ( k r i1 s k r o s ) ((pos "J1%" "OA%" "Pu%" ) ))
( "crisscrossed"  v ( k r i1 s k r o s t ) ((pos "Jc%" "Jd%" ) ))
( "crisscrosses"  v ( k r i1 s k r o s i z ) ((pos "Ja%" ) ))
( "crisscrossing"  v ( k r i1 s k r o s i ng ) ((pos "Jb%" ) ))
( "criteria"  n ( k r ai1 t i@1 r i@ ) ((pos "Kj%" ) ))
( "criterion"  n ( k r ai1 t i@1 r i@ n ) ((pos "K6%" ) ))
( "criterions"  n ( k r ai1 t i@1 r i@ n z ) ((pos "Kj$" ) ))
( "critic"  n ( k r i1 t i k ) ((pos "K6%" ) ))
( "critical"  j ( k r i1 t i k l ) ((pos "OA%" ) ))
( "critically"  a ( k r i1 t i k l ii ) ((pos "Pu%" ) ))
( "criticism"  n ( k r i1 t i s i z @ m ) ((pos "M6%" ) ))
( "criticisms"  n ( k r i1 t i s i z @ m z ) ((pos "Mj%" ) ))
( "criticize"  v ( k r i1 t i s ai z ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "criticized"  v ( k r i1 t i s ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "criticizes"  v ( k r i1 t i s ai z i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "criticizing"  v ( k r i1 t i s ai z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "critics"  n ( k r i1 t i k s ) ((pos "Kj%" ) ))
( "critique"  n ( k r i1 t ii1 k ) ((pos "K6%" ) ))
( "critiques"  n ( k r i1 t ii1 k s ) ((pos "Kj%" ) ))
( "croak"  v ( k r ou1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "croaked"  v ( k r ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "croaking"  v ( k r ou1 k i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "croaks"  v ( k r ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "crochet"  v ( k r ou1 sh ei ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "crochet-hook"  n ( k r ou1 sh ei - h u k ) ((pos "K6%" ) ))
( "crochet-hooks"  n ( k r ou1 sh ei - h u k s ) ((pos "Kj%" ) ))
( "crocheted"  v ( k r ou1 sh ei d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "crocheting"  v ( k r ou1 sh ei i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "crochets"  v ( k r ou1 sh ei z ) ((pos "Ja%" ) (subcat "2A" )))
( "crock"  v ( k r o1 k ) ((pos "J0$" "K6%" ) (subcat "2C" )))
( "crocked"  v ( k r o1 k t ) ((pos "Jc$" "Jd$" ) (subcat "2C" )))
( "crockery"  n ( k r o1 k @ r ii ) ((pos "L@%" ) ))
( "crocking"  v ( k r o1 k i ng ) ((pos "Jb$" ) (subcat "2C" )))
( "crocks"  v ( k r o1 k s ) ((pos "Ja$" "Kj%" ) (subcat "2C" )))
( "crocodile"  n ( k r o1 k @ d ai l ) ((pos "K6%" ) ))
( "crocodiles"  n ( k r o1 k @ d ai l z ) ((pos "Kj%" ) ))
( "crocus"  n ( k r ou1 k @ s ) ((pos "K7%" ) ))
( "crocuses"  n ( k r ou1 k @ s i z ) ((pos "Kj%" ) ))
( "croft"  n ( k r o1 f t ) ((pos "K6%" ) ))
( "crofter"  n ( k r o1 f t @ r ) ((pos "K6%" ) ))
( "crofters"  n ( k r o1 f t @ z ) ((pos "Kj%" ) ))
( "crofts"  n ( k r o1 f t s ) ((pos "Kj%" ) ))
( "cromlech"  n ( k r o1 m l e k ) ((pos "K6$" ) ))
( "cromlechs"  n ( k r o1 m l e k s ) ((pos "Kj$" ) ))
( "crone"  n ( k r ou1 n ) ((pos "K6%" ) ))
( "crones"  n ( k r ou1 n z ) ((pos "Kj%" ) ))
( "cronies"  n ( k r ou1 n i z ) ((pos "Kj%" ) ))
( "crony"  n ( k r ou1 n ii ) ((pos "K8%" ) ))
( "crook"  v ( k r u1 k ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "crook-back"  j ( k r u1 k - b a k ) ((pos "OA$" ) ))
( "crook-backed"  j ( k r u1 k - b a k t ) ((pos "OA$" ) ))
( "crooked"  j ( k r u1 k i d ) ((pos "OA%" ) ))
( "crooked"  v ( k r u1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "crookedly"  a ( k r u1 k i d l ii ) ((pos "Pu%" ) ))
( "crookedness"  n ( k r u1 k i d n @ s ) ((pos "L@%" ) ))
( "crooking"  v ( k r u1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "crooks"  v ( k r u1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "croon"  v ( k r uu1 n ) ((pos "J0%" ) (subcat "2C" "6A" "13A" )))
( "crooned"  v ( k r uu1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "13A" )))
( "crooner"  n ( k r uu1 n @ r ) ((pos "K6%" ) ))
( "crooners"  n ( k r uu1 n @ z ) ((pos "Kj%" ) ))
( "crooning"  v ( k r uu1 n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "13A" )))
( "croons"  v ( k r uu1 n z ) ((pos "Ja%" ) (subcat "2C" "6A" "13A" )))
( "crop"  v ( k r o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "crop-dusting"  n ( k r o1 p - d uh s t i ng ) ((pos "L@%" ) ))
( "cropped"  v ( k r o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "cropper"  n ( k r o1 p @ r ) ((pos "K6%" ) ))
( "croppers"  n ( k r o1 p @ z ) ((pos "Kj$" ) ))
( "cropping"  v ( k r o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "crops"  v ( k r o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "croquet"  n ( k r ou1 k ei ) ((pos "L@%" ) ))
( "croquette"  n ( k r ou1 k e1 t ) ((pos "K6%" ) ))
( "croquettes"  n ( k r ou1 k e1 t s ) ((pos "Kj%" ) ))
( "crore"  n ( k r oo1 r ) ((pos "K6$" ) ))
( "crores"  n ( k r oo1 z ) ((pos "Kj$" ) ))
( "crosier"  n ( k r ou1 z i@ r ) ((pos "K6$" ) ))
( "crosiers"  n ( k r ou1 z i@ z ) ((pos "Kj$" ) ))
( "cross"  v ( k r o1 s ) ((pos "J1%" "K7%" "OA%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "cross-bench"  n ( k r o1 s - b e n ch ) ((pos "K7%" ) ))
( "cross-bencher"  n ( k r o1 s - b e n ch @ r ) ((pos "K6%" ) ))
( "cross-benchers"  n ( k r o1 s - b e n ch @ z ) ((pos "Kj%" ) ))
( "cross-benches"  n ( k r o1 s - b e n ch i z ) ((pos "Kj%" ) ))
( "cross-division"  n ( k r o2 s - d i v i1 z n ) ((pos "M6$" ) ))
( "cross-divisions"  n ( k r o2 s - d i v i1 z n z ) ((pos "Mj$" ) ))
( "cross-examination"  n ( k r o2 s - i g z a2 m i n ei1 sh @ n ) ((pos "K6%" ) ))
( "cross-examinations"  n ( k r o2 s - i g z a2 m i n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "cross-examine"  v ( k r o2 s - i g z a1 m i n ) ((pos "H2%" ) (subcat )))
( "cross-examined"  v ( k r o2 s - i g z a1 m i n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cross-examiner"  n ( k r o2 s - i g z a1 m i n @ r ) ((pos "K6%" ) ))
( "cross-examiners"  n ( k r o2 s - i g z a1 m i n @ z ) ((pos "Kj%" ) ))
( "cross-examines"  v ( k r o2 s - i g z a1 m i n z ) ((pos "Ha%" ) (subcat )))
( "cross-examining"  v ( k r o2 s - i g z a1 m i n i ng ) ((pos "Hb%" ) (subcat )))
( "cross-fertilization"  n ( k r o2 s - f @@2 t @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "cross-fertilizations"  n ( k r o2 s - f @@2 t @ l ai z ei1 sh @ n z ) ((pos "Kj%" ) ))
( "cross-fertilize"  v ( k r o2 s - f @@1 t @ l ai z ) ((pos "H2%" ) ))
( "cross-fertilized"  v ( k r o2 s - f @@1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "cross-fertilizes"  v ( k r o2 s - f @@1 t @ l ai z i z ) ((pos "Ha%" ) ))
( "cross-fertilizing"  v ( k r o2 s - f @@1 t @ l ai z i ng ) ((pos "Hb%" ) ))
( "cross-grained"  j ( k r o2 s - g r ei1 n d ) ((pos "OA%" ) ))
( "cross-heading"  n ( k r o2 s - h e1 d i ng ) ((pos "K6%" ) ))
( "cross-headings"  n ( k r o2 s - h e1 d i ng z ) ((pos "Kj%" ) ))
( "cross-index"  v ( k r o2 s - i1 n d e k s ) ((pos "H1%" "K7%" ) ))
( "cross-indexed"  v ( k r o2 s - i1 n d e k s t ) ((pos "Hc%" "Hd%" ) ))
( "cross-indexes"  v ( k r o2 s - i1 n d e k s i z ) ((pos "Ha%" "Kj%" ) ))
( "cross-indexing"  v ( k r o2 s - i1 n d e k s i ng ) ((pos "Hb%" ) ))
( "cross-indices"  n ( k r o2 s - i1 n d i s ii z ) ((pos "Kj$" ) ))
( "cross-legged"  a ( k r o2 s - l e1 g d ) ((pos "Pu%" ) ))
( "cross-purposes"  n ( k r o2 s - p @@1 p @ s i z ) ((pos "Kj%" ) ))
( "cross-question"  v ( k r o2 s - k w e1 s ch @ n ) ((pos "H0%" ) (subcat )))
( "cross-questioned"  v ( k r o2 s - k w e1 s ch @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cross-questioning"  v ( k r o2 s - k w e1 s ch @ n i ng ) ((pos "Hb%" ) (subcat )))
( "cross-questions"  v ( k r o2 s - k w e1 s ch @ n z ) ((pos "Ha%" ) (subcat )))
( "cross-reference"  n ( k r o2 s - r e1 f @ r @ n s ) ((pos "K6%" ) ))
( "cross-references"  n ( k r o2 s - r e1 f @ r @ n s i z ) ((pos "Kj%" ) ))
( "cross-section"  n ( k r o2 s - s e1 k sh @ n ) ((pos "K6%" ) ))
( "cross-sections"  n ( k r o2 s - s e1 k sh @ n z ) ((pos "Kj%" ) ))
( "cross-stitch"  n ( k r o1 s - s t i ch ) ((pos "M7%" ) ))
( "cross-stitches"  n ( k r o1 s - s t i ch i z ) ((pos "Mj%" ) ))
( "crossbar"  n ( k r o1 s b aa r ) ((pos "K6%" ) ))
( "crossbars"  n ( k r o1 s b aa z ) ((pos "Kj%" ) ))
( "crossbeam"  n ( k r o1 s b ii m ) ((pos "K6$" ) ))
( "crossbeams"  n ( k r o1 s b ii m z ) ((pos "Kj$" ) ))
( "crossbones"  n ( k r o1 s b ou n z ) ((pos "Kj%" ) ))
( "crossbow"  n ( k r o1 s b ou ) ((pos "K6%" ) ))
( "crossbows"  n ( k r o1 s b ou z ) ((pos "Kj%" ) ))
( "crossbred"  v ( k r o1 s b r e d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "2A" )))
( "crossbreed"  v ( k r o1 s b r ii d ) ((pos "H5%" "K6%" ) (subcat "2A" )))
( "crossbreeding"  v ( k r o1 s b r ii d i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "crossbreeds"  v ( k r o1 s b r ii d z ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "crosscheck"  v ( k r o2 s ch e1 k ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "crosschecked"  v ( k r o2 s ch e1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "crosschecking"  v ( k r o2 s ch e1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "crosschecks"  v ( k r o2 s ch e1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "crosscountry"  j ( k r o2 s k uh1 n t r ii ) ((pos "OA%" "Pu%" ) ))
( "crosscurrent"  n ( k r o1 s k uh r @ n t ) ((pos "K6%" ) ))
( "crosscurrents"  n ( k r o1 s k uh r @ n t s ) ((pos "Kj%" ) ))
( "crosscut"  n ( k r o1 s k uh t ) ((pos "K6%" "OA%" ) ))
( "crosscuts"  n ( k r o1 s k uh t s ) ((pos "Kj%" ) ))
( "crosse"  n ( k r o1 s ) ((pos "K6$" ) ))
( "crossed"  v ( k r o1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "crosses"  v ( k r o1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "crosseyed"  j ( k r o1 s ai d ) ((pos "OA%" ) ))
( "crossfire"  n ( k r o1 s f ai @ r ) ((pos "M6%" ) ))
( "crossfires"  n ( k r o1 s f ai @ z ) ((pos "Mj$" ) ))
( "crossing"  v ( k r o1 s i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "crossings"  n ( k r o1 s i ng z ) ((pos "Mj%" ) ))
( "crosskeys"  n ( k r o2 s k ii1 z ) ((pos "Kj$" ) ))
( "crossly"  a ( k r o1 s l ii ) ((pos "Pu%" ) ))
( "crossness"  n ( k r o1 s n @ s ) ((pos "L@%" ) ))
( "crosspatch"  n ( k r o1 s p a ch ) ((pos "K7%" ) ))
( "crosspatches"  n ( k r o1 s p a ch i z ) ((pos "Kj%" ) ))
( "crosspiece"  n ( k r o1 s p ii s ) ((pos "K6%" ) ))
( "crosspieces"  n ( k r o1 s p ii s i z ) ((pos "Kj%" ) ))
( "crossroad"  n ( k r o1 s r ou d ) ((pos "K6%" ) ))
( "crossroads"  n ( k r o1 s r ou d z ) ((pos "Kj%" ) ))
( "crosstalk"  n ( k r o1 s t oo k ) ((pos "L@%" ) ))
( "crosstrees"  n ( k r o1 s t r ii z ) ((pos "Kj%" ) ))
( "crosswalk"  n ( k r o1 s w oo k ) ((pos "K6$" ) ))
( "crosswalks"  n ( k r o1 s w oo k s ) ((pos "Kj$" ) ))
( "crosswind"  n ( k r o1 s w i n d ) ((pos "K6%" ) ))
( "crosswinds"  n ( k r o1 s w i n d z ) ((pos "Kj%" ) ))
( "crosswise"  a ( k r o1 s w ai z ) ((pos "Pu%" ) ))
( "crossword"  n ( k r o1 s w @@ d ) ((pos "K6%" ) ))
( "crosswords"  n ( k r o1 s w @@ d z ) ((pos "Kj%" ) ))
( "crotch"  n ( k r o1 ch ) ((pos "K7%" ) ))
( "crotches"  n ( k r o1 ch i z ) ((pos "Kj%" ) ))
( "crotchet"  n ( k r o1 ch i t ) ((pos "K6%" ) ))
( "crotchets"  n ( k r o1 ch i t s ) ((pos "Kj%" ) ))
( "crotchety"  j ( k r o1 ch i t ii ) ((pos "OA%" ) ))
( "crouch"  v ( k r au1 ch ) ((pos "I1%" "K7%" ) (subcat "2A" "2C" )))
( "crouched"  v ( k r au1 ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "crouches"  v ( k r au1 ch i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "crouching"  v ( k r au1 ch i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "croup"  n ( k r uu1 p ) ((pos "M6%" ) ))
( "croupier"  n ( k r uu1 p i ei ) ((pos "K6%" ) ))
( "croupiers"  n ( k r uu1 p i ei z ) ((pos "Kj%" ) ))
( "croups"  n ( k r uu1 p s ) ((pos "Mj$" ) ))
( "crow"  v ( k r ou1 ) ((pos "I0%" "K6%" ) (subcat "3A" )))
( "crow's-feet"  n ( k r ou1 z - f ii t ) ((pos "Kj%" ) ))
( "crow's-nest"  n ( k r ou1 z - n e s t ) ((pos "K6%" ) ))
( "crow's-nests"  n ( k r ou1 z - n e s t s ) ((pos "Kj%" ) ))
( "crowbar"  n ( k r ou1 b aa r ) ((pos "K6%" ) ))
( "crowbars"  n ( k r ou1 b aa z ) ((pos "Kj%" ) ))
( "crowd"  v ( k r au1 d ) ((pos "J0%" "K6%" ) (subcat "2B" "2C" "6A" "14" "15A" )))
( "crowded"  v ( k r au1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2B" "2C" "6A" "14" "15A" )))
( "crowding"  v ( k r au1 d i ng ) ((pos "Jb%" ) (subcat "2B" "2C" "6A" "14" "15A" )))
( "crowds"  v ( k r au1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2B" "2C" "6A" "14" "15A" )))
( "crowed"  v ( k r ou1 d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "crowing"  v ( k r ou1 i ng ) ((pos "Ib%" ) (subcat "3A" )))
( "crown"  v ( k r au1 n ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "crown-land"  n ( k r au1 n - l a n d ) ((pos "K6%" ) ))
( "crown-lands"  n ( k r au1 n - l a n d z ) ((pos "Kj%" ) ))
( "crowned"  v ( k r au1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "crowning"  v ( k r au1 n i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "crowns"  v ( k r au1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "crows"  v ( k r ou1 z ) ((pos "Ia%" "Kj%" ) (subcat "3A" )))
( "crozier"  n ( k r ou1 z i@ r ) ((pos "K6$" ) ))
( "croziers"  n ( k r ou1 z i@ z ) ((pos "Kj$" ) ))
( "crucial"  j ( k r uu1 sh l ) ((pos "OA%" ) ))
( "crucially"  a ( k r uu1 sh @ l ii ) ((pos "Pu%" ) ))
( "crucible"  n ( k r uu1 s i b l ) ((pos "K6%" ) ))
( "crucibles"  n ( k r uu1 s i b l z ) ((pos "Kj%" ) ))
( "crucified"  v ( k r uu1 s i f ai d ) ((pos "Hc%" "Hd%" ) ))
( "crucifies"  v ( k r uu1 s i f ai z ) ((pos "Ha%" ) ))
( "crucifix"  n ( k r uu1 s i f i k s ) ((pos "K7%" ) ))
( "crucifixes"  n ( k r uu1 s i f i k s i z ) ((pos "Kj%" ) ))
( "crucifixion"  n ( k r uu2 s i f i1 k sh @ n ) ((pos "M6%" ) ))
( "crucifixions"  n ( k r uu2 s i f i1 k sh @ n z ) ((pos "Mj%" ) ))
( "cruciform"  j ( k r uu1 s i f oo m ) ((pos "OA$" ) ))
( "crucify"  v ( k r uu1 s i f ai ) ((pos "H3%" ) ))
( "crucifying"  v ( k r uu1 s i f ai i ng ) ((pos "Hb%" ) ))
( "crude"  j ( k r uu1 d ) ((pos "OB%" ) ))
( "crudely"  a ( k r uu1 d l ii ) ((pos "Pu%" ) ))
( "crudeness"  n ( k r uu1 d n i s ) ((pos "L@%" ) ))
( "cruder"  j ( k r uu1 d @ r ) ((pos "Or%" ) ))
( "crudest"  j ( k r uu1 d i s t ) ((pos "Os%" ) ))
( "crudities"  n ( k r uu1 d i t i z ) ((pos "Mj%" ) ))
( "crudity"  n ( k r uu1 d i t ii ) ((pos "M8%" ) ))
( "cruel"  j ( k r u@1 l ) ((pos "OE%" ) ))
( "crueller"  j ( k r u@1 l @ r ) ((pos "Or%" ) ))
( "cruellest"  j ( k r u@1 l i s t ) ((pos "Os%" ) ))
( "cruelly"  a ( k r u@1 l ii ) ((pos "Pu%" ) ))
( "cruelties"  n ( k r uu1 @ l t i z ) ((pos "Mj%" ) ))
( "cruelty"  n ( k r uu1 @ l t ii ) ((pos "M8%" ) ))
( "cruet"  n ( k r uu1 i t ) ((pos "K6%" ) ))
( "cruet-stand"  n ( k r uu1 i t - s t a n d ) ((pos "K6$" ) ))
( "cruet-stands"  n ( k r uu1 i t - s t a n d z ) ((pos "Kj$" ) ))
( "cruets"  n ( k r uu1 i t s ) ((pos "Kj$" ) ))
( "cruise"  v ( k r uu1 z ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "cruised"  v ( k r uu1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "cruiser"  n ( k r uu1 z @ r ) ((pos "K6%" ) ))
( "cruisers"  n ( k r uu1 z @ z ) ((pos "Kj%" ) ))
( "cruises"  v ( k r uu1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "cruising"  v ( k r uu1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "crumb"  n ( k r uh1 m ) ((pos "M6%" ) ))
( "crumble"  v ( k r uh1 m b l ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "crumbled"  v ( k r uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "crumbles"  v ( k r uh1 m b l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "crumblier"  j ( k r uh1 m b l i@ r ) ((pos "Or%" ) ))
( "crumbliest"  j ( k r uh1 m b l i i s t ) ((pos "Os%" ) ))
( "crumbling"  v ( k r uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "crumbly"  j ( k r uh1 m b l ii ) ((pos "OD%" ) ))
( "crumbs"  n ( k r uh1 m z ) ((pos "Mj%" ) ))
( "crumpet"  n ( k r uh1 m p i t ) ((pos "K6%" ) ))
( "crumpets"  n ( k r uh1 m p i t s ) ((pos "Kj%" ) ))
( "crumple"  v ( k r uh1 m p l ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "crumpled"  v ( k r uh1 m p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "crumples"  v ( k r uh1 m p l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "crumpling"  v ( k r uh1 m p l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "crunch"  v ( k r uh1 n ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" )))
( "crunched"  v ( k r uh1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "crunches"  v ( k r uh1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "crunching"  v ( k r uh1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "crupper"  n ( k r uh1 p @ r ) ((pos "K6$" ) ))
( "cruppers"  n ( k r uh1 p @ z ) ((pos "Kj$" ) ))
( "crusade"  v ( k r uu1 s ei1 d ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "crusaded"  v ( k r uu1 s ei1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "crusader"  n ( k r uu1 s ei1 d @ r ) ((pos "K6%" ) ))
( "crusaders"  n ( k r uu1 s ei1 d @ z ) ((pos "Kj%" ) ))
( "crusades"  v ( k r uu1 s ei1 d z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "crusading"  v ( k r uu1 s ei1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "cruse"  n ( k r uu1 z ) ((pos "K6$" ) ))
( "cruses"  n ( k r uu1 z i z ) ((pos "Kj$" ) ))
( "crush"  v ( k r uh1 sh ) ((pos "J1%" "L@%" ) (subcat "2A" "2C" "6A" "15A" )))
( "crushed"  v ( k r uh1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "crushes"  v ( k r uh1 sh i z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "crushing"  v ( k r uh1 sh i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "crushingly"  a ( k r uh1 sh i ng l ii ) ((pos "Pu%" ) ))
( "crust"  v ( k r uh1 s t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "crustacean"  n ( k r uh1 s t ei1 sh @ n ) ((pos "K6%" ) ))
( "crustaceans"  n ( k r uh1 s t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "crusted"  v ( k r uh1 s t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" )))
( "crustier"  j ( k r uh1 s t i@ r ) ((pos "Or%" ) ))
( "crustiest"  j ( k r uh1 s t i i s t ) ((pos "Os%" ) ))
( "crusting"  v ( k r uh1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "crusts"  v ( k r uh1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "crusty"  j ( k r uh1 s t ii ) ((pos "OD%" ) ))
( "crutch"  n ( k r uh1 ch ) ((pos "K7%" ) ))
( "crutches"  n ( k r uh1 ch i z ) ((pos "Kj%" ) ))
( "crux"  n ( k r uh1 k s ) ((pos "K7%" ) ))
( "cruxes"  n ( k r uh1 k s i z ) ((pos "Kj$" ) ))
( "cruzeiro"  n ( k r uu1 z e@1 r ou ) ((pos "K6$" ) ))
( "cruzeiros"  n ( k r uu1 z e@1 r ou z ) ((pos "Kj$" ) ))
( "cry"  v ( k r ai1 ) ((pos "J3%" "K8%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "9" "14" "15A" )))
( "crybabies"  n ( k r ai1 b ei b i z ) ((pos "Kj%" ) ))
( "crybaby"  n ( k r ai1 b ei b ii ) ((pos "K8%" ) ))
( "crying"  v ( k r ai1 i ng ) ((pos "Jb%" "Oq%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "9" "14" "15A" )))
( "crypt"  n ( k r i1 p t ) ((pos "K6%" ) ))
( "cryptic"  j ( k r i1 p t i k ) ((pos "OA%" ) ))
( "cryptically"  a ( k r i1 p t i k l ii ) ((pos "Pu%" ) ))
( "crypto-"  j ( k r i2 p t ou - ) ((pos "U-%" ) ))
( "cryptogram"  n ( k r i1 p t @ g r a m ) ((pos "K6%" ) ))
( "cryptograms"  n ( k r i1 p t @ g r a m z ) ((pos "Kj%" ) ))
( "crypts"  n ( k r i1 p t s ) ((pos "Kj%" ) ))
( "crystal"  n ( k r i1 s t l ) ((pos "M6%" ) ))
( "crystal-gazing"  n ( k r i1 s t @ l - g ei z i ng ) ((pos "L@%" ) ))
( "crystalline"  j ( k r i1 s t @ l ai n ) ((pos "OA%" ) ))
( "crystallization"  n ( k r i2 s t @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "crystallizations"  n ( k r i2 s t @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "crystallize"  v ( k r i1 s t @ l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "crystallized"  v ( k r i1 s t @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "crystallizes"  v ( k r i1 s t @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "crystallizing"  v ( k r i1 s t @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "crystallography"  n ( k r i2 s t @ l o1 g r @ f ii ) ((pos "L@%" ) ))
( "crystals"  n ( k r i1 s t l z ) ((pos "Mj%" ) ))
( "cub"  n ( k uh1 b ) ((pos "K6%" ) ))
( "cubbyhole"  n ( k uh1 b i h ou l ) ((pos "K6%" ) ))
( "cubbyholes"  n ( k uh1 b i h ou l z ) ((pos "Kj%" ) ))
( "cube"  v ( k y uu1 b ) ((pos "H2%" "K6%" ) (subcat )))
( "cubed"  v ( k y uu1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cubes"  v ( k y uu1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cubic"  j ( k y uu1 b i k ) ((pos "OA%" ) ))
( "cubical"  j ( k y uu1 b i k l ) ((pos "OA$" ) ))
( "cubicle"  n ( k y uu1 b i k l ) ((pos "K6%" ) ))
( "cubicles"  n ( k y uu1 b i k l z ) ((pos "Kj%" ) ))
( "cubing"  v ( k y uu1 b i ng ) ((pos "Hb%" ) (subcat )))
( "cubism"  n ( k y uu1 b i z @ m ) ((pos "L@%" ) ))
( "cubist"  n ( k y uu1 b i s t ) ((pos "K6%" ) ))
( "cubists"  n ( k y uu1 b i s t s ) ((pos "Kj%" ) ))
( "cubit"  n ( k y uu1 b i t ) ((pos "K6$" ) ))
( "cubits"  n ( k y uu1 b i t s ) ((pos "Kj$" ) ))
( "cubs"  n ( k uh1 b z ) ((pos "Kj%" ) ))
( "cuckold"  v ( k uh1 k ou l d ) ((pos "H0%" "K6%" ) (subcat )))
( "cuckolded"  v ( k uh1 k ou l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cuckolding"  v ( k uh1 k ou l d i ng ) ((pos "Hb%" ) (subcat )))
( "cuckolds"  v ( k uh1 k ou l d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cuckoo"  n ( k u1 k uu ) ((pos "K6%" ) ))
( "cuckoo-clock"  n ( k u1 k uu - k l o k ) ((pos "K6%" ) ))
( "cuckoo-clocks"  n ( k u1 k uu - k l o k s ) ((pos "Kj%" ) ))
( "cuckoos"  n ( k u1 k uu z ) ((pos "Kj%" ) ))
( "cucumber"  n ( k y uu1 k uh m b @ r ) ((pos "M6%" ) ))
( "cucumbers"  n ( k y uu1 k uh m b @ z ) ((pos "Mj%" ) ))
( "cud"  n ( k uh1 d ) ((pos "L@%" ) ))
( "cuddle"  v ( k uh1 d l ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" )))
( "cuddled"  v ( k uh1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "cuddles"  v ( k uh1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "cuddlesome"  j ( k uh1 d l s @ m ) ((pos "OA%" ) ))
( "cuddlier"  j ( k uh1 d l i@ r ) ((pos "Or%" ) ))
( "cuddliest"  j ( k uh1 d l i i s t ) ((pos "Os%" ) ))
( "cuddling"  v ( k uh1 d l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "cuddly"  j ( k uh1 d l ii ) ((pos "OD%" ) ))
( "cudgel"  v ( k uh1 jh @ l ) ((pos "H0%" "K6%" ) ))
( "cudgeled"  v ( k uh1 jh @ l d ) ((pos "Hc%" "Hd%" ) ))
( "cudgeling"  v ( k uh1 jh @ l i ng ) ((pos "Hb%" ) ))
( "cudgels"  v ( k uh1 jh @ l z ) ((pos "Ha%" "Kj%" ) ))
( "cue"  n ( k y uu1 ) ((pos "K6%" ) ))
( "cues"  n ( k y uu1 z ) ((pos "Kj%" ) ))
( "cuff"  v ( k uh1 f ) ((pos "H0%" "K6%" ) ))
( "cuff-links"  n ( k uh1 f - l i ng k s ) ((pos "Kj%" ) ))
( "cuffed"  v ( k uh1 f t ) ((pos "Hc%" "Hd%" ) ))
( "cuffing"  v ( k uh1 f i ng ) ((pos "Hb%" ) ))
( "cuffs"  v ( k uh1 f s ) ((pos "Ha%" "Kj%" ) ))
( "cuirass"  n ( k w i1 r a1 s ) ((pos "K7$" ) ))
( "cuirasses"  n ( k w i1 r a1 s i z ) ((pos "Kj$" ) ))
( "cuirassier"  n ( k w i2 r @ s i@1 r ) ((pos "K6$" ) ))
( "cuirassiers"  n ( k w i2 r @ s i@1 z ) ((pos "Kj$" ) ))
( "cuisine"  n ( k w i1 z ii1 n ) ((pos "L@%" ) ))
( "cul-de-sac"  n ( k uh1 l - d @ - s a k ) ((pos "K6%" ) ))
( "cul-de-sacs"  n ( k uh1 l - d @ - s a k s ) ((pos "Kj%" ) ))
( "culinary"  j ( k uh1 l i n @ r ii ) ((pos "OA%" ) ))
( "cull"  v ( k uh1 l ) ((pos "H0%" "K6%" ) (subcat )))
( "culled"  v ( k uh1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cullender"  n ( k uh1 l @ n d @ r ) ((pos "K6$" ) ))
( "cullenders"  n ( k uh1 l @ n d @ z ) ((pos "Kj$" ) ))
( "culling"  v ( k uh1 l i ng ) ((pos "Hb%" ) (subcat )))
( "culls"  v ( k uh1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "culminate"  v ( k uh1 l m i n ei t ) ((pos "H2%" ) (subcat )))
( "culminated"  v ( k uh1 l m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "culminates"  v ( k uh1 l m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "culminating"  v ( k uh1 l m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "culmination"  n ( k uh2 l m i n ei1 sh @ n ) ((pos "K6%" ) ))
( "culminations"  n ( k uh2 l m i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "culpability"  n ( k uh2 l p @ b i1 l i t ii ) ((pos "L@%" ) ))
( "culpable"  j ( k uh1 l p @ b l ) ((pos "OA%" ) ))
( "culpably"  a ( k uh1 l p @ b l ii ) ((pos "Pu%" ) ))
( "culprit"  n ( k uh1 l p r i t ) ((pos "K6%" ) ))
( "culprits"  n ( k uh1 l p r i t s ) ((pos "Kj%" ) ))
( "cult"  n ( k uh1 l t ) ((pos "K6%" ) ))
( "cultivable"  j ( k uh1 l t i v @ b l ) ((pos "OA%" ) ))
( "cultivate"  v ( k uh1 l t i v ei t ) ((pos "H2%" ) (subcat )))
( "cultivated"  v ( k uh1 l t i v ei t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "cultivates"  v ( k uh1 l t i v ei t s ) ((pos "Ha%" ) (subcat )))
( "cultivating"  v ( k uh1 l t i v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "cultivation"  n ( k uh2 l t i v ei1 sh @ n ) ((pos "L@%" ) ))
( "cultivator"  n ( k uh1 l t i v ei t @ r ) ((pos "K6%" ) ))
( "cultivators"  n ( k uh1 l t i v ei t @ z ) ((pos "Kj%" ) ))
( "cults"  n ( k uh1 l t s ) ((pos "Kj%" ) ))
( "cultural"  j ( k uh1 l ch @ r @ l ) ((pos "OA%" ) ))
( "culture"  n ( k uh1 l ch @ r ) ((pos "M6%" ) ))
( "cultured"  j ( k uh1 l ch @ d ) ((pos "OA%" ) ))
( "cultures"  n ( k uh1 l ch @ z ) ((pos "Mj%" ) ))
( "culvert"  n ( k uh1 l v @ t ) ((pos "K6%" ) ))
( "culverts"  n ( k uh1 l v @ t s ) ((pos "Kj%" ) ))
( "cumber"  v ( k uh1 m b @ r ) ((pos "H0$" ) (subcat "6A" )))
( "cumbered"  v ( k uh1 m b @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cumbering"  v ( k uh1 m b @ r i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "cumbers"  v ( k uh1 m b @ z ) ((pos "Ha$" ) (subcat "6A" )))
( "cumbersome"  j ( k uh1 m b @ s @ m ) ((pos "OA%" ) ))
( "cumbrous"  j ( k uh1 m b r @ s ) ((pos "OA$" ) ))
( "cummerbund"  n ( k uh1 m @ b uh n d ) ((pos "K6$" ) ))
( "cummerbunds"  n ( k uh1 m @ b uh n d z ) ((pos "Kj$" ) ))
( "cumulative"  j ( k y uu1 m y u l @ t i v ) ((pos "OA%" ) ))
( "cumulatively"  a ( k y uu1 m y u l @ t i v l ii ) ((pos "Pu%" ) ))
( "cumuli"  n ( k y uu1 m y u l ai ) ((pos "Kj$" ) ))
( "cumulus"  n ( k y uu1 m y u l @ s ) ((pos "Ki%" ) ))
( "cuneiform"  j ( k y uu1 n i f oo m ) ((pos "OA$" ) ))
( "cunning"  n ( k uh1 n i ng ) ((pos "L@%" "OA%" ) ))
( "cunningly"  a ( k uh1 n i ng l ii ) ((pos "Pu%" ) ))
( "cunt"  n ( k uh1 n t ) ((pos "K6%" ) ))
( "cunts"  n ( k uh1 n t s ) ((pos "Kj%" ) ))
( "cup"  v ( k uh1 p ) ((pos "H4%" "K6%" ) (subcat )))
( "cup-bearer"  n ( k uh1 p - b e@ r @ r ) ((pos "K6%" ) ))
( "cup-bearers"  n ( k uh1 p - b e@ r @ z ) ((pos "Kj%" ) ))
( "cup-final"  n ( k uh1 p - f ai1 n l ) ((pos "K6%" ) ))
( "cup-finals"  n ( k uh1 p - f ai1 n l z ) ((pos "Kj%" ) ))
( "cup-tie"  n ( k uh1 p - t ai ) ((pos "K6%" ) ))
( "cup-ties"  n ( k uh1 p - t ai z ) ((pos "Kj%" ) ))
( "cupboard"  n ( k uh1 b @ d ) ((pos "K6%" ) ))
( "cupboard-love"  n ( k uh1 b @ d - l uh v ) ((pos "L@%" ) ))
( "cupboards"  n ( k uh1 b @ d z ) ((pos "Kj%" ) ))
( "cupful"  n ( k uh1 p f u l ) ((pos "K6%" ) ))
( "cupfuls"  n ( k uh1 p f u l z ) ((pos "Kj%" ) ))
( "cupidity"  n ( k y uu1 p i1 d i t ii ) ((pos "L@$" ) ))
( "cupola"  n ( k y uu1 p @ l @ ) ((pos "K6$" ) ))
( "cupolas"  n ( k y uu1 p @ l @ z ) ((pos "Kj$" ) ))
( "cuppa"  n ( k uh1 p @ ) ((pos "K6%" ) ))
( "cuppas"  n ( k uh1 p @ z ) ((pos "Kj%" ) ))
( "cupped"  v ( k uh1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "cupping"  v ( k uh1 p i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "cupric"  j ( k y uu1 p r i k ) ((pos "OA$" ) ))
( "cupro-nickel"  n ( k y uu2 p r ou - n i1 k l ) ((pos "L@$" ) ))
( "cups"  v ( k uh1 p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "cur"  n ( k @@1 r ) ((pos "K6%" ) ))
( "cur_e"  n ( k y u@1 r ei ) ((pos "K6$" ) ))
( "cur_es"  n ( k y u@1 r ei z ) ((pos "Kj$" ) ))
( "cura<cao"  n ( k y u@2 r @ s ou1 ) ((pos "L@$" ) ))
( "cura<coa"  n ( k y u@2 r @ s ou1 ) ((pos "L@$" ) ))
( "curability"  n ( k y u@2 r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "curable"  j ( k y u@1 r @ b l ) ((pos "OA%" ) ))
( "curacies"  n ( k y u@1 r @ s i z ) ((pos "Kj$" ) ))
( "curacy"  n ( k y u@1 r @ s ii ) ((pos "K8$" ) ))
( "curate"  n ( k y u@1 r @ t ) ((pos "K6%" ) ))
( "curates"  n ( k y u@1 r @ t s ) ((pos "Kj%" ) ))
( "curative"  j ( k y u@1 r @ t i v ) ((pos "OA%" ) ))
( "curator"  n ( k y u@1 r ei1 t @ r ) ((pos "K6%" ) ))
( "curators"  n ( k y u@1 r ei1 t @ z ) ((pos "Kj%" ) ))
( "curb"  v ( k @@1 b ) ((pos "H0%" "K6%" ) (subcat )))
( "curbed"  v ( k @@1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "curbing"  v ( k @@1 b i ng ) ((pos "Hb%" ) (subcat )))
( "curbs"  v ( k @@1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "curd"  n ( k @@1 d ) ((pos "M6%" ) ))
( "curdle"  v ( k @@1 d l ) ((pos "J2%" ) (subcat "2A" )))
( "curdled"  v ( k @@1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "curdles"  v ( k @@1 d l z ) ((pos "Ja%" ) (subcat "2A" )))
( "curdling"  v ( k @@1 d l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "curds"  n ( k @@1 d z ) ((pos "Mj%" ) ))
( "cure"  v ( k y u@1 r ) ((pos "J2%" "K6%" ) (subcat "6A" )))
( "cure-all"  n ( k y u@1 r - oo l ) ((pos "K6%" ) ))
( "cure-alls"  n ( k y u@1 r - oo l z ) ((pos "Kj$" ) ))
( "cured"  v ( k y u@1 d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "cures"  v ( k y u@1 z ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "curfew"  n ( k @@1 f y uu ) ((pos "K6%" ) ))
( "curfews"  n ( k @@1 f y uu z ) ((pos "Kj%" ) ))
( "curing"  v ( k y u@1 r i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "curio"  n ( k y u@1 r i@ @ ) ((pos "K6%" ) ))
( "curios"  n ( k y u@1 r i@ u z ) ((pos "Kj%" ) ))
( "curiosities"  n ( k y u@2 r i o1 s i t i z ) ((pos "Mj%" ) ))
( "curiosity"  n ( k y u@2 r i o1 s i t ii ) ((pos "M8%" ) ))
( "curious"  j ( k y u@1 r i@ s ) ((pos "OA%" ) ))
( "curiously"  a ( k y u@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "curl"  v ( k @@1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "curled"  v ( k @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "curler"  n ( k @@1 l @ r ) ((pos "K6%" ) ))
( "curlers"  n ( k @@1 l @ z ) ((pos "Kj%" ) ))
( "curlew"  n ( k @@1 l y uu ) ((pos "K6%" ) ))
( "curlews"  n ( k @@1 l y uu z ) ((pos "Kj%" ) ))
( "curlier"  j ( k @@1 l i@ r ) ((pos "Or%" ) ))
( "curliest"  j ( k @@1 l i i s t ) ((pos "Os%" ) ))
( "curling"  v ( k @@1 l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "15A" )))
( "curling-irons"  n ( k @@1 l i ng - ai @ n z ) ((pos "Kj%" ) ))
( "curling-pins"  n ( k @@1 l i ng - p i n z ) ((pos "Kj%" ) ))
( "curling-stones"  n ( k @@1 l i ng - s t ou n z ) ((pos "Kj%" ) ))
( "curling-tongs"  n ( k @@1 l i ng - t o ng z ) ((pos "Kj%" ) ))
( "curls"  v ( k @@1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "curly"  j ( k @@1 l ii ) ((pos "OD%" ) ))
( "curmudgeon"  n ( k @@1 m uh1 jh @ n ) ((pos "K6$" ) ))
( "curmudgeons"  n ( k @@1 m uh1 jh @ n z ) ((pos "Kj$" ) ))
( "currant"  n ( k uh1 r @ n t ) ((pos "K6%" ) ))
( "currants"  n ( k uh1 r @ n t s ) ((pos "Kj%" ) ))
( "currencies"  n ( k uh1 r @ n s i z ) ((pos "Mj%" ) ))
( "currency"  n ( k uh1 r @ n s ii ) ((pos "M8%" ) ))
( "current"  n ( k uh1 r @ n t ) ((pos "K6%" "OA%" ) ))
( "currently"  a ( k uh1 r @ n t l ii ) ((pos "Pu%" ) ))
( "currents"  n ( k uh1 r @ n t s ) ((pos "Kj%" ) ))
( "curricula"  n ( k @1 r i1 k y u l @ ) ((pos "Kj%" ) ))
( "curricula vitae"  n ( k @1 r i2 k y u l @ - v ii1 t ai ) ((pos "Kj$" ) ))
( "curriculum"  n ( k @1 r i1 k y u l @ m ) ((pos "K6%" ) ))
( "curriculum vitae"  n ( k @1 r i2 k y u l @ m - v ii1 t ai ) ((pos "Ki%" ) ))
( "curriculums"  n ( k @1 r i1 k y u l @ m z ) ((pos "Kj%" ) ))
( "curried"  v ( k uh1 r i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "curries"  v ( k uh1 r i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "currish"  j ( k @@1 r i sh ) ((pos "OA$" ) ))
( "currishly"  a ( k @@1 r i sh l ii ) ((pos "Pu$" ) ))
( "curry"  v ( k uh1 r ii ) ((pos "H3%" "M8%" ) (subcat )))
( "curry-powder"  n ( k uh1 r i - p au d @ r ) ((pos "K6%" ) ))
( "curry-powders"  n ( k uh1 r i - p au d @ z ) ((pos "Kj%" ) ))
( "currying"  v ( k uh1 r i i ng ) ((pos "Hb%" ) (subcat )))
( "curs"  n ( k @@1 z ) ((pos "Kj%" ) ))
( "curse"  v ( k @@1 s ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "cursed"  j ( k @@1 s i d ) ((pos "OA%" ) ))
( "cursed"  v ( k @@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "cursedly"  a ( k @@1 s i d l ii ) ((pos "Pu%" ) ))
( "curses"  v ( k @@1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "cursing"  v ( k @@1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "cursive"  j ( k @@1 s i v ) ((pos "OA%" ) ))
( "cursorily"  a ( k @@1 s @ r @ l ii ) ((pos "Pu%" ) ))
( "cursory"  j ( k @@1 s @ r ii ) ((pos "OA%" ) ))
( "curst"  j ( k @@1 s t ) ((pos "OA$" ) ))
( "curt"  j ( k @@1 t ) ((pos "OA%" ) ))
( "curtail"  v ( k @@1 t ei1 l ) ((pos "H0%" ) (subcat )))
( "curtailed"  v ( k @@1 t ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "curtailing"  v ( k @@1 t ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "curtailment"  n ( k @@1 t ei1 l m @ n t ) ((pos "M6%" ) ))
( "curtailments"  n ( k @@1 t ei1 l m @ n t s ) ((pos "Mj%" ) ))
( "curtails"  v ( k @@1 t ei1 l z ) ((pos "Ha%" ) (subcat )))
( "curtain"  v ( k @@1 t n ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "curtain-call"  n ( k @@1 t n - k oo l ) ((pos "K6%" ) ))
( "curtain-calls"  n ( k @@1 t n - k oo l z ) ((pos "Kj%" ) ))
( "curtain-lecture"  n ( k @@1 t n - l e k ch @ r ) ((pos "K6$" ) ))
( "curtain-lectures"  n ( k @@1 t n - l e k ch @ z ) ((pos "Kj$" ) ))
( "curtain-raiser"  n ( k @@1 t n - r ei z @ r ) ((pos "K6%" ) ))
( "curtain-raisers"  n ( k @@1 t n - r ei z @ z ) ((pos "Kj%" ) ))
( "curtained"  v ( k @@1 t n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "curtaining"  v ( k @@1 t n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "curtains"  v ( k @@1 t n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "curtly"  a ( k @@1 t l ii ) ((pos "Pu%" ) ))
( "curtness"  n ( k @@1 t n @ s ) ((pos "L@%" ) ))
( "curtsey"  v ( k @@1 t s ii ) ((pos "I0%" "K6%" ) ))
( "curtseyed"  v ( k @@1 t s i d ) ((pos "Ic%" "Id%" ) ))
( "curtseying"  v ( k @@1 t s i i ng ) ((pos "Ib%" ) ))
( "curtseys"  v ( k @@1 t s i z ) ((pos "Ia%" "Kj%" ) ))
( "curtsied"  v ( k @@1 t s i d ) ((pos "Ic%" "Id%" ) ))
( "curtsies"  v ( k @@1 t s i z ) ((pos "Ia%" "Kj%" ) ))
( "curtsy"  v ( k @@1 t s ii ) ((pos "I3$" "K8$" ) ))
( "curtsying"  v ( k @@1 t s i i ng ) ((pos "Ib$" ) ))
( "curvature"  n ( k @@1 v @ ch @ r ) ((pos "L@%" ) ))
( "curve"  v ( k @@1 v ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "curved"  v ( k @@1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "curves"  v ( k @@1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "curving"  v ( k @@1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cushier"  j ( k u1 sh i@ r ) ((pos "Or%" ) ))
( "cushiest"  j ( k u1 sh i i s t ) ((pos "Os%" ) ))
( "cushion"  v ( k u1 sh @ n ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "cushioned"  v ( k u1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "cushioning"  v ( k u1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "cushions"  v ( k u1 sh @ n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "cushy"  j ( k u1 sh ii ) ((pos "OD%" ) ))
( "cusp"  n ( k uh1 s p ) ((pos "K6%" ) ))
( "cuspidor"  n ( k uh1 s p i d oo r ) ((pos "K6$" ) ))
( "cuspidors"  n ( k uh1 s p i d oo z ) ((pos "Kj$" ) ))
( "cusps"  n ( k uh1 s p s ) ((pos "Kj%" ) ))
( "cuss"  n ( k uh1 s ) ((pos "K7%" ) ))
( "cussed"  j ( k uh1 s i d ) ((pos "OA%" ) ))
( "cussedly"  a ( k uh1 s i d l ii ) ((pos "Pu%" ) ))
( "cussedness"  n ( k uh1 s i d n @ s ) ((pos "L@%" ) ))
( "cusses"  n ( k uh1 s i z ) ((pos "Kj%" ) ))
( "custard"  n ( k uh1 s t @ d ) ((pos "M6%" ) ))
( "custards"  n ( k uh1 s t @ d z ) ((pos "Mj%" ) ))
( "custodial"  j ( k uh1 s t ou1 d i@ l ) ((pos "OA%" ) ))
( "custodian"  n ( k uh1 s t ou1 d i@ n ) ((pos "K6%" ) ))
( "custodians"  n ( k uh1 s t ou1 d i@ n z ) ((pos "Kj%" ) ))
( "custody"  n ( k uh1 s t @ d ii ) ((pos "L@%" ) ))
( "custom"  n ( k uh1 s t @ m ) ((pos "M6%" ) ))
( "custom-built"  j ( k uh2 s t @ m - b i1 l t ) ((pos "OA%" ) ))
( "custom-made"  j ( k uh2 s t @ m - m ei1 d ) ((pos "OA%" ) ))
( "customarily"  a ( k uh1 s t @ m @ r @ l ii ) ((pos "Pu%" ) ))
( "customary"  j ( k uh1 s t @ m @ r ii ) ((pos "OA%" ) ))
( "customer"  n ( k uh1 s t @ m @ r ) ((pos "K6%" ) ))
( "customers"  n ( k uh1 s t @ m @ z ) ((pos "Kj%" ) ))
( "customs"  n ( k uh1 s t @ m z ) ((pos "Mj%" ) ))
( "cut"  v ( k uh1 t ) ((pos "J5*" "Jc*" "Jd*" "K6*" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "15A" "15B" )))
( "cut-out"  n ( k uh1 t - au t ) ((pos "K6%" ) ))
( "cut-outs"  n ( k uh1 t - au t s ) ((pos "Kj%" ) ))
( "cut-price"  j ( k uh2 t - p r ai1 s ) ((pos "Oq%" ) ))
( "cut-rate"  j ( k uh2 t - r ei1 t ) ((pos "Oq%" ) ))
( "cut-throat"  n ( k uh1 t - th r ou t ) ((pos "K6$" "Oq%" ) ))
( "cut-throats"  n ( k uh1 t - th r ou t s ) ((pos "Kj$" ) ))
( "cutback"  n ( k uh1 t b a k ) ((pos "K6%" ) ))
( "cutbacks"  n ( k uh1 t b a k s ) ((pos "Kj%" ) ))
( "cute"  j ( k y uu1 t ) ((pos "OB%" ) ))
( "cutely"  a ( k y uu1 t l ii ) ((pos "Pu%" ) ))
( "cuteness"  n ( k y uu1 t n @ s ) ((pos "L@%" ) ))
( "cuter"  j ( k y uu1 t @ r ) ((pos "Or%" ) ))
( "cutest"  j ( k y uu1 t i s t ) ((pos "Os%" ) ))
( "cuticle"  n ( k y uu1 t i k l ) ((pos "K6%" ) ))
( "cuticles"  n ( k y uu1 t i k l z ) ((pos "Kj%" ) ))
( "cutlass"  n ( k uh1 t l @ s ) ((pos "K7%" ) ))
( "cutlasses"  n ( k uh1 t l @ s i z ) ((pos "Kj%" ) ))
( "cutler"  n ( k uh1 t l @ r ) ((pos "K6%" ) ))
( "cutlers"  n ( k uh1 t l @ z ) ((pos "Kj%" ) ))
( "cutlery"  n ( k uh1 t l @ r ii ) ((pos "L@%" ) ))
( "cutlet"  n ( k uh1 t l i t ) ((pos "K6%" ) ))
( "cutlets"  n ( k uh1 t l i t s ) ((pos "Kj%" ) ))
( "cutpurse"  n ( k uh1 t p @@ s ) ((pos "K6$" ) ))
( "cutpurses"  n ( k uh1 t p @@ s i z ) ((pos "Kj$" ) ))
( "cuts"  v ( k uh1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "15A" "15B" )))
( "cutter"  n ( k uh1 t @ r ) ((pos "K6%" ) ))
( "cutters"  n ( k uh1 t @ z ) ((pos "Kj%" ) ))
( "cutting"  v ( k uh1 t i ng ) ((pos "Jb%" "M6%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "15A" "15B" )))
( "cutting-room"  n ( k uh1 t i ng - r uu m ) ((pos "K6%" ) ))
( "cutting-rooms"  n ( k uh1 t i ng - r uu m z ) ((pos "Kj%" ) ))
( "cuttings"  n ( k uh1 t i ng z ) ((pos "Mj%" ) ))
( "cuttlefish"  n ( k uh1 t l f i sh ) ((pos "M7%" ) ))
( "cuttlefishes"  n ( k uh1 t l f i sh i z ) ((pos "Mj$" ) ))
( "cutworm"  n ( k uh1 t w @@ m ) ((pos "K6%" ) ))
( "cutworms"  n ( k uh1 t w @@ m z ) ((pos "Kj$" ) ))
( "cwt"  n ( h uh1 d r @ d w ei t ) ((pos "Y]%" ) ))
( "cyanide"  n ( s ai1 @ n ai d ) ((pos "L@%" ) ))
( "cybernetic"  j ( s ai2 b @ n e1 t i k ) ((pos "OA%" ) ))
( "cybernetics"  n ( s ai2 b @ n e1 t i k s ) ((pos "Lk%" ) ))
( "cyclamen"  n ( s i1 k l @ m @ n ) ((pos "K6%" ) ))
( "cyclamens"  n ( s i1 k l @ m @ n z ) ((pos "Kj%" ) ))
( "cycle"  v ( s ai1 k l ) ((pos "I2%" "K6%" ) (subcat "2A" "2B" )))
( "cycled"  v ( s ai1 k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "cycles"  v ( s ai1 k l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" )))
( "cyclic"  j ( s ai1 k l i k ) ((pos "OA%" ) ))
( "cyclical"  j ( s ai1 k l i k l ) ((pos "OA%" ) ))
( "cycling"  v ( s ai1 k l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "cyclist"  n ( s ai1 k l i s t ) ((pos "K6%" ) ))
( "cyclists"  n ( s ai1 k l i s t s ) ((pos "Kj%" ) ))
( "cyclone"  n ( s ai1 k l ou n ) ((pos "K6%" ) ))
( "cyclones"  n ( s ai1 k l ou n z ) ((pos "Kj%" ) ))
( "cyclonic"  j ( s ai1 k l o1 n i k ) ((pos "OA%" ) ))
( "cyclopaedia"  n ( s ai2 k l @ p ii1 d i@ ) ((pos "K6$" ) ))
( "cyclopaedias"  n ( s ai2 k l @ p ii1 d i@ z ) ((pos "Kj$" ) ))
( "cyclostyle"  v ( s ai1 k l @ s t ai l ) ((pos "H2$" "K6$" ) ))
( "cyclostyled"  v ( s ai1 k l @ s t ai l d ) ((pos "Hc$" "Hd%" ) ))
( "cyclostyles"  v ( s ai1 k l @ s t ai l z ) ((pos "Ha$" "Kj$" ) ))
( "cyclostyling"  v ( s ai1 k l @ s t ai l i ng ) ((pos "Hb$" ) ))
( "cyclotron"  n ( s ai1 k l @ t r o n ) ((pos "K6%" ) ))
( "cyclotrons"  n ( s ai1 k l @ t r o n z ) ((pos "Kj%" ) ))
( "cyder"  n ( s ai1 d @ r ) ((pos "M6%" ) ))
( "cyders"  n ( s ai1 d @ z ) ((pos "Mj$" ) ))
( "cygnet"  n ( s i1 g n i t ) ((pos "K6$" ) ))
( "cygnets"  n ( s i1 g n i t s ) ((pos "Kj$" ) ))
( "cylinder"  n ( s i1 l i n d @ r ) ((pos "K6%" ) ))
( "cylinders"  n ( s i1 l i n d @ z ) ((pos "Kj%" ) ))
( "cylindrical"  j ( s i1 l i1 n d r i k l ) ((pos "OA%" ) ))
( "cymbal"  n ( s i1 m b l ) ((pos "K6%" ) ))
( "cymbals"  n ( s i1 m b l z ) ((pos "Kj%" ) ))
( "cynic"  n ( s i1 n i k ) ((pos "K6%" ) ))
( "cynical"  j ( s i1 n i k l ) ((pos "OA%" ) ))
( "cynically"  a ( s i1 n i k l ii ) ((pos "Pu%" ) ))
( "cynicism"  n ( s i1 n i s i z @ m ) ((pos "M6%" ) ))
( "cynicisms"  n ( s i1 n i s i z @ m z ) ((pos "Mj$" ) ))
( "cynics"  n ( s i1 n i k s ) ((pos "Kj%" ) ))
( "cynosure"  n ( s i1 n @ z y u@2 r ) ((pos "K6$" ) ))
( "cynosures"  n ( s i1 n @ z y u@2 z ) ((pos "Kj$" ) ))
( "cypher"  v ( s ai1 f @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "cyphered"  v ( s ai1 f @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "cyphering"  v ( s ai1 f @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "cyphers"  v ( s ai1 f @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "cypress"  n ( s ai1 p r @ s ) ((pos "K7%" ) ))
( "cypresses"  n ( s ai1 p r @ s i z ) ((pos "Kj%" ) ))
( "cyst"  n ( s i1 s t ) ((pos "K6%" ) ))
( "cysts"  n ( s i1 s t s ) ((pos "Kj%" ) ))
( "czar"  n ( z aa1 r ) ((pos "K6%" ) ))
( "czarina"  n ( z aa1 r ii1 n @ ) ((pos "K6%" ) ))
( "czarinas"  n ( z aa1 r ii1 n @ z ) ((pos "Kj%" ) ))
( "czars"  n ( z aa1 z ) ((pos "Kj%" ) ))
( "d"  n ( d ii1 ) ((pos "Ki$" ) ))
( "d's"  n ( d ii1 z ) ((pos "Kj$" ) ))
( "d_eb^acle"  n ( d ei1 b aa1 k l ) ((pos "K6%" ) ))
( "d_eb^acles"  n ( d ei1 b aa1 k l z ) ((pos "Kj%" ) ))
( "d_ebris"  n ( d ei1 b r ii ) ((pos "L@%" ) ))
( "d_ebut"  n ( d ei1 b y uu ) ((pos "K6%" ) ))
( "d_ebutante"  n ( d e1 b y uu t aa n t ) ((pos "K6%" ) ))
( "d_ebutantes"  n ( d e1 b y uu t aa n t s ) ((pos "Kj%" ) ))
( "d_ebuts"  n ( d ei1 b y uu z ) ((pos "Kj%" ) ))
( "d_ecollet_e"  j ( d ei1 k o1 l t ei ) ((pos "OA$" ) ))
( "d_ecor"  n ( d ei1 k oo r ) ((pos "Ki%" ) ))
( "d_emarche"  n ( d ei1 m aa sh ) ((pos "K6$" ) ))
( "d_emarches"  n ( d ei1 m aa sh i z ) ((pos "Kj$" ) ))
( "d_emod_e"  j ( d ei2 m ou1 d ei ) ((pos "OA$" ) ))
( "d_enouement"  n ( d ei2 n uu1 m oo ng ) ((pos "K6%" ) ))
( "d_enouements"  n ( d ei2 n uu1 m oo ng z ) ((pos "Kj%" ) ))
( "d_eshabill_e"  n ( d ei2 z a b ii1 ei ) ((pos "L@%" ) ))
( "d_etente"  n ( d ei2 t aa1 n t ) ((pos "L@%" ) ))
( "da capo"  nil ( d aa2 - k aa1 p ou ) ((pos "Z-$" ) ))
( "dab"  v ( d a1 b ) ((pos "J4%" "K6%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "dabbed"  v ( d a1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "dabbing"  v ( d a1 b i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "dabble"  v ( d a1 b l ) ((pos "J2%" ) (subcat "2C" "3A" "6A" "15A" )))
( "dabbled"  v ( d a1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "dabbler"  n ( d a1 b l @ r ) ((pos "K6%" ) ))
( "dabblers"  n ( d a1 b l @ z ) ((pos "Kj%" ) ))
( "dabbles"  v ( d a1 b l z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "15A" )))
( "dabbling"  v ( d a1 b l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "dabs"  v ( d a1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "dace"  n ( d ei1 s ) ((pos "K9$" ) ))
( "dacha"  n ( d a1 ch @ ) ((pos "K6$" ) ))
( "dachas"  n ( d a1 ch @ z ) ((pos "Kj$" ) ))
( "dachshund"  n ( d a1 k s h u n d ) ((pos "K6%" ) ))
( "dachshunds"  n ( d a1 k s h u n d z ) ((pos "Kj%" ) ))
( "dacoit"  n ( d @1 k oi1 t ) ((pos "K6$" ) ))
( "dacoities"  n ( d @1 k oi1 t i z ) ((pos "Kj$" ) ))
( "dacoits"  n ( d @1 k oi1 t s ) ((pos "Kj$" ) ))
( "dacoity"  n ( d @1 k oi1 t ii ) ((pos "K8$" ) ))
( "dactyl"  n ( d a1 k t i l ) ((pos "K6$" ) ))
( "dactylic"  j ( d a1 k t i1 l i k ) ((pos "OA$" ) ))
( "dactyls"  n ( d a1 k t i l z ) ((pos "Kj$" ) ))
( "dad"  n ( d a1 d ) ((pos "K6%" ) ))
( "daddies"  n ( d a1 d i z ) ((pos "Kj%" ) ))
( "daddy"  n ( d a1 d ii ) ((pos "K8%" ) ))
( "daddy-longlegs"  n ( d a2 d i - l o1 ng l i g z ) ((pos "K9%" ) ))
( "dado"  n ( d ei1 d ou ) ((pos "K6$" ) ))
( "dados"  n ( d ei1 d ou z ) ((pos "Kj$" ) ))
( "dads"  n ( d a1 d z ) ((pos "Kj%" ) ))
( "daemon"  n ( d ii1 m @ n ) ((pos "K6$" ) ))
( "daemons"  n ( d ii1 m @ n z ) ((pos "Kj$" ) ))
( "daffodil"  n ( d a1 f @ d i l ) ((pos "K6%" ) ))
( "daffodils"  n ( d a1 f @ d i l z ) ((pos "Kj%" ) ))
( "daft"  j ( d aa1 f t ) ((pos "OC%" ) ))
( "dafter"  j ( d aa1 f t @ r ) ((pos "Or%" ) ))
( "daftest"  j ( d aa1 f t i s t ) ((pos "Os%" ) ))
( "daftly"  a ( d aa1 f t l ii ) ((pos "Pu$" ) ))
( "dagger"  n ( d a1 g @ r ) ((pos "K6%" ) ))
( "daggers"  n ( d a1 g @ z ) ((pos "Kj%" ) ))
( "dago"  n ( d ei1 g ou ) ((pos "K6$" ) ))
( "dagos"  n ( d ei1 g ou z ) ((pos "Kj$" ) ))
( "daguerreotype"  n ( d @1 g e1 r @ t ai p ) ((pos "K6$" ) ))
( "daguerreotypes"  n ( d @1 g e1 r @ t ai p s ) ((pos "Kj$" ) ))
( "dahlia"  n ( d ei1 l i@ ) ((pos "K6%" ) ))
( "dahlias"  n ( d ei1 l i@ z ) ((pos "Kj%" ) ))
( "dailies"  n ( d ei1 l i z ) ((pos "Kj%" ) ))
( "daily"  n ( d ei1 l ii ) ((pos "K8%" "OA%" "Pu%" ) ))
( "daintier"  j ( d ei1 n t i@ r ) ((pos "Or%" ) ))
( "dainties"  n ( d ei1 n t i z ) ((pos "Kj%" ) ))
( "daintiest"  j ( d ei1 n t i i s t ) ((pos "Os%" ) ))
( "daintily"  a ( d ei1 n t i l ii ) ((pos "Pu%" ) ))
( "daintiness"  n ( d ei1 n t i n @ s ) ((pos "L@%" ) ))
( "dainty"  n ( d ei1 n t ii ) ((pos "K8%" "OD%" ) ))
( "dairies"  n ( d e@1 r i z ) ((pos "Kj%" ) ))
( "dairy"  n ( d e@1 r ii ) ((pos "K8%" ) ))
( "dairy-farm"  n ( d e@1 r i - f aa m ) ((pos "K6%" ) ))
( "dairy-farming"  n ( d e@1 r i - f aa m i ng ) ((pos "L@%" ) ))
( "dairy-farms"  n ( d e@1 r i - f aa m z ) ((pos "Kj%" ) ))
( "dairying"  n ( d e@1 r i i ng ) ((pos "L@%" ) ))
( "dairymaid"  n ( d e@1 r i m ei d ) ((pos "K6%" ) ))
( "dairymaids"  n ( d e@1 r i m ei d z ) ((pos "Kj%" ) ))
( "dairyman"  n ( d e@1 r i m a n ) ((pos "Ki%" ) ))
( "dairymen"  n ( d e@1 r i m e n ) ((pos "Kj%" ) ))
( "dais"  n ( d ei1 i s ) ((pos "K7%" ) ))
( "daises"  n ( d ei1 i s i z ) ((pos "Kj$" ) ))
( "daisies"  n ( d ei1 z i z ) ((pos "Kj%" ) ))
( "daisy"  n ( d ei1 z ii ) ((pos "K8%" ) ))
( "dale"  n ( d ei1 l ) ((pos "K6%" ) ))
( "dales"  n ( d ei1 l z ) ((pos "Kj%" ) ))
( "dalesman"  n ( d ei1 l z m @ n ) ((pos "Ki%" ) ))
( "dalesmen"  n ( d ei1 l z m @ n ) ((pos "Kj%" ) ))
( "dalliance"  n ( d a1 l i@ n s ) ((pos "L@%" ) ))
( "dallied"  v ( d a1 l i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dallies"  v ( d a1 l i z ) ((pos "Ia%" ) (subcat "2A" )))
( "dally"  v ( d a1 l ii ) ((pos "I3%" ) (subcat "2A" )))
( "dallying"  v ( d a1 l i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "dalmatian"  n ( d a2 l m ei1 sh @ n ) ((pos "K6%" ) ))
( "dalmatians"  n ( d a2 l m ei1 sh @ n z ) ((pos "Kj%" ) ))
( "dam"  v ( d a1 m ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "damage"  v ( d a1 m i jh ) ((pos "H2%" "M6%" ) (subcat )))
( "damaged"  v ( d a1 m i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "damages"  v ( d a1 m i jh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "damaging"  v ( d a1 m i jh i ng ) ((pos "Hb%" ) (subcat )))
( "damascene"  v ( d a1 m @ s ii n ) ((pos "H2$" "OA$" ) ))
( "damascened"  v ( d a1 m @ s ii n d ) ((pos "Hc$" "Hd$" ) ))
( "damascenes"  v ( d a1 m @ s ii n z ) ((pos "Ha$" ) ))
( "damascening"  v ( d a1 m @ s ii n i ng ) ((pos "Hb$" ) ))
( "damask"  n ( d a1 m @ s k ) ((pos "L@%" ) ))
( "dame"  n ( d ei1 m ) ((pos "K6%" ) ))
( "dames"  n ( d ei1 m z ) ((pos "Kj%" ) ))
( "dammed"  v ( d a1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "damming"  v ( d a1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "damn"  v ( d a1 m ) ((pos "H0%" "Ki%" "OA%" "Pu%" ) (subcat )))
( "damnable"  j ( d a1 m n @ b l ) ((pos "OA%" ) ))
( "damnably"  a ( d a1 m n @ b l ii ) ((pos "Pu%" ) ))
( "damnation"  n ( d a1 m n ei1 sh @ n ) ((pos "L@%" ) ))
( "damned"  v ( d a1 m d ) ((pos "Hc%" "Hd%" "OA%" "Pu%" ) (subcat )))
( "damning"  v ( d a1 m i ng ) ((pos "Hb%" ) (subcat )))
( "damns"  v ( d a1 m z ) ((pos "Ha%" ) (subcat )))
( "damp"  v ( d a1 m p ) ((pos "J0%" "L@%" "OC%" ) (subcat "2C" "6A" )))
( "damped"  v ( d a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "dampen"  v ( d a1 m p @ n ) ((pos "J0%" ) ))
( "dampened"  v ( d a1 m p @ n d ) ((pos "Jc%" "Jd%" ) ))
( "dampening"  v ( d a1 m p @ n i ng ) ((pos "Jb%" ) ))
( "dampens"  v ( d a1 m p @ n z ) ((pos "Ja%" ) ))
( "damper"  n ( d a1 m p @ r ) ((pos "K6%" "Or%" ) ))
( "dampers"  n ( d a1 m p @ z ) ((pos "Kj%" ) ))
( "dampest"  j ( d a1 m p i s t ) ((pos "Os%" ) ))
( "damping"  v ( d a1 m p i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "dampish"  j ( d a1 m p i sh ) ((pos "OA%" ) ))
( "damply"  a ( d a1 m p l ii ) ((pos "Pu%" ) ))
( "dampness"  n ( d a1 m p n @ s ) ((pos "L@%" ) ))
( "damps"  v ( d a1 m p s ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "dams"  v ( d a1 m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "damsel"  n ( d a1 m z l ) ((pos "K6%" ) ))
( "damsels"  n ( d a1 m z l z ) ((pos "Kj%" ) ))
( "damson"  n ( d a1 m z @ n ) ((pos "K6%" ) ))
( "damsons"  n ( d a1 m z @ n z ) ((pos "Kj%" ) ))
( "dance"  v ( d aa1 n s ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "dance-band"  n ( d aa1 n s - b a n d ) ((pos "K6%" ) ))
( "dance-bands"  n ( d aa1 n s - b a n d z ) ((pos "Kj%" ) ))
( "dance-hall"  n ( d aa1 n s - h oo l ) ((pos "K6%" ) ))
( "dance-halls"  n ( d aa1 n s - h oo l z ) ((pos "Kj%" ) ))
( "dance-orchestra"  n ( d aa1 n s - oo k i s t r @ ) ((pos "K6%" ) ))
( "dance-orchestras"  n ( d aa1 n s - oo k i s t r @ z ) ((pos "Kj%" ) ))
( "danced"  v ( d aa1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "dancer"  n ( d aa1 n s @ r ) ((pos "K6%" ) ))
( "dancers"  n ( d aa1 n s @ z ) ((pos "Kj%" ) ))
( "dances"  v ( d aa1 n s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "dancing"  v ( d aa1 n s i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "dandelion"  n ( d a1 n d i l ai @ n ) ((pos "K6%" ) ))
( "dandelions"  n ( d a1 n d i l ai @ n z ) ((pos "Kj%" ) ))
( "dander"  n ( d a1 n d @ r ) ((pos "Ki$" ) ))
( "dandies"  n ( d a1 n d i z ) ((pos "Kj%" ) ))
( "dandified"  j ( d a1 n d i f ai d ) ((pos "OA%" ) ))
( "dandle"  v ( d a1 n d l ) ((pos "H2$" ) (subcat "6A" "15A" )))
( "dandled"  v ( d a1 n d l d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "15A" )))
( "dandles"  v ( d a1 n d l z ) ((pos "Ha$" ) (subcat "6A" "15A" )))
( "dandling"  v ( d a1 n d l i ng ) ((pos "Hb$" ) (subcat "6A" "15A" )))
( "dandruff"  n ( d a1 n d r uh f ) ((pos "L@%" ) ))
( "dandy"  n ( d a1 n d ii ) ((pos "K8%" "OA%" ) ))
( "danger"  n ( d ei1 n jh @ r ) ((pos "M6%" ) ))
( "dangerous"  j ( d ei1 n jh @ r @ s ) ((pos "OA%" ) ))
( "dangerously"  a ( d ei1 n jh @ r @ s l ii ) ((pos "Pu%" ) ))
( "dangers"  n ( d ei1 n jh @ z ) ((pos "Mj%" ) ))
( "dangle"  v ( d a1 ng g l ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "dangled"  v ( d a1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "dangles"  v ( d a1 ng g l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "dangling"  v ( d a1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "dank"  j ( d a1 ng k ) ((pos "OC%" ) ))
( "danker"  j ( d a1 ng k @ r ) ((pos "Or%" ) ))
( "dankest"  j ( d a1 ng k i s t ) ((pos "Os%" ) ))
( "danse macabre"  n ( d aa1 n s - m @ k aa1 b r @ ) ((pos "Ki%" ) ))
( "daphne"  n ( d a1 f n ii ) ((pos "K6$" ) ))
( "daphnes"  n ( d a1 f n i z ) ((pos "Kj$" ) ))
( "dapper"  j ( d a1 p @ r ) ((pos "OA%" ) ))
( "dapple"  v ( d a1 p l ) ((pos "H2%" ) (subcat )))
( "dapple-grey"  n ( d a2 p l - g r ei1 ) ((pos "K6%" "OA%" ) ))
( "dapple-greys"  n ( d a2 p l - g r ei1 z ) ((pos "Kj$" ) ))
( "dappled"  v ( d a1 p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dapples"  v ( d a1 p l z ) ((pos "Ha%" ) (subcat )))
( "dappling"  v ( d a1 p l i ng ) ((pos "Hb%" ) (subcat )))
( "dare"  v ( d e@ r ) ((pos "G2%" "J2%" "K6%" ) (subcat "4A" "5" "6A" )))
( "dared"  v ( d e@ d ) ((pos "Gc%" "Gd%" "Jc%" "Jd%" ) (subcat "4A" "5" "6A" )))
( "daredevil"  n ( d e@1 d e v @ l ) ((pos "K6%" ) ))
( "daredevils"  n ( d e@1 d e v @ l z ) ((pos "Kj%" ) ))
( "daren't"  v ( d e@ n t ) ((pos "Gg%" ) ))
( "dares"  v ( d e@ z ) ((pos "Ga%" "Ja%" "Kj%" ) (subcat "4A" "5" "6A" )))
( "daring"  v ( d e@1 r i ng ) ((pos "Gb%" "Jb%" "L@%" "OA%" ) (subcat "4A" "5" "6A" )))
( "daringly"  a ( d e@1 r i ng l ii ) ((pos "Pu%" ) ))
( "dark"  n ( d aa1 k ) ((pos "L@%" "OC%" ) ))
( "darken"  v ( d aa1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "darkened"  v ( d aa1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "darkening"  v ( d aa1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "darkens"  v ( d aa1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "darker"  j ( d aa1 k @ r ) ((pos "Or%" ) ))
( "darkest"  j ( d aa1 k i s t ) ((pos "Os%" ) ))
( "darkly"  a ( d aa1 k l ii ) ((pos "Pu%" ) ))
( "darkness"  n ( d aa1 k n @ s ) ((pos "L@%" ) ))
( "darkroom"  n ( d aa1 k r uu m ) ((pos "K6%" ) ))
( "darkrooms"  n ( d aa1 k r uu m z ) ((pos "Kj%" ) ))
( "darling"  n ( d aa1 l i ng ) ((pos "K6%" ) ))
( "darlings"  n ( d aa1 l i ng z ) ((pos "Kj%" ) ))
( "darn"  v ( d aa1 n ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "darned"  v ( d aa1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "darning"  v ( d aa1 n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "darning-needle"  n ( d aa1 n i ng - n ii d l ) ((pos "K6%" ) ))
( "darning-needles"  n ( d aa1 n i ng - n ii d l z ) ((pos "Kj%" ) ))
( "darns"  v ( d aa1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "dart"  v ( d aa1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "darted"  v ( d aa1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "darting"  v ( d aa1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "darts"  v ( d aa1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "dash"  v ( d a1 sh ) ((pos "J1%" "M7%" ) (subcat "2C" "6A" "15A" )))
( "dashboard"  n ( d a1 sh b oo d ) ((pos "K6%" ) ))
( "dashboards"  n ( d a1 sh b oo d z ) ((pos "Kj%" ) ))
( "dashed"  v ( d a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "dashes"  v ( d a1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" "15A" )))
( "dashing"  v ( d a1 sh i ng ) ((pos "Jb%" "OA%" ) (subcat "2C" "6A" "15A" )))
( "dashingly"  a ( d a1 sh i ng l ii ) ((pos "Pu%" ) ))
( "dastard"  n ( d a1 s t @ d ) ((pos "K6$" ) ))
( "dastardly"  j ( d a1 s t @ d l ii ) ((pos "OA%" ) ))
( "dastards"  n ( d a1 s t @ d z ) ((pos "Kj$" ) ))
( "data"  n ( d ei1 t @ ) ((pos "M9%" ) ))
( "datable"  j ( d ei1 t @ b l ) ((pos "OA%" ) ))
( "date"  v ( d ei1 t ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" )))
( "dated"  v ( d ei1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" )))
( "dateless"  j ( d ei1 t l @ s ) ((pos "OA%" ) ))
( "dateline"  n ( d ei1 t l ai n ) ((pos "K6%" ) ))
( "datelines"  n ( d ei1 t l ai n z ) ((pos "Kj$" ) ))
( "dates"  v ( d ei1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" )))
( "dating"  v ( d ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "dative"  n ( d ei1 t i v ) ((pos "K6%" "OA%" ) ))
( "datives"  n ( d ei1 t i v z ) ((pos "Kj$" ) ))
( "datum"  n ( d ei1 t @ m ) ((pos "Ki$" ) ))
( "daub"  v ( d oo1 b ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "14" "15A" )))
( "daubed"  v ( d oo1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15A" )))
( "dauber"  n ( d oo1 b @ r ) ((pos "K6%" ) ))
( "daubers"  n ( d oo1 b @ z ) ((pos "Kj%" ) ))
( "daubing"  v ( d oo1 b i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" "15A" )))
( "daubs"  v ( d oo1 b z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "14" "15A" )))
( "daughter"  n ( d oo1 t @ r ) ((pos "K6%" ) ))
( "daughter-in-law"  n ( d oo1 t @ r - i n - l oo ) ((pos "Ki%" ) ))
( "daughterly"  j ( d oo1 t @ l ii ) ((pos "OA%" ) ))
( "daughters"  n ( d oo1 t @ z ) ((pos "Kj%" ) ))
( "daughters-in-law"  n ( d oo1 t @ z - i n - l oo ) ((pos "Kj%" ) ))
( "daunt"  v ( d oo1 n t ) ((pos "H0$" ) (subcat )))
( "daunted"  v ( d oo1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "daunting"  v ( d oo1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "dauntless"  j ( d oo1 n t l i s ) ((pos "OA%" ) ))
( "dauntlessly"  a ( d oo1 n t l i s l ii ) ((pos "Pu%" ) ))
( "daunts"  v ( d oo1 n t s ) ((pos "Ha$" ) (subcat )))
( "dauphin"  n ( d oo1 f i n ) ((pos "K6$" ) ))
( "dauphins"  n ( d oo1 f i n z ) ((pos "Kj$" ) ))
( "davenport"  n ( d a1 v n p oo t ) ((pos "K6$" ) ))
( "davenports"  n ( d a1 v n p oo t s ) ((pos "Kj$" ) ))
( "davit"  n ( d a1 v i t ) ((pos "K6$" ) ))
( "davits"  n ( d a1 v i t s ) ((pos "Kj$" ) ))
( "daw"  n ( d oo1 ) ((pos "K6$" ) ))
( "dawdle"  v ( d oo1 d l ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "dawdled"  v ( d oo1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "dawdler"  n ( d oo1 d l @ r ) ((pos "K6%" ) ))
( "dawdlers"  n ( d oo1 d l @ z ) ((pos "Kj%" ) ))
( "dawdles"  v ( d oo1 d l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "dawdling"  v ( d oo1 d l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "dawn"  v ( d oo1 n ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "dawned"  v ( d oo1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dawning"  v ( d oo1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "dawns"  v ( d oo1 n z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "daws"  n ( d oo1 z ) ((pos "Kj$" ) ))
( "day"  n ( d ei1 ) ((pos "M6*" ) ))
( "day-boarder"  n ( d ei1 - b oo d @ r ) ((pos "K6%" ) ))
( "day-boarders"  n ( d ei1 - b oo d @ z ) ((pos "Kj%" ) ))
( "day-labourer"  n ( d ei1 - l ei b @ r @ r ) ((pos "K6%" ) ))
( "day-labourers"  n ( d ei1 - l ei b @ r @ z ) ((pos "Kj%" ) ))
( "day-return"  n ( d ei2 - r i t @@1 n ) ((pos "K6%" ) ))
( "day-returns"  n ( d ei2 - r i t @@1 n z ) ((pos "Kj%" ) ))
( "day-school"  n ( d ei1 - s k uu l ) ((pos "K6%" ) ))
( "day-schools"  n ( d ei1 - s k uu l z ) ((pos "Kj%" ) ))
( "daybook"  n ( d ei1 b u k ) ((pos "K6%" ) ))
( "daybooks"  n ( d ei1 b u k s ) ((pos "Kj%" ) ))
( "dayboy"  n ( d ei1 b oi ) ((pos "K6%" ) ))
( "dayboys"  n ( d ei1 b oi z ) ((pos "Kj%" ) ))
( "daybreak"  n ( d ei1 b r ei k ) ((pos "K6%" ) ))
( "daybreaks"  n ( d ei1 b r ei k s ) ((pos "Kj%" ) ))
( "daydream"  v ( d ei1 d r ii m ) ((pos "I0%" "K6%" ) ))
( "daydreamed"  v ( d ei1 d r ii m d ) ((pos "Ic%" "Id%" ) ))
( "daydreaming"  v ( d ei1 d r ii m i ng ) ((pos "Ib%" ) ))
( "daydreams"  v ( d ei1 d r ii m z ) ((pos "Ia%" "Kj%" ) ))
( "daygirl"  n ( d ei1 g @@ l ) ((pos "K6%" ) ))
( "daygirls"  n ( d ei1 g @@ l z ) ((pos "Kj%" ) ))
( "daylight"  n ( d ei1 l ai t ) ((pos "L@%" ) ))
( "daylight-saving"  n ( d ei2 l ai t - s ei1 v i ng ) ((pos "L@%" "OA%" ) ))
( "daylong"  j ( d ei1 l o ng ) ((pos "OA%" "Pu%" ) ))
( "days"  n ( d ei1 z ) ((pos "Mj*" ) ))
( "dayspring"  n ( d ei1 s p r i ng ) ((pos "K6$" ) ))
( "daysprings"  n ( d ei1 s p r i ng z ) ((pos "Kj$" ) ))
( "daytime"  n ( d ei1 t ai m ) ((pos "K6%" ) ))
( "daytimes"  n ( d ei1 t ai m z ) ((pos "Kj%" ) ))
( "daze"  v ( d ei1 z ) ((pos "H2%" "Ki%" ) (subcat )))
( "dazed"  v ( d ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dazedly"  a ( d ei1 z i d l ii ) ((pos "Pu%" ) ))
( "dazes"  v ( d ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "dazing"  v ( d ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "dazzle"  v ( d a1 z l ) ((pos "H2%" "L@%" ) (subcat )))
( "dazzled"  v ( d a1 z l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dazzles"  v ( d a1 z l z ) ((pos "Ha%" ) (subcat )))
( "dazzling"  v ( d a1 z l i ng ) ((pos "Hb%" ) (subcat )))
( "de facto"  j ( d ei2 - f a1 k t ou ) ((pos "OA%" "Pu%" ) ))
( "de jure"  j ( d ei2 - jh u@1 r ii ) ((pos "OA%" "Pu%" ) ))
( "de luxe"  j ( d i1 - l uh1 k s ) ((pos "OA%" ) ))
( "de rigeur"  j ( d @1 - r i g @@1 r ) ((pos "Op%" ) ))
( "de trop"  j ( d @1 - t r ou1 ) ((pos "Op%" ) ))
( "de-escalate"  v ( d ii2 - e1 s k @ l ei t ) ((pos "H2%" ) (subcat )))
( "de-escalated"  v ( d ii2 - e1 s k @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "de-escalates"  v ( d ii2 - e1 s k @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "de-escalating"  v ( d ii2 - e1 s k @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "de-escalation"  n ( d ii1 - e2 s k @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "de-escalations"  n ( d ii1 - e2 s k @ l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "de-ice"  v ( d ii2 - ai1 s ) ((pos "H2%" ) (subcat )))
( "de-iced"  v ( d ii2 - ai1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "de-ices"  v ( d ii2 - ai1 s i z ) ((pos "Ha%" ) (subcat )))
( "de-icing"  v ( d ii2 - ai1 s i ng ) ((pos "Hb%" ) (subcat )))
( "deacon"  n ( d ii1 k @ n ) ((pos "K6%" ) ))
( "deaconess"  n ( d ii1 k @ n e s ) ((pos "K7%" ) ))
( "deaconesses"  n ( d ii1 k @ n e s i z ) ((pos "Kj%" ) ))
( "deacons"  n ( d ii1 k @ n z ) ((pos "Kj%" ) ))
( "dead"  j ( d e1 d ) ((pos "OA%" "Pu%" ) ))
( "deaden"  v ( d e1 d n ) ((pos "H0%" ) (subcat )))
( "deadened"  v ( d e1 d n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deadening"  v ( d e1 d n i ng ) ((pos "Hb%" ) (subcat )))
( "deadens"  v ( d e1 d n z ) ((pos "Ha%" ) (subcat )))
( "deadlier"  j ( d e1 d l i@ r ) ((pos "Or%" ) ))
( "deadliest"  j ( d e1 d l i i s t ) ((pos "Os%" ) ))
( "deadline"  n ( d e1 d l ai n ) ((pos "K6%" ) ))
( "deadlines"  n ( d e1 d l ai n z ) ((pos "Kj%" ) ))
( "deadlock"  n ( d e1 d l o k ) ((pos "M6%" ) ))
( "deadlocked"  j ( d e1 d l o k t ) ((pos "OA%" ) ))
( "deadlocks"  n ( d e1 d l o k s ) ((pos "Mj$" ) ))
( "deadly"  j ( d e1 d l ii ) ((pos "OD%" "Pu%" ) ))
( "deadpan"  j ( d e1 d p a n ) ((pos "OA%" ) ))
( "deaf"  j ( d e1 f ) ((pos "OC%" ) ))
( "deaf mute"  n ( d e2 f - m y uu1 t ) ((pos "K6%" ) ))
( "deaf mutes"  n ( d e2 f - m y uu1 t s ) ((pos "Kj%" ) ))
( "deaf-aid"  n ( d e1 f - ei d ) ((pos "K6%" ) ))
( "deaf-aids"  n ( d e1 f - ei d z ) ((pos "Kj%" ) ))
( "deafen"  v ( d e1 f n ) ((pos "H0%" ) (subcat )))
( "deafened"  v ( d e1 f n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deafening"  v ( d e1 f n i ng ) ((pos "Hb%" ) (subcat )))
( "deafens"  v ( d e1 f n z ) ((pos "Ha%" ) (subcat )))
( "deafer"  j ( d e1 f @ r ) ((pos "Or%" ) ))
( "deafest"  j ( d e1 f i s t ) ((pos "Os%" ) ))
( "deafness"  n ( d e1 f n @ s ) ((pos "L@%" ) ))
( "deal"  v ( d ii1 l ) ((pos "J5%" "K6%" "Pu%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "dealer"  n ( d ii1 l @ r ) ((pos "K6%" ) ))
( "dealers"  n ( d ii1 l @ z ) ((pos "Kj%" ) ))
( "dealing"  v ( d ii1 l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "dealings"  n ( d ii1 l i ng z ) ((pos "Mj%" ) ))
( "deals"  v ( d ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "dealt"  v ( d e1 l t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "dean"  n ( d ii1 n ) ((pos "K6%" ) ))
( "deaneries"  n ( d ii1 n @ r i z ) ((pos "Kj$" ) ))
( "deanery"  n ( d ii1 n @ r ii ) ((pos "K8%" ) ))
( "deans"  n ( d ii1 n z ) ((pos "Kj%" ) ))
( "dear"  n ( d i@1 r ) ((pos "K6%" "OC%" "Pu%" "W-%" ) ))
( "dearer"  j ( d i@1 r @ r ) ((pos "Or%" ) ))
( "dearest"  n ( d i@1 r i s t ) ((pos "Ki%" "Os%" ) ))
( "dearie"  n ( d i@1 r ii ) ((pos "K6%" ) ))
( "dearies"  n ( d i@1 r i z ) ((pos "Kj%" ) ))
( "dearly"  a ( d i@1 l ii ) ((pos "Pu%" ) ))
( "dearness"  n ( d i@1 n @ s ) ((pos "L@%" ) ))
( "dears"  n ( d i@1 z ) ((pos "Kj%" ) ))
( "dearth"  n ( d @@1 th ) ((pos "K6%" ) ))
( "dearths"  n ( d @@1 th s ) ((pos "Kj$" ) ))
( "deary"  n ( d i@1 r ii ) ((pos "K8$" ) ))
( "death"  n ( d e1 th ) ((pos "M6%" ) ))
( "death-duties"  n ( d e1 th - d y uu t i z ) ((pos "Kj%" ) ))
( "death-mask"  n ( d e1 th - m aa s k ) ((pos "K6%" ) ))
( "death-masks"  n ( d e1 th - m aa s k s ) ((pos "Kj%" ) ))
( "death-rate"  n ( d e1 th - r ei t ) ((pos "K6%" ) ))
( "death-rates"  n ( d e1 th - r ei t s ) ((pos "Kj$" ) ))
( "death-roll"  n ( d e1 th - r ou l ) ((pos "K6%" ) ))
( "death-rolls"  n ( d e1 th - r ou l z ) ((pos "Kj$" ) ))
( "death-warrant"  n ( d e1 th - w o r @ n t ) ((pos "K6%" ) ))
( "death-warrants"  n ( d e1 th - w o r @ n t s ) ((pos "Kj$" ) ))
( "deathbed"  n ( d e1 th b e d ) ((pos "K6%" ) ))
( "deathbeds"  n ( d e1 th b e d z ) ((pos "Kj$" ) ))
( "deathblow"  n ( d e1 th b l ou ) ((pos "K6%" ) ))
( "deathblows"  n ( d e1 th b l ou z ) ((pos "Kj%" ) ))
( "deathless"  j ( d e1 th l i s ) ((pos "OA%" ) ))
( "deathlike"  j ( d e1 th l ai k ) ((pos "OA%" ) ))
( "deathly"  j ( d e1 th l ii ) ((pos "OA%" "Pu%" ) ))
( "deaths"  n ( d e1 th s ) ((pos "Mj%" ) ))
( "deathtrap"  n ( d e1 th t r a p ) ((pos "K6%" ) ))
( "deathtraps"  n ( d e1 th t r a p s ) ((pos "Kj%" ) ))
( "deb"  n ( d e1 b ) ((pos "K6%" ) ))
( "debag"  v ( d ii2 b a1 g ) ((pos "H4%" ) (subcat )))
( "debagged"  v ( d ii2 b a1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debagging"  v ( d ii2 b a1 g i ng ) ((pos "Hb%" ) (subcat )))
( "debags"  v ( d ii2 b a1 g z ) ((pos "Ha%" ) (subcat )))
( "debar"  v ( d i1 b aa1 r ) ((pos "H4%" ) (subcat )))
( "debark"  v ( d i1 b aa1 k ) ((pos "J0$" ) ))
( "debarkation"  n ( d ii2 b aa k ei1 sh @ n ) ((pos "K6$" ) ))
( "debarkations"  n ( d ii2 b aa k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "debarked"  v ( d i1 b aa1 k t ) ((pos "Jc$" "Jd$" ) ))
( "debarking"  v ( d i1 b aa1 k i ng ) ((pos "Jb$" ) ))
( "debarks"  v ( d i1 b aa1 k s ) ((pos "Ja$" ) ))
( "debarred"  v ( d i1 b aa1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debarring"  v ( d i1 b aa1 r i ng ) ((pos "Hb%" ) (subcat )))
( "debars"  v ( d i1 b aa1 z ) ((pos "Ha%" ) (subcat )))
( "debase"  v ( d i1 b ei1 s ) ((pos "H2%" ) (subcat )))
( "debased"  v ( d i1 b ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debasement"  n ( d i1 b ei1 s m @ n t ) ((pos "K6%" ) ))
( "debasements"  n ( d i1 b ei1 s m @ n t s ) ((pos "Kj$" ) ))
( "debases"  v ( d i1 b ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "debasing"  v ( d i1 b ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "debatable"  j ( d i1 b ei1 t @ b l ) ((pos "OA%" ) ))
( "debate"  v ( d i1 b ei1 t ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" "8" )))
( "debated"  v ( d i1 b ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "8" )))
( "debater"  n ( d i1 b ei1 t @ r ) ((pos "K6%" ) ))
( "debaters"  n ( d i1 b ei1 t @ z ) ((pos "Kj%" ) ))
( "debates"  v ( d i1 b ei1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "8" )))
( "debating"  v ( d i1 b ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "8" )))
( "debauch"  v ( d i1 b oo1 ch ) ((pos "H1%" "K7%" ) (subcat )))
( "debauched"  v ( d i1 b oo1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debauchee"  n ( d i1 b oo2 ch ii1 ) ((pos "K6%" ) ))
( "debauchees"  n ( d i1 b oo2 ch ii1 z ) ((pos "Kj%" ) ))
( "debaucheries"  n ( d i1 b oo1 ch @ r i z ) ((pos "Mj$" ) ))
( "debauchery"  n ( d i1 b oo1 ch @ r ii ) ((pos "M8%" ) ))
( "debauches"  v ( d i1 b oo1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "debauching"  v ( d i1 b oo1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "debenture"  n ( d i1 b e1 n ch @ r ) ((pos "K6$" ) ))
( "debentures"  n ( d i1 b e1 n ch @ z ) ((pos "Kj$" ) ))
( "debilitate"  v ( d i1 b i1 l i t ei t ) ((pos "H2%" ) (subcat )))
( "debilitated"  v ( d i1 b i1 l i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debilitates"  v ( d i1 b i1 l i t ei t s ) ((pos "Ha%" ) (subcat )))
( "debilitating"  v ( d i1 b i1 l i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "debility"  n ( d i1 b i1 l i t ii ) ((pos "L@%" ) ))
( "debit"  v ( d e1 b i t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "debit-side"  n ( d e1 b i t - s ai d ) ((pos "K6%" ) ))
( "debit-sides"  n ( d e1 b i t - s ai d z ) ((pos "Kj$" ) ))
( "debited"  v ( d e1 b i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "debiting"  v ( d e1 b i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "debits"  v ( d e1 b i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "debonair"  j ( d e2 b @ n e@1 r ) ((pos "OA%" ) ))
( "debouch"  v ( d i1 b au1 ch ) ((pos "J1$" ) (subcat "2A" )))
( "debouched"  v ( d i1 b au1 ch t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "debouches"  v ( d i1 b au1 ch i z ) ((pos "Ja$" ) (subcat "2A" )))
( "debouching"  v ( d i1 b au1 ch i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "debrief"  v ( d ii2 b r ii1 f ) ((pos "H0%" ) (subcat )))
( "debriefed"  v ( d ii2 b r ii1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debriefing"  v ( d ii2 b r ii1 f i ng ) ((pos "Hb%" ) (subcat )))
( "debriefs"  v ( d ii2 b r ii1 f s ) ((pos "Ha%" ) (subcat )))
( "debris"  n ( d ei1 b r ii ) ((pos "L@%" ) ))
( "debs"  n ( d e1 b z ) ((pos "Kj%" ) ))
( "debt"  n ( d e1 t ) ((pos "M6%" ) ))
( "debtor"  n ( d e1 t @ r ) ((pos "K6%" ) ))
( "debtors"  n ( d e1 t @ z ) ((pos "Kj%" ) ))
( "debts"  n ( d e1 t s ) ((pos "Mj%" ) ))
( "debug"  v ( d ii2 b uh1 g ) ((pos "H4%" ) (subcat )))
( "debugged"  v ( d ii2 b uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debugging"  v ( d ii2 b uh1 g i ng ) ((pos "Hb%" ) (subcat )))
( "debugs"  v ( d ii2 b uh1 g z ) ((pos "Ha%" ) (subcat )))
( "debunk"  v ( d ii1 b uh1 ng k ) ((pos "H0%" ) (subcat )))
( "debunked"  v ( d ii1 b uh1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "debunking"  v ( d ii1 b uh1 ng k i ng ) ((pos "Hb%" ) (subcat )))
( "debunks"  v ( d ii1 b uh1 ng k s ) ((pos "Ha%" ) (subcat )))
( "debut"  n ( d ei1 b y uu ) ((pos "K6%" ) ))
( "debutante"  n ( d e1 b y uu t aa n t ) ((pos "K6%" ) ))
( "debutantes"  n ( d e1 b y uu t aa n t s ) ((pos "Kj%" ) ))
( "debuts"  n ( d ei1 b y uu z ) ((pos "Kj%" ) ))
( "deca-"  j ( d e2 k @ - ) ((pos "U-%" ) ))
( "decade"  n ( d e1 k ei d ) ((pos "K6%" ) ))
( "decadence"  n ( d e1 k @ d @ n s ) ((pos "L@%" ) ))
( "decadent"  n ( d e1 k @ d @ n t ) ((pos "K6$" "OA%" ) ))
( "decadents"  n ( d e1 k @ d @ n t s ) ((pos "Kj$" ) ))
( "decades"  n ( d e1 k ei d z ) ((pos "Kj%" ) ))
( "decamp"  v ( d i1 k a1 m p ) ((pos "I0%" ) (subcat "2A" )))
( "decamped"  v ( d i1 k a1 m p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "decamping"  v ( d i1 k a1 m p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "decamps"  v ( d i1 k a1 m p s ) ((pos "Ia%" ) (subcat "2A" )))
( "decant"  v ( d i1 k a1 n t ) ((pos "H0%" ) (subcat )))
( "decanted"  v ( d i1 k a1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decanter"  n ( d i1 k a1 n t @ r ) ((pos "K6%" ) ))
( "decanters"  n ( d i1 k a1 n t @ z ) ((pos "Kj%" ) ))
( "decanting"  v ( d i1 k a1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "decants"  v ( d i1 k a1 n t s ) ((pos "Ha%" ) (subcat )))
( "decapitate"  v ( d i1 k a1 p i t ei t ) ((pos "H2%" ) (subcat )))
( "decapitated"  v ( d i1 k a1 p i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decapitates"  v ( d i1 k a1 p i t ei t s ) ((pos "Ha%" ) (subcat )))
( "decapitating"  v ( d i1 k a1 p i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "decapitation"  n ( d i1 k a2 p i t ei1 sh @ n ) ((pos "K6$" ) ))
( "decapitations"  n ( d i1 k a2 p i t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "decarbonize"  v ( d ii2 k aa1 b @ n ai z ) ((pos "H2$" ) ))
( "decarbonized"  v ( d ii2 k aa1 b @ n ai z d ) ((pos "Hc$" "Hd$" ) ))
( "decarbonizes"  v ( d ii2 k aa1 b @ n ai z i z ) ((pos "Ha$" ) ))
( "decarbonizing"  v ( d ii2 k aa1 b @ n ai z i ng ) ((pos "Hb$" ) ))
( "decasyllabic"  j ( d e2 k @ s i l a1 b i k ) ((pos "OA$" ) ))
( "decasyllable"  n ( d e1 k @ s i l @ b l ) ((pos "K6$" ) ))
( "decasyllables"  n ( d e1 k @ s i l @ b l z ) ((pos "Kj$" ) ))
( "decay"  v ( d i1 k ei1 ) ((pos "I0%" "L@%" ) (subcat )))
( "decayed"  v ( d i1 k ei1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "decaying"  v ( d i1 k ei1 i ng ) ((pos "Ib%" ) (subcat )))
( "decays"  v ( d i1 k ei1 z ) ((pos "Ia%" ) (subcat )))
( "decease"  v ( d i1 s ii1 s ) ((pos "I2%" "L@$" ) ))
( "deceased"  v ( d i1 s ii1 s t ) ((pos "Ic%" "Id%" ) ))
( "deceases"  v ( d i1 s ii1 s i z ) ((pos "Ia$" ) ))
( "deceasing"  v ( d i1 s ii1 s i ng ) ((pos "Ib$" ) ))
( "deceit"  n ( d i1 s ii1 t ) ((pos "M6%" ) ))
( "deceitful"  j ( d i1 s ii1 t f u l ) ((pos "OA%" ) ))
( "deceitfully"  a ( d i1 s ii1 t f u l ii ) ((pos "Pu%" ) ))
( "deceitfulness"  n ( d i1 s ii1 t f u l n @ s ) ((pos "L@%" ) ))
( "deceits"  n ( d i1 s ii1 t s ) ((pos "Mj%" ) ))
( "deceive"  v ( d i1 s ii1 v ) ((pos "H2%" ) (subcat "6A" )))
( "deceived"  v ( d i1 s ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deceiver"  n ( d i1 s ii1 v @ r ) ((pos "K6%" ) ))
( "deceivers"  n ( d i1 s ii1 v @ z ) ((pos "Kj%" ) ))
( "deceives"  v ( d i1 s ii1 v z ) ((pos "Ha%" ) (subcat "6A" )))
( "deceiving"  v ( d i1 s ii1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deceivingly"  a ( d i1 s ii1 v i ng l ii ) ((pos "Pu%" ) ))
( "decelerate"  v ( d ii2 s e1 l @ r ei t ) ((pos "J2%" ) ))
( "decelerated"  v ( d ii2 s e1 l @ r ei t i d ) ((pos "Jc%" "Jd%" ) ))
( "decelerates"  v ( d ii2 s e1 l @ r ei t s ) ((pos "Ja%" ) ))
( "decelerating"  v ( d ii2 s e1 l @ r ei t i ng ) ((pos "Jb%" ) ))
( "decencies"  n ( d ii1 s n s i z ) ((pos "Mj%" ) ))
( "decency"  n ( d ii1 s n s ii ) ((pos "M8%" ) ))
( "decent"  j ( d ii1 s n t ) ((pos "OA%" ) ))
( "decently"  a ( d ii1 s n t l ii ) ((pos "Pu%" ) ))
( "decentralization"  n ( d ii2 s e2 n t r @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "decentralize"  v ( d ii2 s e1 n t r @ l ai z ) ((pos "H2%" ) (subcat )))
( "decentralized"  v ( d ii2 s e1 n t r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decentralizes"  v ( d ii2 s e1 n t r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "decentralizing"  v ( d ii2 s e1 n t r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "deception"  n ( d i1 s e1 p sh @ n ) ((pos "M6%" ) ))
( "deceptions"  n ( d i1 s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "deceptive"  j ( d i1 s e1 p t i v ) ((pos "OA%" ) ))
( "deceptively"  a ( d i1 s e1 p t i v l ii ) ((pos "Pu%" ) ))
( "deci-"  j ( d e2 s i - ) ((pos "U-%" ) ))
( "decibel"  n ( d e1 s i b e l ) ((pos "K6%" ) ))
( "decibels"  n ( d e1 s i b e l z ) ((pos "Kj%" ) ))
( "decide"  v ( d i1 s ai1 d ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "14" )))
( "decided"  v ( d i1 s ai1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "14" )))
( "decidedly"  a ( d i1 s ai1 d i d l ii ) ((pos "Pu%" ) ))
( "decides"  v ( d i1 s ai1 d z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "14" )))
( "deciding"  v ( d i1 s ai1 d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "14" )))
( "deciduous"  j ( d i1 s i1 d y u@ s ) ((pos "OA%" ) ))
( "decimal"  j ( d e1 s i m l ) ((pos "OA%" ) ))
( "decimalization"  n ( d e2 s i m @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "decimalize"  v ( d e1 s i m @ l ai z ) ((pos "H2%" ) (subcat )))
( "decimalized"  v ( d e1 s i m @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decimalizes"  v ( d e1 s i m @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "decimalizing"  v ( d e1 s i m @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "decimals"  n ( d e1 s i m l z ) ((pos "Kj%" ) ))
( "decimate"  v ( d e1 s i m ei t ) ((pos "H2%" ) (subcat )))
( "decimated"  v ( d e1 s i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decimates"  v ( d e1 s i m ei t s ) ((pos "Ha%" ) (subcat )))
( "decimating"  v ( d e1 s i m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "decipher"  v ( d i1 s ai1 f @ r ) ((pos "H0%" ) (subcat )))
( "decipherable"  j ( d i1 s ai1 f r @ b l ) ((pos "OA%" ) ))
( "deciphered"  v ( d i1 s ai1 f @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deciphering"  v ( d i1 s ai1 f @ r i ng ) ((pos "Hb%" ) (subcat )))
( "deciphers"  v ( d i1 s ai1 f @ z ) ((pos "Ha%" ) (subcat )))
( "decision"  n ( d i1 s i1 zh n ) ((pos "M6%" ) ))
( "decisions"  n ( d i1 s i1 zh n z ) ((pos "Mj%" ) ))
( "decisive"  j ( d i1 s ai1 s i v ) ((pos "OA%" ) ))
( "decisively"  a ( d i1 s ai1 s i v l ii ) ((pos "Pu%" ) ))
( "deck"  v ( d e1 k ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "decked"  v ( d e1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "decker"  n ( d e1 k @ r ) ((pos "K6%" ) ))
( "deckers"  n ( d e1 k @ z ) ((pos "Kj%" ) ))
( "decking"  v ( d e1 k i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "deckle-edged"  j ( d e2 k l - e1 jh d ) ((pos "OA%" ) ))
( "decks"  v ( d e1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "declaim"  v ( d i1 k l ei1 m ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "declaimed"  v ( d i1 k l ei1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "declaiming"  v ( d i1 k l ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "declaims"  v ( d i1 k l ei1 m z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "declamation"  n ( d e2 k l @ m ei1 sh @ n ) ((pos "M6$" ) ))
( "declamations"  n ( d e2 k l @ m ei1 sh @ n z ) ((pos "Mj$" ) ))
( "declamatory"  j ( d i1 k l a1 m @ t @ r ii ) ((pos "OA%" ) ))
( "declarable"  j ( d i1 k l e@1 r @ b l ) ((pos "OA%" ) ))
( "declaration"  n ( d e2 k l @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "declarations"  n ( d e2 k l @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "declare"  v ( d i1 k l e@1 r ) ((pos "J2%" ) (subcat "3A" "6A" "9" "14" )))
( "declared"  v ( d i1 k l e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" "14" )))
( "declares"  v ( d i1 k l e@1 z ) ((pos "Ja%" ) (subcat "3A" "6A" "9" "14" )))
( "declaring"  v ( d i1 k l e@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "9" "14" )))
( "declassification"  n ( d ii2 k l a2 s i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "declassifications"  n ( d ii2 k l a2 s i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "declassified"  v ( d ii2 k l a1 s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "declassifies"  v ( d ii2 k l a1 s i f ai z ) ((pos "Ha%" ) (subcat )))
( "declassify"  v ( d ii2 k l a1 s i f ai ) ((pos "H3%" ) (subcat )))
( "declassifying"  v ( d ii2 k l a1 s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "declension"  n ( d i1 k l e1 n sh @ n ) ((pos "M6%" ) ))
( "declensions"  n ( d i1 k l e1 n sh @ n z ) ((pos "Mj%" ) ))
( "declination"  n ( d e2 k l i n ei1 sh @ n ) ((pos "K6$" ) ))
( "declinations"  n ( d e2 k l i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "decline"  v ( d i1 k l ai1 n ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "declined"  v ( d i1 k l ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "declines"  v ( d i1 k l ai1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "declining"  v ( d i1 k l ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "declivities"  n ( d i1 k l i1 v i t i z ) ((pos "Kj$" ) ))
( "declivity"  n ( d i1 k l i1 v i t ii ) ((pos "K8$" ) ))
( "declutch"  v ( d ii2 k l uh1 ch ) ((pos "I1%" ) (subcat )))
( "declutched"  v ( d ii2 k l uh1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "declutches"  v ( d ii2 k l uh1 ch i z ) ((pos "Ia%" ) (subcat )))
( "declutching"  v ( d ii2 k l uh1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "decode"  v ( d ii2 k ou1 d ) ((pos "H2%" ) (subcat )))
( "decoded"  v ( d ii2 k ou1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decoder"  n ( d ii2 k ou1 d @ r ) ((pos "K6%" ) ))
( "decoders"  n ( d ii2 k ou1 d @ z ) ((pos "Kj%" ) ))
( "decodes"  v ( d ii2 k ou1 d z ) ((pos "Ha%" ) (subcat )))
( "decoding"  v ( d ii2 k ou1 d i ng ) ((pos "Hb%" ) (subcat )))
( "decoke"  v ( d ii2 k ou1 k ) ((pos "H2%" ) ))
( "decoked"  v ( d ii2 k ou1 k t ) ((pos "Hc%" "Hd%" ) ))
( "decokes"  v ( d ii2 k ou1 k s ) ((pos "Ha%" ) ))
( "decoking"  v ( d ii2 k ou1 k i ng ) ((pos "Hb%" ) ))
( "decolonization"  n ( d ii2 k o2 l @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "decolonize"  v ( d ii2 k o1 l @ n ai z ) ((pos "H2%" ) (subcat )))
( "decolonized"  v ( d ii2 k o1 l @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decolonizes"  v ( d ii2 k o1 l @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "decolonizing"  v ( d ii2 k o1 l @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "decompose"  v ( d ii2 k @ m p ou1 z ) ((pos "J2%" ) (subcat "2A" )))
( "decomposed"  v ( d ii2 k @ m p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "decomposes"  v ( d ii2 k @ m p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "decomposing"  v ( d ii2 k @ m p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "decomposition"  n ( d ii2 k o m p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "decompositions"  n ( d ii2 k o m p @ z i1 sh @ n z ) ((pos "Mj$" ) ))
( "decompress"  v ( d ii2 k @ m p r e1 s ) ((pos "H1%" ) (subcat )))
( "decompressed"  v ( d ii2 k @ m p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decompresses"  v ( d ii2 k @ m p r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "decompressing"  v ( d ii2 k @ m p r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "decompression"  n ( d ii2 k @ m p r e1 sh @ n ) ((pos "K6%" ) ))
( "decompressions"  n ( d ii2 k @ m p r e1 sh @ n z ) ((pos "Kj$" ) ))
( "decontaminate"  v ( d ii2 k @ n t a1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "decontaminated"  v ( d ii2 k @ n t a1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "decontaminates"  v ( d ii2 k @ n t a1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "decontaminating"  v ( d ii2 k @ n t a1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "decontamination"  n ( d ii2 k @ n t a2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "decontrol"  v ( d ii2 k @ n t r ou1 l ) ((pos "H4$" ) (subcat )))
( "decontrolled"  v ( d ii2 k @ n t r ou1 l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "decontrolling"  v ( d ii2 k @ n t r ou1 l i ng ) ((pos "Hb$" ) (subcat )))
( "decontrols"  v ( d ii2 k @ n t r ou1 l z ) ((pos "Ha$" ) (subcat )))
( "decorate"  v ( d e1 k @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "decorated"  v ( d e1 k @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "decorates"  v ( d e1 k @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "decorating"  v ( d e1 k @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "decoration"  n ( d e2 k @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "decorations"  n ( d e2 k @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "decorative"  j ( d e1 k @ r @ t i v ) ((pos "OA%" ) ))
( "decorator"  n ( d e1 k @ r ei t @ r ) ((pos "K6%" ) ))
( "decorators"  n ( d e1 k @ r ei t @ z ) ((pos "Kj%" ) ))
( "decorous"  j ( d e1 k @ r @ s ) ((pos "OA%" ) ))
( "decorously"  a ( d e1 k @ r @ s l ii ) ((pos "Pu%" ) ))
( "decorum"  n ( d i1 k oo1 r @ m ) ((pos "L@%" ) ))
( "decoy"  n ( d ii1 k oi ) ((pos "K6%" ) ))
( "decoy"  v ( d i1 k oi1 ) ((pos "H0%" ) (subcat "6A" )))
( "decoyed"  v ( d i1 k oi1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "decoying"  v ( d i1 k oi1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "decoys"  n ( d ii1 k oi z ) ((pos "Kj%" ) ))
( "decoys"  v ( d i1 k oi1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "decrease"  n ( d ii1 k r ii s ) ((pos "M6%" ) ))
( "decrease"  v ( d i1 k r ii1 s ) ((pos "J2%" ) (subcat "2A" )))
( "decreased"  v ( d i1 k r ii1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "decreases"  n ( d ii1 k r ii s i z ) ((pos "Mj%" ) ))
( "decreases"  v ( d i1 k r ii1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "decreasing"  v ( d i1 k r ii1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "decree"  v ( d i1 k r ii1 ) ((pos "H5%" "K6%" ) (subcat "6A" )))
( "decree nisi"  n ( d i1 k r ii2 - n ai1 s ai ) ((pos "Ki%" ) ))
( "decreed"  v ( d i1 k r ii1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "decreeing"  v ( d i1 k r ii1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "decrees"  v ( d i1 k r ii1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "decrepit"  j ( d i1 k r e1 p i t ) ((pos "OA%" ) ))
( "decrepitude"  n ( d i1 k r e1 p i t y uu d ) ((pos "L@$" ) ))
( "decried"  v ( d i1 k r ai1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "decries"  v ( d i1 k r ai1 z ) ((pos "Ha$" ) (subcat )))
( "decry"  v ( d i1 k r ai1 ) ((pos "H3$" ) (subcat )))
( "decrying"  v ( d i1 k r ai1 i ng ) ((pos "Hb$" ) (subcat )))
( "dedicate"  v ( d e1 d i k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "dedicated"  v ( d e1 d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dedicates"  v ( d e1 d i k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "dedicating"  v ( d e1 d i k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dedication"  n ( d e2 d i k ei1 sh @ n ) ((pos "M6%" ) ))
( "dedications"  n ( d e2 d i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "deduce"  v ( d i1 d y uu1 s ) ((pos "H2%" ) (subcat "6A" "9" )))
( "deduced"  v ( d i1 d y uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "deduces"  v ( d i1 d y uu1 s i z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "deducing"  v ( d i1 d y uu1 s i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "deduct"  v ( d i1 d uh1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "deducted"  v ( d i1 d uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deductible"  j ( d i1 d uh1 k t @ b l ) ((pos "OA%" ) ))
( "deducting"  v ( d i1 d uh1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deduction"  n ( d i1 d uh1 k sh @ n ) ((pos "M6%" ) ))
( "deductions"  n ( d i1 d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "deductive"  j ( d i1 d uh1 k t i v ) ((pos "OA%" ) ))
( "deducts"  v ( d i1 d uh1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "deed"  n ( d ii1 d ) ((pos "K6%" ) ))
( "deed-box"  n ( d ii1 d - b o k s ) ((pos "K7$" ) ))
( "deed-boxes"  n ( d ii1 d - b o k s i z ) ((pos "Kj$" ) ))
( "deedpoll"  n ( d ii1 d p ou l ) ((pos "M6%" ) ))
( "deedpolls"  n ( d ii1 d p ou l z ) ((pos "Mj$" ) ))
( "deeds"  n ( d ii1 d z ) ((pos "Kj%" ) ))
( "deem"  v ( d ii1 m ) ((pos "H0%" ) (subcat "9" )))
( "deemed"  v ( d ii1 m d ) ((pos "Hc%" "Hd%" ) (subcat "9" )))
( "deeming"  v ( d ii1 m i ng ) ((pos "Hb%" ) (subcat "9" )))
( "deems"  v ( d ii1 m z ) ((pos "Ha%" ) (subcat "9" )))
( "deep"  n ( d ii1 p ) ((pos "K6%" "OC%" "Pu%" ) ))
( "deep-freeze"  v ( d ii1 p - f r ii1 z ) ((pos "H5%" "K6%" ) ))
( "deep-freezes"  v ( d ii1 p - f r ii1 z i z ) ((pos "Ha$" "Kj%" ) ))
( "deep-freezing"  v ( d ii1 p - f r ii1 z i ng ) ((pos "Hb$" ) ))
( "deep-froze"  v ( d ii1 p - f r ou1 z ) ((pos "Hc$" ) ))
( "deep-frozen"  v ( d ii1 p - f r ou1 z n ) ((pos "Hd%" ) ))
( "deep-laid"  j ( d ii1 p - l ei1 d ) ((pos "OA$" ) ))
( "deep-mined"  j ( d ii1 p - m ai1 n d ) ((pos "OA$" ) ))
( "deep-rooted"  j ( d ii1 p - r uu1 t i d ) ((pos "OA%" ) ))
( "deep-sea"  j ( d ii1 p - s ii ) ((pos "Oq%" ) ))
( "deep-seated"  j ( d ii1 p - s ii1 t i d ) ((pos "OA%" ) ))
( "deep-water"  j ( d ii1 p - w oo t @ r ) ((pos "Oq%" ) ))
( "deepen"  v ( d ii1 p @ n ) ((pos "J0%" ) ))
( "deepened"  v ( d ii1 p @ n d ) ((pos "Jc%" "Jd%" ) ))
( "deepening"  v ( d ii1 p @ n i ng ) ((pos "Jb%" ) ))
( "deepens"  v ( d ii1 p @ n z ) ((pos "Ja%" ) ))
( "deeper"  j ( d ii1 p @ r ) ((pos "Or%" ) ))
( "deepest"  j ( d ii1 p i s t ) ((pos "Os%" ) ))
( "deeply"  a ( d ii1 p l ii ) ((pos "Pu%" ) ))
( "deepness"  n ( d ii1 p n @ s ) ((pos "L@%" ) ))
( "deeps"  n ( d ii1 p s ) ((pos "Kj%" ) ))
( "deer"  n ( d i@1 r ) ((pos "K9%" ) ))
( "deerskin"  n ( d i@1 s k i n ) ((pos "K6%" ) ))
( "deerskins"  n ( d i@1 s k i n z ) ((pos "Kj$" ) ))
( "deerstalker"  n ( d i@1 s t oo k @ r ) ((pos "K6%" ) ))
( "deerstalkers"  n ( d i@1 s t oo k @ z ) ((pos "Kj%" ) ))
( "deerstalking"  n ( d i@1 s t oo k i ng ) ((pos "L@%" ) ))
( "deface"  v ( d i1 f ei1 s ) ((pos "H2%" ) (subcat )))
( "defaced"  v ( d i1 f ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defacement"  n ( d i1 f ei1 s m @ n t ) ((pos "M6%" ) ))
( "defacements"  n ( d i1 f ei1 s m @ n t s ) ((pos "Mj$" ) ))
( "defaces"  v ( d i1 f ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "defacing"  v ( d i1 f ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "defalcation"  n ( d ii2 f a l k ei1 sh @ n ) ((pos "M6$" ) ))
( "defalcations"  n ( d ii2 f a l k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "defamation"  n ( d e2 f @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "defamatory"  j ( d i1 f a1 m @ t r ii ) ((pos "OA%" ) ))
( "defame"  v ( d i1 f ei1 m ) ((pos "H2%" ) (subcat )))
( "defamed"  v ( d i1 f ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defames"  v ( d i1 f ei1 m z ) ((pos "Ha%" ) (subcat )))
( "defaming"  v ( d i1 f ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "default"  v ( d i1 f oo1 l t ) ((pos "I0%" "L@%" ) (subcat )))
( "defaulted"  v ( d i1 f oo1 l t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "defaulter"  n ( d i1 f oo1 l t @ r ) ((pos "K6%" ) ))
( "defaulters"  n ( d i1 f oo1 l t @ z ) ((pos "Kj%" ) ))
( "defaulting"  v ( d i1 f oo1 l t i ng ) ((pos "Ib%" ) (subcat )))
( "defaults"  v ( d i1 f oo1 l t s ) ((pos "Ia%" ) (subcat )))
( "defeat"  v ( d i1 f ii1 t ) ((pos "H0%" "M6%" ) (subcat )))
( "defeated"  v ( d i1 f ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defeating"  v ( d i1 f ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "defeatism"  n ( d i1 f ii1 t i z @ m ) ((pos "L@$" ) ))
( "defeatist"  n ( d i1 f ii1 t i s t ) ((pos "K6%" ) ))
( "defeatists"  n ( d i1 f ii1 t i s t s ) ((pos "Kj$" ) ))
( "defeats"  v ( d i1 f ii1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "defecate"  v ( d e1 f @ k ei t ) ((pos "I2%" ) (subcat )))
( "defecated"  v ( d e1 f @ k ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "defecates"  v ( d e1 f @ k ei t s ) ((pos "Ia%" ) (subcat )))
( "defecating"  v ( d e1 f @ k ei t i ng ) ((pos "Ib%" ) (subcat )))
( "defecation"  n ( d e2 f @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "defecations"  n ( d e2 f @ k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "defect"  n ( d ii1 f e k t ) ((pos "K6%" ) ))
( "defect"  v ( d i1 f e1 k t ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "defected"  v ( d i1 f e1 k t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "defecting"  v ( d i1 f e1 k t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "defection"  n ( d i1 f e1 k sh @ n ) ((pos "M6%" ) ))
( "defections"  n ( d i1 f e1 k sh @ n z ) ((pos "Mj%" ) ))
( "defective"  j ( d i1 f e1 k t i v ) ((pos "OA%" ) ))
( "defectively"  a ( d i1 f e1 k t i v l ii ) ((pos "Pu%" ) ))
( "defectiveness"  n ( d i1 f e1 k t i v n @ s ) ((pos "L@%" ) ))
( "defector"  n ( d i1 f e1 k t @ r ) ((pos "K6%" ) ))
( "defectors"  n ( d i1 f e1 k t @ z ) ((pos "Kj%" ) ))
( "defects"  n ( d ii1 f e k t s ) ((pos "Kj%" ) ))
( "defects"  v ( d i1 f e1 k t s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "defence"  n ( d i1 f e1 n s ) ((pos "M6%" ) ))
( "defenceless"  j ( d i1 f e1 n s l @ s ) ((pos "OA%" ) ))
( "defencelessly"  a ( d i1 f e1 n s l @ s l ii ) ((pos "Pu%" ) ))
( "defencelessness"  n ( d i1 f e1 n s l @ s n @ s ) ((pos "L@%" ) ))
( "defences"  n ( d i1 f e1 n s i z ) ((pos "Mj%" ) ))
( "defend"  v ( d i1 f e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "defendant"  n ( d i1 f e1 n d @ n t ) ((pos "K6%" ) ))
( "defendants"  n ( d i1 f e1 n d @ n t s ) ((pos "Kj%" ) ))
( "defended"  v ( d i1 f e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "defender"  n ( d i1 f e1 n d @ r ) ((pos "K6%" ) ))
( "defenders"  n ( d i1 f e1 n d @ z ) ((pos "Kj%" ) ))
( "defending"  v ( d i1 f e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "defends"  v ( d i1 f e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "defensible"  j ( d i1 f e1 n s @ b l ) ((pos "OA%" ) ))
( "defensive"  n ( d i1 f e1 n s i v ) ((pos "Ki%" "OA%" ) ))
( "defensively"  a ( d i1 f e1 n s i v l ii ) ((pos "Pu%" ) ))
( "defer"  v ( d i1 f @@1 r ) ((pos "J4%" ) (subcat "3A" "6A" )))
( "deference"  n ( d e1 f @ r @ n s ) ((pos "L@%" ) ))
( "deferential"  j ( d e2 f @ r e1 n sh l ) ((pos "OA%" ) ))
( "deferentially"  a ( d e2 f @ r e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "deferment"  n ( d i1 f @@1 m @ n t ) ((pos "K6%" ) ))
( "deferments"  n ( d i1 f @@1 m @ n t s ) ((pos "Kj$" ) ))
( "deferred"  v ( d i1 f @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "deferring"  v ( d i1 f @@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "defers"  v ( d i1 f @@1 z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "defiance"  n ( d i1 f ai1 @ n s ) ((pos "L@%" ) ))
( "defiant"  j ( d i1 f ai1 @ n t ) ((pos "OA%" ) ))
( "defiantly"  a ( d i1 f ai1 @ n t l ii ) ((pos "Pu%" ) ))
( "deficiencies"  n ( d i1 f i1 sh @ n s i z ) ((pos "Mj%" ) ))
( "deficiency"  n ( d i1 f i1 sh @ n s ii ) ((pos "M8%" ) ))
( "deficient"  j ( d i1 f i1 sh @ n t ) ((pos "OA%" ) ))
( "deficit"  n ( d e1 f i s i t ) ((pos "K6%" ) ))
( "deficits"  n ( d e1 f i s i t s ) ((pos "Kj%" ) ))
( "defied"  v ( d i1 f ai1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "defies"  v ( d i1 f ai1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "defile"  n ( d ii1 f ai l ) ((pos "K6%" ) ))
( "defile"  v ( d i1 f ai1 l ) ((pos "J2%" ) (subcat )))
( "defiled"  v ( d i1 f ai1 l d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "defilement"  n ( d i1 f ai1 l m @ n t ) ((pos "L@$" ) ))
( "defiles"  n ( d ii1 f ai l z ) ((pos "Kj%" ) ))
( "defiles"  v ( d i1 f ai1 l z ) ((pos "Ja%" ) (subcat )))
( "defiling"  v ( d i1 f ai1 l i ng ) ((pos "Jb%" ) (subcat )))
( "definable"  j ( d i1 f ai1 n @ b l ) ((pos "OA%" ) ))
( "define"  v ( d i1 f ai1 n ) ((pos "H2%" ) (subcat )))
( "defined"  v ( d i1 f ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defines"  v ( d i1 f ai1 n z ) ((pos "Ha%" ) (subcat )))
( "defining"  v ( d i1 f ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "definite"  j ( d e1 f i n @ t ) ((pos "OA%" ) ))
( "definitely"  a ( d e1 f i n @ t l ii ) ((pos "Pu%" ) ))
( "definition"  n ( d e2 f i n i1 sh @ n ) ((pos "M6%" ) ))
( "definitions"  n ( d e2 f i n i1 sh @ n z ) ((pos "Mj%" ) ))
( "definitive"  j ( d i1 f i1 n @ t i v ) ((pos "OA%" ) ))
( "deflate"  v ( d i1 f l ei1 t ) ((pos "H2%" ) (subcat )))
( "deflated"  v ( d i1 f l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deflates"  v ( d i1 f l ei1 t s ) ((pos "Ha%" ) (subcat )))
( "deflating"  v ( d i1 f l ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "deflation"  n ( d i1 f l ei1 sh @ n ) ((pos "L@%" ) ))
( "deflationary"  j ( d ii2 f l ei1 sh @ n @ r ii ) ((pos "OA%" ) ))
( "deflect"  v ( d i1 f l e1 k t ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "deflected"  v ( d i1 f l e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "deflecting"  v ( d i1 f l e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "deflection"  n ( d i1 f l e1 k sh @ n ) ((pos "K6%" ) ))
( "deflections"  n ( d i1 f l e1 k sh @ n z ) ((pos "Kj%" ) ))
( "deflects"  v ( d i1 f l e1 k t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "deflower"  v ( d ii1 f l au1 @ r ) ((pos "H0%" ) ))
( "deflowered"  v ( d ii1 f l au1 @ d ) ((pos "Hc%" "Hd%" ) ))
( "deflowering"  v ( d ii1 f l au1 @ r i ng ) ((pos "Hb%" ) ))
( "deflowers"  v ( d ii1 f l au1 @ z ) ((pos "Ha%" ) ))
( "defoliant"  n ( d ii2 f ou1 l i@ n t ) ((pos "K6%" ) ))
( "defoliants"  n ( d ii2 f ou1 l i@ n t s ) ((pos "Kj%" ) ))
( "defoliate"  v ( d ii2 f ou1 l i ei t ) ((pos "H2%" ) (subcat )))
( "defoliated"  v ( d ii2 f ou1 l i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defoliates"  v ( d ii2 f ou1 l i ei t s ) ((pos "Ha%" ) (subcat )))
( "defoliating"  v ( d ii2 f ou1 l i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "defoliation"  n ( d ii2 f ou l i ei1 sh @ n ) ((pos "K6%" ) ))
( "defoliations"  n ( d ii2 f ou l i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "deforest"  v ( d ii2 f o1 r i s t ) ((pos "H0%" ) ))
( "deforested"  v ( d ii2 f o1 r i s t i d ) ((pos "Hc%" "Hd%" ) ))
( "deforesting"  v ( d ii2 f o1 r i s t i ng ) ((pos "Hb%" ) ))
( "deforests"  v ( d ii2 f o1 r i s t s ) ((pos "Ha%" ) ))
( "deform"  v ( d i1 f oo1 m ) ((pos "H0%" ) (subcat )))
( "deformed"  v ( d i1 f oo1 m d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "deforming"  v ( d i1 f oo1 m i ng ) ((pos "Hb%" ) (subcat )))
( "deformities"  n ( d i1 f oo1 m i t i z ) ((pos "Mj%" ) ))
( "deformity"  n ( d i1 f oo1 m i t ii ) ((pos "M8%" ) ))
( "deforms"  v ( d i1 f oo1 m z ) ((pos "Ha%" ) (subcat )))
( "defraud"  v ( d i1 f r oo1 d ) ((pos "H0%" ) (subcat "6A" )))
( "defrauded"  v ( d i1 f r oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "defrauding"  v ( d i1 f r oo1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "defrauds"  v ( d i1 f r oo1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "defray"  v ( d i1 f r ei1 ) ((pos "H0%" ) (subcat )))
( "defrayal"  n ( d i1 f r ei1 @ l ) ((pos "K6$" ) ))
( "defrayals"  n ( d i1 f r ei1 @ l z ) ((pos "Kj$" ) ))
( "defrayed"  v ( d i1 f r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defraying"  v ( d i1 f r ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "defrayment"  n ( d i1 f r ei1 m @ n t ) ((pos "K6$" ) ))
( "defrayments"  n ( d i1 f r ei1 m @ n t s ) ((pos "Kj$" ) ))
( "defrays"  v ( d i1 f r ei1 z ) ((pos "Ha$" ) (subcat )))
( "defrock"  v ( d ii2 f r o1 k ) ((pos "H0$" ) ))
( "defrocked"  v ( d ii2 f r o1 k t ) ((pos "Hc$" "Hd%" ) ))
( "defrocking"  v ( d ii2 f r o1 k i ng ) ((pos "Hb$" ) ))
( "defrocks"  v ( d ii2 f r o1 k s ) ((pos "Ha$" ) ))
( "defrost"  v ( d ii2 f r o1 s t ) ((pos "H0%" ) (subcat )))
( "defrosted"  v ( d ii2 f r o1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defroster"  n ( d ii2 f r o1 s t @ r ) ((pos "K6%" ) ))
( "defrosters"  n ( d ii2 f r o1 s t @ z ) ((pos "Kj%" ) ))
( "defrosting"  v ( d ii2 f r o1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "defrosts"  v ( d ii2 f r o1 s t s ) ((pos "Ha%" ) (subcat )))
( "deft"  j ( d e1 f t ) ((pos "OA%" ) ))
( "deftly"  a ( d e1 f t l ii ) ((pos "Pu%" ) ))
( "deftness"  n ( d e1 f t n @ s ) ((pos "L@%" ) ))
( "defunct"  j ( d i1 f uh1 ng k t ) ((pos "OA%" ) ))
( "defuse"  v ( d ii2 f y uu1 z ) ((pos "H2%" ) (subcat )))
( "defused"  v ( d ii2 f y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "defuses"  v ( d ii2 f y uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "defusing"  v ( d ii2 f y uu1 z i ng ) ((pos "Hb%" ) (subcat )))
( "defy"  v ( d i1 f ai1 ) ((pos "H3%" ) (subcat "6A" )))
( "defying"  v ( d i1 f ai1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "degauss"  v ( d ii2 g au1 s ) ((pos "H1$" ) (subcat )))
( "degaussed"  v ( d ii2 g au1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "degausses"  v ( d ii2 g au1 s i z ) ((pos "Ha$" ) (subcat )))
( "degaussing"  v ( d ii2 g au1 s i ng ) ((pos "Hb$" ) (subcat )))
( "degeneracy"  n ( d i1 jh e1 n @ r @ s ii ) ((pos "L@$" ) ))
( "degenerate"  n ( d i1 jh e1 n @ r @ t ) ((pos "K6%" "OA%" ) ))
( "degenerate"  v ( d i1 jh e1 n @ r ei t ) ((pos "I2%" ) (subcat "2A" )))
( "degenerated"  v ( d i1 jh e1 n @ r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "degenerates"  n ( d i1 jh e1 n @ r @ t s ) ((pos "Kj%" ) ))
( "degenerates"  v ( d i1 jh e1 n @ r ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "degenerating"  v ( d i1 jh e1 n @ r ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "degeneration"  n ( d i1 jh e2 n @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "degradation"  n ( d e2 g r @ d ei1 sh @ n ) ((pos "M6%" ) ))
( "degradations"  n ( d e2 g r @ d ei1 sh @ n z ) ((pos "Mj$" ) ))
( "degrade"  v ( d i1 g r ei1 d ) ((pos "H2%" ) (subcat )))
( "degraded"  v ( d i1 g r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "degrades"  v ( d i1 g r ei1 d z ) ((pos "Ha%" ) (subcat )))
( "degrading"  v ( d i1 g r ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "degree"  n ( d i1 g r ii1 ) ((pos "M6%" ) ))
( "degrees"  n ( d i1 g r ii1 z ) ((pos "Mj%" ) ))
( "dehorn"  v ( d ii2 h oo1 n ) ((pos "H0%" ) (subcat )))
( "dehorned"  v ( d ii2 h oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dehorning"  v ( d ii2 h oo1 n i ng ) ((pos "Hb%" ) (subcat )))
( "dehorns"  v ( d ii2 h oo1 n z ) ((pos "Ha%" ) (subcat )))
( "dehumanize"  v ( d ii2 h y uu1 m @ n ai z ) ((pos "H2%" ) (subcat )))
( "dehumanized"  v ( d ii2 h y uu1 m @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dehumanizes"  v ( d ii2 h y uu1 m @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "dehumanizing"  v ( d ii2 h y uu1 m @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "dehydrate"  v ( d ii2 h ai1 d r ei t ) ((pos "H2%" ) (subcat )))
( "dehydrated"  v ( d ii2 h ai1 d r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dehydrates"  v ( d ii2 h ai1 d r ei t s ) ((pos "Ha$" ) (subcat )))
( "dehydrating"  v ( d ii2 h ai1 d r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "deification"  n ( d ii2 i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "deified"  v ( d ii1 i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deifies"  v ( d ii1 i f ai z ) ((pos "Ha%" ) (subcat )))
( "deify"  v ( d ii1 i f ai ) ((pos "H3%" ) (subcat )))
( "deifying"  v ( d ii1 i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "deign"  v ( d ei1 n ) ((pos "I0%" ) (subcat )))
( "deigned"  v ( d ei1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "deigning"  v ( d ei1 n i ng ) ((pos "Ib$" ) (subcat )))
( "deigns"  v ( d ei1 n z ) ((pos "Ia$" ) (subcat )))
( "deism"  n ( d ii1 i z @ m ) ((pos "L@$" ) ))
( "deist"  n ( d ii1 i s t ) ((pos "K6$" ) ))
( "deists"  n ( d ii1 i s t s ) ((pos "Kj$" ) ))
( "deities"  n ( d ii1 i t i z ) ((pos "Mj%" ) ))
( "deity"  n ( d ii1 i t ii ) ((pos "M8%" ) ))
( "deject"  v ( d i1 jh e1 k t ) ((pos "H0%" ) ))
( "dejected"  v ( d i1 jh e1 k t i d ) ((pos "Hc%" "Hd%" ) ))
( "dejectedly"  a ( d i1 jh e1 k t i d l ii ) ((pos "Pu%" ) ))
( "dejecting"  v ( d i1 jh e1 k t i ng ) ((pos "Hb%" ) ))
( "dejection"  n ( d i1 jh e1 k sh @ n ) ((pos "L@%" ) ))
( "dejects"  v ( d i1 jh e1 k t s ) ((pos "Ha%" ) ))
( "dekko"  n ( d e1 k ou ) ((pos "Ki$" ) ))
( "delay"  v ( d i1 l ei1 ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "6A" )))
( "delayed"  v ( d i1 l ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "6A" )))
( "delayed-action"  n ( d i1 l ei d - a1 k sh @ n ) ((pos "L@%" "OA%" ) ))
( "delaying"  v ( d i1 l ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "6A" )))
( "delays"  v ( d i1 l ei1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "6A" )))
( "delectable"  j ( d i1 l e1 k t @ b l ) ((pos "OA%" ) ))
( "delectation"  n ( d ii2 l e k t ei1 sh @ n ) ((pos "L@%" ) ))
( "delegacies"  n ( d e1 l i g @ s i z ) ((pos "Kj$" ) ))
( "delegacy"  n ( d e1 l i g @ s ii ) ((pos "K8$" ) ))
( "delegate"  n ( d e1 l i g @ t ) ((pos "K6%" ) ))
( "delegate"  v ( d e1 l i g ei t ) ((pos "H2%" ) (subcat "14" )))
( "delegated"  v ( d e1 l i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "delegates"  n ( d e1 l i g @ t s ) ((pos "Kj%" ) ))
( "delegates"  v ( d e1 l i g ei t s ) ((pos "Ha%" ) (subcat "14" )))
( "delegating"  v ( d e1 l i g ei t i ng ) ((pos "Hb%" ) (subcat "14" )))
( "delegation"  n ( d e2 l i g ei1 sh @ n ) ((pos "M6%" ) ))
( "delegations"  n ( d e2 l i g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "delete"  v ( d i1 l ii1 t ) ((pos "H2%" ) (subcat "6A" )))
( "deleted"  v ( d i1 l ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deleterious"  j ( d e2 l i t i@1 r i@ s ) ((pos "OA%" ) ))
( "deletes"  v ( d i1 l ii1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "deleting"  v ( d i1 l ii1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deletion"  n ( d i1 l ii1 sh @ n ) ((pos "M6%" ) ))
( "deletions"  n ( d i1 l ii1 sh @ n z ) ((pos "Mj%" ) ))
( "delf"  n ( d e1 l f ) ((pos "L@$" ) ))
( "delft"  n ( d e1 l f t ) ((pos "L@$" ) ))
( "deliberate"  j ( d i1 l i1 b @ r @ t ) ((pos "OA%" ) ))
( "deliberate"  v ( d i1 l i1 b @ r ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "8" )))
( "deliberated"  v ( d i1 l i1 b @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" )))
( "deliberately"  a ( d i1 l i1 b @ r @ t l ii ) ((pos "Pu%" ) ))
( "deliberates"  v ( d i1 l i1 b @ r ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" )))
( "deliberating"  v ( d i1 l i1 b @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" )))
( "deliberation"  n ( d i1 l i2 b @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "deliberations"  n ( d i1 l i2 b @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "deliberative"  j ( d i1 l i1 b @ r @ t i v ) ((pos "OA$" ) ))
( "delicacies"  n ( d e1 l i k @ s i z ) ((pos "Mj%" ) ))
( "delicacy"  n ( d e1 l i k @ s ii ) ((pos "M8%" ) ))
( "delicate"  j ( d e1 l i k @ t ) ((pos "OA%" ) ))
( "delicately"  a ( d e1 l i k @ t l ii ) ((pos "Pu%" ) ))
( "delicatessen"  n ( d e2 l i k @ t e1 s n ) ((pos "M6%" ) ))
( "delicatessens"  n ( d e2 l i k @ t e1 s n z ) ((pos "Mj%" ) ))
( "delicious"  j ( d i1 l i1 sh @ s ) ((pos "OA%" ) ))
( "deliciously"  a ( d i1 l i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "delight"  v ( d i1 l ai1 t ) ((pos "J0%" "M6%" ) (subcat "3A" "4C" )))
( "delighted"  v ( d i1 l ai1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "4C" )))
( "delightedly"  a ( d i1 l ai1 t i d l ii ) ((pos "Pu%" ) ))
( "delightful"  j ( d i1 l ai1 t f @ l ) ((pos "OA%" ) ))
( "delightfully"  a ( d i1 l ai1 t f @ l ii ) ((pos "Pu%" ) ))
( "delighting"  v ( d i1 l ai1 t i ng ) ((pos "Jb%" ) (subcat "3A" "4C" )))
( "delights"  v ( d i1 l ai1 t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "4C" )))
( "delimit"  v ( d ii1 l i1 m i t ) ((pos "H0%" ) (subcat )))
( "delimitate"  v ( d ii1 l i1 m i t ei t ) ((pos "H2$" ) (subcat )))
( "delimitated"  v ( d ii1 l i1 m i t ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "delimitates"  v ( d ii1 l i1 m i t ei t s ) ((pos "Ha$" ) (subcat )))
( "delimitating"  v ( d ii1 l i1 m i t ei t i ng ) ((pos "Hb$" ) (subcat )))
( "delimitation"  n ( d ii1 l i2 m i t ei1 sh @ n ) ((pos "M6$" ) ))
( "delimitations"  n ( d ii1 l i2 m i t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "delimited"  v ( d ii1 l i1 m i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "delimiting"  v ( d ii1 l i1 m i t i ng ) ((pos "Hb%" ) (subcat )))
( "delimits"  v ( d ii1 l i1 m i t s ) ((pos "Ha%" ) (subcat )))
( "delineate"  v ( d i1 l i1 n i ei t ) ((pos "H2%" ) (subcat )))
( "delineated"  v ( d i1 l i1 n i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "delineates"  v ( d i1 l i1 n i ei t s ) ((pos "Ha%" ) (subcat )))
( "delineating"  v ( d i1 l i1 n i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "delineation"  n ( d i1 l i2 n i ei1 sh @ n ) ((pos "M6%" ) ))
( "delineations"  n ( d i1 l i2 n i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "delinquencies"  n ( d i1 l i1 ng k w @ n s i z ) ((pos "Mj%" ) ))
( "delinquency"  n ( d i1 l i1 ng k w @ n s ii ) ((pos "M8%" ) ))
( "delinquent"  n ( d i1 l i1 ng k w @ n t ) ((pos "K6%" "OA%" ) ))
( "delinquents"  n ( d i1 l i1 ng k w @ n t s ) ((pos "Kj%" ) ))
( "deliquescent"  j ( d e2 l i k w e1 s n t ) ((pos "OA$" ) ))
( "delirious"  j ( d i1 l i1 r i@ s ) ((pos "OA%" ) ))
( "deliriously"  a ( d i1 l i1 r i@ s l ii ) ((pos "Pu%" ) ))
( "delirium"  n ( d i1 l i1 r i@ m ) ((pos "L@%" ) ))
( "delirium tremens"  n ( d i1 l i2 r i@ m - t r ii1 m e n z ) ((pos "Ki%" ) ))
( "deliver"  v ( d i1 l i1 v @ r ) ((pos "H0%" ) (subcat "6A" "14" )))
( "deliverance"  n ( d i1 l i1 v @ r @ n s ) ((pos "M6%" ) ))
( "deliverances"  n ( d i1 l i1 v @ r @ n s i z ) ((pos "Mj$" ) ))
( "delivered"  v ( d i1 l i1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "deliverer"  n ( d i1 l i1 v @ r @ r ) ((pos "K6%" ) ))
( "deliverers"  n ( d i1 l i1 v @ r @ z ) ((pos "Kj%" ) ))
( "deliveries"  n ( d i1 l i1 v @ r i z ) ((pos "Mj%" ) ))
( "delivering"  v ( d i1 l i1 v @ r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "delivers"  v ( d i1 l i1 v @ z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "delivery"  n ( d i1 l i1 v @ r ii ) ((pos "M8%" ) ))
( "dell"  n ( d e1 l ) ((pos "K6%" ) ))
( "dells"  n ( d e1 l z ) ((pos "Kj%" ) ))
( "delouse"  v ( d ii2 l au1 s ) ((pos "H2%" ) ))
( "deloused"  v ( d ii2 l au1 s t ) ((pos "Hc%" "Hd%" ) ))
( "delouses"  v ( d ii2 l au1 s i z ) ((pos "Ha%" ) ))
( "delousing"  v ( d ii2 l au1 s i ng ) ((pos "Hb%" ) ))
( "delphinium"  n ( d e1 l f i1 n i@ m ) ((pos "K6%" ) ))
( "delphiniums"  n ( d e1 l f i1 n i@ m z ) ((pos "Kj%" ) ))
( "delta"  n ( d e1 l t @ ) ((pos "K6%" ) ))
( "deltas"  n ( d e1 l t @ z ) ((pos "Kj%" ) ))
( "delude"  v ( d i1 l uu1 d ) ((pos "H2%" ) (subcat "6A" )))
( "deluded"  v ( d i1 l uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deludes"  v ( d i1 l uu1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "deluding"  v ( d i1 l uu1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deluge"  v ( d e1 l y uu jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "deluged"  v ( d e1 l y uu jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deluges"  v ( d e1 l y uu jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "deluging"  v ( d e1 l y uu jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "delusion"  n ( d i1 l uu1 zh n ) ((pos "M6%" ) ))
( "delusions"  n ( d i1 l uu1 zh n z ) ((pos "Mj%" ) ))
( "delusive"  j ( d i1 l uu1 s i v ) ((pos "OA$" ) ))
( "delusively"  a ( d i1 l uu1 s i v l ii ) ((pos "Pu$" ) ))
( "delve"  v ( d e1 l v ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "delved"  v ( d e1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "delves"  v ( d e1 l v z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "delving"  v ( d e1 l v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "demagnetization"  n ( d ii2 m a2 g n i t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "demagnetize"  v ( d ii2 m a1 g n i t ai z ) ((pos "H2%" ) (subcat )))
( "demagnetized"  v ( d ii2 m a1 g n i t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demagnetizes"  v ( d ii2 m a1 g n i t ai z i z ) ((pos "Ha%" ) (subcat )))
( "demagnetizing"  v ( d ii2 m a1 g n i t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "demagogic"  j ( d e2 m @ g o1 g i k ) ((pos "OA$" ) ))
( "demagogue"  n ( d e1 m @ g o g ) ((pos "K6%" ) ))
( "demagogues"  n ( d e1 m @ g o g z ) ((pos "Kj%" ) ))
( "demagogy"  n ( d e1 m @ g o g ii ) ((pos "L@$" ) ))
( "demand"  v ( d i1 m aa1 n d ) ((pos "H0%" "M6%" ) (subcat "6A" "7A" )))
( "demanded"  v ( d i1 m aa1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "demanding"  v ( d i1 m aa1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "demands"  v ( d i1 m aa1 n d z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "7A" )))
( "demarcate"  v ( d ii1 m aa k ei t ) ((pos "H2%" ) (subcat )))
( "demarcated"  v ( d ii1 m aa k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demarcates"  v ( d ii1 m aa k ei t s ) ((pos "Ha%" ) (subcat )))
( "demarcating"  v ( d ii1 m aa k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "demarcation"  n ( d ii2 m aa k ei1 sh @ n ) ((pos "L@%" ) ))
( "demean"  v ( d i1 m ii1 n ) ((pos "H0%" ) (subcat )))
( "demeaned"  v ( d i1 m ii1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demeaning"  v ( d i1 m ii1 n i ng ) ((pos "Hb%" ) (subcat )))
( "demeanour"  n ( d i1 m ii1 n @ r ) ((pos "L@%" ) ))
( "demeans"  v ( d i1 m ii1 n z ) ((pos "Ha%" ) (subcat )))
( "demented"  j ( d i1 m e1 n t i d ) ((pos "OA%" ) ))
( "dementedly"  a ( d i1 m e1 n t i d l ii ) ((pos "Pu%" ) ))
( "demerara"  n ( d e2 m @ r e@1 r @ ) ((pos "L@%" ) ))
( "demerit"  n ( d ii1 m e1 r i t ) ((pos "K6$" ) ))
( "demerits"  n ( d ii1 m e1 r i t s ) ((pos "Kj$" ) ))
( "demesne"  n ( d i1 m ei1 n ) ((pos "M6$" ) ))
( "demesnes"  n ( d i1 m ei1 n z ) ((pos "Mj$" ) ))
( "demigod"  n ( d e1 m i g o d ) ((pos "K6%" ) ))
( "demigods"  n ( d e1 m i g o d z ) ((pos "Kj%" ) ))
( "demijohn"  n ( d e1 m i jh o n ) ((pos "K6$" ) ))
( "demijohns"  n ( d e1 m i jh o n z ) ((pos "Kj$" ) ))
( "demilitarize"  v ( d ii2 m i1 l i t @ r ai z ) ((pos "H2%" ) (subcat )))
( "demilitarized"  v ( d ii2 m i1 l i t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demilitarizes"  v ( d ii2 m i1 l i t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "demilitarizing"  v ( d ii2 m i1 l i t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "demimondaine"  n ( d e2 m i m oo n d ei1 n ) ((pos "K6$" ) ))
( "demimondaines"  n ( d e2 m i m oo n d ei1 n z ) ((pos "Kj$" ) ))
( "demimonde"  n ( d e2 m i m oo1 n d ) ((pos "Ki$" ) ))
( "demise"  n ( d i1 m ai1 z ) ((pos "Ki%" ) ))
( "demist"  v ( d ii2 m i1 s t ) ((pos "H0%" ) ))
( "demisted"  v ( d ii2 m i1 s t i d ) ((pos "Hc%" "Hd%" ) ))
( "demister"  n ( d ii2 m i1 s t @ r ) ((pos "K6%" ) ))
( "demisters"  n ( d ii2 m i1 s t @ z ) ((pos "Kj%" ) ))
( "demisting"  v ( d ii2 m i1 s t i ng ) ((pos "Hb%" ) ))
( "demists"  v ( d ii2 m i1 s t s ) ((pos "Ha%" ) ))
( "demo"  n ( d e1 m ou ) ((pos "K6%" ) ))
( "demob"  v ( d ii2 m o1 b ) ((pos "H4%" ) ))
( "demobbed"  v ( d ii2 m o1 b d ) ((pos "Hc%" "Hd%" ) ))
( "demobbing"  v ( d ii2 m o1 b i ng ) ((pos "Hb$" ) ))
( "demobilization"  n ( d i1 m ou2 b @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "demobilize"  v ( d ii1 m ou1 b @ l ai z ) ((pos "H2%" ) (subcat )))
( "demobilized"  v ( d ii1 m ou1 b @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demobilizes"  v ( d ii1 m ou1 b @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "demobilizing"  v ( d ii1 m ou1 b @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "demobs"  v ( d ii2 m o1 b z ) ((pos "Ha$" ) ))
( "democracies"  n ( d i1 m o1 k r @ s i z ) ((pos "Mj%" ) ))
( "democracy"  n ( d i1 m o1 k r @ s ii ) ((pos "M8%" ) ))
( "democrat"  n ( d e1 m @ k r a t ) ((pos "K6%" ) ))
( "democratic"  j ( d e2 m @ k r a1 t i k ) ((pos "OA%" ) ))
( "democratically"  a ( d e2 m @ k r a1 t i k l ii ) ((pos "Pu%" ) ))
( "democratization"  n ( d i1 m o2 k r @ t ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "democratize"  v ( d i1 m o1 k r @ t ai z ) ((pos "H2$" ) (subcat )))
( "democratized"  v ( d i1 m o1 k r @ t ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "democratizes"  v ( d i1 m o1 k r @ t ai z i z ) ((pos "Ha$" ) (subcat )))
( "democratizing"  v ( d i1 m o1 k r @ t ai z i ng ) ((pos "Hb$" ) (subcat )))
( "democrats"  n ( d e1 m @ k r a t s ) ((pos "Kj%" ) ))
( "demographic"  j ( d e2 m @ g r a1 f i k ) ((pos "OA%" ) ))
( "demography"  n ( d i1 m o1 g r @ f ii ) ((pos "L@%" ) ))
( "demolish"  v ( d i1 m o1 l i sh ) ((pos "H1%" ) (subcat )))
( "demolished"  v ( d i1 m o1 l i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demolishes"  v ( d i1 m o1 l i sh i z ) ((pos "Ha%" ) (subcat )))
( "demolishing"  v ( d i1 m o1 l i sh i ng ) ((pos "Hb%" ) (subcat )))
( "demolition"  n ( d e2 m @ l i1 sh @ n ) ((pos "M6%" ) ))
( "demolitions"  n ( d e2 m @ l i1 sh @ n z ) ((pos "Mj$" ) ))
( "demon"  n ( d ii1 m @ n ) ((pos "K6%" ) ))
( "demonetization"  n ( d ii1 m uh2 n i t ai z ei1 sh @ n ) ((pos "M6$" ) ))
( "demonetizations"  n ( d ii1 m uh2 n i t ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "demonetize"  v ( d ii2 m uh1 n i t ai z ) ((pos "H2$" ) ))
( "demonetized"  v ( d ii2 m uh1 n i t ai z d ) ((pos "Hc$" "Hd$" ) ))
( "demonetizes"  v ( d ii2 m uh1 n i t ai z i z ) ((pos "Ha$" ) ))
( "demonetizing"  v ( d ii2 m uh1 n i t ai z i ng ) ((pos "Hb$" ) ))
( "demoniac"  n ( d i1 m ou1 n i a k ) ((pos "K6$" "OA$" ) ))
( "demoniacal"  j ( d ii2 m @ n ai1 @ k l ) ((pos "OA$" ) ))
( "demoniacally"  a ( d ii2 m @ n ai1 @ k l ii ) ((pos "Pu$" ) ))
( "demoniacs"  n ( d i1 m ou1 n i a k s ) ((pos "Kj$" ) ))
( "demonic"  j ( d ii1 m o1 n i k ) ((pos "OA%" ) ))
( "demons"  n ( d ii1 m @ n z ) ((pos "Kj%" ) ))
( "demonstrability"  n ( d i1 m o2 n s t r @ b i1 l i t ii ) ((pos "L@$" ) ))
( "demonstrable"  j ( d e1 m @ n s t r @ b l ) ((pos "OA%" ) ))
( "demonstrably"  a ( d e1 m @ n s t r @ b l ii ) ((pos "Pu%" ) ))
( "demonstrate"  v ( d e1 m @ n s t r ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "demonstrated"  v ( d e1 m @ n s t r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "demonstrates"  v ( d e1 m @ n s t r ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "demonstrating"  v ( d e1 m @ n s t r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "demonstration"  n ( d e2 m @ n s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "demonstrations"  n ( d e2 m @ n s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "demonstrative"  j ( d i1 m o1 n s t r @ t i v ) ((pos "OA%" ) ))
( "demonstratively"  a ( d i1 m o1 n s t r @ t i v l ii ) ((pos "Pu%" ) ))
( "demonstrator"  n ( d e1 m @ n s t r ei t @ r ) ((pos "K6%" ) ))
( "demonstrators"  n ( d e1 m @ n s t r ei t @ z ) ((pos "Kj%" ) ))
( "demoralization"  n ( d i1 m o2 r @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "demoralize"  v ( d i1 m o1 r @ l ai z ) ((pos "H2%" ) (subcat )))
( "demoralized"  v ( d i1 m o1 r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demoralizes"  v ( d i1 m o1 r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "demoralizing"  v ( d i1 m o1 r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "demos"  n ( d e1 m ou z ) ((pos "Kj%" ) ))
( "demote"  v ( d ii2 m ou1 t ) ((pos "H2%" ) (subcat )))
( "demoted"  v ( d ii2 m ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "demotes"  v ( d ii2 m ou1 t s ) ((pos "Ha%" ) (subcat )))
( "demotic"  j ( d i1 m o1 t i k ) ((pos "OA$" ) ))
( "demoting"  v ( d ii2 m ou1 t i ng ) ((pos "Hb%" ) (subcat )))
( "demotion"  n ( d ii2 m ou1 sh @ n ) ((pos "M6%" ) ))
( "demotions"  n ( d ii2 m ou1 sh @ n z ) ((pos "Mj$" ) ))
( "demur"  v ( d i1 m @@1 r ) ((pos "I4%" "L@%" ) (subcat "2A" )))
( "demure"  j ( d i1 m y u@1 r ) ((pos "OA%" ) ))
( "demurely"  a ( d i1 m y u@1 l ii ) ((pos "Pu%" ) ))
( "demureness"  n ( d i1 m y u@1 n @ s ) ((pos "L@%" ) ))
( "demurred"  v ( d i1 m @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "demurring"  v ( d i1 m @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "demurs"  v ( d i1 m @@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "den"  n ( d e1 n ) ((pos "K6%" ) ))
( "denary"  j ( d ii1 n @ r ii ) ((pos "OA$" ) ))
( "denationalization"  n ( d ii2 n a2 sh @ n @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "denationalize"  v ( d ii2 n a1 sh @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "denationalized"  v ( d ii2 n a1 sh @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "denationalizes"  v ( d ii2 n a1 sh @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "denationalizing"  v ( d ii2 n a1 sh @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "denatured"  j ( d ii2 n ei1 ch @ d ) ((pos "OA$" ) ))
( "deniable"  j ( d i1 n ai1 @ b l ) ((pos "OA%" ) ))
( "denial"  n ( d i1 n ai1 @ l ) ((pos "M6%" ) ))
( "denials"  n ( d i1 n ai1 @ l z ) ((pos "Mj%" ) ))
( "denied"  v ( d i1 n ai1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "12A" "13A" )))
( "denier"  n ( d e1 n i@ r ) ((pos "K9$" ) ))
( "denies"  v ( d i1 n ai1 z ) ((pos "Ha%" ) (subcat "6A" "6C" "9" "12A" "13A" )))
( "denigrate"  v ( d e1 n i g r ei t ) ((pos "H2%" ) (subcat )))
( "denigrated"  v ( d e1 n i g r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "denigrates"  v ( d e1 n i g r ei t s ) ((pos "Ha%" ) (subcat )))
( "denigrating"  v ( d e1 n i g r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "denigration"  n ( d e2 n i g r ei1 sh @ n ) ((pos "L@%" ) ))
( "denim"  n ( d e1 n i m ) ((pos "L@%" ) ))
( "denims"  n ( d e1 n i m z ) ((pos "Kj%" ) ))
( "denizen"  n ( d e1 n i z n ) ((pos "K6$" ) ))
( "denizens"  n ( d e1 n i z n z ) ((pos "Kj$" ) ))
( "denominate"  v ( d i1 n o1 m i n ei t ) ((pos "H2$" ) (subcat )))
( "denominated"  v ( d i1 n o1 m i n ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "denominates"  v ( d i1 n o1 m i n ei t s ) ((pos "Ha$" ) (subcat )))
( "denominating"  v ( d i1 n o1 m i n ei t i ng ) ((pos "Hb$" ) (subcat )))
( "denomination"  n ( d i1 n o2 m i n ei1 sh @ n ) ((pos "K6%" ) ))
( "denominational"  j ( d i1 n o2 m i n ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "denominations"  n ( d i1 n o2 m i n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "denominator"  n ( d i1 n o1 m i n ei t @ r ) ((pos "K6%" ) ))
( "denominators"  n ( d i1 n o1 m i n ei t @ z ) ((pos "Kj%" ) ))
( "denote"  v ( d i1 n ou1 t ) ((pos "H2%" ) (subcat "6A" )))
( "denoted"  v ( d i1 n ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "denotes"  v ( d i1 n ou1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "denoting"  v ( d i1 n ou1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "denounce"  v ( d i1 n au1 n s ) ((pos "H2%" ) (subcat "6A" )))
( "denounced"  v ( d i1 n au1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "denounces"  v ( d i1 n au1 n s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "denouncing"  v ( d i1 n au1 n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dens"  n ( d e1 n z ) ((pos "Kj%" ) ))
( "dense"  j ( d e1 n s ) ((pos "OB%" ) ))
( "densely"  a ( d e1 n s l ii ) ((pos "Pu%" ) ))
( "denseness"  n ( d e1 n s n @ s ) ((pos "L@%" ) ))
( "denser"  j ( d e1 n s @ r ) ((pos "Or%" ) ))
( "densest"  j ( d e1 n s i s t ) ((pos "Os%" ) ))
( "densities"  n ( d e1 n s i t i z ) ((pos "Mj%" ) ))
( "density"  n ( d e1 n s i t ii ) ((pos "M8%" ) ))
( "dent"  v ( d e1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "dental"  j ( d e1 n t l ) ((pos "OA%" ) ))
( "dented"  v ( d e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dentifrice"  n ( d e1 n t i f r i s ) ((pos "L@$" ) ))
( "denting"  v ( d e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dentist"  n ( d e1 n t i s t ) ((pos "K6%" ) ))
( "dentistry"  n ( d e1 n t i s t r ii ) ((pos "L@%" ) ))
( "dentists"  n ( d e1 n t i s t s ) ((pos "Kj%" ) ))
( "dents"  v ( d e1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "denture"  n ( d e1 n ch @ r ) ((pos "K6%" ) ))
( "dentures"  n ( d e1 n ch @ z ) ((pos "Kj%" ) ))
( "denudation"  n ( d ii2 n y uu d ei1 sh @ n ) ((pos "L@$" ) ))
( "denude"  v ( d i1 n y uu1 d ) ((pos "H2%" ) (subcat "6A" )))
( "denuded"  v ( d i1 n y uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "denudes"  v ( d i1 n y uu1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "denuding"  v ( d i1 n y uu1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "denunciation"  n ( d i1 n uh2 n s i ei1 sh @ n ) ((pos "M6%" ) ))
( "denunciations"  n ( d i1 n uh2 n s i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "deny"  v ( d i1 n ai1 ) ((pos "H3%" ) (subcat "6A" "6C" "9" "12A" "13A" )))
( "denying"  v ( d i1 n ai1 i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "12A" "13A" )))
( "deodar"  n ( d i@1 d aa r ) ((pos "K6$" ) ))
( "deodars"  n ( d i@1 d aa z ) ((pos "Kj$" ) ))
( "deodorant"  n ( d ii1 ou1 d @ r @ n t ) ((pos "K6%" ) ))
( "deodorants"  n ( d ii1 ou1 d @ r @ n t s ) ((pos "Kj%" ) ))
( "deodorize"  v ( d ii1 ou1 d @ r ai z ) ((pos "H2%" ) (subcat )))
( "deodorized"  v ( d ii1 ou1 d @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deodorizes"  v ( d ii1 ou1 d @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "deodorizing"  v ( d ii1 ou1 d @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "dep"  n ( d e1 p ) ((pos "Y~%" ) ))
( "depart"  v ( d i1 p aa1 t ) ((pos "I0%" ) (subcat "2A" )))
( "departed"  v ( d i1 p aa1 t i d ) ((pos "Ic%" "Id%" "K9%" "OA%" ) (subcat "2A" )))
( "departing"  v ( d i1 p aa1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "department"  n ( d i1 p aa1 t m @ n t ) ((pos "K6%" ) ))
( "departmental"  j ( d ii2 p aa t m e1 n t l ) ((pos "OA%" ) ))
( "departments"  n ( d i1 p aa1 t m @ n t s ) ((pos "Kj%" ) ))
( "departs"  v ( d i1 p aa1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "departure"  n ( d i1 p aa1 ch @ r ) ((pos "M6%" ) ))
( "departures"  n ( d i1 p aa1 ch @ z ) ((pos "Mj%" ) ))
( "depend"  v ( d i1 p e1 n d ) ((pos "I0%" ) (subcat )))
( "dependable"  j ( d i1 p e1 n d @ b l ) ((pos "OA%" ) ))
( "dependant"  n ( d i1 p e1 n d @ n t ) ((pos "K6%" ) ))
( "dependants"  n ( d i1 p e1 n d @ n t s ) ((pos "Kj%" ) ))
( "depended"  v ( d i1 p e1 n d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "dependence"  n ( d i1 p e1 n d @ n s ) ((pos "L@%" ) ))
( "dependencies"  n ( d i1 p e1 n d @ n s i z ) ((pos "Kj%" ) ))
( "dependency"  n ( d i1 p e1 n d @ n s ii ) ((pos "K8%" ) ))
( "dependent"  n ( d i1 p e1 n d @ n t ) ((pos "K6%" "OA%" ) ))
( "dependents"  n ( d i1 p e1 n d @ n t s ) ((pos "Kj%" ) ))
( "depending"  v ( d i1 p e1 n d i ng ) ((pos "Ib%" ) (subcat )))
( "depends"  v ( d i1 p e1 n d z ) ((pos "Ia%" ) (subcat )))
( "depict"  v ( d i1 p i1 k t ) ((pos "H0%" ) (subcat )))
( "depicted"  v ( d i1 p i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "depicting"  v ( d i1 p i1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "depiction"  n ( d i1 p i1 k sh @ n ) ((pos "K6%" ) ))
( "depictions"  n ( d i1 p i1 k sh @ n z ) ((pos "Kj%" ) ))
( "depicts"  v ( d i1 p i1 k t s ) ((pos "Ha%" ) (subcat )))
( "depilatories"  n ( d i1 p i1 l @ t r i z ) ((pos "Mj$" ) ))
( "depilatory"  n ( d i1 p i1 l @ t r ii ) ((pos "M8$" "OA$" ) ))
( "deplane"  v ( d ii2 p l ei1 n ) ((pos "I2$" ) ))
( "deplaned"  v ( d ii2 p l ei1 n d ) ((pos "Ic$" "Id$" ) ))
( "deplanes"  v ( d ii2 p l ei1 n z ) ((pos "Ia$" ) ))
( "deplaning"  v ( d ii2 p l ei1 n i ng ) ((pos "Ib$" ) ))
( "deplete"  v ( d i1 p l ii1 t ) ((pos "H2%" ) (subcat "6A" )))
( "depleted"  v ( d i1 p l ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "depletes"  v ( d i1 p l ii1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "depleting"  v ( d i1 p l ii1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "depletion"  n ( d i1 p l ii1 sh @ n ) ((pos "L@%" ) ))
( "deplorable"  j ( d i1 p l oo1 r @ b l ) ((pos "OA%" ) ))
( "deplorably"  a ( d i1 p l oo1 r @ b l ii ) ((pos "Pu%" ) ))
( "deplore"  v ( d i1 p l oo1 r ) ((pos "H2%" ) (subcat )))
( "deplored"  v ( d i1 p l oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "deplores"  v ( d i1 p l oo1 z ) ((pos "Ha%" ) (subcat )))
( "deploring"  v ( d i1 p l oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "deploy"  v ( d i1 p l oi1 ) ((pos "J0%" ) (subcat "2A" )))
( "deployed"  v ( d i1 p l oi1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "deploying"  v ( d i1 p l oi1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "deployment"  n ( d i1 p l oi1 m @ n t ) ((pos "K6%" ) ))
( "deployments"  n ( d i1 p l oi1 m @ n t s ) ((pos "Kj%" ) ))
( "deploys"  v ( d i1 p l oi1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "deponent"  n ( d i1 p ou1 n @ n t ) ((pos "K6$" ) ))
( "deponents"  n ( d i1 p ou1 n @ n t s ) ((pos "Kj$" ) ))
( "depopulate"  v ( d ii2 p o1 p y u l ei t ) ((pos "H2%" ) (subcat )))
( "depopulated"  v ( d ii2 p o1 p y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "depopulates"  v ( d ii2 p o1 p y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "depopulating"  v ( d ii2 p o1 p y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "depopulation"  n ( d ii2 p o2 p y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "deport"  v ( d i1 p oo1 t ) ((pos "H0%" ) (subcat "6A" )))
( "deportation"  n ( d ii2 p oo t ei1 sh @ n ) ((pos "M6%" ) ))
( "deportations"  n ( d ii2 p oo t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "deported"  v ( d i1 p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deportee"  n ( d ii2 p oo t ii1 ) ((pos "K6%" ) ))
( "deportees"  n ( d ii2 p oo t ii1 z ) ((pos "Kj%" ) ))
( "deporting"  v ( d i1 p oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deportment"  n ( d i1 p oo1 t m @ n t ) ((pos "L@%" ) ))
( "deports"  v ( d i1 p oo1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "depose"  v ( d i1 p ou1 z ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "deposed"  v ( d i1 p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "deposes"  v ( d i1 p ou1 z i z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "deposing"  v ( d i1 p ou1 z i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "deposit"  v ( d i1 p o1 z i t ) ((pos "H0%" "K6%" ) (subcat )))
( "deposited"  v ( d i1 p o1 z i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "depositing"  v ( d i1 p o1 z i t i ng ) ((pos "Hb%" ) (subcat )))
( "deposition"  n ( d e2 p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "depositions"  n ( d e2 p @ z i1 sh @ n z ) ((pos "Mj$" ) ))
( "depositor"  n ( d i1 p o1 z i t @ r ) ((pos "K6%" ) ))
( "depositories"  n ( d i1 p o1 z i t r i z ) ((pos "Kj%" ) ))
( "depositors"  n ( d i1 p o1 z i t @ z ) ((pos "Kj%" ) ))
( "depository"  n ( d i1 p o1 z i t r ii ) ((pos "K8%" ) ))
( "deposits"  v ( d i1 p o1 z i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "depot"  n ( d e1 p ou ) ((pos "K6%" ) ))
( "depots"  n ( d e1 p ou z ) ((pos "Kj%" ) ))
( "deprave"  v ( d i1 p r ei1 v ) ((pos "H2%" ) (subcat )))
( "depraved"  v ( d i1 p r ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "depraves"  v ( d i1 p r ei1 v z ) ((pos "Ha%" ) (subcat )))
( "depraving"  v ( d i1 p r ei1 v i ng ) ((pos "Hb%" ) (subcat )))
( "depravities"  n ( d i1 p r a1 v i t i z ) ((pos "Mj$" ) ))
( "depravity"  n ( d i1 p r a1 v i t ii ) ((pos "M8%" ) ))
( "deprecate"  v ( d e1 p r @ k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "deprecated"  v ( d e1 p r @ k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deprecates"  v ( d e1 p r @ k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "deprecating"  v ( d e1 p r @ k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deprecation"  n ( d e2 p r @ k ei1 sh @ n ) ((pos "K6$" ) ))
( "deprecations"  n ( d e2 p r @ k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "depreciate"  v ( d i1 p r ii1 sh i ei t ) ((pos "J2%" ) (subcat "2A" )))
( "depreciated"  v ( d i1 p r ii1 sh i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "depreciates"  v ( d i1 p r ii1 sh i ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "depreciating"  v ( d i1 p r ii1 sh i ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "depreciation"  n ( d i1 p r ii2 sh i ei1 sh @ n ) ((pos "L@%" ) ))
( "depreciatory"  j ( d i1 p r ii1 sh @ t @ r ii ) ((pos "OA%" ) ))
( "depredation"  n ( d e2 p r @ d ei1 sh @ n ) ((pos "K6$" ) ))
( "depredations"  n ( d e2 p r @ d ei1 sh @ n z ) ((pos "Kj$" ) ))
( "depress"  v ( d i1 p r e1 s ) ((pos "H1%" ) (subcat )))
( "depressed"  v ( d i1 p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "depresses"  v ( d i1 p r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "depressing"  v ( d i1 p r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "depression"  n ( d i1 p r e1 sh @ n ) ((pos "M6%" ) ))
( "depressions"  n ( d i1 p r e1 sh @ n z ) ((pos "Mj%" ) ))
( "depressive"  n ( d i1 p r e1 s i v ) ((pos "K6%" "OA%" ) ))
( "depressives"  n ( d i1 p r e1 s i v z ) ((pos "Kj%" ) ))
( "deprivation"  n ( d e2 p r i v ei1 sh @ n ) ((pos "M6%" ) ))
( "deprivations"  n ( d e2 p r i v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "deprive"  v ( d i1 p r ai1 v ) ((pos "H2%" ) (subcat )))
( "deprived"  v ( d i1 p r ai1 v d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "deprives"  v ( d i1 p r ai1 v z ) ((pos "Ha%" ) (subcat )))
( "depriving"  v ( d i1 p r ai1 v i ng ) ((pos "Hb%" ) (subcat )))
( "depth"  n ( d e1 p th ) ((pos "M6%" ) ))
( "depth-bomb"  n ( d e1 p th - b o m ) ((pos "K6$" ) ))
( "depth-bombs"  n ( d e1 p th - b o m z ) ((pos "Kj$" ) ))
( "depth-charge"  n ( d e1 p th - ch aa jh ) ((pos "K6%" ) ))
( "depth-charges"  n ( d e1 p th - ch aa jh i z ) ((pos "Kj%" ) ))
( "depths"  n ( d e1 p th s ) ((pos "Mj%" ) ))
( "deputation"  n ( d e2 p y u t ei1 sh @ n ) ((pos "K6%" ) ))
( "deputations"  n ( d e2 p y u t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "depute"  v ( d i1 p y uu1 t ) ((pos "H2%" ) (subcat "14" )))
( "deputed"  v ( d i1 p y uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "deputes"  v ( d i1 p y uu1 t s ) ((pos "Ha%" ) (subcat "14" )))
( "deputies"  n ( d e1 p y u t i z ) ((pos "Kj%" ) ))
( "deputing"  v ( d i1 p y uu1 t i ng ) ((pos "Hb%" ) (subcat "14" )))
( "deputize"  v ( d e1 p y u t ai z ) ((pos "I2%" ) (subcat "2A" )))
( "deputized"  v ( d e1 p y u t ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "deputizes"  v ( d e1 p y u t ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "deputizing"  v ( d e1 p y u t ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "deputy"  n ( d e1 p y u t ii ) ((pos "K8%" ) ))
( "derail"  v ( d i1 r ei1 l ) ((pos "H0%" ) (subcat )))
( "derailed"  v ( d i1 r ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "derailing"  v ( d i1 r ei1 l i ng ) ((pos "Hb$" ) (subcat )))
( "derailment"  n ( d i1 r ei1 l m @ n t ) ((pos "K6%" ) ))
( "derailments"  n ( d i1 r ei1 l m @ n t s ) ((pos "Kj%" ) ))
( "derails"  v ( d i1 r ei1 l z ) ((pos "Ha$" ) (subcat )))
( "derange"  v ( d i1 r ei1 n jh ) ((pos "H2$" ) (subcat )))
( "deranged"  v ( d i1 r ei1 n jh d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "derangement"  n ( d i1 r ei1 n jh m @ n t ) ((pos "K6$" ) ))
( "derangements"  n ( d i1 r ei1 n jh m @ n t s ) ((pos "Kj$" ) ))
( "deranges"  v ( d i1 r ei1 n jh i z ) ((pos "Ha$" ) (subcat )))
( "deranging"  v ( d i1 r ei1 n jh i ng ) ((pos "Hb$" ) (subcat )))
( "derate"  v ( d ii2 r ei1 t ) ((pos "H2$" ) (subcat )))
( "derated"  v ( d ii2 r ei1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "derates"  v ( d ii2 r ei1 t s ) ((pos "Ha$" ) (subcat )))
( "derating"  v ( d ii2 r ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "derbies"  n ( d @@1 b i z ) ((pos "Kj$" ) ))
( "derbies"  n ( d aa1 b i z ) ((pos "Kj$" ) ))
( "derby"  n ( d @@1 b ii ) ((pos "K8$" ) ))
( "derby"  n ( d aa1 b ii ) ((pos "K8$" ) ))
( "derelict"  j ( d e1 r @ l i k t ) ((pos "OA%" ) ))
( "dereliction"  n ( d e2 r @ l i1 k sh @ n ) ((pos "L@%" ) ))
( "derequisition"  v ( d ii2 r e2 k w i z i1 sh @ n ) ((pos "H0$" ) (subcat )))
( "derequisitioned"  v ( d ii2 r e2 k w i z i1 sh @ n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "derequisitioning"  v ( d ii2 r e2 k w i z i1 sh @ n i ng ) ((pos "Hb$" ) (subcat )))
( "derequisitions"  v ( d ii2 r e2 k w i z i1 sh @ n z ) ((pos "Ha$" ) (subcat )))
( "derestrict"  v ( d ii2 r i s t r i1 k t ) ((pos "H0%" ) (subcat )))
( "derestricted"  v ( d ii2 r i s t r i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "derestricting"  v ( d ii2 r i s t r i1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "derestricts"  v ( d ii2 r i s t r i1 k t s ) ((pos "Ha%" ) (subcat )))
( "deride"  v ( d i1 r ai1 d ) ((pos "H2%" ) (subcat "6A" )))
( "derided"  v ( d i1 r ai1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "derides"  v ( d i1 r ai1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "deriding"  v ( d i1 r ai1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "derision"  n ( d i1 r i1 zh n ) ((pos "L@%" ) ))
( "derisive"  j ( d i1 r ai1 s i v ) ((pos "OA%" ) ))
( "derisively"  a ( d i1 r ai1 s i v l ii ) ((pos "Pu%" ) ))
( "derisory"  j ( d i1 r ai1 s @ r ii ) ((pos "OA%" ) ))
( "derivation"  n ( d e2 r i v ei1 sh @ n ) ((pos "M6%" ) ))
( "derivations"  n ( d e2 r i v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "derivative"  n ( d i1 r i1 v @ t i v ) ((pos "K6%" "OA%" ) ))
( "derivatives"  n ( d i1 r i1 v @ t i v z ) ((pos "Kj%" ) ))
( "derive"  v ( d i1 r ai1 v ) ((pos "J2%" ) (subcat "3A" )))
( "derived"  v ( d i1 r ai1 v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "derives"  v ( d i1 r ai1 v z ) ((pos "Ja%" ) (subcat "3A" )))
( "deriving"  v ( d i1 r ai1 v i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "dermatitis"  n ( d @@2 m @ t ai1 t i s ) ((pos "L@%" ) ))
( "dermatologist"  n ( d @@2 m @ t o1 l @ jh i s t ) ((pos "K6%" ) ))
( "dermatologists"  n ( d @@2 m @ t o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "dermatology"  n ( d @@2 m @ t o1 l @ jh ii ) ((pos "L@%" ) ))
( "derogate"  v ( d e1 r @ g ei t ) ((pos "I2$" ) (subcat )))
( "derogated"  v ( d e1 r @ g ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "derogates"  v ( d e1 r @ g ei t s ) ((pos "Ia$" ) (subcat )))
( "derogating"  v ( d e1 r @ g ei t i ng ) ((pos "Ib$" ) (subcat )))
( "derogation"  n ( d e2 r @ g ei1 sh @ n ) ((pos "L@$" ) ))
( "derogatory"  j ( d i1 r o1 g @ t r ii ) ((pos "OA%" ) ))
( "derrick"  n ( d e1 r i k ) ((pos "K6%" ) ))
( "derricks"  n ( d e1 r i k s ) ((pos "Kj%" ) ))
( "derring-do"  n ( d e2 r i ng - d uu1 ) ((pos "L@$" ) ))
( "derv"  n ( d @@1 v ) ((pos "L@$" ) ))
( "dervish"  n ( d @@1 v i sh ) ((pos "K7$" ) ))
( "dervishes"  n ( d @@1 v i sh i z ) ((pos "Kj$" ) ))
( "desalinate"  v ( d ii2 s a1 l i n ei t ) ((pos "H2%" ) (subcat )))
( "desalinated"  v ( d ii2 s a1 l i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "desalinates"  v ( d ii2 s a1 l i n ei t s ) ((pos "Ha%" ) (subcat )))
( "desalinating"  v ( d ii2 s a1 l i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "desalination"  n ( d ii2 s a2 l i n ei1 sh @ n ) ((pos "L@%" ) ))
( "desalinization"  n ( d ii2 s a2 l i n ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "desalinize"  v ( d ii2 s a1 l i n ai z ) ((pos "H2$" ) (subcat )))
( "desalinized"  v ( d ii2 s a1 l i n ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "desalinizes"  v ( d ii2 s a1 l i n ai z i z ) ((pos "Ha$" ) (subcat )))
( "desalinizing"  v ( d ii2 s a1 l i n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "desalt"  v ( d ii2 s oo1 l t ) ((pos "H0$" ) ))
( "desalted"  v ( d ii2 s oo1 l t i d ) ((pos "Hc$" "Hd$" ) ))
( "desalting"  v ( d ii2 s oo1 l t i ng ) ((pos "Hb$" ) ))
( "desalts"  v ( d ii2 s oo1 l t s ) ((pos "Ha$" ) ))
( "descale"  v ( d ii2 s k ei1 l ) ((pos "H2%" ) (subcat )))
( "descaled"  v ( d ii2 s k ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "descales"  v ( d ii2 s k ei1 l z ) ((pos "Ha%" ) (subcat )))
( "descaling"  v ( d ii2 s k ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "descant"  n ( d e1 s k a n t ) ((pos "K6%" ) ))
( "descant"  v ( d i1 s k a1 n t ) ((pos "I0$" ) (subcat )))
( "descanted"  v ( d i1 s k a1 n t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "descanting"  v ( d i1 s k a1 n t i ng ) ((pos "Ib$" ) (subcat )))
( "descants"  n ( d e1 s k a n t s ) ((pos "Kj%" ) ))
( "descants"  v ( d i1 s k a1 n t s ) ((pos "Ia$" ) (subcat )))
( "descend"  v ( d i1 s e1 n d ) ((pos "J0%" ) (subcat "2A" "2C" "3A" )))
( "descendant"  n ( d i1 s e1 n d @ n t ) ((pos "K6%" ) ))
( "descendants"  n ( d i1 s e1 n d @ n t s ) ((pos "Kj%" ) ))
( "descended"  v ( d i1 s e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "descending"  v ( d i1 s e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "descends"  v ( d i1 s e1 n d z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "descent"  n ( d i1 s e1 n t ) ((pos "M6%" ) ))
( "descents"  n ( d i1 s e1 n t s ) ((pos "Mj%" ) ))
( "describe"  v ( d i1 s k r ai1 b ) ((pos "H2%" ) (subcat "6A" "10" "14" )))
( "described"  v ( d i1 s k r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "10" "14" )))
( "describes"  v ( d i1 s k r ai1 b z ) ((pos "Ha%" ) (subcat "6A" "10" "14" )))
( "describing"  v ( d i1 s k r ai1 b i ng ) ((pos "Hb%" ) (subcat "6A" "10" "14" )))
( "descried"  v ( d i1 s k r ai1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "descries"  v ( d i1 s k r ai1 z ) ((pos "Ha$" ) (subcat )))
( "description"  n ( d i1 s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "descriptions"  n ( d i1 s k r i1 p sh @ n z ) ((pos "Mj%" ) ))
( "descriptive"  j ( d i1 s k r i1 p t i v ) ((pos "OA%" ) ))
( "descriptively"  a ( d i1 s k r i1 p t i v l ii ) ((pos "Pu%" ) ))
( "descry"  v ( d i1 s k r ai1 ) ((pos "H3$" ) (subcat )))
( "descrying"  v ( d i1 s k r ai1 i ng ) ((pos "Hb$" ) (subcat )))
( "desecrate"  v ( d e1 s i k r ei t ) ((pos "H2%" ) (subcat )))
( "desecrated"  v ( d e1 s i k r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "desecrates"  v ( d e1 s i k r ei t s ) ((pos "Ha%" ) (subcat )))
( "desecrating"  v ( d e1 s i k r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "desecration"  n ( d e2 s i k r ei1 sh @ n ) ((pos "L@%" ) ))
( "desegregate"  v ( d ii2 s e1 g r i g ei t ) ((pos "H2%" ) (subcat )))
( "desegregated"  v ( d ii2 s e1 g r i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "desegregates"  v ( d ii2 s e1 g r i g ei t s ) ((pos "Ha%" ) (subcat )))
( "desegregating"  v ( d ii2 s e1 g r i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "desegregation"  n ( d ii2 s e2 g r i g ei1 sh @ n ) ((pos "L@%" ) ))
( "desensitization"  n ( d ii2 s e2 n s i t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "desensitize"  v ( d ii2 s e1 n s i t ai z ) ((pos "H2%" ) (subcat )))
( "desensitized"  v ( d ii2 s e1 n s i t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "desensitizes"  v ( d ii2 s e1 n s i t ai z i z ) ((pos "Ha%" ) (subcat )))
( "desensitizing"  v ( d ii2 s e1 n s i t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "desert"  n ( d e1 z @ t ) ((pos "M6%" "OA%" ) ))
( "desert"  v ( d i1 z @@1 t ) ((pos "J0%" ) (subcat "2A" )))
( "deserted"  v ( d i1 z @@1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "deserter"  n ( d i1 z @@1 t @ r ) ((pos "K6%" ) ))
( "deserters"  n ( d i1 z @@1 t @ z ) ((pos "Kj%" ) ))
( "deserting"  v ( d i1 z @@1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "desertion"  n ( d i1 z @@1 sh @ n ) ((pos "M6%" ) ))
( "desertions"  n ( d i1 z @@1 sh @ n z ) ((pos "Mj%" ) ))
( "deserts"  n ( d e1 z @ t s ) ((pos "Mj%" ) ))
( "deserts"  v ( d i1 z @@1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "deserve"  v ( d i1 z @@1 v ) ((pos "J2%" ) (subcat "6A" )))
( "deserved"  v ( d i1 z @@1 v d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "6A" )))
( "deservedly"  a ( d i1 z @@1 v i d l ii ) ((pos "Pu%" ) ))
( "deserves"  v ( d i1 z @@1 v z ) ((pos "Ja%" ) (subcat "6A" )))
( "deserving"  v ( d i1 z @@1 v i ng ) ((pos "Jb%" "OA%" ) (subcat "6A" )))
( "desiccant"  n ( d e1 s i k @ n t ) ((pos "K6$" ) ))
( "desiccants"  n ( d e1 s i k @ n t s ) ((pos "Kj$" ) ))
( "desiccate"  v ( d e1 s i k ei t ) ((pos "H2$" ) (subcat )))
( "desiccated"  v ( d e1 s i k ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "desiccates"  v ( d e1 s i k ei t s ) ((pos "Ha$" ) (subcat )))
( "desiccating"  v ( d e1 s i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "desiderata"  n ( d i1 z i2 d @ r aa1 t @ ) ((pos "Kj$" ) ))
( "desideratum"  n ( d i1 z i2 d @ r aa1 t @ m ) ((pos "Ki$" ) ))
( "design"  v ( d i1 z ai1 n ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "designate"  v ( d e1 z i g n ei t ) ((pos "H2%" "OA%" ) (subcat "6A" "16B" )))
( "designated"  v ( d e1 z i g n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "16B" )))
( "designates"  v ( d e1 z i g n ei t s ) ((pos "Ha%" ) (subcat "6A" "16B" )))
( "designating"  v ( d e1 z i g n ei t i ng ) ((pos "Hb%" ) (subcat "6A" "16B" )))
( "designation"  n ( d e2 z i g n ei1 sh @ n ) ((pos "M6%" ) ))
( "designations"  n ( d e2 z i g n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "designed"  v ( d i1 z ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "designedly"  a ( d ii1 z ai1 n i d l ii ) ((pos "Pu%" ) ))
( "designer"  n ( d i1 z ai1 n @ r ) ((pos "K6%" ) ))
( "designers"  n ( d i1 z ai1 n @ z ) ((pos "Kj%" ) ))
( "designing"  v ( d i1 z ai1 n i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "designs"  v ( d i1 z ai1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "desirability"  n ( d i1 z ai2 @ r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "desirable"  j ( d i1 z ai1 @ r @ b l ) ((pos "OA%" ) ))
( "desire"  v ( d i1 z ai1 @ r ) ((pos "H2%" "M6%" ) (subcat "6A" "7A" "9" )))
( "desired"  v ( d i1 z ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" "9" )))
( "desires"  v ( d i1 z ai1 @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "7A" "9" )))
( "desiring"  v ( d i1 z ai1 @ r i ng ) ((pos "Hb%" ) (subcat "6A" "7A" "9" )))
( "desirous"  j ( d i1 z ai1 @ r @ s ) ((pos "OA%" ) ))
( "desist"  v ( d i1 z i1 s t ) ((pos "I0%" ) (subcat "2A" )))
( "desisted"  v ( d i1 z i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "desisting"  v ( d i1 z i1 s t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "desists"  v ( d i1 z i1 s t s ) ((pos "Ia$" ) (subcat "2A" )))
( "desk"  n ( d e1 s k ) ((pos "K6%" ) ))
( "desks"  n ( d e1 s k s ) ((pos "Kj%" ) ))
( "desolate"  j ( d e1 s @ l @ t ) ((pos "OA%" ) ))
( "desolate"  v ( d e1 s @ l ei t ) ((pos "H2%" ) (subcat )))
( "desolated"  v ( d e1 s @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "desolately"  a ( d e1 s @ l @ t l ii ) ((pos "Pu%" ) ))
( "desolates"  v ( d e1 s @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "desolating"  v ( d e1 s @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "desolation"  n ( d e2 s @ l ei1 sh @ n ) ((pos "L@%" ) ))
( "despair"  v ( d i1 s p e@1 r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "despaired"  v ( d i1 s p e@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "despairing"  v ( d i1 s p e@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "despairingly"  a ( d i1 s p e@1 r i ng l ii ) ((pos "Pu%" ) ))
( "despairs"  v ( d i1 s p e@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "despatch"  v ( d i1 s p a1 ch ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "despatched"  v ( d i1 s p a1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "despatches"  v ( d i1 s p a1 ch i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "despatching"  v ( d i1 s p a1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "desperado"  n ( d e2 s p @ r aa1 d ou ) ((pos "K7%" ) ))
( "desperadoes"  n ( d e2 s p @ r aa1 d ou z ) ((pos "Kj%" ) ))
( "desperate"  j ( d e1 s p @ r @ t ) ((pos "OA%" ) ))
( "desperately"  a ( d e1 s p @ r @ t l ii ) ((pos "Pu%" ) ))
( "desperation"  n ( d e2 s p @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "despicable"  j ( d i1 s p i1 k @ b l ) ((pos "OA%" ) ))
( "despicably"  a ( d i1 s p i1 k @ b l ii ) ((pos "Pu%" ) ))
( "despise"  v ( d i1 s p ai1 z ) ((pos "H2%" ) (subcat )))
( "despised"  v ( d i1 s p ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "despises"  v ( d i1 s p ai1 z i z ) ((pos "Ha%" ) (subcat )))
( "despising"  v ( d i1 s p ai1 z i ng ) ((pos "Hb%" ) (subcat )))
( "despite"  n ( d i1 s p ai1 t ) ((pos "L@$" "T-%" ) ))
( "despiteful"  j ( d i1 s p ai1 t f @ l ) ((pos "OA$" ) ))
( "despitefully"  a ( d i1 s p ai1 t f @ l ii ) ((pos "Pu$" ) ))
( "despoil"  v ( d i1 s p oi1 l ) ((pos "H0%" ) (subcat "6A" )))
( "despoiled"  v ( d i1 s p oi1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "despoiling"  v ( d i1 s p oi1 l i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "despoils"  v ( d i1 s p oi1 l z ) ((pos "Ha$" ) (subcat "6A" )))
( "despondency"  n ( d i1 s p o1 n d @ n s ii ) ((pos "L@%" ) ))
( "despondent"  j ( d i1 s p o1 n d @ n t ) ((pos "OA%" ) ))
( "despondently"  a ( d i1 s p o1 n d @ n t l ii ) ((pos "Pu%" ) ))
( "despot"  n ( d e1 s p o t ) ((pos "K6%" ) ))
( "despotic"  j ( d i1 s p o1 t i k ) ((pos "OA%" ) ))
( "despotism"  n ( d e1 s p @ t i z @ m ) ((pos "M6%" ) ))
( "despotisms"  n ( d e1 s p @ t i z @ m z ) ((pos "Mj$" ) ))
( "despots"  n ( d e1 s p o t s ) ((pos "Kj%" ) ))
( "dessert"  n ( d i1 z @@1 t ) ((pos "K6%" ) ))
( "desserts"  n ( d i1 z @@1 t s ) ((pos "Kj%" ) ))
( "dessertspoon"  n ( d i1 z @@1 t s p uu n ) ((pos "K6%" ) ))
( "dessertspoonful"  n ( d i1 z @@1 t s p uu n f u l ) ((pos "K6%" ) ))
( "dessertspoonfuls"  n ( d i1 z @@1 t s p uu n f u l z ) ((pos "Kj%" ) ))
( "dessertspoons"  n ( d i1 z @@1 t s p uu n z ) ((pos "Kj%" ) ))
( "destination"  n ( d e2 s t i n ei1 sh @ n ) ((pos "K6%" ) ))
( "destinations"  n ( d e2 s t i n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "destine"  v ( d e1 s t i n ) ((pos "H2$" ) (subcat "14" )))
( "destined"  v ( d e1 s t i n d ) ((pos "Hc$" "Hd%" ) (subcat "14" )))
( "destines"  v ( d e1 s t i n z ) ((pos "Ha$" ) (subcat "14" )))
( "destinies"  n ( d e1 s t i n i z ) ((pos "Mj%" ) ))
( "destining"  v ( d e1 s t i n i ng ) ((pos "Hb$" ) (subcat "14" )))
( "destiny"  n ( d e1 s t i n ii ) ((pos "M8%" ) ))
( "destitute"  j ( d e1 s t i t y uu t ) ((pos "OA%" ) ))
( "destitution"  n ( d e2 s t i t y uu1 sh @ n ) ((pos "L@%" ) ))
( "destroy"  v ( d i1 s t r oi1 ) ((pos "H0%" ) (subcat )))
( "destroyed"  v ( d i1 s t r oi1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "destroyer"  n ( d i1 s t r oi1 @ r ) ((pos "K6%" ) ))
( "destroyers"  n ( d i1 s t r oi1 @ z ) ((pos "Kj%" ) ))
( "destroying"  v ( d i1 s t r oi1 i ng ) ((pos "Hb%" ) (subcat )))
( "destroys"  v ( d i1 s t r oi1 z ) ((pos "Ha%" ) (subcat )))
( "destructibility"  n ( d i1 s t r uh2 k t @ b i1 l i t ii ) ((pos "L@$" ) ))
( "destructible"  j ( d i1 s t r uh1 k t @ b l ) ((pos "OA%" ) ))
( "destruction"  n ( d i1 s t r uh1 k sh @ n ) ((pos "L@%" ) ))
( "destructive"  j ( d i1 s t r uh1 k t i v ) ((pos "OA%" ) ))
( "destructively"  a ( d i1 s t r uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "destructiveness"  n ( d i1 s t r uh1 k t i v n @ s ) ((pos "L@%" ) ))
( "desuetude"  n ( d i1 s y uu1 i t y uu d ) ((pos "L@$" ) ))
( "desultory"  j ( d e1 s @ l t r ii ) ((pos "OA%" ) ))
( "detach"  v ( d i1 t a1 ch ) ((pos "H1%" ) (subcat "6A" "14" )))
( "detachable"  j ( d i1 t a1 ch @ b l ) ((pos "OA%" ) ))
( "detached"  v ( d i1 t a1 ch t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "14" )))
( "detaches"  v ( d i1 t a1 ch i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "detaching"  v ( d i1 t a1 ch i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "detachment"  n ( d i1 t a1 ch m @ n t ) ((pos "M6%" ) ))
( "detachments"  n ( d i1 t a1 ch m @ n t s ) ((pos "Mj%" ) ))
( "detail"  v ( d ii1 t ei l ) ((pos "H0%" "M6%" ) (subcat "6A" "14" )))
( "detailed"  v ( d ii1 t ei l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "detailing"  v ( d ii1 t ei l i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "details"  v ( d ii1 t ei l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "detain"  v ( d i1 t ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "detained"  v ( d i1 t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "detainee"  n ( d ii2 t ei n ii1 ) ((pos "K6%" ) ))
( "detainees"  n ( d ii2 t ei n ii1 z ) ((pos "Kj%" ) ))
( "detaining"  v ( d i1 t ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "detains"  v ( d i1 t ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "detect"  v ( d i1 t e1 k t ) ((pos "H0%" ) (subcat )))
( "detectable"  j ( d i1 t e1 k t @ b l ) ((pos "OA%" ) ))
( "detected"  v ( d i1 t e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "detecting"  v ( d i1 t e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "detection"  n ( d i1 t e1 k sh @ n ) ((pos "L@%" ) ))
( "detective"  n ( d i1 t e1 k t i v ) ((pos "K6%" ) ))
( "detectives"  n ( d i1 t e1 k t i v z ) ((pos "Kj%" ) ))
( "detector"  n ( d i1 t e1 k t @ r ) ((pos "K6%" ) ))
( "detectors"  n ( d i1 t e1 k t @ z ) ((pos "Kj%" ) ))
( "detects"  v ( d i1 t e1 k t s ) ((pos "Ha%" ) (subcat )))
( "detention"  n ( d i1 t e1 n sh @ n ) ((pos "M6%" ) ))
( "detentions"  n ( d i1 t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "deter"  v ( d i1 t @@1 r ) ((pos "H4%" ) (subcat "6A" )))
( "detergent"  n ( d i1 t @@1 jh @ n t ) ((pos "K6%" "OA%" ) ))
( "detergents"  n ( d i1 t @@1 jh @ n t s ) ((pos "Kj%" ) ))
( "deteriorate"  v ( d i1 t i@1 r i@ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "deteriorated"  v ( d i1 t i@1 r i@ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "deteriorates"  v ( d i1 t i@1 r i@ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "deteriorating"  v ( d i1 t i@1 r i@ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "deterioration"  n ( d i1 t i@2 r i@ r ei1 sh @ n ) ((pos "M6%" ) ))
( "deteriorations"  n ( d i1 t i@2 r i@ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "determinable"  j ( d i1 t @@1 m i n @ b l ) ((pos "OA%" ) ))
( "determinant"  n ( d i1 t @@1 m i n @ n t ) ((pos "K6$" "OA$" ) ))
( "determinants"  n ( d i1 t @@1 m i n @ n t s ) ((pos "Kj$" ) ))
( "determinate"  j ( d i1 t @@1 m i n @ t ) ((pos "OA$" ) ))
( "determination"  n ( d i1 t @@2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "determinative"  n ( d i1 t @@1 m i n @ t i v ) ((pos "K6$" "OA$" ) ))
( "determinatives"  n ( d i1 t @@1 m i n @ t i v z ) ((pos "Kj$" ) ))
( "determine"  v ( d i1 t @@1 m i n ) ((pos "J2%" ) (subcat "3A" "6A" "7A" "8" "9" "10" "14" )))
( "determined"  v ( d i1 t @@1 m i n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "8" "9" "10" "14" )))
( "determinedly"  a ( d i1 t @@1 m i n d l ii ) ((pos "Pu%" ) ))
( "determiner"  n ( d i1 t @@1 m i n @ r ) ((pos "K6$" ) ))
( "determiners"  n ( d i1 t @@1 m i n @ z ) ((pos "Kj$" ) ))
( "determines"  v ( d i1 t @@1 m i n z ) ((pos "Ja$" ) (subcat "3A" "6A" "7A" "8" "9" "10" "14" )))
( "determining"  v ( d i1 t @@1 m i n i ng ) ((pos "Jb$" ) (subcat "3A" "6A" "7A" "8" "9" "10" "14" )))
( "deterred"  v ( d i1 t @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "deterrent"  n ( d i1 t e1 r @ n t ) ((pos "K6%" "OA%" ) ))
( "deterrents"  n ( d i1 t e1 r @ n t s ) ((pos "Kj%" ) ))
( "deterring"  v ( d i1 t @@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "deters"  v ( d i1 t @@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "detest"  v ( d i1 t e1 s t ) ((pos "H0%" ) (subcat "6A" )))
( "detestable"  j ( d i1 t e1 s t @ b l ) ((pos "OA%" ) ))
( "detestably"  a ( d i1 t e1 s t @ b l ii ) ((pos "Pu%" ) ))
( "detestation"  n ( d ii2 t e s t ei1 sh @ n ) ((pos "M6%" ) ))
( "detestations"  n ( d ii2 t e s t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "detested"  v ( d i1 t e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "detesting"  v ( d i1 t e1 s t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "detests"  v ( d i1 t e1 s t s ) ((pos "Ha%" ) (subcat "6A" )))
( "dethrone"  v ( d ii2 th r ou1 n ) ((pos "H2%" ) (subcat )))
( "dethroned"  v ( d ii2 th r ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dethronement"  n ( d ii2 th r ou1 n m @ n t ) ((pos "K6$" ) ))
( "dethronements"  n ( d ii2 th r ou1 n m @ n t s ) ((pos "Kj$" ) ))
( "dethrones"  v ( d ii2 th r ou1 n z ) ((pos "Ha%" ) (subcat )))
( "dethroning"  v ( d ii2 th r ou1 n i ng ) ((pos "Hb$" ) (subcat )))
( "detonate"  v ( d e1 t @ n ei t ) ((pos "J2%" ) (subcat "2A" )))
( "detonated"  v ( d e1 t @ n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "detonates"  v ( d e1 t @ n ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "detonating"  v ( d e1 t @ n ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "detonation"  n ( d e2 t @ n ei1 sh @ n ) ((pos "K6%" ) ))
( "detonations"  n ( d e2 t @ n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "detonator"  n ( d e1 t @ n ei t @ r ) ((pos "K6%" ) ))
( "detonators"  n ( d e1 t @ n ei t @ z ) ((pos "Kj%" ) ))
( "detour"  v ( d ii1 t u@ r ) ((pos "H0$" "K6%" ) (subcat )))
( "detoured"  v ( d ii1 t u@ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "detouring"  v ( d ii1 t u@ r i ng ) ((pos "Hb$" ) (subcat )))
( "detours"  v ( d ii1 t u@ z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "detract"  v ( d i1 t r a1 k t ) ((pos "I0%" ) (subcat )))
( "detracted"  v ( d i1 t r a1 k t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "detracting"  v ( d i1 t r a1 k t i ng ) ((pos "Ib%" ) (subcat )))
( "detraction"  n ( d i1 t r a1 k sh @ n ) ((pos "M6%" ) ))
( "detractions"  n ( d i1 t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "detractor"  n ( d i1 t r a1 k t @ r ) ((pos "K6%" ) ))
( "detractors"  n ( d i1 t r a1 k t @ z ) ((pos "Kj%" ) ))
( "detracts"  v ( d i1 t r a1 k t s ) ((pos "Ia%" ) (subcat )))
( "detrain"  v ( d ii2 t r ei1 n ) ((pos "J0$" ) (subcat "2A" )))
( "detrained"  v ( d ii2 t r ei1 n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "detraining"  v ( d ii2 t r ei1 n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "detrains"  v ( d ii2 t r ei1 n z ) ((pos "Ja$" ) (subcat "2A" )))
( "detribalization"  n ( d ii2 t r ai2 b @ l ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "detribalize"  v ( d ii2 t r ai1 b @ l ai z ) ((pos "H2$" ) (subcat )))
( "detribalized"  v ( d ii2 t r ai1 b @ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "detribalizes"  v ( d ii2 t r ai1 b @ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "detribalizing"  v ( d ii2 t r ai1 b @ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "detriment"  n ( d e1 t r i m @ n t ) ((pos "L@%" ) ))
( "detrimental"  j ( d e2 t r i m e1 n t l ) ((pos "OA%" ) ))
( "detrimentally"  a ( d e2 t r i m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "detritus"  n ( d i1 t r ai1 t @ s ) ((pos "L@%" ) ))
( "deuce"  n ( d y uu1 s ) ((pos "K6$" ) ))
( "deuced"  j ( d y uu1 s t ) ((pos "OA$" ) ))
( "deucedly"  a ( d y uu1 s i d l ii ) ((pos "Pu$" ) ))
( "deuces"  n ( d y uu1 s i z ) ((pos "Kj$" ) ))
( "devaluate"  v ( d ii2 v a1 l y u ei t ) ((pos "H2$" ) (subcat )))
( "devaluated"  v ( d ii2 v a1 l y u ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "devaluates"  v ( d ii2 v a1 l y u ei t s ) ((pos "Ha$" ) (subcat )))
( "devaluating"  v ( d ii2 v a1 l y u ei t i ng ) ((pos "Hb$" ) (subcat )))
( "devaluation"  n ( d ii2 v a2 l y u ei1 sh @ n ) ((pos "M6%" ) ))
( "devaluations"  n ( d ii2 v a2 l y u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "devalue"  v ( d ii2 v a1 l y uu ) ((pos "H2%" ) (subcat )))
( "devalued"  v ( d ii2 v a1 l y uu d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "devalues"  v ( d ii2 v a1 l y uu z ) ((pos "Ha%" ) (subcat )))
( "devaluing"  v ( d ii2 v a1 l y uu i ng ) ((pos "Hb%" ) (subcat )))
( "devastate"  v ( d e1 v @ s t ei t ) ((pos "H2%" ) (subcat )))
( "devastated"  v ( d e1 v @ s t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "devastates"  v ( d e1 v @ s t ei t s ) ((pos "Ha%" ) (subcat )))
( "devastating"  v ( d e1 v @ s t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "devastation"  n ( d e2 v @ s t ei1 sh @ n ) ((pos "L@%" ) ))
( "develop"  v ( d i1 v e1 l @ p ) ((pos "J0%" ) (subcat "2A" "2C" "3A" )))
( "developed"  v ( d i1 v e1 l @ p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "developer"  n ( d i1 v e1 l @ p @ r ) ((pos "K6%" ) ))
( "developers"  n ( d i1 v e1 l @ p @ z ) ((pos "Kj%" ) ))
( "developing"  v ( d i1 v e1 l @ p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "development"  n ( d i1 v e1 l @ p m @ n t ) ((pos "M6%" ) ))
( "developments"  n ( d i1 v e1 l @ p m @ n t s ) ((pos "Mj%" ) ))
( "develops"  v ( d i1 v e1 l @ p s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "deviant"  n ( d ii1 v i@ n t ) ((pos "K6%" "OA%" ) ))
( "deviants"  n ( d ii1 v i@ n t s ) ((pos "Kj%" ) ))
( "deviate"  v ( d ii1 v i ei t ) ((pos "I2%" ) (subcat )))
( "deviated"  v ( d ii1 v i ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "deviates"  v ( d ii1 v i ei t s ) ((pos "Ia%" ) (subcat )))
( "deviating"  v ( d ii1 v i ei t i ng ) ((pos "Ib%" ) (subcat )))
( "deviation"  n ( d ii2 v i ei1 sh @ n ) ((pos "M6%" ) ))
( "deviationism"  n ( d ii2 v i ei1 sh @ n i z @ m ) ((pos "L@$" ) ))
( "deviationist"  n ( d ii2 v i ei1 sh @ n i s t ) ((pos "K6$" ) ))
( "deviationists"  n ( d ii2 v i ei1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "deviations"  n ( d ii2 v i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "device"  n ( d i1 v ai1 s ) ((pos "K6%" ) ))
( "devices"  n ( d i1 v ai1 s i z ) ((pos "Kj%" ) ))
( "devil"  v ( d e1 v @ l ) ((pos "J4%" "K6%" ) (subcat "2A" "3A" )))
( "devil-may-care"  j ( d e2 v l - m ei - k e@1 r ) ((pos "OA%" ) ))
( "devilish"  j ( d e1 v @ l i sh ) ((pos "OA%" "Pu%" ) ))
( "devilishly"  a ( d e1 v @ l i sh l ii ) ((pos "Pu%" ) ))
( "devilled"  v ( d e1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "devilling"  v ( d e1 v @ l i ng ) ((pos "Jb$" ) (subcat "2A" "3A" )))
( "devilment"  n ( d e1 v @ l m @ n t ) ((pos "M6%" ) ))
( "devilments"  n ( d e1 v @ l m @ n t s ) ((pos "Mj$" ) ))
( "devilries"  n ( d e1 v @ l r i z ) ((pos "Mj$" ) ))
( "devilry"  n ( d e1 v @ l r ii ) ((pos "M8%" ) ))
( "devils"  v ( d e1 v @ l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "devious"  j ( d ii1 v i@ s ) ((pos "OA%" ) ))
( "deviously"  a ( d ii1 v i@ s l ii ) ((pos "Pu%" ) ))
( "deviousness"  n ( d ii1 v i@ s n @ s ) ((pos "L@%" ) ))
( "devise"  v ( d i1 v ai1 z ) ((pos "H2%" ) (subcat "6A" "8" )))
( "devised"  v ( d i1 v ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" )))
( "devises"  v ( d i1 v ai1 z i z ) ((pos "Ha%" ) (subcat "6A" "8" )))
( "devising"  v ( d i1 v ai1 z i ng ) ((pos "Hb%" ) (subcat "6A" "8" )))
( "devitalization"  n ( d ii2 v ai2 t @ l ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "devitalize"  v ( d ii2 v ai1 t @ l ai z ) ((pos "H2$" ) (subcat )))
( "devitalized"  v ( d ii2 v ai1 t @ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "devitalizes"  v ( d ii2 v ai1 t @ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "devitalizing"  v ( d ii2 v ai1 t @ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "devoid"  j ( d i1 v oi1 d ) ((pos "OA%" ) ))
( "devolution"  n ( d ii2 v @ l uu1 sh @ n ) ((pos "L@%" ) ))
( "devolve"  v ( d i1 v o1 l v ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "devolved"  v ( d i1 v o1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "devolves"  v ( d i1 v o1 l v z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "devolving"  v ( d i1 v o1 l v i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "devote"  v ( d i1 v ou1 t ) ((pos "H2%" ) (subcat )))
( "devoted"  v ( d i1 v ou1 t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "devotedly"  a ( d i1 v ou1 t i d l ii ) ((pos "Pu%" ) ))
( "devotee"  n ( d e2 v @ t ii1 ) ((pos "K6%" ) ))
( "devotees"  n ( d e2 v @ t ii1 z ) ((pos "Kj%" ) ))
( "devotes"  v ( d i1 v ou1 t s ) ((pos "Ha%" ) (subcat )))
( "devoting"  v ( d i1 v ou1 t i ng ) ((pos "Hb%" ) (subcat )))
( "devotion"  n ( d i1 v ou1 sh @ n ) ((pos "M6%" ) ))
( "devotional"  j ( d i1 v ou1 sh @ n @ l ) ((pos "OA%" ) ))
( "devotions"  n ( d i1 v ou1 sh @ n z ) ((pos "Mj%" ) ))
( "devour"  v ( d i1 v au1 @ r ) ((pos "H0%" ) (subcat )))
( "devoured"  v ( d i1 v au1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "devouring"  v ( d i1 v au1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "devours"  v ( d i1 v au1 @ z ) ((pos "Ha%" ) (subcat )))
( "devout"  j ( d i1 v au1 t ) ((pos "OA%" ) ))
( "devoutly"  a ( d i1 v au1 t l ii ) ((pos "Pu%" ) ))
( "devoutness"  n ( d i1 v au1 t n @ s ) ((pos "L@%" ) ))
( "dew"  n ( d y uu1 ) ((pos "L@%" ) ))
( "dewier"  j ( d y uu1 i@ r ) ((pos "Or%" ) ))
( "dewiest"  j ( d y uu1 i i s t ) ((pos "Os%" ) ))
( "dewlap"  n ( d y uu1 l a p ) ((pos "K6%" ) ))
( "dewlaps"  n ( d y uu1 l a p s ) ((pos "Kj%" ) ))
( "dewy"  j ( d y uu1 ii ) ((pos "OD%" ) ))
( "dexterity"  n ( d e2 k s t e1 r i t ii ) ((pos "L@%" ) ))
( "dexterous"  j ( d e1 k s t r @ s ) ((pos "OA%" ) ))
( "dexterously"  a ( d e1 k s t r @ s l ii ) ((pos "Pu%" ) ))
( "dextrose"  n ( d e1 k s t r ou z ) ((pos "L@%" ) ))
( "dextrous"  j ( d e1 k s t r @ s ) ((pos "OA%" ) ))
( "dhoti"  n ( d ou1 t ii ) ((pos "K6$" ) ))
( "dhotis"  n ( d ou1 t i z ) ((pos "Kj$" ) ))
( "dhow"  n ( d au1 ) ((pos "K6$" ) ))
( "dhows"  n ( d au1 z ) ((pos "Kj$" ) ))
( "diabetes"  n ( d ai2 @ b ii1 t i z ) ((pos "L@%" ) ))
( "diabetic"  n ( d ai2 @ b e1 t i k ) ((pos "K6%" "OA%" ) ))
( "diabetics"  n ( d ai2 @ b e1 t i k s ) ((pos "Kj%" ) ))
( "diabolic"  j ( d ai2 @ b o1 l i k ) ((pos "OA$" ) ))
( "diabolical"  j ( d ai2 @ b o1 l i k l ) ((pos "OA%" ) ))
( "diabolically"  a ( d ai2 @ b o1 l i k l ii ) ((pos "Pu%" ) ))
( "diacritic"  n ( d ai2 @ k r i1 t i k ) ((pos "K6$" "OA$" ) ))
( "diacritical"  j ( d ai2 @ k r i1 t i k @ l ) ((pos "OA$" ) ))
( "diacritics"  n ( d ai2 @ k r i1 t i k s ) ((pos "Kj$" ) ))
( "diadem"  n ( d ai1 @ d e m ) ((pos "K6$" ) ))
( "diadems"  n ( d ai1 @ d e m z ) ((pos "Kj$" ) ))
( "diaereses"  n ( d ai1 e1 r @ s ii z ) ((pos "Kj$" ) ))
( "diaeresis"  n ( d ai1 e1 r @ s i s ) ((pos "Ki$" ) ))
( "diagnose"  v ( d ai1 @ g n ou z ) ((pos "H2%" ) (subcat "6A" )))
( "diagnosed"  v ( d ai1 @ g n ou z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "diagnoses"  v ( d ai1 @ g n ou z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "diagnoses"  n ( d ai2 @ g n ou1 s ii z ) ((pos "Kj%" ) ))
( "diagnosing"  v ( d ai1 @ g n ou z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "diagnosis"  n ( d ai2 @ g n ou1 s i s ) ((pos "Mi%" ) ))
( "diagnostic"  j ( d ai2 @ g n o1 s t i k ) ((pos "OA%" ) ))
( "diagonal"  n ( d ai1 a1 g @ n l ) ((pos "K6%" "OA%" ) ))
( "diagonally"  a ( d ai1 a1 g @ n @ l ii ) ((pos "Pu%" ) ))
( "diagonals"  n ( d ai1 a1 g @ n l z ) ((pos "Kj%" ) ))
( "diagram"  n ( d ai1 @ g r a m ) ((pos "K6%" ) ))
( "diagrammatic"  j ( d ai2 @ g r @ m a1 t i k ) ((pos "OA%" ) ))
( "diagrammatical"  j ( d ai2 @ g r @ m a1 t i k l ) ((pos "OA%" ) ))
( "diagrammatically"  a ( d ai2 @ g r @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "diagrams"  n ( d ai1 @ g r a m z ) ((pos "Kj%" ) ))
( "dial"  v ( d ai1 @ l ) ((pos "H4%" "K6%" ) (subcat )))
( "dialect"  n ( d ai1 @ l e k t ) ((pos "M6%" ) ))
( "dialectal"  j ( d ai2 @ l e1 k t l ) ((pos "OA$" ) ))
( "dialectic"  n ( d ai2 @ l e1 k t i k ) ((pos "K6%" ) ))
( "dialectical"  j ( d ai2 @ l e1 k t i k @ l ) ((pos "OA%" ) ))
( "dialectician"  n ( d ai2 @ l e k t i1 sh @ n ) ((pos "K6$" ) ))
( "dialecticians"  n ( d ai2 @ l e k t i1 sh @ n z ) ((pos "Kj$" ) ))
( "dialectics"  n ( d ai2 @ l e1 k t i k s ) ((pos "Kj$" ) ))
( "dialects"  n ( d ai1 @ l e k t s ) ((pos "Mj%" ) ))
( "dialled"  v ( d ai1 @ l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dialling"  v ( d ai1 @ l i ng ) ((pos "Hb%" ) (subcat )))
( "dialogue"  n ( d ai1 @ l o g ) ((pos "M6%" ) ))
( "dialogues"  n ( d ai1 @ l o g z ) ((pos "Mj%" ) ))
( "dials"  v ( d ai1 @ l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "diameter"  n ( d ai1 a1 m i t @ r ) ((pos "K6%" ) ))
( "diameters"  n ( d ai1 a1 m i t @ z ) ((pos "Kj%" ) ))
( "diametrically"  a ( d ai2 @ m e1 t r i k l ii ) ((pos "Pu%" ) ))
( "diamond"  n ( d ai1 @ m @ n d ) ((pos "K6%" ) ))
( "diamonds"  n ( d ai1 @ m @ n d z ) ((pos "Kj%" ) ))
( "diaper"  n ( d ai1 @ p @ r ) ((pos "K6$" ) ))
( "diapers"  n ( d ai1 @ p @ z ) ((pos "Kj$" ) ))
( "diaphanous"  j ( d ai1 a1 f @ n @ s ) ((pos "OA$" ) ))
( "diaphragm"  n ( d ai1 @ f r a m ) ((pos "K6%" ) ))
( "diaphragms"  n ( d ai1 @ f r a m z ) ((pos "Kj%" ) ))
( "diarchies"  n ( d ai1 aa k i z ) ((pos "Kj$" ) ))
( "diarchy"  n ( d ai1 aa k ii ) ((pos "K8$" ) ))
( "diaries"  n ( d ai1 @ r i z ) ((pos "Kj%" ) ))
( "diarist"  n ( d ai1 @ r i s t ) ((pos "K6%" ) ))
( "diarists"  n ( d ai1 @ r i s t s ) ((pos "Kj%" ) ))
( "diarrhea"  n ( d ai2 @ r i@1 ) ((pos "L@$" ) ))
( "diarrhoea"  n ( d ai2 @ r i@1 ) ((pos "L@%" ) ))
( "diary"  n ( d ai1 @ r ii ) ((pos "K8%" ) ))
( "diatonic"  j ( d ai2 @ t o1 n i k ) ((pos "OA$" ) ))
( "diatribe"  n ( d ai1 @ t r ai b ) ((pos "K6$" ) ))
( "diatribes"  n ( d ai1 @ t r ai b z ) ((pos "Kj$" ) ))
( "dibber"  n ( d i1 b @ r ) ((pos "K6$" ) ))
( "dibbers"  n ( d i1 b @ z ) ((pos "Kj$" ) ))
( "dibble"  v ( d i1 b l ) ((pos "H2$" "K6$" ) (subcat )))
( "dibbled"  v ( d i1 b l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "dibbles"  v ( d i1 b l z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "dibbling"  v ( d i1 b l i ng ) ((pos "Hb$" ) (subcat )))
( "dice"  v ( d ai1 s ) ((pos "J2%" "K9%" ) (subcat "2A" )))
( "dice-box"  n ( d ai1 s - b o k s ) ((pos "K7$" ) ))
( "dice-boxes"  n ( d ai1 s - b o k s i z ) ((pos "Kj$" ) ))
( "diced"  v ( d ai1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dices"  v ( d ai1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "dicey"  j ( d ai1 s ii ) ((pos "OA%" ) ))
( "dichotomies"  n ( d ai1 k o1 t @ m i z ) ((pos "Kj%" ) ))
( "dichotomy"  n ( d ai1 k o1 t @ m ii ) ((pos "K8%" ) ))
( "dicing"  v ( d ai1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dickens"  n ( d i1 k i n z ) ((pos "Ki$" ) ))
( "dicker"  v ( d i1 k @ r ) ((pos "I0$" ) (subcat "2A" )))
( "dickered"  v ( d i1 k @ d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "dickering"  v ( d i1 k @ r i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "dickers"  v ( d i1 k @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "dickey"  n ( d i1 k ii ) ((pos "K6$" ) ))
( "dickeys"  n ( d i1 k i z ) ((pos "Kj$" ) ))
( "dickies"  n ( d i1 k i z ) ((pos "Kj$" ) ))
( "dicky"  n ( d i1 k ii ) ((pos "K8%" "OA%" ) ))
( "dicky-seat"  n ( d i1 k i - s ii t ) ((pos "K6$" ) ))
( "dicky-seats"  n ( d i1 k i - s ii t s ) ((pos "Kj$" ) ))
( "dickybird"  n ( d i1 k i b @@ d ) ((pos "K6%" ) ))
( "dickybirds"  n ( d i1 k i b @@ d z ) ((pos "Kj%" ) ))
( "dicta"  n ( d i1 k t @ ) ((pos "Kj$" ) ))
( "dictate"  n ( d i1 k t ei t ) ((pos "K6%" ) ))
( "dictate"  v ( d i1 k t ei1 t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "dictated"  v ( d i1 k t ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "dictates"  n ( d i1 k t ei t s ) ((pos "Kj%" ) ))
( "dictates"  v ( d i1 k t ei1 t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "dictating"  v ( d i1 k t ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "dictation"  n ( d i1 k t ei1 sh @ n ) ((pos "M6%" ) ))
( "dictations"  n ( d i1 k t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "dictator"  n ( d i1 k t ei1 t @ r ) ((pos "K6%" ) ))
( "dictatorial"  j ( d i2 k t @ t oo1 r i@ l ) ((pos "OA%" ) ))
( "dictatorially"  a ( d i2 k t @ t oo1 r i@ l ii ) ((pos "Pu%" ) ))
( "dictators"  n ( d i1 k t ei1 t @ z ) ((pos "Kj%" ) ))
( "dictatorship"  n ( d i1 k t ei1 t @ sh i p ) ((pos "M6%" ) ))
( "dictatorships"  n ( d i1 k t ei1 t @ sh i p s ) ((pos "Mj%" ) ))
( "diction"  n ( d i1 k sh @ n ) ((pos "L@%" ) ))
( "dictionaries"  n ( d i1 k sh @ n r i z ) ((pos "Kj%" ) ))
( "dictionary"  n ( d i1 k sh @ n r ii ) ((pos "K8%" ) ))
( "dictum"  n ( d i1 k t @ m ) ((pos "K6%" ) ))
( "dictums"  n ( d i1 k t @ m z ) ((pos "Kj$" ) ))
( "did"  v ( d i d ) ((pos "Gc*" "Jc*" ) (subcat "2A" "2B" "2C" "3A" "6A" "6C" "7B" "12B" "13B" "15A" )))
( "didactic"  j ( d i1 d a1 k t i k ) ((pos "OA%" ) ))
( "didactically"  a ( d i1 d a1 k t i k l ii ) ((pos "Pu%" ) ))
( "diddle"  v ( d i1 d l ) ((pos "H2%" ) (subcat "6A" )))
( "diddled"  v ( d i1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "diddles"  v ( d i1 d l z ) ((pos "Ha%" ) (subcat "6A" )))
( "diddling"  v ( d i1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "didn't"  v ( d i1 d n t ) ((pos "Gg*" ) ))
( "die"  v ( d ai1 ) ((pos "I5%" "K6$" ) (subcat "2A" "2C" "2D" "3A" )))
( "die-cast"  j ( d ai1 - k aa s t ) ((pos "OA$" ) ))
( "die-hard"  n ( d ai1 - h aa d ) ((pos "K6%" ) ))
( "die-hards"  n ( d ai1 - h aa d z ) ((pos "Kj%" ) ))
( "died"  v ( d ai1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "2D" "3A" )))
( "diereses"  n ( d ai1 e1 r @ s ii z ) ((pos "Kj$" ) ))
( "dieresis"  n ( d ai1 e1 r @ s i s ) ((pos "Ki$" ) ))
( "dies"  v ( d ai1 z ) ((pos "Ia%" "Kj$" ) (subcat "2A" "2C" "2D" "3A" )))
( "diesel"  n ( d ii1 z l ) ((pos "M6%" ) ))
( "diesels"  n ( d ii1 z l z ) ((pos "Mj%" ) ))
( "diet"  v ( d ai1 @ t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "dietary"  j ( d ai1 @ t @ r ii ) ((pos "OA%" ) ))
( "dieted"  v ( d ai1 @ t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dietetics"  n ( d ai2 @ t e1 t i k s ) ((pos "Lk$" ) ))
( "dietician"  n ( d ai2 @ t i1 sh @ n ) ((pos "K6%" ) ))
( "dieticians"  n ( d ai2 @ t i1 sh @ n z ) ((pos "Kj%" ) ))
( "dieting"  v ( d ai1 @ t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dietitian"  n ( d ai2 @ t i1 sh @ n ) ((pos "K6%" ) ))
( "dietitians"  n ( d ai2 @ t i1 sh @ n z ) ((pos "Kj%" ) ))
( "diets"  v ( d ai1 @ t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "differ"  v ( d i1 f @ r ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "differed"  v ( d i1 f @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "difference"  n ( d i1 f r @ n s ) ((pos "M6%" ) ))
( "differences"  n ( d i1 f r @ n s i z ) ((pos "Mj%" ) ))
( "different"  j ( d i1 f r @ n t ) ((pos "OA*" ) ))
( "differential"  n ( d i2 f @ r e1 n sh l ) ((pos "K6%" "OA%" ) ))
( "differentially"  a ( d i2 f @ r e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "differentials"  n ( d i2 f @ r e1 n sh l z ) ((pos "Kj%" ) ))
( "differentiate"  v ( d i2 f @ r e1 n sh i ei t ) ((pos "H2%" ) (subcat "3A" "6A" )))
( "differentiated"  v ( d i2 f @ r e1 n sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" )))
( "differentiates"  v ( d i2 f @ r e1 n sh i ei t s ) ((pos "Ha%" ) (subcat "3A" "6A" )))
( "differentiating"  v ( d i2 f @ r e1 n sh i ei t i ng ) ((pos "Hb%" ) (subcat "3A" "6A" )))
( "differentiation"  n ( d i2 f @ r e n sh i ei1 sh @ n ) ((pos "M6%" ) ))
( "differentiations"  n ( d i2 f @ r e n sh i ei1 sh @ n z ) ((pos "Mj$" ) ))
( "differently"  a ( d i1 f r @ n t l ii ) ((pos "Pu%" ) ))
( "differing"  v ( d i1 f @ r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "differs"  v ( d i1 f @ z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "difficult"  j ( d i1 f i k @ l t ) ((pos "OA%" ) ))
( "difficulties"  n ( d i1 f i k @ l t i z ) ((pos "Mj%" ) ))
( "difficulty"  n ( d i1 f i k @ l t ii ) ((pos "M8%" ) ))
( "diffidence"  n ( d i1 f i d @ n s ) ((pos "L@%" ) ))
( "diffident"  j ( d i1 f i d @ n t ) ((pos "OA%" ) ))
( "diffidently"  a ( d i1 f i d @ n t l ii ) ((pos "Pu%" ) ))
( "diffract"  v ( d i1 f r a1 k t ) ((pos "H0$" ) (subcat )))
( "diffracted"  v ( d i1 f r a1 k t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "diffracting"  v ( d i1 f r a1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "diffraction"  n ( d i1 f r a1 k sh @ n ) ((pos "K6%" ) ))
( "diffractions"  n ( d i1 f r a1 k sh @ n z ) ((pos "Kj$" ) ))
( "diffracts"  v ( d i1 f r a1 k t s ) ((pos "Ha$" ) (subcat )))
( "diffuse"  j ( d i1 f y uu1 s ) ((pos "OA%" ) ))
( "diffuse"  v ( d i1 f y uu1 z ) ((pos "J2%" ) (subcat "2A" )))
( "diffused"  v ( d i1 f y uu1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "diffusely"  a ( d i1 f y uu1 s l ii ) ((pos "Pu%" ) ))
( "diffuseness"  n ( d i1 f y uu1 s n @ s ) ((pos "L@%" ) ))
( "diffuses"  v ( d i1 f y uu1 z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "diffusing"  v ( d i1 f y uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "diffusion"  n ( d i1 f y uu1 zh n ) ((pos "L@%" ) ))
( "dig"  v ( d i1 g ) ((pos "J5%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "digest"  n ( d ai1 jh e s t ) ((pos "K6%" ) ))
( "digest"  v ( d i1 jh e1 s t ) ((pos "J0%" ) (subcat "2A" )))
( "digested"  v ( d i1 jh e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "digestibility"  n ( d i1 jh e2 s t @ b i1 l i t ii ) ((pos "L@$" ) ))
( "digestible"  j ( d i1 jh e1 s t @ b l ) ((pos "OA%" ) ))
( "digesting"  v ( d i1 jh e1 s t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "digestion"  n ( d i1 jh e1 s ch @ n ) ((pos "M6%" ) ))
( "digestions"  n ( d i1 jh e1 s ch @ n z ) ((pos "Mj$" ) ))
( "digestive"  j ( d i1 jh e1 s t i v ) ((pos "OA%" ) ))
( "digests"  n ( d ai1 jh e s t s ) ((pos "Kj%" ) ))
( "digests"  v ( d i1 jh e1 s t s ) ((pos "Ja%" ) (subcat "2A" )))
( "digger"  n ( d i1 g @ r ) ((pos "K6%" ) ))
( "diggers"  n ( d i1 g @ z ) ((pos "Kj%" ) ))
( "digging"  v ( d i1 g i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "3A" "6A" )))
( "diggings"  n ( d i1 g i ng z ) ((pos "Mj%" ) ))
( "digit"  n ( d i1 jh i t ) ((pos "K6%" ) ))
( "digital"  j ( d i1 jh i t l ) ((pos "OA%" ) ))
( "digitally"  a ( d i1 jh i t @ l ii ) ((pos "Pu%" ) ))
( "digits"  n ( d i1 jh i t s ) ((pos "Kj%" ) ))
( "dignified"  v ( d i1 g n i f ai d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "dignifies"  v ( d i1 g n i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "dignify"  v ( d i1 g n i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "dignifying"  v ( d i1 g n i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dignitaries"  n ( d i1 g n i t @ r i z ) ((pos "Kj%" ) ))
( "dignitary"  n ( d i1 g n i t @ r ii ) ((pos "K8%" ) ))
( "dignities"  n ( d i1 g n i t i z ) ((pos "Mj$" ) ))
( "dignity"  n ( d i1 g n i t ii ) ((pos "M8%" ) ))
( "digraph"  n ( d ai1 g r aa f ) ((pos "K6$" ) ))
( "digraphs"  n ( d ai1 g r aa f s ) ((pos "Kj$" ) ))
( "digress"  v ( d ai1 g r e1 s ) ((pos "I1%" ) (subcat "2A" )))
( "digressed"  v ( d ai1 g r e1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "digresses"  v ( d ai1 g r e1 s i z ) ((pos "Ia%" ) (subcat "2A" )))
( "digressing"  v ( d ai1 g r e1 s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "digression"  n ( d ai1 g r e1 sh @ n ) ((pos "M6%" ) ))
( "digressions"  n ( d ai1 g r e1 sh @ n z ) ((pos "Mj%" ) ))
( "digs"  v ( d i1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "dike"  v ( d ai1 k ) ((pos "J2$" "K6%" ) ))
( "diked"  v ( d ai1 k t ) ((pos "Jc$" "Jd$" ) ))
( "dikes"  v ( d ai1 k s ) ((pos "Ja$" "Kj%" ) ))
( "diking"  v ( d ai1 k i ng ) ((pos "Jb$" ) ))
( "dilapidated"  j ( d i1 l a1 p i d ei t i d ) ((pos "OA%" ) ))
( "dilapidation"  n ( d i1 l a2 p i d ei1 sh @ n ) ((pos "L@%" ) ))
( "dilate"  v ( d ai1 l ei1 t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "dilated"  v ( d ai1 l ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "dilates"  v ( d ai1 l ei1 t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "dilating"  v ( d ai1 l ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "dilation"  n ( d ai1 l ei1 sh @ n ) ((pos "L@%" ) ))
( "dilatory"  j ( d i1 l @ t @ r ii ) ((pos "OA$" ) ))
( "dilemma"  n ( d i1 l e1 m @ ) ((pos "K6%" ) ))
( "dilemmas"  n ( d i1 l e1 m @ z ) ((pos "Kj%" ) ))
( "dilettante"  n ( d i2 l i t a1 n t ii ) ((pos "K6%" ) ))
( "dilettantes"  n ( d i2 l i t a1 n t i z ) ((pos "Kj$" ) ))
( "dilettanti"  n ( d i2 l i t a1 n t ii ) ((pos "Kj$" ) ))
( "diligence"  n ( d i1 l i jh @ n s ) ((pos "L@%" ) ))
( "diligent"  j ( d i1 l i jh @ n t ) ((pos "OA%" ) ))
( "diligently"  a ( d i1 l i jh @ n t l ii ) ((pos "Pu%" ) ))
( "dill"  n ( d i1 l ) ((pos "L@%" ) ))
( "dilly-dallied"  v ( d i1 l i - d a l i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "dilly-dallies"  v ( d i1 l i - d a l i z ) ((pos "Ia%" ) (subcat )))
( "dilly-dally"  v ( d i1 l i - d a l ii ) ((pos "I3%" ) (subcat )))
( "dilly-dallying"  v ( d i1 l i - d a l i i ng ) ((pos "Ib%" ) (subcat )))
( "dilute"  v ( d ai1 l y uu1 t ) ((pos "H2%" "OA%" ) (subcat "6A" )))
( "diluted"  v ( d ai1 l y uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dilutes"  v ( d ai1 l y uu1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "diluting"  v ( d ai1 l y uu1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dilution"  n ( d ai1 l y uu1 sh @ n ) ((pos "M6%" ) ))
( "dilutions"  n ( d ai1 l y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "dim"  v ( d i1 m ) ((pos "J4%" "OE%" ) (subcat "2A" )))
( "dime"  n ( d ai1 m ) ((pos "K6%" ) ))
( "dimension"  n ( d i1 m e1 n sh @ n ) ((pos "M6%" ) ))
( "dimensional"  j ( d i1 m e1 n sh @ n l ) ((pos "OA%" ) ))
( "dimensions"  n ( d i1 m e1 n sh @ n z ) ((pos "Mj%" ) ))
( "dimes"  n ( d ai1 m z ) ((pos "Kj%" ) ))
( "diminish"  v ( d i1 m i1 n i sh ) ((pos "J1%" ) (subcat "2A" )))
( "diminished"  v ( d i1 m i1 n i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "diminishes"  v ( d i1 m i1 n i sh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "diminishing"  v ( d i1 m i1 n i sh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "diminuendo"  n ( d i1 m i2 n y u e1 n d ou ) ((pos "K6%" ) ))
( "diminuendos"  n ( d i1 m i2 n y u e1 n d ou z ) ((pos "Kj$" ) ))
( "diminution"  n ( d i2 m i n y uu1 sh @ n ) ((pos "M6%" ) ))
( "diminutions"  n ( d i2 m i n y uu1 sh @ n z ) ((pos "Mj$" ) ))
( "diminutive"  n ( d i1 m i1 n y u t i v ) ((pos "K6$" "OA$" ) ))
( "diminutives"  n ( d i1 m i1 n y u t i v z ) ((pos "Kj$" ) ))
( "dimity"  n ( d i1 m i t ii ) ((pos "L@$" ) ))
( "dimly"  a ( d i1 m l ii ) ((pos "Pu%" ) ))
( "dimmed"  v ( d i1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dimmer"  j ( d i1 m @ r ) ((pos "Or%" ) ))
( "dimmest"  j ( d i1 m i s t ) ((pos "Os%" ) ))
( "dimming"  v ( d i1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dimness"  n ( d i1 m n @ s ) ((pos "L@%" ) ))
( "dimple"  v ( d i1 m p l ) ((pos "J2$" "K6%" ) (subcat "2A" )))
( "dimpled"  v ( d i1 m p l d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "dimples"  v ( d i1 m p l z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "dimpling"  v ( d i1 m p l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "dims"  v ( d i1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "din"  v ( d i1 n ) ((pos "J4$" "L@%" ) (subcat )))
( "dinar"  n ( d ii1 n aa r ) ((pos "K6$" ) ))
( "dinars"  n ( d ii1 n aa z ) ((pos "Kj$" ) ))
( "dine"  v ( d ai1 n ) ((pos "J2%" ) (subcat "2A" )))
( "dined"  v ( d ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "diner"  n ( d ai1 n @ r ) ((pos "K6%" ) ))
( "diners"  n ( d ai1 n @ z ) ((pos "Kj%" ) ))
( "dines"  v ( d ai1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "ding-dong"  n ( d i2 ng - d o1 ng ) ((pos "K6%" "Pu%" ) ))
( "ding-dongs"  n ( d i2 ng - d o1 ng z ) ((pos "Kj%" ) ))
( "dinghies"  n ( d i1 ng g i z ) ((pos "Kj%" ) ))
( "dinghy"  n ( d i1 ng g ii ) ((pos "K8%" ) ))
( "dingier"  j ( d i1 n jh i@ r ) ((pos "Or%" ) ))
( "dingiest"  j ( d i1 n jh i i s t ) ((pos "Os%" ) ))
( "dingily"  a ( d i1 n jh i l ii ) ((pos "Pu%" ) ))
( "dinginess"  n ( d i1 n jh i n @ s ) ((pos "L@%" ) ))
( "dingle"  n ( d i1 ng g l ) ((pos "K6$" ) ))
( "dingles"  n ( d i1 ng g l z ) ((pos "Kj$" ) ))
( "dingy"  j ( d i1 n jh ii ) ((pos "OD%" ) ))
( "dining"  v ( d ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dining-car"  n ( d ai1 n i ng - k aa r ) ((pos "K6%" ) ))
( "dining-cars"  n ( d ai1 n i ng - k aa z ) ((pos "Kj%" ) ))
( "dining-room"  n ( d ai1 n i ng - r uu m ) ((pos "K6%" ) ))
( "dining-rooms"  n ( d ai1 n i ng - r uu m z ) ((pos "Kj%" ) ))
( "dining-table"  n ( d ai1 n i ng - t ei b l ) ((pos "K6%" ) ))
( "dining-tables"  n ( d ai1 n i ng - t ei b l z ) ((pos "Kj%" ) ))
( "dinkier"  j ( d i1 ng k i@ r ) ((pos "Or$" ) ))
( "dinkiest"  j ( d i1 ng k i i s t ) ((pos "Os$" ) ))
( "dinky"  j ( d i1 ng k ii ) ((pos "OD%" ) ))
( "dinned"  v ( d i1 n d ) ((pos "Jc$" "Jd%" ) (subcat )))
( "dinner"  n ( d i1 n @ r ) ((pos "K6%" ) ))
( "dinner-jacket"  n ( d i1 n @ - jh a k i t ) ((pos "K6%" ) ))
( "dinner-jackets"  n ( d i1 n @ - jh a k i t s ) ((pos "Kj%" ) ))
( "dinner-parties"  n ( d i1 n @ - p aa t i z ) ((pos "Kj%" ) ))
( "dinner-party"  n ( d i1 n @ - p aa t ii ) ((pos "K8%" ) ))
( "dinner-service"  n ( d i1 n @ - s @@ v i s ) ((pos "K6%" ) ))
( "dinner-services"  n ( d i1 n @ - s @@ v i s i z ) ((pos "Kj%" ) ))
( "dinner-set"  n ( d i1 n @ - s e t ) ((pos "K6%" ) ))
( "dinner-sets"  n ( d i1 n @ - s e t s ) ((pos "Kj%" ) ))
( "dinners"  n ( d i1 n @ z ) ((pos "Kj%" ) ))
( "dinning"  v ( d i1 n i ng ) ((pos "Jb$" ) (subcat )))
( "dinosaur"  n ( d ai1 n @ s oo r ) ((pos "K6%" ) ))
( "dinosaurs"  n ( d ai1 n @ s oo z ) ((pos "Kj%" ) ))
( "dins"  v ( d i1 n z ) ((pos "Ja$" ) (subcat )))
( "dint"  n ( d i1 n t ) ((pos "K6%" ) ))
( "dints"  n ( d i1 n t s ) ((pos "Kj$" ) ))
( "diocesan"  n ( d ai1 o1 s i s n ) ((pos "K6$" "OA%" ) ))
( "diocesans"  n ( d ai1 o1 s i s n z ) ((pos "Kj$" ) ))
( "diocese"  n ( d ai1 @ s i s ) ((pos "K6%" ) ))
( "dioceses"  n ( d ai1 @ s i s i z ) ((pos "Kj$" ) ))
( "dioxide"  n ( d ai1 o1 k s ai d ) ((pos "K6%" ) ))
( "dioxides"  n ( d ai1 o1 k s ai d z ) ((pos "Kj%" ) ))
( "dip"  v ( d i1 p ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "diphtheria"  n ( d i1 f th i@1 r i@ ) ((pos "L@%" ) ))
( "diphthong"  n ( d i1 f th o ng ) ((pos "K6$" ) ))
( "diphthongs"  n ( d i1 f th o ng z ) ((pos "Kj$" ) ))
( "diploma"  n ( d i1 p l ou1 m @ ) ((pos "K6%" ) ))
( "diplomacy"  n ( d i1 p l ou1 m @ s ii ) ((pos "L@%" ) ))
( "diplomas"  n ( d i1 p l ou1 m @ z ) ((pos "Kj%" ) ))
( "diplomat"  n ( d i1 p l @ m a t ) ((pos "K6%" ) ))
( "diplomatic"  j ( d i2 p l @ m a1 t i k ) ((pos "OA%" ) ))
( "diplomatically"  a ( d i2 p l @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "diplomatist"  n ( d i1 p l ou1 m @ t i s t ) ((pos "K6$" ) ))
( "diplomatists"  n ( d i1 p l ou1 m @ t i s t s ) ((pos "Kj$" ) ))
( "diplomats"  n ( d i1 p l @ m a t s ) ((pos "Kj%" ) ))
( "dipped"  v ( d i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dipper"  n ( d i1 p @ r ) ((pos "K6%" ) ))
( "dippers"  n ( d i1 p @ z ) ((pos "Kj%" ) ))
( "dipping"  v ( d i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dips"  v ( d i1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dipsomania"  n ( d i2 p s @ m ei1 n i@ ) ((pos "L@%" ) ))
( "dipsomaniac"  n ( d i2 p s @ m ei1 n i a k ) ((pos "K6%" ) ))
( "dipsomaniacs"  n ( d i2 p s @ m ei1 n i a k s ) ((pos "Kj%" ) ))
( "dipstick"  n ( d i1 p s t i k ) ((pos "K6%" ) ))
( "dipsticks"  n ( d i1 p s t i k s ) ((pos "Kj%" ) ))
( "diptych"  n ( d i1 p t i k ) ((pos "K6$" ) ))
( "diptychs"  n ( d i1 p t i k s ) ((pos "Kj$" ) ))
( "dire"  j ( d ai1 @ r ) ((pos "OB%" ) ))
( "direct"  v ( d i1 r e1 k t ) ((pos "J0%" "OA%" "Pu%" ) (subcat "2A" "6A" "9" "14" )))
( "directed"  v ( d i1 r e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" "14" )))
( "directing"  v ( d i1 r e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "9" "14" )))
( "direction"  n ( d i1 r e1 k sh @ n ) ((pos "M6%" ) ))
( "direction-finder"  n ( d i1 r e1 k sh @ n - f ai2 n d @ r ) ((pos "K6%" ) ))
( "direction-finders"  n ( d i1 r e1 k sh @ n - f ai2 n d @ z ) ((pos "Kj%" ) ))
( "directional"  j ( d i1 r e1 k sh @ n l ) ((pos "OA%" ) ))
( "directions"  n ( d i1 r e1 k sh @ n z ) ((pos "Mj%" ) ))
( "directive"  n ( d i1 r e1 k t i v ) ((pos "K6%" ) ))
( "directives"  n ( d i1 r e1 k t i v z ) ((pos "Kj%" ) ))
( "directly"  a ( d i1 r e1 k t l ii ) ((pos "Pu%" "V-%" ) ))
( "directness"  n ( d i1 r e1 k t n @ s ) ((pos "L@%" ) ))
( "director"  n ( d i1 r e1 k t @ r ) ((pos "K6%" ) ))
( "directorate"  n ( d i1 r e1 k t @ r @ t ) ((pos "K6%" ) ))
( "directorates"  n ( d i1 r e1 k t @ r @ t s ) ((pos "Kj%" ) ))
( "directories"  n ( d i1 r e1 k t @ r i z ) ((pos "Kj%" ) ))
( "directors"  n ( d i1 r e1 k t @ z ) ((pos "Kj%" ) ))
( "directorship"  n ( d i1 r e1 k t @ sh i p ) ((pos "K6%" ) ))
( "directorships"  n ( d i1 r e1 k t @ sh i p s ) ((pos "Kj%" ) ))
( "directory"  n ( d i1 r e1 k t @ r ii ) ((pos "K8%" ) ))
( "directs"  v ( d i1 r e1 k t s ) ((pos "Ja%" ) (subcat "2A" "6A" "9" "14" )))
( "direful"  j ( d ai1 @ f @ l ) ((pos "OA$" ) ))
( "direfully"  a ( d ai1 @ f @ l ii ) ((pos "Pu$" ) ))
( "direr"  j ( d ai1 @ r @ r ) ((pos "Or$" ) ))
( "direst"  j ( d ai1 @ r i s t ) ((pos "Os%" ) ))
( "dirge"  n ( d @@1 jh ) ((pos "K6%" ) ))
( "dirges"  n ( d @@1 jh i z ) ((pos "Kj%" ) ))
( "dirigible"  n ( d i1 r i jh @ b l ) ((pos "K6$" ) ))
( "dirigibles"  n ( d i1 r i jh @ b l z ) ((pos "Kj$" ) ))
( "dirk"  n ( d @@1 k ) ((pos "K6$" ) ))
( "dirks"  n ( d @@1 k s ) ((pos "Kj$" ) ))
( "dirndl"  n ( d @@1 n d l ) ((pos "K6$" ) ))
( "dirndls"  n ( d @@1 n d l z ) ((pos "Kj$" ) ))
( "dirt"  n ( d @@1 t ) ((pos "L@%" ) ))
( "dirt-cheap"  j ( d @@1 t - ch ii p ) ((pos "OA%" ) ))
( "dirt-track"  n ( d @@1 t - t r a k ) ((pos "K6%" ) ))
( "dirt-tracks"  n ( d @@1 t - t r a k s ) ((pos "Kj%" ) ))
( "dirtied"  v ( d @@1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dirtier"  j ( d @@1 t i@ r ) ((pos "Or%" ) ))
( "dirties"  v ( d @@1 t i z ) ((pos "Ja%" ) (subcat "2A" )))
( "dirtiest"  j ( d @@1 t i i s t ) ((pos "Os%" ) ))
( "dirtily"  a ( d @@1 t i l ii ) ((pos "Pu%" ) ))
( "dirty"  v ( d @@1 t ii ) ((pos "J3%" "OD%" ) (subcat "2A" )))
( "dirtying"  v ( d @@1 t i i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disabilities"  n ( d i2 s @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "disability"  n ( d i2 s @ b i1 l i t ii ) ((pos "M8%" ) ))
( "disable"  v ( d i1 s ei1 b l ) ((pos "H2%" ) (subcat )))
( "disabled"  v ( d i1 s ei1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disablement"  n ( d i1 s ei1 b l m @ n t ) ((pos "M6%" ) ))
( "disablements"  n ( d i1 s ei1 b l m @ n t s ) ((pos "Mj$" ) ))
( "disables"  v ( d i1 s ei1 b l z ) ((pos "Ha%" ) (subcat )))
( "disabling"  v ( d i1 s ei1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "disabuse"  v ( d i2 s @ b y uu1 z ) ((pos "H2%" ) (subcat "6A" )))
( "disabused"  v ( d i2 s @ b y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "disabuses"  v ( d i2 s @ b y uu1 z i z ) ((pos "Ha$" ) (subcat "6A" )))
( "disabusing"  v ( d i2 s @ b y uu1 z i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "disadvantage"  n ( d i2 s @ d v aa1 n t i jh ) ((pos "M6%" ) ))
( "disadvantageous"  j ( d i2 s a2 d v @ n t ei1 jh @ s ) ((pos "OA%" ) ))
( "disadvantageously"  a ( d i2 s a2 d v @ n t ei1 jh @ s l ii ) ((pos "Pu%" ) ))
( "disadvantages"  n ( d i2 s @ d v aa1 n t i jh i z ) ((pos "Mj%" ) ))
( "disaffected"  j ( d i2 s @ f e1 k t i d ) ((pos "OA%" ) ))
( "disaffection"  n ( d i2 s @ f e1 k sh @ n ) ((pos "L@%" ) ))
( "disafforest"  v ( d i2 s @ f o1 r i s t ) ((pos "H0$" ) ))
( "disafforested"  v ( d i2 s @ f o1 r i s t i d ) ((pos "Hc$" "Hd$" ) ))
( "disafforesting"  v ( d i2 s @ f o1 r i s t i ng ) ((pos "Hb$" ) ))
( "disafforests"  v ( d i2 s @ f o1 r i s t s ) ((pos "Ha$" ) ))
( "disagree"  v ( d i2 s @ g r ii1 ) ((pos "H5%" ) (subcat "2A" )))
( "disagreeable"  j ( d i2 s @ g r ii1 @ b l ) ((pos "OA%" ) ))
( "disagreeableness"  n ( d i2 s @ g r ii1 @ b l n @ s ) ((pos "L@%" ) ))
( "disagreeably"  a ( d i2 s @ g r ii1 @ b l ii ) ((pos "Pu%" ) ))
( "disagreed"  v ( d i2 s @ g r ii1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "disagreeing"  v ( d i2 s @ g r ii1 i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "disagreement"  n ( d i2 s @ g r ii1 m @ n t ) ((pos "M6%" ) ))
( "disagreements"  n ( d i2 s @ g r ii1 m @ n t s ) ((pos "Mj%" ) ))
( "disagrees"  v ( d i2 s @ g r ii1 z ) ((pos "Ha%" ) (subcat "2A" )))
( "disallow"  v ( d i2 s @ l au1 ) ((pos "H0%" ) (subcat )))
( "disallowed"  v ( d i2 s @ l au1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disallowing"  v ( d i2 s @ l au1 i ng ) ((pos "Hb%" ) (subcat )))
( "disallows"  v ( d i2 s @ l au1 z ) ((pos "Ha%" ) (subcat )))
( "disappear"  v ( d i2 s @ p i@1 r ) ((pos "I0%" ) (subcat )))
( "disappearance"  n ( d i2 s @ p i@1 r @ n s ) ((pos "K6%" ) ))
( "disappearances"  n ( d i2 s @ p i@1 r @ n s i z ) ((pos "Kj%" ) ))
( "disappeared"  v ( d i2 s @ p i@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "disappearing"  v ( d i2 s @ p i@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "disappears"  v ( d i2 s @ p i@1 z ) ((pos "Ia%" ) (subcat )))
( "disappoint"  v ( d i2 s @ p oi1 n t ) ((pos "H0%" ) (subcat )))
( "disappointed"  v ( d i2 s @ p oi1 n t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "disappointedly"  a ( d i2 s @ p oi1 n t i d l ii ) ((pos "Pu%" ) ))
( "disappointing"  v ( d i2 s @ p oi1 n t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "disappointingly"  a ( d i2 s @ p oi1 n t i ng l ii ) ((pos "Pu%" ) ))
( "disappointment"  n ( d i2 s @ p oi1 n t m @ n t ) ((pos "M6%" ) ))
( "disappointments"  n ( d i2 s @ p oi1 n t m @ n t s ) ((pos "Mj%" ) ))
( "disappoints"  v ( d i2 s @ p oi1 n t s ) ((pos "Ha%" ) (subcat )))
( "disapprobation"  n ( d i2 s a2 p r @ b ei1 sh @ n ) ((pos "L@$" ) ))
( "disapproval"  n ( d i2 s @ p r uu1 v @ l ) ((pos "L@%" ) ))
( "disapprove"  v ( d i2 s @ p r uu1 v ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "disapproved"  v ( d i2 s @ p r uu1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "disapproves"  v ( d i2 s @ p r uu1 v z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "disapproving"  v ( d i2 s @ p r uu1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "disapprovingly"  a ( d i2 s @ p r uu1 v i ng l ii ) ((pos "Pu%" ) ))
( "disarm"  v ( d i1 s aa1 m ) ((pos "J0%" ) (subcat "2A" )))
( "disarmament"  n ( d i1 s aa1 m @ m @ n t ) ((pos "L@%" ) ))
( "disarmed"  v ( d i1 s aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disarming"  v ( d i1 s aa1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disarms"  v ( d i1 s aa1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "disarrange"  v ( d i2 s @ r ei1 n jh ) ((pos "H2%" ) (subcat )))
( "disarranged"  v ( d i2 s @ r ei1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disarrangement"  n ( d i2 s @ r ei1 n jh m @ n t ) ((pos "K6%" ) ))
( "disarrangements"  n ( d i2 s @ r ei1 n jh m @ n t s ) ((pos "Kj$" ) ))
( "disarranges"  v ( d i2 s @ r ei1 n jh i z ) ((pos "Ha%" ) (subcat )))
( "disarranging"  v ( d i2 s @ r ei1 n jh i ng ) ((pos "Hb%" ) (subcat )))
( "disarray"  v ( d i2 s @ r ei1 ) ((pos "H0$" "L@%" ) ))
( "disarrayed"  v ( d i2 s @ r ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "disarraying"  v ( d i2 s @ r ei1 i ng ) ((pos "Hb$" ) ))
( "disarrays"  v ( d i2 s @ r ei1 z ) ((pos "Ha$" ) ))
( "disassociate"  v ( d i2 s @ s ou1 sh i ei t ) ((pos "H2%" ) (subcat )))
( "disassociated"  v ( d i2 s @ s ou1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disassociates"  v ( d i2 s @ s ou1 sh i ei t s ) ((pos "Ha%" ) (subcat )))
( "disassociating"  v ( d i2 s @ s ou1 sh i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "disaster"  n ( d i1 z aa1 s t @ r ) ((pos "M6%" ) ))
( "disasters"  n ( d i1 z aa1 s t @ z ) ((pos "Mj%" ) ))
( "disastrous"  j ( d i1 z aa1 s t r @ s ) ((pos "OA%" ) ))
( "disastrously"  a ( d i1 z aa1 s t r @ s l ii ) ((pos "Pu%" ) ))
( "disavow"  v ( d i2 s @ v au1 ) ((pos "H0%" ) (subcat )))
( "disavowal"  n ( d i2 s @ v au1 @ l ) ((pos "K6%" ) ))
( "disavowals"  n ( d i2 s @ v au1 @ l z ) ((pos "Kj%" ) ))
( "disavowed"  v ( d i2 s @ v au1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disavowing"  v ( d i2 s @ v au1 i ng ) ((pos "Hb$" ) (subcat )))
( "disavows"  v ( d i2 s @ v au1 z ) ((pos "Ha$" ) (subcat )))
( "disband"  v ( d i1 s b a1 n d ) ((pos "J0%" ) (subcat "2A" )))
( "disbanded"  v ( d i1 s b a1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disbanding"  v ( d i1 s b a1 n d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disbandment"  n ( d i1 s b a1 n d m @ n t ) ((pos "K6%" ) ))
( "disbandments"  n ( d i1 s b a1 n d m @ n t s ) ((pos "Kj%" ) ))
( "disbands"  v ( d i1 s b a1 n d z ) ((pos "Ja%" ) (subcat "2A" )))
( "disbelief"  n ( d i2 s b i l ii1 f ) ((pos "L@%" ) ))
( "disbelieve"  v ( d i2 s b i l ii1 v ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "disbelieved"  v ( d i2 s b i l ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "disbelieves"  v ( d i2 s b i l ii1 v z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "disbelieving"  v ( d i2 s b i l ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "disbelievingly"  a ( d i2 s b i l ii1 v i ng l ii ) ((pos "Pu%" ) ))
( "disbud"  v ( d i1 s b uh1 d ) ((pos "H4$" ) (subcat )))
( "disbudded"  v ( d i1 s b uh1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "disbudding"  v ( d i1 s b uh1 d i ng ) ((pos "Hb$" ) (subcat )))
( "disbuds"  v ( d i1 s b uh1 d z ) ((pos "Ha$" ) (subcat )))
( "disburden"  v ( d i1 s b @@1 d n ) ((pos "H0$" ) (subcat "6A" )))
( "disburdened"  v ( d i1 s b @@1 d n d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "disburdening"  v ( d i1 s b @@1 d n i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "disburdens"  v ( d i1 s b @@1 d n z ) ((pos "Ha$" ) (subcat "6A" )))
( "disburse"  v ( d i1 s b @@1 s ) ((pos "J2%" ) (subcat "2A" )))
( "disbursed"  v ( d i1 s b @@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disbursement"  n ( d i1 s b @@1 s m @ n t ) ((pos "M6%" ) ))
( "disbursements"  n ( d i1 s b @@1 s m @ n t s ) ((pos "Mj%" ) ))
( "disburses"  v ( d i1 s b @@1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "disbursing"  v ( d i1 s b @@1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disc"  n ( d i1 s k ) ((pos "K6%" ) ))
( "discard"  n ( d i1 s k aa d ) ((pos "K6%" ) ))
( "discard"  v ( d i1 s k aa1 d ) ((pos "H0%" ) (subcat )))
( "discarded"  v ( d i1 s k aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "discarding"  v ( d i1 s k aa1 d i ng ) ((pos "Hb%" ) (subcat )))
( "discards"  n ( d i1 s k aa d z ) ((pos "Kj$" ) ))
( "discards"  v ( d i1 s k aa1 d z ) ((pos "Ha%" ) (subcat )))
( "discern"  v ( d i1 s @@1 n ) ((pos "H0%" ) (subcat )))
( "discerned"  v ( d i1 s @@1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "discernible"  j ( d i1 s @@1 n @ b l ) ((pos "OA%" ) ))
( "discerning"  v ( d i1 s @@1 n i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "discernment"  n ( d i1 s @@1 n m @ n t ) ((pos "L@%" ) ))
( "discerns"  v ( d i1 s @@1 n z ) ((pos "Ha%" ) (subcat )))
( "discharge"  n ( d i1 s ch aa jh ) ((pos "M6%" ) ))
( "discharge"  v ( d i1 s ch aa1 jh ) ((pos "J2%" ) (subcat "6A" )))
( "discharged"  v ( d i1 s ch aa1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "discharges"  n ( d i1 s ch aa jh i z ) ((pos "Mj%" ) ))
( "discharges"  v ( d i1 s ch aa1 jh i z ) ((pos "Ja%" ) (subcat "6A" )))
( "discharging"  v ( d i1 s ch aa1 jh i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "disciple"  n ( d i1 s ai1 p l ) ((pos "K6%" ) ))
( "disciples"  n ( d i1 s ai1 p l z ) ((pos "Kj%" ) ))
( "disciplinarian"  n ( d i2 s @ p l i n e@1 r i@ n ) ((pos "K6%" ) ))
( "disciplinarians"  n ( d i2 s @ p l i n e@1 r i@ n z ) ((pos "Kj%" ) ))
( "disciplinary"  j ( d i1 s i p l i n @ r ii ) ((pos "OA%" ) ))
( "discipline"  v ( d i1 s i p l i n ) ((pos "H2%" "M6%" ) (subcat )))
( "disciplined"  v ( d i1 s i p l i n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disciplines"  v ( d i1 s i p l i n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "disciplining"  v ( d i1 s i p l i n i ng ) ((pos "Hb%" ) (subcat )))
( "disclaim"  v ( d i1 s k l ei1 m ) ((pos "H0%" ) (subcat "6A" )))
( "disclaimed"  v ( d i1 s k l ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "disclaimer"  n ( d i1 s k l ei1 m @ r ) ((pos "K6%" ) ))
( "disclaimers"  n ( d i1 s k l ei1 m @ z ) ((pos "Kj%" ) ))
( "disclaiming"  v ( d i1 s k l ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disclaims"  v ( d i1 s k l ei1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "disclose"  v ( d i1 s k l ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "disclosed"  v ( d i1 s k l ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "discloses"  v ( d i1 s k l ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "disclosing"  v ( d i1 s k l ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disclosure"  n ( d i1 s k l ou1 zh @ r ) ((pos "M6%" ) ))
( "disclosures"  n ( d i1 s k l ou1 zh @ z ) ((pos "Mj%" ) ))
( "disco"  n ( d i1 s k ou ) ((pos "K6%" ) ))
( "discolour"  v ( d i1 s k uh1 l @ r ) ((pos "J0%" ) (subcat "2A" )))
( "discolouration"  n ( d i2 s k @ l @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "discolourations"  n ( d i2 s k @ l @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "discoloured"  v ( d i1 s k uh1 l @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "discolouring"  v ( d i1 s k uh1 l @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "discolours"  v ( d i1 s k uh1 l @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "discomfit"  v ( d i1 s k uh1 m f i t ) ((pos "H0$" ) (subcat )))
( "discomfited"  v ( d i1 s k uh1 m f i t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "discomfiting"  v ( d i1 s k uh1 m f i t i ng ) ((pos "Hb$" ) (subcat )))
( "discomfits"  v ( d i1 s k uh1 m f i t s ) ((pos "Ha$" ) (subcat )))
( "discomfiture"  n ( d i1 s k uh1 m f i ch @ r ) ((pos "L@%" ) ))
( "discomfort"  n ( d i1 s k uh1 m f @ t ) ((pos "M6%" ) ))
( "discomforts"  n ( d i1 s k uh1 m f @ t s ) ((pos "Mj%" ) ))
( "discommode"  v ( d i2 s k @ m ou1 d ) ((pos "H2$" ) (subcat )))
( "discommoded"  v ( d i2 s k @ m ou1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "discommodes"  v ( d i2 s k @ m ou1 d z ) ((pos "Ha$" ) (subcat )))
( "discommoding"  v ( d i2 s k @ m ou1 d i ng ) ((pos "Hb$" ) (subcat )))
( "discompose"  v ( d i2 s k @ m p ou1 z ) ((pos "H2$" ) (subcat )))
( "discomposed"  v ( d i2 s k @ m p ou1 z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "discomposes"  v ( d i2 s k @ m p ou1 z i z ) ((pos "Ha$" ) (subcat )))
( "discomposing"  v ( d i2 s k @ m p ou1 z i ng ) ((pos "Hb$" ) (subcat )))
( "discomposure"  n ( d i2 s k @ m p ou1 zh @ r ) ((pos "L@$" ) ))
( "disconcert"  v ( d i2 s k @ n s @@1 t ) ((pos "H0%" ) (subcat )))
( "disconcerted"  v ( d i2 s k @ n s @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disconcerting"  v ( d i2 s k @ n s @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "disconcertingly"  a ( d i2 s k @ n s @@1 t i ng l ii ) ((pos "Pu%" ) ))
( "disconcerts"  v ( d i2 s k @ n s @@1 t s ) ((pos "Ha%" ) (subcat )))
( "disconnect"  v ( d i2 s k @ n e1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "disconnected"  v ( d i2 s k @ n e1 k t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "disconnecting"  v ( d i2 s k @ n e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disconnects"  v ( d i2 s k @ n e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "disconsolate"  j ( d i1 s k o1 n s @ l @ t ) ((pos "OA%" ) ))
( "disconsolately"  a ( d i1 s k o1 n s @ l @ t l ii ) ((pos "Pu%" ) ))
( "discontent"  v ( d i2 s k @ n t e1 n t ) ((pos "H0$" "M6%" ) (subcat )))
( "discontented"  v ( d i2 s k @ n t e1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "discontentedly"  a ( d i2 s k @ n t e1 n t i d l ii ) ((pos "Pu%" ) ))
( "discontenting"  v ( d i2 s k @ n t e1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "discontents"  v ( d i2 s k @ n t e1 n t s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "discontinuance"  n ( d i2 s k @ n t i1 n y u@ n s ) ((pos "K6$" ) ))
( "discontinuances"  n ( d i2 s k @ n t i1 n y u@ n s i z ) ((pos "Kj$" ) ))
( "discontinue"  v ( d i2 s k @ n t i1 n y uu ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "discontinued"  v ( d i2 s k @ n t i1 n y uu d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "discontinues"  v ( d i2 s k @ n t i1 n y uu z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "discontinuing"  v ( d i2 s k @ n t i1 n y uu i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "discontinuities"  n ( d i2 s k @ n t i n y u1 i t i z ) ((pos "Mj%" ) ))
( "discontinuity"  n ( d i2 s k @ n t i n y u1 i t ii ) ((pos "M8%" ) ))
( "discontinuous"  j ( d i2 s k @ n t i1 n y u@ s ) ((pos "OA%" ) ))
( "discord"  n ( d i1 s k oo d ) ((pos "M6%" ) ))
( "discordance"  n ( d i1 s k oo1 d @ n s ) ((pos "L@%" ) ))
( "discordant"  j ( d i1 s k oo1 d @ n t ) ((pos "OA%" ) ))
( "discordantly"  a ( d i1 s k oo1 d @ n t l ii ) ((pos "Pu%" ) ))
( "discords"  n ( d i1 s k oo d z ) ((pos "Mj%" ) ))
( "discos"  n ( d i1 s k ou z ) ((pos "Kj%" ) ))
( "discotheque"  n ( d i1 s k @ t e k ) ((pos "K6%" ) ))
( "discotheques"  n ( d i1 s k @ t e k s ) ((pos "Kj%" ) ))
( "discount"  n ( d i1 s k au n t ) ((pos "M6%" ) ))
( "discount"  v ( d i1 s k au1 n t ) ((pos "H0%" ) (subcat )))
( "discounted"  v ( d i1 s k au1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "discountenance"  v ( d i1 s k au1 n t i n @ n s ) ((pos "H2$" ) (subcat )))
( "discountenanced"  v ( d i1 s k au1 n t i n @ n s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "discountenances"  v ( d i1 s k au1 n t i n @ n s i z ) ((pos "Ha$" ) (subcat )))
( "discountenancing"  v ( d i1 s k au1 n t i n @ n s i ng ) ((pos "Hb$" ) (subcat )))
( "discounting"  v ( d i1 s k au1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "discounts"  n ( d i1 s k au n t s ) ((pos "Mj%" ) ))
( "discounts"  v ( d i1 s k au1 n t s ) ((pos "Ha%" ) (subcat )))
( "discourage"  v ( d i1 s k uh1 r i jh ) ((pos "H2%" ) (subcat "6A" )))
( "discouraged"  v ( d i1 s k uh1 r i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "discouragement"  n ( d i1 s k uh1 r i jh m @ n t ) ((pos "M6%" ) ))
( "discouragements"  n ( d i1 s k uh1 r i jh m @ n t s ) ((pos "Mj%" ) ))
( "discourages"  v ( d i1 s k uh1 r i jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "discouraging"  v ( d i1 s k uh1 r i jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "discourse"  n ( d i1 s k oo s ) ((pos "K6%" ) ))
( "discourse"  v ( d i1 s k oo1 s ) ((pos "I2%" ) ))
( "discoursed"  v ( d i1 s k oo1 s t ) ((pos "Ic%" "Id%" ) ))
( "discourses"  n ( d i1 s k oo s i z ) ((pos "Kj%" ) ))
( "discourses"  v ( d i1 s k oo1 s i z ) ((pos "Ia%" ) ))
( "discoursing"  v ( d i1 s k oo1 s i ng ) ((pos "Ib%" ) ))
( "discourteous"  j ( d i1 s k @@1 t i@ s ) ((pos "OA%" ) ))
( "discourteously"  a ( d i1 s k @@1 t i@ s l ii ) ((pos "Pu%" ) ))
( "discourtesies"  n ( d i1 s k @@1 t @ s i z ) ((pos "Mj%" ) ))
( "discourtesy"  n ( d i1 s k @@1 t @ s ii ) ((pos "M8%" ) ))
( "discover"  v ( d i1 s k uh1 v @ r ) ((pos "H0%" ) (subcat "6A" "8" "9" "10" )))
( "discovered"  v ( d i1 s k uh1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" )))
( "discoverer"  n ( d i1 s k uh1 v @ r @ r ) ((pos "K6%" ) ))
( "discoverers"  n ( d i1 s k uh1 v @ r @ z ) ((pos "Kj%" ) ))
( "discoveries"  n ( d i1 s k uh1 v @ r i z ) ((pos "Mj%" ) ))
( "discovering"  v ( d i1 s k uh1 v @ r i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" )))
( "discovers"  v ( d i1 s k uh1 v @ z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" )))
( "discovery"  n ( d i1 s k uh1 v @ r ii ) ((pos "M8%" ) ))
( "discredit"  v ( d i1 s k r e1 d i t ) ((pos "H0%" "L@%" ) (subcat )))
( "discreditable"  j ( d i1 s k r e1 d i t @ b l ) ((pos "OA%" ) ))
( "discreditably"  a ( d i1 s k r e1 d i t @ b l ii ) ((pos "Pu%" ) ))
( "discredited"  v ( d i1 s k r e1 d i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "discrediting"  v ( d i1 s k r e1 d i t i ng ) ((pos "Hb%" ) (subcat )))
( "discredits"  v ( d i1 s k r e1 d i t s ) ((pos "Ha%" ) (subcat )))
( "discreet"  j ( d i1 s k r ii1 t ) ((pos "OA%" ) ))
( "discreetly"  a ( d i1 s k r ii1 t l ii ) ((pos "Pu%" ) ))
( "discrepancies"  n ( d i1 s k r e1 p @ n s i z ) ((pos "Mj%" ) ))
( "discrepancy"  n ( d i1 s k r e1 p @ n s ii ) ((pos "M8%" ) ))
( "discrete"  j ( d i1 s k r ii1 t ) ((pos "OA$" ) ))
( "discreteness"  n ( d i1 s k r ii1 t n @ s ) ((pos "L@$" ) ))
( "discretion"  n ( d i1 s k r e1 sh @ n ) ((pos "L@%" ) ))
( "discretionary"  j ( d i1 s k r e1 sh @ n @ r ii ) ((pos "OA%" ) ))
( "discriminate"  v ( d i1 s k r i1 m i n ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "discriminated"  v ( d i1 s k r i1 m i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "discriminates"  v ( d i1 s k r i1 m i n ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "discriminating"  v ( d i1 s k r i1 m i n ei t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" )))
( "discrimination"  n ( d i1 s k r i2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "discriminatory"  j ( d i1 s k r i1 m i n @ t @ r ii ) ((pos "OA%" ) ))
( "discs"  n ( d i1 s k s ) ((pos "Kj%" ) ))
( "discursive"  j ( d i1 s k @@1 s i v ) ((pos "OA%" ) ))
( "discursively"  a ( d i1 s k @@1 s i v l ii ) ((pos "Pu%" ) ))
( "discursiveness"  n ( d i1 s k @@1 s i v n @ s ) ((pos "L@%" ) ))
( "discus"  n ( d i1 s k @ s ) ((pos "K7%" ) ))
( "discuses"  n ( d i1 s k @ s i z ) ((pos "Kj%" ) ))
( "discuss"  v ( d i1 s k uh1 s ) ((pos "H1%" ) (subcat "6A" "8" "10" )))
( "discussed"  v ( d i1 s k uh1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "10" )))
( "discusses"  v ( d i1 s k uh1 s i z ) ((pos "Ha%" ) (subcat "6A" "8" "10" )))
( "discussing"  v ( d i1 s k uh1 s i ng ) ((pos "Hb%" ) (subcat "6A" "8" "10" )))
( "discussion"  n ( d i1 s k uh1 sh @ n ) ((pos "M6%" ) ))
( "discussions"  n ( d i1 s k uh1 sh @ n z ) ((pos "Mj%" ) ))
( "disdain"  v ( d i1 s d ei1 n ) ((pos "H0%" "L@%" ) (subcat "6A" "6C" )))
( "disdained"  v ( d i1 s d ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "disdainful"  j ( d i1 s d ei1 n f @ l ) ((pos "OA%" ) ))
( "disdainfully"  a ( d i1 s d ei1 n f @ l ii ) ((pos "Pu%" ) ))
( "disdaining"  v ( d i1 s d ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "disdains"  v ( d i1 s d ei1 n z ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "disease"  n ( d i1 z ii1 z ) ((pos "M6%" ) ))
( "diseased"  j ( d i1 z ii1 z d ) ((pos "OA%" ) ))
( "diseases"  n ( d i1 z ii1 z i z ) ((pos "Mj%" ) ))
( "disembark"  v ( d i2 s i m b aa1 k ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "disembarkation"  n ( d i2 s e2 m b aa k ei1 sh @ n ) ((pos "K6%" ) ))
( "disembarkations"  n ( d i2 s e2 m b aa k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "disembarked"  v ( d i2 s i m b aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "disembarking"  v ( d i2 s i m b aa1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "disembarks"  v ( d i2 s i m b aa1 k s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "disembarrass"  v ( d i2 s i m b a1 r @ s ) ((pos "H1%" ) (subcat )))
( "disembarrassed"  v ( d i2 s i m b a1 r @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disembarrasses"  v ( d i2 s i m b a1 r @ s i z ) ((pos "Ha$" ) (subcat )))
( "disembarrassing"  v ( d i2 s i m b a1 r @ s i ng ) ((pos "Hb$" ) (subcat )))
( "disembarrassment"  n ( d i2 s i m b a1 r @ s m @ n t ) ((pos "K6$" ) ))
( "disembarrassments"  n ( d i2 s i m b a1 r @ s m @ n t s ) ((pos "Kj$" ) ))
( "disembodied"  v ( d i2 s i m b o1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disembodies"  v ( d i2 s i m b o1 d i z ) ((pos "Ha$" ) (subcat )))
( "disembody"  v ( d i2 s i m b o1 d ii ) ((pos "H3$" ) (subcat )))
( "disembodying"  v ( d i2 s i m b o1 d i i ng ) ((pos "Hb$" ) (subcat )))
( "disembowel"  v ( d i2 s i m b au1 @ l ) ((pos "H4%" ) (subcat )))
( "disembowelled"  v ( d i2 s i m b au1 @ l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disembowelling"  v ( d i2 s i m b au1 @ l i ng ) ((pos "Hb%" ) (subcat )))
( "disembowels"  v ( d i2 s i m b au1 @ l z ) ((pos "Ha%" ) (subcat )))
( "disenchant"  v ( d i2 s i n ch aa1 n t ) ((pos "H0%" ) (subcat )))
( "disenchanted"  v ( d i2 s i n ch aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disenchanting"  v ( d i2 s i n ch aa1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "disenchantment"  n ( d i2 s i n ch aa1 n t m @ n t ) ((pos "K6%" ) ))
( "disenchantments"  n ( d i2 s i n ch aa1 n t m @ n t s ) ((pos "Kj%" ) ))
( "disenchants"  v ( d i2 s i n ch aa1 n t s ) ((pos "Ha%" ) (subcat )))
( "disencumber"  v ( d i2 s i n k uh1 m b @ r ) ((pos "H0$" ) (subcat "6A" )))
( "disencumbered"  v ( d i2 s i n k uh1 m b @ d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "disencumbering"  v ( d i2 s i n k uh1 m b @ r i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "disencumbers"  v ( d i2 s i n k uh1 m b @ z ) ((pos "Ha$" ) (subcat "6A" )))
( "disenfranchise"  v ( d i2 s i n f r a1 n ch ai z ) ((pos "H2$" ) ))
( "disenfranchised"  v ( d i2 s i n f r a1 n ch ai z d ) ((pos "Hc$" "Hd$" ) ))
( "disenfranchises"  v ( d i2 s i n f r a1 n ch ai z i z ) ((pos "Ha$" ) ))
( "disenfranchising"  v ( d i2 s i n f r a1 n ch ai z i ng ) ((pos "Hb$" ) ))
( "disengage"  v ( d i2 s i n g ei1 jh ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "disengaged"  v ( d i2 s i n g ei1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "disengagement"  n ( d i2 s i n g ei1 jh m @ n t ) ((pos "K6%" ) ))
( "disengagements"  n ( d i2 s i n g ei1 jh m @ n t s ) ((pos "Kj%" ) ))
( "disengages"  v ( d i2 s i n g ei1 jh i z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "disengaging"  v ( d i2 s i n g ei1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "disentangle"  v ( d i2 s i n t a1 ng g l ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "disentangled"  v ( d i2 s i n t a1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "disentanglement"  n ( d i2 s i n t a1 ng g l m @ n t ) ((pos "K6%" ) ))
( "disentanglements"  n ( d i2 s i n t a1 ng g l m @ n t s ) ((pos "Kj$" ) ))
( "disentangles"  v ( d i2 s i n t a1 ng g l z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "disentangling"  v ( d i2 s i n t a1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "disequilibrium"  n ( d i2 s ii k w i l i1 b r i@ m ) ((pos "L@$" ) ))
( "disestablish"  v ( d i2 s i s t a1 b l i sh ) ((pos "H1%" ) (subcat )))
( "disestablished"  v ( d i2 s i s t a1 b l i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disestablishes"  v ( d i2 s i s t a1 b l i sh i z ) ((pos "Ha$" ) (subcat )))
( "disestablishing"  v ( d i2 s i s t a1 b l i sh i ng ) ((pos "Hb$" ) (subcat )))
( "disestablishment"  n ( d i2 s i s t a1 b l i sh m @ n t ) ((pos "L@%" ) ))
( "disfavour"  v ( d i2 s f ei1 v @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "disfavoured"  v ( d i2 s f ei1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disfavouring"  v ( d i2 s f ei1 v @ r i ng ) ((pos "Hb$" ) (subcat )))
( "disfavours"  v ( d i2 s f ei1 v @ z ) ((pos "Ha$" ) (subcat )))
( "disfigure"  v ( d i1 s f i1 g @ r ) ((pos "H2$" ) (subcat )))
( "disfigured"  v ( d i1 s f i1 g @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disfigurement"  n ( d i1 s f i1 g @ m @ n t ) ((pos "M6%" ) ))
( "disfigurements"  n ( d i1 s f i1 g @ m @ n t s ) ((pos "Mj%" ) ))
( "disfigures"  v ( d i1 s f i1 g @ z ) ((pos "Ha$" ) (subcat )))
( "disfiguring"  v ( d i1 s f i1 g @ r i ng ) ((pos "Hb$" ) (subcat )))
( "disforest"  v ( d i1 s f o1 r i s t ) ((pos "H0$" ) (subcat )))
( "disforested"  v ( d i1 s f o1 r i s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "disforesting"  v ( d i1 s f o1 r i s t i ng ) ((pos "Hb$" ) (subcat )))
( "disforests"  v ( d i1 s f o1 r i s t s ) ((pos "Ha$" ) (subcat )))
( "disfranchise"  v ( d i1 s f r a1 n ch ai z ) ((pos "H2$" ) (subcat )))
( "disfranchised"  v ( d i1 s f r a1 n ch ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "disfranchisement"  n ( d i1 s f r a1 n ch i z m @ n t ) ((pos "L@$" ) ))
( "disfranchises"  v ( d i1 s f r a1 n ch ai z i z ) ((pos "Ha$" ) (subcat )))
( "disfranchising"  v ( d i1 s f r a1 n ch ai z i ng ) ((pos "Hb$" ) (subcat )))
( "disgorge"  v ( d i1 s g oo1 jh ) ((pos "H2%" ) (subcat )))
( "disgorged"  v ( d i1 s g oo1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disgorges"  v ( d i1 s g oo1 jh i z ) ((pos "Ha%" ) (subcat )))
( "disgorging"  v ( d i1 s g oo1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "disgrace"  v ( d i1 s g r ei1 s ) ((pos "H2%" "L@%" ) (subcat )))
( "disgraced"  v ( d i1 s g r ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disgraceful"  j ( d i1 s g r ei1 s f @ l ) ((pos "OA%" ) ))
( "disgracefully"  a ( d i1 s g r ei1 s f @ l ii ) ((pos "Pu%" ) ))
( "disgraces"  v ( d i1 s g r ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "disgracing"  v ( d i1 s g r ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "disgruntled"  j ( d i1 s g r uh1 n t l d ) ((pos "OA%" ) ))
( "disguise"  v ( d i1 s g ai1 z ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "disguised"  v ( d i1 s g ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "disguises"  v ( d i1 s g ai1 z i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "disguising"  v ( d i1 s g ai1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disgust"  v ( d i1 s g uh1 s t ) ((pos "H0%" "L@%" ) (subcat )))
( "disgusted"  v ( d i1 s g uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disgustedly"  a ( d i1 z g uh1 s t i d l ii ) ((pos "Pu%" ) ))
( "disgusting"  v ( d i1 s g uh1 s t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "disgustingly"  a ( d i1 s g uh1 s t i ng l ii ) ((pos "Pu%" ) ))
( "disgusts"  v ( d i1 s g uh1 s t s ) ((pos "Ha%" ) (subcat )))
( "dish"  v ( d i1 sh ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "dishabille"  n ( d i2 s a b ii1 l ) ((pos "L@$" ) ))
( "disharmonious"  j ( d i2 s h aa m ou1 n i@ s ) ((pos "OA$" ) ))
( "disharmony"  n ( d i1 s h aa1 m @ n ii ) ((pos "L@%" ) ))
( "dishcloth"  n ( d i1 sh k l o th ) ((pos "K6%" ) ))
( "dishcloths"  n ( d i1 sh k l o th s ) ((pos "Kj%" ) ))
( "dishearten"  v ( d i1 s h aa1 t n ) ((pos "H0%" ) (subcat )))
( "disheartened"  v ( d i1 s h aa1 t n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disheartening"  v ( d i1 s h aa1 t n i ng ) ((pos "Hb%" ) (subcat )))
( "disheartens"  v ( d i1 s h aa1 t n z ) ((pos "Ha%" ) (subcat )))
( "dished"  v ( d i1 sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dishes"  v ( d i1 sh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "dishevelled"  j ( d i1 sh e1 v @ l d ) ((pos "OA%" ) ))
( "dishful"  n ( d i1 sh f u l ) ((pos "K6%" ) ))
( "dishfuls"  n ( d i1 sh f u l z ) ((pos "Kj%" ) ))
( "dishier"  j ( d i1 sh i@ r ) ((pos "Or%" ) ))
( "dishiest"  j ( d i1 sh i i s t ) ((pos "Os%" ) ))
( "dishing"  v ( d i1 sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dishonest"  j ( d i1 s o1 n i s t ) ((pos "OA%" ) ))
( "dishonesties"  n ( d i1 s o1 n i s t i z ) ((pos "Mj$" ) ))
( "dishonestly"  a ( d i1 s o1 n i s t l ii ) ((pos "Pu%" ) ))
( "dishonesty"  n ( d i1 s o1 n i s t ii ) ((pos "M8%" ) ))
( "dishonour"  v ( d i1 s o1 n @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "dishonourable"  j ( d i1 s o1 n @ r @ b l ) ((pos "OA%" ) ))
( "dishonourably"  a ( d i1 s o1 n @ r @ b l ii ) ((pos "Pu%" ) ))
( "dishonoured"  v ( d i1 s o1 n @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dishonouring"  v ( d i1 s o1 n @ r i ng ) ((pos "Hb%" ) (subcat )))
( "dishonours"  v ( d i1 s o1 n @ z ) ((pos "Ha%" ) (subcat )))
( "dishwasher"  n ( d i1 sh w o sh @ r ) ((pos "K6%" ) ))
( "dishwashers"  n ( d i1 sh w o sh @ z ) ((pos "Kj%" ) ))
( "dishwater"  n ( d i1 sh w oo t @ r ) ((pos "L@%" ) ))
( "dishy"  j ( d i1 sh ii ) ((pos "OD%" ) ))
( "disillusion"  v ( d i2 s i l uu1 zh n ) ((pos "H0%" "L@%" ) (subcat )))
( "disillusioned"  v ( d i2 s i l uu1 zh n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disillusioning"  v ( d i2 s i l uu1 zh n i ng ) ((pos "Hb%" ) (subcat )))
( "disillusionment"  n ( d i2 s i l uu1 zh n m @ n t ) ((pos "M6%" ) ))
( "disillusionments"  n ( d i2 s i l uu1 zh n m @ n t s ) ((pos "Mj%" ) ))
( "disillusions"  v ( d i2 s i l uu1 zh n z ) ((pos "Ha%" ) (subcat )))
( "disincentive"  n ( d i2 s i n s e1 n t i v ) ((pos "K6%" ) ))
( "disincentives"  n ( d i2 s i n s e1 n t i v z ) ((pos "Kj%" ) ))
( "disinclination"  n ( d i2 s i n k l i n ei1 sh @ n ) ((pos "M6%" ) ))
( "disinclinations"  n ( d i2 s i n k l i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "disincline"  v ( d i2 s i n k l ai1 n ) ((pos "H2$" ) (subcat "14" )))
( "disinclined"  v ( d i2 s i n k l ai1 n d ) ((pos "Hc$" "Hd%" ) (subcat "14" )))
( "disinclines"  v ( d i2 s i n k l ai1 n z ) ((pos "Ha%" ) (subcat "14" )))
( "disinclining"  v ( d i2 s i n k l ai1 n i ng ) ((pos "Hb$" ) (subcat "14" )))
( "disinfect"  v ( d i2 s i n f e1 k t ) ((pos "H0%" ) (subcat )))
( "disinfectant"  n ( d i2 s i n f e1 k t @ n t ) ((pos "K6%" "OA%" ) ))
( "disinfectants"  n ( d i2 s i n f e1 k t @ n t s ) ((pos "Kj%" ) ))
( "disinfected"  v ( d i2 s i n f e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disinfecting"  v ( d i2 s i n f e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "disinfection"  n ( d i2 s i n f e1 k sh @ n ) ((pos "L@$" ) ))
( "disinfects"  v ( d i2 s i n f e1 k t s ) ((pos "Ha%" ) (subcat )))
( "disinfest"  v ( d i2 s i n f e1 s t ) ((pos "H0$" ) (subcat )))
( "disinfestation"  n ( d i2 s i n f e s t ei1 sh @ n ) ((pos "L@$" ) ))
( "disinfested"  v ( d i2 s i n f e1 s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "disinfesting"  v ( d i2 s i n f e1 s t i ng ) ((pos "Hb$" ) (subcat )))
( "disinfests"  v ( d i2 s i n f e1 s t s ) ((pos "Ha$" ) (subcat )))
( "disinflation"  n ( d i2 s i n f l ei1 sh @ n ) ((pos "L@$" ) ))
( "disingenuous"  j ( d i2 s i n jh e1 n y u@ s ) ((pos "OA%" ) ))
( "disingenuously"  a ( d i2 s i n jh e1 n y u@ s l ii ) ((pos "Pu%" ) ))
( "disingenuousness"  n ( d i2 s i n jh e1 n y u@ s n @ s ) ((pos "L@%" ) ))
( "disinherit"  v ( d i2 s i n h e1 r i t ) ((pos "H0%" ) (subcat )))
( "disinheritance"  n ( d i2 s i n h e1 r i t @ n s ) ((pos "L@%" ) ))
( "disinherited"  v ( d i2 s i n h e1 r i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disinheriting"  v ( d i2 s i n h e1 r i t i ng ) ((pos "Hb%" ) (subcat )))
( "disinherits"  v ( d i2 s i n h e1 r i t s ) ((pos "Ha%" ) (subcat )))
( "disintegrate"  v ( d i1 s i1 n t i g r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "disintegrated"  v ( d i1 s i1 n t i g r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disintegrates"  v ( d i1 s i1 n t i g r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "disintegrating"  v ( d i1 s i1 n t i g r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disintegration"  n ( d i1 s i2 n t i g r ei1 sh @ n ) ((pos "M6%" ) ))
( "disintegrations"  n ( d i1 s i2 n t i g r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "disinter"  v ( d i2 s i n t @@1 r ) ((pos "H4%" ) (subcat )))
( "disinterested"  j ( d i1 s i1 n t r @ s t i d ) ((pos "OA%" ) ))
( "disinterestedly"  a ( d i1 s i1 n t r @ s t i d l ii ) ((pos "Pu%" ) ))
( "disinterestedness"  n ( d i1 s i1 n t r @ s t i d n @ s ) ((pos "L@$" ) ))
( "disinterment"  n ( d i2 s i n t @@1 m @ n t ) ((pos "K6%" ) ))
( "disinterments"  n ( d i2 s i n t @@1 m @ n t s ) ((pos "Kj$" ) ))
( "disinterred"  v ( d i2 s i n t @@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disinterring"  v ( d i2 s i n t @@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "disinters"  v ( d i2 s i n t @@1 z ) ((pos "Ha%" ) (subcat )))
( "disjoint"  v ( d i1 s jh oi1 n t ) ((pos "H0$" ) (subcat )))
( "disjointed"  v ( d i1 s jh oi1 n t i d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "disjointedly"  a ( d i1 s jh oi1 n t i d l ii ) ((pos "Pu%" ) ))
( "disjointedness"  n ( d i1 s jh oi1 n t i d n @ s ) ((pos "L@$" ) ))
( "disjointing"  v ( d i1 s jh oi1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "disjoints"  v ( d i1 s jh oi1 n t s ) ((pos "Ha$" ) (subcat )))
( "disjunctive"  j ( d i1 s jh uh1 ng k t i v ) ((pos "OA$" ) ))
( "disk"  n ( d i1 s k ) ((pos "K6%" ) ))
( "disks"  n ( d i1 s k s ) ((pos "Kj%" ) ))
( "dislike"  v ( d i1 s l ai1 k ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "disliked"  v ( d i1 s l ai1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dislikes"  v ( d i1 s l ai1 k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "disliking"  v ( d i1 s l ai1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dislocate"  v ( d i1 s l @ k ei t ) ((pos "H2%" ) (subcat )))
( "dislocated"  v ( d i1 s l @ k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dislocates"  v ( d i1 s l @ k ei t s ) ((pos "Ha%" ) (subcat )))
( "dislocating"  v ( d i1 s l @ k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "dislocation"  n ( d i2 s l @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "dislocations"  n ( d i2 s l @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "dislodge"  v ( d i1 s l o1 jh ) ((pos "H2%" ) (subcat "6A" )))
( "dislodged"  v ( d i1 s l o1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dislodgement"  n ( d i1 s l o1 jh m @ n t ) ((pos "K6$" ) ))
( "dislodgements"  n ( d i1 s l o1 jh m @ n t s ) ((pos "Kj$" ) ))
( "dislodges"  v ( d i1 s l o1 jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "dislodging"  v ( d i1 s l o1 jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disloyal"  j ( d i1 s l oi1 @ l ) ((pos "OA%" ) ))
( "disloyally"  a ( d i1 s l oi1 @ l ii ) ((pos "Pu%" ) ))
( "disloyalties"  n ( d i1 s l oi1 @ l t i z ) ((pos "Mj$" ) ))
( "disloyalty"  n ( d i1 s l oi1 @ l t ii ) ((pos "M8%" ) ))
( "dismal"  j ( d i1 z m @ l ) ((pos "OA%" ) ))
( "dismally"  a ( d i1 z m @ l ii ) ((pos "Pu%" ) ))
( "dismantle"  v ( d i1 s m a1 n t l ) ((pos "H2%" ) (subcat )))
( "dismantled"  v ( d i1 s m a1 n t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dismantlement"  n ( d i1 s m a1 n t l m @ n t ) ((pos "L@$" ) ))
( "dismantles"  v ( d i1 s m a1 n t l z ) ((pos "Ha%" ) (subcat )))
( "dismantling"  v ( d i1 s m a1 n t l i ng ) ((pos "Hb%" ) (subcat )))
( "dismay"  v ( d i1 s m ei1 ) ((pos "H0%" "L@%" ) (subcat )))
( "dismayed"  v ( d i1 s m ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dismaying"  v ( d i1 s m ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "dismays"  v ( d i1 s m ei1 z ) ((pos "Ha%" ) (subcat )))
( "dismember"  v ( d i1 s m e1 m b @ r ) ((pos "H0%" ) (subcat )))
( "dismembered"  v ( d i1 s m e1 m b @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dismembering"  v ( d i1 s m e1 m b @ r i ng ) ((pos "Hb%" ) (subcat )))
( "dismemberment"  n ( d i1 s m e1 m b @ m @ n t ) ((pos "K6$" ) ))
( "dismemberments"  n ( d i1 s m e1 m b @ m @ n t s ) ((pos "Kj$" ) ))
( "dismembers"  v ( d i1 s m e1 m b @ z ) ((pos "Ha%" ) (subcat )))
( "dismiss"  v ( d i1 z m i1 s ) ((pos "H1%" ) (subcat "6A" )))
( "dismissal"  n ( d i1 z m i1 s l ) ((pos "M6%" ) ))
( "dismissals"  n ( d i1 z m i1 s l z ) ((pos "Mj%" ) ))
( "dismissed"  v ( d i1 z m i1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dismisses"  v ( d i1 z m i1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "dismissing"  v ( d i1 z m i1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dismount"  v ( d i2 s m au1 n t ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "dismounted"  v ( d i2 s m au1 n t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" )))
( "dismounting"  v ( d i2 s m au1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "dismounts"  v ( d i2 s m au1 n t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "disobedience"  n ( d i2 s @ b ii1 d i@ n s ) ((pos "L@%" ) ))
( "disobedient"  j ( d i2 s @ b ii1 d i@ n t ) ((pos "OA%" ) ))
( "disobediently"  a ( d i2 s @ b ii1 d i@ n t l ii ) ((pos "Pu%" ) ))
( "disobey"  v ( d i2 s @ b ei1 ) ((pos "H0%" ) (subcat "2A" )))
( "disobeyed"  v ( d i2 s @ b ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "disobeying"  v ( d i2 s @ b ei1 i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "disobeys"  v ( d i2 s @ b ei1 z ) ((pos "Ha%" ) (subcat "2A" )))
( "disoblige"  v ( d i2 s @ b l ai1 jh ) ((pos "H2$" ) (subcat )))
( "disobliged"  v ( d i2 s @ b l ai1 jh d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "disobliges"  v ( d i2 s @ b l ai1 jh i z ) ((pos "Ha$" ) (subcat )))
( "disobliging"  v ( d i2 s @ b l ai1 jh i ng ) ((pos "Hb$" ) (subcat )))
( "disorder"  v ( d i1 s oo1 d @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "disordered"  v ( d i1 s oo1 d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disordering"  v ( d i1 s oo1 d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "disorderly"  j ( d i1 s oo1 d @ l ii ) ((pos "OA%" ) ))
( "disorders"  v ( d i1 s oo1 d @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "disorganization"  n ( d i1 s oo2 g @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "disorganize"  v ( d i2 s oo1 g @ n ai z ) ((pos "H2$" ) (subcat )))
( "disorganized"  v ( d i2 s oo1 g @ n ai z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disorganizes"  v ( d i2 s oo1 g @ n ai z i z ) ((pos "Ha$" ) (subcat )))
( "disorganizing"  v ( d i2 s oo1 g @ n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "disorient"  v ( d i1 s oo1 r i@ n t ) ((pos "H0$" ) (subcat )))
( "disorientate"  v ( d i1 s oo1 r i@ n t ei t ) ((pos "H2$" ) (subcat )))
( "disorientated"  v ( d i1 s oo1 r i@ n t ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disorientates"  v ( d i1 s oo1 r i@ n t ei t s ) ((pos "Ha$" ) (subcat )))
( "disorientating"  v ( d i1 s oo1 r i@ n t ei t i ng ) ((pos "Hb$" ) (subcat )))
( "disoriented"  v ( d i1 s oo1 r i@ n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disorienting"  v ( d i1 s oo1 r i@ n t i ng ) ((pos "Hb$" ) (subcat )))
( "disorients"  v ( d i1 s oo1 r i@ n t s ) ((pos "Ha$" ) (subcat )))
( "disown"  v ( d i1 s ou1 n ) ((pos "H0%" ) (subcat )))
( "disowned"  v ( d i1 s ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disowning"  v ( d i1 s ou1 n i ng ) ((pos "Hb%" ) (subcat )))
( "disowns"  v ( d i1 s ou1 n z ) ((pos "Ha%" ) (subcat )))
( "disparage"  v ( d i1 s p a1 r i jh ) ((pos "H2%" ) (subcat )))
( "disparaged"  v ( d i1 s p a1 r i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disparagement"  n ( d i1 s p a1 r i jh m @ n t ) ((pos "K6%" ) ))
( "disparagements"  n ( d i1 s p a1 r i jh m @ n t s ) ((pos "Kj$" ) ))
( "disparages"  v ( d i1 s p a1 r i jh i z ) ((pos "Ha%" ) (subcat )))
( "disparaging"  v ( d i1 s p a1 r i jh i ng ) ((pos "Hb%" ) (subcat )))
( "disparagingly"  a ( d i1 s p a1 r i jh i ng l ii ) ((pos "Pu%" ) ))
( "disparate"  n ( d i1 s p @ r @ t ) ((pos "Kj%" "OA%" ) ))
( "disparities"  n ( d i1 s p a1 r i t i z ) ((pos "Mj%" ) ))
( "disparity"  n ( d i1 s p a1 r i t ii ) ((pos "M8%" ) ))
( "dispassionate"  j ( d i1 s p a1 sh @ n @ t ) ((pos "OA%" ) ))
( "dispassionately"  a ( d i1 s p a1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "dispassionateness"  n ( d i1 s p a1 sh @ n @ t n @ s ) ((pos "L@$" ) ))
( "dispatch"  v ( d i1 s p a1 ch ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "dispatch-box"  n ( d i1 s p a1 ch - b o k s ) ((pos "K7%" ) ))
( "dispatch-boxes"  n ( d i1 s p a1 ch - b o k s i z ) ((pos "Kj%" ) ))
( "dispatch-rider"  n ( d i1 s p a1 ch - r ai d @ r ) ((pos "K6%" ) ))
( "dispatch-riders"  n ( d i1 s p a1 ch - r ai d @ z ) ((pos "Kj%" ) ))
( "dispatched"  v ( d i1 s p a1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dispatches"  v ( d i1 s p a1 ch i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "dispatching"  v ( d i1 s p a1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dispel"  v ( d i1 s p e1 l ) ((pos "H4%" ) (subcat )))
( "dispelled"  v ( d i1 s p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dispelling"  v ( d i1 s p e1 l i ng ) ((pos "Hb%" ) (subcat )))
( "dispels"  v ( d i1 s p e1 l z ) ((pos "Ha%" ) (subcat )))
( "dispensable"  j ( d i1 s p e1 n s @ b l ) ((pos "OA%" ) ))
( "dispensaries"  n ( d i1 s p e1 n s @ r i z ) ((pos "Kj%" ) ))
( "dispensary"  n ( d i1 s p e1 n s @ r ii ) ((pos "K8%" ) ))
( "dispensation"  n ( d i2 s p e n s ei1 sh @ n ) ((pos "M6%" ) ))
( "dispensations"  n ( d i2 s p e n s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "dispense"  v ( d i1 s p e1 n s ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "dispensed"  v ( d i1 s p e1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "dispenser"  n ( d i1 s p e1 n s @ r ) ((pos "K6%" ) ))
( "dispensers"  n ( d i1 s p e1 n s @ z ) ((pos "Kj%" ) ))
( "dispenses"  v ( d i1 s p e1 n s i z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "dispensing"  v ( d i1 s p e1 n s i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "dispersal"  n ( d i1 s p @@1 s l ) ((pos "M6%" ) ))
( "dispersals"  n ( d i1 s p @@1 s l z ) ((pos "Mj$" ) ))
( "disperse"  v ( d i1 s p @@1 s ) ((pos "J2%" ) (subcat "2A" )))
( "dispersed"  v ( d i1 s p @@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disperses"  v ( d i1 s p @@1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "dispersing"  v ( d i1 s p @@1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dispersion"  n ( d i1 s p @@1 sh @ n ) ((pos "K6%" ) ))
( "dispersions"  n ( d i1 s p @@1 sh @ n z ) ((pos "Kj$" ) ))
( "dispirit"  v ( d i1 s p i1 r i t ) ((pos "H0$" ) (subcat )))
( "dispirited"  v ( d i1 s p i1 r i t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "dispiritedly"  a ( d i1 s p i1 r i t i d l ii ) ((pos "Pu%" ) ))
( "dispiriting"  v ( d i1 s p i1 r i t i ng ) ((pos "Hb%" ) (subcat )))
( "dispirits"  v ( d i1 s p i1 r i t s ) ((pos "Ha$" ) (subcat )))
( "displace"  v ( d i1 s p l ei1 s ) ((pos "H2%" ) (subcat )))
( "displaced"  v ( d i1 s p l ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "displacement"  n ( d i1 s p l ei1 s m @ n t ) ((pos "M6%" ) ))
( "displacements"  n ( d i1 s p l ei1 s m @ n t s ) ((pos "Mj%" ) ))
( "displaces"  v ( d i1 s p l ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "displacing"  v ( d i1 s p l ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "display"  v ( d i1 s p l ei1 ) ((pos "H0%" "M6%" ) (subcat )))
( "displayed"  v ( d i1 s p l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "displaying"  v ( d i1 s p l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "displays"  v ( d i1 s p l ei1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "displease"  v ( d i1 s p l ii1 z ) ((pos "H2%" ) (subcat )))
( "displeased"  v ( d i1 s p l ii1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "displeases"  v ( d i1 s p l ii1 z i z ) ((pos "Ha%" ) (subcat )))
( "displeasing"  v ( d i1 s p l ii1 z i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "displeasingly"  a ( d i1 s p l ii1 z i ng l ii ) ((pos "Pu%" ) ))
( "displeasure"  n ( d i1 s p l e1 zh @ r ) ((pos "L@%" ) ))
( "disport"  v ( d i1 s p oo1 t ) ((pos "H0%" ) (subcat )))
( "disported"  v ( d i1 s p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disporting"  v ( d i1 s p oo1 t i ng ) ((pos "Hb%" ) (subcat )))
( "disports"  v ( d i1 s p oo1 t s ) ((pos "Ha%" ) (subcat )))
( "disposable"  j ( d i1 s p ou1 z @ b l ) ((pos "OA%" ) ))
( "disposal"  n ( d i1 s p ou1 z l ) ((pos "L@%" ) ))
( "dispose"  v ( d i1 s p ou1 z ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "disposed"  v ( d i1 s p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "disposes"  v ( d i1 s p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "disposing"  v ( d i1 s p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "disposition"  n ( d i2 s p @ z i1 sh @ n ) ((pos "K6%" ) ))
( "dispositions"  n ( d i2 s p @ z i1 sh @ n z ) ((pos "Kj%" ) ))
( "dispossess"  v ( d i2 s p @ z e1 s ) ((pos "H1%" ) (subcat )))
( "dispossessed"  v ( d i2 s p @ z e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dispossesses"  v ( d i2 s p @ z e1 s i z ) ((pos "Ha$" ) (subcat )))
( "dispossessing"  v ( d i2 s p @ z e1 s i ng ) ((pos "Hb$" ) (subcat )))
( "dispossession"  n ( d i2 s p @ z e1 sh @ n ) ((pos "K6$" ) ))
( "dispossessions"  n ( d i2 s p @ z e1 sh @ n z ) ((pos "Kj$" ) ))
( "disproof"  n ( d i1 s p r uu1 f ) ((pos "M6%" ) ))
( "disproofs"  n ( d i1 s p r uu1 f s ) ((pos "Mj$" ) ))
( "disproportion"  n ( d i2 s p r @ p oo1 sh @ n ) ((pos "L@%" ) ))
( "disproportionate"  j ( d i2 s p r @ p oo1 sh @ n @ t ) ((pos "OA%" ) ))
( "disproportionately"  a ( d i2 s p r @ p oo1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "disprove"  v ( d i2 s p r uu1 v ) ((pos "H2%" ) (subcat )))
( "disproved"  v ( d i2 s p r uu1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disproves"  v ( d i2 s p r uu1 v z ) ((pos "Ha%" ) (subcat )))
( "disproving"  v ( d i2 s p r uu1 v i ng ) ((pos "Hb%" ) (subcat )))
( "disputable"  j ( d i1 s p y uu1 t @ b l ) ((pos "OA%" ) ))
( "disputant"  n ( d i1 s p y uu1 t @ n t ) ((pos "K6%" ) ))
( "disputants"  n ( d i1 s p y uu1 t @ n t s ) ((pos "Kj%" ) ))
( "disputation"  n ( d i2 s p y uu t ei1 sh @ n ) ((pos "M6%" ) ))
( "disputations"  n ( d i2 s p y uu t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "disputatious"  j ( d i2 s p y uu t ei1 sh @ s ) ((pos "OA$" ) ))
( "disputatiously"  a ( d i2 s p y uu t ei1 sh @ s l ii ) ((pos "Pu$" ) ))
( "dispute"  v ( d i1 s p y uu1 t ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" "6A" "8" )))
( "disputed"  v ( d i1 s p y uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" )))
( "disputes"  v ( d i1 s p y uu1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "8" )))
( "disputing"  v ( d i1 s p y uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" )))
( "disqualification"  n ( d i1 s k w o2 l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "disqualifications"  n ( d i1 s k w o2 l i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "disqualified"  v ( d i1 s k w o1 l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "disqualifies"  v ( d i1 s k w o1 l i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "disqualify"  v ( d i1 s k w o1 l i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "disqualifying"  v ( d i1 s k w o1 l i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "disquiet"  v ( d i1 s k w ai1 @ t ) ((pos "H0$" "L@%" ) (subcat )))
( "disquieted"  v ( d i1 s k w ai1 @ t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "disquieting"  v ( d i1 s k w ai1 @ t i ng ) ((pos "Hb$" "OA%" ) (subcat )))
( "disquietingly"  a ( d i1 s k w ai1 @ t i ng l ii ) ((pos "Pu%" ) ))
( "disquiets"  v ( d i1 s k w ai1 @ t s ) ((pos "Ha$" ) (subcat )))
( "disquietude"  n ( d i1 s k w ai1 @ t y uu d ) ((pos "L@$" ) ))
( "disquisition"  n ( d i2 s k w i z i1 sh @ n ) ((pos "K6%" ) ))
( "disquisitions"  n ( d i2 s k w i z i1 sh @ n z ) ((pos "Kj%" ) ))
( "disregard"  v ( d i2 s r i g aa1 d ) ((pos "H0%" "L@%" ) (subcat )))
( "disregarded"  v ( d i2 s r i g aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disregarding"  v ( d i2 s r i g aa1 d i ng ) ((pos "Hb%" ) (subcat )))
( "disregards"  v ( d i2 s r i g aa1 d z ) ((pos "Ha%" ) (subcat )))
( "disrepair"  n ( d i2 s r i p e@1 r ) ((pos "L@%" ) ))
( "disreputable"  j ( d i1 s r e1 p y u t @ b l ) ((pos "OA%" ) ))
( "disreputably"  a ( d i1 s r e1 p y u t @ b l ii ) ((pos "Pu%" ) ))
( "disrepute"  n ( d i2 s r i p y uu1 t ) ((pos "L@%" ) ))
( "disrespect"  n ( d i2 s r i s p e1 k t ) ((pos "L@%" ) ))
( "disrespectful"  j ( d i2 s r i s p e1 k t f @ l ) ((pos "OA%" ) ))
( "disrespectfully"  a ( d i2 s r i s p e1 k t f @ l ii ) ((pos "Pu%" ) ))
( "disrobe"  v ( d i1 s r ou1 b ) ((pos "J2%" ) (subcat "2A" )))
( "disrobed"  v ( d i1 s r ou1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "disrobes"  v ( d i1 s r ou1 b z ) ((pos "Ja%" ) (subcat "2A" )))
( "disrobing"  v ( d i1 s r ou1 b i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disrupt"  v ( d i1 s r uh1 p t ) ((pos "H0%" ) (subcat )))
( "disrupted"  v ( d i1 s r uh1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disrupting"  v ( d i1 s r uh1 p t i ng ) ((pos "Hb%" ) (subcat )))
( "disruption"  n ( d i1 s r uh1 p sh @ n ) ((pos "M6%" ) ))
( "disruptions"  n ( d i1 s r uh1 p sh @ n z ) ((pos "Mj%" ) ))
( "disruptive"  j ( d i1 s r uh1 p t i v ) ((pos "OA%" ) ))
( "disrupts"  v ( d i1 s r uh1 p t s ) ((pos "Ha%" ) (subcat )))
( "dissatisfaction"  n ( d i2 s s a2 t i s f a1 k sh @ n ) ((pos "L@%" ) ))
( "dissatisfied"  v ( d i1 s a1 t i s f ai d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "dissatisfies"  v ( d i1 s a1 t i s f ai z ) ((pos "Ha$" ) (subcat )))
( "dissatisfy"  v ( d i1 s a1 t i s f ai ) ((pos "H3$" ) (subcat )))
( "dissatisfying"  v ( d i1 s a1 t i s f ai i ng ) ((pos "Hb%" ) (subcat )))
( "dissect"  v ( d i1 s e1 k t ) ((pos "H0%" ) (subcat )))
( "dissected"  v ( d i1 s e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dissecting"  v ( d i1 s e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "dissection"  n ( d i1 s e1 k sh @ n ) ((pos "M6%" ) ))
( "dissections"  n ( d i1 s e1 k sh @ n z ) ((pos "Mj%" ) ))
( "dissects"  v ( d i1 s e1 k t s ) ((pos "Ha%" ) (subcat )))
( "dissemble"  v ( d i1 s e1 m b l ) ((pos "J2%" ) (subcat "2A" )))
( "dissembled"  v ( d i1 s e1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dissembler"  n ( d i1 s e1 m b l @ r ) ((pos "K6%" ) ))
( "dissemblers"  n ( d i1 s e1 m b l @ z ) ((pos "Kj%" ) ))
( "dissembles"  v ( d i1 s e1 m b l z ) ((pos "Ja%" ) (subcat "2A" )))
( "dissembling"  v ( d i1 s e1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "disseminate"  v ( d i1 s e1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "disseminated"  v ( d i1 s e1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disseminates"  v ( d i1 s e1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "disseminating"  v ( d i1 s e1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "dissemination"  n ( d i1 s e2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "dissension"  n ( d i1 s e1 n sh @ n ) ((pos "M6%" ) ))
( "dissensions"  n ( d i1 s e1 n sh @ n z ) ((pos "Mj$" ) ))
( "dissent"  v ( d i1 s e1 n t ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "dissented"  v ( d i1 s e1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dissenter"  n ( d i1 s e1 n t @ r ) ((pos "K6%" ) ))
( "dissenters"  n ( d i1 s e1 n t @ z ) ((pos "Kj%" ) ))
( "dissenting"  v ( d i1 s e1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "dissents"  v ( d i1 s e1 n t s ) ((pos "Ia%" ) (subcat "2A" )))
( "dissertation"  n ( d i2 s @ t ei1 sh @ n ) ((pos "K6%" ) ))
( "dissertations"  n ( d i2 s @ t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "disservice"  n ( d i1 s s @@1 v i s ) ((pos "M6%" ) ))
( "disservices"  n ( d i1 s s @@1 v i s i z ) ((pos "Mj$" ) ))
( "dissever"  v ( d i1 s e1 v @ r ) ((pos "H0$" ) (subcat )))
( "dissevered"  v ( d i1 s e1 v @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "dissevering"  v ( d i1 s e1 v @ r i ng ) ((pos "Hb$" ) (subcat )))
( "dissevers"  v ( d i1 s e1 v @ z ) ((pos "Ha$" ) (subcat )))
( "dissidence"  n ( d i1 s i d @ n s ) ((pos "L@$" ) ))
( "dissident"  n ( d i1 s i d @ n t ) ((pos "K6%" "OA%" ) ))
( "dissidents"  n ( d i1 s i d @ n t s ) ((pos "Kj%" ) ))
( "dissimilar"  j ( d i1 s i1 m i l @ r ) ((pos "OA%" ) ))
( "dissimilarities"  n ( d i2 s i2 m i l a1 r i t i z ) ((pos "Mj%" ) ))
( "dissimilarity"  n ( d i2 s i2 m i l a1 r i t ii ) ((pos "M8%" ) ))
( "dissimilitude"  n ( d i2 s i m i1 l i t y uu d ) ((pos "L@$" ) ))
( "dissimulate"  v ( d i1 s i1 m y u l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "dissimulated"  v ( d i1 s i1 m y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dissimulates"  v ( d i1 s i1 m y u l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "dissimulating"  v ( d i1 s i1 m y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dissimulation"  n ( d i1 s i2 m y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "dissimulations"  n ( d i1 s i2 m y u l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "dissipate"  v ( d i1 s i p ei t ) ((pos "J2%" ) (subcat "2A" )))
( "dissipated"  v ( d i1 s i p ei t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "dissipates"  v ( d i1 s i p ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "dissipating"  v ( d i1 s i p ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dissipation"  n ( d i2 s i p ei1 sh @ n ) ((pos "L@%" ) ))
( "dissociate"  v ( d i1 s ou1 sh i ei t ) ((pos "H2%" ) (subcat "6A" )))
( "dissociated"  v ( d i1 s ou1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dissociates"  v ( d i1 s ou1 sh i ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "dissociating"  v ( d i1 s ou1 sh i ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dissociation"  n ( d i1 s ou2 s i ei1 sh @ n ) ((pos "L@%" ) ))
( "dissolubility"  n ( d i1 s o2 l y u b i1 l i t ii ) ((pos "L@$" ) ))
( "dissoluble"  j ( d i1 s o1 l y u b l ) ((pos "OA%" ) ))
( "dissolute"  j ( d i1 s @ l uu t ) ((pos "OA%" ) ))
( "dissolutely"  a ( d i1 s @ l uu t l ii ) ((pos "Pu%" ) ))
( "dissolution"  n ( d i2 s @ l uu1 sh @ n ) ((pos "M6%" ) ))
( "dissolutions"  n ( d i2 s @ l uu1 sh @ n z ) ((pos "Mj$" ) ))
( "dissolve"  v ( d i1 z o1 l v ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dissolved"  v ( d i1 z o1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dissolves"  v ( d i1 z o1 l v z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dissolving"  v ( d i1 z o1 l v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "dissonance"  n ( d i1 s @ n @ n s ) ((pos "M6%" ) ))
( "dissonances"  n ( d i1 s @ n @ n s i z ) ((pos "Mj$" ) ))
( "dissonant"  j ( d i1 s @ n @ n t ) ((pos "OA%" ) ))
( "dissuade"  v ( d i1 s w ei1 d ) ((pos "H2%" ) (subcat "6A" )))
( "dissuaded"  v ( d i1 s w ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dissuades"  v ( d i1 s w ei1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "dissuading"  v ( d i1 s w ei1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dissuasion"  n ( d i1 s w ei1 zh n ) ((pos "L@%" ) ))
( "dissyllable"  n ( d i1 s i1 l @ b l ) ((pos "K6$" ) ))
( "dissyllables"  n ( d i1 s i1 l @ b l z ) ((pos "Kj$" ) ))
( "distaff"  n ( d i1 s t aa f ) ((pos "K6$" ) ))
( "distaffs"  n ( d i1 s t aa f s ) ((pos "Kj$" ) ))
( "distance"  v ( d i1 s t @ n s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "distanced"  v ( d i1 s t @ n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "distances"  v ( d i1 s t @ n s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "distancing"  v ( d i1 s t @ n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "distant"  j ( d i1 s t @ n t ) ((pos "OA%" ) ))
( "distantly"  a ( d i1 s t @ n t l ii ) ((pos "Pu%" ) ))
( "distaste"  n ( d i1 s t ei1 s t ) ((pos "M6%" ) ))
( "distasteful"  j ( d i1 s t ei1 s t f @ l ) ((pos "OA%" ) ))
( "distastefully"  a ( d i1 s t ei1 s t f @ l ii ) ((pos "Pu%" ) ))
( "distastefulness"  n ( d i1 s t ei1 s t f @ l n @ s ) ((pos "L@%" ) ))
( "distastes"  n ( d i1 s t ei1 s t s ) ((pos "Mj$" ) ))
( "distemper"  v ( d i1 s t e1 m p @ r ) ((pos "H0$" "L@%" ) (subcat "6A" )))
( "distempered"  v ( d i1 s t e1 m p @ d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "distempering"  v ( d i1 s t e1 m p @ r i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "distempers"  v ( d i1 s t e1 m p @ z ) ((pos "Ha$" ) (subcat "6A" )))
( "distend"  v ( d i1 s t e1 n d ) ((pos "J0%" ) (subcat "2A" )))
( "distended"  v ( d i1 s t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "distending"  v ( d i1 s t e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "distends"  v ( d i1 s t e1 n d z ) ((pos "Ja%" ) (subcat "2A" )))
( "distension"  n ( d i1 s t e1 n sh @ n ) ((pos "K6%" ) ))
( "distensions"  n ( d i1 s t e1 n sh @ n z ) ((pos "Kj$" ) ))
( "distil"  v ( d i1 s t i1 l ) ((pos "J4%" ) (subcat "2A" "6A" "14" )))
( "distillation"  n ( d i2 s t i l ei1 sh @ n ) ((pos "M6%" ) ))
( "distillations"  n ( d i2 s t i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "distilled"  v ( d i1 s t i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "distiller"  n ( d i1 s t i1 l @ r ) ((pos "K6%" ) ))
( "distilleries"  n ( d i1 s t i1 l @ r i z ) ((pos "Kj%" ) ))
( "distillers"  n ( d i1 s t i1 l @ z ) ((pos "Kj%" ) ))
( "distillery"  n ( d i1 s t i1 l @ r ii ) ((pos "K8%" ) ))
( "distilling"  v ( d i1 s t i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "distils"  v ( d i1 s t i1 l z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "distinct"  j ( d i1 s t i1 ng k t ) ((pos "OA%" ) ))
( "distinction"  n ( d i1 s t i1 ng k sh @ n ) ((pos "M6%" ) ))
( "distinctions"  n ( d i1 s t i1 ng k sh @ n z ) ((pos "Mj%" ) ))
( "distinctive"  j ( d i1 s t i1 ng k t i v ) ((pos "OA%" ) ))
( "distinctively"  a ( d i1 s t i1 ng k t i v l ii ) ((pos "Pu%" ) ))
( "distinctiveness"  n ( d i1 s t i1 ng k t i v n i s ) ((pos "L@%" ) ))
( "distinctly"  a ( d i1 s t i1 ng k t l ii ) ((pos "Pu%" ) ))
( "distinctness"  n ( d i1 s t i1 ng k t n @ s ) ((pos "L@%" ) ))
( "distinguish"  v ( d i1 s t i1 ng g w i sh ) ((pos "J1%" ) (subcat "3A" "6A" )))
( "distinguishable"  j ( d i1 s t i1 ng g w i sh @ b l ) ((pos "OA%" ) ))
( "distinguished"  v ( d i1 s t i1 ng g w i sh t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" )))
( "distinguishes"  v ( d i1 s t i1 ng g w i sh i z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "distinguishing"  v ( d i1 s t i1 ng g w i sh i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "distort"  v ( d i1 s t oo1 t ) ((pos "H0%" ) (subcat )))
( "distorted"  v ( d i1 s t oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "distorting"  v ( d i1 s t oo1 t i ng ) ((pos "Hb%" ) (subcat )))
( "distortion"  n ( d i1 s t oo1 sh @ n ) ((pos "M6%" ) ))
( "distortions"  n ( d i1 s t oo1 sh @ n z ) ((pos "Mj%" ) ))
( "distorts"  v ( d i1 s t oo1 t s ) ((pos "Ha%" ) (subcat )))
( "distract"  v ( d i1 s t r a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "distracted"  v ( d i1 s t r a1 k t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "distractedly"  a ( d i1 s t r a1 k t i d l ii ) ((pos "Pu%" ) ))
( "distracting"  v ( d i1 s t r a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "distraction"  n ( d i1 s t r a1 k sh @ n ) ((pos "M6%" ) ))
( "distractions"  n ( d i1 s t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "distracts"  v ( d i1 s t r a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "distrain"  v ( d i1 s t r ei1 n ) ((pos "I0$" ) (subcat "2A" )))
( "distrained"  v ( d i1 s t r ei1 n d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "distraining"  v ( d i1 s t r ei1 n i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "distrains"  v ( d i1 s t r ei1 n z ) ((pos "Ia$" ) (subcat "2A" )))
( "distraint"  n ( d i1 s t r ei1 n t ) ((pos "K6$" ) ))
( "distraints"  n ( d i1 s t r ei1 n t s ) ((pos "Kj$" ) ))
( "distrait"  j ( d i1 s t r ei1 ) ((pos "OA$" ) ))
( "distraught"  j ( d i1 s t r oo1 t ) ((pos "OA%" ) ))
( "distress"  v ( d i1 s t r e1 s ) ((pos "H1%" "L@%" ) (subcat )))
( "distressed"  v ( d i1 s t r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "distresses"  v ( d i1 s t r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "distressful"  j ( d i1 s t r e1 s f u l ) ((pos "OA$" ) ))
( "distressfully"  a ( d i1 s t r e1 s f @ l ii ) ((pos "Pu$" ) ))
( "distressing"  v ( d i1 s t r e1 s i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "distressingly"  a ( d i1 s t r e1 s i ng l ii ) ((pos "Pu%" ) ))
( "distribute"  v ( d i1 s t r i1 b y uu t ) ((pos "H2%" ) (subcat "6A" )))
( "distributed"  v ( d i1 s t r i1 b y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "distributes"  v ( d i1 s t r i1 b y uu t s ) ((pos "Ha%" ) (subcat "6A" )))
( "distributing"  v ( d i1 s t r i1 b y uu t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "distribution"  n ( d i2 s t r i b y uu1 sh @ n ) ((pos "M6%" ) ))
( "distributions"  n ( d i2 s t r i b y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "distributive"  j ( d i1 s t r i1 b y u t i v ) ((pos "OA%" ) ))
( "distributively"  a ( d i1 s t r i1 b y u t i v l ii ) ((pos "Pu$" ) ))
( "distributor"  n ( d i1 s t r i1 b y uu t @ r ) ((pos "K6%" ) ))
( "distributors"  n ( d i1 s t r i1 b y uu t @ z ) ((pos "Kj%" ) ))
( "district"  n ( d i1 s t r i k t ) ((pos "K6%" ) ))
( "districts"  n ( d i1 s t r i k t s ) ((pos "Kj%" ) ))
( "distrust"  v ( d i1 s t r uh1 s t ) ((pos "H0%" "Mi%" ) (subcat )))
( "distrusted"  v ( d i1 s t r uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "distrustful"  j ( d i1 s t r uh1 s t f @ l ) ((pos "OA%" ) ))
( "distrustfully"  a ( d i1 s t r uh1 s t f @ l ii ) ((pos "Pu%" ) ))
( "distrustfulness"  n ( d i1 s t r uh1 s t f @ l n @ s ) ((pos "L@%" ) ))
( "distrusting"  v ( d i1 s t r uh1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "distrusts"  v ( d i1 s t r uh1 s t s ) ((pos "Ha%" ) (subcat )))
( "disturb"  v ( d i1 s t @@1 b ) ((pos "H0%" ) (subcat )))
( "disturbance"  n ( d i1 s t @@1 b @ n s ) ((pos "M6%" ) ))
( "disturbances"  n ( d i1 s t @@1 b @ n s i z ) ((pos "Mj%" ) ))
( "disturbed"  v ( d i1 s t @@1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "disturbing"  v ( d i1 s t @@1 b i ng ) ((pos "Hb%" ) (subcat )))
( "disturbingly"  a ( d i1 s t @@1 b i ng l ii ) ((pos "Pu%" ) ))
( "disturbs"  v ( d i1 s t @@1 b z ) ((pos "Ha%" ) (subcat )))
( "disunion"  n ( d i1 s y uu1 n i@ n ) ((pos "L@$" ) ))
( "disunite"  v ( d i2 s y uu n ai1 t ) ((pos "J2$" ) (subcat "2A" )))
( "disunited"  v ( d i2 s y uu n ai1 t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "disunites"  v ( d i2 s y uu n ai1 t s ) ((pos "Ja$" ) (subcat "2A" )))
( "disuniting"  v ( d i2 s y uu n ai1 t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "disunity"  n ( d i1 s y uu1 n i t ii ) ((pos "L@%" ) ))
( "disuse"  n ( d i1 s y uu1 s ) ((pos "L@%" ) ))
( "disused"  j ( d i1 s y uu1 z d ) ((pos "OA%" ) ))
( "disyllabic"  j ( d i2 s i l a1 b i k ) ((pos "OA$" ) ))
( "disyllable"  n ( d i1 s i1 l @ b l ) ((pos "K6$" ) ))
( "disyllables"  n ( d i1 s i1 l @ b l z ) ((pos "Kj$" ) ))
( "ditch"  v ( d i1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "ditched"  v ( d i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ditches"  v ( d i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "ditching"  v ( d i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dither"  v ( d i1 dh @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "dithered"  v ( d i1 dh @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dithering"  v ( d i1 dh @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "dithers"  v ( d i1 dh @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "ditties"  n ( d i1 t i z ) ((pos "Kj%" ) ))
( "ditto"  n ( d i1 t ou ) ((pos "Ki%" ) ))
( "ditty"  n ( d i1 t ii ) ((pos "K8%" ) ))
( "diurnal"  j ( d ai1 @@1 n l ) ((pos "OA$" ) ))
( "divagate"  v ( d ai1 v @ g ei t ) ((pos "I2$" ) (subcat "2A" )))
( "divagated"  v ( d ai1 v @ g ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "divagates"  v ( d ai1 v @ g ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "divagating"  v ( d ai1 v @ g ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "divagation"  n ( d ai2 v @ g ei1 sh @ n ) ((pos "M6$" ) ))
( "divagations"  n ( d ai2 v @ g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "divan"  n ( d i1 v a1 n ) ((pos "K6%" ) ))
( "divan-bed"  n ( d i1 v a2 n - b e1 d ) ((pos "K6%" ) ))
( "divan-beds"  n ( d i1 v a2 n - b e1 d z ) ((pos "Kj%" ) ))
( "divans"  n ( d i1 v a1 n z ) ((pos "Kj%" ) ))
( "dive"  v ( d ai1 v ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "dive-bomb"  v ( d ai1 v - b o m ) ((pos "J0%" ) ))
( "dive-bombed"  v ( d ai1 v - b o m d ) ((pos "Jc%" "Jd%" ) ))
( "dive-bomber"  n ( d ai1 v - b o m @ r ) ((pos "K6%" ) ))
( "dive-bombers"  n ( d ai1 v - b o m @ z ) ((pos "Kj%" ) ))
( "dive-bombing"  v ( d ai1 v - b o m i ng ) ((pos "Jb%" ) ))
( "dive-bombs"  v ( d ai1 v - b o m z ) ((pos "Ja%" ) ))
( "dived"  v ( d ai1 v d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "diver"  n ( d ai1 v @ r ) ((pos "K6%" ) ))
( "diverge"  v ( d ai1 v @@1 jh ) ((pos "I2%" ) (subcat "2A" )))
( "diverged"  v ( d ai1 v @@1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "divergence"  n ( d ai1 v @@1 jh @ n s ) ((pos "M6%" ) ))
( "divergences"  n ( d ai1 v @@1 jh @ n s i z ) ((pos "Mj%" ) ))
( "divergencies"  n ( d ai1 v @@1 jh @ n s i z ) ((pos "Mj$" ) ))
( "divergency"  n ( d ai1 v @@1 jh @ n s ii ) ((pos "M8$" ) ))
( "divergent"  j ( d ai1 v @@1 jh @ n t ) ((pos "OA%" ) ))
( "diverges"  v ( d ai1 v @@1 jh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "diverging"  v ( d ai1 v @@1 jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "divers"  j ( d ai1 v @@ z ) ((pos "OA$" ) ))
( "divers"  n ( d ai1 v @ z ) ((pos "Kj%" ) ))
( "diverse"  j ( d ai1 v @@1 s ) ((pos "OA%" ) ))
( "diversely"  a ( d ai1 v @@1 s l ii ) ((pos "Pu%" ) ))
( "diversification"  n ( d ai1 v @@2 s i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "diversified"  v ( d ai1 v @@1 s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "diversifies"  v ( d ai1 v @@1 s i f ai z ) ((pos "Ha%" ) (subcat )))
( "diversify"  v ( d ai1 v @@1 s i f ai ) ((pos "H3%" ) (subcat )))
( "diversifying"  v ( d ai1 v @@1 s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "diversion"  n ( d ai1 v @@1 sh @ n ) ((pos "M6%" ) ))
( "diversionary"  j ( d ai1 v @@1 sh @ n @ r ii ) ((pos "OA%" ) ))
( "diversionist"  n ( d ai1 v @@1 sh @ n i s t ) ((pos "K6$" ) ))
( "diversionists"  n ( d ai1 v @@1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "diversions"  n ( d ai1 v @@1 sh @ n z ) ((pos "Mj%" ) ))
( "diversity"  n ( d ai1 v @@1 s i t ii ) ((pos "L@%" ) ))
( "divert"  v ( d ai1 v @@1 t ) ((pos "H0%" ) (subcat "6A" )))
( "diverted"  v ( d ai1 v @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "diverting"  v ( d ai1 v @@1 t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "divertingly"  a ( d ai1 v @@1 t i ng l ii ) ((pos "Pu%" ) ))
( "diverts"  v ( d ai1 v @@1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "dives"  v ( d ai1 v z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "divest"  v ( d ai1 v e1 s t ) ((pos "H0%" ) (subcat )))
( "divested"  v ( d ai1 v e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "divesting"  v ( d ai1 v e1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "divests"  v ( d ai1 v e1 s t s ) ((pos "Ha%" ) (subcat )))
( "divide"  v ( d i1 v ai1 d ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "divided"  v ( d i1 v ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "dividend"  n ( d i1 v i d e n d ) ((pos "K6%" ) ))
( "dividend-warrant"  n ( d i1 v i d e n d - w o2 r @ n t ) ((pos "K6$" ) ))
( "dividend-warrants"  n ( d i1 v i d e n d - w o2 r @ n t s ) ((pos "Kj$" ) ))
( "dividends"  n ( d i1 v i d e n d z ) ((pos "Kj%" ) ))
( "dividers"  n ( d i1 v ai1 d @ z ) ((pos "Kj%" ) ))
( "divides"  v ( d i1 v ai1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "dividing"  v ( d i1 v ai1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "divination"  n ( d i2 v i n ei1 sh @ n ) ((pos "M6%" ) ))
( "divinations"  n ( d i2 v i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "divine"  v ( d i1 v ai1 n ) ((pos "J2%" "K6%" "OA%" ) (subcat "2A" "6A" )))
( "divined"  v ( d i1 v ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "divinely"  a ( d i1 v ai1 n l ii ) ((pos "Pu%" ) ))
( "diviner"  n ( d i1 v ai1 n @ r ) ((pos "K6%" ) ))
( "diviners"  n ( d i1 v ai1 n @ z ) ((pos "Kj%" ) ))
( "divines"  v ( d i1 v ai1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "diving"  v ( d ai1 v i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "diving-bell"  n ( d ai1 v i ng - b e l ) ((pos "K6%" ) ))
( "diving-bells"  n ( d ai1 v i ng - b e l z ) ((pos "Kj%" ) ))
( "diving-board"  n ( d ai1 v i ng - b oo d ) ((pos "K6%" ) ))
( "diving-boards"  n ( d ai1 v i ng - b oo d z ) ((pos "Kj%" ) ))
( "diving-dress"  n ( d ai1 v i ng - d r e s ) ((pos "L@$" ) ))
( "diving-suit"  n ( d ai1 v i ng - s uu t ) ((pos "K6%" ) ))
( "diving-suits"  n ( d ai1 v i ng - s uu t s ) ((pos "Kj%" ) ))
( "divining"  v ( d i1 v ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "divinities"  n ( d i1 v i1 n i t i z ) ((pos "Mj$" ) ))
( "divinity"  n ( d i1 v i1 n i t ii ) ((pos "M8%" ) ))
( "divisible"  j ( d i1 v i1 z @ b l ) ((pos "OA%" ) ))
( "division"  n ( d i1 v i1 zh n ) ((pos "M6%" ) ))
( "divisional"  j ( d i1 v i1 zh @ n l ) ((pos "OA%" ) ))
( "divisions"  n ( d i1 v i1 zh n z ) ((pos "Mj%" ) ))
( "divisor"  n ( d i1 v ai1 z @ r ) ((pos "K6%" ) ))
( "divisors"  n ( d i1 v ai1 z @ z ) ((pos "Kj%" ) ))
( "divorce"  v ( d i1 v oo1 s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "divorced"  v ( d i1 v oo1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "divorcee"  n ( d i1 v oo2 s ii1 ) ((pos "K6%" ) ))
( "divorcees"  n ( d i1 v oo2 s ii1 z ) ((pos "Kj%" ) ))
( "divorces"  v ( d i1 v oo1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "divorcing"  v ( d i1 v oo1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "divot"  n ( d i1 v @ t ) ((pos "K6$" ) ))
( "divots"  n ( d i1 v @ t s ) ((pos "Kj$" ) ))
( "divulge"  v ( d ai1 v uh1 l jh ) ((pos "H2%" ) (subcat "6A" )))
( "divulged"  v ( d ai1 v uh1 l jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "divulgence"  n ( d ai1 v uh1 l jh @ n s ) ((pos "K6%" ) ))
( "divulgences"  n ( d ai1 v uh1 l jh @ n s i z ) ((pos "Kj$" ) ))
( "divulges"  v ( d ai1 v uh1 l jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "divulging"  v ( d ai1 v uh1 l jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "divvies"  n ( d i1 v i z ) ((pos "Kj$" ) ))
( "divvy"  n ( d i1 v ii ) ((pos "K8$" ) ))
( "dixie"  n ( d i1 k s ii ) ((pos "K6$" ) ))
( "dixies"  n ( d i1 k s i z ) ((pos "Kj$" ) ))
( "dizzied"  v ( d i1 z i d ) ((pos "Hc%" "Hd%" ) ))
( "dizzier"  j ( d i1 z i@ r ) ((pos "Or%" ) ))
( "dizzies"  v ( d i1 z i z ) ((pos "Ha%" ) ))
( "dizziest"  j ( d i1 z i i s t ) ((pos "Os%" ) ))
( "dizzily"  a ( d i1 z i l ii ) ((pos "Pu%" ) ))
( "dizziness"  n ( d i1 z i n @ s ) ((pos "L@%" ) ))
( "dizzy"  v ( d i1 z ii ) ((pos "H3%" "OD%" ) ))
( "dizzying"  v ( d i1 z i i ng ) ((pos "Hb%" ) ))
( "djinn"  n ( jh i1 n ) ((pos "K6$" ) ))
( "djinns"  n ( jh i1 n z ) ((pos "Kj$" ) ))
( "do"  n ( d ou1 ) ((pos "Ki%" ) ))
( "do"  v ( d uu ) ((pos "G5*" "J5*" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "6C" "7B" "12B" "13B" "15A" )))
( "do's"  n ( d uu1 z ) ((pos "Kj%" ) ))
( "do-gooder"  n ( d uu2 - g u1 d @ r ) ((pos "K6%" ) ))
( "do-gooders"  n ( d uu2 - g u1 d @ z ) ((pos "Kj%" ) ))
( "dobbin"  n ( d o1 b i n ) ((pos "K6$" ) ))
( "dobbins"  n ( d o1 b i n z ) ((pos "Kj$" ) ))
( "docile"  j ( d ou1 s ai l ) ((pos "OA%" ) ))
( "docility"  n ( d ou1 s i1 l i t ii ) ((pos "L@%" ) ))
( "dock"  v ( d o1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "dock-dues"  n ( d o1 k - d y uu z ) ((pos "Kj$" ) ))
( "docked"  v ( d o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "docker"  n ( d o1 k @ r ) ((pos "K6%" ) ))
( "dockers"  n ( d o1 k @ z ) ((pos "Kj%" ) ))
( "docket"  v ( d o1 k i t ) ((pos "H0$" "K6%" ) (subcat )))
( "docketed"  v ( d o1 k i t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "docketing"  v ( d o1 k i t i ng ) ((pos "Hb$" ) (subcat )))
( "dockets"  v ( d o1 k i t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "docking"  v ( d o1 k i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "docks"  v ( d o1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "dockyard"  n ( d o1 k y aa d ) ((pos "K6%" ) ))
( "dockyards"  n ( d o1 k y aa d z ) ((pos "Kj%" ) ))
( "doctor"  v ( d o1 k t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "doctorate"  n ( d o1 k t @ r @ t ) ((pos "K6%" ) ))
( "doctorates"  n ( d o1 k t @ r @ t s ) ((pos "Kj%" ) ))
( "doctored"  v ( d o1 k t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "doctoring"  v ( d o1 k t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "doctors"  v ( d o1 k t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "doctrinaire"  n ( d o2 k t r i n e@1 r ) ((pos "K6$" "OA%" ) ))
( "doctrinaires"  n ( d o2 k t r i n e@1 z ) ((pos "Kj$" ) ))
( "doctrinal"  j ( d o1 k t r ai1 n l ) ((pos "OA%" ) ))
( "doctrine"  n ( d o1 k t r i n ) ((pos "M6%" ) ))
( "doctrines"  n ( d o1 k t r i n z ) ((pos "Mj%" ) ))
( "document"  n ( d o1 k y u m @ n t ) ((pos "K6%" ) ))
( "document"  v ( d o1 k y u m e n t ) ((pos "H0%" ) (subcat )))
( "documentary"  j ( d o2 k y u m e1 n t @ r ii ) ((pos "OA%" ) ))
( "documentation"  n ( d o2 k y u m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "documented"  v ( d o1 k y u m e n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "documenting"  v ( d o1 k y u m e n t i ng ) ((pos "Hb%" ) (subcat )))
( "documents"  n ( d o1 k y u m @ n t s ) ((pos "Kj%" ) ))
( "documents"  v ( d o1 k y u m e n t s ) ((pos "Ha$" ) (subcat )))
( "dodder"  v ( d o1 d @ r ) ((pos "I0%" ) (subcat "2A" )))
( "doddered"  v ( d o1 d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dodderer"  n ( d o1 d @ r @ r ) ((pos "K6%" ) ))
( "dodderers"  n ( d o1 d @ r @ z ) ((pos "Kj%" ) ))
( "doddering"  v ( d o1 d @ r i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "dodders"  v ( d o1 d @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "doddery"  j ( d o1 d @ r ii ) ((pos "OA%" ) ))
( "dodge"  v ( d o1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "dodged"  v ( d o1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "dodgem"  n ( d o1 jh @ m ) ((pos "K6$" ) ))
( "dodgems"  n ( d o1 jh @ m z ) ((pos "Kj%" ) ))
( "dodger"  n ( d o1 jh @ r ) ((pos "K6%" ) ))
( "dodgers"  n ( d o1 jh @ z ) ((pos "Kj%" ) ))
( "dodges"  v ( d o1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "dodging"  v ( d o1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "dodgy"  j ( d o1 jh ii ) ((pos "OA%" ) ))
( "dodo"  n ( d ou1 d ou ) ((pos "K6%" ) ))
( "dodoes"  n ( d ou1 d ou z ) ((pos "Kj$" ) ))
( "dodos"  n ( d ou1 d ou z ) ((pos "Kj%" ) ))
( "doe"  n ( d ou1 ) ((pos "K6$" ) ))
( "doer"  n ( d uu1 @ r ) ((pos "K6%" ) ))
( "doers"  n ( d uu1 @ z ) ((pos "Kj%" ) ))
( "does"  n ( d ou1 z ) ((pos "Kj$" ) ))
( "does"  v ( d uh z ) ((pos "Ga*" "Ja*" ) (subcat "2A" "2B" "2C" "3A" "6A" "6C" "7B" "12B" "13B" "15A" )))
( "doesn't"  v ( d uh1 z n t ) ((pos "Gg%" ) ))
( "doff"  v ( d o1 f ) ((pos "H0%" ) (subcat )))
( "doffed"  v ( d o1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "doffing"  v ( d o1 f i ng ) ((pos "Hb%" ) (subcat )))
( "doffs"  v ( d o1 f s ) ((pos "Ha%" ) (subcat )))
( "dog"  v ( d o1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "dog's-tooth"  n ( d o1 g z - t uu th ) ((pos "L@%" ) ))
( "dog-biscuit"  n ( d o1 g - b i s k i t ) ((pos "K6%" ) ))
( "dog-biscuits"  n ( d o1 g - b i s k i t s ) ((pos "Kj%" ) ))
( "dog-cart"  n ( d o1 g - k aa t ) ((pos "K6%" ) ))
( "dog-carts"  n ( d o1 g - k aa t s ) ((pos "Kj%" ) ))
( "dog-collar"  n ( d o1 g - k o l @ r ) ((pos "K6%" ) ))
( "dog-collars"  n ( d o1 g - k o l @ z ) ((pos "Kj%" ) ))
( "dog-days"  n ( d o1 g - d ei z ) ((pos "Kj%" ) ))
( "dog-eared"  j ( d o1 g - i@ d ) ((pos "OA%" ) ))
( "dog-like"  j ( d o1 g - l ai k ) ((pos "OA%" ) ))
( "doge"  n ( d ou1 jh ) ((pos "K6$" ) ))
( "doges"  n ( d ou1 jh i z ) ((pos "Kj$" ) ))
( "dogfish"  n ( d o1 g f i sh ) ((pos "K9%" ) ))
( "dogfishes"  n ( d o1 g f i sh i z ) ((pos "Kj$" ) ))
( "dogged"  j ( d o1 g i d ) ((pos "OA%" ) ))
( "dogged"  v ( d o1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "doggedly"  a ( d o1 g i d l ii ) ((pos "Pu%" ) ))
( "doggedness"  n ( d o1 g i d n @ s ) ((pos "L@%" ) ))
( "doggerel"  n ( d o1 g @ r @ l ) ((pos "L@%" ) ))
( "doggie"  n ( d o1 g ii ) ((pos "K6%" ) ))
( "doggies"  n ( d o1 g i z ) ((pos "Kj%" ) ))
( "dogging"  v ( d o1 g i ng ) ((pos "Hb%" ) (subcat )))
( "doggo"  a ( d o1 g ou ) ((pos "Pu%" ) ))
( "doggy"  n ( d o1 g ii ) ((pos "K8%" ) ))
( "doghouse"  n ( d o1 g h au s ) ((pos "K6%" ) ))
( "doghouses"  n ( d o1 g h au z i z ) ((pos "Kj$" ) ))
( "dogma"  n ( d o1 g m @ ) ((pos "M6%" ) ))
( "dogmas"  n ( d o1 g m @ z ) ((pos "Mj%" ) ))
( "dogmatic"  j ( d o1 g m a1 t i k ) ((pos "OA%" ) ))
( "dogmatically"  a ( d o1 g m a1 t i k l ii ) ((pos "Pu%" ) ))
( "dogmatism"  n ( d o1 g m @ t i z @ m ) ((pos "L@%" ) ))
( "dogmatize"  v ( d o1 g m @ t ai z ) ((pos "J2$" ) (subcat "2A" )))
( "dogmatized"  v ( d o1 g m @ t ai z d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "dogmatizes"  v ( d o1 g m @ t ai z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "dogmatizing"  v ( d o1 g m @ t ai z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "dogs"  v ( d o1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "dogsbodies"  n ( d o1 g z b o d i z ) ((pos "Kj$" ) ))
( "dogsbody"  n ( d o1 g z b o d ii ) ((pos "K8%" ) ))
( "dogtooth"  n ( d o1 g t uu th ) ((pos "L@%" ) ))
( "dogtrot"  n ( d o1 g t r o t ) ((pos "Ki$" ) ))
( "dogwatch"  n ( d o1 g w o ch ) ((pos "K7$" ) ))
( "dogwatches"  n ( d o1 g w o ch i z ) ((pos "Kj$" ) ))
( "dogwood"  n ( d o1 g w u d ) ((pos "K6$" ) ))
( "dogwoods"  n ( d o1 g w u d z ) ((pos "Kj$" ) ))
( "doh"  n ( d ou1 ) ((pos "Ki$" ) ))
( "dohs"  n ( d ou1 z ) ((pos "Kj$" ) ))
( "doilies"  n ( d oi1 l i z ) ((pos "Kj%" ) ))
( "doily"  n ( d oi1 l ii ) ((pos "K8%" ) ))
( "doing"  v ( d uu1 i ng ) ((pos "Gb%" "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" "6C" "7B" "12B" "13B" "15A" )))
( "doings"  n ( d uu1 i ng z ) ((pos "Kj%" ) ))
( "doldrums"  n ( d o1 l d r @ m z ) ((pos "Kj%" ) ))
( "dole"  v ( d ou1 l ) ((pos "H2%" "M6%" ) (subcat )))
( "doled"  v ( d ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "doleful"  j ( d ou1 l f u l ) ((pos "OA%" ) ))
( "dolefully"  a ( d ou1 l f u l ii ) ((pos "Pu%" ) ))
( "doles"  v ( d ou1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "doling"  v ( d ou1 l i ng ) ((pos "Hb%" ) (subcat )))
( "doll"  v ( d o1 l ) ((pos "J0$" "K6%" ) (subcat "2C" )))
( "dollar"  n ( d o1 l @ r ) ((pos "K6%" ) ))
( "dollars"  n ( d o1 l @ z ) ((pos "Kj%" ) ))
( "dolled"  v ( d o1 l d ) ((pos "Jc$" "Jd%" ) (subcat "2C" )))
( "dollies"  n ( d o1 l i z ) ((pos "Kj%" ) ))
( "dolling"  v ( d o1 l i ng ) ((pos "Jb$" ) (subcat "2C" )))
( "dollop"  n ( d o1 l @ p ) ((pos "K6%" ) ))
( "dollops"  n ( d o1 l @ p s ) ((pos "Kj%" ) ))
( "dolls"  v ( d o1 l z ) ((pos "Ja$" "Kj%" ) (subcat "2C" )))
( "dolly"  n ( d o1 l ii ) ((pos "K8%" ) ))
( "dolmen"  n ( d o1 l m e n ) ((pos "K6$" ) ))
( "dolmens"  n ( d o1 l m e n z ) ((pos "Kj$" ) ))
( "dolour"  n ( d o1 l @ r ) ((pos "K6$" ) ))
( "dolourous"  j ( d o1 l @ r @ s ) ((pos "OA$" ) ))
( "dolours"  n ( d o1 l @ z ) ((pos "Kj$" ) ))
( "dolphin"  n ( d o1 l f i n ) ((pos "K6%" ) ))
( "dolphins"  n ( d o1 l f i n z ) ((pos "Kj%" ) ))
( "dolt"  n ( d ou1 l t ) ((pos "K6%" ) ))
( "doltish"  j ( d ou1 l t i sh ) ((pos "OA%" ) ))
( "dolts"  n ( d ou1 l t s ) ((pos "Kj%" ) ))
( "domain"  n ( d ou1 m ei1 n ) ((pos "K6%" ) ))
( "domains"  n ( d ou1 m ei1 n z ) ((pos "Kj%" ) ))
( "dome"  n ( d ou1 m ) ((pos "K6%" ) ))
( "domed"  j ( d ou1 m d ) ((pos "OA%" ) ))
( "domes"  n ( d ou1 m z ) ((pos "Kj%" ) ))
( "domestic"  j ( d @1 m e1 s t i k ) ((pos "OA%" ) ))
( "domestically"  a ( d @1 m e1 s t i k l ii ) ((pos "Pu%" ) ))
( "domesticate"  v ( d @1 m e1 s t i k ei t ) ((pos "H2%" ) (subcat )))
( "domesticated"  v ( d @1 m e1 s t i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "domesticates"  v ( d @1 m e1 s t i k ei t s ) ((pos "Ha%" ) (subcat )))
( "domesticating"  v ( d @1 m e1 s t i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "domestication"  n ( d @1 m e2 s t i k ei1 sh @ n ) ((pos "L@%" ) ))
( "domesticity"  n ( d o2 m e s t i1 s i t ii ) ((pos "L@%" ) ))
( "domicile"  n ( d o1 m i s ai l ) ((pos "K6%" ) ))
( "domiciles"  n ( d o1 m i s ai l z ) ((pos "Kj$" ) ))
( "domiciliary"  j ( d o2 m i s i1 l i@ r ii ) ((pos "OA%" ) ))
( "dominance"  n ( d o1 m i n @ n s ) ((pos "L@%" ) ))
( "dominant"  j ( d o1 m i n @ n t ) ((pos "OA%" ) ))
( "dominantly"  a ( d o1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "dominate"  v ( d o1 m i n ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "dominated"  v ( d o1 m i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "dominates"  v ( d o1 m i n ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "dominating"  v ( d o1 m i n ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "domination"  n ( d o2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "domineer"  v ( d o2 m i n i@1 r ) ((pos "I0%" ) (subcat "2A" )))
( "domineered"  v ( d o2 m i n i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "domineering"  v ( d o2 m i n i@1 r i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "domineeringly"  a ( d o2 m i n i@1 r i ng l ii ) ((pos "Pu%" ) ))
( "domineers"  v ( d o2 m i n i@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "dominie"  n ( d o1 m i n ii ) ((pos "K6$" ) ))
( "dominies"  n ( d o1 m i n i z ) ((pos "Kj$" ) ))
( "dominion"  n ( d @1 m i1 n i@ n ) ((pos "M6%" ) ))
( "dominions"  n ( d @1 m i1 n i@ n z ) ((pos "Mj%" ) ))
( "domino"  n ( d o1 m i n ou ) ((pos "K6%" ) ))
( "dominoes"  n ( d o1 m i n ou z ) ((pos "Kj%" ) ))
( "dominos"  n ( d o1 m i n ou z ) ((pos "Kj$" ) ))
( "don"  v ( d o1 n ) ((pos "H4%" "K6%" ) (subcat )))
( "don't"  v ( d ou n t ) ((pos "Gg*" ) ))
( "don't-know"  n ( d ou1 n t - n ou1 ) ((pos "K6%" ) ))
( "don't-knows"  n ( d ou1 n t - n ou1 z ) ((pos "Kj%" ) ))
( "donate"  v ( d ou1 n ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "donated"  v ( d ou1 n ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "donates"  v ( d ou1 n ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "donating"  v ( d ou1 n ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "donation"  n ( d ou1 n ei1 sh @ n ) ((pos "M6%" ) ))
( "donations"  n ( d ou1 n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "done"  v ( d uh n ) ((pos "Gd*" "Jd*" ) (subcat "2A" "2B" "2C" "3A" "6A" "6C" "7B" "12B" "13B" "15A" )))
( "donjon"  n ( d o1 n jh @ n ) ((pos "K6$" ) ))
( "donjons"  n ( d o1 n jh @ n z ) ((pos "Kj$" ) ))
( "donkey"  n ( d o1 ng k ii ) ((pos "K6%" ) ))
( "donkey-jacket"  n ( d o1 ng k i - jh a k i t ) ((pos "K6%" ) ))
( "donkey-jackets"  n ( d o1 ng k i - jh a k i t s ) ((pos "Kj%" ) ))
( "donkey-work"  n ( d o1 ng k i - w @@ k ) ((pos "L@%" ) ))
( "donkeys"  n ( d o1 ng k i z ) ((pos "Kj%" ) ))
( "donned"  v ( d o1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "donning"  v ( d o1 n i ng ) ((pos "Hb%" ) (subcat )))
( "donnish"  j ( d o1 n i sh ) ((pos "OA%" ) ))
( "donor"  n ( d ou1 n @ r ) ((pos "K6%" ) ))
( "donors"  n ( d ou1 n @ z ) ((pos "Kj%" ) ))
( "dons"  v ( d o1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "doodle"  v ( d uu1 d l ) ((pos "I2%" ) (subcat )))
( "doodlebug"  n ( d uu1 d l b uh g ) ((pos "K6%" ) ))
( "doodlebugs"  n ( d uu1 d l b uh g z ) ((pos "Kj%" ) ))
( "doodled"  v ( d uu1 d l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "doodles"  v ( d uu1 d l z ) ((pos "Ia%" ) (subcat )))
( "doodling"  v ( d uu1 d l i ng ) ((pos "Ib%" ) (subcat )))
( "doom"  v ( d uu1 m ) ((pos "H0$" "K6%" ) (subcat "6A" "14" )))
( "doomed"  v ( d uu1 m d ) ((pos "Hc$" "Hd%" ) (subcat "6A" "14" )))
( "dooming"  v ( d uu1 m i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "dooms"  v ( d uu1 m z ) ((pos "Ha$" "Kj$" ) (subcat "6A" "14" )))
( "door"  n ( d oo1 r ) ((pos "K6*" ) ))
( "door-to-door"  j ( d oo2 - t @ - d oo1 r ) ((pos "OA%" ) ))
( "doorbell"  n ( d oo1 b e l ) ((pos "K6%" ) ))
( "doorbells"  n ( d oo1 b e l z ) ((pos "Kj%" ) ))
( "doorcase"  n ( d oo1 k ei s ) ((pos "K6$" ) ))
( "doorcases"  n ( d oo1 k ei s i z ) ((pos "Kj$" ) ))
( "doorframe"  n ( d oo1 f r ei m ) ((pos "K6%" ) ))
( "doorframes"  n ( d oo1 f r ei m z ) ((pos "Kj%" ) ))
( "doorhandle"  n ( d oo1 h a n d l ) ((pos "K6%" ) ))
( "doorhandles"  n ( d oo1 h a n d l z ) ((pos "Kj%" ) ))
( "doorkeeper"  n ( d oo1 k ii p @ r ) ((pos "K6%" ) ))
( "doorkeepers"  n ( d oo1 k ii p @ z ) ((pos "Kj%" ) ))
( "doorknob"  n ( d oo1 n o b ) ((pos "K6%" ) ))
( "doorknobs"  n ( d oo1 n o b z ) ((pos "Kj%" ) ))
( "doorknocker"  n ( d oo1 n o k @ r ) ((pos "K6%" ) ))
( "doorknockers"  n ( d oo1 n o k @ z ) ((pos "Kj%" ) ))
( "doorman"  n ( d oo1 m a n ) ((pos "Ki%" ) ))
( "doormat"  n ( d oo1 m a t ) ((pos "K6%" ) ))
( "doormats"  n ( d oo1 m a t s ) ((pos "Kj%" ) ))
( "doormen"  n ( d oo1 m e n ) ((pos "Kj%" ) ))
( "doornail"  n ( d oo1 n ei l ) ((pos "K6%" ) ))
( "doornails"  n ( d oo1 n ei l z ) ((pos "Kj%" ) ))
( "doorplate"  n ( d oo1 p l ei t ) ((pos "K6%" ) ))
( "doorplates"  n ( d oo1 p l ei t s ) ((pos "Kj%" ) ))
( "doorpost"  n ( d oo1 p ou s t ) ((pos "K6%" ) ))
( "doorposts"  n ( d oo1 p ou s t s ) ((pos "Kj%" ) ))
( "doors"  n ( d oo1 z ) ((pos "Kj%" ) ))
( "doorstep"  n ( d oo1 s t e p ) ((pos "K6%" ) ))
( "doorsteps"  n ( d oo1 s t e p s ) ((pos "Kj%" ) ))
( "doorstopper"  n ( d oo1 s t o p @ r ) ((pos "K6%" ) ))
( "doorstoppers"  n ( d oo1 s t o p @ z ) ((pos "Kj%" ) ))
( "doorway"  n ( d oo1 w ei ) ((pos "K6%" ) ))
( "doorways"  n ( d oo1 w ei z ) ((pos "Kj%" ) ))
( "dope"  v ( d ou1 p ) ((pos "H2%" "M6%" ) (subcat )))
( "doped"  v ( d ou1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dopes"  v ( d ou1 p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "dopey"  j ( d ou1 p ii ) ((pos "OA%" ) ))
( "doping"  v ( d ou1 p i ng ) ((pos "Hb%" ) (subcat )))
( "dories"  n ( d oo1 r i z ) ((pos "Kj$" ) ))
( "dormant"  j ( d oo1 m @ n t ) ((pos "OA%" ) ))
( "dormer"  n ( d oo1 m @ r ) ((pos "K6$" ) ))
( "dormer-window"  n ( d oo2 m @ - w i1 n d ou ) ((pos "K6%" ) ))
( "dormer-windows"  n ( d oo2 m @ - w i1 n d ou z ) ((pos "Kj%" ) ))
( "dormers"  n ( d oo1 m @ z ) ((pos "Kj%" ) ))
( "dormice"  n ( d oo1 m ai s ) ((pos "Kj%" ) ))
( "dormitories"  n ( d oo1 m i t r i z ) ((pos "Kj%" ) ))
( "dormitory"  n ( d oo1 m i t r ii ) ((pos "K8%" ) ))
( "dormouse"  n ( d oo1 m au s ) ((pos "Ki%" ) ))
( "dorsal"  j ( d oo1 s l ) ((pos "OA%" ) ))
( "dory"  n ( d oo1 r ii ) ((pos "K8$" ) ))
( "dos"  n ( d uu1 z ) ((pos "Kj$" ) ))
( "dosage"  n ( d ou1 s i jh ) ((pos "M6%" ) ))
( "dosages"  n ( d ou1 s i jh i z ) ((pos "Mj%" ) ))
( "dose"  v ( d ou1 s ) ((pos "H2%" "K6%" ) (subcat "6A" "14" )))
( "dosed"  v ( d ou1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "doses"  v ( d ou1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "dosing"  v ( d ou1 s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "doss"  v ( d o1 s ) ((pos "I1%" ) (subcat )))
( "doss-house"  n ( d o1 s - h au s ) ((pos "K6%" ) ))
( "doss-houses"  n ( d o1 s - h au z i z ) ((pos "Kj%" ) ))
( "dossed"  v ( d o1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "dosser"  n ( d o1 s @ r ) ((pos "K6%" ) ))
( "dossers"  n ( d o1 s @ z ) ((pos "Kj%" ) ))
( "dosses"  v ( d o1 s i z ) ((pos "Ia%" ) (subcat )))
( "dossier"  n ( d o1 s i ei ) ((pos "K6%" ) ))
( "dossiers"  n ( d o1 s i ei z ) ((pos "Kj%" ) ))
( "dossing"  v ( d o1 s i ng ) ((pos "Ib%" ) (subcat )))
( "dost"  v ( d uh s t ) ((pos "Ge$" ) ))
( "dot"  v ( d o1 t ) ((pos "H4%" "K6%" ) ))
( "dotage"  n ( d ou1 t i jh ) ((pos "L@%" ) ))
( "dotard"  n ( d ou1 t @ d ) ((pos "K6$" ) ))
( "dotards"  n ( d ou1 t @ d z ) ((pos "Kj$" ) ))
( "dote"  v ( d ou1 t ) ((pos "I2%" ) (subcat )))
( "doted"  v ( d ou1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "dotes"  v ( d ou1 t s ) ((pos "Ia%" ) (subcat )))
( "doth"  v ( d uh th ) ((pos "Ge$" ) ))
( "doting"  v ( d ou1 t i ng ) ((pos "Ib%" ) (subcat )))
( "dots"  v ( d o1 t s ) ((pos "Ha%" "Kj%" ) ))
( "dotted"  v ( d o1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "dottier"  j ( d o1 t i@ r ) ((pos "Or%" ) ))
( "dottiest"  j ( d o1 t i i s t ) ((pos "Os%" ) ))
( "dotting"  v ( d o1 t i ng ) ((pos "Hb%" ) ))
( "dottle"  n ( d o1 t l ) ((pos "K6$" ) ))
( "dottles"  n ( d o1 t l z ) ((pos "Kj$" ) ))
( "dotty"  j ( d o1 t ii ) ((pos "OD%" ) ))
( "double"  v ( d uh1 b l ) ((pos "J2%" "K6%" "OA%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "double-barrelled"  j ( d uh2 b l - b a1 r @ l d ) ((pos "OA%" ) ))
( "double-bass"  n ( d uh2 b l - b ei1 s ) ((pos "K7%" ) ))
( "double-basses"  n ( d uh2 b l - b ei1 s i z ) ((pos "Kj%" ) ))
( "double-bedded"  j ( d uh2 b l - b e1 d i d ) ((pos "OA%" ) ))
( "double-breasted"  j ( d uh2 b l - b r e1 s t i d ) ((pos "OA%" ) ))
( "double-check"  v ( d uh2 b l - ch e1 k ) ((pos "H0%" ) ))
( "double-checked"  v ( d uh2 b l - ch e1 k t ) ((pos "Hc%" "Hd%" ) ))
( "double-checking"  v ( d uh2 b l - ch e1 k i ng ) ((pos "Hb%" ) ))
( "double-checks"  v ( d uh2 b l - ch e1 k s ) ((pos "Ha%" ) ))
( "double-cross"  v ( d uh2 b l - k r o1 s ) ((pos "H1%" "K7%" ) (subcat )))
( "double-crossed"  v ( d uh2 b l - k r o1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "double-crosses"  v ( d uh2 b l - k r o1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "double-crossing"  v ( d uh2 b l - k r o1 s i ng ) ((pos "Hb%" ) (subcat )))
( "double-dealer"  n ( d uh2 b l - d ii1 l @ r ) ((pos "K6%" ) ))
( "double-dealers"  n ( d uh2 b l - d ii1 l @ z ) ((pos "Kj%" ) ))
( "double-dealing"  n ( d uh2 b l - d ii1 l i ng ) ((pos "L@%" "OA%" ) ))
( "double-decker"  n ( d uh2 b l - d e1 k @ r ) ((pos "K6%" ) ))
( "double-deckers"  n ( d uh2 b l - d e1 k @ z ) ((pos "Kj%" ) ))
( "double-dutch"  n ( d uh2 b l - d uh1 ch ) ((pos "L@%" ) ))
( "double-dyed"  j ( d uh2 b l - d ai1 d ) ((pos "OA$" ) ))
( "double-edged"  j ( d uh2 b l - e1 jh d ) ((pos "OA%" ) ))
( "double-entries"  n ( d uh2 b l - e1 n t r i z ) ((pos "Kj%" ) ))
( "double-entry"  n ( d uh2 b l - e1 n t r ii ) ((pos "K8%" ) ))
( "double-faced"  j ( d uh2 b l - f ei1 s t ) ((pos "OA$" ) ))
( "double-first"  n ( d uh2 b l - f @@1 s t ) ((pos "K6%" ) ))
( "double-firsts"  n ( d uh2 b l - f @@1 s t s ) ((pos "Kj%" ) ))
( "double-jointed"  j ( d uh2 b l - jh oi1 n t i d ) ((pos "OA%" ) ))
( "double-park"  v ( d uh2 b l - p aa1 k ) ((pos "J0%" ) ))
( "double-parked"  v ( d uh2 b l - p aa1 k t ) ((pos "Jc%" "Jd%" ) ))
( "double-parking"  v ( d uh2 b l - p aa1 k i ng ) ((pos "Jb%" ) ))
( "double-parks"  v ( d uh2 b l - p aa1 k s ) ((pos "Ja%" ) ))
( "double-quick"  j ( d uh2 b l - k w i1 k ) ((pos "OA%" "Pu%" ) ))
( "double-spacing"  n ( d uh2 b l - s p ei1 s i ng ) ((pos "L@%" ) ))
( "double-talk"  n ( d uh1 b l - t oo k ) ((pos "L@%" ) ))
( "double-think"  n ( d uh1 b l - th i ng k ) ((pos "L@%" ) ))
( "doubled"  v ( d uh1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "doubles"  v ( d uh1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "doublet"  n ( d uh1 b l i t ) ((pos "K6$" ) ))
( "doublets"  n ( d uh1 b l i t s ) ((pos "Kj$" ) ))
( "doubling"  v ( d uh1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "doubloon"  n ( d uh1 b l uu1 n ) ((pos "K6$" ) ))
( "doubloons"  n ( d uh1 b l uu1 n z ) ((pos "Kj$" ) ))
( "doubly"  a ( d uh1 b l ii ) ((pos "Pu%" ) ))
( "doubt"  v ( d au1 t ) ((pos "H0%" "M6%" ) (subcat "6A" "9" )))
( "doubted"  v ( d au1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "doubtful"  j ( d au1 t f u l ) ((pos "OA%" ) ))
( "doubtfully"  a ( d au1 t f u l ii ) ((pos "Pu%" ) ))
( "doubting"  v ( d au1 t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "doubtless"  a ( d au1 t l i s ) ((pos "Pu%" ) ))
( "doubts"  v ( d au1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "9" )))
( "douche"  n ( d uu1 sh ) ((pos "K6$" ) ))
( "douches"  n ( d uu1 sh i z ) ((pos "Kj$" ) ))
( "dough"  n ( d ou1 ) ((pos "L@%" ) ))
( "doughier"  j ( d ou1 i@ r ) ((pos "Or$" ) ))
( "doughiest"  j ( d ou1 i i s t ) ((pos "Os$" ) ))
( "doughnut"  n ( d ou1 n uh t ) ((pos "K6%" ) ))
( "doughnuts"  n ( d ou1 n uh t s ) ((pos "Kj%" ) ))
( "doughty"  j ( d au1 t ii ) ((pos "OA$" ) ))
( "doughy"  j ( d ou1 ii ) ((pos "OD%" ) ))
( "dour"  j ( d u@1 r ) ((pos "OA%" ) ))
( "dourly"  a ( d u@1 l ii ) ((pos "Pu%" ) ))
( "douse"  v ( d au1 s ) ((pos "H2%" ) (subcat )))
( "doused"  v ( d au1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "douses"  v ( d au1 s i z ) ((pos "Ha%" ) (subcat )))
( "dousing"  v ( d au1 s i ng ) ((pos "Hb%" ) (subcat )))
( "dove"  n ( d uh1 v ) ((pos "K6%" ) ))
( "dovecote"  n ( d uh1 v k @ t ) ((pos "K6%" ) ))
( "dovecotes"  n ( d uh1 v k @ t s ) ((pos "Kj%" ) ))
( "doves"  n ( d uh1 v z ) ((pos "Kj%" ) ))
( "dovetail"  v ( d uh1 v t ei l ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" )))
( "dovetailed"  v ( d uh1 v t ei l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "dovetailing"  v ( d uh1 v t ei l i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "dovetails"  v ( d uh1 v t ei l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "dowager"  n ( d au1 @ jh @ r ) ((pos "K6%" ) ))
( "dowagers"  n ( d au1 @ jh @ z ) ((pos "Kj%" ) ))
( "dowdier"  j ( d au1 d i@ r ) ((pos "Or%" ) ))
( "dowdiest"  j ( d au1 d i i s t ) ((pos "Os%" ) ))
( "dowdily"  a ( d au1 d i l ii ) ((pos "Pu%" ) ))
( "dowdiness"  n ( d au1 d i n @ s ) ((pos "L@%" ) ))
( "dowdy"  j ( d au1 d ii ) ((pos "OD%" ) ))
( "dowel"  n ( d au1 @ l ) ((pos "K6$" ) ))
( "dowels"  n ( d au1 @ l z ) ((pos "Kj$" ) ))
( "dower"  v ( d au1 @ r ) ((pos "H0$" "K6$" ) ))
( "dowered"  v ( d au1 @ d ) ((pos "Hc$" "Hd$" ) ))
( "dowering"  v ( d au1 @ r i ng ) ((pos "Hb$" ) ))
( "dowers"  v ( d au1 @ z ) ((pos "Ha$" "Kj$" ) ))
( "down"  v ( d au n ) ((pos "H0%" "M6%" "P+*" "T-*" ) (subcat )))
( "down-and-out"  n ( d au1 n - @ n - au t ) ((pos "K6%" ) ))
( "down-and-outs"  n ( d au1 n - @ n - au t s ) ((pos "Kj%" ) ))
( "down-market"  j ( d au2 n - m aa1 k i t ) ((pos "OA%" ) ))
( "down-to-earth"  j ( d au2 n - t u - @@1 th ) ((pos "OA%" ) ))
( "downbeat"  n ( d au1 n b ii t ) ((pos "K6%" ) ))
( "downbeats"  n ( d au1 n b ii t s ) ((pos "Kj$" ) ))
( "downcast"  j ( d au1 n k aa s t ) ((pos "OA%" ) ))
( "downed"  v ( d au1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "downfall"  n ( d au1 n f oo l ) ((pos "Ki%" ) ))
( "downgrade"  v ( d au2 n g r ei1 d ) ((pos "H2%" ) (subcat )))
( "downgraded"  v ( d au2 n g r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "downgrades"  v ( d au2 n g r ei1 d z ) ((pos "Ha%" ) (subcat )))
( "downgrading"  v ( d au2 n g r ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "downhearted"  j ( d au2 n h aa1 t i d ) ((pos "OA%" ) ))
( "downhill"  a ( d au2 n h i1 l ) ((pos "Pu%" ) ))
( "downing"  v ( d au1 n i ng ) ((pos "Hb%" ) (subcat )))
( "downpour"  n ( d au1 n p oo r ) ((pos "K6%" ) ))
( "downpours"  n ( d au1 n p oo z ) ((pos "Kj%" ) ))
( "downright"  j ( d au1 n r ai t ) ((pos "OA%" "Pu%" ) ))
( "downrightness"  n ( d au1 n r ai t n @ s ) ((pos "L@$" ) ))
( "downs"  v ( d au1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "downstair"  j ( d au1 n s t e@ r ) ((pos "OA$" ) ))
( "downstairs"  j ( d au2 n s t e@1 z ) ((pos "OA%" "Pu%" ) ))
( "downstream"  j ( d au1 n s t r ii1 m ) ((pos "OA%" "Pu%" ) ))
( "downtown"  a ( d au1 n t au n ) ((pos "Pu%" ) ))
( "downtrodden"  j ( d au1 n t r o d n ) ((pos "OA%" ) ))
( "downward"  j ( d au1 n w @ d ) ((pos "OA%" "Pu%" ) ))
( "downwards"  a ( d au1 n w @ d z ) ((pos "Pu%" ) ))
( "downy"  j ( d au1 n ii ) ((pos "OA%" ) ))
( "dowries"  n ( d au1 @ r i z ) ((pos "Kj%" ) ))
( "dowry"  n ( d au1 @ r ii ) ((pos "K8%" ) ))
( "dowse"  v ( d au1 s ) ((pos "H2$" ) (subcat )))
( "dowsed"  v ( d au1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "dowser"  n ( d au1 s @ r ) ((pos "K6$" ) ))
( "dowsers"  n ( d au1 s @ z ) ((pos "Kj$" ) ))
( "dowses"  v ( d au1 s i z ) ((pos "Ha$" ) (subcat )))
( "dowsing"  v ( d au1 s i ng ) ((pos "Hb$" ) (subcat )))
( "dowsing"  n ( d au1 z i ng ) ((pos "L@$" ) ))
( "doxologies"  n ( d o1 k s o1 l @ jh i z ) ((pos "Kj$" ) ))
( "doxology"  n ( d o1 k s o1 l @ jh ii ) ((pos "K8$" ) ))
( "doyen"  n ( d oi1 @ n ) ((pos "K6%" ) ))
( "doyens"  n ( d oi1 @ n z ) ((pos "Kj$" ) ))
( "doyley"  n ( d oi1 l ii ) ((pos "K6%" ) ))
( "doyleys"  n ( d oi1 l i z ) ((pos "Kj%" ) ))
( "doylies"  n ( d oi1 l i z ) ((pos "Kj%" ) ))
( "doyly"  n ( d oi1 l ii ) ((pos "K8%" ) ))
( "doz"  n ( d uh1 z n ) ((pos "Y]%" ) ))
( "doze"  v ( d ou1 z ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "dozed"  v ( d ou1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "dozen"  n ( d uh1 z n ) ((pos "K6%" ) ))
( "dozens"  n ( d uh1 z n z ) ((pos "Kj%" ) ))
( "dozes"  v ( d ou1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "dozing"  v ( d ou1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "drab"  j ( d r a1 b ) ((pos "OE%" ) ))
( "drabber"  j ( d r a1 b @ r ) ((pos "Or%" ) ))
( "drabbest"  j ( d r a1 b i s t ) ((pos "Os%" ) ))
( "drably"  a ( d r a1 b l ii ) ((pos "Pu%" ) ))
( "drabness"  n ( d r a1 b n @ s ) ((pos "L@%" ) ))
( "drachm"  n ( d r a1 m ) ((pos "K6$" ) ))
( "drachma"  n ( d r a1 k m @ ) ((pos "K6$" ) ))
( "drachmas"  n ( d r a1 k m @ z ) ((pos "Kj$" ) ))
( "drachms"  n ( d r a1 m z ) ((pos "Kj$" ) ))
( "draconian"  j ( d r @1 k ou1 n i@ n ) ((pos "OA$" ) ))
( "draft"  v ( d r aa1 f t ) ((pos "H0%" "K6%" ) (subcat )))
( "drafted"  v ( d r aa1 f t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "draftee"  n ( d r aa2 f t ii1 ) ((pos "K6%" ) ))
( "draftees"  n ( d r aa2 f t ii1 z ) ((pos "Kj%" ) ))
( "drafting"  v ( d r aa1 f t i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "draftings"  n ( d r aa1 f t i ng z ) ((pos "Kj%" ) ))
( "drafts"  v ( d r aa1 f t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "draftsman"  n ( d r aa1 f t s m @ n ) ((pos "Ki%" ) ))
( "draftsmen"  n ( d r aa1 f t s m @ n ) ((pos "Kj%" ) ))
( "drag"  v ( d r a1 g ) ((pos "J4%" "L@%" ) (subcat "2A" "2C" "6A" "14" )))
( "dragged"  v ( d r a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "dragging"  v ( d r a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "draggled"  j ( d r a1 g l d ) ((pos "OA%" ) ))
( "dragnet"  n ( d r a1 g n e t ) ((pos "K6%" ) ))
( "dragnets"  n ( d r a1 g n e t s ) ((pos "Kj%" ) ))
( "dragoman"  n ( d r a1 g ou m @ n ) ((pos "K6$" ) ))
( "dragomans"  n ( d r a1 g ou m @ n z ) ((pos "Kj$" ) ))
( "dragon"  n ( d r a1 g @ n ) ((pos "K6%" ) ))
( "dragonflies"  n ( d r a1 g @ n f l ai z ) ((pos "Kj%" ) ))
( "dragonfly"  n ( d r a1 g @ n f l ai ) ((pos "K8%" ) ))
( "dragons"  n ( d r a1 g @ n z ) ((pos "Kj%" ) ))
( "dragoon"  v ( d r @1 g uu1 n ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "dragooned"  v ( d r @1 g uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dragooning"  v ( d r @1 g uu1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dragoons"  v ( d r @1 g uu1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "drags"  v ( d r a1 g z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" )))
( "drain"  v ( d r ei1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "drainage"  n ( d r ei1 n i jh ) ((pos "L@%" ) ))
( "drainage-basin"  n ( d r ei1 n i jh - b ei s n ) ((pos "K6$" ) ))
( "drainage-basins"  n ( d r ei1 n i jh - b ei s n z ) ((pos "Kj$" ) ))
( "drained"  v ( d r ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "draining"  v ( d r ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "draining-board"  n ( d r ei1 n i ng - b oo d ) ((pos "K6%" ) ))
( "draining-boards"  n ( d r ei1 n i ng - b oo d z ) ((pos "Kj%" ) ))
( "drainpipe"  n ( d r ei1 n p ai p ) ((pos "K6%" ) ))
( "drainpipes"  n ( d r ei1 n p ai p s ) ((pos "Kj%" ) ))
( "drains"  v ( d r ei1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "drake"  n ( d r ei1 k ) ((pos "K6%" ) ))
( "drakes"  n ( d r ei1 k s ) ((pos "Kj%" ) ))
( "dram"  n ( d r a1 m ) ((pos "K6%" ) ))
( "drama"  n ( d r aa1 m @ ) ((pos "M6%" ) ))
( "dramas"  n ( d r aa1 m @ z ) ((pos "Mj%" ) ))
( "dramatic"  j ( d r @1 m a1 t i k ) ((pos "OA%" ) ))
( "dramatically"  a ( d r @1 m a1 t i k l ii ) ((pos "Pu%" ) ))
( "dramatics"  n ( d r @1 m a1 t i k s ) ((pos "Lk%" ) ))
( "dramatis personae"  n ( d r a2 m @ t i s - p @@ s ou1 n ai ) ((pos "Kj%" ) ))
( "dramatist"  n ( d r a1 m @ t i s t ) ((pos "K6%" ) ))
( "dramatists"  n ( d r a1 m @ t i s t s ) ((pos "Kj%" ) ))
( "dramatization"  n ( d r a2 m @ t ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "dramatizations"  n ( d r a2 m @ t ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "dramatize"  v ( d r a1 m @ t ai z ) ((pos "H2%" ) (subcat )))
( "dramatized"  v ( d r a1 m @ t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dramatizes"  v ( d r a1 m @ t ai z i z ) ((pos "Ha%" ) (subcat )))
( "dramatizing"  v ( d r a1 m @ t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "drams"  n ( d r a1 m z ) ((pos "Kj%" ) ))
( "drank"  v ( d r a1 ng k ) ((pos "Jc%" ) (subcat "2A" "3A" "6A" )))
( "drape"  v ( d r ei1 p ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "draped"  v ( d r ei1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "draper"  n ( d r ei1 p @ r ) ((pos "K6%" ) ))
( "draperies"  n ( d r ei1 p @ r i z ) ((pos "Mj%" ) ))
( "drapers"  n ( d r ei1 p @ z ) ((pos "Kj%" ) ))
( "drapery"  n ( d r ei1 p @ r ii ) ((pos "M8%" ) ))
( "drapes"  v ( d r ei1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "draping"  v ( d r ei1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "drastic"  j ( d r a1 s t i k ) ((pos "OA%" ) ))
( "drastically"  a ( d r a1 s t i k l ii ) ((pos "Pu%" ) ))
( "drat"  v ( d r a1 t ) ((pos "H4$" ) ))
( "drats"  v ( d r a1 t s ) ((pos "Ha$" ) ))
( "dratted"  v ( d r a1 t i d ) ((pos "Hc$" "Hd%" ) ))
( "dratting"  v ( d r a1 t i ng ) ((pos "Hb$" ) ))
( "draught"  v ( d r aa1 f t ) ((pos "H0$" "M6%" ) ))
( "draught-horse"  n ( d r aa1 f t - h oo s ) ((pos "K6%" ) ))
( "draught-horses"  n ( d r aa1 f t - h oo s i z ) ((pos "Kj%" ) ))
( "draughted"  v ( d r aa1 f t i d ) ((pos "Hc$" "Hd$" ) ))
( "draughtier"  j ( d r aa1 f t i@ r ) ((pos "Or%" ) ))
( "draughtiest"  j ( d r aa1 f t i i s t ) ((pos "Os%" ) ))
( "draughting"  v ( d r aa1 f t i ng ) ((pos "Hb$" ) ))
( "draughts"  v ( d r aa1 f t s ) ((pos "Ha$" "Lk%" "Mj%" ) ))
( "draughtsman"  n ( d r aa1 f t s m @ n ) ((pos "Ki$" ) ))
( "draughtsmen"  n ( d r aa1 f t s m @ n ) ((pos "Kj$" ) ))
( "draughty"  j ( d r aa1 f t ii ) ((pos "OD%" ) ))
( "draw"  v ( d r oo1 ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "14" )))
( "drawback"  n ( d r oo1 b a k ) ((pos "M6%" ) ))
( "drawbacks"  n ( d r oo1 b a k s ) ((pos "Mj%" ) ))
( "drawbridge"  n ( d r oo1 b r i jh ) ((pos "K6%" ) ))
( "drawbridges"  n ( d r oo1 b r i jh i z ) ((pos "Kj%" ) ))
( "drawer"  n ( d r oo1 r ) ((pos "K6%" ) ))
( "drawers"  n ( d r oo1 z ) ((pos "Kj%" ) ))
( "drawing"  v ( d r oo1 i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "14" )))
( "drawing-board"  n ( d r oo1 i ng - b oo d ) ((pos "K6%" ) ))
( "drawing-boards"  n ( d r oo1 i ng - b oo d z ) ((pos "Kj%" ) ))
( "drawing-pin"  n ( d r oo1 i ng - p i n ) ((pos "K6%" ) ))
( "drawing-pins"  n ( d r oo1 i ng - p i n z ) ((pos "Kj%" ) ))
( "drawing-room"  n ( d r oo1 i ng - r u m ) ((pos "K6%" ) ))
( "drawing-rooms"  n ( d r oo1 i ng - r u m z ) ((pos "Kj%" ) ))
( "drawings"  n ( d r oo1 i ng z ) ((pos "Mj%" ) ))
( "drawl"  v ( d r oo1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "drawled"  v ( d r oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "drawling"  v ( d r oo1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "drawls"  v ( d r oo1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "drawn"  v ( d r oo1 n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "14" )))
( "draws"  v ( d r oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "14" )))
( "dray"  n ( d r ei1 ) ((pos "K6$" ) ))
( "drays"  n ( d r ei1 z ) ((pos "Kj$" ) ))
( "dread"  v ( d r e1 d ) ((pos "J0%" "L@%" ) (subcat "6A" "6C" )))
( "dreaded"  v ( d r e1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "6A" "6C" )))
( "dreadful"  j ( d r e1 d f @ l ) ((pos "OA%" ) ))
( "dreadfully"  a ( d r e1 d f @ l ii ) ((pos "Pu%" ) ))
( "dreadfulness"  n ( d r e1 d f @ l n @ s ) ((pos "L@%" ) ))
( "dreading"  v ( d r e1 d i ng ) ((pos "Jb%" ) (subcat "6A" "6C" )))
( "dreadnought"  n ( d r e1 d n oo t ) ((pos "K6$" ) ))
( "dreadnoughts"  n ( d r e1 d n oo t s ) ((pos "Kj$" ) ))
( "dreads"  v ( d r e1 d z ) ((pos "Ja%" ) (subcat "6A" "6C" )))
( "dream"  v ( d r ii1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "dreamed"  v ( d r ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "dreamer"  n ( d r ii1 m @ r ) ((pos "K6%" ) ))
( "dreamers"  n ( d r ii1 m @ z ) ((pos "Kj%" ) ))
( "dreamier"  j ( d r ii1 m i@ r ) ((pos "Or%" ) ))
( "dreamiest"  j ( d r ii1 m i i s t ) ((pos "Os%" ) ))
( "dreamily"  a ( d r ii1 m i l ii ) ((pos "Pu%" ) ))
( "dreaming"  v ( d r ii1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "dreamland"  n ( d r ii1 m l a n d ) ((pos "K6%" ) ))
( "dreamlands"  n ( d r ii1 m l a n d z ) ((pos "Kj$" ) ))
( "dreamless"  j ( d r ii1 m l @ s ) ((pos "OA%" ) ))
( "dreamlike"  j ( d r ii1 m l ai k ) ((pos "OA%" ) ))
( "dreams"  v ( d r ii1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "dreamt"  v ( d r e1 m t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "dreamworld"  n ( d r ii1 m w @@ l d ) ((pos "K6%" ) ))
( "dreamworlds"  n ( d r ii1 m w @@ l d z ) ((pos "Kj$" ) ))
( "dreamy"  j ( d r ii1 m ii ) ((pos "OD%" ) ))
( "drear"  j ( d r i@1 r ) ((pos "OA%" ) ))
( "drearier"  j ( d r i@1 r i@ r ) ((pos "Or%" ) ))
( "dreariest"  j ( d r i@1 r i i s t ) ((pos "Os%" ) ))
( "drearily"  a ( d r i@1 r i l ii ) ((pos "Pu%" ) ))
( "dreariness"  n ( d r i@1 r i n i s ) ((pos "L@%" ) ))
( "dreary"  j ( d r i@1 r ii ) ((pos "OD%" ) ))
( "dredge"  v ( d r e1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "14" )))
( "dredged"  v ( d r e1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "dredger"  n ( d r e1 jh @ r ) ((pos "K6%" ) ))
( "dredgers"  n ( d r e1 jh @ z ) ((pos "Kj%" ) ))
( "dredges"  v ( d r e1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "14" )))
( "dredging"  v ( d r e1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "dregs"  n ( d r e1 g z ) ((pos "Kj%" ) ))
( "drench"  v ( d r e1 n ch ) ((pos "H1%" ) (subcat )))
( "drenched"  v ( d r e1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "drenches"  v ( d r e1 n ch i z ) ((pos "Ha%" ) (subcat )))
( "drenching"  v ( d r e1 n ch i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "drenchings"  n ( d r e1 n ch i ng z ) ((pos "Kj$" ) ))
( "dress"  v ( d r e1 s ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "6A" )))
( "dress-hanger"  n ( d r e1 s - h a ng @ r ) ((pos "K6$" ) ))
( "dress-hangers"  n ( d r e1 s - h a ng @ z ) ((pos "Kj$" ) ))
( "dressage"  n ( d r e1 s aa zh ) ((pos "L@$" ) ))
( "dressed"  v ( d r e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "dresser"  n ( d r e1 s @ r ) ((pos "K6%" ) ))
( "dressers"  n ( d r e1 s @ z ) ((pos "Kj%" ) ))
( "dresses"  v ( d r e1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "dressier"  j ( d r e1 s i@ r ) ((pos "Or%" ) ))
( "dressiest"  j ( d r e1 s i i s t ) ((pos "Os%" ) ))
( "dressing"  v ( d r e1 s i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "dressing-case"  n ( d r e1 s i ng - k ei s ) ((pos "K6$" ) ))
( "dressing-cases"  n ( d r e1 s i ng - k ei s i z ) ((pos "Kj$" ) ))
( "dressing-down"  n ( d r e2 s i ng - d au1 n ) ((pos "Ki%" ) ))
( "dressing-gown"  n ( d r e1 s i ng - g au n ) ((pos "K6%" ) ))
( "dressing-gowns"  n ( d r e1 s i ng - g au n z ) ((pos "Kj%" ) ))
( "dressing-table"  n ( d r e1 s i ng - t ei b l ) ((pos "K6%" ) ))
( "dressing-tables"  n ( d r e1 s i ng - t ei b l z ) ((pos "Kj%" ) ))
( "dressings"  n ( d r e1 s i ng z ) ((pos "Mj%" ) ))
( "dressmaker"  n ( d r e1 s m ei k @ r ) ((pos "K6%" ) ))
( "dressmakers"  n ( d r e1 s m ei k @ z ) ((pos "Kj%" ) ))
( "dressmaking"  n ( d r e1 s m ei k i ng ) ((pos "L@%" ) ))
( "dressy"  j ( d r e1 s ii ) ((pos "OD%" ) ))
( "drew"  v ( d r uu1 ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "14" )))
( "dribble"  v ( d r i1 b l ) ((pos "J2%" ) (subcat "2A" )))
( "dribbled"  v ( d r i1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "dribbler"  n ( d r i1 b l @ r ) ((pos "K6%" ) ))
( "dribblers"  n ( d r i1 b l @ z ) ((pos "Kj%" ) ))
( "dribbles"  v ( d r i1 b l z ) ((pos "Ja%" ) (subcat "2A" )))
( "dribbling"  v ( d r i1 b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "driblet"  n ( d r i1 b l i t ) ((pos "K6$" ) ))
( "driblets"  n ( d r i1 b l i t s ) ((pos "Kj$" ) ))
( "dribs and drabs"  n ( d r i2 b z - n - d r a1 b z ) ((pos "Kj%" ) ))
( "dried"  v ( d r ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "drier"  n ( d r ai1 @ r ) ((pos "K6%" "Or%" ) ))
( "driers"  n ( d r ai1 @ z ) ((pos "Kj%" ) ))
( "dries"  v ( d r ai1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "driest"  j ( d r ai1 i s t ) ((pos "Os%" ) ))
( "drift"  v ( d r i1 f t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "14" )))
( "drift-ice"  n ( d r i1 f t - ai s ) ((pos "L@%" ) ))
( "drift-net"  n ( d r i1 f t - n e t ) ((pos "K6%" ) ))
( "drift-nets"  n ( d r i1 f t - n e t s ) ((pos "Kj%" ) ))
( "drift-wood"  n ( d r i1 f t - w u d ) ((pos "L@%" ) ))
( "driftage"  n ( d r i1 f t i jh ) ((pos "L@$" ) ))
( "drifted"  v ( d r i1 f t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "drifter"  n ( d r i1 f t @ r ) ((pos "K6%" ) ))
( "drifters"  n ( d r i1 f t @ z ) ((pos "Kj%" ) ))
( "drifting"  v ( d r i1 f t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "drifts"  v ( d r i1 f t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" )))
( "drill"  v ( d r i1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "drilled"  v ( d r i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "drilling"  v ( d r i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "drills"  v ( d r i1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "drily"  a ( d r ai1 l ii ) ((pos "Pu%" ) ))
( "drink"  v ( d r i1 ng k ) ((pos "J5%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "drinkable"  j ( d r i1 ng k @ b l ) ((pos "OA%" ) ))
( "drinker"  n ( d r i1 ng k @ r ) ((pos "K6%" ) ))
( "drinkers"  n ( d r i1 ng k @ z ) ((pos "Kj%" ) ))
( "drinking"  v ( d r i1 ng k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" "6A" )))
( "drinking-bout"  n ( d r i1 ng k i ng - b au t ) ((pos "K6%" ) ))
( "drinking-bouts"  n ( d r i1 ng k i ng - b au t s ) ((pos "Kj%" ) ))
( "drinking-fountain"  n ( d r i1 ng k i ng - f au n t i n ) ((pos "K6%" ) ))
( "drinking-fountains"  n ( d r i1 ng k i ng - f au n t i n z ) ((pos "Kj%" ) ))
( "drinking-song"  n ( d r i1 ng k i ng - s o ng ) ((pos "K6%" ) ))
( "drinking-songs"  n ( d r i1 ng k i ng - s o ng z ) ((pos "Kj%" ) ))
( "drinking-water"  n ( d r i1 ng k i ng - w oo t @ r ) ((pos "L@%" ) ))
( "drinks"  v ( d r i1 ng k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "drip"  v ( d r i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "drip-dried"  v ( d r i1 p - d r ai1 d ) ((pos "Hc%" "Hd%" ) ))
( "drip-dries"  v ( d r i1 p - d r ai1 z ) ((pos "Ha%" ) ))
( "drip-dry"  v ( d r i1 p - d r ai1 ) ((pos "H3%" "OA%" ) ))
( "drip-drying"  v ( d r i1 p - d r ai1 i ng ) ((pos "Hb%" ) ))
( "dripped"  v ( d r i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "dripping"  v ( d r i1 p i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" )))
( "dripping-pan"  n ( d r i1 p i ng - p a n ) ((pos "K6$" ) ))
( "dripping-pans"  n ( d r i1 p i ng - p a n z ) ((pos "Kj$" ) ))
( "drips"  v ( d r i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "drive"  v ( d r ai1 v ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15B" "17" )))
( "drive-in"  n ( d r ai1 v - i n ) ((pos "K6%" ) ))
( "drive-ins"  n ( d r ai1 v - i n z ) ((pos "Kj$" ) ))
( "drivel"  v ( d r i1 v @ l ) ((pos "I4$" "L@%" ) (subcat "2A" )))
( "drivelled"  v ( d r i1 v @ l d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "driveller"  n ( d r i1 v l @ r ) ((pos "K6$" ) ))
( "drivellers"  n ( d r i1 v l @ z ) ((pos "Kj$" ) ))
( "drivelling"  v ( d r i1 v @ l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "drivels"  v ( d r i1 v @ l z ) ((pos "Ia$" ) (subcat "2A" )))
( "driven"  v ( d r i1 v n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15B" "17" )))
( "driver"  n ( d r ai1 v @ r ) ((pos "K6%" ) ))
( "drivers"  n ( d r ai1 v @ z ) ((pos "Kj%" ) ))
( "drives"  v ( d r ai1 v z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15B" "17" )))
( "driveway"  n ( d r ai1 v w ei ) ((pos "K6%" ) ))
( "driveways"  n ( d r ai1 v w ei z ) ((pos "Kj%" ) ))
( "driving"  v ( d r ai1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15B" "17" )))
( "driving-belt"  n ( d r ai1 v i ng - b e l t ) ((pos "K6$" ) ))
( "driving-belts"  n ( d r ai1 v i ng - b e l t s ) ((pos "Kj$" ) ))
( "driving-wheel"  n ( d r ai1 v i ng - w ii l ) ((pos "K6%" ) ))
( "driving-wheels"  n ( d r ai1 v i ng - w ii l z ) ((pos "Kj%" ) ))
( "drizzle"  v ( d r i1 z l ) ((pos "I2%" "L@%" ) (subcat )))
( "drizzled"  v ( d r i1 z l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "drizzles"  v ( d r i1 z l z ) ((pos "Ia%" ) (subcat )))
( "drizzling"  v ( d r i1 z l i ng ) ((pos "Ib%" ) (subcat )))
( "drizzly"  j ( d r i1 z l ii ) ((pos "OA%" ) ))
( "drogue"  n ( d r ou1 g ) ((pos "K6$" ) ))
( "drogues"  n ( d r ou1 g z ) ((pos "Kj$" ) ))
( "droll"  j ( d r ou1 l ) ((pos "OA%" ) ))
( "drolleries"  n ( d r o1 l @ r i z ) ((pos "Mj$" ) ))
( "drollery"  n ( d r o1 l @ r ii ) ((pos "M8$" ) ))
( "dromedaries"  n ( d r o1 m @ d @ r i z ) ((pos "Kj$" ) ))
( "dromedary"  n ( d r o1 m @ d @ r ii ) ((pos "K8$" ) ))
( "drone"  v ( d r ou1 n ) ((pos "J2%" "M6%" ) (subcat "2C" )))
( "droned"  v ( d r ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "drones"  v ( d r ou1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "droning"  v ( d r ou1 n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "drool"  v ( d r uu1 l ) ((pos "I0%" ) ))
( "drooled"  v ( d r uu1 l d ) ((pos "Ic%" "Id%" ) ))
( "drooling"  v ( d r uu1 l i ng ) ((pos "Ib%" ) ))
( "drools"  v ( d r uu1 l z ) ((pos "Ia%" ) ))
( "droop"  v ( d r uu1 p ) ((pos "J0%" "Ki%" ) (subcat "2A" "2C" )))
( "drooped"  v ( d r uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "drooping"  v ( d r uu1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "droopingly"  a ( d r uu1 p i ng l ii ) ((pos "Pu%" ) ))
( "droops"  v ( d r uu1 p s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "drop"  v ( d r o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3B" "6A" "12A" "13A" "14" "15A" )))
( "drop-curtain"  n ( d r o1 p - k @@ t n ) ((pos "K6$" ) ))
( "drop-curtains"  n ( d r o1 p - k @@ t n z ) ((pos "Kj$" ) ))
( "drop-kick"  n ( d r o1 p - k i k ) ((pos "K6%" ) ))
( "drop-kicks"  n ( d r o1 p - k i k s ) ((pos "Kj%" ) ))
( "dropout"  n ( d r o1 p au t ) ((pos "K6%" ) ))
( "dropouts"  n ( d r o1 p au t s ) ((pos "Kj%" ) ))
( "dropped"  v ( d r o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3B" "6A" "12A" "13A" "14" "15A" )))
( "dropping"  v ( d r o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3B" "6A" "12A" "13A" "14" "15A" )))
( "dropping-zone"  n ( d r o1 p i ng - z ou n ) ((pos "K6$" ) ))
( "dropping-zones"  n ( d r o1 p i ng - z ou n z ) ((pos "Kj$" ) ))
( "droppings"  n ( d r o1 p i ng z ) ((pos "Kj%" ) ))
( "drops"  v ( d r o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3B" "6A" "12A" "13A" "14" "15A" )))
( "dropsical"  j ( d r o1 p s i k l ) ((pos "OA$" ) ))
( "dropsy"  n ( d r o1 p s ii ) ((pos "L@$" ) ))
( "droshkies"  n ( d r o1 sh k i z ) ((pos "Kj$" ) ))
( "droshky"  n ( d r o1 sh k ii ) ((pos "K8$" ) ))
( "dross"  n ( d r o1 s ) ((pos "L@$" ) ))
( "drought"  n ( d r au1 t ) ((pos "M6%" ) ))
( "droughts"  n ( d r au1 t s ) ((pos "Mj%" ) ))
( "drove"  v ( d r ou1 v ) ((pos "Jc%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15B" "17" )))
( "drover"  n ( d r ou1 v @ r ) ((pos "K6%" ) ))
( "drovers"  n ( d r ou1 v @ z ) ((pos "Kj%" ) ))
( "droves"  n ( d r ou1 v z ) ((pos "Kj%" ) ))
( "drown"  v ( d r au1 n ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "drowned"  v ( d r au1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "drowning"  v ( d r au1 n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "drowns"  v ( d r au1 n z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "drowse"  v ( d r au1 z ) ((pos "J2%" "Ki$" ) (subcat "2A" "2C" )))
( "drowsed"  v ( d r au1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "drowses"  v ( d r au1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "drowsier"  j ( d r au1 z i@ r ) ((pos "Or%" ) ))
( "drowsiest"  j ( d r au1 z i i s t ) ((pos "Os%" ) ))
( "drowsily"  a ( d r au1 z i l ii ) ((pos "Pu%" ) ))
( "drowsiness"  n ( d r au1 z i n @ s ) ((pos "L@%" ) ))
( "drowsing"  v ( d r au1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "drowsy"  j ( d r au1 z ii ) ((pos "OD%" ) ))
( "drub"  v ( d r uh1 b ) ((pos "H4$" ) (subcat "6A" )))
( "drubbed"  v ( d r uh1 b d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "drubbing"  v ( d r uh1 b i ng ) ((pos "Hb$" "Ki%" ) (subcat "6A" )))
( "drubs"  v ( d r uh1 b z ) ((pos "Ha$" ) (subcat "6A" )))
( "drudge"  v ( d r uh1 jh ) ((pos "I2$" "K6%" ) (subcat "2A" "2C" )))
( "drudged"  v ( d r uh1 jh d ) ((pos "Ic$" "Id$" ) (subcat "2A" "2C" )))
( "drudgery"  n ( d r uh1 jh @ r ii ) ((pos "L@%" ) ))
( "drudges"  v ( d r uh1 jh i z ) ((pos "Ia$" "Kj%" ) (subcat "2A" "2C" )))
( "drudging"  v ( d r uh1 jh i ng ) ((pos "Ib$" ) (subcat "2A" "2C" )))
( "drug"  v ( d r uh1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "drugged"  v ( d r uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "drugget"  n ( d r uh1 g i t ) ((pos "M6$" ) ))
( "druggets"  n ( d r uh1 g i t s ) ((pos "Mj$" ) ))
( "drugging"  v ( d r uh1 g i ng ) ((pos "Hb%" ) (subcat )))
( "druggist"  n ( d r uh1 g i s t ) ((pos "K6%" ) ))
( "druggists"  n ( d r uh1 g i s t s ) ((pos "Kj%" ) ))
( "drugs"  v ( d r uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "drugstore"  n ( d r uh1 g s t oo r ) ((pos "K6%" ) ))
( "drugstores"  n ( d r uh1 g s t oo z ) ((pos "Kj%" ) ))
( "druid"  n ( d r uu1 i d ) ((pos "K6%" ) ))
( "druids"  n ( d r uu1 i d z ) ((pos "Kj%" ) ))
( "drum"  v ( d r uh1 m ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "drum-major"  n ( d r uh1 m - m ei1 jh @ r ) ((pos "K6%" ) ))
( "drum-majorette"  n ( d r uh2 m - m ei2 jh @ r e1 t ) ((pos "K6%" ) ))
( "drum-majorettes"  n ( d r uh2 m - m ei2 jh @ r e1 t s ) ((pos "Kj%" ) ))
( "drum-majors"  n ( d r uh1 m - m ei1 jh @ z ) ((pos "Kj%" ) ))
( "drumfire"  n ( d r uh1 m f ai @ r ) ((pos "L@$" ) ))
( "drumhead"  j ( d r uh1 m h e d ) ((pos "OA$" ) ))
( "drummed"  v ( d r uh1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "drummer"  n ( d r uh1 m @ r ) ((pos "K6%" ) ))
( "drummers"  n ( d r uh1 m @ z ) ((pos "Kj%" ) ))
( "drumming"  v ( d r uh1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "drums"  v ( d r uh1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "drumstick"  n ( d r uh1 m s t i k ) ((pos "K6%" ) ))
( "drumsticks"  n ( d r uh1 m s t i k s ) ((pos "Kj%" ) ))
( "drunk"  v ( d r uh1 ng k ) ((pos "Jd%" "K6%" "OC%" ) (subcat "2A" "3A" "6A" )))
( "drunkard"  n ( d r uh1 n k @ d ) ((pos "K6%" ) ))
( "drunkards"  n ( d r uh1 n k @ d z ) ((pos "Kj%" ) ))
( "drunken"  j ( d r uh1 ng k @ n ) ((pos "OA%" ) ))
( "drunkenly"  a ( d r uh1 ng k @ n l ii ) ((pos "Pu%" ) ))
( "drunkenness"  n ( d r uh1 ng k @ n - n @ s ) ((pos "L@%" ) ))
( "drunker"  j ( d r uh1 ng k @ r ) ((pos "Or%" ) ))
( "drunkest"  j ( d r uh1 ng k i s t ) ((pos "Os%" ) ))
( "drunks"  n ( d r uh1 ng k s ) ((pos "Kj%" ) ))
( "drupe"  n ( d r uu1 p ) ((pos "K6$" ) ))
( "drupes"  n ( d r uu1 p s ) ((pos "Kj$" ) ))
( "dry"  v ( d r ai1 ) ((pos "J3%" "OD%" ) (subcat "2A" "2C" "6A" )))
( "dry-clean"  v ( d r ai1 - k l ii1 n ) ((pos "H0%" ) ))
( "dry-cleaned"  v ( d r ai1 - k l ii1 n d ) ((pos "Hc%" "Hd%" ) ))
( "dry-cleaner"  n ( d r ai1 - k l ii1 n @ r ) ((pos "K6%" ) ))
( "dry-cleaners"  n ( d r ai1 - k l ii1 n @ z ) ((pos "Kj%" ) ))
( "dry-cleaning"  v ( d r ai1 - k l ii1 n i ng ) ((pos "Hb%" "L@%" ) ))
( "dry-cleans"  v ( d r ai1 - k l ii1 n z ) ((pos "Ha%" ) ))
( "dry-shod"  j ( d r ai1 - sh o1 d ) ((pos "OA$" ) ))
( "dry-walling"  n ( d r ai1 - w oo1 l i ng ) ((pos "L@$" ) ))
( "dryad"  n ( d r ai1 @ d ) ((pos "K6$" ) ))
( "dryads"  n ( d r ai1 @ d z ) ((pos "Kj$" ) ))
( "dryer"  n ( d r ai1 @ r ) ((pos "K6%" ) ))
( "dryers"  n ( d r ai1 @ z ) ((pos "Kj%" ) ))
( "drying"  v ( d r ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "dryness"  n ( d r ai1 n @ s ) ((pos "L@%" ) ))
( "dual"  j ( d y uu1 @ l ) ((pos "OA%" ) ))
( "dub"  v ( d uh1 b ) ((pos "H4%" ) (subcat "6A" "22" )))
( "dubbed"  v ( d uh1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "22" )))
( "dubbin"  n ( d uh1 b i n ) ((pos "L@%" ) ))
( "dubbing"  v ( d uh1 b i ng ) ((pos "Hb%" ) (subcat "6A" "22" )))
( "dubieties"  n ( d y uu1 b ai1 @ t i z ) ((pos "Mj$" ) ))
( "dubiety"  n ( d y uu1 b ai1 @ t ii ) ((pos "M8$" ) ))
( "dubious"  j ( d y uu1 b i@ s ) ((pos "OA%" ) ))
( "dubiously"  a ( d y uu1 b i@ s l ii ) ((pos "Pu%" ) ))
( "dubiousness"  n ( d y uu1 b i@ s n @ s ) ((pos "L@%" ) ))
( "dubs"  v ( d uh1 b z ) ((pos "Ha%" ) (subcat "6A" "22" )))
( "ducal"  j ( d y uu1 k l ) ((pos "OA$" ) ))
( "ducat"  n ( d uh1 k @ t ) ((pos "K6$" ) ))
( "ducats"  n ( d uh1 k @ t s ) ((pos "Kj$" ) ))
( "duchess"  n ( d uh1 ch i s ) ((pos "K7%" ) ))
( "duchesses"  n ( d uh1 ch i s i z ) ((pos "Kj%" ) ))
( "duchies"  n ( d uh1 ch i z ) ((pos "Kj$" ) ))
( "duchy"  n ( d uh1 ch ii ) ((pos "K8%" ) ))
( "duck"  v ( d uh1 k ) ((pos "J0%" "M9%" ) (subcat "2A" )))
( "duckbilled"  j ( d uh1 k b i l d ) ((pos "OA%" ) ))
( "duckboards"  n ( d uh1 k b oo d z ) ((pos "Kj%" ) ))
( "ducked"  v ( d uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "duckies"  n ( d uh1 k i z ) ((pos "Kj$" ) ))
( "ducking"  v ( d uh1 k i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" )))
( "ducking-stool"  n ( d uh1 k i ng - s t uu l ) ((pos "K6%" ) ))
( "ducking-stools"  n ( d uh1 k i ng - s t uu l z ) ((pos "Kj%" ) ))
( "duckings"  n ( d uh1 k i ng z ) ((pos "Kj%" ) ))
( "duckling"  n ( d uh1 k l i ng ) ((pos "K6%" ) ))
( "ducklings"  n ( d uh1 k l i ng z ) ((pos "Kj%" ) ))
( "ducks"  v ( d uh1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "duckweed"  n ( d uh1 k w ii d ) ((pos "L@%" ) ))
( "ducky"  n ( d uh1 k ii ) ((pos "K8%" ) ))
( "duct"  n ( d uh1 k t ) ((pos "K6%" ) ))
( "ductile"  j ( d uh1 k t ai l ) ((pos "OA$" ) ))
( "ductility"  n ( d uh1 k t i1 l i t ii ) ((pos "L@$" ) ))
( "ducts"  n ( d uh1 k t s ) ((pos "Kj%" ) ))
( "dud"  n ( d uh1 d ) ((pos "K6%" "OA%" ) ))
( "dude"  n ( d y uu1 d ) ((pos "K6$" ) ))
( "dudes"  n ( d y uu1 d z ) ((pos "Kj$" ) ))
( "dudgeon"  n ( d uh1 jh @ n ) ((pos "L@$" ) ))
( "duds"  n ( d uh1 d z ) ((pos "Kj%" ) ))
( "due"  n ( d y uu1 ) ((pos "K6%" "OA%" "Pu%" ) ))
( "duel"  v ( d y uu1 @ l ) ((pos "I4%" "K6%" ) ))
( "duelist"  n ( d y uu1 @ l i s t ) ((pos "K6$" ) ))
( "duelists"  n ( d y uu1 @ l i s t s ) ((pos "Kj$" ) ))
( "duelled"  v ( d y uu1 @ l d ) ((pos "Ic%" "Id%" ) ))
( "duelling"  v ( d y uu1 @ l i ng ) ((pos "Ib%" ) ))
( "duellist"  n ( d y uu1 @ l i s t ) ((pos "K6$" ) ))
( "duellists"  n ( d y uu1 @ l i s t s ) ((pos "Kj$" ) ))
( "duels"  v ( d y uu1 @ l z ) ((pos "Ia%" "Kj%" ) ))
( "duenna"  n ( d y uu1 e1 n @ ) ((pos "K6$" ) ))
( "duennas"  n ( d y uu1 e1 n @ z ) ((pos "Kj$" ) ))
( "dues"  n ( d y uu1 z ) ((pos "Kj%" ) ))
( "duet"  n ( d y uu1 e1 t ) ((pos "K6%" ) ))
( "duets"  n ( d y uu1 e1 t s ) ((pos "Kj%" ) ))
( "duffel"  n ( d uh1 f l ) ((pos "L@%" ) ))
( "duffer"  n ( d uh1 f @ r ) ((pos "K6%" ) ))
( "duffers"  n ( d uh1 f @ z ) ((pos "Kj%" ) ))
( "duffle"  n ( d uh1 f l ) ((pos "L@%" ) ))
( "dug"  v ( d uh1 g ) ((pos "Jc%" "Jd%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "dugong"  n ( d uu1 g o ng ) ((pos "K6$" ) ))
( "dugongs"  n ( d uu1 g o ng z ) ((pos "Kj$" ) ))
( "dugout"  n ( d uh1 g au t ) ((pos "K6%" ) ))
( "dugouts"  n ( d uh1 g au t s ) ((pos "Kj%" ) ))
( "dugs"  n ( d uh1 g z ) ((pos "Kj$" ) ))
( "duke"  n ( d y uu1 k ) ((pos "K6%" ) ))
( "dukedom"  n ( d y uu1 k d @ m ) ((pos "K6%" ) ))
( "dukedoms"  n ( d y uu1 k d @ m z ) ((pos "Kj%" ) ))
( "dukes"  n ( d y uu1 k s ) ((pos "Kj%" ) ))
( "dulcet"  j ( d uh1 l s i t ) ((pos "OA%" ) ))
( "dulcimer"  n ( d uh1 l s i m @ r ) ((pos "K6%" ) ))
( "dulcimers"  n ( d uh1 l s i m @ z ) ((pos "Kj$" ) ))
( "dull"  v ( d uh1 l ) ((pos "J0%" "OC%" ) (subcat "2A" )))
( "dullard"  n ( d uh1 l @ d ) ((pos "K6%" ) ))
( "dullards"  n ( d uh1 l @ d z ) ((pos "Kj%" ) ))
( "dulled"  v ( d uh1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "duller"  j ( d uh1 l @ r ) ((pos "Or%" ) ))
( "dullest"  j ( d uh1 l i s t ) ((pos "Os%" ) ))
( "dulling"  v ( d uh1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "dullness"  n ( d uh1 l n @ s ) ((pos "L@%" ) ))
( "dulls"  v ( d uh1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "dully"  a ( d uh1 l - l ii ) ((pos "Pu%" ) ))
( "duly"  a ( d y uu1 l ii ) ((pos "Pu%" ) ))
( "dumb"  j ( d uh1 m ) ((pos "OC%" ) ))
( "dumbbell"  n ( d uh1 m b e l ) ((pos "K6%" ) ))
( "dumbbells"  n ( d uh1 m b e l z ) ((pos "Kj%" ) ))
( "dumber"  j ( d uh1 m @ r ) ((pos "Or%" ) ))
( "dumbest"  j ( d uh1 m i s t ) ((pos "Os%" ) ))
( "dumbfound"  v ( d uh1 m f au1 n d ) ((pos "H0%" ) (subcat )))
( "dumbfounded"  v ( d uh1 m f au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dumbfounding"  v ( d uh1 m f au1 n d i ng ) ((pos "Hb$" ) (subcat )))
( "dumbfounds"  v ( d uh1 m f au1 n d z ) ((pos "Ha%" ) (subcat )))
( "dumbly"  a ( d uh1 m l ii ) ((pos "Pu%" ) ))
( "dumbness"  n ( d uh1 m n @ s ) ((pos "L@%" ) ))
( "dumbwaiter"  n ( d uh2 m w ei1 t @ r ) ((pos "K6%" ) ))
( "dumbwaiters"  n ( d uh2 m w ei1 t @ z ) ((pos "Kj%" ) ))
( "dumdum"  n ( d uh1 m d uh m ) ((pos "K6$" ) ))
( "dumdums"  n ( d uh1 m d uh m z ) ((pos "Kj$" ) ))
( "dummies"  n ( d uh1 m i z ) ((pos "Kj%" ) ))
( "dummy"  n ( d uh1 m ii ) ((pos "K8%" ) ))
( "dump"  v ( d uh1 m p ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "dumped"  v ( d uh1 m p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dumper"  n ( d uh1 m p @ r ) ((pos "K6%" ) ))
( "dumpers"  n ( d uh1 m p @ z ) ((pos "Kj%" ) ))
( "dumpier"  j ( d uh1 m p i@ r ) ((pos "Or$" ) ))
( "dumpiest"  j ( d uh1 m p i i s t ) ((pos "Os$" ) ))
( "dumping"  v ( d uh1 m p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dumpling"  n ( d uh1 m p l i ng ) ((pos "K6%" ) ))
( "dumplings"  n ( d uh1 m p l i ng z ) ((pos "Kj%" ) ))
( "dumps"  v ( d uh1 m p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "dumpy"  j ( d uh1 m p ii ) ((pos "OD%" ) ))
( "dun"  v ( d uh1 n ) ((pos "H4$" "K6$" "OA%" ) ))
( "dunce"  n ( d uh1 n s ) ((pos "K6%" ) ))
( "dunces"  n ( d uh1 n s i z ) ((pos "Kj%" ) ))
( "dunderhead"  n ( d uh1 n d @ h e d ) ((pos "K6$" ) ))
( "dunderheads"  n ( d uh1 n d @ h e d z ) ((pos "Kj$" ) ))
( "dune"  n ( d y uu1 n ) ((pos "K6%" ) ))
( "dunes"  n ( d y uu1 n z ) ((pos "Kj%" ) ))
( "dung"  n ( d uh1 ng ) ((pos "L@%" ) ))
( "dungarees"  n ( d uh2 ng g @ r ii1 z ) ((pos "Kj%" ) ))
( "dungeon"  n ( d uh1 n jh @ n ) ((pos "K6%" ) ))
( "dungeons"  n ( d uh1 n jh @ n z ) ((pos "Kj%" ) ))
( "dunghill"  n ( d uh1 ng h i l ) ((pos "K6%" ) ))
( "dunghills"  n ( d uh1 ng h i l z ) ((pos "Kj%" ) ))
( "dunk"  v ( d uh1 ng k ) ((pos "H0%" ) (subcat "6A" )))
( "dunked"  v ( d uh1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "dunking"  v ( d uh1 ng k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "dunks"  v ( d uh1 ng k s ) ((pos "Ha%" ) (subcat "6A" )))
( "dunned"  v ( d uh1 n d ) ((pos "Hc$" "Hd$" ) ))
( "dunning"  v ( d uh1 n i ng ) ((pos "Hb$" ) ))
( "duns"  v ( d uh1 n z ) ((pos "Ha$" "Kj$" ) ))
( "duodecimal"  j ( d y uu2 ou d e1 s i m l ) ((pos "OA$" ) ))
( "duodenal"  j ( d y uu2 @ d ii1 n l ) ((pos "OA$" ) ))
( "duodenum"  n ( d y uu2 @ d ii1 n @ m ) ((pos "K6$" ) ))
( "duodenums"  n ( d y uu2 @ d ii1 n @ m z ) ((pos "Kj$" ) ))
( "duologue"  n ( d y uu1 @ l o g ) ((pos "K6$" ) ))
( "duologues"  n ( d y uu1 @ l o g z ) ((pos "Kj$" ) ))
( "dupe"  v ( d y uu1 p ) ((pos "H2%" "K6%" ) (subcat )))
( "duped"  v ( d y uu1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dupes"  v ( d y uu1 p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "duping"  v ( d y uu1 p i ng ) ((pos "Hb%" ) (subcat )))
( "dupl"  n ( d y uu1 p l i k @ t ) ((pos "Y~%" ) ))
( "duplex"  j ( d y uu1 p l e k s ) ((pos "OA%" ) ))
( "duplicate"  n ( d y uu1 p l i k @ t ) ((pos "K6%" "OA%" ) ))
( "duplicate"  v ( d y uu1 p l i k ei t ) ((pos "H2%" ) (subcat )))
( "duplicated"  v ( d y uu1 p l i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "duplicates"  n ( d y uu1 p l i k @ t s ) ((pos "Kj%" ) ))
( "duplicates"  v ( d y uu1 p l i k ei t s ) ((pos "Ha%" ) (subcat )))
( "duplicating"  v ( d y uu1 p l i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "duplication"  n ( d y uu2 p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "duplications"  n ( d y uu2 p l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "duplicator"  n ( d y uu1 p l i k ei t @ r ) ((pos "K6%" ) ))
( "duplicators"  n ( d y uu1 p l i k ei t @ z ) ((pos "Kj%" ) ))
( "duplicity"  n ( d y uu1 p l i1 s i t ii ) ((pos "L@%" ) ))
( "durability"  n ( d y u@2 r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "durable"  n ( d y u@1 r @ b l ) ((pos "K6%" "OA%" ) ))
( "durables"  n ( d y u@1 r @ b l z ) ((pos "Kj%" ) ))
( "durance"  n ( d y u@1 r @ n s ) ((pos "L@$" ) ))
( "duration"  n ( d y u1 r ei1 sh @ n ) ((pos "M6%" ) ))
( "durations"  n ( d y u1 r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "durbar"  n ( d @@1 b aa r ) ((pos "K6$" ) ))
( "durbars"  n ( d @@1 b aa z ) ((pos "Kj$" ) ))
( "duress"  n ( d y u1 r e1 s ) ((pos "L@%" ) ))
( "during"  in ( d y u@1 r i ng ) ((pos "T-*" ) ))
( "durst"  v ( d @@ s t ) ((pos "Gc$" ) (subcat )))
( "dusk"  n ( d uh1 s k ) ((pos "L@%" ) ))
( "duskier"  j ( d uh1 s k i@ r ) ((pos "Or$" ) ))
( "duskiest"  j ( d uh1 s k i i s t ) ((pos "Os$" ) ))
( "dusky"  j ( d uh1 s k ii ) ((pos "OD%" ) ))
( "dust"  v ( d uh1 s t ) ((pos "H0%" "L@%" ) (subcat )))
( "dust-bowl"  n ( d uh1 s t - b ou l ) ((pos "K6%" ) ))
( "dust-bowls"  n ( d uh1 s t - b ou l z ) ((pos "Kj%" ) ))
( "dust-coat"  n ( d uh1 s t - k ou t ) ((pos "K6%" ) ))
( "dust-coats"  n ( d uh1 s t - k ou t s ) ((pos "Kj%" ) ))
( "dust-jacket"  n ( d uh1 s t - jh a k i t ) ((pos "K6%" ) ))
( "dust-jackets"  n ( d uh1 s t - jh a k i t s ) ((pos "Kj%" ) ))
( "dust-sheet"  n ( d uh1 s t - sh ii t ) ((pos "K6%" ) ))
( "dust-sheets"  n ( d uh1 s t - sh ii t s ) ((pos "Kj%" ) ))
( "dust-up"  n ( d uh1 s t - uh p ) ((pos "K6%" ) ))
( "dust-ups"  n ( d uh1 s t - uh p s ) ((pos "Kj%" ) ))
( "dust-wrapper"  n ( d uh1 s t - r a p @ r ) ((pos "K6%" ) ))
( "dust-wrappers"  n ( d uh1 s t - r a p @ z ) ((pos "Kj%" ) ))
( "dustbin"  n ( d uh1 s t b i n ) ((pos "K6%" ) ))
( "dustbins"  n ( d uh1 s t b i n z ) ((pos "Kj%" ) ))
( "dustcart"  n ( d uh1 s t k aa t ) ((pos "K6%" ) ))
( "dustcarts"  n ( d uh1 s t k aa t s ) ((pos "Kj%" ) ))
( "dusted"  v ( d uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "duster"  n ( d uh1 s t @ r ) ((pos "K6%" ) ))
( "dusters"  n ( d uh1 s t @ z ) ((pos "Kj%" ) ))
( "dustier"  j ( d uh1 s t i@ r ) ((pos "Or%" ) ))
( "dustiest"  j ( d uh1 s t i i s t ) ((pos "Os%" ) ))
( "dusting"  v ( d uh1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "dustman"  n ( d uh1 s t m @ n ) ((pos "Ki%" ) ))
( "dustmen"  n ( d uh1 s t m @ n ) ((pos "Kj%" ) ))
( "dustpan"  n ( d uh1 s t p a n ) ((pos "K6%" ) ))
( "dustpans"  n ( d uh1 s t p a n z ) ((pos "Kj%" ) ))
( "dusts"  v ( d uh1 s t s ) ((pos "Ha%" ) (subcat )))
( "dusty"  j ( d uh1 s t ii ) ((pos "OD%" ) ))
( "duteous"  j ( d y uu1 t i@ s ) ((pos "OA$" ) ))
( "dutiable"  j ( d y uu1 t i@ b l ) ((pos "OA$" ) ))
( "duties"  n ( d y uu1 t i z ) ((pos "Mj%" ) ))
( "dutiful"  j ( d y uu1 t i f @ l ) ((pos "OA%" ) ))
( "dutifully"  a ( d y uu1 t i f @ l ii ) ((pos "Pu%" ) ))
( "duty"  n ( d y uu1 t ii ) ((pos "M8%" ) ))
( "duty-free"  j ( d y uu2 t i - f r ii1 ) ((pos "OA%" ) ))
( "duvet"  n ( d y uu1 v ei ) ((pos "K6%" ) ))
( "duvets"  n ( d y uu1 v ei z ) ((pos "Kj%" ) ))
( "dwarf"  v ( d w oo1 f ) ((pos "H0%" "K6%" ) (subcat )))
( "dwarfed"  v ( d w oo1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dwarfing"  v ( d w oo1 f i ng ) ((pos "Hb%" ) (subcat )))
( "dwarfish"  j ( d w oo1 f i sh ) ((pos "OA%" ) ))
( "dwarfs"  v ( d w oo1 f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "dwell"  v ( d w e1 l ) ((pos "I5%" ) (subcat )))
( "dweller"  n ( d w e1 l @ r ) ((pos "K6%" ) ))
( "dwellers"  n ( d w e1 l @ z ) ((pos "Kj%" ) ))
( "dwelling"  v ( d w e1 l i ng ) ((pos "Ib%" "K6%" ) (subcat )))
( "dwelling-house"  n ( d w e1 l i ng - h au s ) ((pos "K6%" ) ))
( "dwelling-houses"  n ( d w e1 l i ng - h au z i z ) ((pos "Kj%" ) ))
( "dwellings"  n ( d w e1 l i ng z ) ((pos "Kj%" ) ))
( "dwells"  v ( d w e1 l z ) ((pos "Ia%" ) (subcat )))
( "dwelt"  v ( d w e1 l t ) ((pos "Ic%" ) (subcat )))
( "dwindle"  v ( d w i1 n d l ) ((pos "I2%" ) (subcat )))
( "dwindled"  v ( d w i1 n d l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "dwindles"  v ( d w i1 n d l z ) ((pos "Ia%" ) (subcat )))
( "dwindling"  v ( d w i1 n d l i ng ) ((pos "Ib%" ) (subcat )))
( "dyarchies"  n ( d ai1 aa k i z ) ((pos "Kj$" ) ))
( "dyarchy"  n ( d ai1 aa k ii ) ((pos "K8$" ) ))
( "dye"  v ( d ai1 ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" )))
( "dye-works"  n ( d ai1 - w @@ k s ) ((pos "K9$" ) ))
( "dyed"  v ( d ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "dyed-in-the-wool"  j ( d ai2 d - i n - dh @ - w u1 l ) ((pos "OA%" ) ))
( "dyer"  n ( d ai1 @ r ) ((pos "K6$" ) ))
( "dyers"  n ( d ai1 @ z ) ((pos "Kj$" ) ))
( "dyes"  v ( d ai1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "dyestuff"  n ( d ai1 s t uh f ) ((pos "K6%" ) ))
( "dyestuffs"  n ( d ai1 s t uh f s ) ((pos "Kj%" ) ))
( "dying"  v ( d ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "4C" "6A" )))
( "dyke"  v ( d ai1 k ) ((pos "J2$" "K6%" ) ))
( "dyked"  v ( d ai1 k t ) ((pos "Jc$" "Jd$" ) ))
( "dykes"  v ( d ai1 k s ) ((pos "Ja$" "Kj%" ) ))
( "dyking"  v ( d ai1 k i ng ) ((pos "Jb$" ) ))
( "dynamic"  n ( d ai1 n a1 m i k ) ((pos "K6%" "OA%" ) ))
( "dynamically"  a ( d ai1 n a1 m i k l ii ) ((pos "Pu%" ) ))
( "dynamics"  n ( d ai1 n a1 m i k s ) ((pos "Kj%" ) ))
( "dynamism"  n ( d ai1 n @ m i z @ m ) ((pos "L@%" ) ))
( "dynamite"  v ( d ai1 n @ m ai t ) ((pos "H2%" "L@%" ) (subcat )))
( "dynamited"  v ( d ai1 n @ m ai t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "dynamites"  v ( d ai1 n @ m ai t s ) ((pos "Ha%" ) (subcat )))
( "dynamiting"  v ( d ai1 n @ m ai t i ng ) ((pos "Hb%" ) (subcat )))
( "dynamo"  n ( d ai1 n @ m ou ) ((pos "K6%" ) ))
( "dynamos"  n ( d ai1 n @ m ou z ) ((pos "Kj%" ) ))
( "dynast"  n ( d i1 n @ s t ) ((pos "K6$" ) ))
( "dynastic"  j ( d i1 n a1 s t i k ) ((pos "OA%" ) ))
( "dynasties"  n ( d i1 n @ s t i z ) ((pos "Kj%" ) ))
( "dynasts"  n ( d i1 n @ s t s ) ((pos "Kj$" ) ))
( "dynasty"  n ( d i1 n @ s t ii ) ((pos "K8%" ) ))
( "dyne"  n ( d ai1 n ) ((pos "K6$" ) ))
( "dynes"  n ( d ai1 n z ) ((pos "Kj$" ) ))
( "dysentery"  n ( d i1 s @ n t r ii ) ((pos "L@%" ) ))
( "dyslexia"  n ( d i1 s l e1 k s i@ ) ((pos "L@%" ) ))
( "dyslexic"  j ( d i1 s l e1 k s i k ) ((pos "OA%" ) ))
( "dyspepsia"  n ( d i1 s p e1 p s i@ ) ((pos "L@%" ) ))
( "dyspeptic"  n ( d i1 s p e1 p t i k ) ((pos "K6%" "OA%" ) ))
( "dyspeptics"  n ( d i1 s p e1 p t i k s ) ((pos "Kj$" ) ))
( "e"  n ( ii1 ) ((pos "Ki$" ) ))
( "e'en"  a ( ii1 n ) ((pos "Pu%" ) ))
( "e'er"  a ( e@1 r ) ((pos "Pu%" ) ))
( "e's"  n ( ii1 z ) ((pos "Kj$" ) ))
( "each"  j ( ii ch ) ((pos "OA*" "Qx*" ) ))
( "eager"  j ( ii1 g @ r ) ((pos "OA%" ) ))
( "eagerly"  a ( ii1 g @ l ii ) ((pos "Pu%" ) ))
( "eagerness"  n ( ii1 g @ n @ s ) ((pos "L@%" ) ))
( "eagle"  n ( ii1 g l ) ((pos "K6%" ) ))
( "eagle-eyed"  j ( ii1 g l - ai d ) ((pos "OA%" ) ))
( "eagles"  n ( ii1 g l z ) ((pos "Kj%" ) ))
( "eaglet"  n ( ii1 g l i t ) ((pos "K6$" ) ))
( "eaglets"  n ( ii1 g l i t s ) ((pos "Kj$" ) ))
( "ear"  n ( i@1 r ) ((pos "K6%" ) ))
( "ear-trumpet"  n ( i@1 - t r uh m p i t ) ((pos "K6%" ) ))
( "ear-trumpets"  n ( i@1 - t r uh m p i t s ) ((pos "Kj%" ) ))
( "earache"  n ( i@1 r ei k ) ((pos "M6%" ) ))
( "earaches"  n ( i@1 r ei k s ) ((pos "Mj$" ) ))
( "eardrop"  n ( i@1 d r o p ) ((pos "K6%" ) ))
( "eardrops"  n ( i@1 d r o p s ) ((pos "Kj%" ) ))
( "eardrum"  n ( i@1 d r uh m ) ((pos "K6%" ) ))
( "eardrums"  n ( i@1 d r uh m z ) ((pos "Kj%" ) ))
( "earful"  n ( i@1 f u l ) ((pos "K6%" ) ))
( "earfuls"  n ( i@1 f u l z ) ((pos "Kj$" ) ))
( "earl"  n ( @@1 l ) ((pos "K6%" ) ))
( "earldom"  n ( @@1 l d @ m ) ((pos "K6%" ) ))
( "earldoms"  n ( @@1 l d @ m z ) ((pos "Kj%" ) ))
( "earlier"  j ( @@1 l i@ r ) ((pos "Or%" ) ))
( "earliest"  j ( @@1 l i i s t ) ((pos "Os%" ) ))
( "earls"  n ( @@1 l z ) ((pos "Kj%" ) ))
( "early"  j ( @@1 l ii ) ((pos "OD*" "Pu*" ) ))
( "early-warning"  j ( @@2 l i - w oo1 n i ng ) ((pos "OA%" ) ))
( "earmark"  v ( i@1 m aa k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "earmarked"  v ( i@1 m aa k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "earmarking"  v ( i@1 m aa k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "earmarks"  v ( i@1 m aa k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "earn"  v ( @@1 n ) ((pos "H0%" ) (subcat "6A" "12B" )))
( "earned"  v ( @@1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12B" )))
( "earnest"  n ( @@1 n i s t ) ((pos "L@%" "OA%" ) ))
( "earnest-money"  n ( @@1 n i s t - m uh n ii ) ((pos "L@$" ) ))
( "earnestly"  a ( @@1 n i s t l ii ) ((pos "Pu%" ) ))
( "earnestness"  n ( @@1 n i s t n @ s ) ((pos "L@%" ) ))
( "earning"  v ( @@1 n i ng ) ((pos "Hb%" ) (subcat "6A" "12B" )))
( "earnings"  n ( @@1 n i ng z ) ((pos "Kj%" ) ))
( "earns"  v ( @@1 n z ) ((pos "Ha%" ) (subcat "6A" "12B" )))
( "earphone"  n ( i@1 f ou n ) ((pos "K6%" ) ))
( "earphones"  n ( i@1 f ou n z ) ((pos "Kj%" ) ))
( "earpiece"  n ( i@1 p ii s ) ((pos "K6%" ) ))
( "earpieces"  n ( i@1 p ii s i z ) ((pos "Kj%" ) ))
( "earring"  n ( i@1 r i ng ) ((pos "K6%" ) ))
( "earrings"  n ( i@1 r i ng z ) ((pos "Kj%" ) ))
( "ears"  n ( i@1 z ) ((pos "Kj%" ) ))
( "earshot"  n ( i@1 sh o t ) ((pos "L@%" ) ))
( "earth"  v ( @@1 th ) ((pos "H0$" "L@%" ) (subcat "6A" )))
( "earth-closet"  n ( @@1 th - k l o z i t ) ((pos "K6$" ) ))
( "earth-closets"  n ( @@1 th - k l o z i t s ) ((pos "Kj$" ) ))
( "earthed"  v ( @@1 th t ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "earthen"  j ( @@1 th n ) ((pos "OA%" ) ))
( "earthenware"  n ( e1 dh n w e@ r ) ((pos "L@%" ) ))
( "earthier"  j ( @@1 th i@ r ) ((pos "Or%" ) ))
( "earthiest"  j ( @@1 th i i s t ) ((pos "Os%" ) ))
( "earthing"  v ( @@1 th i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "earthly"  j ( @@1 th l ii ) ((pos "OA%" ) ))
( "earthnut"  n ( @@1 th n uh t ) ((pos "K6$" ) ))
( "earthnuts"  n ( @@1 th n uh t s ) ((pos "Kj$" ) ))
( "earthquake"  n ( @@1 th k w ei k ) ((pos "K6%" ) ))
( "earthquakes"  n ( @@1 th k w ei k s ) ((pos "Kj%" ) ))
( "earths"  v ( @@1 th s ) ((pos "Ha$" ) (subcat "6A" )))
( "earthwork"  n ( @@1 th w @@ k ) ((pos "K6%" ) ))
( "earthworks"  n ( @@1 th w @@ k s ) ((pos "Kj%" ) ))
( "earthworm"  n ( @@1 th w @@ m ) ((pos "K6%" ) ))
( "earthworms"  n ( @@1 th w @@ m z ) ((pos "Kj%" ) ))
( "earthy"  j ( @@1 th ii ) ((pos "OD%" ) ))
( "earwax"  n ( i@1 w a k s ) ((pos "L@%" ) ))
( "earwig"  n ( i@1 w i g ) ((pos "K6%" ) ))
( "earwigs"  n ( i@1 w i g z ) ((pos "Kj%" ) ))
( "ease"  v ( ii1 z ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "eased"  v ( ii1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "easel"  n ( ii1 z l ) ((pos "K6%" ) ))
( "easels"  n ( ii1 z l z ) ((pos "Kj%" ) ))
( "eases"  v ( ii1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "easier"  j ( ii1 z i@ r ) ((pos "Or%" ) ))
( "easiest"  j ( ii1 z i i s t ) ((pos "Os%" ) ))
( "easily"  a ( ii1 z @ l ii ) ((pos "Pu%" ) ))
( "easing"  v ( ii1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "east"  n ( ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "easterly"  j ( ii1 s t @ l ii ) ((pos "OA%" "Pu%" ) ))
( "eastern"  j ( ii1 s t @ n ) ((pos "OA%" ) ))
( "easternmost"  j ( ii1 s t @ n m ou s t ) ((pos "OA%" ) ))
( "eastward"  j ( ii1 s t w @ d ) ((pos "OA%" "Pu%" ) ))
( "eastwards"  a ( ii1 s t w @ d z ) ((pos "Pu%" ) ))
( "easy"  j ( ii1 z ii ) ((pos "OD%" "Pu%" ) ))
( "easygoing"  j ( ii2 z i g ou1 i ng ) ((pos "OA%" ) ))
( "eat"  v ( ii1 t ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "eatable"  n ( ii1 t @ b l ) ((pos "K6%" "OA%" ) ))
( "eatables"  n ( ii1 t @ b l z ) ((pos "Kj%" ) ))
( "eaten"  v ( ii1 t n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "eater"  n ( ii1 t @ r ) ((pos "K6%" ) ))
( "eaters"  n ( ii1 t @ z ) ((pos "Kj%" ) ))
( "eating"  v ( ii1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "eating-apple"  n ( ii1 t i ng - a p l ) ((pos "K6%" ) ))
( "eating-apples"  n ( ii1 t i ng - a p l z ) ((pos "Kj%" ) ))
( "eating-house"  n ( ii1 t i ng - h au s ) ((pos "K6%" ) ))
( "eating-houses"  n ( ii1 t i ng - h au z i z ) ((pos "Kj%" ) ))
( "eats"  v ( ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "eau de Cologne"  n ( ou2 - d @ - k @ l ou1 n ) ((pos "L@%" ) ))
( "eau-de-vie"  n ( ou2 - d @ - v ii1 ) ((pos "L@$" ) ))
( "eaves"  n ( ii1 v z ) ((pos "Kj%" ) ))
( "eavesdrop"  v ( ii1 v z d r o p ) ((pos "I4%" ) (subcat "2A" )))
( "eavesdropped"  v ( ii1 v z d r o p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "eavesdropper"  n ( ii1 v z d r o p @ r ) ((pos "K6$" ) ))
( "eavesdroppers"  n ( ii1 v z d r o p @ z ) ((pos "Kj$" ) ))
( "eavesdropping"  v ( ii1 v z d r o p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "eavesdrops"  v ( ii1 v z d r o p s ) ((pos "Ia$" ) (subcat "2A" )))
( "ebb"  v ( e1 b ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "ebbed"  v ( e1 b d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "ebbing"  v ( e1 b i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "ebbs"  v ( e1 b z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "ebbtide"  n ( e1 b t ai1 d ) ((pos "K6%" ) ))
( "ebbtides"  n ( e1 b t ai1 d z ) ((pos "Kj%" ) ))
( "ebonite"  n ( e1 b @ n ai t ) ((pos "L@$" ) ))
( "ebony"  n ( e1 b @ n ii ) ((pos "L@%" "OA%" ) ))
( "ebullience"  n ( i1 b uh1 l i@ n s ) ((pos "L@%" ) ))
( "ebullient"  j ( i1 b uh1 l i@ n t ) ((pos "OA%" ) ))
( "ebulliently"  a ( i1 b uh1 l i@ n t l ii ) ((pos "Pu%" ) ))
( "eccentric"  n ( e1 k s e1 n t r i k ) ((pos "K6%" "OA%" ) ))
( "eccentricities"  n ( e2 k s e n t r i1 s i t i z ) ((pos "Mj%" ) ))
( "eccentricity"  n ( e2 k s e n t r i1 s i t ii ) ((pos "M8%" ) ))
( "eccentrics"  n ( e1 k s e1 n t r i k s ) ((pos "Kj%" ) ))
( "ecclesiastic"  n ( i1 k l ii2 z i a1 s t i k ) ((pos "K6%" ) ))
( "ecclesiastical"  j ( i1 k l ii2 s i a1 s t i k l ) ((pos "OA%" ) ))
( "ecclesiastically"  a ( i1 k l ii2 s i a1 s t i k l ii ) ((pos "Pu%" ) ))
( "ecclesiastics"  n ( i1 k l ii2 z i a1 s t i k s ) ((pos "Kj%" ) ))
( "echelon"  n ( e1 sh @ l o n ) ((pos "K6$" ) ))
( "echelons"  n ( e1 sh @ l o n z ) ((pos "Kj%" ) ))
( "echo"  v ( e1 k ou ) ((pos "J0%" "M7%" ) (subcat "2A" "2C" "6A" )))
( "echo-sounder"  n ( e1 k ou - s au n d @ r ) ((pos "K6%" ) ))
( "echo-sounders"  n ( e1 k ou - s au n d @ z ) ((pos "Kj%" ) ))
( "echo-sounding"  n ( e1 k ou - s au n d i ng ) ((pos "K6%" ) ))
( "echo-soundings"  n ( e1 k ou - s au n d i ng z ) ((pos "Kj%" ) ))
( "echoed"  v ( e1 k ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "echoes"  n ( e1 k ou z ) ((pos "Mj%" ) ))
( "echoing"  v ( e1 k ou i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "echos"  v ( e1 k ou z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "eclectic"  j ( i1 k l e1 k t i k ) ((pos "OA%" ) ))
( "eclecticism"  n ( i1 k l e1 k t i s i z @ m ) ((pos "L@%" ) ))
( "eclipse"  v ( i1 k l i1 p s ) ((pos "H2%" "K6%" ) (subcat )))
( "eclipsed"  v ( i1 k l i1 p s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eclipses"  v ( i1 k l i1 p s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "eclipsing"  v ( i1 k l i1 p s i ng ) ((pos "Hb%" ) (subcat )))
( "ecliptic"  n ( i1 k l i1 p t i k ) ((pos "Ki$" ) ))
( "ecological"  j ( ii2 k @ l o1 jh i k l ) ((pos "OA%" ) ))
( "ecologically"  a ( ii2 k @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "ecologist"  n ( ii1 k o1 l @ jh i s t ) ((pos "K6%" ) ))
( "ecologists"  n ( ii1 k o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "ecology"  n ( ii1 k o1 l @ jh ii ) ((pos "L@%" ) ))
( "economic"  j ( ii2 k @ n o1 m i k ) ((pos "OA%" ) ))
( "economical"  j ( ii2 k @ n o1 m i k l ) ((pos "OA%" ) ))
( "economically"  a ( ii2 k @ n o1 m i k l ii ) ((pos "Pu%" ) ))
( "economics"  n ( ii2 k @ n o1 m i k s ) ((pos "Lk%" ) ))
( "economies"  n ( i1 k o1 n @ m i z ) ((pos "Mj%" ) ))
( "economist"  n ( i1 k o1 n @ m i s t ) ((pos "K6%" ) ))
( "economists"  n ( i1 k o1 n @ m i s t s ) ((pos "Kj%" ) ))
( "economize"  v ( i1 k o1 n @ m ai z ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "economized"  v ( i1 k o1 n @ m ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "economizes"  v ( i1 k o1 n @ m ai z i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "economizing"  v ( i1 k o1 n @ m ai z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "economy"  n ( i1 k o1 n @ m ii ) ((pos "M8%" ) ))
( "ecosystem"  n ( ii1 k ou s i s t @ m ) ((pos "K6%" ) ))
( "ecosystems"  n ( ii1 k ou s i s t @ m z ) ((pos "Kj%" ) ))
( "ecstasies"  n ( e1 k s t @ s i z ) ((pos "Mj%" ) ))
( "ecstasy"  n ( e1 k s t @ s ii ) ((pos "M8%" ) ))
( "ecstatic"  j ( e1 k s t a1 t i k ) ((pos "OA%" ) ))
( "ecstatically"  a ( e1 k s t a1 t i k l ii ) ((pos "Pu%" ) ))
( "ectoplasm"  n ( e1 k t @ p l a z @ m ) ((pos "L@$" ) ))
( "ecumenical"  j ( ii2 k y uu m e1 n i k l ) ((pos "OA%" ) ))
( "eczema"  n ( e1 k s i m @ ) ((pos "L@%" ) ))
( "ed"  n ( e1 d ) ((pos "Y~%" ) ))
( "eddied"  v ( e1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "eddies"  v ( e1 d i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "eddy"  v ( e1 d ii ) ((pos "I3%" "K8%" ) (subcat "2A" )))
( "eddying"  v ( e1 d i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "edelweiss"  n ( ei1 d l v ai s ) ((pos "L@$" ) ))
( "edge"  v ( e1 jh ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" "14" "15A" )))
( "edged"  v ( e1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "edges"  v ( e1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "14" "15A" )))
( "edgeways"  a ( e1 jh w ei z ) ((pos "Pu%" ) ))
( "edgewise"  a ( e1 jh w ai z ) ((pos "Pu%" ) ))
( "edgier"  j ( e1 jh i@ r ) ((pos "Or%" ) ))
( "edgiest"  j ( e1 jh i i s t ) ((pos "Os%" ) ))
( "edging"  v ( e1 jh i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "6A" "14" "15A" )))
( "edging-shears"  n ( e1 jh i ng - sh i@ z ) ((pos "Kj%" ) ))
( "edgings"  n ( e1 jh i ng z ) ((pos "Mj$" ) ))
( "edgy"  j ( e1 jh ii ) ((pos "OD%" ) ))
( "edibility"  n ( e2 d i b i1 l i t ii ) ((pos "L@%" ) ))
( "edible"  n ( e1 d i b l ) ((pos "K6%" "OA%" ) ))
( "edibles"  n ( e1 d i b l z ) ((pos "Kj%" ) ))
( "edict"  n ( ii1 d i k t ) ((pos "K6%" ) ))
( "edicts"  n ( ii1 d i k t s ) ((pos "Kj%" ) ))
( "edification"  n ( e2 d i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "edifice"  n ( e1 d i f i s ) ((pos "K6%" ) ))
( "edifices"  n ( e1 d i f i s i z ) ((pos "Kj%" ) ))
( "edified"  v ( e1 d i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "edifies"  v ( e1 d i f ai z ) ((pos "Ha%" ) (subcat )))
( "edify"  v ( e1 d i f ai ) ((pos "H3%" ) (subcat )))
( "edifying"  v ( e1 d i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "edit"  v ( e1 d i t ) ((pos "H0%" ) (subcat )))
( "edited"  v ( e1 d i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "editing"  v ( e1 d i t i ng ) ((pos "Hb%" ) (subcat )))
( "edition"  n ( i1 d i1 sh @ n ) ((pos "K6%" ) ))
( "editions"  n ( i1 d i1 sh @ n z ) ((pos "Kj%" ) ))
( "editor"  n ( e1 d i t @ r ) ((pos "K6%" ) ))
( "editorial"  n ( e2 d i t oo1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "editorials"  n ( e2 d i t oo1 r i@ l z ) ((pos "Kj%" ) ))
( "editors"  n ( e1 d i t @ z ) ((pos "Kj%" ) ))
( "edits"  v ( e1 d i t s ) ((pos "Ha%" ) (subcat )))
( "educate"  v ( e1 jh u k ei t ) ((pos "H2%" ) (subcat "6A" "15A" )))
( "educated"  v ( e1 jh u k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "educates"  v ( e1 jh u k ei t s ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "educating"  v ( e1 jh u k ei t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "education"  n ( e2 jh u k ei1 sh @ n ) ((pos "L@%" ) ))
( "educational"  j ( e2 jh u k ei1 sh @ n l ) ((pos "OA%" ) ))
( "educationalist"  n ( e2 jh u k ei1 sh @ n @ l i s t ) ((pos "K6$" ) ))
( "educationalists"  n ( e2 jh u k ei1 sh @ n @ l i s t s ) ((pos "Kj$" ) ))
( "educationally"  a ( e2 jh u k ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "educationist"  n ( i1 d y u k ei1 sh @ n i s t ) ((pos "K6$" ) ))
( "educationists"  n ( i1 d y u k ei1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "educator"  n ( e1 jh u k ei t @ r ) ((pos "K6%" ) ))
( "educators"  n ( e1 jh u k ei t @ z ) ((pos "Kj%" ) ))
( "educe"  v ( i1 d y uu1 s ) ((pos "H2$" ) (subcat )))
( "educed"  v ( i1 d y uu1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "educes"  v ( i1 d y uu1 s i z ) ((pos "Ha$" ) (subcat )))
( "educing"  v ( i1 d y uu1 s i ng ) ((pos "Hb$" ) (subcat )))
( "eel"  n ( ii1 l ) ((pos "K6%" ) ))
( "eels"  n ( ii1 l z ) ((pos "Kj%" ) ))
( "eerie"  j ( i@1 r ii ) ((pos "OB%" ) ))
( "eerier"  j ( i@1 r i@ r ) ((pos "Or%" ) ))
( "eeriest"  j ( i@1 r i i s t ) ((pos "Os%" ) ))
( "eerily"  a ( i@1 r @ l ii ) ((pos "Pu%" ) ))
( "eeriness"  n ( i@1 r i n @ s ) ((pos "L@%" ) ))
( "eery"  j ( i@1 r ii ) ((pos "OD%" ) ))
( "eff"  v ( e1 f ) ((pos "I0$" ) ))
( "efface"  v ( i1 f ei1 s ) ((pos "H2$" ) (subcat )))
( "effaced"  v ( i1 f ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "effacement"  n ( i1 f ei1 s m @ n t ) ((pos "L@%" ) ))
( "effaces"  v ( i1 f ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "effacing"  v ( i1 f ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "effect"  v ( i1 f e1 k t ) ((pos "H0%" "M6%" ) (subcat )))
( "effected"  v ( i1 f e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "effecting"  v ( i1 f e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "effective"  j ( i1 f e1 k t i v ) ((pos "OA%" ) ))
( "effectively"  a ( i1 f e1 k t i v l ii ) ((pos "Pu%" ) ))
( "effectiveness"  n ( i1 f e1 k t i v n @ s ) ((pos "L@%" ) ))
( "effects"  v ( i1 f e1 k t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "effectual"  j ( i1 f e1 k ch u@ l ) ((pos "OA%" ) ))
( "effectuality"  n ( i1 f e2 k ch u a1 l i t ii ) ((pos "L@$" ) ))
( "effectually"  a ( i1 f e1 k ch u@ l ii ) ((pos "Pu$" ) ))
( "effectualness"  n ( i1 f e1 k ch u@ l n @ s ) ((pos "L@$" ) ))
( "effed"  v ( e1 f t ) ((pos "Ic$" "Id$" ) ))
( "effeminacy"  n ( i1 f e1 m i n @ s ii ) ((pos "L@%" ) ))
( "effeminate"  j ( i1 f e1 m i n @ t ) ((pos "OA%" ) ))
( "effendi"  n ( e1 f e1 n d ii ) ((pos "K6$" ) ))
( "effendis"  n ( e1 f e1 n d i z ) ((pos "Kj$" ) ))
( "effervesce"  v ( e2 f @ v e1 s ) ((pos "I2%" ) (subcat )))
( "effervesced"  v ( e2 f @ v e1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "effervescence"  n ( e2 f @ v e1 s n s ) ((pos "L@%" ) ))
( "effervescent"  j ( e2 f @ v e1 s n t ) ((pos "OA%" ) ))
( "effervesces"  v ( e2 f @ v e1 s i z ) ((pos "Ia%" ) (subcat )))
( "effervescing"  v ( e2 f @ v e1 s i ng ) ((pos "Ib%" ) (subcat )))
( "effete"  j ( i1 f ii1 t ) ((pos "OA%" ) ))
( "effeteness"  n ( i1 f ii1 t n @ s ) ((pos "L@$" ) ))
( "efficacious"  j ( e2 f i k ei1 sh @ s ) ((pos "OA%" ) ))
( "efficaciously"  a ( e2 f i k ei1 sh @ s l ii ) ((pos "Pu$" ) ))
( "efficacy"  n ( e1 f i k @ s ii ) ((pos "L@%" ) ))
( "efficiency"  n ( i1 f i1 sh @ n s ii ) ((pos "L@%" ) ))
( "efficient"  j ( i1 f i1 sh @ n t ) ((pos "OA%" ) ))
( "efficiently"  a ( i1 f i1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "effigies"  n ( e1 f i jh i z ) ((pos "Kj%" ) ))
( "effigy"  n ( e1 f i jh ii ) ((pos "K8%" ) ))
( "effing"  v ( e1 f i ng ) ((pos "Ib$" ) ))
( "efflorescence"  n ( e2 f l @ r e1 s n s ) ((pos "L@$" ) ))
( "efflorescent"  j ( e2 f l @ r e1 s n t ) ((pos "OA$" ) ))
( "effluent"  n ( e1 f l u@ n t ) ((pos "M6%" ) ))
( "effluents"  n ( e1 f l u@ n t s ) ((pos "Mj%" ) ))
( "efflux"  n ( e1 f l uh k s ) ((pos "M7$" ) ))
( "effluxes"  n ( e1 f l uh k s i z ) ((pos "Mj$" ) ))
( "effort"  n ( e1 f @ t ) ((pos "M6%" ) ))
( "effortless"  j ( e1 f @ t l @ s ) ((pos "OA%" ) ))
( "effortlessly"  a ( e1 f @ t l @ s l ii ) ((pos "Pu%" ) ))
( "efforts"  n ( e1 f @ t s ) ((pos "Mj%" ) ))
( "effronteries"  n ( i1 f r uh1 n t @ r i z ) ((pos "Mj$" ) ))
( "effrontery"  n ( i1 f r uh1 n t @ r ii ) ((pos "M8%" ) ))
( "effs"  v ( e1 f s ) ((pos "Ia$" ) ))
( "effulgence"  n ( i1 f uh1 l jh @ n s ) ((pos "L@$" ) ))
( "effulgent"  j ( i1 f uh1 l jh @ n t ) ((pos "OA$" ) ))
( "effusion"  n ( i1 f y uu1 zh n ) ((pos "M6%" ) ))
( "effusions"  n ( i1 f y uu1 zh n z ) ((pos "Mj%" ) ))
( "effusive"  j ( i1 f y uu1 s i v ) ((pos "OA%" ) ))
( "effusively"  a ( i1 f y uu1 s i v l ii ) ((pos "Pu%" ) ))
( "effusiveness"  n ( i1 f y uu1 s i v n @ s ) ((pos "L@%" ) ))
( "eft"  n ( e1 f t ) ((pos "K6$" ) ))
( "efts"  n ( e1 f t s ) ((pos "Kj$" ) ))
( "eg"  n ( ii2 jh ii1 ) ((pos "Y~%" ) ))
( "egalitarian"  n ( i1 g a2 l i t e@1 r i@ n ) ((pos "K6$" "OA%" ) ))
( "egalitarianism"  n ( i1 g a2 l i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "egalitarians"  n ( i1 g a2 l i t e@1 r i@ n z ) ((pos "Kj$" ) ))
( "egg"  v ( e1 g ) ((pos "H0%" "M6%" ) (subcat )))
( "egg-beater"  n ( e1 g - b ii t @ r ) ((pos "K6%" ) ))
( "egg-beaters"  n ( e1 g - b ii t @ z ) ((pos "Kj%" ) ))
( "egg-cup"  n ( e1 g - k uh p ) ((pos "K6%" ) ))
( "egg-cups"  n ( e1 g - k uh p s ) ((pos "Kj%" ) ))
( "egg-whisk"  n ( e1 g - w i s k ) ((pos "K6%" ) ))
( "egg-whisks"  n ( e1 g - w i s k s ) ((pos "Kj%" ) ))
( "egged"  v ( e1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "egghead"  n ( e1 g h e d ) ((pos "K6%" ) ))
( "eggheads"  n ( e1 g h e d z ) ((pos "Kj%" ) ))
( "egging"  v ( e1 g i ng ) ((pos "Hb%" ) (subcat )))
( "eggplant"  n ( e1 g p l aa n t ) ((pos "M6%" ) ))
( "eggplants"  n ( e1 g p l aa n t s ) ((pos "Mj%" ) ))
( "eggs"  v ( e1 g z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "eggshake"  n ( e1 g sh ei k ) ((pos "K6%" ) ))
( "eggshakes"  n ( e1 g sh ei k s ) ((pos "Kj%" ) ))
( "eggshell"  n ( e1 g sh e l ) ((pos "K6%" ) ))
( "eggshells"  n ( e1 g sh e l z ) ((pos "Kj%" ) ))
( "eglantine"  n ( e1 g l @ n t ai n ) ((pos "L@$" ) ))
( "ego"  n ( e1 g ou ) ((pos "K6%" ) ))
( "egocentric"  j ( e2 g ou s e1 n t r i k ) ((pos "OA%" ) ))
( "egoism"  n ( e1 g ou i z @ m ) ((pos "L@%" ) ))
( "egoist"  n ( e1 g ou i s t ) ((pos "K6%" ) ))
( "egoistic"  j ( e2 g ou i1 s t i k ) ((pos "OA%" ) ))
( "egoistical"  j ( e2 g ou i1 s t i k @ l ) ((pos "OA%" ) ))
( "egoists"  n ( e1 g ou i s t s ) ((pos "Kj%" ) ))
( "egos"  n ( e1 g ou z ) ((pos "Kj%" ) ))
( "egotism"  n ( e1 g ou t i z @ m ) ((pos "L@%" ) ))
( "egotist"  n ( e1 g @ t i s t ) ((pos "K6%" ) ))
( "egotistic"  j ( e2 g @ t i1 s t i k ) ((pos "OA%" ) ))
( "egotistically"  a ( e2 g @ t i1 s t i k l ii ) ((pos "Pu%" ) ))
( "egotists"  n ( e1 g @ t i s t s ) ((pos "Kj%" ) ))
( "egotrip"  v ( ii1 g ou t r i p ) ((pos "I4$" "K6%" ) ))
( "egotripped"  v ( ii1 g ou t r i p t ) ((pos "Ic$" "Id$" ) ))
( "egotripping"  v ( ii1 g ou t r i p i ng ) ((pos "Ib$" ) ))
( "egotrips"  v ( ii1 g ou t r i p s ) ((pos "Ia$" "Kj%" ) ))
( "egregious"  j ( i1 g r ii1 jh i@ s ) ((pos "OA$" ) ))
( "egress"  n ( ii1 g r e s ) ((pos "M7$" ) ))
( "egresses"  n ( ii1 g r e s i z ) ((pos "Mj$" ) ))
( "egret"  n ( ii1 g r e t ) ((pos "K6$" ) ))
( "egrets"  n ( ii1 g r e t s ) ((pos "Kj$" ) ))
( "eh"  uh ( ei1 ) ((pos "W-$" ) ))
( "eiderdown"  n ( ai1 d @ d au n ) ((pos "K6%" ) ))
( "eiderdowns"  n ( ai1 d @ d au n z ) ((pos "Kj%" ) ))
( "eight"  n ( ei1 t ) ((pos "K6%" "OA%" ) ))
( "eighteen"  n ( ei2 t ii1 n ) ((pos "K6%" "OA%" ) ))
( "eighteens"  n ( ei2 t ii1 n z ) ((pos "Kj%" ) ))
( "eighteenth"  n ( ei2 t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "eighteenths"  n ( ei2 t ii1 n th s ) ((pos "Kj%" ) ))
( "eighth"  n ( ei1 t th ) ((pos "K6%" "OA%" ) ))
( "eighths"  n ( ei1 t th s ) ((pos "Kj%" ) ))
( "eighties"  n ( ei1 t i z ) ((pos "Kj%" ) ))
( "eightieth"  n ( ei1 t i@ th ) ((pos "K6%" "OA%" ) ))
( "eightieths"  n ( ei1 t i@ th s ) ((pos "Kj%" ) ))
( "eightpence"  n ( ei1 t p @ n s ) ((pos "K6%" ) ))
( "eightpences"  n ( ei1 t p @ n s i z ) ((pos "Kj%" ) ))
( "eightpenny"  j ( ei1 t p @ n ii ) ((pos "OA%" ) ))
( "eights"  n ( ei1 t s ) ((pos "Kj%" ) ))
( "eightsome"  n ( ei1 t s @ m ) ((pos "K6%" ) ))
( "eightsomes"  n ( ei1 t s @ m z ) ((pos "Kj$" ) ))
( "eighty"  n ( ei1 t ii ) ((pos "K8%" "OA%" ) ))
( "eisteddfod"  n ( ai2 s t e1 dh v o d ) ((pos "K6%" ) ))
( "eisteddfods"  n ( ai2 s t e1 dh v o d z ) ((pos "Kj%" ) ))
( "either"  j ( ai1 dh @ r ) ((pos "OA*" "Pu*" "Qx*" "V-*" ) ))
( "ejaculate"  v ( i1 jh a1 k y u l ei t ) ((pos "H2%" ) (subcat )))
( "ejaculated"  v ( i1 jh a1 k y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ejaculates"  v ( i1 jh a1 k y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "ejaculating"  v ( i1 jh a1 k y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "ejaculation"  n ( i1 jh a2 k y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "ejaculations"  n ( i1 jh a2 k y u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "eject"  v ( i1 jh e1 k t ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "ejected"  v ( i1 jh e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "ejecting"  v ( i1 jh e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "ejection"  n ( i1 jh e1 k sh @ n ) ((pos "K6%" ) ))
( "ejections"  n ( i1 jh e1 k sh @ n z ) ((pos "Kj%" ) ))
( "ejector"  n ( i1 jh e1 k t @ r ) ((pos "K6%" ) ))
( "ejector-seat"  n ( i1 jh e1 k t @ - s ii t ) ((pos "K6%" ) ))
( "ejector-seats"  n ( i1 jh e1 k t @ - s ii t s ) ((pos "Kj%" ) ))
( "ejectors"  n ( i1 jh e1 k t @ z ) ((pos "Kj%" ) ))
( "ejects"  v ( i1 jh e1 k t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "eke"  v ( ii1 k ) ((pos "H2%" ) (subcat )))
( "eked"  v ( ii1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ekes"  v ( ii1 k s ) ((pos "Ha%" ) (subcat )))
( "eking"  v ( ii1 k i ng ) ((pos "Hb%" ) (subcat )))
( "elaborate"  j ( i1 l a1 b @ r @ t ) ((pos "OA%" ) ))
( "elaborate"  v ( i1 l a1 b @ r ei t ) ((pos "H2%" ) (subcat )))
( "elaborated"  v ( i1 l a1 b @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "elaborately"  a ( i1 l a1 b @ r @ t l ii ) ((pos "Pu%" ) ))
( "elaborateness"  n ( i1 l a1 b @ r @ t n @ s ) ((pos "L@%" ) ))
( "elaborates"  v ( i1 l a1 b @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "elaborating"  v ( i1 l a1 b @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "elaboration"  n ( i1 l a2 b @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "elaborations"  n ( i1 l a2 b @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "eland"  n ( ii1 l @ n d ) ((pos "K6$" ) ))
( "elands"  n ( ii1 l @ n d z ) ((pos "Kj$" ) ))
( "elapse"  v ( i1 l a1 p s ) ((pos "I2%" ) (subcat )))
( "elapsed"  v ( i1 l a1 p s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "elapses"  v ( i1 l a1 p s i z ) ((pos "Ia%" ) (subcat )))
( "elapsing"  v ( i1 l a1 p s i ng ) ((pos "Ib%" ) (subcat )))
( "elastic"  n ( i1 l a1 s t i k ) ((pos "L@%" "OA%" ) ))
( "elasticities"  n ( e2 l a s t i1 s i t i z ) ((pos "Mj$" ) ))
( "elasticity"  n ( e2 l a s t i1 s i t ii ) ((pos "M8%" ) ))
( "elasticized"  j ( i1 l a1 s t i s ai z d ) ((pos "OA%" ) ))
( "elate"  v ( i1 l ei1 t ) ((pos "H2$" ) (subcat )))
( "elated"  v ( i1 l ei1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "elates"  v ( i1 l ei1 t s ) ((pos "Ha$" ) (subcat )))
( "elating"  v ( i1 l ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "elation"  n ( i1 l ei1 sh @ n ) ((pos "L@%" ) ))
( "elbow"  v ( e1 l b ou ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "elbowed"  v ( e1 l b ou d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "elbowing"  v ( e1 l b ou i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "elbows"  v ( e1 l b ou z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "elder"  n ( e1 l d @ r ) ((pos "K6%" "Or%" ) ))
( "elderly"  j ( e1 l d @ l ii ) ((pos "OA%" ) ))
( "elders"  n ( e1 l d @ z ) ((pos "Kj%" ) ))
( "eldest"  j ( e1 l d i s t ) ((pos "Os%" ) ))
( "elect"  v ( i1 l e1 k t ) ((pos "H0%" "OA%" ) (subcat "6A" "7A" "14" "23" )))
( "elected"  v ( i1 l e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" "14" "23" )))
( "electing"  v ( i1 l e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "7A" "14" "23" )))
( "election"  n ( i1 l e1 k sh @ n ) ((pos "M6%" ) ))
( "electioneering"  n ( i1 l e2 k sh @ n i@1 r i ng ) ((pos "L@%" ) ))
( "elections"  n ( i1 l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "elective"  j ( i1 l e1 k t i v ) ((pos "OA$" ) ))
( "elector"  n ( i1 l e1 k t @ r ) ((pos "K6%" ) ))
( "electoral"  j ( i1 l e1 k t @ r @ l ) ((pos "OA%" ) ))
( "electorate"  n ( i1 l e1 k t @ r @ t ) ((pos "K6%" ) ))
( "electorates"  n ( i1 l e1 k t @ r @ t s ) ((pos "Kj%" ) ))
( "electors"  n ( i1 l e1 k t @ z ) ((pos "Kj%" ) ))
( "electric"  j ( i1 l e1 k t r i k ) ((pos "OA%" ) ))
( "electrical"  j ( i1 l e1 k t r i k l ) ((pos "OA%" ) ))
( "electrically"  a ( i1 l e1 k t r i k l ii ) ((pos "Pu%" ) ))
( "electrician"  n ( i1 l e2 k t r i1 sh @ n ) ((pos "K6%" ) ))
( "electricians"  n ( i1 l e2 k t r i1 sh @ n z ) ((pos "Kj%" ) ))
( "electricity"  n ( i1 l e2 k t r i1 s i t ii ) ((pos "L@%" ) ))
( "electrification"  n ( i1 l e2 k t r i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "electrified"  v ( i1 l e1 k t r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "electrifies"  v ( i1 l e1 k t r i f ai z ) ((pos "Ha%" ) (subcat )))
( "electrify"  v ( i1 l e1 k t r i f ai ) ((pos "H3%" ) (subcat )))
( "electrifying"  v ( i1 l e1 k t r i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "electro-"  j ( i1 l e1 k t r ou - ) ((pos "U-%" ) ))
( "electrocardiogram"  n ( i1 l e2 k t r ou k aa1 d i@ u g r a m ) ((pos "K6%" ) ))
( "electrocardiograms"  n ( i1 l e2 k t r ou k aa1 d i@ u g r a m z ) ((pos "Kj%" ) ))
( "electrocardiograph"  n ( i1 l e2 k t r ou k aa1 d i@ u g r a f ) ((pos "K6%" ) ))
( "electrocardiographs"  n ( i1 l e2 k t r ou k aa1 d i@ u g r a f s ) ((pos "Kj%" ) ))
( "electrochemistry"  n ( i1 l e2 k t r ou k e1 m i s t r ii ) ((pos "L@%" ) ))
( "electrocute"  v ( i1 l e1 k t r @ k y uu t ) ((pos "H2%" ) (subcat )))
( "electrocuted"  v ( i1 l e1 k t r @ k y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "electrocutes"  v ( i1 l e1 k t r @ k y uu t s ) ((pos "Ha%" ) (subcat )))
( "electrocuting"  v ( i1 l e1 k t r @ k y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "electrocution"  n ( i1 l e2 k t r @ k y uu1 sh @ n ) ((pos "M6%" ) ))
( "electrocutions"  n ( i1 l e2 k t r @ k y uu1 sh @ n z ) ((pos "Mj$" ) ))
( "electrode"  n ( i1 l e1 k t r ou d ) ((pos "K6%" ) ))
( "electrodes"  n ( i1 l e1 k t r ou d z ) ((pos "Kj%" ) ))
( "electrolysis"  n ( i1 l e2 k t r o1 l @ s i s ) ((pos "L@%" ) ))
( "electrolyte"  n ( i1 l e1 k t r @ l ai t ) ((pos "K6%" ) ))
( "electrolytes"  n ( i1 l e1 k t r @ l ai t s ) ((pos "Kj%" ) ))
( "electromagnet"  n ( i1 l e1 k t r ou m a2 g n i t ) ((pos "K6%" ) ))
( "electromagnetic"  j ( i1 l e2 k t r ou m a g n e1 t i k ) ((pos "OA%" ) ))
( "electromagnetism"  n ( i1 l e2 k t r ou m a1 g n i t i z @ m ) ((pos "L@%" ) ))
( "electromagnets"  n ( i1 l e1 k t r ou m a2 g n i t s ) ((pos "Kj%" ) ))
( "electron"  n ( i1 l e1 k t r o n ) ((pos "K6%" ) ))
( "electronic"  j ( i2 l e k t r o1 n i k ) ((pos "OA%" ) ))
( "electronically"  a ( i2 l e k t r o1 n i k l ii ) ((pos "Pu%" ) ))
( "electronics"  n ( i2 l e k t r o1 n i k s ) ((pos "Lk%" ) ))
( "electrons"  n ( i1 l e1 k t r o n z ) ((pos "Kj%" ) ))
( "electroplate"  v ( i1 l e1 k t r @ p l ei t ) ((pos "H2$" "L@$" ) ))
( "electroplated"  v ( i1 l e1 k t r @ p l ei t i d ) ((pos "Hc$" "Hd%" ) ))
( "electroplates"  v ( i1 l e1 k t r @ p l ei t s ) ((pos "Ha$" ) ))
( "electroplating"  v ( i1 l e1 k t r @ p l ei t i ng ) ((pos "Hb$" ) ))
( "electrostatic"  j ( i1 l e2 k t r ou s t a1 t i k ) ((pos "OA%" ) ))
( "elects"  v ( i1 l e1 k t s ) ((pos "Ha%" ) (subcat "6A" "7A" "14" "23" )))
( "eleemosynary"  j ( e2 l i ii m o1 s i n @ r ii ) ((pos "OA$" ) ))
( "elegance"  n ( e1 l i g @ n s ) ((pos "L@%" ) ))
( "elegant"  j ( e1 l i g @ n t ) ((pos "OA%" ) ))
( "elegantly"  a ( e1 l i g @ n t l ii ) ((pos "Pu%" ) ))
( "elegiac"  j ( e2 l i jh ai1 @ k ) ((pos "OA$" ) ))
( "elegiacs"  n ( e2 l i jh ai1 @ k s ) ((pos "Kj$" ) ))
( "elegies"  n ( e1 l @ jh i z ) ((pos "Kj%" ) ))
( "elegy"  n ( e1 l @ jh ii ) ((pos "K8%" ) ))
( "element"  n ( e1 l i m @ n t ) ((pos "K6%" ) ))
( "elemental"  j ( e2 l i m e1 n t l ) ((pos "OA%" ) ))
( "elementarily"  a ( e2 l i m e1 n t @ r @ l ii ) ((pos "Pu$" ) ))
( "elementary"  j ( e2 l i m e1 n t @ r ii ) ((pos "OA%" ) ))
( "elements"  n ( e1 l i m @ n t s ) ((pos "Kj%" ) ))
( "elephant"  n ( e1 l i f @ n t ) ((pos "K6%" ) ))
( "elephantiasis"  n ( e2 l i f @ n t ai1 @ s i s ) ((pos "L@%" ) ))
( "elephantine"  j ( e2 l i f a1 n t ai n ) ((pos "OA%" ) ))
( "elephants"  n ( e1 l i f @ n t s ) ((pos "Kj%" ) ))
( "elevate"  v ( e1 l i v ei t ) ((pos "H2%" ) (subcat "6A" )))
( "elevated"  v ( e1 l i v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "elevates"  v ( e1 l i v ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "elevating"  v ( e1 l i v ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "elevation"  n ( e2 l i v ei1 sh @ n ) ((pos "M6%" ) ))
( "elevations"  n ( e2 l i v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "elevator"  n ( e1 l i v ei t @ r ) ((pos "K6%" ) ))
( "elevators"  n ( e1 l i v ei t @ z ) ((pos "Kj%" ) ))
( "eleven"  n ( i1 l e1 v n ) ((pos "K6%" "OA%" ) ))
( "elevens"  n ( i1 l e1 v n z ) ((pos "Kj%" ) ))
( "elevenses"  n ( i1 l e1 v n z i z ) ((pos "Kj%" ) ))
( "eleventh"  n ( i1 l e1 v n th ) ((pos "K6%" "OA%" ) ))
( "elevenths"  n ( i1 l e1 v n th s ) ((pos "Kj%" ) ))
( "elf"  n ( e1 l f ) ((pos "Ki%" ) ))
( "elfin"  j ( e1 l f i n ) ((pos "OA%" ) ))
( "elfish"  j ( e1 l f i sh ) ((pos "OA%" ) ))
( "elicit"  v ( i1 l i1 s i t ) ((pos "H0%" ) (subcat "6A" )))
( "elicitation"  n ( i1 l i2 s i t ei1 sh @ n ) ((pos "K6$" ) ))
( "elicitations"  n ( i1 l i2 s i t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "elicited"  v ( i1 l i1 s i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "eliciting"  v ( i1 l i1 s i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "elicits"  v ( i1 l i1 s i t s ) ((pos "Ha%" ) (subcat "6A" )))
( "elide"  v ( i1 l ai1 d ) ((pos "H2$" ) (subcat )))
( "elided"  v ( i1 l ai1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "elides"  v ( i1 l ai1 d z ) ((pos "Ha$" ) (subcat )))
( "eliding"  v ( i1 l ai1 d i ng ) ((pos "Hb$" ) (subcat )))
( "eligibility"  n ( e2 l i jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "eligible"  j ( e1 l i jh @ b l ) ((pos "OA%" ) ))
( "eliminate"  v ( i1 l i1 m i n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "eliminated"  v ( i1 l i1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "eliminates"  v ( i1 l i1 m i n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "eliminating"  v ( i1 l i1 m i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "elimination"  n ( i1 l i2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "eliminations"  n ( i1 l i2 m i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "elision"  n ( i1 l i1 zh n ) ((pos "M6$" ) ))
( "elisions"  n ( i1 l i1 zh n z ) ((pos "Mj$" ) ))
( "elitism"  n ( i1 l ii1 t i z @ m ) ((pos "L@%" ) ))
( "elitist"  n ( i1 l ii1 t i s t ) ((pos "K6%" ) ))
( "elitists"  n ( i1 l ii1 t i s t s ) ((pos "Kj$" ) ))
( "elixir"  n ( i1 l i1 k s @ r ) ((pos "K6%" ) ))
( "elixirs"  n ( i1 l i1 k s @ z ) ((pos "Kj$" ) ))
( "elk"  n ( e1 l k ) ((pos "K6$" ) ))
( "elks"  n ( e1 l k s ) ((pos "Kj$" ) ))
( "ellipse"  n ( i1 l i1 p s ) ((pos "K6%" ) ))
( "ellipses"  n ( i1 l i1 p s i z ) ((pos "Kj%" ) ))
( "ellipses"  n ( i1 l i1 p s ii z ) ((pos "Kj$" ) ))
( "ellipsis"  n ( i1 l i1 p s i s ) ((pos "Mi$" ) ))
( "elliptic"  j ( i1 l i1 p t i k ) ((pos "OA%" ) ))
( "elliptical"  j ( i1 l i1 p t i k l ) ((pos "OA%" ) ))
( "elm"  n ( e1 l m ) ((pos "M6%" ) ))
( "elms"  n ( e1 l m z ) ((pos "Mj%" ) ))
( "elocution"  n ( e2 l @ k y uu1 sh @ n ) ((pos "L@%" ) ))
( "elocutionary"  j ( i2 l @ k y uu1 sh @ n @ r ii ) ((pos "OA$" ) ))
( "elocutionist"  n ( e2 l @ k y uu1 sh @ n i s t ) ((pos "K6%" ) ))
( "elocutionists"  n ( e2 l @ k y uu1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "elongate"  v ( ii1 l o ng g ei t ) ((pos "J2%" ) (subcat "2A" )))
( "elongated"  v ( ii1 l o ng g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "elongates"  v ( ii1 l o ng g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "elongating"  v ( ii1 l o ng g ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "elongation"  n ( ii2 l o ng g ei1 sh @ n ) ((pos "M6%" ) ))
( "elongations"  n ( ii2 l o ng g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "elope"  v ( i1 l ou1 p ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "eloped"  v ( i1 l ou1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "elopement"  n ( i1 l ou1 p m @ n t ) ((pos "K6%" ) ))
( "elopements"  n ( i1 l ou1 p m @ n t s ) ((pos "Kj%" ) ))
( "elopes"  v ( i1 l ou1 p s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "eloping"  v ( i1 l ou1 p i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "eloquence"  n ( e1 l @ k w @ n s ) ((pos "L@%" ) ))
( "eloquent"  j ( e1 l @ k w @ n t ) ((pos "OA%" ) ))
( "eloquently"  a ( e1 l @ k w @ n t l ii ) ((pos "Pu%" ) ))
( "else"  a ( e1 l s ) ((pos "Pu%" ) ))
( "elsewhere"  a ( e2 l s w e@1 r ) ((pos "Pu%" ) ))
( "elucidate"  v ( i1 l uu1 s i d ei t ) ((pos "H2%" ) (subcat )))
( "elucidated"  v ( i1 l uu1 s i d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "elucidates"  v ( i1 l uu1 s i d ei t s ) ((pos "Ha%" ) (subcat )))
( "elucidating"  v ( i1 l uu1 s i d ei t i ng ) ((pos "Hb%" ) (subcat )))
( "elucidation"  n ( i1 l uu2 s i d ei1 sh @ n ) ((pos "K6%" ) ))
( "elucidations"  n ( i1 l uu2 s i d ei1 sh @ n z ) ((pos "Kj$" ) ))
( "elude"  v ( i1 l uu1 d ) ((pos "H2%" ) (subcat )))
( "eluded"  v ( i1 l uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eludes"  v ( i1 l uu1 d z ) ((pos "Ha%" ) (subcat )))
( "eluding"  v ( i1 l uu1 d i ng ) ((pos "Hb%" ) (subcat )))
( "elusive"  j ( i1 l uu1 s i v ) ((pos "OA%" ) ))
( "elver"  n ( e1 l v @ r ) ((pos "K6$" ) ))
( "elvers"  n ( e1 l v @ z ) ((pos "Kj$" ) ))
( "elves"  n ( e1 l v z ) ((pos "Kj%" ) ))
( "elvish"  j ( e1 l v i sh ) ((pos "OA$" ) ))
( "emaciate"  v ( i1 m ei1 sh i ei t ) ((pos "H2$" ) (subcat )))
( "emaciated"  v ( i1 m ei1 sh i ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "emaciates"  v ( i1 m ei1 sh i ei t s ) ((pos "Ha$" ) (subcat )))
( "emaciating"  v ( i1 m ei1 sh i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "emaciation"  n ( i1 m ei2 s i ei1 sh @ n ) ((pos "L@%" ) ))
( "emanate"  v ( e1 m @ n ei t ) ((pos "I2%" ) (subcat )))
( "emanated"  v ( e1 m @ n ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "emanates"  v ( e1 m @ n ei t s ) ((pos "Ia%" ) (subcat )))
( "emanating"  v ( e1 m @ n ei t i ng ) ((pos "Ib%" ) (subcat )))
( "emanation"  n ( e2 m @ n ei1 sh @ n ) ((pos "M6%" ) ))
( "emanations"  n ( e2 m @ n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "emancipate"  v ( i1 m a1 n s i p ei t ) ((pos "H2%" ) (subcat "6A" )))
( "emancipated"  v ( i1 m a1 n s i p ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "emancipates"  v ( i1 m a1 n s i p ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "emancipating"  v ( i1 m a1 n s i p ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "emancipation"  n ( i1 m a2 n s i p ei1 sh @ n ) ((pos "L@%" ) ))
( "emasculate"  v ( i1 m a1 s k y u l ei t ) ((pos "H2%" ) (subcat )))
( "emasculated"  v ( i1 m a1 s k y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "emasculates"  v ( i1 m a1 s k y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "emasculating"  v ( i1 m a1 s k y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "emasculation"  n ( i1 m a2 s k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "emasculations"  n ( i1 m a2 s k y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "embalm"  v ( i1 m b aa1 m ) ((pos "H0%" ) (subcat )))
( "embalmed"  v ( i1 m b aa1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "embalming"  v ( i1 m b aa1 m i ng ) ((pos "Hb%" ) (subcat )))
( "embalmment"  n ( i1 m b aa1 m @ n t ) ((pos "K6%" ) ))
( "embalmments"  n ( i1 m b aa1 m @ n t s ) ((pos "Kj%" ) ))
( "embalms"  v ( i1 m b aa1 m z ) ((pos "Ha%" ) (subcat )))
( "embankment"  n ( i1 m b a1 ng k m @ n t ) ((pos "K6%" ) ))
( "embankments"  n ( i1 m b a1 ng k m @ n t s ) ((pos "Kj%" ) ))
( "embargo"  v ( i1 m b aa1 g ou ) ((pos "H0$" "K7%" ) (subcat )))
( "embargoed"  v ( i1 m b aa1 g ou d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "embargoes"  n ( i1 m b aa1 g ou z ) ((pos "Kj%" ) ))
( "embargoing"  v ( i1 m b aa1 g ou i ng ) ((pos "Hb$" ) (subcat )))
( "embargos"  v ( i1 m b aa1 g ou z ) ((pos "Ha$" ) (subcat )))
( "embark"  v ( i1 m b aa1 k ) ((pos "J0%" ) (subcat "2A" "2C" "3A" )))
( "embarkation"  n ( e2 m b aa k ei1 sh @ n ) ((pos "M6%" ) ))
( "embarkations"  n ( e2 m b aa k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "embarked"  v ( i1 m b aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "embarking"  v ( i1 m b aa1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "embarks"  v ( i1 m b aa1 k s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "embarrass"  v ( i1 m b a1 r @ s ) ((pos "H1%" ) (subcat )))
( "embarrassed"  v ( i1 m b a1 r @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "embarrasses"  v ( i1 m b a1 r @ s i z ) ((pos "Ha%" ) (subcat )))
( "embarrassing"  v ( i1 m b a1 r @ s i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "embarrassingly"  a ( i1 m b a1 r @ s i ng l ii ) ((pos "Pu%" ) ))
( "embarrassment"  n ( i1 m b a1 r @ s m @ n t ) ((pos "M6%" ) ))
( "embarrassments"  n ( i1 m b a1 r @ s m @ n t s ) ((pos "Mj%" ) ))
( "embassies"  n ( e1 m b @ s i z ) ((pos "Kj%" ) ))
( "embassy"  n ( e1 m b @ s ii ) ((pos "K8%" ) ))
( "embattled"  j ( i1 m b a1 t l d ) ((pos "OA%" ) ))
( "embed"  v ( i1 m b e1 d ) ((pos "H4%" ) (subcat "6A" )))
( "embedded"  v ( i1 m b e1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "embedding"  v ( i1 m b e1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "embeds"  v ( i1 m b e1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "embellish"  v ( i1 m b e1 l i sh ) ((pos "H1%" ) (subcat "6A" )))
( "embellished"  v ( i1 m b e1 l i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "embellishes"  v ( i1 m b e1 l i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "embellishing"  v ( i1 m b e1 l i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "embellishment"  n ( i1 m b e1 l i sh m @ n t ) ((pos "M6%" ) ))
( "embellishments"  n ( i1 m b e1 l i sh m @ n t s ) ((pos "Mj%" ) ))
( "ember"  n ( e1 m b @ r ) ((pos "K6%" ) ))
( "embers"  n ( e1 m b @ z ) ((pos "Kj%" ) ))
( "embezzle"  v ( i1 m b e1 z l ) ((pos "H2%" ) (subcat )))
( "embezzled"  v ( i1 m b e1 z l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "embezzlement"  n ( i1 m b e1 z l m @ n t ) ((pos "M6%" ) ))
( "embezzlements"  n ( i1 m b e1 z l m @ n t s ) ((pos "Mj%" ) ))
( "embezzles"  v ( i1 m b e1 z l z ) ((pos "Ha%" ) (subcat )))
( "embezzling"  v ( i1 m b e1 z l i ng ) ((pos "Hb%" ) (subcat )))
( "embitter"  v ( i1 m b i1 t @ r ) ((pos "H0%" ) (subcat )))
( "embittered"  v ( i1 m b i1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "embittering"  v ( i1 m b i1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "embitterment"  n ( i1 m b i1 t @ m @ n t ) ((pos "K6%" ) ))
( "embitterments"  n ( i1 m b i1 t @ m @ n t s ) ((pos "Kj$" ) ))
( "embitters"  v ( i1 m b i1 t @ z ) ((pos "Ha%" ) (subcat )))
( "emblazon"  v ( i1 m b l ei1 z n ) ((pos "H0%" ) (subcat "6A" )))
( "emblazoned"  v ( i1 m b l ei1 z n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "emblazoning"  v ( i1 m b l ei1 z n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "emblazons"  v ( i1 m b l ei1 z n z ) ((pos "Ha%" ) (subcat "6A" )))
( "emblem"  n ( e1 m b l @ m ) ((pos "K6%" ) ))
( "emblematic"  j ( e2 m b l @ m a1 t i k ) ((pos "OA%" ) ))
( "emblems"  n ( e1 m b l @ m z ) ((pos "Kj%" ) ))
( "embodied"  v ( i1 m b o1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "embodies"  v ( i1 m b o1 d i z ) ((pos "Ha%" ) (subcat "6A" )))
( "embodiment"  n ( i1 m b o1 d i m @ n t ) ((pos "K6%" ) ))
( "embodiments"  n ( i1 m b o1 d i m @ n t s ) ((pos "Kj$" ) ))
( "embody"  v ( i1 m b o1 d ii ) ((pos "H3%" ) (subcat "6A" )))
( "embodying"  v ( i1 m b o1 d i i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "embolden"  v ( i1 m b ou1 l d @ n ) ((pos "H0%" ) (subcat "6A" )))
( "emboldened"  v ( i1 m b ou1 l d @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "emboldening"  v ( i1 m b ou1 l d @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "emboldens"  v ( i1 m b ou1 l d @ n z ) ((pos "Ha%" ) (subcat "6A" )))
( "embonpoint"  n ( o2 m b o n p w aa1 ng ) ((pos "L@$" "Op$" ) ))
( "emboss"  v ( i1 m b o1 s ) ((pos "H1%" ) (subcat "6A" )))
( "embossed"  v ( i1 m b o1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "embosses"  v ( i1 m b o1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "embossing"  v ( i1 m b o1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "embrace"  v ( i1 m b r ei1 s ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "embraced"  v ( i1 m b r ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "embraces"  v ( i1 m b r ei1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "embracing"  v ( i1 m b r ei1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "embrasure"  n ( i1 m b r ei1 zh @ r ) ((pos "K6$" ) ))
( "embrasures"  n ( i1 m b r ei1 zh @ z ) ((pos "Kj$" ) ))
( "embrocation"  n ( e2 m b r @ k ei1 sh @ n ) ((pos "L@%" ) ))
( "embroider"  v ( i1 m b r oi1 d @ r ) ((pos "J0%" ) (subcat "2A" )))
( "embroidered"  v ( i1 m b r oi1 d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "embroidering"  v ( i1 m b r oi1 d @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "embroiders"  v ( i1 m b r oi1 d @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "embroidery"  n ( i1 m b r oi1 d @ r ii ) ((pos "L@%" ) ))
( "embroil"  v ( i1 m b r oi1 l ) ((pos "H0%" ) (subcat "6A" )))
( "embroiled"  v ( i1 m b r oi1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "embroiling"  v ( i1 m b r oi1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "embroils"  v ( i1 m b r oi1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "embryo"  n ( e1 m b r i@ @ ) ((pos "K6%" ) ))
( "embryonic"  j ( e2 m b r i o1 n i k ) ((pos "OA%" ) ))
( "embryos"  n ( e1 m b r i@ u z ) ((pos "Kj%" ) ))
( "emeer"  n ( e1 m i@1 r ) ((pos "K6$" ) ))
( "emeers"  n ( e1 m i@1 z ) ((pos "Kj$" ) ))
( "emend"  v ( i1 m e1 n d ) ((pos "H0$" ) (subcat )))
( "emendation"  n ( ii2 m e n d ei1 sh @ n ) ((pos "M6%" ) ))
( "emendations"  n ( ii2 m e n d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "emended"  v ( i1 m e1 n d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "emending"  v ( i1 m e1 n d i ng ) ((pos "Hb$" ) (subcat )))
( "emends"  v ( i1 m e1 n d z ) ((pos "Ha$" ) (subcat )))
( "emerald"  n ( e1 m @ r @ l d ) ((pos "K6%" ) ))
( "emeralds"  n ( e1 m @ r @ l d z ) ((pos "Kj%" ) ))
( "emerge"  v ( i1 m @@1 jh ) ((pos "I2%" ) (subcat "2A" )))
( "emerged"  v ( i1 m @@1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "emergence"  n ( i1 m @@1 jh @ n s ) ((pos "L@%" ) ))
( "emergencies"  n ( i1 m @@1 jh @ n s i z ) ((pos "Mj%" ) ))
( "emergency"  n ( i1 m @@1 jh @ n s ii ) ((pos "M8%" ) ))
( "emergent"  j ( i1 m @@1 jh @ n t ) ((pos "OA%" ) ))
( "emerges"  v ( i1 m @@1 jh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "emerging"  v ( i1 m @@1 jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "emeritus"  j ( i1 m e1 r i t @ s ) ((pos "OA$" ) ))
( "emery"  n ( e1 m @ r ii ) ((pos "L@%" ) ))
( "emetic"  n ( i1 m e1 t i k ) ((pos "K6$" ) ))
( "emetics"  n ( i1 m e1 t i k s ) ((pos "Kj$" ) ))
( "emigrant"  n ( e1 m i g r @ n t ) ((pos "K6%" ) ))
( "emigrants"  n ( e1 m i g r @ n t s ) ((pos "Kj%" ) ))
( "emigrate"  v ( e1 m i g r ei t ) ((pos "I2%" ) (subcat "2A" )))
( "emigrated"  v ( e1 m i g r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "emigrates"  v ( e1 m i g r ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "emigrating"  v ( e1 m i g r ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "emigration"  n ( e2 m i g r ei1 sh @ n ) ((pos "M6%" ) ))
( "emigrations"  n ( e2 m i g r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "eminence"  n ( e1 m i n @ n s ) ((pos "M6%" ) ))
( "eminences"  n ( e1 m i n @ n s i z ) ((pos "Mj$" ) ))
( "eminent"  j ( e1 m i n @ n t ) ((pos "OA%" ) ))
( "eminently"  a ( e1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "emir"  n ( e1 m i@1 r ) ((pos "K6$" ) ))
( "emirate"  n ( e1 m i@1 r ei t ) ((pos "K6%" ) ))
( "emirates"  n ( e1 m i@1 r ei t s ) ((pos "Kj%" ) ))
( "emirs"  n ( e1 m i@1 z ) ((pos "Kj$" ) ))
( "emissaries"  n ( e1 m i s @ r i z ) ((pos "Kj%" ) ))
( "emissary"  n ( e1 m i s @ r ii ) ((pos "K8%" ) ))
( "emission"  n ( i1 m i1 sh @ n ) ((pos "M6%" ) ))
( "emissions"  n ( i1 m i1 sh @ n z ) ((pos "Mj%" ) ))
( "emit"  v ( i1 m i1 t ) ((pos "H4%" ) (subcat )))
( "emits"  v ( i1 m i1 t s ) ((pos "Ha%" ) (subcat )))
( "emitted"  v ( i1 m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "emitting"  v ( i1 m i1 t i ng ) ((pos "Hb%" ) (subcat )))
( "emolument"  n ( i1 m o1 l y u m @ n t ) ((pos "K6%" ) ))
( "emoluments"  n ( i1 m o1 l y u m @ n t s ) ((pos "Kj%" ) ))
( "emotion"  n ( i1 m ou1 sh @ n ) ((pos "M6%" ) ))
( "emotional"  j ( i1 m ou1 sh @ n l ) ((pos "OA%" ) ))
( "emotionally"  a ( i1 m ou1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "emotionless"  j ( i1 m ou1 sh @ n l @ s ) ((pos "OA%" ) ))
( "emotions"  n ( i1 m ou1 sh @ n z ) ((pos "Mj%" ) ))
( "emotive"  j ( i1 m ou1 t i v ) ((pos "OA%" ) ))
( "empale"  v ( i1 m p ei1 l ) ((pos "H2$" ) ))
( "empaled"  v ( i1 m p ei1 l d ) ((pos "Hc$" "Hd$" ) ))
( "empales"  v ( i1 m p ei1 l z ) ((pos "Ha$" ) ))
( "empaling"  v ( i1 m p ei1 l i ng ) ((pos "Hb$" ) ))
( "empanel"  v ( i1 m p a1 n l ) ((pos "H4$" ) (subcat )))
( "empanelled"  v ( i1 m p a1 n l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "empanelling"  v ( i1 m p a1 n @ l i ng ) ((pos "Hb$" ) (subcat )))
( "empanels"  v ( i1 m p a1 n l z ) ((pos "Ha$" ) (subcat )))
( "empathy"  n ( e1 m p @ th ii ) ((pos "L@%" ) ))
( "emperor"  n ( e1 m p @ r @ r ) ((pos "K6%" ) ))
( "emperors"  n ( e1 m p @ r @ z ) ((pos "Kj%" ) ))
( "emphases"  n ( e1 m f @ s ii z ) ((pos "Kj$" ) ))
( "emphasis"  n ( e1 m f @ s i s ) ((pos "Mi%" ) ))
( "emphasize"  v ( e1 m f @ s ai z ) ((pos "H2%" ) (subcat )))
( "emphasized"  v ( e1 m f @ s ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "emphasizes"  v ( e1 m f @ s ai z i z ) ((pos "Ha%" ) (subcat )))
( "emphasizing"  v ( e1 m f @ s ai z i ng ) ((pos "Hb%" ) (subcat )))
( "emphatic"  j ( i1 m f a1 t i k ) ((pos "OA%" ) ))
( "emphatically"  a ( i1 m f a1 t i k l ii ) ((pos "Pu%" ) ))
( "empire"  n ( e1 m p ai @ r ) ((pos "M6%" ) ))
( "empires"  n ( e1 m p ai @ z ) ((pos "Mj%" ) ))
( "empiric"  j ( i1 m p i1 r i k ) ((pos "OA$" ) ))
( "empirical"  j ( i1 m p i1 r i k l ) ((pos "OA%" ) ))
( "empirically"  a ( i1 m p i1 r i k l ii ) ((pos "Pu%" ) ))
( "empiricism"  n ( i1 m p i1 r i s i z @ m ) ((pos "L@%" ) ))
( "empiricist"  n ( i1 m p i1 r i s i s t ) ((pos "K6%" ) ))
( "empiricists"  n ( i1 m p i1 r i s i s t s ) ((pos "Kj%" ) ))
( "emplacement"  n ( i1 m p l ei1 s m @ n t ) ((pos "K6%" ) ))
( "emplacements"  n ( i1 m p l ei1 s m @ n t s ) ((pos "Kj%" ) ))
( "emplane"  v ( e1 m p l ei1 n ) ((pos "J2$" ) (subcat "2A" )))
( "emplaned"  v ( e1 m p l ei1 n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "emplanes"  v ( e1 m p l ei1 n z ) ((pos "Ja$" ) (subcat "2A" )))
( "emplaning"  v ( e1 m p l ei1 n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "employ"  v ( i1 m p l oi1 ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "employable"  j ( i1 m p l oi1 @ b l ) ((pos "OA%" ) ))
( "employed"  v ( i1 m p l oi1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "employee"  n ( e2 m p l oi ii1 ) ((pos "K6%" ) ))
( "employees"  n ( e2 m p l oi ii1 z ) ((pos "Kj%" ) ))
( "employer"  n ( i1 m p l oi1 @ r ) ((pos "K6%" ) ))
( "employers"  n ( i1 m p l oi1 @ z ) ((pos "Kj%" ) ))
( "employing"  v ( i1 m p l oi1 i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "employment"  n ( i1 m p l oi1 m @ n t ) ((pos "L@%" ) ))
( "employs"  v ( i1 m p l oi1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "emporium"  n ( i1 m p oo1 r i@ m ) ((pos "K6%" ) ))
( "emporiums"  n ( i1 m p oo1 r i@ m z ) ((pos "Kj%" ) ))
( "empower"  v ( i1 m p au1 @ r ) ((pos "H0%" ) (subcat )))
( "empowered"  v ( i1 m p au1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "empowering"  v ( i1 m p au1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "empowers"  v ( i1 m p au1 @ z ) ((pos "Ha%" ) (subcat )))
( "empress"  n ( e1 m p r i s ) ((pos "K7%" ) ))
( "empresses"  n ( e1 m p r i s i z ) ((pos "Kj%" ) ))
( "emptied"  v ( e1 m p t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "emptier"  j ( e1 m p t i@ r ) ((pos "Or%" ) ))
( "empties"  v ( e1 m p t i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "emptiest"  j ( e1 m p t i i s t ) ((pos "Os%" ) ))
( "emptiness"  n ( e1 m p t i n @ s ) ((pos "L@%" ) ))
( "empty"  v ( e1 m p t ii ) ((pos "J3%" "K8%" "OD%" ) (subcat "2A" "2C" "3A" "6A" )))
( "empty-handed"  j ( e2 m p t i - h a1 n d i d ) ((pos "OA%" ) ))
( "empty-headed"  j ( e2 m p t i - h e1 d i d ) ((pos "OA%" ) ))
( "emptying"  v ( e1 m p t i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "empurpled"  j ( i1 m p @@1 p l d ) ((pos "OA$" ) ))
( "empyrean"  n ( e2 m p ai r ii1 @ n ) ((pos "Ki$" "OA$" ) ))
( "emu"  n ( ii1 m y uu ) ((pos "K6%" ) ))
( "emulate"  v ( e1 m y u l ei t ) ((pos "H2%" ) (subcat )))
( "emulated"  v ( e1 m y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "emulates"  v ( e1 m y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "emulating"  v ( e1 m y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "emulation"  n ( e2 m y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "emulous"  j ( e1 m y u l @ s ) ((pos "OA$" ) ))
( "emulously"  a ( e1 m y u l @ s l ii ) ((pos "Pu$" ) ))
( "emulsified"  v ( i1 m uh1 l s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "emulsifies"  v ( i1 m uh1 l s i f ai z ) ((pos "Ha%" ) (subcat )))
( "emulsify"  v ( i1 m uh1 l s i f ai ) ((pos "H3%" ) (subcat )))
( "emulsifying"  v ( i1 m uh1 l s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "emulsion"  n ( i1 m uh1 l sh @ n ) ((pos "M6%" ) ))
( "emulsions"  n ( i1 m uh1 l sh @ n z ) ((pos "Mj$" ) ))
( "emus"  n ( ii1 m y uu z ) ((pos "Kj%" ) ))
( "en clair"  a ( o2 n - k l e@1 r ) ((pos "Pu$" ) ))
( "en famille"  a ( o2 n - f a m ii1 ) ((pos "Pu%" ) ))
( "en masse"  a ( o2 n - m a1 s ) ((pos "Pu%" ) ))
( "en route"  a ( o2 n - r uu1 t ) ((pos "Pu%" ) ))
( "enable"  v ( i1 n ei1 b l ) ((pos "H2%" ) (subcat )))
( "enabled"  v ( i1 n ei1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enables"  v ( i1 n ei1 b l z ) ((pos "Ha%" ) (subcat )))
( "enabling"  v ( i1 n ei1 b l i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "enact"  v ( i1 n a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "enacted"  v ( i1 n a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enacting"  v ( i1 n a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enactment"  n ( i1 n a1 k t m @ n t ) ((pos "M6%" ) ))
( "enactments"  n ( i1 n a1 k t m @ n t s ) ((pos "Mj%" ) ))
( "enacts"  v ( i1 n a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "enamel"  v ( i1 n a1 m l ) ((pos "H4%" "L@%" ) ))
( "enamelled"  v ( i1 n a1 m l d ) ((pos "Hc%" "Hd%" ) ))
( "enamelling"  v ( i1 n a1 m @ l i ng ) ((pos "Hb%" ) ))
( "enamels"  v ( i1 n a1 m l z ) ((pos "Ha%" ) ))
( "enamour"  v ( i1 n a1 m @ r ) ((pos "H0$" ) (subcat )))
( "enamoured"  v ( i1 n a1 m @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "enamouring"  v ( i1 n a1 m @ r i ng ) ((pos "Hb$" ) (subcat )))
( "enamours"  v ( i1 n a1 m @ z ) ((pos "Ha$" ) (subcat )))
( "enc"  n ( e1 n k ) ((pos "Y~%" ) ))
( "encamp"  v ( i1 n k a1 m p ) ((pos "J0%" ) (subcat "2A" )))
( "encamped"  v ( i1 n k a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "encamping"  v ( i1 n k a1 m p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "encampment"  n ( i1 n k a1 m p m @ n t ) ((pos "K6%" ) ))
( "encampments"  n ( i1 n k a1 m p m @ n t s ) ((pos "Kj%" ) ))
( "encamps"  v ( i1 n k a1 m p s ) ((pos "Ja%" ) (subcat "2A" )))
( "encase"  v ( i1 n k ei1 s ) ((pos "H2%" ) (subcat "6A" )))
( "encased"  v ( i1 n k ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "encases"  v ( i1 n k ei1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "encasing"  v ( i1 n k ei1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "encaustic"  j ( e1 n k oo1 s t i k ) ((pos "OA$" ) ))
( "encephalitis"  n ( e2 n k e f @ l ai1 t i s ) ((pos "L@%" ) ))
( "enchain"  v ( i1 n ch ei1 n ) ((pos "H0$" ) (subcat )))
( "enchained"  v ( i1 n ch ei1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "enchaining"  v ( i1 n ch ei1 n i ng ) ((pos "Hb$" ) (subcat )))
( "enchains"  v ( i1 n ch ei1 n z ) ((pos "Ha$" ) (subcat )))
( "enchant"  v ( i1 n ch aa1 n t ) ((pos "H0%" ) (subcat )))
( "enchanted"  v ( i1 n ch aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enchanter"  n ( i1 n ch aa1 n t @ r ) ((pos "K6%" ) ))
( "enchanters"  n ( i1 n ch aa1 n t @ z ) ((pos "Kj%" ) ))
( "enchanting"  v ( i1 n ch aa1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "enchantingly"  a ( i1 n ch aa1 n t i ng l ii ) ((pos "Pu%" ) ))
( "enchantment"  n ( i1 n ch aa1 n t m @ n t ) ((pos "M6%" ) ))
( "enchantments"  n ( i1 n ch aa1 n t m @ n t s ) ((pos "Mj%" ) ))
( "enchantress"  n ( i1 n ch aa1 n t r i s ) ((pos "K7%" ) ))
( "enchantresses"  n ( i1 n ch aa1 n t r i s i z ) ((pos "Kj%" ) ))
( "enchants"  v ( i1 n ch aa1 n t s ) ((pos "Ha%" ) (subcat )))
( "encircle"  v ( i1 n s @@1 k l ) ((pos "H2%" ) (subcat )))
( "encircled"  v ( i1 n s @@1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "encirclement"  n ( i1 n s @@1 k l m @ n t ) ((pos "K6%" ) ))
( "encirclements"  n ( i1 n s @@1 k l m @ n t s ) ((pos "Kj%" ) ))
( "encircles"  v ( i1 n s @@1 k l z ) ((pos "Ha%" ) (subcat )))
( "encircling"  v ( i1 n s @@1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "enclave"  n ( e1 n k l ei v ) ((pos "K6%" ) ))
( "enclaves"  n ( e1 n k l ei v z ) ((pos "Kj%" ) ))
( "enclose"  v ( i1 n k l ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "enclosed"  v ( i1 n k l ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "encloses"  v ( i1 n k l ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "enclosing"  v ( i1 n k l ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enclosure"  n ( i1 n k l ou1 zh @ r ) ((pos "M6%" ) ))
( "enclosures"  n ( i1 n k l ou1 zh @ z ) ((pos "Mj%" ) ))
( "encode"  v ( i1 n k ou1 d ) ((pos "H2%" ) (subcat )))
( "encoded"  v ( i1 n k ou1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "encodes"  v ( i1 n k ou1 d z ) ((pos "Ha%" ) (subcat )))
( "encoding"  v ( i1 n k ou1 d i ng ) ((pos "Hb%" ) (subcat )))
( "encomium"  n ( i1 n k ou1 m i@ m ) ((pos "K6$" ) ))
( "encomiums"  n ( i1 n k ou1 m i@ m z ) ((pos "Kj$" ) ))
( "encompass"  v ( i1 n k uh1 m p @ s ) ((pos "H1%" ) (subcat )))
( "encompassed"  v ( i1 n k uh1 m p @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "encompasses"  v ( i1 n k uh1 m p @ s i z ) ((pos "Ha%" ) (subcat )))
( "encompassing"  v ( i1 n k uh1 m p @ s i ng ) ((pos "Hb%" ) (subcat )))
( "encore"  v ( o1 ng k oo r ) ((pos "H2$" "K6%" "W-%" ) ))
( "encored"  v ( o1 ng k oo d ) ((pos "Hc$" "Hd$" ) ))
( "encores"  v ( o1 ng k oo z ) ((pos "Ha$" "Kj%" ) ))
( "encoring"  v ( o1 ng k oo r i ng ) ((pos "Hb$" ) ))
( "encounter"  v ( i1 n k au1 n t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "encountered"  v ( i1 n k au1 n t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "encountering"  v ( i1 n k au1 n t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "encounters"  v ( i1 n k au1 n t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "encourage"  v ( i1 n k uh1 r i jh ) ((pos "H2%" ) (subcat "6A" "14" )))
( "encouraged"  v ( i1 n k uh1 r i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "encouragement"  n ( i1 n k uh1 r i jh m @ n t ) ((pos "M6%" ) ))
( "encouragements"  n ( i1 n k uh1 r i jh m @ n t s ) ((pos "Mj%" ) ))
( "encourages"  v ( i1 n k uh1 r i jh i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "encouraging"  v ( i1 n k uh1 r i jh i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "encouragingly"  a ( i1 n k uh1 r i jh i ng l ii ) ((pos "Pu%" ) ))
( "encroach"  v ( i1 n k r ou1 ch ) ((pos "I1%" ) (subcat )))
( "encroached"  v ( i1 n k r ou1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "encroaches"  v ( i1 n k r ou1 ch i z ) ((pos "Ia%" ) (subcat )))
( "encroaching"  v ( i1 n k r ou1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "encroachment"  n ( i1 n k r ou1 ch m @ n t ) ((pos "M6%" ) ))
( "encroachments"  n ( i1 n k r ou1 ch m @ n t s ) ((pos "Mj%" ) ))
( "encrust"  v ( i1 n k r uh1 s t ) ((pos "J0$" ) (subcat "2A" "6A" )))
( "encrusted"  v ( i1 n k r uh1 s t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" "6A" )))
( "encrusting"  v ( i1 n k r uh1 s t i ng ) ((pos "Jb$" ) (subcat "2A" "6A" )))
( "encrusts"  v ( i1 n k r uh1 s t s ) ((pos "Ja$" ) (subcat "2A" "6A" )))
( "encumber"  v ( i1 n k uh1 m b @ r ) ((pos "H0$" ) (subcat "6A" )))
( "encumbered"  v ( i1 n k uh1 m b @ d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "encumbering"  v ( i1 n k uh1 m b @ r i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "encumbers"  v ( i1 n k uh1 m b @ z ) ((pos "Ha$" ) (subcat "6A" )))
( "encumbrance"  n ( i1 n k uh1 m b r @ n s ) ((pos "K6%" ) ))
( "encumbrances"  n ( i1 n k uh1 m b r @ n s i z ) ((pos "Kj%" ) ))
( "encyclical"  n ( i1 n s i1 k l i k l ) ((pos "K6%" "OA%" ) ))
( "encyclicals"  n ( i1 n s i1 k l i k l z ) ((pos "Kj%" ) ))
( "encyclopaedia"  n ( i1 n s ai2 k l @ p ii1 d i@ ) ((pos "K6%" ) ))
( "encyclopaedias"  n ( i1 n s ai2 k l @ p ii1 d i@ z ) ((pos "Kj%" ) ))
( "encyclopaedic"  j ( i1 n s ai2 k l @ p ii1 d i k ) ((pos "OA%" ) ))
( "encyclopedia"  n ( i1 n s ai2 k l @ p ii1 d i@ ) ((pos "K6%" ) ))
( "encyclopedias"  n ( i1 n s ai2 k l @ p ii1 d i@ z ) ((pos "Kj%" ) ))
( "encyclopedic"  j ( i1 n s ai2 k l @ p ii1 d i k ) ((pos "OA%" ) ))
( "end"  v ( e1 n d ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "end-all"  n ( e1 n d - oo l ) ((pos "Ki%" ) ))
( "endanger"  v ( i1 n d ei1 n jh @ r ) ((pos "H0%" ) (subcat )))
( "endangered"  v ( i1 n d ei1 n jh @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "endangering"  v ( i1 n d ei1 n jh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "endangers"  v ( i1 n d ei1 n jh @ z ) ((pos "Ha%" ) (subcat )))
( "endear"  v ( i1 n d i@1 r ) ((pos "H0%" ) (subcat )))
( "endeared"  v ( i1 n d i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "endearing"  v ( i1 n d i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "endearingly"  a ( i1 n d i@1 r i ng l ii ) ((pos "Pu%" ) ))
( "endearment"  n ( i1 n d i@1 m @ n t ) ((pos "M6%" ) ))
( "endearments"  n ( i1 n d i@1 m @ n t s ) ((pos "Mj%" ) ))
( "endears"  v ( i1 n d i@1 z ) ((pos "Ha%" ) (subcat )))
( "endeavour"  v ( i1 n d e1 v @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "endeavoured"  v ( i1 n d e1 v @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "endeavouring"  v ( i1 n d e1 v @ r i ng ) ((pos "Ib%" ) (subcat )))
( "endeavours"  v ( i1 n d e1 v @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "ended"  v ( e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "endemic"  n ( e1 n d e1 m i k ) ((pos "K6$" "OA%" ) ))
( "endemics"  n ( e1 n d e1 m i k s ) ((pos "Kj$" ) ))
( "ending"  v ( e1 n d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "endings"  n ( e1 n d i ng z ) ((pos "Kj%" ) ))
( "endive"  n ( e1 n d i v ) ((pos "M6%" ) ))
( "endives"  n ( e1 n d i v z ) ((pos "Mj$" ) ))
( "endless"  j ( e1 n d l i s ) ((pos "OA%" ) ))
( "endlessly"  a ( e1 n d l i s l ii ) ((pos "Pu%" ) ))
( "endorse"  v ( i1 n d oo1 s ) ((pos "H2%" ) (subcat )))
( "endorsed"  v ( i1 n d oo1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "endorsement"  n ( i1 n d oo1 s m @ n t ) ((pos "M6%" ) ))
( "endorsements"  n ( i1 n d oo1 s m @ n t s ) ((pos "Mj%" ) ))
( "endorses"  v ( i1 n d oo1 s i z ) ((pos "Ha%" ) (subcat )))
( "endorsing"  v ( i1 n d oo1 s i ng ) ((pos "Hb%" ) (subcat )))
( "endow"  v ( i1 n d au1 ) ((pos "H0%" ) (subcat "6A" )))
( "endowed"  v ( i1 n d au1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "endowing"  v ( i1 n d au1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "endowment"  n ( i1 n d au1 m @ n t ) ((pos "M6%" ) ))
( "endowments"  n ( i1 n d au1 m @ n t s ) ((pos "Mj%" ) ))
( "endows"  v ( i1 n d au1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "endpapers"  n ( e1 n d p ei p @ z ) ((pos "Kj%" ) ))
( "ends"  v ( e1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "endue"  v ( i1 n d y uu1 ) ((pos "H2$" ) (subcat )))
( "endued"  v ( i1 n d y uu1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "endues"  v ( i1 n d y uu1 z ) ((pos "Ha$" ) (subcat )))
( "enduing"  v ( i1 n d y uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "endurable"  j ( i1 n d y uu1 r @ b l ) ((pos "OA%" ) ))
( "endurance"  n ( i1 n d y u@1 r @ n s ) ((pos "L@%" ) ))
( "endure"  v ( i1 n d y u@1 r ) ((pos "J2%" ) (subcat "2A" "2C" "6A" "6D" )))
( "endured"  v ( i1 n d y u@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "6D" )))
( "endures"  v ( i1 n d y u@1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "6D" )))
( "enduring"  v ( i1 n d y u@1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "6D" )))
( "enduringly"  a ( i1 n d y u@1 r i ng l ii ) ((pos "Pu%" ) ))
( "endways"  a ( e1 n d w ei z ) ((pos "Pu%" ) ))
( "endwise"  a ( e1 n d w ai z ) ((pos "Pu%" ) ))
( "enema"  n ( e1 n i m @ ) ((pos "K6$" ) ))
( "enemas"  n ( e1 n i m @ z ) ((pos "Kj$" ) ))
( "enemies"  n ( e1 n @ m i z ) ((pos "Kj%" ) ))
( "enemy"  n ( e1 n @ m ii ) ((pos "K8%" ) ))
( "energetic"  j ( e2 n @ jh e1 t i k ) ((pos "OA%" ) ))
( "energetically"  a ( e2 n @ jh e1 t i k l ii ) ((pos "Pu%" ) ))
( "energies"  n ( e1 n @ jh i z ) ((pos "Mj%" ) ))
( "energy"  n ( e1 n @ jh ii ) ((pos "M8%" ) ))
( "enervate"  v ( e1 n @ v ei t ) ((pos "H2%" ) (subcat )))
( "enervated"  v ( e1 n @ v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enervates"  v ( e1 n @ v ei t s ) ((pos "Ha%" ) (subcat )))
( "enervating"  v ( e1 n @ v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "enfant terrible"  n ( o2 n f o n - t e r ii1 b l ) ((pos "Ki%" ) ))
( "enfants terribles"  n ( o2 n f o n - t e r ii1 b l ) ((pos "Kj%" ) ))
( "enfeeble"  v ( i1 n f ii1 b l ) ((pos "H2%" ) (subcat )))
( "enfeebled"  v ( i1 n f ii1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enfeebles"  v ( i1 n f ii1 b l z ) ((pos "Ha%" ) (subcat )))
( "enfeebling"  v ( i1 n f ii1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "enfold"  v ( i1 n f ou1 l d ) ((pos "H0%" ) (subcat "6A" )))
( "enfolded"  v ( i1 n f ou1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enfolding"  v ( i1 n f ou1 l d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enfolds"  v ( i1 n f ou1 l d z ) ((pos "Ha%" ) (subcat "6A" )))
( "enforce"  v ( i1 n f oo1 s ) ((pos "H2%" ) (subcat "6A" )))
( "enforceable"  j ( i1 n f oo1 s @ b l ) ((pos "OA%" ) ))
( "enforced"  v ( i1 n f oo1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enforcement"  n ( i1 n f oo1 s m @ n t ) ((pos "L@%" ) ))
( "enforces"  v ( i1 n f oo1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "enforcing"  v ( i1 n f oo1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enfranchise"  v ( i1 n f r a1 n ch ai z ) ((pos "H2%" ) (subcat )))
( "enfranchised"  v ( i1 n f r a1 n ch ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enfranchisement"  n ( i1 n f r a1 n ch i z m @ n t ) ((pos "K6%" ) ))
( "enfranchisements"  n ( i1 n f r a1 n ch i z m @ n t s ) ((pos "Kj%" ) ))
( "enfranchises"  v ( i1 n f r a1 n ch ai z i z ) ((pos "Ha%" ) (subcat )))
( "enfranchising"  v ( i1 n f r a1 n ch ai z i ng ) ((pos "Hb%" ) (subcat )))
( "engage"  v ( i1 n g ei1 jh ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "7A" "9" "14" "16B" )))
( "engaged"  v ( i1 n g ei1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "7A" "9" "14" "16B" )))
( "engagement"  n ( i1 n g ei1 jh m @ n t ) ((pos "M6%" ) ))
( "engagements"  n ( i1 n g ei1 jh m @ n t s ) ((pos "Mj%" ) ))
( "engages"  v ( i1 n g ei1 jh i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "7A" "9" "14" "16B" )))
( "engaging"  v ( i1 n g ei1 jh i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "7A" "9" "14" "16B" )))
( "engagingly"  a ( i1 n g ei1 jh i ng l ii ) ((pos "Pu%" ) ))
( "engender"  v ( i1 n jh e1 n d @ r ) ((pos "H0%" ) (subcat )))
( "engendered"  v ( i1 n jh e1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "engendering"  v ( i1 n jh e1 n d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "engenders"  v ( i1 n jh e1 n d @ z ) ((pos "Ha%" ) (subcat )))
( "engine"  n ( e1 n jh i n ) ((pos "K6%" ) ))
( "engine-driver"  n ( e1 n jh i n - d r ai v @ r ) ((pos "K6%" ) ))
( "engine-drivers"  n ( e1 n jh i n - d r ai v @ z ) ((pos "Kj%" ) ))
( "engineer"  v ( e2 n jh i n i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "engineered"  v ( e2 n jh i n i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "engineering"  v ( e2 n jh i n i@1 r i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "engineers"  v ( e2 n jh i n i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "engines"  n ( e1 n jh i n z ) ((pos "Kj%" ) ))
( "engraft"  v ( i1 n g r aa1 f t ) ((pos "H0$" ) (subcat "6A" )))
( "engrafted"  v ( i1 n g r aa1 f t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "engrafting"  v ( i1 n g r aa1 f t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "engrafts"  v ( i1 n g r aa1 f t s ) ((pos "Ha$" ) (subcat "6A" )))
( "engrave"  v ( i1 n g r ei1 v ) ((pos "H2%" ) (subcat "6A" )))
( "engraved"  v ( i1 n g r ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "engraver"  n ( i1 n g r ei1 v @ r ) ((pos "K6%" ) ))
( "engravers"  n ( i1 n g r ei1 v @ z ) ((pos "Kj%" ) ))
( "engraves"  v ( i1 n g r ei1 v z ) ((pos "Ha%" ) (subcat "6A" )))
( "engraving"  v ( i1 n g r ei1 v i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "engravings"  n ( i1 n g r ei1 v i ng z ) ((pos "Mj%" ) ))
( "engross"  v ( i1 n g r ou1 s ) ((pos "H1%" ) (subcat )))
( "engrossed"  v ( i1 n g r ou1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "engrosses"  v ( i1 n g r ou1 s i z ) ((pos "Ha%" ) (subcat )))
( "engrossing"  v ( i1 n g r ou1 s i ng ) ((pos "Hb%" ) (subcat )))
( "engulf"  v ( i1 n g uh1 l f ) ((pos "H0%" ) (subcat )))
( "engulfed"  v ( i1 n g uh1 l f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "engulfing"  v ( i1 n g uh1 l f i ng ) ((pos "Hb%" ) (subcat )))
( "engulfs"  v ( i1 n g uh1 l f s ) ((pos "Ha%" ) (subcat )))
( "enhance"  v ( i1 n h aa1 n s ) ((pos "H2%" ) (subcat )))
( "enhanced"  v ( i1 n h aa1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enhancement"  n ( i1 n h aa1 n s m @ n t ) ((pos "M6%" ) ))
( "enhancements"  n ( i1 n h aa1 n s m @ n t s ) ((pos "Mj%" ) ))
( "enhances"  v ( i1 n h aa1 n s i z ) ((pos "Ha%" ) (subcat )))
( "enhancing"  v ( i1 n h aa1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "enigma"  n ( i1 n i1 g m @ ) ((pos "K6%" ) ))
( "enigmas"  n ( i1 n i1 g m @ z ) ((pos "Kj%" ) ))
( "enigmatic"  j ( e2 n i g m a1 t i k ) ((pos "OA%" ) ))
( "enigmatically"  a ( e2 n i g m a1 t i k l ii ) ((pos "Pu%" ) ))
( "enjoin"  v ( i1 n jh oi1 n ) ((pos "H0%" ) (subcat "6A" "9" "14" )))
( "enjoined"  v ( i1 n jh oi1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" )))
( "enjoining"  v ( i1 n jh oi1 n i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" )))
( "enjoins"  v ( i1 n jh oi1 n z ) ((pos "Ha%" ) (subcat "6A" "9" "14" )))
( "enjoy"  v ( i1 n jh oi1 ) ((pos "H0%" ) (subcat "6A" )))
( "enjoyable"  j ( i1 n jh oi1 @ b l ) ((pos "OA%" ) ))
( "enjoyably"  a ( i1 n jh oi1 @ b l ii ) ((pos "Pu%" ) ))
( "enjoyed"  v ( i1 n jh oi1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enjoying"  v ( i1 n jh oi1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enjoyment"  n ( i1 n jh oi1 m @ n t ) ((pos "M6%" ) ))
( "enjoyments"  n ( i1 n jh oi1 m @ n t s ) ((pos "Mj%" ) ))
( "enjoys"  v ( i1 n jh oi1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "enkindle"  v ( i1 n k i1 n d l ) ((pos "H2$" ) (subcat )))
( "enkindled"  v ( i1 n k i1 n d l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "enkindles"  v ( i1 n k i1 n d l z ) ((pos "Ha$" ) (subcat )))
( "enkindling"  v ( i1 n k i1 n d l i ng ) ((pos "Hb$" ) (subcat )))
( "enlarge"  v ( i1 n l aa1 jh ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "enlarged"  v ( i1 n l aa1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "enlargement"  n ( i1 n l aa1 jh m @ n t ) ((pos "M6%" ) ))
( "enlargements"  n ( i1 n l aa1 jh m @ n t s ) ((pos "Mj%" ) ))
( "enlarges"  v ( i1 n l aa1 jh i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "enlarging"  v ( i1 n l aa1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "enlighten"  v ( i1 n l ai1 t n ) ((pos "H0%" ) (subcat "6A" )))
( "enlightened"  v ( i1 n l ai1 t n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "enlightening"  v ( i1 n l ai1 t n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enlightenment"  n ( i1 n l ai1 t n m @ n t ) ((pos "L@%" ) ))
( "enlightens"  v ( i1 n l ai1 t n z ) ((pos "Ha%" ) (subcat "6A" )))
( "enlist"  v ( i1 n l i1 s t ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "14" )))
( "enlisted"  v ( i1 n l i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "enlisting"  v ( i1 n l i1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "enlistment"  n ( i1 n l i1 s t m @ n t ) ((pos "M6%" ) ))
( "enlistments"  n ( i1 n l i1 s t m @ n t s ) ((pos "Mj$" ) ))
( "enlists"  v ( i1 n l i1 s t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" )))
( "enliven"  v ( i1 n l ai1 v n ) ((pos "H0%" ) (subcat )))
( "enlivened"  v ( i1 n l ai1 v n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enlivening"  v ( i1 n l ai1 v n i ng ) ((pos "Hb%" ) (subcat )))
( "enlivens"  v ( i1 n l ai1 v n z ) ((pos "Ha%" ) (subcat )))
( "enmesh"  v ( i1 n m e1 sh ) ((pos "H1%" ) (subcat "6A" )))
( "enmeshed"  v ( i1 n m e1 sh t ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "enmeshes"  v ( i1 n m e1 sh i z ) ((pos "Ha$" ) (subcat "6A" )))
( "enmeshing"  v ( i1 n m e1 sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enmities"  n ( e1 n m i t i z ) ((pos "Mj%" ) ))
( "enmity"  n ( e1 n m i t ii ) ((pos "M8%" ) ))
( "ennoble"  v ( i1 n ou1 b l ) ((pos "H2%" ) (subcat )))
( "ennobled"  v ( i1 n ou1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ennoblement"  n ( i1 n ou1 b l m @ n t ) ((pos "K6%" ) ))
( "ennoblements"  n ( i1 n ou1 b l m @ n t s ) ((pos "Kj$" ) ))
( "ennobles"  v ( i1 n ou1 b l z ) ((pos "Ha%" ) (subcat )))
( "ennobling"  v ( i1 n ou1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "ennui"  n ( o1 n w ii1 ) ((pos "M6%" ) ))
( "ennuis"  n ( o1 n w ii1 z ) ((pos "Mj$" ) ))
( "enormities"  n ( i1 n oo1 m i t i z ) ((pos "Mj%" ) ))
( "enormity"  n ( i1 n oo1 m i t ii ) ((pos "M8%" ) ))
( "enormous"  j ( i1 n oo1 m @ s ) ((pos "OA%" ) ))
( "enormously"  a ( i1 n oo1 m @ s l ii ) ((pos "Pu%" ) ))
( "enormousness"  n ( i1 n oo1 m @ s n @ s ) ((pos "L@$" ) ))
( "enough"  n ( i1 n uh1 f ) ((pos "L9*" "OA*" "Pu*" ) ))
( "enplane"  v ( e1 n p l ei1 n ) ((pos "J2$" ) ))
( "enplaned"  v ( e1 n p l ei1 n d ) ((pos "Jc$" "Jd$" ) ))
( "enplanes"  v ( e1 n p l ei1 n z ) ((pos "Ja$" ) ))
( "enplaning"  v ( e1 n p l ei1 n i ng ) ((pos "Jb$" ) ))
( "enquire"  v ( i1 n k w ai1 @ r ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "enquired"  v ( i1 n k w ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "enquirer"  n ( i1 n k w ai1 @ r @ r ) ((pos "K6%" ) ))
( "enquirers"  n ( i1 n k w ai1 @ r @ z ) ((pos "Kj%" ) ))
( "enquires"  v ( i1 n k w ai1 @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "enquiries"  n ( i1 n k w ai1 @ r i z ) ((pos "Mj%" ) ))
( "enquiring"  v ( i1 n k w ai1 @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "enquiringly"  a ( i1 n k w ai1 @ r i ng l ii ) ((pos "Pu%" ) ))
( "enquiry"  n ( i1 n k w ai1 @ r ii ) ((pos "M8%" ) ))
( "enrage"  v ( i1 n r ei1 jh ) ((pos "H2%" ) (subcat )))
( "enraged"  v ( i1 n r ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enrages"  v ( i1 n r ei1 jh i z ) ((pos "Ha%" ) (subcat )))
( "enraging"  v ( i1 n r ei1 jh i ng ) ((pos "Hb$" ) (subcat )))
( "enrapture"  v ( i1 n r a1 p ch @ r ) ((pos "H2%" ) (subcat )))
( "enraptured"  v ( i1 n r a1 p ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enraptures"  v ( i1 n r a1 p ch @ z ) ((pos "Ha%" ) (subcat )))
( "enrapturing"  v ( i1 n r a1 p ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "enrich"  v ( i1 n r i1 ch ) ((pos "H1%" ) (subcat "6A" )))
( "enriched"  v ( i1 n r i1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enriches"  v ( i1 n r i1 ch i z ) ((pos "Ha%" ) (subcat "6A" )))
( "enriching"  v ( i1 n r i1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enrichment"  n ( i1 n r i1 ch m @ n t ) ((pos "L@%" ) ))
( "enrol"  v ( i1 n r ou1 l ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "14" )))
( "enroled"  v ( i1 n r ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "enroling"  v ( i1 n r ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "enroll"  v ( i1 n r ou1 l ) ((pos "J0$" ) (subcat "2A" "2C" "6A" "14" )))
( "enrolled"  v ( i1 n r ou1 l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" "14" )))
( "enrolling"  v ( i1 n r ou1 l i ng ) ((pos "Jb$" ) (subcat "2A" "2C" "6A" "14" )))
( "enrolls"  v ( i1 n r ou1 l z ) ((pos "Ja$" ) (subcat "2A" "2C" "6A" "14" )))
( "enrolment"  n ( i1 n r ou1 l m @ n t ) ((pos "M6%" ) ))
( "enrolments"  n ( i1 n r ou1 l m @ n t s ) ((pos "Mj%" ) ))
( "enrols"  v ( i1 n r ou1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" )))
( "ensconce"  v ( i1 n s k o1 n s ) ((pos "H2%" ) (subcat )))
( "ensconced"  v ( i1 n s k o1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ensconces"  v ( i1 n s k o1 n s i z ) ((pos "Ha%" ) (subcat )))
( "ensconcing"  v ( i1 n s k o1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "ensemble"  n ( o1 n s o1 m b l ) ((pos "K6%" ) ))
( "ensembles"  n ( o1 n s o1 m b l z ) ((pos "Kj%" ) ))
( "enshrine"  v ( i1 n sh r ai1 n ) ((pos "H2%" ) (subcat "6A" )))
( "enshrined"  v ( i1 n sh r ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "enshrines"  v ( i1 n sh r ai1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "enshrining"  v ( i1 n sh r ai1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enshroud"  v ( i1 n sh r au1 d ) ((pos "H0$" ) (subcat )))
( "enshrouded"  v ( i1 n sh r au1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "enshrouding"  v ( i1 n sh r au1 d i ng ) ((pos "Hb$" ) (subcat )))
( "enshrouds"  v ( i1 n sh r au1 d z ) ((pos "Ha$" ) (subcat )))
( "ensign"  n ( e1 n s @ n ) ((pos "K6%" ) ))
( "ensigns"  n ( e1 n s @ n z ) ((pos "Kj%" ) ))
( "ensilage"  n ( e1 n s i l i jh ) ((pos "L@$" ) ))
( "enslave"  v ( i1 n s l ei1 v ) ((pos "H2%" ) (subcat )))
( "enslaved"  v ( i1 n s l ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enslavement"  n ( i1 n s l ei1 v m @ n t ) ((pos "K6%" ) ))
( "enslavements"  n ( i1 n s l ei1 v m @ n t s ) ((pos "Kj$" ) ))
( "enslaves"  v ( i1 n s l ei1 v z ) ((pos "Ha%" ) (subcat )))
( "enslaving"  v ( i1 n s l ei1 v i ng ) ((pos "Hb%" ) (subcat )))
( "ensnare"  v ( i1 n s n e@1 r ) ((pos "H2%" ) (subcat "6A" )))
( "ensnared"  v ( i1 n s n e@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "ensnares"  v ( i1 n s n e@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "ensnaring"  v ( i1 n s n e@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ensue"  v ( i1 n s y uu1 ) ((pos "I2%" ) (subcat "2A" )))
( "ensued"  v ( i1 n s y uu1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "ensues"  v ( i1 n s y uu1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "ensuing"  v ( i1 n s y uu1 i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "ensure"  v ( i1 n sh u@1 r ) ((pos "J2%" ) (subcat "3A" "9" "12A" "13A" )))
( "ensured"  v ( i1 n sh u@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "9" "12A" "13A" )))
( "ensures"  v ( i1 n sh u@1 z ) ((pos "Ja%" ) (subcat "3A" "9" "12A" "13A" )))
( "ensuring"  v ( i1 n sh u@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "9" "12A" "13A" )))
( "entail"  v ( i1 n t ei1 l ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "entailed"  v ( i1 n t ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "entailing"  v ( i1 n t ei1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "entails"  v ( i1 n t ei1 l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "entangle"  v ( i1 n t a1 ng g l ) ((pos "H2%" ) (subcat "6A" "14" )))
( "entangled"  v ( i1 n t a1 ng g l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "entanglement"  n ( i1 n t a1 ng g l m @ n t ) ((pos "M6%" ) ))
( "entanglements"  n ( i1 n t a1 ng g l m @ n t s ) ((pos "Mj%" ) ))
( "entangles"  v ( i1 n t a1 ng g l z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "entangling"  v ( i1 n t a1 ng g l i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "entente"  n ( o1 n t o1 n t ) ((pos "K6$" ) ))
( "entente cordiale"  n ( o1 n t o2 n t - k oo d i aa1 l ) ((pos "Ki$" ) ))
( "ententes"  n ( o1 n t o1 n t s ) ((pos "Kj$" ) ))
( "enter"  v ( e1 n t @ r ) ((pos "J0%" ) (subcat "2A" "3A" "6A" "14" )))
( "entered"  v ( e1 n t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "enteric"  j ( e1 n t e1 r i k ) ((pos "OA$" ) ))
( "entering"  v ( e1 n t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "enteritis"  n ( e2 n t @ r ai1 t i s ) ((pos "L@%" ) ))
( "enterprise"  n ( e1 n t @ p r ai z ) ((pos "M6%" ) ))
( "enterprises"  n ( e1 n t @ p r ai z i z ) ((pos "Mj%" ) ))
( "enterprising"  j ( e1 n t @ p r ai z i ng ) ((pos "OA%" ) ))
( "enterprisingly"  a ( e1 n t @ p r ai z i ng l ii ) ((pos "Pu%" ) ))
( "enters"  v ( e1 n t @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "14" )))
( "entertain"  v ( e2 n t @ t ei1 n ) ((pos "H0%" ) (subcat "2A" "6A" )))
( "entertained"  v ( e2 n t @ t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "entertainer"  n ( e2 n t @ t ei1 n @ r ) ((pos "K6%" ) ))
( "entertainers"  n ( e2 n t @ t ei1 n @ z ) ((pos "Kj%" ) ))
( "entertaining"  v ( e2 n t @ t ei1 n i ng ) ((pos "Hb%" "OA%" ) (subcat "2A" "6A" )))
( "entertainingly"  a ( e2 n t @ t ei1 n i ng l ii ) ((pos "Pu%" ) ))
( "entertainment"  n ( e2 n t @ t ei1 n m @ n t ) ((pos "M6%" ) ))
( "entertainments"  n ( e2 n t @ t ei1 n m @ n t s ) ((pos "Mj%" ) ))
( "entertains"  v ( e2 n t @ t ei1 n z ) ((pos "Ha%" ) (subcat "2A" "6A" )))
( "enthral"  v ( i1 n th r oo1 l ) ((pos "H4%" ) (subcat )))
( "enthrall"  v ( i1 n th r oo1 l ) ((pos "H0$" ) (subcat )))
( "enthralled"  v ( i1 n th r oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enthralling"  v ( i1 n th r oo1 l i ng ) ((pos "Hb%" ) (subcat )))
( "enthralls"  v ( i1 n th r oo1 l z ) ((pos "Ha$" ) (subcat )))
( "enthrals"  v ( i1 n th r oo1 l z ) ((pos "Ha%" ) (subcat )))
( "enthrone"  v ( i1 n th r ou1 n ) ((pos "H2%" ) (subcat )))
( "enthroned"  v ( i1 n th r ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enthronement"  n ( i1 n th r ou1 n m @ n t ) ((pos "K6%" ) ))
( "enthronements"  n ( i1 n th r ou1 n m @ n t s ) ((pos "Kj$" ) ))
( "enthrones"  v ( i1 n th r ou1 n z ) ((pos "Ha%" ) (subcat )))
( "enthroning"  v ( i1 n th r ou1 n i ng ) ((pos "Hb%" ) (subcat )))
( "enthuse"  v ( i1 n th y uu1 z ) ((pos "I2%" ) (subcat )))
( "enthused"  v ( i1 n th y uu1 z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "enthuses"  v ( i1 n th y uu1 z i z ) ((pos "Ia%" ) (subcat )))
( "enthusiasm"  n ( i1 n th y uu1 z i a z @ m ) ((pos "L@%" ) ))
( "enthusiast"  n ( i1 n th y uu1 z i a s t ) ((pos "K6%" ) ))
( "enthusiastic"  j ( i1 n th y uu2 z i a1 s t i k ) ((pos "OA%" ) ))
( "enthusiastically"  a ( i1 n th y uu2 z i a1 s t i k l ii ) ((pos "Pu%" ) ))
( "enthusiasts"  n ( i1 n th y uu1 z i a s t s ) ((pos "Kj%" ) ))
( "enthusing"  v ( i1 n th y uu1 z i ng ) ((pos "Ib%" ) (subcat )))
( "entice"  v ( i1 n t ai1 s ) ((pos "H2%" ) (subcat "6A" "15A" )))
( "enticed"  v ( i1 n t ai1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "enticement"  n ( i1 n t ai1 s m @ n t ) ((pos "M6%" ) ))
( "enticements"  n ( i1 n t ai1 s m @ n t s ) ((pos "Mj%" ) ))
( "entices"  v ( i1 n t ai1 s i z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "enticing"  v ( i1 n t ai1 s i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "entire"  j ( i1 n t ai1 @ r ) ((pos "OA%" ) ))
( "entirely"  a ( i1 n t ai1 @ l ii ) ((pos "Pu%" ) ))
( "entirety"  n ( i1 n t ai1 @ r i t ii ) ((pos "L@%" ) ))
( "entities"  n ( e1 n t i t i z ) ((pos "Mj%" ) ))
( "entitle"  v ( i1 n t ai1 t l ) ((pos "H2%" ) (subcat "14" )))
( "entitled"  v ( i1 n t ai1 t l d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "entitlement"  n ( i1 n t ai1 t l m @ n t ) ((pos "K6%" ) ))
( "entitlements"  n ( i1 n t ai1 t l m @ n t s ) ((pos "Kj%" ) ))
( "entitles"  v ( i1 n t ai1 t l z ) ((pos "Ha%" ) (subcat "14" )))
( "entitling"  v ( i1 n t ai1 t l i ng ) ((pos "Hb%" ) (subcat "14" )))
( "entity"  n ( e1 n t i t ii ) ((pos "M8%" ) ))
( "entomb"  v ( i1 n t uu1 m ) ((pos "H0%" ) (subcat )))
( "entombed"  v ( i1 n t uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "entombing"  v ( i1 n t uu1 m i ng ) ((pos "Hb%" ) (subcat )))
( "entombs"  v ( i1 n t uu1 m z ) ((pos "Ha%" ) (subcat )))
( "entomological"  j ( e2 n t @ m @ l o1 jh i k l ) ((pos "OA$" ) ))
( "entomologist"  n ( e2 n t @ m o1 l @ jh i s t ) ((pos "K6%" ) ))
( "entomologists"  n ( e2 n t @ m o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "entomology"  n ( e2 n t @ m o1 l @ jh ii ) ((pos "L@%" ) ))
( "entourage"  n ( o2 n t u r aa1 zh ) ((pos "Ki%" ) ))
( "entr'acte"  n ( o1 n t r a k t ) ((pos "K6$" ) ))
( "entr'actes"  n ( o1 n t r a k t s ) ((pos "Kj$" ) ))
( "entr_ee"  n ( o1 n t r ei ) ((pos "M6$" ) ))
( "entr_ees"  n ( o1 n t r ei z ) ((pos "Mj$" ) ))
( "entrails"  n ( e1 n t r ei l z ) ((pos "Kj%" ) ))
( "entrain"  v ( e1 n t r ei1 n ) ((pos "J0$" ) (subcat "2A" )))
( "entrained"  v ( e1 n t r ei1 n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "entraining"  v ( e1 n t r ei1 n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "entrains"  v ( e1 n t r ei1 n z ) ((pos "Ja$" ) (subcat "2A" )))
( "entrance"  n ( e1 n t r @ n s ) ((pos "M6%" ) ))
( "entrance"  v ( i1 n t r aa1 n s ) ((pos "H2%" ) (subcat )))
( "entrance-fee"  n ( e1 n t r @ n s - f ii ) ((pos "K6%" ) ))
( "entrance-fees"  n ( e1 n t r @ n s - f ii z ) ((pos "Kj%" ) ))
( "entrance-money"  n ( e1 n t r @ n s - m uh n ii ) ((pos "L@%" ) ))
( "entranced"  v ( i1 n t r aa1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "entrances"  n ( e1 n t r @ n s i z ) ((pos "Mj%" ) ))
( "entrances"  v ( i1 n t r aa1 n s i z ) ((pos "Ha%" ) (subcat )))
( "entrancing"  v ( i1 n t r aa1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "entrant"  n ( e1 n t r @ n t ) ((pos "K6%" ) ))
( "entrants"  n ( e1 n t r @ n t s ) ((pos "Kj%" ) ))
( "entrap"  v ( i1 n t r a1 p ) ((pos "H4%" ) (subcat )))
( "entrapped"  v ( i1 n t r a1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "entrapping"  v ( i1 n t r a1 p i ng ) ((pos "Hb%" ) (subcat )))
( "entraps"  v ( i1 n t r a1 p s ) ((pos "Ha%" ) (subcat )))
( "entreat"  v ( i1 n t r ii1 t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "entreated"  v ( i1 n t r ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "entreaties"  n ( i1 n t r ii1 t i z ) ((pos "Mj%" ) ))
( "entreating"  v ( i1 n t r ii1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "entreatingly"  a ( i1 n t r ii1 t i ng l ii ) ((pos "Pu%" ) ))
( "entreats"  v ( i1 n t r ii1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "entreaty"  n ( i1 n t r ii1 t ii ) ((pos "M8%" ) ))
( "entrench"  v ( i1 n t r e1 n ch ) ((pos "H1$" ) (subcat )))
( "entrenched"  v ( i1 n t r e1 n ch t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "entrenches"  v ( i1 n t r e1 n ch i z ) ((pos "Ha$" ) (subcat )))
( "entrenching"  v ( i1 n t r e1 n ch i ng ) ((pos "Hb$" ) (subcat )))
( "entrenchment"  n ( i1 n t r e1 n ch m @ n t ) ((pos "K6$" ) ))
( "entrenchments"  n ( i1 n t r e1 n ch m @ n t s ) ((pos "Kj$" ) ))
( "entrepot"  n ( o1 n t r @ p ou ) ((pos "K6$" ) ))
( "entrepots"  n ( o1 n t r @ p ou z ) ((pos "Kj$" ) ))
( "entrepreneur"  n ( o2 n t r @ p r @ n @@1 r ) ((pos "K6%" ) ))
( "entrepreneurial"  j ( o2 n t r @ p r @ n @@1 r i@ l ) ((pos "OA%" ) ))
( "entrepreneurs"  n ( o2 n t r @ p r @ n @@1 z ) ((pos "Kj%" ) ))
( "entries"  n ( e1 n t r i z ) ((pos "Mj%" ) ))
( "entrust"  v ( i1 n t r uh1 s t ) ((pos "H0%" ) (subcat )))
( "entrusted"  v ( i1 n t r uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "entrusting"  v ( i1 n t r uh1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "entrusts"  v ( i1 n t r uh1 s t s ) ((pos "Ha%" ) (subcat )))
( "entry"  n ( e1 n t r ii ) ((pos "M8%" ) ))
( "entwine"  v ( i1 n t w ai1 n ) ((pos "H2%" ) (subcat "6A" )))
( "entwined"  v ( i1 n t w ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "entwines"  v ( i1 n t w ai1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "entwining"  v ( i1 n t w ai1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enumerate"  v ( i1 n y uu1 m @ r ei t ) ((pos "H2%" ) (subcat )))
( "enumerated"  v ( i1 n y uu1 m @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "enumerates"  v ( i1 n y uu1 m @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "enumerating"  v ( i1 n y uu1 m @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "enumeration"  n ( i1 n y uu2 m @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "enumerations"  n ( i1 n y uu2 m @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "enunciate"  v ( i1 n uh1 n s i ei t ) ((pos "J2%" ) (subcat "2A" )))
( "enunciated"  v ( i1 n uh1 n s i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "enunciates"  v ( i1 n uh1 n s i ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "enunciating"  v ( i1 n uh1 n s i ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "enunciation"  n ( i1 n uh2 n s i ei1 sh @ n ) ((pos "K6%" ) ))
( "enunciations"  n ( i1 n uh2 n s i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "envelop"  v ( i1 n v e1 l @ p ) ((pos "H0%" ) (subcat "6A" )))
( "envelope"  n ( e1 n v @ l ou p ) ((pos "K6%" ) ))
( "enveloped"  v ( i1 n v e1 l @ p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "envelopes"  n ( e1 n v @ l ou p s ) ((pos "Kj%" ) ))
( "enveloping"  v ( i1 n v e1 l @ p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "envelopment"  n ( i1 n v e1 l @ p m @ n t ) ((pos "K6$" ) ))
( "envelopments"  n ( i1 n v e1 l @ p m @ n t s ) ((pos "Kj$" ) ))
( "envelops"  v ( i1 n v e1 l @ p s ) ((pos "Ha%" ) (subcat "6A" )))
( "envenom"  v ( i1 n v e1 n @ m ) ((pos "H0$" ) (subcat )))
( "envenomed"  v ( i1 n v e1 n @ m d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "envenoming"  v ( i1 n v e1 n @ m i ng ) ((pos "Hb$" ) (subcat )))
( "envenoms"  v ( i1 n v e1 n @ m z ) ((pos "Ha$" ) (subcat )))
( "enviable"  j ( e1 n v i@ b l ) ((pos "OA%" ) ))
( "envied"  v ( e1 n v i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "envies"  v ( e1 n v i z ) ((pos "Ha%" ) (subcat "6A" )))
( "envious"  j ( e1 n v i@ s ) ((pos "OA%" ) ))
( "enviously"  a ( e1 n v i@ s l ii ) ((pos "Pu%" ) ))
( "environ"  v ( i1 n v ai1 @ r @ n ) ((pos "H0$" ) (subcat )))
( "environed"  v ( i1 n v ai1 @ r @ n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "environing"  v ( i1 n v ai1 @ r @ n i ng ) ((pos "Hb$" ) (subcat )))
( "environment"  n ( i1 n v ai1 @ r @ n m @ n t ) ((pos "M6%" ) ))
( "environmental"  j ( i1 n v ai2 @ r @ n m e1 n t l ) ((pos "OA%" ) ))
( "environmentally"  a ( i1 n v ai2 @ r @ n m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "environments"  n ( i1 n v ai1 @ r @ n m @ n t s ) ((pos "Mj%" ) ))
( "environs"  v ( i1 n v ai1 @ r @ n z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "envisage"  v ( i1 n v i1 z i jh ) ((pos "H2%" ) (subcat )))
( "envisaged"  v ( i1 n v i1 z i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "envisages"  v ( i1 n v i1 z i jh i z ) ((pos "Ha%" ) (subcat )))
( "envisaging"  v ( i1 n v i1 z i jh i ng ) ((pos "Hb%" ) (subcat )))
( "envoi"  n ( e1 n v oi ) ((pos "K6$" ) ))
( "envois"  n ( e1 n v oi z ) ((pos "Kj$" ) ))
( "envoy"  n ( e1 n v oi ) ((pos "K6%" ) ))
( "envoys"  n ( e1 n v oi z ) ((pos "Kj%" ) ))
( "envy"  v ( e1 n v ii ) ((pos "H3%" "L@%" ) (subcat "6A" )))
( "envying"  v ( e1 n v i i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "enwrap"  v ( i1 n r a1 p ) ((pos "H4$" ) ))
( "enwrapped"  v ( i1 n r a1 p t ) ((pos "Hc%" "Hd%" ) ))
( "enwrapping"  v ( i1 n r a1 p i ng ) ((pos "Hb$" ) ))
( "enwraps"  v ( i1 n r a1 p s ) ((pos "Ha$" ) ))
( "enzyme"  n ( e1 n z ai m ) ((pos "K6%" ) ))
( "enzymes"  n ( e1 n z ai m z ) ((pos "Kj%" ) ))
( "eon"  n ( ii1 @ n ) ((pos "K6$" ) ))
( "eons"  n ( ii1 @ n z ) ((pos "Kj$" ) ))
( "epaulet"  n ( e1 p @ l e t ) ((pos "K6$" ) ))
( "epaulets"  n ( e1 p @ l e t s ) ((pos "Kj$" ) ))
( "epaulette"  n ( e1 p @ l e t ) ((pos "K6$" ) ))
( "epaulettes"  n ( e1 p @ l e t s ) ((pos "Kj$" ) ))
( "ephemeral"  j ( i1 f e1 m @ r @ l ) ((pos "OA%" ) ))
( "epic"  n ( e1 p i k ) ((pos "K6%" "OA%" ) ))
( "epicentre"  n ( e1 p i s e n t @ r ) ((pos "K6%" ) ))
( "epicentres"  n ( e1 p i s e n t @ z ) ((pos "Kj$" ) ))
( "epics"  n ( e1 p i k s ) ((pos "Kj%" ) ))
( "epicure"  n ( e1 p i k y u@ r ) ((pos "K6$" ) ))
( "epicurean"  n ( e2 p i k y u r ii1 @ n ) ((pos "K6$" "OA$" ) ))
( "epicureans"  n ( e2 p i k y u r ii1 @ n z ) ((pos "Kj$" ) ))
( "epicures"  n ( e1 p i k y u@ z ) ((pos "Kj$" ) ))
( "epidemic"  n ( e2 p i d e1 m i k ) ((pos "K6%" "OA%" ) ))
( "epidemics"  n ( e2 p i d e1 m i k s ) ((pos "Kj%" ) ))
( "epidemiologist"  n ( e2 p i d ii m i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "epidemiologists"  n ( e2 p i d ii m i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "epidemiology"  n ( e2 p i d ii m i o1 l @ jh ii ) ((pos "L@%" ) ))
( "epidermis"  n ( e2 p i d @@1 m i s ) ((pos "L@%" ) ))
( "epidiascope"  n ( e2 p i d ai1 @ s k ou p ) ((pos "K6$" ) ))
( "epidiascopes"  n ( e2 p i d ai1 @ s k ou p s ) ((pos "Kj$" ) ))
( "epiglottis"  n ( e2 p i g l o1 t i s ) ((pos "K7$" ) ))
( "epiglottises"  n ( e2 p i g l o1 t i s i z ) ((pos "Kj$" ) ))
( "epigram"  n ( e1 p i g r a m ) ((pos "K6%" ) ))
( "epigrammatic"  j ( e2 p i g r @ m a1 t i k ) ((pos "OA%" ) ))
( "epigrams"  n ( e1 p i g r a m z ) ((pos "Kj%" ) ))
( "epilepsy"  n ( e1 p i l e p s ii ) ((pos "L@%" ) ))
( "epileptic"  n ( e2 p i l e1 p t i k ) ((pos "K6%" "OA%" ) ))
( "epileptics"  n ( e2 p i l e1 p t i k s ) ((pos "Kj%" ) ))
( "epilogue"  n ( e1 p i l o g ) ((pos "K6%" ) ))
( "epilogues"  n ( e1 p i l o g z ) ((pos "Kj%" ) ))
( "episcopal"  j ( i1 p i1 s k @ p l ) ((pos "OA%" ) ))
( "episcopalian"  n ( i1 p i2 s k @ p ei1 l i@ n ) ((pos "K6$" "OA%" ) ))
( "episcopalians"  n ( i1 p i2 s k @ p ei1 l i@ n z ) ((pos "Kj$" ) ))
( "episode"  n ( e1 p i s ou d ) ((pos "K6%" ) ))
( "episodes"  n ( e1 p i s ou d z ) ((pos "Kj%" ) ))
( "episodic"  j ( e2 p i s o1 d i k ) ((pos "OA$" ) ))
( "epistle"  n ( i1 p i1 s l ) ((pos "K6%" ) ))
( "epistles"  n ( i1 p i1 s l z ) ((pos "Kj%" ) ))
( "epistolary"  j ( i1 p i1 s t @ l @ r ii ) ((pos "OA$" ) ))
( "epitaph"  n ( e1 p i t aa f ) ((pos "K6%" ) ))
( "epitaphs"  n ( e1 p i t aa f s ) ((pos "Kj%" ) ))
( "epithet"  n ( e1 p i th e t ) ((pos "K6%" ) ))
( "epithets"  n ( e1 p i th e t s ) ((pos "Kj%" ) ))
( "epitome"  n ( i1 p i1 t @ m ii ) ((pos "K6%" ) ))
( "epitomes"  n ( i1 p i1 t @ m i z ) ((pos "Kj$" ) ))
( "epitomize"  v ( i1 p i1 t @ m ai z ) ((pos "H2%" ) (subcat )))
( "epitomized"  v ( i1 p i1 t @ m ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "epitomizes"  v ( i1 p i1 t @ m ai z i z ) ((pos "Ha%" ) (subcat )))
( "epitomizing"  v ( i1 p i1 t @ m ai z i ng ) ((pos "Hb%" ) (subcat )))
( "epoch"  n ( ii1 p o k ) ((pos "K6%" ) ))
( "epoch-making"  j ( ii1 p o k - m ei k i ng ) ((pos "OA%" ) ))
( "epochs"  n ( ii1 p o k s ) ((pos "Kj%" ) ))
( "equable"  j ( e1 k w @ b l ) ((pos "OA%" ) ))
( "equably"  a ( e1 k w @ b l ii ) ((pos "Pu%" ) ))
( "equal"  v ( ii1 k w @ l ) ((pos "H4%" "K6%" "OA%" ) (subcat "6A" )))
( "equalitarian"  n ( i1 k w o2 l i t e@1 r i@ n ) ((pos "K6$" ) ))
( "equalitarians"  n ( i1 k w o2 l i t e@1 r i@ n z ) ((pos "Kj$" ) ))
( "equality"  n ( i1 k w o1 l i t ii ) ((pos "L@%" ) ))
( "equalization"  n ( ii2 k w @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "equalizations"  n ( ii2 k w @ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "equalize"  v ( ii1 k w @ l ai z ) ((pos "H2%" ) (subcat )))
( "equalized"  v ( ii1 k w @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "equalizer"  n ( ii1 k w @ l ai z @ r ) ((pos "K6%" ) ))
( "equalizers"  n ( ii1 k w @ l ai z @ z ) ((pos "Kj%" ) ))
( "equalizes"  v ( ii1 k w @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "equalizing"  v ( ii1 k w @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "equalled"  v ( ii1 k w @ l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "equalling"  v ( ii1 k w @ l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "equally"  a ( ii1 k w @ l ii ) ((pos "Pu%" ) ))
( "equals"  v ( ii1 k w @ l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "equanimity"  n ( e2 k w @ n i1 m i t ii ) ((pos "L@%" ) ))
( "equate"  v ( i1 k w ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "equated"  v ( i1 k w ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "equates"  v ( i1 k w ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "equating"  v ( i1 k w ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "equation"  n ( i1 k w ei1 zh n ) ((pos "M6%" ) ))
( "equations"  n ( i1 k w ei1 zh n z ) ((pos "Mj%" ) ))
( "equator"  n ( i1 k w ei1 t @ r ) ((pos "K6%" ) ))
( "equatorial"  j ( e2 k w @ t oo1 r i@ l ) ((pos "OA%" ) ))
( "equators"  n ( i1 k w ei1 t @ z ) ((pos "Kj$" ) ))
( "equerries"  n ( i1 k w e1 r i z ) ((pos "Kj$" ) ))
( "equerry"  n ( i1 k w e1 r ii ) ((pos "K8$" ) ))
( "equestrian"  n ( i1 k w e1 s t r i@ n ) ((pos "K6%" "OA%" ) ))
( "equestrians"  n ( i1 k w e1 s t r i@ n z ) ((pos "Kj%" ) ))
( "equidistant"  j ( ii2 k w i d i1 s t @ n t ) ((pos "OA%" ) ))
( "equilateral"  j ( ii2 k w i l a1 t @ r @ l ) ((pos "OA%" ) ))
( "equilibrium"  n ( ii2 k w i l i1 b r i@ m ) ((pos "L@%" ) ))
( "equine"  j ( e1 k w ai n ) ((pos "OA%" ) ))
( "equinoctial"  j ( ii2 k w i n o1 k sh l ) ((pos "OA$" ) ))
( "equinox"  n ( ii1 k w i n o k s ) ((pos "K7%" ) ))
( "equinoxes"  n ( ii1 k w i n o k s i z ) ((pos "Kj$" ) ))
( "equip"  v ( i1 k w i1 p ) ((pos "H4%" ) (subcat "6A" )))
( "equipage"  n ( e1 k w i p i jh ) ((pos "K6$" ) ))
( "equipages"  n ( e1 k w i p i jh i z ) ((pos "Kj$" ) ))
( "equipment"  n ( i1 k w i1 p m @ n t ) ((pos "L@%" ) ))
( "equipoise"  n ( e1 k w i p oi z ) ((pos "M6$" ) ))
( "equipoises"  n ( e1 k w i p oi z i z ) ((pos "Mj$" ) ))
( "equipped"  v ( i1 k w i1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "equipping"  v ( i1 k w i1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "equips"  v ( i1 k w i1 p s ) ((pos "Ha%" ) (subcat "6A" )))
( "equitable"  j ( e1 k w i t @ b l ) ((pos "OA%" ) ))
( "equitably"  a ( e1 k w i t @ b l ii ) ((pos "Pu%" ) ))
( "equities"  n ( e1 k w i t i z ) ((pos "Mj%" ) ))
( "equity"  n ( e1 k w i t ii ) ((pos "M8%" ) ))
( "equivalence"  n ( i1 k w i1 v @ l @ n s ) ((pos "M6%" ) ))
( "equivalences"  n ( i1 k w i1 v @ l @ n s i z ) ((pos "Mj$" ) ))
( "equivalent"  n ( i1 k w i1 v @ l @ n t ) ((pos "K6%" "OA%" ) ))
( "equivalents"  n ( i1 k w i1 v @ l @ n t s ) ((pos "Kj%" ) ))
( "equivocal"  j ( i1 k w i1 v @ k l ) ((pos "OA%" ) ))
( "equivocation"  n ( i1 k w i2 v @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "equivocations"  n ( i1 k w i2 v @ k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "era"  n ( i@1 r @ ) ((pos "K6%" ) ))
( "eradicate"  v ( i1 r a1 d i k ei t ) ((pos "H2%" ) (subcat )))
( "eradicated"  v ( i1 r a1 d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eradicates"  v ( i1 r a1 d i k ei t s ) ((pos "Ha%" ) (subcat )))
( "eradicating"  v ( i1 r a1 d i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "eradication"  n ( i1 r a2 d i k ei1 sh @ n ) ((pos "K6%" ) ))
( "eradications"  n ( i1 r a2 d i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "eras"  n ( i@1 r @ z ) ((pos "Kj$" ) ))
( "erase"  v ( i1 r ei1 z ) ((pos "H2%" ) (subcat )))
( "erased"  v ( i1 r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eraser"  n ( i1 r ei1 z @ r ) ((pos "K6%" ) ))
( "erasers"  n ( i1 r ei1 z @ z ) ((pos "Kj%" ) ))
( "erases"  v ( i1 r ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "erasing"  v ( i1 r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "erasure"  n ( i1 r ei1 zh @ r ) ((pos "M6%" ) ))
( "erasures"  n ( i1 r ei1 zh @ z ) ((pos "Mj%" ) ))
( "ere"  a ( e@ r ) ((pos "Pu$" "T-$" ) ))
( "erect"  v ( i1 r e1 k t ) ((pos "H0%" "OA%" ) (subcat )))
( "erected"  v ( i1 r e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "erectile"  j ( i1 r e1 k t ai l ) ((pos "OA$" ) ))
( "erecting"  v ( i1 r e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "erection"  n ( i1 r e1 k sh @ n ) ((pos "M6%" ) ))
( "erections"  n ( i1 r e1 k sh @ n z ) ((pos "Mj%" ) ))
( "erectly"  a ( i1 r e1 k t l ii ) ((pos "Pu%" ) ))
( "erectness"  n ( i1 r e1 k t n @ s ) ((pos "L@%" ) ))
( "erects"  v ( i1 r e1 k t s ) ((pos "Ha%" ) (subcat )))
( "eremite"  n ( e1 r i m ai t ) ((pos "K6$" ) ))
( "eremites"  n ( e1 r i m ai t s ) ((pos "Kj$" ) ))
( "erg"  n ( @@1 g ) ((pos "K6$" ) ))
( "ergo"  a ( @@1 g ou ) ((pos "Pu$" ) ))
( "ergonomics"  n ( @@2 g @ n o1 m i k s ) ((pos "Lk%" ) ))
( "ergs"  n ( @@1 g z ) ((pos "Kj$" ) ))
( "ermine"  n ( @@1 m i n ) ((pos "L@$" ) ))
( "erode"  v ( i1 r ou1 d ) ((pos "H2%" ) (subcat )))
( "eroded"  v ( i1 r ou1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "erodes"  v ( i1 r ou1 d z ) ((pos "Ha%" ) (subcat )))
( "eroding"  v ( i1 r ou1 d i ng ) ((pos "Hb%" ) (subcat )))
( "erogenous"  j ( i1 r o1 jh @ n @ s ) ((pos "OA%" ) ))
( "erosion"  n ( i1 r ou1 zh n ) ((pos "L@%" ) ))
( "erosive"  j ( i1 r ou1 s i v ) ((pos "OA$" ) ))
( "erotic"  j ( i1 r o1 t i k ) ((pos "OA%" ) ))
( "erotica"  n ( i1 r o1 t i k @ ) ((pos "Kj%" ) ))
( "eroticism"  n ( i1 r o1 t i s i z @ m ) ((pos "L@%" ) ))
( "err"  v ( @@1 r ) ((pos "I0%" ) (subcat "2A" )))
( "errand"  n ( e1 r @ n d ) ((pos "K6%" ) ))
( "errands"  n ( e1 r @ n d z ) ((pos "Kj%" ) ))
( "errant"  j ( e1 r @ n t ) ((pos "OA%" ) ))
( "errata"  n ( e1 r aa1 t @ ) ((pos "Kj%" ) ))
( "erratic"  j ( i1 r a1 t i k ) ((pos "OA%" ) ))
( "erratically"  a ( i1 r a1 t i k l ii ) ((pos "Pu%" ) ))
( "erratum"  n ( e1 r aa1 t @ m ) ((pos "Ki%" ) ))
( "erred"  v ( @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "erring"  v ( @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "erroneous"  j ( i1 r ou1 n i@ s ) ((pos "OA%" ) ))
( "erroneously"  a ( i1 r ou1 n i@ s l ii ) ((pos "Pu%" ) ))
( "error"  n ( e1 r @ r ) ((pos "M6%" ) ))
( "errors"  n ( e1 r @ z ) ((pos "Mj%" ) ))
( "errs"  v ( @@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "eructation"  n ( ii2 r uh k t ei1 sh @ n ) ((pos "M6$" ) ))
( "eructations"  n ( ii2 r uh k t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "erudite"  j ( e1 r uu d ai t ) ((pos "OA%" ) ))
( "eruditely"  a ( e1 r uu d ai t l ii ) ((pos "Pu%" ) ))
( "erudition"  n ( e2 r uu d i1 sh @ n ) ((pos "L@%" ) ))
( "erupt"  v ( i1 r uh1 p t ) ((pos "I0%" ) (subcat )))
( "erupted"  v ( i1 r uh1 p t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "erupting"  v ( i1 r uh1 p t i ng ) ((pos "Ib%" ) (subcat )))
( "eruption"  n ( i1 r uh1 p sh @ n ) ((pos "M6%" ) ))
( "eruptions"  n ( i1 r uh1 p sh @ n z ) ((pos "Mj%" ) ))
( "erupts"  v ( i1 r uh1 p t s ) ((pos "Ia%" ) (subcat )))
( "erysipelas"  n ( e2 r i s i1 p i l @ s ) ((pos "L@$" ) ))
( "escalate"  v ( e1 s k @ l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "escalated"  v ( e1 s k @ l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "escalates"  v ( e1 s k @ l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "escalating"  v ( e1 s k @ l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "escalation"  n ( e2 s k @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "escalations"  n ( e2 s k @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "escalator"  n ( e1 s k @ l ei t @ r ) ((pos "K6%" ) ))
( "escalators"  n ( e1 s k @ l ei t @ z ) ((pos "Kj%" ) ))
( "escalope"  n ( e1 s k @ l ou p ) ((pos "K6%" ) ))
( "escalopes"  n ( e1 s k @ l ou p s ) ((pos "Kj%" ) ))
( "escapade"  n ( e2 s k @ p ei1 d ) ((pos "K6%" ) ))
( "escapades"  n ( e2 s k @ p ei1 d z ) ((pos "Kj%" ) ))
( "escape"  v ( i1 s k ei1 p ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "escaped"  v ( i1 s k ei1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "escapee"  n ( i1 s k ei2 p ii1 ) ((pos "K6%" ) ))
( "escapees"  n ( i1 s k ei2 p ii1 z ) ((pos "Kj%" ) ))
( "escapement"  n ( i1 s k ei1 p m @ n t ) ((pos "K6$" ) ))
( "escapements"  n ( i1 s k ei1 p m @ n t s ) ((pos "Kj$" ) ))
( "escapes"  v ( i1 s k ei1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "escaping"  v ( i1 s k ei1 p i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "escapism"  n ( i1 s k ei1 p i z @ m ) ((pos "L@%" ) ))
( "escapist"  n ( i1 s k ei1 p i s t ) ((pos "K6%" ) ))
( "escapists"  n ( i1 s k ei1 p i s t s ) ((pos "Kj%" ) ))
( "escapologist"  n ( e2 s k @ p o1 l @ jh i s t ) ((pos "K6%" ) ))
( "escapologists"  n ( e2 s k @ p o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "escarpment"  n ( i1 s k aa1 p m @ n t ) ((pos "K6%" ) ))
( "escarpments"  n ( i1 s k aa1 p m @ n t s ) ((pos "Kj%" ) ))
( "eschatology"  n ( e2 s k @ t o1 l @ jh ii ) ((pos "L@$" ) ))
( "eschew"  v ( i1 s ch uu1 ) ((pos "H0%" ) (subcat )))
( "eschewed"  v ( i1 s ch uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eschewing"  v ( i1 s ch uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "eschews"  v ( i1 s ch uu1 z ) ((pos "Ha%" ) (subcat )))
( "escort"  n ( e1 s k oo t ) ((pos "K6%" ) ))
( "escort"  v ( i1 s k oo1 t ) ((pos "H0%" ) (subcat "6A" )))
( "escorted"  v ( i1 s k oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "escorting"  v ( i1 s k oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "escorts"  n ( e1 s k oo t s ) ((pos "Kj%" ) ))
( "escorts"  v ( i1 s k oo1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "escritoire"  n ( e2 s k r i t w aa1 r ) ((pos "K6$" ) ))
( "escritoires"  n ( e2 s k r i t w aa1 z ) ((pos "Kj$" ) ))
( "escudo"  n ( i1 s k y uu1 d ou ) ((pos "K6$" ) ))
( "escudos"  n ( i1 s k y uu1 d ou z ) ((pos "Kj$" ) ))
( "escutcheon"  n ( i1 s k uh1 ch @ n ) ((pos "K6$" ) ))
( "escutcheons"  n ( i1 s k uh1 ch @ n z ) ((pos "Kj$" ) ))
( "esophagus"  n ( ii1 s o1 f @ g @ s ) ((pos "K7$" ) ))
( "esophaguses"  n ( ii1 s o1 f @ g @ s i z ) ((pos "Kj$" ) ))
( "esoteric"  j ( e2 s ou t e1 r i k ) ((pos "OA%" ) ))
( "espalier"  n ( i1 s p a1 l i ei ) ((pos "K6$" ) ))
( "espaliers"  n ( i1 s p a1 l i ei z ) ((pos "Kj$" ) ))
( "especial"  j ( i1 s p e1 sh l ) ((pos "OA%" ) ))
( "especially"  a ( i1 s p e1 sh @ l ii ) ((pos "Pu%" ) ))
( "espied"  v ( i1 s p ai1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "espies"  v ( i1 s p ai1 z ) ((pos "Ha%" ) (subcat )))
( "espionage"  n ( e1 s p i@ n aa zh ) ((pos "L@%" ) ))
( "esplanade"  n ( e2 s p l @ n ei1 d ) ((pos "K6%" ) ))
( "esplanades"  n ( e2 s p l @ n ei1 d z ) ((pos "Kj$" ) ))
( "espousal"  n ( i1 s p au1 z l ) ((pos "K6%" ) ))
( "espousals"  n ( i1 s p au1 z l z ) ((pos "Kj$" ) ))
( "espouse"  v ( i1 s p au1 z ) ((pos "H2%" ) (subcat )))
( "espoused"  v ( i1 s p au1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "espouses"  v ( i1 s p au1 z i z ) ((pos "Ha%" ) (subcat )))
( "espousing"  v ( i1 s p au1 z i ng ) ((pos "Hb%" ) (subcat )))
( "espresso"  n ( e1 s p r e1 s ou ) ((pos "K6%" ) ))
( "espressos"  n ( e1 s p r e1 s ou z ) ((pos "Kj$" ) ))
( "esprit"  n ( e1 s p r ii1 ) ((pos "L@%" ) ))
( "esprit de corps"  n ( e1 s p r ii2 - d @ - k oo1 r ) ((pos "Ki$" ) ))
( "espy"  v ( i1 s p ai1 ) ((pos "H3%" ) (subcat )))
( "espying"  v ( i1 s p ai1 i ng ) ((pos "Hb%" ) (subcat )))
( "essay"  n ( e1 s ei ) ((pos "K6%" ) ))
( "essay"  v ( e1 s ei1 ) ((pos "J0$" ) (subcat "4A" )))
( "essayed"  v ( e1 s ei1 d ) ((pos "Jc$" "Jd$" ) (subcat "4A" )))
( "essaying"  v ( e1 s ei1 i ng ) ((pos "Jb$" ) (subcat "4A" )))
( "essayist"  n ( e1 s ei i s t ) ((pos "K6%" ) ))
( "essayists"  n ( e1 s ei i s t s ) ((pos "Kj%" ) ))
( "essays"  n ( e1 s ei z ) ((pos "Kj%" ) ))
( "essays"  v ( e1 s ei1 z ) ((pos "Ja$" ) (subcat "4A" )))
( "essence"  n ( e1 s n s ) ((pos "M6%" ) ))
( "essences"  n ( e1 s n s i z ) ((pos "Mj%" ) ))
( "essential"  n ( i1 s e1 n sh l ) ((pos "K6%" "OA%" ) ))
( "essentially"  a ( i1 s e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "essentials"  n ( i1 s e1 n sh l z ) ((pos "Kj%" ) ))
( "establish"  v ( i1 s t a1 b l i sh ) ((pos "H1%" ) (subcat "6A" "14" )))
( "established"  v ( i1 s t a1 b l i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "establishes"  v ( i1 s t a1 b l i sh i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "establishing"  v ( i1 s t a1 b l i sh i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "establishment"  n ( i1 s t a1 b l i sh m @ n t ) ((pos "M6%" ) ))
( "establishments"  n ( i1 s t a1 b l i sh m @ n t s ) ((pos "Mj%" ) ))
( "estaminet"  n ( e1 s t a1 m i n ei ) ((pos "K6$" ) ))
( "estaminets"  n ( e1 s t a1 m i n ei z ) ((pos "Kj$" ) ))
( "estate"  n ( i1 s t ei1 t ) ((pos "M6%" ) ))
( "estates"  n ( i1 s t ei1 t s ) ((pos "Mj%" ) ))
( "esteem"  v ( i1 s t ii1 m ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "esteemed"  v ( i1 s t ii1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "esteeming"  v ( i1 s t ii1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "esteems"  v ( i1 s t ii1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "esthete"  n ( ii1 s th ii t ) ((pos "K6$" ) ))
( "esthetes"  n ( ii1 s th ii t s ) ((pos "Kj$" ) ))
( "esthetic"  n ( ii1 s th e1 t i k ) ((pos "L@$" "OA$" ) ))
( "esthetical"  j ( ii1 s th e1 t i k @ l ) ((pos "OA$" ) ))
( "esthetics"  n ( ii1 s th e1 t i k s ) ((pos "Lk$" ) ))
( "estimable"  j ( e1 s t i m @ b l ) ((pos "OA%" ) ))
( "estimate"  n ( e1 s t i m @ t ) ((pos "K6%" ) ))
( "estimate"  v ( e1 s t i m ei t ) ((pos "J2%" ) (subcat "3A" "9" )))
( "estimated"  v ( e1 s t i m ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "9" )))
( "estimates"  n ( e1 s t i m @ t s ) ((pos "Kj%" ) ))
( "estimates"  v ( e1 s t i m ei t s ) ((pos "Ja%" ) (subcat "3A" "9" )))
( "estimating"  v ( e1 s t i m ei t i ng ) ((pos "Jb%" ) (subcat "3A" "9" )))
( "estimation"  n ( e2 s t i m ei1 sh @ n ) ((pos "L@%" ) ))
( "estrange"  v ( i1 s t r ei1 n jh ) ((pos "H2$" ) (subcat "6A" )))
( "estranged"  v ( i1 s t r ei1 n jh d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "estrangement"  n ( i1 s t r ei1 n jh m @ n t ) ((pos "M6%" ) ))
( "estrangements"  n ( i1 s t r ei1 n jh m @ n t s ) ((pos "Mj%" ) ))
( "estranges"  v ( i1 s t r ei1 n jh i z ) ((pos "Ha$" ) (subcat "6A" )))
( "estranging"  v ( i1 s t r ei1 n jh i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "estuaries"  n ( e1 s ch u@ r i z ) ((pos "Kj%" ) ))
( "estuary"  n ( e1 s ch u@ r ii ) ((pos "K8%" ) ))
( "et al"  n ( e2 t - a1 l ) ((pos "Y~%" ) ))
( "et cetera"  nil ( i1 t - s e1 t @ r @ ) ((pos "Z-$" ) ))
( "et seq"  n ( e2 t - s e1 k ) ((pos "Y~$" ) ))
( "eta"  n ( ii2 t ii2 ei1 ) ((pos "Y>$" ) ))
( "etc"  n ( e1 t s e1 t r @ ) ((pos "Y~%" ) ))
( "etch"  v ( e1 ch ) ((pos "J1%" ) (subcat "2A" )))
( "etched"  v ( e1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "etcher"  n ( e1 ch @ r ) ((pos "K6%" ) ))
( "etchers"  n ( e1 ch @ z ) ((pos "Kj%" ) ))
( "etches"  v ( e1 ch i z ) ((pos "Ja%" ) (subcat "2A" )))
( "etching"  v ( e1 ch i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "etchings"  n ( e1 ch i ng z ) ((pos "Mj%" ) ))
( "etd"  n ( ii2 t ii2 d ii1 ) ((pos "Y>$" ) ))
( "eternal"  j ( i1 t @@1 n l ) ((pos "OA%" ) ))
( "eternally"  a ( i1 t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "eternities"  n ( i1 t @@1 n i t i z ) ((pos "Mj%" ) ))
( "eternity"  n ( i1 t @@1 n i t ii ) ((pos "M8%" ) ))
( "ether"  n ( ii1 th @ r ) ((pos "L@$" ) ))
( "ethereal"  j ( i1 th i@1 r i@ l ) ((pos "OA$" ) ))
( "ethic"  n ( e1 th i k ) ((pos "Ki%" ) ))
( "ethical"  j ( e1 th i k l ) ((pos "OA%" ) ))
( "ethically"  a ( e1 th i k l ii ) ((pos "Pu%" ) ))
( "ethics"  n ( e1 th i k s ) ((pos "Lk%" ) ))
( "ethnic"  j ( e1 th n i k ) ((pos "OA%" ) ))
( "ethnically"  a ( e1 th n i k l ii ) ((pos "Pu%" ) ))
( "ethnographer"  n ( e1 th n o1 g r @ f @ r ) ((pos "K6%" ) ))
( "ethnographers"  n ( e1 th n o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "ethnographic"  j ( e2 th n @ g r a1 f i k ) ((pos "OA%" ) ))
( "ethnography"  n ( e1 th n o1 g r @ f ii ) ((pos "L@%" ) ))
( "ethnological"  j ( e2 th n @ l o1 jh i k l ) ((pos "OA%" ) ))
( "ethnologist"  n ( e1 th n o1 l @ jh i s t ) ((pos "K6%" ) ))
( "ethnologists"  n ( e1 th n o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "ethnology"  n ( e1 th n o1 l @ jh ii ) ((pos "L@%" ) ))
( "ethos"  n ( ii1 th o s ) ((pos "Ki%" ) ))
( "ethyl"  n ( e1 th i l ) ((pos "M6$" ) ))
( "ethyls"  n ( e1 th i l z ) ((pos "Mj$" ) ))
( "etiologies"  n ( ii2 t i o1 l @ jh i z ) ((pos "Kj$" ) ))
( "etiology"  n ( ii2 t i o1 l @ jh ii ) ((pos "K8$" ) ))
( "etiquette"  n ( e1 t i k e t ) ((pos "L@%" ) ))
( "etymological"  j ( e2 t i m @ l o1 jh i k l ) ((pos "OA%" ) ))
( "etymologies"  n ( e2 t i m o1 l @ jh i z ) ((pos "Mj%" ) ))
( "etymologist"  n ( e2 t i m o1 l @ jh i s t ) ((pos "K6%" ) ))
( "etymologists"  n ( e2 t i m o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "etymology"  n ( e2 t i m o1 l @ jh ii ) ((pos "M8%" ) ))
( "eucalyptus"  n ( y uu2 k @ l i1 p t @ s ) ((pos "K7%" ) ))
( "eucalyptuses"  n ( y uu2 k @ l i1 p t @ s i z ) ((pos "Kj%" ) ))
( "eugenics"  n ( y uu1 jh e1 n i k s ) ((pos "Lk%" ) ))
( "eulogies"  n ( y uu1 l @ jh i z ) ((pos "Mj%" ) ))
( "eulogist"  n ( y uu1 l @ jh i s t ) ((pos "K6$" ) ))
( "eulogistic"  j ( y uu2 l @ jh i1 s t i k ) ((pos "OA$" ) ))
( "eulogists"  n ( y uu1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "eulogize"  v ( y uu1 l @ jh ai z ) ((pos "H2%" ) (subcat )))
( "eulogized"  v ( y uu1 l @ jh ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "eulogizes"  v ( y uu1 l @ jh ai z i z ) ((pos "Ha%" ) (subcat )))
( "eulogizing"  v ( y uu1 l @ jh ai z i ng ) ((pos "Hb%" ) (subcat )))
( "eulogy"  n ( y uu1 l @ jh ii ) ((pos "M8%" ) ))
( "eunuch"  n ( y uu1 n @ k ) ((pos "K6%" ) ))
( "eunuchs"  n ( y uu1 n @ k s ) ((pos "Kj%" ) ))
( "euphemism"  n ( y uu1 f @ m i z @ m ) ((pos "M6%" ) ))
( "euphemisms"  n ( y uu1 f @ m i z @ m z ) ((pos "Mj%" ) ))
( "euphemistic"  j ( y uu2 f @ m i1 s t i k ) ((pos "OA%" ) ))
( "euphemistically"  a ( y uu2 f @ m i1 s t i k l ii ) ((pos "Pu%" ) ))
( "euphonies"  n ( y uu1 f @ n i z ) ((pos "Mj$" ) ))
( "euphonium"  n ( y uu1 f ou1 n i@ m ) ((pos "K6%" ) ))
( "euphoniums"  n ( y uu1 f ou1 n i@ m z ) ((pos "Kj%" ) ))
( "euphony"  n ( y uu1 f @ n ii ) ((pos "M8$" ) ))
( "euphoria"  n ( y uu1 f oo1 r i@ ) ((pos "L@%" ) ))
( "euphoric"  j ( y uu1 f o1 r i k ) ((pos "OA%" ) ))
( "euphuism"  n ( y uu1 f y uu i z @ m ) ((pos "K6$" ) ))
( "euphuisms"  n ( y uu1 f y uu i z @ m z ) ((pos "Kj$" ) ))
( "eureka"  uh ( y u@1 r ii1 k @ ) ((pos "W-%" ) ))
( "eurhythmics"  n ( y uu1 r i1 dh m i k s ) ((pos "Lk$" ) ))
( "eurythmics"  n ( y uu1 r i1 dh m i k s ) ((pos "Lk$" ) ))
( "euthanasia"  n ( y uu2 th @ n ei1 z i@ ) ((pos "L@%" ) ))
( "evacuate"  v ( i1 v a1 k y u ei t ) ((pos "H2%" ) (subcat "6A" )))
( "evacuated"  v ( i1 v a1 k y u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "evacuates"  v ( i1 v a1 k y u ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "evacuating"  v ( i1 v a1 k y u ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "evacuation"  n ( i1 v a2 k y u ei1 sh @ n ) ((pos "M6%" ) ))
( "evacuations"  n ( i1 v a2 k y u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "evacuee"  n ( i1 v a2 k y uu ii1 ) ((pos "K6%" ) ))
( "evacuees"  n ( i1 v a2 k y uu ii1 z ) ((pos "Kj%" ) ))
( "evade"  v ( i1 v ei1 d ) ((pos "H2%" ) (subcat "6A" )))
( "evaded"  v ( i1 v ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "evades"  v ( i1 v ei1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "evading"  v ( i1 v ei1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "evaluate"  v ( i1 v a1 l y u ei t ) ((pos "H2%" ) (subcat )))
( "evaluated"  v ( i1 v a1 l y u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "evaluates"  v ( i1 v a1 l y u ei t s ) ((pos "Ha%" ) (subcat )))
( "evaluating"  v ( i1 v a1 l y u ei t i ng ) ((pos "Hb%" ) (subcat )))
( "evaluation"  n ( i1 v a2 l y u ei1 sh @ n ) ((pos "K6%" ) ))
( "evaluations"  n ( i1 v a2 l y u ei1 sh @ n z ) ((pos "Kj%" ) ))
( "evaluative"  j ( i1 v a1 l y u@ t i v ) ((pos "OA%" ) ))
( "evanescence"  n ( ii2 v @ n e1 s n s ) ((pos "L@$" ) ))
( "evanescent"  j ( ii2 v @ n e1 s n t ) ((pos "OA$" ) ))
( "evangelical"  j ( ii2 v a n jh e1 l i k l ) ((pos "OA%" ) ))
( "evangelicalism"  n ( ii2 v a n jh e1 l i k @ l i z @ m ) ((pos "L@%" ) ))
( "evangelism"  n ( i1 v a1 n jh @ l i z @ m ) ((pos "L@%" ) ))
( "evangelist"  n ( i1 v a1 n jh @ l i s t ) ((pos "K6%" ) ))
( "evangelistic"  j ( i1 v a2 n jh @ l i1 s t i k ) ((pos "OA%" ) ))
( "evangelists"  n ( i1 v a1 n jh @ l i s t s ) ((pos "Kj%" ) ))
( "evaporate"  v ( i1 v a1 p @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "evaporated"  v ( i1 v a1 p @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "evaporates"  v ( i1 v a1 p @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "evaporating"  v ( i1 v a1 p @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "evaporation"  n ( i1 v a2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "evaporations"  n ( i1 v a2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "evasion"  n ( i1 v ei1 zh n ) ((pos "M6%" ) ))
( "evasions"  n ( i1 v ei1 zh n z ) ((pos "Mj%" ) ))
( "evasive"  j ( i1 v ei1 s i v ) ((pos "OA%" ) ))
( "evasively"  a ( i1 v ei1 s i v l ii ) ((pos "Pu%" ) ))
( "evasiveness"  n ( i1 v ei1 s i v n @ s ) ((pos "L@%" ) ))
( "eve"  n ( ii1 v ) ((pos "K6%" ) ))
( "even"  v ( ii1 v n ) ((pos "H0%" "K6%" "OA*" "Pu*" ) (subcat "6A" )))
( "even-handed"  j ( ii2 v n - h a1 n d i d ) ((pos "OA%" ) ))
( "evened"  v ( ii1 v n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "evening"  v ( ii1 v n i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "evenings"  n ( ii1 v n i ng z ) ((pos "Mj%" ) ))
( "evenly"  a ( ii1 v n l ii ) ((pos "Pu%" ) ))
( "evenness"  n ( ii1 v n - n @ s ) ((pos "L@%" ) ))
( "evens"  v ( ii1 v n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "evensong"  n ( ii1 v n s o ng ) ((pos "K6%" ) ))
( "evensongs"  n ( ii1 v n s o ng z ) ((pos "Kj$" ) ))
( "event"  n ( i1 v e1 n t ) ((pos "K6%" ) ))
( "eventful"  j ( i1 v e1 n t f @ l ) ((pos "OA%" ) ))
( "eventide"  n ( ii1 v n t ai d ) ((pos "L@%" ) ))
( "events"  n ( i1 v e1 n t s ) ((pos "Kj%" ) ))
( "eventual"  j ( i1 v e1 n ch u@ l ) ((pos "OA%" ) ))
( "eventualities"  n ( i1 v e2 n ch u a1 l i t i z ) ((pos "Kj%" ) ))
( "eventuality"  n ( i1 v e2 n ch u a1 l i t ii ) ((pos "K8%" ) ))
( "eventually"  a ( i1 v e1 n ch u@ l ii ) ((pos "Pu%" ) ))
( "ever"  a ( e1 v @ r ) ((pos "Pu*" ) ))
( "evergreen"  n ( e1 v @ g r ii n ) ((pos "K6%" "OA%" ) ))
( "evergreens"  n ( e1 v @ g r ii n z ) ((pos "Kj%" ) ))
( "everlasting"  j ( e2 v @ l aa1 s t i ng ) ((pos "OA%" ) ))
( "evermore"  a ( e2 v @ m oo1 r ) ((pos "Pu$" ) ))
( "every"  j ( e1 v r ii ) ((pos "OA*" ) ))
( "everybody"  prp ( e1 v r i b o d ii ) ((pos "Qx%" ) ))
( "everyday"  j ( e1 v r i d ei ) ((pos "OA%" ) ))
( "everyone"  prp ( e1 v r i w uh n ) ((pos "Qx%" ) ))
( "everyplace"  a ( e1 v r i p l ei s ) ((pos "Pu$" ) ))
( "everything"  prp ( e1 v r i th i ng ) ((pos "Qx*" ) ))
( "everywhere"  a ( e1 v r i w e@ r ) ((pos "Pu%" ) ))
( "eves"  n ( ii1 v z ) ((pos "Kj%" ) ))
( "evict"  v ( i1 v i1 k t ) ((pos "H0%" ) (subcat )))
( "evicted"  v ( i1 v i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "evicting"  v ( i1 v i1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "eviction"  n ( i1 v i1 k sh @ n ) ((pos "M6%" ) ))
( "evictions"  n ( i1 v i1 k sh @ n z ) ((pos "Mj%" ) ))
( "evicts"  v ( i1 v i1 k t s ) ((pos "Ha%" ) (subcat )))
( "evidence"  v ( e1 v i d @ n s ) ((pos "H2%" "L@%" ) (subcat )))
( "evidenced"  v ( e1 v i d @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "evidences"  v ( e1 v i d @ n s i z ) ((pos "Ha$" ) (subcat )))
( "evidencing"  v ( e1 v i d @ n s i ng ) ((pos "Hb$" ) (subcat )))
( "evident"  j ( e1 v i d @ n t ) ((pos "OA%" ) ))
( "evidently"  a ( e1 v i d @ n t l ii ) ((pos "Pu%" ) ))
( "evil"  n ( ii1 v @ l ) ((pos "M6%" "OA%" ) ))
( "evil-doer"  n ( ii1 v l - d uu @ r ) ((pos "K6%" ) ))
( "evil-doers"  n ( ii1 v l - d uu @ z ) ((pos "Kj%" ) ))
( "evil-minded"  j ( ii2 v l - m ai1 n d i d ) ((pos "OA%" ) ))
( "evilly"  a ( ii1 v @ l ii ) ((pos "Pu%" ) ))
( "evils"  n ( ii1 v @ l z ) ((pos "Mj%" ) ))
( "evince"  v ( i1 v i1 n s ) ((pos "H2%" ) (subcat "6A" )))
( "evinced"  v ( i1 v i1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "evinces"  v ( i1 v i1 n s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "evincing"  v ( i1 v i1 n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "eviscerate"  v ( i1 v i1 s @ r ei t ) ((pos "H2$" ) (subcat )))
( "eviscerated"  v ( i1 v i1 s @ r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "eviscerates"  v ( i1 v i1 s @ r ei t s ) ((pos "Ha$" ) (subcat )))
( "eviscerating"  v ( i1 v i1 s @ r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "evocation"  n ( ii2 v ou k ei1 sh @ n ) ((pos "K6%" ) ))
( "evocations"  n ( ii2 v ou k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "evocative"  j ( i1 v o1 k @ t i v ) ((pos "OA%" ) ))
( "evoke"  v ( i1 v ou1 k ) ((pos "H2%" ) (subcat )))
( "evoked"  v ( i1 v ou1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "evokes"  v ( i1 v ou1 k s ) ((pos "Ha%" ) (subcat )))
( "evoking"  v ( i1 v ou1 k i ng ) ((pos "Hb%" ) (subcat )))
( "evolution"  n ( ii2 v @ l uu1 sh @ n ) ((pos "M6%" ) ))
( "evolutionary"  j ( ii2 v @ l uu1 sh @ n r ii ) ((pos "OA%" ) ))
( "evolutions"  n ( ii2 v @ l uu1 sh @ n z ) ((pos "Mj%" ) ))
( "evolve"  v ( i1 v o1 l v ) ((pos "J2%" ) (subcat "2A" )))
( "evolved"  v ( i1 v o1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "evolves"  v ( i1 v o1 l v z ) ((pos "Ja%" ) (subcat "2A" )))
( "evolving"  v ( i1 v o1 l v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ewe"  n ( y uu1 ) ((pos "K6%" ) ))
( "ewer"  n ( y uu1 @ r ) ((pos "K6$" ) ))
( "ewers"  n ( y uu1 @ z ) ((pos "Kj$" ) ))
( "ewes"  n ( y uu1 z ) ((pos "Kj%" ) ))
( "ex gratia"  j ( e2 k s - g r ei1 sh @ ) ((pos "OA%" ) ))
( "ex officio"  j ( e2 k s - @ f i1 sh i@ @ ) ((pos "OA%" "Pu%" ) ))
( "ex-"  j ( e k s - ) ((pos "U-%" ) ))
( "ex-directory"  j ( e2 k s - d i r e1 k t @ r ii ) ((pos "OA%" ) ))
( "ex-service"  j ( e2 k s - s @@1 v i s ) ((pos "OA%" ) ))
( "ex-serviceman"  n ( e2 k s - s @@1 v i s m @ n ) ((pos "Ki%" ) ))
( "ex-servicemen"  n ( e2 k s - s @@1 v i s m @ n ) ((pos "Kj%" ) ))
( "exacerbate"  v ( i1 g z a1 s @ b ei t ) ((pos "H2%" ) (subcat )))
( "exacerbated"  v ( i1 g z a1 s @ b ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exacerbates"  v ( i1 g z a1 s @ b ei t s ) ((pos "Ha%" ) (subcat )))
( "exacerbating"  v ( i1 g z a1 s @ b ei t i ng ) ((pos "Hb%" ) (subcat )))
( "exacerbation"  n ( i1 g z a2 s @ b ei1 sh @ n ) ((pos "K6%" ) ))
( "exacerbations"  n ( i1 g z a2 s @ b ei1 sh @ n z ) ((pos "Kj%" ) ))
( "exact"  v ( i1 g z a1 k t ) ((pos "H0%" "OA%" ) (subcat "6A" )))
( "exacted"  v ( i1 g z a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exacting"  v ( i1 g z a1 k t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "exaction"  n ( i1 g z a1 k sh @ n ) ((pos "M6$" ) ))
( "exactions"  n ( i1 g z a1 k sh @ n z ) ((pos "Mj$" ) ))
( "exactitude"  n ( i1 g z a1 k t i t y uu d ) ((pos "L@%" ) ))
( "exactly"  a ( i1 g z a1 k t l ii ) ((pos "Pu%" ) ))
( "exactness"  n ( i1 g z a1 k t n @ s ) ((pos "L@%" ) ))
( "exacts"  v ( i1 g z a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "exaggerate"  v ( i1 g z a1 jh @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "exaggerated"  v ( i1 g z a1 jh @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "exaggerates"  v ( i1 g z a1 jh @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "exaggerating"  v ( i1 g z a1 jh @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "exaggeration"  n ( i1 g z a2 jh @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "exaggerations"  n ( i1 g z a2 jh @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "exalt"  v ( i1 g z oo1 l t ) ((pos "H0%" ) (subcat )))
( "exaltation"  n ( e2 g z oo l t ei1 sh @ n ) ((pos "L@%" ) ))
( "exalted"  v ( i1 g z oo1 l t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "exalting"  v ( i1 g z oo1 l t i ng ) ((pos "Hb%" ) (subcat )))
( "exalts"  v ( i1 g z oo1 l t s ) ((pos "Ha%" ) (subcat )))
( "exam"  n ( i1 g z a1 m ) ((pos "K6%" ) ))
( "examination"  n ( i1 g z a2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "examinations"  n ( i1 g z a2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "examine"  v ( i1 g z a1 m i n ) ((pos "H2%" ) (subcat "6A" )))
( "examined"  v ( i1 g z a1 m i n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "examiner"  n ( i1 g z a1 m i n @ r ) ((pos "K6%" ) ))
( "examiners"  n ( i1 g z a1 m i n @ z ) ((pos "Kj%" ) ))
( "examines"  v ( i1 g z a1 m i n z ) ((pos "Ha%" ) (subcat "6A" )))
( "examining"  v ( i1 g z a1 m i n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "example"  n ( i1 g z aa1 m p l ) ((pos "M6*" ) ))
( "examples"  n ( i1 g z aa1 m p l z ) ((pos "Mj%" ) ))
( "exams"  n ( i1 g z a1 m z ) ((pos "Kj%" ) ))
( "exasperate"  v ( i1 g z a1 s p @ r ei t ) ((pos "H2%" ) (subcat )))
( "exasperated"  v ( i1 g z a1 s p @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exasperates"  v ( i1 g z a1 s p @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "exasperating"  v ( i1 g z a1 s p @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "exasperation"  n ( i1 g z a2 s p @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "excavate"  v ( e1 k s k @ v ei t ) ((pos "H2%" ) (subcat )))
( "excavated"  v ( e1 k s k @ v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "excavates"  v ( e1 k s k @ v ei t s ) ((pos "Ha%" ) (subcat )))
( "excavating"  v ( e1 k s k @ v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "excavation"  n ( e2 k s k @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "excavations"  n ( e2 k s k @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "excavator"  n ( e1 k s k @ v ei t @ r ) ((pos "K6%" ) ))
( "excavators"  n ( e1 k s k @ v ei t @ z ) ((pos "Kj%" ) ))
( "exceed"  v ( i1 k s ii1 d ) ((pos "H0%" ) (subcat )))
( "exceeded"  v ( i1 k s ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exceeding"  v ( i1 k s ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "exceedingly"  a ( i1 k s ii1 d i ng l ii ) ((pos "Pu%" ) ))
( "exceeds"  v ( i1 k s ii1 d z ) ((pos "Ha%" ) (subcat )))
( "excel"  v ( i1 k s e1 l ) ((pos "J4%" ) (subcat "2C" "3A" "6A" )))
( "excelled"  v ( i1 k s e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "excellence"  n ( e1 k s @ l @ n s ) ((pos "M6%" ) ))
( "excellences"  n ( e1 k s @ l @ n s i z ) ((pos "Mj$" ) ))
( "excellent"  j ( e1 k s @ l @ n t ) ((pos "OA%" ) ))
( "excellently"  a ( e1 k s @ l @ n t l ii ) ((pos "Pu%" ) ))
( "excelling"  v ( i1 k s e1 l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "excels"  v ( i1 k s e1 l z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" )))
( "excelsior"  n ( e1 k s e1 l s i oo r ) ((pos "L@$" ) ))
( "except"  v ( i1 k s e1 p t ) ((pos "H0%" "T-%" "V-%" ) (subcat "6A" )))
( "excepted"  v ( i1 k s e1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "excepting"  v ( i1 k s e1 p t i ng ) ((pos "Hb%" "T-%" ) (subcat "6A" )))
( "exception"  n ( i1 k s e1 p sh @ n ) ((pos "M6%" ) ))
( "exceptionable"  j ( i1 k s e1 p sh @ n @ b l ) ((pos "OA$" ) ))
( "exceptional"  j ( i1 k s e1 p sh @ n l ) ((pos "OA%" ) ))
( "exceptionally"  a ( i1 k s e1 p sh @ n @ l ii ) ((pos "Pu%" ) ))
( "exceptions"  n ( i1 k s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "excepts"  v ( i1 k s e1 p t s ) ((pos "Ha%" ) (subcat "6A" )))
( "excerpt"  n ( e1 k s @@ p t ) ((pos "K6%" ) ))
( "excerpts"  n ( e1 k s @@ p t s ) ((pos "Kj%" ) ))
( "excess"  j ( e1 k s e s ) ((pos "OA%" ) ))
( "excess"  n ( i1 k s e1 s ) ((pos "M7%" ) ))
( "excesses"  n ( i1 k s e1 s i z ) ((pos "Mj%" ) ))
( "excessive"  j ( i1 k s e1 s i v ) ((pos "OA%" ) ))
( "excessively"  a ( i1 k s e1 s i v l ii ) ((pos "Pu%" ) ))
( "exchange"  v ( i1 k s ch ei1 n jh ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "exchangeable"  j ( i1 k s ch ei1 n jh @ b l ) ((pos "OA%" ) ))
( "exchanged"  v ( i1 k s ch ei1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exchanger"  n ( i1 k s ch ei1 n jh @ r ) ((pos "K6%" ) ))
( "exchangers"  n ( i1 k s ch ei1 n jh @ z ) ((pos "Kj%" ) ))
( "exchanges"  v ( i1 k s ch ei1 n jh i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "exchanging"  v ( i1 k s ch ei1 n jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exchequer"  n ( i1 k s ch e1 k @ r ) ((pos "K6%" ) ))
( "exchequers"  n ( i1 k s ch e1 k @ z ) ((pos "Kj$" ) ))
( "excise"  n ( e1 k s ai z ) ((pos "L@%" ) ))
( "excise"  v ( i1 k s ai1 z ) ((pos "H2$" ) (subcat )))
( "excised"  v ( i1 k s ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exciseman"  n ( e1 k s ai s m @ n ) ((pos "Ki%" ) ))
( "excisemen"  n ( e1 k s ai s m @ n ) ((pos "Kj%" ) ))
( "excises"  v ( i1 k s ai1 z i z ) ((pos "Ha$" ) (subcat )))
( "excising"  v ( i1 k s ai1 z i ng ) ((pos "Hb$" ) (subcat )))
( "excision"  n ( i1 k s i1 zh n ) ((pos "M6%" ) ))
( "excisions"  n ( i1 k s i1 zh n z ) ((pos "Mj%" ) ))
( "excitability"  n ( i1 k s ai2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "excitable"  j ( i1 k s ai1 t @ b l ) ((pos "OA%" ) ))
( "excite"  v ( i1 k s ai1 t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "excited"  v ( i1 k s ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "excitedly"  a ( i1 k s ai1 t i d l ii ) ((pos "Pu%" ) ))
( "excitement"  n ( i1 k s ai1 t m @ n t ) ((pos "M6%" ) ))
( "excitements"  n ( i1 k s ai1 t m @ n t s ) ((pos "Mj%" ) ))
( "excites"  v ( i1 k s ai1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "exciting"  v ( i1 k s ai1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "excitingly"  a ( i1 k s ai1 t i ng l ii ) ((pos "Pu%" ) ))
( "excl"  n ( i1 k s k l uu1 d i ng ) ((pos "Y~%" ) ))
( "exclaim"  v ( i1 k s k l ei1 m ) ((pos "J0%" ) (subcat "2A" )))
( "exclaimed"  v ( i1 k s k l ei1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "exclaiming"  v ( i1 k s k l ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "exclaims"  v ( i1 k s k l ei1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "exclamation"  n ( e2 k s k l @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "exclamations"  n ( e2 k s k l @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "exclamatory"  j ( i1 k s k l a1 m @ t r ii ) ((pos "OA%" ) ))
( "exclude"  v ( i1 k s k l uu1 d ) ((pos "H2%" ) (subcat "6A" )))
( "excluded"  v ( i1 k s k l uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "excludes"  v ( i1 k s k l uu1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "excluding"  v ( i1 k s k l uu1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exclusion"  n ( i1 k s k l uu1 zh n ) ((pos "L@%" ) ))
( "exclusive"  j ( i1 k s k l uu1 s i v ) ((pos "OA%" ) ))
( "exclusively"  a ( i1 k s k l uu1 s i v l ii ) ((pos "Pu%" ) ))
( "excogitate"  v ( e1 k s k o1 jh i t ei t ) ((pos "H2%" ) (subcat )))
( "excogitated"  v ( e1 k s k o1 jh i t ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "excogitates"  v ( e1 k s k o1 jh i t ei t s ) ((pos "Ha$" ) (subcat )))
( "excogitating"  v ( e1 k s k o1 jh i t ei t i ng ) ((pos "Hb$" ) (subcat )))
( "excogitation"  n ( e2 k s k o2 jh i t ei1 sh @ n ) ((pos "M6$" ) ))
( "excogitations"  n ( e2 k s k o2 jh i t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "excommunicate"  v ( e2 k s k @ m y uu1 n i k ei t ) ((pos "H2%" ) (subcat )))
( "excommunicated"  v ( e2 k s k @ m y uu1 n i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "excommunicates"  v ( e2 k s k @ m y uu1 n i k ei t s ) ((pos "Ha%" ) (subcat )))
( "excommunicating"  v ( e2 k s k @ m y uu1 n i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "excommunication"  n ( e2 k s k @ m y uu2 n i k ei1 sh @ n ) ((pos "M6%" ) ))
( "excommunications"  n ( e2 k s k @ m y uu2 n i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "excoriate"  v ( i1 k s k oo1 r i ei t ) ((pos "H2$" ) (subcat )))
( "excoriated"  v ( i1 k s k oo1 r i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "excoriates"  v ( i1 k s k oo1 r i ei t s ) ((pos "Ha$" ) (subcat )))
( "excoriating"  v ( i1 k s k oo1 r i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "excoriation"  n ( i1 k s k oo2 r i ei1 sh @ n ) ((pos "K6$" ) ))
( "excoriations"  n ( i1 k s k oo2 r i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "excrement"  n ( e1 k s k r @ m @ n t ) ((pos "L@%" ) ))
( "excrescence"  n ( i1 k s k r e1 s n s ) ((pos "K6%" ) ))
( "excrescences"  n ( i1 k s k r e1 s n s i z ) ((pos "Kj%" ) ))
( "excreta"  n ( i1 k s k r ii1 t @ ) ((pos "Kj%" ) ))
( "excrete"  v ( i1 k s k r ii1 t ) ((pos "H2%" ) (subcat )))
( "excreted"  v ( i1 k s k r ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "excretes"  v ( i1 k s k r ii1 t s ) ((pos "Ha%" ) (subcat )))
( "excreting"  v ( i1 k s k r ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "excretion"  n ( i1 k s k r ii1 sh @ n ) ((pos "M6%" ) ))
( "excretions"  n ( i1 k s k r ii1 sh @ n z ) ((pos "Mj%" ) ))
( "excruciating"  j ( i1 k s k r uu1 sh i ei t i ng ) ((pos "OA%" ) ))
( "excruciatingly"  a ( i1 k s k r uu1 sh i ei t i ng l ii ) ((pos "Pu%" ) ))
( "exculpate"  v ( e1 k s k uh l p ei t ) ((pos "H2$" ) (subcat "6A" )))
( "exculpated"  v ( e1 k s k uh l p ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "exculpates"  v ( e1 k s k uh l p ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "exculpating"  v ( e1 k s k uh l p ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "excursion"  n ( i1 k s k @@1 sh @ n ) ((pos "K6%" ) ))
( "excursionist"  n ( i1 k s k @@1 sh @ n i s t ) ((pos "K6$" ) ))
( "excursionists"  n ( i1 k s k @@1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "excursions"  n ( i1 k s k @@1 sh @ n z ) ((pos "Kj%" ) ))
( "excusable"  j ( i1 k s k y uu1 z @ b l ) ((pos "OA%" ) ))
( "excusably"  a ( i1 k s k y uu1 z @ b l ii ) ((pos "Pu%" ) ))
( "excuse"  n ( i1 k s k y uu1 s ) ((pos "K6%" ) ))
( "excuse"  v ( i1 k s k y uu1 z ) ((pos "H2%" ) (subcat "6A" "6C" "12B" "13B" "14" )))
( "excused"  v ( i1 k s k y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "12B" "13B" "14" )))
( "excuses"  n ( i1 k s k y uu1 s i z ) ((pos "Kj%" ) ))
( "excuses"  v ( i1 k s k y uu1 z i z ) ((pos "Ha%" ) (subcat "6A" "6C" "12B" "13B" "14" )))
( "excusing"  v ( i1 k s k y uu1 z i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "12B" "13B" "14" )))
( "execrable"  j ( e1 k s i k r @ b l ) ((pos "OA%" ) ))
( "execrate"  v ( e1 k s i k r ei t ) ((pos "H2$" ) (subcat )))
( "execrated"  v ( e1 k s i k r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "execrates"  v ( e1 k s i k r ei t s ) ((pos "Ha$" ) (subcat )))
( "execrating"  v ( e1 k s i k r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "execration"  n ( e2 k s i k r ei1 sh @ n ) ((pos "K6$" ) ))
( "execrations"  n ( e2 k s i k r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "executant"  n ( i1 g z e1 k y u t @ n t ) ((pos "K6%" ) ))
( "executants"  n ( i1 g z e1 k y u t @ n t s ) ((pos "Kj%" ) ))
( "execute"  v ( e1 k s i k y uu t ) ((pos "H2%" ) (subcat )))
( "executed"  v ( e1 k s i k y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "executes"  v ( e1 k s i k y uu t s ) ((pos "Ha%" ) (subcat )))
( "executing"  v ( e1 k s i k y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "execution"  n ( e2 k s i k y uu1 sh @ n ) ((pos "M6%" ) ))
( "executioner"  n ( e2 k s i k y uu1 sh @ n @ r ) ((pos "K6%" ) ))
( "executioners"  n ( e2 k s i k y uu1 sh @ n @ z ) ((pos "Kj%" ) ))
( "executions"  n ( e2 k s i k y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "executive"  n ( i1 g z e1 k y u t i v ) ((pos "K6%" "OA%" ) ))
( "executives"  n ( i1 g z e1 k y u t i v z ) ((pos "Kj%" ) ))
( "executor"  n ( i1 g z e1 k y u t @ r ) ((pos "K6%" ) ))
( "executors"  n ( i1 g z e1 k y u t @ z ) ((pos "Kj%" ) ))
( "executrix"  n ( i1 g z e1 k y u t r i k s ) ((pos "K7$" ) ))
( "executrixes"  n ( i1 g z e1 k y u t r i k s i z ) ((pos "Kj$" ) ))
( "exegesis"  n ( e2 k s i jh ii1 s i s ) ((pos "L@%" ) ))
( "exemplary"  j ( i1 g z e1 m p l @ r ii ) ((pos "OA%" ) ))
( "exemplification"  n ( i1 g z e2 m p l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "exemplifications"  n ( i1 g z e2 m p l i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "exemplified"  v ( i1 g z e1 m p l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exemplifies"  v ( i1 g z e1 m p l i f ai z ) ((pos "Ha%" ) (subcat )))
( "exemplify"  v ( i1 g z e1 m p l i f ai ) ((pos "H3%" ) (subcat )))
( "exemplifying"  v ( i1 g z e1 m p l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "exempt"  v ( i1 g z e1 m p t ) ((pos "H0%" "OA%" ) (subcat "6A" )))
( "exempted"  v ( i1 g z e1 m p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exempting"  v ( i1 g z e1 m p t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exemption"  n ( i1 g z e1 m p sh @ n ) ((pos "M6%" ) ))
( "exemptions"  n ( i1 g z e1 m p sh @ n z ) ((pos "Mj%" ) ))
( "exempts"  v ( i1 g z e1 m p t s ) ((pos "Ha%" ) (subcat "6A" )))
( "exercise"  v ( e1 k s @ s ai z ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" )))
( "exercised"  v ( e1 k s @ s ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "exercises"  v ( e1 k s @ s ai z i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "exercising"  v ( e1 k s @ s ai z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "exert"  v ( i1 g z @@1 t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "exerted"  v ( i1 g z @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "exerting"  v ( i1 g z @@1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "exertion"  n ( i1 g z @@1 sh @ n ) ((pos "M6%" ) ))
( "exertions"  n ( i1 g z @@1 sh @ n z ) ((pos "Mj%" ) ))
( "exerts"  v ( i1 g z @@1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "exeunt"  v ( e1 k s i@ n t ) ((pos "Ie$" ) ))
( "exhalation"  n ( e2 k s h @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "exhalations"  n ( e2 k s h @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "exhale"  v ( e1 k s h ei1 l ) ((pos "J2%" ) (subcat "2A" )))
( "exhaled"  v ( e1 k s h ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "exhales"  v ( e1 k s h ei1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "exhaling"  v ( e1 k s h ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "exhaust"  v ( i1 g z oo1 s t ) ((pos "H0%" "M6%" ) (subcat )))
( "exhaust-pipe"  n ( i1 g z oo1 s t - p ai p ) ((pos "K6%" ) ))
( "exhaust-pipes"  n ( i1 g z oo1 s t - p ai p s ) ((pos "Kj%" ) ))
( "exhausted"  v ( i1 g z oo1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exhausting"  v ( i1 g z oo1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "exhaustion"  n ( i1 g z oo1 s ch @ n ) ((pos "L@%" ) ))
( "exhaustive"  j ( i1 g z oo1 s t i v ) ((pos "OA%" ) ))
( "exhaustively"  a ( i1 g z oo1 s t i v l ii ) ((pos "Pu%" ) ))
( "exhausts"  v ( i1 g z oo1 s t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "exhibit"  v ( i1 g z i1 b i t ) ((pos "H0%" "K6%" ) (subcat "2A" )))
( "exhibited"  v ( i1 g z i1 b i t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "exhibiting"  v ( i1 g z i1 b i t i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "exhibition"  n ( e2 k s i b i1 sh @ n ) ((pos "K6%" ) ))
( "exhibitioner"  n ( e2 k s i b i1 sh @ n @ r ) ((pos "K6$" ) ))
( "exhibitioners"  n ( e2 k s i b i1 sh @ n @ z ) ((pos "Kj$" ) ))
( "exhibitionism"  n ( e2 k s i b i1 sh @ n i z @ m ) ((pos "L@%" ) ))
( "exhibitionist"  n ( e2 k s i b i1 sh @ n i s t ) ((pos "K6%" ) ))
( "exhibitionists"  n ( e2 k s i b i1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "exhibitions"  n ( e2 k s i b i1 sh @ n z ) ((pos "Kj%" ) ))
( "exhibitor"  n ( i1 g z i1 b i t @ r ) ((pos "K6%" ) ))
( "exhibitors"  n ( i1 g z i1 b i t @ z ) ((pos "Kj%" ) ))
( "exhibits"  v ( i1 g z i1 b i t s ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "exhilarate"  v ( i1 g z i1 l @ r ei t ) ((pos "H2%" ) (subcat )))
( "exhilarated"  v ( i1 g z i1 l @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exhilarates"  v ( i1 g z i1 l @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "exhilarating"  v ( i1 g z i1 l @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "exhilaration"  n ( i1 g z i2 l @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "exhort"  v ( i1 g z oo1 t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "exhortation"  n ( e2 k s oo t ei1 sh @ n ) ((pos "M6%" ) ))
( "exhortations"  n ( e2 k s oo t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "exhorted"  v ( i1 g z oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "exhorting"  v ( i1 g z oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "exhorts"  v ( i1 g z oo1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "exhumation"  n ( e2 k s h y uu m ei1 sh @ n ) ((pos "M6%" ) ))
( "exhumations"  n ( e2 k s h y uu m ei1 sh @ n z ) ((pos "Mj$" ) ))
( "exhume"  v ( e1 k s h y uu1 m ) ((pos "H2%" ) (subcat )))
( "exhumed"  v ( e1 k s h y uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exhumes"  v ( e1 k s h y uu1 m z ) ((pos "Ha%" ) (subcat )))
( "exhuming"  v ( e1 k s h y uu1 m i ng ) ((pos "Hb%" ) (subcat )))
( "exigencies"  n ( e1 k s i jh @ n s i z ) ((pos "Kj%" ) ))
( "exigency"  n ( e1 k s i jh @ n s ii ) ((pos "K8%" ) ))
( "exigent"  j ( e1 g z i jh @ n t ) ((pos "OA$" ) ))
( "exiguous"  j ( e1 g z i1 g y u@ s ) ((pos "OA$" ) ))
( "exile"  v ( e1 k s ai l ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "exiled"  v ( e1 k s ai l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exiles"  v ( e1 k s ai l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "exiling"  v ( e1 k s ai l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exist"  v ( i1 g z i1 s t ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "existed"  v ( i1 g z i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "existence"  n ( i1 g z i1 s t @ n s ) ((pos "M6%" ) ))
( "existences"  n ( i1 g z i1 s t @ n s i z ) ((pos "Mj%" ) ))
( "existent"  j ( i1 g z i1 s t @ n t ) ((pos "OA%" ) ))
( "existentialism"  n ( e2 g z i s t e1 n sh @ l i z @ m ) ((pos "L@%" ) ))
( "existentialist"  n ( e2 g z i s t e1 n sh @ l i s t ) ((pos "K6%" ) ))
( "existentialists"  n ( e2 g z i s t e1 n sh @ l i s t s ) ((pos "Kj%" ) ))
( "existing"  v ( i1 g z i1 s t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "exists"  v ( i1 g z i1 s t s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "exit"  v ( e1 k s i t ) ((pos "I0%" "K6%" ) ))
( "exited"  v ( e1 k s i t i d ) ((pos "Ic%" "Id%" ) ))
( "exiting"  v ( e1 k s i t i ng ) ((pos "Ib%" ) ))
( "exits"  v ( e1 k s i t s ) ((pos "Ia%" "Kj%" ) ))
( "exodus"  n ( e1 k s @ d @ s ) ((pos "K7%" ) ))
( "exoduses"  n ( e1 k s @ d @ s i z ) ((pos "Kj$" ) ))
( "exonerate"  v ( i1 g z o1 n @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "exonerated"  v ( i1 g z o1 n @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exonerates"  v ( i1 g z o1 n @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "exonerating"  v ( i1 g z o1 n @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exoneration"  n ( i1 g z o2 n @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "exonerations"  n ( i1 g z o2 n @ r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "exorbitance"  n ( i1 g z oo1 b i t @ n s ) ((pos "L@$" ) ))
( "exorbitant"  j ( i1 g z oo1 b i t @ n t ) ((pos "OA%" ) ))
( "exorbitantly"  a ( i1 g z oo1 b i t @ n t l ii ) ((pos "Pu%" ) ))
( "exorcize"  v ( e1 k s oo s ai z ) ((pos "H2%" ) (subcat "6A" )))
( "exorcized"  v ( e1 k s oo s ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "exorcizes"  v ( e1 k s oo s ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "exorcizing"  v ( e1 k s oo s ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "exotic"  j ( i1 g z o1 t i k ) ((pos "OA%" ) ))
( "expand"  v ( i1 k s p a1 n d ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "expanded"  v ( i1 k s p a1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "expanding"  v ( i1 k s p a1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "expands"  v ( i1 k s p a1 n d z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "expanse"  n ( i1 k s p a1 n s ) ((pos "K6%" ) ))
( "expanses"  n ( i1 k s p a1 n s i z ) ((pos "Kj%" ) ))
( "expansion"  n ( i1 k s p a1 n sh @ n ) ((pos "L@%" ) ))
( "expansive"  j ( i1 k s p a1 n s i v ) ((pos "OA%" ) ))
( "expansively"  a ( i1 k s p a1 n s i v l ii ) ((pos "Pu%" ) ))
( "expansiveness"  n ( i1 k s p a1 n s i v n @ s ) ((pos "L@%" ) ))
( "expatiate"  v ( i1 k s p ei1 sh i ei t ) ((pos "I2$" ) (subcat )))
( "expatiated"  v ( i1 k s p ei1 sh i ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "expatiates"  v ( i1 k s p ei1 sh i ei t s ) ((pos "Ia$" ) (subcat )))
( "expatiating"  v ( i1 k s p ei1 sh i ei t i ng ) ((pos "Ib$" ) (subcat )))
( "expatriate"  n ( e1 k s p a1 t r i@ t ) ((pos "K6$" ) ))
( "expatriate"  v ( e1 k s p a1 t r i ei t ) ((pos "H2$" ) (subcat )))
( "expatriated"  v ( e1 k s p a1 t r i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "expatriates"  n ( e1 k s p a1 t r i@ t s ) ((pos "Kj$" ) ))
( "expatriates"  v ( e1 k s p a1 t r i ei t s ) ((pos "Ha$" ) (subcat )))
( "expatriating"  v ( e1 k s p a1 t r i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "expect"  v ( i1 k s p e1 k t ) ((pos "H0%" ) (subcat "6A" "7A" "9" "14" )))
( "expectancies"  n ( i1 k s p e1 k t @ n s i z ) ((pos "Mj%" ) ))
( "expectancy"  n ( i1 k s p e1 k t @ n s ii ) ((pos "M8%" ) ))
( "expectant"  j ( i1 k s p e1 k t @ n t ) ((pos "OA%" ) ))
( "expectantly"  a ( i1 k s p e1 k t @ n t l ii ) ((pos "Pu%" ) ))
( "expectation"  n ( e2 k s p e k t ei1 sh @ n ) ((pos "M6%" ) ))
( "expectations"  n ( e2 k s p e k t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "expected"  v ( i1 k s p e1 k t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "7A" "9" "14" )))
( "expecting"  v ( i1 k s p e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "7A" "9" "14" )))
( "expectorant"  n ( i1 k s p e1 k t @ r @ n t ) ((pos "K6$" ) ))
( "expectorants"  n ( i1 k s p e1 k t @ r @ n t s ) ((pos "Kj$" ) ))
( "expectorate"  v ( i1 k s p e1 k t @ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "expectorated"  v ( i1 k s p e1 k t @ r ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "expectorates"  v ( i1 k s p e1 k t @ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "expectorating"  v ( i1 k s p e1 k t @ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "expects"  v ( i1 k s p e1 k t s ) ((pos "Ha%" ) (subcat "6A" "7A" "9" "14" )))
( "expedience"  n ( i1 k s p ii1 d i@ n s ) ((pos "L@%" ) ))
( "expediency"  n ( i1 k s p ii1 d i@ n s ii ) ((pos "L@%" ) ))
( "expedient"  n ( i1 k s p ii1 d i@ n t ) ((pos "K6%" "OA%" ) ))
( "expediently"  a ( i1 k s p ii1 d i@ n t l ii ) ((pos "Pu%" ) ))
( "expedients"  n ( i1 k s p ii1 d i@ n t s ) ((pos "Kj%" ) ))
( "expedite"  v ( e1 k s p i d ai t ) ((pos "H2$" ) (subcat )))
( "expedited"  v ( e1 k s p i d ai t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "expedites"  v ( e1 k s p i d ai t s ) ((pos "Ha$" ) (subcat )))
( "expediting"  v ( e1 k s p i d ai t i ng ) ((pos "Hb$" ) (subcat )))
( "expedition"  n ( e2 k s p i d i1 sh @ n ) ((pos "M6%" ) ))
( "expeditionary"  j ( e2 k s p i d i1 sh @ n @ r ii ) ((pos "OA%" ) ))
( "expeditions"  n ( e2 k s p i d i1 sh @ n z ) ((pos "Mj%" ) ))
( "expeditious"  j ( e2 k s p i d i1 sh @ s ) ((pos "OA$" ) ))
( "expeditiously"  a ( e2 k s p i d i1 sh @ s l ii ) ((pos "Pu$" ) ))
( "expel"  v ( i1 k s p e1 l ) ((pos "H4%" ) (subcat "6A" )))
( "expelled"  v ( i1 k s p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "expelling"  v ( i1 k s p e1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "expels"  v ( i1 k s p e1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "expend"  v ( i1 k s p e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "expendable"  j ( i1 k s p e1 n d @ b l ) ((pos "OA%" ) ))
( "expended"  v ( i1 k s p e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "expending"  v ( i1 k s p e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "expenditure"  n ( i1 k s p e1 n d i ch @ r ) ((pos "M6%" ) ))
( "expenditures"  n ( i1 k s p e1 n d i ch @ z ) ((pos "Mj%" ) ))
( "expends"  v ( i1 k s p e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "expense"  n ( i1 k s p e1 n s ) ((pos "M6%" ) ))
( "expenses"  n ( i1 k s p e1 n s i z ) ((pos "Mj%" ) ))
( "expensive"  j ( i1 k s p e1 n s i v ) ((pos "OA%" ) ))
( "expensively"  a ( i1 k s p e1 n s i v l ii ) ((pos "Pu%" ) ))
( "experience"  v ( i1 k s p i@1 r i@ n s ) ((pos "H2%" "M6%" ) (subcat )))
( "experienced"  v ( i1 k s p i@1 r i@ n s t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "experiences"  v ( i1 k s p i@1 r i@ n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "experiencing"  v ( i1 k s p i@1 r i@ n s i ng ) ((pos "Hb%" ) (subcat )))
( "experiment"  v ( i1 k s p e1 r i m @ n t ) ((pos "I0%" "M6%" ) (subcat "2A" "2C" )))
( "experimental"  j ( i1 k s p e2 r i m e1 n t l ) ((pos "OA%" ) ))
( "experimentally"  a ( i1 k s p e2 r i m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "experimentation"  n ( i1 k s p e2 r i m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "experimented"  v ( i1 k s p e1 r i m @ n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "experimenter"  n ( i1 k s p e1 r i m @ n t @ r ) ((pos "K6%" ) ))
( "experimenters"  n ( i1 k s p e1 r i m @ n t @ z ) ((pos "Kj%" ) ))
( "experimenting"  v ( i1 k s p e1 r i m @ n t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "experiments"  v ( i1 k s p e1 r i m @ n t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2C" )))
( "expert"  n ( e1 k s p @@ t ) ((pos "K6%" "OA%" ) ))
( "expertise"  n ( e2 k s p @@ t ii1 z ) ((pos "L@%" ) ))
( "expertly"  a ( e1 k s p @@ t l ii ) ((pos "Pu%" ) ))
( "expertness"  n ( e1 k s p @@ t n @ s ) ((pos "L@$" ) ))
( "experts"  n ( e1 k s p @@ t s ) ((pos "Kj%" ) ))
( "expiate"  v ( e1 k s p i ei t ) ((pos "H2$" ) (subcat )))
( "expiated"  v ( e1 k s p i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "expiates"  v ( e1 k s p i ei t s ) ((pos "Ha$" ) (subcat )))
( "expiating"  v ( e1 k s p i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "expiation"  n ( e2 k s p i ei1 sh @ n ) ((pos "L@$" ) ))
( "expiration"  n ( e2 k s p i r ei1 sh @ n ) ((pos "L@$" ) ))
( "expire"  v ( i1 k s p ai1 @ r ) ((pos "I2$" ) (subcat )))
( "expired"  v ( i1 k s p ai1 @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "expires"  v ( i1 k s p ai1 @ z ) ((pos "Ia%" ) (subcat )))
( "expiries"  n ( i1 k s p ai1 @ r i z ) ((pos "Kj$" ) ))
( "expiring"  v ( i1 k s p ai1 @ r i ng ) ((pos "Ib%" ) (subcat )))
( "expiry"  n ( i1 k s p ai1 @ r ii ) ((pos "K8%" ) ))
( "explain"  v ( i1 k s p l ei1 n ) ((pos "H0%" ) (subcat "6A" "8" "9" "10" "14" )))
( "explained"  v ( i1 k s p l ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" "14" )))
( "explaining"  v ( i1 k s p l ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" "14" )))
( "explains"  v ( i1 k s p l ei1 n z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" "14" )))
( "explanation"  n ( e2 k s p l @ n ei1 sh @ n ) ((pos "M6%" ) ))
( "explanations"  n ( e2 k s p l @ n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "explanatory"  j ( i1 k s p l a1 n @ t r ii ) ((pos "OA%" ) ))
( "expletive"  n ( i1 k s p l ii1 t i v ) ((pos "K6%" ) ))
( "expletives"  n ( i1 k s p l ii1 t i v z ) ((pos "Kj%" ) ))
( "explicable"  j ( e1 k s p l i1 k @ b l ) ((pos "OA%" ) ))
( "explicate"  v ( e1 k s p l i k ei t ) ((pos "H2$" ) (subcat )))
( "explicated"  v ( e1 k s p l i k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "explicates"  v ( e1 k s p l i k ei t s ) ((pos "Ha$" ) (subcat )))
( "explicating"  v ( e1 k s p l i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "explicit"  j ( i1 k s p l i1 s i t ) ((pos "OA%" ) ))
( "explicitly"  a ( i1 k s p l i1 s i t l ii ) ((pos "Pu%" ) ))
( "explicitness"  n ( i1 k s p l i1 s i t n @ s ) ((pos "L@%" ) ))
( "explode"  v ( i1 k s p l ou1 d ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "exploded"  v ( i1 k s p l ou1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "explodes"  v ( i1 k s p l ou1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "exploding"  v ( i1 k s p l ou1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "exploit"  n ( e1 k s p l oi t ) ((pos "K6%" ) ))
( "exploit"  v ( i1 k s p l oi1 t ) ((pos "H0%" ) (subcat )))
( "exploitation"  n ( e2 k s p l oi t ei1 sh @ n ) ((pos "L@%" ) ))
( "exploited"  v ( i1 k s p l oi1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exploiting"  v ( i1 k s p l oi1 t i ng ) ((pos "Hb%" ) (subcat )))
( "exploits"  n ( e1 k s p l oi t s ) ((pos "Kj%" ) ))
( "exploits"  v ( i1 k s p l oi1 t s ) ((pos "Ha%" ) (subcat )))
( "exploration"  n ( e2 k s p l @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "explorations"  n ( e2 k s p l @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "exploratory"  j ( i1 k s p l o1 r @ t r ii ) ((pos "OA%" ) ))
( "explore"  v ( i1 k s p l oo1 r ) ((pos "H2%" ) (subcat )))
( "explored"  v ( i1 k s p l oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "explorer"  n ( i1 k s p l oo1 r @ r ) ((pos "K6%" ) ))
( "explorers"  n ( i1 k s p l oo1 r @ z ) ((pos "Kj%" ) ))
( "explores"  v ( i1 k s p l oo1 z ) ((pos "Ha%" ) (subcat )))
( "exploring"  v ( i1 k s p l oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "explosion"  n ( i1 k s p l ou1 zh n ) ((pos "K6%" ) ))
( "explosions"  n ( i1 k s p l ou1 zh n z ) ((pos "Kj%" ) ))
( "explosive"  n ( i1 k s p l ou1 s i v ) ((pos "K6%" "OA%" ) ))
( "explosively"  a ( i1 k s p l ou1 s i v l ii ) ((pos "Pu%" ) ))
( "explosives"  n ( i1 k s p l ou1 s i v z ) ((pos "Kj%" ) ))
( "expo"  n ( e1 k s p ou ) ((pos "K6$" ) ))
( "exponent"  n ( i1 k s p ou1 n @ n t ) ((pos "K6%" ) ))
( "exponential"  n ( e2 k s p @ n e1 n sh l ) ((pos "K6$" "OA%" ) ))
( "exponentially"  a ( e2 k s p @ n e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "exponentials"  n ( e2 k s p @ n e1 n sh l z ) ((pos "Kj$" ) ))
( "exponents"  n ( i1 k s p ou1 n @ n t s ) ((pos "Kj%" ) ))
( "export"  n ( e1 k s p oo t ) ((pos "M6%" ) ))
( "export"  v ( i1 k s p oo1 t ) ((pos "H0%" ) (subcat )))
( "exportable"  j ( i1 k s p oo1 t @ b l ) ((pos "OA%" ) ))
( "exportation"  n ( e2 k s p oo t ei1 sh @ n ) ((pos "L@$" ) ))
( "exported"  v ( i1 k s p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exporter"  n ( i1 k s p oo1 t @ r ) ((pos "K6%" ) ))
( "exporters"  n ( i1 k s p oo1 t @ z ) ((pos "Kj%" ) ))
( "exporting"  v ( i1 k s p oo1 t i ng ) ((pos "Hb%" ) (subcat )))
( "exports"  n ( e1 k s p oo t s ) ((pos "Mj%" ) ))
( "exports"  v ( i1 k s p oo1 t s ) ((pos "Ha%" ) (subcat )))
( "expos"  n ( e1 k s p ou z ) ((pos "Kj$" ) ))
( "expos_e"  n ( e1 k s p ou1 z ei ) ((pos "K6%" ) ))
( "expos_es"  n ( e1 k s p ou1 z ei z ) ((pos "Kj%" ) ))
( "expose"  v ( i1 k s p ou1 z ) ((pos "H2%" ) (subcat "6A" "14" )))
( "exposed"  v ( i1 k s p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "exposes"  v ( i1 k s p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "exposing"  v ( i1 k s p ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "exposition"  n ( e2 k s p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "expositions"  n ( e2 k s p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "expostulate"  v ( i1 k s p o1 s ch u l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "expostulated"  v ( i1 k s p o1 s ch u l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "expostulates"  v ( i1 k s p o1 s ch u l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "expostulating"  v ( i1 k s p o1 s ch u l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "expostulation"  n ( i1 k s p o2 s ch u l ei1 sh @ n ) ((pos "M6%" ) ))
( "expostulations"  n ( i1 k s p o2 s ch u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "exposure"  n ( i1 k s p ou1 zh @ r ) ((pos "M6%" ) ))
( "exposures"  n ( i1 k s p ou1 zh @ z ) ((pos "Mj%" ) ))
( "expound"  v ( i1 k s p au1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "expounded"  v ( i1 k s p au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "expounding"  v ( i1 k s p au1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "expounds"  v ( i1 k s p au1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "express"  v ( i1 k s p r e1 s ) ((pos "H1%" "M7%" "OA%" "Pu%" ) (subcat "6A" "10" "14" )))
( "expressed"  v ( i1 k s p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "10" "14" )))
( "expresses"  v ( i1 k s p r e1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "10" "14" )))
( "expressing"  v ( i1 k s p r e1 s i ng ) ((pos "Hb%" ) (subcat "6A" "10" "14" )))
( "expression"  n ( i1 k s p r e1 sh @ n ) ((pos "M6%" ) ))
( "expressionism"  n ( i1 k s p r e1 sh @ n i z @ m ) ((pos "L@%" ) ))
( "expressionist"  n ( i1 k s p r e1 sh @ n i s t ) ((pos "K6%" ) ))
( "expressionists"  n ( i1 k s p r e1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "expressionless"  j ( i1 k s p r e1 sh @ n l @ s ) ((pos "OA%" ) ))
( "expressions"  n ( i1 k s p r e1 sh @ n z ) ((pos "Mj%" ) ))
( "expressive"  j ( i1 k s p r e1 s i v ) ((pos "OA%" ) ))
( "expressively"  a ( i1 k s p r e1 s i v l ii ) ((pos "Pu%" ) ))
( "expressly"  a ( i1 k s p r e1 s l ii ) ((pos "Pu%" ) ))
( "expressway"  n ( i1 k s p r e1 s w ei ) ((pos "K6$" ) ))
( "expressways"  n ( i1 k s p r e1 s w ei z ) ((pos "Kj$" ) ))
( "expropriate"  v ( e1 k s p r ou1 p r i ei t ) ((pos "H2$" ) (subcat "6A" )))
( "expropriated"  v ( e1 k s p r ou1 p r i ei t i d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "expropriates"  v ( e1 k s p r ou1 p r i ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "expropriating"  v ( e1 k s p r ou1 p r i ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "expropriation"  n ( e2 k s p r ou2 p r i ei1 sh @ n ) ((pos "L@$" ) ))
( "expulsion"  n ( i1 k s p uh1 l sh @ n ) ((pos "M6%" ) ))
( "expulsions"  n ( i1 k s p uh1 l sh @ n z ) ((pos "Mj%" ) ))
( "expunge"  v ( i1 k s p uh1 n jh ) ((pos "H2%" ) (subcat "6A" )))
( "expunged"  v ( i1 k s p uh1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "expunges"  v ( i1 k s p uh1 n jh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "expunging"  v ( i1 k s p uh1 n jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "expurgate"  v ( e1 k s p @ g ei t ) ((pos "H2%" ) (subcat )))
( "expurgated"  v ( e1 k s p @ g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "expurgates"  v ( e1 k s p @ g ei t s ) ((pos "Ha%" ) (subcat )))
( "expurgating"  v ( e1 k s p @ g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "expurgation"  n ( e2 k s p @ g ei1 sh @ n ) ((pos "K6%" ) ))
( "expurgations"  n ( e2 k s p @ g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "exquisite"  j ( e1 k s k w i z i t ) ((pos "OA%" ) ))
( "exquisitely"  a ( e1 k s k w i z i t l ii ) ((pos "Pu%" ) ))
( "exquisiteness"  n ( e1 k s k w i z i t n @ s ) ((pos "L@%" ) ))
( "extant"  j ( e1 k s t a1 n t ) ((pos "OA%" ) ))
( "extemporaneous"  j ( e1 k s t e2 m p @ r ei1 n i@ s ) ((pos "OA$" ) ))
( "extemporaneously"  a ( e1 k s t e2 m p @ r ei1 n i@ s l ii ) ((pos "Pu$" ) ))
( "extemporarily"  a ( i1 k s t e2 m p @ r e@1 r i l ii ) ((pos "Pu$" ) ))
( "extemporary"  j ( i1 k s t e1 m p @ r @ r ii ) ((pos "OA$" ) ))
( "extempore"  j ( e1 k s t e1 m p @ r ii ) ((pos "OA%" "Pu%" ) ))
( "extemporize"  v ( i1 k s t e1 m p @ r ai z ) ((pos "J3%" ) (subcat "2A" )))
( "extemporized"  v ( i1 k s t e1 m p @ r ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "extemporizes"  v ( i1 k s t e1 m p @ r ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "extemporizing"  v ( i1 k s t e1 m p @ r ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "extend"  v ( i1 k s t e1 n d ) ((pos "J0%" ) (subcat "2B" "2C" "6A" "14" )))
( "extended"  v ( i1 k s t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2B" "2C" "6A" "14" )))
( "extending"  v ( i1 k s t e1 n d i ng ) ((pos "Jb%" ) (subcat "2B" "2C" "6A" "14" )))
( "extends"  v ( i1 k s t e1 n d z ) ((pos "Ja%" ) (subcat "2B" "2C" "6A" "14" )))
( "extension"  n ( i1 k s t e1 n sh @ n ) ((pos "M6%" ) ))
( "extensions"  n ( i1 k s t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "extensive"  j ( i1 k s t e1 n s i v ) ((pos "OA%" ) ))
( "extensively"  a ( i1 k s t e1 n s i v l ii ) ((pos "Pu%" ) ))
( "extent"  n ( i1 k s t e1 n t ) ((pos "L@%" ) ))
( "extenuate"  v ( i1 k s t e1 n y u ei t ) ((pos "H2%" ) (subcat )))
( "extenuated"  v ( i1 k s t e1 n y u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "extenuates"  v ( i1 k s t e1 n y u ei t s ) ((pos "Ha%" ) (subcat )))
( "extenuating"  v ( i1 k s t e1 n y u ei t i ng ) ((pos "Hb%" ) (subcat )))
( "extenuation"  n ( i1 k s t e2 n y u ei1 sh @ n ) ((pos "M6%" ) ))
( "extenuations"  n ( i1 k s t e2 n y u ei1 sh @ n z ) ((pos "Mj$" ) ))
( "exterior"  n ( i1 k s t i@1 r i@ r ) ((pos "K6%" "OA%" ) ))
( "exteriorize"  v ( i1 k s t i@1 r i@ r ai z ) ((pos "H2$" ) ))
( "exteriorized"  v ( i1 k s t i@1 r i@ r ai z d ) ((pos "Hc$" "Hd$" ) ))
( "exteriorizes"  v ( i1 k s t i@1 r i@ r ai z i z ) ((pos "Ha$" ) ))
( "exteriorizing"  v ( i1 k s t i@1 r i@ r ai z i ng ) ((pos "Hb$" ) ))
( "exteriors"  n ( i1 k s t i@1 r i@ z ) ((pos "Kj%" ) ))
( "exterminate"  v ( i1 k s t @@1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "exterminated"  v ( i1 k s t @@1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "exterminates"  v ( i1 k s t @@1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "exterminating"  v ( i1 k s t @@1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "extermination"  n ( i1 k s t @@2 m i n ei1 sh @ n ) ((pos "K6%" ) ))
( "exterminations"  n ( i1 k s t @@2 m i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "external"  n ( i1 k s t @@1 n l ) ((pos "K6%" "OA%" ) ))
( "externalize"  v ( i1 k s t @@1 n @ l ai z ) ((pos "H2%" ) (subcat )))
( "externalized"  v ( i1 k s t @@1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "externalizes"  v ( i1 k s t @@1 n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "externalizing"  v ( i1 k s t @@1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "externally"  a ( e1 k s t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "externals"  n ( i1 k s t @@1 n l z ) ((pos "Kj%" ) ))
( "exterritorial"  j ( e2 k s t e2 r i t oo1 r i@ l ) ((pos "OA$" ) ))
( "extinct"  j ( i1 k s t i1 ng t ) ((pos "OA%" ) ))
( "extinction"  n ( i1 k s t i1 ng k sh @ n ) ((pos "L@%" ) ))
( "extinguish"  v ( i1 k s t i1 ng w i sh ) ((pos "H1%" ) (subcat )))
( "extinguished"  v ( i1 k s t i1 ng w i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "extinguisher"  n ( i1 k s t i1 ng w i sh @ r ) ((pos "K6%" ) ))
( "extinguishers"  n ( i1 k s t i1 ng w i sh @ z ) ((pos "Kj%" ) ))
( "extinguishes"  v ( i1 k s t i1 ng w i sh i z ) ((pos "Ha%" ) (subcat )))
( "extinguishing"  v ( i1 k s t i1 ng w i sh i ng ) ((pos "Hb%" ) (subcat )))
( "extirpate"  v ( e1 k s t @ p ei t ) ((pos "H2$" ) (subcat )))
( "extirpated"  v ( e1 k s t @ p ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "extirpates"  v ( e1 k s t @ p ei t s ) ((pos "Ha$" ) (subcat )))
( "extirpating"  v ( e1 k s t @ p ei t i ng ) ((pos "Hb$" ) (subcat )))
( "extirpation"  n ( e2 k s t @ p ei1 sh @ n ) ((pos "L@$" ) ))
( "extol"  v ( i1 k s t ou1 l ) ((pos "H4%" ) (subcat "6A" )))
( "extolled"  v ( i1 k s t ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "extolling"  v ( i1 k s t ou1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "extols"  v ( i1 k s t ou1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "extort"  v ( i1 k s t oo1 t ) ((pos "H0%" ) (subcat "6A" )))
( "extorted"  v ( i1 k s t oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "extorting"  v ( i1 k s t oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "extortion"  n ( i1 k s t oo1 sh @ n ) ((pos "M6%" ) ))
( "extortionate"  j ( i1 k s t oo1 sh @ n @ t ) ((pos "OA%" ) ))
( "extortionately"  a ( i1 k s t oo1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "extortions"  n ( i1 k s t oo1 sh @ n z ) ((pos "Mj$" ) ))
( "extorts"  v ( i1 k s t oo1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "extra"  n ( e1 k s t r @ ) ((pos "K6%" "OA%" "Pu%" ) ))
( "extract"  n ( e1 k s t r a k t ) ((pos "M6%" ) ))
( "extract"  v ( i1 k s t r a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "extracted"  v ( i1 k s t r a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "extracting"  v ( i1 k s t r a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "extraction"  n ( i1 k s t r a1 k sh @ n ) ((pos "M6%" ) ))
( "extractions"  n ( i1 k s t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "extracts"  n ( e1 k s t r a k t s ) ((pos "Mj%" ) ))
( "extracts"  v ( i1 k s t r a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "extracurricular"  j ( e2 k s t r @ k @ r i1 k y u l @ r ) ((pos "OA%" ) ))
( "extradite"  v ( e1 k s t r @ d ai t ) ((pos "H2%" ) (subcat )))
( "extradited"  v ( e1 k s t r @ d ai t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "extradites"  v ( e1 k s t r @ d ai t s ) ((pos "Ha%" ) (subcat )))
( "extraditing"  v ( e1 k s t r @ d ai t i ng ) ((pos "Hb%" ) (subcat )))
( "extradition"  n ( e2 k s t r @ d i1 sh @ n ) ((pos "M6%" ) ))
( "extraditions"  n ( e2 k s t r @ d i1 sh @ n z ) ((pos "Mj%" ) ))
( "extrajudicial"  j ( e2 k s t r @ jh uu d i1 sh l ) ((pos "OA$" ) ))
( "extramarital"  j ( e2 k s t r @ m a1 r i t l ) ((pos "OA%" ) ))
( "extramural"  j ( e2 k s t r @ m y u@1 r @ l ) ((pos "OA%" ) ))
( "extraneous"  j ( i1 k s t r ei1 n i@ s ) ((pos "OA%" ) ))
( "extraordinarily"  a ( i1 k s t r oo1 d n r @ l ii ) ((pos "Pu%" ) ))
( "extraordinary"  j ( i1 k s t r oo1 d n r ii ) ((pos "OA%" ) ))
( "extrapolate"  v ( i1 k s t r a1 p @ l ei t ) ((pos "J3%" ) (subcat )))
( "extrapolated"  v ( i1 k s t r a1 p @ l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "extrapolates"  v ( i1 k s t r a1 p @ l ei t s ) ((pos "Ja%" ) (subcat )))
( "extrapolating"  v ( i1 k s t r a1 p @ l ei t i ng ) ((pos "Jb%" ) (subcat )))
( "extrapolation"  n ( i1 k s t r a2 p @ l ei1 sh @ n ) ((pos "L@%" ) ))
( "extras"  n ( e1 k s t r @ z ) ((pos "Kj%" ) ))
( "extrasensory"  j ( e2 k s t r @ s e1 n s @ r ii ) ((pos "OA%" ) ))
( "extraterritorial"  j ( e2 k s t r @ t e2 r i t oo1 r i@ l ) ((pos "OA$" ) ))
( "extravagance"  n ( i1 k s t r a1 v @ g @ n s ) ((pos "M6%" ) ))
( "extravagances"  n ( i1 k s t r a1 v @ g @ n s i z ) ((pos "Mj%" ) ))
( "extravagant"  j ( i1 k s t r a1 v @ g @ n t ) ((pos "OA%" ) ))
( "extravagantly"  a ( i1 k s t r a1 v @ g @ n t l ii ) ((pos "Pu%" ) ))
( "extravaganza"  n ( i1 k s t r a2 v @ g a1 n z @ ) ((pos "K6%" ) ))
( "extravaganzas"  n ( i1 k s t r a2 v @ g a1 n z @ z ) ((pos "Kj%" ) ))
( "extreme"  n ( i1 k s t r ii1 m ) ((pos "K6%" "OA%" ) ))
( "extremely"  a ( i1 k s t r ii1 m l ii ) ((pos "Pu%" ) ))
( "extremes"  n ( i1 k s t r ii1 m z ) ((pos "Kj%" ) ))
( "extremist"  n ( i1 k s t r ii1 m i s t ) ((pos "K6%" ) ))
( "extremists"  n ( i1 k s t r ii1 m i s t s ) ((pos "Kj%" ) ))
( "extremities"  n ( i1 k s t r e1 m i t i z ) ((pos "Kj%" ) ))
( "extremity"  n ( i1 k s t r e1 m i t ii ) ((pos "K8%" ) ))
( "extricable"  j ( e1 k s t r i1 k @ b l ) ((pos "OA$" ) ))
( "extricate"  v ( e1 k s t r i k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "extricated"  v ( e1 k s t r i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "extricates"  v ( e1 k s t r i k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "extricating"  v ( e1 k s t r i k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "extrication"  n ( e2 k s t r i k ei1 sh @ n ) ((pos "L@$" ) ))
( "extrinsic"  j ( e1 k s t r i1 n s i k ) ((pos "OA$" ) ))
( "extroversion"  n ( e2 k s t r @ v @@1 sh @ n ) ((pos "L@$" ) ))
( "extrovert"  n ( e1 k s t r @ v @@ t ) ((pos "K6%" ) ))
( "extroverts"  n ( e1 k s t r @ v @@ t s ) ((pos "Kj%" ) ))
( "extrude"  v ( i1 k s t r uu1 d ) ((pos "H2$" ) (subcat "6A" )))
( "extruded"  v ( i1 k s t r uu1 d i d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "extrudes"  v ( i1 k s t r uu1 d z ) ((pos "Ha$" ) (subcat "6A" )))
( "extruding"  v ( i1 k s t r uu1 d i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "extrusion"  n ( i1 k s t r uu1 zh n ) ((pos "M6$" ) ))
( "extrusions"  n ( i1 k s t r uu1 zh n z ) ((pos "Mj$" ) ))
( "exuberance"  n ( i1 g z y uu1 b @ r @ n s ) ((pos "L@%" ) ))
( "exuberant"  j ( i1 g z y uu1 b @ r @ n t ) ((pos "OA%" ) ))
( "exuberantly"  a ( i1 g z y uu1 b @ r @ n t l ii ) ((pos "Pu%" ) ))
( "exude"  v ( i1 g z y uu1 d ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "exuded"  v ( i1 g z y uu1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "exudes"  v ( i1 g z y uu1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "exuding"  v ( i1 g z y uu1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "exult"  v ( i1 g z uh1 l t ) ((pos "I0%" ) (subcat "2A" "3A" )))
( "exultant"  j ( i1 g z uh1 l t @ n t ) ((pos "OA%" ) ))
( "exultantly"  a ( i1 g z uh1 l t @ n t l ii ) ((pos "Pu%" ) ))
( "exultation"  n ( e2 g z uh l t ei1 sh @ n ) ((pos "L@%" ) ))
( "exulted"  v ( i1 g z uh1 l t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "exulting"  v ( i1 g z uh1 l t i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "exults"  v ( i1 g z uh1 l t s ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "eye"  v ( ai1 ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "eye-catching"  j ( ai1 - k a ch i ng ) ((pos "OA%" ) ))
( "eye-opener"  n ( ai1 - ou p @ n @ r ) ((pos "K6%" ) ))
( "eye-openers"  n ( ai1 - ou p @ n @ z ) ((pos "Kj%" ) ))
( "eye-shadow"  n ( ai1 - sh a d ou ) ((pos "L@%" ) ))
( "eyeball"  n ( ai1 b oo l ) ((pos "K6%" ) ))
( "eyeballs"  n ( ai1 b oo l z ) ((pos "Kj%" ) ))
( "eyebath"  n ( ai1 b aa th ) ((pos "K6%" ) ))
( "eyebaths"  n ( ai1 b aa dh z ) ((pos "Kj%" ) ))
( "eyebrow"  n ( ai1 b r au ) ((pos "K6%" ) ))
( "eyebrows"  n ( ai1 b r au z ) ((pos "Kj%" ) ))
( "eyecatching"  j ( ai1 k a ch i ng ) ((pos "OA%" ) ))
( "eyecup"  n ( ai1 k uh p ) ((pos "K6$" ) ))
( "eyecups"  n ( ai1 k uh p s ) ((pos "Kj$" ) ))
( "eyed"  v ( ai1 d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat "6A" )))
( "eyeful"  n ( ai1 f u l ) ((pos "K6%" ) ))
( "eyefuls"  n ( ai1 f u l z ) ((pos "Kj$" ) ))
( "eyeglass"  n ( ai1 g l aa s ) ((pos "K7%" ) ))
( "eyeglasses"  n ( ai1 g l aa s i z ) ((pos "Kj%" ) ))
( "eyeing"  v ( ai1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "eyelash"  n ( ai1 l a sh ) ((pos "K7%" ) ))
( "eyelashes"  n ( ai1 l a sh i z ) ((pos "Kj%" ) ))
( "eyeless"  j ( ai1 l @ s ) ((pos "OA%" ) ))
( "eyelet"  n ( ai1 l i t ) ((pos "K6$" ) ))
( "eyelets"  n ( ai1 l i t s ) ((pos "Kj$" ) ))
( "eyelid"  n ( ai1 l i d ) ((pos "K6%" ) ))
( "eyelids"  n ( ai1 l i d z ) ((pos "Kj%" ) ))
( "eyepiece"  n ( ai1 p ii s ) ((pos "K6$" ) ))
( "eyepieces"  n ( ai1 p ii s i z ) ((pos "Kj$" ) ))
( "eyes"  v ( ai1 z ) ((pos "Ha%" "Kj*" ) (subcat "6A" )))
( "eyeshot"  n ( ai1 sh o t ) ((pos "L@$" ) ))
( "eyesight"  n ( ai1 s ai t ) ((pos "L@%" ) ))
( "eyesore"  n ( ai1 s oo r ) ((pos "K6%" ) ))
( "eyesores"  n ( ai1 s oo z ) ((pos "Kj%" ) ))
( "eyestrain"  n ( ai1 s t r ei n ) ((pos "L@%" ) ))
( "eyeteeth"  n ( ai1 t ii th ) ((pos "Kj$" ) ))
( "eyetooth"  n ( ai1 t uu th ) ((pos "Ki$" ) ))
( "eyewash"  n ( ai1 w o sh ) ((pos "L@%" ) ))
( "eyewitness"  n ( ai1 w i t n @ s ) ((pos "K7%" ) ))
( "eyewitnesses"  n ( ai1 w i t n @ s i z ) ((pos "Kj%" ) ))
( "eying"  v ( ai1 i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "eyrie"  n ( e@1 r ii ) ((pos "K6$" ) ))
( "eyries"  n ( e@1 r i z ) ((pos "Kj$" ) ))
( "eyry"  n ( e@1 r ii ) ((pos "K8%" ) ))
( "f"  n ( e1 f ) ((pos "Ki$" ) ))
( "f\"uhrer"  n ( f y u@1 r @ r ) ((pos "K6$" ) ))
( "f\"uhrers"  n ( f y u@1 r @ z ) ((pos "Kj$" ) ))
( "f's"  n ( e1 f s ) ((pos "Kj$" ) ))
( "f^ete"  v ( f ei1 t ) ((pos "H2%" "K6%" ) (subcat )))
( "f^eted"  v ( f ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "f^etes"  v ( f ei1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "f^eting"  v ( f ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "fa"  n ( f aa1 ) ((pos "Ki$" ) ))
( "fa<cade"  n ( f @1 s aa1 d ) ((pos "K6%" ) ))
( "fa<cades"  n ( f @1 s aa1 d z ) ((pos "Kj%" ) ))
( "fab"  j ( f a1 b ) ((pos "OA%" ) ))
( "fable"  n ( f ei1 b l ) ((pos "M6%" ) ))
( "fabled"  j ( f ei1 b l d ) ((pos "OA%" ) ))
( "fables"  n ( f ei1 b l z ) ((pos "Mj%" ) ))
( "fabric"  n ( f a1 b r i k ) ((pos "M6%" ) ))
( "fabricate"  v ( f a1 b r i k ei t ) ((pos "H2%" ) (subcat )))
( "fabricated"  v ( f a1 b r i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fabricates"  v ( f a1 b r i k ei t s ) ((pos "Ha%" ) (subcat )))
( "fabricating"  v ( f a1 b r i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "fabrication"  n ( f a2 b r i k ei1 sh @ n ) ((pos "M6%" ) ))
( "fabrications"  n ( f a2 b r i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fabrics"  n ( f a1 b r i k s ) ((pos "Mj%" ) ))
( "fabulous"  j ( f a1 b y u l @ s ) ((pos "OA%" ) ))
( "fabulously"  a ( f a1 b y u l @ s l ii ) ((pos "Pu%" ) ))
( "face"  v ( f ei1 s ) ((pos "J2*" "K6*" ) (subcat "2C" "6A" "6C" "14" )))
( "face-ache"  n ( f ei1 s - ei k ) ((pos "L@%" ) ))
( "face-card"  n ( f ei1 s - k aa d ) ((pos "K6%" ) ))
( "face-cards"  n ( f ei1 s - k aa d z ) ((pos "Kj%" ) ))
( "face-cloth"  n ( f ei1 s - k l o th ) ((pos "K6%" ) ))
( "face-cloths"  n ( f ei1 s - k l o th s ) ((pos "Kj%" ) ))
( "face-cream"  n ( f ei1 s - k r ii m ) ((pos "M6%" ) ))
( "face-creams"  n ( f ei1 s - k r ii m z ) ((pos "Mj%" ) ))
( "face-lift"  n ( f ei1 s - l i f t ) ((pos "K6%" ) ))
( "face-lifting"  n ( f ei1 s - l i f t i ng ) ((pos "K6%" ) ))
( "face-liftings"  n ( f ei1 s - l i f t i ng z ) ((pos "Kj$" ) ))
( "face-lifts"  n ( f ei1 s - l i f t s ) ((pos "Kj%" ) ))
( "face-pack"  n ( f ei1 s - p a k ) ((pos "K6%" ) ))
( "face-packs"  n ( f ei1 s - p a k s ) ((pos "Kj%" ) ))
( "face-powder"  n ( f ei1 s - p au d @ r ) ((pos "M6%" ) ))
( "face-powders"  n ( f ei1 s - p au d @ z ) ((pos "Mj%" ) ))
( "face-saver"  n ( f ei1 s - s ei v @ r ) ((pos "K6%" ) ))
( "face-savers"  n ( f ei1 s - s ei v @ z ) ((pos "Kj%" ) ))
( "face-saving"  n ( f ei1 s - s ei v i ng ) ((pos "L@%" "OA%" ) ))
( "faced"  v ( f ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "6C" "14" )))
( "faceless"  j ( f ei1 s l @ s ) ((pos "OA%" ) ))
( "facer"  n ( f ei1 s @ r ) ((pos "K6$" ) ))
( "facers"  n ( f ei1 s @ z ) ((pos "Kj$" ) ))
( "faces"  v ( f ei1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "6C" "14" )))
( "facet"  n ( f a1 s i t ) ((pos "K6%" ) ))
( "facetious"  j ( f @1 s ii1 sh @ s ) ((pos "OA%" ) ))
( "facetiously"  a ( f @1 s ii1 sh @ s l ii ) ((pos "Pu%" ) ))
( "facetiousness"  n ( f @1 s ii1 sh @ s n @ s ) ((pos "L@%" ) ))
( "facets"  n ( f a1 s i t s ) ((pos "Kj%" ) ))
( "facia"  n ( f ei1 sh @ ) ((pos "K6$" ) ))
( "facial"  n ( f ei1 sh l ) ((pos "K6%" "OA%" ) ))
( "facials"  n ( f ei1 sh l z ) ((pos "Kj$" ) ))
( "facias"  n ( f ei1 sh @ z ) ((pos "Kj$" ) ))
( "facile"  j ( f a1 s ai l ) ((pos "OA%" ) ))
( "facilitate"  v ( f @1 s i1 l i t ei t ) ((pos "H2%" ) (subcat )))
( "facilitated"  v ( f @1 s i1 l i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "facilitates"  v ( f @1 s i1 l i t ei t s ) ((pos "Ha%" ) (subcat )))
( "facilitating"  v ( f @1 s i1 l i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "facilities"  n ( f @1 s i1 l i t i z ) ((pos "Mj%" ) ))
( "facility"  n ( f @1 s i1 l i t ii ) ((pos "M8%" ) ))
( "facing"  v ( f ei1 s i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "6C" "14" )))
( "facings"  n ( f ei1 s i ng z ) ((pos "Kj%" ) ))
( "facsimile"  n ( f a1 k s i1 m @ l ii ) ((pos "K6%" ) ))
( "facsimiles"  n ( f a1 k s i1 m @ l i z ) ((pos "Kj%" ) ))
( "fact"  n ( f a1 k t ) ((pos "M6*" ) ))
( "fact-finding"  j ( f a1 k t - f ai n d i ng ) ((pos "OA%" ) ))
( "faction"  n ( f a1 k sh @ n ) ((pos "M6%" ) ))
( "factions"  n ( f a1 k sh @ n z ) ((pos "Mj%" ) ))
( "factious"  j ( f a1 k sh @ s ) ((pos "OA$" ) ))
( "factitious"  j ( f a1 k t i1 sh @ s ) ((pos "OA$" ) ))
( "factor"  n ( f a1 k t @ r ) ((pos "K6%" ) ))
( "factories"  n ( f a1 k t @ r i z ) ((pos "Kj%" ) ))
( "factorize"  v ( f a1 k t @ r ai z ) ((pos "H2$" ) (subcat )))
( "factorized"  v ( f a1 k t @ r ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "factorizes"  v ( f a1 k t @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "factorizing"  v ( f a1 k t @ r ai z i ng ) ((pos "Hb$" ) (subcat )))
( "factors"  n ( f a1 k t @ z ) ((pos "Kj%" ) ))
( "factory"  n ( f a1 k t @ r ii ) ((pos "K8%" ) ))
( "factotum"  n ( f a1 k t ou1 t @ m ) ((pos "K6%" ) ))
( "factotums"  n ( f a1 k t ou1 t @ m z ) ((pos "Kj$" ) ))
( "facts"  n ( f a1 k t s ) ((pos "Mj%" ) ))
( "factual"  j ( f a1 k ch u@ l ) ((pos "OA%" ) ))
( "factually"  a ( f a1 k ch u@ l ii ) ((pos "Pu%" ) ))
( "faculties"  n ( f a1 k l t i z ) ((pos "Kj%" ) ))
( "faculty"  n ( f a1 k l t ii ) ((pos "K8%" ) ))
( "fad"  n ( f a1 d ) ((pos "K6%" ) ))
( "faddier"  j ( f a1 d i@ r ) ((pos "Or%" ) ))
( "faddiest"  j ( f a1 d i i s t ) ((pos "Os%" ) ))
( "faddily"  a ( f a1 d @ l ii ) ((pos "Pu%" ) ))
( "faddy"  j ( f a1 d ii ) ((pos "OD%" ) ))
( "fade"  v ( f ei1 d ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "6A" )))
( "faded"  v ( f ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fades"  v ( f ei1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fading"  v ( f ei1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fads"  n ( f a1 d z ) ((pos "Kj%" ) ))
( "faeces"  n ( f ii1 s ii z ) ((pos "Kj%" ) ))
( "faerie"  n ( f e@1 r ii ) ((pos "K6$" ) ))
( "faeries"  n ( f e@1 r i z ) ((pos "Kj$" ) ))
( "faery"  n ( f e@1 r ii ) ((pos "K8$" ) ))
( "fag"  v ( f a1 g ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fag-end"  n ( f a1 g - e n d ) ((pos "K6%" ) ))
( "fag-ends"  n ( f a1 g - e n d z ) ((pos "Kj%" ) ))
( "fagged"  v ( f a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fagging"  v ( f a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "faggot"  n ( f a1 g @ t ) ((pos "K6%" ) ))
( "faggots"  n ( f a1 g @ t s ) ((pos "Kj%" ) ))
( "fags"  v ( f a1 g z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "faience"  n ( f ei2 aa1 n s ) ((pos "L@$" ) ))
( "fail"  v ( f ei1 l ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "3A" "4A" )))
( "fail-safe"  j ( f ei1 l - s ei f ) ((pos "OA%" ) ))
( "failed"  v ( f ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" )))
( "failing"  v ( f ei1 l i ng ) ((pos "Jb%" "K6%" "T-%" ) (subcat "2A" "2C" "3A" "4A" )))
( "failings"  n ( f ei1 l i ng z ) ((pos "Kj%" ) ))
( "fails"  v ( f ei1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "4A" )))
( "failure"  n ( f ei1 l y @ r ) ((pos "M6%" ) ))
( "failures"  n ( f ei1 l y @ z ) ((pos "Mj%" ) ))
( "fain"  a ( f ei1 n ) ((pos "Pu$" ) ))
( "faint"  v ( f ei1 n t ) ((pos "I0%" "K6%" "OC%" ) (subcat "2A" )))
( "faint-hearted"  j ( f ei1 n t - h aa1 t i d ) ((pos "OA%" ) ))
( "fainted"  v ( f ei1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fainter"  j ( f ei1 n t @ r ) ((pos "Or%" ) ))
( "faintest"  j ( f ei1 n t i s t ) ((pos "Os%" ) ))
( "fainting"  v ( f ei1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "faintly"  a ( f ei1 n t l ii ) ((pos "Pu%" ) ))
( "faintness"  n ( f ei1 n t n @ s ) ((pos "L@%" ) ))
( "faints"  v ( f ei1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "fair"  n ( f e@1 r ) ((pos "K6%" "OC%" "Pu%" ) ))
( "fair-minded"  j ( f e@1 - m ai1 n d i d ) ((pos "OA%" ) ))
( "fairer"  j ( f e@1 r @ r ) ((pos "Or%" ) ))
( "fairest"  j ( f e@1 r i s t ) ((pos "Os%" ) ))
( "fairground"  n ( f e@1 g r au n d ) ((pos "K6%" ) ))
( "fairgrounds"  n ( f e@1 g r au n d z ) ((pos "Kj%" ) ))
( "fairies"  n ( f e@1 r i z ) ((pos "Kj%" ) ))
( "fairish"  j ( f e@1 r i sh ) ((pos "OA%" ) ))
( "fairly"  a ( f e@1 l ii ) ((pos "Pu%" ) ))
( "fairness"  n ( f e@1 n i s ) ((pos "L@%" ) ))
( "fairs"  n ( f e@1 z ) ((pos "Kj%" ) ))
( "fairway"  n ( f e@1 w ei ) ((pos "K6%" ) ))
( "fairways"  n ( f e@1 w ei z ) ((pos "Kj%" ) ))
( "fairy"  n ( f e@1 r ii ) ((pos "K8%" ) ))
( "fairyland"  n ( f e@1 r i l a n d ) ((pos "K6%" ) ))
( "fairylands"  n ( f e@1 r i l a n d z ) ((pos "Kj$" ) ))
( "fairytale"  n ( f e@1 r i t ei l ) ((pos "K6%" ) ))
( "fairytales"  n ( f e@1 r i t ei l z ) ((pos "Kj%" ) ))
( "fait accompli"  n ( f ei2 t - @ k o1 m p l ii ) ((pos "Ki%" ) ))
( "faith"  n ( f ei1 th ) ((pos "M6%" ) ))
( "faith-healing"  n ( f ei1 th - h ii l i ng ) ((pos "L@%" ) ))
( "faithful"  j ( f ei1 th f @ l ) ((pos "OA%" ) ))
( "faithfully"  a ( f ei1 th f @ l ii ) ((pos "Pu%" ) ))
( "faithfulness"  n ( f ei1 th f l n @ s ) ((pos "L@%" ) ))
( "faithless"  j ( f ei1 th l i s ) ((pos "OA%" ) ))
( "faithlessly"  a ( f ei1 th l i s l ii ) ((pos "Pu%" ) ))
( "faithlessness"  n ( f ei1 th l i s n @ s ) ((pos "L@%" ) ))
( "faiths"  n ( f ei1 th s ) ((pos "Mj%" ) ))
( "faits accomplis"  n ( f ei2 t - @ k o1 m p l ii ) ((pos "Kj$" ) ))
( "fake"  v ( f ei1 k ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "faked"  v ( f ei1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fakes"  v ( f ei1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "faking"  v ( f ei1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fakir"  n ( f ei1 k i@ r ) ((pos "K6$" ) ))
( "fakirs"  n ( f ei1 k i@ z ) ((pos "Kj$" ) ))
( "falcon"  n ( f oo1 l k @ n ) ((pos "K6%" ) ))
( "falconry"  n ( f o1 l k @ n r ii ) ((pos "L@%" ) ))
( "falcons"  n ( f oo1 l k @ n z ) ((pos "Kj%" ) ))
( "fall"  v ( f oo1 l ) ((pos "I5%" "K6%" ) (subcat "2A" "2B" "2C" "2D" )))
( "fallacies"  n ( f a1 l @ s i z ) ((pos "Mj%" ) ))
( "fallacious"  j ( f @1 l ei1 sh @ s ) ((pos "OA%" ) ))
( "fallacy"  n ( f a1 l @ s ii ) ((pos "M8%" ) ))
( "fallen"  v ( f oo1 l @ n ) ((pos "Id%" ) (subcat "2A" "2B" "2C" "2D" )))
( "fallibility"  n ( f a2 l @ b i1 l i t ii ) ((pos "L@%" ) ))
( "fallible"  j ( f a1 l @ b l ) ((pos "OA%" ) ))
( "falling"  v ( f oo1 l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" "2D" )))
( "fallout"  n ( f oo1 l au t ) ((pos "L@%" ) ))
( "fallow"  n ( f a1 l ou ) ((pos "L@%" "OA%" ) ))
( "fallow-deer"  n ( f a1 l ou - d i@ r ) ((pos "K9$" ) ))
( "falls"  v ( f oo1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" )))
( "false"  j ( f oo1 l s ) ((pos "OA%" "Pu%" ) ))
( "falsehood"  n ( f oo1 l s h u d ) ((pos "M6%" ) ))
( "falsehoods"  n ( f oo1 l s h u d z ) ((pos "Mj%" ) ))
( "falsely"  a ( f oo1 l s l ii ) ((pos "Pu%" ) ))
( "falseness"  n ( f oo1 l s n @ s ) ((pos "L@%" ) ))
( "falsetto"  n ( f oo1 l s e1 t ou ) ((pos "K6%" "OA%" ) ))
( "falsettos"  n ( f oo1 l s e1 t ou z ) ((pos "Kj%" ) ))
( "falsies"  n ( f oo1 l s i z ) ((pos "Kj%" ) ))
( "falsification"  n ( f oo2 l s i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "falsifications"  n ( f oo2 l s i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "falsified"  v ( f oo1 l s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "falsifies"  v ( f oo1 l s i f ai z ) ((pos "Ha%" ) (subcat )))
( "falsify"  v ( f oo1 l s i f ai ) ((pos "H3%" ) (subcat )))
( "falsifying"  v ( f oo1 l s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "falsities"  n ( f oo1 l s i t i z ) ((pos "Mj$" ) ))
( "falsity"  n ( f oo1 l s i t ii ) ((pos "M8%" ) ))
( "falter"  v ( f oo1 l t @ r ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "faltered"  v ( f oo1 l t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "faltering"  v ( f oo1 l t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "falteringly"  a ( f oo1 l t @ r i ng l ii ) ((pos "Pu%" ) ))
( "falters"  v ( f oo1 l t @ z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "fame"  n ( f ei1 m ) ((pos "L@%" ) ))
( "famed"  j ( f ei1 m d ) ((pos "OA%" ) ))
( "familiar"  n ( f @1 m i1 l i@ r ) ((pos "K6$" "OA%" ) ))
( "familiarities"  n ( f @1 m i2 l i a1 r i t i z ) ((pos "Mj%" ) ))
( "familiarity"  n ( f @1 m i2 l i a1 r i t ii ) ((pos "M8%" ) ))
( "familiarize"  v ( f @1 m i1 l i@ r ai z ) ((pos "H2%" ) (subcat "6A" )))
( "familiarized"  v ( f @1 m i1 l i@ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "familiarizes"  v ( f @1 m i1 l i@ r ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "familiarizing"  v ( f @1 m i1 l i@ r ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "familiarly"  a ( f @1 m i1 l i@ l ii ) ((pos "Pu%" ) ))
( "familiars"  n ( f @1 m i1 l i@ z ) ((pos "Kj$" ) ))
( "families"  n ( f a1 m @ l i z ) ((pos "Mj%" ) ))
( "family"  n ( f a1 m @ l ii ) ((pos "M8*" ) ))
( "famine"  n ( f a1 m i n ) ((pos "M6%" ) ))
( "famines"  n ( f a1 m i n z ) ((pos "Mj%" ) ))
( "famish"  v ( f a1 m i sh ) ((pos "J1$" ) (subcat "2A" "3A" )))
( "famished"  v ( f a1 m i sh t ) ((pos "Jc$" "Jd%" ) (subcat "2A" "3A" )))
( "famishes"  v ( f a1 m i sh i z ) ((pos "Ja$" ) (subcat "2A" "3A" )))
( "famishing"  v ( f a1 m i sh i ng ) ((pos "Jb$" ) (subcat "2A" "3A" )))
( "famous"  j ( f ei1 m @ s ) ((pos "OA%" ) ))
( "famously"  a ( f ei1 m @ s l ii ) ((pos "Pu%" ) ))
( "fan"  v ( f a1 n ) ((pos "J4%" "K6%" ) (subcat "2C" )))
( "fan-belt"  n ( f a1 n - b e l t ) ((pos "K6%" ) ))
( "fan-belts"  n ( f a1 n - b e l t s ) ((pos "Kj%" ) ))
( "fanatic"  n ( f @1 n a1 t i k ) ((pos "K6%" "OA%" ) ))
( "fanatical"  j ( f @1 n a1 t i k l ) ((pos "OA%" ) ))
( "fanatically"  a ( f @1 n a1 t i k l ii ) ((pos "Pu%" ) ))
( "fanaticism"  n ( f @1 n a1 t i s i z @ m ) ((pos "M6%" ) ))
( "fanaticisms"  n ( f @1 n a1 t i s i z @ m z ) ((pos "Mj$" ) ))
( "fanatics"  n ( f @1 n a1 t i k s ) ((pos "Kj%" ) ))
( "fancied"  v ( f a1 n s i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "16B" "19C" )))
( "fancier"  n ( f a1 n s i@ r ) ((pos "K6%" "Or%" ) ))
( "fanciers"  n ( f a1 n s i@ z ) ((pos "Kj%" ) ))
( "fancies"  v ( f a1 n s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6C" "9" "16B" "19C" )))
( "fanciest"  j ( f a1 n s i i s t ) ((pos "Os%" ) ))
( "fanciful"  j ( f a1 n s i f @ l ) ((pos "OA%" ) ))
( "fancifully"  a ( f a1 n s i f @ l ii ) ((pos "Pu%" ) ))
( "fancy"  v ( f a1 n s ii ) ((pos "H3%" "M8%" "OD%" ) (subcat "6A" "6C" "9" "16B" "19C" )))
( "fancy-free"  j ( f a2 n s i - f r ii1 ) ((pos "OA%" ) ))
( "fancying"  v ( f a1 n s i i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "16B" "19C" )))
( "fandango"  n ( f a2 n d a1 ng g ou ) ((pos "K6$" ) ))
( "fandangos"  n ( f a2 n d a1 ng g ou z ) ((pos "Kj$" ) ))
( "fanfare"  n ( f a1 n f e@ r ) ((pos "K6%" ) ))
( "fanfares"  n ( f a1 n f e@ z ) ((pos "Kj%" ) ))
( "fang"  n ( f a1 ng ) ((pos "K6%" ) ))
( "fangs"  n ( f a1 ng z ) ((pos "Kj%" ) ))
( "fanlight"  n ( f a1 n l ai t ) ((pos "K6%" ) ))
( "fanlights"  n ( f a1 n l ai t s ) ((pos "Kj%" ) ))
( "fanned"  v ( f a1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "fannies"  n ( f a1 n i z ) ((pos "Kj$" ) ))
( "fanning"  v ( f a1 n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "fanny"  n ( f a1 n ii ) ((pos "K8%" ) ))
( "fans"  v ( f a1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "fantan"  n ( f a2 n t a1 n ) ((pos "L@$" ) ))
( "fantasia"  n ( f a1 n t ei1 z i@ ) ((pos "K6%" ) ))
( "fantasias"  n ( f a1 n t ei1 z i@ z ) ((pos "Kj%" ) ))
( "fantasies"  n ( f a1 n t @ s i z ) ((pos "Mj%" ) ))
( "fantastic"  j ( f a1 n t a1 s t i k ) ((pos "OA%" ) ))
( "fantastically"  a ( f a1 n t a1 s t i k l ii ) ((pos "Pu%" ) ))
( "fantasy"  n ( f a1 n t @ s ii ) ((pos "M8%" ) ))
( "far"  j ( f aa1 r ) ((pos "OA*" "Pu*" ) ))
( "far-famed"  j ( f aa1 - f ei1 m d ) ((pos "OA%" ) ))
( "far-fetched"  j ( f aa1 - f e1 ch t ) ((pos "OA%" ) ))
( "far-flung"  j ( f aa1 - f l uh1 ng ) ((pos "OA%" ) ))
( "far-off"  j ( f aa1 r - o1 f ) ((pos "OA%" ) ))
( "far-reaching"  j ( f aa1 - r ii1 ch i ng ) ((pos "OA%" ) ))
( "far-seeing"  j ( f aa1 - s ii1 i ng ) ((pos "OA%" ) ))
( "far-sighted"  j ( f aa1 - s ai1 t i d ) ((pos "OA%" ) ))
( "faraway"  j ( f aa1 r @ w ei ) ((pos "OA%" ) ))
( "farce"  n ( f aa1 s ) ((pos "M6%" ) ))
( "farces"  n ( f aa1 s i z ) ((pos "Mj%" ) ))
( "farcical"  j ( f aa1 s i k l ) ((pos "OA%" ) ))
( "farcically"  a ( f aa1 s i k l ii ) ((pos "Pu%" ) ))
( "fare"  v ( f e@1 r ) ((pos "I2%" "M6%" ) (subcat )))
( "fare-stage"  n ( f e@1 - s t ei jh ) ((pos "K6%" ) ))
( "fare-stages"  n ( f e@1 - s t ei jh i z ) ((pos "Kj%" ) ))
( "fared"  v ( f e@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "fares"  v ( f e@1 z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "farewell"  n ( f e@2 w e1 l ) ((pos "K6%" "W-%" ) ))
( "farewells"  n ( f e@2 w e1 l z ) ((pos "Kj%" ) ))
( "farinaceous"  j ( f a2 r i n ei1 sh @ s ) ((pos "OA$" ) ))
( "faring"  v ( f e@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "farm"  v ( f aa1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "farmed"  v ( f aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "farmer"  n ( f aa1 m @ r ) ((pos "K6%" ) ))
( "farmers"  n ( f aa1 m @ z ) ((pos "Kj%" ) ))
( "farmhand"  n ( f aa1 m h a n d ) ((pos "K6%" ) ))
( "farmhands"  n ( f aa1 m h a n d z ) ((pos "Kj%" ) ))
( "farmhouse"  n ( f aa1 m h au s ) ((pos "K6%" ) ))
( "farmhouses"  n ( f aa1 m h au z i z ) ((pos "Kj%" ) ))
( "farming"  v ( f aa1 m i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "farms"  v ( f aa1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "farmstead"  n ( f aa1 m s t e d ) ((pos "K6%" ) ))
( "farmsteads"  n ( f aa1 m s t e d z ) ((pos "Kj%" ) ))
( "farmyard"  n ( f aa1 m y aa d ) ((pos "K6%" ) ))
( "farmyards"  n ( f aa1 m y aa d z ) ((pos "Kj%" ) ))
( "farrago"  n ( f @1 r aa1 g ou ) ((pos "K6%" ) ))
( "farragoes"  n ( f @1 r aa1 g ou z ) ((pos "Kj$" ) ))
( "farragos"  n ( f @1 r aa1 g ou z ) ((pos "Kj$" ) ))
( "farrier"  n ( f a1 r i@ r ) ((pos "K6$" ) ))
( "farriers"  n ( f a1 r i@ z ) ((pos "Kj$" ) ))
( "farrow"  v ( f a1 r ou ) ((pos "I0$" "K6$" ) ))
( "farrowed"  v ( f a1 r ou d ) ((pos "Ic$" "Id$" ) ))
( "farrowing"  v ( f a1 r ou i ng ) ((pos "Ib$" ) ))
( "farrows"  v ( f a1 r ou z ) ((pos "Ia$" "Kj$" ) ))
( "fart"  v ( f aa1 t ) ((pos "I0%" "K6%" ) ))
( "farted"  v ( f aa1 t i d ) ((pos "Ic%" "Id%" ) ))
( "farther"  j ( f aa1 dh @ r ) ((pos "Or%" "Pu%" ) ))
( "farthest"  j ( f aa1 dh i s t ) ((pos "Os%" "Pu%" ) ))
( "farthing"  n ( f aa1 dh i ng ) ((pos "K6%" ) ))
( "farthings"  n ( f aa1 dh i ng z ) ((pos "Kj%" ) ))
( "farting"  v ( f aa1 t i ng ) ((pos "Ib%" ) ))
( "farts"  v ( f aa1 t s ) ((pos "Ia%" "Kj%" ) ))
( "fascia"  n ( f ei1 sh @ ) ((pos "K6$" ) ))
( "fascias"  n ( f ei1 sh @ z ) ((pos "Kj$" ) ))
( "fascinate"  v ( f a1 s i n ei t ) ((pos "H2%" ) (subcat )))
( "fascinated"  v ( f a1 s i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fascinates"  v ( f a1 s i n ei t s ) ((pos "Ha%" ) (subcat )))
( "fascinating"  v ( f a1 s i n ei t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "fascinatingly"  a ( f a1 s i n ei t i ng l ii ) ((pos "Pu%" ) ))
( "fascination"  n ( f a2 s i n ei1 sh @ n ) ((pos "M6%" ) ))
( "fascinations"  n ( f a2 s i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fascism"  n ( f a1 sh i z @ m ) ((pos "L@%" ) ))
( "fascist"  n ( f a1 sh i s t ) ((pos "K6%" "OA%" ) ))
( "fascists"  n ( f a1 sh i s t s ) ((pos "Kj%" ) ))
( "fashion"  v ( f a1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "fashionable"  j ( f a1 sh @ n @ b l ) ((pos "OA%" ) ))
( "fashionably"  a ( f a1 sh @ n @ b l ii ) ((pos "Pu%" ) ))
( "fashioned"  v ( f a1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fashioning"  v ( f a1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fashions"  v ( f a1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "fast"  v ( f aa1 s t ) ((pos "I0%" "K6%" "OC%" "Pu%" ) (subcat "2A" )))
( "fasted"  v ( f aa1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fasten"  v ( f aa1 s n ) ((pos "J0%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "fastened"  v ( f aa1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "fastener"  n ( f aa1 s n @ r ) ((pos "K6%" ) ))
( "fasteners"  n ( f aa1 s n @ z ) ((pos "Kj%" ) ))
( "fastening"  v ( f aa1 s n i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "fastenings"  n ( f aa1 s n i ng z ) ((pos "Kj%" ) ))
( "fastens"  v ( f aa1 s n z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "faster"  j ( f aa1 s t @ r ) ((pos "Or%" ) ))
( "fastest"  j ( f aa1 s t i s t ) ((pos "Os%" ) ))
( "fastidious"  j ( f @1 s t i1 d i@ s ) ((pos "OA%" ) ))
( "fastidiously"  a ( f @1 s t i1 d i@ s l ii ) ((pos "Pu%" ) ))
( "fastidiousness"  n ( f @1 s t i1 d i@ s n @ s ) ((pos "L@%" ) ))
( "fasting"  v ( f aa1 s t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fastness"  n ( f aa1 s t n @ s ) ((pos "M7$" ) ))
( "fastnesses"  n ( f aa1 s t n @ s i z ) ((pos "Mj$" ) ))
( "fasts"  v ( f aa1 s t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "fat"  v ( f a1 t ) ((pos "H4%" "M6%" "OE%" ) ))
( "fatal"  j ( f ei1 t l ) ((pos "OA%" ) ))
( "fatalism"  n ( f ei1 t @ l i z @ m ) ((pos "L@%" ) ))
( "fatalist"  n ( f ei1 t @ l i s t ) ((pos "K6%" ) ))
( "fatalistic"  j ( f ei2 t @ l i1 s t i k ) ((pos "OA%" ) ))
( "fatalists"  n ( f ei1 t @ l i s t s ) ((pos "Kj%" ) ))
( "fatalities"  n ( f @1 t a1 l i t i z ) ((pos "Mj%" ) ))
( "fatality"  n ( f @1 t a1 l i t ii ) ((pos "M8%" ) ))
( "fate"  v ( f ei1 t ) ((pos "H2$" "M6%" ) (subcat )))
( "fated"  v ( f ei1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "fateful"  j ( f ei1 t f u l ) ((pos "OA%" ) ))
( "fatefully"  a ( f ei1 t f u l ii ) ((pos "Pu%" ) ))
( "fates"  v ( f ei1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "fathead"  n ( f a1 t h e d ) ((pos "K6%" ) ))
( "fatheads"  n ( f a1 t h e d z ) ((pos "Kj%" ) ))
( "father"  v ( f aa1 dh @ r ) ((pos "H0%" "K6*" ) (subcat "6A" )))
( "father-in-law"  n ( f aa1 dh @ r - i n - l oo ) ((pos "Ki%" ) ))
( "fathered"  v ( f aa1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fatherhood"  n ( f aa1 dh @ h u d ) ((pos "L@%" ) ))
( "fathering"  v ( f aa1 dh @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fatherland"  n ( f aa1 dh @ l a n d ) ((pos "K6%" ) ))
( "fatherlands"  n ( f aa1 dh @ l a n d z ) ((pos "Kj$" ) ))
( "fatherless"  j ( f aa1 dh @ l @ s ) ((pos "OA%" ) ))
( "fatherly"  j ( f aa1 dh @ l ii ) ((pos "OA%" ) ))
( "fathers"  v ( f aa1 dh @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "fathers-in-law"  n ( f aa1 dh @ z - i n - l oo ) ((pos "Kj%" ) ))
( "fathom"  v ( f a1 dh @ m ) ((pos "H0%" "K6%" ) (subcat )))
( "fathomed"  v ( f a1 dh @ m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fathoming"  v ( f a1 dh @ m i ng ) ((pos "Hb%" ) (subcat )))
( "fathomless"  j ( f a1 dh @ m l @ s ) ((pos "OA%" ) ))
( "fathoms"  v ( f a1 dh @ m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fatigue"  v ( f @1 t ii1 g ) ((pos "H2%" "M6%" ) (subcat )))
( "fatigue-parties"  n ( f @1 t ii1 g - p aa t i z ) ((pos "Kj$" ) ))
( "fatigue-party"  n ( f @1 t ii1 g - p aa t ii ) ((pos "K8$" ) ))
( "fatigued"  v ( f @1 t ii1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fatigues"  v ( f @1 t ii1 g z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "fatiguing"  v ( f @1 t ii1 g i ng ) ((pos "Hb%" ) (subcat )))
( "fating"  v ( f ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "fatless"  j ( f a1 t l i s ) ((pos "OA$" ) ))
( "fatness"  n ( f a1 t n @ s ) ((pos "L@%" ) ))
( "fats"  v ( f a1 t s ) ((pos "Ha%" "Mj%" ) ))
( "fatted"  v ( f a1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "fatten"  v ( f a1 t n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "fattened"  v ( f a1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "fattening"  v ( f a1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "fattens"  v ( f a1 t n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "fatter"  j ( f a1 t @ r ) ((pos "Or%" ) ))
( "fattest"  j ( f a1 t i s t ) ((pos "Os%" ) ))
( "fattier"  j ( f a1 t i@ r ) ((pos "Or%" ) ))
( "fattiest"  j ( f a1 t i i s t ) ((pos "Os%" ) ))
( "fatting"  v ( f a1 t i ng ) ((pos "Hb$" ) ))
( "fattish"  j ( f a1 t i sh ) ((pos "OA%" ) ))
( "fatty"  j ( f a1 t ii ) ((pos "OD%" ) ))
( "fatuities"  n ( f @1 t y uu1 i t i z ) ((pos "Mj$" ) ))
( "fatuity"  n ( f @1 t y uu1 i t ii ) ((pos "M8$" ) ))
( "fatuous"  j ( f a1 ch u@ s ) ((pos "OA%" ) ))
( "fatuously"  a ( f a1 ch u@ s l ii ) ((pos "Pu%" ) ))
( "fatuousness"  n ( f a1 ch u@ s n @ s ) ((pos "L@%" ) ))
( "faucet"  n ( f oo1 s i t ) ((pos "K6$" ) ))
( "faucets"  n ( f oo1 s i t s ) ((pos "Kj$" ) ))
( "faugh"  uh ( f oo1 ) ((pos "W-$" ) ))
( "fault"  v ( f oo1 l t ) ((pos "H0%" "K6%" ) (subcat )))
( "fault-finder"  n ( f oo1 l t - f ai n d @ r ) ((pos "K6%" ) ))
( "fault-finders"  n ( f oo1 l t - f ai n d @ z ) ((pos "Kj%" ) ))
( "fault-finding"  n ( f oo1 l t - f ai n d i ng ) ((pos "L@%" ) ))
( "faulted"  v ( f oo1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "faultier"  j ( f oo1 l t i@ r ) ((pos "Or$" ) ))
( "faultiest"  j ( f oo1 l t i i s t ) ((pos "Os$" ) ))
( "faultily"  a ( f oo1 l t i l ii ) ((pos "Pu%" ) ))
( "faulting"  v ( f oo1 l t i ng ) ((pos "Hb$" ) (subcat )))
( "faultless"  j ( f oo1 l t l @ s ) ((pos "OA%" ) ))
( "faultlessly"  a ( f oo1 l t l i s l ii ) ((pos "Pu%" ) ))
( "faults"  v ( f oo1 l t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "faulty"  j ( f oo1 l t ii ) ((pos "OD%" ) ))
( "faun"  n ( f oo1 n ) ((pos "K6$" ) ))
( "fauna"  n ( f oo1 n @ ) ((pos "Kj%" ) ))
( "fauns"  n ( f oo1 n z ) ((pos "Kj$" ) ))
( "faux pas"  n ( f ou2 - p aa1 ) ((pos "K9%" ) ))
( "favour"  v ( f ei1 v @ r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "favourable"  j ( f ei1 v @ r @ b l ) ((pos "OA%" ) ))
( "favourably"  a ( f ei1 v @ r @ b l ii ) ((pos "Pu%" ) ))
( "favoured"  v ( f ei1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "favouring"  v ( f ei1 v @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "favourite"  n ( f ei1 v @ r i t ) ((pos "K6%" "Oq%" ) ))
( "favourites"  n ( f ei1 v @ r i t s ) ((pos "Kj%" ) ))
( "favouritism"  n ( f ei1 v r i t i z @ m ) ((pos "L@%" ) ))
( "favours"  v ( f ei1 v @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "fawn"  v ( f oo1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "fawned"  v ( f oo1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fawning"  v ( f oo1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fawns"  v ( f oo1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "fealties"  n ( f ii1 @ l t i z ) ((pos "Kj$" ) ))
( "fealty"  n ( f ii1 @ l t ii ) ((pos "K8$" ) ))
( "fear"  v ( f i@1 r ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "4A" "6A" "6C" )))
( "feared"  v ( f i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" "6A" "6C" )))
( "fearful"  j ( f i@1 f @ l ) ((pos "OA%" ) ))
( "fearfully"  a ( f i@1 f @ l ii ) ((pos "Pu%" ) ))
( "fearfulness"  n ( f i@1 f @ l n @ s ) ((pos "L@%" ) ))
( "fearing"  v ( f i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" "6A" "6C" )))
( "fearless"  j ( f i@1 l @ s ) ((pos "OA%" ) ))
( "fearlessly"  a ( f i@1 l @ s l ii ) ((pos "Pu%" ) ))
( "fearlessness"  n ( f i@1 l @ s n @ s ) ((pos "L@%" ) ))
( "fears"  v ( f i@1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "4A" "6A" "6C" )))
( "fearsome"  j ( f i@1 s @ m ) ((pos "OA%" ) ))
( "fearsomely"  a ( f i@1 s @ m l ii ) ((pos "Pu%" ) ))
( "feasibility"  n ( f ii2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "feasible"  j ( f ii1 z @ b l ) ((pos "OA%" ) ))
( "feast"  v ( f ii1 s t ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "6A" )))
( "feast-day"  n ( f ii1 s t - d ei ) ((pos "K6%" ) ))
( "feast-days"  n ( f ii1 s t - d ei z ) ((pos "Kj%" ) ))
( "feasted"  v ( f ii1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "6A" )))
( "feasting"  v ( f ii1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "6A" )))
( "feasts"  v ( f ii1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "6A" )))
( "feat"  n ( f ii1 t ) ((pos "K6%" ) ))
( "feather"  v ( f e1 dh @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "feather-boa"  n ( f e2 dh @ - b ou1 @ ) ((pos "K6$" ) ))
( "feather-boas"  n ( f e2 dh @ - b ou1 @ z ) ((pos "Kj$" ) ))
( "featherbed"  v ( f e1 dh @ b e d ) ((pos "H4$" "K6%" ) ))
( "featherbedded"  v ( f e1 dh @ b e d i d ) ((pos "Hc$" "Hd%" ) ))
( "featherbedding"  v ( f e1 dh @ b e d i ng ) ((pos "Hb$" ) ))
( "featherbeds"  v ( f e1 dh @ b e d z ) ((pos "Ha$" "Kj%" ) ))
( "featherbrained"  j ( f e1 dh @ b r ei n d ) ((pos "OA%" ) ))
( "feathered"  v ( f e1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "feathering"  v ( f e1 dh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "feathers"  v ( f e1 dh @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "featherweight"  n ( f e1 dh @ w ei t ) ((pos "K6%" ) ))
( "featherweights"  n ( f e1 dh @ w ei t s ) ((pos "Kj%" ) ))
( "feathery"  j ( f e1 dh @ r ii ) ((pos "OA%" ) ))
( "feats"  n ( f ii1 t s ) ((pos "Kj%" ) ))
( "feature"  v ( f ii1 ch @ r ) ((pos "H2%" "K6%" ) (subcat )))
( "featured"  v ( f ii1 ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "featureless"  j ( f ii1 ch @ l @ s ) ((pos "OA%" ) ))
( "features"  v ( f ii1 ch @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "featuring"  v ( f ii1 ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "febrile"  j ( f ii1 b r ai l ) ((pos "OA$" ) ))
( "feckless"  j ( f e1 k l i s ) ((pos "OA%" ) ))
( "fecklessly"  a ( f e1 k l i s l ii ) ((pos "Pu%" ) ))
( "fecklessness"  n ( f e1 k l i s n @ s ) ((pos "L@%" ) ))
( "fecund"  j ( f e1 k @ n d ) ((pos "OA%" ) ))
( "fecundity"  n ( f i1 k uh1 n d i t ii ) ((pos "L@%" ) ))
( "fed"  v ( f e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "federal"  j ( f e1 d @ r @ l ) ((pos "OA%" ) ))
( "federalism"  n ( f e1 d @ r @ l i z @ m ) ((pos "L@%" ) ))
( "federalist"  n ( f e1 d @ r @ l i s t ) ((pos "K6%" ) ))
( "federalists"  n ( f e1 d @ r @ l i s t s ) ((pos "Kj%" ) ))
( "federate"  v ( f e1 d @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "federated"  v ( f e1 d @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "federates"  v ( f e1 d @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "federating"  v ( f e1 d @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "federation"  n ( f e2 d @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "federations"  n ( f e2 d @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fee"  v ( f ii1 ) ((pos "H5$" "M6%" ) (subcat )))
( "feeble"  j ( f ii1 b l ) ((pos "OB%" ) ))
( "feeble-minded"  j ( f ii2 b l - m ai1 n d i d ) ((pos "OA%" ) ))
( "feebleness"  n ( f ii1 b l n @ s ) ((pos "L@%" ) ))
( "feebler"  j ( f ii1 b l @ r ) ((pos "Or%" ) ))
( "feeblest"  j ( f ii1 b l i s t ) ((pos "Os%" ) ))
( "feebly"  a ( f ii1 b l ii ) ((pos "Pu%" ) ))
( "feed"  v ( f ii1 d ) ((pos "Hc$" "Hd$" "J5%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "feedback"  n ( f ii1 d b a k ) ((pos "L@%" ) ))
( "feeder"  n ( f ii1 d @ r ) ((pos "K6%" ) ))
( "feeders"  n ( f ii1 d @ z ) ((pos "Kj%" ) ))
( "feeding"  v ( f ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "feeding-bottle"  n ( f ii1 d i ng - b o t l ) ((pos "K6%" ) ))
( "feeding-bottles"  n ( f ii1 d i ng - b o t l z ) ((pos "Kj%" ) ))
( "feeds"  v ( f ii1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "feeing"  v ( f ii1 i ng ) ((pos "Hb$" ) (subcat )))
( "feel"  v ( f ii1 l ) ((pos "J5*" "Ki%" ) (subcat "2A" "2C" "2D" "3A" "6A" "6C" "9" "10" "15B" "18A" "19A" )))
( "feeler"  n ( f ii1 l @ r ) ((pos "K6%" ) ))
( "feelers"  n ( f ii1 l @ z ) ((pos "Kj%" ) ))
( "feeling"  v ( f ii1 l i ng ) ((pos "Jb%" "M6%" "OA%" ) (subcat "2A" "2C" "2D" "3A" "6A" "6C" "9" "10" "15B" "18A" "19A" )))
( "feelingly"  a ( f ii1 l i ng l ii ) ((pos "Pu%" ) ))
( "feelings"  n ( f ii1 l i ng z ) ((pos "Mj%" ) ))
( "feels"  v ( f ii1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "2D" "3A" "6A" "6C" "9" "10" "15B" "18A" "19A" )))
( "fees"  v ( f ii1 z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "feet"  n ( f ii1 t ) ((pos "Kj*" ) ))
( "feign"  v ( f ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "feigned"  v ( f ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "feigning"  v ( f ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "feigns"  v ( f ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "feint"  v ( f ei1 n t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "feinted"  v ( f ei1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "feinting"  v ( f ei1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "feints"  v ( f ei1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "feldspar"  n ( f e1 l d s p aa r ) ((pos "L@$" ) ))
( "felicitate"  v ( f @1 l i1 s i t ei t ) ((pos "H2$" ) (subcat "6A" )))
( "felicitated"  v ( f @1 l i1 s i t ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "felicitates"  v ( f @1 l i1 s i t ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "felicitating"  v ( f @1 l i1 s i t ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "felicitation"  n ( f @1 l i2 s i t ei1 sh @ n ) ((pos "K6$" ) ))
( "felicitations"  n ( f @1 l i2 s i t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "felicities"  n ( f @1 l i1 s i t i z ) ((pos "Mj%" ) ))
( "felicitous"  j ( f @1 l i1 s i t @ s ) ((pos "OA%" ) ))
( "felicitously"  a ( f @1 l i1 s i t @ s l ii ) ((pos "Pu%" ) ))
( "felicity"  n ( f @1 l i1 s i t ii ) ((pos "M8%" ) ))
( "feline"  j ( f ii1 l ai n ) ((pos "OA%" ) ))
( "fell"  v ( f e1 l ) ((pos "H0%" "Ic%" "OA$" ) (subcat "2A" "2B" "2C" "2D" "3A" )))
( "fellah"  n ( f e1 l @ ) ((pos "Ki$" ) ))
( "fellaheen"  n ( f e2 l @ h ii1 n ) ((pos "Kj$" ) ))
( "fellahin"  n ( f e2 l @ h ii1 n ) ((pos "Kj$" ) ))
( "felled"  v ( f e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2B" "2C" "2D" "3A" )))
( "felling"  v ( f e1 l i ng ) ((pos "Hb%" ) (subcat "2A" "2B" "2C" "2D" "3A" )))
( "fellow"  n ( f e1 l ou ) ((pos "K6%" ) ))
( "fellow-feeling"  n ( f e2 l ou - f ii1 l i ng ) ((pos "L@%" ) ))
( "fellow-traveller"  n ( f e2 l ou - t r a1 v l @ r ) ((pos "K6%" ) ))
( "fellow-travellers"  n ( f e2 l ou - t r a1 v l @ z ) ((pos "Kj%" ) ))
( "fellows"  n ( f e1 l ou z ) ((pos "Kj%" ) ))
( "fellowship"  n ( f e1 l ou sh i p ) ((pos "M6%" ) ))
( "fellowships"  n ( f e1 l ou sh i p s ) ((pos "Mj%" ) ))
( "fells"  v ( f e1 l z ) ((pos "Ha%" ) (subcat "2A" "2B" "2C" "2D" "3A" )))
( "felon"  n ( f e1 l @ n ) ((pos "K6%" ) ))
( "felonies"  n ( f e1 l @ n i z ) ((pos "Mj%" ) ))
( "felonious"  j ( f i1 l ou1 n i@ s ) ((pos "OA%" ) ))
( "felons"  n ( f e1 l @ n z ) ((pos "Kj%" ) ))
( "felony"  n ( f e1 l @ n ii ) ((pos "M8%" ) ))
( "felspar"  n ( f e1 l s p aa r ) ((pos "L@$" ) ))
( "felt"  v ( f e1 l t ) ((pos "Jc*" "Jd*" "L@%" ) (subcat "2A" "2C" "2D" "3A" "6A" "6C" "9" "10" "15B" "18A" "19A" )))
( "felucca"  n ( f e1 l uh1 k @ ) ((pos "K6$" ) ))
( "feluccas"  n ( f e1 l uh1 k @ z ) ((pos "Kj$" ) ))
( "fem"  n ( f e1 m ) ((pos "Y~%" ) ))
( "female"  n ( f ii1 m ei l ) ((pos "K6%" "OA%" ) ))
( "females"  n ( f ii1 m ei l z ) ((pos "Kj%" ) ))
( "feminine"  j ( f e1 m @ n i n ) ((pos "OA%" ) ))
( "femininity"  n ( f e2 m @ n i1 n i t ii ) ((pos "L@%" ) ))
( "feminism"  n ( f e1 m i n i z @ m ) ((pos "L@%" ) ))
( "feminist"  n ( f e1 m i n i s t ) ((pos "K6%" ) ))
( "feminists"  n ( f e1 m i n i s t s ) ((pos "Kj%" ) ))
( "femur"  n ( f ii1 m @ r ) ((pos "K6$" ) ))
( "femurs"  n ( f ii1 m @ z ) ((pos "Kj$" ) ))
( "fen"  n ( f e1 n ) ((pos "K6%" ) ))
( "fence"  v ( f e1 n s ) ((pos "H2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fenced"  v ( f e1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fencer"  n ( f e1 n s @ r ) ((pos "K6%" ) ))
( "fencers"  n ( f e1 n s @ z ) ((pos "Kj%" ) ))
( "fences"  v ( f e1 n s i z ) ((pos "Ha%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fencing"  v ( f e1 n s i ng ) ((pos "Hb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fend"  v ( f e1 n d ) ((pos "J0%" ) (subcat )))
( "fended"  v ( f e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "fender"  n ( f e1 n d @ r ) ((pos "K6%" ) ))
( "fenders"  n ( f e1 n d @ z ) ((pos "Kj%" ) ))
( "fending"  v ( f e1 n d i ng ) ((pos "Jb%" ) (subcat )))
( "fends"  v ( f e1 n d z ) ((pos "Ja%" ) (subcat )))
( "fennel"  n ( f e1 n l ) ((pos "L@%" ) ))
( "fens"  n ( f e1 n z ) ((pos "Kj%" ) ))
( "feoff"  n ( f ii1 f ) ((pos "K6$" ) ))
( "feoffs"  n ( f ii1 f s ) ((pos "Kj$" ) ))
( "feral"  j ( f i@1 r @ l ) ((pos "OA$" ) ))
( "ferment"  n ( f @@1 m e n t ) ((pos "K6%" ) ))
( "ferment"  v ( f @1 m e1 n t ) ((pos "J0%" ) (subcat "2A" )))
( "fermentation"  n ( f @@2 m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "fermented"  v ( f @1 m e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fermenting"  v ( f @1 m e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ferments"  n ( f @@1 m e n t s ) ((pos "Kj%" ) ))
( "ferments"  v ( f @1 m e1 n t s ) ((pos "Ja%" ) (subcat "2A" )))
( "fern"  n ( f @@1 n ) ((pos "K6%" ) ))
( "fernier"  j ( f @@1 n i@ r ) ((pos "Or$" ) ))
( "ferniest"  j ( f @@1 n i i s t ) ((pos "Os$" ) ))
( "ferns"  n ( f @@1 n z ) ((pos "Kj%" ) ))
( "ferny"  j ( f @@1 n ii ) ((pos "OD%" ) ))
( "ferocious"  j ( f @1 r ou1 sh @ s ) ((pos "OA%" ) ))
( "ferociously"  a ( f @1 r ou1 sh @ s l ii ) ((pos "Pu%" ) ))
( "ferocities"  n ( f @1 r o1 s i t i z ) ((pos "Mj%" ) ))
( "ferocity"  n ( f @1 r o1 s i t ii ) ((pos "M8%" ) ))
( "ferret"  v ( f e1 r i t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "ferreted"  v ( f e1 r i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "ferreting"  v ( f e1 r i t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "ferrets"  v ( f e1 r i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "ferried"  v ( f e1 r i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ferries"  v ( f e1 r i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ferroconcrete"  n ( f e2 r ou k o1 ng k r ii t ) ((pos "L@$" ) ))
( "ferrous"  j ( f e1 r @ s ) ((pos "OA%" ) ))
( "ferrule"  n ( f e1 r uu l ) ((pos "K6$" ) ))
( "ferrules"  n ( f e1 r uu l z ) ((pos "Kj$" ) ))
( "ferry"  v ( f e1 r ii ) ((pos "J3%" "K8%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ferryboat"  n ( f e1 r i b ou t ) ((pos "K6%" ) ))
( "ferryboats"  n ( f e1 r i b ou t s ) ((pos "Kj%" ) ))
( "ferrying"  v ( f e1 r i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ferryman"  n ( f e1 r i m a n ) ((pos "Ki%" ) ))
( "ferrymen"  n ( f e1 r i m e n ) ((pos "Kj%" ) ))
( "fertile"  j ( f @@1 t ai l ) ((pos "OA%" ) ))
( "fertility"  n ( f @1 t i1 l i t ii ) ((pos "L@%" ) ))
( "fertilization"  n ( f @@2 t @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "fertilize"  v ( f @@1 t @ l ai z ) ((pos "H2%" ) (subcat )))
( "fertilized"  v ( f @@1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fertilizer"  n ( f @@1 t @ l ai z @ r ) ((pos "M6%" ) ))
( "fertilizers"  n ( f @@1 t @ l ai z @ z ) ((pos "Mj%" ) ))
( "fertilizes"  v ( f @@1 t @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "fertilizing"  v ( f @@1 t @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "ferule"  n ( f e1 r uu l ) ((pos "K6$" ) ))
( "ferules"  n ( f e1 r uu l z ) ((pos "Kj$" ) ))
( "fervency"  n ( f @@1 v @ n s ii ) ((pos "L@$" ) ))
( "fervent"  j ( f @@1 v @ n t ) ((pos "OA%" ) ))
( "fervently"  a ( f @@1 v @ n t l ii ) ((pos "Pu%" ) ))
( "fervid"  j ( f @@1 v i d ) ((pos "OA$" ) ))
( "fervidly"  a ( f @@1 v i d l ii ) ((pos "Pu$" ) ))
( "fervour"  n ( f @@1 v @ r ) ((pos "L@%" ) ))
( "festal"  j ( f e1 s t l ) ((pos "OA$" ) ))
( "fester"  v ( f e1 s t @ r ) ((pos "I0%" ) (subcat )))
( "festered"  v ( f e1 s t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "festering"  v ( f e1 s t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "festers"  v ( f e1 s t @ z ) ((pos "Ia%" ) (subcat )))
( "festival"  n ( f e1 s t i v @ l ) ((pos "K6%" ) ))
( "festivals"  n ( f e1 s t i v @ l z ) ((pos "Kj%" ) ))
( "festive"  j ( f e1 s t i v ) ((pos "OA%" ) ))
( "festivities"  n ( f e1 s t i1 v i t i z ) ((pos "Mj%" ) ))
( "festivity"  n ( f e1 s t i1 v i t ii ) ((pos "M8%" ) ))
( "festoon"  v ( f e1 s t uu1 n ) ((pos "H0%" "K6%" ) (subcat )))
( "festooned"  v ( f e1 s t uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "festooning"  v ( f e1 s t uu1 n i ng ) ((pos "Hb%" ) (subcat )))
( "festoons"  v ( f e1 s t uu1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fetal"  j ( f ii1 t l ) ((pos "OA$" ) ))
( "fetch"  v ( f e1 ch ) ((pos "J1%" ) (subcat "6A" "12B" "12C" "13B" "15A" )))
( "fetched"  v ( f e1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "12B" "12C" "13B" "15A" )))
( "fetches"  v ( f e1 ch i z ) ((pos "Ja%" ) (subcat "6A" "12B" "12C" "13B" "15A" )))
( "fetching"  v ( f e1 ch i ng ) ((pos "Jb%" "OA%" ) (subcat "6A" "12B" "12C" "13B" "15A" )))
( "fete-day"  n ( f ei1 t - d ei ) ((pos "K6%" ) ))
( "fete-days"  n ( f ei1 t - d ei z ) ((pos "Kj%" ) ))
( "fetid"  j ( f e1 t i d ) ((pos "OA%" ) ))
( "fetish"  n ( f e1 t i sh ) ((pos "K7%" ) ))
( "fetishes"  n ( f e1 t i sh i z ) ((pos "Kj%" ) ))
( "fetlock"  n ( f e1 t l o k ) ((pos "K6$" ) ))
( "fetlocks"  n ( f e1 t l o k s ) ((pos "Kj$" ) ))
( "fetter"  v ( f e1 t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "fettered"  v ( f e1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fettering"  v ( f e1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "fetters"  v ( f e1 t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fettle"  n ( f e1 t l ) ((pos "L@%" ) ))
( "feud"  n ( f y uu1 d ) ((pos "K6%" ) ))
( "feudal"  j ( f y uu1 d l ) ((pos "OA%" ) ))
( "feudalism"  n ( f y uu1 d l i z @ m ) ((pos "L@%" ) ))
( "feudatories"  n ( f y uu1 d @ t @ r i z ) ((pos "Kj$" ) ))
( "feudatory"  n ( f y uu1 d @ t @ r ii ) ((pos "K8$" "OA$" ) ))
( "feuds"  n ( f y uu1 d z ) ((pos "Kj%" ) ))
( "fever"  n ( f ii1 v @ r ) ((pos "M6%" ) ))
( "fevered"  j ( f ii1 v @ d ) ((pos "OA%" ) ))
( "feverish"  j ( f ii1 v @ r i sh ) ((pos "OA%" ) ))
( "feverishly"  a ( f ii1 v @ r i sh l ii ) ((pos "Pu%" ) ))
( "fevers"  n ( f ii1 v @ z ) ((pos "Mj%" ) ))
( "few"  j ( f y uu ) ((pos "OC*" "Qx*" ) ))
( "fewer"  j ( f y uu1 @ r ) ((pos "Or%" ) ))
( "fewest"  j ( f y uu1 i s t ) ((pos "Os%" ) ))
( "fewness"  n ( f y uu1 n @ s ) ((pos "L@$" ) ))
( "fey"  j ( f ei1 ) ((pos "OA$" ) ))
( "fez"  n ( f e1 z ) ((pos "K7$" ) ))
( "fezes"  n ( f e1 z i z ) ((pos "Kj$" ) ))
( "fianc_e"  n ( f i1 o1 n s ei ) ((pos "K6%" ) ))
( "fianc_ee"  n ( f i1 o1 n s ei ) ((pos "K6%" ) ))
( "fianc_ees"  n ( f i1 o1 n s ei z ) ((pos "Kj%" ) ))
( "fianc_es"  n ( f i1 o1 n s ei z ) ((pos "Kj%" ) ))
( "fiasco"  n ( f i1 a1 s k ou ) ((pos "K6%" ) ))
( "fiascos"  n ( f i1 a1 s k ou z ) ((pos "Kj%" ) ))
( "fiat"  n ( f ai1 a t ) ((pos "K6$" ) ))
( "fiats"  n ( f ai1 a t s ) ((pos "Kj$" ) ))
( "fib"  v ( f i1 b ) ((pos "I4%" "K6%" ) (subcat )))
( "fibbed"  v ( f i1 b d ) ((pos "Ic%" "Id%" ) (subcat )))
( "fibber"  n ( f i1 b @ r ) ((pos "K6%" ) ))
( "fibbers"  n ( f i1 b @ z ) ((pos "Kj%" ) ))
( "fibbing"  v ( f i1 b i ng ) ((pos "Ib%" "L@%" ) (subcat )))
( "fibre"  n ( f ai1 b @ r ) ((pos "M6%" ) ))
( "fibreboard"  n ( f ai1 b @ b oo d ) ((pos "L@%" ) ))
( "fibreglass"  n ( f ai1 b @ g l aa s ) ((pos "L@%" ) ))
( "fibres"  n ( f ai1 b @ z ) ((pos "Mj%" ) ))
( "fibrositis"  n ( f ai2 b r @ s ai1 t i s ) ((pos "L@%" ) ))
( "fibrous"  j ( f ai1 b r @ s ) ((pos "OA%" ) ))
( "fibs"  v ( f i1 b z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "fibula"  n ( f i1 b y u l @ ) ((pos "K6$" ) ))
( "fibulas"  n ( f i1 b y u l @ z ) ((pos "Kj$" ) ))
( "fickle"  j ( f i1 k l ) ((pos "OA%" ) ))
( "fickleness"  n ( f i1 k l n @ s ) ((pos "L@%" ) ))
( "fiction"  n ( f i1 k sh @ n ) ((pos "M6%" ) ))
( "fictional"  j ( f i1 k sh @ n l ) ((pos "OA%" ) ))
( "fictions"  n ( f i1 k sh @ n z ) ((pos "Mj%" ) ))
( "fictitious"  j ( f i1 k t i1 sh @ s ) ((pos "OA%" ) ))
( "fiddle"  v ( f i1 d l ) ((pos "H2%" "K6%" ) (subcat "2A" "2C" )))
( "fiddled"  v ( f i1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" )))
( "fiddler"  n ( f i1 d l @ r ) ((pos "K6%" ) ))
( "fiddlers"  n ( f i1 d l @ z ) ((pos "Kj%" ) ))
( "fiddles"  v ( f i1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "2A" "2C" )))
( "fiddlestick"  n ( f i1 d l s t i k ) ((pos "K6$" ) ))
( "fiddlesticks"  n ( f i1 d l s t i k s ) ((pos "Kj$" "W-%" ) ))
( "fiddling"  v ( f i1 d l i ng ) ((pos "Hb%" "OA%" ) (subcat "2A" "2C" )))
( "fidelity"  n ( f i1 d e1 l i t ii ) ((pos "L@%" ) ))
( "fidget"  v ( f i1 jh i t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "fidgeted"  v ( f i1 jh i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fidgeting"  v ( f i1 jh i t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "fidgets"  v ( f i1 jh i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "fidgety"  j ( f i1 jh i t ii ) ((pos "OA%" ) ))
( "fie"  uh ( f ai1 ) ((pos "W-$" ) ))
( "fief"  n ( f ii1 f ) ((pos "K6$" ) ))
( "fiefs"  n ( f ii1 f s ) ((pos "Kj$" ) ))
( "field"  v ( f ii1 l d ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "field-hospital"  n ( f ii1 l d - h o1 s p i t l ) ((pos "K6$" ) ))
( "field-hospitals"  n ( f ii1 l d - h o1 s p i t l z ) ((pos "Kj$" ) ))
( "field-officer"  n ( f ii1 l d - o1 f i s @ r ) ((pos "K6$" ) ))
( "field-officers"  n ( f ii1 l d - o1 f i s @ z ) ((pos "Kj$" ) ))
( "fielded"  v ( f ii1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fielder"  n ( f ii1 l d @ r ) ((pos "K6%" ) ))
( "fielders"  n ( f ii1 l d @ z ) ((pos "Kj%" ) ))
( "fielding"  v ( f ii1 l d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "fields"  v ( f ii1 l d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "fieldsman"  n ( f ii1 l d z m @ n ) ((pos "Ki%" ) ))
( "fieldsmen"  n ( f ii1 l d z m @ n ) ((pos "Kj%" ) ))
( "fieldwork"  n ( f ii1 l d w @@ k ) ((pos "M6%" ) ))
( "fieldworks"  n ( f ii1 l d w @@ k s ) ((pos "Mj$" ) ))
( "fiend"  n ( f ii1 n d ) ((pos "K6%" ) ))
( "fiendish"  j ( f ii1 n d i sh ) ((pos "OA%" ) ))
( "fiendishly"  a ( f ii1 n d i sh l ii ) ((pos "Pu%" ) ))
( "fiends"  n ( f ii1 n d z ) ((pos "Kj%" ) ))
( "fierce"  j ( f i@1 s ) ((pos "OB%" ) ))
( "fiercely"  a ( f i@1 s l ii ) ((pos "Pu%" ) ))
( "fierceness"  n ( f i@1 s n @ s ) ((pos "L@%" ) ))
( "fiercer"  j ( f i@1 s @ r ) ((pos "Or%" ) ))
( "fiercest"  j ( f i@1 s i s t ) ((pos "Os%" ) ))
( "fierily"  a ( f ai1 @ r i l ii ) ((pos "Pu%" ) ))
( "fieriness"  n ( f ai1 @ r i n @ s ) ((pos "L@%" ) ))
( "fiery"  j ( f ai1 @ r ii ) ((pos "OA%" ) ))
( "fiesta"  n ( f i1 e1 s t @ ) ((pos "K6$" ) ))
( "fiestas"  n ( f i1 e1 s t @ z ) ((pos "Kj$" ) ))
( "fife"  n ( f ai1 f ) ((pos "K6$" ) ))
( "fifes"  n ( f ai1 f s ) ((pos "Kj$" ) ))
( "fifteen"  n ( f i2 f t ii1 n ) ((pos "K6%" "OA%" ) ))
( "fifteens"  n ( f i2 f t ii1 n z ) ((pos "Kj%" ) ))
( "fifteenth"  n ( f i2 f t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "fifteenths"  n ( f i2 f t ii1 n th s ) ((pos "Kj%" ) ))
( "fifth"  n ( f i1 f th ) ((pos "K6%" "OA%" ) ))
( "fifthly"  a ( f i1 f th l ii ) ((pos "Pu%" ) ))
( "fifths"  n ( f i1 f th s ) ((pos "Kj%" ) ))
( "fifties"  n ( f i1 f t i z ) ((pos "Kj%" ) ))
( "fiftieth"  n ( f i1 f t i@ th ) ((pos "K6%" "OA%" ) ))
( "fiftieths"  n ( f i1 f t i@ th s ) ((pos "Kj%" ) ))
( "fifty"  n ( f i1 f t ii ) ((pos "K8%" "OA%" ) ))
( "fig"  n ( f i1 g ) ((pos "K6%" "Y>%" ) ))
( "fig-leaf"  n ( f i1 g - l ii f ) ((pos "Ki%" ) ))
( "fig-leaves"  n ( f i1 g - l ii v z ) ((pos "Kj%" ) ))
( "fight"  v ( f ai1 t ) ((pos "J5%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "15A" )))
( "fighter"  n ( f ai1 t @ r ) ((pos "K6%" ) ))
( "fighters"  n ( f ai1 t @ z ) ((pos "Kj%" ) ))
( "fighting"  v ( f ai1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "15A" )))
( "fights"  v ( f ai1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "15A" )))
( "figment"  n ( f i1 g m @ n t ) ((pos "K6%" ) ))
( "figments"  n ( f i1 g m @ n t s ) ((pos "Kj%" ) ))
( "figs"  n ( f i1 g z ) ((pos "Kj%" ) ))
( "figurative"  j ( f i1 g y @ r @ t i v ) ((pos "OA%" ) ))
( "figuratively"  a ( f i1 g y @ r @ t i v l ii ) ((pos "Pu%" ) ))
( "figure"  v ( f i1 g @ r ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "9" "15A" "15B" )))
( "figured"  v ( f i1 g @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "3A" "9" "15A" "15B" )))
( "figurehead"  n ( f i1 g @ h e d ) ((pos "K6%" ) ))
( "figureheads"  n ( f i1 g @ h e d z ) ((pos "Kj%" ) ))
( "figures"  v ( f i1 g @ z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "9" "15A" "15B" )))
( "figuring"  v ( f i1 g @ r i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "9" "15A" "15B" )))
( "filament"  n ( f i1 l @ m @ n t ) ((pos "K6%" ) ))
( "filaments"  n ( f i1 l @ m @ n t s ) ((pos "Kj%" ) ))
( "filature"  n ( f i1 l @ ch @ r ) ((pos "K6$" ) ))
( "filatures"  n ( f i1 l @ ch @ z ) ((pos "Kj$" ) ))
( "filbert"  n ( f i1 l b @ t ) ((pos "K6$" ) ))
( "filberts"  n ( f i1 l b @ t s ) ((pos "Kj$" ) ))
( "filch"  v ( f i1 l ch ) ((pos "H1$" ) (subcat )))
( "filched"  v ( f i1 l ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "filches"  v ( f i1 l ch i z ) ((pos "Ha$" ) (subcat )))
( "filching"  v ( f i1 l ch i ng ) ((pos "Hb$" ) (subcat )))
( "file"  v ( f ai1 l ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" "15A" "15B" )))
( "filed"  v ( f ai1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" "15B" )))
( "files"  v ( f ai1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" "15B" )))
( "filial"  j ( f i1 l i@ l ) ((pos "OA%" ) ))
( "filibuster"  v ( f i1 l i b uh s t @ r ) ((pos "I0%" "K6%" ) ))
( "filibustered"  v ( f i1 l i b uh s t @ d ) ((pos "Ic%" "Id%" ) ))
( "filibustering"  v ( f i1 l i b uh s t @ r i ng ) ((pos "Ib%" ) ))
( "filibusters"  v ( f i1 l i b uh s t @ z ) ((pos "Ia%" "Kj%" ) ))
( "filigree"  n ( f i1 l i g r ii ) ((pos "L@$" ) ))
( "filing"  v ( f ai1 l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" "15B" )))
( "filings"  n ( f ai1 l i ng z ) ((pos "Kj%" ) ))
( "fill"  v ( f i1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "12B" "13B" "14" )))
( "filled"  v ( f i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12B" "13B" "14" )))
( "fillet"  v ( f i1 l i t ) ((pos "H0%" "K6%" ) (subcat )))
( "filleted"  v ( f i1 l i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "filleting"  v ( f i1 l i t i ng ) ((pos "Hb%" ) (subcat )))
( "fillets"  v ( f i1 l i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fillies"  n ( f i1 l i z ) ((pos "Kj$" ) ))
( "filling"  v ( f i1 l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" "12B" "13B" "14" )))
( "fillings"  n ( f i1 l i ng z ) ((pos "Mj%" ) ))
( "fillip"  n ( f i1 l i p ) ((pos "K6$" ) ))
( "fillips"  n ( f i1 l i p s ) ((pos "Kj$" ) ))
( "fills"  v ( f i1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "12B" "13B" "14" )))
( "filly"  n ( f i1 l ii ) ((pos "K8$" ) ))
( "film"  v ( f i1 l m ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "film-star"  n ( f i1 l m - s t aa r ) ((pos "K6%" ) ))
( "film-stars"  n ( f i1 l m - s t aa z ) ((pos "Kj%" ) ))
( "filmable"  j ( f i1 l m @ b l ) ((pos "OA%" ) ))
( "filmed"  v ( f i1 l m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "filmier"  j ( f i1 l m i@ r ) ((pos "Or$" ) ))
( "filmiest"  j ( f i1 l m i i s t ) ((pos "Os$" ) ))
( "filming"  v ( f i1 l m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "films"  v ( f i1 l m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "filmy"  j ( f i1 l m ii ) ((pos "OD%" ) ))
( "filter"  v ( f i1 l t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "filtered"  v ( f i1 l t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "filtering"  v ( f i1 l t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "filters"  v ( f i1 l t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "filtertipped"  j ( f i1 l t @ t i p t ) ((pos "OA%" ) ))
( "filth"  n ( f i1 l th ) ((pos "L@%" ) ))
( "filthier"  j ( f i1 l th i@ r ) ((pos "Or%" ) ))
( "filthiest"  j ( f i1 l th i i s t ) ((pos "Os%" ) ))
( "filthily"  a ( f i1 l th i l ii ) ((pos "Pu%" ) ))
( "filthiness"  n ( f i1 l th i n @ s ) ((pos "L@%" ) ))
( "filthy"  j ( f i1 l th ii ) ((pos "OD%" ) ))
( "filtrate"  n ( f i1 l t r ei t ) ((pos "K6$" ) ))
( "filtrate"  v ( f i1 l t r ei1 t ) ((pos "J2$" ) ))
( "filtrated"  v ( f i1 l t r ei1 t i d ) ((pos "Jc$" "Jd$" ) ))
( "filtrates"  n ( f i1 l t r ei t s ) ((pos "Kj$" ) ))
( "filtrates"  v ( f i1 l t r ei1 t s ) ((pos "Ja$" ) ))
( "filtrating"  v ( f i1 l t r ei1 t i ng ) ((pos "Jb$" ) ))
( "filtration"  n ( f i1 l t r ei1 sh @ n ) ((pos "L@$" ) ))
( "fin"  n ( f i1 n ) ((pos "K6%" ) ))
( "finable"  j ( f ai1 n @ b l ) ((pos "OA$" ) ))
( "final"  n ( f ai1 n l ) ((pos "K6%" "OA%" ) ))
( "finale"  n ( f i1 n aa1 l ii ) ((pos "K6%" ) ))
( "finales"  n ( f i1 n aa1 l i z ) ((pos "Kj%" ) ))
( "finalist"  n ( f ai1 n @ l i s t ) ((pos "K6%" ) ))
( "finalists"  n ( f ai1 n @ l i s t s ) ((pos "Kj%" ) ))
( "finality"  n ( f ai1 n a1 l i t ii ) ((pos "L@%" ) ))
( "finalize"  v ( f ai1 n @ l ai z ) ((pos "H2%" ) (subcat )))
( "finalized"  v ( f ai1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "finalizes"  v ( f ai1 n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "finalizing"  v ( f ai1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "finally"  a ( f ai1 n @ l ii ) ((pos "Pu%" ) ))
( "finals"  n ( f ai1 n l z ) ((pos "Kj%" ) ))
( "finance"  v ( f ai1 n a n s ) ((pos "H2%" "M6%" ) (subcat )))
( "financed"  v ( f ai1 n a n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "finances"  v ( f ai1 n a n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "financial"  j ( f ai1 n a1 n sh l ) ((pos "OA%" ) ))
( "financially"  a ( f ai1 n a1 n sh @ l ii ) ((pos "Pu%" ) ))
( "financier"  n ( f ai1 n a1 n s i@ r ) ((pos "K6%" ) ))
( "financiers"  n ( f ai1 n a1 n s i@ z ) ((pos "Kj%" ) ))
( "financing"  v ( f ai1 n a n s i ng ) ((pos "Hb%" ) (subcat )))
( "finch"  n ( f i1 n ch ) ((pos "K7%" ) ))
( "finches"  n ( f i1 n ch i z ) ((pos "Kj%" ) ))
( "find"  v ( f ai1 n d ) ((pos "H5*" "K6%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "15A" "15B" "19B" "22" )))
( "finder"  n ( f ai1 n d @ r ) ((pos "K6%" ) ))
( "finders"  n ( f ai1 n d @ z ) ((pos "Kj%" ) ))
( "finding"  v ( f ai1 n d i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "15A" "15B" "19B" "22" )))
( "findings"  n ( f ai1 n d i ng z ) ((pos "Kj%" ) ))
( "finds"  v ( f ai1 n d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "15A" "15B" "19B" "22" )))
( "fine"  v ( f ai1 n ) ((pos "H2%" "K6%" "OB%" "Pu%" ) (subcat "6A" )))
( "fine-tooth"  j ( f ai1 n - t uu th ) ((pos "Oq%" ) ))
( "fineable"  j ( f ai1 n @ b l ) ((pos "OA$" ) ))
( "fined"  v ( f ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "finely"  a ( f ai1 n l ii ) ((pos "Pu%" ) ))
( "fineness"  n ( f ai1 n - n @ s ) ((pos "L@%" ) ))
( "finer"  j ( f ai1 n @ r ) ((pos "Or%" ) ))
( "finery"  n ( f ai1 n @ r ii ) ((pos "L@%" ) ))
( "fines"  v ( f ai1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "finesse"  n ( f i1 n e1 s ) ((pos "M6%" ) ))
( "finesses"  n ( f i1 n e1 s i z ) ((pos "Mj$" ) ))
( "finest"  j ( f ai1 n i s t ) ((pos "Os%" ) ))
( "finger"  v ( f i1 ng g @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "finger-alphabet"  n ( f i2 ng g @ r - a1 l f @ b e t ) ((pos "K6$" ) ))
( "finger-alphabets"  n ( f i2 ng g @ r - a1 l f @ b e t s ) ((pos "Kj$" ) ))
( "finger-bowl"  n ( f i1 ng g @ - b ou l ) ((pos "K6%" ) ))
( "finger-bowls"  n ( f i1 ng g @ - b ou l z ) ((pos "Kj%" ) ))
( "finger-plate"  n ( f i1 ng g @ - p l ei t ) ((pos "K6%" ) ))
( "finger-plates"  n ( f i1 ng g @ - p l ei t s ) ((pos "Kj%" ) ))
( "finger-post"  n ( f i1 ng g @ - p ou s t ) ((pos "K6$" ) ))
( "finger-posts"  n ( f i1 ng g @ - p ou s t s ) ((pos "Kj$" ) ))
( "fingerboard"  n ( f i1 ng g @ b oo d ) ((pos "K6%" ) ))
( "fingerboards"  n ( f i1 ng g @ b oo d z ) ((pos "Kj%" ) ))
( "fingered"  v ( f i1 ng g @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fingering"  v ( f i1 ng g @ r i ng ) ((pos "Hb%" ) (subcat )))
( "fingermark"  n ( f i1 ng g @ m aa k ) ((pos "K6%" ) ))
( "fingermarks"  n ( f i1 ng g @ m aa k s ) ((pos "Kj%" ) ))
( "fingernail"  n ( f i2 ng g @ n ei1 l ) ((pos "K6%" ) ))
( "fingernails"  n ( f i2 ng g @ n ei1 l z ) ((pos "Kj%" ) ))
( "fingerprint"  n ( f i1 ng g @ p r i n t ) ((pos "K6%" ) ))
( "fingerprints"  n ( f i1 ng g @ p r i n t s ) ((pos "Kj%" ) ))
( "fingers"  v ( f i1 ng g @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fingerstall"  n ( f i1 ng g @ s t oo l ) ((pos "K6$" ) ))
( "fingerstalls"  n ( f i1 ng g @ s t oo l z ) ((pos "Kj$" ) ))
( "fingertip"  n ( f i1 ng g @ t i p ) ((pos "K6%" ) ))
( "fingertips"  n ( f i1 ng g @ t i p s ) ((pos "Kj%" ) ))
( "finical"  j ( f i1 n i k l ) ((pos "OA$" ) ))
( "finicky"  j ( f i1 n i k ii ) ((pos "OA%" ) ))
( "fining"  v ( f ai1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "finis"  n ( f i1 n i s ) ((pos "Ki$" ) ))
( "finish"  v ( f i1 n i sh ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "6A" "6C" "15B" )))
( "finished"  v ( f i1 n i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "6C" "15B" )))
( "finishes"  v ( f i1 n i sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "6C" "15B" )))
( "finishing"  v ( f i1 n i sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "6C" "15B" )))
( "finite"  j ( f ai1 n ai t ) ((pos "OA%" ) ))
( "finnan"  n ( f i1 n @ n ) ((pos "L@$" ) ))
( "finnan haddie"  n ( f i2 n @ n - h a1 d ii ) ((pos "L@$" ) ))
( "finnan haddock"  n ( f i2 n @ n - h a1 d @ k ) ((pos "L@%" ) ))
( "fins"  n ( f i1 n z ) ((pos "Kj%" ) ))
( "fiord"  n ( f i1 oo1 d ) ((pos "K6%" ) ))
( "fiords"  n ( f i1 oo1 d z ) ((pos "Kj%" ) ))
( "fir"  n ( f @@1 r ) ((pos "M6%" ) ))
( "fir-cone"  n ( f @@1 - k ou n ) ((pos "K6%" ) ))
( "fir-cones"  n ( f @@1 - k ou n z ) ((pos "Kj%" ) ))
( "fire"  v ( f ai1 @ r ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fire-alarm"  n ( f ai1 @ r - @ l aa m ) ((pos "K6%" ) ))
( "fire-alarms"  n ( f ai1 @ r - @ l aa m z ) ((pos "Kj%" ) ))
( "fire-brigade"  n ( f ai1 @ - b r i g ei d ) ((pos "K6%" ) ))
( "fire-brigades"  n ( f ai1 @ - b r i g ei d z ) ((pos "Kj%" ) ))
( "fire-control"  n ( f ai1 @ - k @ n t r ou l ) ((pos "L@$" ) ))
( "fire-eater"  n ( f ai1 @ r - ii t @ r ) ((pos "K6%" ) ))
( "fire-eaters"  n ( f ai1 @ r - ii t @ z ) ((pos "Kj%" ) ))
( "fire-engine"  n ( f ai1 @ r - e n jh i n ) ((pos "K6%" ) ))
( "fire-engines"  n ( f ai1 @ r - e n jh i n z ) ((pos "Kj%" ) ))
( "fire-escape"  n ( f ai1 @ r - i s k ei p ) ((pos "K6%" ) ))
( "fire-escapes"  n ( f ai1 @ r - i s k ei p s ) ((pos "Kj%" ) ))
( "fire-extinguisher"  n ( f ai1 @ r - i k s t i2 ng w i sh @ r ) ((pos "K6%" ) ))
( "fire-extinguishers"  n ( f ai1 @ r - i k s t i2 ng w i sh @ z ) ((pos "Kj%" ) ))
( "fire-fighter"  n ( f ai1 @ - f ai t @ r ) ((pos "K6%" ) ))
( "fire-fighters"  n ( f ai1 @ - f ai t @ z ) ((pos "Kj%" ) ))
( "fire-hose"  n ( f ai1 @ - h ou z ) ((pos "K6%" ) ))
( "fire-hoses"  n ( f ai1 @ - h ou z i z ) ((pos "Kj%" ) ))
( "fire-irons"  n ( f ai1 @ r - ai @ n z ) ((pos "Kj$" ) ))
( "fire-power"  n ( f ai1 @ - p au @ r ) ((pos "L@%" ) ))
( "fire-raising"  n ( f ai1 @ - r ei z i ng ) ((pos "L@%" ) ))
( "fire-walker"  n ( f ai1 @ - w oo k @ r ) ((pos "K6%" ) ))
( "fire-walkers"  n ( f ai1 @ - w oo k @ z ) ((pos "Kj%" ) ))
( "fire-walking"  n ( f ai1 @ - w oo k i ng ) ((pos "L@%" ) ))
( "fire-watcher"  n ( f ai1 @ - w o ch @ r ) ((pos "K6%" ) ))
( "fire-watchers"  n ( f ai1 @ - w o ch @ z ) ((pos "Kj%" ) ))
( "fire-watching"  n ( f ai1 @ - w o ch i ng ) ((pos "L@%" ) ))
( "firearm"  n ( f ai1 @ r aa m ) ((pos "K6%" ) ))
( "firearms"  n ( f ai1 @ r aa m z ) ((pos "Kj%" ) ))
( "fireball"  n ( f ai1 @ b oo l ) ((pos "K6%" ) ))
( "fireballs"  n ( f ai1 @ b oo l z ) ((pos "Kj%" ) ))
( "firebird"  n ( f ai1 @ b @@ d ) ((pos "K6$" ) ))
( "firebirds"  n ( f ai1 @ b @@ d z ) ((pos "Kj$" ) ))
( "firebomb"  n ( f ai1 @ b o m ) ((pos "K6%" ) ))
( "firebombs"  n ( f ai1 @ b o m z ) ((pos "Kj%" ) ))
( "firebox"  n ( f ai1 @ b o k s ) ((pos "K7$" ) ))
( "fireboxes"  n ( f ai1 @ b o k s i z ) ((pos "Kj$" ) ))
( "firebrand"  n ( f ai1 @ b r a n d ) ((pos "K6%" ) ))
( "firebrands"  n ( f ai1 @ b r a n d z ) ((pos "Kj%" ) ))
( "firebreak"  n ( f ai1 @ b r ei k ) ((pos "K6%" ) ))
( "firebreaks"  n ( f ai1 @ b r ei k s ) ((pos "Kj%" ) ))
( "firebrick"  n ( f ai1 @ b r i k ) ((pos "K6%" ) ))
( "firebricks"  n ( f ai1 @ b r i k s ) ((pos "Kj%" ) ))
( "firebug"  n ( f ai1 @ b uh g ) ((pos "K6$" ) ))
( "firebugs"  n ( f ai1 @ b uh g z ) ((pos "Kj$" ) ))
( "fireclay"  n ( f ai1 @ k l ei ) ((pos "L@%" ) ))
( "firecracker"  n ( f ai1 @ k r a k @ r ) ((pos "K6%" ) ))
( "firecrackers"  n ( f ai1 @ k r a k @ z ) ((pos "Kj%" ) ))
( "fired"  v ( f ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "firedamp"  n ( f ai1 @ d a m p ) ((pos "L@%" ) ))
( "firedog"  n ( f ai1 @ d o g ) ((pos "K6$" ) ))
( "firedogs"  n ( f ai1 @ d o g z ) ((pos "Kj$" ) ))
( "fireflies"  n ( f ai1 @ f l ai z ) ((pos "Kj%" ) ))
( "firefly"  n ( f ai1 @ f l ai ) ((pos "K8%" ) ))
( "fireguard"  n ( f ai1 @ g aa d ) ((pos "K6%" ) ))
( "fireguards"  n ( f ai1 @ g aa d z ) ((pos "Kj%" ) ))
( "firelight"  n ( f ai1 @ l ai t ) ((pos "K6%" ) ))
( "firelighter"  n ( f ai1 @ l ai t @ r ) ((pos "K6%" ) ))
( "firelighters"  n ( f ai1 @ l ai t @ z ) ((pos "Kj%" ) ))
( "firelights"  n ( f ai1 @ l ai t s ) ((pos "Kj$" ) ))
( "fireman"  n ( f ai1 @ m @ n ) ((pos "Ki%" ) ))
( "firemen"  n ( f ai1 @ m @ n ) ((pos "Kj%" ) ))
( "fireplace"  n ( f ai1 @ p l ei s ) ((pos "K6%" ) ))
( "fireplaces"  n ( f ai1 @ p l ei s i z ) ((pos "Kj%" ) ))
( "fireproof"  j ( f ai1 @ p r uu f ) ((pos "OA%" ) ))
( "fires"  v ( f ai1 @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fireside"  n ( f ai1 @ s ai d ) ((pos "K6%" ) ))
( "firesides"  n ( f ai1 @ s ai d z ) ((pos "Kj%" ) ))
( "firestone"  n ( f ai1 @ s t ou n ) ((pos "L@$" ) ))
( "firewater"  n ( f ai1 @ w oo t @ r ) ((pos "L@%" ) ))
( "firewood"  n ( f ai1 @ w u d ) ((pos "L@%" ) ))
( "firework"  n ( f ai1 @ w @@ k ) ((pos "K6%" ) ))
( "fireworks"  n ( f ai1 @ w @@ k s ) ((pos "Kj%" ) ))
( "firing"  v ( f ai1 @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "firing-line"  n ( f ai1 r i ng - l ai n ) ((pos "K6%" ) ))
( "firing-lines"  n ( f ai1 r i ng - l ai n z ) ((pos "Kj$" ) ))
( "firing-parties"  n ( f ai1 r i ng - p aa t i z ) ((pos "Kj$" ) ))
( "firing-party"  n ( f ai1 r i ng - p aa t ii ) ((pos "K8$" ) ))
( "firing-squad"  n ( f ai1 r i ng - s k w o d ) ((pos "K6%" ) ))
( "firing-squads"  n ( f ai1 r i ng - s k w o d z ) ((pos "Kj%" ) ))
( "firkin"  n ( f @@1 k i n ) ((pos "K6$" ) ))
( "firkins"  n ( f @@1 k i n z ) ((pos "Kj$" ) ))
( "firm"  v ( f @@1 m ) ((pos "J0%" "K6%" "OC%" "Pu%" ) ))
( "firmament"  n ( f @@1 m @ m @ n t ) ((pos "K6%" ) ))
( "firmaments"  n ( f @@1 m @ m @ n t s ) ((pos "Kj$" ) ))
( "firmed"  v ( f @@1 m d ) ((pos "Jc%" "Jd%" ) ))
( "firmer"  j ( f @@1 m @ r ) ((pos "Or%" ) ))
( "firmest"  j ( f @@1 m i s t ) ((pos "Os%" ) ))
( "firming"  v ( f @@1 m i ng ) ((pos "Jb%" ) ))
( "firmly"  a ( f @@1 m l ii ) ((pos "Pu%" ) ))
( "firmness"  n ( f @@1 m n @ s ) ((pos "L@%" ) ))
( "firms"  v ( f @@1 m z ) ((pos "Ja%" "Kj%" ) ))
( "firs"  n ( f @@1 z ) ((pos "Mj%" ) ))
( "first"  n ( f @@1 s t ) ((pos "K6%" "OA*" "Pu*" ) ))
( "first-class"  j ( f @@1 s t - k l aa1 s ) ((pos "OA%" "Pu%" ) ))
( "first-fruits"  n ( f @@1 s t - f r uu1 t s ) ((pos "Kj%" ) ))
( "first-hand"  j ( f @@1 s t - h a1 n d ) ((pos "OA%" "Pu%" ) ))
( "first-nighter"  n ( f @@1 s t - n ai1 t @ r ) ((pos "K6%" ) ))
( "first-nighters"  n ( f @@1 s t - n ai1 t @ z ) ((pos "Kj%" ) ))
( "first-rate"  j ( f @@1 s t - r ei1 t ) ((pos "OA%" "Pu%" ) ))
( "firstborn"  n ( f @@1 s t b oo n ) ((pos "K6%" "OA%" ) ))
( "firstborns"  n ( f @@1 s t b oo n z ) ((pos "Kj$" ) ))
( "firstly"  a ( f @@1 s t l ii ) ((pos "Pu%" ) ))
( "firsts"  n ( f @@1 s t s ) ((pos "Kj$" ) ))
( "firth"  n ( f @@1 th ) ((pos "K6$" ) ))
( "firths"  n ( f @@1 th s ) ((pos "Kj$" ) ))
( "fiscal"  j ( f i1 s k l ) ((pos "OA%" ) ))
( "fish"  v ( f i1 sh ) ((pos "J1%" "M9%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fish-hook"  n ( f i1 sh - h u k ) ((pos "K6%" ) ))
( "fish-hooks"  n ( f i1 sh - h u k s ) ((pos "Kj%" ) ))
( "fish-knife"  n ( f i1 sh - n ai f ) ((pos "Ki%" ) ))
( "fish-knives"  n ( f i1 sh - n ai v z ) ((pos "Kj%" ) ))
( "fish-slice"  n ( f i1 sh - s l ai s ) ((pos "K6%" ) ))
( "fish-slices"  n ( f i1 sh - s l ai s i z ) ((pos "Kj%" ) ))
( "fishball"  n ( f i1 sh b oo l ) ((pos "K6$" ) ))
( "fishballs"  n ( f i1 sh b oo l z ) ((pos "Kj$" ) ))
( "fishbone"  n ( f i1 sh b ou n ) ((pos "K6%" ) ))
( "fishbones"  n ( f i1 sh b ou n z ) ((pos "Kj%" ) ))
( "fishcake"  n ( f i1 sh k ei k ) ((pos "K6%" ) ))
( "fishcakes"  n ( f i1 sh k ei k s ) ((pos "Kj%" ) ))
( "fished"  v ( f i1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fisher"  n ( f i1 sh @ r ) ((pos "K6%" ) ))
( "fisheries"  n ( f i1 sh @ r i z ) ((pos "Kj%" ) ))
( "fisherman"  n ( f i1 sh @ m @ n ) ((pos "Ki%" ) ))
( "fishermen"  n ( f i1 sh @ m @ n ) ((pos "Kj%" ) ))
( "fishers"  n ( f i1 sh @ z ) ((pos "Kj%" ) ))
( "fishery"  n ( f i1 sh @ r ii ) ((pos "K8%" ) ))
( "fishes"  v ( f i1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fishier"  j ( f i1 sh i@ r ) ((pos "Or%" ) ))
( "fishiest"  j ( f i1 sh i i s t ) ((pos "Os%" ) ))
( "fishing"  v ( f i1 sh i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fishing-line"  n ( f i1 sh i ng - l ai n ) ((pos "K6%" ) ))
( "fishing-lines"  n ( f i1 sh i ng - l ai n z ) ((pos "Kj%" ) ))
( "fishing-rod"  n ( f i1 sh i ng - r o d ) ((pos "K6%" ) ))
( "fishing-rods"  n ( f i1 sh i ng - r o d z ) ((pos "Kj%" ) ))
( "fishing-tackle"  n ( f i1 sh i ng - t a k l ) ((pos "L@%" ) ))
( "fishmonger"  n ( f i1 sh m uh ng g @ r ) ((pos "K6%" ) ))
( "fishmongers"  n ( f i1 sh m uh ng g @ z ) ((pos "Kj%" ) ))
( "fishpaste"  n ( f i1 sh p ei s t ) ((pos "M6%" ) ))
( "fishpastes"  n ( f i1 sh p ei s t s ) ((pos "Mj%" ) ))
( "fishplate"  n ( f i1 sh p l ei t ) ((pos "K6$" ) ))
( "fishplates"  n ( f i1 sh p l ei t s ) ((pos "Kj$" ) ))
( "fishwife"  n ( f i1 sh w ai f ) ((pos "Ki%" ) ))
( "fishwives"  n ( f i1 sh w ai v z ) ((pos "Kj%" ) ))
( "fishy"  j ( f i1 sh ii ) ((pos "OD%" ) ))
( "fissile"  j ( f i1 s ai l ) ((pos "OA$" ) ))
( "fission"  n ( f i1 sh @ n ) ((pos "L@%" ) ))
( "fissionable"  j ( f i1 sh @ n @ b l ) ((pos "OA$" ) ))
( "fissiparous"  j ( f i1 s i1 p @ r @ s ) ((pos "OA$" ) ))
( "fissure"  n ( f i1 sh @ r ) ((pos "K6%" ) ))
( "fissures"  n ( f i1 sh @ z ) ((pos "Kj%" ) ))
( "fist"  n ( f i1 s t ) ((pos "K6%" ) ))
( "fisticuffs"  n ( f i1 s t i k uh f s ) ((pos "Kj%" ) ))
( "fists"  n ( f i1 s t s ) ((pos "Kj%" ) ))
( "fistula"  n ( f i1 s t y u l @ ) ((pos "K6$" ) ))
( "fistulas"  n ( f i1 s t y u l @ z ) ((pos "Kj$" ) ))
( "fit"  v ( f i1 t ) ((pos "J4%" "K6%" "OE%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "fitful"  j ( f i1 t f @ l ) ((pos "OA%" ) ))
( "fitfully"  a ( f i1 t f @ l ii ) ((pos "Pu%" ) ))
( "fitly"  a ( f i1 t l ii ) ((pos "Pu$" ) ))
( "fitment"  n ( f i1 t m @ n t ) ((pos "K6$" ) ))
( "fitments"  n ( f i1 t m @ n t s ) ((pos "Kj%" ) ))
( "fitness"  n ( f i1 t n @ s ) ((pos "L@%" ) ))
( "fits"  v ( f i1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "fitted"  v ( f i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "fitter"  n ( f i1 t @ r ) ((pos "K6%" "Or%" ) ))
( "fitters"  n ( f i1 t @ z ) ((pos "Kj%" ) ))
( "fittest"  j ( f i1 t i s t ) ((pos "Os%" ) ))
( "fitting"  v ( f i1 t i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "fittings"  n ( f i1 t i ng z ) ((pos "Kj%" ) ))
( "five"  n ( f ai1 v ) ((pos "K6%" "OA*" ) ))
( "fivefold"  j ( f ai1 v f ou l d ) ((pos "OA%" ) ))
( "fivepence"  n ( f ai1 v p @ n s ) ((pos "K6%" ) ))
( "fivepences"  n ( f ai1 v p @ n s i z ) ((pos "Kj$" ) ))
( "fivepenny"  j ( f ai1 v p @ n ii ) ((pos "OA%" ) ))
( "fiver"  n ( f ai1 v @ r ) ((pos "K6%" ) ))
( "fivers"  n ( f ai1 v @ z ) ((pos "Kj%" ) ))
( "fives"  n ( f ai1 v z ) ((pos "Kj%" "Lk$" ) ))
( "fix"  v ( f i1 k s ) ((pos "J1%" "K7%" ) (subcat "3A" "6A" "14" "15A" )))
( "fixate"  v ( f i1 k s ei1 t ) ((pos "H2%" ) (subcat )))
( "fixated"  v ( f i1 k s ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fixates"  v ( f i1 k s ei1 t s ) ((pos "Ha%" ) (subcat )))
( "fixating"  v ( f i1 k s ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "fixation"  n ( f i1 k s ei1 sh @ n ) ((pos "M6%" ) ))
( "fixations"  n ( f i1 k s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fixative"  n ( f i1 k s @ t i v ) ((pos "K6%" ) ))
( "fixatives"  n ( f i1 k s @ t i v z ) ((pos "Kj%" ) ))
( "fixed"  v ( f i1 k s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" "14" "15A" )))
( "fixedly"  a ( f i1 k s i d l ii ) ((pos "Pu%" ) ))
( "fixes"  v ( f i1 k s i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" "15A" )))
( "fixing"  v ( f i1 k s i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" "15A" )))
( "fixture"  n ( f i1 k s ch @ r ) ((pos "K6%" ) ))
( "fixtures"  n ( f i1 k s ch @ z ) ((pos "Kj%" ) ))
( "fizz"  v ( f i1 z ) ((pos "I1%" "L@%" ) (subcat "2A" )))
( "fizzed"  v ( f i1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fizzes"  v ( f i1 z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "fizzier"  j ( f i1 z i@ r ) ((pos "Or%" ) ))
( "fizziest"  j ( f i1 z i i s t ) ((pos "Os%" ) ))
( "fizzing"  v ( f i1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fizzle"  v ( f i1 z l ) ((pos "I2%" ) (subcat "2A" )))
( "fizzled"  v ( f i1 z l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fizzles"  v ( f i1 z l z ) ((pos "Ia%" ) (subcat "2A" )))
( "fizzling"  v ( f i1 z l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fizzy"  j ( f i1 z ii ) ((pos "OD%" ) ))
( "fjord"  n ( f i1 oo1 d ) ((pos "K6$" ) ))
( "fjords"  n ( f i1 oo1 d z ) ((pos "Kj$" ) ))
( "flabbergast"  v ( f l a1 b @ g aa s t ) ((pos "H0$" ) (subcat )))
( "flabbergasted"  v ( f l a1 b @ g aa s t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "flabbergasting"  v ( f l a1 b @ g aa s t i ng ) ((pos "Hb$" ) (subcat )))
( "flabbergasts"  v ( f l a1 b @ g aa s t s ) ((pos "Ha$" ) (subcat )))
( "flabbier"  j ( f l a1 b i@ r ) ((pos "Or%" ) ))
( "flabbiest"  j ( f l a1 b i i s t ) ((pos "Os%" ) ))
( "flabbily"  a ( f l a1 b i l ii ) ((pos "Pu%" ) ))
( "flabbiness"  n ( f l a1 b i n @ s ) ((pos "L@%" ) ))
( "flabby"  j ( f l a1 b ii ) ((pos "OD%" ) ))
( "flaccid"  j ( f l a1 k s i d ) ((pos "OA%" ) ))
( "flaccidity"  n ( f l a1 k s i1 d i t ii ) ((pos "L@$" ) ))
( "flag"  v ( f l a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" )))
( "flag-captain"  n ( f l a1 g - k a p t i n ) ((pos "K6$" ) ))
( "flag-captains"  n ( f l a1 g - k a p t i n z ) ((pos "Kj$" ) ))
( "flag-day"  n ( f l a1 g - d ei ) ((pos "K6%" ) ))
( "flag-days"  n ( f l a1 g - d ei z ) ((pos "Kj%" ) ))
( "flagellant"  n ( f l a1 jh @ l @ n t ) ((pos "K6$" ) ))
( "flagellants"  n ( f l a1 jh @ l @ n t s ) ((pos "Kj$" ) ))
( "flagellate"  v ( f l a1 jh @ l ei t ) ((pos "H2%" ) (subcat )))
( "flagellated"  v ( f l a1 jh @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flagellates"  v ( f l a1 jh @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "flagellating"  v ( f l a1 jh @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "flagellation"  n ( f l a2 jh @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "flagellations"  n ( f l a2 jh @ l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "flageolet"  n ( f l a2 jh ou l e1 t ) ((pos "K6$" ) ))
( "flageolets"  n ( f l a2 jh ou l e1 t s ) ((pos "Kj$" ) ))
( "flagged"  v ( f l a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "flagging"  v ( f l a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "flagon"  n ( f l a1 g @ n ) ((pos "K6%" ) ))
( "flagons"  n ( f l a1 g @ n z ) ((pos "Kj%" ) ))
( "flagpole"  n ( f l a1 g p ou l ) ((pos "K6%" ) ))
( "flagpoles"  n ( f l a1 g p ou l z ) ((pos "Kj%" ) ))
( "flagrant"  j ( f l ei1 g r @ n t ) ((pos "OA%" ) ))
( "flagrantly"  a ( f l ei1 g r @ n t l ii ) ((pos "Pu%" ) ))
( "flags"  v ( f l a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "flagship"  n ( f l a1 g sh i p ) ((pos "K6%" ) ))
( "flagships"  n ( f l a1 g sh i p s ) ((pos "Kj%" ) ))
( "flagstaff"  n ( f l a1 g s t aa f ) ((pos "K6%" ) ))
( "flagstaffs"  n ( f l a1 g s t aa f s ) ((pos "Kj%" ) ))
( "flagstone"  n ( f l a1 g s t ou n ) ((pos "K6%" ) ))
( "flagstones"  n ( f l a1 g s t ou n z ) ((pos "Kj%" ) ))
( "flail"  v ( f l ei1 l ) ((pos "H0%" "K6%" ) (subcat )))
( "flailed"  v ( f l ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flailing"  v ( f l ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "flails"  v ( f l ei1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "flair"  n ( f l e@1 r ) ((pos "M6%" ) ))
( "flairs"  n ( f l e@1 z ) ((pos "Mj$" ) ))
( "flak"  n ( f l a1 k ) ((pos "L@%" ) ))
( "flake"  v ( f l ei1 k ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "flaked"  v ( f l ei1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flakes"  v ( f l ei1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "flakier"  j ( f l ei1 k i@ r ) ((pos "Or%" ) ))
( "flakiest"  j ( f l ei1 k i i s t ) ((pos "Os%" ) ))
( "flakiness"  n ( f l ei1 k i n @ s ) ((pos "L@%" ) ))
( "flaking"  v ( f l ei1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flaky"  j ( f l ei1 k ii ) ((pos "OD%" ) ))
( "flambeau"  n ( f l a1 m b ou ) ((pos "K6$" ) ))
( "flambeaus"  n ( f l a1 m b ou z ) ((pos "Kj$" ) ))
( "flambeaux"  n ( f l a1 m b ou z ) ((pos "Kj$" ) ))
( "flamboyance"  n ( f l a1 m b oi1 @ n s ) ((pos "L@%" ) ))
( "flamboyant"  j ( f l a1 m b oi1 @ n t ) ((pos "OA%" ) ))
( "flamboyantly"  a ( f l a1 m b oi1 @ n t l ii ) ((pos "Pu%" ) ))
( "flame"  v ( f l ei1 m ) ((pos "I2%" "M6%" ) (subcat "2A" )))
( "flamed"  v ( f l ei1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flames"  v ( f l ei1 m z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "flamethrower"  n ( f l ei1 m th r ou @ r ) ((pos "K6%" ) ))
( "flamethrowers"  n ( f l ei1 m th r ou @ z ) ((pos "Kj%" ) ))
( "flaming"  v ( f l ei1 m i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "flamingo"  n ( f l @1 m i1 ng g ou ) ((pos "K6%" ) ))
( "flamingoes"  n ( f l @1 m ii1 ng g ou z ) ((pos "Kj%" ) ))
( "flamingos"  n ( f l @1 m i1 ng g ou z ) ((pos "Kj%" ) ))
( "flammable"  j ( f l a1 m @ b l ) ((pos "OA%" ) ))
( "flan"  n ( f l a1 n ) ((pos "K6%" ) ))
( "flange"  n ( f l a1 n jh ) ((pos "K6%" ) ))
( "flanges"  n ( f l a1 n jh i z ) ((pos "Kj%" ) ))
( "flank"  v ( f l a1 ng k ) ((pos "H0%" "K6%" ) (subcat )))
( "flanked"  v ( f l a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flanking"  v ( f l a1 ng k i ng ) ((pos "Hb%" ) (subcat )))
( "flanks"  v ( f l a1 ng k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "flannel"  n ( f l a1 n l ) ((pos "M6%" ) ))
( "flannelette"  n ( f l a2 n @ l e1 t ) ((pos "L@%" ) ))
( "flannels"  n ( f l a1 n l z ) ((pos "Mj%" ) ))
( "flans"  n ( f l a1 n z ) ((pos "Kj%" ) ))
( "flap"  v ( f l a1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "flapjack"  n ( f l a1 p jh a k ) ((pos "M6%" ) ))
( "flapjacks"  n ( f l a1 p jh a k s ) ((pos "Mj$" ) ))
( "flapped"  v ( f l a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "flapper"  n ( f l a1 p @ r ) ((pos "K6$" ) ))
( "flappers"  n ( f l a1 p @ z ) ((pos "Kj$" ) ))
( "flapping"  v ( f l a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "flaps"  v ( f l a1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "flare"  v ( f l e@1 r ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" )))
( "flare-path"  n ( f l e@1 - p aa th ) ((pos "K6$" ) ))
( "flare-paths"  n ( f l e@1 - p aa dh z ) ((pos "Kj$" ) ))
( "flare-up"  n ( f l e@1 r - uh p ) ((pos "K6%" ) ))
( "flare-ups"  n ( f l e@1 r - uh p s ) ((pos "Kj%" ) ))
( "flared"  v ( f l e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "flares"  v ( f l e@1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "flaring"  v ( f l e@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "flash"  v ( f l a1 sh ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "12C" )))
( "flashback"  n ( f l a1 sh b a k ) ((pos "K6%" ) ))
( "flashbacks"  n ( f l a1 sh b a k s ) ((pos "Kj%" ) ))
( "flashbulb"  n ( f l a1 sh b uh l b ) ((pos "K6%" ) ))
( "flashbulbs"  n ( f l a1 sh b uh l b z ) ((pos "Kj%" ) ))
( "flashed"  v ( f l a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12C" )))
( "flashes"  v ( f l a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "12C" )))
( "flashgun"  n ( f l a1 sh g uh n ) ((pos "K6%" ) ))
( "flashguns"  n ( f l a1 sh g uh n z ) ((pos "Kj%" ) ))
( "flashier"  j ( f l a1 sh i@ r ) ((pos "Or%" ) ))
( "flashiest"  j ( f l a1 sh i i s t ) ((pos "Os%" ) ))
( "flashily"  a ( f l a1 sh i l ii ) ((pos "Pu%" ) ))
( "flashing"  v ( f l a1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "12C" )))
( "flashlight"  n ( f l a1 sh l ai t ) ((pos "K6%" ) ))
( "flashlights"  n ( f l a1 sh l ai t s ) ((pos "Kj%" ) ))
( "flashpoint"  n ( f l a1 sh p oi n t ) ((pos "K6%" ) ))
( "flashpoints"  n ( f l a1 sh p oi n t s ) ((pos "Kj%" ) ))
( "flashy"  j ( f l a1 sh ii ) ((pos "OD%" ) ))
( "flask"  n ( f l aa1 s k ) ((pos "K6%" ) ))
( "flasks"  n ( f l aa1 s k s ) ((pos "Kj%" ) ))
( "flat"  n ( f l a1 t ) ((pos "K6%" "OE%" "Pu%" ) ))
( "flat-bottomed"  j ( f l a1 t - b o1 t @ m d ) ((pos "OA%" ) ))
( "flat-car"  n ( f l a1 t - k aa r ) ((pos "K6$" ) ))
( "flat-cars"  n ( f l a1 t - k aa z ) ((pos "Kj$" ) ))
( "flat-footed"  j ( f l a1 t - f u1 t i d ) ((pos "OA%" ) ))
( "flat-iron"  n ( f l a1 t - ai @ n ) ((pos "K6%" ) ))
( "flat-irons"  n ( f l a1 t - ai @ n z ) ((pos "Kj%" ) ))
( "flatfish"  n ( f l a1 t f i sh ) ((pos "K9%" ) ))
( "flatfishes"  n ( f l a1 t f i sh i z ) ((pos "Kj%" ) ))
( "flatlet"  n ( f l a1 t l i t ) ((pos "K6$" ) ))
( "flatlets"  n ( f l a1 t l i t s ) ((pos "Kj$" ) ))
( "flatly"  a ( f l a1 t l ii ) ((pos "Pu%" ) ))
( "flatness"  n ( f l a1 t n @ s ) ((pos "L@%" ) ))
( "flats"  n ( f l a1 t s ) ((pos "Kj%" ) ))
( "flatten"  v ( f l a1 t n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flattened"  v ( f l a1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flattening"  v ( f l a1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flattens"  v ( f l a1 t n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flatter"  v ( f l a1 t @ r ) ((pos "H0%" "Or%" ) (subcat )))
( "flattered"  v ( f l a1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flatterer"  n ( f l a1 t @ r @ r ) ((pos "K6%" ) ))
( "flatterers"  n ( f l a1 t @ r @ z ) ((pos "Kj%" ) ))
( "flatteries"  n ( f l a1 t @ r i z ) ((pos "Mj%" ) ))
( "flattering"  v ( f l a1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "flatters"  v ( f l a1 t @ z ) ((pos "Ha%" ) (subcat )))
( "flattery"  n ( f l a1 t @ r ii ) ((pos "M8%" ) ))
( "flattest"  j ( f l a1 t i s t ) ((pos "Os%" ) ))
( "flattop"  n ( f l a1 t - t o p ) ((pos "K6$" ) ))
( "flattops"  n ( f l a1 t - t o p s ) ((pos "Kj$" ) ))
( "flatulence"  n ( f l a1 t y u l @ n s ) ((pos "L@%" ) ))
( "flaunt"  v ( f l oo1 n t ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "flaunted"  v ( f l oo1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "flaunting"  v ( f l oo1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "flaunts"  v ( f l oo1 n t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "flautist"  n ( f l oo1 t i s t ) ((pos "K6%" ) ))
( "flautists"  n ( f l oo1 t i s t s ) ((pos "Kj%" ) ))
( "flavour"  v ( f l ei1 v @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "flavoured"  v ( f l ei1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flavouring"  v ( f l ei1 v @ r i ng ) ((pos "Hb%" "M6%" ) (subcat )))
( "flavourings"  n ( f l ei1 v @ r i ng z ) ((pos "Mj%" ) ))
( "flavourless"  j ( f l ei1 v @ l @ s ) ((pos "OA%" ) ))
( "flavours"  v ( f l ei1 v @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "flaw"  n ( f l oo1 ) ((pos "K6%" ) ))
( "flawed"  j ( f l oo1 d ) ((pos "OA%" ) ))
( "flawless"  j ( f l oo1 l @ s ) ((pos "OA%" ) ))
( "flawlessly"  a ( f l oo1 l @ s l ii ) ((pos "Pu%" ) ))
( "flaws"  n ( f l oo1 z ) ((pos "Kj%" ) ))
( "flax"  n ( f l a1 k s ) ((pos "L@$" ) ))
( "flaxen"  j ( f l a1 k s n ) ((pos "OA%" ) ))
( "flay"  v ( f l ei1 ) ((pos "H0%" ) (subcat )))
( "flayed"  v ( f l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flaying"  v ( f l ei1 i ng ) ((pos "Hb$" ) (subcat )))
( "flays"  v ( f l ei1 z ) ((pos "Ha$" ) (subcat )))
( "flea"  n ( f l ii1 ) ((pos "K6%" ) ))
( "flea-bite"  n ( f l ii1 - b ai t ) ((pos "K6%" ) ))
( "flea-bites"  n ( f l ii1 - b ai t s ) ((pos "Kj%" ) ))
( "flea-bitten"  j ( f l ii1 - b i t n ) ((pos "OA%" ) ))
( "fleapit"  n ( f l ii1 p i t ) ((pos "K6%" ) ))
( "fleapits"  n ( f l ii1 p i t s ) ((pos "Kj%" ) ))
( "fleas"  n ( f l ii1 z ) ((pos "Kj%" ) ))
( "fleck"  v ( f l e1 k ) ((pos "H0$" "K6%" ) (subcat )))
( "flecked"  v ( f l e1 k t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "flecking"  v ( f l e1 k i ng ) ((pos "Hb$" ) (subcat )))
( "flecks"  v ( f l e1 k s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "fled"  v ( f l e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fledged"  j ( f l e1 jh d ) ((pos "OA$" ) ))
( "fledgeling"  n ( f l e1 jh l i ng ) ((pos "K6%" ) ))
( "fledgelings"  n ( f l e1 jh l i ng z ) ((pos "Kj%" ) ))
( "fledgling"  n ( f l e1 jh l i ng ) ((pos "K6%" ) ))
( "fledglings"  n ( f l e1 jh l i ng z ) ((pos "Kj%" ) ))
( "flee"  v ( f l ii1 ) ((pos "J5%" ) (subcat "2A" "2C" )))
( "fleece"  v ( f l ii1 s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "fleeced"  v ( f l ii1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fleeces"  v ( f l ii1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "fleecier"  j ( f l ii1 s i@ r ) ((pos "Or$" ) ))
( "fleeciest"  j ( f l ii1 s i i s t ) ((pos "Os$" ) ))
( "fleecing"  v ( f l ii1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fleecy"  j ( f l ii1 s ii ) ((pos "OD%" ) ))
( "fleeing"  v ( f l ii1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "flees"  v ( f l ii1 z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "fleet"  n ( f l ii1 t ) ((pos "K6%" "OC%" ) ))
( "fleeter"  j ( f l ii1 t @ r ) ((pos "Or$" ) ))
( "fleetest"  j ( f l ii1 t i s t ) ((pos "Os$" ) ))
( "fleeting"  j ( f l ii1 t i ng ) ((pos "OA%" ) ))
( "fleetly"  a ( f l ii1 t l ii ) ((pos "Pu$" ) ))
( "fleetness"  n ( f l ii1 t n @ s ) ((pos "L@%" ) ))
( "fleets"  n ( f l ii1 t s ) ((pos "Kj%" ) ))
( "flesh"  n ( f l e1 sh ) ((pos "L@%" ) ))
( "flesh-wound"  n ( f l e1 sh - w uu n d ) ((pos "K6%" ) ))
( "flesh-wounds"  n ( f l e1 sh - w uu n d z ) ((pos "Kj%" ) ))
( "fleshier"  j ( f l e1 sh i@ r ) ((pos "Or$" ) ))
( "fleshiest"  j ( f l e1 sh i i s t ) ((pos "Os$" ) ))
( "fleshings"  n ( f l e1 sh i ng z ) ((pos "Kj$" ) ))
( "fleshly"  j ( f l e1 sh l ii ) ((pos "OA$" ) ))
( "fleshpots"  n ( f l e1 sh p o t s ) ((pos "Kj%" ) ))
( "fleshy"  j ( f l e1 sh ii ) ((pos "OD%" ) ))
( "fleur-de-lis"  n ( f l @@2 - d @ - l ii1 ) ((pos "Ki$" ) ))
( "fleur-de-lys"  n ( f l @@2 - d @ - l ii1 ) ((pos "Ki%" ) ))
( "fleurs-de-lis"  n ( f l @@2 - d @ - l ii1 ) ((pos "Kj$" ) ))
( "fleurs-de-lys"  n ( f l @@2 - d @ - l ii1 ) ((pos "Kj$" ) ))
( "flew"  v ( f l uu1 ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "flex"  v ( f l e1 k s ) ((pos "H1%" "M7%" ) (subcat )))
( "flexed"  v ( f l e1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flexes"  v ( f l e1 k s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "flexibility"  n ( f l e2 k s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "flexible"  j ( f l e1 k s @ b l ) ((pos "OA%" ) ))
( "flexibly"  a ( f l e1 k s @ b l ii ) ((pos "Pu%" ) ))
( "flexing"  v ( f l e1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "flibbertigibbet"  n ( f l i2 b @ t i jh i1 b i t ) ((pos "K6$" ) ))
( "flibbertigibbets"  n ( f l i2 b @ t i jh i1 b i t s ) ((pos "Kj$" ) ))
( "flick"  v ( f l i1 k ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" "15B" )))
( "flick-knife"  n ( f l i1 k - n ai f ) ((pos "Ki%" ) ))
( "flick-knives"  n ( f l i1 k - n ai v z ) ((pos "Kj%" ) ))
( "flicked"  v ( f l i1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" "15B" )))
( "flicker"  v ( f l i1 k @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "flickered"  v ( f l i1 k @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flickering"  v ( f l i1 k @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flickers"  v ( f l i1 k @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "flicking"  v ( f l i1 k i ng ) ((pos "Hb%" ) (subcat "6A" "15A" "15B" )))
( "flicks"  v ( f l i1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" "15B" )))
( "flier"  n ( f l ai1 @ r ) ((pos "K6%" ) ))
( "fliers"  n ( f l ai1 @ z ) ((pos "Kj%" ) ))
( "flies"  v ( f l ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "flight"  v ( f l ai1 t ) ((pos "H0$" "M6%" ) (subcat )))
( "flighted"  v ( f l ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flighting"  v ( f l ai1 t i ng ) ((pos "Hb$" ) (subcat )))
( "flightless"  j ( f l ai1 t l @ s ) ((pos "OA%" ) ))
( "flights"  v ( f l ai1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "flighty"  j ( f l ai1 t ii ) ((pos "OA%" ) ))
( "flimsier"  j ( f l i1 m z i@ r ) ((pos "Or%" ) ))
( "flimsiest"  j ( f l i1 m z i i s t ) ((pos "Os%" ) ))
( "flimsily"  a ( f l i1 m z i l ii ) ((pos "Pu%" ) ))
( "flimsiness"  n ( f l i1 m s i n @ s ) ((pos "L@%" ) ))
( "flimsy"  n ( f l i1 m z ii ) ((pos "L@%" "OD%" ) ))
( "flinch"  v ( f l i1 n ch ) ((pos "I1%" ) (subcat "2A" )))
( "flinched"  v ( f l i1 n ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flinches"  v ( f l i1 n ch i z ) ((pos "Ia%" ) (subcat "2A" )))
( "flinching"  v ( f l i1 n ch i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fling"  v ( f l i1 ng ) ((pos "J5%" "K6%" ) (subcat "2C" "6A" "12A" "13A" "15A" "15B" )))
( "flinging"  v ( f l i1 ng i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "12A" "13A" "15A" "15B" )))
( "flings"  v ( f l i1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "12A" "13A" "15A" "15B" )))
( "flint"  n ( f l i1 n t ) ((pos "M6%" ) ))
( "flintier"  j ( f l i1 n t i@ r ) ((pos "Or$" ) ))
( "flintiest"  j ( f l i1 n t i i s t ) ((pos "Os$" ) ))
( "flints"  n ( f l i1 n t s ) ((pos "Mj%" ) ))
( "flintstone"  n ( f l i1 n t s t ou n ) ((pos "L@%" ) ))
( "flinty"  j ( f l i1 n t ii ) ((pos "OD%" ) ))
( "flip"  v ( f l i1 p ) ((pos "J4%" "K6%" "OA%" ) (subcat "6A" "15A" )))
( "flippancies"  n ( f l i1 p @ n s i z ) ((pos "Mj$" ) ))
( "flippancy"  n ( f l i1 p @ n s ii ) ((pos "M8%" ) ))
( "flippant"  j ( f l i1 p @ n t ) ((pos "OA%" ) ))
( "flippantly"  a ( f l i1 p @ n t l ii ) ((pos "Pu%" ) ))
( "flipped"  v ( f l i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15A" )))
( "flipper"  n ( f l i1 p @ r ) ((pos "K6%" ) ))
( "flippers"  n ( f l i1 p @ z ) ((pos "Kj%" ) ))
( "flipping"  v ( f l i1 p i ng ) ((pos "Jb%" ) (subcat "6A" "15A" )))
( "flips"  v ( f l i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "6A" "15A" )))
( "flirt"  v ( f l @@1 t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "flirtation"  n ( f l @@1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "flirtations"  n ( f l @@1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "flirtatious"  j ( f l @@1 t ei1 sh @ s ) ((pos "OA%" ) ))
( "flirted"  v ( f l @@1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flirting"  v ( f l @@1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flirts"  v ( f l @@1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "flit"  v ( f l i1 t ) ((pos "I4%" "K6%" ) (subcat )))
( "flits"  v ( f l i1 t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "flitted"  v ( f l i1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "flitting"  v ( f l i1 t i ng ) ((pos "Ib%" ) (subcat )))
( "float"  v ( f l ou1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "floatation"  n ( f l ou1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "floatations"  n ( f l ou1 t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "floated"  v ( f l ou1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "floating"  v ( f l ou1 t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "floats"  v ( f l ou1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flock"  v ( f l o1 k ) ((pos "I0%" "K6%" ) (subcat "2C" )))
( "flocked"  v ( f l o1 k t ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "flocking"  v ( f l o1 k i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "flocks"  v ( f l o1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2C" )))
( "floe"  n ( f l ou1 ) ((pos "K6$" ) ))
( "floes"  n ( f l ou1 z ) ((pos "Kj$" ) ))
( "flog"  v ( f l o1 g ) ((pos "H4%" ) (subcat )))
( "flogged"  v ( f l o1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flogging"  v ( f l o1 g i ng ) ((pos "Hb%" "M6%" ) (subcat )))
( "floggings"  n ( f l o1 g i ng z ) ((pos "Mj%" ) ))
( "flogs"  v ( f l o1 g z ) ((pos "Ha%" ) (subcat )))
( "flood"  v ( f l uh1 d ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" "14" "15B" )))
( "flood-tide"  n ( f l uh1 d - t ai1 d ) ((pos "K6%" ) ))
( "flood-tides"  n ( f l uh1 d - t ai1 d z ) ((pos "Kj%" ) ))
( "flooded"  v ( f l uh1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" "15B" )))
( "floodgate"  n ( f l uh1 d g ei t ) ((pos "K6%" ) ))
( "floodgates"  n ( f l uh1 d g ei t s ) ((pos "Kj%" ) ))
( "flooding"  v ( f l uh1 d i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" "15B" )))
( "floodlight"  v ( f l uh1 d l ai t ) ((pos "H0%" ) ))
( "floodlighted"  v ( f l uh1 d l ai t i d ) ((pos "Hc$" "Hd$" ) ))
( "floodlighting"  v ( f l uh1 d l ai t i ng ) ((pos "Hb%" ) ))
( "floodlights"  v ( f l uh1 d l ai t s ) ((pos "Ha%" "Kj%" ) ))
( "floodlit"  v ( f l uh1 d l i t ) ((pos "Hc%" "Hd%" ) ))
( "floods"  v ( f l uh1 d z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" "15B" )))
( "floor"  v ( f l oo1 r ) ((pos "H0%" "K6%" ) (subcat )))
( "floor-walker"  n ( f l oo1 - w oo k @ r ) ((pos "K6%" ) ))
( "floor-walkers"  n ( f l oo1 - w oo k @ z ) ((pos "Kj%" ) ))
( "floorboard"  n ( f l oo1 b oo d ) ((pos "K6%" ) ))
( "floorboards"  n ( f l oo1 b oo d z ) ((pos "Kj%" ) ))
( "floored"  v ( f l oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flooring"  v ( f l oo1 r i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "floors"  v ( f l oo1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "floozie"  n ( f l uu1 z ii ) ((pos "K6$" ) ))
( "floozies"  n ( f l uu1 z i z ) ((pos "Kj$" ) ))
( "floozy"  n ( f l uu1 z ii ) ((pos "K8$" ) ))
( "flop"  v ( f l o1 p ) ((pos "J4%" "K6%" "Pu%" ) (subcat "2A" "2C" "15A" )))
( "flopped"  v ( f l o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "15A" )))
( "floppier"  j ( f l o1 p i@ r ) ((pos "Or%" ) ))
( "floppiest"  j ( f l o1 p i i s t ) ((pos "Os%" ) ))
( "flopping"  v ( f l o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "15A" )))
( "floppy"  j ( f l o1 p ii ) ((pos "OD%" ) ))
( "flops"  v ( f l o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "15A" )))
( "flora"  n ( f l oo1 r @ ) ((pos "Kj%" ) ))
( "floral"  j ( f l oo1 r @ l ) ((pos "OA%" ) ))
( "floriculture"  n ( f l oo1 r i k uh l ch @ r ) ((pos "L@$" ) ))
( "florid"  j ( f l o1 r i d ) ((pos "OA%" ) ))
( "floridly"  a ( f l o1 r i d l ii ) ((pos "Pu$" ) ))
( "florin"  n ( f l o1 r i n ) ((pos "K6$" ) ))
( "florins"  n ( f l o1 r i n z ) ((pos "Kj$" ) ))
( "florist"  n ( f l o1 r i s t ) ((pos "K6%" ) ))
( "florists"  n ( f l o1 r i s t s ) ((pos "Kj%" ) ))
( "floss"  n ( f l o1 s ) ((pos "L@%" ) ))
( "flotation"  n ( f l ou1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "flotations"  n ( f l ou1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "flotilla"  n ( f l @1 t i1 l @ ) ((pos "K6%" ) ))
( "flotillas"  n ( f l @1 t i1 l @ z ) ((pos "Kj%" ) ))
( "flotsam"  n ( f l o1 t s @ m ) ((pos "L@%" ) ))
( "flounce"  v ( f l au1 n s ) ((pos "J2%" "K6%" ) (subcat "2C" )))
( "flounced"  v ( f l au1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "flounces"  v ( f l au1 n s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "flouncing"  v ( f l au1 n s i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "flounder"  v ( f l au1 n d @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "floundered"  v ( f l au1 n d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "floundering"  v ( f l au1 n d @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flounders"  v ( f l au1 n d @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "flour"  v ( f l au1 @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "floured"  v ( f l au1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flourier"  j ( f l au1 @ r i@ r ) ((pos "Or$" ) ))
( "flouriest"  j ( f l au1 @ r i i s t ) ((pos "Os$" ) ))
( "flouring"  v ( f l au1 @ r i ng ) ((pos "Hb$" ) (subcat )))
( "flourish"  v ( f l uh1 r i sh ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "flourished"  v ( f l uh1 r i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "flourishes"  v ( f l uh1 r i sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "flourishing"  v ( f l uh1 r i sh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "flours"  v ( f l au1 @ z ) ((pos "Ha$" ) (subcat )))
( "floury"  j ( f l au1 @ r ii ) ((pos "OD%" ) ))
( "flout"  v ( f l au1 t ) ((pos "H0%" ) (subcat )))
( "flouted"  v ( f l au1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flouting"  v ( f l au1 t i ng ) ((pos "Hb%" ) (subcat )))
( "flouts"  v ( f l au1 t s ) ((pos "Ha%" ) (subcat )))
( "flow"  v ( f l ou1 ) ((pos "I0%" "Ki%" ) (subcat "2A" )))
( "flowed"  v ( f l ou1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "flower"  v ( f l au1 @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "flower-girl"  n ( f l au1 @ - g @@ l ) ((pos "K6%" ) ))
( "flower-girls"  n ( f l au1 @ - g @@ l z ) ((pos "Kj%" ) ))
( "flowerbed"  n ( f l au1 @ b e d ) ((pos "K6%" ) ))
( "flowerbeds"  n ( f l au1 @ b e d z ) ((pos "Kj%" ) ))
( "flowered"  v ( f l au1 @ d ) ((pos "Ic%" "Id%" "OA%" ) (subcat "2A" )))
( "flowerier"  j ( f l au1 @ r i@ r ) ((pos "Or%" ) ))
( "floweriest"  j ( f l au1 @ r i i s t ) ((pos "Os%" ) ))
( "flowering"  v ( f l au1 @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flowerless"  j ( f l au1 @ l @ s ) ((pos "OA$" ) ))
( "flowerpot"  n ( f l au1 @ p o t ) ((pos "K6%" ) ))
( "flowerpots"  n ( f l au1 @ p o t s ) ((pos "Kj%" ) ))
( "flowers"  v ( f l au1 @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "flowery"  j ( f l au1 @ r ii ) ((pos "OD%" ) ))
( "flowing"  v ( f l ou1 i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "flown"  v ( f l ou1 n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "flows"  v ( f l ou1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "flu"  n ( f l uu1 ) ((pos "L@%" ) ))
( "fluctuate"  v ( f l uh1 k ch u ei t ) ((pos "I2%" ) (subcat "2A" )))
( "fluctuated"  v ( f l uh1 k ch u ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "fluctuates"  v ( f l uh1 k ch u ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "fluctuating"  v ( f l uh1 k ch u ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "fluctuation"  n ( f l uh2 k ch u ei1 sh @ n ) ((pos "M6%" ) ))
( "fluctuations"  n ( f l uh2 k ch u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "flue"  n ( f l uu1 ) ((pos "K6%" ) ))
( "fluency"  n ( f l uu1 @ n s ii ) ((pos "L@%" ) ))
( "fluent"  j ( f l uu1 @ n t ) ((pos "OA%" ) ))
( "fluently"  a ( f l uu1 @ n t l ii ) ((pos "Pu%" ) ))
( "flues"  n ( f l uu1 z ) ((pos "Kj%" ) ))
( "fluff"  v ( f l uh1 f ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "fluffed"  v ( f l uh1 f t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fluffier"  j ( f l uh1 f i@ r ) ((pos "Or%" ) ))
( "fluffiest"  j ( f l uh1 f i i s t ) ((pos "Os%" ) ))
( "fluffing"  v ( f l uh1 f i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fluffs"  v ( f l uh1 f s ) ((pos "Ha%" ) (subcat "6A" )))
( "fluffy"  j ( f l uh1 f ii ) ((pos "OD%" ) ))
( "fluid"  n ( f l uu1 i d ) ((pos "M6%" "OA%" ) ))
( "fluidities"  n ( f l uu1 i1 d i t i z ) ((pos "Mj$" ) ))
( "fluidity"  n ( f l uu1 i1 d i t ii ) ((pos "M8%" ) ))
( "fluids"  n ( f l uu1 i d z ) ((pos "Mj%" ) ))
( "fluke"  n ( f l uu1 k ) ((pos "K6%" ) ))
( "flukes"  n ( f l uu1 k s ) ((pos "Kj%" ) ))
( "flume"  n ( f l uu1 m ) ((pos "K6$" ) ))
( "flumes"  n ( f l uu1 m z ) ((pos "Kj$" ) ))
( "flummox"  v ( f l uh1 m @ k s ) ((pos "H1$" ) (subcat )))
( "flummoxed"  v ( f l uh1 m @ k s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "flummoxes"  v ( f l uh1 m @ k s i z ) ((pos "Ha$" ) (subcat )))
( "flummoxing"  v ( f l uh1 m @ k s i ng ) ((pos "Hb$" ) (subcat )))
( "flung"  v ( f l uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "12A" "13A" "15A" "15B" )))
( "flunk"  v ( f l uh1 ng k ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "flunked"  v ( f l uh1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "flunkey"  n ( f l uh1 ng k ii ) ((pos "K6%" ) ))
( "flunkeys"  n ( f l uh1 ng k i z ) ((pos "Kj%" ) ))
( "flunkies"  n ( f l uh1 ng k i z ) ((pos "Kj$" ) ))
( "flunking"  v ( f l uh1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "flunks"  v ( f l uh1 ng k s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "flunky"  n ( f l uh1 ng k ii ) ((pos "K8$" ) ))
( "fluorescence"  n ( f l u@1 r e1 s n s ) ((pos "M6%" ) ))
( "fluorescences"  n ( f l u@1 r e1 s n s i z ) ((pos "Mj$" ) ))
( "fluorescent"  j ( f l u@1 r e1 s n t ) ((pos "OA%" ) ))
( "fluoridate"  v ( f l u@1 r i d ei t ) ((pos "H2%" ) (subcat )))
( "fluoridated"  v ( f l u@1 r i d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fluoridates"  v ( f l u@1 r i d ei t s ) ((pos "Ha%" ) (subcat )))
( "fluoridating"  v ( f l u@1 r i d ei t i ng ) ((pos "Hb%" ) (subcat )))
( "fluoridation"  n ( f l u@2 r i d ei1 sh @ n ) ((pos "K6%" ) ))
( "fluoridations"  n ( f l u@2 r i d ei1 sh @ n z ) ((pos "Kj$" ) ))
( "fluoride"  n ( f l u@1 r ai d ) ((pos "M6%" ) ))
( "fluorides"  n ( f l u@1 r ai d z ) ((pos "Mj$" ) ))
( "fluoridization"  n ( f l u@2 r i d ai z ei1 sh @ n ) ((pos "K6$" ) ))
( "fluoridizations"  n ( f l u@2 r i d ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "fluoridize"  v ( f l u@1 r i d ai z ) ((pos "H2$" ) ))
( "fluoridized"  v ( f l u@1 r i d ai z d ) ((pos "Hc$" "Hd$" ) ))
( "fluoridizes"  v ( f l u@1 r i d ai z i z ) ((pos "Ha$" ) ))
( "fluoridizing"  v ( f l u@1 r i d ai z i ng ) ((pos "Hb$" ) ))
( "fluorine"  n ( f l u@1 r ii n ) ((pos "L@%" ) ))
( "flurried"  v ( f l uh1 r i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flurries"  v ( f l uh1 r i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "flurry"  v ( f l uh1 r ii ) ((pos "H3$" "K8%" ) (subcat )))
( "flurrying"  v ( f l uh1 r i i ng ) ((pos "Hb$" ) (subcat )))
( "flush"  v ( f l uh1 sh ) ((pos "J1%" "M7%" "OA%" ) (subcat "2A" "2C" "2D" "6A" )))
( "flushed"  v ( f l uh1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "2D" "6A" )))
( "flushes"  v ( f l uh1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "6A" )))
( "flushing"  v ( f l uh1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "6A" )))
( "fluster"  v ( f l uh1 s t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "flustered"  v ( f l uh1 s t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "flustering"  v ( f l uh1 s t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "flusters"  v ( f l uh1 s t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "flute"  v ( f l uu1 t ) ((pos "J2%" "K6%" ) (subcat )))
( "fluted"  v ( f l uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "flutes"  v ( f l uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat )))
( "fluting"  v ( f l uu1 t i ng ) ((pos "Jb%" "L@%" ) (subcat )))
( "flutist"  n ( f l uu1 t i s t ) ((pos "K6$" ) ))
( "flutists"  n ( f l uu1 t i s t s ) ((pos "Kj$" ) ))
( "flutter"  v ( f l uh1 t @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fluttered"  v ( f l uh1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fluttering"  v ( f l uh1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "flutters"  v ( f l uh1 t @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "fluvial"  j ( f l uu1 v i@ l ) ((pos "OA$" ) ))
( "flux"  n ( f l uh1 k s ) ((pos "M7%" ) ))
( "fluxes"  n ( f l uh1 k s i z ) ((pos "Mj$" ) ))
( "fly"  v ( f l ai1 ) ((pos "J5%" "K8%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "fly-fish"  v ( f l ai1 - f i sh ) ((pos "I1%" ) (subcat )))
( "fly-fished"  v ( f l ai1 - f i sh t ) ((pos "Ic%" "Id%" ) (subcat )))
( "fly-fishes"  v ( f l ai1 - f i sh i z ) ((pos "Ia%" ) (subcat )))
( "fly-fishing"  v ( f l ai1 - f i sh i ng ) ((pos "Ib%" "L@%" ) (subcat )))
( "fly-swat"  n ( f l ai1 - s w o t ) ((pos "K6%" ) ))
( "fly-swats"  n ( f l ai1 - s w o t s ) ((pos "Kj$" ) ))
( "fly-swatter"  n ( f l ai1 - s w o t @ r ) ((pos "K6%" ) ))
( "fly-swatters"  n ( f l ai1 - s w o t @ z ) ((pos "Kj%" ) ))
( "flyblown"  j ( f l ai1 b l ou n ) ((pos "OA%" ) ))
( "flycatcher"  n ( f l ai1 k a ch @ r ) ((pos "K6%" ) ))
( "flycatchers"  n ( f l ai1 k a ch @ z ) ((pos "Kj%" ) ))
( "flyer"  n ( f l ai1 @ r ) ((pos "K6%" ) ))
( "flyers"  n ( f l ai1 @ z ) ((pos "Kj%" ) ))
( "flying"  v ( f l ai1 i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "flying-bomb"  n ( f l ai1 i ng - b o m ) ((pos "K6%" ) ))
( "flying-bombs"  n ( f l ai1 i ng - b o m z ) ((pos "Kj%" ) ))
( "flying-fish"  n ( f l ai1 i ng - f i sh ) ((pos "K9%" ) ))
( "flying-fishes"  n ( f l ai1 i ng - f i sh i z ) ((pos "Kj%" ) ))
( "flying-fox"  n ( f l ai1 i ng - f o k s ) ((pos "K7$" ) ))
( "flying-foxes"  n ( f l ai1 i ng - f o k s i z ) ((pos "Kj$" ) ))
( "flying-squad"  n ( f l ai1 i ng - s k w o d ) ((pos "K6%" ) ))
( "flying-squads"  n ( f l ai1 i ng - s k w o d z ) ((pos "Kj$" ) ))
( "flyleaf"  n ( f l ai1 l ii f ) ((pos "Ki%" ) ))
( "flyleaves"  n ( f l ai1 l ii v z ) ((pos "Kj%" ) ))
( "flyover"  n ( f l ai1 ou v @ r ) ((pos "K6%" ) ))
( "flyovers"  n ( f l ai1 ou v @ z ) ((pos "Kj%" ) ))
( "flypaper"  n ( f l ai1 p ei p @ r ) ((pos "K6%" ) ))
( "flypapers"  n ( f l ai1 p ei p @ z ) ((pos "Kj%" ) ))
( "flypast"  n ( f l ai1 p aa s t ) ((pos "K6%" ) ))
( "flypasts"  n ( f l ai1 p aa s t s ) ((pos "Kj%" ) ))
( "flytrap"  n ( f l ai1 t r a p ) ((pos "K6%" ) ))
( "flytraps"  n ( f l ai1 t r a p s ) ((pos "Kj%" ) ))
( "flyweight"  n ( f l ai1 w ei t ) ((pos "K6%" ) ))
( "flyweights"  n ( f l ai1 w ei t s ) ((pos "Kj%" ) ))
( "flywheel"  n ( f l ai1 w ii l ) ((pos "K6%" ) ))
( "flywheels"  n ( f l ai1 w ii l z ) ((pos "Kj%" ) ))
( "fo'c'sle"  n ( f ou1 k s l ) ((pos "K6$" ) ))
( "fo'c'sles"  n ( f ou1 k s l z ) ((pos "Kj$" ) ))
( "foal"  v ( f ou1 l ) ((pos "I0$" "K6%" ) (subcat )))
( "foaled"  v ( f ou1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "foaling"  v ( f ou1 l i ng ) ((pos "Ib$" ) (subcat )))
( "foals"  v ( f ou1 l z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "foam"  v ( f ou1 m ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "foam-rubber"  n ( f ou1 m - r uh1 b @ r ) ((pos "L@%" ) ))
( "foamed"  v ( f ou1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "foamier"  j ( f ou1 m i@ r ) ((pos "Or$" ) ))
( "foamiest"  j ( f ou1 m i i s t ) ((pos "Os$" ) ))
( "foaming"  v ( f ou1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "foams"  v ( f ou1 m z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "foamy"  j ( f ou1 m ii ) ((pos "OD%" ) ))
( "fob"  v ( f o1 b ) ((pos "H4%" ) (subcat )))
( "fobbed"  v ( f o1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fobbing"  v ( f o1 b i ng ) ((pos "Hb%" ) (subcat )))
( "fobs"  v ( f o1 b z ) ((pos "Ha%" ) (subcat )))
( "focal"  j ( f ou1 k l ) ((pos "OA%" ) ))
( "foci"  n ( f ou1 s ai ) ((pos "Kj$" ) ))
( "focus"  v ( f ou1 k @ s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" )))
( "focused"  v ( f ou1 k @ s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "focuses"  v ( f ou1 k @ s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "focusing"  v ( f ou1 k @ s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "focussed"  v ( f ou1 k @ s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "focussing"  v ( f ou1 k @ s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "fodder"  n ( f o1 d @ r ) ((pos "L@%" ) ))
( "foe"  n ( f ou1 ) ((pos "K6%" ) ))
( "foes"  n ( f ou1 z ) ((pos "Kj%" ) ))
( "foetal"  j ( f ii1 t l ) ((pos "OA%" ) ))
( "foetus"  n ( f ii1 t @ s ) ((pos "K7%" ) ))
( "foetuses"  n ( f ii1 t @ s i z ) ((pos "Kj%" ) ))
( "fog"  v ( f o1 g ) ((pos "H4%" "M6%" ) ))
( "fogbank"  n ( f o1 g b a ng k ) ((pos "K6$" ) ))
( "fogbanks"  n ( f o1 g b a ng k s ) ((pos "Kj$" ) ))
( "fogbound"  j ( f o1 g b au n d ) ((pos "OA%" ) ))
( "fogey"  n ( f ou1 g ii ) ((pos "K6%" ) ))
( "fogeys"  n ( f ou1 g i z ) ((pos "Kj%" ) ))
( "fogged"  v ( f o1 g d ) ((pos "Hc%" "Hd%" ) ))
( "foggier"  j ( f o1 g i@ r ) ((pos "Or%" ) ))
( "foggiest"  j ( f o1 g i i s t ) ((pos "Os%" ) ))
( "fogging"  v ( f o1 g i ng ) ((pos "Hb%" ) ))
( "foggy"  j ( f o1 g ii ) ((pos "OD%" ) ))
( "foghorn"  n ( f o1 g h oo n ) ((pos "K6%" ) ))
( "foghorns"  n ( f o1 g h oo n z ) ((pos "Kj%" ) ))
( "foglamp"  n ( f o1 g l a m p ) ((pos "K6%" ) ))
( "foglamps"  n ( f o1 g l a m p s ) ((pos "Kj%" ) ))
( "fogs"  v ( f o1 g z ) ((pos "Ha%" "Mj%" ) ))
( "fogsignal"  n ( f o1 g s i g n @ l ) ((pos "K6%" ) ))
( "fogsignals"  n ( f o1 g s i g n @ l z ) ((pos "Kj%" ) ))
( "foible"  n ( f oi1 b l ) ((pos "K6$" ) ))
( "foibles"  n ( f oi1 b l z ) ((pos "Kj%" ) ))
( "foil"  v ( f oi1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "foiled"  v ( f oi1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "foiling"  v ( f oi1 l i ng ) ((pos "Hb%" ) (subcat )))
( "foils"  v ( f oi1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "foist"  v ( f oi1 s t ) ((pos "H0%" ) (subcat "14" )))
( "foisted"  v ( f oi1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "foisting"  v ( f oi1 s t i ng ) ((pos "Hb%" ) (subcat "14" )))
( "foists"  v ( f oi1 s t s ) ((pos "Ha%" ) (subcat "14" )))
( "fold"  v ( f ou1 l d ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "folded"  v ( f ou1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "folder"  n ( f ou1 l d @ r ) ((pos "K6%" ) ))
( "folders"  n ( f ou1 l d @ z ) ((pos "Kj%" ) ))
( "folding"  v ( f ou1 l d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "folds"  v ( f ou1 l d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "foliage"  n ( f ou1 l i i jh ) ((pos "L@%" ) ))
( "folio"  n ( f ou1 l i@ @ ) ((pos "K6%" ) ))
( "folios"  n ( f ou1 l i@ u z ) ((pos "Kj%" ) ))
( "folk"  n ( f ou1 k ) ((pos "K6%" ) ))
( "folk-dance"  n ( f ou1 k - d aa n s ) ((pos "K6%" ) ))
( "folk-dances"  n ( f ou1 k - d aa n s i z ) ((pos "Kj%" ) ))
( "folklore"  n ( f ou1 k l oo r ) ((pos "L@%" ) ))
( "folks"  n ( f ou1 k s ) ((pos "Kj%" ) ))
( "folksong"  n ( f ou1 k s o ng ) ((pos "K6%" ) ))
( "folksongs"  n ( f ou1 k s o ng z ) ((pos "Kj%" ) ))
( "folksy"  j ( f ou1 k s ii ) ((pos "OA%" ) ))
( "folktale"  n ( f ou1 k t ei l ) ((pos "K6%" ) ))
( "folktales"  n ( f ou1 k t ei l z ) ((pos "Kj%" ) ))
( "follies"  n ( f o1 l i z ) ((pos "Mj%" ) ))
( "follow"  v ( f o1 l ou ) ((pos "J0%" ) (subcat "2A" "2B" "2C" "6A" )))
( "follow-on"  n ( f o2 l ou - o1 n ) ((pos "K6%" ) ))
( "follow-ons"  n ( f o2 l ou - o1 n z ) ((pos "Kj$" ) ))
( "follow-through"  n ( f o2 l ou - th r uu1 ) ((pos "K6%" ) ))
( "follow-throughs"  n ( f o2 l ou - th r uu1 z ) ((pos "Kj$" ) ))
( "follow-up"  n ( f o1 l ou - uh p ) ((pos "K6%" ) ))
( "follow-ups"  n ( f o1 l ou - uh p s ) ((pos "Kj$" ) ))
( "followed"  v ( f o1 l ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" )))
( "follower"  n ( f o1 l ou @ r ) ((pos "K6%" ) ))
( "followers"  n ( f o1 l ou @ z ) ((pos "Kj%" ) ))
( "following"  v ( f o1 l ou i ng ) ((pos "Jb*" "K6%" "OA*" ) (subcat "2A" "2B" "2C" "6A" )))
( "followings"  n ( f o1 l ou i ng z ) ((pos "Kj%" ) ))
( "follows"  v ( f o1 l ou z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" )))
( "folly"  n ( f o1 l ii ) ((pos "M8%" ) ))
( "foment"  v ( f ou1 m e1 n t ) ((pos "H0$" ) (subcat )))
( "fomentation"  n ( f ou2 m e n t ei1 sh @ n ) ((pos "M6$" ) ))
( "fomentations"  n ( f ou2 m e n t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "fomented"  v ( f ou1 m e1 n t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "fomenting"  v ( f ou1 m e1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "foments"  v ( f ou1 m e1 n t s ) ((pos "Ha$" ) (subcat )))
( "fond"  j ( f o1 n d ) ((pos "OC%" ) ))
( "fondant"  n ( f o1 n d @ n t ) ((pos "K6%" ) ))
( "fondants"  n ( f o1 n d @ n t s ) ((pos "Kj%" ) ))
( "fonder"  j ( f o1 n d @ r ) ((pos "Or%" ) ))
( "fondest"  j ( f o1 n d i s t ) ((pos "Os%" ) ))
( "fondle"  v ( f o1 n d l ) ((pos "H2%" ) (subcat )))
( "fondled"  v ( f o1 n d l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fondles"  v ( f o1 n d l z ) ((pos "Ha%" ) (subcat )))
( "fondling"  v ( f o1 n d l i ng ) ((pos "Hb%" ) (subcat )))
( "fondly"  a ( f o1 n d l ii ) ((pos "Pu%" ) ))
( "fondness"  n ( f o1 n d n @ s ) ((pos "L@%" ) ))
( "font"  n ( f o1 n t ) ((pos "K6%" ) ))
( "fonts"  n ( f o1 n t s ) ((pos "Kj%" ) ))
( "food"  n ( f uu1 d ) ((pos "M6*" ) ))
( "foodless"  j ( f uu1 d l @ s ) ((pos "OA$" ) ))
( "foods"  n ( f uu1 d z ) ((pos "Mj%" ) ))
( "foodstuff"  n ( f uu1 d s t uh f ) ((pos "K6%" ) ))
( "foodstuffs"  n ( f uu1 d s t uh f s ) ((pos "Kj%" ) ))
( "fool"  v ( f uu1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "fooled"  v ( f uu1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "foolery"  n ( f uu1 l @ r ii ) ((pos "L@%" ) ))
( "foolhardiness"  n ( f uu1 l h aa d i n @ s ) ((pos "L@%" ) ))
( "foolhardy"  j ( f uu1 l h aa d ii ) ((pos "OA%" ) ))
( "fooling"  v ( f uu1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "foolish"  j ( f uu1 l i sh ) ((pos "OA%" ) ))
( "foolishly"  a ( f uu1 l i sh l ii ) ((pos "Pu%" ) ))
( "foolishness"  n ( f uu1 l i sh n @ s ) ((pos "L@%" ) ))
( "foolproof"  j ( f uu1 l p r uu f ) ((pos "OA%" ) ))
( "fools"  v ( f uu1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "foolscap"  n ( f uu1 l s k a p ) ((pos "L@%" ) ))
( "foot"  v ( f u1 t ) ((pos "J0%" "Ki%" ) (subcat )))
( "foot-and-mouth"  n ( f u2 t - n - m au1 th ) ((pos "L@%" "OA%" ) ))
( "foot-bath"  n ( f u1 t - b aa th ) ((pos "K6%" ) ))
( "foot-baths"  n ( f u1 t - b aa dh z ) ((pos "Kj%" ) ))
( "foot-pound"  n ( f u1 t - p au n d ) ((pos "K6$" ) ))
( "foot-pounds"  n ( f u1 t - p au n d z ) ((pos "Kj$" ) ))
( "foot-race"  n ( f u1 t - r ei s ) ((pos "K6$" ) ))
( "foot-races"  n ( f u1 t - r ei s i z ) ((pos "Kj$" ) ))
( "foot-rot"  n ( f u1 t - r o t ) ((pos "L@%" ) ))
( "footage"  n ( f u1 t i jh ) ((pos "L@%" ) ))
( "football"  n ( f u1 t b oo l ) ((pos "M6%" ) ))
( "footballer"  n ( f u1 t b oo l @ r ) ((pos "K6%" ) ))
( "footballers"  n ( f u1 t b oo l @ z ) ((pos "Kj%" ) ))
( "footballs"  n ( f u1 t b oo l z ) ((pos "Mj%" ) ))
( "footboard"  n ( f u1 t b oo d ) ((pos "K6%" ) ))
( "footboards"  n ( f u1 t b oo d z ) ((pos "Kj%" ) ))
( "footbridge"  n ( f u1 t b r i jh ) ((pos "K6%" ) ))
( "footbridges"  n ( f u1 t b r i jh i z ) ((pos "Kj%" ) ))
( "footed"  v ( f u1 t i d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat )))
( "footer"  n ( f u1 t @ r ) ((pos "K6%" ) ))
( "footers"  n ( f u1 t @ z ) ((pos "Kj%" ) ))
( "footfall"  n ( f u1 t f oo l ) ((pos "K6%" ) ))
( "footfalls"  n ( f u1 t f oo l z ) ((pos "Kj%" ) ))
( "footfault"  n ( f u1 t f oo l t ) ((pos "K6%" ) ))
( "footfaults"  n ( f u1 t f oo l t s ) ((pos "Kj%" ) ))
( "foothills"  n ( f u1 t h i l z ) ((pos "Kj%" ) ))
( "foothold"  n ( f u1 t h ou l d ) ((pos "K6%" ) ))
( "footholds"  n ( f u1 t h ou l d z ) ((pos "Kj%" ) ))
( "footing"  v ( f u1 t i ng ) ((pos "Jb%" "Ki%" ) (subcat )))
( "footle"  v ( f uu1 t l ) ((pos "J2%" ) ))
( "footled"  v ( f uu1 t l d ) ((pos "Jc%" "Jd%" ) ))
( "footles"  v ( f uu1 t l z ) ((pos "Ja%" ) ))
( "footlights"  n ( f u1 t l ai t s ) ((pos "Kj%" ) ))
( "footling"  v ( f uu1 t l i ng ) ((pos "Jb%" "OA%" ) ))
( "footloose"  j ( f u1 t l uu s ) ((pos "OA%" ) ))
( "footman"  n ( f u1 t m @ n ) ((pos "Ki%" ) ))
( "footmark"  n ( f u1 t m aa k ) ((pos "K6%" ) ))
( "footmarks"  n ( f u1 t m aa k s ) ((pos "Kj%" ) ))
( "footmen"  n ( f u1 t m @ n ) ((pos "Kj%" ) ))
( "footnote"  n ( f u1 t n ou t ) ((pos "K6%" ) ))
( "footnotes"  n ( f u1 t n ou t s ) ((pos "Kj%" ) ))
( "footpath"  n ( f u1 t p aa th ) ((pos "K6%" ) ))
( "footpaths"  n ( f u1 t p aa dh z ) ((pos "Kj%" ) ))
( "footplate"  n ( f u1 t p l ei t ) ((pos "K6%" ) ))
( "footplates"  n ( f u1 t p l ei t s ) ((pos "Kj%" ) ))
( "footprint"  n ( f u1 t p r i n t ) ((pos "K6%" ) ))
( "footprints"  n ( f u1 t p r i n t s ) ((pos "Kj%" ) ))
( "foots"  v ( f u1 t s ) ((pos "Ja$" ) (subcat )))
( "footslog"  v ( f u1 t s l o g ) ((pos "I4%" ) ))
( "footslogged"  v ( f u1 t s l o g d ) ((pos "Ic%" "Id%" ) ))
( "footslogger"  n ( f u1 t s l o g @ r ) ((pos "K6%" ) ))
( "footsloggers"  n ( f u1 t s l o g @ z ) ((pos "Kj%" ) ))
( "footslogging"  v ( f u1 t s l o g i ng ) ((pos "Ib%" ) ))
( "footslogs"  v ( f u1 t s l o g z ) ((pos "Ia%" ) ))
( "footsore"  j ( f u1 t s oo r ) ((pos "OA%" ) ))
( "footstep"  n ( f u1 t s t e p ) ((pos "K6%" ) ))
( "footsteps"  n ( f u1 t s t e p s ) ((pos "Kj%" ) ))
( "footstool"  n ( f u1 t s t uu l ) ((pos "K6%" ) ))
( "footstools"  n ( f u1 t s t uu l z ) ((pos "Kj%" ) ))
( "footsure"  j ( f u1 ch u@ r ) ((pos "OA%" ) ))
( "footwear"  n ( f u1 t w e@ r ) ((pos "L@%" ) ))
( "footwork"  n ( f u1 t w @@ k ) ((pos "L@%" ) ))
( "fop"  n ( f o1 p ) ((pos "K6%" ) ))
( "foppish"  j ( f o1 p i sh ) ((pos "OA%" ) ))
( "fops"  n ( f o1 p s ) ((pos "Kj%" ) ))
( "for"  in ( f oo r ) ((pos "T-*" "V-*" ) ))
( "forage"  v ( f o1 r i jh ) ((pos "I2%" "L@%" ) (subcat "2A" )))
( "foraged"  v ( f o1 r i jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "forages"  v ( f o1 r i jh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "foraging"  v ( f o1 r i jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "forasmuch as"  cc ( f oo2 r @ z m uh1 ch - @ z ) ((pos "V-%" ) ))
( "foray"  v ( f o1 r ei ) ((pos "I0$" "K6%" ) (subcat )))
( "forayed"  v ( f o1 r ei d ) ((pos "Ic$" "Id$" ) (subcat )))
( "foraying"  v ( f o1 r ei i ng ) ((pos "Ib$" ) (subcat )))
( "forays"  v ( f o1 r ei z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "forbad"  v ( f @1 b a1 d ) ((pos "Hc$" ) (subcat "6A" "12C" )))
( "forbade"  v ( f @1 b a1 d ) ((pos "Hc%" ) (subcat "6A" "12C" )))
( "forbear"  n ( f oo1 b e@ r ) ((pos "K6%" ) ))
( "forbear"  v ( f oo1 b e@1 r ) ((pos "J5%" ) (subcat "2A" "3A" "6C" )))
( "forbearance"  n ( f oo1 b e@1 r @ n s ) ((pos "L@%" ) ))
( "forbearing"  v ( f oo1 b e@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6C" )))
( "forbears"  n ( f oo1 b e@ z ) ((pos "Kj%" ) ))
( "forbears"  v ( f oo1 b e@1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6C" )))
( "forbid"  v ( f @1 b i1 d ) ((pos "H5%" ) (subcat "6A" "12C" )))
( "forbidden"  v ( f @1 b i1 d n ) ((pos "Hd%" ) (subcat "6A" "12C" )))
( "forbidding"  v ( f @1 b i1 d i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "12C" )))
( "forbiddingly"  a ( f @1 b i1 d i ng l ii ) ((pos "Pu%" ) ))
( "forbids"  v ( f @1 b i1 d z ) ((pos "Ha%" ) (subcat "6A" "12C" )))
( "forbore"  v ( f oo1 b oo1 r ) ((pos "Jc%" ) (subcat "2A" "3A" "6C" )))
( "forborne"  v ( f oo1 b oo1 n ) ((pos "Jd%" ) (subcat "2A" "3A" "6C" )))
( "force"  v ( f oo1 s ) ((pos "H2%" "M6%" ) (subcat "6A" "15A" "15B" "17" )))
( "force majeure"  n ( f oo2 s - m a zh @@1 r ) ((pos "L@%" ) ))
( "force-fed"  v ( f oo1 s - f e d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "force-feed"  v ( f oo1 s - f ii d ) ((pos "H5%" ) (subcat )))
( "force-feeding"  v ( f oo1 s - f ii d i ng ) ((pos "Hb%" ) (subcat )))
( "force-feeds"  v ( f oo1 s - f ii d z ) ((pos "Ha%" ) (subcat )))
( "force-land"  v ( f oo1 s - l a1 n d ) ((pos "J0%" ) ))
( "force-landed"  v ( f oo1 s - l a1 n d i d ) ((pos "Jc%" "Jd%" ) ))
( "force-landing"  v ( f oo1 s - l a1 n d i ng ) ((pos "Jb%" ) ))
( "force-lands"  v ( f oo1 s - l a1 n d z ) ((pos "Ja%" ) ))
( "forced"  v ( f oo1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" "15B" "17" )))
( "forceful"  j ( f oo1 s f @ l ) ((pos "OA%" ) ))
( "forcefully"  a ( f oo1 s f @ l ii ) ((pos "Pu%" ) ))
( "forcefulness"  n ( f oo1 s f @ l n @ s ) ((pos "L@%" ) ))
( "forcemeat"  n ( f oo1 s m ii t ) ((pos "L@$" ) ))
( "forceps"  n ( f oo1 s e p s ) ((pos "Kj%" ) ))
( "forces"  v ( f oo1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" "15B" "17" )))
( "forcible"  j ( f oo1 s @ b l ) ((pos "OA%" ) ))
( "forcibly"  a ( f oo1 s @ b l ii ) ((pos "Pu%" ) ))
( "forcing"  v ( f oo1 s i ng ) ((pos "Hb%" ) (subcat "6A" "15A" "15B" "17" )))
( "ford"  v ( f oo1 d ) ((pos "H0%" "K6%" ) (subcat )))
( "fordable"  j ( f oo1 d @ b l ) ((pos "OA%" ) ))
( "forded"  v ( f oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fording"  v ( f oo1 d i ng ) ((pos "Hb%" ) (subcat )))
( "fords"  v ( f oo1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fore"  n ( f oo1 r ) ((pos "L@%" "OA%" "Pu%" "W-%" ) ))
( "forearm"  n ( f oo1 r aa m ) ((pos "K6$" ) ))
( "forearm"  v ( f oo2 r aa1 m ) ((pos "H0%" ) (subcat )))
( "forearmed"  v ( f oo2 r aa1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "forearming"  v ( f oo2 r aa1 m i ng ) ((pos "Hb$" ) (subcat )))
( "forearms"  n ( f oo1 r aa m z ) ((pos "Kj%" ) ))
( "forearms"  v ( f oo2 r aa1 m z ) ((pos "Ha$" ) (subcat )))
( "forebode"  v ( f oo1 b ou1 d ) ((pos "H2$" ) (subcat "6A" )))
( "foreboded"  v ( f oo1 b ou1 d i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "forebodes"  v ( f oo1 b ou1 d z ) ((pos "Ha$" ) (subcat "6A" )))
( "foreboding"  v ( f oo1 b ou1 d i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "forebodings"  n ( f oo1 b ou1 d i ng z ) ((pos "Mj%" ) ))
( "forecast"  v ( f oo1 k aa s t ) ((pos "H0%" "Hc%" "Hd%" "K6%" ) (subcat )))
( "forecasted"  v ( f oo1 k aa s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "forecaster"  n ( f oo1 k aa s t @ r ) ((pos "K6%" ) ))
( "forecasters"  n ( f oo1 k aa s t @ z ) ((pos "Kj%" ) ))
( "forecasting"  v ( f oo1 k aa s t i ng ) ((pos "Hb%" ) (subcat )))
( "forecastle"  n ( f ou1 k s l ) ((pos "K6$" ) ))
( "forecastles"  n ( f ou1 k s l z ) ((pos "Kj$" ) ))
( "forecasts"  v ( f oo1 k aa s t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "foreclose"  v ( f oo1 k l ou1 z ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "foreclosed"  v ( f oo1 k l ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "forecloses"  v ( f oo1 k l ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "foreclosing"  v ( f oo1 k l ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "foreclosure"  n ( f oo1 k l ou1 zh @ r ) ((pos "M6%" ) ))
( "foreclosures"  n ( f oo1 k l ou1 zh @ z ) ((pos "Mj%" ) ))
( "forecourt"  n ( f oo1 k oo t ) ((pos "K6%" ) ))
( "forecourts"  n ( f oo1 k oo t s ) ((pos "Kj%" ) ))
( "foredoom"  v ( f oo1 d uu1 m ) ((pos "H0$" ) (subcat "6A" )))
( "foredoomed"  v ( f oo1 d uu1 m d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "foredooming"  v ( f oo1 d uu1 m i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "foredooms"  v ( f oo1 d uu1 m z ) ((pos "Ha$" ) (subcat "6A" )))
( "forefather"  n ( f oo1 f aa dh @ r ) ((pos "K6%" ) ))
( "forefathers"  n ( f oo1 f aa dh @ z ) ((pos "Kj%" ) ))
( "forefeet"  n ( f oo1 f ii t ) ((pos "Kj%" ) ))
( "forefinger"  n ( f oo1 f i ng g @ r ) ((pos "K6%" ) ))
( "forefingers"  n ( f oo1 f i ng g @ z ) ((pos "Kj%" ) ))
( "forefoot"  n ( f oo1 f u t ) ((pos "Ki$" ) ))
( "forefront"  n ( f oo1 f r uh n t ) ((pos "K6%" ) ))
( "forefronts"  n ( f oo1 f r uh n t s ) ((pos "Kj$" ) ))
( "foregather"  v ( f oo1 g a1 dh @ r ) ((pos "I0%" ) (subcat "2A" )))
( "foregathered"  v ( f oo1 g a1 dh @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "foregathering"  v ( f oo1 g a1 dh @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "foregathers"  v ( f oo1 g a1 dh @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "forego"  v ( f oo1 g ou1 ) ((pos "J5%" ) ))
( "foregoes"  v ( f oo1 g ou1 z ) ((pos "Ja%" ) ))
( "foregoing"  v ( f oo1 g ou1 i ng ) ((pos "Jb%" "OA%" ) ))
( "foregone"  j ( f oo1 g o n ) ((pos "OA%" ) ))
( "foregone"  v ( f oo1 g o1 n ) ((pos "Jd%" ) ))
( "foreground"  n ( f oo1 g r au n d ) ((pos "K6%" ) ))
( "foregrounds"  n ( f oo1 g r au n d z ) ((pos "Kj$" ) ))
( "forehand"  j ( f oo1 h a n d ) ((pos "OA%" ) ))
( "forehead"  n ( f o1 r i d ) ((pos "K6%" ) ))
( "foreheads"  n ( f o1 r i d z ) ((pos "Kj%" ) ))
( "foreign"  j ( f o1 r @ n ) ((pos "OA%" ) ))
( "foreigner"  n ( f o1 r @ n @ r ) ((pos "K6%" ) ))
( "foreigners"  n ( f o1 r @ n @ z ) ((pos "Kj%" ) ))
( "foreknew"  v ( f oo1 n y uu1 ) ((pos "Jc$" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "foreknow"  v ( f oo1 n ou1 ) ((pos "J5$" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "foreknowing"  v ( f oo1 n ou1 i ng ) ((pos "Jb$" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "foreknowledge"  n ( f oo2 n o1 l i jh ) ((pos "L@%" ) ))
( "foreknown"  v ( f oo1 n ou1 n ) ((pos "Jd%" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "foreknows"  v ( f oo1 n ou1 z ) ((pos "Ja$" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "foreland"  n ( f oo1 l @ n d ) ((pos "K6%" ) ))
( "forelands"  n ( f oo1 l @ n d z ) ((pos "Kj$" ) ))
( "foreleg"  n ( f oo1 l e g ) ((pos "K6%" ) ))
( "forelegs"  n ( f oo1 l e g z ) ((pos "Kj%" ) ))
( "forelock"  n ( f oo1 l o k ) ((pos "K6%" ) ))
( "forelocks"  n ( f oo1 l o k s ) ((pos "Kj$" ) ))
( "foreman"  n ( f oo1 m @ n ) ((pos "Ki%" ) ))
( "foremast"  n ( f oo1 m aa s t ) ((pos "K6%" ) ))
( "foremasts"  n ( f oo1 m aa s t s ) ((pos "Kj%" ) ))
( "foremen"  n ( f oo1 m @ n ) ((pos "Kj%" ) ))
( "foremost"  j ( f oo1 m ou s t ) ((pos "OA%" "Pu%" ) ))
( "forename"  n ( f oo1 n ei m ) ((pos "K6%" ) ))
( "forenames"  n ( f oo1 n ei m z ) ((pos "Kj%" ) ))
( "forenoon"  n ( f oo1 n uu n ) ((pos "K6%" ) ))
( "forenoons"  n ( f oo1 n uu n z ) ((pos "Kj%" ) ))
( "forensic"  j ( f @1 r e1 n s i k ) ((pos "OA%" ) ))
( "foreordain"  v ( f oo2 r oo d ei1 n ) ((pos "H0$" ) (subcat "6A" "14" )))
( "foreordained"  v ( f oo2 r oo d ei1 n d ) ((pos "Hc$" "Hd%" ) (subcat "6A" "14" )))
( "foreordaining"  v ( f oo2 r oo d ei1 n i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "foreordains"  v ( f oo2 r oo d ei1 n z ) ((pos "Ha$" ) (subcat "6A" "14" )))
( "forerunner"  n ( f oo1 r uh n @ r ) ((pos "K6%" ) ))
( "forerunners"  n ( f oo1 r uh n @ z ) ((pos "Kj%" ) ))
( "foresail"  n ( f oo1 s ei l ) ((pos "K6%" ) ))
( "foresails"  n ( f oo1 s ei l z ) ((pos "Kj%" ) ))
( "foresaw"  v ( f oo1 s oo1 ) ((pos "Hc%" ) (subcat "6A" "9" )))
( "foresee"  v ( f oo1 s ii1 ) ((pos "H5%" ) (subcat "6A" "9" )))
( "foreseeable"  j ( f oo1 s ii1 @ b l ) ((pos "OA%" ) ))
( "foreseeing"  v ( f oo1 s ii1 i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "foreseen"  v ( f oo1 s ii1 n ) ((pos "Hd%" ) (subcat "6A" "9" )))
( "foresees"  v ( f oo1 s ii1 z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "foreshadow"  v ( f oo1 sh a1 d ou ) ((pos "H0%" ) (subcat )))
( "foreshadowed"  v ( f oo1 sh a1 d ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "foreshadowing"  v ( f oo1 sh a1 d ou i ng ) ((pos "Hb%" ) (subcat )))
( "foreshadows"  v ( f oo1 sh a1 d ou z ) ((pos "Ha%" ) (subcat )))
( "foreshore"  n ( f oo1 sh oo r ) ((pos "K6%" ) ))
( "foreshores"  n ( f oo1 sh oo z ) ((pos "Kj$" ) ))
( "foreshorten"  v ( f oo1 sh oo1 t n ) ((pos "H0%" ) (subcat )))
( "foreshortened"  v ( f oo1 sh oo1 t n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "foreshortening"  v ( f oo1 sh oo1 t n i ng ) ((pos "Hb%" ) (subcat )))
( "foreshortens"  v ( f oo1 sh oo1 t n z ) ((pos "Ha%" ) (subcat )))
( "foresight"  n ( f oo1 s ai t ) ((pos "L@%" ) ))
( "foreskin"  n ( f oo1 s k i n ) ((pos "K6%" ) ))
( "foreskins"  n ( f oo1 s k i n z ) ((pos "Kj$" ) ))
( "forest"  n ( f o1 r i s t ) ((pos "M6%" ) ))
( "forestall"  v ( f oo1 s t oo1 l ) ((pos "H0%" ) (subcat )))
( "forestalled"  v ( f oo1 s t oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "forestalling"  v ( f oo1 s t oo1 l i ng ) ((pos "Hb%" ) (subcat )))
( "forestalls"  v ( f oo1 s t oo1 l z ) ((pos "Ha%" ) (subcat )))
( "forester"  n ( f o1 r i s t @ r ) ((pos "K6%" ) ))
( "foresters"  n ( f o1 r i s t @ z ) ((pos "Kj%" ) ))
( "forestry"  n ( f o1 r @ s t r ii ) ((pos "L@%" ) ))
( "forests"  n ( f o1 r i s t s ) ((pos "Mj%" ) ))
( "foreswear"  v ( f oo1 s w e@1 r ) ((pos "H5%" ) (subcat )))
( "foreswearing"  v ( f oo1 s w e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "foreswears"  v ( f oo1 s w e@1 z ) ((pos "Ha%" ) (subcat )))
( "foreswore"  v ( f oo1 s w oo1 r ) ((pos "Hc%" ) (subcat )))
( "foresworn"  v ( f oo1 s w oo1 n ) ((pos "Hd%" ) (subcat )))
( "foretaste"  n ( f oo1 t ei s t ) ((pos "K6%" ) ))
( "foretastes"  n ( f oo1 t ei s t s ) ((pos "Kj%" ) ))
( "foretell"  v ( f oo1 t e1 l ) ((pos "H5%" ) (subcat "6A" "9" "10" "12A" )))
( "foretelling"  v ( f oo1 t e1 l i ng ) ((pos "Hb%" ) (subcat "6A" "9" "10" "12A" )))
( "foretells"  v ( f oo1 t e1 l z ) ((pos "Ha%" ) (subcat "6A" "9" "10" "12A" )))
( "forethought"  n ( f oo1 th oo t ) ((pos "L@%" ) ))
( "foretold"  v ( f oo1 t ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "10" "12A" )))
( "foretop"  n ( f oo1 t o p ) ((pos "K6$" ) ))
( "foretops"  n ( f oo1 t o p s ) ((pos "Kj$" ) ))
( "forever"  a ( f @1 r e1 v @ r ) ((pos "Pu%" ) ))
( "forewarn"  v ( f oo1 w oo1 n ) ((pos "H0%" ) (subcat )))
( "forewarned"  v ( f oo1 w oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "forewarning"  v ( f oo1 w oo1 n i ng ) ((pos "Hb%" ) (subcat )))
( "forewarns"  v ( f oo1 w oo1 n z ) ((pos "Ha%" ) (subcat )))
( "forewent"  v ( f oo1 w e1 n t ) ((pos "Jc$" ) ))
( "forewoman"  n ( f oo1 w u m @ n ) ((pos "Ki$" ) ))
( "forewomen"  n ( f oo1 w i m i n ) ((pos "Kj$" ) ))
( "foreword"  n ( f oo1 w @@ d ) ((pos "K6%" ) ))
( "forewords"  n ( f oo1 w @@ d z ) ((pos "Kj%" ) ))
( "forfeit"  v ( f oo1 f i t ) ((pos "H0%" "K6%" ) (subcat )))
( "forfeited"  v ( f oo1 f i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "forfeiting"  v ( f oo1 f i t i ng ) ((pos "Hb%" ) (subcat )))
( "forfeits"  v ( f oo1 f i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "forfeiture"  n ( f oo1 f i ch @ r ) ((pos "L@%" ) ))
( "forgather"  v ( f oo1 g a1 dh @ r ) ((pos "I0%" ) (subcat "2A" )))
( "forgathered"  v ( f oo1 g a1 dh @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "forgathering"  v ( f oo1 g a1 dh @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "forgathers"  v ( f oo1 g a1 dh @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "forgave"  v ( f @1 g ei1 v ) ((pos "Jc%" ) (subcat "2A" "6A" "12C" )))
( "forge"  v ( f oo1 jh ) ((pos "J2%" "K6%" ) (subcat "2C" )))
( "forged"  v ( f oo1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "forger"  n ( f oo1 jh @ r ) ((pos "K6%" ) ))
( "forgeries"  n ( f oo1 jh @ r i z ) ((pos "Mj%" ) ))
( "forgers"  n ( f oo1 jh @ z ) ((pos "Kj%" ) ))
( "forgery"  n ( f oo1 jh @ r ii ) ((pos "M8%" ) ))
( "forges"  v ( f oo1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "forget"  v ( f @1 g e1 t ) ((pos "J5%" ) (subcat "2A" "3A" "6A" "6C" "6D" "7A" "8" "9" )))
( "forget-me-not"  n ( f @1 g e1 t - m i - n o t ) ((pos "K6%" ) ))
( "forget-me-nots"  n ( f @1 g e1 t - m i - n o t s ) ((pos "Kj%" ) ))
( "forgetful"  j ( f @1 g e1 t f @ l ) ((pos "OA%" ) ))
( "forgetfully"  a ( f @1 g e1 t f @ l ii ) ((pos "Pu%" ) ))
( "forgetfulness"  n ( f @1 g e1 t f @ l n @ s ) ((pos "L@%" ) ))
( "forgets"  v ( f @1 g e1 t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "6C" "6D" "7A" "8" "9" )))
( "forgetting"  v ( f @1 g e1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "6C" "6D" "7A" "8" "9" )))
( "forging"  v ( f oo1 jh i ng ) ((pos "Jb$" "K6%" ) (subcat "2C" )))
( "forgings"  n ( f oo1 jh i ng z ) ((pos "Kj$" ) ))
( "forgivable"  j ( f @1 g i1 v @ b l ) ((pos "OA%" ) ))
( "forgivably"  a ( f @1 g i1 v @ b l ii ) ((pos "Pu%" ) ))
( "forgive"  v ( f @1 g i1 v ) ((pos "J5%" ) (subcat "2A" "6A" "12C" )))
( "forgiven"  v ( f @1 g i1 v n ) ((pos "Jd%" ) (subcat "2A" "6A" "12C" )))
( "forgiveness"  n ( f @1 g i1 v n @ s ) ((pos "L@%" ) ))
( "forgives"  v ( f @1 g i1 v z ) ((pos "Ja%" ) (subcat "2A" "6A" "12C" )))
( "forgiving"  v ( f @1 g i1 v i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "12C" )))
( "forgivingly"  a ( f @1 g i1 v i ng l ii ) ((pos "Pu%" ) ))
( "forgo"  v ( f oo1 g ou1 ) ((pos "H5%" ) ))
( "forgoes"  v ( f oo1 g ou1 z ) ((pos "Ja%" ) ))
( "forgoing"  v ( f oo1 g ou1 i ng ) ((pos "Jb%" ) ))
( "forgone"  v ( f oo1 g o1 n ) ((pos "Jd%" ) ))
( "forgot"  v ( f @1 g o1 t ) ((pos "Jc%" ) (subcat "2A" "3A" "6A" "6C" "6D" "7A" "8" "9" )))
( "forgotten"  v ( f @1 g o1 t n ) ((pos "Jd%" ) (subcat "2A" "3A" "6A" "6C" "6D" "7A" "8" "9" )))
( "fork"  v ( f oo1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "forked"  v ( f oo1 k t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "forking"  v ( f oo1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "forks"  v ( f oo1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "forlorn"  j ( f @1 l oo1 n ) ((pos "OA%" ) ))
( "forlornly"  a ( f @1 l oo1 n l ii ) ((pos "Pu%" ) ))
( "forlornness"  n ( f @1 l oo1 n - n @ s ) ((pos "L@%" ) ))
( "form"  v ( f oo1 m ) ((pos "J0*" "M6*" ) (subcat "2A" "2C" "6A" "14" )))
( "formal"  j ( f oo1 m l ) ((pos "OA%" ) ))
( "formaldehyde"  n ( f oo1 m a1 l d i h ai d ) ((pos "L@$" ) ))
( "formalin"  n ( f oo1 m @ l i n ) ((pos "L@$" ) ))
( "formalism"  n ( f oo1 m @ l i z @ m ) ((pos "L@$" ) ))
( "formalities"  n ( f oo1 m a1 l i t i z ) ((pos "Mj%" ) ))
( "formality"  n ( f oo1 m a1 l i t ii ) ((pos "M8%" ) ))
( "formalize"  v ( f oo1 m @ l ai z ) ((pos "H2%" ) (subcat )))
( "formalized"  v ( f oo1 m @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "formalizes"  v ( f oo1 m @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "formalizing"  v ( f oo1 m @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "formally"  a ( f oo1 m @ l ii ) ((pos "Pu%" ) ))
( "format"  n ( f oo1 m a t ) ((pos "K6%" ) ))
( "formation"  n ( f oo1 m ei1 sh @ n ) ((pos "M6%" ) ))
( "formations"  n ( f oo1 m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "formative"  j ( f oo1 m @ t i v ) ((pos "OA%" ) ))
( "formats"  n ( f oo1 m a t s ) ((pos "Kj%" ) ))
( "formed"  v ( f oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "former"  j ( f oo1 m @ r ) ((pos "OA%" ) ))
( "formerly"  a ( f oo1 m @ l ii ) ((pos "Pu%" ) ))
( "formic"  j ( f oo1 m i k ) ((pos "OA$" ) ))
( "formidable"  j ( f oo1 m i d @ b l ) ((pos "OA%" ) ))
( "formidably"  a ( f oo1 m i d @ b l ii ) ((pos "Pu%" ) ))
( "forming"  v ( f oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "formless"  j ( f oo1 m l @ s ) ((pos "OA%" ) ))
( "formlessly"  a ( f oo1 m l @ s l ii ) ((pos "Pu%" ) ))
( "forms"  v ( f oo1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" )))
( "formula"  n ( f oo1 m y u l @ ) ((pos "K6%" ) ))
( "formulae"  n ( f oo1 m y u l ii ) ((pos "Kj%" ) ))
( "formulas"  n ( f oo1 m y u l @ z ) ((pos "Kj%" ) ))
( "formulate"  v ( f oo1 m y u l ei t ) ((pos "H2%" ) (subcat )))
( "formulated"  v ( f oo1 m y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "formulates"  v ( f oo1 m y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "formulating"  v ( f oo1 m y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "formulation"  n ( f oo2 m y u l ei1 sh @ n ) ((pos "M6$" ) ))
( "formulations"  n ( f oo2 m y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "fornicate"  v ( f oo1 n i k ei t ) ((pos "I2%" ) (subcat )))
( "fornicated"  v ( f oo1 n i k ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "fornicates"  v ( f oo1 n i k ei t s ) ((pos "Ia%" ) (subcat )))
( "fornicating"  v ( f oo1 n i k ei t i ng ) ((pos "Ib%" ) (subcat )))
( "fornication"  n ( f oo2 n i k ei1 sh @ n ) ((pos "L@%" ) ))
( "forrader"  a ( f o1 r @ d @ r ) ((pos "Pu$" ) ))
( "forsake"  v ( f @1 s ei1 k ) ((pos "H5%" ) (subcat )))
( "forsaken"  v ( f @1 s ei1 k @ n ) ((pos "Hd%" ) (subcat )))
( "forsakes"  v ( f @1 s ei1 k s ) ((pos "Ha%" ) (subcat )))
( "forsaking"  v ( f @1 s ei1 k i ng ) ((pos "Hb%" ) (subcat )))
( "forsook"  v ( f @1 s u1 k ) ((pos "Hc%" ) (subcat )))
( "forsooth"  a ( f oo1 s uu1 th ) ((pos "Pu$" ) ))
( "forswear"  v ( f oo1 s w e@1 r ) ((pos "H5%" ) (subcat )))
( "forswearing"  v ( f oo1 s w e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "forswears"  v ( f oo1 s w e@1 z ) ((pos "Ha%" ) (subcat )))
( "forswore"  v ( f oo1 s w oo1 r ) ((pos "Hc%" ) (subcat )))
( "forsworn"  v ( f oo1 s w oo1 n ) ((pos "Hd%" ) (subcat )))
( "forsythia"  n ( f oo1 s ai1 th i@ ) ((pos "L@$" ) ))
( "fort"  n ( f oo1 t ) ((pos "K6%" ) ))
( "forte"  n ( f oo1 t ei ) ((pos "K6$" "OA$" "Pu$" ) ))
( "fortes"  n ( f oo1 t ei z ) ((pos "Kj$" ) ))
( "forth"  a ( f oo1 th ) ((pos "Pu%" ) ))
( "forthcoming"  j ( f oo2 th k uh1 m i ng ) ((pos "OA%" ) ))
( "forthright"  j ( f oo1 th r ai t ) ((pos "OA%" ) ))
( "forthwith"  a ( f oo2 th w i1 th ) ((pos "Pu%" ) ))
( "forties"  n ( f oo1 t i z ) ((pos "Kj%" ) ))
( "fortieth"  n ( f oo1 t i@ th ) ((pos "K6%" "OA%" ) ))
( "fortieths"  n ( f oo1 t i@ th s ) ((pos "Kj%" ) ))
( "fortification"  n ( f oo2 t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "fortifications"  n ( f oo2 t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fortified"  v ( f oo1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "fortifies"  v ( f oo1 t i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "fortify"  v ( f oo1 t i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "fortifying"  v ( f oo1 t i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "fortissimo"  j ( f oo1 t i1 s i m ou ) ((pos "OA$" "Pu%" ) ))
( "fortitude"  n ( f oo1 t i t y uu d ) ((pos "L@%" ) ))
( "fortnight"  n ( f oo1 t n ai t ) ((pos "K6%" ) ))
( "fortnightly"  j ( f oo1 t n ai t l ii ) ((pos "OA%" "Pu%" ) ))
( "fortnights"  n ( f oo1 t n ai t s ) ((pos "Kj%" ) ))
( "fortress"  n ( f oo1 t r i s ) ((pos "K7%" ) ))
( "fortresses"  n ( f oo1 t r i s i z ) ((pos "Kj%" ) ))
( "forts"  n ( f oo1 t s ) ((pos "Kj%" ) ))
( "fortuitous"  j ( f oo1 t y uu1 i t @ s ) ((pos "OA%" ) ))
( "fortuitously"  a ( f oo1 t y uu1 i t @ s l ii ) ((pos "Pu%" ) ))
( "fortunate"  j ( f oo1 ch @ n @ t ) ((pos "OA%" ) ))
( "fortunately"  a ( f oo1 ch @ n @ t l ii ) ((pos "Pu%" ) ))
( "fortune"  n ( f oo1 ch uu n ) ((pos "M6%" ) ))
( "fortunes"  n ( f oo1 ch uu n z ) ((pos "Mj%" ) ))
( "forty"  n ( f oo1 t ii ) ((pos "K8%" "OA%" ) ))
( "fortyish"  j ( f oo1 t i i sh ) ((pos "OA%" ) ))
( "forum"  n ( f oo1 r @ m ) ((pos "K6%" ) ))
( "forums"  n ( f oo1 r @ m z ) ((pos "Kj$" ) ))
( "forward"  v ( f oo1 w @ d ) ((pos "H0%" "K6%" "OA%" "Pu%" ) (subcat "6A" "12A" "13A" )))
( "forwarded"  v ( f oo1 w @ d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" )))
( "forwarding"  v ( f oo1 w @ d i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" )))
( "forwardness"  n ( f oo1 w @ d n @ s ) ((pos "L@%" ) ))
( "forwards"  v ( f oo1 w @ d z ) ((pos "Ha%" "Kj%" "Pu%" ) (subcat "6A" "12A" "13A" )))
( "forwent"  v ( f oo1 w e1 n t ) ((pos "Jc$" ) ))
( "fosse"  n ( f o1 s ) ((pos "K6$" ) ))
( "fosses"  n ( f o1 s i z ) ((pos "Kj$" ) ))
( "fossil"  n ( f o1 s l ) ((pos "K6%" ) ))
( "fossilization"  n ( f o2 s @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "fossilizations"  n ( f o2 s @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "fossilize"  v ( f o1 s @ l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "fossilized"  v ( f o1 s @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fossilizes"  v ( f o1 s @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "fossilizing"  v ( f o1 s @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "fossils"  n ( f o1 s l z ) ((pos "Kj%" ) ))
( "foster"  v ( f o1 s t @ r ) ((pos "H0%" ) (subcat )))
( "foster-brother"  n ( f o1 s t @ - b r uh dh @ r ) ((pos "K6%" ) ))
( "foster-brothers"  n ( f o1 s t @ - b r uh dh @ z ) ((pos "Kj%" ) ))
( "foster-child"  n ( f o1 s t @ - ch ai l d ) ((pos "Ki%" ) ))
( "foster-children"  n ( f o1 s t @ - ch i l d r @ n ) ((pos "Kj%" ) ))
( "foster-father"  n ( f o1 s t @ - f aa dh @ r ) ((pos "K6%" ) ))
( "foster-fathers"  n ( f o1 s t @ - f aa dh @ z ) ((pos "Kj%" ) ))
( "foster-mother"  n ( f o1 s t @ - m uh dh @ r ) ((pos "K6%" ) ))
( "foster-mothers"  n ( f o1 s t @ - m uh dh @ z ) ((pos "Kj%" ) ))
( "foster-parent"  n ( f o1 s t @ - p e@ r @ n t ) ((pos "K6%" ) ))
( "foster-parents"  n ( f o1 s t @ - p e@ r @ n t s ) ((pos "Kj%" ) ))
( "foster-sister"  n ( f o1 s t @ - s i s t @ r ) ((pos "K6%" ) ))
( "foster-sisters"  n ( f o1 s t @ - s i s t @ z ) ((pos "Kj%" ) ))
( "fostered"  v ( f o1 s t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fostering"  v ( f o1 s t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "fosters"  v ( f o1 s t @ z ) ((pos "Ha%" ) (subcat )))
( "fought"  v ( f oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "15A" )))
( "foul"  v ( f au1 l ) ((pos "J0%" "M6%" "OC%" ) (subcat "2A" "2C" )))
( "foul-mouthed"  j ( f au1 l - m au1 dh d ) ((pos "OA%" ) ))
( "foul-spoken"  j ( f au1 l - s p ou1 k @ n ) ((pos "OA%" ) ))
( "fouled"  v ( f au1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fouler"  j ( f au1 l @ r ) ((pos "Or%" ) ))
( "foulest"  j ( f au1 l i s t ) ((pos "Os%" ) ))
( "fouling"  v ( f au1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "foully"  a ( f au1 l ii ) ((pos "Pu%" ) ))
( "foulness"  n ( f au1 l n @ s ) ((pos "L@%" ) ))
( "fouls"  v ( f au1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "found"  v ( f au1 n d ) ((pos "H0%" "Hc*" "Hd*" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "14" "15A" "15B" "19B" "22" )))
( "foundation"  n ( f au1 n d ei1 sh @ n ) ((pos "M6%" ) ))
( "foundation-stone"  n ( f au1 n d ei1 sh @ n - s t ou n ) ((pos "K6%" ) ))
( "foundation-stones"  n ( f au1 n d ei1 sh @ n - s t ou n z ) ((pos "Kj%" ) ))
( "foundations"  n ( f au1 n d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "founded"  v ( f au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "14" "15A" "15B" "19B" "22" )))
( "founder"  v ( f au1 n d @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "foundered"  v ( f au1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "foundering"  v ( f au1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "founders"  v ( f au1 n d @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "founding"  v ( f au1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "14" "15A" "15B" "19B" "22" )))
( "foundling"  n ( f au1 n d l i ng ) ((pos "K6$" ) ))
( "foundlings"  n ( f au1 n d l i ng z ) ((pos "Kj$" ) ))
( "foundress"  n ( f au1 n d r i s ) ((pos "K7$" ) ))
( "foundresses"  n ( f au1 n d r i s i z ) ((pos "Kj$" ) ))
( "foundries"  n ( f au1 n d r i z ) ((pos "Kj%" ) ))
( "foundry"  n ( f au1 n d r ii ) ((pos "K8%" ) ))
( "founds"  v ( f au1 n d z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" "12A" "12B" "13A" "13B" "14" "15A" "15B" "19B" "22" )))
( "fount"  n ( f au1 n t ) ((pos "K6%" ) ))
( "fountain"  n ( f au1 n t i n ) ((pos "K6%" ) ))
( "fountain-head"  n ( f au1 n t i n - h e d ) ((pos "K6%" ) ))
( "fountain-heads"  n ( f au1 n t i n - h e d z ) ((pos "Kj$" ) ))
( "fountain-pen"  n ( f au1 n t i n - p e n ) ((pos "K6%" ) ))
( "fountain-pens"  n ( f au1 n t i n - p e n z ) ((pos "Kj%" ) ))
( "fountains"  n ( f au1 n t i n z ) ((pos "Kj%" ) ))
( "founts"  n ( f au1 n t s ) ((pos "Kj%" ) ))
( "four"  n ( f oo1 r ) ((pos "K6%" "OA*" ) ))
( "four-in-hand"  n ( f oo2 r - i n - h a1 n d ) ((pos "K6$" ) ))
( "four-in-hands"  n ( f oo2 r - i n - h a1 n d z ) ((pos "Kj$" ) ))
( "four-part"  j ( f oo1 - p aa t ) ((pos "OA%" ) ))
( "four-ply"  j ( f oo1 - p l ai ) ((pos "OA%" ) ))
( "four-poster"  n ( f oo1 - p ou1 s t @ r ) ((pos "K6%" ) ))
( "four-posters"  n ( f oo1 - p ou1 s t @ z ) ((pos "Kj%" ) ))
( "four-pounder"  n ( f oo1 - p au1 n d @ r ) ((pos "K6%" ) ))
( "four-pounders"  n ( f oo1 - p au1 n d @ z ) ((pos "Kj%" ) ))
( "four-wheeler"  n ( f oo1 - w ii1 l @ r ) ((pos "K6%" ) ))
( "four-wheelers"  n ( f oo1 - w ii1 l @ z ) ((pos "Kj%" ) ))
( "fourfold"  j ( f oo1 f ou l d ) ((pos "OA%" "Pu%" ) ))
( "fourpence"  n ( f oo1 p @ n s ) ((pos "K6%" ) ))
( "fourpences"  n ( f oo1 p @ n s i z ) ((pos "Kj%" ) ))
( "fourpenny"  j ( f oo1 p n ii ) ((pos "OA%" ) ))
( "fours"  n ( f oo1 z ) ((pos "Kj%" ) ))
( "fourscore"  n ( f oo1 s k oo r ) ((pos "Ki%" "OA%" ) ))
( "foursome"  n ( f oo1 s @ m ) ((pos "K6%" ) ))
( "foursomes"  n ( f oo1 s @ m z ) ((pos "Kj%" ) ))
( "foursquare"  j ( f oo1 s k w e@ r ) ((pos "OA%" ) ))
( "fourteen"  n ( f oo2 t ii1 n ) ((pos "K6%" "OA%" ) ))
( "fourteens"  n ( f oo2 t ii1 n z ) ((pos "Kj%" ) ))
( "fourteenth"  n ( f oo2 t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "fourteenths"  n ( f oo2 t ii1 n th s ) ((pos "Kj%" ) ))
( "fourth"  n ( f oo1 th ) ((pos "K6%" "OA%" ) ))
( "fourthly"  a ( f oo1 th l ii ) ((pos "Pu%" ) ))
( "fourths"  n ( f oo1 th s ) ((pos "Kj%" ) ))
( "fowl"  v ( f au1 l ) ((pos "I0%" "M6%" ) ))
( "fowl-run"  n ( f au1 l - r uh n ) ((pos "K6%" ) ))
( "fowl-runs"  n ( f au1 l - r uh n z ) ((pos "Kj$" ) ))
( "fowled"  v ( f au1 l d ) ((pos "Ic$" "Id$" ) ))
( "fowler"  n ( f au1 l @ r ) ((pos "K6$" ) ))
( "fowlers"  n ( f au1 l @ z ) ((pos "Kj$" ) ))
( "fowling"  v ( f au1 l i ng ) ((pos "Ib$" ) ))
( "fowlingpiece"  n ( f au1 l i ng p ii s ) ((pos "K6$" ) ))
( "fowlingpieces"  n ( f au1 l i ng p ii s i z ) ((pos "Kj$" ) ))
( "fowlpest"  n ( f au1 l p e s t ) ((pos "L@%" ) ))
( "fowls"  v ( f au1 l z ) ((pos "Ia%" "Mj%" ) ))
( "fox"  v ( f o1 k s ) ((pos "H1%" "K7%" ) (subcat )))
( "fox-terrier"  n ( f o1 k s - t e1 r i@ r ) ((pos "K6%" ) ))
( "fox-terriers"  n ( f o1 k s - t e1 r i@ z ) ((pos "Kj%" ) ))
( "foxed"  v ( f o1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "foxes"  v ( f o1 k s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "foxglove"  n ( f o1 k s g l uh v ) ((pos "K6%" ) ))
( "foxgloves"  n ( f o1 k s g l uh v z ) ((pos "Kj%" ) ))
( "foxhole"  n ( f o1 k s h ou l ) ((pos "K6%" ) ))
( "foxholes"  n ( f o1 k s h ou l z ) ((pos "Kj%" ) ))
( "foxhound"  n ( f o1 k s h au n d ) ((pos "K6%" ) ))
( "foxhounds"  n ( f o1 k s h au n d z ) ((pos "Kj%" ) ))
( "foxhunt"  v ( f o1 k s h uh n t ) ((pos "I0%" "K6%" ) ))
( "foxhunted"  v ( f o1 k s h uh n t i d ) ((pos "Ic%" "Id%" ) ))
( "foxhunter"  n ( f o1 k s h uh n t @ r ) ((pos "K6%" ) ))
( "foxhunters"  n ( f o1 k s h uh n t @ z ) ((pos "Kj%" ) ))
( "foxhunting"  v ( f o1 k s h uh n t i ng ) ((pos "Ib%" ) ))
( "foxhunts"  v ( f o1 k s h uh n t s ) ((pos "Ia%" "Kj%" ) ))
( "foxier"  j ( f o1 k s i@ r ) ((pos "Or%" ) ))
( "foxiest"  j ( f o1 k s i i s t ) ((pos "Os%" ) ))
( "foxing"  v ( f o1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "foxtrot"  n ( f o1 k s t r o t ) ((pos "K6%" ) ))
( "foxtrots"  n ( f o1 k s t r o t s ) ((pos "Kj%" ) ))
( "foxy"  j ( f o1 k s ii ) ((pos "OD%" ) ))
( "foyer"  n ( f oi1 ei ) ((pos "K6%" ) ))
( "foyers"  n ( f oi1 ei z ) ((pos "Kj$" ) ))
( "fracas"  n ( f r a1 k aa ) ((pos "Ki$" ) ))
( "fracas"  n ( f r a1 k aa z ) ((pos "Kj$" ) ))
( "fraction"  n ( f r a1 k sh @ n ) ((pos "K6%" ) ))
( "fractional"  j ( f r a1 k sh @ n @ l ) ((pos "OA%" ) ))
( "fractions"  n ( f r a1 k sh @ n z ) ((pos "Kj%" ) ))
( "fractious"  j ( f r a1 k sh @ s ) ((pos "OA$" ) ))
( "fractiously"  a ( f r a1 k sh @ s l ii ) ((pos "Pu$" ) ))
( "fractiousness"  n ( f r a1 k sh @ s n @ s ) ((pos "L@$" ) ))
( "fracture"  v ( f r a1 k ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "fractured"  v ( f r a1 k ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fractures"  v ( f r a1 k ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "fracturing"  v ( f r a1 k ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "fragile"  j ( f r a1 jh ai l ) ((pos "OA%" ) ))
( "fragility"  n ( f r @1 jh i1 l i t ii ) ((pos "L@%" ) ))
( "fragment"  n ( f r a1 g m @ n t ) ((pos "K6%" ) ))
( "fragment"  v ( f r a1 g m e1 n t ) ((pos "I0%" ) (subcat )))
( "fragmentary"  j ( f r a1 g m @ n t r ii ) ((pos "OA%" ) ))
( "fragmentation"  n ( f r a2 g m e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "fragmentations"  n ( f r a2 g m e n t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "fragmented"  v ( f r a1 g m e1 n t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "fragmenting"  v ( f r a1 g m e1 n t i ng ) ((pos "Ib%" ) (subcat )))
( "fragments"  n ( f r a1 g m @ n t s ) ((pos "Kj%" ) ))
( "fragments"  v ( f r a1 g m e1 n t s ) ((pos "Ia%" ) (subcat )))
( "fragrance"  n ( f r ei1 g r @ n s ) ((pos "M6%" ) ))
( "fragrances"  n ( f r ei1 g r @ n s i z ) ((pos "Mj%" ) ))
( "fragrant"  j ( f r ei1 g r @ n t ) ((pos "OA%" ) ))
( "frail"  j ( f r ei1 l ) ((pos "OC%" ) ))
( "frailer"  j ( f r ei1 l @ r ) ((pos "Or%" ) ))
( "frailest"  j ( f r ei1 l i s t ) ((pos "Os%" ) ))
( "frailties"  n ( f r ei1 l t i z ) ((pos "Mj%" ) ))
( "frailty"  n ( f r ei1 l t ii ) ((pos "M8%" ) ))
( "frame"  v ( f r ei1 m ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "frame-up"  n ( f r ei1 m - uh p ) ((pos "K6%" ) ))
( "frame-ups"  n ( f r ei1 m - uh p s ) ((pos "Kj%" ) ))
( "framed"  v ( f r ei1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "frames"  v ( f r ei1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "framework"  n ( f r ei1 m w @@ k ) ((pos "K6%" ) ))
( "frameworks"  n ( f r ei1 m w @@ k s ) ((pos "Kj%" ) ))
( "framing"  v ( f r ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "franc"  n ( f r a1 ng k ) ((pos "K6%" ) ))
( "franchise"  n ( f r a1 n ch ai z ) ((pos "M6%" ) ))
( "franchises"  n ( f r a1 n ch ai z i z ) ((pos "Mj$" ) ))
( "francs"  n ( f r a1 ng k s ) ((pos "Kj%" ) ))
( "frank"  v ( f r a1 ng k ) ((pos "H0%" "OC%" ) (subcat )))
( "franked"  v ( f r a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "franker"  j ( f r a1 ng k @ r ) ((pos "Or%" ) ))
( "frankest"  j ( f r a1 ng k i s t ) ((pos "Os%" ) ))
( "frankfurter"  n ( f r a1 ng k f @@ t @ r ) ((pos "K6%" ) ))
( "frankfurters"  n ( f r a1 ng k f @@ t @ z ) ((pos "Kj%" ) ))
( "frankincense"  n ( f r a1 ng k i n s e n s ) ((pos "L@%" ) ))
( "franking"  v ( f r a1 ng k i ng ) ((pos "Hb%" ) (subcat )))
( "franking-machine"  n ( f r a1 n k i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "franking-machines"  n ( f r a1 n k i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "franklin"  n ( f r a1 ng k l i n ) ((pos "K6$" ) ))
( "franklins"  n ( f r a1 ng k l i n z ) ((pos "Kj$" ) ))
( "frankly"  a ( f r a1 ng k l ii ) ((pos "Pu%" ) ))
( "frankness"  n ( f r a1 ng k n @ s ) ((pos "L@%" ) ))
( "franks"  v ( f r a1 ng k s ) ((pos "Ha%" ) (subcat )))
( "frantic"  j ( f r a1 n t i k ) ((pos "OA%" ) ))
( "frantically"  a ( f r a1 n t i k l ii ) ((pos "Pu%" ) ))
( "fraternal"  j ( f r @1 t @@1 n l ) ((pos "OA%" ) ))
( "fraternally"  a ( f r @1 t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "fraternities"  n ( f r @1 t @@1 n i t i z ) ((pos "Mj%" ) ))
( "fraternity"  n ( f r @1 t @@1 n i t ii ) ((pos "M8%" ) ))
( "fraternization"  n ( f r a2 t @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "fraternize"  v ( f r a1 t @ n ai z ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "fraternized"  v ( f r a1 t @ n ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "fraternizes"  v ( f r a1 t @ n ai z i z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "fraternizing"  v ( f r a1 t @ n ai z i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "fratricide"  n ( f r a1 t r i s ai d ) ((pos "M6%" ) ))
( "fratricides"  n ( f r a1 t r i s ai d z ) ((pos "Mj%" ) ))
( "fraud"  n ( f r oo1 d ) ((pos "M6%" ) ))
( "frauds"  n ( f r oo1 d z ) ((pos "Mj%" ) ))
( "fraudulent"  j ( f r oo1 d y u l @ n t ) ((pos "OA%" ) ))
( "fraudulently"  a ( f r oo1 d y u l @ n t l ii ) ((pos "Pu%" ) ))
( "fraught"  j ( f r oo1 t ) ((pos "Op%" ) ))
( "fray"  v ( f r ei1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "frayed"  v ( f r ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fraying"  v ( f r ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "frays"  v ( f r ei1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "frazzle"  n ( f r a1 z l ) ((pos "K6%" ) ))
( "frazzles"  n ( f r a1 z l z ) ((pos "Kj%" ) ))
( "freak"  v ( f r ii1 k ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "freak-out"  n ( f r ii1 k - au t ) ((pos "K6%" ) ))
( "freak-outs"  n ( f r ii1 k - au t s ) ((pos "Kj$" ) ))
( "freaked"  v ( f r ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "freakier"  j ( f r ii1 k i@ r ) ((pos "Or%" ) ))
( "freakiest"  j ( f r ii1 k i i s t ) ((pos "Os%" ) ))
( "freaking"  v ( f r ii1 k i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "freakish"  j ( f r ii1 k i sh ) ((pos "OA%" ) ))
( "freakishly"  a ( f r ii1 k i sh l ii ) ((pos "Pu%" ) ))
( "freakishness"  n ( f r ii1 k i sh n @ s ) ((pos "L@%" ) ))
( "freaks"  v ( f r ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "freaky"  j ( f r ii1 k ii ) ((pos "OD%" ) ))
( "freckle"  v ( f r e1 k l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "freckled"  v ( f r e1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "freckles"  v ( f r e1 k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "freckling"  v ( f r e1 k l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "free"  v ( f r ii1 ) ((pos "H5%" "OB%" ) (subcat "6A" )))
( "free-and-easy"  j ( f r ii2 - @ n - ii1 z ii ) ((pos "OA%" ) ))
( "free-for-all"  n ( f r ii1 - f @ r - oo l ) ((pos "K6%" ) ))
( "free-for-alls"  n ( f r ii1 - f @ r - oo l z ) ((pos "Kj$" ) ))
( "free-handed"  j ( f r ii1 - h a1 n d i d ) ((pos "OA$" ) ))
( "free-list"  n ( f r ii1 - l i s t ) ((pos "K6$" ) ))
( "free-lists"  n ( f r ii1 - l i s t s ) ((pos "Kj$" ) ))
( "free-liver"  n ( f r ii1 - l i1 v @ r ) ((pos "K6$" ) ))
( "free-livers"  n ( f r ii1 - l i1 v @ z ) ((pos "Kj$" ) ))
( "free-living"  n ( f r ii1 - l i1 v i ng ) ((pos "L@%" ) ))
( "free-range"  j ( f r ii1 - r ei1 n jh ) ((pos "OA%" ) ))
( "free-spoken"  j ( f r ii1 - s p ou1 k @ n ) ((pos "OA%" ) ))
( "free-standing"  j ( f r ii1 - s t a1 n d i ng ) ((pos "OA%" ) ))
( "free-thinker"  n ( f r ii1 - th i1 ng k @ r ) ((pos "K6%" ) ))
( "free-thinkers"  n ( f r ii1 - th i1 ng k @ z ) ((pos "Kj%" ) ))
( "free-thinking"  j ( f r ii1 - th i1 ng k i ng ) ((pos "OA%" ) ))
( "free-thought"  n ( f r ii1 - th oo1 t ) ((pos "L@$" ) ))
( "free-trader"  n ( f r ii1 - t r ei1 d @ r ) ((pos "K6$" ) ))
( "free-traders"  n ( f r ii1 - t r ei1 d @ z ) ((pos "Kj$" ) ))
( "freebooter"  n ( f r ii1 b uu t @ r ) ((pos "K6$" ) ))
( "freebooters"  n ( f r ii1 b uu t @ z ) ((pos "Kj$" ) ))
( "freeborn"  j ( f r ii1 b oo n ) ((pos "OA%" ) ))
( "freed"  v ( f r ii1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "freedman"  n ( f r ii1 d m @ n ) ((pos "Ki$" ) ))
( "freedmen"  n ( f r ii1 d m @ n ) ((pos "Kj$" ) ))
( "freedom"  n ( f r ii1 d @ m ) ((pos "M6%" ) ))
( "freedoms"  n ( f r ii1 d @ m z ) ((pos "Mj%" ) ))
( "freehand"  j ( f r ii1 h a n d ) ((pos "OA%" ) ))
( "freehold"  n ( f r ii1 h ou l d ) ((pos "K6%" ) ))
( "freeholder"  n ( f r ii1 h ou l d @ r ) ((pos "K6%" ) ))
( "freeholders"  n ( f r ii1 h ou l d @ z ) ((pos "Kj%" ) ))
( "freeholds"  n ( f r ii1 h ou l d z ) ((pos "Kj%" ) ))
( "freeing"  v ( f r ii1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "freelance"  v ( f r ii1 l aa n s ) ((pos "I2%" "K6%" ) ))
( "freelanced"  v ( f r ii1 l aa n s t ) ((pos "Ic%" "Id%" ) ))
( "freelances"  v ( f r ii1 l aa n s i z ) ((pos "Ia%" "Kj%" ) ))
( "freelancing"  v ( f r ii1 l aa n s i ng ) ((pos "Ib%" ) ))
( "freely"  a ( f r ii1 l ii ) ((pos "Pu%" ) ))
( "freeman"  n ( f r ii1 m @ n ) ((pos "Ki%" ) ))
( "freemen"  n ( f r ii1 m @ n ) ((pos "Kj%" ) ))
( "freer"  j ( f r ii1 @ r ) ((pos "Or%" ) ))
( "frees"  v ( f r ii1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "freesia"  n ( f r ii1 z i@ ) ((pos "K6$" ) ))
( "freesias"  n ( f r ii1 z i@ z ) ((pos "Kj$" ) ))
( "freest"  j ( f r ii1 i s t ) ((pos "Os$" ) ))
( "freestone"  n ( f r ii1 s t ou n ) ((pos "L@$" ) ))
( "freestyle"  n ( f r ii1 s t ai l ) ((pos "L@%" ) ))
( "freeway"  n ( f r ii1 w ei ) ((pos "K6%" ) ))
( "freeways"  n ( f r ii1 w ei z ) ((pos "Kj%" ) ))
( "freewheel"  v ( f r ii1 w ii1 l ) ((pos "I0%" ) (subcat "2A" )))
( "freewheeled"  v ( f r ii1 w ii1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "freewheeling"  v ( f r ii1 w ii1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "freewheels"  v ( f r ii1 w ii1 l z ) ((pos "Ia%" ) (subcat "2A" )))
( "freewill"  j ( f r ii1 w i l ) ((pos "OA%" ) ))
( "freeze"  v ( f r ii1 z ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "freezer"  n ( f r ii1 z @ r ) ((pos "K6%" ) ))
( "freezers"  n ( f r ii1 z @ z ) ((pos "Kj%" ) ))
( "freezes"  v ( f r ii1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "freezing"  v ( f r ii1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "freezing-mixture"  n ( f r ii1 z i ng - m i k s ch @ r ) ((pos "K6$" ) ))
( "freezing-mixtures"  n ( f r ii1 z i ng - m i k s ch @ z ) ((pos "Kj$" ) ))
( "freezing-point"  n ( f r ii1 z i ng - p oi n t ) ((pos "K6%" ) ))
( "freezing-points"  n ( f r ii1 z i ng - p oi n t s ) ((pos "Kj%" ) ))
( "freight"  v ( f r ei1 t ) ((pos "H0$" "L@%" ) (subcat "6A" )))
( "freight-train"  n ( f r ei1 t - t r ei n ) ((pos "K6%" ) ))
( "freight-trains"  n ( f r ei1 t - t r ei n z ) ((pos "Kj%" ) ))
( "freighted"  v ( f r ei1 t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "freighter"  n ( f r ei1 t @ r ) ((pos "K6%" ) ))
( "freighters"  n ( f r ei1 t @ z ) ((pos "Kj%" ) ))
( "freighting"  v ( f r ei1 t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "freightliner"  n ( f r ei1 t l ai n @ r ) ((pos "K6%" ) ))
( "freightliners"  n ( f r ei1 t l ai n @ z ) ((pos "Kj%" ) ))
( "freights"  v ( f r ei1 t s ) ((pos "Ha$" ) (subcat "6A" )))
( "frenetic"  j ( f r @1 n e1 t i k ) ((pos "OA%" ) ))
( "frenzied"  j ( f r e1 n z i d ) ((pos "OA%" ) ))
( "frenziedly"  a ( f r e1 n z i d l ii ) ((pos "Pu%" ) ))
( "frenzy"  n ( f r e1 n z ii ) ((pos "L@%" ) ))
( "frequencies"  n ( f r ii1 k w @ n s i z ) ((pos "Mj%" ) ))
( "frequency"  n ( f r ii1 k w @ n s ii ) ((pos "M8%" ) ))
( "frequent"  j ( f r ii1 k w @ n t ) ((pos "OA%" ) ))
( "frequent"  v ( f r i1 k w e1 n t ) ((pos "H0%" ) (subcat )))
( "frequented"  v ( f r i1 k w e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "frequenting"  v ( f r i1 k w e1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "frequently"  a ( f r ii1 k w @ n t l ii ) ((pos "Pu%" ) ))
( "frequents"  v ( f r i1 k w e1 n t s ) ((pos "Ha%" ) (subcat )))
( "fresco"  v ( f r e1 s k ou ) ((pos "H0$" "M6%" ) ))
( "frescoed"  v ( f r e1 s k ou d ) ((pos "Hc$" "Hd$" ) ))
( "frescoes"  n ( f r e1 s k ou z ) ((pos "Kj%" ) ))
( "frescoing"  v ( f r e1 s k ou i ng ) ((pos "Hb$" ) ))
( "frescos"  v ( f r e1 s k ou z ) ((pos "Ha$" "Mj$" ) ))
( "fresh"  j ( f r e1 sh ) ((pos "OC%" "Pu%" ) ))
( "freshen"  v ( f r e1 sh @ n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "freshened"  v ( f r e1 sh @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "freshening"  v ( f r e1 sh @ n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "freshens"  v ( f r e1 sh @ n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "fresher"  n ( f r e1 sh @ r ) ((pos "K6%" "Or%" ) ))
( "freshers"  n ( f r e1 sh @ z ) ((pos "Kj%" ) ))
( "freshest"  j ( f r e1 sh i s t ) ((pos "Os%" ) ))
( "freshly"  a ( f r e1 sh l ii ) ((pos "Pu%" ) ))
( "freshman"  n ( f r e1 sh m @ n ) ((pos "Ki%" ) ))
( "freshmen"  n ( f r e1 sh m @ n ) ((pos "Kj%" ) ))
( "freshness"  n ( f r e1 sh n @ s ) ((pos "L@%" ) ))
( "freshwater"  j ( f r e1 sh w oo t @ r ) ((pos "OA%" ) ))
( "fret"  v ( f r e1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fretful"  j ( f r e1 t f @ l ) ((pos "OA%" ) ))
( "fretfully"  a ( f r e1 t f @ l ii ) ((pos "Pu%" ) ))
( "frets"  v ( f r e1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fretsaw"  n ( f r e1 t s oo ) ((pos "K6%" ) ))
( "fretsaws"  n ( f r e1 t s oo z ) ((pos "Kj%" ) ))
( "fretted"  v ( f r e1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fretting"  v ( f r e1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fretwork"  n ( f r e1 t w @@ k ) ((pos "L@%" ) ))
( "friability"  n ( f r ai2 @ b i1 l i t ii ) ((pos "L@$" ) ))
( "friable"  j ( f r ai1 @ b l ) ((pos "OA%" ) ))
( "friar"  n ( f r ai1 @ r ) ((pos "K6%" ) ))
( "friars"  n ( f r ai1 @ z ) ((pos "Kj%" ) ))
( "fricassee"  n ( f r i1 k @ s ii ) ((pos "M6$" ) ))
( "fricassee"  v ( f r i2 k @ s ii1 ) ((pos "H5$" ) ))
( "fricasseed"  v ( f r i2 k @ s ii1 d ) ((pos "Hc$" "Hd$" ) ))
( "fricasseeing"  v ( f r i2 k @ s ii1 i ng ) ((pos "Hb$" ) ))
( "fricassees"  n ( f r i1 k @ s ii z ) ((pos "Mj$" ) ))
( "fricassees"  v ( f r i2 k @ s ii1 z ) ((pos "Ha$" ) ))
( "fricative"  n ( f r i1 k @ t i v ) ((pos "K6$" "OA$" ) ))
( "fricatives"  n ( f r i1 k @ t i v z ) ((pos "Kj$" ) ))
( "friction"  n ( f r i1 k sh @ n ) ((pos "M6%" ) ))
( "frictions"  n ( f r i1 k sh @ n z ) ((pos "Mj$" ) ))
( "fridge"  n ( f r i1 jh ) ((pos "K6%" ) ))
( "fridges"  n ( f r i1 jh i z ) ((pos "Kj%" ) ))
( "fried"  v ( f r ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "friend"  n ( f r e1 n d ) ((pos "K6%" ) ))
( "friendless"  j ( f r e1 n d l @ s ) ((pos "OA%" ) ))
( "friendlessness"  n ( f r e1 n d l @ s n @ s ) ((pos "L@%" ) ))
( "friendlier"  j ( f r e1 n d l i@ r ) ((pos "Or%" ) ))
( "friendliest"  j ( f r e1 n d l i i s t ) ((pos "Os%" ) ))
( "friendliness"  n ( f r e1 n d l i n @ s ) ((pos "L@%" ) ))
( "friendly"  j ( f r e1 n d l ii ) ((pos "OD%" ) ))
( "friends"  n ( f r e1 n d z ) ((pos "Kj%" ) ))
( "friendship"  n ( f r e1 n d sh i p ) ((pos "M6%" ) ))
( "friendships"  n ( f r e1 n d sh i p s ) ((pos "Mj%" ) ))
( "frier"  n ( f r ai1 @ r ) ((pos "K6$" ) ))
( "friers"  n ( f r ai1 @ z ) ((pos "Kj$" ) ))
( "fries"  v ( f r ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "frieze"  n ( f r ii1 z ) ((pos "K6%" ) ))
( "friezes"  n ( f r ii1 z i z ) ((pos "Kj%" ) ))
( "frigate"  n ( f r i1 g @ t ) ((pos "K6%" ) ))
( "frigates"  n ( f r i1 g @ t s ) ((pos "Kj%" ) ))
( "fright"  v ( f r ai1 t ) ((pos "H0$" "M6%" ) ))
( "frighted"  v ( f r ai1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "frighten"  v ( f r ai1 t n ) ((pos "H0%" ) (subcat "6A" "14" )))
( "frightened"  v ( f r ai1 t n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "14" )))
( "frightening"  v ( f r ai1 t n i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "frighteningly"  a ( f r ai1 t n i ng l ii ) ((pos "Pu%" ) ))
( "frightens"  v ( f r ai1 t n z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "frightful"  j ( f r ai1 t f @ l ) ((pos "OA%" ) ))
( "frightfully"  a ( f r ai1 t f @ l ii ) ((pos "Pu%" ) ))
( "frightfulness"  n ( f r ai1 t f @ l n @ s ) ((pos "L@%" ) ))
( "frighting"  v ( f r ai1 t i ng ) ((pos "Hb$" ) ))
( "frights"  v ( f r ai1 t s ) ((pos "Ha$" "Mj%" ) ))
( "frigid"  j ( f r i1 jh i d ) ((pos "OA%" ) ))
( "frigidity"  n ( f r i1 jh i1 d i t ii ) ((pos "L@%" ) ))
( "frigidly"  a ( f r i1 jh i d l ii ) ((pos "Pu%" ) ))
( "frill"  n ( f r i1 l ) ((pos "K6%" ) ))
( "frilled"  j ( f r i1 l d ) ((pos "OA%" ) ))
( "frillier"  j ( f r i1 l i@ r ) ((pos "Or$" ) ))
( "frilliest"  j ( f r i1 l i i s t ) ((pos "Os$" ) ))
( "frills"  n ( f r i1 l z ) ((pos "Kj%" ) ))
( "frilly"  j ( f r i1 l ii ) ((pos "OD%" ) ))
( "fringe"  v ( f r i1 n jh ) ((pos "H2%" "K6%" ) (subcat )))
( "fringed"  v ( f r i1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fringes"  v ( f r i1 n jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "fringing"  v ( f r i1 n jh i ng ) ((pos "Hb%" ) (subcat )))
( "fripperies"  n ( f r i1 p @ r i z ) ((pos "Mj$" ) ))
( "frippery"  n ( f r i1 p @ r ii ) ((pos "M8$" ) ))
( "frisk"  v ( f r i1 s k ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "frisked"  v ( f r i1 s k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "friskier"  j ( f r i1 s k i@ r ) ((pos "Or%" ) ))
( "friskiest"  j ( f r i1 s k i i s t ) ((pos "Os%" ) ))
( "friskily"  a ( f r i1 s k i l ii ) ((pos "Pu%" ) ))
( "frisking"  v ( f r i1 s k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "frisks"  v ( f r i1 s k s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "frisky"  j ( f r i1 s k ii ) ((pos "OD%" ) ))
( "frisson"  n ( f r ii1 s o n ) ((pos "K6$" ) ))
( "frissons"  n ( f r ii1 s o n z ) ((pos "Kj$" ) ))
( "fritter"  v ( f r i1 t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "frittered"  v ( f r i1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "frittering"  v ( f r i1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "fritters"  v ( f r i1 t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "frivol"  v ( f r i1 v @ l ) ((pos "J4$" ) (subcat "2A" )))
( "frivolities"  n ( f r i1 v o1 l i t i z ) ((pos "Mj%" ) ))
( "frivolity"  n ( f r i1 v o1 l i t ii ) ((pos "M8%" ) ))
( "frivolled"  v ( f r i1 v @ l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "frivolling"  v ( f r i1 v @ l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "frivolous"  j ( f r i1 v @ l @ s ) ((pos "OA%" ) ))
( "frivolously"  a ( f r i1 v @ l @ s l ii ) ((pos "Pu%" ) ))
( "frivols"  v ( f r i1 v @ l z ) ((pos "Ja$" ) (subcat "2A" )))
( "frizz"  v ( f r i1 z ) ((pos "H1$" ) (subcat )))
( "frizzed"  v ( f r i1 z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "frizzes"  v ( f r i1 z i z ) ((pos "Ha$" ) (subcat )))
( "frizzier"  j ( f r i1 z i@ r ) ((pos "Or$" ) ))
( "frizziest"  j ( f r i1 z i i s t ) ((pos "Os$" ) ))
( "frizzing"  v ( f r i1 z i ng ) ((pos "Hb$" ) (subcat )))
( "frizzle"  v ( f r i1 z l ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "frizzled"  v ( f r i1 z l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "frizzles"  v ( f r i1 z l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "frizzling"  v ( f r i1 z l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "frizzy"  j ( f r i1 z ii ) ((pos "OD%" ) ))
( "fro"  a ( f r ou1 ) ((pos "Pu%" ) ))
( "frock"  n ( f r o1 k ) ((pos "K6%" ) ))
( "frock-coat"  n ( f r o1 k - k ou t ) ((pos "K6%" ) ))
( "frock-coats"  n ( f r o1 k - k ou t s ) ((pos "Kj%" ) ))
( "frocks"  n ( f r o1 k s ) ((pos "Kj%" ) ))
( "frog"  n ( f r o1 g ) ((pos "K6%" ) ))
( "frogman"  n ( f r o1 g m @ n ) ((pos "Ki%" ) ))
( "frogmarch"  v ( f r o1 g m aa ch ) ((pos "H1%" ) (subcat )))
( "frogmarched"  v ( f r o1 g m aa ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "frogmarches"  v ( f r o1 g m aa ch i z ) ((pos "Ha%" ) (subcat )))
( "frogmarching"  v ( f r o1 g m aa ch i ng ) ((pos "Hb%" ) (subcat )))
( "frogmen"  n ( f r o1 g m @ n ) ((pos "Kj%" ) ))
( "frogs"  n ( f r o1 g z ) ((pos "Kj%" ) ))
( "frolic"  v ( f r o1 l i k ) ((pos "I5%" "K6%" ) (subcat "2A" )))
( "frolicked"  v ( f r o1 l i k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "frolicking"  v ( f r o1 l i k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "frolics"  v ( f r o1 l i k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "frolicsome"  j ( f r o1 l i k s @ m ) ((pos "OA%" ) ))
( "from"  in ( f r o m ) ((pos "T-*" ) ))
( "frond"  n ( f r o1 n d ) ((pos "K6%" ) ))
( "fronds"  n ( f r o1 n d z ) ((pos "Kj%" ) ))
( "front"  v ( f r uh1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "front-bench"  n ( f r uh1 n t - b e1 n ch ) ((pos "K7%" ) ))
( "front-bencher"  n ( f r uh1 n t - b e1 n ch @ r ) ((pos "K6%" ) ))
( "front-benchers"  n ( f r uh1 n t - b e1 n ch @ z ) ((pos "Kj%" ) ))
( "front-benches"  n ( f r uh1 n t - b e1 n ch i z ) ((pos "Kj%" ) ))
( "frontage"  n ( f r uh1 n t i jh ) ((pos "K6$" ) ))
( "frontages"  n ( f r uh1 n t i jh i z ) ((pos "Kj$" ) ))
( "frontal"  j ( f r uh1 n t l ) ((pos "OA%" ) ))
( "fronted"  v ( f r uh1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "frontier"  n ( f r uh1 n t i@ r ) ((pos "K6%" ) ))
( "frontiers"  n ( f r uh1 n t i@ z ) ((pos "Kj%" ) ))
( "frontiersman"  n ( f r uh1 n t i@ z m @ n ) ((pos "Ki%" ) ))
( "frontiersmen"  n ( f r uh1 n t i@ z m @ n ) ((pos "Kj%" ) ))
( "fronting"  v ( f r uh1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "frontispiece"  n ( f r uh1 n t i s p ii s ) ((pos "K6%" ) ))
( "frontispieces"  n ( f r uh1 n t i s p ii s i z ) ((pos "Kj%" ) ))
( "fronts"  v ( f r uh1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "frost"  v ( f r o1 s t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "frost-bound"  j ( f r o1 s t - b au n d ) ((pos "OA%" ) ))
( "frostbite"  n ( f r o1 s t b ai t ) ((pos "L@%" ) ))
( "frostbitten"  j ( f r o1 s t b i t n ) ((pos "OA%" ) ))
( "frosted"  v ( f r o1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "frostier"  j ( f r o1 s t i@ r ) ((pos "Or%" ) ))
( "frostiest"  j ( f r o1 s t i i s t ) ((pos "Os%" ) ))
( "frostily"  a ( f r o1 s t i l ii ) ((pos "Pu%" ) ))
( "frostiness"  n ( f r o1 s t i n @ s ) ((pos "L@%" ) ))
( "frosting"  v ( f r o1 s t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" )))
( "frosts"  v ( f r o1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "frosty"  j ( f r o1 s t ii ) ((pos "OD%" ) ))
( "froth"  v ( f r o1 th ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "frothed"  v ( f r o1 th t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "frothier"  j ( f r o1 th i@ r ) ((pos "Or%" ) ))
( "frothiest"  j ( f r o1 th i i s t ) ((pos "Os%" ) ))
( "frothily"  a ( f r o1 th i l ii ) ((pos "Pu%" ) ))
( "frothiness"  n ( f r o1 th i n @ s ) ((pos "L@%" ) ))
( "frothing"  v ( f r o1 th i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "froths"  v ( f r o1 th s ) ((pos "Ia%" ) (subcat "2A" )))
( "frothy"  j ( f r o1 th ii ) ((pos "OD%" ) ))
( "froward"  j ( f r ou1 @ d ) ((pos "OA$" ) ))
( "frown"  v ( f r au1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "frowned"  v ( f r au1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "frowning"  v ( f r au1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "frowningly"  a ( f r au1 n i ng l ii ) ((pos "Pu%" ) ))
( "frowns"  v ( f r au1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "frowsty"  j ( f r au1 s t ii ) ((pos "OA$" ) ))
( "frowzy"  j ( f r au1 z ii ) ((pos "OA$" ) ))
( "froze"  v ( f r ou1 z ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" )))
( "frozen"  v ( f r ou1 z @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "fructification"  n ( f r uh2 k t i f i k ei1 sh @ n ) ((pos "L@$" ) ))
( "fructified"  v ( f r uh1 k t i f ai d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "fructifies"  v ( f r uh1 k t i f ai z ) ((pos "Ja$" ) (subcat "2A" )))
( "fructify"  v ( f r uh1 k t i f ai ) ((pos "J3$" ) (subcat "2A" )))
( "fructifying"  v ( f r uh1 k t i f ai i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "frugal"  j ( f r uu1 g l ) ((pos "OA%" ) ))
( "frugalities"  n ( f r uu1 g a1 l i t i z ) ((pos "Mj$" ) ))
( "frugality"  n ( f r uu1 g a1 l i t ii ) ((pos "M8%" ) ))
( "frugally"  a ( f r uu1 g @ l ii ) ((pos "Pu%" ) ))
( "fruit"  v ( f r uu1 t ) ((pos "I0%" "M6%" ) ))
( "fruit-flies"  n ( f r uu1 t - f l ai z ) ((pos "Kj%" ) ))
( "fruit-fly"  n ( f r uu1 t - f l ai ) ((pos "K8%" ) ))
( "fruitcake"  n ( f r uu1 t k ei k ) ((pos "K6%" ) ))
( "fruitcakes"  n ( f r uu1 t k ei k s ) ((pos "Kj%" ) ))
( "fruited"  v ( f r uu1 t i d ) ((pos "Ic%" "Id%" ) ))
( "fruiterer"  n ( f r uu1 t @ r @ r ) ((pos "K6%" ) ))
( "fruiterers"  n ( f r uu1 t @ r @ z ) ((pos "Kj%" ) ))
( "fruitful"  j ( f r uu1 t f @ l ) ((pos "OA%" ) ))
( "fruitfully"  a ( f r uu1 t f @ l ii ) ((pos "Pu%" ) ))
( "fruitfulness"  n ( f r uu1 t f @ l n @ s ) ((pos "L@%" ) ))
( "fruitier"  j ( f r uu1 t i@ r ) ((pos "Or%" ) ))
( "fruitiest"  j ( f r uu1 t i i s t ) ((pos "Os%" ) ))
( "fruiting"  v ( f r uu1 t i ng ) ((pos "Ib%" ) ))
( "fruition"  n ( f r uu1 i1 sh @ n ) ((pos "L@%" ) ))
( "fruitless"  j ( f r uu1 t l @ s ) ((pos "OA%" ) ))
( "fruitlessly"  a ( f r uu1 t l @ s l ii ) ((pos "Pu%" ) ))
( "fruitlessness"  n ( f r uu1 t l @ s n @ s ) ((pos "L@%" ) ))
( "fruits"  v ( f r uu1 t s ) ((pos "Ia%" "Mj%" ) ))
( "fruity"  j ( f r uu1 t ii ) ((pos "OD%" ) ))
( "frump"  n ( f r uh1 m p ) ((pos "K6%" ) ))
( "frumpier"  j ( f r uh1 m p i@ r ) ((pos "Or$" ) ))
( "frumpiest"  j ( f r uh1 m p i i s t ) ((pos "Os$" ) ))
( "frumpish"  j ( f r uh1 m p i sh ) ((pos "OA%" ) ))
( "frumps"  n ( f r uh1 m p s ) ((pos "Kj$" ) ))
( "frumpy"  j ( f r uh1 m p ii ) ((pos "OD%" ) ))
( "frustrate"  v ( f r uh1 s t r ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "frustrated"  v ( f r uh1 s t r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "frustrates"  v ( f r uh1 s t r ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "frustrating"  v ( f r uh1 s t r ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "frustration"  n ( f r uh1 s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "frustrations"  n ( f r uh1 s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "fry"  v ( f r ai1 ) ((pos "J3%" "K9%" ) (subcat "2A" )))
( "fry-pan"  n ( f r ai1 - p a n ) ((pos "K6%" ) ))
( "fry-pans"  n ( f r ai1 - p a n z ) ((pos "Kj%" ) ))
( "fryer"  n ( f r ai1 @ r ) ((pos "K6$" ) ))
( "fryers"  n ( f r ai1 @ z ) ((pos "Kj$" ) ))
( "frying"  v ( f r ai1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "frying-pan"  n ( f r ai1 i ng - p a n ) ((pos "K6%" ) ))
( "frying-pans"  n ( f r ai1 i ng - p a n z ) ((pos "Kj%" ) ))
( "ft"  n ( f ii1 t ) ((pos "Y]%" ) ))
( "fuchsia"  n ( f y uu1 sh @ ) ((pos "K6%" ) ))
( "fuchsias"  n ( f y uu1 sh @ z ) ((pos "Kj%" ) ))
( "fuck"  v ( f uh1 k ) ((pos "J0%" ) (subcat "2A" )))
( "fuck-all"  n ( f uh1 k - oo1 l ) ((pos "L@%" ) ))
( "fucked"  v ( f uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fucker"  n ( f uh1 k @ r ) ((pos "K6%" ) ))
( "fuckers"  n ( f uh1 k @ z ) ((pos "Kj%" ) ))
( "fucking"  v ( f uh1 k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "fucks"  v ( f uh1 k s ) ((pos "Ja%" ) (subcat "2A" )))
( "fuddle"  v ( f uh1 d l ) ((pos "H2$" ) (subcat "6A" )))
( "fuddled"  v ( f uh1 d l d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "fuddles"  v ( f uh1 d l z ) ((pos "Ha$" ) (subcat "6A" )))
( "fuddling"  v ( f uh1 d l i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "fuddy-duddies"  n ( f uh1 d i - d uh d i z ) ((pos "Kj%" ) ))
( "fuddy-duddy"  n ( f uh1 d i - d uh d ii ) ((pos "K8%" ) ))
( "fudge"  n ( f uh1 jh ) ((pos "M6%" "W-%" ) ))
( "fudges"  n ( f uh1 jh i z ) ((pos "Mj%" ) ))
( "fuel"  v ( f y uu1 @ l ) ((pos "J4%" "M6%" ) (subcat "2A" )))
( "fuelled"  v ( f y uu1 @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "fuelling"  v ( f y uu1 @ l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "fuels"  v ( f y uu1 @ l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "fug"  n ( f uh1 g ) ((pos "K6%" ) ))
( "fuggier"  j ( f uh1 g i@ r ) ((pos "Or$" ) ))
( "fuggiest"  j ( f uh1 g i i s t ) ((pos "Os$" ) ))
( "fuggy"  j ( f uh1 g ii ) ((pos "OD%" ) ))
( "fugitive"  n ( f y uu1 jh @ t i v ) ((pos "K6%" "OA%" ) ))
( "fugitives"  n ( f y uu1 jh @ t i v z ) ((pos "Kj%" ) ))
( "fugs"  n ( f uh1 g z ) ((pos "Kj$" ) ))
( "fugue"  n ( f y uu1 g ) ((pos "K6%" ) ))
( "fugues"  n ( f y uu1 g z ) ((pos "Kj%" ) ))
( "fulcra"  n ( f u1 l k r @ ) ((pos "Kj$" ) ))
( "fulcrum"  n ( f u1 l k r @ m ) ((pos "K6%" ) ))
( "fulcrums"  n ( f u1 l k r @ m z ) ((pos "Kj$" ) ))
( "fulfil"  v ( f u1 l f i1 l ) ((pos "H4%" ) (subcat )))
( "fulfilled"  v ( f u1 l f i1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fulfilling"  v ( f u1 l f i1 l i ng ) ((pos "Hb%" ) (subcat )))
( "fulfilment"  n ( f u1 l f i1 l m @ n t ) ((pos "K6%" ) ))
( "fulfilments"  n ( f u1 l f i1 l m @ n t s ) ((pos "Kj$" ) ))
( "fulfils"  v ( f u1 l f i1 l z ) ((pos "Ha%" ) (subcat )))
( "full"  j ( f u1 l ) ((pos "OC*" ) ))
( "full-blooded"  j ( f u1 l - b l uh1 d i d ) ((pos "OA%" ) ))
( "full-blown"  j ( f u1 l - b l ou1 n ) ((pos "OA%" ) ))
( "full-dress"  j ( f u1 l - d r e1 s ) ((pos "OA%" ) ))
( "full-fashioned"  j ( f u1 l - f a1 sh @ n d ) ((pos "OA%" ) ))
( "full-fledged"  j ( f u1 l - f l e1 jh d ) ((pos "OA%" ) ))
( "full-grown"  j ( f u1 l - g r ou1 n ) ((pos "OA%" ) ))
( "full-length"  j ( f u1 l - l e1 ng th ) ((pos "OA%" ) ))
( "full-page"  j ( f u1 l - p ei1 jh ) ((pos "OA%" ) ))
( "full-scale"  j ( f u1 l - s k ei1 l ) ((pos "OA%" ) ))
( "full-time"  j ( f u1 l - t ai1 m ) ((pos "OA%" "Pu%" ) ))
( "fullback"  n ( f u1 l b a k ) ((pos "K6%" ) ))
( "fullbacks"  n ( f u1 l b a k s ) ((pos "Kj%" ) ))
( "fuller"  n ( f u1 l @ r ) ((pos "K6$" "Or%" ) ))
( "fullers"  n ( f u1 l @ z ) ((pos "Kj$" ) ))
( "fullest"  j ( f u1 l i s t ) ((pos "Os%" ) ))
( "fullness"  n ( f u1 l n @ s ) ((pos "L@%" ) ))
( "fully"  a ( f u1 l ii ) ((pos "Pu%" ) ))
( "fully-fashioned"  j ( f u2 l i - f a1 sh @ n d ) ((pos "OA%" ) ))
( "fully-fledged"  j ( f u2 l i - f l e1 jh d ) ((pos "OA%" ) ))
( "fully-grown"  j ( f u2 l i - g r ou1 n ) ((pos "OA%" ) ))
( "fulmar"  n ( f u1 l m @ r ) ((pos "K6$" ) ))
( "fulmars"  n ( f u1 l m @ z ) ((pos "Kj$" ) ))
( "fulminate"  v ( f uh1 l m i n ei t ) ((pos "I2$" ) (subcat "2A" )))
( "fulminated"  v ( f uh1 l m i n ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "fulminates"  v ( f uh1 l m i n ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "fulminating"  v ( f uh1 l m i n ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "fulmination"  n ( f uh2 l m i n ei1 sh @ n ) ((pos "M6$" ) ))
( "fulminations"  n ( f uh2 l m i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "fulsome"  j ( f u1 l s @ m ) ((pos "OA%" ) ))
( "fulsomely"  a ( f u1 l s @ m l ii ) ((pos "Pu%" ) ))
( "fulsomeness"  n ( f u1 l s @ m n @ s ) ((pos "L@$" ) ))
( "fumble"  v ( f uh1 m b l ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "fumbled"  v ( f uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fumbler"  n ( f uh1 m b l @ r ) ((pos "K6%" ) ))
( "fumblers"  n ( f uh1 m b l @ z ) ((pos "Kj%" ) ))
( "fumbles"  v ( f uh1 m b l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "fumbling"  v ( f uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "fume"  v ( f y uu1 m ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "fumed"  v ( f y uu1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "fumes"  v ( f y uu1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "fumigate"  v ( f y uu1 m i g ei t ) ((pos "H2%" ) (subcat )))
( "fumigated"  v ( f y uu1 m i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "fumigates"  v ( f y uu1 m i g ei t s ) ((pos "Ha%" ) (subcat )))
( "fumigating"  v ( f y uu1 m i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "fumigation"  n ( f y uu2 m i g ei1 sh @ n ) ((pos "K6%" ) ))
( "fumigations"  n ( f y uu2 m i g ei1 sh @ n z ) ((pos "Kj%" ) ))
( "fuming"  v ( f y uu1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "fun"  n ( f uh1 n ) ((pos "L@%" ) ))
( "function"  v ( f uh1 ng k sh @ n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "functional"  j ( f uh1 ng k sh @ n l ) ((pos "OA%" ) ))
( "functionalism"  n ( f uh1 ng k sh @ n @ l i z @ m ) ((pos "L@$" ) ))
( "functionalist"  n ( f uh1 ng k sh @ n @ l i s t ) ((pos "K6$" ) ))
( "functionalists"  n ( f uh1 ng k sh @ n @ l i s t s ) ((pos "Kj$" ) ))
( "functionally"  a ( f uh1 ng k sh @ n @ l ii ) ((pos "Pu%" ) ))
( "functionaries"  n ( f uh1 ng k sh @ n @ r i z ) ((pos "Kj$" ) ))
( "functionary"  n ( f uh1 ng k sh @ n @ r ii ) ((pos "K8$" ) ))
( "functioned"  v ( f uh1 ng k sh @ n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "functioning"  v ( f uh1 ng k sh @ n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "functions"  v ( f uh1 ng k sh @ n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "fund"  v ( f uh1 n d ) ((pos "H0%" "K6%" ) (subcat )))
( "fundamental"  n ( f uh2 n d @ m e1 n t l ) ((pos "K6%" "OA%" ) ))
( "fundamentalism"  n ( f uh2 n d @ m e1 n t @ l i z @ m ) ((pos "L@%" ) ))
( "fundamentalist"  n ( f uh2 n d @ m e1 n t @ l i s t ) ((pos "K6%" ) ))
( "fundamentalists"  n ( f uh2 n d @ m e1 n t @ l i s t s ) ((pos "Kj%" ) ))
( "fundamentally"  a ( f uh2 n d @ m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "fundamentals"  n ( f uh2 n d @ m e1 n t l z ) ((pos "Kj%" ) ))
( "funded"  v ( f uh1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "funding"  v ( f uh1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "funds"  v ( f uh1 n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "funeral"  n ( f y uu1 n @ r @ l ) ((pos "K6%" ) ))
( "funerals"  n ( f y uu1 n @ r @ l z ) ((pos "Kj%" ) ))
( "funereal"  j ( f y uu1 n i@1 r i@ l ) ((pos "OA%" ) ))
( "funfair"  n ( f uh1 n f e@ r ) ((pos "K6%" ) ))
( "funfairs"  n ( f uh1 n f e@ z ) ((pos "Kj%" ) ))
( "fungi"  n ( f uh1 ng g ai ) ((pos "Kj%" ) ))
( "fungicide"  n ( f uh1 n jh i s ai d ) ((pos "M6%" ) ))
( "fungicides"  n ( f uh1 n jh i s ai jh ) ((pos "Mj%" ) ))
( "fungoid"  j ( f uh1 ng g oi d ) ((pos "OA$" ) ))
( "fungous"  j ( f uh1 ng g @ s ) ((pos "OA$" ) ))
( "fungus"  n ( f uh1 ng g @ s ) ((pos "M7%" ) ))
( "funguses"  n ( f uh1 ng g @ s i z ) ((pos "Mj%" ) ))
( "funicular"  n ( f y uu1 n i1 k y u l @ r ) ((pos "K6$" ) ))
( "funiculars"  n ( f y uu1 n i1 k y u l @ z ) ((pos "Kj$" ) ))
( "funk"  v ( f uh1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "funked"  v ( f uh1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "funkier"  j ( f uh1 ng k i@ r ) ((pos "Or$" ) ))
( "funkiest"  j ( f uh1 ng k i i s t ) ((pos "Os$" ) ))
( "funking"  v ( f uh1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "funks"  v ( f uh1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "funky"  j ( f uh1 ng k ii ) ((pos "OD%" ) ))
( "funnel"  v ( f uh1 n l ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "funnelled"  v ( f uh1 n l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "funnelling"  v ( f uh1 n @ l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "funnels"  v ( f uh1 n l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "funnier"  j ( f uh1 n i@ r ) ((pos "Or%" ) ))
( "funniest"  j ( f uh1 n i i s t ) ((pos "Os%" ) ))
( "funnily"  a ( f uh1 n i l ii ) ((pos "Pu%" ) ))
( "funniness"  n ( f uh1 n i n @ s ) ((pos "L@%" ) ))
( "funny"  j ( f uh1 n ii ) ((pos "OD%" ) ))
( "funny-bone"  n ( f uh1 n i - b ou n ) ((pos "K6%" ) ))
( "funny-bones"  n ( f uh1 n i - b ou n z ) ((pos "Kj%" ) ))
( "fur"  n ( f @@1 r ) ((pos "M6%" ) ))
( "furbelow"  n ( f @@1 b i l ou ) ((pos "K6$" ) ))
( "furbelows"  n ( f @@1 b i l ou z ) ((pos "Kj%" ) ))
( "furbish"  v ( f @@1 b i sh ) ((pos "H1%" ) (subcat )))
( "furbished"  v ( f @@1 b i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "furbishes"  v ( f @@1 b i sh i z ) ((pos "Ha%" ) (subcat )))
( "furbishing"  v ( f @@1 b i sh i ng ) ((pos "Hb%" ) (subcat )))
( "furies"  n ( f y u@1 r i z ) ((pos "Mj%" ) ))
( "furious"  j ( f y u@1 r i@ s ) ((pos "OA%" ) ))
( "furiously"  a ( f y u@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "furl"  v ( f @@1 l ) ((pos "J0%" ) (subcat "2A" )))
( "furled"  v ( f @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "furling"  v ( f @@1 l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "furlong"  n ( f @@1 l o ng ) ((pos "K6%" ) ))
( "furlongs"  n ( f @@1 l o ng z ) ((pos "Kj%" ) ))
( "furlough"  n ( f @@1 l ou ) ((pos "M6$" ) ))
( "furloughs"  n ( f @@1 l ou z ) ((pos "Mj$" ) ))
( "furls"  v ( f @@1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "furnace"  n ( f @@1 n i s ) ((pos "K6%" ) ))
( "furnaces"  n ( f @@1 n i s i z ) ((pos "Kj%" ) ))
( "furnish"  v ( f @@1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "furnished"  v ( f @@1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "furnishes"  v ( f @@1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "furnishing"  v ( f @@1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "furnishings"  n ( f @@1 n i sh i ng z ) ((pos "Kj%" ) ))
( "furniture"  n ( f @@1 n i ch @ r ) ((pos "L@%" ) ))
( "furore"  n ( f y u1 r oo1 r ii ) ((pos "K6%" ) ))
( "furores"  n ( f y u1 r oo1 r i z ) ((pos "Kj%" ) ))
( "furrier"  j ( f @@1 r i@ r ) ((pos "Or%" ) ))
( "furrier"  n ( f uh1 r i@ r ) ((pos "K6%" ) ))
( "furriers"  n ( f uh1 r i@ z ) ((pos "Kj%" ) ))
( "furriest"  j ( f @@1 r i i s t ) ((pos "Os%" ) ))
( "furrow"  v ( f uh1 r ou ) ((pos "H0%" "K6%" ) (subcat )))
( "furrowed"  v ( f uh1 r ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "furrowing"  v ( f uh1 r ou i ng ) ((pos "Hb%" ) (subcat )))
( "furrows"  v ( f uh1 r ou z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "furry"  j ( f @@1 r ii ) ((pos "OD%" ) ))
( "furs"  n ( f @@1 z ) ((pos "Mj%" ) ))
( "further"  v ( f @@1 dh @ r ) ((pos "H0%" "Or%" "Pu%" ) (subcat )))
( "furtherance"  n ( f @@1 dh @ r @ n s ) ((pos "L@%" ) ))
( "furthered"  v ( f @@1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "furthering"  v ( f @@1 dh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "furthermore"  a ( f @@2 dh @ m oo1 r ) ((pos "Pu%" ) ))
( "furthermost"  j ( f @@1 dh @ m ou s t ) ((pos "OA%" ) ))
( "furthers"  v ( f @@1 dh @ z ) ((pos "Ha%" ) (subcat )))
( "furthest"  j ( f @@1 dh i s t ) ((pos "OA%" "Pu%" ) ))
( "furtive"  j ( f @@1 t i v ) ((pos "OA%" ) ))
( "furtively"  a ( f @@1 t i v l ii ) ((pos "Pu%" ) ))
( "furtiveness"  n ( f @@1 t i v n @ s ) ((pos "L@%" ) ))
( "fury"  n ( f y u@1 r ii ) ((pos "M8%" ) ))
( "furze"  n ( f @@1 z ) ((pos "L@$" ) ))
( "fuse"  v ( f y uu1 z ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" "15A" )))
( "fused"  v ( f y uu1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "fuselage"  n ( f y uu1 z @ l aa zh ) ((pos "K6%" ) ))
( "fuselages"  n ( f y uu1 z @ l aa zh i z ) ((pos "Kj$" ) ))
( "fuses"  v ( f y uu1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "fusilier"  n ( f y uu2 z @ l i@1 r ) ((pos "K6%" ) ))
( "fusiliers"  n ( f y uu2 z @ l i@1 z ) ((pos "Kj%" ) ))
( "fusillade"  n ( f y uu2 z i l ei1 d ) ((pos "K6$" ) ))
( "fusillades"  n ( f y uu2 z i l ei1 d z ) ((pos "Kj$" ) ))
( "fusing"  v ( f y uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "fusion"  n ( f y uu1 zh n ) ((pos "M6%" ) ))
( "fusions"  n ( f y uu1 zh n z ) ((pos "Mj%" ) ))
( "fuss"  v ( f uh1 s ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" )))
( "fussed"  v ( f uh1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "fusses"  v ( f uh1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "fussier"  j ( f uh1 s i@ r ) ((pos "Or%" ) ))
( "fussiest"  j ( f uh1 s i i s t ) ((pos "Os%" ) ))
( "fussily"  a ( f uh1 s i l ii ) ((pos "Pu%" ) ))
( "fussiness"  n ( f uh1 s i n @ s ) ((pos "L@%" ) ))
( "fussing"  v ( f uh1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "fusspot"  n ( f uh1 s p o t ) ((pos "K6%" ) ))
( "fusspots"  n ( f uh1 s p o t s ) ((pos "Kj%" ) ))
( "fussy"  j ( f uh1 s ii ) ((pos "OD%" ) ))
( "fustian"  n ( f uh1 s t i@ n ) ((pos "L@$" ) ))
( "fusty"  j ( f uh1 s t ii ) ((pos "OA$" ) ))
( "futile"  j ( f y uu1 t ai l ) ((pos "OA%" ) ))
( "futilities"  n ( f y uu1 t i1 l i t i z ) ((pos "Mj%" ) ))
( "futility"  n ( f y uu1 t i1 l i t ii ) ((pos "M8%" ) ))
( "future"  n ( f y uu1 ch @ r ) ((pos "M6%" "OA%" ) ))
( "futureless"  j ( f y uu1 ch @ l @ s ) ((pos "OA%" ) ))
( "futures"  n ( f y uu1 ch @ z ) ((pos "Mj%" ) ))
( "futurism"  n ( f y uu1 ch @ r i z @ m ) ((pos "L@$" ) ))
( "futurist"  n ( f y uu1 ch @ r i s t ) ((pos "K6$" ) ))
( "futurists"  n ( f y uu1 ch @ r i s t s ) ((pos "Kj$" ) ))
( "futurities"  n ( f y uu1 t y u@1 r i t i z ) ((pos "Mj$" ) ))
( "futurity"  n ( f y uu1 t y u@1 r i t ii ) ((pos "M8$" ) ))
( "fuze"  n ( f y uu1 z ) ((pos "K6%" ) ))
( "fuzes"  n ( f y uu1 z i z ) ((pos "Kj%" ) ))
( "fuzz"  n ( f uh1 z ) ((pos "L@%" ) ))
( "fuzzier"  j ( f uh1 z i@ r ) ((pos "Or%" ) ))
( "fuzziest"  j ( f uh1 z i i s t ) ((pos "Os%" ) ))
( "fuzzy"  j ( f uh1 z ii ) ((pos "OD%" ) ))
( "fwd"  n ( f oo1 w @ d ) ((pos "Y~%" ) ))
( "g"  n ( jh ii1 ) ((pos "Ki$" ) ))
( "g's"  n ( jh ii1 z ) ((pos "Kj$" ) ))
( "g^ateau"  n ( g a1 t ou ) ((pos "K6%" ) ))
( "g^ateaus"  n ( g a1 t ou z ) ((pos "Kj%" ) ))
( "gab"  n ( g a1 b ) ((pos "L@%" ) ))
( "gabardine"  n ( g a2 b @ d ii1 n ) ((pos "L@$" ) ))
( "gabble"  v ( g a1 b l ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "gabbled"  v ( g a1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "gabbles"  v ( g a1 b l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "gabbling"  v ( g a1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "gaberdine"  n ( g a2 b @ d ii1 n ) ((pos "L@%" ) ))
( "gable"  n ( g ei1 b l ) ((pos "K6%" ) ))
( "gabled"  j ( g ei1 b l d ) ((pos "OA%" ) ))
( "gables"  n ( g ei1 b l z ) ((pos "Kj%" ) ))
( "gad"  v ( g a1 d ) ((pos "I4$" "W-$" ) (subcat )))
( "gadabout"  n ( g a1 d @ b au t ) ((pos "K6%" ) ))
( "gadabouts"  n ( g a1 d @ b au t s ) ((pos "Kj%" ) ))
( "gadded"  v ( g a1 d i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "gadding"  v ( g a1 d i ng ) ((pos "Ib$" ) (subcat )))
( "gadflies"  n ( g a1 d f l ai z ) ((pos "Kj%" ) ))
( "gadfly"  n ( g a1 d f l ai ) ((pos "K8%" ) ))
( "gadget"  n ( g a1 jh i t ) ((pos "K6%" ) ))
( "gadgetry"  n ( g a1 jh i t r ii ) ((pos "L@%" ) ))
( "gadgets"  n ( g a1 jh i t s ) ((pos "Kj%" ) ))
( "gads"  v ( g a1 d z ) ((pos "Ia$" ) (subcat )))
( "gaff"  n ( g a1 f ) ((pos "K6$" ) ))
( "gaffe"  n ( g a1 f ) ((pos "K6$" ) ))
( "gaffer"  n ( g a1 f @ r ) ((pos "K6%" ) ))
( "gaffers"  n ( g a1 f @ z ) ((pos "Kj%" ) ))
( "gaffes"  n ( g a1 f s ) ((pos "Kj$" ) ))
( "gaffs"  n ( g a1 f s ) ((pos "Kj$" ) ))
( "gag"  v ( g a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "gaga"  j ( g aa1 g aa ) ((pos "OA$" ) ))
( "gage"  v ( g ei1 jh ) ((pos "H2$" "K6$" ) (subcat )))
( "gaged"  v ( g ei1 jh d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "gages"  v ( g ei1 jh i z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "gagged"  v ( g a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "gagging"  v ( g a1 g i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "gaggle"  n ( g a1 g l ) ((pos "K6%" ) ))
( "gaggles"  n ( g a1 g l z ) ((pos "Kj%" ) ))
( "gaging"  v ( g ei1 jh i ng ) ((pos "Hb$" ) (subcat )))
( "gags"  v ( g a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "gaiety"  n ( g ei1 @ t ii ) ((pos "L@%" ) ))
( "gaily"  a ( g ei1 l ii ) ((pos "Pu%" ) ))
( "gain"  v ( g ei1 n ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "13B" )))
( "gained"  v ( g ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "13B" )))
( "gainful"  j ( g ei1 n f @ l ) ((pos "OA%" ) ))
( "gainfully"  a ( g ei1 n f @ l ii ) ((pos "Pu%" ) ))
( "gaining"  v ( g ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "13B" )))
( "gainings"  n ( g ei1 n ii ng z ) ((pos "Kj%" ) ))
( "gains"  v ( g ei1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "13B" )))
( "gainsaid"  v ( g ei2 n s e1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gainsay"  v ( g ei2 n s ei1 ) ((pos "H5%" ) (subcat )))
( "gainsaying"  v ( g ei2 n s ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "gainsays"  v ( g ei2 n s ei1 z ) ((pos "Ha%" ) (subcat )))
( "gait"  n ( g ei1 t ) ((pos "K6%" ) ))
( "gaiter"  n ( g ei1 t @ r ) ((pos "K6%" ) ))
( "gaiters"  n ( g ei1 t @ z ) ((pos "Kj%" ) ))
( "gaits"  n ( g ei1 t s ) ((pos "Kj$" ) ))
( "gal"  n ( g a1 l ) ((pos "K6$" ) ))
( "gala"  n ( g aa1 l @ ) ((pos "K6%" ) ))
( "galactic"  j ( g @1 l a1 k t i k ) ((pos "OA%" ) ))
( "galantine"  n ( g a1 l @ n t ii n ) ((pos "K6$" ) ))
( "galantines"  n ( g a1 l @ n t ii n z ) ((pos "Kj$" ) ))
( "galas"  n ( g aa1 l @ z ) ((pos "Kj%" ) ))
( "galaxies"  n ( g a1 l @ k s i z ) ((pos "Kj%" ) ))
( "galaxy"  n ( g a1 l @ k s ii ) ((pos "K8%" ) ))
( "gale"  n ( g ei1 l ) ((pos "K6%" ) ))
( "gales"  n ( g ei1 l z ) ((pos "Kj%" ) ))
( "gall"  v ( g oo1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "gallant"  j ( g a1 l @ n t ) ((pos "OA%" ) ))
( "gallant"  n ( g @1 l a1 n t ) ((pos "K6$" ) ))
( "gallantly"  a ( g a1 l @ n t l ii ) ((pos "Pu%" ) ))
( "gallantries"  n ( g a1 l @ n t r i z ) ((pos "Mj%" ) ))
( "gallantry"  n ( g a1 l @ n t r ii ) ((pos "M8%" ) ))
( "gallants"  n ( g @1 l a1 n t s ) ((pos "Kj$" ) ))
( "galled"  v ( g oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "galleon"  n ( g a1 l i@ n ) ((pos "K6%" ) ))
( "galleons"  n ( g a1 l i@ n z ) ((pos "Kj%" ) ))
( "galleries"  n ( g a1 l @ r i z ) ((pos "Kj%" ) ))
( "gallery"  n ( g a1 l @ r ii ) ((pos "K8%" ) ))
( "galley"  n ( g a1 l ii ) ((pos "K6%" ) ))
( "galley-proof"  n ( g a1 l i - p r uu f ) ((pos "K6%" ) ))
( "galley-proofs"  n ( g a1 l i - p r uu f s ) ((pos "Kj%" ) ))
( "galley-slave"  n ( g a1 l i - s l ei v ) ((pos "K6%" ) ))
( "galley-slaves"  n ( g a1 l i - s l ei v z ) ((pos "Kj%" ) ))
( "galleys"  n ( g a1 l i z ) ((pos "Kj%" ) ))
( "gallicism"  n ( g a1 l i s i z @ m ) ((pos "K6$" ) ))
( "gallicisms"  n ( g a1 l i s i z @ m z ) ((pos "Kj$" ) ))
( "galling"  v ( g oo1 l i ng ) ((pos "Hb%" ) (subcat )))
( "gallivant"  v ( g a2 l i v a1 n t ) ((pos "I0%" ) (subcat )))
( "gallivanted"  v ( g a2 l i v a1 n t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "gallivanting"  v ( g a2 l i v a1 n t i ng ) ((pos "Ib%" ) (subcat )))
( "gallivants"  v ( g a2 l i v a1 n t s ) ((pos "Ia%" ) (subcat )))
( "gallon"  n ( g a1 l @ n ) ((pos "K6%" ) ))
( "gallons"  n ( g a1 l @ n z ) ((pos "Kj%" ) ))
( "gallop"  v ( g a1 l @ p ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "galloped"  v ( g a1 l @ p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "galloping"  v ( g a1 l @ p i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "gallops"  v ( g a1 l @ p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "gallows"  n ( g a1 l ou z ) ((pos "Kj%" ) ))
( "gallows-bird"  n ( g a1 l ou z - b @@ d ) ((pos "K6$" ) ))
( "gallows-birds"  n ( g a1 l ou z - b @@ d z ) ((pos "Kj$" ) ))
( "galls"  v ( g oo1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "gallstone"  n ( g oo1 l s t ou n ) ((pos "K6%" ) ))
( "gallstones"  n ( g oo1 l s t ou n z ) ((pos "Kj%" ) ))
( "galore"  a ( g @1 l oo1 r ) ((pos "Pu%" ) ))
( "galosh"  n ( g @1 l o1 sh ) ((pos "K7$" ) ))
( "galoshes"  n ( g @1 l o1 sh i z ) ((pos "Kj%" ) ))
( "gals"  n ( g a1 l z ) ((pos "Kj$" ) ))
( "galumph"  v ( g @1 l uh1 m f ) ((pos "I0$" ) ))
( "galumphed"  v ( g @1 l uh1 m f t ) ((pos "Ic$" "Id$" ) ))
( "galumphing"  v ( g @1 l uh1 m f i ng ) ((pos "Ib$" ) ))
( "galumphs"  v ( g @1 l uh1 m f s ) ((pos "Ia$" ) ))
( "galvanic"  j ( g a1 l v a1 n i k ) ((pos "OA$" ) ))
( "galvanism"  n ( g a1 l v @ n i z @ m ) ((pos "L@$" ) ))
( "galvanize"  v ( g a1 l v @ n ai z ) ((pos "H2%" ) (subcat "6A" )))
( "galvanized"  v ( g a1 l v @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "galvanizes"  v ( g a1 l v @ n ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "galvanizing"  v ( g a1 l v @ n ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "gambit"  n ( g a1 m b i t ) ((pos "K6%" ) ))
( "gambits"  n ( g a1 m b i t s ) ((pos "Kj%" ) ))
( "gamble"  v ( g a1 m b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "gambled"  v ( g a1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "gambler"  n ( g a1 m b l @ r ) ((pos "K6%" ) ))
( "gamblers"  n ( g a1 m b l @ z ) ((pos "Kj%" ) ))
( "gambles"  v ( g a1 m b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "gambling"  v ( g a1 m b l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" )))
( "gambling-den"  n ( g a1 m b l i ng - d e n ) ((pos "K6%" ) ))
( "gambling-dens"  n ( g a1 m b l i ng - d e n z ) ((pos "Kj%" ) ))
( "gamboge"  n ( g a1 m b uu1 zh ) ((pos "L@$" ) ))
( "gambol"  v ( g a1 m b l ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "gambolled"  v ( g a1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gambolling"  v ( g a1 m b l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gambols"  v ( g a1 m b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "game"  v ( g ei1 m ) ((pos "J2$" "M6%" "OA%" ) (subcat "2A" "2C" )))
( "game-bag"  n ( g ei1 m - b a g ) ((pos "K6$" ) ))
( "game-bags"  n ( g ei1 m - b a g z ) ((pos "Kj$" ) ))
( "game-bird"  n ( g ei1 m - b @@ d ) ((pos "K6%" ) ))
( "game-birds"  n ( g ei1 m - b @@ d z ) ((pos "Kj%" ) ))
( "game-licence"  n ( g ei1 m - l ai s n s ) ((pos "K6%" ) ))
( "game-licences"  n ( g ei1 m - l ai s n s i z ) ((pos "Kj%" ) ))
( "gamecock"  n ( g ei1 m k o k ) ((pos "K6$" ) ))
( "gamecocks"  n ( g ei1 m k o k s ) ((pos "Kj$" ) ))
( "gamed"  v ( g ei1 m d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" )))
( "gamekeeper"  n ( g ei1 m k ii p @ r ) ((pos "K6%" ) ))
( "gamekeepers"  n ( g ei1 m k ii p @ z ) ((pos "Kj%" ) ))
( "gamely"  a ( g ei1 m l ii ) ((pos "Pu%" ) ))
( "games"  v ( g ei1 m z ) ((pos "Ja$" "Mj%" ) (subcat "2A" "2C" )))
( "games-master"  n ( g ei1 m z - m aa s t @ r ) ((pos "K6%" ) ))
( "games-masters"  n ( g ei1 m z - m aa s t @ z ) ((pos "Kj%" ) ))
( "games-mistress"  n ( g ei1 m z - m i s t r i s ) ((pos "K7%" ) ))
( "games-mistresses"  n ( g ei1 m z - m i s t r i s i z ) ((pos "Kj%" ) ))
( "gamesmanship"  n ( g ei1 m z m @ n sh i p ) ((pos "L@%" ) ))
( "gamier"  j ( g ei1 m i@ r ) ((pos "Or$" ) ))
( "gamiest"  j ( g ei1 m i i s t ) ((pos "Os$" ) ))
( "gaming"  v ( g ei1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "gaming-house"  n ( g ei1 m i ng - h au s ) ((pos "K6%" ) ))
( "gaming-houses"  n ( g ei1 m i ng - h au z i z ) ((pos "Kj%" ) ))
( "gaming-rooms"  n ( g ei1 m i ng - r uu m z ) ((pos "Kj%" ) ))
( "gaming-table"  n ( g ei1 m i ng - t ei b l ) ((pos "K6%" ) ))
( "gaming-tables"  n ( g ei1 m i ng - t ei b l z ) ((pos "Kj%" ) ))
( "gamma"  n ( g a1 m @ ) ((pos "K6$" ) ))
( "gamma-rays"  n ( g a1 m @ - r ei z ) ((pos "Kj%" ) ))
( "gammas"  n ( g a1 m @ z ) ((pos "Kj$" ) ))
( "gammon"  n ( g a1 m @ n ) ((pos "M6%" ) ))
( "gammons"  n ( g a1 m @ n z ) ((pos "Mj$" ) ))
( "gammy"  j ( g a1 m ii ) ((pos "OA$" ) ))
( "gamp"  n ( g a1 m p ) ((pos "K6$" ) ))
( "gamps"  n ( g a1 m p s ) ((pos "Kj$" ) ))
( "gamut"  n ( g a1 m @ t ) ((pos "K6%" ) ))
( "gamuts"  n ( g a1 m @ t s ) ((pos "Kj$" ) ))
( "gamy"  j ( g ei1 m ii ) ((pos "OD$" ) ))
( "gander"  n ( g a1 n d @ r ) ((pos "K6%" ) ))
( "ganders"  n ( g a1 n d @ z ) ((pos "Kj%" ) ))
( "gang"  v ( g a1 ng ) ((pos "I0%" "K6%" ) (subcat )))
( "ganged"  v ( g a1 ng d ) ((pos "Ic%" "Id%" ) (subcat )))
( "ganger"  n ( g a1 ng @ r ) ((pos "K6%" ) ))
( "gangers"  n ( g a1 ng @ z ) ((pos "Kj%" ) ))
( "ganging"  v ( g a1 ng i ng ) ((pos "Ib%" ) (subcat )))
( "ganglia"  n ( g a1 ng g l i@ ) ((pos "Kj$" ) ))
( "gangling"  j ( g a1 ng g l i ng ) ((pos "OA%" ) ))
( "ganglion"  n ( g a1 ng g l i@ n ) ((pos "K6$" ) ))
( "ganglions"  n ( g a1 ng g l i@ n z ) ((pos "Kj$" ) ))
( "gangplank"  n ( g a1 ng p l a ng k ) ((pos "K6%" ) ))
( "gangplanks"  n ( g a1 ng p l a ng k s ) ((pos "Kj%" ) ))
( "gangrene"  v ( g a1 ng g r ii n ) ((pos "J2$" "L@%" ) (subcat "2A" )))
( "gangrened"  v ( g a1 ng g r ii n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "gangrenes"  v ( g a1 ng g r ii n z ) ((pos "Ja$" ) (subcat "2A" )))
( "gangrening"  v ( g a1 ng g r ii n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "gangrenous"  j ( g a1 ng g r i n @ s ) ((pos "OA%" ) ))
( "gangs"  v ( g a1 ng z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "gangster"  n ( g a1 ng s t @ r ) ((pos "K6%" ) ))
( "gangsters"  n ( g a1 ng s t @ z ) ((pos "Kj%" ) ))
( "gangway"  n ( g a1 ng w ei ) ((pos "K6%" "W-%" ) ))
( "gangways"  n ( g a1 ng w ei z ) ((pos "Kj%" ) ))
( "gannet"  n ( g a1 n i t ) ((pos "K6%" ) ))
( "gannets"  n ( g a1 n i t s ) ((pos "Kj%" ) ))
( "gantries"  n ( g a1 n t r i z ) ((pos "Kj%" ) ))
( "gantry"  n ( g a1 n t r ii ) ((pos "K8%" ) ))
( "gaol"  v ( jh ei1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "gaolbird"  n ( jh ei1 l b @@ d ) ((pos "K6%" ) ))
( "gaolbirds"  n ( jh ei1 l b @@ d z ) ((pos "Kj%" ) ))
( "gaolbreak"  n ( jh ei1 l b r ei k ) ((pos "K6%" ) ))
( "gaolbreaks"  n ( jh ei1 l b r ei k s ) ((pos "Kj%" ) ))
( "gaoled"  v ( jh ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gaoler"  n ( jh ei1 l @ r ) ((pos "K6%" ) ))
( "gaolers"  n ( jh ei1 l @ z ) ((pos "Kj%" ) ))
( "gaoling"  v ( jh ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "gaols"  v ( jh ei1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "gap"  n ( g a1 p ) ((pos "K6%" ) ))
( "gap-toothed"  j ( g a1 p - t uu th t ) ((pos "OA%" ) ))
( "gape"  v ( g ei1 p ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "gaped"  v ( g ei1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gapes"  v ( g ei1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "gaping"  v ( g ei1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gaps"  n ( g a1 p s ) ((pos "Kj%" ) ))
( "garage"  v ( g a1 r aa zh ) ((pos "H2%" "K6%" ) (subcat )))
( "garaged"  v ( g a1 r aa zh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "garages"  v ( g a1 r aa zh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "garaging"  v ( g a1 r aa zh i ng ) ((pos "Hb%" ) (subcat )))
( "garb"  v ( g aa1 b ) ((pos "H0$" "L@%" ) (subcat )))
( "garbage"  n ( g aa1 b i jh ) ((pos "L@%" ) ))
( "garbage-can"  n ( g aa1 b i jh - k a n ) ((pos "K6%" ) ))
( "garbage-cans"  n ( g aa1 b i jh - k a n z ) ((pos "Kj%" ) ))
( "garbed"  v ( g aa1 b d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "garbing"  v ( g aa1 b i ng ) ((pos "Hb$" ) (subcat )))
( "garble"  v ( g aa1 b l ) ((pos "H2$" ) (subcat )))
( "garbled"  v ( g aa1 b l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "garbles"  v ( g aa1 b l z ) ((pos "Ha$" ) (subcat )))
( "garbling"  v ( g aa1 b l i ng ) ((pos "Hb$" ) (subcat )))
( "garbs"  v ( g aa1 b z ) ((pos "Ha$" ) (subcat )))
( "garden"  v ( g aa1 d n ) ((pos "I0%" "M6%" ) (subcat )))
( "garden-truck"  n ( g aa2 d n - t r uh1 k ) ((pos "K6$" ) ))
( "garden-trucks"  n ( g aa2 d n - t r uh1 k s ) ((pos "Kj$" ) ))
( "gardened"  v ( g aa1 d n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "gardener"  n ( g aa1 d n @ r ) ((pos "K6%" ) ))
( "gardeners"  n ( g aa1 d n @ z ) ((pos "Kj%" ) ))
( "gardenia"  n ( g aa1 d ii1 n i@ ) ((pos "K6%" ) ))
( "gardenias"  n ( g aa1 d ii1 n i@ z ) ((pos "Kj%" ) ))
( "gardening"  v ( g aa1 d n i ng ) ((pos "Ib%" "L@%" ) (subcat )))
( "gardens"  v ( g aa1 d n z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "gargantuan"  j ( g aa1 g a1 n t y u@ n ) ((pos "OA%" ) ))
( "gargle"  v ( g aa1 g l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "gargled"  v ( g aa1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "gargles"  v ( g aa1 g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "gargling"  v ( g aa1 g l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "gargoyle"  n ( g aa1 g oi l ) ((pos "K6%" ) ))
( "gargoyles"  n ( g aa1 g oi l z ) ((pos "Kj%" ) ))
( "garish"  j ( g e@1 r i sh ) ((pos "OA%" ) ))
( "garishly"  a ( g e@1 r i sh l ii ) ((pos "Pu%" ) ))
( "garland"  v ( g aa1 l @ n d ) ((pos "H0%" "K6%" ) (subcat )))
( "garlanded"  v ( g aa1 l @ n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "garlanding"  v ( g aa1 l @ n d i ng ) ((pos "Hb%" ) (subcat )))
( "garlands"  v ( g aa1 l @ n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "garlic"  n ( g aa1 l i k ) ((pos "L@%" ) ))
( "garment"  n ( g aa1 m @ n t ) ((pos "K6%" ) ))
( "garments"  n ( g aa1 m @ n t s ) ((pos "Kj%" ) ))
( "garner"  v ( g aa1 n @ r ) ((pos "H0%" "K6$" ) (subcat "6A" )))
( "garnered"  v ( g aa1 n @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "garnering"  v ( g aa1 n @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "garners"  v ( g aa1 n @ z ) ((pos "Ha%" "Kj$" ) (subcat "6A" )))
( "garnet"  n ( g aa1 n i t ) ((pos "K6%" ) ))
( "garnets"  n ( g aa1 n i t s ) ((pos "Kj%" ) ))
( "garnish"  v ( g aa1 n i sh ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "garnished"  v ( g aa1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "garnishes"  v ( g aa1 n i sh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "garnishing"  v ( g aa1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "garotte"  v ( g @1 r o1 t ) ((pos "H2%" "K6%" ) (subcat )))
( "garotted"  v ( g @1 r o1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "garottes"  v ( g @1 r o1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "garotting"  v ( g @1 r o1 t i ng ) ((pos "Hb%" ) (subcat )))
( "garret"  n ( g a1 r @ t ) ((pos "K6%" ) ))
( "garrets"  n ( g a1 r @ t s ) ((pos "Kj%" ) ))
( "garrison"  v ( g a1 r i s n ) ((pos "H0$" "K6%" ) (subcat )))
( "garrisoned"  v ( g a1 r i s n d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "garrisoning"  v ( g a1 r i s n i ng ) ((pos "Hb$" ) (subcat )))
( "garrisons"  v ( g a1 r i s n z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "garrotte"  v ( g @1 r o1 t ) ((pos "H2$" "K6$" ) (subcat )))
( "garrotted"  v ( g @1 r o1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "garrottes"  v ( g @1 r o1 t s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "garrotting"  v ( g @1 r o1 t i ng ) ((pos "Hb$" ) (subcat )))
( "garrulity"  n ( g @1 r uu1 l i t ii ) ((pos "L@$" ) ))
( "garrulous"  j ( g a1 r @ l @ s ) ((pos "OA%" ) ))
( "garter"  n ( g aa1 t @ r ) ((pos "K6%" ) ))
( "garters"  n ( g aa1 t @ z ) ((pos "Kj%" ) ))
( "gas"  v ( g a1 s ) ((pos "J5%" "M7%" ) (subcat "2A" "2C" )))
( "gas-bracket"  n ( g a1 s - b r a k i t ) ((pos "K6%" ) ))
( "gas-brackets"  n ( g a1 s - b r a k i t s ) ((pos "Kj%" ) ))
( "gas-cooker"  n ( g a1 s - k u k @ r ) ((pos "K6%" ) ))
( "gas-cookers"  n ( g a1 s - k u k @ z ) ((pos "Kj%" ) ))
( "gas-engine"  n ( g a1 s - e n jh i n ) ((pos "K6$" ) ))
( "gas-engines"  n ( g a1 s - e n jh i n z ) ((pos "Kj$" ) ))
( "gas-fitter"  n ( g a1 s - f i t @ r ) ((pos "K6%" ) ))
( "gas-fitters"  n ( g a1 s - f i t @ z ) ((pos "Kj%" ) ))
( "gas-fittings"  n ( g a1 s - f i t i ng z ) ((pos "Kj%" ) ))
( "gas-helmet"  n ( g a1 s - h e l m i t ) ((pos "K6$" ) ))
( "gas-helmets"  n ( g a1 s - h e l m i t s ) ((pos "Kj$" ) ))
( "gas-holder"  n ( g a1 s - h ou l d @ r ) ((pos "K6%" ) ))
( "gas-holders"  n ( g a1 s - h ou l d @ z ) ((pos "Kj%" ) ))
( "gas-mask"  n ( g a1 s - m aa s k ) ((pos "K6%" ) ))
( "gas-masks"  n ( g a1 s - m aa s k s ) ((pos "Kj%" ) ))
( "gas-meter"  n ( g a1 s - m ii t @ r ) ((pos "K6%" ) ))
( "gas-meters"  n ( g a1 s - m ii t @ z ) ((pos "Kj%" ) ))
( "gas-oven"  n ( g a1 s - uh v n ) ((pos "K6%" ) ))
( "gas-ovens"  n ( g a1 s - uh v n z ) ((pos "Kj%" ) ))
( "gas-ring"  n ( g a1 s - r i ng ) ((pos "K6%" ) ))
( "gas-rings"  n ( g a1 s - r i ng z ) ((pos "Kj%" ) ))
( "gas-station"  n ( g a1 s - s t ei sh @ n ) ((pos "K6%" ) ))
( "gas-stations"  n ( g a1 s - s t ei sh @ n z ) ((pos "Kj%" ) ))
( "gas-stove"  n ( g a1 s - s t ou v ) ((pos "K6%" ) ))
( "gas-stoves"  n ( g a1 s - s t ou v z ) ((pos "Kj%" ) ))
( "gasbag"  n ( g a1 s b a g ) ((pos "K6%" ) ))
( "gasbags"  n ( g a1 s b a g z ) ((pos "Kj%" ) ))
( "gaseous"  j ( g a1 s i@ s ) ((pos "OA%" ) ))
( "gases"  n ( g a1 s i z ) ((pos "Mj%" ) ))
( "gash"  v ( g a1 sh ) ((pos "H1%" "K7%" ) (subcat )))
( "gashed"  v ( g a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gashes"  v ( g a1 sh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "gashing"  v ( g a1 sh i ng ) ((pos "Hb%" ) (subcat )))
( "gasification"  n ( g a2 s i f i k ei1 sh @ n ) ((pos "M6$" ) ))
( "gasifications"  n ( g a2 s i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "gasified"  v ( g a1 s i f ai d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "gasifies"  v ( g a1 s i f ai z ) ((pos "Ja$" ) (subcat "2A" )))
( "gasify"  v ( g a1 s i f ai ) ((pos "J3$" ) (subcat "2A" )))
( "gasifying"  v ( g a1 s i f ai i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "gasket"  n ( g a1 s k i t ) ((pos "K6%" ) ))
( "gaskets"  n ( g a1 s k i t s ) ((pos "Kj%" ) ))
( "gaslight"  n ( g a1 s l ai t ) ((pos "L@%" ) ))
( "gasmask"  n ( g a1 s m aa s k ) ((pos "K6%" ) ))
( "gasmasks"  n ( g a1 s m aa s k s ) ((pos "Kj%" ) ))
( "gasolene"  n ( g a1 s @ l ii n ) ((pos "L@$" ) ))
( "gasoline"  n ( g a1 s @ l ii n ) ((pos "L@%" ) ))
( "gasometer"  n ( g @1 s o1 m i t @ r ) ((pos "K6%" ) ))
( "gasometers"  n ( g @1 s o1 m i t @ z ) ((pos "Kj%" ) ))
( "gasp"  v ( g aa1 s p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "gasped"  v ( g aa1 s p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "gasping"  v ( g aa1 s p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "gasps"  v ( g aa1 s p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "gassed"  v ( g a1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "gasses"  v ( g a1 s i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "gassier"  j ( g a1 s i@ r ) ((pos "Or%" ) ))
( "gassiest"  j ( g a1 s i i s t ) ((pos "Os%" ) ))
( "gassing"  v ( g a1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "gassy"  j ( g a1 s ii ) ((pos "OD%" ) ))
( "gastric"  j ( g a1 s t r i k ) ((pos "OA%" ) ))
( "gastritis"  n ( g a1 s t r ai1 t i s ) ((pos "L@%" ) ))
( "gastronomic"  j ( g a2 s t r @ n o1 m i k ) ((pos "OA%" ) ))
( "gastronomy"  n ( g a1 s t r o1 n @ m ii ) ((pos "L@%" ) ))
( "gasworks"  n ( g a1 s w @@ k s ) ((pos "K9%" ) ))
( "gate"  v ( g ei1 t ) ((pos "H2$" "K6%" ) (subcat )))
( "gateaux"  n ( g a1 t ou z ) ((pos "Kj%" ) ))
( "gatecrash"  v ( g ei1 t k r a sh ) ((pos "H1%" ) (subcat )))
( "gatecrashed"  v ( g ei1 t k r a sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gatecrasher"  n ( g ei1 t k r a sh @ r ) ((pos "K6%" ) ))
( "gatecrashers"  n ( g ei1 t k r a sh @ z ) ((pos "Kj%" ) ))
( "gatecrashes"  v ( g ei1 t k r a sh i z ) ((pos "Ha%" ) (subcat )))
( "gatecrashing"  v ( g ei1 t k r a sh i ng ) ((pos "Hb%" ) (subcat )))
( "gated"  v ( g ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gatehouse"  n ( g ei1 t h au s ) ((pos "K6%" ) ))
( "gatehouses"  n ( g ei1 t h au z i z ) ((pos "Kj%" ) ))
( "gatepost"  n ( g ei1 t p ou s t ) ((pos "K6%" ) ))
( "gateposts"  n ( g ei1 t p ou s t s ) ((pos "Kj%" ) ))
( "gates"  v ( g ei1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "gateway"  n ( g ei1 t w ei ) ((pos "K6%" ) ))
( "gateways"  n ( g ei1 t w ei z ) ((pos "Kj%" ) ))
( "gather"  v ( g a1 dh @ r ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "9" "12B" "13B" "15A" )))
( "gathered"  v ( g a1 dh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" "12B" "13B" "15A" )))
( "gatherer"  n ( g a1 dh @ r @ r ) ((pos "K6%" ) ))
( "gatherers"  n ( g a1 dh @ r @ z ) ((pos "Kj%" ) ))
( "gathering"  v ( g a1 dh @ r i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "9" "12B" "13B" "15A" )))
( "gatherings"  n ( g a1 dh @ r i ng z ) ((pos "Kj%" ) ))
( "gathers"  v ( g a1 dh @ z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "9" "12B" "13B" "15A" )))
( "gating"  v ( g ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "gauche"  j ( g ou1 sh ) ((pos "OA%" ) ))
( "gaucherie"  n ( g ou1 sh @ r ii ) ((pos "M6$" ) ))
( "gaucheries"  n ( g ou1 sh @ r i z ) ((pos "Mj$" ) ))
( "gaucho"  n ( g au1 ch ou ) ((pos "K6$" ) ))
( "gauchos"  n ( g au1 ch ou z ) ((pos "Kj$" ) ))
( "gaud"  n ( g oo1 d ) ((pos "K6$" ) ))
( "gaudier"  j ( g oo1 d i@ r ) ((pos "Or%" ) ))
( "gaudies"  n ( g oo1 d i z ) ((pos "Kj$" ) ))
( "gaudiest"  j ( g oo1 d i i s t ) ((pos "Os%" ) ))
( "gaudily"  a ( g oo1 d i l ii ) ((pos "Pu%" ) ))
( "gauds"  n ( g oo1 d z ) ((pos "Kj$" ) ))
( "gaudy"  n ( g oo1 d ii ) ((pos "K8%" "OD%" ) ))
( "gauge"  v ( g ei1 jh ) ((pos "H2%" "M6%" ) (subcat )))
( "gauged"  v ( g ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gauges"  v ( g ei1 jh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "gauging"  v ( g ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "gaunt"  j ( g oo1 n t ) ((pos "OA%" ) ))
( "gauntlet"  n ( g oo1 n t l i t ) ((pos "K6%" ) ))
( "gauntlets"  n ( g oo1 n t l i t s ) ((pos "Kj%" ) ))
( "gauntness"  n ( g oo1 n t n @ s ) ((pos "L@%" ) ))
( "gauze"  n ( g oo1 z ) ((pos "L@%" ) ))
( "gauzier"  j ( g oo1 z i@ r ) ((pos "Or$" ) ))
( "gauziest"  j ( g oo1 z i i s t ) ((pos "Os$" ) ))
( "gauzy"  j ( g oo1 z ii ) ((pos "OD%" ) ))
( "gave"  v ( g ei1 v ) ((pos "Jc*" ) (subcat "2A" "2C" "3A" "6A" "12A" "12B" "13A" "15B" )))
( "gavel"  n ( g a1 v @ l ) ((pos "K6%" ) ))
( "gavels"  n ( g a1 v @ l z ) ((pos "Kj%" ) ))
( "gavotte"  n ( g @1 v o1 t ) ((pos "K6%" ) ))
( "gavottes"  n ( g @1 v o1 t s ) ((pos "Kj%" ) ))
( "gawk"  n ( g oo1 k ) ((pos "K6%" ) ))
( "gawkier"  j ( g oo1 k i@ r ) ((pos "Or%" ) ))
( "gawkiest"  j ( g oo1 k i i s t ) ((pos "Os%" ) ))
( "gawkiness"  n ( g oo1 k i n @ s ) ((pos "L@%" ) ))
( "gawks"  n ( g oo1 k s ) ((pos "Kj%" ) ))
( "gawky"  j ( g oo1 k ii ) ((pos "OD%" ) ))
( "gawp"  v ( g oo1 p ) ((pos "I0%" ) (subcat "2A" )))
( "gawped"  v ( g oo1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gawping"  v ( g oo1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gawps"  v ( g oo1 p s ) ((pos "Ia%" ) (subcat "2A" )))
( "gay"  n ( g ei1 ) ((pos "K6%" "OC%" ) ))
( "gayer"  j ( g ei1 @ r ) ((pos "Or%" ) ))
( "gayest"  j ( g ei1 i s t ) ((pos "Os%" ) ))
( "gayness"  n ( g ei1 n @ s ) ((pos "L@%" ) ))
( "gays"  n ( g ei1 z ) ((pos "Kj%" ) ))
( "gaze"  v ( g ei1 z ) ((pos "I2%" "Ki%" ) (subcat "2A" "2C" )))
( "gazed"  v ( g ei1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "gazelle"  n ( g @1 z e1 l ) ((pos "K6%" ) ))
( "gazelles"  n ( g @1 z e1 l z ) ((pos "Kj%" ) ))
( "gazes"  v ( g ei1 z i z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "gazette"  v ( g @1 z e1 t ) ((pos "H2$" "K6%" ) ))
( "gazetted"  v ( g @1 z e1 t i d ) ((pos "Hc$" "Hd%" ) ))
( "gazetteer"  n ( g a2 z @ t i@1 r ) ((pos "K6%" ) ))
( "gazetteers"  n ( g a2 z @ t i@1 z ) ((pos "Kj%" ) ))
( "gazettes"  v ( g @1 z e1 t s ) ((pos "Ha$" "Kj%" ) ))
( "gazetting"  v ( g @1 z e1 t i ng ) ((pos "Hb$" ) ))
( "gazing"  v ( g ei1 z i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "gazump"  v ( g @1 z uh1 m p ) ((pos "J0%" ) (subcat "2A" )))
( "gazumped"  v ( g @1 z uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "gazumping"  v ( g @1 z uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "gazumps"  v ( g @1 z uh1 m p s ) ((pos "Ja%" ) (subcat "2A" )))
( "gear"  v ( g i@1 r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "14" )))
( "gear-case"  n ( g i@1 - k ei s ) ((pos "K6$" ) ))
( "gear-cases"  n ( g i@1 - k ei s i z ) ((pos "Kj$" ) ))
( "gearbox"  n ( g i@1 b o k s ) ((pos "K7%" ) ))
( "gearboxes"  n ( g i@1 b o k s i z ) ((pos "Kj%" ) ))
( "geared"  v ( g i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "14" )))
( "gearing"  v ( g i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "14" )))
( "gears"  v ( g i@1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "14" )))
( "gearshift"  n ( g i@1 sh i f t ) ((pos "K6%" ) ))
( "gearshifts"  n ( g i@1 sh i f t s ) ((pos "Kj%" ) ))
( "gecko"  n ( g e1 k ou ) ((pos "K6$" ) ))
( "geckoes"  n ( g e1 k ou z ) ((pos "Kj$" ) ))
( "geckos"  n ( g e1 k ou z ) ((pos "Kj$" ) ))
( "gee"  uh ( jh ii1 ) ((pos "W-%" ) ))
( "gee whiz"  uh ( jh ii2 - w i1 z ) ((pos "W-%" ) ))
( "gee-gee"  n ( jh ii1 - jh ii ) ((pos "K6%" ) ))
( "gee-gees"  n ( jh ii1 - jh ii z ) ((pos "Kj%" ) ))
( "gee-up"  uh ( jh ii2 - uh1 p ) ((pos "W-%" ) ))
( "geese"  n ( g ii1 s ) ((pos "Kj%" ) ))
( "geezer"  n ( g ii1 z @ r ) ((pos "K6%" ) ))
( "geezers"  n ( g ii1 z @ z ) ((pos "Kj%" ) ))
( "geisha"  n ( g ei1 sh @ ) ((pos "K6%" ) ))
( "geishas"  n ( g ei1 sh @ z ) ((pos "Kj%" ) ))
( "gel"  v ( jh e1 l ) ((pos "I4%" "K6%" ) ))
( "gelatin"  n ( jh e1 l @ t i n ) ((pos "L@$" ) ))
( "gelatine"  n ( jh e1 l @ t ii n ) ((pos "L@%" ) ))
( "gelatinous"  j ( jh i1 l a1 t i n @ s ) ((pos "OA%" ) ))
( "geld"  v ( g e1 l d ) ((pos "H0$" ) (subcat )))
( "gelded"  v ( g e1 l d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "gelding"  v ( g e1 l d i ng ) ((pos "Hb$" "K6%" ) (subcat )))
( "geldings"  n ( g e1 l d i ng z ) ((pos "Kj%" ) ))
( "gelds"  v ( g e1 l d z ) ((pos "Ha$" ) (subcat )))
( "gelignite"  n ( jh e1 l i g n ai t ) ((pos "L@%" ) ))
( "gelled"  v ( jh e1 l d ) ((pos "Ic%" "Id%" ) ))
( "gelling"  v ( jh e1 l i ng ) ((pos "Ib%" ) ))
( "gels"  v ( jh e1 l z ) ((pos "Ia%" "Kj%" ) ))
( "gem"  n ( jh e1 m ) ((pos "K6%" ) ))
( "gemmed"  j ( jh e1 m d ) ((pos "OA%" ) ))
( "gems"  n ( jh e1 m z ) ((pos "Kj%" ) ))
( "gen"  v ( jh e1 n ) ((pos "H4%" "L@%" ) (subcat )))
( "gendarme"  n ( zh o1 n d aa m ) ((pos "K6%" ) ))
( "gendarmerie"  n ( zh o1 n d aa1 m @ r ii ) ((pos "K6$" ) ))
( "gendarmeries"  n ( zh o1 n d aa1 m @ r i z ) ((pos "Kj$" ) ))
( "gendarmes"  n ( zh o1 n d aa m z ) ((pos "Kj%" ) ))
( "gender"  n ( jh e1 n d @ r ) ((pos "K6%" ) ))
( "genders"  n ( jh e1 n d @ z ) ((pos "Kj%" ) ))
( "gene"  n ( jh ii1 n ) ((pos "K6%" ) ))
( "genealogical"  j ( jh ii2 n i@ l o1 jh i k l ) ((pos "OA%" ) ))
( "genealogically"  a ( jh ii2 n i@ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "genealogies"  n ( jh ii2 n i a1 l @ jh i z ) ((pos "Mj%" ) ))
( "genealogist"  n ( jh ii2 n i a1 l @ jh i s t ) ((pos "K6%" ) ))
( "genealogists"  n ( jh ii2 n i a1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "genealogy"  n ( jh ii2 n i a1 l @ jh ii ) ((pos "M8%" ) ))
( "genera"  n ( jh e1 n @ r @ ) ((pos "Kj$" ) ))
( "general"  n ( jh e1 n r @ l ) ((pos "K6%" "OA%" ) ))
( "generalissimo"  n ( jh e2 n r @ l i1 s i m ou ) ((pos "K6$" ) ))
( "generalissimos"  n ( jh e2 n r @ l i1 s i m ou z ) ((pos "Kj$" ) ))
( "generalities"  n ( jh e2 n @ r a1 l i t i z ) ((pos "Mj%" ) ))
( "generality"  n ( jh e2 n @ r a1 l i t ii ) ((pos "M8%" ) ))
( "generalization"  n ( jh e2 n r @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "generalizations"  n ( jh e2 n r @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "generalize"  v ( jh e1 n r @ l ai z ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "generalized"  v ( jh e1 n r @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "generalizes"  v ( jh e1 n r @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "generalizing"  v ( jh e1 n r @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "generally"  a ( jh e1 n r @ l ii ) ((pos "Pu%" ) ))
( "generals"  n ( jh e1 n r @ l z ) ((pos "Kj%" ) ))
( "generate"  v ( jh e1 n @ r ei t ) ((pos "H2%" ) (subcat )))
( "generated"  v ( jh e1 n @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "generates"  v ( jh e1 n @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "generating"  v ( jh e1 n @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "generation"  n ( jh e2 n @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "generations"  n ( jh e2 n @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "generative"  j ( jh e1 n @ r @ t i v ) ((pos "OA%" ) ))
( "generator"  n ( jh e1 n @ r ei t @ r ) ((pos "K6%" ) ))
( "generators"  n ( jh e1 n @ r ei t @ z ) ((pos "Kj%" ) ))
( "generic"  j ( jh i1 n e1 r i k ) ((pos "OA%" ) ))
( "generically"  a ( jh i1 n e1 r i k l ii ) ((pos "Pu%" ) ))
( "generosities"  n ( jh e2 n @ r o1 s i t i z ) ((pos "Mj$" ) ))
( "generosity"  n ( jh e2 n @ r o1 s i t ii ) ((pos "M8%" ) ))
( "generous"  j ( jh e1 n @ r @ s ) ((pos "OA%" ) ))
( "generously"  a ( jh e1 n @ r @ s l ii ) ((pos "Pu%" ) ))
( "genes"  n ( jh ii1 n z ) ((pos "Kj%" ) ))
( "genesis"  n ( jh e1 n @ s i s ) ((pos "Ki%" ) ))
( "genetic"  j ( jh i1 n e1 t i k ) ((pos "OA%" ) ))
( "genetically"  a ( jh i1 n e1 t i k l ii ) ((pos "Pu%" ) ))
( "geneticist"  n ( jh i1 n e1 t i s i s t ) ((pos "K6%" ) ))
( "geneticists"  n ( jh i1 n e1 t i s i s t s ) ((pos "Kj%" ) ))
( "genetics"  n ( jh i1 n e1 t i k s ) ((pos "Lk%" ) ))
( "genial"  j ( jh ii1 n i@ l ) ((pos "OA%" ) ))
( "genialities"  n ( jh ii2 n i a1 l i t i z ) ((pos "Mj$" ) ))
( "geniality"  n ( jh ii2 n i a1 l i t ii ) ((pos "M8%" ) ))
( "genially"  a ( jh ii1 n i@ l ii ) ((pos "Pu%" ) ))
( "genie"  n ( jh ii1 n ii ) ((pos "K6%" ) ))
( "genies"  n ( jh ii1 n i z ) ((pos "Kj%" ) ))
( "genii"  n ( jh ii1 n i ai ) ((pos "Kj$" ) ))
( "genital"  j ( jh e1 n i t l ) ((pos "OA%" ) ))
( "genitals"  n ( jh e1 n i t l z ) ((pos "Kj%" ) ))
( "genitive"  j ( jh e1 n @ t i v ) ((pos "OA%" ) ))
( "genius"  n ( jh ii1 n i@ s ) ((pos "M7%" ) ))
( "genius loci"  n ( jh ii2 n i@ s - l ou1 s ai ) ((pos "Ki$" ) ))
( "geniuses"  n ( jh ii1 n i@ s i z ) ((pos "Mj%" ) ))
( "genned"  v ( jh e1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "genning"  v ( jh e1 n i ng ) ((pos "Hb$" ) (subcat )))
( "genocide"  n ( jh e1 n @ s ai d ) ((pos "L@%" ) ))
( "genre"  n ( zh aa1 n r @ ) ((pos "K6%" ) ))
( "genre-painting"  n ( zh aa1 n r @ - p ei n t i ng ) ((pos "K6$" ) ))
( "genre-paintings"  n ( zh aa1 n r @ - p ei n t i ng z ) ((pos "Kj$" ) ))
( "genres"  n ( zh aa1 n r @ z ) ((pos "Kj%" ) ))
( "gens"  v ( jh e1 n z ) ((pos "Ha$" ) (subcat )))
( "gent"  n ( jh e1 n t ) ((pos "K6%" ) ))
( "genteel"  j ( jh e1 n t ii1 l ) ((pos "OA%" ) ))
( "genteelly"  a ( jh e1 n t ii1 @ l ii ) ((pos "Pu%" ) ))
( "gentian"  n ( jh e1 n sh @ n ) ((pos "K6$" ) ))
( "gentians"  n ( jh e1 n sh @ n z ) ((pos "Kj$" ) ))
( "gentile"  n ( jh e1 n t ai l ) ((pos "K6%" "OA%" ) ))
( "gentiles"  n ( jh e1 n t ai l z ) ((pos "Kj%" ) ))
( "gentility"  n ( jh e1 n t i1 l i t ii ) ((pos "L@%" ) ))
( "gentle"  j ( jh e1 n t l ) ((pos "OB%" ) ))
( "gentlefolk"  n ( jh e1 n t l f ou k ) ((pos "Kj%" ) ))
( "gentleman"  n ( jh e1 n t l m @ n ) ((pos "Ki%" ) ))
( "gentleman-at-arms"  n ( jh e2 n t l m @ n - @ t - aa1 m z ) ((pos "Ki$" ) ))
( "gentlemanly"  j ( jh e1 n t l m @ n l ii ) ((pos "OA%" ) ))
( "gentlemen"  n ( jh e1 n t l m @ n ) ((pos "Kj%" ) ))
( "gentlemen-at-arms"  n ( jh e2 n t l m @ n - @ t - aa1 m z ) ((pos "Kj$" ) ))
( "gentleness"  n ( jh e1 n t l n @ s ) ((pos "L@%" ) ))
( "gentler"  j ( jh e1 n t l @ r ) ((pos "Or%" ) ))
( "gentlest"  j ( jh e1 n t l i s t ) ((pos "Os%" ) ))
( "gentlewoman"  n ( jh e1 n t l w u m @ n ) ((pos "Ki%" ) ))
( "gentlewomen"  n ( jh e1 n t l w i m i n ) ((pos "Kj%" ) ))
( "gently"  a ( jh e1 n t l ii ) ((pos "Pu%" ) ))
( "gentry"  n ( jh e1 n t r ii ) ((pos "K9%" ) ))
( "gents"  n ( jh e1 n t s ) ((pos "Kj%" ) ))
( "genuflect"  v ( jh e1 n y u f l e k t ) ((pos "I0%" ) (subcat )))
( "genuflected"  v ( jh e1 n y u f l e k t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "genuflecting"  v ( jh e1 n y u f l e k t i ng ) ((pos "Ib%" ) (subcat )))
( "genuflection"  n ( jh e2 n y u f l e1 k sh @ n ) ((pos "K6%" ) ))
( "genuflections"  n ( jh e2 n y u f l e1 k sh @ n z ) ((pos "Kj%" ) ))
( "genuflects"  v ( jh e1 n y u f l e k t s ) ((pos "Ia%" ) (subcat )))
( "genuflexion"  n ( jh e2 n y u f l e1 k sh @ n ) ((pos "K6$" ) ))
( "genuflexions"  n ( jh e2 n y u f l e1 k sh @ n z ) ((pos "Kj$" ) ))
( "genuine"  j ( jh e1 n y u i n ) ((pos "OA%" ) ))
( "genuinely"  a ( jh e1 n y u i n l ii ) ((pos "Pu%" ) ))
( "genuineness"  n ( jh e1 n y u i n - n @ s ) ((pos "L@%" ) ))
( "genus"  n ( jh ii1 n @ s ) ((pos "Ki%" ) ))
( "geo-"  j ( jh i@2 u - ) ((pos "U-%" ) ))
( "geocentric"  j ( jh ii2 ou s e1 n t r i k ) ((pos "OA%" ) ))
( "geographer"  n ( jh i1 o1 g r @ f @ r ) ((pos "K6%" ) ))
( "geographers"  n ( jh i1 o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "geographical"  j ( jh i@2 g r a1 f i k l ) ((pos "OA%" ) ))
( "geographically"  a ( jh i@2 g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "geography"  n ( jh i1 o1 g r @ f ii ) ((pos "L@%" ) ))
( "geological"  j ( jh i@2 l o1 jh i k l ) ((pos "OA%" ) ))
( "geologically"  a ( jh i@2 l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "geologist"  n ( jh i1 o1 l @ jh i s t ) ((pos "K6%" ) ))
( "geologists"  n ( jh i1 o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "geology"  n ( jh i1 o1 l @ jh ii ) ((pos "L@%" ) ))
( "geometric"  j ( jh i@2 m e1 t r i k ) ((pos "OA%" ) ))
( "geometrical"  j ( jh i@2 m e1 t r i k l ) ((pos "OA%" ) ))
( "geometrically"  a ( jh i@2 m e1 t r i k l ii ) ((pos "Pu%" ) ))
( "geometry"  n ( jh i1 o1 m @ t r ii ) ((pos "L@%" ) ))
( "geophysical"  j ( jh i@2 f i1 z i k l ) ((pos "OA%" ) ))
( "geophysics"  n ( jh ii2 ou f i1 z i k s ) ((pos "Lk%" ) ))
( "geopolitics"  n ( jh i@2 u p o1 l @ t i k s ) ((pos "Lk$" ) ))
( "georgette"  n ( jh oo1 jh e1 t ) ((pos "L@$" ) ))
( "geranium"  n ( jh @1 r ei1 n i@ m ) ((pos "K6%" ) ))
( "geraniums"  n ( jh @1 r ei1 n i@ m z ) ((pos "Kj%" ) ))
( "geriatric"  j ( jh e2 r i a1 t r i k ) ((pos "OA%" ) ))
( "geriatrician"  n ( jh e2 r i@ t r i1 sh @ n ) ((pos "K6%" ) ))
( "geriatricians"  n ( jh e2 r i@ t r i1 sh @ n z ) ((pos "Kj%" ) ))
( "geriatrics"  n ( jh e2 r i a1 t r i k s ) ((pos "Lk%" ) ))
( "germ"  n ( jh @@1 m ) ((pos "K6%" ) ))
( "germane"  j ( jh @1 m ei1 n ) ((pos "OA%" ) ))
( "germicide"  n ( jh @@1 m i s ai d ) ((pos "K6%" ) ))
( "germicides"  n ( jh @@1 m i s ai d z ) ((pos "Kj$" ) ))
( "germinate"  v ( jh @@1 m i n ei t ) ((pos "J2%" ) (subcat "2A" )))
( "germinated"  v ( jh @@1 m i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "germinates"  v ( jh @@1 m i n ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "germinating"  v ( jh @@1 m i n ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "germination"  n ( jh @@2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "germs"  n ( jh @@1 m z ) ((pos "Kj%" ) ))
( "gerontology"  n ( jh e2 r o n t o1 l @ jh ii ) ((pos "L@$" ) ))
( "gerrymander"  v ( jh e2 r i m a1 n d @ r ) ((pos "H0$" "K6$" ) (subcat )))
( "gerrymandered"  v ( jh e2 r i m a1 n d @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "gerrymandering"  v ( jh e2 r i m a1 n d @ r i ng ) ((pos "Hb$" ) (subcat )))
( "gerrymanders"  v ( jh e2 r i m a1 n d @ z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "gerund"  n ( jh e1 r @ n d ) ((pos "K6$" ) ))
( "gerunds"  n ( jh e1 r @ n d z ) ((pos "Kj$" ) ))
( "gestation"  n ( jh e1 s t ei1 sh @ n ) ((pos "K6%" ) ))
( "gestations"  n ( jh e1 s t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "gesticulate"  v ( jh e1 s t i1 k y u l ei t ) ((pos "I2%" ) (subcat )))
( "gesticulated"  v ( jh e1 s t i1 k y u l ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "gesticulates"  v ( jh e1 s t i1 k y u l ei t s ) ((pos "Ia%" ) (subcat )))
( "gesticulating"  v ( jh e1 s t i1 k y u l ei t i ng ) ((pos "Ib%" ) (subcat )))
( "gesticulation"  n ( jh e1 s t i2 k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "gesticulations"  n ( jh e1 s t i2 k y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "gesture"  v ( jh e1 s ch @ r ) ((pos "I2%" "M6%" ) (subcat )))
( "gestured"  v ( jh e1 s ch @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "gestures"  v ( jh e1 s ch @ z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "gesturing"  v ( jh e1 s ch @ r i ng ) ((pos "Ib%" ) (subcat )))
( "get"  v ( g e1 t ) ((pos "J5*" ) (subcat "2C" "2D" "2E" "3A" "4A" "6A" "7A" "7B" "12B" "13B" "14" "15A" "15B" "17" "19B" "22" )))
( "get-at-able"  j ( g e2 t - a1 t - @ b l ) ((pos "OA$" ) ))
( "get-together"  n ( g e1 t - t @ g e dh @ r ) ((pos "K6%" ) ))
( "get-togethers"  n ( g e1 t - t @ g e dh @ z ) ((pos "Kj$" ) ))
( "get-up"  n ( g e1 t - uh p ) ((pos "K6%" ) ))
( "get-ups"  n ( g e1 t - uh p s ) ((pos "Kj$" ) ))
( "getaway"  n ( g e1 t @ w ei ) ((pos "K6%" ) ))
( "getaways"  n ( g e1 t @ w ei z ) ((pos "Kj$" ) ))
( "gets"  v ( g e1 t s ) ((pos "Ja%" ) (subcat "2C" "2D" "2E" "3A" "4A" "6A" "7A" "7B" "12B" "13B" "14" "15A" "15B" "17" "19B" "22" )))
( "getting"  v ( g e1 t i ng ) ((pos "Jb%" ) (subcat "2C" "2D" "2E" "3A" "4A" "6A" "7A" "7B" "12B" "13B" "14" "15A" "15B" "17" "19B" "22" )))
( "geum"  n ( jh ii1 @ m ) ((pos "K6$" ) ))
( "geums"  n ( jh ii1 @ m z ) ((pos "Kj$" ) ))
( "geyser"  n ( g ii1 z @ r ) ((pos "K6%" ) ))
( "geysers"  n ( g ii1 z @ z ) ((pos "Kj%" ) ))
( "gharries"  n ( g a1 r i z ) ((pos "Kj$" ) ))
( "gharry"  n ( g a1 r ii ) ((pos "K8$" ) ))
( "ghastlier"  j ( g aa1 s t l i@ r ) ((pos "Or%" ) ))
( "ghastliest"  j ( g aa1 s t l i i s t ) ((pos "Os%" ) ))
( "ghastly"  j ( g aa1 s t l ii ) ((pos "OD%" ) ))
( "ghat"  n ( g oo1 t ) ((pos "K6$" ) ))
( "ghats"  n ( g oo1 t s ) ((pos "Kj$" ) ))
( "ghee"  n ( g ii1 ) ((pos "L@$" ) ))
( "gherkin"  n ( g @@1 k i n ) ((pos "K6%" ) ))
( "gherkins"  n ( g @@1 k i n z ) ((pos "Kj%" ) ))
( "ghetto"  n ( g e1 t ou ) ((pos "K6%" ) ))
( "ghettos"  n ( g e1 t ou z ) ((pos "Kj%" ) ))
( "ghost"  v ( g ou1 s t ) ((pos "J0%" "K6%" ) ))
( "ghost-writer"  n ( g ou1 s t - r ai t @ r ) ((pos "K6%" ) ))
( "ghost-writers"  n ( g ou1 s t - r ai t @ z ) ((pos "Kj%" ) ))
( "ghosted"  v ( g ou1 s t i d ) ((pos "Jc%" "Jd%" ) ))
( "ghosting"  v ( g ou1 s t i ng ) ((pos "Jb%" ) ))
( "ghostliness"  n ( g ou1 s t l i n @ s ) ((pos "L@%" ) ))
( "ghostly"  j ( g ou1 s t l ii ) ((pos "OA%" ) ))
( "ghosts"  v ( g ou1 s t s ) ((pos "Ja%" "Kj%" ) ))
( "ghoul"  n ( g uu1 l ) ((pos "K6%" ) ))
( "ghoulish"  j ( g uu1 l i sh ) ((pos "OA%" ) ))
( "ghouls"  n ( g uu1 l z ) ((pos "Kj%" ) ))
( "giant"  n ( jh ai1 @ n t ) ((pos "K6%" ) ))
( "giantess"  n ( jh ai1 @ n t e s ) ((pos "K7$" ) ))
( "giantesses"  n ( jh ai1 @ n t e s i z ) ((pos "Kj$" ) ))
( "giants"  n ( jh ai1 @ n t s ) ((pos "Kj%" ) ))
( "gibber"  v ( jh i1 b @ r ) ((pos "I0$" ) (subcat "2A" )))
( "gibbered"  v ( jh i1 b @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gibbering"  v ( jh i1 b @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gibberish"  n ( jh i1 b @ r i sh ) ((pos "L@%" ) ))
( "gibbers"  v ( jh i1 b @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "gibbet"  v ( jh i1 b i t ) ((pos "H0$" "K6%" ) ))
( "gibbeted"  v ( jh i1 b i t i d ) ((pos "Hc$" "Hd$" ) ))
( "gibbeting"  v ( jh i1 b i t i ng ) ((pos "Hb$" ) ))
( "gibbets"  v ( jh i1 b i t s ) ((pos "Ha$" "Kj$" ) ))
( "gibbon"  n ( g i1 b @ n ) ((pos "K6%" ) ))
( "gibbons"  n ( g i1 b @ n z ) ((pos "Kj%" ) ))
( "gibbous"  j ( jh i1 b @ s ) ((pos "OA$" ) ))
( "gibe"  v ( jh ai1 b ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "gibed"  v ( jh ai1 b d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gibes"  v ( jh ai1 b z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "gibing"  v ( jh ai1 b i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gibingly"  a ( jh ai1 b i ng l ii ) ((pos "Pu%" ) ))
( "giblets"  n ( jh i1 b l i t s ) ((pos "Kj%" ) ))
( "giddier"  j ( g i1 d i@ r ) ((pos "Or%" ) ))
( "giddiest"  j ( g i1 d i i s t ) ((pos "Os%" ) ))
( "giddily"  a ( g i1 d i l ii ) ((pos "Pu%" ) ))
( "giddiness"  n ( g i1 d i n @ s ) ((pos "L@%" ) ))
( "giddy"  j ( g i1 d ii ) ((pos "OD%" ) ))
( "gift"  v ( g i1 f t ) ((pos "H0$" "M6%" ) (subcat )))
( "gifted"  v ( g i1 f t i d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "gifting"  v ( g i1 f t i ng ) ((pos "Hb$" ) (subcat )))
( "gifts"  v ( g i1 f t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "gig"  n ( g i1 g ) ((pos "K6%" ) ))
( "gigantic"  j ( jh ai1 g a1 n t i k ) ((pos "OA%" ) ))
( "giggle"  v ( g i1 g l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "giggled"  v ( g i1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "giggles"  v ( g i1 g l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "giggling"  v ( g i1 g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gigolo"  n ( zh i1 g @ l ou ) ((pos "K6%" ) ))
( "gigolos"  n ( zh i1 g @ l ou z ) ((pos "Kj%" ) ))
( "gigs"  n ( g i1 g z ) ((pos "Kj%" ) ))
( "gild"  v ( g i1 l d ) ((pos "H0%" "K6%" ) (subcat )))
( "gilded"  v ( g i1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gilder"  n ( g i1 l d @ r ) ((pos "K6$" ) ))
( "gilders"  n ( g i1 l d @ z ) ((pos "Kj$" ) ))
( "gilding"  v ( g i1 l d i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "gilds"  v ( g i1 l d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "gill"  n ( jh i1 l ) ((pos "K6%" ) ))
( "gill"  n ( g i1 l ) ((pos "K6$" ) ))
( "gillie"  n ( g i1 l ii ) ((pos "K6$" ) ))
( "gillies"  n ( g i1 l i z ) ((pos "Kj$" ) ))
( "gills"  n ( jh i1 l z ) ((pos "Kj%" ) ))
( "gills"  n ( g i1 l z ) ((pos "Kj%" ) ))
( "gilt"  v ( g i1 l t ) ((pos "Hd$" "L@%" ) (subcat )))
( "gilt-edged"  j ( g i1 l t - e jh d ) ((pos "OA%" ) ))
( "gimbals"  n ( jh i1 m b l z ) ((pos "Kj$" ) ))
( "gimcrack"  j ( g i1 m k r a k ) ((pos "OA$" ) ))
( "gimlet"  n ( g i1 m l i t ) ((pos "K6%" ) ))
( "gimlets"  n ( g i1 m l i t s ) ((pos "Kj%" ) ))
( "gimmick"  n ( g i1 m i k ) ((pos "K6%" ) ))
( "gimmicks"  n ( g i1 m i k s ) ((pos "Kj%" ) ))
( "gin"  v ( jh i1 n ) ((pos "H4$" "M6%" ) (subcat )))
( "ginger"  v ( jh i1 n jh @ r ) ((pos "H0%" "L@%" ) (subcat "6A" )))
( "gingerbread"  n ( jh i1 n jh @ b r e d ) ((pos "M6%" ) ))
( "gingerbreads"  n ( jh i1 n jh @ b r e d z ) ((pos "Mj%" ) ))
( "gingered"  v ( jh i1 n jh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "gingering"  v ( jh i1 n jh @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "gingerly"  j ( jh i1 n jh @ l ii ) ((pos "OA%" "Pu%" ) ))
( "gingers"  v ( jh i1 n jh @ z ) ((pos "Ha%" ) (subcat "6A" )))
( "gingham"  n ( g i1 ng @ m ) ((pos "L@%" ) ))
( "gingko"  n ( g i1 ng k ou ) ((pos "K6$" ) ))
( "gingkoes"  n ( g i1 ng k ou z ) ((pos "Kj$" ) ))
( "gingkos"  n ( g i1 ng k ou z ) ((pos "Kj$" ) ))
( "ginned"  v ( jh i1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "ginning"  v ( jh i1 n i ng ) ((pos "Hb$" ) (subcat )))
( "gins"  v ( jh i1 n z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "ginseng"  n ( jh i1 n s e ng ) ((pos "L@%" ) ))
( "gipsies"  n ( jh i1 p s i z ) ((pos "Kj%" ) ))
( "gipsy"  n ( jh i1 p s ii ) ((pos "K8%" ) ))
( "giraffe"  n ( jh i1 r aa1 f ) ((pos "K6%" ) ))
( "giraffes"  n ( jh i1 r aa1 f s ) ((pos "Kj%" ) ))
( "gird"  v ( g @@1 d ) ((pos "H0%" ) (subcat )))
( "girded"  v ( g @@1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "girder"  n ( g @@1 d @ r ) ((pos "K6%" ) ))
( "girders"  n ( g @@1 d @ z ) ((pos "Kj%" ) ))
( "girding"  v ( g @@1 d i ng ) ((pos "Hb%" ) (subcat )))
( "girdle"  v ( g @@1 d l ) ((pos "H2%" "K6%" ) (subcat "15A" )))
( "girdled"  v ( g @@1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "girdles"  v ( g @@1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "girdling"  v ( g @@1 d l i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "girds"  v ( g @@1 d z ) ((pos "Ha%" ) (subcat )))
( "girl"  n ( g @@1 l ) ((pos "K6*" ) ))
( "girlfriend"  n ( g @@1 l f r e n d ) ((pos "K6%" ) ))
( "girlfriends"  n ( g @@1 l f r e n d z ) ((pos "Kj%" ) ))
( "girlhood"  n ( g @@1 l h u d ) ((pos "L@%" ) ))
( "girlish"  j ( g @@1 l i sh ) ((pos "OA%" ) ))
( "girlishly"  a ( g @@1 l i sh l ii ) ((pos "Pu%" ) ))
( "girlishness"  n ( g @@1 l i sh n @ s ) ((pos "L@%" ) ))
( "girls"  n ( g @@1 l z ) ((pos "Kj%" ) ))
( "giro"  n ( jh ai1 r ou ) ((pos "L@%" ) ))
( "girt"  v ( g @@1 t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "girth"  n ( g @@1 th ) ((pos "K6%" ) ))
( "girths"  n ( g @@1 th s ) ((pos "Kj%" ) ))
( "gist"  n ( jh i1 s t ) ((pos "K6%" ) ))
( "gists"  n ( jh i1 s t s ) ((pos "Kj$" ) ))
( "give"  v ( g i1 v ) ((pos "J5*" "L@%" ) (subcat "2A" "2C" "3A" "6A" "12A" "12B" "13A" "15B" )))
( "giveaway"  n ( g i1 v @ w ei ) ((pos "K6%" ) ))
( "giveaways"  n ( g i1 v @ w ei z ) ((pos "Kj$" ) ))
( "given"  v ( g i1 v n ) ((pos "Jd*" ) (subcat "2A" "2C" "3A" "6A" "12A" "12B" "13A" "15B" )))
( "giver"  n ( g i1 v @ r ) ((pos "K6%" ) ))
( "givers"  n ( g i1 v @ z ) ((pos "Kj%" ) ))
( "gives"  v ( g i1 v z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "12A" "12B" "13A" "15B" )))
( "giving"  v ( g i1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "12A" "12B" "13A" "15B" )))
( "gizzard"  n ( g i1 z @ d ) ((pos "K6$" ) ))
( "gizzards"  n ( g i1 z @ d z ) ((pos "Kj$" ) ))
( "glac_e"  j ( g l a1 s ei ) ((pos "OA%" ) ))
( "glacial"  j ( g l ei1 s i@ l ) ((pos "OA%" ) ))
( "glacier"  n ( g l a1 s i@ r ) ((pos "K6%" ) ))
( "glaciers"  n ( g l a1 s i@ z ) ((pos "Kj%" ) ))
( "glad"  j ( g l a1 d ) ((pos "OE%" ) ))
( "gladden"  v ( g l a1 d n ) ((pos "H0%" ) (subcat )))
( "gladdened"  v ( g l a1 d n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gladdening"  v ( g l a1 d n i ng ) ((pos "Hb%" ) (subcat )))
( "gladdens"  v ( g l a1 d n z ) ((pos "Ha%" ) (subcat )))
( "gladder"  j ( g l a1 d @ r ) ((pos "Or$" ) ))
( "gladdest"  j ( g l a1 d i s t ) ((pos "Os$" ) ))
( "glade"  n ( g l ei1 d ) ((pos "K6%" ) ))
( "glades"  n ( g l ei1 d z ) ((pos "Kj%" ) ))
( "gladiator"  n ( g l a1 d i ei t @ r ) ((pos "K6%" ) ))
( "gladiatorial"  j ( g l a2 d i@ t oo1 r i@ l ) ((pos "OA%" ) ))
( "gladiators"  n ( g l a1 d i ei t @ z ) ((pos "Kj%" ) ))
( "gladioli"  n ( g l a2 d i ou1 l ai ) ((pos "Kj$" ) ))
( "gladiolus"  n ( g l a2 d i ou1 l @ s ) ((pos "K7$" ) ))
( "gladioluses"  n ( g l a2 d i ou1 l @ s i z ) ((pos "Kj$" ) ))
( "gladly"  a ( g l a1 d l ii ) ((pos "Pu%" ) ))
( "gladness"  n ( g l a1 d n @ s ) ((pos "L@%" ) ))
( "gladsome"  j ( g l a1 d s @ m ) ((pos "OA$" ) ))
( "glamorization"  n ( g l a2 m @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "glamorize"  v ( g l a1 m @ r ai z ) ((pos "H2%" ) (subcat )))
( "glamorized"  v ( g l a1 m @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "glamorizes"  v ( g l a1 m @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "glamorizing"  v ( g l a1 m @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "glamorous"  j ( g l a1 m @ r @ s ) ((pos "OA%" ) ))
( "glamour"  n ( g l a1 m @ r ) ((pos "L@%" ) ))
( "glance"  v ( g l aa1 n s ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" )))
( "glanced"  v ( g l aa1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" )))
( "glances"  v ( g l aa1 n s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" )))
( "glancing"  v ( g l aa1 n s i ng ) ((pos "Jb%" ) (subcat "2C" "3A" )))
( "gland"  n ( g l a1 n d ) ((pos "K6%" ) ))
( "glanders"  n ( g l a1 n d @ z ) ((pos "L@$" ) ))
( "glands"  n ( g l a1 n d z ) ((pos "Kj%" ) ))
( "glandular"  j ( g l a1 n d y u l @ r ) ((pos "OA%" ) ))
( "glare"  v ( g l e@1 r ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "glared"  v ( g l e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "glares"  v ( g l e@1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "glaring"  v ( g l e@1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "3A" "6A" )))
( "glass"  v ( g l aa1 s ) ((pos "H1$" "M7%" ) (subcat "6A" )))
( "glass-blower"  n ( g l aa1 s - b l ou @ r ) ((pos "K6%" ) ))
( "glass-blowers"  n ( g l aa1 s - b l ou @ z ) ((pos "Kj%" ) ))
( "glass-cutter"  n ( g l aa1 s - k uh t @ r ) ((pos "K6%" ) ))
( "glass-cutters"  n ( g l aa1 s - k uh t @ z ) ((pos "Kj%" ) ))
( "glass-wool"  n ( g l aa1 s - w u l ) ((pos "L@$" ) ))
( "glassed"  v ( g l aa1 s t ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "glasses"  v ( g l aa1 s i z ) ((pos "Ha$" "Mj%" ) (subcat "6A" )))
( "glassful"  n ( g l aa1 s f u l ) ((pos "K6%" ) ))
( "glassfuls"  n ( g l aa1 s f u l z ) ((pos "Kj%" ) ))
( "glasshouse"  n ( g l aa1 s h au s ) ((pos "K6%" ) ))
( "glasshouses"  n ( g l aa1 s h au z i z ) ((pos "Kj%" ) ))
( "glassier"  j ( g l aa1 s i@ r ) ((pos "Or$" ) ))
( "glassiest"  j ( g l aa1 s i i s t ) ((pos "Os$" ) ))
( "glassing"  v ( g l aa1 s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "glassware"  n ( g l aa1 s w e@ r ) ((pos "L@%" ) ))
( "glassworks"  n ( g l aa1 s w @@ k s ) ((pos "K9%" ) ))
( "glassy"  j ( g l aa1 s ii ) ((pos "OD%" ) ))
( "glaucoma"  n ( g l oo1 k ou1 m @ ) ((pos "L@%" ) ))
( "glaucous"  j ( g l oo1 k @ s ) ((pos "OA$" ) ))
( "glaze"  v ( g l ei1 z ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "glazed"  v ( g l ei1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "glazes"  v ( g l ei1 z i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "glazier"  n ( g l ei1 z i@ r ) ((pos "K6%" ) ))
( "glaziers"  n ( g l ei1 z i@ z ) ((pos "Kj%" ) ))
( "glazing"  v ( g l ei1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "gleam"  v ( g l ii1 m ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "gleamed"  v ( g l ii1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gleaming"  v ( g l ii1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gleams"  v ( g l ii1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "glean"  v ( g l ii1 n ) ((pos "J0%" ) (subcat "2A" )))
( "gleaned"  v ( g l ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "gleaner"  n ( g l ii1 n @ r ) ((pos "K6%" ) ))
( "gleaners"  n ( g l ii1 n @ z ) ((pos "Kj%" ) ))
( "gleaning"  v ( g l ii1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "gleanings"  n ( g l ii1 n i ng z ) ((pos "Kj%" ) ))
( "gleans"  v ( g l ii1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "glebe"  n ( g l ii1 b ) ((pos "K6$" ) ))
( "glebes"  n ( g l ii1 b z ) ((pos "Kj$" ) ))
( "glee"  n ( g l ii1 ) ((pos "M6%" ) ))
( "gleeful"  j ( g l ii1 f @ l ) ((pos "OA%" ) ))
( "gleefully"  a ( g l ii1 f @ l ii ) ((pos "Pu%" ) ))
( "glees"  n ( g l ii1 z ) ((pos "Mj$" ) ))
( "glen"  n ( g l e1 n ) ((pos "K6%" ) ))
( "glens"  n ( g l e1 n z ) ((pos "Kj%" ) ))
( "glib"  j ( g l i1 b ) ((pos "OE%" ) ))
( "glibber"  j ( g l i1 b @ r ) ((pos "Or$" ) ))
( "glibbest"  j ( g l i1 b i s t ) ((pos "Os$" ) ))
( "glibly"  a ( g l i1 b l ii ) ((pos "Pu%" ) ))
( "glibness"  n ( g l i1 b n @ s ) ((pos "L@%" ) ))
( "glide"  v ( g l ai1 d ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "glided"  v ( g l ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glider"  n ( g l ai1 d @ r ) ((pos "K6%" ) ))
( "gliders"  n ( g l ai1 d @ z ) ((pos "Kj%" ) ))
( "glides"  v ( g l ai1 d z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "gliding"  v ( g l ai1 d i ng ) ((pos "Ib%" "L@%" ) (subcat "2A" )))
( "glimmer"  v ( g l i1 m @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "glimmered"  v ( g l i1 m @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glimmering"  v ( g l i1 m @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "glimmers"  v ( g l i1 m @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "glimpse"  v ( g l i1 m p s ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "glimpsed"  v ( g l i1 m p s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "glimpses"  v ( g l i1 m p s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "glimpsing"  v ( g l i1 m p s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "glint"  v ( g l i1 n t ) ((pos "I0%" "K6%" ) ))
( "glinted"  v ( g l i1 n t i d ) ((pos "Ic%" "Id%" ) ))
( "glinting"  v ( g l i1 n t i ng ) ((pos "Ib%" ) ))
( "glints"  v ( g l i1 n t s ) ((pos "Ia%" "Kj%" ) ))
( "glissade"  v ( g l i1 s ei1 d ) ((pos "I2$" "K6$" ) ))
( "glissaded"  v ( g l i1 s ei1 d i d ) ((pos "Ic$" "Id$" ) ))
( "glissades"  v ( g l i1 s ei1 d z ) ((pos "Ia$" "Kj$" ) ))
( "glissading"  v ( g l i1 s ei1 d i ng ) ((pos "Ib$" ) ))
( "glissando"  j ( g l i1 s a1 n d ou ) ((pos "OA$" "Pu$" ) ))
( "glisten"  v ( g l i1 s n ) ((pos "I0%" ) (subcat "2A" )))
( "glistened"  v ( g l i1 s n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glistening"  v ( g l i1 s n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "glistens"  v ( g l i1 s n z ) ((pos "Ia%" ) (subcat "2A" )))
( "glister"  v ( g l i1 s t @ r ) ((pos "I0$" "Ki$" ) ))
( "glistered"  v ( g l i1 s t @ d ) ((pos "Ic$" "Id$" ) ))
( "glistering"  v ( g l i1 s t @ r i ng ) ((pos "Ib$" ) ))
( "glisters"  v ( g l i1 s t @ z ) ((pos "Ia$" ) ))
( "glitter"  v ( g l i1 t @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "glittered"  v ( g l i1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glittering"  v ( g l i1 t @ r i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "glitters"  v ( g l i1 t @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "gloaming"  n ( g l ou1 m i ng ) ((pos "Ki%" ) ))
( "gloat"  v ( g l ou1 t ) ((pos "I0%" ) (subcat "2A" )))
( "gloated"  v ( g l ou1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gloating"  v ( g l ou1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gloatingly"  a ( g l ou1 t i ng l ii ) ((pos "Pu%" ) ))
( "gloats"  v ( g l ou1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "global"  j ( g l ou1 b l ) ((pos "OA%" ) ))
( "globe"  n ( g l ou1 b ) ((pos "K6%" ) ))
( "globes"  n ( g l ou1 b z ) ((pos "Kj%" ) ))
( "globetrot"  v ( g l ou1 b t r o t ) ((pos "I4%" ) (subcat )))
( "globetrots"  v ( g l ou1 b t r o t s ) ((pos "Ia%" ) (subcat )))
( "globetrotted"  v ( g l ou1 b t r o t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "globetrotter"  n ( g l ou1 b t r o t @ r ) ((pos "K6%" ) ))
( "globetrotters"  n ( g l ou1 b t r o t @ z ) ((pos "Kj%" ) ))
( "globetrotting"  v ( g l ou1 b t r o t i ng ) ((pos "Ib%" ) (subcat )))
( "globular"  j ( g l o1 b y u l @ r ) ((pos "OA%" ) ))
( "globule"  n ( g l o1 b y uu l ) ((pos "K6%" ) ))
( "globules"  n ( g l o1 b y uu l z ) ((pos "Kj%" ) ))
( "glockenspiel"  n ( g l o1 k @ n s p ii l ) ((pos "K6%" ) ))
( "glockenspiels"  n ( g l o1 k @ n s p ii l z ) ((pos "Kj%" ) ))
( "gloom"  n ( g l uu1 m ) ((pos "M6%" ) ))
( "gloomier"  j ( g l uu1 m i@ r ) ((pos "Or%" ) ))
( "gloomiest"  j ( g l uu1 m i i s t ) ((pos "Os%" ) ))
( "gloomily"  a ( g l uu1 m i l ii ) ((pos "Pu%" ) ))
( "glooms"  n ( g l uu1 m z ) ((pos "Mj$" ) ))
( "gloomy"  j ( g l uu1 m ii ) ((pos "OD%" ) ))
( "gloried"  v ( g l oo1 r i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "glories"  v ( g l oo1 r i z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "glorification"  n ( g l oo2 r i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "glorified"  v ( g l oo1 r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "glorifies"  v ( g l oo1 r i f ai z ) ((pos "Ha%" ) (subcat )))
( "glorify"  v ( g l oo1 r i f ai ) ((pos "H3%" ) (subcat )))
( "glorifying"  v ( g l oo1 r i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "glorious"  j ( g l oo1 r i@ s ) ((pos "OA%" ) ))
( "gloriously"  a ( g l oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "glory"  v ( g l oo1 r ii ) ((pos "I3%" "M8%" ) (subcat )))
( "glory-hole"  n ( g l oo1 r i - h ou l ) ((pos "K6%" ) ))
( "glory-holes"  n ( g l oo1 r i - h ou l z ) ((pos "Kj$" ) ))
( "glorying"  v ( g l oo1 r i i ng ) ((pos "Ib%" ) (subcat )))
( "gloss"  v ( g l o1 s ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "glossaries"  n ( g l o1 s @ r i z ) ((pos "Kj%" ) ))
( "glossary"  n ( g l o1 s @ r ii ) ((pos "K8%" ) ))
( "glossed"  v ( g l o1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "glosses"  v ( g l o1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "glossier"  j ( g l o1 s i@ r ) ((pos "Or%" ) ))
( "glossiest"  j ( g l o1 s i i s t ) ((pos "Os%" ) ))
( "glossily"  a ( g l o1 s i l ii ) ((pos "Pu%" ) ))
( "glossiness"  n ( g l o1 s i n @ s ) ((pos "L@%" ) ))
( "glossing"  v ( g l o1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "glossy"  j ( g l o1 s ii ) ((pos "OD%" ) ))
( "glottal"  j ( g l o1 t l ) ((pos "OA%" ) ))
( "glottis"  n ( g l o1 t i s ) ((pos "K7$" ) ))
( "glottises"  n ( g l o1 t i s i z ) ((pos "Kj$" ) ))
( "glove"  n ( g l uh1 v ) ((pos "K6%" ) ))
( "glove-compartment"  n ( g l uh1 v - k @ m p aa2 t m @ n t ) ((pos "K6%" ) ))
( "glove-compartments"  n ( g l uh1 v - k @ m p aa2 t m @ n t s ) ((pos "Kj%" ) ))
( "gloved"  j ( g l uh1 v d ) ((pos "OA%" ) ))
( "gloves"  n ( g l uh1 v z ) ((pos "Kj%" ) ))
( "glow"  v ( g l ou1 ) ((pos "I0%" "Ki%" ) (subcat "2A" )))
( "glow-worm"  n ( g l ou1 - w @@ m ) ((pos "K6%" ) ))
( "glow-worms"  n ( g l ou1 - w @@ m z ) ((pos "Kj%" ) ))
( "glowed"  v ( g l ou1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glower"  v ( g l au1 @ r ) ((pos "I0%" ) (subcat "2A" )))
( "glowered"  v ( g l au1 @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "glowering"  v ( g l au1 @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gloweringly"  a ( g l au1 w @ r i ng l ii ) ((pos "Pu%" ) ))
( "glowers"  v ( g l au1 @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "glowing"  v ( g l ou1 i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "glowingly"  a ( g l ou1 i ng l ii ) ((pos "Pu%" ) ))
( "glows"  v ( g l ou1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "glucose"  n ( g l uu1 k ou s ) ((pos "L@%" ) ))
( "glue"  v ( g l uu1 ) ((pos "H5%" "M6%" ) (subcat "6A" "15A" )))
( "glued"  v ( g l uu1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "glues"  v ( g l uu1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "gluey"  j ( g l uu1 ii ) ((pos "OA%" ) ))
( "gluing"  v ( g l uu1 i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "glum"  j ( g l uh1 m ) ((pos "OE%" ) ))
( "glumly"  a ( g l uh1 m l ii ) ((pos "Pu%" ) ))
( "glummer"  j ( g l uh1 m @ r ) ((pos "Or%" ) ))
( "glummest"  j ( g l uh1 m i s t ) ((pos "Os%" ) ))
( "glumness"  n ( g l uh1 m n @ s ) ((pos "L@%" ) ))
( "glut"  v ( g l uh1 t ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "gluten"  n ( g l uu1 t @ n ) ((pos "L@$" ) ))
( "glutinous"  j ( g l uu1 t i n @ s ) ((pos "OA%" ) ))
( "gluts"  v ( g l uh1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "glutted"  v ( g l uh1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "glutting"  v ( g l uh1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "glutton"  n ( g l uh1 t n ) ((pos "K6%" ) ))
( "gluttonous"  j ( g l uh1 t @ n @ s ) ((pos "OA%" ) ))
( "gluttonously"  a ( g l uh1 t @ n @ s l ii ) ((pos "Pu$" ) ))
( "gluttons"  n ( g l uh1 t n z ) ((pos "Kj%" ) ))
( "gluttony"  n ( g l uh1 t n ii ) ((pos "L@%" ) ))
( "glycerine"  n ( g l i1 s @ r ii n ) ((pos "L@%" ) ))
( "gnarled"  j ( n aa1 l d ) ((pos "OA%" ) ))
( "gnash"  v ( n a1 sh ) ((pos "J1%" ) (subcat "2A" )))
( "gnashed"  v ( n a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "gnashes"  v ( n a1 sh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "gnashing"  v ( n a1 sh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "gnat"  n ( n a1 t ) ((pos "K6%" ) ))
( "gnats"  n ( n a1 t s ) ((pos "Kj%" ) ))
( "gnaw"  v ( n oo1 ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "gnawed"  v ( n oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "gnawing"  v ( n oo1 i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "gnaws"  v ( n oo1 z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "gnome"  n ( n ou1 m ) ((pos "K6%" ) ))
( "gnomes"  n ( n ou1 m z ) ((pos "Kj%" ) ))
( "gnu"  n ( n uu1 ) ((pos "K6$" ) ))
( "gnus"  n ( n uu1 z ) ((pos "Kj$" ) ))
( "go"  v ( g ou1 ) ((pos "I5*" "Ki%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" )))
( "go-ahead"  n ( g ou1 - @ h e d ) ((pos "Ki%" ) ))
( "go-as-you-please"  j ( g ou2 - @ z - y uu - p l ii1 z ) ((pos "Oq$" ) ))
( "go-between"  n ( g ou1 - b i t w ii n ) ((pos "K6%" ) ))
( "go-betweens"  n ( g ou1 - b i t w ii n z ) ((pos "Kj%" ) ))
( "go-by"  n ( g ou1 - b ai ) ((pos "Ki$" ) ))
( "go-cart"  n ( g ou1 - k aa t ) ((pos "K6$" ) ))
( "go-carts"  n ( g ou1 - k aa t s ) ((pos "Kj$" ) ))
( "go-getter"  n ( g ou2 - g e1 t @ r ) ((pos "K6%" ) ))
( "go-getters"  n ( g ou2 - g e1 t @ z ) ((pos "Kj%" ) ))
( "go-kart"  n ( g ou1 - k aa t ) ((pos "K6%" ) ))
( "go-karts"  n ( g ou1 - k aa t s ) ((pos "Kj%" ) ))
( "go-slow"  n ( g ou1 - s l ou ) ((pos "K6%" ) ))
( "go-slows"  n ( g ou1 - s l ou z ) ((pos "Kj$" ) ))
( "go-to-meeting"  j ( g ou1 - t @ - m ii t i ng ) ((pos "Oq$" ) ))
( "goad"  v ( g ou1 d ) ((pos "H0%" "K6%" ) (subcat "6A" "14" "15B" )))
( "goaded"  v ( g ou1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "15B" )))
( "goading"  v ( g ou1 d i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15B" )))
( "goads"  v ( g ou1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" "15B" )))
( "goal"  n ( g ou1 l ) ((pos "K6%" ) ))
( "goal-kick"  n ( g ou1 l - k i k ) ((pos "K6%" ) ))
( "goal-kicks"  n ( g ou1 l - k i k s ) ((pos "Kj%" ) ))
( "goal-line"  n ( g ou1 l - l ai n ) ((pos "K6%" ) ))
( "goal-lines"  n ( g ou1 l - l ai n z ) ((pos "Kj%" ) ))
( "goalie"  n ( g ou1 l ii ) ((pos "K6%" ) ))
( "goalies"  n ( g ou1 l i z ) ((pos "Kj%" ) ))
( "goalkeeper"  n ( g ou1 l k ii p @ r ) ((pos "K6%" ) ))
( "goalkeepers"  n ( g ou1 l k ii p @ z ) ((pos "Kj%" ) ))
( "goalless"  j ( g ou1 l - l i s ) ((pos "OA%" ) ))
( "goalmouth"  n ( g ou1 l m au th ) ((pos "K6%" ) ))
( "goalmouths"  n ( g ou1 l m au dh z ) ((pos "Kj$" ) ))
( "goals"  n ( g ou1 l z ) ((pos "Kj%" ) ))
( "goat"  n ( g ou1 t ) ((pos "K6%" ) ))
( "goatee"  n ( g ou1 t ii1 ) ((pos "K6$" ) ))
( "goatees"  n ( g ou1 t ii1 z ) ((pos "Kj$" ) ))
( "goatherd"  n ( g ou1 t h @@ d ) ((pos "K6%" ) ))
( "goatherds"  n ( g ou1 t h @@ d z ) ((pos "Kj%" ) ))
( "goats"  n ( g ou1 t s ) ((pos "Kj%" ) ))
( "goatskin"  n ( g ou1 t s k i n ) ((pos "M6%" ) ))
( "goatskins"  n ( g ou1 t s k i n z ) ((pos "Mj%" ) ))
( "gob"  n ( g o1 b ) ((pos "K6%" ) ))
( "gobbet"  n ( g o1 b i t ) ((pos "K6$" ) ))
( "gobbets"  n ( g o1 b i t s ) ((pos "Kj$" ) ))
( "gobble"  v ( g o1 b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "gobbled"  v ( g o1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "gobbledygook"  n ( g o1 b l d i g u k ) ((pos "L@%" ) ))
( "gobbler"  n ( g o1 b l @ r ) ((pos "K6$" ) ))
( "gobblers"  n ( g o1 b l @ z ) ((pos "Kj$" ) ))
( "gobbles"  v ( g o1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "gobbling"  v ( g o1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "goblet"  n ( g o1 b l i t ) ((pos "K6%" ) ))
( "goblets"  n ( g o1 b l i t s ) ((pos "Kj%" ) ))
( "goblin"  n ( g o1 b l i n ) ((pos "K6%" ) ))
( "goblins"  n ( g o1 b l i n z ) ((pos "Kj%" ) ))
( "gobs"  n ( g o1 b z ) ((pos "Kj$" ) ))
( "god"  n ( g o1 d ) ((pos "K6%" ) ))
( "god-damn"  j ( g o1 d - d a m ) ((pos "OA$" ) ))
( "god-damned"  j ( g o1 d - d a m d ) ((pos "OA%" ) ))
( "godchild"  n ( g o1 d ch ai l d ) ((pos "Ki%" ) ))
( "godchildren"  n ( g o1 d ch i l d r @ n ) ((pos "Kj%" ) ))
( "goddam"  j ( g o1 d a m ) ((pos "OA%" "Pu%" ) ))
( "goddaughter"  n ( g o1 d - d oo t @ r ) ((pos "K6%" ) ))
( "goddaughters"  n ( g o1 d - d oo t @ z ) ((pos "Kj%" ) ))
( "goddess"  n ( g o1 d i s ) ((pos "K7%" ) ))
( "goddesses"  n ( g o1 d i s i z ) ((pos "Kj%" ) ))
( "godfather"  n ( g o1 d f aa dh @ r ) ((pos "K6%" ) ))
( "godfathers"  n ( g o1 d f aa dh @ z ) ((pos "Kj%" ) ))
( "godfearing"  j ( g o1 d f i@ r i ng ) ((pos "OA%" ) ))
( "godforsaken"  j ( g o1 d f @ s ei k @ n ) ((pos "OA%" ) ))
( "godhead"  n ( g o1 d h e d ) ((pos "L@%" ) ))
( "godless"  j ( g o1 d l i s ) ((pos "OA%" ) ))
( "godlessness"  n ( g o1 d l i s n @ s ) ((pos "L@%" ) ))
( "godlier"  j ( g o1 d l i@ r ) ((pos "Or$" ) ))
( "godliest"  j ( g o1 d l i i s t ) ((pos "Os$" ) ))
( "godlike"  j ( g o1 d l ai k ) ((pos "OA%" ) ))
( "godliness"  n ( g o1 d l i n @ s ) ((pos "L@%" ) ))
( "godly"  j ( g o1 d l ii ) ((pos "OD%" ) ))
( "godmother"  n ( g o1 d m uh dh @ r ) ((pos "K6%" ) ))
( "godmothers"  n ( g o1 d m uh dh @ z ) ((pos "Kj%" ) ))
( "godown"  n ( g ou1 d au n ) ((pos "K6$" ) ))
( "godowns"  n ( g ou1 d au n z ) ((pos "Kj$" ) ))
( "godparent"  n ( g o1 d p e@ r @ n t ) ((pos "K6%" ) ))
( "godparents"  n ( g o1 d p e@ r @ n t s ) ((pos "Kj%" ) ))
( "gods"  n ( g o1 d z ) ((pos "Kj%" ) ))
( "godsend"  n ( g o1 d s e n d ) ((pos "K6%" ) ))
( "godsends"  n ( g o1 d s e n d z ) ((pos "Kj%" ) ))
( "godson"  n ( g o1 d s uh n ) ((pos "K6%" ) ))
( "godsons"  n ( g o1 d s uh n z ) ((pos "Kj%" ) ))
( "godspeed"  n ( g o1 d s p ii1 d ) ((pos "L@%" ) ))
( "goes"  v ( g ou1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" )))
( "goggle"  v ( g o1 g l ) ((pos "I2%" ) (subcat "2A" )))
( "goggle-box"  n ( g o1 g l - b o k s ) ((pos "K7%" ) ))
( "goggle-boxes"  n ( g o1 g l - b o k s i z ) ((pos "Kj%" ) ))
( "goggle-eyed"  j ( g o1 g l - ai d ) ((pos "OA%" ) ))
( "goggled"  v ( g o1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "goggles"  v ( g o1 g l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "goggling"  v ( g o1 g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "going"  v ( g ou1 i ng ) ((pos "Ib*" "M6%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" )))
( "going-over"  n ( g ou2 i ng - ou1 v @ r ) ((pos "K6%" ) ))
( "going-overs"  n ( g ou2 i ng - ou1 v @ z ) ((pos "Kj$" ) ))
( "goings"  n ( g ou1 i ng z ) ((pos "Mj%" ) ))
( "goings-on"  n ( g ou2 i ng z - o1 n ) ((pos "Kj%" ) ))
( "goitre"  n ( g oi1 t @ r ) ((pos "K6$" ) ))
( "goitres"  n ( g oi1 t @ z ) ((pos "Kj$" ) ))
( "gold"  n ( g ou1 l d ) ((pos "L@%" ) ))
( "gold-beater"  n ( g ou1 l d - b ii t @ r ) ((pos "K6$" ) ))
( "gold-beaters"  n ( g ou1 l d - b ii t @ z ) ((pos "Kj$" ) ))
( "gold-digger"  n ( g ou1 l d - d i g @ r ) ((pos "K6%" ) ))
( "gold-diggers"  n ( g ou1 l d - d i g @ z ) ((pos "Kj%" ) ))
( "gold-dust"  n ( g ou1 l d - d uh s t ) ((pos "L@%" ) ))
( "gold-foil"  n ( g ou1 l d - f oi1 l ) ((pos "L@%" ) ))
( "gold-leaf"  n ( g ou1 l d - l ii1 f ) ((pos "L@%" ) ))
( "gold-plate"  n ( g ou1 l d - p l ei1 t ) ((pos "L@%" ) ))
( "gold-rush"  n ( g ou1 l d - r uh sh ) ((pos "K7%" ) ))
( "gold-rushes"  n ( g ou1 l d - r uh sh i z ) ((pos "Kj$" ) ))
( "golden"  j ( g ou1 l d @ n ) ((pos "OA%" ) ))
( "goldfield"  n ( g ou1 l d f ii l d ) ((pos "K6%" ) ))
( "goldfields"  n ( g ou1 l d f ii l d z ) ((pos "Kj%" ) ))
( "goldfinch"  n ( g ou1 l d f i n ch ) ((pos "K7%" ) ))
( "goldfinches"  n ( g ou1 l d f i n ch i z ) ((pos "Kj%" ) ))
( "goldfish"  n ( g ou1 l d f i sh ) ((pos "K9%" ) ))
( "goldfishes"  n ( g ou1 l d f i sh i z ) ((pos "Kj%" ) ))
( "goldmine"  n ( g ou1 l d m ai n ) ((pos "K6%" ) ))
( "goldmines"  n ( g ou1 l d m ai n z ) ((pos "Kj%" ) ))
( "goldsmith"  n ( g ou1 l d s m i th ) ((pos "K6%" ) ))
( "goldsmiths"  n ( g ou1 l d s m i th s ) ((pos "Kj%" ) ))
( "golf"  v ( g o1 l f ) ((pos "I0$" "L@%" ) ))
( "golf-ball"  n ( g o1 l f - b oo l ) ((pos "K6%" ) ))
( "golf-balls"  n ( g o1 l f - b oo l z ) ((pos "Kj%" ) ))
( "golf-club"  n ( g o1 l f - k l uh b ) ((pos "K6%" ) ))
( "golf-clubs"  n ( g o1 l f - k l uh b z ) ((pos "Kj%" ) ))
( "golf-course"  n ( g o1 l f - k oo s ) ((pos "K6%" ) ))
( "golf-courses"  n ( g o1 l f - k oo s i z ) ((pos "Kj%" ) ))
( "golf-links"  n ( g o1 l f - l i ng k s ) ((pos "K9%" ) ))
( "golfed"  v ( g o1 l f t ) ((pos "Ic$" "Id$" ) ))
( "golfer"  n ( g o1 l f @ r ) ((pos "K6%" ) ))
( "golfers"  n ( g o1 l f @ z ) ((pos "Kj%" ) ))
( "golfing"  v ( g o1 l f i ng ) ((pos "Ib%" ) ))
( "golfs"  v ( g o1 l f s ) ((pos "Ia$" ) ))
( "golliwog"  n ( g o1 l i w o g ) ((pos "K6%" ) ))
( "golliwogs"  n ( g o1 l i w o g z ) ((pos "Kj%" ) ))
( "golly"  uh ( g o1 l ii ) ((pos "W-%" ) ))
( "golosh"  n ( g @1 l o1 sh ) ((pos "K7$" ) ))
( "goloshes"  n ( g @1 l o1 sh i z ) ((pos "Kj%" ) ))
( "gondola"  n ( g o1 n d @ l @ ) ((pos "K6%" ) ))
( "gondolas"  n ( g o1 n d @ l @ z ) ((pos "Kj%" ) ))
( "gondolier"  n ( g o2 n d @ l i@1 r ) ((pos "K6%" ) ))
( "gondoliers"  n ( g o2 n d @ l i@1 z ) ((pos "Kj%" ) ))
( "gone"  v ( g o1 n ) ((pos "Id*" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" )))
( "goner"  n ( g o1 n @ r ) ((pos "K6%" ) ))
( "goners"  n ( g o1 n @ z ) ((pos "Kj%" ) ))
( "gong"  v ( g o1 ng ) ((pos "H0$" "K6%" ) ))
( "gonged"  v ( g o1 ng d ) ((pos "Hc$" "Hd$" ) ))
( "gonging"  v ( g o1 ng i ng ) ((pos "Hb$" ) ))
( "gongs"  v ( g o1 ng z ) ((pos "Ha$" "Kj%" ) ))
( "gonna"  v ( g o1 n @ ) ((pos "Gh%" ) ))
( "gonorrhea"  n ( g o2 n @ r i@1 ) ((pos "L@$" ) ))
( "gonorrhoea"  n ( g o2 n @ r i@1 ) ((pos "L@%" ) ))
( "goo"  n ( g uu1 ) ((pos "L@%" ) ))
( "good"  n ( g u1 d ) ((pos "L@%" "OE*" ) ))
( "good-fellowship"  n ( g u1 d - f e1 l ou sh i p ) ((pos "L@$" ) ))
( "good-for-naught"  n ( g u1 d - f @ - n oo t ) ((pos "K6$" "OA$" ) ))
( "good-for-naughts"  n ( g u1 d - f @ - n oo t s ) ((pos "Kj$" ) ))
( "good-for-nothing"  n ( g u1 d - f @ - n uh th i ng ) ((pos "K6%" "OA%" ) ))
( "good-for-nothings"  n ( g u1 d - f @ - n uh th i ng z ) ((pos "Kj%" ) ))
( "good-humoured"  j ( g u1 d - h y uu1 m @ d ) ((pos "OA%" ) ))
( "good-looking"  j ( g u1 d - l u1 k i ng ) ((pos "OA%" ) ))
( "good-natured"  j ( g u1 d - n ei1 ch @ d ) ((pos "OA%" ) ))
( "good-neighbourliness"  n ( g u1 d - n ei1 b @ l i n @ s ) ((pos "L@%" ) ))
( "good-tempered"  j ( g u1 d - t e1 m p @ d ) ((pos "OA%" ) ))
( "good-time"  j ( g u1 d - t ai m ) ((pos "OA%" ) ))
( "goodbye"  n ( g u2 d b ai1 ) ((pos "K6%" "W-%" ) ))
( "goodbyes"  n ( g u2 d b ai1 z ) ((pos "Kj%" ) ))
( "goodies"  n ( g u1 d i z ) ((pos "Kj%" ) ))
( "goodish"  j ( g u1 d i sh ) ((pos "Oq%" ) ))
( "goodlier"  j ( g u1 d l i@ r ) ((pos "Or$" ) ))
( "goodliest"  j ( g u1 d l i i s t ) ((pos "Os$" ) ))
( "goodly"  j ( g u1 d l ii ) ((pos "OD%" ) ))
( "goodness"  n ( g u1 d n @ s ) ((pos "L@%" ) ))
( "goods"  n ( g u1 d z ) ((pos "Kj%" ) ))
( "goodwill"  n ( g u2 d w i1 l ) ((pos "L@%" ) ))
( "goody"  n ( g u1 d ii ) ((pos "K8$" ) ))
( "goody-goodies"  n ( g u1 d i - g u d i z ) ((pos "Kj%" ) ))
( "goody-goody"  n ( g u1 d i - g u d ii ) ((pos "K8%" "OA%" ) ))
( "gooey"  j ( g uu1 ii ) ((pos "OE%" ) ))
( "goof"  v ( g uu1 f ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "goofed"  v ( g uu1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "goofier"  j ( g uu1 f i@ r ) ((pos "Or$" ) ))
( "goofiest"  j ( g uu1 f i i s t ) ((pos "Os$" ) ))
( "goofing"  v ( g uu1 f i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "goofs"  v ( g uu1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "goofy"  j ( g uu1 f ii ) ((pos "OD%" ) ))
( "googlies"  n ( g uu1 g l i z ) ((pos "Kj%" ) ))
( "googly"  n ( g uu1 g l ii ) ((pos "K8%" ) ))
( "gooier"  j ( g uu1 i@ r ) ((pos "Or%" ) ))
( "gooiest"  j ( g uu1 i i s t ) ((pos "Os%" ) ))
( "goon"  n ( g uu1 n ) ((pos "K6%" ) ))
( "goons"  n ( g uu1 n z ) ((pos "Kj%" ) ))
( "goose"  n ( g uu1 s ) ((pos "Ki%" ) ))
( "goose-flesh"  n ( g uu1 s - f l e sh ) ((pos "L@%" ) ))
( "goose-step"  n ( g uu1 s - s t e p ) ((pos "K6%" ) ))
( "goose-steps"  n ( g uu1 s - s t e p s ) ((pos "Kj$" ) ))
( "gooseberries"  n ( g u1 z b @ r i z ) ((pos "Kj%" ) ))
( "gooseberry"  n ( g u1 z b @ r ii ) ((pos "K8%" ) ))
( "gopher"  n ( g ou1 f @ r ) ((pos "K6$" ) ))
( "gophers"  n ( g ou1 f @ z ) ((pos "Kj$" ) ))
( "gore"  v ( g oo1 r ) ((pos "H2%" "L@%" ) (subcat )))
( "gored"  v ( g oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gores"  v ( g oo1 z ) ((pos "Ha%" ) (subcat )))
( "gorge"  v ( g oo1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "gorged"  v ( g oo1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "gorgeous"  j ( g oo1 jh @ s ) ((pos "OA%" ) ))
( "gorgeously"  a ( g oo1 jh @ s l ii ) ((pos "Pu%" ) ))
( "gorges"  v ( g oo1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "gorging"  v ( g oo1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "gorier"  j ( g oo1 r i@ r ) ((pos "Or$" ) ))
( "goriest"  j ( g oo1 r i i s t ) ((pos "Os%" ) ))
( "gorilla"  n ( g @1 r i1 l @ ) ((pos "K6%" ) ))
( "gorillas"  n ( g @1 r i1 l @ z ) ((pos "Kj%" ) ))
( "goring"  v ( g oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "gormandize"  v ( g oo1 m @ n d ai z ) ((pos "I2$" ) ))
( "gormandized"  v ( g oo1 m @ n d ai z d ) ((pos "Ic$" "Id$" ) ))
( "gormandizes"  v ( g oo1 m @ n d ai z i z ) ((pos "Ia$" ) ))
( "gormandizing"  v ( g oo1 m @ n d ai z i ng ) ((pos "Ib$" ) ))
( "gormless"  j ( g oo1 m l i s ) ((pos "OA%" ) ))
( "gorse"  n ( g oo1 s ) ((pos "L@%" ) ))
( "gory"  j ( g oo1 r ii ) ((pos "OD%" ) ))
( "gosh"  uh ( g o1 sh ) ((pos "W-%" ) ))
( "gosling"  n ( g o1 z l i ng ) ((pos "K6%" ) ))
( "goslings"  n ( g o1 z l i ng z ) ((pos "Kj%" ) ))
( "gospel"  n ( g o1 s p l ) ((pos "M6%" ) ))
( "gospels"  n ( g o1 s p l z ) ((pos "Mj%" ) ))
( "gossamer"  n ( g o1 s @ m @ r ) ((pos "M6%" ) ))
( "gossamers"  n ( g o1 s @ m @ z ) ((pos "Mj$" ) ))
( "gossip"  v ( g o1 s i p ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "gossiped"  v ( g o1 s i p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gossiping"  v ( g o1 s i p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gossipped"  v ( g o1 s i p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "gossipping"  v ( g o1 s i p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "gossips"  v ( g o1 s i p s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "got"  v ( g o1 t ) ((pos "Jc*" "Jd*" ) (subcat "2C" "2D" "2E" "3A" "4A" "6A" "7A" "7B" "12B" "13B" "14" "15A" "15B" "17" "19B" "22" )))
( "gotta"  v ( g o1 t @ ) ((pos "Gh%" ) ))
( "gotten"  v ( g o1 t @ n ) ((pos "Jd$" ) (subcat "2C" "2D" "2E" "3A" "4A" "6A" "7A" "12B" "13B" "14" "15A" "15B" "17" "19B" "22" )))
( "gouache"  n ( g u1 aa1 sh ) ((pos "L@$" ) ))
( "gouge"  v ( g au1 jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "gouged"  v ( g au1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "gouges"  v ( g au1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "gouging"  v ( g au1 jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "goulash"  n ( g uu1 l a sh ) ((pos "M7%" ) ))
( "goulashes"  n ( g uu1 l a sh i z ) ((pos "Mj$" ) ))
( "gourd"  n ( g u@1 d ) ((pos "K6%" ) ))
( "gourds"  n ( g u@1 d z ) ((pos "Kj%" ) ))
( "gourmand"  n ( g u@1 m @ n d ) ((pos "K6$" ) ))
( "gourmands"  n ( g u@1 m @ n d z ) ((pos "Kj$" ) ))
( "gourmet"  n ( g u@1 m ei ) ((pos "K6%" ) ))
( "gourmets"  n ( g u@1 m ei z ) ((pos "Kj%" ) ))
( "gout"  n ( g au1 t ) ((pos "L@%" ) ))
( "goutier"  j ( g au1 t i@ r ) ((pos "Or$" ) ))
( "goutiest"  j ( g au1 t i i s t ) ((pos "Os$" ) ))
( "gouty"  j ( g au1 t ii ) ((pos "OD%" ) ))
( "govern"  v ( g uh1 v n ) ((pos "J0%" ) (subcat "2A" )))
( "governance"  n ( g uh1 v @ n @ n s ) ((pos "L@%" ) ))
( "governed"  v ( g uh1 v n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "governess"  n ( g uh1 v @ n e s ) ((pos "K7%" ) ))
( "governesses"  n ( g uh1 v @ n e s i z ) ((pos "Kj%" ) ))
( "governing"  v ( g uh1 v n i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "government"  n ( g uh1 v @ n m @ n t ) ((pos "M6%" ) ))
( "governmental"  j ( g uh2 v n m e1 n t l ) ((pos "OA%" ) ))
( "governments"  n ( g uh1 v @ n m @ n t s ) ((pos "Mj%" ) ))
( "governor"  n ( g uh1 v @ n @ r ) ((pos "K6%" ) ))
( "governors"  n ( g uh1 v @ n @ z ) ((pos "Kj%" ) ))
( "governs"  v ( g uh1 v n z ) ((pos "Ja%" ) (subcat "2A" )))
( "govt"  n ( g uh1 v @ n m @ n t ) ((pos "Y>%" ) ))
( "gown"  v ( g au1 n ) ((pos "H0$" "K6%" ) ))
( "gowned"  v ( g au1 n d ) ((pos "Hc$" "Hd%" ) ))
( "gowning"  v ( g au1 n i ng ) ((pos "Hb$" ) ))
( "gowns"  v ( g au1 n z ) ((pos "Ha$" "Kj%" ) ))
( "grab"  v ( g r a1 b ) ((pos "J4%" "K6%" ) (subcat "3A" )))
( "grabbed"  v ( g r a1 b d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "grabber"  n ( g r a1 b @ r ) ((pos "K6%" ) ))
( "grabbers"  n ( g r a1 b @ z ) ((pos "Kj%" ) ))
( "grabbing"  v ( g r a1 b i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "grabs"  v ( g r a1 b z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "grace"  v ( g r ei1 s ) ((pos "H2%" "M6%" ) (subcat )))
( "graced"  v ( g r ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "graceful"  j ( g r ei1 s f @ l ) ((pos "OA%" ) ))
( "gracefully"  a ( g r ei1 s f @ l ii ) ((pos "Pu%" ) ))
( "graceless"  j ( g r ei1 s l i s ) ((pos "OA%" ) ))
( "gracelessly"  a ( g r ei1 s l i s l ii ) ((pos "Pu%" ) ))
( "graces"  v ( g r ei1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "gracing"  v ( g r ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "gracious"  j ( g r ei1 sh @ s ) ((pos "OA%" ) ))
( "graciously"  a ( g r ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "graciousness"  n ( g r ei1 sh @ s n @ s ) ((pos "L@%" ) ))
( "gradation"  n ( g r @1 d ei1 sh @ n ) ((pos "M6%" ) ))
( "gradations"  n ( g r @1 d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "grade"  v ( g r ei1 d ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "graded"  v ( g r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "grades"  v ( g r ei1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "gradient"  n ( g r ei1 d i@ n t ) ((pos "K6%" ) ))
( "gradients"  n ( g r ei1 d i@ n t s ) ((pos "Kj%" ) ))
( "grading"  v ( g r ei1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "gradual"  j ( g r a1 jh u@ l ) ((pos "OA%" ) ))
( "gradually"  a ( g r a1 jh u@ l ii ) ((pos "Pu%" ) ))
( "gradualness"  n ( g r a1 jh u@ l n @ s ) ((pos "L@%" ) ))
( "graduate"  n ( g r a1 jh u@ t ) ((pos "K6%" ) ))
( "graduate"  v ( g r a1 jh u ei t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "graduated"  v ( g r a1 jh u ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "graduates"  n ( g r a1 jh u@ t s ) ((pos "Kj%" ) ))
( "graduates"  v ( g r a1 jh u ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "graduating"  v ( g r a1 jh u ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "graduation"  n ( g r a2 jh u ei1 sh @ n ) ((pos "K6%" ) ))
( "graduations"  n ( g r a2 jh u ei1 sh @ n z ) ((pos "Kj$" ) ))
( "graffiti"  n ( g r @1 f ii1 t ii ) ((pos "Kj%" ) ))
( "graffito"  n ( g r @1 f ii1 t ou ) ((pos "Ki$" ) ))
( "graft"  v ( g r aa1 f t ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "15A" )))
( "grafted"  v ( g r aa1 f t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "grafting"  v ( g r aa1 f t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "grafts"  v ( g r aa1 f t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "15A" )))
( "grail"  n ( g r ei1 l ) ((pos "K6$" ) ))
( "grails"  n ( g r ei1 l z ) ((pos "Kj$" ) ))
( "grain"  n ( g r ei1 n ) ((pos "M6%" ) ))
( "grained"  j ( g r ei1 n d ) ((pos "Ot%" ) ))
( "grains"  n ( g r ei1 n z ) ((pos "Mj%" ) ))
( "gram"  n ( g r a1 m ) ((pos "K6%" ) ))
( "grammar"  n ( g r a1 m @ r ) ((pos "M6%" ) ))
( "grammarian"  n ( g r @1 m e@1 r i@ n ) ((pos "K6%" ) ))
( "grammarians"  n ( g r @1 m e@1 r i@ n z ) ((pos "Kj%" ) ))
( "grammars"  n ( g r a1 m @ z ) ((pos "Mj%" ) ))
( "grammatical"  j ( g r @1 m a1 t i k l ) ((pos "OA%" ) ))
( "grammatically"  a ( g r @1 m a1 t i k l ii ) ((pos "Pu%" ) ))
( "gramme"  n ( g r a1 m ) ((pos "K6%" ) ))
( "grammes"  n ( g r a1 m z ) ((pos "Kj%" ) ))
( "gramophone"  n ( g r a1 m @ f ou n ) ((pos "K6%" ) ))
( "gramophones"  n ( g r a1 m @ f ou n z ) ((pos "Kj%" ) ))
( "grampus"  n ( g r a1 m p @ s ) ((pos "K7$" ) ))
( "grampuses"  n ( g r a1 m p @ s i z ) ((pos "Kj$" ) ))
( "grams"  n ( g r a1 m z ) ((pos "Kj%" ) ))
( "granaries"  n ( g r a1 n @ r i z ) ((pos "Kj%" ) ))
( "granary"  n ( g r a1 n @ r ii ) ((pos "K8%" ) ))
( "grand"  j ( g r a1 n d ) ((pos "OC%" ) ))
( "grand-"  j ( g r a n d - ) ((pos "U-%" ) ))
( "grandad"  n ( g r a1 n d a d ) ((pos "K6%" ) ))
( "grandads"  n ( g r a1 n d a d z ) ((pos "Kj%" ) ))
( "grandaunt"  n ( g r a1 n d aa1 n t ) ((pos "K6$" ) ))
( "grandaunts"  n ( g r a1 n d aa1 n t s ) ((pos "Kj$" ) ))
( "grandchild"  n ( g r a1 n ch ai l d ) ((pos "Ki%" ) ))
( "grandchildren"  n ( g r a1 n ch i l d r @ n ) ((pos "Kj%" ) ))
( "granddad"  n ( g r a1 n d a d ) ((pos "K6$" ) ))
( "granddads"  n ( g r a1 n d a d z ) ((pos "Kj$" ) ))
( "granddaughter"  n ( g r a1 n d oo t @ r ) ((pos "K6%" ) ))
( "granddaughters"  n ( g r a1 n d oo t @ z ) ((pos "Kj%" ) ))
( "grandee"  n ( g r a1 n d ii1 ) ((pos "K6$" ) ))
( "grandees"  n ( g r a1 n d ii1 z ) ((pos "Kj$" ) ))
( "grander"  j ( g r a1 n d @ r ) ((pos "Or%" ) ))
( "grandest"  j ( g r a1 n d i s t ) ((pos "Os%" ) ))
( "grandeur"  n ( g r a1 n jh @ r ) ((pos "L@%" ) ))
( "grandfather"  n ( g r a1 n f aa dh @ r ) ((pos "K6%" ) ))
( "grandfathers"  n ( g r a1 n f aa dh @ z ) ((pos "Kj%" ) ))
( "grandiloquence"  n ( g r a1 n d i1 l @ k w @ n s ) ((pos "L@%" ) ))
( "grandiloquent"  j ( g r a1 n d i1 l @ k w @ n t ) ((pos "OA%" ) ))
( "grandiose"  j ( g r a1 n d i@ u s ) ((pos "OA%" ) ))
( "grandly"  a ( g r a1 n d l ii ) ((pos "Pu%" ) ))
( "grandma"  n ( g r a1 n m aa ) ((pos "K6%" ) ))
( "grandmas"  n ( g r a1 n m aa z ) ((pos "Kj%" ) ))
( "grandmother"  n ( g r a1 n m uh dh @ r ) ((pos "K6%" ) ))
( "grandmothers"  n ( g r a1 n m uh dh @ z ) ((pos "Kj%" ) ))
( "grandnephew"  n ( g r a1 n - n e f y uu ) ((pos "K6$" ) ))
( "grandnephews"  n ( g r a1 n - n e f y uu z ) ((pos "Kj$" ) ))
( "grandniece"  n ( g r a1 n n ii1 s ) ((pos "K6$" ) ))
( "grandnieces"  n ( g r a1 n n ii1 s i z ) ((pos "Kj$" ) ))
( "grandpa"  n ( g r a1 n p aa ) ((pos "K6%" ) ))
( "grandparent"  n ( g r a1 n p e@ r @ n t ) ((pos "K6%" ) ))
( "grandparents"  n ( g r a1 n p e@ r @ n t s ) ((pos "Kj%" ) ))
( "grandpas"  n ( g r a1 n p aa z ) ((pos "Kj%" ) ))
( "grandson"  n ( g r a1 n s uh n ) ((pos "K6%" ) ))
( "grandsons"  n ( g r a1 n s uh n z ) ((pos "Kj%" ) ))
( "grandstand"  n ( g r a1 n s t a n d ) ((pos "K6%" ) ))
( "grandstands"  n ( g r a1 n s t a n d z ) ((pos "Kj%" ) ))
( "granduncle"  n ( g r a1 n d uh ng k l ) ((pos "K6$" ) ))
( "granduncles"  n ( g r a1 n d uh ng k l z ) ((pos "Kj$" ) ))
( "grange"  n ( g r ei1 n jh ) ((pos "K6$" ) ))
( "granges"  n ( g r ei1 n jh i z ) ((pos "Kj$" ) ))
( "granite"  n ( g r a1 n i t ) ((pos "L@%" ) ))
( "grannie"  n ( g r a1 n ii ) ((pos "K6%" ) ))
( "grannies"  n ( g r a1 n i z ) ((pos "Kj%" ) ))
( "granny"  n ( g r a1 n ii ) ((pos "K8%" ) ))
( "grant"  v ( g r aa1 n t ) ((pos "H0%" "K6%" ) (subcat "6A" "9" "12A" "13A" )))
( "granted"  v ( g r aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "12A" "13A" )))
( "granting"  v ( g r aa1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "9" "12A" "13A" )))
( "grants"  v ( g r aa1 n t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "9" "12A" "13A" )))
( "granular"  j ( g r a1 n y u l @ r ) ((pos "OA%" ) ))
( "granularity"  n ( g r a2 n y u l a1 r i t ii ) ((pos "L@%" ) ))
( "granulate"  v ( g r a1 n y u l ei t ) ((pos "J2$" ) (subcat "2A" )))
( "granulated"  v ( g r a1 n y u l ei t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "granulates"  v ( g r a1 n y u l ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "granulating"  v ( g r a1 n y u l ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "granule"  n ( g r a1 n y uu l ) ((pos "K6%" ) ))
( "granules"  n ( g r a1 n y uu l z ) ((pos "Kj%" ) ))
( "grape"  n ( g r ei1 p ) ((pos "K6%" ) ))
( "grape-sugar"  n ( g r ei1 p - sh u g @ r ) ((pos "L@%" ) ))
( "grapefruit"  n ( g r ei1 p f r uu t ) ((pos "M9%" ) ))
( "grapefruits"  n ( g r ei1 p f r uu t s ) ((pos "Kj%" ) ))
( "grapes"  n ( g r ei1 p s ) ((pos "Kj%" ) ))
( "grapeshot"  n ( g r ei1 p sh o t ) ((pos "L@%" ) ))
( "grapevine"  n ( g r ei1 p v ai n ) ((pos "K6%" ) ))
( "grapevines"  n ( g r ei1 p v ai n z ) ((pos "Kj%" ) ))
( "graph"  n ( g r aa1 f ) ((pos "K6%" ) ))
( "graphic"  j ( g r a1 f i k ) ((pos "OA%" ) ))
( "graphical"  j ( g r a1 f i k l ) ((pos "OA%" ) ))
( "graphically"  a ( g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "graphics"  n ( g r a1 f i k s ) ((pos "Lk%" ) ))
( "graphite"  n ( g r a1 f ai t ) ((pos "L@%" ) ))
( "graphs"  n ( g r aa1 f s ) ((pos "Kj%" ) ))
( "grapnel"  n ( g r a1 p n l ) ((pos "K6$" ) ))
( "grapnels"  n ( g r a1 p n l z ) ((pos "Kj$" ) ))
( "grapple"  v ( g r a1 p l ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "grappled"  v ( g r a1 p l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "grapples"  v ( g r a1 p l z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "grappling"  v ( g r a1 p l i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "grappling-iron"  n ( g r a1 p l i ng - ai @ n ) ((pos "K6$" ) ))
( "grappling-irons"  n ( g r a1 p l i ng - ai @ n z ) ((pos "Kj$" ) ))
( "grasp"  v ( g r aa1 s p ) ((pos "J0%" "K6%" ) (subcat "3A" )))
( "grasped"  v ( g r aa1 s p t ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "grasping"  v ( g r aa1 s p i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "grasps"  v ( g r aa1 s p s ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "grass"  v ( g r aa1 s ) ((pos "J1%" "M7%" ) (subcat "2A" "3A" "6A" )))
( "grassed"  v ( g r aa1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "grasses"  v ( g r aa1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "grasshopper"  n ( g r aa1 s h o p @ r ) ((pos "K6%" ) ))
( "grasshoppers"  n ( g r aa1 s h o p @ z ) ((pos "Kj%" ) ))
( "grassier"  j ( g r aa1 s i@ r ) ((pos "Or%" ) ))
( "grassiest"  j ( g r aa1 s i i s t ) ((pos "Os%" ) ))
( "grassing"  v ( g r aa1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "grassland"  n ( g r aa1 s l a n d ) ((pos "K6%" ) ))
( "grasslands"  n ( g r aa1 s l a n d z ) ((pos "Kj%" ) ))
( "grassroots"  n ( g r aa1 s r uu t s ) ((pos "Kj%" ) ))
( "grassy"  j ( g r aa1 s ii ) ((pos "OD%" ) ))
( "grate"  v ( g r ei1 t ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "grated"  v ( g r ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "grateful"  j ( g r ei1 t f @ l ) ((pos "OA%" ) ))
( "gratefully"  a ( g r ei1 t f @ l ii ) ((pos "Pu%" ) ))
( "gratefulness"  n ( g r ei1 t f @ l n @ s ) ((pos "L@%" ) ))
( "grater"  n ( g r ei1 t @ r ) ((pos "K6%" ) ))
( "graters"  n ( g r ei1 t @ z ) ((pos "Kj%" ) ))
( "grates"  v ( g r ei1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "gratification"  n ( g r a2 t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "gratifications"  n ( g r a2 t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "gratified"  v ( g r a1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gratifies"  v ( g r a1 t i f ai z ) ((pos "Ha%" ) (subcat )))
( "gratify"  v ( g r a1 t i f ai ) ((pos "H3%" ) (subcat )))
( "gratifying"  v ( g r a1 t i f ai i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "grating"  v ( g r ei1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "gratingly"  a ( g r ei1 t i ng l ii ) ((pos "Pu%" ) ))
( "gratings"  n ( g r ei1 t i ng z ) ((pos "Kj$" ) ))
( "gratis"  j ( g r a1 t i s ) ((pos "OA%" "Pu%" ) ))
( "gratitude"  n ( g r a1 t i t y uu d ) ((pos "L@%" ) ))
( "gratuities"  n ( g r @1 t y uu1 i t i z ) ((pos "Kj%" ) ))
( "gratuitous"  j ( g r @1 t y uu1 i t @ s ) ((pos "OA%" ) ))
( "gratuitously"  a ( g r @1 t y uu1 i t @ s l ii ) ((pos "Pu%" ) ))
( "gratuity"  n ( g r @1 t y uu1 i t ii ) ((pos "K8%" ) ))
( "grave"  n ( g r aa1 v ) ((pos "K6%" ) ))
( "grave"  v ( g r ei1 v ) ((pos "H5$" "K6%" "OB%" ) ))
( "grave-clothes"  n ( g r ei1 v - k l ou dh z ) ((pos "Kj%" ) ))
( "gravel"  v ( g r a1 v @ l ) ((pos "H4$" "L@%" ) (subcat )))
( "gravelled"  v ( g r a1 v @ l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "gravelling"  v ( g r a1 v @ l i ng ) ((pos "Hb$" ) (subcat )))
( "gravelly"  j ( g r a1 v @ l ii ) ((pos "OA%" ) ))
( "gravels"  v ( g r a1 v @ l z ) ((pos "Ha$" ) (subcat )))
( "gravely"  a ( g r ei1 v l ii ) ((pos "Pu$" ) ))
( "graven"  v ( g r ei1 v @ n ) ((pos "Hd$" "OA%" ) ))
( "graver"  j ( g r ei1 v @ r ) ((pos "Or%" ) ))
( "graves"  n ( g r aa1 v z ) ((pos "Kj$" ) ))
( "graves"  n ( g r ei1 v z ) ((pos "Kj%" ) ))
( "gravest"  j ( g r ei1 v i s t ) ((pos "Os%" ) ))
( "gravestone"  n ( g r ei1 v s t ou n ) ((pos "K6%" ) ))
( "gravestones"  n ( g r ei1 v s t ou n z ) ((pos "Kj%" ) ))
( "graveyard"  n ( g r ei1 v y aa d ) ((pos "K6%" ) ))
( "graveyards"  n ( g r ei1 v y aa d z ) ((pos "Kj%" ) ))
( "graving dock"  n ( g r ei1 v i ng - d o k ) ((pos "K6$" ) ))
( "graving docks"  n ( g r ei1 v i ng - d o k s ) ((pos "Kj$" ) ))
( "gravitate"  v ( g r a1 v i t ei t ) ((pos "I2%" ) (subcat )))
( "gravitated"  v ( g r a1 v i t ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "gravitates"  v ( g r a1 v i t ei t s ) ((pos "Ia%" ) (subcat )))
( "gravitating"  v ( g r a1 v i t ei t i ng ) ((pos "Ib%" ) (subcat )))
( "gravitation"  n ( g r a2 v i t ei1 sh @ n ) ((pos "L@%" ) ))
( "gravity"  n ( g r a1 v i t ii ) ((pos "L@%" ) ))
( "gravure"  n ( g r @1 v y u@1 r ) ((pos "K6$" ) ))
( "gravures"  n ( g r @1 v y u@1 z ) ((pos "Kj$" ) ))
( "gravy"  n ( g r ei1 v ii ) ((pos "L@%" ) ))
( "gravy-boat"  n ( g r ei1 v i - b ou t ) ((pos "K6%" ) ))
( "gravy-boats"  n ( g r ei1 v i - b ou t s ) ((pos "Kj%" ) ))
( "gray"  v ( g r ei1 ) ((pos "J0$" "M6%" "OC%" ) (subcat "2A" )))
( "grayed"  v ( g r ei1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "grayer"  j ( g r ei1 @ r ) ((pos "Or%" ) ))
( "grayest"  j ( g r ei1 i s t ) ((pos "Os%" ) ))
( "graying"  v ( g r ei1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "grays"  v ( g r ei1 z ) ((pos "Ja$" "Mj%" ) (subcat "2A" )))
( "graze"  v ( g r ei1 z ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "grazed"  v ( g r ei1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "grazes"  v ( g r ei1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "grazier"  n ( g r ei1 z i@ r ) ((pos "K6$" ) ))
( "graziers"  n ( g r ei1 z i@ z ) ((pos "Kj$" ) ))
( "grazing"  v ( g r ei1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "grazing-land"  n ( g r ei1 z i ng - l a n d ) ((pos "K6%" ) ))
( "grazing-lands"  n ( g r ei1 z i ng - l a n d z ) ((pos "Kj%" ) ))
( "grease"  v ( g r ii1 s ) ((pos "H2%" "L@%" ) ))
( "grease-gun"  n ( g r ii1 s - g uh n ) ((pos "K6%" ) ))
( "grease-guns"  n ( g r ii1 s - g uh n z ) ((pos "Kj%" ) ))
( "greased"  v ( g r ii1 s t ) ((pos "Hc%" "Hd%" ) ))
( "greasepaint"  n ( g r ii1 s p ei n t ) ((pos "L@%" ) ))
( "greaseproof"  j ( g r ii1 s p r uu f ) ((pos "OA%" ) ))
( "greaser"  n ( g r ii1 s @ r ) ((pos "K6%" ) ))
( "greasers"  n ( g r ii1 s @ z ) ((pos "Kj%" ) ))
( "greases"  v ( g r ii1 s i z ) ((pos "Ha%" ) ))
( "greasier"  j ( g r ii1 s i@ r ) ((pos "Or%" ) ))
( "greasiest"  j ( g r ii1 s i i s t ) ((pos "Os%" ) ))
( "greasily"  a ( g r ii1 s i l ii ) ((pos "Pu%" ) ))
( "greasiness"  n ( g r ii1 s i n @ s ) ((pos "L@%" ) ))
( "greasing"  v ( g r ii1 s i ng ) ((pos "Hb%" ) ))
( "greasy"  j ( g r ii1 s ii ) ((pos "OD%" ) ))
( "great"  j ( g r ei1 t ) ((pos "OC*" ) ))
( "greatcoat"  n ( g r ei1 t k ou t ) ((pos "K6%" ) ))
( "greatcoats"  n ( g r ei1 t k ou t s ) ((pos "Kj%" ) ))
( "greater"  j ( g r ei1 t @ r ) ((pos "Or%" ) ))
( "greatest"  j ( g r ei1 t i s t ) ((pos "Os%" ) ))
( "greatly"  a ( g r ei1 t l ii ) ((pos "Pu%" ) ))
( "greatness"  n ( g r ei1 t n @ s ) ((pos "L@%" ) ))
( "greaves"  n ( g r ii1 v z ) ((pos "Kj$" ) ))
( "grebe"  n ( g r ii1 b ) ((pos "K6$" ) ))
( "grebes"  n ( g r ii1 b z ) ((pos "Kj$" ) ))
( "greed"  n ( g r ii1 d ) ((pos "L@%" ) ))
( "greedier"  j ( g r ii1 d i@ r ) ((pos "Or%" ) ))
( "greediest"  j ( g r ii1 d i i s t ) ((pos "Os%" ) ))
( "greedily"  a ( g r ii1 d i l ii ) ((pos "Pu%" ) ))
( "greediness"  n ( g r ii1 d i n @ s ) ((pos "L@%" ) ))
( "greedy"  j ( g r ii1 d ii ) ((pos "OD%" ) ))
( "green"  n ( g r ii1 n ) ((pos "M6%" "OC%" ) ))
( "greenback"  n ( g r ii1 n b a k ) ((pos "K6$" ) ))
( "greenbacks"  n ( g r ii1 n b a k s ) ((pos "Kj$" ) ))
( "greener"  j ( g r ii1 n @ r ) ((pos "Or%" ) ))
( "greenery"  n ( g r ii1 n @ r ii ) ((pos "L@%" ) ))
( "greenest"  j ( g r ii1 n i s t ) ((pos "Os%" ) ))
( "greeneyed"  j ( g r ii1 n ai d ) ((pos "OA%" ) ))
( "greenfly"  n ( g r ii1 n f l ai ) ((pos "L@%" ) ))
( "greengage"  n ( g r ii1 n g ei jh ) ((pos "K6%" ) ))
( "greengages"  n ( g r ii1 n g ei jh i z ) ((pos "Kj%" ) ))
( "greengrocer"  n ( g r ii1 n g r ou s @ r ) ((pos "K6%" ) ))
( "greengroceries"  n ( g r ii1 n g r ou s @ r i z ) ((pos "Kj$" ) ))
( "greengrocers"  n ( g r ii1 n g r ou s @ z ) ((pos "Kj%" ) ))
( "greengrocery"  n ( g r ii1 n g r ou s @ r ii ) ((pos "K8%" ) ))
( "greenhorn"  n ( g r ii1 n h oo n ) ((pos "K6$" ) ))
( "greenhorns"  n ( g r ii1 n h oo n z ) ((pos "Kj$" ) ))
( "greenhouse"  n ( g r ii1 n h au s ) ((pos "K6%" ) ))
( "greenhouses"  n ( g r ii1 n h au z i z ) ((pos "Kj%" ) ))
( "greenish"  j ( g r ii1 n i sh ) ((pos "OA%" ) ))
( "greenishness"  n ( g r ii1 n i sh n @ s ) ((pos "L@%" ) ))
( "greens"  n ( g r ii1 n z ) ((pos "Mj%" ) ))
( "greenstuffs"  n ( g r ii1 n s t uh f s ) ((pos "Kj$" ) ))
( "greensward"  n ( g r ii1 n s w oo d ) ((pos "L@$" ) ))
( "greenwood"  n ( g r ii1 n w u d ) ((pos "K6$" ) ))
( "greenwoods"  n ( g r ii1 n w u d z ) ((pos "Kj$" ) ))
( "greet"  v ( g r ii1 t ) ((pos "H0%" ) (subcat "6A" )))
( "greeted"  v ( g r ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "greeting"  v ( g r ii1 t i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" )))
( "greetings"  n ( g r ii1 t i ng z ) ((pos "Kj%" ) ))
( "greets"  v ( g r ii1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "gregarious"  j ( g r i1 g e@1 r i@ s ) ((pos "OA%" ) ))
( "gregariously"  a ( g r i1 g e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "gregariousness"  n ( g r i1 g e@1 r i@ s n @ s ) ((pos "L@%" ) ))
( "gremlin"  n ( g r e1 m l i n ) ((pos "K6%" ) ))
( "gremlins"  n ( g r e1 m l i n z ) ((pos "Kj%" ) ))
( "grenade"  n ( g r i1 n ei1 d ) ((pos "K6%" ) ))
( "grenades"  n ( g r i1 n ei1 d z ) ((pos "Kj%" ) ))
( "grenadier"  n ( g r e2 n @ d i@1 r ) ((pos "K6%" ) ))
( "grenadiers"  n ( g r e2 n @ d i@1 z ) ((pos "Kj%" ) ))
( "grew"  v ( g r uu1 ) ((pos "Jc%" ) (subcat "2A" "2C" "2D" "3A" "4A" "6A" "12B" )))
( "grey"  v ( g r ei1 ) ((pos "J0$" "M6%" "OC%" ) (subcat "2A" )))
( "grey-headed"  j ( g r ei1 - h e1 d i d ) ((pos "OA%" ) ))
( "greybeard"  n ( g r ei1 b i@ d ) ((pos "K6%" ) ))
( "greybeards"  n ( g r ei1 b i@ d z ) ((pos "Kj%" ) ))
( "greyed"  v ( g r ei1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "greyer"  j ( g r ei1 @ r ) ((pos "Or%" ) ))
( "greyest"  j ( g r ei1 i s t ) ((pos "Os%" ) ))
( "greyhound"  n ( g r ei1 h au n d ) ((pos "K6%" ) ))
( "greyhounds"  n ( g r ei1 h au n d z ) ((pos "Kj%" ) ))
( "greying"  v ( g r ei1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "greyish"  j ( g r ei1 i sh ) ((pos "OA%" ) ))
( "greyly"  a ( g r ei1 l ii ) ((pos "Pu%" ) ))
( "greys"  v ( g r ei1 z ) ((pos "Ja$" "Mj%" ) (subcat "2A" )))
( "grid"  n ( g r i1 d ) ((pos "K6%" ) ))
( "griddle"  n ( g r i1 d l ) ((pos "K6%" ) ))
( "griddles"  n ( g r i1 d l z ) ((pos "Kj%" ) ))
( "gridiron"  n ( g r i1 d ai @ n ) ((pos "K6%" ) ))
( "gridirons"  n ( g r i1 d ai @ n z ) ((pos "Kj$" ) ))
( "grids"  n ( g r i1 d z ) ((pos "Kj%" ) ))
( "grief"  n ( g r ii1 f ) ((pos "M6%" ) ))
( "griefs"  n ( g r ii1 f s ) ((pos "Mj%" ) ))
( "grievance"  n ( g r ii1 v n s ) ((pos "K6%" ) ))
( "grievances"  n ( g r ii1 v n s i z ) ((pos "Kj%" ) ))
( "grieve"  v ( g r ii1 v ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "grieved"  v ( g r ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "grieves"  v ( g r ii1 v z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "grieving"  v ( g r ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "grievous"  j ( g r ii1 v @ s ) ((pos "OA%" ) ))
( "grievously"  a ( g r ii1 v @ s l ii ) ((pos "Pu%" ) ))
( "griffin"  n ( g r i1 f i n ) ((pos "K6$" ) ))
( "griffins"  n ( g r i1 f i n z ) ((pos "Kj$" ) ))
( "griffon"  n ( g r i1 f @ n ) ((pos "K6$" ) ))
( "griffons"  n ( g r i1 f @ n z ) ((pos "Kj$" ) ))
( "grill"  v ( g r i1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "grille"  n ( g r i1 l ) ((pos "K6$" ) ))
( "grilled"  v ( g r i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "grilles"  n ( g r i1 l z ) ((pos "Kj$" ) ))
( "grilling"  v ( g r i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "grillroom"  n ( g r i1 l r uu m ) ((pos "K6%" ) ))
( "grillrooms"  n ( g r i1 l r uu m z ) ((pos "Kj%" ) ))
( "grills"  v ( g r i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "grim"  j ( g r i1 m ) ((pos "OE%" ) ))
( "grimace"  v ( g r i1 m ei1 s ) ((pos "I2%" "K6%" ) (subcat )))
( "grimaced"  v ( g r i1 m ei1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "grimaces"  v ( g r i1 m ei1 s i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "grimacing"  v ( g r i1 m ei1 s i ng ) ((pos "Ib%" ) (subcat )))
( "grime"  v ( g r ai1 m ) ((pos "H2$" "L@%" ) (subcat )))
( "grimed"  v ( g r ai1 m d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "grimes"  v ( g r ai1 m z ) ((pos "Ha$" ) (subcat )))
( "grimier"  j ( g r ai1 m i@ r ) ((pos "Or%" ) ))
( "grimiest"  j ( g r ai1 m i i s t ) ((pos "Os%" ) ))
( "griming"  v ( g r ai1 m i ng ) ((pos "Hb$" ) (subcat )))
( "grimly"  a ( g r i1 m l ii ) ((pos "Pu%" ) ))
( "grimmer"  j ( g r i1 m @ r ) ((pos "Or%" ) ))
( "grimmest"  j ( g r i1 m i s t ) ((pos "Os%" ) ))
( "grimness"  n ( g r i1 m n @ s ) ((pos "L@%" ) ))
( "grimy"  j ( g r ai1 m ii ) ((pos "OD%" ) ))
( "grin"  v ( g r i1 n ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "grind"  v ( g r ai1 n d ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "grinder"  n ( g r ai1 n d @ r ) ((pos "K6%" ) ))
( "grinders"  n ( g r ai1 n d @ z ) ((pos "Kj%" ) ))
( "grinding"  v ( g r ai1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "grinds"  v ( g r ai1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "grindstone"  n ( g r ai1 n d s t ou n ) ((pos "K6$" ) ))
( "grindstones"  n ( g r ai1 n d s t ou n z ) ((pos "Kj$" ) ))
( "grinned"  v ( g r i1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "grinning"  v ( g r i1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "grins"  v ( g r i1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "grip"  v ( g r i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "gripes"  n ( g r ai1 p s ) ((pos "Kj%" ) ))
( "grippe"  n ( g r ii1 p ) ((pos "K6$" ) ))
( "gripped"  v ( g r i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "grippes"  n ( g r ii1 p s ) ((pos "Kj$" ) ))
( "gripping"  v ( g r i1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "grips"  v ( g r i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "gripsack"  n ( g r i1 p s a k ) ((pos "K6$" ) ))
( "gripsacks"  n ( g r i1 p s a k s ) ((pos "Kj$" ) ))
( "grisly"  j ( g r i1 z l ii ) ((pos "OA%" ) ))
( "grist"  n ( g r i1 s t ) ((pos "L@%" ) ))
( "gristle"  n ( g r i1 s l ) ((pos "L@%" ) ))
( "grit"  v ( g r i1 t ) ((pos "H4%" "L@%" ) ))
( "grits"  v ( g r i1 t s ) ((pos "Ha%" "Lk%" ) ))
( "gritted"  v ( g r i1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "grittier"  j ( g r i1 t i@ r ) ((pos "Or%" ) ))
( "grittiest"  j ( g r i1 t i i s t ) ((pos "Os%" ) ))
( "gritting"  v ( g r i1 t i ng ) ((pos "Hb%" ) ))
( "gritty"  j ( g r i1 t ii ) ((pos "OD%" ) ))
( "grizzle"  v ( g r i1 z l ) ((pos "I2%" ) ))
( "grizzled"  v ( g r i1 z l d ) ((pos "Ic%" "Id%" "OA%" ) ))
( "grizzles"  v ( g r i1 z l z ) ((pos "Ia%" ) ))
( "grizzlies"  n ( g r i1 z l i z ) ((pos "Kj$" ) ))
( "grizzling"  v ( g r i1 z l i ng ) ((pos "Ib%" ) ))
( "grizzly"  n ( g r i1 z l ii ) ((pos "K8$" ) ))
( "groan"  v ( g r ou1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "groaned"  v ( g r ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "groaning"  v ( g r ou1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "groans"  v ( g r ou1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "groat"  n ( g r ou1 t ) ((pos "K6$" ) ))
( "groats"  n ( g r ou1 t s ) ((pos "Kj$" ) ))
( "grocer"  n ( g r ou1 s @ r ) ((pos "K6%" ) ))
( "groceries"  n ( g r ou1 s @ r i z ) ((pos "Mj%" ) ))
( "grocers"  n ( g r ou1 s @ z ) ((pos "Kj%" ) ))
( "grocery"  n ( g r ou1 s @ r ii ) ((pos "M8%" ) ))
( "grog"  n ( g r o1 g ) ((pos "L@%" ) ))
( "groggier"  j ( g r o1 g i@ r ) ((pos "Or$" ) ))
( "groggiest"  j ( g r o1 g i i s t ) ((pos "Os$" ) ))
( "groggy"  j ( g r o1 g ii ) ((pos "OD%" ) ))
( "groin"  v ( g r oi1 n ) ((pos "H0$" "K6%" ) ))
( "groined"  v ( g r oi1 n d ) ((pos "Hc$" "Hd%" ) ))
( "groining"  v ( g r oi1 n i ng ) ((pos "Hb$" ) ))
( "groins"  v ( g r oi1 n z ) ((pos "Ha$" "Kj$" ) ))
( "groom"  v ( g r uu1 m ) ((pos "H0%" "K6%" ) (subcat )))
( "groomed"  v ( g r uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "grooming"  v ( g r uu1 m i ng ) ((pos "Hb%" ) (subcat )))
( "grooms"  v ( g r uu1 m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "groove"  v ( g r uu1 v ) ((pos "H2$" "K6%" ) ))
( "grooved"  v ( g r uu1 v d ) ((pos "Hc$" "Hd%" ) ))
( "groover"  n ( g r uu1 v @ r ) ((pos "K6%" ) ))
( "groovers"  n ( g r uu1 v @ z ) ((pos "Kj%" ) ))
( "grooves"  v ( g r uu1 v z ) ((pos "Ha$" "Kj%" ) ))
( "groovier"  j ( g r uu1 v i@ r ) ((pos "Or%" ) ))
( "grooviest"  j ( g r uu1 v i i s t ) ((pos "Os%" ) ))
( "grooving"  v ( g r uu1 v i ng ) ((pos "Hb$" ) ))
( "groovy"  j ( g r uu1 v ii ) ((pos "OD%" ) ))
( "grope"  v ( g r ou1 p ) ((pos "J2%" ) (subcat "2A" "2C" "3A" )))
( "groped"  v ( g r ou1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "gropes"  v ( g r ou1 p s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "groping"  v ( g r ou1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "gropingly"  a ( g r ou1 p i ng l ii ) ((pos "Pu%" ) ))
( "gross"  v ( g r ou1 s ) ((pos "H1%" "K9%" "OA%" ) (subcat )))
( "grossed"  v ( g r ou1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "grosses"  v ( g r ou1 s i z ) ((pos "Ha%" ) (subcat )))
( "grossing"  v ( g r ou1 s i ng ) ((pos "Hb%" ) (subcat )))
( "grossly"  a ( g r ou1 s l ii ) ((pos "Pu%" ) ))
( "grossness"  n ( g r ou1 s n @ s ) ((pos "L@%" ) ))
( "grot"  n ( g r o1 t ) ((pos "M6%" ) ))
( "grotesque"  n ( g r ou1 t e1 s k ) ((pos "K6%" "OA%" ) ))
( "grotesquely"  a ( g r ou1 t e1 s k l ii ) ((pos "Pu%" ) ))
( "grotesqueness"  n ( g r ou1 t e1 s k n @ s ) ((pos "L@%" ) ))
( "grotesques"  n ( g r ou1 t e1 s k s ) ((pos "Kj$" ) ))
( "grots"  n ( g r o1 t s ) ((pos "Mj$" ) ))
( "grottier"  j ( g r o1 t i@ r ) ((pos "Or%" ) ))
( "grottiest"  j ( g r o1 t i i s t ) ((pos "Os%" ) ))
( "grotto"  n ( g r o1 t ou ) ((pos "K6%" ) ))
( "grottoes"  n ( g r o1 t ou z ) ((pos "Kj%" ) ))
( "grottos"  n ( g r o1 t ou z ) ((pos "Kj%" ) ))
( "grotty"  j ( g r o1 t ii ) ((pos "OD%" ) ))
( "grouch"  v ( g r au1 ch ) ((pos "I1%" "K7%" ) ))
( "grouched"  v ( g r au1 ch t ) ((pos "Ic%" "Id%" ) ))
( "grouches"  v ( g r au1 ch i z ) ((pos "Ia%" "Kj%" ) ))
( "grouchier"  j ( g r au1 ch i@ r ) ((pos "Or$" ) ))
( "grouchiest"  j ( g r au1 ch i i s t ) ((pos "Os$" ) ))
( "grouching"  v ( g r au1 ch i ng ) ((pos "Ib%" ) ))
( "grouchy"  j ( g r au1 ch ii ) ((pos "OD%" ) ))
( "ground"  v ( g r au1 n d ) ((pos "J0%" "Jc%" "Jd%" "M6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "ground-bait"  n ( g r au1 n d - b ei t ) ((pos "L@%" ) ))
( "ground-fish"  n ( g r au1 n d - f i sh ) ((pos "K9$" ) ))
( "ground-fishes"  n ( g r au1 n d - f i sh i z ) ((pos "Kj$" ) ))
( "ground-plan"  n ( g r au1 n d - p l a n ) ((pos "K6%" ) ))
( "ground-plans"  n ( g r au1 n d - p l a n z ) ((pos "Kj%" ) ))
( "ground-rent"  n ( g r au1 n d - r e n t ) ((pos "M6%" ) ))
( "ground-rents"  n ( g r au1 n d - r e n t s ) ((pos "Mj$" ) ))
( "grounded"  v ( g r au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "grounding"  v ( g r au1 n d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "groundings"  n ( g r au1 n d i ng z ) ((pos "Kj%" ) ))
( "groundless"  j ( g r au1 n d l i s ) ((pos "OA%" ) ))
( "groundnut"  n ( g r au1 n d n uh t ) ((pos "K6%" ) ))
( "groundnuts"  n ( g r au1 n d n uh t s ) ((pos "Kj%" ) ))
( "grounds"  v ( g r au1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "groundsel"  n ( g r au1 n s l ) ((pos "L@$" ) ))
( "groundsheet"  n ( g r au1 n d sh ii t ) ((pos "K6%" ) ))
( "groundsheets"  n ( g r au1 n d sh ii t s ) ((pos "Kj%" ) ))
( "groundsman"  n ( g r au1 n d z m @ n ) ((pos "Ki%" ) ))
( "groundsmen"  n ( g r au1 n d z m @ n ) ((pos "Kj%" ) ))
( "groundwork"  n ( g r au1 n d w @@ k ) ((pos "L@%" ) ))
( "group"  v ( g r uu1 p ) ((pos "J0%" "K6*" ) (subcat "2C" "6A" "15A" )))
( "grouped"  v ( g r uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "grouping"  v ( g r uu1 p i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "groupings"  n ( g r uu1 p i ng z ) ((pos "Kj%" ) ))
( "groups"  v ( g r uu1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "grouse"  v ( g r au1 s ) ((pos "I2%" "K9%" ) (subcat "2A" )))
( "groused"  v ( g r au1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "grouses"  v ( g r au1 s i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "grousing"  v ( g r au1 s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "grove"  n ( g r ou1 v ) ((pos "K6%" ) ))
( "grovel"  v ( g r o1 v @ l ) ((pos "I4%" ) (subcat "2A" )))
( "grovelled"  v ( g r o1 v @ l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "groveller"  n ( g r o1 v l @ r ) ((pos "K6%" ) ))
( "grovellers"  n ( g r o1 v l @ z ) ((pos "Kj%" ) ))
( "grovelling"  v ( g r o1 v @ l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "grovels"  v ( g r o1 v @ l z ) ((pos "Ia%" ) (subcat "2A" )))
( "groves"  n ( g r ou1 v z ) ((pos "Kj%" ) ))
( "grow"  v ( g r ou1 ) ((pos "J5%" ) (subcat "2A" "2C" "2D" "3A" "4A" "6A" "12B" )))
( "grower"  n ( g r ou1 @ r ) ((pos "K6%" ) ))
( "growers"  n ( g r ou1 @ z ) ((pos "Kj%" ) ))
( "growing"  v ( g r ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "4A" "6A" "12B" )))
( "growing-pains"  n ( g r ou1 i ng - p ei n z ) ((pos "Kj%" ) ))
( "growl"  v ( g r au1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "growled"  v ( g r au1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "growler"  n ( g r au1 l @ r ) ((pos "K6%" ) ))
( "growlers"  n ( g r au1 l @ z ) ((pos "Kj%" ) ))
( "growling"  v ( g r au1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "growlingly"  a ( g r au1 l i ng l ii ) ((pos "Pu$" ) ))
( "growls"  v ( g r au1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "grown"  v ( g r ou1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "2D" "3A" "4A" "6A" "12B" )))
( "grown-up"  n ( g r ou1 n - uh p ) ((pos "K6%" "OA%" ) ))
( "grown-ups"  n ( g r ou1 n - uh p s ) ((pos "Kj%" ) ))
( "grows"  v ( g r ou1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "2D" "3A" "4A" "6A" "12B" )))
( "growth"  n ( g r ou1 th ) ((pos "M6%" ) ))
( "growths"  n ( g r ou1 th s ) ((pos "Mj%" ) ))
( "groyne"  n ( g r oi1 n ) ((pos "K6$" ) ))
( "groynes"  n ( g r oi1 n z ) ((pos "Kj$" ) ))
( "grub"  v ( g r uh1 b ) ((pos "J4$" "M6%" ) (subcat "2C" "6A" )))
( "grubbed"  v ( g r uh1 b d ) ((pos "Jc$" "Jd$" ) (subcat "2C" "6A" )))
( "grubbier"  j ( g r uh1 b i@ r ) ((pos "Or%" ) ))
( "grubbiest"  j ( g r uh1 b i i s t ) ((pos "Os%" ) ))
( "grubbiness"  n ( g r uh1 b i n i s ) ((pos "L@%" ) ))
( "grubbing"  v ( g r uh1 b i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "grubby"  j ( g r uh1 b ii ) ((pos "OD%" ) ))
( "grubs"  v ( g r uh1 b z ) ((pos "Ja$" "Mj%" ) (subcat "2C" "6A" )))
( "grudge"  v ( g r uh1 jh ) ((pos "H2%" "K6%" ) (subcat "6C" "12A" )))
( "grudged"  v ( g r uh1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6C" "12A" )))
( "grudges"  v ( g r uh1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6C" "12A" )))
( "grudging"  v ( g r uh1 jh i ng ) ((pos "Hb%" ) (subcat "6C" "12A" )))
( "grudgingly"  a ( g r uh1 jh i ng l ii ) ((pos "Pu%" ) ))
( "gruel"  n ( g r uu1 @ l ) ((pos "L@$" ) ))
( "gruelling"  j ( g r uu1 @ l i ng ) ((pos "OA%" ) ))
( "gruesome"  j ( g r uu1 s @ m ) ((pos "OA%" ) ))
( "gruesomely"  a ( g r uu1 s @ m l ii ) ((pos "Pu%" ) ))
( "gruesomeness"  n ( g r uu1 s @ m n @ s ) ((pos "L@$" ) ))
( "gruff"  j ( g r uh1 f ) ((pos "OC%" ) ))
( "gruffer"  j ( g r uh1 f @ r ) ((pos "Or%" ) ))
( "gruffest"  j ( g r uh1 f i s t ) ((pos "Os%" ) ))
( "gruffly"  a ( g r uh1 f l ii ) ((pos "Pu%" ) ))
( "gruffness"  n ( g r uh1 f n @ s ) ((pos "L@%" ) ))
( "grumble"  v ( g r uh1 m b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "grumbled"  v ( g r uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "grumbler"  n ( g r uh1 m b l @ r ) ((pos "K6%" ) ))
( "grumblers"  n ( g r uh1 m b l @ z ) ((pos "Kj%" ) ))
( "grumbles"  v ( g r uh1 m b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "grumbling"  v ( g r uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "grumpier"  j ( g r uh1 m p i@ r ) ((pos "Or%" ) ))
( "grumpiest"  j ( g r uh1 m p i i s t ) ((pos "Os%" ) ))
( "grumpily"  a ( g r uh1 m p i l ii ) ((pos "Pu%" ) ))
( "grumpiness"  n ( g r uh1 m p i n @ s ) ((pos "L@%" ) ))
( "grumpy"  j ( g r uh1 m p ii ) ((pos "OD%" ) ))
( "grunt"  v ( g r uh1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "grunted"  v ( g r uh1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "grunting"  v ( g r uh1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "grunts"  v ( g r uh1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "gryphon"  n ( g r i1 f @ n ) ((pos "K6$" ) ))
( "gryphons"  n ( g r i1 f @ n z ) ((pos "Kj$" ) ))
( "guano"  n ( g w aa1 n ou ) ((pos "M6$" ) ))
( "guanos"  n ( g w aa1 n ou z ) ((pos "Mj$" ) ))
( "guarantee"  v ( g a2 r @ n t ii1 ) ((pos "H5%" "K6%" ) (subcat "6A" "7A" "9" "12A" "13A" )))
( "guaranteed"  v ( g a2 r @ n t ii1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" "9" "12A" "13A" )))
( "guaranteeing"  v ( g a2 r @ n t ii1 i ng ) ((pos "Hb%" ) (subcat "6A" "7A" "9" "12A" "13A" )))
( "guarantees"  v ( g a2 r @ n t ii1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "7A" "9" "12A" "13A" )))
( "guaranties"  n ( g a1 r @ n t i z ) ((pos "Kj$" ) ))
( "guarantor"  n ( g a2 r @ n t oo1 r ) ((pos "K6%" ) ))
( "guarantors"  n ( g a2 r @ n t oo1 z ) ((pos "Kj%" ) ))
( "guaranty"  n ( g a1 r @ n t ii ) ((pos "K8$" ) ))
( "guard"  v ( g aa1 d ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" )))
( "guard-boat"  n ( g aa1 d - b ou t ) ((pos "K6$" ) ))
( "guard-boats"  n ( g aa1 d - b ou t s ) ((pos "Kj$" ) ))
( "guarded"  v ( g aa1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" )))
( "guardedly"  a ( g aa1 d i d l ii ) ((pos "Pu%" ) ))
( "guardhouse"  n ( g aa1 d h au s ) ((pos "K6%" ) ))
( "guardhouses"  n ( g aa1 d h au z i z ) ((pos "Kj%" ) ))
( "guardian"  n ( g aa1 d i@ n ) ((pos "K6%" ) ))
( "guardians"  n ( g aa1 d i@ n z ) ((pos "Kj%" ) ))
( "guardianship"  n ( g aa1 d i@ n sh i p ) ((pos "K6%" ) ))
( "guardianships"  n ( g aa1 d i@ n sh i p s ) ((pos "Kj$" ) ))
( "guarding"  v ( g aa1 d i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "guardrail"  n ( g aa1 d r ei l ) ((pos "K6%" ) ))
( "guardrails"  n ( g aa1 d r ei l z ) ((pos "Kj%" ) ))
( "guardroom"  n ( g aa1 d r uu m ) ((pos "K6%" ) ))
( "guardrooms"  n ( g aa1 d r uu m z ) ((pos "Kj%" ) ))
( "guards"  v ( g aa1 d z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" )))
( "guardship"  n ( g aa1 d sh i p ) ((pos "K6$" ) ))
( "guardships"  n ( g aa1 d sh i p s ) ((pos "Kj$" ) ))
( "guardsman"  n ( g aa1 d z m @ n ) ((pos "Ki%" ) ))
( "guardsmen"  n ( g aa1 d z m @ n ) ((pos "Kj%" ) ))
( "guava"  n ( g w aa1 v @ ) ((pos "M6%" ) ))
( "guavas"  n ( g w aa1 v @ z ) ((pos "Mj%" ) ))
( "gudgeon"  n ( g uh1 jh @ n ) ((pos "K6$" ) ))
( "gudgeons"  n ( g uh1 jh @ n z ) ((pos "Kj$" ) ))
( "guelder rose"  n ( g e1 l d @ - r ou z ) ((pos "K6$" ) ))
( "guelder roses"  n ( g e1 l d @ - r ou z i z ) ((pos "Kj$" ) ))
( "guerilla"  n ( g @1 r i1 l @ ) ((pos "K6%" ) ))
( "guerillas"  n ( g @1 r i1 l @ z ) ((pos "Kj%" ) ))
( "guerrilla"  n ( g @1 r i1 l @ ) ((pos "K6$" ) ))
( "guerrillas"  n ( g @1 r i1 l @ z ) ((pos "Kj$" ) ))
( "guess"  v ( g e1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" "6A" "8" "9" "10" )))
( "guessed"  v ( g e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "8" "9" "10" )))
( "guesses"  v ( g e1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "8" "9" "10" )))
( "guessing"  v ( g e1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "8" "9" "10" )))
( "guesstimate"  n ( g e1 s t i m @ t ) ((pos "K6$" ) ))
( "guesstimates"  n ( g e1 s t i m @ t s ) ((pos "Kj$" ) ))
( "guesswork"  n ( g e1 s w @@ k ) ((pos "L@%" ) ))
( "guest"  n ( g e1 s t ) ((pos "K6%" ) ))
( "guest-night"  n ( g e1 s t - n ai t ) ((pos "K6%" ) ))
( "guest-nights"  n ( g e1 s t - n ai t s ) ((pos "Kj%" ) ))
( "guesthouse"  n ( g e1 s t h au s ) ((pos "K6%" ) ))
( "guesthouses"  n ( g e1 s t h au z i z ) ((pos "Kj%" ) ))
( "guestroom"  n ( g e1 s t r uu m ) ((pos "K6%" ) ))
( "guestrooms"  n ( g e1 s t r uu m z ) ((pos "Kj%" ) ))
( "guests"  n ( g e1 s t s ) ((pos "Kj%" ) ))
( "guffaw"  v ( g @1 f oo1 ) ((pos "I0%" "K6%" ) ))
( "guffawed"  v ( g @1 f oo1 d ) ((pos "Ic%" "Id%" ) ))
( "guffawing"  v ( g @1 f oo1 i ng ) ((pos "Ib$" ) ))
( "guffaws"  v ( g @1 f oo1 z ) ((pos "Ia%" "Kj%" ) ))
( "guidance"  n ( g ai1 d n s ) ((pos "L@%" ) ))
( "guide"  v ( g ai1 d ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "guidebook"  n ( g ai1 d b u k ) ((pos "K6%" ) ))
( "guidebooks"  n ( g ai1 d b u k s ) ((pos "Kj%" ) ))
( "guided"  v ( g ai1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "guideline"  n ( g ai1 d l ai n ) ((pos "K6%" ) ))
( "guidelines"  n ( g ai1 d l ai n z ) ((pos "Kj%" ) ))
( "guides"  v ( g ai1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "guiding"  v ( g ai1 d i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "guild"  n ( g i1 l d ) ((pos "K6%" ) ))
( "guilder"  n ( g i1 l d @ r ) ((pos "K6$" ) ))
( "guilders"  n ( g i1 l d @ z ) ((pos "Kj$" ) ))
( "guildhall"  n ( g i1 l d h oo l ) ((pos "K6%" ) ))
( "guildhalls"  n ( g i1 l d h oo l z ) ((pos "Kj$" ) ))
( "guilds"  n ( g i1 l d z ) ((pos "Kj%" ) ))
( "guile"  n ( g ai1 l ) ((pos "L@%" ) ))
( "guileful"  j ( g ai1 l f @ l ) ((pos "OA%" ) ))
( "guileless"  j ( g ai1 l - l @ s ) ((pos "OA%" ) ))
( "guillemot"  n ( g i1 l i m o t ) ((pos "K6$" ) ))
( "guillemots"  n ( g i1 l i m o t s ) ((pos "Kj$" ) ))
( "guillotine"  v ( g i1 l @ t ii n ) ((pos "H2%" "K6%" ) (subcat )))
( "guillotined"  v ( g i1 l @ t ii n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "guillotines"  v ( g i1 l @ t ii n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "guillotining"  v ( g i1 l @ t ii n i ng ) ((pos "Hb%" ) (subcat )))
( "guilt"  n ( g i1 l t ) ((pos "L@%" ) ))
( "guiltier"  j ( g i1 l t i@ r ) ((pos "Or%" ) ))
( "guiltiest"  j ( g i1 l t i i s t ) ((pos "Os%" ) ))
( "guiltily"  a ( g i1 l t i l ii ) ((pos "Pu%" ) ))
( "guiltiness"  n ( g i1 l t i n @ s ) ((pos "L@%" ) ))
( "guiltless"  j ( g i1 l t l @ s ) ((pos "OA%" ) ))
( "guilty"  j ( g i1 l t ii ) ((pos "OD%" ) ))
( "guinea"  n ( g i1 n ii ) ((pos "K6%" ) ))
( "guinea-fowl"  n ( g i1 n i - f au l ) ((pos "K9%" ) ))
( "guinea-pig"  n ( g i1 n i - p i g ) ((pos "K6%" ) ))
( "guinea-pigs"  n ( g i1 n i - p i g z ) ((pos "Kj%" ) ))
( "guineas"  n ( g i1 n i z ) ((pos "Kj%" ) ))
( "guise"  n ( g ai1 z ) ((pos "K6%" ) ))
( "guises"  n ( g ai1 z i z ) ((pos "Kj%" ) ))
( "guitar"  n ( g i1 t aa1 r ) ((pos "K6%" ) ))
( "guitarist"  n ( g i1 t aa1 r i s t ) ((pos "K6%" ) ))
( "guitarists"  n ( g i1 t aa1 r i s t s ) ((pos "Kj%" ) ))
( "guitars"  n ( g i1 t aa1 z ) ((pos "Kj%" ) ))
( "gulch"  n ( g uh1 l ch ) ((pos "K7$" ) ))
( "gulches"  n ( g uh1 l ch i z ) ((pos "Kj$" ) ))
( "gulden"  n ( g u1 l d @ n ) ((pos "K6$" ) ))
( "guldens"  n ( g u1 l d @ n z ) ((pos "Kj$" ) ))
( "gulf"  n ( g uh1 l f ) ((pos "K6%" ) ))
( "gulfs"  n ( g uh1 l f s ) ((pos "Kj%" ) ))
( "gull"  v ( g uh1 l ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "gulled"  v ( g uh1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "gullet"  n ( g uh1 l i t ) ((pos "K6%" ) ))
( "gullets"  n ( g uh1 l i t s ) ((pos "Kj%" ) ))
( "gullibility"  n ( g uh2 l @ b i1 l i t ii ) ((pos "L@%" ) ))
( "gullible"  j ( g uh1 l i b l ) ((pos "OA%" ) ))
( "gullies"  n ( g uh1 l i z ) ((pos "Kj%" ) ))
( "gulling"  v ( g uh1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "gulls"  v ( g uh1 l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "gully"  n ( g uh1 l ii ) ((pos "K8%" ) ))
( "gulp"  v ( g uh1 l p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "gulped"  v ( g uh1 l p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "gulping"  v ( g uh1 l p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "gulps"  v ( g uh1 l p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "gum"  v ( g uh1 m ) ((pos "H4%" "M6%" ) (subcat "6A" "15A" )))
( "gumbo"  n ( g uh1 m b ou ) ((pos "K6$" ) ))
( "gumboil"  n ( g uh1 m b oi l ) ((pos "K6%" ) ))
( "gumboils"  n ( g uh1 m b oi l z ) ((pos "Kj%" ) ))
( "gumboot"  n ( g uh1 m b uu t ) ((pos "K6%" ) ))
( "gumboots"  n ( g uh1 m b uu t s ) ((pos "Kj%" ) ))
( "gumbos"  n ( g uh1 m b ou z ) ((pos "Kj$" ) ))
( "gummed"  v ( g uh1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "gummier"  j ( g uh1 m i@ r ) ((pos "Or$" ) ))
( "gummiest"  j ( g uh1 m i i s t ) ((pos "Os$" ) ))
( "gumming"  v ( g uh1 m i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "gummy"  j ( g uh1 m ii ) ((pos "OD%" ) ))
( "gumption"  n ( g uh1 m p sh @ n ) ((pos "L@%" ) ))
( "gums"  v ( g uh1 m z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "gumshoe"  n ( g uh1 m sh uu ) ((pos "K6$" ) ))
( "gumshoes"  n ( g uh1 m sh uu z ) ((pos "Kj$" ) ))
( "gun"  v ( g uh1 n ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "gun-carriage"  n ( g uh1 n - k a r i jh ) ((pos "K6%" ) ))
( "gun-carriages"  n ( g uh1 n - k a r i jh i z ) ((pos "Kj%" ) ))
( "gunboat"  n ( g uh1 n b ou t ) ((pos "K6%" ) ))
( "gunboats"  n ( g uh1 n b ou t s ) ((pos "Kj%" ) ))
( "guncotton"  n ( g uh1 n k o t n ) ((pos "L@$" ) ))
( "gundog"  n ( g uh1 n d o g ) ((pos "K6%" ) ))
( "gundogs"  n ( g uh1 n d o g z ) ((pos "Kj%" ) ))
( "gunfire"  n ( g uh1 n f ai @ r ) ((pos "L@%" ) ))
( "gunman"  n ( g uh1 n m @ n ) ((pos "Ki%" ) ))
( "gunmen"  n ( g uh1 n m @ n ) ((pos "Kj%" ) ))
( "gunmetal"  n ( g uh1 n m e t l ) ((pos "L@%" ) ))
( "gunned"  v ( g uh1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "gunner"  n ( g uh1 n @ r ) ((pos "K6%" ) ))
( "gunners"  n ( g uh1 n @ z ) ((pos "Kj%" ) ))
( "gunnery"  n ( g uh1 n @ r ii ) ((pos "L@%" ) ))
( "gunning"  v ( g uh1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "gunny"  n ( g uh1 n ii ) ((pos "L@$" ) ))
( "gunplay"  n ( g uh1 n p l ei ) ((pos "L@%" ) ))
( "gunpoint"  n ( g uh1 n p oi n t ) ((pos "Ki%" ) ))
( "gunpowder"  n ( g uh1 n p au d @ r ) ((pos "L@%" ) ))
( "gunroom"  n ( g uh1 n r uu m ) ((pos "K6%" ) ))
( "gunrooms"  n ( g uh1 n r uu m z ) ((pos "Kj%" ) ))
( "gunrunner"  n ( g uh1 n r uh n @ r ) ((pos "K6%" ) ))
( "gunrunners"  n ( g uh1 n r uh n @ z ) ((pos "Kj%" ) ))
( "gunrunning"  n ( g uh1 n r uh n i ng ) ((pos "L@%" ) ))
( "guns"  v ( g uh1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "gunshot"  n ( g uh1 n sh o t ) ((pos "M6%" ) ))
( "gunshots"  n ( g uh1 n sh o t s ) ((pos "Mj%" ) ))
( "gunsmith"  n ( g uh1 n s m i th ) ((pos "K6%" ) ))
( "gunsmiths"  n ( g uh1 n s m i th s ) ((pos "Kj%" ) ))
( "gunwale"  n ( g uh1 n l ) ((pos "K6$" ) ))
( "gunwales"  n ( g uh1 n l z ) ((pos "Kj$" ) ))
( "gurgle"  v ( g @@1 g l ) ((pos "I2%" "M6%" ) ))
( "gurgled"  v ( g @@1 g l d ) ((pos "Ic%" "Id%" ) ))
( "gurgles"  v ( g @@1 g l z ) ((pos "Ia%" "Mj%" ) ))
( "gurgling"  v ( g @@1 g l i ng ) ((pos "Ib%" ) ))
( "guru"  n ( g u1 r uu ) ((pos "K6%" ) ))
( "gurus"  n ( g u1 r uu z ) ((pos "Kj%" ) ))
( "gush"  v ( g uh1 sh ) ((pos "I1%" "K7%" ) (subcat "2A" "2C" )))
( "gushed"  v ( g uh1 sh t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "gusher"  n ( g uh1 sh @ r ) ((pos "K6$" ) ))
( "gushers"  n ( g uh1 sh @ z ) ((pos "Kj$" ) ))
( "gushes"  v ( g uh1 sh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "gushing"  v ( g uh1 sh i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" "2C" )))
( "gushingly"  a ( g uh1 sh i ng l ii ) ((pos "Pu%" ) ))
( "gusset"  n ( g uh1 s i t ) ((pos "K6%" ) ))
( "gusseted"  j ( g uh1 s i t i d ) ((pos "OA%" ) ))
( "gussets"  n ( g uh1 s i t s ) ((pos "Kj%" ) ))
( "gust"  n ( g uh1 s t ) ((pos "K6%" ) ))
( "gustation"  n ( g uh1 s t ei1 sh @ n ) ((pos "L@$" ) ))
( "gustier"  j ( g uh1 s t i@ r ) ((pos "Or$" ) ))
( "gustiest"  j ( g uh1 s t i i s t ) ((pos "Os$" ) ))
( "gusto"  n ( g uh1 s t ou ) ((pos "L@%" ) ))
( "gusts"  n ( g uh1 s t s ) ((pos "Kj%" ) ))
( "gusty"  j ( g uh1 s t ii ) ((pos "OD%" ) ))
( "gut"  v ( g uh1 t ) ((pos "H4%" "M6%" ) (subcat )))
( "gutless"  j ( g uh1 t l @ s ) ((pos "OA%" ) ))
( "guts"  v ( g uh1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "gutta-percha"  n ( g uh2 t @ - p @@1 ch @ ) ((pos "L@$" ) ))
( "gutted"  v ( g uh1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "gutter"  v ( g uh1 t @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "guttered"  v ( g uh1 t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "guttering"  v ( g uh1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "gutters"  v ( g uh1 t @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "guttersnipe"  n ( g uh1 t @ s n ai p ) ((pos "K6%" ) ))
( "guttersnipes"  n ( g uh1 t @ s n ai p s ) ((pos "Kj%" ) ))
( "gutting"  v ( g uh1 t i ng ) ((pos "Hb%" ) (subcat )))
( "guttural"  n ( g uh1 t @ r @ l ) ((pos "K6%" "OA%" ) ))
( "gutturally"  a ( g uh1 t @ r @ l ii ) ((pos "Pu%" ) ))
( "gutturals"  n ( g uh1 t @ r @ l z ) ((pos "Kj%" ) ))
( "guvnor"  n ( g uh1 v n @ r ) ((pos "K6$" ) ))
( "guvnors"  n ( g uh1 v n @ z ) ((pos "Kj$" ) ))
( "guy"  v ( g ai1 ) ((pos "H0$" "K6%" ) (subcat )))
( "guyed"  v ( g ai1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "guying"  v ( g ai1 i ng ) ((pos "Hb$" ) (subcat )))
( "guys"  v ( g ai1 z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "guzzle"  v ( g uh1 z l ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "guzzled"  v ( g uh1 z l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "guzzler"  n ( g uh1 z l @ r ) ((pos "K6%" ) ))
( "guzzlers"  n ( g uh1 z l @ z ) ((pos "Kj%" ) ))
( "guzzles"  v ( g uh1 z l z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "guzzling"  v ( g uh1 z l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "gybe"  v ( jh ai1 b ) ((pos "J2$" ) (subcat "2A" )))
( "gybed"  v ( jh ai1 b d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "gybes"  v ( jh ai1 b z ) ((pos "Ja$" ) (subcat "2A" )))
( "gybing"  v ( jh ai1 b i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "gym"  n ( jh i1 m ) ((pos "K6%" ) ))
( "gymkhana"  n ( jh i1 m k aa1 n @ ) ((pos "K6%" ) ))
( "gymkhanas"  n ( jh i1 m k aa1 n @ z ) ((pos "Kj%" ) ))
( "gymnasium"  n ( jh i1 m n ei1 z i@ m ) ((pos "K6%" ) ))
( "gymnasiums"  n ( jh i1 m n ei1 z i@ m z ) ((pos "Kj%" ) ))
( "gymnast"  n ( jh i1 m n a s t ) ((pos "K6%" ) ))
( "gymnastic"  j ( jh i1 m n a1 s t i k ) ((pos "OA%" ) ))
( "gymnastics"  n ( jh i1 m n a1 s t i k s ) ((pos "Lk%" ) ))
( "gymnasts"  n ( jh i1 m n a s t s ) ((pos "Kj%" ) ))
( "gyms"  n ( jh i1 m z ) ((pos "Kj%" ) ))
( "gymslip"  n ( jh i1 m s l i p ) ((pos "K6%" ) ))
( "gymslips"  n ( jh i1 m s l i p s ) ((pos "Kj%" ) ))
( "gynaecological"  j ( g ai2 n i k @ l o1 jh i k l ) ((pos "OA%" ) ))
( "gynaecologist"  n ( g ai2 n i k o1 l @ jh i s t ) ((pos "K6%" ) ))
( "gynaecologists"  n ( g ai2 n i k o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "gynaecology"  n ( g ai2 n i k o1 l @ jh ii ) ((pos "L@%" ) ))
( "gynecological"  j ( g ai2 n i k @ l o1 jh i k l ) ((pos "OA$" ) ))
( "gynecologist"  n ( g ai2 n i k o1 l @ jh i s t ) ((pos "K6$" ) ))
( "gynecologists"  n ( g ai2 n i k o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "gyp"  v ( jh i1 p ) ((pos "H4$" "K6%" ) (subcat )))
( "gypped"  v ( jh i1 p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "gypping"  v ( jh i1 p i ng ) ((pos "Hb$" ) (subcat )))
( "gyps"  v ( jh i1 p s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "gypsum"  n ( jh i1 p s @ m ) ((pos "L@%" ) ))
( "gyrate"  v ( jh ai2 r ei1 t ) ((pos "I2%" ) ))
( "gyrated"  v ( jh ai2 r ei1 t i d ) ((pos "Ic%" "Id%" ) ))
( "gyrates"  v ( jh ai2 r ei1 t s ) ((pos "Ia%" ) ))
( "gyrating"  v ( jh ai2 r ei1 t i ng ) ((pos "Ib%" ) ))
( "gyration"  n ( jh ai2 r ei1 sh @ n ) ((pos "M6%" ) ))
( "gyrations"  n ( jh ai2 r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "gyro"  n ( jh ai1 @ r ou ) ((pos "K6$" ) ))
( "gyros"  n ( jh ai1 @ r ou z ) ((pos "Kj$" ) ))
( "gyroscope"  n ( jh ai1 r @ s k ou p ) ((pos "K6%" ) ))
( "gyroscopes"  n ( jh ai1 r @ s k ou p s ) ((pos "Kj%" ) ))
( "gyroscopic"  j ( jh ai2 r @ s k o1 p i k ) ((pos "OA%" ) ))
( "h"  n ( ei1 ch ) ((pos "Ki$" ) ))
( "h'm"  v ( h @1 m ) ((pos "I4$" "W-$" ) ))
( "h'mmed"  v ( h @1 m d ) ((pos "Ic$" "Id$" ) ))
( "h'mming"  v ( h @1 m i ng ) ((pos "Ib$" ) ))
( "h'ms"  v ( h @1 m z ) ((pos "Ia$" ) ))
( "h's"  n ( ei1 ch i z ) ((pos "Kj$" ) ))
( "ha"  uh ( h aa1 ) ((pos "W-%" ) ))
( "ha'p'orth"  n ( h ei1 p @ th ) ((pos "K6$" ) ))
( "ha'p'orths"  n ( h ei1 p @ th s ) ((pos "Kj$" ) ))
( "ha'pennies"  n ( h ei1 p n i z ) ((pos "Kj%" ) ))
( "ha'penny"  n ( h ei1 p n ii ) ((pos "K8%" ) ))
( "habeas corpus"  n ( h ei2 b i@ s - k oo1 p @ s ) ((pos "L@%" ) ))
( "haberdasher"  n ( h a1 b @ d a sh @ r ) ((pos "K6%" ) ))
( "haberdashers"  n ( h a1 b @ d a sh @ z ) ((pos "Kj%" ) ))
( "haberdashery"  n ( h a2 b @ d a1 sh @ r ii ) ((pos "L@%" ) ))
( "habiliments"  n ( h @1 b i1 l i m @ n t s ) ((pos "Kj$" ) ))
( "habit"  n ( h a1 b i t ) ((pos "M6%" ) ))
( "habitable"  j ( h a1 b i t @ b l ) ((pos "OA%" ) ))
( "habitat"  n ( h a1 b i t a t ) ((pos "K6%" ) ))
( "habitation"  n ( h a2 b i t ei1 sh @ n ) ((pos "M6%" ) ))
( "habitations"  n ( h a2 b i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "habitats"  n ( h a1 b i t a t s ) ((pos "Kj%" ) ))
( "habits"  n ( h a1 b i t s ) ((pos "Mj%" ) ))
( "habitu_e"  n ( h @1 b i1 ch u ei ) ((pos "K6%" ) ))
( "habitu_es"  n ( h @1 b i1 ch u ei z ) ((pos "Kj%" ) ))
( "habitual"  j ( h @1 b i1 ch u@ l ) ((pos "OA%" ) ))
( "habitually"  a ( h @1 b i1 ch u@ l ii ) ((pos "Pu%" ) ))
( "habituate"  v ( h @1 b i1 ch u ei t ) ((pos "H2$" ) (subcat )))
( "habituated"  v ( h @1 b i1 ch u ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "habituates"  v ( h @1 b i1 ch u ei t s ) ((pos "Ha$" ) (subcat )))
( "habituating"  v ( h @1 b i1 ch u ei t i ng ) ((pos "Hb$" ) (subcat )))
( "habituation"  n ( h @1 b i2 ch u ei1 sh @ n ) ((pos "L@%" ) ))
( "habitude"  n ( h a1 b i t y uu d ) ((pos "L@$" ) ))
( "hacienda"  n ( h a2 s i e1 n d @ ) ((pos "K6$" ) ))
( "haciendas"  n ( h a2 s i e1 n d @ z ) ((pos "Kj$" ) ))
( "hack"  v ( h a1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hacked"  v ( h a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hacking"  v ( h a1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hackles"  n ( h a1 k l z ) ((pos "Kj%" ) ))
( "hackney"  n ( h a1 k n ii ) ((pos "K6$" ) ))
( "hackneyed"  j ( h a1 k n i d ) ((pos "OA%" ) ))
( "hackneys"  n ( h a1 k n i z ) ((pos "Kj$" ) ))
( "hacks"  v ( h a1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hacksaw"  n ( h a1 k s oo ) ((pos "K6%" ) ))
( "hacksaws"  n ( h a1 k s oo z ) ((pos "Kj%" ) ))
( "had"  v ( h a d ) ((pos "Gc*" "Gd*" "Jc*" "Jd*" ) (subcat "6A" "6B" "7B" "15B" "18C" "19B" "24B" )))
( "haddock"  n ( h a1 d @ k ) ((pos "M9%" ) ))
( "hadn't"  v ( h a1 d n t ) ((pos "Gg%" ) ))
( "haematite"  n ( h e1 m @ t ai t ) ((pos "K6$" ) ))
( "haematites"  n ( h e1 m @ t ai t s ) ((pos "Kj$" ) ))
( "haemoglobin"  n ( h ii2 m @ g l ou1 b i n ) ((pos "L@%" ) ))
( "haemophilia"  n ( h ii2 m @ f i1 l i@ ) ((pos "L@%" ) ))
( "haemophiliac"  n ( h ii2 m @ f i1 l i a k ) ((pos "K6%" ) ))
( "haemophiliacs"  n ( h ii2 m @ f i1 l i a k s ) ((pos "Kj%" ) ))
( "haemorrhage"  n ( h e1 m @ r i jh ) ((pos "M6%" ) ))
( "haemorrhages"  n ( h e1 m @ r i jh i z ) ((pos "Mj%" ) ))
( "haemorrhoids"  n ( h e1 m @ r oi d z ) ((pos "Kj%" ) ))
( "haft"  n ( h aa1 f t ) ((pos "K6$" ) ))
( "hafts"  n ( h aa1 f t s ) ((pos "Kj$" ) ))
( "hag"  n ( h a1 g ) ((pos "K6%" ) ))
( "hag-ridden"  j ( h a1 g - r i d n ) ((pos "OA$" ) ))
( "haggard"  j ( h a1 g @ d ) ((pos "OA%" ) ))
( "haggis"  n ( h a1 g i s ) ((pos "K7%" ) ))
( "haggises"  n ( h a1 g i s i z ) ((pos "Kj%" ) ))
( "haggle"  v ( h a1 g l ) ((pos "I2%" ) (subcat "2A" )))
( "haggled"  v ( h a1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "haggles"  v ( h a1 g l z ) ((pos "Ia%" ) (subcat "2A" )))
( "haggling"  v ( h a1 g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "hagiologies"  n ( h a2 g i o1 l @ jh i z ) ((pos "Kj$" ) ))
( "hagiology"  n ( h a2 g i o1 l @ jh ii ) ((pos "K8$" ) ))
( "hags"  n ( h a1 g z ) ((pos "Kj%" ) ))
( "haha"  n ( h aa1 h aa ) ((pos "K6$" ) ))
( "hahas"  n ( h aa1 h aa z ) ((pos "Kj$" ) ))
( "hail"  v ( h ei1 l ) ((pos "J0%" "M6%" ) (subcat "2C" "3A" "6A" "15B" "16B" )))
( "hailed"  v ( h ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15B" "16B" )))
( "hailing"  v ( h ei1 l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15B" "16B" )))
( "hails"  v ( h ei1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" "15B" "16B" )))
( "hailstone"  n ( h ei1 l s t ou n ) ((pos "K6%" ) ))
( "hailstones"  n ( h ei1 l s t ou n z ) ((pos "Kj%" ) ))
( "hailstorm"  n ( h ei1 l s t oo m ) ((pos "K6%" ) ))
( "hailstorms"  n ( h ei1 l s t oo m z ) ((pos "Kj%" ) ))
( "hair"  n ( h e@1 r ) ((pos "M6%" ) ))
( "hair's-breadth"  n ( h e@1 z - b r e t th ) ((pos "K6%" ) ))
( "hair's-breadths"  n ( h e@1 z - b r e t th s ) ((pos "Kj%" ) ))
( "hair-breadth"  n ( h e@1 - b r e t th ) ((pos "K6$" ) ))
( "hair-breadths"  n ( h e@1 - b r e t th s ) ((pos "Kj$" ) ))
( "hair-dye"  n ( h e@1 - d ai ) ((pos "K6%" ) ))
( "hair-dyes"  n ( h e@1 - d ai z ) ((pos "Kj%" ) ))
( "hair-oil"  n ( h e@1 r - oi l ) ((pos "M6%" ) ))
( "hair-oils"  n ( h e@1 r - oi l z ) ((pos "Mj%" ) ))
( "hair-raising"  j ( h e@1 - r ei z i ng ) ((pos "OA%" ) ))
( "hair-shirt"  n ( h e@1 - sh @@ t ) ((pos "K6%" ) ))
( "hair-shirts"  n ( h e@1 - sh @@ t s ) ((pos "Kj%" ) ))
( "hair-slide"  n ( h e@1 - s l ai d ) ((pos "K6%" ) ))
( "hair-slides"  n ( h e@1 - s l ai d z ) ((pos "Kj%" ) ))
( "hair-trigger"  n ( h e@1 - t r i g @ r ) ((pos "K6$" ) ))
( "hair-triggers"  n ( h e@1 - t r i g @ z ) ((pos "Kj$" ) ))
( "hairbrush"  n ( h e@1 b r uh sh ) ((pos "K7%" ) ))
( "hairbrushes"  n ( h e@1 b r uh sh i z ) ((pos "Kj%" ) ))
( "haircloth"  n ( h e@1 k l o th ) ((pos "K6$" ) ))
( "haircloths"  n ( h e@1 k l o th s ) ((pos "Kj$" ) ))
( "haircut"  n ( h e@1 k uh t ) ((pos "K6%" ) ))
( "haircuts"  n ( h e@1 k uh t s ) ((pos "Kj%" ) ))
( "hairdo"  n ( h e@1 d uu ) ((pos "K6%" ) ))
( "hairdos"  n ( h e@1 d uu z ) ((pos "Kj%" ) ))
( "hairdresser"  n ( h e@1 d r e s @ r ) ((pos "K6%" ) ))
( "hairdressers"  n ( h e@1 d r e s @ z ) ((pos "Kj%" ) ))
( "hairdressing"  n ( h e@1 d r e s i ng ) ((pos "L@%" ) ))
( "hairier"  j ( h e@1 r i@ r ) ((pos "Or%" ) ))
( "hairiest"  j ( h e@1 r i i s t ) ((pos "Os%" ) ))
( "hairiness"  n ( h e@1 r i n @ s ) ((pos "L@%" ) ))
( "hairless"  j ( h e@1 l @ s ) ((pos "OA%" ) ))
( "hairlike"  j ( h e@1 l ai k ) ((pos "OA%" ) ))
( "hairline"  n ( h e@1 l ai n ) ((pos "K6%" ) ))
( "hairlines"  n ( h e@1 l ai n z ) ((pos "Kj%" ) ))
( "hairnet"  n ( h e@1 n e t ) ((pos "K6%" ) ))
( "hairnets"  n ( h e@1 n e t s ) ((pos "Kj%" ) ))
( "hairpiece"  n ( h e@1 p ii s ) ((pos "K6%" ) ))
( "hairpieces"  n ( h e@1 p ii s i z ) ((pos "Kj%" ) ))
( "hairpin"  n ( h e@1 p i n ) ((pos "K6%" ) ))
( "hairpins"  n ( h e@1 p i n z ) ((pos "Kj%" ) ))
( "hairs"  n ( h e@1 z ) ((pos "Mj%" ) ))
( "hairsplitting"  n ( h e@1 s p l i t i ng ) ((pos "L@%" "OA$" ) ))
( "hairspring"  n ( h e@1 s p r i ng ) ((pos "K6$" ) ))
( "hairsprings"  n ( h e@1 s p r i ng z ) ((pos "Kj$" ) ))
( "hairstyle"  n ( h e@1 s t ai l ) ((pos "K6%" ) ))
( "hairstyles"  n ( h e@1 s t ai l z ) ((pos "Kj%" ) ))
( "hairstylist"  n ( h e@1 s t ai l i s t ) ((pos "K6%" ) ))
( "hairstylists"  n ( h e@1 s t ai l i s t s ) ((pos "Kj%" ) ))
( "hairy"  j ( h e@1 r ii ) ((pos "OD%" ) ))
( "hake"  n ( h ei1 k ) ((pos "M9%" ) ))
( "halberd"  n ( h a1 l b @ d ) ((pos "K6$" ) ))
( "halberdier"  n ( h a2 l b @ d i@1 r ) ((pos "K6$" ) ))
( "halberdiers"  n ( h a2 l b @ d i@1 z ) ((pos "Kj$" ) ))
( "halberds"  n ( h a1 l b @ d z ) ((pos "Kj$" ) ))
( "halcyon"  j ( h a1 l s i@ n ) ((pos "OA%" ) ))
( "hale"  j ( h ei1 l ) ((pos "OA%" ) ))
( "half"  n ( h aa1 f ) ((pos "Ki*" ) ))
( "half-baked"  j ( h aa1 f - b ei1 k t ) ((pos "OA%" ) ))
( "half-blood"  n ( h aa1 f - b l uh d ) ((pos "K6$" ) ))
( "half-bloods"  n ( h aa1 f - b l uh d z ) ((pos "Kj$" ) ))
( "half-breed"  n ( h aa1 f - b r ii d ) ((pos "K6%" ) ))
( "half-breeds"  n ( h aa1 f - b r ii d z ) ((pos "Kj%" ) ))
( "half-brother"  n ( h aa1 f - b r uh dh @ r ) ((pos "K6%" ) ))
( "half-brothers"  n ( h aa1 f - b r uh dh @ z ) ((pos "Kj%" ) ))
( "half-caste"  n ( h aa1 f - k aa s t ) ((pos "K6%" ) ))
( "half-castes"  n ( h aa1 f - k aa s t s ) ((pos "Kj%" ) ))
( "half-crazed"  j ( h aa1 f - k r ei1 z d ) ((pos "OA%" ) ))
( "half-crown"  n ( h aa1 f - k r au1 n ) ((pos "K6%" ) ))
( "half-crowns"  n ( h aa1 f - k r au1 n z ) ((pos "Kj%" ) ))
( "half-hardy"  j ( h aa1 f - h aa1 d ii ) ((pos "OA$" ) ))
( "half-hearted"  j ( h aa1 f - h aa t i d ) ((pos "OA%" ) ))
( "half-heartedly"  a ( h aa1 f - h aa1 t i d l ii ) ((pos "Pu%" ) ))
( "half-holiday"  n ( h aa1 f - h o1 l @ d ei ) ((pos "K6%" ) ))
( "half-holidays"  n ( h aa1 f - h o1 l @ d ei z ) ((pos "Kj%" ) ))
( "half-hour"  n ( h aa1 f - au1 @ r ) ((pos "K6%" ) ))
( "half-hourly"  j ( h aa1 f - au1 @ l ii ) ((pos "OA%" "Pu%" ) ))
( "half-hours"  n ( h aa1 f - au1 @ z ) ((pos "Kj%" ) ))
( "half-length"  j ( h aa1 f - l e1 ng th ) ((pos "OA%" ) ))
( "half-pay"  n ( h aa1 f - p ei1 ) ((pos "L@%" ) ))
( "half-price"  a ( h aa1 f - p r ai1 s ) ((pos "Pu%" ) ))
( "half-seas-over"  j ( h aa2 f - s ii z - ou1 v @ r ) ((pos "Op$" ) ))
( "half-sister"  n ( h aa1 f - s i s t @ r ) ((pos "K6%" ) ))
( "half-sisters"  n ( h aa1 f - s i s t @ z ) ((pos "Kj%" ) ))
( "half-size"  j ( h aa1 f - s ai1 z ) ((pos "OA%" ) ))
( "half-timbered"  j ( h aa1 f - t i1 m b @ d ) ((pos "OA%" ) ))
( "half-time"  n ( h aa1 f - t ai1 m ) ((pos "L@%" ) ))
( "half-track"  n ( h aa1 f - t r a k ) ((pos "K6$" ) ))
( "half-tracked"  j ( h aa1 f - t r a k t ) ((pos "OA$" ) ))
( "half-tracks"  n ( h aa1 f - t r a k s ) ((pos "Kj$" ) ))
( "half-truth"  n ( h aa1 f - t r uu th ) ((pos "K6%" ) ))
( "half-truths"  n ( h aa1 f - t r uu dh z ) ((pos "Kj%" ) ))
( "half-volley"  n ( h aa1 f - v o1 l ii ) ((pos "K6%" ) ))
( "half-volleys"  n ( h aa1 f - v o1 l i z ) ((pos "Kj%" ) ))
( "half-yearly"  j ( h aa1 f - y @@1 l ii ) ((pos "OA%" "Pu%" ) ))
( "halfback"  n ( h aa1 f b a k ) ((pos "K6%" ) ))
( "halfbacks"  n ( h aa1 f b a k s ) ((pos "Kj%" ) ))
( "halfpennies"  n ( h ei1 p n i z ) ((pos "Kj%" ) ))
( "halfpenny"  n ( h ei1 p n ii ) ((pos "K8%" ) ))
( "halfpennyworth"  n ( h ei1 p n i w @@ th ) ((pos "K6$" ) ))
( "halfpennyworths"  n ( h ei1 p n i w @@ th s ) ((pos "Kj$" ) ))
( "halftone"  n ( h aa1 f t ou n ) ((pos "K6%" ) ))
( "halftones"  n ( h aa1 f t ou n z ) ((pos "Kj%" ) ))
( "halfway"  j ( h aa1 f w ei1 ) ((pos "OA%" "Pu%" ) ))
( "halfwit"  n ( h aa1 f w i t ) ((pos "K6%" ) ))
( "halfwits"  n ( h aa1 f w i t s ) ((pos "Kj%" ) ))
( "halfwitted"  j ( h aa1 f w i t i d ) ((pos "OA%" ) ))
( "halibut"  n ( h a1 l i b @ t ) ((pos "M9%" ) ))
( "halitosis"  n ( h a2 l i t ou1 s i s ) ((pos "L@%" ) ))
( "hall"  n ( h oo1 l ) ((pos "K6%" ) ))
( "hall-stand"  n ( h oo1 l - s t a n d ) ((pos "K6%" ) ))
( "hall-stands"  n ( h oo1 l - s t a n d z ) ((pos "Kj%" ) ))
( "hallelujah"  n ( h a2 l i l uu1 y @ ) ((pos "K6%" "W-%" ) ))
( "hallelujahs"  n ( h a2 l i l uu1 y @ z ) ((pos "Kj%" ) ))
( "halliard"  n ( h a1 l y @ d ) ((pos "K6$" ) ))
( "halliards"  n ( h a1 l y @ d z ) ((pos "Kj$" ) ))
( "hallmark"  n ( h oo1 l m aa k ) ((pos "K6%" ) ))
( "hallmarks"  n ( h oo1 l m aa k s ) ((pos "Kj%" ) ))
( "hallo"  uh ( h @1 l ou1 ) ((pos "W-$" ) ))
( "halloo"  v ( h @1 l uu1 ) ((pos "I0$" "K6$" "W-$" ) ))
( "hallooed"  v ( h @1 l uu1 d ) ((pos "Ic$" "Id$" ) ))
( "hallooing"  v ( h @1 l uu1 i ng ) ((pos "Ib$" ) ))
( "halloos"  v ( h @1 l uu1 z ) ((pos "Ia$" "Kj$" ) ))
( "hallow"  v ( h a1 l ou ) ((pos "H0%" "K6$" ) (subcat )))
( "hallowed"  v ( h a1 l ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hallowing"  v ( h a1 l ou i ng ) ((pos "Hb%" ) (subcat )))
( "hallows"  v ( h a1 l ou z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "halls"  n ( h oo1 l z ) ((pos "Kj%" ) ))
( "hallucination"  n ( h @1 l uu2 s i n ei1 sh @ n ) ((pos "M6%" ) ))
( "hallucinations"  n ( h @1 l uu2 s i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "hallucinatory"  j ( h @1 l uu1 s i n @ t r ii ) ((pos "OA%" ) ))
( "hallucinogenic"  j ( h @1 l uu2 s i n @ jh e1 n i k ) ((pos "OA%" ) ))
( "halma"  n ( h a1 l m @ ) ((pos "L@$" ) ))
( "halo"  n ( h ei1 l ou ) ((pos "K6%" ) ))
( "haloes"  n ( h ei1 l ou z ) ((pos "Kj%" ) ))
( "halos"  n ( h ei1 l ou z ) ((pos "Kj%" ) ))
( "halt"  v ( h oo1 l t ) ((pos "J0%" "K6%" "OA%" ) (subcat "2A" "2C" )))
( "halted"  v ( h oo1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "halter"  n ( h oo1 l t @ r ) ((pos "K6%" ) ))
( "halters"  n ( h oo1 l t @ z ) ((pos "Kj%" ) ))
( "halting"  v ( h oo1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "haltingly"  a ( h oo1 l t i ng l ii ) ((pos "Pu%" ) ))
( "halts"  v ( h oo1 l t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "halve"  v ( h aa1 v ) ((pos "H2%" ) (subcat )))
( "halved"  v ( h aa1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "halves"  v ( h aa1 v z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "halving"  v ( h aa1 v i ng ) ((pos "Hb%" ) (subcat )))
( "halyard"  n ( h a1 l y @ d ) ((pos "K6$" ) ))
( "halyards"  n ( h a1 l y @ d z ) ((pos "Kj$" ) ))
( "ham"  v ( h a1 m ) ((pos "J4%" "M6%" ) (subcat "2A" "6A" )))
( "ham-fisted"  j ( h a1 m - f i1 s t i d ) ((pos "OA%" ) ))
( "ham-handed"  j ( h a1 m - h a1 n d i d ) ((pos "OA%" ) ))
( "hamadryad"  n ( h a2 m @ d r ai1 @ d ) ((pos "K6$" ) ))
( "hamadryads"  n ( h a2 m @ d r ai1 @ d z ) ((pos "Kj$" ) ))
( "hamburger"  n ( h a1 m b @@ g @ r ) ((pos "K6%" ) ))
( "hamburgers"  n ( h a1 m b @@ g @ z ) ((pos "Kj%" ) ))
( "hamlet"  n ( h a1 m l i t ) ((pos "K6%" ) ))
( "hamlets"  n ( h a1 m l i t s ) ((pos "Kj%" ) ))
( "hammed"  v ( h a1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "hammer"  v ( h a1 m @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "hammered"  v ( h a1 m @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "hammering"  v ( h a1 m @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "hammers"  v ( h a1 m @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "hamming"  v ( h a1 m i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "hammock"  n ( h a1 m @ k ) ((pos "K6%" ) ))
( "hammocks"  n ( h a1 m @ k s ) ((pos "Kj%" ) ))
( "hamper"  v ( h a1 m p @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "hampered"  v ( h a1 m p @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hampering"  v ( h a1 m p @ r i ng ) ((pos "Hb%" ) (subcat )))
( "hampers"  v ( h a1 m p @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hams"  v ( h a1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "hamster"  n ( h a1 m s t @ r ) ((pos "K6%" ) ))
( "hamsters"  n ( h a1 m s t @ z ) ((pos "Kj%" ) ))
( "hamstring"  v ( h a1 m s t r i ng ) ((pos "H5%" "K6%" ) (subcat )))
( "hamstringing"  v ( h a1 m s t r i ng i ng ) ((pos "Hb$" ) (subcat )))
( "hamstrings"  v ( h a1 m s t r i ng z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hamstrung"  v ( h a1 m s t r uh ng ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hand"  v ( h a1 n d ) ((pos "H0%" "K6*" ) (subcat "12A" "13A" "15A" )))
( "hand-barrow"  n ( h a1 n d - b a r ou ) ((pos "K6%" ) ))
( "hand-barrows"  n ( h a1 n d - b a r ou z ) ((pos "Kj%" ) ))
( "hand-grenade"  n ( h a1 n d - g r i n ei d ) ((pos "K6%" ) ))
( "hand-grenades"  n ( h a1 n d - g r i n ei d z ) ((pos "Kj%" ) ))
( "hand-luggage"  n ( h a1 n d - l uh g i jh ) ((pos "L@%" ) ))
( "hand-me-down"  n ( h a1 n d - m i - d au n ) ((pos "K6%" ) ))
( "hand-me-downs"  n ( h a1 n d - m i - d au n z ) ((pos "Kj%" ) ))
( "hand-organ"  n ( h a1 n d - oo g @ n ) ((pos "K6$" ) ))
( "hand-organs"  n ( h a1 n d - oo g @ n z ) ((pos "Kj$" ) ))
( "hand-out"  n ( h a1 n d - au t ) ((pos "K6%" ) ))
( "hand-outs"  n ( h a1 n d - au t s ) ((pos "Kj%" ) ))
( "hand-picked"  j ( h a1 n d - p i1 k t ) ((pos "OA%" ) ))
( "handbag"  n ( h a1 n d b a g ) ((pos "K6%" ) ))
( "handbags"  n ( h a1 n d b a g z ) ((pos "Kj%" ) ))
( "handbill"  n ( h a1 n d b i l ) ((pos "K6%" ) ))
( "handbills"  n ( h a1 n d b i l z ) ((pos "Kj%" ) ))
( "handbook"  n ( h a1 n d b u k ) ((pos "K6%" ) ))
( "handbooks"  n ( h a1 n d b u k s ) ((pos "Kj%" ) ))
( "handbrake"  n ( h a1 n d b r ei k ) ((pos "K6%" ) ))
( "handbrakes"  n ( h a1 n d b r ei k s ) ((pos "Kj%" ) ))
( "handcart"  n ( h a1 n d k aa t ) ((pos "K6%" ) ))
( "handcarts"  n ( h a1 n d k aa t s ) ((pos "Kj%" ) ))
( "handclap"  n ( h a1 n d k l a p ) ((pos "K6%" ) ))
( "handclaps"  n ( h a1 n d k l a p s ) ((pos "Kj%" ) ))
( "handcuff"  v ( h a1 n d k uh f ) ((pos "H0%" "K6%" ) ))
( "handcuffed"  v ( h a1 n d k uh f t ) ((pos "Hc%" "Hd%" ) ))
( "handcuffing"  v ( h a1 n d k uh f i ng ) ((pos "Hb%" ) ))
( "handcuffs"  v ( h a1 n d k uh f s ) ((pos "Ha%" "Kj%" ) ))
( "handed"  v ( h a1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "12A" "13A" "15A" )))
( "handful"  n ( h a1 n d f u l ) ((pos "K6%" ) ))
( "handfuls"  n ( h a1 n d f u l z ) ((pos "Kj%" ) ))
( "handhold"  n ( h a1 n d h ou l d ) ((pos "K6%" ) ))
( "handholds"  n ( h a1 n d h ou l d z ) ((pos "Kj%" ) ))
( "handicap"  v ( h a1 n d i k a p ) ((pos "H4%" "K6%" ) (subcat )))
( "handicapped"  v ( h a1 n d i k a p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "handicapping"  v ( h a1 n d i k a p i ng ) ((pos "Hb%" ) (subcat )))
( "handicaps"  v ( h a1 n d i k a p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "handicraft"  n ( h a1 n d i k r aa f t ) ((pos "M6%" ) ))
( "handicrafts"  n ( h a1 n d i k r aa f t s ) ((pos "Mj%" ) ))
( "handier"  j ( h a1 n d i@ r ) ((pos "Or%" ) ))
( "handiest"  j ( h a1 n d i i s t ) ((pos "Os%" ) ))
( "handily"  a ( h a1 n d i l ii ) ((pos "Pu%" ) ))
( "handiness"  n ( h a1 n d i n @ s ) ((pos "L@%" ) ))
( "handing"  v ( h a1 n d i ng ) ((pos "Hb%" ) (subcat "12A" "13A" "15A" )))
( "handiwork"  n ( h a1 n d i w @@ k ) ((pos "M6%" ) ))
( "handiworks"  n ( h a1 n d i w @@ k s ) ((pos "Mj$" ) ))
( "handkerchief"  n ( h a1 ng k @ ch i f ) ((pos "K6%" ) ))
( "handkerchiefs"  n ( h a1 ng k @ ch i f s ) ((pos "Kj%" ) ))
( "handle"  v ( h a1 n d l ) ((pos "H2%" "K6%" ) (subcat )))
( "handlebar"  n ( h a1 n d l b aa r ) ((pos "K6%" ) ))
( "handlebars"  n ( h a1 n d l b aa z ) ((pos "Kj%" ) ))
( "handled"  v ( h a1 n d l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "handler"  n ( h a1 n d l @ r ) ((pos "K6%" ) ))
( "handlers"  n ( h a1 n d l @ z ) ((pos "Kj%" ) ))
( "handles"  v ( h a1 n d l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "handling"  v ( h a1 n d l i ng ) ((pos "Hb%" ) (subcat )))
( "handmade"  j ( h a1 n d m ei1 d ) ((pos "OA%" ) ))
( "handmaid"  n ( h a1 n d m ei d ) ((pos "K6%" ) ))
( "handmaids"  n ( h a1 n d m ei d z ) ((pos "Kj%" ) ))
( "handover"  n ( h a1 n d ou v @ r ) ((pos "K6%" ) ))
( "handovers"  n ( h a1 n d ou v @ z ) ((pos "Kj$" ) ))
( "handrail"  n ( h a1 n d r ei l ) ((pos "K6%" ) ))
( "handrails"  n ( h a1 n d r ei l z ) ((pos "Kj%" ) ))
( "hands"  v ( h a1 n d z ) ((pos "Ha%" "Kj*" ) (subcat "12A" "13A" "15A" )))
( "handsaw"  n ( h a1 n d s oo ) ((pos "K6$" ) ))
( "handsaws"  n ( h a1 n d s oo z ) ((pos "Kj$" ) ))
( "handshake"  n ( h a1 n d sh ei k ) ((pos "K6%" ) ))
( "handshakes"  n ( h a1 n d sh ei k s ) ((pos "Kj%" ) ))
( "handshaking"  n ( h a1 n d sh ei k i ng ) ((pos "L@%" ) ))
( "handsome"  j ( h a1 n s @ m ) ((pos "OA%" ) ))
( "handsomely"  a ( h a1 n s @ m l ii ) ((pos "Pu%" ) ))
( "handsomest"  j ( h a1 n s @ m i s t ) ((pos "Os%" ) ))
( "handstand"  n ( h a1 n d s t a n d ) ((pos "K6%" ) ))
( "handstands"  n ( h a1 n d s t a n d z ) ((pos "Kj%" ) ))
( "handwork"  n ( h a1 n d w @@ k ) ((pos "L@%" ) ))
( "handwriting"  n ( h a1 n d r ai t i ng ) ((pos "L@%" ) ))
( "handy"  j ( h a1 n d ii ) ((pos "OD%" ) ))
( "handyman"  n ( h a1 n d i m a n ) ((pos "Ki%" ) ))
( "handymen"  n ( h a1 n d i m e n ) ((pos "Kj%" ) ))
( "hang"  v ( h a1 ng ) ((pos "J0%" "Ki%" ) (subcat "2A" "2B" "2C" "3A" "6A" "15A" )))
( "hang-up"  n ( h a1 ng - uh p ) ((pos "K6%" ) ))
( "hang-ups"  n ( h a1 ng - uh p s ) ((pos "Kj%" ) ))
( "hangar"  n ( h a1 ng @ r ) ((pos "K6%" ) ))
( "hangars"  n ( h a1 ng @ z ) ((pos "Kj%" ) ))
( "hangdog"  j ( h a1 ng d o g ) ((pos "Oq%" ) ))
( "hanged"  v ( h a1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "15A" )))
( "hanger"  n ( h a1 ng @ r ) ((pos "K6%" ) ))
( "hanger-on"  n ( h a2 ng @ r - o1 n ) ((pos "Ki%" ) ))
( "hangers"  n ( h a1 ng @ z ) ((pos "Kj%" ) ))
( "hangers-on"  n ( h a2 ng @ z - o1 n ) ((pos "Kj%" ) ))
( "hanging"  v ( h a1 ng i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "15A" )))
( "hangings"  n ( h a1 ng i ng z ) ((pos "Mj%" ) ))
( "hangman"  n ( h a1 ng m @ n ) ((pos "Ki%" ) ))
( "hangmen"  n ( h a1 ng m @ n ) ((pos "Kj%" ) ))
( "hangnail"  n ( h a1 ng n ei l ) ((pos "K6$" ) ))
( "hangnails"  n ( h a1 ng n ei l z ) ((pos "Kj$" ) ))
( "hangover"  n ( h a1 ng ou v @ r ) ((pos "K6%" ) ))
( "hangovers"  n ( h a1 ng ou v @ z ) ((pos "Kj%" ) ))
( "hangs"  v ( h a1 ng z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "6A" "15A" )))
( "hank"  n ( h a1 ng k ) ((pos "K6$" ) ))
( "hanker"  v ( h a1 ng k @ r ) ((pos "I0%" ) (subcat )))
( "hankered"  v ( h a1 ng k @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "hankering"  v ( h a1 ng k @ r i ng ) ((pos "Ib%" "K6%" ) (subcat )))
( "hankerings"  n ( h a1 ng k @ r i ng z ) ((pos "Kj%" ) ))
( "hankers"  v ( h a1 ng k @ z ) ((pos "Ia%" ) (subcat )))
( "hankies"  n ( h a1 ng k i z ) ((pos "Kj%" ) ))
( "hanks"  n ( h a1 ng k s ) ((pos "Kj$" ) ))
( "hanky"  n ( h a1 ng k ii ) ((pos "K8%" ) ))
( "hanky-panky"  n ( h a2 ng k i - p a1 ng k ii ) ((pos "L@%" ) ))
( "hansom"  n ( h a1 n s @ m ) ((pos "K6$" ) ))
( "hansoms"  n ( h a1 n s @ m z ) ((pos "Kj$" ) ))
( "hap"  v ( h a1 p ) ((pos "I4$" "K6$" ) ))
( "haphazard"  j ( h a1 p h a1 z @ d ) ((pos "OA%" "Pu%" ) ))
( "haphazardly"  a ( h a1 p h a1 z @ d l ii ) ((pos "Pu%" ) ))
( "hapless"  j ( h a1 p l i s ) ((pos "OA%" ) ))
( "haply"  a ( h a1 p l ii ) ((pos "Pu$" ) ))
( "happed"  v ( h a1 p t ) ((pos "Ic$" "Id$" ) ))
( "happen"  v ( h a1 p @ n ) ((pos "I0%" ) (subcat "2A" "3A" )))
( "happened"  v ( h a1 p @ n d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "happening"  v ( h a1 p @ n i ng ) ((pos "Ib%" "K6%" ) (subcat "2A" "3A" )))
( "happenings"  n ( h a1 p @ n i ng z ) ((pos "Kj%" ) ))
( "happens"  v ( h a1 p @ n z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "happier"  j ( h a1 p i@ r ) ((pos "Or%" ) ))
( "happiest"  j ( h a1 p i i s t ) ((pos "Os%" ) ))
( "happily"  a ( h a1 p i l ii ) ((pos "Pu%" ) ))
( "happiness"  n ( h a1 p i n @ s ) ((pos "L@%" ) ))
( "happing"  v ( h a1 p i ng ) ((pos "Ib$" ) ))
( "happy"  j ( h a1 p ii ) ((pos "OD%" ) ))
( "happy-go-lucky"  j ( h a2 p i - g ou - l uh1 k ii ) ((pos "OA%" ) ))
( "haps"  v ( h a1 p s ) ((pos "Ia$" "Kj$" ) ))
( "hara-kiri"  n ( h a2 r @ - k i1 r ii ) ((pos "L@$" ) ))
( "harangue"  v ( h @1 r a1 ng ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "harangued"  v ( h @1 r a1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "harangues"  v ( h @1 r a1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "haranguing"  v ( h @1 r a1 ng i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "harass"  v ( h a1 r @ s ) ((pos "H1%" ) (subcat )))
( "harassed"  v ( h a1 r @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harasses"  v ( h a1 r @ s i z ) ((pos "Ha%" ) (subcat )))
( "harassing"  v ( h a1 r @ s i ng ) ((pos "Hb%" ) (subcat )))
( "harassment"  n ( h a1 r @ s m @ n t ) ((pos "L@%" ) ))
( "harbinger"  n ( h aa1 b i n jh @ r ) ((pos "K6$" ) ))
( "harbingers"  n ( h aa1 b i n jh @ z ) ((pos "Kj$" ) ))
( "harbour"  v ( h aa1 b @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "harbourage"  n ( h aa1 b @ r i jh ) ((pos "K6$" ) ))
( "harbourages"  n ( h aa1 b @ r i jh i z ) ((pos "Kj$" ) ))
( "harboured"  v ( h aa1 b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "harbouring"  v ( h aa1 b @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "harbours"  v ( h aa1 b @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "hard"  j ( h aa1 d ) ((pos "OC*" "Pu*" ) ))
( "hard-baked"  j ( h aa1 d - b ei1 k t ) ((pos "OA%" ) ))
( "hard-bitten"  j ( h aa1 d - b i1 t n ) ((pos "OA%" ) ))
( "hard-boiled"  j ( h aa1 d - b oi1 l d ) ((pos "OA%" ) ))
( "hard-headed"  j ( h aa1 d - h e1 d i d ) ((pos "OA%" ) ))
( "hardback"  n ( h aa1 d b a k ) ((pos "K6%" ) ))
( "hardbacked"  j ( h aa1 d b a k t ) ((pos "OA%" ) ))
( "hardbacks"  n ( h aa1 d b a k s ) ((pos "Kj%" ) ))
( "hardboard"  n ( h aa1 d b oo d ) ((pos "L@%" ) ))
( "hardbound"  j ( h aa1 d b au n d ) ((pos "OA%" ) ))
( "hardcover"  n ( h aa1 d k uh v @ r ) ((pos "K6%" ) ))
( "hardcovered"  j ( h aa1 d k uh v @ d ) ((pos "OA$" ) ))
( "hardcovers"  n ( h aa1 d k uh v @ z ) ((pos "Kj%" ) ))
( "harden"  v ( h aa1 d n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "hardened"  v ( h aa1 d n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "hardening"  v ( h aa1 d n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "hardens"  v ( h aa1 d n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "harder"  j ( h aa1 d @ r ) ((pos "Or%" ) ))
( "hardest"  j ( h aa1 d i s t ) ((pos "Os%" ) ))
( "hardhearted"  j ( h aa1 d h aa1 t i d ) ((pos "OA%" ) ))
( "hardhitting"  j ( h aa1 d h i1 t i ng ) ((pos "OA%" ) ))
( "hardier"  j ( h aa1 d i@ r ) ((pos "Or%" ) ))
( "hardiest"  j ( h aa1 d i i s t ) ((pos "Os%" ) ))
( "hardihood"  n ( h aa1 d i h u d ) ((pos "L@$" ) ))
( "hardiness"  n ( h aa1 d i n @ s ) ((pos "L@%" ) ))
( "hardliner"  n ( h aa1 d l ai1 n @ r ) ((pos "K6%" ) ))
( "hardliners"  n ( h aa1 d l ai1 n @ z ) ((pos "Kj%" ) ))
( "hardly"  a ( h aa1 d l ii ) ((pos "Pu%" ) ))
( "hardness"  n ( h aa1 d n @ s ) ((pos "L@%" ) ))
( "hardship"  n ( h aa1 d sh i p ) ((pos "M6%" ) ))
( "hardships"  n ( h aa1 d sh i p s ) ((pos "Mj%" ) ))
( "hardtop"  n ( h aa1 d t o p ) ((pos "K6$" ) ))
( "hardtops"  n ( h aa1 d t o p s ) ((pos "Kj$" ) ))
( "hardware"  n ( h aa1 d w e@ r ) ((pos "L@%" ) ))
( "hardwood"  n ( h aa1 d w u d ) ((pos "L@%" ) ))
( "hardworking"  j ( h aa1 d w @@1 k i ng ) ((pos "OA%" ) ))
( "hardy"  j ( h aa1 d ii ) ((pos "OD%" ) ))
( "hare"  v ( h e@1 r ) ((pos "I2%" "K6%" ) (subcat )))
( "harebell"  n ( h e@1 b e l ) ((pos "K6$" ) ))
( "harebells"  n ( h e@1 b e l z ) ((pos "Kj$" ) ))
( "harebrained"  j ( h e@1 b r ei n d ) ((pos "OA%" ) ))
( "hared"  v ( h e@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "harelip"  n ( h e@1 l i p ) ((pos "K6%" ) ))
( "harelips"  n ( h e@1 l i p s ) ((pos "Kj$" ) ))
( "harem"  n ( h aa1 r ii m ) ((pos "K6%" ) ))
( "harems"  n ( h aa1 r ii m z ) ((pos "Kj%" ) ))
( "hares"  v ( h e@1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "haricot"  n ( h a1 r i k ou ) ((pos "K6%" ) ))
( "haricots"  n ( h a1 r i k ou z ) ((pos "Kj%" ) ))
( "haring"  v ( h e@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "hark"  v ( h aa1 k ) ((pos "I0%" ) ))
( "harked"  v ( h aa1 k t ) ((pos "Ic%" "Id%" ) ))
( "harking"  v ( h aa1 k i ng ) ((pos "Ib%" ) ))
( "harks"  v ( h aa1 k s ) ((pos "Ia%" ) ))
( "harlequin"  n ( h aa1 l i k w i n ) ((pos "K6%" ) ))
( "harlequinade"  n ( h aa2 l i k w i n ei1 d ) ((pos "K6$" ) ))
( "harlequinades"  n ( h aa2 l i k w i n ei1 d z ) ((pos "Kj$" ) ))
( "harlequins"  n ( h aa1 l i k w i n z ) ((pos "Kj%" ) ))
( "harlot"  n ( h aa1 l @ t ) ((pos "K6%" ) ))
( "harlots"  n ( h aa1 l @ t s ) ((pos "Kj%" ) ))
( "harm"  v ( h aa1 m ) ((pos "H0%" "L@%" ) (subcat )))
( "harmattan"  n ( h aa2 m @ t a1 n ) ((pos "K6$" ) ))
( "harmattans"  n ( h aa2 m @ t a1 n z ) ((pos "Kj$" ) ))
( "harmed"  v ( h aa1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harmful"  j ( h aa1 m f @ l ) ((pos "OA%" ) ))
( "harmfully"  a ( h aa1 m f @ l ii ) ((pos "Pu%" ) ))
( "harming"  v ( h aa1 m i ng ) ((pos "Hb%" ) (subcat )))
( "harmless"  j ( h aa1 m l @ s ) ((pos "OA%" ) ))
( "harmlessly"  a ( h aa1 m l @ s l ii ) ((pos "Pu%" ) ))
( "harmonic"  n ( h aa1 m o1 n i k ) ((pos "K6%" ) ))
( "harmonica"  n ( h aa1 m o1 n i k @ ) ((pos "K6%" ) ))
( "harmonicas"  n ( h aa1 m o1 n i k @ z ) ((pos "Kj%" ) ))
( "harmonics"  n ( h aa1 m o1 n i k s ) ((pos "Kj%" ) ))
( "harmonies"  n ( h aa1 m @ n i z ) ((pos "Mj%" ) ))
( "harmonious"  j ( h aa1 m ou1 n i@ s ) ((pos "OA%" ) ))
( "harmoniously"  a ( h aa1 m ou1 n i@ s l ii ) ((pos "Pu%" ) ))
( "harmonium"  n ( h aa1 m ou1 n i@ m ) ((pos "K6%" ) ))
( "harmoniums"  n ( h aa1 m ou1 n i@ m z ) ((pos "Kj%" ) ))
( "harmonization"  n ( h aa2 m @ n ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "harmonizations"  n ( h aa2 m @ n ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "harmonize"  v ( h aa1 m @ n ai z ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "harmonized"  v ( h aa1 m @ n ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "harmonizes"  v ( h aa1 m @ n ai z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "harmonizing"  v ( h aa1 m @ n ai z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "harmony"  n ( h aa1 m @ n ii ) ((pos "M8%" ) ))
( "harms"  v ( h aa1 m z ) ((pos "Ha%" ) (subcat )))
( "harness"  v ( h aa1 n @ s ) ((pos "H1%" "K7%" ) (subcat )))
( "harnessed"  v ( h aa1 n @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harnesses"  v ( h aa1 n @ s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "harnessing"  v ( h aa1 n @ s i ng ) ((pos "Hb%" ) (subcat )))
( "harp"  v ( h aa1 p ) ((pos "I0%" "K6%" ) ))
( "harped"  v ( h aa1 p t ) ((pos "Ic%" "Id%" ) ))
( "harper"  n ( h aa1 p @ r ) ((pos "K6%" ) ))
( "harpers"  n ( h aa1 p @ z ) ((pos "Kj%" ) ))
( "harpies"  n ( h aa1 p i z ) ((pos "Kj%" ) ))
( "harping"  v ( h aa1 p i ng ) ((pos "Ib%" ) ))
( "harpist"  n ( h aa1 p i s t ) ((pos "K6%" ) ))
( "harpists"  n ( h aa1 p i s t s ) ((pos "Kj$" ) ))
( "harpoon"  v ( h aa1 p uu1 n ) ((pos "H0%" "K6%" ) ))
( "harpooned"  v ( h aa1 p uu1 n d ) ((pos "Hc%" "Hd%" ) ))
( "harpooning"  v ( h aa1 p uu1 n i ng ) ((pos "Hb%" ) ))
( "harpoons"  v ( h aa1 p uu1 n z ) ((pos "Ha%" "Kj%" ) ))
( "harps"  v ( h aa1 p s ) ((pos "Ia%" "Kj%" ) ))
( "harpsichord"  n ( h aa1 p s i k oo d ) ((pos "K6%" ) ))
( "harpsichordist"  n ( h aa1 p s i k oo d i s t ) ((pos "K6%" ) ))
( "harpsichordists"  n ( h aa1 p s i k oo d i s t s ) ((pos "Kj%" ) ))
( "harpsichords"  n ( h aa1 p s i k oo d z ) ((pos "Kj%" ) ))
( "harpy"  n ( h aa1 p ii ) ((pos "K8%" ) ))
( "harridan"  n ( h a1 r i d @ n ) ((pos "K6$" ) ))
( "harridans"  n ( h a1 r i d @ n z ) ((pos "Kj$" ) ))
( "harried"  v ( h a1 r i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harrier"  n ( h a1 r i@ r ) ((pos "K6%" ) ))
( "harriers"  n ( h a1 r i@ z ) ((pos "Kj%" ) ))
( "harries"  v ( h a1 r i z ) ((pos "Ha%" ) (subcat )))
( "harrow"  v ( h a1 r ou ) ((pos "H0%" "K6%" ) (subcat )))
( "harrowed"  v ( h a1 r ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harrowing"  v ( h a1 r ou i ng ) ((pos "Hb%" ) (subcat )))
( "harrows"  v ( h a1 r ou z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "harry"  v ( h a1 r ii ) ((pos "H3%" ) (subcat )))
( "harrying"  v ( h a1 r i i ng ) ((pos "Hb%" ) (subcat )))
( "harsh"  j ( h aa1 sh ) ((pos "OC%" ) ))
( "harsher"  j ( h aa1 sh @ r ) ((pos "Or%" ) ))
( "harshest"  j ( h aa1 sh i s t ) ((pos "Os%" ) ))
( "harshly"  a ( h aa1 sh l ii ) ((pos "Pu%" ) ))
( "harshness"  n ( h aa1 sh n @ s ) ((pos "L@%" ) ))
( "hart"  n ( h aa1 t ) ((pos "K6$" ) ))
( "harts"  n ( h aa1 t s ) ((pos "Kj$" ) ))
( "harum-scarum"  n ( h e@2 r @ m - s k e@1 r @ m ) ((pos "K6$" "OA$" ) ))
( "harum-scarums"  n ( h e@2 r @ m - s k e@1 r @ m z ) ((pos "Kj$" ) ))
( "harvest"  v ( h aa1 v i s t ) ((pos "H0%" "K6%" ) (subcat )))
( "harvested"  v ( h aa1 v i s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "harvester"  n ( h aa1 v i s t @ r ) ((pos "K6%" ) ))
( "harvesters"  n ( h aa1 v i s t @ z ) ((pos "Kj%" ) ))
( "harvesting"  v ( h aa1 v i s t i ng ) ((pos "Hb%" ) (subcat )))
( "harvests"  v ( h aa1 v i s t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "has"  v ( h a z ) ((pos "Ga*" "Ja*" ) (subcat "6A" "6B" "7B" "15B" "18C" "19B" "24B" )))
( "has-been"  n ( h a1 z - b ii n ) ((pos "K6%" ) ))
( "has-beens"  n ( h a1 z - b ii n z ) ((pos "Kj$" ) ))
( "hash"  v ( h a1 sh ) ((pos "H1%" "L@%" ) (subcat "6A" )))
( "hashed"  v ( h a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hashes"  v ( h a1 sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "hashing"  v ( h a1 sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hashish"  n ( h a1 sh ii sh ) ((pos "L@%" ) ))
( "hasn't"  v ( h a1 z n t ) ((pos "Gg%" ) ))
( "hasp"  n ( h aa1 s p ) ((pos "K6$" ) ))
( "hasps"  n ( h aa1 s p s ) ((pos "Kj$" ) ))
( "hassle"  v ( h a1 s l ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "hassled"  v ( h a1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "hassles"  v ( h a1 s l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "hassling"  v ( h a1 s l i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "hassock"  n ( h a1 s @ k ) ((pos "K6%" ) ))
( "hassocks"  n ( h a1 s @ k s ) ((pos "Kj%" ) ))
( "hast"  v ( h a s t ) ((pos "Ge$" ) ))
( "haste"  n ( h ei1 s t ) ((pos "L@%" ) ))
( "hasten"  v ( h ei1 s n ) ((pos "J0%" ) (subcat "2A" "2C" "4A" )))
( "hastened"  v ( h ei1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" )))
( "hastening"  v ( h ei1 s n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" )))
( "hastens"  v ( h ei1 s n z ) ((pos "Ja%" ) (subcat "2A" "2C" "4A" )))
( "hastier"  j ( h ei1 s t i@ r ) ((pos "Or$" ) ))
( "hastiest"  j ( h ei1 s t i i s t ) ((pos "Os$" ) ))
( "hastily"  a ( h ei1 s t i l ii ) ((pos "Pu%" ) ))
( "hastiness"  n ( h ei1 s t i n @ s ) ((pos "L@%" ) ))
( "hasty"  j ( h ei1 s t ii ) ((pos "OD%" ) ))
( "hat"  n ( h a1 t ) ((pos "K6%" ) ))
( "hatband"  n ( h a1 t b a n d ) ((pos "K6$" ) ))
( "hatbands"  n ( h a1 t b a n d z ) ((pos "Kj$" ) ))
( "hatch"  v ( h a1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "hatched"  v ( h a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "hatcheries"  n ( h a1 ch @ r i z ) ((pos "Kj%" ) ))
( "hatchery"  n ( h a1 ch @ r ii ) ((pos "K8%" ) ))
( "hatches"  v ( h a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "hatchet"  n ( h a1 ch i t ) ((pos "K6%" ) ))
( "hatchets"  n ( h a1 ch i t s ) ((pos "Kj%" ) ))
( "hatching"  v ( h a1 ch i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "hatchway"  n ( h a1 ch w ei ) ((pos "K6%" ) ))
( "hatchways"  n ( h a1 ch w ei z ) ((pos "Kj%" ) ))
( "hate"  v ( h ei1 t ) ((pos "H2%" "M6%" ) (subcat "6A" "6D" "7A" "17" )))
( "hated"  v ( h ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" "17" )))
( "hateful"  j ( h ei1 t f @ l ) ((pos "OA%" ) ))
( "hatefully"  a ( h ei1 t f @ l ii ) ((pos "Pu%" ) ))
( "hates"  v ( h ei1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6D" "7A" "17" )))
( "hatful"  n ( h a1 t f u l ) ((pos "K6%" ) ))
( "hatfuls"  n ( h a1 t f u l z ) ((pos "Kj%" ) ))
( "hath"  v ( h a th ) ((pos "Ge$" ) ))
( "hating"  v ( h ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" "6D" "7A" "17" )))
( "hatless"  j ( h a1 t l @ s ) ((pos "OA%" ) ))
( "hatpin"  n ( h a1 t p i n ) ((pos "K6%" ) ))
( "hatpins"  n ( h a1 t p i n z ) ((pos "Kj%" ) ))
( "hatred"  n ( h ei1 t r i d ) ((pos "M6%" ) ))
( "hatreds"  n ( h ei1 t r i d z ) ((pos "Mj$" ) ))
( "hats"  n ( h a1 t s ) ((pos "Kj%" ) ))
( "hatter"  n ( h a1 t @ r ) ((pos "K6%" ) ))
( "hatters"  n ( h a1 t @ z ) ((pos "Kj%" ) ))
( "hauberk"  n ( h oo1 b @@ k ) ((pos "K6$" ) ))
( "hauberks"  n ( h oo1 b @@ k s ) ((pos "Kj$" ) ))
( "haughtier"  j ( h oo1 t i@ r ) ((pos "Or%" ) ))
( "haughtiest"  j ( h oo1 t i i s t ) ((pos "Os%" ) ))
( "haughtily"  a ( h oo1 t i l ii ) ((pos "Pu%" ) ))
( "haughtiness"  n ( h oo1 t i n @ s ) ((pos "L@%" ) ))
( "haughty"  j ( h oo1 t ii ) ((pos "OD%" ) ))
( "haul"  v ( h oo1 l ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" "15A" )))
( "haulage"  n ( h oo1 l i jh ) ((pos "L@%" ) ))
( "hauled"  v ( h oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "haulier"  n ( h oo1 l i@ r ) ((pos "K6%" ) ))
( "hauliers"  n ( h oo1 l i@ z ) ((pos "Kj%" ) ))
( "hauling"  v ( h oo1 l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "haulm"  n ( h oo1 m ) ((pos "L@$" ) ))
( "hauls"  v ( h oo1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "15A" )))
( "haunch"  n ( h oo1 n ch ) ((pos "K7%" ) ))
( "haunches"  n ( h oo1 n ch i z ) ((pos "Kj%" ) ))
( "haunt"  v ( h oo1 n t ) ((pos "H0%" "K6%" ) (subcat )))
( "haunted"  v ( h oo1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "haunting"  v ( h oo1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "haunts"  v ( h oo1 n t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hautboy"  n ( h ou1 b oi ) ((pos "K6$" ) ))
( "hautboys"  n ( h ou1 b oi z ) ((pos "Kj$" ) ))
( "hauteur"  n ( ou1 t @@1 r ) ((pos "L@$" ) ))
( "have"  v ( h a v ) ((pos "G5*" "J5*" ) (subcat "6A" "6B" "7B" "15B" "18C" "19B" "24B" )))
( "haven"  n ( h ei1 v n ) ((pos "K6%" ) ))
( "haven't"  v ( h a1 v n t ) ((pos "Gg%" ) ))
( "havens"  n ( h ei1 v n z ) ((pos "Kj%" ) ))
( "haversack"  n ( h a1 v @ s a k ) ((pos "K6%" ) ))
( "haversacks"  n ( h a1 v @ s a k s ) ((pos "Kj%" ) ))
( "haves"  n ( h a1 v z ) ((pos "Kj%" ) ))
( "having"  v ( h a1 v i ng ) ((pos "Gb%" "Jb%" ) (subcat "6A" "6B" "7B" "15B" "18C" "19B" "24B" )))
( "havoc"  n ( h a1 v @ k ) ((pos "L@%" ) ))
( "haw"  v ( h oo1 ) ((pos "I0%" "K6%" ) ))
( "haw-haw"  n ( h oo1 - h oo ) ((pos "K6$" "W-$" ) ))
( "haw-haws"  n ( h oo1 - h oo z ) ((pos "Kj$" ) ))
( "hawed"  v ( h oo1 d ) ((pos "Ic$" "Id$" ) ))
( "hawing"  v ( h oo1 i ng ) ((pos "Ib$" ) ))
( "hawk"  v ( h oo1 k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "hawk-eyed"  j ( h oo1 k - ai d ) ((pos "OA%" ) ))
( "hawked"  v ( h oo1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hawker"  n ( h oo1 k @ r ) ((pos "K6%" ) ))
( "hawkers"  n ( h oo1 k @ z ) ((pos "Kj%" ) ))
( "hawking"  v ( h oo1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hawks"  v ( h oo1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "haws"  v ( h oo1 z ) ((pos "Ia$" "Kj$" ) ))
( "hawser"  n ( h oo1 z @ r ) ((pos "K6%" ) ))
( "hawsers"  n ( h oo1 z @ z ) ((pos "Kj%" ) ))
( "hawthorn"  n ( h oo1 th oo n ) ((pos "K6%" ) ))
( "hawthorns"  n ( h oo1 th oo n z ) ((pos "Kj%" ) ))
( "hay"  n ( h ei1 ) ((pos "L@%" ) ))
( "haycock"  n ( h ei1 k o k ) ((pos "K6$" ) ))
( "haycocks"  n ( h ei1 k o k s ) ((pos "Kj$" ) ))
( "hayfork"  n ( h ei1 f oo k ) ((pos "K6%" ) ))
( "hayforks"  n ( h ei1 f oo k s ) ((pos "Kj%" ) ))
( "haymaker"  n ( h ei1 m ei k @ r ) ((pos "K6%" ) ))
( "haymakers"  n ( h ei1 m ei k @ z ) ((pos "Kj%" ) ))
( "haymaking"  n ( h ei1 m ei k i ng ) ((pos "L@%" ) ))
( "hayrick"  n ( h ei1 r i k ) ((pos "K6%" ) ))
( "hayricks"  n ( h ei1 r i k s ) ((pos "Kj%" ) ))
( "haystack"  n ( h ei1 s t a k ) ((pos "K6%" ) ))
( "haystacks"  n ( h ei1 s t a k s ) ((pos "Kj%" ) ))
( "haywire"  n ( h ei1 w ai @ r ) ((pos "L@$" "Op%" ) ))
( "hazard"  v ( h a1 z @ d ) ((pos "H0%" "M6%" ) (subcat )))
( "hazarded"  v ( h a1 z @ d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hazarding"  v ( h a1 z @ d i ng ) ((pos "Hb%" ) (subcat )))
( "hazardous"  j ( h a1 z @ d @ s ) ((pos "OA%" ) ))
( "hazards"  v ( h a1 z @ d z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "haze"  v ( h ei1 z ) ((pos "H2$" "M6%" ) ))
( "hazed"  v ( h ei1 z d ) ((pos "Hc$" "Hd$" ) ))
( "hazel"  n ( h ei1 z l ) ((pos "M6%" ) ))
( "hazels"  n ( h ei1 z l z ) ((pos "Mj%" ) ))
( "hazes"  v ( h ei1 z i z ) ((pos "Ha$" "Mj%" ) ))
( "hazier"  j ( h ei1 z i@ r ) ((pos "Or%" ) ))
( "haziest"  j ( h ei1 z i i s t ) ((pos "Os%" ) ))
( "hazily"  a ( h ei1 z i l ii ) ((pos "Pu%" ) ))
( "haziness"  n ( h ei1 z i n @ s ) ((pos "L@%" ) ))
( "hazing"  v ( h ei1 z i ng ) ((pos "Hb%" ) ))
( "hazy"  j ( h ei1 z ii ) ((pos "OD%" ) ))
( "he"  prp ( h ii ) ((pos "Qx*" ) ))
( "he'd"  v ( h ii d ) ((pos "Gf%" ) ))
( "he'll"  v ( h ii l ) ((pos "Gf%" ) ))
( "he's"  v ( h ii z ) ((pos "Gf%" ) ))
( "he-goat"  n ( h ii1 - g ou t ) ((pos "K6%" ) ))
( "he-goats"  n ( h ii1 - g ou t s ) ((pos "Kj%" ) ))
( "he-man"  n ( h ii1 - m a n ) ((pos "Ki%" ) ))
( "he-men"  n ( h ii1 - m e n ) ((pos "Kj%" ) ))
( "head"  v ( h e1 d ) ((pos "J0%" "K6*" ) (subcat "2C" "6A" )))
( "head-hunter"  n ( h e1 d - h uh n t @ r ) ((pos "K6%" ) ))
( "head-hunters"  n ( h e1 d - h uh n t @ z ) ((pos "Kj%" ) ))
( "head-on"  j ( h e1 d - o n ) ((pos "OA%" "Pu%" ) ))
( "headache"  n ( h e1 d ei k ) ((pos "M6%" ) ))
( "headaches"  n ( h e1 d ei k s ) ((pos "Mj%" ) ))
( "headband"  n ( h e1 d b a n d ) ((pos "K6%" ) ))
( "headbands"  n ( h e1 d b a n d z ) ((pos "Kj%" ) ))
( "headdress"  n ( h e1 d - d r e s ) ((pos "K7%" ) ))
( "headdresses"  n ( h e1 d - d r e s i z ) ((pos "Kj%" ) ))
( "headed"  v ( h e1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "6A" )))
( "header"  n ( h e1 d @ r ) ((pos "K6%" ) ))
( "headers"  n ( h e1 d @ z ) ((pos "Kj%" ) ))
( "headgear"  n ( h e1 d g i@ r ) ((pos "L@%" ) ))
( "headier"  j ( h e1 d i@ r ) ((pos "Or$" ) ))
( "headiest"  j ( h e1 d i i s t ) ((pos "Os$" ) ))
( "heading"  v ( h e1 d i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" )))
( "headings"  n ( h e1 d i ng z ) ((pos "Kj%" ) ))
( "headlamp"  n ( h e1 d l a m p ) ((pos "K6%" ) ))
( "headlamps"  n ( h e1 d l a m p s ) ((pos "Kj%" ) ))
( "headland"  n ( h e1 d l a n d ) ((pos "K6%" ) ))
( "headlands"  n ( h e1 d l a n d z ) ((pos "Kj%" ) ))
( "headless"  j ( h e1 d l @ s ) ((pos "OA%" ) ))
( "headlight"  n ( h e1 d l ai t ) ((pos "K6%" ) ))
( "headlights"  n ( h e1 d l ai t s ) ((pos "Kj%" ) ))
( "headline"  n ( h e1 d l ai n ) ((pos "K6%" ) ))
( "headlines"  n ( h e1 d l ai n z ) ((pos "Kj%" ) ))
( "headlong"  j ( h e1 d l o ng ) ((pos "OA%" "Pu%" ) ))
( "headman"  n ( h e1 d m a n ) ((pos "Ki%" ) ))
( "headmaster"  n ( h e1 d m aa1 s t @ r ) ((pos "K6%" ) ))
( "headmasters"  n ( h e1 d m aa1 s t @ z ) ((pos "Kj%" ) ))
( "headmen"  n ( h e1 d m e n ) ((pos "Kj%" ) ))
( "headmistress"  n ( h e1 d m i1 s t r i s ) ((pos "K7%" ) ))
( "headmistresses"  n ( h e1 d m i1 s t r i s i z ) ((pos "Kj%" ) ))
( "headphones"  n ( h e1 d f ou n z ) ((pos "Kj%" ) ))
( "headpiece"  n ( h e1 d p ii s ) ((pos "K6$" ) ))
( "headpieces"  n ( h e1 d p ii s i z ) ((pos "Kj$" ) ))
( "headquarters"  n ( h e1 d k w oo1 t @ z ) ((pos "Kj%" ) ))
( "headrest"  n ( h e1 d r e s t ) ((pos "K6%" ) ))
( "headrests"  n ( h e1 d r e s t s ) ((pos "Kj%" ) ))
( "headroom"  n ( h e1 d r uu m ) ((pos "L@%" ) ))
( "heads"  v ( h e1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "headset"  n ( h e1 d s e t ) ((pos "K6%" ) ))
( "headsets"  n ( h e1 d s e t s ) ((pos "Kj%" ) ))
( "headship"  n ( h e1 d sh i p ) ((pos "K6%" ) ))
( "headships"  n ( h e1 d sh i p s ) ((pos "Kj%" ) ))
( "headstall"  n ( h e1 d s t oo l ) ((pos "K6$" ) ))
( "headstalls"  n ( h e1 d s t oo l z ) ((pos "Kj$" ) ))
( "headstone"  n ( h e1 d s t ou n ) ((pos "K6%" ) ))
( "headstones"  n ( h e1 d s t ou n z ) ((pos "Kj%" ) ))
( "headstrong"  j ( h e1 d s t r o ng ) ((pos "OA%" ) ))
( "headwaters"  n ( h e1 d w oo t @ z ) ((pos "Kj$" ) ))
( "headway"  n ( h e1 d w ei ) ((pos "L@%" ) ))
( "headwind"  n ( h e1 d w i n d ) ((pos "K6%" ) ))
( "headwinds"  n ( h e1 d w i n d z ) ((pos "Kj%" ) ))
( "headword"  n ( h e1 d w @@ d ) ((pos "K6$" ) ))
( "headwords"  n ( h e1 d w @@ d z ) ((pos "Kj$" ) ))
( "heady"  j ( h e1 d ii ) ((pos "OD%" ) ))
( "heal"  v ( h ii1 l ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "healed"  v ( h ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "healer"  n ( h ii1 l @ r ) ((pos "K6%" ) ))
( "healers"  n ( h ii1 l @ z ) ((pos "Kj%" ) ))
( "healing"  v ( h ii1 l i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" )))
( "heals"  v ( h ii1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "health"  n ( h e1 l th ) ((pos "L@%" ) ))
( "healthful"  j ( h e1 l th f @ l ) ((pos "OA$" ) ))
( "healthier"  j ( h e1 l th i@ r ) ((pos "Or%" ) ))
( "healthiest"  j ( h e1 l th i i s t ) ((pos "Os%" ) ))
( "healthily"  a ( h e1 l th i l ii ) ((pos "Pu%" ) ))
( "healthy"  j ( h e1 l th ii ) ((pos "OD%" ) ))
( "heap"  v ( h ii1 p ) ((pos "H0%" "K6%" ) (subcat "6A" "14" "15A" )))
( "heaped"  v ( h ii1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "15A" )))
( "heaping"  v ( h ii1 p i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15A" )))
( "heaps"  v ( h ii1 p s ) ((pos "Ha%" "Kj%" "Pu%" ) (subcat "6A" "14" "15A" )))
( "hear"  v ( h i@1 r ) ((pos "J5%" ) (subcat "2A" "3A" "6A" "9" "10" "15A" "18A" "19A" )))
( "heard"  v ( h @@1 d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "3A" "6A" "9" "10" "15A" "18A" "19A" )))
( "hearer"  n ( h i@1 r @ r ) ((pos "K6%" ) ))
( "hearers"  n ( h i@1 r @ z ) ((pos "Kj%" ) ))
( "hearing"  v ( h i@1 r i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "3A" "6A" "9" "10" "15A" "18A" "19A" )))
( "hearing-aid"  n ( h i@1 r i ng - ei d ) ((pos "K6%" ) ))
( "hearing-aids"  n ( h i@1 r i ng - ei d z ) ((pos "Kj%" ) ))
( "hearings"  n ( h i@1 r i ng z ) ((pos "Mj$" ) ))
( "hearken"  v ( h aa1 k @ n ) ((pos "I0%" ) ))
( "hearkened"  v ( h aa1 k @ n d ) ((pos "Ic%" "Id%" ) ))
( "hearkening"  v ( h aa1 k @ n i ng ) ((pos "Ib%" ) ))
( "hearkens"  v ( h aa1 k @ n z ) ((pos "Ia%" ) ))
( "hears"  v ( h i@1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "9" "10" "15A" "18A" "19A" )))
( "hearsay"  n ( h i@1 s ei ) ((pos "L@%" ) ))
( "hearse"  n ( h @@1 s ) ((pos "K6%" ) ))
( "hearses"  n ( h @@1 s i z ) ((pos "Kj$" ) ))
( "heart"  n ( h aa1 t ) ((pos "M6%" ) ))
( "heart's-ease"  n ( h aa1 t s - ii z ) ((pos "L@$" ) ))
( "heart-disease"  n ( h aa1 t - d i z ii z ) ((pos "M6%" ) ))
( "heart-diseases"  n ( h aa1 t - d i z ii z i z ) ((pos "Mj%" ) ))
( "heart-failure"  n ( h aa1 t - f ei l y @ r ) ((pos "L@%" ) ))
( "heart-rending"  j ( h aa1 t - r e n d i ng ) ((pos "OA%" ) ))
( "heartache"  n ( h aa1 t ei k ) ((pos "M6%" ) ))
( "heartaches"  n ( h aa1 t ei k s ) ((pos "Mj%" ) ))
( "heartbeat"  n ( h aa1 t b ii t ) ((pos "K6%" ) ))
( "heartbeats"  n ( h aa1 t b ii t s ) ((pos "Kj%" ) ))
( "heartbreak"  n ( h aa1 t b r ei k ) ((pos "L@%" ) ))
( "heartbreaking"  j ( h aa1 t b r ei k i ng ) ((pos "OA%" ) ))
( "heartbroken"  j ( h aa1 t b r ou k @ n ) ((pos "OA%" ) ))
( "heartburn"  n ( h aa1 t b @@ n ) ((pos "L@%" ) ))
( "heartburning"  n ( h aa1 t b @@ n i ng ) ((pos "L@$" ) ))
( "hearted"  j ( h aa1 t i d ) ((pos "Ot%" ) ))
( "hearten"  v ( h aa1 t n ) ((pos "H0%" ) (subcat )))
( "heartened"  v ( h aa1 t n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "heartening"  v ( h aa1 t n i ng ) ((pos "Hb%" ) (subcat )))
( "heartens"  v ( h aa1 t n z ) ((pos "Ha%" ) (subcat )))
( "heartfelt"  j ( h aa1 t f e l t ) ((pos "OA%" ) ))
( "hearth"  n ( h aa1 th ) ((pos "K6%" ) ))
( "hearthrug"  n ( h aa1 th r uh g ) ((pos "K6%" ) ))
( "hearthrugs"  n ( h aa1 th r uh g z ) ((pos "Kj%" ) ))
( "hearths"  n ( h aa1 th s ) ((pos "Kj%" ) ))
( "heartier"  j ( h aa1 t i@ r ) ((pos "Or%" ) ))
( "heartiest"  j ( h aa1 t i i s t ) ((pos "Os%" ) ))
( "heartily"  a ( h aa1 t i l ii ) ((pos "Pu%" ) ))
( "heartless"  j ( h aa1 t l @ s ) ((pos "OA%" ) ))
( "heartlessly"  a ( h aa1 t l @ s l ii ) ((pos "Pu%" ) ))
( "heartlessness"  n ( h aa1 t l @ s n @ s ) ((pos "L@%" ) ))
( "hearts"  n ( h aa1 t s ) ((pos "Mj%" ) ))
( "heartsick"  j ( h aa1 t s i k ) ((pos "OA$" ) ))
( "heartstrings"  n ( h aa1 t s t r i ng z ) ((pos "Kj%" ) ))
( "hearty"  j ( h aa1 t ii ) ((pos "OD%" ) ))
( "heat"  v ( h ii1 t ) ((pos "J0%" "M6%" ) (subcat "2C" "6A" )))
( "heat-flash"  n ( h ii1 t - f l a sh ) ((pos "K7$" ) ))
( "heat-flashes"  n ( h ii1 t - f l a sh i z ) ((pos "Kj$" ) ))
( "heated"  v ( h ii1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "6A" )))
( "heatedly"  a ( h ii1 t i d l ii ) ((pos "Pu%" ) ))
( "heater"  n ( h ii1 t @ r ) ((pos "K6%" ) ))
( "heaters"  n ( h ii1 t @ z ) ((pos "Kj%" ) ))
( "heath"  n ( h ii1 th ) ((pos "M6%" ) ))
( "heathen"  n ( h ii1 dh n ) ((pos "K6%" ) ))
( "heathenish"  j ( h ii1 dh n i sh ) ((pos "OA%" ) ))
( "heathens"  n ( h ii1 dh n z ) ((pos "Kj%" ) ))
( "heather"  n ( h e1 dh @ r ) ((pos "M6%" ) ))
( "heather-mixture"  n ( h e1 dh @ - m i k s ch @ r ) ((pos "L@$" ) ))
( "heathers"  n ( h e1 dh @ z ) ((pos "Mj$" ) ))
( "heaths"  n ( h ii1 th s ) ((pos "Mj%" ) ))
( "heating"  v ( h ii1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2C" "6A" )))
( "heats"  v ( h ii1 t s ) ((pos "Ja%" "Mj$" ) (subcat "2C" "6A" )))
( "heatspot"  n ( h ii1 t s p o t ) ((pos "K6%" ) ))
( "heatspots"  n ( h ii1 t s p o t s ) ((pos "Kj%" ) ))
( "heatstroke"  n ( h ii1 t s t r ou k ) ((pos "L@%" ) ))
( "heatwave"  n ( h ii1 t w ei v ) ((pos "K6%" ) ))
( "heatwaves"  n ( h ii1 t w ei v z ) ((pos "Kj%" ) ))
( "heave"  v ( h ii1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "heaved"  v ( h ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "heaven"  n ( h e1 v n ) ((pos "K6%" ) ))
( "heavenly"  j ( h e1 v n l ii ) ((pos "OA%" ) ))
( "heavens"  n ( h e1 v n z ) ((pos "Kj%" ) ))
( "heavensent"  j ( h e1 v n s e n t ) ((pos "OA%" ) ))
( "heavenward"  j ( h e1 v n w @ d ) ((pos "OA%" "Pu%" ) ))
( "heavenwards"  j ( h e1 v n w @ d z ) ((pos "OA%" "Pu%" ) ))
( "heaves"  v ( h ii1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "heavier"  j ( h e1 v i@ r ) ((pos "Or%" ) ))
( "heaviest"  j ( h e1 v i i s t ) ((pos "Os%" ) ))
( "heavily"  a ( h e1 v i l ii ) ((pos "Pu%" ) ))
( "heaviness"  n ( h e1 v i n @ s ) ((pos "L@%" ) ))
( "heaving"  v ( h ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "heavy"  j ( h e1 v ii ) ((pos "OD%" "Pu%" ) ))
( "heavy-handed"  j ( h e2 v i - h a1 n d i d ) ((pos "OA%" ) ))
( "heavy-hearted"  j ( h e2 v i - h aa1 t i d ) ((pos "OA%" ) ))
( "heavy-laden"  j ( h e2 v i - l ei1 d n ) ((pos "OA%" ) ))
( "heavyweight"  n ( h e1 v i w ei t ) ((pos "K6%" ) ))
( "heavyweights"  n ( h e1 v i w ei t s ) ((pos "Kj%" ) ))
( "hebdomadal"  j ( h e1 b d o1 m @ d l ) ((pos "OA$" ) ))
( "hecatomb"  n ( h e1 k @ t uu m ) ((pos "K6$" ) ))
( "hecatombs"  n ( h e1 k @ t uu m z ) ((pos "Kj$" ) ))
( "heck"  n ( h e1 k ) ((pos "K6$" ) ))
( "heckle"  v ( h e1 k l ) ((pos "H2%" ) (subcat )))
( "heckled"  v ( h e1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "heckler"  n ( h e1 k l @ r ) ((pos "K6%" ) ))
( "hecklers"  n ( h e1 k l @ z ) ((pos "Kj%" ) ))
( "heckles"  v ( h e1 k l z ) ((pos "Ha%" ) (subcat )))
( "heckling"  v ( h e1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "hecks"  n ( h e1 k s ) ((pos "Kj$" ) ))
( "hectare"  n ( h e1 k t e@ r ) ((pos "K6%" ) ))
( "hectares"  n ( h e1 k t e@ z ) ((pos "Kj%" ) ))
( "hectic"  j ( h e1 k t i k ) ((pos "OA%" ) ))
( "hecto-"  j ( h e1 k t ou - ) ((pos "U-%" ) ))
( "hector"  v ( h e1 k t @ r ) ((pos "J0$" ) ))
( "hectored"  v ( h e1 k t @ d ) ((pos "Jc$" "Jd$" ) ))
( "hectoring"  v ( h e1 k t @ r i ng ) ((pos "Jb%" ) ))
( "hectors"  v ( h e1 k t @ z ) ((pos "Ja$" ) ))
( "hedge"  v ( h e1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" "15A" )))
( "hedge-sparrow"  n ( h e1 jh - s p a r ou ) ((pos "K6%" ) ))
( "hedge-sparrows"  n ( h e1 jh - s p a r ou z ) ((pos "Kj%" ) ))
( "hedged"  v ( h e1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "hedgehog"  n ( h e1 jh h o g ) ((pos "K6%" ) ))
( "hedgehogs"  n ( h e1 jh h o g z ) ((pos "Kj%" ) ))
( "hedgehop"  v ( h e1 jh h o p ) ((pos "I4$" ) ))
( "hedgehopped"  v ( h e1 jh h o p t ) ((pos "Ic$" "Id$" ) ))
( "hedgehopping"  v ( h e1 jh h o p i ng ) ((pos "Ib$" ) ))
( "hedgehops"  v ( h e1 jh h o p s ) ((pos "Ia$" ) ))
( "hedgerow"  n ( h e1 jh r ou ) ((pos "K6%" ) ))
( "hedgerows"  n ( h e1 jh r ou z ) ((pos "Kj%" ) ))
( "hedges"  v ( h e1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "hedging"  v ( h e1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "hedonism"  n ( h ii1 d @ n i z @ m ) ((pos "L@$" ) ))
( "hedonist"  n ( h ii1 d @ n i s t ) ((pos "K6%" ) ))
( "hedonistic"  j ( h ii2 d @ n i1 s t i k ) ((pos "OA$" ) ))
( "hedonists"  n ( h ii1 d @ n i s t s ) ((pos "Kj$" ) ))
( "heed"  v ( h ii1 d ) ((pos "H0%" "L@%" ) (subcat )))
( "heeded"  v ( h ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "heedful"  j ( h ii1 d f @ l ) ((pos "OA%" ) ))
( "heeding"  v ( h ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "heedless"  j ( h ii1 d l @ s ) ((pos "OA%" ) ))
( "heeds"  v ( h ii1 d z ) ((pos "Ha%" ) (subcat )))
( "heehaw"  n ( h ii1 h oo ) ((pos "K6$" ) ))
( "heehaws"  n ( h ii1 h oo z ) ((pos "Kj$" ) ))
( "heel"  v ( h ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "heeled"  v ( h ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "heeling"  v ( h ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "heels"  v ( h ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "heftier"  j ( h e1 f t i@ r ) ((pos "Or%" ) ))
( "heftiest"  j ( h e1 f t i i s t ) ((pos "Os%" ) ))
( "hefty"  j ( h e1 f t ii ) ((pos "OD%" ) ))
( "hegemonies"  n ( h i1 g e1 m @ n i z ) ((pos "Mj$" ) ))
( "hegemony"  n ( h i1 g e1 m @ n ii ) ((pos "M8$" ) ))
( "heifer"  n ( h e1 f @ r ) ((pos "K6%" ) ))
( "heifers"  n ( h e1 f @ z ) ((pos "Kj%" ) ))
( "heighho"  uh ( h ei2 h ou1 ) ((pos "W-$" ) ))
( "height"  n ( h ai1 t ) ((pos "M6%" ) ))
( "heighten"  v ( h ai1 t n ) ((pos "J0%" ) (subcat "2A" )))
( "heightened"  v ( h ai1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "heightening"  v ( h ai1 t n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "heightens"  v ( h ai1 t n z ) ((pos "Ja%" ) (subcat "2A" )))
( "heights"  n ( h ai1 t s ) ((pos "Mj%" ) ))
( "heinous"  j ( h ei1 n @ s ) ((pos "OA%" ) ))
( "heinously"  a ( h ei1 n @ s l ii ) ((pos "Pu$" ) ))
( "heinousness"  n ( h ei1 n @ s n @ s ) ((pos "L@$" ) ))
( "heir"  n ( e@1 r ) ((pos "K6%" ) ))
( "heiress"  n ( e@1 r i s ) ((pos "K7%" ) ))
( "heiresses"  n ( e@1 r i s i z ) ((pos "Kj%" ) ))
( "heirloom"  n ( e@1 l uu m ) ((pos "K6%" ) ))
( "heirlooms"  n ( e@1 l uu m z ) ((pos "Kj%" ) ))
( "heirs"  n ( e@1 z ) ((pos "Kj%" ) ))
( "held"  v ( h e1 l d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2C" "2D" "3A" "6A" "9" "14" "15A" "15B" "22" )))
( "helicopter"  n ( h e1 l i k o p t @ r ) ((pos "K6%" ) ))
( "helicopters"  n ( h e1 l i k o p t @ z ) ((pos "Kj%" ) ))
( "heliograph"  v ( h ii1 l i@ g r aa f ) ((pos "H0$" "K6$" ) ))
( "heliographed"  v ( h ii1 l i@ g r aa f t ) ((pos "Hc$" "Hd$" ) ))
( "heliographing"  v ( h ii1 l i@ g r aa f i ng ) ((pos "Hb$" ) ))
( "heliographs"  v ( h ii1 l i@ g r aa f s ) ((pos "Ha$" "Kj$" ) ))
( "heliotrope"  n ( h ii1 l i@ t r ou p ) ((pos "K6$" ) ))
( "heliotropes"  n ( h ii1 l i@ t r ou p s ) ((pos "Kj$" ) ))
( "heliport"  n ( h e1 l i p oo t ) ((pos "K6$" ) ))
( "heliports"  n ( h e1 l i p oo t s ) ((pos "Kj$" ) ))
( "helium"  n ( h ii1 l i@ m ) ((pos "L@%" ) ))
( "hell"  n ( h e1 l ) ((pos "K6%" ) ))
( "hellcat"  n ( h e1 l k a t ) ((pos "K6%" ) ))
( "hellcats"  n ( h e1 l k a t s ) ((pos "Kj%" ) ))
( "hellish"  j ( h e1 l i sh ) ((pos "OA%" ) ))
( "hello"  uh ( h @1 l ou1 ) ((pos "W-%" ) ))
( "hells"  n ( h e1 l z ) ((pos "Kj$" ) ))
( "helm"  n ( h e1 l m ) ((pos "K6%" ) ))
( "helmet"  n ( h e1 l m i t ) ((pos "K6%" ) ))
( "helmeted"  j ( h e1 l m i t i d ) ((pos "OA%" ) ))
( "helmets"  n ( h e1 l m i t s ) ((pos "Kj%" ) ))
( "helms"  n ( h e1 l m z ) ((pos "Kj%" ) ))
( "helmsman"  n ( h e1 l m z m @ n ) ((pos "Ki%" ) ))
( "helmsmen"  n ( h e1 l m z m @ n ) ((pos "Kj%" ) ))
( "helot"  n ( h e1 l @ t ) ((pos "K6$" ) ))
( "helots"  n ( h e1 l @ t s ) ((pos "Kj$" ) ))
( "help"  v ( h e1 l p ) ((pos "J0*" "M6*" ) (subcat "2A" "2C" "6A" "6C" "14" "15A" "15B" "17" )))
( "helped"  v ( h e1 l p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "6C" "14" "15A" "15B" "17" )))
( "helper"  n ( h e1 l p @ r ) ((pos "K6%" ) ))
( "helpers"  n ( h e1 l p @ z ) ((pos "Kj%" ) ))
( "helpful"  j ( h e1 l p f @ l ) ((pos "OA%" ) ))
( "helpfully"  a ( h e1 l p f @ l ii ) ((pos "Pu%" ) ))
( "helpfulness"  n ( h e1 l p f @ l n @ s ) ((pos "L@%" ) ))
( "helping"  v ( h e1 l p i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "6C" "14" "15A" "15B" "17" )))
( "helpings"  n ( h e1 l p i ng z ) ((pos "Kj%" ) ))
( "helpless"  j ( h e1 l p l @ s ) ((pos "OA%" ) ))
( "helplessly"  a ( h e1 l p l @ s l ii ) ((pos "Pu%" ) ))
( "helplessness"  n ( h e1 l p l @ s n @ s ) ((pos "L@%" ) ))
( "helpmate"  n ( h e1 l p m ei t ) ((pos "K6$" ) ))
( "helpmates"  n ( h e1 l p m ei t s ) ((pos "Kj$" ) ))
( "helpmeet"  n ( h e1 l p m ii t ) ((pos "K6$" ) ))
( "helpmeets"  n ( h e1 l p m ii t s ) ((pos "Kj$" ) ))
( "helps"  v ( h e1 l p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "6C" "14" "15A" "15B" "17" )))
( "helter-skelter"  n ( h e2 l t @ - s k e1 l t @ r ) ((pos "K6%" "Pu%" ) ))
( "helter-skelters"  n ( h e2 l t @ - s k e1 l t @ z ) ((pos "Kj%" ) ))
( "helve"  n ( h e1 l v ) ((pos "K6$" ) ))
( "helves"  n ( h e1 l v z ) ((pos "Kj$" ) ))
( "hem"  v ( h e1 m ) ((pos "J4%" "K6%" "W-%" ) (subcat "6A" )))
( "hematite"  n ( h e1 m @ t ai t ) ((pos "K6$" ) ))
( "hematites"  n ( h e1 m @ t ai t s ) ((pos "Kj$" ) ))
( "hemisphere"  n ( h e1 m i s f i@ r ) ((pos "K6%" ) ))
( "hemispheres"  n ( h e1 m i s f i@ z ) ((pos "Kj%" ) ))
( "hemline"  n ( h e1 m l ai n ) ((pos "K6%" ) ))
( "hemlines"  n ( h e1 m l ai n z ) ((pos "Kj%" ) ))
( "hemlock"  n ( h e1 m l o k ) ((pos "M6%" ) ))
( "hemlocks"  n ( h e1 m l o k s ) ((pos "Mj$" ) ))
( "hemmed"  v ( h e1 m d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "hemming"  v ( h e1 m i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "hemming-stitch"  n ( h e1 m i ng - s t i ch ) ((pos "K7%" ) ))
( "hemming-stitches"  n ( h e1 m i ng - s t i ch i z ) ((pos "Kj$" ) ))
( "hemoglobin"  n ( h ii2 m @ g l ou1 b i n ) ((pos "L@$" ) ))
( "hemophilia"  n ( h ii2 m @ f i1 l i@ ) ((pos "L@$" ) ))
( "hemophiliac"  n ( h ii2 m @ f i1 l i a k ) ((pos "K6$" ) ))
( "hemophiliacs"  n ( h ii2 m @ f i1 l i a k s ) ((pos "Kj$" ) ))
( "hemorrhage"  n ( h e1 m @ r i jh ) ((pos "M6$" ) ))
( "hemorrhages"  n ( h e1 m @ r i jh i z ) ((pos "Mj$" ) ))
( "hemorrhoids"  n ( h e1 m @ r oi d z ) ((pos "Kj$" ) ))
( "hemp"  n ( h e1 m p ) ((pos "L@%" ) ))
( "hempen"  j ( h e1 m p @ n ) ((pos "OA$" ) ))
( "hems"  v ( h e1 m z ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "hemstitch"  v ( h e1 m s t i ch ) ((pos "H1$" "K7%" ) ))
( "hemstitched"  v ( h e1 m s t i ch t ) ((pos "Hc$" "Hd%" ) ))
( "hemstitches"  v ( h e1 m s t i ch i z ) ((pos "Ha$" "Kj%" ) ))
( "hemstitching"  v ( h e1 m s t i ch i ng ) ((pos "Hb%" ) ))
( "hen"  n ( h e1 n ) ((pos "K6%" ) ))
( "hen-parties"  n ( h e1 n - p aa t i z ) ((pos "Kj%" ) ))
( "hen-party"  n ( h e1 n - p aa t ii ) ((pos "K8%" ) ))
( "henbane"  n ( h e1 n b ei n ) ((pos "L@$" ) ))
( "hence"  a ( h e1 n s ) ((pos "Pu%" ) ))
( "henceforth"  a ( h e1 n s f oo1 th ) ((pos "Pu%" ) ))
( "henceforward"  a ( h e1 n s f oo1 w @ d ) ((pos "Pu%" ) ))
( "henchman"  n ( h e1 n ch m @ n ) ((pos "Ki%" ) ))
( "henchmen"  n ( h e1 n ch m @ n ) ((pos "Kj%" ) ))
( "hencoop"  n ( h e1 n k uu p ) ((pos "K6%" ) ))
( "hencoops"  n ( h e1 n k uu p s ) ((pos "Kj%" ) ))
( "henhouse"  n ( h e1 n h au s ) ((pos "K6%" ) ))
( "henhouses"  n ( h e1 n h au z i z ) ((pos "Kj%" ) ))
( "henna"  n ( h e1 n @ ) ((pos "L@%" ) ))
( "hennaed"  j ( h e1 n @ d ) ((pos "OA%" ) ))
( "henpecked"  j ( h e1 n p e k t ) ((pos "OA%" ) ))
( "henroost"  n ( h e1 n r uu s t ) ((pos "K6%" ) ))
( "henroosts"  n ( h e1 n r uu s t s ) ((pos "Kj%" ) ))
( "hens"  n ( h e1 n z ) ((pos "Kj%" ) ))
( "hep"  j ( h e1 p ) ((pos "OA$" ) ))
( "hepatitis"  n ( h e2 p @ t ai1 t i s ) ((pos "L@%" ) ))
( "heptagon"  n ( h e1 p t @ g @ n ) ((pos "K6%" ) ))
( "heptagons"  n ( h e1 p t @ g @ n z ) ((pos "Kj%" ) ))
( "her"  j ( h @@ r ) ((pos "Oq*" "Qx*" ) ))
( "herald"  v ( h e1 r @ l d ) ((pos "H0%" "K6%" ) (subcat )))
( "heralded"  v ( h e1 r @ l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "heraldic"  j ( h e1 r a1 l d i k ) ((pos "OA%" ) ))
( "heralding"  v ( h e1 r @ l d i ng ) ((pos "Hb%" ) (subcat )))
( "heraldry"  n ( h e1 r @ l d r ii ) ((pos "L@%" ) ))
( "heralds"  v ( h e1 r @ l d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "herb"  n ( h @@1 b ) ((pos "K6%" ) ))
( "herbaceous"  j ( h @@1 b ei1 sh @ s ) ((pos "OA%" ) ))
( "herbage"  n ( h @@1 b i jh ) ((pos "L@$" ) ))
( "herbal"  j ( h @@1 b l ) ((pos "OA%" ) ))
( "herbalist"  n ( h @@1 b @ l i s t ) ((pos "K6%" ) ))
( "herbalists"  n ( h @@1 b @ l i s t s ) ((pos "Kj%" ) ))
( "herbivorous"  j ( h @@1 b i1 v @ r @ s ) ((pos "OA%" ) ))
( "herbs"  n ( h @@1 b z ) ((pos "Kj%" ) ))
( "herculean"  j ( h @@2 k y u l ii1 @ n ) ((pos "OA%" ) ))
( "herd"  v ( h @@1 d ) ((pos "J0%" "K6%" ) (subcat "2C" "15A" )))
( "herded"  v ( h @@1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "15A" )))
( "herding"  v ( h @@1 d i ng ) ((pos "Jb%" ) (subcat "2C" "15A" )))
( "herds"  v ( h @@1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "15A" )))
( "herdsman"  n ( h @@1 d z m @ n ) ((pos "Ki%" ) ))
( "herdsmen"  n ( h @@1 d z m @ n ) ((pos "Kj%" ) ))
( "here"  a ( h i@1 r ) ((pos "Pu*" ) ))
( "here's"  v ( h i@ z ) ((pos "Gh%" ) ))
( "hereabouts"  a ( h i@2 r @ b au1 t s ) ((pos "Pu%" ) ))
( "hereafter"  n ( h i@1 r aa1 f t @ r ) ((pos "Ki%" "Pu%" ) ))
( "hereby"  a ( h i@1 b ai1 ) ((pos "Pu%" ) ))
( "hereditament"  n ( h e2 r i d i1 t @ m @ n t ) ((pos "K6$" ) ))
( "hereditaments"  n ( h e2 r i d i1 t @ m @ n t s ) ((pos "Kj$" ) ))
( "hereditary"  j ( h i1 r e1 d i t r ii ) ((pos "OA%" ) ))
( "heredity"  n ( h i1 r e1 d i t ii ) ((pos "L@%" ) ))
( "herein"  a ( h i@2 r i1 n ) ((pos "Pu%" ) ))
( "hereinafter"  a ( h i@2 r i n aa1 f t @ r ) ((pos "Pu%" ) ))
( "hereinbefore"  a ( h i@2 r i n b i f oo1 r ) ((pos "Pu$" ) ))
( "hereof"  a ( h i@1 r o1 v ) ((pos "Pu%" ) ))
( "heresies"  n ( h e1 r @ s i z ) ((pos "Mj%" ) ))
( "heresy"  n ( h e1 r @ s ii ) ((pos "M8%" ) ))
( "heretic"  n ( h e1 r @ t i k ) ((pos "K6%" ) ))
( "heretical"  j ( h i1 r e1 t i k l ) ((pos "OA%" ) ))
( "heretics"  n ( h e1 r @ t i k s ) ((pos "Kj%" ) ))
( "hereto"  a ( h i@1 t uu1 ) ((pos "Pu%" ) ))
( "heretofore"  a ( h i@2 t uu f oo1 r ) ((pos "Pu$" ) ))
( "hereupon"  a ( h i@2 r @ p o1 n ) ((pos "Pu%" ) ))
( "herewith"  a ( h i@2 w i1 dh ) ((pos "Pu%" ) ))
( "heritable"  j ( h e1 r i t @ b l ) ((pos "OA$" ) ))
( "heritage"  n ( h e1 r i t i jh ) ((pos "Ki%" ) ))
( "hermaphrodite"  n ( h @@1 m a1 f r @ d ai t ) ((pos "K6$" ) ))
( "hermaphrodites"  n ( h @@1 m a1 f r @ d ai t s ) ((pos "Kj$" ) ))
( "hermetic"  j ( h @@1 m e1 t i k ) ((pos "OA$" ) ))
( "hermetically"  a ( h @@1 m e1 t i k l ii ) ((pos "Pu%" ) ))
( "hermit"  n ( h @@1 m i t ) ((pos "K6%" ) ))
( "hermitage"  n ( h @@1 m i t i jh ) ((pos "K6%" ) ))
( "hermitages"  n ( h @@1 m i t i jh i z ) ((pos "Kj%" ) ))
( "hermits"  n ( h @@1 m i t s ) ((pos "Kj%" ) ))
( "hernia"  n ( h @@1 n i@ ) ((pos "M6%" ) ))
( "hernias"  n ( h @@1 n i@ z ) ((pos "Mj%" ) ))
( "hero"  n ( h i@1 r ou ) ((pos "K7%" ) ))
( "heroes"  n ( h i@1 r ou z ) ((pos "Kj%" ) ))
( "heroic"  j ( h i1 r ou1 i k ) ((pos "OA%" ) ))
( "heroically"  a ( h i1 r ou1 i k l ii ) ((pos "Pu%" ) ))
( "heroics"  n ( h i1 r ou1 i k s ) ((pos "Lk%" ) ))
( "heroin"  n ( h e1 r ou i n ) ((pos "L@%" ) ))
( "heroine"  n ( h e1 r ou i n ) ((pos "K6%" ) ))
( "heroines"  n ( h e1 r ou i n z ) ((pos "Kj%" ) ))
( "heroism"  n ( h e1 r ou i z @ m ) ((pos "L@%" ) ))
( "heron"  n ( h e1 r @ n ) ((pos "K6%" ) ))
( "heronries"  n ( h e1 r @ n r i z ) ((pos "Kj$" ) ))
( "heronry"  n ( h e1 r @ n r ii ) ((pos "K8$" ) ))
( "herons"  n ( h e1 r @ n z ) ((pos "Kj%" ) ))
( "herring"  n ( h e1 r i ng ) ((pos "M9%" ) ))
( "herringbone"  n ( h e1 r i ng b ou n ) ((pos "L@%" ) ))
( "herrings"  n ( h e1 r i ng z ) ((pos "Kj%" ) ))
( "hers"  prp ( h @@ z ) ((pos "Qx%" ) ))
( "herself"  prp ( h @@1 s e1 l f ) ((pos "Qx%" ) ))
( "hertz"  n ( h @@1 t s ) ((pos "K9$" ) ))
( "hesitance"  n ( h e1 z i t @ n s ) ((pos "Mi$" ) ))
( "hesitancy"  n ( h e1 z i t @ n s ii ) ((pos "L@$" ) ))
( "hesitant"  j ( h e1 z i t @ n t ) ((pos "OA%" ) ))
( "hesitantly"  a ( h e1 z i t @ n t l ii ) ((pos "Pu%" ) ))
( "hesitate"  v ( h e1 z i t ei t ) ((pos "I2%" ) (subcat "2A" "3A" "3B" )))
( "hesitated"  v ( h e1 z i t ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" "3B" )))
( "hesitates"  v ( h e1 z i t ei t s ) ((pos "Ia%" ) (subcat "2A" "3A" "3B" )))
( "hesitating"  v ( h e1 z i t ei t i ng ) ((pos "Ib%" ) (subcat "2A" "3A" "3B" )))
( "hesitatingly"  a ( h e1 s i t ei t i ng l ii ) ((pos "Pu%" ) ))
( "hesitation"  n ( h e2 z i t ei1 sh @ n ) ((pos "M6%" ) ))
( "hesitations"  n ( h e2 z i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "hessian"  n ( h e1 s i@ n ) ((pos "L@%" ) ))
( "het-up"  j ( h e2 t - uh1 p ) ((pos "OA%" ) ))
( "heterodox"  j ( h e1 t @ r @ d o k s ) ((pos "OA$" ) ))
( "heterodoxy"  n ( h e1 t @ r @ d o k s ii ) ((pos "L@$" ) ))
( "heterogeneous"  j ( h e2 t @ r @ jh ii1 n i@ s ) ((pos "OA%" ) ))
( "heterosexual"  n ( h e2 t @ r @ s e1 k sh u@ l ) ((pos "K6%" "OA%" ) ))
( "heterosexuality"  n ( h e2 t @ r @ s e2 k sh u a1 l i t ii ) ((pos "L@%" ) ))
( "heterosexuals"  n ( h e2 t @ r @ s e1 k sh u@ l z ) ((pos "Kj%" ) ))
( "heuristic"  j ( h y u@1 r i1 s t i k ) ((pos "OA$" ) ))
( "heuristics"  n ( h y u@1 r i1 s t i k s ) ((pos "Lk$" ) ))
( "hew"  v ( h y uu1 ) ((pos "J0$" ) (subcat "2A" "2C" "6A" "15A" )))
( "hewed"  v ( h y uu1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" "15A" )))
( "hewer"  n ( h y uu1 @ r ) ((pos "K6$" ) ))
( "hewers"  n ( h y uu1 @ z ) ((pos "Kj$" ) ))
( "hewing"  v ( h y uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hewn"  v ( h y uu1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hews"  v ( h y uu1 z ) ((pos "Ja$" ) (subcat "2A" "2C" "6A" "15A" )))
( "hexagon"  n ( h e1 k s @ g @ n ) ((pos "K6%" ) ))
( "hexagonal"  j ( h e1 k s a1 g @ n l ) ((pos "OA%" ) ))
( "hexagons"  n ( h e1 k s @ g @ n z ) ((pos "Kj%" ) ))
( "hexameter"  n ( h e1 k s a1 m i t @ r ) ((pos "K6$" ) ))
( "hexameters"  n ( h e1 k s a1 m i t @ z ) ((pos "Kj$" ) ))
( "hey"  uh ( h ei1 ) ((pos "W-%" ) ))
( "heyday"  n ( h ei1 d ei ) ((pos "Ki%" ) ))
( "hi"  uh ( h ai1 ) ((pos "W-%" ) ))
( "hi-fi"  n ( h ai2 - f ai1 ) ((pos "K6%" "OA%" ) ))
( "hi-fis"  n ( h ai2 - f ai1 z ) ((pos "Kj$" ) ))
( "hiatus"  n ( h ai1 ei1 t @ s ) ((pos "K7$" ) ))
( "hiatuses"  n ( h ai1 ei1 t @ s i z ) ((pos "Kj$" ) ))
( "hibernate"  v ( h ai1 b @ n ei t ) ((pos "I2%" ) (subcat )))
( "hibernated"  v ( h ai1 b @ n ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "hibernates"  v ( h ai1 b @ n ei t s ) ((pos "Ia%" ) (subcat )))
( "hibernating"  v ( h ai1 b @ n ei t i ng ) ((pos "Ib%" ) (subcat )))
( "hibernation"  n ( h ai2 b @ n ei1 sh @ n ) ((pos "L@%" ) ))
( "hibiscus"  n ( h i1 b i1 s k @ s ) ((pos "L@$" ) ))
( "hiccough"  v ( h i1 k uh p ) ((pos "I0$" "K6$" ) ))
( "hiccoughed"  v ( h i1 k uh p t ) ((pos "Ic$" "Id$" ) ))
( "hiccoughing"  v ( h i1 k uh p i ng ) ((pos "Ib$" ) ))
( "hiccoughs"  v ( h i1 k uh p s ) ((pos "Ia$" "Kj$" ) ))
( "hiccup"  v ( h i1 k uh p ) ((pos "I4%" "K6%" ) ))
( "hiccupped"  v ( h i1 k uh p t ) ((pos "Ic%" "Id%" ) ))
( "hiccupping"  v ( h i1 k uh p i ng ) ((pos "Ib%" ) ))
( "hiccups"  v ( h i1 k uh p s ) ((pos "Ia%" "Kj%" ) ))
( "hick"  n ( h i1 k ) ((pos "K6$" "OA$" ) ))
( "hickories"  n ( h i1 k @ r i z ) ((pos "Mj$" ) ))
( "hickory"  n ( h i1 k @ r ii ) ((pos "M8$" ) ))
( "hicks"  n ( h i1 k s ) ((pos "Kj$" ) ))
( "hid"  v ( h i1 d ) ((pos "Jc%" ) (subcat "2A" "6A" )))
( "hidden"  v ( h i1 d n ) ((pos "Jd%" ) (subcat "2A" "6A" )))
( "hide"  v ( h ai1 d ) ((pos "J5%" "M6%" ) (subcat "2A" "6A" )))
( "hide-and-seek"  n ( h ai2 d - n - s ii1 k ) ((pos "L@%" ) ))
( "hide-out"  n ( h ai1 d - au t ) ((pos "K6%" ) ))
( "hide-outs"  n ( h ai1 d - au t s ) ((pos "Kj$" ) ))
( "hideaway"  n ( h ai1 d @ w ei ) ((pos "K6%" ) ))
( "hideaways"  n ( h ai1 d @ w ei z ) ((pos "Kj$" ) ))
( "hidebound"  j ( h ai1 d b au n d ) ((pos "OA%" ) ))
( "hideous"  j ( h i1 d i@ s ) ((pos "OA%" ) ))
( "hideously"  a ( h i1 d i@ s l ii ) ((pos "Pu%" ) ))
( "hides"  v ( h ai1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "hiding"  v ( h ai1 d i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" )))
( "hiding-place"  n ( h ai1 d i ng - p l ei s ) ((pos "K6%" ) ))
( "hiding-places"  n ( h ai1 d i ng - p l ei s i z ) ((pos "Kj%" ) ))
( "hidings"  n ( h ai1 d i ng z ) ((pos "Mj$" ) ))
( "hie"  v ( h ai1 ) ((pos "I5$" ) ))
( "hied"  v ( h ai1 d ) ((pos "Ic$" "Id$" ) ))
( "hieing"  v ( h ai1 i ng ) ((pos "Ib$" ) ))
( "hierarchic"  j ( h ai2 @ r aa1 k i k ) ((pos "OA$" ) ))
( "hierarchical"  j ( h ai2 @ r aa1 k i k l ) ((pos "OA%" ) ))
( "hierarchies"  n ( h ai1 @ r aa k i z ) ((pos "Kj%" ) ))
( "hierarchy"  n ( h ai1 @ r aa k ii ) ((pos "K8%" ) ))
( "hieroglyph"  n ( h ai1 @ r @ g l i f ) ((pos "K6$" ) ))
( "hieroglyphic"  j ( h ai2 @ r @ g l i1 f i k ) ((pos "OA$" ) ))
( "hieroglyphics"  n ( h ai2 @ r @ g l i1 f i k s ) ((pos "Lk%" ) ))
( "hieroglyphs"  n ( h ai1 @ r @ g l i f s ) ((pos "Kj$" ) ))
( "hies"  v ( h ai1 z ) ((pos "Ia$" ) ))
( "higgledy-piggledy"  j ( h i2 g l d i - p i1 g l d ii ) ((pos "OA%" "Pu%" ) ))
( "high"  j ( h ai1 ) ((pos "OC*" "Pu*" ) ))
( "high-class"  j ( h ai1 - k l aa s ) ((pos "OA%" ) ))
( "high-falutin"  j ( h ai2 - f @ l uu1 t n ) ((pos "OA%" ) ))
( "high-fidelity"  n ( h ai2 - f i d e1 l i t ii ) ((pos "L@%" "OA%" ) ))
( "high-frequencies"  n ( h ai1 - f r ii1 k w @ n s i z ) ((pos "Mj$" ) ))
( "high-frequency"  n ( h ai1 - f r ii1 k w @ n s ii ) ((pos "M8%" ) ))
( "high-grade"  j ( h ai1 - g r ei d ) ((pos "OA%" ) ))
( "high-handed"  j ( h ai1 - h a1 n d i d ) ((pos "OA%" ) ))
( "high-handedly"  a ( h ai1 - h a1 n d i d l ii ) ((pos "Pu%" ) ))
( "high-keyed"  j ( h ai1 - k ii1 d ) ((pos "OA$" ) ))
( "high-level"  j ( h ai1 - l e v @ l ) ((pos "OA%" ) ))
( "high-minded"  j ( h ai1 - m ai1 n d i d ) ((pos "OA%" ) ))
( "high-mindedly"  a ( h ai2 - m ai1 n d i d l ii ) ((pos "Pu%" ) ))
( "high-mindedness"  n ( h ai2 - m ai1 n d i d n @ s ) ((pos "L@%" ) ))
( "high-necked"  j ( h ai1 - n e1 k t ) ((pos "OA%" ) ))
( "high-octane"  j ( h ai1 - o1 k t ei n ) ((pos "OA%" ) ))
( "high-pitched"  j ( h ai1 - p i1 ch t ) ((pos "OA%" ) ))
( "high-powered"  j ( h ai1 - p au1 @ d ) ((pos "OA%" ) ))
( "high-pressure"  n ( h ai1 - p r e1 sh @ r ) ((pos "L@%" ) ))
( "high-priced"  j ( h ai1 - p r ai1 s t ) ((pos "OA%" ) ))
( "high-principled"  j ( h ai1 - p r i1 n s @ p l d ) ((pos "OA%" ) ))
( "high-ranking"  j ( h ai1 - r a1 n k i ng ) ((pos "OA%" ) ))
( "high-rise"  j ( h ai1 - r ai z ) ((pos "Oq%" ) ))
( "high-sounding"  j ( h ai1 - s au1 n d i ng ) ((pos "OA%" ) ))
( "high-speed"  j ( h ai1 - s p ii d ) ((pos "OA%" ) ))
( "high-spirited"  j ( h ai1 - s p i1 r i t i d ) ((pos "OA%" ) ))
( "high-tension"  j ( h ai1 - t e1 n sh @ n ) ((pos "OA%" ) ))
( "high-toned"  j ( h ai1 - t ou1 n d ) ((pos "OA%" ) ))
( "high-up"  n ( h ai1 - uh p ) ((pos "K6$" ) ))
( "high-ups"  n ( h ai1 - uh p s ) ((pos "Kj%" ) ))
( "highball"  n ( h ai1 b oo l ) ((pos "K6$" ) ))
( "highballs"  n ( h ai1 b oo l z ) ((pos "Kj$" ) ))
( "highborn"  j ( h ai1 b oo n ) ((pos "OA%" ) ))
( "highboy"  n ( h ai1 b oi ) ((pos "K6$" ) ))
( "highboys"  n ( h ai1 b oi z ) ((pos "Kj$" ) ))
( "highbrow"  n ( h ai1 b r au ) ((pos "K6$" "OA%" ) ))
( "highbrows"  n ( h ai1 b r au z ) ((pos "Kj$" ) ))
( "higher"  j ( h ai1 @ r ) ((pos "Or%" ) ))
( "higher-ups"  n ( h ai2 @ r - uh1 p s ) ((pos "Kj$" ) ))
( "highest"  j ( h ai1 i s t ) ((pos "Os%" ) ))
( "highflier"  n ( h ai1 f l ai1 @ r ) ((pos "K6%" ) ))
( "highfliers"  n ( h ai1 f l ai1 @ z ) ((pos "Kj%" ) ))
( "highflown"  j ( h ai1 f l ou n ) ((pos "OA%" ) ))
( "highflyer"  n ( h ai1 f l ai1 @ r ) ((pos "K6%" ) ))
( "highflyers"  n ( h ai1 f l ai1 @ z ) ((pos "Kj%" ) ))
( "highflying"  j ( h ai1 f l ai1 i ng ) ((pos "OA%" ) ))
( "highjack"  v ( h ai1 jh a k ) ((pos "H0$" "K6$" ) (subcat )))
( "highjacked"  v ( h ai1 jh a k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "highjacking"  v ( h ai1 jh a k i ng ) ((pos "Hb$" ) (subcat )))
( "highjacks"  v ( h ai1 jh a k s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "highland"  n ( h ai1 l a n d ) ((pos "K6%" ) ))
( "highlands"  n ( h ai1 l a n d z ) ((pos "Kj%" ) ))
( "highlight"  v ( h ai1 l ai t ) ((pos "H0%" "K6%" ) ))
( "highlighted"  v ( h ai1 l ai t i d ) ((pos "Hc%" "Hd%" ) ))
( "highlighting"  v ( h ai1 l ai t i ng ) ((pos "Hb%" ) ))
( "highlights"  v ( h ai1 l ai t s ) ((pos "Ha%" "Kj%" ) ))
( "highly"  a ( h ai1 l ii ) ((pos "Pu%" ) ))
( "highness"  n ( h ai1 n @ s ) ((pos "M7%" ) ))
( "highnesses"  n ( h ai1 n @ s i z ) ((pos "Mj%" ) ))
( "highroad"  n ( h ai1 r ou d ) ((pos "K6%" ) ))
( "highroads"  n ( h ai1 r ou d z ) ((pos "Kj%" ) ))
( "highway"  n ( h ai1 w ei ) ((pos "K6%" ) ))
( "highwayman"  n ( h ai1 w ei m @ n ) ((pos "Ki%" ) ))
( "highwaymen"  n ( h ai1 w ei m @ n ) ((pos "Kj%" ) ))
( "highways"  n ( h ai1 w ei z ) ((pos "Kj%" ) ))
( "hijack"  v ( h ai1 jh a k ) ((pos "H0%" "K6%" ) (subcat )))
( "hijacked"  v ( h ai1 jh a k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hijacker"  n ( h ai1 jh a k @ r ) ((pos "K6%" ) ))
( "hijackers"  n ( h ai1 jh a k @ z ) ((pos "Kj%" ) ))
( "hijacking"  v ( h ai1 jh a k i ng ) ((pos "Hb%" ) (subcat )))
( "hijacks"  v ( h ai1 jh a k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hike"  v ( h ai1 k ) ((pos "I2%" "K6%" ) ))
( "hiked"  v ( h ai1 k t ) ((pos "Ic%" "Id%" ) ))
( "hiker"  n ( h ai1 k @ r ) ((pos "K6%" ) ))
( "hikers"  n ( h ai1 k @ z ) ((pos "Kj%" ) ))
( "hikes"  v ( h ai1 k s ) ((pos "Ia%" "Kj%" ) ))
( "hiking"  v ( h ai1 k i ng ) ((pos "Ib%" ) ))
( "hilarious"  j ( h i1 l e@1 r i@ s ) ((pos "OA%" ) ))
( "hilariously"  a ( h i1 l e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "hilarity"  n ( h i1 l a1 r i t ii ) ((pos "L@%" ) ))
( "hill"  n ( h i1 l ) ((pos "K6%" ) ))
( "hill-billies"  n ( h i1 l - b i l i z ) ((pos "Kj%" ) ))
( "hill-billy"  n ( h i1 l - b i l ii ) ((pos "K8%" ) ))
( "hillier"  j ( h i1 l i@ r ) ((pos "Or%" ) ))
( "hilliest"  j ( h i1 l i i s t ) ((pos "Os%" ) ))
( "hillock"  n ( h i1 l @ k ) ((pos "K6%" ) ))
( "hillocks"  n ( h i1 l @ k s ) ((pos "Kj%" ) ))
( "hills"  n ( h i1 l z ) ((pos "Kj%" ) ))
( "hillside"  n ( h i1 l s ai d ) ((pos "K6%" ) ))
( "hillsides"  n ( h i1 l s ai d z ) ((pos "Kj%" ) ))
( "hilly"  j ( h i1 l ii ) ((pos "OD%" ) ))
( "hilt"  n ( h i1 l t ) ((pos "K6%" ) ))
( "hilts"  n ( h i1 l t s ) ((pos "Kj$" ) ))
( "him"  prp ( h i m ) ((pos "Qx*" ) ))
( "himself"  prp ( h i1 m s e1 l f ) ((pos "Qx*" ) ))
( "hind"  n ( h ai1 n d ) ((pos "K6$" "OA$" ) ))
( "hinder"  v ( h i1 n d @ r ) ((pos "H0%" ) (subcat "6A" "6C" )))
( "hindered"  v ( h i1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "hindering"  v ( h i1 n d @ r i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "hinders"  v ( h i1 n d @ z ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "hindmost"  j ( h ai1 n d m ou s t ) ((pos "OA%" ) ))
( "hindrance"  n ( h i1 n d r @ n s ) ((pos "K6%" ) ))
( "hindrances"  n ( h i1 n d r @ n s i z ) ((pos "Kj%" ) ))
( "hinds"  n ( h ai1 n d z ) ((pos "Kj$" ) ))
( "hindsight"  n ( h ai1 n d s ai t ) ((pos "L@%" ) ))
( "hinge"  v ( h i1 n jh ) ((pos "J2%" "K6%" ) (subcat "3A" )))
( "hinged"  v ( h i1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "hinges"  v ( h i1 n jh i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "hinging"  v ( h i1 n jh i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "hint"  v ( h i1 n t ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" )))
( "hinted"  v ( h i1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "hinterland"  n ( h i1 n t @ l a n d ) ((pos "K6%" ) ))
( "hinterlands"  n ( h i1 n t @ l a n d z ) ((pos "Kj$" ) ))
( "hinting"  v ( h i1 n t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "hints"  v ( h i1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "hip"  n ( h i1 p ) ((pos "K6%" "OA%" "W-%" ) ))
( "hip-bath"  n ( h i1 p - b aa th ) ((pos "K6%" ) ))
( "hip-baths"  n ( h i1 p - b aa dh z ) ((pos "Kj%" ) ))
( "hip-flask"  n ( h i1 p - f l aa s k ) ((pos "K6%" ) ))
( "hip-flasks"  n ( h i1 p - f l aa s k s ) ((pos "Kj%" ) ))
( "hip-pocket"  n ( h i1 p - p o1 k i t ) ((pos "K6%" ) ))
( "hip-pockets"  n ( h i1 p - p o1 k i t s ) ((pos "Kj%" ) ))
( "hippie"  n ( h i1 p ii ) ((pos "K6%" ) ))
( "hippies"  n ( h i1 p i z ) ((pos "Kj%" ) ))
( "hippo"  n ( h i1 p ou ) ((pos "K6%" ) ))
( "hippodrome"  n ( h i1 p @ d r ou m ) ((pos "K6$" ) ))
( "hippodromes"  n ( h i1 p @ d r ou m z ) ((pos "Kj$" ) ))
( "hippopotami"  n ( h i2 p @ p o1 t @ m ai ) ((pos "Kj%" ) ))
( "hippopotamus"  n ( h i2 p @ p o1 t @ m @ s ) ((pos "K7%" ) ))
( "hippopotamuses"  n ( h i2 p @ p o1 t @ m @ s i z ) ((pos "Kj%" ) ))
( "hippos"  n ( h i1 p ou z ) ((pos "Kj%" ) ))
( "hippy"  n ( h i1 p ii ) ((pos "K8%" ) ))
( "hips"  n ( h i1 p s ) ((pos "Kj%" ) ))
( "hire"  v ( h ai1 @ r ) ((pos "H2%" "L@%" ) (subcat "6A" )))
( "hired"  v ( h ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hireling"  n ( h ai1 @ l i ng ) ((pos "K6%" ) ))
( "hirelings"  n ( h ai1 @ l i ng z ) ((pos "Kj%" ) ))
( "hires"  v ( h ai1 @ z ) ((pos "Ha%" ) (subcat "6A" )))
( "hiring"  v ( h ai1 @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hirsute"  j ( h @@1 s y uu t ) ((pos "OA$" ) ))
( "his"  j ( h i z ) ((pos "OA*" "Qx*" ) ))
( "hiss"  v ( h i1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "3A" "6A" )))
( "hissed"  v ( h i1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "hisses"  v ( h i1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "hissing"  v ( h i1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "histogram"  n ( h i1 s t @ g r a m ) ((pos "K6%" ) ))
( "histograms"  n ( h i1 s t @ g r a m z ) ((pos "Kj%" ) ))
( "historian"  n ( h i1 s t oo1 r i@ n ) ((pos "K6%" ) ))
( "historians"  n ( h i1 s t oo1 r i@ n z ) ((pos "Kj%" ) ))
( "historic"  j ( h i1 s t o1 r i k ) ((pos "OA%" ) ))
( "historical"  j ( h i1 s t o1 r i k l ) ((pos "OA%" ) ))
( "historically"  a ( h i1 s t o1 r i k l ii ) ((pos "Pu%" ) ))
( "histories"  n ( h i1 s t r i z ) ((pos "Mj%" ) ))
( "history"  n ( h i1 s t r ii ) ((pos "M8%" ) ))
( "histrionic"  j ( h i2 s t r i o1 n i k ) ((pos "OA%" ) ))
( "histrionics"  n ( h i2 s t r i o1 n i k s ) ((pos "Lk%" ) ))
( "hit"  v ( h i1 t ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2C" "3A" "6A" "12C" "15A" )))
( "hit-and-run"  j ( h i2 t - n - r uh1 n ) ((pos "Oq%" ) ))
( "hitch"  v ( h i1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hitched"  v ( h i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hitches"  v ( h i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hitchhike"  v ( h i1 ch h ai k ) ((pos "I2%" ) (subcat )))
( "hitchhiked"  v ( h i1 ch h ai k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "hitchhiker"  n ( h i1 ch h ai k @ r ) ((pos "K6%" ) ))
( "hitchhikers"  n ( h i1 ch h ai k @ z ) ((pos "Kj%" ) ))
( "hitchhikes"  v ( h i1 ch h ai k s ) ((pos "Ia%" ) (subcat )))
( "hitchhiking"  v ( h i1 ch h ai k i ng ) ((pos "Ib%" ) (subcat )))
( "hitching"  v ( h i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hither"  a ( h i1 dh @ r ) ((pos "Pu%" ) ))
( "hitherto"  a ( h i2 dh @ t uu1 ) ((pos "Pu%" ) ))
( "hits"  v ( h i1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "12C" "15A" )))
( "hitting"  v ( h i1 t i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "12C" "15A" )))
( "hive"  v ( h ai1 v ) ((pos "J2$" "K6%" ) (subcat "2C" )))
( "hived"  v ( h ai1 v d ) ((pos "Jc$" "Jd$" ) (subcat "2C" )))
( "hives"  v ( h ai1 v z ) ((pos "Ja$" "Kj%" ) (subcat "2C" )))
( "hiving"  v ( h ai1 v i ng ) ((pos "Jb$" ) (subcat "2C" )))
( "ho"  uh ( h ou1 ) ((pos "W-$" ) ))
( "hoar"  j ( h oo1 r ) ((pos "OA$" ) ))
( "hoard"  v ( h oo1 d ) ((pos "J0%" "K6%" ) (subcat "6A" )))
( "hoarded"  v ( h oo1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "hoarder"  n ( h oo1 d @ r ) ((pos "K6%" ) ))
( "hoarders"  n ( h oo1 d @ z ) ((pos "Kj%" ) ))
( "hoarding"  v ( h oo1 d i ng ) ((pos "Jb%" "K6%" ) (subcat "6A" )))
( "hoardings"  n ( h oo1 d i ng z ) ((pos "Kj%" ) ))
( "hoards"  v ( h oo1 d z ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "hoarfrost"  n ( h oo1 f r o s t ) ((pos "L@%" ) ))
( "hoarier"  j ( h oo1 r i@ r ) ((pos "Or$" ) ))
( "hoariest"  j ( h oo1 r i i s t ) ((pos "Os$" ) ))
( "hoariness"  n ( h oo1 r i n @ s ) ((pos "L@$" ) ))
( "hoarse"  j ( h oo1 s ) ((pos "OB%" ) ))
( "hoarsely"  a ( h oo1 s l ii ) ((pos "Pu%" ) ))
( "hoarseness"  n ( h oo1 s n @ s ) ((pos "L@%" ) ))
( "hoarser"  j ( h oo1 s @ r ) ((pos "Or%" ) ))
( "hoarsest"  j ( h oo1 s i s t ) ((pos "Os%" ) ))
( "hoary"  j ( h oo1 r ii ) ((pos "OD%" ) ))
( "hoax"  v ( h ou1 k s ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "hoaxed"  v ( h ou1 k s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hoaxer"  n ( h ou1 k s @ r ) ((pos "K6%" ) ))
( "hoaxers"  n ( h ou1 k s @ z ) ((pos "Kj%" ) ))
( "hoaxes"  v ( h ou1 k s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "hoaxing"  v ( h ou1 k s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hob"  n ( h o1 b ) ((pos "K6%" ) ))
( "hobbies"  n ( h o1 b i z ) ((pos "Kj%" ) ))
( "hobble"  v ( h o1 b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "hobble-skirt"  n ( h o1 b l - s k @@ t ) ((pos "K6$" ) ))
( "hobble-skirts"  n ( h o1 b l - s k @@ t s ) ((pos "Kj$" ) ))
( "hobbled"  v ( h o1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "hobbledehoy"  n ( h o1 b l d i h oi ) ((pos "K6$" ) ))
( "hobbledehoys"  n ( h o1 b l d i h oi z ) ((pos "Kj$" ) ))
( "hobbles"  v ( h o1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "hobbling"  v ( h o1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "hobby"  n ( h o1 b ii ) ((pos "K8%" ) ))
( "hobbyhorse"  n ( h o1 b i h oo s ) ((pos "K6%" ) ))
( "hobbyhorses"  n ( h o1 b i h oo s i z ) ((pos "Kj%" ) ))
( "hobgoblin"  n ( h o2 b g o1 b l i n ) ((pos "K6%" ) ))
( "hobgoblins"  n ( h o2 b g o1 b l i n z ) ((pos "Kj%" ) ))
( "hobnail"  n ( h o1 b n ei l ) ((pos "K6%" ) ))
( "hobnailed"  j ( h o1 b n ei l d ) ((pos "OA%" ) ))
( "hobnails"  n ( h o1 b n ei l z ) ((pos "Kj$" ) ))
( "hobnob"  v ( h o1 b n o b ) ((pos "I4%" ) (subcat "2A" "2C" )))
( "hobnobbed"  v ( h o1 b n o b d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "hobnobbing"  v ( h o1 b n o b i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "hobnobs"  v ( h o1 b n o b z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "hobo"  n ( h ou1 b ou ) ((pos "K6%" ) ))
( "hoboes"  n ( h ou1 b ou z ) ((pos "Kj$" ) ))
( "hobos"  n ( h ou1 b ou z ) ((pos "Kj%" ) ))
( "hobs"  n ( h o1 b z ) ((pos "Kj%" ) ))
( "hock"  v ( h o1 k ) ((pos "H0$" "M6$" ) (subcat )))
( "hocked"  v ( h o1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "hockey"  n ( h o1 k ii ) ((pos "L@%" ) ))
( "hocking"  v ( h o1 k i ng ) ((pos "Hb$" ) (subcat )))
( "hocks"  v ( h o1 k s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "hocus-pocus"  n ( h ou2 k @ s - p ou1 k @ s ) ((pos "L@%" ) ))
( "hod"  n ( h o1 d ) ((pos "K6%" ) ))
( "hodgepodge"  n ( h o1 jh p o jh ) ((pos "L@%" ) ))
( "hods"  n ( h o1 d z ) ((pos "Kj%" ) ))
( "hoe"  v ( h ou1 ) ((pos "J5%" "K6%" ) (subcat "2A" "6A" )))
( "hoed"  v ( h ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "hoeing"  v ( h ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "hoes"  v ( h ou1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "hog"  v ( h o1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "hogged"  v ( h o1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hogging"  v ( h o1 g i ng ) ((pos "Hb%" ) (subcat )))
( "hoggish"  j ( h o1 g i sh ) ((pos "OA$" ) ))
( "hogs"  v ( h o1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hogshead"  n ( h o1 g z h e d ) ((pos "K6%" ) ))
( "hogsheads"  n ( h o1 g z h e d z ) ((pos "Kj%" ) ))
( "hogwash"  n ( h o1 g w o sh ) ((pos "L@%" ) ))
( "hoi polloi"  n ( h oi2 - p @ l oi1 ) ((pos "Kj$" ) ))
( "hoist"  v ( h oi1 s t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "hoisted"  v ( h oi1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hoisting"  v ( h oi1 s t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hoists"  v ( h oi1 s t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "hoity-toity"  j ( h oi2 t i - t oi1 t ii ) ((pos "OA%" "W-$" ) ))
( "hold"  v ( h ou1 l d ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "9" "14" "15A" "15B" "22" )))
( "hold-up"  n ( h ou1 l d - uh p ) ((pos "K6%" ) ))
( "hold-ups"  n ( h ou1 l d - uh p s ) ((pos "Kj$" ) ))
( "holdall"  n ( h ou1 l d oo l ) ((pos "K6%" ) ))
( "holdalls"  n ( h ou1 l d oo l z ) ((pos "Kj%" ) ))
( "holder"  n ( h ou1 l d @ r ) ((pos "K6%" ) ))
( "holders"  n ( h ou1 l d @ z ) ((pos "Kj%" ) ))
( "holding"  v ( h ou1 l d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "9" "14" "15A" "15B" "22" )))
( "holdings"  n ( h ou1 l d i ng z ) ((pos "Kj%" ) ))
( "holds"  v ( h ou1 l d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "9" "14" "15A" "15B" "22" )))
( "holdup"  n ( h ou1 l d uh p ) ((pos "K6$" ) ))
( "holdups"  n ( h ou1 l d uh p s ) ((pos "Kj$" ) ))
( "hole"  v ( h ou1 l ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" )))
( "hole-and-corner"  j ( h ou2 l - n - k oo1 n @ r ) ((pos "OA%" ) ))
( "holed"  v ( h ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "holes"  v ( h ou1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "holiday"  v ( h o1 l @ d ei ) ((pos "I0%" "K6%" ) ))
( "holiday-maker"  n ( h o1 l @ d i - m ei2 k @ r ) ((pos "K6%" ) ))
( "holiday-makers"  n ( h o1 l @ d i - m ei2 k @ z ) ((pos "Kj%" ) ))
( "holidayed"  v ( h o1 l @ d ei d ) ((pos "Ic%" ) ))
( "holidaying"  v ( h o1 l @ d ei i ng ) ((pos "Ib%" ) ))
( "holidays"  n ( h o1 l @ d ei z ) ((pos "Kj%" ) ))
( "holier"  j ( h ou1 l i@ r ) ((pos "Or%" ) ))
( "holies"  n ( h ou1 l i z ) ((pos "Kj%" ) ))
( "holiest"  j ( h ou1 l i i s t ) ((pos "Os%" ) ))
( "holiness"  n ( h ou1 l i n @ s ) ((pos "L@%" ) ))
( "holing"  v ( h ou1 l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "holler"  v ( h o1 l @ r ) ((pos "J0%" ) ))
( "hollered"  v ( h o1 l @ d ) ((pos "Jc%" "Jd%" ) ))
( "hollering"  v ( h o1 l @ r i ng ) ((pos "Jb%" ) ))
( "hollers"  v ( h o1 l @ z ) ((pos "Ja%" ) ))
( "holloa"  n ( h o1 l ou ) ((pos "K6$" "W-$" ) ))
( "holloas"  n ( h o1 l ou z ) ((pos "Kj$" ) ))
( "hollow"  v ( h o1 l ou ) ((pos "H0%" "K6%" "OA%" ) (subcat "6A" "15A" )))
( "hollowed"  v ( h o1 l ou d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "hollowing"  v ( h o1 l ou i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "hollows"  v ( h o1 l ou z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "holly"  n ( h o1 l ii ) ((pos "L@%" ) ))
( "hollyhock"  n ( h o1 l i h o k ) ((pos "K6%" ) ))
( "hollyhocks"  n ( h o1 l i h o k s ) ((pos "Kj%" ) ))
( "holm-oak"  n ( h ou1 m - ou k ) ((pos "K6$" ) ))
( "holm-oaks"  n ( h ou1 m - ou k s ) ((pos "Kj$" ) ))
( "holocaust"  n ( h o1 l @ k oo s t ) ((pos "K6%" ) ))
( "holocausts"  n ( h o1 l @ k oo s t s ) ((pos "Kj$" ) ))
( "holograph"  n ( h o1 l @ g r aa f ) ((pos "K6%" ) ))
( "holographs"  n ( h o1 l @ g r aa f s ) ((pos "Kj%" ) ))
( "holster"  n ( h ou1 l s t @ r ) ((pos "K6%" ) ))
( "holsters"  n ( h ou1 l s t @ z ) ((pos "Kj%" ) ))
( "holy"  n ( h ou1 l ii ) ((pos "K8%" "OD%" ) ))
( "holystone"  v ( h ou1 l i s t ou n ) ((pos "H2$" "L@$" ) ))
( "holystoned"  v ( h ou1 l i s t ou n d ) ((pos "Hc$" "Hd$" ) ))
( "holystones"  v ( h ou1 l i s t ou n z ) ((pos "Ha$" ) ))
( "holystoning"  v ( h ou1 l i s t ou n i ng ) ((pos "Hb$" ) ))
( "homage"  n ( h o1 m i jh ) ((pos "L@%" ) ))
( "home"  n ( h ou1 m ) ((pos "K6*" "Pu*" ) ))
( "home-baked"  j ( h ou1 m - b ei1 k t ) ((pos "OA%" ) ))
( "home-brewed"  j ( h ou1 m - b r uu1 d ) ((pos "OA%" ) ))
( "home-cured"  j ( h ou1 m - k y uu1 @ d ) ((pos "OA%" ) ))
( "home-farm"  n ( h ou1 m - f aa1 m ) ((pos "K6%" ) ))
( "home-farms"  n ( h ou1 m - f aa1 m z ) ((pos "Kj%" ) ))
( "home-grown"  j ( h ou1 m - g r ou n ) ((pos "OA%" ) ))
( "home-made"  j ( h ou1 m - m ei d ) ((pos "OA%" ) ))
( "homecoming"  n ( h ou1 m k uh m i ng ) ((pos "K6%" ) ))
( "homecomings"  n ( h ou1 m k uh m i ng z ) ((pos "Kj%" ) ))
( "homeland"  n ( h ou1 m l a n d ) ((pos "K6%" ) ))
( "homelands"  n ( h ou1 m l a n d z ) ((pos "Kj%" ) ))
( "homeless"  j ( h ou1 m l @ s ) ((pos "OA%" ) ))
( "homelier"  j ( h ou1 m l i@ r ) ((pos "Or%" ) ))
( "homeliest"  j ( h ou1 m l i i s t ) ((pos "Os%" ) ))
( "homelike"  j ( h ou1 m l ai k ) ((pos "OA%" ) ))
( "homeliness"  n ( h ou1 m l i n @ s ) ((pos "L@%" ) ))
( "homely"  j ( h ou1 m l ii ) ((pos "OD%" ) ))
( "homeopath"  n ( h ou1 m i@ p a th ) ((pos "K6%" ) ))
( "homeopathic"  j ( h ou2 m i@ p a1 th i k ) ((pos "OA%" ) ))
( "homeopaths"  n ( h ou1 m ii ou p a th s ) ((pos "Kj%" ) ))
( "homes"  n ( h ou1 m z ) ((pos "Kj%" ) ))
( "homesick"  j ( h ou1 m s i k ) ((pos "OA%" ) ))
( "homesickness"  n ( h ou1 m s i k n @ s ) ((pos "L@%" ) ))
( "homespun"  n ( h ou1 m s p uh n ) ((pos "L@$" "OA%" ) ))
( "homestead"  n ( h ou1 m s t e d ) ((pos "K6%" ) ))
( "homesteads"  n ( h ou1 m s t e d z ) ((pos "Kj%" ) ))
( "homeward"  j ( h ou1 m w @ d ) ((pos "OA%" "Pu%" ) ))
( "homewards"  a ( h ou1 m w @ d z ) ((pos "Pu%" ) ))
( "homework"  n ( h ou1 m w @@ k ) ((pos "L@%" ) ))
( "homey"  j ( h ou1 m ii ) ((pos "OA$" ) ))
( "homicidal"  j ( h o2 m i s ai1 d l ) ((pos "OA%" ) ))
( "homicide"  n ( h o1 m i s ai d ) ((pos "M6%" ) ))
( "homicides"  n ( h o1 m i s ai d z ) ((pos "Mj%" ) ))
( "homiletic"  j ( h o2 m i l e1 t i k ) ((pos "OA$" ) ))
( "homiletics"  n ( h o2 m i l e1 t i k s ) ((pos "Lk$" ) ))
( "homilies"  n ( h o1 m i l i z ) ((pos "Kj%" ) ))
( "homily"  n ( h o1 m i l ii ) ((pos "K8%" ) ))
( "homing"  j ( h ou1 m i ng ) ((pos "OA%" ) ))
( "hominy"  n ( h o1 m i n ii ) ((pos "L@$" ) ))
( "homo"  n ( h ou1 m ou ) ((pos "K6$" ) ))
( "homo sapiens"  n ( h ou2 m ou - s a1 p i e n z ) ((pos "Ki%" ) ))
( "homoeopath"  n ( h ou1 m ii ou p a th ) ((pos "K6%" ) ))
( "homoeopaths"  n ( h ou1 m ii ou p a th s ) ((pos "Kj%" ) ))
( "homoeopathy"  n ( h ou2 m i o1 p @ th ii ) ((pos "L@%" ) ))
( "homogeneity"  n ( h ou2 m @ jh i n i@1 t ii ) ((pos "L@%" ) ))
( "homogeneous"  j ( h o2 m @ jh ii1 n i@ s ) ((pos "OA%" ) ))
( "homogenize"  v ( h @1 m o1 jh i n ai z ) ((pos "H2%" ) (subcat )))
( "homogenized"  v ( h @1 m o1 jh i n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "homogenizes"  v ( h @1 m o1 jh i n ai z i z ) ((pos "Ha$" ) (subcat )))
( "homogenizing"  v ( h @1 m o1 jh i n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "homograph"  n ( h o1 m @ g r aa f ) ((pos "K6$" ) ))
( "homographs"  n ( h o1 m @ g r aa f s ) ((pos "Kj$" ) ))
( "homonym"  n ( h o1 m @ n i m ) ((pos "K6%" ) ))
( "homonyms"  n ( h o1 m @ n i m z ) ((pos "Kj%" ) ))
( "homophone"  n ( h o1 m @ f ou n ) ((pos "K6%" ) ))
( "homophones"  n ( h o1 m @ f ou n z ) ((pos "Kj%" ) ))
( "homos"  n ( h ou1 m ou z ) ((pos "Kj$" ) ))
( "homosexual"  n ( h o2 m @ s e1 k sh u@ l ) ((pos "K6%" "OA%" ) ))
( "homosexuality"  n ( h ou2 m @ s e k sh u a1 l i t ii ) ((pos "L@%" ) ))
( "homosexuals"  n ( h o2 m @ s e1 k sh u@ l z ) ((pos "Kj%" ) ))
( "hone"  v ( h ou1 n ) ((pos "H2%" "K6$" ) (subcat )))
( "honed"  v ( h ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hones"  v ( h ou1 n z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "honest"  j ( o1 n i s t ) ((pos "OA%" ) ))
( "honestly"  a ( o1 n i s t l ii ) ((pos "Pu%" ) ))
( "honesty"  n ( o1 n i s t ii ) ((pos "L@%" ) ))
( "honey"  n ( h uh1 n ii ) ((pos "M6%" ) ))
( "honeybee"  n ( h uh1 n i b ii ) ((pos "K6%" ) ))
( "honeybees"  n ( h uh1 n i b ii z ) ((pos "Kj%" ) ))
( "honeycomb"  v ( h uh1 n i k ou m ) ((pos "H0$" "M6%" ) (subcat )))
( "honeycombed"  v ( h uh1 n i k ou m d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "honeycombing"  v ( h uh1 n i k ou m i ng ) ((pos "Hb$" ) (subcat )))
( "honeycombs"  v ( h uh1 n i k ou m z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "honeydew"  n ( h uh1 n i d y uu ) ((pos "L@%" ) ))
( "honeyed"  j ( h uh1 n i d ) ((pos "OA%" ) ))
( "honeymoon"  v ( h uh1 n i m uu n ) ((pos "I0%" "K6%" ) ))
( "honeymooned"  v ( h uh1 n i m uu n d ) ((pos "Ic%" "Id%" ) ))
( "honeymooning"  v ( h uh1 n i m uu n i ng ) ((pos "Ib%" ) ))
( "honeymoons"  v ( h uh1 n i m uu n z ) ((pos "Ia$" "Kj%" ) ))
( "honeys"  n ( h uh1 n i z ) ((pos "Mj%" ) ))
( "honeysuckle"  n ( h uh1 n i s uh k l ) ((pos "L@%" ) ))
( "honing"  v ( h ou1 n i ng ) ((pos "Hb%" ) (subcat )))
( "honk"  v ( h o1 ng k ) ((pos "I0%" "K6%" ) ))
( "honked"  v ( h o1 ng k t ) ((pos "Ic%" "Id%" ) ))
( "honking"  v ( h o1 ng k i ng ) ((pos "Ib%" ) ))
( "honks"  v ( h o1 ng k s ) ((pos "Ia%" "Kj%" ) ))
( "honorarium"  n ( o2 n @ r e@1 r i@ m ) ((pos "K6%" ) ))
( "honorariums"  n ( o2 n @ r e@1 r i@ m z ) ((pos "Kj$" ) ))
( "honorary"  j ( o1 n @ r @ r ii ) ((pos "OA%" ) ))
( "honorific"  n ( o2 n @ r i1 f i k ) ((pos "K6$" "OA%" ) ))
( "honorifics"  n ( o2 n @ r i1 f i k s ) ((pos "Kj$" ) ))
( "honour"  v ( o1 n @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "honourable"  j ( o1 n @ r @ b l ) ((pos "OA%" ) ))
( "honourably"  a ( o1 n @ r @ b l ii ) ((pos "Pu%" ) ))
( "honoured"  v ( o1 n @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "honouring"  v ( o1 n @ r i ng ) ((pos "Hb%" ) (subcat )))
( "honours"  v ( o1 n @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "hooch"  n ( h uu1 ch ) ((pos "L@%" ) ))
( "hood"  v ( h u1 d ) ((pos "H0%" "K6%" ) ))
( "hooded"  v ( h u1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "hooding"  v ( h u1 d i ng ) ((pos "Hb%" ) ))
( "hoodlum"  n ( h uu1 d l @ m ) ((pos "K6%" ) ))
( "hoodlums"  n ( h uu1 d l @ m z ) ((pos "Kj%" ) ))
( "hoodoo"  v ( h uu1 d uu ) ((pos "H0$" "K6%" ) ))
( "hoodooed"  v ( h uu1 d uu d ) ((pos "Hc$" "Hd$" ) ))
( "hoodooing"  v ( h uu1 d uu i ng ) ((pos "Hb$" ) ))
( "hoodoos"  v ( h uu1 d uu z ) ((pos "Ha$" "Kj$" ) ))
( "hoods"  v ( h u1 d z ) ((pos "Ha%" "Kj%" ) ))
( "hoodwink"  v ( h u1 d w i ng k ) ((pos "H0%" ) (subcat "6A" )))
( "hoodwinked"  v ( h u1 d w i ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hoodwinking"  v ( h u1 d w i ng k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hoodwinks"  v ( h u1 d w i ng k s ) ((pos "Ha%" ) (subcat "6A" )))
( "hooey"  n ( h uu1 ii ) ((pos "L@$" ) ))
( "hoof"  n ( h uu1 f ) ((pos "K6%" ) ))
( "hoofs"  n ( h uu1 f s ) ((pos "Kj%" ) ))
( "hook"  v ( h u1 k ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "hook-nosed"  j ( h u1 k - n ou z d ) ((pos "OA%" ) ))
( "hook-up"  n ( h u1 k - uh p ) ((pos "K6$" ) ))
( "hook-ups"  n ( h u1 k - uh p s ) ((pos "Kj$" ) ))
( "hookah"  n ( h u1 k @ ) ((pos "K6$" ) ))
( "hookahs"  n ( h u1 k @ z ) ((pos "Kj$" ) ))
( "hooked"  v ( h u1 k t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "6A" "15A" )))
( "hooker"  n ( h u1 k @ r ) ((pos "K6%" ) ))
( "hookers"  n ( h u1 k @ z ) ((pos "Kj%" ) ))
( "hooking"  v ( h u1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "hooks"  v ( h u1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "hookworm"  n ( h u1 k w @@ m ) ((pos "M6%" ) ))
( "hookworms"  n ( h u1 k w @@ m z ) ((pos "Mj%" ) ))
( "hooky"  n ( h u1 k ii ) ((pos "Ki%" ) ))
( "hooligan"  n ( h uu1 l i g @ n ) ((pos "K6%" ) ))
( "hooliganism"  n ( h uu1 l i g @ n i z @ m ) ((pos "L@%" ) ))
( "hooligans"  n ( h uu1 l i g @ n z ) ((pos "Kj%" ) ))
( "hoop"  v ( h uu1 p ) ((pos "H0$" "K6%" ) ))
( "hoop-la"  n ( h uu1 p - l aa ) ((pos "L@%" ) ))
( "hooped"  v ( h uu1 p t ) ((pos "Hc$" "Hd$" ) ))
( "hooping"  v ( h uu1 p i ng ) ((pos "Hb$" ) ))
( "hoops"  v ( h uu1 p s ) ((pos "Ha$" "Kj%" ) ))
( "hooray"  uh ( h uu1 r ei1 ) ((pos "W-%" ) ))
( "hoot"  v ( h uu1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hooted"  v ( h uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hooter"  n ( h uu1 t @ r ) ((pos "K6%" ) ))
( "hooters"  n ( h uu1 t @ z ) ((pos "Kj%" ) ))
( "hooting"  v ( h uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hoots"  v ( h uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hooves"  n ( h uu1 v z ) ((pos "Kj%" ) ))
( "hop"  v ( h o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "hop-field"  n ( h o1 p - f ii l d ) ((pos "K6%" ) ))
( "hop-fields"  n ( h o1 p - f ii l d z ) ((pos "Kj%" ) ))
( "hop-garden"  n ( h o1 p - g aa d n ) ((pos "K6%" ) ))
( "hop-gardens"  n ( h o1 p - g aa d n z ) ((pos "Kj%" ) ))
( "hop-picker"  n ( h o1 p - p i k @ r ) ((pos "K6%" ) ))
( "hop-pickers"  n ( h o1 p - p i k @ z ) ((pos "Kj%" ) ))
( "hop-pole"  n ( h o1 p - p ou l ) ((pos "K6%" ) ))
( "hop-poles"  n ( h o1 p - p ou l z ) ((pos "Kj%" ) ))
( "hope"  v ( h ou1 p ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" "7A" )))
( "hoped"  v ( h ou1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "7A" )))
( "hopeful"  j ( h ou1 p f @ l ) ((pos "OA%" ) ))
( "hopefully"  a ( h ou1 p f @ l ii ) ((pos "Pu%" ) ))
( "hopefulness"  n ( h ou1 p f l n @ s ) ((pos "L@%" ) ))
( "hopeless"  j ( h ou1 p l i s ) ((pos "OA%" ) ))
( "hopelessly"  a ( h ou1 p l i s l ii ) ((pos "Pu%" ) ))
( "hopelessness"  n ( h ou1 p l i s n @ s ) ((pos "L@%" ) ))
( "hopes"  v ( h ou1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "7A" )))
( "hoping"  v ( h ou1 p i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "7A" )))
( "hopped"  v ( h o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "hopped-up"  j ( h o1 p t - uh1 p ) ((pos "OA$" ) ))
( "hopper"  n ( h o1 p @ r ) ((pos "K6%" ) ))
( "hoppers"  n ( h o1 p @ z ) ((pos "Kj%" ) ))
( "hopping"  v ( h o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "hops"  v ( h o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "hopscotch"  n ( h o1 p s k o ch ) ((pos "L@%" ) ))
( "horde"  n ( h oo1 d ) ((pos "K6%" ) ))
( "hordes"  n ( h oo1 d z ) ((pos "Kj%" ) ))
( "horizon"  n ( h @1 r ai1 z n ) ((pos "K6%" ) ))
( "horizons"  n ( h @1 r ai1 z n z ) ((pos "Kj%" ) ))
( "horizontal"  j ( h o2 r i z o1 n t l ) ((pos "OA%" ) ))
( "horizontally"  a ( h o2 r i z o1 n t @ l ii ) ((pos "Pu%" ) ))
( "hormone"  n ( h oo1 m ou n ) ((pos "K6%" ) ))
( "hormones"  n ( h oo1 m ou n z ) ((pos "Kj%" ) ))
( "horn"  v ( h oo1 n ) ((pos "I0%" "M6%" ) ))
( "horn-rimmed"  j ( h oo1 n - r i m d ) ((pos "OA%" ) ))
( "hornbeam"  n ( h oo1 n b ii m ) ((pos "K6%" ) ))
( "hornbeams"  n ( h oo1 n b ii m z ) ((pos "Kj%" ) ))
( "hornbill"  n ( h oo1 n b i l ) ((pos "K6%" ) ))
( "hornbills"  n ( h oo1 n b i l z ) ((pos "Kj%" ) ))
( "horned"  v ( h oo1 n d ) ((pos "Ic%" "Id%" "OA%" ) ))
( "hornet"  n ( h oo1 n i t ) ((pos "K6%" ) ))
( "hornets"  n ( h oo1 n i t s ) ((pos "Kj%" ) ))
( "hornier"  j ( h oo1 n i@ r ) ((pos "Or$" ) ))
( "horniest"  j ( h oo1 n i i s t ) ((pos "Os$" ) ))
( "horning"  v ( h oo1 n i ng ) ((pos "Ib$" ) ))
( "hornless"  j ( h oo1 n l @ s ) ((pos "OA%" ) ))
( "hornlike"  j ( h oo1 n l ai k ) ((pos "OA$" ) ))
( "hornpipe"  n ( h oo1 n p ai p ) ((pos "K6%" ) ))
( "hornpipes"  n ( h oo1 n p ai p s ) ((pos "Kj%" ) ))
( "horns"  v ( h oo1 n z ) ((pos "Ia$" "Mj%" ) ))
( "horny"  j ( h oo1 n ii ) ((pos "OD%" ) ))
( "horology"  n ( h o1 r o1 l @ jh ii ) ((pos "L@$" ) ))
( "horoscope"  n ( h o1 r @ s k ou p ) ((pos "K6%" ) ))
( "horoscopes"  n ( h o1 r @ s k ou p s ) ((pos "Kj%" ) ))
( "horrible"  j ( h o1 r @ b l ) ((pos "OA%" ) ))
( "horribly"  a ( h o1 r @ b l ii ) ((pos "Pu%" ) ))
( "horrid"  j ( h o1 r i d ) ((pos "OA%" ) ))
( "horridly"  a ( h o1 r i d l ii ) ((pos "Pu%" ) ))
( "horridness"  n ( h o1 r i d n @ s ) ((pos "L@%" ) ))
( "horrific"  j ( h @1 r i1 f i k ) ((pos "OA%" ) ))
( "horrified"  v ( h o1 r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "horrifies"  v ( h o1 r i f ai z ) ((pos "Ha%" ) (subcat )))
( "horrify"  v ( h o1 r i f ai ) ((pos "H3%" ) (subcat )))
( "horrifying"  v ( h o1 r i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "horrifyingly"  a ( h o1 r i f ai i ng l ii ) ((pos "Pu%" ) ))
( "horror"  n ( h o1 r @ r ) ((pos "M6%" ) ))
( "horror-stricken"  j ( h o1 r @ - s t r i k @ n ) ((pos "OA%" ) ))
( "horror-struck"  j ( h o1 r @ - s t r uh k ) ((pos "OA%" ) ))
( "horrors"  n ( h o1 r @ z ) ((pos "Mj%" ) ))
( "hors d'oeuvres"  n ( oo2 - d @@1 v ) ((pos "Kj%" ) ))
( "hors de combat"  j ( oo2 - d @ - k o1 m b aa ) ((pos "Op%" ) ))
( "horse"  n ( h oo1 s ) ((pos "K6%" ) ))
( "horse-chestnut"  n ( h oo1 s - ch e1 s n uh t ) ((pos "K6%" ) ))
( "horse-chestnuts"  n ( h oo1 s - ch e1 s n uh t s ) ((pos "Kj%" ) ))
( "horse-laugh"  n ( h oo1 s - l aa f ) ((pos "K6$" ) ))
( "horse-laughs"  n ( h oo1 s - l aa f s ) ((pos "Kj$" ) ))
( "horse-pond"  n ( h oo1 s - p o n d ) ((pos "K6$" ) ))
( "horse-ponds"  n ( h oo1 s - p o n d z ) ((pos "Kj$" ) ))
( "horse-sense"  n ( h oo1 s - s e n s ) ((pos "L@%" ) ))
( "horseback"  n ( h oo1 s b a k ) ((pos "L@%" ) ))
( "horsebox"  n ( h oo1 s b o k s ) ((pos "K7%" ) ))
( "horseboxes"  n ( h oo1 s b o k s i z ) ((pos "Kj%" ) ))
( "horseflesh"  n ( h oo1 s f l e sh ) ((pos "L@%" ) ))
( "horseflies"  n ( h oo1 s f l ai z ) ((pos "Kj%" ) ))
( "horsefly"  n ( h oo1 s f l ai ) ((pos "K8%" ) ))
( "horsehair"  n ( h oo1 s h e@ r ) ((pos "L@%" ) ))
( "horseman"  n ( h oo1 s m @ n ) ((pos "Ki%" ) ))
( "horsemanship"  n ( h oo1 s m @ n sh i p ) ((pos "L@%" ) ))
( "horsemeat"  n ( h oo1 s m ii t ) ((pos "L@%" ) ))
( "horsemen"  n ( h oo1 s m @ n ) ((pos "Kj%" ) ))
( "horseplay"  n ( h oo1 s p l ei ) ((pos "L@%" ) ))
( "horsepower"  n ( h oo1 s p au @ r ) ((pos "M9%" ) ))
( "horserace"  n ( h oo1 s r ei s ) ((pos "K6%" ) ))
( "horseraces"  n ( h oo1 s r ei s i z ) ((pos "Kj%" ) ))
( "horseracing"  n ( h oo1 s r ei s i ng ) ((pos "L@%" ) ))
( "horseradish"  n ( h oo1 s r a d i sh ) ((pos "L@%" ) ))
( "horses"  n ( h oo1 s i z ) ((pos "Kj%" ) ))
( "horseshoe"  n ( h oo1 s - sh uu ) ((pos "K6%" ) ))
( "horseshoes"  n ( h oo1 s - sh uu z ) ((pos "Kj%" ) ))
( "horsewhip"  v ( h oo1 s w i p ) ((pos "H4%" "K6%" ) ))
( "horsewhipped"  v ( h oo1 s w i p t ) ((pos "Hc%" "Hd%" ) ))
( "horsewhipping"  v ( h oo1 s w i p i ng ) ((pos "Hb$" ) ))
( "horsewhips"  v ( h oo1 s w i p s ) ((pos "Ha$" "Kj$" ) ))
( "horsewoman"  n ( h oo1 s w u m @ n ) ((pos "Ki%" ) ))
( "horsewomen"  n ( h oo1 s w i m i n ) ((pos "Kj%" ) ))
( "horsy"  j ( h oo1 s ii ) ((pos "OA%" ) ))
( "hortative"  j ( h oo1 t @ t i v ) ((pos "OA$" ) ))
( "horticultural"  j ( h oo2 t i k uh1 l ch @ r @ l ) ((pos "OA%" ) ))
( "horticulture"  n ( h oo1 t i k uh l ch @ r ) ((pos "L@%" ) ))
( "horticulturist"  n ( h oo2 t i k uh1 l ch @ r i s t ) ((pos "K6$" ) ))
( "horticulturists"  n ( h oo2 t i k uh1 l ch @ r i s t s ) ((pos "Kj$" ) ))
( "hosanna"  n ( h ou1 z a1 n @ ) ((pos "K6$" "W-$" ) ))
( "hosannas"  n ( h ou1 z a1 n @ z ) ((pos "Kj$" ) ))
( "hose"  v ( h ou1 z ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "hosed"  v ( h ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hosepipe"  n ( h ou1 z p ai p ) ((pos "K6%" ) ))
( "hosepipes"  n ( h ou1 z p ai p s ) ((pos "Kj%" ) ))
( "hoses"  v ( h ou1 z i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "hosier"  n ( h ou1 z i@ r ) ((pos "K6$" ) ))
( "hosiers"  n ( h ou1 z i@ z ) ((pos "Kj$" ) ))
( "hosiery"  n ( h ou1 z i@ r ii ) ((pos "L@%" ) ))
( "hosing"  v ( h ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hospice"  n ( h o1 s p i s ) ((pos "K6%" ) ))
( "hospices"  n ( h o1 s p i s i z ) ((pos "Kj%" ) ))
( "hospitable"  j ( h o1 s p i1 t @ b l ) ((pos "OA%" ) ))
( "hospitably"  a ( h o1 s p i1 t @ b l ii ) ((pos "Pu%" ) ))
( "hospital"  n ( h o1 s p i t l ) ((pos "K6%" ) ))
( "hospitality"  n ( h o2 s p i t a1 l i t ii ) ((pos "L@%" ) ))
( "hospitalization"  n ( h o2 s p i t @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "hospitalizations"  n ( h o2 s p i t @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "hospitalize"  v ( h o1 s p i t @ l ai z ) ((pos "H2$" ) ))
( "hospitalized"  v ( h o1 s p i t @ l ai z d ) ((pos "Hc$" "Hd%" ) ))
( "hospitalizes"  v ( h o1 s p i t @ l ai z i z ) ((pos "Ha$" ) ))
( "hospitalizing"  v ( h o1 s p i t @ l ai z i ng ) ((pos "Hb$" ) ))
( "hospitals"  n ( h o1 s p i t l z ) ((pos "Kj%" ) ))
( "host"  v ( h ou1 s t ) ((pos "H0%" "K6%" ) (subcat )))
( "hostage"  n ( h o1 s t i jh ) ((pos "K6%" ) ))
( "hostages"  n ( h o1 s t i jh i z ) ((pos "Kj%" ) ))
( "hosted"  v ( h ou1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hostel"  n ( h o1 s t l ) ((pos "K6%" ) ))
( "hosteller"  n ( h o1 s t @ l @ r ) ((pos "K6%" ) ))
( "hostellers"  n ( h o1 s t @ l @ z ) ((pos "Kj%" ) ))
( "hostelries"  n ( h o1 s t l r i z ) ((pos "Kj%" ) ))
( "hostelry"  n ( h o1 s t l r ii ) ((pos "K8%" ) ))
( "hostels"  n ( h o1 s t l z ) ((pos "Kj%" ) ))
( "hostess"  n ( h ou1 s t i s ) ((pos "K7%" ) ))
( "hostesses"  n ( h ou1 s t i s i z ) ((pos "Kj%" ) ))
( "hostile"  j ( h o1 s t ai l ) ((pos "OA%" ) ))
( "hostilely"  a ( h o1 s t ai l ii ) ((pos "Pu$" ) ))
( "hostilities"  n ( h o1 s t i1 l i t i z ) ((pos "Mj%" ) ))
( "hostility"  n ( h o1 s t i1 l i t ii ) ((pos "M8%" ) ))
( "hosting"  v ( h ou1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "hosts"  v ( h ou1 s t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hot"  v ( h o1 t ) ((pos "J4$" "OE%" ) (subcat "2C" )))
( "hot-blooded"  j ( h o1 t - b l uh1 d i d ) ((pos "OA%" ) ))
( "hot-headed"  j ( h o1 t - h e1 d i d ) ((pos "OA%" ) ))
( "hot-tempered"  j ( h o1 t - t e1 m p @ d ) ((pos "OA%" ) ))
( "hot-water-bottle"  n ( h o1 t - w oo1 t @ - b o t l ) ((pos "K6%" ) ))
( "hot-water-bottles"  n ( h o1 t - w oo1 t @ - b o t l z ) ((pos "Kj%" ) ))
( "hotbed"  n ( h o1 t b e d ) ((pos "K6%" ) ))
( "hotbeds"  n ( h o1 t b e d z ) ((pos "Kj%" ) ))
( "hotchpotch"  n ( h o1 ch p o ch ) ((pos "K7%" ) ))
( "hotchpotches"  n ( h o1 ch p o ch i z ) ((pos "Kj$" ) ))
( "hotel"  n ( h ou1 t e1 l ) ((pos "K6%" ) ))
( "hotelier"  n ( h ou1 t e1 l i@ r ) ((pos "K6%" ) ))
( "hoteliers"  n ( h ou1 t e1 l i@ z ) ((pos "Kj%" ) ))
( "hotels"  n ( h ou1 t e1 l z ) ((pos "Kj%" ) ))
( "hotfoot"  v ( h o1 t f u t ) ((pos "I0%" "Pu%" ) ))
( "hotfooted"  v ( h o1 t f u t i d ) ((pos "Ic%" "Id%" ) ))
( "hotfooting"  v ( h o1 t f u t i ng ) ((pos "Ib$" ) ))
( "hotfoots"  v ( h o1 t f u t s ) ((pos "Ia$" ) ))
( "hothead"  n ( h o1 t h e d ) ((pos "K6%" ) ))
( "hotheads"  n ( h o1 t h e d z ) ((pos "Kj%" ) ))
( "hothouse"  n ( h o1 t h au s ) ((pos "K6%" ) ))
( "hothouses"  n ( h o1 t h au z i z ) ((pos "Kj%" ) ))
( "hotly"  a ( h o1 t l ii ) ((pos "Pu%" ) ))
( "hotplate"  n ( h o1 t p l ei t ) ((pos "K6%" ) ))
( "hotplates"  n ( h o1 t p l ei t s ) ((pos "Kj%" ) ))
( "hots"  v ( h o1 t s ) ((pos "Ja$" ) (subcat "2C" )))
( "hotted"  v ( h o1 t i d ) ((pos "Jc$" "Jd$" ) (subcat "2C" )))
( "hotter"  j ( h o1 t @ r ) ((pos "Or%" ) ))
( "hottest"  j ( h o1 t i s t ) ((pos "Os%" ) ))
( "hotting"  v ( h o1 t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "hound"  v ( h au1 n d ) ((pos "H0%" "K6%" ) (subcat )))
( "hounded"  v ( h au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hounding"  v ( h au1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "hounds"  v ( h au1 n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hour"  n ( au1 @ r ) ((pos "K6%" ) ))
( "hourglass"  n ( au1 @ g l aa s ) ((pos "K7%" ) ))
( "hourglasses"  n ( au1 @ g l aa s i z ) ((pos "Kj%" ) ))
( "houri"  n ( h u@1 r ii ) ((pos "K6$" ) ))
( "houris"  n ( h u@1 r i z ) ((pos "Kj$" ) ))
( "hourly"  j ( au1 @ l ii ) ((pos "OA%" "Pu%" ) ))
( "hours"  n ( au1 @ z ) ((pos "Kj%" ) ))
( "house"  n ( h au1 s ) ((pos "K6*" ) ))
( "house"  v ( h au1 z ) ((pos "H2%" ) (subcat )))
( "house-parties"  n ( h au1 s - p aa t i z ) ((pos "Kj%" ) ))
( "house-party"  n ( h au1 s - p aa t ii ) ((pos "K8%" ) ))
( "house-warming"  n ( h au1 s - w oo m i ng ) ((pos "K6%" ) ))
( "house-warmings"  n ( h au1 s - w oo m i ng z ) ((pos "Kj%" ) ))
( "houseboat"  n ( h au1 s b ou t ) ((pos "K6%" ) ))
( "houseboats"  n ( h au1 s b ou t s ) ((pos "Kj%" ) ))
( "housebound"  j ( h au1 s b au n d ) ((pos "OA%" ) ))
( "housebreaker"  n ( h au1 s b r ei k @ r ) ((pos "K6$" ) ))
( "housebreakers"  n ( h au1 s b r ei k @ z ) ((pos "Kj$" ) ))
( "housecoat"  n ( h au1 s k ou t ) ((pos "K6%" ) ))
( "housecoats"  n ( h au1 s k ou t s ) ((pos "Kj%" ) ))
( "housecraft"  n ( h au1 s k r aa f t ) ((pos "L@%" ) ))
( "housed"  v ( h au1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "housedog"  n ( h au1 s d o g ) ((pos "K6%" ) ))
( "housedogs"  n ( h au1 s d o g z ) ((pos "Kj%" ) ))
( "housefather"  n ( h au1 s f aa dh @ r ) ((pos "K6%" ) ))
( "housefathers"  n ( h au1 s f aa dh @ z ) ((pos "Kj%" ) ))
( "houseflies"  n ( h au1 s f l ai z ) ((pos "Kj%" ) ))
( "housefly"  n ( h au1 s f l ai ) ((pos "K8%" ) ))
( "houseful"  n ( h au1 s f u l ) ((pos "K6%" ) ))
( "housefuls"  n ( h au1 s f u l z ) ((pos "Kj%" ) ))
( "household"  n ( h au1 s h ou l d ) ((pos "K6%" ) ))
( "householder"  n ( h au1 s h ou l d @ r ) ((pos "K6%" ) ))
( "householders"  n ( h au1 s h ou l d @ z ) ((pos "Kj%" ) ))
( "households"  n ( h au1 s h ou l d z ) ((pos "Kj%" ) ))
( "housekeeper"  n ( h au1 s k ii p @ r ) ((pos "K6%" ) ))
( "housekeepers"  n ( h au1 s k ii p @ z ) ((pos "Kj%" ) ))
( "houselights"  n ( h au1 s l ai t s ) ((pos "Kj%" ) ))
( "housemaid"  n ( h au1 s m ei d ) ((pos "K6%" ) ))
( "housemaids"  n ( h au1 s m ei d z ) ((pos "Kj%" ) ))
( "houseman"  n ( h au1 s m @ n ) ((pos "Ki%" ) ))
( "housemaster"  n ( h au1 s m aa s t @ r ) ((pos "K6%" ) ))
( "housemasters"  n ( h au1 s m aa s t @ z ) ((pos "Kj%" ) ))
( "housemen"  n ( h au1 s m @ n ) ((pos "Kj%" ) ))
( "housemother"  n ( h au1 s m uh dh @ r ) ((pos "K6%" ) ))
( "housemothers"  n ( h au1 s m uh dh @ z ) ((pos "Kj%" ) ))
( "houseproud"  j ( h au1 s p r au d ) ((pos "OA%" ) ))
( "houseroom"  n ( h au1 s r uu m ) ((pos "L@%" ) ))
( "houses"  v ( h au1 z i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "housetop"  n ( h au1 s t o p ) ((pos "K6%" ) ))
( "housetops"  n ( h au1 s t o p s ) ((pos "Kj%" ) ))
( "housetrained"  j ( h au1 s t r ei n d ) ((pos "OA%" ) ))
( "housewife"  n ( h au1 s w ai f ) ((pos "Ki%" ) ))
( "housewifely"  j ( h au1 s w ai f l ii ) ((pos "OA$" ) ))
( "housewifery"  n ( h au1 s w i f r ii ) ((pos "L@%" ) ))
( "housewives"  n ( h au1 s w ai v z ) ((pos "Kj%" ) ))
( "housework"  n ( h au1 s w @@ k ) ((pos "L@%" ) ))
( "housewrecker"  n ( h au1 s r e k @ r ) ((pos "K6$" ) ))
( "housewreckers"  n ( h au1 s r e k @ z ) ((pos "Kj$" ) ))
( "housing"  v ( h au1 z i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "hove"  v ( h ou1 v ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hovel"  n ( h o1 v @ l ) ((pos "K6%" ) ))
( "hovels"  n ( h o1 v @ l z ) ((pos "Kj%" ) ))
( "hover"  v ( h o1 v @ r ) ((pos "I0%" ) (subcat "2A" )))
( "hovercraft"  n ( h o1 v @ k r aa f t ) ((pos "K9%" ) ))
( "hovered"  v ( h o1 v @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "hovering"  v ( h o1 v @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "hovers"  v ( h o1 v @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "how"  a ( h au1 ) ((pos "Pu*" ) ))
( "how's"  v ( h au z ) ((pos "Gh%" ) ))
( "how-d'ye-do"  n ( h au2 - d y u - d uu1 ) ((pos "K6$" ) ))
( "how-d'ye-dos"  n ( h au2 - d y u - d uu1 z ) ((pos "Kj$" ) ))
( "howbeit"  cc ( h au1 b ii1 i t ) ((pos "V-$" ) ))
( "howdah"  n ( h au1 d @ ) ((pos "K6$" ) ))
( "howdahs"  n ( h au1 d @ z ) ((pos "Kj$" ) ))
( "however"  a ( h au1 e1 v @ r ) ((pos "Pu*" ) ))
( "howitzer"  n ( h au1 i t s @ r ) ((pos "K6%" ) ))
( "howitzers"  n ( h au1 i t s @ z ) ((pos "Kj%" ) ))
( "howl"  v ( h au1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "howled"  v ( h au1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "howler"  n ( h au1 l @ r ) ((pos "K6%" ) ))
( "howlers"  n ( h au1 l @ z ) ((pos "Kj%" ) ))
( "howling"  v ( h au1 l i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "howls"  v ( h au1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hoyden"  n ( h oi1 d n ) ((pos "K6$" ) ))
( "hoydenish"  j ( h oi1 d n i sh ) ((pos "OA$" ) ))
( "hoydens"  n ( h oi1 d n z ) ((pos "Kj$" ) ))
( "hub"  n ( h uh1 b ) ((pos "K6%" ) ))
( "hubbies"  n ( h uh1 b i z ) ((pos "Kj%" ) ))
( "hubble-bubble"  n ( h uh1 b l - b uh b l ) ((pos "K6$" ) ))
( "hubble-bubbles"  n ( h uh1 b l - b uh b l z ) ((pos "Kj$" ) ))
( "hubbub"  n ( h uh1 b uh b ) ((pos "L@%" ) ))
( "hubby"  n ( h uh1 b ii ) ((pos "K8%" ) ))
( "hubris"  n ( h y uu1 b r i s ) ((pos "L@$" ) ))
( "hubs"  n ( h uh1 b z ) ((pos "Kj%" ) ))
( "huckaback"  n ( h uh1 k @ b a k ) ((pos "L@$" ) ))
( "huckleberries"  n ( h uh1 k l b @ r i z ) ((pos "Kj$" ) ))
( "huckleberry"  n ( h uh1 k l b @ r ii ) ((pos "K8$" ) ))
( "huckster"  n ( h uh1 k s t @ r ) ((pos "K6$" ) ))
( "hucksters"  n ( h uh1 k s t @ z ) ((pos "Kj$" ) ))
( "huddle"  v ( h uh1 d l ) ((pos "J2%" "K6%" ) (subcat "2C" "15A" )))
( "huddled"  v ( h uh1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "15A" )))
( "huddles"  v ( h uh1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "15A" )))
( "huddling"  v ( h uh1 d l i ng ) ((pos "Jb%" ) (subcat "2C" "15A" )))
( "hue"  n ( h y uu1 ) ((pos "K6%" ) ))
( "hued"  j ( h y uu1 d ) ((pos "Ot%" ) ))
( "hues"  n ( h y uu1 z ) ((pos "Kj%" ) ))
( "huff"  v ( h uh1 f ) ((pos "I0%" "K6%" ) ))
( "huffed"  v ( h uh1 f t ) ((pos "Ic%" "Id%" ) ))
( "huffier"  j ( h uh1 f i@ r ) ((pos "Or$" ) ))
( "huffiest"  j ( h uh1 f i i s t ) ((pos "Os$" ) ))
( "huffily"  a ( h uh1 f i l ii ) ((pos "Pu%" ) ))
( "huffing"  v ( h uh1 f i ng ) ((pos "Ib%" ) ))
( "huffish"  j ( h uh1 f i sh ) ((pos "OA$" ) ))
( "huffs"  v ( h uh1 f s ) ((pos "Ia%" "Kj%" ) ))
( "huffy"  j ( h uh1 f ii ) ((pos "OD%" ) ))
( "hug"  v ( h uh1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "huge"  j ( h y uu1 jh ) ((pos "OA%" ) ))
( "hugely"  a ( h y uu1 jh l ii ) ((pos "Pu%" ) ))
( "hugged"  v ( h uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hugger-mugger"  n ( h uh1 g @ - m uh g @ r ) ((pos "Ki%" "OA%" "Pu%" ) ))
( "hugging"  v ( h uh1 g i ng ) ((pos "Hb%" ) (subcat )))
( "hugs"  v ( h uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "hula"  n ( h uu1 l @ ) ((pos "K6$" ) ))
( "hulas"  n ( h uu1 l @ z ) ((pos "Kj$" ) ))
( "hulk"  n ( h uh1 l k ) ((pos "K6%" ) ))
( "hulking"  j ( h uh1 l k i ng ) ((pos "OA%" ) ))
( "hulks"  n ( h uh1 l k s ) ((pos "Kj%" ) ))
( "hull"  v ( h uh1 l ) ((pos "H0$" "K6%" ) ))
( "hullabaloo"  n ( h uh2 l @ b @ l uu1 ) ((pos "K6%" ) ))
( "hullabaloos"  n ( h uh2 l @ b @ l uu1 z ) ((pos "Kj%" ) ))
( "hulled"  v ( h uh1 l d ) ((pos "Hc$" "Hd$" ) ))
( "hulling"  v ( h uh1 l i ng ) ((pos "Hb$" ) ))
( "hullo"  uh ( h @1 l ou1 ) ((pos "W-%" ) ))
( "hulls"  v ( h uh1 l z ) ((pos "Ha$" "Kj%" ) ))
( "hum"  v ( h uh1 m ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "human"  n ( h y uu1 m @ n ) ((pos "K6$" "OA%" ) ))
( "humane"  j ( h y uu1 m ei1 n ) ((pos "OA%" ) ))
( "humanely"  a ( h y uu1 m ei1 n l ii ) ((pos "Pu%" ) ))
( "humanism"  n ( h y uu1 m @ n i z @ m ) ((pos "L@%" ) ))
( "humanist"  n ( h y uu1 m @ n i s t ) ((pos "K6%" ) ))
( "humanists"  n ( h y uu1 m @ n i s t s ) ((pos "Kj%" ) ))
( "humanitarian"  n ( h y uu1 m a2 n i t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "humanitarianism"  n ( h y uu1 m a2 n i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "humanitarians"  n ( h y uu1 m a2 n i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "humanities"  n ( h y uu1 m a1 n i t i z ) ((pos "Mj%" ) ))
( "humanity"  n ( h y uu1 m a1 n i t ii ) ((pos "M8%" ) ))
( "humanize"  v ( h y uu1 m @ n ai z ) ((pos "J2%" ) (subcat "2A" )))
( "humanized"  v ( h y uu1 m @ n ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "humanizes"  v ( h y uu1 m @ n ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "humanizing"  v ( h y uu1 m @ n ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "humankind"  n ( h y uu2 m @ n k ai1 n d ) ((pos "L@%" ) ))
( "humanly"  a ( h y uu1 m @ n l ii ) ((pos "Pu%" ) ))
( "humans"  n ( h y uu1 m @ n z ) ((pos "Kj%" ) ))
( "humble"  v ( h uh1 m b l ) ((pos "H2%" "OB%" ) (subcat )))
( "humbled"  v ( h uh1 m b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "humbler"  j ( h uh1 m b l @ r ) ((pos "Or%" ) ))
( "humbles"  v ( h uh1 m b l z ) ((pos "Ha%" ) (subcat )))
( "humblest"  j ( h uh1 m b l i s t ) ((pos "Os%" ) ))
( "humbling"  v ( h uh1 m b l i ng ) ((pos "Hb%" ) (subcat )))
( "humbly"  a ( h uh1 m b l ii ) ((pos "Pu%" ) ))
( "humbug"  v ( h uh1 m b uh g ) ((pos "H4$" "M6%" "W-%" ) (subcat "6A" )))
( "humbugged"  v ( h uh1 m b uh g d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "humbugging"  v ( h uh1 m b uh g i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "humbugs"  v ( h uh1 m b uh g z ) ((pos "Ha$" "Mj%" ) (subcat "6A" )))
( "humdinger"  n ( h uh2 m d i1 ng @ r ) ((pos "K6%" ) ))
( "humdingers"  n ( h uh2 m d i1 ng @ z ) ((pos "Kj$" ) ))
( "humdrum"  j ( h uh1 m d r uh m ) ((pos "OA%" ) ))
( "humerus"  n ( h y uu1 m @ r @ s ) ((pos "K7%" ) ))
( "humeruses"  n ( h y uu1 m @ r @ s i z ) ((pos "Kj$" ) ))
( "humid"  j ( h y uu1 m i d ) ((pos "OA%" ) ))
( "humidified"  v ( h y uu1 m i1 d i f ai d ) ((pos "Hc$" "Hd$" ) ))
( "humidifies"  v ( h y uu1 m i1 d i f ai z ) ((pos "Ha$" ) ))
( "humidify"  v ( h y uu1 m i1 d i f ai ) ((pos "H3$" ) ))
( "humidifying"  v ( h y uu1 m i1 d i f ai i ng ) ((pos "Hb$" ) ))
( "humidity"  n ( h y uu1 m i1 d i t ii ) ((pos "L@%" ) ))
( "humiliate"  v ( h y uu1 m i1 l i ei t ) ((pos "H2%" ) (subcat )))
( "humiliated"  v ( h y uu1 m i1 l i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "humiliates"  v ( h y uu1 m i1 l i ei t s ) ((pos "Ha%" ) (subcat )))
( "humiliating"  v ( h y uu1 m i1 l i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "humiliation"  n ( h y uu1 m i2 l i ei1 sh @ n ) ((pos "M6%" ) ))
( "humiliations"  n ( h y uu1 m i2 l i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "humility"  n ( h y uu1 m i1 l i t ii ) ((pos "L@%" ) ))
( "hummed"  v ( h uh1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "humming"  v ( h uh1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "humming-top"  n ( h uh1 m i ng - t o p ) ((pos "K6$" ) ))
( "humming-tops"  n ( h uh1 m i ng - t o p s ) ((pos "Kj$" ) ))
( "hummingbird"  n ( h uh1 m i ng b @@ d ) ((pos "K6%" ) ))
( "hummingbirds"  n ( h uh1 m i ng b @@ d z ) ((pos "Kj%" ) ))
( "hummock"  n ( h uh1 m @ k ) ((pos "K6%" ) ))
( "hummocks"  n ( h uh1 m @ k s ) ((pos "Kj%" ) ))
( "humorist"  n ( h y uu1 m @ r i s t ) ((pos "K6%" ) ))
( "humorists"  n ( h y uu1 m @ r i s t s ) ((pos "Kj%" ) ))
( "humorous"  j ( h y uu1 m @ r @ s ) ((pos "OA%" ) ))
( "humorously"  a ( h y uu1 m @ r @ s l ii ) ((pos "Pu%" ) ))
( "humour"  v ( h y uu1 m @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "humoured"  v ( h y uu1 m @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "humouring"  v ( h y uu1 m @ r i ng ) ((pos "Hb%" ) (subcat )))
( "humourist"  n ( h y uu1 m @ r i s t ) ((pos "K6%" ) ))
( "humourists"  n ( h y uu1 m @ r i s t s ) ((pos "Kj%" ) ))
( "humourless"  j ( h y uu1 m @ l i s ) ((pos "OA%" ) ))
( "humours"  v ( h y uu1 m @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "hump"  v ( h uh1 m p ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "humpback"  n ( h uh1 m p - b a k ) ((pos "K6%" ) ))
( "humpbacked"  j ( h uh1 m p - b a k t ) ((pos "OA%" ) ))
( "humpbacks"  n ( h uh1 m p - b a k s ) ((pos "Kj%" ) ))
( "humped"  v ( h uh1 m p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "humph"  uh ( h uh1 m p f ) ((pos "W-$" ) ))
( "humping"  v ( h uh1 m p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "humps"  v ( h uh1 m p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "hums"  v ( h uh1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "humus"  n ( h y uu1 m @ s ) ((pos "L@%" ) ))
( "hunch"  v ( h uh1 n ch ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "hunchback"  n ( h uh1 n ch b a k ) ((pos "K6%" "OA%" ) ))
( "hunchbacked"  j ( h uh1 n ch b a k t ) ((pos "OA%" ) ))
( "hunchbacks"  n ( h uh1 n ch b a k s ) ((pos "Kj%" ) ))
( "hunched"  v ( h uh1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "hunches"  v ( h uh1 n ch i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "hunching"  v ( h uh1 n ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "hundred"  n ( h uh1 n d r @ d ) ((pos "K6%" "OA%" ) ))
( "hundredfold"  a ( h uh1 n d r @ d f ou l d ) ((pos "Pu%" ) ))
( "hundreds"  n ( h uh1 n d r @ d z ) ((pos "Kj%" ) ))
( "hundredth"  n ( h uh1 n d r @ d th ) ((pos "K6%" "OA%" ) ))
( "hundredths"  n ( h uh1 n d r @ d th s ) ((pos "Kj%" ) ))
( "hundredweight"  n ( h uh1 n d r @ d w ei t ) ((pos "K6%" ) ))
( "hundredweights"  n ( h uh1 n d r @ d w ei t s ) ((pos "Kj%" ) ))
( "hung"  v ( h uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "15A" )))
( "hunger"  v ( h uh1 ng g @ r ) ((pos "I0%" "L@%" ) (subcat "2A" "3A" )))
( "hunger-march"  n ( h uh1 ng g @ - m aa ch ) ((pos "K7%" ) ))
( "hunger-marcher"  n ( h uh1 ng g @ - m aa2 ch @ r ) ((pos "K6%" ) ))
( "hunger-marchers"  n ( h uh1 ng g @ - m aa2 ch @ z ) ((pos "Kj%" ) ))
( "hunger-marches"  n ( h uh1 ng g @ - m aa ch i z ) ((pos "Kj%" ) ))
( "hungered"  v ( h uh1 ng g @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "hungering"  v ( h uh1 ng g @ r i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "hungers"  v ( h uh1 ng g @ z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "hungrier"  j ( h uh1 ng g r i@ r ) ((pos "Or%" ) ))
( "hungriest"  j ( h uh1 ng g r i i s t ) ((pos "Os%" ) ))
( "hungrily"  a ( h uh1 ng g r @ l ii ) ((pos "Pu%" ) ))
( "hungry"  j ( h uh1 ng g r ii ) ((pos "OD%" ) ))
( "hunk"  n ( h uh1 ng k ) ((pos "K6%" ) ))
( "hunkers"  n ( h uh1 ng k @ z ) ((pos "Kj$" ) ))
( "hunks"  n ( h uh1 ng k s ) ((pos "Kj%" ) ))
( "hunt"  v ( h uh1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hunted"  v ( h uh1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hunter"  n ( h uh1 n t @ r ) ((pos "K6%" ) ))
( "hunters"  n ( h uh1 n t @ z ) ((pos "Kj%" ) ))
( "hunting"  v ( h uh1 n t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "hunting-crop"  n ( h uh1 n t i ng - k r o p ) ((pos "K6%" ) ))
( "hunting-crops"  n ( h uh1 n t i ng - k r o p s ) ((pos "Kj%" ) ))
( "huntress"  n ( h uh1 n t r i s ) ((pos "K7%" ) ))
( "huntresses"  n ( h uh1 n t r i s i z ) ((pos "Kj$" ) ))
( "hunts"  v ( h uh1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "huntsman"  n ( h uh1 n t s m @ n ) ((pos "Ki%" ) ))
( "huntsmen"  n ( h uh1 n t s m @ n ) ((pos "Kj%" ) ))
( "hurdle"  v ( h @@1 d l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "hurdled"  v ( h @@1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "hurdler"  n ( h @@1 d l @ r ) ((pos "K6%" ) ))
( "hurdlers"  n ( h @@1 d l @ z ) ((pos "Kj%" ) ))
( "hurdles"  v ( h @@1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "hurdling"  v ( h @@1 d l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "hurdy-gurdies"  n ( h @@1 d i - g @@ d i z ) ((pos "Kj$" ) ))
( "hurdy-gurdy"  n ( h @@1 d i - g @@ d ii ) ((pos "K8%" ) ))
( "hurl"  v ( h @@1 l ) ((pos "H0%" "K6$" ) (subcat "6A" "15A" )))
( "hurled"  v ( h @@1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "hurling"  v ( h @@1 l i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" "15A" )))
( "hurls"  v ( h @@1 l z ) ((pos "Ha%" "Kj$" ) (subcat "6A" "15A" )))
( "hurly-burly"  n ( h @@1 l i - b @@ l ii ) ((pos "L@%" ) ))
( "hurrah"  v ( h u1 r aa1 ) ((pos "I0%" "W-%" ) ))
( "hurrahed"  v ( h u1 r aa1 d ) ((pos "Ic%" "Id%" ) ))
( "hurrahing"  v ( h u1 r aa1 i ng ) ((pos "Ib$" ) ))
( "hurrahs"  v ( h u1 r aa1 z ) ((pos "Ia$" ) ))
( "hurray"  uh ( h u1 r ei1 ) ((pos "W-%" ) ))
( "hurricane"  n ( h uh1 r i k @ n ) ((pos "K6%" ) ))
( "hurricanes"  n ( h uh1 r i k @ n z ) ((pos "Kj%" ) ))
( "hurried"  v ( h uh1 r i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hurriedly"  a ( h uh1 r i d l ii ) ((pos "Pu%" ) ))
( "hurries"  v ( h uh1 r i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hurry"  v ( h uh1 r ii ) ((pos "J3%" "M8%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hurrying"  v ( h uh1 r i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hurt"  v ( h @@1 t ) ((pos "J5%" "Jc%" "Jd%" "L@%" ) (subcat "2A" "6A" )))
( "hurtful"  j ( h @@1 t f @ l ) ((pos "OA%" ) ))
( "hurting"  v ( h @@1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "hurtle"  v ( h @@1 t l ) ((pos "I2%" ) (subcat )))
( "hurtled"  v ( h @@1 t l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "hurtles"  v ( h @@1 t l z ) ((pos "Ia%" ) (subcat )))
( "hurtling"  v ( h @@1 t l i ng ) ((pos "Ib%" ) (subcat )))
( "hurts"  v ( h @@1 t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "husband"  v ( h uh1 z b @ n d ) ((pos "H0%" "K6%" ) (subcat "1" )))
( "husbanded"  v ( h uh1 z b @ n d i d ) ((pos "Hc%" "Hd%" ) (subcat "1" )))
( "husbanding"  v ( h uh1 z b @ n d i ng ) ((pos "Hb%" ) (subcat "1" )))
( "husbandman"  n ( h uh1 z b @ n d m @ n ) ((pos "Ki%" ) ))
( "husbandmen"  n ( h uh1 z b @ n d m @ n ) ((pos "Kj%" ) ))
( "husbandry"  n ( h uh1 z b @ n d r ii ) ((pos "L@%" ) ))
( "husbands"  v ( h uh1 z b @ n d z ) ((pos "Ha%" "Kj%" ) (subcat "1" )))
( "hush"  v ( h uh1 sh ) ((pos "J1%" "K7%" ) (subcat "2A" "15A" )))
( "hush-hush"  j ( h uh1 sh - h uh sh ) ((pos "OA%" ) ))
( "hush-money"  n ( h uh1 sh - m uh n ii ) ((pos "L@%" ) ))
( "hushed"  v ( h uh1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "15A" )))
( "hushes"  v ( h uh1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "15A" )))
( "hushing"  v ( h uh1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "15A" )))
( "husk"  v ( h uh1 s k ) ((pos "H0$" "K6%" ) ))
( "husked"  v ( h uh1 s k t ) ((pos "Hc$" "Hd$" ) ))
( "huskier"  j ( h uh1 s k i@ r ) ((pos "Or%" ) ))
( "huskies"  n ( h uh1 s k i z ) ((pos "Kj%" ) ))
( "huskiest"  j ( h uh1 s k i i s t ) ((pos "Os%" ) ))
( "huskily"  a ( h uh1 s k i l ii ) ((pos "Pu%" ) ))
( "huskiness"  n ( h uh1 s k i n @ s ) ((pos "L@%" ) ))
( "husking"  v ( h uh1 s k i ng ) ((pos "Hb$" ) ))
( "husks"  v ( h uh1 s k s ) ((pos "Ha$" "Kj%" ) ))
( "husky"  n ( h uh1 s k ii ) ((pos "K8%" "OD%" ) ))
( "hussar"  n ( h u1 z aa1 r ) ((pos "K6$" ) ))
( "hussars"  n ( h u1 z aa1 z ) ((pos "Kj$" ) ))
( "hussies"  n ( h uh1 s i z ) ((pos "Kj%" ) ))
( "hussy"  n ( h uh1 s ii ) ((pos "K8%" ) ))
( "hustings"  n ( h uh1 s t i ng z ) ((pos "Kj%" ) ))
( "hustle"  v ( h uh1 s l ) ((pos "J2%" "Ki%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hustled"  v ( h uh1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hustler"  n ( h uh1 s l @ r ) ((pos "K6%" ) ))
( "hustlers"  n ( h uh1 s l @ z ) ((pos "Kj%" ) ))
( "hustles"  v ( h uh1 s l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hustling"  v ( h uh1 s l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "hut"  n ( h uh1 t ) ((pos "K6%" ) ))
( "hutch"  n ( h uh1 ch ) ((pos "K7%" ) ))
( "hutches"  n ( h uh1 ch i z ) ((pos "Kj%" ) ))
( "hutment"  n ( h uh1 t m @ n t ) ((pos "K6$" ) ))
( "hutments"  n ( h uh1 t m @ n t s ) ((pos "Kj$" ) ))
( "huts"  n ( h uh1 t s ) ((pos "Kj%" ) ))
( "hutted"  j ( h uh1 t i d ) ((pos "OA$" ) ))
( "hyacinth"  n ( h ai1 @ s i n th ) ((pos "K6%" ) ))
( "hyacinths"  n ( h ai1 @ s i n th s ) ((pos "Kj%" ) ))
( "hyaena"  n ( h ai1 ii1 n @ ) ((pos "K6$" ) ))
( "hyaenas"  n ( h ai1 ii1 n @ z ) ((pos "Kj$" ) ))
( "hybrid"  n ( h ai1 b r i d ) ((pos "K6%" "OA%" ) ))
( "hybridize"  v ( h ai1 b r i d ai z ) ((pos "J2$" ) (subcat "2A" )))
( "hybridized"  v ( h ai1 b r i d ai z d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "hybridizes"  v ( h ai1 b r i d ai z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "hybridizing"  v ( h ai1 b r i d ai z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "hybrids"  n ( h ai1 b r i d z ) ((pos "Kj%" ) ))
( "hydra"  n ( h ai1 d r @ ) ((pos "K6$" ) ))
( "hydrangea"  n ( h ai1 d r ei1 n jh @ ) ((pos "K6%" ) ))
( "hydrangeas"  n ( h ai1 d r ei1 n jh @ z ) ((pos "Kj%" ) ))
( "hydrant"  n ( h ai1 d r @ n t ) ((pos "K6%" ) ))
( "hydrants"  n ( h ai1 d r @ n t s ) ((pos "Kj%" ) ))
( "hydras"  n ( h ai1 d r @ z ) ((pos "Kj%" ) ))
( "hydrate"  v ( h ai1 d r ei t ) ((pos "J2$" "K6$" ) ))
( "hydrated"  v ( h ai1 d r ei t i d ) ((pos "Jc$" "Jd$" ) ))
( "hydrates"  v ( h ai1 d r ei t s ) ((pos "Ja$" "Kj$" ) ))
( "hydrating"  v ( h ai1 d r ei t i ng ) ((pos "Jb$" ) ))
( "hydraulic"  j ( h ai1 d r oo1 l i k ) ((pos "OA%" ) ))
( "hydraulicly"  a ( h ai1 d r oo1 l i k l ii ) ((pos "Pu%" ) ))
( "hydraulics"  n ( h ai1 d r oo1 l i k s ) ((pos "Lk%" ) ))
( "hydrocarbon"  n ( h ai2 d r @ k aa1 b @ n ) ((pos "K6%" ) ))
( "hydrocarbons"  n ( h ai2 d r @ k aa1 b @ n z ) ((pos "Kj%" ) ))
( "hydrochloric"  j ( h ai2 d r @ k l o1 r i k ) ((pos "OA%" ) ))
( "hydroelectric"  j ( h ai2 d r ou i l e1 k t r i k ) ((pos "OA%" ) ))
( "hydrofoil"  n ( h ai1 d r @ f oi l ) ((pos "K6%" ) ))
( "hydrofoils"  n ( h ai1 d r @ f oi l z ) ((pos "Kj%" ) ))
( "hydrogen"  n ( h ai1 d r @ jh @ n ) ((pos "L@%" ) ))
( "hydropathic"  j ( h ai2 d r @ p a1 th i k ) ((pos "OA$" ) ))
( "hydropathy"  n ( h ai1 d r o1 p @ th ii ) ((pos "L@$" ) ))
( "hydrophobia"  n ( h ai2 d r @ f ou1 b i@ ) ((pos "L@%" ) ))
( "hydroplane"  n ( h ai1 d r @ p l ei n ) ((pos "K6%" ) ))
( "hydroplanes"  n ( h ai1 d r @ p l ei n z ) ((pos "Kj%" ) ))
( "hydroponics"  n ( h ai2 d r @ p o1 n i k s ) ((pos "Lk$" ) ))
( "hyena"  n ( h ai1 ii1 n @ ) ((pos "K6%" ) ))
( "hyenas"  n ( h ai1 ii1 n @ z ) ((pos "Kj%" ) ))
( "hygiene"  n ( h ai1 jh ii n ) ((pos "L@%" ) ))
( "hygienic"  j ( h ai1 jh ii1 n i k ) ((pos "OA%" ) ))
( "hygienically"  a ( h ai1 jh ii1 n i k l ii ) ((pos "Pu%" ) ))
( "hying"  v ( h ai1 i ng ) ((pos "Ib$" ) ))
( "hymen"  n ( h ai1 m @ n ) ((pos "K6$" ) ))
( "hymens"  n ( h ai1 m @ n z ) ((pos "Kj$" ) ))
( "hymn"  v ( h i1 m ) ((pos "H0%" "K6%" ) ))
( "hymnal"  n ( h i1 m n @ l ) ((pos "K6%" ) ))
( "hymnals"  n ( h i1 m n @ l z ) ((pos "Kj%" ) ))
( "hymned"  v ( h i1 m d ) ((pos "Hc%" "Hd%" ) ))
( "hymning"  v ( h i1 m i ng ) ((pos "Hb%" ) ))
( "hymns"  v ( h i1 m z ) ((pos "Ha%" "Kj%" ) ))
( "hyperbola"  n ( h ai1 p @@1 b @ l @ ) ((pos "K6$" ) ))
( "hyperbolas"  n ( h ai1 p @@1 b @ l @ z ) ((pos "Kj$" ) ))
( "hyperbole"  n ( h ai1 p @@1 b @ l ii ) ((pos "M6%" ) ))
( "hyperboles"  n ( h ai1 p @@1 b @ l i z ) ((pos "Mj%" ) ))
( "hyperbolic"  j ( h ai2 p @ b o1 l i k ) ((pos "OA$" ) ))
( "hypercritical"  j ( h ai2 p @ k r i1 t i k l ) ((pos "OA%" ) ))
( "hypermarket"  n ( h ai1 p @ m aa k i t ) ((pos "K6%" ) ))
( "hypermarkets"  n ( h ai1 p @ m aa k i t s ) ((pos "Kj%" ) ))
( "hypersensitive"  j ( h ai2 p @ s e1 n s i t i v ) ((pos "OA%" ) ))
( "hyphen"  v ( h ai1 f n ) ((pos "H0%" "K6%" ) ))
( "hyphenate"  v ( h ai1 f @ n ei t ) ((pos "H2%" ) ))
( "hyphenated"  v ( h ai1 f @ n ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "hyphenates"  v ( h ai1 f @ n ei t s ) ((pos "Ha%" ) ))
( "hyphenating"  v ( h ai1 f @ n ei t i ng ) ((pos "Hb%" ) ))
( "hyphened"  v ( h ai1 f n d ) ((pos "Hc$" "Hd%" ) ))
( "hyphening"  v ( h ai1 f n i ng ) ((pos "Hb$" ) ))
( "hyphens"  v ( h ai1 f n z ) ((pos "Ha$" "Kj%" ) ))
( "hypnoses"  n ( h i1 p n ou s ii z ) ((pos "Kj%" ) ))
( "hypnosis"  n ( h i1 p n ou1 s i s ) ((pos "Mi%" ) ))
( "hypnotic"  j ( h i1 p n o1 t i k ) ((pos "OA%" ) ))
( "hypnotism"  n ( h i1 p n @ t i z @ m ) ((pos "L@%" ) ))
( "hypnotist"  n ( h i1 p n @ t i s t ) ((pos "K6%" ) ))
( "hypnotists"  n ( h i1 p n @ t i s t s ) ((pos "Kj%" ) ))
( "hypnotize"  v ( h i1 p n @ t ai z ) ((pos "H2%" ) (subcat )))
( "hypnotized"  v ( h i1 p n @ t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "hypnotizes"  v ( h i1 p n @ t ai z i z ) ((pos "Ha%" ) (subcat )))
( "hypnotizing"  v ( h i1 p n @ t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "hypo"  n ( h ai1 p ou ) ((pos "L@$" ) ))
( "hypochondria"  n ( h ai2 p @ k o1 n d r i@ ) ((pos "M6%" ) ))
( "hypochondriac"  n ( h ai2 p @ k o1 n d r i a k ) ((pos "K6%" "OA%" ) ))
( "hypochondriacal"  j ( h ai2 p @ k @ n d r ai1 @ k l ) ((pos "OA%" ) ))
( "hypochondriacs"  n ( h ai2 p @ k o1 n d r i a k s ) ((pos "Kj%" ) ))
( "hypochondrias"  n ( h ai2 p @ k o1 n d r i@ z ) ((pos "Mj$" ) ))
( "hypocrisies"  n ( h i1 p o1 k r @ s i z ) ((pos "Mj$" ) ))
( "hypocrisy"  n ( h i1 p o1 k r @ s ii ) ((pos "M8%" ) ))
( "hypocrite"  n ( h i1 p @ k r i t ) ((pos "K6%" ) ))
( "hypocrites"  n ( h i1 p @ k r i t s ) ((pos "Kj%" ) ))
( "hypocritical"  j ( h i2 p @ k r i1 t i k l ) ((pos "OA%" ) ))
( "hypocritically"  a ( h i2 p @ k r i1 t i k l ii ) ((pos "Pu%" ) ))
( "hypodermic"  n ( h ai2 p @ d @@1 m i k ) ((pos "K6$" "OA%" ) ))
( "hypodermics"  n ( h ai2 p @ d @@1 m i k s ) ((pos "Kj$" ) ))
( "hypotenuse"  n ( h ai2 p o1 t @ n y uu z ) ((pos "K6%" ) ))
( "hypotenuses"  n ( h ai2 p o1 t @ n y uu z i z ) ((pos "Kj$" ) ))
( "hypothecate"  v ( h ai1 p o1 th i k ei t ) ((pos "H2$" ) ))
( "hypothecated"  v ( h ai1 p o1 th i k ei t i d ) ((pos "Hc$" "Hd$" ) ))
( "hypothecates"  v ( h ai1 p o1 th i k ei t s ) ((pos "Ha$" ) ))
( "hypothecating"  v ( h ai1 p o1 th i k ei t i ng ) ((pos "Hb$" ) ))
( "hypotheses"  n ( h ai1 p o1 th i s ii z ) ((pos "Kj%" ) ))
( "hypothesis"  n ( h ai1 p o1 th @ s i s ) ((pos "Ki%" ) ))
( "hypothetical"  j ( h ai2 p @ th e1 t i k l ) ((pos "OA%" ) ))
( "hyssop"  n ( h i1 s @ p ) ((pos "L@$" ) ))
( "hysteria"  n ( h i1 s t i@1 r i@ ) ((pos "L@%" ) ))
( "hysterical"  j ( h i1 s t e1 r i k l ) ((pos "OA%" ) ))
( "hysterically"  a ( h i1 s t e1 r i k l ii ) ((pos "Pu%" ) ))
( "hysterics"  n ( h i1 s t e1 r i k s ) ((pos "Lk%" ) ))
( "i"  n ( ai1 ) ((pos "Ki$" ) ))
( "i's"  n ( ai1 z ) ((pos "Kj$" ) ))
( "iamb"  n ( ai1 a m ) ((pos "K6$" ) ))
( "iambi"  n ( ai1 a1 m b ai ) ((pos "Kj$" ) ))
( "iambic"  j ( ai1 a1 m b i k ) ((pos "OA$" ) ))
( "iambics"  n ( ai1 a1 m b i k s ) ((pos "Lk$" ) ))
( "iambs"  n ( ai1 a m z ) ((pos "Kj$" ) ))
( "iambus"  n ( ai1 a1 m b @ s ) ((pos "K7$" ) ))
( "iambuses"  n ( ai1 a1 m b @ s i z ) ((pos "Kj$" ) ))
( "ibex"  n ( ai1 b e k s ) ((pos "K7$" ) ))
( "ibexes"  n ( ai1 b e k s i z ) ((pos "Kj$" ) ))
( "ibid"  n ( i1 b i1 d ) ((pos "Y~%" ) ))
( "ibidem"  a ( i1 b i d e m ) ((pos "Pu$" ) ))
( "ibis"  n ( ai1 b i s ) ((pos "K7$" ) ))
( "ibises"  n ( ai1 b i s i z ) ((pos "Kj$" ) ))
( "ice"  v ( ai1 s ) ((pos "J2%" "M6%" ) (subcat "2C" "6A" )))
( "ice-axe"  n ( ai1 s - a k s ) ((pos "K6%" ) ))
( "ice-axes"  n ( ai1 s - a k s i z ) ((pos "Kj%" ) ))
( "ice-cream"  n ( ai1 s - k r ii1 m ) ((pos "M6%" ) ))
( "ice-creams"  n ( ai1 s - k r ii1 m z ) ((pos "Mj%" ) ))
( "ice-lollies"  n ( ai1 s - l o1 l i z ) ((pos "Kj%" ) ))
( "ice-lolly"  n ( ai1 s - l o1 l ii ) ((pos "K8%" ) ))
( "ice-show"  n ( ai1 s - sh ou ) ((pos "K6%" ) ))
( "ice-shows"  n ( ai1 s - sh ou z ) ((pos "Kj%" ) ))
( "ice-skate"  v ( ai1 s - s k ei t ) ((pos "I2%" "K6%" ) ))
( "ice-skated"  v ( ai1 s - s k ei t i d ) ((pos "Ic%" "Id%" ) ))
( "ice-skates"  v ( ai1 s - s k ei t s ) ((pos "Ia%" "Kj%" ) ))
( "ice-skating"  v ( ai1 s - s k ei t i ng ) ((pos "Ib%" "L@%" ) ))
( "ice-tray"  n ( ai1 s - t r ei ) ((pos "K6%" ) ))
( "ice-trays"  n ( ai1 s - t r ei z ) ((pos "Kj%" ) ))
( "iceberg"  n ( ai1 s b @@ g ) ((pos "K6%" ) ))
( "icebergs"  n ( ai1 s b @@ g z ) ((pos "Kj%" ) ))
( "iceboat"  n ( ai1 s b ou t ) ((pos "K6$" ) ))
( "iceboats"  n ( ai1 s b ou t s ) ((pos "Kj$" ) ))
( "icebound"  j ( ai1 s b au n d ) ((pos "OA%" ) ))
( "icebox"  n ( ai1 s b o k s ) ((pos "K7%" ) ))
( "iceboxes"  n ( ai1 s b o k s i z ) ((pos "Kj%" ) ))
( "icebreaker"  n ( ai1 s b r ei k @ r ) ((pos "K6%" ) ))
( "icebreakers"  n ( ai1 s b r ei k @ z ) ((pos "Kj%" ) ))
( "icecap"  n ( ai1 s k a p ) ((pos "K6%" ) ))
( "icecaps"  n ( ai1 s k a p s ) ((pos "Kj%" ) ))
( "icecube"  n ( ai1 s k y uu b ) ((pos "K6%" ) ))
( "icecubes"  n ( ai1 s k y uu b z ) ((pos "Kj%" ) ))
( "iced"  v ( ai1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "icefall"  n ( ai1 s f oo l ) ((pos "K6$" ) ))
( "icefalls"  n ( ai1 s f oo l z ) ((pos "Kj$" ) ))
( "icefield"  n ( ai1 s f ii l d ) ((pos "K6%" ) ))
( "icefields"  n ( ai1 s f ii l d z ) ((pos "Kj%" ) ))
( "icefloe"  n ( ai1 s f l ou ) ((pos "K6%" ) ))
( "icefloes"  n ( ai1 s f l ou z ) ((pos "Kj%" ) ))
( "icefree"  j ( ai1 s f r ii ) ((pos "OA$" ) ))
( "icehouse"  n ( ai1 s h au s ) ((pos "K6%" ) ))
( "icehouses"  n ( ai1 s h au z i z ) ((pos "Kj%" ) ))
( "iceman"  n ( ai1 s m a n ) ((pos "Ki$" ) ))
( "icemen"  n ( ai1 s m e n ) ((pos "Kj$" ) ))
( "icepack"  n ( ai1 s p a k ) ((pos "K6%" ) ))
( "icepacks"  n ( ai1 s p a k s ) ((pos "Kj%" ) ))
( "icepick"  n ( ai1 s p i k ) ((pos "K6%" ) ))
( "icepicks"  n ( ai1 s p i k s ) ((pos "Kj%" ) ))
( "icerink"  n ( ai1 s r i ng k ) ((pos "K6%" ) ))
( "icerinks"  n ( ai1 s r i ng k s ) ((pos "Kj%" ) ))
( "ices"  v ( ai1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "ichneumon"  n ( i1 k n y uu1 m @ n ) ((pos "K6$" ) ))
( "ichneumon-flies"  n ( i1 k n y uu1 m @ n - f l ai z ) ((pos "Kj$" ) ))
( "ichneumon-fly"  n ( i1 k n y uu1 m @ n - f l ai ) ((pos "K8$" ) ))
( "ichneumons"  n ( i1 k n y uu1 m @ n z ) ((pos "Kj$" ) ))
( "icicle"  n ( ai1 s i k l ) ((pos "K6%" ) ))
( "icicles"  n ( ai1 s i k l z ) ((pos "Kj%" ) ))
( "icier"  j ( ai1 s i@ r ) ((pos "Or%" ) ))
( "iciest"  j ( ai1 s i i s t ) ((pos "Os%" ) ))
( "icily"  a ( ai1 s i l ii ) ((pos "Pu%" ) ))
( "icing"  v ( ai1 s i ng ) ((pos "Jb%" "L@%" ) (subcat "2C" "6A" )))
( "icon"  n ( ai1 k o n ) ((pos "K6%" ) ))
( "iconoclast"  n ( ai1 k o1 n @ k l a s t ) ((pos "K6%" ) ))
( "iconoclasts"  n ( ai1 k o1 n @ k l a s t s ) ((pos "Kj%" ) ))
( "icons"  n ( ai1 k o n z ) ((pos "Kj%" ) ))
( "icy"  j ( ai1 s ii ) ((pos "OD%" ) ))
( "id"  n ( i1 d ) ((pos "Ki$" ) ))
( "id est"  nil ( i2 d - e1 s t ) ((pos "Z-$" ) ))
( "idea"  n ( ai1 d i@1 ) ((pos "K6*" ) ))
( "ideal"  n ( ai1 d i@1 l ) ((pos "K6%" "OA%" ) ))
( "idealism"  n ( ai1 d i@1 l i z @ m ) ((pos "L@%" ) ))
( "idealist"  n ( ai1 d i@1 l i s t ) ((pos "K6%" ) ))
( "idealistic"  j ( ai1 d i@2 l i1 s t i k ) ((pos "OA%" ) ))
( "idealists"  n ( ai1 d i@1 l i s t s ) ((pos "Kj%" ) ))
( "idealization"  n ( ai1 d i@2 l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "idealizations"  n ( ai1 d i@2 l ai z ei1 sh @ n z ) ((pos "Kj%" ) ))
( "idealize"  v ( ai1 d i@1 l ai z ) ((pos "H2%" ) (subcat )))
( "idealized"  v ( ai1 d i@1 l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "idealizes"  v ( ai1 d i@1 l ai z i z ) ((pos "Ha%" ) (subcat )))
( "idealizing"  v ( ai1 d i@1 l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "ideally"  a ( ai1 d i@1 l ii ) ((pos "Pu%" ) ))
( "ideals"  n ( ai1 d i@1 l z ) ((pos "Kj%" ) ))
( "ideas"  n ( ai1 d i@1 z ) ((pos "Kj%" ) ))
( "idem"  n ( i1 d e m ) ((pos "Y~$" ) ))
( "identical"  j ( ai1 d e1 n t i k l ) ((pos "OA%" ) ))
( "identically"  a ( ai1 d e1 n t i k l ii ) ((pos "Pu%" ) ))
( "identifiable"  j ( ai1 d e2 n t i f ai1 @ b l ) ((pos "OA%" ) ))
( "identifiably"  a ( ai1 d e2 n t i f ai1 @ b l ii ) ((pos "Pu%" ) ))
( "identification"  n ( ai1 d e2 n t i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "identified"  v ( ai1 d e1 n t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" )))
( "identifies"  v ( ai1 d e1 n t i f ai z ) ((pos "Ha%" ) (subcat "3A" "6A" )))
( "identify"  v ( ai1 d e1 n t i f ai ) ((pos "H3%" ) (subcat "3A" "6A" )))
( "identifying"  v ( ai1 d e1 n t i f ai i ng ) ((pos "Hb%" ) (subcat "3A" "6A" )))
( "identikit"  n ( ai1 d e1 n t i k i t ) ((pos "K6%" ) ))
( "identikits"  n ( ai1 d e1 n t i k i t s ) ((pos "Kj$" ) ))
( "identities"  n ( ai1 d e1 n t i t i z ) ((pos "Mj%" ) ))
( "identity"  n ( ai1 d e1 n t i t ii ) ((pos "M8%" ) ))
( "ideogram"  n ( i1 d i@ g r a m ) ((pos "K6$" ) ))
( "ideograms"  n ( i1 d i@ g r a m z ) ((pos "Kj$" ) ))
( "ideograph"  n ( i1 d i@ g r aa f ) ((pos "K6$" ) ))
( "ideographic"  j ( i2 d i@ g r a1 f i k ) ((pos "OA$" ) ))
( "ideographs"  n ( i1 d i@ g r aa f s ) ((pos "Kj$" ) ))
( "ideological"  j ( ai2 d i@ l o1 jh i k l ) ((pos "OA$" ) ))
( "ideologically"  a ( ai2 d i@ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "ideologies"  n ( ai2 d i o1 l @ jh i z ) ((pos "Mj%" ) ))
( "ideology"  n ( ai2 d i o1 l @ jh ii ) ((pos "M8%" ) ))
( "ides"  n ( ai1 d z ) ((pos "Kj$" ) ))
( "idiocies"  n ( i1 d i@ s i z ) ((pos "Mj%" ) ))
( "idiocy"  n ( i1 d i@ s ii ) ((pos "M8%" ) ))
( "idiolect"  n ( i1 d i@ l e k t ) ((pos "K6$" ) ))
( "idiolects"  n ( i1 d i@ l e k t s ) ((pos "Kj$" ) ))
( "idiom"  n ( i1 d i@ m ) ((pos "M6%" ) ))
( "idiomatic"  j ( i2 d i@ m a1 t i k ) ((pos "OA%" ) ))
( "idiomatically"  a ( i2 d i@ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "idioms"  n ( i1 d i@ m z ) ((pos "Mj%" ) ))
( "idiosyncrasies"  n ( i2 d i@ s i1 ng k r @ s i z ) ((pos "Kj%" ) ))
( "idiosyncrasy"  n ( i2 d i@ s i1 ng k r @ s ii ) ((pos "K8%" ) ))
( "idiosyncratic"  j ( i2 d i@ s i ng k r a1 t i k ) ((pos "OA%" ) ))
( "idiot"  n ( i1 d i@ t ) ((pos "K6%" ) ))
( "idiotic"  j ( i2 d i o1 t i k ) ((pos "OA%" ) ))
( "idiotically"  a ( i2 d i o1 t i k l ii ) ((pos "Pu%" ) ))
( "idiots"  n ( i1 d i@ t s ) ((pos "Kj%" ) ))
( "idle"  v ( ai1 d l ) ((pos "J2%" "OB%" ) (subcat "2A" "2C" )))
( "idled"  v ( ai1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "idleness"  n ( ai1 d l n @ s ) ((pos "L@%" ) ))
( "idler"  n ( ai1 d l @ r ) ((pos "K6%" "Or%" ) ))
( "idlers"  n ( ai1 d l @ z ) ((pos "Kj%" ) ))
( "idles"  v ( ai1 d l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "idlest"  j ( ai1 d l i s t ) ((pos "Os%" ) ))
( "idling"  v ( ai1 d l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "idly"  a ( ai1 d l ii ) ((pos "Pu%" ) ))
( "idol"  n ( ai1 d l ) ((pos "K6%" ) ))
( "idolater"  n ( ai1 d o1 l @ t @ r ) ((pos "K6%" ) ))
( "idolaters"  n ( ai1 d o1 l @ t @ z ) ((pos "Kj%" ) ))
( "idolatress"  n ( ai1 d o1 l @ t r i s ) ((pos "K7$" ) ))
( "idolatresses"  n ( ai1 d o1 l @ t r i s i z ) ((pos "Kj$" ) ))
( "idolatries"  n ( ai1 d o1 l @ t r i z ) ((pos "Mj$" ) ))
( "idolatrous"  j ( ai1 d o1 l @ t r @ s ) ((pos "OA%" ) ))
( "idolatrously"  a ( ai1 d o1 l @ t r @ s l ii ) ((pos "Pu%" ) ))
( "idolatry"  n ( ai1 d o1 l @ t r ii ) ((pos "M8%" ) ))
( "idolization"  n ( ai2 d @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "idolizations"  n ( ai2 d @ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "idolize"  v ( ai1 d @ l ai z ) ((pos "H2%" ) (subcat )))
( "idolized"  v ( ai1 d @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "idolizes"  v ( ai1 d @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "idolizing"  v ( ai1 d @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "idols"  n ( ai1 d l z ) ((pos "Kj%" ) ))
( "idyll"  n ( i1 d i l ) ((pos "K6%" ) ))
( "idyllic"  j ( i1 d i1 l i k ) ((pos "OA%" ) ))
( "idylls"  n ( i1 d i l z ) ((pos "Kj%" ) ))
( "ie"  n ( ai2 ii1 ) ((pos "Y~%" ) ))
( "if"  cc ( i f ) ((pos "V-*" ) ))
( "igloo"  n ( i1 g l uu ) ((pos "K6%" ) ))
( "igloos"  n ( i1 g l uu z ) ((pos "Kj%" ) ))
( "igneous"  j ( i1 g n i@ s ) ((pos "OA$" ) ))
( "ignes fatui"  n ( i2 g n ii z - f a1 t y u ai ) ((pos "Kj$" ) ))
( "ignis fatuus"  n ( i2 g n i s - f a1 t y u@ s ) ((pos "Ki$" ) ))
( "ignite"  v ( i1 g n ai1 t ) ((pos "J2%" ) (subcat "2A" )))
( "ignited"  v ( i1 g n ai1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ignites"  v ( i1 g n ai1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "igniting"  v ( i1 g n ai1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ignition"  n ( i1 g n i1 sh @ n ) ((pos "L@%" ) ))
( "ignoble"  j ( i1 g n ou1 b l ) ((pos "OA%" ) ))
( "ignobly"  a ( i1 g n ou1 b l ii ) ((pos "Pu%" ) ))
( "ignominies"  n ( i1 g n @ m i n i z ) ((pos "Mj$" ) ))
( "ignominious"  j ( i2 g n @ m i1 n i@ s ) ((pos "OA%" ) ))
( "ignominiously"  a ( i2 g n @ m i1 n i@ s l ii ) ((pos "Pu%" ) ))
( "ignominy"  n ( i1 g n @ m i n ii ) ((pos "M8%" ) ))
( "ignoramus"  n ( i2 g n @ r ei1 m @ s ) ((pos "K7%" ) ))
( "ignoramuses"  n ( i2 g n @ r ei1 m @ s i z ) ((pos "Kj%" ) ))
( "ignorance"  n ( i1 g n @ r @ n s ) ((pos "L@%" ) ))
( "ignorant"  j ( i1 g n @ r @ n t ) ((pos "OA%" ) ))
( "ignorantly"  a ( i1 g n @ r @ n t l ii ) ((pos "Pu%" ) ))
( "ignore"  v ( i1 g n oo1 r ) ((pos "H2%" ) (subcat )))
( "ignored"  v ( i1 g n oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ignores"  v ( i1 g n oo1 z ) ((pos "Ha%" ) (subcat )))
( "ignoring"  v ( i1 g n oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "iguana"  n ( i1 g w aa1 n @ ) ((pos "K6$" ) ))
( "iguanas"  n ( i1 g w aa1 n @ z ) ((pos "Kj$" ) ))
( "ikon"  n ( ai1 k o n ) ((pos "K6$" ) ))
( "ikons"  n ( ai1 k o n z ) ((pos "Kj$" ) ))
( "ilex"  n ( ai1 l e k s ) ((pos "K7$" ) ))
( "ilexes"  n ( ai1 l e k s i z ) ((pos "Kj$" ) ))
( "ilk"  n ( i1 l k ) ((pos "L@%" ) ))
( "ill"  n ( i1 l ) ((pos "M6$" "OA%" "Pu%" ) ))
( "ill-advised"  j ( i2 l - @ d v ai1 z d ) ((pos "OA%" ) ))
( "ill-affected"  j ( i2 l - @ f e1 k t i d ) ((pos "OA$" ) ))
( "ill-bred"  j ( i1 l - b r e1 d ) ((pos "OA%" ) ))
( "ill-breeding"  n ( i1 l - b r ii1 d i ng ) ((pos "L@%" ) ))
( "ill-disposed"  j ( i2 l - d i s p ou1 z d ) ((pos "OA%" ) ))
( "ill-fated"  j ( i1 l - f ei1 t i d ) ((pos "OA%" ) ))
( "ill-favoured"  j ( i1 l - f ei1 v @ d ) ((pos "OA%" ) ))
( "ill-gotten"  j ( i1 l - g o1 t @ n ) ((pos "OA%" ) ))
( "ill-judged"  j ( i1 l - jh uh1 jh d ) ((pos "OA%" ) ))
( "ill-mannered"  j ( i1 l - m a1 n @ d ) ((pos "OA%" ) ))
( "ill-natured"  j ( i1 l - n ei1 ch @ d ) ((pos "OA%" ) ))
( "ill-omened"  j ( i1 l - ou1 m n d ) ((pos "OA%" ) ))
( "ill-starred"  j ( i1 l - s t aa1 d ) ((pos "OA%" ) ))
( "ill-timed"  j ( i1 l - t ai1 m d ) ((pos "OA%" ) ))
( "ill-treat"  v ( i1 l - t r ii1 t ) ((pos "H0%" ) ))
( "ill-treated"  v ( i1 l - t r ii1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "ill-treating"  v ( i1 l - t r ii1 t i ng ) ((pos "Hb%" ) ))
( "ill-treatment"  n ( i1 l - t r ii1 t m @ n t ) ((pos "L@%" ) ))
( "ill-treats"  v ( i1 l - t r ii1 t s ) ((pos "Ha%" ) ))
( "ill-usage"  n ( i1 l - y uu1 z i jh ) ((pos "L@%" ) ))
( "ill-use"  v ( i1 l - y uu1 z ) ((pos "H2$" ) ))
( "ill-used"  v ( i1 l - y uu1 z d ) ((pos "Hc$" "Hd%" ) ))
( "ill-uses"  v ( i1 l - y uu1 z i z ) ((pos "Ha$" ) ))
( "ill-using"  v ( i1 l - y uu1 z i ng ) ((pos "Hb$" ) ))
( "illegal"  j ( i1 l ii1 g l ) ((pos "OA%" ) ))
( "illegalities"  n ( i2 l i g a1 l i t i z ) ((pos "Mj%" ) ))
( "illegality"  n ( i2 l i g a1 l i t ii ) ((pos "M8%" ) ))
( "illegally"  a ( i1 l ii1 g @ l ii ) ((pos "Pu%" ) ))
( "illegibility"  n ( i1 l e2 jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "illegible"  j ( i1 l e1 jh @ b l ) ((pos "OA%" ) ))
( "illegibly"  a ( i1 l e1 jh @ b l ii ) ((pos "Pu%" ) ))
( "illegitimacy"  n ( i2 l i jh i1 t i m @ s ii ) ((pos "L@%" ) ))
( "illegitimate"  n ( i2 l i jh i1 t i m @ t ) ((pos "K9%" "OA%" ) ))
( "illegitimately"  a ( i2 l i jh i1 t i m @ t l ii ) ((pos "Pu%" ) ))
( "illiberal"  j ( i1 l i1 b @ r @ l ) ((pos "OA%" ) ))
( "illiberality"  n ( i1 l i2 b @ r a1 l i t ii ) ((pos "L@$" ) ))
( "illiberally"  a ( i1 l i1 b @ r @ l ii ) ((pos "Pu%" ) ))
( "illicit"  j ( i1 l i1 s i t ) ((pos "OA%" ) ))
( "illicitly"  a ( i1 l i1 s i t l ii ) ((pos "Pu%" ) ))
( "illimitable"  j ( i1 l i1 m i t @ b l ) ((pos "OA%" ) ))
( "illiteracies"  n ( i1 l i1 t @ r @ s i z ) ((pos "Mj$" ) ))
( "illiteracy"  n ( i1 l i1 t @ r @ s ii ) ((pos "M8%" ) ))
( "illiterate"  n ( i1 l i1 t @ r @ t ) ((pos "K6%" "OA%" ) ))
( "illiterates"  n ( i1 l i1 t @ r @ t s ) ((pos "Kj%" ) ))
( "illness"  n ( i1 l n @ s ) ((pos "M7%" ) ))
( "illnesses"  n ( i1 l n @ s i z ) ((pos "Mj%" ) ))
( "illogical"  j ( i1 l o1 jh i k l ) ((pos "OA%" ) ))
( "illogicalities"  n ( i1 l o2 jh i k a1 l i t i z ) ((pos "Mj%" ) ))
( "illogicality"  n ( i1 l o2 jh i k a1 l i t ii ) ((pos "M8%" ) ))
( "illogically"  a ( i1 l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "illogicalness"  n ( i1 l o1 jh i k l n @ s ) ((pos "L@$" ) ))
( "ills"  n ( i1 l z ) ((pos "Mj%" ) ))
( "illume"  v ( i1 l uu1 m ) ((pos "H2$" ) ))
( "illumed"  v ( i1 l uu1 m d ) ((pos "Hc$" "Hd$" ) ))
( "illumes"  v ( i1 l uu1 m z ) ((pos "Ha$" ) ))
( "illuminate"  v ( i1 l uu1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "illuminated"  v ( i1 l uu1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "illuminates"  v ( i1 l uu1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "illuminating"  v ( i1 l uu1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "illumination"  n ( i1 l uu2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "illuminations"  n ( i1 l uu2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "illumine"  v ( i1 l uu1 m i n ) ((pos "H2%" ) (subcat )))
( "illumined"  v ( i1 l uu1 m i n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "illumines"  v ( i1 l uu1 m i n z ) ((pos "Ha%" ) (subcat )))
( "illuming"  v ( i1 l uu1 m i ng ) ((pos "Hb$" ) ))
( "illumining"  v ( i1 l uu1 m i n i ng ) ((pos "Hb$" ) (subcat )))
( "illusion"  n ( i1 l uu1 zh n ) ((pos "M6%" ) ))
( "illusionist"  n ( i1 l uu1 zh @ n i s t ) ((pos "K6%" ) ))
( "illusionists"  n ( i1 l uu1 zh @ n i s t s ) ((pos "Kj%" ) ))
( "illusions"  n ( i1 l uu1 zh n z ) ((pos "Mj%" ) ))
( "illusive"  j ( i1 l uu1 s i v ) ((pos "OA$" ) ))
( "illusory"  j ( i1 l uu1 s @ r ii ) ((pos "OA%" ) ))
( "illustrate"  v ( i1 l @ s t r ei t ) ((pos "H2%" ) (subcat )))
( "illustrated"  v ( i1 l @ s t r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "illustrates"  v ( i1 l @ s t r ei t s ) ((pos "Ha%" ) (subcat )))
( "illustrating"  v ( i1 l @ s t r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "illustration"  n ( i2 l @ s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "illustrations"  n ( i2 l @ s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "illustrative"  j ( i1 l @ s t r @ t i v ) ((pos "OA%" ) ))
( "illustrator"  n ( i1 l @ s t r ei t @ r ) ((pos "K6%" ) ))
( "illustrators"  n ( i1 l @ s t r ei t @ z ) ((pos "Kj%" ) ))
( "illustrious"  j ( i1 l uh1 s t r i@ s ) ((pos "OA%" ) ))
( "illustriously"  a ( i1 l uh1 s t r i@ s l ii ) ((pos "Pu%" ) ))
( "image"  v ( i1 m i jh ) ((pos "H2$" "K6%" ) (subcat )))
( "imaged"  v ( i1 m i jh d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "imagery"  n ( i1 m i jh @ r ii ) ((pos "L@%" ) ))
( "images"  v ( i1 m i jh i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "imaginable"  j ( i1 m a1 jh i n @ b l ) ((pos "OA%" ) ))
( "imaginary"  j ( i1 m a1 jh i n @ r ii ) ((pos "OA%" ) ))
( "imagination"  n ( i1 m a2 jh i n ei1 sh @ n ) ((pos "M6%" ) ))
( "imaginations"  n ( i1 m a2 jh i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "imaginative"  j ( i1 m a1 jh i n @ t i v ) ((pos "OA%" ) ))
( "imaginatively"  a ( i1 m a1 jh i n @ t i v l ii ) ((pos "Pu%" ) ))
( "imagine"  v ( i1 m a1 jh i n ) ((pos "H2%" ) (subcat "6A" "6C" "9" "10" "16B" "19A" "19C" )))
( "imagined"  v ( i1 m a1 jh i n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "10" "16B" "19A" "19C" )))
( "imagines"  v ( i1 m a1 jh i n z ) ((pos "Ha%" ) (subcat "6A" "6C" "9" "10" "16B" "19A" "19C" )))
( "imaging"  v ( i1 m i jh i ng ) ((pos "Hb$" ) (subcat )))
( "imagining"  v ( i1 m a1 jh i n i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "10" "16B" "19A" "19C" )))
( "imam"  n ( i1 m aa1 m ) ((pos "K6$" ) ))
( "imams"  n ( i1 m aa1 m z ) ((pos "Kj$" ) ))
( "imbalance"  n ( i2 m b a1 l @ n s ) ((pos "K6%" ) ))
( "imbalances"  n ( i2 m b a1 l @ n s i z ) ((pos "Kj$" ) ))
( "imbecile"  n ( i1 m b @ s ii l ) ((pos "K6%" "OA%" ) ))
( "imbeciles"  n ( i1 m b @ s ii l z ) ((pos "Kj%" ) ))
( "imbecilities"  n ( i2 m b @ s i1 l i t i z ) ((pos "Mj%" ) ))
( "imbecility"  n ( i2 m b @ s i1 l i t ii ) ((pos "M8%" ) ))
( "imbed"  v ( i1 m b e1 d ) ((pos "H4$" ) ))
( "imbedded"  v ( i1 m b e1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "imbedding"  v ( i1 m b e1 d i ng ) ((pos "Hb$" ) ))
( "imbeds"  v ( i1 m b e1 d z ) ((pos "Ha$" ) ))
( "imbibe"  v ( i1 m b ai1 b ) ((pos "H2%" ) ))
( "imbibed"  v ( i1 m b ai1 b d ) ((pos "Hc%" "Hd%" ) ))
( "imbibes"  v ( i1 m b ai1 b z ) ((pos "Ha%" ) ))
( "imbibing"  v ( i1 m b ai1 b i ng ) ((pos "Hb%" ) ))
( "imbroglio"  n ( i1 m b r ou1 l i@ @ ) ((pos "K6$" ) ))
( "imbroglios"  n ( i1 m b r ou1 l i@ u z ) ((pos "Kj$" ) ))
( "imbue"  v ( i1 m b y uu1 ) ((pos "H2$" ) (subcat )))
( "imbued"  v ( i1 m b y uu1 d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "imbues"  v ( i1 m b y uu1 z ) ((pos "Ha$" ) (subcat )))
( "imbuing"  v ( i1 m b y uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "imitate"  v ( i1 m i t ei t ) ((pos "H2%" ) (subcat )))
( "imitated"  v ( i1 m i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "imitates"  v ( i1 m i t ei t s ) ((pos "Ha%" ) (subcat )))
( "imitating"  v ( i1 m i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "imitation"  n ( i2 m i t ei1 sh @ n ) ((pos "M6%" ) ))
( "imitations"  n ( i2 m i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "imitative"  j ( i1 m i t @ t i v ) ((pos "OA%" ) ))
( "imitator"  n ( i1 m i t ei t @ r ) ((pos "K6%" ) ))
( "imitators"  n ( i1 m i t ei t @ z ) ((pos "Kj%" ) ))
( "immaculate"  j ( i1 m a1 k y u l @ t ) ((pos "OA%" ) ))
( "immaculately"  a ( i1 m a1 k y u l @ t l ii ) ((pos "Pu%" ) ))
( "immanence"  n ( i1 m @ n @ n s ) ((pos "K6$" ) ))
( "immanences"  n ( i1 m @ n @ n s i z ) ((pos "Kj$" ) ))
( "immanent"  j ( i1 m @ n @ n t ) ((pos "OA$" ) ))
( "immaterial"  j ( i2 m @ t i@1 r i@ l ) ((pos "OA%" ) ))
( "immature"  j ( i2 m @ t y u@1 r ) ((pos "OA%" ) ))
( "immaturity"  n ( i2 m @ t y u@1 r i t ii ) ((pos "L@%" ) ))
( "immeasurable"  j ( i1 m e1 zh @ r @ b l ) ((pos "OA%" ) ))
( "immeasurably"  a ( i1 m e1 zh @ r @ b l ii ) ((pos "Pu%" ) ))
( "immediacy"  n ( i1 m ii1 d i@ s ii ) ((pos "L@%" ) ))
( "immediate"  j ( i1 m ii1 d i@ t ) ((pos "OA%" ) ))
( "immediately"  a ( i1 m ii1 d i@ t l ii ) ((pos "Pu%" "V-%" ) ))
( "immemorial"  j ( i2 m @ m oo1 r i@ l ) ((pos "OA%" ) ))
( "immense"  j ( i1 m e1 n s ) ((pos "OA%" ) ))
( "immensely"  a ( i1 m e1 n s l ii ) ((pos "Pu%" ) ))
( "immensities"  n ( i1 m e1 n s i t i z ) ((pos "Mj%" ) ))
( "immensity"  n ( i1 m e1 n s i t ii ) ((pos "M8%" ) ))
( "immerse"  v ( i1 m @@1 s ) ((pos "H2%" ) (subcat "6A" )))
( "immersed"  v ( i1 m @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "immerses"  v ( i1 m @@1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "immersing"  v ( i1 m @@1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "immersion"  n ( i1 m @@1 sh @ n ) ((pos "K6%" ) ))
( "immersions"  n ( i1 m @@1 sh @ n z ) ((pos "Kj%" ) ))
( "immigrant"  n ( i1 m i g r @ n t ) ((pos "K6%" ) ))
( "immigrants"  n ( i1 m i g r @ n t s ) ((pos "Kj%" ) ))
( "immigrate"  v ( i1 m i g r ei t ) ((pos "I2%" ) (subcat "2A" )))
( "immigrated"  v ( i1 m i g r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "immigrates"  v ( i1 m i g r ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "immigrating"  v ( i1 m i g r ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "immigration"  n ( i2 m i g r ei1 sh @ n ) ((pos "M6%" ) ))
( "immigrations"  n ( i2 m i g r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "imminence"  n ( i1 m i n @ n s ) ((pos "L@%" ) ))
( "imminent"  j ( i1 m i n @ n t ) ((pos "OA%" ) ))
( "imminently"  a ( i1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "immobile"  j ( i1 m ou1 b ai l ) ((pos "OA%" ) ))
( "immobility"  n ( i2 m @ b i1 l i t ii ) ((pos "L@%" ) ))
( "immobilization"  n ( i1 m ou2 b i l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "immobilize"  v ( i1 m ou1 b i l ai z ) ((pos "H2%" ) (subcat )))
( "immobilized"  v ( i1 m ou1 b i l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "immobilizes"  v ( i1 m ou1 b i l ai z i z ) ((pos "Ha%" ) (subcat )))
( "immobilizing"  v ( i1 m ou1 b i l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "immoderate"  j ( i1 m o1 d @ r @ t ) ((pos "OA%" ) ))
( "immoderately"  a ( i1 m o1 d @ r @ t l ii ) ((pos "Pu%" ) ))
( "immodest"  j ( i1 m o1 d i s t ) ((pos "OA%" ) ))
( "immodesties"  n ( i1 m o1 d i s t i z ) ((pos "Mj$" ) ))
( "immodestly"  a ( i1 m o1 d i s t l ii ) ((pos "Pu%" ) ))
( "immodesty"  n ( i1 m o1 d i s t ii ) ((pos "M8%" ) ))
( "immolate"  v ( i1 m @ l ei t ) ((pos "H2$" ) (subcat "6A" )))
( "immolated"  v ( i1 m @ l ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "immolates"  v ( i1 m @ l ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "immolating"  v ( i1 m @ l ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "immolation"  n ( i2 m @ l ei1 sh @ n ) ((pos "M6$" ) ))
( "immolations"  n ( i2 m @ l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "immoral"  j ( i1 m o1 r @ l ) ((pos "OA%" ) ))
( "immoralities"  n ( i2 m @ r a1 l i t i z ) ((pos "Mj%" ) ))
( "immorality"  n ( i2 m @ r a1 l i t ii ) ((pos "M8%" ) ))
( "immorally"  a ( i1 m o1 r @ l ii ) ((pos "Pu%" ) ))
( "immortal"  n ( i1 m oo1 t l ) ((pos "K6%" "OA%" ) ))
( "immortality"  n ( i2 m oo t a1 l i t ii ) ((pos "L@%" ) ))
( "immortalize"  v ( i1 m oo1 t @ l ai z ) ((pos "H2%" ) (subcat )))
( "immortalized"  v ( i1 m oo1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "immortalizes"  v ( i1 m oo1 t @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "immortalizing"  v ( i1 m oo1 t @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "immortals"  n ( i1 m oo1 t l z ) ((pos "Kj%" ) ))
( "immovable"  j ( i1 m uu1 v @ b l ) ((pos "OA%" ) ))
( "immovably"  a ( i1 m uu1 v @ b l ii ) ((pos "Pu%" ) ))
( "immune"  j ( i1 m y uu1 n ) ((pos "OA%" ) ))
( "immunity"  n ( i1 m y uu1 n i t ii ) ((pos "L@%" ) ))
( "immunization"  n ( i2 m y u n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "immunize"  v ( i1 m y u n ai z ) ((pos "H2%" ) (subcat "6A" )))
( "immunized"  v ( i1 m y u n ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "immunizes"  v ( i1 m y u n ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "immunizing"  v ( i1 m y u n ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "immunology"  n ( i2 m y u n o1 l @ jh ii ) ((pos "L@%" ) ))
( "immure"  v ( i1 m y u@1 r ) ((pos "H2$" ) (subcat )))
( "immured"  v ( i1 m y u@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "immures"  v ( i1 m y u@1 z ) ((pos "Ha$" ) (subcat )))
( "immuring"  v ( i1 m y u@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "immutability"  n ( i1 m y uu2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "immutable"  j ( i1 m y uu1 t @ b l ) ((pos "OA%" ) ))
( "immutably"  a ( i1 m y uu1 t @ b l ii ) ((pos "Pu%" ) ))
( "imp"  n ( i1 m p ) ((pos "K6%" ) ))
( "impact"  n ( i1 m p a k t ) ((pos "M6%" ) ))
( "impact"  v ( i1 m p a1 k t ) ((pos "H0$" ) ))
( "impacted"  v ( i1 m p a1 k t i d ) ((pos "Hc%" "Hd%" ) ))
( "impacting"  v ( i1 m p a1 k t i ng ) ((pos "Hb$" ) ))
( "impacts"  n ( i1 m p a k t s ) ((pos "Mj%" ) ))
( "impacts"  v ( i1 m p a1 k t s ) ((pos "Ha$" ) ))
( "impair"  v ( i1 m p e@1 r ) ((pos "H0%" ) (subcat )))
( "impaired"  v ( i1 m p e@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "impairing"  v ( i1 m p e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "impairment"  n ( i1 m p e@1 m @ n t ) ((pos "K6%" ) ))
( "impairments"  n ( i1 m p e@1 m @ n t s ) ((pos "Kj$" ) ))
( "impairs"  v ( i1 m p e@1 z ) ((pos "Ha%" ) (subcat )))
( "impala"  n ( i1 m p aa1 l @ ) ((pos "K6$" ) ))
( "impalas"  n ( i1 m p aa1 l @ z ) ((pos "Kj$" ) ))
( "impale"  v ( i1 m p ei1 l ) ((pos "H2%" ) (subcat "6A" )))
( "impaled"  v ( i1 m p ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "impalement"  n ( i1 m p ei1 l m @ n t ) ((pos "K6$" ) ))
( "impalements"  n ( i1 m p ei1 l m @ n t s ) ((pos "Kj$" ) ))
( "impales"  v ( i1 m p ei1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "impaling"  v ( i1 m p ei1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "impalpable"  j ( i1 m p a1 l p @ b l ) ((pos "OA$" ) ))
( "impanel"  v ( i1 m p a1 n l ) ((pos "H4$" ) (subcat )))
( "impanelled"  v ( i1 m p a1 n l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "impanelling"  v ( i1 m p a1 n @ l i ng ) ((pos "Hb$" ) (subcat )))
( "impanels"  v ( i1 m p a1 n l z ) ((pos "Ha$" ) (subcat )))
( "impart"  v ( i1 m p aa1 t ) ((pos "H0%" ) (subcat "6A" )))
( "imparted"  v ( i1 m p aa1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "impartial"  j ( i1 m p aa1 sh l ) ((pos "OA%" ) ))
( "impartiality"  n ( i2 m p aa2 sh i a1 l i t ii ) ((pos "L@%" ) ))
( "impartially"  a ( i1 m p aa1 sh @ l ii ) ((pos "Pu%" ) ))
( "imparting"  v ( i1 m p aa1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "imparts"  v ( i1 m p aa1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "impassable"  j ( i1 m p aa1 s @ b l ) ((pos "OA%" ) ))
( "impasse"  n ( a1 m p aa s ) ((pos "K6%" ) ))
( "impasses"  n ( a1 m p aa s i z ) ((pos "Kj$" ) ))
( "impassioned"  j ( i1 m p a1 sh @ n d ) ((pos "OA%" ) ))
( "impassive"  j ( i1 m p a1 s i v ) ((pos "OA%" ) ))
( "impassively"  a ( i1 m p a1 s i v l ii ) ((pos "Pu%" ) ))
( "impassiveness"  n ( i1 m p a1 s i v n @ s ) ((pos "L@%" ) ))
( "impassivity"  n ( i2 m p a s i1 v i t ii ) ((pos "L@$" ) ))
( "impatience"  n ( i1 m p ei1 sh @ n s ) ((pos "L@%" ) ))
( "impatient"  j ( i1 m p ei1 sh @ n t ) ((pos "OA%" ) ))
( "impatiently"  a ( i1 m p ei1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "impeach"  v ( i1 m p ii1 ch ) ((pos "H1%" ) (subcat "6A" )))
( "impeached"  v ( i1 m p ii1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "impeaches"  v ( i1 m p ii1 ch i z ) ((pos "Ha%" ) (subcat "6A" )))
( "impeaching"  v ( i1 m p ii1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "impeachment"  n ( i1 m p ii1 ch m @ n t ) ((pos "M6%" ) ))
( "impeachments"  n ( i1 m p ii1 ch m @ n t s ) ((pos "Mj%" ) ))
( "impeccable"  j ( i1 m p e1 k @ b l ) ((pos "OA%" ) ))
( "impeccably"  a ( i1 m p e1 k @ b l ii ) ((pos "Pu%" ) ))
( "impecunious"  j ( i2 m p i k y uu1 n i@ s ) ((pos "OA%" ) ))
( "impede"  v ( i1 m p ii1 d ) ((pos "H2%" ) (subcat )))
( "impeded"  v ( i1 m p ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "impedes"  v ( i1 m p ii1 d z ) ((pos "Ha%" ) (subcat )))
( "impediment"  n ( i1 m p e1 d i m @ n t ) ((pos "K6%" ) ))
( "impedimenta"  n ( i1 m p e2 d i m e1 n t @ ) ((pos "Kj%" ) ))
( "impediments"  n ( i1 m p e1 d i m @ n t s ) ((pos "Kj%" ) ))
( "impeding"  v ( i1 m p ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "impel"  v ( i1 m p e1 l ) ((pos "H4%" ) (subcat "14" )))
( "impelled"  v ( i1 m p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "impeller"  n ( i1 m p e1 l @ r ) ((pos "K6$" ) ))
( "impellers"  n ( i1 m p e1 l @ z ) ((pos "Kj$" ) ))
( "impelling"  v ( i1 m p e1 l i ng ) ((pos "Hb%" ) (subcat "14" )))
( "impels"  v ( i1 m p e1 l z ) ((pos "Ha%" ) (subcat "14" )))
( "impend"  v ( i1 m p e1 n d ) ((pos "I0$" ) ))
( "impended"  v ( i1 m p e1 n d i d ) ((pos "Ic$" "Id$" ) ))
( "impending"  v ( i1 m p e1 n d i ng ) ((pos "Ib%" ) ))
( "impends"  v ( i1 m p e1 n d z ) ((pos "Ia$" ) ))
( "impenetrable"  j ( i1 m p e1 n i t r @ b l ) ((pos "OA%" ) ))
( "impenitence"  n ( i1 m p e1 n i t @ n s ) ((pos "L@$" ) ))
( "impenitent"  j ( i1 m p e1 n i t @ n t ) ((pos "OA$" ) ))
( "impenitently"  a ( i1 m p e1 n i t @ n t l ii ) ((pos "Pu$" ) ))
( "imperative"  j ( i1 m p e1 r @ t i v ) ((pos "OA%" ) ))
( "imperatively"  a ( i1 m p e1 r @ t i v l ii ) ((pos "Pu%" ) ))
( "imperceptible"  j ( i2 m p @ s e1 p t @ b l ) ((pos "OA%" ) ))
( "imperceptibly"  a ( i2 m p @ s e1 p t @ b l ii ) ((pos "Pu%" ) ))
( "imperfect"  n ( i1 m p @@1 f i k t ) ((pos "Ki%" "OA%" ) ))
( "imperfection"  n ( i2 m p @ f e1 k sh @ n ) ((pos "M6%" ) ))
( "imperfections"  n ( i2 m p @ f e1 k sh @ n z ) ((pos "Mj%" ) ))
( "imperfectly"  a ( i1 m p @@1 f i k t l ii ) ((pos "Pu%" ) ))
( "imperial"  n ( i1 m p i@1 r i@ l ) ((pos "K6$" "OA%" ) ))
( "imperialism"  n ( i1 m p i@1 r i@ l i z @ m ) ((pos "L@%" ) ))
( "imperialist"  n ( i1 m p i@1 r i@ l i s t ) ((pos "K6%" ) ))
( "imperialistic"  j ( i1 m p i@2 r i@ l i1 s t i k ) ((pos "OA%" ) ))
( "imperialists"  n ( i1 m p i@1 r i@ l i s t s ) ((pos "Kj%" ) ))
( "imperially"  a ( i1 m p i@1 r i@ l ii ) ((pos "Pu%" ) ))
( "imperials"  n ( i1 m p i@1 r i@ l z ) ((pos "Kj$" ) ))
( "imperil"  v ( i1 m p e1 r @ l ) ((pos "H4%" ) (subcat )))
( "imperilled"  v ( i1 m p e1 r @ l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "imperilling"  v ( i1 m p e1 r @ l i ng ) ((pos "Hb%" ) (subcat )))
( "imperils"  v ( i1 m p e1 r @ l z ) ((pos "Ha%" ) (subcat )))
( "imperious"  j ( i1 m p i@1 r i@ s ) ((pos "OA%" ) ))
( "imperiously"  a ( i1 m p i@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "imperiousness"  n ( i1 m p i@1 r i@ s n @ s ) ((pos "L@%" ) ))
( "imperishable"  j ( i1 m p e1 r i sh @ b l ) ((pos "OA%" ) ))
( "impermanence"  n ( i1 m p @@1 m @ n @ n s ) ((pos "L@%" ) ))
( "impermanent"  j ( i1 m p @@1 m @ n @ n t ) ((pos "OA%" ) ))
( "impermeable"  j ( i1 m p @@1 m i@ b l ) ((pos "OA%" ) ))
( "impersonal"  j ( i2 m p @@1 s @ n l ) ((pos "OA%" ) ))
( "impersonally"  a ( i2 m p @@1 s @ n @ l ii ) ((pos "Pu%" ) ))
( "impersonate"  v ( i1 m p @@1 s @ n ei t ) ((pos "H2%" ) (subcat )))
( "impersonated"  v ( i1 m p @@1 s @ n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "impersonates"  v ( i1 m p @@1 s @ n ei t s ) ((pos "Ha%" ) (subcat )))
( "impersonating"  v ( i1 m p @@1 s @ n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "impersonation"  n ( i1 m p @@2 s @ n ei1 sh @ n ) ((pos "M6%" ) ))
( "impersonations"  n ( i1 m p @@2 s @ n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "impersonator"  n ( i1 m p @@1 s @ n ei t @ r ) ((pos "K6%" ) ))
( "impersonators"  n ( i1 m p @@1 s @ n ei t @ z ) ((pos "Kj%" ) ))
( "impertinence"  n ( i1 m p @@1 t i n @ n s ) ((pos "M6%" ) ))
( "impertinences"  n ( i1 m p @@1 t i n @ n s i z ) ((pos "Mj%" ) ))
( "impertinent"  j ( i1 m p @@1 t i n @ n t ) ((pos "OA%" ) ))
( "impertinently"  a ( i1 m p @@1 t i n @ n t l ii ) ((pos "Pu%" ) ))
( "imperturbability"  n ( i2 m p @ t @@2 b @ b i1 l i t ii ) ((pos "L@%" ) ))
( "imperturbable"  j ( i2 m p @ t @@1 b @ b l ) ((pos "OA%" ) ))
( "impervious"  j ( i1 m p @@1 v i@ s ) ((pos "OA%" ) ))
( "impetigo"  n ( i2 m p i t ai1 g ou ) ((pos "L@$" ) ))
( "impetuosities"  n ( i1 m p e2 ch u o1 s i t i z ) ((pos "Mj$" ) ))
( "impetuosity"  n ( i1 m p e2 ch u o1 s i t ii ) ((pos "M8%" ) ))
( "impetuous"  j ( i1 m p e1 ch u@ s ) ((pos "OA%" ) ))
( "impetuously"  a ( i1 m p e1 ch u@ s l ii ) ((pos "Pu%" ) ))
( "impetus"  n ( i1 m p i t @ s ) ((pos "Mi%" ) ))
( "impieties"  n ( i1 m p ai1 @ t i z ) ((pos "Mj$" ) ))
( "impiety"  n ( i1 m p ai1 @ t ii ) ((pos "M8%" ) ))
( "impinge"  v ( i1 m p i1 n jh ) ((pos "I2%" ) (subcat )))
( "impinged"  v ( i1 m p i1 n jh d ) ((pos "Ic%" "Id%" ) (subcat )))
( "impingement"  n ( i1 m p i1 n jh m @ n t ) ((pos "K6$" ) ))
( "impingements"  n ( i1 m p i1 n jh m @ n t s ) ((pos "Kj$" ) ))
( "impinges"  v ( i1 m p i1 n jh i z ) ((pos "Ia%" ) (subcat )))
( "impinging"  v ( i1 m p i1 n jh i ng ) ((pos "Ib%" ) (subcat )))
( "impious"  j ( i1 m p i@ s ) ((pos "OA%" ) ))
( "impiously"  a ( i1 m p i@ s l ii ) ((pos "Pu%" ) ))
( "impish"  j ( i1 m p i sh ) ((pos "OA%" ) ))
( "impishly"  a ( i1 m p i sh l ii ) ((pos "Pu%" ) ))
( "impishness"  n ( i1 m p i sh n @ s ) ((pos "L@%" ) ))
( "implacable"  j ( i1 m p l a1 k @ b l ) ((pos "OA%" ) ))
( "implant"  v ( i1 m p l aa1 n t ) ((pos "H0%" ) (subcat "6A" )))
( "implanted"  v ( i1 m p l aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "implanting"  v ( i1 m p l aa1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "implants"  v ( i1 m p l aa1 n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "implausibility"  n ( i1 m p l oo2 z i b i1 l i t ii ) ((pos "L@%" ) ))
( "implausible"  j ( i1 m p l oo1 z i b l ) ((pos "OA%" ) ))
( "implausibly"  a ( i1 m p l oo1 z i b l ii ) ((pos "Pu%" ) ))
( "implement"  n ( i1 m p l i m @ n t ) ((pos "K6%" ) ))
( "implement"  v ( i1 m p l i m e n t ) ((pos "H0%" ) (subcat )))
( "implementation"  n ( i2 m p l i m e n t ei1 sh @ n ) ((pos "K6%" ) ))
( "implementations"  n ( i2 m p l i m e n t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "implemented"  v ( i1 m p l i m e n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "implementing"  v ( i1 m p l i m e n t i ng ) ((pos "Hb%" ) (subcat )))
( "implements"  n ( i1 m p l i m @ n t s ) ((pos "Kj%" ) ))
( "implements"  v ( i1 m p l i m e n t s ) ((pos "Ha%" ) (subcat )))
( "implicate"  v ( i1 m p l i k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "implicated"  v ( i1 m p l i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "implicates"  v ( i1 m p l i k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "implicating"  v ( i1 m p l i k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "implication"  n ( i2 m p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "implications"  n ( i2 m p l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "implicit"  j ( i1 m p l i1 s i t ) ((pos "OA%" ) ))
( "implicitly"  a ( i1 m p l i1 s i t l ii ) ((pos "Pu%" ) ))
( "implied"  v ( i1 m p l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "implies"  v ( i1 m p l ai1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "implore"  v ( i1 m p l oo1 r ) ((pos "H2%" ) (subcat "6A" "14" )))
( "implored"  v ( i1 m p l oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "implores"  v ( i1 m p l oo1 z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "imploring"  v ( i1 m p l oo1 r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "imploringly"  a ( i1 m p l oo1 r i ng l ii ) ((pos "Pu%" ) ))
( "implosion"  n ( i1 m p l ou1 zh n ) ((pos "M6$" ) ))
( "implosions"  n ( i1 m p l ou1 zh n z ) ((pos "Mj$" ) ))
( "imply"  v ( i1 m p l ai1 ) ((pos "H3%" ) (subcat "6A" )))
( "implying"  v ( i1 m p l ai1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "impolite"  j ( i2 m p @ l ai1 t ) ((pos "OA%" ) ))
( "impolitely"  a ( i2 m p @ l ai1 t l ii ) ((pos "Pu%" ) ))
( "impoliteness"  n ( i2 m p @ l ai1 t n @ s ) ((pos "L@%" ) ))
( "impolitic"  j ( i1 m p o1 l @ t i k ) ((pos "OA%" ) ))
( "imponderable"  n ( i1 m p o1 n d @ r @ b l ) ((pos "K6%" "OA%" ) ))
( "imponderables"  n ( i1 m p o1 n d @ r @ b l z ) ((pos "Kj%" ) ))
( "import"  n ( i1 m p oo t ) ((pos "M6%" ) ))
( "import"  v ( i1 m p oo1 t ) ((pos "H0%" ) (subcat "6A" "9" )))
( "importance"  n ( i1 m p oo1 t n s ) ((pos "L@%" ) ))
( "important"  j ( i1 m p oo1 t n t ) ((pos "OA*" ) ))
( "importantly"  a ( i1 m p oo1 t n t l ii ) ((pos "Pu%" ) ))
( "importation"  n ( i2 m p oo t ei1 sh @ n ) ((pos "M6%" ) ))
( "importations"  n ( i2 m p oo t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "imported"  v ( i1 m p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "importer"  n ( i1 m p oo1 t @ r ) ((pos "K6%" ) ))
( "importers"  n ( i1 m p oo1 t @ z ) ((pos "Kj%" ) ))
( "importing"  v ( i1 m p oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "imports"  n ( i1 m p oo t s ) ((pos "Mj%" ) ))
( "imports"  v ( i1 m p oo1 t s ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "importunate"  j ( i1 m p oo1 ch u n @ t ) ((pos "OA%" ) ))
( "importunately"  a ( i1 m p oo1 ch u n @ t l ii ) ((pos "Pu%" ) ))
( "importune"  v ( i2 m p @ t y uu1 n ) ((pos "H2%" ) (subcat "6A" "9" "14" )))
( "importuned"  v ( i2 m p @ t y uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" )))
( "importunes"  v ( i2 m p @ t y uu1 n z ) ((pos "Ha%" ) (subcat "6A" "9" "14" )))
( "importuning"  v ( i2 m p @ t y uu1 n i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" )))
( "importunity"  n ( i2 m p @ t y uu1 n i t ii ) ((pos "L@%" ) ))
( "impose"  v ( i1 m p ou1 z ) ((pos "J2%" ) (subcat "3A" )))
( "imposed"  v ( i1 m p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "imposes"  v ( i1 m p ou1 z i z ) ((pos "Ja%" ) (subcat "3A" )))
( "imposing"  v ( i1 m p ou1 z i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "imposingly"  a ( i1 m p ou1 z i ng l ii ) ((pos "Pu%" ) ))
( "imposition"  n ( i2 m p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "impositions"  n ( i2 m p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "impossibilities"  n ( i1 m p o2 s @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "impossibility"  n ( i1 m p o2 s @ b i1 l i t ii ) ((pos "M8%" ) ))
( "impossible"  j ( i1 m p o1 s @ b l ) ((pos "OA%" ) ))
( "impossibly"  a ( i1 m p o1 s @ b l ii ) ((pos "Pu%" ) ))
( "impostor"  n ( i1 m p o1 s t @ r ) ((pos "K6%" ) ))
( "impostors"  n ( i1 m p o1 s t @ z ) ((pos "Kj%" ) ))
( "imposture"  n ( i1 m p o1 s ch @ r ) ((pos "M6$" ) ))
( "impostures"  n ( i1 m p o1 s ch @ z ) ((pos "Mj$" ) ))
( "impotence"  n ( i1 m p @ t @ n s ) ((pos "L@%" ) ))
( "impotent"  j ( i1 m p @ t @ n t ) ((pos "OA%" ) ))
( "impotently"  a ( i1 m p @ t @ n t l ii ) ((pos "Pu%" ) ))
( "impound"  v ( i1 m p au1 n d ) ((pos "H0%" ) (subcat )))
( "impounded"  v ( i1 m p au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "impounding"  v ( i1 m p au1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "impounds"  v ( i1 m p au1 n d z ) ((pos "Ha%" ) (subcat )))
( "impoverish"  v ( i1 m p o1 v @ r i sh ) ((pos "H1%" ) (subcat )))
( "impoverished"  v ( i1 m p o1 v @ r i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "impoverishes"  v ( i1 m p o1 v @ r i sh i z ) ((pos "Ha%" ) (subcat )))
( "impoverishing"  v ( i1 m p o1 v @ r i sh i ng ) ((pos "Hb%" ) (subcat )))
( "impoverishment"  n ( i1 m p o1 v @ r i sh m @ n t ) ((pos "K6%" ) ))
( "impoverishments"  n ( i1 m p o1 v @ r i sh m @ n t s ) ((pos "Kj$" ) ))
( "impracticability"  n ( i1 m p r a2 k t i k @ b i1 l i t ii ) ((pos "L@%" ) ))
( "impracticable"  j ( i1 m p r a1 k t i k @ b l ) ((pos "OA%" ) ))
( "impracticableness"  n ( i1 m p r a1 k t i k @ b l n @ s ) ((pos "L@$" ) ))
( "impracticably"  a ( i1 m p r a1 k t i k @ b l ii ) ((pos "Pu%" ) ))
( "impractical"  j ( i1 m p r a1 k t i k l ) ((pos "OA%" ) ))
( "imprecate"  v ( i1 m p r i k ei t ) ((pos "H2$" ) (subcat )))
( "imprecated"  v ( i1 m p r i k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "imprecates"  v ( i1 m p r i k ei t s ) ((pos "Ha$" ) (subcat )))
( "imprecating"  v ( i1 m p r i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "imprecation"  n ( i2 m p r i k ei1 sh @ n ) ((pos "K6$" ) ))
( "imprecations"  n ( i2 m p r i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "imprecise"  j ( i2 m p r i s ai1 s ) ((pos "OA%" ) ))
( "imprecisely"  a ( i2 m p r i s ai1 s l ii ) ((pos "Pu%" ) ))
( "imprecision"  n ( i2 m p r i s i1 zh n ) ((pos "L@%" ) ))
( "impregnability"  n ( i1 m p r e2 g n @ b i1 l i t ii ) ((pos "L@%" ) ))
( "impregnable"  j ( i1 m p r e1 g n @ b l ) ((pos "OA%" ) ))
( "impregnably"  a ( i1 m p r e1 g n @ b l ii ) ((pos "Pu%" ) ))
( "impregnate"  v ( i1 m p r e g n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "impregnated"  v ( i1 m p r e g n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "impregnates"  v ( i1 m p r e g n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "impregnating"  v ( i1 m p r e g n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "impresario"  n ( i2 m p r i s aa1 r i@ @ ) ((pos "K6%" ) ))
( "impresarios"  n ( i2 m p r i s aa1 r i@ u z ) ((pos "Kj%" ) ))
( "impress"  n ( i1 m p r e s ) ((pos "K7$" ) ))
( "impress"  v ( i1 m p r e1 s ) ((pos "H1%" ) (subcat "6A" )))
( "impressed"  v ( i1 m p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "impresses"  n ( i1 m p r e s i z ) ((pos "Kj$" ) ))
( "impresses"  v ( i1 m p r e1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "impressing"  v ( i1 m p r e1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "impression"  n ( i1 m p r e1 sh @ n ) ((pos "K6%" ) ))
( "impressionable"  j ( i1 m p r e1 sh @ n @ b l ) ((pos "OA%" ) ))
( "impressionism"  n ( i1 m p r e1 sh @ n i z @ m ) ((pos "L@%" ) ))
( "impressionist"  n ( i1 m p r e1 sh @ n i s t ) ((pos "K6%" ) ))
( "impressionistic"  j ( i1 m p r e2 sh @ n i1 s t i k ) ((pos "OA%" ) ))
( "impressionists"  n ( i1 m p r e1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "impressions"  n ( i1 m p r e1 sh @ n z ) ((pos "Kj%" ) ))
( "impressive"  j ( i1 m p r e1 s i v ) ((pos "OA%" ) ))
( "impressively"  a ( i1 m p r e1 s i v l ii ) ((pos "Pu%" ) ))
( "impressiveness"  n ( i1 m p r e1 s i v n @ s ) ((pos "L@%" ) ))
( "imprimatur"  n ( i2 m p r i m ei1 t @ r ) ((pos "K6$" ) ))
( "imprimaturs"  n ( i2 m p r i m ei1 t @ z ) ((pos "Kj$" ) ))
( "imprint"  n ( i1 m p r i n t ) ((pos "K6%" ) ))
( "imprint"  v ( i1 m p r i1 n t ) ((pos "H0%" ) (subcat )))
( "imprinted"  v ( i1 m p r i1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "imprinting"  v ( i1 m p r i1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "imprints"  n ( i1 m p r i n t s ) ((pos "Kj%" ) ))
( "imprints"  v ( i1 m p r i1 n t s ) ((pos "Ha%" ) (subcat )))
( "imprison"  v ( i1 m p r i1 z n ) ((pos "H0%" ) (subcat )))
( "imprisoned"  v ( i1 m p r i1 z n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "imprisoning"  v ( i1 m p r i1 z n i ng ) ((pos "Hb%" ) (subcat )))
( "imprisonment"  n ( i1 m p r i1 z n m @ n t ) ((pos "L@%" ) ))
( "imprisons"  v ( i1 m p r i1 z n z ) ((pos "Ha%" ) (subcat )))
( "improbabilities"  n ( i1 m p r o2 b @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "improbability"  n ( i1 m p r o2 b @ b i1 l i t ii ) ((pos "M8%" ) ))
( "improbable"  j ( i1 m p r o1 b @ b l ) ((pos "OA%" ) ))
( "improbably"  a ( i1 m p r o1 b @ b l ii ) ((pos "Pu%" ) ))
( "impromptu"  n ( i1 m p r o1 m p t y uu ) ((pos "K6%" "OA%" "Pu%" ) ))
( "impromptus"  n ( i1 m p r o1 m p t y uu z ) ((pos "Kj$" ) ))
( "improper"  j ( i1 m p r o1 p @ r ) ((pos "OA%" ) ))
( "improperly"  a ( i1 m p r o1 p @ l ii ) ((pos "Pu%" ) ))
( "improprieties"  n ( i2 m p r @ p r ai1 @ t i z ) ((pos "Mj%" ) ))
( "impropriety"  n ( i2 m p r @ p r ai1 @ t ii ) ((pos "M8%" ) ))
( "improve"  v ( i1 m p r uu1 v ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "improved"  v ( i1 m p r uu1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "improvement"  n ( i1 m p r uu1 v m @ n t ) ((pos "M6%" ) ))
( "improvements"  n ( i1 m p r uu1 v m @ n t s ) ((pos "Mj%" ) ))
( "improver"  n ( i1 m p r uu1 v @ r ) ((pos "K6%" ) ))
( "improvers"  n ( i1 m p r uu1 v @ z ) ((pos "Kj%" ) ))
( "improves"  v ( i1 m p r uu1 v z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "improvidence"  n ( i1 m p r o1 v i d @ n s ) ((pos "L@%" ) ))
( "improvident"  j ( i1 m p r o1 v i d @ n t ) ((pos "OA%" ) ))
( "improvidently"  a ( i1 m p r o1 v i d @ n t l ii ) ((pos "Pu%" ) ))
( "improving"  v ( i1 m p r uu1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "improvisation"  n ( i2 m p r @ v ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "improvisations"  n ( i2 m p r @ v ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "improvise"  v ( i1 m p r @ v ai z ) ((pos "J2%" ) (subcat "2A" )))
( "improvised"  v ( i1 m p r @ v ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "improvises"  v ( i1 m p r @ v ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "improvising"  v ( i1 m p r @ v ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "imprudence"  n ( i1 m p r uu1 d n s ) ((pos "M6%" ) ))
( "imprudences"  n ( i1 m p r uu1 d n s i z ) ((pos "Mj$" ) ))
( "imprudent"  j ( i1 m p r uu1 d n t ) ((pos "OA%" ) ))
( "imprudently"  a ( i1 m p r uu1 d n t l ii ) ((pos "Pu%" ) ))
( "imps"  n ( i1 m p s ) ((pos "Kj%" ) ))
( "impudence"  n ( i1 m p y u d @ n s ) ((pos "L@%" ) ))
( "impudent"  j ( i1 m p y u d @ n t ) ((pos "OA%" ) ))
( "impudently"  a ( i1 m p y u d @ n t l ii ) ((pos "Pu%" ) ))
( "impugn"  v ( i1 m p y uu1 n ) ((pos "H0$" ) (subcat )))
( "impugned"  v ( i1 m p y uu1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "impugning"  v ( i1 m p y uu1 n i ng ) ((pos "Hb$" ) (subcat )))
( "impugns"  v ( i1 m p y uu1 n z ) ((pos "Ha$" ) (subcat )))
( "impulse"  n ( i1 m p uh l s ) ((pos "M6%" ) ))
( "impulse-bought"  v ( i2 m p @ l s - b oo1 t ) ((pos "Jc$" "Jd$" ) ))
( "impulse-buy"  v ( i2 m p @ l s - b ai1 ) ((pos "J5$" ) ))
( "impulse-buying"  v ( i2 m p @ l s - b ai1 i ng ) ((pos "Jb$" ) ))
( "impulse-buys"  v ( i2 m p @ l s - b ai1 z ) ((pos "Ja$" ) ))
( "impulses"  n ( i1 m p uh l s i z ) ((pos "Mj%" ) ))
( "impulsion"  n ( i1 m p uh1 l sh @ n ) ((pos "M6$" ) ))
( "impulsions"  n ( i1 m p uh1 l sh @ n z ) ((pos "Mj$" ) ))
( "impulsive"  j ( i1 m p uh1 l s i v ) ((pos "OA%" ) ))
( "impulsively"  a ( i1 m p uh1 l s i v l ii ) ((pos "Pu%" ) ))
( "impulsiveness"  n ( i1 m p uh1 l s i v n @ s ) ((pos "L@%" ) ))
( "impunity"  n ( i1 m p y uu1 n i t ii ) ((pos "L@%" ) ))
( "impure"  j ( i1 m p y u@1 r ) ((pos "OA%" ) ))
( "impurities"  n ( i1 m p y uu1 r i t i z ) ((pos "Mj%" ) ))
( "impurity"  n ( i1 m p y uu1 r i t ii ) ((pos "M8%" ) ))
( "imputation"  n ( i2 m p y u t ei1 sh @ n ) ((pos "M6%" ) ))
( "imputations"  n ( i2 m p y u t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "impute"  v ( i1 m p y uu1 t ) ((pos "H2%" ) (subcat )))
( "imputed"  v ( i1 m p y uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "imputes"  v ( i1 m p y uu1 t s ) ((pos "Ha%" ) (subcat )))
( "imputing"  v ( i1 m p y uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "in"  n ( i n ) ((pos "K6%" "P+*" "T-*" ) ))
( "in loco parentis"  a ( i1 n - l ou2 k ou - p @ r e1 n t i s ) ((pos "Pu%" ) ))
( "in memoriam"  nil ( i2 n - m @ m oo1 r i@ m ) ((pos "Z-%" ) ))
( "in situ"  a ( i2 n - s i1 t y uu ) ((pos "Pu%" ) ))
( "in toto"  a ( i1 n - t ou1 t ou ) ((pos "Pu$" ) ))
( "in-chief"  j ( i1 n - ch ii1 f ) ((pos "Ot%" ) ))
( "in-fighting"  n ( i1 n - f ai t i ng ) ((pos "L@%" ) ))
( "in-laws"  n ( i1 n - l oo z ) ((pos "Kj%" ) ))
( "in-patient"  n ( i1 n - p ei sh @ n t ) ((pos "K6%" ) ))
( "in-patients"  n ( i1 n - p ei sh @ n t s ) ((pos "Kj%" ) ))
( "in-service"  j ( i1 n - s @@ v i s ) ((pos "Oq%" ) ))
( "in-tray"  n ( i1 n - t r ei ) ((pos "K6%" ) ))
( "in-trays"  n ( i1 n - t r ei z ) ((pos "Kj%" ) ))
( "inability"  n ( i2 n @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inaccessibility"  n ( i2 n a k s e2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inaccessible"  j ( i2 n a k s e1 s @ b l ) ((pos "OA%" ) ))
( "inaccuracies"  n ( i1 n a1 k y @ r @ s i z ) ((pos "Mj%" ) ))
( "inaccuracy"  n ( i1 n a1 k y @ r @ s ii ) ((pos "M8%" ) ))
( "inaccurate"  j ( i1 n a1 k y u r @ t ) ((pos "OA%" ) ))
( "inaccurately"  a ( i1 n a1 k y u r @ t l ii ) ((pos "Pu%" ) ))
( "inaction"  n ( i1 n a1 k sh @ n ) ((pos "L@%" ) ))
( "inactivate"  v ( i2 n a1 k t i v ei t ) ((pos "H2%" ) ))
( "inactivated"  v ( i2 n a1 k t i v ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "inactivates"  v ( i2 n a1 k t i v ei t s ) ((pos "Ha$" ) ))
( "inactivating"  v ( i2 n a1 k t i v ei t i ng ) ((pos "Hb$" ) ))
( "inactive"  j ( i1 n a1 k t i v ) ((pos "OA%" ) ))
( "inactivity"  n ( i2 n a k t i1 v i t ii ) ((pos "L@%" ) ))
( "inadequacies"  n ( i1 n a1 d i k w @ s i z ) ((pos "Mj%" ) ))
( "inadequacy"  n ( i1 n a1 d i k w @ s ii ) ((pos "M8%" ) ))
( "inadequate"  j ( i1 n a1 d i k w @ t ) ((pos "OA%" ) ))
( "inadequately"  a ( i1 n a1 d i k w @ t l ii ) ((pos "Pu%" ) ))
( "inadmissible"  j ( i2 n @ d m i1 s @ b l ) ((pos "OA%" ) ))
( "inadvertence"  n ( i2 n @ d v @@1 t @ n s ) ((pos "M6$" ) ))
( "inadvertences"  n ( i2 n @ d v @@1 t @ n s i z ) ((pos "Mj$" ) ))
( "inadvertent"  j ( i2 n @ d v @@1 t @ n t ) ((pos "OA%" ) ))
( "inadvertently"  a ( i2 n @ d v @@1 t @ n t l ii ) ((pos "Pu%" ) ))
( "inadvisability"  n ( i2 n @ d v ai2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inadvisable"  j ( i2 n @ d v ai1 z @ b l ) ((pos "OA%" ) ))
( "inadvisably"  a ( i2 n @ d v ai1 z @ b l ii ) ((pos "Pu%" ) ))
( "inalienable"  j ( i2 n ei1 l i@ n @ b l ) ((pos "OA%" ) ))
( "inane"  j ( i1 n ei1 n ) ((pos "OA%" ) ))
( "inanely"  a ( i1 n ei1 n l ii ) ((pos "Pu%" ) ))
( "inanimate"  j ( i1 n a1 n i m @ t ) ((pos "OA%" ) ))
( "inanities"  n ( i1 n a1 n i t i z ) ((pos "Mj%" ) ))
( "inanition"  n ( i2 n @ n i1 sh @ n ) ((pos "L@$" ) ))
( "inanity"  n ( i1 n a1 n i t ii ) ((pos "M8%" ) ))
( "inapplicable"  j ( i1 n a1 p l i k @ b l ) ((pos "OA%" ) ))
( "inappreciable"  j ( i2 n @ p r ii1 sh @ b l ) ((pos "OA$" ) ))
( "inappropriate"  j ( i2 n @ p r ou1 p r i@ t ) ((pos "OA%" ) ))
( "inappropriately"  a ( i2 n @ p r ou1 p r i@ t l ii ) ((pos "Pu%" ) ))
( "inappropriateness"  n ( i2 n @ p r ou1 p r i@ t n i s ) ((pos "L@%" ) ))
( "inapt"  j ( i1 n a1 p t ) ((pos "OA$" ) ))
( "inaptitude"  n ( i1 n a1 p t i t y uu d ) ((pos "L@$" ) ))
( "inarticulate"  j ( i2 n aa t i1 k y u l @ t ) ((pos "OA%" ) ))
( "inasmuch as"  a ( i2 n @ z m uh1 ch - @ z ) ((pos "Pu%" ) ))
( "inattention"  n ( i2 n @ t e1 n sh @ n ) ((pos "L@%" ) ))
( "inattentive"  j ( i2 n @ t e1 n t i v ) ((pos "OA%" ) ))
( "inaudibility"  n ( i1 n oo2 d @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inaudible"  j ( i1 n oo1 d @ b l ) ((pos "OA%" ) ))
( "inaugural"  n ( i1 n oo1 g y u r @ l ) ((pos "K6$" "OA%" ) ))
( "inaugurals"  n ( i1 n oo1 g y u r @ l z ) ((pos "Kj$" ) ))
( "inaugurate"  v ( i1 n oo1 g y u r ei t ) ((pos "H2%" ) (subcat )))
( "inaugurated"  v ( i1 n oo1 g y u r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inaugurates"  v ( i1 n oo1 g y u r ei t s ) ((pos "Ha%" ) (subcat )))
( "inaugurating"  v ( i1 n oo1 g y u r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "inauguration"  n ( i1 n oo2 g y u r ei1 sh @ n ) ((pos "M6%" ) ))
( "inaugurations"  n ( i1 n oo2 g y u r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "inauspicious"  j ( i2 n oo s p i1 sh @ s ) ((pos "OA%" ) ))
( "inauspiciously"  a ( i2 n oo s p i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "inboard"  j ( i1 n b oo d ) ((pos "OA$" ) ))
( "inborn"  j ( i2 n b oo1 n ) ((pos "OA%" ) ))
( "inbound"  j ( i1 n b au n d ) ((pos "OA$" ) ))
( "inbred"  j ( i2 n b r e1 d ) ((pos "OA%" ) ))
( "inbreeding"  n ( i2 n b r ii1 d i ng ) ((pos "L@%" ) ))
( "inbuilt"  j ( i2 n b i1 l t ) ((pos "OA%" ) ))
( "incalculable"  j ( i1 n k a1 l k y u l @ b l ) ((pos "OA%" ) ))
( "incandescence"  n ( i2 n k a n d e1 s n s ) ((pos "L@%" ) ))
( "incandescent"  j ( i2 n k a n d e1 s n t ) ((pos "OA%" ) ))
( "incantation"  n ( i2 n k a n t ei1 sh @ n ) ((pos "M6%" ) ))
( "incantations"  n ( i2 n k a n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "incapability"  n ( i1 n k ei2 p @ b i1 l i t ii ) ((pos "L@%" ) ))
( "incapable"  j ( i1 n k ei1 p @ b l ) ((pos "OA%" ) ))
( "incapacitate"  v ( i2 n k @ p a1 s i t ei t ) ((pos "H2%" ) (subcat "6A" )))
( "incapacitated"  v ( i2 n k @ p a1 s i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "incapacitates"  v ( i2 n k @ p a1 s i t ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "incapacitating"  v ( i2 n k @ p a1 s i t ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "incapacity"  n ( i2 n k @ p a1 s i t ii ) ((pos "L@%" ) ))
( "incarcerate"  v ( i1 n k aa1 s @ r ei t ) ((pos "H2%" ) (subcat )))
( "incarcerated"  v ( i1 n k aa1 s @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "incarcerates"  v ( i1 n k aa1 s @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "incarcerating"  v ( i1 n k aa1 s @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "incarceration"  n ( i1 n k aa2 s @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "incarcerations"  n ( i1 n k aa2 s @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "incarnate"  j ( i1 n k aa1 n @ t ) ((pos "OA%" ) ))
( "incarnate"  v ( i1 n k aa1 n ei t ) ((pos "H2$" ) (subcat )))
( "incarnated"  v ( i1 n k aa1 n ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "incarnates"  v ( i1 n k aa1 n ei t s ) ((pos "Ha$" ) (subcat )))
( "incarnating"  v ( i1 n k aa1 n ei t i ng ) ((pos "Hb$" ) (subcat )))
( "incarnation"  n ( i2 n k aa n ei1 sh @ n ) ((pos "K6%" ) ))
( "incarnations"  n ( i2 n k aa n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "incautious"  j ( i1 n k oo1 sh @ s ) ((pos "OA%" ) ))
( "incautiously"  a ( i1 n k oo1 sh @ s l ii ) ((pos "Pu%" ) ))
( "incendiaries"  n ( i1 n s e1 n d i@ r i z ) ((pos "Kj%" ) ))
( "incendiarism"  n ( i1 n s e1 n d i@ r i z @ m ) ((pos "L@$" ) ))
( "incendiary"  n ( i1 n s e1 n d i@ r ii ) ((pos "K8%" ) ))
( "incense"  n ( i1 n s e n s ) ((pos "L@%" ) ))
( "incense"  v ( i1 n s e1 n s ) ((pos "H2%" ) (subcat )))
( "incensed"  v ( i1 n s e1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "incenses"  v ( i1 n s e1 n s i z ) ((pos "Ha%" ) (subcat )))
( "incensing"  v ( i1 n s e1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "incentive"  n ( i1 n s e1 n t i v ) ((pos "M6%" ) ))
( "incentives"  n ( i1 n s e1 n t i v z ) ((pos "Mj%" ) ))
( "inception"  n ( i1 n s e1 p sh @ n ) ((pos "K6%" ) ))
( "inceptions"  n ( i1 n s e1 p sh @ n z ) ((pos "Kj$" ) ))
( "incertitude"  n ( i1 n s @@1 t i t y uu d ) ((pos "L@$" ) ))
( "incessant"  j ( i1 n s e1 s n t ) ((pos "OA%" ) ))
( "incessantly"  a ( i1 n s e1 s n t l ii ) ((pos "Pu%" ) ))
( "incest"  n ( i1 n s e s t ) ((pos "L@%" ) ))
( "incestuous"  j ( i1 n s e1 s t y u@ s ) ((pos "OA%" ) ))
( "inch"  v ( i1 n ch ) ((pos "J1%" "K7%" ) (subcat "2C" "15A" )))
( "inched"  v ( i1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "15A" )))
( "inches"  v ( i1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "15A" )))
( "inching"  v ( i1 n ch i ng ) ((pos "Jb%" ) (subcat "2C" "15A" )))
( "inchoate"  j ( i1 n k ou1 i t ) ((pos "OA$" ) ))
( "inchoative"  j ( i1 n k ou1 @ t i v ) ((pos "OA$" ) ))
( "incidence"  n ( i1 n s i d @ n s ) ((pos "K6%" ) ))
( "incidences"  n ( i1 n s i d @ n s i z ) ((pos "Kj$" ) ))
( "incident"  n ( i1 n s i d @ n t ) ((pos "K6%" "OA%" ) ))
( "incidental"  j ( i2 n s i d e1 n t l ) ((pos "OA%" ) ))
( "incidentally"  a ( i2 n s i d e1 n t @ l ii ) ((pos "Pu%" ) ))
( "incidents"  n ( i1 n s i d @ n t s ) ((pos "Kj%" ) ))
( "incinerate"  v ( i1 n s i1 n @ r ei t ) ((pos "H2%" ) (subcat )))
( "incinerated"  v ( i1 n s i1 n @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "incinerates"  v ( i1 n s i1 n @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "incinerating"  v ( i1 n s i1 n @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "incineration"  n ( i1 n s i2 n @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "incinerator"  n ( i1 n s i1 n @ r ei t @ r ) ((pos "K6%" ) ))
( "incinerators"  n ( i1 n s i1 n @ r ei t @ z ) ((pos "Kj%" ) ))
( "incipient"  j ( i1 n s i1 p i@ n t ) ((pos "OA%" ) ))
( "incise"  v ( i1 n s ai1 z ) ((pos "H2$" ) (subcat )))
( "incised"  v ( i1 n s ai1 z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "incises"  v ( i1 n s ai1 z i z ) ((pos "Ha$" ) (subcat )))
( "incising"  v ( i1 n s ai1 z i ng ) ((pos "Hb$" ) (subcat )))
( "incision"  n ( i1 n s i1 zh n ) ((pos "M6%" ) ))
( "incisions"  n ( i1 n s i1 zh n z ) ((pos "Mj%" ) ))
( "incisive"  j ( i1 n s ai1 s i v ) ((pos "OA%" ) ))
( "incisively"  a ( i1 n s ai1 s i v l ii ) ((pos "Pu%" ) ))
( "incisor"  n ( i1 n s ai1 z @ r ) ((pos "K6%" ) ))
( "incisors"  n ( i1 n s ai1 z @ z ) ((pos "Kj%" ) ))
( "incite"  v ( i1 n s ai1 t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "incited"  v ( i1 n s ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "incitement"  n ( i1 n s ai1 t m @ n t ) ((pos "M6%" ) ))
( "incitements"  n ( i1 n s ai1 t m @ n t s ) ((pos "Mj%" ) ))
( "incites"  v ( i1 n s ai1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "inciting"  v ( i1 n s ai1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "incivilities"  n ( i2 n s i v i1 l i t i z ) ((pos "Mj$" ) ))
( "incivility"  n ( i2 n s i v i1 l i t ii ) ((pos "M8$" ) ))
( "incl"  n ( i1 n k l uu1 d i ng ) ((pos "Y~%" ) ))
( "inclemency"  n ( i1 n k l e1 m @ n s ii ) ((pos "L@%" ) ))
( "inclement"  j ( i1 n k l e1 m @ n t ) ((pos "OA%" ) ))
( "inclination"  n ( i2 n k l i n ei1 sh @ n ) ((pos "M6%" ) ))
( "inclinations"  n ( i2 n k l i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "incline"  n ( i1 n k l ai n ) ((pos "K6%" ) ))
( "incline"  v ( i1 n k l ai1 n ) ((pos "J2%" ) (subcat "2A" "3A" "4C" "6A" "15A" )))
( "inclined"  v ( i1 n k l ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4C" "6A" "15A" )))
( "inclines"  n ( i1 n k l ai n z ) ((pos "Kj%" ) ))
( "inclines"  v ( i1 n k l ai1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "4C" "6A" "15A" )))
( "inclining"  v ( i1 n k l ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4C" "6A" "15A" )))
( "inclose"  v ( i1 n k l ou1 z ) ((pos "H2$" ) (subcat "6A" )))
( "inclosed"  v ( i1 n k l ou1 z d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "incloses"  v ( i1 n k l ou1 z i z ) ((pos "Ha$" ) (subcat "6A" )))
( "inclosing"  v ( i1 n k l ou1 z i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "inclosure"  n ( i1 n k l ou1 zh @ r ) ((pos "M6$" ) ))
( "inclosures"  n ( i1 n k l ou1 zh @ z ) ((pos "Mj$" ) ))
( "include"  v ( i1 n k l uu1 d ) ((pos "H2%" ) (subcat "6A" )))
( "included"  v ( i1 n k l uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "includes"  v ( i1 n k l uu1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "including"  v ( i1 n k l uu1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inclusion"  n ( i1 n k l uu1 zh n ) ((pos "L@%" ) ))
( "inclusive"  j ( i1 n k l uu1 s i v ) ((pos "OA%" ) ))
( "inclusively"  a ( i1 n k l uu1 s i v l ii ) ((pos "Pu%" ) ))
( "incognito"  j ( i2 n k o g n ii1 t ou ) ((pos "OA%" "Pu%" ) ))
( "incoherence"  n ( i2 n k ou h i@1 r @ n s ) ((pos "L@%" ) ))
( "incoherent"  j ( i2 n k ou h i@1 r @ n t ) ((pos "OA%" ) ))
( "incoherently"  a ( i2 n k ou h i@1 r @ n t l ii ) ((pos "Pu%" ) ))
( "incombustible"  j ( i2 n k @ m b uh1 s t @ b l ) ((pos "OA$" ) ))
( "income"  n ( i1 ng k uh m ) ((pos "K6%" ) ))
( "income-tax"  n ( i1 ng k @ m - t a k s ) ((pos "M7%" ) ))
( "income-taxes"  n ( i1 ng k @ m - t a k s i z ) ((pos "Mj%" ) ))
( "incomes"  n ( i1 ng k uh m z ) ((pos "Kj%" ) ))
( "incoming"  j ( i2 n k uh1 m i ng ) ((pos "OA%" ) ))
( "incommensurable"  j ( i2 n k @ m e1 n sh @ r @ b l ) ((pos "OA%" ) ))
( "incommensurate"  j ( i2 n k @ m e1 n sh @ r @ t ) ((pos "OA$" ) ))
( "incommode"  v ( i2 n k @ m ou1 d ) ((pos "H2$" ) (subcat )))
( "incommoded"  v ( i2 n k @ m ou1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "incommodes"  v ( i2 n k @ m ou1 d z ) ((pos "Ha$" ) (subcat )))
( "incommoding"  v ( i2 n k @ m ou1 d i ng ) ((pos "Hb$" ) (subcat )))
( "incommunicado"  j ( i2 n k @ m y uu2 n i k aa1 d ou ) ((pos "OA%" ) ))
( "incomparable"  j ( i1 n k o1 m p r @ b l ) ((pos "OA%" ) ))
( "incomparably"  a ( i1 n k o1 m p r @ b l ii ) ((pos "Pu%" ) ))
( "incompatibilities"  n ( i2 n k @ m p a2 t @ b i1 l i t i z ) ((pos "Mj$" ) ))
( "incompatibility"  n ( i2 n k @ m p a2 t @ b i1 l i t ii ) ((pos "M8%" ) ))
( "incompatible"  j ( i2 n k @ m p a1 t @ b l ) ((pos "OA%" ) ))
( "incompetence"  n ( i1 n k o1 m p i t @ n s ) ((pos "L@%" ) ))
( "incompetency"  n ( i1 n k o1 m p i t @ n s ii ) ((pos "L@$" ) ))
( "incompetent"  j ( i1 n k o1 m p i t @ n t ) ((pos "OA%" ) ))
( "incompetently"  a ( i1 n k o1 m p i t @ n t l ii ) ((pos "Pu%" ) ))
( "incomplete"  j ( i2 n k @ m p l ii1 t ) ((pos "OA%" ) ))
( "incompletely"  a ( i2 n k @ m p l ii1 t l ii ) ((pos "Pu%" ) ))
( "incomprehensibility"  n ( i1 n k o2 m p r i h e2 n s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "incomprehensible"  j ( i2 n k o2 m p r i h e1 n s @ b l ) ((pos "OA%" ) ))
( "incomprehension"  n ( i1 n k o2 m p r i h e1 n sh @ n ) ((pos "L@%" ) ))
( "incompressible"  j ( i1 n k @ m p r e1 s @ b l ) ((pos "OA$" ) ))
( "inconceivable"  j ( i2 n k @ n s ii1 v @ b l ) ((pos "OA%" ) ))
( "inconclusive"  j ( i2 n k @ n k l uu1 s i v ) ((pos "OA%" ) ))
( "inconclusively"  a ( i2 n k @ n k l uu1 s i v l ii ) ((pos "Pu%" ) ))
( "incongruities"  n ( i2 n k o ng g r uu1 i t i z ) ((pos "Mj%" ) ))
( "incongruity"  n ( i2 n k o ng g r uu1 i t ii ) ((pos "M8%" ) ))
( "incongruous"  j ( i1 n k o1 ng g r u@ s ) ((pos "OA%" ) ))
( "incongruously"  a ( i1 n k o1 ng g r u@ s l ii ) ((pos "Pu%" ) ))
( "inconsequent"  j ( i1 n k o1 n s i k w @ n t ) ((pos "OA$" ) ))
( "inconsequential"  j ( i1 n k o2 n s i k w e1 n sh l ) ((pos "OA%" ) ))
( "inconsequentially"  a ( i1 n k o2 n s i k w e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "inconsequently"  a ( i1 n k o1 n s i k w @ n t l ii ) ((pos "Pu%" ) ))
( "inconsiderable"  j ( i2 n k @ n s i1 d r @ b l ) ((pos "OA%" ) ))
( "inconsiderate"  j ( i2 n k @ n s i1 d @ r @ t ) ((pos "OA%" ) ))
( "inconsiderately"  a ( i2 n k @ n s i1 d @ r @ t l ii ) ((pos "Pu%" ) ))
( "inconsistencies"  n ( i2 n k @ n s i1 s t @ n s i z ) ((pos "Mj%" ) ))
( "inconsistency"  n ( i2 n k @ n s i1 s t @ n s ii ) ((pos "M8%" ) ))
( "inconsistent"  j ( i2 n k @ n s i1 s t @ n t ) ((pos "OA%" ) ))
( "inconsistently"  a ( i2 n k @ n s i1 s t @ n t l ii ) ((pos "Pu%" ) ))
( "inconsolable"  j ( i2 n k @ n s ou1 l @ b l ) ((pos "OA%" ) ))
( "inconspicuous"  j ( i2 n k @ n s p i1 k y u@ s ) ((pos "OA%" ) ))
( "inconspicuously"  a ( i2 n k @ n s p i1 k y u@ s l ii ) ((pos "Pu%" ) ))
( "inconstancies"  n ( i1 n k o1 n s t @ n s i z ) ((pos "Mj$" ) ))
( "inconstancy"  n ( i1 n k o1 n s t @ n s ii ) ((pos "M8%" ) ))
( "inconstant"  j ( i1 n k o1 n s t @ n t ) ((pos "OA%" ) ))
( "incontestable"  j ( i2 n k @ n t e1 s t @ b l ) ((pos "OA%" ) ))
( "incontinence"  n ( i1 n k o1 n t i n @ n s ) ((pos "L@%" ) ))
( "incontinent"  j ( i1 n k o1 n t i n @ n t ) ((pos "OA%" ) ))
( "incontrovertible"  j ( i1 n k o2 n t r @ v @@1 t @ b l ) ((pos "OA%" ) ))
( "inconvenience"  v ( i2 n k @ n v ii1 n i@ n s ) ((pos "H2%" "M6%" ) (subcat )))
( "inconvenienced"  v ( i2 n k @ n v ii1 n i@ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inconveniences"  v ( i2 n k @ n v ii1 n i@ n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "inconveniencing"  v ( i2 n k @ n v ii1 n i@ n s i ng ) ((pos "Hb%" ) (subcat )))
( "inconvenient"  j ( i2 n k @ n v ii1 n i@ n t ) ((pos "OA%" ) ))
( "inconveniently"  a ( i2 n k @ n v ii1 n i@ n t l ii ) ((pos "Pu%" ) ))
( "inconvertibility"  n ( i2 n k @ n v @@2 t @ b i1 l i t ii ) ((pos "L@$" ) ))
( "inconvertible"  j ( i2 n k @ n v @@1 t @ b l ) ((pos "OA$" ) ))
( "incorporate"  j ( i1 n k oo1 p @ r @ t ) ((pos "OA$" ) ))
( "incorporate"  v ( i1 n k oo1 p @ r ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "14" )))
( "incorporated"  v ( i1 n k oo1 p @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" )))
( "incorporates"  v ( i1 n k oo1 p @ r ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "14" )))
( "incorporating"  v ( i1 n k oo1 p @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "incorporation"  n ( i1 n k oo2 p @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "incorporeal"  j ( i2 n k oo p oo1 r i@ l ) ((pos "OA$" ) ))
( "incorrect"  j ( i2 n k @ r e1 k t ) ((pos "OA%" ) ))
( "incorrectly"  a ( i2 n k @ r e1 k t l ii ) ((pos "Pu%" ) ))
( "incorrectness"  n ( i2 n k @ r e1 k t n @ s ) ((pos "L@%" ) ))
( "incorrigible"  j ( i1 n k o1 r i jh @ b l ) ((pos "OA%" ) ))
( "incorruptibility"  n ( i2 n k @ r uh2 p t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "incorruptible"  j ( i2 n k @ r uh1 p t @ b l ) ((pos "OA%" ) ))
( "increase"  n ( i1 ng k r ii s ) ((pos "M6%" ) ))
( "increase"  v ( i1 n k r ii1 s ) ((pos "J2%" ) (subcat "2A" )))
( "increased"  v ( i1 n k r ii1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "increases"  n ( i1 ng k r ii s i z ) ((pos "Mj%" ) ))
( "increases"  v ( i1 n k r ii1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "increasing"  v ( i1 n k r ii1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "increasingly"  a ( i1 n k r ii1 s i ng l ii ) ((pos "Pu%" ) ))
( "incredibility"  n ( i1 n k r e2 d @ b i1 l i t ii ) ((pos "L@%" ) ))
( "incredible"  j ( i1 n k r e1 d @ b l ) ((pos "OA%" ) ))
( "incredibly"  a ( i1 n k r e1 d @ b l ii ) ((pos "Pu%" ) ))
( "incredulity"  n ( i2 n k r i d y uu1 l i t ii ) ((pos "L@%" ) ))
( "incredulous"  j ( i1 n k r e1 d y u l @ s ) ((pos "OA%" ) ))
( "incredulously"  a ( i1 n k r e1 d y u l @ s l ii ) ((pos "Pu%" ) ))
( "increment"  n ( i1 n k r @ m @ n t ) ((pos "M6%" ) ))
( "incremental"  j ( i2 n k r @ m e1 n t l ) ((pos "OA%" ) ))
( "increments"  n ( i1 n k r @ m @ n t s ) ((pos "Mj%" ) ))
( "incriminate"  v ( i1 n k r i1 m i n ei t ) ((pos "H2%" ) (subcat )))
( "incriminated"  v ( i1 n k r i1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "incriminates"  v ( i1 n k r i1 m i n ei t s ) ((pos "Ha%" ) (subcat )))
( "incriminating"  v ( i1 n k r i1 m i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "incrustation"  n ( i2 n k r uh s t ei1 sh @ n ) ((pos "M6$" ) ))
( "incrustations"  n ( i2 n k r uh s t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "incubate"  v ( i1 n k y u b ei t ) ((pos "J2%" ) (subcat "2A" )))
( "incubated"  v ( i1 n k y u b ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "incubates"  v ( i1 n k y u b ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "incubating"  v ( i1 n k y u b ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "incubation"  n ( i2 n k y u b ei1 sh @ n ) ((pos "K6%" ) ))
( "incubations"  n ( i2 n k y u b ei1 sh @ n z ) ((pos "Kj%" ) ))
( "incubator"  n ( i1 n k y u b ei t @ r ) ((pos "K6%" ) ))
( "incubators"  n ( i1 n k y u b ei t @ z ) ((pos "Kj%" ) ))
( "incubi"  n ( i1 ng k y u b ai ) ((pos "Kj$" ) ))
( "incubus"  n ( i1 ng k y u b @ s ) ((pos "K7$" ) ))
( "incubuses"  n ( i1 ng k y u b @ s i z ) ((pos "Kj$" ) ))
( "inculcate"  v ( i1 n k uh l k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "inculcated"  v ( i1 n k uh l k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inculcates"  v ( i1 n k uh l k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inculcating"  v ( i1 n k uh l k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inculpate"  v ( i1 n k uh l p ei t ) ((pos "H2%" ) (subcat )))
( "inculpated"  v ( i1 n k uh l p ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "inculpates"  v ( i1 n k uh l p ei t s ) ((pos "Ha$" ) (subcat )))
( "inculpating"  v ( i1 n k uh l p ei t i ng ) ((pos "Hb$" ) (subcat )))
( "incumbencies"  n ( i1 n k uh1 m b @ n s i z ) ((pos "Kj$" ) ))
( "incumbency"  n ( i1 n k uh1 m b @ n s ii ) ((pos "K8$" ) ))
( "incumbent"  n ( i1 n k uh1 m b @ n t ) ((pos "K6%" "OA%" ) ))
( "incumbents"  n ( i1 n k uh1 m b @ n t s ) ((pos "Kj%" ) ))
( "incur"  v ( i1 n k @@1 r ) ((pos "H4%" ) (subcat )))
( "incurable"  n ( i1 n k y u@1 r @ b l ) ((pos "K6%" "OA%" ) ))
( "incurables"  n ( i1 n k y u@1 r @ b l z ) ((pos "Kj%" ) ))
( "incurably"  a ( i1 n k y u@1 r @ b l ii ) ((pos "Pu%" ) ))
( "incurious"  j ( i1 n k y u@1 r i@ s ) ((pos "OA%" ) ))
( "incurred"  v ( i1 n k @@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "incurring"  v ( i1 n k @@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "incurs"  v ( i1 n k @@1 z ) ((pos "Ha%" ) (subcat )))
( "incursion"  n ( i1 n k @@1 sh @ n ) ((pos "K6%" ) ))
( "incursions"  n ( i1 n k @@1 sh @ n z ) ((pos "Kj%" ) ))
( "incurved"  j ( i2 n k @@1 v d ) ((pos "OA$" ) ))
( "indebted"  j ( i1 n d e1 t i d ) ((pos "OA%" ) ))
( "indebtedness"  n ( i1 n d e1 t i d n @ s ) ((pos "L@%" ) ))
( "indecencies"  n ( i1 n d ii1 s @ n s i z ) ((pos "Mj%" ) ))
( "indecency"  n ( i1 n d ii1 s @ n s ii ) ((pos "M8%" ) ))
( "indecent"  j ( i1 n d ii1 s n t ) ((pos "OA%" ) ))
( "indecently"  a ( i1 n d ii1 s n t l ii ) ((pos "Pu%" ) ))
( "indecipherable"  j ( i2 n d i s ai1 f r @ b l ) ((pos "OA%" ) ))
( "indecision"  n ( i2 n d i s i1 zh n ) ((pos "L@%" ) ))
( "indecisive"  j ( i2 n d i s ai1 s i v ) ((pos "OA%" ) ))
( "indecisively"  a ( i2 n d i s ai1 s i v l ii ) ((pos "Pu%" ) ))
( "indecorous"  j ( i1 n d e1 k @ r @ s ) ((pos "OA%" ) ))
( "indecorously"  a ( i1 n d e1 k @ r @ s l ii ) ((pos "Pu%" ) ))
( "indecorum"  n ( i2 n d i k oo1 r @ m ) ((pos "L@$" ) ))
( "indeed"  a ( i1 n d ii1 d ) ((pos "Pu%" ) ))
( "indefatigable"  j ( i2 n d i f a1 t i g @ b l ) ((pos "OA%" ) ))
( "indefeasible"  j ( i2 n d i f ii1 z @ b l ) ((pos "OA$" ) ))
( "indefensible"  j ( i2 n d i f e1 n s @ b l ) ((pos "OA%" ) ))
( "indefinable"  j ( i2 n d i f ai1 n @ b l ) ((pos "OA%" ) ))
( "indefinite"  j ( i1 n d e1 f i n @ t ) ((pos "OA%" ) ))
( "indefinitely"  a ( i1 n d e1 f i n @ t l ii ) ((pos "Pu%" ) ))
( "indelible"  j ( i1 n d e1 l @ b l ) ((pos "OA%" ) ))
( "indelibly"  a ( i1 n d e1 l @ b l ii ) ((pos "Pu%" ) ))
( "indelicacies"  n ( i1 n d e1 l i k @ s i z ) ((pos "Mj%" ) ))
( "indelicacy"  n ( i1 n d e1 l i k @ s ii ) ((pos "M8%" ) ))
( "indelicate"  j ( i1 n d e1 l i k @ t ) ((pos "OA%" ) ))
( "indemnification"  n ( i1 n d e2 m n i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "indemnifications"  n ( i1 n d e2 m n i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "indemnified"  v ( i1 n d e1 m n i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "indemnifies"  v ( i1 n d e1 m n i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "indemnify"  v ( i1 n d e1 m n i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "indemnifying"  v ( i1 n d e1 m n i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "indemnities"  n ( i1 n d e1 m n i t i z ) ((pos "Mj%" ) ))
( "indemnity"  n ( i1 n d e1 m n i t ii ) ((pos "M8%" ) ))
( "indent"  n ( i1 n d e n t ) ((pos "K6%" ) ))
( "indent"  v ( i1 n d e1 n t ) ((pos "J0%" ) (subcat "3A" )))
( "indentation"  n ( i2 n d e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "indentations"  n ( i2 n d e n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "indented"  v ( i1 n d e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "indenting"  v ( i1 n d e1 n t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "indents"  n ( i1 n d e n t s ) ((pos "Kj$" ) ))
( "indents"  v ( i1 n d e1 n t s ) ((pos "Ja%" ) (subcat "3A" )))
( "indenture"  v ( i1 n d e1 n ch @ r ) ((pos "H2%" "K6%" ) (subcat )))
( "indentured"  v ( i1 n d e1 n ch @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "indentures"  v ( i1 n d e1 n ch @ z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "indenturing"  v ( i1 n d e1 n ch @ r i ng ) ((pos "Hb$" ) (subcat )))
( "independence"  n ( i2 n d i p e1 n d @ n s ) ((pos "L@%" ) ))
( "independent"  n ( i2 n d i p e1 n d @ n t ) ((pos "K6%" "OA%" ) ))
( "independently"  a ( i2 n d i p e1 n d @ n t l ii ) ((pos "Pu%" ) ))
( "independents"  n ( i2 n d i p e1 n d @ n t s ) ((pos "Kj%" ) ))
( "indescribable"  j ( i2 n d i s k r ai1 b @ b l ) ((pos "OA%" ) ))
( "indescribably"  a ( i2 n d i s k r ai1 b @ b l ii ) ((pos "Pu%" ) ))
( "indestructibility"  n ( i2 n d i s t r uh2 k t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "indestructible"  j ( i2 n d i s t r uh1 k t @ b l ) ((pos "OA%" ) ))
( "indeterminable"  j ( i2 n d i t @@1 m i n @ b l ) ((pos "OA%" ) ))
( "indeterminably"  a ( i2 n d i t @@1 m i n @ b l ii ) ((pos "Pu%" ) ))
( "indeterminacy"  n ( i2 n d i t @@1 m i n @ s ii ) ((pos "L@%" ) ))
( "indeterminate"  j ( i2 n d i t @@1 m i n @ t ) ((pos "OA%" ) ))
( "index"  v ( i1 n d e k s ) ((pos "H1%" "K7%" ) (subcat )))
( "indexed"  v ( i1 n d e k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "indexer"  n ( i1 n d e k s @ r ) ((pos "K6%" ) ))
( "indexers"  n ( i1 n d e k s @ z ) ((pos "Kj%" ) ))
( "indexes"  v ( i1 n d e k s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "indexing"  v ( i1 n d e k s i ng ) ((pos "Hb%" ) (subcat )))
( "indicate"  v ( i1 n d i k ei t ) ((pos "H2%" ) (subcat "6A" "9" )))
( "indicated"  v ( i1 n d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "indicates"  v ( i1 n d i k ei t s ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "indicating"  v ( i1 n d i k ei t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "indication"  n ( i2 n d i k ei1 sh @ n ) ((pos "M6%" ) ))
( "indications"  n ( i2 n d i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "indicative"  j ( i1 n d i1 k @ t i v ) ((pos "OA%" ) ))
( "indicator"  n ( i1 n d i k ei t @ r ) ((pos "K6%" ) ))
( "indicators"  n ( i1 n d i k ei t @ z ) ((pos "Kj%" ) ))
( "indices"  n ( i1 n d i s ii z ) ((pos "Kj%" ) ))
( "indict"  v ( i1 n d ai1 t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "indictable"  j ( i1 n d ai1 t @ b l ) ((pos "OA%" ) ))
( "indicted"  v ( i1 n d ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "indicting"  v ( i1 n d ai1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "indictment"  n ( i1 n d ai1 t m @ n t ) ((pos "M6%" ) ))
( "indictments"  n ( i1 n d ai1 t m @ n t s ) ((pos "Mj%" ) ))
( "indicts"  v ( i1 n d ai1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "indifference"  n ( i1 n d i1 f r @ n s ) ((pos "L@%" ) ))
( "indifferent"  j ( i1 n d i1 f r @ n t ) ((pos "OA%" ) ))
( "indifferently"  a ( i1 n d i1 f r @ n t l ii ) ((pos "Pu%" ) ))
( "indigence"  n ( i1 n d i jh @ n s ) ((pos "L@$" ) ))
( "indigenous"  j ( i1 n d i1 jh i n @ s ) ((pos "OA%" ) ))
( "indigent"  j ( i1 n d i jh @ n t ) ((pos "OA$" ) ))
( "indigestible"  j ( i2 n d i jh @1 s t @ b l ) ((pos "OA%" ) ))
( "indigestion"  n ( i2 n d i jh e1 s ch @ n ) ((pos "L@%" ) ))
( "indignant"  j ( i1 n d i1 g n @ n t ) ((pos "OA%" ) ))
( "indignantly"  a ( i1 n d i1 g n @ n t l ii ) ((pos "Pu%" ) ))
( "indignation"  n ( i2 n d i g n ei1 sh @ n ) ((pos "L@%" ) ))
( "indignities"  n ( i1 n d i1 g n i t i z ) ((pos "Mj%" ) ))
( "indignity"  n ( i1 n d i1 g n i t ii ) ((pos "M8%" ) ))
( "indigo"  n ( i1 n d i g ou ) ((pos "L@%" ) ))
( "indirect"  j ( i2 n d i r e1 k t ) ((pos "OA%" ) ))
( "indirectly"  a ( i2 n d i r e1 k t l ii ) ((pos "Pu%" ) ))
( "indirectness"  n ( i2 n d i r e1 k t n @ s ) ((pos "L@%" ) ))
( "indiscernible"  j ( i2 n d i s @@1 n @ b l ) ((pos "OA%" ) ))
( "indiscipline"  n ( i1 n d i1 s i p l i n ) ((pos "L@%" ) ))
( "indiscreet"  j ( i2 n d i s k r ii1 t ) ((pos "OA%" ) ))
( "indiscreetly"  a ( i2 n d i s k r ii1 t l ii ) ((pos "Pu%" ) ))
( "indiscrete"  j ( i2 n d i s k r ii1 t ) ((pos "OA$" ) ))
( "indiscretion"  n ( i2 n d i s k r e1 sh @ n ) ((pos "M6%" ) ))
( "indiscretions"  n ( i2 n d i s k r e1 sh @ n z ) ((pos "Mj%" ) ))
( "indiscriminate"  j ( i2 n d i s k r i1 m i n @ t ) ((pos "OA%" ) ))
( "indiscriminately"  a ( i2 n d i s k r i1 m i n @ t l ii ) ((pos "Pu%" ) ))
( "indispensability"  n ( i2 n d i s p e2 n s @ b i1 l i t ii ) ((pos "L@$" ) ))
( "indispensable"  j ( i2 n d i s p e1 n s @ b l ) ((pos "OA%" ) ))
( "indisposed"  j ( i2 n d i s p ou1 z d ) ((pos "OA%" ) ))
( "indisposition"  n ( i2 n d i s p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "indispositions"  n ( i2 n d i s p @ z i1 sh @ n z ) ((pos "Mj$" ) ))
( "indisputable"  j ( i2 n d i s p y uu1 t @ b l ) ((pos "OA%" ) ))
( "indissoluble"  j ( i2 n d i s o1 l y u b l ) ((pos "OA%" ) ))
( "indistinct"  j ( i2 n d i s t i1 ng k t ) ((pos "OA%" ) ))
( "indistinctly"  a ( i2 n d i s t i1 ng k t l ii ) ((pos "Pu%" ) ))
( "indistinctness"  n ( i2 n d i s t i1 ng k t n @ s ) ((pos "L@$" ) ))
( "indistinguishable"  j ( i2 n d i s t i1 ng g w i sh @ b l ) ((pos "OA%" ) ))
( "indite"  v ( i1 n d ai1 t ) ((pos "H2$" ) (subcat )))
( "indited"  v ( i1 n d ai1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "indites"  v ( i1 n d ai1 t s ) ((pos "Ha$" ) (subcat )))
( "inditing"  v ( i1 n d ai1 t i ng ) ((pos "Hb$" ) (subcat )))
( "individual"  n ( i2 n d i v i1 jh u@ l ) ((pos "K6%" "OA%" ) ))
( "individualism"  n ( i2 n d i v i1 jh u@ l i z @ m ) ((pos "L@%" ) ))
( "individualist"  n ( i2 n d i v i1 jh u@ l i s t ) ((pos "K6%" ) ))
( "individualistic"  j ( i2 n d i v i2 jh u@ l i1 s t i k ) ((pos "OA%" ) ))
( "individualists"  n ( i2 n d i v i1 jh u@ l i s t s ) ((pos "Kj%" ) ))
( "individualities"  n ( i2 n d i v i2 jh u a1 l i t i z ) ((pos "Mj$" ) ))
( "individuality"  n ( i2 n d i v i2 jh u a1 l i t ii ) ((pos "M8%" ) ))
( "individualize"  v ( i2 n d i v i1 jh u@ l ai z ) ((pos "H2$" ) (subcat )))
( "individualized"  v ( i2 n d i v i1 jh u@ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "individualizes"  v ( i2 n d i v i1 jh u@ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "individualizing"  v ( i2 n d i v i1 jh u@ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "individually"  a ( i2 n d i v i1 jh u@ l ii ) ((pos "Pu%" ) ))
( "individuals"  n ( i2 n d i v i1 jh u@ l z ) ((pos "Kj%" ) ))
( "indivisible"  j ( i2 n d i v i1 z @ b l ) ((pos "OA%" ) ))
( "indoctrinate"  v ( i1 n d o1 k t r i n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "indoctrinated"  v ( i1 n d o1 k t r i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "indoctrinates"  v ( i1 n d o1 k t r i n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "indoctrinating"  v ( i1 n d o1 k t r i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "indoctrination"  n ( i1 n d o2 k t r i n ei1 sh @ n ) ((pos "L@%" ) ))
( "indolence"  n ( i1 n d @ l @ n s ) ((pos "L@%" ) ))
( "indolent"  j ( i1 n d @ l @ n t ) ((pos "OA%" ) ))
( "indolently"  a ( i1 n d @ l @ n t l ii ) ((pos "Pu%" ) ))
( "indomitable"  j ( i1 n d o1 m i t @ b l ) ((pos "OA%" ) ))
( "indoor"  j ( i1 n d oo r ) ((pos "OA%" ) ))
( "indoors"  a ( i2 n d oo1 z ) ((pos "Pu%" ) ))
( "indorse"  v ( i1 n d oo1 s ) ((pos "H2$" ) (subcat )))
( "indorsed"  v ( i1 n d oo1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "indorses"  v ( i1 n d oo1 s i z ) ((pos "Ha$" ) (subcat )))
( "indorsing"  v ( i1 n d oo1 s i ng ) ((pos "Hb$" ) (subcat )))
( "indrawn"  j ( i2 n d r oo1 n ) ((pos "OA%" ) ))
( "indubitable"  j ( i1 n d y uu1 b i t @ b l ) ((pos "OA%" ) ))
( "indubitably"  a ( i1 n d y uu1 b i t @ b l ii ) ((pos "Pu%" ) ))
( "induce"  v ( i1 n d y uu1 s ) ((pos "H2%" ) (subcat "6A" "14" )))
( "induced"  v ( i1 n d y uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "inducement"  n ( i1 n d y uu1 s m @ n t ) ((pos "M6%" ) ))
( "inducements"  n ( i1 n d y uu1 s m @ n t s ) ((pos "Mj%" ) ))
( "induces"  v ( i1 n d y uu1 s i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "inducing"  v ( i1 n d y uu1 s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "induct"  v ( i1 n d uh1 k t ) ((pos "H0$" ) (subcat "6A" "14" )))
( "inducted"  v ( i1 n d uh1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "14" )))
( "inducting"  v ( i1 n d uh1 k t i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "induction"  n ( i1 n d uh1 k sh @ n ) ((pos "K6%" ) ))
( "inductions"  n ( i1 n d uh1 k sh @ n z ) ((pos "Kj%" ) ))
( "inductive"  j ( i1 n d uh1 k t i v ) ((pos "OA%" ) ))
( "inducts"  v ( i1 n d uh1 k t s ) ((pos "Ha$" ) (subcat "6A" "14" )))
( "indue"  v ( i1 n d y uu1 ) ((pos "H2$" ) (subcat )))
( "indued"  v ( i1 n d y uu1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "indues"  v ( i1 n d y uu1 z ) ((pos "Ha$" ) (subcat )))
( "induing"  v ( i1 n d y uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "indulge"  v ( i1 n d uh1 l jh ) ((pos "J2%" ) (subcat "3A" )))
( "indulged"  v ( i1 n d uh1 l jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "indulgence"  n ( i1 n d uh1 l jh @ n s ) ((pos "M6%" ) ))
( "indulgences"  n ( i1 n d uh1 l jh @ n s i z ) ((pos "Mj%" ) ))
( "indulgent"  j ( i1 n d uh1 l jh @ n t ) ((pos "OA%" ) ))
( "indulgently"  a ( i1 n d uh1 l jh @ n t l ii ) ((pos "Pu%" ) ))
( "indulges"  v ( i1 n d uh1 l jh i z ) ((pos "Ja%" ) (subcat "3A" )))
( "indulging"  v ( i1 n d uh1 l jh i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "industrial"  j ( i1 n d uh1 s t r i@ l ) ((pos "OA%" ) ))
( "industrialism"  n ( i1 n d uh1 s t r i@ l i z @ m ) ((pos "L@%" ) ))
( "industrialist"  n ( i1 n d uh1 s t r i@ l i s t ) ((pos "K6%" ) ))
( "industrialists"  n ( i1 n d uh1 s t r i@ l i s t s ) ((pos "Kj%" ) ))
( "industrialization"  n ( i1 n d uh2 s t r i@ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "industrialized"  j ( i1 n d uh1 s t r i@ l ai z d ) ((pos "OA%" ) ))
( "industries"  n ( i1 n d @ s t r i z ) ((pos "Mj%" ) ))
( "industrious"  j ( i1 n d uh1 s t r i@ s ) ((pos "OA%" ) ))
( "industriously"  a ( i1 n d uh1 s t r i@ s l ii ) ((pos "Pu%" ) ))
( "industry"  n ( i1 n d @ s t r ii ) ((pos "M8%" ) ))
( "indwelling"  j ( i2 n d w e1 l i ng ) ((pos "OA$" ) ))
( "inebriate"  n ( i1 n ii1 b r i@ t ) ((pos "K6$" "OA%" ) ))
( "inebriate"  v ( i1 n ii1 b r i ei t ) ((pos "H2$" ) (subcat )))
( "inebriated"  v ( i1 n ii1 b r i ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "inebriates"  n ( i1 n ii1 b r i@ t s ) ((pos "Kj$" ) ))
( "inebriates"  v ( i1 n ii1 b r i ei t s ) ((pos "Ha$" ) (subcat )))
( "inebriating"  v ( i1 n ii1 b r i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "inebriation"  n ( i1 n ii2 b r i ei1 sh @ n ) ((pos "K6%" ) ))
( "inebriations"  n ( i1 n ii2 b r i ei1 sh @ n z ) ((pos "Kj$" ) ))
( "inebriety"  n ( i2 n i b r ai1 @ t ii ) ((pos "L@$" ) ))
( "inedible"  j ( i1 n e1 d @ b l ) ((pos "OA%" ) ))
( "ineffable"  j ( i1 n e1 f @ b l ) ((pos "OA%" ) ))
( "ineffably"  a ( i1 n e1 f @ b l ii ) ((pos "Pu%" ) ))
( "ineffective"  j ( i2 n i f e1 k t i v ) ((pos "OA%" ) ))
( "ineffectively"  a ( i2 n i f e1 k t i v l ii ) ((pos "Pu%" ) ))
( "ineffectiveness"  n ( i2 n i f e1 k t i v n @ s ) ((pos "L@%" ) ))
( "ineffectual"  j ( i2 n i f e1 k ch u@ l ) ((pos "OA%" ) ))
( "ineffectuality"  n ( i2 n i f e2 k ch u a1 l i t ii ) ((pos "L@%" ) ))
( "ineffectually"  a ( i2 n i f e1 k ch u@ l ii ) ((pos "Pu%" ) ))
( "inefficiencies"  n ( i2 n i f i1 sh @ n s i z ) ((pos "Kj%" ) ))
( "inefficiency"  n ( i2 n i f i1 sh @ n s ii ) ((pos "M8%" ) ))
( "inefficient"  j ( i2 n i f i1 sh @ n t ) ((pos "OA%" ) ))
( "inefficiently"  a ( i2 n i f i1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "inelastic"  j ( i2 n i l a1 s t i k ) ((pos "OA%" ) ))
( "inelegance"  n ( i2 n e1 l i g @ n s ) ((pos "K6%" ) ))
( "inelegances"  n ( i2 n e1 l i g @ n s i z ) ((pos "Kj$" ) ))
( "inelegant"  j ( i2 n e1 l i g @ n t ) ((pos "OA%" ) ))
( "inelegantly"  a ( i2 n e1 l i g @ n t l ii ) ((pos "Pu%" ) ))
( "ineligibility"  n ( i1 n e2 l i jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "ineligible"  j ( i1 n e1 l i jh @ b l ) ((pos "OA%" ) ))
( "ineluctable"  j ( i2 n i l uh1 k t @ b l ) ((pos "OA%" ) ))
( "inept"  j ( i1 n e1 p t ) ((pos "OA%" ) ))
( "ineptitude"  n ( i1 n e1 p t i t y uu d ) ((pos "M6%" ) ))
( "ineptitudes"  n ( i1 n e1 p t i t y uu d z ) ((pos "Mj$" ) ))
( "ineptly"  a ( i1 n e1 p t l ii ) ((pos "Pu%" ) ))
( "inequalities"  n ( i2 n i k w o1 l i t i z ) ((pos "Mj%" ) ))
( "inequality"  n ( i2 n i k w o1 l i t ii ) ((pos "M8%" ) ))
( "inequitable"  j ( i1 n e1 k w i t @ b l ) ((pos "OA%" ) ))
( "inequities"  n ( i1 n e1 k w i t i z ) ((pos "Mj$" ) ))
( "inequity"  n ( i1 n e1 k w i t ii ) ((pos "M8$" ) ))
( "ineradicable"  j ( i2 n i r a1 d i k @ b l ) ((pos "OA%" ) ))
( "inert"  j ( i1 n @@1 t ) ((pos "OA%" ) ))
( "inertia"  n ( i1 n @@1 sh @ ) ((pos "L@%" ) ))
( "inescapable"  j ( i2 n i s k ei1 p @ b l ) ((pos "OA%" ) ))
( "inescapably"  a ( i2 n i s k ei1 p @ b l ii ) ((pos "Pu%" ) ))
( "inessential"  n ( i2 n i s e1 n sh l ) ((pos "K6$" "OA%" ) ))
( "inessentials"  n ( i2 n i s e1 n sh l z ) ((pos "Kj$" ) ))
( "inestimable"  j ( i1 n e1 s t i m @ b l ) ((pos "OA%" ) ))
( "inevitability"  n ( i1 n e2 v i t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inevitable"  j ( i1 n e1 v i t @ b l ) ((pos "OA%" ) ))
( "inevitably"  a ( i1 n e1 v i t @ b l ii ) ((pos "Pu%" ) ))
( "inexact"  j ( i2 n i g z a1 k t ) ((pos "OA%" ) ))
( "inexactitude"  n ( i2 n i g z a1 k t i t y uu d ) ((pos "M6%" ) ))
( "inexactitudes"  n ( i2 n i g z a1 k t i t y uu d z ) ((pos "Mj$" ) ))
( "inexcusable"  j ( i2 n i k s k y uu1 z @ b l ) ((pos "OA%" ) ))
( "inexhaustible"  j ( i2 n i g z oo1 s t @ b l ) ((pos "OA%" ) ))
( "inexorable"  j ( i2 n e1 k s @ r @ b l ) ((pos "OA%" ) ))
( "inexorably"  a ( i2 n e1 k s @ r @ b l ii ) ((pos "Pu%" ) ))
( "inexpediency"  n ( i2 n i k s p ii1 d i@ n s ii ) ((pos "L@$" ) ))
( "inexpedient"  j ( i2 n i k s p ii1 d i@ n t ) ((pos "OA$" ) ))
( "inexpensive"  j ( i2 n i k s p e1 n s i v ) ((pos "OA%" ) ))
( "inexpensively"  a ( i2 n i k s p e1 n s i v l ii ) ((pos "Pu%" ) ))
( "inexperience"  n ( i2 n i k s p i@1 r i@ n s ) ((pos "L@%" ) ))
( "inexperienced"  j ( i2 n i k s p i@1 r i@ n s t ) ((pos "OA%" ) ))
( "inexpert"  j ( i1 n e1 k s p @@ t ) ((pos "OA%" ) ))
( "inexpertly"  a ( i1 n e1 k s p @@ t l ii ) ((pos "Pu%" ) ))
( "inexpiable"  j ( i1 n e1 k s p i@ b l ) ((pos "OA$" ) ))
( "inexplicable"  j ( i2 n i k s p l i1 k @ b l ) ((pos "OA%" ) ))
( "inexpressible"  j ( i2 n i k s p r e1 s @ b l ) ((pos "OA%" ) ))
( "inextinguishable"  j ( i2 n i k s t i1 ng g w i sh @ b l ) ((pos "OA%" ) ))
( "inextricable"  j ( i2 n i k s t r i1 k @ b l ) ((pos "OA%" ) ))
( "inextricably"  a ( i2 n i k s t r i1 k @ b l ii ) ((pos "Pu%" ) ))
( "infallibility"  n ( i1 n f a2 l @ b i1 l i t ii ) ((pos "L@%" ) ))
( "infallible"  j ( i1 n f a1 l @ b l ) ((pos "OA%" ) ))
( "infamies"  n ( i1 n f @ m i z ) ((pos "Mj%" ) ))
( "infamous"  j ( i1 n f @ m @ s ) ((pos "OA%" ) ))
( "infamy"  n ( i1 n f @ m ii ) ((pos "M8%" ) ))
( "infancies"  n ( i1 n f @ n s i z ) ((pos "Mj$" ) ))
( "infancy"  n ( i1 n f @ n s ii ) ((pos "M8%" ) ))
( "infant"  n ( i1 n f @ n t ) ((pos "K6%" ) ))
( "infanticide"  n ( i1 n f a1 n t i s ai d ) ((pos "L@%" ) ))
( "infantile"  j ( i1 n f @ n t ai l ) ((pos "OA%" ) ))
( "infantilism"  n ( i1 n f a1 n t i l i z @ m ) ((pos "L@%" ) ))
( "infantries"  n ( i1 n f @ n t r i z ) ((pos "Mj%" ) ))
( "infantry"  n ( i1 n f @ n t r ii ) ((pos "M8%" ) ))
( "infantryman"  n ( i1 n f @ n t r i m @ n ) ((pos "Ki%" ) ))
( "infantrymen"  n ( i1 n f @ n t r i m @ n ) ((pos "Kj%" ) ))
( "infants"  n ( i1 n f @ n t s ) ((pos "Kj%" ) ))
( "infatuate"  v ( i1 n f a1 ch u ei t ) ((pos "H2$" ) ))
( "infatuated"  v ( i1 n f a1 ch u ei t i d ) ((pos "Hc$" "Hd%" ) ))
( "infatuates"  v ( i1 n f a1 ch u ei t s ) ((pos "Ha$" ) ))
( "infatuating"  v ( i1 n f a1 ch u ei t i ng ) ((pos "Hb$" ) ))
( "infatuation"  n ( i1 n f a2 ch u ei1 sh @ n ) ((pos "M6%" ) ))
( "infatuations"  n ( i1 n f a2 ch u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "infect"  v ( i1 n f e1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "infected"  v ( i1 n f e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "infecting"  v ( i1 n f e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "infection"  n ( i1 n f e1 k sh @ n ) ((pos "M6%" ) ))
( "infections"  n ( i1 n f e1 k sh @ n z ) ((pos "Mj%" ) ))
( "infectious"  j ( i1 n f e1 k sh @ s ) ((pos "OA%" ) ))
( "infects"  v ( i1 n f e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "infer"  v ( i1 n f @@1 r ) ((pos "H4%" ) (subcat "6A" "9" )))
( "inference"  n ( i1 n f @ r @ n s ) ((pos "M6%" ) ))
( "inferences"  n ( i1 n f @ r @ n s i z ) ((pos "Mj%" ) ))
( "inferential"  j ( i2 n f @ r e1 n sh l ) ((pos "OA$" ) ))
( "inferior"  n ( i1 n f i@1 r i@ r ) ((pos "K6%" "OA%" ) ))
( "inferiority"  n ( i1 n f i@2 r i o1 r i t ii ) ((pos "L@%" ) ))
( "inferiors"  n ( i1 n f i@1 r i@ z ) ((pos "Kj%" ) ))
( "infernal"  j ( i1 n f @@1 n l ) ((pos "OA%" ) ))
( "infernally"  a ( i1 n f @@1 n @ l ii ) ((pos "Pu%" ) ))
( "inferno"  n ( i1 n f @@1 n ou ) ((pos "K6%" ) ))
( "infernos"  n ( i1 n f @@1 n ou z ) ((pos "Kj%" ) ))
( "inferred"  v ( i1 n f @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "inferring"  v ( i1 n f @@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "infers"  v ( i1 n f @@1 z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "infertile"  j ( i1 n f @@1 t ai l ) ((pos "OA%" ) ))
( "infertility"  n ( i2 n f @ t i1 l i t ii ) ((pos "L@%" ) ))
( "infest"  v ( i1 n f e1 s t ) ((pos "H0%" ) (subcat )))
( "infestation"  n ( i2 n f e s t ei1 sh @ n ) ((pos "M6%" ) ))
( "infestations"  n ( i2 n f e s t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "infested"  v ( i1 n f e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "infesting"  v ( i1 n f e1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "infests"  v ( i1 n f e1 s t s ) ((pos "Ha%" ) (subcat )))
( "infidel"  n ( i1 n f i d @ l ) ((pos "K6%" ) ))
( "infidelities"  n ( i2 n f i d e1 l i t i z ) ((pos "Mj%" ) ))
( "infidelity"  n ( i2 n f i d e1 l i t ii ) ((pos "M8%" ) ))
( "infidels"  n ( i1 n f i d @ l z ) ((pos "Kj%" ) ))
( "infield"  n ( i1 n f ii l d ) ((pos "L@%" ) ))
( "infiltrate"  v ( i1 n f i l t r ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "infiltrated"  v ( i1 n f i l t r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "infiltrates"  v ( i1 n f i l t r ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "infiltrating"  v ( i1 n f i l t r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "infiltration"  n ( i2 n f i l t r ei1 sh @ n ) ((pos "L@%" ) ))
( "infinite"  j ( i1 n f i n @ t ) ((pos "OA%" ) ))
( "infinitely"  a ( i1 n f i n @ t l ii ) ((pos "Pu%" ) ))
( "infinitesimal"  j ( i2 n f i n i t e1 s i m l ) ((pos "OA%" ) ))
( "infinitive"  n ( i1 n f i1 n @ t i v ) ((pos "K6%" "OA%" ) ))
( "infinitives"  n ( i1 n f i1 n @ t i v z ) ((pos "Kj%" ) ))
( "infinitude"  n ( i1 n f i1 n i t y uu d ) ((pos "M6%" ) ))
( "infinitudes"  n ( i1 n f i1 n i t y uu d z ) ((pos "Mj$" ) ))
( "infinity"  n ( i1 n f i1 n i t ii ) ((pos "L@%" ) ))
( "infirm"  j ( i1 n f @@1 m ) ((pos "OA%" ) ))
( "infirmaries"  n ( i1 n f @@1 m @ r i z ) ((pos "Kj%" ) ))
( "infirmary"  n ( i1 n f @@1 m @ r ii ) ((pos "K8%" ) ))
( "infirmities"  n ( i1 n f @@1 m i t i z ) ((pos "Mj%" ) ))
( "infirmity"  n ( i1 n f @@1 m i t ii ) ((pos "M8%" ) ))
( "inflame"  v ( i1 n f l ei1 m ) ((pos "J2$" ) (subcat "2A" )))
( "inflamed"  v ( i1 n f l ei1 m d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "inflames"  v ( i1 n f l ei1 m z ) ((pos "Ja$" ) (subcat "2A" )))
( "inflaming"  v ( i1 n f l ei1 m i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "inflammable"  j ( i1 n f l a1 m @ b l ) ((pos "OA%" ) ))
( "inflammation"  n ( i2 n f l @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "inflammations"  n ( i2 n f l @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "inflammatory"  j ( i1 n f l a1 m @ t r ii ) ((pos "OA%" ) ))
( "inflatable"  j ( i1 n f l ei1 t @ b l ) ((pos "OA%" ) ))
( "inflate"  v ( i1 n f l ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "inflated"  v ( i1 n f l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inflates"  v ( i1 n f l ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inflating"  v ( i1 n f l ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inflation"  n ( i1 n f l ei1 sh @ n ) ((pos "L@%" ) ))
( "inflationary"  j ( i1 n f l ei1 sh @ n r ii ) ((pos "OA%" ) ))
( "inflect"  v ( i1 n f l e1 k t ) ((pos "H0%" ) (subcat )))
( "inflected"  v ( i1 n f l e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inflecting"  v ( i1 n f l e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "inflection"  n ( i1 n f l e1 k sh @ n ) ((pos "M6%" ) ))
( "inflectional"  j ( i1 n f l e1 k sh @ n @ l ) ((pos "OA%" ) ))
( "inflections"  n ( i1 n f l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "inflects"  v ( i1 n f l e1 k t s ) ((pos "Ha%" ) (subcat )))
( "inflexibility"  n ( i1 n f l e2 k s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "inflexible"  j ( i1 n f l e1 k s @ b l ) ((pos "OA%" ) ))
( "inflexibly"  a ( i1 n f l e1 k s @ b l ii ) ((pos "Pu%" ) ))
( "inflexion"  n ( i1 n f l e1 k sh @ n ) ((pos "M6$" ) ))
( "inflexions"  n ( i1 n f l e1 k sh @ n z ) ((pos "Mj$" ) ))
( "inflict"  v ( i1 n f l i1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "inflicted"  v ( i1 n f l i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inflicting"  v ( i1 n f l i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "infliction"  n ( i1 n f l i1 k sh @ n ) ((pos "M6$" ) ))
( "inflictions"  n ( i1 n f l i1 k sh @ n z ) ((pos "Mj$" ) ))
( "inflicts"  v ( i1 n f l i1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inflorescence"  n ( i2 n f l oo r e1 s n s ) ((pos "L@$" ) ))
( "inflow"  n ( i1 n f l ou ) ((pos "M6$" ) ))
( "inflows"  n ( i1 n f l ou z ) ((pos "Mj$" ) ))
( "influence"  v ( i1 n f l u@ n s ) ((pos "H2%" "M6%" ) (subcat )))
( "influenced"  v ( i1 n f l u@ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "influences"  v ( i1 n f l u@ n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "influencing"  v ( i1 n f l u@ n s i ng ) ((pos "Hb%" ) (subcat )))
( "influential"  j ( i2 n f l u e1 n sh l ) ((pos "OA%" ) ))
( "influentially"  a ( i2 n f l u e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "influenza"  n ( i2 n f l u e1 n z @ ) ((pos "L@%" ) ))
( "influx"  n ( i1 n f l uh k s ) ((pos "M7%" ) ))
( "influxes"  n ( i1 n f l uh k s i z ) ((pos "Mj$" ) ))
( "info"  n ( i1 n f ou ) ((pos "L@%" ) ))
( "inform"  v ( i1 n f oo1 m ) ((pos "J0%" ) (subcat "3A" "6A" "11" "14" )))
( "informal"  j ( i1 n f oo1 m l ) ((pos "OA%" ) ))
( "informalities"  n ( i2 n f oo m a1 l i t i z ) ((pos "Mj%" ) ))
( "informality"  n ( i2 n f oo m a1 l i t ii ) ((pos "M8%" ) ))
( "informally"  a ( i1 n f oo1 m @ l ii ) ((pos "Pu%" ) ))
( "informant"  n ( i1 n f oo1 m @ n t ) ((pos "K6%" ) ))
( "informants"  n ( i1 n f oo1 m @ n t s ) ((pos "Kj%" ) ))
( "information"  n ( i2 n f @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "informative"  j ( i1 n f oo1 m @ t i v ) ((pos "OA%" ) ))
( "informatively"  a ( i1 n f oo1 m @ t i v l ii ) ((pos "Pu%" ) ))
( "informed"  v ( i1 n f oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "11" "14" )))
( "informer"  n ( i1 n f oo1 m @ r ) ((pos "K6%" ) ))
( "informers"  n ( i1 n f oo1 m @ z ) ((pos "Kj%" ) ))
( "informing"  v ( i1 n f oo1 m i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "11" "14" )))
( "informs"  v ( i1 n f oo1 m z ) ((pos "Ja%" ) (subcat "3A" "6A" "11" "14" )))
( "infra"  a ( i1 n f r @ ) ((pos "Pu%" "U-%" ) ))
( "infra dig"  j ( i2 n f r @ - d i1 g ) ((pos "Op$" ) ))
( "infra-red"  j ( i2 n f r @ - r e1 d ) ((pos "OA%" ) ))
( "infraction"  n ( i1 n f r a1 k sh @ n ) ((pos "M6$" ) ))
( "infractions"  n ( i1 n f r a1 k sh @ n z ) ((pos "Mj$" ) ))
( "infrastructure"  n ( i1 n f r @ s t r uh k ch @ r ) ((pos "K6%" ) ))
( "infrastructures"  n ( i1 n f r @ s t r uh k ch @ z ) ((pos "Kj$" ) ))
( "infrequency"  n ( i1 n f r ii1 k w @ n s ii ) ((pos "L@%" ) ))
( "infrequent"  j ( i1 n f r ii1 k w @ n t ) ((pos "OA%" ) ))
( "infrequently"  a ( i1 n f r ii1 k w @ n t l ii ) ((pos "Pu%" ) ))
( "infringe"  v ( i1 n f r i1 n jh ) ((pos "J2%" ) (subcat "3A" )))
( "infringed"  v ( i1 n f r i1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "infringement"  n ( i1 n f r i1 n jh m @ n t ) ((pos "M6%" ) ))
( "infringements"  n ( i1 n f r i1 n jh m @ n t s ) ((pos "Mj%" ) ))
( "infringes"  v ( i1 n f r i1 n jh i z ) ((pos "Ja%" ) (subcat "3A" )))
( "infringing"  v ( i1 n f r i1 n jh i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "infuriate"  v ( i1 n f y u@1 r i ei t ) ((pos "H2%" ) (subcat )))
( "infuriated"  v ( i1 n f y u@1 r i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "infuriates"  v ( i1 n f y u@1 r i ei t s ) ((pos "Ha%" ) (subcat )))
( "infuriating"  v ( i1 n f y u@1 r i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "infuse"  v ( i1 n f y uu1 z ) ((pos "J2$" ) (subcat "2A" "6A" )))
( "infused"  v ( i1 n f y uu1 z d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "6A" )))
( "infuses"  v ( i1 n f y uu1 z i z ) ((pos "Ja$" ) (subcat "2A" "6A" )))
( "infusing"  v ( i1 n f y uu1 z i ng ) ((pos "Jb$" ) (subcat "2A" "6A" )))
( "infusion"  n ( i1 n f y uu1 zh n ) ((pos "M6%" ) ))
( "infusions"  n ( i1 n f y uu1 zh n z ) ((pos "Mj%" ) ))
( "ing_enue"  n ( a1 n zh ei n y uu ) ((pos "K6%" ) ))
( "ing_enues"  n ( a1 n zh ei n y uu z ) ((pos "Kj%" ) ))
( "ingathering"  n ( i1 n g a dh @ r i ng ) ((pos "K6$" ) ))
( "ingatherings"  n ( i1 n g a dh @ r i ng z ) ((pos "Kj$" ) ))
( "ingenious"  j ( i1 n jh ii1 n i@ s ) ((pos "OA%" ) ))
( "ingeniously"  a ( i1 n jh ii1 n i@ s l ii ) ((pos "Pu%" ) ))
( "ingenuity"  n ( i2 n jh i n y uu1 i t ii ) ((pos "L@%" ) ))
( "ingenuous"  j ( i1 n jh e1 n y u@ s ) ((pos "OA%" ) ))
( "ingenuously"  a ( i1 n jh e1 n y u@ s l ii ) ((pos "Pu%" ) ))
( "ingenuousness"  n ( i1 n jh e1 n y u@ s n @ s ) ((pos "L@%" ) ))
( "ingest"  v ( i1 n jh e1 s t ) ((pos "H0%" ) (subcat )))
( "ingested"  v ( i1 n jh e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ingesting"  v ( i1 n jh e1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "ingests"  v ( i1 n jh e1 s t s ) ((pos "Ha%" ) (subcat )))
( "ingle-nook"  n ( i1 ng g l - n u k ) ((pos "K6$" ) ))
( "ingle-nooks"  n ( i1 ng g l - n u k s ) ((pos "Kj$" ) ))
( "inglorious"  j ( i1 n g l oo1 r i@ s ) ((pos "OA%" ) ))
( "ingloriously"  a ( i1 n g l oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "ingoing"  j ( i1 n g ou i ng ) ((pos "OA%" ) ))
( "ingot"  n ( i1 ng g @ t ) ((pos "K6$" ) ))
( "ingots"  n ( i1 ng g @ t s ) ((pos "Kj$" ) ))
( "ingraft"  v ( i1 n g r aa1 f t ) ((pos "H0$" ) (subcat "6A" )))
( "ingrafted"  v ( i1 n g r aa1 f t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "ingrafting"  v ( i1 n g r aa1 f t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "ingrafts"  v ( i1 n g r aa1 f t s ) ((pos "Ha$" ) (subcat "6A" )))
( "ingrained"  j ( i2 n g r ei1 n d ) ((pos "OA%" ) ))
( "ingratiate"  v ( i1 n g r ei1 sh i ei t ) ((pos "H2%" ) (subcat )))
( "ingratiated"  v ( i1 n g r ei1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ingratiates"  v ( i1 n g r ei1 sh i ei t s ) ((pos "Ha%" ) (subcat )))
( "ingratiating"  v ( i1 n g r ei1 sh i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "ingratiatingly"  a ( i1 n g r ei1 sh i ei t i ng l ii ) ((pos "Pu%" ) ))
( "ingratitude"  n ( i1 n g r a1 t i t y uu d ) ((pos "L@%" ) ))
( "ingredient"  n ( i1 n g r ii1 d i@ n t ) ((pos "K6%" ) ))
( "ingredients"  n ( i1 n g r ii1 d i@ n t s ) ((pos "Kj%" ) ))
( "ingress"  n ( i1 n g r e s ) ((pos "L@$" ) ))
( "ingrowing"  j ( i1 n g r ou i ng ) ((pos "OA%" ) ))
( "inhabit"  v ( i1 n h a1 b i t ) ((pos "H0%" ) (subcat )))
( "inhabitable"  j ( i1 n h a1 b i t @ b l ) ((pos "OA%" ) ))
( "inhabitant"  n ( i1 n h a1 b i t @ n t ) ((pos "K6%" ) ))
( "inhabitants"  n ( i1 n h a1 b i t @ n t s ) ((pos "Kj%" ) ))
( "inhabited"  v ( i1 n h a1 b i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inhabiting"  v ( i1 n h a1 b i t i ng ) ((pos "Hb%" ) (subcat )))
( "inhabits"  v ( i1 n h a1 b i t s ) ((pos "Ha%" ) (subcat )))
( "inhalation"  n ( i2 n h @ l ei1 sh @ n ) ((pos "K6%" ) ))
( "inhalations"  n ( i2 n h @ l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "inhale"  v ( i1 n h ei1 l ) ((pos "J2%" ) (subcat "2A" )))
( "inhaled"  v ( i1 n h ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "inhaler"  n ( i1 n h ei1 l @ r ) ((pos "K6%" ) ))
( "inhalers"  n ( i1 n h ei1 l @ z ) ((pos "Kj%" ) ))
( "inhales"  v ( i1 n h ei1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "inhaling"  v ( i1 n h ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "inharmonious"  j ( i2 n h aa m ou1 n i@ s ) ((pos "OA$" ) ))
( "inherent"  j ( i1 n h i@1 r @ n t ) ((pos "OA%" ) ))
( "inherently"  a ( i1 n h i@1 r @ n t l ii ) ((pos "Pu%" ) ))
( "inherit"  v ( i1 n h e1 r i t ) ((pos "J0%" ) (subcat "2A" )))
( "inheritance"  n ( i1 n h e1 r i t @ n s ) ((pos "M6%" ) ))
( "inheritances"  n ( i1 n h e1 r i t @ n s i z ) ((pos "Mj%" ) ))
( "inherited"  v ( i1 n h e1 r i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "inheriting"  v ( i1 n h e1 r i t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "inheritor"  n ( i1 n h e1 r i t @ r ) ((pos "K6%" ) ))
( "inheritors"  n ( i1 n h e1 r i t @ z ) ((pos "Kj%" ) ))
( "inherits"  v ( i1 n h e1 r i t s ) ((pos "Ja%" ) (subcat "2A" )))
( "inhibit"  v ( i1 n h i1 b i t ) ((pos "H0%" ) (subcat "6A" )))
( "inhibited"  v ( i1 n h i1 b i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inhibiting"  v ( i1 n h i1 b i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inhibition"  n ( i2 n i b i1 sh @ n ) ((pos "M6%" ) ))
( "inhibitions"  n ( i2 n i b i1 sh @ n z ) ((pos "Mj%" ) ))
( "inhibitor"  n ( i1 n h i1 b i t @ r ) ((pos "K6%" ) ))
( "inhibitors"  n ( i1 n h i1 b i t @ z ) ((pos "Kj%" ) ))
( "inhibitory"  j ( i1 n h i1 b i t r ii ) ((pos "OA$" ) ))
( "inhibits"  v ( i1 n h i1 b i t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inhospitable"  j ( i2 n h o s p i1 t @ b l ) ((pos "OA%" ) ))
( "inhuman"  j ( i1 n h y uu1 m @ n ) ((pos "OA%" ) ))
( "inhumane"  j ( i2 n h y uu m ei1 n ) ((pos "OA%" ) ))
( "inhumanely"  a ( i2 n h y uu m ei1 n l ii ) ((pos "Pu%" ) ))
( "inhumanities"  n ( i2 n h y uu m a1 n i t i z ) ((pos "Mj$" ) ))
( "inhumanity"  n ( i2 n h y uu m a1 n i t ii ) ((pos "M8%" ) ))
( "inimical"  j ( i1 n i1 m i k l ) ((pos "OA%" ) ))
( "inimitable"  j ( i1 n i1 m i t @ b l ) ((pos "OA%" ) ))
( "inimitably"  a ( i1 n i1 m i t @ b l ii ) ((pos "Pu%" ) ))
( "iniquities"  n ( i1 n i1 k w i t i z ) ((pos "Mj%" ) ))
( "iniquitous"  j ( i1 n i1 k w i t @ s ) ((pos "OA%" ) ))
( "iniquitously"  a ( i1 n i1 k w i t @ s l ii ) ((pos "Pu%" ) ))
( "iniquity"  n ( i1 n i1 k w i t ii ) ((pos "M8%" ) ))
( "initial"  v ( i1 n i1 sh l ) ((pos "H4%" "K6%" "OA%" ) (subcat )))
( "initialled"  v ( i1 n i1 sh l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "initialling"  v ( i1 n i1 sh @ l i ng ) ((pos "Hb%" ) (subcat )))
( "initially"  a ( i1 n i1 sh @ l ii ) ((pos "Pu%" ) ))
( "initials"  v ( i1 n i1 sh l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "initiate"  n ( i1 n i1 sh i@ t ) ((pos "K6%" "OA%" ) ))
( "initiate"  v ( i1 n i1 sh i ei t ) ((pos "H2%" ) (subcat "6A" )))
( "initiated"  v ( i1 n i1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "initiates"  n ( i1 n i1 sh i@ t s ) ((pos "Kj%" ) ))
( "initiates"  v ( i1 n i1 sh i ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "initiating"  v ( i1 n i1 sh i ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "initiation"  n ( i1 n i2 sh i ei1 sh @ n ) ((pos "L@%" ) ))
( "initiative"  n ( i1 n i1 sh @ t i v ) ((pos "M6%" ) ))
( "initiatives"  n ( i1 n i1 sh @ t i v z ) ((pos "Mj%" ) ))
( "inject"  v ( i1 n jh e1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "injected"  v ( i1 n jh e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "injecting"  v ( i1 n jh e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "injection"  n ( i1 n jh e1 k sh @ n ) ((pos "M6%" ) ))
( "injections"  n ( i1 n jh e1 k sh @ n z ) ((pos "Mj%" ) ))
( "injects"  v ( i1 n jh e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "injudicious"  j ( i2 n jh uu d i1 sh @ s ) ((pos "OA%" ) ))
( "injudiciously"  a ( i2 n jh uu d i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "injunction"  n ( i1 n jh uh1 ng k sh @ n ) ((pos "K6%" ) ))
( "injunctions"  n ( i1 n jh uh1 ng k sh @ n z ) ((pos "Kj%" ) ))
( "injure"  v ( i1 n jh @ r ) ((pos "H2%" ) (subcat )))
( "injured"  v ( i1 n jh @ d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "injures"  v ( i1 n jh @ z ) ((pos "Ha%" ) (subcat )))
( "injuries"  n ( i1 n jh @ r i z ) ((pos "Mj%" ) ))
( "injuring"  v ( i1 n jh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "injurious"  j ( i1 n jh u@1 r i@ s ) ((pos "OA%" ) ))
( "injury"  n ( i1 n jh @ r ii ) ((pos "M8%" ) ))
( "injustice"  n ( i1 n jh uh1 s t i s ) ((pos "M6%" ) ))
( "injustices"  n ( i1 n jh uh1 s t i s i z ) ((pos "Mj%" ) ))
( "ink"  v ( i1 ng k ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "ink-bottle"  n ( i1 ng k - b o t l ) ((pos "K6%" ) ))
( "ink-bottles"  n ( i1 ng k - b o t l z ) ((pos "Kj%" ) ))
( "ink-pad"  n ( i1 ng k - p a d ) ((pos "K6%" ) ))
( "ink-pads"  n ( i1 ng k - p a d z ) ((pos "Kj%" ) ))
( "ink-pot"  n ( i1 ng k - p o t ) ((pos "K6%" ) ))
( "ink-pots"  n ( i1 ng k - p o t s ) ((pos "Kj%" ) ))
( "inked"  v ( i1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inkier"  j ( i1 ng k i@ r ) ((pos "Or$" ) ))
( "inkiest"  j ( i1 ng k i i s t ) ((pos "Os$" ) ))
( "inking"  v ( i1 ng k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inkling"  n ( i1 ng k l i ng ) ((pos "K6%" ) ))
( "inklings"  n ( i1 ng k l i ng z ) ((pos "Kj$" ) ))
( "inks"  v ( i1 ng k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "inkstand"  n ( i1 n k s t a n d ) ((pos "K6%" ) ))
( "inkstands"  n ( i1 n k s t a n d z ) ((pos "Kj%" ) ))
( "inkwell"  n ( i1 ng k w e l ) ((pos "K6%" ) ))
( "inkwells"  n ( i1 ng k w e l z ) ((pos "Kj%" ) ))
( "inky"  j ( i1 ng k ii ) ((pos "OD%" ) ))
( "inlaid"  v ( i2 n l ei1 d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "inland"  j ( i1 n l @ n d ) ((pos "OA%" ) ))
( "inland"  a ( i1 n l a1 n d ) ((pos "Pu%" ) ))
( "inlay"  n ( i1 n l ei ) ((pos "M6%" ) ))
( "inlay"  v ( i2 n l ei1 ) ((pos "H5$" ) (subcat "6A" )))
( "inlaying"  v ( i2 n l ei1 i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "inlays"  n ( i1 n l ei z ) ((pos "Mj$" ) ))
( "inlays"  v ( i2 n l ei1 z ) ((pos "Ha$" ) (subcat "6A" )))
( "inlet"  n ( i1 n l e t ) ((pos "K6%" ) ))
( "inlets"  n ( i1 n l e t s ) ((pos "Kj%" ) ))
( "inmate"  n ( i1 n m ei t ) ((pos "K6%" ) ))
( "inmates"  n ( i1 n m ei t s ) ((pos "Kj%" ) ))
( "inmost"  j ( i1 n m ou s t ) ((pos "OA%" ) ))
( "inn"  n ( i1 n ) ((pos "K6%" ) ))
( "innards"  n ( i1 n @ jh ) ((pos "Kj%" ) ))
( "innate"  j ( i1 n ei1 t ) ((pos "OA%" ) ))
( "innately"  a ( i1 n ei1 t l ii ) ((pos "Pu%" ) ))
( "inner"  j ( i1 n @ r ) ((pos "OA%" ) ))
( "innermost"  j ( i1 n @ m ou s t ) ((pos "OA%" ) ))
( "inning"  n ( i1 n i ng ) ((pos "K6$" ) ))
( "innings"  n ( i1 n i ng z ) ((pos "K9%" ) ))
( "innkeeper"  n ( i1 n k ii p @ r ) ((pos "K6%" ) ))
( "innkeepers"  n ( i1 n k ii p @ z ) ((pos "Kj%" ) ))
( "innocence"  n ( i1 n @ s n s ) ((pos "L@%" ) ))
( "innocent"  n ( i1 n @ s n t ) ((pos "K6%" "OA%" ) ))
( "innocently"  a ( i1 n @ s n t l ii ) ((pos "Pu%" ) ))
( "innocents"  n ( i1 n @ s n t s ) ((pos "Kj%" ) ))
( "innocuous"  j ( i1 n o1 k y u@ s ) ((pos "OA%" ) ))
( "innovate"  v ( i1 n @ v ei t ) ((pos "I2$" ) (subcat )))
( "innovated"  v ( i1 n @ v ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "innovates"  v ( i1 n @ v ei t s ) ((pos "Ia$" ) (subcat )))
( "innovating"  v ( i1 n @ v ei t i ng ) ((pos "Ib$" ) (subcat )))
( "innovation"  n ( i2 n @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "innovations"  n ( i2 n @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "innovator"  n ( i1 n @ v ei t @ r ) ((pos "K6%" ) ))
( "innovators"  n ( i1 n @ v ei t @ z ) ((pos "Kj%" ) ))
( "inns"  n ( i1 n z ) ((pos "Kj%" ) ))
( "innuendo"  n ( i2 n y uu e1 n d ou ) ((pos "K7%" ) ))
( "innuendoes"  n ( i2 n y uu e1 n d ou z ) ((pos "Kj%" ) ))
( "innumerable"  j ( i1 n y uu1 m @ r @ b l ) ((pos "OA%" ) ))
( "inoculate"  v ( i1 n o1 k y u l ei t ) ((pos "H2%" ) (subcat "6A" )))
( "inoculated"  v ( i1 n o1 k y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inoculates"  v ( i1 n o1 k y u l ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inoculating"  v ( i1 n o1 k y u l ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inoculation"  n ( i1 n o2 k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "inoculations"  n ( i1 n o2 k y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "inoffensive"  j ( i2 n @ f e1 n s i v ) ((pos "OA%" ) ))
( "inoffensively"  a ( i2 n @ f e1 n s i v l ii ) ((pos "Pu%" ) ))
( "inoperable"  j ( i2 n o1 p @ r @ b l ) ((pos "OA%" ) ))
( "inoperative"  j ( i2 n o1 p @ r @ t i v ) ((pos "OA%" ) ))
( "inopportune"  j ( i2 n o1 p @ t y uu n ) ((pos "OA%" ) ))
( "inopportunely"  a ( i2 n o1 p @ t y uu n l ii ) ((pos "Pu%" ) ))
( "inordinate"  j ( i1 n oo1 d i n @ t ) ((pos "OA%" ) ))
( "inordinately"  a ( i1 n oo1 d i n @ t l ii ) ((pos "Pu%" ) ))
( "inorganic"  j ( i2 n oo g a1 n i k ) ((pos "OA%" ) ))
( "inorganically"  a ( i2 n oo g a1 n i k l ii ) ((pos "Pu%" ) ))
( "inpouring"  n ( i1 n p oo r i ng ) ((pos "K6$" "OA$" ) ))
( "inpourings"  n ( i1 n p oo r i ng z ) ((pos "Kj$" ) ))
( "input"  n ( i1 n p u t ) ((pos "M6%" ) ))
( "inputs"  n ( i1 n p u t s ) ((pos "Mj$" ) ))
( "inquest"  n ( i1 n k w e s t ) ((pos "K6%" ) ))
( "inquests"  n ( i1 n k w e s t s ) ((pos "Kj%" ) ))
( "inquietude"  n ( i1 n k w ai1 @ t y uu d ) ((pos "L@$" ) ))
( "inquire"  v ( i1 n k w ai1 @ r ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "inquired"  v ( i1 n k w ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "inquirer"  n ( i1 n k w ai1 @ r @ r ) ((pos "K6%" ) ))
( "inquirers"  n ( i1 n k w ai1 @ r @ z ) ((pos "Kj%" ) ))
( "inquires"  v ( i1 n k w ai1 @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "inquiries"  n ( i1 n k w ai1 @ r i z ) ((pos "Mj%" ) ))
( "inquiring"  v ( i1 n k w ai1 @ r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "8" "10" )))
( "inquiringly"  a ( i1 n k w ai1 @ r i ng l ii ) ((pos "Pu%" ) ))
( "inquiry"  n ( i1 n k w ai1 @ r ii ) ((pos "M8%" ) ))
( "inquisition"  n ( i2 n k w i z i1 sh @ n ) ((pos "M6%" ) ))
( "inquisitions"  n ( i2 n k w i z i1 sh @ n z ) ((pos "Mj%" ) ))
( "inquisitive"  j ( i1 n k w i1 z @ t i v ) ((pos "OA%" ) ))
( "inquisitively"  a ( i1 n k w i1 z @ t i v l ii ) ((pos "Pu%" ) ))
( "inquisitiveness"  n ( i1 n k w i1 z @ t i v n @ s ) ((pos "L@%" ) ))
( "inquisitor"  n ( i1 n k w i1 z i t @ r ) ((pos "K6%" ) ))
( "inquisitorial"  j ( i1 n k w i2 z i t oo1 r i@ l ) ((pos "OA%" ) ))
( "inquisitors"  n ( i1 n k w i1 z i t @ z ) ((pos "Kj%" ) ))
( "inroad"  n ( i1 n r ou d ) ((pos "K6%" ) ))
( "inroads"  n ( i1 n r ou d z ) ((pos "Kj%" ) ))
( "inrush"  n ( i1 n r uh sh ) ((pos "K7$" ) ))
( "inrushes"  n ( i1 n r uh sh i z ) ((pos "Kj$" ) ))
( "ins"  n ( i1 n ch i z ) ((pos "Y)%" ) ))
( "ins"  n ( i1 n z ) ((pos "Kj$" ) ))
( "insane"  j ( i1 n s ei1 n ) ((pos "OA%" ) ))
( "insanely"  a ( i1 n s ei1 n l ii ) ((pos "Pu%" ) ))
( "insanitary"  j ( i1 n s a1 n i t r ii ) ((pos "OA%" ) ))
( "insanity"  n ( i1 n s a1 n i t ii ) ((pos "L@%" ) ))
( "insatiable"  j ( i1 n s ei1 sh @ b l ) ((pos "OA%" ) ))
( "insatiably"  a ( i1 n s ei1 sh @ b l ii ) ((pos "Pu%" ) ))
( "insatiate"  j ( i1 n s ei1 sh i@ t ) ((pos "OA$" ) ))
( "inscribe"  v ( i1 n s k r ai1 b ) ((pos "H2%" ) (subcat "6A" )))
( "inscribed"  v ( i1 n s k r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inscribes"  v ( i1 n s k r ai1 b z ) ((pos "Ha%" ) (subcat "6A" )))
( "inscribing"  v ( i1 n s k r ai1 b i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "inscription"  n ( i1 n s k r i1 p sh @ n ) ((pos "K6%" ) ))
( "inscriptions"  n ( i1 n s k r i1 p sh @ n z ) ((pos "Kj%" ) ))
( "inscrutable"  j ( i1 n s k r uu1 t @ b l ) ((pos "OA%" ) ))
( "insect"  n ( i1 n s e k t ) ((pos "K6%" ) ))
( "insect-powder"  n ( i1 n s e k t - p au d @ r ) ((pos "K6%" ) ))
( "insect-powders"  n ( i1 n s e k t - p au d @ z ) ((pos "Kj%" ) ))
( "insecticide"  n ( i1 n s e1 k t i s ai d ) ((pos "K6%" ) ))
( "insecticides"  n ( i1 n s e1 k t i s ai d z ) ((pos "Kj%" ) ))
( "insectivorous"  j ( i2 n s e k t i1 v @ r @ s ) ((pos "OA%" ) ))
( "insects"  n ( i1 n s e k t s ) ((pos "Kj%" ) ))
( "insecure"  j ( i2 n s i k y u@1 r ) ((pos "OA%" ) ))
( "insecurely"  a ( i2 n s i k y u@1 l ii ) ((pos "Pu%" ) ))
( "insecurity"  n ( i2 n s i k y u@1 r i t ii ) ((pos "L@%" ) ))
( "inseminate"  v ( i1 n s e1 m i n ei t ) ((pos "H2$" ) (subcat )))
( "inseminated"  v ( i1 n s e1 m i n ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "inseminates"  v ( i1 n s e1 m i n ei t s ) ((pos "Ha$" ) (subcat )))
( "inseminating"  v ( i1 n s e1 m i n ei t i ng ) ((pos "Hb$" ) (subcat )))
( "insemination"  n ( i1 n s e2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "insensate"  j ( i1 n s e1 n s ei t ) ((pos "OA$" ) ))
( "insensibility"  n ( i1 n s e2 n s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "insensible"  j ( i1 n s e1 n s @ b l ) ((pos "OA%" ) ))
( "insensibly"  a ( i1 n s e1 n s @ b l ii ) ((pos "Pu%" ) ))
( "insensitive"  j ( i1 n s e1 n s @ t i v ) ((pos "OA%" ) ))
( "insensitively"  a ( i1 n s e1 n s @ t i v l ii ) ((pos "Pu%" ) ))
( "insensitivity"  n ( i1 n s e2 n s @ t i1 v i t ii ) ((pos "L@%" ) ))
( "insentient"  j ( i1 n s e1 n sh @ n t ) ((pos "OA$" ) ))
( "inseparable"  j ( i1 n s e1 p r @ b l ) ((pos "OA%" ) ))
( "insert"  n ( i1 n s @@ t ) ((pos "K6%" ) ))
( "insert"  v ( i1 n s @@1 t ) ((pos "H0%" ) (subcat "6A" )))
( "inserted"  v ( i1 n s @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "inserting"  v ( i1 n s @@1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "insertion"  n ( i1 n s @@1 sh @ n ) ((pos "M6%" ) ))
( "insertions"  n ( i1 n s @@1 sh @ n z ) ((pos "Mj%" ) ))
( "inserts"  n ( i1 n s @@ t s ) ((pos "Kj%" ) ))
( "inserts"  v ( i1 n s @@1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "inset"  n ( i1 n s e t ) ((pos "K6%" ) ))
( "inset"  v ( i2 n s e1 t ) ((pos "H5$" "Hc$" "Hd$" ) ))
( "insets"  n ( i1 n s e t s ) ((pos "Kj$" ) ))
( "insets"  v ( i2 n s e1 t s ) ((pos "Ha$" ) ))
( "insetting"  v ( i2 n s e1 t i ng ) ((pos "Hb$" ) ))
( "inshore"  j ( i2 n sh oo1 r ) ((pos "OA%" "Pu%" ) ))
( "inside"  n ( i1 n s ai1 d ) ((pos "K6%" "OA%" "Pu%" "T-%" ) ))
( "insider"  n ( i1 n s ai1 d @ r ) ((pos "K6%" ) ))
( "insiders"  n ( i1 n s ai1 d @ z ) ((pos "Kj%" ) ))
( "insides"  n ( i1 n s ai1 d z ) ((pos "Kj%" ) ))
( "insidious"  j ( i1 n s i1 d i@ s ) ((pos "OA%" ) ))
( "insidiously"  a ( i1 n s i1 d i@ s l ii ) ((pos "Pu%" ) ))
( "insidiousness"  n ( i1 n s i1 d i@ s n @ s ) ((pos "L@%" ) ))
( "insight"  n ( i1 n s ai t ) ((pos "M6%" ) ))
( "insights"  n ( i1 n s ai t s ) ((pos "Mj%" ) ))
( "insignia"  n ( i1 n s i1 g n i@ ) ((pos "Kj%" ) ))
( "insignificance"  n ( i2 n s i g n i1 f i k @ n s ) ((pos "L@%" ) ))
( "insignificant"  j ( i2 n s i g n i1 f i k @ n t ) ((pos "OA%" ) ))
( "insignificantly"  a ( i2 n s i g n i1 f i k @ n t l ii ) ((pos "Pu%" ) ))
( "insincere"  j ( i2 n s i n s i@1 r ) ((pos "OA%" ) ))
( "insincerely"  a ( i2 n s i n s i@1 l ii ) ((pos "Pu%" ) ))
( "insincerity"  n ( i2 n s i n s e1 r i t ii ) ((pos "L@$" ) ))
( "insinuate"  v ( i1 n s i1 n y u ei t ) ((pos "H2$" ) (subcat "6A" "9" )))
( "insinuated"  v ( i1 n s i1 n y u ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "9" )))
( "insinuates"  v ( i1 n s i1 n y u ei t s ) ((pos "Ha$" ) (subcat "6A" "9" )))
( "insinuating"  v ( i1 n s i1 n y u ei t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "insinuatingly"  a ( i1 n s i1 n y u ei t i ng l ii ) ((pos "Pu%" ) ))
( "insinuation"  n ( i1 n s i2 n y u ei1 sh @ n ) ((pos "M6%" ) ))
( "insinuations"  n ( i1 n s i2 n y u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "insipid"  j ( i1 n s i1 p i d ) ((pos "OA%" ) ))
( "insipidity"  n ( i2 n s i p i1 d i t ii ) ((pos "L@$" ) ))
( "insipidly"  a ( i1 n s i1 p i d l ii ) ((pos "Pu%" ) ))
( "insipidness"  n ( i1 n s i1 p i d n @ s ) ((pos "L@$" ) ))
( "insist"  v ( i1 n s i1 s t ) ((pos "J0%" ) (subcat "3A" )))
( "insisted"  v ( i1 n s i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "insistence"  n ( i1 n s i1 s t @ n s ) ((pos "L@%" ) ))
( "insistent"  j ( i1 n s i1 s t @ n t ) ((pos "OA%" ) ))
( "insisting"  v ( i1 n s i1 s t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "insists"  v ( i1 n s i1 s t s ) ((pos "Ja%" ) (subcat "3A" )))
( "insofar"  a ( i2 n s @ f aa1 r ) ((pos "Pu%" ) ))
( "insole"  n ( i1 n s ou l ) ((pos "K6%" ) ))
( "insolence"  n ( i1 n s @ l @ n s ) ((pos "L@%" ) ))
( "insolent"  j ( i1 n s @ l @ n t ) ((pos "OA%" ) ))
( "insolently"  a ( i1 n s @ l @ n t l ii ) ((pos "Pu%" ) ))
( "insoles"  n ( i1 n s ou l z ) ((pos "Kj%" ) ))
( "insoluble"  j ( i1 n s o1 l y u b l ) ((pos "OA%" ) ))
( "insolvency"  n ( i1 n s o1 l v @ n s ii ) ((pos "L@%" ) ))
( "insolvent"  n ( i1 n s o1 l v @ n t ) ((pos "K6$" "OA%" ) ))
( "insolvents"  n ( i1 n s o1 l v @ n t s ) ((pos "Kj$" ) ))
( "insomnia"  n ( i1 n s o1 m n i@ ) ((pos "L@%" ) ))
( "insomniac"  n ( i1 n s o1 m n i a k ) ((pos "K6%" ) ))
( "insomniacs"  n ( i1 n s o1 m n i a k s ) ((pos "Kj%" ) ))
( "insomuch"  a ( i2 n s ou m uh1 ch ) ((pos "Pu%" ) ))
( "insouciance"  n ( i1 n s uu1 s i@ n s ) ((pos "L@$" ) ))
( "insouciant"  j ( i1 n s uu1 s i@ n t ) ((pos "OA$" ) ))
( "inspan"  v ( i1 n s p a1 n ) ((pos "H4$" ) ))
( "inspanned"  v ( i1 n s p a1 n d ) ((pos "Hc$" "Hd$" ) ))
( "inspanning"  v ( i1 n s p a1 n i ng ) ((pos "Hb$" ) ))
( "inspans"  v ( i1 n s p a1 n z ) ((pos "Ha$" ) ))
( "inspect"  v ( i1 n s p e1 k t ) ((pos "H0%" ) (subcat )))
( "inspected"  v ( i1 n s p e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inspecting"  v ( i1 n s p e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "inspection"  n ( i1 n s p e1 k sh @ n ) ((pos "M6%" ) ))
( "inspections"  n ( i1 n s p e1 k sh @ n z ) ((pos "Mj%" ) ))
( "inspector"  n ( i1 n s p e1 k t @ r ) ((pos "K6%" ) ))
( "inspectorate"  n ( i1 n s p e1 k t @ r @ t ) ((pos "K6%" ) ))
( "inspectorates"  n ( i1 n s p e1 k t @ r @ t s ) ((pos "Kj%" ) ))
( "inspectors"  n ( i1 n s p e1 k t @ z ) ((pos "Kj%" ) ))
( "inspects"  v ( i1 n s p e1 k t s ) ((pos "Ha%" ) (subcat )))
( "inspiration"  n ( i2 n s p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "inspirational"  j ( i2 n s p @ r ei1 sh @ n l ) ((pos "OA%" ) ))
( "inspirationally"  a ( i2 n s p @ r ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "inspirations"  n ( i2 n s p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "inspire"  v ( i1 n s p ai1 @ r ) ((pos "H2%" ) (subcat "6A" "14" )))
( "inspired"  v ( i1 n s p ai1 @ d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "14" )))
( "inspires"  v ( i1 n s p ai1 @ z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "inspiring"  v ( i1 n s p ai1 @ r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "instabilities"  n ( i2 n s t @ b i1 l i t i z ) ((pos "Mj$" ) ))
( "instability"  n ( i2 n s t @ b i1 l i t ii ) ((pos "M8%" ) ))
( "install"  v ( i1 n s t oo1 l ) ((pos "H0%" ) (subcat "6A" )))
( "installation"  n ( i2 n s t @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "installations"  n ( i2 n s t @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "installed"  v ( i1 n s t oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "installing"  v ( i1 n s t oo1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "installs"  v ( i1 n s t oo1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "instalment"  n ( i1 n s t oo1 l m @ n t ) ((pos "K6%" ) ))
( "instalments"  n ( i1 n s t oo1 l m @ n t s ) ((pos "Kj%" ) ))
( "instance"  v ( i1 n s t @ n s ) ((pos "H2$" "K6%" ) (subcat )))
( "instanced"  v ( i1 n s t @ n s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "instances"  v ( i1 n s t @ n s i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "instancing"  v ( i1 n s t @ n s i ng ) ((pos "Hb$" ) (subcat )))
( "instant"  n ( i1 n s t @ n t ) ((pos "K6%" "OA%" ) ))
( "instantaneous"  j ( i2 n s t @ n t ei1 n i@ s ) ((pos "OA%" ) ))
( "instantaneously"  a ( i2 n s t @ n t ei1 n i@ s l ii ) ((pos "Pu%" ) ))
( "instantly"  a ( i1 n s t @ n t l ii ) ((pos "Pu%" "V-%" ) ))
( "instants"  n ( i1 n s t @ n t s ) ((pos "Kj%" ) ))
( "instead"  a ( i1 n s t e1 d ) ((pos "Pu%" ) ))
( "instep"  n ( i1 n s t e p ) ((pos "K6%" ) ))
( "insteps"  n ( i1 n s t e p s ) ((pos "Kj%" ) ))
( "instigate"  v ( i1 n s t i g ei t ) ((pos "H2%" ) (subcat "6A" )))
( "instigated"  v ( i1 n s t i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "instigates"  v ( i1 n s t i g ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "instigating"  v ( i1 n s t i g ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "instigation"  n ( i2 n s t i g ei1 sh @ n ) ((pos "K6%" ) ))
( "instigations"  n ( i2 n s t i g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "instigator"  n ( i1 n s t i g ei t @ r ) ((pos "K6%" ) ))
( "instigators"  n ( i1 n s t i g ei t @ z ) ((pos "Kj%" ) ))
( "instil"  v ( i1 n s t i1 l ) ((pos "H4%" ) (subcat "6A" )))
( "instillation"  n ( i2 n s t i l ei1 sh @ n ) ((pos "K6$" ) ))
( "instillations"  n ( i2 n s t i l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "instilled"  v ( i1 n s t i1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "instilling"  v ( i1 n s t i1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "instils"  v ( i1 n s t i1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "instinct"  n ( i1 n s t i ng k t ) ((pos "M6%" ) ))
( "instinct"  j ( i2 n s t i1 ng k t ) ((pos "Op%" ) ))
( "instinctive"  j ( i1 n s t i1 ng k t i v ) ((pos "OA%" ) ))
( "instinctively"  a ( i1 n s t i1 ng k t i v l ii ) ((pos "Pu%" ) ))
( "instincts"  n ( i1 n s t i ng k t s ) ((pos "Mj%" ) ))
( "institute"  v ( i1 n s t i t y uu t ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "instituted"  v ( i1 n s t i t y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "institutes"  v ( i1 n s t i t y uu t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "instituting"  v ( i1 n s t i t y uu t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "institution"  n ( i2 n s t i t y uu1 sh @ n ) ((pos "M6%" ) ))
( "institutional"  j ( i2 n s t i t y uu1 sh @ n @ l ) ((pos "OA%" ) ))
( "institutionalize"  v ( i2 n s t i t y uu1 sh @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "institutionalized"  v ( i2 n s t i t y uu1 sh @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "institutionalizes"  v ( i2 n s t i t y uu1 sh @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "institutionalizing"  v ( i2 n s t i t y uu1 sh @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "institutionally"  a ( i2 n s t i t y uu1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "institutions"  n ( i2 n s t i t y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "instruct"  v ( i1 n s t r uh1 k t ) ((pos "H0%" ) (subcat "6A" "11" "15A" "17" "20" )))
( "instructed"  v ( i1 n s t r uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "15A" "17" "20" )))
( "instructing"  v ( i1 n s t r uh1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "11" "15A" "17" "20" )))
( "instruction"  n ( i1 n s t r uh1 k sh @ n ) ((pos "M6%" ) ))
( "instructional"  j ( i1 n s t r uh1 k sh @ n @ l ) ((pos "OA%" ) ))
( "instructions"  n ( i1 n s t r uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "instructive"  j ( i1 n s t r uh1 k t i v ) ((pos "OA%" ) ))
( "instructively"  a ( i1 n s t r uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "instructor"  n ( i1 n s t r uh1 k t @ r ) ((pos "K6%" ) ))
( "instructors"  n ( i1 n s t r uh1 k t @ z ) ((pos "Kj%" ) ))
( "instructress"  n ( i1 n s t r uh1 k t r i s ) ((pos "K7%" ) ))
( "instructresses"  n ( i1 n s t r uh1 k t r i s i z ) ((pos "Kj$" ) ))
( "instructs"  v ( i1 n s t r uh1 k t s ) ((pos "Ha%" ) (subcat "6A" "11" "15A" "17" "20" )))
( "instrument"  n ( i1 n s t r u m @ n t ) ((pos "K6%" ) ))
( "instrumental"  j ( i2 n s t r u m e1 n t l ) ((pos "OA%" ) ))
( "instrumentalist"  n ( i2 n s t r u m e1 n t @ l i s t ) ((pos "K6%" ) ))
( "instrumentalists"  n ( i2 n s t r u m e1 n t @ l i s t s ) ((pos "Kj%" ) ))
( "instrumentality"  n ( i2 n s t r u m e n t a1 l i t ii ) ((pos "L@$" ) ))
( "instrumentation"  n ( i2 n s t r u m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "instruments"  n ( i1 n s t r u m @ n t s ) ((pos "Kj%" ) ))
( "insubordinate"  j ( i2 n s @ b oo1 d i n @ t ) ((pos "OA%" ) ))
( "insubordination"  n ( i2 n s @ b oo2 d i n ei1 sh @ n ) ((pos "M6%" ) ))
( "insubordinations"  n ( i2 n s @ b oo2 d i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "insubstantial"  j ( i2 n s @ b s t a1 n sh l ) ((pos "OA%" ) ))
( "insubstantially"  a ( i2 n s @ b s t a1 n sh @ l ii ) ((pos "Pu%" ) ))
( "insufferable"  j ( i1 n s uh1 f r @ b l ) ((pos "OA%" ) ))
( "insufficiency"  n ( i2 n s @ f i1 sh @ n s ii ) ((pos "L@%" ) ))
( "insufficient"  j ( i2 n s @ f i1 sh @ n t ) ((pos "OA%" ) ))
( "insufficiently"  a ( i2 n s @ f i1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "insular"  j ( i1 n s y u l @ r ) ((pos "OA%" ) ))
( "insularism"  n ( i1 n s y u l @ r i z @ m ) ((pos "L@$" ) ))
( "insularity"  n ( i2 n s y u l a1 r i t ii ) ((pos "L@%" ) ))
( "insulate"  v ( i1 n s y u l ei t ) ((pos "H2%" ) (subcat "6A" )))
( "insulated"  v ( i1 n s y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "insulates"  v ( i1 n s y u l ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "insulating"  v ( i1 n s y u l ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "insulation"  n ( i2 n s y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "insulator"  n ( i1 n s y u l ei t @ r ) ((pos "K6%" ) ))
( "insulators"  n ( i1 n s y u l ei t @ z ) ((pos "Kj%" ) ))
( "insulin"  n ( i1 n s y u l i n ) ((pos "L@%" ) ))
( "insult"  n ( i1 n s uh l t ) ((pos "M6%" ) ))
( "insult"  v ( i1 n s uh1 l t ) ((pos "H0%" ) (subcat )))
( "insulted"  v ( i1 n s uh1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "insulting"  v ( i1 n s uh1 l t i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "insultingly"  a ( i1 n s uh1 l t i ng l ii ) ((pos "Pu%" ) ))
( "insults"  n ( i1 n s uh l t s ) ((pos "Mj%" ) ))
( "insults"  v ( i1 n s uh1 l t s ) ((pos "Ha%" ) (subcat )))
( "insuperable"  j ( i1 n s y uu1 p r @ b l ) ((pos "OA%" ) ))
( "insuperably"  a ( i1 n s y uu1 p r @ b l ii ) ((pos "Pu%" ) ))
( "insupportable"  j ( i2 n s @ p oo1 t @ b l ) ((pos "OA%" ) ))
( "insurance"  n ( i1 n sh u@1 r @ n s ) ((pos "M6%" ) ))
( "insurances"  n ( i1 n sh u@1 r @ n s i z ) ((pos "Mj$" ) ))
( "insure"  v ( i1 n sh u@1 r ) ((pos "H2%" ) (subcat "6A" )))
( "insured"  v ( i1 n sh u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "insures"  v ( i1 n sh u@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "insurgent"  n ( i1 n s @@1 jh @ n t ) ((pos "K6%" "OA%" ) ))
( "insurgents"  n ( i1 n s @@1 jh @ n t s ) ((pos "Kj%" ) ))
( "insuring"  v ( i1 n sh u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "insurmountable"  j ( i2 n s @ m au1 n t @ b l ) ((pos "OA%" ) ))
( "insurrection"  n ( i2 n s @ r e1 k sh @ n ) ((pos "M6%" ) ))
( "insurrections"  n ( i2 n s @ r e1 k sh @ n z ) ((pos "Mj%" ) ))
( "intact"  j ( i1 n t a1 k t ) ((pos "OA%" ) ))
( "intaglio"  n ( i1 n t aa1 l i@ @ ) ((pos "M6$" ) ))
( "intaglios"  n ( i1 n t aa1 l i@ u z ) ((pos "Mj$" ) ))
( "intake"  n ( i1 n t ei k ) ((pos "M6%" ) ))
( "intakes"  n ( i1 n t ei k s ) ((pos "Mj%" ) ))
( "intangibility"  n ( i1 n t a2 n jh @ b i1 l i t ii ) ((pos "L@$" ) ))
( "intangible"  j ( i1 n t a1 n jh @ b l ) ((pos "OA%" ) ))
( "intangibles"  n ( i1 n t a1 n jh @ b l z ) ((pos "K6%" ) ))
( "integer"  n ( i1 n t i jh @ r ) ((pos "K6$" ) ))
( "integers"  n ( i1 n t i jh @ z ) ((pos "Kj$" ) ))
( "integral"  j ( i1 n t i g r @ l ) ((pos "OA%" ) ))
( "integrally"  a ( i1 n t i g r @ l ii ) ((pos "Pu%" ) ))
( "integrate"  v ( i1 n t i g r ei t ) ((pos "H2%" ) (subcat "2A" )))
( "integrated"  v ( i1 n t i g r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "integrates"  v ( i1 n t i g r ei t s ) ((pos "Ha%" ) (subcat "2A" )))
( "integrating"  v ( i1 n t i g r ei t i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "integration"  n ( i2 n t i g r ei1 sh @ n ) ((pos "L@%" ) ))
( "integrity"  n ( i1 n t e1 g r i t ii ) ((pos "L@%" ) ))
( "integument"  n ( i1 n t e1 g y u m @ n t ) ((pos "K6$" ) ))
( "integuments"  n ( i1 n t e1 g y u m @ n t s ) ((pos "Kj$" ) ))
( "intellect"  n ( i1 n t @ l e k t ) ((pos "M6%" ) ))
( "intellects"  n ( i1 n t @ l e k t s ) ((pos "Mj%" ) ))
( "intellectual"  n ( i2 n t @ l e1 k ch u@ l ) ((pos "K6%" "OA%" ) ))
( "intellectually"  a ( i2 n t @ l e1 k ch u@ l ii ) ((pos "Pu%" ) ))
( "intellectuals"  n ( i2 n t @ l e1 k ch u@ l z ) ((pos "Kj%" ) ))
( "intelligence"  n ( i1 n t e1 l i jh @ n s ) ((pos "L@%" ) ))
( "intelligent"  j ( i1 n t e1 l i jh @ n t ) ((pos "OA%" ) ))
( "intelligently"  a ( i1 n t e1 l i jh @ n t l ii ) ((pos "Pu%" ) ))
( "intelligentsia"  n ( i1 n t e2 l i jh e1 n t s i@ ) ((pos "K6%" ) ))
( "intelligentsias"  n ( i1 n t e2 l i jh e1 n t s i@ z ) ((pos "Kj$" ) ))
( "intelligibility"  n ( i1 n t e2 l i jh @ b i1 l i t ii ) ((pos "L@$" ) ))
( "intelligible"  j ( i1 n t e1 l i jh @ b l ) ((pos "OA%" ) ))
( "intelligibly"  a ( i1 n t e1 l i jh @ b l ii ) ((pos "Pu%" ) ))
( "intemperance"  n ( i1 n t e1 m p @ r @ n s ) ((pos "L@%" ) ))
( "intemperate"  j ( i1 n t e1 m p @ r @ t ) ((pos "OA%" ) ))
( "intemperately"  a ( i1 n t e1 m p @ r @ t l ii ) ((pos "Pu%" ) ))
( "intend"  v ( i1 n t e1 n d ) ((pos "H0%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "intended"  v ( i1 n t e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "intending"  v ( i1 n t e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "intends"  v ( i1 n t e1 n d z ) ((pos "Ha%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "intense"  j ( i1 n t e1 n s ) ((pos "OA%" ) ))
( "intensely"  a ( i1 n t e1 n s l ii ) ((pos "Pu%" ) ))
( "intensification"  n ( i1 n t e2 n s i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "intensifications"  n ( i1 n t e2 n s i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "intensified"  v ( i1 n t e1 n s i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "intensifies"  v ( i1 n t e1 n s i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "intensify"  v ( i1 n t e1 n s i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "intensifying"  v ( i1 n t e1 n s i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "intensities"  n ( i1 n t e1 n s i t i z ) ((pos "Mj%" ) ))
( "intensity"  n ( i1 n t e1 n s i t ii ) ((pos "M8%" ) ))
( "intensive"  j ( i1 n t e1 n s i v ) ((pos "OA%" ) ))
( "intensively"  a ( i1 n t e1 n s i v l ii ) ((pos "Pu%" ) ))
( "intent"  n ( i1 n t e1 n t ) ((pos "M6%" "OA%" ) ))
( "intention"  n ( i1 n t e1 n sh @ n ) ((pos "M6%" ) ))
( "intentional"  j ( i1 n t e1 n sh @ n @ l ) ((pos "OA%" ) ))
( "intentionally"  a ( i1 n t e1 n sh @ n @ l ii ) ((pos "Pu%" ) ))
( "intentions"  n ( i1 n t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "intently"  a ( i1 n t e1 n t l ii ) ((pos "Pu%" ) ))
( "intentness"  n ( i1 n t e1 n t n @ s ) ((pos "L@$" ) ))
( "intents"  n ( i1 n t e1 n t s ) ((pos "Mj%" ) ))
( "inter"  v ( i1 n t @@1 r ) ((pos "H4%" ) ))
( "inter alia"  a ( i2 n t @ r - ei1 l i@ ) ((pos "Pu%" ) ))
( "interact"  v ( i2 n t @ r a1 k t ) ((pos "I0%" ) ))
( "interacted"  v ( i2 n t @ r a1 k t i d ) ((pos "Ic%" "Id%" ) ))
( "interacting"  v ( i2 n t @ r a1 k t i ng ) ((pos "Ib%" ) ))
( "interaction"  n ( i2 n t @ r a1 k sh @ n ) ((pos "K6%" ) ))
( "interactions"  n ( i2 n t @ r a1 k sh @ n z ) ((pos "Kj%" ) ))
( "interactive"  j ( i2 n t @ r a1 k t i v ) ((pos "OA$" ) ))
( "interacts"  v ( i2 n t @ r a1 k t s ) ((pos "Ia%" ) ))
( "interbred"  v ( i2 n t @ b r e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "interbreed"  v ( i2 n t @ b r ii1 d ) ((pos "J5%" ) (subcat "2A" )))
( "interbreeding"  v ( i2 n t @ b r ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "interbreeds"  v ( i2 n t @ b r ii1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "intercalary"  j ( i1 n t @@1 k @ l @ r ii ) ((pos "OA$" ) ))
( "intercede"  v ( i2 n t @ s ii1 d ) ((pos "I2%" ) (subcat )))
( "interceded"  v ( i2 n t @ s ii1 d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "intercedes"  v ( i2 n t @ s ii1 d z ) ((pos "Ia%" ) (subcat )))
( "interceding"  v ( i2 n t @ s ii1 d i ng ) ((pos "Ib%" ) (subcat )))
( "intercept"  v ( i2 n t @ s e1 p t ) ((pos "H0%" ) (subcat )))
( "intercepted"  v ( i2 n t @ s e1 p t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "intercepting"  v ( i2 n t @ s e1 p t i ng ) ((pos "Hb%" ) (subcat )))
( "interception"  n ( i2 n t @ s e1 p sh @ n ) ((pos "M6%" ) ))
( "interceptions"  n ( i2 n t @ s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "interceptor"  n ( i2 n t @ s e1 p t @ r ) ((pos "K6%" ) ))
( "interceptors"  n ( i2 n t @ s e1 p t @ z ) ((pos "Kj%" ) ))
( "intercepts"  v ( i2 n t @ s e1 p t s ) ((pos "Ha%" ) (subcat )))
( "intercession"  n ( i2 n t @ s e1 sh @ n ) ((pos "M6%" ) ))
( "intercessions"  n ( i2 n t @ s e1 sh @ n z ) ((pos "Mj$" ) ))
( "interchange"  n ( i1 n t @ ch ei n jh ) ((pos "M6%" ) ))
( "interchange"  v ( i2 n t @ ch ei1 n jh ) ((pos "H2%" ) (subcat "2A" )))
( "interchangeable"  j ( i2 n t @ ch ei1 n jh @ b l ) ((pos "OA%" ) ))
( "interchangeably"  a ( i2 n t @ ch ei1 n jh @ b l ii ) ((pos "Pu%" ) ))
( "interchanged"  v ( i2 n t @ ch ei1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "interchanges"  n ( i1 n t @ ch ei n jh i z ) ((pos "Mj%" ) ))
( "interchanges"  v ( i2 n t @ ch ei1 n jh i z ) ((pos "Ha%" ) (subcat "2A" )))
( "interchanging"  v ( i2 n t @ ch ei1 n jh i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "intercollegiate"  j ( i2 n t @ k @ l ii1 jh i@ t ) ((pos "OA%" ) ))
( "intercom"  n ( i1 n t @ k o m ) ((pos "K6%" ) ))
( "intercommunicate"  v ( i2 n t @ k @ m y uu1 n i k ei t ) ((pos "I2$" ) ))
( "intercommunicated"  v ( i2 n t @ k @ m y uu1 n i k ei t i d ) ((pos "Ic$" "Id$" ) ))
( "intercommunicates"  v ( i2 n t @ k @ m y uu1 n i k ei t s ) ((pos "Ia$" ) ))
( "intercommunicating"  v ( i2 n t @ k @ m y uu1 n i k ei t i ng ) ((pos "Ib$" ) ))
( "intercommunication"  n ( i2 n t @ k @ m y uu2 n i k ei1 sh @ n ) ((pos "L@$" ) ))
( "intercommunion"  n ( i2 n t @ k @ m y uu1 n i@ n ) ((pos "K6$" ) ))
( "intercommunions"  n ( i2 n t @ k @ m y uu1 n i@ n z ) ((pos "Kj$" ) ))
( "intercoms"  n ( i1 n t @ k o m z ) ((pos "Kj%" ) ))
( "interconnect"  v ( i2 n t @ k @ n e1 k t ) ((pos "J0%" ) (subcat )))
( "interconnected"  v ( i2 n t @ k @ n e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "interconnectedness"  n ( i2 n t @ k @ n e1 k t i d n i s ) ((pos "L@%" ) ))
( "interconnecting"  v ( i2 n t @ k @ n e1 k t i ng ) ((pos "Jb%" ) (subcat )))
( "interconnection"  n ( i2 n t @ k @ n e1 k sh @ n ) ((pos "M6%" ) ))
( "interconnections"  n ( i2 n t @ k @ n e1 k sh @ n z ) ((pos "Mj%" ) ))
( "interconnects"  v ( i2 n t @ k @ n e1 k t s ) ((pos "Ja%" ) (subcat )))
( "intercontinental"  j ( i2 n t @ k o2 n t i n e1 n t l ) ((pos "OA%" ) ))
( "intercourse"  n ( i1 n t @ k oo s ) ((pos "L@%" ) ))
( "interdenominational"  j ( i2 n t @ d i n o2 m i n ei1 sh @ n l ) ((pos "OA%" ) ))
( "interdepartmental"  j ( i2 n t @ d ii2 p aa t m e1 n t l ) ((pos "OA%" ) ))
( "interdepartmentally"  a ( i2 n t @ d ii2 p aa t m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "interdependence"  n ( i2 n t @ d i p e1 n d @ n s ) ((pos "K6%" ) ))
( "interdependences"  n ( i2 n t @ d i p e1 n d @ n s i z ) ((pos "Kj%" ) ))
( "interdependent"  j ( i2 n t @ d i p e1 n d @ n t ) ((pos "OA%" ) ))
( "interdict"  n ( i1 n t @ d i k t ) ((pos "K6$" ) ))
( "interdict"  v ( i2 n t @ d i1 k t ) ((pos "H0$" ) (subcat )))
( "interdicted"  v ( i2 n t @ d i1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "interdicting"  v ( i2 n t @ d i1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "interdiction"  n ( i2 n t @ d i1 k sh @ n ) ((pos "M6$" ) ))
( "interdictions"  n ( i2 n t @ d i1 k sh @ n z ) ((pos "Mj$" ) ))
( "interdicts"  n ( i1 n t @ d i k t s ) ((pos "Kj$" ) ))
( "interdicts"  v ( i2 n t @ d i1 k t s ) ((pos "Ha$" ) (subcat )))
( "interdisciplinary"  j ( i2 n t @ d i2 s i p l i1 n @ r ii ) ((pos "OA%" ) ))
( "interest"  v ( i1 n t r @ s t ) ((pos "H0*" "M6*" ) (subcat "6A" )))
( "interested"  v ( i1 n t r @ s t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "interesting"  v ( i1 n t r @ s t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "interestingly"  a ( i1 n t r @ s t i ng l ii ) ((pos "Pu%" ) ))
( "interests"  v ( i1 n t r @ s t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "interface"  n ( i1 n t @ f ei s ) ((pos "K6%" ) ))
( "interfaces"  n ( i1 n t @ f ei s i z ) ((pos "Kj%" ) ))
( "interfere"  v ( i2 n t @ f i@1 r ) ((pos "I2%" ) (subcat "2A" )))
( "interfered"  v ( i2 n t @ f i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "interference"  n ( i2 n t @ f i@1 r @ n s ) ((pos "L@%" ) ))
( "interferes"  v ( i2 n t @ f i@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "interfering"  v ( i2 n t @ f i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "interim"  n ( i1 n t @ r i m ) ((pos "L@%" ) ))
( "interior"  n ( i1 n t i@1 r i@ r ) ((pos "K6%" "OA%" ) ))
( "interiors"  n ( i1 n t i@1 r i@ z ) ((pos "Kj%" ) ))
( "interject"  v ( i2 n t @ jh e1 k t ) ((pos "H0%" ) (subcat )))
( "interjected"  v ( i2 n t @ jh e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "interjecting"  v ( i2 n t @ jh e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "interjection"  n ( i2 n t @ jh e1 k sh @ n ) ((pos "K6%" ) ))
( "interjections"  n ( i2 n t @ jh e1 k sh @ n z ) ((pos "Kj%" ) ))
( "interjects"  v ( i2 n t @ jh e1 k t s ) ((pos "Ha%" ) (subcat )))
( "interlace"  v ( i2 n t @ l ei1 s ) ((pos "J2$" ) (subcat "2A" "6A" )))
( "interlaced"  v ( i2 n t @ l ei1 s t ) ((pos "Jc$" "Jd%" ) (subcat "2A" "6A" )))
( "interlaces"  v ( i2 n t @ l ei1 s i z ) ((pos "Ja$" ) (subcat "2A" "6A" )))
( "interlacing"  v ( i2 n t @ l ei1 s i ng ) ((pos "Jb$" ) (subcat "2A" "6A" )))
( "interlard"  v ( i2 n t @ l aa1 d ) ((pos "H0$" ) (subcat )))
( "interlarded"  v ( i2 n t @ l aa1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "interlarding"  v ( i2 n t @ l aa1 d i ng ) ((pos "Hb$" ) (subcat )))
( "interlards"  v ( i2 n t @ l aa1 d z ) ((pos "Ha$" ) (subcat )))
( "interleave"  v ( i2 n t @ l ii1 v ) ((pos "H2$" ) (subcat "6A" )))
( "interleaved"  v ( i2 n t @ l ii1 v d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "interleaves"  v ( i2 n t @ l ii1 v z ) ((pos "Ha$" ) (subcat "6A" )))
( "interleaving"  v ( i2 n t @ l ii1 v i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "interlink"  v ( i2 n t @ l i1 ng k ) ((pos "J0$" ) (subcat "2A" )))
( "interlinked"  v ( i2 n t @ l i1 ng k t ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "interlinking"  v ( i2 n t @ l i1 ng k i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "interlinks"  v ( i2 n t @ l i1 ng k s ) ((pos "Ja$" ) (subcat "2A" )))
( "interlock"  v ( i2 n t @ l o1 k ) ((pos "J0%" ) (subcat "2A" )))
( "interlocked"  v ( i2 n t @ l o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "interlocking"  v ( i2 n t @ l o1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "interlocks"  v ( i2 n t @ l o1 k s ) ((pos "Ja$" ) (subcat "2A" )))
( "interlocutor"  n ( i2 n t @ l o1 k y u t @ r ) ((pos "K6%" ) ))
( "interlocutors"  n ( i2 n t @ l o1 k y u t @ z ) ((pos "Kj%" ) ))
( "interloper"  n ( i1 n t @ l ou p @ r ) ((pos "K6%" ) ))
( "interlopers"  n ( i1 n t @ l ou p @ z ) ((pos "Kj%" ) ))
( "interlude"  n ( i1 n t @ l uu d ) ((pos "K6%" ) ))
( "interludes"  n ( i1 n t @ l uu d z ) ((pos "Kj%" ) ))
( "intermarriage"  n ( i2 n t @ m a1 r i jh ) ((pos "M6%" ) ))
( "intermarriages"  n ( i2 n t @ m a1 r i jh i z ) ((pos "Mj%" ) ))
( "intermarried"  v ( i2 n t @ m a1 r i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "intermarries"  v ( i2 n t @ m a1 r i z ) ((pos "Ia$" ) (subcat "2A" )))
( "intermarry"  v ( i2 n t @ m a1 r ii ) ((pos "I3%" ) (subcat "2A" )))
( "intermarrying"  v ( i2 n t @ m a1 r i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "intermediaries"  n ( i2 n t @ m ii1 d i@ r i z ) ((pos "Kj%" ) ))
( "intermediary"  n ( i2 n t @ m ii1 d i@ r ii ) ((pos "K8%" ) ))
( "intermediate"  n ( i2 n t @ m ii1 d i@ t ) ((pos "K6%" "OA%" ) ))
( "intermediately"  a ( i2 n t @ m ii1 d i@ t l ii ) ((pos "Pu$" ) ))
( "intermediates"  n ( i2 n t @ m ii1 d i@ t s ) ((pos "Kj$" ) ))
( "interment"  n ( i1 n t @@1 m @ n t ) ((pos "M6$" ) ))
( "interments"  n ( i1 n t @@1 m @ n t s ) ((pos "Mj$" ) ))
( "intermezzi"  n ( i2 n t @ m e1 t s ii ) ((pos "Kj$" ) ))
( "intermezzo"  n ( i2 n t @ m e1 t s ou ) ((pos "K6%" ) ))
( "intermezzos"  n ( i2 n t @ m e1 t s ou z ) ((pos "Kj$" ) ))
( "interminable"  j ( i1 n t @@1 m i n @ b l ) ((pos "OA%" ) ))
( "interminably"  a ( i1 n t @@1 m i n @ b l ii ) ((pos "Pu%" ) ))
( "intermingle"  v ( i2 n t @ m i1 ng g l ) ((pos "J2%" ) (subcat "6A" )))
( "intermingled"  v ( i2 n t @ m i1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "intermingles"  v ( i2 n t @ m i1 ng g l z ) ((pos "Ja%" ) (subcat "6A" )))
( "intermingling"  v ( i2 n t @ m i1 ng g l i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "intermission"  n ( i2 n t @ m i1 sh @ n ) ((pos "K6%" ) ))
( "intermissions"  n ( i2 n t @ m i1 sh @ n z ) ((pos "Kj%" ) ))
( "intermittent"  j ( i2 n t @ m i1 t n t ) ((pos "OA%" ) ))
( "intermittently"  a ( i2 n t @ m i1 t n t l ii ) ((pos "Pu%" ) ))
( "intermix"  v ( i2 n t @ m i1 k s ) ((pos "J1$" ) ))
( "intermixed"  v ( i2 n t @ m i1 k s t ) ((pos "Jc$" "Jd%" ) ))
( "intermixes"  v ( i2 n t @ m i1 k s i z ) ((pos "Ja$" ) ))
( "intermixing"  v ( i2 n t @ m i1 k s i ng ) ((pos "Jb$" ) ))
( "intermixture"  n ( i2 n t @ m i1 k s ch @ r ) ((pos "K6$" ) ))
( "intermixtures"  n ( i2 n t @ m i1 k s ch @ z ) ((pos "Kj$" ) ))
( "intern"  n ( i1 n t @@ n ) ((pos "K6%" ) ))
( "intern"  v ( i1 n t @@1 n ) ((pos "H0%" ) (subcat )))
( "internal"  j ( i1 n t @@1 n l ) ((pos "OA%" ) ))
( "internalize"  v ( i1 n t @@1 n @ l ai z ) ((pos "H2%" ) (subcat )))
( "internalized"  v ( i1 n t @@1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "internalizes"  v ( i1 n t @@1 n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "internalizing"  v ( i1 n t @@1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "internally"  a ( i1 n t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "international"  n ( i2 n t @ n a1 sh @ n l ) ((pos "K6%" "OA%" ) ))
( "internationale"  n ( i2 n t @ n a sh @ n aa1 l ) ((pos "K6%" ) ))
( "internationales"  n ( i2 n t @ n a sh @ n aa1 l z ) ((pos "Kj$" ) ))
( "internationalism"  n ( i2 n t @ n a1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "internationalist"  n ( i2 n t @ n a1 sh @ n @ l i s t ) ((pos "K6%" ) ))
( "internationalists"  n ( i2 n t @ n a1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "internationalization"  n ( i2 n t @ n a2 sh @ n @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "internationalizations"  n ( i2 n t @ n a2 sh @ n @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "internationalize"  v ( i2 n t @ n a1 sh @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "internationalized"  v ( i2 n t @ n a1 sh @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "internationalizes"  v ( i2 n t @ n a1 sh @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "internationalizing"  v ( i2 n t @ n a1 sh @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "internationally"  a ( i2 n t @ n a1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "internationals"  n ( i2 n t @ n a1 sh @ n l z ) ((pos "Kj%" ) ))
( "interne"  n ( i1 n t @@ n ) ((pos "K6$" ) ))
( "internecine"  j ( i2 n t @ n ii1 s ai n ) ((pos "OA%" ) ))
( "interned"  v ( i1 n t @@1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "internee"  n ( i2 n t @@ n ii1 ) ((pos "K6%" ) ))
( "internees"  n ( i2 n t @@ n ii1 z ) ((pos "Kj%" ) ))
( "internes"  n ( i1 n t @@ n z ) ((pos "Kj$" ) ))
( "interning"  v ( i1 n t @@1 n i ng ) ((pos "Hb$" ) (subcat )))
( "internment"  n ( i1 n t @@1 n m @ n t ) ((pos "L@%" ) ))
( "interns"  n ( i1 n t @@ n z ) ((pos "Kj%" ) ))
( "interns"  v ( i1 n t @@1 n z ) ((pos "Ha%" ) (subcat )))
( "interpellate"  v ( i1 n t @@1 p @ l ei t ) ((pos "H2$" ) (subcat )))
( "interpellated"  v ( i1 n t @@1 p @ l ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "interpellates"  v ( i1 n t @@1 p @ l ei t s ) ((pos "Ha$" ) (subcat )))
( "interpellating"  v ( i1 n t @@1 p @ l ei t i ng ) ((pos "Hb$" ) (subcat )))
( "interpellation"  n ( i1 n t @@2 p @ l ei1 sh @ n ) ((pos "M6$" ) ))
( "interpellations"  n ( i1 n t @@2 p @ l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "interphone"  n ( i1 n t @ f ou n ) ((pos "K6$" ) ))
( "interphones"  n ( i1 n t @ f ou n z ) ((pos "Kj$" ) ))
( "interplanetary"  j ( i2 n t @ p l a1 n i t r ii ) ((pos "OA%" ) ))
( "interplay"  n ( i1 n t @ p l ei ) ((pos "L@%" ) ))
( "interpolate"  v ( i1 n t @@1 p @ l ei t ) ((pos "H2%" ) (subcat )))
( "interpolated"  v ( i1 n t @@1 p @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "interpolates"  v ( i1 n t @@1 p @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "interpolating"  v ( i1 n t @@1 p @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "interpolation"  n ( i1 n t @@2 p @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "interpolations"  n ( i1 n t @@2 p @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "interpose"  v ( i2 n t @ p ou1 z ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "interposed"  v ( i2 n t @ p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "interposes"  v ( i2 n t @ p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "interposing"  v ( i2 n t @ p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "interposition"  n ( i2 n t @ p @ z i1 sh @ n ) ((pos "M6$" ) ))
( "interpositions"  n ( i2 n t @ p @ z i1 sh @ n z ) ((pos "Mj$" ) ))
( "interpret"  v ( i1 n t @@1 p r i t ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "interpretation"  n ( i1 n t @@2 p r i t ei1 sh @ n ) ((pos "M6%" ) ))
( "interpretations"  n ( i1 n t @@2 p r i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "interpretative"  j ( i1 n t @@1 p r i t @ t i v ) ((pos "OA%" ) ))
( "interpreted"  v ( i1 n t @@1 p r i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "interpreter"  n ( i1 n t @@1 p r i t @ r ) ((pos "K6%" ) ))
( "interpreters"  n ( i1 n t @@1 p r i t @ z ) ((pos "Kj%" ) ))
( "interpreting"  v ( i1 n t @@1 p r i t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "interprets"  v ( i1 n t @@1 p r i t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "interracial"  j ( i2 n t @ r ei1 sh l ) ((pos "OA%" ) ))
( "interred"  v ( i1 n t @@1 d ) ((pos "Hc%" "Hd%" ) ))
( "interregna"  n ( i2 n t @ r e1 g n @ ) ((pos "Kj$" ) ))
( "interregnum"  n ( i2 n t @ r e1 g n @ m ) ((pos "K6%" ) ))
( "interregnums"  n ( i2 n t @ r e1 g n @ m z ) ((pos "Kj$" ) ))
( "interrelate"  v ( i2 n t @ r i l ei1 t ) ((pos "J2%" ) (subcat "2A" )))
( "interrelated"  v ( i2 n t @ r i l ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "interrelates"  v ( i2 n t @ r i l ei1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "interrelating"  v ( i2 n t @ r i l ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "interrelation"  n ( i2 n t @ r i l ei1 sh @ n ) ((pos "M6%" ) ))
( "interrelations"  n ( i2 n t @ r i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "interrelationship"  n ( i2 n t @ r i l ei1 sh @ n sh i p ) ((pos "M6%" ) ))
( "interrelationships"  n ( i2 n t @ r i l ei1 sh @ n sh i p s ) ((pos "Mj%" ) ))
( "interring"  v ( i1 n t @@1 r i ng ) ((pos "Hb%" ) ))
( "interrogate"  v ( i1 n t e1 r @ g ei t ) ((pos "H2%" ) (subcat )))
( "interrogated"  v ( i1 n t e1 r @ g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "interrogates"  v ( i1 n t e1 r @ g ei t s ) ((pos "Ha%" ) (subcat )))
( "interrogating"  v ( i1 n t e1 r @ g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "interrogation"  n ( i1 n t e2 r @ g ei1 sh @ n ) ((pos "M6%" ) ))
( "interrogations"  n ( i1 n t e2 r @ g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "interrogative"  n ( i2 n t @ r o1 g @ t i v ) ((pos "K6%" "OA%" ) ))
( "interrogatively"  a ( i2 n t @ r o1 g @ t i v l ii ) ((pos "Pu%" ) ))
( "interrogatives"  n ( i2 n t @ r o1 g @ t i v z ) ((pos "Kj%" ) ))
( "interrogator"  n ( i1 n t e1 r @ g ei t @ r ) ((pos "K6%" ) ))
( "interrogators"  n ( i1 n t e1 r @ g ei t @ z ) ((pos "Kj%" ) ))
( "interrogatory"  j ( i2 n t @ r o1 g @ t r ii ) ((pos "OA%" ) ))
( "interrupt"  v ( i2 n t @ r uh1 p t ) ((pos "J0%" ) (subcat "2A" )))
( "interrupted"  v ( i2 n t @ r uh1 p t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "interrupter"  n ( i2 n t @ r uh1 p t @ r ) ((pos "K6%" ) ))
( "interrupters"  n ( i2 n t @ r uh1 p t @ z ) ((pos "Kj%" ) ))
( "interrupting"  v ( i2 n t @ r uh1 p t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "interruption"  n ( i2 n t @ r uh1 p sh @ n ) ((pos "M6%" ) ))
( "interruptions"  n ( i2 n t @ r uh1 p sh @ n z ) ((pos "Mj%" ) ))
( "interrupts"  v ( i2 n t @ r uh1 p t s ) ((pos "Ja%" ) (subcat "2A" )))
( "inters"  v ( i1 n t @@1 z ) ((pos "Ha$" ) ))
( "intersect"  v ( i2 n t @ s e1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "intersected"  v ( i2 n t @ s e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "intersecting"  v ( i2 n t @ s e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "intersection"  n ( i2 n t @ s e1 k sh @ n ) ((pos "M6%" ) ))
( "intersections"  n ( i2 n t @ s e1 k sh @ n z ) ((pos "Mj%" ) ))
( "intersects"  v ( i2 n t @ s e1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "intersperse"  v ( i2 n t @ s p @@1 s ) ((pos "H2%" ) (subcat )))
( "interspersed"  v ( i2 n t @ s p @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "intersperses"  v ( i2 n t @ s p @@1 s i z ) ((pos "Ha%" ) (subcat )))
( "interspersing"  v ( i2 n t @ s p @@1 s i ng ) ((pos "Hb%" ) (subcat )))
( "interstate"  j ( i2 n t @ s t ei1 t ) ((pos "OA%" ) ))
( "interstellar"  j ( i2 n t @ s t e1 l @ r ) ((pos "OA%" ) ))
( "interstice"  n ( i1 n t @@1 s t i s ) ((pos "K6$" ) ))
( "interstices"  n ( i1 n t @@1 s t i s i z ) ((pos "Kj%" ) ))
( "intertribal"  j ( i2 n t @ t r ai1 b l ) ((pos "OA%" ) ))
( "intertwine"  v ( i2 n t @ t w ai1 n ) ((pos "J2%" ) (subcat "2A" )))
( "intertwined"  v ( i2 n t @ t w ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "intertwines"  v ( i2 n t @ t w ai1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "intertwining"  v ( i2 n t @ t w ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "interval"  n ( i1 n t @ v @ l ) ((pos "K6%" ) ))
( "intervals"  n ( i1 n t @ v @ l z ) ((pos "Kj%" ) ))
( "intervene"  v ( i2 n t @ v ii1 n ) ((pos "I2%" ) (subcat "2A" )))
( "intervened"  v ( i2 n t @ v ii1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "intervenes"  v ( i2 n t @ v ii1 n z ) ((pos "Ia%" ) (subcat "2A" )))
( "intervening"  v ( i2 n t @ v ii1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "intervention"  n ( i2 n t @ v e1 n sh @ n ) ((pos "M6%" ) ))
( "interventions"  n ( i2 n t @ v e1 n sh @ n z ) ((pos "Mj%" ) ))
( "interview"  v ( i1 n t @ v y uu ) ((pos "H0%" "K6%" ) (subcat )))
( "interviewed"  v ( i1 n t @ v y uu d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "interviewer"  n ( i1 n t @ v y uu @ r ) ((pos "K6%" ) ))
( "interviewers"  n ( i1 n t @ v y uu @ z ) ((pos "Kj%" ) ))
( "interviewing"  v ( i1 n t @ v y uu i ng ) ((pos "Hb%" ) (subcat )))
( "interviews"  v ( i1 n t @ v y uu z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "interweave"  v ( i2 n t @ w ii1 v ) ((pos "H5%" ) (subcat "6A" )))
( "interweaves"  v ( i2 n t @ w ii1 v z ) ((pos "Ha%" ) (subcat "6A" )))
( "interweaving"  v ( i2 n t @ w ii1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "interwove"  v ( i2 n t @ w ou1 v ) ((pos "Hc%" ) (subcat "6A" )))
( "interwoven"  v ( i2 n t @ w ou1 v n ) ((pos "Hd%" ) (subcat "6A" )))
( "intestate"  j ( i1 n t e1 s t ei t ) ((pos "OA$" ) ))
( "intestinal"  j ( i1 n t e1 s t i n l ) ((pos "OA%" ) ))
( "intestine"  n ( i1 n t e1 s t i n ) ((pos "K6%" ) ))
( "intestines"  n ( i1 n t e1 s t i n z ) ((pos "Kj%" ) ))
( "intimacies"  n ( i1 n t i m @ s i z ) ((pos "Mj%" ) ))
( "intimacy"  n ( i1 n t i m @ s ii ) ((pos "M8%" ) ))
( "intimate"  n ( i1 n t i m @ t ) ((pos "K6%" "OA%" ) ))
( "intimate"  v ( i1 n t i m ei t ) ((pos "H2%" ) (subcat "6A" "9" )))
( "intimated"  v ( i1 n t i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "intimately"  a ( i1 n t i m @ t l ii ) ((pos "Pu%" ) ))
( "intimates"  n ( i1 n t i m @ t s ) ((pos "Kj%" ) ))
( "intimates"  v ( i1 n t i m ei t s ) ((pos "Ha$" ) (subcat "6A" "9" )))
( "intimating"  v ( i1 n t i m ei t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "intimation"  n ( i2 n t i m ei1 sh @ n ) ((pos "M6%" ) ))
( "intimations"  n ( i2 n t i m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "intimidate"  v ( i1 n t i1 m i d ei t ) ((pos "H2%" ) (subcat "6A" )))
( "intimidated"  v ( i1 n t i1 m i d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "intimidates"  v ( i1 n t i1 m i d ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "intimidating"  v ( i1 n t i1 m i d ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "intimidation"  n ( i1 n t i2 m i d ei1 sh @ n ) ((pos "L@%" ) ))
( "into"  in ( i1 n t @ ) ((pos "T-*" ) ))
( "intolerable"  j ( i1 n t o1 l @ r @ b l ) ((pos "OA%" ) ))
( "intolerably"  a ( i1 n t o1 l @ r @ b l ii ) ((pos "Pu%" ) ))
( "intolerance"  n ( i1 n t o1 l @ r @ n s ) ((pos "L@%" ) ))
( "intolerant"  j ( i1 n t o1 l @ r @ n t ) ((pos "OA%" ) ))
( "intolerantly"  a ( i1 n t o1 l @ r @ n t l ii ) ((pos "Pu%" ) ))
( "intonation"  n ( i2 n t @ n ei1 sh @ n ) ((pos "L@%" ) ))
( "intone"  v ( i1 n t ou1 n ) ((pos "J2%" ) (subcat "2A" )))
( "intoned"  v ( i1 n t ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "intones"  v ( i1 n t ou1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "intoning"  v ( i1 n t ou1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "intoxicant"  n ( i1 n t o1 k s i k @ n t ) ((pos "K6%" "OA%" ) ))
( "intoxicants"  n ( i1 n t o1 k s i k @ n t s ) ((pos "Kj%" ) ))
( "intoxicate"  v ( i1 n t o1 k s i k ei t ) ((pos "H2%" ) (subcat )))
( "intoxicated"  v ( i1 n t o1 k s i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "intoxicates"  v ( i1 n t o1 k s i k ei t s ) ((pos "Ha%" ) (subcat )))
( "intoxicating"  v ( i1 n t o1 k s i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "intoxication"  n ( i1 n t o2 k s i k ei1 sh @ n ) ((pos "L@%" ) ))
( "intra-uterine"  j ( i2 n t r @ - y uu1 t @ r ai n ) ((pos "OA$" ) ))
( "intractability"  n ( i1 n t r a2 k t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "intractable"  j ( i2 n t r a1 k t @ b l ) ((pos "OA%" ) ))
( "intramural"  j ( i2 n t r @ m y u@1 r @ l ) ((pos "OA$" ) ))
( "intransigence"  n ( i1 n t r a1 n s i jh @ n s ) ((pos "L@%" ) ))
( "intransigent"  j ( i1 n t r a1 n s i jh @ n t ) ((pos "OA%" ) ))
( "intransitive"  j ( i1 n t r a1 n s @ t i v ) ((pos "OA%" ) ))
( "intransitively"  a ( i1 n t r a1 n s @ t i v l ii ) ((pos "Pu%" ) ))
( "intravenous"  j ( i2 n t r @ v ii1 n @ s ) ((pos "OA%" ) ))
( "intravenously"  a ( i2 n t r @ v ii1 n @ s l ii ) ((pos "Pu%" ) ))
( "intrench"  v ( i1 n t r e1 n ch ) ((pos "H1$" ) (subcat )))
( "intrenched"  v ( i1 n t r e1 n ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "intrenches"  v ( i1 n t r e1 n ch i z ) ((pos "Ha$" ) (subcat )))
( "intrenching"  v ( i1 n t r e1 n ch i ng ) ((pos "Hb$" ) (subcat )))
( "intrepid"  j ( i1 n t r e1 p i d ) ((pos "OA%" ) ))
( "intrepidities"  n ( i2 n t r i p i1 d i t i z ) ((pos "Mj$" ) ))
( "intrepidity"  n ( i2 n t r i p i1 d i t ii ) ((pos "M8$" ) ))
( "intrepidly"  a ( i1 n t r e1 p i d l ii ) ((pos "Pu%" ) ))
( "intricacies"  n ( i1 n t r i k @ s i z ) ((pos "Mj%" ) ))
( "intricacy"  n ( i1 n t r i k @ s ii ) ((pos "M8%" ) ))
( "intricate"  j ( i1 n t r i k @ t ) ((pos "OA%" ) ))
( "intricately"  a ( i1 n t r i k @ t l ii ) ((pos "Pu%" ) ))
( "intrigue"  v ( i1 n t r ii1 g ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" )))
( "intrigued"  v ( i1 n t r ii1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "intrigues"  v ( i1 n t r ii1 g z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" )))
( "intriguing"  v ( i1 n t r ii1 g i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "intrinsic"  j ( i1 n t r i1 n s i k ) ((pos "OA%" ) ))
( "intrinsically"  a ( i1 n t r i1 n s i k l ii ) ((pos "Pu%" ) ))
( "intro"  n ( i1 n t r ou ) ((pos "Ki%" ) ))
( "introduce"  v ( i2 n t r @ d y uu1 s ) ((pos "H2%" ) (subcat "6A" "14" )))
( "introduced"  v ( i2 n t r @ d y uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "introduces"  v ( i2 n t r @ d y uu1 s i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "introducing"  v ( i2 n t r @ d y uu1 s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "introduction"  n ( i2 n t r @ d uh1 k sh @ n ) ((pos "M6%" ) ))
( "introductions"  n ( i2 n t r @ d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "introductory"  j ( i2 n t r @ d uh1 k t @ r ii ) ((pos "OA%" ) ))
( "introspect"  v ( i2 n t r @ s p e1 k t ) ((pos "I0$" ) (subcat )))
( "introspected"  v ( i2 n t r @ s p e1 k t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "introspecting"  v ( i2 n t r @ s p e1 k t i ng ) ((pos "Ib$" ) (subcat )))
( "introspection"  n ( i2 n t r @ s p e1 k sh @ n ) ((pos "L@%" ) ))
( "introspective"  j ( i2 n t r @ s p e1 k t i v ) ((pos "OA%" ) ))
( "introspects"  v ( i2 n t r @ s p e1 k t s ) ((pos "Ia$" ) (subcat )))
( "introversion"  n ( i2 n t r @ v @@1 sh @ n ) ((pos "L@%" ) ))
( "introvert"  n ( i1 n t r @ v @@ t ) ((pos "K6%" ) ))
( "introvert"  v ( i2 n t r @ v @@1 t ) ((pos "H0$" ) (subcat )))
( "introverted"  v ( i2 n t r @ v @@1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "introverting"  v ( i2 n t r @ v @@1 t i ng ) ((pos "Hb$" ) (subcat )))
( "introverts"  n ( i1 n t r @ v @@ t s ) ((pos "Kj%" ) ))
( "introverts"  v ( i2 n t r @ v @@1 t s ) ((pos "Ha$" ) (subcat )))
( "intrude"  v ( i1 n t r uu1 d ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "intruded"  v ( i1 n t r uu1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "intruder"  n ( i1 n t r uu1 d @ r ) ((pos "K6%" ) ))
( "intruders"  n ( i1 n t r uu1 d @ z ) ((pos "Kj%" ) ))
( "intrudes"  v ( i1 n t r uu1 d z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "intruding"  v ( i1 n t r uu1 d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "intrusion"  n ( i1 n t r uu1 zh n ) ((pos "M6%" ) ))
( "intrusions"  n ( i1 n t r uu1 zh n z ) ((pos "Mj%" ) ))
( "intrusive"  j ( i1 n t r uu1 s i v ) ((pos "OA%" ) ))
( "intrust"  v ( i1 n t r uh1 s t ) ((pos "H0$" ) (subcat )))
( "intrusted"  v ( i1 n t r uh1 s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "intrusting"  v ( i1 n t r uh1 s t i ng ) ((pos "Hb$" ) (subcat )))
( "intrusts"  v ( i1 n t r uh1 s t s ) ((pos "Ha$" ) (subcat )))
( "intuit"  v ( i1 n t y uu1 i t ) ((pos "J0$" ) (subcat "2A" )))
( "intuited"  v ( i1 n t y uu1 i t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "intuiting"  v ( i1 n t y uu1 i t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "intuition"  n ( i2 n t y uu i1 sh @ n ) ((pos "M6%" ) ))
( "intuitions"  n ( i2 n t y uu i1 sh @ n z ) ((pos "Mj%" ) ))
( "intuitive"  j ( i1 n t y uu1 i t i v ) ((pos "OA%" ) ))
( "intuitively"  a ( i1 n t y uu1 i t i v l ii ) ((pos "Pu%" ) ))
( "intuits"  v ( i1 n t y uu1 i t s ) ((pos "Ja$" ) (subcat "2A" )))
( "intumescence"  n ( i2 n t y uu m e1 s n s ) ((pos "L@$" ) ))
( "inundate"  v ( i1 n uh n d ei t ) ((pos "H2$" ) (subcat "6A" )))
( "inundated"  v ( i1 n uh n d ei t i d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "inundates"  v ( i1 n uh n d ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "inundating"  v ( i1 n uh n d ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "inundation"  n ( i2 n uh n d ei1 sh @ n ) ((pos "M6%" ) ))
( "inundations"  n ( i2 n uh n d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "inure"  v ( i1 n y u@1 r ) ((pos "H2$" ) (subcat )))
( "inured"  v ( i1 n y u@1 d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "inures"  v ( i1 n y u@1 z ) ((pos "Ha$" ) (subcat )))
( "inuring"  v ( i1 n y u@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "invade"  v ( i1 n v ei1 d ) ((pos "H2%" ) (subcat )))
( "invaded"  v ( i1 n v ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "invader"  n ( i1 n v ei1 d @ r ) ((pos "K6%" ) ))
( "invaders"  n ( i1 n v ei1 d @ z ) ((pos "Kj%" ) ))
( "invades"  v ( i1 n v ei1 d z ) ((pos "Ha%" ) (subcat )))
( "invading"  v ( i1 n v ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "invalid"  v ( i1 n v @ l i d ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "invalid"  j ( i1 n v a1 l i d ) ((pos "OA%" ) ))
( "invalidate"  v ( i1 n v a1 l i d ei t ) ((pos "H2%" ) (subcat )))
( "invalidated"  v ( i1 n v a1 l i d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "invalidates"  v ( i1 n v a1 l i d ei t s ) ((pos "Ha%" ) (subcat )))
( "invalidating"  v ( i1 n v a1 l i d ei t i ng ) ((pos "Hb%" ) (subcat )))
( "invalidation"  n ( i1 n v a2 l i d ei1 sh @ n ) ((pos "M6%" ) ))
( "invalidations"  n ( i1 n v a2 l i d ei1 sh @ n z ) ((pos "Mj$" ) ))
( "invalided"  v ( i1 n v @ l i d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "invaliding"  v ( i1 n v @ l i d i ng ) ((pos "Hb$" ) (subcat )))
( "invalidism"  n ( i1 n v @ l i d i z @ m ) ((pos "L@$" ) ))
( "invalidity"  n ( i2 n v @ l i1 d i t ii ) ((pos "L@%" ) ))
( "invalids"  v ( i1 n v @ l i d z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "invaluable"  j ( i1 n v a1 l y u@ b l ) ((pos "OA%" ) ))
( "invariable"  j ( i1 n v e@1 r i@ b l ) ((pos "OA%" ) ))
( "invariably"  a ( i1 n v e@1 r i@ b l ii ) ((pos "Pu%" ) ))
( "invasion"  n ( i1 n v ei1 zh n ) ((pos "M6%" ) ))
( "invasions"  n ( i1 n v ei1 zh n z ) ((pos "Mj%" ) ))
( "invasive"  j ( i1 n v ei1 s i v ) ((pos "OA$" ) ))
( "invective"  n ( i1 n v e1 k t i v ) ((pos "L@%" ) ))
( "inveigh"  v ( i1 n v ei1 ) ((pos "I0$" ) (subcat )))
( "inveighed"  v ( i1 n v ei1 d ) ((pos "Ic$" "Id$" ) (subcat )))
( "inveighing"  v ( i1 n v ei1 i ng ) ((pos "Ib$" ) (subcat )))
( "inveighs"  v ( i1 n v ei1 z ) ((pos "Ia$" ) (subcat )))
( "inveigle"  v ( i1 n v ei1 g l ) ((pos "H2%" ) (subcat )))
( "inveigled"  v ( i1 n v ei1 g l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inveigles"  v ( i1 n v ei1 g l z ) ((pos "Ha%" ) (subcat )))
( "inveigling"  v ( i1 n v ei1 g l i ng ) ((pos "Hb%" ) (subcat )))
( "invent"  v ( i1 n v e1 n t ) ((pos "H0%" ) (subcat )))
( "invented"  v ( i1 n v e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inventing"  v ( i1 n v e1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "invention"  n ( i1 n v e1 n sh @ n ) ((pos "M6%" ) ))
( "inventions"  n ( i1 n v e1 n sh @ n z ) ((pos "Mj%" ) ))
( "inventive"  j ( i1 n v e1 n t i v ) ((pos "OA%" ) ))
( "inventively"  a ( i1 n v e1 n t i v l ii ) ((pos "Pu%" ) ))
( "inventor"  n ( i1 n v e1 n t @ r ) ((pos "K6%" ) ))
( "inventories"  n ( i1 n v @ n t r i z ) ((pos "Kj%" ) ))
( "inventors"  n ( i1 n v e1 n t @ z ) ((pos "Kj%" ) ))
( "inventory"  n ( i1 n v @ n t r ii ) ((pos "K8%" ) ))
( "invents"  v ( i1 n v e1 n t s ) ((pos "Ha%" ) (subcat )))
( "inverse"  n ( i1 n v @@ s ) ((pos "L@$" ) ))
( "inverse"  j ( i1 n v @@1 s ) ((pos "OA%" ) ))
( "inversely"  a ( i1 n v @@1 s l ii ) ((pos "Pu%" ) ))
( "inversion"  n ( i1 n v @@1 sh @ n ) ((pos "M6%" ) ))
( "inversions"  n ( i1 n v @@1 sh @ n z ) ((pos "Mj%" ) ))
( "invert"  v ( i1 n v @@1 t ) ((pos "H0%" ) (subcat )))
( "invertebrate"  n ( i1 n v @@1 t i b r ei t ) ((pos "K6%" "OA%" ) ))
( "invertebrates"  n ( i1 n v @@1 t i b r ei t s ) ((pos "Kj%" ) ))
( "inverted"  v ( i1 n v @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "inverting"  v ( i1 n v @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "inverts"  v ( i1 n v @@1 t s ) ((pos "Ha%" ) (subcat )))
( "invest"  v ( i1 n v e1 s t ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "invested"  v ( i1 n v e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "investigate"  v ( i1 n v e1 s t i g ei t ) ((pos "H2%" ) (subcat )))
( "investigated"  v ( i1 n v e1 s t i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "investigates"  v ( i1 n v e1 s t i g ei t s ) ((pos "Ha%" ) (subcat )))
( "investigating"  v ( i1 n v e1 s t i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "investigation"  n ( i1 n v e2 s t i g ei1 sh @ n ) ((pos "M6%" ) ))
( "investigations"  n ( i1 n v e2 s t i g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "investigator"  n ( i1 n v e1 s t i g ei t @ r ) ((pos "K6%" ) ))
( "investigators"  n ( i1 n v e1 s t i g ei t @ z ) ((pos "Kj%" ) ))
( "investing"  v ( i1 n v e1 s t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "investiture"  n ( i1 n v e1 s t i ch @ r ) ((pos "K6$" ) ))
( "investitures"  n ( i1 n v e1 s t i ch @ z ) ((pos "Kj$" ) ))
( "investment"  n ( i1 n v e1 s t m @ n t ) ((pos "M6%" ) ))
( "investments"  n ( i1 n v e1 s t m @ n t s ) ((pos "Mj%" ) ))
( "investor"  n ( i1 n v e1 s t @ r ) ((pos "K6%" ) ))
( "investors"  n ( i1 n v e1 s t @ z ) ((pos "Kj%" ) ))
( "invests"  v ( i1 n v e1 s t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "inveterate"  j ( i1 n v e1 t @ r @ t ) ((pos "OA%" ) ))
( "invidious"  j ( i1 n v i1 d i@ s ) ((pos "OA%" ) ))
( "invidiously"  a ( i1 n v i1 d i@ s l ii ) ((pos "Pu%" ) ))
( "invigilate"  v ( i1 n v i1 jh i l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "invigilated"  v ( i1 n v i1 jh i l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "invigilates"  v ( i1 n v i1 jh i l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "invigilating"  v ( i1 n v i1 jh i l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "invigilation"  n ( i1 n v i2 jh i l ei1 sh @ n ) ((pos "M6%" ) ))
( "invigilations"  n ( i1 n v i2 jh i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "invigilator"  n ( i1 n v i1 jh i l ei t @ r ) ((pos "K6%" ) ))
( "invigilators"  n ( i1 n v i1 jh i l ei t @ z ) ((pos "Kj%" ) ))
( "invigorate"  v ( i1 n v i1 g @ r ei t ) ((pos "H2%" ) (subcat )))
( "invigorated"  v ( i1 n v i1 g @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "invigorates"  v ( i1 n v i1 g @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "invigorating"  v ( i1 n v i1 g @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "invincibility"  n ( i1 n v i2 n s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "invincible"  j ( i1 n v i1 n s @ b l ) ((pos "OA%" ) ))
( "invincibly"  a ( i1 n v i1 n s @ b l ii ) ((pos "Pu%" ) ))
( "inviolable"  j ( i1 n v ai1 @ l @ b l ) ((pos "OA%" ) ))
( "inviolate"  j ( i2 n v ai1 @ l @ t ) ((pos "OA%" ) ))
( "invisibility"  n ( i1 n v i2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "invisible"  j ( i1 n v i1 z @ b l ) ((pos "OA%" ) ))
( "invisibly"  a ( i1 n v i1 z @ b l ii ) ((pos "Pu%" ) ))
( "invitation"  n ( i2 n v i t ei1 sh @ n ) ((pos "M6%" ) ))
( "invitations"  n ( i2 n v i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "invite"  n ( i1 n v ai t ) ((pos "K6%" ) ))
( "invite"  v ( i1 n v ai1 t ) ((pos "H2%" ) (subcat "6A" "15A" "15B" )))
( "invited"  v ( i1 n v ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" "15B" )))
( "invites"  n ( i1 n v ai t s ) ((pos "Kj%" ) ))
( "invites"  v ( i1 n v ai1 t s ) ((pos "Ha%" ) (subcat "6A" "15A" "15B" )))
( "inviting"  v ( i1 n v ai1 t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "15A" "15B" )))
( "invitingly"  a ( i1 n v ai1 t i ng l ii ) ((pos "Pu%" ) ))
( "invocation"  n ( i2 n v @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "invocations"  n ( i2 n v @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "invoice"  v ( i1 n v oi s ) ((pos "H2%" "K6%" ) ))
( "invoiced"  v ( i1 n v oi s t ) ((pos "Hc%" "Hd%" ) ))
( "invoices"  v ( i1 n v oi s i z ) ((pos "Ha%" "Kj%" ) ))
( "invoicing"  v ( i1 n v oi s i ng ) ((pos "Hb%" ) ))
( "invoke"  v ( i1 n v ou1 k ) ((pos "H2%" ) (subcat "6A" )))
( "invoked"  v ( i1 n v ou1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "invokes"  v ( i1 n v ou1 k s ) ((pos "Ha%" ) (subcat "6A" )))
( "invoking"  v ( i1 n v ou1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "involuntarily"  a ( i1 n v o1 l @ n t r @ l ii ) ((pos "Pu%" ) ))
( "involuntary"  j ( i1 n v o1 l @ n t r ii ) ((pos "OA%" ) ))
( "involute"  j ( i1 n v @ l uu t ) ((pos "OA$" ) ))
( "involution"  n ( i2 n v @ l uu1 sh @ n ) ((pos "K6$" ) ))
( "involutions"  n ( i2 n v @ l uu1 sh @ n z ) ((pos "Kj$" ) ))
( "involve"  v ( i1 n v o1 l v ) ((pos "H2%" ) (subcat "6A" "6B" "14" )))
( "involved"  v ( i1 n v o1 l v d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "6B" "14" )))
( "involvement"  n ( i1 n v o1 l v m @ n t ) ((pos "K6%" ) ))
( "involvements"  n ( i1 n v o1 l v m @ n t s ) ((pos "Kj%" ) ))
( "involves"  v ( i1 n v o1 l v z ) ((pos "Ha%" ) (subcat "6A" "6B" "14" )))
( "involving"  v ( i1 n v o1 l v i ng ) ((pos "Hb%" ) (subcat "6A" "6B" "14" )))
( "invulnerable"  j ( i1 n v uh1 l n @ r @ b l ) ((pos "OA%" ) ))
( "inward"  j ( i1 n w @ d ) ((pos "OA%" "Pu%" ) ))
( "inwardly"  a ( i1 n w @ d l ii ) ((pos "Pu%" ) ))
( "inwardness"  n ( i1 n w @ d n @ s ) ((pos "L@%" ) ))
( "inwards"  a ( i1 n w @ d z ) ((pos "Pu%" ) ))
( "inwrought"  j ( i2 n r oo1 t ) ((pos "OA$" ) ))
( "iodine"  n ( ai1 @ d ii n ) ((pos "L@%" ) ))
( "ion"  n ( ai1 @ n ) ((pos "K6$" ) ))
( "ionization"  n ( ai2 @ n ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "ionize"  v ( ai1 @ n ai z ) ((pos "J2$" ) (subcat "2A" )))
( "ionized"  v ( ai1 @ n ai z d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "ionizes"  v ( ai1 @ n ai z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "ionizing"  v ( ai1 @ n ai z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "ionosphere"  n ( ai1 o1 n @ s f i@ r ) ((pos "K6$" ) ))
( "ionospheres"  n ( ai1 o1 n @ s f i@ z ) ((pos "Kj$" ) ))
( "ions"  n ( ai1 @ n z ) ((pos "Kj$" ) ))
( "iota"  n ( ai1 ou1 t @ ) ((pos "K6$" ) ))
( "iotas"  n ( ai1 ou1 t @ z ) ((pos "Kj$" ) ))
( "ipse dixit"  n ( i2 p s i - d i1 k s i t ) ((pos "K6$" ) ))
( "ipse dixits"  n ( i2 p s i - d i1 k s i t s ) ((pos "Kj$" ) ))
( "ipso facto"  a ( i2 p s ou - f a1 k t ou ) ((pos "Pu%" ) ))
( "irascibility"  n ( i1 r a2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "irascible"  j ( i1 r a1 s @ b l ) ((pos "OA%" ) ))
( "irate"  j ( ai1 r ei1 t ) ((pos "OA%" ) ))
( "irately"  a ( ai1 r ei1 t l ii ) ((pos "Pu%" ) ))
( "ire"  n ( ai1 @ r ) ((pos "L@$" ) ))
( "ireful"  j ( ai1 @ f @ l ) ((pos "OA$" ) ))
( "iridescence"  n ( i2 r i d e1 s n s ) ((pos "L@%" ) ))
( "iridescent"  j ( i2 r i d e1 s n t ) ((pos "OA%" ) ))
( "iridium"  n ( i1 r i1 d i@ m ) ((pos "L@$" ) ))
( "iris"  n ( ai1 @ r i s ) ((pos "K7%" ) ))
( "irises"  n ( ai1 @ r i s i z ) ((pos "Kj%" ) ))
( "irk"  v ( @@1 k ) ((pos "H0$" ) ))
( "irked"  v ( @@1 k t ) ((pos "Hc$" "Hd%" ) ))
( "irking"  v ( @@1 k i ng ) ((pos "Hb$" ) ))
( "irks"  v ( @@1 k s ) ((pos "Ha%" ) ))
( "irksome"  j ( @@1 k s @ m ) ((pos "OA%" ) ))
( "iron"  v ( ai1 @ n ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "iron-foundries"  n ( ai1 @ n - f au n d r i z ) ((pos "Kj%" ) ))
( "iron-foundry"  n ( ai1 @ n - f au n d r ii ) ((pos "K8%" ) ))
( "iron-grey"  n ( ai2 @ n - g r ei1 ) ((pos "L@%" "OA%" ) ))
( "ironclad"  j ( ai1 @ n k l a d ) ((pos "OA$" ) ))
( "ironed"  v ( ai1 @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ironic"  j ( ai1 r o1 n i k ) ((pos "OA%" ) ))
( "ironical"  j ( ai1 r o1 n i k l ) ((pos "OA%" ) ))
( "ironically"  a ( ai1 r o1 n i k l ii ) ((pos "Pu%" ) ))
( "ironies"  n ( ai1 @ r @ n i z ) ((pos "Mj%" ) ))
( "ironing"  v ( ai1 @ n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ironing-board"  n ( ai1 @ n i ng - b oo d ) ((pos "K6%" ) ))
( "ironing-boards"  n ( ai1 @ n i ng - b oo d z ) ((pos "Kj%" ) ))
( "ironmonger"  n ( ai1 @ n m uh ng g @ r ) ((pos "K6%" ) ))
( "ironmongeries"  n ( ai1 @ n m uh ng g @ r i z ) ((pos "Mj$" ) ))
( "ironmongers"  n ( ai1 @ n m uh ng g @ z ) ((pos "Kj%" ) ))
( "ironmongery"  n ( ai1 @ n m uh ng g @ r ii ) ((pos "M8%" ) ))
( "ironmould"  n ( ai1 @ n m ou l d ) ((pos "K6$" ) ))
( "ironmoulds"  n ( ai1 @ n m ou l d z ) ((pos "Kj$" ) ))
( "irons"  v ( ai1 @ n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ironside"  n ( ai1 @ n s ai d ) ((pos "K6$" ) ))
( "ironsides"  n ( ai1 @ n s ai d z ) ((pos "Kj$" ) ))
( "ironware"  n ( ai1 @ n w e@ r ) ((pos "L@$" ) ))
( "ironwork"  n ( ai1 @ n w @@ k ) ((pos "K6%" ) ))
( "ironworks"  n ( ai1 @ n w @@ k s ) ((pos "K9$" ) ))
( "irony"  n ( ai1 @ r @ n ii ) ((pos "M8%" ) ))
( "irradiate"  v ( i1 r ei1 d i ei t ) ((pos "H2%" ) (subcat )))
( "irradiated"  v ( i1 r ei1 d i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "irradiates"  v ( i1 r ei1 d i ei t s ) ((pos "Ha%" ) (subcat )))
( "irradiating"  v ( i1 r ei1 d i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "irrational"  j ( i1 r a1 sh @ n l ) ((pos "OA%" ) ))
( "irrationally"  a ( i1 r a1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "irreconcilable"  j ( i1 r e2 k @ n s ai1 l @ b l ) ((pos "OA%" ) ))
( "irrecoverable"  j ( i2 r i k uh1 v @ r @ b l ) ((pos "OA%" ) ))
( "irredeemable"  j ( i2 r i d ii1 m @ b l ) ((pos "OA%" ) ))
( "irredentist"  n ( i2 r i d e1 n t i s t ) ((pos "K6$" ) ))
( "irredentists"  n ( i2 r i d e1 n t i s t s ) ((pos "Kj$" ) ))
( "irreducible"  j ( i2 r i d y uu1 s @ b l ) ((pos "OA%" ) ))
( "irrefutable"  j ( i2 r i f y uu1 t @ b l ) ((pos "OA%" ) ))
( "irregular"  n ( i1 r e1 g y u l @ r ) ((pos "K6$" "OA%" ) ))
( "irregularities"  n ( i1 r e2 g y u l a1 r i t i z ) ((pos "Mj%" ) ))
( "irregularity"  n ( i1 r e2 g y u l a1 r i t ii ) ((pos "M8%" ) ))
( "irregularly"  a ( i1 r e1 g y u l @ l ii ) ((pos "Pu%" ) ))
( "irregulars"  n ( i1 r e1 g y u l @ z ) ((pos "Kj$" ) ))
( "irrelevance"  n ( i1 r e1 l @ v @ n s ) ((pos "M6%" ) ))
( "irrelevances"  n ( i1 r e1 l @ v @ n s i z ) ((pos "Mj$" ) ))
( "irrelevancies"  n ( i1 r e1 l @ v @ n s i z ) ((pos "Mj$" ) ))
( "irrelevancy"  n ( i1 r e1 l @ v @ n s ii ) ((pos "M8$" ) ))
( "irrelevant"  j ( i1 r e1 l @ v @ n t ) ((pos "OA%" ) ))
( "irrelevantly"  a ( i1 r e1 l @ v @ n t l ii ) ((pos "Pu%" ) ))
( "irreligious"  j ( i2 r i l i1 jh @ s ) ((pos "OA%" ) ))
( "irremediable"  j ( i2 r i m ii1 d i@ b l ) ((pos "OA%" ) ))
( "irremovable"  j ( i2 r i m uu1 v @ b l ) ((pos "OA%" ) ))
( "irreparable"  j ( i1 r e1 p @ r @ b l ) ((pos "OA%" ) ))
( "irreplaceable"  j ( i2 r i p l ei1 s @ b l ) ((pos "OA%" ) ))
( "irrepressible"  j ( i2 r i p r e1 s @ b l ) ((pos "OA%" ) ))
( "irreproachable"  j ( i2 r i p r ou1 ch @ b l ) ((pos "OA%" ) ))
( "irresistible"  j ( i2 r i z i1 s t @ b l ) ((pos "OA%" ) ))
( "irresolute"  j ( i1 r e1 z @ l uu t ) ((pos "OA%" ) ))
( "irresolution"  n ( i1 r e2 z @ l uu1 sh @ n ) ((pos "L@$" ) ))
( "irrespective"  j ( i2 r i s p e1 k t i v ) ((pos "OA%" ) ))
( "irresponsibility"  n ( i2 r i s p o2 n s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "irresponsible"  j ( i2 r i s p o1 n s @ b l ) ((pos "OA%" ) ))
( "irretrievable"  j ( i2 r i t r ii1 v @ b l ) ((pos "OA%" ) ))
( "irretrievably"  a ( i2 r i t r ii1 v @ b l ii ) ((pos "Pu%" ) ))
( "irreverence"  n ( i1 r e1 v @ r @ n s ) ((pos "L@%" ) ))
( "irreverent"  j ( i1 r e1 v @ r @ n t ) ((pos "OA%" ) ))
( "irreverently"  a ( i1 r e1 v @ r @ n t l ii ) ((pos "Pu%" ) ))
( "irreversibility"  n ( i2 r i v @@2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "irreversible"  j ( i2 r i v @@1 s @ b l ) ((pos "OA%" ) ))
( "irreversibly"  a ( i2 r i v @@1 s @ b l ii ) ((pos "Pu%" ) ))
( "irrevocable"  j ( i1 r e1 v @ k @ b l ) ((pos "OA%" ) ))
( "irridentism"  n ( i2 r i d e1 n t i z @ m ) ((pos "L@$" ) ))
( "irrigate"  v ( i1 r i g ei t ) ((pos "H2%" ) (subcat )))
( "irrigated"  v ( i1 r i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "irrigates"  v ( i1 r i g ei t s ) ((pos "Ha%" ) (subcat )))
( "irrigating"  v ( i1 r i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "irrigation"  n ( i2 r i g ei1 sh @ n ) ((pos "L@%" ) ))
( "irritability"  n ( i2 r i t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "irritable"  j ( i1 r i t @ b l ) ((pos "OA%" ) ))
( "irritably"  a ( i1 r i t @ b l ii ) ((pos "Pu%" ) ))
( "irritant"  n ( i1 r i t @ n t ) ((pos "K6%" "OA%" ) ))
( "irritants"  n ( i1 r i t @ n t s ) ((pos "Kj%" ) ))
( "irritate"  v ( i1 r i t ei t ) ((pos "H2%" ) (subcat )))
( "irritated"  v ( i1 r i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "irritates"  v ( i1 r i t ei t s ) ((pos "Ha%" ) (subcat )))
( "irritating"  v ( i1 r i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "irritation"  n ( i2 r i t ei1 sh @ n ) ((pos "M6%" ) ))
( "irritations"  n ( i2 r i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "irruption"  n ( i1 r uh1 p sh @ n ) ((pos "K6$" ) ))
( "irruptions"  n ( i1 r uh1 p sh @ n z ) ((pos "Kj$" ) ))
( "is"  v ( i z ) ((pos "Ga*" "Ia%" ) (subcat )))
( "isinglass"  n ( ai1 z i ng g l aa s ) ((pos "L@$" ) ))
( "island"  n ( ai1 l @ n d ) ((pos "K6%" ) ))
( "islander"  n ( ai1 l @ n d @ r ) ((pos "K6%" ) ))
( "islanders"  n ( ai1 l @ n d @ z ) ((pos "Kj%" ) ))
( "islands"  n ( ai1 l @ n d z ) ((pos "Kj%" ) ))
( "isle"  n ( ai1 l ) ((pos "K6%" ) ))
( "isles"  n ( ai1 l z ) ((pos "Kj%" ) ))
( "islet"  n ( ai1 l i t ) ((pos "K6$" ) ))
( "islets"  n ( ai1 l i t s ) ((pos "Kj$" ) ))
( "ism"  n ( i1 z @ m ) ((pos "K6$" ) ))
( "isms"  n ( i1 z @ m z ) ((pos "Kj$" ) ))
( "isn't"  v ( i1 z n t ) ((pos "Gg%" ) ))
( "isobar"  n ( ai1 s @ b aa r ) ((pos "K6%" ) ))
( "isobars"  n ( ai1 s @ b aa z ) ((pos "Kj%" ) ))
( "isolate"  v ( ai1 s @ l ei t ) ((pos "H2%" ) (subcat "6A" )))
( "isolated"  v ( ai1 s @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "isolates"  v ( ai1 s @ l ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "isolating"  v ( ai1 s @ l ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "isolation"  n ( ai2 s @ l ei1 sh @ n ) ((pos "L@%" ) ))
( "isolationism"  n ( ai2 s @ l ei1 sh @ n i z @ m ) ((pos "L@%" ) ))
( "isolationist"  n ( ai2 s @ l ei1 sh @ n i s t ) ((pos "K6%" ) ))
( "isolationists"  n ( ai2 s @ l ei1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "isosceles"  j ( ai1 s o1 s @ l ii z ) ((pos "OA%" ) ))
( "isotherm"  n ( ai1 s @ th @@ m ) ((pos "K6$" ) ))
( "isotherms"  n ( ai1 s @ th @@ m z ) ((pos "Kj$" ) ))
( "isotope"  n ( ai1 s @ t ou p ) ((pos "K6%" ) ))
( "isotopes"  n ( ai1 s @ t ou p s ) ((pos "Kj%" ) ))
( "issue"  v ( i1 sh uu ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "issued"  v ( i1 sh uu d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "issues"  v ( i1 sh uu z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" )))
( "issuing"  v ( i1 sh uu i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "isthmus"  n ( i1 s m @ s ) ((pos "K7%" ) ))
( "isthmuses"  n ( i1 s m @ s i z ) ((pos "Kj%" ) ))
( "it"  prp ( i t ) ((pos "Qx*" ) ))
( "it'll"  v ( i1 t l ) ((pos "Gf%" ) ))
( "it's"  v ( i t s ) ((pos "Gf*" ) ))
( "italic"  j ( i1 t a1 l i k ) ((pos "OA%" ) ))
( "italicize"  v ( i1 t a1 l i s ai z ) ((pos "H2%" ) ))
( "italicized"  v ( i1 t a1 l i s ai z d ) ((pos "Hc%" "Hd%" ) ))
( "italicizes"  v ( i1 t a1 l i s ai z i z ) ((pos "Ha%" ) ))
( "italicizing"  v ( i1 t a1 l i s ai z i ng ) ((pos "Hb%" ) ))
( "italics"  n ( i1 t a1 l i k s ) ((pos "Kj%" ) ))
( "itch"  v ( i1 ch ) ((pos "I1%" "K7%" ) (subcat "2A" "3A" )))
( "itched"  v ( i1 ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "itches"  v ( i1 ch i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "3A" )))
( "itchier"  j ( i1 ch i@ r ) ((pos "Or$" ) ))
( "itchiest"  j ( i1 ch i i s t ) ((pos "Os$" ) ))
( "itching"  v ( i1 ch i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "itchy"  j ( i1 ch ii ) ((pos "OD%" ) ))
( "item"  n ( ai1 t @ m ) ((pos "K6%" "Pu%" ) ))
( "itemize"  v ( ai1 t @ m ai z ) ((pos "H2%" ) (subcat )))
( "itemized"  v ( ai1 t @ m ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "itemizes"  v ( ai1 t @ m ai z i z ) ((pos "Ha%" ) (subcat )))
( "itemizing"  v ( ai1 t @ m ai z i ng ) ((pos "Hb%" ) (subcat )))
( "items"  n ( ai1 t @ m z ) ((pos "Kj%" ) ))
( "iterate"  v ( i1 t @ r ei t ) ((pos "H2$" ) (subcat )))
( "iterated"  v ( i1 t @ r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "iterates"  v ( i1 t @ r ei t s ) ((pos "Ha$" ) (subcat )))
( "iterating"  v ( i1 t @ r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "iteration"  n ( i2 t @ r ei1 sh @ n ) ((pos "M6$" ) ))
( "iterations"  n ( i2 t @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "itinerant"  j ( ai1 t i1 n @ r @ n t ) ((pos "OA%" ) ))
( "itineraries"  n ( ai1 t i1 n @ r @ r i z ) ((pos "Kj%" ) ))
( "itinerary"  n ( ai1 t i1 n @ r @ r ii ) ((pos "K8%" ) ))
( "its"  j ( i1 t s ) ((pos "OA*" ) ))
( "itself"  prp ( i1 t s e1 l f ) ((pos "Qx%" ) ))
( "ivied"  j ( ai1 v i d ) ((pos "OA$" ) ))
( "ivies"  n ( ai1 v i z ) ((pos "Mj$" ) ))
( "ivory"  n ( ai1 v @ r ii ) ((pos "L@%" ) ))
( "ivy"  n ( ai1 v ii ) ((pos "M8%" ) ))
( "j"  n ( jh ei1 ) ((pos "Ki$" ) ))
( "j's"  n ( jh ei1 z ) ((pos "Kj$" ) ))
( "jab"  v ( jh a1 b ) ((pos "J4%" "K6%" ) (subcat "3A" "14" )))
( "jabbed"  v ( jh a1 b d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "14" )))
( "jabber"  v ( jh a1 b @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "jabbered"  v ( jh a1 b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "jabberer"  n ( jh a1 b @ r @ r ) ((pos "K6%" ) ))
( "jabberers"  n ( jh a1 b @ r @ z ) ((pos "Kj%" ) ))
( "jabbering"  v ( jh a1 b @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "jabbers"  v ( jh a1 b @ z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "jabbing"  v ( jh a1 b i ng ) ((pos "Jb%" ) (subcat "3A" "14" )))
( "jabot"  n ( zh a1 b ou ) ((pos "K6$" ) ))
( "jabots"  n ( zh a1 b ou z ) ((pos "Kj$" ) ))
( "jabs"  v ( jh a1 b z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "14" )))
( "jack"  v ( jh a1 k ) ((pos "H0%" "K6%" ) (subcat )))
( "jack-in-the-box"  n ( jh a1 k - i n - dh @ - b o k s ) ((pos "K7%" ) ))
( "jack-in-the-boxes"  n ( jh a1 k - i n - dh @ - b o k s i z ) ((pos "Kj%" ) ))
( "jack-knife"  v ( jh a1 k - n ai f ) ((pos "I2%" "Ki%" ) (subcat )))
( "jack-knifed"  v ( jh a1 k - n ai f t ) ((pos "Ic%" "Id%" ) (subcat )))
( "jack-knifes"  v ( jh a1 k - n ai f s ) ((pos "Ia%" ) (subcat )))
( "jack-knifing"  v ( jh a1 k - n ai f i ng ) ((pos "Ib%" ) (subcat )))
( "jack-knives"  n ( jh a1 k - n ai v z ) ((pos "Kj%" ) ))
( "jack-o'-lantern"  n ( jh a1 k - @ - l a1 n t @ n ) ((pos "K6$" ) ))
( "jack-o'-lanterns"  n ( jh a1 k - @ - l a1 n t @ n z ) ((pos "Kj$" ) ))
( "jack-plane"  n ( jh a1 k - p l ei n ) ((pos "K6$" ) ))
( "jack-planes"  n ( jh a1 k - p l ei n z ) ((pos "Kj$" ) ))
( "jackal"  n ( jh a1 k oo l ) ((pos "K6%" ) ))
( "jackals"  n ( jh a1 k oo l z ) ((pos "Kj%" ) ))
( "jackanapes"  n ( jh a1 k @ n ei p s ) ((pos "Ki$" ) ))
( "jackass"  n ( jh a1 k a s ) ((pos "K7%" ) ))
( "jackasses"  n ( jh a1 k a s i z ) ((pos "Kj%" ) ))
( "jackboot"  n ( jh a1 k b uu t ) ((pos "K6%" ) ))
( "jackboots"  n ( jh a1 k b uu t s ) ((pos "Kj%" ) ))
( "jackdaw"  n ( jh a1 k d oo ) ((pos "K6%" ) ))
( "jackdaws"  n ( jh a1 k d oo z ) ((pos "Kj%" ) ))
( "jacked"  v ( jh a1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jacket"  n ( jh a1 k i t ) ((pos "K6%" ) ))
( "jackets"  n ( jh a1 k i t s ) ((pos "Kj%" ) ))
( "jacking"  v ( jh a1 k i ng ) ((pos "Hb%" ) (subcat )))
( "jackpot"  n ( jh a1 k p o t ) ((pos "K6%" ) ))
( "jackpots"  n ( jh a1 k p o t s ) ((pos "Kj%" ) ))
( "jacks"  v ( jh a1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "jade"  n ( jh ei1 d ) ((pos "M6%" ) ))
( "jaded"  j ( jh ei1 d i d ) ((pos "OA%" ) ))
( "jades"  n ( jh ei1 d z ) ((pos "Mj$" ) ))
( "jag"  v ( jh a1 g ) ((pos "H4$" "K6$" ) (subcat )))
( "jagged"  j ( jh a1 g i d ) ((pos "OA%" ) ))
( "jagged"  v ( jh a1 g d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "jaggier"  j ( jh a1 g i@ r ) ((pos "Or$" ) ))
( "jaggiest"  j ( jh a1 g i i s t ) ((pos "Os$" ) ))
( "jagging"  v ( jh a1 g i ng ) ((pos "Hb$" ) (subcat )))
( "jaggy"  j ( jh a1 g ii ) ((pos "OD$" ) ))
( "jags"  v ( jh a1 g z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "jaguar"  n ( jh a1 g y u@ r ) ((pos "K6%" ) ))
( "jaguars"  n ( jh a1 g y u@ z ) ((pos "Kj%" ) ))
( "jail"  v ( jh ei1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "jailed"  v ( jh ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jailer"  n ( jh ei1 l @ r ) ((pos "K6%" ) ))
( "jailers"  n ( jh ei1 l @ z ) ((pos "Kj%" ) ))
( "jailing"  v ( jh ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "jailor"  n ( jh ei1 l @ r ) ((pos "K6%" ) ))
( "jailors"  n ( jh ei1 l @ z ) ((pos "Kj%" ) ))
( "jails"  v ( jh ei1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "jakes"  n ( jh ei1 k s ) ((pos "Ki$" ) ))
( "jalopies"  n ( jh @1 l o1 p i z ) ((pos "Kj%" ) ))
( "jalopy"  n ( jh @1 l o1 p ii ) ((pos "K8%" ) ))
( "jam"  v ( jh a1 m ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "jamb"  n ( jh a1 m ) ((pos "K6$" ) ))
( "jamboree"  n ( jh a2 m b @ r ii1 ) ((pos "K6%" ) ))
( "jamborees"  n ( jh a2 m b @ r ii1 z ) ((pos "Kj%" ) ))
( "jambs"  n ( jh a1 m z ) ((pos "Kj$" ) ))
( "jamjar"  n ( jh a1 m jh aa r ) ((pos "K6%" ) ))
( "jamjars"  n ( jh a1 m jh aa z ) ((pos "Kj%" ) ))
( "jammed"  v ( jh a1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "jamming"  v ( jh a1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "jampack"  v ( jh a2 m p a1 k ) ((pos "H0$" ) (subcat )))
( "jampacked"  v ( jh a2 m p a1 k t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "jampacking"  v ( jh a2 m p a1 k i ng ) ((pos "Hb$" ) (subcat )))
( "jampacks"  v ( jh a2 m p a1 k s ) ((pos "Ha$" ) (subcat )))
( "jampot"  n ( jh a1 m p o t ) ((pos "K6%" ) ))
( "jampots"  n ( jh a1 m p o t s ) ((pos "Kj%" ) ))
( "jams"  v ( jh a1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "jangle"  v ( jh a1 ng g l ) ((pos "J2%" "L@%" ) (subcat "2A" )))
( "jangled"  v ( jh a1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "jangles"  v ( jh a1 ng g l z ) ((pos "Ja%" ) (subcat "2A" )))
( "jangling"  v ( jh a1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "janitor"  n ( jh a1 n i t @ r ) ((pos "K6%" ) ))
( "janitors"  n ( jh a1 n i t @ z ) ((pos "Kj%" ) ))
( "japan"  v ( jh @1 p a1 n ) ((pos "H4$" ) ))
( "japanned"  v ( jh @1 p a1 n d ) ((pos "Hc$" "Hd$" ) ))
( "japanning"  v ( jh @1 p a1 n i ng ) ((pos "Hb$" ) ))
( "japans"  v ( jh @1 p a1 n z ) ((pos "Ha$" ) ))
( "jape"  n ( jh ei1 p ) ((pos "K6%" ) ))
( "japes"  n ( jh ei1 p s ) ((pos "Kj%" ) ))
( "japonica"  n ( jh @1 p o1 n i k @ ) ((pos "L@$" ) ))
( "jar"  v ( jh aa1 r ) ((pos "J4%" "K6%" ) (subcat "2A" "3A" )))
( "jarful"  n ( jh aa1 f u l ) ((pos "K6%" ) ))
( "jarfuls"  n ( jh aa1 f u l z ) ((pos "Kj%" ) ))
( "jargon"  n ( jh aa1 g @ n ) ((pos "L@%" ) ))
( "jarred"  v ( jh aa1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "jarring"  v ( jh aa1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" )))
( "jarringly"  a ( jh aa1 r i ng l ii ) ((pos "Pu%" ) ))
( "jars"  v ( jh aa1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "jasmine"  n ( jh a1 s m i n ) ((pos "L@%" ) ))
( "jasper"  n ( jh a1 s p @ r ) ((pos "L@%" ) ))
( "jaundice"  v ( jh oo1 n d i s ) ((pos "H2$" "L@%" ) ))
( "jaundiced"  v ( jh oo1 n d i s t ) ((pos "Hc$" "Hd%" ) ))
( "jaundices"  v ( jh oo1 n d i s i z ) ((pos "Ha$" ) ))
( "jaundicing"  v ( jh oo1 n d i s i ng ) ((pos "Hb$" ) ))
( "jaunt"  v ( jh oo1 n t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "jaunted"  v ( jh oo1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "jauntier"  j ( jh oo1 n t i@ r ) ((pos "Or$" ) ))
( "jauntiest"  j ( jh oo1 n t i i s t ) ((pos "Os$" ) ))
( "jauntily"  a ( jh oo1 n t i l ii ) ((pos "Pu%" ) ))
( "jauntiness"  n ( jh oo1 n t i n @ s ) ((pos "L@%" ) ))
( "jaunting"  v ( jh oo1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "jaunting-car"  n ( jh oo1 n t i ng - k aa r ) ((pos "K6$" ) ))
( "jaunting-cars"  n ( jh oo1 n t i ng - k aa z ) ((pos "Kj$" ) ))
( "jaunts"  v ( jh oo1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "jaunty"  j ( jh oo1 n t ii ) ((pos "OD%" ) ))
( "javelin"  n ( jh a1 v l i n ) ((pos "K6%" ) ))
( "javelins"  n ( jh a1 v l i n z ) ((pos "Kj%" ) ))
( "jaw"  v ( jh oo1 ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "jawbone"  n ( jh oo1 b ou n ) ((pos "K6%" ) ))
( "jawbones"  n ( jh oo1 b ou n z ) ((pos "Kj%" ) ))
( "jawbreaker"  n ( jh oo1 b r ei k @ r ) ((pos "K6%" ) ))
( "jawbreakers"  n ( jh oo1 b r ei k @ z ) ((pos "Kj%" ) ))
( "jawed"  v ( jh oo1 d ) ((pos "Ic$" "Id$" ) (subcat "2A" "2C" )))
( "jawing"  v ( jh oo1 i ng ) ((pos "Ib$" ) (subcat "2A" "2C" )))
( "jaws"  v ( jh oo1 z ) ((pos "Ia$" "Kj%" ) (subcat "2A" "2C" )))
( "jay"  n ( jh ei1 ) ((pos "K6%" ) ))
( "jays"  n ( jh ei1 z ) ((pos "Kj%" ) ))
( "jaywalk"  v ( jh ei1 w oo k ) ((pos "I0%" ) ))
( "jaywalked"  v ( jh ei1 w oo k t ) ((pos "Ic%" "Id%" ) ))
( "jaywalker"  n ( jh ei1 w oo k @ r ) ((pos "K6%" ) ))
( "jaywalkers"  n ( jh ei1 w oo k @ z ) ((pos "Kj%" ) ))
( "jaywalking"  v ( jh ei1 w oo k i ng ) ((pos "Ib%" ) ))
( "jaywalks"  v ( jh ei1 w oo k s ) ((pos "Ia%" ) ))
( "jazz"  v ( jh a1 z ) ((pos "H1$" "L@%" ) (subcat "6A" )))
( "jazzed"  v ( jh a1 z d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "jazzes"  v ( jh a1 z i z ) ((pos "Ha$" ) (subcat "6A" )))
( "jazzier"  j ( jh a1 z i@ r ) ((pos "Or$" ) ))
( "jazziest"  j ( jh a1 z i i s t ) ((pos "Os$" ) ))
( "jazzing"  v ( jh a1 z i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "jazzy"  j ( jh a1 z ii ) ((pos "OD%" ) ))
( "jealous"  j ( jh e1 l @ s ) ((pos "OA%" ) ))
( "jealousies"  n ( jh e1 l @ s i z ) ((pos "Mj%" ) ))
( "jealously"  a ( jh e1 l @ s l ii ) ((pos "Pu%" ) ))
( "jealousy"  n ( jh e1 l @ s ii ) ((pos "M8%" ) ))
( "jean"  n ( jh ii1 n ) ((pos "L@$" ) ))
( "jeans"  n ( jh ii1 n z ) ((pos "Kj%" ) ))
( "jeep"  n ( jh ii1 p ) ((pos "K6%" ) ))
( "jeeps"  n ( jh ii1 p s ) ((pos "Kj%" ) ))
( "jeer"  v ( jh i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" )))
( "jeered"  v ( jh i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "jeering"  v ( jh i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "jeeringly"  a ( jh i@1 r i ng l ii ) ((pos "Pu%" ) ))
( "jeers"  v ( jh i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "jejune"  j ( jh i1 jh uu1 n ) ((pos "OA$" ) ))
( "jejunely"  a ( jh i1 jh uu1 n l ii ) ((pos "Pu$" ) ))
( "jejuneness"  n ( jh i1 jh uu1 n - n @ s ) ((pos "L@$" ) ))
( "jell"  v ( jh e1 l ) ((pos "J0$" ) (subcat "2A" )))
( "jellaba"  n ( jh e1 l @ b @ ) ((pos "K6$" ) ))
( "jellabas"  n ( jh e1 l @ b @ z ) ((pos "Kj$" ) ))
( "jelled"  v ( jh e1 l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "jellied"  v ( jh e1 l i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "jellies"  v ( jh e1 l i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "jelling"  v ( jh e1 l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "jells"  v ( jh e1 l z ) ((pos "Ja$" ) (subcat "2A" )))
( "jelly"  v ( jh e1 l ii ) ((pos "J3%" "M8%" ) (subcat "2A" )))
( "jellyfish"  n ( jh e1 l i f i sh ) ((pos "K9%" ) ))
( "jellyfishes"  n ( jh e1 l i f i sh i z ) ((pos "Kj%" ) ))
( "jellying"  v ( jh e1 l i i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "jemmies"  n ( jh e1 m i z ) ((pos "Kj%" ) ))
( "jemmy"  n ( jh e1 m ii ) ((pos "K8%" ) ))
( "jennies"  n ( jh e1 n i z ) ((pos "Kj$" ) ))
( "jenny"  n ( jh e1 n ii ) ((pos "K8$" ) ))
( "jeopardize"  v ( jh e1 p @ d ai z ) ((pos "H2%" ) (subcat )))
( "jeopardized"  v ( jh e1 p @ d ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jeopardizes"  v ( jh e1 p @ d ai z i z ) ((pos "Ha%" ) (subcat )))
( "jeopardizing"  v ( jh e1 p @ d ai z i ng ) ((pos "Hb%" ) (subcat )))
( "jeopardy"  n ( jh e1 p @ d ii ) ((pos "L@%" ) ))
( "jerboa"  n ( jh @@1 b ou1 @ ) ((pos "K6$" ) ))
( "jerboas"  n ( jh @@1 b ou1 @ z ) ((pos "Kj$" ) ))
( "jeremiad"  n ( jh e2 r i m ai1 a d ) ((pos "K6$" ) ))
( "jeremiads"  n ( jh e2 r i m ai1 a d z ) ((pos "Kj$" ) ))
( "jerk"  v ( jh @@1 k ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "jerked"  v ( jh @@1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "jerkier"  j ( jh @@1 k i@ r ) ((pos "Or$" ) ))
( "jerkiest"  j ( jh @@1 k i i s t ) ((pos "Os$" ) ))
( "jerkily"  a ( jh @@1 k i l ii ) ((pos "Pu%" ) ))
( "jerkin"  n ( jh @@1 k i n ) ((pos "K6%" ) ))
( "jerkiness"  n ( jh @@1 k i n @ s ) ((pos "L@%" ) ))
( "jerking"  v ( jh @@1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "jerkins"  n ( jh @@1 k i n z ) ((pos "Kj%" ) ))
( "jerks"  v ( jh @@1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "jerky"  j ( jh @@1 k ii ) ((pos "OD%" ) ))
( "jerries"  n ( jh e1 r i z ) ((pos "Kj$" ) ))
( "jerry"  n ( jh e1 r ii ) ((pos "K8$" ) ))
( "jerry-builder"  n ( jh e1 r i - b i l d @ r ) ((pos "K6%" ) ))
( "jerry-builders"  n ( jh e1 r i - b i l d @ z ) ((pos "Kj%" ) ))
( "jerry-building"  n ( jh e1 r i - b i l d i ng ) ((pos "L@%" ) ))
( "jerry-built"  j ( jh e1 r i - b i l t ) ((pos "OA%" ) ))
( "jersey"  n ( jh @@1 z ii ) ((pos "M6%" ) ))
( "jerseys"  n ( jh @@1 z i z ) ((pos "Mj%" ) ))
( "jest"  v ( jh e1 s t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "jested"  v ( jh e1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "jester"  n ( jh e1 s t @ r ) ((pos "K6%" ) ))
( "jesters"  n ( jh e1 s t @ z ) ((pos "Kj%" ) ))
( "jesting"  v ( jh e1 s t i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "jestingly"  a ( jh e1 s t i ng l ii ) ((pos "Pu%" ) ))
( "jests"  v ( jh e1 s t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "jet"  v ( jh e1 t ) ((pos "J4%" "M6%" ) ))
( "jet-black"  j ( jh e1 t - b l a2 k ) ((pos "OA%" ) ))
( "jet-propelled"  j ( jh e1 t - p r @ p e1 l d ) ((pos "OA%" ) ))
( "jets"  v ( jh e1 t s ) ((pos "Ja%" "Mj%" ) ))
( "jetsam"  n ( jh e1 t s @ m ) ((pos "L@$" ) ))
( "jetted"  v ( jh e1 t i d ) ((pos "Jc%" "Jd%" ) ))
( "jetties"  n ( jh e1 t i z ) ((pos "Kj%" ) ))
( "jetting"  v ( jh e1 t i ng ) ((pos "Jb%" ) ))
( "jettison"  v ( jh e1 t i s n ) ((pos "H0%" ) (subcat )))
( "jettisoned"  v ( jh e1 t i s n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jettisoning"  v ( jh e1 t i s n i ng ) ((pos "Hb$" ) (subcat )))
( "jettisons"  v ( jh e1 t i s n z ) ((pos "Ha$" ) (subcat )))
( "jetty"  n ( jh e1 t ii ) ((pos "K8%" ) ))
( "jewel"  v ( jh uu1 @ l ) ((pos "H4$" "K6%" ) ))
( "jewelled"  v ( jh uu1 @ l d ) ((pos "Hc$" "Hd%" ) ))
( "jeweller"  n ( jh uu1 @ l @ r ) ((pos "K6%" ) ))
( "jewellers"  n ( jh uu1 @ l @ z ) ((pos "Kj%" ) ))
( "jewellery"  n ( jh uu1 @ l r ii ) ((pos "L@%" ) ))
( "jewelling"  v ( jh uu1 @ l i ng ) ((pos "Hb$" ) ))
( "jewelry"  n ( jh uu1 @ l r ii ) ((pos "L@%" ) ))
( "jewels"  v ( jh uu1 @ l z ) ((pos "Ha$" "Kj%" ) ))
( "jib"  v ( jh i1 b ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "jib-boom"  n ( jh i1 b - b uu m ) ((pos "K6$" ) ))
( "jib-booms"  n ( jh i1 b - b uu m z ) ((pos "Kj$" ) ))
( "jibbed"  v ( jh i1 b d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "jibbing"  v ( jh i1 b i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "jibe"  v ( jh ai1 b ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "jibed"  v ( jh ai1 b d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "jibes"  v ( jh ai1 b z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "jibing"  v ( jh ai1 b i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "jibs"  v ( jh i1 b z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "jiffies"  n ( jh i1 f i z ) ((pos "Kj$" ) ))
( "jiffy"  n ( jh i1 f ii ) ((pos "K8%" ) ))
( "jig"  v ( jh i1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "jigged"  v ( jh i1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "jigger"  n ( jh i1 g @ r ) ((pos "K6$" ) ))
( "jiggered"  j ( jh i1 g @ d ) ((pos "Op%" ) ))
( "jiggers"  n ( jh i1 g @ z ) ((pos "Kj$" ) ))
( "jiggery-pokery"  n ( jh i2 g @ r i - p ou1 k @ r ii ) ((pos "L@%" ) ))
( "jigging"  v ( jh i1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "jiggle"  v ( jh i1 g l ) ((pos "J2$" "K6$" ) ))
( "jiggled"  v ( jh i1 g l d ) ((pos "Jc$" "Jd$" ) ))
( "jiggles"  v ( jh i1 g l z ) ((pos "Ja$" "Kj$" ) ))
( "jiggling"  v ( jh i1 g l i ng ) ((pos "Jb$" ) ))
( "jigs"  v ( jh i1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "jigsaw"  n ( jh i1 g s oo ) ((pos "K6%" ) ))
( "jigsaws"  n ( jh i1 g s oo z ) ((pos "Kj%" ) ))
( "jihad"  n ( jh i1 h aa1 d ) ((pos "K6$" ) ))
( "jihads"  n ( jh i1 h aa1 d z ) ((pos "Kj$" ) ))
( "jilt"  v ( jh i1 l t ) ((pos "H0%" "K6%" ) (subcat )))
( "jilted"  v ( jh i1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jilting"  v ( jh i1 l t i ng ) ((pos "Hb%" ) (subcat )))
( "jilts"  v ( jh i1 l t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "jim-jams"  n ( jh i1 m - jh a m z ) ((pos "Kj$" ) ))
( "jiminy"  uh ( jh i1 m @ n ii ) ((pos "W-$" ) ))
( "jimmies"  n ( jh i1 m i z ) ((pos "Kj$" ) ))
( "jimmy"  n ( jh i1 m ii ) ((pos "K8$" ) ))
( "jingle"  v ( jh i1 ng g l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "jingled"  v ( jh i1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "jingles"  v ( jh i1 ng g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "jingling"  v ( jh i1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "jingo"  n ( jh i1 ng g ou ) ((pos "K7$" ) ))
( "jingoes"  n ( jh i1 ng g ou z ) ((pos "Kj$" ) ))
( "jingoism"  n ( jh i1 ng g ou i z @ m ) ((pos "L@%" ) ))
( "jingoist"  n ( jh i1 ng g ou i s t ) ((pos "K6%" ) ))
( "jingoistic"  j ( jh i2 ng g ou i1 s t i k ) ((pos "OA%" ) ))
( "jingoists"  n ( jh i1 ng g ou i s t s ) ((pos "Kj%" ) ))
( "jinks"  n ( jh i1 ng k s ) ((pos "L@%" ) ))
( "jinn"  n ( jh i1 n ) ((pos "K6$" ) ))
( "jinns"  n ( jh i1 n z ) ((pos "Kj$" ) ))
( "jinx"  n ( jh i1 ng k s ) ((pos "K7%" ) ))
( "jinxes"  n ( jh i1 ng k s i z ) ((pos "Kj$" ) ))
( "jitney"  n ( jh i1 t n ii ) ((pos "K6$" ) ))
( "jitneys"  n ( jh i1 t n i z ) ((pos "Kj$" ) ))
( "jitterbug"  n ( jh i1 t @ b uh g ) ((pos "K6%" ) ))
( "jitterbugs"  n ( jh i1 t @ b uh g z ) ((pos "Kj$" ) ))
( "jitters"  n ( jh i1 t @ z ) ((pos "Kj%" ) ))
( "jittery"  j ( jh i1 t @ r ii ) ((pos "OA%" ) ))
( "jive"  v ( jh ai1 v ) ((pos "I2%" "K6%" ) ))
( "jived"  v ( jh ai1 v d ) ((pos "Ic%" "Id%" ) ))
( "jives"  v ( jh ai1 v z ) ((pos "Ia%" "Kj%" ) ))
( "jiving"  v ( jh ai1 v i ng ) ((pos "Ib%" ) ))
( "job"  v ( jh o1 b ) ((pos "J4$" "K6*" ) (subcat "2A" "6A" )))
( "jobbed"  v ( jh o1 b d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "6A" )))
( "jobber"  n ( jh o1 b @ r ) ((pos "K6%" ) ))
( "jobbers"  n ( jh o1 b @ z ) ((pos "Kj%" ) ))
( "jobbery"  n ( jh o1 b @ r ii ) ((pos "L@$" ) ))
( "jobbing"  v ( jh o1 b i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "jobs"  v ( jh o1 b z ) ((pos "Ja$" "Kj%" ) (subcat "2A" "6A" )))
( "jockey"  v ( jh o1 k ii ) ((pos "J0%" "K6%" ) (subcat "3A" )))
( "jockeyed"  v ( jh o1 k i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "jockeying"  v ( jh o1 k i i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "jockeys"  v ( jh o1 k i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "jocose"  j ( jh ou1 k ou1 s ) ((pos "OA%" ) ))
( "jocosely"  a ( jh ou1 k ou1 s l ii ) ((pos "Pu%" ) ))
( "jocoseness"  n ( jh ou1 k ou1 s n @ s ) ((pos "L@$" ) ))
( "jocosity"  n ( jh ou1 k o1 s i t ii ) ((pos "L@$" ) ))
( "jocular"  j ( jh o1 k y u l @ r ) ((pos "OA%" ) ))
( "jocularities"  n ( jh o2 k y u l a1 r i t i z ) ((pos "Mj$" ) ))
( "jocularity"  n ( jh o2 k y u l a1 r i t ii ) ((pos "M8%" ) ))
( "jocularly"  a ( jh o1 k y u l @ l ii ) ((pos "Pu%" ) ))
( "jocund"  j ( jh o1 k @ n d ) ((pos "OA%" ) ))
( "jocundities"  n ( jh ou1 k uh1 n d i t i z ) ((pos "Mj$" ) ))
( "jocundity"  n ( jh ou1 k uh1 n d i t ii ) ((pos "M8$" ) ))
( "jodhpurs"  n ( jh o1 d p @ z ) ((pos "Kj%" ) ))
( "jog"  v ( jh o1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "jog-trot"  n ( jh o1 g - t r o t ) ((pos "K6%" ) ))
( "jog-trots"  n ( jh o1 g - t r o t s ) ((pos "Kj%" ) ))
( "jogged"  v ( jh o1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "jogger"  n ( jh o1 g @ r ) ((pos "K6%" ) ))
( "joggers"  n ( jh o1 g @ z ) ((pos "Kj%" ) ))
( "jogging"  v ( jh o1 g i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "joggle"  v ( jh o1 g l ) ((pos "J2$" "K6$" ) (subcat "2A" )))
( "joggled"  v ( jh o1 g l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "joggles"  v ( jh o1 g l z ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "joggling"  v ( jh o1 g l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "jogs"  v ( jh o1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "john"  n ( jh o1 n ) ((pos "Ki$" ) ))
( "joie de vivre"  n ( zh w aa2 - d @ - v ii1 v r @ ) ((pos "L@%" ) ))
( "join"  v ( jh oi1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "joined"  v ( jh oi1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "joiner"  n ( jh oi1 n @ r ) ((pos "K6%" ) ))
( "joiners"  n ( jh oi1 n @ z ) ((pos "Kj%" ) ))
( "joinery"  n ( jh oi1 n @ r ii ) ((pos "L@%" ) ))
( "joining"  v ( jh oi1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "joins"  v ( jh oi1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "joint"  v ( jh oi1 n t ) ((pos "H0$" "K6%" "OA%" ) (subcat )))
( "jointed"  v ( jh oi1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "jointing"  v ( jh oi1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "jointly"  a ( jh oi1 n t l ii ) ((pos "Pu%" ) ))
( "joints"  v ( jh oi1 n t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "jointure"  n ( jh oi1 n ch @ r ) ((pos "K6$" ) ))
( "jointures"  n ( jh oi1 n ch @ z ) ((pos "Kj$" ) ))
( "joist"  n ( jh oi1 s t ) ((pos "K6%" ) ))
( "joists"  n ( jh oi1 s t s ) ((pos "Kj%" ) ))
( "joke"  v ( jh ou1 k ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "joked"  v ( jh ou1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "joker"  n ( jh ou1 k @ r ) ((pos "K6%" ) ))
( "jokers"  n ( jh ou1 k @ z ) ((pos "Kj%" ) ))
( "jokes"  v ( jh ou1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "joking"  v ( jh ou1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "jokingly"  a ( jh ou1 k i ng l ii ) ((pos "Pu%" ) ))
( "jollied"  v ( jh o1 l i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "jollier"  j ( jh o1 l i@ r ) ((pos "Or%" ) ))
( "jollies"  v ( jh o1 l i z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "jolliest"  j ( jh o1 l i i s t ) ((pos "Os%" ) ))
( "jollification"  n ( jh o2 l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "jollifications"  n ( jh o2 l i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "jollities"  n ( jh o1 l i t i z ) ((pos "Mj%" ) ))
( "jollity"  n ( jh o1 l i t ii ) ((pos "M8%" ) ))
( "jolly"  v ( jh o1 l ii ) ((pos "H3%" "OD%" "Pu%" ) (subcat "6A" "15A" )))
( "jollyboat"  n ( jh o1 l i b ou t ) ((pos "K6$" ) ))
( "jollyboats"  n ( jh o1 l i b ou t s ) ((pos "Kj$" ) ))
( "jollying"  v ( jh o1 l i i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "jolt"  v ( jh ou1 l t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "jolted"  v ( jh ou1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "joltier"  j ( jh ou1 l t i@ r ) ((pos "Or$" ) ))
( "joltiest"  j ( jh ou1 l t i i s t ) ((pos "Os$" ) ))
( "jolting"  v ( jh ou1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "jolts"  v ( jh ou1 l t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "jolty"  j ( jh ou1 l t ii ) ((pos "OD%" ) ))
( "jonquil"  n ( jh o1 ng k w i l ) ((pos "K6$" ) ))
( "jonquils"  n ( jh o1 ng k w i l z ) ((pos "Kj$" ) ))
( "joss"  n ( jh o1 s ) ((pos "K7$" ) ))
( "joss-house"  n ( jh o1 s - h au s ) ((pos "K6$" ) ))
( "joss-houses"  n ( jh o1 s - h au z i z ) ((pos "Kj$" ) ))
( "joss-stick"  n ( jh o1 s - s t i k ) ((pos "K6%" ) ))
( "joss-sticks"  n ( jh o1 s - s t i k s ) ((pos "Kj%" ) ))
( "josses"  n ( jh o1 s i z ) ((pos "Kj$" ) ))
( "jostle"  v ( jh o1 s l ) ((pos "J2%" ) (subcat "2C" )))
( "jostled"  v ( jh o1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "jostles"  v ( jh o1 s l z ) ((pos "Ja%" ) (subcat "2C" )))
( "jostling"  v ( jh o1 s l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "jot"  v ( jh o1 t ) ((pos "H4%" "K6%" ) (subcat )))
( "jots"  v ( jh o1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "jotted"  v ( jh o1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "jotter"  n ( jh o1 t @ r ) ((pos "K6%" ) ))
( "jotters"  n ( jh o1 t @ z ) ((pos "Kj%" ) ))
( "jotting"  v ( jh o1 t i ng ) ((pos "Hb%" ) (subcat )))
( "jottings"  n ( jh o1 t i ng s ) ((pos "Kj%" ) ))
( "joule"  n ( jh uu1 l ) ((pos "K6$" ) ))
( "joules"  n ( jh uu1 l z ) ((pos "Kj$" ) ))
( "journal"  n ( jh @@1 n l ) ((pos "K6%" ) ))
( "journalese"  n ( jh @@2 n @ l ii1 z ) ((pos "L@%" ) ))
( "journalism"  n ( jh @@1 n @ l i z @ m ) ((pos "L@%" ) ))
( "journalist"  n ( jh @@1 n @ l i s t ) ((pos "K6%" ) ))
( "journalistic"  j ( jh @@2 n @ l i1 s t i k ) ((pos "OA%" ) ))
( "journalistically"  a ( jh @@2 n @ l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "journalists"  n ( jh @@1 n @ l i s t s ) ((pos "Kj%" ) ))
( "journals"  n ( jh @@1 n l z ) ((pos "Kj%" ) ))
( "journey"  v ( jh @@1 n ii ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "journeyed"  v ( jh @@1 n i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "journeying"  v ( jh @@1 n i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "journeyman"  n ( jh @@1 n i m @ n ) ((pos "Ki%" ) ))
( "journeymen"  n ( jh @@1 n i m @ n ) ((pos "Kj%" ) ))
( "journeys"  v ( jh @@1 n i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "joust"  v ( jh au1 s t ) ((pos "I0%" "K6%" ) ))
( "jousted"  v ( jh au1 s t i d ) ((pos "Ic%" "Id%" ) ))
( "jousting"  v ( jh au1 s t i ng ) ((pos "Ib%" ) ))
( "jousts"  v ( jh au1 s t s ) ((pos "Ia%" "Kj%" ) ))
( "jovial"  j ( jh ou1 v i@ l ) ((pos "OA%" ) ))
( "jovialities"  n ( jh ou2 v i a1 l i t i z ) ((pos "Mj$" ) ))
( "joviality"  n ( jh ou2 v i a1 l i t ii ) ((pos "M8%" ) ))
( "jovially"  a ( jh ou1 v i@ l ii ) ((pos "Pu%" ) ))
( "jowl"  n ( jh au1 l ) ((pos "K6%" ) ))
( "jowlier"  j ( jh au1 l i@ r ) ((pos "Or$" ) ))
( "jowliest"  j ( jh au1 l i i s t ) ((pos "Os$" ) ))
( "jowls"  n ( jh au1 l z ) ((pos "Kj%" ) ))
( "jowly"  j ( jh au1 l ii ) ((pos "OD$" ) ))
( "joy"  v ( jh oi1 ) ((pos "I0$" "M6%" ) ))
( "joy-bells"  n ( jh oi1 - b e l z ) ((pos "Kj$" ) ))
( "joy-ride"  n ( jh oi1 - r ai d ) ((pos "K6%" ) ))
( "joy-rides"  n ( jh oi1 - r ai d z ) ((pos "Kj%" ) ))
( "joy-stick"  n ( jh oi1 - s t i k ) ((pos "K6%" ) ))
( "joy-sticks"  n ( jh oi1 - s t i k s ) ((pos "Kj%" ) ))
( "joyed"  v ( jh oi1 d ) ((pos "Ic$" "Id$" ) ))
( "joyful"  j ( jh oi1 f @ l ) ((pos "OA%" ) ))
( "joyfully"  a ( jh oi1 f @ l ii ) ((pos "Pu%" ) ))
( "joyfulness"  n ( jh oi1 f @ l n @ s ) ((pos "L@%" ) ))
( "joying"  v ( jh oi1 i ng ) ((pos "Ib$" ) ))
( "joyless"  j ( jh oi1 l @ s ) ((pos "OA%" ) ))
( "joylessly"  a ( jh oi1 l @ s l ii ) ((pos "Pu%" ) ))
( "joylessness"  n ( jh oi1 l @ s n @ s ) ((pos "L@%" ) ))
( "joyous"  j ( jh oi1 @ s ) ((pos "OA%" ) ))
( "joyously"  a ( jh oi1 @ s l ii ) ((pos "Pu%" ) ))
( "joyousness"  n ( jh oi1 @ s n @ s ) ((pos "L@%" ) ))
( "joys"  v ( jh oi1 z ) ((pos "Ia$" "Mj%" ) ))
( "ju-jitsu"  n ( jh uu2 - jh i1 t s uu ) ((pos "L@%" ) ))
( "jubilant"  j ( jh uu1 b i l @ n t ) ((pos "OA%" ) ))
( "jubilantly"  a ( jh uu1 b i l @ n t l ii ) ((pos "Pu%" ) ))
( "jubilation"  n ( jh uu2 b i l ei1 sh @ n ) ((pos "M6%" ) ))
( "jubilations"  n ( jh uu2 b i l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "jubilee"  n ( jh uu1 b i l ii ) ((pos "K6%" ) ))
( "jubilees"  n ( jh uu1 b i l ii z ) ((pos "Kj%" ) ))
( "judder"  v ( jh uh1 d @ r ) ((pos "I0%" ) ))
( "juddered"  v ( jh uh1 d @ d ) ((pos "Ic%" "Id%" ) ))
( "juddering"  v ( jh uh1 d @ r i ng ) ((pos "Ib%" ) ))
( "judders"  v ( jh uh1 d @ z ) ((pos "Ia%" ) ))
( "judge"  v ( jh uh1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "9" "10" "22" )))
( "judged"  v ( jh uh1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "9" "10" "22" )))
( "judgement"  n ( jh uh1 jh m @ n t ) ((pos "M6%" ) ))
( "judgements"  n ( jh uh1 jh m @ n t s ) ((pos "Mj%" ) ))
( "judges"  v ( jh uh1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "9" "10" "22" )))
( "judging"  v ( jh uh1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "9" "10" "22" )))
( "judicature"  n ( jh uu1 d i k @ ch @ r ) ((pos "M6$" ) ))
( "judicatures"  n ( jh uu1 d i k @ ch @ z ) ((pos "Mj$" ) ))
( "judicial"  j ( jh uu1 d i1 sh l ) ((pos "OA%" ) ))
( "judiciaries"  n ( jh uu1 d i1 sh @ r i z ) ((pos "Kj$" ) ))
( "judiciary"  n ( jh uu1 d i1 sh @ r ii ) ((pos "K8%" ) ))
( "judicious"  j ( jh uu1 d i1 sh @ s ) ((pos "OA%" ) ))
( "judiciously"  a ( jh uu1 d i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "judiciousness"  n ( jh uu1 d i1 sh @ s n @ s ) ((pos "L@%" ) ))
( "judo"  n ( jh uu1 d ou ) ((pos "L@%" ) ))
( "jug"  v ( jh uh1 g ) ((pos "H4$" "K6%" ) (subcat )))
( "jugful"  n ( jh uh1 g f u l ) ((pos "K6%" ) ))
( "jugfuls"  n ( jh uh1 g f u l z ) ((pos "Kj%" ) ))
( "jugged"  v ( jh uh1 g d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "juggernaut"  n ( jh uh1 g @ n oo t ) ((pos "K6%" ) ))
( "juggernauts"  n ( jh uh1 g @ n oo t s ) ((pos "Kj%" ) ))
( "jugging"  v ( jh uh1 g i ng ) ((pos "Hb$" ) (subcat )))
( "juggle"  v ( jh uh1 g l ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "juggled"  v ( jh uh1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "juggler"  n ( jh uh1 g l @ r ) ((pos "K6%" ) ))
( "jugglers"  n ( jh uh1 g l @ z ) ((pos "Kj%" ) ))
( "juggles"  v ( jh uh1 g l z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "juggling"  v ( jh uh1 g l i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "jugs"  v ( jh uh1 g z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "jugular"  j ( jh uh1 g y u l @ r ) ((pos "OA%" ) ))
( "juice"  n ( jh uu1 s ) ((pos "M6%" ) ))
( "juices"  n ( jh uu1 s i z ) ((pos "Mj%" ) ))
( "juicier"  j ( jh uu1 s i@ r ) ((pos "Or%" ) ))
( "juiciest"  j ( jh uu1 s i i s t ) ((pos "Os%" ) ))
( "juiciness"  n ( jh uu1 s i n @ s ) ((pos "L@%" ) ))
( "juicy"  j ( jh uu1 s ii ) ((pos "OD%" ) ))
( "juju"  n ( jh uu1 jh uu ) ((pos "K6$" ) ))
( "jujube"  n ( jh uu1 jh uu b ) ((pos "K6$" ) ))
( "jujubes"  n ( jh uu1 jh uu b z ) ((pos "Kj$" ) ))
( "jujus"  n ( jh uu1 jh uu z ) ((pos "Kj$" ) ))
( "jukebox"  n ( jh uu1 k b o k s ) ((pos "K7%" ) ))
( "jukeboxes"  n ( jh uu1 k b o k s i z ) ((pos "Kj%" ) ))
( "julep"  n ( jh uu1 l i p ) ((pos "K6$" ) ))
( "juleps"  n ( jh uu1 l i p s ) ((pos "Kj$" ) ))
( "jumble"  v ( jh uh1 m b l ) ((pos "J2%" "M6%" ) (subcat "2C" )))
( "jumble-sale"  n ( jh uh1 m b l - s ei l ) ((pos "K6%" ) ))
( "jumble-sales"  n ( jh uh1 m b l - s ei l z ) ((pos "Kj%" ) ))
( "jumbled"  v ( jh uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "jumbles"  v ( jh uh1 m b l z ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "jumbling"  v ( jh uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "jumbo"  j ( jh uh1 m b ou ) ((pos "OA%" ) ))
( "jump"  v ( jh uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "jumped"  v ( jh uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "jumped-up"  j ( jh uh1 m p t - uh p ) ((pos "OA%" ) ))
( "jumper"  n ( jh uh1 m p @ r ) ((pos "K6%" ) ))
( "jumpers"  n ( jh uh1 m p @ z ) ((pos "Kj%" ) ))
( "jumpier"  j ( jh uh1 m p i@ r ) ((pos "Or$" ) ))
( "jumpiest"  j ( jh uh1 m p i i s t ) ((pos "Os$" ) ))
( "jumpiness"  n ( jh uh1 m p i n @ s ) ((pos "L@%" ) ))
( "jumping"  v ( jh uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "jumps"  v ( jh uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "jumpy"  j ( jh uh1 m p ii ) ((pos "OD%" ) ))
( "junction"  n ( jh uh1 ng k sh @ n ) ((pos "M6%" ) ))
( "junctions"  n ( jh uh1 ng k sh @ n z ) ((pos "Mj%" ) ))
( "juncture"  n ( jh uh1 ng k ch @ r ) ((pos "K6%" ) ))
( "junctures"  n ( jh uh1 ng k ch @ z ) ((pos "Kj$" ) ))
( "jungle"  n ( jh uh1 ng g l ) ((pos "K6%" ) ))
( "jungles"  n ( jh uh1 ng g l z ) ((pos "Kj%" ) ))
( "junglier"  j ( jh uh1 ng g l i@ r ) ((pos "Or$" ) ))
( "jungliest"  j ( jh uh1 ng g l i i s t ) ((pos "Os$" ) ))
( "jungly"  j ( jh uh1 ng g l ii ) ((pos "OD%" ) ))
( "junior"  n ( jh uu1 n i@ r ) ((pos "K6%" "OA%" ) ))
( "juniors"  n ( jh uu1 n i@ z ) ((pos "Kj%" ) ))
( "juniper"  n ( jh uu1 n i p @ r ) ((pos "K6%" ) ))
( "junipers"  n ( jh uu1 n i p @ z ) ((pos "Kj%" ) ))
( "junk"  n ( jh uh1 ng k ) ((pos "M6%" ) ))
( "junk-shop"  n ( jh uh1 ng k - sh o p ) ((pos "K6%" ) ))
( "junk-shops"  n ( jh uh1 ng k - sh o p s ) ((pos "Kj%" ) ))
( "junket"  v ( jh uh1 ng k i t ) ((pos "I0%" "M6%" ) ))
( "junketed"  v ( jh uh1 ng k i t i d ) ((pos "Ic%" "Id%" ) ))
( "junketing"  v ( jh uh1 ng k i t i ng ) ((pos "Ib%" "L@%" ) ))
( "junkets"  v ( jh uh1 ng k i t s ) ((pos "Ia%" "Mj%" ) ))
( "junkie"  n ( jh uh1 ng k ii ) ((pos "K6%" ) ))
( "junkies"  n ( jh uh1 ng k i z ) ((pos "Kj%" ) ))
( "junks"  n ( jh uh1 ng k s ) ((pos "Mj%" ) ))
( "junky"  n ( jh uh1 ng k ii ) ((pos "K8%" ) ))
( "junta"  n ( jh uh1 n t @ ) ((pos "K6%" ) ))
( "juntas"  n ( jh uh1 n t @ z ) ((pos "Kj%" ) ))
( "juridical"  j ( jh u@1 r i1 d i k l ) ((pos "OA$" ) ))
( "juries"  n ( jh u@1 r i z ) ((pos "Kj%" ) ))
( "jurisdiction"  n ( jh u@2 r i s d i1 k sh @ n ) ((pos "L@%" ) ))
( "jurisprudence"  n ( jh u@2 r i s p r uu1 d n s ) ((pos "L@%" ) ))
( "jurist"  n ( jh u@1 r i s t ) ((pos "K6%" ) ))
( "jurists"  n ( jh u@1 r i s t s ) ((pos "Kj%" ) ))
( "juror"  n ( jh u@1 r @ r ) ((pos "K6%" ) ))
( "jurors"  n ( jh u@1 r @ z ) ((pos "Kj%" ) ))
( "jury"  n ( jh u@1 r ii ) ((pos "K8%" ) ))
( "jury-box"  n ( jh u@1 r i - b o k s ) ((pos "K7%" ) ))
( "jury-boxes"  n ( jh u@1 r i - b o k s i z ) ((pos "Kj%" ) ))
( "jury-mast"  n ( jh u@1 r i - m aa s t ) ((pos "K6$" ) ))
( "jury-masts"  n ( jh u@1 r i - m aa s t s ) ((pos "Kj$" ) ))
( "juryman"  n ( jh u@1 r i m @ n ) ((pos "Ki%" ) ))
( "jurymen"  n ( jh u@1 r i m @ n ) ((pos "Kj%" ) ))
( "just"  j ( jh uh1 s t ) ((pos "OA%" "Pu*" ) ))
( "justice"  n ( jh uh1 s t i s ) ((pos "M6%" ) ))
( "justices"  n ( jh uh1 s t i s i z ) ((pos "Mj%" ) ))
( "justiciaries"  n ( jh uh1 s t i1 sh @ r i z ) ((pos "Kj$" ) ))
( "justiciary"  n ( jh uh1 s t i1 sh @ r ii ) ((pos "K8$" ) ))
( "justifiable"  j ( jh uh2 s t i f ai1 @ b l ) ((pos "OA%" ) ))
( "justifiably"  a ( jh uh2 s t i f ai1 @ b l ii ) ((pos "Pu%" ) ))
( "justification"  n ( jh uh2 s t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "justifications"  n ( jh uh2 s t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "justified"  v ( jh uh1 s t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "justifies"  v ( jh uh1 s t i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "justify"  v ( jh uh1 s t i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "justifying"  v ( jh uh1 s t i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "justly"  a ( jh uh1 s t l ii ) ((pos "Pu%" ) ))
( "justness"  n ( jh uh1 s t n @ s ) ((pos "L@%" ) ))
( "jut"  v ( jh uh1 t ) ((pos "I4%" ) (subcat )))
( "jute"  n ( jh uu1 t ) ((pos "L@%" ) ))
( "juts"  v ( jh uh1 t s ) ((pos "Ia%" ) (subcat )))
( "jutted"  v ( jh uh1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "jutting"  v ( jh uh1 t i ng ) ((pos "Ib%" ) (subcat )))
( "juvenile"  n ( jh uu1 v @ n ai l ) ((pos "K6%" "OA%" ) ))
( "juveniles"  n ( jh uu1 v @ n ai l z ) ((pos "Kj%" ) ))
( "juxtapose"  v ( jh uh2 k s t @ p ou1 z ) ((pos "H2%" ) (subcat )))
( "juxtaposed"  v ( jh uh2 k s t @ p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "juxtaposes"  v ( jh uh2 k s t @ p ou1 z i z ) ((pos "Ha%" ) (subcat )))
( "juxtaposing"  v ( jh uh2 k s t @ p ou1 z i ng ) ((pos "Hb%" ) (subcat )))
( "juxtaposition"  n ( jh uh2 k s t @ p @ z i1 sh @ n ) ((pos "L@%" ) ))
( "k"  n ( k ei1 ) ((pos "Ki$" ) ))
( "k\"ummel"  n ( k u1 m @ l ) ((pos "L@$" ) ))
( "k's"  n ( k ei1 z ) ((pos "Kj$" ) ))
( "kaffir"  n ( k a1 f @ r ) ((pos "K6$" ) ))
( "kaffirs"  n ( k a1 f @ z ) ((pos "Kj$" ) ))
( "kail"  n ( k ei1 l ) ((pos "L@$" ) ))
( "kakemono"  n ( k a2 k i m ou1 n ou ) ((pos "K6$" ) ))
( "kakemonos"  n ( k a2 k i m ou1 n ou z ) ((pos "Kj$" ) ))
( "kale"  n ( k ei1 l ) ((pos "L@$" ) ))
( "kaleidoscope"  n ( k @1 l ai1 d @ s k ou p ) ((pos "K6%" ) ))
( "kaleidoscopes"  n ( k @1 l ai1 d @ s k ou p s ) ((pos "Kj%" ) ))
( "kaleidoscopic"  j ( k @1 l ai2 d @ s k o1 p i k ) ((pos "OA%" ) ))
( "kalends"  n ( k a1 l e n d z ) ((pos "Kj$" ) ))
( "kampong"  n ( k a1 m p o ng ) ((pos "K6$" ) ))
( "kampongs"  n ( k a1 m p o ng z ) ((pos "Kj$" ) ))
( "kangaroo"  n ( k a2 ng g @ r uu1 ) ((pos "K6%" ) ))
( "kangaroos"  n ( k a2 ng g @ r uu1 z ) ((pos "Kj%" ) ))
( "kaolin"  n ( k ei1 @ l i n ) ((pos "L@$" ) ))
( "kapok"  n ( k ei1 p o k ) ((pos "L@$" ) ))
( "kaput"  j ( k @1 p u1 t ) ((pos "OA$" ) ))
( "karat"  n ( k a1 r @ t ) ((pos "K6$" ) ))
( "karate"  n ( k @1 r aa1 t ii ) ((pos "L@%" ) ))
( "karats"  n ( k a1 r @ t s ) ((pos "Kj$" ) ))
( "karma"  n ( k aa1 m @ ) ((pos "K6$" ) ))
( "karmas"  n ( k aa1 m @ z ) ((pos "Kj$" ) ))
( "kava"  n ( k aa1 v @ ) ((pos "L@$" ) ))
( "kayak"  n ( k ai1 a k ) ((pos "K6$" ) ))
( "kayaks"  n ( k ai1 a k s ) ((pos "Kj$" ) ))
( "kebab"  n ( k @1 b a1 b ) ((pos "K6%" ) ))
( "kebabs"  n ( k @1 b a1 b z ) ((pos "Kj%" ) ))
( "kedgeree"  n ( k e1 jh @ r ii ) ((pos "L@%" ) ))
( "keel"  v ( k ii1 l ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "keeled"  v ( k ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "keeling"  v ( k ii1 l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "keels"  v ( k ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "keen"  v ( k ii1 n ) ((pos "J0$" "K6$" "OC%" ) ))
( "keened"  v ( k ii1 n d ) ((pos "Jc$" "Jd$" ) ))
( "keener"  j ( k ii1 n @ r ) ((pos "Or%" ) ))
( "keenest"  j ( k ii1 n i s t ) ((pos "Os%" ) ))
( "keening"  v ( k ii1 n i ng ) ((pos "Jb$" ) ))
( "keenly"  a ( k ii1 n l ii ) ((pos "Pu%" ) ))
( "keenness"  n ( k ii1 n - n @ s ) ((pos "L@%" ) ))
( "keens"  v ( k ii1 n z ) ((pos "Ja$" "Kj$" ) ))
( "keep"  v ( k ii1 p ) ((pos "J5*" "M6%" ) (subcat "2A" "2C" "2E" "3A" "6A" "14" "15A" "15B" "19B" )))
( "keeper"  n ( k ii1 p @ r ) ((pos "K6%" ) ))
( "keepers"  n ( k ii1 p @ z ) ((pos "Kj%" ) ))
( "keeping"  v ( k ii1 p i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "2E" "3A" "6A" "14" "15A" "15B" "19B" )))
( "keeps"  v ( k ii1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2E" "3A" "6A" "14" "15A" "15B" "19B" )))
( "keepsake"  n ( k ii1 p s ei k ) ((pos "K6%" ) ))
( "keepsakes"  n ( k ii1 p s ei k s ) ((pos "Kj%" ) ))
( "keg"  n ( k e1 g ) ((pos "K6%" ) ))
( "kegs"  n ( k e1 g z ) ((pos "Kj%" ) ))
( "kelp"  n ( k e1 l p ) ((pos "L@$" ) ))
( "ken"  v ( k e1 n ) ((pos "H4$" "L@$" ) (subcat "6A" )))
( "kenned"  v ( k e1 n d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "kennel"  v ( k e1 n l ) ((pos "J4$" "K6%" ) ))
( "kennelled"  v ( k e1 n l d ) ((pos "Jc$" "Jd$" ) ))
( "kennelling"  v ( k e1 n @ l i ng ) ((pos "Jb$" ) ))
( "kennels"  v ( k e1 n l z ) ((pos "Ja$" "Kj%" ) ))
( "kenning"  v ( k e1 n i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "kens"  v ( k e1 n z ) ((pos "Ha$" ) (subcat "6A" )))
( "kepi"  n ( k ei1 p ii ) ((pos "K6$" ) ))
( "kepis"  n ( k ei1 p i z ) ((pos "Kj$" ) ))
( "kept"  v ( k e1 p t ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2C" "2E" "3A" "6A" "14" "15A" "15B" "19B" )))
( "kerb"  n ( k @@1 b ) ((pos "K6%" ) ))
( "kerbs"  n ( k @@1 b z ) ((pos "Kj%" ) ))
( "kerbstone"  n ( k @@1 b s t ou n ) ((pos "K6%" ) ))
( "kerbstones"  n ( k @@1 b s t ou n z ) ((pos "Kj%" ) ))
( "kerchief"  n ( k @@1 ch i f ) ((pos "K6$" ) ))
( "kerchiefs"  n ( k @@1 ch i f s ) ((pos "Kj$" ) ))
( "kernel"  n ( k @@1 n l ) ((pos "K6%" ) ))
( "kernels"  n ( k @@1 n l z ) ((pos "Kj%" ) ))
( "kerosene"  n ( k e1 r @ s ii n ) ((pos "L@%" ) ))
( "kestrel"  n ( k e1 s t r @ l ) ((pos "K6%" ) ))
( "kestrels"  n ( k e1 s t r @ l z ) ((pos "Kj%" ) ))
( "ketch"  n ( k e1 ch ) ((pos "K7$" ) ))
( "ketches"  n ( k e1 ch i z ) ((pos "Kj$" ) ))
( "ketchup"  n ( k e1 ch @ p ) ((pos "L@%" ) ))
( "kettle"  n ( k e1 t l ) ((pos "K6%" ) ))
( "kettledrum"  n ( k e1 t l d r uh m ) ((pos "K6%" ) ))
( "kettledrums"  n ( k e1 t l d r uh m z ) ((pos "Kj%" ) ))
( "kettles"  n ( k e1 t l z ) ((pos "Kj%" ) ))
( "key"  v ( k ii1 ) ((pos "H0%" "K6%" ) (subcat )))
( "keyboard"  n ( k ii1 b oo d ) ((pos "K6%" ) ))
( "keyboards"  n ( k ii1 b oo d z ) ((pos "Kj%" ) ))
( "keyed"  v ( k ii1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "keyhole"  n ( k ii1 h ou l ) ((pos "K6%" ) ))
( "keyholes"  n ( k ii1 h ou l z ) ((pos "Kj%" ) ))
( "keying"  v ( k ii1 i ng ) ((pos "Hb%" ) (subcat )))
( "keyless"  j ( k ii1 l @ s ) ((pos "OA$" ) ))
( "keynote"  n ( k ii1 n ou t ) ((pos "K6%" ) ))
( "keynotes"  n ( k ii1 n ou t s ) ((pos "Kj%" ) ))
( "keyring"  n ( k ii1 r i ng ) ((pos "K6%" ) ))
( "keyrings"  n ( k ii1 r i ng z ) ((pos "Kj%" ) ))
( "keys"  v ( k ii1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "keystone"  n ( k ii1 s t ou n ) ((pos "K6%" ) ))
( "keystones"  n ( k ii1 s t ou n z ) ((pos "Kj$" ) ))
( "keyword"  n ( k ii1 w @@ d ) ((pos "K6%" ) ))
( "keywords"  n ( k ii1 w @@ d z ) ((pos "Kj%" ) ))
( "kg"  n ( k i1 l @ g r a m z ) ((pos "Y]%" ) ))
( "khaki"  n ( k aa1 k ii ) ((pos "L@%" "OA%" ) ))
( "khan"  n ( k aa1 n ) ((pos "K6$" ) ))
( "khans"  n ( k aa1 n z ) ((pos "Kj$" ) ))
( "kibbutz"  n ( k i1 b u1 t s ) ((pos "Ki%" ) ))
( "kibbutzim"  n ( k i1 b u t s ii1 m ) ((pos "Kj$" ) ))
( "kibbutznik"  n ( k i1 b u1 t s n i k ) ((pos "K6$" ) ))
( "kibbutzniks"  n ( k i1 b u1 t s n i k s ) ((pos "Kj$" ) ))
( "kick"  v ( k i1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "kick-start"  n ( k i1 k - s t aa t ) ((pos "K6%" ) ))
( "kick-starter"  n ( k i1 k - s t aa t @ r ) ((pos "K6%" ) ))
( "kick-starters"  n ( k i1 k - s t aa t @ z ) ((pos "Kj%" ) ))
( "kick-starts"  n ( k i1 k - s t aa t s ) ((pos "Kj$" ) ))
( "kickback"  n ( k i1 k b a k ) ((pos "K6%" ) ))
( "kickbacks"  n ( k i1 k b a k s ) ((pos "Kj$" ) ))
( "kicked"  v ( k i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "kicking"  v ( k i1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "kickoff"  n ( k i1 k o f ) ((pos "K6%" ) ))
( "kickoffs"  n ( k i1 k o f s ) ((pos "Kj%" ) ))
( "kicks"  v ( k i1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "kid"  v ( k i1 d ) ((pos "J4%" "M6%" ) ))
( "kidded"  v ( k i1 d i d ) ((pos "Jc%" "Jd%" ) ))
( "kiddies"  n ( k i1 d i z ) ((pos "Kj%" ) ))
( "kidding"  v ( k i1 d i ng ) ((pos "Jb%" ) ))
( "kiddy"  n ( k i1 d ii ) ((pos "K8%" ) ))
( "kidnap"  v ( k i1 d n a p ) ((pos "H4%" ) ))
( "kidnapped"  v ( k i1 d n a p t ) ((pos "Hc%" "Hd%" ) ))
( "kidnapper"  n ( k i1 d n a p @ r ) ((pos "K6%" ) ))
( "kidnappers"  n ( k i1 d n a p @ z ) ((pos "Kj%" ) ))
( "kidnapping"  v ( k i1 d n a p i ng ) ((pos "Hb%" ) ))
( "kidnaps"  v ( k i1 d n a p s ) ((pos "Ha%" ) ))
( "kidney"  n ( k i1 d n ii ) ((pos "K6%" ) ))
( "kidney-bean"  n ( k i2 d n i - b ii1 n ) ((pos "K6%" ) ))
( "kidney-beans"  n ( k i2 d n i - b ii1 n z ) ((pos "Kj%" ) ))
( "kidneys"  n ( k i1 d n i z ) ((pos "Kj%" ) ))
( "kids"  v ( k i1 d z ) ((pos "Ja%" "Mj%" ) ))
( "kill"  v ( k i1 l ) ((pos "J0%" "Ki%" ) (subcat "2A" "6A" )))
( "killed"  v ( k i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "killer"  n ( k i1 l @ r ) ((pos "K6%" ) ))
( "killers"  n ( k i1 l @ z ) ((pos "Kj%" ) ))
( "killing"  v ( k i1 l i ng ) ((pos "Jb%" "M6%" "OA%" ) (subcat "2A" "6A" )))
( "killingly"  a ( k i1 l i ng l ii ) ((pos "Pu$" ) ))
( "killings"  n ( k i1 l i ng z ) ((pos "Mj%" ) ))
( "killjoy"  n ( k i1 l jh oi ) ((pos "K6%" ) ))
( "killjoys"  n ( k i1 l jh oi z ) ((pos "Kj$" ) ))
( "kills"  v ( k i1 l z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "kiln"  n ( k i1 l n ) ((pos "K6%" ) ))
( "kilns"  n ( k i1 l n z ) ((pos "Kj%" ) ))
( "kilo"  n ( k ii1 l ou ) ((pos "K6%" ) ))
( "kilo-"  j ( k i1 l @ - ) ((pos "U-%" ) ))
( "kilocycle"  n ( k i1 l @ s ai k l ) ((pos "K6$" ) ))
( "kilocycles"  n ( k i1 l @ s ai k l z ) ((pos "Kj$" ) ))
( "kilogram"  n ( k i1 l @ g r a m ) ((pos "K6%" ) ))
( "kilograms"  n ( k i1 l @ g r a m z ) ((pos "Kj%" ) ))
( "kilolitre"  n ( k i1 l @ l ii t @ r ) ((pos "K6$" ) ))
( "kilolitres"  n ( k i1 l @ l ii t @ z ) ((pos "Kj$" ) ))
( "kilometre"  n ( k i1 l o1 m i t @ r ) ((pos "K6%" ) ))
( "kilometres"  n ( k i1 l o1 m i t @ z ) ((pos "Kj%" ) ))
( "kilos"  n ( k ii1 l ou z ) ((pos "Kj%" ) ))
( "kilowatt"  n ( k i1 l @ w o t ) ((pos "K6%" ) ))
( "kilowatts"  n ( k i1 l @ w o t s ) ((pos "Kj%" ) ))
( "kilt"  n ( k i1 l t ) ((pos "K6%" ) ))
( "kilts"  n ( k i1 l t s ) ((pos "Kj%" ) ))
( "kimono"  n ( k i1 m ou1 n ou ) ((pos "K6%" ) ))
( "kimonos"  n ( k i1 m ou1 n ou z ) ((pos "Kj%" ) ))
( "kin"  n ( k i1 n ) ((pos "Kj%" ) ))
( "kind"  n ( k ai1 n d ) ((pos "M6*" "OC%" ) ))
( "kind-hearted"  j ( k ai1 n d - h aa1 t i d ) ((pos "OA%" ) ))
( "kinda"  a ( k ai1 n d @ ) ((pos "Pu$" ) ))
( "kinder"  j ( k ai1 n d @ r ) ((pos "Or%" ) ))
( "kindergarten"  n ( k i1 n d @ g aa t n ) ((pos "K6%" ) ))
( "kindergartens"  n ( k i1 n d @ g aa t n z ) ((pos "Kj%" ) ))
( "kindest"  j ( k ai1 n d i s t ) ((pos "Os%" ) ))
( "kindle"  v ( k i1 n d l ) ((pos "J2%" ) (subcat "2A" )))
( "kindled"  v ( k i1 n d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "kindles"  v ( k i1 n d l z ) ((pos "Ja%" ) (subcat "2A" )))
( "kindlier"  j ( k ai1 n d l i@ r ) ((pos "Or%" ) ))
( "kindliest"  j ( k ai1 n d l i i s t ) ((pos "Os%" ) ))
( "kindling"  v ( k i1 n d l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "kindly"  j ( k ai1 n d l ii ) ((pos "OD%" "Pu%" ) ))
( "kindness"  n ( k ai1 n d n @ s ) ((pos "M7%" ) ))
( "kindnesses"  n ( k ai1 n d n @ s i z ) ((pos "Mj%" ) ))
( "kindred"  n ( k i1 n d r i d ) ((pos "L@%" "OA%" ) ))
( "kinds"  n ( k ai1 n d z ) ((pos "Mj%" ) ))
( "kine"  n ( k ai1 n ) ((pos "Kj$" ) ))
( "kinetic"  j ( k i1 n e1 t i k ) ((pos "OA$" ) ))
( "kinetics"  n ( k i1 n e1 t i k s ) ((pos "Lk$" ) ))
( "king"  n ( k i1 ng ) ((pos "K6%" ) ))
( "king-size"  j ( k i1 ng - s ai z ) ((pos "OA%" ) ))
( "king-sized"  j ( k i1 ng - s ai z d ) ((pos "OA%" ) ))
( "kingcup"  n ( k i1 ng k uh p ) ((pos "K6%" ) ))
( "kingcups"  n ( k i1 ng k uh p s ) ((pos "Kj%" ) ))
( "kingdom"  n ( k i1 ng d @ m ) ((pos "K6%" ) ))
( "kingdoms"  n ( k i1 ng d @ m z ) ((pos "Kj%" ) ))
( "kingfisher"  n ( k i1 ng f i sh @ r ) ((pos "K6%" ) ))
( "kingfishers"  n ( k i1 ng f i sh @ z ) ((pos "Kj%" ) ))
( "kinglike"  j ( k i1 ng l ai k ) ((pos "OA%" ) ))
( "kingly"  j ( k i1 ng l ii ) ((pos "OA%" ) ))
( "kingpin"  n ( k i1 ng p i n ) ((pos "K6%" ) ))
( "kingpins"  n ( k i1 ng p i n z ) ((pos "Kj$" ) ))
( "kings"  n ( k i1 ng z ) ((pos "Kj%" ) ))
( "kingship"  n ( k i1 ng sh i p ) ((pos "L@%" ) ))
( "kink"  v ( k i1 ng k ) ((pos "J0%" "K6%" ) ))
( "kinked"  v ( k i1 ng k t ) ((pos "Jc%" "Jd%" ) ))
( "kinkier"  j ( k i1 ng k i@ r ) ((pos "Or%" ) ))
( "kinkiest"  j ( k i1 ng k i i s t ) ((pos "Os%" ) ))
( "kinking"  v ( k i1 ng k i ng ) ((pos "Jb$" ) ))
( "kinks"  v ( k i1 ng k s ) ((pos "Ja%" "Kj%" ) ))
( "kinky"  j ( k i1 ng k ii ) ((pos "OD%" ) ))
( "kinsfolk"  n ( k i1 n z f ou k ) ((pos "Kj%" ) ))
( "kinship"  n ( k ii1 n sh i p ) ((pos "L@%" ) ))
( "kinsman"  n ( k i1 n z m @ n ) ((pos "Ki%" ) ))
( "kinsmen"  n ( k i1 n z m @ n ) ((pos "Kj%" ) ))
( "kinswoman"  n ( k i1 n z w u m @ n ) ((pos "Ki%" ) ))
( "kinswomen"  n ( k i1 n z w i m i n ) ((pos "Kj%" ) ))
( "kiosk"  n ( k ii1 o s k ) ((pos "K6%" ) ))
( "kiosks"  n ( k ii1 o s k s ) ((pos "Kj%" ) ))
( "kip"  v ( k i1 p ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "kipped"  v ( k i1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "kipper"  n ( k i1 p @ r ) ((pos "K6%" ) ))
( "kippers"  n ( k i1 p @ z ) ((pos "Kj%" ) ))
( "kipping"  v ( k i1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "kips"  v ( k i1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "kirk"  n ( k @@1 k ) ((pos "K6$" ) ))
( "kirks"  n ( k @@1 k s ) ((pos "Kj$" ) ))
( "kirsch"  n ( k i@1 sh ) ((pos "L@$" ) ))
( "kirtle"  n ( k @@1 t l ) ((pos "K6$" ) ))
( "kirtles"  n ( k @@1 t l z ) ((pos "Kj$" ) ))
( "kismet"  n ( k i1 z m e t ) ((pos "L@$" ) ))
( "kiss"  v ( k i1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" "15A" )))
( "kissed"  v ( k i1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "kisser"  n ( k i1 s @ r ) ((pos "K6%" ) ))
( "kissers"  n ( k i1 s @ z ) ((pos "Kj%" ) ))
( "kisses"  v ( k i1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "kissing"  v ( k i1 s i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "kit"  v ( k i1 t ) ((pos "H4%" "M6%" ) (subcat )))
( "kitbag"  n ( k i1 t b a g ) ((pos "K6%" ) ))
( "kitbags"  n ( k i1 t b a g z ) ((pos "Kj%" ) ))
( "kitchen"  n ( k i1 ch i n ) ((pos "K6%" ) ))
( "kitchenette"  n ( k i2 ch i n e1 t ) ((pos "K6%" ) ))
( "kitchenettes"  n ( k i2 ch i n e1 t s ) ((pos "Kj%" ) ))
( "kitchens"  n ( k i1 ch i n z ) ((pos "Kj%" ) ))
( "kite"  n ( k ai1 t ) ((pos "K6%" ) ))
( "kite-balloon"  n ( k ai1 t - b @ l uu n ) ((pos "K6$" ) ))
( "kite-balloons"  n ( k ai1 t - b @ l uu n z ) ((pos "Kj$" ) ))
( "kites"  n ( k ai1 t s ) ((pos "Kj%" ) ))
( "kith"  n ( k i1 th ) ((pos "Kj%" ) ))
( "kits"  v ( k i1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "kitsch"  j ( k i1 ch ) ((pos "OA%" ) ))
( "kitted"  v ( k i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "kitten"  n ( k i1 t n ) ((pos "K6%" ) ))
( "kittenish"  j ( k i1 t n i sh ) ((pos "OA%" ) ))
( "kittens"  n ( k i1 t n z ) ((pos "Kj%" ) ))
( "kitties"  n ( k i1 t i z ) ((pos "Kj%" ) ))
( "kitting"  v ( k i1 t i ng ) ((pos "Hb$" ) (subcat )))
( "kitty"  n ( k i1 t ii ) ((pos "K8%" ) ))
( "kiwi"  n ( k ii1 w ii ) ((pos "K6%" ) ))
( "kiwis"  n ( k ii1 w ii z ) ((pos "Kj%" ) ))
( "klaxon"  n ( k l a1 k s n ) ((pos "K6%" ) ))
( "klaxons"  n ( k l a1 k s n z ) ((pos "Kj%" ) ))
( "kleenex"  n ( k l ii1 n e k s ) ((pos "M7%" ) ))
( "kleenexes"  n ( k l ii1 n e k s i z ) ((pos "Mj$" ) ))
( "kleptomania"  n ( k l e2 p t @ m ei1 n i@ ) ((pos "L@%" ) ))
( "kleptomaniac"  n ( k l e2 p t @ m ei1 n i a k ) ((pos "K6%" ) ))
( "kleptomaniacs"  n ( k l e2 p t @ m ei1 n i a k s ) ((pos "Kj%" ) ))
( "km"  n ( k i1 l @ m ii t @ z ) ((pos "Y]%" ) ))
( "knack"  n ( n a1 k ) ((pos "K6%" ) ))
( "knacker"  n ( n a1 k @ r ) ((pos "K6%" ) ))
( "knackers"  n ( n a1 k @ z ) ((pos "Kj%" ) ))
( "knacks"  n ( n a1 k s ) ((pos "Kj$" ) ))
( "knap"  v ( n a1 p ) ((pos "H4$" ) ))
( "knapped"  v ( n a1 p t ) ((pos "Hc$" "Hd%" ) ))
( "knapping"  v ( n a1 p i ng ) ((pos "Hb$" ) ))
( "knaps"  v ( n a1 p s ) ((pos "Ha$" ) ))
( "knapsack"  n ( n a1 p s a k ) ((pos "K6%" ) ))
( "knapsacks"  n ( n a1 p s a k s ) ((pos "Kj%" ) ))
( "knave"  n ( n ei1 v ) ((pos "K6%" ) ))
( "knaveries"  n ( n ei1 v @ r i z ) ((pos "Mj%" ) ))
( "knavery"  n ( n ei1 v @ r ii ) ((pos "M8%" ) ))
( "knaves"  n ( n ei1 v z ) ((pos "Kj%" ) ))
( "knavish"  j ( n ei1 v i sh ) ((pos "OA%" ) ))
( "knavishly"  a ( n ei1 v i sh l ii ) ((pos "Pu%" ) ))
( "knead"  v ( n ii1 d ) ((pos "H0$" ) (subcat )))
( "kneaded"  v ( n ii1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "kneading"  v ( n ii1 d i ng ) ((pos "Hb$" ) (subcat )))
( "kneads"  v ( n ii1 d z ) ((pos "Ha$" ) (subcat )))
( "knee"  n ( n ii1 ) ((pos "K6%" ) ))
( "knee-breeches"  n ( n ii1 - b r i ch i z ) ((pos "Kj%" ) ))
( "knee-deep"  j ( n ii1 - d ii1 p ) ((pos "OA%" "Pu%" ) ))
( "knee-high"  j ( n ii1 - h ai1 ) ((pos "OA%" "Pu%" ) ))
( "kneecap"  n ( n ii1 k a p ) ((pos "K6%" ) ))
( "kneecaps"  n ( n ii1 k a p s ) ((pos "Kj%" ) ))
( "kneel"  v ( n ii1 l ) ((pos "I0%" ) (subcat "2A" )))
( "kneeled"  v ( n ii1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "kneeling"  v ( n ii1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "kneels"  v ( n ii1 l z ) ((pos "Ia%" ) (subcat "2A" )))
( "knees"  n ( n ii1 z ) ((pos "Kj%" ) ))
( "knell"  n ( n e1 l ) ((pos "Ki$" ) ))
( "knelt"  v ( n e1 l t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "knew"  v ( n y uu1 ) ((pos "Jc*" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "knick-knack"  n ( n i1 k - n a k ) ((pos "K6%" ) ))
( "knick-knacks"  n ( n i1 k - n a k s ) ((pos "Kj%" ) ))
( "knickerbockers"  n ( n i1 k @ b o k @ z ) ((pos "Kj%" ) ))
( "knickers"  n ( n i1 k @ z ) ((pos "Kj%" ) ))
( "knife"  v ( n ai1 f ) ((pos "H2%" "Ki%" ) (subcat )))
( "knife-edge"  n ( n ai1 f - e jh ) ((pos "K6%" ) ))
( "knife-edges"  n ( n ai1 f - e jh i z ) ((pos "Kj$" ) ))
( "knifed"  v ( n ai1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "knifes"  v ( n ai1 f s ) ((pos "Ha%" ) (subcat )))
( "knifing"  v ( n ai1 f i ng ) ((pos "Hb%" ) (subcat )))
( "knight"  v ( n ai1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "knight-errant"  n ( n ai1 t - e1 r @ n t ) ((pos "Ki%" ) ))
( "knighted"  v ( n ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "knighthood"  n ( n ai1 t h u d ) ((pos "M6%" ) ))
( "knighthoods"  n ( n ai1 t h u d z ) ((pos "Mj%" ) ))
( "knighting"  v ( n ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "knightly"  j ( n ai1 t l ii ) ((pos "OA$" ) ))
( "knights"  v ( n ai1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "knights-errant"  n ( n ai1 t s - e1 r @ n t ) ((pos "Kj%" ) ))
( "knit"  v ( n i1 t ) ((pos "J4%" "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "knits"  v ( n i1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "knitted"  v ( n i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "knitter"  n ( n i1 t @ r ) ((pos "K6%" ) ))
( "knitters"  n ( n i1 t @ z ) ((pos "Kj%" ) ))
( "knitting"  v ( n i1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "15A" )))
( "knitting-machine"  n ( n i1 t i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "knitting-machines"  n ( n i1 t i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "knitting-needle"  n ( n i1 t i ng - n ii d l ) ((pos "K6%" ) ))
( "knitting-needles"  n ( n i1 t i ng - n ii d l z ) ((pos "Kj%" ) ))
( "knitwear"  n ( n i1 t w e@ r ) ((pos "L@%" ) ))
( "knives"  n ( n ai1 v z ) ((pos "Kj%" ) ))
( "knob"  n ( n o1 b ) ((pos "K6%" ) ))
( "knobble"  n ( n o1 b l ) ((pos "K6$" ) ))
( "knobbles"  n ( n o1 b l z ) ((pos "Kj$" ) ))
( "knobblier"  j ( n o1 b l i@ r ) ((pos "Or$" ) ))
( "knobbliest"  j ( n o1 b l i i s t ) ((pos "Os$" ) ))
( "knobbly"  j ( n o1 b l ii ) ((pos "OD%" ) ))
( "knobkerrie"  n ( n o1 b k e r ii ) ((pos "K6$" ) ))
( "knobkerries"  n ( n o1 b k e r i z ) ((pos "Kj$" ) ))
( "knobs"  n ( n o1 b z ) ((pos "Kj%" ) ))
( "knock"  v ( n o1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "knock-kneed"  j ( n o1 k - n ii1 d ) ((pos "OA%" ) ))
( "knock-on"  n ( n o1 k - o1 n ) ((pos "K6%" ) ))
( "knock-ons"  n ( n o1 k - o1 n z ) ((pos "Kj$" ) ))
( "knockabout"  j ( n o1 k @ b au t ) ((pos "OA%" ) ))
( "knockdown"  j ( n o1 k d au n ) ((pos "OA%" ) ))
( "knocked"  v ( n o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "knocker"  n ( n o1 k @ r ) ((pos "K6%" ) ))
( "knockers"  n ( n o1 k @ z ) ((pos "Kj%" ) ))
( "knocking"  v ( n o1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "knockout"  n ( n o1 k au t ) ((pos "K6%" "OA%" ) ))
( "knockouts"  n ( n o1 k au t s ) ((pos "Kj%" ) ))
( "knocks"  v ( n o1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "knoll"  n ( n ou1 l ) ((pos "K6$" ) ))
( "knolls"  n ( n ou1 l z ) ((pos "Kj$" ) ))
( "knot"  v ( n o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" )))
( "knothole"  n ( n o1 t h ou l ) ((pos "K6%" ) ))
( "knotholes"  n ( n o1 t h ou l z ) ((pos "Kj%" ) ))
( "knots"  v ( n o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "knotted"  v ( n o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "knottier"  j ( n o1 t i@ r ) ((pos "Or%" ) ))
( "knottiest"  j ( n o1 t i i s t ) ((pos "Os%" ) ))
( "knotting"  v ( n o1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "knotty"  j ( n o1 t ii ) ((pos "OD%" ) ))
( "knout"  n ( n au1 t ) ((pos "K6$" ) ))
( "knouts"  n ( n au1 t s ) ((pos "Kj$" ) ))
( "know"  v ( n ou1 ) ((pos "J5*" "Ki%" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "know-all"  n ( n ou1 - oo l ) ((pos "K6%" ) ))
( "know-alls"  n ( n ou1 - oo l z ) ((pos "Kj%" ) ))
( "know-how"  n ( n ou1 - h au ) ((pos "L@%" ) ))
( "knowing"  v ( n ou1 i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "knowingly"  a ( n ou1 i ng l ii ) ((pos "Pu%" ) ))
( "knowledge"  n ( n o1 l i jh ) ((pos "L@%" ) ))
( "knowledgeable"  j ( n o1 l i jh @ b l ) ((pos "OA%" ) ))
( "known"  v ( n ou1 n ) ((pos "Jd*" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "knows"  v ( n ou1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "9" "10" "17" "18B" )))
( "knuckle"  v ( n uh1 k l ) ((pos "I2%" "K6%" ) ))
( "knuckled"  v ( n uh1 k l d ) ((pos "Ic%" "Id%" ) ))
( "knuckles"  v ( n uh1 k l z ) ((pos "Ia%" "Kj%" ) ))
( "knuckling"  v ( n uh1 k l i ng ) ((pos "Ib%" ) ))
( "koala"  n ( k ou1 aa1 l @ ) ((pos "K6%" ) ))
( "koalas"  n ( k ou1 aa1 l @ z ) ((pos "Kj%" ) ))
( "kobo"  n ( k ou1 b ou ) ((pos "K6$" ) ))
( "kobos"  n ( k ou1 b ou z ) ((pos "Kj$" ) ))
( "kohl"  n ( k ou1 l ) ((pos "L@$" ) ))
( "kohlrabi"  n ( k ou2 l r aa1 b ii ) ((pos "M6%" ) ))
( "kohlrabis"  n ( k ou2 l r aa1 b i z ) ((pos "Mj$" ) ))
( "kola"  n ( k ou1 l @ ) ((pos "K6$" ) ))
( "kola-nut"  n ( k ou1 l @ - n uh t ) ((pos "K6$" ) ))
( "kola-nuts"  n ( k ou1 l @ - n uh t s ) ((pos "Kj$" ) ))
( "kolas"  n ( k ou1 l @ z ) ((pos "Kj$" ) ))
( "kookaburra"  n ( k u1 k @ b uh r @ ) ((pos "K6$" ) ))
( "kookaburras"  n ( k u1 k @ b uh r @ z ) ((pos "Kj$" ) ))
( "kopeck"  n ( k ou1 p e k ) ((pos "K6$" ) ))
( "kopecks"  n ( k ou1 p e k s ) ((pos "Kj$" ) ))
( "kopje"  n ( k o1 p ii ) ((pos "K6$" ) ))
( "kopjes"  n ( k o1 p i z ) ((pos "Kj$" ) ))
( "koppie"  n ( k o1 p ii ) ((pos "K6$" ) ))
( "koppies"  n ( k o1 p i z ) ((pos "Kj$" ) ))
( "kosher"  n ( k ou1 sh @ r ) ((pos "L@$" "OA$" ) ))
( "kotow"  v ( k au2 t au1 ) ((pos "I0$" "K6$" ) ))
( "kotowed"  v ( k au2 t au1 d ) ((pos "Ic$" "Id$" ) ))
( "kotowing"  v ( k au2 t au1 i ng ) ((pos "Ib$" ) ))
( "kotows"  v ( k au2 t au1 z ) ((pos "Ia$" "Kj$" ) ))
( "koumiss"  n ( k uu1 m i s ) ((pos "L@$" ) ))
( "kowtow"  v ( k au2 t au1 ) ((pos "I0$" "K6$" ) ))
( "kowtowed"  v ( k au2 t au1 d ) ((pos "Ic$" "Id$" ) ))
( "kowtowing"  v ( k au2 t au1 i ng ) ((pos "Ib$" ) ))
( "kowtows"  v ( k au2 t au1 z ) ((pos "Ia$" "Kj$" ) ))
( "kraal"  n ( k r aa1 l ) ((pos "K6$" ) ))
( "kraals"  n ( k r aa1 l z ) ((pos "Kj$" ) ))
( "krona"  n ( k r ou1 n @ ) ((pos "Ki$" ) ))
( "krone"  n ( k r ou1 n @ ) ((pos "Ki$" ) ))
( "kroner"  n ( k r ou1 n @ r ) ((pos "Kj$" ) ))
( "kronor"  n ( k r ou1 n @ r ) ((pos "Kj$" ) ))
( "kudos"  n ( k y uu1 d o s ) ((pos "L@$" ) ))
( "kumis"  n ( k uu1 m i s ) ((pos "L@$" ) ))
( "kung fu"  n ( k u2 ng - f uu1 ) ((pos "L@%" ) ))
( "kvass"  n ( k v a1 s ) ((pos "L@$" ) ))
( "kw"  n ( k i1 l @ w o t s ) ((pos "Y]%" ) ))
( "kwacha"  n ( k w a1 ch @ ) ((pos "K9$" ) ))
( "kwela"  n ( k w ei1 l @ ) ((pos "L@$" ) ))
( "l"  n ( e1 l ) ((pos "Ki$" ) ))
( "l's"  n ( e1 l z ) ((pos "Kj$" ) ))
( "la"  n ( l aa1 ) ((pos "Ki$" ) ))
( "la-di-da"  j ( l aa2 - d i - d aa1 ) ((pos "OA$" ) ))
( "laager"  n ( l aa1 g @ r ) ((pos "K6$" ) ))
( "laagers"  n ( l aa1 g @ z ) ((pos "Kj$" ) ))
( "lab"  n ( l a1 b ) ((pos "K6%" ) ))
( "label"  v ( l ei1 b l ) ((pos "H4%" "K6%" ) (subcat )))
( "labelled"  v ( l ei1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "labelling"  v ( l ei1 b @ l i ng ) ((pos "Hb%" ) (subcat )))
( "labels"  v ( l ei1 b l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "labial"  j ( l ei1 b i@ l ) ((pos "OA%" ) ))
( "laboratories"  n ( l @1 b o1 r @ t r i z ) ((pos "Kj%" ) ))
( "laboratory"  n ( l @1 b o1 r @ t r ii ) ((pos "K8%" ) ))
( "laborious"  j ( l @1 b oo1 r i@ s ) ((pos "OA%" ) ))
( "laboriously"  a ( l @1 b oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "labour"  v ( l ei1 b @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "4A" )))
( "labour-saving"  j ( l ei1 b @ - s ei v i ng ) ((pos "OA%" ) ))
( "laboured"  v ( l ei1 b @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" "4A" )))
( "labourer"  n ( l ei1 b @ r @ r ) ((pos "K6%" ) ))
( "labourers"  n ( l ei1 b @ r @ z ) ((pos "Kj%" ) ))
( "labouring"  v ( l ei1 b @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" )))
( "labours"  v ( l ei1 b @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "4A" )))
( "labs"  n ( l a1 b z ) ((pos "Kj%" ) ))
( "laburnum"  n ( l @1 b @@1 n @ m ) ((pos "K6%" ) ))
( "laburnums"  n ( l @1 b @@1 n @ m z ) ((pos "Kj%" ) ))
( "labyrinth"  n ( l a1 b @ r i n th ) ((pos "K6%" ) ))
( "labyrinthine"  j ( l a2 b @ r i1 n th ai n ) ((pos "OA%" ) ))
( "labyrinths"  n ( l a1 b @ r i n th s ) ((pos "Kj%" ) ))
( "lace"  v ( l ei1 s ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "laced"  v ( l ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "lacerate"  v ( l a1 s @ r ei t ) ((pos "H2%" ) (subcat )))
( "lacerated"  v ( l a1 s @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lacerates"  v ( l a1 s @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "lacerating"  v ( l a1 s @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "laceration"  n ( l a2 s @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "lacerations"  n ( l a2 s @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "laces"  v ( l ei1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "lachrymal"  j ( l a1 k r i m l ) ((pos "OA$" ) ))
( "lachrymose"  j ( l a1 k r i m ou s ) ((pos "OA$" ) ))
( "lacier"  j ( l ei1 s i@ r ) ((pos "Or$" ) ))
( "laciest"  j ( l ei1 s i i s t ) ((pos "Os$" ) ))
( "lacing"  v ( l ei1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "lack"  v ( l a1 k ) ((pos "J0%" "L@%" ) (subcat "3A" )))
( "lack-lustre"  j ( l a1 k - l uh s t @ r ) ((pos "OA%" ) ))
( "lackadaisical"  j ( l a2 k @ d ei1 z i k l ) ((pos "OA%" ) ))
( "lackadaisically"  a ( l a2 k @ d ei1 z i k l ii ) ((pos "Pu%" ) ))
( "lacked"  v ( l a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "lackey"  n ( l a1 k ii ) ((pos "K6%" ) ))
( "lackeys"  n ( l a1 k i z ) ((pos "Kj%" ) ))
( "lacking"  v ( l a1 k i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "lacks"  v ( l a1 k s ) ((pos "Ja%" ) (subcat "3A" )))
( "laconic"  j ( l @1 k o1 n i k ) ((pos "OA%" ) ))
( "laconically"  a ( l @1 k o1 n i k l ii ) ((pos "Pu%" ) ))
( "laconicism"  n ( l @1 k o1 n i s i z @ m ) ((pos "M6$" ) ))
( "laconicisms"  n ( l @1 k o1 n i s i z @ m z ) ((pos "Mj$" ) ))
( "laconism"  n ( l a1 k @ n i z @ m ) ((pos "M6$" ) ))
( "laconisms"  n ( l a1 k @ n i z @ m z ) ((pos "Mj$" ) ))
( "lacquer"  v ( l a1 k @ r ) ((pos "H0$" "M6%" ) (subcat )))
( "lacquered"  v ( l a1 k @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "lacquering"  v ( l a1 k @ r i ng ) ((pos "Hb$" ) (subcat )))
( "lacquers"  v ( l a1 k @ z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "lacrosse"  n ( l @1 k r o1 s ) ((pos "L@%" ) ))
( "lactic"  j ( l a1 k t i k ) ((pos "OA%" ) ))
( "lacuna"  n ( l @1 k y uu1 n @ ) ((pos "K6$" ) ))
( "lacunae"  n ( l @1 k y uu1 n ii ) ((pos "Kj$" ) ))
( "lacunas"  n ( l @1 k y uu1 n @ z ) ((pos "Kj$" ) ))
( "lacy"  j ( l ei1 s ii ) ((pos "OD%" ) ))
( "lad"  n ( l a1 d ) ((pos "K6%" ) ))
( "ladder"  v ( l a1 d @ r ) ((pos "I0%" "K6%" ) ))
( "ladder-proof"  j ( l a1 d @ - p r uu f ) ((pos "OA%" ) ))
( "laddered"  v ( l a1 d @ d ) ((pos "Ic%" "Id%" ) ))
( "laddering"  v ( l a1 d @ r i ng ) ((pos "Ib%" ) ))
( "ladders"  v ( l a1 d @ z ) ((pos "Ia%" "Kj%" ) ))
( "laddie"  n ( l a1 d ii ) ((pos "K6%" ) ))
( "laddies"  n ( l a1 d i z ) ((pos "Kj%" ) ))
( "lade"  v ( l ei1 d ) ((pos "H5$" ) (subcat )))
( "laden"  v ( l ei1 d n ) ((pos "Hd%" "OA%" ) (subcat )))
( "ladies"  n ( l ei1 d i z ) ((pos "Kj%" ) ))
( "ladies-in-waiting"  n ( l ei2 d i z - i n - w ei1 t i ng ) ((pos "Kj%" ) ))
( "lading"  n ( l ei1 d i ng ) ((pos "L@$" ) ))
( "ladle"  v ( l ei1 d l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "ladled"  v ( l ei1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "ladles"  v ( l ei1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "ladling"  v ( l ei1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "lads"  n ( l a1 d z ) ((pos "Kj%" ) ))
( "lady"  n ( l ei1 d ii ) ((pos "K8%" ) ))
( "lady's-maid"  n ( l ei1 d i z - m ei d ) ((pos "K6%" ) ))
( "lady's-maids"  n ( l ei1 d i z - m ei d z ) ((pos "Kj%" ) ))
( "lady-in-waiting"  n ( l ei2 d i - i n - w ei1 t i ng ) ((pos "Ki%" ) ))
( "lady-killer"  n ( l ei1 d i - k i l @ r ) ((pos "K6$" ) ))
( "lady-killers"  n ( l ei1 d i - k i l @ z ) ((pos "Kj$" ) ))
( "ladybird"  n ( l ei1 d i b @@ d ) ((pos "K6%" ) ))
( "ladybirds"  n ( l ei1 d i b @@ d z ) ((pos "Kj%" ) ))
( "ladylike"  j ( l ei1 d i l ai k ) ((pos "OA%" ) ))
( "ladyship"  n ( l ei1 d i sh i p ) ((pos "K6%" ) ))
( "ladyships"  n ( l ei1 d i sh i p s ) ((pos "Kj%" ) ))
( "lag"  v ( l a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "lager"  n ( l aa1 g @ r ) ((pos "M6%" ) ))
( "lagers"  n ( l aa1 g @ z ) ((pos "Mj%" ) ))
( "laggard"  n ( l a1 g @ d ) ((pos "K6%" ) ))
( "laggards"  n ( l a1 g @ d z ) ((pos "Kj%" ) ))
( "lagged"  v ( l a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "lagging"  v ( l a1 g i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "lagoon"  n ( l @1 g uu1 n ) ((pos "K6%" ) ))
( "lagoons"  n ( l @1 g uu1 n z ) ((pos "Kj%" ) ))
( "lags"  v ( l a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "laic"  j ( l ei1 i k ) ((pos "OA$" ) ))
( "laicize"  v ( l ei1 i s ai z ) ((pos "H2$" ) (subcat )))
( "laicized"  v ( l ei1 i s ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "laicizes"  v ( l ei1 i s ai z i z ) ((pos "Ha$" ) (subcat )))
( "laicizing"  v ( l ei1 i s ai z i ng ) ((pos "Hb$" ) (subcat )))
( "laid"  v ( l ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "12C" "14" "15A" "15B" )))
( "lain"  v ( l ei1 n ) ((pos "Id$" ) (subcat "2A" "2C" "2D" )))
( "lair"  n ( l e@1 r ) ((pos "K6%" ) ))
( "laird"  n ( l e@1 d ) ((pos "K6%" ) ))
( "lairds"  n ( l e@1 d z ) ((pos "Kj%" ) ))
( "lairs"  n ( l e@1 z ) ((pos "Kj%" ) ))
( "laissez-faire"  n ( l ei2 s ei - f e@1 r ) ((pos "L@%" ) ))
( "laities"  n ( l ei1 @ t i z ) ((pos "Kj$" ) ))
( "laity"  n ( l ei1 @ t ii ) ((pos "K8%" ) ))
( "lake"  n ( l ei1 k ) ((pos "K6%" ) ))
( "lakes"  n ( l ei1 k s ) ((pos "Kj%" ) ))
( "lakh"  n ( l aa1 k ) ((pos "K6$" ) ))
( "lakhs"  n ( l aa1 k s ) ((pos "Kj$" ) ))
( "lam"  v ( l a1 m ) ((pos "J4$" ) (subcat "3A" )))
( "lam_e"  n ( l aa1 m ei ) ((pos "L@%" ) ))
( "lama"  n ( l aa1 m @ ) ((pos "K6$" ) ))
( "lamas"  n ( l aa1 m @ z ) ((pos "Kj$" ) ))
( "lamaseries"  n ( l aa1 m @ s @ r i z ) ((pos "Kj$" ) ))
( "lamasery"  n ( l aa1 m @ s @ r ii ) ((pos "K8$" ) ))
( "lamb"  v ( l a1 m ) ((pos "I0%" "M6%" ) ))
( "lambaste"  v ( l a1 m b ei1 s t ) ((pos "H2$" ) ))
( "lambasted"  v ( l a1 m b ei1 s t i d ) ((pos "Hc$" "Hd%" ) ))
( "lambastes"  v ( l a1 m b ei1 s t s ) ((pos "Ha$" ) ))
( "lambasting"  v ( l a1 m b ei1 s t i ng ) ((pos "Hb$" ) ))
( "lambed"  v ( l a1 m d ) ((pos "Ic%" "Id%" ) ))
( "lambency"  n ( l a1 m b @ n s ii ) ((pos "L@$" ) ))
( "lambent"  j ( l a1 m b @ n t ) ((pos "OA$" ) ))
( "lambing"  v ( l a1 m i ng ) ((pos "Ib%" ) ))
( "lambkin"  n ( l a1 m k i n ) ((pos "K6%" ) ))
( "lambkins"  n ( l a1 m k i n z ) ((pos "Kj%" ) ))
( "lambs"  v ( l a1 m z ) ((pos "Ia%" "Mj%" ) ))
( "lambskin"  n ( l a1 m s k i n ) ((pos "M6%" ) ))
( "lambskins"  n ( l a1 m s k i n z ) ((pos "Mj$" ) ))
( "lame"  v ( l ei1 m ) ((pos "H2%" "OB%" ) ))
( "lamed"  v ( l ei1 m d ) ((pos "Hc%" "Hd%" ) ))
( "lamely"  a ( l ei1 m l ii ) ((pos "Pu%" ) ))
( "lameness"  n ( l ei1 m n @ s ) ((pos "L@%" ) ))
( "lament"  v ( l @1 m e1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" )))
( "lamentable"  j ( l a1 m @ n t @ b l ) ((pos "OA%" ) ))
( "lamentably"  a ( l a1 m @ n t @ b l ii ) ((pos "Pu%" ) ))
( "lamentation"  n ( l a2 m e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "lamentations"  n ( l a2 m e n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "lamented"  v ( l @1 m e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "lamenting"  v ( l @1 m e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "laments"  v ( l @1 m e1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "lamer"  j ( l ei1 m @ r ) ((pos "Or%" ) ))
( "lames"  v ( l ei1 m z ) ((pos "Ha%" ) ))
( "lamest"  j ( l ei1 m i s t ) ((pos "Os%" ) ))
( "laminate"  v ( l a1 m i n ei t ) ((pos "J2%" ) (subcat "2A" )))
( "laminated"  v ( l a1 m i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "laminates"  v ( l a1 m i n ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "laminating"  v ( l a1 m i n ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "laming"  v ( l ei1 m i ng ) ((pos "Hb%" ) ))
( "lammed"  v ( l a1 m d ) ((pos "Jc$" "Jd$" ) (subcat "3A" )))
( "lamming"  v ( l a1 m i ng ) ((pos "Jb$" ) (subcat "3A" )))
( "lamp"  n ( l a1 m p ) ((pos "K6%" ) ))
( "lamp-black"  n ( l a1 m p - b l a k ) ((pos "L@%" ) ))
( "lamplight"  n ( l a1 m p l ai t ) ((pos "L@%" ) ))
( "lamplighter"  n ( l a1 m p l ai t @ r ) ((pos "K6%" ) ))
( "lamplighters"  n ( l a1 m p l ai t @ z ) ((pos "Kj%" ) ))
( "lamplit"  j ( l a1 m p l i t ) ((pos "OA%" ) ))
( "lampoon"  v ( l a1 m p uu1 n ) ((pos "H0%" "K6%" ) (subcat )))
( "lampooned"  v ( l a1 m p uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lampooning"  v ( l a1 m p uu1 n i ng ) ((pos "Hb%" ) (subcat )))
( "lampoons"  v ( l a1 m p uu1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "lamppost"  n ( l a1 m p ou s t ) ((pos "K6%" ) ))
( "lampposts"  n ( l a1 m p ou s t s ) ((pos "Kj%" ) ))
( "lamprey"  n ( l a1 m p r ii ) ((pos "K6%" ) ))
( "lampreys"  n ( l a1 m p r i z ) ((pos "Kj%" ) ))
( "lamps"  n ( l a1 m p s ) ((pos "Kj%" ) ))
( "lampshade"  n ( l a1 m p sh ei d ) ((pos "K6%" ) ))
( "lampshades"  n ( l a1 m p sh ei d z ) ((pos "Kj%" ) ))
( "lams"  v ( l a1 m z ) ((pos "Ja$" ) (subcat "3A" )))
( "lance"  v ( l aa1 n s ) ((pos "H2%" "K6%" ) (subcat )))
( "lance-corporal"  n ( l aa1 n s - k oo1 p @ r @ l ) ((pos "K6%" ) ))
( "lance-corporals"  n ( l aa1 n s - k oo1 p @ r @ l z ) ((pos "Kj%" ) ))
( "lanced"  v ( l aa1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lancer"  n ( l aa1 n s @ r ) ((pos "K6%" ) ))
( "lancers"  n ( l aa1 n s @ z ) ((pos "Kj%" ) ))
( "lances"  v ( l aa1 n s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "lancet"  n ( l aa1 n s i t ) ((pos "K6%" ) ))
( "lancets"  n ( l aa1 n s i t s ) ((pos "Kj%" ) ))
( "lancing"  v ( l aa1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "land"  v ( l a1 n d ) ((pos "J0%" "M6*" ) (subcat "2A" "2C" "6A" )))
( "land-agent"  n ( l a1 n d - ei jh @ n t ) ((pos "K6%" ) ))
( "land-agents"  n ( l a1 n d - ei jh @ n t s ) ((pos "Kj%" ) ))
( "landau"  n ( l a1 n d oo ) ((pos "K6$" ) ))
( "landaus"  n ( l a1 n d oo z ) ((pos "Kj$" ) ))
( "landed"  v ( l a1 n d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "landfall"  n ( l a1 n d f oo l ) ((pos "K6%" ) ))
( "landfalls"  n ( l a1 n d f oo l z ) ((pos "Kj%" ) ))
( "landgrave"  n ( l a1 n d g r ei v ) ((pos "K6$" ) ))
( "landgraves"  n ( l a1 n d g r ei v z ) ((pos "Kj$" ) ))
( "landholder"  n ( l a1 n d h ou l d @ r ) ((pos "K6%" ) ))
( "landholders"  n ( l a1 n d h ou l d @ z ) ((pos "Kj%" ) ))
( "landing"  v ( l a1 n d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "landing-craft"  n ( l a1 n d i ng - k r aa f t ) ((pos "K9%" ) ))
( "landing-field"  n ( l a1 n d i ng - f ii l d ) ((pos "K6%" ) ))
( "landing-fields"  n ( l a1 n d i ng - f ii l d z ) ((pos "Kj%" ) ))
( "landing-gear"  n ( l a1 n d i ng - g i@ r ) ((pos "L@%" ) ))
( "landing-net"  n ( l a1 n d i ng - n e t ) ((pos "K6%" ) ))
( "landing-nets"  n ( l a1 n d i ng - n e t s ) ((pos "Kj%" ) ))
( "landing-parties"  n ( l a1 n d i ng - p aa t i z ) ((pos "Kj%" ) ))
( "landing-party"  n ( l a1 n d i ng - p aa t ii ) ((pos "K8%" ) ))
( "landing-place"  n ( l a1 n d i ng - p l ei s ) ((pos "K6%" ) ))
( "landing-places"  n ( l a1 n d i ng - p l ei s i z ) ((pos "Kj%" ) ))
( "landing-stage"  n ( l a1 n d i ng - s t ei jh ) ((pos "K6%" ) ))
( "landing-stages"  n ( l a1 n d i ng - s t ei jh i z ) ((pos "Kj%" ) ))
( "landing-strip"  n ( l a1 n d i ng - s t r i p ) ((pos "K6%" ) ))
( "landing-strips"  n ( l a1 n d i ng - s t r i p s ) ((pos "Kj%" ) ))
( "landings"  n ( l a1 n d i ng z ) ((pos "Kj%" ) ))
( "landladies"  n ( l a1 n d l ei d i z ) ((pos "Kj%" ) ))
( "landlady"  n ( l a1 n d l ei d ii ) ((pos "K8%" ) ))
( "landless"  j ( l a1 n d l @ s ) ((pos "OA%" ) ))
( "landlocked"  j ( l a1 n d l o k t ) ((pos "OA%" ) ))
( "landlord"  n ( l a1 n d l oo d ) ((pos "K6%" ) ))
( "landlords"  n ( l a1 n d l oo d z ) ((pos "Kj%" ) ))
( "landlubber"  n ( l a1 n d l uh b @ r ) ((pos "K6%" ) ))
( "landlubbers"  n ( l a1 n d l uh b @ z ) ((pos "Kj%" ) ))
( "landmark"  n ( l a1 n d m aa k ) ((pos "K6%" ) ))
( "landmarks"  n ( l a1 n d m aa k s ) ((pos "Kj%" ) ))
( "landmine"  n ( l a1 n d m ai n ) ((pos "K6%" ) ))
( "landmines"  n ( l a1 n d m ai n z ) ((pos "Kj%" ) ))
( "landowner"  n ( l a1 n d ou n @ r ) ((pos "K6%" ) ))
( "landowners"  n ( l a1 n d ou n @ z ) ((pos "Kj%" ) ))
( "lands"  v ( l a1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "landscape"  v ( l a1 n d s k ei p ) ((pos "H2%" "M6%" ) (subcat )))
( "landscaped"  v ( l a1 n d s k ei p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "landscapes"  v ( l a1 n d s k ei p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "landscaping"  v ( l a1 n d s k ei p i ng ) ((pos "Hb%" ) (subcat )))
( "landslide"  n ( l a1 n d s l ai d ) ((pos "K6%" ) ))
( "landslides"  n ( l a1 n d s l ai d z ) ((pos "Kj%" ) ))
( "landslip"  n ( l a1 n d s l i p ) ((pos "K6%" ) ))
( "landslips"  n ( l a1 n d s l i p s ) ((pos "Kj%" ) ))
( "landsman"  n ( l a1 n d z m @ n ) ((pos "Ki%" ) ))
( "landsmen"  n ( l a1 n d z m @ n ) ((pos "Kj%" ) ))
( "landward"  a ( l a1 n d w @ d ) ((pos "Pu%" ) ))
( "lane"  n ( l ei1 n ) ((pos "K6%" ) ))
( "lanes"  n ( l ei1 n z ) ((pos "Kj%" ) ))
( "langsyne"  n ( l a2 ng s ai1 n ) ((pos "L@$" "Pu$" ) ))
( "language"  n ( l a1 ng g w i jh ) ((pos "M6%" ) ))
( "languages"  n ( l a1 ng g w i jh i z ) ((pos "Mj%" ) ))
( "languid"  j ( l a1 ng g w i d ) ((pos "OA%" ) ))
( "languidly"  a ( l a1 ng g w i d l ii ) ((pos "Pu%" ) ))
( "languish"  v ( l a1 ng g w i sh ) ((pos "I1%" ) (subcat "2A" )))
( "languished"  v ( l a1 ng g w i sh t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "languishes"  v ( l a1 ng g w i sh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "languishing"  v ( l a1 ng g w i sh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "languor"  n ( l a1 ng g @ r ) ((pos "L@$" ) ))
( "languorous"  j ( l a1 n g @ r @ s ) ((pos "OA$" ) ))
( "languorously"  a ( l a1 ng g @ r @ s l ii ) ((pos "Pu$" ) ))
( "langur"  n ( l uh1 ng g u@1 r ) ((pos "K6$" ) ))
( "langurs"  n ( l uh1 ng g u@1 z ) ((pos "Kj$" ) ))
( "lank"  j ( l a1 ng k ) ((pos "OA%" ) ))
( "lankier"  j ( l a1 ng k i@ r ) ((pos "Or%" ) ))
( "lankiest"  j ( l a1 ng k i i s t ) ((pos "Os%" ) ))
( "lanky"  j ( l a1 ng k ii ) ((pos "OD%" ) ))
( "lanolin"  n ( l a1 n @ l i n ) ((pos "L@%" ) ))
( "lantern"  n ( l a1 n t @ n ) ((pos "K6%" ) ))
( "lantern-jawed"  j ( l a1 n t @ n - jh oo d ) ((pos "OA%" ) ))
( "lanterns"  n ( l a1 n t @ n z ) ((pos "Kj%" ) ))
( "lanyard"  n ( l a1 n y @ d ) ((pos "K6$" ) ))
( "lanyards"  n ( l a1 n y @ d z ) ((pos "Kj$" ) ))
( "lap"  v ( l a1 p ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "lap-dog"  n ( l a1 p - d o g ) ((pos "K6%" ) ))
( "lap-dogs"  n ( l a1 p - d o g z ) ((pos "Kj%" ) ))
( "lapel"  n ( l @1 p e1 l ) ((pos "K6%" ) ))
( "lapels"  n ( l @1 p e1 l z ) ((pos "Kj%" ) ))
( "lapidaries"  n ( l a1 p i d @ r i z ) ((pos "Kj$" ) ))
( "lapidary"  n ( l a1 p i d @ r ii ) ((pos "K8$" "OA$" ) ))
( "lapis lazuli"  n ( l a2 p i s - l a1 z y u l ii ) ((pos "M6$" ) ))
( "lapis lazulis"  n ( l a2 p i s - l a1 z y u l i z ) ((pos "Mj$" ) ))
( "lapped"  v ( l a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "lapping"  v ( l a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "laps"  v ( l a1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "lapse"  v ( l a1 p s ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "lapsed"  v ( l a1 p s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "lapses"  v ( l a1 p s i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "lapsing"  v ( l a1 p s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "lapwing"  n ( l a1 p w i ng ) ((pos "K6%" ) ))
( "lapwings"  n ( l a1 p w i ng z ) ((pos "Kj%" ) ))
( "larboard"  n ( l aa1 b @ d ) ((pos "L@$" "OA$" ) ))
( "larcenies"  n ( l aa1 s @ n i z ) ((pos "Mj$" ) ))
( "larceny"  n ( l aa1 s @ n ii ) ((pos "M8%" ) ))
( "larch"  n ( l aa1 ch ) ((pos "M7%" ) ))
( "larches"  n ( l aa1 ch i z ) ((pos "Mj%" ) ))
( "lard"  v ( l aa1 d ) ((pos "H0%" "L@%" ) (subcat )))
( "larded"  v ( l aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "larder"  n ( l aa1 d @ r ) ((pos "K6%" ) ))
( "larders"  n ( l aa1 d @ z ) ((pos "Kj%" ) ))
( "larding"  v ( l aa1 d i ng ) ((pos "Hb%" ) (subcat )))
( "lards"  v ( l aa1 d z ) ((pos "Ha%" ) (subcat )))
( "large"  n ( l aa1 jh ) ((pos "L@%" "OB*" "Pu*" ) ))
( "large-scale"  j ( l aa1 jh - s k ei l ) ((pos "OA%" ) ))
( "largely"  a ( l aa1 jh l ii ) ((pos "Pu%" ) ))
( "largeness"  n ( l aa1 jh n @ s ) ((pos "L@%" ) ))
( "larger"  j ( l aa1 jh @ r ) ((pos "Or%" ) ))
( "largesse"  n ( l aa1 jh e1 s ) ((pos "L@%" ) ))
( "largest"  j ( l aa1 jh i s t ) ((pos "Os%" ) ))
( "largish"  j ( l aa1 jh i sh ) ((pos "OA$" ) ))
( "largo"  n ( l aa1 g ou ) ((pos "K6$" ) ))
( "largos"  n ( l aa1 g ou z ) ((pos "Kj$" ) ))
( "lariat"  n ( l a1 r i@ t ) ((pos "K6$" ) ))
( "lariats"  n ( l a1 r i@ t s ) ((pos "Kj$" ) ))
( "lark"  v ( l aa1 k ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "larked"  v ( l aa1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "larking"  v ( l aa1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "larks"  v ( l aa1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "larkspur"  n ( l aa1 k s p @@ r ) ((pos "K6%" ) ))
( "larkspurs"  n ( l aa1 k s p @@ z ) ((pos "Kj%" ) ))
( "larn"  v ( l aa1 n ) ((pos "J0$" ) ))
( "larned"  v ( l aa1 n d ) ((pos "Jc$" "Jd$" ) ))
( "larning"  v ( l aa1 n i ng ) ((pos "Jb$" ) ))
( "larns"  v ( l aa1 n z ) ((pos "Ja$" ) ))
( "larva"  n ( l aa1 v @ ) ((pos "Ki%" ) ))
( "larvae"  n ( l aa1 v ii ) ((pos "Kj%" ) ))
( "larval"  j ( l aa1 v @ l ) ((pos "OA%" ) ))
( "laryngitis"  n ( l a2 r i n jh ai1 t i s ) ((pos "L@%" ) ))
( "larynx"  n ( l a1 r i ng k s ) ((pos "K7%" ) ))
( "larynxes"  n ( l a1 r i ng k s i z ) ((pos "Kj%" ) ))
( "lascar"  n ( l a1 s k @ r ) ((pos "K6$" ) ))
( "lascars"  n ( l a1 s k @ z ) ((pos "Kj$" ) ))
( "lascivious"  j ( l @1 s i1 v i@ s ) ((pos "OA%" ) ))
( "lasciviously"  a ( l @1 s i1 v i@ s l ii ) ((pos "Pu%" ) ))
( "lasciviousness"  n ( l @1 s i1 v i@ s n @ s ) ((pos "L@%" ) ))
( "laser"  n ( l ei1 z @ r ) ((pos "K6%" ) ))
( "lasers"  n ( l ei1 z @ z ) ((pos "Kj%" ) ))
( "lash"  v ( l a1 sh ) ((pos "J1%" "K7%" ) (subcat "2C" "6A" "14" "15A" )))
( "lash-up"  n ( l a1 sh - uh p ) ((pos "K6$" ) ))
( "lash-ups"  n ( l a1 sh - uh p s ) ((pos "Kj$" ) ))
( "lashed"  v ( l a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "lashes"  v ( l a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "14" "15A" )))
( "lashing"  v ( l a1 sh i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "14" "15A" )))
( "lashings"  n ( l a1 sh i ng z ) ((pos "Kj%" ) ))
( "lass"  n ( l a1 s ) ((pos "K7%" ) ))
( "lasses"  n ( l a1 s i z ) ((pos "Kj%" ) ))
( "lassie"  n ( l a1 s ii ) ((pos "K6%" ) ))
( "lassies"  n ( l a1 s i z ) ((pos "Kj%" ) ))
( "lassitude"  n ( l a1 s i t y uu d ) ((pos "L@%" ) ))
( "lasso"  v ( l a1 s uu1 ) ((pos "H0%" "K6%" ) ))
( "lassoed"  v ( l a1 s uu1 d ) ((pos "Hc%" "Hd%" ) ))
( "lassoes"  n ( l a1 s uu1 z ) ((pos "Kj%" ) ))
( "lassoing"  v ( l a1 s uu1 i ng ) ((pos "Hb%" ) ))
( "lassos"  v ( l a1 s uu1 z ) ((pos "Ha%" "Kj%" ) ))
( "last"  v ( l aa1 s t ) ((pos "I0%" "K6%" "OA*" "Pu*" ) (subcat "2A" "2B" )))
( "lasted"  v ( l aa1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "lasting"  v ( l aa1 s t i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" "2B" )))
( "lastly"  a ( l aa1 s t l ii ) ((pos "Pu%" ) ))
( "lasts"  v ( l aa1 s t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" )))
( "lat"  n ( l a1 t ) ((pos "Y>%" ) ))
( "latch"  v ( l a1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" )))
( "latched"  v ( l a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "latches"  v ( l a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "latching"  v ( l a1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "latchkey"  n ( l a1 ch k ii ) ((pos "K6%" ) ))
( "latchkeys"  n ( l a1 ch k ii z ) ((pos "Kj%" ) ))
( "late"  j ( l ei1 t ) ((pos "OB%" "Pu%" ) ))
( "lateen"  j ( l @1 t ii1 n ) ((pos "OA$" ) ))
( "lately"  a ( l ei1 t l ii ) ((pos "Pu%" ) ))
( "latent"  j ( l ei1 t n t ) ((pos "OA%" ) ))
( "later"  j ( l ei1 t @ r ) ((pos "Or*" ) ))
( "lateral"  j ( l a1 t @ r @ l ) ((pos "OA%" ) ))
( "laterally"  a ( l a1 t r @ l ii ) ((pos "Pu%" ) ))
( "laterite"  n ( l a1 t @ r ai t ) ((pos "L@$" ) ))
( "latest"  j ( l ei1 t i s t ) ((pos "Os%" ) ))
( "latex"  n ( l ei1 t e k s ) ((pos "L@%" ) ))
( "lath"  n ( l aa1 th ) ((pos "K6$" ) ))
( "lathe"  n ( l ei1 dh ) ((pos "K6%" ) ))
( "lather"  v ( l aa1 dh @ r ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "lathered"  v ( l aa1 dh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "lathering"  v ( l aa1 dh @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "lathers"  v ( l aa1 dh @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "lathes"  n ( l ei1 dh z ) ((pos "Kj%" ) ))
( "lathi"  n ( l aa1 t ii ) ((pos "K6$" ) ))
( "lathis"  n ( l aa1 t i z ) ((pos "Kj$" ) ))
( "laths"  n ( l aa1 dh z ) ((pos "Kj%" ) ))
( "latish"  j ( l ei1 t i sh ) ((pos "OA%" ) ))
( "latitude"  n ( l a1 t i t y uu d ) ((pos "M6%" ) ))
( "latitudes"  n ( l a1 t i t y uu d z ) ((pos "M6%" ) ))
( "latitudinal"  j ( l a2 t i t y uu1 d i n l ) ((pos "OA%" ) ))
( "latitudinarian"  n ( l a2 t i t y uu d i n e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "latitudinarians"  n ( l a2 t i t y uu d i n e@1 r i@ n z ) ((pos "Kj$" ) ))
( "latrine"  n ( l @1 t r ii1 n ) ((pos "K6%" ) ))
( "latrines"  n ( l @1 t r ii1 n z ) ((pos "Kj%" ) ))
( "latter"  j ( l a1 t @ r ) ((pos "OA%" ) ))
( "latter-day"  j ( l a1 t @ - d ei ) ((pos "OA%" ) ))
( "latterly"  a ( l a1 t @ l ii ) ((pos "Pu%" ) ))
( "lattice"  n ( l a1 t i s ) ((pos "K6%" ) ))
( "latticed"  j ( l a1 t i s t ) ((pos "OA%" ) ))
( "lattices"  n ( l a1 t i s i z ) ((pos "Kj%" ) ))
( "laud"  v ( l oo1 d ) ((pos "H0$" ) (subcat )))
( "laudable"  j ( l oo1 d @ b l ) ((pos "OA%" ) ))
( "laudably"  a ( l oo1 d @ b l ii ) ((pos "Pu%" ) ))
( "laudanum"  n ( l oo1 d @ n @ m ) ((pos "L@%" ) ))
( "laudatory"  j ( l oo1 d @ t @ r ii ) ((pos "OA%" ) ))
( "lauded"  v ( l oo1 d i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "lauding"  v ( l oo1 d i ng ) ((pos "Hb$" ) (subcat )))
( "lauds"  v ( l oo1 d z ) ((pos "Ha$" ) (subcat )))
( "laugh"  v ( l aa1 f ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6B" "15A" "15B" )))
( "laughable"  j ( l aa1 f @ b l ) ((pos "OA%" ) ))
( "laughably"  a ( l aa1 f @ b l ii ) ((pos "Pu%" ) ))
( "laughed"  v ( l aa1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6B" "15A" "15B" )))
( "laughing"  v ( l aa1 f i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6B" "15A" "15B" )))
( "laughing-gas"  n ( l aa1 f i ng - g a s ) ((pos "L@%" ) ))
( "laughing-stock"  n ( l aa1 f i ng - s t o k ) ((pos "K6%" ) ))
( "laughing-stocks"  n ( l aa1 f i ng - s t o k s ) ((pos "Kj%" ) ))
( "laughingly"  a ( l aa1 f i ng l ii ) ((pos "Pu%" ) ))
( "laughs"  v ( l aa1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "6B" "15A" "15B" )))
( "laughter"  n ( l aa1 f t @ r ) ((pos "L@%" ) ))
( "launch"  v ( l oo1 n ch ) ((pos "J1%" "K7%" ) (subcat "2C" "3A" "6A" )))
( "launched"  v ( l oo1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "launcher"  n ( l oo1 n ch @ r ) ((pos "K6%" ) ))
( "launchers"  n ( l oo1 n ch @ z ) ((pos "Kj%" ) ))
( "launches"  v ( l oo1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "launching"  v ( l oo1 n ch i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "launching-pad"  n ( l oo1 n ch i ng - p a d ) ((pos "K6%" ) ))
( "launching-pads"  n ( l oo1 n ch i ng - p a d z ) ((pos "Kj%" ) ))
( "launching-site"  n ( l oo1 n ch i ng - s ai t ) ((pos "K6%" ) ))
( "launching-sites"  n ( l oo1 n ch i ng - s ai t s ) ((pos "Kj%" ) ))
( "launder"  v ( l oo1 n d @ r ) ((pos "J0$" ) (subcat "2A" )))
( "laundered"  v ( l oo1 n d @ d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "launderette"  n ( l oo1 n d r e1 t ) ((pos "K6%" ) ))
( "launderettes"  n ( l oo1 n d r e1 t s ) ((pos "Kj%" ) ))
( "laundering"  v ( l oo1 n d @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "launders"  v ( l oo1 n d @ z ) ((pos "Ja$" ) (subcat "2A" )))
( "laundress"  n ( l oo1 n d r i s ) ((pos "K7$" ) ))
( "laundresses"  n ( l oo1 n d r i s i z ) ((pos "Kj$" ) ))
( "laundries"  n ( l oo1 n d r i z ) ((pos "Kj%" ) ))
( "laundry"  n ( l oo1 n d r ii ) ((pos "K8%" ) ))
( "laundryman"  n ( l oo1 n d r i m a n ) ((pos "Ki%" ) ))
( "laundrymen"  n ( l oo1 n d r i m e n ) ((pos "Kj%" ) ))
( "laundrywoman"  n ( l oo1 n d r i w u m @ n ) ((pos "Ki%" ) ))
( "laundrywomen"  n ( l oo1 n d r i w i m i n ) ((pos "Kj%" ) ))
( "laureate"  n ( l o1 r i@ t ) ((pos "K6%" "OA%" ) ))
( "laureates"  n ( l o1 r i@ t s ) ((pos "Kj%" ) ))
( "laurel"  n ( l o1 r @ l ) ((pos "K6%" ) ))
( "laurelled"  j ( l o1 r @ l d ) ((pos "OA$" ) ))
( "laurels"  n ( l o1 r @ l z ) ((pos "Kj%" ) ))
( "lav"  n ( l a1 v ) ((pos "K6%" ) ))
( "lava"  n ( l aa1 v @ ) ((pos "L@%" ) ))
( "lavatories"  n ( l a1 v @ t r i z ) ((pos "Kj%" ) ))
( "lavatory"  n ( l a1 v @ t r ii ) ((pos "K8%" ) ))
( "lave"  v ( l ei1 v ) ((pos "H2$" ) ))
( "laved"  v ( l ei1 v d ) ((pos "Hc$" "Hd$" ) ))
( "lavender"  n ( l a1 v @ n d @ r ) ((pos "L@%" ) ))
( "laves"  v ( l ei1 v z ) ((pos "Ha$" ) ))
( "laving"  v ( l ei1 v i ng ) ((pos "Hb$" ) ))
( "lavish"  v ( l a1 v i sh ) ((pos "H1%" "OA%" ) (subcat )))
( "lavished"  v ( l a1 v i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lavishes"  v ( l a1 v i sh i z ) ((pos "Ha%" ) (subcat )))
( "lavishing"  v ( l a1 v i sh i ng ) ((pos "Hb%" ) (subcat )))
( "lavishly"  a ( l a1 v i sh l ii ) ((pos "Pu%" ) ))
( "lavs"  n ( l a1 v z ) ((pos "Kj%" ) ))
( "law"  n ( l oo1 ) ((pos "M6%" ) ))
( "law-abiding"  j ( l oo1 - @ b ai d i ng ) ((pos "OA%" ) ))
( "law-officer"  n ( l oo1 - o f i s @ r ) ((pos "K6%" ) ))
( "law-officers"  n ( l oo1 - o f i s @ z ) ((pos "Kj%" ) ))
( "lawbreaker"  n ( l oo1 b r ei k @ r ) ((pos "K6%" ) ))
( "lawbreakers"  n ( l oo1 b r ei k @ z ) ((pos "Kj%" ) ))
( "lawful"  j ( l oo1 f @ l ) ((pos "OA%" ) ))
( "lawfully"  a ( l oo1 f @ l ii ) ((pos "Pu%" ) ))
( "lawgiver"  n ( l oo1 g i v @ r ) ((pos "K6%" ) ))
( "lawgivers"  n ( l oo1 g i v @ z ) ((pos "Kj%" ) ))
( "lawless"  j ( l oo1 l @ s ) ((pos "OA%" ) ))
( "lawlessly"  a ( l oo1 l @ s l ii ) ((pos "Pu%" ) ))
( "lawlessness"  n ( l oo1 l @ s n @ s ) ((pos "L@%" ) ))
( "lawmaker"  n ( l oo1 m ei k @ r ) ((pos "K6%" ) ))
( "lawmakers"  n ( l oo1 m ei k @ z ) ((pos "Kj%" ) ))
( "lawmaking"  n ( l oo1 m ei k i ng ) ((pos "L@%" ) ))
( "lawn"  n ( l oo1 n ) ((pos "M6%" ) ))
( "lawn-mower"  n ( l oo1 n - m ou @ r ) ((pos "K6%" ) ))
( "lawn-mowers"  n ( l oo1 n - m ou @ z ) ((pos "Kj%" ) ))
( "lawns"  n ( l oo1 n z ) ((pos "Mj%" ) ))
( "laws"  n ( l oo1 z ) ((pos "Mj%" ) ))
( "lawyer"  n ( l oo1 y @ r ) ((pos "K6%" ) ))
( "lawyers"  n ( l oo1 y @ z ) ((pos "Kj%" ) ))
( "lax"  j ( l a1 k s ) ((pos "OA%" ) ))
( "laxative"  n ( l a1 k s @ t i v ) ((pos "M6%" "OA%" ) ))
( "laxatives"  n ( l a1 k s @ t i v z ) ((pos "Mj%" ) ))
( "laxities"  n ( l a1 k s i t i z ) ((pos "Mj$" ) ))
( "laxity"  n ( l a1 k s i t ii ) ((pos "M8%" ) ))
( "laxly"  a ( l a1 k s l ii ) ((pos "Pu$" ) ))
( "lay"  v ( l ei1 ) ((pos "Ic%" "J5%" "K6%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "12C" "14" "15A" "15B" )))
( "lay figure"  n ( l ei2 - f i1 g @ r ) ((pos "K6$" ) ))
( "lay figures"  n ( l ei2 - f i1 g @ z ) ((pos "Kj$" ) ))
( "lay-off"  n ( l ei1 - o f ) ((pos "K6%" ) ))
( "lay-offs"  n ( l ei1 - o f s ) ((pos "Kj$" ) ))
( "lay-out"  n ( l ei1 - au t ) ((pos "K6%" ) ))
( "lay-outs"  n ( l ei1 - au t s ) ((pos "Kj$" ) ))
( "layabout"  n ( l ei1 @ b au t ) ((pos "K6%" ) ))
( "layabouts"  n ( l ei1 @ b au t s ) ((pos "Kj%" ) ))
( "layby"  n ( l ei1 b ai ) ((pos "K6%" ) ))
( "laybys"  n ( l ei1 b ai z ) ((pos "Kj%" ) ))
( "layer"  v ( l ei1 @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "layer-cake"  n ( l ei1 @ - k ei k ) ((pos "K6%" ) ))
( "layer-cakes"  n ( l ei1 @ - k ei k s ) ((pos "Kj%" ) ))
( "layered"  v ( l ei1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "layering"  v ( l ei1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "layers"  v ( l ei1 @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "layette"  n ( l ei1 e1 t ) ((pos "K6$" ) ))
( "layettes"  n ( l ei1 e1 t s ) ((pos "Kj$" ) ))
( "laying"  v ( l ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" "12C" "14" "15A" "15B" )))
( "layman"  n ( l ei1 m @ n ) ((pos "Ki%" ) ))
( "laymen"  n ( l ei1 m @ n ) ((pos "Kj%" ) ))
( "layover"  n ( l ei1 ou v @ r ) ((pos "K6$" ) ))
( "layovers"  n ( l ei1 ou v @ z ) ((pos "Kj$" ) ))
( "lays"  v ( l ei1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "12C" "14" "15A" "15B" )))
( "lazar"  n ( l a1 z @ r ) ((pos "K6$" ) ))
( "lazaret"  n ( l a2 z @ r e1 t ) ((pos "K6$" ) ))
( "lazarets"  n ( l a2 z @ r e1 t s ) ((pos "Kj$" ) ))
( "lazarette"  n ( l a2 z @ r e1 t ) ((pos "K6$" ) ))
( "lazarettes"  n ( l a2 z @ r e1 t s ) ((pos "Kj$" ) ))
( "lazaretto"  n ( l a2 z @ r e1 t ou ) ((pos "K6$" ) ))
( "lazarettos"  n ( l a2 z @ r e1 t ou z ) ((pos "Kj$" ) ))
( "lazars"  n ( l a1 z @ z ) ((pos "Kj$" ) ))
( "laze"  v ( l ei1 z ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "lazed"  v ( l ei1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "lazes"  v ( l ei1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "lazier"  j ( l ei1 z i@ r ) ((pos "Or%" ) ))
( "laziest"  j ( l ei1 z i i s t ) ((pos "Os%" ) ))
( "lazily"  a ( l ei1 z i l ii ) ((pos "Pu%" ) ))
( "laziness"  n ( l ei1 z i n @ s ) ((pos "L@%" ) ))
( "lazing"  v ( l ei1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "lazy"  j ( l ei1 z ii ) ((pos "OD%" ) ))
( "lazybones"  n ( l ei1 z i b ou n z ) ((pos "Ki%" ) ))
( "lb"  n ( p au1 n d ) ((pos "Y]%" ) ))
( "lbs"  n ( p au1 n d z ) ((pos "Y)%" ) ))
( "lbw"  n ( e2 l b ii2 d uh1 b l y uu ) ((pos "Y~%" ) ))
( "lea"  n ( l ii1 ) ((pos "K6$" ) ))
( "leach"  v ( l ii1 ch ) ((pos "H1$" ) (subcat "6A" )))
( "leached"  v ( l ii1 ch t ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "leaches"  v ( l ii1 ch i z ) ((pos "Ha$" ) (subcat "6A" )))
( "leaching"  v ( l ii1 ch i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "lead"  n ( l e1 d ) ((pos "M6%" ) ))
( "lead"  v ( l ii1 d ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "lead-in"  n ( l ii1 d - i n ) ((pos "K6%" ) ))
( "lead-ins"  n ( l ii1 d - i n z ) ((pos "Kj%" ) ))
( "lead-ore"  n ( l e1 d - oo r ) ((pos "L@%" ) ))
( "leaded"  j ( l e1 d i d ) ((pos "OA%" ) ))
( "leaden"  j ( l e1 d n ) ((pos "OA%" ) ))
( "leader"  n ( l ii1 d @ r ) ((pos "K6%" ) ))
( "leaderless"  j ( l ii1 d @ l @ s ) ((pos "OA%" ) ))
( "leaders"  n ( l ii1 d @ z ) ((pos "Kj%" ) ))
( "leadership"  n ( l ii1 d @ sh i p ) ((pos "L@%" ) ))
( "leading"  n ( l e1 d i ng ) ((pos "L@%" ) ))
( "leading"  v ( l ii1 d i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "leading-rein"  n ( l ii1 d i ng - r ei n ) ((pos "K6%" ) ))
( "leading-reins"  n ( l ii1 d i ng - r ei n z ) ((pos "Kj%" ) ))
( "leading-strings"  n ( l ii1 d i ng - s t r i ng z ) ((pos "Kj$" ) ))
( "leads"  n ( l e1 d z ) ((pos "Mj$" ) ))
( "leads"  v ( l ii1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "leaf"  v ( l ii1 f ) ((pos "I0%" "Ki%" ) ))
( "leaf-bud"  n ( l ii1 f - b uh d ) ((pos "K6$" ) ))
( "leaf-buds"  n ( l ii1 f - b uh d z ) ((pos "Kj$" ) ))
( "leaf-mould"  n ( l ii1 f - m ou l d ) ((pos "L@%" ) ))
( "leafed"  v ( l ii1 f t ) ((pos "Ic%" "Id%" ) ))
( "leafier"  j ( l ii1 f i@ r ) ((pos "Or%" ) ))
( "leafiest"  j ( l ii1 f i i s t ) ((pos "Os%" ) ))
( "leafing"  v ( l ii1 f i ng ) ((pos "Ib%" ) ))
( "leafless"  j ( l ii1 f l @ s ) ((pos "OA%" ) ))
( "leaflet"  n ( l ii1 f l i t ) ((pos "K6%" ) ))
( "leaflets"  n ( l ii1 f l i t s ) ((pos "Kj%" ) ))
( "leafs"  v ( l ii1 f s ) ((pos "Ia$" ) ))
( "leafy"  j ( l ii1 f ii ) ((pos "OD%" ) ))
( "league"  v ( l ii1 g ) ((pos "J2$" "K6%" ) (subcat "2C" "6A" "15A" )))
( "leagued"  v ( l ii1 g d ) ((pos "Jc$" "Jd$" ) (subcat "2C" "6A" "15A" )))
( "leagues"  v ( l ii1 g z ) ((pos "Ja$" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "leaguing"  v ( l ii1 g i ng ) ((pos "Jb$" ) (subcat "2C" "6A" "15A" )))
( "leak"  v ( l ii1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "leakage"  n ( l ii1 k i jh ) ((pos "M6%" ) ))
( "leakages"  n ( l ii1 k i jh i z ) ((pos "Mj%" ) ))
( "leaked"  v ( l ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "leakier"  j ( l ii1 k i@ r ) ((pos "Or%" ) ))
( "leakiest"  j ( l ii1 k i i s t ) ((pos "Os%" ) ))
( "leaking"  v ( l ii1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "leaks"  v ( l ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "leaky"  j ( l ii1 k ii ) ((pos "OD%" ) ))
( "leal"  j ( l ii1 l ) ((pos "OA$" ) ))
( "lean"  v ( l ii1 n ) ((pos "J0%" "L@%" "OC%" ) (subcat "2A" "2C" "3A" )))
( "lean-to"  n ( l ii1 n - t uu ) ((pos "K6%" ) ))
( "lean-tos"  n ( l ii1 n - t uu z ) ((pos "Kj$" ) ))
( "leaned"  v ( l ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "leaner"  j ( l ii1 n @ r ) ((pos "Or%" ) ))
( "leanest"  j ( l ii1 n i s t ) ((pos "Os%" ) ))
( "leaning"  v ( l ii1 n i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "leanings"  n ( l ii1 n i ng z ) ((pos "Kj%" ) ))
( "leanness"  n ( l ii1 n - n @ s ) ((pos "L@%" ) ))
( "leans"  v ( l ii1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "leant"  v ( l e1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "leap"  v ( l ii1 p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "leap-year"  n ( l ii1 p - y @@ r ) ((pos "K6%" ) ))
( "leap-years"  n ( l ii1 p - y @@ z ) ((pos "Kj%" ) ))
( "leaped"  v ( l ii1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "leapfrog"  v ( l ii1 p f r o g ) ((pos "H4%" "L@%" ) ))
( "leapfrogged"  v ( l ii1 p f r o g d ) ((pos "Hc%" "Hd%" ) ))
( "leapfrogging"  v ( l ii1 p f r o g i ng ) ((pos "Hb%" ) ))
( "leapfrogs"  v ( l ii1 p f r o g z ) ((pos "Ha%" ) ))
( "leaping"  v ( l ii1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "leaps"  v ( l ii1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "leapt"  v ( l e1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "learn"  v ( l @@1 n ) ((pos "J0%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "15A" )))
( "learned"  j ( l @@1 n i d ) ((pos "OA%" ) ))
( "learned"  v ( l @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "15A" )))
( "learnedly"  a ( l @@1 n i d l ii ) ((pos "Pu%" ) ))
( "learner"  n ( l @@1 n @ r ) ((pos "K6%" ) ))
( "learners"  n ( l @@1 n @ z ) ((pos "Kj%" ) ))
( "learning"  v ( l @@1 n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "15A" )))
( "learns"  v ( l @@1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "15A" )))
( "learnt"  v ( l @@1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "7A" "8" "9" "10" "15A" )))
( "leas"  n ( l ii1 z ) ((pos "Kj$" ) ))
( "lease"  v ( l ii1 s ) ((pos "H2%" "K6%" ) (subcat )))
( "lease-lend"  n ( l ii1 s - l e n d ) ((pos "L@$" ) ))
( "leased"  v ( l ii1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "leasehold"  n ( l ii1 s h ou l d ) ((pos "K6%" "OA%" ) ))
( "leaseholder"  n ( l ii1 s h ou l d @ r ) ((pos "K6%" ) ))
( "leaseholders"  n ( l ii1 s h ou l d @ z ) ((pos "Kj%" ) ))
( "leaseholds"  n ( l ii1 s h ou l d z ) ((pos "Kj%" ) ))
( "leases"  v ( l ii1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "leash"  n ( l ii1 sh ) ((pos "K7%" ) ))
( "leashes"  n ( l ii1 sh i z ) ((pos "Kj$" ) ))
( "leasing"  v ( l ii1 s i ng ) ((pos "Hb%" ) (subcat )))
( "least"  n ( l ii1 s t ) ((pos "Ki%" "OA*" "Pu*" ) ))
( "leastways"  a ( l ii1 s t w ei z ) ((pos "Pu%" ) ))
( "leastwise"  a ( l ii1 s t w ai z ) ((pos "Pu%" ) ))
( "leather"  n ( l e1 dh @ r ) ((pos "M6%" ) ))
( "leather-jacket"  n ( l e1 dh @ - jh a k i t ) ((pos "K6%" ) ))
( "leather-jackets"  n ( l e1 dh @ - jh a k i t s ) ((pos "Kj%" ) ))
( "leatherette"  n ( l e2 dh @ r e1 t ) ((pos "L@$" ) ))
( "leatherneck"  n ( l e1 dh @ n e k ) ((pos "K6$" ) ))
( "leathernecks"  n ( l e1 dh @ n e k s ) ((pos "Kj$" ) ))
( "leathers"  n ( l e1 dh @ z ) ((pos "Mj%" ) ))
( "leatherwork"  n ( l e1 dh @ w @@ k ) ((pos "L@%" ) ))
( "leathery"  j ( l e1 dh @ r ii ) ((pos "OA%" ) ))
( "leave"  v ( l ii1 v ) ((pos "J5*" "M6%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" "15A" "15B" "16A" "19B" "22" "24B" )))
( "leave-taking"  n ( l ii1 v - t ei k i ng ) ((pos "K6%" ) ))
( "leave-takings"  n ( l ii1 v - t ei k i ng z ) ((pos "Kj%" ) ))
( "leaven"  v ( l e1 v n ) ((pos "H0$" "L@$" ) (subcat )))
( "leavened"  v ( l e1 v n d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "leavening"  v ( l e1 v n i ng ) ((pos "Hb$" ) (subcat )))
( "leavens"  v ( l e1 v n z ) ((pos "Ha$" ) (subcat )))
( "leaves"  v ( l ii1 v z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" "15A" "15B" "16A" "19B" "22" "24B" )))
( "leaving"  v ( l ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" "15A" "15B" "16A" "19B" "22" "24B" )))
( "leavings"  n ( l ii1 v i ng z ) ((pos "Kj%" ) ))
( "lecher"  n ( l e1 ch @ r ) ((pos "K6%" ) ))
( "lecheries"  n ( l e1 ch @ r i z ) ((pos "Mj$" ) ))
( "lecherous"  j ( l e1 ch @ r @ s ) ((pos "OA%" ) ))
( "lechers"  n ( l e1 ch @ z ) ((pos "Kj%" ) ))
( "lechery"  n ( l e1 ch @ r ii ) ((pos "M8%" ) ))
( "lectern"  n ( l e1 k t @ n ) ((pos "K6%" ) ))
( "lecterns"  n ( l e1 k t @ n z ) ((pos "Kj%" ) ))
( "lecture"  v ( l e1 k ch @ r ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "lectured"  v ( l e1 k ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "lecturer"  n ( l e1 k ch @ r @ r ) ((pos "K6%" ) ))
( "lecturers"  n ( l e1 k ch @ r @ z ) ((pos "Kj%" ) ))
( "lectures"  v ( l e1 k ch @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "lectureship"  n ( l e1 k ch @ sh i p ) ((pos "K6%" ) ))
( "lectureships"  n ( l e1 k ch @ sh i p s ) ((pos "Kj%" ) ))
( "lecturing"  v ( l e1 k ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "led"  v ( l e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "ledge"  n ( l e1 jh ) ((pos "K6%" ) ))
( "ledger"  n ( l e1 jh @ r ) ((pos "K6%" ) ))
( "ledgers"  n ( l e1 jh @ z ) ((pos "Kj%" ) ))
( "ledges"  n ( l e1 jh i z ) ((pos "Kj%" ) ))
( "lee"  n ( l ii1 ) ((pos "L@$" ) ))
( "leech"  n ( l ii1 ch ) ((pos "K7%" ) ))
( "leeches"  n ( l ii1 ch i z ) ((pos "Kj%" ) ))
( "leek"  n ( l ii1 k ) ((pos "K6%" ) ))
( "leeks"  n ( l ii1 k s ) ((pos "Kj%" ) ))
( "leer"  v ( l i@1 r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "leered"  v ( l i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "leering"  v ( l i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "leers"  v ( l i@1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "lees"  n ( l ii1 z ) ((pos "Kj$" ) ))
( "leeward"  j ( l ii1 w @ d ) ((pos "OA%" "Pu%" ) ))
( "leeway"  n ( l ii1 w ei ) ((pos "L@%" ) ))
( "left"  v ( l e1 f t ) ((pos "Jc*" "Jd*" "L@%" "OA*" "Pu*" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" "15A" "15B" "16A" "19B" "22" "24B" )))
( "left-hand"  j ( l e1 f t - h a n d ) ((pos "OA%" ) ))
( "left-handed"  j ( l e1 f t - h a1 n d i d ) ((pos "OA%" ) ))
( "left-wing"  n ( l e1 f t - w i1 ng ) ((pos "K6%" ) ))
( "left-winger"  n ( l e1 f t - w i1 ng @ r ) ((pos "K6%" ) ))
( "left-wingers"  n ( l e1 f t - w i1 ng @ z ) ((pos "Kj%" ) ))
( "left-wings"  n ( l e1 f t - w i1 ng z ) ((pos "Kj$" ) ))
( "leftist"  n ( l e1 f t i s t ) ((pos "K6%" ) ))
( "leftists"  n ( l e1 f t i s t s ) ((pos "Kj%" ) ))
( "leg"  n ( l e1 g ) ((pos "M6%" ) ))
( "leg-pull"  n ( l e1 g - p u l ) ((pos "K6%" ) ))
( "leg-pulling"  n ( l e1 g - p u l i ng ) ((pos "L@%" ) ))
( "leg-pulls"  n ( l e1 g - p u l z ) ((pos "Kj%" ) ))
( "legacies"  n ( l e1 g @ s i z ) ((pos "Kj%" ) ))
( "legacy"  n ( l e1 g @ s ii ) ((pos "K8%" ) ))
( "legal"  j ( l ii1 g l ) ((pos "OA%" ) ))
( "legalism"  n ( l ii1 g @ l i z @ m ) ((pos "L@$" ) ))
( "legalistic"  j ( l ii2 g @ l i1 s t i k ) ((pos "OA%" ) ))
( "legality"  n ( l ii1 g a1 l i t ii ) ((pos "L@%" ) ))
( "legalization"  n ( l ii2 g @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "legalize"  v ( l ii1 g @ l ai z ) ((pos "H2%" ) (subcat )))
( "legalized"  v ( l ii1 g @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "legalizes"  v ( l ii1 g @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "legalizing"  v ( l ii1 g @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "legally"  a ( l ii1 g @ l ii ) ((pos "Pu%" ) ))
( "legate"  n ( l e1 g i t ) ((pos "K6$" ) ))
( "legatee"  n ( l e2 g @ t ii1 ) ((pos "K6$" ) ))
( "legatees"  n ( l e2 g @ t ii1 z ) ((pos "Kj$" ) ))
( "legates"  n ( l e1 g i t s ) ((pos "Kj$" ) ))
( "legation"  n ( l i1 g ei1 sh @ n ) ((pos "K6$" ) ))
( "legations"  n ( l i1 g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "legato"  j ( l i1 g aa1 t ou ) ((pos "OA$" "Pu$" ) ))
( "legend"  n ( l e1 jh @ n d ) ((pos "M6%" ) ))
( "legendary"  j ( l e1 jh @ n d r ii ) ((pos "OA%" ) ))
( "legends"  n ( l e1 jh @ n d z ) ((pos "Mj%" ) ))
( "leger line"  n ( l e1 jh @ - l ai n ) ((pos "K6$" ) ))
( "leger lines"  n ( l e1 jh @ - l ai n z ) ((pos "Kj$" ) ))
( "legerdemain"  n ( l e2 jh @ d @ m ei1 n ) ((pos "L@$" ) ))
( "legged"  j ( l e1 g i d ) ((pos "Ot%" ) ))
( "legging"  n ( l e1 g i ng ) ((pos "K6$" ) ))
( "leggings"  n ( l e1 g i ng z ) ((pos "Kj%" ) ))
( "leggy"  j ( l e1 g ii ) ((pos "OA%" ) ))
( "leghorn"  n ( l e1 g oo1 n ) ((pos "K6$" ) ))
( "leghorns"  n ( l e1 g oo1 n z ) ((pos "Kj$" ) ))
( "legibility"  n ( l e2 jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "legible"  j ( l e1 jh @ b l ) ((pos "OA%" ) ))
( "legibly"  a ( l e1 jh @ b l ii ) ((pos "Pu%" ) ))
( "legion"  n ( l ii1 jh @ n ) ((pos "K6%" ) ))
( "legionaries"  n ( l i1 jh @ n @ r i z ) ((pos "Kj%" ) ))
( "legionary"  n ( l i1 jh @ n @ r ii ) ((pos "K8%" ) ))
( "legions"  n ( l ii1 jh @ n z ) ((pos "Kj%" ) ))
( "legislate"  v ( l e1 jh i s l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "legislated"  v ( l e1 jh i s l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "legislates"  v ( l e1 jh i s l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "legislating"  v ( l e1 jh i s l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "legislation"  n ( l e2 jh i s l ei1 sh @ n ) ((pos "L@%" ) ))
( "legislative"  j ( l e1 jh i s l @ t i v ) ((pos "OA%" ) ))
( "legislator"  n ( l e1 jh i s l ei t @ r ) ((pos "K6%" ) ))
( "legislators"  n ( l e1 jh i s l ei t @ z ) ((pos "Kj%" ) ))
( "legislature"  n ( l e1 jh i s l ei ch @ r ) ((pos "K6%" ) ))
( "legislatures"  n ( l e1 jh i s l ei ch @ z ) ((pos "Kj%" ) ))
( "legitimacy"  n ( l i1 jh i1 t i m @ s ii ) ((pos "L@%" ) ))
( "legitimate"  j ( l i1 jh i1 t i m @ t ) ((pos "OA%" ) ))
( "legitimately"  a ( l i1 jh i1 t i m @ t l ii ) ((pos "Pu%" ) ))
( "legitimatize"  v ( l i1 jh i1 t i m @ t ai z ) ((pos "H2%" ) ))
( "legitimatized"  v ( l i1 jh i1 t i m @ t ai z d ) ((pos "Hc%" "Hd%" ) ))
( "legitimatizes"  v ( l i1 jh i1 t i m @ t ai z i z ) ((pos "Ha%" ) ))
( "legitimatizing"  v ( l i1 jh i1 t i m @ t ai z i ng ) ((pos "Hb%" ) ))
( "legless"  j ( l e1 g l @ s ) ((pos "OA%" ) ))
( "legs"  n ( l e1 g z ) ((pos "Mj%" ) ))
( "leguminous"  j ( l i1 g y uu1 m i n @ s ) ((pos "OA%" ) ))
( "lei"  n ( l ei1 ii ) ((pos "K6$" ) ))
( "leis"  n ( l ei1 ii z ) ((pos "Kj$" ) ))
( "leisure"  n ( l e1 zh @ r ) ((pos "L@%" ) ))
( "leisured"  j ( l e1 zh @ d ) ((pos "OA%" ) ))
( "leisurely"  j ( l e1 zh @ l ii ) ((pos "OA%" "Pu%" ) ))
( "lemming"  n ( l e1 m i ng ) ((pos "K6%" ) ))
( "lemmings"  n ( l e1 m i ng z ) ((pos "Kj%" ) ))
( "lemon"  n ( l e1 m @ n ) ((pos "K6%" ) ))
( "lemonade"  n ( l e2 m @ n ei1 d ) ((pos "L@%" ) ))
( "lemons"  n ( l e1 m @ n z ) ((pos "Kj%" ) ))
( "lemur"  n ( l ii1 m @ r ) ((pos "K6$" ) ))
( "lemurs"  n ( l ii1 m @ z ) ((pos "Kj$" ) ))
( "lend"  v ( l e1 n d ) ((pos "H5%" ) (subcat "6A" "12A" "13A" )))
( "lender"  n ( l e1 n d @ r ) ((pos "K6%" ) ))
( "lenders"  n ( l e1 n d @ z ) ((pos "Kj%" ) ))
( "lending"  v ( l e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" )))
( "lending-libraries"  n ( l e1 n d i ng - l ai b r @ r i z ) ((pos "Kj%" ) ))
( "lending-library"  n ( l e1 n d i ng - l ai b r @ r ii ) ((pos "K8%" ) ))
( "lends"  v ( l e1 n d z ) ((pos "Ha%" ) (subcat "6A" "12A" "13A" )))
( "length"  n ( l e1 ng th ) ((pos "M6%" ) ))
( "lengthen"  v ( l e1 ng th @ n ) ((pos "J0%" ) (subcat "2A" )))
( "lengthened"  v ( l e1 ng th @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "lengthening"  v ( l e1 ng th @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "lengthens"  v ( l e1 ng th @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "lengthier"  j ( l e1 ng th i@ r ) ((pos "Or%" ) ))
( "lengthiest"  j ( l e1 ng th i i s t ) ((pos "Os%" ) ))
( "lengthily"  a ( l e1 ng th i l ii ) ((pos "Pu%" ) ))
( "lengths"  n ( l e1 ng th s ) ((pos "Mj%" ) ))
( "lengthways"  j ( l e1 ng th w ei z ) ((pos "OA%" "Pu%" ) ))
( "lengthwise"  a ( l e1 ng th w ai z ) ((pos "Pu%" ) ))
( "lengthy"  j ( l e1 ng th ii ) ((pos "OD%" ) ))
( "lenience"  n ( l ii1 n i@ n s ) ((pos "L@%" ) ))
( "leniency"  n ( l ii1 n i@ n s ii ) ((pos "L@%" ) ))
( "lenient"  j ( l ii1 n i@ n t ) ((pos "OA%" ) ))
( "leniently"  a ( l ii1 n i@ n t l ii ) ((pos "Pu%" ) ))
( "lenity"  n ( l e1 n i t ii ) ((pos "L@$" ) ))
( "lens"  n ( l e1 n z ) ((pos "K7%" ) ))
( "lenses"  n ( l e1 n z i z ) ((pos "Kj%" ) ))
( "lent"  v ( l e1 n t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" )))
( "lentil"  n ( l e1 n t l ) ((pos "K6%" ) ))
( "lentils"  n ( l e1 n t l z ) ((pos "Kj%" ) ))
( "lento"  j ( l e1 n t ou ) ((pos "OA$" "Pu$" ) ))
( "leonine"  j ( l i@1 n ai n ) ((pos "OA%" ) ))
( "leopard"  n ( l e1 p @ d ) ((pos "K6%" ) ))
( "leopardess"  n ( l e2 p @ d e1 s ) ((pos "K7%" ) ))
( "leopardesses"  n ( l e2 p @ d e1 s i z ) ((pos "Kj%" ) ))
( "leopards"  n ( l e1 p @ d z ) ((pos "Kj%" ) ))
( "leper"  n ( l e1 p @ r ) ((pos "K6%" ) ))
( "lepers"  n ( l e1 p @ z ) ((pos "Kj%" ) ))
( "leprechaun"  n ( l e1 p r @ k oo n ) ((pos "K6%" ) ))
( "leprechauns"  n ( l e1 p r @ k oo n z ) ((pos "Kj%" ) ))
( "leprosy"  n ( l e1 p r @ s ii ) ((pos "L@%" ) ))
( "leprous"  j ( l e1 p r @ s ) ((pos "OA%" ) ))
( "lesbian"  n ( l e1 z b i@ n ) ((pos "K6%" ) ))
( "lesbianism"  n ( l e1 z b i@ n i z @ m ) ((pos "L@%" ) ))
( "lesbians"  n ( l e1 z b i@ n z ) ((pos "Kj%" ) ))
( "lese majesty"  n ( l ei2 z - m a1 jh i s t ii ) ((pos "L@$" ) ))
( "lesion"  n ( l ii1 zh n ) ((pos "K6%" ) ))
( "lesions"  n ( l ii1 zh n z ) ((pos "Kj%" ) ))
( "less"  n ( l e s ) ((pos "L@%" "OA*" "Pu*" "T-*" ) ))
( "lessee"  n ( l e1 s ii1 ) ((pos "K6%" ) ))
( "lessees"  n ( l e1 s ii1 z ) ((pos "Kj%" ) ))
( "lessen"  v ( l e1 s n ) ((pos "J0%" ) (subcat "2A" )))
( "lessened"  v ( l e1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "lessening"  v ( l e1 s n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "lessens"  v ( l e1 s n z ) ((pos "Ja%" ) (subcat "2A" )))
( "lesser"  j ( l e1 s @ r ) ((pos "OA%" ) ))
( "lesson"  n ( l e1 s n ) ((pos "K6%" ) ))
( "lessons"  n ( l e1 s n z ) ((pos "Kj%" ) ))
( "lessor"  n ( l e1 s oo r ) ((pos "K6$" ) ))
( "lessors"  n ( l e1 s oo z ) ((pos "Kj$" ) ))
( "lest"  cc ( l e s t ) ((pos "V-%" ) ))
( "let"  v ( l e1 t ) ((pos "J5*" "Jc*" "Jd*" "K6%" ) (subcat "2C" "6A" "14" "15A" "15B" "18B" "22" )))
( "let's"  v ( l e t s ) ((pos "Gf%" ) ))
( "let-down"  n ( l e1 t - d au n ) ((pos "K6%" ) ))
( "let-downs"  n ( l e1 t - d au n z ) ((pos "Kj$" ) ))
( "let-up"  n ( l e1 t - uh p ) ((pos "K6%" ) ))
( "let-ups"  n ( l e1 t - uh p s ) ((pos "Kj$" ) ))
( "lethal"  j ( l ii1 th l ) ((pos "OA%" ) ))
( "lethargic"  j ( l i1 th aa1 jh i k ) ((pos "OA%" ) ))
( "lethargically"  a ( l i1 th aa1 jh i k l ii ) ((pos "Pu%" ) ))
( "lethargy"  n ( l e1 th @ jh ii ) ((pos "L@%" ) ))
( "lets"  v ( l e1 t s ) ((pos "Ja%" "Kj$" ) (subcat "2C" "6A" "14" "15A" "15B" "18B" "22" )))
( "letter"  n ( l e1 t @ r ) ((pos "K6%" ) ))
( "letter-box"  n ( l e1 t @ - b o k s ) ((pos "K7%" ) ))
( "letter-boxes"  n ( l e1 t @ - b o k s i z ) ((pos "Kj%" ) ))
( "letter-card"  n ( l e1 t @ - k aa d ) ((pos "K6$" ) ))
( "letter-cards"  n ( l e1 t @ - k aa d z ) ((pos "Kj$" ) ))
( "letter-case"  n ( l e1 t @ - k ei s ) ((pos "K6$" ) ))
( "letter-cases"  n ( l e1 t @ - k ei s i z ) ((pos "Kj$" ) ))
( "lettered"  j ( l e1 t @ d ) ((pos "OA%" ) ))
( "letterhead"  n ( l e1 t @ h e d ) ((pos "K6%" ) ))
( "letterheads"  n ( l e1 t @ h e d z ) ((pos "Kj%" ) ))
( "lettering"  n ( l e1 t @ r i ng ) ((pos "L@%" ) ))
( "letterpress"  n ( l e1 t @ p r e s ) ((pos "L@%" ) ))
( "letters"  n ( l e1 t @ z ) ((pos "Kj%" ) ))
( "letting"  v ( l e1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "14" "15A" "15B" "18B" "22" )))
( "lettings"  n ( l e1 t i ng z ) ((pos "Kj%" ) ))
( "lettuce"  n ( l e1 t i s ) ((pos "M6%" ) ))
( "lettuces"  n ( l e1 t i s i z ) ((pos "Mj%" ) ))
( "leucocyte"  n ( l uu1 k @ s ai t ) ((pos "K6$" ) ))
( "leucocytes"  n ( l uu1 k @ s ai t s ) ((pos "Kj$" ) ))
( "leukaemia"  n ( l uu1 k ii1 m i@ ) ((pos "L@%" ) ))
( "levant"  v ( l i1 v a1 n t ) ((pos "I0$" ) ))
( "levanted"  v ( l i1 v a1 n t i d ) ((pos "Ic$" "Id$" ) ))
( "levanting"  v ( l i1 v a1 n t i ng ) ((pos "Ib$" ) ))
( "levants"  v ( l i1 v a1 n t s ) ((pos "Ia$" ) ))
( "levee"  n ( l e1 v ii ) ((pos "K6$" ) ))
( "levees"  n ( l e1 v i z ) ((pos "Kj$" ) ))
( "level"  v ( l e1 v @ l ) ((pos "J4%" "M6%" "OA%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "level-headed"  j ( l e2 v l - h e1 d i d ) ((pos "OA%" ) ))
( "levelled"  v ( l e1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "leveller"  n ( l e1 v l @ r ) ((pos "K6%" ) ))
( "levellers"  n ( l e1 v l @ z ) ((pos "Kj%" ) ))
( "levelling"  v ( l e1 v @ l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "levels"  v ( l e1 v @ l z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "lever"  v ( l ii1 v @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "leverage"  n ( l ii1 v @ r i jh ) ((pos "L@%" ) ))
( "levered"  v ( l ii1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "leveret"  n ( l e1 v @ r i t ) ((pos "K6$" ) ))
( "leverets"  n ( l e1 v @ r i t s ) ((pos "Kj$" ) ))
( "levering"  v ( l ii1 v @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "levers"  v ( l ii1 v @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "leviathan"  n ( l i1 v ai1 @ th @ n ) ((pos "K6$" ) ))
( "leviathans"  n ( l i1 v ai1 @ th @ n z ) ((pos "Kj$" ) ))
( "levied"  v ( l e1 v i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "levies"  v ( l e1 v i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "levis"  n ( l ii1 v ai z ) ((pos "Kj%" ) ))
( "levitate"  v ( l e1 v i t ei t ) ((pos "J2%" ) (subcat "2A" )))
( "levitated"  v ( l e1 v i t ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "levitates"  v ( l e1 v i t ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "levitating"  v ( l e1 v i t ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "levitation"  n ( l e2 v i t ei1 sh @ n ) ((pos "M6%" ) ))
( "levitations"  n ( l e2 v i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "levities"  n ( l e1 v i t i z ) ((pos "Mj$" ) ))
( "levity"  n ( l e1 v i t ii ) ((pos "M8%" ) ))
( "levy"  v ( l e1 v ii ) ((pos "J3%" "K8%" ) (subcat "3A" "6A" )))
( "levying"  v ( l e1 v i i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "lewd"  j ( l y uu1 d ) ((pos "OC%" ) ))
( "lewder"  j ( l y uu1 d @ r ) ((pos "Or%" ) ))
( "lewdest"  j ( l y uu1 d i s t ) ((pos "Os%" ) ))
( "lewdly"  a ( l y uu1 d l ii ) ((pos "Pu%" ) ))
( "lewdness"  n ( l y uu1 d n @ s ) ((pos "L@%" ) ))
( "lexical"  j ( l e1 k s i k l ) ((pos "OA%" ) ))
( "lexically"  a ( l e1 k s i k l ii ) ((pos "Pu%" ) ))
( "lexicographer"  n ( l e2 k s i k o1 g r @ f @ r ) ((pos "K6%" ) ))
( "lexicographers"  n ( l e2 k s i k o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "lexicography"  n ( l e2 k s i k o1 g r @ f ii ) ((pos "L@%" ) ))
( "lexicon"  n ( l e1 k s i k @ n ) ((pos "K6%" ) ))
( "lexicons"  n ( l e1 k s i k @ n z ) ((pos "Kj%" ) ))
( "lexis"  n ( l e1 k s i s ) ((pos "L@$" ) ))
( "ley"  n ( l ei1 ) ((pos "K6$" ) ))
( "leys"  n ( l ei1 z ) ((pos "Kj$" ) ))
( "liabilities"  n ( l ai2 @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "liability"  n ( l ai2 @ b i1 l i t ii ) ((pos "M8%" ) ))
( "liable"  j ( l ai1 @ b l ) ((pos "OA%" ) ))
( "liaise"  v ( l i1 ei1 z ) ((pos "I2%" ) (subcat "2A" )))
( "liaised"  v ( l i1 ei1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "liaises"  v ( l i1 ei1 z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "liaising"  v ( l i1 ei1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "liaison"  n ( l i1 ei1 z n ) ((pos "M6%" ) ))
( "liaisons"  n ( l i1 ei1 z n z ) ((pos "Mj%" ) ))
( "liana"  n ( l i1 aa1 n @ ) ((pos "K6$" ) ))
( "lianas"  n ( l i1 aa1 n @ z ) ((pos "Kj$" ) ))
( "liar"  n ( l ai1 @ r ) ((pos "K6%" ) ))
( "liars"  n ( l ai1 @ z ) ((pos "Kj%" ) ))
( "lib"  n ( l i1 b ) ((pos "Ki%" ) ))
( "libation"  n ( l ai1 b ei1 sh @ n ) ((pos "K6%" ) ))
( "libations"  n ( l ai1 b ei1 sh @ n z ) ((pos "Kj%" ) ))
( "libel"  v ( l ai1 b l ) ((pos "H4%" "M6%" ) (subcat )))
( "libelled"  v ( l ai1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "libelling"  v ( l ai1 b @ l i ng ) ((pos "Hb%" ) (subcat )))
( "libellous"  j ( l ai1 b @ l @ s ) ((pos "OA%" ) ))
( "libels"  v ( l ai1 b l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "liberal"  n ( l i1 b @ r @ l ) ((pos "K6%" "OA%" ) ))
( "liberalism"  n ( l i1 b @ r @ l i z @ m ) ((pos "L@%" ) ))
( "liberalities"  n ( l i2 b @ r a1 l i t i z ) ((pos "Mj$" ) ))
( "liberality"  n ( l i2 b @ r a1 l i t ii ) ((pos "M8%" ) ))
( "liberalization"  n ( l i2 b r @ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "liberalizations"  n ( l i2 b r @ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "liberalize"  v ( l i1 b r @ l ai z ) ((pos "H2%" ) ))
( "liberalized"  v ( l i1 b r @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "liberalizes"  v ( l i1 b r @ l ai z i z ) ((pos "Ha%" ) ))
( "liberalizing"  v ( l i1 b r @ l ai z i ng ) ((pos "Hb%" ) ))
( "liberally"  a ( l i1 b r @ l ii ) ((pos "Pu%" ) ))
( "liberals"  n ( l i1 b @ r @ l z ) ((pos "Kj%" ) ))
( "liberate"  v ( l i1 b @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "liberated"  v ( l i1 b @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "liberates"  v ( l i1 b @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "liberating"  v ( l i1 b @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "liberation"  n ( l i2 b @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "liberations"  n ( l i2 b @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "liberator"  n ( l i1 b @ r ei t @ r ) ((pos "K6%" ) ))
( "liberators"  n ( l i1 b @ r ei t @ z ) ((pos "Kj%" ) ))
( "liberties"  n ( l i1 b i t i z ) ((pos "Mj%" ) ))
( "libertine"  n ( l i1 b @ t ii n ) ((pos "K6%" ) ))
( "libertines"  n ( l i1 b @ t ii n z ) ((pos "Kj%" ) ))
( "liberty"  n ( l i1 b i t ii ) ((pos "M8%" ) ))
( "libidinous"  j ( l i1 b i1 d i n @ s ) ((pos "OA%" ) ))
( "libido"  n ( l i1 b ii1 d ou ) ((pos "M6%" ) ))
( "libidos"  n ( l i1 b ii1 d ou z ) ((pos "Mj$" ) ))
( "librarian"  n ( l ai1 b r e@1 r i@ n ) ((pos "K6%" ) ))
( "librarians"  n ( l ai1 b r e@1 r i@ n z ) ((pos "Kj%" ) ))
( "librarianship"  n ( l ai1 b r e@1 r ii @ n sh i p ) ((pos "L@%" ) ))
( "libraries"  n ( l ai1 b r @ r i z ) ((pos "Kj%" ) ))
( "library"  n ( l ai1 b r @ r ii ) ((pos "K8%" ) ))
( "libretti"  n ( l i1 b r e1 t ii ) ((pos "Kj%" ) ))
( "librettist"  n ( l i1 b r e1 t i s t ) ((pos "K6%" ) ))
( "librettists"  n ( l i1 b r e1 t i s t s ) ((pos "Kj%" ) ))
( "libretto"  n ( l i1 b r e1 t ou ) ((pos "K6%" ) ))
( "librettos"  n ( l i1 b r e1 t ou z ) ((pos "Kj%" ) ))
( "lice"  n ( l ai1 s ) ((pos "Kj%" ) ))
( "licence"  v ( l ai1 s n s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "licenced"  v ( l ai1 s n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "licences"  v ( l ai1 s n s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "licencing"  v ( l ai1 s n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "license"  v ( l ai1 s n s ) ((pos "H2%" ) (subcat "6A" )))
( "licensed"  v ( l ai1 s n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "licensee"  n ( l ai2 s @ n s ii1 ) ((pos "K6%" ) ))
( "licensees"  n ( l ai2 s @ n s ii1 z ) ((pos "Kj%" ) ))
( "licenses"  v ( l ai1 s n s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "licensing"  v ( l ai1 s n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "licentiate"  n ( l ai1 s e1 n sh i@ t ) ((pos "K6$" ) ))
( "licentiates"  n ( l ai1 s e1 n sh i@ t s ) ((pos "Kj$" ) ))
( "licentious"  j ( l ai1 s e1 n sh @ s ) ((pos "OA%" ) ))
( "licentiously"  a ( l ai1 s e1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "licentiousness"  n ( l ai1 s e1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "lichee"  n ( l ai1 ch ii ) ((pos "K6%" ) ))
( "lichees"  n ( l ai1 ch ii z ) ((pos "Kj%" ) ))
( "lichen"  n ( l ai1 k @ n ) ((pos "L@%" ) ))
( "lichgate"  n ( l i1 ch g ei t ) ((pos "K6$" ) ))
( "lichgates"  n ( l i1 ch g ei t s ) ((pos "Kj$" ) ))
( "lichi"  n ( l ai1 ch ii ) ((pos "K6$" ) ))
( "lichis"  n ( l ai1 ch ii z ) ((pos "Kj$" ) ))
( "licit"  j ( l i1 s i t ) ((pos "OA$" ) ))
( "lick"  v ( l i1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "15A" "15B" )))
( "licked"  v ( l i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" "15B" )))
( "licking"  v ( l i1 k i ng ) ((pos "Jb%" "Ki%" ) (subcat "2A" "6A" "15A" "15B" )))
( "licks"  v ( l i1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" "15B" )))
( "lid"  n ( l i1 d ) ((pos "K6%" ) ))
( "lidless"  j ( l i1 d l @ s ) ((pos "OA%" ) ))
( "lido"  n ( l ii1 d ou ) ((pos "K6%" ) ))
( "lidos"  n ( l ii1 d ou z ) ((pos "Kj%" ) ))
( "lids"  n ( l i1 d z ) ((pos "Kj%" ) ))
( "lie"  v ( l ai1 ) ((pos "I5%" "K6%" ) (subcat "2A" "2C" "2D" )))
( "lie-abed"  n ( l ai1 - @ b e d ) ((pos "K6$" ) ))
( "lie-abeds"  n ( l ai1 - @ b e d z ) ((pos "Kj$" ) ))
( "lie-detector"  n ( l ai1 - d i t e k t @ r ) ((pos "K6%" ) ))
( "lie-detectors"  n ( l ai1 - d i t e k t @ z ) ((pos "Kj%" ) ))
( "lie-in"  n ( l ai1 - i1 n ) ((pos "K6%" ) ))
( "lie-ins"  n ( l ai1 - i1 n z ) ((pos "Kj$" ) ))
( "lied"  v ( l ai1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "lied"  n ( l ii1 t ) ((pos "Ki$" ) ))
( "lieder"  n ( l ii1 d @ r ) ((pos "Kj%" ) ))
( "lieder-singer"  n ( l ii1 d @ - s i ng @ r ) ((pos "K6%" ) ))
( "lieder-singers"  n ( l ii1 d @ - s i ng @ z ) ((pos "Kj%" ) ))
( "lief"  a ( l ii1 f ) ((pos "Pu$" ) ))
( "liege"  n ( l ii1 jh ) ((pos "K6$" "OA$" ) ))
( "liegeman"  n ( l ii1 jh m a n ) ((pos "Ki$" ) ))
( "liegemen"  n ( l ii1 jh m e n ) ((pos "Kj$" ) ))
( "lieges"  n ( l ii1 jh i z ) ((pos "Kj$" ) ))
( "lien"  n ( l i@1 n ) ((pos "K6$" ) ))
( "liens"  n ( l i@1 n z ) ((pos "Kj$" ) ))
( "lies"  v ( l ai1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" "2D" )))
( "lieu"  n ( l uu1 ) ((pos "L@%" ) ))
( "lieutenancies"  n ( l e1 f t e1 n @ n s i z ) ((pos "Kj$" ) ))
( "lieutenancy"  n ( l e1 f t e1 n @ n s ii ) ((pos "K8$" ) ))
( "lieutenant"  n ( l e1 f t e1 n @ n t ) ((pos "K6%" ) ))
( "lieutenants"  n ( l e1 f t e1 n @ n t s ) ((pos "Kj%" ) ))
( "life"  n ( l ai1 f ) ((pos "Mi*" ) ))
( "life-buoy"  n ( l ai1 f - b oi ) ((pos "K6%" ) ))
( "life-buoys"  n ( l ai1 f - b oi z ) ((pos "Kj%" ) ))
( "life-giving"  j ( l ai1 f - g i v i ng ) ((pos "OA%" ) ))
( "life-jacket"  n ( l ai1 f - jh a k i t ) ((pos "K6%" ) ))
( "life-jackets"  n ( l ai1 f - jh a k i t s ) ((pos "Kj%" ) ))
( "life-office"  n ( l ai1 f - o f i s ) ((pos "K6$" ) ))
( "life-offices"  n ( l ai1 f - o f i s i z ) ((pos "Kj$" ) ))
( "life-preserver"  n ( l ai1 f - p r i z @@ v @ r ) ((pos "K6%" ) ))
( "life-preservers"  n ( l ai1 f - p r i z @@ v @ z ) ((pos "Kj%" ) ))
( "life-raft"  n ( l ai1 f - r aa f t ) ((pos "K6%" ) ))
( "life-rafts"  n ( l ai1 f - r aa f t s ) ((pos "Kj%" ) ))
( "life-saver"  n ( l ai1 f - s ei v @ r ) ((pos "K6%" ) ))
( "life-savers"  n ( l ai1 f - s ei v @ z ) ((pos "Kj%" ) ))
( "life-size"  j ( l ai1 f - s ai z ) ((pos "OA%" ) ))
( "life-sized"  j ( l ai1 f - s ai z d ) ((pos "OA%" ) ))
( "life-span"  n ( l ai1 f - s p a n ) ((pos "K6%" ) ))
( "life-spans"  n ( l ai1 f - s p a n z ) ((pos "Kj%" ) ))
( "life-work"  n ( l ai1 f - w @@1 k ) ((pos "K6$" ) ))
( "life-works"  n ( l ai1 f - w @@1 k s ) ((pos "Kj$" ) ))
( "lifebelt"  n ( l ai1 f b e l t ) ((pos "K6%" ) ))
( "lifebelts"  n ( l ai1 f b e l t s ) ((pos "Kj%" ) ))
( "lifeblood"  n ( l ai1 f b l uh d ) ((pos "L@%" ) ))
( "lifeboat"  n ( l ai1 f b ou t ) ((pos "K6%" ) ))
( "lifeboats"  n ( l ai1 f b ou t s ) ((pos "Kj%" ) ))
( "lifebuoy"  n ( l ai1 f b oi ) ((pos "K6%" ) ))
( "lifebuoys"  n ( l ai1 f b oi z ) ((pos "Kj%" ) ))
( "lifeguard"  n ( l ai1 f g aa d ) ((pos "K6%" ) ))
( "lifeguards"  n ( l ai1 f g aa d z ) ((pos "Kj%" ) ))
( "lifeless"  j ( l ai1 f l @ s ) ((pos "OA%" ) ))
( "lifelessly"  a ( l ai1 f l @ s l ii ) ((pos "Pu%" ) ))
( "lifelike"  j ( l ai1 f l ai k ) ((pos "OA%" ) ))
( "lifeline"  n ( l ai1 f l ai n ) ((pos "K6%" ) ))
( "lifelines"  n ( l ai1 f l ai n z ) ((pos "Kj%" ) ))
( "lifelong"  j ( l ai1 f l o ng ) ((pos "OA%" ) ))
( "lifer"  n ( l ai1 f @ r ) ((pos "K6$" ) ))
( "lifers"  n ( l ai1 f @ z ) ((pos "Kj$" ) ))
( "lifetime"  n ( l ai1 f t ai m ) ((pos "K6%" ) ))
( "lifetimes"  n ( l ai1 f t ai m z ) ((pos "Kj%" ) ))
( "lift"  v ( l i1 f t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "lift-off"  n ( l i1 f t - o f ) ((pos "K6%" ) ))
( "lift-offs"  n ( l i1 f t - o f s ) ((pos "Kj%" ) ))
( "lifted"  v ( l i1 f t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "lifting"  v ( l i1 f t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "liftman"  n ( l i1 f t m a n ) ((pos "Ki%" ) ))
( "liftmen"  n ( l i1 f t m e n ) ((pos "Kj%" ) ))
( "lifts"  v ( l i1 f t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ligament"  n ( l i1 g @ m @ n t ) ((pos "K6%" ) ))
( "ligaments"  n ( l i1 g @ m @ n t s ) ((pos "Kj%" ) ))
( "ligature"  n ( l i1 g @ ch @ r ) ((pos "K6$" ) ))
( "ligatures"  n ( l i1 g @ ch @ z ) ((pos "Kj$" ) ))
( "light"  v ( l ai1 t ) ((pos "J0%" "M6*" "OC*" "Pu%" ) (subcat "2C" "3A" "6A" "15A" )))
( "light-armed"  j ( l ai1 t - aa1 m d ) ((pos "OA$" ) ))
( "light-coloured"  j ( l ai1 t - k uh1 l @ d ) ((pos "OA%" ) ))
( "light-fingered"  j ( l ai1 t - f i1 ng g @ d ) ((pos "OA%" ) ))
( "light-handed"  j ( l ai1 t - h a1 n d i d ) ((pos "OA$" ) ))
( "light-handedly"  a ( l ai1 t - h a1 n d i d l ii ) ((pos "Pu$" ) ))
( "light-headed"  j ( l ai1 t - h e1 d i d ) ((pos "OA%" ) ))
( "light-headedly"  a ( l ai1 t - h e1 d i d l ii ) ((pos "Pu%" ) ))
( "light-headedness"  n ( l ai1 t - h e1 d i d n @ s ) ((pos "L@%" ) ))
( "light-hearted"  j ( l ai1 t - h aa1 t i d ) ((pos "OA%" ) ))
( "light-heartedly"  a ( l ai1 t - h aa1 t i d l ii ) ((pos "Pu%" ) ))
( "light-heartedness"  n ( l ai1 t - h aa1 t i d n @ s ) ((pos "L@%" ) ))
( "light-heavyweight"  n ( l ai1 t - h e1 v i w ei t ) ((pos "K6%" ) ))
( "light-heavyweights"  n ( l ai1 t - h e1 v i w ei t s ) ((pos "Kj%" ) ))
( "light-minded"  j ( l ai1 t - m ai1 n d i d ) ((pos "OA$" ) ))
( "light-mindedness"  n ( l ai1 t - m ai1 n d i d n @ s ) ((pos "L@$" ) ))
( "light-o'-love"  n ( l ai1 t - @ - l uh1 v ) ((pos "K6$" ) ))
( "light-o'-loves"  n ( l ai1 t - @ - l uh1 v z ) ((pos "Kj$" ) ))
( "lighted"  v ( l ai1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "lighten"  v ( l ai1 t n ) ((pos "J0%" ) (subcat "2A" )))
( "lightened"  v ( l ai1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "lightening"  v ( l ai1 t n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "lightens"  v ( l ai1 t n z ) ((pos "Ja%" ) (subcat "2A" )))
( "lighter"  v ( l ai1 t @ r ) ((pos "H0$" "K6%" "Or%" ) ))
( "lighterage"  n ( l ai1 t @ r i jh ) ((pos "L@$" ) ))
( "lightered"  v ( l ai1 t @ d ) ((pos "Hc$" "Hd$" ) ))
( "lightering"  v ( l ai1 t @ r i ng ) ((pos "Hb$" ) ))
( "lighters"  v ( l ai1 t @ z ) ((pos "Ha$" "Kj%" ) ))
( "lightest"  j ( l ai1 t i s t ) ((pos "Os%" ) ))
( "lighthouse"  n ( l ai1 t h au s ) ((pos "K6%" ) ))
( "lighthouses"  n ( l ai1 t h au z i z ) ((pos "Kj%" ) ))
( "lighting"  v ( l ai1 t i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "lighting-up"  j ( l ai2 t i ng - uh1 p ) ((pos "Oq%" ) ))
( "lightly"  a ( l ai1 t l ii ) ((pos "Pu%" ) ))
( "lightness"  n ( l ai1 t n e s ) ((pos "L@%" ) ))
( "lightning"  n ( l ai1 t n i ng ) ((pos "L@%" ) ))
( "lightning-conductor"  n ( l ai1 t n i ng - k @ n d uh2 k t @ r ) ((pos "K6%" ) ))
( "lightning-conductors"  n ( l ai1 t n i ng - k @ n d uh2 k t @ z ) ((pos "Kj%" ) ))
( "lightning-rod"  n ( l ai1 t n i ng - r o d ) ((pos "K6%" ) ))
( "lightning-rods"  n ( l ai1 t n i ng - r o d z ) ((pos "Kj%" ) ))
( "lights"  v ( l ai1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" "15A" )))
( "lightship"  n ( l ai1 ch i p ) ((pos "K6%" ) ))
( "lightships"  n ( l ai1 ch i p s ) ((pos "Kj%" ) ))
( "lightsome"  j ( l ai1 t s @ m ) ((pos "OA$" ) ))
( "lightsomely"  a ( l ai1 t s @ m l ii ) ((pos "Pu$" ) ))
( "lightsomeness"  n ( l ai1 t s @ m n @ s ) ((pos "L@$" ) ))
( "lightweight"  n ( l ai1 t w ei t ) ((pos "K6%" "OA%" ) ))
( "lightweights"  n ( l ai1 t w ei t s ) ((pos "Kj%" ) ))
( "ligneous"  j ( l i1 g n i@ s ) ((pos "OA$" ) ))
( "lignite"  n ( l i1 g n ai t ) ((pos "L@$" ) ))
( "likable"  j ( l ai1 k @ b l ) ((pos "OA$" ) ))
( "like"  v ( l ai k ) ((pos "H2*" "K6%" "OA%" "Pu%" "T-*" "V-*" ) (subcat "6A" "6D" "7A" "17" "19B" "19C" )))
( "like-minded"  j ( l ai1 k - m ai1 n d i d ) ((pos "OA%" ) ))
( "likeable"  j ( l ai1 k @ b l ) ((pos "OA%" ) ))
( "liked"  v ( l ai1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" "17" "19B" "19C" )))
( "likelier"  j ( l ai1 k l i@ r ) ((pos "Or%" ) ))
( "likeliest"  j ( l ai1 k l i i s t ) ((pos "Os%" ) ))
( "likelihood"  n ( l ai1 k l i h u d ) ((pos "L@%" ) ))
( "likely"  j ( l ai1 k l ii ) ((pos "OD%" "Pu%" ) ))
( "liken"  v ( l ai1 k @ n ) ((pos "H0%" ) (subcat )))
( "likened"  v ( l ai1 k @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "likeness"  n ( l ai1 k n @ s ) ((pos "M7%" ) ))
( "likenesses"  n ( l ai1 k n @ s i z ) ((pos "Mj%" ) ))
( "likening"  v ( l ai1 k @ n i ng ) ((pos "Hb$" ) (subcat )))
( "likens"  v ( l ai1 k @ n z ) ((pos "Ha$" ) (subcat )))
( "likes"  v ( l ai1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "6D" "7A" "17" "19B" "19C" )))
( "likewise"  a ( l ai1 k w ai z ) ((pos "Pu%" "V-%" ) ))
( "liking"  v ( l ai1 k i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" "6D" "7A" "17" "19B" "19C" )))
( "likings"  n ( l ai1 k i ng z ) ((pos "Kj%" ) ))
( "lilac"  n ( l ai1 l @ k ) ((pos "K6%" ) ))
( "lilacs"  n ( l ai1 l @ k s ) ((pos "Kj%" ) ))
( "lilies"  n ( l i1 l i z ) ((pos "Kj%" ) ))
( "lilt"  v ( l i1 l t ) ((pos "J0%" "K6%" ) ))
( "lilted"  v ( l i1 l t i d ) ((pos "Jc%" "Jd%" ) ))
( "lilting"  v ( l i1 l t i ng ) ((pos "Jb%" ) ))
( "lilts"  v ( l i1 l t s ) ((pos "Ja%" "Kj%" ) ))
( "lily"  n ( l i1 l ii ) ((pos "K8%" ) ))
( "lily-livered"  j ( l i1 l i - l i v @ d ) ((pos "OA%" ) ))
( "lily-white"  j ( l i1 l i - w ai t ) ((pos "OA%" ) ))
( "limb"  n ( l i1 m ) ((pos "K6%" ) ))
( "limbed"  j ( l i1 m d ) ((pos "Ot%" ) ))
( "limber"  v ( l i1 m b @ r ) ((pos "J0%" "OA$" ) (subcat "2C" )))
( "limbered"  v ( l i1 m b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "limbering"  v ( l i1 m b @ r i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "limbers"  v ( l i1 m b @ z ) ((pos "Ja%" ) (subcat "2C" )))
( "limbless"  j ( l i1 m l @ s ) ((pos "OA%" ) ))
( "limbo"  n ( l i1 m b ou ) ((pos "M6%" ) ))
( "limbos"  n ( l i1 m b ou z ) ((pos "Mj$" ) ))
( "limbs"  n ( l i1 m z ) ((pos "Kj%" ) ))
( "lime"  v ( l ai1 m ) ((pos "H2$" "M6%" ) (subcat )))
( "lime-tree"  n ( l ai1 m - t r ii ) ((pos "K6%" ) ))
( "lime-trees"  n ( l ai1 m - t r ii z ) ((pos "Kj%" ) ))
( "limed"  v ( l ai1 m d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "limejuice"  n ( l ai1 m jh uu s ) ((pos "L@%" ) ))
( "limekiln"  n ( l ai1 m k i l n ) ((pos "K6$" ) ))
( "limekilns"  n ( l ai1 m k i l n z ) ((pos "Kj$" ) ))
( "limelight"  n ( l ai1 m l ai t ) ((pos "L@%" ) ))
( "limerick"  n ( l i1 m @ r i k ) ((pos "K6%" ) ))
( "limericks"  n ( l i1 m @ r i k s ) ((pos "Kj%" ) ))
( "limes"  v ( l ai1 m z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "limestone"  n ( l ai1 m s t ou n ) ((pos "L@%" ) ))
( "limey"  n ( l ai1 m ii ) ((pos "K6$" ) ))
( "limeys"  n ( l ai1 m i z ) ((pos "Kj$" ) ))
( "liming"  v ( l ai1 m i ng ) ((pos "Hb$" ) (subcat )))
( "limit"  v ( l i1 m i t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "limitation"  n ( l i2 m i t ei1 sh @ n ) ((pos "M6%" ) ))
( "limitations"  n ( l i2 m i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "limited"  v ( l i1 m i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "limiting"  v ( l i1 m i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "limitless"  j ( l i1 m i t l @ s ) ((pos "OA%" ) ))
( "limits"  v ( l i1 m i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "limn"  v ( l i1 m ) ((pos "H0$" ) ))
( "limned"  v ( l i1 m d ) ((pos "Hc$" "Hd$" ) ))
( "limning"  v ( l i1 m i ng ) ((pos "Hb$" ) ))
( "limns"  v ( l i1 m z ) ((pos "Ha$" ) ))
( "limousine"  n ( l i1 m @ z ii n ) ((pos "K6%" ) ))
( "limousines"  n ( l i1 m @ z ii n z ) ((pos "Kj%" ) ))
( "limp"  v ( l i1 m p ) ((pos "I0%" "K6%" "OA%" ) (subcat "2A" )))
( "limped"  v ( l i1 m p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "limpet"  n ( l i1 m p i t ) ((pos "K6%" ) ))
( "limpets"  n ( l i1 m p i t s ) ((pos "Kj%" ) ))
( "limpid"  j ( l i1 m p i d ) ((pos "OA%" ) ))
( "limpidity"  n ( l i1 m p i1 d i t ii ) ((pos "L@%" ) ))
( "limpidly"  a ( l i1 m p i d l ii ) ((pos "Pu%" ) ))
( "limping"  v ( l i1 m p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "limply"  a ( l i1 m p l ii ) ((pos "Pu%" ) ))
( "limpness"  n ( l i1 m p n @ s ) ((pos "L@%" ) ))
( "limps"  v ( l i1 m p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "linchpin"  n ( l i1 n ch p i n ) ((pos "K6%" ) ))
( "linchpins"  n ( l i1 n ch p i n z ) ((pos "Kj%" ) ))
( "linden"  n ( l i1 n d @ n ) ((pos "K6$" ) ))
( "linden-tree"  n ( l i1 n d @ n - t r ii ) ((pos "K6$" ) ))
( "linden-trees"  n ( l i1 n d @ n - t r ii z ) ((pos "Kj$" ) ))
( "lindens"  n ( l i1 n d @ n z ) ((pos "Kj$" ) ))
( "line"  v ( l ai1 n ) ((pos "J2%" "M6*" ) (subcat "2C" "6A" "14" )))
( "line-shooter"  n ( l ai1 n - sh uu t @ r ) ((pos "K6$" ) ))
( "line-shooters"  n ( l ai1 n - sh uu t @ z ) ((pos "Kj$" ) ))
( "line-shooting"  n ( l ai1 n - sh uu t i ng ) ((pos "L@$" ) ))
( "line-up"  n ( l ai1 n - uh p ) ((pos "K6%" ) ))
( "line-ups"  n ( l ai1 n - uh p s ) ((pos "Kj%" ) ))
( "lineage"  n ( l i1 n i i jh ) ((pos "L@%" ) ))
( "lineal"  j ( l i1 n i@ l ) ((pos "OA$" ) ))
( "lineally"  a ( l i1 n i@ l ii ) ((pos "Pu$" ) ))
( "lineament"  n ( l i1 n i@ m @ n t ) ((pos "K6$" ) ))
( "lineaments"  n ( l i1 n i@ m @ n t s ) ((pos "Kj%" ) ))
( "linear"  j ( l i1 n i@ r ) ((pos "OA%" ) ))
( "linearly"  a ( l i1 n i@ l ii ) ((pos "Pu%" ) ))
( "lined"  v ( l ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" )))
( "lineman"  n ( l ai1 n m @ n ) ((pos "Ki%" ) ))
( "linemen"  n ( l ai1 n m @ n ) ((pos "Kj%" ) ))
( "linen"  n ( l i1 n i n ) ((pos "L@%" ) ))
( "linen-draper"  n ( l i1 n i n - d r ei p @ r ) ((pos "K6$" ) ))
( "linen-drapers"  n ( l i1 n i n - d r ei p @ z ) ((pos "Kj$" ) ))
( "liner"  n ( l ai1 n @ r ) ((pos "K6%" ) ))
( "liner-train"  n ( l ai1 n @ - t r ei n ) ((pos "K6$" ) ))
( "liner-trains"  n ( l ai1 n @ - t r ei n z ) ((pos "Kj$" ) ))
( "liners"  n ( l ai1 n @ z ) ((pos "Kj%" ) ))
( "lines"  v ( l ai1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" "14" )))
( "linesman"  n ( l ai1 n z m @ n ) ((pos "Ki%" ) ))
( "linesmen"  n ( l ai1 n z m @ n ) ((pos "Kj%" ) ))
( "ling"  n ( l i1 ng ) ((pos "M6$" ) ))
( "lingam"  n ( l i1 ng g @ m ) ((pos "K6$" ) ))
( "lingams"  n ( l i1 ng g @ m z ) ((pos "Kj$" ) ))
( "linger"  v ( l i1 ng g @ r ) ((pos "I0%" ) (subcat "2A" )))
( "lingered"  v ( l i1 ng g @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "lingerer"  n ( l i1 ng g @ r @ r ) ((pos "K6%" ) ))
( "lingerers"  n ( l i1 ng g @ r @ z ) ((pos "Kj%" ) ))
( "lingerie"  n ( l a1 n zh @ r ii ) ((pos "L@%" ) ))
( "lingering"  v ( l i1 ng g @ r i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "lingeringly"  a ( l i1 ng g @ r i ng l ii ) ((pos "Pu%" ) ))
( "lingers"  v ( l i1 ng g @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "lingo"  n ( l i1 ng g ou ) ((pos "K7%" ) ))
( "lingoes"  n ( l i1 ng g ou z ) ((pos "Kj$" ) ))
( "lings"  n ( l i1 ng z ) ((pos "Mj$" ) ))
( "lingua franca"  n ( l i2 ng g w @ - f r a1 ng k @ ) ((pos "Ki%" ) ))
( "lingual"  j ( l i1 ng g w @ l ) ((pos "OA%" ) ))
( "linguist"  n ( l i1 ng g w i s t ) ((pos "K6%" ) ))
( "linguistic"  j ( l i1 ng g w i1 s t i k ) ((pos "OA%" ) ))
( "linguistically"  a ( l i1 ng g w i1 s t i k l ii ) ((pos "Pu%" ) ))
( "linguistics"  n ( l i1 ng g w i1 s t i k s ) ((pos "Lk%" ) ))
( "linguists"  n ( l i1 ng g w i s t s ) ((pos "Kj%" ) ))
( "liniment"  n ( l i1 n i m @ n t ) ((pos "M6%" ) ))
( "liniments"  n ( l i1 n i m @ n t s ) ((pos "Mj$" ) ))
( "lining"  v ( l ai1 n i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "6A" "14" )))
( "linings"  n ( l ai1 n i ng z ) ((pos "Mj%" ) ))
( "link"  v ( l i1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "link-up"  n ( l i1 ng k - uh p ) ((pos "K6%" ) ))
( "link-ups"  n ( l i1 ng k - uh p s ) ((pos "Kj%" ) ))
( "linkage"  n ( l i1 ng k i jh ) ((pos "M6%" ) ))
( "linkages"  n ( l i1 ng k i jh i z ) ((pos "Mj%" ) ))
( "linkboy"  n ( l i1 n k b oi ) ((pos "K6$" ) ))
( "linkboys"  n ( l i1 n k b oi z ) ((pos "Kj$" ) ))
( "linked"  v ( l i1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "linking"  v ( l i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "linkman"  n ( l i1 ng k m a n ) ((pos "Ki$" ) ))
( "linkmen"  n ( l i1 ng k m e n ) ((pos "Kj$" ) ))
( "links"  v ( l i1 ng k s ) ((pos "Ja%" "K9%" ) (subcat "2A" "2C" "6A" "15A" )))
( "linnet"  n ( l i1 n i t ) ((pos "K6%" ) ))
( "linnets"  n ( l i1 n i t s ) ((pos "Kj%" ) ))
( "lino"  n ( l ai1 n ou ) ((pos "L@%" ) ))
( "linocut"  n ( l ai1 n ou k uh t ) ((pos "K6%" ) ))
( "linocuts"  n ( l ai1 n ou k uh t s ) ((pos "Kj%" ) ))
( "linoleum"  n ( l i1 n ou1 l i@ m ) ((pos "L@%" ) ))
( "linotype"  n ( l ai1 n ou t ai p ) ((pos "K6%" ) ))
( "linotypes"  n ( l ai1 n ou t ai p s ) ((pos "Kj%" ) ))
( "linseed"  n ( l i1 n s ii d ) ((pos "L@%" ) ))
( "linsey-woolsey"  n ( l i2 n z i - w u1 l z ii ) ((pos "L@$" ) ))
( "lint"  n ( l i1 n t ) ((pos "L@%" ) ))
( "lintel"  n ( l i1 n t l ) ((pos "K6%" ) ))
( "lintels"  n ( l i1 n t l z ) ((pos "Kj%" ) ))
( "lion"  n ( l ai1 @ n ) ((pos "K6%" ) ))
( "lion-hearted"  j ( l ai1 @ n - h aa t i d ) ((pos "OA%" ) ))
( "lion-hunter"  n ( l ai1 @ n - h uh n t @ r ) ((pos "K6%" ) ))
( "lion-hunters"  n ( l ai1 @ n - h uh n t @ z ) ((pos "Kj%" ) ))
( "lioness"  n ( l ai1 @ n e s ) ((pos "K7%" ) ))
( "lionesses"  n ( l ai1 @ n e s i z ) ((pos "Kj%" ) ))
( "lionize"  v ( l ai1 @ n ai z ) ((pos "H2$" ) (subcat )))
( "lionized"  v ( l ai1 @ n ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "lionizes"  v ( l ai1 @ n ai z i z ) ((pos "Ha$" ) (subcat )))
( "lionizing"  v ( l ai1 @ n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "lions"  n ( l ai1 @ n z ) ((pos "Kj%" ) ))
( "lip"  n ( l i1 p ) ((pos "M6%" ) ))
( "lip-read"  v ( l i1 p - r e d ) ((pos "Hc$" "Hd$" ) ))
( "lip-read"  v ( l i1 p - r ii d ) ((pos "H5%" ) ))
( "lip-reading"  v ( l i1 p - r ii d i ng ) ((pos "Hb%" "L@%" ) ))
( "lip-reads"  v ( l i1 p - r ii d z ) ((pos "Ha%" ) ))
( "lipped"  j ( l i1 p t ) ((pos "Ot%" ) ))
( "lips"  n ( l i1 p s ) ((pos "Mj%" ) ))
( "lipstick"  n ( l i1 p s t i k ) ((pos "M6%" ) ))
( "lipsticks"  n ( l i1 p s t i k s ) ((pos "Mj%" ) ))
( "liquefaction"  n ( l i2 k w i f a1 k sh @ n ) ((pos "L@$" ) ))
( "liquefied"  v ( l i1 k w i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "liquefies"  v ( l i1 k w i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "liquefy"  v ( l i1 k w i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "liquefying"  v ( l i1 k w i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "liquescent"  j ( l i1 k w e1 s n t ) ((pos "OA$" ) ))
( "liqueur"  n ( l i1 k y u@1 r ) ((pos "K6%" ) ))
( "liqueurs"  n ( l i1 k y u@1 z ) ((pos "Kj%" ) ))
( "liquid"  n ( l i1 k w i d ) ((pos "M6%" "OA%" ) ))
( "liquidate"  v ( l i1 k w i d ei t ) ((pos "J2%" ) (subcat "2A" )))
( "liquidated"  v ( l i1 k w i d ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "liquidates"  v ( l i1 k w i d ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "liquidating"  v ( l i1 k w i d ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "liquidation"  n ( l i2 k w i d ei1 sh @ n ) ((pos "L@%" ) ))
( "liquidator"  n ( l i1 k w i d ei t @ r ) ((pos "K6%" ) ))
( "liquidators"  n ( l i1 k w i d ei t @ z ) ((pos "Kj%" ) ))
( "liquidity"  n ( l i1 k w i1 d i t ii ) ((pos "L@%" ) ))
( "liquidize"  v ( l i1 k w i d ai z ) ((pos "H2%" ) (subcat )))
( "liquidized"  v ( l i1 k w i d ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "liquidizer"  n ( l i1 k w i d ai z @ r ) ((pos "K6%" ) ))
( "liquidizers"  n ( l i1 k w i d ai z @ z ) ((pos "Kj%" ) ))
( "liquidizes"  v ( l i1 k w i d ai z i z ) ((pos "Ha%" ) (subcat )))
( "liquidizing"  v ( l i1 k w i d ai z i ng ) ((pos "Hb%" ) (subcat )))
( "liquids"  n ( l i1 k w i d z ) ((pos "Mj%" ) ))
( "liquor"  n ( l i1 k @ r ) ((pos "M6%" ) ))
( "liquorice"  n ( l i1 k @ r i s ) ((pos "L@%" ) ))
( "liquors"  n ( l i1 k @ z ) ((pos "Mj$" ) ))
( "lira"  n ( l i@1 r @ ) ((pos "K6$" ) ))
( "liras"  n ( l i@1 r @ z ) ((pos "Kj$" ) ))
( "lire"  n ( l i@1 r @ ) ((pos "Kj$" ) ))
( "lisle"  n ( l ai1 l ) ((pos "L@$" ) ))
( "lisp"  v ( l i1 s p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "lisped"  v ( l i1 s p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "lisping"  v ( l i1 s p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "lispingly"  a ( l i1 s p i ng l ii ) ((pos "Pu%" ) ))
( "lisps"  v ( l i1 s p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "lissom"  j ( l i1 s @ m ) ((pos "OA%" ) ))
( "lissome"  j ( l i1 s @ m ) ((pos "OA$" ) ))
( "lissomness"  n ( l i1 s @ m n @ s ) ((pos "L@$" ) ))
( "list"  v ( l i1 s t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "list-price"  n ( l i1 s t - p r ai s ) ((pos "K6%" ) ))
( "list-prices"  n ( l i1 s t - p r ai s i z ) ((pos "Kj$" ) ))
( "listed"  v ( l i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "listen"  v ( l i1 s n ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "listened"  v ( l i1 s n d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "listener"  n ( l i1 s n @ r ) ((pos "K6%" ) ))
( "listeners"  n ( l i1 s n @ z ) ((pos "Kj%" ) ))
( "listening"  v ( l i1 s n i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "listens"  v ( l i1 s n z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "listing"  v ( l i1 s t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" )))
( "listings"  n ( l i1 s t i ng z ) ((pos "Kj%" ) ))
( "listless"  j ( l i1 s t l i s ) ((pos "OA%" ) ))
( "listlessly"  a ( l i1 s t l i s l ii ) ((pos "Pu%" ) ))
( "listlessness"  n ( l i1 s t l i s n @ s ) ((pos "L@%" ) ))
( "lists"  v ( l i1 s t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "lists"  n ( l ii1 s t s ) ((pos "Kj$" ) ))
( "lit"  v ( l i1 t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "litanies"  n ( l i1 t @ n i z ) ((pos "Kj%" ) ))
( "litany"  n ( l i1 t @ n ii ) ((pos "K8%" ) ))
( "litchee"  n ( l ai1 ch ii ) ((pos "K6$" ) ))
( "litchees"  n ( l ai1 ch ii z ) ((pos "Kj$" ) ))
( "litchi"  n ( l ai1 ch ii ) ((pos "K6$" ) ))
( "litchis"  n ( l ai1 ch ii z ) ((pos "Kj$" ) ))
( "literacy"  n ( l i1 t @ r @ s ii ) ((pos "L@%" ) ))
( "literal"  n ( l i1 t @ r @ l ) ((pos "K6$" "OA%" ) ))
( "literally"  a ( l i1 t @ r @ l ii ) ((pos "Pu%" ) ))
( "literals"  n ( l i1 t @ r @ l z ) ((pos "Kj$" ) ))
( "literary"  j ( l i1 t @ r @ r ii ) ((pos "OA%" ) ))
( "literate"  n ( l i1 t @ r @ t ) ((pos "K6$" "OA%" ) ))
( "literates"  n ( l i1 t @ r @ t s ) ((pos "Kj$" ) ))
( "literati"  n ( l i2 t @ r aa1 t ii ) ((pos "Kj%" ) ))
( "literature"  n ( l i1 t r @ ch @ r ) ((pos "L@%" ) ))
( "lithe"  j ( l ai1 dh ) ((pos "OA%" ) ))
( "lithograph"  v ( l i1 th @ g r aa f ) ((pos "J0$" "K6%" ) ))
( "lithographed"  v ( l i1 th @ g r aa f t ) ((pos "Jc$" "Jd%" ) ))
( "lithographic"  j ( l i2 th @ g r a1 f i k ) ((pos "OA%" ) ))
( "lithographing"  v ( l i1 th @ g r aa f i ng ) ((pos "Jb$" ) ))
( "lithographs"  v ( l i1 th @ g r aa f s ) ((pos "Ja$" "Kj%" ) ))
( "lithography"  n ( l i1 th o1 g r @ f ii ) ((pos "L@%" ) ))
( "litigant"  n ( l i1 t i g @ n t ) ((pos "K6%" ) ))
( "litigants"  n ( l i1 t i g @ n t s ) ((pos "Kj%" ) ))
( "litigate"  v ( l i1 t i g ei t ) ((pos "J2%" ) (subcat "2A" )))
( "litigated"  v ( l i1 t i g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "litigates"  v ( l i1 t i g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "litigating"  v ( l i1 t i g ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "litigation"  n ( l i2 t i g ei1 sh @ n ) ((pos "L@%" ) ))
( "litigious"  j ( l i1 t i1 jh @ s ) ((pos "OA$" ) ))
( "litmus"  n ( l i1 t m @ s ) ((pos "L@%" ) ))
( "litmus-paper"  n ( l i1 t m @ s - p ei p @ r ) ((pos "M6%" ) ))
( "litmus-papers"  n ( l i1 t m @ s - p ei p @ z ) ((pos "Mj%" ) ))
( "litotes"  n ( l ai1 t ou t ii z ) ((pos "K9$" ) ))
( "litre"  n ( l ii1 t @ r ) ((pos "K6%" ) ))
( "litres"  n ( l ii1 t @ z ) ((pos "Kj%" ) ))
( "litter"  v ( l i1 t @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "14" "15A" )))
( "litter-basket"  n ( l i1 t @ - b aa s k i t ) ((pos "K6%" ) ))
( "litter-baskets"  n ( l i1 t @ - b aa s k i t s ) ((pos "Kj%" ) ))
( "litter-lout"  n ( l i1 t @ - l au t ) ((pos "K6%" ) ))
( "litter-louts"  n ( l i1 t @ - l au t s ) ((pos "Kj%" ) ))
( "litterbin"  n ( l i1 t @ b i n ) ((pos "K6%" ) ))
( "litterbins"  n ( l i1 t @ b i n z ) ((pos "Kj%" ) ))
( "littered"  v ( l i1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15A" )))
( "littering"  v ( l i1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" "15A" )))
( "litters"  v ( l i1 t @ z ) ((pos "Ja%" "Mj$" ) (subcat "2A" "6A" "14" "15A" )))
( "little"  n ( l i1 t l ) ((pos "L@%" "OB*" "Pu*" ) ))
( "littleness"  n ( l i1 t l n @ s ) ((pos "L@%" ) ))
( "littler"  j ( l i1 t l @ r ) ((pos "Or%" ) ))
( "littlest"  j ( l i1 t l i s t ) ((pos "Os%" ) ))
( "littoral"  n ( l i1 t @ r @ l ) ((pos "K6$" "OA$" ) ))
( "littorals"  n ( l i1 t @ r @ l z ) ((pos "Kj$" ) ))
( "liturgical"  j ( l i1 t @@1 jh i k l ) ((pos "OA%" ) ))
( "liturgies"  n ( l i1 t @ jh i z ) ((pos "Mj%" ) ))
( "liturgy"  n ( l i1 t @ jh ii ) ((pos "M8%" ) ))
( "livable"  j ( l i1 v @ b l ) ((pos "OA%" ) ))
( "live"  v ( l i1 v ) ((pos "J2%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6B" )))
( "live"  j ( l ai1 v ) ((pos "OA%" "Pu%" ) ))
( "live-birth"  n ( l ai1 v - b @@ th ) ((pos "K6%" ) ))
( "live-births"  n ( l ai1 v - b @@ th s ) ((pos "Kj%" ) ))
( "liveable"  j ( l i1 v @ b l ) ((pos "OA%" ) ))
( "lived"  v ( l i1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6B" )))
( "livelier"  j ( l ai1 v l i@ r ) ((pos "Or%" ) ))
( "liveliest"  j ( l ai1 v l i i s t ) ((pos "Os%" ) ))
( "livelihood"  n ( l ai1 v l i h u d ) ((pos "K6%" ) ))
( "livelihoods"  n ( l ai1 v l i h u d z ) ((pos "Kj%" ) ))
( "liveliness"  n ( l ai1 v l i n @ s ) ((pos "L@%" ) ))
( "livelong"  j ( l i1 v l o ng ) ((pos "OA$" ) ))
( "lively"  j ( l ai1 v l ii ) ((pos "OD%" ) ))
( "liven"  v ( l ai1 v n ) ((pos "J0%" ) (subcat "2C" )))
( "livened"  v ( l ai1 v n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "livening"  v ( l ai1 v n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "livens"  v ( l ai1 v n z ) ((pos "Ja%" ) (subcat "2C" )))
( "liver"  n ( l i1 v @ r ) ((pos "M6%" ) ))
( "liveried"  j ( l i1 v @ r i d ) ((pos "OA%" ) ))
( "liveries"  n ( l i1 v @ r i z ) ((pos "Kj$" ) ))
( "liverish"  j ( l i1 v @ r i sh ) ((pos "OA%" ) ))
( "livers"  n ( l i1 v @ z ) ((pos "Mj%" ) ))
( "liverwurst"  n ( l i1 v @ w @@ s t ) ((pos "L@$" ) ))
( "livery"  n ( l i1 v @ r ii ) ((pos "K8%" "OA%" ) ))
( "liveryman"  n ( l i1 v @ r i m a n ) ((pos "Ki$" ) ))
( "liverymen"  n ( l i1 v @ r i m e n ) ((pos "Kj$" ) ))
( "lives"  v ( l i1 v z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6B" )))
( "lives"  n ( l ai1 v z ) ((pos "Kj%" ) ))
( "livestock"  n ( l ai1 v s t o k ) ((pos "L@%" ) ))
( "livid"  j ( l i1 v i d ) ((pos "OA%" ) ))
( "lividly"  a ( l i1 v i d l ii ) ((pos "Pu$" ) ))
( "living"  v ( l i1 v i ng ) ((pos "Jb*" "M6%" "OA*" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6B" )))
( "living-room"  n ( l i1 v i ng - r uu m ) ((pos "K6%" ) ))
( "living-rooms"  n ( l i1 v i ng - r uu m z ) ((pos "Kj%" ) ))
( "living-space"  n ( l i1 v i ng - s p ei s ) ((pos "K6%" ) ))
( "living-spaces"  n ( l i1 v i ng - s p ei s i z ) ((pos "Kj%" ) ))
( "livings"  n ( l i1 v i ng z ) ((pos "Mj%" ) ))
( "lizard"  n ( l i1 z @ d ) ((pos "K6%" ) ))
( "lizards"  n ( l i1 z @ d z ) ((pos "Kj%" ) ))
( "ll"  n ( l ai1 n z ) ((pos "Y)%" ) ))
( "llama"  n ( l aa1 m @ ) ((pos "K6%" ) ))
( "llamas"  n ( l aa1 m @ z ) ((pos "Kj%" ) ))
( "lo"  uh ( l ou1 ) ((pos "W-$" ) ))
( "load"  v ( l ou1 d ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "load-line"  n ( l ou1 d - l ai n ) ((pos "K6$" ) ))
( "load-lines"  n ( l ou1 d - l ai n z ) ((pos "Kj$" ) ))
( "load-shedding"  n ( l ou1 d - sh e d i ng ) ((pos "L@$" ) ))
( "loaded"  v ( l ou1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "loader"  n ( l ou1 d @ r ) ((pos "K6%" ) ))
( "loaders"  n ( l ou1 d @ z ) ((pos "Kj%" ) ))
( "loading"  v ( l ou1 d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "loadings"  n ( l ou1 d i ng z ) ((pos "Kj%" ) ))
( "loads"  v ( l ou1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "loadstar"  n ( l ou1 d s t aa r ) ((pos "K6$" ) ))
( "loadstars"  n ( l ou1 d s t aa z ) ((pos "Kj$" ) ))
( "loadstone"  n ( l ou1 d s t ou n ) ((pos "L@$" ) ))
( "loaf"  v ( l ou1 f ) ((pos "J0%" "Mi%" ) (subcat "2A" "2C" "15A" )))
( "loaf-sugar"  n ( l ou1 f - sh u g @ r ) ((pos "L@$" ) ))
( "loafed"  v ( l ou1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "15A" )))
( "loafer"  n ( l ou1 f @ r ) ((pos "K6%" ) ))
( "loafers"  n ( l ou1 f @ z ) ((pos "Kj%" ) ))
( "loafing"  v ( l ou1 f i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "15A" )))
( "loafs"  v ( l ou1 f s ) ((pos "Ja%" ) (subcat "2A" "2C" "15A" )))
( "loam"  n ( l ou1 m ) ((pos "L@%" ) ))
( "loamier"  j ( l ou1 m i@ r ) ((pos "Or%" ) ))
( "loamiest"  j ( l ou1 m i i s t ) ((pos "Os%" ) ))
( "loamy"  j ( l ou1 m ii ) ((pos "OD%" ) ))
( "loan"  v ( l ou1 n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "loan-collection"  n ( l ou1 n - k @ l e2 k sh @ n ) ((pos "K6$" ) ))
( "loan-collections"  n ( l ou1 n - k @ l e2 k sh @ n z ) ((pos "Kj$" ) ))
( "loan-office"  n ( l ou1 n - o f i s ) ((pos "K6$" ) ))
( "loan-offices"  n ( l ou1 n - o f i s i z ) ((pos "Kj$" ) ))
( "loaned"  v ( l ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "loaning"  v ( l ou1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "loans"  v ( l ou1 n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "loanword"  n ( l ou1 n w @@ d ) ((pos "K6%" ) ))
( "loanwords"  n ( l ou1 n w @@ d z ) ((pos "Kj%" ) ))
( "loath"  j ( l ou1 th ) ((pos "OA%" ) ))
( "loathe"  v ( l ou1 dh ) ((pos "H2%" ) (subcat "6A" )))
( "loathed"  v ( l ou1 dh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "loathes"  v ( l ou1 dh z ) ((pos "Ha%" ) (subcat "6A" )))
( "loathing"  v ( l ou1 dh i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" )))
( "loathlier"  j ( l ou1 th l i@ r ) ((pos "Or$" ) ))
( "loathliest"  j ( l ou1 th l i i s t ) ((pos "Os$" ) ))
( "loathly"  j ( l ou1 th l ii ) ((pos "OD$" ) ))
( "loathsome"  j ( l ou1 dh s @ m ) ((pos "OA%" ) ))
( "loaves"  n ( l ou1 v z ) ((pos "Kj%" ) ))
( "lob"  v ( l o1 b ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" )))
( "lobbed"  v ( l o1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "lobbied"  v ( l o1 b i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "lobbies"  v ( l o1 b i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "lobbing"  v ( l o1 b i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "lobby"  v ( l o1 b ii ) ((pos "J3%" "K8%" ) (subcat "2A" "2C" "6A" )))
( "lobbying"  v ( l o1 b i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "lobbyist"  n ( l o1 b i i s t ) ((pos "K6%" ) ))
( "lobbyists"  n ( l o1 b i i s t s ) ((pos "Kj%" ) ))
( "lobe"  n ( l ou1 b ) ((pos "K6%" ) ))
( "lobed"  j ( l ou1 b d ) ((pos "OA%" ) ))
( "lobes"  n ( l ou1 b z ) ((pos "Kj%" ) ))
( "lobs"  v ( l o1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "lobster"  n ( l o1 b s t @ r ) ((pos "M6%" ) ))
( "lobster-pot"  n ( l o1 b s t @ - p o t ) ((pos "K6%" ) ))
( "lobster-pots"  n ( l o1 b s t @ - p o t s ) ((pos "Kj%" ) ))
( "lobsters"  n ( l o1 b s t @ z ) ((pos "Mj%" ) ))
( "loc cit"  n ( l o2 k - s i1 t ) ((pos "Y~$" ) ))
( "local"  n ( l ou1 k l ) ((pos "K6%" "OA%" ) ))
( "locale"  n ( l ou1 k aa1 l ) ((pos "K6%" ) ))
( "locales"  n ( l ou1 k aa1 l z ) ((pos "Kj$" ) ))
( "localism"  n ( l ou1 k @ l i z @ m ) ((pos "M6$" ) ))
( "localisms"  n ( l ou1 k @ l i z @ m z ) ((pos "Mj$" ) ))
( "localities"  n ( l ou1 k a1 l i t i z ) ((pos "Mj%" ) ))
( "locality"  n ( l ou1 k a1 l i t ii ) ((pos "M8%" ) ))
( "localization"  n ( l ou2 k @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "localizations"  n ( l ou2 k @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "localize"  v ( l ou1 k @ l ai z ) ((pos "H2%" ) (subcat )))
( "localized"  v ( l ou1 k @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "localizes"  v ( l ou1 k @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "localizing"  v ( l ou1 k @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "locally"  a ( l ou1 k @ l ii ) ((pos "Pu%" ) ))
( "locals"  n ( l ou1 k l z ) ((pos "Kj%" ) ))
( "locate"  v ( l ou1 k ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "located"  v ( l ou1 k ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "locates"  v ( l ou1 k ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "locating"  v ( l ou1 k ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "location"  n ( l ou1 k ei1 sh @ n ) ((pos "M6%" ) ))
( "locations"  n ( l ou1 k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "loch"  n ( l o1 k ) ((pos "K6%" ) ))
( "lochs"  n ( l o1 k s ) ((pos "Kj%" ) ))
( "loci"  n ( l ou1 s ai ) ((pos "Kj$" ) ))
( "lock"  v ( l o1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "lock-gate"  n ( l o1 k - g ei t ) ((pos "K6%" ) ))
( "lock-gates"  n ( l o1 k - g ei t s ) ((pos "Kj%" ) ))
( "lock-keeper"  n ( l o1 k - k ii p @ r ) ((pos "K6%" ) ))
( "lock-keepers"  n ( l o1 k - k ii p @ z ) ((pos "Kj%" ) ))
( "locked"  v ( l o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "locker"  n ( l o1 k @ r ) ((pos "K6%" ) ))
( "lockers"  n ( l o1 k @ z ) ((pos "Kj%" ) ))
( "locket"  n ( l o1 k i t ) ((pos "K6%" ) ))
( "lockets"  n ( l o1 k i t s ) ((pos "Kj%" ) ))
( "locking"  v ( l o1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "lockjaw"  n ( l o1 k jh oo ) ((pos "L@%" ) ))
( "locknut"  n ( l o1 k n uh t ) ((pos "K6$" ) ))
( "locknuts"  n ( l o1 k n uh t s ) ((pos "Kj$" ) ))
( "lockout"  n ( l o1 k au t ) ((pos "K6%" ) ))
( "lockouts"  n ( l o1 k au t s ) ((pos "Kj$" ) ))
( "locks"  v ( l o1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "locksmith"  n ( l o1 k s m i th ) ((pos "K6%" ) ))
( "locksmiths"  n ( l o1 k s m i th s ) ((pos "Kj%" ) ))
( "lockstitch"  n ( l o1 k s t i ch ) ((pos "K7$" ) ))
( "lockstitches"  n ( l o1 k s t i ch i z ) ((pos "Kj$" ) ))
( "lockup"  n ( l o1 k uh p ) ((pos "K6%" "OA%" ) ))
( "lockups"  n ( l o1 k uh p s ) ((pos "Kj$" ) ))
( "loco"  j ( l ou1 k ou ) ((pos "OA$" ) ))
( "locomotion"  n ( l ou1 k @ m ou sh @ n ) ((pos "L@%" ) ))
( "locomotive"  n ( l ou2 k @ m ou1 t i v ) ((pos "K6%" "OA%" ) ))
( "locomotives"  n ( l ou2 k @ m ou1 t i v z ) ((pos "Kj%" ) ))
( "locum"  n ( l ou1 k @ m ) ((pos "K6%" ) ))
( "locum tenens"  n ( l ou2 k @ m - t ii1 n e n z ) ((pos "Ki$" ) ))
( "locums"  n ( l ou1 k @ m z ) ((pos "Kj$" ) ))
( "locus"  n ( l ou1 k @ s ) ((pos "Ki$" ) ))
( "locus classicus"  n ( l ou2 k @ s - k l a1 s i k @ s ) ((pos "Ki$" ) ))
( "locust"  n ( l ou1 k @ s t ) ((pos "K6%" ) ))
( "locust-tree"  n ( l ou1 k @ s t - t r ii ) ((pos "K6$" ) ))
( "locust-trees"  n ( l ou1 k @ s t - t r ii z ) ((pos "Kj$" ) ))
( "locusts"  n ( l ou1 k @ s t s ) ((pos "Kj%" ) ))
( "locution"  n ( l @1 k y uu1 sh @ n ) ((pos "M6%" ) ))
( "locutions"  n ( l @1 k y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "lode"  n ( l ou1 d ) ((pos "K6$" ) ))
( "lodes"  n ( l ou1 d z ) ((pos "Kj$" ) ))
( "lodestar"  n ( l ou1 d s t aa r ) ((pos "K6%" ) ))
( "lodestars"  n ( l ou1 d s t aa z ) ((pos "Kj%" ) ))
( "lodestone"  n ( l ou1 d s t ou n ) ((pos "L@%" ) ))
( "lodge"  v ( l o1 jh ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "14" )))
( "lodged"  v ( l o1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" )))
( "lodgement"  n ( l o1 jh m @ n t ) ((pos "M6$" ) ))
( "lodgements"  n ( l o1 jh m @ n t s ) ((pos "Mj$" ) ))
( "lodger"  n ( l o1 jh @ r ) ((pos "K6%" ) ))
( "lodgers"  n ( l o1 jh @ z ) ((pos "Kj%" ) ))
( "lodges"  v ( l o1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" )))
( "lodging"  v ( l o1 jh i ng ) ((pos "Jb%" "K6%" ) (subcat "3A" "6A" "14" )))
( "lodging-house"  n ( l o1 jh i ng - h au s ) ((pos "K6%" ) ))
( "lodging-houses"  n ( l o1 jh i ng - h au z i z ) ((pos "Kj%" ) ))
( "lodgings"  n ( l o1 jh i ng z ) ((pos "Kj%" ) ))
( "lodgment"  n ( l o1 jh m @ n t ) ((pos "M6$" ) ))
( "lodgments"  n ( l o1 jh m @ n t s ) ((pos "Mj$" ) ))
( "loess"  n ( l ou1 e s ) ((pos "L@$" ) ))
( "loft"  v ( l o1 f t ) ((pos "H0$" "K6%" ) (subcat )))
( "lofted"  v ( l o1 f t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "loftier"  j ( l o1 f t i@ r ) ((pos "Or%" ) ))
( "loftiest"  j ( l o1 f t i i s t ) ((pos "Os%" ) ))
( "loftily"  a ( l o1 f t i l ii ) ((pos "Pu%" ) ))
( "loftiness"  n ( l o1 f t i n @ s ) ((pos "L@%" ) ))
( "lofting"  v ( l o1 f t i ng ) ((pos "Hb$" ) (subcat )))
( "lofts"  v ( l o1 f t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "lofty"  j ( l o1 f t ii ) ((pos "OD%" ) ))
( "log"  v ( l o1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "log-cabin"  n ( l o1 g - k a1 b i n ) ((pos "K6%" ) ))
( "log-cabins"  n ( l o1 g - k a1 b i n z ) ((pos "Kj%" ) ))
( "log-jam"  n ( l o1 g - jh a m ) ((pos "K6%" ) ))
( "log-jams"  n ( l o1 g - jh a m z ) ((pos "Kj$" ) ))
( "log-rolling"  n ( l o1 g - r ou l i ng ) ((pos "L@%" ) ))
( "loganberries"  n ( l ou1 g @ n b e r i z ) ((pos "Kj%" ) ))
( "loganberry"  n ( l ou1 g @ n b e r ii ) ((pos "K8%" ) ))
( "logarithm"  n ( l o1 g @ r i dh @ m ) ((pos "K6%" ) ))
( "logarithmic"  j ( l o2 g @ r i1 dh m i k ) ((pos "OA%" ) ))
( "logarithmically"  a ( l o2 g @ r i1 dh m i k l ii ) ((pos "Pu%" ) ))
( "logarithms"  n ( l o1 g @ r i dh @ m z ) ((pos "Kj%" ) ))
( "logbook"  n ( l o1 g b u k ) ((pos "K6%" ) ))
( "logbooks"  n ( l o1 g b u k s ) ((pos "Kj%" ) ))
( "logged"  v ( l o1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "loggerheads"  n ( l o1 g @ h e d z ) ((pos "L@%" ) ))
( "loggia"  n ( l o1 jh i@ ) ((pos "K6$" ) ))
( "loggias"  n ( l o1 jh i@ z ) ((pos "Kj$" ) ))
( "logging"  v ( l o1 g i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "logic"  n ( l o1 jh i k ) ((pos "M6%" ) ))
( "logical"  j ( l o1 jh i k l ) ((pos "OA%" ) ))
( "logicality"  n ( l o2 jh i k a1 l i t ii ) ((pos "L@$" ) ))
( "logically"  a ( l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "logician"  n ( l @1 jh i1 sh @ n ) ((pos "K6%" ) ))
( "logicians"  n ( l @1 jh i1 sh @ n z ) ((pos "Kj%" ) ))
( "logics"  n ( l o1 jh i k s ) ((pos "Mj%" ) ))
( "logistics"  n ( l @1 jh i1 s t i k s ) ((pos "Lk%" ) ))
( "logs"  v ( l o1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "loin"  n ( l oi1 n ) ((pos "K6%" ) ))
( "loincloth"  n ( l oi1 n k l o th ) ((pos "K6%" ) ))
( "loincloths"  n ( l oi1 n k l o th s ) ((pos "Kj%" ) ))
( "loins"  n ( l oi1 n z ) ((pos "Kj%" ) ))
( "loiter"  v ( l oi1 t @ r ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "loitered"  v ( l oi1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "loiterer"  n ( l oi1 t @ r @ r ) ((pos "K6%" ) ))
( "loiterers"  n ( l oi1 t @ r @ z ) ((pos "Kj%" ) ))
( "loitering"  v ( l oi1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "loiters"  v ( l oi1 t @ z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "loll"  v ( l o1 l ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "lolled"  v ( l o1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "lollies"  n ( l o1 l i z ) ((pos "Kj%" ) ))
( "lolling"  v ( l o1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "lollipop"  n ( l o1 l i p o p ) ((pos "K6%" ) ))
( "lollipops"  n ( l o1 l i p o p s ) ((pos "Kj%" ) ))
( "lolls"  v ( l o1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "lolly"  n ( l o1 l ii ) ((pos "K8%" ) ))
( "lone"  j ( l ou1 n ) ((pos "OA%" ) ))
( "lonelier"  j ( l ou1 n l i@ r ) ((pos "Or%" ) ))
( "loneliest"  j ( l ou1 n l i i s t ) ((pos "Os%" ) ))
( "loneliness"  n ( l ou1 n l i n @ s ) ((pos "L@%" ) ))
( "lonely"  j ( l ou1 n l ii ) ((pos "OD%" ) ))
( "lonesome"  j ( l ou1 n s @ m ) ((pos "OA%" ) ))
( "long"  v ( l o1 ng ) ((pos "I0%" "M6%" "OC*" "Pu*" "Y>$" ) (subcat "3A" )))
( "long-distance"  j ( l o1 ng - d i s t @ n s ) ((pos "Oq%" ) ))
( "long-drawn-out"  j ( l o2 ng - d r oo n - ou1 t ) ((pos "OA%" ) ))
( "long-haired"  j ( l o1 ng - h e@1 d ) ((pos "OA%" ) ))
( "long-headed"  j ( l o1 ng - h e1 d i d ) ((pos "OA$" ) ))
( "long-lived"  j ( l o1 ng - l i1 v d ) ((pos "OA%" ) ))
( "long-play"  j ( l o1 ng - p l ei ) ((pos "OA$" ) ))
( "long-playing"  j ( l o1 ng - p l ei i ng ) ((pos "OA%" ) ))
( "long-range"  j ( l o1 ng - r ei n jh ) ((pos "Oq%" ) ))
( "long-sighted"  j ( l o1 ng - s ai1 t i d ) ((pos "OA%" ) ))
( "long-standing"  j ( l o1 ng - s t a n d i ng ) ((pos "OA%" ) ))
( "long-suffering"  j ( l o1 ng - s uh1 f @ r i ng ) ((pos "OA%" ) ))
( "long-term"  j ( l o1 ng - t @@ m ) ((pos "Oq%" ) ))
( "long-time"  j ( l o1 ng - t ai m ) ((pos "Oq%" ) ))
( "long-winded"  j ( l o1 ng - w i1 n d i d ) ((pos "OA%" ) ))
( "long-windedness"  n ( l o1 ng - w i1 n d i d n @ s ) ((pos "L@%" ) ))
( "longboat"  n ( l o1 ng b ou t ) ((pos "K6%" ) ))
( "longboats"  n ( l o1 ng b ou t s ) ((pos "Kj%" ) ))
( "longbow"  n ( l o1 ng b au ) ((pos "K6%" ) ))
( "longbows"  n ( l o1 ng b au z ) ((pos "Kj%" ) ))
( "longed"  v ( l o1 ng d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "longer"  j ( l o1 ng g @ r ) ((pos "Or%" "Pu%" ) ))
( "longest"  j ( l o1 ng g i s t ) ((pos "Os%" "Pu%" ) ))
( "longevity"  n ( l o1 n jh e1 v i t ii ) ((pos "L@%" ) ))
( "longhand"  n ( l o1 ng h a n d ) ((pos "L@%" ) ))
( "longing"  v ( l o1 ng i ng ) ((pos "Ib%" "M6%" "OA%" ) (subcat "3A" )))
( "longingly"  a ( l o1 ng i ng l ii ) ((pos "Pu%" ) ))
( "longings"  n ( l o1 ng i ng z ) ((pos "Mj$" ) ))
( "longish"  j ( l o1 ng i sh ) ((pos "OA%" ) ))
( "longitude"  n ( l o1 n jh i t y uu d ) ((pos "K6%" ) ))
( "longitudes"  n ( l o1 n jh i t y uu d z ) ((pos "Kj$" ) ))
( "longitudinal"  j ( l o2 n jh i t y uu1 d i n l ) ((pos "OA%" ) ))
( "longitudinally"  a ( l o2 n jh i t y uu1 d i n @ l ii ) ((pos "Pu%" ) ))
( "longs"  v ( l o1 ng z ) ((pos "Ia%" "Mj$" ) (subcat "3A" )))
( "longshoreman"  n ( l o1 ng sh oo m @ n ) ((pos "Ki%" ) ))
( "longshoremen"  n ( l o1 ng sh oo m @ n ) ((pos "Kj%" ) ))
( "longways"  a ( l o1 ng w ei z ) ((pos "Pu%" ) ))
( "longwise"  a ( l o1 ng w ai z ) ((pos "Pu%" ) ))
( "loo"  n ( l uu1 ) ((pos "K6%" ) ))
( "loofa"  n ( l uu1 f @ ) ((pos "K6$" ) ))
( "loofah"  n ( l uu1 f @ ) ((pos "K6%" ) ))
( "loofahs"  n ( l uu1 f @ z ) ((pos "Kj%" ) ))
( "loofas"  n ( l uu1 f @ z ) ((pos "Kj$" ) ))
( "look"  v ( l u1 k ) ((pos "J0*" "K6*" ) (subcat "2A" "2C" "2D" "3A" "4A" "4D" "6A" "8" )))
( "look-over"  n ( l u1 k - ou v @ r ) ((pos "Ki$" ) ))
( "looked"  v ( l u1 k t ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2C" "2D" "3A" "4A" "4D" "6A" "8" )))
( "looker"  n ( l u1 k @ r ) ((pos "K6%" ) ))
( "looker-on"  n ( l u2 k @ r - o1 n ) ((pos "Ki%" ) ))
( "lookers"  n ( l u1 k @ z ) ((pos "Kj%" ) ))
( "lookers-on"  n ( l u2 k @ z - o1 n ) ((pos "Kj%" ) ))
( "looking"  v ( l u1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "4A" "4D" "6A" "8" )))
( "looking-glass"  n ( l u1 k i ng - g l aa s ) ((pos "K7%" ) ))
( "looking-glasses"  n ( l u1 k i ng - g l aa s i z ) ((pos "Kj%" ) ))
( "lookout"  n ( l u1 k au t ) ((pos "K6%" ) ))
( "lookouts"  n ( l u1 k au t s ) ((pos "Kj%" ) ))
( "looks"  v ( l u1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "2D" "3A" "4A" "4D" "6A" "8" )))
( "loom"  v ( l uu1 m ) ((pos "I0%" "K6%" ) (subcat "2C" )))
( "loomed"  v ( l uu1 m d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "looming"  v ( l uu1 m i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "looms"  v ( l uu1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2C" )))
( "loon"  n ( l uu1 n ) ((pos "K6$" ) ))
( "loonier"  j ( l uu1 n i@ r ) ((pos "Or%" ) ))
( "loonies"  n ( l uu1 n i z ) ((pos "Kj%" ) ))
( "looniest"  j ( l uu1 n i i s t ) ((pos "Os%" ) ))
( "loons"  n ( l uu1 n z ) ((pos "Kj$" ) ))
( "loony"  n ( l uu1 n ii ) ((pos "K8%" "OD%" ) ))
( "loonybin"  n ( l uu1 n i b i n ) ((pos "K6$" ) ))
( "loonybins"  n ( l uu1 n i b i n z ) ((pos "Kj$" ) ))
( "loop"  v ( l uu1 p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "loop-line"  n ( l uu1 p - l ai n ) ((pos "K6$" ) ))
( "loop-lines"  n ( l uu1 p - l ai n z ) ((pos "Kj$" ) ))
( "looped"  v ( l uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "loophole"  n ( l uu1 p h ou l ) ((pos "K6%" ) ))
( "loopholes"  n ( l uu1 p h ou l z ) ((pos "Kj%" ) ))
( "looping"  v ( l uu1 p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "loops"  v ( l uu1 p s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "loopy"  j ( l uu1 p ii ) ((pos "OA%" ) ))
( "loos"  n ( l uu1 z ) ((pos "Kj%" ) ))
( "loose"  v ( l uu1 s ) ((pos "H2%" "OB%" ) (subcat )))
( "loose-leaf"  j ( l uu1 s - l ii f ) ((pos "Oq%" ) ))
( "loosed"  v ( l uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "loosely"  a ( l uu1 s l ii ) ((pos "Pu%" ) ))
( "loosen"  v ( l uu1 s n ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "loosened"  v ( l uu1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "loosening"  v ( l uu1 s n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "loosens"  v ( l uu1 s n z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "looser"  j ( l uu1 s @ r ) ((pos "Or%" ) ))
( "looses"  v ( l uu1 s i z ) ((pos "Ha%" ) (subcat )))
( "loosest"  j ( l uu1 s i s t ) ((pos "Os%" ) ))
( "loosing"  v ( l uu1 s i ng ) ((pos "Hb%" ) (subcat )))
( "loot"  v ( l uu1 t ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "looted"  v ( l uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "looter"  n ( l uu1 t @ r ) ((pos "K6%" ) ))
( "looters"  n ( l uu1 t @ z ) ((pos "Kj%" ) ))
( "looting"  v ( l uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "loots"  v ( l uu1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "lop"  v ( l o1 p ) ((pos "J4%" ) (subcat "6A" )))
( "lop-eared"  j ( l o1 p - i@ d ) ((pos "OA$" ) ))
( "lop-ears"  n ( l o1 p - i@ z ) ((pos "Kj$" ) ))
( "lope"  v ( l ou1 p ) ((pos "I2%" "Ki%" ) (subcat "2A" )))
( "loped"  v ( l ou1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "lopes"  v ( l ou1 p s ) ((pos "Ia%" ) (subcat "2A" )))
( "loping"  v ( l ou1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "lopped"  v ( l o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "lopping"  v ( l o1 p i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "lops"  v ( l o1 p s ) ((pos "Ja%" ) (subcat "6A" )))
( "lopsided"  j ( l o1 p s ai1 d i d ) ((pos "OA%" ) ))
( "loquacious"  j ( l @1 k w ei1 sh @ s ) ((pos "OA%" ) ))
( "loquaciously"  a ( l @1 k w ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "loquaciousness"  n ( l @1 k w ei1 sh @ s n @ s ) ((pos "L@%" ) ))
( "loquacity"  n ( l @1 k w a1 s i t ii ) ((pos "L@$" ) ))
( "loquat"  n ( l ou1 k w o t ) ((pos "K6$" ) ))
( "loquats"  n ( l ou1 k w o t s ) ((pos "Kj$" ) ))
( "lor"  uh ( l oo1 r ) ((pos "W-$" ) ))
( "lord"  v ( l oo1 d ) ((pos "H0%" "K6%" ) ))
( "lorded"  v ( l oo1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "lording"  v ( l oo1 d i ng ) ((pos "Hb%" ) ))
( "lordless"  j ( l oo1 d l @ s ) ((pos "OA%" ) ))
( "lordlier"  j ( l oo1 d l i@ r ) ((pos "Or$" ) ))
( "lordliest"  j ( l oo1 d l i i s t ) ((pos "Os$" ) ))
( "lordliness"  n ( l oo1 d l i n @ s ) ((pos "L@%" ) ))
( "lordly"  j ( l oo1 d l ii ) ((pos "OD%" ) ))
( "lords"  v ( l oo1 d z ) ((pos "Ha%" "Kj%" ) ))
( "lordship"  n ( l oo1 d sh i p ) ((pos "M6%" ) ))
( "lordships"  n ( l oo1 d sh i p s ) ((pos "Mj%" ) ))
( "lore"  n ( l oo1 r ) ((pos "L@%" ) ))
( "lorgnette"  n ( l oo1 n y e1 t ) ((pos "K6$" ) ))
( "lorgnettes"  n ( l oo1 n y e1 t s ) ((pos "Kj$" ) ))
( "lorn"  j ( l oo1 n ) ((pos "OA$" ) ))
( "lorries"  n ( l o1 r i z ) ((pos "Kj%" ) ))
( "lorry"  n ( l o1 r ii ) ((pos "K8%" ) ))
( "lose"  v ( l uu1 z ) ((pos "J5%" ) (subcat "2A" "2B" "3A" "6A" "12C" )))
( "loser"  n ( l uu1 z @ r ) ((pos "K6%" ) ))
( "losers"  n ( l uu1 z @ z ) ((pos "Kj%" ) ))
( "loses"  v ( l uu1 z i z ) ((pos "Ja%" ) (subcat "2A" "2B" "3A" "6A" "12C" )))
( "losing"  v ( l uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "3A" "6A" "12C" )))
( "loss"  n ( l o1 s ) ((pos "M7%" ) ))
( "loss-leader"  n ( l o1 s - l ii d @ r ) ((pos "K6%" ) ))
( "loss-leaders"  n ( l o1 s - l ii d @ z ) ((pos "Kj%" ) ))
( "losses"  n ( l o1 s i z ) ((pos "Mj%" ) ))
( "lost"  v ( l o1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "3A" "6A" "12C" )))
( "lot"  n ( l o1 t ) ((pos "M6%" ) ))
( "loth"  j ( l ou1 th ) ((pos "OA$" ) ))
( "lotion"  n ( l ou1 sh @ n ) ((pos "M6%" ) ))
( "lotions"  n ( l ou1 sh @ n z ) ((pos "Mj%" ) ))
( "lots"  n ( l o1 t s ) ((pos "Mj%" ) ))
( "lotteries"  n ( l o1 t @ r i z ) ((pos "Kj%" ) ))
( "lottery"  n ( l o1 t @ r ii ) ((pos "K8%" ) ))
( "lotto"  n ( l o1 t ou ) ((pos "L@$" ) ))
( "lotus"  n ( l ou1 t @ s ) ((pos "K7%" ) ))
( "lotus-eater"  n ( l ou1 t @ s - ii t @ r ) ((pos "K6$" ) ))
( "lotus-eaters"  n ( l ou1 t @ s - ii t @ z ) ((pos "Kj$" ) ))
( "lotuses"  n ( l ou1 t @ s i z ) ((pos "Kj$" ) ))
( "loud"  j ( l au1 d ) ((pos "OC%" "Pu%" ) ))
( "loud-hailer"  n ( l au1 d - h ei1 l @ r ) ((pos "K6%" ) ))
( "loud-hailers"  n ( l au1 d - h ei1 l @ z ) ((pos "Kj%" ) ))
( "louder"  j ( l au1 d @ r ) ((pos "Or%" ) ))
( "loudest"  j ( l au1 d i s t ) ((pos "Os%" ) ))
( "loudly"  a ( l au1 d l ii ) ((pos "Pu%" ) ))
( "loudness"  n ( l au1 d n @ s ) ((pos "L@%" ) ))
( "loudspeaker"  n ( l au1 d s p ii1 k @ r ) ((pos "K6%" ) ))
( "loudspeakers"  n ( l au1 d s p ii1 k @ z ) ((pos "Kj%" ) ))
( "lough"  n ( l o1 k ) ((pos "K6$" ) ))
( "loughs"  n ( l o1 k s ) ((pos "Kj$" ) ))
( "lounge"  v ( l au1 n jh ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "lounge-chair"  n ( l au1 n jh - ch e@ r ) ((pos "K6%" ) ))
( "lounge-chairs"  n ( l au1 n jh - ch e@ z ) ((pos "Kj%" ) ))
( "lounge-lizard"  n ( l au1 n jh - l i z @ d ) ((pos "K6%" ) ))
( "lounge-lizards"  n ( l au1 n jh - l i z @ d z ) ((pos "Kj%" ) ))
( "lounge-suit"  n ( l au1 n jh - s uu t ) ((pos "K6%" ) ))
( "lounge-suits"  n ( l au1 n jh - s uu t s ) ((pos "Kj%" ) ))
( "lounged"  v ( l au1 n jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "lounger"  n ( l au1 n jh @ r ) ((pos "K6%" ) ))
( "loungers"  n ( l au1 n jh @ z ) ((pos "Kj%" ) ))
( "lounges"  v ( l au1 n jh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "lounging"  v ( l au1 n jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "lour"  v ( l au1 @ r ) ((pos "I0$" ) (subcat "2A" )))
( "loured"  v ( l au1 @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "louring"  v ( l au1 @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "louringly"  a ( l au1 @ r i ng l ii ) ((pos "Pu%" ) ))
( "lours"  v ( l au1 @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "louse"  n ( l au1 s ) ((pos "Ki%" ) ))
( "lousier"  j ( l au1 z i@ r ) ((pos "Or%" ) ))
( "lousiest"  j ( l au1 z i i s t ) ((pos "Os%" ) ))
( "lousy"  j ( l au1 z ii ) ((pos "OD%" ) ))
( "lout"  n ( l au1 t ) ((pos "K6%" ) ))
( "loutish"  j ( l au1 t i sh ) ((pos "OA%" ) ))
( "louts"  n ( l au1 t s ) ((pos "Kj%" ) ))
( "louvered"  j ( l uu1 v @ d ) ((pos "OA$" ) ))
( "louvre"  n ( l uu1 v @ r ) ((pos "K6$" ) ))
( "louvres"  n ( l uu1 v @ z ) ((pos "Kj$" ) ))
( "lovable"  j ( l uh1 v @ b l ) ((pos "OA%" ) ))
( "love"  v ( l uh1 v ) ((pos "H2*" "M6*" ) (subcat "6A" "6D" "7A" )))
( "love-affair"  n ( l uh1 v - @ f e@ r ) ((pos "K6%" ) ))
( "love-affairs"  n ( l uh1 v - @ f e@ z ) ((pos "Kj%" ) ))
( "love-child"  n ( l uh1 v - ch ai l d ) ((pos "Ki%" ) ))
( "love-children"  n ( l uh1 v - ch i l d r @ n ) ((pos "Kj$" ) ))
( "love-feast"  n ( l uh1 v - f ii s t ) ((pos "K6$" ) ))
( "love-feasts"  n ( l uh1 v - f ii s t s ) ((pos "Kj$" ) ))
( "love-knot"  n ( l uh1 v - n o t ) ((pos "K6%" ) ))
( "love-knots"  n ( l uh1 v - n o t s ) ((pos "Kj$" ) ))
( "love-letter"  n ( l uh1 v - l e t @ r ) ((pos "K6%" ) ))
( "love-letters"  n ( l uh1 v - l e t @ z ) ((pos "Kj%" ) ))
( "love-match"  n ( l uh1 v - m a ch ) ((pos "K7%" ) ))
( "love-matches"  n ( l uh1 v - m a ch i z ) ((pos "Kj$" ) ))
( "love-philtre"  n ( l uh1 v - f i l t @ r ) ((pos "K6$" ) ))
( "love-philtres"  n ( l uh1 v - f i l t @ z ) ((pos "Kj$" ) ))
( "love-potion"  n ( l uh1 v - p ou sh @ n ) ((pos "K6%" ) ))
( "love-potions"  n ( l uh1 v - p ou sh @ n z ) ((pos "Kj%" ) ))
( "love-seat"  n ( l uh1 v - s ii t ) ((pos "K6$" ) ))
( "love-seats"  n ( l uh1 v - s ii t s ) ((pos "Kj$" ) ))
( "love-song"  n ( l uh1 v - s o ng ) ((pos "K6%" ) ))
( "love-songs"  n ( l uh1 v - s o ng z ) ((pos "Kj%" ) ))
( "love-stories"  n ( l uh1 v - s t oo r i z ) ((pos "Kj%" ) ))
( "love-story"  n ( l uh1 v - s t oo r ii ) ((pos "K8%" ) ))
( "love-token"  n ( l uh1 v - t ou k @ n ) ((pos "K6%" ) ))
( "love-tokens"  n ( l uh1 v - t ou k @ n z ) ((pos "Kj%" ) ))
( "lovebird"  n ( l uh1 v b @@ d ) ((pos "K6%" ) ))
( "lovebirds"  n ( l uh1 v b @@ d z ) ((pos "Kj%" ) ))
( "loved"  v ( l uh1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" )))
( "loveless"  j ( l uh1 v l @ s ) ((pos "OA%" ) ))
( "lovelier"  j ( l uh1 v l i@ r ) ((pos "Or%" ) ))
( "loveliest"  j ( l uh1 v l i i s t ) ((pos "Os%" ) ))
( "loveliness"  n ( l uh1 v l i n @ s ) ((pos "L@%" ) ))
( "lovelorn"  j ( l uh1 v l oo n ) ((pos "OA%" ) ))
( "lovely"  j ( l uh1 v l ii ) ((pos "OD%" ) ))
( "lovemaking"  n ( l uh1 v m ei k i ng ) ((pos "L@%" ) ))
( "lover"  n ( l uh1 v @ r ) ((pos "K6%" ) ))
( "loverlike"  j ( l uh1 v @ l ai k ) ((pos "OA$" ) ))
( "lovers"  n ( l uh1 v @ z ) ((pos "Kj%" ) ))
( "loves"  v ( l uh1 v z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6D" "7A" )))
( "lovesick"  j ( l uh1 v s i k ) ((pos "OA%" ) ))
( "loving"  v ( l uh1 v i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "6D" "7A" )))
( "loving-cup"  n ( l uh1 v i ng - k uh p ) ((pos "K6%" ) ))
( "loving-cups"  n ( l uh1 v i ng - k uh p s ) ((pos "Kj%" ) ))
( "loving-kindness"  n ( l uh2 v i ng - k ai1 n d n @ s ) ((pos "L@%" ) ))
( "lovingly"  a ( l uh1 v i ng l ii ) ((pos "Pu%" ) ))
( "low"  v ( l ou1 ) ((pos "I0%" "K6%" "OC%" "Pu%" ) ))
( "low-down"  j ( l ou1 - d au1 n ) ((pos "OA%" ) ))
( "low-keyed"  j ( l ou1 - k ii1 d ) ((pos "OA%" ) ))
( "low-pitched"  j ( l ou1 - p i1 ch t ) ((pos "OA%" ) ))
( "low-relief"  n ( l ou2 - r i l ii1 f ) ((pos "M6%" ) ))
( "low-reliefs"  n ( l ou2 - r i l ii1 f s ) ((pos "Mj%" ) ))
( "low-spirited"  j ( l ou1 - s p i1 r i t i d ) ((pos "OA%" ) ))
( "lowborn"  j ( l ou1 b oo1 n ) ((pos "OA%" ) ))
( "lowbred"  j ( l ou1 b r e1 d ) ((pos "OA%" ) ))
( "lowbrow"  n ( l ou1 b r au ) ((pos "K6$" "OA%" ) ))
( "lowbrows"  n ( l ou1 b r au z ) ((pos "Kj$" ) ))
( "lowed"  v ( l ou1 d ) ((pos "Ic$" "Id$" ) ))
( "lower"  v ( l ou1 @ r ) ((pos "J0%" "Or%" "Pu%" ) (subcat "2A" "6A" )))
( "lower"  v ( l au1 @ r ) ((pos "I0$" ) (subcat "2A" )))
( "lowered"  v ( l ou1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "lowered"  v ( l au1 @ d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "lowering"  v ( l ou1 @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "lowering"  v ( l au1 @ r i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "lowermost"  j ( l ou1 @ m ou s t ) ((pos "OA%" ) ))
( "lowers"  v ( l ou1 @ z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "lowers"  v ( l au1 @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "lowest"  j ( l ou1 i s t ) ((pos "Os%" "Pu%" ) ))
( "lowing"  v ( l ou1 i ng ) ((pos "Ib$" ) ))
( "lowlander"  n ( l ou1 l @ n d @ r ) ((pos "K6%" ) ))
( "lowlanders"  n ( l ou1 l @ n d @ z ) ((pos "Kj%" ) ))
( "lowlands"  n ( l ou1 l @ n d z ) ((pos "Kj%" ) ))
( "lowlier"  j ( l ou1 l i@ r ) ((pos "Or%" ) ))
( "lowliest"  j ( l ou1 l i i s t ) ((pos "Os%" ) ))
( "lowliness"  n ( l ou1 l i n @ s ) ((pos "L@%" ) ))
( "lowly"  j ( l ou1 l ii ) ((pos "OD%" ) ))
( "lowness"  n ( l ou1 n e s ) ((pos "L@%" ) ))
( "lows"  v ( l ou1 z ) ((pos "Ia$" "Kj$" ) ))
( "loyal"  j ( l oi1 @ l ) ((pos "OE%" ) ))
( "loyalist"  n ( l oi1 @ l i s t ) ((pos "K6%" ) ))
( "loyalists"  n ( l oi1 @ l i s t s ) ((pos "Kj%" ) ))
( "loyaller"  j ( l oi1 @ l @ r ) ((pos "Or$" ) ))
( "loyallest"  j ( l oi1 @ l i s t ) ((pos "Os$" ) ))
( "loyally"  a ( l oi1 @ l ii ) ((pos "Pu%" ) ))
( "loyalties"  n ( l oi1 @ l t i z ) ((pos "Mj%" ) ))
( "loyalty"  n ( l oi1 @ l t ii ) ((pos "M8%" ) ))
( "lozenge"  n ( l o1 z i n jh ) ((pos "K6%" ) ))
( "lozenges"  n ( l o1 z i n jh i z ) ((pos "Kj%" ) ))
( "lubber"  n ( l uh1 b @ r ) ((pos "K6$" ) ))
( "lubberly"  j ( l uh1 b @ l ii ) ((pos "OA$" ) ))
( "lubbers"  n ( l uh1 b @ z ) ((pos "Kj$" ) ))
( "lubricant"  n ( l uu1 b r i k @ n t ) ((pos "M6%" ) ))
( "lubricants"  n ( l uu1 b r i k @ n t s ) ((pos "Mj%" ) ))
( "lubricate"  v ( l uu1 b r i k ei t ) ((pos "H2%" ) (subcat )))
( "lubricated"  v ( l uu1 b r i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lubricates"  v ( l uu1 b r i k ei t s ) ((pos "Ha%" ) (subcat )))
( "lubricating"  v ( l uu1 b r i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "lubrication"  n ( l uu2 b r i k ei1 sh @ n ) ((pos "M6%" ) ))
( "lubrications"  n ( l uu2 b r i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "lucent"  j ( l uu1 s n t ) ((pos "OA$" ) ))
( "lucerne"  n ( l uu1 s @@1 n ) ((pos "L@$" ) ))
( "lucid"  j ( l uu1 s i d ) ((pos "OA%" ) ))
( "lucidity"  n ( l uu1 s i1 d i t ii ) ((pos "L@%" ) ))
( "lucidly"  a ( l uu1 s i d l ii ) ((pos "Pu%" ) ))
( "luck"  n ( l uh1 k ) ((pos "L@%" ) ))
( "luckier"  j ( l uh1 k i@ r ) ((pos "Or%" ) ))
( "luckiest"  j ( l uh1 k i i s t ) ((pos "Os%" ) ))
( "luckily"  a ( l uh1 k i l ii ) ((pos "Pu%" ) ))
( "luckless"  j ( l uh1 k l @ s ) ((pos "OA%" ) ))
( "lucky"  j ( l uh1 k ii ) ((pos "OD%" ) ))
( "lucrative"  j ( l uu1 k r @ t i v ) ((pos "OA%" ) ))
( "lucre"  n ( l uu1 k @ r ) ((pos "L@%" ) ))
( "ludicrous"  j ( l uu1 d i k r @ s ) ((pos "OA%" ) ))
( "ludicrously"  a ( l uu1 d i k r @ s l ii ) ((pos "Pu%" ) ))
( "ludo"  n ( l uu1 d ou ) ((pos "L@%" ) ))
( "luff"  v ( l uh1 f ) ((pos "J0$" ) ))
( "luffed"  v ( l uh1 f t ) ((pos "Jc$" "Jd$" ) ))
( "luffing"  v ( l uh1 f i ng ) ((pos "Jb$" ) ))
( "luffs"  v ( l uh1 f s ) ((pos "Ja$" ) ))
( "lug"  v ( l uh1 g ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" )))
( "luge"  n ( l uu1 zh ) ((pos "K6$" ) ))
( "luges"  n ( l uu1 zh i z ) ((pos "Kj$" ) ))
( "luggage"  n ( l uh1 g i jh ) ((pos "L@%" ) ))
( "luggage-carrier"  n ( l uh1 g i jh - k a r i@ r ) ((pos "K6%" ) ))
( "luggage-carriers"  n ( l uh1 g i jh - k a r i@ z ) ((pos "Kj%" ) ))
( "luggage-rack"  n ( l uh1 g i jh - r a k ) ((pos "K6%" ) ))
( "luggage-racks"  n ( l uh1 g i jh - r a k s ) ((pos "Kj%" ) ))
( "luggage-van"  n ( l uh1 g i jh - v a n ) ((pos "K6%" ) ))
( "luggage-vans"  n ( l uh1 g i jh - v a n z ) ((pos "Kj%" ) ))
( "lugged"  v ( l uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "lugger"  n ( l uh1 g @ r ) ((pos "K6$" ) ))
( "luggers"  n ( l uh1 g @ z ) ((pos "Kj$" ) ))
( "lugging"  v ( l uh1 g i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "lugs"  v ( l uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "lugsail"  n ( l uh1 g s ei l ) ((pos "K6$" ) ))
( "lugsails"  n ( l uh1 g s ei l z ) ((pos "Kj$" ) ))
( "lugubrious"  j ( l @1 g uu1 b r i@ s ) ((pos "OA%" ) ))
( "lugubriously"  a ( l @1 g uu1 b r i@ s l ii ) ((pos "Pu%" ) ))
( "lugubriousness"  n ( l @1 g uu1 b r i@ s n @ s ) ((pos "L@%" ) ))
( "lukewarm"  j ( l uu2 k w oo1 m ) ((pos "OA%" ) ))
( "lukewarmly"  a ( l uu2 k w oo1 m l ii ) ((pos "Pu$" ) ))
( "lukewarmness"  n ( l uu2 k w oo1 m n @ s ) ((pos "L@$" ) ))
( "lull"  v ( l uh1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "lullabies"  n ( l uh1 l @ b ai z ) ((pos "Kj%" ) ))
( "lullaby"  n ( l uh1 l @ b ai ) ((pos "K8%" ) ))
( "lulled"  v ( l uh1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "lulling"  v ( l uh1 l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "lulls"  v ( l uh1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "lumbago"  n ( l uh1 m b ei1 g ou ) ((pos "L@%" ) ))
( "lumbar"  j ( l uh1 m b @ r ) ((pos "OA$" ) ))
( "lumber"  v ( l uh1 m b @ r ) ((pos "J0%" "L@%" ) (subcat "2C" "6A" "14" "15A" )))
( "lumber-mill"  n ( l uh1 m b @ - m i l ) ((pos "K6%" ) ))
( "lumber-mills"  n ( l uh1 m b @ - m i l z ) ((pos "Kj%" ) ))
( "lumbered"  v ( l uh1 m b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "lumbering"  v ( l uh1 m b @ r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" "15A" )))
( "lumberjack"  n ( l uh1 m b @ jh a k ) ((pos "K6%" ) ))
( "lumberjacks"  n ( l uh1 m b @ jh a k s ) ((pos "Kj%" ) ))
( "lumberman"  n ( l uh1 m b @ m a n ) ((pos "Ki$" ) ))
( "lumbermen"  n ( l uh1 m b @ m e n ) ((pos "Kj$" ) ))
( "lumberroom"  n ( l uh1 m b @ r uu m ) ((pos "K6%" ) ))
( "lumberrooms"  n ( l uh1 m b @ r uu m z ) ((pos "Kj%" ) ))
( "lumbers"  v ( l uh1 m b @ z ) ((pos "Ja$" ) (subcat "2C" "6A" "14" "15A" )))
( "lumberyard"  n ( l uh1 m b @ y aa d ) ((pos "K6%" ) ))
( "lumberyards"  n ( l uh1 m b @ y aa d z ) ((pos "Kj%" ) ))
( "luminaries"  n ( l uu1 m i n @ r i z ) ((pos "Kj%" ) ))
( "luminary"  n ( l uu1 m i n @ r ii ) ((pos "K8%" ) ))
( "luminosity"  n ( l uu2 m i n o1 s i t ii ) ((pos "L@%" ) ))
( "luminous"  j ( l uu1 m i n @ s ) ((pos "OA%" ) ))
( "lumme"  uh ( l uh1 m ii ) ((pos "W-$" ) ))
( "lummy"  uh ( l uh1 m ii ) ((pos "W-$" ) ))
( "lump"  v ( l uh1 m p ) ((pos "H0%" "K6%" ) (subcat "2A" "6A" )))
( "lumped"  v ( l uh1 m p t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "lumpier"  j ( l uh1 m p i@ r ) ((pos "Or%" ) ))
( "lumpiest"  j ( l uh1 m p i i s t ) ((pos "Os%" ) ))
( "lumping"  v ( l uh1 m p i ng ) ((pos "Hb%" ) (subcat "2A" "6A" )))
( "lumpish"  j ( l uh1 m p i sh ) ((pos "OA$" ) ))
( "lumps"  v ( l uh1 m p s ) ((pos "Ha%" "Kj%" ) (subcat "2A" "6A" )))
( "lumpy"  j ( l uh1 m p ii ) ((pos "OD%" ) ))
( "lunacies"  n ( l uu1 n @ s i z ) ((pos "Mj$" ) ))
( "lunacy"  n ( l uu1 n @ s ii ) ((pos "M8%" ) ))
( "lunar"  j ( l uu1 n @ r ) ((pos "OA%" ) ))
( "lunatic"  n ( l uu1 n @ t i k ) ((pos "K6%" ) ))
( "lunatics"  n ( l uu1 n @ t i k s ) ((pos "Kj%" ) ))
( "lunch"  v ( l uh1 n ch ) ((pos "J1%" "K7%" ) ))
( "lunched"  v ( l uh1 n ch t ) ((pos "Jc%" "Jd%" ) ))
( "luncheon"  n ( l uh1 n ch @ n ) ((pos "K6%" ) ))
( "luncheons"  n ( l uh1 n ch @ n z ) ((pos "Kj%" ) ))
( "lunches"  v ( l uh1 n ch i z ) ((pos "Ja%" "Kj%" ) ))
( "lunching"  v ( l uh1 n ch i ng ) ((pos "Jb%" ) ))
( "lunchtime"  n ( l uh1 n ch t ai m ) ((pos "M6%" ) ))
( "lunchtimes"  n ( l uh1 n ch t ai m z ) ((pos "Mj%" ) ))
( "lung"  n ( l uh1 ng ) ((pos "K6%" ) ))
( "lung-power"  n ( l uh1 ng - p au @ r ) ((pos "L@$" ) ))
( "lunge"  v ( l uh1 n jh ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "lunged"  v ( l uh1 n jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "lunges"  v ( l uh1 n jh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "lunging"  v ( l uh1 n jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "lungs"  n ( l uh1 ng z ) ((pos "Kj%" ) ))
( "lupin"  n ( l uu1 p i n ) ((pos "K6%" ) ))
( "lupins"  n ( l uu1 p i n z ) ((pos "Kj%" ) ))
( "lurch"  v ( l @@1 ch ) ((pos "I1%" "K7%" ) (subcat )))
( "lurched"  v ( l @@1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "lurcher"  n ( l @@1 ch @ r ) ((pos "K6$" ) ))
( "lurchers"  n ( l @@1 ch @ z ) ((pos "Kj$" ) ))
( "lurches"  v ( l @@1 ch i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "lurching"  v ( l @@1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "lure"  v ( l u@1 r ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "lured"  v ( l u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "lures"  v ( l u@1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "lurid"  j ( l u@1 r i d ) ((pos "OA%" ) ))
( "luridly"  a ( l u@1 r i d l ii ) ((pos "Pu%" ) ))
( "luridness"  n ( l u@1 r i d n @ s ) ((pos "L@$" ) ))
( "luring"  v ( l u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "lurk"  v ( l @@1 k ) ((pos "I0%" ) (subcat )))
( "lurked"  v ( l @@1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "lurking"  v ( l @@1 k i ng ) ((pos "Ib%" ) (subcat )))
( "lurking-place"  n ( l @@1 k i ng - p l ei s ) ((pos "K6$" ) ))
( "lurking-places"  n ( l @@1 k i ng - p l ei s i z ) ((pos "Kj$" ) ))
( "lurks"  v ( l @@1 k s ) ((pos "Ia%" ) (subcat )))
( "luscious"  j ( l uh1 sh @ s ) ((pos "OA%" ) ))
( "lusciously"  a ( l uh1 sh @ s l ii ) ((pos "Pu%" ) ))
( "lusciousness"  n ( l uh1 sh @ s n @ s ) ((pos "L@%" ) ))
( "lush"  n ( l uh1 sh ) ((pos "K7$" "OC%" ) ))
( "lusher"  j ( l uh1 sh @ r ) ((pos "Or%" ) ))
( "lushes"  n ( l uh1 sh i z ) ((pos "Kj$" ) ))
( "lushest"  j ( l uh1 sh i s t ) ((pos "Os%" ) ))
( "lust"  v ( l uh1 s t ) ((pos "I0%" "M6%" ) (subcat )))
( "lusted"  v ( l uh1 s t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "lustful"  j ( l uh1 s t f @ l ) ((pos "OA%" ) ))
( "lustfully"  a ( l uh1 s t f @ l ii ) ((pos "Pu%" ) ))
( "lustier"  j ( l uh1 s t i@ r ) ((pos "Or%" ) ))
( "lustiest"  j ( l uh1 s t i i s t ) ((pos "Os%" ) ))
( "lustily"  a ( l uh1 s t i l ii ) ((pos "Pu%" ) ))
( "lusting"  v ( l uh1 s t i ng ) ((pos "Ib%" ) (subcat )))
( "lustre"  n ( l uh1 s t @ r ) ((pos "L@%" ) ))
( "lustrous"  j ( l uh1 s t r @ s ) ((pos "OA%" ) ))
( "lusts"  v ( l uh1 s t s ) ((pos "Ia%" "Mj%" ) (subcat )))
( "lusty"  j ( l uh1 s t ii ) ((pos "OD%" ) ))
( "lutanist"  n ( l uu1 t @ n i s t ) ((pos "K6%" ) ))
( "lutanists"  n ( l uu1 t @ n i s t s ) ((pos "Kj%" ) ))
( "lute"  n ( l uu1 t ) ((pos "K6%" ) ))
( "lutenist"  n ( l uu1 t @ n i s t ) ((pos "K6%" ) ))
( "lutenists"  n ( l uu1 t @ n i s t s ) ((pos "Kj%" ) ))
( "lutes"  n ( l uu1 t s ) ((pos "Kj%" ) ))
( "luxe"  j ( l uh1 k s ) ((pos "OA$" ) ))
( "luxuriance"  n ( l uh1 g zh u@1 r i@ n s ) ((pos "L@%" ) ))
( "luxuriant"  j ( l uh1 g zh u@1 r i@ n t ) ((pos "OA%" ) ))
( "luxuriantly"  a ( l uh1 g zh u@1 r i@ n t l ii ) ((pos "Pu%" ) ))
( "luxuriate"  v ( l uh1 g zh u@1 r i ei t ) ((pos "I2%" ) (subcat )))
( "luxuriated"  v ( l uh1 g zh u@1 r i ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "luxuriates"  v ( l uh1 g zh u@1 r i ei t s ) ((pos "Ia%" ) (subcat )))
( "luxuriating"  v ( l uh1 g zh u@1 r i ei t i ng ) ((pos "Ib%" ) (subcat )))
( "luxuries"  n ( l uh1 k sh @ r i z ) ((pos "Mj%" ) ))
( "luxurious"  j ( l uh1 g zh u@1 r i@ s ) ((pos "OA%" ) ))
( "luxuriously"  a ( l uh1 g zh u@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "luxury"  n ( l uh1 k sh @ r ii ) ((pos "M8%" ) ))
( "lyc_ee"  n ( l ii1 s ei ) ((pos "K6$" ) ))
( "lyc_ees"  n ( l ii1 s ei z ) ((pos "Kj$" ) ))
( "lyceum"  n ( l ai1 s ii1 @ m ) ((pos "K6$" ) ))
( "lyceums"  n ( l ai1 s ii1 @ m z ) ((pos "Kj$" ) ))
( "lychee"  n ( l ai1 ch ii ) ((pos "K6%" ) ))
( "lychees"  n ( l ai1 ch ii z ) ((pos "Kj%" ) ))
( "lychgate"  n ( l i1 ch g ei t ) ((pos "K6%" ) ))
( "lychgates"  n ( l i1 ch g ei t s ) ((pos "Kj%" ) ))
( "lye"  n ( l ai1 ) ((pos "K6$" ) ))
( "lyes"  n ( l ai1 z ) ((pos "Kj$" ) ))
( "lying"  v ( l ai1 i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "2D" )))
( "lying-in"  j ( l ai2 i ng - i1 n ) ((pos "OA%" ) ))
( "lymph"  n ( l i1 m f ) ((pos "L@%" ) ))
( "lymphatic"  j ( l i1 m f a1 t i k ) ((pos "OA%" ) ))
( "lynch"  v ( l i1 n ch ) ((pos "H1%" "L@%" ) (subcat )))
( "lynched"  v ( l i1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "lynches"  v ( l i1 n ch i z ) ((pos "Ha%" ) (subcat )))
( "lynching"  v ( l i1 n ch i ng ) ((pos "Hb%" ) (subcat )))
( "lynchpin"  n ( l i1 n ch p i n ) ((pos "K6%" ) ))
( "lynchpins"  n ( l i1 n ch p i n z ) ((pos "Kj%" ) ))
( "lynx"  n ( l i1 ng k s ) ((pos "K7%" ) ))
( "lynx-eyed"  j ( l i1 ng k s - ai d ) ((pos "OA%" ) ))
( "lynxes"  n ( l i1 ng k s i z ) ((pos "Kj%" ) ))
( "lyre"  n ( l ai1 @ r ) ((pos "K6%" ) ))
( "lyre-bird"  n ( l ai1 @ - b @@ d ) ((pos "K6%" ) ))
( "lyre-birds"  n ( l ai1 @ - b @@ d z ) ((pos "Kj%" ) ))
( "lyres"  n ( l ai1 @ z ) ((pos "Kj%" ) ))
( "lyric"  n ( l i1 r i k ) ((pos "K6%" "OA%" ) ))
( "lyrical"  j ( l i1 r i k l ) ((pos "OA%" ) ))
( "lyrically"  a ( l i1 r i k l ii ) ((pos "Pu%" ) ))
( "lyricism"  n ( l i1 r i s i z m ) ((pos "L@%" ) ))
( "lyricist"  n ( l i1 r i s i s t ) ((pos "K6%" ) ))
( "lyricists"  n ( l i1 r i s i s t s ) ((pos "Kj%" ) ))
( "lyrics"  n ( l i1 r i k s ) ((pos "Kj%" ) ))
( "lysol"  n ( l ai1 s o l ) ((pos "L@$" ) ))
( "m"  n ( e1 m ) ((pos "Ki$" ) ))
( "m's"  n ( e1 m z ) ((pos "Kj$" ) ))
( "m^el_ee"  n ( m e1 l ei ) ((pos "K6%" ) ))
( "m^el_ees"  n ( m e1 l ei z ) ((pos "Kj%" ) ))
( "m_elange"  n ( m ei1 l aa n zh ) ((pos "K6$" ) ))
( "m_elanges"  n ( m ei1 l aa n zh i z ) ((pos "Kj$" ) ))
( "m_enage"  n ( m ei1 n aa1 zh ) ((pos "K6%" ) ))
( "m_enages"  n ( m ei1 n aa1 zh i z ) ((pos "Kj%" ) ))
( "m_esalliance"  n ( m ei2 z a1 l i aa n s ) ((pos "K6$" ) ))
( "m_esalliances"  n ( m ei2 z a1 l i aa n s i z ) ((pos "Kj$" ) ))
( "m_etier"  n ( m e1 t i ei ) ((pos "K6%" ) ))
( "m_etiers"  n ( m e1 t i ei z ) ((pos "Kj%" ) ))
( "ma"  n ( m aa1 ) ((pos "Ki%" ) ))
( "ma'am"  n ( m a1 m ) ((pos "Ki%" ) ))
( "mac"  n ( m a1 k ) ((pos "K6%" ) ))
( "macabre"  j ( m @1 k aa1 b r @ ) ((pos "OA%" ) ))
( "macadam"  n ( m @1 k a1 d @ m ) ((pos "L@%" ) ))
( "macadamize"  v ( m @1 k a1 d @ m ai z ) ((pos "H2$" ) ))
( "macadamized"  v ( m @1 k a1 d @ m ai z d ) ((pos "Hc$" "Hd$" ) ))
( "macadamizes"  v ( m @1 k a1 d @ m ai z i z ) ((pos "Ha$" ) ))
( "macadamizing"  v ( m @1 k a1 d @ m ai z i ng ) ((pos "Hb$" ) ))
( "macaroni"  n ( m a2 k @ r ou1 n ii ) ((pos "L@%" ) ))
( "macaroon"  n ( m a2 k @ r uu1 n ) ((pos "K6%" ) ))
( "macaroons"  n ( m a2 k @ r uu1 n z ) ((pos "Kj%" ) ))
( "macaw"  n ( m @1 k oo1 ) ((pos "K6%" ) ))
( "macaws"  n ( m @1 k oo1 z ) ((pos "Kj%" ) ))
( "mace"  n ( m ei1 s ) ((pos "M6%" ) ))
( "mace-bearer"  n ( m ei1 s - b e@ r @ r ) ((pos "K6%" ) ))
( "mace-bearers"  n ( m ei1 s - b e@ r @ z ) ((pos "Kj%" ) ))
( "macerate"  v ( m a1 s @ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "macerated"  v ( m a1 s @ r ei t i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "macerates"  v ( m a1 s @ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "macerating"  v ( m a1 s @ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "maces"  n ( m ei1 s i z ) ((pos "Mj%" ) ))
( "machete"  n ( m @1 ch e1 t ii ) ((pos "K6%" ) ))
( "machetes"  n ( m @1 ch e1 t i z ) ((pos "Kj%" ) ))
( "machiavellian"  j ( m a2 k i@ v e1 l i@ n ) ((pos "OA%" ) ))
( "machination"  n ( m a2 k i n ei1 sh @ n ) ((pos "M6%" ) ))
( "machinations"  n ( m a2 k i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "machine"  v ( m @1 sh ii1 n ) ((pos "H2%" "K6%" ) (subcat )))
( "machine-gun"  n ( m @1 sh ii1 n - g uh n ) ((pos "K6%" ) ))
( "machine-guns"  n ( m @1 sh ii1 n - g uh n z ) ((pos "Kj%" ) ))
( "machine-made"  j ( m @1 sh ii1 n - m ei d ) ((pos "OA%" ) ))
( "machined"  v ( m @1 sh ii1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "machinery"  n ( m @1 sh ii1 n @ r ii ) ((pos "L@%" ) ))
( "machines"  v ( m @1 sh ii1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "machining"  v ( m @1 sh ii1 n i ng ) ((pos "Hb%" ) (subcat )))
( "machinist"  n ( m @1 sh ii1 n i s t ) ((pos "K6%" ) ))
( "machinists"  n ( m @1 sh ii1 n i s t s ) ((pos "Kj%" ) ))
( "machismo"  n ( m @1 ch i1 z m ou ) ((pos "L@%" ) ))
( "mackerel"  n ( m a1 k r @ l ) ((pos "M9%" ) ))
( "mackintosh"  n ( m a1 k i n t o sh ) ((pos "K7%" ) ))
( "mackintoshes"  n ( m a1 k i n t o sh i z ) ((pos "Kj%" ) ))
( "macrobiotic"  j ( m a2 k r ou b ai o1 t i k ) ((pos "OA%" ) ))
( "macrocosm"  n ( m a1 k r ou k o z @ m ) ((pos "K6$" ) ))
( "macrocosms"  n ( m a1 k r ou k o z @ m z ) ((pos "Kj$" ) ))
( "macs"  n ( m a1 k s ) ((pos "Kj%" ) ))
( "mad"  j ( m a1 d ) ((pos "OE%" ) ))
( "madam"  n ( m a1 d @ m ) ((pos "K6%" ) ))
( "madams"  n ( m a1 d @ m z ) ((pos "Kj$" ) ))
( "madcap"  n ( m a1 d k a p ) ((pos "K6$" ) ))
( "madcaps"  n ( m a1 d k a p s ) ((pos "Kj$" ) ))
( "madden"  v ( m a1 d n ) ((pos "H0%" ) (subcat )))
( "maddened"  v ( m a1 d n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "maddening"  v ( m a1 d n i ng ) ((pos "Hb%" ) (subcat )))
( "maddens"  v ( m a1 d n z ) ((pos "Ha%" ) (subcat )))
( "madder"  n ( m a1 d @ r ) ((pos "L@$" "Or%" ) ))
( "maddest"  j ( m a1 d i s t ) ((pos "Os%" ) ))
( "made"  v ( m ei1 d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2C" "2D" "3A" "6A" "12A" "12B" "13A" "13B" "14" "15B" "16A" "18B" "22" "23" "24A" )))
( "madhouse"  n ( m a1 d h au s ) ((pos "K6%" ) ))
( "madhouses"  n ( m a1 d h au z i z ) ((pos "Kj$" ) ))
( "madly"  a ( m a1 d l ii ) ((pos "Pu%" ) ))
( "madman"  n ( m a1 d m @ n ) ((pos "Ki%" ) ))
( "madmen"  n ( m a1 d m @ n ) ((pos "Kj%" ) ))
( "madness"  n ( m a1 d n @ s ) ((pos "L@%" ) ))
( "madrigal"  n ( m a1 d r i g l ) ((pos "K6%" ) ))
( "madrigals"  n ( m a1 d r i g l z ) ((pos "Kj%" ) ))
( "madwoman"  n ( m a1 d w u m @ n ) ((pos "Ki%" ) ))
( "madwomen"  n ( m a1 d w i m i n ) ((pos "Kj%" ) ))
( "maelstrom"  n ( m ei1 l s t r @ m ) ((pos "K6%" ) ))
( "maelstroms"  n ( m ei1 l s t r @ m z ) ((pos "Kj$" ) ))
( "maenad"  n ( m ii1 n a d ) ((pos "K6$" ) ))
( "maenads"  n ( m ii1 n a d z ) ((pos "Kj$" ) ))
( "maestri"  n ( m ai1 s t r ii ) ((pos "Kj$" ) ))
( "maestro"  n ( m ai1 s t r ou ) ((pos "K6%" ) ))
( "maestros"  n ( m ai1 s t r ou z ) ((pos "Kj%" ) ))
( "maffick"  v ( m a1 f i k ) ((pos "I0$" ) ))
( "mafficked"  v ( m a1 f i k t ) ((pos "Ic$" "Id$" ) ))
( "mafficking"  v ( m a1 f i k i ng ) ((pos "Ib$" ) ))
( "mafficks"  v ( m a1 f i k s ) ((pos "Ia$" ) ))
( "mag"  n ( m a1 g ) ((pos "K6%" ) ))
( "magazine"  n ( m a2 g @ z ii1 n ) ((pos "K6%" ) ))
( "magazines"  n ( m a2 g @ z ii1 n z ) ((pos "Kj%" ) ))
( "magenta"  n ( m @1 jh e1 n t @ ) ((pos "L@%" "OA%" ) ))
( "maggot"  n ( m a1 g @ t ) ((pos "K6%" ) ))
( "maggots"  n ( m a1 g @ t s ) ((pos "Kj%" ) ))
( "maggoty"  j ( m a1 g @ t ii ) ((pos "OA%" ) ))
( "magic"  n ( m a1 jh i k ) ((pos "L@%" "OA%" ) ))
( "magical"  j ( m a1 jh i k l ) ((pos "OA%" ) ))
( "magically"  a ( m a1 jh i k l ii ) ((pos "Pu%" ) ))
( "magician"  n ( m @1 jh i1 sh @ n ) ((pos "K6%" ) ))
( "magicians"  n ( m @1 jh i1 sh @ n z ) ((pos "Kj%" ) ))
( "magisterial"  j ( m a2 jh i s t i@1 r i@ l ) ((pos "OA%" ) ))
( "magisterially"  a ( m a2 jh i s t i@1 r i@ l ii ) ((pos "Pu%" ) ))
( "magistracies"  n ( m a1 jh i s t r @ s i z ) ((pos "Kj$" ) ))
( "magistracy"  n ( m a1 jh i s t r @ s ii ) ((pos "K8$" ) ))
( "magistrate"  n ( m a1 jh i s t r ei t ) ((pos "K6%" ) ))
( "magistrates"  n ( m a1 jh i s t r ei t s ) ((pos "Kj%" ) ))
( "magnanimities"  n ( m a2 g n @ n i1 m i t i z ) ((pos "Mj$" ) ))
( "magnanimity"  n ( m a2 g n @ n i1 m i t ii ) ((pos "M8%" ) ))
( "magnanimous"  j ( m a1 g n a1 n i m @ s ) ((pos "OA%" ) ))
( "magnanimously"  a ( m a1 g n a1 n i m @ s l ii ) ((pos "Pu%" ) ))
( "magnate"  n ( m a1 g n ei t ) ((pos "K6%" ) ))
( "magnates"  n ( m a1 g n ei t s ) ((pos "Kj%" ) ))
( "magnesia"  n ( m a1 g n ii1 sh @ ) ((pos "L@$" ) ))
( "magnesium"  n ( m a1 g n ii1 z i@ m ) ((pos "L@%" ) ))
( "magnet"  n ( m a1 g n i t ) ((pos "K6%" ) ))
( "magnetic"  j ( m a1 g n e1 t i k ) ((pos "OA%" ) ))
( "magnetically"  a ( m a1 g n e1 t i k l ii ) ((pos "Pu%" ) ))
( "magnetism"  n ( m a1 g n i t i z @ m ) ((pos "L@%" ) ))
( "magnetization"  n ( m a2 g n i t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "magnetize"  v ( m a1 g n i t ai z ) ((pos "H2%" ) (subcat )))
( "magnetized"  v ( m a1 g n i t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "magnetizes"  v ( m a1 g n i t ai z i z ) ((pos "Ha%" ) (subcat )))
( "magnetizing"  v ( m a1 g n i t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "magneto"  n ( m a1 g n ii1 t ou ) ((pos "K6$" ) ))
( "magnetos"  n ( m a1 g n ii1 t ou z ) ((pos "Kj$" ) ))
( "magnets"  n ( m a1 g n i t s ) ((pos "Kj%" ) ))
( "magnification"  n ( m a2 g n i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "magnifications"  n ( m a2 g n i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "magnificence"  n ( m a1 g n i1 f i s n s ) ((pos "L@%" ) ))
( "magnificent"  j ( m a1 g n i1 f i s n t ) ((pos "OA%" ) ))
( "magnificently"  a ( m a1 g n i1 f i s n t l ii ) ((pos "Pu%" ) ))
( "magnified"  v ( m a1 g n i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "magnifier"  n ( m a1 g n i f ai @ r ) ((pos "K6%" ) ))
( "magnifiers"  n ( m a1 g n i f ai @ z ) ((pos "Kj%" ) ))
( "magnifies"  v ( m a1 g n i f ai z ) ((pos "Ha%" ) (subcat )))
( "magnify"  v ( m a1 g n i f ai ) ((pos "H3%" ) (subcat )))
( "magnifying"  v ( m a1 g n i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "magniloquence"  n ( m a1 g n i1 l @ k w @ n s ) ((pos "L@$" ) ))
( "magniloquent"  j ( m a1 g n i1 l @ k w @ n t ) ((pos "OA$" ) ))
( "magniloquently"  a ( m a1 g n i1 l @ k w @ n t l ii ) ((pos "Pu$" ) ))
( "magnitude"  n ( m a1 g n i t y uu d ) ((pos "L@%" ) ))
( "magnolia"  n ( m a1 g n ou1 l i@ ) ((pos "K6%" ) ))
( "magnolias"  n ( m a1 g n ou1 l i@ z ) ((pos "Kj%" ) ))
( "magnum"  n ( m a1 g n @ m ) ((pos "K6%" ) ))
( "magnum opus"  n ( m a2 g n @ m - ou1 p @ s ) ((pos "Ki$" ) ))
( "magnums"  n ( m a1 g n @ m z ) ((pos "Kj$" ) ))
( "magpie"  n ( m a1 g p ai ) ((pos "K6%" ) ))
( "magpies"  n ( m a1 g p ai z ) ((pos "Kj%" ) ))
( "mags"  n ( m a1 g z ) ((pos "Kj%" ) ))
( "mahjong"  n ( m aa2 jh o1 ng ) ((pos "L@%" ) ))
( "mahoganies"  n ( m @1 h o1 g @ n i z ) ((pos "Mj$" ) ))
( "mahogany"  n ( m @1 h o1 g @ n ii ) ((pos "M8%" ) ))
( "mahout"  n ( m @1 h au1 t ) ((pos "K6$" ) ))
( "mahouts"  n ( m @1 h au1 t s ) ((pos "Kj$" ) ))
( "maid"  n ( m ei1 d ) ((pos "K6%" ) ))
( "maiden"  n ( m ei1 d n ) ((pos "K6%" "OA%" ) ))
( "maidenhair"  n ( m ei1 d n h e@ r ) ((pos "K6%" ) ))
( "maidenhairs"  n ( m ei1 d n h e@ z ) ((pos "Kj$" ) ))
( "maidenhead"  n ( m ei1 d n h e d ) ((pos "L@%" ) ))
( "maidenhood"  n ( m ei1 d n h u d ) ((pos "K6%" ) ))
( "maidenhoods"  n ( m ei1 d n h u d z ) ((pos "Kj$" ) ))
( "maidenlike"  j ( m ei1 d n l ai k ) ((pos "OA$" ) ))
( "maidenly"  j ( m ei1 d n l ii ) ((pos "OA%" ) ))
( "maidens"  n ( m ei1 d n z ) ((pos "Kj%" ) ))
( "maids"  n ( m ei1 d z ) ((pos "Kj%" ) ))
( "maidservant"  n ( m ei1 d s @@ v n t ) ((pos "K6%" ) ))
( "maidservants"  n ( m ei1 d s @@ v n t s ) ((pos "Kj%" ) ))
( "mail"  v ( m ei1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "mail-train"  n ( m ei1 l - t r ei n ) ((pos "K6%" ) ))
( "mail-trains"  n ( m ei1 l - t r ei n z ) ((pos "Kj%" ) ))
( "mailbag"  n ( m ei1 l b a g ) ((pos "K6%" ) ))
( "mailbags"  n ( m ei1 l b a g z ) ((pos "Kj%" ) ))
( "mailboat"  n ( m ei1 l b ou t ) ((pos "K6%" ) ))
( "mailboats"  n ( m ei1 l b ou t s ) ((pos "Kj%" ) ))
( "mailbox"  n ( m ei1 l b o k s ) ((pos "K7%" ) ))
( "mailboxes"  n ( m ei1 l b o k s i z ) ((pos "Kj%" ) ))
( "mailed"  v ( m ei1 l d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "mailing"  v ( m ei1 l i ng ) ((pos "Hb%" ) (subcat )))
( "mailing-card"  n ( m ei1 l i ng - k aa d ) ((pos "K6%" ) ))
( "mailing-cards"  n ( m ei1 l i ng - k aa d z ) ((pos "Kj%" ) ))
( "mailing-list"  n ( m ei1 l i ng - l i s t ) ((pos "K6%" ) ))
( "mailing-lists"  n ( m ei1 l i ng - l i s t s ) ((pos "Kj%" ) ))
( "mailman"  n ( m ei1 l m a n ) ((pos "Ki%" ) ))
( "mailmen"  n ( m ei1 l m e n ) ((pos "Kj%" ) ))
( "mails"  v ( m ei1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "maim"  v ( m ei1 m ) ((pos "H0%" ) (subcat )))
( "maimed"  v ( m ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "maiming"  v ( m ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "maims"  v ( m ei1 m z ) ((pos "Ha%" ) (subcat )))
( "main"  n ( m ei1 n ) ((pos "K6%" "OA%" ) ))
( "mainland"  n ( m ei1 n l a n d ) ((pos "K6%" ) ))
( "mainlands"  n ( m ei1 n l a n d z ) ((pos "Kj$" ) ))
( "mainly"  a ( m ei1 n l ii ) ((pos "Pu%" ) ))
( "mainmast"  n ( m ei1 n m aa s t ) ((pos "K6%" ) ))
( "mainmasts"  n ( m ei1 n m aa s t s ) ((pos "Kj$" ) ))
( "mains"  n ( m ei1 n z ) ((pos "Kj%" ) ))
( "mainspring"  n ( m ei1 n s p r i ng ) ((pos "K6%" ) ))
( "mainsprings"  n ( m ei1 n s p r i ng z ) ((pos "Kj$" ) ))
( "mainstay"  n ( m ei1 n s t ei ) ((pos "K6%" ) ))
( "mainstays"  n ( m ei1 n s t ei z ) ((pos "Kj$" ) ))
( "mainstream"  n ( m ei1 n s t r ii m ) ((pos "Ki%" ) ))
( "maintain"  v ( m ei1 n t ei1 n ) ((pos "H0%" ) (subcat "6A" "9" )))
( "maintainable"  j ( m ei1 n t ei1 n @ b l ) ((pos "OA%" ) ))
( "maintained"  v ( m ei1 n t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "maintaining"  v ( m ei1 n t ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "maintains"  v ( m ei1 n t ei1 n z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "maintenance"  n ( m ei1 n t @ n @ n s ) ((pos "L@%" ) ))
( "maisonnette"  n ( m ei2 z @ n e1 t ) ((pos "K6%" ) ))
( "maisonnettes"  n ( m ei2 z @ n e1 t s ) ((pos "Kj%" ) ))
( "maize"  n ( m ei1 z ) ((pos "L@%" ) ))
( "majestic"  j ( m @1 jh e1 s t i k ) ((pos "OA%" ) ))
( "majestically"  a ( m @1 jh e1 s t i k l ii ) ((pos "Pu%" ) ))
( "majesties"  n ( m a1 jh @ s t i z ) ((pos "Mj%" ) ))
( "majesty"  n ( m a1 jh @ s t ii ) ((pos "M8%" ) ))
( "majolica"  n ( m @1 jh o1 l i k @ ) ((pos "L@$" ) ))
( "major"  v ( m ei1 jh @ r ) ((pos "I0%" "K6%" "OA%" ) (subcat )))
( "major-domo"  n ( m ei2 jh @ - d ou1 m ou ) ((pos "K6$" ) ))
( "major-domos"  n ( m ei2 jh @ - d ou1 m ou z ) ((pos "Kj$" ) ))
( "major-general"  n ( m ei2 jh @ - jh e1 n r @ l ) ((pos "K6%" ) ))
( "major-generals"  n ( m ei2 jh @ - jh e1 n r @ l z ) ((pos "Kj%" ) ))
( "majored"  v ( m ei1 jh @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "majoring"  v ( m ei1 jh @ r i ng ) ((pos "Ib%" ) (subcat )))
( "majorities"  n ( m @1 jh o1 r i t i z ) ((pos "Kj%" ) ))
( "majority"  n ( m @1 jh o1 r i t ii ) ((pos "K8%" ) ))
( "majors"  v ( m ei1 jh @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "make"  v ( m ei1 k ) ((pos "J5*" "M6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12A" "12B" "13A" "13B" "14" "15B" "16A" "18B" "22" "23" "24A" )))
( "make-believe"  n ( m ei1 k - b i l ii v ) ((pos "M6%" ) ))
( "make-believes"  n ( m ei1 k - b i l ii v z ) ((pos "Mj$" ) ))
( "make-up"  n ( m ei1 k - uh p ) ((pos "M6%" ) ))
( "make-ups"  n ( m ei1 k - uh p s ) ((pos "Mj$" ) ))
( "maker"  n ( m ei1 k @ r ) ((pos "K6%" ) ))
( "makers"  n ( m ei1 k @ z ) ((pos "Kj%" ) ))
( "makes"  v ( m ei1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12A" "12B" "13A" "13B" "14" "15B" "16A" "18B" "22" "23" "24A" )))
( "makeshift"  n ( m ei1 k sh i f t ) ((pos "K6%" ) ))
( "makeshifts"  n ( m ei1 k sh i f t s ) ((pos "Kj$" ) ))
( "makeweight"  n ( m ei1 k w ei t ) ((pos "K6%" ) ))
( "makeweights"  n ( m ei1 k w ei t s ) ((pos "Kj$" ) ))
( "making"  v ( m ei1 k i ng ) ((pos "Jb*" "K6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12A" "12B" "13A" "13B" "14" "15B" "16A" "18B" "22" "23" "24A" )))
( "makings"  n ( m ei1 k i ng z ) ((pos "Kj%" ) ))
( "malacca"  n ( m @1 l a1 k @ ) ((pos "L@$" ) ))
( "malachite"  n ( m a1 l @ k ai t ) ((pos "L@$" ) ))
( "maladies"  n ( m a1 l @ d i z ) ((pos "Kj%" ) ))
( "maladjusted"  j ( m a2 l @ jh uh1 s t i d ) ((pos "OA%" ) ))
( "maladjustment"  n ( m a2 l @ jh uh1 s m @ n t ) ((pos "L@%" ) ))
( "maladroit"  j ( m a1 l @ d r oi t ) ((pos "OA%" ) ))
( "maladroitly"  a ( m a1 l @ d r oi t l ii ) ((pos "Pu$" ) ))
( "maladroitness"  n ( m a1 l @ d r oi t n @ s ) ((pos "L@$" ) ))
( "malady"  n ( m a1 l @ d ii ) ((pos "K8%" ) ))
( "malaise"  n ( m a1 l ei1 z ) ((pos "M6%" ) ))
( "malaises"  n ( m a1 l ei1 z i z ) ((pos "Mj$" ) ))
( "malapropism"  n ( m a1 l @ p r o p i z @ m ) ((pos "K6%" ) ))
( "malapropisms"  n ( m a1 l @ p r o p i z @ m z ) ((pos "Kj%" ) ))
( "malapropos"  j ( m a2 l a2 p r @ p ou1 ) ((pos "OA$" "Pu$" ) ))
( "malaria"  n ( m @1 l e@1 r i@ ) ((pos "L@%" ) ))
( "malarial"  j ( m @1 l e@1 r i@ l ) ((pos "OA%" ) ))
( "malcontent"  n ( m a1 l k @ n t e n t ) ((pos "K6%" "OA$" ) ))
( "malcontents"  n ( m a1 l k @ n t e n t s ) ((pos "Kj%" ) ))
( "male"  n ( m ei1 l ) ((pos "K6%" "OA%" ) ))
( "malediction"  n ( m a2 l i d i1 k sh @ n ) ((pos "K6$" ) ))
( "maledictions"  n ( m a2 l i d i1 k sh @ n z ) ((pos "Kj$" ) ))
( "malefactor"  n ( m a1 l i f a k t @ r ) ((pos "K6%" ) ))
( "malefactors"  n ( m a1 l i f a k t @ z ) ((pos "Kj%" ) ))
( "maleficent"  j ( m @1 l e1 f i s n t ) ((pos "OA$" ) ))
( "males"  n ( m ei1 l z ) ((pos "Kj%" ) ))
( "malevolence"  n ( m @1 l e1 v @ l n s ) ((pos "L@%" ) ))
( "malevolent"  j ( m @1 l e1 v @ l n t ) ((pos "OA%" ) ))
( "malevolently"  a ( m @1 l e1 v @ l n t l ii ) ((pos "Pu%" ) ))
( "malfeasance"  n ( m a2 l f ii1 z n s ) ((pos "M6$" ) ))
( "malfeasances"  n ( m a2 l f ii1 z n s i z ) ((pos "Mj$" ) ))
( "malformation"  n ( m a2 l f oo m ei1 sh @ n ) ((pos "M6%" ) ))
( "malformations"  n ( m a2 l f oo m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "malformed"  j ( m a2 l f oo1 m d ) ((pos "OA%" ) ))
( "malfunction"  v ( m a2 l f uh1 ng k sh @ n ) ((pos "I0%" "M6%" ) (subcat )))
( "malfunctioned"  v ( m a2 l f uh1 ng k sh @ n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "malfunctioning"  v ( m a2 l f uh1 ng k sh @ n i ng ) ((pos "Ib%" ) (subcat )))
( "malfunctions"  v ( m a2 l f uh1 ng k sh @ n z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "malice"  n ( m a1 l i s ) ((pos "L@%" ) ))
( "malicious"  j ( m @1 l i1 sh @ s ) ((pos "OA%" ) ))
( "maliciously"  a ( m @1 l i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "malign"  v ( m @1 l ai1 n ) ((pos "H0%" "OA%" ) (subcat )))
( "malignancy"  n ( m @1 l i1 g n @ n s ii ) ((pos "L@%" ) ))
( "malignant"  j ( m @1 l i1 g n @ n t ) ((pos "OA%" ) ))
( "malignantly"  a ( m @1 l i1 g n @ n t l ii ) ((pos "Pu%" ) ))
( "maligned"  v ( m @1 l ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "maligning"  v ( m @1 l ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "malignities"  n ( m @1 l i1 g n i t i z ) ((pos "Mj$" ) ))
( "malignity"  n ( m @1 l i1 g n i t ii ) ((pos "M8$" ) ))
( "maligns"  v ( m @1 l ai1 n z ) ((pos "Ha%" ) (subcat )))
( "malinger"  v ( m @1 l i1 ng g @ r ) ((pos "I0%" ) (subcat )))
( "malingered"  v ( m @1 l i1 ng g @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "malingerer"  n ( m @1 l i1 ng g @ r @ r ) ((pos "K6%" ) ))
( "malingerers"  n ( m @1 l i1 ng g @ r @ z ) ((pos "Kj%" ) ))
( "malingering"  v ( m @1 l i1 ng g @ r i ng ) ((pos "Ib%" ) (subcat )))
( "malingers"  v ( m @1 l i1 ng g @ z ) ((pos "Ia%" ) (subcat )))
( "mallard"  n ( m a1 l aa d ) ((pos "K6%" ) ))
( "mallards"  n ( m a1 l aa d z ) ((pos "Kj%" ) ))
( "malleability"  n ( m a2 l i@ b i1 l i t ii ) ((pos "L@%" ) ))
( "malleable"  j ( m a1 l i@ b l ) ((pos "OA%" ) ))
( "mallet"  n ( m a1 l i t ) ((pos "K6%" ) ))
( "mallets"  n ( m a1 l i t s ) ((pos "Kj%" ) ))
( "mallow"  n ( m a1 l ou ) ((pos "K6$" ) ))
( "mallows"  n ( m a1 l ou z ) ((pos "Kj$" ) ))
( "malmsey"  n ( m aa1 m z ii ) ((pos "L@$" ) ))
( "malnutrition"  n ( m a2 l n y uu t r i1 sh @ n ) ((pos "L@%" ) ))
( "malodorous"  j ( m a2 l ou1 d @ r @ s ) ((pos "OA%" ) ))
( "malpractice"  n ( m a2 l p r a1 k t i s ) ((pos "M6%" ) ))
( "malpractices"  n ( m a2 l p r a1 k t i s i z ) ((pos "Mj%" ) ))
( "malt"  v ( m oo1 l t ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "malted"  v ( m oo1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "malting"  v ( m oo1 l t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "maltreat"  v ( m a2 l t r ii1 t ) ((pos "H0%" ) (subcat )))
( "maltreated"  v ( m a2 l t r ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "maltreating"  v ( m a2 l t r ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "maltreatment"  n ( m a2 l t r ii1 t m @ n t ) ((pos "L@%" ) ))
( "maltreats"  v ( m a2 l t r ii1 t s ) ((pos "Ha%" ) (subcat )))
( "malts"  v ( m oo1 l t s ) ((pos "Ja$" ) (subcat "2A" )))
( "maltster"  n ( m oo1 l t s t @ r ) ((pos "K6$" ) ))
( "maltsters"  n ( m oo1 l t s t @ z ) ((pos "Kj$" ) ))
( "malversation"  n ( m a2 l v @ s ei1 sh @ n ) ((pos "L@$" ) ))
( "mama"  n ( m @1 m aa1 ) ((pos "K6%" ) ))
( "mamas"  n ( m @1 m aa1 z ) ((pos "Kj$" ) ))
( "mamba"  n ( m a1 m b @ ) ((pos "K6%" ) ))
( "mambas"  n ( m a1 m b @ z ) ((pos "Kj%" ) ))
( "mamma"  n ( m @1 m aa1 ) ((pos "K6$" ) ))
( "mammal"  n ( m a1 m l ) ((pos "K6%" ) ))
( "mammals"  n ( m a1 m l z ) ((pos "Kj%" ) ))
( "mammas"  n ( m @1 m aa1 z ) ((pos "Kj$" ) ))
( "mammies"  n ( m a1 m i z ) ((pos "Kj%" ) ))
( "mammon"  n ( m a1 m @ n ) ((pos "L@%" ) ))
( "mammoth"  n ( m a1 m @ th ) ((pos "K6%" ) ))
( "mammoths"  n ( m a1 m @ th s ) ((pos "Kj%" ) ))
( "mammy"  n ( m a1 m ii ) ((pos "K8%" ) ))
( "man"  v ( m a1 n ) ((pos "H4%" "Ki*" ) (subcat )))
( "man-at-arms"  n ( m a2 n - @ t - aa1 m z ) ((pos "Ki%" ) ))
( "man-eater"  n ( m a1 n - ii t @ r ) ((pos "K6%" ) ))
( "man-eaters"  n ( m a1 n - ii t @ z ) ((pos "Kj%" ) ))
( "man-hour"  n ( m a1 n - au @ r ) ((pos "K6%" ) ))
( "man-hours"  n ( m a1 n - au @ z ) ((pos "Kj%" ) ))
( "man-of-war"  n ( m a2 n - @ v - w oo1 r ) ((pos "Ki%" ) ))
( "man-sized"  j ( m a1 n - s ai z d ) ((pos "OA%" ) ))
( "manacle"  v ( m a1 n @ k l ) ((pos "H2%" "K6%" ) (subcat )))
( "manacled"  v ( m a1 n @ k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "manacles"  v ( m a1 n @ k l z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "manacling"  v ( m a1 n @ k l i ng ) ((pos "Hb$" ) (subcat )))
( "manage"  v ( m a1 n i jh ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "4A" )))
( "manageability"  n ( m a2 n i jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "manageable"  j ( m a1 n i jh @ b l ) ((pos "OA%" ) ))
( "manageably"  a ( m a1 n i jh @ b l ii ) ((pos "Pu%" ) ))
( "managed"  v ( m a1 n i jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" )))
( "management"  n ( m a1 n i jh m @ n t ) ((pos "M6%" ) ))
( "managements"  n ( m a1 n i jh m @ n t s ) ((pos "Mj%" ) ))
( "manager"  n ( m a1 n i jh @ r ) ((pos "K6%" ) ))
( "manageress"  n ( m a2 n i jh @ r e1 s ) ((pos "K7%" ) ))
( "manageresses"  n ( m a2 n i jh @ r e1 s i z ) ((pos "Kj%" ) ))
( "managerial"  j ( m a2 n i jh i@1 r i@ l ) ((pos "OA%" ) ))
( "managers"  n ( m a1 n i jh @ z ) ((pos "Kj%" ) ))
( "manages"  v ( m a1 n i jh i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "4A" )))
( "managing"  v ( m a1 n i jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" )))
( "manatee"  n ( m a2 n @ t ii1 ) ((pos "K6$" ) ))
( "manatees"  n ( m a2 n @ t ii1 z ) ((pos "Kj$" ) ))
( "mandarin"  n ( m a1 n d @ r i n ) ((pos "K6%" ) ))
( "mandarins"  n ( m a1 n d @ r i n z ) ((pos "Kj%" ) ))
( "mandataries"  n ( m a1 n d @ t @ r i z ) ((pos "Kj$" ) ))
( "mandatary"  n ( m a1 n d @ t @ r ii ) ((pos "K8$" ) ))
( "mandate"  v ( m a1 n d ei t ) ((pos "H2%" "K6%" ) (subcat )))
( "mandated"  v ( m a1 n d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mandates"  v ( m a1 n d ei t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "mandating"  v ( m a1 n d ei t i ng ) ((pos "Hb$" ) (subcat )))
( "mandatories"  n ( m a1 n d @ t r i z ) ((pos "Kj$" ) ))
( "mandatory"  n ( m a1 n d @ t r ii ) ((pos "K8$" "OA%" ) ))
( "mandible"  n ( m a1 n d i b l ) ((pos "K6%" ) ))
( "mandibles"  n ( m a1 n d i b l z ) ((pos "Kj%" ) ))
( "mandolin"  n ( m a1 n d @ l i n ) ((pos "K6%" ) ))
( "mandolins"  n ( m a1 n d @ l i n z ) ((pos "Kj%" ) ))
( "mandragora"  n ( m a1 n d r a1 g @ r @ ) ((pos "L@$" ) ))
( "mandrake"  n ( m a1 n d r ei k ) ((pos "K6$" ) ))
( "mandrakes"  n ( m a1 n d r ei k s ) ((pos "Kj$" ) ))
( "mandrill"  n ( m a1 n d r i l ) ((pos "K6$" ) ))
( "mandrills"  n ( m a1 n d r i l z ) ((pos "Kj$" ) ))
( "mane"  n ( m ei1 n ) ((pos "K6%" ) ))
( "manes"  n ( m aa1 n ei z ) ((pos "Kj%" ) ))
( "manes"  n ( m ei1 n z ) ((pos "Kj%" ) ))
( "manful"  j ( m a1 n f @ l ) ((pos "OA%" ) ))
( "manfully"  a ( m a1 n f @ l ii ) ((pos "Pu%" ) ))
( "manganese"  n ( m a1 ng g @ n ii z ) ((pos "L@%" ) ))
( "mange"  n ( m ei1 n jh ) ((pos "L@$" ) ))
( "mangel-wurzel"  n ( m a1 ng g l - w @@ z l ) ((pos "K6%" ) ))
( "mangel-wurzels"  n ( m a1 ng g l - w @@ z l z ) ((pos "Kj%" ) ))
( "manger"  n ( m ei1 n jh @ r ) ((pos "K6%" ) ))
( "mangers"  n ( m ei1 n jh @ z ) ((pos "Kj%" ) ))
( "mangier"  j ( m ei1 n jh i@ r ) ((pos "Or%" ) ))
( "mangiest"  j ( m ei1 n jh i i s t ) ((pos "Os%" ) ))
( "mangily"  a ( m ei1 n jh i l ii ) ((pos "Pu$" ) ))
( "mangle"  v ( m a1 ng g l ) ((pos "H2%" "K6%" ) (subcat )))
( "mangled"  v ( m a1 ng g l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mangles"  v ( m a1 ng g l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mangling"  v ( m a1 ng g l i ng ) ((pos "Hb%" ) (subcat )))
( "mango"  n ( m a1 ng g ou ) ((pos "K6%" ) ))
( "mangoes"  n ( m a1 ng g ou z ) ((pos "Kj%" ) ))
( "mangos"  n ( m a1 ng g ou z ) ((pos "Kj%" ) ))
( "mangosteen"  n ( m a2 ng g @ s t ii1 n ) ((pos "K6$" ) ))
( "mangosteens"  n ( m a2 ng g @ s t ii1 n z ) ((pos "Kj$" ) ))
( "mangrove"  n ( m a1 ng g r ou v ) ((pos "K6%" ) ))
( "mangroves"  n ( m a1 ng g r ou v z ) ((pos "Kj%" ) ))
( "mangy"  j ( m ei1 n jh ii ) ((pos "OD%" ) ))
( "manhandle"  v ( m a1 n h a1 n d l ) ((pos "H2%" ) ))
( "manhandled"  v ( m a1 n h a1 n d l d ) ((pos "Hc%" "Hd%" ) ))
( "manhandles"  v ( m a1 n h a1 n d l z ) ((pos "Ha$" ) ))
( "manhandling"  v ( m a1 n h a1 n d l i ng ) ((pos "Hb%" ) ))
( "manhattan"  n ( m a1 n h a1 t n ) ((pos "K6$" ) ))
( "manhattans"  n ( m a1 n h a1 t n z ) ((pos "Kj$" ) ))
( "manhole"  n ( m a1 n h ou l ) ((pos "K6%" ) ))
( "manholes"  n ( m a1 n h ou l z ) ((pos "Kj%" ) ))
( "manhood"  n ( m a1 n h u d ) ((pos "L@%" ) ))
( "mania"  n ( m ei1 n i@ ) ((pos "M6%" ) ))
( "maniac"  n ( m ei1 n i a k ) ((pos "K6%" ) ))
( "maniacal"  j ( m @1 n ai1 @ k l ) ((pos "OA%" ) ))
( "maniacally"  a ( m @1 n ai1 @ k l ii ) ((pos "Pu%" ) ))
( "maniacs"  n ( m ei1 n i a k s ) ((pos "Kj%" ) ))
( "manias"  n ( m ei1 n i@ z ) ((pos "Mj%" ) ))
( "manic-depressive"  n ( m a2 n i k - d i p r e1 s i v ) ((pos "K6%" "OA%" ) ))
( "manic-depressives"  n ( m a2 n i k - d i p r e1 s i v z ) ((pos "Kj%" ) ))
( "manicure"  v ( m a1 n i k y u@ r ) ((pos "H2%" "M6%" ) (subcat )))
( "manicured"  v ( m a1 n i k y u@ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "manicures"  v ( m a1 n i k y u@ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "manicuring"  v ( m a1 n i k y u@ r i ng ) ((pos "Hb%" ) (subcat )))
( "manicurist"  n ( m a1 n i k y uu r i s t ) ((pos "K6%" ) ))
( "manicurists"  n ( m a1 n i k y uu r i s t s ) ((pos "Kj%" ) ))
( "manifest"  v ( m a1 n i f e s t ) ((pos "H0%" "K6$" "OA%" ) (subcat )))
( "manifestation"  n ( m a2 n i f e s t ei1 sh @ n ) ((pos "M6%" ) ))
( "manifestations"  n ( m a2 n i f e s t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "manifested"  v ( m a1 n i f e s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "manifesting"  v ( m a1 n i f e s t i ng ) ((pos "Hb%" ) (subcat )))
( "manifestly"  a ( m a1 n i f e s t l ii ) ((pos "Pu%" ) ))
( "manifesto"  n ( m a2 n i f e1 s t ou ) ((pos "K6%" ) ))
( "manifestoes"  n ( m a2 n i f e1 s t ou z ) ((pos "Kj%" ) ))
( "manifestos"  n ( m a2 n i f e1 s t ou z ) ((pos "Kj%" ) ))
( "manifests"  v ( m a1 n i f e s t s ) ((pos "Ha%" "Kj$" ) (subcat )))
( "manifold"  v ( m a1 n i f ou l d ) ((pos "H0$" "K6$" "OA%" ) (subcat )))
( "manifolded"  v ( m a1 n i f ou l d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "manifolding"  v ( m a1 n i f ou l d i ng ) ((pos "Hb$" ) (subcat )))
( "manifolds"  v ( m a1 n i f ou l d z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "manikin"  n ( m a1 n i k i n ) ((pos "K6$" ) ))
( "manikins"  n ( m a1 n i k i n z ) ((pos "Kj$" ) ))
( "manipulate"  v ( m @1 n i1 p y u l ei t ) ((pos "H2%" ) (subcat )))
( "manipulated"  v ( m @1 n i1 p y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "manipulates"  v ( m @1 n i1 p y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "manipulating"  v ( m @1 n i1 p y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "manipulation"  n ( m @1 n i2 p y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "manipulations"  n ( m @1 n i2 p y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "manipulative"  j ( m @1 n i1 p y u l @ t i v ) ((pos "OA%" ) ))
( "mankind"  n ( m a1 n k ai n d ) ((pos "L@%" ) ))
( "mankind"  n ( m a2 n k ai1 n d ) ((pos "L@%" ) ))
( "manlier"  j ( m a1 n l i@ r ) ((pos "Or%" ) ))
( "manliest"  j ( m a1 n l i i s t ) ((pos "Os%" ) ))
( "manlike"  j ( m a1 n l ai k ) ((pos "OA$" ) ))
( "manliness"  n ( m a1 n l i n @ s ) ((pos "L@%" ) ))
( "manly"  j ( m a1 n l ii ) ((pos "OD%" ) ))
( "manna"  n ( m a1 n @ ) ((pos "L@%" ) ))
( "manned"  v ( m a1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mannequin"  n ( m a1 n i k i n ) ((pos "K6%" ) ))
( "mannequins"  n ( m a1 n i k i n z ) ((pos "Kj%" ) ))
( "manner"  n ( m a1 n @ r ) ((pos "K6%" ) ))
( "mannered"  j ( m a1 n @ d ) ((pos "OA%" ) ))
( "mannerism"  n ( m a1 n @ r i z @ m ) ((pos "K6%" ) ))
( "mannerisms"  n ( m a1 n @ r i z @ m z ) ((pos "Kj%" ) ))
( "mannerly"  j ( m a1 n @ l ii ) ((pos "OA%" ) ))
( "manners"  n ( m a1 n @ z ) ((pos "Kj%" ) ))
( "manning"  v ( m a1 n i ng ) ((pos "Hb%" ) (subcat )))
( "mannish"  j ( m a1 n i sh ) ((pos "OA%" ) ))
( "manoeuvrability"  n ( m @1 n uu2 v r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "manoeuvrable"  j ( m @1 n uu1 v r @ b l ) ((pos "OA%" ) ))
( "manoeuvre"  v ( m @1 n uu1 v @ r ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "manoeuvred"  v ( m @1 n uu1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "manoeuvrer"  n ( m @1 n uu1 v @ r @ r ) ((pos "K6$" ) ))
( "manoeuvrers"  n ( m @1 n uu1 v @ r @ z ) ((pos "Kj$" ) ))
( "manoeuvres"  v ( m @1 n uu1 v @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "manoeuvring"  v ( m @1 n uu1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "manor"  n ( m a1 n @ r ) ((pos "K6%" ) ))
( "manor-house"  n ( m a1 n @ - h au s ) ((pos "K6%" ) ))
( "manor-houses"  n ( m a1 n @ - h au z i z ) ((pos "Kj%" ) ))
( "manorial"  j ( m @1 n oo1 r i@ l ) ((pos "OA%" ) ))
( "manors"  n ( m a1 n @ z ) ((pos "Kj$" ) ))
( "manpower"  n ( m a1 n p au @ r ) ((pos "L@%" ) ))
( "mans"  v ( m a1 n z ) ((pos "Ha%" ) (subcat )))
( "mansard"  n ( m a1 n s aa d ) ((pos "K6$" ) ))
( "mansards"  n ( m a1 n s aa d z ) ((pos "Kj$" ) ))
( "manse"  n ( m a1 n s ) ((pos "K6%" ) ))
( "manservant"  n ( m a1 n s @@ v @ n t ) ((pos "K6%" ) ))
( "manservants"  n ( m a1 n s @@ v @ n t s ) ((pos "Kj%" ) ))
( "manses"  n ( m a1 n s i z ) ((pos "Kj$" ) ))
( "mansion"  n ( m a1 n sh @ n ) ((pos "K6%" ) ))
( "mansions"  n ( m a1 n sh @ n z ) ((pos "Kj%" ) ))
( "manslaughter"  n ( m a1 n s l oo t @ r ) ((pos "L@%" ) ))
( "mantel"  n ( m a1 n t l ) ((pos "K6%" ) ))
( "mantelpiece"  n ( m a1 n t l p ii s ) ((pos "K6%" ) ))
( "mantelpieces"  n ( m a1 n t l p ii s i z ) ((pos "Kj%" ) ))
( "mantels"  n ( m a1 n t l z ) ((pos "Kj%" ) ))
( "mantilla"  n ( m a1 n t i1 l @ ) ((pos "K6$" ) ))
( "mantillas"  n ( m a1 n t i1 l @ z ) ((pos "Kj$" ) ))
( "mantis"  n ( m a1 n t i s ) ((pos "K7%" ) ))
( "mantises"  n ( m a1 n t i s i z ) ((pos "Kj%" ) ))
( "mantle"  v ( m a1 n t l ) ((pos "J2%" "K6%" ) (subcat "2C" )))
( "mantled"  v ( m a1 n t l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "mantles"  v ( m a1 n t l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "mantling"  v ( m a1 n t l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "mantrap"  n ( m a1 n t r a p ) ((pos "K6%" ) ))
( "mantraps"  n ( m a1 n t r a p s ) ((pos "Kj%" ) ))
( "manual"  n ( m a1 n y u@ l ) ((pos "K6%" "OA%" ) ))
( "manually"  a ( m a1 n y u@ l ii ) ((pos "Pu%" ) ))
( "manuals"  n ( m a1 n y u@ l z ) ((pos "Kj%" ) ))
( "manufacture"  v ( m a2 n y u f a1 k ch @ r ) ((pos "H2%" "L@%" ) (subcat )))
( "manufactured"  v ( m a2 n y u f a1 k ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "manufacturer"  n ( m a2 n y u f a1 k ch @ r @ r ) ((pos "K6%" ) ))
( "manufacturers"  n ( m a2 n y u f a1 k ch @ r @ z ) ((pos "Kj%" ) ))
( "manufactures"  v ( m a2 n y u f a1 k ch @ z ) ((pos "Ha%" ) (subcat )))
( "manufacturing"  v ( m a2 n y u f a1 k ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "manumission"  n ( m a2 n y u m i1 sh @ n ) ((pos "M6$" ) ))
( "manumissions"  n ( m a2 n y u m i1 sh @ n z ) ((pos "Mj$" ) ))
( "manumit"  v ( m a2 n y u m i1 t ) ((pos "H4$" ) ))
( "manumits"  v ( m a2 n y u m i1 t s ) ((pos "Ha$" ) ))
( "manumitted"  v ( m a2 n y u m i1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "manumitting"  v ( m a2 n y u m i1 t i ng ) ((pos "Hb$" ) ))
( "manure"  v ( m @1 n y u@1 r ) ((pos "H2$" "L@%" ) (subcat )))
( "manured"  v ( m @1 n y u@1 d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "manures"  v ( m @1 n y u@1 z ) ((pos "Ha$" ) (subcat )))
( "manuring"  v ( m @1 n y u@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "manuscript"  n ( m a1 n y u s k r i p t ) ((pos "K6%" ) ))
( "manuscripts"  n ( m a1 n y u s k r i p t s ) ((pos "Kj%" ) ))
( "many"  n ( m e1 n ii ) ((pos "Kj%" "OA*" ) ))
( "many-sided"  j ( m e2 n i - s ai1 d i d ) ((pos "OA%" ) ))
( "map"  v ( m a1 p ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "map-reader"  n ( m a1 p - r ii d @ r ) ((pos "K6%" ) ))
( "map-readers"  n ( m a1 p - r ii d @ z ) ((pos "Kj%" ) ))
( "maple"  n ( m ei1 p l ) ((pos "M6%" ) ))
( "maple-leaf"  n ( m ei1 p l - l ii f ) ((pos "Ki%" ) ))
( "maple-leaves"  n ( m ei1 p l - l ii v z ) ((pos "Kj%" ) ))
( "maples"  n ( m ei1 p l z ) ((pos "Mj%" ) ))
( "mapped"  v ( m a1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mapping"  v ( m a1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "maps"  v ( m a1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "maquis"  n ( m a1 k ii ) ((pos "Ki$" ) ))
( "mar"  v ( m aa1 r ) ((pos "H4%" ) (subcat )))
( "marabou"  n ( m a1 r @ b uu ) ((pos "K6$" ) ))
( "marabous"  n ( m a1 r @ b uu z ) ((pos "Kj$" ) ))
( "maraschino"  n ( m a2 r @ s k ii1 n ou ) ((pos "K6%" ) ))
( "maraschinos"  n ( m a2 r @ s k ii1 n ou z ) ((pos "Kj$" ) ))
( "marathon"  n ( m a1 r @ th @ n ) ((pos "K6%" ) ))
( "marathons"  n ( m a1 r @ th @ n z ) ((pos "Kj%" ) ))
( "maraud"  v ( m @1 r oo1 d ) ((pos "I0$" ) (subcat )))
( "marauded"  v ( m @1 r oo1 d i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "marauder"  n ( m @1 r oo1 d @ r ) ((pos "K6%" ) ))
( "marauders"  n ( m @1 r oo1 d @ z ) ((pos "Kj%" ) ))
( "marauding"  v ( m @1 r oo1 d i ng ) ((pos "Ib%" ) (subcat )))
( "marauds"  v ( m @1 r oo1 d z ) ((pos "Ia$" ) (subcat )))
( "marble"  n ( m aa1 b l ) ((pos "M6%" ) ))
( "marbled"  j ( m aa1 b l d ) ((pos "OA%" ) ))
( "marbles"  n ( m aa1 b l z ) ((pos "Mj%" ) ))
( "marbling"  n ( m aa1 b l i ng ) ((pos "L@%" ) ))
( "march"  v ( m aa1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "2B" "2C" "3A" "15A" )))
( "marched"  v ( m aa1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "15A" )))
( "marcher"  n ( m aa1 ch @ r ) ((pos "K6%" ) ))
( "marchers"  n ( m aa1 ch @ z ) ((pos "Kj%" ) ))
( "marches"  v ( m aa1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "15A" )))
( "marching"  v ( m aa1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "15A" )))
( "marchioness"  n ( m aa2 sh @ n e1 s ) ((pos "K7%" ) ))
( "marchionesses"  n ( m aa2 sh @ n e1 s i z ) ((pos "Kj$" ) ))
( "mare"  n ( m e@1 r ) ((pos "K6%" ) ))
( "mares"  n ( m e@1 z ) ((pos "Kj%" ) ))
( "margarine"  n ( m aa2 jh @ r ii1 n ) ((pos "L@%" ) ))
( "marge"  n ( m aa1 jh ) ((pos "L@%" ) ))
( "margin"  n ( m aa1 jh i n ) ((pos "K6%" ) ))
( "marginal"  j ( m aa1 jh i n l ) ((pos "OA%" ) ))
( "marginally"  a ( m aa1 jh i n @ l ii ) ((pos "Pu%" ) ))
( "margins"  n ( m aa1 jh i n z ) ((pos "Kj%" ) ))
( "marguerite"  n ( m aa2 g @ r ii1 t ) ((pos "K6$" ) ))
( "marguerites"  n ( m aa2 g @ r ii1 t s ) ((pos "Kj$" ) ))
( "marigold"  n ( m a1 r i g ou l d ) ((pos "K6%" ) ))
( "marigolds"  n ( m a1 r i g ou l d z ) ((pos "Kj%" ) ))
( "marihuana"  n ( m a2 r i w aa1 n @ ) ((pos "L@%" ) ))
( "marijuana"  n ( m a2 r i w aa1 n @ ) ((pos "L@%" ) ))
( "marimba"  n ( m @1 r i1 m b @ ) ((pos "K6$" ) ))
( "marimbas"  n ( m @1 r i1 m b @ z ) ((pos "Kj$" ) ))
( "marina"  n ( m @1 r ii1 n @ ) ((pos "K6%" ) ))
( "marinade"  v ( m a2 r i n ei1 d ) ((pos "H2%" "L@%" ) ))
( "marinaded"  v ( m a2 r i n ei1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "marinades"  v ( m a2 r i n ei1 d z ) ((pos "Ha%" ) ))
( "marinading"  v ( m a2 r i n ei1 d i ng ) ((pos "Hb%" ) ))
( "marinas"  n ( m @1 r ii1 n @ z ) ((pos "Kj%" ) ))
( "marinate"  v ( m a1 r i n ei t ) ((pos "H2%" ) ))
( "marinated"  v ( m a1 r i n ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "marinates"  v ( m a1 r i n ei t s ) ((pos "Ha%" ) ))
( "marinating"  v ( m a1 r i n ei t i ng ) ((pos "Hb%" ) ))
( "marine"  n ( m @1 r ii1 n ) ((pos "K6%" "OA%" ) ))
( "mariner"  n ( m a1 r i n @ r ) ((pos "K6%" ) ))
( "mariners"  n ( m a1 r i n @ z ) ((pos "Kj%" ) ))
( "marines"  n ( m @1 r ii1 n z ) ((pos "Kj%" ) ))
( "marionette"  n ( m a2 r i@ n e1 t ) ((pos "K6%" ) ))
( "marionettes"  n ( m a2 r i@ n e1 t s ) ((pos "Kj%" ) ))
( "marital"  j ( m a1 r i t l ) ((pos "OA%" ) ))
( "maritime"  j ( m a1 r i t ai m ) ((pos "OA%" ) ))
( "marjoram"  n ( m aa1 jh @ r @ m ) ((pos "L@%" ) ))
( "mark"  v ( m aa1 k ) ((pos "H0%" "M6%" ) (subcat "2A" "6A" "8" "10" "15A" "15B" )))
( "mark-up"  n ( m aa1 k - uh p ) ((pos "K6%" ) ))
( "mark-ups"  n ( m aa1 k - uh p s ) ((pos "Kj$" ) ))
( "marked"  v ( m aa1 k t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "2A" "6A" "8" "10" "15A" "15B" )))
( "markedly"  a ( m aa1 k i d l ii ) ((pos "Pu%" ) ))
( "marker"  n ( m aa1 k @ r ) ((pos "K6%" ) ))
( "markers"  n ( m aa1 k @ z ) ((pos "Kj%" ) ))
( "market"  v ( m aa1 k i t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "market-cross"  n ( m aa2 k i t - k r o1 s ) ((pos "K7%" ) ))
( "market-crosses"  n ( m aa2 k i t - k r o1 s i z ) ((pos "Kj%" ) ))
( "market-day"  n ( m aa1 k i t - d ei ) ((pos "K6%" ) ))
( "market-days"  n ( m aa1 k i t - d ei z ) ((pos "Kj%" ) ))
( "market-garden"  n ( m aa2 k i t - g aa1 d n ) ((pos "K6%" ) ))
( "market-gardening"  n ( m aa2 k i t - g aa1 d n i ng ) ((pos "L@%" ) ))
( "market-gardens"  n ( m aa2 k i t - g aa1 d n z ) ((pos "Kj%" ) ))
( "market-square"  n ( m aa2 k i t - s k w e@1 r ) ((pos "K6%" ) ))
( "market-squares"  n ( m aa2 k i t - s k w e@1 z ) ((pos "Kj%" ) ))
( "market-town"  n ( m aa1 k i t - t au n ) ((pos "K6%" ) ))
( "market-towns"  n ( m aa1 k i t - t au n z ) ((pos "Kj%" ) ))
( "marketable"  j ( m aa1 k i t @ b l ) ((pos "OA%" ) ))
( "marketed"  v ( m aa1 k i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "marketing"  v ( m aa1 k i t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "marketings"  n ( m aa1 k i t i ng z ) ((pos "Kj$" ) ))
( "marketplace"  n ( m aa1 k i t p l ei s ) ((pos "K6%" ) ))
( "marketplaces"  n ( m aa1 k i t p l ei s i z ) ((pos "Kj%" ) ))
( "markets"  v ( m aa1 k i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "marking"  v ( m aa1 k i ng ) ((pos "Hb%" "K6%" ) (subcat "2A" "6A" "8" "10" "15A" "15B" )))
( "marking-ink"  n ( m aa1 k i ng - i ng k ) ((pos "M6%" ) ))
( "marking-inks"  n ( m aa1 k i ng - i ng k s ) ((pos "M6%" ) ))
( "markings"  n ( m aa1 k i ng z ) ((pos "Kj%" ) ))
( "marks"  v ( m aa1 k s ) ((pos "Ha%" "Mj%" ) (subcat "2A" "6A" "8" "10" "15A" "15B" )))
( "marksman"  n ( m aa1 k s m @ n ) ((pos "Ki%" ) ))
( "marksmanship"  n ( m aa1 k s m @ n sh i p ) ((pos "L@%" ) ))
( "marksmen"  n ( m aa1 k s m @ n ) ((pos "Kj%" ) ))
( "marl"  n ( m aa1 l ) ((pos "L@$" ) ))
( "marlinespike"  n ( m aa1 l i n s p ai k ) ((pos "K6$" ) ))
( "marlinespikes"  n ( m aa1 l i n s p ai k s ) ((pos "Kj$" ) ))
( "marmalade"  n ( m aa1 m @ l ei d ) ((pos "L@%" ) ))
( "marmoreal"  j ( m aa1 m oo1 r i@ l ) ((pos "OA$" ) ))
( "marmoset"  n ( m aa1 m @ z e t ) ((pos "K6$" ) ))
( "marmosets"  n ( m aa1 m @ z e t s ) ((pos "Kj$" ) ))
( "marmot"  n ( m aa1 m @ t ) ((pos "K6$" ) ))
( "marmots"  n ( m aa1 m @ t s ) ((pos "Kj$" ) ))
( "marocain"  n ( m a1 r @ k ei n ) ((pos "L@$" ) ))
( "maroon"  v ( m @1 r uu1 n ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "marooned"  v ( m @1 r uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "marooning"  v ( m @1 r uu1 n i ng ) ((pos "Hb$" ) (subcat )))
( "maroons"  v ( m @1 r uu1 n z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "marque"  n ( m aa1 k ) ((pos "K6$" ) ))
( "marquee"  n ( m aa1 k ii1 ) ((pos "K6%" ) ))
( "marquees"  n ( m aa1 k ii1 z ) ((pos "Kj%" ) ))
( "marques"  n ( m aa1 k s ) ((pos "Kj$" ) ))
( "marquess"  n ( m aa1 k w i s ) ((pos "K7%" ) ))
( "marquesses"  n ( m aa1 k w i s i z ) ((pos "Kj%" ) ))
( "marquetry"  n ( m aa1 k i t r ii ) ((pos "L@%" ) ))
( "marquis"  n ( m aa1 k w i s ) ((pos "K7%" ) ))
( "marquises"  n ( m aa1 k w i s i z ) ((pos "Kj%" ) ))
( "marred"  v ( m aa1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "marriage"  n ( m a1 r i jh ) ((pos "M6%" ) ))
( "marriageability"  n ( m a2 r i jh @ b i1 l i t ii ) ((pos "L@$" ) ))
( "marriageable"  j ( m a1 r i jh @ b l ) ((pos "OA%" ) ))
( "marriages"  n ( m a1 r i jh i z ) ((pos "Mj%" ) ))
( "married"  v ( m a1 r i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2D" "4A" "6A" )))
( "marries"  v ( m a1 r i z ) ((pos "Ja%" ) (subcat "2A" "2D" "4A" "6A" )))
( "marring"  v ( m aa1 r i ng ) ((pos "Hb%" ) (subcat )))
( "marrow"  n ( m a1 r ou ) ((pos "M6%" ) ))
( "marrowbone"  n ( m a1 r ou b ou n ) ((pos "K6%" ) ))
( "marrowbones"  n ( m a1 r ou b ou n z ) ((pos "Kj$" ) ))
( "marrows"  n ( m a1 r ou z ) ((pos "Mj%" ) ))
( "marry"  v ( m a1 r ii ) ((pos "J3%" ) (subcat "2A" "2D" "4A" "6A" )))
( "marrying"  v ( m a1 r i i ng ) ((pos "Jb%" ) (subcat "2A" "2D" "4A" "6A" )))
( "mars"  v ( m aa1 z ) ((pos "Ha%" ) (subcat )))
( "marsh"  n ( m aa1 sh ) ((pos "M7%" ) ))
( "marshal"  v ( m aa1 sh l ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" )))
( "marshalled"  v ( m aa1 sh l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "marshalling"  v ( m aa1 sh @ l i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "marshalling-yard"  n ( m aa1 sh @ l i ng - y aa d ) ((pos "K6%" ) ))
( "marshalling-yards"  n ( m aa1 sh @ l i ng - y aa d z ) ((pos "Kj%" ) ))
( "marshals"  v ( m aa1 sh l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "marshes"  n ( m aa1 sh i z ) ((pos "Mj%" ) ))
( "marshier"  j ( m aa1 sh i@ r ) ((pos "Or%" ) ))
( "marshiest"  j ( m aa1 sh i i s t ) ((pos "Os%" ) ))
( "marshmallow"  n ( m aa1 sh m a1 l ou ) ((pos "M6%" ) ))
( "marshmallows"  n ( m aa1 sh m a1 l ou z ) ((pos "Mj%" ) ))
( "marshy"  j ( m aa1 sh ii ) ((pos "OD%" ) ))
( "marsupial"  n ( m aa1 s uu1 p i@ l ) ((pos "K6%" "OA%" ) ))
( "marsupials"  n ( m aa1 s uu1 p i@ l z ) ((pos "Kj%" ) ))
( "mart"  n ( m aa1 t ) ((pos "K6$" ) ))
( "marten"  n ( m aa1 t i n ) ((pos "M6$" ) ))
( "martens"  n ( m aa1 t i n z ) ((pos "Mj$" ) ))
( "martial"  j ( m aa1 sh l ) ((pos "OA%" ) ))
( "martially"  a ( m aa1 sh @ l ii ) ((pos "Pu%" ) ))
( "martin"  n ( m aa1 t i n ) ((pos "K6$" ) ))
( "martinet"  n ( m aa2 t i n e1 t ) ((pos "K6%" ) ))
( "martinets"  n ( m aa2 t i n e1 t s ) ((pos "Kj%" ) ))
( "martini"  n ( m aa1 t ii1 n ii ) ((pos "K6%" ) ))
( "martinis"  n ( m aa1 t ii1 n i z ) ((pos "Kj%" ) ))
( "martins"  n ( m aa1 t i n z ) ((pos "Kj$" ) ))
( "marts"  n ( m aa1 t s ) ((pos "Kj$" ) ))
( "martyr"  v ( m aa1 t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "martyrdom"  n ( m aa1 t @ d @ m ) ((pos "K6%" ) ))
( "martyrdoms"  n ( m aa1 t @ d @ m z ) ((pos "Kj$" ) ))
( "martyred"  v ( m aa1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "martyring"  v ( m aa1 t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "martyrs"  v ( m aa1 t @ z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "marvel"  v ( m aa1 v @ l ) ((pos "I4%" "K6%" ) (subcat "3A" )))
( "marvelled"  v ( m aa1 v @ l d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "marvelling"  v ( m aa1 v @ l i ng ) ((pos "Ib%" ) (subcat "3A" )))
( "marvellous"  j ( m aa1 v @ l @ s ) ((pos "OA%" ) ))
( "marvellously"  a ( m aa1 v @ l @ s l ii ) ((pos "Pu%" ) ))
( "marvelous"  j ( m aa1 v @ l @ s ) ((pos "OA$" ) ))
( "marvelously"  a ( m aa1 v @ l @ s l ii ) ((pos "Pu$" ) ))
( "marvels"  v ( m aa1 v @ l z ) ((pos "Ia%" "Kj%" ) (subcat "3A" )))
( "marzipan"  n ( m aa1 z i p a n ) ((pos "M6%" ) ))
( "marzipans"  n ( m aa1 z i p a n z ) ((pos "Mj$" ) ))
( "masc"  n ( m a1 s k ) ((pos "Y~%" ) ))
( "mascara"  n ( m a1 s k aa1 r @ ) ((pos "L@%" ) ))
( "mascot"  n ( m a1 s k @ t ) ((pos "K6%" ) ))
( "mascots"  n ( m a1 s k @ t s ) ((pos "Kj%" ) ))
( "masculine"  j ( m a1 s k y u l i n ) ((pos "OA%" ) ))
( "masculinity"  n ( m a2 s k y u l i1 n i t ii ) ((pos "L@%" ) ))
( "maser"  n ( m ei1 z @ r ) ((pos "K6$" ) ))
( "masers"  n ( m ei1 z @ z ) ((pos "Kj$" ) ))
( "mash"  v ( m a1 sh ) ((pos "H1%" "M7%" ) (subcat )))
( "mashed"  v ( m a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "masher"  n ( m a1 sh @ r ) ((pos "K6%" ) ))
( "mashers"  n ( m a1 sh @ z ) ((pos "Kj$" ) ))
( "mashes"  v ( m a1 sh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "mashing"  v ( m a1 sh i ng ) ((pos "Hb%" ) (subcat )))
( "mask"  v ( m aa1 s k ) ((pos "H0%" "K6%" ) (subcat )))
( "masked"  v ( m aa1 s k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "masking"  v ( m aa1 s k i ng ) ((pos "Hb%" ) (subcat )))
( "masks"  v ( m aa1 s k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "masochism"  n ( m a1 s @ k i z @ m ) ((pos "L@%" ) ))
( "masochist"  n ( m a1 s @ k i s t ) ((pos "K6%" ) ))
( "masochistic"  j ( m a2 s @ k i1 s t i k ) ((pos "OA%" ) ))
( "masochistically"  a ( m a2 s @ k i1 s t i k l ii ) ((pos "Pu%" ) ))
( "masochists"  n ( m a1 s @ k i s t s ) ((pos "Kj%" ) ))
( "mason"  n ( m ei1 s n ) ((pos "K6%" ) ))
( "masonic"  j ( m @1 s o1 n i k ) ((pos "OA%" ) ))
( "masonry"  n ( m ei1 s n r ii ) ((pos "L@%" ) ))
( "masons"  n ( m ei1 s n z ) ((pos "Kj%" ) ))
( "masque"  n ( m aa1 s k ) ((pos "K6%" ) ))
( "masquerade"  v ( m aa2 s k @ r ei1 d ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "masqueraded"  v ( m aa2 s k @ r ei1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "masquerades"  v ( m aa2 s k @ r ei1 d z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "masquerading"  v ( m aa2 s k @ r ei1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "masques"  n ( m aa1 s k s ) ((pos "Kj%" ) ))
( "mass"  v ( m a1 s ) ((pos "J1%" "M7%" ) (subcat "2A" )))
( "mass-produce"  v ( m a2 s - p r @ d y uu1 s ) ((pos "H2%" ) ))
( "mass-produced"  v ( m a2 s - p r @ d y uu1 s t ) ((pos "Hc%" "Hd%" ) ))
( "mass-produces"  v ( m a2 s - p r @ d y uu1 s i z ) ((pos "Ha%" ) ))
( "mass-producing"  v ( m a2 s - p r @ d y uu1 s i ng ) ((pos "Hb%" ) ))
( "massacre"  v ( m a1 s @ k @ r ) ((pos "H2%" "K6%" ) (subcat )))
( "massacred"  v ( m a1 s @ k @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "massacres"  v ( m a1 s @ k @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "massacring"  v ( m a1 s @ k @ r i ng ) ((pos "Hb%" ) (subcat )))
( "massage"  v ( m a1 s aa zh ) ((pos "H2%" "M6%" ) (subcat )))
( "massaged"  v ( m a1 s aa zh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "massages"  v ( m a1 s aa zh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "massaging"  v ( m a1 s aa zh i ng ) ((pos "Hb%" ) (subcat )))
( "massed"  v ( m a1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "masses"  v ( m a1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "masseur"  n ( m a1 s @@1 r ) ((pos "K6%" ) ))
( "masseurs"  n ( m a1 s @@1 z ) ((pos "Kj%" ) ))
( "masseuse"  n ( m a1 s @@1 z ) ((pos "K6%" ) ))
( "masseuses"  n ( m a1 s @@1 z ) ((pos "Kj$" ) ))
( "massier"  j ( m a1 s i@ r ) ((pos "Or$" ) ))
( "massiest"  j ( m a1 s i i s t ) ((pos "Os$" ) ))
( "massif"  n ( m a1 s ii1 f ) ((pos "K6$" ) ))
( "massifs"  n ( m a1 s ii1 f s ) ((pos "Kj$" ) ))
( "massing"  v ( m a1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "massive"  j ( m a1 s i v ) ((pos "OA%" ) ))
( "massively"  a ( m a1 s i v l ii ) ((pos "Pu%" ) ))
( "massiveness"  n ( m a1 s i v n @ s ) ((pos "L@%" ) ))
( "massy"  j ( m a1 s ii ) ((pos "OD$" ) ))
( "mast"  n ( m aa1 s t ) ((pos "M6%" ) ))
( "master"  v ( m aa1 s t @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "master-at-arms"  n ( m aa2 s t @ r - @ t - aa1 m z ) ((pos "Ki$" ) ))
( "master-key"  n ( m aa2 s t @ - k ii1 ) ((pos "K6%" ) ))
( "master-keys"  n ( m aa2 s t @ - k ii1 z ) ((pos "Kj%" ) ))
( "mastered"  v ( m aa1 s t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "masterful"  j ( m aa1 s t @ f @ l ) ((pos "OA%" ) ))
( "masterfully"  a ( m aa1 s t @ f @ l ii ) ((pos "Pu%" ) ))
( "mastering"  v ( m aa1 s t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "masterless"  j ( m aa1 s t @ l @ s ) ((pos "OA%" ) ))
( "masterly"  j ( m aa1 s t @ l ii ) ((pos "OA%" ) ))
( "mastermind"  v ( m aa1 s t @ m ai n d ) ((pos "H0%" "K6%" ) ))
( "masterminded"  v ( m aa1 s t @ m ai n d i d ) ((pos "Hc%" "Hd%" ) ))
( "masterminding"  v ( m aa1 s t @ m ai n d i ng ) ((pos "Hb%" ) ))
( "masterminds"  v ( m aa1 s t @ m ai n d z ) ((pos "Ha%" "Kj%" ) ))
( "masterpiece"  n ( m aa1 s t @ p ii s ) ((pos "K6%" ) ))
( "masterpieces"  n ( m aa1 s t @ p ii s i z ) ((pos "Kj%" ) ))
( "masters"  v ( m aa1 s t @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "masters-at-arms"  n ( m a2 s t @ z - @ t - aa1 m z ) ((pos "Kj$" ) ))
( "mastership"  n ( m aa1 s t @ sh i p ) ((pos "M6%" ) ))
( "masterships"  n ( m aa1 s t @ sh i p s ) ((pos "Mj$" ) ))
( "masterstroke"  n ( m aa1 s t @ s t r ou k ) ((pos "K6%" ) ))
( "masterstrokes"  n ( m aa1 s t @ s t r ou k s ) ((pos "Kj$" ) ))
( "mastery"  n ( m aa1 s t @ r ii ) ((pos "L@%" ) ))
( "masthead"  n ( m aa1 s t h e d ) ((pos "K6%" ) ))
( "mastheads"  n ( m aa1 s t h e d z ) ((pos "Kj$" ) ))
( "masticate"  v ( m a1 s t i k ei t ) ((pos "H2$" ) (subcat )))
( "masticated"  v ( m a1 s t i k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "masticates"  v ( m a1 s t i k ei t s ) ((pos "Ha$" ) (subcat )))
( "masticating"  v ( m a1 s t i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "mastication"  n ( m a2 s t i k ei1 sh @ n ) ((pos "L@$" ) ))
( "mastiff"  n ( m a1 s t i f ) ((pos "K6%" ) ))
( "mastiffs"  n ( m a1 s t i f s ) ((pos "Kj%" ) ))
( "mastodon"  n ( m a1 s t @ d o n ) ((pos "K6$" ) ))
( "mastodons"  n ( m a1 s t @ d o n z ) ((pos "Kj$" ) ))
( "mastoid"  n ( m a1 s t oi d ) ((pos "K6%" ) ))
( "mastoiditis"  n ( m a2 s t oi d ai1 t i s ) ((pos "L@%" ) ))
( "mastoids"  n ( m a1 s t oi d z ) ((pos "Kj%" ) ))
( "masts"  n ( m aa1 s t s ) ((pos "Mj%" ) ))
( "masturbate"  v ( m a1 s t @ b ei t ) ((pos "J2%" ) (subcat "2A" )))
( "masturbated"  v ( m a1 s t @ b ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "masturbates"  v ( m a1 s t @ b ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "masturbating"  v ( m a1 s t @ b ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "masturbation"  n ( m a2 s t @ b ei1 sh @ n ) ((pos "L@%" ) ))
( "mat"  v ( m a1 t ) ((pos "J4%" "K6%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "mat_e"  n ( m aa1 t ei ) ((pos "L@$" ) ))
( "matador"  n ( m a1 t @ d oo r ) ((pos "K6%" ) ))
( "matadors"  n ( m a1 t @ d oo z ) ((pos "Kj%" ) ))
( "match"  v ( m a1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" "12B" "13B" )))
( "match-point"  n ( m a1 ch - p oi1 n t ) ((pos "K6%" ) ))
( "match-points"  n ( m a1 ch - p oi1 n t s ) ((pos "Kj%" ) ))
( "matchbox"  n ( m a1 ch b o k s ) ((pos "K7%" ) ))
( "matchboxes"  n ( m a1 ch b o k s i z ) ((pos "Kj%" ) ))
( "matched"  v ( m a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "12B" "13B" )))
( "matches"  v ( m a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "12B" "13B" )))
( "matchet"  n ( m a1 ch i t ) ((pos "K6$" ) ))
( "matchets"  n ( m a1 ch i t s ) ((pos "Kj$" ) ))
( "matching"  v ( m a1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "12B" "13B" )))
( "matchless"  j ( m a1 ch l @ s ) ((pos "OA%" ) ))
( "matchlock"  n ( m a1 ch l o k ) ((pos "K6$" ) ))
( "matchlocks"  n ( m a1 ch l o k s ) ((pos "Kj$" ) ))
( "matchmaker"  n ( m a1 ch m ei k @ r ) ((pos "K6%" ) ))
( "matchmakers"  n ( m a1 ch m ei k @ z ) ((pos "Kj%" ) ))
( "matchwood"  n ( m a1 ch w u d ) ((pos "L@%" ) ))
( "mate"  v ( m ei1 t ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "mated"  v ( m ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "material"  n ( m @1 t i@1 r i@ l ) ((pos "M6%" "OA%" ) ))
( "materialism"  n ( m @1 t i@1 r i@ l i z @ m ) ((pos "L@%" ) ))
( "materialist"  n ( m @1 t i@1 r i@ l i s t ) ((pos "K6%" ) ))
( "materialistic"  j ( m @1 t i@2 r i@ l i1 s t i k ) ((pos "OA%" ) ))
( "materialistically"  a ( m @1 t i@2 r i@ l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "materialists"  n ( m @1 t i@1 r i@ l i s t s ) ((pos "Kj%" ) ))
( "materialization"  n ( m @1 t i@2 r i@ l ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "materializations"  n ( m @1 t i@2 r i@ l ai z ei1 sh @ n z ) ((pos "Kj$" ) ))
( "materialize"  v ( m @1 t i@1 r i@ l ai z ) ((pos "I2%" ) (subcat "2A" )))
( "materialized"  v ( m @1 t i@1 r i@ l ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "materializes"  v ( m @1 t i@1 r i@ l ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "materializing"  v ( m @1 t i@1 r i@ l ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "materially"  a ( m @1 t i@1 r i@ l ii ) ((pos "Pu%" ) ))
( "materials"  n ( m @1 t i@1 r i@ l z ) ((pos "Mj%" ) ))
( "maternal"  j ( m @1 t @@1 n l ) ((pos "OA%" ) ))
( "maternally"  a ( m @1 t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "maternity"  n ( m @1 t @@1 n i t ii ) ((pos "L@%" ) ))
( "mates"  v ( m ei1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "matey"  j ( m ei1 t ii ) ((pos "OA%" ) ))
( "mathematical"  j ( m a2 th @ m a1 t i k l ) ((pos "OA%" ) ))
( "mathematically"  a ( m a2 th @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "mathematician"  n ( m a2 th @ m @ t i1 sh @ n ) ((pos "K6%" ) ))
( "mathematicians"  n ( m a2 th @ m @ t i1 sh @ n z ) ((pos "Kj%" ) ))
( "mathematics"  n ( m a2 th @ m a1 t i k s ) ((pos "Lk%" ) ))
( "maths"  n ( m a1 th s ) ((pos "Lk%" ) ))
( "matin_ee"  n ( m a1 t i n ei ) ((pos "K6%" ) ))
( "matin_ees"  n ( m a1 t i n ei z ) ((pos "Kj%" ) ))
( "mating"  v ( m ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "matins"  n ( m a1 t i n z ) ((pos "Kj%" ) ))
( "matriarch"  n ( m ei1 t r i aa k ) ((pos "K6%" ) ))
( "matriarchal"  j ( m ei2 t r i aa1 k l ) ((pos "OA%" ) ))
( "matriarchies"  n ( m ei1 t r i aa k i z ) ((pos "Kj%" ) ))
( "matriarchs"  n ( m ei1 t r i aa k s ) ((pos "Kj%" ) ))
( "matriarchy"  n ( m ei1 t r i aa k ii ) ((pos "K8%" ) ))
( "matric"  n ( m @1 t r i1 k ) ((pos "K6$" ) ))
( "matrices"  n ( m ei1 t r i s ii z ) ((pos "Kj%" ) ))
( "matricide"  n ( m a1 t r i s ai d ) ((pos "M6%" ) ))
( "matricides"  n ( m a1 t r i s ai d z ) ((pos "Mj%" ) ))
( "matrics"  n ( m @1 t r i1 k s ) ((pos "Kj$" ) ))
( "matriculate"  v ( m @1 t r i1 k y u l ei t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "matriculated"  v ( m @1 t r i1 k y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "matriculates"  v ( m @1 t r i1 k y u l ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "matriculating"  v ( m @1 t r i1 k y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "matriculation"  n ( m @1 t r i2 k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "matriculations"  n ( m @1 t r i2 k y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "matrimonial"  j ( m a2 t r i m ou1 n i@ l ) ((pos "OA%" ) ))
( "matrimony"  n ( m a1 t r i m @ n ii ) ((pos "L@%" ) ))
( "matrix"  n ( m ei1 t r i k s ) ((pos "K7%" ) ))
( "matrixes"  n ( m ei1 t r i k s i z ) ((pos "Kj%" ) ))
( "matron"  n ( m ei1 t r @ n ) ((pos "K6%" ) ))
( "matronly"  j ( m ei1 t r @ n l ii ) ((pos "OA%" ) ))
( "matrons"  n ( m ei1 t r @ n z ) ((pos "Kj%" ) ))
( "mats"  v ( m a1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "matt"  j ( m a1 t ) ((pos "OA%" ) ))
( "matted"  v ( m a1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "matter"  v ( m a1 t @ r ) ((pos "I0*" "M6*" ) (subcat "2A" )))
( "matter-of-course"  j ( m a2 t @ r - @ v - k oo1 s ) ((pos "OA%" ) ))
( "matter-of-fact"  j ( m a2 t @ r - @ v - f a1 k t ) ((pos "OA%" ) ))
( "mattered"  v ( m a1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "mattering"  v ( m a1 t @ r i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "matters"  v ( m a1 t @ z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "matting"  v ( m a1 t i ng ) ((pos "Jb$" "L@%" ) (subcat "2A" "2C" "6A" )))
( "mattins"  n ( m a1 t i n z ) ((pos "Kj%" ) ))
( "mattock"  n ( m a1 t @ k ) ((pos "K6$" ) ))
( "mattocks"  n ( m a1 t @ k s ) ((pos "Kj$" ) ))
( "mattress"  n ( m a1 t r i s ) ((pos "K7%" ) ))
( "mattresses"  n ( m a1 t r i s i z ) ((pos "Kj%" ) ))
( "maturate"  v ( m a1 t y u r ei t ) ((pos "I2$" ) (subcat )))
( "maturated"  v ( m a1 t y u r ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "maturates"  v ( m a1 t y u r ei t s ) ((pos "Ia$" ) (subcat )))
( "maturating"  v ( m a1 t y u r ei t i ng ) ((pos "Ib$" ) (subcat )))
( "maturation"  n ( m a2 t y u r ei1 sh @ n ) ((pos "L@%" ) ))
( "mature"  v ( m @1 t y u@1 r ) ((pos "J2%" "OA%" ) (subcat "2A" )))
( "matured"  v ( m @1 t y u@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "maturely"  a ( m @1 t y u@1 l ii ) ((pos "Pu%" ) ))
( "matures"  v ( m @1 t y u@1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "maturing"  v ( m @1 t y u@1 r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "maturity"  n ( m @1 t y u@1 r i t ii ) ((pos "L@%" ) ))
( "matutinal"  j ( m @1 t y uu1 t i n l ) ((pos "OA$" ) ))
( "maudlin"  j ( m oo1 d l i n ) ((pos "OA%" ) ))
( "maul"  v ( m oo1 l ) ((pos "H0%" ) (subcat "6A" )))
( "mauled"  v ( m oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mauling"  v ( m oo1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "mauls"  v ( m oo1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "maulstick"  n ( m oo1 l s t i k ) ((pos "K6$" ) ))
( "maulsticks"  n ( m oo1 l s t i k s ) ((pos "Kj$" ) ))
( "maunder"  v ( m oo1 n d @ r ) ((pos "I0$" ) (subcat "2A" )))
( "maundered"  v ( m oo1 n d @ d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "maundering"  v ( m oo1 n d @ r i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "maunders"  v ( m oo1 n d @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "mausoleum"  n ( m oo2 s @ l ii1 @ m ) ((pos "K6%" ) ))
( "mausoleums"  n ( m oo2 s @ l ii1 @ m z ) ((pos "Kj%" ) ))
( "mauve"  n ( m ou1 v ) ((pos "M6%" "OA%" ) ))
( "mauves"  n ( m ou1 v z ) ((pos "Mj$" ) ))
( "maverick"  n ( m a1 v @ r i k ) ((pos "K6%" ) ))
( "mavericks"  n ( m a1 v @ r i k s ) ((pos "Kj%" ) ))
( "mavis"  n ( m ei1 v i s ) ((pos "K7$" ) ))
( "mavises"  n ( m ei1 v i s i z ) ((pos "Kj$" ) ))
( "maw"  n ( m oo1 ) ((pos "K6%" ) ))
( "mawkish"  j ( m oo1 k i sh ) ((pos "OA%" ) ))
( "mawkishly"  a ( m oo1 k i sh l ii ) ((pos "Pu%" ) ))
( "mawkishness"  n ( m oo1 k i sh n @ s ) ((pos "L@%" ) ))
( "maws"  n ( m oo1 z ) ((pos "Kj$" ) ))
( "max"  n ( m a1 k s ) ((pos "Y>%" ) ))
( "maxi-"  j ( m a1 k s i - ) ((pos "U-%" ) ))
( "maxim"  n ( m a1 k s i m ) ((pos "K6%" ) ))
( "maxima"  n ( m a1 k s i m @ ) ((pos "Kj$" ) ))
( "maximal"  j ( m a1 k s i m l ) ((pos "OA$" ) ))
( "maximally"  a ( m a1 k s i m @ l ii ) ((pos "Pu$" ) ))
( "maximization"  n ( m a2 k s i m ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "maximizations"  n ( m a2 k s i m ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "maximize"  v ( m a1 k s i m ai z ) ((pos "H2%" ) (subcat )))
( "maximized"  v ( m a1 k s i m ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "maximizes"  v ( m a1 k s i m ai z i z ) ((pos "Ha%" ) (subcat )))
( "maximizing"  v ( m a1 k s i m ai z i ng ) ((pos "Hb%" ) (subcat )))
( "maxims"  n ( m a1 k s i m z ) ((pos "Kj%" ) ))
( "maximum"  n ( m a1 k s i m @ m ) ((pos "K6%" "OA%" ) ))
( "maximums"  n ( m a1 k s i m @ m z ) ((pos "Kj$" ) ))
( "may"  v ( m ei ) ((pos "G5*" "Ga*" ) (subcat )))
( "may-beetle"  n ( m ei1 - b ii t l ) ((pos "K6$" ) ))
( "may-beetles"  n ( m ei1 - b ii t l z ) ((pos "Kj$" ) ))
( "may-bug"  n ( m ei1 - b uh g ) ((pos "K6$" ) ))
( "may-bugs"  n ( m ei1 - b uh g z ) ((pos "Kj$" ) ))
( "maybe"  a ( m ei1 b ii ) ((pos "Pu%" ) ))
( "mayday"  n ( m ei1 d ei ) ((pos "K6%" ) ))
( "maydays"  n ( m ei1 d ei z ) ((pos "Kj%" ) ))
( "mayflies"  n ( m ei1 f l ai z ) ((pos "Kj%" ) ))
( "mayfly"  n ( m ei1 f l ai ) ((pos "K8%" ) ))
( "mayhem"  n ( m ei1 h e m ) ((pos "L@%" ) ))
( "mayn't"  v ( m ei1 @ n t ) ((pos "Gg%" ) ))
( "mayonnaise"  n ( m ei2 @ n ei1 z ) ((pos "L@%" ) ))
( "mayor"  n ( m e@1 r ) ((pos "K6%" ) ))
( "mayoral"  j ( m e@1 r @ l ) ((pos "OA%" ) ))
( "mayoralties"  n ( m e@1 r @ l t i z ) ((pos "Kj$" ) ))
( "mayoralty"  n ( m e@1 r @ l t ii ) ((pos "K8%" ) ))
( "mayoress"  n ( m e@1 r e1 s ) ((pos "K7%" ) ))
( "mayoresses"  n ( m e@1 r e1 s i z ) ((pos "Kj%" ) ))
( "mayors"  n ( m e@1 z ) ((pos "Kj%" ) ))
( "maypole"  n ( m ei1 p ou l ) ((pos "K6%" ) ))
( "maypoles"  n ( m ei1 p ou l z ) ((pos "Kj%" ) ))
( "maze"  n ( m ei1 z ) ((pos "K6%" ) ))
( "mazed"  j ( m ei1 z d ) ((pos "OA$" ) ))
( "mazes"  n ( m ei1 z i z ) ((pos "Kj%" ) ))
( "mazurka"  n ( m @1 z @@1 k @ ) ((pos "K6%" ) ))
( "mazurkas"  n ( m @1 z @@1 k @ z ) ((pos "Kj%" ) ))
( "me"  prp ( m ii ) ((pos "Qx*" ) ))
( "mead"  n ( m ii1 d ) ((pos "M6$" ) ))
( "meadow"  n ( m e1 d ou ) ((pos "M6%" ) ))
( "meadows"  n ( m e1 d ou z ) ((pos "Mj%" ) ))
( "meads"  n ( m ii1 d z ) ((pos "Mj$" ) ))
( "meagre"  j ( m ii1 g @ r ) ((pos "OA%" ) ))
( "meagrely"  a ( m ii1 g @ l ii ) ((pos "Pu%" ) ))
( "meagreness"  n ( m ii1 g @ n @ s ) ((pos "L@%" ) ))
( "meal"  n ( m ii1 l ) ((pos "M6%" ) ))
( "mealie"  n ( m ii1 l ii ) ((pos "K6$" ) ))
( "mealier"  j ( m ii1 l i@ r ) ((pos "Or$" ) ))
( "mealies"  n ( m ii1 l i z ) ((pos "Kj$" ) ))
( "mealiest"  j ( m ii1 l i i s t ) ((pos "Os$" ) ))
( "meals"  n ( m ii1 l z ) ((pos "Mj%" ) ))
( "mealtime"  n ( m ii1 l t ai m ) ((pos "K6%" ) ))
( "mealtimes"  n ( m ii1 l t ai m z ) ((pos "Kj%" ) ))
( "mealy"  j ( m ii1 l ii ) ((pos "OD$" ) ))
( "mealy-bug"  n ( m ii1 l i - b uh g ) ((pos "K6%" ) ))
( "mealy-bugs"  n ( m ii1 l i - b uh g z ) ((pos "Kj%" ) ))
( "mealy-mouthed"  j ( m ii1 l i - m au dh d ) ((pos "OA%" ) ))
( "mean"  v ( m ii1 n ) ((pos "H5*" "K6%" "OC*" ) (subcat "6A" "6A" "6C" "7A" "9" "12A" "13A" "14" "16B" )))
( "meander"  v ( m i1 a1 n d @ r ) ((pos "I0%" ) (subcat "2A" )))
( "meandered"  v ( m i1 a1 n d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "meandering"  v ( m i1 a1 n d @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "meanderingly"  a ( m i1 a1 n d r i ng l ii ) ((pos "Pu%" ) ))
( "meanderings"  n ( m i1 a1 n d r i ng z ) ((pos "Kj%" ) ))
( "meanders"  v ( m i1 a1 n d @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "meaner"  j ( m ii1 n @ r ) ((pos "Or%" ) ))
( "meanest"  j ( m ii1 n i s t ) ((pos "Os%" ) ))
( "meanie"  n ( m ii1 n ii ) ((pos "K6%" ) ))
( "meanies"  n ( m ii1 n i z ) ((pos "Kj%" ) ))
( "meaning"  v ( m ii1 n i ng ) ((pos "Hb%" "M6%" "OA%" ) (subcat "6A" "6A" "6C" "7A" "9" "12A" "13A" "14" "16B" )))
( "meaningful"  j ( m ii1 n i ng f @ l ) ((pos "OA%" ) ))
( "meaningfully"  a ( m ii1 n i ng f @ l ii ) ((pos "Pu%" ) ))
( "meaningless"  j ( m ii1 n i ng l @ s ) ((pos "OA%" ) ))
( "meaningly"  a ( m ii1 n i ng l ii ) ((pos "Pu%" ) ))
( "meanings"  n ( m ii1 n i ng z ) ((pos "Mj%" ) ))
( "meanly"  a ( m ii1 n l ii ) ((pos "Pu%" ) ))
( "meanness"  n ( m ii1 n - n @ s ) ((pos "L@%" ) ))
( "means"  v ( m ii1 n z ) ((pos "Ha*" "Kj*" ) (subcat "6A" "6C" "7A" "9" "12A" "13A" "14" "16B" )))
( "meant"  v ( m e1 n t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6A" "6C" "7A" "9" "12A" "13A" "14" "16B" )))
( "meantime"  n ( m ii1 n t ai m ) ((pos "L@%" "Pu%" ) ))
( "meanwhile"  a ( m ii1 n w ai l ) ((pos "Pu%" ) ))
( "meany"  n ( m ii1 n ii ) ((pos "K8%" ) ))
( "measles"  n ( m ii1 z l z ) ((pos "Lk%" ) ))
( "measly"  j ( m ii1 z l ii ) ((pos "OA%" ) ))
( "measurable"  j ( m e1 zh @ r @ b l ) ((pos "OA%" ) ))
( "measurably"  a ( m e1 zh @ r @ b l ii ) ((pos "Pu%" ) ))
( "measure"  v ( m e1 zh @ r ) ((pos "J2%" "M6%" ) (subcat "2A" "2B" "6A" "15A" )))
( "measured"  v ( m e1 zh @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "6A" "15A" )))
( "measureless"  j ( m e1 zh @ l @ s ) ((pos "OA%" ) ))
( "measurement"  n ( m e1 zh @ m @ n t ) ((pos "M6%" ) ))
( "measurements"  n ( m e1 zh @ m @ n t s ) ((pos "Mj%" ) ))
( "measures"  v ( m e1 zh @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "6A" "15A" )))
( "measuring"  v ( m e1 zh @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "6A" "15A" )))
( "meat"  n ( m ii1 t ) ((pos "M6%" ) ))
( "meat-safe"  n ( m ii1 t - s ei f ) ((pos "K6%" ) ))
( "meat-safes"  n ( m ii1 t - s ei f s ) ((pos "Kj$" ) ))
( "meatball"  n ( m ii1 t b oo l ) ((pos "K6%" ) ))
( "meatballs"  n ( m ii1 t b oo l z ) ((pos "Kj%" ) ))
( "meatier"  j ( m ii1 t i@ r ) ((pos "Or%" ) ))
( "meatiest"  j ( m ii1 t i i s t ) ((pos "Os%" ) ))
( "meatless"  j ( m ii1 t l @ s ) ((pos "OA$" ) ))
( "meats"  n ( m ii1 t s ) ((pos "Mj%" ) ))
( "meaty"  j ( m ii1 t ii ) ((pos "OD%" ) ))
( "mechanic"  n ( m i1 k a1 n i k ) ((pos "K6%" ) ))
( "mechanical"  j ( m i1 k a1 n i k l ) ((pos "OA%" ) ))
( "mechanically"  a ( m i1 k a1 n i k l ii ) ((pos "Pu%" ) ))
( "mechanics"  n ( m i1 k a1 n i k s ) ((pos "Kj%" "Lk%" ) ))
( "mechanism"  n ( m e1 k @ n i z @ m ) ((pos "M6%" ) ))
( "mechanisms"  n ( m e1 k @ n i z @ m z ) ((pos "Mj%" ) ))
( "mechanistic"  j ( m e2 k @ n i1 s t i k ) ((pos "OA%" ) ))
( "mechanistically"  a ( m e2 k @ n i1 s t i k l ii ) ((pos "Pu%" ) ))
( "mechanization"  n ( m e2 k @ n ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "mechanizations"  n ( m e2 k @ n ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "mechanize"  v ( m e1 k @ n ai z ) ((pos "H2%" ) (subcat )))
( "mechanized"  v ( m e1 k @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mechanizes"  v ( m e1 k @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "mechanizing"  v ( m e1 k @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "medal"  n ( m e1 d l ) ((pos "K6%" ) ))
( "medalist"  n ( m e1 d @ l i s t ) ((pos "K6%" ) ))
( "medalists"  n ( m e1 d @ l i s t s ) ((pos "Kj%" ) ))
( "medallion"  n ( m i1 d a1 l i@ n ) ((pos "K6%" ) ))
( "medallions"  n ( m i1 d a1 l i@ n z ) ((pos "Kj%" ) ))
( "medallist"  n ( m e1 d @ l i s t ) ((pos "K6$" ) ))
( "medallists"  n ( m e1 d @ l i s t s ) ((pos "Kj$" ) ))
( "medals"  n ( m e1 d l z ) ((pos "Kj%" ) ))
( "meddle"  v ( m e1 d l ) ((pos "I2%" ) (subcat "2A" )))
( "meddled"  v ( m e1 d l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "meddler"  n ( m e1 d l @ r ) ((pos "K6%" ) ))
( "meddlers"  n ( m e1 d l @ z ) ((pos "Kj%" ) ))
( "meddles"  v ( m e1 d l z ) ((pos "Ia%" ) (subcat "2A" )))
( "meddlesome"  j ( m e1 d l s @ m ) ((pos "OA%" ) ))
( "meddling"  v ( m e1 d l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "media"  n ( m ii1 d i@ ) ((pos "Kj%" ) ))
( "mediaeval"  j ( m e2 d i ii1 v @ l ) ((pos "OA%" ) ))
( "medial"  j ( m ii1 d i@ l ) ((pos "OA%" ) ))
( "medially"  a ( m ii1 d i@ l ii ) ((pos "Pu$" ) ))
( "median"  n ( m ii1 d i@ n ) ((pos "K6$" "OA$" ) ))
( "medians"  n ( m ii1 d i@ n z ) ((pos "Kj$" ) ))
( "mediate"  v ( m ii1 d i ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "mediated"  v ( m ii1 d i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "mediates"  v ( m ii1 d i ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "mediating"  v ( m ii1 d i ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "mediation"  n ( m ii2 d i ei1 sh @ n ) ((pos "L@%" ) ))
( "mediator"  n ( m ii1 d i ei t @ r ) ((pos "K6%" ) ))
( "mediators"  n ( m ii1 d i ei t @ z ) ((pos "Kj%" ) ))
( "medic"  n ( m e1 d i k ) ((pos "K6%" ) ))
( "medical"  n ( m e1 d i k l ) ((pos "K6%" "OA%" ) ))
( "medically"  a ( m e1 d i k l ii ) ((pos "Pu%" ) ))
( "medicals"  n ( m e1 d i k l z ) ((pos "Kj%" ) ))
( "medicament"  n ( m i1 d i1 k @ m @ n t ) ((pos "K6%" ) ))
( "medicaments"  n ( m i1 d i1 k @ m @ n t s ) ((pos "Kj%" ) ))
( "medicate"  v ( m e1 d i k ei t ) ((pos "H2%" ) (subcat )))
( "medicated"  v ( m e1 d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "medicates"  v ( m e1 d i k ei t s ) ((pos "Ha%" ) (subcat )))
( "medicating"  v ( m e1 d i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "medication"  n ( m e2 d i k ei1 sh @ n ) ((pos "M6%" ) ))
( "medications"  n ( m e2 d i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "medicinal"  j ( m i1 d i1 s n @ l ) ((pos "OA%" ) ))
( "medicine"  n ( m e1 d s n ) ((pos "M6%" ) ))
( "medicine-ball"  n ( m e1 d s n - b oo l ) ((pos "K6%" ) ))
( "medicine-balls"  n ( m e1 d s n - b oo l z ) ((pos "Kj%" ) ))
( "medicine-chest"  n ( m e1 d s n - ch e s t ) ((pos "K6%" ) ))
( "medicine-chests"  n ( m e1 d s n - ch e s t s ) ((pos "Kj%" ) ))
( "medicine-man"  n ( m e1 d s n - m a n ) ((pos "Ki%" ) ))
( "medicine-men"  n ( m e1 d s n - m e n ) ((pos "Kj%" ) ))
( "medicines"  n ( m e1 d s n z ) ((pos "Mj%" ) ))
( "medico"  n ( m e1 d i k ou ) ((pos "K6$" ) ))
( "medicos"  n ( m e1 d i k ou z ) ((pos "Kj$" ) ))
( "medics"  n ( m e1 d i k s ) ((pos "Kj%" ) ))
( "medieval"  j ( m e2 d i ii1 v @ l ) ((pos "OA%" ) ))
( "mediocre"  j ( m ii2 d i ou1 k @ r ) ((pos "OA%" ) ))
( "mediocrities"  n ( m ii2 d i o1 k r i t i z ) ((pos "Mj%" ) ))
( "mediocrity"  n ( m ii2 d i o1 k r i t ii ) ((pos "M8%" ) ))
( "meditate"  v ( m e1 d i t ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "meditated"  v ( m e1 d i t ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "meditates"  v ( m e1 d i t ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "meditating"  v ( m e1 d i t ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "meditation"  n ( m e2 d i t ei1 sh @ n ) ((pos "M6%" ) ))
( "meditations"  n ( m e2 d i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "meditative"  j ( m e1 d i t @ t i v ) ((pos "OA%" ) ))
( "meditatively"  a ( m e1 d i t @ t i v l ii ) ((pos "Pu%" ) ))
( "medium"  n ( m ii1 d i@ m ) ((pos "K6%" "OA%" ) ))
( "mediums"  n ( m ii1 d i@ m z ) ((pos "Kj%" ) ))
( "medlar"  n ( m e1 d l @ r ) ((pos "K6$" ) ))
( "medlars"  n ( m e1 d l @ z ) ((pos "Kj$" ) ))
( "medley"  n ( m e1 d l ii ) ((pos "K6%" ) ))
( "medleys"  n ( m e1 d l i z ) ((pos "Kj%" ) ))
( "meed"  n ( m ii1 d ) ((pos "K6$" ) ))
( "meeds"  n ( m ii1 d z ) ((pos "Kj$" ) ))
( "meek"  j ( m ii1 k ) ((pos "OC%" ) ))
( "meeker"  j ( m ii1 k @ r ) ((pos "Or%" ) ))
( "meekest"  j ( m ii1 k i s t ) ((pos "Os%" ) ))
( "meekly"  a ( m ii1 k l ii ) ((pos "Pu%" ) ))
( "meekness"  n ( m ii1 k n @ s ) ((pos "L@%" ) ))
( "meerschaum"  n ( m i@1 sh @ m ) ((pos "M6$" ) ))
( "meerschaums"  n ( m i@1 sh @ m z ) ((pos "Mj$" ) ))
( "meet"  v ( m ii1 t ) ((pos "J5%" "K6$" "OA$" ) (subcat "2A" "2C" )))
( "meeting"  v ( m ii1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" )))
( "meeting-house"  n ( m ii1 t i ng - h au s ) ((pos "K6%" ) ))
( "meeting-houses"  n ( m ii1 t i ng - h au z i z ) ((pos "Kj%" ) ))
( "meeting-place"  n ( m ii1 t i ng - p l ei s ) ((pos "K6%" ) ))
( "meeting-places"  n ( m ii1 t i ng - p l ei s i z ) ((pos "Kj%" ) ))
( "meetings"  n ( m ii1 t i ng z ) ((pos "Kj%" ) ))
( "meets"  v ( m ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "megacycle"  n ( m e1 g @ s ai k l ) ((pos "K6$" ) ))
( "megacycles"  n ( m e1 g @ s ai k l z ) ((pos "Kj$" ) ))
( "megadeath"  n ( m e1 g @ d e th ) ((pos "K6$" ) ))
( "megadeaths"  n ( m e1 g @ d e th s ) ((pos "Kj$" ) ))
( "megalith"  n ( m e1 g @ l i th ) ((pos "K6$" ) ))
( "megalithic"  j ( m e2 g @ l i1 th i k ) ((pos "OA$" ) ))
( "megaliths"  n ( m e1 g @ l i th s ) ((pos "Kj$" ) ))
( "megalomania"  n ( m e2 g @ l @ m ei1 n i@ ) ((pos "L@%" ) ))
( "megalomaniac"  n ( m e2 g @ l @ m ei1 n i a k ) ((pos "K6%" ) ))
( "megalomaniacs"  n ( m e2 g @ l @ m ei1 n i a k s ) ((pos "Kj%" ) ))
( "megaphone"  n ( m e1 g @ f ou n ) ((pos "K6%" ) ))
( "megaphones"  n ( m e1 g @ f ou n z ) ((pos "Kj%" ) ))
( "megaton"  n ( m e1 g @ t uh n ) ((pos "K6%" ) ))
( "megatons"  n ( m e1 g @ t uh n z ) ((pos "Kj%" ) ))
( "megrim"  n ( m ii1 g r i m ) ((pos "K6$" ) ))
( "megrims"  n ( m ii1 g r i m z ) ((pos "Kj$" ) ))
( "meiosis"  n ( m ai1 ou1 s i s ) ((pos "L@$" ) ))
( "melancholia"  n ( m e2 l @ n k ou1 l i@ ) ((pos "L@%" ) ))
( "melancholic"  j ( m e2 l @ ng k o1 l i k ) ((pos "OA%" ) ))
( "melancholy"  n ( m e1 l @ ng k o l ii ) ((pos "L@%" "OA%" ) ))
( "meliorate"  v ( m ii1 l i@ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "meliorated"  v ( m ii1 l i@ r ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "meliorates"  v ( m ii1 l i@ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "meliorating"  v ( m ii1 l i@ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "melioration"  n ( m ii2 l i@ r ei1 sh @ n ) ((pos "M6$" ) ))
( "meliorations"  n ( m ii2 l i@ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "meliorism"  n ( m ii1 l i@ r i z @ m ) ((pos "L@$" ) ))
( "mellifluous"  j ( m e1 l i1 f l u@ s ) ((pos "OA%" ) ))
( "mellow"  v ( m e1 l ou ) ((pos "J0%" "OC%" ) (subcat "2A" )))
( "mellowed"  v ( m e1 l ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mellower"  j ( m e1 l ou @ r ) ((pos "Or%" ) ))
( "mellowest"  j ( m e1 l ou i s t ) ((pos "Os%" ) ))
( "mellowing"  v ( m e1 l ou i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mellowly"  a ( m e1 l ou l ii ) ((pos "Pu%" ) ))
( "mellowness"  n ( m e1 l ou n @ s ) ((pos "L@%" ) ))
( "mellows"  v ( m e1 l ou z ) ((pos "Ja%" ) (subcat "2A" )))
( "melodic"  j ( m i1 l o1 d i k ) ((pos "OA%" ) ))
( "melodies"  n ( m e1 l @ d i z ) ((pos "Mj%" ) ))
( "melodious"  j ( m i1 l ou1 d i@ s ) ((pos "OA%" ) ))
( "melodiously"  a ( m i1 l ou1 d i@ s l ii ) ((pos "Pu%" ) ))
( "melodiousness"  n ( m i1 l ou1 d i@ s n @ s ) ((pos "L@%" ) ))
( "melodrama"  n ( m e1 l @ d r aa m @ ) ((pos "M6%" ) ))
( "melodramas"  n ( m e1 l @ d r aa m @ z ) ((pos "Mj%" ) ))
( "melodramatic"  j ( m e2 l @ d r @ m a1 t i k ) ((pos "OA%" ) ))
( "melodramatically"  a ( m e2 l @ d r @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "melody"  n ( m e1 l @ d ii ) ((pos "M8%" ) ))
( "melon"  n ( m e1 l @ n ) ((pos "K6%" ) ))
( "melons"  n ( m e1 l @ n z ) ((pos "Kj%" ) ))
( "melt"  v ( m e1 l t ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "melted"  v ( m e1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "melting"  v ( m e1 l t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "melting-point"  n ( m e1 l t i ng - p oi n t ) ((pos "K6%" ) ))
( "melting-points"  n ( m e1 l t i ng - p oi n t s ) ((pos "Kj%" ) ))
( "melting-pot"  n ( m e1 l t i ng - p o t ) ((pos "K6%" ) ))
( "melting-pots"  n ( m e1 l t i ng - p o t s ) ((pos "Kj$" ) ))
( "melts"  v ( m e1 l t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "member"  n ( m e1 m b @ r ) ((pos "K6%" ) ))
( "members"  n ( m e1 m b @ z ) ((pos "Kj%" ) ))
( "membership"  n ( m e1 m b @ sh i p ) ((pos "L@%" ) ))
( "membrane"  n ( m e1 m b r ei n ) ((pos "M6%" ) ))
( "membranes"  n ( m e1 m b r ei n z ) ((pos "Mj%" ) ))
( "membranous"  j ( m e1 m b r @ n @ s ) ((pos "OA$" ) ))
( "memento"  n ( m i1 m e1 n t ou ) ((pos "K6%" ) ))
( "mementoes"  n ( m i1 m e1 n t ou z ) ((pos "Kj%" ) ))
( "mementos"  n ( m i1 m e1 n t ou z ) ((pos "Kj%" ) ))
( "memo"  n ( m e1 m ou ) ((pos "K6%" ) ))
( "memoir"  n ( m e1 m w aa r ) ((pos "K6%" ) ))
( "memoirs"  n ( m e1 m w aa z ) ((pos "Kj%" ) ))
( "memorable"  j ( m e1 m @ r @ b l ) ((pos "OA%" ) ))
( "memorably"  a ( m e1 m @ r @ b l ii ) ((pos "Pu%" ) ))
( "memoranda"  n ( m e2 m @ r a1 n d @ ) ((pos "Kj%" ) ))
( "memorandum"  n ( m e2 m @ r a1 n d @ m ) ((pos "K6%" ) ))
( "memorandums"  n ( m e2 m @ r a1 n d @ m z ) ((pos "Kj%" ) ))
( "memorial"  n ( m i1 m oo1 r i@ l ) ((pos "K6%" ) ))
( "memorialize"  v ( m i1 m oo1 r i@ l ai z ) ((pos "H2$" ) (subcat )))
( "memorialized"  v ( m i1 m oo1 r i@ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "memorializes"  v ( m i1 m oo1 r i@ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "memorializing"  v ( m i1 m oo1 r i@ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "memorials"  n ( m i1 m oo1 r i@ l z ) ((pos "Kj%" ) ))
( "memories"  n ( m e1 m @ r i z ) ((pos "Mj%" ) ))
( "memorize"  v ( m e1 m @ r ai z ) ((pos "H2%" ) (subcat )))
( "memorized"  v ( m e1 m @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "memorizes"  v ( m e1 m @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "memorizing"  v ( m e1 m @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "memory"  n ( m e1 m @ r ii ) ((pos "M8%" ) ))
( "memos"  n ( m e1 m ou z ) ((pos "Kj%" ) ))
( "memsahib"  n ( m e1 m s aa b ) ((pos "K6$" ) ))
( "memsahibs"  n ( m e1 m s aa b z ) ((pos "Kj$" ) ))
( "men"  n ( m e1 n ) ((pos "Kj*" ) ))
( "men-at-arms"  n ( m e2 n - @ t - aa1 m z ) ((pos "Kj%" ) ))
( "men-of-war"  n ( m e2 n - @ v - w oo1 r ) ((pos "Kj%" ) ))
( "menace"  v ( m e1 n @ s ) ((pos "H2%" "M6%" ) (subcat )))
( "menaced"  v ( m e1 n @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "menaces"  v ( m e1 n @ s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "menacing"  v ( m e1 n @ s i ng ) ((pos "Hb%" ) (subcat )))
( "menacingly"  a ( m e1 n @ s i ng l ii ) ((pos "Pu%" ) ))
( "menagerie"  n ( m i1 n a1 jh @ r ii ) ((pos "K6%" ) ))
( "menageries"  n ( m i1 n a1 jh @ r i z ) ((pos "Kj%" ) ))
( "mend"  v ( m e1 n d ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "mendacious"  j ( m e1 n d ei1 sh @ s ) ((pos "OA%" ) ))
( "mendaciously"  a ( m e1 n d ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "mendacities"  n ( m e1 n d a1 s i t i z ) ((pos "Mj$" ) ))
( "mendacity"  n ( m e1 n d a1 s i t ii ) ((pos "M8%" ) ))
( "mended"  v ( m e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mender"  n ( m e1 n d @ r ) ((pos "K6%" ) ))
( "menders"  n ( m e1 n d @ z ) ((pos "Kj%" ) ))
( "mendicant"  n ( m e1 n d i k @ n t ) ((pos "K6$" "OA$" ) ))
( "mendicants"  n ( m e1 n d i k @ n t s ) ((pos "Kj$" ) ))
( "mending"  v ( m e1 n d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "mends"  v ( m e1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "menfolk"  n ( m e1 n f ou k ) ((pos "Kj%" ) ))
( "menial"  n ( m ii1 n i@ l ) ((pos "K6$" "OA%" ) ))
( "menially"  a ( m ii1 n i@ l ii ) ((pos "Pu%" ) ))
( "menials"  n ( m ii1 n i@ l z ) ((pos "Kj$" ) ))
( "meningitis"  n ( m e2 n i n jh ai1 t i s ) ((pos "L@%" ) ))
( "menopause"  n ( m e1 n @ p oo z ) ((pos "K6%" ) ))
( "menopauses"  n ( m e1 n @ p oo z i z ) ((pos "Kj$" ) ))
( "menses"  n ( m e1 n s ii z ) ((pos "Kj$" ) ))
( "menstrual"  j ( m e1 n s t r u@ l ) ((pos "OA%" ) ))
( "menstruate"  v ( m e1 n s t r u ei t ) ((pos "I2%" ) (subcat )))
( "menstruated"  v ( m e1 n s t r u ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "menstruates"  v ( m e1 n s t r u ei t s ) ((pos "Ia%" ) (subcat )))
( "menstruating"  v ( m e1 n s t r u ei t i ng ) ((pos "Ib%" ) (subcat )))
( "menstruation"  n ( m e2 n s t r u ei1 sh @ n ) ((pos "L@%" ) ))
( "mensurable"  j ( m e1 n s y u r @ b l ) ((pos "OA$" ) ))
( "mensuration"  n ( m e2 n s y u r ei1 sh @ n ) ((pos "M6$" ) ))
( "mensurations"  n ( m e2 n s y u r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "mental"  j ( m e1 n t l ) ((pos "OA%" ) ))
( "mentalities"  n ( m e1 n t a1 l i t i z ) ((pos "Mj$" ) ))
( "mentality"  n ( m e1 n t a1 l i t ii ) ((pos "M8%" ) ))
( "mentally"  a ( m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "menthol"  n ( m e1 n th o l ) ((pos "L@%" ) ))
( "mentholated"  j ( m e1 n th @ l ei t i d ) ((pos "OA%" ) ))
( "mention"  v ( m e1 n sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" "6C" "9" )))
( "mentioned"  v ( m e1 n sh @ n d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat "6A" "6C" "9" )))
( "mentioning"  v ( m e1 n sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" )))
( "mentions"  v ( m e1 n sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6C" "9" )))
( "mentor"  n ( m e1 n t oo r ) ((pos "K6%" ) ))
( "mentors"  n ( m e1 n t oo z ) ((pos "Kj%" ) ))
( "menu"  n ( m e1 n y uu ) ((pos "K6%" ) ))
( "menus"  n ( m e1 n y uu z ) ((pos "Kj%" ) ))
( "mercantile"  j ( m @@1 k @ n t ai l ) ((pos "OA%" ) ))
( "mercenaries"  n ( m @@1 s i n @ r i z ) ((pos "Kj%" ) ))
( "mercenary"  n ( m @@1 s i n @ r ii ) ((pos "K8%" "OA%" ) ))
( "mercer"  n ( m @@1 s @ r ) ((pos "K6$" ) ))
( "mercerize"  v ( m @@1 s @ r ai z ) ((pos "H2$" ) (subcat )))
( "mercerized"  v ( m @@1 s @ r ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "mercerizes"  v ( m @@1 s @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "mercerizing"  v ( m @@1 s @ r ai z i ng ) ((pos "Hb$" ) (subcat )))
( "mercers"  n ( m @@1 s @ z ) ((pos "Kj$" ) ))
( "merchandise"  n ( m @@1 ch @ n d ai z ) ((pos "L@%" ) ))
( "merchant"  n ( m @@1 ch @ n t ) ((pos "K6%" ) ))
( "merchantman"  n ( m @@1 ch @ n t m @ n ) ((pos "Ki%" ) ))
( "merchantmen"  n ( m @@1 ch @ n t m @ n ) ((pos "Kj%" ) ))
( "merchants"  n ( m @@1 ch @ n t s ) ((pos "Kj%" ) ))
( "mercies"  n ( m @@1 s i z ) ((pos "Mj%" ) ))
( "merciful"  j ( m @@1 s i f @ l ) ((pos "OA%" ) ))
( "mercifully"  a ( m @@1 s i f @ l ii ) ((pos "Pu%" ) ))
( "merciless"  j ( m @@1 s i l i s ) ((pos "OA%" ) ))
( "mercilessly"  a ( m @@1 s i l i s l ii ) ((pos "Pu%" ) ))
( "mercurial"  j ( m @@1 k y u@1 r i@ l ) ((pos "OA%" ) ))
( "mercury"  n ( m @@1 k y u r ii ) ((pos "L@%" ) ))
( "mercy"  n ( m @@1 s ii ) ((pos "M8%" ) ))
( "mere"  n ( m i@1 r ) ((pos "K6$" "OA%" ) ))
( "merely"  a ( m i@1 l ii ) ((pos "Pu%" ) ))
( "meres"  n ( m i@1 z ) ((pos "Kj$" ) ))
( "meretricious"  j ( m e2 r i t r i1 sh @ s ) ((pos "OA%" ) ))
( "meretriciously"  a ( m e2 r i t r i1 sh @ s l ii ) ((pos "Pu$" ) ))
( "meretriciousness"  n ( m e2 r i t r i1 sh @ s n @ s ) ((pos "L@$" ) ))
( "merge"  v ( m @@1 jh ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "merged"  v ( m @@1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "merger"  n ( m @@1 jh @ r ) ((pos "M6%" ) ))
( "mergers"  n ( m @@1 jh @ z ) ((pos "Mj%" ) ))
( "merges"  v ( m @@1 jh i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "merging"  v ( m @@1 jh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "meridian"  n ( m @1 r i1 d i@ n ) ((pos "K6%" ) ))
( "meridians"  n ( m @1 r i1 d i@ n z ) ((pos "Kj%" ) ))
( "meridional"  j ( m @1 r i1 d i@ n l ) ((pos "OA$" ) ))
( "meringue"  n ( m @1 r a1 ng ) ((pos "M6%" ) ))
( "meringues"  n ( m @1 r a1 ng z ) ((pos "Mj%" ) ))
( "merino"  n ( m @1 r ii1 n ou ) ((pos "L@$" ) ))
( "merino-sheep"  n ( m @1 r ii2 n ou - sh ii1 p ) ((pos "K9$" ) ))
( "merit"  v ( m e1 r i t ) ((pos "H0%" "M6%" ) (subcat )))
( "merited"  v ( m e1 r i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "meriting"  v ( m e1 r i t i ng ) ((pos "Hb%" ) (subcat )))
( "meritocracies"  n ( m e2 r i t o1 k r @ s i z ) ((pos "Kj%" ) ))
( "meritocracy"  n ( m e2 r i t o1 k r @ s ii ) ((pos "K8%" ) ))
( "meritocratic"  j ( m e2 r i t @ k r a1 t i k ) ((pos "OA%" ) ))
( "meritorious"  j ( m e2 r i t oo1 r i@ s ) ((pos "OA%" ) ))
( "meritoriously"  a ( m e2 r i t oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "merits"  v ( m e1 r i t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "mermaid"  n ( m @@1 m ei d ) ((pos "K6%" ) ))
( "mermaids"  n ( m @@1 m ei d z ) ((pos "Kj%" ) ))
( "merman"  n ( m @@1 m @ n ) ((pos "Ki$" ) ))
( "mermen"  n ( m @@1 m @ n ) ((pos "Kj$" ) ))
( "merrier"  j ( m e1 r i@ r ) ((pos "Or%" ) ))
( "merriest"  j ( m e1 r i i s t ) ((pos "Os%" ) ))
( "merrily"  a ( m e1 r @ l ii ) ((pos "Pu%" ) ))
( "merriment"  n ( m e1 r i m @ n t ) ((pos "L@%" ) ))
( "merry"  j ( m e1 r ii ) ((pos "OD%" ) ))
( "merry-go-round"  n ( m e1 r i - g @ - r au n d ) ((pos "K6%" ) ))
( "merry-go-rounds"  n ( m e1 r i - g @ - r au n d z ) ((pos "Kj%" ) ))
( "merrymaker"  n ( m e1 r i m ei k @ r ) ((pos "K6%" ) ))
( "merrymakers"  n ( m e1 r i m ei k @ z ) ((pos "Kj%" ) ))
( "merrymaking"  n ( m e1 r i m ei k i ng ) ((pos "L@%" ) ))
( "mescal"  n ( m e1 s k l ) ((pos "M6$" ) ))
( "mescaline"  n ( m e1 s k @ l i n ) ((pos "L@$" ) ))
( "mescals"  n ( m e1 s k l z ) ((pos "Mj$" ) ))
( "meseems"  a ( m i1 s ii1 m z ) ((pos "Pu$" ) ))
( "mesh"  v ( m e1 sh ) ((pos "J1%" "K7%" ) (subcat "2A" "3A" )))
( "meshed"  v ( m e1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "meshes"  v ( m e1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "meshing"  v ( m e1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "mesmeric"  j ( m e1 z m e1 r i k ) ((pos "OA$" ) ))
( "mesmerism"  n ( m e1 z m @ r i z @ m ) ((pos "L@$" ) ))
( "mesmerist"  n ( m e1 z m @ r i s t ) ((pos "K6$" ) ))
( "mesmerists"  n ( m e1 z m @ r i s t s ) ((pos "Kj$" ) ))
( "mesmerize"  v ( m e1 z m @ r ai z ) ((pos "H2%" ) (subcat )))
( "mesmerized"  v ( m e1 z m @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mesmerizes"  v ( m e1 z m @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "mesmerizing"  v ( m e1 z m @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "meson"  n ( m ii1 s @ n ) ((pos "K6$" ) ))
( "mesons"  n ( m ii1 s @ n z ) ((pos "Kj$" ) ))
( "mess"  v ( m e1 s ) ((pos "J1%" "M7%" ) (subcat "2C" "6A" )))
( "mess-jacket"  n ( m e1 s - jh a k i t ) ((pos "K6$" ) ))
( "mess-jackets"  n ( m e1 s - jh a k i t s ) ((pos "Kj$" ) ))
( "mess-up"  n ( m e1 s - uh p ) ((pos "K6%" ) ))
( "mess-ups"  n ( m e1 s - uh p s ) ((pos "Kj$" ) ))
( "message"  n ( m e1 s i jh ) ((pos "K6%" ) ))
( "messages"  n ( m e1 s i jh i z ) ((pos "Kj%" ) ))
( "messed"  v ( m e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "messenger"  n ( m e1 s i n jh @ r ) ((pos "K6%" ) ))
( "messengers"  n ( m e1 s i n jh @ z ) ((pos "Kj%" ) ))
( "messes"  v ( m e1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "messianic"  j ( m e2 s i a1 n i k ) ((pos "OA%" ) ))
( "messier"  j ( m e1 s i@ r ) ((pos "Or%" ) ))
( "messiest"  j ( m e1 s i i s t ) ((pos "Os%" ) ))
( "messily"  a ( m e1 s i l ii ) ((pos "Pu%" ) ))
( "messiness"  n ( m e1 s i n i s ) ((pos "L@%" ) ))
( "messing"  v ( m e1 s i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "messmate"  n ( m e1 s m ei t ) ((pos "K6%" ) ))
( "messmates"  n ( m e1 s m ei t s ) ((pos "Kj%" ) ))
( "messuage"  n ( m e1 s w i jh ) ((pos "K6$" ) ))
( "messuages"  n ( m e1 s w i jh i z ) ((pos "Kj$" ) ))
( "messy"  j ( m e1 s ii ) ((pos "OD%" ) ))
( "met"  v ( m e1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "metabolic"  j ( m e2 t @ b o1 l i k ) ((pos "OA%" ) ))
( "metabolism"  n ( m i1 t a1 b @ l i z @ m ) ((pos "L@%" ) ))
( "metacarpal"  n ( m e2 t @ k aa1 p l ) ((pos "K6$" "OA$" ) ))
( "metacarpals"  n ( m e2 t @ k aa1 p l z ) ((pos "Kj$" ) ))
( "metal"  v ( m e1 t l ) ((pos "H4%" "M6%" ) (subcat )))
( "metalled"  v ( m e1 t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "metallic"  j ( m i1 t a1 l i k ) ((pos "OA%" ) ))
( "metalling"  v ( m e1 t @ l i ng ) ((pos "Hb$" ) (subcat )))
( "metallurgical"  j ( m e2 t @ l @@1 jh i k l ) ((pos "OA%" ) ))
( "metallurgist"  n ( m i1 t a1 l @ jh i s t ) ((pos "K6%" ) ))
( "metallurgists"  n ( m i1 t a1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "metallurgy"  n ( m i1 t a1 l @ jh ii ) ((pos "L@%" ) ))
( "metals"  v ( m e1 t l z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "metalwork"  n ( m e1 t l w @@ k ) ((pos "K6%" ) ))
( "metalworker"  n ( m e1 t l w @@ k @ r ) ((pos "K6%" ) ))
( "metalworkers"  n ( m e1 t l w @@ k @ z ) ((pos "Kj%" ) ))
( "metalworks"  n ( m e1 t l w @@ k s ) ((pos "Kj%" ) ))
( "metamorphose"  v ( m e2 t @ m oo1 f ou z ) ((pos "J2$" ) (subcat "3A" "6A" )))
( "metamorphosed"  v ( m e2 t @ m oo1 f ou z d ) ((pos "Jc$" "Jd$" ) (subcat "3A" "6A" )))
( "metamorphoses"  v ( m e2 t @ m oo1 f ou z i z ) ((pos "Ja$" ) (subcat "3A" "6A" )))
( "metamorphoses"  n ( m e2 t @ m oo1 f @ s ii z ) ((pos "Kj%" ) ))
( "metamorphosing"  v ( m e2 t @ m oo1 f ou z i ng ) ((pos "Jb$" ) (subcat "3A" "6A" )))
( "metamorphosis"  n ( m e2 t @ m oo1 f @ s i s ) ((pos "Ki%" ) ))
( "metaphor"  n ( m e1 t @ f @ r ) ((pos "M6%" ) ))
( "metaphorical"  j ( m e2 t @ f o1 r i k l ) ((pos "OA%" ) ))
( "metaphorically"  a ( m e2 t @ f o1 r i k l ii ) ((pos "Pu%" ) ))
( "metaphors"  n ( m e1 t @ f @ z ) ((pos "Mj%" ) ))
( "metaphysical"  j ( m e2 t @ f i1 z i k l ) ((pos "OA%" ) ))
( "metaphysically"  a ( m e2 t @ f i1 z i k l ii ) ((pos "Pu%" ) ))
( "metaphysics"  n ( m e2 t @ f i1 z i k s ) ((pos "Lk%" ) ))
( "metatarsal"  n ( m e2 t @ t aa1 s l ) ((pos "K6$" "OA$" ) ))
( "metatarsals"  n ( m e2 t @ t aa1 s l z ) ((pos "Kj$" ) ))
( "mete"  v ( m ii1 t ) ((pos "H2%" ) (subcat )))
( "meted"  v ( m ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "meteor"  n ( m ii1 t i@ r ) ((pos "K6%" ) ))
( "meteoric"  j ( m ii2 t i o1 r i k ) ((pos "OA%" ) ))
( "meteorite"  n ( m ii1 t i@ r ai t ) ((pos "K6%" ) ))
( "meteorites"  n ( m ii1 t i@ r ai t s ) ((pos "Kj%" ) ))
( "meteorological"  j ( m ii2 t y @ r @ l o1 jh i k l ) ((pos "OA%" ) ))
( "meteorologist"  n ( m ii2 t i@ r o1 l @ jh i s t ) ((pos "K6%" ) ))
( "meteorologists"  n ( m ii2 t i@ r o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "meteorology"  n ( m ii2 t i@ r o1 l @ jh ii ) ((pos "L@%" ) ))
( "meteors"  n ( m ii1 t i@ z ) ((pos "Kj%" ) ))
( "meter"  n ( m ii1 t @ r ) ((pos "K6%" ) ))
( "meters"  n ( m ii1 t @ z ) ((pos "Kj%" ) ))
( "metes"  v ( m ii1 t s ) ((pos "Ha$" ) (subcat )))
( "methane"  n ( m ii1 th ei n ) ((pos "L@%" ) ))
( "methinks"  a ( m i1 th i1 ng k s ) ((pos "Pu$" ) ))
( "method"  n ( m e1 th @ d ) ((pos "M6%" ) ))
( "methodical"  j ( m i1 th o1 d i k l ) ((pos "OA%" ) ))
( "methodically"  a ( m i1 th o1 d i k l ii ) ((pos "Pu%" ) ))
( "methodological"  j ( m e2 th @ d @ l o1 jh i k l ) ((pos "OA%" ) ))
( "methodologically"  a ( m e2 th @ d @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "methodologies"  n ( m e2 th @ d o1 l @ jh i z ) ((pos "Mj%" ) ))
( "methodology"  n ( m e2 th @ d o1 l @ jh ii ) ((pos "M8%" ) ))
( "methods"  n ( m e1 th @ d z ) ((pos "Mj%" ) ))
( "methought"  a ( m i1 th oo1 t ) ((pos "Pu$" ) ))
( "meths"  n ( m e1 th s ) ((pos "Kj%" ) ))
( "methyl"  n ( m e1 th i l ) ((pos "M6$" ) ))
( "methylated"  j ( m e1 th i l ei t i d ) ((pos "OA%" ) ))
( "methyls"  n ( m e1 th i l z ) ((pos "Mj$" ) ))
( "meticulous"  j ( m i1 t i1 k y u l @ s ) ((pos "OA%" ) ))
( "meticulously"  a ( m i1 t i1 k y u l @ s l ii ) ((pos "Pu%" ) ))
( "meticulousness"  n ( m i1 t i1 k y u l @ s n i s ) ((pos "L@%" ) ))
( "meting"  v ( m ii1 t i ng ) ((pos "Hb$" ) (subcat )))
( "metre"  n ( m ii1 t @ r ) ((pos "M6%" ) ))
( "metres"  n ( m ii1 t @ z ) ((pos "Mj%" ) ))
( "metric"  j ( m e1 t r i k ) ((pos "OA%" ) ))
( "metrical"  j ( m e1 t r i k l ) ((pos "OA%" ) ))
( "metrically"  a ( m e1 t r i k l ii ) ((pos "Pu%" ) ))
( "metrication"  n ( m e2 t r i k ei1 sh @ n ) ((pos "M6%" ) ))
( "metrications"  n ( m e2 t r i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "metricize"  v ( m e1 t r i s ai z ) ((pos "H2$" ) ))
( "metricized"  v ( m e1 t r i s ai z d ) ((pos "Hc$" "Hd$" ) ))
( "metricizes"  v ( m e1 t r i s ai z i z ) ((pos "Ha$" ) ))
( "metricizing"  v ( m e1 t r i s ai z i ng ) ((pos "Hb$" ) ))
( "metronome"  n ( m e1 t r @ n ou m ) ((pos "K6%" ) ))
( "metronomes"  n ( m e1 t r @ n ou m z ) ((pos "Kj%" ) ))
( "metropolis"  n ( m @1 t r o1 p @ l i s ) ((pos "K7%" ) ))
( "metropolises"  n ( m @1 t r o1 p @ l i s i z ) ((pos "Kj$" ) ))
( "metropolitan"  n ( m e2 t r @ p o1 l i t @ n ) ((pos "K6%" "OA%" ) ))
( "metropolitans"  n ( m e2 t r @ p o1 l i t @ n z ) ((pos "Kj$" ) ))
( "mettle"  n ( m e1 t l ) ((pos "L@%" ) ))
( "mettlesome"  j ( m e1 t l s @ m ) ((pos "OA$" ) ))
( "mew"  v ( m y uu1 ) ((pos "I0%" "K6%" ) (subcat )))
( "mewed"  v ( m y uu1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "mewing"  v ( m y uu1 i ng ) ((pos "Ib%" ) (subcat )))
( "mews"  v ( m y uu1 z ) ((pos "Ia%" "K9%" ) (subcat )))
( "mezzanine"  n ( m e1 z @ n ii n ) ((pos "K6%" "OA%" ) ))
( "mezzanines"  n ( m e1 z @ n ii n z ) ((pos "Kj$" ) ))
( "mezzo"  a ( m e1 t s ou ) ((pos "Pu$" ) ))
( "mezzo-soprano"  n ( m e2 t s ou - s @ p r aa1 n ou ) ((pos "K6%" ) ))
( "mezzo-sopranos"  n ( m e2 t s ou - s @ p r aa1 n ou z ) ((pos "Kj%" ) ))
( "mezzotint"  n ( m e1 t s ou t i n t ) ((pos "M6$" ) ))
( "mezzotints"  n ( m e1 t s ou t i n t s ) ((pos "Mj$" ) ))
( "mg"  n ( m i1 l i g r a m z ) ((pos "Y]%" ) ))
( "mi"  n ( m ii1 ) ((pos "Ki$" ) ))
( "miaou"  v ( m ii1 au1 ) ((pos "I0$" "K6$" ) ))
( "miaoued"  v ( m ii1 au1 d ) ((pos "Ic$" "Id$" ) ))
( "miaouing"  v ( m ii1 au1 i ng ) ((pos "Ib$" ) ))
( "miaous"  v ( m ii1 au1 z ) ((pos "Ia$" "Kj$" ) ))
( "miaow"  v ( m ii1 au1 ) ((pos "I0%" "K6%" ) (subcat )))
( "miaowed"  v ( m ii1 au1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "miaowing"  v ( m ii1 au1 i ng ) ((pos "Ib%" ) (subcat )))
( "miaows"  v ( m ii1 au1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "miasma"  n ( m i1 a1 z m @ ) ((pos "K6$" ) ))
( "miasmas"  n ( m i1 a1 z m @ z ) ((pos "Kj$" ) ))
( "mica"  n ( m ai1 k @ ) ((pos "L@$" ) ))
( "mice"  n ( m ai1 s ) ((pos "Kj%" ) ))
( "mickey"  n ( m i1 k ii ) ((pos "Ki%" ) ))
( "mickle"  n ( m i1 k l ) ((pos "Ki$" ) ))
( "micro-organism"  n ( m ai2 k r ou - oo1 g @ n i z @ m ) ((pos "K6%" ) ))
( "micro-organisms"  n ( m ai2 k r ou - oo1 g @ n i z @ m z ) ((pos "Kj%" ) ))
( "microbe"  n ( m ai1 k r ou b ) ((pos "K6%" ) ))
( "microbes"  n ( m ai1 k r ou b z ) ((pos "Kj%" ) ))
( "microbiology"  n ( m ai2 k r ou b ai o1 l @ jh ii ) ((pos "L@%" ) ))
( "microcosm"  n ( m ai1 k r ou k o z @ m ) ((pos "K6%" ) ))
( "microcosms"  n ( m ai1 k r ou k o z @ m z ) ((pos "Kj%" ) ))
( "microdot"  n ( m ai1 k r ou d o t ) ((pos "K6$" ) ))
( "microdots"  n ( m ai1 k r ou d o t s ) ((pos "Kj$" ) ))
( "microelectronics"  n ( m ai2 k r ou ii2 l e k t r o1 n i k s ) ((pos "Lk%" ) ))
( "microfiche"  n ( m ai1 k r ou f ii sh ) ((pos "M6%" ) ))
( "microfiches"  n ( m ai1 k r ou f ii sh i z ) ((pos "Mj$" ) ))
( "microfilm"  v ( m ai1 k r ou f i l m ) ((pos "H0$" "M6%" ) (subcat )))
( "microfilmed"  v ( m ai1 k r ou f i l m d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "microfilming"  v ( m ai1 k r ou f i l m i ng ) ((pos "Hb$" ) (subcat )))
( "microfilms"  v ( m ai1 k r ou f i l m z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "micrometer"  n ( m ai1 k r o1 m i t @ r ) ((pos "K6$" ) ))
( "micrometers"  n ( m ai1 k r o1 m i t @ z ) ((pos "Kj$" ) ))
( "micron"  n ( m ai1 k r o n ) ((pos "K6$" ) ))
( "microns"  n ( m ai1 k r o n z ) ((pos "Kj$" ) ))
( "microphone"  n ( m ai1 k r @ f ou n ) ((pos "K6%" ) ))
( "microphones"  n ( m ai1 k r @ f ou n z ) ((pos "Kj%" ) ))
( "microscope"  n ( m ai1 k r @ s k ou p ) ((pos "K6%" ) ))
( "microscopes"  n ( m ai1 k r @ s k ou p s ) ((pos "Kj%" ) ))
( "microscopic"  j ( m ai2 k r @ s k o1 p i k ) ((pos "OA%" ) ))
( "microscopical"  j ( m ai2 k r @ s k o1 p i k l ) ((pos "OA$" ) ))
( "microscopically"  a ( m ai2 k r @ s k o1 p i k l ii ) ((pos "Pu%" ) ))
( "microscopy"  n ( m ai1 k r o1 s k @ p ii ) ((pos "L@%" ) ))
( "microwave"  n ( m ai1 k r ou w ei v ) ((pos "K6%" ) ))
( "microwaves"  n ( m ai1 k r ou w ei v z ) ((pos "Kj%" ) ))
( "mid"  j ( m i d ) ((pos "OA%" "T-%" ) ))
( "mid-off"  n ( m i1 d - o1 f ) ((pos "Ki%" ) ))
( "mid-on"  n ( m i1 d - o1 n ) ((pos "Ki%" ) ))
( "midday"  n ( m i2 d d ei1 ) ((pos "L@%" ) ))
( "midden"  n ( m i1 d n ) ((pos "K6$" ) ))
( "middens"  n ( m i1 d n z ) ((pos "Kj$" ) ))
( "middies"  n ( m i1 d i z ) ((pos "Kj$" ) ))
( "middle"  n ( m i1 d l ) ((pos "K6%" ) ))
( "middle-aged"  j ( m i2 d l - ei1 jh d ) ((pos "OA%" ) ))
( "middle-class"  j ( m i2 d l - k l aa1 s ) ((pos "OA%" ) ))
( "middle-distance"  j ( m i2 d l - d i1 s t @ n s ) ((pos "OA%" ) ))
( "middle-of-the-road"  j ( m i2 d l - @ v - dh @ - r ou1 d ) ((pos "OA%" ) ))
( "middleman"  n ( m i1 d l m a n ) ((pos "Ki%" ) ))
( "middlemen"  n ( m i1 d l m e n ) ((pos "Kj%" ) ))
( "middles"  n ( m i1 d l z ) ((pos "Kj%" ) ))
( "middleweight"  n ( m i1 d l w ei t ) ((pos "K6%" "OA%" ) ))
( "middleweights"  n ( m i1 d l w ei t s ) ((pos "Kj%" ) ))
( "middling"  n ( m i1 d l i ng ) ((pos "K6$" "OA%" "Pu%" ) ))
( "middlings"  n ( m i1 d l i ng z ) ((pos "Kj$" ) ))
( "middy"  n ( m i1 d ii ) ((pos "K8$" ) ))
( "midfield"  n ( m i1 d f ii l d ) ((pos "Ki%" ) ))
( "midge"  n ( m i1 jh ) ((pos "K6%" ) ))
( "midges"  n ( m i1 jh i z ) ((pos "Kj%" ) ))
( "midget"  n ( m i1 jh i t ) ((pos "K6%" ) ))
( "midgets"  n ( m i1 jh i t s ) ((pos "Kj%" ) ))
( "midinette"  n ( m i2 d i n e1 t ) ((pos "K6$" ) ))
( "midinettes"  n ( m i2 d i n e1 t s ) ((pos "Kj$" ) ))
( "midland"  n ( m i1 d l @ n d ) ((pos "K6%" ) ))
( "midlands"  n ( m i1 d l @ n d z ) ((pos "Kj%" ) ))
( "midmost"  j ( m i1 d m ou s t ) ((pos "OA$" "Pu$" ) ))
( "midnight"  n ( m i1 d n ai t ) ((pos "L@%" ) ))
( "midriff"  n ( m i1 d r i f ) ((pos "K6%" ) ))
( "midriffs"  n ( m i1 d r i f s ) ((pos "Kj$" ) ))
( "midshipman"  n ( m i1 d sh i p m @ n ) ((pos "Ki%" ) ))
( "midshipmen"  n ( m i1 d sh i p m @ n ) ((pos "Kj%" ) ))
( "midships"  a ( m i1 d sh i p s ) ((pos "Pu$" ) ))
( "midst"  n ( m i d s t ) ((pos "Ki%" "T-%" ) ))
( "midstream"  n ( m i2 d s t r ii1 m ) ((pos "L@%" ) ))
( "midsummer"  n ( m i2 d s uh1 m @ r ) ((pos "L@%" ) ))
( "midway"  j ( m i2 d w ei1 ) ((pos "OA%" "Pu%" ) ))
( "midweek"  j ( m i1 d w ii1 k ) ((pos "Oq%" "Pu%" ) ))
( "midwife"  n ( m i1 d w ai f ) ((pos "Ki%" ) ))
( "midwifery"  n ( m i1 d w i f r ii ) ((pos "L@%" ) ))
( "midwives"  n ( m i1 d w ai v z ) ((pos "Kj%" ) ))
( "mien"  n ( m ii1 n ) ((pos "K6$" ) ))
( "miens"  n ( m ii1 n z ) ((pos "Kj$" ) ))
( "might"  v ( m ai t ) ((pos "Gc*" "L@%" ) (subcat )))
( "might-have-been"  n ( m ai1 t - @ v - b ii n ) ((pos "K6$" ) ))
( "might-have-beens"  n ( m ai1 t - @ v - b ii n z ) ((pos "Kj$" ) ))
( "mightier"  j ( m ai1 t i@ r ) ((pos "Or%" ) ))
( "mightiest"  j ( m ai1 t i i s t ) ((pos "Os%" ) ))
( "mightily"  a ( m ai1 t i l ii ) ((pos "Pu%" ) ))
( "mightn't"  v ( m ai1 t n t ) ((pos "Gg%" ) ))
( "mighty"  j ( m ai1 t ii ) ((pos "OD%" "Pu%" ) ))
( "mignonette"  n ( m i2 n y @ n e1 t ) ((pos "L@$" ) ))
( "migraine"  n ( m ii1 g r ei n ) ((pos "K6%" ) ))
( "migraines"  n ( m ii1 g r ei n z ) ((pos "Kj$" ) ))
( "migrant"  n ( m ai1 g r @ n t ) ((pos "K6%" ) ))
( "migrants"  n ( m ai1 g r @ n t s ) ((pos "Kj%" ) ))
( "migrate"  v ( m ai1 g r ei1 t ) ((pos "I2%" ) (subcat "2A" )))
( "migrated"  v ( m ai1 g r ei1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "migrates"  v ( m ai1 g r ei1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "migrating"  v ( m ai1 g r ei1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "migration"  n ( m ai1 g r ei1 sh @ n ) ((pos "M6%" ) ))
( "migrations"  n ( m ai1 g r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "migratory"  j ( m ai1 g r @ t r ii ) ((pos "OA%" ) ))
( "mikado"  n ( m i1 k aa1 d ou ) ((pos "K6$" ) ))
( "mikados"  n ( m i1 k aa1 d ou z ) ((pos "Kj$" ) ))
( "mike"  n ( m ai1 k ) ((pos "K6%" ) ))
( "mikes"  n ( m ai1 k s ) ((pos "Kj%" ) ))
( "miladies"  n ( m i1 l ei1 d i z ) ((pos "Kj$" ) ))
( "milady"  n ( m i1 l ei1 d ii ) ((pos "K8$" ) ))
( "milage"  n ( m ai1 l i jh ) ((pos "K6%" ) ))
( "milages"  n ( m ai1 l i jh i z ) ((pos "Kj%" ) ))
( "milch"  j ( m i1 l ch ) ((pos "OA$" ) ))
( "mild"  j ( m ai1 l d ) ((pos "OC%" ) ))
( "milder"  j ( m ai1 l d @ r ) ((pos "Or%" ) ))
( "mildest"  j ( m ai1 l d i s t ) ((pos "Os%" ) ))
( "mildew"  v ( m i1 l d y uu ) ((pos "J0$" "L@%" ) (subcat "2A" )))
( "mildewed"  v ( m i1 l d y uu d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "mildewing"  v ( m i1 l d y uu i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "mildews"  v ( m i1 l d y uu z ) ((pos "Ja$" ) (subcat "2A" )))
( "mildly"  a ( m ai1 l d l ii ) ((pos "Pu%" ) ))
( "mildness"  n ( m ai1 l d n @ s ) ((pos "L@%" ) ))
( "mile"  n ( m ai1 l ) ((pos "K6%" ) ))
( "mileage"  n ( m ai1 l i jh ) ((pos "K6%" ) ))
( "mileages"  n ( m ai1 l i jh i z ) ((pos "Kj%" ) ))
( "mileometer"  n ( m ai1 l o1 m i t @ r ) ((pos "K6%" ) ))
( "mileometers"  n ( m ai1 l o1 m i t @ z ) ((pos "Kj%" ) ))
( "miler"  n ( m ai1 l @ r ) ((pos "K6%" ) ))
( "milers"  n ( m ai1 l @ z ) ((pos "Kj%" ) ))
( "miles"  n ( m ai1 l z ) ((pos "Kj%" ) ))
( "milestone"  n ( m ai1 l s t ou n ) ((pos "K6%" ) ))
( "milestones"  n ( m ai1 l s t ou n z ) ((pos "Kj%" ) ))
( "milieu"  n ( m ii1 l y @@ ) ((pos "K6%" ) ))
( "milieus"  n ( m ii1 l y @@ z ) ((pos "Kj$" ) ))
( "militancy"  n ( m i1 l i t @ n s ii ) ((pos "L@%" ) ))
( "militant"  n ( m i1 l i t @ n t ) ((pos "K6%" "OA%" ) ))
( "militants"  n ( m i1 l i t @ n t s ) ((pos "Kj%" ) ))
( "militarily"  a ( m i1 l i t r i l ii ) ((pos "Pu%" ) ))
( "militarism"  n ( m i1 l i t @ r i z @ m ) ((pos "L@%" ) ))
( "militarist"  n ( m i1 l i t @ r i s t ) ((pos "K6%" ) ))
( "militaristic"  j ( m i2 l i t @ r i1 s t i k ) ((pos "OA%" ) ))
( "militarists"  n ( m i1 l i t @ r i s t s ) ((pos "Kj$" ) ))
( "military"  n ( m i1 l i t r ii ) ((pos "L@%" "OA%" ) ))
( "militate"  v ( m i1 l i t ei t ) ((pos "I2%" ) (subcat )))
( "militated"  v ( m i1 l i t ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "militates"  v ( m i1 l i t ei t s ) ((pos "Ia%" ) (subcat )))
( "militating"  v ( m i1 l i t ei t i ng ) ((pos "Ib%" ) (subcat )))
( "militia"  n ( m i1 l i1 sh @ ) ((pos "K6%" ) ))
( "militiaman"  n ( m i1 l i1 sh @ m @ n ) ((pos "Ki%" ) ))
( "militiamen"  n ( m i1 l i1 sh @ m @ n ) ((pos "Kj%" ) ))
( "militias"  n ( m i1 l i1 sh @ z ) ((pos "Kj%" ) ))
( "milk"  v ( m i1 l k ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "milk-churn"  n ( m i1 l k - ch @@ n ) ((pos "K6%" ) ))
( "milk-churns"  n ( m i1 l k - ch @@ n z ) ((pos "Kj%" ) ))
( "milk-powder"  n ( m i1 l k - p au d @ r ) ((pos "M6%" ) ))
( "milk-powders"  n ( m i1 l k - p au d @ z ) ((pos "Mj%" ) ))
( "milk-shake"  n ( m i1 l k - sh ei k ) ((pos "K6%" ) ))
( "milk-shakes"  n ( m i1 l k - sh ei k s ) ((pos "Kj%" ) ))
( "milk-teeth"  n ( m i1 l k - t ii th ) ((pos "Kj%" ) ))
( "milk-tooth"  n ( m i1 l k - t uu th ) ((pos "Ki$" ) ))
( "milk-white"  j ( m i1 l k - w ai t ) ((pos "OA%" ) ))
( "milkbar"  n ( m i1 l k b aa r ) ((pos "K6%" ) ))
( "milkbars"  n ( m i1 l k b aa z ) ((pos "Kj%" ) ))
( "milked"  v ( m i1 l k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "milkier"  j ( m i1 l k i@ r ) ((pos "Or%" ) ))
( "milkiest"  j ( m i1 l k i i s t ) ((pos "Os%" ) ))
( "milking"  v ( m i1 l k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "milking-machine"  n ( m i1 l k i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "milking-machines"  n ( m i1 l k i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "milkmaid"  n ( m i1 l k m ei d ) ((pos "K6%" ) ))
( "milkmaids"  n ( m i1 l k m ei d z ) ((pos "Kj%" ) ))
( "milkman"  n ( m i1 l k m @ n ) ((pos "Ki%" ) ))
( "milkmen"  n ( m i1 l k m @ n ) ((pos "Kj%" ) ))
( "milks"  v ( m i1 l k s ) ((pos "Ja%" ) (subcat "2A" )))
( "milkshake"  n ( m i1 l k sh ei k ) ((pos "K6%" ) ))
( "milkshakes"  n ( m i1 l k sh ei k s ) ((pos "Kj%" ) ))
( "milksop"  n ( m i1 l k s o p ) ((pos "K6%" ) ))
( "milksops"  n ( m i1 l k s o p s ) ((pos "Kj%" ) ))
( "milkweed"  n ( m i1 l k w ii d ) ((pos "M6$" ) ))
( "milkweeds"  n ( m i1 l k w ii d z ) ((pos "Mj$" ) ))
( "milky"  j ( m i1 l k ii ) ((pos "OD%" ) ))
( "mill"  v ( m i1 l ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "mill-dam"  n ( m i1 l - d a m ) ((pos "K6$" ) ))
( "mill-dams"  n ( m i1 l - d a m z ) ((pos "Kj$" ) ))
( "mill-girl"  n ( m i1 l - g @@ l ) ((pos "K6%" ) ))
( "mill-girls"  n ( m i1 l - g @@ l z ) ((pos "Kj%" ) ))
( "mill-hand"  n ( m i1 l - h a n d ) ((pos "K6%" ) ))
( "mill-hands"  n ( m i1 l - h a n d z ) ((pos "Kj%" ) ))
( "millboard"  n ( m i1 l b oo d ) ((pos "L@$" ) ))
( "milled"  v ( m i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "millenarian"  n ( m i2 l i n e@1 r i@ n ) ((pos "K6$" ) ))
( "millenarians"  n ( m i2 l i n e@1 r i@ n z ) ((pos "Kj$" ) ))
( "millennia"  n ( m i1 l e1 n i@ ) ((pos "Kj%" ) ))
( "millennial"  j ( m i1 l e1 n i@ l ) ((pos "OA%" ) ))
( "millennium"  n ( m i1 l e1 n i@ m ) ((pos "Ki%" ) ))
( "millepede"  n ( m i1 l i p ii d ) ((pos "K6%" ) ))
( "millepedes"  n ( m i1 l i p ii d z ) ((pos "Kj%" ) ))
( "miller"  n ( m i1 l @ r ) ((pos "K6%" ) ))
( "millers"  n ( m i1 l @ z ) ((pos "Kj%" ) ))
( "millet"  n ( m i1 l i t ) ((pos "L@%" ) ))
( "milli-"  j ( m i1 l i - ) ((pos "U-%" ) ))
( "milliard"  n ( m i1 l i aa d ) ((pos "K6$" ) ))
( "milliards"  n ( m i1 l i aa d z ) ((pos "Kj$" ) ))
( "millibar"  n ( m i1 l i b aa r ) ((pos "K6%" ) ))
( "millibars"  n ( m i1 l i b aa z ) ((pos "Kj%" ) ))
( "milligram"  n ( m i1 l i g r a m ) ((pos "K6%" ) ))
( "milligrams"  n ( m i1 l i g r a m z ) ((pos "Kj%" ) ))
( "millimetre"  n ( m i1 l i m ii t @ r ) ((pos "K6%" ) ))
( "millimetres"  n ( m i1 l i m ii t @ z ) ((pos "Kj%" ) ))
( "milliner"  n ( m i1 l i n @ r ) ((pos "K6%" ) ))
( "milliners"  n ( m i1 l i n @ z ) ((pos "Kj%" ) ))
( "millinery"  n ( m i1 l i n @ r ii ) ((pos "L@%" ) ))
( "milling"  v ( m i1 l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "million"  n ( m i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "millionaire"  n ( m i2 l i@ n e@1 r ) ((pos "K6%" ) ))
( "millionaires"  n ( m i2 l i@ n e@1 z ) ((pos "Kj%" ) ))
( "millionairess"  n ( m i2 l i@ n e@1 r i s ) ((pos "K6%" ) ))
( "millionairesses"  n ( m i2 l i@ n e@1 r i s i z ) ((pos "Kj$" ) ))
( "millionfold"  a ( m i1 l i@ n f ou l d ) ((pos "Pu%" ) ))
( "millions"  n ( m i1 l i@ n z ) ((pos "Kj%" ) ))
( "millionth"  n ( m i1 l i@ n th ) ((pos "K6%" "OA%" ) ))
( "millionths"  n ( m i1 l i@ n th s ) ((pos "Kj%" ) ))
( "millipede"  n ( m i1 l i p ii d ) ((pos "K6%" ) ))
( "millipedes"  n ( m i1 l i p ii d z ) ((pos "Kj%" ) ))
( "millpond"  n ( m i1 l p o n d ) ((pos "K6%" ) ))
( "millponds"  n ( m i1 l p o n d z ) ((pos "Kj$" ) ))
( "millrace"  n ( m i1 l r ei s ) ((pos "K6%" ) ))
( "millraces"  n ( m i1 l r ei s i z ) ((pos "Kj$" ) ))
( "mills"  v ( m i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "millstone"  n ( m i1 l s t ou n ) ((pos "K6%" ) ))
( "millstones"  n ( m i1 l s t ou n z ) ((pos "Kj%" ) ))
( "millwheel"  n ( m i1 l w ii l ) ((pos "K6%" ) ))
( "millwheels"  n ( m i1 l w ii l z ) ((pos "Kj%" ) ))
( "millwright"  n ( m i1 l r ai t ) ((pos "K6$" ) ))
( "millwrights"  n ( m i1 l r ai t s ) ((pos "Kj$" ) ))
( "milometer"  n ( m ai1 l o1 m i t @ r ) ((pos "K6%" ) ))
( "milometers"  n ( m ai1 l o1 m i t @ z ) ((pos "Kj%" ) ))
( "milord"  n ( m i1 l oo1 d ) ((pos "K6$" ) ))
( "milords"  n ( m i1 l oo1 d z ) ((pos "Kj$" ) ))
( "milt"  n ( m i1 l t ) ((pos "L@$" ) ))
( "mime"  v ( m ai1 m ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "mimed"  v ( m ai1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mimeograph"  v ( m i1 m i@ g r aa f ) ((pos "H0%" "K6%" ) (subcat )))
( "mimeographed"  v ( m i1 m i@ g r aa f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mimeographing"  v ( m i1 m i@ g r aa f i ng ) ((pos "Hb$" ) (subcat )))
( "mimeographs"  v ( m i1 m i@ g r aa f s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "mimes"  v ( m ai1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "mimetic"  j ( m i1 m e1 t i k ) ((pos "OA$" ) ))
( "mimic"  v ( m i1 m i k ) ((pos "H5%" "K6%" "Oq%" ) (subcat )))
( "mimicked"  v ( m i1 m i k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mimicking"  v ( m i1 m i k i ng ) ((pos "Hb%" ) (subcat )))
( "mimicry"  n ( m i1 m i k r ii ) ((pos "L@%" ) ))
( "mimics"  v ( m i1 m i k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "miming"  v ( m ai1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mimosa"  n ( m i1 m ou1 z @ ) ((pos "M6%" ) ))
( "mimosas"  n ( m i1 m ou1 z @ z ) ((pos "Mj$" ) ))
( "min"  n ( m ii1 n ) ((pos "Y>%" ) ))
( "minaret"  n ( m i2 n @ r e1 t ) ((pos "K6%" ) ))
( "minarets"  n ( m i2 n @ r e1 t s ) ((pos "Kj%" ) ))
( "minatory"  j ( m i1 n @ t @ r ii ) ((pos "OA$" ) ))
( "mince"  v ( m i1 n s ) ((pos "J2%" "L@%" ) (subcat "2A" )))
( "mince-pie"  n ( m i1 n s - p ai1 ) ((pos "K6%" ) ))
( "mince-pies"  n ( m i1 n s - p ai1 z ) ((pos "Kj%" ) ))
( "minced"  v ( m i1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mincemeat"  n ( m i1 n s m ii t ) ((pos "L@%" ) ))
( "mincer"  n ( m i1 n s @ r ) ((pos "K6%" ) ))
( "mincers"  n ( m i1 n s @ z ) ((pos "Kj%" ) ))
( "minces"  v ( m i1 n s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "mincing"  v ( m i1 n s i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "mincingly"  a ( m i1 n s i ng l ii ) ((pos "Pu%" ) ))
( "mind"  v ( m ai1 n d ) ((pos "J0*" "M6*" ) (subcat "2A" "6A" "6C" "9" )))
( "mind-bending"  j ( m ai1 n d - b e n d i ng ) ((pos "OA%" ) ))
( "mind-blowing"  j ( m ai1 n d - b l ou i ng ) ((pos "OA%" ) ))
( "mind-boggling"  j ( m ai1 n d - b o g l i ng ) ((pos "OA%" ) ))
( "mind-reader"  n ( m ai1 n d - r ii d @ r ) ((pos "K6%" ) ))
( "mind-readers"  n ( m ai1 n d - r ii d @ z ) ((pos "Kj%" ) ))
( "minded"  v ( m ai1 n d i d ) ((pos "Jc%" "Jd%" "Op%" "Ot%" ) (subcat "2A" "6A" "6C" "9" )))
( "minder"  n ( m ai1 n d @ r ) ((pos "K6%" ) ))
( "minders"  n ( m ai1 n d @ z ) ((pos "Kj%" ) ))
( "mindful"  j ( m ai1 n d f @ l ) ((pos "OA%" ) ))
( "mindfully"  a ( m ai1 n d f @ l ii ) ((pos "Pu%" ) ))
( "mindfulness"  n ( m ai1 n d f l n @ s ) ((pos "L@%" ) ))
( "minding"  v ( m ai1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "6C" "9" )))
( "mindless"  j ( m ai1 n d l i s ) ((pos "OA%" ) ))
( "mindlessly"  a ( m ai1 n d l i s l ii ) ((pos "Pu%" ) ))
( "mindlessness"  n ( m ai1 n d l i s n @ s ) ((pos "L@%" ) ))
( "minds"  v ( m ai1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "6C" "9" )))
( "mine"  v ( m ai n ) ((pos "J2%" "K6%" "OA%" "Qx%" ) (subcat "2A" "3A" )))
( "mine-detector"  n ( m ai1 n - d i t e k t @ r ) ((pos "K6%" ) ))
( "mine-detectors"  n ( m ai1 n - d i t e k t @ z ) ((pos "Kj%" ) ))
( "mine-disposal"  n ( m ai1 n - d i s p ou z l ) ((pos "M6%" ) ))
( "mine-disposals"  n ( m ai1 n - d i s p ou z l z ) ((pos "Mj$" ) ))
( "mined"  v ( m ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "minefield"  n ( m ai1 n f ii l d ) ((pos "K6%" ) ))
( "minefields"  n ( m ai1 n f ii l d z ) ((pos "Kj%" ) ))
( "minelayer"  n ( m ai1 n l ei @ r ) ((pos "K6%" ) ))
( "minelayers"  n ( m ai1 n l ei @ z ) ((pos "Kj%" ) ))
( "minelaying"  n ( m ai1 n l ei i ng ) ((pos "L@%" ) ))
( "miner"  n ( m ai1 n @ r ) ((pos "K6%" ) ))
( "mineral"  n ( m i1 n @ r @ l ) ((pos "K6%" "OA%" ) ))
( "mineralogist"  n ( m i2 n @ r a1 l @ jh i s t ) ((pos "K6%" ) ))
( "mineralogists"  n ( m i2 n @ r a1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "mineralogy"  n ( m i2 n @ r a1 l @ jh ii ) ((pos "L@%" ) ))
( "minerals"  n ( m i1 n @ r @ l z ) ((pos "Kj%" ) ))
( "miners"  n ( m ai1 n @ z ) ((pos "Kj%" ) ))
( "mines"  v ( m ai1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "minestrone"  n ( m i2 n i s t r ou1 n ii ) ((pos "L@%" ) ))
( "minesweeper"  n ( m ai1 n s w ii p @ r ) ((pos "K6%" ) ))
( "minesweepers"  n ( m ai1 n s w ii p @ z ) ((pos "Kj%" ) ))
( "minesweeping"  n ( m ai1 n s w ii p i ng ) ((pos "L@%" ) ))
( "mineworker"  n ( m ai1 n w @@ k @ r ) ((pos "K6%" ) ))
( "mineworkers"  n ( m ai1 n w @@ k @ z ) ((pos "Kj%" ) ))
( "mingier"  j ( m i1 n jh i@ r ) ((pos "Or$" ) ))
( "mingiest"  j ( m i1 n jh i i s t ) ((pos "Os$" ) ))
( "mingle"  v ( m i1 ng g l ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "mingled"  v ( m i1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "mingles"  v ( m i1 ng g l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "mingling"  v ( m i1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "mingy"  j ( m i1 n jh ii ) ((pos "OD%" ) ))
( "mini-"  j ( m i1 n i - ) ((pos "U-%" ) ))
( "miniature"  n ( m i1 n i ch @ r ) ((pos "M6%" ) ))
( "miniatures"  n ( m i1 n i ch @ z ) ((pos "Mj%" ) ))
( "miniaturist"  n ( m i1 n i ch @ r i s t ) ((pos "K6%" ) ))
( "miniaturists"  n ( m i1 n i ch @ r i s t s ) ((pos "Kj%" ) ))
( "miniaturization"  n ( m i2 n i ch @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "miniaturize"  v ( m i1 n i ch @ r ai z ) ((pos "H2%" ) (subcat )))
( "miniaturized"  v ( m i1 n i ch @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "miniaturizes"  v ( m i1 n i ch @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "miniaturizing"  v ( m i1 n i ch @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "minim"  n ( m i1 n i m ) ((pos "K6%" ) ))
( "minima"  n ( m i1 n i m @ ) ((pos "Kj$" ) ))
( "minimal"  j ( m i1 n i m l ) ((pos "OA%" ) ))
( "minimally"  a ( m i1 n i m @ l ii ) ((pos "Pu%" ) ))
( "minimize"  v ( m i1 n i m ai z ) ((pos "H2%" ) (subcat )))
( "minimized"  v ( m i1 n i m ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "minimizes"  v ( m i1 n i m ai z i z ) ((pos "Ha%" ) (subcat )))
( "minimizing"  v ( m i1 n i m ai z i ng ) ((pos "Hb%" ) (subcat )))
( "minims"  n ( m i1 n i m z ) ((pos "Kj%" ) ))
( "minimum"  n ( m i1 n i m @ m ) ((pos "K6%" "OA%" ) ))
( "minimums"  n ( m i1 n i m @ m z ) ((pos "Kj$" ) ))
( "mining"  v ( m ai1 n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" )))
( "minion"  n ( m i1 n i@ n ) ((pos "K6$" ) ))
( "minions"  n ( m i1 n i@ n z ) ((pos "Kj%" ) ))
( "minister"  v ( m i1 n i s t @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "ministered"  v ( m i1 n i s t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "ministerial"  j ( m i2 n i s t i@1 r i@ l ) ((pos "OA%" ) ))
( "ministerially"  a ( m i2 n i s t i@1 r i@ l ii ) ((pos "Pu%" ) ))
( "ministering"  v ( m i1 n i s t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "ministers"  v ( m i1 n i s t @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "ministrant"  n ( m i1 n i s t r @ n t ) ((pos "K6$" "Oq$" ) ))
( "ministrants"  n ( m i1 n i s t r @ n t s ) ((pos "Kj$" ) ))
( "ministration"  n ( m i2 n i s t r ei1 sh @ n ) ((pos "M6$" ) ))
( "ministrations"  n ( m i2 n i s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "ministries"  n ( m i1 n i s t r i z ) ((pos "Kj%" ) ))
( "ministry"  n ( m i1 n i s t r ii ) ((pos "K8%" ) ))
( "miniver"  n ( m i1 n i v @ r ) ((pos "L@$" ) ))
( "mink"  n ( m i1 ng k ) ((pos "M6%" ) ))
( "minks"  n ( m i1 ng k s ) ((pos "Mj%" ) ))
( "minnow"  n ( m i1 n ou ) ((pos "K6%" ) ))
( "minnows"  n ( m i1 n ou z ) ((pos "Kj%" ) ))
( "minor"  n ( m ai1 n @ r ) ((pos "K6%" "OA%" ) ))
( "minorities"  n ( m ai1 n o1 r i t i z ) ((pos "Mj%" ) ))
( "minority"  n ( m ai1 n o1 r i t ii ) ((pos "M8%" ) ))
( "minors"  n ( m ai1 n @ z ) ((pos "Kj%" ) ))
( "minster"  n ( m i1 n s t @ r ) ((pos "K6%" ) ))
( "minsters"  n ( m i1 n s t @ z ) ((pos "Kj$" ) ))
( "minstrel"  n ( m i1 n s t r @ l ) ((pos "K6%" ) ))
( "minstrels"  n ( m i1 n s t r @ l z ) ((pos "Kj%" ) ))
( "minstrelsy"  n ( m i1 n s t r @ l s ii ) ((pos "L@$" ) ))
( "mint"  v ( m i1 n t ) ((pos "H0%" "M6%" ) (subcat )))
( "minted"  v ( m i1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "minting"  v ( m i1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "mints"  v ( m i1 n t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "minuet"  n ( m i2 n y u e1 t ) ((pos "K6%" ) ))
( "minuets"  n ( m i2 n y u e1 t s ) ((pos "Kj%" ) ))
( "minus"  n ( m ai1 n @ s ) ((pos "K7%" "OA%" "T-%" ) ))
( "minuscule"  j ( m i1 n @ s k y uu l ) ((pos "OA%" ) ))
( "minuses"  n ( m ai1 n @ s i z ) ((pos "Kj$" ) ))
( "minute"  v ( m i1 n i t ) ((pos "H2%" "K6%" ) (subcat )))
( "minute"  j ( m ai1 n y uu1 t ) ((pos "OB%" ) ))
( "minute-book"  n ( m i1 n i t - b u k ) ((pos "K6%" ) ))
( "minute-books"  n ( m i1 n i t - b u k s ) ((pos "Kj%" ) ))
( "minute-gun"  n ( m i1 n i t - g uh n ) ((pos "K6$" ) ))
( "minute-guns"  n ( m i1 n i t - g uh n z ) ((pos "Kj$" ) ))
( "minute-hand"  n ( m i1 n i t - h a n d ) ((pos "K6%" ) ))
( "minute-hands"  n ( m i1 n i t - h a n d z ) ((pos "Kj$" ) ))
( "minuted"  v ( m i1 n i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "minutely"  a ( m ai1 n y uu1 t l ii ) ((pos "Pu%" ) ))
( "minuteman"  n ( m i1 n i t m a n ) ((pos "Ki$" ) ))
( "minutemen"  n ( m i1 n i t m e n ) ((pos "Kj$" ) ))
( "minuteness"  n ( m ai1 n y uu1 t n @ s ) ((pos "L@%" ) ))
( "minuter"  j ( m ai1 n y uu1 t @ r ) ((pos "Or$" ) ))
( "minutes"  v ( m i1 n i t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "minutest"  j ( m ai1 n y uu1 t i s t ) ((pos "Os%" ) ))
( "minutiae"  n ( m ai1 n y uu1 sh i ii ) ((pos "Kj%" ) ))
( "minuting"  v ( m i1 n i t i ng ) ((pos "Hb%" ) (subcat )))
( "minx"  n ( m i1 ng k s ) ((pos "K7%" ) ))
( "minxes"  n ( m i1 ng k s i z ) ((pos "Kj%" ) ))
( "miracle"  n ( m i1 r @ k l ) ((pos "K6%" ) ))
( "miracles"  n ( m i1 r @ k l z ) ((pos "Kj%" ) ))
( "miraculous"  j ( m i1 r a1 k y u l @ s ) ((pos "OA%" ) ))
( "miraculously"  a ( m i1 r a1 k y u l @ s l ii ) ((pos "Pu%" ) ))
( "mirage"  n ( m i1 r aa zh ) ((pos "K6%" ) ))
( "mirages"  n ( m i1 r aa zh i z ) ((pos "Kj%" ) ))
( "mire"  v ( m ai1 @ r ) ((pos "J2$" "L@%" ) (subcat "2A" )))
( "mired"  v ( m ai1 @ d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "mires"  v ( m ai1 @ z ) ((pos "Ja$" ) (subcat "2A" )))
( "mirier"  j ( m ai1 @ r i@ r ) ((pos "Or$" ) ))
( "miriest"  j ( m ai1 @ r i i s t ) ((pos "Os$" ) ))
( "miring"  v ( m ai1 @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "mirror"  v ( m i1 r @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "mirrored"  v ( m i1 r @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mirroring"  v ( m i1 r @ r i ng ) ((pos "Hb%" ) (subcat )))
( "mirrors"  v ( m i1 r @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mirth"  n ( m @@1 th ) ((pos "L@%" ) ))
( "mirthful"  j ( m @@1 th f @ l ) ((pos "OA%" ) ))
( "mirthfully"  a ( m @@1 th f @ l ii ) ((pos "Pu%" ) ))
( "mirthless"  j ( m @@1 th l @ s ) ((pos "OA%" ) ))
( "miry"  j ( m ai1 @ r ii ) ((pos "OD$" ) ))
( "misadventure"  n ( m i2 s @ d v e1 n ch @ r ) ((pos "M6%" ) ))
( "misadventures"  n ( m i2 s @ d v e1 n ch @ z ) ((pos "Mj$" ) ))
( "misadvise"  v ( m i2 s @ d v ai1 z ) ((pos "H2$" ) (subcat )))
( "misadvised"  v ( m i2 s @ d v ai1 z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "misadvises"  v ( m i2 s @ d v ai1 z i z ) ((pos "Ha$" ) (subcat )))
( "misadvising"  v ( m i2 s @ d v ai1 z i ng ) ((pos "Hb$" ) (subcat )))
( "misalliance"  n ( m i2 s @ l ai1 @ n s ) ((pos "K6$" ) ))
( "misalliances"  n ( m i2 s @ l ai1 @ n s i z ) ((pos "Kj$" ) ))
( "misanthrope"  n ( m i1 s n th r ou p ) ((pos "K6%" ) ))
( "misanthropes"  n ( m i1 s n th r ou p s ) ((pos "Kj$" ) ))
( "misanthropic"  j ( m i2 s n th r o1 p i k ) ((pos "OA%" ) ))
( "misanthropy"  n ( m i1 s a1 n th r @ p ii ) ((pos "L@%" ) ))
( "misapplication"  n ( m i2 s a2 p l i k ei1 sh @ n ) ((pos "K6%" ) ))
( "misapplications"  n ( m i2 s a2 p l i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "misapplied"  v ( m i2 s @ p l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misapplies"  v ( m i2 s @ p l ai1 z ) ((pos "Ha%" ) (subcat )))
( "misapply"  v ( m i2 s @ p l ai1 ) ((pos "H3%" ) (subcat )))
( "misapplying"  v ( m i2 s @ p l ai1 i ng ) ((pos "Hb%" ) (subcat )))
( "misapprehend"  v ( m i2 s a p r i h e1 n d ) ((pos "H0%" ) (subcat )))
( "misapprehended"  v ( m i2 s a p r i h e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misapprehending"  v ( m i2 s a p r i h e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "misapprehends"  v ( m i2 s a p r i h e1 n d z ) ((pos "Ha%" ) (subcat )))
( "misapprehension"  n ( m i2 s a p r i h e1 n sh @ n ) ((pos "M6%" ) ))
( "misapprehensions"  n ( m i2 s a p r i h e1 n sh @ n z ) ((pos "Mj%" ) ))
( "misappropriate"  v ( m i2 s @ p r ou1 p r i ei t ) ((pos "H2%" ) (subcat )))
( "misappropriated"  v ( m i2 s @ p r ou1 p r i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misappropriates"  v ( m i2 s @ p r ou1 p r i ei t s ) ((pos "Ha%" ) (subcat )))
( "misappropriating"  v ( m i2 s @ p r ou1 p r i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "misappropriation"  n ( m i2 s @ p r ou2 p r i ei1 sh @ n ) ((pos "M6%" ) ))
( "misappropriations"  n ( m i2 s @ p r ou2 p r i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "misbegotten"  j ( m i2 s b i g o1 t n ) ((pos "OA%" ) ))
( "misbehave"  v ( m i2 s b i h ei1 v ) ((pos "J2%" ) (subcat "2A" )))
( "misbehaved"  v ( m i2 s b i h ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "misbehaves"  v ( m i2 s b i h ei1 v z ) ((pos "Ja%" ) (subcat "2A" )))
( "misbehaving"  v ( m i2 s b i h ei1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "misbehaviour"  n ( m i2 s b i h ei1 v i@ r ) ((pos "L@%" ) ))
( "misc"  n ( m i1 s k ) ((pos "Y~%" ) ))
( "miscalculate"  v ( m i2 s k a1 l k y u l ei t ) ((pos "J2%" ) ))
( "miscalculated"  v ( m i2 s k a1 l k y u l ei t i d ) ((pos "Jc%" "Jd%" ) ))
( "miscalculates"  v ( m i2 s k a1 l k y u l ei t s ) ((pos "Ja%" ) ))
( "miscalculating"  v ( m i2 s k a1 l k y u l ei t i ng ) ((pos "Jb%" ) ))
( "miscalculation"  n ( m i2 s k a2 l k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "miscalculations"  n ( m i2 s k a2 l k y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "miscall"  v ( m i2 s k oo1 l ) ((pos "H0$" ) ))
( "miscalled"  v ( m i2 s k oo1 l d ) ((pos "Hc$" "Hd$" ) ))
( "miscalling"  v ( m i2 s k oo1 l i ng ) ((pos "Hb$" ) ))
( "miscalls"  v ( m i2 s k oo1 l z ) ((pos "Ha$" ) ))
( "miscarriage"  n ( m i2 s k a1 r i jh ) ((pos "M6%" ) ))
( "miscarriages"  n ( m i2 s k a1 r i jh i z ) ((pos "Mj%" ) ))
( "miscarried"  v ( m i2 s k a1 r i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "miscarries"  v ( m i2 s k a1 r i z ) ((pos "Ha%" ) (subcat )))
( "miscarry"  v ( m i2 s k a1 r ii ) ((pos "H3%" ) (subcat )))
( "miscarrying"  v ( m i2 s k a1 r i i ng ) ((pos "Hb%" ) (subcat )))
( "miscast"  v ( m i2 s k aa1 s t ) ((pos "H5$" "Hc%" "Hd%" ) (subcat )))
( "miscasting"  v ( m i2 s k aa1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "miscasts"  v ( m i2 s k aa1 s t s ) ((pos "Ha$" ) (subcat )))
( "miscegenation"  n ( m i2 s i jh i n ei1 sh @ n ) ((pos "L@$" ) ))
( "miscellanea"  n ( m i2 s @ l ei1 n i@ ) ((pos "Kj%" ) ))
( "miscellaneous"  j ( m i2 s @ l ei1 n i@ s ) ((pos "OA%" ) ))
( "miscellanies"  n ( m i1 s e1 l @ n i z ) ((pos "Kj$" ) ))
( "miscellany"  n ( m i1 s e1 l @ n ii ) ((pos "K8%" ) ))
( "mischance"  n ( m i2 s ch aa1 n s ) ((pos "M6%" ) ))
( "mischances"  n ( m i2 s ch aa1 n s i z ) ((pos "Mj$" ) ))
( "mischief"  n ( m i1 s ch i f ) ((pos "M6%" ) ))
( "mischief-maker"  n ( m i1 s ch ii f - m ei k @ r ) ((pos "K6%" ) ))
( "mischief-makers"  n ( m i1 s ch ii f - m ei k @ z ) ((pos "Kj%" ) ))
( "mischief-making"  n ( m i1 s ch ii f - m ei k i ng ) ((pos "L@%" ) ))
( "mischiefs"  n ( m i1 s ch i f s ) ((pos "Mj$" ) ))
( "mischievous"  j ( m i1 s ch i v @ s ) ((pos "OA%" ) ))
( "mischievously"  a ( m i1 s ch i v @ s l ii ) ((pos "Pu%" ) ))
( "mischievousness"  n ( m i1 s ch i v @ s n @ s ) ((pos "L@%" ) ))
( "misconceive"  v ( m i2 s k @ n s ii1 v ) ((pos "J2$" ) (subcat "3A" )))
( "misconceived"  v ( m i2 s k @ n s ii1 v d ) ((pos "Jc$" "Jd%" ) (subcat "3A" )))
( "misconceives"  v ( m i2 s k @ n s ii1 v z ) ((pos "Ja$" ) (subcat "3A" )))
( "misconceiving"  v ( m i2 s k @ n s ii1 v i ng ) ((pos "Jb$" ) (subcat "3A" )))
( "misconception"  n ( m i2 s k @ n s e1 p sh @ n ) ((pos "M6%" ) ))
( "misconceptions"  n ( m i2 s k @ n s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "misconduct"  n ( m i2 s k o1 n d uh k t ) ((pos "L@%" ) ))
( "misconduct"  v ( m i2 s k @ n d uh1 k t ) ((pos "H0$" ) (subcat "6A" "6B" )))
( "misconducted"  v ( m i2 s k @ n d uh1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "6B" )))
( "misconducting"  v ( m i2 s k @ n d uh1 k t i ng ) ((pos "Hb$" ) (subcat "6A" "6B" )))
( "misconducts"  v ( m i2 s k @ n d uh1 k t s ) ((pos "Ha$" ) (subcat "6A" "6B" )))
( "misconstruction"  n ( m i2 s k @ n s t r uh1 k sh @ n ) ((pos "M6%" ) ))
( "misconstructions"  n ( m i2 s k @ n s t r uh1 k sh @ n z ) ((pos "Mj$" ) ))
( "misconstrue"  v ( m i2 s k @ n s t r uu1 ) ((pos "H2%" ) (subcat )))
( "misconstrued"  v ( m i2 s k @ n s t r uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misconstrues"  v ( m i2 s k @ n s t r uu1 z ) ((pos "Ha%" ) (subcat )))
( "misconstruing"  v ( m i2 s k @ n s t r uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "miscount"  v ( m i2 s k au1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "miscounted"  v ( m i2 s k au1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "miscounting"  v ( m i2 s k au1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "miscounts"  v ( m i2 s k au1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "miscreant"  n ( m i1 s k r i@ n t ) ((pos "K6%" ) ))
( "miscreants"  n ( m i1 s k r i@ n t s ) ((pos "Kj%" ) ))
( "misdate"  v ( m i2 s d ei1 t ) ((pos "H2%" ) (subcat )))
( "misdated"  v ( m i2 s d ei1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "misdates"  v ( m i2 s d ei1 t s ) ((pos "Ha$" ) (subcat )))
( "misdating"  v ( m i2 s d ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "misdeal"  v ( m i2 s d ii1 l ) ((pos "J5$" "K6$" ) (subcat "2A" )))
( "misdealing"  v ( m i2 s d ii1 l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "misdeals"  v ( m i2 s d ii1 l z ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "misdealt"  v ( m i2 s d e1 l t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "misdeed"  n ( m i2 s d ii1 d ) ((pos "K6%" ) ))
( "misdeeds"  n ( m i2 s d ii1 d z ) ((pos "Kj%" ) ))
( "misdemeanour"  n ( m i2 s d i m ii1 n @ r ) ((pos "K6%" ) ))
( "misdemeanours"  n ( m i2 s d i m ii1 n @ z ) ((pos "Kj%" ) ))
( "misdirect"  v ( m i2 s d i r e1 k t ) ((pos "H0%" ) (subcat )))
( "misdirected"  v ( m i2 s d i r e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misdirecting"  v ( m i2 s d i r e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "misdirection"  n ( m i2 s d i r e1 k sh @ n ) ((pos "K6$" ) ))
( "misdirections"  n ( m i2 s d i r e1 k sh @ n z ) ((pos "Kj$" ) ))
( "misdirects"  v ( m i2 s d i r e1 k t s ) ((pos "Ha%" ) (subcat )))
( "misdoing"  n ( m i2 s d uu1 i ng ) ((pos "K6%" ) ))
( "misdoings"  n ( m i2 s d uu1 i ng z ) ((pos "Kj%" ) ))
( "mise en sc`ene"  n ( m ii2 z - o n - s ei1 n ) ((pos "L@$" ) ))
( "miser"  n ( m ai1 z @ r ) ((pos "K6%" ) ))
( "miserable"  j ( m i1 z r @ b l ) ((pos "OA%" ) ))
( "miserably"  a ( m i1 z r @ b l ii ) ((pos "Pu%" ) ))
( "miseries"  n ( m i1 z @ r i z ) ((pos "Mj%" ) ))
( "miserliness"  n ( m ai1 z @ l i n @ s ) ((pos "L@%" ) ))
( "miserly"  j ( m ai1 z @ l ii ) ((pos "OA%" ) ))
( "misers"  n ( m ai1 z @ z ) ((pos "Kj%" ) ))
( "misery"  n ( m i1 z @ r ii ) ((pos "M8%" ) ))
( "misfire"  v ( m i2 s f ai1 @ r ) ((pos "I2%" "K6%" ) (subcat )))
( "misfired"  v ( m i2 s f ai1 @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "misfires"  v ( m i2 s f ai1 @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "misfiring"  v ( m i2 s f ai1 @ r i ng ) ((pos "Ib%" ) (subcat )))
( "misfit"  n ( m i1 s f i t ) ((pos "K6%" ) ))
( "misfits"  n ( m i1 s f i t s ) ((pos "Kj%" ) ))
( "misfortune"  n ( m i2 s f oo1 ch uu n ) ((pos "M6%" ) ))
( "misfortunes"  n ( m i2 s f oo1 ch uu n z ) ((pos "Mj%" ) ))
( "misgave"  v ( m i2 s g ei1 v ) ((pos "Hc$" ) ))
( "misgive"  v ( m i2 s g i1 v ) ((pos "H5$" ) ))
( "misgiven"  v ( m i2 s g i1 v n ) ((pos "Hd$" ) ))
( "misgives"  v ( m i2 s g i1 v z ) ((pos "Ha$" ) ))
( "misgiving"  v ( m i2 s g i1 v i ng ) ((pos "Hb$" "M6$" ) ))
( "misgivings"  n ( m i2 s g i1 v i ng z ) ((pos "Mj%" ) ))
( "misgovern"  v ( m i2 s g uh1 v n ) ((pos "H0%" ) (subcat )))
( "misgoverned"  v ( m i2 s g uh1 v n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misgoverning"  v ( m i2 s g uh1 v n i ng ) ((pos "Hb%" ) (subcat )))
( "misgovernment"  n ( m i2 s g uh1 v n m @ n t ) ((pos "L@%" ) ))
( "misgoverns"  v ( m i2 s g uh1 v n z ) ((pos "Ha%" ) (subcat )))
( "misguide"  v ( m i2 s g ai1 d ) ((pos "H2$" ) (subcat "6A" )))
( "misguided"  v ( m i2 s g ai1 d i d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat "6A" )))
( "misguides"  v ( m i2 s g ai1 d z ) ((pos "Ha$" ) (subcat "6A" )))
( "misguiding"  v ( m i2 s g ai1 d i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "mishandle"  v ( m i2 s h a1 n d l ) ((pos "H2%" ) ))
( "mishandled"  v ( m i2 s h a1 n d l d ) ((pos "Hc%" "Hd%" ) ))
( "mishandles"  v ( m i2 s h a1 n d l z ) ((pos "Ha%" ) ))
( "mishandling"  v ( m i2 s h a1 n d l i ng ) ((pos "Hb%" ) ))
( "mishap"  n ( m i1 s h a p ) ((pos "M6%" ) ))
( "mishaps"  n ( m i1 s h a p s ) ((pos "Mj%" ) ))
( "mishmash"  n ( m i1 sh m a sh ) ((pos "L@%" ) ))
( "misinform"  v ( m i2 s i n f oo1 m ) ((pos "H0%" ) (subcat )))
( "misinformation"  n ( m i2 s i n f @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "misinformed"  v ( m i2 s i n f oo1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misinforming"  v ( m i2 s i n f oo1 m i ng ) ((pos "Hb%" ) (subcat )))
( "misinforms"  v ( m i2 s i n f oo1 m z ) ((pos "Ha%" ) (subcat )))
( "misinterpret"  v ( m i2 s i n t @@1 p r i t ) ((pos "H0%" ) (subcat )))
( "misinterpretation"  n ( m i2 s i n t @@2 p r i t ei1 sh @ n ) ((pos "M6%" ) ))
( "misinterpretations"  n ( m i2 s i n t @@2 p r i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "misinterpreted"  v ( m i2 s i n t @@1 p r i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misinterpreting"  v ( m i2 s i n t @@1 p r i t i ng ) ((pos "Hb%" ) (subcat )))
( "misinterprets"  v ( m i2 s i n t @@1 p r i t s ) ((pos "Ha%" ) (subcat )))
( "misjudge"  v ( m i2 s jh uh1 jh ) ((pos "J2%" ) (subcat "2A" )))
( "misjudged"  v ( m i2 s jh uh1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "misjudges"  v ( m i2 s jh uh1 jh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "misjudging"  v ( m i2 s jh uh1 jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mislaid"  v ( m i2 s l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mislay"  v ( m i2 s l ei1 ) ((pos "H5%" ) (subcat )))
( "mislaying"  v ( m i2 s l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "mislays"  v ( m i2 s l ei1 z ) ((pos "Ha%" ) (subcat )))
( "mislead"  v ( m i2 s l ii1 d ) ((pos "H5%" ) (subcat )))
( "misleading"  v ( m i2 s l ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "misleads"  v ( m i2 s l ii1 d z ) ((pos "Ha%" ) (subcat )))
( "misled"  v ( m i2 s l e1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mismanage"  v ( m i2 s m a1 n i jh ) ((pos "H2%" ) (subcat )))
( "mismanaged"  v ( m i2 s m a1 n i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mismanagement"  n ( m i2 s m a1 n i jh m @ n t ) ((pos "L@%" ) ))
( "mismanages"  v ( m i2 s m a1 n i jh i z ) ((pos "Ha%" ) (subcat )))
( "mismanaging"  v ( m i2 s m a1 n i jh i ng ) ((pos "Hb%" ) (subcat )))
( "misname"  v ( m i2 s n ei1 m ) ((pos "H2%" ) (subcat )))
( "misnamed"  v ( m i2 s n ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misnames"  v ( m i2 s n ei1 m z ) ((pos "Ha%" ) (subcat )))
( "misnaming"  v ( m i2 s n ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "misnomer"  n ( m i2 s n ou1 m @ r ) ((pos "K6%" ) ))
( "misnomers"  n ( m i2 s n ou1 m @ z ) ((pos "Kj%" ) ))
( "misogynist"  n ( m i1 s o1 jh i n i s t ) ((pos "K6%" ) ))
( "misogynists"  n ( m i1 s o1 jh i n i s t s ) ((pos "Kj%" ) ))
( "misplace"  v ( m i2 s p l ei1 s ) ((pos "H2%" ) (subcat )))
( "misplaced"  v ( m i2 s p l ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misplaces"  v ( m i2 s p l ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "misplacing"  v ( m i2 s p l ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "misprint"  n ( m i1 s p r i n t ) ((pos "K6%" ) ))
( "misprint"  v ( m i2 s p r i1 n t ) ((pos "H0$" ) (subcat )))
( "misprinted"  v ( m i2 s p r i1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "misprinting"  v ( m i2 s p r i1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "misprints"  n ( m i1 s p r i n t s ) ((pos "Kj%" ) ))
( "misprints"  v ( m i2 s p r i1 n t s ) ((pos "Ha$" ) (subcat )))
( "mispronounce"  v ( m i2 s p r @ n au1 n s ) ((pos "H2%" ) (subcat )))
( "mispronounced"  v ( m i2 s p r @ n au1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mispronounces"  v ( m i2 s p r @ n au1 n s i z ) ((pos "Ha%" ) (subcat )))
( "mispronouncing"  v ( m i2 s p r @ n au1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "mispronunciation"  n ( m i2 s p r @ n uh2 n s i ei1 sh @ n ) ((pos "K6%" ) ))
( "mispronunciations"  n ( m i2 s p r @ n uh2 n s i ei1 sh @ n z ) ((pos "Kj%" ) ))
( "misquotation"  n ( m i2 s k w ou t ei1 sh @ n ) ((pos "M6%" ) ))
( "misquotations"  n ( m i2 s k w ou t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "misquote"  v ( m i2 s k w ou1 t ) ((pos "H2%" ) (subcat )))
( "misquoted"  v ( m i2 s k w ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misquotes"  v ( m i2 s k w ou1 t s ) ((pos "Ha%" ) (subcat )))
( "misquoting"  v ( m i2 s k w ou1 t i ng ) ((pos "Hb%" ) (subcat )))
( "misread"  v ( m i2 s r e1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misread"  v ( m i2 s r ii1 d ) ((pos "H5%" ) (subcat )))
( "misreading"  v ( m i2 s r ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "misreads"  v ( m i2 s r ii1 d z ) ((pos "Ha%" ) (subcat )))
( "misrepresent"  v ( m i2 s r e2 p r i z e1 n t ) ((pos "H0%" ) ))
( "misrepresentation"  n ( m i2 s r e2 p r i z e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "misrepresentations"  n ( m i2 s r e2 p r i z e n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "misrepresented"  v ( m i2 s r e2 p r i z e1 n t i d ) ((pos "Hc%" "Hd%" ) ))
( "misrepresenting"  v ( m i2 s r e2 p r i z e1 n t i ng ) ((pos "Hb%" ) ))
( "misrepresents"  v ( m i2 s r e2 p r i z e1 n t s ) ((pos "Ha%" ) ))
( "misrule"  n ( m i2 s r uu1 l ) ((pos "L@%" ) ))
( "miss"  v ( m i1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "6B" "6C" )))
( "missal"  n ( m i1 s l ) ((pos "K6$" ) ))
( "missals"  n ( m i1 s l z ) ((pos "Kj$" ) ))
( "missed"  v ( m i1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "6B" "6C" )))
( "misses"  v ( m i1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "6B" "6C" )))
( "misshapen"  j ( m i2 s sh ei1 p @ n ) ((pos "OA%" ) ))
( "missies"  n ( m i1 s i z ) ((pos "Kj$" ) ))
( "missile"  n ( m i1 s ai l ) ((pos "K6%" ) ))
( "missiles"  n ( m i1 s ai l z ) ((pos "Kj%" ) ))
( "missing"  v ( m i1 s i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "6B" "6C" )))
( "mission"  n ( m i1 sh @ n ) ((pos "K6%" ) ))
( "missionaries"  n ( m i1 sh @ n r i z ) ((pos "Kj%" ) ))
( "missionary"  n ( m i1 sh @ n r ii ) ((pos "K8%" ) ))
( "missions"  n ( m i1 sh @ n z ) ((pos "Kj%" ) ))
( "missis"  n ( m i1 s i z ) ((pos "Ki$" ) ))
( "missive"  n ( m i1 s i v ) ((pos "K6%" ) ))
( "missives"  n ( m i1 s i v z ) ((pos "Kj%" ) ))
( "misspell"  v ( m i2 s s p e1 l ) ((pos "H0%" ) (subcat )))
( "misspelled"  v ( m i2 s s p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misspelling"  v ( m i2 s s p e1 l i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "misspellings"  n ( m i2 s s p e1 l i ng z ) ((pos "Kj%" ) ))
( "misspells"  v ( m i2 s s p e1 l z ) ((pos "Ha%" ) (subcat )))
( "misspelt"  v ( m i2 s s p e1 l t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misspend"  v ( m i2 s s p e1 n d ) ((pos "H5%" ) (subcat )))
( "misspending"  v ( m i2 s s p e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "misspends"  v ( m i2 s s p e1 n d z ) ((pos "Ha%" ) (subcat )))
( "misspent"  v ( m i2 s s p e1 n t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misstate"  v ( m i2 s s t ei1 t ) ((pos "H2$" ) (subcat )))
( "misstated"  v ( m i2 s s t ei1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "misstatement"  n ( m i2 s s t ei1 t m @ n t ) ((pos "K6%" ) ))
( "misstatements"  n ( m i2 s s t ei1 t m @ n t s ) ((pos "Kj%" ) ))
( "misstates"  v ( m i2 s s t ei1 t s ) ((pos "Ha$" ) (subcat )))
( "misstating"  v ( m i2 s s t ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "missus"  n ( m i1 s i z ) ((pos "Ki$" ) ))
( "missy"  n ( m i1 s ii ) ((pos "K8$" ) ))
( "mist"  v ( m i1 s t ) ((pos "J0%" "M6%" ) (subcat "2C" )))
( "mistake"  v ( m i1 s t ei1 k ) ((pos "J5%" "K6%" ) (subcat "6A" "10" )))
( "mistaken"  v ( m i1 s t ei1 k n ) ((pos "Jd%" "OA%" ) (subcat "6A" "10" )))
( "mistakenly"  a ( m i1 s t ei1 k n l ii ) ((pos "Pu%" ) ))
( "mistakes"  v ( m i1 s t ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "6A" "10" )))
( "mistaking"  v ( m i1 s t ei1 k i ng ) ((pos "Jb%" ) (subcat "6A" "10" )))
( "misted"  v ( m i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "mister"  n ( m i1 s t @ r ) ((pos "Ki%" ) ))
( "mistier"  j ( m i1 s t i@ r ) ((pos "Or%" ) ))
( "mistiest"  j ( m i1 s t i i s t ) ((pos "Os%" ) ))
( "mistily"  a ( m i1 s t i l ii ) ((pos "Pu%" ) ))
( "mistime"  v ( m i2 s t ai1 m ) ((pos "H2%" ) ))
( "mistimed"  v ( m i2 s t ai1 m d ) ((pos "Hc%" "Hd%" ) ))
( "mistimes"  v ( m i2 s t ai1 m z ) ((pos "Ha%" ) ))
( "mistiming"  v ( m i2 s t ai1 m i ng ) ((pos "Hb%" ) ))
( "mistiness"  n ( m i1 s t i n @ s ) ((pos "L@%" ) ))
( "misting"  v ( m i1 s t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "mistletoe"  n ( m i1 s l t ou ) ((pos "L@%" ) ))
( "mistook"  v ( m i1 s t u1 k ) ((pos "Jc%" ) (subcat "6A" "10" )))
( "mistral"  n ( m i1 s t r @ l ) ((pos "K6$" ) ))
( "mistrals"  n ( m i1 s t r @ l z ) ((pos "Kj$" ) ))
( "mistranslate"  v ( m i2 s t r a n z l ei1 t ) ((pos "H2%" ) (subcat )))
( "mistranslated"  v ( m i2 s t r a n z l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mistranslates"  v ( m i2 s t r a n z l ei1 t s ) ((pos "Ha%" ) (subcat )))
( "mistranslating"  v ( m i2 s t r a n z l ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "mistranslation"  n ( m i2 s t r a n s l ei1 sh @ n ) ((pos "M6%" ) ))
( "mistranslations"  n ( m i2 s t r a n s l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "mistress"  n ( m i1 s t r i s ) ((pos "K7%" ) ))
( "mistresses"  n ( m i1 s t r i s i z ) ((pos "Kj%" ) ))
( "mistrial"  n ( m i2 s t r ai1 @ l ) ((pos "K6$" ) ))
( "mistrials"  n ( m i2 s t r ai1 @ l z ) ((pos "Kj$" ) ))
( "mistrust"  v ( m i2 s t r uh1 s t ) ((pos "H0%" "L@%" ) (subcat )))
( "mistrusted"  v ( m i2 s t r uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mistrustful"  j ( m i2 s t r uh1 s t f @ l ) ((pos "OA%" ) ))
( "mistrustfully"  a ( m i1 s t r uh1 s t f @ l ii ) ((pos "Pu%" ) ))
( "mistrusting"  v ( m i2 s t r uh1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "mistrusts"  v ( m i2 s t r uh1 s t s ) ((pos "Ha%" ) (subcat )))
( "mists"  v ( m i1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "misty"  j ( m i1 s t ii ) ((pos "OD%" ) ))
( "misunderstand"  v ( m i2 s uh2 n d @ s t a1 n d ) ((pos "H5%" ) (subcat )))
( "misunderstanding"  v ( m i2 s uh2 n d @ s t a1 n d i ng ) ((pos "Hb%" "M6%" ) (subcat )))
( "misunderstandings"  n ( m i2 s uh2 n d @ s t a1 n d i ng z ) ((pos "Mj%" ) ))
( "misunderstands"  v ( m i2 s uh2 n d @ s t a1 n d z ) ((pos "Ha%" ) (subcat )))
( "misunderstood"  v ( m i2 s uh2 n d @ s t u1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misuse"  n ( m i2 s y uu1 s ) ((pos "M6%" ) ))
( "misuse"  v ( m i2 s y uu1 z ) ((pos "H2%" ) (subcat )))
( "misused"  v ( m i2 s y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "misuses"  n ( m i2 s y uu1 s i z ) ((pos "Mj$" ) ))
( "misuses"  v ( m i2 s y uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "misusing"  v ( m i2 s y uu1 z i ng ) ((pos "Hb%" ) (subcat )))
( "mite"  n ( m ai1 t ) ((pos "K6$" ) ))
( "mites"  n ( m ai1 t s ) ((pos "Kj$" ) ))
( "mitigate"  v ( m i1 t i g ei t ) ((pos "H2%" ) (subcat )))
( "mitigated"  v ( m i1 t i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mitigates"  v ( m i1 t i g ei t s ) ((pos "Ha%" ) (subcat )))
( "mitigating"  v ( m i1 t i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "mitigation"  n ( m i2 t i g ei1 sh @ n ) ((pos "L@%" ) ))
( "mitre"  n ( m ai1 t @ r ) ((pos "K6%" ) ))
( "mitre-joint"  n ( m ai1 t @ - jh oi n t ) ((pos "K6$" ) ))
( "mitre-joints"  n ( m ai1 t @ - jh oi n t s ) ((pos "Kj$" ) ))
( "mitres"  n ( m ai1 t @ z ) ((pos "Kj%" ) ))
( "mitt"  n ( m i1 t ) ((pos "K6%" ) ))
( "mitten"  n ( m i1 t n ) ((pos "K6%" ) ))
( "mittens"  n ( m i1 t n z ) ((pos "Kj%" ) ))
( "mitts"  n ( m i1 t s ) ((pos "Kj%" ) ))
( "mix"  v ( m i1 k s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" )))
( "mix-up"  n ( m i1 k s - uh p ) ((pos "K6%" ) ))
( "mix-ups"  n ( m i1 k s - uh p s ) ((pos "Kj%" ) ))
( "mixed"  v ( m i1 k s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" )))
( "mixed-up"  j ( m i1 k s t - uh1 p ) ((pos "OA%" ) ))
( "mixer"  n ( m i1 k s @ r ) ((pos "K6%" ) ))
( "mixers"  n ( m i1 k s @ z ) ((pos "Kj%" ) ))
( "mixes"  v ( m i1 k s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" )))
( "mixing"  v ( m i1 k s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" "14" )))
( "mixture"  n ( m i1 k s ch @ r ) ((pos "M6%" ) ))
( "mixtures"  n ( m i1 k s ch @ z ) ((pos "Mj%" ) ))
( "mizen"  n ( m i1 z n ) ((pos "K6$" ) ))
( "mizens"  n ( m i1 z n z ) ((pos "Kj$" ) ))
( "mizzen"  n ( m i1 z n ) ((pos "K6$" ) ))
( "mizzen-mast"  n ( m i1 z n - m aa s t ) ((pos "K6$" ) ))
( "mizzen-masts"  n ( m i1 z n - m aa s t s ) ((pos "Kj$" ) ))
( "mizzens"  n ( m i1 z n z ) ((pos "Kj$" ) ))
( "mizzle"  v ( m i1 z l ) ((pos "I2$" ) (subcat )))
( "mizzled"  v ( m i1 z l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "mizzles"  v ( m i1 z l z ) ((pos "Ia$" ) (subcat )))
( "mizzling"  v ( m i1 z l i ng ) ((pos "Ib$" ) (subcat )))
( "ml"  n ( m i1 l i l ii t @ z ) ((pos "Y]%" ) ))
( "mm"  n ( m i1 l i m ii t @ z ) ((pos "Y]%" ) ))
( "mnemonic"  j ( n i1 m o1 n i k ) ((pos "OA%" ) ))
( "mnemonics"  n ( n i1 m o1 n i k s ) ((pos "Lk%" ) ))
( "mo"  n ( m ou1 ) ((pos "Ki$" ) ))
( "moan"  v ( m ou1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "moaned"  v ( m ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "moaning"  v ( m ou1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "moans"  v ( m ou1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "moat"  n ( m ou1 t ) ((pos "K6%" ) ))
( "moated"  j ( m ou1 t i d ) ((pos "OA%" ) ))
( "moats"  n ( m ou1 t s ) ((pos "Kj%" ) ))
( "mob"  v ( m o1 b ) ((pos "H4%" "K6%" ) (subcat )))
( "mobbed"  v ( m o1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mobbing"  v ( m o1 b i ng ) ((pos "Hb%" ) (subcat )))
( "mobcap"  n ( m o1 b k a p ) ((pos "K6$" ) ))
( "mobcaps"  n ( m o1 b k a p s ) ((pos "Kj$" ) ))
( "mobile"  n ( m ou1 b ai l ) ((pos "K6%" "OA%" ) ))
( "mobiles"  n ( m ou1 b ai l z ) ((pos "Kj%" ) ))
( "mobility"  n ( m ou1 b i1 l i t ii ) ((pos "L@%" ) ))
( "mobilization"  n ( m ou2 b i l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "mobilizations"  n ( m ou2 b i l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "mobilize"  v ( m ou1 b i l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "mobilized"  v ( m ou1 b i l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mobilizes"  v ( m ou1 b i l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "mobilizing"  v ( m ou1 b i l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mobs"  v ( m o1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mobster"  n ( m o1 b s t @ r ) ((pos "K6%" ) ))
( "mobsters"  n ( m o1 b s t @ z ) ((pos "Kj%" ) ))
( "moccasin"  n ( m o1 k @ s i n ) ((pos "M6%" ) ))
( "moccasins"  n ( m o1 k @ s i n z ) ((pos "Mj%" ) ))
( "mocha"  n ( m o1 k @ ) ((pos "L@$" ) ))
( "mock"  v ( m o1 k ) ((pos "J0%" "K6%" "Oq%" ) (subcat "3A" )))
( "mock-up"  n ( m o1 k - uh p ) ((pos "K6%" ) ))
( "mock-ups"  n ( m o1 k - uh p s ) ((pos "Kj%" ) ))
( "mocked"  v ( m o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "mocker"  n ( m o1 k @ r ) ((pos "K6%" ) ))
( "mockeries"  n ( m o1 k @ r i z ) ((pos "Mj%" ) ))
( "mockers"  n ( m o1 k @ z ) ((pos "Kj%" ) ))
( "mockery"  n ( m o1 k @ r ii ) ((pos "M8%" ) ))
( "mocking"  v ( m o1 k i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "mockingly"  a ( m o1 k i ng l ii ) ((pos "Pu%" ) ))
( "mocks"  v ( m o1 k s ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "mod"  n ( m o1 d ) ((pos "K6$" "OA$" ) ))
( "mod cons"  n ( m o2 d - k o1 n z ) ((pos "Y)%" ) ))
( "modal"  j ( m ou1 d l ) ((pos "OA%" ) ))
( "modalities"  n ( m ou1 d a1 l i t i z ) ((pos "Mj%" ) ))
( "modality"  n ( m ou1 d a1 l i t ii ) ((pos "M8%" ) ))
( "mode"  n ( m ou1 d ) ((pos "K6%" ) ))
( "model"  v ( m o1 d l ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" "14" )))
( "modeler"  n ( m o1 d l @ r ) ((pos "K6%" ) ))
( "modelers"  n ( m o1 d l @ z ) ((pos "Kj%" ) ))
( "modelled"  v ( m o1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "modeller"  n ( m o1 d l @ r ) ((pos "K6%" ) ))
( "modellers"  n ( m o1 d l @ z ) ((pos "Kj%" ) ))
( "modelling"  v ( m o1 d @ l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "6A" "14" )))
( "models"  v ( m o1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "14" )))
( "moderate"  n ( m o1 d @ r @ t ) ((pos "K6%" "OA%" ) ))
( "moderate"  v ( m o1 d @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "moderated"  v ( m o1 d @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "moderately"  a ( m o1 d @ r @ t l ii ) ((pos "Pu%" ) ))
( "moderates"  n ( m o1 d @ r @ t s ) ((pos "Kj%" ) ))
( "moderates"  v ( m o1 d @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "moderating"  v ( m o1 d @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "moderation"  n ( m o2 d @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "moderator"  n ( m o1 d @ r ei t @ r ) ((pos "K6%" ) ))
( "moderators"  n ( m o1 d @ r ei t @ z ) ((pos "Kj%" ) ))
( "modern"  n ( m o1 d n ) ((pos "K6%" "OA%" ) ))
( "modernism"  n ( m o1 d @ n i z @ m ) ((pos "L@%" ) ))
( "modernist"  n ( m o1 d @ n i s t ) ((pos "K6%" ) ))
( "modernistic"  j ( m o2 d @ n i1 s t i k ) ((pos "OA%" ) ))
( "modernists"  n ( m o1 d @ n i s t s ) ((pos "Kj%" ) ))
( "modernity"  n ( m @1 d @@1 n i t ii ) ((pos "L@%" ) ))
( "modernization"  n ( m o2 d @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "modernize"  v ( m o1 d @ n ai z ) ((pos "H2%" ) (subcat )))
( "modernized"  v ( m o1 d @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "modernizes"  v ( m o1 d @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "modernizing"  v ( m o1 d @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "moderns"  n ( m o1 d n z ) ((pos "Kj$" ) ))
( "modes"  n ( m ou1 d z ) ((pos "Kj%" ) ))
( "modest"  j ( m o1 d i s t ) ((pos "OA%" ) ))
( "modestly"  a ( m o1 d i s t l ii ) ((pos "Pu%" ) ))
( "modesty"  n ( m o1 d i s t ii ) ((pos "L@%" ) ))
( "modicum"  n ( m o1 d i k @ m ) ((pos "Ki%" ) ))
( "modification"  n ( m o2 d i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "modifications"  n ( m o2 d i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "modified"  v ( m o1 d i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "modifier"  n ( m o1 d i f ai @ r ) ((pos "K6%" ) ))
( "modifiers"  n ( m o1 d i f ai @ z ) ((pos "Kj%" ) ))
( "modifies"  v ( m o1 d i f ai z ) ((pos "Ha%" ) (subcat )))
( "modify"  v ( m o1 d i f ai ) ((pos "H3%" ) (subcat )))
( "modifying"  v ( m o1 d i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "modish"  j ( m ou1 d i sh ) ((pos "OA%" ) ))
( "modishly"  a ( m ou1 d i sh l ii ) ((pos "Pu%" ) ))
( "modiste"  n ( m ou1 d ii1 s t ) ((pos "K6$" ) ))
( "modistes"  n ( m ou1 d ii1 s t s ) ((pos "Kj$" ) ))
( "mods"  n ( m o1 d z ) ((pos "Kj%" ) ))
( "modular"  j ( m o1 d y u l @ r ) ((pos "OA%" ) ))
( "modulate"  v ( m o1 d y u l ei t ) ((pos "J2%" ) (subcat "2C" )))
( "modulated"  v ( m o1 d y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "modulates"  v ( m o1 d y u l ei t s ) ((pos "Ja%" ) (subcat "2C" )))
( "modulating"  v ( m o1 d y u l ei t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "modulation"  n ( m o2 d y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "modulations"  n ( m o2 d y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "module"  n ( m o1 d y uu l ) ((pos "K6%" ) ))
( "modules"  n ( m o1 d y uu l z ) ((pos "Kj%" ) ))
( "modus operandi"  n ( m ou2 d @ s - o2 p @ r a1 n d ii ) ((pos "Ki%" ) ))
( "modus vivendi"  n ( m ou2 d @ s - v i v e1 n d ii ) ((pos "Ki%" ) ))
( "mogul"  n ( m ou1 g l ) ((pos "K6$" ) ))
( "moguls"  n ( m ou1 g l z ) ((pos "Kj$" ) ))
( "mohair"  n ( m ou1 h e@ r ) ((pos "L@%" ) ))
( "moieties"  n ( m oi1 @ t i z ) ((pos "Kj$" ) ))
( "moiety"  n ( m oi1 @ t ii ) ((pos "K8$" ) ))
( "moil"  v ( m oi1 l ) ((pos "I0$" ) ))
( "moiled"  v ( m oi1 l d ) ((pos "Ic$" "Id$" ) ))
( "moiling"  v ( m oi1 l i ng ) ((pos "Ib$" ) ))
( "moils"  v ( m oi1 l z ) ((pos "Ia$" ) ))
( "moist"  j ( m oi1 s t ) ((pos "OA%" ) ))
( "moisten"  v ( m oi1 s n ) ((pos "J0%" ) (subcat "2A" )))
( "moistened"  v ( m oi1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "moistening"  v ( m oi1 s n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "moistens"  v ( m oi1 s n z ) ((pos "Ja%" ) (subcat "2A" )))
( "moistly"  a ( m oi1 s t l ii ) ((pos "Pu%" ) ))
( "moisture"  n ( m oi1 s ch @ r ) ((pos "L@%" ) ))
( "moke"  n ( m ou1 k ) ((pos "K6$" ) ))
( "mokes"  n ( m ou1 k s ) ((pos "Kj$" ) ))
( "molar"  n ( m ou1 l @ r ) ((pos "K6%" "OA%" ) ))
( "molars"  n ( m ou1 l @ z ) ((pos "Kj%" ) ))
( "molasses"  n ( m @1 l a1 s i z ) ((pos "L@%" ) ))
( "moldier"  j ( m ou1 l d i@ r ) ((pos "Or%" ) ))
( "moldiest"  j ( m ou1 l d i i s t ) ((pos "Os%" ) ))
( "moldy"  j ( m ou1 l d ii ) ((pos "OD%" ) ))
( "mole"  n ( m ou1 l ) ((pos "K6%" ) ))
( "molecular"  j ( m @1 l e1 k y u l @ r ) ((pos "OA%" ) ))
( "molecule"  n ( m o1 l i k y uu l ) ((pos "K6%" ) ))
( "molecules"  n ( m o1 l i k y uu l z ) ((pos "Kj%" ) ))
( "molehill"  n ( m ou1 l h i l ) ((pos "K6%" ) ))
( "molehills"  n ( m ou1 l h i l z ) ((pos "Kj%" ) ))
( "moles"  n ( m ou1 l z ) ((pos "Kj%" ) ))
( "moleskin"  n ( m ou1 l s k i n ) ((pos "K6%" ) ))
( "moleskins"  n ( m ou1 l s k i n z ) ((pos "Kj%" ) ))
( "molest"  v ( m @1 l e1 s t ) ((pos "H0%" ) (subcat )))
( "molestation"  n ( m ou2 l @ s t ei1 sh @ n ) ((pos "L@%" ) ))
( "molested"  v ( m @1 l e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "molesting"  v ( m @1 l e1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "molests"  v ( m @1 l e1 s t s ) ((pos "Ha%" ) (subcat )))
( "moll"  n ( m o1 l ) ((pos "K6%" ) ))
( "mollification"  n ( m o2 l i f i k ei1 sh @ n ) ((pos "L@$" ) ))
( "mollified"  v ( m o1 l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mollifies"  v ( m o1 l i f ai z ) ((pos "Ha%" ) (subcat )))
( "mollify"  v ( m o1 l i f ai ) ((pos "H3%" ) (subcat )))
( "mollifying"  v ( m o1 l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "molls"  n ( m o1 l z ) ((pos "Kj$" ) ))
( "mollusc"  n ( m o1 l @ s k ) ((pos "K6%" ) ))
( "molluscs"  n ( m o1 l @ s k s ) ((pos "Kj%" ) ))
( "mollycoddle"  v ( m o1 l i k o d l ) ((pos "H2%" "K6%" ) (subcat )))
( "mollycoddled"  v ( m o1 l i k o d l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mollycoddles"  v ( m o1 l i k o d l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mollycoddling"  v ( m o1 l i k o d l i ng ) ((pos "Hb%" ) (subcat )))
( "molten"  v ( m ou1 l t @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" )))
( "molto"  a ( m o1 l t ou ) ((pos "Pu$" ) ))
( "molybdenum"  n ( m @1 l i1 b d @ n @ m ) ((pos "L@$" ) ))
( "moment"  n ( m ou1 m @ n t ) ((pos "M6*" ) ))
( "momentarily"  a ( m ou1 m @ n t r @ l ii ) ((pos "Pu%" ) ))
( "momentary"  j ( m ou1 m @ n t r ii ) ((pos "OA%" ) ))
( "momentous"  j ( m @1 m e1 n t @ s ) ((pos "OA%" ) ))
( "momentously"  a ( m @1 m e1 n t @ s l ii ) ((pos "Pu%" ) ))
( "momentousness"  n ( m @1 m e1 n t @ s n @ s ) ((pos "L@$" ) ))
( "moments"  n ( m ou1 m @ n t s ) ((pos "Mj%" ) ))
( "momentum"  n ( m @1 m e1 n t @ m ) ((pos "L@%" ) ))
( "monarch"  n ( m o1 n @ k ) ((pos "K6%" ) ))
( "monarchic"  j ( m @1 n aa1 k i k ) ((pos "OA$" ) ))
( "monarchies"  n ( m o1 n @ k i z ) ((pos "Mj%" ) ))
( "monarchism"  n ( m o1 n @ k i z @ m ) ((pos "L@%" ) ))
( "monarchist"  n ( m o1 n @ k i s t ) ((pos "K6%" ) ))
( "monarchists"  n ( m o1 n @ k i s t s ) ((pos "Kj%" ) ))
( "monarchs"  n ( m o1 n @ k s ) ((pos "Kj%" ) ))
( "monarchy"  n ( m o1 n @ k ii ) ((pos "M8%" ) ))
( "monasteries"  n ( m o1 n @ s t r i z ) ((pos "Kj%" ) ))
( "monastery"  n ( m o1 n @ s t r ii ) ((pos "K8%" ) ))
( "monastic"  j ( m @1 n a1 s t i k ) ((pos "OA%" ) ))
( "monasticism"  n ( m @1 n a1 s t i s i z @ m ) ((pos "L@%" ) ))
( "monaural"  j ( m o2 n oo1 r @ l ) ((pos "OA$" ) ))
( "monetary"  j ( m uh1 n i t r ii ) ((pos "OA%" ) ))
( "monetize"  v ( m uh1 n @ t ai z ) ((pos "H2$" ) (subcat )))
( "monetized"  v ( m uh1 n @ t ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "monetizes"  v ( m uh1 n @ t ai z i z ) ((pos "Ha$" ) (subcat )))
( "monetizing"  v ( m uh1 n @ t ai z i ng ) ((pos "Hb$" ) (subcat )))
( "money"  n ( m uh1 n ii ) ((pos "L@*" ) ))
( "money-grubber"  n ( m uh1 n i - g r uh b @ r ) ((pos "K6%" ) ))
( "money-grubbers"  n ( m uh1 n i - g r uh b @ z ) ((pos "Kj%" ) ))
( "money-order"  n ( m uh1 n i - oo d @ r ) ((pos "K6%" ) ))
( "money-orders"  n ( m uh1 n i - oo d @ z ) ((pos "Kj%" ) ))
( "money-spinner"  n ( m uh1 n i - s p i n @ r ) ((pos "K6%" ) ))
( "money-spinners"  n ( m uh1 n i - s p i n @ z ) ((pos "Kj%" ) ))
( "moneybox"  n ( m uh1 n i b o k s ) ((pos "K7%" ) ))
( "moneyboxes"  n ( m uh1 n i b o k s i z ) ((pos "Kj%" ) ))
( "moneychanger"  n ( m uh1 n i ch ei n jh @ r ) ((pos "K6%" ) ))
( "moneychangers"  n ( m uh1 n i ch ei n jh @ z ) ((pos "Kj%" ) ))
( "moneyed"  j ( m uh1 n i d ) ((pos "OA%" ) ))
( "moneylender"  n ( m uh1 n i l e n d @ r ) ((pos "K6%" ) ))
( "moneylenders"  n ( m uh1 n i l e n d @ z ) ((pos "Kj%" ) ))
( "moneyless"  j ( m uh1 n i l @ s ) ((pos "OA%" ) ))
( "monger"  n ( m uh1 ng g @ r ) ((pos "K6$" ) ))
( "mongers"  n ( m uh1 ng g @ z ) ((pos "Kj$" ) ))
( "mongol"  n ( m o1 ng g l ) ((pos "K6%" "Oq%" ) ))
( "mongolism"  n ( m o1 ng g @ l i z @ m ) ((pos "L@%" ) ))
( "mongols"  n ( m o1 ng g l z ) ((pos "Kj%" ) ))
( "mongoose"  n ( m o1 ng g uu s ) ((pos "K6%" ) ))
( "mongooses"  n ( m o1 ng g uu s i z ) ((pos "Kj%" ) ))
( "mongrel"  n ( m uh1 ng g r @ l ) ((pos "K6%" "Oq%" ) ))
( "mongrels"  n ( m uh1 ng g r @ l z ) ((pos "Kj%" ) ))
( "monies"  n ( m uh1 n i z ) ((pos "Kj%" ) ))
( "monitor"  v ( m o1 n i t @ r ) ((pos "J0%" "K6%" ) ))
( "monitored"  v ( m o1 n i t @ d ) ((pos "Jc%" "Jd%" ) ))
( "monitoring"  v ( m o1 n i t @ r i ng ) ((pos "Jb%" ) ))
( "monitors"  v ( m o1 n i t @ z ) ((pos "Ja%" "Kj%" ) ))
( "monk"  n ( m uh1 ng k ) ((pos "K6%" ) ))
( "monkey"  v ( m uh1 ng k ii ) ((pos "I0%" "K6%" ) (subcat )))
( "monkey-jacket"  n ( m uh1 ng k i - jh a k i t ) ((pos "K6$" ) ))
( "monkey-jackets"  n ( m uh1 ng k i - jh a k i t s ) ((pos "Kj$" ) ))
( "monkey-nut"  n ( m uh1 ng k i - n uh t ) ((pos "K6%" ) ))
( "monkey-nuts"  n ( m uh1 ng k i - n uh t s ) ((pos "Kj%" ) ))
( "monkey-puzzle"  n ( m uh1 ng k i - p uh z l ) ((pos "K6%" ) ))
( "monkey-puzzles"  n ( m uh1 ng k i - p uh z l z ) ((pos "Kj%" ) ))
( "monkey-wrench"  n ( m uh1 ng k i - r e n ch ) ((pos "K7%" ) ))
( "monkey-wrenches"  n ( m uh1 ng k i - r e n ch i z ) ((pos "Kj%" ) ))
( "monkeyed"  v ( m uh1 ng k i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "monkeying"  v ( m uh1 ng k i i ng ) ((pos "Ib%" ) (subcat )))
( "monkeys"  v ( m uh1 ng k i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "monkish"  j ( m uh1 ng k i sh ) ((pos "OA%" ) ))
( "monks"  n ( m uh1 ng k s ) ((pos "Kj%" ) ))
( "mono"  j ( m o1 n ou ) ((pos "OA%" ) ))
( "monochrome"  n ( m o1 n @ k r ou m ) ((pos "K6%" "OA%" ) ))
( "monochromes"  n ( m o1 n @ k r ou m z ) ((pos "Kj$" ) ))
( "monocle"  n ( m o1 n @ k l ) ((pos "K6%" ) ))
( "monocled"  j ( m o1 n @ k l d ) ((pos "OA%" ) ))
( "monocles"  n ( m o1 n @ k l z ) ((pos "Kj%" ) ))
( "monogamist"  n ( m @1 n o1 g @ m i s t ) ((pos "K6$" ) ))
( "monogamists"  n ( m @1 n o1 g @ m i s t s ) ((pos "Kj$" ) ))
( "monogamous"  j ( m @1 n o1 g @ m @ s ) ((pos "OA%" ) ))
( "monogamy"  n ( m @1 n o1 g @ m ii ) ((pos "L@%" ) ))
( "monogram"  n ( m o1 n @ g r a m ) ((pos "K6%" ) ))
( "monograms"  n ( m o1 n @ g r a m z ) ((pos "Kj%" ) ))
( "monograph"  n ( m o1 n @ g r aa f ) ((pos "K6%" ) ))
( "monographs"  n ( m o1 n @ g r aa f s ) ((pos "Kj%" ) ))
( "monolith"  n ( m o1 n @ l i th ) ((pos "K6%" ) ))
( "monolithic"  j ( m o2 n @ l i1 th i k ) ((pos "OA%" ) ))
( "monoliths"  n ( m o1 n @ l i th s ) ((pos "Kj%" ) ))
( "monologue"  n ( m o1 n @ l o g ) ((pos "K6%" ) ))
( "monologues"  n ( m o1 n @ l o g z ) ((pos "Kj%" ) ))
( "monomania"  n ( m o2 n ou m ei1 n i@ ) ((pos "M6%" ) ))
( "monomaniac"  n ( m o2 n ou m ei1 n i a k ) ((pos "K6%" ) ))
( "monomaniacs"  n ( m o2 n ou m ei1 n i a k s ) ((pos "Kj%" ) ))
( "monomanias"  n ( m o2 n ou m ei1 n i@ z ) ((pos "Mj$" ) ))
( "monoplane"  n ( m o1 n @ p l ei n ) ((pos "K6%" ) ))
( "monoplanes"  n ( m o1 n @ p l ei n z ) ((pos "Kj%" ) ))
( "monopolies"  n ( m @1 n o1 p @ l i z ) ((pos "Kj%" ) ))
( "monopolist"  n ( m @1 n o1 p @ l i s t ) ((pos "K6$" ) ))
( "monopolistic"  j ( m @1 n o2 p @ l i1 s t i k ) ((pos "OA%" ) ))
( "monopolists"  n ( m @1 n o1 p @ l i s t s ) ((pos "Kj$" ) ))
( "monopolization"  n ( m @1 n o2 p @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "monopolizations"  n ( m @1 n o2 p @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "monopolize"  v ( m @1 n o1 p @ l ai z ) ((pos "H2%" ) (subcat )))
( "monopolized"  v ( m @1 n o1 p @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "monopolizes"  v ( m @1 n o1 p @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "monopolizing"  v ( m @1 n o1 p @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "monopoly"  n ( m @1 n o1 p @ l ii ) ((pos "K8%" ) ))
( "monorail"  n ( m o1 n ou r ei l ) ((pos "K6%" ) ))
( "monorails"  n ( m o1 n ou r ei l z ) ((pos "Kj%" ) ))
( "monosyllabic"  j ( m o2 n @ s i l a1 b i k ) ((pos "OA%" ) ))
( "monosyllable"  n ( m o1 n @ s i l @ b l ) ((pos "K6%" ) ))
( "monosyllables"  n ( m o1 n @ s i l @ b l z ) ((pos "Kj%" ) ))
( "monotheism"  n ( m o1 n ou th ii i z @ m ) ((pos "L@$" ) ))
( "monotheist"  n ( m o1 n ou th ii i s t ) ((pos "K6$" ) ))
( "monotheistic"  j ( m o2 n ou th ii i1 s t ii k ) ((pos "OA$" ) ))
( "monotheists"  n ( m o1 n ou th ii i s t s ) ((pos "Kj$" ) ))
( "monotone"  n ( m o1 n @ t ou n ) ((pos "K6%" ) ))
( "monotones"  n ( m o1 n @ t ou n z ) ((pos "Kj%" ) ))
( "monotonous"  j ( m @1 n o1 t @ n @ s ) ((pos "OA%" ) ))
( "monotonously"  a ( m @1 n o1 t @ n @ s l ii ) ((pos "Pu%" ) ))
( "monotony"  n ( m @1 n o1 t @ n ii ) ((pos "L@%" ) ))
( "monotype"  n ( m o1 n @ t ai p ) ((pos "K6$" ) ))
( "monotypes"  n ( m o1 n @ t ai p s ) ((pos "Kj$" ) ))
( "monoxide"  n ( m o1 n o1 k s ai d ) ((pos "M6%" ) ))
( "monoxides"  n ( m o1 n o1 k s ai d z ) ((pos "Mj%" ) ))
( "monsoon"  n ( m o1 n s uu1 n ) ((pos "K6%" ) ))
( "monsoons"  n ( m o1 n s uu1 n z ) ((pos "Kj%" ) ))
( "monster"  n ( m o1 n s t @ r ) ((pos "K6%" ) ))
( "monsters"  n ( m o1 n s t @ z ) ((pos "Kj%" ) ))
( "monstrance"  n ( m o1 n s t r @ n s ) ((pos "K6$" ) ))
( "monstrances"  n ( m o1 n s t r @ n s i z ) ((pos "Kj$" ) ))
( "monstrosities"  n ( m o1 n s t r o1 s i t i z ) ((pos "Mj%" ) ))
( "monstrosity"  n ( m o1 n s t r o1 s i t ii ) ((pos "M8%" ) ))
( "monstrous"  j ( m o1 n s t r @ s ) ((pos "OA%" ) ))
( "monstrously"  a ( m o1 n s t r @ s l ii ) ((pos "Pu%" ) ))
( "montage"  n ( m o1 n t aa zh ) ((pos "L@%" ) ))
( "month"  n ( m uh1 n th ) ((pos "K6%" ) ))
( "monthlies"  n ( m uh1 n th l i z ) ((pos "Kj%" ) ))
( "monthly"  n ( m uh1 n th l ii ) ((pos "K8%" "OA%" "Pu%" ) ))
( "months"  n ( m uh1 n th s ) ((pos "Kj%" ) ))
( "monument"  n ( m o1 n y u m @ n t ) ((pos "K6%" ) ))
( "monumental"  j ( m o2 n y u m e1 n t l ) ((pos "OA%" ) ))
( "monuments"  n ( m o1 n y u m @ n t s ) ((pos "Kj%" ) ))
( "moo"  v ( m uu1 ) ((pos "I0%" "K6%" ) ))
( "moo-cow"  n ( m uu1 - k au ) ((pos "K6%" ) ))
( "moo-cows"  n ( m uu1 - k au z ) ((pos "Kj%" ) ))
( "mooch"  v ( m uu1 ch ) ((pos "I1%" ) (subcat )))
( "mooched"  v ( m uu1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "mooches"  v ( m uu1 ch i z ) ((pos "Ia%" ) (subcat )))
( "mooching"  v ( m uu1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "mood"  n ( m uu1 d ) ((pos "K6%" ) ))
( "moodier"  j ( m uu1 d i@ r ) ((pos "Or%" ) ))
( "moodiest"  j ( m uu1 d i i s t ) ((pos "Os%" ) ))
( "moodily"  a ( m uu1 d i l ii ) ((pos "Pu%" ) ))
( "moodiness"  n ( m uu1 d i n @ s ) ((pos "L@%" ) ))
( "moods"  n ( m uu1 d z ) ((pos "Kj%" ) ))
( "moody"  j ( m uu1 d ii ) ((pos "OD%" ) ))
( "mooed"  v ( m uu1 d ) ((pos "Ic%" "Id%" ) ))
( "mooing"  v ( m uu1 i ng ) ((pos "Ib%" ) ))
( "moon"  v ( m uu1 n ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "moonbeam"  n ( m uu1 n b ii m ) ((pos "K6%" ) ))
( "moonbeams"  n ( m uu1 n b ii m z ) ((pos "Kj%" ) ))
( "mooned"  v ( m uu1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "moonflower"  n ( m uu1 n f l au @ r ) ((pos "K6$" ) ))
( "moonflowers"  n ( m uu1 n f l au @ z ) ((pos "Kj$" ) ))
( "moonier"  j ( m uu1 n i@ r ) ((pos "Or$" ) ))
( "mooniest"  j ( m uu1 n i i s t ) ((pos "Os$" ) ))
( "mooning"  v ( m uu1 n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "moonless"  j ( m uu1 n l @ s ) ((pos "OA%" ) ))
( "moonlight"  n ( m uu1 n l ai t ) ((pos "L@%" ) ))
( "moonlit"  j ( m uu1 n l i t ) ((pos "OA%" ) ))
( "moons"  v ( m uu1 n z ) ((pos "Ja$" "Kj%" ) (subcat "2C" )))
( "moonshine"  n ( m uu1 n sh ai n ) ((pos "L@%" ) ))
( "moonstone"  n ( m uu1 n s t ou n ) ((pos "K6%" ) ))
( "moonstones"  n ( m uu1 n s t ou n z ) ((pos "Kj%" ) ))
( "moonstruck"  j ( m uu1 n s t r uh k ) ((pos "OA%" ) ))
( "moony"  j ( m uu1 n ii ) ((pos "OD%" ) ))
( "moor"  v ( m u@1 r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "moorcock"  n ( m u@1 k o k ) ((pos "K6$" ) ))
( "moorcocks"  n ( m u@1 k o k s ) ((pos "Kj$" ) ))
( "moored"  v ( m u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "moorfowl"  n ( m u@1 f au l ) ((pos "K6%" ) ))
( "moorfowls"  n ( m u@1 f au l z ) ((pos "Kj$" ) ))
( "moorgame"  n ( m u@1 g ei m ) ((pos "K9$" ) ))
( "moorhen"  n ( m u@1 h e n ) ((pos "K6%" ) ))
( "moorhens"  n ( m u@1 h e n z ) ((pos "Kj%" ) ))
( "mooring"  v ( m u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "mooring-mast"  n ( m u@1 r i ng - m aa s t ) ((pos "K6$" ) ))
( "mooring-masts"  n ( m u@1 r i ng - m aa s t s ) ((pos "Kj$" ) ))
( "moorings"  n ( m u@1 r i ng z ) ((pos "Kj%" ) ))
( "moorland"  n ( m u@1 l a n d ) ((pos "M6%" ) ))
( "moorlands"  n ( m u@1 l a n d z ) ((pos "Mj%" ) ))
( "moors"  v ( m u@1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "moos"  v ( m uu1 z ) ((pos "Ia$" "Kj$" ) ))
( "moose"  n ( m uu1 s ) ((pos "K9%" ) ))
( "mooses"  n ( m uu1 s i z ) ((pos "Kj$" ) ))
( "moot"  v ( m uu1 t ) ((pos "H0$" "OA$" ) (subcat )))
( "mooted"  v ( m uu1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "mooting"  v ( m uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "moots"  v ( m uu1 t s ) ((pos "Ha$" ) (subcat )))
( "mop"  v ( m o1 p ) ((pos "J4%" "K6%" ) (subcat "6A" )))
( "mope"  v ( m ou1 p ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "moped"  n ( m ou1 p e d ) ((pos "K6%" ) ))
( "moped"  v ( m ou1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "mopeds"  n ( m ou1 p e d z ) ((pos "Kj%" ) ))
( "mopes"  v ( m ou1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "moping"  v ( m ou1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "mopped"  v ( m o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "mopping"  v ( m o1 p i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "mops"  v ( m o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "moquette"  n ( m @1 k e1 t ) ((pos "L@$" ) ))
( "moraine"  n ( m o1 r ei1 n ) ((pos "K6$" ) ))
( "moraines"  n ( m o1 r ei1 n z ) ((pos "Kj$" ) ))
( "moral"  n ( m o1 r @ l ) ((pos "K6%" "OA%" ) ))
( "morale"  n ( m @1 r aa1 l ) ((pos "L@%" ) ))
( "moralist"  n ( m o1 r @ l i s t ) ((pos "K6%" ) ))
( "moralistic"  j ( m o2 r @ l i1 s t i k ) ((pos "OA%" ) ))
( "moralists"  n ( m o1 r @ l i s t s ) ((pos "Kj%" ) ))
( "moralities"  n ( m @1 r a1 l i t i z ) ((pos "Mj%" ) ))
( "morality"  n ( m @1 r a1 l i t ii ) ((pos "M8%" ) ))
( "moralize"  v ( m o1 r @ l ai z ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "moralized"  v ( m o1 r @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "moralizes"  v ( m o1 r @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "moralizing"  v ( m o1 r @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "morally"  a ( m o1 r @ l ii ) ((pos "Pu%" ) ))
( "morals"  n ( m o1 r @ l z ) ((pos "Kj%" ) ))
( "morass"  n ( m @1 r a1 s ) ((pos "K7%" ) ))
( "morasses"  n ( m @1 r a1 s i z ) ((pos "Kj$" ) ))
( "moratoria"  n ( m o2 r @ t oo1 r i@ ) ((pos "Kj%" ) ))
( "moratorium"  n ( m o2 r @ t oo1 r i@ m ) ((pos "K6%" ) ))
( "moratoriums"  n ( m o2 r @ t oo1 r i@ m z ) ((pos "Kj%" ) ))
( "morbid"  j ( m oo1 b i d ) ((pos "OA%" ) ))
( "morbidities"  n ( m oo1 b i1 d i t i z ) ((pos "Mj$" ) ))
( "morbidity"  n ( m oo1 b i1 d i t ii ) ((pos "M8%" ) ))
( "morbidly"  a ( m oo1 b i d l ii ) ((pos "Pu%" ) ))
( "morbidness"  n ( m oo1 b i d n @ s ) ((pos "L@$" ) ))
( "mordant"  j ( m oo1 d n t ) ((pos "OA%" ) ))
( "more"  n ( m oo1 r ) ((pos "L@*" "Or*" "Pu*" ) ))
( "morello"  n ( m @1 r e1 l ou ) ((pos "K6%" ) ))
( "morellos"  n ( m @1 r e1 l ou z ) ((pos "Kj$" ) ))
( "moreover"  a ( m oo1 r ou1 v @ r ) ((pos "Pu%" ) ))
( "mores"  n ( m oo1 r ei z ) ((pos "Kj%" ) ))
( "morganatic"  j ( m oo2 g @ n a1 t i k ) ((pos "OA$" ) ))
( "morgue"  n ( m oo1 g ) ((pos "K6%" ) ))
( "morgues"  n ( m oo1 g z ) ((pos "Kj%" ) ))
( "moribund"  j ( m o1 r i b uh n d ) ((pos "OA%" ) ))
( "morn"  n ( m oo1 n ) ((pos "K6%" ) ))
( "morning"  n ( m oo1 n i ng ) ((pos "M6*" ) ))
( "morning-glories"  n ( m oo2 n i ng - g l oo1 r i z ) ((pos "Kj$" ) ))
( "morning-glory"  n ( m oo2 n i ng - g l oo1 r ii ) ((pos "K8$" ) ))
( "morning-room"  n ( m oo1 n i ng - r uu m ) ((pos "K6%" ) ))
( "morning-rooms"  n ( m oo1 n i ng - r uu m z ) ((pos "Kj%" ) ))
( "mornings"  n ( m oo1 n i ng z ) ((pos "Mj%" ) ))
( "morns"  n ( m oo1 n z ) ((pos "Kj$" ) ))
( "morocco"  n ( m @1 r o1 k ou ) ((pos "L@%" ) ))
( "moron"  n ( m oo1 r o n ) ((pos "K6%" ) ))
( "moronic"  j ( m @1 r o1 n i k ) ((pos "OA%" ) ))
( "morons"  n ( m oo1 r o n z ) ((pos "Kj%" ) ))
( "morose"  j ( m @1 r ou1 s ) ((pos "OA%" ) ))
( "morosely"  a ( m @1 r ou1 s l ii ) ((pos "Pu%" ) ))
( "moroseness"  n ( m @1 r ou1 s n @ s ) ((pos "L@%" ) ))
( "morpheme"  n ( m oo1 f ii m ) ((pos "K6$" ) ))
( "morphemes"  n ( m oo1 f ii m z ) ((pos "Kj$" ) ))
( "morphemic"  j ( m oo1 f ii1 m i k ) ((pos "OA$" ) ))
( "morphia"  n ( m oo1 f i@ ) ((pos "L@$" ) ))
( "morphine"  n ( m oo1 f ii n ) ((pos "L@%" ) ))
( "morphological"  j ( m oo2 f @ l o1 jh i k l ) ((pos "OA$" ) ))
( "morphologically"  a ( m oo2 f @ l o1 jh i k l ii ) ((pos "Pu$" ) ))
( "morphology"  n ( m oo1 f o1 l @ jh ii ) ((pos "L@$" ) ))
( "morris dance"  n ( m o1 r i s - d aa n s ) ((pos "K6$" ) ))
( "morris dances"  n ( m o1 r i s - d aa n s i z ) ((pos "Kj$" ) ))
( "morrow"  n ( m o1 r ou ) ((pos "K6%" ) ))
( "morrows"  n ( m o1 r ou z ) ((pos "Kj$" ) ))
( "morsel"  n ( m oo1 s l ) ((pos "K6%" ) ))
( "morsels"  n ( m oo1 s l z ) ((pos "Kj%" ) ))
( "mortal"  n ( m oo1 t l ) ((pos "K6%" "OA%" ) ))
( "mortality"  n ( m oo1 t a1 l i t ii ) ((pos "L@%" ) ))
( "mortally"  a ( m oo1 t @ l ii ) ((pos "Pu%" ) ))
( "mortals"  n ( m oo1 t l z ) ((pos "Kj%" ) ))
( "mortar"  v ( m oo1 t @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "mortarboard"  n ( m oo1 t @ b oo d ) ((pos "K6%" ) ))
( "mortarboards"  n ( m oo1 t @ b oo d z ) ((pos "Kj%" ) ))
( "mortared"  v ( m oo1 t @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "mortaring"  v ( m oo1 t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "mortars"  v ( m oo1 t @ z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "mortgage"  v ( m oo1 g i jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "mortgaged"  v ( m oo1 g i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mortgagee"  n ( m oo2 g i jh ii1 ) ((pos "K6%" ) ))
( "mortgagees"  n ( m oo2 g i jh ii1 z ) ((pos "Kj%" ) ))
( "mortgages"  v ( m oo1 g i jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "mortgaging"  v ( m oo1 g i jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "mortgagor"  n ( m oo2 g i jh oo1 r ) ((pos "K6%" ) ))
( "mortgagors"  n ( m oo2 g i jh oo1 z ) ((pos "Kj%" ) ))
( "mortice"  v ( m oo1 t i s ) ((pos "H2%" "K6%" ) (subcat "15A" )))
( "morticed"  v ( m oo1 t i s t ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "mortices"  v ( m oo1 t i s i z ) ((pos "Ha$" "Kj$" ) (subcat "15A" )))
( "mortician"  n ( m oo1 t i1 sh @ n ) ((pos "K6%" ) ))
( "morticians"  n ( m oo1 t i1 sh @ n z ) ((pos "Kj%" ) ))
( "morticing"  v ( m oo1 t i s i ng ) ((pos "Hb$" ) (subcat "15A" )))
( "mortification"  n ( m oo2 t i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "mortified"  v ( m oo1 t i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mortifies"  v ( m oo1 t i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "mortify"  v ( m oo1 t i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "mortifying"  v ( m oo1 t i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mortise"  v ( m oo1 t i s ) ((pos "H2$" "K6$" ) (subcat "15A" )))
( "mortised"  v ( m oo1 t i s t ) ((pos "Hc$" "Hd%" ) (subcat "15A" )))
( "mortises"  v ( m oo1 t i s i z ) ((pos "Ha$" "Kj$" ) (subcat "15A" )))
( "mortising"  v ( m oo1 t i s i ng ) ((pos "Hb$" ) (subcat "15A" )))
( "mortuaries"  n ( m oo1 ch @ r i z ) ((pos "Kj%" ) ))
( "mortuary"  n ( m oo1 ch @ r ii ) ((pos "K8%" ) ))
( "mosaic"  n ( m ou1 z ei1 i k ) ((pos "K6%" "OA%" ) ))
( "mosaics"  n ( m ou1 z ei1 i k s ) ((pos "Kj%" ) ))
( "moselle"  n ( m ou1 z e1 l ) ((pos "M6%" ) ))
( "moselles"  n ( m ou1 z e1 l z ) ((pos "Mj$" ) ))
( "mosey"  v ( m ou1 z ii ) ((pos "I0%" ) (subcat "2A" )))
( "moseyed"  v ( m ou1 z i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "moseying"  v ( m ou1 z i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "moseys"  v ( m ou1 z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "mosque"  n ( m o1 s k ) ((pos "K6%" ) ))
( "mosques"  n ( m o1 s k s ) ((pos "Kj%" ) ))
( "mosquito"  n ( m @1 s k ii1 t ou ) ((pos "K7%" ) ))
( "mosquito-craft"  n ( m @1 s k ii1 t ou - k r aa f t ) ((pos "K9$" ) ))
( "mosquito-net"  n ( m @1 s k ii1 t ou - n e t ) ((pos "K6%" ) ))
( "mosquito-nets"  n ( m @1 s k ii1 t ou - n e t s ) ((pos "Kj%" ) ))
( "mosquitoes"  n ( m @1 s k ii1 t ou z ) ((pos "Kj%" ) ))
( "moss"  n ( m o1 s ) ((pos "M7%" ) ))
( "moss-grown"  j ( m o1 s - g r ou n ) ((pos "OA$" ) ))
( "mosses"  n ( m o1 s i z ) ((pos "Mj%" ) ))
( "mossier"  j ( m o1 s i@ r ) ((pos "Or%" ) ))
( "mossiest"  j ( m o1 s i i s t ) ((pos "Os%" ) ))
( "mossy"  j ( m o1 s ii ) ((pos "OD%" ) ))
( "most"  n ( m ou1 s t ) ((pos "K9*" "Os*" "Pu*" ) ))
( "mostly"  a ( m ou1 s t l ii ) ((pos "Pu%" ) ))
( "mote"  n ( m ou1 t ) ((pos "K6$" ) ))
( "motel"  n ( m ou1 t e1 l ) ((pos "K6%" ) ))
( "motels"  n ( m ou1 t e1 l z ) ((pos "Kj%" ) ))
( "motes"  n ( m ou1 t s ) ((pos "Kj$" ) ))
( "moth"  n ( m o1 th ) ((pos "K6%" ) ))
( "moth-eaten"  j ( m o1 th - ii t n ) ((pos "OA%" ) ))
( "mothball"  n ( m o1 th b oo l ) ((pos "K6%" ) ))
( "mothballs"  n ( m o1 th b oo l z ) ((pos "Kj%" ) ))
( "mother"  v ( m uh1 dh @ r ) ((pos "H0%" "K6*" ) (subcat )))
( "mother-in-law"  n ( m uh1 dh @ r - i n - l oo ) ((pos "Ki%" ) ))
( "mother-of-pearl"  n ( m uh2 dh @ r - @ v - p @@1 l ) ((pos "L@%" ) ))
( "mothered"  v ( m uh1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "motherhood"  n ( m uh1 dh @ h u d ) ((pos "L@%" ) ))
( "mothering"  v ( m uh1 dh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "motherland"  n ( m uh1 dh @ l a n d ) ((pos "K6%" ) ))
( "motherlands"  n ( m uh1 dh @ l a n d z ) ((pos "Kj$" ) ))
( "motherless"  j ( m uh1 dh @ l @ s ) ((pos "OA%" ) ))
( "motherlike"  j ( m uh1 dh @ l ai k ) ((pos "OA%" ) ))
( "motherliness"  n ( m uh1 dh @ l i n @ s ) ((pos "L@%" ) ))
( "motherly"  j ( m uh1 dh @ l ii ) ((pos "OA%" ) ))
( "mothers"  v ( m uh1 dh @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mothers-in-law"  n ( m uh1 dh @ z - i n - l oo ) ((pos "Kj$" ) ))
( "mothproof"  v ( m o1 th p r uu f ) ((pos "H0%" "OA%" ) ))
( "mothproofed"  v ( m o1 th p r uu f t ) ((pos "Hc%" "Hd%" ) ))
( "mothproofing"  v ( m o1 th p r uu f i ng ) ((pos "Hb$" ) ))
( "mothproofs"  v ( m o1 th p r uu f s ) ((pos "Ha$" ) ))
( "moths"  n ( m o1 th s ) ((pos "Kj%" ) ))
( "motif"  n ( m ou1 t ii1 f ) ((pos "K6%" ) ))
( "motifs"  n ( m ou1 t ii1 f s ) ((pos "Kj%" ) ))
( "motion"  v ( m ou1 sh @ n ) ((pos "J0%" "M6%" ) (subcat "3A" "15A" "15B" )))
( "motioned"  v ( m ou1 sh @ n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "15A" "15B" )))
( "motioning"  v ( m ou1 sh @ n i ng ) ((pos "Jb$" ) (subcat "3A" "15A" "15B" )))
( "motionless"  j ( m ou1 sh @ n l @ s ) ((pos "OA%" ) ))
( "motionlessly"  a ( m ou1 sh @ n l @ s l ii ) ((pos "Pu%" ) ))
( "motions"  v ( m ou1 sh @ n z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "15A" "15B" )))
( "motivate"  v ( m ou1 t i v ei t ) ((pos "H2%" ) (subcat )))
( "motivated"  v ( m ou1 t i v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "motivates"  v ( m ou1 t i v ei t s ) ((pos "Ha%" ) (subcat )))
( "motivating"  v ( m ou1 t i v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "motivation"  n ( m ou2 t i v ei1 sh @ n ) ((pos "M6%" ) ))
( "motivations"  n ( m ou2 t i v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "motive"  n ( m ou1 t i v ) ((pos "K6%" "OA%" ) ))
( "motiveless"  j ( m ou1 t i v l @ s ) ((pos "OA%" ) ))
( "motives"  n ( m ou1 t i v z ) ((pos "Kj%" ) ))
( "motley"  n ( m o1 t l ii ) ((pos "L@$" "OA%" ) ))
( "motor"  v ( m ou1 t @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "motor-assisted"  j ( m ou2 t @ r - @ s i1 s t i d ) ((pos "OA%" ) ))
( "motorbike"  n ( m ou1 t @ b ai k ) ((pos "K6%" ) ))
( "motorbikes"  n ( m ou1 t @ b ai k s ) ((pos "Kj%" ) ))
( "motorboat"  n ( m ou1 t @ b ou t ) ((pos "K6%" ) ))
( "motorboats"  n ( m ou1 t @ b ou t s ) ((pos "Kj%" ) ))
( "motorcade"  n ( m ou1 t @ k ei d ) ((pos "K6%" ) ))
( "motorcades"  n ( m ou1 t @ k ei d z ) ((pos "Kj%" ) ))
( "motorcar"  n ( m ou1 t @ k aa r ) ((pos "K6%" ) ))
( "motorcars"  n ( m ou1 t @ k aa z ) ((pos "Kj%" ) ))
( "motorcoach"  n ( m ou1 t @ k ou ch ) ((pos "K7%" ) ))
( "motorcoaches"  n ( m ou1 t @ k ou ch i z ) ((pos "Kj%" ) ))
( "motorcycle"  n ( m ou1 t @ s ai k l ) ((pos "K6%" ) ))
( "motorcycles"  n ( m ou1 t @ s ai k l z ) ((pos "Kj%" ) ))
( "motored"  v ( m ou1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "motoring"  v ( m ou1 t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "motorist"  n ( m ou1 t @ r i s t ) ((pos "K6%" ) ))
( "motorists"  n ( m ou1 t @ r i s t s ) ((pos "Kj%" ) ))
( "motorize"  v ( m ou1 t @ r ai z ) ((pos "H2$" ) (subcat )))
( "motorized"  v ( m ou1 t @ r ai z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "motorizes"  v ( m ou1 t @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "motorizing"  v ( m ou1 t @ r ai z i ng ) ((pos "Hb$" ) (subcat )))
( "motorman"  n ( m ou1 t @ m @ n ) ((pos "Ki%" ) ))
( "motormen"  n ( m ou1 t @ m @ n ) ((pos "Kj%" ) ))
( "motors"  v ( m ou1 t @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "motorway"  n ( m ou1 t @ w ei ) ((pos "K6%" ) ))
( "motorways"  n ( m ou1 t @ w ei z ) ((pos "Kj%" ) ))
( "mottle"  v ( m o1 t l ) ((pos "H2$" ) (subcat )))
( "mottled"  v ( m o1 t l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "mottles"  v ( m o1 t l z ) ((pos "Ha$" ) (subcat )))
( "mottling"  v ( m o1 t l i ng ) ((pos "Hb$" ) (subcat )))
( "motto"  n ( m o1 t ou ) ((pos "K6%" ) ))
( "mottoes"  n ( m o1 t ou z ) ((pos "Kj%" ) ))
( "mottos"  n ( m o1 t ou z ) ((pos "Kj%" ) ))
( "moujik"  n ( m uu1 zh i k ) ((pos "K6$" ) ))
( "moujiks"  n ( m uu1 zh i k s ) ((pos "Kj$" ) ))
( "mould"  v ( m ou1 l d ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "moulded"  v ( m ou1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "moulder"  v ( m ou1 l d @ r ) ((pos "I0%" ) (subcat "2A" )))
( "mouldered"  v ( m ou1 l d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "mouldering"  v ( m ou1 l d @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "moulders"  v ( m ou1 l d @ z ) ((pos "Ia$" ) (subcat "2A" )))
( "mouldier"  j ( m ou1 l d i@ r ) ((pos "Or%" ) ))
( "mouldiest"  j ( m ou1 l d i i s t ) ((pos "Os%" ) ))
( "moulding"  v ( m ou1 l d i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" )))
( "mouldings"  n ( m ou1 l d i ng z ) ((pos "Mj%" ) ))
( "moulds"  v ( m ou1 l d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "mouldy"  j ( m ou1 l d ii ) ((pos "OD%" ) ))
( "moult"  v ( m ou1 l t ) ((pos "J0%" "K6$" ) (subcat "2A" )))
( "moulted"  v ( m ou1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "moulting"  v ( m ou1 l t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "moults"  v ( m ou1 l t s ) ((pos "Ja%" "Kj$" ) (subcat "2A" )))
( "mound"  n ( m au1 n d ) ((pos "K6%" ) ))
( "mounds"  n ( m au1 n d z ) ((pos "Kj%" ) ))
( "mount"  v ( m au1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "mountain"  n ( m au1 n t i n ) ((pos "K6%" ) ))
( "mountaineer"  n ( m au2 n t i n i@1 r ) ((pos "K6%" ) ))
( "mountaineering"  n ( m au2 n t i n i@1 r i ng ) ((pos "L@%" ) ))
( "mountaineers"  n ( m au2 n t i n i@1 z ) ((pos "Kj%" ) ))
( "mountainous"  j ( m au1 n t i n @ s ) ((pos "OA%" ) ))
( "mountains"  n ( m au1 n t i n z ) ((pos "Kj%" ) ))
( "mountebank"  n ( m au1 n t i b a ng k ) ((pos "K6$" ) ))
( "mountebanks"  n ( m au1 n t i b a ng k s ) ((pos "Kj$" ) ))
( "mounted"  v ( m au1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "mounting"  v ( m au1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "mounts"  v ( m au1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "mourn"  v ( m oo1 n ) ((pos "J0%" ) (subcat "3A" )))
( "mourned"  v ( m oo1 n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "mourner"  n ( m oo1 n @ r ) ((pos "K6%" ) ))
( "mourners"  n ( m oo1 n @ z ) ((pos "Kj%" ) ))
( "mournful"  j ( m oo1 n f @ l ) ((pos "OA%" ) ))
( "mournfully"  a ( m oo1 n f @ l ii ) ((pos "Pu%" ) ))
( "mourning"  v ( m oo1 n i ng ) ((pos "Jb%" "L@%" ) (subcat "3A" )))
( "mourning-band"  n ( m oo1 n i ng - b a n d ) ((pos "K6$" ) ))
( "mourning-bands"  n ( m oo1 n i ng - b a n d z ) ((pos "Kj$" ) ))
( "mourning-ring"  n ( m oo1 n i ng - r i ng ) ((pos "K6$" ) ))
( "mourning-rings"  n ( m oo1 n i ng - r i ng z ) ((pos "Kj$" ) ))
( "mourns"  v ( m oo1 n z ) ((pos "Ja%" ) (subcat "3A" )))
( "mouse"  v ( m au1 s ) ((pos "I2$" "Ki%" ) (subcat )))
( "moused"  v ( m au1 s t ) ((pos "Ic$" "Id$" ) (subcat )))
( "mouser"  n ( m au1 z @ r ) ((pos "K6%" ) ))
( "mousers"  n ( m au1 z @ z ) ((pos "Kj%" ) ))
( "mouses"  v ( m au1 s i z ) ((pos "Ia$" ) (subcat )))
( "mousetrap"  n ( m au1 s t r a p ) ((pos "K6%" ) ))
( "mousetraps"  n ( m au1 s t r a p s ) ((pos "Kj%" ) ))
( "mousier"  j ( m au1 s i@ r ) ((pos "Or%" ) ))
( "mousiest"  j ( m au1 s i i s t ) ((pos "Os%" ) ))
( "mousing"  v ( m au1 s i ng ) ((pos "Ib%" ) (subcat )))
( "mousse"  n ( m uu1 s ) ((pos "M6%" ) ))
( "mousses"  n ( m uu1 s i z ) ((pos "Mj%" ) ))
( "moustache"  n ( m @1 s t aa1 sh ) ((pos "K6%" ) ))
( "moustaches"  n ( m @1 s t aa1 sh i z ) ((pos "Kj%" ) ))
( "mousy"  j ( m au1 s ii ) ((pos "OD%" ) ))
( "mouth"  v ( m au1 dh ) ((pos "J0%" ) (subcat "2A" )))
( "mouth"  n ( m au1 th ) ((pos "K6%" ) ))
( "mouth-organ"  n ( m au1 th - oo g @ n ) ((pos "K6%" ) ))
( "mouth-organs"  n ( m au1 th - oo g @ n z ) ((pos "Kj%" ) ))
( "mouth-watering"  j ( m au1 th - w oo t @ r i ng ) ((pos "OA%" ) ))
( "mouthed"  v ( m au1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "mouthful"  n ( m au1 th f uh l ) ((pos "K6%" ) ))
( "mouthfuls"  n ( m au1 th f uh l z ) ((pos "Kj%" ) ))
( "mouthing"  v ( m au1 dh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "mouthpiece"  n ( m au1 th p ii s ) ((pos "K6%" ) ))
( "mouthpieces"  n ( m au1 th p ii s i z ) ((pos "Kj%" ) ))
( "mouths"  v ( m au1 dh z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "movable"  j ( m uu1 v @ b l ) ((pos "OA%" ) ))
( "movables"  n ( m uu1 v @ b l z ) ((pos "Kj%" ) ))
( "move"  v ( m uu1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "9" "15A" "15B" )))
( "moved"  v ( m uu1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "9" "15A" "15B" )))
( "movement"  n ( m uu1 v m @ n t ) ((pos "M6%" ) ))
( "movements"  n ( m uu1 v m @ n t s ) ((pos "Mj%" ) ))
( "mover"  n ( m uu1 v @ r ) ((pos "K6%" ) ))
( "movers"  n ( m uu1 v @ z ) ((pos "Kj%" ) ))
( "moves"  v ( m uu1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "9" "15A" "15B" )))
( "movie"  n ( m uu1 v ii ) ((pos "K6%" ) ))
( "movies"  n ( m uu1 v i z ) ((pos "Kj%" ) ))
( "moving"  v ( m uu1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "9" "15A" "15B" )))
( "mow"  v ( m ou1 ) ((pos "H0%" "K6%" ) (subcat "2A" "6A" )))
( "mow"  v ( m au1 ) ((pos "I0$" ) ))
( "mowed"  v ( m ou1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "mowed"  v ( m au1 d ) ((pos "Ic$" "Id$" ) ))
( "mower"  n ( m ou1 @ r ) ((pos "K6%" ) ))
( "mowers"  n ( m ou1 @ z ) ((pos "Kj%" ) ))
( "mowing"  v ( m ou1 i ng ) ((pos "Hb%" ) (subcat "2A" "6A" )))
( "mowing"  v ( m au1 i ng ) ((pos "Ib$" ) ))
( "mown"  v ( m ou1 n ) ((pos "Hd%" ) (subcat "2A" "6A" )))
( "mows"  v ( m ou1 z ) ((pos "Ha%" "Kj%" ) (subcat "2A" "6A" )))
( "mows"  v ( m au1 z ) ((pos "Ia$" ) ))
( "mpg"  n ( e2 m p ii2 jh ii1 ) ((pos "Y]%" ) ))
( "mph"  n ( e2 m p ii2 ei1 ch ) ((pos "Y]%" ) ))
( "much"  n ( m uh1 ch ) ((pos "Ki*" "OA*" "Pu*" ) ))
( "muchness"  n ( m uh1 ch n @ s ) ((pos "L@%" ) ))
( "mucilage"  n ( m y uu1 s i l i jh ) ((pos "L@$" ) ))
( "muck"  v ( m uh1 k ) ((pos "J0%" "L@%" ) (subcat "2C" "6A" )))
( "muck-heap"  n ( m uh1 k - h ii p ) ((pos "K6%" ) ))
( "muck-heaps"  n ( m uh1 k - h ii p s ) ((pos "Kj%" ) ))
( "mucked"  v ( m uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "muckier"  j ( m uh1 k i@ r ) ((pos "Or%" ) ))
( "muckiest"  j ( m uh1 k i i s t ) ((pos "Os%" ) ))
( "mucking"  v ( m uh1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "muckle"  n ( m uh1 k l ) ((pos "Ki$" ) ))
( "muckraker"  n ( m uh1 k r ei k @ r ) ((pos "K6%" ) ))
( "muckrakers"  n ( m uh1 k r ei k @ z ) ((pos "Kj$" ) ))
( "muckraking"  n ( m uh1 k r ei k i ng ) ((pos "L@%" ) ))
( "mucks"  v ( m uh1 k s ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "mucky"  j ( m uh1 k ii ) ((pos "OD%" ) ))
( "mucous"  j ( m y uu1 k @ s ) ((pos "OA%" ) ))
( "mucus"  n ( m y uu1 k @ s ) ((pos "L@%" ) ))
( "mud"  v ( m uh1 d ) ((pos "H4%" "L@%" ) (subcat )))
( "mud-bath"  n ( m uh1 d - b aa th ) ((pos "K6%" ) ))
( "mud-baths"  n ( m uh1 d - b aa dh z ) ((pos "Kj%" ) ))
( "mudded"  v ( m uh1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "muddied"  v ( m uh1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "muddier"  j ( m uh1 d i@ r ) ((pos "Or%" ) ))
( "muddies"  v ( m uh1 d i z ) ((pos "Ha%" ) (subcat )))
( "muddiest"  j ( m uh1 d i i s t ) ((pos "Os%" ) ))
( "mudding"  v ( m uh1 d i ng ) ((pos "Hb$" ) (subcat )))
( "muddle"  v ( m uh1 d l ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" )))
( "muddle-headed"  j ( m uh2 d l - h e1 d i d ) ((pos "OA%" ) ))
( "muddled"  v ( m uh1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "muddles"  v ( m uh1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "muddling"  v ( m uh1 d l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "muddy"  v ( m uh1 d ii ) ((pos "H3%" "OD%" ) (subcat )))
( "muddying"  v ( m uh1 d i i ng ) ((pos "Hb%" ) (subcat )))
( "mudguard"  n ( m uh1 d g aa d ) ((pos "K6%" ) ))
( "mudguards"  n ( m uh1 d g aa d z ) ((pos "Kj%" ) ))
( "muds"  v ( m uh1 d z ) ((pos "Ha$" ) (subcat )))
( "mudslinger"  n ( m uh1 d s l i ng @ r ) ((pos "K6%" ) ))
( "mudslingers"  n ( m uh1 d s l i ng @ z ) ((pos "Kj%" ) ))
( "muesli"  n ( m y uu1 z l ii ) ((pos "L@%" ) ))
( "muezzin"  n ( m uu1 e1 z i n ) ((pos "K6$" ) ))
( "muezzins"  n ( m uu1 e1 z i n z ) ((pos "Kj$" ) ))
( "muff"  v ( m uh1 f ) ((pos "H0%" "K6%" ) (subcat )))
( "muffed"  v ( m uh1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "muffin"  n ( m uh1 f i n ) ((pos "K6%" ) ))
( "muffin-man"  n ( m uh1 f i n - m a n ) ((pos "Ki%" ) ))
( "muffin-men"  n ( m uh1 f i n - m e n ) ((pos "Kj%" ) ))
( "muffing"  v ( m uh1 f i ng ) ((pos "Hb%" ) (subcat )))
( "muffins"  n ( m uh1 f i n z ) ((pos "Kj%" ) ))
( "muffle"  v ( m uh1 f l ) ((pos "H2%" ) (subcat "6A" )))
( "muffled"  v ( m uh1 f l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "muffler"  n ( m uh1 f l @ r ) ((pos "K6%" ) ))
( "mufflers"  n ( m uh1 f l @ z ) ((pos "Kj%" ) ))
( "muffles"  v ( m uh1 f l z ) ((pos "Ha%" ) (subcat "6A" )))
( "muffling"  v ( m uh1 f l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "muffs"  v ( m uh1 f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "mufti"  n ( m uh1 f t ii ) ((pos "L@%" ) ))
( "mug"  v ( m uh1 g ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "mugged"  v ( m uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mugger"  n ( m uh1 g @ r ) ((pos "K6%" ) ))
( "muggers"  n ( m uh1 g @ z ) ((pos "Kj%" ) ))
( "muggier"  j ( m uh1 g i@ r ) ((pos "Or$" ) ))
( "muggiest"  j ( m uh1 g i i s t ) ((pos "Os$" ) ))
( "mugginess"  n ( m uh1 g i n @ s ) ((pos "L@$" ) ))
( "mugging"  v ( m uh1 g i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "muggings"  n ( m uh1 g i ng z ) ((pos "Mj%" ) ))
( "muggins"  n ( m uh1 g i n z ) ((pos "K7%" ) ))
( "mugginses"  n ( m uh1 g i n z i z ) ((pos "Kj$" ) ))
( "muggy"  j ( m uh1 g ii ) ((pos "OD%" ) ))
( "mugs"  v ( m uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "mugwump"  n ( m uh1 g w uh m p ) ((pos "K6$" ) ))
( "mugwumps"  n ( m uh1 g w uh m p s ) ((pos "Kj$" ) ))
( "mulatto"  n ( m y uu1 l a1 t ou ) ((pos "K6$" ) ))
( "mulattoes"  n ( m y uu1 l a1 t ou z ) ((pos "Kj$" ) ))
( "mulattos"  n ( m y uu1 l a1 t ou z ) ((pos "Kj$" ) ))
( "mulberries"  n ( m uh1 l b r i z ) ((pos "Kj%" ) ))
( "mulberry"  n ( m uh1 l b r ii ) ((pos "K8%" ) ))
( "mulch"  v ( m uh1 l ch ) ((pos "H1$" "K7$" ) ))
( "mulched"  v ( m uh1 l ch t ) ((pos "Hc$" "Hd$" ) ))
( "mulches"  v ( m uh1 l ch i z ) ((pos "Ha$" "Kj$" ) ))
( "mulching"  v ( m uh1 l ch i ng ) ((pos "Hb$" ) ))
( "mulct"  v ( m uh1 l k t ) ((pos "H0$" ) (subcat "12C" )))
( "mulcted"  v ( m uh1 l k t i d ) ((pos "Hc$" "Hd$" ) (subcat "12C" )))
( "mulcting"  v ( m uh1 l k t i ng ) ((pos "Hb$" ) (subcat "12C" )))
( "mulcts"  v ( m uh1 l k t s ) ((pos "Ha$" ) (subcat "12C" )))
( "mule"  n ( m y uu1 l ) ((pos "K6%" ) ))
( "mules"  n ( m y uu1 l z ) ((pos "Kj%" ) ))
( "muleteer"  n ( m y uu2 l @ t i@1 r ) ((pos "K6%" ) ))
( "muleteers"  n ( m y uu2 l @ t i@1 z ) ((pos "Kj%" ) ))
( "mulish"  j ( m y uu1 l i sh ) ((pos "OA%" ) ))
( "mulishly"  a ( m y uu1 l i sh l ii ) ((pos "Pu%" ) ))
( "mulishness"  n ( m y uu1 l i sh n @ s ) ((pos "L@%" ) ))
( "mull"  v ( m uh1 l ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "mullah"  n ( m uh1 l @ ) ((pos "K6$" ) ))
( "mullahs"  n ( m uh1 l @ z ) ((pos "Kj$" ) ))
( "mulled"  v ( m uh1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mullein"  n ( m uh1 l i n ) ((pos "K6$" ) ))
( "mulleins"  n ( m uh1 l i n z ) ((pos "Kj$" ) ))
( "mullet"  n ( m uh1 l i t ) ((pos "K6%" ) ))
( "mullets"  n ( m uh1 l i t s ) ((pos "Kj$" ) ))
( "mulligatawny"  n ( m uh2 l i g @ t oo1 n ii ) ((pos "L@%" ) ))
( "mulling"  v ( m uh1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "mullion"  n ( m uh1 l i@ n ) ((pos "K6%" ) ))
( "mullioned"  j ( m uh1 l i@ n d ) ((pos "OA%" ) ))
( "mullions"  n ( m uh1 l i@ n z ) ((pos "Kj%" ) ))
( "mulls"  v ( m uh1 l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "multi-"  j ( m uh1 l t i - ) ((pos "U-%" ) ))
( "multi-lingual"  j ( m uh2 l t i - l i1 ng g w @ l ) ((pos "OA%" ) ))
( "multifarious"  j ( m uh2 l t i f e@1 r i@ s ) ((pos "OA%" ) ))
( "multifariously"  a ( m uh2 l t i f e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "multiform"  j ( m uh1 l t i f oo m ) ((pos "OA%" ) ))
( "multilateral"  j ( m uh2 l t i l a1 t @ r @ l ) ((pos "OA%" ) ))
( "multiple"  n ( m uh1 l t i p l ) ((pos "K6%" "OA%" ) ))
( "multiples"  n ( m uh1 l t i p l z ) ((pos "Kj%" ) ))
( "multiplex"  j ( m uh1 l t i p l e k s ) ((pos "OA$" ) ))
( "multiplication"  n ( m uh2 l t i p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "multiplications"  n ( m uh2 l t i p l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "multiplicity"  n ( m uh2 l t i p l i1 s i t ii ) ((pos "L@%" ) ))
( "multiplied"  v ( m uh1 l t i p l ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "multiplies"  v ( m uh1 l t i p l ai z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "multiply"  v ( m uh1 l t i p l ai ) ((pos "J3%" ) (subcat "2A" "6A" )))
( "multiplying"  v ( m uh1 l t i p l ai i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "multitude"  n ( m uh1 l t i t y uu d ) ((pos "M6%" ) ))
( "multitudes"  n ( m uh1 l t i t y uu d z ) ((pos "Mj%" ) ))
( "multitudinous"  j ( m uu2 l t i t y uu1 d i n @ s ) ((pos "OA%" ) ))
( "multum in parvo"  n ( m u2 l t @ m - i n - p aa1 v ou ) ((pos "L@$" ) ))
( "mum"  n ( m uh1 m ) ((pos "K6%" "OA%" "W-%" ) ))
( "mumble"  v ( m uh1 m b l ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "mumbled"  v ( m uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "mumbles"  v ( m uh1 m b l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "mumbling"  v ( m uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "mumbo-jumbo"  n ( m uh2 m b ou - jh uh1 m b ou ) ((pos "L@%" ) ))
( "mummer"  n ( m uh1 m @ r ) ((pos "K6$" ) ))
( "mummeries"  n ( m uh1 m @ r i z ) ((pos "Mj$" ) ))
( "mummers"  n ( m uh1 m @ z ) ((pos "Kj$" ) ))
( "mummery"  n ( m uh1 m @ r ii ) ((pos "M8$" ) ))
( "mummies"  n ( m uh1 m i z ) ((pos "Kj%" ) ))
( "mummification"  n ( m uh2 m i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "mummifications"  n ( m uh2 m i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "mummified"  v ( m uh1 m i f ai d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "mummifies"  v ( m uh1 m i f ai z ) ((pos "Ha$" ) (subcat )))
( "mummify"  v ( m uh1 m i f ai ) ((pos "H3$" ) (subcat )))
( "mummifying"  v ( m uh1 m i f ai i ng ) ((pos "Hb$" ) (subcat )))
( "mummy"  n ( m uh1 m ii ) ((pos "K8%" ) ))
( "mumps"  n ( m uh1 m p s ) ((pos "Lk%" ) ))
( "mums"  n ( m uh1 m z ) ((pos "Kj%" ) ))
( "munch"  v ( m uh1 n ch ) ((pos "J1%" ) (subcat "2A" "2C" )))
( "munched"  v ( m uh1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "munches"  v ( m uh1 n ch i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "munching"  v ( m uh1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "mundane"  j ( m uh1 n d ei1 n ) ((pos "OA%" ) ))
( "mundanely"  a ( m uh1 n d ei1 n l ii ) ((pos "Pu%" ) ))
( "municipal"  j ( m y uu1 n i1 s i p l ) ((pos "OA%" ) ))
( "municipalities"  n ( m y uu1 n i2 s i p a1 l i t i z ) ((pos "Kj%" ) ))
( "municipality"  n ( m y uu1 n i2 s i p a1 l i t ii ) ((pos "K8%" ) ))
( "municipally"  a ( m y uu1 n i1 s i p l ii ) ((pos "Pu%" ) ))
( "munificence"  n ( m y uu1 n i1 f i s n s ) ((pos "L@%" ) ))
( "munificent"  j ( m y uu1 n i1 f i s n t ) ((pos "OA%" ) ))
( "munificently"  a ( m y uu1 n i1 f i s n t l ii ) ((pos "Pu%" ) ))
( "muniments"  n ( m y uu1 n i m @ n t s ) ((pos "Kj$" ) ))
( "munition"  v ( m y uu1 n i1 sh @ n ) ((pos "H0$" "K6%" ) (subcat )))
( "munitioned"  v ( m y uu1 n i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "munitioning"  v ( m y uu1 n i1 sh @ n i ng ) ((pos "Hb$" ) (subcat )))
( "munitions"  v ( m y uu1 n i1 sh @ n z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "mural"  n ( m y u@1 r @ l ) ((pos "K6%" "OA%" ) ))
( "murals"  n ( m y u@1 r @ l z ) ((pos "Kj%" ) ))
( "murder"  v ( m @@1 d @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "murdered"  v ( m @@1 d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "murderer"  n ( m @@1 d @ r @ r ) ((pos "K6%" ) ))
( "murderers"  n ( m @@1 d @ r @ z ) ((pos "Kj%" ) ))
( "murderess"  n ( m @@1 d @ r i s ) ((pos "K7%" ) ))
( "murderesses"  n ( m @@1 d @ r i s i z ) ((pos "Kj%" ) ))
( "murdering"  v ( m @@1 d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "murderous"  j ( m @@1 d @ r @ s ) ((pos "OA%" ) ))
( "murderously"  a ( m @@1 d @ r @ s l ii ) ((pos "Pu%" ) ))
( "murders"  v ( m @@1 d @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "murk"  n ( m @@1 k ) ((pos "L@%" ) ))
( "murkier"  j ( m @@1 k i@ r ) ((pos "Or%" ) ))
( "murkiest"  j ( m @@1 k i i s t ) ((pos "Os%" ) ))
( "murkily"  a ( m @@1 k i l ii ) ((pos "Pu%" ) ))
( "murky"  j ( m @@1 k ii ) ((pos "OD%" ) ))
( "murmur"  v ( m @@1 m @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "murmured"  v ( m @@1 m @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "murmuring"  v ( m @@1 m @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "murmurs"  v ( m @@1 m @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "murphies"  n ( m @@1 f i z ) ((pos "Kj$" ) ))
( "murphy"  n ( m @@1 f ii ) ((pos "K8$" ) ))
( "murrain"  n ( m uh1 r i n ) ((pos "L@$" ) ))
( "muscatel"  n ( m uh2 s k @ t e1 l ) ((pos "L@$" ) ))
( "muscle"  v ( m uh1 s l ) ((pos "I2%" "M6%" ) (subcat )))
( "muscle-bound"  j ( m uh1 s l - b au n d ) ((pos "OA%" ) ))
( "muscled"  v ( m uh1 s l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "muscleman"  n ( m uh1 s l m a n ) ((pos "Ki%" ) ))
( "musclemen"  n ( m uh1 s l m e n ) ((pos "Kj%" ) ))
( "muscles"  v ( m uh1 s l z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "muscling"  v ( m uh1 s l i ng ) ((pos "Ib%" ) (subcat )))
( "muscular"  j ( m uh1 s k y u l @ r ) ((pos "OA%" ) ))
( "muse"  v ( m y uu1 z ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "mused"  v ( m y uu1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "muses"  v ( m y uu1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "museum"  n ( m y uu1 z i@1 m ) ((pos "K6%" ) ))
( "museums"  n ( m y uu1 z i@1 m z ) ((pos "Kj%" ) ))
( "mush"  n ( m uh1 sh ) ((pos "L@%" ) ))
( "mushier"  j ( m uh1 sh i@ r ) ((pos "Or%" ) ))
( "mushiest"  j ( m uh1 sh i i s t ) ((pos "Os%" ) ))
( "mushroom"  v ( m uh1 sh r u m ) ((pos "I0%" "K6%" ) (subcat )))
( "mushroomed"  v ( m uh1 sh r u m d ) ((pos "Ic%" "Id%" ) (subcat )))
( "mushrooming"  v ( m uh1 sh r u m i ng ) ((pos "Ib%" ) (subcat )))
( "mushrooms"  v ( m uh1 sh r u m z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "mushy"  j ( m uh1 sh ii ) ((pos "OD%" ) ))
( "music"  n ( m y uu1 z i k ) ((pos "L@%" ) ))
( "music-box"  n ( m y uu1 z i k - b o k s ) ((pos "K7%" ) ))
( "music-boxes"  n ( m y uu1 z i k - b o k s i z ) ((pos "Kj%" ) ))
( "music-hall"  n ( m y uu1 z i k - h oo l ) ((pos "K6%" ) ))
( "music-halls"  n ( m y uu1 z i k - h oo l z ) ((pos "Kj%" ) ))
( "music-stand"  n ( m y uu1 z i k - s t a n d ) ((pos "K6%" ) ))
( "music-stands"  n ( m y uu1 z i k - s t a n d z ) ((pos "Kj%" ) ))
( "music-stool"  n ( m y uu1 z i k - s t uu l ) ((pos "K6%" ) ))
( "music-stools"  n ( m y uu1 z i k - s t uu l z ) ((pos "Kj%" ) ))
( "musical"  n ( m y uu1 z i k l ) ((pos "K6%" "OA%" ) ))
( "musical-box"  n ( m y uu1 z i k l - b o k s ) ((pos "K7%" ) ))
( "musical-boxes"  n ( m y uu1 z i k l - b o k s i z ) ((pos "Kj%" ) ))
( "musically"  a ( m y uu1 z i k l ii ) ((pos "Pu%" ) ))
( "musicals"  n ( m y uu1 z i k l z ) ((pos "Kj%" ) ))
( "musician"  n ( m y uu1 z i1 sh @ n ) ((pos "K6%" ) ))
( "musicians"  n ( m y uu1 z i1 sh @ n z ) ((pos "Kj%" ) ))
( "musicianship"  n ( m y uu1 z i1 sh @ n sh i p ) ((pos "L@%" ) ))
( "musing"  v ( m y uu1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "musingly"  a ( m y uu1 z i ng l ii ) ((pos "Pu%" ) ))
( "musk"  n ( m uh1 s k ) ((pos "L@$" ) ))
( "musk-deer"  n ( m uh1 s k - d i@ r ) ((pos "K9$" ) ))
( "musk-rose"  n ( m uh1 s k - r ou z ) ((pos "K6$" ) ))
( "musk-roses"  n ( m uh1 s k - r ou z i z ) ((pos "Kj$" ) ))
( "musket"  n ( m uh1 s k i t ) ((pos "K6%" ) ))
( "musketeer"  n ( m uh2 s k i t i@1 r ) ((pos "K6%" ) ))
( "musketeers"  n ( m uh2 s k i t i@1 z ) ((pos "Kj%" ) ))
( "musketry"  n ( m uh1 s k i t r ii ) ((pos "L@$" ) ))
( "muskets"  n ( m uh1 s k i t s ) ((pos "Kj%" ) ))
( "muskier"  j ( m uh1 s k i@ r ) ((pos "Or$" ) ))
( "muskiest"  j ( m uh1 s k i i s t ) ((pos "Os$" ) ))
( "muskrat"  n ( m uh1 s k r a t ) ((pos "K6$" ) ))
( "muskrats"  n ( m uh1 s k r a t s ) ((pos "Kj$" ) ))
( "musky"  j ( m uh1 s k ii ) ((pos "OD%" ) ))
( "muslin"  n ( m uh1 z l i n ) ((pos "L@%" ) ))
( "musquash"  n ( m uh1 s k w o sh ) ((pos "L@$" ) ))
( "muss"  v ( m uh1 s ) ((pos "H1$" "M7$" ) (subcat "6A" )))
( "mussed"  v ( m uh1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "mussel"  n ( m uh1 s l ) ((pos "K6%" ) ))
( "mussels"  n ( m uh1 s l z ) ((pos "Kj%" ) ))
( "musses"  v ( m uh1 s i z ) ((pos "Ha$" "Mj$" ) (subcat "6A" )))
( "mussing"  v ( m uh1 s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "must"  v ( m uh s t ) ((pos "G5*" "M6%" ) (subcat )))
( "mustachio"  n ( m @1 s t aa1 sh i@ @ ) ((pos "K6$" ) ))
( "mustachios"  n ( m @1 s t aa1 sh i@ u z ) ((pos "Kj$" ) ))
( "mustang"  n ( m uh1 s t a ng ) ((pos "K6$" ) ))
( "mustangs"  n ( m uh1 s t a ng z ) ((pos "Kj$" ) ))
( "mustard"  n ( m uh1 s t @ d ) ((pos "L@%" ) ))
( "muster"  v ( m uh1 s t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "mustered"  v ( m uh1 s t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "mustering"  v ( m uh1 s t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "musters"  v ( m uh1 s t @ z ) ((pos "Ja$" "Kj$" ) (subcat "2A" "6A" )))
( "mustier"  j ( m uh1 s t i@ r ) ((pos "Or%" ) ))
( "mustiest"  j ( m uh1 s t i i s t ) ((pos "Os%" ) ))
( "mustiness"  n ( m uh1 s t i n @ s ) ((pos "L@%" ) ))
( "mustn't"  v ( m uh1 s n t ) ((pos "Gg%" ) ))
( "musts"  n ( m uh1 s t s ) ((pos "Mj$" ) ))
( "musty"  j ( m uh1 s t ii ) ((pos "OD%" ) ))
( "mutability"  n ( m y uu2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "mutable"  j ( m y uu1 t @ b l ) ((pos "OA%" ) ))
( "mutation"  n ( m y uu1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "mutations"  n ( m y uu1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "mutatis mutandis"  a ( m uu1 t aa2 t i s - m uu t a1 n d i s ) ((pos "Pu$" ) ))
( "mute"  v ( m y uu1 t ) ((pos "H2$" "K6%" "OA%" ) (subcat )))
( "muted"  v ( m y uu1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "mutely"  a ( m y uu1 t l ii ) ((pos "Pu%" ) ))
( "mutes"  v ( m y uu1 t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "mutilate"  v ( m y uu1 t i l ei t ) ((pos "H2%" ) (subcat )))
( "mutilated"  v ( m y uu1 t i l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mutilates"  v ( m y uu1 t i l ei t s ) ((pos "Ha%" ) (subcat )))
( "mutilating"  v ( m y uu1 t i l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "mutilation"  n ( m y uu2 t i l ei1 sh @ n ) ((pos "M6%" ) ))
( "mutilations"  n ( m y uu2 t i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "mutineer"  n ( m y uu2 t i n i@1 r ) ((pos "K6%" ) ))
( "mutineers"  n ( m y uu2 t i n i@1 z ) ((pos "Kj%" ) ))
( "muting"  v ( m y uu1 t i ng ) ((pos "Hb$" ) (subcat )))
( "mutinied"  v ( m y uu1 t i n i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "mutinies"  v ( m y uu1 t i n i z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "mutinous"  j ( m y uu1 t i n @ s ) ((pos "OA%" ) ))
( "mutiny"  v ( m y uu1 t i n ii ) ((pos "I3%" "M8%" ) (subcat "2A" )))
( "mutinying"  v ( m y uu1 t i n i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "mutt"  n ( m uh1 t ) ((pos "K6$" ) ))
( "mutter"  v ( m uh1 t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "muttered"  v ( m uh1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "mutterer"  n ( m uh1 t @ r @ r ) ((pos "K6%" ) ))
( "mutterers"  n ( m uh1 t @ r @ z ) ((pos "Kj%" ) ))
( "muttering"  v ( m uh1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "mutters"  v ( m uh1 t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "mutton"  n ( m uh1 t n ) ((pos "L@%" ) ))
( "mutton-head"  n ( m uh1 t n - h e d ) ((pos "K6$" ) ))
( "mutton-heads"  n ( m uh1 t n - h e d z ) ((pos "Kj$" ) ))
( "mutts"  n ( m uh1 t s ) ((pos "Kj$" ) ))
( "mutual"  j ( m y uu1 ch u@ l ) ((pos "OA%" ) ))
( "mutually"  a ( m y uu1 ch u@ l ii ) ((pos "Pu%" ) ))
( "muzzier"  j ( m uh1 z i@ r ) ((pos "Or%" ) ))
( "muzziest"  j ( m uh1 z i i s t ) ((pos "Os%" ) ))
( "muzzle"  v ( m uh1 z l ) ((pos "H2%" "K6%" ) (subcat )))
( "muzzle-velocities"  n ( m uh2 z l - v i l o1 s i t i z ) ((pos "Kj$" ) ))
( "muzzle-velocity"  n ( m uh2 z l - v i l o1 s i t ii ) ((pos "K8$" ) ))
( "muzzled"  v ( m uh1 z l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "muzzles"  v ( m uh1 z l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "muzzling"  v ( m uh1 z l i ng ) ((pos "Hb%" ) (subcat )))
( "muzzy"  j ( m uh1 z ii ) ((pos "OD%" ) ))
( "my"  j ( m ai1 ) ((pos "OA*" ) ))
( "mycology"  n ( m ai1 k o1 l @ jh ii ) ((pos "L@$" ) ))
( "myelitis"  n ( m ai1 @ l ai1 t i s ) ((pos "L@$" ) ))
( "myna"  n ( m ai1 n @ ) ((pos "K6$" ) ))
( "mynah"  n ( m ai1 n @ ) ((pos "K6$" ) ))
( "mynahs"  n ( m ai1 n @ z ) ((pos "Kj$" ) ))
( "mynas"  n ( m ai1 n @ z ) ((pos "Kj$" ) ))
( "myopia"  n ( m ai1 ou1 p i@ ) ((pos "L@%" ) ))
( "myopic"  j ( m ai1 o1 p i k ) ((pos "OA%" ) ))
( "myriad"  n ( m i1 r i@ d ) ((pos "K6%" "Oq%" ) ))
( "myriads"  n ( m i1 r i@ d z ) ((pos "Kj%" ) ))
( "myrmidon"  n ( m @@1 m i d @ n ) ((pos "K6$" ) ))
( "myrmidons"  n ( m @@1 m i d @ n z ) ((pos "Kj$" ) ))
( "myrrh"  n ( m @@1 r ) ((pos "L@%" ) ))
( "myrtle"  n ( m @@1 t l ) ((pos "K6%" ) ))
( "myrtles"  n ( m @@1 t l z ) ((pos "Kj$" ) ))
( "myself"  prp ( m ai1 s e1 l f ) ((pos "Qx%" ) ))
( "mysteries"  n ( m i1 s t @ r i z ) ((pos "Mj%" ) ))
( "mysterious"  j ( m i1 s t i@1 r i@ s ) ((pos "OA%" ) ))
( "mysteriously"  a ( m i1 s t i@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "mystery"  n ( m i1 s t @ r ii ) ((pos "M8%" ) ))
( "mystic"  n ( m i1 s t i k ) ((pos "K6%" "OA%" ) ))
( "mystical"  j ( m i1 s t i k l ) ((pos "OA%" ) ))
( "mysticism"  n ( m i1 s t i s i z @ m ) ((pos "L@%" ) ))
( "mystics"  n ( m i1 s t i k s ) ((pos "Kj%" ) ))
( "mystification"  n ( m i2 s t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "mystifications"  n ( m i2 s t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "mystified"  v ( m i1 s t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "mystifies"  v ( m i1 s t i f ai z ) ((pos "Ha%" ) (subcat )))
( "mystify"  v ( m i1 s t i f ai ) ((pos "H3%" ) (subcat )))
( "mystifying"  v ( m i1 s t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "mystique"  n ( m i1 s t ii1 k ) ((pos "K6%" ) ))
( "mystiques"  n ( m i1 s t ii1 k s ) ((pos "Kj$" ) ))
( "myth"  n ( m i1 th ) ((pos "M6%" ) ))
( "mythical"  j ( m i1 th i k l ) ((pos "OA%" ) ))
( "mythological"  j ( m i2 th @ l o1 jh i k l ) ((pos "OA%" ) ))
( "mythologies"  n ( m i1 th o1 l @ jh i z ) ((pos "Mj%" ) ))
( "mythologist"  n ( m i1 th o1 l @ jh i s t ) ((pos "K6%" ) ))
( "mythologists"  n ( m i1 th o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "mythology"  n ( m i1 th o1 l @ jh ii ) ((pos "M8%" ) ))
( "myths"  n ( m i1 th s ) ((pos "Mj%" ) ))
( "myxomatosis"  n ( m i2 k s @ m @ t ou1 s i s ) ((pos "L@%" ) ))
( "n"  n ( e1 n ) ((pos "Ki$" ) ))
( "n's"  n ( e1 n z ) ((pos "Kj$" ) ))
( "n_ee"  j ( n ei1 ) ((pos "OA%" ) ))
( "n_eglig_e"  n ( n e1 g l i zh ei ) ((pos "M6%" ) ))
( "n_eglig_es"  n ( n e1 g l i zh ei z ) ((pos "Mj%" ) ))
( "na\"ive"  j ( n ai1 ii1 v ) ((pos "OA%" ) ))
( "nab"  v ( n a1 b ) ((pos "H4%" ) ))
( "nabbed"  v ( n a1 b d ) ((pos "Hc%" "Hd%" ) ))
( "nabbing"  v ( n a1 b i ng ) ((pos "Hb$" ) ))
( "nabob"  n ( n ei1 b o b ) ((pos "K6$" ) ))
( "nabobs"  n ( n ei1 b o b z ) ((pos "Kj$" ) ))
( "nabs"  v ( n a1 b z ) ((pos "Ha%" ) ))
( "nacelle"  n ( n a1 s e1 l ) ((pos "K6$" ) ))
( "nacelles"  n ( n a1 s e1 l z ) ((pos "Kj$" ) ))
( "nacre"  n ( n ei1 k @ r ) ((pos "L@$" ) ))
( "nadir"  n ( n ei1 d i@ r ) ((pos "K6%" ) ))
( "nadirs"  n ( n ei1 d i@ z ) ((pos "Kj$" ) ))
( "nag"  v ( n a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "nagged"  v ( n a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "nagger"  n ( n a1 g @ r ) ((pos "K6%" ) ))
( "naggers"  n ( n a1 g @ z ) ((pos "Kj%" ) ))
( "nagging"  v ( n a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "nags"  v ( n a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "naiad"  n ( n ai1 a d ) ((pos "K6$" ) ))
( "naiades"  n ( n ai1 a d ii z ) ((pos "Kj$" ) ))
( "naiads"  n ( n ai1 a d z ) ((pos "Kj$" ) ))
( "nail"  v ( n ei1 l ) ((pos "H0%" "K6%" ) (subcat "15A" )))
( "nail-polish"  n ( n ei1 l - p o l i sh ) ((pos "M7%" ) ))
( "nail-polishes"  n ( n ei1 l - p o l i sh i z ) ((pos "Mj$" ) ))
( "nail-scissors"  n ( n ei1 l - s i z @ z ) ((pos "Kj%" ) ))
( "nail-varnish"  n ( n ei1 l - v aa n i sh ) ((pos "M7%" ) ))
( "nail-varnishes"  n ( n ei1 l - v aa n i sh i z ) ((pos "Mj$" ) ))
( "nailbrush"  n ( n ei1 l b r uh sh ) ((pos "K7%" ) ))
( "nailbrushes"  n ( n ei1 l b r uh sh i z ) ((pos "Kj%" ) ))
( "nailed"  v ( n ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "nailfile"  n ( n ei1 l f ai l ) ((pos "K6%" ) ))
( "nailfiles"  n ( n ei1 l f ai l z ) ((pos "Kj%" ) ))
( "nailing"  v ( n ei1 l i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "nails"  v ( n ei1 l z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "nainsook"  n ( n ei1 n s u k ) ((pos "L@$" ) ))
( "naira"  n ( n ai1 r @ ) ((pos "K6$" ) ))
( "nairas"  n ( n ai1 r @ z ) ((pos "Kj$" ) ))
( "naive"  j ( n ai1 ii1 v ) ((pos "OA%" ) ))
( "naively"  a ( n ai1 ii1 v l ii ) ((pos "Pu%" ) ))
( "naivet_e"  n ( n ai1 ii1 v @ t ii ) ((pos "M6$" ) ))
( "naivet_es"  n ( n ai1 ii1 v @ t i z ) ((pos "Mj$" ) ))
( "naiveties"  n ( n ai1 ii1 v @ t i z ) ((pos "Mj%" ) ))
( "naivety"  n ( n ai1 ii1 v @ t ii ) ((pos "M8%" ) ))
( "naked"  j ( n ei1 k i d ) ((pos "OA%" ) ))
( "nakedly"  a ( n ei1 k i d l ii ) ((pos "Pu%" ) ))
( "nakedness"  n ( n ei1 k i d n @ s ) ((pos "L@%" ) ))
( "namby-pambies"  n ( n a2 m b i - p a1 m b i z ) ((pos "Kj%" ) ))
( "namby-pamby"  n ( n a2 m b i - p a1 m b ii ) ((pos "K8%" "OA%" ) ))
( "name"  v ( n ei1 m ) ((pos "H2%" "K6*" ) (subcat "6A" "14" )))
( "name-day"  n ( n ei1 m - d ei ) ((pos "K6$" ) ))
( "name-days"  n ( n ei1 m - d ei z ) ((pos "Kj$" ) ))
( "name-drop"  v ( n ei1 m - d r o p ) ((pos "I4$" ) (subcat )))
( "name-dropped"  v ( n ei1 m - d r o p t ) ((pos "Ic$" "Id$" ) (subcat )))
( "name-dropping"  v ( n ei1 m - d r o p i ng ) ((pos "Ib%" "L@%" ) (subcat )))
( "name-drops"  v ( n ei1 m - d r o p s ) ((pos "Ia$" ) (subcat )))
( "name-part"  n ( n ei1 m - p aa t ) ((pos "K6$" ) ))
( "name-parts"  n ( n ei1 m - p aa t s ) ((pos "Kj$" ) ))
( "named"  v ( n ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "nameless"  j ( n ei1 m l i s ) ((pos "OA%" ) ))
( "namely"  a ( n ei1 m l ii ) ((pos "Pu%" ) ))
( "nameplate"  n ( n ei1 m p l ei t ) ((pos "K6%" ) ))
( "nameplates"  n ( n ei1 m p l ei t s ) ((pos "Kj%" ) ))
( "names"  v ( n ei1 m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "namesake"  n ( n ei1 m s ei k ) ((pos "K6%" ) ))
( "namesakes"  n ( n ei1 m s ei k s ) ((pos "Kj%" ) ))
( "naming"  v ( n ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "nankeen"  n ( n a1 n k ii1 n ) ((pos "L@$" ) ))
( "nannies"  n ( n a1 n i z ) ((pos "Kj%" ) ))
( "nanny"  n ( n a1 n ii ) ((pos "K8%" ) ))
( "nanny-goat"  n ( n a1 n i - g ou t ) ((pos "K6%" ) ))
( "nanny-goats"  n ( n a1 n i - g ou t s ) ((pos "Kj%" ) ))
( "nap"  v ( n a1 p ) ((pos "I4%" "M6%" ) ))
( "napalm"  n ( n ei1 p aa m ) ((pos "L@%" ) ))
( "nape"  n ( n ei1 p ) ((pos "K6%" ) ))
( "napery"  n ( n ei1 p @ r ii ) ((pos "L@$" ) ))
( "napes"  n ( n ei1 p s ) ((pos "Kj$" ) ))
( "naphtha"  n ( n a1 f th @ ) ((pos "L@%" ) ))
( "naphthalene"  n ( n a1 f th @ l ii n ) ((pos "L@%" ) ))
( "napkin"  n ( n a1 p k i n ) ((pos "K6%" ) ))
( "napkin-ring"  n ( n a1 p k i n - r i ng ) ((pos "K6%" ) ))
( "napkin-rings"  n ( n a1 p k i n - r i ng z ) ((pos "Kj%" ) ))
( "napkins"  n ( n a1 p k i n z ) ((pos "Kj%" ) ))
( "napped"  v ( n a1 p t ) ((pos "Ic$" "Id$" ) ))
( "nappies"  n ( n a1 p i z ) ((pos "Kj%" ) ))
( "napping"  v ( n a1 p i ng ) ((pos "Ib%" ) ))
( "nappy"  n ( n a1 p ii ) ((pos "K8%" ) ))
( "naps"  v ( n a1 p s ) ((pos "Ia$" "Mj%" ) ))
( "narcissi"  n ( n aa1 s i1 s ai ) ((pos "Kj%" ) ))
( "narcissism"  n ( n aa1 s i s i z @ m ) ((pos "L@%" ) ))
( "narcissus"  n ( n aa1 s i1 s @ s ) ((pos "K7%" ) ))
( "narcissuses"  n ( n aa1 s i1 s @ s i z ) ((pos "Kj$" ) ))
( "narcotic"  n ( n aa1 k o1 t i k ) ((pos "K6%" "OA%" ) ))
( "narcotics"  n ( n aa1 k o1 t i k s ) ((pos "Kj%" ) ))
( "nark"  v ( n aa1 k ) ((pos "H0$" "K6%" ) ))
( "narked"  v ( n aa1 k t ) ((pos "Hc$" "Hd%" ) ))
( "narking"  v ( n aa1 k i ng ) ((pos "Hb$" ) ))
( "narks"  v ( n aa1 k s ) ((pos "Ha$" "Kj%" ) ))
( "narrate"  v ( n @1 r ei1 t ) ((pos "H2%" ) (subcat )))
( "narrated"  v ( n @1 r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "narrates"  v ( n @1 r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "narrating"  v ( n @1 r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "narration"  n ( n @1 r ei1 sh @ n ) ((pos "M6%" ) ))
( "narrations"  n ( n @1 r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "narrative"  n ( n a1 r @ t i v ) ((pos "M6%" ) ))
( "narratives"  n ( n a1 r @ t i v z ) ((pos "Mj%" ) ))
( "narrator"  n ( n @1 r ei1 t @ r ) ((pos "K6%" ) ))
( "narrators"  n ( n @1 r ei1 t @ z ) ((pos "Kj%" ) ))
( "narrow"  v ( n a1 r ou ) ((pos "J0%" "K6%" "OC%" ) (subcat "2A" )))
( "narrow-minded"  j ( n a2 r ou - m ai1 n d i d ) ((pos "OA%" ) ))
( "narrow-mindedly"  a ( n a2 r ou - m ai1 n d i d l ii ) ((pos "Pu%" ) ))
( "narrow-mindedness"  n ( n a2 r ou - m ai1 n d i d n @ s ) ((pos "L@%" ) ))
( "narrowed"  v ( n a1 r ou d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "narrower"  j ( n a1 r ou @ r ) ((pos "Or%" ) ))
( "narrowest"  j ( n a1 r ou i s t ) ((pos "Os%" ) ))
( "narrowing"  v ( n a1 r ou i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "narrowly"  a ( n a1 r ou l ii ) ((pos "Pu%" ) ))
( "narrowness"  n ( n a1 r ou n @ s ) ((pos "L@%" ) ))
( "narrows"  v ( n a1 r ou z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "narwhal"  n ( n aa1 w @ l ) ((pos "K6$" ) ))
( "narwhals"  n ( n aa1 w @ l z ) ((pos "Kj$" ) ))
( "nasal"  n ( n ei1 z l ) ((pos "K6%" "OA%" ) ))
( "nasalize"  v ( n ei1 z @ l ai z ) ((pos "H2$" ) (subcat )))
( "nasalized"  v ( n ei1 z @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nasalizes"  v ( n ei1 z @ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "nasalizing"  v ( n ei1 z @ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "nasals"  n ( n ei1 z l z ) ((pos "Kj%" ) ))
( "nascent"  j ( n a1 s n t ) ((pos "OA$" ) ))
( "nastier"  j ( n aa1 s t i@ r ) ((pos "Or%" ) ))
( "nastiest"  j ( n aa1 s t i i s t ) ((pos "Os%" ) ))
( "nastily"  a ( n aa1 s t i l ii ) ((pos "Pu%" ) ))
( "nastiness"  n ( n a1 s t i n @ s ) ((pos "L@%" ) ))
( "nasturtium"  n ( n @1 s t @@1 sh @ m ) ((pos "K6%" ) ))
( "nasturtiums"  n ( n @1 s t @@1 sh @ m z ) ((pos "Kj%" ) ))
( "nasty"  j ( n aa1 s t ii ) ((pos "OD%" ) ))
( "natal"  j ( n ei1 t l ) ((pos "OA$" ) ))
( "nation"  n ( n ei1 sh @ n ) ((pos "K6%" ) ))
( "national"  n ( n a1 sh @ n @ l ) ((pos "K6%" "OA%" ) ))
( "nationalism"  n ( n a1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "nationalist"  n ( n a1 sh @ n @ l i s t ) ((pos "K6%" "OA%" ) ))
( "nationalistic"  j ( n a2 sh @ n @ l i1 s t i k ) ((pos "OA%" ) ))
( "nationalists"  n ( n a1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "nationalities"  n ( n a2 sh @ n a1 l i t i z ) ((pos "Mj%" ) ))
( "nationality"  n ( n a2 sh @ n a1 l i t ii ) ((pos "M8%" ) ))
( "nationalization"  n ( n a2 sh @ n @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "nationalizations"  n ( n a2 sh @ n @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "nationalize"  v ( n a1 sh @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "nationalized"  v ( n a1 sh @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nationalizes"  v ( n a1 sh @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "nationalizing"  v ( n a1 sh @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "nationally"  a ( n a1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "nationals"  n ( n a1 sh @ n @ l z ) ((pos "Kj%" ) ))
( "nations"  n ( n ei1 sh @ n z ) ((pos "Kj%" ) ))
( "nationwide"  j ( n ei1 sh @ n w ai d ) ((pos "OA%" "Pu%" ) ))
( "native"  n ( n ei1 t i v ) ((pos "K6%" "OA%" ) ))
( "natives"  n ( n ei1 t i v z ) ((pos "Kj%" ) ))
( "nativities"  n ( n @1 t i1 v i t i z ) ((pos "Kj%" ) ))
( "nativity"  n ( n @1 t i1 v i t ii ) ((pos "K8%" ) ))
( "natter"  v ( n a1 t @ r ) ((pos "I0%" ) (subcat "2A" )))
( "nattered"  v ( n a1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "nattering"  v ( n a1 t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "natters"  v ( n a1 t @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "nattier"  j ( n a1 t i@ r ) ((pos "Or%" ) ))
( "nattiest"  j ( n a1 t i i s t ) ((pos "Os%" ) ))
( "nattily"  a ( n a1 t i l ii ) ((pos "Pu%" ) ))
( "natty"  j ( n a1 t ii ) ((pos "OD%" ) ))
( "natural"  n ( n a1 ch r @ l ) ((pos "K6%" "OA%" ) ))
( "naturalism"  n ( n a1 ch r @ l i z @ m ) ((pos "L@%" ) ))
( "naturalist"  n ( n a1 ch r @ l i s t ) ((pos "K6%" ) ))
( "naturalistic"  j ( n a2 ch r @ l i1 s t i k ) ((pos "OA%" ) ))
( "naturalists"  n ( n a1 ch r @ l i s t s ) ((pos "Kj%" ) ))
( "naturalization"  n ( n a2 ch r @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "naturalize"  v ( n a1 ch r @ l ai z ) ((pos "J2$" ) (subcat "2A" )))
( "naturalized"  v ( n a1 ch r @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "naturalizes"  v ( n a1 ch r @ l ai z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "naturalizing"  v ( n a1 ch r @ l ai z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "naturally"  a ( n a1 ch r @ l ii ) ((pos "Pu%" ) ))
( "naturalness"  n ( n a1 ch r @ l n i s ) ((pos "L@%" ) ))
( "naturals"  n ( n a1 ch r @ l z ) ((pos "Kj%" ) ))
( "nature"  n ( n ei1 ch @ r ) ((pos "M6%" ) ))
( "natures"  n ( n ei1 ch @ z ) ((pos "Mj%" ) ))
( "naturism"  n ( n ei1 ch @ r i z @ m ) ((pos "L@%" ) ))
( "naturist"  n ( n ei1 ch @ r i s t ) ((pos "K6%" ) ))
( "naturists"  n ( n ei1 ch @ r i s t s ) ((pos "Kj%" ) ))
( "naught"  n ( n oo1 t ) ((pos "M6%" ) ))
( "naughtier"  j ( n oo1 t i@ r ) ((pos "Or%" ) ))
( "naughtiest"  j ( n oo1 t i i s t ) ((pos "Os%" ) ))
( "naughtily"  a ( n oo1 t i l ii ) ((pos "Pu%" ) ))
( "naughtiness"  n ( n oo1 t i n @ s ) ((pos "L@%" ) ))
( "naughts"  n ( n oo1 t s ) ((pos "Mj%" ) ))
( "naughty"  j ( n oo1 t ii ) ((pos "OD%" ) ))
( "nausea"  n ( n oo1 s i@ ) ((pos "L@%" ) ))
( "nauseate"  v ( n oo1 s i ei t ) ((pos "H2$" ) (subcat )))
( "nauseated"  v ( n oo1 s i ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "nauseates"  v ( n oo1 s i ei t s ) ((pos "Ha$" ) (subcat )))
( "nauseating"  v ( n oo1 s i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "nauseous"  j ( n oo1 s i@ s ) ((pos "OA%" ) ))
( "nautch"  n ( n oo1 ch ) ((pos "K7$" ) ))
( "nautch-girl"  n ( n oo1 ch - g @@ l ) ((pos "K6$" ) ))
( "nautch-girls"  n ( n oo1 ch - g @@ l z ) ((pos "Kj$" ) ))
( "nautches"  n ( n oo1 ch i z ) ((pos "Kj$" ) ))
( "nautical"  j ( n oo1 t i k l ) ((pos "OA%" ) ))
( "nautilus"  n ( n oo1 t i l @ s ) ((pos "K7$" ) ))
( "nautiluses"  n ( n oo1 t i l @ s i z ) ((pos "Kj$" ) ))
( "naval"  j ( n ei1 v @ l ) ((pos "OA%" ) ))
( "nave"  n ( n ei1 v ) ((pos "K6%" ) ))
( "navel"  n ( n ei1 v @ l ) ((pos "K6%" ) ))
( "navels"  n ( n ei1 v @ l z ) ((pos "Kj%" ) ))
( "naves"  n ( n ei1 v z ) ((pos "Kj%" ) ))
( "navies"  n ( n ei1 v i z ) ((pos "Kj%" ) ))
( "navigability"  n ( n a2 v i g @ b i1 l i t ii ) ((pos "L@%" ) ))
( "navigable"  j ( n a1 v i g @ b l ) ((pos "OA%" ) ))
( "navigate"  v ( n a1 v i g ei t ) ((pos "J2%" ) (subcat "2A" )))
( "navigated"  v ( n a1 v i g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "navigates"  v ( n a1 v i g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "navigating"  v ( n a1 v i g ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "navigation"  n ( n a2 v i g ei1 sh @ n ) ((pos "L@%" ) ))
( "navigator"  n ( n a1 v i g ei t @ r ) ((pos "K6%" ) ))
( "navigators"  n ( n a1 v i g ei t @ z ) ((pos "Kj%" ) ))
( "navvies"  n ( n a1 v i z ) ((pos "Kj%" ) ))
( "navvy"  n ( n a1 v ii ) ((pos "K8%" ) ))
( "navy"  n ( n ei1 v ii ) ((pos "K8%" ) ))
( "nay"  a ( n ei1 ) ((pos "Pu%" ) ))
( "ne plus ultra"  n ( n ii2 - p l uh2 s - uh1 l t r @ ) ((pos "Ki$" ) ))
( "ne'er"  a ( n e@1 r ) ((pos "Pu$" ) ))
( "ne'er-do-well"  n ( n e@1 - d uu - w e l ) ((pos "K6%" ) ))
( "ne'er-do-wells"  n ( n e@1 - d uu - w e l z ) ((pos "Kj%" ) ))
( "neap"  n ( n ii1 p ) ((pos "K6$" ) ))
( "neap-tide"  n ( n ii1 p - t ai d ) ((pos "K6%" ) ))
( "neap-tides"  n ( n ii1 p - t ai d z ) ((pos "Kj%" ) ))
( "neaps"  n ( n ii1 p s ) ((pos "Kj$" ) ))
( "near"  v ( n i@ r ) ((pos "J0%" "OC*" "Pu*" "T-*" ) (subcat "2A" )))
( "near-sighted"  j ( n i@1 - s ai1 t i d ) ((pos "OA%" ) ))
( "nearby"  j ( n i@1 b ai ) ((pos "OA%" ) ))
( "neared"  v ( n i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "nearer"  j ( n i@1 r @ r ) ((pos "Or%" "Pu%" ) ))
( "nearest"  j ( n i@1 r i s t ) ((pos "Os%" "Pu%" ) ))
( "nearing"  v ( n i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "nearly"  a ( n i@1 l ii ) ((pos "Pu%" ) ))
( "nearness"  n ( n i@1 n @ s ) ((pos "L@%" ) ))
( "nears"  v ( n i@1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "nearside"  n ( n i@1 s ai d ) ((pos "K6%" ) ))
( "nearsides"  n ( n i@1 s ai d z ) ((pos "Kj$" ) ))
( "neat"  j ( n ii1 t ) ((pos "OC%" ) ))
( "neater"  j ( n ii1 t @ r ) ((pos "Or%" ) ))
( "neatest"  j ( n ii1 t i s t ) ((pos "Os%" ) ))
( "neatly"  a ( n ii1 t l ii ) ((pos "Pu%" ) ))
( "neatness"  n ( n ii1 t n @ s ) ((pos "L@%" ) ))
( "nebula"  n ( n e1 b y u l @ ) ((pos "K6%" ) ))
( "nebulae"  n ( n e1 b y u l ii ) ((pos "Kj%" ) ))
( "nebular"  j ( n e1 b y u l @ r ) ((pos "OA$" ) ))
( "nebulas"  n ( n e1 b y u l @ z ) ((pos "Kj%" ) ))
( "nebulous"  j ( n e1 b y u l @ s ) ((pos "OA%" ) ))
( "necessaries"  n ( n e1 s @ s @ r i z ) ((pos "Kj%" ) ))
( "necessarily"  a ( n e2 s @ s e1 r @ l ii ) ((pos "Pu%" ) ))
( "necessary"  n ( n e1 s @ s @ r ii ) ((pos "K8%" "OA%" ) ))
( "necessitate"  v ( n i1 s e1 s i t ei t ) ((pos "H2%" ) (subcat "6A" )))
( "necessitated"  v ( n i1 s e1 s i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "necessitates"  v ( n i1 s e1 s i t ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "necessitating"  v ( n i1 s e1 s i t ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "necessities"  n ( n i1 s e1 s i t i z ) ((pos "Mj%" ) ))
( "necessitous"  j ( n i1 s e1 s i t @ s ) ((pos "OA$" ) ))
( "necessity"  n ( n i1 s e1 s i t ii ) ((pos "M8%" ) ))
( "neck"  v ( n e1 k ) ((pos "I0%" "K6%" ) ))
( "neckband"  n ( n e1 k b a n d ) ((pos "K6%" ) ))
( "neckbands"  n ( n e1 k b a n d z ) ((pos "Kj%" ) ))
( "neckcloth"  n ( n e1 k - k l o th ) ((pos "K6$" ) ))
( "neckcloths"  n ( n e1 k - k l o th s ) ((pos "Kj$" ) ))
( "necked"  v ( n e1 k t ) ((pos "Ic%" "Id%" ) ))
( "neckerchief"  n ( n e1 k @ ch i f ) ((pos "K6$" ) ))
( "neckerchiefs"  n ( n e1 k @ ch i f s ) ((pos "Kj$" ) ))
( "necking"  v ( n e1 k i ng ) ((pos "Ib%" ) ))
( "necklace"  n ( n e1 k l @ s ) ((pos "K6%" ) ))
( "necklaces"  n ( n e1 k l @ s i z ) ((pos "Kj%" ) ))
( "necklet"  n ( n e1 k l i t ) ((pos "K6$" ) ))
( "necklets"  n ( n e1 k l i t s ) ((pos "Kj$" ) ))
( "neckline"  n ( n e1 k l ai n ) ((pos "K6%" ) ))
( "necklines"  n ( n e1 k l ai n z ) ((pos "Kj%" ) ))
( "necks"  v ( n e1 k s ) ((pos "Ia%" "Kj%" ) ))
( "necktie"  n ( n e1 k t ai ) ((pos "K6%" ) ))
( "neckties"  n ( n e1 k t ai z ) ((pos "Kj%" ) ))
( "neckwear"  n ( n e1 k w e@ r ) ((pos "L@%" ) ))
( "necromancer"  n ( n e1 k r @ m a n s @ r ) ((pos "K6$" ) ))
( "necromancers"  n ( n e1 k r @ m a n s @ z ) ((pos "Kj$" ) ))
( "necromancy"  n ( n e1 k r @ m a n s ii ) ((pos "L@$" ) ))
( "necropolis"  n ( n i1 k r o1 p @ l i s ) ((pos "K7$" ) ))
( "necropolises"  n ( n i1 k r o1 p @ l i s i z ) ((pos "Kj$" ) ))
( "nectar"  n ( n e1 k t @ r ) ((pos "L@%" ) ))
( "nectarine"  n ( n e1 k t @ r i n ) ((pos "K6%" ) ))
( "nectarines"  n ( n e1 k t @ r i n z ) ((pos "Kj%" ) ))
( "need"  v ( n ii d ) ((pos "G0*" "H0*" "M6*" ) (subcat "5" "6A" "6E" )))
( "needed"  v ( n ii1 d i d ) ((pos "Gc%" "Gd%" "Hc%" "Hd%" ) (subcat "5" "6A" "6E" )))
( "needful"  j ( n ii1 d f @ l ) ((pos "OA%" ) ))
( "needfully"  a ( n ii1 d f @ l ii ) ((pos "Pu$" ) ))
( "needier"  j ( n ii1 d i@ r ) ((pos "Or%" ) ))
( "neediest"  j ( n ii1 d i i s t ) ((pos "Os%" ) ))
( "needing"  v ( n ii1 d i ng ) ((pos "Gb%" "Hb%" ) (subcat "5" "6A" "6E" )))
( "needle"  v ( n ii1 d l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "needlecraft"  n ( n ii1 d l k r aa f t ) ((pos "L@%" ) ))
( "needled"  v ( n ii1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "needles"  v ( n ii1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "needless"  j ( n ii1 d l @ s ) ((pos "OA%" ) ))
( "needlessly"  a ( n ii1 d l @ s l ii ) ((pos "Pu%" ) ))
( "needlewoman"  n ( n ii1 d l w u m @ n ) ((pos "Ki%" ) ))
( "needlewomen"  n ( n ii1 d l w i m i n ) ((pos "Kj%" ) ))
( "needlework"  n ( n ii1 d l w @@ k ) ((pos "L@%" ) ))
( "needling"  v ( n ii1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "needn't"  v ( n ii1 d n t ) ((pos "Gg%" ) ))
( "needs"  v ( n ii d z ) ((pos "Ga%" "Ha%" "Mj%" "Pu$" ) (subcat "5" "6A" "6E" )))
( "needy"  j ( n ii1 d ii ) ((pos "OD%" ) ))
( "nefarious"  j ( n i1 f e@1 r i@ s ) ((pos "OA%" ) ))
( "nefariously"  a ( n i1 f e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "nefariousness"  n ( n i1 f e@1 r i@ s n @ s ) ((pos "L@$" ) ))
( "negate"  v ( n i1 g ei1 t ) ((pos "H2%" ) (subcat )))
( "negated"  v ( n i1 g ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "negates"  v ( n i1 g ei1 t s ) ((pos "Ha%" ) (subcat )))
( "negating"  v ( n i1 g ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "negation"  n ( n i1 g ei1 sh @ n ) ((pos "L@%" ) ))
( "negative"  v ( n e1 g @ t i v ) ((pos "H2$" "K6%" "OA%" ) (subcat )))
( "negatived"  v ( n e1 g @ t i v d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "negatively"  a ( n e1 g @ t i v l ii ) ((pos "Pu%" ) ))
( "negatives"  v ( n e1 g @ t i v z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "negativing"  v ( n e1 g @ t i v i ng ) ((pos "Hb$" ) (subcat )))
( "neglect"  v ( n i1 g l e1 k t ) ((pos "H0%" "L@%" ) (subcat "6A" "6C" )))
( "neglected"  v ( n i1 g l e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "neglectful"  j ( n i1 g l e1 k t f @ l ) ((pos "OA%" ) ))
( "neglectfully"  a ( n i1 g l e1 k t f @ l ii ) ((pos "Pu%" ) ))
( "neglectfulness"  n ( n i1 g l e1 k t f @ l n @ s ) ((pos "L@%" ) ))
( "neglecting"  v ( n i1 g l e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "neglects"  v ( n i1 g l e1 k t s ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "negligee"  n ( n e1 g l i zh ei ) ((pos "M6%" ) ))
( "negligees"  n ( n e1 g l i zh ei z ) ((pos "Mj%" ) ))
( "negligence"  n ( n e1 g l i jh @ n s ) ((pos "L@%" ) ))
( "negligent"  j ( n e1 g l i jh @ n t ) ((pos "OA%" ) ))
( "negligently"  a ( n e1 g l i jh @ n t l ii ) ((pos "Pu%" ) ))
( "negligible"  j ( n e1 g l i jh @ b l ) ((pos "OA%" ) ))
( "negotiable"  j ( n i1 g ou1 sh i@ b l ) ((pos "OA%" ) ))
( "negotiate"  v ( n i1 g ou1 sh i ei t ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "negotiated"  v ( n i1 g ou1 sh i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "negotiates"  v ( n i1 g ou1 sh i ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "negotiating"  v ( n i1 g ou1 sh i ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "negotiation"  n ( n i1 g ou2 sh i ei1 sh @ n ) ((pos "M6%" ) ))
( "negotiations"  n ( n i1 g ou2 sh i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "negotiator"  n ( n i1 g ou1 sh i ei t @ r ) ((pos "K6%" ) ))
( "negotiators"  n ( n i1 g ou1 sh i ei t @ z ) ((pos "Kj%" ) ))
( "negus"  n ( n ii1 g @ s ) ((pos "L@$" ) ))
( "neigh"  v ( n ei1 ) ((pos "I0%" "K6%" ) ))
( "neighbour"  v ( n ei1 b @ r ) ((pos "J0$" "K6%" ) (subcat "3A" )))
( "neighboured"  v ( n ei1 b @ d ) ((pos "Jc$" "Jd$" ) (subcat "3A" )))
( "neighbourhood"  n ( n ei1 b @ h u d ) ((pos "K6%" ) ))
( "neighbourhoods"  n ( n ei1 b @ h u d z ) ((pos "Kj%" ) ))
( "neighbouring"  v ( n ei1 b @ r i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "neighbourliness"  n ( n ei1 b @ l i n @ s ) ((pos "L@%" ) ))
( "neighbourly"  j ( n ei1 b @ l ii ) ((pos "OA%" ) ))
( "neighbours"  v ( n ei1 b @ z ) ((pos "Ja$" "Kj%" ) (subcat "3A" )))
( "neighed"  v ( n ei1 d ) ((pos "Ic%" "Id%" ) ))
( "neighing"  v ( n ei1 i ng ) ((pos "Ib%" ) ))
( "neighs"  v ( n ei1 z ) ((pos "Ia%" "Kj%" ) ))
( "neither"  j ( n ai1 dh @ r ) ((pos "OA%" "Pu%" "Qx%" "V-%" ) ))
( "nem con"  a ( n e2 m - k o1 n ) ((pos "Pu$" ) ))
( "nemeses"  n ( n e1 m @ s ii z ) ((pos "Kj$" ) ))
( "nemesis"  n ( n e1 m @ s i s ) ((pos "Ki%" ) ))
( "neo-"  j ( n ii1 ou - ) ((pos "U-%" ) ))
( "neocolonialism"  n ( n ii2 ou k @ l ou1 n i@ l i z @ m ) ((pos "L@%" ) ))
( "neolithic"  j ( n ii2 @ l i1 th i k ) ((pos "OA%" ) ))
( "neologism"  n ( n ii1 o1 l @ jh i z @ m ) ((pos "M6%" ) ))
( "neologisms"  n ( n ii1 o1 l @ jh i z @ m z ) ((pos "Mj%" ) ))
( "neon"  n ( n ii1 o n ) ((pos "L@%" ) ))
( "neophyte"  n ( n ii1 @ f ai t ) ((pos "K6$" ) ))
( "neophytes"  n ( n ii1 @ f ai t s ) ((pos "Kj$" ) ))
( "neoplasm"  n ( n ii1 ou p l a z @ m ) ((pos "K6$" ) ))
( "neoplasms"  n ( n ii1 ou p l a z @ m z ) ((pos "Kj$" ) ))
( "nephew"  n ( n e1 v y uu ) ((pos "K6%" ) ))
( "nephews"  n ( n e1 v y uu z ) ((pos "Kj%" ) ))
( "nephritis"  n ( n i1 f r ai1 t i s ) ((pos "L@$" ) ))
( "nepotism"  n ( n e1 p @ t i z @ m ) ((pos "L@%" ) ))
( "nereid"  n ( n i@1 r i i d ) ((pos "K6$" ) ))
( "nereids"  n ( n i@1 r i i d z ) ((pos "Kj$" ) ))
( "nerve"  v ( n @@1 v ) ((pos "H2%" "M6%" ) (subcat "6A" "14" )))
( "nerve-cell"  n ( n @@1 v - s e l ) ((pos "K6%" ) ))
( "nerve-cells"  n ( n @@1 v - s e l z ) ((pos "Kj%" ) ))
( "nerve-centre"  n ( n @@1 v - s e n t @ r ) ((pos "K6%" ) ))
( "nerve-centres"  n ( n @@1 v - s e n t @ z ) ((pos "Kj%" ) ))
( "nerve-racking"  j ( n @@1 v - r a k i ng ) ((pos "OA%" ) ))
( "nerved"  v ( n @@1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "nerveless"  j ( n @@1 v l @ s ) ((pos "OA%" ) ))
( "nervelessly"  a ( n @@1 v l @ s l ii ) ((pos "Pu%" ) ))
( "nerves"  v ( n @@1 v z ) ((pos "Ha$" "Mj%" ) (subcat "6A" "14" )))
( "nerving"  v ( n @@1 v i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "nervous"  j ( n @@1 v @ s ) ((pos "OA%" ) ))
( "nervously"  a ( n @@1 v @ s l ii ) ((pos "Pu%" ) ))
( "nervousness"  n ( n @@1 v @ s n @ s ) ((pos "L@%" ) ))
( "nervy"  j ( n @@1 v ii ) ((pos "OA%" ) ))
( "nescience"  n ( n e1 s i@ n s ) ((pos "L@$" ) ))
( "nescient"  j ( n e1 s i@ n t ) ((pos "OA$" ) ))
( "ness"  n ( n e1 s ) ((pos "K7$" ) ))
( "nesses"  n ( n e1 s i z ) ((pos "Kj$" ) ))
( "nest"  v ( n e1 s t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "nest-egg"  n ( n e1 s t - e g ) ((pos "K6%" ) ))
( "nest-eggs"  n ( n e1 s t - e g z ) ((pos "Kj%" ) ))
( "nested"  v ( n e1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "nesting"  v ( n e1 s t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "nestle"  v ( n e1 s l ) ((pos "J2%" ) (subcat "2C" )))
( "nestled"  v ( n e1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "nestles"  v ( n e1 s l z ) ((pos "Ja%" ) (subcat "2C" )))
( "nestling"  v ( n e1 s l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "nestling"  n ( n e1 s t l i ng ) ((pos "K6%" ) ))
( "nestlings"  n ( n e1 s t l i ng z ) ((pos "Kj%" ) ))
( "nests"  v ( n e1 s t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "net"  v ( n e1 t ) ((pos "H4%" "M6%" "OA%" ) (subcat )))
( "netball"  n ( n e1 t b oo l ) ((pos "L@%" ) ))
( "nether"  j ( n e1 dh @ r ) ((pos "OA%" ) ))
( "nethermost"  j ( n e1 dh @ m ou s t ) ((pos "OA%" ) ))
( "nets"  v ( n e1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "nett"  v ( n e1 t ) ((pos "H0$" "OA$" ) (subcat )))
( "netted"  v ( n e1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "netting"  v ( n e1 t i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "nettle"  v ( n e1 t l ) ((pos "H2%" "K6%" ) (subcat )))
( "nettled"  v ( n e1 t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nettlerash"  n ( n e1 t l r a sh ) ((pos "L@$" ) ))
( "nettles"  v ( n e1 t l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "nettling"  v ( n e1 t l i ng ) ((pos "Hb%" ) (subcat )))
( "netts"  v ( n e1 t s ) ((pos "Ha$" ) (subcat )))
( "network"  n ( n e1 t w @@ k ) ((pos "K6%" ) ))
( "networks"  n ( n e1 t w @@ k s ) ((pos "Kj%" ) ))
( "neural"  j ( n y u@1 r @ l ) ((pos "OA%" ) ))
( "neuralgia"  n ( n y u@1 r a1 l jh @ ) ((pos "L@%" ) ))
( "neuralgic"  j ( n y uu1 r a1 l jh i k ) ((pos "OA$" ) ))
( "neurasthenia"  n ( n y u@2 r @ s th ii1 n i@ ) ((pos "L@%" ) ))
( "neurasthenic"  n ( n y uu2 r @ s th e1 n i k ) ((pos "K6$" "OA$" ) ))
( "neurasthenics"  n ( n y uu2 r @ s th e1 n i k s ) ((pos "Kj$" ) ))
( "neuritis"  n ( n y u@1 r ai1 t i s ) ((pos "L@$" ) ))
( "neurologist"  n ( n y u@1 r o1 l @ jh i s t ) ((pos "K6%" ) ))
( "neurologists"  n ( n y u@1 r o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "neurology"  n ( n y u@1 r o1 l @ jh ii ) ((pos "L@%" ) ))
( "neuroses"  n ( n y u@1 r ou1 s ii z ) ((pos "Kj%" ) ))
( "neurosis"  n ( n y u@1 r ou1 s i s ) ((pos "Ki%" ) ))
( "neurotic"  n ( n y u@1 r o1 t i k ) ((pos "K6%" "OA%" ) ))
( "neurotically"  a ( n y u@1 r o1 t i k l ii ) ((pos "Pu%" ) ))
( "neurotics"  n ( n y u@1 r o1 t i k s ) ((pos "Kj%" ) ))
( "neuter"  v ( n y uu1 t @ r ) ((pos "H0$" "K6%" "OA%" ) ))
( "neutered"  v ( n y uu1 t @ d ) ((pos "Hc$" "Hd%" ) ))
( "neutering"  v ( n y uu1 t @ r i ng ) ((pos "Hb$" ) ))
( "neuters"  v ( n y uu1 t @ z ) ((pos "Ha$" "Kj$" ) ))
( "neutral"  n ( n y uu1 t r @ l ) ((pos "K6%" "OA%" ) ))
( "neutrality"  n ( n y uu1 t r a1 l i t ii ) ((pos "L@%" ) ))
( "neutralization"  n ( n y uu2 t r @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "neutralizations"  n ( n y uu2 t r @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "neutralize"  v ( n y uu1 t r @ l ai z ) ((pos "H2%" ) (subcat )))
( "neutralized"  v ( n y uu1 t r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "neutralizes"  v ( n y uu1 t r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "neutralizing"  v ( n y uu1 t r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "neutrals"  n ( n y uu1 t r @ l z ) ((pos "Kj%" ) ))
( "neutron"  n ( n y uu1 t r o n ) ((pos "K6%" ) ))
( "neutrons"  n ( n y uu1 t r o n z ) ((pos "Kj%" ) ))
( "never"  a ( n e1 v @ r ) ((pos "Pu*" ) ))
( "nevermore"  a ( n e2 v @ m oo1 r ) ((pos "Pu%" ) ))
( "nevertheless"  a ( n e2 v @ dh @ l e1 s ) ((pos "Pu%" "V-%" ) ))
( "new"  j ( n y uu1 ) ((pos "OC*" "Pu*" ) ))
( "newcomer"  n ( n y uu1 k uh m @ r ) ((pos "K6%" ) ))
( "newcomers"  n ( n y uu1 k uh m @ z ) ((pos "Kj%" ) ))
( "newel"  n ( n y uu1 @ l ) ((pos "K6$" ) ))
( "newels"  n ( n y uu1 @ l z ) ((pos "Kj$" ) ))
( "newer"  j ( n y uu1 @ r ) ((pos "Or%" ) ))
( "newest"  j ( n y uu1 i s t ) ((pos "Os%" ) ))
( "newfangled"  j ( n y uu2 f a1 ng g l d ) ((pos "OA%" ) ))
( "newly"  a ( n y uu1 l ii ) ((pos "Pu%" ) ))
( "newlywed"  n ( n y uu1 l i w e d ) ((pos "K6%" ) ))
( "newlyweds"  n ( n y uu1 l i w e d z ) ((pos "Kj%" ) ))
( "newness"  n ( n y uu1 n @ s ) ((pos "L@%" ) ))
( "news"  n ( n y uu1 z ) ((pos "L@%" ) ))
( "newsagent"  n ( n y uu1 z ei jh @ n t ) ((pos "K6%" ) ))
( "newsagents"  n ( n y uu1 z ei jh @ n t s ) ((pos "Kj%" ) ))
( "newsboy"  n ( n y uu1 z b oi ) ((pos "K6%" ) ))
( "newsboys"  n ( n y uu1 z b oi z ) ((pos "Kj%" ) ))
( "newscast"  n ( n y uu1 z k aa s t ) ((pos "K6%" ) ))
( "newscaster"  n ( n y uu1 z k aa s t @ r ) ((pos "K6%" ) ))
( "newscasters"  n ( n y uu1 z k aa s t @ z ) ((pos "Kj%" ) ))
( "newscasts"  n ( n y uu1 z k aa s t s ) ((pos "Kj%" ) ))
( "newsdealer"  n ( n y uu1 z d ii l @ r ) ((pos "K6%" ) ))
( "newsdealers"  n ( n y uu1 z d ii l @ z ) ((pos "Kj%" ) ))
( "newsflash"  n ( n y uu1 z f l a sh ) ((pos "K7%" ) ))
( "newsflashes"  n ( n y uu1 z f l a sh i z ) ((pos "Kj%" ) ))
( "newsier"  j ( n y uu1 z i@ r ) ((pos "Or$" ) ))
( "newsiest"  j ( n y uu1 z i i s t ) ((pos "Os$" ) ))
( "newsless"  j ( n y uu1 z l @ s ) ((pos "OA$" ) ))
( "newsletter"  n ( n y uu1 z l e t @ r ) ((pos "K6%" ) ))
( "newsletters"  n ( n y uu1 z l e t @ z ) ((pos "Kj%" ) ))
( "newsmonger"  n ( n y uu1 z m uh ng g @ r ) ((pos "K6$" ) ))
( "newsmongers"  n ( n y uu1 z m uh ng g @ z ) ((pos "Kj$" ) ))
( "newspaper"  n ( n y uu1 s p ei p @ r ) ((pos "K6%" ) ))
( "newspaperman"  n ( n y uu1 s p ei p @ m a n ) ((pos "Ki%" ) ))
( "newspapermen"  n ( n y uu1 s p ei p @ m e n ) ((pos "Kj%" ) ))
( "newspapers"  n ( n y uu1 s p ei p @ z ) ((pos "Kj%" ) ))
( "newsprint"  n ( n y uu1 z p r i n t ) ((pos "L@%" ) ))
( "newsreel"  n ( n y uu1 z r ii l ) ((pos "K6%" ) ))
( "newsreels"  n ( n y uu1 z r ii l z ) ((pos "Kj%" ) ))
( "newsroom"  n ( n y uu1 z r uu m ) ((pos "K6%" ) ))
( "newsrooms"  n ( n y uu1 z r uu m z ) ((pos "Kj%" ) ))
( "newssheet"  n ( n y uu1 z - sh ii t ) ((pos "K6%" ) ))
( "newssheets"  n ( n y uu1 z - sh ii t s ) ((pos "Kj%" ) ))
( "newsstand"  n ( n y uu1 z - s t a n d ) ((pos "K6%" ) ))
( "newsstands"  n ( n y uu1 z - s t a n d z ) ((pos "Kj%" ) ))
( "newsvendor"  n ( n y uu1 z v e n d @ r ) ((pos "K6%" ) ))
( "newsvendors"  n ( n y uu1 z v e n d @ z ) ((pos "Kj%" ) ))
( "newsworthy"  j ( n y uu1 z w @@ dh ii ) ((pos "OA%" ) ))
( "newsy"  j ( n y uu1 z ii ) ((pos "OD%" ) ))
( "newt"  n ( n y uu1 t ) ((pos "K6%" ) ))
( "newts"  n ( n y uu1 t s ) ((pos "Kj%" ) ))
( "next"  n ( n e k s t ) ((pos "K9%" "OA*" "Pu*" "T-*" ) ))
( "nexus"  n ( n e1 k s @ s ) ((pos "K7%" ) ))
( "nexuses"  n ( n e1 k s @ s i z ) ((pos "Kj$" ) ))
( "nib"  n ( n i1 b ) ((pos "K6%" ) ))
( "nibble"  v ( n i1 b l ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" )))
( "nibbled"  v ( n i1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "nibbles"  v ( n i1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "nibbling"  v ( n i1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "nibs"  n ( n i1 b z ) ((pos "Kj%" ) ))
( "nice"  j ( n ai1 s ) ((pos "OB%" ) ))
( "nicely"  a ( n ai1 s l ii ) ((pos "Pu%" ) ))
( "niceness"  n ( n ai1 s n @ s ) ((pos "L@%" ) ))
( "nicer"  j ( n ai1 s @ r ) ((pos "Or%" ) ))
( "nicest"  j ( n ai1 s i s t ) ((pos "Os%" ) ))
( "niceties"  n ( n ai1 s @ t i z ) ((pos "Mj%" ) ))
( "nicety"  n ( n ai1 s @ t ii ) ((pos "M8$" ) ))
( "niche"  n ( n i1 ch ) ((pos "K6%" ) ))
( "niches"  n ( n i1 ch i z ) ((pos "Kj%" ) ))
( "nick"  v ( n i1 k ) ((pos "H0%" "K6%" ) ))
( "nicked"  v ( n i1 k t ) ((pos "Hc%" "Hd%" ) ))
( "nickel"  v ( n i1 k l ) ((pos "H4%" "M6%" ) ))
( "nickelled"  v ( n i1 k l d ) ((pos "Hc$" "Hd$" ) ))
( "nickelling"  v ( n i1 k @ l i ng ) ((pos "Hb$" ) ))
( "nickels"  v ( n i1 k l z ) ((pos "Ha$" "Mj$" ) ))
( "nicking"  v ( n i1 k i ng ) ((pos "Hb%" ) ))
( "nicknack"  n ( n i1 k n a k ) ((pos "K6%" ) ))
( "nicknacks"  n ( n i1 k n a k s ) ((pos "Kj%" ) ))
( "nickname"  v ( n i1 k n ei m ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "nicknamed"  v ( n i1 k n ei m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "nicknames"  v ( n i1 k n ei m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "nicknaming"  v ( n i1 k n ei m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "nicks"  v ( n i1 k s ) ((pos "Ha%" "Kj%" ) ))
( "nicotine"  n ( n i1 k @ t ii n ) ((pos "L@%" ) ))
( "niece"  n ( n ii1 s ) ((pos "K6%" ) ))
( "nieces"  n ( n ii1 s i z ) ((pos "Kj%" ) ))
( "niff"  n ( n i1 f ) ((pos "K6$" ) ))
( "niffier"  j ( n i1 f i@ r ) ((pos "Or$" ) ))
( "niffiest"  j ( n i1 f i i s t ) ((pos "Os$" ) ))
( "niffs"  n ( n i1 f s ) ((pos "Kj$" ) ))
( "niffy"  j ( n i1 f ii ) ((pos "OD$" ) ))
( "niftier"  j ( n i1 f t i@ r ) ((pos "Or%" ) ))
( "niftiest"  j ( n i1 f t i i s t ) ((pos "Os%" ) ))
( "nifty"  j ( n i1 f t ii ) ((pos "OD%" ) ))
( "niggard"  n ( n i1 g @ d ) ((pos "K6$" ) ))
( "niggardliness"  n ( n i1 g @ d l i n @ s ) ((pos "L@%" ) ))
( "niggardly"  j ( n i1 g @ d l ii ) ((pos "OA%" ) ))
( "niggards"  n ( n i1 g @ d z ) ((pos "Kj$" ) ))
( "nigger"  n ( n i1 g @ r ) ((pos "K6%" ) ))
( "niggers"  n ( n i1 g @ z ) ((pos "Kj%" ) ))
( "niggle"  v ( n i1 g l ) ((pos "I2%" ) ))
( "niggled"  v ( n i1 g l d ) ((pos "Ic%" "Id%" ) ))
( "niggles"  v ( n i1 g l z ) ((pos "Ia%" ) ))
( "niggling"  v ( n i1 g l i ng ) ((pos "Ib%" "OA%" ) ))
( "nigh"  a ( n ai ) ((pos "Pu$" "T-$" ) ))
( "nigher"  a ( n ai1 @ r ) ((pos "Pu$" "T-$" ) ))
( "nighest"  a ( n ai1 i s t ) ((pos "Pu$" "T-$" ) ))
( "night"  n ( n ai1 t ) ((pos "M6*" ) ))
( "night-bell"  n ( n ai1 t - b e l ) ((pos "K6$" ) ))
( "night-bells"  n ( n ai1 t - b e l z ) ((pos "Kj$" ) ))
( "night-bird"  n ( n ai1 t - b @@ d ) ((pos "K6%" ) ))
( "night-birds"  n ( n ai1 t - b @@ d z ) ((pos "Kj%" ) ))
( "night-light"  n ( n ai1 t - l ai t ) ((pos "K6%" ) ))
( "night-lights"  n ( n ai1 t - l ai t s ) ((pos "Kj%" ) ))
( "night-line"  n ( n ai1 t - l ai n ) ((pos "K6$" ) ))
( "night-lines"  n ( n ai1 t - l ai n z ) ((pos "Kj$" ) ))
( "night-porter"  n ( n ai1 t - p oo1 t @ r ) ((pos "K6%" ) ))
( "night-porters"  n ( n ai1 t - p oo1 t @ z ) ((pos "Kj%" ) ))
( "night-soil"  n ( n ai1 t - s oi l ) ((pos "L@%" ) ))
( "night-stop"  n ( n ai1 t - s t o p ) ((pos "K6$" ) ))
( "night-stops"  n ( n ai1 t - s t o p s ) ((pos "Kj$" ) ))
( "night-time"  n ( n ai1 t - ai m ) ((pos "L@%" ) ))
( "night-watch"  n ( n ai1 t - w o ch ) ((pos "K7%" ) ))
( "night-watches"  n ( n ai1 t - w o ch i z ) ((pos "Kj%" ) ))
( "night-watchman"  n ( n ai1 t - w o1 ch m @ n ) ((pos "Ki%" ) ))
( "night-watchmen"  n ( n ai1 t - w o1 ch m @ n ) ((pos "Kj%" ) ))
( "nightcap"  n ( n ai1 t k a p ) ((pos "K6%" ) ))
( "nightcaps"  n ( n ai1 t k a p s ) ((pos "Kj%" ) ))
( "nightclub"  n ( n ai1 t k l uh b ) ((pos "K6%" ) ))
( "nightclubs"  n ( n ai1 t k l uh b z ) ((pos "Kj%" ) ))
( "nightdress"  n ( n ai1 t d r e s ) ((pos "K7%" ) ))
( "nightdresses"  n ( n ai1 t d r e s i z ) ((pos "Kj%" ) ))
( "nightfall"  n ( n ai1 t f oo l ) ((pos "L@%" ) ))
( "nightgown"  n ( n ai1 t g au n ) ((pos "K6%" ) ))
( "nightgowns"  n ( n ai1 t g au n z ) ((pos "Kj%" ) ))
( "nightie"  n ( n ai1 t ii ) ((pos "K6%" ) ))
( "nighties"  n ( n ai1 t i z ) ((pos "Kj%" ) ))
( "nightingale"  n ( n ai1 t i ng g ei l ) ((pos "K6%" ) ))
( "nightingales"  n ( n ai1 t i ng g ei l z ) ((pos "Kj%" ) ))
( "nightjar"  n ( n ai1 t jh aa r ) ((pos "K6%" ) ))
( "nightjars"  n ( n ai1 t jh aa z ) ((pos "Kj%" ) ))
( "nightlong"  j ( n ai1 t l o ng ) ((pos "OA%" ) ))
( "nightly"  j ( n ai1 t l ii ) ((pos "OA%" "Pu%" ) ))
( "nightmare"  n ( n ai1 t m e@ r ) ((pos "K6%" ) ))
( "nightmares"  n ( n ai1 t m e@ z ) ((pos "Kj%" ) ))
( "nightmarish"  j ( n ai1 t m e@ r i sh ) ((pos "OA%" ) ))
( "nights"  n ( n ai1 t s ) ((pos "Mj%" ) ))
( "nightshade"  n ( n ai1 ch ei d ) ((pos "L@%" ) ))
( "nightshirt"  n ( n ai1 ch @@ t ) ((pos "K6%" ) ))
( "nightshirts"  n ( n ai1 ch @@ t s ) ((pos "Kj%" ) ))
( "nightwork"  n ( n ai1 t w @@ k ) ((pos "L@%" ) ))
( "nihilism"  n ( n ai1 i l i z @ m ) ((pos "L@%" ) ))
( "nihilist"  n ( n ai1 i l i s t ) ((pos "K6$" ) ))
( "nihilistic"  j ( n ai2 i l i1 s t i k ) ((pos "OA%" ) ))
( "nihilists"  n ( n ai1 i l i s t s ) ((pos "Kj$" ) ))
( "nil"  n ( n i1 l ) ((pos "L@%" ) ))
( "nimble"  j ( n i1 m b l ) ((pos "OB%" ) ))
( "nimbleness"  n ( n i1 m b l n @ s ) ((pos "L@%" ) ))
( "nimbler"  j ( n i1 m b l @ r ) ((pos "Or%" ) ))
( "nimblest"  j ( n i1 m b l i s t ) ((pos "Os%" ) ))
( "nimbly"  a ( n i1 m b l ii ) ((pos "Pu%" ) ))
( "nimbus"  n ( n i1 m b @ s ) ((pos "K7$" ) ))
( "nimbuses"  n ( n i1 m b @ s i z ) ((pos "Kj$" ) ))
( "niminy-piminy"  j ( n i2 m i n i - p i1 m i n ii ) ((pos "OA$" ) ))
( "nincompoop"  n ( n i1 ng k @ m p uu p ) ((pos "K6%" ) ))
( "nincompoops"  n ( n i1 ng k @ m p uu p s ) ((pos "Kj%" ) ))
( "nine"  n ( n ai1 n ) ((pos "K6%" "OA%" ) ))
( "ninefold"  j ( n ai1 n f ou l d ) ((pos "OA%" "Pu%" ) ))
( "ninepence"  n ( n ai1 n p @ n s ) ((pos "K6%" ) ))
( "ninepences"  n ( n ai1 n p @ n s i z ) ((pos "Kj%" ) ))
( "ninepenny"  j ( n ai1 n p @ n ii ) ((pos "OA%" ) ))
( "ninepins"  n ( n ai1 n p i n z ) ((pos "Lk%" ) ))
( "nines"  n ( n ai1 n z ) ((pos "Kj%" ) ))
( "nineteen"  n ( n ai2 n t ii1 n ) ((pos "K6%" "OA%" ) ))
( "nineteens"  n ( n ai2 n t ii1 n z ) ((pos "Kj%" ) ))
( "nineteenth"  n ( n ai2 n t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "nineteenths"  n ( n ai2 n t ii1 n th s ) ((pos "Kj%" ) ))
( "nineties"  n ( n ai1 n t i z ) ((pos "Kj%" ) ))
( "ninetieth"  n ( n ai1 n t i@ th ) ((pos "K6%" "OA%" ) ))
( "ninetieths"  n ( n ai1 n t i@ th s ) ((pos "Kj%" ) ))
( "ninety"  n ( n ai1 n t ii ) ((pos "K8%" "OA%" ) ))
( "ninnies"  n ( n i1 n i z ) ((pos "Kj%" ) ))
( "ninny"  n ( n i1 n ii ) ((pos "K8%" ) ))
( "ninth"  n ( n ai1 n th ) ((pos "K6%" "OA%" ) ))
( "ninths"  n ( n ai1 n th s ) ((pos "Kj%" ) ))
( "nip"  v ( n i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "nipped"  v ( n i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "nipper"  n ( n i1 p @ r ) ((pos "K6%" ) ))
( "nippers"  n ( n i1 p @ z ) ((pos "Kj%" ) ))
( "nippier"  j ( n i1 p i@ r ) ((pos "Or$" ) ))
( "nippiest"  j ( n i1 p i i s t ) ((pos "Os$" ) ))
( "nipping"  v ( n i1 p i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "nipple"  n ( n i1 p l ) ((pos "K6%" ) ))
( "nipples"  n ( n i1 p l z ) ((pos "Kj%" ) ))
( "nippy"  j ( n i1 p ii ) ((pos "OD%" ) ))
( "nips"  v ( n i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "nirvana"  n ( n i@1 v aa1 n @ ) ((pos "K6$" ) ))
( "nirvanas"  n ( n i@1 v aa1 n @ z ) ((pos "Kj$" ) ))
( "nisi"  j ( n ai1 s ai ) ((pos "OA$" "V-$" ) ))
( "nit"  n ( n i1 t ) ((pos "K6%" ) ))
( "nitrate"  n ( n ai1 t r ei t ) ((pos "K6%" ) ))
( "nitrates"  n ( n ai1 t r ei t s ) ((pos "Kj%" ) ))
( "nitre"  n ( n ai1 t @ r ) ((pos "L@$" ) ))
( "nitric"  j ( n ai1 t r i k ) ((pos "OA%" ) ))
( "nitrochalk"  n ( n ai2 t r ou ch oo1 k ) ((pos "L@$" ) ))
( "nitrogen"  n ( n ai1 t r @ jh @ n ) ((pos "L@%" ) ))
( "nitroglycerin"  n ( n ai2 t r ou g l i1 s @ r ii n ) ((pos "L@%" ) ))
( "nitroglycerine"  n ( n ai2 t r ou g l i1 s @ r ii n ) ((pos "L@%" ) ))
( "nitrous"  j ( n ai1 t r @ s ) ((pos "OA%" ) ))
( "nits"  n ( n i1 t s ) ((pos "Kj%" ) ))
( "nitty-gritty"  n ( n i2 t i - g r i1 t ii ) ((pos "L@%" ) ))
( "nitwit"  n ( n i1 t w i t ) ((pos "K6%" ) ))
( "nitwits"  n ( n i1 t w i t s ) ((pos "Kj%" ) ))
( "nitwitted"  j ( n i2 t w i1 t i d ) ((pos "OA%" ) ))
( "nix"  n ( n i1 k s ) ((pos "L@$" ) ))
( "no"  n ( n uh1 m b @ r ) ((pos "Y>%" ) ))
( "no"  n ( n ou1 ) ((pos "K6%" "OA*" "Pu*" "W-*" ) ))
( "no one"  prp ( n ou1 - w uh n ) ((pos "Qx%" ) ))
( "no-ball"  n ( n ou1 - b oo l ) ((pos "K6%" ) ))
( "no-balls"  n ( n ou1 - b oo l z ) ((pos "Kj%" ) ))
( "no-go"  j ( n ou1 - g ou ) ((pos "Oq%" ) ))
( "no-man's-land"  n ( n ou1 - m a n z - l a n d ) ((pos "Ki%" ) ))
( "no-one"  prp ( n ou1 - w uh n ) ((pos "Qx%" ) ))
( "nob"  n ( n o1 b ) ((pos "K6$" ) ))
( "nobble"  v ( n o1 b l ) ((pos "H2%" ) (subcat )))
( "nobbled"  v ( n o1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nobbles"  v ( n o1 b l z ) ((pos "Ha%" ) (subcat )))
( "nobbling"  v ( n o1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "nobility"  n ( n ou1 b i1 l i t ii ) ((pos "L@%" ) ))
( "noble"  n ( n ou1 b l ) ((pos "K6%" "OB%" ) ))
( "noble-mindedness"  n ( n ou2 b l - m ai1 n d i d n @ s ) ((pos "L@%" ) ))
( "nobleman"  n ( n ou1 b l m @ n ) ((pos "Ki%" ) ))
( "noblemen"  n ( n ou1 b l m @ n ) ((pos "Kj%" ) ))
( "nobler"  j ( n ou1 b l @ r ) ((pos "Or%" ) ))
( "nobles"  n ( n ou1 b l z ) ((pos "Kj%" ) ))
( "noblesse"  n ( n ou1 b l e1 s ) ((pos "Ki%" ) ))
( "noblesse oblige"  n ( n ou1 b l e2 s - @ b l ii1 zh ) ((pos "L@%" ) ))
( "noblest"  j ( n ou1 b l i s t ) ((pos "Os%" ) ))
( "nobly"  a ( n ou1 b l ii ) ((pos "Pu%" ) ))
( "nobodies"  n ( n ou1 b @ d i z ) ((pos "Kj%" ) ))
( "nobody"  n ( n ou1 b @ d ii ) ((pos "K8%" "Qx%" ) ))
( "nobs"  n ( n o1 b z ) ((pos "Kj%" ) ))
( "noctambulist"  n ( n o1 k t a1 m b y u l i s t ) ((pos "K6$" ) ))
( "noctambulists"  n ( n o1 k t a1 m b y u l i s t s ) ((pos "Kj$" ) ))
( "nocturnal"  j ( n o1 k t @@1 n l ) ((pos "OA%" ) ))
( "nocturne"  n ( n o1 k t @@ n ) ((pos "K6%" ) ))
( "nocturnes"  n ( n o1 k t @@ n z ) ((pos "Kj%" ) ))
( "nod"  v ( n o1 d ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" )))
( "nodded"  v ( n o1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" )))
( "nodding"  v ( n o1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" )))
( "noddle"  n ( n o1 d l ) ((pos "K6$" ) ))
( "noddles"  n ( n o1 d l z ) ((pos "Kj$" ) ))
( "node"  n ( n ou1 d ) ((pos "K6$" ) ))
( "nodes"  n ( n ou1 d z ) ((pos "Kj$" ) ))
( "nods"  v ( n o1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" )))
( "nodular"  j ( n o1 d y u l @ r ) ((pos "OA$" ) ))
( "nodulated"  j ( n o1 d y u l ei t i d ) ((pos "OA$" ) ))
( "nodule"  n ( n o1 d y uu l ) ((pos "K6%" ) ))
( "nodules"  n ( n o1 d y uu l z ) ((pos "Kj%" ) ))
( "noggin"  n ( n o1 g i n ) ((pos "K6$" ) ))
( "noggins"  n ( n o1 g i n z ) ((pos "Kj$" ) ))
( "nohow"  a ( n ou1 h au ) ((pos "Pu$" ) ))
( "noise"  v ( n oi1 z ) ((pos "H2%" "M6%" ) ))
( "noised"  v ( n oi1 z d ) ((pos "Hc%" "Hd%" ) ))
( "noiseless"  j ( n oi1 z l @ s ) ((pos "OA%" ) ))
( "noiselessly"  a ( n oi1 z l @ s l ii ) ((pos "Pu%" ) ))
( "noiselessness"  n ( n oi1 z l @ s n @ s ) ((pos "L@%" ) ))
( "noises"  v ( n oi1 z i z ) ((pos "Ha%" "Mj%" ) ))
( "noisier"  j ( n oi1 z i@ r ) ((pos "Or%" ) ))
( "noisiest"  j ( n oi1 z i i s t ) ((pos "Os%" ) ))
( "noisily"  a ( n oi1 z i l ii ) ((pos "Pu%" ) ))
( "noisiness"  n ( n oi1 z i n @ s ) ((pos "L@%" ) ))
( "noising"  v ( n oi1 z i ng ) ((pos "Hb$" ) ))
( "noisome"  j ( n oi1 s @ m ) ((pos "OA$" ) ))
( "noisy"  j ( n oi1 z ii ) ((pos "OD%" ) ))
( "nom de plume"  n ( n o2 m - d @ - p l uu1 m ) ((pos "Ki%" ) ))
( "nomad"  n ( n ou1 m a d ) ((pos "K6%" ) ))
( "nomadic"  j ( n ou1 m a1 d i k ) ((pos "OA%" ) ))
( "nomads"  n ( n ou1 m a d z ) ((pos "Kj%" ) ))
( "nomenclature"  n ( n @1 m e1 n k l @ ch @ r ) ((pos "K6%" ) ))
( "nomenclatures"  n ( n @1 m e1 n k l @ ch @ z ) ((pos "Kj$" ) ))
( "nominal"  j ( n o1 m i n l ) ((pos "OA%" ) ))
( "nominally"  a ( n o1 m i n @ l ii ) ((pos "Pu%" ) ))
( "nominate"  v ( n o1 m i n ei t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "nominated"  v ( n o1 m i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "nominates"  v ( n o1 m i n ei t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "nominating"  v ( n o1 m i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "nomination"  n ( n o2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "nominations"  n ( n o2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "nominative"  n ( n o1 m i n @ t i v ) ((pos "K6$" "OA$" ) ))
( "nominatives"  n ( n o1 m i n @ t i v z ) ((pos "Kj$" ) ))
( "nominee"  n ( n o2 m i n ii1 ) ((pos "K6%" ) ))
( "nominees"  n ( n o2 m i n ii1 z ) ((pos "Kj%" ) ))
( "noms de plume"  n ( n o2 m - d @ - p l uu1 m ) ((pos "Kj$" ) ))
( "non compos mentis"  j ( n o2 n - k o2 m p @ s - m e1 n t i s ) ((pos "OA$" ) ))
( "non sequitur"  n ( n o2 n - s e1 k w i t @ r ) ((pos "K6%" ) ))
( "non sequiturs"  n ( n o2 n - s e1 k w i t @ z ) ((pos "Kj%" ) ))
( "non-"  j ( n o2 n - ) ((pos "U-%" ) ))
( "non-U"  j ( n o1 n - y uu ) ((pos "OA$" ) ))
( "non-compliance"  n ( n o2 n - k @ m p l ai1 @ n s ) ((pos "L@%" ) ))
( "non-contentious"  j ( n o2 n - k @ n t e1 n sh @ s ) ((pos "OA%" ) ))
( "non-interference"  n ( n o2 n - i n t @ f i@1 r @ n s ) ((pos "L@%" ) ))
( "non-skid"  j ( n o1 n - s k i d ) ((pos "OA%" ) ))
( "nonage"  n ( n ou1 n i jh ) ((pos "L@$" ) ))
( "nonagenarian"  n ( n o2 n @ jh i n e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "nonagenarians"  n ( n o2 n @ jh i n e@1 r i@ n z ) ((pos "Kj$" ) ))
( "nonaggression"  n ( n o2 n @ g r e1 sh @ n ) ((pos "L@%" ) ))
( "nonalignment"  n ( n o2 n @ l ai1 n m @ n t ) ((pos "L@%" ) ))
( "nonce"  n ( n o1 n s ) ((pos "K6$" ) ))
( "nonce-word"  n ( n o1 n s - w @@ d ) ((pos "K6$" ) ))
( "nonce-words"  n ( n o1 n s - w @@ d z ) ((pos "Kj$" ) ))
( "nonces"  n ( n o1 n s i z ) ((pos "Kj$" ) ))
( "nonchalance"  n ( n o1 n sh @ l @ n s ) ((pos "L@%" ) ))
( "nonchalant"  j ( n o1 n sh @ l @ n t ) ((pos "OA%" ) ))
( "nonchalantly"  a ( n o1 n sh @ l @ n t l ii ) ((pos "Pu%" ) ))
( "noncombatant"  n ( n o1 n k o1 m b @ t @ n t ) ((pos "K6%" ) ))
( "noncombatants"  n ( n o1 n k o1 m b @ t @ n t s ) ((pos "Kj%" ) ))
( "noncommissioned"  j ( n o2 n k @ m i1 sh @ n d ) ((pos "OA%" ) ))
( "noncommittal"  j ( n o2 n k @ m i1 t l ) ((pos "OA%" ) ))
( "nonconductor"  n ( n o2 n k @ n d uh1 k t @ r ) ((pos "K6$" ) ))
( "nonconductors"  n ( n o2 n k @ n d uh1 k t @ z ) ((pos "Kj$" ) ))
( "nonconformist"  n ( n o2 n k @ n f oo1 m i s t ) ((pos "K6%" ) ))
( "nonconformists"  n ( n o2 n k @ n f oo1 m i s t s ) ((pos "Kj%" ) ))
( "nonconformity"  n ( n o2 n k @ n f oo1 m i t ii ) ((pos "L@%" ) ))
( "nondescript"  n ( n o1 n d i s k r i p t ) ((pos "K6$" "OA%" ) ))
( "nondescripts"  n ( n o1 n d i s k r i p t s ) ((pos "Kj$" ) ))
( "none"  a ( n uh n ) ((pos "Pu%" "Qx%" ) ))
( "nonentities"  n ( n o1 n e1 n t i t i z ) ((pos "Kj%" ) ))
( "nonentity"  n ( n o1 n e1 n t i t ii ) ((pos "K8%" ) ))
( "nonesuch"  n ( n uh1 n s uh ch ) ((pos "Ki$" ) ))
( "nonevent"  n ( n o1 n i v e n t ) ((pos "K6%" ) ))
( "nonevents"  n ( n o1 n i v e n t s ) ((pos "Kj$" ) ))
( "nonfiction"  n ( n o1 n f i1 k sh @ n ) ((pos "L@%" ) ))
( "nonflammable"  j ( n o1 n f l a1 m @ b l ) ((pos "OA%" ) ))
( "nonintervention"  n ( n o2 n i n t @ v e1 n sh @ n ) ((pos "L@%" ) ))
( "nonmoral"  j ( n o1 n m o1 r @ l ) ((pos "OA$" ) ))
( "nonobservance"  n ( n o2 n @ b z @@1 v @ n s ) ((pos "L@%" ) ))
( "nonpareil"  n ( n o2 n p @ r ei1 l ) ((pos "K6$" "OA$" ) ))
( "nonpareils"  n ( n o2 n p @ r ei1 l z ) ((pos "Kj$" ) ))
( "nonpayment"  n ( n o1 n p ei1 m @ n t ) ((pos "M6%" ) ))
( "nonpayments"  n ( n o1 n p ei1 m @ n t s ) ((pos "Mj$" ) ))
( "nonplus"  v ( n o1 n p l uh1 s ) ((pos "H5$" ) (subcat )))
( "nonplussed"  v ( n o1 n p l uh1 s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "nonplusses"  v ( n o1 n p l uh1 s i z ) ((pos "Ha$" ) (subcat )))
( "nonplussing"  v ( n o1 n p l uh1 s i ng ) ((pos "Hb$" ) (subcat )))
( "nonresident"  j ( n o1 n r e1 z i d @ n t ) ((pos "OA%" ) ))
( "nonsense"  n ( n o1 n s n s ) ((pos "M6%" ) ))
( "nonsenses"  n ( n o1 n s n s i z ) ((pos "Mj%" ) ))
( "nonsensical"  j ( n o1 n s e1 n s i k l ) ((pos "OA%" ) ))
( "nonsmoker"  n ( n o1 n s m ou1 k @ r ) ((pos "K6%" ) ))
( "nonsmokers"  n ( n o1 n s m ou1 k @ z ) ((pos "Kj%" ) ))
( "nonstarter"  n ( n o1 n s t aa1 t @ r ) ((pos "K6%" ) ))
( "nonstarters"  n ( n o1 n s t aa1 t @ z ) ((pos "Kj%" ) ))
( "nonstick"  j ( n o1 n s t i k ) ((pos "OA%" ) ))
( "nonstop"  j ( n o1 n s t o p ) ((pos "OA%" "Pu%" ) ))
( "nonsuch"  n ( n uh1 n s uh ch ) ((pos "Ki$" ) ))
( "nonunion"  j ( n o1 n y uu1 n i@ n ) ((pos "OA%" ) ))
( "nonviolence"  n ( n o1 n v ai1 @ l @ n s ) ((pos "L@%" ) ))
( "noodle"  n ( n uu1 d l ) ((pos "K6%" ) ))
( "noodles"  n ( n uu1 d l z ) ((pos "Kj%" ) ))
( "nook"  n ( n u1 k ) ((pos "K6%" ) ))
( "nooks"  n ( n u1 k s ) ((pos "Kj%" ) ))
( "noon"  n ( n uu1 n ) ((pos "L@%" ) ))
( "noonday"  j ( n uu1 n d ei ) ((pos "OA%" ) ))
( "noontide"  n ( n uu1 n t ai d ) ((pos "L@%" ) ))
( "noose"  v ( n uu1 s ) ((pos "H2$" "K6%" ) ))
( "noosed"  v ( n uu1 s t ) ((pos "Hc$" "Hd$" ) ))
( "nooses"  v ( n uu1 s i z ) ((pos "Ha$" "Kj%" ) ))
( "noosing"  v ( n uu1 s i ng ) ((pos "Hb$" ) ))
( "nope"  uh ( n ou1 p ) ((pos "W-$" ) ))
( "nor"  cc ( n oo r ) ((pos "V-%" ) ))
( "nor'-"  j ( n oo2 r - ) ((pos "U-$" ) ))
( "nor'-east"  n ( n oo2 r - ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "nor'-nor'-east"  n ( n oo2 - n oo r - ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "nor'-nor'-west"  n ( n oo2 - n oo - w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "nor'-west"  n ( n oo2 - w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "norm"  n ( n oo1 m ) ((pos "K6%" ) ))
( "normal"  n ( n oo1 m l ) ((pos "L@%" "OA%" ) ))
( "normalcy"  n ( n oo1 m l s ii ) ((pos "L@$" ) ))
( "normality"  n ( n oo1 m a1 l i t ii ) ((pos "L@%" ) ))
( "normalization"  n ( n oo2 m @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "normalizations"  n ( n oo2 m @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "normalize"  v ( n oo1 m @ l ai z ) ((pos "H2%" ) ))
( "normalized"  v ( n oo1 m @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "normalizes"  v ( n oo1 m @ l ai z i z ) ((pos "Ha%" ) ))
( "normalizing"  v ( n oo1 m @ l ai z i ng ) ((pos "Hb%" ) ))
( "normally"  a ( n oo1 m @ l ii ) ((pos "Pu%" ) ))
( "normative"  j ( n oo1 m @ t i v ) ((pos "OA$" ) ))
( "norms"  n ( n oo1 m z ) ((pos "Kj%" ) ))
( "north"  n ( n oo1 th ) ((pos "L@%" "Pu%" ) ))
( "north-northeast"  n ( n oo2 th - n oo th ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "north-northwest"  n ( n oo2 th - n oo th w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "northeast"  n ( n oo1 th ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "northeaster"  n ( n oo1 th ii1 s t @ r ) ((pos "K6%" ) ))
( "northeasterly"  j ( n oo1 th ii1 s t @ l ii ) ((pos "OA%" ) ))
( "northeastern"  j ( n oo1 th ii1 s t @ n ) ((pos "OA%" ) ))
( "northeasters"  n ( n oo1 th ii1 s t @ z ) ((pos "Kj%" ) ))
( "northerly"  j ( n oo1 dh @ l ii ) ((pos "OA%" "Pu%" ) ))
( "northern"  j ( n oo1 dh @ n ) ((pos "OA%" ) ))
( "northerner"  n ( n oo1 dh @ n @ r ) ((pos "K6%" ) ))
( "northerners"  n ( n oo1 dh @ n @ z ) ((pos "Kj%" ) ))
( "northernmost"  j ( n oo1 dh @ n m ou s t ) ((pos "OA%" ) ))
( "northwards"  a ( n oo1 th w @ d z ) ((pos "Pu%" ) ))
( "northwest"  n ( n oo1 th w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "northwester"  n ( n oo1 th w e1 s t @ r ) ((pos "K6$" ) ))
( "northwesterly"  j ( n oo1 th w e1 s t @ l ii ) ((pos "OA%" ) ))
( "northwestern"  j ( n oo1 th w e1 s t @ n ) ((pos "OA%" ) ))
( "northwesters"  n ( n oo1 th w e1 s t @ z ) ((pos "Kj$" ) ))
( "nos"  n ( n uh1 m b @ z ) ((pos "Y)%" ) ))
( "nos"  n ( n ou1 z ) ((pos "Kj$" ) ))
( "nose"  v ( n ou1 z ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "15A" )))
( "nose-flute"  n ( n ou1 z - f l uu t ) ((pos "K6$" ) ))
( "nose-flutes"  n ( n ou1 z - f l uu t s ) ((pos "Kj$" ) ))
( "nose-wheel"  n ( n ou1 z - w ii l ) ((pos "K6$" ) ))
( "nose-wheels"  n ( n ou1 z - w ii l z ) ((pos "Kj$" ) ))
( "nosebag"  n ( n ou1 z b a g ) ((pos "K6%" ) ))
( "nosebags"  n ( n ou1 z b a g z ) ((pos "Kj%" ) ))
( "nosebleed"  n ( n ou1 z b l ii d ) ((pos "K6%" ) ))
( "nosebleeds"  n ( n ou1 z b l ii d z ) ((pos "Kj%" ) ))
( "nosecone"  n ( n ou1 z k ou n ) ((pos "K6%" ) ))
( "nosecones"  n ( n ou1 z k ou n z ) ((pos "Kj%" ) ))
( "nosed"  v ( n ou1 z d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2C" "3A" "15A" )))
( "nosedive"  v ( n ou1 z d ai v ) ((pos "I2%" "K6%" ) ))
( "nosedived"  v ( n ou1 z d ai v d ) ((pos "Ic%" "Id%" ) ))
( "nosedives"  v ( n ou1 z d ai v z ) ((pos "Ia%" "Kj%" ) ))
( "nosediving"  v ( n ou1 z d ai v i ng ) ((pos "Ib%" ) ))
( "nosegay"  n ( n ou1 z g ei ) ((pos "K6%" ) ))
( "nosegays"  n ( n ou1 z g ei z ) ((pos "Kj%" ) ))
( "nosering"  n ( n ou1 z r i ng ) ((pos "K6$" ) ))
( "noserings"  n ( n ou1 z r i ng z ) ((pos "Kj$" ) ))
( "noses"  v ( n ou1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "15A" )))
( "nosey"  j ( n ou1 z ii ) ((pos "OA%" ) ))
( "nosh"  v ( n o1 sh ) ((pos "I1%" "L@%" ) ))
( "nosh-up"  n ( n o1 sh - uh p ) ((pos "K6%" ) ))
( "nosh-ups"  n ( n o1 sh - uh p s ) ((pos "Kj$" ) ))
( "noshed"  v ( n o1 sh t ) ((pos "Ic%" "Id%" ) ))
( "noshes"  v ( n o1 sh i z ) ((pos "Ia$" ) ))
( "noshing"  v ( n o1 sh i ng ) ((pos "Ib$" ) ))
( "nosier"  j ( n ou1 z i@ r ) ((pos "Or%" ) ))
( "nosiest"  j ( n ou1 z i i s t ) ((pos "Os%" ) ))
( "nosing"  v ( n ou1 z i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "15A" )))
( "nostalgia"  n ( n o1 s t a1 l jh @ ) ((pos "L@%" ) ))
( "nostalgic"  j ( n o1 s t a1 l jh i k ) ((pos "OA%" ) ))
( "nostalgically"  a ( n o1 s t a1 l jh i k l ii ) ((pos "Pu%" ) ))
( "nostril"  n ( n o1 s t r @ l ) ((pos "K6%" ) ))
( "nostrils"  n ( n o1 s t r @ l z ) ((pos "Kj%" ) ))
( "nostrum"  n ( n o1 s t r @ m ) ((pos "K6$" ) ))
( "nostrums"  n ( n o1 s t r @ m z ) ((pos "Kj$" ) ))
( "nosy"  j ( n ou1 z ii ) ((pos "OD%" ) ))
( "not"  a ( n o1 t ) ((pos "Pu*" ) ))
( "nota bene"  nil ( n ou2 t @ - b e1 n ei ) ((pos "Z-$" ) ))
( "notabilities"  n ( n ou2 t @ b i1 l i t i z ) ((pos "Mj$" ) ))
( "notability"  n ( n ou2 t @ b i1 l i t ii ) ((pos "M8$" ) ))
( "notable"  n ( n ou1 t @ b l ) ((pos "K6%" "OA%" ) ))
( "notables"  n ( n ou1 t @ b l z ) ((pos "Kj%" ) ))
( "notably"  a ( n ou1 t @ b l ii ) ((pos "Pu%" ) ))
( "notaries"  n ( n ou1 t @ r i z ) ((pos "Kj%" ) ))
( "notary"  n ( n ou1 t @ r ii ) ((pos "K8%" ) ))
( "notation"  n ( n ou1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "notations"  n ( n ou1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "notch"  v ( n o1 ch ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "notched"  v ( n o1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "notches"  v ( n o1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "notching"  v ( n o1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "note"  v ( n ou1 t ) ((pos "H2%" "M6%" ) (subcat "6A" "8" "9" "10" )))
( "notebook"  n ( n ou1 t b u k ) ((pos "K6%" ) ))
( "notebooks"  n ( n ou1 t b u k s ) ((pos "Kj%" ) ))
( "notecase"  n ( n ou1 t k ei s ) ((pos "K6$" ) ))
( "notecases"  n ( n ou1 t k ei s i z ) ((pos "Kj$" ) ))
( "noted"  v ( n ou1 t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "8" "9" "10" )))
( "notepaper"  n ( n ou1 t p ei p @ r ) ((pos "L@%" ) ))
( "notes"  v ( n ou1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "8" "9" "10" )))
( "noteworthy"  j ( n ou1 t w @@ dh ii ) ((pos "OA%" ) ))
( "nothing"  n ( n uh1 th i ng ) ((pos "K6*" "Pu*" ) ))
( "nothingness"  n ( n uh1 th i ng n @ s ) ((pos "L@%" ) ))
( "nothings"  n ( n uh1 th i ng z ) ((pos "Kj%" ) ))
( "notice"  v ( n ou1 t i s ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" "8" "9" "10" "18A" )))
( "notice-board"  n ( n ou1 t i s - b oo d ) ((pos "K6%" ) ))
( "notice-boards"  n ( n ou1 t i s - b oo d z ) ((pos "Kj%" ) ))
( "noticeable"  j ( n ou1 t i s @ b l ) ((pos "OA%" ) ))
( "noticeably"  a ( n ou1 t i s @ b l ii ) ((pos "Pu%" ) ))
( "noticed"  v ( n ou1 t i s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "8" "9" "10" "18A" )))
( "notices"  v ( n ou1 t i s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "8" "9" "10" "18A" )))
( "noticing"  v ( n ou1 t i s i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "8" "9" "10" "18A" )))
( "notifiable"  j ( n ou1 t i f ai @ b l ) ((pos "OA%" ) ))
( "notification"  n ( n ou2 t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "notifications"  n ( n ou2 t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "notified"  v ( n ou1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" )))
( "notifies"  v ( n ou1 t i f ai z ) ((pos "Ha%" ) (subcat "6A" "11" )))
( "notify"  v ( n ou1 t i f ai ) ((pos "H3%" ) (subcat "6A" "11" )))
( "notifying"  v ( n ou1 t i f ai i ng ) ((pos "Hb%" ) (subcat "6A" "11" )))
( "noting"  v ( n ou1 t i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" )))
( "notion"  n ( n ou1 sh @ n ) ((pos "K6%" ) ))
( "notional"  j ( n ou1 sh @ n @ l ) ((pos "OA%" ) ))
( "notions"  n ( n ou1 sh @ n z ) ((pos "Kj%" ) ))
( "notoriety"  n ( n ou2 t @ r ai1 @ t ii ) ((pos "L@%" ) ))
( "notorious"  j ( n ou1 t oo1 r i@ s ) ((pos "OA%" ) ))
( "notoriously"  a ( n ou1 t oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "notwithstanding"  a ( n o2 t w i th s t a1 n d i ng ) ((pos "Pu%" "T-%" "V-%" ) ))
( "nougat"  n ( n uu1 g aa ) ((pos "M6%" ) ))
( "nougats"  n ( n uu1 g aa z ) ((pos "Mj$" ) ))
( "nought"  n ( n oo1 t ) ((pos "K6%" ) ))
( "noughts"  n ( n oo1 t s ) ((pos "Kj%" ) ))
( "noun"  n ( n au1 n ) ((pos "K6%" ) ))
( "nouns"  n ( n au1 n z ) ((pos "Kj%" ) ))
( "nourish"  v ( n uh1 r i sh ) ((pos "H1%" ) (subcat )))
( "nourished"  v ( n uh1 r i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nourishes"  v ( n uh1 r i sh i z ) ((pos "Ha%" ) (subcat )))
( "nourishing"  v ( n uh1 r i sh i ng ) ((pos "Hb%" ) (subcat )))
( "nourishment"  n ( n uh1 r i sh m @ n t ) ((pos "L@%" ) ))
( "nous"  n ( n au1 s ) ((pos "L@$" ) ))
( "nouveau riche"  n ( n uu2 v ou - r ii1 sh ) ((pos "Ki$" ) ))
( "nouveaux riches"  n ( n uu2 v ou - r ii1 sh ) ((pos "Kj$" ) ))
( "nova"  n ( n ou1 v @ ) ((pos "K6$" ) ))
( "novae"  n ( n ou1 v ii ) ((pos "Kj$" ) ))
( "novas"  n ( n ou1 v @ z ) ((pos "Kj$" ) ))
( "novel"  n ( n o1 v @ l ) ((pos "K6%" "OA%" ) ))
( "novelette"  n ( n o2 v @ l e1 t ) ((pos "K6%" ) ))
( "novelettes"  n ( n o2 v @ l e1 t s ) ((pos "Kj%" ) ))
( "novelist"  n ( n o1 v @ l i s t ) ((pos "K6%" ) ))
( "novelists"  n ( n o1 v @ l i s t s ) ((pos "Kj%" ) ))
( "novels"  n ( n o1 v @ l z ) ((pos "Kj%" ) ))
( "novelties"  n ( n o1 v @ l t i z ) ((pos "Mj%" ) ))
( "novelty"  n ( n o1 v @ l t ii ) ((pos "M8%" ) ))
( "novice"  n ( n o1 v i s ) ((pos "K6%" ) ))
( "novices"  n ( n o1 v i s i z ) ((pos "Kj%" ) ))
( "noviciate"  n ( n @1 v i1 sh i@ t ) ((pos "K6%" ) ))
( "noviciates"  n ( n @1 v i1 sh i@ t s ) ((pos "Kj$" ) ))
( "novitiate"  n ( n @1 v i1 sh i@ t ) ((pos "K6%" ) ))
( "novitiates"  n ( n @1 v i1 sh i@ t s ) ((pos "Kj$" ) ))
( "now"  a ( n au ) ((pos "Pu*" "V-*" ) ))
( "nowadays"  a ( n au1 @ d ei z ) ((pos "Pu%" ) ))
( "nowhere"  a ( n ou1 w e@ r ) ((pos "Pu%" ) ))
( "nowise"  a ( n ou1 w ai z ) ((pos "Pu$" ) ))
( "noxious"  j ( n o1 k sh @ s ) ((pos "OA%" ) ))
( "noxiously"  a ( n o1 k sh @ s l ii ) ((pos "Pu%" ) ))
( "noxiousness"  n ( n o1 k sh @ s n @ s ) ((pos "L@%" ) ))
( "nozzle"  n ( n o1 z l ) ((pos "K6%" ) ))
( "nozzles"  n ( n o1 z l z ) ((pos "Kj%" ) ))
( "nr"  n ( n i@1 r ) ((pos "Y~%" ) ))
( "nuance"  n ( n y uu1 aa n s ) ((pos "K6%" ) ))
( "nuances"  n ( n y uu1 aa n s i z ) ((pos "Kj%" ) ))
( "nub"  n ( n uh1 b ) ((pos "K6%" ) ))
( "nubile"  j ( n y uu1 b ai l ) ((pos "OA%" ) ))
( "nubs"  n ( n uh1 b z ) ((pos "Kj$" ) ))
( "nuclear"  j ( n y uu1 k l i@ r ) ((pos "OA%" ) ))
( "nuclei"  n ( n y uu1 k l i ai ) ((pos "Kj%" ) ))
( "nucleic"  j ( n y uu1 k l ei1 i k ) ((pos "OA%" ) ))
( "nucleus"  n ( n y uu1 k l i@ s ) ((pos "Ki%" ) ))
( "nude"  n ( n y uu1 d ) ((pos "K6%" "OA%" ) ))
( "nudes"  n ( n y uu1 d z ) ((pos "Kj%" ) ))
( "nudge"  v ( n uh1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "nudged"  v ( n uh1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nudges"  v ( n uh1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "nudging"  v ( n uh1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "nudism"  n ( n y uu1 d i z @ m ) ((pos "L@%" ) ))
( "nudist"  n ( n y uu1 d i s t ) ((pos "K6%" ) ))
( "nudists"  n ( n y uu1 d i s t s ) ((pos "Kj%" ) ))
( "nudity"  n ( n y uu1 d i t ii ) ((pos "L@%" ) ))
( "nugatory"  j ( n y uu1 g @ t @ r ii ) ((pos "OA$" ) ))
( "nugget"  n ( n uh1 g i t ) ((pos "K6%" ) ))
( "nuggets"  n ( n uh1 g i t s ) ((pos "Kj%" ) ))
( "nuisance"  n ( n y uu1 s n s ) ((pos "K6%" ) ))
( "nuisances"  n ( n y uu1 s n s i z ) ((pos "Kj%" ) ))
( "null"  j ( n uh1 l ) ((pos "OA%" ) ))
( "nullification"  n ( n uh2 l i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "nullified"  v ( n uh1 l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nullifies"  v ( n uh1 l i f ai z ) ((pos "Ha%" ) (subcat )))
( "nullify"  v ( n uh1 l i f ai ) ((pos "H3%" ) (subcat )))
( "nullifying"  v ( n uh1 l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "nullity"  n ( n uh1 l i t ii ) ((pos "L@%" ) ))
( "numb"  v ( n uh1 m ) ((pos "H0%" "OA%" ) (subcat )))
( "numbed"  v ( n uh1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "number"  v ( n uh1 m b @ r ) ((pos "H0%" "K6*" ) (subcat "2C" "6A" )))
( "numbered"  v ( n uh1 m b @ d ) ((pos "Hc%" "Hd%" ) (subcat "2C" "6A" )))
( "numbering"  v ( n uh1 m b @ r i ng ) ((pos "Hb%" ) (subcat "2C" "6A" )))
( "numberless"  j ( n uh1 m b @ l i s ) ((pos "OA%" ) ))
( "numberplate"  n ( n uh1 m b @ p l ei t ) ((pos "K6%" ) ))
( "numberplates"  n ( n uh1 m b @ p l ei t s ) ((pos "Kj%" ) ))
( "numbers"  v ( n uh1 m b @ z ) ((pos "Ha%" "Kj%" ) (subcat "2C" "6A" )))
( "numbing"  v ( n uh1 m i ng ) ((pos "Hb%" ) (subcat )))
( "numbly"  a ( n uh1 m l ii ) ((pos "Pu%" ) ))
( "numbness"  n ( n uh1 m n @ s ) ((pos "L@%" ) ))
( "numbs"  v ( n uh1 m z ) ((pos "Ha%" ) (subcat )))
( "numerable"  j ( n y uu1 m @ r @ b l ) ((pos "OA$" ) ))
( "numeracy"  n ( n y uu1 m @ r @ s ii ) ((pos "L@%" ) ))
( "numeral"  n ( n y uu1 m @ r @ l ) ((pos "K6%" "OA%" ) ))
( "numerals"  n ( n y uu1 m @ r @ l z ) ((pos "Kj%" ) ))
( "numerate"  j ( n y uu1 m @ r @ t ) ((pos "OA%" ) ))
( "numeration"  n ( n y uu2 m @ r ei1 sh @ n ) ((pos "M6$" ) ))
( "numerations"  n ( n y uu2 m @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "numerator"  n ( n y uu1 m @ r ei t @ r ) ((pos "K6$" ) ))
( "numerators"  n ( n y uu1 m @ r ei t @ z ) ((pos "Kj$" ) ))
( "numeric"  j ( n y uu1 m e1 r i k ) ((pos "OA%" ) ))
( "numerical"  j ( n y uu1 m e1 r i k l ) ((pos "OA%" ) ))
( "numerically"  a ( n y uu1 m e1 r i k l ii ) ((pos "Pu%" ) ))
( "numerous"  j ( n y uu1 m @ r @ s ) ((pos "OA%" ) ))
( "numinous"  j ( n y uu1 m i n @ s ) ((pos "OA$" ) ))
( "numismatics"  n ( n y uu2 m i z m a1 t i k s ) ((pos "Lk$" ) ))
( "numismatist"  n ( n y uu1 m i1 z m @ t i s t ) ((pos "K6$" ) ))
( "numismatists"  n ( n y uu1 m i1 z m @ t i s t s ) ((pos "Kj$" ) ))
( "numskull"  n ( n uh1 m s k uh l ) ((pos "K6%" ) ))
( "numskulls"  n ( n uh1 m s k uh l z ) ((pos "Kj%" ) ))
( "nun"  n ( n uh1 n ) ((pos "K6%" ) ))
( "nuncio"  n ( n uh1 n s i@ @ ) ((pos "K6$" ) ))
( "nuncios"  n ( n uh1 n s i@ u z ) ((pos "Kj$" ) ))
( "nunneries"  n ( n uh1 n @ r i z ) ((pos "Kj%" ) ))
( "nunnery"  n ( n uh1 n @ r ii ) ((pos "K8%" ) ))
( "nuns"  n ( n uh1 n z ) ((pos "Kj%" ) ))
( "nuptial"  j ( n uh1 p sh l ) ((pos "OA%" ) ))
( "nuptials"  n ( n uh1 p sh l z ) ((pos "Kj%" ) ))
( "nurse"  v ( n @@1 s ) ((pos "H2%" "M6%" ) (subcat )))
( "nursed"  v ( n @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "nurseling"  n ( n @@1 s l i ng ) ((pos "K6%" ) ))
( "nurselings"  n ( n @@1 s l i ng z ) ((pos "Kj%" ) ))
( "nursemaid"  n ( n @@1 s m ei d ) ((pos "K6%" ) ))
( "nursemaids"  n ( n @@1 s m ei d z ) ((pos "Kj%" ) ))
( "nurseries"  n ( n @@1 s @ r i z ) ((pos "Kj%" ) ))
( "nursery"  n ( n @@1 s @ r ii ) ((pos "K8%" ) ))
( "nurseryman"  n ( n @@1 s @ r i m @ n ) ((pos "Ki%" ) ))
( "nurserymen"  n ( n @@1 s @ r i m @ n ) ((pos "Kj%" ) ))
( "nurses"  v ( n @@1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "nursing"  v ( n @@1 s i ng ) ((pos "Hb%" ) (subcat )))
( "nursing-home"  n ( n @@1 s i ng - h ou m ) ((pos "K6%" ) ))
( "nursing-homes"  n ( n @@1 s i ng - h ou m z ) ((pos "Kj%" ) ))
( "nursling"  n ( n @@1 s l i ng ) ((pos "K6%" ) ))
( "nurslings"  n ( n @@1 s l i ng z ) ((pos "Kj%" ) ))
( "nurture"  v ( n @@1 ch @ r ) ((pos "H2%" "L@%" ) ))
( "nurtured"  v ( n @@1 ch @ d ) ((pos "Hc%" "Hd%" ) ))
( "nurtures"  v ( n @@1 ch @ z ) ((pos "Ha%" ) ))
( "nurturing"  v ( n @@1 ch @ r i ng ) ((pos "Hb%" ) ))
( "nut"  v ( n uh1 t ) ((pos "I4%" "K6%" ) ))
( "nut-brown"  j ( n uh1 t - b r au n ) ((pos "OA%" ) ))
( "nut-butter"  n ( n uh1 t - b uh1 t @ r ) ((pos "M6$" ) ))
( "nut-butters"  n ( n uh1 t - b uh1 t @ z ) ((pos "Mj$" ) ))
( "nutcrackers"  n ( n uh1 t k r a k @ z ) ((pos "Kj%" ) ))
( "nuthouse"  n ( n uh1 t h au s ) ((pos "K6%" ) ))
( "nuthouses"  n ( n uh1 t h au z i z ) ((pos "Kj$" ) ))
( "nutmeg"  n ( n uh1 t m e g ) ((pos "M6%" ) ))
( "nutmegs"  n ( n uh1 t m e g z ) ((pos "Mj$" ) ))
( "nutria"  n ( n y uu1 t r i@ ) ((pos "L@$" ) ))
( "nutrient"  n ( n y uu1 t r i@ n t ) ((pos "K6%" "OA%" ) ))
( "nutrients"  n ( n y uu1 t r i@ n t s ) ((pos "Kj%" ) ))
( "nutriment"  n ( n y uu1 t r i m @ n t ) ((pos "K6%" ) ))
( "nutriments"  n ( n y uu1 t r i m @ n t s ) ((pos "Kj%" ) ))
( "nutrition"  n ( n y uu1 t r i1 sh @ n ) ((pos "L@%" ) ))
( "nutritional"  j ( n y uu1 t r i1 sh @ n l ) ((pos "OA%" ) ))
( "nutritionally"  a ( n y uu1 t r i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "nutritious"  j ( n y uu1 t r i1 sh @ s ) ((pos "OA%" ) ))
( "nutritive"  j ( n y uu1 t r i t i v ) ((pos "OA$" ) ))
( "nuts"  v ( n uh1 t s ) ((pos "Ia$" "Kj%" "OA%" ) ))
( "nutshell"  n ( n uh1 t - sh e l ) ((pos "K6%" ) ))
( "nutshells"  n ( n uh1 t - sh e l z ) ((pos "Kj%" ) ))
( "nutted"  v ( n uh1 t i d ) ((pos "Ic%" "Id%" ) ))
( "nuttier"  j ( n uh1 t i@ r ) ((pos "Or%" ) ))
( "nuttiest"  j ( n uh1 t i i s t ) ((pos "Os%" ) ))
( "nutting"  v ( n uh1 t i ng ) ((pos "Ib$" ) ))
( "nutty"  j ( n uh1 t ii ) ((pos "OD%" ) ))
( "nuzzle"  v ( n uh1 z l ) ((pos "J2%" ) (subcat "2C" )))
( "nuzzled"  v ( n uh1 z l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "nuzzles"  v ( n uh1 z l z ) ((pos "Ja%" ) (subcat "2C" )))
( "nuzzling"  v ( n uh1 z l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "nylon"  n ( n ai1 l o n ) ((pos "M6%" ) ))
( "nylons"  n ( n ai1 l o n z ) ((pos "Mj%" ) ))
( "nymph"  n ( n i1 m f ) ((pos "K6%" ) ))
( "nymphet"  n ( n i1 m f e1 t ) ((pos "K6$" ) ))
( "nymphets"  n ( n i1 m f e1 t s ) ((pos "Kj$" ) ))
( "nympho"  n ( n i1 m f ou ) ((pos "K6%" ) ))
( "nymphomania"  n ( n i2 m f @ m ei1 n i@ ) ((pos "L@%" ) ))
( "nymphomaniac"  n ( n i2 m f @ m ei1 n i a k ) ((pos "K6%" "OA%" ) ))
( "nymphomaniacs"  n ( n i2 m f @ m ei1 n i a k s ) ((pos "Kj%" ) ))
( "nymphos"  n ( n i1 m f ou z ) ((pos "Kj%" ) ))
( "nymphs"  n ( n i1 m f s ) ((pos "Kj%" ) ))
( "o"  n ( ou1 ) ((pos "Ki$" ) ))
( "o'clock"  nil ( @1 k l o1 k ) ((pos "Z-%" ) ))
( "o'er"  a ( oo r ) ((pos "Pu$" "T-$" ) ))
( "o's"  n ( ou1 z ) ((pos "Kj$" ) ))
( "oaf"  n ( ou1 f ) ((pos "K6%" ) ))
( "oafish"  j ( ou1 f i sh ) ((pos "OA%" ) ))
( "oafs"  n ( ou1 f s ) ((pos "Kj%" ) ))
( "oak"  n ( ou1 k ) ((pos "M6%" ) ))
( "oak-apple"  n ( ou1 k - a p l ) ((pos "K6%" ) ))
( "oak-apples"  n ( ou1 k - a p l z ) ((pos "Kj%" ) ))
( "oaken"  j ( ou1 k @ n ) ((pos "OA%" ) ))
( "oaks"  n ( ou1 k s ) ((pos "Mj%" ) ))
( "oakum"  n ( ou1 k @ m ) ((pos "L@$" ) ))
( "oar"  n ( oo1 r ) ((pos "K6%" ) ))
( "oars"  n ( oo1 z ) ((pos "Kj%" ) ))
( "oarsman"  n ( oo1 z m @ n ) ((pos "Ki%" ) ))
( "oarsmanship"  n ( oo1 z m @ n sh i p ) ((pos "L@%" ) ))
( "oarsmen"  n ( oo1 z m @ n ) ((pos "Kj%" ) ))
( "oarswoman"  n ( oo1 z w u m @ n ) ((pos "Ki%" ) ))
( "oarswomen"  n ( oo1 z w i m i n ) ((pos "Kj%" ) ))
( "oases"  n ( ou1 ei1 s ii z ) ((pos "Kj%" ) ))
( "oasis"  n ( ou1 ei1 s i s ) ((pos "Ki%" ) ))
( "oast"  n ( ou1 s t ) ((pos "K6$" ) ))
( "oasthouse"  n ( ou1 s t h au s ) ((pos "K6%" ) ))
( "oasthouses"  n ( ou1 s t h au z i z ) ((pos "Kj%" ) ))
( "oasts"  n ( ou1 s t s ) ((pos "Kj$" ) ))
( "oat"  n ( ou1 t ) ((pos "K6%" ) ))
( "oatcake"  n ( ou1 t k ei k ) ((pos "K6%" ) ))
( "oatcakes"  n ( ou1 t k ei k s ) ((pos "Kj%" ) ))
( "oath"  n ( ou1 th ) ((pos "K6%" ) ))
( "oaths"  n ( ou1 dh z ) ((pos "Kj%" ) ))
( "oatmeal"  n ( ou1 t m ii l ) ((pos "L@%" ) ))
( "oats"  n ( ou1 t s ) ((pos "Kj%" ) ))
( "oaves"  n ( ou1 v z ) ((pos "Kj$" ) ))
( "ob"  n ( o1 b ) ((pos "Y~%" ) ))
( "obbligato"  n ( o2 b l i g aa1 t ou ) ((pos "K6$" ) ))
( "obbligatos"  n ( o2 b l i g aa1 t ou z ) ((pos "Kj$" ) ))
( "obduracy"  n ( o1 b d y u@ r @ s ii ) ((pos "L@%" ) ))
( "obdurate"  j ( o1 b d y u@ r @ t ) ((pos "OA%" ) ))
( "obdurately"  a ( o1 b d y u@ r @ t l ii ) ((pos "Pu%" ) ))
( "obedience"  n ( @1 b ii1 d i@ n s ) ((pos "L@%" ) ))
( "obedient"  j ( @1 b ii1 d i@ n t ) ((pos "OA%" ) ))
( "obediently"  a ( @1 b ii1 d i@ n t l ii ) ((pos "Pu%" ) ))
( "obeisance"  n ( ou1 b ei1 s n s ) ((pos "M6$" ) ))
( "obeisances"  n ( ou1 b ei1 s n s i z ) ((pos "Mj$" ) ))
( "obelisk"  n ( o1 b @ l i s k ) ((pos "K6%" ) ))
( "obelisks"  n ( o1 b @ l i s k s ) ((pos "Kj%" ) ))
( "obese"  j ( ou1 b ii1 s ) ((pos "OA%" ) ))
( "obesity"  n ( ou1 b ii1 s i t ii ) ((pos "L@%" ) ))
( "obey"  v ( @1 b ei1 ) ((pos "J0%" ) (subcat "2A" )))
( "obeyed"  v ( @1 b ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "obeying"  v ( @1 b ei1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "obeys"  v ( @1 b ei1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "obfuscate"  v ( o1 b f @ s k ei t ) ((pos "H2$" ) (subcat )))
( "obfuscated"  v ( o1 b f @ s k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "obfuscates"  v ( o1 b f @ s k ei t s ) ((pos "Ha$" ) (subcat )))
( "obfuscating"  v ( o1 b f @ s k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "obi"  n ( ou1 b ii ) ((pos "K6$" ) ))
( "obis"  n ( ou1 b i z ) ((pos "Kj$" ) ))
( "obiter dicta"  n ( o2 b i t @ - d i1 k t @ ) ((pos "Kj$" ) ))
( "obiter dictum"  n ( o2 b i t @ - d i1 k t @ m ) ((pos "Ki$" ) ))
( "obituaries"  n ( @1 b i1 ch u@ r i z ) ((pos "Kj%" ) ))
( "obituary"  n ( @1 b i1 ch u@ r ii ) ((pos "K8%" ) ))
( "object"  n ( o1 b jh i k t ) ((pos "K6%" ) ))
( "object"  v ( @1 b jh e1 k t ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "objected"  v ( @1 b jh e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "objecting"  v ( @1 b jh e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "objection"  n ( @1 b jh e1 k sh @ n ) ((pos "M6%" ) ))
( "objectionable"  j ( @1 b jh e1 k sh @ n @ b l ) ((pos "OA%" ) ))
( "objectionably"  a ( @1 b jh e1 k sh @ n @ b l ii ) ((pos "Pu%" ) ))
( "objections"  n ( @1 b jh e1 k sh @ n z ) ((pos "Mj%" ) ))
( "objective"  n ( @1 b jh e1 k t i v ) ((pos "K6%" "OA%" ) ))
( "objectively"  a ( @1 b jh e1 k t i v l ii ) ((pos "Pu%" ) ))
( "objectives"  n ( @1 b jh e1 k t i v z ) ((pos "Kj%" ) ))
( "objectivity"  n ( o2 b jh e k t i1 v i t ii ) ((pos "L@%" ) ))
( "objector"  n ( @1 b jh e1 k t @ r ) ((pos "K6%" ) ))
( "objectors"  n ( @1 b jh e1 k t @ z ) ((pos "Kj%" ) ))
( "objects"  n ( o1 b jh i k t s ) ((pos "Kj%" ) ))
( "objects"  v ( @1 b jh e1 k t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "objurgate"  v ( o1 b jh @ g ei t ) ((pos "H2$" ) (subcat )))
( "objurgated"  v ( o1 b jh @ g ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "objurgates"  v ( o1 b jh @ g ei t s ) ((pos "Ha$" ) (subcat )))
( "objurgating"  v ( o1 b jh @ g ei t i ng ) ((pos "Hb$" ) (subcat )))
( "objurgation"  n ( o2 b jh @ g ei1 sh @ n ) ((pos "M6$" ) ))
( "objurgations"  n ( o2 b jh @ g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "oblate"  j ( o1 b l ei t ) ((pos "OA$" ) ))
( "oblation"  n ( @1 b l ei1 sh @ n ) ((pos "K6$" ) ))
( "oblations"  n ( @1 b l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "obligate"  v ( o1 b l i g ei t ) ((pos "H2%" ) (subcat )))
( "obligated"  v ( o1 b l i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "obligates"  v ( o1 b l i g ei t s ) ((pos "Ha%" ) (subcat )))
( "obligating"  v ( o1 b l i g ei t i ng ) ((pos "Hb$" ) (subcat )))
( "obligation"  n ( o2 b l i g ei1 sh @ n ) ((pos "K6%" ) ))
( "obligations"  n ( o2 b l i g ei1 sh @ n z ) ((pos "Kj%" ) ))
( "obligatory"  j ( @1 b l i1 g @ t r ii ) ((pos "OA%" ) ))
( "oblige"  v ( @1 b l ai1 jh ) ((pos "H2%" ) (subcat "6A" "14" )))
( "obliged"  v ( @1 b l ai1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "obliges"  v ( @1 b l ai1 jh i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "obliging"  v ( @1 b l ai1 jh i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "obligingly"  a ( @1 b l ai1 jh i ng l ii ) ((pos "Pu%" ) ))
( "oblique"  j ( @1 b l ii1 k ) ((pos "OA%" ) ))
( "obliquely"  a ( @1 b l ii1 k l ii ) ((pos "Pu%" ) ))
( "obliquities"  n ( @1 b l i1 k w i t i z ) ((pos "Mj$" ) ))
( "obliquity"  n ( @1 b l i1 k w i t ii ) ((pos "M8$" ) ))
( "obliterate"  v ( @1 b l i1 t @ r ei t ) ((pos "H2%" ) (subcat )))
( "obliterated"  v ( @1 b l i1 t @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "obliterates"  v ( @1 b l i1 t @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "obliterating"  v ( @1 b l i1 t @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "obliteration"  n ( @1 b l i2 t @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "oblivion"  n ( @1 b l i1 v i@ n ) ((pos "L@%" ) ))
( "oblivious"  j ( @1 b l i1 v i@ s ) ((pos "OA%" ) ))
( "oblong"  n ( o1 b l o ng ) ((pos "K6%" "OA%" ) ))
( "oblongs"  n ( o1 b l o ng z ) ((pos "Kj%" ) ))
( "obloquy"  n ( o1 b l @ k w ii ) ((pos "L@$" ) ))
( "obnoxious"  j ( @1 b n o1 k sh @ s ) ((pos "OA%" ) ))
( "obnoxiously"  a ( @1 b n o1 k sh @ s l ii ) ((pos "Pu%" ) ))
( "obnoxiousness"  n ( @1 b n o1 k sh @ s n @ s ) ((pos "L@%" ) ))
( "oboe"  n ( ou1 b ou ) ((pos "K6%" ) ))
( "oboes"  n ( ou1 b ou z ) ((pos "Kj%" ) ))
( "oboist"  n ( ou1 b ou i s t ) ((pos "K6%" ) ))
( "oboists"  n ( ou1 b ou i s t s ) ((pos "Kj%" ) ))
( "obscene"  j ( @1 b s ii1 n ) ((pos "OA%" ) ))
( "obscenely"  a ( @1 b s ii1 n l ii ) ((pos "Pu%" ) ))
( "obscenities"  n ( @1 b s e1 n i t i z ) ((pos "Mj%" ) ))
( "obscenity"  n ( @1 b s e1 n i t ii ) ((pos "M8%" ) ))
( "obscurantism"  n ( o2 b s k y u r a1 n t i z @ m ) ((pos "L@$" ) ))
( "obscurantist"  n ( o2 b s k y u r a1 n t i s t ) ((pos "K6$" ) ))
( "obscurantists"  n ( o2 b s k y u r a1 n t i s t s ) ((pos "Kj$" ) ))
( "obscure"  v ( @1 b s k y u@1 r ) ((pos "H2%" "OA%" ) (subcat )))
( "obscured"  v ( @1 b s k y u@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "obscurely"  a ( @1 b s k y u@1 l ii ) ((pos "Pu%" ) ))
( "obscures"  v ( @1 b s k y u@1 z ) ((pos "Ha%" ) (subcat )))
( "obscuring"  v ( @1 b s k y u@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "obscurities"  n ( @1 b s k y u@1 r i t i z ) ((pos "Mj%" ) ))
( "obscurity"  n ( @1 b s k y u@1 r i t ii ) ((pos "M8%" ) ))
( "obsequies"  n ( o1 b s i k w i z ) ((pos "Kj$" ) ))
( "obsequious"  j ( @1 b s ii1 k w i@ s ) ((pos "OA%" ) ))
( "obsequiously"  a ( @1 b s ii1 k w i@ s l ii ) ((pos "Pu%" ) ))
( "obsequiousness"  n ( @1 b s ii1 k w i@ s n @ s ) ((pos "L@%" ) ))
( "observable"  j ( @1 b z @@1 v @ b l ) ((pos "OA%" ) ))
( "observably"  a ( @1 b z @@1 v @ b l ii ) ((pos "Pu%" ) ))
( "observance"  n ( @1 b z @@1 v @ n s ) ((pos "M6%" ) ))
( "observances"  n ( @1 b z @@1 v @ n s i z ) ((pos "Mj%" ) ))
( "observant"  j ( @1 b z @@1 v @ n t ) ((pos "OA%" ) ))
( "observantly"  a ( @1 b z @@1 v @ n t l ii ) ((pos "Pu%" ) ))
( "observation"  n ( o2 b z @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "observations"  n ( o2 b z @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "observatories"  n ( @1 b z @@1 v @ t r i z ) ((pos "Kj%" ) ))
( "observatory"  n ( @1 b z @@1 v @ t r ii ) ((pos "K8%" ) ))
( "observe"  v ( @1 b z @@1 v ) ((pos "J2%" ) (subcat "2A" "6A" "8" "9" "10" "18A" "19A" )))
( "observed"  v ( @1 b z @@1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "8" "9" "10" "18A" "19A" )))
( "observer"  n ( @1 b z @@1 v @ r ) ((pos "K6%" ) ))
( "observers"  n ( @1 b z @@1 v @ z ) ((pos "Kj%" ) ))
( "observes"  v ( @1 b z @@1 v z ) ((pos "Ja%" ) (subcat "2A" "6A" "8" "9" "10" "18A" "19A" )))
( "observing"  v ( @1 b z @@1 v i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "8" "9" "10" "18A" "19A" )))
( "observingly"  a ( @1 b z @@1 v i ng l ii ) ((pos "Pu$" ) ))
( "obsess"  v ( @1 b s e1 s ) ((pos "H1$" ) (subcat )))
( "obsessed"  v ( @1 b s e1 s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "obsesses"  v ( @1 b s e1 s i z ) ((pos "Ha$" ) (subcat )))
( "obsessing"  v ( @1 b s e1 s i ng ) ((pos "Hb$" ) (subcat )))
( "obsession"  n ( @1 b s e1 sh @ n ) ((pos "M6%" ) ))
( "obsessional"  j ( @1 b s e1 sh @ n l ) ((pos "OA%" ) ))
( "obsessionally"  a ( @1 b s e1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "obsessions"  n ( @1 b s e1 sh @ n z ) ((pos "Mj%" ) ))
( "obsessive"  j ( @1 b s e1 s i v ) ((pos "OA%" ) ))
( "obsessively"  a ( @1 b s e1 s i v l ii ) ((pos "Pu%" ) ))
( "obsidian"  n ( @1 b s i1 d i@ n ) ((pos "L@$" ) ))
( "obsolescence"  n ( o2 b s @ l e1 s n s ) ((pos "L@%" ) ))
( "obsolescent"  j ( o2 b s @ l e1 s n t ) ((pos "OA%" ) ))
( "obsolete"  j ( o1 b s @ l ii t ) ((pos "OA%" ) ))
( "obstacle"  n ( o1 b s t @ k l ) ((pos "K6%" ) ))
( "obstacles"  n ( o1 b s t @ k l z ) ((pos "Kj%" ) ))
( "obstetric"  j ( @1 b s t e1 t r i k ) ((pos "OA%" ) ))
( "obstetrical"  j ( @1 b s t e1 t r i k l ) ((pos "OA$" ) ))
( "obstetrician"  n ( o2 b s t i t r i1 sh @ n ) ((pos "K6%" ) ))
( "obstetricians"  n ( o2 b s t i t r i1 sh @ n z ) ((pos "Kj%" ) ))
( "obstetrics"  n ( @1 b s t e1 t r i k s ) ((pos "Lk%" ) ))
( "obstinacy"  n ( o1 b s t i n @ s ii ) ((pos "L@%" ) ))
( "obstinate"  j ( o1 b s t i n @ t ) ((pos "OA%" ) ))
( "obstinately"  a ( o1 b s t i n @ t l ii ) ((pos "Pu%" ) ))
( "obstreperous"  j ( @1 b s t r e1 p @ r @ s ) ((pos "OA%" ) ))
( "obstreperously"  a ( @1 b s t r e1 p @ r @ s l ii ) ((pos "Pu%" ) ))
( "obstreperousness"  n ( @1 b s t r e1 p @ r @ s n @ s ) ((pos "L@%" ) ))
( "obstruct"  v ( @1 b s t r uh1 k t ) ((pos "H0%" ) (subcat )))
( "obstructed"  v ( @1 b s t r uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "obstructing"  v ( @1 b s t r uh1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "obstruction"  n ( @1 b s t r uh1 k sh @ n ) ((pos "M6%" ) ))
( "obstructionism"  n ( @1 b s t r uh1 k sh @ n i z @ m ) ((pos "L@$" ) ))
( "obstructionist"  n ( @1 b s t r uh1 k sh @ n i s t ) ((pos "K6$" ) ))
( "obstructionists"  n ( @1 b s t r uh1 k sh @ n i s t s ) ((pos "Kj$" ) ))
( "obstructions"  n ( @1 b s t r uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "obstructive"  j ( @1 b s t r uh1 k t i v ) ((pos "OA%" ) ))
( "obstructively"  a ( @1 b s t r uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "obstructs"  v ( @1 b s t r uh1 k t s ) ((pos "Ha%" ) (subcat )))
( "obtain"  v ( @1 b t ei1 n ) ((pos "J0%" ) (subcat "2A" )))
( "obtainable"  j ( @1 b t ei1 n @ b l ) ((pos "OA%" ) ))
( "obtained"  v ( @1 b t ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "obtaining"  v ( @1 b t ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "obtains"  v ( @1 b t ei1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "obtrude"  v ( @1 b t r uu1 d ) ((pos "J2%" ) (subcat "2A" )))
( "obtruded"  v ( @1 b t r uu1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "obtrudes"  v ( @1 b t r uu1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "obtruding"  v ( @1 b t r uu1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "obtrusive"  j ( @1 b t r uu1 s i v ) ((pos "OA%" ) ))
( "obtrusively"  a ( @1 b t r uu1 s i v l ii ) ((pos "Pu%" ) ))
( "obtuse"  j ( @1 b t y uu1 s ) ((pos "OA%" ) ))
( "obtusely"  a ( @1 b t y uu1 s l ii ) ((pos "Pu%" ) ))
( "obtuseness"  n ( @1 b t y uu1 s n @ s ) ((pos "L@%" ) ))
( "obverse"  n ( o1 b v @@ s ) ((pos "K6%" ) ))
( "obverses"  n ( o1 b v @@ s i z ) ((pos "Kj$" ) ))
( "obviate"  v ( o1 b v i ei t ) ((pos "H2%" ) (subcat )))
( "obviated"  v ( o1 b v i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "obviates"  v ( o1 b v i ei t s ) ((pos "Ha%" ) (subcat )))
( "obviating"  v ( o1 b v i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "obvious"  j ( o1 b v i@ s ) ((pos "OA%" ) ))
( "obviously"  a ( o1 b v i@ s l ii ) ((pos "Pu%" ) ))
( "obviousness"  n ( o1 b v i@ s n @ s ) ((pos "L@%" ) ))
( "ocarina"  n ( o2 k @ r ii1 n @ ) ((pos "K6$" ) ))
( "ocarinas"  n ( o2 k @ r ii1 n @ z ) ((pos "Kj$" ) ))
( "occasion"  v ( @1 k ei1 zh n ) ((pos "H0%" "M6%" ) (subcat "6A" "12A" )))
( "occasional"  j ( @1 k ei1 zh @ n l ) ((pos "OA%" ) ))
( "occasionally"  a ( @1 k ei1 zh @ n @ l ii ) ((pos "Pu%" ) ))
( "occasioned"  v ( @1 k ei1 zh n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" )))
( "occasioning"  v ( @1 k ei1 zh n i ng ) ((pos "Hb%" ) (subcat "6A" "12A" )))
( "occasions"  v ( @1 k ei1 zh n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "12A" )))
( "occult"  n ( o1 k uh1 l t ) ((pos "Ki%" "OA%" ) ))
( "occupancies"  n ( o1 k y u p @ n s i z ) ((pos "Kj%" ) ))
( "occupancy"  n ( o1 k y u p @ n s ii ) ((pos "K8%" ) ))
( "occupant"  n ( o1 k y u p @ n t ) ((pos "K6%" ) ))
( "occupants"  n ( o1 k y u p @ n t s ) ((pos "Kj%" ) ))
( "occupation"  n ( o2 k y u p ei1 sh @ n ) ((pos "M6%" ) ))
( "occupational"  j ( o2 k y u p ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "occupations"  n ( o2 k y u p ei1 sh @ n z ) ((pos "Mj%" ) ))
( "occupied"  v ( o1 k y u p ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "occupier"  n ( o1 k y u p ai @ r ) ((pos "K6%" ) ))
( "occupiers"  n ( o1 k y u p ai @ z ) ((pos "Kj%" ) ))
( "occupies"  v ( o1 k y u p ai z ) ((pos "Ha%" ) (subcat )))
( "occupy"  v ( o1 k y u p ai ) ((pos "H3%" ) (subcat )))
( "occupying"  v ( o1 k y u p ai i ng ) ((pos "Hb%" ) (subcat )))
( "occur"  v ( @1 k @@1 r ) ((pos "I4%" ) (subcat "2A" "2C" )))
( "occurred"  v ( @1 k @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "occurrence"  n ( @1 k uh1 r @ n s ) ((pos "M6%" ) ))
( "occurrences"  n ( @1 k uh1 r @ n s i z ) ((pos "Mj%" ) ))
( "occurring"  v ( @1 k @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "occurs"  v ( @1 k @@1 z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "ocean"  n ( ou1 sh @ n ) ((pos "K6%" ) ))
( "oceanic"  j ( ou2 sh i a1 n i k ) ((pos "OA%" ) ))
( "oceans"  n ( ou1 sh @ n z ) ((pos "Kj%" ) ))
( "ochre"  n ( ou1 k @ r ) ((pos "L@$" ) ))
( "octagon"  n ( o1 k t @ g @ n ) ((pos "K6%" ) ))
( "octagonal"  j ( o1 k t a1 g @ n l ) ((pos "OA%" ) ))
( "octagons"  n ( o1 k t @ g @ n z ) ((pos "Kj%" ) ))
( "octane"  n ( o1 k t ei n ) ((pos "L@%" ) ))
( "octave"  n ( o1 k t i v ) ((pos "K6%" ) ))
( "octaves"  n ( o1 k t i v z ) ((pos "Kj%" ) ))
( "octavo"  n ( o1 k t ei1 v ou ) ((pos "K6$" ) ))
( "octavos"  n ( o1 k t ei1 v ou z ) ((pos "Kj$" ) ))
( "octet"  n ( o1 k t e1 t ) ((pos "K6%" ) ))
( "octets"  n ( o1 k t e1 t s ) ((pos "Kj%" ) ))
( "octette"  n ( o1 k t e1 t ) ((pos "K6$" ) ))
( "octettes"  n ( o1 k t e1 t s ) ((pos "Kj$" ) ))
( "octogenarian"  n ( o2 k t @ jh i n e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "octogenarians"  n ( o2 k t @ jh i n e@1 r i@ n z ) ((pos "Kj$" ) ))
( "octopus"  n ( o1 k t @ p @ s ) ((pos "K7%" ) ))
( "octopuses"  n ( o1 k t @ p @ s i z ) ((pos "Kj%" ) ))
( "octroi"  n ( o1 k t r w aa ) ((pos "K6$" ) ))
( "octrois"  n ( o1 k t r w aa z ) ((pos "Kj$" ) ))
( "ocular"  j ( o1 k y u l @ r ) ((pos "OA%" ) ))
( "oculist"  n ( o1 k y u l i s t ) ((pos "K6%" ) ))
( "oculists"  n ( o1 k y u l i s t s ) ((pos "Kj%" ) ))
( "odalisque"  n ( ou1 d @ l i s k ) ((pos "K6$" ) ))
( "odalisques"  n ( ou1 d @ l i s k s ) ((pos "Kj$" ) ))
( "odd"  j ( o1 d ) ((pos "OC%" ) ))
( "odd-job"  j ( o1 d - jh o1 b ) ((pos "Oq%" ) ))
( "odder"  j ( o1 d @ r ) ((pos "Or%" ) ))
( "oddest"  j ( o1 d i s t ) ((pos "Os%" ) ))
( "oddities"  n ( o1 d i t i z ) ((pos "Mj%" ) ))
( "oddity"  n ( o1 d i t ii ) ((pos "M8%" ) ))
( "oddly"  a ( o1 d l ii ) ((pos "Pu%" ) ))
( "oddment"  n ( o1 d m @ n t ) ((pos "K6%" ) ))
( "oddments"  n ( o1 d m @ n t s ) ((pos "Kj%" ) ))
( "odds"  n ( o1 d z ) ((pos "Kj%" ) ))
( "odds-on"  j ( o1 d z - o1 n ) ((pos "OA%" "Pu%" ) ))
( "ode"  n ( ou1 d ) ((pos "K6%" ) ))
( "odes"  n ( ou1 d z ) ((pos "Kj%" ) ))
( "odious"  j ( ou1 d i@ s ) ((pos "OA%" ) ))
( "odiously"  a ( ou1 d i@ s l ii ) ((pos "Pu%" ) ))
( "odium"  n ( ou1 d i@ m ) ((pos "L@%" ) ))
( "odoriferous"  j ( ou2 d @ r i1 f @ r @ s ) ((pos "OA$" ) ))
( "odorous"  j ( ou1 d @ r @ s ) ((pos "OA%" ) ))
( "odour"  n ( ou1 d @ r ) ((pos "M6%" ) ))
( "odourless"  j ( ou1 d @ l @ s ) ((pos "OA%" ) ))
( "odours"  n ( ou1 d @ z ) ((pos "Mj%" ) ))
( "odyssey"  n ( o1 d i s ii ) ((pos "K6%" ) ))
( "odysseys"  n ( o1 d i s i z ) ((pos "Kj%" ) ))
( "oecumenical"  j ( ii2 k y uu m e1 n i k l ) ((pos "OA$" ) ))
( "oesophagus"  n ( ii1 s o1 f @ g @ s ) ((pos "K7%" ) ))
( "oesophaguses"  n ( ii1 s o1 f @ g @ s i z ) ((pos "Kj$" ) ))
( "of"  in ( o v ) ((pos "T-*" ) ))
( "off"  j ( o f ) ((pos "OA%" "P+*" "T-*" "U-*" ) ))
( "off-day"  n ( o1 f - d ei ) ((pos "K6%" ) ))
( "off-days"  n ( o1 f - d ei z ) ((pos "Kj$" ) ))
( "off-licence"  n ( o1 f - l ai s n s ) ((pos "K6%" ) ))
( "off-licences"  n ( o1 f - l ai s n s i z ) ((pos "Kj%" ) ))
( "off-peak"  j ( o1 f - p ii k ) ((pos "Oq%" ) ))
( "off-putting"  j ( o2 f - p u1 t i ng ) ((pos "OA%" ) ))
( "off-scourings"  n ( o1 f - s k au @ r i ng z ) ((pos "Kj$" ) ))
( "off-street"  j ( o1 f - s t r ii t ) ((pos "Oq$" ) ))
( "off-white"  j ( o2 f - w ai1 t ) ((pos "OA%" ) ))
( "offal"  n ( o1 f l ) ((pos "L@%" ) ))
( "offbeat"  j ( o2 f b ii1 t ) ((pos "OA%" ) ))
( "offence"  n ( @1 f e1 n s ) ((pos "M6%" ) ))
( "offenceless"  j ( @1 f e1 n s l @ s ) ((pos "OA$" ) ))
( "offences"  n ( @1 f e1 n s i z ) ((pos "Mj%" ) ))
( "offend"  v ( @1 f e1 n d ) ((pos "J0%" ) (subcat "3A" )))
( "offended"  v ( @1 f e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "offender"  n ( @1 f e1 n d @ r ) ((pos "K6%" ) ))
( "offenders"  n ( @1 f e1 n d @ z ) ((pos "Kj%" ) ))
( "offending"  v ( @1 f e1 n d i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "offends"  v ( @1 f e1 n d z ) ((pos "Ja%" ) (subcat "3A" )))
( "offensive"  n ( @1 f e1 n s i v ) ((pos "K6%" "OA%" ) ))
( "offensively"  a ( @1 f e1 n s i v l ii ) ((pos "Pu%" ) ))
( "offensiveness"  n ( @1 f e1 n s i v n @ s ) ((pos "L@%" ) ))
( "offensives"  n ( @1 f e1 n s i v z ) ((pos "Kj%" ) ))
( "offer"  v ( o1 f @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "7A" "12A" "13A" "14" )))
( "offered"  v ( o1 f @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "7A" "12A" "13A" "14" )))
( "offering"  v ( o1 f @ r i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" "7A" "12A" "13A" "14" )))
( "offerings"  n ( o1 f @ r i ng z ) ((pos "Mj%" ) ))
( "offers"  v ( o1 f @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "7A" "12A" "13A" "14" )))
( "offertories"  n ( o1 f @ t r i z ) ((pos "Kj$" ) ))
( "offertory"  n ( o1 f @ t r ii ) ((pos "K8$" ) ))
( "offhand"  j ( o2 f h a1 n d ) ((pos "OA%" "Pu%" ) ))
( "offhanded"  j ( o2 f h a1 n d i d ) ((pos "OA%" "Pu%" ) ))
( "offhandedly"  j ( o2 f h a1 n d i d l ii ) ((pos "OA%" "Pu%" ) ))
( "office"  n ( o1 f i s ) ((pos "K6*" ) ))
( "office-bearer"  n ( o1 f i s - b e@ r @ r ) ((pos "K6%" ) ))
( "office-bearers"  n ( o1 f i s - b e@ r @ z ) ((pos "Kj%" ) ))
( "office-block"  n ( o1 f i s - b l o k ) ((pos "K6%" ) ))
( "office-blocks"  n ( o1 f i s - b l o k s ) ((pos "Kj%" ) ))
( "office-boy"  n ( o1 f i s - b oi ) ((pos "K6%" ) ))
( "office-boys"  n ( o1 f i s - b oi z ) ((pos "Kj%" ) ))
( "office-holder"  n ( o1 f i s - h ou l d @ r ) ((pos "K6%" ) ))
( "office-holders"  n ( o1 f i s - h ou l d @ z ) ((pos "Kj%" ) ))
( "officer"  n ( o1 f i s @ r ) ((pos "K6%" ) ))
( "officers"  n ( o1 f i s @ z ) ((pos "Kj%" ) ))
( "offices"  n ( o1 f i s i z ) ((pos "Kj%" ) ))
( "official"  n ( @1 f i1 sh l ) ((pos "K6%" "OA%" ) ))
( "officialdom"  n ( @1 f i1 sh l d @ m ) ((pos "K6%" ) ))
( "officialdoms"  n ( @1 f i1 sh l d @ m z ) ((pos "Kj$" ) ))
( "officialese"  n ( @1 f i2 sh @ l ii1 z ) ((pos "L@$" ) ))
( "officially"  a ( @1 f i1 sh @ l ii ) ((pos "Pu%" ) ))
( "officials"  n ( @1 f i1 sh l z ) ((pos "Kj%" ) ))
( "officiate"  v ( @1 f i1 sh i ei t ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "officiated"  v ( @1 f i1 sh i ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "officiates"  v ( @1 f i1 sh i ei t s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "officiating"  v ( @1 f i1 sh i ei t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "officious"  j ( @1 f i1 sh @ s ) ((pos "OA%" ) ))
( "officiously"  a ( @1 f i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "officiousness"  n ( @1 f i1 sh @ s n @ s ) ((pos "L@%" ) ))
( "offing"  n ( o1 f i ng ) ((pos "Ki%" ) ))
( "offish"  j ( o1 f i sh ) ((pos "OA$" ) ))
( "offprint"  n ( o1 f p r i n t ) ((pos "K6%" ) ))
( "offprints"  n ( o1 f p r i n t s ) ((pos "Kj%" ) ))
( "offset"  v ( o1 f s e t ) ((pos "H5%" "Hc%" "Hd%" "L@%" ) (subcat "6A" )))
( "offsets"  v ( o1 f s e t s ) ((pos "Ha%" ) (subcat "6A" )))
( "offsetting"  v ( o1 f s e1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "offshoot"  n ( o1 f sh uu t ) ((pos "K6%" ) ))
( "offshoots"  n ( o1 f sh uu t s ) ((pos "Kj%" ) ))
( "offshore"  j ( o2 f sh oo1 r ) ((pos "OA%" ) ))
( "offside"  j ( o1 f s ai1 d ) ((pos "Oq%" "Pu%" ) ))
( "offspring"  n ( o1 f s p r i ng ) ((pos "K9%" ) ))
( "offstage"  j ( o1 f s t ei1 jh ) ((pos "OA%" "Pu%" ) ))
( "oft"  a ( o1 f t ) ((pos "Pu$" ) ))
( "oft-times"  a ( o1 f - t ai m z ) ((pos "Pu$" ) ))
( "often"  a ( o1 f n ) ((pos "Pu*" ) ))
( "oftener"  a ( o1 f n @ r ) ((pos "Pu%" ) ))
( "ogle"  v ( ou1 g l ) ((pos "J2%" ) (subcat "3A" )))
( "ogled"  v ( ou1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "ogles"  v ( ou1 g l z ) ((pos "Ja%" ) (subcat "3A" )))
( "ogling"  v ( ou1 g l i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "ogre"  n ( ou1 g @ r ) ((pos "K6%" ) ))
( "ogreish"  j ( ou1 g @ r i sh ) ((pos "OA$" ) ))
( "ogres"  n ( ou1 g @ z ) ((pos "Kj%" ) ))
( "ogress"  n ( ou1 g r e s ) ((pos "K7$" ) ))
( "ogresses"  n ( ou1 g r e s i z ) ((pos "Kj$" ) ))
( "oh"  uh ( ou1 ) ((pos "W-%" ) ))
( "ohm"  n ( ou1 m ) ((pos "K6$" ) ))
( "ohms"  n ( ou1 m z ) ((pos "Kj$" ) ))
( "oho"  uh ( ou1 h ou1 ) ((pos "W-$" ) ))
( "oil"  v ( oi1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "oil-bearing"  j ( oi1 l - b e@ r i ng ) ((pos "OA%" ) ))
( "oil-burner"  n ( oi1 l - b @@ n @ r ) ((pos "K6%" ) ))
( "oil-burners"  n ( oi1 l - b @@ n @ z ) ((pos "Kj%" ) ))
( "oil-cake"  n ( oi1 l - k ei k ) ((pos "L@$" ) ))
( "oil-colours"  n ( oi1 l - k uh l @ z ) ((pos "Kj%" ) ))
( "oil-painting"  n ( oi1 l - p ei n t i ng ) ((pos "M6%" ) ))
( "oil-paintings"  n ( oi1 l - p ei n t i ng z ) ((pos "Mj%" ) ))
( "oil-palm"  n ( oi1 l - p aa m ) ((pos "K6%" ) ))
( "oil-palms"  n ( oi1 l - p aa m z ) ((pos "Kj%" ) ))
( "oil-paper"  n ( oi1 l - p ei p @ r ) ((pos "L@$" ) ))
( "oil-rig"  n ( oi1 l - r i g ) ((pos "K6%" ) ))
( "oil-rigs"  n ( oi1 l - r i g z ) ((pos "Kj%" ) ))
( "oil-silk"  n ( oi1 l - s i l k ) ((pos "L@$" ) ))
( "oil-slick"  n ( oi1 l - s l i k ) ((pos "K6%" ) ))
( "oil-slicks"  n ( oi1 l - s l i k s ) ((pos "Kj%" ) ))
( "oil-tanker"  n ( oi1 l - t a ng k @ r ) ((pos "K6%" ) ))
( "oil-tankers"  n ( oi1 l - t a ng k @ z ) ((pos "Kj%" ) ))
( "oil-well"  n ( oi1 l - w e l ) ((pos "K6%" ) ))
( "oil-wells"  n ( oi1 l - w e l z ) ((pos "Kj%" ) ))
( "oilcan"  n ( oi1 l k a n ) ((pos "K6%" ) ))
( "oilcans"  n ( oi1 l k a n z ) ((pos "Kj%" ) ))
( "oilcloth"  n ( oi1 l k l o th ) ((pos "L@%" ) ))
( "oiled"  v ( oi1 l d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "oiler"  n ( oi1 l @ r ) ((pos "K6$" ) ))
( "oilers"  n ( oi1 l @ z ) ((pos "Kj$" ) ))
( "oilfield"  n ( oi1 l f ii l d ) ((pos "K6%" ) ))
( "oilfields"  n ( oi1 l f ii l d z ) ((pos "Kj%" ) ))
( "oilfired"  j ( oi1 l f ai @ d ) ((pos "OA%" ) ))
( "oilier"  j ( oi1 l i@ r ) ((pos "Or%" ) ))
( "oiliest"  j ( oi1 l i i s t ) ((pos "Os%" ) ))
( "oiliness"  n ( oi1 l i n @ s ) ((pos "L@%" ) ))
( "oiling"  v ( oi1 l i ng ) ((pos "Hb%" ) (subcat )))
( "oils"  v ( oi1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "oilskin"  n ( oi1 l s k i n ) ((pos "M6%" ) ))
( "oilskins"  n ( oi1 l s k i n z ) ((pos "Mj%" ) ))
( "oily"  j ( oi1 l ii ) ((pos "OD%" ) ))
( "ointment"  n ( oi1 n t m @ n t ) ((pos "M6%" ) ))
( "ointments"  n ( oi1 n t m @ n t s ) ((pos "Mj%" ) ))
( "okapi"  n ( ou1 k aa1 p ii ) ((pos "K6$" ) ))
( "okapis"  n ( ou1 k aa1 p i z ) ((pos "Kj$" ) ))
( "okay"  v ( ou2 k ei1 ) ((pos "H0%" "Ki%" "OA%" "Pu%" ) (subcat )))
( "okayed"  v ( ou2 k ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "okaying"  v ( ou2 k ei1 i ng ) ((pos "Hb$" ) (subcat )))
( "okays"  v ( ou2 k ei1 z ) ((pos "Ha%" ) (subcat )))
( "okra"  n ( ou1 k r @ ) ((pos "L@$" ) ))
( "old"  n ( ou1 l d ) ((pos "L@%" "OC*" ) ))
( "old-fashioned"  j ( ou1 l - f a1 sh @ n d ) ((pos "OA%" ) ))
( "old-maidish"  j ( ou1 l - m ei1 d i sh ) ((pos "OA%" ) ))
( "old-time"  j ( ou1 l - t ai m ) ((pos "OA%" ) ))
( "old-timer"  n ( ou1 l - t ai1 m @ r ) ((pos "K6%" ) ))
( "old-timers"  n ( ou1 l - t ai1 m @ z ) ((pos "Kj%" ) ))
( "old-womanish"  j ( ou1 l - w u1 m @ n i sh ) ((pos "OA%" ) ))
( "old-world"  j ( ou1 l d - w @@ l d ) ((pos "OA%" ) ))
( "olden"  j ( ou1 l d @ n ) ((pos "OA%" ) ))
( "older"  j ( ou1 l d @ r ) ((pos "Or%" ) ))
( "oldest"  j ( ou1 l d i s t ) ((pos "Os%" ) ))
( "oldish"  j ( ou1 l d i sh ) ((pos "OA%" ) ))
( "oldster"  n ( ou1 l d s t @ r ) ((pos "K6%" ) ))
( "oldsters"  n ( ou1 l d s t @ z ) ((pos "Kj%" ) ))
( "oleaginous"  j ( ou2 l i a1 jh i n @ s ) ((pos "OA$" ) ))
( "oleander"  n ( ou2 l i a1 n d @ r ) ((pos "K6$" ) ))
( "oleanders"  n ( ou2 l i a1 n d @ z ) ((pos "Kj$" ) ))
( "olfactory"  j ( o1 l f a1 k t @ r ii ) ((pos "OA$" ) ))
( "oligarch"  n ( o1 l i g aa k ) ((pos "K6$" ) ))
( "oligarchies"  n ( o1 l i g aa k i z ) ((pos "Mj$" ) ))
( "oligarchs"  n ( o1 l i g aa k s ) ((pos "Kj$" ) ))
( "oligarchy"  n ( o1 l i g aa k ii ) ((pos "M8$" ) ))
( "olive"  n ( o1 l i v ) ((pos "K6%" "OA%" ) ))
( "olive-tree"  n ( o1 l i v - t r ii ) ((pos "K6%" ) ))
( "olive-trees"  n ( o1 l i v - t r ii z ) ((pos "Kj%" ) ))
( "olives"  n ( o1 l i v z ) ((pos "Kj%" ) ))
( "ombudsman"  n ( o1 m b u d z m a n ) ((pos "Ki%" ) ))
( "ombudsmen"  n ( o1 m b u d z m e n ) ((pos "Kj%" ) ))
( "omega"  n ( ou1 m i g @ ) ((pos "K6%" ) ))
( "omegas"  n ( ou1 m i g @ z ) ((pos "Kj$" ) ))
( "omelet"  n ( o1 m l i t ) ((pos "K6$" ) ))
( "omelets"  n ( o1 m l i t s ) ((pos "Kj$" ) ))
( "omelette"  n ( o1 m l i t ) ((pos "K6%" ) ))
( "omelettes"  n ( o1 m l i t s ) ((pos "Kj%" ) ))
( "omen"  v ( ou1 m e n ) ((pos "H0$" "M6%" ) (subcat )))
( "omened"  v ( ou1 m e n d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "omening"  v ( ou1 m e n i ng ) ((pos "Hb$" ) (subcat )))
( "omens"  v ( ou1 m e n z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "ominous"  j ( o1 m i n @ s ) ((pos "OA%" ) ))
( "ominously"  a ( o1 m i n @ s l ii ) ((pos "Pu%" ) ))
( "omission"  n ( @1 m i1 sh @ n ) ((pos "M6%" ) ))
( "omissions"  n ( @1 m i1 sh @ n z ) ((pos "Mj%" ) ))
( "omit"  v ( @1 m i1 t ) ((pos "H4%" ) (subcat "6A" "6C" )))
( "omits"  v ( @1 m i1 t s ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "omitted"  v ( @1 m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "omitting"  v ( @1 m i1 t i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "omnibus"  n ( o1 m n i b @ s ) ((pos "K7%" ) ))
( "omnibuses"  n ( o1 m n i b @ s i z ) ((pos "Kj%" ) ))
( "omnipotence"  n ( o1 m n i1 p @ t @ n s ) ((pos "L@%" ) ))
( "omnipotent"  j ( o1 m n i1 p @ t @ n t ) ((pos "OA%" ) ))
( "omniscience"  n ( o1 m n i1 s i@ n s ) ((pos "L@%" ) ))
( "omniscient"  j ( o1 m n i1 s i@ n t ) ((pos "OA%" ) ))
( "omnivorous"  j ( o1 m n i1 v @ r @ s ) ((pos "OA%" ) ))
( "on"  a ( o n ) ((pos "P+*" "T-*" ) ))
( "on to"  in ( o1 n - t @ ) ((pos "T-%" ) ))
( "on-licence"  n ( o1 n - l ai s n s ) ((pos "K6$" ) ))
( "on-licences"  n ( o1 n - l ai s n s i z ) ((pos "Kj$" ) ))
( "once"  a ( w uh1 n s ) ((pos "Pu*" ) ))
( "oncoming"  n ( o1 n k uh m i ng ) ((pos "Ki%" "OA%" ) ))
( "one"  n ( w uh n ) ((pos "K6%" "OA*" "Qx*" ) ))
( "one-armed"  j ( w uh1 n - aa m d ) ((pos "OA%" ) ))
( "one-eyed"  j ( w uh1 n - ai d ) ((pos "OA%" ) ))
( "one-horse"  j ( w uh1 n - h oo s ) ((pos "OA%" ) ))
( "one-idea'd"  j ( w uh1 n - ai d i@1 d ) ((pos "OA$" ) ))
( "one-sided"  j ( w uh1 n - s ai1 d i d ) ((pos "OA%" ) ))
( "one-step"  n ( w uh1 n - s t e p ) ((pos "K6$" ) ))
( "one-steps"  n ( w uh1 n - s t e p s ) ((pos "Kj$" ) ))
( "one-time"  j ( w uh1 n - t ai m ) ((pos "OA%" ) ))
( "one-upmanship"  n ( w uh1 n - uh1 p m @ n sh i p ) ((pos "L@%" ) ))
( "oneness"  n ( w uh1 n - n i s ) ((pos "L@%" ) ))
( "onerous"  j ( o1 n @ r @ s ) ((pos "OA%" ) ))
( "onerously"  a ( o1 n @ r @ s l ii ) ((pos "Pu%" ) ))
( "ones"  n ( w uh1 n z ) ((pos "Kj%" ) ))
( "oneself"  prp ( w uh1 n s e1 l f ) ((pos "Qx%" ) ))
( "ongoing"  j ( o1 n g ou i ng ) ((pos "OA%" ) ))
( "onion"  n ( uh1 n i@ n ) ((pos "M6%" ) ))
( "onions"  n ( uh1 n i@ n z ) ((pos "Mj%" ) ))
( "onlooker"  n ( o1 n l u k @ r ) ((pos "K6%" ) ))
( "onlookers"  n ( o1 n l u k @ z ) ((pos "Kj%" ) ))
( "only"  j ( ou1 n l ii ) ((pos "OA*" "Pu*" "V-*" ) ))
( "onomatopoeia"  n ( o2 n @ m a2 t @ p i@1 ) ((pos "L@$" ) ))
( "onrush"  n ( o1 n r uh sh ) ((pos "K7%" ) ))
( "onrushes"  n ( o1 n r uh sh i z ) ((pos "Kj$" ) ))
( "onset"  n ( o1 n s e t ) ((pos "K6%" ) ))
( "onsets"  n ( o1 n s e t s ) ((pos "Kj$" ) ))
( "onshore"  j ( o1 n sh oo r ) ((pos "OA%" "Pu%" ) ))
( "onslaught"  n ( o1 n s l oo t ) ((pos "K6%" ) ))
( "onslaughts"  n ( o1 n s l oo t s ) ((pos "Kj%" ) ))
( "onto"  in ( o1 n t @ ) ((pos "T-%" ) ))
( "ontologies"  n ( o1 n t o1 l @ jh i z ) ((pos "Mj$" ) ))
( "ontology"  n ( o1 n t o1 l @ jh ii ) ((pos "M8$" ) ))
( "onus"  n ( ou1 n @ s ) ((pos "Ki%" ) ))
( "onward"  j ( o1 n w @ d ) ((pos "OA%" "Pu%" ) ))
( "onwards"  a ( o1 n w @ d z ) ((pos "Pu%" ) ))
( "onyx"  n ( o1 n i k s ) ((pos "L@$" ) ))
( "oodles"  n ( uu1 d l z ) ((pos "Kj%" ) ))
( "oomph"  n ( u1 m f ) ((pos "L@$" ) ))
( "ooze"  v ( uu1 z ) ((pos "J2%" "L@%" ) (subcat "2C" )))
( "oozed"  v ( uu1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "oozes"  v ( uu1 z i z ) ((pos "Ja%" ) (subcat "2C" )))
( "oozier"  j ( uu1 z i@ r ) ((pos "Or$" ) ))
( "ooziest"  j ( uu1 z i i s t ) ((pos "Os$" ) ))
( "oozing"  v ( uu1 z i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "oozy"  j ( uu1 z ii ) ((pos "OD$" ) ))
( "op"  n ( o1 p ) ((pos "Y~%" ) ))
( "op art"  n ( o1 p - aa t ) ((pos "L@$" ) ))
( "op cit"  n ( o2 p - s i1 t ) ((pos "Y~%" ) ))
( "opacity"  n ( ou1 p a1 s i t ii ) ((pos "L@%" ) ))
( "opal"  n ( ou1 p l ) ((pos "K6%" ) ))
( "opalescent"  j ( ou2 p @ l e1 s n t ) ((pos "OA%" ) ))
( "opals"  n ( ou1 p l z ) ((pos "Kj%" ) ))
( "opaque"  j ( ou1 p ei1 k ) ((pos "OA%" ) ))
( "opaquely"  a ( ou1 p ei1 k l ii ) ((pos "Pu%" ) ))
( "opaqueness"  n ( ou1 p ei1 k n @ s ) ((pos "L@%" ) ))
( "open"  v ( ou1 p @ n ) ((pos "J0*" "L@%" "OA*" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "open-air"  j ( ou1 p @ n - e@ r ) ((pos "Oq%" ) ))
( "open-ended"  j ( ou2 p @ n - e1 n d i d ) ((pos "OA%" ) ))
( "open-eyed"  j ( ou2 p @ n - ai1 d ) ((pos "OA%" ) ))
( "open-handed"  j ( ou2 p @ n - h a1 n d i d ) ((pos "OA%" ) ))
( "open-hearted"  j ( ou2 p @ n - h aa1 t i d ) ((pos "OA%" ) ))
( "open-minded"  j ( ou1 p @ n - m ai1 n d i d ) ((pos "OA%" ) ))
( "open-mouthed"  j ( ou2 p @ n - m au1 dh d ) ((pos "OA%" ) ))
( "opencast"  j ( ou1 p @ n k aa s t ) ((pos "OA%" ) ))
( "opened"  v ( ou1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "opener"  n ( ou1 p @ n @ r ) ((pos "K6%" ) ))
( "openers"  n ( ou1 p @ n @ z ) ((pos "Kj%" ) ))
( "opening"  v ( ou1 p @ n i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "openings"  n ( ou1 p @ n i ng z ) ((pos "Kj%" ) ))
( "openly"  a ( ou1 p @ n l ii ) ((pos "Pu%" ) ))
( "openness"  n ( ou1 p @ n - n @ s ) ((pos "L@%" ) ))
( "opens"  v ( ou1 p @ n z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "15B" )))
( "openwork"  n ( ou1 p @ n w @@ k ) ((pos "L@$" ) ))
( "opera"  n ( o1 p @ r @ ) ((pos "Kj$" ) ))
( "opera"  n ( o1 p r @ ) ((pos "M6%" ) ))
( "opera-cloak"  n ( o1 p @ r @ - k l ou k ) ((pos "K6%" ) ))
( "opera-cloaks"  n ( o1 p @ r @ - k l ou k s ) ((pos "Kj%" ) ))
( "opera-glasses"  n ( o1 p @ r @ - g l aa s i z ) ((pos "Kj%" ) ))
( "opera-hat"  n ( o1 p @ r @ - h a t ) ((pos "K6$" ) ))
( "opera-hats"  n ( o1 p @ r @ - h a t s ) ((pos "Kj$" ) ))
( "opera-house"  n ( o1 p @ r @ - h au s ) ((pos "K6%" ) ))
( "opera-houses"  n ( o1 p @ r @ - h au z i z ) ((pos "Kj%" ) ))
( "operable"  j ( o1 p @ r @ b l ) ((pos "OA%" ) ))
( "operas"  n ( o1 p r @ z ) ((pos "Mj%" ) ))
( "operate"  v ( o1 p @ r ei t ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "4A" )))
( "operated"  v ( o1 p @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" )))
( "operates"  v ( o1 p @ r ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "4A" )))
( "operatic"  j ( o2 p @ r a1 t i k ) ((pos "OA%" ) ))
( "operating"  v ( o1 p @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" )))
( "operating-table"  n ( o1 p @ r ei t i ng - t ei b l ) ((pos "K6%" ) ))
( "operating-tables"  n ( o1 p @ r ei t i ng - t ei b l z ) ((pos "Kj%" ) ))
( "operating-theatre"  n ( o1 p @ r ei t i ng - th i@ t @ r ) ((pos "K6%" ) ))
( "operating-theatres"  n ( o1 p @ r ei t i ng - th i@ t @ z ) ((pos "Kj%" ) ))
( "operation"  n ( o2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "operational"  j ( o2 p @ r ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "operations"  n ( o2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "operative"  n ( o1 p @ r @ t i v ) ((pos "K6%" "OA%" ) ))
( "operatives"  n ( o1 p @ r @ t i v z ) ((pos "Kj%" ) ))
( "operator"  n ( o1 p @ r ei t @ r ) ((pos "K6%" ) ))
( "operators"  n ( o1 p @ r ei t @ z ) ((pos "Kj%" ) ))
( "operetta"  n ( o2 p @ r e1 t @ ) ((pos "K6%" ) ))
( "operettas"  n ( o2 p @ r e1 t @ z ) ((pos "Kj%" ) ))
( "ophthalmia"  n ( o1 f th a1 l m i@ ) ((pos "L@$" ) ))
( "ophthalmic"  j ( o1 f th a1 l m i k ) ((pos "OA%" ) ))
( "ophthalmoscope"  n ( o1 f th a1 l m @ s k ou p ) ((pos "K6%" ) ))
( "ophthalmoscopes"  n ( o1 f th a1 l m @ s k ou p s ) ((pos "Kj%" ) ))
( "opiate"  n ( ou1 p i@ t ) ((pos "K6$" ) ))
( "opiates"  n ( ou1 p i@ t s ) ((pos "Kj$" ) ))
( "opine"  v ( @1 p ai1 n ) ((pos "H2$" ) (subcat )))
( "opined"  v ( @1 p ai1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "opines"  v ( @1 p ai1 n z ) ((pos "Ha$" ) (subcat )))
( "opining"  v ( @1 p ai1 n i ng ) ((pos "Hb$" ) (subcat )))
( "opinion"  n ( @1 p i1 n i@ n ) ((pos "M6%" ) ))
( "opinionated"  j ( @1 p i1 n i@ n ei t i d ) ((pos "OA%" ) ))
( "opinionative"  j ( @1 p i1 n i@ n @ t i v ) ((pos "OA$" ) ))
( "opinions"  n ( @1 p i1 n i@ n z ) ((pos "Mj%" ) ))
( "opium"  n ( ou1 p i@ m ) ((pos "L@%" ) ))
( "opium-den"  n ( ou1 p i@ m - d e n ) ((pos "K6%" ) ))
( "opium-dens"  n ( ou1 p i@ m - d e n z ) ((pos "Kj%" ) ))
( "opossum"  n ( @1 p o1 s @ m ) ((pos "K6$" ) ))
( "opossums"  n ( @1 p o1 s @ m z ) ((pos "Kj$" ) ))
( "opp"  n ( o1 p ) ((pos "Y~$" ) ))
( "opponent"  n ( @1 p ou1 n @ n t ) ((pos "K6%" ) ))
( "opponents"  n ( @1 p ou1 n @ n t s ) ((pos "Kj%" ) ))
( "opportune"  j ( o1 p @ t y uu n ) ((pos "OA%" ) ))
( "opportunely"  a ( o1 p @ t y uu n l ii ) ((pos "Pu%" ) ))
( "opportunism"  n ( o2 p @ t y uu1 n i z @ m ) ((pos "L@%" ) ))
( "opportunist"  n ( o2 p @ t y uu1 n i s t ) ((pos "K6%" ) ))
( "opportunists"  n ( o2 p @ t y uu1 n i s t s ) ((pos "Kj%" ) ))
( "opportunities"  n ( o2 p @ t y uu1 n i t i z ) ((pos "Mj%" ) ))
( "opportunity"  n ( o2 p @ t y uu1 n i t ii ) ((pos "M8%" ) ))
( "oppose"  v ( @1 p ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "opposed"  v ( @1 p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "opposes"  v ( @1 p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "opposing"  v ( @1 p ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "opposite"  n ( o1 p @ z i t ) ((pos "K6%" "OA%" ) ))
( "opposites"  n ( o1 p @ z i t s ) ((pos "Kj%" ) ))
( "opposition"  n ( o2 p @ z i1 sh @ n ) ((pos "L@%" ) ))
( "oppress"  v ( @1 p r e1 s ) ((pos "H1%" ) (subcat )))
( "oppressed"  v ( @1 p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "oppresses"  v ( @1 p r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "oppressing"  v ( @1 p r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "oppression"  n ( @1 p r e1 sh @ n ) ((pos "M6%" ) ))
( "oppressions"  n ( @1 p r e1 sh @ n z ) ((pos "Mj%" ) ))
( "oppressive"  j ( @1 p r e1 s i v ) ((pos "OA%" ) ))
( "oppressively"  a ( @1 p r e1 s i v l ii ) ((pos "Pu%" ) ))
( "oppressor"  n ( @1 p r e1 s @ r ) ((pos "K6%" ) ))
( "oppressors"  n ( @1 p r e1 s @ z ) ((pos "Kj%" ) ))
( "opprobrious"  j ( @1 p r ou1 b r i@ s ) ((pos "OA$" ) ))
( "opprobriously"  a ( @1 p r ou1 b r i@ s l ii ) ((pos "Pu$" ) ))
( "opprobrium"  n ( @1 p r ou1 b r i@ m ) ((pos "L@%" ) ))
( "oppugn"  v ( @1 p y uu1 n ) ((pos "H0$" ) (subcat )))
( "oppugned"  v ( @1 p y uu1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "oppugning"  v ( @1 p y uu1 n i ng ) ((pos "Hb$" ) (subcat )))
( "oppugns"  v ( @1 p y uu1 n z ) ((pos "Ha$" ) (subcat )))
( "opt"  v ( o1 p t ) ((pos "I0%" ) (subcat )))
( "optative"  n ( o1 p t @ t i v ) ((pos "K6$" "OA$" ) ))
( "optatives"  n ( o1 p t @ t i v z ) ((pos "Kj$" ) ))
( "opted"  v ( o1 p t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "optic"  j ( o1 p t i k ) ((pos "OA%" ) ))
( "optical"  j ( o1 p t i k l ) ((pos "OA%" ) ))
( "optically"  a ( o1 p t i k l ii ) ((pos "Pu%" ) ))
( "optician"  n ( o1 p t i1 sh @ n ) ((pos "K6%" ) ))
( "opticians"  n ( o1 p t i1 sh @ n z ) ((pos "Kj%" ) ))
( "optics"  n ( o1 p t i k s ) ((pos "Lk%" ) ))
( "optimal"  j ( o1 p t i m l ) ((pos "OA%" ) ))
( "optimally"  a ( o1 p t i m @ l ii ) ((pos "Pu%" ) ))
( "optimism"  n ( o1 p t i m i z @ m ) ((pos "L@%" ) ))
( "optimist"  n ( o1 p t i m i s t ) ((pos "K6%" ) ))
( "optimistic"  j ( o2 p t i m i1 s t i k ) ((pos "OA%" ) ))
( "optimistically"  a ( o2 p t i m i1 s t i k l ii ) ((pos "Pu%" ) ))
( "optimists"  n ( o1 p t i m i s t s ) ((pos "Kj%" ) ))
( "optimum"  n ( o1 p t i m @ m ) ((pos "K6%" ) ))
( "optimums"  n ( o1 p t i m @ m z ) ((pos "Kj$" ) ))
( "opting"  v ( o1 p t i ng ) ((pos "Ib%" ) (subcat )))
( "option"  n ( o1 p sh @ n ) ((pos "M6%" ) ))
( "optional"  j ( o1 p sh @ n @ l ) ((pos "OA%" ) ))
( "optionally"  a ( o1 p sh @ n @ l ii ) ((pos "Pu%" ) ))
( "options"  n ( o1 p sh @ n z ) ((pos "Mj%" ) ))
( "opts"  v ( o1 p t s ) ((pos "Ia%" ) (subcat )))
( "opulence"  n ( o1 p y u l @ n s ) ((pos "L@%" ) ))
( "opulent"  j ( o1 p y u l @ n t ) ((pos "OA%" ) ))
( "opulently"  a ( o1 p y uu l @ n t l ii ) ((pos "Pu%" ) ))
( "opus"  n ( ou1 p @ s ) ((pos "Ki%" ) ))
( "or"  cc ( oo r ) ((pos "V-*" ) ))
( "oracle"  n ( o1 r @ k l ) ((pos "K6%" ) ))
( "oracles"  n ( o1 r @ k l z ) ((pos "Kj%" ) ))
( "oracular"  j ( @1 r a1 k y u l @ r ) ((pos "OA%" ) ))
( "oral"  n ( oo1 r @ l ) ((pos "K6$" "OA%" ) ))
( "orally"  a ( oo1 r @ l ii ) ((pos "Pu%" ) ))
( "orals"  n ( oo1 r @ l z ) ((pos "Kj$" ) ))
( "orang-outan"  n ( oo1 r a2 ng - uu t a1 n ) ((pos "K6$" ) ))
( "orang-outang"  n ( oo1 r a2 ng - uu t a1 ng ) ((pos "K6$" ) ))
( "orang-outangs"  n ( oo1 r a2 ng - uu t a1 ng z ) ((pos "Kj$" ) ))
( "orang-outans"  n ( oo1 r a2 ng - uu t a1 n z ) ((pos "Kj$" ) ))
( "orang-utan"  n ( oo1 r a2 ng - uu t a1 n ) ((pos "K6$" ) ))
( "orang-utans"  n ( oo1 r a2 ng - uu t a1 n z ) ((pos "Kj$" ) ))
( "orange"  n ( o1 r i n jh ) ((pos "M6%" "OA%" ) ))
( "orangeade"  n ( o2 r i n jh ei1 d ) ((pos "L@%" ) ))
( "oranges"  n ( o1 r i n jh i z ) ((pos "Mj%" ) ))
( "orate"  v ( oo1 r ei1 t ) ((pos "I2$" ) (subcat )))
( "orated"  v ( oo1 r ei1 t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "orates"  v ( oo1 r ei1 t s ) ((pos "Ia$" ) (subcat )))
( "orating"  v ( oo1 r ei1 t i ng ) ((pos "Ib$" ) (subcat )))
( "oration"  n ( oo1 r ei1 sh @ n ) ((pos "K6%" ) ))
( "orations"  n ( oo1 r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "orator"  n ( o1 r @ t @ r ) ((pos "K6%" ) ))
( "oratorical"  j ( o2 r @ t o1 r i k l ) ((pos "OA%" ) ))
( "oratories"  n ( o1 r @ t r i z ) ((pos "Mj%" ) ))
( "oratorio"  n ( o2 r @ t oo1 r i@ @ ) ((pos "M6%" ) ))
( "oratorios"  n ( o2 r @ t oo1 r i@ u z ) ((pos "Mj%" ) ))
( "orators"  n ( o1 r @ t @ z ) ((pos "Kj%" ) ))
( "oratory"  n ( o1 r @ t r ii ) ((pos "M8%" ) ))
( "orb"  n ( oo1 b ) ((pos "K6$" ) ))
( "orbit"  v ( oo1 b i t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "orbital"  j ( oo1 b i t l ) ((pos "OA%" ) ))
( "orbited"  v ( oo1 b i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "orbiting"  v ( oo1 b i t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "orbits"  v ( oo1 b i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "orbs"  n ( oo1 b z ) ((pos "Kj$" ) ))
( "orchard"  n ( oo1 ch @ d ) ((pos "K6%" ) ))
( "orchards"  n ( oo1 ch @ d z ) ((pos "Kj%" ) ))
( "orchestra"  n ( oo1 k i s t r @ ) ((pos "K6%" ) ))
( "orchestral"  j ( oo1 k e1 s t r @ l ) ((pos "OA%" ) ))
( "orchestras"  n ( oo1 k i s t r @ z ) ((pos "Kj%" ) ))
( "orchestrate"  v ( oo1 k i s t r ei t ) ((pos "H2%" ) (subcat )))
( "orchestrated"  v ( oo1 k i s t r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "orchestrates"  v ( oo1 k i s t r ei t s ) ((pos "Ha%" ) (subcat )))
( "orchestrating"  v ( oo1 k i s t r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "orchestration"  n ( oo2 k i s t r ei1 sh @ n ) ((pos "K6%" ) ))
( "orchestrations"  n ( oo2 k i s t r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "orchid"  n ( oo1 k i d ) ((pos "K6%" ) ))
( "orchids"  n ( oo1 k i d z ) ((pos "Kj%" ) ))
( "orchis"  n ( oo1 k i s ) ((pos "K7$" ) ))
( "orchises"  n ( oo1 k i s i z ) ((pos "Kj$" ) ))
( "ordain"  v ( oo1 d ei1 n ) ((pos "H0%" ) (subcat "6A" "9" )))
( "ordained"  v ( oo1 d ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "ordaining"  v ( oo1 d ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "ordains"  v ( oo1 d ei1 n z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "ordeal"  n ( oo1 d ii1 l ) ((pos "K6%" ) ))
( "ordeals"  n ( oo1 d ii1 l z ) ((pos "Kj%" ) ))
( "order"  v ( oo1 d @ r ) ((pos "H0%" "M6*" ) (subcat "6A" "9" "12B" "13B" "15A" "15B" )))
( "order-book"  n ( oo1 d @ - b u k ) ((pos "K6%" ) ))
( "order-books"  n ( oo1 d @ - b u k s ) ((pos "Kj%" ) ))
( "order-form"  n ( oo1 d @ - f oo m ) ((pos "K6%" ) ))
( "order-forms"  n ( oo1 d @ - f oo m z ) ((pos "Kj%" ) ))
( "order-paper"  n ( oo1 d @ - p ei p @ r ) ((pos "K6%" ) ))
( "order-papers"  n ( oo1 d @ - p ei p @ z ) ((pos "Kj%" ) ))
( "ordered"  v ( oo1 d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "12B" "13B" "15A" "15B" )))
( "ordering"  v ( oo1 d @ r i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" "9" "12B" "13B" "15A" "15B" )))
( "orderings"  n ( oo1 d @ r i ng z ) ((pos "Kj$" ) ))
( "orderlies"  n ( oo1 d @ l i z ) ((pos "Kj%" ) ))
( "orderliness"  n ( oo1 d @ l i n @ s ) ((pos "L@%" ) ))
( "orderly"  n ( oo1 d @ l ii ) ((pos "K8%" "OA%" ) ))
( "orders"  v ( oo1 d @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "9" "12B" "13B" "15A" "15B" )))
( "ordinal"  n ( oo1 d i n l ) ((pos "K6$" "OA%" ) ))
( "ordinals"  n ( oo1 d i n l z ) ((pos "Kj$" ) ))
( "ordinance"  n ( oo1 d i n @ n s ) ((pos "K6$" ) ))
( "ordinances"  n ( oo1 d i n @ n s i z ) ((pos "Kj$" ) ))
( "ordinand"  n ( oo2 d i n a1 n d ) ((pos "K6$" ) ))
( "ordinands"  n ( oo2 d i n a1 n d z ) ((pos "Kj$" ) ))
( "ordinarily"  a ( oo1 d i n @ r @ l ii ) ((pos "Pu%" ) ))
( "ordinary"  j ( oo1 d i n r ii ) ((pos "OA%" ) ))
( "ordination"  n ( oo2 d i n ei1 sh @ n ) ((pos "M6%" ) ))
( "ordinations"  n ( oo2 d i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "ordnance"  n ( oo1 d n @ n s ) ((pos "L@%" ) ))
( "ordure"  n ( oo1 d y u@ r ) ((pos "L@$" ) ))
( "ore"  n ( oo1 r ) ((pos "M6%" ) ))
( "ores"  n ( oo1 z ) ((pos "Mj$" ) ))
( "organ"  n ( oo1 g @ n ) ((pos "K6%" ) ))
( "organ-blower"  n ( oo1 g @ n - b l ou @ r ) ((pos "K6%" ) ))
( "organ-blowers"  n ( oo1 g @ n - b l ou @ z ) ((pos "Kj%" ) ))
( "organ-grinder"  n ( oo1 g @ n - g r ai n d @ r ) ((pos "K6%" ) ))
( "organ-grinders"  n ( oo1 g @ n - g r ai n d @ z ) ((pos "Kj%" ) ))
( "organ-loft"  n ( oo1 g @ n - l o f t ) ((pos "K6%" ) ))
( "organ-lofts"  n ( oo1 g @ n - l o f t s ) ((pos "Kj%" ) ))
( "organdie"  n ( oo1 g a1 n d ii ) ((pos "L@$" ) ))
( "organic"  j ( oo1 g a1 n i k ) ((pos "OA%" ) ))
( "organically"  a ( oo1 g a1 n i k l ii ) ((pos "Pu%" ) ))
( "organism"  n ( oo1 g @ n i z @ m ) ((pos "K6%" ) ))
( "organisms"  n ( oo1 g @ n i z @ m z ) ((pos "Kj%" ) ))
( "organist"  n ( oo1 g @ n i s t ) ((pos "K6%" ) ))
( "organists"  n ( oo1 g @ n i s t s ) ((pos "Kj%" ) ))
( "organization"  n ( oo2 g @ n ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "organizational"  j ( oo2 g @ n ai z ei1 sh @ n l ) ((pos "OA%" ) ))
( "organizationally"  a ( oo2 g @ n ai z ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "organizations"  n ( oo2 g @ n ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "organize"  v ( oo1 g @ n ai z ) ((pos "H2%" ) (subcat )))
( "organized"  v ( oo1 g @ n ai z d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "organizer"  n ( oo1 g @ n ai z @ r ) ((pos "K6%" ) ))
( "organizers"  n ( oo1 g @ n ai z @ z ) ((pos "Kj%" ) ))
( "organizes"  v ( oo1 g @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "organizing"  v ( oo1 g @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "organs"  n ( oo1 g @ n z ) ((pos "Kj%" ) ))
( "orgasm"  n ( oo1 g a z @ m ) ((pos "K6%" ) ))
( "orgasms"  n ( oo1 g a z @ m z ) ((pos "Kj%" ) ))
( "orgiastic"  j ( oo2 jh i a1 s t i k ) ((pos "OA%" ) ))
( "orgies"  n ( oo1 jh i z ) ((pos "Kj%" ) ))
( "orgy"  n ( oo1 jh ii ) ((pos "K8%" ) ))
( "oriel"  n ( oo1 r i@ l ) ((pos "K6$" ) ))
( "oriels"  n ( oo1 r i@ l z ) ((pos "Kj$" ) ))
( "orient"  v ( oo1 r i@ n t ) ((pos "H0%" "Ki%" "OA%" ) ))
( "oriental"  n ( oo2 r i e1 n t l ) ((pos "K6%" "OA%" ) ))
( "orientalist"  n ( oo2 r i e1 n t @ l i s t ) ((pos "K6%" ) ))
( "orientalists"  n ( oo2 r i e1 n t @ l i s t s ) ((pos "Kj%" ) ))
( "orientals"  n ( oo2 r i e1 n t l z ) ((pos "Kj%" ) ))
( "orientate"  v ( oo1 r i@ n t ei t ) ((pos "H2%" ) (subcat )))
( "orientated"  v ( oo1 r i@ n t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "orientates"  v ( oo1 r i@ n t ei t s ) ((pos "Ha%" ) (subcat )))
( "orientating"  v ( oo1 r i@ n t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "orientation"  n ( oo2 r i e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "oriented"  v ( oo1 r i@ n t i d ) ((pos "Hc%" "Hd%" ) ))
( "orienting"  v ( oo1 r i@ n t i ng ) ((pos "Hb%" ) ))
( "orients"  v ( oo1 r i@ n t s ) ((pos "Ha%" ) ))
( "orifice"  n ( o1 r i f i s ) ((pos "K6%" ) ))
( "orifices"  n ( o1 r i f i s i z ) ((pos "Kj%" ) ))
( "origin"  n ( o1 r i jh i n ) ((pos "M6%" ) ))
( "original"  n ( @1 r i1 jh @ n l ) ((pos "K6%" "OA%" ) ))
( "originality"  n ( @1 r i2 jh @ n a1 l i t ii ) ((pos "L@%" ) ))
( "originally"  a ( @1 r i1 jh @ n @ l ii ) ((pos "Pu%" ) ))
( "originals"  n ( @1 r i1 jh @ n l z ) ((pos "Kj%" ) ))
( "originate"  v ( @1 r i1 jh i n ei t ) ((pos "J2%" ) (subcat "2C" "3A" )))
( "originated"  v ( @1 r i1 jh i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" )))
( "originates"  v ( @1 r i1 jh i n ei t s ) ((pos "Ja%" ) (subcat "2C" "3A" )))
( "originating"  v ( @1 r i1 jh i n ei t i ng ) ((pos "Jb%" ) (subcat "2C" "3A" )))
( "originator"  n ( @1 r i1 jh i n ei t @ r ) ((pos "K6%" ) ))
( "originators"  n ( @1 r i1 jh i n ei t @ z ) ((pos "Kj%" ) ))
( "origins"  n ( o1 r i jh i n z ) ((pos "Mj%" ) ))
( "oriole"  n ( oo1 r i@ u l ) ((pos "K6$" ) ))
( "orioles"  n ( oo1 r i@ u l z ) ((pos "Kj$" ) ))
( "orison"  n ( o1 r i z n ) ((pos "K6$" ) ))
( "orisons"  n ( o1 r i z n z ) ((pos "Kj$" ) ))
( "orlop"  n ( oo1 l o p ) ((pos "K6$" ) ))
( "orlops"  n ( oo1 l o p s ) ((pos "Kj$" ) ))
( "ormolu"  n ( oo1 m @ l uu ) ((pos "M6$" ) ))
( "ormolus"  n ( oo1 m @ l uu z ) ((pos "Mj$" ) ))
( "ornament"  n ( oo1 n @ m @ n t ) ((pos "M6%" ) ))
( "ornament"  v ( oo1 n @ m e n t ) ((pos "H0%" ) (subcat "6A" )))
( "ornamental"  j ( oo2 n @ m e1 n t l ) ((pos "OA%" ) ))
( "ornamentation"  n ( oo2 n @ m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "ornamented"  v ( oo1 n @ m e n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "ornamenting"  v ( oo1 n @ m e n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ornaments"  n ( oo1 n @ m @ n t s ) ((pos "Mj%" ) ))
( "ornaments"  v ( oo1 n @ m e n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "ornate"  j ( oo1 n ei1 t ) ((pos "OA%" ) ))
( "ornately"  a ( oo1 n ei1 t l ii ) ((pos "Pu%" ) ))
( "ornateness"  n ( oo1 n ei1 t n @ s ) ((pos "L@%" ) ))
( "ornery"  j ( oo1 n @ r ii ) ((pos "OA$" ) ))
( "ornithological"  j ( oo2 n i th @ l o1 jh i k l ) ((pos "OA%" ) ))
( "ornithologist"  n ( oo2 n i th o1 l @ jh i s t ) ((pos "K6%" ) ))
( "ornithologists"  n ( oo2 n i th o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "ornithology"  n ( oo2 n i th o1 l @ jh ii ) ((pos "L@%" ) ))
( "orotund"  j ( oo1 r ou t uh n d ) ((pos "OA$" ) ))
( "orphan"  v ( oo1 f n ) ((pos "H0%" "K6%" ) (subcat )))
( "orphanage"  n ( oo1 f @ n i jh ) ((pos "K6%" ) ))
( "orphanages"  n ( oo1 f @ n i jh i z ) ((pos "Kj%" ) ))
( "orphaned"  v ( oo1 f n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "orphaning"  v ( oo1 f n i ng ) ((pos "Hb$" ) (subcat )))
( "orphans"  v ( oo1 f n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "orrisroot"  n ( o1 r i s r uu t ) ((pos "L@$" ) ))
( "orthodontic"  j ( oo2 th @ d o1 n t i k ) ((pos "OA%" ) ))
( "orthodontics"  n ( oo2 th @ d o1 n t i k s ) ((pos "Lk$" ) ))
( "orthodontist"  n ( oo2 th @ d o1 n t i s t ) ((pos "K6%" ) ))
( "orthodontists"  n ( oo2 th @ d o1 n t i s t s ) ((pos "Kj$" ) ))
( "orthodox"  j ( oo1 th @ d o k s ) ((pos "OA%" ) ))
( "orthodoxies"  n ( oo1 th @ d o k s i z ) ((pos "Mj%" ) ))
( "orthodoxy"  n ( oo1 th @ d o k s ii ) ((pos "M8%" ) ))
( "orthogonal"  j ( oo1 th o1 g @ n l ) ((pos "OA%" ) ))
( "orthogonally"  a ( oo1 th o1 g @ n @ l ii ) ((pos "Pu%" ) ))
( "orthographic"  j ( oo2 th @ g r a1 f i k ) ((pos "OA%" ) ))
( "orthographies"  n ( oo1 th o1 g r @ f i z ) ((pos "Mj%" ) ))
( "orthography"  n ( oo1 th o1 g r @ f ii ) ((pos "M8%" ) ))
( "orthopaedic"  j ( oo2 th @ p ii1 d i k ) ((pos "OA%" ) ))
( "orthopaedics"  n ( oo2 th @ p ii1 d i k s ) ((pos "Lk%" ) ))
( "orthopedic"  j ( oo2 th @ p ii1 d i k ) ((pos "OA%" ) ))
( "orthopedics"  n ( oo2 th @ p ii1 d i k s ) ((pos "Lk%" ) ))
( "ortolan"  n ( oo1 t @ l @ n ) ((pos "K6$" ) ))
( "ortolans"  n ( oo1 t @ l @ n z ) ((pos "Kj$" ) ))
( "oryx"  n ( o1 r i k s ) ((pos "K7$" ) ))
( "oryxes"  n ( o1 r i k s i z ) ((pos "Kj$" ) ))
( "oscillate"  v ( o1 s i l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "oscillated"  v ( o1 s i l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "oscillates"  v ( o1 s i l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "oscillating"  v ( o1 s i l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "oscillation"  n ( o2 s i l ei1 sh @ n ) ((pos "M6%" ) ))
( "oscillations"  n ( o2 s i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "oscillator"  n ( o1 s i l ei t @ r ) ((pos "K6%" ) ))
( "oscillators"  n ( o1 s i l ei t @ z ) ((pos "Kj%" ) ))
( "oscillograph"  n ( @1 s i1 l @ g r aa f ) ((pos "K6$" ) ))
( "oscillographs"  n ( @1 s i1 l @ g r aa f s ) ((pos "Kj$" ) ))
( "oscilloscope"  n ( @1 s i1 l @ s k ou p ) ((pos "K6%" ) ))
( "oscilloscopes"  n ( @1 s i1 l @ s k ou p s ) ((pos "Kj%" ) ))
( "osier"  n ( ou1 z i@ r ) ((pos "K6$" ) ))
( "osiers"  n ( ou1 z i@ z ) ((pos "Kj$" ) ))
( "osprey"  n ( o1 s p r ii ) ((pos "K6%" ) ))
( "ospreys"  n ( o1 s p r i z ) ((pos "Kj%" ) ))
( "osseous"  j ( o1 s i@ s ) ((pos "OA$" ) ))
( "ossification"  n ( o2 s i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "ossified"  v ( o1 s i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ossifies"  v ( o1 s i f ai z ) ((pos "Ja$" ) (subcat "2A" )))
( "ossify"  v ( o1 s i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "ossifying"  v ( o1 s i f ai i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "ostensible"  j ( o1 s t e1 n s @ b l ) ((pos "OA%" ) ))
( "ostensibly"  a ( o1 s t e1 n s @ b l ii ) ((pos "Pu%" ) ))
( "ostentation"  n ( o2 s t e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "ostentatious"  j ( o2 s t e n t ei1 sh @ s ) ((pos "OA%" ) ))
( "ostentatiously"  a ( o2 s t e n t ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "osteopath"  n ( o1 s t i@ p a th ) ((pos "K6%" ) ))
( "osteopaths"  n ( o1 s t i@ p a th s ) ((pos "Kj%" ) ))
( "osteopathy"  n ( o2 s t i o1 p @ th ii ) ((pos "L@%" ) ))
( "ostler"  n ( o1 s l @ r ) ((pos "K6$" ) ))
( "ostlers"  n ( o1 s l @ z ) ((pos "Kj$" ) ))
( "ostracism"  n ( o1 s t r @ s i z @ m ) ((pos "L@%" ) ))
( "ostracize"  v ( o1 s t r @ s ai z ) ((pos "H2%" ) (subcat )))
( "ostracized"  v ( o1 s t r @ s ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ostracizes"  v ( o1 s t r @ s ai z i z ) ((pos "Ha%" ) (subcat )))
( "ostracizing"  v ( o1 s t r @ s ai z i ng ) ((pos "Hb%" ) (subcat )))
( "ostrich"  n ( o1 s t r i ch ) ((pos "K7%" ) ))
( "ostriches"  n ( o1 s t r i ch i z ) ((pos "Kj%" ) ))
( "other"  n ( uh1 dh @ r ) ((pos "K6*" "OA*" "Pu%" ) ))
( "others"  n ( uh1 dh @ z ) ((pos "Kj*" ) ))
( "otherwise"  a ( uh1 dh @ w ai z ) ((pos "Pu%" "V-%" ) ))
( "otherworldly"  j ( uh2 dh @ w @@1 l d l ii ) ((pos "OA%" ) ))
( "otiose"  j ( ou1 sh i@ u s ) ((pos "OA$" ) ))
( "otter"  n ( o1 t @ r ) ((pos "M6%" ) ))
( "otters"  n ( o1 t @ z ) ((pos "Mj%" ) ))
( "ottoman"  n ( o1 t @ m @ n ) ((pos "K6$" ) ))
( "ottomans"  n ( o1 t @ m @ n z ) ((pos "Kj$" ) ))
( "oubliette"  n ( uu2 b l i e1 t ) ((pos "K6$" ) ))
( "oubliettes"  n ( uu2 b l i e1 t s ) ((pos "Kj$" ) ))
( "ouch"  uh ( au1 ch ) ((pos "W-%" ) ))
( "ought"  v ( oo t ) ((pos "G5%" ) (subcat )))
( "oughtn't"  v ( oo1 t n t ) ((pos "Gg%" ) ))
( "ouija"  n ( w ii1 jh @ ) ((pos "K6$" ) ))
( "ouija-board"  n ( w ii1 jh @ - b oo d ) ((pos "K6%" ) ))
( "ouija-boards"  n ( w ii1 jh @ - b oo d z ) ((pos "Kj%" ) ))
( "ouijas"  n ( w ii1 jh @ z ) ((pos "Kj$" ) ))
( "ounce"  n ( au1 n s ) ((pos "K6%" ) ))
( "ounces"  n ( au1 n s i z ) ((pos "Kj%" ) ))
( "our"  j ( au1 @ r ) ((pos "OA*" ) ))
( "ours"  j ( au1 @ z ) ((pos "Op%" "Qx%" ) ))
( "ourselves"  prp ( au1 @ s e1 l v z ) ((pos "Qx%" ) ))
( "oust"  v ( au1 s t ) ((pos "H0$" ) (subcat "6A" )))
( "ousted"  v ( au1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "ousting"  v ( au1 s t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "ousts"  v ( au1 s t s ) ((pos "Ha$" ) (subcat "6A" )))
( "out"  v ( au1 t ) ((pos "H0%" "P+*" ) ))
( "out-herod"  v ( au2 t - h e1 r @ d ) ((pos "H0$" ) (subcat )))
( "out-heroded"  v ( au2 t - h e1 r @ d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "out-heroding"  v ( au2 t - h e1 r @ d i ng ) ((pos "Hb$" ) (subcat )))
( "out-herods"  v ( au2 t - h e1 r @ d z ) ((pos "Ha$" ) (subcat )))
( "out-of-date"  j ( au2 t - @ v - d ei1 t ) ((pos "OA%" ) ))
( "out-of-door"  j ( au2 t - @ v - d oo1 r ) ((pos "Oq$" ) ))
( "out-of-doors"  a ( au2 t - @ v - d oo1 z ) ((pos "Pu$" ) ))
( "out-of-the-way"  j ( au2 t - @ v - dh @ - w ei1 ) ((pos "OA%" "Pu%" ) ))
( "out-of-work"  j ( au1 t - @ v - w @@ k ) ((pos "OA%" ) ))
( "out-tray"  n ( au1 t - t r ei ) ((pos "K6%" ) ))
( "out-trays"  n ( au1 t - t r ei z ) ((pos "Kj%" ) ))
( "outback"  n ( au1 t b a k ) ((pos "Ki%" "OA%" ) ))
( "outbalance"  v ( au1 t b a1 l @ n s ) ((pos "H2$" ) (subcat )))
( "outbalanced"  v ( au1 t b a1 l @ n s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outbalances"  v ( au1 t b a1 l @ n s i z ) ((pos "Ha$" ) (subcat )))
( "outbalancing"  v ( au1 t b a1 l @ n s i ng ) ((pos "Hb$" ) (subcat )))
( "outbid"  v ( au1 t b i1 d ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "outbidding"  v ( au1 t b i1 d i ng ) ((pos "Hb%" ) (subcat )))
( "outbids"  v ( au1 t b i1 d z ) ((pos "Ha%" ) (subcat )))
( "outboard"  j ( au1 t b oo d ) ((pos "Oq%" ) ))
( "outbound"  j ( au1 t b au n d ) ((pos "OA$" ) ))
( "outbrave"  v ( au1 t b r ei1 v ) ((pos "H2$" ) (subcat )))
( "outbraved"  v ( au1 t b r ei1 v d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outbraves"  v ( au1 t b r ei1 v z ) ((pos "Ha$" ) (subcat )))
( "outbraving"  v ( au1 t b r ei1 v i ng ) ((pos "Hb$" ) (subcat )))
( "outbreak"  n ( au1 t b r ei k ) ((pos "K6%" ) ))
( "outbreaks"  n ( au1 t b r ei k s ) ((pos "Kj%" ) ))
( "outbuilding"  n ( au1 t b i l d i ng ) ((pos "K6%" ) ))
( "outbuildings"  n ( au1 t b i l d i ng z ) ((pos "Kj%" ) ))
( "outburst"  n ( au1 t b @@ s t ) ((pos "K6%" ) ))
( "outbursts"  n ( au1 t b @@ s t s ) ((pos "Kj%" ) ))
( "outcast"  n ( au1 t k aa s t ) ((pos "K6%" "OA%" ) ))
( "outcaste"  n ( au1 t k aa s t ) ((pos "K6$" "OA$" ) ))
( "outcastes"  n ( au1 t k aa s t s ) ((pos "Kj$" ) ))
( "outcasts"  n ( au1 t k aa s t s ) ((pos "Kj%" ) ))
( "outclass"  v ( au1 t k l aa1 s ) ((pos "H1$" ) (subcat )))
( "outclassed"  v ( au1 t k l aa1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outclasses"  v ( au1 t k l aa1 s i z ) ((pos "Ha$" ) (subcat )))
( "outclassing"  v ( au1 t k l aa1 s i ng ) ((pos "Hb$" ) (subcat )))
( "outcome"  n ( au1 t k uh m ) ((pos "K6%" ) ))
( "outcomes"  n ( au1 t k uh m z ) ((pos "Kj%" ) ))
( "outcries"  n ( au1 t k r ai z ) ((pos "Mj%" ) ))
( "outcrop"  n ( au1 t k r o p ) ((pos "K6%" ) ))
( "outcrops"  n ( au1 t k r o p s ) ((pos "Kj%" ) ))
( "outcry"  n ( au1 t k r ai ) ((pos "M8%" ) ))
( "outdated"  j ( au1 t d ei1 t i d ) ((pos "OA%" ) ))
( "outdid"  v ( au1 t d i1 d ) ((pos "Hc%" ) (subcat )))
( "outdistance"  v ( au1 t d i1 s t @ n s ) ((pos "H2%" ) (subcat )))
( "outdistanced"  v ( au1 t d i1 s t @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outdistances"  v ( au1 t d i1 s t @ n s i z ) ((pos "Ha%" ) (subcat )))
( "outdistancing"  v ( au1 t d i1 s t @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "outdo"  v ( au1 t d uu1 ) ((pos "H5%" ) (subcat )))
( "outdoes"  v ( au1 t d uh1 z ) ((pos "Ha%" ) (subcat )))
( "outdoing"  v ( au1 t d uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "outdone"  v ( au1 t d uh1 n ) ((pos "Hd%" ) (subcat )))
( "outdoor"  j ( au1 t d oo r ) ((pos "Oq%" ) ))
( "outdoors"  a ( au2 t d oo1 z ) ((pos "Pu%" ) ))
( "outed"  v ( au1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "outer"  j ( au1 t @ r ) ((pos "OA%" ) ))
( "outermost"  j ( au1 t @ m ou s t ) ((pos "OA%" ) ))
( "outface"  v ( au2 t f ei1 s ) ((pos "H2%" ) (subcat )))
( "outfaced"  v ( au2 t f ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outfaces"  v ( au2 t f ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "outfacing"  v ( au2 t f ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "outfall"  n ( au1 t f oo l ) ((pos "K6%" ) ))
( "outfalls"  n ( au1 t f oo l z ) ((pos "Kj%" ) ))
( "outfield"  n ( au1 t f ii l d ) ((pos "K6%" ) ))
( "outfielder"  n ( au1 t f ii l d @ r ) ((pos "K6%" ) ))
( "outfielders"  n ( au1 t f ii l d @ z ) ((pos "Kj%" ) ))
( "outfields"  n ( au1 t f ii l d z ) ((pos "Kj$" ) ))
( "outfight"  v ( au2 t f ai1 t ) ((pos "H5$" ) (subcat )))
( "outfighting"  v ( au2 t f ai1 t i ng ) ((pos "Hb$" ) (subcat )))
( "outfights"  v ( au2 t f ai1 t s ) ((pos "Ha$" ) (subcat )))
( "outfit"  v ( au1 t f i t ) ((pos "H4%" "K6%" ) ))
( "outfits"  v ( au1 t f i t s ) ((pos "Ha%" "Kj%" ) ))
( "outfitted"  v ( au1 t f i t i d ) ((pos "Hc%" "Hd%" ) ))
( "outfitter"  n ( au1 t f i t @ r ) ((pos "K6%" ) ))
( "outfitters"  n ( au1 t f i t @ z ) ((pos "Kj%" ) ))
( "outfitting"  v ( au1 t f i t i ng ) ((pos "Hb%" ) ))
( "outflank"  v ( au2 t f l a1 ng k ) ((pos "H0%" ) (subcat )))
( "outflanked"  v ( au2 t f l a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outflanking"  v ( au2 t f l a1 ng k i ng ) ((pos "Hb%" ) (subcat )))
( "outflanks"  v ( au2 t f l a1 ng k s ) ((pos "Ha%" ) (subcat )))
( "outflow"  n ( au1 t f l ou ) ((pos "K6%" ) ))
( "outflows"  n ( au1 t f l ou z ) ((pos "Kj%" ) ))
( "outfought"  v ( au2 t f oo1 t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outfox"  v ( au2 t f o1 k s ) ((pos "H1$" ) (subcat )))
( "outfoxed"  v ( au2 t f o1 k s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outfoxes"  v ( au2 t f o1 k s i z ) ((pos "Ha$" ) (subcat )))
( "outfoxing"  v ( au2 t f o1 k s i ng ) ((pos "Hb$" ) (subcat )))
( "outgo"  v ( au1 t g ou ) ((pos "I5$" "K7$" ) ))
( "outgoes"  v ( au1 t g ou z ) ((pos "Ia$" "Kj$" ) ))
( "outgoing"  j ( au1 t g ou i ng ) ((pos "OA%" ) ))
( "outgoings"  n ( au1 t g ou i ng z ) ((pos "Kj%" ) ))
( "outgone"  v ( au1 t g o1 n ) ((pos "Id$" ) ))
( "outgrew"  v ( au2 t g r uu1 ) ((pos "Hc%" ) (subcat )))
( "outgrow"  v ( au2 t g r ou1 ) ((pos "H5%" ) (subcat )))
( "outgrowing"  v ( au2 t g r ou1 i ng ) ((pos "Hb%" ) (subcat )))
( "outgrown"  v ( au2 t g r ou1 n ) ((pos "Hd%" ) (subcat )))
( "outgrows"  v ( au2 t g r ou1 z ) ((pos "Ha%" ) (subcat )))
( "outgrowth"  n ( au1 t g r ou th ) ((pos "K6%" ) ))
( "outgrowths"  n ( au1 t g r ou th s ) ((pos "Kj%" ) ))
( "outhouse"  n ( au1 t h au s ) ((pos "K6%" ) ))
( "outhouses"  n ( au1 t h au z i z ) ((pos "Kj%" ) ))
( "outing"  v ( au1 t i ng ) ((pos "Hb%" "K6%" ) ))
( "outings"  n ( au1 t i ng z ) ((pos "Kj%" ) ))
( "outlandish"  j ( au1 t l a1 n d i sh ) ((pos "OA%" ) ))
( "outlandishly"  a ( au1 t l a1 n d i sh l ii ) ((pos "Pu%" ) ))
( "outlandishness"  n ( au1 t l a1 n d i sh n @ s ) ((pos "L@%" ) ))
( "outlast"  v ( au2 t l aa1 s t ) ((pos "H0%" ) (subcat )))
( "outlasted"  v ( au2 t l aa1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outlasting"  v ( au2 t l aa1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "outlasts"  v ( au2 t l aa1 s t s ) ((pos "Ha%" ) (subcat )))
( "outlaw"  v ( au1 t l oo ) ((pos "H0%" "K6%" ) (subcat )))
( "outlawed"  v ( au1 t l oo d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outlawing"  v ( au1 t l oo i ng ) ((pos "Hb$" ) (subcat )))
( "outlawry"  n ( au1 t l oo r ii ) ((pos "L@$" ) ))
( "outlaws"  v ( au1 t l oo z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "outlay"  n ( au1 t l ei ) ((pos "M6%" ) ))
( "outlays"  n ( au1 t l ei z ) ((pos "Mj%" ) ))
( "outlet"  n ( au1 t l e t ) ((pos "K6%" ) ))
( "outlets"  n ( au1 t l e t s ) ((pos "Kj%" ) ))
( "outlier"  n ( au1 t l ai @ r ) ((pos "K6%" ) ))
( "outliers"  n ( au1 t l ai @ z ) ((pos "Kj%" ) ))
( "outline"  v ( au1 t l ai n ) ((pos "H2%" "K6%" ) (subcat )))
( "outlined"  v ( au1 t l ai n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outlines"  v ( au1 t l ai n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "outlining"  v ( au1 t l ai n i ng ) ((pos "Hb%" ) (subcat )))
( "outlive"  v ( au2 t l i1 v ) ((pos "H2%" ) (subcat )))
( "outlived"  v ( au2 t l i1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outlives"  v ( au2 t l i1 v z ) ((pos "Ha%" ) (subcat )))
( "outliving"  v ( au2 t l i1 v i ng ) ((pos "Hb%" ) (subcat )))
( "outlook"  n ( au1 t l u k ) ((pos "K6%" ) ))
( "outlooks"  n ( au1 t l u k s ) ((pos "Kj%" ) ))
( "outlying"  j ( au1 t l ai i ng ) ((pos "OA%" ) ))
( "outmanoeuvre"  v ( au2 t m @ n uu1 v @ r ) ((pos "H2%" ) (subcat )))
( "outmanoeuvred"  v ( au2 t m @ n uu1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outmanoeuvres"  v ( au2 t m @ n uu1 v @ z ) ((pos "Ha%" ) (subcat )))
( "outmanoeuvring"  v ( au2 t m @ n uu1 v @ r i ng ) ((pos "Hb%" ) (subcat )))
( "outmarch"  v ( au2 t m aa1 ch ) ((pos "H1$" ) (subcat )))
( "outmarched"  v ( au2 t m aa1 ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outmarches"  v ( au2 t m aa1 ch i z ) ((pos "Ha$" ) (subcat )))
( "outmarching"  v ( au2 t m aa1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "outmatch"  v ( au2 t m a1 ch ) ((pos "H1$" ) (subcat )))
( "outmatched"  v ( au2 t m a1 ch t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "outmatches"  v ( au2 t m a1 ch i z ) ((pos "Ha$" ) (subcat )))
( "outmatching"  v ( au2 t m a1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "outmoded"  j ( au2 t m ou1 d i d ) ((pos "OA%" ) ))
( "outmost"  j ( au1 t m ou s t ) ((pos "OA$" ) ))
( "outnumber"  v ( au2 t n uh1 m b @ r ) ((pos "H0%" ) (subcat )))
( "outnumbered"  v ( au2 t n uh1 m b @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outnumbering"  v ( au2 t n uh1 m b @ r i ng ) ((pos "Hb%" ) (subcat )))
( "outnumbers"  v ( au2 t n uh1 m b @ z ) ((pos "Ha%" ) (subcat )))
( "outpatient"  n ( au1 t p ei sh @ n t ) ((pos "K6%" ) ))
( "outpatients"  n ( au1 t p ei sh @ n t s ) ((pos "Kj%" ) ))
( "outplay"  v ( au2 t p l ei1 ) ((pos "H0%" ) (subcat )))
( "outplayed"  v ( au2 t p l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outplaying"  v ( au2 t p l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "outplays"  v ( au2 t p l ei1 z ) ((pos "Ha%" ) (subcat )))
( "outpoint"  v ( au2 t p oi1 n t ) ((pos "H0$" ) (subcat )))
( "outpointed"  v ( au2 t p oi1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "outpointing"  v ( au2 t p oi1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "outpoints"  v ( au2 t p oi1 n t s ) ((pos "Ha$" ) (subcat )))
( "outport"  n ( au1 t p oo t ) ((pos "K6$" ) ))
( "outports"  n ( au1 t p oo t s ) ((pos "Kj$" ) ))
( "outpost"  n ( au1 t p ou s t ) ((pos "K6%" ) ))
( "outposts"  n ( au1 t p ou s t s ) ((pos "Kj%" ) ))
( "outpouring"  n ( au1 t p oo r i ng ) ((pos "K6%" ) ))
( "outpourings"  n ( au1 t p oo r i ng z ) ((pos "Kj%" ) ))
( "output"  n ( au1 t p u t ) ((pos "Ki%" ) ))
( "outr_e"  j ( uu1 t r ei ) ((pos "OA$" ) ))
( "outrage"  v ( au1 t r ei jh ) ((pos "H2$" "M6%" ) (subcat )))
( "outraged"  v ( au1 t r ei jh d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "outrageous"  j ( au1 t r ei1 jh @ s ) ((pos "OA%" ) ))
( "outrageously"  a ( au1 t r ei1 jh @ s l ii ) ((pos "Pu%" ) ))
( "outrages"  v ( au1 t r ei jh i z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "outraging"  v ( au1 t r ei jh i ng ) ((pos "Hb$" ) (subcat )))
( "outran"  v ( au2 t r a1 n ) ((pos "Hc%" ) (subcat )))
( "outrange"  v ( au2 t r ei1 n jh ) ((pos "H2$" ) (subcat )))
( "outranged"  v ( au2 t r ei1 n jh d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outranges"  v ( au2 t r ei1 n jh i z ) ((pos "Ha$" ) (subcat )))
( "outranging"  v ( au2 t r ei1 n jh i ng ) ((pos "Hb$" ) (subcat )))
( "outrank"  v ( au2 t r a1 ng k ) ((pos "H0%" ) (subcat )))
( "outranked"  v ( au2 t r a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outranking"  v ( au2 t r a1 ng k i ng ) ((pos "Hb%" ) (subcat )))
( "outranks"  v ( au2 t r a1 ng k s ) ((pos "Ha%" ) (subcat )))
( "outridden"  v ( au2 t r i1 d n ) ((pos "Hd$" ) (subcat )))
( "outride"  v ( au2 t r ai1 d ) ((pos "H5$" ) (subcat )))
( "outrider"  n ( au1 t r ai d @ r ) ((pos "K6%" ) ))
( "outriders"  n ( au1 t r ai d @ z ) ((pos "Kj%" ) ))
( "outrides"  v ( au2 t r ai1 d z ) ((pos "Ha$" ) (subcat )))
( "outriding"  v ( au2 t r ai1 d i ng ) ((pos "Hb$" ) (subcat )))
( "outrigged"  j ( au1 t r i g d ) ((pos "OA$" ) ))
( "outrigger"  n ( au1 t r i g @ r ) ((pos "K6%" ) ))
( "outriggers"  n ( au1 t r i g @ z ) ((pos "Kj%" ) ))
( "outright"  j ( au1 t r ai t ) ((pos "OA%" "Pu%" ) ))
( "outrival"  v ( au2 t r ai1 v @ l ) ((pos "H4$" ) (subcat )))
( "outrivalled"  v ( au2 t r ai1 v @ l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outrivalling"  v ( au2 t r ai1 v @ l i ng ) ((pos "Hb$" ) (subcat )))
( "outrivals"  v ( au2 t r ai1 v @ l z ) ((pos "Ha$" ) (subcat )))
( "outrode"  v ( au2 t r ou1 d ) ((pos "Hc$" ) (subcat )))
( "outrun"  v ( au2 t r uh1 n ) ((pos "H5%" "Hd%" ) (subcat )))
( "outrunning"  v ( au2 t r uh1 n i ng ) ((pos "Hb%" ) (subcat )))
( "outruns"  v ( au2 t r uh1 n z ) ((pos "Ha%" ) (subcat )))
( "outs"  v ( au1 t s ) ((pos "Ha$" ) ))
( "outsail"  v ( au1 t s ei1 l ) ((pos "H0$" ) (subcat )))
( "outsailed"  v ( au1 t s ei1 l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outsailing"  v ( au1 t s ei1 l i ng ) ((pos "Hb$" ) (subcat )))
( "outsails"  v ( au1 t s ei1 l z ) ((pos "Ha$" ) (subcat )))
( "outset"  n ( au1 t s e t ) ((pos "K6%" ) ))
( "outsets"  n ( au1 t s e t s ) ((pos "Kj$" ) ))
( "outshine"  v ( au2 t sh ai1 n ) ((pos "H5%" ) (subcat )))
( "outshines"  v ( au2 t sh ai1 n z ) ((pos "Ha%" ) (subcat )))
( "outshining"  v ( au2 t sh ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "outshone"  v ( au2 t sh o1 n ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outside"  j ( au1 t s ai d ) ((pos "OA*" "Pu*" "T-*" ) ))
( "outside"  n ( au2 t s ai1 d ) ((pos "K6%" ) ))
( "outsider"  n ( au2 t s ai1 d @ r ) ((pos "K6%" ) ))
( "outsiders"  n ( au2 t s ai1 d @ z ) ((pos "Kj%" ) ))
( "outsides"  n ( au2 t s ai1 d z ) ((pos "Kj%" ) ))
( "outsize"  j ( au1 t s ai z ) ((pos "OA%" ) ))
( "outskirts"  n ( au1 t s k @@ t s ) ((pos "Kj%" ) ))
( "outsmart"  v ( au2 t s m aa1 t ) ((pos "H0%" ) (subcat )))
( "outsmarted"  v ( au2 t s m aa1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outsmarting"  v ( au2 t s m aa1 t i ng ) ((pos "Hb$" ) (subcat )))
( "outsmarts"  v ( au2 t s m aa1 t s ) ((pos "Ha%" ) (subcat )))
( "outspan"  v ( au2 t s p a1 n ) ((pos "J4$" ) (subcat "2A" )))
( "outspanned"  v ( au2 t s p a1 n d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "outspanning"  v ( au2 t s p a1 n i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "outspans"  v ( au2 t s p a1 n z ) ((pos "Ja$" ) (subcat "2A" )))
( "outspoken"  j ( au2 t s p ou1 k @ n ) ((pos "OA%" ) ))
( "outspokenly"  a ( au2 t s p ou1 k @ n l ii ) ((pos "Pu%" ) ))
( "outspokenness"  n ( au2 t s p ou1 k @ n - n @ s ) ((pos "L@%" ) ))
( "outspread"  j ( au1 t s p r e d ) ((pos "OA%" ) ))
( "outstanding"  j ( au2 t s t a1 n d i ng ) ((pos "OA%" ) ))
( "outstandingly"  a ( au2 t s t a1 n d i ng l ii ) ((pos "Pu%" ) ))
( "outstation"  n ( au1 t s t ei sh @ n ) ((pos "K6%" ) ))
( "outstations"  n ( au1 t s t ei sh @ n z ) ((pos "Kj%" ) ))
( "outstay"  v ( au2 t s t ei1 ) ((pos "H0%" ) (subcat )))
( "outstayed"  v ( au2 t s t ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outstaying"  v ( au2 t s t ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "outstays"  v ( au2 t s t ei1 z ) ((pos "Ha%" ) (subcat )))
( "outstretched"  j ( au2 t s t r e1 ch t ) ((pos "OA%" ) ))
( "outstrip"  v ( au2 t s t r i1 p ) ((pos "H4%" ) (subcat )))
( "outstripped"  v ( au2 t s t r i1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outstripping"  v ( au2 t s t r i1 p i ng ) ((pos "Hb%" ) (subcat )))
( "outstrips"  v ( au2 t s t r i1 p s ) ((pos "Ha%" ) (subcat )))
( "outvie"  v ( au2 t v ai1 ) ((pos "H5$" ) (subcat )))
( "outvied"  v ( au2 t v ai1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "outvies"  v ( au2 t v ai1 z ) ((pos "Ha$" ) (subcat )))
( "outvote"  v ( au2 t v ou1 t ) ((pos "H2%" ) (subcat )))
( "outvoted"  v ( au2 t v ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outvotes"  v ( au2 t v ou1 t s ) ((pos "Ha%" ) (subcat )))
( "outvoting"  v ( au2 t v ou1 t i ng ) ((pos "Hb%" ) (subcat )))
( "outvying"  v ( au2 t v ai1 i ng ) ((pos "Hb$" ) (subcat )))
( "outward"  j ( au1 t w @ d ) ((pos "OA%" "Pu%" ) ))
( "outwardly"  a ( au1 t w @ d l ii ) ((pos "Pu%" ) ))
( "outwards"  a ( au1 t w @ d z ) ((pos "Pu%" ) ))
( "outwear"  v ( au1 t w e@1 r ) ((pos "H5%" ) (subcat )))
( "outwearing"  v ( au2 t w e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "outwears"  v ( au2 t w e@1 z ) ((pos "Ha%" ) (subcat )))
( "outweigh"  v ( au2 t w ei1 ) ((pos "H0%" ) (subcat )))
( "outweighed"  v ( au2 t w ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outweighing"  v ( au2 t w ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "outweighs"  v ( au2 t w ei1 z ) ((pos "Ha%" ) (subcat )))
( "outwent"  v ( au1 t w e1 n t ) ((pos "Ic$" ) ))
( "outwit"  v ( au2 t w i1 t ) ((pos "H4%" ) (subcat )))
( "outwits"  v ( au2 t w i1 t s ) ((pos "Ha%" ) (subcat )))
( "outwitted"  v ( au2 t w i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "outwitting"  v ( au2 t w i1 t i ng ) ((pos "Hb%" ) (subcat )))
( "outwore"  v ( au2 t w oo1 r ) ((pos "Hc$" ) (subcat )))
( "outwork"  n ( au1 t w @@ k ) ((pos "K6$" ) ))
( "outworks"  n ( au1 t w @@ k s ) ((pos "Kj$" ) ))
( "outworn"  v ( au2 t w oo1 n ) ((pos "Hd%" ) (subcat )))
( "ouzel"  n ( uu1 z l ) ((pos "K6$" ) ))
( "ouzels"  n ( uu1 z l z ) ((pos "Kj$" ) ))
( "ouzo"  n ( uu1 z ou ) ((pos "L@$" ) ))
( "ova"  n ( ou1 v @ ) ((pos "Kj$" ) ))
( "oval"  n ( ou1 v @ l ) ((pos "K6%" "OA%" ) ))
( "ovals"  n ( ou1 v @ l z ) ((pos "Kj%" ) ))
( "ovaries"  n ( ou1 v @ r i z ) ((pos "Kj%" ) ))
( "ovary"  n ( ou1 v @ r ii ) ((pos "K8%" ) ))
( "ovation"  n ( ou1 v ei1 sh @ n ) ((pos "K6%" ) ))
( "ovations"  n ( ou1 v ei1 sh @ n z ) ((pos "Kj%" ) ))
( "oven"  n ( uh1 v n ) ((pos "K6%" ) ))
( "ovens"  n ( uh1 v n z ) ((pos "Kj%" ) ))
( "ovenware"  n ( uh1 v n w e@ r ) ((pos "L@%" ) ))
( "over"  n ( ou1 v @ r ) ((pos "K6%" "Pu*" "T-*" ) ))
( "over-"  j ( ou2 v @ r - ) ((pos "U-%" ) ))
( "over-abundance"  n ( ou2 v @ r - @ b uh1 n d @ n s ) ((pos "L@%" ) ))
( "over-ripe"  j ( ou2 v @ - r ai1 p ) ((pos "OA%" ) ))
( "overabundant"  j ( ou2 v @ r @ b uh1 n d @ n t ) ((pos "OA%" ) ))
( "overact"  v ( ou2 v @ r a1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "overacted"  v ( ou2 v @ r a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overacting"  v ( ou2 v @ r a1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overactive"  j ( ou2 v @ r a1 k t i v ) ((pos "OA%" ) ))
( "overacts"  v ( ou2 v @ r a1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "overall"  n ( ou1 v @ r oo l ) ((pos "K6$" ) ))
( "overall"  j ( ou2 v @ r oo1 l ) ((pos "OA%" ) ))
( "overalls"  n ( ou1 v @ r oo l z ) ((pos "Kj%" ) ))
( "overambitious"  j ( ou2 v @ r a m b i1 sh @ s ) ((pos "OA%" ) ))
( "overanxiety"  n ( ou2 v @ r a ng z ai1 @ t ii ) ((pos "L@%" ) ))
( "overanxious"  j ( ou2 v @ r a1 ng k sh @ s ) ((pos "OA%" ) ))
( "overarch"  v ( ou2 v @ r aa1 ch ) ((pos "J1$" ) (subcat "2A" )))
( "overarched"  v ( ou2 v @ r aa1 ch t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "overarches"  v ( ou2 v @ r aa1 ch i z ) ((pos "Ja$" ) (subcat "2A" )))
( "overarching"  v ( ou2 v @ r aa1 ch i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "overarm"  j ( ou1 v @ r aa m ) ((pos "OA%" "Pu%" ) ))
( "overate"  v ( ou2 v @ r ei1 t ) ((pos "Ic%" ) ))
( "overawe"  v ( ou2 v @ r oo1 ) ((pos "H2%" ) (subcat )))
( "overawed"  v ( ou2 v @ r oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overawes"  v ( ou2 v @ r oo1 z ) ((pos "Ha%" ) (subcat )))
( "overawing"  v ( ou2 v @ r oo1 i ng ) ((pos "Hb$" ) (subcat )))
( "overbalance"  v ( ou2 v @ b a1 l @ n s ) ((pos "J2%" ) (subcat "2A" )))
( "overbalanced"  v ( ou2 v @ b a1 l @ n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overbalances"  v ( ou2 v @ b a1 l @ n s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "overbalancing"  v ( ou2 v @ b a1 l @ n s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overbear"  v ( ou2 v @ b e@1 r ) ((pos "H5$" ) (subcat )))
( "overbearing"  v ( ou2 v @ b e@1 r i ng ) ((pos "Hb$" "OA%" ) (subcat )))
( "overbearingly"  a ( ou2 v @ b e@1 r i ng l ii ) ((pos "Pu%" ) ))
( "overbears"  v ( ou2 v @ b e@1 z ) ((pos "Ha$" ) (subcat )))
( "overbid"  n ( ou1 v @ b i d ) ((pos "K6$" ) ))
( "overbid"  v ( ou2 v @ b i1 d ) ((pos "J5%" "Jc%" "Jd%" ) (subcat "2A" )))
( "overbidding"  v ( ou2 v @ b i1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overbids"  n ( ou1 v @ b i d z ) ((pos "Kj$" ) ))
( "overbids"  v ( ou2 v @ b i1 d z ) ((pos "Ja$" ) (subcat "2A" )))
( "overblown"  j ( ou2 v @ b l ou1 n ) ((pos "OA%" ) ))
( "overboard"  a ( ou1 v @ b oo d ) ((pos "Pu%" ) ))
( "overbold"  j ( ou2 v @ b ou1 l d ) ((pos "OA$" ) ))
( "overbore"  v ( ou2 v @ b oo1 r ) ((pos "Hc$" ) (subcat )))
( "overborne"  v ( ou2 v @ b oo1 n ) ((pos "Hd$" ) (subcat )))
( "overburden"  v ( ou2 v @ b @@1 d @ n ) ((pos "H0%" "L@$" ) (subcat )))
( "overburdened"  v ( ou2 v @ b @@1 d @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overburdening"  v ( ou2 v @ b @@1 d @ n i ng ) ((pos "Hb$" ) (subcat )))
( "overburdens"  v ( ou2 v @ b @@1 d @ n z ) ((pos "Ha$" ) (subcat )))
( "overbusy"  j ( ou2 v @ b i1 z ii ) ((pos "OA%" ) ))
( "overcall"  v ( ou2 v @ k oo1 l ) ((pos "J0$" ) ))
( "overcalled"  v ( ou2 v @ k oo1 l d ) ((pos "Jc$" "Jd$" ) ))
( "overcalling"  v ( ou2 v @ k oo1 l i ng ) ((pos "Jb$" ) ))
( "overcalls"  v ( ou2 v @ k oo1 l z ) ((pos "Ja$" ) ))
( "overcame"  v ( ou2 v @ k ei1 m ) ((pos "Hc%" ) (subcat )))
( "overcapitalization"  n ( ou2 v @ k a2 p i t @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "overcapitalize"  v ( ou2 v @ k a1 p i t @ l ai z ) ((pos "H2$" ) ))
( "overcapitalized"  v ( ou2 v @ k a1 p i t @ l ai z d ) ((pos "Hc$" "Hd%" ) ))
( "overcapitalizes"  v ( ou2 v @ k a1 p i t @ l ai z i z ) ((pos "Ha$" ) ))
( "overcapitalizing"  v ( ou2 v @ k a1 p i t @ l ai z i ng ) ((pos "Hb$" ) ))
( "overcareful"  j ( ou2 v @ k e@1 f @ l ) ((pos "OA%" ) ))
( "overcast"  n ( ou2 v @ k aa1 s t ) ((pos "Ki%" "OA%" ) ))
( "overcautious"  j ( ou2 v @ k oo1 sh @ s ) ((pos "OA%" ) ))
( "overcharge"  n ( ou1 v @ ch aa jh ) ((pos "K6$" ) ))
( "overcharge"  v ( ou2 v @ ch aa1 jh ) ((pos "J2%" ) (subcat "2A" )))
( "overcharged"  v ( ou2 v @ ch aa1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overcharges"  n ( ou1 v @ ch aa jh i z ) ((pos "Kj$" ) ))
( "overcharges"  v ( ou2 v @ ch aa1 jh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "overcharging"  v ( ou2 v @ ch aa1 jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overclothe"  v ( ou2 v @ k l ou1 dh ) ((pos "H2%" ) (subcat )))
( "overclothed"  v ( ou2 v @ k l ou1 dh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overclothes"  v ( ou2 v @ k l ou1 dh z ) ((pos "Ha%" ) (subcat )))
( "overclothing"  v ( ou2 v @ k l ou1 dh i ng ) ((pos "Hb%" ) (subcat )))
( "overcloud"  v ( ou2 v @ k l au1 d ) ((pos "J0$" ) (subcat "2A" )))
( "overclouded"  v ( ou2 v @ k l au1 d i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "overclouding"  v ( ou2 v @ k l au1 d i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "overclouds"  v ( ou2 v @ k l au1 d z ) ((pos "Ja$" ) (subcat "2A" )))
( "overcoat"  n ( ou1 v @ k ou t ) ((pos "K6%" ) ))
( "overcoats"  n ( ou1 v @ k ou t s ) ((pos "Kj%" ) ))
( "overcome"  v ( ou2 v @ k uh1 m ) ((pos "H5%" "Hd%" ) (subcat )))
( "overcomes"  v ( ou2 v @ k uh1 m z ) ((pos "Ha%" ) (subcat )))
( "overcoming"  v ( ou2 v @ k uh1 m i ng ) ((pos "Hb%" ) (subcat )))
( "overconfidence"  n ( ou2 v @ k o1 n f i d @ n s ) ((pos "L@%" ) ))
( "overconfident"  j ( ou2 v @ k o1 n f i d @ n t ) ((pos "OA%" ) ))
( "overcook"  v ( ou2 v @ k u1 k ) ((pos "H0%" ) ))
( "overcooked"  v ( ou2 v @ k u1 k t ) ((pos "Hc%" "Hd%" ) ))
( "overcooking"  v ( ou2 v @ k u1 k i ng ) ((pos "Hb%" ) ))
( "overcooks"  v ( ou2 v @ k u1 k s ) ((pos "Ha%" ) ))
( "overcredulity"  n ( ou2 v @ k r i d y uu1 l i t ii ) ((pos "L@$" ) ))
( "overcredulous"  j ( ou2 v @ k r e1 d y u l @ s ) ((pos "OA$" ) ))
( "overcritical"  j ( ou2 v @ k r i1 t i k l ) ((pos "OA%" ) ))
( "overcrop"  v ( ou2 v @ k r o1 p ) ((pos "H4$" ) (subcat )))
( "overcropped"  v ( ou2 v @ k r o1 p t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "overcropping"  v ( ou2 v @ k r o1 p i ng ) ((pos "Hb$" ) (subcat )))
( "overcrops"  v ( ou2 v @ k r o1 p s ) ((pos "Ha$" ) (subcat )))
( "overcrowd"  v ( ou2 v @ k r au1 d ) ((pos "H0%" ) (subcat )))
( "overcrowded"  v ( ou2 v @ k r au1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overcrowding"  v ( ou2 v @ k r au1 d i ng ) ((pos "Hb%" ) (subcat )))
( "overcrowds"  v ( ou2 v @ k r au1 d z ) ((pos "Ha%" ) (subcat )))
( "overcurious"  j ( ou2 v @ k y u@1 r i@ s ) ((pos "OA%" ) ))
( "overdelicate"  j ( ou2 v @ d e1 l i k @ t ) ((pos "OA%" ) ))
( "overdid"  v ( ou2 v @ d i1 d ) ((pos "Hc%" ) (subcat )))
( "overdo"  v ( ou2 v @ d uu1 ) ((pos "H5%" ) (subcat )))
( "overdoes"  v ( ou2 v @ d uh1 z ) ((pos "Ha%" ) (subcat )))
( "overdoing"  v ( ou2 v @ d uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "overdone"  v ( ou2 v @ d uh1 n ) ((pos "Hd%" ) (subcat )))
( "overdraft"  n ( ou1 v @ d r aa f t ) ((pos "K6%" ) ))
( "overdrafts"  n ( ou1 v @ d r aa f t s ) ((pos "Kj%" ) ))
( "overdraw"  v ( ou2 v @ d r oo1 ) ((pos "J5%" ) (subcat "2A" )))
( "overdrawing"  v ( ou2 v @ d r oo1 i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "overdrawn"  v ( ou2 v @ d r oo1 n ) ((pos "Jd%" ) (subcat "2A" )))
( "overdraws"  v ( ou2 v @ d r oo1 z ) ((pos "Ja%" ) (subcat "2A" )))
( "overdress"  v ( ou2 v @ d r e1 s ) ((pos "J1%" ) (subcat "2A" )))
( "overdressed"  v ( ou2 v @ d r e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overdresses"  v ( ou2 v @ d r e1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "overdressing"  v ( ou2 v @ d r e1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overdrew"  v ( ou2 v @ d r uu1 ) ((pos "Jc$" ) (subcat "2A" )))
( "overdrive"  n ( ou1 v @ d r ai v ) ((pos "M6%" ) ))
( "overdrives"  n ( ou1 v @ d r ai v z ) ((pos "Mj$" ) ))
( "overdue"  j ( ou2 v @ d y uu1 ) ((pos "OA%" ) ))
( "overeager"  j ( ou2 v @ r ii1 g @ r ) ((pos "OA%" ) ))
( "overeat"  v ( ou2 v @ r ii1 t ) ((pos "I5%" ) ))
( "overeaten"  v ( ou2 v @ r ii1 t n ) ((pos "Id%" ) ))
( "overeating"  v ( ou2 v @ r ii1 t i ng ) ((pos "Ib%" ) ))
( "overeats"  v ( ou2 v @ r ii1 t s ) ((pos "Ia%" ) ))
( "overemotional"  j ( ou2 v @ r i m ou1 sh @ n l ) ((pos "OA%" ) ))
( "overemphasis"  n ( ou2 v @ e1 m f @ s i s ) ((pos "L@%" ) ))
( "overemphasize"  v ( ou2 v @ r e1 m f @ s ai z ) ((pos "H2%" ) ))
( "overemphasized"  v ( ou2 v @ r e1 m f @ s ai z d ) ((pos "Hc%" "Hd%" ) ))
( "overemphasizes"  v ( ou2 v @ r e1 m f @ s ai z i z ) ((pos "Ha%" ) ))
( "overemphasizing"  v ( ou2 v @ r e1 m f @ s ai z i ng ) ((pos "Hb%" ) ))
( "overenthusiastic"  j ( ou2 v @ r i n th y uu2 z i a1 s t i k ) ((pos "OA%" ) ))
( "overestimate"  v ( ou2 v @ r e1 s t i m ei t ) ((pos "H2%" ) ))
( "overestimated"  v ( ou2 v @ r e1 s t i m ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "overestimates"  v ( ou2 v @ r e1 s t i m ei t s ) ((pos "Ha%" ) ))
( "overestimating"  v ( ou2 v @ r e1 s t i m ei t i ng ) ((pos "Hb%" ) ))
( "overexcited"  j ( ou2 v @ r i k s ai1 t i d ) ((pos "OA%" ) ))
( "overexert"  v ( ou2 v @ r i g z @@1 t ) ((pos "H0%" ) ))
( "overexerted"  v ( ou2 v @ r i g z @@1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "overexerting"  v ( ou2 v @ r i g z @@1 t i ng ) ((pos "Hb%" ) ))
( "overexertion"  n ( ou2 v @ r i g z @@1 sh @ n ) ((pos "M6%" ) ))
( "overexertions"  n ( ou2 v @ r i g z @@1 sh @ n z ) ((pos "Mj%" ) ))
( "overexerts"  v ( ou2 v @ r i g z @@1 t s ) ((pos "Ha%" ) ))
( "overexpose"  v ( ou2 v @ r i k s p ou1 z ) ((pos "H2%" ) ))
( "overexposed"  v ( ou2 v @ r i k s p ou1 z d ) ((pos "Hc%" "Hd%" ) ))
( "overexposes"  v ( ou2 v @ r i k s p ou1 z i z ) ((pos "Ha%" ) ))
( "overexposing"  v ( ou2 v @ r i k s p ou1 z i ng ) ((pos "Hb%" ) ))
( "overexposure"  n ( ou2 v @ r i k s p ou1 zh @ r ) ((pos "M6%" ) ))
( "overexposures"  n ( ou2 v @ r i k s p ou1 zh @ z ) ((pos "Mj$" ) ))
( "overfamiliar"  j ( ou2 v @ f @ m i1 l i@ r ) ((pos "OA%" ) ))
( "overfed"  v ( ou2 v @ f e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "overfeed"  v ( ou2 v @ f ii1 d ) ((pos "J0%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "overfeeding"  v ( ou2 v @ f ii1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "overfeeds"  v ( ou2 v @ f ii1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "overflew"  v ( ou2 v @ f l uu1 ) ((pos "Hc%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "overflies"  v ( ou2 v @ f l ai1 z ) ((pos "Ha%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "overflow"  n ( ou1 v @ f l ou ) ((pos "K6%" ) ))
( "overflow"  v ( ou2 v @ f l ou1 ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "overflowed"  v ( ou2 v @ f l ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "overflowing"  v ( ou2 v @ f l ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "overflown"  v ( ou2 v @ f l ou1 n ) ((pos "Hb%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "overflows"  n ( ou1 v @ f l ou z ) ((pos "Kj%" ) ))
( "overflows"  v ( ou2 v @ f l ou1 z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "overfly"  v ( ou2 v @ f l ai1 ) ((pos "H5%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "overflying"  v ( ou2 v @ f l ai1 i ng ) ((pos "Hb%" ) (subcat "2A" "2B" "2C" "2D" "4A" "6A" "15A" )))
( "overfond"  j ( ou2 v @ f o1 n d ) ((pos "OA%" ) ))
( "overfull"  j ( ou2 v @ f u1 l ) ((pos "OA%" ) ))
( "overgenerous"  j ( ou2 v @ jh e1 n @ r @ s ) ((pos "OA%" ) ))
( "overgreedy"  j ( ou2 v @ g r ii1 d ii ) ((pos "OA%" ) ))
( "overgrown"  j ( ou2 v @ g r ou1 n ) ((pos "OA%" ) ))
( "overgrowth"  n ( ou1 v @ g r ou th ) ((pos "M6%" ) ))
( "overgrowths"  n ( ou1 v @ g r ou th s ) ((pos "Mj$" ) ))
( "overhand"  j ( ou1 v @ h a n d ) ((pos "OA$" ) ))
( "overhang"  n ( ou1 v @ h a ng ) ((pos "K6%" ) ))
( "overhang"  v ( ou2 v @ h a1 ng ) ((pos "J5%" ) (subcat "2A" )))
( "overhanging"  v ( ou2 v @ h a1 ng i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overhangs"  n ( ou1 v @ h a ng z ) ((pos "Kj$" ) ))
( "overhangs"  v ( ou2 v @ h a1 ng z ) ((pos "Ja%" ) (subcat "2A" )))
( "overhasty"  j ( ou2 v @ h ei1 s t ii ) ((pos "OA%" ) ))
( "overhaul"  n ( ou1 v @ h oo l ) ((pos "K6%" ) ))
( "overhaul"  v ( ou2 v @ h oo1 l ) ((pos "H0%" ) (subcat )))
( "overhauled"  v ( ou2 v @ h oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overhauling"  v ( ou2 v @ h oo1 l i ng ) ((pos "Hb%" ) (subcat )))
( "overhauls"  n ( ou1 v @ h oo l z ) ((pos "Kj%" ) ))
( "overhauls"  v ( ou2 v @ h oo1 l z ) ((pos "Ha%" ) (subcat )))
( "overhead"  j ( ou1 v @ h e d ) ((pos "OA%" ) ))
( "overhead"  a ( ou2 v @ h e1 d ) ((pos "Pu%" ) ))
( "overheads"  n ( ou1 v @ h e d z ) ((pos "Kj%" ) ))
( "overhear"  v ( ou2 v @ h i@1 r ) ((pos "H5%" ) (subcat "6A" "18A" )))
( "overheard"  v ( ou2 v @ h @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "18A" )))
( "overhearing"  v ( ou2 v @ h i@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "18A" )))
( "overhears"  v ( ou2 v @ h i@1 z ) ((pos "Ha%" ) (subcat "6A" "18A" )))
( "overheat"  v ( ou2 v @ h ii1 t ) ((pos "H0%" ) ))
( "overheated"  v ( ou2 v @ h ii1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "overheating"  v ( ou2 v @ h ii1 t i ng ) ((pos "Hb%" ) ))
( "overheats"  v ( ou2 v @ h ii1 t s ) ((pos "Ha%" ) ))
( "overhung"  v ( ou2 v @ h uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overindulge"  v ( ou2 v @ r i n d uh1 l jh ) ((pos "J2%" ) ))
( "overindulged"  v ( ou2 v @ r i n d uh1 l jh d ) ((pos "Jc%" "Jd%" ) ))
( "overindulgence"  n ( ou2 v @ r i n d uh1 l jh @ n s ) ((pos "M6%" ) ))
( "overindulgences"  n ( ou2 v @ r i n d uh1 l jh @ n s i z ) ((pos "Mj$" ) ))
( "overindulges"  v ( ou2 v @ r i n d uh1 l jh i z ) ((pos "Ja%" ) ))
( "overindulging"  v ( ou2 v @ r i n d uh1 l jh i ng ) ((pos "Jb%" ) ))
( "overjealous"  j ( ou2 v @ jh e1 l @ s ) ((pos "OA$" ) ))
( "overjoyed"  j ( ou2 v @ jh oi1 d ) ((pos "OA%" ) ))
( "overkill"  n ( ou1 v @ k i l ) ((pos "L@%" ) ))
( "overladen"  j ( ou2 v @ l ei1 d n ) ((pos "OA%" ) ))
( "overlaid"  v ( ou2 v @ l ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "overlain"  v ( ou1 v @ l ei1 n ) ((pos "Id$" ) (subcat "2A" "2C" "2D" )))
( "overland"  j ( ou1 v @ l a n d ) ((pos "OA%" ) ))
( "overlap"  n ( ou1 v @ l a p ) ((pos "M6%" ) ))
( "overlap"  v ( ou2 v @ l a1 p ) ((pos "J4%" ) (subcat "2A" )))
( "overlapped"  v ( ou2 v @ l a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overlapping"  v ( ou2 v @ l a1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overlaps"  n ( ou1 v @ l a p s ) ((pos "Mj%" ) ))
( "overlaps"  v ( ou2 v @ l a1 p s ) ((pos "Ja%" ) (subcat "2A" )))
( "overlarge"  j ( ou2 v @ l aa1 jh ) ((pos "OA%" ) ))
( "overlay"  n ( ou1 v @ l ei ) ((pos "K6%" ) ))
( "overlay"  v ( ou2 v @ l ei1 ) ((pos "H5%" ) ))
( "overlaying"  v ( ou2 v @ l ei1 i ng ) ((pos "Hb%" ) ))
( "overlays"  n ( ou1 v @ l ei z ) ((pos "Kj%" ) ))
( "overlays"  v ( ou2 v @ l ei1 z ) ((pos "Ha%" ) ))
( "overleaf"  a ( ou2 v @ l ii1 f ) ((pos "Pu%" ) ))
( "overleap"  v ( ou2 v @ l ii1 p ) ((pos "H0%" ) (subcat )))
( "overleaped"  v ( ou2 v @ l ii1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overleaping"  v ( ou2 v @ l ii1 p i ng ) ((pos "Hb%" ) (subcat )))
( "overleaps"  v ( ou2 v @ l ii1 p s ) ((pos "Ha%" ) (subcat )))
( "overleapt"  v ( ou2 v @ l e1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overlie"  v ( ou2 v @ l ai1 ) ((pos "I5$" ) (subcat "2A" "2C" "2D" )))
( "overlies"  v ( ou2 v @ l ai1 z ) ((pos "Ia$" ) (subcat "2A" "2C" "2D" )))
( "overload"  v ( ou2 v @ l ou1 d ) ((pos "H0%" ) (subcat )))
( "overloaded"  v ( ou2 v @ l ou1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overloading"  v ( ou2 v @ l ou1 d i ng ) ((pos "Hb%" ) (subcat )))
( "overloads"  v ( ou2 v @ l ou1 d z ) ((pos "Ha%" ) (subcat )))
( "overlook"  v ( ou2 v @ l u1 k ) ((pos "H0%" ) (subcat )))
( "overlooked"  v ( ou2 v @ l u1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overlooking"  v ( ou2 v @ l u1 k i ng ) ((pos "Hb%" ) (subcat )))
( "overlooks"  v ( ou2 v @ l u1 k s ) ((pos "Ha%" ) (subcat )))
( "overlord"  n ( ou1 v @ l oo d ) ((pos "K6%" ) ))
( "overlords"  n ( ou1 v @ l oo d z ) ((pos "Kj%" ) ))
( "overly"  a ( ou1 v @ l ii ) ((pos "Pu$" ) ))
( "overlying"  v ( ou2 v @ l ai1 i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "2D" )))
( "overmantel"  n ( ou1 v @ m a n t l ) ((pos "K6$" ) ))
( "overmantels"  n ( ou1 v @ m a n t l z ) ((pos "Kj$" ) ))
( "overmaster"  v ( ou2 v @ m aa1 s t @ r ) ((pos "H0$" ) (subcat )))
( "overmastered"  v ( ou2 v @ m aa1 s t @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "overmastering"  v ( ou2 v @ m aa1 s t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "overmasters"  v ( ou2 v @ m aa1 s t @ z ) ((pos "Ha$" ) (subcat )))
( "overmodest"  j ( ou2 v @ m o1 d i s t ) ((pos "OA%" ) ))
( "overmuch"  j ( ou2 v @ m uh1 ch ) ((pos "OA%" "Pu%" ) ))
( "overnervous"  j ( ou2 v @ n @@1 v @ s ) ((pos "OA$" ) ))
( "overnight"  j ( ou1 v @ n ai t ) ((pos "OA%" ) ))
( "overnight"  a ( ou2 v @ n ai1 t ) ((pos "Pu%" ) ))
( "overpaid"  v ( ou2 v @ p ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "overpass"  n ( ou1 v @ p aa s ) ((pos "K7$" ) ))
( "overpasses"  n ( ou1 v @ p aa s i z ) ((pos "Kj$" ) ))
( "overpay"  v ( ou2 v @ p ei1 ) ((pos "H5%" ) (subcat "6A" )))
( "overpaying"  v ( ou2 v @ p ei1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "overpayment"  n ( ou2 v @ p ei1 m @ n t ) ((pos "M6%" ) ))
( "overpayments"  n ( ou2 v @ p ei1 m @ n t s ) ((pos "Mj%" ) ))
( "overpays"  v ( ou2 v @ p ei1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "overplay"  v ( ou2 v @ p l ei1 ) ((pos "H0%" ) ))
( "overplayed"  v ( ou2 v @ p l ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "overplaying"  v ( ou2 v @ p l ei1 i ng ) ((pos "Hb%" ) ))
( "overplays"  v ( ou2 v @ p l ei1 z ) ((pos "Ha%" ) ))
( "overplus"  n ( ou1 v @ p l uh s ) ((pos "K7$" ) ))
( "overpluses"  n ( ou1 v @ p l uh s i z ) ((pos "Kj$" ) ))
( "overpopulation"  n ( ou2 v @ p o2 p y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "overpower"  v ( ou2 v @ p au1 @ r ) ((pos "H0%" ) (subcat )))
( "overpowered"  v ( ou2 v @ p au1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overpowering"  v ( ou2 v @ p au1 @ r i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "overpowers"  v ( ou2 v @ p au1 @ z ) ((pos "Ha%" ) (subcat )))
( "overpraise"  v ( ou2 v @ p r ei1 z ) ((pos "H2%" ) ))
( "overpraised"  v ( ou2 v @ p r ei1 z d ) ((pos "Hc%" "Hd%" ) ))
( "overpraises"  v ( ou2 v @ p r ei1 z i z ) ((pos "Ha%" ) ))
( "overpraising"  v ( ou2 v @ p r ei1 z i ng ) ((pos "Hb%" ) ))
( "overprint"  n ( ou1 v @ p r i n t ) ((pos "K6$" ) ))
( "overprint"  v ( ou2 v @ p r i1 n t ) ((pos "H0%" ) (subcat )))
( "overprinted"  v ( ou2 v @ p r i1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overprinting"  v ( ou2 v @ p r i1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "overprints"  n ( ou1 v @ p r i n t s ) ((pos "Kj$" ) ))
( "overprints"  v ( ou2 v @ p r i1 n t s ) ((pos "Ha%" ) (subcat )))
( "overproduce"  v ( ou2 v @ p r @ d y uu1 s ) ((pos "J2%" ) ))
( "overproduced"  v ( ou2 v @ p r @ d y uu1 s t ) ((pos "Jc%" "Jd%" ) ))
( "overproduces"  v ( ou2 v @ p r @ d y uu1 s i z ) ((pos "Ja%" ) ))
( "overproducing"  v ( ou2 v @ p r @ d y uu1 s i ng ) ((pos "Jb%" ) ))
( "overproduction"  n ( ou2 v @ p r @ d uh1 k sh @ n ) ((pos "L@%" ) ))
( "overproud"  j ( ou2 v @ p r au1 d ) ((pos "OA%" ) ))
( "overran"  v ( ou2 v @ r a1 n ) ((pos "Hc%" ) (subcat )))
( "overrate"  v ( ou2 v @ r ei1 t ) ((pos "H2%" ) (subcat )))
( "overrated"  v ( ou2 v @ r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overrates"  v ( ou2 v @ r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "overrating"  v ( ou2 v @ r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "overreach"  v ( ou2 v @ r ii1 ch ) ((pos "H1%" ) (subcat )))
( "overreached"  v ( ou2 v @ r ii1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overreaches"  v ( ou2 v @ r ii1 ch i z ) ((pos "Ha%" ) (subcat )))
( "overreaching"  v ( ou2 v @ r ii1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "overridden"  v ( ou2 v @ r i1 d n ) ((pos "Hd%" ) (subcat )))
( "override"  v ( ou2 v @ r ai1 d ) ((pos "H5%" ) (subcat )))
( "overrides"  v ( ou2 v @ r ai1 d z ) ((pos "Ha%" ) (subcat )))
( "overriding"  v ( ou2 v @ r ai1 d i ng ) ((pos "Hb%" ) (subcat )))
( "overripe"  j ( ou2 v @ r ai1 p ) ((pos "OA%" ) ))
( "overrode"  v ( ou2 v @ r ou1 d ) ((pos "Hc%" ) (subcat )))
( "overrule"  v ( ou2 v @ r uu1 l ) ((pos "H2%" ) (subcat )))
( "overruled"  v ( ou2 v @ r uu1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overrules"  v ( ou2 v @ r uu1 l z ) ((pos "Ha%" ) (subcat )))
( "overruling"  v ( ou2 v @ r uu1 l i ng ) ((pos "Hb%" ) (subcat )))
( "overrun"  v ( ou2 v @ r uh1 n ) ((pos "H5%" "Hd%" ) (subcat )))
( "overrunning"  v ( ou2 v @ r uh1 n i ng ) ((pos "Hb%" ) (subcat )))
( "overruns"  v ( ou2 v @ r uh1 n z ) ((pos "Ha%" ) (subcat )))
( "overs"  n ( ou1 v @ z ) ((pos "Kj%" ) ))
( "oversaw"  v ( ou2 v @ s oo1 ) ((pos "Hc%" ) (subcat )))
( "oversea"  j ( ou2 v @ s ii1 ) ((pos "OA%" "Pu%" ) ))
( "overseas"  j ( ou2 v @ s ii1 z ) ((pos "OA%" "Pu%" ) ))
( "oversee"  v ( ou2 v @ s ii1 ) ((pos "H5%" ) (subcat )))
( "overseeing"  v ( ou2 v @ s ii1 i ng ) ((pos "Hb%" ) (subcat )))
( "overseen"  v ( ou2 v @ s ii1 n ) ((pos "Hd%" ) (subcat )))
( "overseer"  n ( ou1 v @ s i@ r ) ((pos "K6%" ) ))
( "overseers"  n ( ou1 v @ s i@ z ) ((pos "Kj%" ) ))
( "oversees"  v ( ou2 v @ s ii1 z ) ((pos "Ha%" ) (subcat )))
( "oversensitive"  j ( ou2 v @ s e1 n s @ t i v ) ((pos "OA%" ) ))
( "overserious"  j ( ou2 v @ s i@1 r i@ s ) ((pos "OA%" ) ))
( "oversew"  v ( ou2 v @ s ou1 ) ((pos "H5%" ) (subcat "2A" "2B" "2C" "6A" )))
( "oversewed"  v ( ou2 v @ s ou1 d ) ((pos "Hc%" ) (subcat "2A" "2B" "2C" "6A" )))
( "oversewing"  v ( ou2 v @ s ou1 i ng ) ((pos "Hb%" ) (subcat "2A" "2B" "2C" "6A" )))
( "oversewn"  v ( ou2 v @ s ou1 n ) ((pos "Hd%" "OA%" ) (subcat "2A" "2B" "2C" "6A" )))
( "oversews"  v ( ou2 v @ s ou1 z ) ((pos "Ha%" ) (subcat "2A" "2B" "2C" "6A" )))
( "oversexed"  j ( ou2 v @ s e1 k s t ) ((pos "OA%" ) ))
( "overshadow"  v ( ou2 v @ sh a1 d ou ) ((pos "H0%" ) (subcat )))
( "overshadowed"  v ( ou2 v @ sh a1 d ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overshadowing"  v ( ou2 v @ sh a1 d ou i ng ) ((pos "Hb%" ) (subcat )))
( "overshadows"  v ( ou2 v @ sh a1 d ou z ) ((pos "Ha%" ) (subcat )))
( "overshoe"  n ( ou1 v @ sh uu ) ((pos "K6$" ) ))
( "overshoes"  n ( ou1 v @ sh uu z ) ((pos "Kj%" ) ))
( "overshoot"  v ( ou2 v @ sh uu1 t ) ((pos "H5%" ) (subcat )))
( "overshooting"  v ( ou2 v @ sh uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "overshoots"  v ( ou2 v @ sh uu1 t s ) ((pos "Ha%" ) (subcat )))
( "overshot"  v ( ou2 v @ sh o1 t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "overside"  a ( ou1 v @ s ai d ) ((pos "Pu$" ) ))
( "oversight"  n ( ou1 v @ s ai t ) ((pos "M6%" ) ))
( "oversights"  n ( ou1 v @ s ai t s ) ((pos "Mj%" ) ))
( "oversimplified"  v ( ou2 v @ s i1 m p l i f ai d ) ((pos "Hc%" "Hd%" ) ))
( "oversimplifies"  v ( ou2 v @ s i1 m p l i f ai z ) ((pos "Ha%" ) ))
( "oversimplify"  v ( ou2 v @ s i1 m p l i f ai ) ((pos "H3%" ) ))
( "oversimplifying"  v ( ou2 v @ s i1 m p l i f ai i ng ) ((pos "Hb%" ) ))
( "oversize"  j ( ou2 v @ s ai1 z ) ((pos "OA%" ) ))
( "overskirt"  n ( ou1 v @ s k @@ t ) ((pos "K6$" ) ))
( "overskirts"  n ( ou1 v @ s k @@ t s ) ((pos "Kj$" ) ))
( "oversleep"  v ( ou2 v @ s l ii1 p ) ((pos "I5%" ) (subcat )))
( "oversleeping"  v ( ou2 v @ s l ii1 p i ng ) ((pos "Ib%" ) (subcat )))
( "oversleeps"  v ( ou2 v @ s l ii1 p s ) ((pos "Ia%" ) (subcat )))
( "overslept"  v ( ou2 v @ s l e1 p t ) ((pos "Ic%" "Id%" ) (subcat )))
( "overspend"  v ( ou2 v @ s p e1 n d ) ((pos "J5%" ) (subcat "2A" "6A" "14" )))
( "overspending"  v ( ou2 v @ s p e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "overspends"  v ( ou2 v @ s p e1 n d z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "overspent"  v ( ou2 v @ s p e1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "overspill"  n ( ou1 v @ s p i l ) ((pos "K6%" ) ))
( "overspills"  n ( ou1 v @ s p i l z ) ((pos "Kj$" ) ))
( "overstate"  v ( ou2 v @ s t ei1 t ) ((pos "H2%" ) (subcat )))
( "overstated"  v ( ou2 v @ s t ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overstatement"  n ( ou1 v @ s t ei t m @ n t ) ((pos "M6%" ) ))
( "overstatements"  n ( ou1 v @ s t ei t m @ n t s ) ((pos "Mj%" ) ))
( "overstates"  v ( ou2 v @ s t ei1 t s ) ((pos "Ha%" ) (subcat )))
( "overstating"  v ( ou2 v @ s t ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "overstay"  v ( ou2 v @ s t ei1 ) ((pos "H0%" ) (subcat )))
( "overstayed"  v ( ou2 v @ s t ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overstaying"  v ( ou2 v @ s t ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "overstays"  v ( ou2 v @ s t ei1 z ) ((pos "Ha%" ) (subcat )))
( "overstep"  v ( ou2 v @ s t e1 p ) ((pos "H4%" ) (subcat )))
( "overstepped"  v ( ou2 v @ s t e1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overstepping"  v ( ou2 v @ s t e1 p i ng ) ((pos "Hb%" ) (subcat )))
( "oversteps"  v ( ou2 v @ s t e1 p s ) ((pos "Ha%" ) (subcat )))
( "overstock"  v ( ou2 v @ s t o1 k ) ((pos "H0%" ) (subcat )))
( "overstocked"  v ( ou2 v @ s t o1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overstocking"  v ( ou2 v @ s t o1 k i ng ) ((pos "Hb%" ) (subcat )))
( "overstocks"  v ( ou2 v @ s t o1 k s ) ((pos "Ha%" ) (subcat )))
( "overstrain"  n ( ou1 v @ s t r ei n ) ((pos "L@%" ) ))
( "overstrain"  v ( ou2 v @ s t r ei1 n ) ((pos "H0%" ) ))
( "overstrained"  v ( ou2 v @ s t r ei1 n d ) ((pos "Hc%" "Hd%" ) ))
( "overstraining"  v ( ou2 v @ s t r ei1 n i ng ) ((pos "Hb%" ) ))
( "overstrains"  v ( ou2 v @ s t r ei1 n z ) ((pos "Ha%" ) ))
( "overstrung"  j ( ou2 v @ s t r uh1 ng ) ((pos "OA%" ) ))
( "overstuffed"  j ( ou2 v @ s t uh1 f t ) ((pos "OA%" ) ))
( "oversubscribed"  j ( ou2 v @ s @ b s k r ai1 b d ) ((pos "OA%" ) ))
( "oversuspicious"  j ( ou2 v @ s @ s p i1 sh @ s ) ((pos "OA%" ) ))
( "overt"  j ( ou1 v @@1 t ) ((pos "OA%" ) ))
( "overtake"  v ( ou2 v @ t ei1 k ) ((pos "H5%" ) (subcat )))
( "overtaken"  v ( ou2 v @ t ei1 k @ n ) ((pos "Hd%" ) (subcat )))
( "overtakes"  v ( ou2 v @ t ei1 k s ) ((pos "Ha%" ) (subcat )))
( "overtaking"  v ( ou2 v @ t ei1 k i ng ) ((pos "Hb%" ) (subcat )))
( "overtax"  v ( ou2 v @ t a1 k s ) ((pos "H1%" ) (subcat )))
( "overtaxed"  v ( ou2 v @ t a1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overtaxes"  v ( ou2 v @ t a1 k s i z ) ((pos "Ha%" ) (subcat )))
( "overtaxing"  v ( ou2 v @ t a1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "overthrew"  v ( ou2 v @ th r uu1 ) ((pos "Hc%" ) (subcat )))
( "overthrow"  n ( ou1 v @ th r ou ) ((pos "K6%" ) ))
( "overthrow"  v ( ou2 v @ th r ou1 ) ((pos "H5%" ) (subcat )))
( "overthrowing"  v ( ou2 v @ th r ou1 i ng ) ((pos "Hb%" ) (subcat )))
( "overthrown"  v ( ou2 v @ th r ou1 n ) ((pos "Hd%" ) (subcat )))
( "overthrows"  n ( ou1 v @ th r ou z ) ((pos "Kj%" ) ))
( "overthrows"  v ( ou2 v @ th r ou1 z ) ((pos "Ha%" ) (subcat )))
( "overtime"  n ( ou1 v @ t ai m ) ((pos "L@%" "Pu%" ) ))
( "overtly"  a ( ou1 v @@1 t l ii ) ((pos "Pu%" ) ))
( "overtolerance"  n ( ou2 v @ t o1 l @ r @ n s ) ((pos "L@$" ) ))
( "overtone"  n ( ou1 v @ t ou n ) ((pos "K6%" ) ))
( "overtones"  n ( ou1 v @ t ou n z ) ((pos "Kj%" ) ))
( "overtook"  v ( ou2 v @ t u1 k ) ((pos "Hc%" ) (subcat )))
( "overtop"  v ( ou2 v @ t o1 p ) ((pos "H4$" ) (subcat )))
( "overtopped"  v ( ou2 v @ t o1 p t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "overtopping"  v ( ou2 v @ t o1 p i ng ) ((pos "Hb$" ) (subcat )))
( "overtops"  v ( ou2 v @ t o1 p s ) ((pos "Ha$" ) (subcat )))
( "overtrump"  v ( ou2 v @ t r uh1 m p ) ((pos "H0$" ) (subcat )))
( "overtrumped"  v ( ou2 v @ t r uh1 m p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "overtrumping"  v ( ou2 v @ t r uh1 m p i ng ) ((pos "Hb$" ) (subcat )))
( "overtrumps"  v ( ou2 v @ t r uh1 m p s ) ((pos "Ha$" ) (subcat )))
( "overture"  n ( ou1 v @ t y u@ r ) ((pos "K6%" ) ))
( "overtures"  n ( ou1 v @ t y u@ z ) ((pos "Kj%" ) ))
( "overturn"  v ( ou2 v @ t @@1 n ) ((pos "J0%" ) (subcat "2A" )))
( "overturned"  v ( ou2 v @ t @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overturning"  v ( ou2 v @ t @@1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overturns"  v ( ou2 v @ t @@1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "overvalue"  v ( ou2 v @ v a1 l y uu ) ((pos "H2%" ) ))
( "overvalued"  v ( ou2 v @ v a1 l y uu d ) ((pos "Hc%" "Hd%" ) ))
( "overvalues"  v ( ou2 v @ v a1 l y uu z ) ((pos "Ha%" ) ))
( "overvaluing"  v ( ou2 v @ v a1 l y uu i ng ) ((pos "Hb%" ) ))
( "overweening"  j ( ou2 v @ w ii1 n i ng ) ((pos "OA%" ) ))
( "overweight"  n ( ou1 v @ w ei t ) ((pos "L@$" ) ))
( "overweight"  j ( ou2 v @ w ei1 t ) ((pos "OA%" ) ))
( "overweighted"  j ( ou2 v @ w ei1 t i d ) ((pos "OA%" ) ))
( "overwhelm"  v ( ou2 v @ w e1 l m ) ((pos "H0%" ) (subcat )))
( "overwhelmed"  v ( ou2 v @ w e1 l m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "overwhelming"  v ( ou2 v @ w e1 l m i ng ) ((pos "Hb%" ) (subcat )))
( "overwhelmingly"  a ( ou2 v @ w e1 l m i ng l ii ) ((pos "Pu%" ) ))
( "overwhelms"  v ( ou2 v @ w e1 l m z ) ((pos "Ha%" ) (subcat )))
( "overwork"  n ( ou1 v @ w @@ k ) ((pos "L@$" ) ))
( "overwork"  v ( ou2 v @ w @@1 k ) ((pos "J0%" ) (subcat "2A" )))
( "overworked"  v ( ou2 v @ w @@1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "overworking"  v ( ou2 v @ w @@1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "overworks"  v ( ou2 v @ w @@1 k s ) ((pos "Ja%" ) (subcat "2A" )))
( "overwrought"  j ( ou2 v @ r oo1 t ) ((pos "OA%" ) ))
( "overzealous"  j ( ou2 v @ z e1 l @ s ) ((pos "OA%" ) ))
( "oviduct"  n ( ou1 v i d uh k t ) ((pos "K6$" ) ))
( "oviducts"  n ( ou1 v i d uh k t s ) ((pos "Kj$" ) ))
( "oviparous"  j ( ou1 v i1 p @ r @ s ) ((pos "OA$" ) ))
( "ovoid"  n ( ou1 v oi d ) ((pos "K6$" "OA$" ) ))
( "ovoids"  n ( ou1 v oi d z ) ((pos "Kj$" ) ))
( "ovum"  n ( ou1 v @ m ) ((pos "Ki$" ) ))
( "owe"  v ( ou1 ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "owed"  v ( ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "owes"  v ( ou1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "owing"  v ( ou1 i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "12A" "13A" )))
( "owl"  n ( au1 l ) ((pos "K6%" ) ))
( "owlet"  n ( au1 l i t ) ((pos "K6$" ) ))
( "owlets"  n ( au1 l i t s ) ((pos "Kj$" ) ))
( "owlish"  j ( au1 l i sh ) ((pos "OA%" ) ))
( "owlishly"  a ( au1 l i sh l ii ) ((pos "Pu%" ) ))
( "owls"  n ( au1 l z ) ((pos "Kj%" ) ))
( "own"  v ( ou n ) ((pos "J0%" "OA*" "Qx*" ) (subcat "2C" "3B" "6A" )))
( "owned"  v ( ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3B" "6A" )))
( "owner"  n ( ou1 n @ r ) ((pos "K6%" ) ))
( "owner-driven"  j ( ou2 n @ - d r i1 v n ) ((pos "OA$" ) ))
( "owner-driver"  n ( ou2 n @ - d r ai1 v @ r ) ((pos "K6%" ) ))
( "owner-drivers"  n ( ou2 n @ - d r ai1 v @ z ) ((pos "Kj%" ) ))
( "owner-occupied"  j ( ou2 n @ r - o1 k y @ p ai d ) ((pos "OA%" ) ))
( "owner-occupier"  n ( ou2 n @ r - o1 k y u p ai @ r ) ((pos "K6%" ) ))
( "owner-occupiers"  n ( ou2 n @ r - o1 k y u p ai @ z ) ((pos "Kj%" ) ))
( "ownerless"  j ( ou1 n @ l @ s ) ((pos "OA%" ) ))
( "owners"  n ( ou1 n @ z ) ((pos "Kj%" ) ))
( "ownership"  n ( ou1 n @ sh i p ) ((pos "L@%" ) ))
( "owning"  v ( ou1 n i ng ) ((pos "Jb%" ) (subcat "2C" "3B" "6A" )))
( "owns"  v ( ou1 n z ) ((pos "Ja%" ) (subcat "2C" "3B" "6A" )))
( "ox"  n ( o1 k s ) ((pos "Ki%" ) ))
( "oxen"  n ( o1 k s n ) ((pos "Kj%" ) ))
( "oxeye"  n ( o1 k s ai ) ((pos "K6$" ) ))
( "oxeyed"  j ( o1 k s ai d ) ((pos "OA$" ) ))
( "oxeyes"  n ( o1 k s ai z ) ((pos "Kj$" ) ))
( "oxidation"  n ( o2 k s i d ei1 sh @ n ) ((pos "L@%" ) ))
( "oxide"  n ( o1 k s ai d ) ((pos "M6%" ) ))
( "oxides"  n ( o1 k s ai d z ) ((pos "Mj%" ) ))
( "oxidization"  n ( o2 k s i d ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "oxidizations"  n ( o2 k s i d ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "oxidize"  v ( o1 k s i d ai z ) ((pos "J2%" ) (subcat "2A" )))
( "oxidized"  v ( o1 k s i d ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "oxidizes"  v ( o1 k s i d ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "oxidizing"  v ( o1 k s i d ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "oxtail"  n ( o1 k s t ei l ) ((pos "K6%" ) ))
( "oxtails"  n ( o1 k s t ei l z ) ((pos "Kj$" ) ))
( "oxyacetylene"  n ( o2 k s i@ s e1 t @ l ii n ) ((pos "L@%" "OA%" ) ))
( "oxygen"  n ( o1 k s i jh @ n ) ((pos "L@%" ) ))
( "oxygenate"  v ( o1 k s i jh @ n ei t ) ((pos "H2%" ) ))
( "oxygenated"  v ( o1 k s i jh @ n ei t i d ) ((pos "Hc%" "Hd%" ) ))
( "oxygenates"  v ( o1 k s i jh @ n ei t s ) ((pos "Ha%" ) ))
( "oxygenating"  v ( o1 k s i jh @ n ei t i ng ) ((pos "Hb%" ) ))
( "oxygenize"  v ( o1 k s i jh @ n ai z ) ((pos "H2$" ) ))
( "oxygenized"  v ( o1 k s i jh @ n ai z d ) ((pos "Hc$" "Hd$" ) ))
( "oxygenizes"  v ( o1 k s i jh @ n ai z i z ) ((pos "Ha$" ) ))
( "oxygenizing"  v ( o1 k s i jh @ n ai z i ng ) ((pos "Hb$" ) ))
( "oyes"  uh ( ou1 y e1 s ) ((pos "W-$" ) ))
( "oyez"  uh ( ou1 y e1 z ) ((pos "W-$" ) ))
( "oyster"  n ( oi1 s t @ r ) ((pos "K6%" ) ))
( "oyster-bank"  n ( oi1 s t @ - b a ng k ) ((pos "K6$" ) ))
( "oyster-banks"  n ( oi1 s t @ - b a ng k s ) ((pos "Kj$" ) ))
( "oyster-bar"  n ( oi1 s t @ - b aa r ) ((pos "K6$" ) ))
( "oyster-bars"  n ( oi1 s t @ - b aa z ) ((pos "Kj$" ) ))
( "oyster-bed"  n ( oi1 s t @ - b e d ) ((pos "K6%" ) ))
( "oyster-beds"  n ( oi1 s t @ - b e d z ) ((pos "Kj%" ) ))
( "oyster-catcher"  n ( oi1 s t @ - k a ch @ r ) ((pos "K6%" ) ))
( "oyster-catchers"  n ( oi1 s t @ - k a ch @ z ) ((pos "Kj%" ) ))
( "oysters"  n ( oi1 s t @ z ) ((pos "Kj%" ) ))
( "oz"  n ( o1 z ) ((pos "Y]%" ) ))
( "ozone"  n ( ou1 z ou n ) ((pos "L@%" ) ))
( "p"  n ( p ii1 ) ((pos "Ki$" ) ))
( "p's"  n ( p ii1 z ) ((pos "Kj$" ) ))
( "p^at_e"  n ( p a1 t ei ) ((pos "M6%" ) ))
( "p^at_es"  n ( p a1 t ei z ) ((pos "Mj%" ) ))
( "pa"  n ( p aa1 ) ((pos "Ki%" ) ))
( "pabulum"  n ( p a1 b y u l @ m ) ((pos "L@$" ) ))
( "pace"  v ( p ei1 s ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "paced"  v ( p ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "pacemaker"  n ( p ei1 s m ei k @ r ) ((pos "K6%" ) ))
( "pacemakers"  n ( p ei1 s m ei k @ z ) ((pos "Kj%" ) ))
( "paces"  v ( p ei1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "pacesetter"  n ( p ei1 s - s e t @ r ) ((pos "K6%" ) ))
( "pacesetters"  n ( p ei1 s - s e t @ z ) ((pos "Kj%" ) ))
( "pachyderm"  n ( p a1 k i d @@ m ) ((pos "K6$" ) ))
( "pachyderms"  n ( p a1 k i d @@ m z ) ((pos "Kj$" ) ))
( "pacific"  j ( p @1 s i1 f i k ) ((pos "OA$" ) ))
( "pacifically"  a ( p @1 s i1 f i k l ii ) ((pos "Pu$" ) ))
( "pacification"  n ( p a2 s i f i k ei1 sh @ n ) ((pos "L@$" ) ))
( "pacified"  v ( p a1 s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pacifies"  v ( p a1 s i f ai z ) ((pos "Ha%" ) (subcat )))
( "pacifism"  n ( p a1 s i f i z @ m ) ((pos "L@%" ) ))
( "pacifist"  n ( p a1 s ii f i s t ) ((pos "K6%" ) ))
( "pacifists"  n ( p a1 s ii f i s t s ) ((pos "Kj%" ) ))
( "pacify"  v ( p a1 s i f ai ) ((pos "H3%" ) (subcat )))
( "pacifying"  v ( p a1 s i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "pacing"  v ( p ei1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "pack"  v ( p a1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "pack-animal"  n ( p a1 k - a n i m l ) ((pos "K6%" ) ))
( "pack-animals"  n ( p a1 k - a n i m l z ) ((pos "Kj%" ) ))
( "pack-saddle"  n ( p a1 k - s a d l ) ((pos "K6%" ) ))
( "pack-saddles"  n ( p a1 k - s a d l z ) ((pos "Kj%" ) ))
( "pack-thread"  n ( p a1 k - th r e d ) ((pos "L@$" ) ))
( "package"  v ( p a1 k i jh ) ((pos "H2%" "K6%" ) ))
( "packaged"  v ( p a1 k i jh d ) ((pos "Hc%" "Hd%" ) ))
( "packages"  v ( p a1 k i jh i z ) ((pos "Ha%" "Kj%" ) ))
( "packaging"  v ( p a1 k i jh i ng ) ((pos "Hb%" ) ))
( "packed"  v ( p a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "packer"  n ( p a1 k @ r ) ((pos "K6%" ) ))
( "packers"  n ( p a1 k @ z ) ((pos "Kj%" ) ))
( "packet"  n ( p a1 k i t ) ((pos "K6%" ) ))
( "packet-boat"  n ( p a1 k i t - b ou t ) ((pos "K6%" ) ))
( "packet-boats"  n ( p a1 k i t - b ou t s ) ((pos "Kj%" ) ))
( "packets"  n ( p a1 k i t s ) ((pos "Kj%" ) ))
( "packhorse"  n ( p a1 k h oo s ) ((pos "K6%" ) ))
( "packhorses"  n ( p a1 k h oo s i z ) ((pos "Kj%" ) ))
( "packing"  v ( p a1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "packing-case"  n ( p a1 k i ng - k ei s ) ((pos "K6%" ) ))
( "packing-cases"  n ( p a1 k i ng - k ei s i z ) ((pos "Kj%" ) ))
( "packing-needle"  n ( p a1 k i ng - n ii d l ) ((pos "K6$" ) ))
( "packing-needles"  n ( p a1 k i ng - n ii d l z ) ((pos "Kj$" ) ))
( "packs"  v ( p a1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "pact"  n ( p a1 k t ) ((pos "K6%" ) ))
( "pacts"  n ( p a1 k t s ) ((pos "Kj$" ) ))
( "pad"  v ( p a1 d ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "padded"  v ( p a1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "paddies"  n ( p a1 d i z ) ((pos "Mj$" ) ))
( "padding"  v ( p a1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "paddle"  v ( p a1 d l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "paddle-box"  n ( p a1 d l - b o k s ) ((pos "K7$" ) ))
( "paddle-boxes"  n ( p a1 d l - b o k s i z ) ((pos "Kj$" ) ))
( "paddle-steamer"  n ( p a1 d l - s t ii m @ r ) ((pos "K6%" ) ))
( "paddle-steamers"  n ( p a1 d l - s t ii m @ z ) ((pos "Kj%" ) ))
( "paddle-wheel"  n ( p a1 d l - w ii l ) ((pos "K6%" ) ))
( "paddle-wheels"  n ( p a1 d l - w ii l z ) ((pos "Kj%" ) ))
( "paddled"  v ( p a1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "paddles"  v ( p a1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "paddling"  v ( p a1 d l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "paddock"  n ( p a1 d @ k ) ((pos "K6%" ) ))
( "paddocks"  n ( p a1 d @ k s ) ((pos "Kj%" ) ))
( "paddy"  n ( p a1 d ii ) ((pos "M8$" ) ))
( "paddy-field"  n ( p a1 d i - f ii l d ) ((pos "K6%" ) ))
( "paddy-fields"  n ( p a1 d i - f ii l d z ) ((pos "Kj%" ) ))
( "padlock"  v ( p a1 d l o k ) ((pos "H0%" "K6%" ) (subcat )))
( "padlocked"  v ( p a1 d l o k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "padlocking"  v ( p a1 d l o k i ng ) ((pos "Hb%" ) (subcat )))
( "padlocks"  v ( p a1 d l o k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "padre"  n ( p aa1 d r ei ) ((pos "K6%" ) ))
( "padres"  n ( p aa1 d r ei z ) ((pos "Kj%" ) ))
( "pads"  v ( p a1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "paean"  n ( p ii1 @ n ) ((pos "K6$" ) ))
( "paeans"  n ( p ii1 @ n z ) ((pos "Kj$" ) ))
( "paederasty"  n ( p e1 d @ r a s t ii ) ((pos "L@$" ) ))
( "paediatrics"  n ( p ii2 d i a1 t r i k s ) ((pos "Lk%" ) ))
( "paeonies"  n ( p ii1 @ n i z ) ((pos "Kj$" ) ))
( "paeony"  n ( p ii1 @ n ii ) ((pos "K8$" ) ))
( "pagan"  n ( p ei1 g @ n ) ((pos "K6%" "OA%" ) ))
( "paganism"  n ( p ei1 g @ n i z @ m ) ((pos "L@$" ) ))
( "pagans"  n ( p ei1 g @ n z ) ((pos "Kj$" ) ))
( "page"  v ( p ei1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "pageant"  n ( p a1 jh @ n t ) ((pos "K6%" ) ))
( "pageantry"  n ( p a1 jh @ n t r ii ) ((pos "L@%" ) ))
( "pageants"  n ( p a1 jh @ n t s ) ((pos "Kj%" ) ))
( "paged"  v ( p ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pages"  v ( p ei1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "pagination"  n ( p a2 jh i n ei1 sh @ n ) ((pos "L@%" ) ))
( "paging"  v ( p ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "pagoda"  n ( p @1 g ou1 d @ ) ((pos "K6%" ) ))
( "pagodas"  n ( p @1 g ou1 d @ z ) ((pos "Kj%" ) ))
( "pah"  uh ( p aa1 ) ((pos "W-$" ) ))
( "paid"  v ( p ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "12A" "12B" "13A" "13B" "14" )))
( "pail"  n ( p ei1 l ) ((pos "K6%" ) ))
( "pailful"  n ( p ei1 l f u l ) ((pos "K6%" ) ))
( "pailfuls"  n ( p ei1 l f u l z ) ((pos "Kj%" ) ))
( "paillasse"  n ( p a1 l i a s ) ((pos "K6$" ) ))
( "paillasses"  n ( p a1 l i a s i z ) ((pos "Kj$" ) ))
( "pailliasse"  n ( p a1 l i a s ) ((pos "K6$" ) ))
( "pailliasses"  n ( p a1 l i a s i z ) ((pos "Kj$" ) ))
( "pails"  n ( p ei1 l z ) ((pos "Kj%" ) ))
( "pain"  v ( p ei1 n ) ((pos "H0%" "M6%" ) (subcat )))
( "pained"  v ( p ei1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "painful"  j ( p ei1 n f @ l ) ((pos "OA%" ) ))
( "painfully"  a ( p ei1 n f @ l ii ) ((pos "Pu%" ) ))
( "paining"  v ( p ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "painkiller"  n ( p ei1 n k i l @ r ) ((pos "K6%" ) ))
( "painkillers"  n ( p ei1 n k i l @ z ) ((pos "Kj%" ) ))
( "painless"  j ( p ei1 n l @ s ) ((pos "OA%" ) ))
( "painlessly"  a ( p ei1 n l @ s l ii ) ((pos "Pu%" ) ))
( "pains"  v ( p ei1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "painstaking"  j ( p ei1 n z t ei k i ng ) ((pos "OA%" ) ))
( "painstakingly"  a ( p ei1 n z t ei k i ng l ii ) ((pos "Pu%" ) ))
( "paint"  v ( p ei1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "paintbox"  n ( p ei1 n t b o k s ) ((pos "K7%" ) ))
( "paintboxes"  n ( p ei1 n t b o k s i z ) ((pos "Kj%" ) ))
( "paintbrush"  n ( p ei1 n t b r uh sh ) ((pos "K7%" ) ))
( "paintbrushes"  n ( p ei1 n t b r uh sh i z ) ((pos "Kj%" ) ))
( "painted"  v ( p ei1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "painter"  n ( p ei1 n t @ r ) ((pos "K6%" ) ))
( "painters"  n ( p ei1 n t @ z ) ((pos "Kj%" ) ))
( "painting"  v ( p ei1 n t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "paintings"  n ( p ei1 n t i ng z ) ((pos "Mj%" ) ))
( "paints"  v ( p ei1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pair"  v ( p e@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "paired"  v ( p e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "pairing"  v ( p e@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "pairs"  v ( p e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "paisley"  n ( p ei1 z l ii ) ((pos "L@%" ) ))
( "pajamas"  n ( p @1 jh aa1 m @ z ) ((pos "Kj%" ) ))
( "pal"  v ( p a1 l ) ((pos "I4%" "K6%" ) ))
( "palace"  n ( p a1 l i s ) ((pos "K6%" ) ))
( "palaces"  n ( p a1 l i s i z ) ((pos "Kj%" ) ))
( "paladin"  n ( p a1 l @ d i n ) ((pos "K6$" ) ))
( "paladins"  n ( p a1 l @ d i n z ) ((pos "Kj$" ) ))
( "palaeo-"  j ( p a2 l i@ u - ) ((pos "U-%" ) ))
( "palaeolithic"  j ( p a2 l i@ u l i1 th i k ) ((pos "OA%" ) ))
( "palaeontologist"  n ( p a2 l i o n t o1 l @ jh i s t ) ((pos "K6%" ) ))
( "palaeontologists"  n ( p a2 l i o n t o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "palaeontology"  n ( p a2 l i o n t o1 l @ jh ii ) ((pos "L@%" ) ))
( "palankeen"  n ( p a2 l @ n k ii1 n ) ((pos "K6$" ) ))
( "palankeens"  n ( p a2 l @ n k ii1 n z ) ((pos "Kj$" ) ))
( "palanquin"  n ( p a2 l @ n k ii1 n ) ((pos "K6$" ) ))
( "palanquins"  n ( p a2 l @ n k ii1 n z ) ((pos "Kj$" ) ))
( "palatable"  j ( p a1 l @ t @ b l ) ((pos "OA%" ) ))
( "palatably"  a ( p a1 l @ t @ b l ii ) ((pos "Pu%" ) ))
( "palatal"  n ( p a1 l @ t l ) ((pos "K6$" "OA$" ) ))
( "palatals"  n ( p a1 l @ t l z ) ((pos "Kj$" ) ))
( "palate"  n ( p a1 l @ t ) ((pos "K6%" ) ))
( "palates"  n ( p a1 l @ t s ) ((pos "Kj%" ) ))
( "palatial"  j ( p @1 l ei1 sh l ) ((pos "OA%" ) ))
( "palatinate"  n ( p @1 l a1 t i n @ t ) ((pos "K6$" ) ))
( "palatinates"  n ( p @1 l a1 t i n @ t s ) ((pos "Kj$" ) ))
( "palaver"  v ( p @1 l aa1 v @ r ) ((pos "I0$" "M6$" ) (subcat )))
( "palavered"  v ( p @1 l aa1 v @ d ) ((pos "Ic$" "Id$" ) (subcat )))
( "palavering"  v ( p @1 l aa1 v @ r i ng ) ((pos "Ib$" ) (subcat )))
( "palavers"  v ( p @1 l aa1 v @ z ) ((pos "Ia$" "Mj$" ) (subcat )))
( "pale"  v ( p ei1 l ) ((pos "I2%" "K6%" "OB%" ) (subcat "2A" )))
( "paled"  v ( p ei1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "paleface"  n ( p ei1 l f ei s ) ((pos "K6%" ) ))
( "palefaces"  n ( p ei1 l f ei s i z ) ((pos "Kj%" ) ))
( "palely"  a ( p ei1 l - l ii ) ((pos "Pu$" ) ))
( "paleness"  n ( p ei1 l n @ s ) ((pos "L@%" ) ))
( "paleolithic"  j ( p a2 l i@ u l i1 th i k ) ((pos "OA%" ) ))
( "paleontologist"  n ( p a2 l i o n t o1 l @ jh i s t ) ((pos "K6%" ) ))
( "paleontologists"  n ( p a2 l i o n t o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "paleontology"  n ( p a2 l i o n t o1 l @ jh ii ) ((pos "L@%" ) ))
( "paler"  j ( p ei1 l @ r ) ((pos "Or%" ) ))
( "pales"  v ( p ei1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "palest"  j ( p ei1 l i s t ) ((pos "Os%" ) ))
( "palette"  n ( p a1 l i t ) ((pos "K6%" ) ))
( "palette-knife"  n ( p a1 l i t - n ai f ) ((pos "Ki%" ) ))
( "palette-knives"  n ( p a1 l i t - n ai v z ) ((pos "Kj%" ) ))
( "palettes"  n ( p a1 l i t s ) ((pos "Kj%" ) ))
( "palfrey"  n ( p oo1 l f r ii ) ((pos "K6$" ) ))
( "palfreys"  n ( p oo1 l f r i z ) ((pos "Kj$" ) ))
( "palimpsest"  n ( p a1 l i m p s e s t ) ((pos "K6$" ) ))
( "palimpsests"  n ( p a1 l i m p s e s t s ) ((pos "Kj$" ) ))
( "palindrome"  n ( p a1 l i n d r ou m ) ((pos "K6%" ) ))
( "palindromes"  n ( p a1 l i n d r ou m z ) ((pos "Kj%" ) ))
( "paling"  v ( p ei1 l i ng ) ((pos "Ib$" "K6$" ) (subcat "2A" )))
( "palings"  n ( p ei1 l i ng z ) ((pos "Kj%" ) ))
( "palisade"  v ( p a2 l i s ei1 d ) ((pos "H2$" "K6$" ) (subcat )))
( "palisaded"  v ( p a2 l i s ei1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "palisades"  v ( p a2 l i s ei1 d z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "palisading"  v ( p a2 l i s ei1 d i ng ) ((pos "Hb$" ) (subcat )))
( "palish"  j ( p ei1 l i sh ) ((pos "OA$" ) ))
( "pall"  v ( p oo1 l ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "pallbearer"  n ( p oo1 l b e@ r @ r ) ((pos "K6%" ) ))
( "pallbearers"  n ( p oo1 l b e@ r @ z ) ((pos "Kj%" ) ))
( "palled"  v ( p a1 l d ) ((pos "Ic$" "Id$" ) ))
( "palled"  v ( p oo1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "pallet"  n ( p a1 l i t ) ((pos "K6%" ) ))
( "pallets"  n ( p a1 l i t s ) ((pos "Kj%" ) ))
( "palliasse"  n ( p a1 l i a s ) ((pos "K6$" ) ))
( "palliasses"  n ( p a1 l i a s i z ) ((pos "Kj$" ) ))
( "palliate"  v ( p a1 l i ei t ) ((pos "H2$" ) (subcat )))
( "palliated"  v ( p a1 l i ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "palliates"  v ( p a1 l i ei t s ) ((pos "Ha$" ) (subcat )))
( "palliating"  v ( p a1 l i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "palliation"  n ( p a2 l i ei1 sh @ n ) ((pos "M6$" ) ))
( "palliations"  n ( p a2 l i ei1 sh @ n z ) ((pos "Mj$" ) ))
( "palliative"  n ( p a1 l i@ t i v ) ((pos "K6%" "OA%" ) ))
( "palliatives"  n ( p a1 l i@ t i v z ) ((pos "Kj%" ) ))
( "pallid"  j ( p a1 l i d ) ((pos "OA%" ) ))
( "pallidly"  a ( p a1 l i d l ii ) ((pos "Pu%" ) ))
( "pallidness"  n ( p a1 l i d n @ s ) ((pos "L@$" ) ))
( "pallier"  j ( p a1 l i@ r ) ((pos "Or$" ) ))
( "palliest"  j ( p a1 l i i s t ) ((pos "Os$" ) ))
( "palling"  v ( p a1 l i ng ) ((pos "Ib%" ) ))
( "palling"  v ( p oo1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "pallor"  n ( p a1 l @ r ) ((pos "L@%" ) ))
( "palls"  v ( p oo1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "pally"  j ( p a1 l ii ) ((pos "OD%" ) ))
( "palm"  v ( p aa1 m ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "palm-oil"  n ( p aa1 m - oi l ) ((pos "L@%" ) ))
( "palmed"  v ( p aa1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "palmer"  n ( p aa1 m @ r ) ((pos "K6$" ) ))
( "palmers"  n ( p aa1 m @ z ) ((pos "Kj$" ) ))
( "palmetto"  n ( p a1 l m e1 t ou ) ((pos "K6$" ) ))
( "palmettoes"  n ( p a1 l m e1 t ou z ) ((pos "Kj$" ) ))
( "palmettos"  n ( p a1 l m e1 t ou z ) ((pos "Kj$" ) ))
( "palmier"  j ( p aa1 m i@ r ) ((pos "Or$" ) ))
( "palmiest"  j ( p aa1 m i i s t ) ((pos "Os$" ) ))
( "palming"  v ( p aa1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "palmist"  n ( p aa1 m i s t ) ((pos "K6%" ) ))
( "palmistry"  n ( p aa1 m i s t r ii ) ((pos "L@%" ) ))
( "palmists"  n ( p aa1 m i s t s ) ((pos "Kj%" ) ))
( "palms"  v ( p aa1 m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "palmy"  j ( p aa1 m ii ) ((pos "OD%" ) ))
( "palpable"  j ( p a1 l p @ b l ) ((pos "OA%" ) ))
( "palpably"  a ( p a1 l p @ b l ii ) ((pos "Pu%" ) ))
( "palpitate"  v ( p a1 l p i t ei t ) ((pos "I2%" ) (subcat "2A" )))
( "palpitated"  v ( p a1 l p i t ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "palpitates"  v ( p a1 l p i t ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "palpitating"  v ( p a1 l p i t ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "palpitation"  n ( p a2 l p i t ei1 sh @ n ) ((pos "K6%" ) ))
( "palpitations"  n ( p a2 l p i t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "pals"  v ( p a1 l z ) ((pos "Ia%" "Kj%" ) ))
( "palsied"  v ( p oo1 l z i d ) ((pos "Hc$" "Hd%" ) ))
( "palsies"  v ( p oo1 l z i z ) ((pos "Ha$" ) ))
( "palsy"  v ( p oo1 l z ii ) ((pos "H3$" "L@%" ) ))
( "palsying"  v ( p oo1 l z i i ng ) ((pos "Hb$" ) ))
( "palter"  v ( p oo1 l t @ r ) ((pos "I0$" ) (subcat )))
( "paltered"  v ( p oo1 l t @ d ) ((pos "Ic$" "Id$" ) (subcat )))
( "paltering"  v ( p oo1 l t @ r i ng ) ((pos "Ib$" ) (subcat )))
( "palters"  v ( p oo1 l t @ z ) ((pos "Ia$" ) (subcat )))
( "paltrier"  j ( p oo1 l t r i@ r ) ((pos "Or%" ) ))
( "paltriest"  j ( p oo1 l t r i i s t ) ((pos "Os%" ) ))
( "paltry"  j ( p oo1 l t r ii ) ((pos "OD%" ) ))
( "pampas"  n ( p a1 m p @ s ) ((pos "Kj%" ) ))
( "pampas-grass"  n ( p a1 m p @ s - g r aa s ) ((pos "L@%" ) ))
( "pamper"  v ( p a1 m p @ r ) ((pos "H0%" ) (subcat )))
( "pampered"  v ( p a1 m p @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pampering"  v ( p a1 m p @ r i ng ) ((pos "Hb%" ) (subcat )))
( "pampers"  v ( p a1 m p @ z ) ((pos "Ha%" ) (subcat )))
( "pamphlet"  n ( p a1 m f l i t ) ((pos "K6%" ) ))
( "pamphleteer"  n ( p a2 m f l @ t i@1 r ) ((pos "K6%" ) ))
( "pamphleteers"  n ( p a2 m f l @ t i@1 z ) ((pos "Kj%" ) ))
( "pamphlets"  n ( p a1 m f l i t s ) ((pos "Kj%" ) ))
( "pan"  v ( p a1 n ) ((pos "J4%" "M6%" ) (subcat "2C" "3A" "6A" )))
( "pan-"  j ( p a n - ) ((pos "U-%" ) ))
( "pan-pipes"  n ( p a1 n - p ai p s ) ((pos "Kj%" ) ))
( "panacea"  n ( p a2 n @ s i@1 ) ((pos "K6%" ) ))
( "panaceas"  n ( p a2 n @ s i@1 z ) ((pos "Kj%" ) ))
( "panache"  n ( p a1 n a1 sh ) ((pos "L@%" ) ))
( "panama"  n ( p a1 n @ m aa ) ((pos "K6$" ) ))
( "panamas"  n ( p a1 n @ m aa z ) ((pos "Kj$" ) ))
( "panatella"  n ( p a2 n @ t e1 l @ ) ((pos "K6$" ) ))
( "panatellas"  n ( p a2 n @ t e1 l @ z ) ((pos "Kj$" ) ))
( "pancake"  n ( p a1 n k ei k ) ((pos "K6%" ) ))
( "pancakes"  n ( p a1 n k ei k s ) ((pos "Kj%" ) ))
( "panchromatic"  j ( p a2 n k r @ m a1 t i k ) ((pos "OA$" ) ))
( "pancreas"  n ( p a1 ng k r i@ s ) ((pos "K7%" ) ))
( "pancreases"  n ( p a1 ng k r i@ s i z ) ((pos "Kj$" ) ))
( "pancreatic"  j ( p a2 ng k r i a1 t i k ) ((pos "OA%" ) ))
( "panda"  n ( p a1 n d @ ) ((pos "K6%" ) ))
( "pandas"  n ( p a1 n d @ z ) ((pos "Kj%" ) ))
( "pandemic"  n ( p a1 n d e1 m i k ) ((pos "K6$" "OA$" ) ))
( "pandemics"  n ( p a1 n d e1 m i k s ) ((pos "Kj$" ) ))
( "pandemonium"  n ( p a2 n d i m ou1 n i@ m ) ((pos "M6%" ) ))
( "pandemoniums"  n ( p a2 n d i m ou1 n i@ m z ) ((pos "Mj$" ) ))
( "pander"  v ( p a1 n d @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "pandered"  v ( p a1 n d @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "pandering"  v ( p a1 n d @ r i ng ) ((pos "Ib%" ) (subcat )))
( "panders"  v ( p a1 n d @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "pane"  n ( p ei1 n ) ((pos "K6%" ) ))
( "panegyric"  n ( p a2 n i jh i1 r i k ) ((pos "K6%" ) ))
( "panegyrics"  n ( p a2 n i jh i1 r i k s ) ((pos "Kj%" ) ))
( "panel"  v ( p a1 n l ) ((pos "H4%" "K6%" ) ))
( "panelled"  v ( p a1 n l d ) ((pos "Hc%" "Hd%" ) ))
( "panelling"  v ( p a1 n @ l i ng ) ((pos "Hb%" "L@%" ) ))
( "panels"  v ( p a1 n l z ) ((pos "Ha%" "Kj%" ) ))
( "panes"  n ( p ei1 n z ) ((pos "Kj%" ) ))
( "pang"  n ( p a1 ng ) ((pos "K6%" ) ))
( "panga"  n ( p a1 ng g @ ) ((pos "K6$" ) ))
( "pangas"  n ( p a1 ng g @ z ) ((pos "Kj$" ) ))
( "pangs"  n ( p a1 ng z ) ((pos "Kj%" ) ))
( "panhandle"  v ( p a1 n h a n d l ) ((pos "I2$" "K6$" ) ))
( "panhandled"  v ( p a1 n h a n d l d ) ((pos "Ic$" "Id$" ) ))
( "panhandles"  v ( p a1 n h a n d l z ) ((pos "Ia$" "Kj$" ) ))
( "panhandling"  v ( p a1 n h a n d l i ng ) ((pos "Ib$" ) ))
( "panic"  v ( p a1 n i k ) ((pos "I5%" "M6%" ) ))
( "panic-stricken"  j ( p a1 n i k - s t r i k @ n ) ((pos "OA%" ) ))
( "panicked"  v ( p a1 n i k t ) ((pos "Ic%" "Id%" ) ))
( "panicking"  v ( p a1 n i k i ng ) ((pos "Ib%" ) ))
( "panicky"  j ( p a1 n i k ii ) ((pos "OA%" ) ))
( "panics"  v ( p a1 n i k s ) ((pos "Ia%" "Mj%" ) ))
( "panjandrum"  n ( p a1 n jh a1 n d r @ m ) ((pos "K6$" ) ))
( "panjandrums"  n ( p a1 n jh a1 n d r @ m z ) ((pos "Kj$" ) ))
( "panned"  v ( p a1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "pannier"  n ( p a1 n i@ r ) ((pos "K6%" ) ))
( "panniers"  n ( p a1 n i@ z ) ((pos "Kj%" ) ))
( "pannikin"  n ( p a1 n i k i n ) ((pos "K6$" ) ))
( "pannikins"  n ( p a1 n i k i n z ) ((pos "Kj$" ) ))
( "panning"  v ( p a1 n i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "panoplied"  j ( p a1 n @ p l i d ) ((pos "OA%" ) ))
( "panoplies"  n ( p a1 n @ p l i z ) ((pos "Kj%" ) ))
( "panoply"  n ( p a1 n @ p l ii ) ((pos "K8%" ) ))
( "panoptic"  j ( p a1 n o1 p t i k ) ((pos "OA$" ) ))
( "panorama"  n ( p a2 n @ r aa1 m @ ) ((pos "K6%" ) ))
( "panoramas"  n ( p a2 n @ r aa1 m @ z ) ((pos "Kj%" ) ))
( "panoramic"  j ( p a2 n @ r a1 m i k ) ((pos "OA%" ) ))
( "pans"  v ( p a1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" )))
( "pansies"  n ( p a1 n z i z ) ((pos "Kj%" ) ))
( "pansy"  n ( p a1 n z ii ) ((pos "K8%" ) ))
( "pant"  v ( p a1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "pantaloon"  n ( p a2 n t @ l uu1 n ) ((pos "K6$" ) ))
( "pantaloons"  n ( p a2 n t @ l uu1 n z ) ((pos "Kj$" ) ))
( "pantechnicon"  n ( p a1 n t e1 k n i k @ n ) ((pos "K6%" ) ))
( "pantechnicons"  n ( p a1 n t e1 k n i k @ n z ) ((pos "Kj$" ) ))
( "panted"  v ( p a1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "pantheism"  n ( p a1 n th i i z @ m ) ((pos "L@%" ) ))
( "pantheist"  n ( p a1 n th i i s t ) ((pos "K6%" ) ))
( "pantheistic"  j ( p a2 n th i i1 s t i k ) ((pos "OA%" ) ))
( "pantheists"  n ( p a1 n th i i s t s ) ((pos "Kj$" ) ))
( "pantheon"  n ( p a1 n th i@ n ) ((pos "K6%" ) ))
( "pantheons"  n ( p a1 n th i@ n z ) ((pos "Kj$" ) ))
( "panther"  n ( p a1 n th @ r ) ((pos "K6%" ) ))
( "panthers"  n ( p a1 n th @ z ) ((pos "Kj%" ) ))
( "panties"  n ( p a1 n t i z ) ((pos "Kj%" ) ))
( "pantile"  n ( p a1 n t ai l ) ((pos "K6$" ) ))
( "pantiles"  n ( p a1 n t ai l z ) ((pos "Kj$" ) ))
( "panting"  v ( p a1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "pantingly"  a ( p a1 n t i ng l ii ) ((pos "Pu$" ) ))
( "panto"  n ( p a1 n t ou ) ((pos "K6%" ) ))
( "pantograph"  n ( p a1 n t @ g r aa f ) ((pos "K6$" ) ))
( "pantographs"  n ( p a1 n t @ g r aa f s ) ((pos "Kj$" ) ))
( "pantomime"  n ( p a1 n t @ m ai m ) ((pos "M6%" ) ))
( "pantomimes"  n ( p a1 n t @ m ai m z ) ((pos "Mj%" ) ))
( "pantos"  n ( p a1 n t ou z ) ((pos "Kj%" ) ))
( "pantries"  n ( p a1 n t r i z ) ((pos "Kj%" ) ))
( "pantry"  n ( p a1 n t r ii ) ((pos "K8%" ) ))
( "pantryman"  n ( p a1 n t r i m a n ) ((pos "Ki$" ) ))
( "pantrymen"  n ( p a1 n t r i m e n ) ((pos "Kj$" ) ))
( "pants"  v ( p a1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "panty-hose"  n ( p a1 n t i - h ou z ) ((pos "L@%" ) ))
( "panzer"  j ( p a1 n t s @ r ) ((pos "Oq$" ) ))
( "pap"  n ( p a1 p ) ((pos "L@%" ) ))
( "papa"  n ( p @1 p aa1 ) ((pos "K6%" ) ))
( "papacies"  n ( p ei1 p @ s i z ) ((pos "Kj$" ) ))
( "papacy"  n ( p ei1 p @ s ii ) ((pos "K8%" ) ))
( "papal"  j ( p ei1 p l ) ((pos "OA%" ) ))
( "papas"  n ( p @1 p aa1 z ) ((pos "Kj$" ) ))
( "papaw"  n ( p oo1 p oo ) ((pos "K6$" ) ))
( "papaws"  n ( p oo1 p oo1 z ) ((pos "Kj$" ) ))
( "papaya"  n ( p @1 p ai1 @ ) ((pos "K6$" ) ))
( "papayas"  n ( p @1 p ai1 @ z ) ((pos "Kj$" ) ))
( "paper"  v ( p ei1 p @ r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "paper-chase"  n ( p ei1 p @ - ch ei s ) ((pos "K6%" ) ))
( "paper-chases"  n ( p ei1 p @ - ch ei s i z ) ((pos "Kj$" ) ))
( "paperback"  n ( p ei1 p @ b a k ) ((pos "K6%" ) ))
( "paperbacked"  j ( p ei1 p @ b a k t ) ((pos "OA%" ) ))
( "paperbacks"  n ( p ei1 p @ b a k s ) ((pos "Kj%" ) ))
( "paperclip"  n ( p ei1 p @ k l i p ) ((pos "K6%" ) ))
( "paperclips"  n ( p ei1 p @ k l i p s ) ((pos "Kj%" ) ))
( "papered"  v ( p ei1 p @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "paperhanger"  n ( p ei1 p @ h a ng @ r ) ((pos "K6$" ) ))
( "paperhangers"  n ( p ei1 p @ h a ng @ z ) ((pos "Kj$" ) ))
( "papering"  v ( p ei1 p @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "paperknife"  n ( p ei1 p @ n ai f ) ((pos "Ki%" ) ))
( "paperknives"  n ( p ei1 p @ n ai v z ) ((pos "Kj%" ) ))
( "paperless"  j ( p ei1 p @ l i s ) ((pos "OA%" ) ))
( "papermill"  n ( p ei1 p @ m i l ) ((pos "K6%" ) ))
( "papermills"  n ( p ei1 p @ m i l z ) ((pos "Kj%" ) ))
( "papers"  v ( p ei1 p @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "paperweight"  n ( p ei1 p @ w ei t ) ((pos "K6%" ) ))
( "paperweights"  n ( p ei1 p @ w ei t s ) ((pos "Kj%" ) ))
( "paperwork"  n ( p ei1 p @ w @@ k ) ((pos "L@%" ) ))
( "papier-m^ach_e"  n ( p a2 p i ei - m a1 sh ei ) ((pos "L@%" ) ))
( "papist"  n ( p ei1 p i s t ) ((pos "K6%" "OA%" ) ))
( "papists"  n ( p ei1 p i s t s ) ((pos "Kj%" ) ))
( "papoose"  n ( p @1 p uu1 s ) ((pos "K6$" ) ))
( "papooses"  n ( p @1 p uu1 s i z ) ((pos "Kj$" ) ))
( "paprika"  n ( p a1 p r i k @ ) ((pos "L@%" ) ))
( "papyri"  n ( p @1 p ai1 r ai ) ((pos "Kj$" ) ))
( "papyrus"  n ( p @1 p ai1 r @ s ) ((pos "M7$" ) ))
( "papyruses"  n ( p @1 p ai1 r @ s i z ) ((pos "Mj$" ) ))
( "par"  n ( p aa1 r ) ((pos "M6%" ) ))
( "par excellence"  a ( p aa2 r - e1 k s @ l aa n s ) ((pos "Pu%" ) ))
( "para"  n ( p a1 r @ ) ((pos "Y>%" ) ))
( "parable"  n ( p a1 r @ b l ) ((pos "K6%" ) ))
( "parables"  n ( p a1 r @ b l z ) ((pos "Kj%" ) ))
( "parabola"  n ( p @1 r a1 b @ l @ ) ((pos "K6%" ) ))
( "parabolas"  n ( p @1 r a1 b @ l @ z ) ((pos "Kj%" ) ))
( "parabolic"  j ( p a2 r @ b o1 l i k ) ((pos "OA%" ) ))
( "parabolical"  j ( p a2 r @ b o1 l i k l ) ((pos "OA$" ) ))
( "parachute"  v ( p a1 r @ sh uu t ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "parachuted"  v ( p a1 r @ sh uu t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "parachutes"  v ( p a1 r @ sh uu t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "parachuting"  v ( p a1 r @ sh uu t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "parachutist"  n ( p a1 r @ sh uu t i s t ) ((pos "K6%" ) ))
( "parachutists"  n ( p a1 r @ sh uu t i s t s ) ((pos "Kj%" ) ))
( "parade"  v ( p @1 r ei1 d ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "parade-ground"  n ( p @1 r ei1 d - g r au n d ) ((pos "K6%" ) ))
( "parade-grounds"  n ( p @1 r ei1 d - g r au n d z ) ((pos "Kj%" ) ))
( "paraded"  v ( p @1 r ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "parades"  v ( p @1 r ei1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "paradigm"  n ( p a1 r @ d ai m ) ((pos "K6%" ) ))
( "paradigms"  n ( p a1 r @ d ai m z ) ((pos "Kj%" ) ))
( "parading"  v ( p @1 r ei1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "paradise"  n ( p a1 r @ d ai s ) ((pos "M6%" ) ))
( "paradises"  n ( p a1 r @ d ai s i z ) ((pos "Mj$" ) ))
( "paradisiac"  j ( p a2 r @ d i1 z i a k ) ((pos "OA$" ) ))
( "paradisiacal"  j ( p a2 r @ d i z ai1 @ k l ) ((pos "OA$" ) ))
( "paradox"  n ( p a1 r @ d o k s ) ((pos "K7%" ) ))
( "paradoxes"  n ( p a1 r @ d o k s i z ) ((pos "Kj%" ) ))
( "paradoxical"  j ( p a2 r @ d o1 k s i k l ) ((pos "OA%" ) ))
( "paradoxically"  a ( p a2 r @ d o1 k s i k l ii ) ((pos "Pu%" ) ))
( "paraffin"  n ( p a1 r @ f i n ) ((pos "L@%" ) ))
( "paragon"  n ( p a1 r @ g @ n ) ((pos "K6%" ) ))
( "paragons"  n ( p a1 r @ g @ n z ) ((pos "Kj%" ) ))
( "paragraph"  v ( p a1 r @ g r aa f ) ((pos "H0%" "K6%" ) ))
( "paragraphed"  v ( p a1 r @ g r aa f t ) ((pos "Hc%" "Hd%" ) ))
( "paragraphing"  v ( p a1 r @ g r aa f i ng ) ((pos "Hb%" ) ))
( "paragraphs"  v ( p a1 r @ g r aa f s ) ((pos "Ha%" "Kj%" ) ))
( "parakeet"  n ( p a1 r @ k ii t ) ((pos "K6%" ) ))
( "parakeets"  n ( p a1 r @ k ii t s ) ((pos "Kj%" ) ))
( "parallel"  v ( p a1 r @ l e l ) ((pos "H4%" "M6%" "OA%" ) (subcat )))
( "paralleled"  v ( p a1 r @ l e l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "paralleling"  v ( p a1 r @ l e l i ng ) ((pos "Hb%" ) (subcat )))
( "parallelism"  n ( p a1 r @ l e l i z @ m ) ((pos "M6%" ) ))
( "parallelisms"  n ( p a1 r @ l e l i z @ m z ) ((pos "Mj$" ) ))
( "parallelogram"  n ( p a2 r @ l e1 l @ g r a m ) ((pos "K6%" ) ))
( "parallelograms"  n ( p a2 r @ l e1 l @ g r a m z ) ((pos "Kj%" ) ))
( "parallels"  v ( p a1 r @ l e l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "paralyse"  v ( p a1 r @ l ai z ) ((pos "H2%" ) (subcat )))
( "paralysed"  v ( p a1 r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "paralyses"  v ( p a1 r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "paralysing"  v ( p a1 r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "paralysis"  n ( p @1 r a1 l @ s i s ) ((pos "L@%" ) ))
( "paralytic"  n ( p a2 r @ l i1 t i k ) ((pos "K6%" "OA%" ) ))
( "paralytics"  n ( p a2 r @ l i1 t i k s ) ((pos "Kj%" ) ))
( "paralyze"  v ( p a1 r @ l ai z ) ((pos "H2%" ) (subcat )))
( "paralyzed"  v ( p a1 r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "paralyzes"  v ( p a1 r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "paralyzing"  v ( p a1 r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "parameter"  n ( p @1 r a1 m i t @ r ) ((pos "K6%" ) ))
( "parameters"  n ( p @1 r a1 m i t @ z ) ((pos "Kj%" ) ))
( "paramilitary"  j ( p a2 r @ m i1 l i t r ii ) ((pos "OA%" ) ))
( "paramount"  j ( p a1 r @ m au n t ) ((pos "OA%" ) ))
( "paramountcy"  n ( p a1 r @ m au n t s ii ) ((pos "L@$" ) ))
( "paramour"  n ( p a1 r @ m u@ r ) ((pos "K6%" ) ))
( "paramours"  n ( p a1 r @ m u@ z ) ((pos "Kj%" ) ))
( "paranoia"  n ( p a2 r @ n oi1 @ ) ((pos "L@%" ) ))
( "paranoiac"  n ( p a2 r @ n oi1 a k ) ((pos "K6%" ) ))
( "paranoiacs"  n ( p a2 r @ n oi1 a k s ) ((pos "Kj%" ) ))
( "paranoid"  n ( p a1 r @ n oi d ) ((pos "K6$" "OA%" ) ))
( "paranoids"  n ( p a1 r @ n oi d z ) ((pos "Kj$" ) ))
( "parapet"  n ( p a1 r @ p i t ) ((pos "K6%" ) ))
( "parapets"  n ( p a1 r @ p i t s ) ((pos "Kj%" ) ))
( "paraphernalia"  n ( p a2 r @ f @ n ei1 l i@ ) ((pos "L@%" ) ))
( "paraphrase"  v ( p a1 r @ f r ei z ) ((pos "H2%" ) (subcat )))
( "paraphrased"  v ( p a1 r @ f r ei z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "paraphrases"  v ( p a1 r @ f r ei z i z ) ((pos "Ha%" ) (subcat )))
( "paraphrasing"  v ( p a1 r @ f r ei z i ng ) ((pos "Hb%" ) (subcat )))
( "paraplegia"  n ( p a2 r @ p l ii1 jh @ ) ((pos "L@%" ) ))
( "paraplegic"  n ( p a2 r @ p l ii1 jh i k ) ((pos "K6%" "OA%" ) ))
( "paraplegics"  n ( p a2 r @ p l ii1 jh i k s ) ((pos "Kj%" ) ))
( "parasite"  n ( p a1 r @ s ai t ) ((pos "K6%" ) ))
( "parasites"  n ( p a1 r @ s ai t s ) ((pos "Kj%" ) ))
( "parasitic"  j ( p a2 r @ s i1 t i k ) ((pos "OA%" ) ))
( "parasitical"  j ( p a2 r @ s i1 t i k l ) ((pos "OA%" ) ))
( "parasol"  n ( p a1 r @ s o l ) ((pos "K6%" ) ))
( "parasols"  n ( p a1 r @ s o l z ) ((pos "Kj%" ) ))
( "paratrooper"  n ( p a1 r @ t r uu p @ r ) ((pos "K6%" ) ))
( "paratroopers"  n ( p a1 r @ t r uu p @ z ) ((pos "Kj%" ) ))
( "paratroops"  n ( p a1 r @ t r uu p s ) ((pos "Kj%" ) ))
( "paratyphoid"  n ( p a2 r @ t ai1 f oi d ) ((pos "L@%" ) ))
( "parboil"  v ( p aa1 b oi l ) ((pos "H0%" ) (subcat )))
( "parboiled"  v ( p aa1 b oi l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "parboiling"  v ( p aa1 b oi l i ng ) ((pos "Hb$" ) (subcat )))
( "parboils"  v ( p aa1 b oi l z ) ((pos "Ha$" ) (subcat )))
( "parcel"  v ( p aa1 s l ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "parcelled"  v ( p aa1 s l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "parcelling"  v ( p aa1 s l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "parcels"  v ( p aa1 s l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "parch"  v ( p aa1 ch ) ((pos "H1$" ) (subcat )))
( "parched"  v ( p aa1 ch t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "parches"  v ( p aa1 ch i z ) ((pos "Ha$" ) (subcat )))
( "parching"  v ( p aa1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "parchment"  n ( p aa1 ch m @ n t ) ((pos "M6%" ) ))
( "parchments"  n ( p aa1 ch m @ n t s ) ((pos "Mj%" ) ))
( "pardon"  v ( p aa1 d n ) ((pos "H0%" "M6%" ) (subcat "6A" "12B" )))
( "pardonable"  j ( p aa1 d n @ b l ) ((pos "OA%" ) ))
( "pardonably"  a ( p aa1 d n @ b l ii ) ((pos "Pu%" ) ))
( "pardoned"  v ( p aa1 d n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12B" )))
( "pardoner"  n ( p aa1 d @ n @ r ) ((pos "K6%" ) ))
( "pardoners"  n ( p aa1 d @ n @ z ) ((pos "Kj%" ) ))
( "pardoning"  v ( p aa1 d @ n i ng ) ((pos "Hb%" ) (subcat "6A" "12B" )))
( "pardons"  v ( p aa1 d n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "12B" )))
( "pare"  v ( p e@1 r ) ((pos "H2%" ) (subcat "6A" )))
( "pared"  v ( p e@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "paregoric"  n ( p a2 r @ g o1 r i k ) ((pos "L@$" ) ))
( "parent"  n ( p e@1 r @ n t ) ((pos "K6%" ) ))
( "parentage"  n ( p e@1 r @ n t i jh ) ((pos "L@%" ) ))
( "parental"  j ( p @1 r e1 n t l ) ((pos "OA%" ) ))
( "parentally"  a ( p @1 r e1 n t @ l ii ) ((pos "Pu%" ) ))
( "parentheses"  n ( p @1 r e1 n th @ s ii z ) ((pos "Kj%" ) ))
( "parenthesis"  n ( p @1 r e1 n th @ s i s ) ((pos "Ki%" ) ))
( "parenthetic"  j ( p a2 r @ n th e1 t i k ) ((pos "OA$" ) ))
( "parenthetical"  j ( p a2 r @ n th e1 t i k l ) ((pos "OA$" ) ))
( "parenthetically"  a ( p a2 r @ n th e1 t i k l ii ) ((pos "Pu%" ) ))
( "parents"  n ( p e@1 r @ n t s ) ((pos "Kj%" ) ))
( "pares"  v ( p e@1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "pari passu"  a ( p a2 r i - p a1 s uu ) ((pos "Pu$" ) ))
( "pari-mutuel"  n ( p a2 r i - m y uu1 t y u e l ) ((pos "K6$" ) ))
( "pari-mutuels"  n ( p a2 r i - m y uu1 t y u e l z ) ((pos "Kj$" ) ))
( "pariah"  n ( p @1 r ai1 @ ) ((pos "K6%" ) ))
( "pariah-dog"  n ( p @1 - r ai1 @ d o g ) ((pos "K6$" ) ))
( "pariah-dogs"  n ( p @1 - r ai1 @ d o g z ) ((pos "Kj$" ) ))
( "pariahs"  n ( p @1 r ai1 @ z ) ((pos "Kj%" ) ))
( "paring"  v ( p e@1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "parings"  n ( p e@1 r i ng z ) ((pos "Kj%" ) ))
( "parish"  n ( p a1 r i sh ) ((pos "K7%" ) ))
( "parishes"  n ( p a1 r i sh i z ) ((pos "Kj%" ) ))
( "parishioner"  n ( p @1 r i1 sh @ n @ r ) ((pos "K6%" ) ))
( "parishioners"  n ( p @1 r i1 sh @ n @ z ) ((pos "Kj%" ) ))
( "parity"  n ( p a1 r i t ii ) ((pos "L@%" ) ))
( "park"  v ( p aa1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "parka"  n ( p aa1 k @ ) ((pos "K6$" ) ))
( "parkas"  n ( p aa1 k @ z ) ((pos "Kj$" ) ))
( "parked"  v ( p aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "parking"  v ( p aa1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "6A" )))
( "parks"  v ( p aa1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "parky"  j ( p aa1 k ii ) ((pos "OA%" ) ))
( "parlance"  n ( p aa1 l @ n s ) ((pos "K6%" ) ))
( "parlances"  n ( p aa1 l @ n s i z ) ((pos "Kj$" ) ))
( "parley"  v ( p aa1 l ii ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "parleyed"  v ( p aa1 l i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "parleying"  v ( p aa1 l i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "parleys"  v ( p aa1 l i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "parliament"  n ( p aa1 l @ m @ n t ) ((pos "K6%" ) ))
( "parliamentarian"  n ( p aa2 l @ m @ n t e@1 r i@ n ) ((pos "K6%" ) ))
( "parliamentarians"  n ( p aa2 l @ m @ n t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "parliamentary"  j ( p aa2 l @ m e1 n t r ii ) ((pos "OA%" ) ))
( "parliaments"  n ( p aa1 l @ m @ n t s ) ((pos "Kj%" ) ))
( "parlour"  n ( p aa1 l @ r ) ((pos "K6%" ) ))
( "parlour-car"  n ( p aa1 l @ - k aa r ) ((pos "K6$" ) ))
( "parlour-cars"  n ( p aa1 l @ - k aa z ) ((pos "Kj$" ) ))
( "parlours"  n ( p aa1 l @ z ) ((pos "Kj%" ) ))
( "parlous"  j ( p aa1 l @ s ) ((pos "OA$" ) ))
( "parochial"  j ( p @1 r ou1 k i@ l ) ((pos "OA%" ) ))
( "parochialism"  n ( p @1 r ou1 k i@ l i z @ m ) ((pos "L@%" ) ))
( "parochially"  a ( p @1 r ou1 k i@ l ii ) ((pos "Pu%" ) ))
( "parodied"  v ( p a1 r @ d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "parodies"  v ( p a1 r @ d i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "parodist"  n ( p a1 r @ d i s t ) ((pos "K6$" ) ))
( "parodists"  n ( p a1 r @ d i s t s ) ((pos "Kj$" ) ))
( "parody"  v ( p a1 r @ d ii ) ((pos "H3%" "M8%" ) (subcat )))
( "parodying"  v ( p a1 r @ d i i ng ) ((pos "Hb%" ) (subcat )))
( "parole"  v ( p @1 r ou1 l ) ((pos "H2$" "L@%" ) (subcat )))
( "paroled"  v ( p @1 r ou1 l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "paroles"  v ( p @1 r ou1 l z ) ((pos "Ha$" ) (subcat )))
( "paroling"  v ( p @1 r ou1 l i ng ) ((pos "Hb$" ) (subcat )))
( "paroquet"  n ( p a1 r @ k ii t ) ((pos "K6$" ) ))
( "paroquets"  n ( p a1 r @ k ii t s ) ((pos "Kj$" ) ))
( "paroxysm"  n ( p a1 r @ k s i z @ m ) ((pos "K6%" ) ))
( "paroxysms"  n ( p a1 r @ k s i z @ m z ) ((pos "Kj%" ) ))
( "parquet"  n ( p aa1 k ei ) ((pos "L@%" ) ))
( "parr"  n ( p aa1 r ) ((pos "K6$" ) ))
( "parricide"  n ( p a1 r i s ai d ) ((pos "M6$" ) ))
( "parricides"  n ( p a1 r i s ai d z ) ((pos "Mj$" ) ))
( "parried"  v ( p a1 r i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "parries"  v ( p a1 r i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "parrot"  n ( p a1 r @ t ) ((pos "K6%" ) ))
( "parrots"  n ( p a1 r @ t s ) ((pos "Kj%" ) ))
( "parrs"  n ( p aa1 z ) ((pos "Kj$" ) ))
( "parry"  v ( p a1 r ii ) ((pos "H3%" "K8%" ) (subcat )))
( "parrying"  v ( p a1 r i i ng ) ((pos "Hb%" ) (subcat )))
( "pars"  n ( p aa1 z ) ((pos "Mj$" ) ))
( "parse"  v ( p aa1 z ) ((pos "H2%" ) (subcat )))
( "parsed"  v ( p aa1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "parses"  v ( p aa1 z i z ) ((pos "Ha%" ) (subcat )))
( "parsimonious"  j ( p aa2 s i m ou1 n i@ s ) ((pos "OA%" ) ))
( "parsimony"  n ( p aa1 s i m @ n ii ) ((pos "L@%" ) ))
( "parsing"  v ( p aa1 z i ng ) ((pos "Hb%" ) (subcat )))
( "parsley"  n ( p aa1 s l ii ) ((pos "L@%" ) ))
( "parsnip"  n ( p aa1 s n i p ) ((pos "K6%" ) ))
( "parsnips"  n ( p aa1 s n i p s ) ((pos "Kj%" ) ))
( "parson"  n ( p aa1 s n ) ((pos "K6%" ) ))
( "parsonage"  n ( p aa1 s @ n i jh ) ((pos "K6%" ) ))
( "parsonages"  n ( p aa1 s @ n i jh i z ) ((pos "Kj$" ) ))
( "parsons"  n ( p aa1 s n z ) ((pos "Kj%" ) ))
( "part"  v ( p aa1 t ) ((pos "J0%" "K6*" "Pu%" ) (subcat "2A" "2D" "3A" )))
( "part-owner"  n ( p aa1 t - ou1 n @ r ) ((pos "K6%" ) ))
( "part-owners"  n ( p aa1 t - ou1 n @ z ) ((pos "Kj%" ) ))
( "part-singing"  n ( p aa1 t - s i ng i ng ) ((pos "L@%" ) ))
( "part-song"  n ( p aa1 t - s o ng ) ((pos "K6%" ) ))
( "part-songs"  n ( p aa1 t - s o ng z ) ((pos "Kj%" ) ))
( "part-time"  j ( p aa1 t - t ai m ) ((pos "OA%" "Pu%" ) ))
( "part-timer"  n ( p aa1 t - t ai1 m @ r ) ((pos "K6%" ) ))
( "part-timers"  n ( p aa1 t - t ai1 m @ z ) ((pos "Kj%" ) ))
( "partake"  v ( p aa1 t ei1 k ) ((pos "J5%" ) (subcat )))
( "partaken"  v ( p aa1 t ei1 k @ n ) ((pos "Jd%" ) (subcat )))
( "partakes"  v ( p aa1 t ei1 k s ) ((pos "Ja%" ) (subcat )))
( "partaking"  v ( p aa1 t ei1 k i ng ) ((pos "Jb%" ) (subcat )))
( "parted"  v ( p aa1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2D" "3A" )))
( "parterre"  n ( p aa1 t e@1 r ) ((pos "K6$" ) ))
( "parterres"  n ( p aa1 t e@1 z ) ((pos "Kj$" ) ))
( "parthenogenesis"  n ( p aa2 th i n ou jh e1 n @ s i s ) ((pos "L@$" ) ))
( "parti-coloured"  j ( p aa1 t i - k uh l @ d ) ((pos "OA$" ) ))
( "partial"  j ( p aa1 sh l ) ((pos "OA%" ) ))
( "partialities"  n ( p aa2 sh i a1 l i t i z ) ((pos "Mj%" ) ))
( "partiality"  n ( p aa2 sh i a1 l i t ii ) ((pos "M8%" ) ))
( "partially"  a ( p aa1 sh @ l ii ) ((pos "Pu%" ) ))
( "participant"  n ( p aa1 t i1 s i p @ n t ) ((pos "K6%" ) ))
( "participants"  n ( p aa1 t i1 s i p @ n t s ) ((pos "Kj%" ) ))
( "participate"  v ( p aa1 t i1 s i p ei t ) ((pos "I2%" ) (subcat "2A" )))
( "participated"  v ( p aa1 t i1 s i p ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "participates"  v ( p aa1 t i1 s i p ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "participating"  v ( p aa1 t i1 s i p ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "participation"  n ( p aa1 t i2 s i p ei1 sh @ n ) ((pos "L@%" ) ))
( "participial"  j ( p aa2 t i s i1 p i@ l ) ((pos "OA$" ) ))
( "participle"  n ( p aa1 t i s i p l ) ((pos "K6%" ) ))
( "participles"  n ( p aa1 t i s i p l z ) ((pos "Kj%" ) ))
( "particle"  n ( p aa1 t i k l ) ((pos "K6%" ) ))
( "particles"  n ( p aa1 t i k l z ) ((pos "Kj%" ) ))
( "particular"  n ( p @1 t i1 k y u l @ r ) ((pos "K6%" "OA%" ) ))
( "particularities"  n ( p @1 t i2 k y u l a1 r i t i z ) ((pos "Mj%" ) ))
( "particularity"  n ( p @1 t i2 k y u l a1 r i t ii ) ((pos "M8%" ) ))
( "particularize"  v ( p @1 t i1 k y uu l @ r ai z ) ((pos "J2%" ) (subcat "2A" )))
( "particularized"  v ( p @1 t i1 k y uu l @ r ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "particularizes"  v ( p @1 t i1 k y uu l @ r ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "particularizing"  v ( p @1 t i1 k y uu l @ r ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "particularly"  a ( p @1 t i1 k y u l @ l ii ) ((pos "Pu%" ) ))
( "particulars"  n ( p @1 t i1 k y u l @ z ) ((pos "Kj%" ) ))
( "parties"  n ( p aa1 t i z ) ((pos "Mj%" ) ))
( "parting"  v ( p aa1 t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2D" "3A" )))
( "partings"  n ( p aa1 t i ng z ) ((pos "Mj%" ) ))
( "partisan"  n ( p aa2 t i z a1 n ) ((pos "K6%" "OA%" ) ))
( "partisans"  n ( p aa2 t i z a1 n z ) ((pos "Kj%" ) ))
( "partisanship"  n ( p aa2 t i z a1 n sh i p ) ((pos "L@%" ) ))
( "partition"  v ( p aa1 t i1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "partitioned"  v ( p aa1 t i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "partitioning"  v ( p aa1 t i1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "partitions"  v ( p aa1 t i1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "partitive"  n ( p aa1 t i t i v ) ((pos "K6$" "OA$" ) ))
( "partitives"  n ( p aa1 t i t i v z ) ((pos "Kj$" ) ))
( "partly"  a ( p aa1 t l ii ) ((pos "Pu%" ) ))
( "partner"  v ( p aa1 t n @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "partnered"  v ( p aa1 t n @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "partnering"  v ( p aa1 t n @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "partners"  v ( p aa1 t n @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "partnership"  n ( p aa1 t n @ sh i p ) ((pos "M6%" ) ))
( "partnerships"  n ( p aa1 t n @ sh i p s ) ((pos "Mj%" ) ))
( "partook"  v ( p aa1 t u1 k ) ((pos "Jc%" ) (subcat )))
( "partridge"  n ( p aa1 t r i jh ) ((pos "M6%" ) ))
( "partridges"  n ( p aa1 t r i jh i z ) ((pos "Mj%" ) ))
( "parts"  v ( p aa1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2D" "3A" )))
( "parturition"  n ( p aa2 t y u r i1 sh @ n ) ((pos "L@%" ) ))
( "party"  n ( p aa1 t ii ) ((pos "M8*" ) ))
( "party-spirit"  n ( p aa2 t i - s p i1 r i t ) ((pos "L@%" ) ))
( "party-spirited"  j ( p aa2 t i - s p i1 r i t i d ) ((pos "OA$" ) ))
( "party-wall"  n ( p aa2 t i - w oo1 l ) ((pos "K6%" ) ))
( "party-walls"  n ( p aa2 t i - w oo1 l z ) ((pos "Kj%" ) ))
( "parvenu"  n ( p aa1 v @ n y uu ) ((pos "K6$" ) ))
( "parvenus"  n ( p aa1 v @ n y uu z ) ((pos "Kj$" ) ))
( "paschal"  j ( p a1 s k l ) ((pos "OA$" ) ))
( "pasha"  n ( p aa1 sh @ ) ((pos "K6$" ) ))
( "pashas"  n ( p aa1 sh @ z ) ((pos "Kj$" ) ))
( "pass"  v ( p aa1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" "6A" "12A" "13A" "14" "15A" )))
( "pass_e"  j ( p a1 s ei ) ((pos "OA%" ) ))
( "pass_ee"  j ( p a1 s ei ) ((pos "OA$" ) ))
( "passable"  j ( p aa1 s @ b l ) ((pos "OA%" ) ))
( "passably"  a ( p aa1 s @ b l ii ) ((pos "Pu%" ) ))
( "passage"  n ( p a1 s i jh ) ((pos "M6%" ) ))
( "passages"  n ( p a1 s i jh i z ) ((pos "Mj%" ) ))
( "passageway"  n ( p a1 s i jh w ei ) ((pos "K6%" ) ))
( "passageways"  n ( p a1 s i jh w ei z ) ((pos "Kj%" ) ))
( "passbook"  n ( p aa1 s b u k ) ((pos "K6%" ) ))
( "passbooks"  n ( p aa1 s b u k s ) ((pos "Kj%" ) ))
( "passed"  v ( p aa1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "12A" "13A" "14" "15A" )))
( "passenger"  n ( p a1 s i n jh @ r ) ((pos "K6%" ) ))
( "passengers"  n ( p a1 s i n jh @ z ) ((pos "Kj%" ) ))
( "passepartout"  n ( p a2 s p aa t uu1 ) ((pos "M6$" ) ))
( "passepartouts"  n ( p a2 s p aa t uu1 z ) ((pos "Mj$" ) ))
( "passer-by"  n ( p aa2 s @ - b ai1 ) ((pos "Ki%" ) ))
( "passers-by"  n ( p aa2 s @ z - b ai1 ) ((pos "Kj%" ) ))
( "passes"  v ( p aa1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "12A" "13A" "14" "15A" )))
( "passim"  a ( p a1 s i m ) ((pos "Pu$" ) ))
( "passing"  v ( p aa1 s i ng ) ((pos "Jb%" "L@%" "OA%" "Pu%" ) (subcat "2A" "2C" "3A" "6A" "12A" "13A" "14" "15A" )))
( "passing-out"  j ( p aa2 s i ng - au1 t ) ((pos "OA%" ) ))
( "passion"  n ( p a1 sh @ n ) ((pos "M6%" ) ))
( "passion-flower"  n ( p a1 sh @ n - f l au @ r ) ((pos "K6%" ) ))
( "passion-flowers"  n ( p a1 sh @ n - f l au @ z ) ((pos "Kj%" ) ))
( "passionate"  j ( p a1 sh @ n @ t ) ((pos "OA%" ) ))
( "passionately"  a ( p a1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "passionless"  j ( p a1 sh @ n l @ s ) ((pos "OA%" ) ))
( "passions"  n ( p a1 sh @ n z ) ((pos "Mj%" ) ))
( "passive"  n ( p a1 s i v ) ((pos "K6%" "OA%" ) ))
( "passively"  a ( p a1 s i v l ii ) ((pos "Pu%" ) ))
( "passiveness"  n ( p a1 s i v n @ s ) ((pos "L@%" ) ))
( "passives"  n ( p a1 s i v z ) ((pos "Kj$" ) ))
( "passivity"  n ( p a1 s i1 v i t ii ) ((pos "L@%" ) ))
( "passkey"  n ( p aa1 s k ii ) ((pos "K6%" ) ))
( "passkeys"  n ( p aa1 s k ii z ) ((pos "Kj%" ) ))
( "passport"  n ( p aa1 s p oo t ) ((pos "K6%" ) ))
( "passports"  n ( p aa1 s p oo t s ) ((pos "Kj%" ) ))
( "password"  n ( p aa1 s w @@ d ) ((pos "K6%" ) ))
( "passwords"  n ( p aa1 s w @@ d z ) ((pos "Kj%" ) ))
( "past"  v ( p aa s t ) ((pos "Jd%" "K6*" "OA*" "Pu*" "T-*" ) (subcat "2A" "2C" "3A" "6A" "12A" "13A" "14" "15A" )))
( "pasta"  n ( p a1 s t @ ) ((pos "L@%" ) ))
( "paste"  v ( p ei1 s t ) ((pos "H2%" "M6%" ) (subcat "6A" "15A" )))
( "paste-up"  n ( p ei1 s t - uh p ) ((pos "K6$" ) ))
( "paste-ups"  n ( p ei1 s t - uh p s ) ((pos "Kj$" ) ))
( "pasteboard"  n ( p ei1 s t b oo d ) ((pos "L@%" ) ))
( "pasted"  v ( p ei1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "pastel"  n ( p a1 s t l ) ((pos "K6%" ) ))
( "pastels"  n ( p a1 s t l z ) ((pos "Kj%" ) ))
( "pastern"  n ( p a1 s t @ n ) ((pos "K6$" ) ))
( "pasterns"  n ( p a1 s t @ n z ) ((pos "Kj$" ) ))
( "pastes"  v ( p ei1 s t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "pasteurization"  n ( p aa2 s ch @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "pasteurize"  v ( p aa1 s ch @ r ai z ) ((pos "H2%" ) (subcat )))
( "pasteurized"  v ( p aa1 s ch @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pasteurizes"  v ( p aa1 s ch @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "pasteurizing"  v ( p aa1 s ch @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "pastiche"  n ( p a1 s t ii1 sh ) ((pos "K6%" ) ))
( "pastiches"  n ( p a1 s t ii1 sh i z ) ((pos "Kj$" ) ))
( "pastier"  j ( p ei1 s t i@ r ) ((pos "Or%" ) ))
( "pasties"  n ( p a1 s t i z ) ((pos "Kj%" ) ))
( "pastiest"  j ( p ei1 s t i i s t ) ((pos "Os%" ) ))
( "pastille"  n ( p a1 s t i l ) ((pos "K6%" ) ))
( "pastilles"  n ( p a1 s t i l z ) ((pos "Kj%" ) ))
( "pastime"  n ( p aa1 s t ai m ) ((pos "K6%" ) ))
( "pastimes"  n ( p aa1 s t ai m z ) ((pos "Kj%" ) ))
( "pasting"  v ( p ei1 s t i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" "15A" )))
( "pastings"  n ( p ei1 s t i ng z ) ((pos "Kj%" ) ))
( "pastor"  n ( p aa1 s t @ r ) ((pos "K6%" ) ))
( "pastoral"  n ( p aa1 s t @ r @ l ) ((pos "K6%" "OA%" ) ))
( "pastorals"  n ( p aa1 s t @ r @ l z ) ((pos "Kj%" ) ))
( "pastorate"  n ( p aa1 s t @ r @ t ) ((pos "K6$" ) ))
( "pastorates"  n ( p aa1 s t @ r @ t s ) ((pos "Kj$" ) ))
( "pastors"  n ( p aa1 s t @ z ) ((pos "Kj%" ) ))
( "pastries"  n ( p ei1 s t r i z ) ((pos "Mj%" ) ))
( "pastry"  n ( p ei1 s t r ii ) ((pos "M8%" ) ))
( "pastry-cook"  n ( p ei1 s t r i - k u k ) ((pos "K6%" ) ))
( "pastry-cooks"  n ( p ei1 s t r i - k u k s ) ((pos "Kj%" ) ))
( "pasts"  n ( p aa1 s t s ) ((pos "Kj%" ) ))
( "pasturage"  n ( p aa1 s ch @ r i jh ) ((pos "L@%" ) ))
( "pasture"  v ( p aa1 s ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "pastured"  v ( p aa1 s ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pastures"  v ( p aa1 s ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "pasturing"  v ( p aa1 s ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pasty"  n ( p a1 s t ii ) ((pos "K8%" ) ))
( "pasty"  j ( p ei1 s t ii ) ((pos "OD%" ) ))
( "pat"  v ( p a1 t ) ((pos "J4%" "K6%" "Pu%" ) (subcat "2A" "6A" )))
( "patch"  v ( p a1 ch ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "patch-pocket"  n ( p a1 ch - p o k i t ) ((pos "K6$" ) ))
( "patch-pockets"  n ( p a1 ch - p o k i t s ) ((pos "Kj$" ) ))
( "patched"  v ( p a1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "patches"  v ( p a1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "patchier"  j ( p a1 ch i@ r ) ((pos "Or%" ) ))
( "patchiest"  j ( p a1 ch i i s t ) ((pos "Os%" ) ))
( "patchily"  a ( p a1 ch i l ii ) ((pos "Pu%" ) ))
( "patchiness"  n ( p a1 ch i n @ s ) ((pos "L@%" ) ))
( "patching"  v ( p a1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "patchouli"  n ( p a1 ch u l ii ) ((pos "L@$" ) ))
( "patchwork"  n ( p a1 ch w @@ k ) ((pos "L@%" ) ))
( "patchy"  j ( p a1 ch ii ) ((pos "OD%" ) ))
( "pate"  n ( p ei1 t ) ((pos "K6%" ) ))
( "pate de foie gras"  n ( p a2 t ei - d @ - f w aa2 - g r aa1 ) ((pos "L@%" ) ))
( "patella"  n ( p @1 t e1 l @ ) ((pos "K6%" ) ))
( "patellas"  n ( p @1 t e1 l @ z ) ((pos "Kj$" ) ))
( "patent"  v ( p ei1 t n t ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "patented"  v ( p ei1 t n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "patentee"  n ( p ei2 t n t ii1 ) ((pos "K6$" ) ))
( "patentees"  n ( p ei2 t n t ii1 z ) ((pos "Kj$" ) ))
( "patenting"  v ( p ei1 t n t i ng ) ((pos "Hb%" ) (subcat )))
( "patently"  a ( p ei1 t n t l ii ) ((pos "Pu%" ) ))
( "patents"  v ( p ei1 t n t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "paterfamilias"  n ( p ei2 t @ f @ m i1 l i a s ) ((pos "Ki%" ) ))
( "paternal"  j ( p @1 t @@1 n l ) ((pos "OA%" ) ))
( "paternalism"  n ( p @1 t @@1 n @ l i z @ m ) ((pos "L@%" ) ))
( "paternally"  a ( p @1 t @@1 n @ l ii ) ((pos "Pu%" ) ))
( "paternity"  n ( p @1 t @@1 n i t ii ) ((pos "L@%" ) ))
( "paternoster"  n ( p a2 t @ n o1 s t @ r ) ((pos "K6$" ) ))
( "paternosters"  n ( p a2 t @ n o1 s t @ z ) ((pos "Kj$" ) ))
( "pates"  n ( p ei1 t s ) ((pos "Kj%" ) ))
( "path"  n ( p aa1 th ) ((pos "K6%" ) ))
( "path-finder"  n ( p aa1 th - f ai n d @ r ) ((pos "K6%" ) ))
( "path-finders"  n ( p aa1 th - f ai n d @ z ) ((pos "Kj%" ) ))
( "pathetic"  j ( p @1 th e1 t i k ) ((pos "OA%" ) ))
( "pathetically"  a ( p @1 th e1 t i k l ii ) ((pos "Pu%" ) ))
( "pathless"  j ( p aa1 th l @ s ) ((pos "OA%" ) ))
( "pathogen"  n ( p a1 th @ jh @ n ) ((pos "K6%" ) ))
( "pathogens"  n ( p a1 th @ jh @ n z ) ((pos "Kj%" ) ))
( "pathological"  j ( p a2 th @ l o1 jh i k l ) ((pos "OA%" ) ))
( "pathologically"  a ( p a2 th @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "pathologist"  n ( p @1 th o1 l @ jh i s t ) ((pos "K6%" ) ))
( "pathologists"  n ( p @1 th o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "pathology"  n ( p @1 th o1 l @ jh ii ) ((pos "L@%" ) ))
( "pathos"  n ( p ei1 th o s ) ((pos "L@%" ) ))
( "paths"  n ( p aa1 dh z ) ((pos "Kj%" ) ))
( "pathway"  n ( p aa1 th w ei ) ((pos "K6%" ) ))
( "pathways"  n ( p aa1 th w ei z ) ((pos "Kj%" ) ))
( "patience"  n ( p ei1 sh @ n s ) ((pos "L@%" ) ))
( "patient"  n ( p ei1 sh @ n t ) ((pos "K6%" "OA%" ) ))
( "patiently"  a ( p ei1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "patients"  n ( p ei1 sh @ n t s ) ((pos "Kj%" ) ))
( "patina"  n ( p a1 t i n @ ) ((pos "K6$" ) ))
( "patinas"  n ( p a1 t i n @ z ) ((pos "Kj$" ) ))
( "patio"  n ( p a1 t i@ @ ) ((pos "K6%" ) ))
( "patios"  n ( p a1 t i@ u z ) ((pos "Kj%" ) ))
( "patisserie"  n ( p @1 t ii1 s @ r ii ) ((pos "K6%" ) ))
( "patisseries"  n ( p @1 t ii1 s @ r i z ) ((pos "Kj%" ) ))
( "patois"  n ( p a1 t w aa ) ((pos "Ki$" ) ))
( "patois"  n ( p a1 t w aa z ) ((pos "Kj$" ) ))
( "patrial"  n ( p ei1 t r i@ l ) ((pos "K6$" ) ))
( "patrials"  n ( p ei1 t r i@ l z ) ((pos "Kj$" ) ))
( "patriarch"  n ( p ei1 t r i aa k ) ((pos "K6%" ) ))
( "patriarchal"  j ( p ei2 t r i aa1 k l ) ((pos "OA%" ) ))
( "patriarchate"  n ( p ei1 t r i aa k ei t ) ((pos "K6$" ) ))
( "patriarchates"  n ( p ei1 t r i aa k ei t s ) ((pos "Kj$" ) ))
( "patriarchies"  n ( p ei1 t r i aa k i z ) ((pos "Kj$" ) ))
( "patriarchs"  n ( p ei1 t r i aa k s ) ((pos "Kj%" ) ))
( "patriarchy"  n ( p ei1 t r i aa k ii ) ((pos "K8%" ) ))
( "patrician"  n ( p @1 t r i1 sh @ n ) ((pos "K6%" "OA%" ) ))
( "patricians"  n ( p @1 t r i1 sh @ n z ) ((pos "Kj%" ) ))
( "patricide"  n ( p a1 t r i s ai d ) ((pos "M6%" ) ))
( "patricides"  n ( p a1 t r i s ai d z ) ((pos "Mj%" ) ))
( "patrimonial"  j ( p a2 t r i m ou1 n i@ l ) ((pos "OA$" ) ))
( "patrimonies"  n ( p a1 t r i m @ n i z ) ((pos "Kj$" ) ))
( "patrimony"  n ( p a1 t r i m @ n ii ) ((pos "K8$" ) ))
( "patriot"  n ( p a1 t r i@ t ) ((pos "K6%" ) ))
( "patriotic"  j ( p a2 t r i o1 t i k ) ((pos "OA%" ) ))
( "patriotically"  a ( p a2 t r i o1 t i k l ii ) ((pos "Pu%" ) ))
( "patriotism"  n ( p a1 t r i@ t i z @ m ) ((pos "L@%" ) ))
( "patriots"  n ( p a1 t r i@ t s ) ((pos "Kj%" ) ))
( "patrol"  v ( p @1 t r ou1 l ) ((pos "J4%" "M6%" ) (subcat "2A" )))
( "patrolled"  v ( p @1 t r ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "patrolling"  v ( p @1 t r ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "patrolman"  n ( p @1 t r ou1 l m @ n ) ((pos "Ki%" ) ))
( "patrolmen"  n ( p @1 t r ou1 l m @ n ) ((pos "Kj%" ) ))
( "patrols"  v ( p @1 t r ou1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "patron"  n ( p ei1 t r @ n ) ((pos "K6%" ) ))
( "patronage"  n ( p a1 t r @ n i jh ) ((pos "L@%" ) ))
( "patroness"  n ( p ei1 t r @ n e s ) ((pos "K7%" ) ))
( "patronesses"  n ( p ei1 t r @ n e s i z ) ((pos "Kj%" ) ))
( "patronize"  v ( p a1 t r @ n ai z ) ((pos "H2%" ) (subcat )))
( "patronized"  v ( p a1 t r @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "patronizes"  v ( p a1 t r @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "patronizing"  v ( p a1 t r @ n ai z i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "patronizingly"  a ( p a1 t r @ n ai z i ng l ii ) ((pos "Pu%" ) ))
( "patrons"  n ( p ei1 t r @ n z ) ((pos "Kj%" ) ))
( "patronymic"  n ( p a2 t r @ n i1 m i k ) ((pos "K6%" "OA%" ) ))
( "patronymics"  n ( p a2 t r @ n i1 m i k s ) ((pos "Kj%" ) ))
( "pats"  v ( p a1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "patted"  v ( p a1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "patten"  n ( p a1 t n ) ((pos "K6$" ) ))
( "pattens"  n ( p a1 t n z ) ((pos "Kj$" ) ))
( "patter"  v ( p a1 t @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" )))
( "pattered"  v ( p a1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "pattering"  v ( p a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "pattern"  v ( p a1 t n ) ((pos "H0%" "K6%" ) (subcat )))
( "patterned"  v ( p a1 t n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "patterning"  v ( p a1 t n i ng ) ((pos "Hb%" ) (subcat )))
( "patterns"  v ( p a1 t n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "patters"  v ( p a1 t @ z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "patties"  n ( p a1 t i z ) ((pos "Kj%" ) ))
( "patting"  v ( p a1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "patty"  n ( p a1 t ii ) ((pos "K8%" ) ))
( "patty-pan"  n ( p a1 t i - p a n ) ((pos "K6$" ) ))
( "patty-pans"  n ( p a1 t i - p a n z ) ((pos "Kj$" ) ))
( "paucity"  n ( p oo1 s i t ii ) ((pos "L@%" ) ))
( "paunch"  n ( p oo1 n ch ) ((pos "K7%" ) ))
( "paunches"  n ( p oo1 n ch i z ) ((pos "Kj%" ) ))
( "paunchier"  j ( p oo1 n ch i@ r ) ((pos "Or%" ) ))
( "paunchiest"  j ( p oo1 n ch i i s t ) ((pos "Os%" ) ))
( "paunchiness"  n ( p oo1 n ch i n @ s ) ((pos "L@%" ) ))
( "paunchy"  j ( p oo1 n ch ii ) ((pos "OD%" ) ))
( "pauper"  n ( p oo1 p @ r ) ((pos "K6%" ) ))
( "pauperism"  n ( p oo1 p @ r i z @ m ) ((pos "L@$" ) ))
( "pauperization"  n ( p oo2 p @ r ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "pauperize"  v ( p oo1 p @ r ai z ) ((pos "H2%" ) (subcat )))
( "pauperized"  v ( p oo1 p @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pauperizes"  v ( p oo1 p @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "pauperizing"  v ( p oo1 p @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "paupers"  n ( p oo1 p @ z ) ((pos "Kj%" ) ))
( "pause"  v ( p oo1 z ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "paused"  v ( p oo1 z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "pauses"  v ( p oo1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "pausing"  v ( p oo1 z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "pave"  v ( p ei1 v ) ((pos "H2%" ) (subcat )))
( "paved"  v ( p ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pavement"  n ( p ei1 v m @ n t ) ((pos "K6%" ) ))
( "pavements"  n ( p ei1 v m @ n t s ) ((pos "Kj%" ) ))
( "paves"  v ( p ei1 v z ) ((pos "Ha%" ) (subcat )))
( "pavilion"  n ( p @1 v i1 l i@ n ) ((pos "K6%" ) ))
( "pavilions"  n ( p @1 v i1 l i@ n z ) ((pos "Kj%" ) ))
( "paving"  v ( p ei1 v i ng ) ((pos "Hb%" ) (subcat )))
( "paving-stone"  n ( p ei1 v i ng - s t ou n ) ((pos "K6%" ) ))
( "paving-stones"  n ( p ei1 v i ng - s t ou n z ) ((pos "Kj%" ) ))
( "paw"  v ( p oo1 ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "pawed"  v ( p oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "pawing"  v ( p oo1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pawkily"  a ( p oo1 k i l ii ) ((pos "Pu$" ) ))
( "pawky"  j ( p oo1 k ii ) ((pos "OA$" ) ))
( "pawl"  n ( p oo1 l ) ((pos "K6$" ) ))
( "pawls"  n ( p oo1 l z ) ((pos "Kj$" ) ))
( "pawn"  v ( p oo1 n ) ((pos "H0%" "M6%" ) (subcat )))
( "pawn-ticket"  n ( p oo1 n - t i k i t ) ((pos "K6%" ) ))
( "pawn-tickets"  n ( p oo1 n - t i k i t s ) ((pos "Kj%" ) ))
( "pawnbroker"  n ( p oo1 n b r ou k @ r ) ((pos "K6%" ) ))
( "pawnbrokers"  n ( p oo1 n b r ou k @ z ) ((pos "Kj%" ) ))
( "pawned"  v ( p oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pawning"  v ( p oo1 n i ng ) ((pos "Hb%" ) (subcat )))
( "pawns"  v ( p oo1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "pawnshop"  n ( p oo1 n sh o p ) ((pos "K6%" ) ))
( "pawnshops"  n ( p oo1 n sh o p s ) ((pos "Kj%" ) ))
( "pawpaw"  n ( p oo1 p oo ) ((pos "K6%" ) ))
( "pawpaws"  n ( p oo1 p oo1 z ) ((pos "Kj%" ) ))
( "paws"  v ( p oo1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "pax"  n ( p a1 k s ) ((pos "K7$" ) ))
( "paxes"  n ( p a1 k s i z ) ((pos "Kj$" ) ))
( "pay"  v ( p ei1 ) ((pos "J5%" "L@%" ) (subcat "2A" "3A" "6A" "12A" "12B" "13A" "13B" "14" )))
( "pay-as-you-earn"  n ( p ei2 - @ z - y uu - @@1 n ) ((pos "L@%" ) ))
( "pay-claim"  n ( p ei1 - k l ei m ) ((pos "K6%" ) ))
( "pay-claims"  n ( p ei1 - k l ei m z ) ((pos "Kj%" ) ))
( "pay-packet"  n ( p ei1 - p a k i t ) ((pos "K6%" ) ))
( "pay-packets"  n ( p ei1 - p a k i t s ) ((pos "Kj%" ) ))
( "pay-station"  n ( p ei1 - s t ei sh @ n ) ((pos "K6$" ) ))
( "pay-stations"  n ( p ei1 - s t ei sh @ n z ) ((pos "Kj$" ) ))
( "payable"  j ( p ei1 @ b l ) ((pos "OA%" ) ))
( "payday"  n ( p ei1 d ei ) ((pos "K6%" ) ))
( "paydays"  n ( p ei1 d ei z ) ((pos "Kj%" ) ))
( "paydirt"  n ( p ei1 d @@ t ) ((pos "L@%" ) ))
( "payee"  n ( p ei1 ii1 ) ((pos "K6%" ) ))
( "payees"  n ( p ei1 ii1 z ) ((pos "Kj%" ) ))
( "payer"  n ( p ei1 @ r ) ((pos "K6%" ) ))
( "payers"  n ( p ei1 @ z ) ((pos "Kj%" ) ))
( "paying"  v ( p ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "12A" "12B" "13A" "13B" "14" )))
( "payload"  n ( p ei1 l ou d ) ((pos "K6%" ) ))
( "payloads"  n ( p ei1 l ou d z ) ((pos "Kj%" ) ))
( "paymaster"  n ( p ei1 m aa s t @ r ) ((pos "K6%" ) ))
( "paymasters"  n ( p ei1 m aa s t @ z ) ((pos "Kj%" ) ))
( "payment"  n ( p ei1 m @ n t ) ((pos "M6%" ) ))
( "payments"  n ( p ei1 m @ n t s ) ((pos "Mj%" ) ))
( "paynim"  n ( p ei1 n i m ) ((pos "K6$" ) ))
( "paynims"  n ( p ei1 n i m z ) ((pos "Kj$" ) ))
( "payoff"  n ( p ei1 o f ) ((pos "K6%" ) ))
( "payoffs"  n ( p ei1 o f s ) ((pos "Kj%" ) ))
( "payphone"  n ( p ei1 f ou n ) ((pos "K6%" ) ))
( "payphones"  n ( p ei1 f ou n z ) ((pos "Kj%" ) ))
( "payroll"  n ( p ei1 r ou l ) ((pos "K6%" ) ))
( "payrolls"  n ( p ei1 r ou l z ) ((pos "Kj%" ) ))
( "pays"  v ( p ei1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "12A" "12B" "13A" "13B" "14" )))
( "paysheet"  n ( p ei1 sh ii t ) ((pos "K6%" ) ))
( "paysheets"  n ( p ei1 sh ii t s ) ((pos "Kj%" ) ))
( "payslip"  n ( p ei1 s l i p ) ((pos "K6%" ) ))
( "payslips"  n ( p ei1 s l i p s ) ((pos "Kj%" ) ))
( "pea"  n ( p ii1 ) ((pos "K6%" ) ))
( "pea-chick"  n ( p ii1 - ch i k ) ((pos "K6$" ) ))
( "pea-chicks"  n ( p ii1 - ch i k s ) ((pos "Kj$" ) ))
( "pea-flour"  n ( p ii1 - f l au @ r ) ((pos "L@$" ) ))
( "pea-green"  n ( p ii1 - g r ii n ) ((pos "M6%" "OA%" ) ))
( "pea-greens"  n ( p ii1 - g r ii n z ) ((pos "Mj$" ) ))
( "pea-jacket"  n ( p ii1 - jh a k i t ) ((pos "K6$" ) ))
( "pea-jackets"  n ( p ii1 - jh a k i t s ) ((pos "Kj$" ) ))
( "pea-soup"  n ( p ii1 - s uu1 p ) ((pos "M6%" ) ))
( "pea-soups"  n ( p ii1 - s uu1 p s ) ((pos "Mj$" ) ))
( "peace"  n ( p ii1 s ) ((pos "L@%" ) ))
( "peace-offering"  n ( p ii1 s - o f @ r i ng ) ((pos "K6%" ) ))
( "peace-offerings"  n ( p ii1 s - o f @ r i ng z ) ((pos "Kj%" ) ))
( "peaceable"  j ( p ii1 s @ b l ) ((pos "OA%" ) ))
( "peaceably"  a ( p ii1 s @ b l ii ) ((pos "Pu%" ) ))
( "peaceful"  j ( p ii1 s f @ l ) ((pos "OA%" ) ))
( "peacefully"  a ( p ii1 s f @ l ii ) ((pos "Pu%" ) ))
( "peacefulness"  n ( p ii1 s f l n @ s ) ((pos "L@%" ) ))
( "peacemaker"  n ( p ii1 s m ei k @ r ) ((pos "K6%" ) ))
( "peacemakers"  n ( p ii1 s m ei k @ z ) ((pos "Kj%" ) ))
( "peacetime"  n ( p ii1 s t ai m ) ((pos "L@%" ) ))
( "peach"  v ( p ii1 ch ) ((pos "J1$" "K7%" ) (subcat "2A" "3A" )))
( "peached"  v ( p ii1 ch t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "3A" )))
( "peaches"  v ( p ii1 ch i z ) ((pos "Ja$" "Kj%" ) (subcat "2A" "3A" )))
( "peaching"  v ( p ii1 ch i ng ) ((pos "Jb$" ) (subcat "2A" "3A" )))
( "peacock"  n ( p ii1 k o k ) ((pos "K6%" ) ))
( "peacock-blue"  n ( p ii2 k o k - b l uu1 ) ((pos "M6%" "OA%" ) ))
( "peacock-blues"  n ( p ii2 k o k - b l uu1 z ) ((pos "Mj$" ) ))
( "peacocks"  n ( p ii1 k o k s ) ((pos "Kj%" ) ))
( "peafowl"  n ( p ii1 f au l ) ((pos "K6$" ) ))
( "peafowls"  n ( p ii1 f au l z ) ((pos "Kj$" ) ))
( "peahen"  n ( p ii1 h e n ) ((pos "K6%" ) ))
( "peahens"  n ( p ii1 h e n z ) ((pos "Kj%" ) ))
( "peak"  v ( p ii1 k ) ((pos "I0%" "K6%" ) (subcat )))
( "peaked"  v ( p ii1 k t ) ((pos "Ic%" "Id%" "OA%" ) (subcat )))
( "peakier"  j ( p ii1 k i@ r ) ((pos "Or$" ) ))
( "peakiest"  j ( p ii1 k i i s t ) ((pos "Os$" ) ))
( "peaking"  v ( p ii1 k i ng ) ((pos "Ib%" ) (subcat )))
( "peaks"  v ( p ii1 k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "peaky"  j ( p ii1 k ii ) ((pos "OD%" ) ))
( "peal"  v ( p ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "pealed"  v ( p ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "pealing"  v ( p ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "peals"  v ( p ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "peanut"  n ( p ii1 n uh t ) ((pos "K6%" ) ))
( "peanuts"  n ( p ii1 n uh t s ) ((pos "Kj%" ) ))
( "pear"  n ( p e@1 r ) ((pos "M6%" ) ))
( "pearl"  v ( p @@1 l ) ((pos "I0%" "K6%" ) ))
( "pearl-barley"  n ( p @@1 l - b aa1 l ii ) ((pos "L@%" ) ))
( "pearl-diver"  n ( p @@1 l - d ai v @ r ) ((pos "K6%" ) ))
( "pearl-divers"  n ( p @@1 l - d ai v @ z ) ((pos "Kj%" ) ))
( "pearl-fisheries"  n ( p @@1 l - f i sh @ r i z ) ((pos "Kj%" ) ))
( "pearl-fishery"  n ( p @@1 l - f i sh @ r ii ) ((pos "K8%" ) ))
( "pearl-oyster"  n ( p @@1 l - oi s t @ r ) ((pos "K6%" ) ))
( "pearl-oysters"  n ( p @@1 l - oi s t @ z ) ((pos "Kj%" ) ))
( "pearl-sago"  n ( p @@1 l - s ei1 g ou ) ((pos "L@%" ) ))
( "pearled"  v ( p @@1 l d ) ((pos "Ic$" "Id$" ) ))
( "pearlier"  j ( p @@1 l i@ r ) ((pos "Or$" ) ))
( "pearlies"  n ( p @@1 l ii z ) ((pos "Kj$" ) ))
( "pearliest"  j ( p @@1 l i i s t ) ((pos "Os$" ) ))
( "pearling"  v ( p @@1 l i ng ) ((pos "Ib$" ) ))
( "pearls"  v ( p @@1 l z ) ((pos "Ia$" "Kj%" ) ))
( "pearly"  j ( p @@1 l ii ) ((pos "OD%" ) ))
( "pearmain"  n ( p e@1 m ei n ) ((pos "K6$" ) ))
( "pearmains"  n ( p e@1 m ei n z ) ((pos "Kj$" ) ))
( "pears"  n ( p e@1 z ) ((pos "Mj%" ) ))
( "peas"  n ( p ii1 z ) ((pos "Kj%" ) ))
( "peasant"  n ( p e1 s n t ) ((pos "K6%" ) ))
( "peasantries"  n ( p e1 z n t r i z ) ((pos "Kj$" ) ))
( "peasantry"  n ( p e1 z n t r ii ) ((pos "K8%" ) ))
( "peasants"  n ( p e1 z n t s ) ((pos "Kj%" ) ))
( "pease"  n ( p ii1 z ) ((pos "L@$" ) ))
( "pease-pudding"  n ( p ii1 z - p u1 d i ng ) ((pos "M6%" ) ))
( "pease-puddings"  n ( p ii1 z - p u1 d i ng z ) ((pos "Mj%" ) ))
( "peashooter"  n ( p ii1 sh uu t @ r ) ((pos "K6%" ) ))
( "peashooters"  n ( p ii1 sh uu t @ z ) ((pos "Kj%" ) ))
( "peasouper"  n ( p ii1 s uu1 p @ r ) ((pos "K6%" ) ))
( "peasoupers"  n ( p ii1 s uu1 p @ z ) ((pos "Kj$" ) ))
( "peat"  n ( p ii1 t ) ((pos "L@%" ) ))
( "peatier"  j ( p ii1 t i@ r ) ((pos "Or$" ) ))
( "peatiest"  j ( p ii1 t i i s t ) ((pos "Os$" ) ))
( "peaty"  j ( p ii1 t ii ) ((pos "OD%" ) ))
( "pebble"  n ( p e1 b l ) ((pos "K6%" ) ))
( "pebbles"  n ( p e1 b l z ) ((pos "Kj%" ) ))
( "pebblier"  j ( p e1 b l i@ r ) ((pos "Or$" ) ))
( "pebbliest"  j ( p e1 b l i i s t ) ((pos "Os$" ) ))
( "pebbly"  j ( p e1 b l ii ) ((pos "OD%" ) ))
( "pecan"  n ( p i1 k a1 n ) ((pos "K6%" ) ))
( "pecans"  n ( p i1 k a1 n z ) ((pos "Kj$" ) ))
( "peccable"  j ( p e1 k @ b l ) ((pos "OA$" ) ))
( "peccadillo"  n ( p e2 k @ d i1 l ou ) ((pos "K6%" ) ))
( "peccadilloes"  n ( p e2 k @ d i1 l ou z ) ((pos "Kj%" ) ))
( "peccadillos"  n ( p e2 k @ d i1 l ou z ) ((pos "Kj%" ) ))
( "peccaries"  n ( p e1 k @ r i z ) ((pos "Kj$" ) ))
( "peccary"  n ( p e1 k @ r ii ) ((pos "K8$" ) ))
( "peck"  v ( p e1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "pecked"  v ( p e1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "pecker"  n ( p e1 k @ r ) ((pos "K6%" ) ))
( "peckers"  n ( p e1 k @ z ) ((pos "Kj%" ) ))
( "pecking"  v ( p e1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "peckish"  j ( p e1 k i sh ) ((pos "OA%" ) ))
( "pecks"  v ( p e1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "pectic"  j ( p e1 k t i k ) ((pos "OA$" ) ))
( "pectin"  n ( p e1 k t i n ) ((pos "L@%" ) ))
( "pectoral"  j ( p e1 k t @ r @ l ) ((pos "OA%" ) ))
( "peculate"  v ( p e1 k y u l ei t ) ((pos "J2$" ) ))
( "peculated"  v ( p e1 k y u l ei t i d ) ((pos "Jc$" "Jd$" ) ))
( "peculates"  v ( p e1 k y u l ei t s ) ((pos "Ja$" ) ))
( "peculating"  v ( p e1 k y u l ei t i ng ) ((pos "Jb$" ) ))
( "peculation"  n ( p e2 k y u l ei1 sh @ n ) ((pos "M6$" ) ))
( "peculations"  n ( p e2 k y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "peculiar"  j ( p i1 k y uu1 l i@ r ) ((pos "OA%" ) ))
( "peculiarities"  n ( p i1 k y uu2 l i a1 r i t i z ) ((pos "Mj%" ) ))
( "peculiarity"  n ( p i1 k y uu2 l i a1 r i t ii ) ((pos "M8%" ) ))
( "peculiarly"  a ( p i1 k y uu1 l i@ l ii ) ((pos "Pu%" ) ))
( "pecuniary"  j ( p i1 k y uu1 n i@ r ii ) ((pos "OA%" ) ))
( "pedagogic"  j ( p e2 d @ g o1 jh i k ) ((pos "OA%" ) ))
( "pedagogical"  j ( p e2 d @ g o1 jh i k @ l ) ((pos "OA%" ) ))
( "pedagogue"  n ( p e1 d @ g o g ) ((pos "K6%" ) ))
( "pedagogues"  n ( p e1 d @ g o g z ) ((pos "Kj%" ) ))
( "pedagogy"  n ( p e1 d @ g o jh ii ) ((pos "L@%" ) ))
( "pedal"  v ( p e1 d l ) ((pos "J4%" "K6%" "OA%" ) (subcat "2A" "2C" )))
( "pedalled"  v ( p e1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "pedalling"  v ( p e1 d @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "pedals"  v ( p e1 d l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "pedant"  n ( p e1 d n t ) ((pos "K6%" ) ))
( "pedantic"  j ( p i1 d a1 n t i k ) ((pos "OA%" ) ))
( "pedantically"  a ( p i1 d a1 n t i k l ii ) ((pos "Pu%" ) ))
( "pedantries"  n ( p e1 d n t r i z ) ((pos "Mj%" ) ))
( "pedantry"  n ( p e1 d n t r ii ) ((pos "M8%" ) ))
( "pedants"  n ( p e1 d n t s ) ((pos "Kj%" ) ))
( "peddle"  v ( p e1 d l ) ((pos "J2%" ) (subcat "2A" )))
( "peddled"  v ( p e1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "peddler"  n ( p e1 d l @ r ) ((pos "K6%" ) ))
( "peddlers"  n ( p e1 d l @ z ) ((pos "Kj%" ) ))
( "peddles"  v ( p e1 d l z ) ((pos "Ja%" ) (subcat "2A" )))
( "peddling"  v ( p e1 d l i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "pederast"  n ( p e1 d @ r a s t ) ((pos "K6$" ) ))
( "pederasts"  n ( p e1 d @ r a s t s ) ((pos "Kj$" ) ))
( "pederasty"  n ( p e1 d @ r a s t ii ) ((pos "L@$" ) ))
( "pedestal"  n ( p e1 d i s t l ) ((pos "K6%" ) ))
( "pedestals"  n ( p e1 d i s t l z ) ((pos "Kj%" ) ))
( "pedestrian"  n ( p i1 d e1 s t r i@ n ) ((pos "K6%" "OA%" ) ))
( "pedestrians"  n ( p i1 d e1 s t r i@ n z ) ((pos "Kj%" ) ))
( "pediatrician"  n ( p ii2 d i@ t r i1 sh @ n ) ((pos "K6%" ) ))
( "pediatricians"  n ( p ii2 d i@ t r i1 sh @ n z ) ((pos "Kj%" ) ))
( "pediatrics"  n ( p ii2 d i a1 t r i k s ) ((pos "Lk%" ) ))
( "pedicab"  n ( p e1 d i k a b ) ((pos "K6$" ) ))
( "pedicabs"  n ( p e1 d i k a b z ) ((pos "Kj$" ) ))
( "pedicure"  n ( p e1 d i k y u@ r ) ((pos "K6%" ) ))
( "pedicures"  n ( p e1 d i k y u@ z ) ((pos "Kj$" ) ))
( "pedigree"  n ( p e1 d i g r ii ) ((pos "M6%" ) ))
( "pedigrees"  n ( p e1 d i g r ii z ) ((pos "Mj%" ) ))
( "pediment"  n ( p e1 d i m @ n t ) ((pos "K6$" ) ))
( "pediments"  n ( p e1 d i m @ n t s ) ((pos "Kj$" ) ))
( "pedlar"  n ( p e1 d l @ r ) ((pos "K6%" ) ))
( "pedlars"  n ( p e1 d l @ z ) ((pos "Kj%" ) ))
( "pedometer"  n ( p i1 d o1 m i t @ r ) ((pos "K6$" ) ))
( "pedometers"  n ( p i1 d o1 m i t @ z ) ((pos "Kj$" ) ))
( "pee"  v ( p ii1 ) ((pos "I5%" "M6%" ) (subcat )))
( "peed"  v ( p ii1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "peeing"  v ( p ii1 i ng ) ((pos "Ib%" ) (subcat )))
( "peek"  v ( p ii1 k ) ((pos "I0%" "K6%" ) ))
( "peek-a-boo"  n ( p ii2 k - @ - b uu1 ) ((pos "L@%" ) ))
( "peeked"  v ( p ii1 k t ) ((pos "Ic%" "Id%" ) ))
( "peeking"  v ( p ii1 k i ng ) ((pos "Ib%" ) ))
( "peeks"  v ( p ii1 k s ) ((pos "Ia%" "Kj%" ) ))
( "peel"  v ( p ii1 l ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "peeled"  v ( p ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "peeler"  n ( p ii1 l @ r ) ((pos "K6%" ) ))
( "peelers"  n ( p ii1 l @ z ) ((pos "Kj%" ) ))
( "peeling"  v ( p ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "peelings"  n ( p ii1 l i ng z ) ((pos "Kj%" ) ))
( "peels"  v ( p ii1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "peep"  v ( p ii1 p ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "peeped"  v ( p ii1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "peeper"  n ( p ii1 p @ r ) ((pos "K6$" ) ))
( "peepers"  n ( p ii1 p @ z ) ((pos "Kj$" ) ))
( "peephole"  n ( p ii1 p h ou l ) ((pos "K6%" ) ))
( "peepholes"  n ( p ii1 p h ou l z ) ((pos "Kj%" ) ))
( "peeping"  v ( p ii1 p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "peeps"  v ( p ii1 p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "peepshow"  n ( p ii1 p sh ou ) ((pos "K6%" ) ))
( "peepshows"  n ( p ii1 p sh ou z ) ((pos "Kj%" ) ))
( "peepul"  n ( p ii1 p @ l ) ((pos "K6$" ) ))
( "peepuls"  n ( p ii1 p @ l z ) ((pos "Kj$" ) ))
( "peer"  v ( p i@1 r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "peerage"  n ( p i@1 r i jh ) ((pos "K6%" ) ))
( "peerages"  n ( p i@1 r i jh i z ) ((pos "Kj%" ) ))
( "peered"  v ( p i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "peeress"  n ( p i@1 r e s ) ((pos "K7%" ) ))
( "peeresses"  n ( p i@1 r e s i z ) ((pos "Kj$" ) ))
( "peering"  v ( p i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "peerless"  j ( p i@1 l @ s ) ((pos "OA%" ) ))
( "peers"  v ( p i@1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "pees"  v ( p ii1 z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "peeve"  v ( p ii1 v ) ((pos "H2$" ) ))
( "peeved"  v ( p ii1 v d ) ((pos "Hc$" "Hd%" "OA%" ) ))
( "peeves"  v ( p ii1 v z ) ((pos "Ha$" ) ))
( "peeving"  v ( p ii1 v i ng ) ((pos "Hb$" ) ))
( "peevish"  j ( p ii1 v i sh ) ((pos "OA%" ) ))
( "peevishly"  a ( p ii1 v i sh l ii ) ((pos "Pu%" ) ))
( "peevishness"  n ( p ii1 v i sh n @ s ) ((pos "L@%" ) ))
( "peewit"  n ( p ii1 w i t ) ((pos "K6%" ) ))
( "peewits"  n ( p ii1 w i t s ) ((pos "Kj%" ) ))
( "peg"  v ( p e1 g ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" )))
( "pegged"  v ( p e1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "pegging"  v ( p e1 g i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "pegs"  v ( p e1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "peignoir"  n ( p ei1 n w aa r ) ((pos "K6$" ) ))
( "peignoirs"  n ( p ei1 n w aa z ) ((pos "Kj$" ) ))
( "pejorative"  j ( p i1 jh o1 r @ t i v ) ((pos "OA%" ) ))
( "pejoratively"  a ( p i1 jh o1 r @ t i v l ii ) ((pos "Pu%" ) ))
( "peke"  n ( p ii1 k ) ((pos "K6$" ) ))
( "pekes"  n ( p ii1 k s ) ((pos "Kj$" ) ))
( "pekinese"  n ( p ii2 k i n ii1 z ) ((pos "K9%" ) ))
( "pekoe"  n ( p ii1 k ou ) ((pos "L@$" ) ))
( "pelf"  n ( p e1 l f ) ((pos "L@$" ) ))
( "pelican"  n ( p e1 l i k @ n ) ((pos "K6%" ) ))
( "pelicans"  n ( p e1 l i k @ n z ) ((pos "Kj%" ) ))
( "pelisse"  n ( p e1 l ii1 s ) ((pos "K6$" ) ))
( "pelisses"  n ( p e1 l ii1 s i z ) ((pos "Kj$" ) ))
( "pell-mell"  a ( p e2 l - m e1 l ) ((pos "Pu%" ) ))
( "pellet"  n ( p e1 l i t ) ((pos "K6%" ) ))
( "pellets"  n ( p e1 l i t s ) ((pos "Kj%" ) ))
( "pellucid"  j ( p e1 l uu1 s i d ) ((pos "OA%" ) ))
( "pellucidly"  a ( p e1 l uu1 s i d l ii ) ((pos "Pu%" ) ))
( "pelmet"  n ( p e1 l m i t ) ((pos "K6%" ) ))
( "pelmets"  n ( p e1 l m i t s ) ((pos "Kj%" ) ))
( "pelota"  n ( p @1 l ou1 t @ ) ((pos "L@$" ) ))
( "pelt"  v ( p e1 l t ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "pelted"  v ( p e1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "pelting"  v ( p e1 l t i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "pelts"  v ( p e1 l t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "pelves"  n ( p e1 l v ii z ) ((pos "Kj$" ) ))
( "pelvic"  j ( p e1 l v i k ) ((pos "OA%" ) ))
( "pelvis"  n ( p e1 l v i s ) ((pos "K7%" ) ))
( "pelvises"  n ( p e1 l v i s i z ) ((pos "Kj%" ) ))
( "pemmican"  n ( p e1 m i k @ n ) ((pos "L@$" ) ))
( "pen"  v ( p e1 n ) ((pos "H4%" "K6%" ) (subcat "15A" )))
( "pen-and-ink"  n ( p e2 n - @ n - i1 ng k ) ((pos "L@%" ) ))
( "pen-friend"  n ( p e1 n - f r e n d ) ((pos "K6%" ) ))
( "pen-friends"  n ( p e1 n - f r e n d z ) ((pos "Kj%" ) ))
( "pen-name"  n ( p e1 n - n ei m ) ((pos "K6%" ) ))
( "pen-names"  n ( p e1 n - n ei m z ) ((pos "Kj%" ) ))
( "pen-pusher"  n ( p e1 n - p u sh @ r ) ((pos "K6%" ) ))
( "pen-pushers"  n ( p e1 n - p u sh @ z ) ((pos "Kj%" ) ))
( "penal"  j ( p ii1 n l ) ((pos "OA%" ) ))
( "penalization"  n ( p ii2 n @ l ai z ei1 sh @ n ) ((pos "M6$" ) ))
( "penalizations"  n ( p ii2 n @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "penalize"  v ( p ii1 n @ l ai z ) ((pos "H2%" ) (subcat "6A" )))
( "penalized"  v ( p ii1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "penalizes"  v ( p ii1 n @ l ai z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "penalizing"  v ( p ii1 n @ l ai z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "penally"  a ( p ii1 n @ l ii ) ((pos "Pu%" ) ))
( "penalties"  n ( p e1 n l t i z ) ((pos "Mj%" ) ))
( "penalty"  n ( p e1 n l t ii ) ((pos "M8%" ) ))
( "penance"  n ( p e1 n @ n s ) ((pos "L@%" ) ))
( "pence"  n ( p e1 n s ) ((pos "Kj%" ) ))
( "penchant"  n ( p aa1 n sh aa n ) ((pos "K6%" ) ))
( "penchants"  n ( p aa1 n sh aa n z ) ((pos "Kj$" ) ))
( "pencil"  v ( p e1 n s l ) ((pos "H4%" "K6%" ) (subcat )))
( "pencilled"  v ( p e1 n s l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pencilling"  v ( p e1 n s @ l i ng ) ((pos "Hb%" ) (subcat )))
( "pencils"  v ( p e1 n s l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "pendant"  n ( p e1 n d @ n t ) ((pos "K6%" ) ))
( "pendants"  n ( p e1 n d @ n t s ) ((pos "Kj%" ) ))
( "pendent"  j ( p e1 n d @ n t ) ((pos "OA$" ) ))
( "pending"  j ( p e1 n d i ng ) ((pos "OA%" "T-%" ) ))
( "pendulous"  j ( p e1 n d y u l @ s ) ((pos "OA%" ) ))
( "pendulum"  n ( p e1 n d y u l @ m ) ((pos "K6%" ) ))
( "pendulums"  n ( p e1 n d y u l @ m z ) ((pos "Kj%" ) ))
( "penetrability"  n ( p e2 n i t r @ b i1 l i t ii ) ((pos "L@$" ) ))
( "penetrable"  j ( p e1 n i t r @ b l ) ((pos "OA%" ) ))
( "penetrate"  v ( p e1 n i t r ei t ) ((pos "J2%" ) (subcat "3A" )))
( "penetrated"  v ( p e1 n i t r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "penetrates"  v ( p e1 n i t r ei t s ) ((pos "Ja%" ) (subcat "3A" )))
( "penetrating"  v ( p e1 n i t r ei t i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "penetratingly"  a ( p e1 n i t r ei t i ng l ii ) ((pos "Pu%" ) ))
( "penetration"  n ( p e2 n i t r ei1 sh @ n ) ((pos "M6%" ) ))
( "penetrations"  n ( p e2 n i t r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "penetrative"  j ( p e1 n i t r @ t i v ) ((pos "OA$" ) ))
( "penguin"  n ( p e1 ng g w i n ) ((pos "K6%" ) ))
( "penguins"  n ( p e1 ng g w i n z ) ((pos "Kj%" ) ))
( "penicillin"  n ( p e2 n i s i1 l i n ) ((pos "L@%" ) ))
( "peninsula"  n ( p @1 n i1 n s y u l @ ) ((pos "K6%" ) ))
( "peninsular"  j ( p @1 n i1 n s y u l @ r ) ((pos "OA%" ) ))
( "peninsulas"  n ( p @1 n i1 n s y u l @ z ) ((pos "Kj%" ) ))
( "penis"  n ( p ii1 n i s ) ((pos "K7%" ) ))
( "penises"  n ( p ii1 n i s i z ) ((pos "Kj%" ) ))
( "penitence"  n ( p e1 n i t @ n s ) ((pos "L@%" ) ))
( "penitent"  n ( p e1 n i t @ n t ) ((pos "K6%" "OA%" ) ))
( "penitential"  j ( p e2 n i t e1 n sh l ) ((pos "OA%" ) ))
( "penitentially"  a ( p e2 n i t e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "penitentiaries"  n ( p e2 n i t e1 n sh @ r i z ) ((pos "Kj%" ) ))
( "penitentiary"  n ( p e2 n i t e1 n sh @ r ii ) ((pos "K8%" "OA%" ) ))
( "penitently"  a ( p e1 n i t @ n t l ii ) ((pos "Pu%" ) ))
( "penitents"  n ( p e1 n i t @ n t s ) ((pos "Kj%" ) ))
( "penknife"  n ( p e1 n - n ai f ) ((pos "Ki%" ) ))
( "penknives"  n ( p e1 n - n ai v z ) ((pos "Kj%" ) ))
( "penmanship"  n ( p e1 n m @ n sh i p ) ((pos "L@%" ) ))
( "penn'orth"  n ( p e1 n @ th ) ((pos "K6$" ) ))
( "penn'orths"  n ( p e1 n @ th s ) ((pos "Kj$" ) ))
( "pennant"  n ( p e1 n @ n t ) ((pos "K6%" ) ))
( "pennants"  n ( p e1 n @ n t s ) ((pos "Kj%" ) ))
( "penned"  v ( p e1 n d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "pennies"  n ( p e1 n i z ) ((pos "Kj%" ) ))
( "penniless"  j ( p e1 n i l i s ) ((pos "OA%" ) ))
( "penning"  v ( p e1 n i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "pennon"  n ( p e1 n @ n ) ((pos "K6$" ) ))
( "pennons"  n ( p e1 n @ n z ) ((pos "Kj$" ) ))
( "penny"  n ( p e1 n ii ) ((pos "K8%" ) ))
( "pennyweight"  n ( p e1 n i w ei t ) ((pos "K6$" ) ))
( "pennyweights"  n ( p e1 n i w ei t s ) ((pos "Kj$" ) ))
( "pennyworth"  n ( p e1 n i w @ th ) ((pos "K6%" ) ))
( "pennyworths"  n ( p e1 n i w @ th s ) ((pos "Kj$" ) ))
( "penology"  n ( p ii1 n o1 l @ jh ii ) ((pos "L@$" ) ))
( "pens"  v ( p e1 n z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "pension"  n ( p o1 n s i o n ) ((pos "K6$" ) ))
( "pension"  v ( p e1 n sh @ n ) ((pos "H0%" "K6%" ) (subcat )))
( "pensionable"  j ( p e1 n sh @ n @ b l ) ((pos "OA%" ) ))
( "pensioned"  v ( p e1 n sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pensioner"  n ( p e1 n sh @ n @ r ) ((pos "K6%" ) ))
( "pensioners"  n ( p e1 n sh @ n @ z ) ((pos "Kj%" ) ))
( "pensioning"  v ( p e1 n sh @ n i ng ) ((pos "Hb$" ) (subcat )))
( "pensions"  n ( p o1 n s i o n z ) ((pos "Kj$" ) ))
( "pensions"  v ( p e1 n sh @ n z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "pensive"  j ( p e1 n s i v ) ((pos "OA%" ) ))
( "pensively"  a ( p e1 n s i v l ii ) ((pos "Pu%" ) ))
( "pensiveness"  n ( p e1 n s i v n @ s ) ((pos "L@%" ) ))
( "penstock"  n ( p e1 n s t o k ) ((pos "K6$" ) ))
( "penstocks"  n ( p e1 n s t o k s ) ((pos "Kj$" ) ))
( "pent-up"  j ( p e2 n t - uh1 p ) ((pos "OA%" ) ))
( "pentagon"  n ( p e1 n t @ g @ n ) ((pos "K6%" ) ))
( "pentagonal"  j ( p e1 n t a1 g @ n l ) ((pos "OA%" ) ))
( "pentagons"  n ( p e1 n t @ g @ n z ) ((pos "Kj%" ) ))
( "pentameter"  n ( p e1 n t a1 m i t @ r ) ((pos "K6%" ) ))
( "pentameters"  n ( p e1 n t a1 m i t @ z ) ((pos "Kj%" ) ))
( "pentathlon"  n ( p e1 n t a1 th l @ n ) ((pos "K6%" ) ))
( "pentathlons"  n ( p e1 n t a1 th l @ n z ) ((pos "Kj$" ) ))
( "penthouse"  n ( p e1 n t h au s ) ((pos "K6%" ) ))
( "penthouses"  n ( p e1 n t h au z i z ) ((pos "Kj%" ) ))
( "penultimate"  n ( p e1 n uh1 l t i m @ t ) ((pos "K6$" "OA%" ) ))
( "penultimates"  n ( p e1 n uh1 l t i m @ t s ) ((pos "Kj$" ) ))
( "penumbra"  n ( p i1 n uh1 m b r @ ) ((pos "K6%" ) ))
( "penumbras"  n ( p i1 n uh1 m b r @ z ) ((pos "Kj$" ) ))
( "penurious"  j ( p i1 n y u@1 r i@ s ) ((pos "OA%" ) ))
( "penuriously"  a ( p i1 n y u@1 r i@ s l ii ) ((pos "Pu$" ) ))
( "penuriousness"  n ( p i1 n y u@1 r i@ s n @ s ) ((pos "L@$" ) ))
( "penury"  n ( p e1 n y u@ r ii ) ((pos "L@%" ) ))
( "peon"  n ( p ii1 @ n ) ((pos "K6$" ) ))
( "peonage"  n ( p ii1 @ n i jh ) ((pos "L@$" ) ))
( "peonies"  n ( p ii1 @ n i z ) ((pos "Kj$" ) ))
( "peons"  n ( p ii1 @ n z ) ((pos "Kj$" ) ))
( "peony"  n ( p ii1 @ n ii ) ((pos "K8$" ) ))
( "people"  v ( p ii1 p l ) ((pos "H2$" "M6*" ) (subcat )))
( "peopled"  v ( p ii1 p l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "peoples"  v ( p ii1 p l z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "peopling"  v ( p ii1 p l i ng ) ((pos "Hb$" ) (subcat )))
( "pep"  v ( p e1 p ) ((pos "H4%" "L@%" ) (subcat )))
( "pepped"  v ( p e1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pepper"  v ( p e1 p @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "pepper-and-salt"  n ( p e2 p @ r - @ n - s o1 l t ) ((pos "L@%" ) ))
( "pepper-mill"  n ( p e1 p @ - m i l ) ((pos "K6%" ) ))
( "pepper-mills"  n ( p e1 p @ - m i l z ) ((pos "Kj%" ) ))
( "pepper-pot"  n ( p e1 p @ - p o t ) ((pos "K6%" ) ))
( "pepper-pots"  n ( p e1 p @ - p o t s ) ((pos "Kj%" ) ))
( "peppercorn"  n ( p e1 p @ k oo n ) ((pos "K6%" ) ))
( "peppercorns"  n ( p e1 p @ k oo n z ) ((pos "Kj%" ) ))
( "peppered"  v ( p e1 p @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "peppering"  v ( p e1 p @ r i ng ) ((pos "Hb%" ) (subcat )))
( "peppermint"  n ( p e1 p @ m i n t ) ((pos "M6%" ) ))
( "peppermints"  n ( p e1 p @ m i n t s ) ((pos "Mj%" ) ))
( "peppers"  v ( p e1 p @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "peppery"  j ( p e1 p @ r ii ) ((pos "OA%" ) ))
( "pepping"  v ( p e1 p i ng ) ((pos "Hb%" ) (subcat )))
( "peps"  v ( p e1 p s ) ((pos "Ha%" ) (subcat )))
( "pepsin"  n ( p e1 p s i n ) ((pos "L@%" ) ))
( "peptic"  j ( p e1 p t i k ) ((pos "OA%" ) ))
( "per"  in ( p @@ r ) ((pos "T-%" ) ))
( "peradventure"  a ( p @@2 r @ d v e1 n ch @ r ) ((pos "Pu%" ) ))
( "perambulate"  v ( p @1 r a1 m b y u l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "perambulated"  v ( p @1 r a1 m b y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "perambulates"  v ( p @1 r a1 m b y u l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "perambulating"  v ( p @1 r a1 m b y u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "perambulation"  n ( p @1 r a2 m b y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "perambulations"  n ( p @1 r a2 m b y u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "perambulator"  n ( p @1 r a1 m b y u l ei t @ r ) ((pos "K6%" ) ))
( "perambulators"  n ( p @1 r a1 m b y u l ei t @ z ) ((pos "Kj%" ) ))
( "perceivable"  j ( p @1 s ii1 v @ b l ) ((pos "OA%" ) ))
( "perceive"  v ( p @1 s ii1 v ) ((pos "H2%" ) (subcat "6A" "8" "9" "10" "18A" "19A" )))
( "perceived"  v ( p @1 s ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" "18A" "19A" )))
( "perceiver"  n ( p @1 s ii1 v @ r ) ((pos "K6%" ) ))
( "perceivers"  n ( p @1 s ii1 v @ z ) ((pos "Kj%" ) ))
( "perceives"  v ( p @1 s ii1 v z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" "18A" "19A" )))
( "perceiving"  v ( p @1 s ii1 v i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" "18A" "19A" )))
( "percent"  n ( p @1 s e1 n t ) ((pos "K9%" ) ))
( "percentage"  n ( p @1 s e1 n t i jh ) ((pos "K6%" ) ))
( "percentages"  n ( p @1 s e1 n t i jh i z ) ((pos "Kj%" ) ))
( "perceptibility"  n ( p @1 s e2 p t @ b i1 l i t ii ) ((pos "L@$" ) ))
( "perceptible"  j ( p @1 s e1 p t @ b l ) ((pos "OA%" ) ))
( "perceptibly"  a ( p @1 s e1 p t @ b l ii ) ((pos "Pu%" ) ))
( "perception"  n ( p @1 s e1 p sh @ n ) ((pos "M6%" ) ))
( "perceptions"  n ( p @1 s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "perceptive"  j ( p @1 s e1 p t i v ) ((pos "OA%" ) ))
( "perceptively"  a ( p @1 s e1 p t i v l ii ) ((pos "Pu%" ) ))
( "perceptual"  j ( p @1 s e1 p t y u@ l ) ((pos "OA%" ) ))
( "perceptually"  a ( p @1 s e1 p t y u@ l ii ) ((pos "Pu%" ) ))
( "perch"  v ( p @@1 ch ) ((pos "J1%" "K9%" ) (subcat )))
( "perchance"  a ( p @1 ch aa1 n s ) ((pos "Pu%" ) ))
( "perched"  v ( p @@1 ch t ) ((pos "Jc%" "Jd%" ) (subcat )))
( "perches"  v ( p @@1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat )))
( "perching"  v ( p @@1 ch i ng ) ((pos "Jb%" ) (subcat )))
( "percipient"  j ( p @1 s i1 p i@ n t ) ((pos "OA$" ) ))
( "percolate"  v ( p @@1 k @ l ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "percolated"  v ( p @@1 k @ l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "percolates"  v ( p @@1 k @ l ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "percolating"  v ( p @@1 k @ l ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "percolator"  n ( p @@1 k @ l ei t @ r ) ((pos "K6%" ) ))
( "percolators"  n ( p @@1 k @ l ei t @ z ) ((pos "Kj%" ) ))
( "percussion"  n ( p @1 k uh1 sh @ n ) ((pos "M6%" ) ))
( "percussionist"  n ( p @1 k uh1 sh @ n i s t ) ((pos "K6%" ) ))
( "percussionists"  n ( p @1 k uh1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "percussions"  n ( p @1 k uh1 sh @ n z ) ((pos "Mj$" ) ))
( "perdition"  n ( p @1 d i1 sh @ n ) ((pos "L@$" ) ))
( "peregrination"  n ( p e2 r i g r i n ei1 sh @ n ) ((pos "M6%" ) ))
( "peregrinations"  n ( p e2 r i g r i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "peremptorily"  a ( p @1 r e1 m p t r @ l ii ) ((pos "Pu%" ) ))
( "peremptory"  j ( p @1 r e1 m p t @ r ii ) ((pos "OA%" ) ))
( "perennial"  n ( p @1 r e1 n i@ l ) ((pos "K6%" "OA%" ) ))
( "perennially"  a ( p @1 r e1 n i@ l ii ) ((pos "Pu%" ) ))
( "perennials"  n ( p @1 r e1 n i@ l z ) ((pos "Kj%" ) ))
( "perfect"  j ( p @@1 f i k t ) ((pos "OA%" ) ))
( "perfect"  v ( p @1 f e1 k t ) ((pos "H0%" ) (subcat )))
( "perfected"  v ( p @1 f e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perfectibility"  n ( p @1 f e2 k t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "perfectible"  j ( p @1 f e1 k t @ b l ) ((pos "OA%" ) ))
( "perfecting"  v ( p @1 f e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "perfection"  n ( p @1 f e1 k sh @ n ) ((pos "L@%" ) ))
( "perfectionist"  n ( p @1 f e1 k sh @ n i s t ) ((pos "K6%" ) ))
( "perfectionists"  n ( p @1 f e1 k sh @ n i s t s ) ((pos "Kj%" ) ))
( "perfectly"  a ( p @@1 f i k t l ii ) ((pos "Pu%" ) ))
( "perfects"  v ( p @1 f e1 k t s ) ((pos "Ha%" ) (subcat )))
( "perfervid"  j ( p @@1 f @@1 v i d ) ((pos "OA$" ) ))
( "perfidies"  n ( p @@1 f i d i z ) ((pos "Mj%" ) ))
( "perfidious"  j ( p @1 f i1 d i@ s ) ((pos "OA%" ) ))
( "perfidiously"  a ( p @1 f i1 d i@ s l ii ) ((pos "Pu%" ) ))
( "perfidiousness"  n ( p @1 f i1 d i@ s n @ s ) ((pos "L@%" ) ))
( "perfidy"  n ( p @@1 f i d ii ) ((pos "M8%" ) ))
( "perforate"  v ( p @@1 f @ r ei t ) ((pos "H2%" ) (subcat )))
( "perforated"  v ( p @@1 f @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perforates"  v ( p @@1 f @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "perforating"  v ( p @@1 f @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "perforation"  n ( p @@2 f @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "perforations"  n ( p @@2 f @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "perforce"  a ( p @1 f oo1 s ) ((pos "Pu%" ) ))
( "perform"  v ( p @1 f oo1 m ) ((pos "J0%" ) (subcat "2A" )))
( "performance"  n ( p @1 f oo1 m @ n s ) ((pos "M6%" ) ))
( "performances"  n ( p @1 f oo1 m @ n s i z ) ((pos "Mj%" ) ))
( "performed"  v ( p @1 f oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "performer"  n ( p @1 f oo1 m @ r ) ((pos "K6%" ) ))
( "performers"  n ( p @1 f oo1 m @ z ) ((pos "Kj%" ) ))
( "performing"  v ( p @1 f oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "performs"  v ( p @1 f oo1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "perfume"  n ( p @@1 f y uu m ) ((pos "M6%" ) ))
( "perfume"  v ( p @1 f y uu1 m ) ((pos "H2%" ) (subcat )))
( "perfumed"  v ( p @1 f y uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perfumer"  n ( p @@1 f y uu m @ r ) ((pos "K6%" ) ))
( "perfumers"  n ( p @@1 f y uu m @ z ) ((pos "Kj%" ) ))
( "perfumes"  n ( p @@1 f y uu m z ) ((pos "Mj%" ) ))
( "perfumes"  v ( p @1 f y uu1 m z ) ((pos "Ha%" ) (subcat )))
( "perfuming"  v ( p @1 f y uu1 m i ng ) ((pos "Hb%" ) (subcat )))
( "perfunctorily"  a ( p @1 f uh1 ng k t @ r i l ii ) ((pos "Pu%" ) ))
( "perfunctory"  j ( p @1 f uh1 ng k t @ r ii ) ((pos "OA%" ) ))
( "pergola"  n ( p @@1 g @ l @ ) ((pos "K6$" ) ))
( "pergolas"  n ( p @@1 g @ l @ z ) ((pos "Kj$" ) ))
( "perhaps"  a ( p @1 h a1 p s ) ((pos "Pu*" ) ))
( "peri"  n ( p i@1 r ii ) ((pos "K6$" ) ))
( "perigee"  n ( p e1 r i jh ii ) ((pos "K6$" ) ))
( "perigees"  n ( p e1 r i jh ii z ) ((pos "Kj$" ) ))
( "perihelion"  n ( p e2 r i h ii1 l i@ n ) ((pos "K6$" ) ))
( "perihelions"  n ( p e2 r i h ii1 l i@ n z ) ((pos "Kj$" ) ))
( "peril"  v ( p e1 r @ l ) ((pos "H4%" "M6%" ) ))
( "perilled"  v ( p e1 r @ l d ) ((pos "Hc%" "Hd%" ) ))
( "perilling"  v ( p e1 r @ l i ng ) ((pos "Hb%" ) ))
( "perilous"  j ( p e1 r @ l @ s ) ((pos "OA%" ) ))
( "perilously"  a ( p e1 r @ l @ s l ii ) ((pos "Pu%" ) ))
( "perils"  v ( p e1 r @ l z ) ((pos "Ha%" "Mj%" ) ))
( "perimeter"  n ( p @1 r i1 m i t @ r ) ((pos "K6%" ) ))
( "perimeters"  n ( p @1 r i1 m i t @ z ) ((pos "Kj%" ) ))
( "period"  n ( p i@1 r i@ d ) ((pos "K6%" ) ))
( "periodic"  j ( p i@2 r i o1 d i k ) ((pos "OA%" ) ))
( "periodical"  n ( p i@2 r i o1 d i k l ) ((pos "K6%" "OA%" ) ))
( "periodically"  a ( p i@2 r i o1 d i k l ii ) ((pos "Pu%" ) ))
( "periodicals"  n ( p i@2 r i o1 d i k l z ) ((pos "Kj%" ) ))
( "periods"  n ( p i@1 r i@ d z ) ((pos "Kj%" ) ))
( "peripatetic"  j ( p e2 r i p @ t e1 t i k ) ((pos "OA%" ) ))
( "peripheral"  j ( p @1 r i1 f @ r @ l ) ((pos "OA%" ) ))
( "peripheries"  n ( p @1 r i1 f @ r i z ) ((pos "Kj$" ) ))
( "periphery"  n ( p @1 r i1 f @ r ii ) ((pos "K8%" ) ))
( "periphrases"  n ( p @1 r i1 f r @ s ii z ) ((pos "Kj$" ) ))
( "periphrasis"  n ( p @1 r i1 f r @ s i s ) ((pos "Ki$" ) ))
( "periphrastic"  j ( p e2 r i f r a1 s t i k ) ((pos "OA$" ) ))
( "peris"  n ( p i@1 r i z ) ((pos "Kj$" ) ))
( "periscope"  n ( p e1 r i s k ou p ) ((pos "K6%" ) ))
( "periscopes"  n ( p e1 r i s k ou p s ) ((pos "Kj%" ) ))
( "perish"  v ( p e1 r i sh ) ((pos "J1%" ) (subcat "2A" "2C" )))
( "perishable"  j ( p e1 r i sh @ b l ) ((pos "OA%" ) ))
( "perishables"  n ( p e1 r i sh @ b l z ) ((pos "Kj%" ) ))
( "perished"  v ( p e1 r i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "perisher"  n ( p e1 r i sh @ r ) ((pos "K6%" ) ))
( "perishers"  n ( p e1 r i sh @ z ) ((pos "Kj%" ) ))
( "perishes"  v ( p e1 r i sh i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "perishing"  v ( p e1 r i sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "peristyle"  n ( p e1 r i s t ai l ) ((pos "K6$" ) ))
( "peristyles"  n ( p e1 r i s t ai l z ) ((pos "Kj$" ) ))
( "peritonitis"  n ( p e2 r i t @ n ai1 t i s ) ((pos "L@%" ) ))
( "periwig"  n ( p e1 r i w i g ) ((pos "K6$" ) ))
( "periwigs"  n ( p e1 r i w i g z ) ((pos "Kj$" ) ))
( "periwinkle"  n ( p e1 r i w i ng k l ) ((pos "K6%" ) ))
( "periwinkles"  n ( p e1 r i w i ng k l z ) ((pos "Kj%" ) ))
( "perjure"  v ( p @@1 jh @ r ) ((pos "H2%" ) (subcat )))
( "perjured"  v ( p @@1 jh @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perjurer"  n ( p @@1 jh @ r @ r ) ((pos "K6%" ) ))
( "perjurers"  n ( p @@1 jh @ r @ z ) ((pos "Kj%" ) ))
( "perjures"  v ( p @@1 jh @ z ) ((pos "Ha%" ) (subcat )))
( "perjuries"  n ( p @@1 jh @ r i z ) ((pos "Mj%" ) ))
( "perjuring"  v ( p @@1 jh @ r i ng ) ((pos "Hb%" ) (subcat )))
( "perjury"  n ( p @@1 jh @ r ii ) ((pos "M8%" ) ))
( "perk"  v ( p @@1 k ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "perked"  v ( p @@1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "perkier"  j ( p @@1 k i@ r ) ((pos "Or%" ) ))
( "perkiest"  j ( p @@1 k i i s t ) ((pos "Os%" ) ))
( "perkily"  a ( p @@1 k i l ii ) ((pos "Pu%" ) ))
( "perkiness"  n ( p @@1 k i n @ s ) ((pos "L@%" ) ))
( "perking"  v ( p @@1 k i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "perks"  v ( p @@1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "perky"  j ( p @@1 k ii ) ((pos "OD%" ) ))
( "perm"  v ( p @@1 m ) ((pos "H0%" "K6%" ) ))
( "permafrost"  n ( p @@1 m @ f r o s t ) ((pos "L@$" ) ))
( "permanence"  n ( p @@1 m @ n @ n s ) ((pos "L@%" ) ))
( "permanencies"  n ( p @@1 m @ n @ n s i z ) ((pos "Mj$" ) ))
( "permanency"  n ( p @@1 m @ n @ n s ii ) ((pos "M8$" ) ))
( "permanent"  j ( p @@1 m @ n @ n t ) ((pos "OA%" ) ))
( "permanently"  a ( p @@1 m @ n @ n t l ii ) ((pos "Pu%" ) ))
( "permanganate"  n ( p @1 m a1 ng g @ n ei t ) ((pos "M6$" ) ))
( "permanganates"  n ( p @1 m a1 ng g @ n ei t s ) ((pos "Mj$" ) ))
( "permeability"  n ( p @@2 m i@ b i1 l i t ii ) ((pos "L@%" ) ))
( "permeable"  j ( p @@1 m i@ b l ) ((pos "OA%" ) ))
( "permeate"  v ( p @@1 m i ei t ) ((pos "J2%" ) (subcat "3A" )))
( "permeated"  v ( p @@1 m i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "permeates"  v ( p @@1 m i ei t s ) ((pos "Ja%" ) (subcat "3A" )))
( "permeating"  v ( p @@1 m i ei t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "permeation"  n ( p @@2 m i ei1 sh @ n ) ((pos "L@$" ) ))
( "permed"  v ( p @@1 m d ) ((pos "Hc%" "Hd%" ) ))
( "perming"  v ( p @@1 m i ng ) ((pos "Hb%" ) ))
( "permissible"  j ( p @1 m i1 s @ b l ) ((pos "OA%" ) ))
( "permissibly"  a ( p @1 m i1 s @ b l ii ) ((pos "Pu%" ) ))
( "permission"  n ( p @1 m i1 sh @ n ) ((pos "L@%" ) ))
( "permissive"  j ( p @1 m i1 s i v ) ((pos "OA%" ) ))
( "permissiveness"  n ( p @1 m i1 s i v n @ s ) ((pos "L@%" ) ))
( "permit"  n ( p @@1 m i t ) ((pos "K6%" ) ))
( "permit"  v ( p @1 m i1 t ) ((pos "J4%" ) (subcat "3A" "6A" "6C" "17" )))
( "permits"  n ( p @@1 m i t s ) ((pos "Kj%" ) ))
( "permits"  v ( p @1 m i1 t s ) ((pos "Ja%" ) (subcat "3A" "6A" "6C" "17" )))
( "permitted"  v ( p @1 m i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "6C" "17" )))
( "permitting"  v ( p @1 m i1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "6C" "17" )))
( "perms"  v ( p @@1 m z ) ((pos "Ha%" "Kj%" ) ))
( "permutation"  n ( p @@2 m y uu t ei1 sh @ n ) ((pos "M6%" ) ))
( "permutations"  n ( p @@2 m y uu t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "permute"  v ( p @1 m y uu1 t ) ((pos "H2$" ) (subcat )))
( "permuted"  v ( p @1 m y uu1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "permutes"  v ( p @1 m y uu1 t s ) ((pos "Ha$" ) (subcat )))
( "permuting"  v ( p @1 m y uu1 t i ng ) ((pos "Hb$" ) (subcat )))
( "pernicious"  j ( p @1 n i1 sh @ s ) ((pos "OA%" ) ))
( "perniciously"  a ( p @1 n i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "perniciousness"  n ( p @1 n i1 sh @ s n @ s ) ((pos "L@%" ) ))
( "pernickety"  j ( p @1 n i1 k @ t ii ) ((pos "OA%" ) ))
( "peroration"  n ( p e2 r @ r ei1 sh @ n ) ((pos "K6$" ) ))
( "perorations"  n ( p e2 r @ r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "peroxide"  n ( p @1 r o1 k s ai d ) ((pos "M6%" ) ))
( "peroxides"  n ( p @1 r o1 k s ai d z ) ((pos "Mj%" ) ))
( "perpendicular"  n ( p @@2 p @ n d i1 k y u l @ r ) ((pos "M6%" "OA%" ) ))
( "perpendicularly"  a ( p @@2 p @ n d i1 k y u l @ l ii ) ((pos "Pu%" ) ))
( "perpendiculars"  n ( p @@2 p @ n d i1 k y u l @ z ) ((pos "Mj%" ) ))
( "perpetrate"  v ( p @@1 p i t r ei t ) ((pos "H2%" ) (subcat )))
( "perpetrated"  v ( p @@1 p i t r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perpetrates"  v ( p @@1 p i t r ei t s ) ((pos "Ha%" ) (subcat )))
( "perpetrating"  v ( p @@1 p i t r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "perpetration"  n ( p @@2 p i t r ei1 sh @ n ) ((pos "K6%" ) ))
( "perpetrations"  n ( p @@2 p i t r ei1 sh @ n z ) ((pos "Kj$" ) ))
( "perpetrator"  n ( p @@1 p i t r ei t @ r ) ((pos "K6%" ) ))
( "perpetrators"  n ( p @@1 p i t r ei t @ z ) ((pos "Kj%" ) ))
( "perpetual"  j ( p @1 p e1 ch u@ l ) ((pos "OA%" ) ))
( "perpetually"  a ( p @1 p e1 ch u@ l ii ) ((pos "Pu%" ) ))
( "perpetuate"  v ( p @1 p e1 ch u ei t ) ((pos "H2%" ) (subcat )))
( "perpetuated"  v ( p @1 p e1 ch u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perpetuates"  v ( p @1 p e1 ch u ei t s ) ((pos "Ha%" ) (subcat )))
( "perpetuating"  v ( p @1 p e1 ch u ei t i ng ) ((pos "Hb%" ) (subcat )))
( "perpetuation"  n ( p @1 p e2 ch u ei1 sh @ n ) ((pos "K6%" ) ))
( "perpetuations"  n ( p @1 p e2 ch u ei1 sh @ n z ) ((pos "Kj$" ) ))
( "perpetuities"  n ( p @@2 p i t y uu1 i t i z ) ((pos "Mj$" ) ))
( "perpetuity"  n ( p @@2 p i t y uu1 i t ii ) ((pos "M8%" ) ))
( "perplex"  v ( p @1 p l e1 k s ) ((pos "H1%" ) (subcat "6A" )))
( "perplexed"  v ( p @1 p l e1 k s t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "perplexedly"  a ( p @1 p l e1 k s @ d l ii ) ((pos "Pu%" ) ))
( "perplexes"  v ( p @1 p l e1 k s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "perplexing"  v ( p @1 p l e1 k s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "perplexities"  n ( p @1 p l e1 k s i t i z ) ((pos "Mj%" ) ))
( "perplexity"  n ( p @1 p l e1 k s i t ii ) ((pos "M8%" ) ))
( "perquisite"  n ( p @@1 k w i z i t ) ((pos "K6$" ) ))
( "perquisites"  n ( p @@1 k w i z i t s ) ((pos "Kj$" ) ))
( "perry"  n ( p e1 r ii ) ((pos "L@$" ) ))
( "persecute"  v ( p @@1 s i k y uu t ) ((pos "H2%" ) (subcat )))
( "persecuted"  v ( p @@1 s i k y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "persecutes"  v ( p @@1 s i k y uu t s ) ((pos "Ha%" ) (subcat )))
( "persecuting"  v ( p @@1 s i k y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "persecution"  n ( p @@2 s i k y uu1 sh @ n ) ((pos "M6%" ) ))
( "persecutions"  n ( p @@2 s i k y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "persecutor"  n ( p @@1 s i k y uu t @ r ) ((pos "K6%" ) ))
( "persecutors"  n ( p @@1 s i k y uu t @ z ) ((pos "Kj%" ) ))
( "perseverance"  n ( p @@2 s i v i@1 r @ n s ) ((pos "L@%" ) ))
( "persevere"  v ( p @@2 s i v i@1 r ) ((pos "I2%" ) (subcat "2A" )))
( "persevered"  v ( p @@2 s i v i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "perseveres"  v ( p @@2 s i v i@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "persevering"  v ( p @@2 s i v i@1 r i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "perseveringly"  a ( p @@2 s i v i@1 r i ng l ii ) ((pos "Pu%" ) ))
( "persiflage"  n ( p @@1 s i f l aa zh ) ((pos "L@$" ) ))
( "persimmon"  n ( p @1 s i1 m @ n ) ((pos "K6$" ) ))
( "persimmons"  n ( p @1 s i1 m @ n z ) ((pos "Kj$" ) ))
( "persist"  v ( p @1 s i1 s t ) ((pos "I0%" ) (subcat "2A" )))
( "persisted"  v ( p @1 s i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "persistence"  n ( p @1 s i1 s t @ n s ) ((pos "L@%" ) ))
( "persistent"  j ( p @1 s i1 s t @ n t ) ((pos "OA%" ) ))
( "persistently"  a ( p @1 s i1 s t @ n t l ii ) ((pos "Pu%" ) ))
( "persisting"  v ( p @1 s i1 s t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "persists"  v ( p @1 s i1 s t s ) ((pos "Ia%" ) (subcat "2A" )))
( "person"  n ( p @@1 s n ) ((pos "K6%" ) ))
( "persona"  n ( p @@1 s ou1 n @ ) ((pos "K6%" ) ))
( "persona grata"  n ( p @1 s ou2 n @ - g r aa1 t @ ) ((pos "Ki%" ) ))
( "persona non grata"  n ( p @1 s ou2 n @ - n o2 n - g r aa1 t @ ) ((pos "Ki%" ) ))
( "personable"  j ( p @@1 s @ n @ b l ) ((pos "OA%" ) ))
( "personage"  n ( p @@1 s @ n i jh ) ((pos "K6%" ) ))
( "personages"  n ( p @@1 s @ n i jh i z ) ((pos "Kj%" ) ))
( "personal"  n ( p @@1 s @ n l ) ((pos "K6%" "OA%" ) ))
( "personalities"  n ( p @@2 s @ n a1 l i t i z ) ((pos "Mj%" ) ))
( "personality"  n ( p @@2 s @ n a1 l i t ii ) ((pos "M8%" ) ))
( "personalize"  v ( p @@1 s @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "personalized"  v ( p @@1 s @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "personalizes"  v ( p @@1 s @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "personalizing"  v ( p @@1 s @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "personally"  a ( p @@1 s @ n @ l ii ) ((pos "Pu%" ) ))
( "personals"  n ( p @@1 s @ n l z ) ((pos "Kj$" ) ))
( "personalty"  n ( p @@1 s @ n l t ii ) ((pos "L@$" ) ))
( "personas"  n ( p @@1 s ou1 n @ z ) ((pos "Kj%" ) ))
( "personate"  v ( p @@1 s @ n ei t ) ((pos "H2$" ) (subcat )))
( "personated"  v ( p @@1 s @ n ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "personates"  v ( p @@1 s @ n ei t s ) ((pos "Ha$" ) (subcat )))
( "personating"  v ( p @@1 s @ n ei t i ng ) ((pos "Hb$" ) (subcat )))
( "personation"  n ( p @@2 s @ n ei1 sh @ n ) ((pos "K6$" ) ))
( "personations"  n ( p @@2 s @ n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "personification"  n ( p @1 s o2 n i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "personifications"  n ( p @1 s o2 n i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "personified"  v ( p @1 s o1 n i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "personifies"  v ( p @1 s o1 n i f ai z ) ((pos "Ha%" ) (subcat )))
( "personify"  v ( p @1 s o1 n i f ai ) ((pos "H3%" ) (subcat )))
( "personifying"  v ( p @1 s o1 n i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "personnel"  n ( p @@2 s @ n e1 l ) ((pos "Kj%" ) ))
( "persons"  n ( p @@1 s n z ) ((pos "Kj%" ) ))
( "perspective"  n ( p @1 s p e1 k t i v ) ((pos "M6%" ) ))
( "perspectives"  n ( p @1 s p e1 k t i v z ) ((pos "Mj%" ) ))
( "perspex"  n ( p @@1 s p e k s ) ((pos "L@%" ) ))
( "perspicacious"  j ( p @@2 s p i k ei1 sh @ s ) ((pos "OA%" ) ))
( "perspicacity"  n ( p @@2 s p i k a1 s i t ii ) ((pos "L@%" ) ))
( "perspicuity"  n ( p @@2 s p i k y uu1 i t ii ) ((pos "L@%" ) ))
( "perspicuous"  j ( p @1 s p i1 k y u@ s ) ((pos "OA%" ) ))
( "perspicuously"  a ( p @1 s p i1 k y u@ s l ii ) ((pos "Pu$" ) ))
( "perspicuousness"  n ( p @1 s p i1 k y u@ s n @ s ) ((pos "L@%" ) ))
( "perspiration"  n ( p @@2 s p @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "perspire"  v ( p @1 s p ai1 @ r ) ((pos "I2%" ) (subcat )))
( "perspired"  v ( p @1 s p ai1 @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "perspires"  v ( p @1 s p ai1 @ z ) ((pos "Ia%" ) (subcat )))
( "perspiring"  v ( p @1 s p ai1 @ r i ng ) ((pos "Ib%" ) (subcat )))
( "persuadable"  j ( p @1 s w ei1 d @ b l ) ((pos "OA%" ) ))
( "persuade"  v ( p @1 s w ei1 d ) ((pos "H2%" ) (subcat "11" "14" )))
( "persuaded"  v ( p @1 s w ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "11" "14" )))
( "persuades"  v ( p @1 s w ei1 d z ) ((pos "Ha%" ) (subcat "11" "14" )))
( "persuading"  v ( p @1 s w ei1 d i ng ) ((pos "Hb%" ) (subcat "11" "14" )))
( "persuasion"  n ( p @1 s w ei1 zh n ) ((pos "M6%" ) ))
( "persuasions"  n ( p @1 s w ei1 zh n z ) ((pos "Mj%" ) ))
( "persuasive"  j ( p @1 s w ei1 s i v ) ((pos "OA%" ) ))
( "persuasively"  a ( p @1 s w ei1 s i v l ii ) ((pos "Pu%" ) ))
( "persuasiveness"  n ( p @1 s w ei1 s i v n @ s ) ((pos "L@%" ) ))
( "pert"  j ( p @@1 t ) ((pos "OA%" ) ))
( "pertain"  v ( p @1 t ei1 n ) ((pos "I0%" ) (subcat )))
( "pertained"  v ( p @1 t ei1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "pertaining"  v ( p @1 t ei1 n i ng ) ((pos "Ib%" ) (subcat )))
( "pertains"  v ( p @1 t ei1 n z ) ((pos "Ia%" ) (subcat )))
( "pertinacious"  j ( p @@2 t i n ei1 sh @ s ) ((pos "OA$" ) ))
( "pertinaciously"  a ( p @@2 t i n ei1 sh @ s l ii ) ((pos "Pu$" ) ))
( "pertinacity"  n ( p @@2 t i n a1 s i t ii ) ((pos "L@$" ) ))
( "pertinence"  n ( p @@1 t i n @ n s ) ((pos "L@$" ) ))
( "pertinent"  j ( p @@1 t i n @ n t ) ((pos "OA%" ) ))
( "pertinently"  a ( p @@1 t i n @ n t l ii ) ((pos "Pu%" ) ))
( "pertly"  a ( p @@1 t l ii ) ((pos "Pu%" ) ))
( "pertness"  n ( p @@1 t n @ s ) ((pos "L@%" ) ))
( "perturb"  v ( p @1 t @@1 b ) ((pos "H0%" ) (subcat )))
( "perturbation"  n ( p @@2 t @ b ei1 sh @ n ) ((pos "M6%" ) ))
( "perturbations"  n ( p @@2 t @ b ei1 sh @ n z ) ((pos "Mj%" ) ))
( "perturbed"  v ( p @1 t @@1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perturbing"  v ( p @1 t @@1 b i ng ) ((pos "Hb%" ) (subcat )))
( "perturbs"  v ( p @1 t @@1 b z ) ((pos "Ha%" ) (subcat )))
( "peruke"  n ( p @1 r uu1 k ) ((pos "K6$" ) ))
( "perukes"  n ( p @1 r uu1 k s ) ((pos "Kj$" ) ))
( "perusal"  n ( p @1 r uu1 z l ) ((pos "M6%" ) ))
( "perusals"  n ( p @1 r uu1 z l z ) ((pos "Mj$" ) ))
( "peruse"  v ( p @1 r uu1 z ) ((pos "H2%" ) (subcat )))
( "perused"  v ( p @1 r uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "peruses"  v ( p @1 r uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "perusing"  v ( p @1 r uu1 z i ng ) ((pos "Hb%" ) (subcat )))
( "pervade"  v ( p @1 v ei1 d ) ((pos "H2%" ) (subcat )))
( "pervaded"  v ( p @1 v ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pervades"  v ( p @1 v ei1 d z ) ((pos "Ha%" ) (subcat )))
( "pervading"  v ( p @1 v ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "pervasion"  n ( p @1 v ei1 zh n ) ((pos "L@$" ) ))
( "pervasive"  j ( p @1 v ei1 s i v ) ((pos "OA%" ) ))
( "pervasively"  a ( p @1 v ei1 s i v l ii ) ((pos "Pu%" ) ))
( "pervasiveness"  n ( p @1 v ei1 s i v n @ s ) ((pos "L@%" ) ))
( "perverse"  j ( p @1 v @@1 s ) ((pos "OA%" ) ))
( "perversely"  a ( p @1 v @@1 s l ii ) ((pos "Pu%" ) ))
( "perverseness"  n ( p @1 v @@1 s n @ s ) ((pos "L@%" ) ))
( "perversion"  n ( p @1 v @@1 sh @ n ) ((pos "M6%" ) ))
( "perversions"  n ( p @1 v @@1 sh @ n z ) ((pos "Mj%" ) ))
( "perversities"  n ( p @1 v @@1 s i t i z ) ((pos "Mj%" ) ))
( "perversity"  n ( p @1 v @@1 s i t ii ) ((pos "M8%" ) ))
( "pervert"  n ( p @@1 v @@ t ) ((pos "K6%" ) ))
( "pervert"  v ( p @1 v @@1 t ) ((pos "H0%" ) (subcat )))
( "perverted"  v ( p @1 v @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "perverting"  v ( p @1 v @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "perverts"  n ( p @@1 v @@ t s ) ((pos "Kj%" ) ))
( "perverts"  v ( p @1 v @@1 t s ) ((pos "Ha%" ) (subcat )))
( "peseta"  n ( p @1 s ei1 t @ ) ((pos "K6%" ) ))
( "pesetas"  n ( p @1 s ei1 t @ z ) ((pos "Kj%" ) ))
( "peskier"  j ( p e1 s k i@ r ) ((pos "Or%" ) ))
( "peskiest"  j ( p e1 s k i i s t ) ((pos "Os%" ) ))
( "pesky"  j ( p e1 s k ii ) ((pos "OD%" ) ))
( "peso"  n ( p ei1 s ou ) ((pos "K6%" ) ))
( "pesos"  n ( p ei1 s ou z ) ((pos "Kj%" ) ))
( "pessaries"  n ( p e1 s @ r i z ) ((pos "Kj%" ) ))
( "pessary"  n ( p e1 s @ r ii ) ((pos "K8%" ) ))
( "pessimism"  n ( p e1 s i m i z @ m ) ((pos "L@%" ) ))
( "pessimist"  n ( p e1 s i m i s t ) ((pos "K6%" ) ))
( "pessimistic"  j ( p e2 s i m i1 s t i k ) ((pos "OA%" ) ))
( "pessimistically"  a ( p e2 s i m i1 s t i k l ii ) ((pos "Pu%" ) ))
( "pessimists"  n ( p e1 s i m i s t s ) ((pos "Kj%" ) ))
( "pest"  n ( p e1 s t ) ((pos "K6%" ) ))
( "pester"  v ( p e1 s t @ r ) ((pos "H0%" ) (subcat "6A" "14" )))
( "pestered"  v ( p e1 s t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "pestering"  v ( p e1 s t @ r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "pesters"  v ( p e1 s t @ z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "pesticide"  n ( p e1 s t i s ai d ) ((pos "M6%" ) ))
( "pesticides"  n ( p e1 s t i s ai d z ) ((pos "Mj%" ) ))
( "pestiferous"  j ( p e1 s t i1 f @ r @ s ) ((pos "OA$" ) ))
( "pestilence"  n ( p e1 s t i l @ n s ) ((pos "M6%" ) ))
( "pestilences"  n ( p e1 s t i l @ n s i z ) ((pos "Mj$" ) ))
( "pestilent"  j ( p e1 s t i l @ n t ) ((pos "OA%" ) ))
( "pestilential"  j ( p e2 s t i l e1 n sh l ) ((pos "OA%" ) ))
( "pestle"  v ( p e1 s l ) ((pos "H2$" "K6$" ) ))
( "pestled"  v ( p e1 s l d ) ((pos "Hc$" "Hd$" ) ))
( "pestles"  v ( p e1 s l z ) ((pos "Ha$" "Kj$" ) ))
( "pestling"  v ( p e1 s l i ng ) ((pos "Hb$" ) ))
( "pests"  n ( p e1 s t s ) ((pos "Kj%" ) ))
( "pet"  v ( p e1 t ) ((pos "H4%" "K6%" ) ))
( "petal"  n ( p e1 t l ) ((pos "K6%" ) ))
( "petaled"  j ( p e1 t l d ) ((pos "OA$" ) ))
( "petalled"  j ( p e1 t l d ) ((pos "OA%" ) ))
( "petals"  n ( p e1 t l z ) ((pos "Kj%" ) ))
( "petard"  n ( p i1 t aa1 d ) ((pos "K6%" ) ))
( "petards"  n ( p i1 t aa1 d z ) ((pos "Kj$" ) ))
( "peter"  v ( p ii1 t @ r ) ((pos "I0%" ) (subcat )))
( "petered"  v ( p ii1 t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "petering"  v ( p ii1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "peters"  v ( p ii1 t @ z ) ((pos "Ia%" ) (subcat )))
( "petit bourgeois"  n ( p e2 t i - b u@1 zh w aa ) ((pos "Ki%" ) ))
( "petite"  j ( p @1 t ii1 t ) ((pos "OA%" ) ))
( "petition"  v ( p i1 t i1 sh @ n ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" "11" "14" )))
( "petitioned"  v ( p i1 t i1 sh @ n d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "11" "14" )))
( "petitioner"  n ( p i1 t i1 sh @ n @ r ) ((pos "K6%" ) ))
( "petitioners"  n ( p i1 t i1 sh @ n @ z ) ((pos "Kj%" ) ))
( "petitioning"  v ( p i1 t i1 sh @ n i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "11" "14" )))
( "petitions"  v ( p i1 t i1 sh @ n z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "11" "14" )))
( "petits bourgeois"  n ( p e2 t i - b u@1 zh w aa z ) ((pos "Kj$" ) ))
( "petrel"  n ( p e1 t r @ l ) ((pos "K6$" ) ))
( "petrels"  n ( p e1 t r @ l z ) ((pos "Kj$" ) ))
( "petrifaction"  n ( p e2 t r i f a1 k sh @ n ) ((pos "M6%" ) ))
( "petrifactions"  n ( p e2 t r i f a1 k sh @ n z ) ((pos "Mj$" ) ))
( "petrified"  v ( p e1 t r i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "petrifies"  v ( p e1 t r i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "petrify"  v ( p e1 t r i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "petrifying"  v ( p e1 t r i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "petro-"  j ( p e2 t r ou - ) ((pos "U-%" ) ))
( "petro-chemical"  n ( p e2 t r ou - k e1 m i k l ) ((pos "K6%" ) ))
( "petro-chemicals"  n ( p e2 t r ou - k e1 m i k l z ) ((pos "Kj%" ) ))
( "petrol"  n ( p e1 t r @ l ) ((pos "L@%" ) ))
( "petroleum"  n ( p i1 t r ou1 l i@ m ) ((pos "L@%" ) ))
( "petrology"  n ( p i1 t r o1 l @ jh ii ) ((pos "L@$" ) ))
( "pets"  v ( p e1 t s ) ((pos "Ha%" "Kj%" ) ))
( "petted"  v ( p e1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "petticoat"  n ( p e1 t i k ou t ) ((pos "K6%" ) ))
( "petticoats"  n ( p e1 t i k ou t s ) ((pos "Kj%" ) ))
( "pettier"  j ( p e1 t i@ r ) ((pos "Or%" ) ))
( "pettiest"  j ( p e1 t i i s t ) ((pos "Os%" ) ))
( "pettifogging"  j ( p e1 t i f o g i ng ) ((pos "OA%" ) ))
( "pettily"  a ( p e1 t i l ii ) ((pos "Pu%" ) ))
( "pettiness"  n ( p e1 t i n @ s ) ((pos "M7%" ) ))
( "pettinesses"  n ( p e1 t i n @ s i z ) ((pos "Kj%" ) ))
( "petting"  v ( p e1 t i ng ) ((pos "Hb%" ) ))
( "pettish"  j ( p e1 t i sh ) ((pos "OA%" ) ))
( "pettishly"  a ( p e1 t i sh l ii ) ((pos "Pu%" ) ))
( "pettishness"  n ( p e1 t i sh n @ s ) ((pos "L@%" ) ))
( "petty"  j ( p e1 t ii ) ((pos "OD%" ) ))
( "petulance"  n ( p e1 t y u l @ n s ) ((pos "L@%" ) ))
( "petulant"  j ( p e1 t y u l @ n t ) ((pos "OA%" ) ))
( "petulantly"  a ( p e1 t y u l @ n t l ii ) ((pos "Pu%" ) ))
( "petunia"  n ( p i1 t y uu1 n i@ ) ((pos "K6%" ) ))
( "petunias"  n ( p i1 t y uu1 n i@ z ) ((pos "Kj%" ) ))
( "pew"  n ( p y uu1 ) ((pos "K6%" ) ))
( "pew-opener"  n ( p y uu1 - ou p @ n @ r ) ((pos "K6$" ) ))
( "pew-openers"  n ( p y uu1 - ou p @ n @ z ) ((pos "Kj$" ) ))
( "pewit"  n ( p ii1 w i t ) ((pos "K6$" ) ))
( "pewits"  n ( p ii1 w i t s ) ((pos "Kj$" ) ))
( "pews"  n ( p y uu1 z ) ((pos "Kj%" ) ))
( "pewter"  n ( p y uu1 t @ r ) ((pos "L@%" ) ))
( "peyote"  n ( p ei1 ou1 t ii ) ((pos "K6$" ) ))
( "peyotes"  n ( p ei1 ou1 t i z ) ((pos "Kj$" ) ))
( "pfennig"  n ( f e1 n i g ) ((pos "K6$" ) ))
( "pfennigs"  n ( f e1 n i g z ) ((pos "Kj$" ) ))
( "phaeton"  n ( f ei1 t n ) ((pos "K6$" ) ))
( "phaetons"  n ( f ei1 t n z ) ((pos "Kj$" ) ))
( "phagocyte"  n ( f a1 g @ s ai t ) ((pos "K6$" ) ))
( "phagocytes"  n ( f a1 g @ s ai t s ) ((pos "Kj$" ) ))
( "phalanges"  n ( f @1 l a1 n jh ii z ) ((pos "Kj$" ) ))
( "phalanx"  n ( f a1 l a ng k s ) ((pos "K7%" ) ))
( "phalanxes"  n ( f a1 l a ng k s i z ) ((pos "Kj%" ) ))
( "phallic"  j ( f a1 l i k ) ((pos "OA%" ) ))
( "phallus"  n ( f a1 l @ s ) ((pos "K7%" ) ))
( "phalluses"  n ( f a1 l @ s i z ) ((pos "Kj%" ) ))
( "phantasies"  n ( f a1 n t @ s i z ) ((pos "Kj$" ) ))
( "phantasm"  n ( f a1 n t a z @ m ) ((pos "K6%" ) ))
( "phantasmagoria"  n ( f a2 n t a z m @ g o1 r i@ ) ((pos "K6%" ) ))
( "phantasmagorias"  n ( f a2 n t a z m @ g o1 r i@ z ) ((pos "Kj$" ) ))
( "phantasmal"  j ( f a1 n t a1 z m @ l ) ((pos "OA%" ) ))
( "phantasms"  n ( f a1 n t a z @ m z ) ((pos "Kj%" ) ))
( "phantasy"  n ( f a1 n t @ s ii ) ((pos "K8%" ) ))
( "phantom"  n ( f a1 n t @ m ) ((pos "K6%" ) ))
( "phantoms"  n ( f a1 n t @ m z ) ((pos "Kj%" ) ))
( "pharisaic"  j ( f a2 r i s ei1 i k ) ((pos "OA$" ) ))
( "pharisaical"  j ( f a2 r i s ei1 i k @ l ) ((pos "OA$" ) ))
( "pharmaceutical"  j ( f aa2 m @ s y uu1 t i k l ) ((pos "OA%" ) ))
( "pharmacies"  n ( f aa1 m @ s i z ) ((pos "Mj%" ) ))
( "pharmacist"  n ( f aa1 m @ s i s t ) ((pos "K6%" ) ))
( "pharmacists"  n ( f aa1 m @ s i s t s ) ((pos "Kj%" ) ))
( "pharmacological"  j ( f aa2 m @ k @ l o1 jh i k l ) ((pos "OA%" ) ))
( "pharmacologically"  a ( f aa2 m @ k @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "pharmacologist"  n ( f aa2 m @ k o1 l @ jh i s t ) ((pos "K6%" ) ))
( "pharmacologists"  n ( f aa2 m @ k o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "pharmacology"  n ( f aa2 m @ k o1 l @ jh ii ) ((pos "L@%" ) ))
( "pharmacopoeia"  n ( f aa2 m @ k @ p ii1 @ ) ((pos "K6%" ) ))
( "pharmacopoeias"  n ( f aa2 m @ k @ p ii1 @ z ) ((pos "Kj$" ) ))
( "pharmacy"  n ( f aa1 m @ s ii ) ((pos "M8%" ) ))
( "pharos"  n ( f e@1 r o s ) ((pos "Ki$" ) ))
( "pharyngitis"  n ( f a2 r i n jh ai1 t i s ) ((pos "L@%" ) ))
( "pharynx"  n ( f a1 r i ng k s ) ((pos "K7%" ) ))
( "pharynxes"  n ( f a1 r i ng k s i z ) ((pos "Kj$" ) ))
( "phase"  v ( f ei1 z ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "phased"  v ( f ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "phases"  v ( f ei1 z i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "phasing"  v ( f ei1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pheasant"  n ( f e1 z n t ) ((pos "M6%" ) ))
( "pheasants"  n ( f e1 z n t s ) ((pos "Mj%" ) ))
( "phenobarbitone"  n ( f ii2 n ou b aa1 b i t ou n ) ((pos "L@$" ) ))
( "phenol"  n ( f ii1 n o l ) ((pos "L@$" ) ))
( "phenomena"  n ( f i1 n o1 m i n @ ) ((pos "Kj%" ) ))
( "phenomenal"  j ( f i1 n o1 m i n l ) ((pos "OA%" ) ))
( "phenomenally"  a ( f i1 n o1 m i n @ l ii ) ((pos "Pu%" ) ))
( "phenomenon"  n ( f i1 n o1 m i n @ n ) ((pos "Ki%" ) ))
( "phew"  uh ( f y uu1 ) ((pos "W-$" ) ))
( "phial"  n ( f ai1 @ l ) ((pos "K6%" ) ))
( "phials"  n ( f ai1 @ l z ) ((pos "Kj%" ) ))
( "philander"  v ( f i1 l a1 n d @ r ) ((pos "I0%" ) (subcat )))
( "philandered"  v ( f i1 l a1 n d @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "philanderer"  n ( f i1 l a1 n d @ r @ r ) ((pos "K6%" ) ))
( "philanderers"  n ( f i1 l a1 n d @ r @ z ) ((pos "Kj%" ) ))
( "philandering"  v ( f i1 l a1 n d @ r i ng ) ((pos "Ib%" ) (subcat )))
( "philanders"  v ( f i1 l a1 n d @ z ) ((pos "Ia%" ) (subcat )))
( "philanthropic"  j ( f i2 l @ n th r o1 p i k ) ((pos "OA%" ) ))
( "philanthropically"  a ( f i2 l @ n th r o1 p i k l ii ) ((pos "Pu%" ) ))
( "philanthropist"  n ( f i1 l a1 n th r @ p i s t ) ((pos "K6%" ) ))
( "philanthropists"  n ( f i1 l a1 n th r @ p i s t s ) ((pos "Kj%" ) ))
( "philanthropy"  n ( f i1 l a1 n th r @ p ii ) ((pos "L@%" ) ))
( "philatelic"  j ( f i2 l @ t e1 l i k ) ((pos "OA%" ) ))
( "philatelist"  n ( f ii1 l a1 t @ l i s t ) ((pos "K6%" ) ))
( "philatelists"  n ( f ii1 l a1 t @ l i s t s ) ((pos "Kj%" ) ))
( "philately"  n ( f i1 l a1 t @ l ii ) ((pos "L@%" ) ))
( "philharmonic"  j ( f i2 l @ m o1 n i k ) ((pos "OA%" ) ))
( "philhellene"  n ( f i1 l h e1 l ii n ) ((pos "K6$" "OA$" ) ))
( "philhellenes"  n ( f i1 l h e1 l ii n z ) ((pos "Kj$" ) ))
( "philhellenic"  j ( f i2 l h e l ii1 n i k ) ((pos "OA$" ) ))
( "philological"  j ( f i2 l @ l o1 jh i k l ) ((pos "OA%" ) ))
( "philologist"  n ( f i1 l o1 l @ jh i s t ) ((pos "K6%" ) ))
( "philologists"  n ( f i1 l o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "philology"  n ( f i1 l o1 l @ jh ii ) ((pos "L@%" ) ))
( "philosopher"  n ( f i1 l o1 s @ f @ r ) ((pos "K6%" ) ))
( "philosophers"  n ( f i1 l o1 s @ f @ z ) ((pos "Kj%" ) ))
( "philosophic"  j ( f i2 l @ s o1 f i k ) ((pos "OA%" ) ))
( "philosophical"  j ( f i2 l @ s o1 f i k l ) ((pos "OA%" ) ))
( "philosophically"  a ( f i2 l @ s o1 f i k l ii ) ((pos "Pu%" ) ))
( "philosophies"  n ( f i1 l o1 s @ f i z ) ((pos "Mj%" ) ))
( "philosophize"  v ( f i1 l o1 s @ f ai z ) ((pos "I2%" ) (subcat )))
( "philosophized"  v ( f i1 l o1 s @ f ai z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "philosophizes"  v ( f i1 l o1 s @ f ai z i z ) ((pos "Ia%" ) (subcat )))
( "philosophizing"  v ( f i1 l o1 s @ f ai z i ng ) ((pos "Ib%" ) (subcat )))
( "philosophy"  n ( f i1 l o1 s @ f ii ) ((pos "M8%" ) ))
( "philtre"  n ( f i1 l t @ r ) ((pos "K6$" ) ))
( "philtres"  n ( f i1 l t @ z ) ((pos "Kj$" ) ))
( "phlebitis"  n ( f l i1 b ai1 t i s ) ((pos "L@%" ) ))
( "phlegm"  n ( f l e1 m ) ((pos "L@%" ) ))
( "phlegmatic"  j ( f l e1 g m a1 t i k ) ((pos "OA%" ) ))
( "phlegmatically"  a ( f l e1 g m a1 t i k l ii ) ((pos "Pu%" ) ))
( "phlox"  n ( f l o1 k s ) ((pos "L@$" ) ))
( "phobia"  n ( f ou1 b i@ ) ((pos "K6%" ) ))
( "phobias"  n ( f ou1 b i@ z ) ((pos "Kj%" ) ))
( "phoenix"  n ( f ii1 n i k s ) ((pos "K7%" ) ))
( "phoenixes"  n ( f ii1 n i k s i z ) ((pos "Kj$" ) ))
( "phone"  v ( f ou1 n ) ((pos "J2%" "K6%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "phone-in"  n ( f ou1 n - i n ) ((pos "K6%" ) ))
( "phone-ins"  n ( f ou1 n - i n z ) ((pos "Kj%" ) ))
( "phonebooth"  n ( f ou1 n b uu dh ) ((pos "K6%" ) ))
( "phonebooths"  n ( f ou1 n b uu dh z ) ((pos "Kj%" ) ))
( "phonecall"  n ( f ou1 n k oo l ) ((pos "K6%" ) ))
( "phonecalls"  n ( f ou1 n k oo l z ) ((pos "Kj%" ) ))
( "phoned"  v ( f ou1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "phoneme"  n ( f ou1 n ii m ) ((pos "K6%" ) ))
( "phonemes"  n ( f ou1 n ii m z ) ((pos "Kj%" ) ))
( "phonemic"  j ( f @1 n ii1 m i k ) ((pos "OA%" ) ))
( "phonemics"  n ( f @1 n ii1 m i k s ) ((pos "Lk%" ) ))
( "phones"  v ( f ou1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "phonetic"  j ( f @1 n e1 t i k ) ((pos "OA%" ) ))
( "phonetically"  a ( f @1 n e1 t i k l ii ) ((pos "Pu%" ) ))
( "phonetician"  n ( f ou2 n i t i1 sh @ n ) ((pos "K6%" ) ))
( "phoneticians"  n ( f ou2 n i t i1 sh @ n z ) ((pos "Kj%" ) ))
( "phonetics"  n ( f @1 n e1 t i k s ) ((pos "Lk%" ) ))
( "phoney"  n ( f ou1 n ii ) ((pos "K6%" "OA%" ) ))
( "phoneys"  n ( f ou1 n i z ) ((pos "Kj$" ) ))
( "phonic"  j ( f o1 n i k ) ((pos "OA$" ) ))
( "phonics"  n ( f o1 n i k s ) ((pos "Lk$" ) ))
( "phonies"  n ( f ou1 n i z ) ((pos "Kj$" ) ))
( "phoning"  v ( f ou1 n i ng ) ((pos "Jb%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "phonograph"  n ( f ou1 n @ g r aa f ) ((pos "K6%" ) ))
( "phonographs"  n ( f ou1 n @ g r aa f s ) ((pos "Kj%" ) ))
( "phonological"  j ( f ou2 n @ l o1 jh i k l ) ((pos "OA%" ) ))
( "phonology"  n ( f @1 n o1 l @ jh ii ) ((pos "L@%" ) ))
( "phony"  n ( f ou1 n ii ) ((pos "K8$" "OA$" ) ))
( "phooey"  uh ( f uu1 ii ) ((pos "W-$" ) ))
( "phosgene"  n ( f o1 z jh ii n ) ((pos "L@$" ) ))
( "phosphate"  n ( f o1 s f ei t ) ((pos "M6%" ) ))
( "phosphates"  n ( f o1 s f ei t s ) ((pos "Mj%" ) ))
( "phosphorescence"  n ( f o2 s f @ r e1 s n s ) ((pos "L@%" ) ))
( "phosphorescent"  j ( f o2 s f @ r e1 s n t ) ((pos "OA%" ) ))
( "phosphoric"  j ( f o1 s f o1 r i k ) ((pos "OA%" ) ))
( "phosphorous"  j ( f o1 s f @ r @ s ) ((pos "OA$" ) ))
( "phosphorus"  n ( f o1 s f @ r @ s ) ((pos "L@%" ) ))
( "photo"  n ( f ou1 t ou ) ((pos "K6%" ) ))
( "photo-"  j ( f ou1 t ou - ) ((pos "U-%" ) ))
( "photocopied"  v ( f ou1 t ou k o p i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "photocopier"  n ( f ou1 t ou k o p i@ r ) ((pos "K6%" ) ))
( "photocopiers"  n ( f ou1 t ou k o p i@ z ) ((pos "Kj%" ) ))
( "photocopies"  v ( f ou1 t ou k o p i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "photocopy"  v ( f ou1 t ou k o p ii ) ((pos "H3%" "K8%" ) (subcat )))
( "photocopying"  v ( f ou1 t ou k o p i i ng ) ((pos "Hb%" ) (subcat )))
( "photoelectric"  j ( f ou2 t ou i l e1 k t r i k ) ((pos "OA%" ) ))
( "photoflash"  n ( f ou1 t ou f l a sh ) ((pos "K7%" ) ))
( "photoflashes"  n ( f ou1 t ou f l a sh i z ) ((pos "Kj%" ) ))
( "photogenic"  j ( f ou2 t @ jh e1 n i k ) ((pos "OA%" ) ))
( "photograph"  v ( f ou1 t @ g r aa f ) ((pos "H0%" "K6%" ) (subcat )))
( "photographed"  v ( f ou1 t @ g r aa f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "photographer"  n ( f @1 t o1 g r @ f @ r ) ((pos "K6%" ) ))
( "photographers"  n ( f @1 t o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "photographic"  j ( f ou2 t @ g r a1 f i k ) ((pos "OA%" ) ))
( "photographically"  a ( f ou2 t @ g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "photographing"  v ( f ou1 t @ g r aa f i ng ) ((pos "Hb%" ) (subcat )))
( "photographs"  v ( f ou1 t @ g r aa f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "photography"  n ( f @1 t o1 g r @ f ii ) ((pos "L@%" ) ))
( "photogravure"  n ( f ou2 t @ g r @ v y u@1 r ) ((pos "M6%" ) ))
( "photogravures"  n ( f ou2 t @ g r @ v y u@1 z ) ((pos "Mj%" ) ))
( "photolithography"  n ( f ou2 t ou l i th o1 g r @ f ii ) ((pos "L@%" ) ))
( "photometer"  n ( f ou1 t o1 m i t @ r ) ((pos "K6$" ) ))
( "photometers"  n ( f ou1 t o1 m i t @ z ) ((pos "Kj$" ) ))
( "photon"  n ( f ou1 t o n ) ((pos "K6%" ) ))
( "photons"  n ( f ou1 t o n z ) ((pos "Kj%" ) ))
( "photos"  n ( f ou1 t ou z ) ((pos "Kj%" ) ))
( "photosensitize"  v ( f ou2 t ou s e1 n s i t ai z ) ((pos "H2%" ) ))
( "photosensitized"  v ( f ou2 t ou s e1 n s i t ai z d ) ((pos "Hc%" "Hd%" ) ))
( "photosensitizes"  v ( f ou2 t ou s e1 n s i t ai z i z ) ((pos "Ha%" ) ))
( "photosensitizing"  v ( f ou2 t ou s e1 n s i t ai z i ng ) ((pos "Hb%" ) ))
( "photostat"  v ( f ou1 t @ s t a t ) ((pos "H4%" "K6%" ) ))
( "photostats"  v ( f ou1 t @ s t a t s ) ((pos "Ha%" "Kj%" ) ))
( "photostatted"  v ( f ou1 t @ s t a t i d ) ((pos "Hc%" "Hd%" ) ))
( "photostatting"  v ( f ou1 t @ s t a t i ng ) ((pos "Hb%" ) ))
( "photosynthesis"  n ( f ou2 t ou s i1 n th @ s i s ) ((pos "L@%" ) ))
( "phrasal"  j ( f r ei1 z l ) ((pos "OA%" ) ))
( "phrase"  v ( f r ei1 z ) ((pos "H2%" "K6%" ) (subcat )))
( "phrase-book"  n ( f r ei1 z - b u k ) ((pos "K6%" ) ))
( "phrase-books"  n ( f r ei1 z - b u k s ) ((pos "Kj%" ) ))
( "phrased"  v ( f r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "phraseology"  n ( f r ei2 z i o1 l @ jh ii ) ((pos "L@%" ) ))
( "phrases"  v ( f r ei1 z i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "phrasing"  v ( f r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "phrenetic"  j ( f r @1 n e1 t i k ) ((pos "OA$" ) ))
( "phrenologist"  n ( f r i1 n o1 l @ jh i s t ) ((pos "K6%" ) ))
( "phrenologists"  n ( f r i1 n o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "phrenology"  n ( f r @1 n o1 l @ jh ii ) ((pos "L@%" ) ))
( "phthisis"  n ( th ai1 s i s ) ((pos "L@$" ) ))
( "phut"  a ( f uh1 t ) ((pos "Pu$" ) ))
( "phylum"  n ( f ai1 l @ m ) ((pos "K6$" ) ))
( "phylums"  n ( f ai1 l @ m z ) ((pos "Kj$" ) ))
( "physic"  n ( f i1 z i k ) ((pos "K6%" ) ))
( "physical"  j ( f i1 z i k l ) ((pos "OA%" ) ))
( "physically"  a ( f i1 z i k l ii ) ((pos "Pu%" ) ))
( "physician"  n ( f i1 z i1 sh @ n ) ((pos "K6%" ) ))
( "physicians"  n ( f i1 z i1 sh @ n z ) ((pos "Kj%" ) ))
( "physicist"  n ( f i1 z i s i s t ) ((pos "K6%" ) ))
( "physicists"  n ( f i1 z i s i s t s ) ((pos "Kj%" ) ))
( "physics"  n ( f i1 z i k s ) ((pos "Lk%" ) ))
( "physiognomies"  n ( f i2 z i o1 n @ m i z ) ((pos "Mj%" ) ))
( "physiognomy"  n ( f i2 z i o1 n @ m ii ) ((pos "M8%" ) ))
( "physiological"  j ( f i2 z i@ l o1 jh i k l ) ((pos "OA%" ) ))
( "physiologist"  n ( f i2 z i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "physiologists"  n ( f i2 z i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "physiology"  n ( f i2 z i o1 l @ jh ii ) ((pos "L@%" ) ))
( "physiotherapist"  n ( f i2 z i@ u th e1 r @ p i s t ) ((pos "K6%" ) ))
( "physiotherapists"  n ( f i2 z i@ u th e1 r @ p i s t s ) ((pos "Kj%" ) ))
( "physiotherapy"  n ( f i2 z i@ u th e1 r @ p ii ) ((pos "L@%" ) ))
( "physique"  n ( f i1 z ii1 k ) ((pos "M6%" ) ))
( "physiques"  n ( f i1 z ii1 k s ) ((pos "Mj%" ) ))
( "pi"  n ( p ai1 ) ((pos "K6$" ) ))
( "pianissimo"  j ( p y a1 n i1 s i m ou ) ((pos "OA%" "Pu%" ) ))
( "pianist"  n ( p i@1 n i s t ) ((pos "K6%" ) ))
( "pianists"  n ( p i@1 n i s t s ) ((pos "Kj%" ) ))
( "piano"  j ( p y aa1 n ou ) ((pos "OA$" "Pu$" ) ))
( "piano"  n ( p i1 a1 n ou ) ((pos "K6%" ) ))
( "pianoforte"  n ( p i1 a2 n ou f oo1 t ii ) ((pos "K6%" ) ))
( "pianofortes"  n ( p i1 a2 n ou f oo1 t i z ) ((pos "Kj%" ) ))
( "pianola"  n ( p i@1 n ou1 l @ ) ((pos "K6$" ) ))
( "pianolas"  n ( p i@1 n ou1 l @ z ) ((pos "Kj$" ) ))
( "pianos"  n ( p i1 a1 n ou z ) ((pos "Kj%" ) ))
( "piastre"  n ( p i1 a1 s t @ r ) ((pos "K6$" ) ))
( "piastres"  n ( p i1 a1 s t @ z ) ((pos "Kj$" ) ))
( "piazza"  n ( p i1 a1 t s @ ) ((pos "K6%" ) ))
( "piazzas"  n ( p i1 a1 t s @ z ) ((pos "Kj%" ) ))
( "pibroch"  n ( p ii1 b r o k ) ((pos "K6$" ) ))
( "pibrochs"  n ( p ii1 b r o k s ) ((pos "Kj$" ) ))
( "pica"  n ( p ai1 k @ ) ((pos "K9$" ) ))
( "picador"  n ( p i1 k @ d oo r ) ((pos "K6%" ) ))
( "picadors"  n ( p i1 k @ d oo z ) ((pos "Kj%" ) ))
( "picaresque"  j ( p i2 k @ r e1 s k ) ((pos "OA%" ) ))
( "piccalilli"  n ( p i2 k @ l i1 l ii ) ((pos "L@%" ) ))
( "piccaninnies"  n ( p i2 k @ n i1 n i z ) ((pos "Kj$" ) ))
( "piccaninny"  n ( p i2 k @ n i1 n ii ) ((pos "K8$" ) ))
( "piccolo"  n ( p i1 k @ l ou ) ((pos "K6%" ) ))
( "piccolos"  n ( p i1 k @ l ou z ) ((pos "Kj%" ) ))
( "pick"  v ( p i1 k ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" )))
( "pick-me-up"  n ( p i1 k - m i - uh p ) ((pos "K6$" ) ))
( "pick-me-ups"  n ( p i1 k - m i - uh p s ) ((pos "Kj$" ) ))
( "pick-up"  n ( p i1 k - uh p ) ((pos "K6%" ) ))
( "pick-ups"  n ( p i1 k - uh p s ) ((pos "Kj%" ) ))
( "pickaback"  a ( p i1 k @ b a k ) ((pos "Pu%" ) ))
( "pickaxe"  n ( p i1 k a k s ) ((pos "K6%" ) ))
( "pickaxes"  n ( p i1 k a k s i z ) ((pos "Kj%" ) ))
( "picked"  v ( p i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "picker"  n ( p i1 k @ r ) ((pos "K6$" ) ))
( "pickerel"  n ( p i1 k @ r @ l ) ((pos "K9$" ) ))
( "pickers"  n ( p i1 k @ z ) ((pos "Kj$" ) ))
( "picket"  v ( p i1 k i t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "picketed"  v ( p i1 k i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "picketing"  v ( p i1 k i t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pickets"  v ( p i1 k i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "picking"  v ( p i1 k i ng ) ((pos "Jb%" "M6%" ) (subcat "3A" "6A" )))
( "pickings"  n ( p i1 k i ng z ) ((pos "Mj%" ) ))
( "pickle"  v ( p i1 k l ) ((pos "H2%" "M6%" ) (subcat )))
( "pickled"  v ( p i1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pickles"  v ( p i1 k l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "pickling"  v ( p i1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "pickpocket"  n ( p i1 k p o k i t ) ((pos "K6%" ) ))
( "pickpockets"  n ( p i1 k p o k i t s ) ((pos "Kj%" ) ))
( "picks"  v ( p i1 k s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "picnic"  v ( p i1 k n i k ) ((pos "I5%" "K6%" ) ))
( "picnicked"  v ( p i1 k n i k t ) ((pos "Ic%" "Id%" ) ))
( "picnicker"  n ( p i1 k n i k @ r ) ((pos "K6%" ) ))
( "picnickers"  n ( p i1 k n i k @ z ) ((pos "Kj%" ) ))
( "picnicking"  v ( p i1 k n i k i ng ) ((pos "Ib%" ) ))
( "picnics"  v ( p i1 k n i k s ) ((pos "Ia%" "Kj%" ) ))
( "picric"  j ( p i1 k r i k ) ((pos "OA$" ) ))
( "pictorial"  n ( p i1 k t oo1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "pictorials"  n ( p i1 k t oo1 r i@ l z ) ((pos "Kj%" ) ))
( "picture"  v ( p i1 k ch @ r ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "picture-book"  n ( p i1 k ch @ - b u k ) ((pos "K6%" ) ))
( "picture-books"  n ( p i1 k ch @ - b u k s ) ((pos "Kj%" ) ))
( "picture-card"  n ( p i1 k ch @ - k aa d ) ((pos "K6%" ) ))
( "picture-cards"  n ( p i1 k ch @ - k aa d z ) ((pos "Kj%" ) ))
( "picture-galleries"  n ( p i1 k ch @ - g a l @ r i z ) ((pos "Kj%" ) ))
( "picture-gallery"  n ( p i1 k ch @ - g a l @ r ii ) ((pos "K8%" ) ))
( "pictured"  v ( p i1 k ch @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "pictures"  v ( p i1 k ch @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "picturesque"  j ( p i2 k ch @ r e1 s k ) ((pos "OA%" ) ))
( "picturesquely"  a ( p i2 k ch @ r e1 s k l ii ) ((pos "Pu%" ) ))
( "picturesqueness"  n ( p i2 k ch @ r e1 s k n @ s ) ((pos "L@%" ) ))
( "picturing"  v ( p i1 k ch @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "piddle"  v ( p i1 d l ) ((pos "I2%" "K6%" ) ))
( "piddled"  v ( p i1 d l d ) ((pos "Ic%" "Id%" ) ))
( "piddles"  v ( p i1 d l z ) ((pos "Ia%" "Kj%" ) ))
( "piddling"  v ( p i1 d l i ng ) ((pos "Ib%" "OA%" ) ))
( "pidgin"  n ( p i1 jh i n ) ((pos "K6%" ) ))
( "pidgins"  n ( p i1 jh i n z ) ((pos "Kj$" ) ))
( "pie"  n ( p ai1 ) ((pos "M6%" ) ))
( "pie-crust"  n ( p ai1 - k r uh s t ) ((pos "L@%" ) ))
( "piebald"  j ( p ai1 b oo l d ) ((pos "OA%" ) ))
( "piece"  v ( p ii1 s ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "piece-goods"  n ( p ii1 s - g u d z ) ((pos "Kj$" ) ))
( "pieced"  v ( p ii1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "piecemeal"  j ( p ii1 s m ii l ) ((pos "OA%" "Pu%" ) ))
( "pieces"  v ( p ii1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "piecework"  n ( p ii1 s w @@ k ) ((pos "L@%" ) ))
( "piecing"  v ( p ii1 s i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "pied"  j ( p ai1 d ) ((pos "OA%" ) ))
( "pied-`a-terre"  n ( p y ei2 d - aa - t e@1 r ) ((pos "K6%" ) ))
( "pied-`a-terres"  n ( p y ei2 d - aa - t e@1 z ) ((pos "Kj$" ) ))
( "pier"  n ( p i@1 r ) ((pos "K6%" ) ))
( "pier-glass"  n ( p i@1 - g l aa s ) ((pos "K7$" ) ))
( "pier-glasses"  n ( p i@1 - g l aa s i z ) ((pos "Kj$" ) ))
( "pierce"  v ( p i@1 s ) ((pos "J2%" ) (subcat "2C" )))
( "pierced"  v ( p i@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "pierces"  v ( p i@1 s i z ) ((pos "Ja%" ) (subcat "2C" )))
( "piercing"  v ( p i@1 s i ng ) ((pos "Jb%" "OA%" ) (subcat "2C" )))
( "piercingly"  a ( p i@1 s i ng l ii ) ((pos "Pu%" ) ))
( "pierrot"  n ( p i@1 r ou ) ((pos "K6%" ) ))
( "pierrots"  n ( p i@1 r ou z ) ((pos "Kj%" ) ))
( "piers"  n ( p i@1 z ) ((pos "Kj%" ) ))
( "pies"  n ( p ai1 z ) ((pos "Mj%" ) ))
( "piet`a"  n ( p ii2 e t aa1 ) ((pos "K6$" ) ))
( "piet`as"  n ( p ii2 e t aa1 z ) ((pos "Kj$" ) ))
( "pieties"  n ( p ai1 @ t i z ) ((pos "Mj%" ) ))
( "piety"  n ( p ai1 @ t ii ) ((pos "M8%" ) ))
( "piffle"  v ( p i1 f l ) ((pos "I2$" "L@%" ) ))
( "piffled"  v ( p i1 f l d ) ((pos "Ic$" "Id$" ) ))
( "piffles"  v ( p i1 f l z ) ((pos "Ia$" ) ))
( "piffling"  v ( p i1 f l i ng ) ((pos "Ib$" "OA%" ) ))
( "pig"  v ( p i1 g ) ((pos "I4$" "M6%" ) ))
( "pig-headed"  j ( p i1 g - h e1 d i d ) ((pos "OA%" ) ))
( "pig-headedly"  a ( p i1 g - h e1 d i d l ii ) ((pos "Pu%" ) ))
( "pig-headedness"  n ( p i1 g - h e1 d i d n @ s ) ((pos "L@%" ) ))
( "pig-iron"  n ( p i1 g - ai @ n ) ((pos "L@%" ) ))
( "pig-sticking"  n ( p i1 g - s t i k i ng ) ((pos "L@%" ) ))
( "pigboat"  n ( p i1 g b ou t ) ((pos "K6$" ) ))
( "pigboats"  n ( p i1 g b ou t s ) ((pos "Kj$" ) ))
( "pigeon"  n ( p i1 jh @ n ) ((pos "K6%" ) ))
( "pigeon-breasted"  j ( p i2 jh i n - b r e1 s t i d ) ((pos "OA%" ) ))
( "pigeon-toed"  j ( p i1 jh i n - t ou d ) ((pos "OA%" ) ))
( "pigeonhole"  v ( p i1 jh @ n h ou l ) ((pos "H2%" "K6%" ) ))
( "pigeonholed"  v ( p i1 jh @ n h ou l d ) ((pos "Hc%" "Hd%" ) ))
( "pigeonholes"  v ( p i1 jh @ n h ou l z ) ((pos "Ha%" "Kj%" ) ))
( "pigeonholing"  v ( p i1 jh @ n h ou l i ng ) ((pos "Hb%" ) ))
( "pigeons"  n ( p i1 jh @ n z ) ((pos "Kj%" ) ))
( "pigged"  v ( p i1 g d ) ((pos "Ic$" "Id$" ) ))
( "piggeries"  n ( p i1 g @ r i z ) ((pos "Kj%" ) ))
( "piggery"  n ( p i1 g @ r ii ) ((pos "K8%" ) ))
( "piggier"  j ( p i1 g i@ r ) ((pos "Or$" ) ))
( "piggies"  n ( p i1 g i z ) ((pos "Kj%" ) ))
( "piggiest"  j ( p i1 g i i s t ) ((pos "Os$" ) ))
( "pigging"  v ( p i1 g i ng ) ((pos "Ib$" ) ))
( "piggish"  j ( p i1 g i sh ) ((pos "OA%" ) ))
( "piggishly"  a ( p i1 g i sh l ii ) ((pos "Pu%" ) ))
( "piggishness"  n ( p i1 g i sh n @ s ) ((pos "L@%" ) ))
( "piggy"  n ( p i1 g ii ) ((pos "K8%" "OD%" ) ))
( "piggyback"  n ( p i1 g i b a k ) ((pos "K6%" ) ))
( "piggybacks"  n ( p i1 g i b a k s ) ((pos "Kj%" ) ))
( "piglet"  n ( p i1 g l i t ) ((pos "K6%" ) ))
( "piglets"  n ( p i1 g l i t s ) ((pos "Kj%" ) ))
( "pigment"  n ( p i1 g m @ n t ) ((pos "M6%" ) ))
( "pigmentation"  n ( p i2 g m e n t ei1 sh @ n ) ((pos "K6%" ) ))
( "pigmentations"  n ( p i2 g m e n t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "pigments"  n ( p i1 g m @ n t s ) ((pos "Mj%" ) ))
( "pigmies"  n ( p i1 g m i z ) ((pos "Kj%" ) ))
( "pigmy"  n ( p i1 g m ii ) ((pos "K8%" ) ))
( "pigs"  v ( p i1 g z ) ((pos "Ia%" "Mj%" ) ))
( "pigskin"  n ( p i1 g s k i n ) ((pos "L@%" ) ))
( "pigsties"  n ( p i1 g s t ai z ) ((pos "Kj%" ) ))
( "pigsty"  n ( p i1 g s t ai ) ((pos "K8%" ) ))
( "pigswill"  n ( p i1 g s w i l ) ((pos "L@%" ) ))
( "pigtail"  n ( p i1 g t ei l ) ((pos "K6%" ) ))
( "pigtails"  n ( p i1 g t ei l z ) ((pos "Kj%" ) ))
( "pigwash"  n ( p i1 g w o sh ) ((pos "L@$" ) ))
( "pike"  n ( p ai1 k ) ((pos "K9%" ) ))
( "pikes"  n ( p ai1 k s ) ((pos "Kj%" ) ))
( "pikestaff"  n ( p ai1 k s t aa f ) ((pos "K6%" ) ))
( "pikestaffs"  n ( p ai1 k s t aa f s ) ((pos "Kj%" ) ))
( "pilaf"  n ( p i1 l a1 f ) ((pos "L@%" ) ))
( "pilaff"  n ( p i1 l a1 f ) ((pos "L@%" ) ))
( "pilaster"  n ( p i1 l a1 s t @ r ) ((pos "K6$" ) ))
( "pilasters"  n ( p i1 l a1 s t @ z ) ((pos "Kj$" ) ))
( "pilau"  n ( p i1 l au1 ) ((pos "L@%" ) ))
( "pilchard"  n ( p i1 l ch @ d ) ((pos "M6%" ) ))
( "pilchards"  n ( p i1 l ch @ d z ) ((pos "Mj%" ) ))
( "pile"  v ( p ai1 l ) ((pos "J2%" "M6%" ) (subcat "2C" "6A" "15A" )))
( "pile-driver"  n ( p ai1 l - d r ai v @ r ) ((pos "K6%" ) ))
( "pile-drivers"  n ( p ai1 l - d r ai v @ z ) ((pos "Kj%" ) ))
( "pile-dwelling"  n ( p ai1 l - d w e l i ng ) ((pos "K6$" ) ))
( "pile-dwellings"  n ( p ai1 l - d w e l i ng z ) ((pos "Kj$" ) ))
( "pile-up"  n ( p ai1 l - uh p ) ((pos "K6%" ) ))
( "pile-ups"  n ( p ai1 l - uh p s ) ((pos "Kj%" ) ))
( "piled"  v ( p ai1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "piles"  v ( p ai1 l z ) ((pos "Ja%" "Lk%" "Mj%" ) (subcat "2C" "6A" "15A" )))
( "pilfer"  v ( p i1 l f @ r ) ((pos "J0%" ) (subcat "2A" )))
( "pilferage"  n ( p i1 l f @ r i jh ) ((pos "L@$" ) ))
( "pilfered"  v ( p i1 l f @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pilferer"  n ( p i1 l f @ r @ r ) ((pos "K6%" ) ))
( "pilferers"  n ( p i1 l f @ r @ z ) ((pos "Kj%" ) ))
( "pilfering"  v ( p i1 l f @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pilfers"  v ( p i1 l f @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "pilgrim"  n ( p i1 l g r i m ) ((pos "K6%" ) ))
( "pilgrimage"  n ( p i1 l g r i m i jh ) ((pos "K6%" ) ))
( "pilgrimages"  n ( p i1 l g r i m i jh i z ) ((pos "Kj%" ) ))
( "pilgrims"  n ( p i1 l g r i m z ) ((pos "Kj%" ) ))
( "piling"  v ( p ai1 l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "pill"  n ( p i1 l ) ((pos "K6%" ) ))
( "pillage"  v ( p i1 l i jh ) ((pos "H2%" "K6%" ) ))
( "pillaged"  v ( p i1 l i jh d ) ((pos "Hc%" "Hd%" ) ))
( "pillager"  n ( p i1 l i jh @ r ) ((pos "K6$" ) ))
( "pillagers"  n ( p i1 l i jh @ z ) ((pos "Kj$" ) ))
( "pillages"  v ( p i1 l i jh i z ) ((pos "Ha%" "Kj%" ) ))
( "pillaging"  v ( p i1 l i jh i ng ) ((pos "Hb%" ) ))
( "pillar"  n ( p i1 l @ r ) ((pos "K6%" ) ))
( "pillar-box"  n ( p i1 l @ - b o k s ) ((pos "K7%" ) ))
( "pillar-boxes"  n ( p i1 l @ - b o k s i z ) ((pos "Kj%" ) ))
( "pillared"  j ( p i1 l @ d ) ((pos "OA%" ) ))
( "pillars"  n ( p i1 l @ z ) ((pos "Kj%" ) ))
( "pillbox"  n ( p i1 l b o k s ) ((pos "K7%" ) ))
( "pillboxes"  n ( p i1 l b o k s i z ) ((pos "Kj%" ) ))
( "pillion"  n ( p i1 l i@ n ) ((pos "K6%" ) ))
( "pillions"  n ( p i1 l i@ n z ) ((pos "Kj%" ) ))
( "pillories"  n ( p i1 l @ r i z ) ((pos "Kj$" ) ))
( "pillory"  n ( p i1 l @ r ii ) ((pos "K8$" ) ))
( "pillow"  v ( p i1 l ou ) ((pos "H0%" "K6%" ) (subcat )))
( "pillow-fight"  n ( p i1 l ou - f ai t ) ((pos "K6%" ) ))
( "pillow-fights"  n ( p i1 l ou - f ai t s ) ((pos "Kj%" ) ))
( "pillowcase"  n ( p i1 l ou k ei s ) ((pos "K6%" ) ))
( "pillowcases"  n ( p i1 l ou k ei s i z ) ((pos "Kj%" ) ))
( "pillowed"  v ( p i1 l ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pillowing"  v ( p i1 l ou i ng ) ((pos "Hb$" ) (subcat )))
( "pillows"  v ( p i1 l ou z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "pillowslip"  n ( p i1 l ou s l i p ) ((pos "K6%" ) ))
( "pillowslips"  n ( p i1 l ou s l i p s ) ((pos "Kj%" ) ))
( "pills"  n ( p i1 l z ) ((pos "Kj%" ) ))
( "pilot"  v ( p ai1 l @ t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "pilot-boat"  n ( p ai1 l @ t - b ou t ) ((pos "K6%" ) ))
( "pilot-boats"  n ( p ai1 l @ t - b ou t s ) ((pos "Kj%" ) ))
( "pilot-burner"  n ( p ai1 l @ t - b @@ n @ r ) ((pos "K6$" ) ))
( "pilot-burners"  n ( p ai1 l @ t - b @@ n @ z ) ((pos "Kj$" ) ))
( "pilot-cloth"  n ( p ai1 l @ t - k l o th ) ((pos "L@$" ) ))
( "pilot-engine"  n ( p ai1 l @ t - e n jh i n ) ((pos "K6$" ) ))
( "pilot-engines"  n ( p ai1 l @ t - e n jh i n z ) ((pos "Kj$" ) ))
( "pilot-fish"  n ( p ai1 l @ t - f i sh ) ((pos "K9%" ) ))
( "pilot-fishes"  n ( p ai1 l @ t - f i sh i z ) ((pos "Kj%" ) ))
( "pilot-light"  n ( p ai1 l @ t - l ai t ) ((pos "K6%" ) ))
( "pilot-lights"  n ( p ai1 l @ t - l ai t s ) ((pos "Kj%" ) ))
( "piloted"  v ( p ai1 l @ t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "piloting"  v ( p ai1 l @ t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pilots"  v ( p ai1 l @ t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "pimento"  n ( p i1 m e1 n t ou ) ((pos "M6%" ) ))
( "pimentos"  n ( p i1 m e1 n t ou z ) ((pos "Mj%" ) ))
( "pimp"  v ( p i1 m p ) ((pos "I0$" "K6%" ) (subcat "2A" )))
( "pimped"  v ( p i1 m p t ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "pimpernel"  n ( p i1 m p @ n e l ) ((pos "K6$" ) ))
( "pimpernels"  n ( p i1 m p @ n e l z ) ((pos "Kj$" ) ))
( "pimping"  v ( p i1 m p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "pimple"  n ( p i1 m p l ) ((pos "K6%" ) ))
( "pimpled"  j ( p i1 m p l d ) ((pos "OA%" ) ))
( "pimples"  n ( p i1 m p l z ) ((pos "Kj%" ) ))
( "pimplier"  j ( p i1 m p l i@ r ) ((pos "Or%" ) ))
( "pimpliest"  j ( p i1 m p l i i s t ) ((pos "Os%" ) ))
( "pimply"  j ( p i1 m p l ii ) ((pos "OD%" ) ))
( "pimps"  v ( p i1 m p s ) ((pos "Ia$" "Kj%" ) (subcat "2A" )))
( "pin"  v ( p i1 n ) ((pos "H4%" "K6%" ) (subcat "15A" )))
( "pin-money"  n ( p i1 n - m uh n ii ) ((pos "L@%" ) ))
( "pin-table"  n ( p i1 n - t ei b l ) ((pos "K6%" ) ))
( "pin-tables"  n ( p i1 n - t ei b l z ) ((pos "Kj%" ) ))
( "pin-up"  n ( p i1 n - uh p ) ((pos "K6%" ) ))
( "pin-ups"  n ( p i1 n - uh p s ) ((pos "Kj%" ) ))
( "pinafore"  n ( p i1 n @ f oo r ) ((pos "K6%" ) ))
( "pinafores"  n ( p i1 n @ f oo z ) ((pos "Kj%" ) ))
( "pinball"  n ( p i1 n b oo l ) ((pos "L@%" ) ))
( "pince-nez"  n ( p a1 n s - n ei ) ((pos "K9$" ) ))
( "pincer"  n ( p i1 n s @ r ) ((pos "K6%" ) ))
( "pincers"  n ( p i1 n s @ z ) ((pos "Kj%" ) ))
( "pinch"  v ( p i1 n ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" "15A" )))
( "pinchbeck"  n ( p i1 n ch b e k ) ((pos "K6$" "OA$" ) ))
( "pinchbecks"  n ( p i1 n ch b e k s ) ((pos "Kj$" ) ))
( "pinched"  v ( p i1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "pinches"  v ( p i1 n ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "pinching"  v ( p i1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "pincushion"  n ( p i1 n k u sh @ n ) ((pos "K6%" ) ))
( "pincushions"  n ( p i1 n k u sh @ n z ) ((pos "Kj%" ) ))
( "pine"  v ( p ai1 n ) ((pos "I2%" "M6%" ) (subcat "2A" "2C" "3A" )))
( "pineal"  j ( p ai1 n i@ l ) ((pos "OA$" ) ))
( "pineapple"  n ( p ai1 n a p l ) ((pos "M6%" ) ))
( "pineapples"  n ( p ai1 n a p l z ) ((pos "Mj%" ) ))
( "pined"  v ( p ai1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "3A" )))
( "pines"  v ( p ai1 n z ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2C" "3A" )))
( "ping"  v ( p i1 ng ) ((pos "I0%" "K6%" ) ))
( "pinged"  v ( p i1 ng d ) ((pos "Ic%" "Id%" ) ))
( "pinging"  v ( p i1 ng i ng ) ((pos "Ib%" ) ))
( "pingpong"  n ( p i1 ng p o ng ) ((pos "L@%" ) ))
( "pings"  v ( p i1 ng z ) ((pos "Ia%" "Kj%" ) ))
( "pinhead"  n ( p i1 n h e d ) ((pos "K6%" ) ))
( "pinheads"  n ( p i1 n h e d z ) ((pos "Kj%" ) ))
( "pining"  v ( p ai1 n i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "3A" )))
( "pinion"  v ( p i1 n i@ n ) ((pos "H0$" "K6%" ) (subcat "6A" "15A" )))
( "pinioned"  v ( p i1 n i@ n d ) ((pos "Hc$" "Hd%" ) (subcat "6A" "15A" )))
( "pinioning"  v ( p i1 n i@ n i ng ) ((pos "Hb$" ) (subcat "6A" "15A" )))
( "pinions"  v ( p i1 n i@ n z ) ((pos "Ha$" "Kj$" ) (subcat "6A" "15A" )))
( "pink"  v ( p i1 ng k ) ((pos "J0$" "M6%" "OC%" ) (subcat "6A" )))
( "pinked"  v ( p i1 ng k t ) ((pos "Jc$" "Jd$" ) (subcat "6A" )))
( "pinker"  j ( p i1 ng k @ r ) ((pos "Or%" ) ))
( "pinkest"  j ( p i1 ng k i s t ) ((pos "Os%" ) ))
( "pinking"  v ( p i1 ng k i ng ) ((pos "Jb$" ) (subcat "6A" )))
( "pinkish"  j ( p i1 ng k i sh ) ((pos "OA%" ) ))
( "pinks"  v ( p i1 ng k s ) ((pos "Ja$" "Mj%" ) (subcat "6A" )))
( "pinnace"  n ( p i1 n i s ) ((pos "K6$" ) ))
( "pinnaces"  n ( p i1 n i s i z ) ((pos "Kj$" ) ))
( "pinnacle"  v ( p i1 n @ k l ) ((pos "H2%" "K6%" ) ))
( "pinnacled"  v ( p i1 n @ k l d ) ((pos "Hc%" "Hd%" ) ))
( "pinnacles"  v ( p i1 n @ k l z ) ((pos "Ha%" "Kj%" ) ))
( "pinnacling"  v ( p i1 n @ k l i ng ) ((pos "Hb$" ) ))
( "pinnate"  j ( p i1 n ei t ) ((pos "OA$" ) ))
( "pinned"  v ( p i1 n d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "pinnies"  n ( p i1 n i z ) ((pos "Kj%" ) ))
( "pinning"  v ( p i1 n i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "pinny"  n ( p i1 n ii ) ((pos "K8%" ) ))
( "pinpoint"  v ( p i1 n p oi n t ) ((pos "H0%" "K6%" ) ))
( "pinpointed"  v ( p i1 n p oi n t i d ) ((pos "Hc%" "Hd%" ) ))
( "pinpointing"  v ( p i1 n p oi n t i ng ) ((pos "Hb%" ) ))
( "pinpoints"  v ( p i1 n p oi n t s ) ((pos "Ha%" "Kj%" ) ))
( "pinprick"  n ( p i1 n p r i k ) ((pos "K6%" ) ))
( "pinpricks"  n ( p i1 n p r i k s ) ((pos "Kj%" ) ))
( "pins"  v ( p i1 n z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "pinstripe"  j ( p i1 n s t r ai p ) ((pos "OA%" ) ))
( "pint"  n ( p ai1 n t ) ((pos "K6%" ) ))
( "pints"  n ( p ai1 n t s ) ((pos "Kj%" ) ))
( "pioneer"  v ( p ai2 @ n i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "pioneered"  v ( p ai2 @ n i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pioneering"  v ( p ai2 @ n i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pioneers"  v ( p ai2 @ n i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "pious"  j ( p ai1 @ s ) ((pos "OA%" ) ))
( "piously"  a ( p ai1 @ s l ii ) ((pos "Pu%" ) ))
( "pip"  v ( p i1 p ) ((pos "H4%" "K6%" ) (subcat )))
( "pipal"  n ( p ii1 p @ l ) ((pos "K6$" ) ))
( "pipals"  n ( p ii1 p @ l z ) ((pos "Kj$" ) ))
( "pipe"  v ( p ai1 p ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "pipe-organ"  n ( p ai1 p - oo g @ n ) ((pos "K6%" ) ))
( "pipe-organs"  n ( p ai1 p - oo g @ n z ) ((pos "Kj%" ) ))
( "pipe-rack"  n ( p ai1 p - r a k ) ((pos "K6%" ) ))
( "pipe-racks"  n ( p ai1 p - r a k s ) ((pos "Kj%" ) ))
( "pipeclay"  n ( p ai1 p k l ei ) ((pos "L@%" ) ))
( "piped"  v ( p ai1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "pipedream"  n ( p ai1 p d r ii m ) ((pos "K6%" ) ))
( "pipedreams"  n ( p ai1 p d r ii m z ) ((pos "Kj%" ) ))
( "pipeful"  n ( p ai1 p f u l ) ((pos "K6%" ) ))
( "pipefuls"  n ( p ai1 p f u l z ) ((pos "Kj%" ) ))
( "pipeline"  n ( p ai1 p l ai n ) ((pos "K6%" ) ))
( "pipelines"  n ( p ai1 p l ai n z ) ((pos "Kj%" ) ))
( "piper"  n ( p ai1 p @ r ) ((pos "K6%" ) ))
( "pipers"  n ( p ai1 p @ z ) ((pos "Kj%" ) ))
( "pipes"  v ( p ai1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "pipette"  n ( p i1 p e1 t ) ((pos "K6%" ) ))
( "pipettes"  n ( p i1 p e1 t s ) ((pos "Kj%" ) ))
( "pipework"  n ( p ai1 p w @@ k ) ((pos "L@%" ) ))
( "piping"  v ( p ai1 p i ng ) ((pos "Jb%" "L@%" "OA%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "pipped"  v ( p i1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pippin"  n ( p i1 p i n ) ((pos "K6%" ) ))
( "pipping"  v ( p i1 p i ng ) ((pos "Hb%" ) (subcat )))
( "pippins"  n ( p i1 p i n z ) ((pos "Kj%" ) ))
( "pips"  v ( p i1 p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "pipsqueak"  n ( p i1 p s k w ii k ) ((pos "K6%" ) ))
( "pipsqueaks"  n ( p i1 p s k w ii k s ) ((pos "Kj%" ) ))
( "piquancy"  n ( p ii1 k @ n s ii ) ((pos "L@%" ) ))
( "piquant"  j ( p ii1 k @ n t ) ((pos "OA%" ) ))
( "piquantly"  a ( p ii1 k @ n t l ii ) ((pos "Pu%" ) ))
( "pique"  v ( p ii1 k ) ((pos "H2%" "M6%" ) (subcat )))
( "piqued"  v ( p ii1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "piques"  v ( p ii1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "piquet"  n ( p i1 k e1 t ) ((pos "L@$" ) ))
( "piquing"  v ( p ii1 k i ng ) ((pos "Hb$" ) (subcat )))
( "piracies"  n ( p ai1 @ r @ s i z ) ((pos "Mj$" ) ))
( "piracy"  n ( p ai1 @ r @ s ii ) ((pos "M8%" ) ))
( "piranha"  n ( p i1 r aa1 n y @ ) ((pos "K6%" ) ))
( "piranhas"  n ( p i1 r aa1 n y @ z ) ((pos "Kj%" ) ))
( "pirate"  v ( p ai1 r @ t ) ((pos "H2%" "K6%" ) (subcat )))
( "pirated"  v ( p ai1 r @ t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pirates"  v ( p ai1 r @ t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "piratical"  j ( p ai2 @ r a1 t i k l ) ((pos "OA%" ) ))
( "piratically"  a ( p ai2 @ r a1 t i k l ii ) ((pos "Pu%" ) ))
( "pirating"  v ( p ai1 r @ t i ng ) ((pos "Hb%" ) (subcat )))
( "pirouette"  v ( p i2 r u e1 t ) ((pos "I2%" "K6%" ) ))
( "pirouetted"  v ( p i2 r u e1 t i d ) ((pos "Ic%" "Id%" ) ))
( "pirouettes"  v ( p i2 r u e1 t s ) ((pos "Ia%" "Kj%" ) ))
( "pirouetting"  v ( p i2 r u e1 t i ng ) ((pos "Ib%" ) ))
( "pis"  n ( p ai1 z ) ((pos "Kj$" ) ))
( "pis aller"  n ( p ii2 z - a1 l ei ) ((pos "Ki$" ) ))
( "piscatorial"  j ( p i2 s k @ t oo1 r i@ l ) ((pos "OA$" ) ))
( "piss"  v ( p i1 s ) ((pos "J1%" "K7%" ) ))
( "pissed"  v ( p i1 s t ) ((pos "Jc%" "Jd%" "OA%" ) ))
( "pisses"  v ( p i1 s i z ) ((pos "Ja%" "Kj%" ) ))
( "pissing"  v ( p i1 s i ng ) ((pos "Jb%" ) ))
( "pistachio"  n ( p i1 s t aa1 ch i@ @ ) ((pos "K6%" ) ))
( "pistachios"  n ( p i1 s t aa1 ch i@ u z ) ((pos "Kj%" ) ))
( "pistil"  n ( p i1 s t l ) ((pos "K6$" ) ))
( "pistils"  n ( p i1 s t l z ) ((pos "Kj$" ) ))
( "pistol"  n ( p i1 s t l ) ((pos "K6%" ) ))
( "pistols"  n ( p i1 s t l z ) ((pos "Kj%" ) ))
( "piston"  n ( p i1 s t @ n ) ((pos "K6%" ) ))
( "pistons"  n ( p i1 s t @ n z ) ((pos "Kj%" ) ))
( "pit"  v ( p i1 t ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "pit-a-pat"  a ( p i2 t - @ - p a1 t ) ((pos "Pu%" ) ))
( "pit-prop"  n ( p i1 t - p r o p ) ((pos "K6%" ) ))
( "pit-props"  n ( p i1 t - p r o p s ) ((pos "Kj%" ) ))
( "pitch"  v ( p i1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pitch-black"  j ( p i1 ch - b l a k ) ((pos "OA%" ) ))
( "pitch-dark"  j ( p i1 ch - d aa k ) ((pos "OA%" ) ))
( "pitchblende"  n ( p i1 ch b l e n d ) ((pos "L@$" ) ))
( "pitched"  v ( p i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pitcher"  n ( p i1 ch @ r ) ((pos "K6%" ) ))
( "pitchers"  n ( p i1 ch @ z ) ((pos "Kj%" ) ))
( "pitches"  v ( p i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pitchfork"  v ( p i1 ch f oo k ) ((pos "H0%" "K6%" ) ))
( "pitchforked"  v ( p i1 ch f oo k t ) ((pos "Hc%" "Hd%" ) ))
( "pitchforking"  v ( p i1 ch f oo k i ng ) ((pos "Hb%" ) ))
( "pitchforks"  v ( p i1 ch f oo k s ) ((pos "Ha%" "Kj%" ) ))
( "pitching"  v ( p i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "piteous"  j ( p i1 t i@ s ) ((pos "OA%" ) ))
( "piteously"  a ( p i1 t i@ s l ii ) ((pos "Pu%" ) ))
( "pitfall"  n ( p i1 t f oo l ) ((pos "K6%" ) ))
( "pitfalls"  n ( p i1 t f oo l z ) ((pos "Kj%" ) ))
( "pith"  n ( p i1 th ) ((pos "L@%" ) ))
( "pithead"  n ( p i1 t h e d ) ((pos "K6%" ) ))
( "pitheads"  n ( p i1 t h e d z ) ((pos "Kj$" ) ))
( "pithier"  j ( p i1 th i@ r ) ((pos "Or$" ) ))
( "pithiest"  j ( p i1 th i i s t ) ((pos "Os$" ) ))
( "pithily"  a ( p i1 th i l ii ) ((pos "Pu%" ) ))
( "pithy"  j ( p i1 th ii ) ((pos "OD%" ) ))
( "pitiable"  j ( p i1 t i@ b l ) ((pos "OA%" ) ))
( "pitiably"  a ( p i1 t i@ b l ii ) ((pos "Pu%" ) ))
( "pitied"  v ( p i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pities"  v ( p i1 t i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "pitiful"  j ( p i1 t i f @ l ) ((pos "OA%" ) ))
( "pitifully"  a ( p i1 t i f @ l ii ) ((pos "Pu%" ) ))
( "pitiless"  j ( p i1 t i l i s ) ((pos "OA%" ) ))
( "pitilessly"  a ( p i1 t i l i s l ii ) ((pos "Pu%" ) ))
( "pitman"  n ( p i1 t m a n ) ((pos "Ki%" ) ))
( "pitmen"  n ( p i1 t m e n ) ((pos "Kj%" ) ))
( "piton"  n ( p ii1 t o n ) ((pos "K6$" ) ))
( "pitons"  n ( p ii1 t o n z ) ((pos "Kj$" ) ))
( "pits"  v ( p i1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "pitsaw"  n ( p i1 t s oo ) ((pos "K6$" ) ))
( "pitsaws"  n ( p i1 t s oo z ) ((pos "Kj$" ) ))
( "pittance"  n ( p i1 t n s ) ((pos "K6%" ) ))
( "pittances"  n ( p i1 t n s i z ) ((pos "Kj$" ) ))
( "pitted"  v ( p i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "pitter-patter"  n ( p i1 t @ - p a t @ r ) ((pos "L@%" ) ))
( "pitting"  v ( p i1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pituitaries"  n ( p i1 t y uu1 i t @ r i z ) ((pos "Kj$" ) ))
( "pituitary"  n ( p i1 t y uu1 i t @ r ii ) ((pos "K8%" "OA%" ) ))
( "pity"  v ( p i1 t ii ) ((pos "H3%" "M8%" ) (subcat )))
( "pitying"  v ( p i1 t i i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "pityingly"  a ( p i1 t i i ng l ii ) ((pos "Pu%" ) ))
( "pivot"  v ( p i1 v @ t ) ((pos "J0%" "K6%" ) (subcat "3A" )))
( "pivotal"  j ( p i1 v @ t @ l ) ((pos "OA%" ) ))
( "pivoted"  v ( p i1 v @ t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "pivoting"  v ( p i1 v @ t i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "pivots"  v ( p i1 v @ t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "pixie"  n ( p i1 k s ii ) ((pos "K6%" ) ))
( "pixies"  n ( p i1 k s i z ) ((pos "Kj%" ) ))
( "pixilated"  j ( p i1 k s i l ei t i d ) ((pos "OA%" ) ))
( "pixy"  n ( p i1 k s ii ) ((pos "K8%" ) ))
( "pizza"  n ( p ii1 t s @ ) ((pos "K6%" ) ))
( "pizzas"  n ( p ii1 t s @ z ) ((pos "Kj%" ) ))
( "pizzicato"  j ( p i2 t s i k aa1 t ou ) ((pos "OA%" "Pu%" ) ))
( "placard"  v ( p l a1 k aa d ) ((pos "H0$" "K6%" ) (subcat )))
( "placarded"  v ( p l a1 k aa d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "placarding"  v ( p l a1 k aa d i ng ) ((pos "Hb$" ) (subcat )))
( "placards"  v ( p l a1 k aa d z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "placate"  v ( p l @1 k ei1 t ) ((pos "H2%" ) (subcat )))
( "placated"  v ( p l @1 k ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "placates"  v ( p l @1 k ei1 t s ) ((pos "Ha%" ) (subcat )))
( "placating"  v ( p l @1 k ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "placatingly"  a ( p l @1 k ei1 t i ng l ii ) ((pos "Pu%" ) ))
( "place"  v ( p l ei1 s ) ((pos "H2%" "K6*" ) (subcat "6A" "15A" )))
( "place-bet"  n ( p l ei1 s - b e t ) ((pos "K6$" ) ))
( "place-bets"  n ( p l ei1 s - b e t s ) ((pos "Kj$" ) ))
( "place-name"  n ( p l ei1 s - n ei m ) ((pos "K6%" ) ))
( "place-names"  n ( p l ei1 s - n ei m z ) ((pos "Kj%" ) ))
( "placebo"  n ( p l @1 s ii1 b ou ) ((pos "K6%" ) ))
( "placebos"  n ( p l @1 s ii1 b ou z ) ((pos "Kj%" ) ))
( "placed"  v ( p l ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "placeman"  n ( p l ei1 s m @ n ) ((pos "Ki$" ) ))
( "placemen"  n ( p l ei1 s m @ n ) ((pos "Kj$" ) ))
( "placement"  n ( p l ei1 s m @ n t ) ((pos "K6%" ) ))
( "placements"  n ( p l ei1 s m @ n t s ) ((pos "Kj%" ) ))
( "placenta"  n ( p l @1 s e1 n t @ ) ((pos "K6%" ) ))
( "placentas"  n ( p l @1 s e1 n t @ z ) ((pos "Kj%" ) ))
( "places"  v ( p l ei1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "placeseeker"  n ( p l ei1 s - s ii k @ r ) ((pos "K6$" ) ))
( "placeseekers"  n ( p l ei1 s - s ii k @ z ) ((pos "Kj$" ) ))
( "placid"  j ( p l a1 s i d ) ((pos "OA%" ) ))
( "placidity"  n ( p l @1 s i1 d i t ii ) ((pos "L@%" ) ))
( "placidly"  a ( p l a1 s i d l ii ) ((pos "Pu%" ) ))
( "placing"  v ( p l ei1 s i ng ) ((pos "Hb%" "K6$" ) (subcat "6A" "15A" )))
( "placings"  n ( p l ei1 s i ng z ) ((pos "Kj%" ) ))
( "placket"  n ( p l a1 k i t ) ((pos "K6$" ) ))
( "plackets"  n ( p l a1 k i t s ) ((pos "Kj$" ) ))
( "plage"  n ( p l aa1 zh ) ((pos "Ki$" ) ))
( "plagiarism"  n ( p l ei1 jh i@ r i z @ m ) ((pos "M6%" ) ))
( "plagiarisms"  n ( p l ei1 jh i@ r i z @ m z ) ((pos "Mj$" ) ))
( "plagiarist"  n ( p l ei1 jh i@ r i s t ) ((pos "K6%" ) ))
( "plagiarists"  n ( p l ei1 jh i@ r i s t s ) ((pos "Kj%" ) ))
( "plagiarize"  v ( p l ei1 jh @ r ai z ) ((pos "H2%" ) (subcat )))
( "plagiarized"  v ( p l ei1 jh @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "plagiarizes"  v ( p l ei1 jh @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "plagiarizing"  v ( p l ei1 jh @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "plague"  v ( p l ei1 g ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "plague-spot"  n ( p l ei1 g - s p o t ) ((pos "K6%" ) ))
( "plague-spots"  n ( p l ei1 g - s p o t s ) ((pos "Kj%" ) ))
( "plagued"  v ( p l ei1 g d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "plagues"  v ( p l ei1 g z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "plaguily"  a ( p l ei1 g i l ii ) ((pos "Pu$" ) ))
( "plaguing"  v ( p l ei1 g i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "plaguy"  j ( p l ei1 g ii ) ((pos "OA$" ) ))
( "plaice"  n ( p l ei1 s ) ((pos "M9%" ) ))
( "plaid"  n ( p l a1 d ) ((pos "M6%" ) ))
( "plaids"  n ( p l a1 d z ) ((pos "Mj$" ) ))
( "plain"  v ( p l ei1 n ) ((pos "J0$" "K6%" "OC%" "Pu%" ) ))
( "plain-spoken"  j ( p l ei1 n - s p ou k @ n ) ((pos "OA%" ) ))
( "plained"  v ( p l ei1 n d ) ((pos "Jc$" "Jd$" ) ))
( "plainer"  j ( p l ei1 n @ r ) ((pos "Or%" ) ))
( "plainest"  j ( p l ei1 n i s t ) ((pos "Os%" ) ))
( "plaining"  v ( p l ei1 n i ng ) ((pos "Jb$" ) ))
( "plainly"  a ( p l ei1 n l ii ) ((pos "Pu%" ) ))
( "plainness"  n ( p l ei1 n - n @ s ) ((pos "L@%" ) ))
( "plains"  v ( p l ei1 n z ) ((pos "Ja$" "Kj%" ) ))
( "plainsman"  n ( p l ei1 n z m @ n ) ((pos "Ki%" ) ))
( "plainsmen"  n ( p l ei1 n z m @ n ) ((pos "Kj%" ) ))
( "plaint"  n ( p l ei1 n t ) ((pos "K6%" ) ))
( "plaintiff"  n ( p l ei1 n t i f ) ((pos "K6%" ) ))
( "plaintiffs"  n ( p l ei1 n t i f s ) ((pos "Kj%" ) ))
( "plaintive"  j ( p l ei1 n t i v ) ((pos "OA%" ) ))
( "plaintively"  a ( p l ei1 n t i v l ii ) ((pos "Pu%" ) ))
( "plaintiveness"  n ( p l ei1 n t i v n @ s ) ((pos "L@%" ) ))
( "plaints"  n ( p l ei1 n t s ) ((pos "Kj$" ) ))
( "plait"  v ( p l a1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "plaited"  v ( p l a1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "plaiting"  v ( p l a1 t i ng ) ((pos "Hb%" ) (subcat )))
( "plaits"  v ( p l a1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "plan"  v ( p l a1 n ) ((pos "H4%" "K6%" ) (subcat "6A" "7A" )))
( "planchette"  n ( p l aa1 n sh e1 t ) ((pos "K6$" ) ))
( "planchettes"  n ( p l aa1 n sh e1 t s ) ((pos "Kj$" ) ))
( "plane"  v ( p l ei1 n ) ((pos "J2%" "K6%" ) (subcat "2A" "2D" "15B" )))
( "plane-tree"  n ( p l ei1 n - t r ii ) ((pos "K6%" ) ))
( "plane-trees"  n ( p l ei1 n - t r ii z ) ((pos "Kj%" ) ))
( "planed"  v ( p l ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2D" "15B" )))
( "planes"  v ( p l ei1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2D" "15B" )))
( "planet"  n ( p l a1 n i t ) ((pos "K6%" ) ))
( "planetarium"  n ( p l a2 n i t e@1 r i@ m ) ((pos "K6%" ) ))
( "planetariums"  n ( p l a2 n i t e@1 r i@ m z ) ((pos "Kj%" ) ))
( "planetary"  j ( p l a1 n i t r ii ) ((pos "OA%" ) ))
( "planets"  n ( p l a1 n i t s ) ((pos "Kj%" ) ))
( "plangent"  j ( p l a1 n jh @ n t ) ((pos "OA$" ) ))
( "planing"  v ( p l ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2D" "15B" )))
( "plank"  v ( p l a1 ng k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "plank-bed"  n ( p l a1 ng k - b e d ) ((pos "K6$" ) ))
( "plank-beds"  n ( p l a1 ng k - b e d z ) ((pos "Kj$" ) ))
( "planked"  v ( p l a1 ng k t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "planking"  v ( p l a1 ng k i ng ) ((pos "Hb$" "L@%" ) (subcat "6A" )))
( "planks"  v ( p l a1 ng k s ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "plankton"  n ( p l a1 ng k t @ n ) ((pos "L@%" ) ))
( "planless"  j ( p l a1 n l @ s ) ((pos "OA$" ) ))
( "planned"  v ( p l a1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "planner"  n ( p l a1 n @ r ) ((pos "K6%" ) ))
( "planners"  n ( p l a1 n @ z ) ((pos "Kj%" ) ))
( "planning"  v ( p l a1 n i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "plans"  v ( p l a1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "7A" )))
( "plant"  v ( p l aa1 n t ) ((pos "H0%" "M6%" ) (subcat "6A" "15A" )))
( "plant-lice"  n ( p l aa1 n t - l ai s ) ((pos "Kj$" ) ))
( "plant-louse"  n ( p l aa1 n t - l au s ) ((pos "Ki$" ) ))
( "plantain"  n ( p l a1 n t i n ) ((pos "K6%" ) ))
( "plantains"  n ( p l a1 n t i n z ) ((pos "Kj%" ) ))
( "plantation"  n ( p l a1 n t ei1 sh @ n ) ((pos "K6%" ) ))
( "plantations"  n ( p l a1 n t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "planted"  v ( p l aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "planter"  n ( p l aa1 n t @ r ) ((pos "K6%" ) ))
( "planters"  n ( p l aa1 n t @ z ) ((pos "Kj%" ) ))
( "planting"  v ( p l aa1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "plants"  v ( p l aa1 n t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "plaque"  n ( p l aa1 k ) ((pos "M6%" ) ))
( "plaques"  n ( p l aa1 k s ) ((pos "Mj%" ) ))
( "plash"  v ( p l a1 sh ) ((pos "J1$" "Ki$" ) (subcat )))
( "plashed"  v ( p l a1 sh t ) ((pos "Jc$" "Jd$" ) (subcat )))
( "plashes"  v ( p l a1 sh i z ) ((pos "Ja$" ) (subcat )))
( "plashing"  v ( p l a1 sh i ng ) ((pos "Jb$" ) (subcat )))
( "plasm"  n ( p l a1 z @ m ) ((pos "L@$" ) ))
( "plasma"  n ( p l a1 z m @ ) ((pos "L@%" ) ))
( "plaster"  v ( p l aa1 s t @ r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "plasterboard"  n ( p l aa1 s t @ b oo d ) ((pos "L@%" ) ))
( "plastered"  v ( p l aa1 s t @ d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "plasterer"  n ( p l aa1 s t @ r @ r ) ((pos "K6%" ) ))
( "plasterers"  n ( p l aa1 s t @ r @ z ) ((pos "Kj%" ) ))
( "plastering"  v ( p l aa1 s t @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "plasters"  v ( p l aa1 s t @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "plastic"  n ( p l a1 s t i k ) ((pos "K6%" "OA%" ) ))
( "plastic-bomb"  n ( p l a2 s t i k - b o1 m ) ((pos "K6%" ) ))
( "plastic-bombs"  n ( p l a2 s t i k - b o1 m z ) ((pos "Kj%" ) ))
( "plasticine"  n ( p l a1 s t i s ii n ) ((pos "L@%" ) ))
( "plasticity"  n ( p l a1 s t i1 s i t ii ) ((pos "L@%" ) ))
( "plastics"  n ( p l a1 s t i k s ) ((pos "Kj%" "Lk%" ) ))
( "plate"  v ( p l ei1 t ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "plate-glass"  n ( p l ei1 t - g l aa1 s ) ((pos "L@%" ) ))
( "plate-powder"  n ( p l ei1 t - p au d @ r ) ((pos "L@$" ) ))
( "plate-rack"  n ( p l ei1 t - r a k ) ((pos "K6%" ) ))
( "plate-racks"  n ( p l ei1 t - r a k s ) ((pos "Kj%" ) ))
( "plateau"  n ( p l a1 t ou ) ((pos "K6%" ) ))
( "plateaus"  n ( p l a1 t ou z ) ((pos "Kj%" ) ))
( "plateaux"  n ( p l a1 t ou z ) ((pos "Kj$" ) ))
( "plated"  v ( p l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "plateful"  n ( p l ei1 t f u l ) ((pos "K6%" ) ))
( "platefuls"  n ( p l ei1 t f u l z ) ((pos "Kj%" ) ))
( "platelayer"  n ( p l ei1 t l ei @ r ) ((pos "K6%" ) ))
( "platelayers"  n ( p l ei1 t l ei @ z ) ((pos "Kj%" ) ))
( "plates"  v ( p l ei1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "platform"  n ( p l a1 t f oo m ) ((pos "K6%" ) ))
( "platforms"  n ( p l a1 t f oo m z ) ((pos "Kj%" ) ))
( "plating"  v ( p l ei1 t i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" )))
( "platinum"  n ( p l a1 t i n @ m ) ((pos "L@%" ) ))
( "platitude"  n ( p l a1 t i t y uu d ) ((pos "M6%" ) ))
( "platitudes"  n ( p l a1 t i t y uu d z ) ((pos "Mj%" ) ))
( "platitudinous"  j ( p l a2 t i t y uu1 d i n @ s ) ((pos "OA%" ) ))
( "platoon"  n ( p l @1 t uu1 n ) ((pos "K6%" ) ))
( "platoons"  n ( p l @1 t uu1 n z ) ((pos "Kj%" ) ))
( "platter"  n ( p l a1 t @ r ) ((pos "K6%" ) ))
( "platters"  n ( p l a1 t @ z ) ((pos "Kj%" ) ))
( "platypus"  n ( p l a1 t i p @ s ) ((pos "K7%" ) ))
( "platypuses"  n ( p l a1 t i p @ s i z ) ((pos "Kj%" ) ))
( "plaudit"  n ( p l oo1 d i t ) ((pos "K6%" ) ))
( "plaudits"  n ( p l oo1 d i t s ) ((pos "Kj%" ) ))
( "plausibilities"  n ( p l oo2 z @ b i1 l i t i z ) ((pos "Mj$" ) ))
( "plausibility"  n ( p l oo2 z @ b i1 l i t ii ) ((pos "M8%" ) ))
( "plausible"  j ( p l oo1 z @ b l ) ((pos "OA%" ) ))
( "plausibly"  a ( p l oo1 z @ b l ii ) ((pos "Pu%" ) ))
( "play"  v ( p l ei1 ) ((pos "J0*" "M6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "12C" "13B" "14" "15A" "15B" )))
( "play-acting"  n ( p l ei1 - a k t i ng ) ((pos "L@%" ) ))
( "play-actor"  n ( p l ei1 - a k t @ r ) ((pos "K6%" ) ))
( "play-actors"  n ( p l ei1 - a k t @ z ) ((pos "Kj%" ) ))
( "play-box"  n ( p l ei1 - b o k s ) ((pos "K7$" ) ))
( "play-boxes"  n ( p l ei1 - b o k s i z ) ((pos "Kj$" ) ))
( "play-off"  n ( p l ei1 - o f ) ((pos "K6%" ) ))
( "play-offs"  n ( p l ei1 - o f s ) ((pos "Kj%" ) ))
( "play-reading"  n ( p l ei1 - r ii d i ng ) ((pos "K6%" ) ))
( "play-readings"  n ( p l ei1 - r ii d i ng z ) ((pos "Kj%" ) ))
( "playback"  n ( p l ei1 b a k ) ((pos "K6%" ) ))
( "playbacks"  n ( p l ei1 b a k s ) ((pos "Kj$" ) ))
( "playbill"  n ( p l ei1 b i l ) ((pos "K6%" ) ))
( "playbills"  n ( p l ei1 b i l z ) ((pos "Kj%" ) ))
( "playboy"  n ( p l ei1 b oi ) ((pos "K6%" ) ))
( "playboys"  n ( p l ei1 b oi z ) ((pos "Kj%" ) ))
( "played"  v ( p l ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "12C" "13B" "14" "15A" "15B" )))
( "player"  n ( p l ei1 @ r ) ((pos "K6%" ) ))
( "player-piano"  n ( p l ei2 @ - p i a1 n ou ) ((pos "K6$" ) ))
( "player-pianos"  n ( p l ei2 @ - p i a1 n ou z ) ((pos "Kj$" ) ))
( "players"  n ( p l ei1 @ z ) ((pos "Kj%" ) ))
( "playfellow"  n ( p l ei1 f e l ou ) ((pos "K6%" ) ))
( "playfellows"  n ( p l ei1 f e l ou z ) ((pos "Kj%" ) ))
( "playful"  j ( p l ei1 f @ l ) ((pos "OA%" ) ))
( "playfully"  a ( p l ei1 f @ l ii ) ((pos "Pu%" ) ))
( "playfulness"  n ( p l ei1 f @ l n @ s ) ((pos "L@%" ) ))
( "playgoer"  n ( p l ei1 g ou @ r ) ((pos "K6%" ) ))
( "playgoers"  n ( p l ei1 g ou @ z ) ((pos "Kj%" ) ))
( "playground"  n ( p l ei1 g r au n d ) ((pos "K6%" ) ))
( "playgrounds"  n ( p l ei1 g r au n d z ) ((pos "Kj%" ) ))
( "playgroup"  n ( p l ei1 g r uu p ) ((pos "K6%" ) ))
( "playgroups"  n ( p l ei1 g r uu p s ) ((pos "Kj%" ) ))
( "playhouse"  n ( p l ei1 h au s ) ((pos "K6%" ) ))
( "playhouses"  n ( p l ei1 h au z i z ) ((pos "Kj%" ) ))
( "playing"  v ( p l ei1 i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "12C" "13B" "14" "15A" "15B" )))
( "playing-card"  n ( p l ei1 i ng - k aa d ) ((pos "K6%" ) ))
( "playing-cards"  n ( p l ei1 i ng - k aa d z ) ((pos "Kj%" ) ))
( "playing-field"  n ( p l ei1 i ng - f ii l d ) ((pos "K6%" ) ))
( "playing-fields"  n ( p l ei1 i ng - f ii l d z ) ((pos "Kj%" ) ))
( "playings"  n ( p l ei1 i ng z ) ((pos "Kj$" ) ))
( "playlet"  n ( p l ei1 l i t ) ((pos "K6%" ) ))
( "playlets"  n ( p l ei1 l i t s ) ((pos "Kj%" ) ))
( "playmate"  n ( p l ei1 m ei t ) ((pos "K6%" ) ))
( "playmates"  n ( p l ei1 m ei t s ) ((pos "Kj%" ) ))
( "playpen"  n ( p l ei1 p e n ) ((pos "K6%" ) ))
( "playpens"  n ( p l ei1 p e n z ) ((pos "Kj%" ) ))
( "playroom"  n ( p l ei1 r uu m ) ((pos "K6%" ) ))
( "playrooms"  n ( p l ei1 r uu m z ) ((pos "Kj%" ) ))
( "plays"  v ( p l ei1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12B" "12C" "13B" "14" "15A" "15B" )))
( "playschool"  n ( p l ei1 s k uu l ) ((pos "K6%" ) ))
( "playschools"  n ( p l ei1 s k uu l z ) ((pos "Kj%" ) ))
( "playsuit"  n ( p l ei1 s uu t ) ((pos "K6%" ) ))
( "playsuits"  n ( p l ei1 s uu t s ) ((pos "Kj%" ) ))
( "plaything"  n ( p l ei1 th i ng ) ((pos "K6%" ) ))
( "playthings"  n ( p l ei1 th i ng z ) ((pos "Kj%" ) ))
( "playtime"  n ( p l ei1 t ai m ) ((pos "K6%" ) ))
( "playtimes"  n ( p l ei1 t ai m z ) ((pos "Kj%" ) ))
( "playwright"  n ( p l ei1 r ai t ) ((pos "K6%" ) ))
( "playwrights"  n ( p l ei1 r ai t s ) ((pos "Kj%" ) ))
( "plaza"  n ( p l aa1 z @ ) ((pos "K6%" ) ))
( "plazas"  n ( p l aa1 z @ z ) ((pos "Kj%" ) ))
( "plea"  n ( p l ii1 ) ((pos "K6%" ) ))
( "pleach"  v ( p l ii1 ch ) ((pos "H1$" ) (subcat )))
( "pleached"  v ( p l ii1 ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "pleaches"  v ( p l ii1 ch i z ) ((pos "Ha$" ) (subcat )))
( "pleaching"  v ( p l ii1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "plead"  v ( p l ii1 d ) ((pos "J0%" ) (subcat "3A" )))
( "pleaded"  v ( p l ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "pleading"  v ( p l ii1 d i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "pleadingly"  a ( p l ii1 d i ng l ii ) ((pos "Pu%" ) ))
( "pleadings"  n ( p l ii1 d i ng z ) ((pos "Kj%" ) ))
( "pleads"  v ( p l ii1 d z ) ((pos "Ja%" ) (subcat "3A" )))
( "pleas"  n ( p l ii1 z ) ((pos "Kj%" ) ))
( "pleasance"  n ( p l e1 z n s ) ((pos "K6$" ) ))
( "pleasances"  n ( p l e1 z n s i z ) ((pos "Kj$" ) ))
( "pleasant"  j ( p l e1 z n t ) ((pos "OC%" ) ))
( "pleasanter"  j ( p l e1 z @ n t @ r ) ((pos "Or%" ) ))
( "pleasantest"  j ( p l e1 z @ n t i s t ) ((pos "Os%" ) ))
( "pleasantly"  a ( p l e1 z n t l ii ) ((pos "Pu%" ) ))
( "pleasantness"  n ( p l e1 z n t n @ s ) ((pos "L@$" ) ))
( "pleasantries"  n ( p l e1 z n t r i z ) ((pos "Mj%" ) ))
( "pleasantry"  n ( p l e1 z n t r ii ) ((pos "M8%" ) ))
( "please"  v ( p l ii1 z ) ((pos "J2%" ) (subcat "2A" )))
( "pleased"  v ( p l ii1 z d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "pleases"  v ( p l ii1 z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "pleasing"  v ( p l ii1 z i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "pleasingly"  a ( p l ii1 z i ng l ii ) ((pos "Pu%" ) ))
( "pleasurable"  j ( p l e1 zh @ r @ b l ) ((pos "OA%" ) ))
( "pleasurably"  a ( p l e1 zh @ r @ b l ii ) ((pos "Pu%" ) ))
( "pleasure"  n ( p l e1 zh @ r ) ((pos "M6%" ) ))
( "pleasure-boat"  n ( p l e1 zh @ - b ou t ) ((pos "K6%" ) ))
( "pleasure-boats"  n ( p l e1 zh @ - b ou t s ) ((pos "Kj%" ) ))
( "pleasure-craft"  n ( p l e1 zh @ - k r aa f t ) ((pos "K6%" ) ))
( "pleasure-crafts"  n ( p l e1 zh @ - k r aa f t s ) ((pos "Kj%" ) ))
( "pleasure-ground"  n ( p l e1 zh @ - g r au n d ) ((pos "K6%" ) ))
( "pleasure-grounds"  n ( p l e1 zh @ - g r au n d z ) ((pos "Kj%" ) ))
( "pleasures"  n ( p l e1 zh @ z ) ((pos "Mj%" ) ))
( "pleat"  v ( p l ii1 t ) ((pos "H0$" "K6%" ) (subcat )))
( "pleated"  v ( p l ii1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "pleating"  v ( p l ii1 t i ng ) ((pos "Hb$" ) (subcat )))
( "pleats"  v ( p l ii1 t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "pleb"  n ( p l e1 b ) ((pos "K6$" ) ))
( "plebeian"  n ( p l i1 b ii1 @ n ) ((pos "K6%" "OA%" ) ))
( "plebeians"  n ( p l i1 b ii1 @ n z ) ((pos "Kj%" ) ))
( "plebiscite"  n ( p l e1 b i s i t ) ((pos "K6%" ) ))
( "plebiscites"  n ( p l e1 b i s i t s ) ((pos "Kj%" ) ))
( "plebs"  n ( p l e1 b z ) ((pos "Kj%" ) ))
( "plectrum"  n ( p l e1 k t r @ m ) ((pos "K6%" ) ))
( "plectrums"  n ( p l e1 k t r @ m z ) ((pos "Kj%" ) ))
( "pled"  v ( p l e1 d ) ((pos "Jc$" "Jd$" ) (subcat "3A" )))
( "pledge"  v ( p l e1 jh ) ((pos "H2%" "M6%" ) (subcat )))
( "pledged"  v ( p l e1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pledges"  v ( p l e1 jh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "pledging"  v ( p l e1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "plenarily"  a ( p l ii1 n @ r @ l ii ) ((pos "Pu$" ) ))
( "plenary"  j ( p l ii1 n @ r ii ) ((pos "OA%" ) ))
( "plenipotentiaries"  n ( p l e2 n i p @ t e1 n sh @ r i z ) ((pos "Kj%" ) ))
( "plenipotentiary"  n ( p l e2 n i p @ t e1 n sh @ r ii ) ((pos "K8%" ) ))
( "plenitude"  n ( p l e1 n i t y uu d ) ((pos "Ki%" ) ))
( "plenteous"  j ( p l e1 n t i@ s ) ((pos "OA%" ) ))
( "plenteously"  a ( p l e1 n t i@ s l ii ) ((pos "Pu%" ) ))
( "plentiful"  j ( p l e1 n t i f @ l ) ((pos "OA%" ) ))
( "plentifully"  a ( p l e1 n t i f @ l ii ) ((pos "Pu%" ) ))
( "plenty"  n ( p l e1 n t ii ) ((pos "L@%" "Pu%" ) ))
( "pleonasm"  n ( p l i@1 n a z @ m ) ((pos "M6$" ) ))
( "pleonasms"  n ( p l i@1 n a z @ m z ) ((pos "Mj$" ) ))
( "plethora"  n ( p l e1 th @ r @ ) ((pos "K6%" ) ))
( "plethoras"  n ( p l e1 th @ r @ z ) ((pos "Kj$" ) ))
( "pleurisy"  n ( p l u@1 r @ s ii ) ((pos "L@%" ) ))
( "plexus"  n ( p l e1 k s @ s ) ((pos "K9$" ) ))
( "plexuses"  n ( p l e1 k s @ s i z ) ((pos "Kj$" ) ))
( "pliability"  n ( p l ai2 @ b i1 l i t ii ) ((pos "L@%" ) ))
( "pliable"  j ( p l ai1 @ b l ) ((pos "OA%" ) ))
( "pliancy"  n ( p l ai1 @ n s ii ) ((pos "L@%" ) ))
( "pliant"  j ( p l ai1 @ n t ) ((pos "OA%" ) ))
( "pliantly"  a ( p l ai1 @ n t l ii ) ((pos "Pu%" ) ))
( "plied"  v ( p l ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "pliers"  n ( p l ai1 @ z ) ((pos "Kj%" ) ))
( "plies"  v ( p l ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "plight"  v ( p l ai1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "plighted"  v ( p l ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "plighting"  v ( p l ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "plights"  v ( p l ai1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "plimsolls"  n ( p l i1 m s @ l z ) ((pos "Kj%" ) ))
( "plinth"  n ( p l i1 n th ) ((pos "K6%" ) ))
( "plinths"  n ( p l i1 n th s ) ((pos "Kj%" ) ))
( "plod"  v ( p l o1 d ) ((pos "J4%" ) (subcat )))
( "plodded"  v ( p l o1 d i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "plodder"  n ( p l o1 d @ r ) ((pos "K6%" ) ))
( "plodders"  n ( p l o1 d @ z ) ((pos "Kj%" ) ))
( "plodding"  v ( p l o1 d i ng ) ((pos "Jb%" "OA%" ) (subcat )))
( "ploddingly"  a ( p l o1 d i ng l ii ) ((pos "Pu%" ) ))
( "plods"  v ( p l o1 d z ) ((pos "Ja%" ) (subcat )))
( "plonk"  v ( p l o1 ng k ) ((pos "I0%" "M6%" "Pu%" ) ))
( "plonked"  v ( p l o1 ng k t ) ((pos "Ic%" "Id%" ) ))
( "plonking"  v ( p l o1 ng k i ng ) ((pos "Ib%" ) ))
( "plonks"  v ( p l o1 ng k s ) ((pos "Ia%" "Mj%" ) ))
( "plop"  v ( p l o1 p ) ((pos "I4%" "K6%" "Pu%" ) ))
( "plopped"  v ( p l o1 p t ) ((pos "Ic%" "Id%" ) ))
( "plopping"  v ( p l o1 p i ng ) ((pos "Ib%" ) ))
( "plops"  v ( p l o1 p s ) ((pos "Ia%" "Kj%" ) ))
( "plosive"  n ( p l ou1 s i v ) ((pos "K6$" "OA$" ) ))
( "plosives"  n ( p l ou1 s i v z ) ((pos "Kj$" ) ))
( "plot"  v ( p l o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "3A" "4A" "6A" "8" "10" )))
( "plots"  v ( p l o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "4A" "6A" "8" "10" )))
( "plotted"  v ( p l o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" "6A" "8" "10" )))
( "plotter"  n ( p l o1 t @ r ) ((pos "K6%" ) ))
( "plotters"  n ( p l o1 t @ z ) ((pos "Kj%" ) ))
( "plotting"  v ( p l o1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" "6A" "8" "10" )))
( "plough"  v ( p l au1 ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" )))
( "ploughboy"  n ( p l au1 b oi ) ((pos "K6%" ) ))
( "ploughboys"  n ( p l au1 b oi z ) ((pos "Kj%" ) ))
( "ploughed"  v ( p l au1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "ploughing"  v ( p l au1 i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "ploughman"  n ( p l au1 m @ n ) ((pos "Ki%" ) ))
( "ploughmen"  n ( p l au1 m @ n ) ((pos "Kj%" ) ))
( "ploughs"  v ( p l au1 z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" )))
( "ploughshare"  n ( p l au1 sh e@ r ) ((pos "K6%" ) ))
( "ploughshares"  n ( p l au1 sh e@ z ) ((pos "Kj%" ) ))
( "plover"  n ( p l uh1 v @ r ) ((pos "K6%" ) ))
( "plovers"  n ( p l uh1 v @ z ) ((pos "Kj%" ) ))
( "ploy"  n ( p l oi1 ) ((pos "K6%" ) ))
( "ploys"  n ( p l oi1 z ) ((pos "Kj%" ) ))
( "pluck"  v ( p l uh1 k ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" )))
( "plucked"  v ( p l uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "pluckier"  j ( p l uh1 k i@ r ) ((pos "Or%" ) ))
( "pluckiest"  j ( p l uh1 k i i s t ) ((pos "Os%" ) ))
( "pluckily"  a ( p l uh1 k i l ii ) ((pos "Pu%" ) ))
( "plucking"  v ( p l uh1 k i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "plucks"  v ( p l uh1 k s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" )))
( "plucky"  j ( p l uh1 k ii ) ((pos "OD%" ) ))
( "plug"  v ( p l uh1 g ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" )))
( "plugged"  v ( p l uh1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "plugging"  v ( p l uh1 g i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "plughole"  n ( p l uh1 g h ou l ) ((pos "K6%" ) ))
( "plugholes"  n ( p l uh1 g h ou l z ) ((pos "Kj%" ) ))
( "plugs"  v ( p l uh1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "plum"  n ( p l uh1 m ) ((pos "K6%" ) ))
( "plum-pudding"  n ( p l uh1 m - p u1 d i ng ) ((pos "M6%" ) ))
( "plum-puddings"  n ( p l uh1 m - p u1 d i ng z ) ((pos "Mj%" ) ))
( "plumage"  n ( p l uu1 m i jh ) ((pos "L@%" ) ))
( "plumb"  v ( p l uh1 m ) ((pos "H0$" "K6%" "Pu%" ) (subcat )))
( "plumbago"  n ( p l uh1 m b ei1 g ou ) ((pos "M6$" ) ))
( "plumbagos"  n ( p l uh1 m b ei1 g ou z ) ((pos "Mj$" ) ))
( "plumbed"  v ( p l uh1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "plumber"  n ( p l uh1 m @ r ) ((pos "K6%" ) ))
( "plumbers"  n ( p l uh1 m @ z ) ((pos "Kj%" ) ))
( "plumbing"  v ( p l uh1 m i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "plumbs"  v ( p l uh1 m z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "plume"  v ( p l uu1 m ) ((pos "H2$" "K6%" ) (subcat )))
( "plumed"  v ( p l uu1 m d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "plumes"  v ( p l uu1 m z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "pluming"  v ( p l uu1 m i ng ) ((pos "Hb$" ) (subcat )))
( "plummet"  v ( p l uh1 m i t ) ((pos "I4%" "K6%" ) (subcat )))
( "plummets"  v ( p l uh1 m i t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "plummetted"  v ( p l uh1 m i t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "plummetting"  v ( p l uh1 m i t i ng ) ((pos "Ib%" ) (subcat )))
( "plummier"  j ( p l uh1 m i@ r ) ((pos "Or%" ) ))
( "plummiest"  j ( p l uh1 m i i s t ) ((pos "Os%" ) ))
( "plummy"  j ( p l uh1 m ii ) ((pos "OD%" ) ))
( "plump"  v ( p l uh1 m p ) ((pos "J0%" "K6%" "OC%" "Pu%" ) (subcat "2A" "2C" "3A" "6A" )))
( "plumped"  v ( p l uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "plumper"  j ( p l uh1 m p @ r ) ((pos "Or%" ) ))
( "plumpest"  j ( p l uh1 m p i s t ) ((pos "Os%" ) ))
( "plumping"  v ( p l uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "plumps"  v ( p l uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "plums"  n ( p l uh1 m z ) ((pos "Kj%" ) ))
( "plunder"  v ( p l uh1 n d @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "6A" )))
( "plundered"  v ( p l uh1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "plunderer"  n ( p l uh1 n d @ r @ r ) ((pos "K6%" ) ))
( "plunderers"  n ( p l uh1 n d @ r @ z ) ((pos "Kj%" ) ))
( "plundering"  v ( p l uh1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "plunders"  v ( p l uh1 n d @ z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "plunge"  v ( p l uh1 n jh ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "plunged"  v ( p l uh1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "plunger"  n ( p l uh1 n jh @ r ) ((pos "K6%" ) ))
( "plungers"  n ( p l uh1 n jh @ z ) ((pos "Kj%" ) ))
( "plunges"  v ( p l uh1 n jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "plunging"  v ( p l uh1 n jh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "plunk"  n ( p l uh1 ng k ) ((pos "K6$" ) ))
( "plunks"  n ( p l uh1 ng k s ) ((pos "Kj$" ) ))
( "pluperfect"  n ( p l uu2 p @@1 f i k t ) ((pos "K6%" "OA%" ) ))
( "pluperfects"  n ( p l uu2 p @@1 f i k t s ) ((pos "Kj$" ) ))
( "plural"  n ( p l u@1 r @ l ) ((pos "K6%" "OA%" ) ))
( "pluralism"  n ( p l u@1 r @ l i z @ m ) ((pos "L@%" ) ))
( "pluralist"  n ( p l u@1 r @ l i s t ) ((pos "K6%" ) ))
( "pluralists"  n ( p l u@1 r @ l i s t s ) ((pos "Kj%" ) ))
( "pluralities"  n ( p l u@1 r a1 l i t i z ) ((pos "Mj%" ) ))
( "plurality"  n ( p l u@1 r a1 l i t ii ) ((pos "M8%" ) ))
( "plurals"  n ( p l u@1 r @ l z ) ((pos "Kj%" ) ))
( "plus"  n ( p l uh s ) ((pos "K7%" "OA%" "T-%" ) ))
( "plus-fours"  n ( p l uh1 s - f oo1 z ) ((pos "Kj%" ) ))
( "pluses"  n ( p l uh1 s i z ) ((pos "Kj%" ) ))
( "plush"  n ( p l uh1 sh ) ((pos "L@%" "OC%" ) ))
( "plusher"  j ( p l uh1 sh @ r ) ((pos "Or%" ) ))
( "plushest"  j ( p l uh1 sh i s t ) ((pos "Os%" ) ))
( "plushier"  j ( p l uh1 sh i@ r ) ((pos "Or$" ) ))
( "plushiest"  j ( p l uh1 sh i i s t ) ((pos "Os$" ) ))
( "plushy"  j ( p l uh1 sh ii ) ((pos "OD$" ) ))
( "plutocracies"  n ( p l uu1 t o1 k r @ s i z ) ((pos "Mj%" ) ))
( "plutocracy"  n ( p l uu1 t o1 k r @ s ii ) ((pos "M8%" ) ))
( "plutocrat"  n ( p l uu1 t @ k r a t ) ((pos "K6%" ) ))
( "plutocratic"  j ( p l uu2 t @ k r a1 t i k ) ((pos "OA%" ) ))
( "plutocrats"  n ( p l uu1 t @ k r a t s ) ((pos "Kj%" ) ))
( "plutonium"  n ( p l uu1 t ou1 n i@ m ) ((pos "L@%" ) ))
( "ply"  v ( p l ai1 ) ((pos "J3%" "K9%" ) (subcat "2C" )))
( "plying"  v ( p l ai1 i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "plywood"  n ( p l ai1 w u d ) ((pos "L@%" ) ))
( "pm"  n ( p ii2 e1 m ) ((pos "Y~%" ) ))
( "pneumatic"  j ( n y uu1 m a1 t i k ) ((pos "OA%" ) ))
( "pneumatically"  a ( n y uu1 m a1 t i k l ii ) ((pos "Pu%" ) ))
( "pneumonia"  n ( n y uu1 m ou1 n i@ ) ((pos "L@%" ) ))
( "poach"  v ( p ou1 ch ) ((pos "J1%" ) (subcat "2A" "3A" )))
( "poached"  v ( p ou1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "poacher"  n ( p ou1 ch @ r ) ((pos "K6%" ) ))
( "poachers"  n ( p ou1 ch @ z ) ((pos "Kj%" ) ))
( "poaches"  v ( p ou1 ch i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "poaching"  v ( p ou1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "pock"  n ( p o1 k ) ((pos "K6$" ) ))
( "pocked"  j ( p o1 k t ) ((pos "OA$" ) ))
( "pocket"  v ( p o1 k i t ) ((pos "H0%" "K6%" ) (subcat )))
( "pocket-book"  n ( p o1 k i t - b u k ) ((pos "K6%" ) ))
( "pocket-books"  n ( p o1 k i t - b u k s ) ((pos "Kj%" ) ))
( "pocket-handkerchief"  n ( p o2 k i t - h a1 ng k @ ch i f ) ((pos "K6%" ) ))
( "pocket-handkerchiefs"  n ( p o2 k i t - h a1 ng k @ ch i f s ) ((pos "Kj%" ) ))
( "pocket-knife"  n ( p o1 k i t - n ai f ) ((pos "Ki%" ) ))
( "pocket-knives"  n ( p o1 k i t - n ai v z ) ((pos "Kj%" ) ))
( "pocket-money"  n ( p o1 k i t - m uh n ii ) ((pos "L@%" ) ))
( "pocketed"  v ( p o1 k i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pocketful"  n ( p o1 k i t f u l ) ((pos "K6%" ) ))
( "pocketfuls"  n ( p o1 k i t f u l z ) ((pos "Kj%" ) ))
( "pocketing"  v ( p o1 k i t i ng ) ((pos "Hb%" ) (subcat )))
( "pockets"  v ( p o1 k i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "pockmarked"  j ( p o1 k m aa k t ) ((pos "OA%" ) ))
( "pocks"  n ( p o1 k s ) ((pos "Kj$" ) ))
( "pod"  v ( p o1 d ) ((pos "J4$" "K6%" ) (subcat "2A" "2C" )))
( "podded"  v ( p o1 d i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" )))
( "podding"  v ( p o1 d i ng ) ((pos "Jb$" ) (subcat "2A" "2C" )))
( "podgier"  j ( p o1 jh i@ r ) ((pos "Or%" ) ))
( "podgiest"  j ( p o1 jh i i s t ) ((pos "Os%" ) ))
( "podgy"  j ( p o1 jh ii ) ((pos "OD%" ) ))
( "podiatry"  n ( p @1 d ai1 @ t r ii ) ((pos "L@$" ) ))
( "podium"  n ( p ou1 d i@ m ) ((pos "K6%" ) ))
( "podiums"  n ( p ou1 d i@ m z ) ((pos "Kj%" ) ))
( "pods"  v ( p o1 d z ) ((pos "Ja$" "Kj%" ) (subcat "2A" "2C" )))
( "poem"  n ( p ou1 i m ) ((pos "K6%" ) ))
( "poems"  n ( p ou1 i m z ) ((pos "Kj%" ) ))
( "poesy"  n ( p ou1 i z ii ) ((pos "L@$" ) ))
( "poet"  n ( p ou1 i t ) ((pos "K6%" ) ))
( "poetess"  n ( p ou1 i t e s ) ((pos "K7%" ) ))
( "poetesses"  n ( p ou1 i t e s i z ) ((pos "Kj%" ) ))
( "poetic"  j ( p ou1 e1 t i k ) ((pos "OA%" ) ))
( "poetical"  j ( p ou1 e1 t i k l ) ((pos "OA%" ) ))
( "poetically"  a ( p ou1 e1 t i k l ii ) ((pos "Pu%" ) ))
( "poetry"  n ( p ou1 i t r ii ) ((pos "L@%" ) ))
( "poets"  n ( p ou1 i t s ) ((pos "Kj%" ) ))
( "pogrom"  n ( p o1 g r @ m ) ((pos "K6$" ) ))
( "pogroms"  n ( p o1 g r @ m z ) ((pos "Kj$" ) ))
( "poignancy"  n ( p oi1 n y @ n s ii ) ((pos "L@%" ) ))
( "poignant"  j ( p oi1 n y @ n t ) ((pos "OA%" ) ))
( "poignantly"  a ( p oi1 n y @ n t l ii ) ((pos "Pu%" ) ))
( "poinsettia"  n ( p oi1 n s e1 t i@ ) ((pos "K6%" ) ))
( "poinsettias"  n ( p oi1 n s e1 t i@ z ) ((pos "Kj%" ) ))
( "point"  v ( p oi1 n t ) ((pos "J0*" "M6*" ) (subcat "2A" "3A" "6A" "14" )))
( "point-blank"  j ( p oi2 n t - b l a1 ng k ) ((pos "OA%" "Pu%" ) ))
( "point-duty"  n ( p oi1 n t - d y uu t ii ) ((pos "L@%" ) ))
( "pointed"  v ( p oi1 n t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" "14" )))
( "pointedly"  a ( p oi1 n t i d l ii ) ((pos "Pu%" ) ))
( "pointer"  n ( p oi1 n t @ r ) ((pos "K6%" ) ))
( "pointers"  n ( p oi1 n t @ z ) ((pos "Kj%" ) ))
( "pointing"  v ( p oi1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" )))
( "pointless"  j ( p oi1 n t l i s ) ((pos "OA%" ) ))
( "pointlessly"  a ( p oi1 n t l i s l ii ) ((pos "Pu%" ) ))
( "points"  v ( p oi1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "14" )))
( "pointsman"  n ( p oi1 n t s m @ n ) ((pos "Ki%" ) ))
( "pointsmen"  n ( p oi1 n t s m @ n ) ((pos "Kj%" ) ))
( "poise"  v ( p oi1 z ) ((pos "J2%" "L@%" ) (subcat "2C" "6A" )))
( "poised"  v ( p oi1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "poises"  v ( p oi1 z i z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "poising"  v ( p oi1 z i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "poison"  v ( p oi1 z n ) ((pos "H0%" "M6%" ) (subcat )))
( "poison-gas"  n ( p oi2 z n - g a1 s ) ((pos "M7%" ) ))
( "poison-gases"  n ( p oi2 z n - g a1 s i z ) ((pos "Mj$" ) ))
( "poison-ivies"  n ( p oi2 z n - ai1 v i z ) ((pos "Mj$" ) ))
( "poison-ivy"  n ( p oi2 z n - ai1 v ii ) ((pos "M8%" ) ))
( "poisoned"  v ( p oi1 z n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "poisoner"  n ( p oi1 z n @ r ) ((pos "K6%" ) ))
( "poisoners"  n ( p oi1 z n @ z ) ((pos "Kj%" ) ))
( "poisoning"  v ( p oi1 z n i ng ) ((pos "Hb%" ) (subcat )))
( "poisonous"  j ( p oi1 z @ n @ s ) ((pos "OA%" ) ))
( "poisonously"  a ( p oi1 z @ n @ s l ii ) ((pos "Pu%" ) ))
( "poisons"  v ( p oi1 z n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "poke"  v ( p ou1 k ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "poke-bonnet"  n ( p ou2 k - b o1 n i t ) ((pos "K6$" ) ))
( "poke-bonnets"  n ( p ou2 k - b o1 n i t s ) ((pos "Kj$" ) ))
( "poked"  v ( p ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "poker"  n ( p ou1 k @ r ) ((pos "M6%" ) ))
( "poker-face"  n ( p ou1 k @ - f ei s ) ((pos "K6%" ) ))
( "poker-faces"  n ( p ou1 k @ - f ei s i z ) ((pos "Kj%" ) ))
( "pokers"  n ( p ou1 k @ z ) ((pos "Mj%" ) ))
( "pokes"  v ( p ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "pokier"  j ( p ou1 k i@ r ) ((pos "Or%" ) ))
( "pokiest"  j ( p ou1 k i i s t ) ((pos "Os%" ) ))
( "poking"  v ( p ou1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "poky"  j ( p ou1 k ii ) ((pos "OD%" ) ))
( "polar"  j ( p ou1 l @ r ) ((pos "OA%" ) ))
( "polarities"  n ( p @1 l a1 r i t i z ) ((pos "Mj%" ) ))
( "polarity"  n ( p @1 l a1 r i t ii ) ((pos "M8%" ) ))
( "polarization"  n ( p ou2 l @ r ai z ei1 sh @ n ) ((pos "K6%" ) ))
( "polarizations"  n ( p ou2 l @ r ai z ei1 sh @ n z ) ((pos "Kj%" ) ))
( "polarize"  v ( p ou1 l @ r ai z ) ((pos "H2%" ) (subcat )))
( "polarized"  v ( p ou1 l @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "polarizes"  v ( p ou1 l @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "polarizing"  v ( p ou1 l @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "pole"  n ( p ou1 l ) ((pos "K6%" ) ))
( "pole-jumping"  n ( p ou1 l - jh uh m p i ng ) ((pos "L@%" ) ))
( "pole-star"  n ( p ou1 l - s t aa r ) ((pos "K6%" ) ))
( "pole-stars"  n ( p ou1 l - s t aa z ) ((pos "Kj%" ) ))
( "pole-vault"  n ( p ou1 l - v oo l t ) ((pos "K6%" ) ))
( "pole-vaults"  n ( p ou1 l - v oo l t s ) ((pos "Kj%" ) ))
( "poleax"  v ( p ou1 l a k s ) ((pos "H1$" "K7$" ) (subcat )))
( "poleaxe"  v ( p ou1 l a k s ) ((pos "H2%" "K6%" ) (subcat )))
( "poleaxed"  v ( p ou1 l a k s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "poleaxes"  v ( p ou1 l a k s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "poleaxing"  v ( p ou1 l a k s i ng ) ((pos "Hb%" ) (subcat )))
( "polecat"  n ( p ou1 l k a t ) ((pos "K6%" ) ))
( "polecats"  n ( p ou1 l k a t s ) ((pos "Kj%" ) ))
( "polemic"  n ( p @1 l e1 m i k ) ((pos "K6%" "OA%" ) ))
( "polemically"  a ( p @1 l e1 m i k l ii ) ((pos "Pu%" ) ))
( "polemics"  n ( p @1 l e1 m i k s ) ((pos "Kj%" ) ))
( "poles"  n ( p ou1 l z ) ((pos "Kj%" ) ))
( "police"  v ( p @1 l ii1 s ) ((pos "H2%" "Kj%" ) (subcat )))
( "police-office"  n ( p @1 l ii1 s - o f i s ) ((pos "K6%" ) ))
( "police-officer"  n ( p @1 l ii1 s - o f i s @ r ) ((pos "K6%" ) ))
( "police-officers"  n ( p @1 l ii1 s - o f i s @ z ) ((pos "Kj%" ) ))
( "police-offices"  n ( p @1 l ii1 s - o f i s i z ) ((pos "Kj%" ) ))
( "police-station"  n ( p @1 l ii1 s - s t ei sh @ n ) ((pos "K6%" ) ))
( "police-stations"  n ( p @1 l ii1 s - s t ei sh @ n z ) ((pos "Kj%" ) ))
( "policed"  v ( p @1 l ii1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "policeman"  n ( p @1 l ii1 s m @ n ) ((pos "Ki%" ) ))
( "policemen"  n ( p @1 l ii1 s m @ n ) ((pos "Kj%" ) ))
( "polices"  v ( p @1 l ii1 s i z ) ((pos "Ha%" ) (subcat )))
( "policewoman"  n ( p @1 l ii1 s w u m @ n ) ((pos "Ki%" ) ))
( "policewomen"  n ( p @1 l ii1 s w i m i n ) ((pos "Kj%" ) ))
( "policies"  n ( p o1 l @ s i z ) ((pos "Mj%" ) ))
( "policing"  v ( p @1 l ii1 s i ng ) ((pos "Hb%" ) (subcat )))
( "policy"  n ( p o1 l @ s ii ) ((pos "M8%" ) ))
( "polio"  n ( p ou1 l i@ @ ) ((pos "L@%" ) ))
( "poliomyelitis"  n ( p ou2 l i@ u m ai2 @ l ai1 t i s ) ((pos "L@%" ) ))
( "polish"  v ( p o1 l i sh ) ((pos "J1%" "M7%" ) (subcat "2A" "6A" )))
( "polished"  v ( p o1 l i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "polisher"  n ( p o1 l i sh @ r ) ((pos "K6%" ) ))
( "polishers"  n ( p o1 l i sh @ z ) ((pos "Kj%" ) ))
( "polishes"  v ( p o1 l i sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "polishing"  v ( p o1 l i sh i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "politburo"  n ( p o1 l i t b y u@ r ou ) ((pos "K6%" ) ))
( "politburos"  n ( p o1 l i t b y u@ r ou z ) ((pos "Kj$" ) ))
( "polite"  j ( p @1 l ai1 t ) ((pos "OB%" ) ))
( "politely"  a ( p @1 l ai1 t l ii ) ((pos "Pu%" ) ))
( "politeness"  n ( p @1 l ai1 t n @ s ) ((pos "L@%" ) ))
( "politer"  j ( p @1 l ai1 t @ r ) ((pos "Or%" ) ))
( "politest"  j ( p @1 l ai1 t i s t ) ((pos "Os%" ) ))
( "politic"  j ( p o1 l i t i k ) ((pos "OA%" ) ))
( "political"  j ( p @1 l i1 t i k l ) ((pos "OA%" ) ))
( "politically"  a ( p @1 l i1 t i k l ii ) ((pos "Pu%" ) ))
( "politician"  n ( p o2 l i t i1 sh @ n ) ((pos "K6%" ) ))
( "politicians"  n ( p o2 l i t i1 sh @ n z ) ((pos "Kj%" ) ))
( "politicize"  v ( p @1 l i1 t i s ai z ) ((pos "J2%" ) (subcat "2A" )))
( "politicized"  v ( p @1 l i1 t i s ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "politicizes"  v ( p @1 l i1 t i s ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "politicizing"  v ( p @1 l i1 t i s ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "politick"  v ( p o1 l i t i k ) ((pos "I0$" ) (subcat )))
( "politicked"  v ( p o1 l i t i k t ) ((pos "Ic$" "Id$" ) (subcat )))
( "politicking"  v ( p o1 l i t i k i ng ) ((pos "Ib$" ) (subcat )))
( "politicks"  v ( p o1 l i t i k s ) ((pos "Ia$" ) (subcat )))
( "politics"  n ( p o1 l i t i k s ) ((pos "Lk$" ) ))
( "polities"  n ( p o1 l i t i z ) ((pos "Mj$" ) ))
( "polity"  n ( p o1 l i t ii ) ((pos "M8$" ) ))
( "polka"  n ( p o1 l k @ ) ((pos "K6%" ) ))
( "polkas"  n ( p o1 l k @ z ) ((pos "Kj%" ) ))
( "poll"  n ( p o1 l ) ((pos "K6$" ) ))
( "poll"  v ( p ou1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "poll-tax"  n ( p ou1 l - t a k s ) ((pos "K7%" ) ))
( "poll-taxes"  n ( p ou1 l - t a k s i z ) ((pos "Kj%" ) ))
( "pollard"  v ( p o1 l @ d ) ((pos "H0%" "K6$" ) (subcat )))
( "pollarded"  v ( p o1 l @ d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pollarding"  v ( p o1 l @ d i ng ) ((pos "Hb%" ) (subcat )))
( "pollards"  v ( p o1 l @ d z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "polled"  v ( p ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "pollen"  n ( p o1 l @ n ) ((pos "L@%" ) ))
( "pollinate"  v ( p o1 l i n ei t ) ((pos "H2%" ) (subcat )))
( "pollinated"  v ( p o1 l i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pollinates"  v ( p o1 l i n ei t s ) ((pos "Ha%" ) (subcat )))
( "pollinating"  v ( p o1 l i n ei t i ng ) ((pos "Hb%" ) (subcat )))
( "pollination"  n ( p o2 l i n ei1 sh @ n ) ((pos "L@%" ) ))
( "polling"  v ( p ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "polling-booth"  n ( p ou1 l i ng - b uu dh ) ((pos "K6%" ) ))
( "polling-booths"  n ( p ou1 l i ng - b uu dh z ) ((pos "Kj%" ) ))
( "polling-day"  n ( p ou1 l i ng - d ei ) ((pos "K6%" ) ))
( "polling-days"  n ( p ou1 l i ng - d ei z ) ((pos "Kj%" ) ))
( "polling-station"  n ( p ou1 l i ng - s t ei sh @ n ) ((pos "K6%" ) ))
( "polling-stations"  n ( p ou1 l i ng - s t ei sh @ n z ) ((pos "Kj%" ) ))
( "polls"  n ( p o1 l z ) ((pos "Kj$" ) ))
( "polls"  v ( p ou1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "pollster"  n ( p ou1 l s t @ r ) ((pos "K6%" ) ))
( "pollsters"  n ( p ou1 l s t @ z ) ((pos "Kj%" ) ))
( "pollutant"  n ( p @1 l uu1 t @ n t ) ((pos "K6%" ) ))
( "pollutants"  n ( p @1 l uu1 t @ n t s ) ((pos "Kj%" ) ))
( "pollute"  v ( p @1 l uu1 t ) ((pos "H2%" ) (subcat )))
( "polluted"  v ( p @1 l uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pollutes"  v ( p @1 l uu1 t s ) ((pos "Ha%" ) (subcat )))
( "polluting"  v ( p @1 l uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "pollution"  n ( p @1 l uu1 sh @ n ) ((pos "L@%" ) ))
( "polo"  n ( p ou1 l ou ) ((pos "L@%" ) ))
( "polo-neck"  j ( p ou1 l ou - n e k ) ((pos "OA%" ) ))
( "polonaise"  n ( p o2 l @ n ei1 z ) ((pos "K6$" ) ))
( "polonaises"  n ( p o2 l @ n ei1 z i z ) ((pos "Kj$" ) ))
( "polony"  n ( p @1 l ou1 n ii ) ((pos "L@%" ) ))
( "poltergeist"  n ( p o1 l t @ g ai s t ) ((pos "K6%" ) ))
( "poltergeists"  n ( p o1 l t @ g ai s t s ) ((pos "Kj%" ) ))
( "poltroon"  n ( p o1 l t r uu1 n ) ((pos "K6$" ) ))
( "poltroonery"  n ( p o1 l t r uu1 n @ r ii ) ((pos "L@$" ) ))
( "poltroons"  n ( p o1 l t r uu1 n z ) ((pos "Kj$" ) ))
( "poly"  n ( p o1 l ii ) ((pos "K6$" ) ))
( "polyandrous"  j ( p o2 l i a1 n d r @ s ) ((pos "OA$" ) ))
( "polyandry"  n ( p o1 l i a n d r ii ) ((pos "L@$" ) ))
( "polyanthus"  n ( p o2 l i a1 n th @ s ) ((pos "K7%" ) ))
( "polyanthuses"  n ( p o2 l i a1 n th @ s i z ) ((pos "Kj$" ) ))
( "polygamist"  n ( p @1 l i1 g @ m i s t ) ((pos "K6%" ) ))
( "polygamists"  n ( p @1 l i1 g @ m i s t s ) ((pos "Kj%" ) ))
( "polygamous"  j ( p @1 l i1 g @ m @ s ) ((pos "OA%" ) ))
( "polygamy"  n ( p @1 l i1 g @ m ii ) ((pos "L@%" ) ))
( "polyglot"  n ( p o1 l i g l o t ) ((pos "K6%" "OA%" ) ))
( "polyglots"  n ( p o1 l i g l o t s ) ((pos "Kj%" ) ))
( "polygon"  n ( p o1 l i g @ n ) ((pos "K6%" ) ))
( "polygonal"  j ( p @1 l i1 g @ n @ l ) ((pos "OA%" ) ))
( "polygons"  n ( p o1 l i g @ n z ) ((pos "Kj%" ) ))
( "polymorphic"  j ( p o2 l i m oo1 f i k ) ((pos "OA$" ) ))
( "polymorphous"  j ( p o2 l i m oo1 f @ s ) ((pos "OA$" ) ))
( "polynomial"  n ( p o2 l i n ou1 m i@ l ) ((pos "K6$" "OA$" ) ))
( "polynomials"  n ( p o2 l i n ou1 m i@ l z ) ((pos "Kj$" ) ))
( "polyp"  n ( p o1 l i p ) ((pos "K6$" ) ))
( "polyphonic"  j ( p o2 l i f o1 n i k ) ((pos "OA%" ) ))
( "polyphony"  n ( p @1 l i1 f @ n ii ) ((pos "L@%" ) ))
( "polypi"  n ( p o1 l i p ai ) ((pos "Kj$" ) ))
( "polyps"  n ( p o1 l i p s ) ((pos "Kj$" ) ))
( "polypus"  n ( p o1 l i p @ s ) ((pos "K7$" ) ))
( "polypuses"  n ( p o1 l i p @ s i z ) ((pos "Kj$" ) ))
( "polys"  n ( p o1 l i z ) ((pos "Kj$" ) ))
( "polysyllabic"  j ( p o2 l i s i l a1 b i k ) ((pos "OA%" ) ))
( "polysyllable"  n ( p o1 l i s i l @ b l ) ((pos "K6%" ) ))
( "polysyllables"  n ( p o1 l i s i l @ b l z ) ((pos "Kj%" ) ))
( "polytechnic"  n ( p o2 l i t e1 k n i k ) ((pos "K6%" ) ))
( "polytechnics"  n ( p o2 l i t e1 k n i k s ) ((pos "Kj%" ) ))
( "polytheism"  n ( p o1 l i th ii i z @ m ) ((pos "L@%" ) ))
( "polytheistic"  j ( p o2 l i th ii i1 s t i k ) ((pos "OA%" ) ))
( "polythene"  n ( p o1 l i th ii n ) ((pos "L@%" ) ))
( "pom"  n ( p o1 m ) ((pos "K6$" ) ))
( "pomade"  v ( p @1 m aa1 d ) ((pos "H2$" "L@$" ) ))
( "pomaded"  v ( p @1 m aa1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "pomades"  v ( p @1 m aa1 d z ) ((pos "Ha$" ) ))
( "pomading"  v ( p @1 m aa1 d i ng ) ((pos "Hb$" ) ))
( "pomegranate"  n ( p o1 m i g r a n i t ) ((pos "K6%" ) ))
( "pomegranates"  n ( p o1 m i g r a n i t s ) ((pos "Kj%" ) ))
( "pomelo"  n ( p o1 m @ l ou ) ((pos "K6$" ) ))
( "pomelos"  n ( p o1 m @ l ou z ) ((pos "Kj$" ) ))
( "pommel"  n ( p o1 m l ) ((pos "K6$" ) ))
( "pommel"  v ( p uh1 m l ) ((pos "H4$" ) ))
( "pommelled"  v ( p uh1 m l d ) ((pos "Hc$" "Hd$" ) ))
( "pommelling"  v ( p uh1 m @ l i ng ) ((pos "Hb$" ) ))
( "pommels"  n ( p o1 m l z ) ((pos "Kj$" ) ))
( "pommels"  v ( p uh1 m l z ) ((pos "Ha$" ) ))
( "pommies"  n ( p o1 m i z ) ((pos "Kj$" ) ))
( "pommy"  n ( p o1 m ii ) ((pos "K8$" ) ))
( "pomp"  n ( p o1 m p ) ((pos "L@%" ) ))
( "pompon"  n ( p o1 m p o n ) ((pos "K6%" ) ))
( "pompons"  n ( p o1 m p o n z ) ((pos "Kj%" ) ))
( "pomposities"  n ( p o1 m p o1 s i t i z ) ((pos "Mj$" ) ))
( "pomposity"  n ( p o1 m p o1 s i t ii ) ((pos "M8%" ) ))
( "pompous"  j ( p o1 m p @ s ) ((pos "OA%" ) ))
( "pompously"  a ( p o1 m p @ s l ii ) ((pos "Pu%" ) ))
( "poms"  n ( p o1 m z ) ((pos "Kj$" ) ))
( "ponce"  n ( p o1 n s ) ((pos "K6%" ) ))
( "ponces"  n ( p o1 n s i z ) ((pos "Kj%" ) ))
( "poncho"  n ( p o1 n ch ou ) ((pos "K6%" ) ))
( "ponchos"  n ( p o1 n ch ou z ) ((pos "Kj%" ) ))
( "pond"  n ( p o1 n d ) ((pos "K6%" ) ))
( "ponder"  v ( p o1 n d @ r ) ((pos "J0%" ) (subcat "2A" "3A" "6A" "8" )))
( "ponderable"  j ( p o1 n d @ r @ b l ) ((pos "OA$" ) ))
( "ponderables"  n ( p o1 n d @ r @ b l z ) ((pos "Kj$" ) ))
( "pondered"  v ( p o1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" )))
( "pondering"  v ( p o1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" )))
( "ponderous"  j ( p o1 n d @ r @ s ) ((pos "OA%" ) ))
( "ponderously"  a ( p o1 n d @ r @ s l ii ) ((pos "Pu%" ) ))
( "ponders"  v ( p o1 n d @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" )))
( "ponds"  n ( p o1 n d z ) ((pos "Kj%" ) ))
( "pone"  n ( p ou1 n ) ((pos "K6$" ) ))
( "pones"  n ( p ou1 n z ) ((pos "Kj$" ) ))
( "pongee"  n ( p o1 n jh ii1 ) ((pos "L@$" ) ))
( "poniard"  v ( p o1 n y @ d ) ((pos "H0$" "K6$" ) (subcat )))
( "poniarded"  v ( p o1 n y @ d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "poniarding"  v ( p o1 n y @ d i ng ) ((pos "Hb$" ) (subcat )))
( "poniards"  v ( p o1 n y @ d z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "ponies"  n ( p ou1 n i z ) ((pos "Kj%" ) ))
( "pontiff"  n ( p o1 n t i f ) ((pos "K6%" ) ))
( "pontiffs"  n ( p o1 n t i f s ) ((pos "Kj%" ) ))
( "pontifical"  j ( p o1 n t i1 f i k l ) ((pos "OA%" ) ))
( "pontificals"  n ( p o1 n t i1 f i k l z ) ((pos "Kj$" ) ))
( "pontificate"  n ( p o1 n t i1 f i k @ t ) ((pos "K6$" ) ))
( "pontificate"  v ( p o1 n t i1 f i k ei t ) ((pos "I2%" ) (subcat )))
( "pontificated"  v ( p o1 n t i1 f i k ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "pontificates"  n ( p o1 n t i1 f i k @ t s ) ((pos "Kj$" ) ))
( "pontificates"  v ( p o1 n t i1 f i k ei t s ) ((pos "Ia%" ) (subcat )))
( "pontificating"  v ( p o1 n t i1 f i k ei t i ng ) ((pos "Ib%" ) (subcat )))
( "pontoon"  n ( p o1 n t uu1 n ) ((pos "M6%" ) ))
( "pontoons"  n ( p o1 n t uu1 n z ) ((pos "Mj$" ) ))
( "pony"  n ( p ou1 n ii ) ((pos "K8%" ) ))
( "pony-trekking"  n ( p ou1 n i - t r e k i ng ) ((pos "L@%" ) ))
( "ponytail"  n ( p ou1 n i t ei l ) ((pos "K6%" ) ))
( "ponytails"  n ( p ou1 n i t ei l z ) ((pos "Kj%" ) ))
( "poodle"  n ( p uu1 d l ) ((pos "K6%" ) ))
( "poodles"  n ( p uu1 d l z ) ((pos "Kj%" ) ))
( "poof"  n ( p uu1 f ) ((pos "K6$" ) ))
( "poofs"  n ( p uu1 f s ) ((pos "Kj$" ) ))
( "pooh"  uh ( p uu1 ) ((pos "W-$" ) ))
( "pooh-pooh"  v ( p uu2 - p uu1 ) ((pos "H0$" ) (subcat )))
( "pooh-poohed"  v ( p uu2 - p uu1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "pooh-poohing"  v ( p uu2 - p uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "pooh-poohs"  v ( p uu2 - p uu1 z ) ((pos "Ha$" ) (subcat )))
( "pool"  v ( p uu1 l ) ((pos "H0%" "M6%" ) (subcat )))
( "pooled"  v ( p uu1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pooling"  v ( p uu1 l i ng ) ((pos "Hb%" ) (subcat )))
( "poolroom"  n ( p uu1 l r uu m ) ((pos "K6%" ) ))
( "poolrooms"  n ( p uu1 l r uu m z ) ((pos "Kj%" ) ))
( "pools"  v ( p uu1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "poop"  n ( p uu1 p ) ((pos "K6%" ) ))
( "poops"  n ( p uu1 p s ) ((pos "Kj%" ) ))
( "poor"  j ( p u@1 r ) ((pos "OC%" ) ))
( "poor-box"  n ( p u@1 - b o k s ) ((pos "K7%" ) ))
( "poor-boxes"  n ( p u@1 - b o k s i z ) ((pos "Kj%" ) ))
( "poor-rate"  n ( p u@1 - r ei t ) ((pos "K6$" ) ))
( "poor-rates"  n ( p u@1 - r ei t s ) ((pos "Kj$" ) ))
( "poor-spirited"  j ( p u@1 - s p i1 r i t i d ) ((pos "OA%" ) ))
( "poorer"  j ( p u@1 r @ r ) ((pos "Or%" ) ))
( "poorest"  j ( p u@1 r i s t ) ((pos "Os%" ) ))
( "poorhouse"  n ( p u@1 h au s ) ((pos "K6%" ) ))
( "poorhouses"  n ( p u@1 h au z i z ) ((pos "Kj%" ) ))
( "poorly"  j ( p u@1 l ii ) ((pos "Op%" "Pu%" ) ))
( "poorness"  n ( p u@1 n @ s ) ((pos "L@%" ) ))
( "pop"  v ( p o1 p ) ((pos "J4%" "M6%" "OA%" "Pu%" "Y>$" ) (subcat "2A" "2C" "6A" "15A" )))
( "popcorn"  n ( p o1 p k oo n ) ((pos "K6%" ) ))
( "popcorns"  n ( p o1 p k oo n z ) ((pos "Kj$" ) ))
( "popery"  n ( p ou1 p @ r ii ) ((pos "L@%" ) ))
( "popeyed"  j ( p o1 p ai d ) ((pos "OA%" ) ))
( "popgun"  n ( p o1 p g uh n ) ((pos "K6%" ) ))
( "popguns"  n ( p o1 p g uh n z ) ((pos "Kj%" ) ))
( "popinjay"  n ( p o1 p i n jh ei ) ((pos "K6$" ) ))
( "popinjays"  n ( p o1 p i n jh ei z ) ((pos "Kj$" ) ))
( "popish"  j ( p ou1 p i sh ) ((pos "OA%" ) ))
( "popishly"  a ( p ou1 p i sh l ii ) ((pos "Pu%" ) ))
( "poplar"  n ( p o1 p l @ r ) ((pos "M6%" ) ))
( "poplars"  n ( p o1 p l @ z ) ((pos "Mj%" ) ))
( "poplin"  n ( p o1 p l i n ) ((pos "L@%" ) ))
( "poppa"  n ( p o1 p @ ) ((pos "K6$" ) ))
( "poppas"  n ( p o1 p @ z ) ((pos "Kj$" ) ))
( "popped"  v ( p o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "poppet"  n ( p o1 p i t ) ((pos "K6%" ) ))
( "poppets"  n ( p o1 p i t s ) ((pos "Kj%" ) ))
( "poppies"  n ( p o1 p i z ) ((pos "Kj%" ) ))
( "popping"  v ( p o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "poppy"  n ( p o1 p ii ) ((pos "K8%" ) ))
( "poppycock"  n ( p o1 p i k o k ) ((pos "L@%" ) ))
( "pops"  v ( p o1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "populace"  n ( p o1 p y u l @ s ) ((pos "K6%" ) ))
( "populaces"  n ( p o1 p y u l @ s i z ) ((pos "Kj$" ) ))
( "popular"  j ( p o1 p y u l @ r ) ((pos "OA%" ) ))
( "popularity"  n ( p o2 p y u l a1 r i t ii ) ((pos "L@%" ) ))
( "popularization"  n ( p o2 p y u l @ r ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "popularizations"  n ( p o2 p y u l @ r ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "popularize"  v ( p o1 p y u l @ r ai z ) ((pos "H2%" ) (subcat )))
( "popularized"  v ( p o1 p y u l @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "popularizes"  v ( p o1 p y u l @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "popularizing"  v ( p o1 p y u l @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "popularly"  a ( p o1 p y u l @ l ii ) ((pos "Pu%" ) ))
( "populate"  v ( p o1 p y u l ei t ) ((pos "H2%" ) (subcat )))
( "populated"  v ( p o1 p y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "populates"  v ( p o1 p y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "populating"  v ( p o1 p y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "population"  n ( p o2 p y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "populations"  n ( p o2 p y u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "populism"  n ( p o1 p y u l i z @ m ) ((pos "L@$" ) ))
( "populist"  n ( p o1 p y u l i s t ) ((pos "K6$" ) ))
( "populists"  n ( p o1 p y u l i s t s ) ((pos "Kj$" ) ))
( "populous"  j ( p o1 p y u l @ s ) ((pos "OA%" ) ))
( "porcelain"  n ( p oo1 s @ l i n ) ((pos "L@%" ) ))
( "porch"  n ( p oo1 ch ) ((pos "K7%" ) ))
( "porches"  n ( p oo1 ch i z ) ((pos "Kj%" ) ))
( "porcine"  j ( p oo1 s ai n ) ((pos "OA$" ) ))
( "porcupine"  n ( p oo1 k y u p ai n ) ((pos "K6%" ) ))
( "porcupines"  n ( p oo1 k y u p ai n z ) ((pos "Kj%" ) ))
( "pore"  v ( p oo1 r ) ((pos "I2%" "K6%" ) (subcat )))
( "pored"  v ( p oo1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "pores"  v ( p oo1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "poring"  v ( p oo1 r i ng ) ((pos "Ib%" ) (subcat )))
( "pork"  n ( p oo1 k ) ((pos "L@%" ) ))
( "pork-barrel"  n ( p oo1 k - b a r @ l ) ((pos "K6$" ) ))
( "pork-barrels"  n ( p oo1 k - b a r @ l z ) ((pos "Kj$" ) ))
( "pork-butcher"  n ( p oo1 k - b u ch @ r ) ((pos "K6%" ) ))
( "pork-butchers"  n ( p oo1 k - b u ch @ z ) ((pos "Kj%" ) ))
( "porker"  n ( p oo1 k @ r ) ((pos "K6%" ) ))
( "porkers"  n ( p oo1 k @ z ) ((pos "Kj%" ) ))
( "porn"  n ( p oo1 n ) ((pos "L@%" ) ))
( "pornographer"  n ( p oo1 n o1 g r @ f @ r ) ((pos "K6%" ) ))
( "pornographers"  n ( p oo1 n o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "pornographic"  j ( p oo2 n @ g r a1 f i k ) ((pos "OA%" ) ))
( "pornography"  n ( p oo1 n o1 g r @ f ii ) ((pos "L@%" ) ))
( "porosities"  n ( p oo1 r o1 s i t i z ) ((pos "Mj$" ) ))
( "porosity"  n ( p oo1 r o1 s i t ii ) ((pos "M8%" ) ))
( "porous"  j ( p oo1 r @ s ) ((pos "OA%" ) ))
( "porousness"  n ( p oo1 r @ s n @ s ) ((pos "L@%" ) ))
( "porphyry"  n ( p oo1 f i r ii ) ((pos "L@$" ) ))
( "porpoise"  n ( p oo1 p @ s ) ((pos "K6%" ) ))
( "porpoises"  n ( p oo1 p @ s i z ) ((pos "Kj%" ) ))
( "porridge"  n ( p o1 r i jh ) ((pos "L@%" ) ))
( "porringer"  n ( p o1 r i n jh @ r ) ((pos "K6$" ) ))
( "porringers"  n ( p o1 r i n jh @ z ) ((pos "Kj$" ) ))
( "port"  v ( p oo1 t ) ((pos "H0%" "M6%" ) (subcat )))
( "portability"  n ( p oo2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "portable"  j ( p oo1 t @ b l ) ((pos "OA%" ) ))
( "portage"  n ( p oo1 t i jh ) ((pos "M6$" ) ))
( "portages"  n ( p oo1 t i jh i z ) ((pos "Mj$" ) ))
( "portal"  n ( p oo1 t l ) ((pos "K6%" ) ))
( "portals"  n ( p oo1 t l z ) ((pos "Kj%" ) ))
( "portcullis"  n ( p oo2 t k uh1 l i s ) ((pos "K7%" ) ))
( "portcullises"  n ( p oo2 t k uh1 l i s i z ) ((pos "Kj%" ) ))
( "porte-coch`ere"  n ( p oo2 t - k @ sh e@1 r ) ((pos "K6$" ) ))
( "porte-coch`eres"  n ( p oo2 t - k @ sh e@1 z ) ((pos "Kj$" ) ))
( "ported"  v ( p oo1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "portend"  v ( p oo1 t e1 n d ) ((pos "H0$" ) (subcat )))
( "portended"  v ( p oo1 t e1 n d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "portending"  v ( p oo1 t e1 n d i ng ) ((pos "Hb$" ) (subcat )))
( "portends"  v ( p oo1 t e1 n d z ) ((pos "Ha$" ) (subcat )))
( "portent"  n ( p oo1 t e n t ) ((pos "K6%" ) ))
( "portentous"  j ( p oo1 t e1 n t @ s ) ((pos "OA%" ) ))
( "portentously"  a ( p oo1 t e1 n t @ s l ii ) ((pos "Pu%" ) ))
( "portents"  n ( p oo1 t e n t s ) ((pos "Kj%" ) ))
( "porter"  n ( p oo1 t @ r ) ((pos "M6%" ) ))
( "porterage"  n ( p oo1 t @ r i jh ) ((pos "L@$" ) ))
( "porterhouse"  n ( p oo1 t @ h au s ) ((pos "K6$" ) ))
( "porterhouses"  n ( p oo1 t @ h au z i z ) ((pos "Kj$" ) ))
( "porters"  n ( p oo1 t @ z ) ((pos "Mj%" ) ))
( "portfolio"  n ( p oo1 t f ou1 l i@ @ ) ((pos "K6%" ) ))
( "portfolios"  n ( p oo1 t f ou1 l i@ u z ) ((pos "Kj%" ) ))
( "porthole"  n ( p oo1 t h ou l ) ((pos "K6%" ) ))
( "portholes"  n ( p oo1 t h ou l z ) ((pos "Kj%" ) ))
( "porti`ere"  n ( p oo2 t i e@1 r ) ((pos "K6$" ) ))
( "porti`eres"  n ( p oo2 t i e@1 z ) ((pos "Kj$" ) ))
( "portico"  n ( p oo1 t i k ou ) ((pos "K6%" ) ))
( "porticoes"  n ( p oo1 t ii k ou z ) ((pos "Kj%" ) ))
( "porticos"  n ( p oo1 t i k ou z ) ((pos "Kj%" ) ))
( "porting"  v ( p oo1 t i ng ) ((pos "Hb$" ) (subcat )))
( "portion"  v ( p oo1 sh @ n ) ((pos "H0%" "K6%" ) (subcat "14" )))
( "portioned"  v ( p oo1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "portioning"  v ( p oo1 sh @ n i ng ) ((pos "Hb%" ) (subcat "14" )))
( "portions"  v ( p oo1 sh @ n z ) ((pos "Ha%" "Kj%" ) (subcat "14" )))
( "portly"  j ( p oo1 t l ii ) ((pos "OA%" ) ))
( "portmanteau"  n ( p oo1 t m a1 n t ou ) ((pos "K6%" ) ))
( "portmanteaus"  n ( p oo1 t m a1 n t ou z ) ((pos "Kj%" ) ))
( "portmanteaux"  n ( p oo1 t m a1 n t ou z ) ((pos "Kj$" ) ))
( "portrait"  n ( p oo1 t r i t ) ((pos "K6%" ) ))
( "portraitist"  n ( p oo1 t r i t i s t ) ((pos "K6%" ) ))
( "portraitists"  n ( p oo1 t r i t i s t s ) ((pos "Kj%" ) ))
( "portraits"  n ( p oo1 t r i t s ) ((pos "Kj%" ) ))
( "portraiture"  n ( p oo1 t r ei ch @ r ) ((pos "L@%" ) ))
( "portray"  v ( p oo1 t r ei1 ) ((pos "H0%" ) (subcat )))
( "portrayal"  n ( p oo1 t r ei1 @ l ) ((pos "M6%" ) ))
( "portrayals"  n ( p oo1 t r ei1 @ l z ) ((pos "Mj%" ) ))
( "portrayed"  v ( p oo1 t r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "portraying"  v ( p oo1 t r ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "portrays"  v ( p oo1 t r ei1 z ) ((pos "Ha%" ) (subcat )))
( "ports"  v ( p oo1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "pose"  v ( p ou1 z ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "posed"  v ( p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "poser"  n ( p ou1 z @ r ) ((pos "K6%" ) ))
( "posers"  n ( p ou1 z @ z ) ((pos "Kj%" ) ))
( "poses"  v ( p ou1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "poseur"  n ( p ou1 z @@1 r ) ((pos "K6%" ) ))
( "poseurs"  n ( p ou1 z @@1 z ) ((pos "Kj%" ) ))
( "poseuse"  n ( p ou1 z @@1 z ) ((pos "Ki$" ) ))
( "posh"  v ( p o1 sh ) ((pos "H1$" "OC%" ) (subcat )))
( "poshed"  v ( p o1 sh t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "posher"  j ( p o1 sh @ r ) ((pos "Or%" ) ))
( "poshes"  v ( p o1 sh i z ) ((pos "Ha$" ) (subcat )))
( "poshest"  j ( p o1 sh i s t ) ((pos "Os%" ) ))
( "poshing"  v ( p o1 sh i ng ) ((pos "Hb$" ) (subcat )))
( "posies"  n ( p ou1 z i z ) ((pos "Kj%" ) ))
( "posing"  v ( p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "posit"  v ( p o1 z i t ) ((pos "H0$" ) (subcat )))
( "posited"  v ( p o1 z i t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "positing"  v ( p o1 z i t i ng ) ((pos "Hb$" ) (subcat )))
( "position"  v ( p @1 z i1 sh @ n ) ((pos "H0%" "M6%" ) (subcat )))
( "positioned"  v ( p @1 z i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "positioning"  v ( p @1 z i1 sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "positions"  v ( p @1 z i1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "positive"  n ( p o1 z @ t i v ) ((pos "K6%" "OA%" ) ))
( "positively"  a ( p o1 z @ t i v l ii ) ((pos "Pu%" ) ))
( "positiveness"  n ( p o1 z @ t i v n @ s ) ((pos "L@%" ) ))
( "positives"  n ( p o1 z @ t i v z ) ((pos "Kj%" ) ))
( "positivism"  n ( p o1 z i t i v i z @ m ) ((pos "L@%" ) ))
( "positivist"  n ( p o1 s i t i v i s t ) ((pos "K6%" ) ))
( "positivists"  n ( p o1 s i t i v i s t s ) ((pos "Kj%" ) ))
( "posits"  v ( p o1 z i t s ) ((pos "Ha$" ) (subcat )))
( "posse"  n ( p o1 s ii ) ((pos "K6$" ) ))
( "posses"  n ( p o1 s i z ) ((pos "Kj$" ) ))
( "possess"  v ( p @1 z e1 s ) ((pos "H1%" ) (subcat )))
( "possessed"  v ( p @1 z e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "possesses"  v ( p @1 z e1 s i z ) ((pos "Ha%" ) (subcat )))
( "possessing"  v ( p @1 z e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "possession"  n ( p @1 z e1 sh @ n ) ((pos "M6%" ) ))
( "possessions"  n ( p @1 z e1 sh @ n z ) ((pos "Mj%" ) ))
( "possessive"  j ( p @1 z e1 s i v ) ((pos "OA%" ) ))
( "possessively"  a ( p @1 z e1 s i v l ii ) ((pos "Pu%" ) ))
( "possessiveness"  n ( p @1 z e1 s i v n i s ) ((pos "L@%" ) ))
( "possessor"  n ( p @1 z e1 s @ r ) ((pos "K6%" ) ))
( "possessors"  n ( p @1 z e1 s @ z ) ((pos "Kj%" ) ))
( "posset"  n ( p o1 s i t ) ((pos "K6$" ) ))
( "possets"  n ( p o1 s i t s ) ((pos "Kj$" ) ))
( "possibilities"  n ( p o2 s @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "possibility"  n ( p o2 s @ b i1 l i t ii ) ((pos "M8%" ) ))
( "possible"  n ( p o1 s @ b l ) ((pos "K6%" "OA*" ) ))
( "possibles"  n ( p o1 s @ b l z ) ((pos "Kj%" ) ))
( "possibly"  a ( p o1 s @ b l ii ) ((pos "Pu%" ) ))
( "possum"  n ( p o1 s @ m ) ((pos "K6$" ) ))
( "possums"  n ( p o1 s @ m z ) ((pos "Kj$" ) ))
( "post"  v ( p ou1 s t ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" "15A" )))
( "post meridiem"  a ( p ou2 s t - m @ r i1 d i@ m ) ((pos "Pu$" ) ))
( "post-"  j ( p ou s t - ) ((pos "U-%" ) ))
( "post-chaise"  n ( p ou1 s t - sh ei1 z ) ((pos "K6$" ) ))
( "post-chaises"  n ( p ou1 s t - sh ei1 z i z ) ((pos "Kj$" ) ))
( "post-free"  j ( p ou1 s t - f r ii1 ) ((pos "OA%" "Pu%" ) ))
( "post-haste"  a ( p ou1 s t - h ei1 s t ) ((pos "Pu%" ) ))
( "post-horse"  n ( p ou1 s t - h oo s ) ((pos "K6$" ) ))
( "post-horses"  n ( p ou1 s t - h oo s i z ) ((pos "Kj$" ) ))
( "post-mortem"  n ( p ou2 s t - m oo1 t @ m ) ((pos "K6%" "OA%" ) ))
( "post-mortems"  n ( p ou2 s t - m oo1 t @ m z ) ((pos "Kj%" ) ))
( "post-paid"  j ( p ou1 s t - p ei1 d ) ((pos "OA%" "Pu%" ) ))
( "postage"  n ( p ou1 s t i jh ) ((pos "L@%" ) ))
( "postage-stamp"  n ( p ou1 s t i jh - s t a m p ) ((pos "K6%" ) ))
( "postage-stamps"  n ( p ou1 s t i jh - s t a m p s ) ((pos "Kj%" ) ))
( "postal"  j ( p ou1 s t l ) ((pos "OA%" ) ))
( "postbag"  n ( p ou1 s t b a g ) ((pos "K6%" ) ))
( "postbags"  n ( p ou1 s t b a g z ) ((pos "Kj%" ) ))
( "postbox"  n ( p ou1 s t b o k s ) ((pos "K7%" ) ))
( "postboxes"  n ( p ou1 s t b o k s i z ) ((pos "Kj%" ) ))
( "postcard"  n ( p ou1 s t k aa d ) ((pos "K6%" ) ))
( "postcards"  n ( p ou1 s t k aa d z ) ((pos "Kj%" ) ))
( "postcode"  n ( p ou1 s t k ou d ) ((pos "K6%" ) ))
( "postcodes"  n ( p ou1 s t k ou d z ) ((pos "Kj%" ) ))
( "postdate"  v ( p ou2 s t d ei1 t ) ((pos "H2%" ) (subcat )))
( "postdated"  v ( p ou2 s t d ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "postdates"  v ( p ou2 s t d ei1 t s ) ((pos "Ha%" ) (subcat )))
( "postdating"  v ( p ou2 s t d ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "poste restante"  n ( p ou2 s t - r e1 s t aa n t ) ((pos "L@%" ) ))
( "posted"  v ( p ou1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "15A" )))
( "poster"  n ( p ou1 s t @ r ) ((pos "K6%" ) ))
( "posterior"  n ( p o1 s t i@1 r i@ r ) ((pos "K6%" "OA%" ) ))
( "posteriors"  n ( p o1 s t i@1 r i@ z ) ((pos "Kj%" ) ))
( "posterity"  n ( p o1 s t e1 r i t ii ) ((pos "L@%" ) ))
( "postern"  n ( p o1 s t @ n ) ((pos "K6$" ) ))
( "posterns"  n ( p o1 s t @ n z ) ((pos "Kj$" ) ))
( "posters"  n ( p ou1 s t @ z ) ((pos "Kj%" ) ))
( "postgraduate"  n ( p ou2 s t g r a1 jh u@ t ) ((pos "K6%" "OA%" ) ))
( "postgraduates"  n ( p ou2 s t g r a1 jh u@ t s ) ((pos "Kj%" ) ))
( "posthumous"  j ( p o1 s t y u m @ s ) ((pos "OA%" ) ))
( "posthumously"  a ( p o1 s t y u m @ s l ii ) ((pos "Pu%" ) ))
( "postilion"  n ( p o1 s t i1 l i@ n ) ((pos "K6$" ) ))
( "postilions"  n ( p o1 s t i1 l i@ n z ) ((pos "Kj$" ) ))
( "postillion"  n ( p o1 s t i1 l i@ n ) ((pos "K6$" ) ))
( "postillions"  n ( p o1 s t i1 l i@ n z ) ((pos "Kj$" ) ))
( "posting"  v ( p ou1 s t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "15A" )))
( "postman"  n ( p ou1 s t m @ n ) ((pos "Ki%" ) ))
( "postmark"  v ( p ou1 s t m aa k ) ((pos "H0%" "K6%" ) ))
( "postmarked"  v ( p ou1 s t m aa k t ) ((pos "Hc%" "Hd%" ) ))
( "postmarking"  v ( p ou1 s t m aa k i ng ) ((pos "Hb%" ) ))
( "postmarks"  v ( p ou1 s t m aa k s ) ((pos "Ha%" "Kj%" ) ))
( "postmaster"  n ( p ou1 s t m aa s t @ r ) ((pos "K6%" ) ))
( "postmasters"  n ( p ou1 s t m aa s t @ z ) ((pos "Kj%" ) ))
( "postmen"  n ( p ou1 s t m @ n ) ((pos "Kj%" ) ))
( "postmistress"  n ( p ou1 s t m i s t r i s ) ((pos "K7%" ) ))
( "postmistresses"  n ( p ou1 s t m i s t r i s i z ) ((pos "Kj%" ) ))
( "postpone"  v ( p @1 s p ou1 n ) ((pos "H2%" ) (subcat "6A" )))
( "postponed"  v ( p @1 s p ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "postponement"  n ( p @1 s p ou1 n m @ n t ) ((pos "M6%" ) ))
( "postponements"  n ( p @1 s p ou1 n m @ n t s ) ((pos "Mj%" ) ))
( "postpones"  v ( p @1 s p ou1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "postponing"  v ( p @1 s p ou1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "postprandial"  j ( p ou2 s t p r a1 n d i@ l ) ((pos "OA$" ) ))
( "posts"  v ( p ou1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "15A" )))
( "postscript"  n ( p ou1 s - s k r i p t ) ((pos "K6%" ) ))
( "postscripts"  n ( p ou1 s - s k r i p t s ) ((pos "Kj%" ) ))
( "postulant"  n ( p o1 s t y u l @ n t ) ((pos "K6$" ) ))
( "postulants"  n ( p o1 s t y u l @ n t s ) ((pos "Kj$" ) ))
( "postulate"  v ( p o1 s t y u l ei t ) ((pos "H2%" "K6%" ) (subcat )))
( "postulated"  v ( p o1 s t y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "postulates"  v ( p o1 s t y u l ei t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "postulating"  v ( p o1 s t y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "postural"  j ( p o1 s ch @ r @ l ) ((pos "OA%" ) ))
( "posture"  v ( p o1 s ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "postured"  v ( p o1 s ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "postures"  v ( p o1 s ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "posturing"  v ( p o1 s ch @ r i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "posturings"  n ( p o1 s ch @ r i ng z ) ((pos "Mj%" ) ))
( "posy"  n ( p ou1 z ii ) ((pos "K8%" ) ))
( "pot"  v ( p o1 t ) ((pos "J4%" "K6%" ) (subcat "3A" "6A" )))
( "pot-shot"  n ( p o1 t - sh o t ) ((pos "K6%" ) ))
( "pot-shots"  n ( p o1 t - sh o t s ) ((pos "Kj%" ) ))
( "pot-trained"  j ( p o1 t - t r ei n d ) ((pos "OA%" ) ))
( "potable"  j ( p ou1 t @ b l ) ((pos "OA$" ) ))
( "potash"  n ( p o1 t a sh ) ((pos "L@%" ) ))
( "potassium"  n ( p @1 t a1 s i@ m ) ((pos "L@%" ) ))
( "potation"  n ( p ou1 t ei1 sh @ n ) ((pos "K6$" ) ))
( "potations"  n ( p ou1 t ei1 sh @ n z ) ((pos "Kj$" ) ))
( "potato"  n ( p @1 t ei1 t ou ) ((pos "K7%" ) ))
( "potatoes"  n ( p @1 t ei1 t ou z ) ((pos "Kj%" ) ))
( "potbellied"  j ( p o1 t b e1 l i d ) ((pos "OA%" ) ))
( "potbellies"  n ( p o1 t b e1 l i z ) ((pos "Kj%" ) ))
( "potbelly"  n ( p o1 t b e1 l ii ) ((pos "K8%" ) ))
( "potboiler"  n ( p o1 t b oi l @ r ) ((pos "K6%" ) ))
( "potboilers"  n ( p o1 t b oi l @ z ) ((pos "Kj%" ) ))
( "potbound"  j ( p o1 t b au n d ) ((pos "OA$" ) ))
( "potboy"  n ( p o1 t b oi ) ((pos "K6$" ) ))
( "potboys"  n ( p o1 t b oi z ) ((pos "Kj$" ) ))
( "poteen"  n ( p o1 t ii1 n ) ((pos "L@$" ) ))
( "potencies"  n ( p ou1 t n s i z ) ((pos "Mj$" ) ))
( "potency"  n ( p ou1 t n s ii ) ((pos "M8%" ) ))
( "potent"  j ( p ou1 t n t ) ((pos "OA%" ) ))
( "potentate"  n ( p ou1 t n t ei t ) ((pos "K6%" ) ))
( "potentates"  n ( p ou1 t n t ei t s ) ((pos "Kj%" ) ))
( "potential"  n ( p @1 t e1 n sh l ) ((pos "M6%" "OA%" ) ))
( "potentialities"  n ( p @1 t e2 n sh i a1 l i t i z ) ((pos "Mj%" ) ))
( "potentiality"  n ( p @1 t e2 n sh i a1 l i t ii ) ((pos "M8%" ) ))
( "potentially"  a ( p @1 t e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "potentials"  n ( p @1 t e1 n sh l z ) ((pos "Mj%" ) ))
( "potently"  a ( p ou1 t n t l ii ) ((pos "Pu%" ) ))
( "pothead"  n ( p o1 t h e d ) ((pos "K6$" ) ))
( "potheads"  n ( p o1 t h e d z ) ((pos "Kj$" ) ))
( "pother"  n ( p o1 dh @ r ) ((pos "K6$" ) ))
( "potherb"  n ( p o1 t h @@ b ) ((pos "K6$" ) ))
( "potherbs"  n ( p o1 t h @@ b z ) ((pos "Kj$" ) ))
( "pothers"  n ( p o1 dh @ z ) ((pos "Kj$" ) ))
( "pothole"  n ( p o1 t h ou l ) ((pos "K6%" ) ))
( "potholer"  n ( p o1 t h ou l @ r ) ((pos "K6%" ) ))
( "potholers"  n ( p o1 t h ou l @ z ) ((pos "Kj%" ) ))
( "potholes"  n ( p o1 t h ou l z ) ((pos "Kj%" ) ))
( "pothook"  n ( p o1 t h u k ) ((pos "K6$" ) ))
( "pothooks"  n ( p o1 t h u k s ) ((pos "Kj$" ) ))
( "pothouse"  n ( p o1 t h au s ) ((pos "K6$" ) ))
( "pothouses"  n ( p o1 t h au z i z ) ((pos "Kj$" ) ))
( "pothunter"  n ( p o1 t h uh n t @ r ) ((pos "K6$" ) ))
( "pothunters"  n ( p o1 t h uh n t @ z ) ((pos "Kj$" ) ))
( "potion"  n ( p ou1 sh @ n ) ((pos "K6%" ) ))
( "potions"  n ( p ou1 sh @ n z ) ((pos "Kj%" ) ))
( "potman"  n ( p o1 t m a n ) ((pos "Ki$" ) ))
( "potmen"  n ( p o1 t m e n ) ((pos "Kj$" ) ))
( "potpourri"  n ( p ou2 p u@1 r ii ) ((pos "K6%" ) ))
( "potpourris"  n ( p ou2 p u@1 r ii z ) ((pos "Kj$" ) ))
( "pots"  v ( p o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "potsherd"  n ( p o1 t - sh @@ d ) ((pos "K6$" ) ))
( "potsherds"  n ( p o1 t - sh @@ d z ) ((pos "Kj$" ) ))
( "pottage"  n ( p o1 t i jh ) ((pos "M6$" ) ))
( "pottages"  n ( p o1 t i jh i z ) ((pos "Mj$" ) ))
( "potted"  v ( p o1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" )))
( "potter"  v ( p o1 t @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "pottered"  v ( p o1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "potterer"  n ( p o1 t @ r @ r ) ((pos "K6%" ) ))
( "potterers"  n ( p o1 t @ r @ z ) ((pos "Kj%" ) ))
( "potteries"  n ( p o1 t @ r i z ) ((pos "Mj%" ) ))
( "pottering"  v ( p o1 t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "potters"  v ( p o1 t @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "pottery"  n ( p o1 t @ r ii ) ((pos "M8%" ) ))
( "pottier"  j ( p o1 t i@ r ) ((pos "Or%" ) ))
( "potties"  n ( p o1 t i z ) ((pos "Kj%" ) ))
( "pottiest"  j ( p o1 t i i s t ) ((pos "Os%" ) ))
( "potting"  v ( p o1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "potty"  n ( p o1 t ii ) ((pos "K8%" "OD%" ) ))
( "pouch"  v ( p au1 ch ) ((pos "H1%" "K7%" ) (subcat "2A" )))
( "pouched"  v ( p au1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "pouches"  v ( p au1 ch i z ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "pouching"  v ( p au1 ch i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "pouf"  n ( p uu1 f ) ((pos "K6$" ) ))
( "pouffe"  n ( p uu1 f ) ((pos "K6$" ) ))
( "pouffes"  n ( p uu1 f s ) ((pos "Kj$" ) ))
( "poufs"  n ( p uu1 f s ) ((pos "Kj$" ) ))
( "poulterer"  n ( p ou1 l t @ r @ r ) ((pos "K6%" ) ))
( "poulterers"  n ( p ou1 l t @ r @ z ) ((pos "Kj%" ) ))
( "poultice"  v ( p ou1 l t i s ) ((pos "H2$" "K6%" ) (subcat )))
( "poulticed"  v ( p ou1 l t i s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "poultices"  v ( p ou1 l t i s i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "poulticing"  v ( p ou1 l t i s i ng ) ((pos "Hb$" ) (subcat )))
( "poultry"  n ( p ou1 l t r ii ) ((pos "Kj%" ) ))
( "pounce"  v ( p au1 n s ) ((pos "I2%" "K6%" ) (subcat )))
( "pounced"  v ( p au1 n s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "pounces"  v ( p au1 n s i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "pouncing"  v ( p au1 n s i ng ) ((pos "Ib%" ) (subcat )))
( "pound"  v ( p au1 n d ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "poundage"  n ( p au1 n d i jh ) ((pos "L@%" ) ))
( "pounded"  v ( p au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "pounder"  n ( p au1 n d @ r ) ((pos "K6$" ) ))
( "pounders"  n ( p au1 n d @ z ) ((pos "Kj$" ) ))
( "pounding"  v ( p au1 n d i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "pounds"  v ( p au1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "pour"  v ( p oo1 r ) ((pos "J0%" ) (subcat "2C" "6A" "12B" "13B" "14" "15A" )))
( "poured"  v ( p oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "12B" "13B" "14" "15A" )))
( "pouring"  v ( p oo1 r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "12B" "13B" "14" "15A" )))
( "pours"  v ( p oo1 z ) ((pos "Ja%" ) (subcat "2C" "6A" "12B" "13B" "14" "15A" )))
( "pout"  v ( p au1 t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "pouted"  v ( p au1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pouting"  v ( p au1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "poutingly"  a ( p au1 t i ng l ii ) ((pos "Pu%" ) ))
( "pouts"  v ( p au1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "poverty"  n ( p o1 v @ t ii ) ((pos "L@%" ) ))
( "poverty-stricken"  j ( p o1 v @ t i - s t r i k @ n ) ((pos "OA%" ) ))
( "powder"  v ( p au1 d @ r ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "powder-flask"  n ( p au1 d @ - f l aa s k ) ((pos "K6%" ) ))
( "powder-flasks"  n ( p au1 d @ - f l aa s k s ) ((pos "Kj%" ) ))
( "powder-horn"  n ( p au1 d @ - h oo n ) ((pos "K6%" ) ))
( "powder-horns"  n ( p au1 d @ - h oo n z ) ((pos "Kj%" ) ))
( "powder-magazine"  n ( p au1 d @ - m a g @ z ii n ) ((pos "K6%" ) ))
( "powder-magazines"  n ( p au1 d @ - m a g @ z ii n z ) ((pos "Kj%" ) ))
( "powder-puff"  n ( p au1 d @ - p uh f ) ((pos "K6%" ) ))
( "powder-puffs"  n ( p au1 d @ - p uh f s ) ((pos "Kj%" ) ))
( "powder-room"  n ( p au1 d @ - r uu m ) ((pos "K6%" ) ))
( "powder-rooms"  n ( p au1 d @ - r uu m z ) ((pos "Kj%" ) ))
( "powdered"  v ( p au1 d @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "powdering"  v ( p au1 d @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "powders"  v ( p au1 d @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "powdery"  j ( p au1 d @ r ii ) ((pos "OA%" ) ))
( "power"  v ( p au1 @ r ) ((pos "H0%" "M6*" ) (subcat )))
( "power-dive"  v ( p au1 @ - d ai v ) ((pos "H2$" "K6%" ) ))
( "power-dived"  v ( p au1 @ - d ai v d ) ((pos "Hc$" "Hd$" ) ))
( "power-dives"  v ( p au1 @ - d ai v z ) ((pos "Ha$" "Kj$" ) ))
( "power-diving"  v ( p au1 @ - d ai v i ng ) ((pos "Hb$" ) ))
( "power-point"  n ( p au1 @ - p oi n t ) ((pos "K6%" ) ))
( "power-points"  n ( p au1 @ - p oi n t s ) ((pos "Kj%" ) ))
( "power-station"  n ( p au1 @ - s t ei sh @ n ) ((pos "K6%" ) ))
( "power-stations"  n ( p au1 @ - s t ei sh @ n z ) ((pos "Kj%" ) ))
( "powerboat"  n ( p au1 @ b ou t ) ((pos "K6%" ) ))
( "powerboats"  n ( p au1 @ b ou t s ) ((pos "Kj%" ) ))
( "powered"  v ( p au1 @ d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "powerful"  j ( p au1 @ f @ l ) ((pos "OA%" ) ))
( "powerfully"  a ( p au1 @ f @ l ii ) ((pos "Pu%" ) ))
( "powerhouse"  n ( p au1 @ h au s ) ((pos "K6%" ) ))
( "powerhouses"  n ( p au1 @ h au z i z ) ((pos "Kj%" ) ))
( "powering"  v ( p au1 @ r i ng ) ((pos "Hb%" ) (subcat )))
( "powerless"  j ( p au1 @ l i s ) ((pos "OA%" ) ))
( "powerlessly"  a ( p au1 @ l i s l ii ) ((pos "Pu%" ) ))
( "powers"  v ( p au1 @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "powwow"  v ( p au1 w au ) ((pos "I0$" "K6$" ) ))
( "powwowed"  v ( p au1 w au d ) ((pos "Ic$" "Id$" ) ))
( "powwowing"  v ( p au1 w au i ng ) ((pos "Ib$" ) ))
( "powwows"  v ( p au1 w au z ) ((pos "Ia$" "Kj$" ) ))
( "pox"  n ( p o1 k s ) ((pos "Ki%" ) ))
( "pp"  n ( p ei1 jh i z ) ((pos "Y)%" ) ))
( "pr_ecis"  v ( p r ei1 s ii ) ((pos "H5%" "K9%" ) ))
( "practicability"  n ( p r a2 k t i k @ b i1 l i t ii ) ((pos "L@%" ) ))
( "practicable"  j ( p r a1 k t i k @ b l ) ((pos "OA%" ) ))
( "practicably"  a ( p r a1 k t i k @ b l ii ) ((pos "Pu%" ) ))
( "practical"  j ( p r a1 k t i k l ) ((pos "OA%" ) ))
( "practicalities"  n ( p r a2 k t i k a1 l i t i z ) ((pos "Mj%" ) ))
( "practicality"  n ( p r a2 k t i k a1 l i t ii ) ((pos "M8%" ) ))
( "practically"  a ( p r a1 k t i k l ii ) ((pos "Pu%" ) ))
( "practice"  n ( p r a1 k t i s ) ((pos "M6%" ) ))
( "practices"  n ( p r a1 k t i s i z ) ((pos "Mj%" ) ))
( "practician"  n ( p r a1 k t i1 sh @ n ) ((pos "K6$" ) ))
( "practicians"  n ( p r a1 k t i1 sh @ n z ) ((pos "Kj$" ) ))
( "practise"  v ( p r a1 k t i s ) ((pos "J2%" ) (subcat "2A" "2B" "3A" "4A" "6A" )))
( "practised"  v ( p r a1 k t i s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "3A" "4A" "6A" )))
( "practises"  v ( p r a1 k t i s i z ) ((pos "Ja%" ) (subcat "2A" "2B" "3A" "4A" "6A" )))
( "practising"  v ( p r a1 k t i s i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "3A" "4A" "6A" )))
( "practitioner"  n ( p r a1 k t i1 sh @ n @ r ) ((pos "K6%" ) ))
( "practitioners"  n ( p r a1 k t i1 sh @ n @ z ) ((pos "Kj%" ) ))
( "praesidium"  n ( p r i1 s i1 d i@ m ) ((pos "K6$" ) ))
( "praesidiums"  n ( p r i1 s i1 d i@ m z ) ((pos "Kj$" ) ))
( "praetor"  n ( p r ii1 t @ r ) ((pos "K6$" ) ))
( "praetorian"  j ( p r i1 t oo1 r i@ n ) ((pos "OA$" ) ))
( "praetors"  n ( p r ii1 t @ z ) ((pos "Kj$" ) ))
( "pragmatic"  j ( p r a1 g m a1 t i k ) ((pos "OA%" ) ))
( "pragmatically"  a ( p r a1 g m a1 t i k l ii ) ((pos "Pu%" ) ))
( "pragmatism"  n ( p r a1 g m @ t i z @ m ) ((pos "L@%" ) ))
( "pragmatist"  n ( p r a1 g m @ t i s t ) ((pos "K6%" ) ))
( "pragmatists"  n ( p r a1 g m @ t i s t s ) ((pos "Kj%" ) ))
( "prairie"  n ( p r e@1 r ii ) ((pos "K6%" ) ))
( "prairies"  n ( p r e@1 r i z ) ((pos "Kj%" ) ))
( "praise"  v ( p r ei1 z ) ((pos "H2%" "M6%" ) (subcat )))
( "praised"  v ( p r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "praises"  v ( p r ei1 z i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "praiseworthily"  a ( p r ei1 z w @@ dh i l ii ) ((pos "Pu%" ) ))
( "praiseworthiness"  n ( p r ei1 z w @@ dh i n @ s ) ((pos "L@%" ) ))
( "praiseworthy"  j ( p r ei1 z w @@ dh ii ) ((pos "OA%" ) ))
( "praising"  v ( p r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "pram"  n ( p r a1 m ) ((pos "K6%" ) ))
( "prams"  n ( p r a1 m z ) ((pos "Kj%" ) ))
( "prance"  v ( p r aa1 n s ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "pranced"  v ( p r aa1 n s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "prances"  v ( p r aa1 n s i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "prancing"  v ( p r aa1 n s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "prank"  n ( p r a1 ng k ) ((pos "K6%" ) ))
( "pranks"  n ( p r a1 ng k s ) ((pos "Kj%" ) ))
( "prate"  v ( p r ei1 t ) ((pos "I2$" ) (subcat "2A" )))
( "prated"  v ( p r ei1 t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "prates"  v ( p r ei1 t s ) ((pos "Ia$" ) (subcat "2A" )))
( "prating"  v ( p r ei1 t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "prattle"  v ( p r a1 t l ) ((pos "I2%" "L@%" ) (subcat "2A" )))
( "prattled"  v ( p r a1 t l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "prattler"  n ( p r a1 t l @ r ) ((pos "K6%" ) ))
( "prattlers"  n ( p r a1 t l @ z ) ((pos "Kj%" ) ))
( "prattles"  v ( p r a1 t l z ) ((pos "Ia%" ) (subcat "2A" )))
( "prattling"  v ( p r a1 t l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "prawn"  v ( p r oo1 n ) ((pos "I0$" "K6%" ) ))
( "prawned"  v ( p r oo1 n d ) ((pos "Ic$" "Id$" ) ))
( "prawning"  v ( p r oo1 n i ng ) ((pos "Ib$" ) ))
( "prawns"  v ( p r oo1 n z ) ((pos "Ia$" "Kj%" ) ))
( "pray"  v ( p r ei1 ) ((pos "J0%" ) (subcat "2A" "3A" "11" "14" )))
( "prayed"  v ( p r ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "11" "14" )))
( "prayer"  n ( p r e@1 r ) ((pos "M6%" ) ))
( "prayer-book"  n ( p r e@1 - b u k ) ((pos "K6%" ) ))
( "prayer-books"  n ( p r e@1 - b u k s ) ((pos "Kj%" ) ))
( "prayer-mat"  n ( p r e@1 - m a t ) ((pos "K6%" ) ))
( "prayer-mats"  n ( p r e@1 - m a t s ) ((pos "Kj%" ) ))
( "prayer-meeting"  n ( p r e@1 - m ii t i ng ) ((pos "K6%" ) ))
( "prayer-meetings"  n ( p r e@1 - m ii t i ng z ) ((pos "Kj%" ) ))
( "prayer-rug"  n ( p r e@1 - r uh g ) ((pos "K6%" ) ))
( "prayer-rugs"  n ( p r e@1 - r uh g z ) ((pos "Kj%" ) ))
( "prayer-wheel"  n ( p r e@1 - w ii l ) ((pos "K6%" ) ))
( "prayer-wheels"  n ( p r e@1 - w ii l z ) ((pos "Kj%" ) ))
( "prayers"  n ( p r e@1 z ) ((pos "Mj%" ) ))
( "praying"  v ( p r ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "11" "14" )))
( "prays"  v ( p r ei1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "11" "14" )))
( "pre-"  j ( p r ii - ) ((pos "U-%" ) ))
( "pre-eminence"  n ( p r ii1 - e1 m i n @ n s ) ((pos "L@%" ) ))
( "pre-eminent"  j ( p r ii2 - e1 m i n @ n t ) ((pos "OA%" ) ))
( "pre-eminently"  a ( p r ii2 - e1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "pre-empt"  v ( p r ii2 - e1 m p t ) ((pos "H0%" ) (subcat )))
( "pre-empted"  v ( p r ii2 - e1 m p t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pre-empting"  v ( p r ii2 - e1 m p t i ng ) ((pos "Hb%" ) (subcat )))
( "pre-emption"  n ( p r ii2 - e1 m p sh @ n ) ((pos "L@%" ) ))
( "pre-emptive"  j ( p r ii1 - e1 m p t i v ) ((pos "OA%" ) ))
( "pre-empts"  v ( p r ii2 - e1 m p t s ) ((pos "Ha%" ) (subcat )))
( "pre-exist"  v ( p r ii2 - i g z i1 s t ) ((pos "I0%" ) (subcat )))
( "pre-existed"  v ( p r ii2 - i g z i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "pre-existence"  n ( p r ii2 - i g z i1 s t @ n s ) ((pos "K6%" ) ))
( "pre-existences"  n ( p r ii2 - i g z i1 s t @ n s i z ) ((pos "Kj$" ) ))
( "pre-existent"  j ( p r ii2 - i g z i1 s t @ n t ) ((pos "OA%" ) ))
( "pre-existing"  v ( p r ii2 - i g z i1 s t i ng ) ((pos "Ib%" ) (subcat )))
( "pre-exists"  v ( p r ii2 - i g z i1 s t s ) ((pos "Ia%" ) (subcat )))
( "pre-packaged"  j ( p r ii2 - p a1 k i jh d ) ((pos "OA%" ) ))
( "preach"  v ( p r ii1 ch ) ((pos "J1%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12A" )))
( "preached"  v ( p r ii1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12A" )))
( "preacher"  n ( p r ii1 ch @ r ) ((pos "K6%" ) ))
( "preachers"  n ( p r ii1 ch @ z ) ((pos "Kj%" ) ))
( "preaches"  v ( p r ii1 ch i z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12A" )))
( "preachified"  v ( p r ii1 ch i f ai d ) ((pos "Ic$" "Id$" ) ))
( "preachifies"  v ( p r ii1 ch i f ai z ) ((pos "Ia$" ) ))
( "preachify"  v ( p r ii1 ch i f ai ) ((pos "I3$" ) ))
( "preachifying"  v ( p r ii1 ch i f ai i ng ) ((pos "Ib$" ) ))
( "preaching"  v ( p r ii1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" "12A" )))
( "preamble"  n ( p r ii1 a1 m b l ) ((pos "K6%" ) ))
( "preambles"  n ( p r ii1 a1 m b l z ) ((pos "Kj%" ) ))
( "prearrange"  v ( p r ii2 @ r ei1 n jh ) ((pos "H2%" ) ))
( "prearranged"  v ( p r ii2 @ r ei1 n jh d ) ((pos "Hc%" "Hd%" ) ))
( "prearrangement"  n ( p r ii2 @ r ei1 n jh m @ n t ) ((pos "K6%" ) ))
( "prearrangements"  n ( p r ii2 @ r ei1 n jh m @ n t s ) ((pos "Kj%" ) ))
( "prearranges"  v ( p r ii2 @ r ei1 n jh i z ) ((pos "Ha%" ) ))
( "prearranging"  v ( p r ii2 @ r ei1 n jh i ng ) ((pos "Hb%" ) ))
( "prebend"  n ( p r e1 b @ n d ) ((pos "K6$" ) ))
( "prebendaries"  n ( p r e1 b @ n d r i z ) ((pos "Kj$" ) ))
( "prebendary"  n ( p r e1 b @ n d r ii ) ((pos "K8$" ) ))
( "prebends"  n ( p r e1 b @ n d z ) ((pos "Kj$" ) ))
( "precarious"  j ( p r i1 k e@1 r i@ s ) ((pos "OA%" ) ))
( "precariously"  a ( p r i1 k e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "precariousness"  n ( p r i1 k e@1 r i@ s n i s ) ((pos "L@%" ) ))
( "precast"  j ( p r ii2 k aa1 s t ) ((pos "OA%" ) ))
( "precaution"  n ( p r i1 k oo1 sh @ n ) ((pos "M6%" ) ))
( "precautionary"  j ( p r i2 k oo1 sh @ n @ r ii ) ((pos "OA%" ) ))
( "precautions"  n ( p r i1 k oo1 sh @ n z ) ((pos "Mj%" ) ))
( "precede"  v ( p r i1 s ii1 d ) ((pos "J2%" ) (subcat "2A" )))
( "preceded"  v ( p r i1 s ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "precedence"  n ( p r e1 s i d @ n s ) ((pos "L@%" ) ))
( "precedent"  n ( p r e1 s i d @ n t ) ((pos "K6%" ) ))
( "precedented"  j ( p r e1 s i d @ n t i d ) ((pos "OA%" ) ))
( "precedents"  n ( p r e1 s i d @ n t s ) ((pos "Kj%" ) ))
( "precedes"  v ( p r i1 s ii1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "preceding"  v ( p r i1 s ii1 d i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "precentor"  n ( p r i1 s e1 n t @ r ) ((pos "K6$" ) ))
( "precentors"  n ( p r i1 s e1 n t @ z ) ((pos "Kj$" ) ))
( "precept"  n ( p r ii1 s e p t ) ((pos "M6%" ) ))
( "preceptor"  n ( p r i1 s e1 p t @ r ) ((pos "K6$" ) ))
( "preceptors"  n ( p r i1 s e1 p t @ z ) ((pos "Kj$" ) ))
( "precepts"  n ( p r ii1 s e p t s ) ((pos "Mj%" ) ))
( "precession"  n ( p r i1 s e1 sh @ n ) ((pos "K6$" ) ))
( "precessions"  n ( p r i1 s e1 sh @ n z ) ((pos "Kj$" ) ))
( "precinct"  n ( p r ii1 s i ng k t ) ((pos "K6%" ) ))
( "precincts"  n ( p r ii1 s i ng k t s ) ((pos "Kj%" ) ))
( "preciosities"  n ( p r e2 sh i o1 s i t i z ) ((pos "Mj$" ) ))
( "preciosity"  n ( p r e2 sh i o1 s i t ii ) ((pos "M8$" ) ))
( "precious"  j ( p r e1 sh @ s ) ((pos "OA%" "Pu%" ) ))
( "preciously"  a ( p r e1 sh @ s l ii ) ((pos "Pu%" ) ))
( "preciousness"  n ( p r e1 sh @ s n @ s ) ((pos "L@%" ) ))
( "precipice"  n ( p r e1 s i p i s ) ((pos "K6%" ) ))
( "precipices"  n ( p r e1 s i p i s i z ) ((pos "Kj%" ) ))
( "precipitate"  j ( p r i1 s i1 p i t @ t ) ((pos "OA%" ) ))
( "precipitate"  v ( p r i1 s i1 p i t ei t ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "precipitated"  v ( p r i1 s i1 p i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "precipitately"  a ( p r i1 s i1 p i t @ t l ii ) ((pos "Pu%" ) ))
( "precipitates"  v ( p r i1 s i1 p i t ei t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "precipitating"  v ( p r i1 s i1 p i t ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "precipitation"  n ( p r i1 s i2 p i t ei1 sh @ n ) ((pos "L@%" ) ))
( "precipitous"  j ( p r i1 s i1 p i t @ s ) ((pos "OA%" ) ))
( "precipitously"  a ( p r i1 s i1 p i t @ s l ii ) ((pos "Pu%" ) ))
( "precise"  j ( p r i1 s ai1 s ) ((pos "OA%" ) ))
( "precised"  v ( p r ei1 s ii d ) ((pos "Hc$" "Hd$" ) ))
( "precisely"  a ( p r i1 s ai1 s l ii ) ((pos "Pu%" ) ))
( "preciseness"  n ( p r i1 s ai1 s n @ s ) ((pos "L@%" ) ))
( "precises"  v ( p r ei1 s ii z ) ((pos "Ha$" ) ))
( "precising"  v ( p r ei1 s ii i ng ) ((pos "Hb$" ) ))
( "precision"  n ( p r i1 s i1 zh n ) ((pos "L@%" ) ))
( "preclude"  v ( p r i1 k l uu1 d ) ((pos "H2%" ) (subcat "6A" "6C" )))
( "precluded"  v ( p r i1 k l uu1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "precludes"  v ( p r i1 k l uu1 d z ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "precluding"  v ( p r i1 k l uu1 d i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "preclusion"  n ( p r i1 k l uu1 zh n ) ((pos "K6$" ) ))
( "preclusions"  n ( p r i1 k l uu1 zh n z ) ((pos "Kj$" ) ))
( "precocious"  j ( p r i1 k ou1 sh @ s ) ((pos "OA%" ) ))
( "precociously"  a ( p r i1 k ou1 sh @ s l ii ) ((pos "Pu%" ) ))
( "precociousness"  n ( p r i1 k ou1 sh @ s n @ s ) ((pos "L@%" ) ))
( "precocity"  n ( p r i1 k o1 s i t ii ) ((pos "L@%" ) ))
( "precognition"  n ( p r ii2 k o g n i1 sh @ n ) ((pos "L@%" ) ))
( "preconceive"  v ( p r ii2 k @ n s ii1 v ) ((pos "H2%" ) (subcat )))
( "preconceived"  v ( p r ii2 k @ n s ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "preconceives"  v ( p r ii2 k @ n s ii1 v z ) ((pos "Ha%" ) (subcat )))
( "preconceiving"  v ( p r ii2 k @ n s ii1 v i ng ) ((pos "Hb%" ) (subcat )))
( "preconception"  n ( p r ii2 k @ n s e1 p sh @ n ) ((pos "M6%" ) ))
( "preconceptions"  n ( p r ii2 k @ n s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "preconcerted"  j ( p r ii2 k @ n s @@1 t i d ) ((pos "OA$" ) ))
( "precondition"  n ( p r ii2 k @ n d i1 sh @ n ) ((pos "K6%" ) ))
( "preconditioned"  j ( p r ii2 k @ n d i1 sh @ n d ) ((pos "OA%" ) ))
( "preconditions"  n ( p r ii2 k @ n d i1 sh @ n z ) ((pos "Kj%" ) ))
( "precursor"  n ( p r ii2 k @@1 s @ r ) ((pos "K6%" ) ))
( "precursors"  n ( p r ii2 k @@1 s @ z ) ((pos "Kj%" ) ))
( "precursory"  j ( p r ii2 k @@1 s @ r ii ) ((pos "OA$" ) ))
( "predator"  n ( p r e1 d @ t @ r ) ((pos "K6%" ) ))
( "predators"  n ( p r e1 d @ t @ z ) ((pos "Kj%" ) ))
( "predatory"  j ( p r e1 d @ t r ii ) ((pos "OA%" ) ))
( "predecease"  v ( p r ii2 d i s ii1 s ) ((pos "H2$" ) ))
( "predeceased"  v ( p r ii2 d i s ii1 s t ) ((pos "Hc$" "Hd$" ) ))
( "predeceases"  v ( p r ii2 d i s ii1 s i z ) ((pos "Ha$" ) ))
( "predeceasing"  v ( p r ii2 d i s ii1 s i ng ) ((pos "Hb$" ) ))
( "predecessor"  n ( p r ii1 d i s e s @ r ) ((pos "K6%" ) ))
( "predecessors"  n ( p r ii1 d i s e s @ z ) ((pos "Kj%" ) ))
( "predestinate"  v ( p r ii2 d e1 s t i n ei t ) ((pos "H2$" "OA$" ) ))
( "predestinated"  v ( p r ii2 d e1 s t i n ei t i d ) ((pos "Hc$" "Hd$" ) ))
( "predestinates"  v ( p r ii2 d e1 s t i n ei t s ) ((pos "Ha$" ) ))
( "predestinating"  v ( p r ii2 d e1 s t i n ei t i ng ) ((pos "Hb$" ) ))
( "predestination"  n ( p r ii2 d e s t i n ei1 sh @ n ) ((pos "K6$" ) ))
( "predestinations"  n ( p r ii2 d e s t i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "predestine"  v ( p r ii2 d e1 s t i n ) ((pos "H2%" ) (subcat "14" )))
( "predestined"  v ( p r ii2 d e1 s t i n d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "predestines"  v ( p r ii2 d e1 s t i n z ) ((pos "Ha%" ) (subcat "14" )))
( "predestining"  v ( p r ii2 d e1 s t i n i ng ) ((pos "Hb%" ) (subcat "14" )))
( "predetermination"  n ( p r ii2 d i t @@2 m i n ei1 sh @ n ) ((pos "K6$" ) ))
( "predeterminations"  n ( p r ii2 d i t @@2 m i n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "predetermine"  v ( p r ii2 d i t @@1 m i n ) ((pos "H2%" ) (subcat "6A" )))
( "predetermined"  v ( p r ii2 d i t @@1 m i n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "predetermines"  v ( p r ii2 d i t @@1 m i n z ) ((pos "Ha%" ) (subcat "6A" )))
( "predetermining"  v ( p r ii2 d i t @@1 m i n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "predicament"  n ( p r i1 d i1 k @ m @ n t ) ((pos "K6%" ) ))
( "predicaments"  n ( p r i1 d i1 k @ m @ n t s ) ((pos "Kj%" ) ))
( "predicate"  n ( p r e1 d i k @ t ) ((pos "K6%" ) ))
( "predicate"  v ( p r e1 d i k ei t ) ((pos "H2$" ) (subcat "6A" "9" )))
( "predicated"  v ( p r e1 d i k ei t i d ) ((pos "Hc$" "Hd%" ) (subcat "6A" "9" )))
( "predicates"  n ( p r e1 d i k @ t s ) ((pos "Kj%" ) ))
( "predicates"  v ( p r e1 d i k ei t s ) ((pos "Ha$" ) (subcat "6A" "9" )))
( "predicating"  v ( p r e1 d i k ei t i ng ) ((pos "Hb$" ) (subcat "6A" "9" )))
( "predicative"  j ( p r i1 d i1 k @ t i v ) ((pos "OA%" ) ))
( "predict"  v ( p r i1 d i1 k t ) ((pos "H0%" ) (subcat "6A" "9" )))
( "predictability"  n ( p r i1 d i2 k t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "predictable"  j ( p r i1 d i1 k t @ b l ) ((pos "OA%" ) ))
( "predictably"  a ( p r i1 d i1 k t @ b l ii ) ((pos "Pu%" ) ))
( "predicted"  v ( p r i1 d i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "predicting"  v ( p r i1 d i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "prediction"  n ( p r i1 d i1 k sh @ n ) ((pos "M6%" ) ))
( "predictions"  n ( p r i1 d i1 k sh @ n z ) ((pos "Mj%" ) ))
( "predictor"  n ( p r i1 d i1 k t @ r ) ((pos "K6%" ) ))
( "predictors"  n ( p r i1 d i1 k t @ z ) ((pos "Kj%" ) ))
( "predicts"  v ( p r i1 d i1 k t s ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "predigest"  v ( p r ii2 d ai jh e1 s t ) ((pos "H0$" ) ))
( "predigested"  v ( p r ii2 d ai jh e1 s t i d ) ((pos "Hc$" "Hd%" ) ))
( "predigesting"  v ( p r ii2 d ai jh e1 s t i ng ) ((pos "Hb$" ) ))
( "predigests"  v ( p r ii2 d ai jh e1 s t s ) ((pos "Ha$" ) ))
( "predilection"  n ( p r ii2 d i l e1 k sh @ n ) ((pos "K6%" ) ))
( "predilections"  n ( p r ii2 d i l e1 k sh @ n z ) ((pos "Kj%" ) ))
( "predispose"  v ( p r ii2 d i s p ou1 z ) ((pos "H2%" ) (subcat "14" )))
( "predisposed"  v ( p r ii2 d i s p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "predisposes"  v ( p r ii2 d i s p ou1 z i z ) ((pos "Ha%" ) (subcat "14" )))
( "predisposing"  v ( p r ii2 d i s p ou1 z i ng ) ((pos "Hb%" ) (subcat "14" )))
( "predisposition"  n ( p r ii2 d i s p @ z i1 sh @ n ) ((pos "K6%" ) ))
( "predispositions"  n ( p r ii2 d i s p @ z i1 sh @ n z ) ((pos "Kj%" ) ))
( "predominance"  n ( p r i1 d o1 m i n @ n s ) ((pos "L@%" ) ))
( "predominant"  j ( p r i1 d o1 m i n @ n t ) ((pos "OA%" ) ))
( "predominantly"  a ( p r i1 d o1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "predominate"  v ( p r i1 d o1 m i n ei t ) ((pos "I2%" ) (subcat "2A" )))
( "predominated"  v ( p r i1 d o1 m i n ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "predominates"  v ( p r i1 d o1 m i n ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "predominating"  v ( p r i1 d o1 m i n ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "preen"  v ( p r ii1 n ) ((pos "H0%" ) (subcat "6A" )))
( "preened"  v ( p r ii1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "preening"  v ( p r ii1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "preens"  v ( p r ii1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "prefab"  n ( p r ii1 f a b ) ((pos "K6%" ) ))
( "prefabricate"  v ( p r ii2 f a1 b r i k ei t ) ((pos "H2%" ) (subcat )))
( "prefabricated"  v ( p r ii2 f a1 b r i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prefabricates"  v ( p r ii2 f a1 b r i k ei t s ) ((pos "Ha%" ) (subcat )))
( "prefabricating"  v ( p r ii2 f a1 b r i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "prefabrication"  n ( p r ii2 f a b r i k ei1 sh @ n ) ((pos "K6%" ) ))
( "prefabrications"  n ( p r ii2 f a b r i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "prefabs"  n ( p r ii1 f a b z ) ((pos "Kj%" ) ))
( "preface"  v ( p r e1 f i s ) ((pos "H2%" "K6%" ) (subcat )))
( "prefaced"  v ( p r e1 f i s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prefaces"  v ( p r e1 f i s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "prefacing"  v ( p r e1 f i s i ng ) ((pos "Hb%" ) (subcat )))
( "prefatory"  j ( p r e1 f @ t r ii ) ((pos "OA%" ) ))
( "prefect"  n ( p r ii1 f e k t ) ((pos "K6%" ) ))
( "prefects"  n ( p r ii1 f e k t s ) ((pos "Kj%" ) ))
( "prefectural"  j ( p r ii1 f e1 k ch @ r @ l ) ((pos "OA$" ) ))
( "prefecture"  n ( p r ii1 f e k t y u@ r ) ((pos "K6%" ) ))
( "prefectures"  n ( p r ii1 f e k t y u@ z ) ((pos "Kj%" ) ))
( "prefer"  v ( p r i1 f @@1 r ) ((pos "H4%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "preferable"  j ( p r e1 f r @ b l ) ((pos "OA%" ) ))
( "preferably"  a ( p r e1 f r @ b l ii ) ((pos "Pu%" ) ))
( "preference"  n ( p r e1 f r @ n s ) ((pos "M6%" ) ))
( "preferences"  n ( p r e1 f r @ n s i z ) ((pos "Mj%" ) ))
( "preferential"  j ( p r e2 f @ r e1 n sh l ) ((pos "OA%" ) ))
( "preferment"  n ( p r i1 f @@1 m @ n t ) ((pos "L@%" ) ))
( "preferred"  v ( p r i1 f @@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "preferring"  v ( p r i1 f @@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "prefers"  v ( p r i1 f @@1 z ) ((pos "Ha%" ) (subcat "6A" "6D" "7A" "9" "14" )))
( "prefigure"  v ( p r ii2 f i1 g @ r ) ((pos "H2$" ) (subcat "6A" "9" )))
( "prefigured"  v ( p r ii2 f i1 g @ d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "9" )))
( "prefigures"  v ( p r ii2 f i1 g @ z ) ((pos "Ha$" ) (subcat "6A" "9" )))
( "prefiguring"  v ( p r ii2 f i1 g @ r i ng ) ((pos "Hb$" ) (subcat "6A" "9" )))
( "prefix"  n ( p r ii1 f i k s ) ((pos "K7%" ) ))
( "prefix"  v ( p r ii2 f i1 k s ) ((pos "H1%" ) (subcat "6A" )))
( "prefixed"  v ( p r ii2 f i1 k s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prefixes"  n ( p r ii1 f i k s i z ) ((pos "Kj%" ) ))
( "prefixes"  v ( p r ii2 f i1 k s i z ) ((pos "Ha$" ) (subcat "6A" )))
( "prefixing"  v ( p r ii2 f i1 k s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "pregnancies"  n ( p r e1 g n @ n s i z ) ((pos "Mj%" ) ))
( "pregnancy"  n ( p r e1 g n @ n s ii ) ((pos "M8%" ) ))
( "pregnant"  j ( p r e1 g n @ n t ) ((pos "OA%" ) ))
( "preheat"  v ( p r ii2 h ii1 t ) ((pos "H0%" ) (subcat )))
( "preheated"  v ( p r ii2 h ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "preheating"  v ( p r ii2 h ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "preheats"  v ( p r ii2 h ii1 t s ) ((pos "Ha%" ) (subcat )))
( "prehensile"  j ( p r ii2 h e1 n s ai l ) ((pos "OA%" ) ))
( "prehistoric"  j ( p r ii2 h i s t o1 r i k ) ((pos "OA%" ) ))
( "prehistorical"  j ( p r ii2 h i s t o1 r i k l ) ((pos "OA$" ) ))
( "prehistories"  n ( p r ii2 h i1 s t r i z ) ((pos "Mj$" ) ))
( "prehistory"  n ( p r ii2 h i1 s t r ii ) ((pos "M8%" ) ))
( "prejudge"  v ( p r ii2 jh uh1 jh ) ((pos "H2%" ) (subcat )))
( "prejudged"  v ( p r ii2 jh uh1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prejudgement"  n ( p r ii2 jh uh1 jh m @ n t ) ((pos "K6%" ) ))
( "prejudgements"  n ( p r ii2 jh uh1 jh m @ n t s ) ((pos "Kj%" ) ))
( "prejudges"  v ( p r ii2 jh uh1 jh i z ) ((pos "Ha%" ) (subcat )))
( "prejudging"  v ( p r ii2 jh uh1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "prejudice"  v ( p r e1 jh u d i s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "prejudiced"  v ( p r e1 jh u d i s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prejudices"  v ( p r e1 jh u d i s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "prejudicial"  j ( p r e2 jh u d i1 sh l ) ((pos "OA%" ) ))
( "prejudicing"  v ( p r e1 jh u d i s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "prelacies"  n ( p r e1 l @ s i z ) ((pos "Kj$" ) ))
( "prelacy"  n ( p r e1 l @ s ii ) ((pos "K8$" ) ))
( "prelate"  n ( p r e1 l @ t ) ((pos "K6%" ) ))
( "prelates"  n ( p r e1 l @ t s ) ((pos "Kj%" ) ))
( "prelim"  n ( p r i1 l i1 m ) ((pos "K6$" ) ))
( "preliminaries"  n ( p r i1 l i1 m i n @ r i z ) ((pos "Kj%" ) ))
( "preliminary"  n ( p r i1 l i1 m i n @ r ii ) ((pos "K8%" "OA%" ) ))
( "prelims"  n ( p r i1 l i1 m z ) ((pos "Kj$" ) ))
( "prelude"  v ( p r e1 l y uu d ) ((pos "H2$" "K6%" ) (subcat )))
( "preluded"  v ( p r e1 l y uu d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "preludes"  v ( p r e1 l y uu d z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "preluding"  v ( p r e1 l y uu d i ng ) ((pos "Hb$" ) (subcat )))
( "premarital"  j ( p r ii2 m a1 r i t l ) ((pos "OA%" ) ))
( "premature"  j ( p r e1 m @ t y u@ r ) ((pos "OA%" ) ))
( "prematurely"  a ( p r e2 m @ t y u@1 l ii ) ((pos "Pu%" ) ))
( "premeditate"  v ( p r ii2 m e1 d i t ei t ) ((pos "H2%" ) (subcat )))
( "premeditated"  v ( p r ii2 m e1 d i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "premeditates"  v ( p r ii2 m e1 d i t ei t s ) ((pos "Ha%" ) (subcat )))
( "premeditating"  v ( p r ii2 m e1 d i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "premeditation"  n ( p r ii2 m e d i t ei1 sh @ n ) ((pos "L@%" ) ))
( "premi`ere"  n ( p r e1 m i e@ r ) ((pos "K6%" ) ))
( "premi`eres"  n ( p r e1 m i e@ z ) ((pos "Kj%" ) ))
( "premier"  n ( p r e1 m i@ r ) ((pos "K6%" "OA%" ) ))
( "premiers"  n ( p r e1 m i@ z ) ((pos "Kj%" ) ))
( "premiership"  n ( p r e1 m i@ sh i p ) ((pos "K6%" ) ))
( "premierships"  n ( p r e1 m i@ sh i p s ) ((pos "Kj%" ) ))
( "premise"  v ( p r e1 m i s ) ((pos "H2$" "K6%" ) (subcat "6A" )))
( "premised"  v ( p r e1 m i s t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "premises"  v ( p r e1 m i s i z ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "premising"  v ( p r e1 m i s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "premiss"  v ( p r e1 m i s ) ((pos "H1$" "K7$" ) (subcat "6A" )))
( "premissed"  v ( p r e1 m i s t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "premisses"  v ( p r e1 m i s i z ) ((pos "Ha$" "Kj$" ) (subcat "6A" )))
( "premissing"  v ( p r e1 m i s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "premium"  n ( p r ii1 m i@ m ) ((pos "K6%" ) ))
( "premiums"  n ( p r ii1 m i@ m z ) ((pos "Kj%" ) ))
( "premonition"  n ( p r ii2 m @ n i1 sh @ n ) ((pos "K6%" ) ))
( "premonitions"  n ( p r ii2 m @ n i1 sh @ n z ) ((pos "Kj%" ) ))
( "premonitory"  j ( p r i1 m o1 n i t @ r ii ) ((pos "OA$" ) ))
( "prenatal"  j ( p r ii2 n ei1 t l ) ((pos "OA%" ) ))
( "prentice"  n ( p r e1 n t i s ) ((pos "K6$" ) ))
( "prentices"  n ( p r e1 n t i s i z ) ((pos "Kj$" ) ))
( "preoccupation"  n ( p r ii1 o2 k y u p ei1 sh @ n ) ((pos "M6%" ) ))
( "preoccupations"  n ( p r ii1 o2 k y u p ei1 sh @ n z ) ((pos "Mj%" ) ))
( "preoccupied"  v ( p r ii1 o1 k y u p ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "preoccupies"  v ( p r ii1 o1 k y u p ai z ) ((pos "Ha%" ) (subcat )))
( "preoccupy"  v ( p r ii1 o1 k y u p ai ) ((pos "H3%" ) (subcat )))
( "preoccupying"  v ( p r ii1 o1 k y u p ai i ng ) ((pos "Hb%" ) (subcat )))
( "preordain"  v ( p r ii2 oo d ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "preordained"  v ( p r ii2 oo d ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "preordaining"  v ( p r ii2 oo d ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "preordains"  v ( p r ii2 oo d ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "prep"  n ( p r e1 p ) ((pos "M6$" ) ))
( "prepacked"  j ( p r ii2 p a1 k t ) ((pos "OA%" ) ))
( "prepaid"  v ( p r ii2 p ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "preparation"  n ( p r e2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "preparations"  n ( p r e2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "preparatory"  j ( p r i1 p a1 r @ t r ii ) ((pos "OA%" ) ))
( "prepare"  v ( p r i1 p e@1 r ) ((pos "J2%" ) (subcat "3A" "6A" "7A" )))
( "prepared"  v ( p r i1 p e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" )))
( "preparedness"  n ( p r i1 p e@1 r i d n @ s ) ((pos "L@%" ) ))
( "prepares"  v ( p r i1 p e@1 z ) ((pos "Ja%" ) (subcat "3A" "6A" "7A" )))
( "preparing"  v ( p r i1 p e@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "7A" )))
( "prepay"  v ( p r ii2 p ei1 ) ((pos "H5%" ) (subcat )))
( "prepaying"  v ( p r ii2 p ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "prepays"  v ( p r i2 p ei1 z ) ((pos "Ha%" ) (subcat )))
( "preponderance"  n ( p r i1 p o1 n d @ r @ n s ) ((pos "K6%" ) ))
( "preponderances"  n ( p r i1 p o1 n d @ r @ n s i z ) ((pos "Kj$" ) ))
( "preponderant"  j ( p r i1 p o1 n d @ r @ n t ) ((pos "OA%" ) ))
( "preponderantly"  a ( p r i1 p o1 n d @ r @ n t l ii ) ((pos "Pu%" ) ))
( "preponderate"  v ( p r i1 p o1 n d @ r ei t ) ((pos "I2$" ) (subcat "2A" )))
( "preponderated"  v ( p r i1 p o1 n d @ r ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "preponderates"  v ( p r i1 p o1 n d @ r ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "preponderating"  v ( p r i1 p o1 n d @ r ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "preposition"  n ( p r e2 p @ z i1 sh @ n ) ((pos "K6%" ) ))
( "prepositional"  j ( p r e2 p @ z i1 sh @ n l ) ((pos "OA%" ) ))
( "prepositions"  n ( p r e2 p @ z i1 sh @ n z ) ((pos "Kj%" ) ))
( "prepossess"  v ( p r ii2 p @ z e1 s ) ((pos "H1$" ) (subcat "6A" )))
( "prepossessed"  v ( p r ii2 p @ z e1 s t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "prepossesses"  v ( p r ii2 p @ z e1 s i z ) ((pos "Ha$" ) (subcat "6A" )))
( "prepossessing"  v ( p r ii2 p @ z e1 s i ng ) ((pos "Hb$" "OA%" ) (subcat "6A" )))
( "prepossession"  n ( p r ii2 p @ z e1 sh @ n ) ((pos "K6$" ) ))
( "prepossessions"  n ( p r ii2 p @ z e1 sh @ n z ) ((pos "Kj$" ) ))
( "preposterous"  j ( p r i1 p o1 s t @ r @ s ) ((pos "OA%" ) ))
( "preposterously"  a ( p r i1 p o1 s t @ r @ s l ii ) ((pos "Pu%" ) ))
( "preps"  n ( p r e1 p s ) ((pos "Mj$" ) ))
( "prepuce"  n ( p r ii1 p y uu s ) ((pos "K6$" ) ))
( "prepuces"  n ( p r ii1 p y uu s i z ) ((pos "Kj$" ) ))
( "prerecord"  v ( p r ii2 r i k oo1 d ) ((pos "H0%" ) (subcat )))
( "prerecorded"  v ( p r ii2 r i k oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prerecording"  v ( p r ii2 r i k oo1 d i ng ) ((pos "Hb%" ) (subcat )))
( "prerecords"  v ( p r ii2 r i k oo1 d z ) ((pos "Ha%" ) (subcat )))
( "prerequisite"  n ( p r ii2 r e1 k w i z i t ) ((pos "K6%" "OA%" ) ))
( "prerequisites"  n ( p r ii2 r e1 k w i z i t s ) ((pos "Kj%" ) ))
( "prerogative"  n ( p r i1 r o1 g @ t i v ) ((pos "K6%" ) ))
( "prerogatives"  n ( p r i1 r o1 g @ t i v z ) ((pos "Kj%" ) ))
( "presage"  n ( p r e1 s i jh ) ((pos "K6$" ) ))
( "presage"  v ( p r i1 s ei1 jh ) ((pos "H2$" ) (subcat )))
( "presaged"  v ( p r i1 s ei1 jh d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "presages"  n ( p r e1 s i jh i z ) ((pos "Kj$" ) ))
( "presages"  v ( p r i1 s ei1 jh i z ) ((pos "Ha$" ) (subcat )))
( "presaging"  v ( p r i1 s ei1 jh i ng ) ((pos "Hb$" ) (subcat )))
( "presbyter"  n ( p r e1 z b i t @ r ) ((pos "K6$" ) ))
( "presbyteries"  n ( p r e1 z b i t r i z ) ((pos "Kj$" ) ))
( "presbyters"  n ( p r e1 z b i t @ z ) ((pos "Kj$" ) ))
( "presbytery"  n ( p r e1 z b i t r ii ) ((pos "K8%" ) ))
( "prescience"  n ( p r e1 s i@ n s ) ((pos "L@$" ) ))
( "prescient"  j ( p r e1 s i@ n t ) ((pos "OA$" ) ))
( "presciently"  a ( p r e1 s i@ n t l ii ) ((pos "Pu$" ) ))
( "prescribe"  v ( p r i1 s k r ai1 b ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "8" "10" "14" )))
( "prescribed"  v ( p r i1 s k r ai1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "10" "14" )))
( "prescribes"  v ( p r i1 s k r ai1 b z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "10" "14" )))
( "prescribing"  v ( p r i1 s k r ai1 b i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" "10" "14" )))
( "prescript"  n ( p r ii1 s k r i p t ) ((pos "K6$" ) ))
( "prescription"  n ( p r i1 s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "prescriptions"  n ( p r i1 s k r i1 p sh @ n z ) ((pos "Mj%" ) ))
( "prescriptive"  j ( p r i1 s k r i1 p t i v ) ((pos "OA%" ) ))
( "prescripts"  n ( p r ii1 s k r i p t s ) ((pos "Kj$" ) ))
( "presence"  n ( p r e1 z n s ) ((pos "L@%" ) ))
( "present"  n ( p r e1 z n t ) ((pos "K6*" "OA*" ) ))
( "present"  v ( p r i1 z e1 n t ) ((pos "H0%" "Ki%" ) (subcat "6A" "14" )))
( "present-day"  j ( p r e2 z n t - d ei1 ) ((pos "Oq%" ) ))
( "presentable"  j ( p r i1 z e1 n t @ b l ) ((pos "OA%" ) ))
( "presentably"  a ( p r i1 z e1 n t @ b l ii ) ((pos "Pu%" ) ))
( "presentation"  n ( p r e2 z n t ei1 sh @ n ) ((pos "M6%" ) ))
( "presentations"  n ( p r e2 z n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "presented"  v ( p r i1 z e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "presentiment"  n ( p r i1 z e1 n t i m @ n t ) ((pos "K6%" ) ))
( "presentiments"  n ( p r i1 z e1 n t i m @ n t s ) ((pos "Kj%" ) ))
( "presenting"  v ( p r i1 z e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "presently"  a ( p r e1 z n t l ii ) ((pos "Pu%" ) ))
( "presents"  n ( p r e1 z n t s ) ((pos "Kj%" ) ))
( "presents"  v ( p r i1 z e1 n t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "preservable"  j ( p r i1 z @@1 v @ b l ) ((pos "OA%" ) ))
( "preservation"  n ( p r e2 z @ v ei1 sh @ n ) ((pos "L@%" ) ))
( "preservative"  n ( p r i1 z @@1 v @ t i v ) ((pos "K6%" "OA%" ) ))
( "preservatives"  n ( p r i1 z @@1 v @ t i v z ) ((pos "Kj%" ) ))
( "preserve"  v ( p r i1 z @@1 v ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "preserved"  v ( p r i1 z @@1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "preserver"  n ( p r i1 z @@1 v @ r ) ((pos "K6%" ) ))
( "preservers"  n ( p r i1 z @@1 v @ z ) ((pos "Kj%" ) ))
( "preserves"  v ( p r i1 z @@1 v z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "preserving"  v ( p r i1 z @@1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "preside"  v ( p r i1 z ai1 d ) ((pos "I2%" ) (subcat "2A" "2C" )))
( "presided"  v ( p r i1 z ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "presidencies"  n ( p r e1 z i d @ n s i z ) ((pos "Kj%" ) ))
( "presidency"  n ( p r e1 z i d @ n s ii ) ((pos "K8%" ) ))
( "president"  n ( p r e1 z i d @ n t ) ((pos "K6*" ) ))
( "presidential"  j ( p r e2 z i d e1 n sh l ) ((pos "OA%" ) ))
( "presidents"  n ( p r e1 z i d @ n t s ) ((pos "Kj%" ) ))
( "presides"  v ( p r i1 z ai1 d z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "presiding"  v ( p r i1 z ai1 d i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "presidium"  n ( p r i1 s i1 d i@ m ) ((pos "K6$" ) ))
( "presidiums"  n ( p r i1 s i1 d i@ m z ) ((pos "Kj$" ) ))
( "press"  v ( p r e1 s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" "15B" "17" )))
( "press-agencies"  n ( p r e1 s - ei jh @ n s i z ) ((pos "Kj%" ) ))
( "press-agency"  n ( p r e1 s - ei jh @ n s ii ) ((pos "K8%" ) ))
( "press-agent"  n ( p r e1 s - ei jh @ n t ) ((pos "K6%" ) ))
( "press-agents"  n ( p r e1 s - ei jh @ n t s ) ((pos "Kj%" ) ))
( "press-box"  n ( p r e1 s - b o k s ) ((pos "K7%" ) ))
( "press-boxes"  n ( p r e1 s - b o k s i z ) ((pos "Kj%" ) ))
( "press-clipping"  n ( p r e1 s - k l i p i ng ) ((pos "K6%" ) ))
( "press-clippings"  n ( p r e1 s - k l i p i ng z ) ((pos "Kj%" ) ))
( "press-cutting"  n ( p r e1 s - k uh t i ng ) ((pos "K6%" ) ))
( "press-cuttings"  n ( p r e1 s - k uh t i ng z ) ((pos "Kj%" ) ))
( "press-galleries"  n ( p r e1 s - g a l @ r i z ) ((pos "Kj%" ) ))
( "press-gallery"  n ( p r e1 s - g a l @ r ii ) ((pos "K8%" ) ))
( "press-gang"  n ( p r e1 s - g a ng ) ((pos "K6%" ) ))
( "press-gangs"  n ( p r e1 s - g a ng z ) ((pos "Kj%" ) ))
( "press-lord"  n ( p r e1 s - l oo d ) ((pos "K6$" ) ))
( "press-lords"  n ( p r e1 s - l oo d z ) ((pos "Kj$" ) ))
( "press-photographer"  n ( p r e2 s - f @ t o1 g r @ f @ r ) ((pos "K6%" ) ))
( "press-photographers"  n ( p r e2 s - f @ t o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "press-stud"  n ( p r e1 s - s t uh d ) ((pos "K6%" ) ))
( "press-studs"  n ( p r e1 s - s t uh d z ) ((pos "Kj%" ) ))
( "press-up"  n ( p r e1 s - uh p ) ((pos "K6%" ) ))
( "press-ups"  n ( p r e1 s - uh p s ) ((pos "Kj%" ) ))
( "pressed"  v ( p r e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" "15B" "17" )))
( "presses"  v ( p r e1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" "15B" "17" )))
( "pressing"  v ( p r e1 s i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" "15B" "17" )))
( "pressingly"  a ( p r e1 s i ng l ii ) ((pos "Pu%" ) ))
( "pressings"  n ( p r e1 s i ng z ) ((pos "Kj%" ) ))
( "pressman"  n ( p r e1 s m a n ) ((pos "Ki%" ) ))
( "pressmark"  n ( p r e1 s m aa k ) ((pos "K6$" ) ))
( "pressmarks"  n ( p r e1 s m aa k s ) ((pos "Kj$" ) ))
( "pressmen"  n ( p r e1 s m e n ) ((pos "Kj%" ) ))
( "pressure"  n ( p r e1 sh @ r ) ((pos "M6%" ) ))
( "pressure-cooker"  n ( p r e1 sh @ - k u k @ r ) ((pos "K6%" ) ))
( "pressure-cookers"  n ( p r e1 sh @ - k u k @ z ) ((pos "Kj%" ) ))
( "pressure-gauge"  n ( p r e1 sh @ - g ei jh ) ((pos "K6%" ) ))
( "pressure-gauges"  n ( p r e1 sh @ - g ei jh i z ) ((pos "Kj%" ) ))
( "pressures"  n ( p r e1 sh @ z ) ((pos "Mj%" ) ))
( "pressurized"  j ( p r e1 sh @ r ai z d ) ((pos "OA%" ) ))
( "prestidigitation"  n ( p r e2 s t i d i2 jh i t ei1 sh @ n ) ((pos "M6$" ) ))
( "prestidigitations"  n ( p r e2 s t i d i2 jh i t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "prestidigitator"  n ( p r e2 s t i d i1 jh i t ei t @ r ) ((pos "K6$" ) ))
( "prestidigitators"  n ( p r e2 s t i d i1 jh i t ei t @ z ) ((pos "Kj$" ) ))
( "prestige"  n ( p r e1 s t ii1 zh ) ((pos "L@%" ) ))
( "prestigious"  j ( p r e1 s t i1 jh @ s ) ((pos "OA%" ) ))
( "prestissimo"  j ( p r e1 s t i1 s i m ou ) ((pos "OA$" "Pu$" ) ))
( "presto"  j ( p r e1 s t ou ) ((pos "OA$" "Pu$" ) ))
( "prestressed"  j ( p r ii2 s t r e1 s t ) ((pos "OA%" ) ))
( "presumable"  j ( p r i1 z y uu1 m @ b l ) ((pos "OA$" ) ))
( "presumably"  a ( p r i1 z y uu1 m @ b l ii ) ((pos "Pu%" ) ))
( "presume"  v ( p r i1 z y uu1 m ) ((pos "J2%" ) (subcat "3A" "6A" "7A" "9" )))
( "presumed"  v ( p r i1 z y uu1 m d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "9" )))
( "presumes"  v ( p r i1 z y uu1 m z ) ((pos "Ja%" ) (subcat "3A" "6A" "7A" "9" )))
( "presuming"  v ( p r i1 z y uu1 m i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" "6A" "7A" "9" )))
( "presumption"  n ( p r i1 z uh1 m p sh @ n ) ((pos "M6%" ) ))
( "presumptions"  n ( p r i1 z uh1 m p sh @ n z ) ((pos "Mj$" ) ))
( "presumptive"  j ( p r i1 z uh1 m p t i v ) ((pos "OA$" ) ))
( "presumptively"  a ( p r i1 z uh1 m p t i v l ii ) ((pos "Pu$" ) ))
( "presumptuous"  j ( p r i1 z uh1 m p ch u@ s ) ((pos "OA%" ) ))
( "presumptuously"  a ( p r i1 z uh1 m p ch u@ s l ii ) ((pos "Pu%" ) ))
( "presuppose"  v ( p r ii2 s @ p ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "presupposed"  v ( p r ii2 s @ p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "presupposes"  v ( p r ii2 s @ p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "presupposing"  v ( p r ii2 s @ p ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "presupposition"  n ( p r ii2 s uh p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "presuppositions"  n ( p r ii2 s uh p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "pretence"  n ( p r i1 t e1 n s ) ((pos "M6%" ) ))
( "pretences"  n ( p r i1 t e1 n s i z ) ((pos "Mj%" ) ))
( "pretend"  v ( p r i1 t e1 n d ) ((pos "J0%" ) (subcat "3A" "6A" "7A" )))
( "pretended"  v ( p r i1 t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" )))
( "pretendedly"  a ( p r i1 t e1 n d i d l ii ) ((pos "Pu$" ) ))
( "pretender"  n ( p r i1 t e1 n d @ r ) ((pos "K6%" ) ))
( "pretenders"  n ( p r i1 t e1 n d @ z ) ((pos "Kj%" ) ))
( "pretending"  v ( p r i1 t e1 n d i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "7A" )))
( "pretends"  v ( p r i1 t e1 n d z ) ((pos "Ja%" ) (subcat "3A" "6A" "7A" )))
( "pretension"  n ( p r i1 t e1 n sh @ n ) ((pos "M6%" ) ))
( "pretensions"  n ( p r i1 t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "pretentious"  j ( p r i1 t e1 n sh @ s ) ((pos "OA%" ) ))
( "pretentiously"  a ( p r i1 t e1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "pretentiousness"  n ( p r i1 t e1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "preterit"  n ( p r e1 t @ r i t ) ((pos "K6$" "OA$" ) ))
( "preterite"  n ( p r e1 t @ r i t ) ((pos "K6$" "OA$" ) ))
( "preterites"  n ( p r e1 t @ r i t s ) ((pos "Kj$" ) ))
( "preterits"  n ( p r e1 t @ r i t s ) ((pos "Kj$" ) ))
( "preternatural"  j ( p r ii2 t @ n a1 ch r @ l ) ((pos "OA%" ) ))
( "preternaturally"  a ( p r ii2 t @ n a1 ch r @ l ii ) ((pos "Pu%" ) ))
( "pretext"  n ( p r ii1 t e k s t ) ((pos "K6%" ) ))
( "pretexts"  n ( p r ii1 t e k s t s ) ((pos "Kj%" ) ))
( "pretor"  n ( p r ii1 t @ r ) ((pos "K6$" ) ))
( "pretors"  n ( p r ii1 t @ z ) ((pos "Kj$" ) ))
( "prettier"  j ( p r i1 t i@ r ) ((pos "Or%" ) ))
( "pretties"  n ( p r i1 t i z ) ((pos "Kj$" ) ))
( "prettiest"  j ( p r i1 t i i s t ) ((pos "Os%" ) ))
( "prettified"  v ( p r i1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prettifies"  v ( p r i1 t i f ai z ) ((pos "Ha%" ) (subcat )))
( "prettify"  v ( p r i1 t i f ai ) ((pos "H3%" ) (subcat )))
( "prettifying"  v ( p r i1 t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "prettily"  a ( p r i1 t i l ii ) ((pos "Pu%" ) ))
( "prettiness"  n ( p r i1 t i n @ s ) ((pos "L@%" ) ))
( "pretty"  n ( p r i1 t ii ) ((pos "K8%" "OD%" "Pu%" ) ))
( "pretty-pretty"  j ( p r i1 t i - p r i t ii ) ((pos "OA%" ) ))
( "pretzel"  n ( p r e1 t s l ) ((pos "K6%" ) ))
( "pretzels"  n ( p r e1 t s l z ) ((pos "Kj%" ) ))
( "prevail"  v ( p r i1 v ei1 l ) ((pos "I0%" ) (subcat "2A" )))
( "prevailed"  v ( p r i1 v ei1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "prevailing"  v ( p r i1 v ei1 l i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "prevails"  v ( p r i1 v ei1 l z ) ((pos "Ia%" ) (subcat "2A" )))
( "prevalence"  n ( p r e1 v @ l @ n s ) ((pos "L@%" ) ))
( "prevalent"  j ( p r e1 v @ l @ n t ) ((pos "OA%" ) ))
( "prevaricate"  v ( p r i1 v a1 r i k ei t ) ((pos "I2%" ) (subcat )))
( "prevaricated"  v ( p r i1 v a1 r i k ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "prevaricates"  v ( p r i1 v a1 r i k ei t s ) ((pos "Ia%" ) (subcat )))
( "prevaricating"  v ( p r i1 v a1 r i k ei t i ng ) ((pos "Ib%" ) (subcat )))
( "prevarication"  n ( p r i1 v a2 r i k ei1 sh @ n ) ((pos "M6%" ) ))
( "prevarications"  n ( p r i1 v a2 r i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "prevent"  v ( p r i1 v e1 n t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "preventable"  j ( p r i1 v e1 n t @ b l ) ((pos "OA%" ) ))
( "preventative"  n ( p r i1 v e1 n t @ t i v ) ((pos "K6%" ) ))
( "preventatives"  n ( p r i1 v e1 n t @ t i v z ) ((pos "Kj%" ) ))
( "prevented"  v ( p r i1 v e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "preventing"  v ( p r i1 v e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "prevention"  n ( p r i1 v e1 n sh @ n ) ((pos "L@%" ) ))
( "preventive"  j ( p r i1 v e1 n t i v ) ((pos "OA%" ) ))
( "prevents"  v ( p r i1 v e1 n t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "preview"  v ( p r ii1 v y uu ) ((pos "H0%" "K6%" ) ))
( "previewed"  v ( p r ii1 v y uu d ) ((pos "Hc%" "Hd%" ) ))
( "previewing"  v ( p r ii1 v y uu i ng ) ((pos "Hb%" ) ))
( "previews"  v ( p r ii1 v y uu z ) ((pos "Ha%" "Kj%" ) ))
( "previous"  j ( p r ii1 v i@ s ) ((pos "OA%" ) ))
( "previously"  a ( p r ii1 v i@ s l ii ) ((pos "Pu%" ) ))
( "prevision"  n ( p r ii2 v i1 zh n ) ((pos "M6$" ) ))
( "previsions"  n ( p r ii2 v i1 zh n z ) ((pos "Mj$" ) ))
( "prey"  v ( p r ei1 ) ((pos "I0%" "Ki%" ) (subcat )))
( "preyed"  v ( p r ei1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "preying"  v ( p r ei1 i ng ) ((pos "Ib%" ) (subcat )))
( "preys"  v ( p r ei1 z ) ((pos "Ia%" ) (subcat )))
( "price"  v ( p r ai1 s ) ((pos "H2%" "M6%" ) (subcat )))
( "price-control"  n ( p r ai1 s - k @ n t r ou l ) ((pos "K6%" ) ))
( "price-controlled"  j ( p r ai1 s - k @ n t r ou l d ) ((pos "OA%" ) ))
( "price-controls"  n ( p r ai1 s - k @ n t r ou l z ) ((pos "Kj%" ) ))
( "priced"  v ( p r ai1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "priceless"  j ( p r ai1 s l @ s ) ((pos "OA%" ) ))
( "pricelist"  n ( p r ai1 s l i s t ) ((pos "K6%" ) ))
( "pricelists"  n ( p r ai1 s l i s t s ) ((pos "Kj%" ) ))
( "prices"  v ( p r ai1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "pricey"  j ( p r ai1 s ii ) ((pos "OE%" ) ))
( "pricier"  j ( p r ai1 s i@ r ) ((pos "Or%" ) ))
( "priciest"  j ( p r ai1 s i i s t ) ((pos "Os%" ) ))
( "pricing"  v ( p r ai1 s i ng ) ((pos "Hb%" ) (subcat )))
( "prick"  v ( p r i1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "pricked"  v ( p r i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "pricker"  n ( p r i1 k @ r ) ((pos "K6%" ) ))
( "prickers"  n ( p r i1 k @ z ) ((pos "Kj%" ) ))
( "pricking"  v ( p r i1 k i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "6A" )))
( "prickings"  n ( p r i1 k i ng z ) ((pos "Kj%" ) ))
( "prickle"  v ( p r i1 k l ) ((pos "J2%" "K6%" ) ))
( "prickled"  v ( p r i1 k l d ) ((pos "Jc%" "Jd%" ) ))
( "prickles"  v ( p r i1 k l z ) ((pos "Ja%" "Kj%" ) ))
( "pricklier"  j ( p r i1 k l i@ r ) ((pos "Or%" ) ))
( "prickliest"  j ( p r i1 k l i i s t ) ((pos "Os%" ) ))
( "prickling"  v ( p r i1 k l i ng ) ((pos "Jb%" ) ))
( "prickly"  j ( p r i1 k l ii ) ((pos "OD%" ) ))
( "pricks"  v ( p r i1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "pride"  v ( p r ai1 d ) ((pos "H2%" "M6%" ) ))
( "prided"  v ( p r ai1 d i d ) ((pos "Hc%" "Hd%" ) ))
( "prides"  v ( p r ai1 d z ) ((pos "Ha%" "Mj%" ) ))
( "priding"  v ( p r ai1 d i ng ) ((pos "Hb%" ) ))
( "prie-dieu"  n ( p r ii1 - d y @@ ) ((pos "K6$" ) ))
( "prie-dieus"  n ( p r ii1 - d y @@ z ) ((pos "Kj$" ) ))
( "pried"  v ( p r ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "15A" "15B" )))
( "pries"  v ( p r ai1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "15A" "15B" )))
( "priest"  n ( p r ii1 s t ) ((pos "K6%" ) ))
( "priest-ridden"  j ( p r ii1 s t - r i d n ) ((pos "OA$" ) ))
( "priestcraft"  n ( p r ii1 s t k r aa f t ) ((pos "L@%" ) ))
( "priestess"  n ( p r ii1 s t e s ) ((pos "K7%" ) ))
( "priestesses"  n ( p r ii1 s t e s i z ) ((pos "Kj%" ) ))
( "priesthood"  n ( p r ii1 s t h u d ) ((pos "K6%" ) ))
( "priesthoods"  n ( p r ii1 s t h u d z ) ((pos "Kj%" ) ))
( "priestlier"  j ( p r ii1 s t l i@ r ) ((pos "Or$" ) ))
( "priestliest"  j ( p r ii1 s t l i i s t ) ((pos "Os$" ) ))
( "priestlike"  j ( p r ii1 s t l ai k ) ((pos "OA%" ) ))
( "priestly"  j ( p r ii1 s t l ii ) ((pos "OD%" ) ))
( "priests"  n ( p r ii1 s t s ) ((pos "Kj%" ) ))
( "prig"  n ( p r i1 g ) ((pos "K6%" ) ))
( "priggish"  j ( p r i1 g i sh ) ((pos "OA%" ) ))
( "priggishly"  a ( p r i1 g i sh l ii ) ((pos "Pu%" ) ))
( "priggishness"  n ( p r i1 g i sh n @ s ) ((pos "L@%" ) ))
( "prigs"  n ( p r i1 g z ) ((pos "Kj%" ) ))
( "prim"  v ( p r i1 m ) ((pos "H4$" "OE%" ) ))
( "prima"  j ( p r ii1 m @ ) ((pos "OA%" ) ))
( "prima ballerina"  n ( p r ii2 m @ - b a2 l @ r ii1 n @ ) ((pos "K6%" ) ))
( "prima ballerinas"  n ( p r ii2 m @ - b a2 l @ r ii1 n @ z ) ((pos "Kj%" ) ))
( "prima donna"  n ( p r ii2 m @ - d o1 n @ ) ((pos "K6%" ) ))
( "prima donnas"  n ( p r ii2 m @ - d o1 n @ z ) ((pos "Kj%" ) ))
( "prima facie"  j ( p r ai2 m @ - f ei1 sh ii ) ((pos "OA%" "Pu%" ) ))
( "primacies"  n ( p r ai1 m @ s i z ) ((pos "Kj$" ) ))
( "primacy"  n ( p r ai1 m @ s ii ) ((pos "K8%" ) ))
( "primaeval"  j ( p r ai1 m ii1 v @ l ) ((pos "OA%" ) ))
( "primal"  j ( p r ai1 m l ) ((pos "OA%" ) ))
( "primaries"  n ( p r ai1 m @ r i z ) ((pos "Kj%" ) ))
( "primarily"  a ( p r ai1 m @ r @ l ii ) ((pos "Pu%" ) ))
( "primary"  n ( p r ai1 m @ r ii ) ((pos "K8%" "OA%" ) ))
( "primate"  n ( p r ai1 m ei t ) ((pos "K6%" ) ))
( "primates"  n ( p r ai1 m ei t s ) ((pos "Kj%" ) ))
( "prime"  v ( p r ai1 m ) ((pos "H2%" "L@%" "OA%" ) (subcat )))
( "primed"  v ( p r ai1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "primer"  n ( p r ai1 m @ r ) ((pos "K6%" ) ))
( "primers"  n ( p r ai1 m @ z ) ((pos "Kj%" ) ))
( "primes"  v ( p r ai1 m z ) ((pos "Ha%" ) (subcat )))
( "primeval"  j ( p r ai1 m ii1 v @ l ) ((pos "OA%" ) ))
( "priming"  v ( p r ai1 m i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "primings"  n ( p r ai1 m i ng z ) ((pos "Kj$" ) ))
( "primitive"  n ( p r i1 m i t i v ) ((pos "K6%" "OA%" ) ))
( "primitively"  a ( p r i1 m i t i v l ii ) ((pos "Pu%" ) ))
( "primitiveness"  n ( p r i1 m i t i v n @ s ) ((pos "L@%" ) ))
( "primitives"  n ( p r i1 m i t i v z ) ((pos "Kj%" ) ))
( "primly"  a ( p r i1 m l ii ) ((pos "Pu%" ) ))
( "primmed"  v ( p r i1 m d ) ((pos "Hc$" "Hd$" ) ))
( "primmer"  j ( p r i1 m @ r ) ((pos "Or%" ) ))
( "primmest"  j ( p r i1 m i s t ) ((pos "Os%" ) ))
( "primming"  v ( p r i1 m i ng ) ((pos "Hb$" ) ))
( "primness"  n ( p r i1 m n @ s ) ((pos "L@%" ) ))
( "primogeniture"  n ( p r ai2 m ou jh e1 n i ch @ r ) ((pos "L@%" ) ))
( "primordial"  j ( p r ai1 m oo1 d i@ l ) ((pos "OA%" ) ))
( "primp"  v ( p r i1 m p ) ((pos "H0$" ) ))
( "primped"  v ( p r i1 m p t ) ((pos "Hc$" "Hd$" ) ))
( "primping"  v ( p r i1 m p i ng ) ((pos "Hb$" ) ))
( "primps"  v ( p r i1 m p s ) ((pos "Ha$" ) ))
( "primrose"  n ( p r i1 m r ou z ) ((pos "K6%" ) ))
( "primroses"  n ( p r i1 m r ou z i z ) ((pos "Kj%" ) ))
( "prims"  v ( p r i1 m z ) ((pos "Ha$" ) ))
( "primula"  n ( p r i1 m y u l @ ) ((pos "K6%" ) ))
( "primulas"  n ( p r i1 m y u l @ z ) ((pos "Kj%" ) ))
( "primus"  n ( p r ai1 m @ s ) ((pos "K7%" ) ))
( "primuses"  n ( p r ai1 m @ s i z ) ((pos "Kj%" ) ))
( "prince"  n ( p r i1 n s ) ((pos "K6%" ) ))
( "princedom"  n ( p r i1 n s d @ m ) ((pos "K6%" ) ))
( "princedoms"  n ( p r i1 n s d @ m z ) ((pos "Kj%" ) ))
( "princelier"  j ( p r i1 n s l i@ r ) ((pos "Or%" ) ))
( "princeliest"  j ( p r i1 n s l i i s t ) ((pos "Os%" ) ))
( "princely"  j ( p r i1 n s l ii ) ((pos "OD%" ) ))
( "princes"  n ( p r i1 n s i z ) ((pos "Kj%" ) ))
( "princess"  n ( p r i1 n s e1 s ) ((pos "K7%" ) ))
( "princesses"  n ( p r i1 n s e1 s i z ) ((pos "Kj%" ) ))
( "principal"  n ( p r i1 n s @ p l ) ((pos "K6%" "OA%" ) ))
( "principalities"  n ( p r i2 n s i p a1 l i t i z ) ((pos "Kj%" ) ))
( "principality"  n ( p r i2 n s i p a1 l i t ii ) ((pos "K8%" ) ))
( "principally"  a ( p r i1 n s @ p l ii ) ((pos "Pu%" ) ))
( "principals"  n ( p r i1 n s @ p l z ) ((pos "Kj%" ) ))
( "principle"  n ( p r i1 n s @ p l ) ((pos "K6%" ) ))
( "principled"  j ( p r i1 n s @ p l d ) ((pos "OA%" ) ))
( "principles"  n ( p r i1 n s @ p l z ) ((pos "Kj%" ) ))
( "prink"  v ( p r i1 ng k ) ((pos "H0$" ) ))
( "prinked"  v ( p r i1 ng k t ) ((pos "Hc$" "Hd$" ) ))
( "prinking"  v ( p r i1 ng k i ng ) ((pos "Hb$" ) ))
( "prinks"  v ( p r i1 ng k s ) ((pos "Ha$" ) ))
( "print"  v ( p r i1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "print-seller"  n ( p r i1 n t - s e l @ r ) ((pos "K6%" ) ))
( "print-sellers"  n ( p r i1 n t - s e l @ z ) ((pos "Kj%" ) ))
( "print-shop"  n ( p r i1 n t - sh o p ) ((pos "K6%" ) ))
( "print-shops"  n ( p r i1 n t - sh o p s ) ((pos "Kj%" ) ))
( "printable"  j ( p r i1 n t @ b l ) ((pos "OA%" ) ))
( "printed"  v ( p r i1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "printer"  n ( p r i1 n t @ r ) ((pos "K6%" ) ))
( "printers"  n ( p r i1 n t @ z ) ((pos "Kj%" ) ))
( "printing"  v ( p r i1 n t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" )))
( "printing-ink"  n ( p r i1 n t i ng - i ng k ) ((pos "K6%" ) ))
( "printing-inks"  n ( p r i1 n t i ng - i ng k s ) ((pos "Kj%" ) ))
( "printing-press"  n ( p r i1 n t i ng - p r e s ) ((pos "K7%" ) ))
( "printing-presses"  n ( p r i1 n t i ng - p r e s i z ) ((pos "Kj%" ) ))
( "printings"  n ( p r i1 n t i ng z ) ((pos "Mj%" ) ))
( "printout"  n ( p r i1 n t au t ) ((pos "K6%" ) ))
( "printouts"  n ( p r i1 n t au t s ) ((pos "Kj%" ) ))
( "prints"  v ( p r i1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "prior"  n ( p r ai1 @ r ) ((pos "K6%" "OA%" ) ))
( "prioress"  n ( p r ai1 @ r i s ) ((pos "K7%" ) ))
( "prioresses"  n ( p r ai1 @ r i s i z ) ((pos "Kj%" ) ))
( "priories"  n ( p r ai1 @ r i z ) ((pos "Kj%" ) ))
( "priorities"  n ( p r ai1 o1 r i t i z ) ((pos "Mj%" ) ))
( "priority"  n ( p r ai1 o1 r i t ii ) ((pos "M8%" ) ))
( "priors"  n ( p r ai1 @ z ) ((pos "Kj%" ) ))
( "priory"  n ( p r ai1 @ r ii ) ((pos "K8%" ) ))
( "prise"  v ( p r ai1 z ) ((pos "H2%" ) (subcat "15A" )))
( "prised"  v ( p r ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "prises"  v ( p r ai1 z i z ) ((pos "Ha%" ) (subcat "15A" )))
( "prising"  v ( p r ai1 z i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "prism"  n ( p r i1 z @ m ) ((pos "K6%" ) ))
( "prismatic"  j ( p r i1 z m a1 t i k ) ((pos "OA%" ) ))
( "prisms"  n ( p r i1 z @ m z ) ((pos "Kj%" ) ))
( "prison"  n ( p r i1 z n ) ((pos "M6%" ) ))
( "prison-breaking"  n ( p r i1 z n - b r ei k i ng ) ((pos "M6$" ) ))
( "prison-breakings"  n ( p r i1 z n - b r ei k i ng z ) ((pos "Mj$" ) ))
( "prisoner"  n ( p r i1 z n @ r ) ((pos "K6%" ) ))
( "prisoners"  n ( p r i1 z n @ z ) ((pos "Kj%" ) ))
( "prisons"  n ( p r i1 z n z ) ((pos "Mj%" ) ))
( "pristine"  j ( p r i1 s t ii n ) ((pos "OA%" ) ))
( "prithee"  uh ( p r i1 dh ii ) ((pos "W-$" ) ))
( "privacy"  n ( p r i1 v @ s ii ) ((pos "L@%" ) ))
( "private"  n ( p r ai1 v i t ) ((pos "K6%" "OA%" ) ))
( "privateer"  n ( p r ai2 v @ t i@1 r ) ((pos "K6%" ) ))
( "privateers"  n ( p r ai2 v @ t i@1 z ) ((pos "Kj%" ) ))
( "privately"  a ( p r ai1 v i t l ii ) ((pos "Pu%" ) ))
( "privates"  n ( p r ai1 v i t s ) ((pos "Kj%" ) ))
( "privation"  n ( p r ai1 v ei1 sh @ n ) ((pos "M6%" ) ))
( "privations"  n ( p r ai1 v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "privet"  n ( p r i1 v i t ) ((pos "L@%" ) ))
( "privies"  n ( p r i1 v i z ) ((pos "Kj%" ) ))
( "privilege"  n ( p r i1 v @ l i jh ) ((pos "M6%" ) ))
( "privileged"  j ( p r i1 v @ l i jh d ) ((pos "OA%" ) ))
( "privileges"  n ( p r i1 v @ l i jh i z ) ((pos "Mj%" ) ))
( "privily"  a ( p r i1 v i l ii ) ((pos "Pu%" ) ))
( "privy"  n ( p r i1 v ii ) ((pos "K8%" "OA%" ) ))
( "prize"  v ( p r ai1 z ) ((pos "H2%" "K6%" ) (subcat "15A" )))
( "prize-fight"  n ( p r ai1 z - f ai t ) ((pos "K6%" ) ))
( "prize-fights"  n ( p r ai1 z - f ai t s ) ((pos "Kj%" ) ))
( "prize-money"  n ( p r ai1 z - m uh n ii ) ((pos "L@%" ) ))
( "prize-ring"  n ( p r ai1 z - r i ng ) ((pos "K6$" ) ))
( "prize-rings"  n ( p r ai1 z - r i ng z ) ((pos "Kj$" ) ))
( "prized"  v ( p r ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "prizes"  v ( p r ai1 z i z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "prizing"  v ( p r ai1 z i ng ) ((pos "Hb$" ) (subcat "15A" )))
( "pro"  n ( p r ou1 ) ((pos "K6%" "Pu%" ) ))
( "pro forma"  j ( p r ou2 - f oo1 m @ ) ((pos "OA%" "Pu%" ) ))
( "pro rata"  a ( p r ou2 - r aa1 t @ ) ((pos "Pu%" ) ))
( "pro tem"  a ( p r ou2 - t e1 m ) ((pos "Pu%" ) ))
( "pro tempore"  a ( p r ou2 - t e1 m p @ r ii ) ((pos "Pu$" ) ))
( "pro-"  j ( p r ou2 - ) ((pos "U-%" ) ))
( "probabilistic"  j ( p r o2 b @ b i l i1 s t i k ) ((pos "OA%" ) ))
( "probabilistically"  a ( p r o2 b @ b i l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "probabilities"  n ( p r o2 b @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "probability"  n ( p r o2 b @ b i1 l i t ii ) ((pos "M8%" ) ))
( "probable"  n ( p r o1 b @ b l ) ((pos "K6%" "OA%" ) ))
( "probables"  n ( p r o1 b @ b l z ) ((pos "Kj%" ) ))
( "probably"  a ( p r o1 b @ b l ii ) ((pos "Pu*" ) ))
( "probate"  v ( p r ou1 b ei t ) ((pos "H2$" "M6%" ) ))
( "probated"  v ( p r ou1 b ei t i d ) ((pos "Hc$" "Hd$" ) ))
( "probates"  v ( p r ou1 b ei t s ) ((pos "Ha$" "Mj$" ) ))
( "probating"  v ( p r ou1 b ei t i ng ) ((pos "Hb$" ) ))
( "probation"  n ( p r @1 b ei1 sh @ n ) ((pos "L@%" ) ))
( "probationary"  j ( p r @1 b ei1 sh @ n r ii ) ((pos "OA%" ) ))
( "probationer"  n ( p r @1 b ei1 sh @ n @ r ) ((pos "K6%" ) ))
( "probationers"  n ( p r @1 b ei1 sh @ n @ z ) ((pos "Kj%" ) ))
( "probe"  v ( p r ou1 b ) ((pos "H2%" "K6%" ) (subcat )))
( "probed"  v ( p r ou1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "probes"  v ( p r ou1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "probing"  v ( p r ou1 b i ng ) ((pos "Hb%" ) (subcat )))
( "probity"  n ( p r ou1 b i t ii ) ((pos "L@%" ) ))
( "problem"  n ( p r o1 b l @ m ) ((pos "K6*" ) ))
( "problematic"  j ( p r o2 b l @ m a1 t i k ) ((pos "OA%" ) ))
( "problematically"  a ( p r o2 b l @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "problems"  n ( p r o1 b l @ m z ) ((pos "Kj%" ) ))
( "proboscis"  n ( p r @1 b o1 s i s ) ((pos "K7%" ) ))
( "proboscises"  n ( p r @1 b o1 s i s i z ) ((pos "Kj$" ) ))
( "procedural"  j ( p r @1 s ii1 jh @ r @ l ) ((pos "OA%" ) ))
( "procedure"  n ( p r @1 s ii1 jh @ r ) ((pos "M6%" ) ))
( "procedures"  n ( p r @1 s ii1 jh @ z ) ((pos "Mj%" ) ))
( "proceed"  v ( p r @1 s ii1 d ) ((pos "I0%" ) (subcat "2A" "3A" )))
( "proceeded"  v ( p r @1 s ii1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "3A" )))
( "proceeding"  v ( p r @1 s ii1 d i ng ) ((pos "Ib%" "M6%" ) (subcat "2A" "3A" )))
( "proceedings"  n ( p r @1 s ii1 d i ng z ) ((pos "Mj%" ) ))
( "proceeds"  n ( p r ou1 s ii d z ) ((pos "Kj%" ) ))
( "proceeds"  v ( p r @1 s ii1 d z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "process"  v ( p r ou1 s e s ) ((pos "H1%" "M7%" ) (subcat )))
( "process"  v ( p r @1 s e1 s ) ((pos "I1%" ) ))
( "process-server"  n ( p r ou1 s i s - s @@ v @ r ) ((pos "K6$" ) ))
( "process-servers"  n ( p r ou1 s i s - s @@ v @ z ) ((pos "Kj$" ) ))
( "processed"  v ( p r ou1 s e s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "processed"  v ( p r @1 s e1 s t ) ((pos "Ic%" "Id%" ) ))
( "processes"  v ( p r ou1 s e s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "processes"  v ( p r @1 s e1 s i z ) ((pos "Ia%" ) ))
( "processing"  v ( p r ou1 s e s i ng ) ((pos "Hb%" ) (subcat )))
( "processing"  v ( p r @1 s e1 s i ng ) ((pos "Ib%" ) ))
( "procession"  n ( p r @1 s e1 sh @ n ) ((pos "M6%" ) ))
( "processional"  j ( p r @1 s e1 sh @ n @ l ) ((pos "OA%" ) ))
( "processions"  n ( p r @1 s e1 sh @ n z ) ((pos "Mj%" ) ))
( "processor"  n ( p r ou1 s e s @ r ) ((pos "K6%" ) ))
( "processors"  n ( p r ou1 s e s @ z ) ((pos "Kj%" ) ))
( "proclaim"  v ( p r @1 k l ei1 m ) ((pos "H0%" ) (subcat "6A" "9" "23" )))
( "proclaimed"  v ( p r @1 k l ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "23" )))
( "proclaiming"  v ( p r @1 k l ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" "9" "23" )))
( "proclaims"  v ( p r @1 k l ei1 m z ) ((pos "Ha%" ) (subcat "6A" "9" "23" )))
( "proclamation"  n ( p r o2 k l @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "proclamations"  n ( p r o2 k l @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "proclivities"  n ( p r @1 k l i1 v i t i z ) ((pos "Kj%" ) ))
( "proclivity"  n ( p r @1 k l i1 v i t ii ) ((pos "K8%" ) ))
( "proconsul"  n ( p r ou2 k o1 n s l ) ((pos "K6%" ) ))
( "proconsular"  j ( p r ou2 k o1 n s y u l @ r ) ((pos "OA%" ) ))
( "proconsulate"  n ( p r ou1 k o1 n s y u l @ t ) ((pos "K6%" ) ))
( "proconsulates"  n ( p r ou1 k o1 n s y u l @ t s ) ((pos "Kj%" ) ))
( "proconsuls"  n ( p r ou2 k o1 n s l z ) ((pos "Kj%" ) ))
( "procrastinate"  v ( p r ou1 k r a1 s t i n ei t ) ((pos "I2%" ) (subcat )))
( "procrastinated"  v ( p r ou1 k r a1 s t i n ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "procrastinates"  v ( p r ou1 k r a1 s t i n ei t s ) ((pos "Ia%" ) (subcat )))
( "procrastinating"  v ( p r ou1 k r a1 s t i n ei t i ng ) ((pos "Ib%" ) (subcat )))
( "procrastination"  n ( p r ou1 k r a2 s t i n ei1 sh @ n ) ((pos "L@%" ) ))
( "procreate"  v ( p r ou1 k r i ei t ) ((pos "H2%" ) (subcat )))
( "procreated"  v ( p r ou1 k r i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "procreates"  v ( p r ou1 k r i ei t s ) ((pos "Ha%" ) (subcat )))
( "procreating"  v ( p r ou1 k r i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "procreation"  n ( p r ou2 k r i ei1 sh @ n ) ((pos "K6%" ) ))
( "procreations"  n ( p r ou2 k r i ei1 sh @ n z ) ((pos "Kj%" ) ))
( "proctor"  n ( p r o1 k t @ r ) ((pos "K6%" ) ))
( "proctors"  n ( p r o1 k t @ z ) ((pos "Kj%" ) ))
( "procurable"  j ( p r @1 k y u@1 r @ b l ) ((pos "OA%" ) ))
( "procurator"  n ( p r o1 k y u r ei t @ r ) ((pos "K6%" ) ))
( "procurators"  n ( p r o1 k y u r ei t @ z ) ((pos "Kj%" ) ))
( "procure"  v ( p r @1 k y u@1 r ) ((pos "H2%" ) (subcat "6A" "12B" )))
( "procured"  v ( p r @1 k y u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12B" )))
( "procurement"  n ( p r @1 k y u@1 m @ n t ) ((pos "K6%" ) ))
( "procurements"  n ( p r @1 k y u@1 m @ n t s ) ((pos "Kj$" ) ))
( "procurer"  n ( p r @1 k y u@1 r @ r ) ((pos "K6%" ) ))
( "procurers"  n ( p r @1 k y u@1 r @ z ) ((pos "Kj%" ) ))
( "procures"  v ( p r @1 k y u@1 z ) ((pos "Ha%" ) (subcat "6A" "12B" )))
( "procuress"  n ( p r o1 k y u r i s ) ((pos "K7%" ) ))
( "procuresses"  n ( p r o1 k y u r i s i z ) ((pos "Kj$" ) ))
( "procuring"  v ( p r @1 k y u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "12B" )))
( "prod"  v ( p r o1 d ) ((pos "J4%" "K6%" ) (subcat "3A" )))
( "prodded"  v ( p r o1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "prodding"  v ( p r o1 d i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "prodigal"  n ( p r o1 d i g l ) ((pos "K6$" "OA%" ) ))
( "prodigality"  n ( p r o2 d i g a1 l i t ii ) ((pos "L@%" ) ))
( "prodigally"  a ( p r o1 d i g @ l ii ) ((pos "Pu%" ) ))
( "prodigals"  n ( p r o1 d i g l z ) ((pos "Kj$" ) ))
( "prodigies"  n ( p r o1 d i jh i z ) ((pos "Kj%" ) ))
( "prodigious"  j ( p r @1 d i1 jh @ s ) ((pos "OA%" ) ))
( "prodigiously"  a ( p r @1 d i1 jh @ s l ii ) ((pos "Pu%" ) ))
( "prodigy"  n ( p r o1 d i jh ii ) ((pos "K8%" ) ))
( "prods"  v ( p r o1 d z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "produce"  n ( p r o1 d y uu s ) ((pos "L@%" ) ))
( "produce"  v ( p r @1 d y uu1 s ) ((pos "J2%" ) (subcat "2A" )))
( "produced"  v ( p r @1 d y uu1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "producer"  n ( p r @1 d y uu1 s @ r ) ((pos "K6%" ) ))
( "producers"  n ( p r @1 d y uu1 s @ z ) ((pos "Kj%" ) ))
( "produces"  v ( p r @1 d y uu1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "producing"  v ( p r @1 d y uu1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "product"  n ( p r o1 d uh k t ) ((pos "K6%" ) ))
( "production"  n ( p r @1 d uh1 k sh @ n ) ((pos "M6%" ) ))
( "productions"  n ( p r @1 d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "productive"  j ( p r @1 d uh1 k t i v ) ((pos "OA%" ) ))
( "productively"  a ( p r @1 d uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "productivity"  n ( p r o2 d uh k t i1 v i t ii ) ((pos "L@%" ) ))
( "products"  n ( p r o1 d uh k t s ) ((pos "Kj%" ) ))
( "profanation"  n ( p r o2 f @ n ei1 sh @ n ) ((pos "M6$" ) ))
( "profanations"  n ( p r o2 f @ n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "profane"  v ( p r @1 f ei1 n ) ((pos "H2%" "OA%" ) (subcat )))
( "profaned"  v ( p r @1 f ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "profanely"  a ( p r @1 f ei1 n l ii ) ((pos "Pu%" ) ))
( "profaneness"  n ( p r @1 f ei1 n - n @ s ) ((pos "L@$" ) ))
( "profanes"  v ( p r @1 f ei1 n z ) ((pos "Ha%" ) (subcat )))
( "profaning"  v ( p r @1 f ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "profanities"  n ( p r @1 f a1 n i t i z ) ((pos "Mj%" ) ))
( "profanity"  n ( p r @1 f a1 n i t ii ) ((pos "M8%" ) ))
( "profess"  v ( p r @1 f e1 s ) ((pos "J1%" ) (subcat "6A" "7A" "9" )))
( "professed"  v ( p r @1 f e1 s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "6A" "7A" "9" )))
( "professedly"  a ( p r @1 f e1 s t l ii ) ((pos "Pu%" ) ))
( "professes"  v ( p r @1 f e1 s i z ) ((pos "Ja%" ) (subcat "6A" "7A" "9" )))
( "professing"  v ( p r @1 f e1 s i ng ) ((pos "Jb%" ) (subcat "6A" "7A" "9" )))
( "profession"  n ( p r @1 f e1 sh @ n ) ((pos "K6%" ) ))
( "professional"  n ( p r @1 f e1 sh @ n @ l ) ((pos "K6%" "OA%" ) ))
( "professionalism"  n ( p r @1 f e1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "professionally"  a ( p r @1 f e1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "professionals"  n ( p r @1 f e1 sh @ n @ l z ) ((pos "Kj%" ) ))
( "professions"  n ( p r @1 f e1 sh @ n z ) ((pos "Kj%" ) ))
( "professor"  n ( p r @1 f e1 s @ r ) ((pos "K6%" ) ))
( "professorial"  j ( p r o2 f i s oo1 r i@ l ) ((pos "OA%" ) ))
( "professors"  n ( p r @1 f e1 s @ z ) ((pos "Kj%" ) ))
( "professorship"  n ( p r @1 f e1 s @ sh i p ) ((pos "K6%" ) ))
( "professorships"  n ( p r @1 f e1 s @ sh i p s ) ((pos "Kj%" ) ))
( "proffer"  v ( p r o1 f @ r ) ((pos "H0%" "K6$" ) (subcat "6A" )))
( "proffered"  v ( p r o1 f @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "proffering"  v ( p r o1 f @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "proffers"  v ( p r o1 f @ z ) ((pos "Ha%" "Kj$" ) (subcat "6A" )))
( "proficiency"  n ( p r @1 f i1 sh @ n s ii ) ((pos "L@%" ) ))
( "proficient"  j ( p r @1 f i1 sh @ n t ) ((pos "OA%" ) ))
( "proficiently"  a ( p r @1 f i1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "profile"  v ( p r ou1 f ai l ) ((pos "H2%" "M6%" ) ))
( "profiled"  v ( p r ou1 f ai l d ) ((pos "Hc%" "Hd%" ) ))
( "profiles"  v ( p r ou1 f ai l z ) ((pos "Ha%" "Mj%" ) ))
( "profiling"  v ( p r ou1 f ai l i ng ) ((pos "Hb%" ) ))
( "profit"  v ( p r o1 f i t ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" )))
( "profit-margin"  n ( p r o1 f i t - m aa jh i n ) ((pos "K6%" ) ))
( "profit-margins"  n ( p r o1 f i t - m aa jh i n z ) ((pos "Kj%" ) ))
( "profit-sharing"  n ( p r o1 f i t - sh e@ r i ng ) ((pos "L@%" ) ))
( "profitable"  j ( p r o1 f i t @ b l ) ((pos "OA%" ) ))
( "profitably"  a ( p r o1 f i t @ b l ii ) ((pos "Pu%" ) ))
( "profited"  v ( p r o1 f i t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "profiteer"  v ( p r o2 f i t i@1 r ) ((pos "I0%" "K6%" ) (subcat )))
( "profiteered"  v ( p r o2 f i t i@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "profiteering"  v ( p r o2 f i t i@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "profiteers"  v ( p r o2 f i t i@1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "profiting"  v ( p r o1 f i t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "profitless"  j ( p r o1 f i t l @ s ) ((pos "OA%" ) ))
( "profitlessly"  a ( p r o1 f i t l @ s l ii ) ((pos "Pu%" ) ))
( "profits"  v ( p r o1 f i t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" )))
( "profligacy"  n ( p r o1 f l i g @ s ii ) ((pos "L@%" ) ))
( "profligate"  n ( p r o1 f l i g @ t ) ((pos "K6$" "OA%" ) ))
( "profligates"  n ( p r o1 f l i g @ t s ) ((pos "Kj$" ) ))
( "profound"  j ( p r @1 f au1 n d ) ((pos "OA%" ) ))
( "profoundly"  a ( p r @1 f au1 n d l ii ) ((pos "Pu%" ) ))
( "profundities"  n ( p r @1 f uh1 n d i t i z ) ((pos "Mj%" ) ))
( "profundity"  n ( p r @1 f uh1 n d i t ii ) ((pos "M8%" ) ))
( "profuse"  j ( p r @1 f y uu1 s ) ((pos "OA%" ) ))
( "profusely"  a ( p r @1 f y uu1 s l ii ) ((pos "Pu%" ) ))
( "profuseness"  n ( p r @1 f y uu1 s n @ s ) ((pos "L@%" ) ))
( "profusion"  n ( p r @1 f y uu1 zh n ) ((pos "L@%" ) ))
( "progenitor"  n ( p r ou1 jh e1 n i t @ r ) ((pos "K6%" ) ))
( "progenitors"  n ( p r ou1 jh e1 n i t @ z ) ((pos "Kj%" ) ))
( "progeny"  n ( p r o1 jh @ n ii ) ((pos "Kj%" ) ))
( "prognoses"  n ( p r o1 g n ou1 s ii z ) ((pos "Kj%" ) ))
( "prognosis"  n ( p r o1 g n ou1 s i s ) ((pos "Ki%" ) ))
( "prognostic"  n ( p r o1 g n o1 s t i k ) ((pos "K6$" "OA%" ) ))
( "prognosticate"  v ( p r o1 g n o1 s t i k ei t ) ((pos "H2%" ) (subcat "6A" )))
( "prognosticated"  v ( p r o1 g n o1 s t i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prognosticates"  v ( p r o1 g n o1 s t i k ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "prognosticating"  v ( p r o1 g n o1 s t i k ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "prognostication"  n ( p r o1 g n o2 s t i k ei1 sh @ n ) ((pos "M6%" ) ))
( "prognostications"  n ( p r o1 g n o2 s t i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "prognostics"  n ( p r o1 g n o1 s t i k s ) ((pos "Kj$" ) ))
( "program"  v ( p r ou1 g r a m ) ((pos "H4%" "K6%" ) (subcat )))
( "programme"  v ( p r ou1 g r a m ) ((pos "H2%" "K6%" ) (subcat )))
( "programmed"  v ( p r ou1 g r a m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "programmer"  n ( p r ou1 g r a m @ r ) ((pos "K6%" ) ))
( "programmers"  n ( p r ou1 g r a m @ z ) ((pos "Kj%" ) ))
( "programmes"  v ( p r ou1 g r a m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "programming"  v ( p r ou1 g r a m i ng ) ((pos "Hb%" ) (subcat )))
( "programs"  v ( p r ou1 g r a m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "progress"  n ( p r ou1 g r e s ) ((pos "M7%" ) ))
( "progress"  v ( p r @1 g r e1 s ) ((pos "I1%" ) (subcat "2A" )))
( "progressed"  v ( p r @1 g r e1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "progresses"  n ( p r ou1 g r e s i z ) ((pos "Mj$" ) ))
( "progresses"  v ( p r @1 g r e1 s i z ) ((pos "Ia%" ) (subcat "2A" )))
( "progressing"  v ( p r @1 g r e1 s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "progression"  n ( p r @1 g r e1 sh @ n ) ((pos "L@%" ) ))
( "progressive"  n ( p r @1 g r e1 s i v ) ((pos "K6$" "OA%" ) ))
( "progressively"  a ( p r @1 g r e1 s i v l ii ) ((pos "Pu%" ) ))
( "progressiveness"  n ( p r @1 g r e1 s i v n @ s ) ((pos "L@%" ) ))
( "progressives"  n ( p r @1 g r e1 s i v z ) ((pos "Kj$" ) ))
( "prohibit"  v ( p r @1 h i1 b i t ) ((pos "H0%" ) (subcat "6A" )))
( "prohibited"  v ( p r @1 h i1 b i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prohibiting"  v ( p r @1 h i1 b i t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "prohibition"  n ( p r ou2 i b i1 sh @ n ) ((pos "M6%" ) ))
( "prohibitionist"  n ( p r ou2 i b i1 sh @ n i s t ) ((pos "K6%" ) ))
( "prohibitionists"  n ( p r ou2 i b i1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "prohibitions"  n ( p r ou2 i b i1 sh @ n z ) ((pos "Mj%" ) ))
( "prohibitive"  j ( p r @1 h i1 b @ t i v ) ((pos "OA%" ) ))
( "prohibitively"  a ( p r @1 h i1 b @ t i v l ii ) ((pos "Pu%" ) ))
( "prohibitory"  j ( p r @1 h i1 b i t @ r ii ) ((pos "OA%" ) ))
( "prohibits"  v ( p r @1 h i1 b i t s ) ((pos "Ha%" ) (subcat "6A" )))
( "project"  n ( p r o1 jh e k t ) ((pos "K6%" ) ))
( "project"  v ( p r @1 jh e1 k t ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "14" )))
( "projected"  v ( p r @1 jh e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "projectile"  n ( p r @1 jh e1 k t ai l ) ((pos "K6%" "OA%" ) ))
( "projectiles"  n ( p r @1 jh e1 k t ai l z ) ((pos "Kj%" ) ))
( "projecting"  v ( p r @1 jh e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "projection"  n ( p r @1 jh e1 k sh @ n ) ((pos "M6%" ) ))
( "projectionist"  n ( p r @1 jh e1 k sh @ n i s t ) ((pos "K6%" ) ))
( "projectionists"  n ( p r @1 jh e1 k sh @ n i s t s ) ((pos "Kj%" ) ))
( "projections"  n ( p r @1 jh e1 k sh @ n z ) ((pos "Mj%" ) ))
( "projector"  n ( p r @1 jh e1 k t @ r ) ((pos "K6%" ) ))
( "projectors"  n ( p r @1 jh e1 k t @ z ) ((pos "Kj%" ) ))
( "projects"  n ( p r o1 jh e k t s ) ((pos "Kj%" ) ))
( "projects"  v ( p r @1 jh e1 k t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" )))
( "prolapse"  n ( p r ou1 l a p s ) ((pos "K6$" ) ))
( "prolapse"  v ( p r ou1 l a1 p s ) ((pos "I2$" ) ))
( "prolapsed"  v ( p r ou1 l a1 p s t ) ((pos "Ic$" "Id$" ) ))
( "prolapses"  n ( p r ou1 l a p s i z ) ((pos "Kj$" ) ))
( "prolapses"  v ( p r ou1 l a1 p s i z ) ((pos "Ia$" ) ))
( "prolapsing"  v ( p r ou1 l a1 p s i ng ) ((pos "Ib$" ) ))
( "prole"  n ( p r ou1 l ) ((pos "K6$" ) ))
( "proles"  n ( p r ou1 l z ) ((pos "Kj$" ) ))
( "proletarian"  n ( p r ou2 l i t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "proletarians"  n ( p r ou2 l i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "proletariat"  n ( p r ou2 l i t e@1 r i@ t ) ((pos "K6%" ) ))
( "proletariats"  n ( p r ou2 l i t e@1 r i@ t s ) ((pos "Kj%" ) ))
( "proliferate"  v ( p r @1 l i1 f @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "proliferated"  v ( p r @1 l i1 f @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "proliferates"  v ( p r @1 l i1 f @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "proliferating"  v ( p r @1 l i1 f @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "proliferation"  n ( p r @1 l i2 f @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "proliferations"  n ( p r @1 l i2 f @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "prolific"  j ( p r @1 l i1 f i k ) ((pos "OA%" ) ))
( "prolix"  j ( p r ou1 l i k s ) ((pos "OA%" ) ))
( "prolixity"  n ( p r ou1 l i1 k s i t ii ) ((pos "L@%" ) ))
( "prologue"  n ( p r ou1 l o g ) ((pos "K6%" ) ))
( "prologues"  n ( p r ou1 l o g z ) ((pos "Kj%" ) ))
( "prolong"  v ( p r @1 l o1 ng ) ((pos "H0%" ) (subcat )))
( "prolongation"  n ( p r ou2 l o ng g ei1 sh @ n ) ((pos "M6%" ) ))
( "prolongations"  n ( p r ou2 l o ng g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "prolonged"  v ( p r @1 l o1 ng d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "prolonging"  v ( p r @1 l o1 ng i ng ) ((pos "Hb%" ) (subcat )))
( "prolongs"  v ( p r @1 l o1 ng z ) ((pos "Ha%" ) (subcat )))
( "prom"  n ( p r o1 m ) ((pos "K6%" ) ))
( "promenade"  v ( p r o2 m @ n aa1 d ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "promenaded"  v ( p r o2 m @ n aa1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "promenades"  v ( p r o2 m @ n aa1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "promenading"  v ( p r o2 m @ n aa1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "prominence"  n ( p r o1 m i n @ n s ) ((pos "M6%" ) ))
( "prominences"  n ( p r o1 m i n @ n s i z ) ((pos "Mj$" ) ))
( "prominent"  j ( p r o1 m i n @ n t ) ((pos "OA%" ) ))
( "prominently"  a ( p r o1 m i n @ n t l ii ) ((pos "Pu%" ) ))
( "promiscuity"  n ( p r o2 m i s k y uu1 i t ii ) ((pos "L@%" ) ))
( "promiscuous"  j ( p r @1 m i1 s k y u@ s ) ((pos "OA%" ) ))
( "promiscuously"  a ( p r @1 m i1 s k y u@ s l ii ) ((pos "Pu%" ) ))
( "promise"  v ( p r o1 m i s ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" "7A" "9" "11" "12A" "13A" )))
( "promised"  v ( p r o1 m i s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "7A" "9" "11" "12A" "13A" )))
( "promises"  v ( p r o1 m i s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "7A" "9" "11" "12A" "13A" )))
( "promising"  v ( p r o1 m i s i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "7A" "9" "11" "12A" "13A" )))
( "promisingly"  a ( p r o1 m i s i ng l ii ) ((pos "Pu%" ) ))
( "promissory"  j ( p r o1 m i s @ r ii ) ((pos "OA$" ) ))
( "promontories"  n ( p r o1 m @ n t r i z ) ((pos "Kj%" ) ))
( "promontory"  n ( p r o1 m @ n t r ii ) ((pos "K8%" ) ))
( "promote"  v ( p r @1 m ou1 t ) ((pos "H2%" ) (subcat "6A" )))
( "promoted"  v ( p r @1 m ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "promoter"  n ( p r @1 m ou1 t @ r ) ((pos "K6%" ) ))
( "promoters"  n ( p r @1 m ou1 t @ z ) ((pos "Kj%" ) ))
( "promotes"  v ( p r @1 m ou1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "promoting"  v ( p r @1 m ou1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "promotion"  n ( p r @1 m ou1 sh @ n ) ((pos "M6%" ) ))
( "promotional"  j ( p r @1 m ou1 sh @ n l ) ((pos "OA%" ) ))
( "promotions"  n ( p r @1 m ou1 sh @ n z ) ((pos "Mj%" ) ))
( "prompt"  v ( p r o1 m p t ) ((pos "H0%" "K6%" "OA%" ) (subcat "6A" )))
( "prompt-box"  n ( p r o1 m p t - b o k s ) ((pos "K7$" ) ))
( "prompt-boxes"  n ( p r o1 m p t - b o k s i z ) ((pos "Kj$" ) ))
( "prompt-copies"  n ( p r o1 m p t - k o p i z ) ((pos "Kj%" ) ))
( "prompt-copy"  n ( p r o1 m p t - k o p ii ) ((pos "K8%" ) ))
( "prompted"  v ( p r o1 m p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prompter"  n ( p r o1 m p t @ r ) ((pos "K6%" ) ))
( "prompters"  n ( p r o1 m p t @ z ) ((pos "Kj%" ) ))
( "prompting"  v ( p r o1 m p t i ng ) ((pos "Hb%" "K6$" ) (subcat "6A" )))
( "promptings"  n ( p r o1 m p t i ng z ) ((pos "Kj%" ) ))
( "promptitude"  n ( p r o1 m p t i t y uu d ) ((pos "L@$" ) ))
( "promptly"  a ( p r o1 m p t l ii ) ((pos "Pu%" ) ))
( "promptness"  n ( p r o1 m p t n @ s ) ((pos "L@%" ) ))
( "prompts"  v ( p r o1 m p t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "proms"  n ( p r o1 m z ) ((pos "Kj%" ) ))
( "promulgate"  v ( p r o1 m @ l g ei t ) ((pos "H2%" ) (subcat )))
( "promulgated"  v ( p r o1 m @ l g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "promulgates"  v ( p r o1 m @ l g ei t s ) ((pos "Ha%" ) (subcat )))
( "promulgating"  v ( p r o1 m @ l g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "promulgation"  n ( p r o2 m @ l g ei1 sh @ n ) ((pos "L@%" ) ))
( "prone"  j ( p r ou1 n ) ((pos "OA%" ) ))
( "proneness"  n ( p r ou1 n - n @ s ) ((pos "L@%" ) ))
( "prong"  n ( p r o1 ng ) ((pos "K6%" ) ))
( "pronged"  j ( p r o1 ng d ) ((pos "OA%" ) ))
( "prongs"  n ( p r o1 ng z ) ((pos "Kj%" ) ))
( "pronominal"  j ( p r ou1 n o1 m i n l ) ((pos "OA$" ) ))
( "pronoun"  n ( p r ou1 n au n ) ((pos "K6%" ) ))
( "pronounce"  v ( p r @1 n au1 n s ) ((pos "J2%" ) (subcat "2A" "3A" "6A" "9" "22" )))
( "pronounceable"  j ( p r @1 n au1 n s @ b l ) ((pos "OA%" ) ))
( "pronounced"  v ( p r @1 n au1 n s t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" "9" "22" )))
( "pronouncement"  n ( p r @1 n au1 n s m @ n t ) ((pos "K6%" ) ))
( "pronouncements"  n ( p r @1 n au1 n s m @ n t s ) ((pos "Kj%" ) ))
( "pronounces"  v ( p r @1 n au1 n s i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "9" "22" )))
( "pronouncing"  v ( p r @1 n au1 n s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "9" "22" )))
( "pronouns"  n ( p r ou1 n au n z ) ((pos "Kj%" ) ))
( "pronto"  a ( p r o1 n t ou ) ((pos "Pu$" ) ))
( "pronunciamento"  n ( p r @1 n uh2 n s i@ m e1 n t ou ) ((pos "K6$" ) ))
( "pronunciamentos"  n ( p r @1 n uh2 n s i@ m e1 n t ou z ) ((pos "Kj$" ) ))
( "pronunciation"  n ( p r @1 n uh2 n s i ei1 sh @ n ) ((pos "M6%" ) ))
( "pronunciations"  n ( p r @1 n uh2 n s i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "proof"  v ( p r uu1 f ) ((pos "H0%" "M6%" "OA%" ) (subcat )))
( "proofed"  v ( p r uu1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "proofing"  v ( p r uu1 f i ng ) ((pos "Hb%" ) (subcat )))
( "proofread"  v ( p r uu1 f r e d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "proofread"  v ( p r uu1 f r ii d ) ((pos "J5%" ) (subcat "2A" )))
( "proofreader"  n ( p r uu1 f r ii d @ r ) ((pos "K6%" ) ))
( "proofreaders"  n ( p r uu1 f r ii d @ z ) ((pos "Kj%" ) ))
( "proofreading"  v ( p r uu1 f r ii d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "proofreads"  v ( p r uu1 f r ii d z ) ((pos "Ja%" ) (subcat "2A" )))
( "proofs"  v ( p r uu1 f s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "prop"  v ( p r o1 p ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" "15B" )))
( "propaganda"  n ( p r o2 p @ g a1 n d @ ) ((pos "L@%" ) ))
( "propagandist"  n ( p r o2 p @ g a1 n d i s t ) ((pos "K6%" ) ))
( "propagandists"  n ( p r o2 p @ g a1 n d i s t s ) ((pos "Kj%" ) ))
( "propagandize"  v ( p r o2 p @ g a1 n d ai z ) ((pos "I2$" ) ))
( "propagandized"  v ( p r o2 p @ g a1 n d ai z d ) ((pos "Ic$" "Id$" ) ))
( "propagandizes"  v ( p r o2 p @ g a1 n d ai z i z ) ((pos "Ia$" ) ))
( "propagandizing"  v ( p r o2 p @ g a1 n d ai z i ng ) ((pos "Ib$" ) ))
( "propagate"  v ( p r o1 p @ g ei t ) ((pos "J2%" ) (subcat "2A" )))
( "propagated"  v ( p r o1 p @ g ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "propagates"  v ( p r o1 p @ g ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "propagating"  v ( p r o1 p @ g ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "propagation"  n ( p r o2 p @ g ei1 sh @ n ) ((pos "L@%" ) ))
( "propagator"  n ( p r o1 p @ g ei t @ r ) ((pos "K6%" ) ))
( "propagators"  n ( p r o1 p @ g ei t @ z ) ((pos "Kj%" ) ))
( "propane"  n ( p r ou1 p ei n ) ((pos "L@$" ) ))
( "propel"  v ( p r @1 p e1 l ) ((pos "H4%" ) (subcat "6A" )))
( "propellant"  n ( p r @1 p e1 l @ n t ) ((pos "M6%" "OA%" ) ))
( "propellants"  n ( p r @1 p e1 l @ n t s ) ((pos "Mj%" ) ))
( "propelled"  v ( p r @1 p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "propellent"  n ( p r @1 p e1 l @ n t ) ((pos "M6%" "OA%" ) ))
( "propellents"  n ( p r @1 p e1 l @ n t s ) ((pos "Mj%" ) ))
( "propeller"  n ( p r @1 p e1 l @ r ) ((pos "K6%" ) ))
( "propellers"  n ( p r @1 p e1 l @ z ) ((pos "Kj%" ) ))
( "propelling"  v ( p r @1 p e1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "propels"  v ( p r @1 p e1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "propensities"  n ( p r @1 p e1 n s i t i z ) ((pos "Mj%" ) ))
( "propensity"  n ( p r @1 p e1 n s i t ii ) ((pos "M8%" ) ))
( "proper"  j ( p r o1 p @ r ) ((pos "OA%" ) ))
( "properly"  a ( p r o1 p @ l ii ) ((pos "Pu%" ) ))
( "propertied"  j ( p r o1 p @ t i d ) ((pos "OA%" ) ))
( "properties"  n ( p r o1 p @ t i z ) ((pos "Mj%" ) ))
( "property"  n ( p r o1 p @ t ii ) ((pos "M8%" ) ))
( "property-man"  n ( p r o1 p @ t i - m a n ) ((pos "Ki$" ) ))
( "property-master"  n ( p r o1 p @ t i - m aa s t @ r ) ((pos "K6$" ) ))
( "property-masters"  n ( p r o1 p @ t i - m aa s t @ z ) ((pos "Kj$" ) ))
( "property-men"  n ( p r o1 p @ t i - m e n ) ((pos "Kj$" ) ))
( "prophecies"  n ( p r o1 f @ s i z ) ((pos "Mj%" ) ))
( "prophecy"  n ( p r o1 f @ s ii ) ((pos "M8%" ) ))
( "prophesied"  v ( p r o1 f i s ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" )))
( "prophesies"  v ( p r o1 f i s ai z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "9" )))
( "prophesy"  v ( p r o1 f i s ai ) ((pos "J3%" ) (subcat "2A" "2C" "6A" "9" )))
( "prophesying"  v ( p r o1 f i s ai i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "9" )))
( "prophet"  n ( p r o1 f i t ) ((pos "K6%" ) ))
( "prophetess"  n ( p r o1 f i t e s ) ((pos "K7%" ) ))
( "prophetesses"  n ( p r o1 f i t e s i z ) ((pos "Kj%" ) ))
( "prophetic"  j ( p r @1 f e1 t i k ) ((pos "OA%" ) ))
( "prophetical"  j ( p r @1 f e1 t i k l ) ((pos "OA$" ) ))
( "prophetically"  a ( p r @1 f e1 t i k l ii ) ((pos "Pu%" ) ))
( "prophets"  n ( p r o1 f i t s ) ((pos "Kj%" ) ))
( "prophylactic"  n ( p r o2 f i l a1 k t i k ) ((pos "K6%" "OA%" ) ))
( "prophylactics"  n ( p r o2 f i l a1 k t i k s ) ((pos "Kj%" ) ))
( "prophylaxis"  n ( p r o2 f i l a1 k s i s ) ((pos "L@%" ) ))
( "propinquity"  n ( p r @1 p i1 ng k w i t ii ) ((pos "L@$" ) ))
( "propitiate"  v ( p r @1 p i1 sh i ei t ) ((pos "H2%" ) (subcat )))
( "propitiated"  v ( p r @1 p i1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "propitiates"  v ( p r @1 p i1 sh i ei t s ) ((pos "Ha%" ) (subcat )))
( "propitiating"  v ( p r @1 p i1 sh i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "propitiation"  n ( p r @1 p i2 sh i ei1 sh @ n ) ((pos "L@%" ) ))
( "propitiatory"  j ( p r @1 p i1 sh i@ t r ii ) ((pos "OA%" ) ))
( "propitious"  j ( p r @1 p i1 sh @ s ) ((pos "OA%" ) ))
( "propitiously"  a ( p r @1 p i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "proponent"  n ( p r @1 p ou1 n @ n t ) ((pos "K6%" ) ))
( "proponents"  n ( p r @1 p ou1 n @ n t s ) ((pos "Kj%" ) ))
( "proportion"  v ( p r @1 p oo1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "proportionable"  j ( p r @1 p oo1 sh @ n @ b l ) ((pos "OA$" ) ))
( "proportional"  j ( p r @1 p oo1 sh @ n l ) ((pos "OA%" ) ))
( "proportionality"  n ( p r @1 p oo2 sh @ n a1 l i t ii ) ((pos "L@%" ) ))
( "proportionally"  a ( p r @1 p oo1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "proportionate"  j ( p r @1 p oo1 sh @ n @ t ) ((pos "OA%" ) ))
( "proportionately"  a ( p r @1 p oo1 sh @ n @ t l ii ) ((pos "Pu%" ) ))
( "proportioned"  v ( p r @1 p oo1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "proportioning"  v ( p r @1 p oo1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "proportions"  v ( p r @1 p oo1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "proposal"  n ( p r @1 p ou1 z l ) ((pos "M6%" ) ))
( "proposals"  n ( p r @1 p ou1 z l z ) ((pos "Mj%" ) ))
( "propose"  v ( p r @1 p ou1 z ) ((pos "J2%" ) (subcat "2A" "6A" "6D" "7A" "9" )))
( "proposed"  v ( p r @1 p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "6D" "7A" "9" )))
( "proposer"  n ( p r @1 p ou1 z @ r ) ((pos "K6%" ) ))
( "proposers"  n ( p r @1 p ou1 z @ z ) ((pos "Kj%" ) ))
( "proposes"  v ( p r @1 p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "6A" "6D" "7A" "9" )))
( "proposing"  v ( p r @1 p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "6D" "7A" "9" )))
( "proposition"  v ( p r o2 p @ z i1 sh @ n ) ((pos "H0%" "K6%" ) (subcat )))
( "propositioned"  v ( p r o2 p @ z i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "propositioning"  v ( p r o2 p @ z i1 sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "propositions"  v ( p r o2 p @ z i1 sh @ n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "propound"  v ( p r @1 p au1 n d ) ((pos "H0%" ) (subcat )))
( "propounded"  v ( p r @1 p au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "propounding"  v ( p r @1 p au1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "propounds"  v ( p r @1 p au1 n d z ) ((pos "Ha%" ) (subcat )))
( "propped"  v ( p r o1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" "15B" )))
( "propping"  v ( p r o1 p i ng ) ((pos "Hb%" ) (subcat "6A" "15A" "15B" )))
( "proprietary"  j ( p r @1 p r ai1 @ t r ii ) ((pos "OA%" ) ))
( "proprieties"  n ( p r @1 p r ai1 @ t i z ) ((pos "Mj%" ) ))
( "proprietor"  n ( p r @1 p r ai1 @ t @ r ) ((pos "K6%" ) ))
( "proprietors"  n ( p r @1 p r ai1 @ t @ z ) ((pos "Kj%" ) ))
( "proprietress"  n ( p r @1 p r ai1 @ t r i s ) ((pos "K7%" ) ))
( "proprietresses"  n ( p r @1 p r ai1 @ t r i s i z ) ((pos "Kj%" ) ))
( "propriety"  n ( p r @1 p r ai1 @ t ii ) ((pos "M8%" ) ))
( "props"  v ( p r o1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" "15B" )))
( "propulsion"  n ( p r @1 p uh1 l sh @ n ) ((pos "L@%" ) ))
( "propulsive"  j ( p r @1 p uh1 l s i v ) ((pos "OA%" ) ))
( "prorogation"  n ( p r ou2 r @ g ei1 sh @ n ) ((pos "K6$" ) ))
( "prorogations"  n ( p r ou2 r @ g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "prorogue"  v ( p r ou1 r ou1 g ) ((pos "H2$" ) (subcat )))
( "prorogued"  v ( p r ou1 r ou1 g d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "prorogues"  v ( p r ou1 r ou1 g z ) ((pos "Ha$" ) (subcat )))
( "proroguing"  v ( p r ou1 r ou1 g i ng ) ((pos "Hb$" ) (subcat )))
( "pros"  n ( p r ou1 z ) ((pos "Kj%" ) ))
( "prosaic"  j ( p r @1 z ei1 i k ) ((pos "OA%" ) ))
( "prosaically"  a ( p r @1 z ei1 i k l ii ) ((pos "Pu%" ) ))
( "proscenium"  n ( p r @1 s ii1 n i@ m ) ((pos "K6%" ) ))
( "prosceniums"  n ( p r @1 s ii1 n i@ m z ) ((pos "Kj$" ) ))
( "proscribe"  v ( p r @1 s k r ai1 b ) ((pos "H2%" ) (subcat )))
( "proscribed"  v ( p r @1 s k r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "proscribes"  v ( p r @1 s k r ai1 b z ) ((pos "Ha%" ) (subcat )))
( "proscribing"  v ( p r @1 s k r ai1 b i ng ) ((pos "Hb%" ) (subcat )))
( "proscription"  n ( p r @1 s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "proscriptions"  n ( p r @1 s k r i1 p sh @ n z ) ((pos "Mj%" ) ))
( "prose"  n ( p r ou1 z ) ((pos "L@%" ) ))
( "prosecute"  v ( p r o1 s i k y uu t ) ((pos "H2%" ) (subcat "6A" )))
( "prosecuted"  v ( p r o1 s i k y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "prosecutes"  v ( p r o1 s i k y uu t s ) ((pos "Ha%" ) (subcat "6A" )))
( "prosecuting"  v ( p r o1 s i k y uu t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "prosecution"  n ( p r o2 s i k y uu1 sh @ n ) ((pos "M6%" ) ))
( "prosecutions"  n ( p r o2 s i k y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "prosecutor"  n ( p r o1 s i k y uu t @ r ) ((pos "K6%" ) ))
( "prosecutors"  n ( p r o1 s i k y uu t @ z ) ((pos "Kj%" ) ))
( "proselyte"  n ( p r o1 s @ l ai t ) ((pos "K6$" ) ))
( "proselytes"  n ( p r o1 s @ l ai t s ) ((pos "Kj$" ) ))
( "proselytize"  v ( p r o1 s @ l i t ai z ) ((pos "J2$" ) (subcat "2A" )))
( "proselytized"  v ( p r o1 s @ l i t ai z d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "proselytizes"  v ( p r o1 s @ l i t ai z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "proselytizing"  v ( p r o1 s @ l i t ai z i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "prosier"  j ( p r ou1 z i@ r ) ((pos "Or%" ) ))
( "prosiest"  j ( p r ou1 z i i s t ) ((pos "Os%" ) ))
( "prosily"  a ( p r ou1 z i l ii ) ((pos "Pu%" ) ))
( "prosiness"  n ( p r ou1 z i n @ s ) ((pos "L@%" ) ))
( "prosody"  n ( p r o1 s @ d ii ) ((pos "L@%" ) ))
( "prospect"  n ( p r o1 s p e k t ) ((pos "M6%" ) ))
( "prospect"  v ( p r @1 s p e1 k t ) ((pos "I0%" ) (subcat "2A" )))
( "prospected"  v ( p r @1 s p e1 k t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "prospecting"  v ( p r @1 s p e1 k t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "prospective"  j ( p r @1 s p e1 k t i v ) ((pos "OA%" ) ))
( "prospector"  n ( p r @1 s p e1 k t @ r ) ((pos "K6%" ) ))
( "prospectors"  n ( p r @1 s p e1 k t @ z ) ((pos "Kj%" ) ))
( "prospects"  n ( p r o1 s p e k t s ) ((pos "Mj%" ) ))
( "prospects"  v ( p r @1 s p e1 k t s ) ((pos "Ia%" ) (subcat "2A" )))
( "prospectus"  n ( p r @1 s p e1 k t @ s ) ((pos "K7%" ) ))
( "prospectuses"  n ( p r @1 s p e1 k t @ s i z ) ((pos "Kj%" ) ))
( "prosper"  v ( p r o1 s p @ r ) ((pos "J0%" ) (subcat "2A" )))
( "prospered"  v ( p r o1 s p @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "prospering"  v ( p r o1 s p @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "prosperity"  n ( p r o1 s p e1 r i t ii ) ((pos "L@%" ) ))
( "prosperous"  j ( p r o1 s p @ r @ s ) ((pos "OA%" ) ))
( "prosperously"  a ( p r o1 s p @ r @ s l ii ) ((pos "Pu%" ) ))
( "prospers"  v ( p r o1 s p @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "prostate"  n ( p r o1 s t ei t ) ((pos "K6$" ) ))
( "prostates"  n ( p r o1 s t ei t s ) ((pos "Kj$" ) ))
( "prostitute"  v ( p r o1 s t i t y uu t ) ((pos "H2%" "K6%" ) (subcat )))
( "prostituted"  v ( p r o1 s t i t y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prostitutes"  v ( p r o1 s t i t y uu t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "prostituting"  v ( p r o1 s t i t y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "prostitution"  n ( p r o2 s t i t y uu1 sh @ n ) ((pos "L@%" ) ))
( "prostrate"  j ( p r o1 s t r ei t ) ((pos "OA%" ) ))
( "prostrate"  v ( p r o1 s t r ei1 t ) ((pos "H2%" ) (subcat )))
( "prostrated"  v ( p r o1 s t r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "prostrates"  v ( p r o1 s t r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "prostrating"  v ( p r o1 s t r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "prostration"  n ( p r o1 s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "prostrations"  n ( p r o1 s t r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "prosy"  j ( p r ou1 z ii ) ((pos "OD%" ) ))
( "prot_eg_e"  n ( p r o1 t i zh ei ) ((pos "K6%" ) ))
( "prot_eg_ee"  n ( p r o1 t i zh ei ) ((pos "K6$" ) ))
( "prot_eg_ees"  n ( p r o1 t i zh ei z ) ((pos "Kj$" ) ))
( "prot_eg_es"  n ( p r o1 t i zh ei z ) ((pos "Kj%" ) ))
( "protagonist"  n ( p r @1 t a1 g @ n i s t ) ((pos "K6%" ) ))
( "protagonists"  n ( p r @1 t a1 g @ n i s t s ) ((pos "Kj%" ) ))
( "protean"  j ( p r ou1 t i@ n ) ((pos "OA$" ) ))
( "protect"  v ( p r @1 t e1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "protected"  v ( p r @1 t e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "protecting"  v ( p r @1 t e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "protection"  n ( p r @1 t e1 k sh @ n ) ((pos "M6%" ) ))
( "protectionism"  n ( p r @1 t e1 k sh @ n i z @ m ) ((pos "L@%" ) ))
( "protectionist"  n ( p r @1 t e1 k sh @ n i s t ) ((pos "K6%" ) ))
( "protectionists"  n ( p r @1 t e1 k sh @ n i s t s ) ((pos "Kj%" ) ))
( "protections"  n ( p r @1 t e1 k sh @ n z ) ((pos "Mj%" ) ))
( "protective"  j ( p r @1 t e1 k t i v ) ((pos "OA%" ) ))
( "protectively"  a ( p r @1 t e1 k t i v l ii ) ((pos "Pu%" ) ))
( "protector"  n ( p r @1 t e1 k t @ r ) ((pos "K6%" ) ))
( "protectorate"  n ( p r @1 t e1 k t @ r @ t ) ((pos "K6%" ) ))
( "protectorates"  n ( p r @1 t e1 k t @ r @ t s ) ((pos "Kj%" ) ))
( "protectors"  n ( p r @1 t e1 k t @ z ) ((pos "Kj%" ) ))
( "protects"  v ( p r @1 t e1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "protein"  n ( p r ou1 t ii n ) ((pos "M6%" ) ))
( "proteins"  n ( p r ou1 t ii n z ) ((pos "Mj%" ) ))
( "protest"  n ( p r ou1 t e s t ) ((pos "M6%" ) ))
( "protest"  v ( p r @1 t e1 s t ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "protestation"  n ( p r o2 t e s t ei1 sh @ n ) ((pos "K6%" ) ))
( "protestations"  n ( p r o2 t e s t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "protested"  v ( p r @1 t e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "protester"  n ( p r @1 t e1 s t @ r ) ((pos "K6%" ) ))
( "protesters"  n ( p r @1 t e1 s t @ z ) ((pos "Kj%" ) ))
( "protesting"  v ( p r @1 t e1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "protestingly"  a ( p r @1 t e1 s t i ng l ii ) ((pos "Pu%" ) ))
( "protests"  n ( p r ou1 t e s t s ) ((pos "Mj%" ) ))
( "protests"  v ( p r @1 t e1 s t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "protocol"  n ( p r ou1 t @ k o l ) ((pos "M6%" ) ))
( "protocols"  n ( p r ou1 t @ k o l z ) ((pos "Mj%" ) ))
( "proton"  n ( p r ou1 t o n ) ((pos "K6%" ) ))
( "protons"  n ( p r ou1 t o n z ) ((pos "Kj%" ) ))
( "protoplasm"  n ( p r ou1 t @ p l a z @ m ) ((pos "L@%" ) ))
( "prototype"  n ( p r ou1 t @ t ai p ) ((pos "K6%" ) ))
( "prototypes"  n ( p r ou1 t @ t ai p s ) ((pos "Kj%" ) ))
( "protozoa"  n ( p r ou2 t @ z ou1 @ ) ((pos "Kj%" ) ))
( "protract"  v ( p r @1 t r a1 k t ) ((pos "H0$" ) (subcat )))
( "protracted"  v ( p r @1 t r a1 k t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "protracting"  v ( p r @1 t r a1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "protraction"  n ( p r @1 t r a1 k sh @ n ) ((pos "M6$" ) ))
( "protractions"  n ( p r @1 t r a1 k sh @ n z ) ((pos "Mj$" ) ))
( "protractor"  n ( p r @1 t r a1 k t @ r ) ((pos "K6%" ) ))
( "protractors"  n ( p r @1 t r a1 k t @ z ) ((pos "Kj%" ) ))
( "protracts"  v ( p r @1 t r a1 k t s ) ((pos "Ha$" ) (subcat )))
( "protrude"  v ( p r @1 t r uu1 d ) ((pos "J2%" ) (subcat "2A" )))
( "protruded"  v ( p r @1 t r uu1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "protrudes"  v ( p r @1 t r uu1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "protruding"  v ( p r @1 t r uu1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "protrusion"  n ( p r @1 t r uu1 zh n ) ((pos "M6%" ) ))
( "protrusions"  n ( p r @1 t r uu1 zh n z ) ((pos "Mj%" ) ))
( "protrusive"  j ( p r @1 t r uu1 s i v ) ((pos "OA%" ) ))
( "protuberance"  n ( p r @1 t y uu1 b @ r @ n s ) ((pos "M6%" ) ))
( "protuberances"  n ( p r @1 t y uu1 b @ r @ n s i z ) ((pos "Mj%" ) ))
( "protuberant"  j ( p r @1 t y uu1 b @ r @ n t ) ((pos "OA%" ) ))
( "proud"  j ( p r au1 d ) ((pos "OC%" ) ))
( "prouder"  j ( p r au1 d @ r ) ((pos "Or%" ) ))
( "proudest"  j ( p r au1 d i s t ) ((pos "Os%" ) ))
( "proudly"  a ( p r au1 d l ii ) ((pos "Pu%" ) ))
( "provable"  j ( p r uu1 v @ b l ) ((pos "OA%" ) ))
( "prove"  v ( p r uu1 v ) ((pos "J2%" ) (subcat "4D" "6A" "9" "14" )))
( "proved"  v ( p r uu1 v d ) ((pos "Jc%" "Jd%" ) (subcat "4D" "6A" "9" "14" )))
( "proven"  v ( p r uu1 v n ) ((pos "Jd%" ) (subcat "4D" "6A" "9" "14" )))
( "provenance"  n ( p r o1 v @ n @ n s ) ((pos "L@%" ) ))
( "provender"  n ( p r o1 v i n d @ r ) ((pos "L@%" ) ))
( "proverb"  n ( p r o1 v @@ b ) ((pos "K6%" ) ))
( "proverbial"  j ( p r @1 v @@1 b i@ l ) ((pos "OA%" ) ))
( "proverbially"  a ( p r @1 v @@1 b i@ l ii ) ((pos "Pu%" ) ))
( "proverbs"  n ( p r o1 v @@ b z ) ((pos "Kj%" ) ))
( "proves"  v ( p r uu1 v z ) ((pos "Ja%" ) (subcat "4D" "6A" "9" "14" )))
( "provide"  v ( p r @1 v ai1 d ) ((pos "J2%" ) (subcat "3A" "6A" "9" )))
( "provided"  v ( p r @1 v ai1 d i d ) ((pos "Jc%" "Jd%" "V-%" ) (subcat "3A" "6A" "9" )))
( "providence"  n ( p r o1 v i d @ n s ) ((pos "L@%" ) ))
( "provident"  j ( p r o1 v i d @ n t ) ((pos "OA%" ) ))
( "providential"  j ( p r o2 v i d e1 n sh l ) ((pos "OA%" ) ))
( "providentially"  a ( p r o2 v i d e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "providently"  a ( p r o1 v i d @ n t l ii ) ((pos "Pu%" ) ))
( "provider"  n ( p r @1 v ai1 d @ r ) ((pos "K6%" ) ))
( "providers"  n ( p r @1 v ai1 d @ z ) ((pos "Kj%" ) ))
( "provides"  v ( p r @1 v ai1 d z ) ((pos "Ja%" ) (subcat "3A" "6A" "9" )))
( "providing"  v ( p r @1 v ai1 d i ng ) ((pos "Jb%" "V-%" ) (subcat "3A" "6A" "9" )))
( "province"  n ( p r o1 v i n s ) ((pos "K6%" ) ))
( "provinces"  n ( p r o1 v i n s i z ) ((pos "Kj%" ) ))
( "provincial"  n ( p r @1 v i1 n sh l ) ((pos "K6%" "OA%" ) ))
( "provincialism"  n ( p r @1 v i1 n sh @ l i z @ m ) ((pos "M6%" ) ))
( "provincialisms"  n ( p r @1 v i1 n sh @ l i z @ m z ) ((pos "Mj$" ) ))
( "provincially"  a ( p r @1 v i1 n sh @ l ii ) ((pos "Pu%" ) ))
( "provincials"  n ( p r @1 v i1 n sh l z ) ((pos "Kj%" ) ))
( "proving"  v ( p r uu1 v i ng ) ((pos "Jb%" ) (subcat "4D" "6A" "9" "14" )))
( "provision"  v ( p r @1 v i1 zh n ) ((pos "H0%" "M6%" ) (subcat )))
( "provisional"  j ( p r @1 v i1 zh @ n l ) ((pos "OA%" ) ))
( "provisionally"  a ( p r @1 v i1 zh @ n @ l ii ) ((pos "Pu%" ) ))
( "provisioned"  v ( p r @1 v i1 zh n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "provisioning"  v ( p r @1 v i1 zh n i ng ) ((pos "Hb$" ) (subcat )))
( "provisions"  v ( p r @1 v i1 zh n z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "proviso"  n ( p r @1 v ai1 z ou ) ((pos "K6%" ) ))
( "provisory"  j ( p r @1 v ai1 z @ r ii ) ((pos "OA%" ) ))
( "provisos"  n ( p r @1 v ai1 z ou z ) ((pos "Kj%" ) ))
( "provocation"  n ( p r o2 v @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "provocations"  n ( p r o2 v @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "provocative"  j ( p r @1 v o1 k @ t i v ) ((pos "OA%" ) ))
( "provocatively"  a ( p r @1 v o1 k @ t i v l ii ) ((pos "Pu%" ) ))
( "provoke"  v ( p r @1 v ou1 k ) ((pos "H2%" ) (subcat "6A" "14" )))
( "provoked"  v ( p r @1 v ou1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "provokes"  v ( p r @1 v ou1 k s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "provoking"  v ( p r @1 v ou1 k i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "provokingly"  a ( p r @1 v ou1 k i ng l ii ) ((pos "Pu%" ) ))
( "provost"  n ( p r o1 v @ s t ) ((pos "K6%" ) ))
( "provosts"  n ( p r o1 v @ s t s ) ((pos "Kj%" ) ))
( "prow"  n ( p r au1 ) ((pos "K6%" ) ))
( "prowess"  n ( p r au1 i s ) ((pos "L@%" ) ))
( "prowl"  v ( p r au1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "prowled"  v ( p r au1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "prowler"  n ( p r au1 l @ r ) ((pos "K6%" ) ))
( "prowlers"  n ( p r au1 l @ z ) ((pos "Kj%" ) ))
( "prowling"  v ( p r au1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "prowls"  v ( p r au1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "prows"  n ( p r au1 z ) ((pos "Kj%" ) ))
( "prox"  a ( p r o1 k s ) ((pos "Pu$" ) ))
( "proxies"  n ( p r o1 k s i z ) ((pos "Mj%" ) ))
( "proximate"  j ( p r o1 k s i m @ t ) ((pos "OA$" ) ))
( "proximity"  n ( p r o1 k s i1 m i t ii ) ((pos "L@%" ) ))
( "proximo"  j ( p r o1 k s i m ou ) ((pos "OA$" ) ))
( "proxy"  n ( p r o1 k s ii ) ((pos "M8%" ) ))
( "prude"  n ( p r uu1 d ) ((pos "K6%" ) ))
( "prudence"  n ( p r uu1 d n s ) ((pos "L@%" ) ))
( "prudent"  j ( p r uu1 d n t ) ((pos "OA%" ) ))
( "prudential"  j ( p r uu1 d e1 n sh l ) ((pos "OA%" ) ))
( "prudently"  a ( p r uu1 d n t l ii ) ((pos "Pu%" ) ))
( "pruderies"  n ( p r uu1 d @ r i z ) ((pos "Mj$" ) ))
( "prudery"  n ( p r uu1 d @ r ii ) ((pos "M8%" ) ))
( "prudes"  n ( p r uu1 d z ) ((pos "Kj%" ) ))
( "prudish"  j ( p r uu1 d i sh ) ((pos "OA%" ) ))
( "prudishly"  a ( p r uu1 d i sh l ii ) ((pos "Pu%" ) ))
( "prune"  v ( p r uu1 n ) ((pos "H2%" "K6%" ) (subcat "6A" "14" )))
( "pruned"  v ( p r uu1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "pruners"  n ( p r uu1 n @ z ) ((pos "Kj$" ) ))
( "prunes"  v ( p r uu1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "pruning"  v ( p r uu1 n i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" "14" )))
( "pruning-hook"  n ( p r uu1 n ii ng - h u k ) ((pos "K6%" ) ))
( "pruning-hooks"  n ( p r uu1 n ii ng - h u k s ) ((pos "Kj%" ) ))
( "pruning-knife"  n ( p r uu1 n ii ng - n ai f ) ((pos "Ki%" ) ))
( "pruning-knives"  n ( p r uu1 n ii ng - n ai v z ) ((pos "Kj%" ) ))
( "pruning-saw"  n ( p r uu1 n ii ng - s oo ) ((pos "K6%" ) ))
( "pruning-saws"  n ( p r uu1 n ii ng - s oo z ) ((pos "Kj%" ) ))
( "pruning-scissors"  n ( p r uu1 n ii ng - s i z @ z ) ((pos "Kj%" ) ))
( "pruning-shears"  n ( p r uu1 n ii ng - sh i@ z ) ((pos "Kj%" ) ))
( "prurience"  n ( p r u@1 r i@ n s ) ((pos "L@%" ) ))
( "pruriency"  n ( p r u@1 r i@ n s ii ) ((pos "L@$" ) ))
( "prurient"  j ( p r u@1 r i@ n t ) ((pos "OA%" ) ))
( "pruriently"  a ( p r u@1 r i@ n t l ii ) ((pos "Pu%" ) ))
( "prussic"  j ( p r uh1 s i k ) ((pos "OA%" ) ))
( "pry"  v ( p r ai1 ) ((pos "J3%" ) (subcat "2A" "2C" "3A" "15A" "15B" )))
( "prying"  v ( p r ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "15A" "15B" )))
( "pryingly"  a ( p r ai1 i ng l ii ) ((pos "Pu%" ) ))
( "psalm"  n ( s aa1 m ) ((pos "K6%" ) ))
( "psalmist"  n ( s aa1 m i s t ) ((pos "K6%" ) ))
( "psalmists"  n ( s aa1 m i s t s ) ((pos "Kj$" ) ))
( "psalmodies"  n ( s aa1 m @ d i z ) ((pos "Mj$" ) ))
( "psalmody"  n ( s aa1 m @ d ii ) ((pos "M8$" ) ))
( "psalms"  n ( s aa1 m z ) ((pos "Kj%" ) ))
( "psalter"  n ( s oo1 l t @ r ) ((pos "K6$" ) ))
( "psalteries"  n ( s oo1 l t @ r i z ) ((pos "Kj$" ) ))
( "psalters"  n ( s oo1 l t @ z ) ((pos "Kj$" ) ))
( "psaltery"  n ( s oo1 l t @ r ii ) ((pos "K8$" ) ))
( "psephologist"  n ( s e1 f o1 l @ jh i s t ) ((pos "K6$" ) ))
( "psephologists"  n ( s e1 f o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "psephology"  n ( s e1 f o1 l @ jh ii ) ((pos "L@$" ) ))
( "pseud"  n ( s y uu1 d ) ((pos "K6%" ) ))
( "pseudo"  n ( s y uu1 d ou ) ((pos "K6$" "OA%" ) ))
( "pseudo-"  j ( s y uu2 d ou - ) ((pos "U-%" ) ))
( "pseudonym"  n ( s y uu1 d @ n i m ) ((pos "K6%" ) ))
( "pseudonymous"  j ( s y uu1 d o1 n i m @ s ) ((pos "OA%" ) ))
( "pseudonyms"  n ( s y uu1 d @ n i m z ) ((pos "Kj%" ) ))
( "pseudos"  n ( s y uu1 d ou z ) ((pos "Kj$" ) ))
( "pseuds"  n ( s y uu1 d z ) ((pos "Kj%" ) ))
( "pshaw"  uh ( p f @1 ) ((pos "W-$" ) ))
( "psittacosis"  n ( s i2 t @ k ou1 s i s ) ((pos "L@$" ) ))
( "psyche"  n ( s ai1 k ii ) ((pos "K6%" ) ))
( "psychedelic"  j ( s ai2 k i d e1 l i k ) ((pos "OA%" ) ))
( "psyches"  n ( s ai1 k i z ) ((pos "Kj%" ) ))
( "psychiatric"  j ( s ai2 k i a1 t r i k ) ((pos "OA%" ) ))
( "psychiatrist"  n ( s i1 k ai1 @ t r i s t ) ((pos "K6%" ) ))
( "psychiatrists"  n ( s i1 k ai1 @ t r i s t s ) ((pos "Kj%" ) ))
( "psychiatry"  n ( s ai1 k ai1 @ t r ii ) ((pos "L@%" ) ))
( "psychic"  n ( s ai1 k i k ) ((pos "K6%" "OA%" ) ))
( "psychical"  j ( s ai1 k i k l ) ((pos "OA%" ) ))
( "psychics"  n ( s ai1 k i k s ) ((pos "Kj%" ) ))
( "psychoanalyse"  v ( s ai2 k ou a1 n @ l ai z ) ((pos "H2%" ) ))
( "psychoanalysed"  v ( s ai2 k ou a1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "psychoanalyses"  v ( s ai2 k ou a1 n @ l ai z i z ) ((pos "Ha%" ) ))
( "psychoanalysing"  v ( s ai2 k ou a1 n @ l ai z i ng ) ((pos "Hb%" ) ))
( "psychoanalysis"  n ( s ai2 k ou @ n a1 l @ s i s ) ((pos "L@%" ) ))
( "psychoanalyst"  n ( s ai2 k ou a1 n @ l i s t ) ((pos "K6%" ) ))
( "psychoanalysts"  n ( s ai2 k ou a1 n @ l i s t s ) ((pos "Kj%" ) ))
( "psychoanalytic"  j ( s ai2 k ou a2 n @ l i1 t i k ) ((pos "OA%" ) ))
( "psychoanalytical"  j ( s ai2 k ou a2 n @ l i1 t i k l ) ((pos "OA%" ) ))
( "psychoanalyze"  v ( s ai2 k ou a1 n @ l ai z ) ((pos "H2%" ) ))
( "psychoanalyzed"  v ( s ai2 k ou a1 n @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "psychoanalyzes"  v ( s ai2 k ou a1 n @ l ai z i z ) ((pos "Ha%" ) ))
( "psychoanalyzing"  v ( s ai2 k ou a1 n @ l ai z i ng ) ((pos "Hb%" ) ))
( "psychological"  j ( s ai2 k @ l o1 jh i k l ) ((pos "OA%" ) ))
( "psychologically"  a ( s ai2 k @ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "psychologies"  n ( s ai1 k o1 l @ jh i z ) ((pos "Mj$" ) ))
( "psychologist"  n ( s ai1 k o1 l @ jh i s t ) ((pos "K6%" ) ))
( "psychologists"  n ( s ai1 k o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "psychology"  n ( s ai1 k o1 l @ jh ii ) ((pos "M8%" ) ))
( "psychopath"  n ( s ai1 k ou p a th ) ((pos "K6%" ) ))
( "psychopathic"  j ( s ai2 k ou p a1 th i k ) ((pos "OA%" ) ))
( "psychopaths"  n ( s ai1 k ou p a th s ) ((pos "Kj%" ) ))
( "psychoses"  n ( s ai1 k ou1 s ii z ) ((pos "Kj%" ) ))
( "psychosis"  n ( s ai1 k ou1 s i s ) ((pos "Ki%" ) ))
( "psychosomatic"  j ( s ai2 k ou s @ m a1 t i k ) ((pos "OA%" ) ))
( "psychotherapy"  n ( s ai2 k ou th e1 r @ p ii ) ((pos "L@%" ) ))
( "psychotic"  n ( s ai1 k o1 t i k ) ((pos "K6%" ) ))
( "psychotics"  n ( s ai1 k o1 t i k s ) ((pos "Kj%" ) ))
( "pt"  n ( p ai1 n t ) ((pos "Y]%" ) ))
( "ptarmigan"  n ( t aa1 m i g @ n ) ((pos "K6$" ) ))
( "ptarmigans"  n ( t aa1 m i g @ n z ) ((pos "Kj$" ) ))
( "pterodactyl"  n ( t e2 r @ d a1 k t i l ) ((pos "K6$" ) ))
( "pterodactyls"  n ( t e2 r @ d a1 k t i l z ) ((pos "Kj$" ) ))
( "ptomaine"  n ( t ou1 m ei n ) ((pos "K6$" ) ))
( "ptomaines"  n ( t ou1 m ei n z ) ((pos "Kj$" ) ))
( "pub"  n ( p uh1 b ) ((pos "K6%" ) ))
( "pub-crawl"  v ( p uh1 b - k r oo l ) ((pos "I0$" "K6%" ) ))
( "pub-crawled"  v ( p uh1 b - k r oo l d ) ((pos "Ic$" "Id$" ) ))
( "pub-crawling"  v ( p uh1 b - k r oo l i ng ) ((pos "Ib$" ) ))
( "pub-crawls"  v ( p uh1 b - k r oo l z ) ((pos "Ia$" "Kj$" ) ))
( "puberty"  n ( p y uu1 b @ t ii ) ((pos "L@%" ) ))
( "pubic"  j ( p y uu1 b i k ) ((pos "OA%" ) ))
( "public"  n ( p uh1 b l i k ) ((pos "K6%" "OA*" ) ))
( "public-spirited"  j ( p uh2 b l i k - s p i1 r i t i d ) ((pos "OA%" ) ))
( "publican"  n ( p uh1 b l i k @ n ) ((pos "K6%" ) ))
( "publicans"  n ( p uh1 b l i k @ n z ) ((pos "Kj%" ) ))
( "publication"  n ( p uh2 b l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "publications"  n ( p uh2 b l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "publicist"  n ( p uh1 b l i s i s t ) ((pos "K6%" ) ))
( "publicists"  n ( p uh1 b l i s i s t s ) ((pos "Kj%" ) ))
( "publicity"  n ( p uh1 b l i1 s i t ii ) ((pos "L@%" ) ))
( "publicize"  v ( p uh1 b l i s ai z ) ((pos "H2%" ) (subcat )))
( "publicized"  v ( p uh1 b l i s ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "publicizes"  v ( p uh1 b l i s ai z i z ) ((pos "Ha%" ) (subcat )))
( "publicizing"  v ( p uh1 b l i s ai z i ng ) ((pos "Hb%" ) (subcat )))
( "publicly"  a ( p uh1 b l i k l ii ) ((pos "Pu%" ) ))
( "publics"  n ( p uh1 b l i k s ) ((pos "Kj$" ) ))
( "publish"  v ( p uh1 b l i sh ) ((pos "H1%" ) (subcat )))
( "published"  v ( p uh1 b l i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "publisher"  n ( p uh1 b l i sh @ r ) ((pos "K6%" ) ))
( "publishers"  n ( p uh1 b l i sh @ z ) ((pos "Kj%" ) ))
( "publishes"  v ( p uh1 b l i sh i z ) ((pos "Ha%" ) (subcat )))
( "publishing"  v ( p uh1 b l i sh i ng ) ((pos "Hb%" ) (subcat )))
( "pubs"  n ( p uh1 b z ) ((pos "Kj%" ) ))
( "puce"  n ( p y uu1 s ) ((pos "L@$" ) ))
( "puck"  n ( p uh1 k ) ((pos "K6$" ) ))
( "pucker"  v ( p uh1 k @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "puckered"  v ( p uh1 k @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "puckering"  v ( p uh1 k @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "puckers"  v ( p uh1 k @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "puckish"  j ( p uh1 k i sh ) ((pos "OA%" ) ))
( "puckishly"  a ( p uh1 k i sh l ii ) ((pos "Pu%" ) ))
( "pucks"  n ( p uh1 k s ) ((pos "Kj$" ) ))
( "pud"  n ( p u1 d ) ((pos "L@%" ) ))
( "pudden"  n ( p u1 d n ) ((pos "K6$" ) ))
( "pudden-head"  n ( p u1 d n - h e d ) ((pos "K6$" ) ))
( "pudden-heads"  n ( p u1 d n - h e d z ) ((pos "Kj$" ) ))
( "puddens"  n ( p u1 d n z ) ((pos "Kj$" ) ))
( "pudding"  n ( p u1 d i ng ) ((pos "M6%" ) ))
( "pudding-face"  n ( p u1 d i ng - f ei s ) ((pos "K6$" ) ))
( "pudding-faces"  n ( p u1 d i ng - f ei s i z ) ((pos "Kj$" ) ))
( "puddings"  n ( p u1 d i ng z ) ((pos "Mj%" ) ))
( "puddle"  v ( p uh1 d l ) ((pos "H2$" "M6%" ) ))
( "puddled"  v ( p uh1 d l d ) ((pos "Hc$" "Hd$" ) ))
( "puddler"  n ( p uh1 d l @ r ) ((pos "K6$" ) ))
( "puddlers"  n ( p uh1 d l @ z ) ((pos "Kj$" ) ))
( "puddles"  v ( p uh1 d l z ) ((pos "Ha$" "Mj%" ) ))
( "puddling"  v ( p uh1 d l i ng ) ((pos "Hb$" ) ))
( "pudenda"  n ( p y uu1 d e1 n d @ ) ((pos "Kj$" ) ))
( "pudgier"  j ( p uh1 jh i@ r ) ((pos "Or%" ) ))
( "pudgiest"  j ( p uh1 jh i i s t ) ((pos "Os%" ) ))
( "pudgy"  j ( p uh1 jh ii ) ((pos "OD%" ) ))
( "pueblo"  n ( p w e1 b l ou ) ((pos "K6$" ) ))
( "pueblos"  n ( p w e1 b l ou z ) ((pos "Kj$" ) ))
( "puerile"  j ( p y u@1 r ai l ) ((pos "OA%" ) ))
( "puerilities"  n ( p y u@2 r i1 l i t i z ) ((pos "Mj$" ) ))
( "puerility"  n ( p y u@2 r i1 l i t ii ) ((pos "M8$" ) ))
( "puerperal"  j ( p y uu1 @@1 p @ r @ l ) ((pos "OA$" ) ))
( "puff"  v ( p uh1 f ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "puffball"  n ( p uh1 f b oo l ) ((pos "K6$" ) ))
( "puffballs"  n ( p uh1 f b oo l z ) ((pos "Kj$" ) ))
( "puffed"  v ( p uh1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "puffier"  j ( p uh1 f i@ r ) ((pos "Or%" ) ))
( "puffiest"  j ( p uh1 f i i s t ) ((pos "Os%" ) ))
( "puffin"  n ( p uh1 f i n ) ((pos "K6%" ) ))
( "puffiness"  n ( p uh1 f i n @ s ) ((pos "L@%" ) ))
( "puffing"  v ( p uh1 f i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "puffins"  n ( p uh1 f i n z ) ((pos "Kj%" ) ))
( "puffs"  v ( p uh1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "puffy"  j ( p uh1 f ii ) ((pos "OD%" ) ))
( "pug"  n ( p uh1 g ) ((pos "K6%" ) ))
( "pug-dog"  n ( p uh1 g - d o g ) ((pos "K6%" ) ))
( "pug-dogs"  n ( p uh1 g - d o g z ) ((pos "Kj%" ) ))
( "pug-nose"  n ( p uh1 g - n ou z ) ((pos "K6%" "OA%" ) ))
( "pug-nosed"  j ( p uh1 g - n ou z d ) ((pos "OA%" ) ))
( "pug-noses"  n ( p uh1 g - n ou z i z ) ((pos "Kj%" ) ))
( "pugilism"  n ( p y uu1 jh i l i z @ m ) ((pos "L@$" ) ))
( "pugilist"  n ( p y uu1 jh i l i s t ) ((pos "K6$" ) ))
( "pugilistic"  j ( p y uu2 jh i l i1 s t i k ) ((pos "OA$" ) ))
( "pugilists"  n ( p y uu1 jh i l i s t s ) ((pos "Kj$" ) ))
( "pugnacious"  j ( p uh1 g n ei1 sh @ s ) ((pos "OA%" ) ))
( "pugnaciously"  a ( p uh1 g n ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "pugnacity"  n ( p uh1 g n a1 s i t ii ) ((pos "L@%" ) ))
( "pugs"  n ( p uh1 g z ) ((pos "Kj%" ) ))
( "puissance"  n ( p w ii1 s aa n s ) ((pos "L@$" ) ))
( "puissant"  j ( p w ii1 s @ n t ) ((pos "OA$" ) ))
( "puke"  v ( p y uu1 k ) ((pos "J2%" "L@%" ) ))
( "puked"  v ( p y uu1 k t ) ((pos "Jc%" "Jd%" ) ))
( "pukes"  v ( p y uu1 k s ) ((pos "Ja%" ) ))
( "puking"  v ( p y uu1 k i ng ) ((pos "Jb%" ) ))
( "pukka"  j ( p uh1 k @ ) ((pos "OA%" ) ))
( "pulchritude"  n ( p uh1 l k r i t y uu d ) ((pos "L@$" ) ))
( "pulchritudinous"  j ( p uh2 l k r i t y uu1 d i n @ s ) ((pos "OA$" ) ))
( "pule"  v ( p y uu1 l ) ((pos "I2$" ) (subcat )))
( "puled"  v ( p y uu1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "pules"  v ( p y uu1 l z ) ((pos "Ia$" ) (subcat )))
( "puling"  v ( p y uu1 l i ng ) ((pos "Ib$" ) (subcat )))
( "pull"  v ( p u1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "pull-in"  n ( p u1 l - i n ) ((pos "K6%" ) ))
( "pull-ins"  n ( p u1 l - i n z ) ((pos "Kj$" ) ))
( "pull-off"  n ( p u1 l - o f ) ((pos "K6$" ) ))
( "pull-offs"  n ( p u1 l - o f s ) ((pos "Kj$" ) ))
( "pull-out"  n ( p u1 l - au t ) ((pos "K6%" ) ))
( "pull-outs"  n ( p u1 l - au t s ) ((pos "Kj$" ) ))
( "pull-through"  n ( p u1 l - th r uu ) ((pos "K6%" ) ))
( "pull-throughs"  n ( p u1 l - th r uu z ) ((pos "Kj$" ) ))
( "pull-up"  n ( p u1 l - uh p ) ((pos "K6%" ) ))
( "pull-ups"  n ( p u1 l - uh p s ) ((pos "Kj$" ) ))
( "pulled"  v ( p u1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "pullet"  n ( p u1 l i t ) ((pos "K6%" ) ))
( "pullets"  n ( p u1 l i t s ) ((pos "Kj%" ) ))
( "pulley"  n ( p u1 l ii ) ((pos "K6%" ) ))
( "pulley-block"  n ( p u1 l i - b l o k ) ((pos "K6%" ) ))
( "pulley-blocks"  n ( p u1 l i - b l o k s ) ((pos "Kj%" ) ))
( "pulleys"  n ( p u1 l i z ) ((pos "Kj%" ) ))
( "pulling"  v ( p u1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "pullover"  n ( p u1 l ou v @ r ) ((pos "K6%" ) ))
( "pullovers"  n ( p u1 l ou v @ z ) ((pos "Kj%" ) ))
( "pulls"  v ( p u1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "pullulate"  v ( p uh1 l y u l ei t ) ((pos "I2$" ) ))
( "pullulated"  v ( p uh1 l y u l ei t i d ) ((pos "Ic$" "Id$" ) ))
( "pullulates"  v ( p uh1 l y u l ei t s ) ((pos "Ia$" ) ))
( "pullulating"  v ( p uh1 l y u l ei t i ng ) ((pos "Ib$" ) ))
( "pulmonary"  j ( p uh1 l m @ n @ r ii ) ((pos "OA%" ) ))
( "pulp"  v ( p uh1 l p ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "pulped"  v ( p uh1 l p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pulpier"  j ( p uh1 l p i@ r ) ((pos "Or$" ) ))
( "pulpiest"  j ( p uh1 l p i i s t ) ((pos "Os$" ) ))
( "pulping"  v ( p uh1 l p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pulpit"  n ( p u1 l p i t ) ((pos "K6%" ) ))
( "pulpits"  n ( p u1 l p i t s ) ((pos "Kj%" ) ))
( "pulps"  v ( p uh1 l p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "pulpy"  j ( p uh1 l p ii ) ((pos "OD%" ) ))
( "pulque"  n ( p uu1 l k ei ) ((pos "L@$" ) ))
( "pulsar"  n ( p uh1 l s aa r ) ((pos "K6$" ) ))
( "pulsars"  n ( p uh1 l s aa z ) ((pos "Kj$" ) ))
( "pulsate"  v ( p uh1 l s ei1 t ) ((pos "J2%" ) (subcat "2A" )))
( "pulsated"  v ( p uh1 l s ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pulsates"  v ( p uh1 l s ei1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "pulsating"  v ( p uh1 l s ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pulsation"  n ( p uh1 l s ei1 sh @ n ) ((pos "M6%" ) ))
( "pulsations"  n ( p uh1 l s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "pulse"  v ( p uh1 l s ) ((pos "I2%" "M6%" ) (subcat )))
( "pulsed"  v ( p uh1 l s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "pulses"  v ( p uh1 l s i z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "pulsing"  v ( p uh1 l s i ng ) ((pos "Ib%" ) (subcat )))
( "pulverize"  v ( p uh1 l v @ r ai z ) ((pos "J2%" ) (subcat "2A" )))
( "pulverized"  v ( p uh1 l v @ r ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "pulverizes"  v ( p uh1 l v @ r ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "pulverizing"  v ( p uh1 l v @ r ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "puma"  n ( p y uu1 m @ ) ((pos "K6$" ) ))
( "pumas"  n ( p y uu1 m @ z ) ((pos "Kj$" ) ))
( "pumice"  n ( p uh1 m i s ) ((pos "L@$" ) ))
( "pumice-stone"  n ( p uh1 m i s - s t ou n ) ((pos "K6$" ) ))
( "pumice-stones"  n ( p uh1 m i s - s t ou n z ) ((pos "Kj$" ) ))
( "pummel"  v ( p uh1 m @ l ) ((pos "H4%" ) (subcat "6A" )))
( "pummelled"  v ( p uh1 m @ l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "pummelling"  v ( p uh1 m @ l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pummels"  v ( p uh1 m @ l z ) ((pos "Ha%" ) (subcat "6A" )))
( "pump"  v ( p uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pump-room"  n ( p uh1 m p - r uu m ) ((pos "K6%" ) ))
( "pump-rooms"  n ( p uh1 m p - r uu m z ) ((pos "Kj%" ) ))
( "pumped"  v ( p uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pumpernickel"  n ( p uh1 m p @ n i k l ) ((pos "L@$" ) ))
( "pumping"  v ( p uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pumpkin"  n ( p uh1 m p k i n ) ((pos "K6%" ) ))
( "pumpkins"  n ( p uh1 m p k i n z ) ((pos "Kj%" ) ))
( "pumps"  v ( p uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "pun"  v ( p uh1 n ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "punch"  v ( p uh1 n ch ) ((pos "H1%" "M7%" ) (subcat "6A" "15A" )))
( "punch-drunk"  j ( p uh1 n ch - d r uh ng k ) ((pos "OA%" ) ))
( "punch-up"  n ( p uh1 n ch - uh p ) ((pos "K6%" ) ))
( "punch-ups"  n ( p uh1 n ch - uh p s ) ((pos "Kj%" ) ))
( "punchball"  n ( p uh1 n ch b oo l ) ((pos "K6%" ) ))
( "punchballs"  n ( p uh1 n ch b oo l z ) ((pos "Kj%" ) ))
( "punchbowl"  n ( p uh1 n ch b ou l ) ((pos "K6%" ) ))
( "punchbowls"  n ( p uh1 n ch b ou l z ) ((pos "Kj%" ) ))
( "punched"  v ( p uh1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "punches"  v ( p uh1 n ch i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "punching"  v ( p uh1 n ch i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "punching-ball"  n ( p uh1 n ch i ng - b oo l ) ((pos "K6$" ) ))
( "punching-balls"  n ( p uh1 n ch i ng - b oo l z ) ((pos "Kj$" ) ))
( "punctilio"  n ( p uh1 ng k t i1 l i@ @ ) ((pos "M6$" ) ))
( "punctilios"  n ( p uh1 ng k t i1 l i@ u z ) ((pos "Mj$" ) ))
( "punctilious"  j ( p uh1 ng k t i1 l i@ s ) ((pos "OA%" ) ))
( "punctiliously"  a ( p uh1 ng k t i1 l i@ s l ii ) ((pos "Pu%" ) ))
( "punctiliousness"  n ( p uh1 ng k t i1 l i@ s n @ s ) ((pos "L@%" ) ))
( "punctual"  j ( p uh1 ng k ch u@ l ) ((pos "OA%" ) ))
( "punctuality"  n ( p uh2 ng k ch u a1 l i t ii ) ((pos "L@%" ) ))
( "punctually"  a ( p uh1 ng k ch u@ l ii ) ((pos "Pu%" ) ))
( "punctuate"  v ( p uh1 ng k ch u ei t ) ((pos "H2%" ) (subcat "6A" )))
( "punctuated"  v ( p uh1 ng k ch u ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "punctuates"  v ( p uh1 ng k ch u ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "punctuating"  v ( p uh1 ng k ch u ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "punctuation"  n ( p uh2 ng k ch u ei1 sh @ n ) ((pos "L@%" ) ))
( "puncture"  v ( p uh1 ng k ch @ r ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "punctured"  v ( p uh1 ng k ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "punctures"  v ( p uh1 ng k ch @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "puncturing"  v ( p uh1 ng k ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "pundit"  n ( p uh1 n d i t ) ((pos "K6%" ) ))
( "pundits"  n ( p uh1 n d i t s ) ((pos "Kj%" ) ))
( "pungency"  n ( p uh1 n jh @ n s ii ) ((pos "L@%" ) ))
( "pungent"  j ( p uh1 n jh @ n t ) ((pos "OA%" ) ))
( "pungently"  a ( p uh1 n jh @ n t l ii ) ((pos "Pu%" ) ))
( "punier"  j ( p y uu1 n i@ r ) ((pos "Or%" ) ))
( "puniest"  j ( p y uu1 n i i s t ) ((pos "Os%" ) ))
( "punily"  a ( p y uu1 n i l ii ) ((pos "Pu%" ) ))
( "punish"  v ( p uh1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "punishable"  j ( p uh1 n i sh @ b l ) ((pos "OA%" ) ))
( "punished"  v ( p uh1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "punishes"  v ( p uh1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "punishing"  v ( p uh1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "punishment"  n ( p uh1 n i sh m @ n t ) ((pos "M6%" ) ))
( "punishments"  n ( p uh1 n i sh m @ n t s ) ((pos "Mj%" ) ))
( "punitive"  j ( p y uu1 n i t i v ) ((pos "OA%" ) ))
( "punk"  n ( p uh1 ng k ) ((pos "M6%" ) ))
( "punkah"  n ( p uh1 ng k @ ) ((pos "K6$" ) ))
( "punkahs"  n ( p uh1 ng k @ z ) ((pos "Kj$" ) ))
( "punks"  n ( p uh1 ng k s ) ((pos "Mj%" ) ))
( "punned"  v ( p uh1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "punnet"  n ( p uh1 n i t ) ((pos "K6%" ) ))
( "punnets"  n ( p uh1 n i t s ) ((pos "Kj%" ) ))
( "punning"  v ( p uh1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "puns"  v ( p uh1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "punster"  n ( p uh1 n s t @ r ) ((pos "K6$" ) ))
( "punsters"  n ( p uh1 n s t @ z ) ((pos "Kj$" ) ))
( "punt"  v ( p uh1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "punted"  v ( p uh1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "punter"  n ( p uh1 n t @ r ) ((pos "K6%" ) ))
( "punters"  n ( p uh1 n t @ z ) ((pos "Kj%" ) ))
( "punting"  v ( p uh1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "punts"  v ( p uh1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "puny"  j ( p y uu1 n ii ) ((pos "OD%" ) ))
( "pup"  n ( p uh1 p ) ((pos "K6%" ) ))
( "pupa"  n ( p y uu1 p @ ) ((pos "K6$" ) ))
( "pupae"  n ( p y uu1 p ii ) ((pos "Kj$" ) ))
( "pupas"  n ( p y uu1 p @ z ) ((pos "Kj$" ) ))
( "pupil"  n ( p y uu1 p l ) ((pos "K6%" ) ))
( "pupils"  n ( p y uu1 p l z ) ((pos "Kj%" ) ))
( "puppet"  n ( p uh1 p i t ) ((pos "K6%" ) ))
( "puppeteer"  n ( p uh2 p i t i@1 r ) ((pos "K6%" ) ))
( "puppeteers"  n ( p uh2 p i t i@1 z ) ((pos "Kj%" ) ))
( "puppets"  n ( p uh1 p i t s ) ((pos "Kj%" ) ))
( "puppies"  n ( p uh1 p i z ) ((pos "Kj%" ) ))
( "puppy"  n ( p uh1 p ii ) ((pos "K8%" ) ))
( "pups"  n ( p uh1 p s ) ((pos "Kj%" ) ))
( "pur_ee"  n ( p y u@1 r ei ) ((pos "K6%" ) ))
( "pur_ees"  n ( p y u@1 r ei z ) ((pos "Kj$" ) ))
( "purblind"  j ( p @@1 b l ai n d ) ((pos "OA$" ) ))
( "purchasable"  j ( p @@1 ch @ s @ b l ) ((pos "OA%" ) ))
( "purchase"  v ( p @@1 ch @ s ) ((pos "H2%" "M6%" ) (subcat )))
( "purchased"  v ( p @@1 ch @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "purchaser"  n ( p @@1 ch @ s @ r ) ((pos "K6%" ) ))
( "purchasers"  n ( p @@1 ch @ s @ z ) ((pos "Kj%" ) ))
( "purchases"  v ( p @@1 ch @ s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "purchasing"  v ( p @@1 ch @ s i ng ) ((pos "Hb%" ) (subcat )))
( "purdah"  n ( p @@1 d @ ) ((pos "L@$" ) ))
( "pure"  j ( p y u@1 r ) ((pos "OB%" ) ))
( "purely"  a ( p y u@1 l ii ) ((pos "Pu%" ) ))
( "pureness"  n ( p y u@1 n @ s ) ((pos "L@%" ) ))
( "purer"  j ( p y u@1 r @ r ) ((pos "Or%" ) ))
( "purest"  j ( p y u@1 r i s t ) ((pos "Os%" ) ))
( "purgation"  n ( p @@1 g ei1 sh @ n ) ((pos "L@$" ) ))
( "purgative"  n ( p @@1 g @ t i v ) ((pos "K6%" "OA%" ) ))
( "purgatives"  n ( p @@1 g @ t i v z ) ((pos "Kj%" ) ))
( "purgatorial"  j ( p @@2 g @ t oo1 r i@ l ) ((pos "OA$" ) ))
( "purgatories"  n ( p @@1 g @ t r i z ) ((pos "Kj%" ) ))
( "purgatory"  n ( p @@1 g @ t r ii ) ((pos "K8%" ) ))
( "purge"  v ( p @@1 jh ) ((pos "H2%" "K6%" ) (subcat "6A" "14" "15A" )))
( "purged"  v ( p @@1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "15A" )))
( "purges"  v ( p @@1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" "15A" )))
( "purging"  v ( p @@1 jh i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15A" )))
( "purification"  n ( p y u@2 r i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "purifications"  n ( p y u@2 r i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "purified"  v ( p y u@1 r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "purifier"  n ( p y u@1 r i f ai @ r ) ((pos "K6%" ) ))
( "purifiers"  n ( p y u@1 r i f ai @ z ) ((pos "Kj%" ) ))
( "purifies"  v ( p y u@1 r i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "purify"  v ( p y u@1 r i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "purifying"  v ( p y u@1 r i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "purist"  n ( p y u@1 r i s t ) ((pos "K6%" ) ))
( "purists"  n ( p y u@1 r i s t s ) ((pos "Kj%" ) ))
( "puritan"  n ( p y u@1 r i t @ n ) ((pos "K6%" "OA%" ) ))
( "puritanical"  j ( p y u@2 r i t a1 n i k l ) ((pos "OA%" ) ))
( "puritanically"  a ( p y u@2 r i t a1 n i k l ii ) ((pos "Pu%" ) ))
( "puritanism"  n ( p y u@1 r i t @ n i z @ m ) ((pos "L@%" ) ))
( "puritans"  n ( p y u@1 r i t @ n z ) ((pos "Kj%" ) ))
( "purity"  n ( p y u@1 r i t ii ) ((pos "L@%" ) ))
( "purl"  v ( p @@1 l ) ((pos "J0$" "K6$" ) ))
( "purled"  v ( p @@1 l d ) ((pos "Jc$" "Jd$" ) ))
( "purlieus"  n ( p @@1 l y uu z ) ((pos "Kj$" ) ))
( "purling"  v ( p @@1 l i ng ) ((pos "Jb$" ) ))
( "purloin"  v ( p @@1 l oi1 n ) ((pos "H0%" ) (subcat )))
( "purloined"  v ( p @@1 l oi1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "purloining"  v ( p @@1 l oi1 n i ng ) ((pos "Hb%" ) (subcat )))
( "purloins"  v ( p @@1 l oi1 n z ) ((pos "Ha%" ) (subcat )))
( "purls"  v ( p @@1 l z ) ((pos "Ja$" "Kj$" ) ))
( "purple"  n ( p @@1 p l ) ((pos "M6%" "OA%" ) ))
( "purples"  n ( p @@1 p l z ) ((pos "Mj%" ) ))
( "purplish"  j ( p @@1 p l i sh ) ((pos "OA%" ) ))
( "purport"  n ( p @@1 p @ t ) ((pos "L@%" ) ))
( "purport"  v ( p @1 p oo1 t ) ((pos "H0%" ) (subcat "6A" "7A" )))
( "purported"  v ( p @1 p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "purporting"  v ( p @1 p oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "purports"  v ( p @1 p oo1 t s ) ((pos "Ha%" ) (subcat "6A" "7A" )))
( "purpose"  v ( p @@1 p @ s ) ((pos "H2$" "M6%" ) (subcat "6A" "6D" "7A" )))
( "purpose-built"  j ( p @@1 p @ s - b i l t ) ((pos "OA%" ) ))
( "purposed"  v ( p @@1 p @ s t ) ((pos "Hc$" "Hd$" ) (subcat "6A" "6D" "7A" )))
( "purposeful"  j ( p @@1 p @ s f @ l ) ((pos "OA%" ) ))
( "purposefully"  a ( p @@1 p @ s f @ l ii ) ((pos "Pu%" ) ))
( "purposeless"  j ( p @@1 p @ s l @ s ) ((pos "OA%" ) ))
( "purposelessly"  a ( p @@1 p @ s l @ s l ii ) ((pos "Pu%" ) ))
( "purposely"  a ( p @@1 p @ s l ii ) ((pos "Pu%" ) ))
( "purposes"  v ( p @@1 p @ s i z ) ((pos "Ha$" "Mj%" ) (subcat "6A" "6D" "7A" )))
( "purposing"  v ( p @@1 p @ s i ng ) ((pos "Hb$" ) (subcat "6A" "6D" "7A" )))
( "purposive"  j ( p @@1 p @ s i v ) ((pos "OA$" ) ))
( "purr"  v ( p @@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "purred"  v ( p @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "purring"  v ( p @@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "purrs"  v ( p @@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "purse"  v ( p @@1 s ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "purse-proud"  j ( p @@1 s - p r au d ) ((pos "OA$" ) ))
( "pursed"  v ( p @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "purser"  n ( p @@1 s @ r ) ((pos "K6%" ) ))
( "pursers"  n ( p @@1 s @ z ) ((pos "Kj%" ) ))
( "purses"  v ( p @@1 s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "pursing"  v ( p @@1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "pursuance"  n ( p @1 s y uu1 @ n s ) ((pos "K6%" ) ))
( "pursuances"  n ( p @1 s y uu1 @ n s i z ) ((pos "Kj$" ) ))
( "pursuant"  j ( p @1 s y uu1 @ n t ) ((pos "OA%" ) ))
( "pursue"  v ( p @1 s y uu1 ) ((pos "H2%" ) (subcat )))
( "pursued"  v ( p @1 s y uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "pursuer"  n ( p @1 s y uu1 @ r ) ((pos "K6%" ) ))
( "pursuers"  n ( p @1 s y uu1 @ z ) ((pos "Kj%" ) ))
( "pursues"  v ( p @1 s y uu1 z ) ((pos "Ha%" ) (subcat )))
( "pursuing"  v ( p @1 s y uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "pursuit"  n ( p @1 s y uu1 t ) ((pos "M6%" ) ))
( "pursuits"  n ( p @1 s y uu1 t s ) ((pos "Mj%" ) ))
( "pursy"  j ( p @@1 s ii ) ((pos "OA$" ) ))
( "purulence"  n ( p y u@1 r @ l @ n s ) ((pos "L@$" ) ))
( "purulent"  j ( p y u@1 r @ l @ n t ) ((pos "OA$" ) ))
( "purvey"  v ( p @1 v ei1 ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "purveyance"  n ( p @1 v ei1 @ n s ) ((pos "M6%" ) ))
( "purveyances"  n ( p @1 v ei1 @ n s i z ) ((pos "Mj$" ) ))
( "purveyed"  v ( p @1 v ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "purveying"  v ( p @1 v ei1 i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "purveyor"  n ( p @1 v ei1 @ r ) ((pos "K6%" ) ))
( "purveyors"  n ( p @1 v ei1 @ z ) ((pos "Kj%" ) ))
( "purveys"  v ( p @1 v ei1 z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "purview"  n ( p @@1 v y uu ) ((pos "K6$" ) ))
( "purviews"  n ( p @@1 v y uu z ) ((pos "Kj$" ) ))
( "pus"  n ( p uh1 s ) ((pos "L@%" ) ))
( "push"  v ( p u1 sh ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "push-bike"  n ( p u1 sh - b ai k ) ((pos "K6%" ) ))
( "push-bikes"  n ( p u1 sh - b ai k s ) ((pos "Kj%" ) ))
( "pushcart"  n ( p u1 sh k aa t ) ((pos "K6%" ) ))
( "pushcarts"  n ( p u1 sh k aa t s ) ((pos "Kj%" ) ))
( "pushchair"  n ( p u1 sh ch e@ r ) ((pos "K6%" ) ))
( "pushchairs"  n ( p u1 sh ch e@ z ) ((pos "Kj%" ) ))
( "pushed"  v ( p u1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "pusher"  n ( p u1 sh @ r ) ((pos "K6%" ) ))
( "pushers"  n ( p u1 sh @ z ) ((pos "Kj%" ) ))
( "pushes"  v ( p u1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "pushful"  j ( p u1 sh f u l ) ((pos "OA$" ) ))
( "pushing"  v ( p u1 sh i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "pushover"  n ( p u1 sh ou v @ r ) ((pos "K6%" ) ))
( "pushovers"  n ( p u1 sh ou v @ z ) ((pos "Kj$" ) ))
( "pusillanimity"  n ( p y uu2 s i l @ n i1 m i t ii ) ((pos "L@$" ) ))
( "pusillanimous"  j ( p y uu2 s i l a1 n i m @ s ) ((pos "OA%" ) ))
( "puss"  n ( p u1 s ) ((pos "Ki%" ) ))
( "pussies"  n ( p u1 s i z ) ((pos "Kj%" ) ))
( "pussy"  n ( p u1 s ii ) ((pos "K8%" ) ))
( "pussycat"  n ( p u1 s i k a t ) ((pos "K6%" ) ))
( "pussycats"  n ( p u1 s i k a t s ) ((pos "Kj%" ) ))
( "pussyfoot"  v ( p u1 s i f u t ) ((pos "I0%" ) (subcat "2A" )))
( "pussyfooted"  v ( p u1 s i f u t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "pussyfooting"  v ( p u1 s i f u t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "pussyfoots"  v ( p u1 s i f u t s ) ((pos "Ia%" ) (subcat "2A" )))
( "pustule"  n ( p uh1 s t y uu l ) ((pos "K6$" ) ))
( "pustules"  n ( p uh1 s t y uu l z ) ((pos "Kj$" ) ))
( "put"  v ( p u1 t ) ((pos "J5*" "Jc*" "Jd*" ) (subcat "2C" "6A" "14" "15A" "15B" )))
( "put"  v ( p uh1 t ) ((pos "J4%" "K6%" ) ))
( "put-down"  n ( p u1 t - d au n ) ((pos "K6%" ) ))
( "put-downs"  n ( p u1 t - d au n z ) ((pos "Kj$" ) ))
( "put-on"  n ( p u1 t - o n ) ((pos "K6%" ) ))
( "put-ons"  n ( p u1 t - o n z ) ((pos "Kj$" ) ))
( "putative"  j ( p y uu1 t @ t i v ) ((pos "OA%" ) ))
( "putrefaction"  n ( p y uu2 t r i f a1 k sh @ n ) ((pos "K6%" ) ))
( "putrefactions"  n ( p y uu2 t r i f a1 k sh @ n z ) ((pos "Kj$" ) ))
( "putrefied"  v ( p y uu1 t r i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "putrefies"  v ( p y uu1 t r i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "putrefy"  v ( p y uu1 t r i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "putrefying"  v ( p y uu1 t r i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "putrescence"  n ( p y uu1 t r e1 s n s ) ((pos "K6$" ) ))
( "putrescences"  n ( p y uu1 t r e1 s n s i z ) ((pos "Kj$" ) ))
( "putrescent"  j ( p y uu1 t r e1 s n t ) ((pos "OA$" ) ))
( "putrid"  j ( p y uu1 t r i d ) ((pos "OA%" ) ))
( "putridity"  n ( p y uu1 t r i1 d i t ii ) ((pos "L@$" ) ))
( "puts"  v ( p u1 t s ) ((pos "Ja%" ) (subcat "2C" "6A" "14" "15A" "15B" )))
( "puts"  v ( p uh1 t s ) ((pos "Ja$" "Kj$" ) ))
( "putsch"  n ( p u1 ch ) ((pos "K7$" ) ))
( "putsches"  n ( p u1 ch i z ) ((pos "Kj$" ) ))
( "putt"  v ( p uh1 t ) ((pos "J0$" "K6$" ) (subcat "2A" )))
( "putted"  v ( p uh1 t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "puttee"  n ( p uh1 t ii ) ((pos "K6$" ) ))
( "puttees"  n ( p uh1 t i z ) ((pos "Kj$" ) ))
( "putter"  v ( p uh1 t @ r ) ((pos "J0$" ) ))
( "puttered"  v ( p uh1 t @ d ) ((pos "Jc$" "Jd$" ) ))
( "puttering"  v ( p uh1 t @ r i ng ) ((pos "Jb$" ) ))
( "putters"  v ( p uh1 t @ z ) ((pos "Ja$" ) ))
( "puttied"  v ( p uh1 t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "putties"  v ( p uh1 t i z ) ((pos "Ha$" ) (subcat "6A" )))
( "putting"  v ( p uh1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "putting-green"  n ( p uh1 t i ng - g r ii n ) ((pos "K6%" ) ))
( "putting-greens"  n ( p uh1 t i ng - g r ii n z ) ((pos "Kj%" ) ))
( "putting-iron"  n ( p uh1 t i ng - ai @ n ) ((pos "K6%" ) ))
( "putting-irons"  n ( p uh1 t i ng - ai @ n z ) ((pos "Kj%" ) ))
( "putts"  v ( p uh1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "putty"  v ( p uh1 t ii ) ((pos "H3$" "L@%" ) (subcat "6A" )))
( "puttying"  v ( p uh1 t i i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "puzzle"  v ( p uh1 z l ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" )))
( "puzzled"  v ( p uh1 z l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "puzzlement"  n ( p uh1 z l m @ n t ) ((pos "K6%" ) ))
( "puzzlements"  n ( p uh1 z l m @ n t s ) ((pos "Kj%" ) ))
( "puzzler"  n ( p uh1 z l @ r ) ((pos "K6%" ) ))
( "puzzlers"  n ( p uh1 z l @ z ) ((pos "Kj%" ) ))
( "puzzles"  v ( p uh1 z l z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "puzzling"  v ( p uh1 z l i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "pygmies"  n ( p i1 g m i z ) ((pos "Kj%" ) ))
( "pygmy"  n ( p i1 g m ii ) ((pos "K8%" ) ))
( "pyjama"  j ( p @1 jh aa1 m @ ) ((pos "Oq%" ) ))
( "pyjamas"  n ( p @1 jh aa1 m @ z ) ((pos "Kj%" ) ))
( "pylon"  n ( p ai1 l o n ) ((pos "K6%" ) ))
( "pylons"  n ( p ai1 l o n z ) ((pos "Kj%" ) ))
( "pyorrhoea"  n ( p ai2 @ r i@1 ) ((pos "L@$" ) ))
( "pyramid"  n ( p i1 r @ m i d ) ((pos "K6%" ) ))
( "pyramids"  n ( p i1 r @ m i d z ) ((pos "Kj%" ) ))
( "pyre"  n ( p ai1 @ r ) ((pos "K6$" ) ))
( "pyres"  n ( p ai1 @ z ) ((pos "Kj$" ) ))
( "pyrites"  n ( p ai2 @ r ai1 t ii z ) ((pos "L@$" ) ))
( "pyrotechnic"  j ( p ai2 r ou t e1 k n i k ) ((pos "OA%" ) ))
( "pyrotechnics"  n ( p ai2 r @ t e1 k n i k s ) ((pos "Kj%" ) ))
( "python"  n ( p ai1 th n ) ((pos "K6%" ) ))
( "pythons"  n ( p ai1 th n z ) ((pos "Kj%" ) ))
( "pyx"  n ( p i1 k s ) ((pos "K7$" ) ))
( "pyxes"  n ( p i1 k s i z ) ((pos "Kj$" ) ))
( "q"  n ( k y uu1 ) ((pos "Ki$" ) ))
( "q's"  n ( k y uu1 z ) ((pos "Kj$" ) ))
( "qt"  n ( k w oo1 t ) ((pos "Y]$" ) ))
( "qua"  in ( k w ei ) ((pos "T-$" "V-$" ) ))
( "quack"  v ( k w a1 k ) ((pos "I0%" "K6%" ) ))
( "quack-quack"  n ( k w a1 k - k w a k ) ((pos "K6%" ) ))
( "quack-quacks"  n ( k w a1 k - k w a k s ) ((pos "Kj%" ) ))
( "quacked"  v ( k w a1 k t ) ((pos "Ic%" "Id%" ) ))
( "quackery"  n ( k w a1 k @ r ii ) ((pos "L@$" ) ))
( "quacking"  v ( k w a1 k i ng ) ((pos "Ib%" ) ))
( "quacks"  v ( k w a1 k s ) ((pos "Ia%" "Kj%" ) ))
( "quad"  n ( k w o1 d ) ((pos "K6%" ) ))
( "quadrangle"  n ( k w o1 d r a ng g l ) ((pos "K6%" ) ))
( "quadrangles"  n ( k w o1 d r a ng g l z ) ((pos "Kj%" ) ))
( "quadrangular"  j ( k w o1 d r a1 ng g y u l @ r ) ((pos "OA%" ) ))
( "quadrant"  n ( k w o1 d r @ n t ) ((pos "K6%" ) ))
( "quadrants"  n ( k w o1 d r @ n t s ) ((pos "Kj%" ) ))
( "quadratic"  j ( k w o1 d r a1 t i k ) ((pos "OA%" ) ))
( "quadrilateral"  n ( k w o2 d r i l a1 t @ r @ l ) ((pos "K6%" "OA%" ) ))
( "quadrilaterals"  n ( k w o2 d r i l a1 t @ r @ l z ) ((pos "Kj%" ) ))
( "quadrille"  n ( k w @1 d r i1 l ) ((pos "K6$" ) ))
( "quadrilles"  n ( k w @1 d r i1 l z ) ((pos "Kj$" ) ))
( "quadrillion"  n ( k w o1 d r i1 l i@ n ) ((pos "K6$" ) ))
( "quadrillions"  n ( k w o1 d r i1 l i@ n z ) ((pos "Kj$" ) ))
( "quadrophonic"  j ( k w o2 d r @ f o1 n i k ) ((pos "OA%" ) ))
( "quadrophony"  n ( k w o1 d r o1 f @ n ii ) ((pos "L@$" ) ))
( "quadruped"  n ( k w o1 d r u p e d ) ((pos "K6%" ) ))
( "quadrupeds"  n ( k w o1 d r u p e d z ) ((pos "Kj%" ) ))
( "quadruple"  n ( k w o1 d r uu p l ) ((pos "K6%" "OA%" ) ))
( "quadruple"  v ( k w o1 d r uu1 p l ) ((pos "J2%" ) (subcat "2A" )))
( "quadrupled"  v ( k w o1 d r uu1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "quadruples"  n ( k w o1 d r uu p l z ) ((pos "Kj%" ) ))
( "quadruples"  v ( k w o1 d r uu1 p l z ) ((pos "Ja%" ) (subcat "2A" )))
( "quadruplet"  n ( k w o1 d r uu p l @ t ) ((pos "K6$" ) ))
( "quadruplets"  n ( k w o1 d r uu p l @ t s ) ((pos "Kj$" ) ))
( "quadruplicate"  n ( k w o1 d r uu1 p l i k @ t ) ((pos "K6%" "OA%" ) ))
( "quadruplicate"  v ( k w o1 d r uu1 p l i k ei t ) ((pos "H2$" ) (subcat )))
( "quadruplicated"  v ( k w o1 d r uu1 p l i k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "quadruplicates"  n ( k w o1 d r uu1 p l i k @ t s ) ((pos "Kj$" ) ))
( "quadruplicates"  v ( k w o1 d r uu1 p l i k ei t s ) ((pos "Ha$" ) (subcat )))
( "quadruplicating"  v ( k w o1 d r uu1 p l i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "quadrupling"  v ( k w o1 d r uu1 p l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "quads"  n ( k w o1 d z ) ((pos "Kj%" ) ))
( "quaff"  v ( k w o1 f ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "quaffed"  v ( k w o1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "quaffing"  v ( k w o1 f i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "quaffs"  v ( k w o1 f s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "quagga"  n ( k w a1 g @ ) ((pos "K6$" ) ))
( "quaggas"  n ( k w a1 g @ z ) ((pos "Kj$" ) ))
( "quagmire"  n ( k w a1 g m ai @ r ) ((pos "K6%" ) ))
( "quagmires"  n ( k w a1 g m ai @ z ) ((pos "Kj%" ) ))
( "quail"  v ( k w ei1 l ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "quailed"  v ( k w ei1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "quailing"  v ( k w ei1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "quails"  v ( k w ei1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "quaint"  j ( k w ei1 n t ) ((pos "OC%" ) ))
( "quainter"  j ( k w ei1 n t @ r ) ((pos "Or%" ) ))
( "quaintest"  j ( k w ei1 n t i s t ) ((pos "Os%" ) ))
( "quaintly"  a ( k w ei1 n t l ii ) ((pos "Pu%" ) ))
( "quaintness"  n ( k w ei1 n t n @ s ) ((pos "L@%" ) ))
( "quake"  v ( k w ei1 k ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "quaked"  v ( k w ei1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "quakes"  v ( k w ei1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "quaking"  v ( k w ei1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "qualification"  n ( k w o2 l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "qualifications"  n ( k w o2 l i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "qualified"  v ( k w o1 l i f ai d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "3A" "4A" "6A" "14" "16B" )))
( "qualifier"  n ( k w o1 l i f ai @ r ) ((pos "K6%" ) ))
( "qualifiers"  n ( k w o1 l i f ai @ z ) ((pos "Kj%" ) ))
( "qualifies"  v ( k w o1 l i f ai z ) ((pos "Ja%" ) (subcat "2C" "3A" "4A" "6A" "14" "16B" )))
( "qualify"  v ( k w o1 l i f ai ) ((pos "J3%" ) (subcat "2C" "3A" "4A" "6A" "14" "16B" )))
( "qualifying"  v ( k w o1 l i f ai i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "4A" "6A" "14" "16B" )))
( "qualitative"  j ( k w o1 l i t @ t i v ) ((pos "OA%" ) ))
( "qualitatively"  a ( k w o1 l i t @ t i v l ii ) ((pos "Pu%" ) ))
( "qualities"  n ( k w o1 l i t i z ) ((pos "Mj%" ) ))
( "quality"  n ( k w o1 l i t ii ) ((pos "M8%" ) ))
( "qualm"  n ( k w aa1 m ) ((pos "K6%" ) ))
( "qualms"  n ( k w aa1 m z ) ((pos "Kj%" ) ))
( "quandaries"  n ( k w o1 n d @ r i z ) ((pos "Kj$" ) ))
( "quandary"  n ( k w o1 n d @ r ii ) ((pos "K8%" ) ))
( "quango"  n ( k w a1 ng g ou ) ((pos "K6$" ) ))
( "quangos"  n ( k w a1 ng g ou z ) ((pos "Kj$" ) ))
( "quanta"  n ( k w o1 n t @ ) ((pos "Kj$" ) ))
( "quantified"  v ( k w o1 n t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quantifies"  v ( k w o1 n t i f ai z ) ((pos "Ha%" ) (subcat )))
( "quantify"  v ( k w o1 n t i f ai ) ((pos "H3%" ) (subcat )))
( "quantifying"  v ( k w o1 n t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "quantitative"  j ( k w o1 n t i t @ t i v ) ((pos "OA%" ) ))
( "quantitatively"  a ( k w o1 n t i t @ t i v l ii ) ((pos "Pu%" ) ))
( "quantities"  n ( k w o1 n t i t i z ) ((pos "Mj%" ) ))
( "quantity"  n ( k w o1 n t i t ii ) ((pos "M8%" ) ))
( "quantum"  n ( k w o1 n t @ m ) ((pos "Ki%" ) ))
( "quarantine"  v ( k w o1 r @ n t ii n ) ((pos "H2%" "L@%" ) (subcat )))
( "quarantined"  v ( k w o1 r @ n t ii n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quarantines"  v ( k w o1 r @ n t ii n z ) ((pos "Ha%" ) (subcat )))
( "quarantining"  v ( k w o1 r @ n t ii n i ng ) ((pos "Hb%" ) (subcat )))
( "quark"  n ( k w aa1 k ) ((pos "K6$" ) ))
( "quarks"  n ( k w aa1 k s ) ((pos "Kj$" ) ))
( "quarrel"  v ( k w o1 r @ l ) ((pos "I4%" "K6%" ) (subcat "2A" "2C" )))
( "quarrelled"  v ( k w o1 r @ l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "quarrelling"  v ( k w o1 r @ l i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "quarrels"  v ( k w o1 r @ l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "quarrelsome"  j ( k w o1 r @ l s @ m ) ((pos "OA%" ) ))
( "quarried"  v ( k w o1 r i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "quarries"  v ( k w o1 r i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "quarry"  v ( k w o1 r ii ) ((pos "J3%" "K8%" ) (subcat "2A" "2C" "6A" "15A" )))
( "quarrying"  v ( k w o1 r i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "quarryman"  n ( k w o1 r i m @ n ) ((pos "Ki%" ) ))
( "quarrymen"  n ( k w o1 r i m @ n ) ((pos "Kj%" ) ))
( "quart"  n ( k w oo1 t ) ((pos "K6%" ) ))
( "quarter"  v ( k w oo1 t @ r ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "quarter-day"  n ( k w oo1 t @ - d ei ) ((pos "K6%" ) ))
( "quarter-days"  n ( k w oo1 t @ - d ei z ) ((pos "Kj%" ) ))
( "quarter-deck"  n ( k w oo1 t @ - d e k ) ((pos "K6%" ) ))
( "quarter-decks"  n ( k w oo1 t @ - d e k s ) ((pos "Kj%" ) ))
( "quarter-plate"  n ( k w oo1 t @ - p l ei t ) ((pos "K6$" ) ))
( "quarter-plates"  n ( k w oo1 t @ - p l ei t s ) ((pos "Kj$" ) ))
( "quartered"  v ( k w oo1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "quarterfinal"  n ( k w oo2 t @ f ai1 n l ) ((pos "K6%" ) ))
( "quarterfinals"  n ( k w oo2 t @ f ai1 n l z ) ((pos "Kj%" ) ))
( "quartering"  v ( k w oo1 t @ r i ng ) ((pos "Hb$" "K6$" ) (subcat "6A" )))
( "quarterings"  n ( k w oo1 t @ r i ng z ) ((pos "Kj$" ) ))
( "quarterlies"  n ( k w oo1 t @ l i z ) ((pos "Kj%" ) ))
( "quarterlight"  n ( k w oo1 t @ l ai t ) ((pos "K6$" ) ))
( "quarterlights"  n ( k w oo1 t @ l ai t s ) ((pos "Kj$" ) ))
( "quarterly"  n ( k w oo1 t @ l ii ) ((pos "K8%" "OA%" "Pu%" ) ))
( "quartermaster"  n ( k w oo1 t @ m aa s t @ r ) ((pos "K6%" ) ))
( "quartermaster-general"  n ( k w oo2 t @ m aa s t @ - jh e1 n r @ l ) ((pos "K6%" ) ))
( "quartermaster-generals"  n ( k w oo2 t @ m aa s t @ - jh e1 n r @ l z ) ((pos "Kj%" ) ))
( "quartermasters"  n ( k w oo1 t @ m aa s t @ z ) ((pos "Kj%" ) ))
( "quarters"  v ( k w oo1 t @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "quarterstaff"  n ( k w oo1 t @ s t aa f ) ((pos "K6$" ) ))
( "quarterstaffs"  n ( k w oo1 t @ s t aa f s ) ((pos "Kj$" ) ))
( "quartet"  n ( k w oo1 t e1 t ) ((pos "K6%" ) ))
( "quartets"  n ( k w oo1 t e1 t s ) ((pos "Kj%" ) ))
( "quarto"  n ( k w oo1 t ou ) ((pos "K6$" ) ))
( "quartos"  n ( k w oo1 t ou z ) ((pos "Kj$" ) ))
( "quarts"  n ( k w oo1 t s ) ((pos "Kj%" ) ))
( "quartz"  n ( k w oo1 t s ) ((pos "L@%" ) ))
( "quasar"  n ( k w ei1 z aa r ) ((pos "K6$" ) ))
( "quasars"  n ( k w ei1 z aa z ) ((pos "Kj$" ) ))
( "quash"  v ( k w o1 sh ) ((pos "H1%" ) (subcat )))
( "quashed"  v ( k w o1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quashes"  v ( k w o1 sh i z ) ((pos "Ha%" ) (subcat )))
( "quashing"  v ( k w o1 sh i ng ) ((pos "Hb%" ) (subcat )))
( "quasi-"  j ( k w ei1 s ai - ) ((pos "U-%" ) ))
( "quassia"  n ( k w o1 sh @ ) ((pos "L@$" ) ))
( "quatercentenaries"  n ( k w o2 t @ s e n t ii1 n @ r i z ) ((pos "Kj%" ) ))
( "quatercentenary"  n ( k w o2 t @ s e n t ii1 n @ r ii ) ((pos "K8%" ) ))
( "quatrain"  n ( k w o1 t r ei n ) ((pos "K6$" ) ))
( "quatrains"  n ( k w o1 t r ei n z ) ((pos "Kj$" ) ))
( "quattrocento"  n ( k w a2 t r ou ch e1 n t ou ) ((pos "K6$" ) ))
( "quattrocentos"  n ( k w a2 t r ou ch e1 n t ou z ) ((pos "Kj$" ) ))
( "quaver"  v ( k w ei1 v @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "quavered"  v ( k w ei1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "quavering"  v ( k w ei1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "quavers"  v ( k w ei1 v @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "quay"  n ( k ii1 ) ((pos "K6%" ) ))
( "quays"  n ( k ii1 z ) ((pos "Kj%" ) ))
( "queasier"  j ( k w ii1 z i@ r ) ((pos "Or%" ) ))
( "queasiest"  j ( k w ii1 z i i s t ) ((pos "Os%" ) ))
( "queasily"  a ( k w ii1 z i l ii ) ((pos "Pu%" ) ))
( "queasiness"  n ( k w ii1 z i n @ s ) ((pos "L@%" ) ))
( "queasy"  j ( k w ii1 z ii ) ((pos "OD%" ) ))
( "queen"  v ( k w ii1 n ) ((pos "H0$" "K6%" ) ))
( "queened"  v ( k w ii1 n d ) ((pos "Hc$" "Hd$" ) ))
( "queening"  v ( k w ii1 n i ng ) ((pos "Hb$" ) ))
( "queenlier"  j ( k w ii1 n l i@ r ) ((pos "Or$" ) ))
( "queenliest"  j ( k w ii1 n l i i s t ) ((pos "Os$" ) ))
( "queenly"  j ( k w ii1 n l ii ) ((pos "OD%" ) ))
( "queens"  v ( k w ii1 n z ) ((pos "Ha$" "Kj%" ) ))
( "queer"  v ( k w i@1 r ) ((pos "H0%" "K6%" "OC%" ) (subcat )))
( "queered"  v ( k w i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "queerer"  j ( k w i@1 r @ r ) ((pos "Or%" ) ))
( "queerest"  j ( k w i@1 r i s t ) ((pos "Os%" ) ))
( "queering"  v ( k w i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "queerly"  a ( k w i@1 l ii ) ((pos "Pu%" ) ))
( "queerness"  n ( k w i@1 n @ s ) ((pos "L@%" ) ))
( "queers"  v ( k w i@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "quell"  v ( k w e1 l ) ((pos "H0%" ) (subcat )))
( "quelled"  v ( k w e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quelling"  v ( k w e1 l i ng ) ((pos "Hb%" ) (subcat )))
( "quells"  v ( k w e1 l z ) ((pos "Ha%" ) (subcat )))
( "quench"  v ( k w e1 n ch ) ((pos "H1%" ) (subcat )))
( "quenched"  v ( k w e1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quenches"  v ( k w e1 n ch i z ) ((pos "Ha%" ) (subcat )))
( "quenching"  v ( k w e1 n ch i ng ) ((pos "Hb%" ) (subcat )))
( "quenchless"  j ( k w e1 n ch l @ s ) ((pos "OA$" ) ))
( "queried"  v ( k w i@1 r i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "queries"  v ( k w i@1 r i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "quern"  n ( k w @@1 n ) ((pos "K6$" ) ))
( "querns"  n ( k w @@1 n z ) ((pos "Kj$" ) ))
( "querulous"  j ( k w e1 r u l @ s ) ((pos "OA%" ) ))
( "querulously"  a ( k w e1 r u l @ s l ii ) ((pos "Pu%" ) ))
( "querulousness"  n ( k w e1 r u l @ s n @ s ) ((pos "L@%" ) ))
( "query"  v ( k w i@1 r ii ) ((pos "H3%" "K8%" ) (subcat "6A" )))
( "querying"  v ( k w i@1 r i i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "quest"  v ( k w e1 s t ) ((pos "I0$" "K6%" ) (subcat )))
( "quested"  v ( k w e1 s t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "questing"  v ( k w e1 s t i ng ) ((pos "Ib$" ) (subcat )))
( "question"  v ( k w e1 s ch @ n ) ((pos "H0%" "M6*" ) (subcat "6A" )))
( "question-mark"  n ( k w e1 s ch @ n - m aa k ) ((pos "K6%" ) ))
( "question-marks"  n ( k w e1 s ch @ n - m aa k s ) ((pos "Kj%" ) ))
( "question-master"  n ( k w e1 s ch @ n - m aa s t @ r ) ((pos "K6%" ) ))
( "question-masters"  n ( k w e1 s ch @ n - m aa s t @ z ) ((pos "Kj%" ) ))
( "questionable"  j ( k w e1 s ch @ n @ b l ) ((pos "OA%" ) ))
( "questionably"  a ( k w e1 s ch @ n @ b l ii ) ((pos "Pu%" ) ))
( "questioned"  v ( k w e1 s ch @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "questioner"  n ( k w e1 s ch @ n @ r ) ((pos "K6%" ) ))
( "questioners"  n ( k w e1 s ch @ n @ z ) ((pos "Kj%" ) ))
( "questioning"  v ( k w e1 s ch @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "questioningly"  a ( k w e1 s ch @ n i ng l ii ) ((pos "Pu%" ) ))
( "questionnaire"  n ( k w e2 s ch @ n e@1 r ) ((pos "K6%" ) ))
( "questionnaires"  n ( k w e2 s ch @ n e@1 z ) ((pos "Kj%" ) ))
( "questions"  v ( k w e1 s ch @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "quests"  v ( k w e1 s t s ) ((pos "Ia$" "Kj%" ) (subcat )))
( "quetzal"  n ( k w e1 t s l ) ((pos "K6$" ) ))
( "quetzals"  n ( k w e1 t s l z ) ((pos "Kj$" ) ))
( "queue"  v ( k y uu1 ) ((pos "I5%" "K6%" ) (subcat "2A" "2C" )))
( "queued"  v ( k y uu1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "queueing"  v ( k y uu1 i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "queues"  v ( k y uu1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "qui vive"  n ( k ii2 - v ii1 v ) ((pos "L@$" ) ))
( "quibble"  v ( k w i1 b l ) ((pos "I2%" "K6%" ) ))
( "quibbled"  v ( k w i1 b l d ) ((pos "Ic%" "Id%" ) ))
( "quibbler"  n ( k w i1 b l @ r ) ((pos "K6%" ) ))
( "quibblers"  n ( k w i1 b l @ z ) ((pos "Kj%" ) ))
( "quibbles"  v ( k w i1 b l z ) ((pos "Ia%" "Kj%" ) ))
( "quibbling"  v ( k w i1 b l i ng ) ((pos "Ib%" "OA%" ) ))
( "quiche"  n ( k ii1 sh ) ((pos "K6%" ) ))
( "quiches"  n ( k ii1 sh i z ) ((pos "Kj%" ) ))
( "quick"  n ( k w i1 k ) ((pos "L@%" "OC%" "Pu%" ) ))
( "quick-change"  j ( k w i1 k - ch ei1 n jh ) ((pos "Oq%" ) ))
( "quick-eared"  j ( k w i1 k - i@1 d ) ((pos "OA$" ) ))
( "quick-eyed"  j ( k w i1 k - ai d ) ((pos "OA$" ) ))
( "quick-freeze"  v ( k w ii1 k - f r ii1 z ) ((pos "H5$" ) ))
( "quick-freezes"  v ( k w ii1 k - f r ii1 z i z ) ((pos "Ha$" ) ))
( "quick-freezing"  v ( k w ii1 k - f r ii1 z i ng ) ((pos "Hb$" ) ))
( "quick-froze"  v ( k w ii1 k - f r ou1 z ) ((pos "Hc$" ) ))
( "quick-frozen"  v ( k w ii1 k - f r ou1 z n ) ((pos "Hd%" ) ))
( "quick-sighted"  j ( k w i1 k - s ai1 t i d ) ((pos "OA%" ) ))
( "quick-tempered"  j ( k w i1 k - t e1 m p @ d ) ((pos "OA%" ) ))
( "quick-witted"  j ( k w i1 k - w i1 t i d ) ((pos "OA%" ) ))
( "quicken"  v ( k w i1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "quickened"  v ( k w i1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "quickening"  v ( k w i1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "quickens"  v ( k w i1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "quicker"  j ( k w i1 k @ r ) ((pos "Or%" "Pu%" ) ))
( "quickest"  j ( k w i1 k i s t ) ((pos "Os%" "Pu%" ) ))
( "quickie"  n ( k w i1 k ii ) ((pos "K6%" ) ))
( "quickies"  n ( k w i1 k i z ) ((pos "Kj%" ) ))
( "quicklime"  n ( k w i1 k l ai m ) ((pos "L@%" ) ))
( "quickly"  a ( k w i1 k l ii ) ((pos "Pu%" ) ))
( "quickness"  n ( k w i1 k n @ s ) ((pos "L@%" ) ))
( "quicksand"  n ( k w i1 k s a n d ) ((pos "M6%" ) ))
( "quicksands"  n ( k w i1 k s a n d z ) ((pos "Mj%" ) ))
( "quickset"  j ( k w i1 k s e t ) ((pos "OA%" ) ))
( "quicksilver"  n ( k w i1 k s i l v @ r ) ((pos "L@%" ) ))
( "quickstep"  n ( k w i1 k s t e p ) ((pos "K6%" ) ))
( "quicksteps"  n ( k w i1 k s t e p s ) ((pos "Kj$" ) ))
( "quid"  n ( k w i1 d ) ((pos "K9%" ) ))
( "quid pro quo"  n ( k w i2 d - p r ou - k w ou1 ) ((pos "Ki%" ) ))
( "quids"  n ( k w i1 d z ) ((pos "Kj$" ) ))
( "quiescence"  n ( k w ai1 e1 s n s ) ((pos "Ki%" ) ))
( "quiescent"  j ( k w ai1 e1 s n t ) ((pos "OA%" ) ))
( "quiescently"  a ( k w ai1 e1 s n t l ii ) ((pos "Pu%" ) ))
( "quiet"  v ( k w ai1 @ t ) ((pos "J0%" "L@%" "OC%" ) ))
( "quieted"  v ( k w ai1 @ t i d ) ((pos "Jc%" "Jd%" ) ))
( "quieten"  v ( k w ai1 @ t n ) ((pos "J0%" ) (subcat "2C" "6A" )))
( "quietened"  v ( k w ai1 @ t n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "quietening"  v ( k w ai1 @ t n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "quietens"  v ( k w ai1 @ t n z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "quieter"  j ( k w ai1 @ t @ r ) ((pos "Or%" ) ))
( "quietest"  j ( k w ai1 @ t i s t ) ((pos "Os%" ) ))
( "quieting"  v ( k w ai1 @ t i ng ) ((pos "Jb%" ) ))
( "quietism"  n ( k w ai1 i t i z @ m ) ((pos "L@$" ) ))
( "quietist"  n ( k w ai1 @ t i s t ) ((pos "K6$" ) ))
( "quietists"  n ( k w ai1 @ t i s t s ) ((pos "Kj$" ) ))
( "quietly"  a ( k w ai1 @ t l ii ) ((pos "Pu%" ) ))
( "quietness"  n ( k w ai1 @ t n @ s ) ((pos "L@%" ) ))
( "quiets"  v ( k w ai1 @ t s ) ((pos "Ja%" ) ))
( "quietude"  n ( k w ai1 i t y uu d ) ((pos "Ki$" ) ))
( "quietus"  n ( k w ai1 ii1 t @ s ) ((pos "K7$" ) ))
( "quietuses"  n ( k w ai1 ii1 t @ s i z ) ((pos "Kj$" ) ))
( "quiff"  n ( k w i1 f ) ((pos "K6$" ) ))
( "quiffs"  n ( k w i1 f s ) ((pos "Kj$" ) ))
( "quill"  n ( k w i1 l ) ((pos "K6%" ) ))
( "quill-feather"  n ( k w i1 l - f e dh @ r ) ((pos "K6%" ) ))
( "quill-feathers"  n ( k w i1 l - f e dh @ z ) ((pos "Kj%" ) ))
( "quills"  n ( k w i1 l z ) ((pos "Kj%" ) ))
( "quilt"  v ( k w i1 l t ) ((pos "H0%" "K6%" ) ))
( "quilted"  v ( k w i1 l t i d ) ((pos "Hc%" "Hd%" ) ))
( "quilting"  v ( k w i1 l t i ng ) ((pos "Hb%" ) ))
( "quilts"  v ( k w i1 l t s ) ((pos "Ha%" "Kj%" ) ))
( "quin"  n ( k w i1 n ) ((pos "K6%" ) ))
( "quince"  n ( k w i1 n s ) ((pos "K6%" ) ))
( "quincentenaries"  n ( k w i2 n s e n t ii1 n @ r i z ) ((pos "Kj%" ) ))
( "quincentenary"  n ( k w i2 n s e n t ii1 n @ r ii ) ((pos "K8%" "OA%" ) ))
( "quinces"  n ( k w i1 n s i z ) ((pos "Kj%" ) ))
( "quinine"  n ( k w i1 n ii1 n ) ((pos "L@%" ) ))
( "quins"  n ( k w i1 n z ) ((pos "Kj%" ) ))
( "quinsy"  n ( k w i1 n z ii ) ((pos "L@$" ) ))
( "quintal"  n ( k w i1 n t l ) ((pos "K6$" ) ))
( "quintals"  n ( k w i1 n t l z ) ((pos "Kj$" ) ))
( "quintessence"  n ( k w i1 n t e1 s n s ) ((pos "K6%" ) ))
( "quintessences"  n ( k w i1 n t e1 s n s i z ) ((pos "Kj$" ) ))
( "quintet"  n ( k w i1 n t e1 t ) ((pos "K6%" ) ))
( "quintets"  n ( k w i1 n t e1 t s ) ((pos "Kj%" ) ))
( "quintuplet"  n ( k w i1 n t y uu p l e t ) ((pos "K6%" ) ))
( "quintuplets"  n ( k w i1 n t y uu p l e t s ) ((pos "Kj%" ) ))
( "quip"  v ( k w i1 p ) ((pos "I4%" "K6%" ) ))
( "quipped"  v ( k w i1 p t ) ((pos "Ic%" "Id%" ) ))
( "quipping"  v ( k w i1 p i ng ) ((pos "Ib%" ) ))
( "quips"  v ( k w i1 p s ) ((pos "Ia%" "Kj%" ) ))
( "quire"  n ( k w ai1 @ r ) ((pos "K6$" ) ))
( "quires"  n ( k w ai1 @ z ) ((pos "Kj$" ) ))
( "quirk"  n ( k w @@1 k ) ((pos "K6%" ) ))
( "quirks"  n ( k w @@1 k s ) ((pos "Kj%" ) ))
( "quisling"  n ( k w i1 z l i ng ) ((pos "K6$" ) ))
( "quislings"  n ( k w i1 z l i ng z ) ((pos "Kj$" ) ))
( "quit"  v ( k w i1 t ) ((pos "H4%" "Hc%" "Hd%" "Op%" ) (subcat "2A" "6A" )))
( "quite"  a ( k w ai1 t ) ((pos "Pu*" ) ))
( "quits"  v ( k w i1 t s ) ((pos "Ha%" "Op%" ) (subcat "2A" "6A" )))
( "quittance"  n ( k w i1 t n s ) ((pos "K6$" ) ))
( "quittances"  n ( k w i1 t n s i z ) ((pos "Kj$" ) ))
( "quitted"  v ( k w i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "quitter"  n ( k w i1 t @ r ) ((pos "K6%" ) ))
( "quitters"  n ( k w i1 t @ z ) ((pos "Kj%" ) ))
( "quitting"  v ( k w i1 t i ng ) ((pos "Hb%" ) (subcat "2A" "6A" )))
( "quiver"  v ( k w i1 v @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "quivered"  v ( k w i1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "quivering"  v ( k w i1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "quivers"  v ( k w i1 v @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "quixotic"  j ( k w i1 k s o1 t i k ) ((pos "OA%" ) ))
( "quixotically"  a ( k w i1 k s o1 t i k l ii ) ((pos "Pu%" ) ))
( "quiz"  v ( k w i1 z ) ((pos "H5%" "K7%" ) (subcat )))
( "quizes"  n ( k w i1 z i z ) ((pos "Kj%" ) ))
( "quizmaster"  n ( k w i1 z m aa s t @ r ) ((pos "K6%" ) ))
( "quizmasters"  n ( k w i1 z m aa s t @ z ) ((pos "Kj%" ) ))
( "quizzed"  v ( k w i1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "quizzes"  v ( k w i1 z i z ) ((pos "Ha%" ) (subcat )))
( "quizzical"  j ( k w i1 z i k l ) ((pos "OA%" ) ))
( "quizzically"  a ( k w i1 z i k l ii ) ((pos "Pu%" ) ))
( "quizzing"  v ( k w i1 z i ng ) ((pos "Hb%" ) (subcat )))
( "quoin"  n ( k oi1 n ) ((pos "K6$" ) ))
( "quoins"  n ( k oi1 n z ) ((pos "Kj$" ) ))
( "quoit"  n ( k oi1 t ) ((pos "K6%" ) ))
( "quoits"  n ( k oi1 t s ) ((pos "Kj%" ) ))
( "quorum"  n ( k w oo1 r @ m ) ((pos "K6%" ) ))
( "quorums"  n ( k w oo1 r @ m z ) ((pos "Kj$" ) ))
( "quota"  n ( k w ou1 t @ ) ((pos "K6%" ) ))
( "quotability"  n ( k w ou2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "quotable"  j ( k w ou1 t @ b l ) ((pos "OA%" ) ))
( "quotas"  n ( k w ou1 t @ z ) ((pos "Kj%" ) ))
( "quotation"  n ( k w ou1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "quotations"  n ( k w ou1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "quote"  v ( k w ou1 t ) ((pos "H2%" "K6%" ) (subcat "6A" "13A" )))
( "quoted"  v ( k w ou1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "13A" )))
( "quotes"  v ( k w ou1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "13A" )))
( "quoth"  v ( k w ou1 th ) ((pos "Hc$" ) ))
( "quotidian"  j ( k w ou1 t i1 d i@ n ) ((pos "OA$" ) ))
( "quotient"  n ( k w ou1 sh @ n t ) ((pos "K6%" ) ))
( "quotients"  n ( k w ou1 sh @ n t s ) ((pos "Kj%" ) ))
( "quoting"  v ( k w ou1 t i ng ) ((pos "Hb%" ) (subcat "6A" "13A" )))
( "qv"  n ( k y uu2 v ii1 ) ((pos "Y~$" ) ))
( "r"  n ( aa1 r ) ((pos "Ki$" ) ))
( "r's"  n ( aa1 z ) ((pos "Kj$" ) ))
( "r^ole"  n ( r ou1 l ) ((pos "K6%" ) ))
( "r^oles"  n ( r ou1 l z ) ((pos "Kj%" ) ))
( "r_echauff_e"  n ( r ei1 sh ou1 f ei ) ((pos "K6$" ) ))
( "r_echauff_es"  n ( r ei1 sh ou1 f ei z ) ((pos "Kj$" ) ))
( "r_egime"  n ( r ei1 zh ii1 m ) ((pos "K6%" ) ))
( "r_egimes"  n ( r ei1 zh ii1 m z ) ((pos "Kj%" ) ))
( "r_esum_e"  n ( r e1 z y uu m ei ) ((pos "K6%" ) ))
( "r_esum_es"  n ( r e1 z y uu m ei z ) ((pos "Kj%" ) ))
( "rabbi"  n ( r a1 b ai ) ((pos "K6%" ) ))
( "rabbinical"  j ( r @1 b i1 n i k l ) ((pos "OA$" ) ))
( "rabbis"  n ( r a1 b ai z ) ((pos "Kj%" ) ))
( "rabbit"  v ( r a1 b i t ) ((pos "I0%" "K6%" ) ))
( "rabbit-burrow"  n ( r a1 b i t - b uh r ou ) ((pos "K6%" ) ))
( "rabbit-burrows"  n ( r a1 b i t - b uh r ou z ) ((pos "Kj%" ) ))
( "rabbit-hole"  n ( r a1 b i t - h ou l ) ((pos "K6%" ) ))
( "rabbit-holes"  n ( r a1 b i t - h ou l z ) ((pos "Kj%" ) ))
( "rabbit-hutch"  n ( r a1 b i t - h uh ch ) ((pos "K7%" ) ))
( "rabbit-hutches"  n ( r a1 b i t - h uh ch i z ) ((pos "Kj%" ) ))
( "rabbit-punch"  n ( r a1 b i t - p uh n ch ) ((pos "K7%" ) ))
( "rabbit-punches"  n ( r a1 b i t - p uh n ch i z ) ((pos "Kj%" ) ))
( "rabbit-warren"  n ( r a1 b i t - w o r @ n ) ((pos "K6%" ) ))
( "rabbit-warrens"  n ( r a1 b i t - w o r @ n z ) ((pos "Kj%" ) ))
( "rabbited"  v ( r a1 b i t i d ) ((pos "Ic%" "Id%" ) ))
( "rabbiting"  v ( r a1 b i t i ng ) ((pos "Ib%" ) ))
( "rabbits"  v ( r a1 b i t s ) ((pos "Ia%" "Kj%" ) ))
( "rabble"  n ( r a1 b l ) ((pos "K6%" ) ))
( "rabble-rousing"  j ( r a1 b l - r au z i ng ) ((pos "OA%" ) ))
( "rabbles"  n ( r a1 b l z ) ((pos "Kj%" ) ))
( "rabid"  j ( r a1 b i d ) ((pos "OA%" ) ))
( "rabies"  n ( r ei1 b ii z ) ((pos "L@%" ) ))
( "raccoon"  n ( r @1 k uu1 n ) ((pos "K6%" ) ))
( "raccoons"  n ( r @1 k uu1 n z ) ((pos "Kj%" ) ))
( "race"  v ( r ei1 s ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "race-meeting"  n ( r ei1 s - m ii t i ng ) ((pos "K6%" ) ))
( "race-meetings"  n ( r ei1 s - m ii t i ng z ) ((pos "Kj%" ) ))
( "racecard"  n ( r ei1 s k aa d ) ((pos "K6%" ) ))
( "racecards"  n ( r ei1 s k aa d z ) ((pos "Kj%" ) ))
( "racecourse"  n ( r ei1 s k oo s ) ((pos "K6%" ) ))
( "racecourses"  n ( r ei1 s k oo s i z ) ((pos "Kj%" ) ))
( "raced"  v ( r ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "racehorse"  n ( r ei1 s h oo s ) ((pos "K6%" ) ))
( "racehorses"  n ( r ei1 s h oo s i z ) ((pos "Kj%" ) ))
( "raceme"  n ( r a1 s ii1 m ) ((pos "K6$" ) ))
( "racemes"  n ( r a1 s ii1 m z ) ((pos "Kj$" ) ))
( "racer"  n ( r ei1 s @ r ) ((pos "K6%" ) ))
( "racers"  n ( r ei1 s @ z ) ((pos "Kj%" ) ))
( "races"  v ( r ei1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "racial"  j ( r ei1 sh l ) ((pos "OA%" ) ))
( "racialism"  n ( r ei1 sh @ l i z @ m ) ((pos "L@%" ) ))
( "racialist"  n ( r ei1 sh @ l i s t ) ((pos "K6%" ) ))
( "racialists"  n ( r ei1 sh @ l i s t s ) ((pos "Kj%" ) ))
( "racially"  a ( r ei1 sh @ l ii ) ((pos "Pu%" ) ))
( "racier"  j ( r ei1 s i@ r ) ((pos "Or%" ) ))
( "raciest"  j ( r ei1 s i i s t ) ((pos "Os%" ) ))
( "racily"  a ( r ei1 s i l ii ) ((pos "Pu%" ) ))
( "raciness"  n ( r ei1 s i n @ s ) ((pos "L@%" ) ))
( "racing"  v ( r ei1 s i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "4A" "6A" )))
( "racism"  n ( r ei1 s i z @ m ) ((pos "L@%" ) ))
( "racist"  n ( r ei1 s i s t ) ((pos "K6%" ) ))
( "racists"  n ( r ei1 s i s t s ) ((pos "Kj%" ) ))
( "rack"  v ( r a1 k ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "rack-railway"  n ( r a1 k - r ei l w ei ) ((pos "K6$" ) ))
( "rack-railways"  n ( r a1 k - r ei l w ei z ) ((pos "Kj$" ) ))
( "rack-rent"  n ( r a1 k - r e n t ) ((pos "K6$" ) ))
( "rack-rents"  n ( r a1 k - r e n t s ) ((pos "Kj$" ) ))
( "racked"  v ( r a1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "racket"  v ( r a1 k i t ) ((pos "I0$" "M6%" ) (subcat "2A" )))
( "racketed"  v ( r a1 k i t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "racketeer"  n ( r a2 k @ t i@1 r ) ((pos "K6%" ) ))
( "racketeering"  n ( r a2 k @ t i@1 r i ng ) ((pos "L@%" ) ))
( "racketeers"  n ( r a2 k @ t i@1 z ) ((pos "Kj%" ) ))
( "racketing"  v ( r a1 k i t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "rackets"  v ( r a1 k i t s ) ((pos "Ia$" "Mj%" ) (subcat "2A" )))
( "racking"  v ( r a1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "racks"  v ( r a1 k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "raconteur"  n ( r a2 k o n t @@1 r ) ((pos "K6%" ) ))
( "raconteurs"  n ( r a2 k o n t @@1 z ) ((pos "Kj%" ) ))
( "racoon"  n ( r @1 k uu1 n ) ((pos "K6%" ) ))
( "racoons"  n ( r @1 k uu1 n z ) ((pos "Kj%" ) ))
( "racquet"  n ( r a1 k i t ) ((pos "K6%" ) ))
( "racquets"  n ( r a1 k i t s ) ((pos "Kj%" ) ))
( "racy"  j ( r ei1 s ii ) ((pos "OD%" ) ))
( "radar"  n ( r ei1 d aa r ) ((pos "L@%" ) ))
( "raddled"  j ( r a1 d l d ) ((pos "OA%" ) ))
( "radial"  n ( r ei1 d i@ l ) ((pos "K6%" "OA%" ) ))
( "radially"  a ( r ei1 d i@ l ii ) ((pos "Pu%" ) ))
( "radials"  n ( r ei1 d i@ l z ) ((pos "Kj%" ) ))
( "radiance"  n ( r ei1 d i@ n s ) ((pos "L@%" ) ))
( "radiant"  j ( r ei1 d i@ n t ) ((pos "OA%" ) ))
( "radiantly"  a ( r ei1 d i@ n t l ii ) ((pos "Pu%" ) ))
( "radiate"  v ( r ei1 d i ei t ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "radiated"  v ( r ei1 d i ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "radiates"  v ( r ei1 d i ei t s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "radiating"  v ( r ei1 d i ei t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "radiation"  n ( r ei2 d i ei1 sh @ n ) ((pos "M6%" ) ))
( "radiations"  n ( r ei2 d i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "radiator"  n ( r ei1 d i ei t @ r ) ((pos "K6%" ) ))
( "radiators"  n ( r ei1 d i ei t @ z ) ((pos "Kj%" ) ))
( "radical"  n ( r a1 d i k l ) ((pos "K6%" "OA%" ) ))
( "radicalism"  n ( r a1 d i k @ l i z @ m ) ((pos "L@%" ) ))
( "radically"  a ( r a1 d i k l ii ) ((pos "Pu%" ) ))
( "radicals"  n ( r a1 d i k l z ) ((pos "Kj%" ) ))
( "radicle"  n ( r a1 d i k l ) ((pos "K6$" ) ))
( "radicles"  n ( r a1 d i k l z ) ((pos "Kj$" ) ))
( "radii"  n ( r ei1 d i ai ) ((pos "Kj%" ) ))
( "radio"  n ( r ei1 d i@ @ ) ((pos "M6%" ) ))
( "radio-"  j ( r ei2 d i@ u - ) ((pos "U-%" ) ))
( "radio-gramophone"  n ( r ei2 d i@ u - g r a1 m @ f ou n ) ((pos "K6%" ) ))
( "radio-gramophones"  n ( r ei2 d i@ u - g r a1 m @ f ou n z ) ((pos "Kj%" ) ))
( "radio-location"  n ( r ei2 d i@ u - l @ k ei1 sh @ n ) ((pos "K6$" ) ))
( "radio-locations"  n ( r ei2 d i@ u - l @ k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "radio-set"  n ( r ei1 d i@ u - s e t ) ((pos "K6%" ) ))
( "radio-sets"  n ( r ei1 d i@ u - s e t s ) ((pos "Kj%" ) ))
( "radio-telescope"  n ( r ei2 d i@ u - t e1 l i s k ou p ) ((pos "K6%" ) ))
( "radio-telescopes"  n ( r ei2 d i@ u - t e1 l i s k ou p s ) ((pos "Kj%" ) ))
( "radioactive"  j ( r ei2 d i@ u a1 k t i v ) ((pos "OA%" ) ))
( "radioactivity"  n ( r ei2 d i@ u a k t i1 v i t ii ) ((pos "L@%" ) ))
( "radiogram"  n ( r ei1 d i@ u g r a m ) ((pos "K6%" ) ))
( "radiograms"  n ( r ei1 d i@ u g r a m z ) ((pos "Kj%" ) ))
( "radiograph"  n ( r ei1 d i@ u g r a f ) ((pos "K6%" ) ))
( "radiographer"  n ( r ei2 d i o1 g r @ f @ r ) ((pos "K6%" ) ))
( "radiographers"  n ( r ei2 d i o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "radiographs"  n ( r ei1 d i@ u g r a f s ) ((pos "Kj%" ) ))
( "radiography"  n ( r ei2 d i o1 g r @ f ii ) ((pos "L@%" ) ))
( "radioisotope"  n ( r ei2 d i@ u ai1 s @ t ou p ) ((pos "K6%" ) ))
( "radioisotopes"  n ( r ei2 d i@ u ai1 s @ t ou p s ) ((pos "Kj%" ) ))
( "radiologist"  n ( r ei2 d i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "radiologists"  n ( r ei2 d i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "radiology"  n ( r ei2 d i o1 l @ jh ii ) ((pos "L@%" ) ))
( "radios"  n ( r ei1 d i@ u z ) ((pos "Mj%" ) ))
( "radiotherapist"  n ( r ei2 d i@ u th e1 r @ p i s t ) ((pos "K6%" ) ))
( "radiotherapists"  n ( r ei2 d i@ u th e1 r @ p i s t s ) ((pos "Kj%" ) ))
( "radiotherapy"  n ( r ei2 d i@ u th e1 r @ p ii ) ((pos "L@%" ) ))
( "radish"  n ( r a1 d i sh ) ((pos "K7%" ) ))
( "radishes"  n ( r a1 d i sh i z ) ((pos "Kj%" ) ))
( "radium"  n ( r ei1 d i@ m ) ((pos "L@%" ) ))
( "radius"  n ( r ei1 d i@ s ) ((pos "Ki%" ) ))
( "raffia"  n ( r a1 f i@ ) ((pos "L@%" ) ))
( "raffish"  j ( r a1 f i sh ) ((pos "OA$" ) ))
( "raffishly"  a ( r a1 f i sh l ii ) ((pos "Pu$" ) ))
( "raffle"  v ( r a1 f l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "raffled"  v ( r a1 f l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "raffles"  v ( r a1 f l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "raffling"  v ( r a1 f l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "raft"  v ( r aa1 f t ) ((pos "J0$" "K6%" ) (subcat "2C" "6A" "15A" )))
( "rafted"  v ( r aa1 f t i d ) ((pos "Jc$" "Jd$" ) (subcat "2C" "6A" "15A" )))
( "rafter"  n ( r aa1 f t @ r ) ((pos "K6%" ) ))
( "raftered"  j ( r aa1 f t @ d ) ((pos "OA%" ) ))
( "rafters"  n ( r aa1 f t @ z ) ((pos "Kj%" ) ))
( "rafting"  v ( r aa1 f t i ng ) ((pos "Jb$" ) (subcat "2C" "6A" "15A" )))
( "rafts"  v ( r aa1 f t s ) ((pos "Ja$" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "raftsman"  n ( r aa1 f t s m @ n ) ((pos "Ki%" ) ))
( "raftsmen"  n ( r aa1 f t s m @ n ) ((pos "Kj%" ) ))
( "rag"  v ( r a1 g ) ((pos "H4%" "K6%" ) (subcat )))
( "rag-day"  n ( r a1 g - d ei ) ((pos "K6%" ) ))
( "rag-days"  n ( r a1 g - d ei z ) ((pos "Kj%" ) ))
( "ragamuffin"  n ( r a1 g @ m uh f i n ) ((pos "K6%" ) ))
( "ragamuffins"  n ( r a1 g @ m uh f i n z ) ((pos "Kj%" ) ))
( "ragbag"  n ( r a1 g b a g ) ((pos "K6%" ) ))
( "ragbags"  n ( r a1 g b a g z ) ((pos "Kj%" ) ))
( "rage"  v ( r ei1 jh ) ((pos "I2%" "M6%" ) (subcat "2A" )))
( "raged"  v ( r ei1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "rages"  v ( r ei1 jh i z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "ragged"  j ( r a1 g i d ) ((pos "OA%" ) ))
( "ragged"  v ( r a1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "raggedly"  a ( r a1 g i d l ii ) ((pos "Pu%" ) ))
( "raggedness"  n ( r a1 g i d n @ s ) ((pos "L@%" ) ))
( "ragging"  v ( r a1 g i ng ) ((pos "Hb%" ) (subcat )))
( "raging"  v ( r ei1 jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "raglan"  n ( r a1 g l @ n ) ((pos "K6$" ) ))
( "raglans"  n ( r a1 g l @ n z ) ((pos "Kj$" ) ))
( "ragout"  n ( r a1 g uu ) ((pos "M6%" ) ))
( "ragouts"  n ( r a1 g uu z ) ((pos "Mj$" ) ))
( "rags"  v ( r a1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ragtag"  n ( r a1 g t a g ) ((pos "K6%" ) ))
( "ragtags"  n ( r a1 g t a g z ) ((pos "Kj$" ) ))
( "ragtime"  n ( r a1 g t ai m ) ((pos "L@%" ) ))
( "rah"  uh ( r aa1 ) ((pos "W-$" ) ))
( "raid"  v ( r ei1 d ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "raided"  v ( r ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "raider"  n ( r ei1 d @ r ) ((pos "K6%" ) ))
( "raiders"  n ( r ei1 d @ z ) ((pos "Kj%" ) ))
( "raiding"  v ( r ei1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "raids"  v ( r ei1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "rail"  v ( r ei1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "railcar"  n ( r ei1 l k aa r ) ((pos "K6%" ) ))
( "railcars"  n ( r ei1 l k aa z ) ((pos "Kj%" ) ))
( "railed"  v ( r ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "railhead"  n ( r ei1 l h e d ) ((pos "K6%" ) ))
( "railheads"  n ( r ei1 l h e d z ) ((pos "Kj%" ) ))
( "railing"  v ( r ei1 l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "railings"  n ( r ei1 l i ng z ) ((pos "Mj%" ) ))
( "railleries"  n ( r ei1 l @ r i z ) ((pos "Mj$" ) ))
( "raillery"  n ( r ei1 l @ r ii ) ((pos "M8%" ) ))
( "railroad"  v ( r ei1 l r ou d ) ((pos "H0%" "K6%" ) (subcat "15A" )))
( "railroaded"  v ( r ei1 l r ou d i d ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "railroading"  v ( r ei1 l r ou d i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "railroads"  v ( r ei1 l r ou d z ) ((pos "Ha%" "Kj%" ) (subcat "15A" )))
( "rails"  v ( r ei1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "railway"  n ( r ei1 l w ei ) ((pos "K6%" ) ))
( "railwayman"  n ( r ei1 l w ei m @ n ) ((pos "Ki%" ) ))
( "railwaymen"  n ( r ei1 l w ei m e n ) ((pos "Kj%" ) ))
( "railways"  n ( r ei1 l w ei z ) ((pos "Kj%" ) ))
( "raiment"  n ( r ei1 m @ n t ) ((pos "L@$" ) ))
( "rain"  v ( r ei1 n ) ((pos "J0%" "M6%" ) (subcat "2C" )))
( "rain-gauge"  n ( r ei1 n - g ei jh ) ((pos "K6%" ) ))
( "rain-gauges"  n ( r ei1 n - g ei jh i z ) ((pos "Kj%" ) ))
( "rainbow"  n ( r ei1 n b ou ) ((pos "K6%" ) ))
( "rainbows"  n ( r ei1 n b ou z ) ((pos "Kj%" ) ))
( "raincoat"  n ( r ei1 n k ou t ) ((pos "K6%" ) ))
( "raincoats"  n ( r ei1 n k ou t s ) ((pos "Kj%" ) ))
( "raindrop"  n ( r ei1 n d r o p ) ((pos "K6%" ) ))
( "raindrops"  n ( r ei1 n d r o p s ) ((pos "Kj%" ) ))
( "rained"  v ( r ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "rainfall"  n ( r ei1 n f oo l ) ((pos "M6%" ) ))
( "rainfalls"  n ( r ei1 n f oo l z ) ((pos "Mj%" ) ))
( "rainier"  j ( r ei1 n i@ r ) ((pos "Or%" ) ))
( "rainiest"  j ( r ei1 n i i s t ) ((pos "Os%" ) ))
( "raining"  v ( r ei1 n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "rainproof"  j ( r ei1 n p r uu f ) ((pos "OA%" ) ))
( "rains"  v ( r ei1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "rainwater"  n ( r ei1 n w oo t @ r ) ((pos "L@%" ) ))
( "rainy"  j ( r ei1 n ii ) ((pos "OD%" ) ))
( "raise"  v ( r ei1 z ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "raised"  v ( r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "raiser"  n ( r ei1 z @ r ) ((pos "K6$" ) ))
( "raisers"  n ( r ei1 z @ z ) ((pos "Kj$" ) ))
( "raises"  v ( r ei1 z i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "raisin"  n ( r ei1 z n ) ((pos "K6%" ) ))
( "raising"  v ( r ei1 z i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "raisins"  n ( r ei1 z n z ) ((pos "Kj%" ) ))
( "raison d'^etre"  n ( r ei2 z o n - d e1 t r @ ) ((pos "Ki%" ) ))
( "raj"  n ( r aa1 jh ) ((pos "Ki$" ) ))
( "rajah"  n ( r aa1 jh @ ) ((pos "K6%" ) ))
( "rajahs"  n ( r aa1 jh @ z ) ((pos "Kj%" ) ))
( "rake"  v ( r ei1 k ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "rake-off"  n ( r ei1 k - o f ) ((pos "K6%" ) ))
( "rake-offs"  n ( r ei1 k - o f s ) ((pos "Kj$" ) ))
( "raked"  v ( r ei1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "rakes"  v ( r ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "raking"  v ( r ei1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "rakish"  j ( r ei1 k i sh ) ((pos "OA%" ) ))
( "rakishly"  a ( r ei1 k i sh l ii ) ((pos "Pu%" ) ))
( "rakishness"  n ( r ei1 k i sh n @ s ) ((pos "L@%" ) ))
( "rallentando"  j ( r a2 l @ n t a1 n d ou ) ((pos "OA$" "Pu$" ) ))
( "rallied"  v ( r a1 l i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "rallies"  v ( r a1 l i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "rally"  v ( r a1 l ii ) ((pos "J3%" "K8%" ) (subcat "2A" "2C" "6A" )))
( "rallying"  v ( r a1 l i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "ram"  v ( r a1 m ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" )))
( "ramble"  v ( r a1 m b l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "rambled"  v ( r a1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "rambler"  n ( r a1 m b l @ r ) ((pos "K6%" ) ))
( "ramblers"  n ( r a1 m b l @ z ) ((pos "Kj%" ) ))
( "rambles"  v ( r a1 m b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "rambling"  v ( r a1 m b l i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "rambunctious"  j ( r a1 m b uh1 ng k sh @ s ) ((pos "OA$" ) ))
( "ramification"  n ( r a2 m i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "ramifications"  n ( r a2 m i f i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "ramified"  v ( r a1 m i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ramifies"  v ( r a1 m i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "ramify"  v ( r a1 m i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "ramifying"  v ( r a1 m i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ramjet"  n ( r a1 m jh e t ) ((pos "K6$" ) ))
( "ramjets"  n ( r a1 m jh e t s ) ((pos "Kj$" ) ))
( "rammed"  v ( r a1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "ramming"  v ( r a1 m i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "ramp"  v ( r a1 m p ) ((pos "H0%" "K6%" ) (subcat )))
( "rampage"  n ( r a1 m p ei jh ) ((pos "K6%" ) ))
( "rampage"  v ( r a1 m p ei1 jh ) ((pos "I2%" ) (subcat )))
( "rampaged"  v ( r a1 m p ei1 jh d ) ((pos "Ic%" "Id%" ) (subcat )))
( "rampageous"  j ( r a1 m p ei1 jh @ s ) ((pos "OA$" ) ))
( "rampages"  n ( r a1 m p ei jh i z ) ((pos "Kj$" ) ))
( "rampages"  v ( r a1 m p ei1 jh i z ) ((pos "Ia%" ) (subcat )))
( "rampaging"  v ( r a1 m p ei1 jh i ng ) ((pos "Ib%" ) (subcat )))
( "rampant"  j ( r a1 m p @ n t ) ((pos "OA%" ) ))
( "rampantly"  a ( r a1 m p @ n t l ii ) ((pos "Pu%" ) ))
( "rampart"  n ( r a1 m p aa t ) ((pos "K6%" ) ))
( "ramparts"  n ( r a1 m p aa t s ) ((pos "Kj%" ) ))
( "ramped"  v ( r a1 m p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ramping"  v ( r a1 m p i ng ) ((pos "Hb%" ) (subcat )))
( "ramps"  v ( r a1 m p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ramrod"  n ( r a1 m r o d ) ((pos "K6%" ) ))
( "ramrods"  n ( r a1 m r o d z ) ((pos "Kj%" ) ))
( "rams"  v ( r a1 m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "ramshackle"  j ( r a1 m sh a k l ) ((pos "OA%" ) ))
( "ran"  v ( r a1 n ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" "14" "15A" )))
( "ranch"  n ( r aa1 n ch ) ((pos "K7%" ) ))
( "rancher"  n ( r aa1 n ch @ r ) ((pos "K6%" ) ))
( "ranchers"  n ( r aa1 n ch @ z ) ((pos "Kj%" ) ))
( "ranches"  n ( r aa1 n ch i z ) ((pos "Kj%" ) ))
( "rancid"  j ( r a1 n s i d ) ((pos "OA%" ) ))
( "rancorous"  j ( r a1 ng k @ r @ s ) ((pos "OA%" ) ))
( "rancour"  n ( r a1 ng k @ r ) ((pos "L@%" ) ))
( "rand"  n ( r a1 n d ) ((pos "K6$" ) ))
( "randier"  j ( r a1 n d i@ r ) ((pos "Or%" ) ))
( "randiest"  j ( r a1 n d i i s t ) ((pos "Os%" ) ))
( "random"  n ( r a1 n d @ m ) ((pos "Ki%" ) ))
( "randomly"  a ( r a1 n d @ m l ii ) ((pos "Pu%" ) ))
( "randomness"  n ( r a1 n d @ m n i s ) ((pos "L@%" ) ))
( "rands"  n ( r a1 n d z ) ((pos "Kj$" ) ))
( "randy"  j ( r a1 n d ii ) ((pos "OD%" ) ))
( "ranee"  n ( r aa1 n ii ) ((pos "K6$" ) ))
( "ranees"  n ( r aa1 n ii z ) ((pos "Kj$" ) ))
( "rang"  v ( r a1 ng ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "range"  v ( r ei1 n jh ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "ranged"  v ( r ei1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "rangefinder"  n ( r ei1 n jh f ai n d @ r ) ((pos "K6%" ) ))
( "rangefinders"  n ( r ei1 n jh f ai n d @ z ) ((pos "Kj%" ) ))
( "ranger"  n ( r ei1 n jh @ r ) ((pos "K6%" ) ))
( "rangers"  n ( r ei1 n jh @ z ) ((pos "Kj%" ) ))
( "ranges"  v ( r ei1 n jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "ranging"  v ( r ei1 n jh i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "rani"  n ( r aa1 n ii ) ((pos "K6$" ) ))
( "ranis"  n ( r aa1 n ii z ) ((pos "Kj$" ) ))
( "rank"  v ( r a1 ng k ) ((pos "J0%" "M6%" "OA%" ) (subcat "3A" "6A" "15A" )))
( "ranked"  v ( r a1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "15A" )))
( "ranker"  n ( r a1 ng k @ r ) ((pos "K6$" ) ))
( "rankers"  n ( r a1 ng k @ z ) ((pos "Kj$" ) ))
( "ranking"  v ( r a1 ng k i ng ) ((pos "Jb%" "K6$" ) (subcat "3A" "6A" "15A" )))
( "rankings"  n ( r a1 ng k i ng z ) ((pos "Kj%" ) ))
( "rankle"  v ( r a1 ng k l ) ((pos "I2%" ) (subcat )))
( "rankled"  v ( r a1 ng k l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "rankles"  v ( r a1 ng k l z ) ((pos "Ia%" ) (subcat )))
( "rankling"  v ( r a1 ng k l i ng ) ((pos "Ib%" ) (subcat )))
( "rankly"  a ( r a1 ng k l ii ) ((pos "Pu%" ) ))
( "rankness"  n ( r a1 ng k n @ s ) ((pos "L@%" ) ))
( "ranks"  v ( r a1 ng k s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "15A" )))
( "ransack"  v ( r a1 n s a k ) ((pos "H0%" ) (subcat "6A" "14" )))
( "ransacked"  v ( r a1 n s a k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "ransacking"  v ( r a1 n s a k i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "ransacks"  v ( r a1 n s a k s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "ransom"  v ( r a1 n s @ m ) ((pos "H0%" "M6%" ) (subcat )))
( "ransomed"  v ( r a1 n s @ m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ransoming"  v ( r a1 n s @ m i ng ) ((pos "Hb%" ) (subcat )))
( "ransoms"  v ( r a1 n s @ m z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "rant"  v ( r a1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "ranted"  v ( r a1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ranter"  n ( r a1 n t @ r ) ((pos "K6%" ) ))
( "ranters"  n ( r a1 n t @ z ) ((pos "Kj%" ) ))
( "ranting"  v ( r a1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rants"  v ( r a1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "rap"  v ( r a1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "rapacious"  j ( r @1 p ei1 sh @ s ) ((pos "OA%" ) ))
( "rapaciously"  a ( r @1 p ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "rapacity"  n ( r @1 p a1 s i t ii ) ((pos "L@%" ) ))
( "rape"  v ( r ei1 p ) ((pos "H2%" "M6%" ) (subcat )))
( "raped"  v ( r ei1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rapes"  v ( r ei1 p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "rapid"  n ( r a1 p i d ) ((pos "K6%" "OA%" ) ))
( "rapidity"  n ( r @1 p i1 d i t ii ) ((pos "L@%" ) ))
( "rapidly"  a ( r a1 p i d l ii ) ((pos "Pu%" ) ))
( "rapids"  n ( r a1 p i d z ) ((pos "Kj%" ) ))
( "rapier"  n ( r ei1 p i@ r ) ((pos "K6%" ) ))
( "rapier-thrust"  n ( r ei1 p i@ - th r uh s t ) ((pos "K6%" ) ))
( "rapier-thrusts"  n ( r ei1 p i@ - th r uh s t s ) ((pos "Kj%" ) ))
( "rapiers"  n ( r ei1 p i@ z ) ((pos "Kj%" ) ))
( "rapine"  n ( r a1 p ai n ) ((pos "L@$" ) ))
( "raping"  v ( r ei1 p i ng ) ((pos "Hb%" ) (subcat )))
( "rapist"  n ( r ei1 p i s t ) ((pos "K6%" ) ))
( "rapists"  n ( r ei1 p i s t s ) ((pos "Kj%" ) ))
( "rapped"  v ( r a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "rapping"  v ( r a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "rapport"  n ( r a1 p oo1 r ) ((pos "M6%" ) ))
( "rapports"  n ( r a1 p oo1 z ) ((pos "Mj$" ) ))
( "rapprochement"  n ( r a1 p r o1 sh m o ng ) ((pos "K6%" ) ))
( "rapprochements"  n ( r a1 p r o1 sh m o ng z ) ((pos "Kj$" ) ))
( "raps"  v ( r a1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "rapscallion"  n ( r a1 p s k a1 l i@ n ) ((pos "K6%" ) ))
( "rapscallions"  n ( r a1 p s k a1 l i@ n z ) ((pos "Kj%" ) ))
( "rapt"  j ( r a1 p t ) ((pos "OA%" ) ))
( "rapture"  n ( r a1 p ch @ r ) ((pos "M6%" ) ))
( "raptures"  n ( r a1 p ch @ z ) ((pos "Mj%" ) ))
( "rapturous"  j ( r a1 p ch @ r @ s ) ((pos "OA%" ) ))
( "rapturously"  a ( r a1 p ch @ r @ s l ii ) ((pos "Pu%" ) ))
( "rare"  j ( r e@1 r ) ((pos "OB%" ) ))
( "rarebit"  n ( r e@1 b i t ) ((pos "M6%" ) ))
( "rarebits"  n ( r e@1 b i t s ) ((pos "Mj$" ) ))
( "rarefaction"  n ( r e@2 r i f a1 k sh @ n ) ((pos "L@$" ) ))
( "rarefied"  v ( r e@1 r i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rarefies"  v ( r e@1 r i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "rarefy"  v ( r e@1 r i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "rarefying"  v ( r e@1 r i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rarely"  a ( r e@1 l ii ) ((pos "Pu%" ) ))
( "rareness"  n ( r e@1 n @ s ) ((pos "L@%" ) ))
( "rarer"  j ( r e@1 r @ r ) ((pos "Or%" ) ))
( "rarest"  j ( r e@1 r i s t ) ((pos "Os%" ) ))
( "raring"  j ( r e@1 r i ng ) ((pos "OA%" ) ))
( "rarities"  n ( r e@1 r i t i z ) ((pos "Mj%" ) ))
( "rarity"  n ( r e@1 r i t ii ) ((pos "M8%" ) ))
( "rascal"  n ( r aa1 s k l ) ((pos "K6%" ) ))
( "rascally"  j ( r aa1 s k @ l ii ) ((pos "OA%" ) ))
( "rascals"  n ( r aa1 s k l z ) ((pos "Kj%" ) ))
( "rase"  v ( r ei1 z ) ((pos "H2$" ) (subcat )))
( "rased"  v ( r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rases"  v ( r ei1 z i z ) ((pos "Ha$" ) (subcat )))
( "rash"  n ( r a1 sh ) ((pos "K7%" "OC%" ) ))
( "rasher"  n ( r a1 sh @ r ) ((pos "K6%" "Or%" ) ))
( "rashers"  n ( r a1 sh @ z ) ((pos "Kj%" ) ))
( "rashes"  n ( r a1 sh i z ) ((pos "Kj%" ) ))
( "rashest"  j ( r a1 sh i s t ) ((pos "Os%" ) ))
( "rashly"  a ( r a1 sh l ii ) ((pos "Pu%" ) ))
( "rashness"  n ( r a1 sh n @ s ) ((pos "L@%" ) ))
( "rasing"  v ( r ei1 z i ng ) ((pos "Hb$" ) (subcat )))
( "rasp"  v ( r aa1 s p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "raspberries"  n ( r aa1 z b @ r i z ) ((pos "Kj%" ) ))
( "raspberry"  n ( r aa1 z b @ r ii ) ((pos "K8%" ) ))
( "rasped"  v ( r aa1 s p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rasping"  v ( r aa1 s p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "raspingly"  a ( r aa1 s p i ng l ii ) ((pos "Pu%" ) ))
( "rasps"  v ( r aa1 s p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rat"  v ( r a1 t ) ((pos "H4%" "K6%" ) (subcat "2A" )))
( "rat-a-tat-tat"  n ( r a2 t - @ - t a2 t - t a1 t ) ((pos "K6%" ) ))
( "rat-a-tat-tats"  n ( r a2 t - @ - t a2 t - t a1 t s ) ((pos "Kj%" ) ))
( "rat-tat"  n ( r a1 t - t a1 t ) ((pos "K6$" ) ))
( "rat-tats"  n ( r a1 t - t a1 t s ) ((pos "Kj$" ) ))
( "ratability"  n ( r ei2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "ratable"  j ( r ei1 t @ b l ) ((pos "OA%" ) ))
( "ratan"  n ( r a1 t a1 n ) ((pos "M6$" ) ))
( "ratans"  n ( r a1 t a1 n z ) ((pos "Mj$" ) ))
( "ratch"  n ( r a1 ch ) ((pos "K7$" ) ))
( "ratches"  n ( r a1 ch i z ) ((pos "Kj$" ) ))
( "ratchet"  n ( r a1 ch i t ) ((pos "K6%" ) ))
( "ratchets"  n ( r a1 ch i t s ) ((pos "Kj%" ) ))
( "rate"  v ( r ei1 t ) ((pos "J2%" "K6%" ) (subcat "2C" "2D" "6A" "14" "15A" )))
( "rateability"  n ( r ei2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "rateable"  j ( r ei1 t @ b l ) ((pos "OA%" ) ))
( "rated"  v ( r ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "2D" "6A" "14" "15A" )))
( "ratepayer"  n ( r ei1 t p ei @ r ) ((pos "K6%" ) ))
( "ratepayers"  n ( r ei1 t p ei @ z ) ((pos "Kj%" ) ))
( "rates"  v ( r ei1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "2D" "6A" "14" "15A" )))
( "rather"  a ( r aa1 dh @ r ) ((pos "Pu*" ) ))
( "ratification"  n ( r a2 t i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "ratifications"  n ( r a2 t i f i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "ratified"  v ( r a1 t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ratifies"  v ( r a1 t i f ai z ) ((pos "Ha%" ) (subcat )))
( "ratify"  v ( r a1 t i f ai ) ((pos "H3%" ) (subcat )))
( "ratifying"  v ( r a1 t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "rating"  v ( r ei1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "2D" "6A" "14" "15A" )))
( "ratings"  n ( r ei1 t i ng z ) ((pos "Kj%" ) ))
( "ratio"  n ( r ei1 sh i@ @ ) ((pos "K6%" ) ))
( "ratiocination"  n ( r a2 t i o2 s i n ei1 sh @ n ) ((pos "L@$" ) ))
( "ration"  v ( r a1 sh @ n ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "rational"  j ( r a1 sh @ n @ l ) ((pos "OA%" ) ))
( "rationale"  n ( r a2 sh @ n aa1 l ) ((pos "K6%" ) ))
( "rationales"  n ( r a2 sh @ n aa1 l z ) ((pos "Kj$" ) ))
( "rationalism"  n ( r a1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "rationalist"  n ( r a1 sh @ n @ l i s t ) ((pos "K6%" ) ))
( "rationalistic"  j ( r a2 sh @ n @ l i1 s t i k ) ((pos "OA%" ) ))
( "rationalists"  n ( r a1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "rationalities"  n ( r a2 sh @ n a1 l i t i z ) ((pos "Mj%" ) ))
( "rationality"  n ( r a2 sh @ n a1 l i t ii ) ((pos "M8%" ) ))
( "rationalization"  n ( r a2 sh @ n @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "rationalizations"  n ( r a2 sh @ n @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "rationalize"  v ( r a1 sh @ n @ l ai z ) ((pos "H2%" ) (subcat )))
( "rationalized"  v ( r a1 sh @ n @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rationalizes"  v ( r a1 sh @ n @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "rationalizing"  v ( r a1 sh @ n @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "rationally"  a ( r a1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "rationed"  v ( r a1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "rationing"  v ( r a1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "rations"  v ( r a1 sh @ n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "ratios"  n ( r ei1 sh i@ u z ) ((pos "Kj%" ) ))
( "ratlin"  n ( r a1 t l i n ) ((pos "K6$" ) ))
( "ratline"  n ( r a1 t l i n ) ((pos "K6$" ) ))
( "ratlines"  n ( r a1 t l i n z ) ((pos "Kj$" ) ))
( "ratlins"  n ( r a1 t l i n z ) ((pos "Kj$" ) ))
( "rats"  v ( r a1 t s ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "rattan"  n ( r a1 t a1 n ) ((pos "M6%" ) ))
( "rattans"  n ( r a1 t a1 n z ) ((pos "Mj$" ) ))
( "ratted"  v ( r a1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "ratter"  n ( r a1 t @ r ) ((pos "K6%" ) ))
( "ratters"  n ( r a1 t @ z ) ((pos "Kj%" ) ))
( "rattier"  j ( r a1 t i@ r ) ((pos "Or%" ) ))
( "rattiest"  j ( r a1 t i i s t ) ((pos "Os%" ) ))
( "ratting"  v ( r a1 t i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "rattle"  v ( r a1 t l ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "rattlebrained"  j ( r a1 t l b r ei n d ) ((pos "OA$" ) ))
( "rattled"  v ( r a1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "rattlepated"  j ( r a1 t l p ei t i d ) ((pos "OA$" ) ))
( "rattler"  n ( r a1 t l @ r ) ((pos "K6%" ) ))
( "rattlers"  n ( r a1 t l @ z ) ((pos "Kj%" ) ))
( "rattles"  v ( r a1 t l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "rattlesnake"  n ( r a1 t l s n ei k ) ((pos "K6%" ) ))
( "rattlesnakes"  n ( r a1 t l s n ei k s ) ((pos "Kj%" ) ))
( "rattling"  v ( r a1 t l i ng ) ((pos "Jb%" "OA%" "Pu%" ) (subcat "2A" "2C" "6A" "15A" )))
( "ratty"  j ( r a1 t ii ) ((pos "OD%" ) ))
( "raucous"  j ( r oo1 k @ s ) ((pos "OA%" ) ))
( "raucously"  a ( r oo1 k @ s l ii ) ((pos "Pu%" ) ))
( "ravage"  v ( r a1 v i jh ) ((pos "J2%" "M6%" ) (subcat )))
( "ravaged"  v ( r a1 v i jh d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "ravages"  v ( r a1 v i jh i z ) ((pos "Ja%" "Mj%" ) (subcat )))
( "ravaging"  v ( r a1 v i jh i ng ) ((pos "Jb%" ) (subcat )))
( "rave"  v ( r ei1 v ) ((pos "I2%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "rave-up"  n ( r ei1 v - uh p ) ((pos "K6%" ) ))
( "rave-ups"  n ( r ei1 v - uh p s ) ((pos "Kj$" ) ))
( "raved"  v ( r ei1 v d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "3A" )))
( "ravel"  v ( r a1 v @ l ) ((pos "J4%" ) (subcat "2A" "2C" "6A" )))
( "ravelled"  v ( r a1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "ravelling"  v ( r a1 v @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "ravels"  v ( r a1 v @ l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "raven"  n ( r ei1 v n ) ((pos "K6%" ) ))
( "ravening"  j ( r a1 v @ n i ng ) ((pos "OA%" ) ))
( "ravenous"  j ( r a1 v @ n @ s ) ((pos "OA%" ) ))
( "ravenously"  a ( r a1 v @ n @ s l ii ) ((pos "Pu%" ) ))
( "ravens"  n ( r ei1 v n z ) ((pos "Kj%" ) ))
( "raver"  n ( r ei1 v @ r ) ((pos "K6%" ) ))
( "ravers"  n ( r ei1 v @ z ) ((pos "Kj%" ) ))
( "raves"  v ( r ei1 v z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "ravine"  n ( r @1 v ii1 n ) ((pos "K6%" ) ))
( "ravines"  n ( r @1 v ii1 n z ) ((pos "Kj%" ) ))
( "raving"  v ( r ei1 v i ng ) ((pos "Ib%" "OA%" "Pu%" ) (subcat "2A" "2C" "3A" )))
( "ravings"  n ( r ei1 v i ng z ) ((pos "Kj%" ) ))
( "ravioli"  n ( r a2 v i ou1 l ii ) ((pos "M6%" ) ))
( "raviolis"  n ( r a2 v i ou1 l i z ) ((pos "Mj%" ) ))
( "ravish"  v ( r a1 v i sh ) ((pos "H1%" ) (subcat )))
( "ravished"  v ( r a1 v i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ravishes"  v ( r a1 v i sh i z ) ((pos "Ha%" ) (subcat )))
( "ravishing"  v ( r a1 v i sh i ng ) ((pos "Hb%" ) (subcat )))
( "ravishingly"  a ( r a1 v i sh i ng l ii ) ((pos "Pu%" ) ))
( "ravishment"  n ( r a1 v i sh m @ n t ) ((pos "M6$" ) ))
( "ravishments"  n ( r a1 v i sh m @ n t s ) ((pos "Mj$" ) ))
( "raw"  n ( r oo1 ) ((pos "Ki%" "OA%" ) ))
( "rawboned"  j ( r oo1 b ou n d ) ((pos "OA%" ) ))
( "rawhide"  j ( r oo1 h ai d ) ((pos "OA%" ) ))
( "ray"  v ( r ei1 ) ((pos "J0$" "K6%" ) ))
( "rayed"  v ( r ei1 d ) ((pos "Jc$" "Jd$" ) ))
( "raying"  v ( r ei1 i ng ) ((pos "Jb$" ) ))
( "rayon"  n ( r ei1 o n ) ((pos "L@%" ) ))
( "rays"  v ( r ei1 z ) ((pos "Ja$" "Kj%" ) ))
( "raze"  v ( r ei1 z ) ((pos "H2%" ) (subcat )))
( "razed"  v ( r ei1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "razes"  v ( r ei1 z i z ) ((pos "Ha%" ) (subcat )))
( "razing"  v ( r ei1 z i ng ) ((pos "Hb%" ) (subcat )))
( "razor"  v ( r ei1 z @ r ) ((pos "H0%" "K6%" ) ))
( "razor-edge"  n ( r ei1 z @ r - e1 jh ) ((pos "K6%" ) ))
( "razor-edges"  n ( r ei1 z @ r - e1 jh i z ) ((pos "Kj%" ) ))
( "razorback"  n ( r ei1 z @ b a k ) ((pos "K6$" ) ))
( "razorbacked"  j ( r ei1 z @ b a k t ) ((pos "OA$" ) ))
( "razorbacks"  n ( r ei1 z @ b a k s ) ((pos "Kj$" ) ))
( "razorblade"  n ( r ei1 z @ b l ei d ) ((pos "K6%" ) ))
( "razorblades"  n ( r ei1 z @ b l ei d z ) ((pos "Kj%" ) ))
( "razored"  v ( r ei1 z @ d ) ((pos "Hc%" "Hd%" ) ))
( "razoring"  v ( r ei1 z @ r i ng ) ((pos "Hb%" ) ))
( "razors"  v ( r ei1 z @ z ) ((pos "Ha%" "Kj%" ) ))
( "razzle"  n ( r a1 z l ) ((pos "Ki%" ) ))
( "razzle-dazzle"  n ( r a2 z l - d a1 z l ) ((pos "Ki%" ) ))
( "re"  n ( r ei1 ) ((pos "Ki$" ) ))
( "re"  in ( r ii ) ((pos "T-$" ) ))
( "re-"  j ( r ii - ) ((pos "U-%" ) ))
( "re-address"  v ( r ii2 - @ d r e1 s ) ((pos "H1%" ) (subcat )))
( "re-addressed"  v ( r ii2 - @ d r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "re-addresses"  v ( r ii2 - @ d r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "re-addressing"  v ( r ii2 - @ d r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "re-afforest"  v ( r ii2 - @ f o1 r i s t ) ((pos "H0$" ) ))
( "re-afforestation"  n ( r ii2 - @ f o2 r i s t ei1 sh @ n ) ((pos "M6$" ) ))
( "re-afforestations"  n ( r ii2 - @ f o2 r i s t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "re-afforested"  v ( r ii2 - @ f o1 r i s t i d ) ((pos "Hc$" "Hd$" ) ))
( "re-afforesting"  v ( r ii2 - @ f o1 r i s t i ng ) ((pos "Hb$" ) ))
( "re-afforests"  v ( r ii2 - @ f o1 r i s t s ) ((pos "Ha$" ) ))
( "re-count"  n ( r ii1 - k au n t ) ((pos "K6%" ) ))
( "re-count"  v ( r ii2 - k au1 n t ) ((pos "H0$" ) (subcat )))
( "re-counted"  v ( r ii2 - k au1 n t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "re-counting"  v ( r ii2 - k au1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "re-counts"  n ( r ii1 - k au n t s ) ((pos "Kj%" ) ))
( "re-counts"  v ( r ii2 - k au1 n t s ) ((pos "Ha$" ) (subcat )))
( "re-cover"  v ( r ii2 - k uh1 v @ r ) ((pos "H0$" ) (subcat )))
( "re-covered"  v ( r ii2 - k uh1 v @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "re-covering"  v ( r ii2 - k uh1 v @ r i ng ) ((pos "Hb$" ) (subcat )))
( "re-covers"  v ( r ii2 - k uh1 v @ z ) ((pos "Ha$" ) (subcat )))
( "re-echo"  v ( r ii1 - e1 k ou ) ((pos "I0%" "K7%" ) (subcat )))
( "re-echoed"  v ( r ii1 - e1 k ou d ) ((pos "Ic%" "Id%" ) (subcat )))
( "re-echoes"  n ( r ii1 - e1 k ou z ) ((pos "Kj%" ) ))
( "re-echoing"  v ( r ii1 - e1 k ou i ng ) ((pos "Ib%" ) (subcat )))
( "re-echos"  v ( r ii1 - e1 k ou z ) ((pos "Ia%" ) (subcat )))
( "re-entries"  n ( r ii2 - e1 n t r i z ) ((pos "Kj%" ) ))
( "re-entry"  n ( r ii2 - e1 n t r ii ) ((pos "K8%" ) ))
( "re-form"  v ( r ii2 - f oo1 m ) ((pos "J0$" ) ))
( "re-formation"  n ( r ii2 - f oo m ei1 sh @ n ) ((pos "M6$" ) ))
( "re-formations"  n ( r ii2 - f oo m ei1 sh @ n z ) ((pos "Mj$" ) ))
( "re-formed"  v ( r ii2 - f oo1 m d ) ((pos "Jc$" "Jd$" ) ))
( "re-forming"  v ( r ii2 - f oo1 m i ng ) ((pos "Jb$" ) ))
( "re-forms"  v ( r ii2 - f oo1 m z ) ((pos "Ja$" ) ))
( "re-join"  v ( r ii2 - jh oi1 n ) ((pos "H0$" ) (subcat )))
( "re-joined"  v ( r ii2 - jh oi1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "re-joining"  v ( r ii2 - jh oi1 n i ng ) ((pos "Hb$" ) (subcat )))
( "re-joins"  v ( r ii2 - jh oi1 n z ) ((pos "Ha$" ) (subcat )))
( "reach"  v ( r ii1 ch ) ((pos "J1%" "M7%" ) (subcat "2C" "3A" "6A" "12B" "13B" )))
( "reach-me-downs"  n ( r ii1 ch - m i - d au n z ) ((pos "Kj%" ) ))
( "reached"  v ( r ii1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "12B" "13B" )))
( "reaches"  v ( r ii1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" "12B" "13B" )))
( "reaching"  v ( r ii1 ch i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "12B" "13B" )))
( "react"  v ( r i1 a1 k t ) ((pos "I0%" ) (subcat "2A" )))
( "reacted"  v ( r i1 a1 k t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "reacting"  v ( r i1 a1 k t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "reaction"  n ( r i1 a1 k sh @ n ) ((pos "M6%" ) ))
( "reactionaries"  n ( r i1 a1 k sh @ n r i z ) ((pos "Kj%" ) ))
( "reactionary"  n ( r i1 a1 k sh @ n r ii ) ((pos "K8%" ) ))
( "reactions"  n ( r i1 a1 k sh @ n z ) ((pos "Mj%" ) ))
( "reactor"  n ( r i1 a1 k t @ r ) ((pos "K6%" ) ))
( "reactors"  n ( r i1 a1 k t @ z ) ((pos "Kj%" ) ))
( "reacts"  v ( r i1 a1 k t s ) ((pos "Ia%" ) (subcat "2A" )))
( "read"  v ( r e1 d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "read"  v ( r ii1 d ) ((pos "J5*" "Ki%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "readability"  n ( r ii2 d @ b i1 l i t ii ) ((pos "L@%" ) ))
( "readable"  j ( r ii1 d @ b l ) ((pos "OA%" ) ))
( "reader"  n ( r ii1 d @ r ) ((pos "K6%" ) ))
( "readers"  n ( r ii1 d @ z ) ((pos "Kj%" ) ))
( "readership"  n ( r ii1 d @ sh i p ) ((pos "K6%" ) ))
( "readerships"  n ( r ii1 d @ sh i p s ) ((pos "Kj%" ) ))
( "readier"  j ( r e1 d i@ r ) ((pos "Or%" ) ))
( "readies"  n ( r e1 d i z ) ((pos "Kj%" ) ))
( "readiest"  j ( r e1 d i i s t ) ((pos "Os%" ) ))
( "readily"  a ( r e1 d i l ii ) ((pos "Pu%" ) ))
( "readiness"  n ( r e1 d i n @ s ) ((pos "L@%" ) ))
( "reading"  v ( r ii1 d i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "reading-glasses"  n ( r ii1 d i ng - g l a s i z ) ((pos "Kj%" ) ))
( "reading-lamp"  n ( r ii1 d i ng - l a m p ) ((pos "K6%" ) ))
( "reading-lamps"  n ( r ii1 d i ng - l a m p s ) ((pos "Kj%" ) ))
( "reading-room"  n ( r ii1 d i ng - r uu m ) ((pos "K6%" ) ))
( "reading-rooms"  n ( r ii1 d i ng - r uu m z ) ((pos "Kj%" ) ))
( "readings"  n ( r ii1 d i ng z ) ((pos "Mj%" ) ))
( "readjust"  v ( r ii2 @ jh uh1 s t ) ((pos "H0%" ) (subcat "3A" "6A" )))
( "readjusted"  v ( r ii2 @ jh uh1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" )))
( "readjusting"  v ( r ii2 @ jh uh1 s t i ng ) ((pos "Hb%" ) (subcat "3A" "6A" )))
( "readjustment"  n ( r ii2 @ jh uh1 s t m @ n t ) ((pos "K6%" ) ))
( "readjustments"  n ( r ii2 @ jh uh1 s t m @ n t s ) ((pos "Kj%" ) ))
( "readjusts"  v ( r ii2 @ jh uh1 s t s ) ((pos "Ha%" ) (subcat "3A" "6A" )))
( "readmission"  n ( r ii2 @ d m i1 sh @ n ) ((pos "M6%" ) ))
( "readmissions"  n ( r ii2 @ d m i1 sh @ n z ) ((pos "Mj%" ) ))
( "readmit"  v ( r ii2 @ d m i1 t ) ((pos "H4%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "readmits"  v ( r ii2 @ d m i1 t s ) ((pos "Ha%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "readmitted"  v ( r ii2 @ d m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "readmitting"  v ( r ii2 @ d m i1 t i ng ) ((pos "Hb%" ) (subcat "3A" "6A" "6C" "9" "14" )))
( "reads"  v ( r ii1 d z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "ready"  n ( r e1 d ii ) ((pos "K8%" "OD%" ) ))
( "ready-made"  j ( r e1 d i - m ei d ) ((pos "OA%" ) ))
( "reaffiliation"  n ( r ii2 @ f i2 l i ei1 sh @ n ) ((pos "M6%" ) ))
( "reaffiliations"  n ( r ii2 @ f i2 l i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reaffirm"  v ( r ii2 @ f @@1 m ) ((pos "H0%" ) (subcat "6A" )))
( "reaffirmed"  v ( r ii2 @ f @@1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reaffirming"  v ( r ii2 @ f @@1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reaffirms"  v ( r ii2 @ f @@1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "reagent"  n ( r ii1 ei1 jh @ n t ) ((pos "K6$" ) ))
( "reagents"  n ( r ii1 ei1 jh @ n t s ) ((pos "Kj$" ) ))
( "real"  j ( r i@1 l ) ((pos "OA*" ) ))
( "real"  n ( r ei1 aa1 l ) ((pos "K6$" ) ))
( "realism"  n ( r i@1 l i z @ m ) ((pos "L@%" ) ))
( "realist"  n ( r i@1 l i s t ) ((pos "K6%" ) ))
( "realistic"  j ( r i@2 l i1 s t i k ) ((pos "OA%" ) ))
( "realistically"  a ( r i@2 l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "realists"  n ( r i@1 l i s t s ) ((pos "Kj%" ) ))
( "realities"  n ( r i1 a1 l i t i z ) ((pos "Mj%" ) ))
( "reality"  n ( r i1 a1 l i t ii ) ((pos "M8%" ) ))
( "realizable"  j ( r i@2 l ai1 z @ b l ) ((pos "OA%" ) ))
( "realization"  n ( r i@2 l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "realize"  v ( r i@1 l ai z ) ((pos "H2%" ) (subcat "6A" "9" "10" )))
( "realized"  v ( r i@1 l ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "10" )))
( "realizes"  v ( r i@1 l ai z i z ) ((pos "Ha%" ) (subcat "6A" "9" "10" )))
( "realizing"  v ( r i@1 l ai z i ng ) ((pos "Hb%" ) (subcat "6A" "9" "10" )))
( "reallocation"  n ( r ii1 a2 l @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "reallocations"  n ( r ii1 a2 l @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "really"  a ( r i@1 l ii ) ((pos "Pu*" ) ))
( "realm"  n ( r e1 l m ) ((pos "K6%" ) ))
( "realms"  n ( r e1 l m z ) ((pos "Kj%" ) ))
( "reals"  n ( r ei1 aa1 l z ) ((pos "Kj$" ) ))
( "realties"  n ( r i@1 l t i z ) ((pos "Kj$" ) ))
( "realty"  n ( r i@1 l t ii ) ((pos "K8$" ) ))
( "ream"  n ( r ii1 m ) ((pos "K6%" ) ))
( "reams"  n ( r ii1 m z ) ((pos "Kj%" ) ))
( "reanimate"  v ( r ii1 a1 n i m ei t ) ((pos "H2%" ) (subcat )))
( "reanimated"  v ( r ii1 a1 n i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reanimates"  v ( r ii1 a1 n i m ei t s ) ((pos "Ha%" ) (subcat )))
( "reanimating"  v ( r ii1 a1 n i m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "reap"  v ( r ii1 p ) ((pos "J0%" ) (subcat "2A" )))
( "reaped"  v ( r ii1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reaper"  n ( r ii1 p @ r ) ((pos "K6%" ) ))
( "reapers"  n ( r ii1 p @ z ) ((pos "Kj%" ) ))
( "reaping"  v ( r ii1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reaping-hook"  n ( r ii1 p i ng - h u k ) ((pos "K6%" ) ))
( "reaping-hooks"  n ( r ii1 p i ng - h u k s ) ((pos "Kj%" ) ))
( "reappear"  v ( r ii2 @ p i@1 r ) ((pos "I0%" ) (subcat )))
( "reappearance"  n ( r i@2 p i@1 r @ n s ) ((pos "K6%" ) ))
( "reappearances"  n ( r i@2 p i@1 r @ n s i z ) ((pos "Kj%" ) ))
( "reappeared"  v ( r ii2 @ p i@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "reappearing"  v ( r ii2 @ p i@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "reappears"  v ( r ii2 @ p i@1 z ) ((pos "Ia%" ) (subcat )))
( "reappraisal"  n ( r ii2 @ p r ei1 z l ) ((pos "K6%" ) ))
( "reappraisals"  n ( r ii2 @ p r ei1 z l z ) ((pos "Kj%" ) ))
( "reaps"  v ( r ii1 p s ) ((pos "Ja%" ) (subcat "2A" )))
( "rear"  v ( r i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "rear-admiral"  n ( r i@2 r - a1 d m @ r @ l ) ((pos "K6%" ) ))
( "rear-admirals"  n ( r i@2 r - a1 d m @ r @ l z ) ((pos "Kj%" ) ))
( "reared"  v ( r i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "rearguard"  n ( r i@1 g aa d ) ((pos "K6%" ) ))
( "rearguards"  n ( r i@1 g aa d z ) ((pos "Kj%" ) ))
( "rearing"  v ( r i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "rearm"  v ( r ii2 aa1 m ) ((pos "J0%" ) (subcat "2A" )))
( "rearmament"  n ( r ii1 aa1 m @ m @ n t ) ((pos "M6%" ) ))
( "rearmaments"  n ( r ii1 aa1 m @ m @ n t s ) ((pos "Mj%" ) ))
( "rearmed"  v ( r ii2 aa1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rearming"  v ( r ii2 aa1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rearmost"  j ( r i@1 m ou s t ) ((pos "OA%" ) ))
( "rearms"  v ( r ii2 aa1 m z ) ((pos "Ja%" ) (subcat "2A" )))
( "rearrange"  v ( r ii2 @ r ei1 n jh ) ((pos "H2%" ) (subcat "3A" "4C" "6A" "14" )))
( "rearranged"  v ( r ii2 @ r ei1 n jh ) ((pos "Hc%" "Hd%" ) (subcat "3A" "4C" "6A" "14" )))
( "rearrangement"  n ( r ii2 @ r ei1 n jh m @ n t ) ((pos "M6%" ) ))
( "rearrangements"  n ( r ii2 @ r ei1 n jh m @ n t s ) ((pos "Mj%" ) ))
( "rearranges"  v ( r ii2 @ r ei1 n jh i z ) ((pos "Ha%" ) (subcat "3A" "4C" "6A" "14" )))
( "rearranging"  v ( r ii2 @ r ei1 n jh i ng ) ((pos "Hb%" ) (subcat "3A" "4C" "6A" "14" )))
( "rears"  v ( r i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "rearward"  n ( r i@1 w @ d ) ((pos "K6%" ) ))
( "rearwards"  n ( r i@1 w @ d z ) ((pos "Kj%" "Pu%" ) ))
( "reason"  v ( r ii1 z @ n ) ((pos "J0%" "M6*" ) (subcat "2A" "3A" "6A" "9" )))
( "reasonable"  j ( r ii1 z n @ b l ) ((pos "OA%" ) ))
( "reasonableness"  n ( r ii1 z n @ b l n @ s ) ((pos "L@%" ) ))
( "reasonably"  a ( r ii1 z n @ b l ii ) ((pos "Pu%" ) ))
( "reasoned"  v ( r ii1 z @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "9" )))
( "reasoning"  v ( r ii1 z @ n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "3A" "6A" "9" )))
( "reasonless"  j ( r ii1 z @ n l @ s ) ((pos "OA$" ) ))
( "reasons"  v ( r ii1 z @ n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "9" )))
( "reassemble"  v ( r ii2 @ s e1 m b l ) ((pos "J2%" ) (subcat "2A" )))
( "reassembled"  v ( r ii2 @ s e1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reassembles"  v ( r ii2 @ s e1 m b l z ) ((pos "Ja%" ) (subcat "2A" )))
( "reassembling"  v ( r ii2 @ s e1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reassess"  v ( r ii2 @ s e1 s ) ((pos "H1%" ) (subcat "6A" )))
( "reassessed"  v ( r ii2 @ s e1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reassesses"  v ( r ii2 @ s e1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "reassessing"  v ( r ii2 @ s e1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reassessment"  n ( r ii2 @ s e1 s m @ n t ) ((pos "M6%" ) ))
( "reassessments"  n ( r ii2 @ s e1 s m @ n t s ) ((pos "Mj%" ) ))
( "reassurance"  n ( r i@2 sh uu1 r @ n s ) ((pos "M6%" ) ))
( "reassurances"  n ( r i@2 sh uu1 r @ n s i z ) ((pos "Mj%" ) ))
( "reassure"  v ( r ii2 @ sh u@1 r ) ((pos "H2%" ) (subcat )))
( "reassured"  v ( r ii2 @ sh u@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reassures"  v ( r ii2 @ sh u@1 z ) ((pos "Ha%" ) (subcat )))
( "reassuring"  v ( r ii2 @ sh u@1 r i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "reassuringly"  a ( r ii2 @ sh u@1 r i ng l ii ) ((pos "Pu%" ) ))
( "reattribute"  v ( r i1 a1 t r i b y uu t ) ((pos "H2%" ) (subcat )))
( "reattributed"  v ( r ii2 @ t r i1 b y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reattributes"  v ( r i1 a1 t r i b y uu t s ) ((pos "Ha%" ) (subcat )))
( "reattributing"  v ( r ii2 @ t r i1 b y uu t i ng ) ((pos "Hb%" ) (subcat )))
( "rebarbative"  j ( r i1 b aa1 b @ t i v ) ((pos "OA$" ) ))
( "rebate"  n ( r ii1 b ei t ) ((pos "M6%" ) ))
( "rebates"  n ( r ii1 b ei t s ) ((pos "Mj%" ) ))
( "rebel"  n ( r e1 b @ l ) ((pos "K6%" ) ))
( "rebel"  v ( r i1 b e1 l ) ((pos "I4%" ) (subcat "2A" )))
( "rebelled"  v ( r i1 b e1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "rebelling"  v ( r i1 b e1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "rebellion"  n ( r i1 b e1 l i@ n ) ((pos "M6%" ) ))
( "rebellions"  n ( r i1 b e1 l i@ n z ) ((pos "Mj%" ) ))
( "rebellious"  j ( r i1 b e1 l i@ s ) ((pos "OA%" ) ))
( "rebelliously"  a ( r i1 b e1 l i@ s l ii ) ((pos "Pu%" ) ))
( "rebelliousness"  n ( r i1 b e1 l i@ s n @ s ) ((pos "L@%" ) ))
( "rebels"  n ( r e1 b @ l z ) ((pos "Kj%" ) ))
( "rebels"  v ( r i1 b e1 l z ) ((pos "Ia%" ) (subcat "2A" )))
( "rebind"  v ( r ii2 b ai1 n d ) ((pos "H5$" ) (subcat )))
( "rebinding"  v ( r ii2 b ai1 n d i ng ) ((pos "Hb$" ) (subcat )))
( "rebinds"  v ( r ii2 b ai1 n d z ) ((pos "Ha$" ) (subcat )))
( "rebirth"  n ( r ii2 b @@1 th ) ((pos "K6$" ) ))
( "rebirths"  n ( r ii2 b @@1 th s ) ((pos "Kj$" ) ))
( "reborn"  j ( r ii2 b oo1 n ) ((pos "OA%" ) ))
( "rebound"  n ( r ii1 b au n d ) ((pos "K6%" ) ))
( "rebound"  v ( r ii2 b au1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rebound"  v ( r i1 b au1 n d ) ((pos "I0%" ) (subcat "2A" )))
( "rebounded"  v ( r i1 b au1 n d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "rebounding"  v ( r i1 b au1 n d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "rebounds"  n ( r ii1 b au n d z ) ((pos "Kj%" ) ))
( "rebounds"  v ( r i1 b au1 n d z ) ((pos "Ia%" ) (subcat "2A" )))
( "rebuff"  v ( r i1 b uh1 f ) ((pos "H0%" "K6%" ) (subcat )))
( "rebuffed"  v ( r i1 b uh1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rebuffing"  v ( r i1 b uh1 f i ng ) ((pos "Hb%" ) (subcat )))
( "rebuffs"  v ( r i1 b uh1 f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "rebuild"  v ( r ii2 b i1 l d ) ((pos "H5%" ) (subcat )))
( "rebuilding"  v ( r ii2 b i1 l d i ng ) ((pos "Hb%" ) (subcat )))
( "rebuilds"  v ( r ii2 b i1 l d z ) ((pos "Ha%" ) (subcat )))
( "rebuilt"  v ( r ii2 b i1 l t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rebuke"  v ( r i1 b y uu1 k ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "rebuked"  v ( r i1 b y uu1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "rebukes"  v ( r i1 b y uu1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "rebuking"  v ( r i1 b y uu1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "rebukingly"  a ( r i1 b y uu1 k i ng l ii ) ((pos "Pu$" ) ))
( "rebus"  n ( r ii1 b @ s ) ((pos "K7$" ) ))
( "rebuses"  n ( r ii1 b @ s i z ) ((pos "Kj$" ) ))
( "rebut"  v ( r i1 b uh1 t ) ((pos "H4%" ) (subcat )))
( "rebuts"  v ( r i1 b uh1 t s ) ((pos "Ha%" ) (subcat )))
( "rebuttal"  n ( r i1 b uh1 t @ l ) ((pos "K6%" ) ))
( "rebuttals"  n ( r i1 b uh1 t @ l z ) ((pos "Kj%" ) ))
( "rebutted"  v ( r i1 b uh1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rebutting"  v ( r i1 b uh1 t i ng ) ((pos "Hb%" ) (subcat )))
( "rec"  n ( r i1 s ii1 v d ) ((pos "Y~%" ) ))
( "recalcitrance"  n ( r i1 k a1 l s i t r @ n s ) ((pos "L@%" ) ))
( "recalcitrancy"  n ( r i1 k a1 l s i t r @ n s ii ) ((pos "L@$" ) ))
( "recalcitrant"  j ( r i1 k a1 l s i t r @ n t ) ((pos "OA%" ) ))
( "recall"  v ( r i1 k oo1 l ) ((pos "H0%" "M6%" ) (subcat "6A" "6C" "8" "9" "10" "14" )))
( "recalled"  v ( r i1 k oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "8" "9" "10" "14" )))
( "recalling"  v ( r i1 k oo1 l i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "8" "9" "10" "14" )))
( "recalls"  v ( r i1 k oo1 l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6C" "8" "9" "10" "14" )))
( "recant"  v ( r i1 k a1 n t ) ((pos "J0%" ) (subcat "2A" )))
( "recantation"  n ( r ii2 k a n t ei1 sh @ n ) ((pos "M6%" ) ))
( "recantations"  n ( r ii2 k a n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recanted"  v ( r i1 k a1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "recanting"  v ( r i1 k a1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "recants"  v ( r i1 k a1 n t s ) ((pos "Ja%" ) (subcat "2A" )))
( "recap"  v ( r ii1 k a p ) ((pos "J4%" "K6%" ) ))
( "recap"  v ( r ii2 k a1 p ) ((pos "H4$" ) ))
( "recapitulate"  v ( r ii2 k @ p i1 ch u l ei t ) ((pos "J2%" ) (subcat "2A" )))
( "recapitulated"  v ( r ii2 k @ p i1 ch u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "recapitulates"  v ( r ii2 k @ p i1 ch u l ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "recapitulating"  v ( r ii2 k @ p i1 ch u l ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "recapitulation"  n ( r ii2 k @ p i2 ch u l ei1 sh @ n ) ((pos "M6%" ) ))
( "recapitulations"  n ( r ii2 k @ p i2 ch u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recapped"  v ( r ii1 k a p t ) ((pos "Jc%" "Jd%" ) ))
( "recapped"  v ( r ii2 k a1 p t ) ((pos "Hc$" "Hd$" ) ))
( "recapping"  v ( r ii1 k a p i ng ) ((pos "Jb%" ) ))
( "recapping"  v ( r ii2 k a1 p i ng ) ((pos "Hb$" ) ))
( "recaps"  v ( r ii1 k a p s ) ((pos "Ja%" "Kj%" ) ))
( "recaps"  v ( r ii2 k a1 p s ) ((pos "Ha$" ) ))
( "recapture"  v ( r ii2 k a1 p ch @ r ) ((pos "H2%" ) (subcat )))
( "recaptured"  v ( r ii2 k a1 p ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "recaptures"  v ( r ii2 k a1 p ch @ z ) ((pos "Ha%" ) (subcat )))
( "recapturing"  v ( r ii2 k a1 p ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "recast"  v ( r ii2 k aa1 s t ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "recasting"  v ( r ii2 k aa1 s t i ng ) ((pos "Hb%" ) (subcat )))
( "recasts"  v ( r ii2 k aa1 s t s ) ((pos "Ha%" ) (subcat )))
( "recce"  n ( r e1 k ii ) ((pos "K6$" ) ))
( "recces"  n ( r e1 k i z ) ((pos "Kj$" ) ))
( "recd"  n ( r i1 s ii1 v d ) ((pos "Y~%" ) ))
( "recede"  v ( r i1 s ii1 d ) ((pos "I2%" ) (subcat "2A" )))
( "receded"  v ( r i1 s ii1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "recedes"  v ( r i1 s ii1 d z ) ((pos "Ia%" ) (subcat "2A" )))
( "receding"  v ( r i1 s ii1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "receipt"  v ( r i1 s ii1 t ) ((pos "H0$" "M6%" ) (subcat )))
( "receipted"  v ( r i1 s ii1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "receipting"  v ( r i1 s ii1 t i ng ) ((pos "Hb$" ) (subcat )))
( "receipts"  v ( r i1 s ii1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "receivable"  j ( r i1 s ii1 v @ b l ) ((pos "OA$" ) ))
( "receive"  v ( r i1 s ii1 v ) ((pos "J2%" ) (subcat "2A" )))
( "received"  v ( r i1 s ii1 v d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "receiver"  n ( r i1 s ii1 v @ r ) ((pos "K6%" ) ))
( "receivers"  n ( r i1 s ii1 v @ z ) ((pos "Kj%" ) ))
( "receivership"  n ( r i1 s ii1 v @ sh i p ) ((pos "K6%" ) ))
( "receiverships"  n ( r i1 s ii1 v @ sh i p s ) ((pos "Kj$" ) ))
( "receives"  v ( r i1 s ii1 v z ) ((pos "Ja%" ) (subcat "2A" )))
( "receiving"  v ( r i1 s ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "receiving-set"  n ( r i1 s ii1 v i ng - s e t ) ((pos "K6%" ) ))
( "receiving-sets"  n ( r i1 s ii1 v i ng - s e t s ) ((pos "Kj%" ) ))
( "recent"  j ( r ii1 s n t ) ((pos "OA%" ) ))
( "recently"  a ( r ii1 s n t l ii ) ((pos "Pu%" ) ))
( "receptacle"  n ( r i1 s e1 p t @ k l ) ((pos "K6%" ) ))
( "receptacles"  n ( r i1 s e1 p t @ k l z ) ((pos "Kj%" ) ))
( "reception"  n ( r i1 s e1 p sh @ n ) ((pos "M6%" ) ))
( "reception-desk"  n ( r i1 s e1 p sh @ n - d e s k ) ((pos "K6%" ) ))
( "reception-desks"  n ( r i1 s e1 p sh @ n - d e s k s ) ((pos "Kj%" ) ))
( "receptionist"  n ( r i1 s e1 p sh @ n i s t ) ((pos "K6%" ) ))
( "receptionists"  n ( r i1 s e1 p sh @ n i s t s ) ((pos "Kj%" ) ))
( "receptions"  n ( r i1 s e1 p sh @ n z ) ((pos "Mj%" ) ))
( "receptive"  j ( r i1 s e1 p t i v ) ((pos "OA%" ) ))
( "receptively"  a ( r i1 s e1 p t i v l ii ) ((pos "Pu%" ) ))
( "receptiveness"  n ( r i1 s e1 p t i v n i s ) ((pos "L@%" ) ))
( "receptivity"  n ( r ii2 s e p t i1 v i t ii ) ((pos "L@%" ) ))
( "recess"  v ( r i1 s e1 s ) ((pos "H1$" "K7%" ) (subcat )))
( "recessed"  v ( r i1 s e1 s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "recesses"  v ( r i1 s e1 s i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "recessing"  v ( r i1 s e1 s i ng ) ((pos "Hb$" ) (subcat )))
( "recession"  n ( r i1 s e1 sh @ n ) ((pos "M6%" ) ))
( "recessional"  n ( r i1 s e1 sh @ n l ) ((pos "K6$" "OA$" ) ))
( "recessionals"  n ( r i1 s e1 sh @ n l z ) ((pos "Kj$" ) ))
( "recessions"  n ( r i1 s e1 sh @ n z ) ((pos "Mj%" ) ))
( "recessive"  j ( r i1 s e1 s i v ) ((pos "OA%" ) ))
( "recherch_e"  j ( r @1 sh e@1 sh ei ) ((pos "OA%" ) ))
( "recidivism"  n ( r i1 s i1 d i v i z @ m ) ((pos "L@$" ) ))
( "recidivist"  n ( r i1 s i1 d i v i s t ) ((pos "K6$" ) ))
( "recidivists"  n ( r i1 s i1 d i v i s t s ) ((pos "Kj$" ) ))
( "recipe"  n ( r e1 s @ p ii ) ((pos "K6%" ) ))
( "recipes"  n ( r e1 s @ p i z ) ((pos "Kj%" ) ))
( "recipient"  n ( r i1 s i1 p i@ n t ) ((pos "K6%" ) ))
( "recipients"  n ( r i1 s i1 p i@ n t s ) ((pos "Kj%" ) ))
( "reciprocal"  n ( r i1 s i1 p r @ k l ) ((pos "K6$" "OA%" ) ))
( "reciprocally"  a ( r i1 s i1 p r @ k l ii ) ((pos "Pu%" ) ))
( "reciprocals"  n ( r i1 s i1 p r @ k l z ) ((pos "Kj$" ) ))
( "reciprocate"  v ( r i1 s i1 p r @ k ei t ) ((pos "J2%" ) (subcat "2A" )))
( "reciprocated"  v ( r i1 s i1 p r @ k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reciprocates"  v ( r i1 s i1 p r @ k ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "reciprocating"  v ( r i1 s i1 p r @ k ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reciprocation"  n ( r i1 s i2 p r @ k ei1 sh @ n ) ((pos "L@%" ) ))
( "reciprocity"  n ( r e2 s i p r o1 s i t ii ) ((pos "L@%" ) ))
( "recital"  n ( r i1 s ai1 t l ) ((pos "K6%" ) ))
( "recitalist"  n ( r i1 s ai1 t @ l i s t ) ((pos "K6%" ) ))
( "recitalists"  n ( r i1 s ai1 t @ l i s t s ) ((pos "Kj%" ) ))
( "recitals"  n ( r i1 s ai1 t l z ) ((pos "Kj%" ) ))
( "recitation"  n ( r e2 s i t ei1 sh @ n ) ((pos "M6%" ) ))
( "recitations"  n ( r e2 s i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recitative"  n ( r e2 s i t @ t ii1 v ) ((pos "M6$" ) ))
( "recitatives"  n ( r e2 s i t @ t ii1 v z ) ((pos "Mj$" ) ))
( "recite"  v ( r i1 s ai1 t ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "recited"  v ( r i1 s ai1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "recites"  v ( r i1 s ai1 t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "reciting"  v ( r i1 s ai1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "reckless"  j ( r e1 k l @ s ) ((pos "OA%" ) ))
( "recklessly"  a ( r e1 k l @ s l ii ) ((pos "Pu%" ) ))
( "recklessness"  n ( r e1 k l @ s n @ s ) ((pos "L@%" ) ))
( "reckon"  v ( r e1 k @ n ) ((pos "J0%" ) (subcat "2A" "6A" "7A" "9" "15B" "16B" )))
( "reckoned"  v ( r e1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "7A" "9" "15B" "16B" )))
( "reckoner"  n ( r e1 k @ n @ r ) ((pos "K6%" ) ))
( "reckoners"  n ( r e1 k @ n @ z ) ((pos "Kj%" ) ))
( "reckoning"  v ( r e1 k @ n i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" "7A" "9" "15B" "16B" )))
( "reckonings"  n ( r e1 k @ n i ng z ) ((pos "Mj%" ) ))
( "reckons"  v ( r e1 k @ n z ) ((pos "Ja%" ) (subcat "2A" "6A" "7A" "9" "15B" "16B" )))
( "reclaim"  v ( r i1 k l ei1 m ) ((pos "H0%" ) (subcat )))
( "reclaimed"  v ( r i1 k l ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reclaiming"  v ( r i1 k l ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "reclaims"  v ( r i1 k l ei1 m z ) ((pos "Ha%" ) (subcat )))
( "reclamation"  n ( r e2 k l @ m ei1 sh @ n ) ((pos "L@%" ) ))
( "recline"  v ( r i1 k l ai1 n ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "reclined"  v ( r i1 k l ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "reclines"  v ( r i1 k l ai1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "reclining"  v ( r i1 k l ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "recluse"  n ( r i1 k l uu1 s ) ((pos "K6%" ) ))
( "recluses"  n ( r i1 k l uu1 s i z ) ((pos "Kj$" ) ))
( "recognition"  n ( r e2 k @ g n i1 sh @ n ) ((pos "L@%" ) ))
( "recognizable"  j ( r e1 k @ g n ai z @ b l ) ((pos "OA%" ) ))
( "recognizably"  a ( r e2 k @ g n ai1 z @ b l ii ) ((pos "Pu%" ) ))
( "recognizance"  n ( r i1 k o1 g n i z @ n s ) ((pos "K6$" ) ))
( "recognizances"  n ( r i1 k o1 g n i z @ n s i z ) ((pos "Kj$" ) ))
( "recognize"  v ( r e1 k @ g n ai z ) ((pos "H2%" ) (subcat "6A" "9" "16A" )))
( "recognized"  v ( r e1 k @ g n ai z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "16A" )))
( "recognizes"  v ( r e1 k @ g n ai z i z ) ((pos "Ha%" ) (subcat "6A" "9" "16A" )))
( "recognizing"  v ( r e1 k @ g n ai z i ng ) ((pos "Hb%" ) (subcat "6A" "9" "16A" )))
( "recoil"  v ( r i1 k oi1 l ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "recoiled"  v ( r i1 k oi1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "recoiling"  v ( r i1 k oi1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "recoils"  v ( r i1 k oi1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "recollect"  v ( r e2 k @ l e1 k t ) ((pos "J0%" ) (subcat "2A" "6A" "6C" "8" "9" )))
( "recollected"  v ( r e2 k @ l e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "6C" "8" "9" )))
( "recollecting"  v ( r e2 k @ l e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "6C" "8" "9" )))
( "recollection"  n ( r e2 k @ l e1 k sh @ n ) ((pos "M6%" ) ))
( "recollections"  n ( r e2 k @ l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "recollects"  v ( r e2 k @ l e1 k t s ) ((pos "Ja%" ) (subcat "2A" "6A" "6C" "8" "9" )))
( "recommend"  v ( r e2 k @ m e1 n d ) ((pos "H0%" ) (subcat "6A" "6C" "9" "12A" "13A" "14" "16A" )))
( "recommendation"  n ( r e2 k @ m e n d ei1 sh @ n ) ((pos "M6%" ) ))
( "recommendations"  n ( r e2 k @ m e n d ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recommended"  v ( r e2 k @ m e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "12A" "13A" "14" "16A" )))
( "recommending"  v ( r e2 k @ m e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "12A" "13A" "14" "16A" )))
( "recommends"  v ( r e2 k @ m e1 n d z ) ((pos "Ha%" ) (subcat "6A" "6C" "9" "12A" "13A" "14" "16A" )))
( "recommit"  v ( r ii2 k @ m i1 t ) ((pos "H4%" ) (subcat "6A" "14" )))
( "recommits"  v ( r ii2 k @ m i1 t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "recommitted"  v ( r ii2 k @ m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "recommitting"  v ( r ii2 k @ m i1 t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "recompense"  v ( r e1 k @ m p e n s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "recompensed"  v ( r e1 k @ m p e n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "recompenses"  v ( r e1 k @ m p e n s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "recompensing"  v ( r e1 k @ m p e n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reconcilable"  j ( r e2 k @ n s ai1 l @ b l ) ((pos "OA%" ) ))
( "reconcile"  v ( r e1 k @ n s ai l ) ((pos "H2%" ) (subcat "6A" )))
( "reconciled"  v ( r e1 k @ n s ai l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reconciles"  v ( r e1 k @ n s ai l z ) ((pos "Ha%" ) (subcat "6A" )))
( "reconciliation"  n ( r e2 k @ n s i2 l i ei1 sh @ n ) ((pos "M6%" ) ))
( "reconciliations"  n ( r e2 k @ n s i2 l i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reconciling"  v ( r e1 k @ n s ai l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "recondite"  j ( r e1 k @ n d ai t ) ((pos "OA$" ) ))
( "recondition"  v ( r ii2 k @ n d i1 sh @ n ) ((pos "H0%" ) (subcat )))
( "reconditioned"  v ( r ii2 k @ n d i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reconditioning"  v ( r ii2 k @ n d i1 sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "reconditions"  v ( r ii2 k @ n d i1 sh @ n z ) ((pos "Ha%" ) (subcat )))
( "reconnaissance"  n ( r i1 k o1 n i s @ n s ) ((pos "M6%" ) ))
( "reconnaissances"  n ( r i1 k o1 n i s @ n s i z ) ((pos "Mj%" ) ))
( "reconnoitre"  v ( r e2 k @ n oi1 t @ r ) ((pos "J2%" ) (subcat "2A" )))
( "reconnoitred"  v ( r e2 k @ n oi1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reconnoitres"  v ( r e2 k @ n oi1 t @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "reconnoitring"  v ( r e2 k @ n oi1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reconsecrate"  v ( r ii1 k o1 n s i k r ei t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "reconsecrated"  v ( r ii1 k o1 n s i k r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "reconsecrates"  v ( r ii1 k o1 n s i k r ei t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "reconsecrating"  v ( r ii1 k o1 n s i k r ei t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "reconsider"  v ( r ii2 k @ n s i1 d @ r ) ((pos "H0%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "reconsidered"  v ( r ii2 k @ n s i1 d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "reconsidering"  v ( r ii2 k @ n s i1 d @ r i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "reconsiders"  v ( r ii2 k @ n s i1 d @ z ) ((pos "Ha%" ) (subcat "6A" "6C" "8" "9" "10" )))
( "reconstruct"  v ( r ii2 k @ n s t r uh1 k t ) ((pos "H0%" ) (subcat )))
( "reconstructed"  v ( r ii2 k @ n s t r uh1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reconstructing"  v ( r ii2 k @ n s t r uh1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "reconstruction"  n ( r ii2 k @ n s t r uh1 k sh @ n ) ((pos "M6%" ) ))
( "reconstructions"  n ( r ii2 k @ n s t r uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "reconstructs"  v ( r ii2 k @ n s t r uh1 k t s ) ((pos "Ha%" ) (subcat )))
( "reconvict"  v ( r ii2 k @ n v i1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "reconvicted"  v ( r ii2 k @ n v i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reconvicting"  v ( r ii2 k @ n v i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reconvicts"  v ( r ii2 k @ n v i1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "record"  n ( r e1 k oo d ) ((pos "M6%" ) ))
( "record"  v ( r i1 k oo1 d ) ((pos "H0%" ) (subcat )))
( "record-breaking"  j ( r e1 k oo d - b r ei k i ng ) ((pos "OA%" ) ))
( "record-player"  n ( r e1 k oo d - p l ei @ r ) ((pos "K6%" ) ))
( "record-players"  n ( r e1 k oo d - p l ei @ z ) ((pos "Kj%" ) ))
( "recorded"  v ( r i1 k oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "recorder"  n ( r i1 k oo1 d @ r ) ((pos "K6%" ) ))
( "recorders"  n ( r i1 k oo1 d @ z ) ((pos "Kj%" ) ))
( "recording"  v ( r i1 k oo1 d i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "recordings"  n ( r i1 k oo1 d i ng z ) ((pos "Kj%" ) ))
( "records"  n ( r e1 k oo d z ) ((pos "Mj%" ) ))
( "records"  v ( r i1 k oo1 d z ) ((pos "Ha%" ) (subcat )))
( "recount"  v ( r i1 k au1 n t ) ((pos "H0%" ) (subcat )))
( "recounted"  v ( r i1 k au1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "recounting"  v ( r i1 k au1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "recounts"  v ( r i1 k au1 n t s ) ((pos "Ha%" ) (subcat )))
( "recoup"  v ( r i1 k uu1 p ) ((pos "H0%" ) (subcat "6A" )))
( "recouped"  v ( r i1 k uu1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "recouping"  v ( r i1 k uu1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "recoups"  v ( r i1 k uu1 p s ) ((pos "Ha%" ) (subcat "6A" )))
( "recourse"  n ( r i1 k oo1 s ) ((pos "L@%" ) ))
( "recover"  v ( r i1 k uh1 v @ r ) ((pos "J0%" ) (subcat "2A" "3A" )))
( "recoverable"  j ( r i1 k uh1 v @ r @ b l ) ((pos "OA%" ) ))
( "recovered"  v ( r i1 k uh1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "recoveries"  n ( r i1 k uh1 v @ r i z ) ((pos "Mj%" ) ))
( "recovering"  v ( r i1 k uh1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "recovers"  v ( r i1 k uh1 v @ z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "recovery"  n ( r i1 k uh1 v @ r ii ) ((pos "M8%" ) ))
( "recreant"  n ( r e1 k r i@ n t ) ((pos "K6$" "OA$" ) ))
( "recreants"  n ( r e1 k r i@ n t s ) ((pos "Kj$" ) ))
( "recreate"  v ( r ii2 k r ii ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "recreated"  v ( r ii2 k r ii ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "recreates"  v ( r ii2 k r ii ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "recreating"  v ( r ii2 k r ii ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "recreation"  n ( r e2 k r i ei1 sh @ n ) ((pos "M6%" ) ))
( "recreational"  j ( r e2 k r i ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "recreations"  n ( r e2 k r i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recriminate"  v ( r i1 k r i1 m i n ei t ) ((pos "I2$" ) (subcat "2A" )))
( "recriminated"  v ( r i1 k r i1 m i n ei t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "recriminates"  v ( r i1 k r i1 m i n ei t s ) ((pos "Ia$" ) (subcat "2A" )))
( "recriminating"  v ( r i1 k r i1 m i n ei t i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "recrimination"  n ( r i1 k r i2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "recriminations"  n ( r i1 k r i2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recriminatory"  j ( r i1 k r i1 m i n @ t r ii ) ((pos "OA$" ) ))
( "recrudescence"  n ( r ii2 k r uu d e1 s @ n s ) ((pos "M6$" ) ))
( "recrudescences"  n ( r ii2 k r uu d e1 s @ n s i z ) ((pos "Mj$" ) ))
( "recruit"  v ( r i1 k r uu1 t ) ((pos "J0%" "K6%" ) (subcat )))
( "recruited"  v ( r i1 k r uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "recruiting"  v ( r i1 k r uu1 t i ng ) ((pos "Jb%" ) (subcat )))
( "recruitment"  n ( r i1 k r uu1 t m @ n t ) ((pos "K6%" ) ))
( "recruitments"  n ( r i1 k r uu1 t m @ n t s ) ((pos "Kj%" ) ))
( "recruits"  v ( r i1 k r uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat )))
( "rectal"  j ( r e1 k t @ l ) ((pos "OA%" ) ))
( "rectangle"  n ( r e1 k t a ng g l ) ((pos "K6%" ) ))
( "rectangles"  n ( r e1 k t a ng g l z ) ((pos "Kj%" ) ))
( "rectangular"  j ( r e1 k t a1 ng g y u l @ r ) ((pos "OA%" ) ))
( "rectification"  n ( r e2 k t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "rectifications"  n ( r e2 k t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "rectified"  v ( r e1 k t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rectifier"  n ( r e1 k t i f ai @ r ) ((pos "K6%" ) ))
( "rectifiers"  n ( r e1 k t i f ai @ z ) ((pos "Kj%" ) ))
( "rectifies"  v ( r e1 k t i f ai z ) ((pos "Ha%" ) (subcat )))
( "rectify"  v ( r e1 k t i f ai ) ((pos "H3%" ) (subcat )))
( "rectifying"  v ( r e1 k t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "rectilinear"  j ( r e2 k t i l i1 n i@ r ) ((pos "OA%" ) ))
( "rectitude"  n ( r e1 k t i t y uu d ) ((pos "L@%" ) ))
( "recto"  n ( r e1 k t ou ) ((pos "K6$" ) ))
( "rector"  n ( r e1 k t @ r ) ((pos "K6%" ) ))
( "rectories"  n ( r e1 k t @ r i z ) ((pos "Kj%" ) ))
( "rectors"  n ( r e1 k t @ z ) ((pos "Kj%" ) ))
( "rectory"  n ( r e1 k t @ r ii ) ((pos "K8%" ) ))
( "rectos"  n ( r e1 k t ou z ) ((pos "Kj$" ) ))
( "rectum"  n ( r e1 k t @ m ) ((pos "K6%" ) ))
( "rectums"  n ( r e1 k t @ m z ) ((pos "Kj$" ) ))
( "recumbent"  j ( r i1 k uh1 m b @ n t ) ((pos "OA%" ) ))
( "recuperate"  v ( r i1 k uu1 p @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "recuperated"  v ( r i1 k uu1 p @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "recuperates"  v ( r i1 k uu1 p @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "recuperating"  v ( r i1 k uu1 p @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "recuperation"  n ( r i1 k uu2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "recuperations"  n ( r i1 k uu2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "recuperative"  j ( r i1 k uu1 p @ r @ t i v ) ((pos "OA%" ) ))
( "recur"  v ( r i1 k @@1 r ) ((pos "I4%" ) (subcat "2A" )))
( "recurred"  v ( r i1 k @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "recurrence"  n ( r i1 k uh1 r @ n s ) ((pos "M6%" ) ))
( "recurrences"  n ( r i1 k uh1 r @ n s i z ) ((pos "Mj%" ) ))
( "recurrent"  j ( r i1 k uh1 r @ n t ) ((pos "OA%" ) ))
( "recurring"  v ( r i1 k @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "recurs"  v ( r i1 k @@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "recurve"  v ( r ii2 k @@1 v ) ((pos "J2$" ) ))
( "recurved"  v ( r ii2 k @@1 v d ) ((pos "Jc$" "Jd$" ) ))
( "recurves"  v ( r ii2 k @@1 v z ) ((pos "Ja$" ) ))
( "recurving"  v ( r ii2 k @@1 v i ng ) ((pos "Jb$" ) ))
( "recusancy"  n ( r e1 k y u z @ n s ii ) ((pos "L@$" ) ))
( "recusant"  n ( r e1 k y u z @ n t ) ((pos "K6$" "OA$" ) ))
( "recusants"  n ( r e1 k y u z @ n t s ) ((pos "Kj$" ) ))
( "recycle"  v ( r ii2 s ai1 k l ) ((pos "H2%" ) (subcat )))
( "recycled"  v ( r ii2 s ai1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "recycles"  v ( r ii2 s ai1 k l z ) ((pos "Ha%" ) (subcat )))
( "recycling"  v ( r ii2 s ai1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "red"  n ( r e1 d ) ((pos "M6%" "OE%" ) ))
( "red-hot"  j ( r e1 d - h o t ) ((pos "OA%" ) ))
( "red-rimmed"  j ( r e1 d - r i m d ) ((pos "OA%" ) ))
( "redact"  v ( r i1 d a1 k t ) ((pos "H0$" ) (subcat )))
( "redacted"  v ( r i1 d a1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "redacting"  v ( r i1 d a1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "redaction"  n ( r i1 d a1 k sh @ n ) ((pos "M6$" ) ))
( "redactions"  n ( r i1 d a1 k sh @ n z ) ((pos "Mj$" ) ))
( "redacts"  v ( r i1 d a1 k t s ) ((pos "Ha$" ) (subcat )))
( "redbreast"  n ( r e1 d b r e s t ) ((pos "K6%" ) ))
( "redbreasts"  n ( r e1 d b r e s t s ) ((pos "Kj%" ) ))
( "redcap"  n ( r e1 d k a p ) ((pos "K6%" ) ))
( "redcaps"  n ( r e1 d k a p s ) ((pos "Kj%" ) ))
( "redcoat"  n ( r e1 d k ou t ) ((pos "K6%" ) ))
( "redcoats"  n ( r e1 d k ou t s ) ((pos "Kj%" ) ))
( "redden"  v ( r e1 d n ) ((pos "J0%" ) (subcat "2A" )))
( "reddened"  v ( r e1 d n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reddening"  v ( r e1 d n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reddens"  v ( r e1 d n z ) ((pos "Ja%" ) (subcat "2A" )))
( "redder"  j ( r e1 d @ r ) ((pos "Or%" ) ))
( "reddest"  j ( r e1 d i s t ) ((pos "Os%" ) ))
( "reddish"  j ( r e1 d i sh ) ((pos "OA%" ) ))
( "redecorate"  v ( r ii1 d e1 k @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "redecorated"  v ( r ii1 d e1 k @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "redecorates"  v ( r ii1 d e1 k @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "redecorating"  v ( r ii1 d e1 k @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "redeem"  v ( r i1 d ii1 m ) ((pos "H0%" ) (subcat "6A" )))
( "redeemable"  j ( r i1 d ii1 m @ b l ) ((pos "OA%" ) ))
( "redeemed"  v ( r i1 d ii1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "redeemer"  n ( r i1 d ii1 m @ r ) ((pos "K6%" ) ))
( "redeemers"  n ( r i1 d ii1 m @ z ) ((pos "Kj%" ) ))
( "redeeming"  v ( r i1 d ii1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "redeems"  v ( r i1 d ii1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "redefine"  v ( r ii2 d i f ai1 n ) ((pos "H2%" ) (subcat )))
( "redefined"  v ( r ii2 d i f ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "redefines"  v ( r ii2 d i f ai1 n z ) ((pos "Ha%" ) (subcat )))
( "redefining"  v ( r ii2 d i f ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "redemption"  n ( r i1 d e1 m p sh @ n ) ((pos "L@%" ) ))
( "redemptive"  j ( r i1 d e1 m p t i v ) ((pos "OA%" ) ))
( "redeploy"  v ( r ii2 d i p l oi1 ) ((pos "H0%" ) (subcat )))
( "redeployed"  v ( r ii2 d i p l oi1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "redeploying"  v ( r ii2 d i p l oi1 i ng ) ((pos "Hb%" ) (subcat )))
( "redeployment"  n ( r ii2 d i p l oi1 m @ n t ) ((pos "M6%" ) ))
( "redeployments"  n ( r ii2 d i p l oi1 m @ n t s ) ((pos "Mj%" ) ))
( "redeploys"  v ( r ii2 d i p l oi1 z ) ((pos "Ha%" ) (subcat )))
( "redesign"  v ( r ii2 d i z ai1 n ) ((pos "H0%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "redesigned"  v ( r ii2 d i z ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "redesigning"  v ( r ii2 d i z ai1 n i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "redesigns"  v ( r ii2 d i z ai1 n z ) ((pos "Ha%" ) (subcat "2A" "2C" "6A" "14" "16A" )))
( "redevelop"  v ( r ii2 d i v e1 l @ p ) ((pos "H0%" ) (subcat "2A" "2C" "3A" )))
( "redeveloped"  v ( r ii2 d i v e1 l @ p t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "3A" )))
( "redeveloping"  v ( r ii2 d i v e1 l @ p i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "3A" )))
( "redevelopment"  n ( r ii2 d i v e1 l @ p m @ n t ) ((pos "M6%" ) ))
( "redevelopments"  n ( r ii2 d i v e1 l @ p m @ n t s ) ((pos "Mj%" ) ))
( "redevelops"  v ( r ii2 d i v e1 l @ p s ) ((pos "Ha%" ) (subcat "2A" "2C" "3A" )))
( "redhead"  n ( r e1 d h e d ) ((pos "K6%" ) ))
( "redheads"  n ( r e1 d h e d z ) ((pos "Kj%" ) ))
( "redid"  v ( r ii2 d i1 d ) ((pos "Hc%" ) (subcat )))
( "rediffusion"  n ( r ii2 d i f y uu1 zh n ) ((pos "L@%" ) ))
( "rediscover"  v ( r ii2 d i s k uh1 v @ r ) ((pos "H0%" ) (subcat "6A" "8" "9" "10" )))
( "rediscovered"  v ( r ii2 d i s k uh1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "8" "9" "10" )))
( "rediscoveries"  n ( r ii2 d i s k uh1 v @ r i z ) ((pos "Mj%" ) ))
( "rediscovering"  v ( r ii2 d i s k uh1 v @ r i ng ) ((pos "Hb%" ) (subcat "6A" "8" "9" "10" )))
( "rediscovers"  v ( r ii2 d i s k uh1 v @ z ) ((pos "Ha%" ) (subcat "6A" "8" "9" "10" )))
( "rediscovery"  n ( r ii2 d i s k uh1 v @ r ii ) ((pos "M8%" ) ))
( "redisposition"  n ( r ii2 d i s p @ z i1 sh @ n ) ((pos "K6%" ) ))
( "redispositions"  n ( r ii2 d i s p @ z i1 sh @ n z ) ((pos "Kj%" ) ))
( "redistribute"  v ( r ii2 d i s t r i1 b y uu t ) ((pos "H2%" ) (subcat "6A" )))
( "redistributed"  v ( r ii2 d i s t r i1 b y uu t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "redistributes"  v ( r ii2 d i s t r i1 b y uu t s ) ((pos "Ha%" ) (subcat "6A" )))
( "redistributing"  v ( r ii2 d i s t r i1 b y uu t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "redistribution"  n ( r ii2 d i s t r i b y uu1 sh @ n ) ((pos "M6%" ) ))
( "redistributions"  n ( r ii2 d i s t r i b y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "redness"  n ( r e1 d n i s ) ((pos "L@%" ) ))
( "redo"  v ( r ii2 d uu1 ) ((pos "H5%" ) (subcat )))
( "redoes"  v ( r ii2 d uh1 z ) ((pos "Ha%" ) (subcat )))
( "redoing"  v ( r ii2 d uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "redolence"  n ( r e1 d @ l @ n s ) ((pos "L@%" ) ))
( "redolent"  j ( r e1 d @ l @ n t ) ((pos "OA%" ) ))
( "redone"  v ( r ii2 d uh1 n ) ((pos "Hd%" ) (subcat )))
( "redouble"  v ( r i1 d uh1 b l ) ((pos "J2%" ) (subcat "2A" )))
( "redoubled"  v ( r i1 d uh1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "redoubles"  v ( r i1 d uh1 b l z ) ((pos "Ja%" ) (subcat "2A" )))
( "redoubling"  v ( r i1 d uh1 b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "redoubt"  n ( r i1 d au1 t ) ((pos "K6$" ) ))
( "redoubtable"  j ( r i1 d au1 t @ b l ) ((pos "OA%" ) ))
( "redoubts"  n ( r i1 d au1 t s ) ((pos "Kj$" ) ))
( "redound"  v ( r i1 d au1 n d ) ((pos "I0$" ) (subcat )))
( "redounded"  v ( r i1 d au1 n d i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "redounding"  v ( r i1 d au1 n d i ng ) ((pos "Ib$" ) (subcat )))
( "redounds"  v ( r i1 d au1 n d z ) ((pos "Ia$" ) (subcat )))
( "redress"  v ( r i1 d r e1 s ) ((pos "H1%" "L@%" ) (subcat )))
( "redressed"  v ( r i1 d r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "redresses"  v ( r i1 d r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "redressing"  v ( r i1 d r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "reds"  n ( r e1 d z ) ((pos "Mj%" ) ))
( "redskin"  n ( r e1 d s k i n ) ((pos "K6%" ) ))
( "redskins"  n ( r e1 d s k i n z ) ((pos "Kj%" ) ))
( "reduce"  v ( r i1 d y uu1 s ) ((pos "J2%" ) (subcat "2A" "2B" "6A" )))
( "reduced"  v ( r i1 d y uu1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "6A" )))
( "reduces"  v ( r i1 d y uu1 s i z ) ((pos "Ja%" ) (subcat "2A" "2B" "6A" )))
( "reducible"  j ( r i1 d y uu1 s @ b l ) ((pos "OA%" ) ))
( "reducing"  v ( r i1 d y uu1 s i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "6A" )))
( "reductio ad absurdum"  n ( r i1 d uh2 k t i@ u - a2 d - @ b s @@1 d @ m ) ((pos "Mi%" ) ))
( "reduction"  n ( r i1 d uh1 k sh @ n ) ((pos "M6%" ) ))
( "reductions"  n ( r i1 d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "redundance"  n ( r i1 d uh1 n d @ n s ) ((pos "K6$" ) ))
( "redundances"  n ( r i1 d uh1 n d @ n s i z ) ((pos "Kj$" ) ))
( "redundancies"  n ( r i1 d uh1 n d @ n s i z ) ((pos "Mj%" ) ))
( "redundancy"  n ( r i1 d uh1 n d @ n s ii ) ((pos "M8%" ) ))
( "redundant"  j ( r i1 d uh1 n d @ n t ) ((pos "OA%" ) ))
( "reduplicate"  v ( r i1 d y uu1 p l i k ei t ) ((pos "H2%" ) (subcat )))
( "reduplicated"  v ( r i1 d y uu1 p l i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reduplicates"  v ( r i1 d y uu1 p l i k ei t s ) ((pos "Ha%" ) (subcat )))
( "reduplicating"  v ( r i1 d y uu1 p l i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "reduplication"  n ( r i1 d y uu2 p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "reduplications"  n ( r i1 d y uu2 p l i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "redwing"  n ( r e1 d w i ng ) ((pos "K6%" ) ))
( "redwings"  n ( r e1 d w i ng z ) ((pos "Kj%" ) ))
( "redwood"  n ( r e1 d w u d ) ((pos "K6%" ) ))
( "redwoods"  n ( r e1 d w u d z ) ((pos "Kj%" ) ))
( "reed"  n ( r ii1 d ) ((pos "M6%" ) ))
( "reedier"  j ( r ii1 d i@ r ) ((pos "Or%" ) ))
( "reediest"  j ( r ii1 d i i s t ) ((pos "Os%" ) ))
( "reeds"  n ( r ii1 d z ) ((pos "Mj%" ) ))
( "reedy"  j ( r ii1 d ii ) ((pos "OD%" ) ))
( "reef"  v ( r ii1 f ) ((pos "H0$" "K6%" ) (subcat )))
( "reef-knot"  n ( r ii1 f - n o t ) ((pos "K6%" ) ))
( "reef-knots"  n ( r ii1 f - n o t s ) ((pos "Kj%" ) ))
( "reefed"  v ( r ii1 f t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "reefer"  n ( r ii1 f @ r ) ((pos "K6%" ) ))
( "reefers"  n ( r ii1 f @ z ) ((pos "Kj%" ) ))
( "reefing"  v ( r ii1 f i ng ) ((pos "Hb$" ) (subcat )))
( "reefs"  v ( r ii1 f s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "reek"  v ( r ii1 k ) ((pos "I0%" "L@%" ) (subcat )))
( "reeked"  v ( r ii1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "reeking"  v ( r ii1 k i ng ) ((pos "Ib%" ) (subcat )))
( "reeks"  v ( r ii1 k s ) ((pos "Ia%" ) (subcat )))
( "reel"  v ( r ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "reeled"  v ( r ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "reeling"  v ( r ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "reels"  v ( r ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "reeve"  n ( r ii1 v ) ((pos "K6$" ) ))
( "reeves"  n ( r ii1 v z ) ((pos "Kj$" ) ))
( "ref"  n ( r e1 f ) ((pos "Y>%" ) ))
( "reface"  v ( r ii2 f ei1 s ) ((pos "H2$" ) ))
( "refaced"  v ( r ii2 f ei1 s t ) ((pos "Hc$" "Hd%" ) ))
( "refaces"  v ( r ii2 f ei1 s i z ) ((pos "Ha$" ) ))
( "refacing"  v ( r ii2 f ei1 s i ng ) ((pos "Hb$" ) ))
( "refashion"  v ( r ii1 f a1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "refashioned"  v ( r ii1 f a1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "refashioning"  v ( r ii1 f a1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "refashions"  v ( r ii1 f a1 sh @ n z ) ((pos "Ha%" ) (subcat "6A" )))
( "refection"  n ( r i1 f e1 k sh @ n ) ((pos "M6$" ) ))
( "refections"  n ( r i1 f e1 k sh @ n z ) ((pos "Mj$" ) ))
( "refectories"  n ( r i1 f e1 k t @ r i z ) ((pos "Kj%" ) ))
( "refectory"  n ( r i1 f e1 k t @ r ii ) ((pos "K8%" ) ))
( "refer"  v ( r i1 f @@1 r ) ((pos "J4%" ) (subcat "3A" "14" )))
( "referable"  j ( r i1 f @@1 r @ b l ) ((pos "OA%" ) ))
( "referee"  v ( r e2 f @ r ii1 ) ((pos "J5%" "K6%" ) (subcat "2A" )))
( "refereed"  v ( r e2 f @ r ii1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "refereeing"  v ( r e2 f @ r ii1 i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "referees"  v ( r e2 f @ r ii1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "reference"  n ( r e1 f r @ n s ) ((pos "M6%" ) ))
( "references"  n ( r e1 f r @ n s i z ) ((pos "Mj%" ) ))
( "referenda"  n ( r e2 f @ r e1 n d @ ) ((pos "Kj%" ) ))
( "referendum"  n ( r e2 f @ r e1 n d @ m ) ((pos "K6%" ) ))
( "referendums"  n ( r e2 f @ r e1 n d @ m z ) ((pos "Kj%" ) ))
( "referential"  j ( r e2 f @ r e1 n sh l ) ((pos "OA%" ) ))
( "referred"  v ( r i1 f @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "14" )))
( "referring"  v ( r i1 f @@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "14" )))
( "refers"  v ( r i1 f @@1 z ) ((pos "Ja%" ) (subcat "3A" "14" )))
( "refill"  n ( r ii1 f i l ) ((pos "K6%" ) ))
( "refill"  v ( r ii2 f i1 l ) ((pos "H0%" ) ))
( "refilled"  v ( r ii2 f i1 l d ) ((pos "Hc%" "Hd%" ) ))
( "refilling"  v ( r ii2 f i1 l i ng ) ((pos "Hb%" ) ))
( "refills"  n ( r ii1 f i l z ) ((pos "Kj%" ) ))
( "refills"  v ( r ii2 f i1 l z ) ((pos "Ha%" ) ))
( "refine"  v ( r i1 f ai1 n ) ((pos "J2%" ) (subcat "2A" )))
( "refined"  v ( r i1 f ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "refinement"  n ( r i1 f ai1 n m @ n t ) ((pos "M6%" ) ))
( "refinements"  n ( r i1 f ai1 n m @ n t s ) ((pos "Mj%" ) ))
( "refiner"  n ( r i1 f ai1 n @ r ) ((pos "K6%" ) ))
( "refineries"  n ( r i1 f ai1 n @ r i z ) ((pos "Kj%" ) ))
( "refiners"  n ( r i1 f ai1 n @ z ) ((pos "Kj%" ) ))
( "refinery"  n ( r i1 f ai1 n @ r ii ) ((pos "K8%" ) ))
( "refines"  v ( r i1 f ai1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "refining"  v ( r i1 f ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "refit"  n ( r ii1 f i t ) ((pos "K6%" ) ))
( "refit"  v ( r ii2 f i1 t ) ((pos "J4%" ) (subcat "2A" )))
( "refits"  n ( r ii1 f i t s ) ((pos "Kj%" ) ))
( "refits"  v ( r ii2 f i1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "refitted"  v ( r ii2 f i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "refitting"  v ( r ii2 f i1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reflate"  v ( r ii2 f l ei1 t ) ((pos "H2$" ) (subcat )))
( "reflated"  v ( r ii2 f l ei1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "reflates"  v ( r ii2 f l ei1 t s ) ((pos "Ha$" ) (subcat )))
( "reflating"  v ( r ii2 f l ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "reflation"  n ( r ii1 f l ei1 sh @ n ) ((pos "L@%" ) ))
( "reflect"  v ( r i1 f l e1 k t ) ((pos "J0%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "reflected"  v ( r i1 f l e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "reflecting"  v ( r i1 f l e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "reflection"  n ( r i1 f l e1 k sh @ n ) ((pos "M6%" ) ))
( "reflections"  n ( r i1 f l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "reflective"  j ( r i1 f l e1 k t i v ) ((pos "OA%" ) ))
( "reflectively"  a ( r i1 f l e1 k t i v l ii ) ((pos "Pu%" ) ))
( "reflector"  n ( r i1 f l e1 k t @ r ) ((pos "K6%" ) ))
( "reflectors"  n ( r i1 f l e1 k t @ z ) ((pos "Kj%" ) ))
( "reflects"  v ( r i1 f l e1 k t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "9" "10" )))
( "reflex"  n ( r ii1 f l e k s ) ((pos "K7%" "OA%" ) ))
( "reflexes"  n ( r ii1 f l e k s i z ) ((pos "Kj%" ) ))
( "reflexion"  n ( r i1 f l e1 k sh @ n ) ((pos "M6%" ) ))
( "reflexions"  n ( r i1 f l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "reflexive"  n ( r i1 f l e1 k s i v ) ((pos "K6%" "OA%" ) ))
( "reflexives"  n ( r i1 f l e1 k s i v z ) ((pos "Kj%" ) ))
( "refloat"  v ( r ii2 f l ou1 t ) ((pos "J0%" ) (subcat "2A" )))
( "refloated"  v ( r ii2 f l ou1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "refloating"  v ( r ii2 f l ou1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "refloats"  v ( r ii2 f l ou1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "reflux"  n ( r ii2 f l uh1 k s ) ((pos "K7$" ) ))
( "refluxes"  n ( r ii2 f l uh1 k s i z ) ((pos "Kj$" ) ))
( "reforest"  v ( r ii2 f o1 r i s t ) ((pos "H0$" ) ))
( "reforestation"  n ( r ii1 f o2 r i s t ei1 sh @ n ) ((pos "M6%" ) ))
( "reforestations"  n ( r ii1 f o2 r i s t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reforested"  v ( r ii2 f o1 r i s t i d ) ((pos "Hc$" "Hd$" ) ))
( "reforesting"  v ( r ii2 f o1 r i s t i ng ) ((pos "Hb$" ) ))
( "reforests"  v ( r ii2 f o1 r i s t s ) ((pos "Ha$" ) ))
( "reform"  v ( r i1 f oo1 m ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "reformation"  n ( r e2 f @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "reformations"  n ( r e2 f @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reformatories"  n ( r i1 f oo1 m @ t r i z ) ((pos "Kj%" ) ))
( "reformatory"  n ( r i1 f oo1 m @ t r ii ) ((pos "K8%" "OA%" ) ))
( "reformed"  v ( r i1 f oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reformer"  n ( r i1 f oo1 m @ r ) ((pos "K6%" ) ))
( "reformers"  n ( r i1 f oo1 m @ z ) ((pos "Kj%" ) ))
( "reforming"  v ( r i1 f oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reforms"  v ( r i1 f oo1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "refract"  v ( r i1 f r a1 k t ) ((pos "H0%" ) (subcat )))
( "refracted"  v ( r i1 f r a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refracting"  v ( r i1 f r a1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "refraction"  n ( r i1 f r a1 k sh @ n ) ((pos "M6%" ) ))
( "refractions"  n ( r i1 f r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "refractory"  j ( r i1 f r a1 k t @ r ii ) ((pos "OA%" ) ))
( "refracts"  v ( r i1 f r a1 k t s ) ((pos "Ha%" ) (subcat )))
( "refrain"  v ( r i1 f r ei1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "refrained"  v ( r i1 f r ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "refraining"  v ( r i1 f r ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "refrains"  v ( r i1 f r ei1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "refresh"  v ( r i1 f r e1 sh ) ((pos "H1%" ) (subcat )))
( "refreshed"  v ( r i1 f r e1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refresher"  n ( r i1 f r e1 sh @ r ) ((pos "K6%" ) ))
( "refreshers"  n ( r i1 f r e1 sh @ z ) ((pos "Kj%" ) ))
( "refreshes"  v ( r i1 f r e1 sh i z ) ((pos "Ha%" ) (subcat )))
( "refreshing"  v ( r i1 f r e1 sh i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "refreshingly"  a ( r i1 f r e1 sh i ng l ii ) ((pos "Pu%" ) ))
( "refreshment"  n ( r i1 f r e1 sh m @ n t ) ((pos "M6%" ) ))
( "refreshments"  n ( r i1 f r e1 sh m @ n t s ) ((pos "Mj%" ) ))
( "refrigerant"  n ( r i1 f r i1 jh @ r @ n t ) ((pos "K6$" "OA$" ) ))
( "refrigerants"  n ( r i1 f r i1 jh @ r @ n t s ) ((pos "Kj$" ) ))
( "refrigerate"  v ( r i1 f r i1 jh @ r ei t ) ((pos "H2%" ) (subcat )))
( "refrigerated"  v ( r i1 f r i1 jh @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refrigerates"  v ( r i1 f r i1 jh @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "refrigerating"  v ( r i1 f r i1 jh @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "refrigeration"  n ( r i1 f r i2 jh @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "refrigerations"  n ( r i1 f r i2 jh @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "refrigerator"  n ( r i1 f r i1 jh @ r ei t @ r ) ((pos "K6%" ) ))
( "refrigerators"  n ( r i1 f r i1 jh @ r ei t @ z ) ((pos "Kj%" ) ))
( "reft"  v ( r e1 f t ) ((pos "Hd$" ) (subcat )))
( "refuel"  v ( r ii2 f y uu1 @ l ) ((pos "J4%" ) (subcat "2A" )))
( "refuelled"  v ( r ii2 f y uu1 @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "refuelling"  v ( r ii2 f y uu1 @ l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "refuels"  v ( r ii2 f y uu1 @ l z ) ((pos "Ja%" ) (subcat "2A" )))
( "refuge"  n ( r e1 f y uu jh ) ((pos "M6%" ) ))
( "refugee"  n ( r e2 f y u jh ii1 ) ((pos "K6%" ) ))
( "refugees"  n ( r e2 f y u jh ii1 z ) ((pos "Kj%" ) ))
( "refuges"  n ( r e1 f y uu jh i z ) ((pos "Mj%" ) ))
( "refulgence"  n ( r i1 f uh1 l jh e n s ) ((pos "L@$" ) ))
( "refulgent"  j ( r i1 f uh1 l jh e n t ) ((pos "OA$" ) ))
( "refund"  n ( r ii1 f uh n d ) ((pos "M6%" ) ))
( "refund"  v ( r i1 f uh1 n d ) ((pos "H0%" ) (subcat )))
( "refunded"  v ( r i1 f uh1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refunding"  v ( r i1 f uh1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "refunds"  n ( r ii1 f uh n d z ) ((pos "Mj%" ) ))
( "refunds"  v ( r i1 f uh1 n d z ) ((pos "Ha%" ) (subcat )))
( "refurbish"  v ( r ii2 f @@1 b i sh ) ((pos "H1%" ) (subcat )))
( "refurbished"  v ( r ii2 f @@1 b i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refurbishes"  v ( r ii2 f @@1 b i sh i z ) ((pos "Ha%" ) (subcat )))
( "refurbishing"  v ( r ii2 f @@1 b i sh i ng ) ((pos "Hb%" ) (subcat )))
( "refurnish"  v ( r ii1 f @@1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "refurnished"  v ( r ii1 f @@1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "refurnishes"  v ( r ii1 f @@1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "refurnishing"  v ( r ii1 f @@1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "refusal"  n ( r i1 f y uu1 z l ) ((pos "M6%" ) ))
( "refusals"  n ( r i1 f y uu1 z l z ) ((pos "Mj%" ) ))
( "refuse"  n ( r e1 f y uu s ) ((pos "L@%" ) ))
( "refuse"  v ( r i1 f y uu1 z ) ((pos "J2%" ) (subcat "2A" "6A" "7A" )))
( "refuse-collector"  n ( r e1 f y uu s - k @ l e2 k t @ r ) ((pos "K6%" ) ))
( "refuse-collectors"  n ( r e1 f y uu s - k @ l e2 k t @ z ) ((pos "Kj%" ) ))
( "refused"  v ( r i1 f y uu1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "7A" )))
( "refuses"  v ( r i1 f y uu1 z i z ) ((pos "Ja%" ) (subcat "2A" "6A" "7A" )))
( "refusing"  v ( r i1 f y uu1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "7A" )))
( "refutable"  j ( r i1 f y uu1 t @ b l ) ((pos "OA%" ) ))
( "refutation"  n ( r e2 f y u t ei1 sh @ n ) ((pos "M6%" ) ))
( "refutations"  n ( r e2 f y u t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "refute"  v ( r i1 f y uu1 t ) ((pos "H2%" ) (subcat )))
( "refuted"  v ( r i1 f y uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "refutes"  v ( r i1 f y uu1 t s ) ((pos "Ha%" ) (subcat )))
( "refuting"  v ( r i1 f y uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "regain"  v ( r i1 g ei1 n ) ((pos "H0%" ) (subcat )))
( "regained"  v ( r i1 g ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "regaining"  v ( r i1 g ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "regains"  v ( r i1 g ei1 n z ) ((pos "Ha%" ) (subcat )))
( "regal"  j ( r ii1 g @ l ) ((pos "OA%" ) ))
( "regale"  v ( r i1 g ei1 l ) ((pos "H2%" ) (subcat "6A" )))
( "regaled"  v ( r i1 g ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "regales"  v ( r i1 g ei1 l z ) ((pos "Ha%" ) (subcat "6A" )))
( "regalia"  n ( r i1 g ei1 l i@ ) ((pos "Kj%" ) ))
( "regaling"  v ( r i1 g ei1 l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "regally"  a ( r ii1 g @ l ii ) ((pos "Pu%" ) ))
( "regard"  v ( r i1 g aa1 d ) ((pos "H0%" "M6%" ) (subcat "6A" "14" )))
( "regarded"  v ( r i1 g aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "regardful"  j ( r i1 g aa1 d f @ l ) ((pos "OA%" ) ))
( "regarding"  v ( r i1 g aa1 d i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "regardless"  j ( r i1 g aa1 d l @ s ) ((pos "OA%" ) ))
( "regards"  v ( r i1 g aa1 d z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "14" )))
( "regatta"  n ( r i1 g a1 t @ ) ((pos "K6%" ) ))
( "regattas"  n ( r i1 g a1 t @ z ) ((pos "Kj%" ) ))
( "regencies"  n ( r ii1 jh @ n s i z ) ((pos "Kj$" ) ))
( "regency"  n ( r ii1 jh @ n s ii ) ((pos "K8$" ) ))
( "regenerate"  j ( r i1 jh e1 n @ r @ t ) ((pos "OA%" ) ))
( "regenerate"  v ( r i1 jh e1 n @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "regenerated"  v ( r i1 jh e1 n @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "regenerates"  v ( r i1 jh e1 n @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "regenerating"  v ( r i1 jh e1 n @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "regeneration"  n ( r i1 jh e2 n @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "regent"  n ( r ii1 jh @ n t ) ((pos "K6$" "OA$" ) ))
( "regents"  n ( r ii1 jh @ n t s ) ((pos "Kj$" ) ))
( "reggae"  n ( r e1 g ei ) ((pos "L@%" ) ))
( "regicide"  n ( r e1 jh i s ai d ) ((pos "M6%" ) ))
( "regicides"  n ( r e1 jh i s ai d z ) ((pos "Mj%" ) ))
( "regime"  n ( r ei1 zh ii1 m ) ((pos "K6%" ) ))
( "regimen"  n ( r e1 jh i m @ n ) ((pos "K6%" ) ))
( "regimens"  n ( r e1 jh i m @ n z ) ((pos "Kj$" ) ))
( "regiment"  v ( r e1 jh i m @ n t ) ((pos "H0%" "K6%" ) (subcat )))
( "regimental"  j ( r e2 jh i m e1 n t l ) ((pos "OA%" ) ))
( "regimentals"  n ( r e2 jh i m e1 n t l z ) ((pos "Kj$" ) ))
( "regimentation"  n ( r e2 jh i m e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "regimented"  v ( r e1 jh i m @ n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "regimenting"  v ( r e1 jh i m @ n t i ng ) ((pos "Hb%" ) (subcat )))
( "regiments"  v ( r e1 jh i m @ n t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "regimes"  n ( r ei1 zh ii1 m z ) ((pos "Kj%" ) ))
( "region"  n ( r ii1 jh @ n ) ((pos "K6%" ) ))
( "regional"  j ( r ii1 jh @ n l ) ((pos "OA%" ) ))
( "regionally"  a ( r ii1 jh @ n @ l ii ) ((pos "Pu%" ) ))
( "regions"  n ( r ii1 jh @ n z ) ((pos "Kj%" ) ))
( "register"  v ( r e1 jh i s t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "registered"  v ( r e1 jh i s t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "registering"  v ( r e1 jh i s t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "registers"  v ( r e1 jh i s t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "registrar"  n ( r e2 jh i s t r aa1 r ) ((pos "K6%" ) ))
( "registrars"  n ( r e2 jh i s t r aa1 z ) ((pos "Kj%" ) ))
( "registration"  n ( r e2 jh i s t r ei1 sh @ n ) ((pos "M6%" ) ))
( "registrations"  n ( r e2 jh i s t r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "registries"  n ( r e1 jh i s t r i z ) ((pos "Mj%" ) ))
( "registry"  n ( r e1 jh i s t r ii ) ((pos "M8%" ) ))
( "regnant"  j ( r e1 g n @ n t ) ((pos "OA$" ) ))
( "regress"  v ( r i1 g r e1 s ) ((pos "I1%" ) (subcat )))
( "regressed"  v ( r i1 g r e1 s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "regresses"  v ( r i1 g r e1 s i z ) ((pos "Ia%" ) (subcat )))
( "regressing"  v ( r i1 g r e1 s i ng ) ((pos "Ib%" ) (subcat )))
( "regression"  n ( r i1 g r e1 sh @ n ) ((pos "M6%" ) ))
( "regressions"  n ( r i1 g r e1 sh @ n z ) ((pos "Mj%" ) ))
( "regressive"  j ( r i1 g r e1 s i v ) ((pos "OA%" ) ))
( "regret"  v ( r i1 g r e1 t ) ((pos "H4%" "M6%" ) (subcat "6A" "6D" "7A" )))
( "regretful"  j ( r i1 g r e1 t f @ l ) ((pos "OA%" ) ))
( "regretfully"  a ( r i1 g r e1 t f @ l ii ) ((pos "Pu%" ) ))
( "regrets"  v ( r i1 g r e1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6D" "7A" )))
( "regrettable"  j ( r i1 g r e1 t @ b l ) ((pos "OA%" ) ))
( "regrettably"  a ( r i1 g r e1 t @ b l ii ) ((pos "Pu%" ) ))
( "regretted"  v ( r i1 g r e1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "7A" )))
( "regretting"  v ( r i1 g r e1 t i ng ) ((pos "Hb%" ) (subcat "6A" "6D" "7A" )))
( "regroup"  v ( r ii2 g r uu1 p ) ((pos "J0%" ) (subcat "2A" )))
( "regrouped"  v ( r ii2 g r uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "regrouping"  v ( r ii2 g r uu1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "regroups"  v ( r ii2 g r uu1 p s ) ((pos "Ja%" ) (subcat "2A" )))
( "regular"  n ( r e1 g y u l @ r ) ((pos "K6%" "OA%" ) ))
( "regularities"  n ( r e2 g y u l a1 r i t i z ) ((pos "Mj%" ) ))
( "regularity"  n ( r e2 g y u l a1 r i t ii ) ((pos "M8%" ) ))
( "regularization"  n ( r e2 g y u l @ r ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "regularizations"  n ( r e2 g y u l @ r ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "regularize"  v ( r e1 g y u l @ r ai z ) ((pos "H2%" ) (subcat )))
( "regularized"  v ( r e1 g y u l @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "regularizes"  v ( r e1 g y u l @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "regularizing"  v ( r e1 g y u l @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "regularly"  a ( r e1 g y u l @ l ii ) ((pos "Pu%" ) ))
( "regulars"  n ( r e1 g y u l @ z ) ((pos "Kj%" ) ))
( "regulate"  v ( r e1 g y u l ei t ) ((pos "H2%" ) (subcat )))
( "regulated"  v ( r e1 g y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "regulates"  v ( r e1 g y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "regulating"  v ( r e1 g y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "regulation"  n ( r e2 g y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "regulations"  n ( r e2 g y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "regulator"  n ( r e1 g y u l ei t @ r ) ((pos "K6%" ) ))
( "regulators"  n ( r e1 g y u l ei t @ z ) ((pos "Kj%" ) ))
( "regurgitate"  v ( r ii1 g @@1 jh i t ei t ) ((pos "J2%" ) (subcat "2A" )))
( "regurgitated"  v ( r ii1 g @@1 jh i t ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "regurgitates"  v ( r ii1 g @@1 jh i t ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "regurgitating"  v ( r ii1 g @@1 jh i t ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rehabilitate"  v ( r ii2 @ b i1 l i t ei t ) ((pos "H2%" ) (subcat )))
( "rehabilitated"  v ( r ii2 @ b i1 l i t ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rehabilitates"  v ( r ii2 @ b i1 l i t ei t s ) ((pos "Ha%" ) (subcat )))
( "rehabilitating"  v ( r ii2 @ b i1 l i t ei t i ng ) ((pos "Hb%" ) (subcat )))
( "rehabilitation"  n ( r ii2 @ b i2 l i t ei1 sh @ n ) ((pos "M6%" ) ))
( "rehabilitations"  n ( r ii2 @ b i2 l i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "rehash"  n ( r ii1 h a sh ) ((pos "K7%" ) ))
( "rehash"  v ( r ii2 h a1 sh ) ((pos "H1%" ) ))
( "rehashed"  v ( r ii2 h a1 sh t ) ((pos "Hc%" "Hd%" ) ))
( "rehashes"  n ( r ii1 h a sh i z ) ((pos "Kj$" ) ))
( "rehashes"  v ( r ii2 h a1 sh i z ) ((pos "Ha$" ) ))
( "rehashing"  v ( r ii2 h a1 sh i ng ) ((pos "Hb%" ) ))
( "rehear"  v ( r ii2 h i@1 r ) ((pos "H5$" ) (subcat )))
( "reheard"  v ( r ii2 h @@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rehearing"  v ( r ii2 h i@1 r i ng ) ((pos "Hb$" "K6$" ) (subcat )))
( "rehearings"  n ( r ii2 h i@1 r i ng z ) ((pos "Kj$" ) ))
( "rehears"  v ( r ii2 h i@1 z ) ((pos "Ha$" ) (subcat )))
( "rehearsal"  n ( r i1 h @@1 s l ) ((pos "M6%" ) ))
( "rehearsals"  n ( r i1 h @@1 s l z ) ((pos "Mj%" ) ))
( "rehearse"  v ( r i1 h @@1 s ) ((pos "J2%" ) (subcat "2A" )))
( "rehearsed"  v ( r i1 h @@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rehearses"  v ( r i1 h @@1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "rehearsing"  v ( r i1 h @@1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rehouse"  v ( r ii2 h au1 z ) ((pos "H2%" ) (subcat )))
( "rehoused"  v ( r ii2 h au1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rehouses"  v ( r ii2 h au1 z i z ) ((pos "Ha%" ) (subcat )))
( "rehousing"  v ( r ii2 h au1 z i ng ) ((pos "Hb%" ) (subcat )))
( "reign"  v ( r ei1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "reigned"  v ( r ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "reigning"  v ( r ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "reigns"  v ( r ei1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "reimburse"  v ( r ii2 i m b @@1 s ) ((pos "H2%" ) (subcat "6A" "12A" "13A" )))
( "reimbursed"  v ( r ii2 i m b @@1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" )))
( "reimbursement"  n ( r ii2 i m b @@1 s m @ n t ) ((pos "M6%" ) ))
( "reimbursements"  n ( r ii2 i m b @@1 s m @ n t s ) ((pos "Mj%" ) ))
( "reimburses"  v ( r ii2 i m b @@1 s i z ) ((pos "Ha%" ) (subcat "6A" "12A" "13A" )))
( "reimbursing"  v ( r ii2 i m b @@1 s i ng ) ((pos "Hb%" ) (subcat "6A" "12A" "13A" )))
( "reimposition"  n ( r ii2 i m p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "reimpositions"  n ( r ii2 i m p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "rein"  v ( r ei1 n ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "reincarnate"  j ( r ii2 i n k aa1 n @ t ) ((pos "OA%" ) ))
( "reincarnate"  v ( r ii2 i n k aa1 n ei t ) ((pos "H2$" ) ))
( "reincarnated"  v ( r ii2 i n k aa1 n ei t i d ) ((pos "Hc$" "Hd%" ) ))
( "reincarnates"  v ( r ii2 i n k aa1 n ei t s ) ((pos "Ha$" ) ))
( "reincarnating"  v ( r ii2 i n k aa n ei1 t i ng ) ((pos "Hb$" ) ))
( "reincarnation"  n ( r ii2 i n k aa n ei1 sh @ n ) ((pos "M6%" ) ))
( "reincarnations"  n ( r ii2 i n k aa n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "reindeer"  n ( r ei1 n d i@ r ) ((pos "K9%" ) ))
( "reined"  v ( r ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reinforce"  v ( r ii2 i n f oo1 s ) ((pos "H2%" ) (subcat )))
( "reinforced"  v ( r ii2 i n f oo1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reinforcement"  n ( r ii2 i n f oo1 s m @ n t ) ((pos "M6%" ) ))
( "reinforcements"  n ( r ii2 i n f oo1 s m @ n t s ) ((pos "Mj%" ) ))
( "reinforces"  v ( r ii2 i n f oo1 s i z ) ((pos "Ha%" ) (subcat )))
( "reinforcing"  v ( r ii2 i n f oo1 s i ng ) ((pos "Hb%" ) (subcat )))
( "reining"  v ( r ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reins"  v ( r ei1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "reinstate"  v ( r ii2 i n s t ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "reinstated"  v ( r ii2 i n s t ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reinstatement"  n ( r ii2 i n s t ei1 t m @ n t ) ((pos "K6%" ) ))
( "reinstatements"  n ( r ii2 i n s t ei1 t m @ n t s ) ((pos "Kj%" ) ))
( "reinstates"  v ( r ii2 i n s t ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "reinstating"  v ( r ii2 i n s t ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reinsurance"  n ( r ii2 i n sh uu1 r @ n s ) ((pos "M6%" ) ))
( "reinsurances"  n ( r ii2 i n sh uu1 r @ n s i z ) ((pos "Mj$" ) ))
( "reinsure"  v ( r ii2 i n sh u@1 r ) ((pos "H2%" ) (subcat )))
( "reinsured"  v ( r ii2 i n sh u@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reinsures"  v ( r ii2 i n sh u@1 z ) ((pos "Ha%" ) (subcat )))
( "reinsuring"  v ( r ii2 i n sh u@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "reintegrate"  v ( r ii1 i1 n t i g r ei t ) ((pos "H2%" ) (subcat "2A" )))
( "reintegrated"  v ( r ii1 i1 n t i g r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "reintegrates"  v ( r ii1 i1 n t i g r ei t s ) ((pos "Ha%" ) (subcat "2A" )))
( "reintegrating"  v ( r ii1 i1 n t i g r ei t i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "reinterpret"  v ( r ii2 i n t @@1 p r i t ) ((pos "H0%" ) (subcat "2A" "6A" )))
( "reinterpretation"  n ( r ii2 i n t @@2 p r i t ei1 sh @ n ) ((pos "M6%" ) ))
( "reinterpretations"  n ( r ii2 i n t @@2 p r i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reinterpreted"  v ( r ii2 i n t @@1 p r i t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "reinterpreting"  v ( r ii2 i n t @@1 p r i t i ng ) ((pos "Hb%" ) (subcat "2A" "6A" )))
( "reinterprets"  v ( r ii2 i n t @@1 p r i t s ) ((pos "Ha%" ) (subcat "2A" "6A" )))
( "reissue"  v ( r ii2 i1 sh uu ) ((pos "H2%" "K6%" ) (subcat )))
( "reissued"  v ( r ii2 i1 sh uu d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reissues"  v ( r ii2 i1 sh uu z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "reissuing"  v ( r ii2 i1 sh uu i ng ) ((pos "Hb%" ) (subcat )))
( "reiterate"  v ( r ii1 i1 t @ r ei t ) ((pos "H2%" ) (subcat )))
( "reiterated"  v ( r ii1 i1 t @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reiterates"  v ( r ii1 i1 t @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "reiterating"  v ( r ii1 i1 t @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "reiteration"  n ( r ii1 i2 t @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "reiterations"  n ( r ii1 i2 t @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reject"  n ( r ii1 jh e k t ) ((pos "K6%" ) ))
( "reject"  v ( r i1 jh e1 k t ) ((pos "H0%" ) (subcat )))
( "rejected"  v ( r i1 jh e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rejecting"  v ( r i1 jh e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "rejection"  n ( r i1 jh e1 k sh @ n ) ((pos "M6%" ) ))
( "rejections"  n ( r i1 jh e1 k sh @ n z ) ((pos "Mj%" ) ))
( "rejects"  n ( r ii1 jh e k t s ) ((pos "Kj%" ) ))
( "rejects"  v ( r i1 jh e1 k t s ) ((pos "Ha%" ) (subcat )))
( "rejig"  v ( r ii2 jh i1 g ) ((pos "H4$" ) (subcat )))
( "rejigged"  v ( r ii2 jh i1 g d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rejigging"  v ( r ii2 jh i1 g i ng ) ((pos "Hb$" ) (subcat )))
( "rejigs"  v ( r ii2 jh i1 g z ) ((pos "Ha$" ) (subcat )))
( "rejoice"  v ( r i1 jh oi1 s ) ((pos "J2%" ) (subcat "2A" "2C" "3B" "4C" )))
( "rejoiced"  v ( r i1 jh oi1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3B" "4C" )))
( "rejoices"  v ( r i1 jh oi1 s i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3B" "4C" )))
( "rejoicing"  v ( r i1 jh oi1 s i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3B" "4C" )))
( "rejoin"  v ( r ii2 jh oi1 n ) ((pos "H0%" ) (subcat )))
( "rejoin"  v ( r i1 jh oi1 n ) ((pos "J0%" ) (subcat "2A" )))
( "rejoinder"  n ( r i1 jh oi1 n d @ r ) ((pos "K6%" ) ))
( "rejoinders"  n ( r i1 jh oi1 n d @ z ) ((pos "Kj%" ) ))
( "rejoined"  v ( r ii2 jh oi1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rejoined"  v ( r i1 jh oi1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rejoining"  v ( r ii2 jh oi1 n i ng ) ((pos "Hb%" ) (subcat )))
( "rejoining"  v ( r i1 jh oi1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rejoins"  v ( r ii2 jh oi1 n z ) ((pos "Ha%" ) (subcat )))
( "rejoins"  v ( r i1 jh oi1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "rejuvenate"  v ( r ii1 jh uu1 v @ n ei t ) ((pos "J2%" ) (subcat "2A" )))
( "rejuvenated"  v ( r ii1 jh uu1 v @ n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rejuvenates"  v ( r ii1 jh uu1 v @ n ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "rejuvenating"  v ( r ii1 jh uu1 v @ n ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rejuvenation"  n ( r i1 jh uu2 v @ n ei1 sh @ n ) ((pos "K6%" ) ))
( "rejuvenations"  n ( r i1 jh uu2 v @ n ei1 sh @ n z ) ((pos "Kj$" ) ))
( "rekindle"  v ( r ii2 k i1 n d l ) ((pos "J2%" ) (subcat "2A" )))
( "rekindled"  v ( r ii2 k i1 n d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rekindles"  v ( r ii2 k i1 n d l z ) ((pos "Ja%" ) (subcat "2A" )))
( "rekindling"  v ( r ii2 k i1 n d l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "relaid"  v ( r ii2 l ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "relapse"  v ( r i1 l a1 p s ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "relapsed"  v ( r i1 l a1 p s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "relapses"  v ( r i1 l a1 p s i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "relapsing"  v ( r i1 l a1 p s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "relate"  v ( r i1 l ei1 t ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "related"  v ( r i1 l ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "relates"  v ( r i1 l ei1 t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "relating"  v ( r i1 l ei1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "relation"  n ( r i1 l ei1 sh @ n ) ((pos "M6%" ) ))
( "relations"  n ( r i1 l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "relationship"  n ( r i1 l ei1 sh @ n sh i p ) ((pos "M6%" ) ))
( "relationships"  n ( r i1 l ei1 sh @ n sh i p s ) ((pos "Mj%" ) ))
( "relative"  n ( r e1 l @ t i v ) ((pos "K6%" "OA%" ) ))
( "relatively"  a ( r e1 l @ t i v l ii ) ((pos "Pu%" ) ))
( "relatives"  n ( r e1 l @ t i v z ) ((pos "Kj%" ) ))
( "relativity"  n ( r e2 l @ t i1 v i t ii ) ((pos "L@%" ) ))
( "relax"  v ( r i1 l a1 k s ) ((pos "J1%" ) (subcat "2A" "2C" )))
( "relaxation"  n ( r ii2 l a k s ei1 sh @ n ) ((pos "M6%" ) ))
( "relaxations"  n ( r ii2 l a k s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "relaxed"  v ( r i1 l a1 k s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "relaxes"  v ( r i1 l a1 k s i z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "relaxing"  v ( r i1 l a1 k s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "relay"  n ( r ii1 l ei ) ((pos "K6%" ) ))
( "relay"  v ( r ii2 l ei1 ) ((pos "H5$" ) ))
( "relay"  v ( r i1 l ei1 ) ((pos "H0%" ) ))
( "relayed"  v ( r i1 l ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "relaying"  v ( r ii2 l ei1 i ng ) ((pos "Hb$" ) ))
( "relaying"  v ( r i1 l ei1 i ng ) ((pos "Hb%" ) ))
( "relays"  n ( r ii1 l ei z ) ((pos "Kj%" ) ))
( "relays"  v ( r ii2 l ei1 z ) ((pos "Ha$" ) ))
( "relays"  v ( r i1 l ei1 z ) ((pos "Ha%" ) ))
( "release"  v ( r i1 l ii1 s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "released"  v ( r i1 l ii1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "releases"  v ( r i1 l ii1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "releasing"  v ( r i1 l ii1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "relegate"  v ( r e1 l i g ei t ) ((pos "H2%" ) (subcat )))
( "relegated"  v ( r e1 l i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "relegates"  v ( r e1 l i g ei t s ) ((pos "Ha%" ) (subcat )))
( "relegating"  v ( r e1 l i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "relegation"  n ( r e2 l i g ei1 sh @ n ) ((pos "L@%" ) ))
( "relent"  v ( r i1 l e1 n t ) ((pos "I0%" ) (subcat )))
( "relented"  v ( r i1 l e1 n t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "relenting"  v ( r i1 l e1 n t i ng ) ((pos "Ib%" ) (subcat )))
( "relentless"  j ( r i1 l e1 n t l @ s ) ((pos "OA%" ) ))
( "relentlessly"  a ( r i1 l e1 n t l @ s l ii ) ((pos "Pu%" ) ))
( "relents"  v ( r i1 l e1 n t s ) ((pos "Ia%" ) (subcat )))
( "relevance"  n ( r e1 l @ v @ n s ) ((pos "K6%" ) ))
( "relevancy"  n ( r e1 l @ v @ n s ii ) ((pos "L@$" ) ))
( "relevant"  j ( r e1 l @ v @ n t ) ((pos "OA%" ) ))
( "relevantly"  a ( r e1 l @ v @ n t l ii ) ((pos "Pu%" ) ))
( "reliability"  n ( r i1 l ai2 @ b i1 l i t ii ) ((pos "L@%" ) ))
( "reliable"  j ( r i1 l ai1 @ b l ) ((pos "OA%" ) ))
( "reliably"  a ( r i1 l ai1 @ b l ii ) ((pos "Pu%" ) ))
( "reliance"  n ( r i1 l ai1 @ n s ) ((pos "L@%" ) ))
( "reliant"  j ( r i1 l ai1 @ n t ) ((pos "OA%" ) ))
( "relic"  n ( r e1 l i k ) ((pos "K6%" ) ))
( "relics"  n ( r e1 l i k s ) ((pos "Kj%" ) ))
( "relict"  n ( r e1 l i k t ) ((pos "K6$" ) ))
( "relicts"  n ( r e1 l i k t s ) ((pos "Kj$" ) ))
( "relied"  v ( r i1 l ai1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "relief"  n ( r i1 l ii1 f ) ((pos "M6%" ) ))
( "reliefs"  n ( r i1 l ii1 f s ) ((pos "Mj%" ) ))
( "relies"  v ( r i1 l ai1 z ) ((pos "Ia%" ) (subcat )))
( "relieve"  v ( r i1 l ii1 v ) ((pos "H2%" ) (subcat )))
( "relieved"  v ( r i1 l ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "relieves"  v ( r i1 l ii1 v z ) ((pos "Ha%" ) (subcat )))
( "relieving"  v ( r i1 l ii1 v i ng ) ((pos "Hb%" ) (subcat )))
( "religion"  n ( r i1 l i1 jh @ n ) ((pos "M6%" ) ))
( "religions"  n ( r i1 l i1 jh @ n z ) ((pos "Mj%" ) ))
( "religious"  n ( r i1 l i1 jh @ s ) ((pos "K9%" "OA%" ) ))
( "religiously"  a ( r i1 l i1 jh @ s l ii ) ((pos "Pu%" ) ))
( "reline"  v ( r ii2 l ai1 n ) ((pos "H2%" ) ))
( "relined"  v ( r ii2 l ai1 n d ) ((pos "Hc%" "Hd%" ) ))
( "relines"  v ( r ii2 l ai1 n z ) ((pos "Ha%" ) ))
( "relining"  v ( r ii2 l ai1 n i ng ) ((pos "Hb%" ) ))
( "relinquish"  v ( r i1 l i1 ng k w i sh ) ((pos "H1%" ) (subcat "6A" )))
( "relinquished"  v ( r i1 l i1 ng k w i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "relinquishes"  v ( r i1 l i1 ng k w i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "relinquishing"  v ( r i1 l i1 ng k w i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reliquaries"  n ( r e1 l i k w @ r i z ) ((pos "Kj$" ) ))
( "reliquary"  n ( r e1 l i k w @ r ii ) ((pos "K8$" ) ))
( "relish"  v ( r e1 l i sh ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "relished"  v ( r e1 l i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "relishes"  v ( r e1 l i sh i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "relishing"  v ( r e1 l i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "relive"  v ( r ii2 l i1 v ) ((pos "H2%" ) ))
( "relived"  v ( r ii2 l i1 v d ) ((pos "Hc%" "Hd%" ) ))
( "relives"  v ( r ii2 l i1 v z ) ((pos "Ha%" ) ))
( "reliving"  v ( r ii2 l i1 v i ng ) ((pos "Hb%" ) ))
( "relocate"  v ( r ii2 l ou k ei1 t ) ((pos "J2%" ) ))
( "relocated"  v ( r ii2 l ou k ei1 t i d ) ((pos "Jc%" "Jd%" ) ))
( "relocates"  v ( r ii2 l ou k ei1 t s ) ((pos "Ja%" ) ))
( "relocating"  v ( r ii2 l ou k ei1 t i ng ) ((pos "Jb%" ) ))
( "relocation"  n ( r ii2 l ou k ei1 sh @ n ) ((pos "L@%" ) ))
( "reluctance"  n ( r i1 l uh1 k t @ n s ) ((pos "L@%" ) ))
( "reluctant"  j ( r i1 l uh1 k t @ n t ) ((pos "OA%" ) ))
( "reluctantly"  a ( r i1 l uh1 k t @ n t l ii ) ((pos "Pu%" ) ))
( "rely"  v ( r i1 l ai1 ) ((pos "I3%" ) (subcat )))
( "relying"  v ( r i1 l ai1 i ng ) ((pos "Ib%" ) (subcat )))
( "remade"  v ( r ii2 m ei1 d ) ((pos "Hc%" "Hd%" ) ))
( "remain"  v ( r i1 m ei1 n ) ((pos "I0%" ) (subcat "2A" "2B" "2C" )))
( "remainder"  n ( r i1 m ei1 n d @ r ) ((pos "K6%" ) ))
( "remainders"  n ( r i1 m ei1 n d @ z ) ((pos "Kj%" ) ))
( "remained"  v ( r i1 m ei1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" )))
( "remaining"  v ( r i1 m ei1 n i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" )))
( "remains"  v ( r i1 m ei1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "remake"  n ( r ii1 m ei k ) ((pos "K6%" ) ))
( "remake"  v ( r ii2 m ei1 k ) ((pos "H5%" ) ))
( "remakes"  n ( r ii1 m ei k s ) ((pos "Kj%" ) ))
( "remakes"  v ( r ii2 m ei1 k s ) ((pos "Ha%" ) ))
( "remaking"  v ( r ii2 m ei1 k i ng ) ((pos "Hb%" ) ))
( "remand"  v ( r i1 m aa1 n d ) ((pos "H0%" "L@%" ) (subcat )))
( "remanded"  v ( r i1 m aa1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "remanding"  v ( r i1 m aa1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "remands"  v ( r i1 m aa1 n d z ) ((pos "Ha%" ) (subcat )))
( "remark"  v ( r i1 m aa1 k ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" "9" )))
( "remarkable"  j ( r i1 m aa1 k @ b l ) ((pos "OA%" ) ))
( "remarkably"  a ( r i1 m aa1 k @ b l ii ) ((pos "Pu%" ) ))
( "remarked"  v ( r i1 m aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" )))
( "remarking"  v ( r i1 m aa1 k i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "9" )))
( "remarks"  v ( r i1 m aa1 k s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "9" )))
( "remarriage"  n ( r ii2 m a1 r i jh ) ((pos "K6%" ) ))
( "remarriages"  n ( r ii2 m a1 r i jh i z ) ((pos "Kj%" ) ))
( "remarried"  v ( r ii2 m a1 r i d ) ((pos "Jc%" "Jd%" ) ))
( "remarries"  v ( r ii2 m a1 r i z ) ((pos "Ja%" ) ))
( "remarry"  v ( r ii2 m a1 r ii ) ((pos "J3%" ) ))
( "remarrying"  v ( r ii2 m a1 r i i ng ) ((pos "Jb%" ) ))
( "remediable"  j ( r i1 m ii1 d i@ b l ) ((pos "OA%" ) ))
( "remedial"  j ( r i1 m ii1 d i@ l ) ((pos "OA%" ) ))
( "remedied"  v ( r e1 m @ d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "remedies"  v ( r e1 m @ d i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "remedy"  v ( r e1 m @ d ii ) ((pos "H3%" "M8%" ) (subcat )))
( "remedying"  v ( r e1 m @ d i i ng ) ((pos "Hb%" ) (subcat )))
( "remember"  v ( r i1 m e1 m b @ r ) ((pos "J0%" ) (subcat "6A" "6C" "7A" "8" "9" "10" "14" "16B" )))
( "remembered"  v ( r i1 m e1 m b @ d ) ((pos "Jc%" "Jd%" ) (subcat "6A" "6C" "7A" "8" "9" "10" "14" "16B" )))
( "remembering"  v ( r i1 m e1 m b @ r i ng ) ((pos "Jb%" ) (subcat "6A" "6C" "7A" "8" "9" "10" "14" "16B" )))
( "remembers"  v ( r i1 m e1 m b @ z ) ((pos "Ja%" ) (subcat "6A" "6C" "7A" "8" "9" "10" "14" "16B" )))
( "remembrance"  n ( r i1 m e1 m b r @ n s ) ((pos "M6%" ) ))
( "remembrances"  n ( r i1 m e1 m b r @ n s i z ) ((pos "Mj%" ) ))
( "remilitarization"  n ( r ii2 m i2 l i t @ r ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "remilitarizations"  n ( r ii2 m i2 l i t @ r ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "remilitarize"  v ( r ii2 m i1 l i t @ r ai z ) ((pos "H2%" ) (subcat )))
( "remilitarized"  v ( r ii2 m i1 l i t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "remilitarizes"  v ( r ii2 m i1 l i t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "remilitarizing"  v ( r ii2 m i1 l i t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "remind"  v ( r i1 m ai1 n d ) ((pos "H0%" ) (subcat "6A" "11" "14" "17" "20" )))
( "reminded"  v ( r i1 m ai1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "14" "17" "20" )))
( "reminder"  n ( r i1 m ai1 n d @ r ) ((pos "K6%" ) ))
( "reminders"  n ( r i1 m ai1 n d @ z ) ((pos "Kj%" ) ))
( "reminding"  v ( r i1 m ai1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "11" "14" "17" "20" )))
( "reminds"  v ( r i1 m ai1 n d z ) ((pos "Ha%" ) (subcat "6A" "11" "14" "17" "20" )))
( "reminisce"  v ( r e2 m i n i1 s ) ((pos "I2%" ) (subcat "2A" )))
( "reminisced"  v ( r e2 m i n i1 s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "reminiscence"  n ( r e2 m i n i1 s n s ) ((pos "M6%" ) ))
( "reminiscences"  n ( r e2 m i n i1 s n s i z ) ((pos "Mj%" ) ))
( "reminiscent"  j ( r e2 m i n i1 s n t ) ((pos "OA%" ) ))
( "reminiscently"  a ( r e2 m i n i1 s n t l ii ) ((pos "Pu%" ) ))
( "reminisces"  v ( r e2 m i n i1 s i z ) ((pos "Ia%" ) (subcat "2A" )))
( "reminiscing"  v ( r e2 m i n i1 s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "remiss"  j ( r i1 m i1 s ) ((pos "OA%" ) ))
( "remission"  n ( r i1 m i1 sh @ n ) ((pos "M6%" ) ))
( "remissions"  n ( r i1 m i1 sh @ n z ) ((pos "Mj%" ) ))
( "remissness"  n ( r i1 m i1 s n @ s ) ((pos "L@%" ) ))
( "remit"  v ( r i1 m i1 t ) ((pos "J4%" ) (subcat "2C" "6A" "12A" "13A" )))
( "remits"  v ( r i1 m i1 t s ) ((pos "Ja%" ) (subcat "2C" "6A" "12A" "13A" )))
( "remittance"  n ( r i1 m i1 t n s ) ((pos "M6%" ) ))
( "remittances"  n ( r i1 m i1 t n s i z ) ((pos "Mj%" ) ))
( "remitted"  v ( r i1 m i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "12A" "13A" )))
( "remittent"  j ( r i1 m i1 t n t ) ((pos "OA$" ) ))
( "remitting"  v ( r i1 m i1 t i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "12A" "13A" )))
( "remnant"  n ( r e1 m n @ n t ) ((pos "K6%" ) ))
( "remnants"  n ( r e1 m n @ n t s ) ((pos "Kj%" ) ))
( "remodel"  v ( r ii1 m o1 d l ) ((pos "H4%" ) (subcat "2A" "6A" "14" )))
( "remodelled"  v ( r ii1 m o1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" "14" )))
( "remodelling"  v ( r ii1 m o1 d @ l i ng ) ((pos "Hb%" ) (subcat "2A" "6A" "14" )))
( "remodels"  v ( r ii1 m o1 d l z ) ((pos "Ha%" ) (subcat "2A" "6A" "14" )))
( "remonstrance"  n ( r i1 m o1 n s t r @ n s ) ((pos "M6$" ) ))
( "remonstrances"  n ( r i1 m o1 n s t r @ n s i z ) ((pos "Mj$" ) ))
( "remonstrate"  v ( r e1 m @ n s t r ei t ) ((pos "I2%" ) (subcat "2A" )))
( "remonstrated"  v ( r e1 m @ n s t r ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "remonstrates"  v ( r e1 m @ n s t r ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "remonstrating"  v ( r e1 m @ n s t r ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "remorse"  n ( r i1 m oo1 s ) ((pos "L@%" ) ))
( "remorseful"  j ( r i1 m oo1 s f @ l ) ((pos "OA%" ) ))
( "remorsefully"  a ( r i1 m oo1 s f @ l ii ) ((pos "Pu%" ) ))
( "remorseless"  j ( r i1 m oo1 s l @ s ) ((pos "OA%" ) ))
( "remorselessly"  a ( r i1 m oo1 s l @ s l ii ) ((pos "Pu%" ) ))
( "remote"  j ( r i1 m ou1 t ) ((pos "OB%" ) ))
( "remotely"  a ( r i1 m ou1 t l ii ) ((pos "Pu%" ) ))
( "remoteness"  n ( r i1 m ou1 t n @ s ) ((pos "L@%" ) ))
( "remoter"  j ( r i1 m ou1 t @ r ) ((pos "Or%" ) ))
( "remotest"  j ( r i1 m ou1 t i s t ) ((pos "Os%" ) ))
( "remould"  v ( r ii1 m ou1 l d ) ((pos "H0%" ) (subcat "2A" "6A" )))
( "remoulded"  v ( r ii1 m ou1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" )))
( "remoulding"  v ( r ii1 m ou1 l d i ng ) ((pos "Hb%" ) (subcat "2A" "6A" )))
( "remoulds"  v ( r ii1 m ou1 l d z ) ((pos "Ha%" ) (subcat "2A" "6A" )))
( "remount"  n ( r ii1 m au n t ) ((pos "K6%" ) ))
( "remount"  v ( r ii2 m au1 n t ) ((pos "J0%" ) (subcat "2A" )))
( "remounted"  v ( r ii2 m au1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "remounting"  v ( r ii2 m au1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "remounts"  n ( r ii1 m au n t s ) ((pos "Kj$" ) ))
( "remounts"  v ( r ii2 m au1 n t s ) ((pos "Ja%" ) (subcat "2A" )))
( "removable"  j ( r i1 m uu1 v @ b l ) ((pos "OA%" ) ))
( "removal"  n ( r i1 m uu1 v @ l ) ((pos "M6%" ) ))
( "removals"  n ( r i1 m uu1 v @ l z ) ((pos "Mj%" ) ))
( "remove"  v ( r i1 m uu1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "removed"  v ( r i1 m uu1 v d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "remover"  n ( r i1 m uu1 v @ r ) ((pos "K6%" ) ))
( "removers"  n ( r i1 m uu1 v @ z ) ((pos "Kj%" ) ))
( "removes"  v ( r i1 m uu1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "removing"  v ( r i1 m uu1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "remunerate"  v ( r i1 m y uu1 n @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "remunerated"  v ( r i1 m y uu1 n @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "remunerates"  v ( r i1 m y uu1 n @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "remunerating"  v ( r i1 m y uu1 n @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "remuneration"  n ( r i1 m y uu2 n @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "remunerative"  j ( r i1 m y uu1 n @ r @ t i v ) ((pos "OA%" ) ))
( "renaissance"  n ( r i1 n ei1 s n s ) ((pos "K6%" ) ))
( "renaissances"  n ( r i1 n ei1 s n s i z ) ((pos "Kj$" ) ))
( "renal"  j ( r ii1 n l ) ((pos "OA%" ) ))
( "rename"  v ( r ii2 n ei1 m ) ((pos "H2%" ) (subcat )))
( "renamed"  v ( r ii2 n ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "renames"  v ( r ii2 n ei1 m z ) ((pos "Ha%" ) (subcat )))
( "renaming"  v ( r ii2 n ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "renascence"  n ( r i1 n a1 s n s ) ((pos "K6$" ) ))
( "renascences"  n ( r i1 n a1 s n s i z ) ((pos "Kj$" ) ))
( "renascent"  j ( r i1 n a1 s n t ) ((pos "OA$" ) ))
( "rend"  v ( r e1 n d ) ((pos "H5%" ) (subcat "6A" "14" )))
( "render"  v ( r e1 n d @ r ) ((pos "H0%" ) (subcat "6A" "12A" "13A" "14" "15A" "15B" )))
( "rendered"  v ( r e1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12A" "13A" "14" "15A" "15B" )))
( "rendering"  v ( r e1 n d @ r i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" "12A" "13A" "14" "15A" "15B" )))
( "renderings"  n ( r e1 n d @ r i ng z ) ((pos "Mj%" ) ))
( "renders"  v ( r e1 n d @ z ) ((pos "Ha%" ) (subcat "6A" "12A" "13A" "14" "15A" "15B" )))
( "rendezvoued"  v ( r o1 n d i v uu d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "rendezvouing"  v ( r o1 n d i v uu i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "rendezvous"  v ( r o1 n d i v uu ) ((pos "I5%" "K9%" ) (subcat "2A" )))
( "rendezvous"  v ( r o1 n d i v uu z ) ((pos "Ia$" ) (subcat "2A" )))
( "rending"  v ( r e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "rendition"  n ( r e1 n d i1 sh @ n ) ((pos "K6%" ) ))
( "renditions"  n ( r e1 n d i1 sh @ n z ) ((pos "Kj%" ) ))
( "rends"  v ( r e1 n d z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "renegade"  v ( r e1 n i g ei d ) ((pos "I2$" "K6%" ) ))
( "renegaded"  v ( r e1 n i g ei d i d ) ((pos "Ic$" "Id$" ) ))
( "renegades"  v ( r e1 n i g ei d z ) ((pos "Ia$" "Kj%" ) ))
( "renegading"  v ( r e1 n i g ei d i ng ) ((pos "Ib$" ) ))
( "renege"  v ( r i1 n ei1 g ) ((pos "I2%" ) (subcat )))
( "reneged"  v ( r i1 n ei1 g d ) ((pos "Ic%" "Id%" ) (subcat )))
( "reneges"  v ( r i1 n ei1 g z ) ((pos "Ia%" ) (subcat )))
( "reneging"  v ( r i1 n ei1 g i ng ) ((pos "Ib%" ) (subcat )))
( "renegue"  v ( r i1 n ei1 g ) ((pos "I2$" ) (subcat )))
( "renegued"  v ( r i1 n ei1 g d ) ((pos "Ic$" "Id$" ) (subcat )))
( "renegues"  v ( r i1 n ei1 g z ) ((pos "Ia$" ) (subcat )))
( "reneguing"  v ( r i1 n ei1 g i ng ) ((pos "Ib$" ) (subcat )))
( "renew"  v ( r i1 n y uu1 ) ((pos "H0%" ) (subcat )))
( "renewable"  j ( r i1 n y uu1 @ b l ) ((pos "OA%" ) ))
( "renewal"  n ( r i1 n y uu1 @ l ) ((pos "M6%" ) ))
( "renewals"  n ( r i1 n y uu1 @ l z ) ((pos "Mj%" ) ))
( "renewed"  v ( r i1 n y uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "renewing"  v ( r i1 n y uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "renews"  v ( r i1 n y uu1 z ) ((pos "Ha%" ) (subcat )))
( "rennet"  n ( r e1 n i t ) ((pos "L@%" ) ))
( "renounce"  v ( r i1 n au1 n s ) ((pos "H2%" ) (subcat )))
( "renounced"  v ( r i1 n au1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "renounces"  v ( r i1 n au1 n s i z ) ((pos "Ha%" ) (subcat )))
( "renouncing"  v ( r i1 n au1 n s i ng ) ((pos "Hb%" ) (subcat )))
( "renovate"  v ( r e1 n @ v ei t ) ((pos "H2%" ) (subcat )))
( "renovated"  v ( r e1 n @ v ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "renovates"  v ( r e1 n @ v ei t s ) ((pos "Ha%" ) (subcat )))
( "renovating"  v ( r e1 n @ v ei t i ng ) ((pos "Hb%" ) (subcat )))
( "renovation"  n ( r e2 n @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "renovations"  n ( r e2 n @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "renovator"  n ( r e1 n @ v ei t @ r ) ((pos "K6%" ) ))
( "renovators"  n ( r e1 n @ v ei t @ z ) ((pos "Kj%" ) ))
( "renown"  n ( r i1 n au1 n ) ((pos "L@%" ) ))
( "renowned"  j ( r i1 n au1 n d ) ((pos "OA%" ) ))
( "rent"  v ( r e1 n t ) ((pos "Hc%" "Hd%" "J0%" "M6%" ) (subcat "2A" "2C" "6A" "14" )))
( "rent-collector"  n ( r e1 n t - k @ l e2 k t @ r ) ((pos "K6%" ) ))
( "rent-collectors"  n ( r e1 n t - k @ l e2 k t @ z ) ((pos "Kj%" ) ))
( "rent-free"  j ( r e1 n t - f r ii1 ) ((pos "OA%" "Pu%" ) ))
( "rent-rebate"  n ( r e1 n t - r ii b ei t ) ((pos "M6%" ) ))
( "rent-rebates"  n ( r e1 n t - r ii b ei t s ) ((pos "Mj%" ) ))
( "rent-roll"  n ( r e1 n t - r ou l ) ((pos "K6$" ) ))
( "rent-rolls"  n ( r e1 n t - r ou l z ) ((pos "Kj$" ) ))
( "rentable"  j ( r e1 n t @ b l ) ((pos "OA%" ) ))
( "rental"  n ( r e1 n t l ) ((pos "K6%" ) ))
( "rentals"  n ( r e1 n t l z ) ((pos "Kj%" ) ))
( "rented"  v ( r e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "rentier"  n ( r aa1 n t i ei ) ((pos "K6$" ) ))
( "rentiers"  n ( r aa1 n t i ei z ) ((pos "Kj$" ) ))
( "renting"  v ( r e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "rents"  v ( r e1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" )))
( "renunciation"  n ( r i1 n uh2 n s i ei1 sh @ n ) ((pos "L@%" ) ))
( "reopen"  v ( r ii1 ou1 p @ n ) ((pos "J0%" ) (subcat "2A" )))
( "reopened"  v ( r ii1 ou1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reopening"  v ( r ii1 ou1 p @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reopens"  v ( r ii1 ou1 p @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "reorganization"  n ( r ii1 oo2 g @ n ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "reorganizations"  n ( r ii1 oo2 g @ n ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reorganize"  v ( r ii1 oo1 g @ n ai z ) ((pos "J2%" ) ))
( "reorganized"  v ( r ii1 oo1 g @ n ai z d ) ((pos "Jc%" "Jd%" ) ))
( "reorganizes"  v ( r ii1 oo1 g @ n ai z i z ) ((pos "Ja%" ) ))
( "reorganizing"  v ( r ii1 oo1 g @ n ai z i ng ) ((pos "Jb%" ) ))
( "reorient"  v ( r ii1 oo1 r i@ n t ) ((pos "J0%" ) ))
( "reorientate"  v ( r ii1 oo1 r i@ n t ei t ) ((pos "J2%" ) ))
( "reorientated"  v ( r ii1 oo1 r i@ n t ei t i d ) ((pos "Jc%" "Jd%" ) ))
( "reorientates"  v ( r ii1 oo1 r i@ n t ei t s ) ((pos "Ja%" ) ))
( "reorientating"  v ( r ii1 oo1 r i@ n t ei t i ng ) ((pos "Jb%" ) ))
( "reorientation"  n ( r ii1 oo2 r i e n t ei1 sh @ n ) ((pos "L@%" ) ))
( "reoriented"  v ( r ii1 oo1 r i@ n t i d ) ((pos "Jc%" "Jd%" ) ))
( "reorienting"  v ( r ii1 oo1 r i@ n t i ng ) ((pos "Jb%" ) ))
( "reorients"  v ( r ii1 oo1 r i@ n t s ) ((pos "Ja%" ) ))
( "rep"  n ( r e1 p ) ((pos "M6%" ) ))
( "repaid"  v ( r i1 p ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "repaint"  v ( r ii1 p ei1 n t ) ((pos "H0%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "repainted"  v ( r ii1 p ei1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "repainting"  v ( r ii1 p ei1 n t i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "repaints"  v ( r ii1 p ei1 n t s ) ((pos "Ha%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "repair"  v ( r i1 p e@1 r ) ((pos "J0%" "M6%" ) (subcat "3A" )))
( "repairable"  j ( r i1 p e@1 r @ b l ) ((pos "OA%" ) ))
( "repaired"  v ( r i1 p e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "repairer"  n ( r i1 p e@1 r @ r ) ((pos "K6%" ) ))
( "repairers"  n ( r i1 p e@1 r @ z ) ((pos "Kj%" ) ))
( "repairing"  v ( r i1 p e@1 r i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "repairs"  v ( r i1 p e@1 z ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "reparable"  j ( r e1 p @ r @ b l ) ((pos "OA$" ) ))
( "reparation"  n ( r e2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "reparations"  n ( r e2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "repartee"  n ( r e2 p aa t ii1 ) ((pos "M6%" ) ))
( "repartees"  n ( r e2 p aa t ii1 z ) ((pos "Mj$" ) ))
( "repast"  n ( r i1 p aa1 s t ) ((pos "K6%" ) ))
( "repasts"  n ( r i1 p aa1 s t s ) ((pos "Kj%" ) ))
( "repatriate"  v ( r ii1 p a1 t r i ei t ) ((pos "H2%" "K6%" ) (subcat )))
( "repatriated"  v ( r ii1 p a1 t r i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "repatriates"  v ( r ii1 p a1 t r i ei t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "repatriating"  v ( r ii1 p a1 t r i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "repatriation"  n ( r ii2 p a t r i ei1 sh @ n ) ((pos "M6%" ) ))
( "repatriations"  n ( r ii2 p a t r i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "repay"  v ( r i1 p ei1 ) ((pos "J5%" ) (subcat "2A" "6A" )))
( "repayable"  j ( r i1 p ei1 @ b l ) ((pos "OA%" ) ))
( "repaying"  v ( r i1 p ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "repayment"  n ( r i1 p ei1 m @ n t ) ((pos "M6%" ) ))
( "repayments"  n ( r i1 p ei1 m @ n t s ) ((pos "Mj%" ) ))
( "repays"  v ( r i1 p ei1 z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "repeal"  v ( r i1 p ii1 l ) ((pos "H0%" "K6%" ) (subcat )))
( "repealed"  v ( r i1 p ii1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "repealing"  v ( r i1 p ii1 l i ng ) ((pos "Hb%" ) (subcat )))
( "repeals"  v ( r i1 p ii1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "repeat"  v ( r i1 p ii1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "repeatable"  j ( r i1 p ii1 t @ b l ) ((pos "OA%" ) ))
( "repeated"  v ( r i1 p ii1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "6A" )))
( "repeatedly"  a ( r i1 p ii1 t i d l ii ) ((pos "Pu%" ) ))
( "repeater"  n ( r i1 p ii1 t @ r ) ((pos "K6%" ) ))
( "repeaters"  n ( r i1 p ii1 t @ z ) ((pos "Kj%" ) ))
( "repeating"  v ( r i1 p ii1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "repeats"  v ( r i1 p ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "repel"  v ( r i1 p e1 l ) ((pos "H4%" ) (subcat )))
( "repelled"  v ( r i1 p e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "repellent"  n ( r i1 p e1 l @ n t ) ((pos "L@%" "OA%" ) ))
( "repelling"  v ( r i1 p e1 l i ng ) ((pos "Hb%" ) (subcat )))
( "repels"  v ( r i1 p e1 l z ) ((pos "Ha%" ) (subcat )))
( "repent"  v ( r i1 p e1 n t ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "repentance"  n ( r i1 p e1 n t @ n s ) ((pos "L@%" ) ))
( "repentant"  j ( r i1 p e1 n t @ n t ) ((pos "OA%" ) ))
( "repentantly"  a ( r i1 p e1 n t @ n t l ii ) ((pos "Pu%" ) ))
( "repented"  v ( r i1 p e1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "repenting"  v ( r i1 p e1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "repents"  v ( r i1 p e1 n t s ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "repercussion"  n ( r ii2 p @ k uh1 sh @ n ) ((pos "M6%" ) ))
( "repercussions"  n ( r ii2 p @ k uh1 sh @ n z ) ((pos "Mj%" ) ))
( "repertoire"  n ( r e1 p @ t w aa r ) ((pos "K6%" ) ))
( "repertoires"  n ( r e1 p @ t w aa z ) ((pos "Kj%" ) ))
( "repertories"  n ( r e1 p @ t r i z ) ((pos "Kj$" ) ))
( "repertory"  n ( r e1 p @ t r ii ) ((pos "K8%" ) ))
( "repetition"  n ( r e2 p i t i1 sh @ n ) ((pos "M6%" ) ))
( "repetitions"  n ( r e2 p i t i1 sh @ n z ) ((pos "Mj%" ) ))
( "repetitious"  j ( r e2 p i t i1 sh @ s ) ((pos "OA%" ) ))
( "repetitive"  j ( r i1 p e1 t @ t i v ) ((pos "OA%" ) ))
( "repetitively"  a ( r i1 p e1 t @ t i v l ii ) ((pos "Pu%" ) ))
( "repine"  v ( r i1 p ai1 n ) ((pos "I2$" ) (subcat "2A" )))
( "repined"  v ( r i1 p ai1 n d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "repines"  v ( r i1 p ai1 n z ) ((pos "Ia$" ) (subcat "2A" )))
( "repining"  v ( r i1 p ai1 n i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "replace"  v ( r i1 p l ei1 s ) ((pos "H2%" ) (subcat "6A" "14" )))
( "replaceable"  j ( r i1 p l ei1 s @ b l ) ((pos "OA%" ) ))
( "replaced"  v ( r i1 p l ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "replacement"  n ( r i1 p l ei1 s m @ n t ) ((pos "M6%" ) ))
( "replacements"  n ( r i1 p l ei1 s m @ n t s ) ((pos "Mj%" ) ))
( "replaces"  v ( r i1 p l ei1 s i z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "replacing"  v ( r i1 p l ei1 s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "replant"  v ( r ii1 p l aa1 n t ) ((pos "H0%" ) (subcat "6A" "15A" )))
( "replanted"  v ( r ii1 p l aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "replanting"  v ( r ii1 p l aa1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "replants"  v ( r ii1 p l aa1 n t s ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "replay"  n ( r ii1 p l ei ) ((pos "K6%" ) ))
( "replay"  v ( r ii2 p l ei1 ) ((pos "H0%" ) (subcat )))
( "replayed"  v ( r ii2 p l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "replaying"  v ( r ii2 p l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "replays"  n ( r ii1 p l ei z ) ((pos "Kj%" ) ))
( "replays"  v ( r ii2 p l ei1 z ) ((pos "Ha%" ) (subcat )))
( "replenish"  v ( r i1 p l e1 n i sh ) ((pos "H1%" ) (subcat "6A" )))
( "replenished"  v ( r i1 p l e1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "replenishes"  v ( r i1 p l e1 n i sh i z ) ((pos "Ha%" ) (subcat "6A" )))
( "replenishing"  v ( r i1 p l e1 n i sh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "replenishment"  n ( r i1 p l e1 n i sh m @ n t ) ((pos "M6%" ) ))
( "replenishments"  n ( r i1 p l e1 n i sh m @ n t s ) ((pos "Mj%" ) ))
( "replete"  j ( r i1 p l ii1 t ) ((pos "OA%" ) ))
( "repletion"  n ( r i1 p l ii1 sh @ n ) ((pos "L@$" ) ))
( "replica"  n ( r e1 p l i k @ ) ((pos "K6%" ) ))
( "replicas"  n ( r e1 p l i k @ z ) ((pos "Kj%" ) ))
( "replied"  v ( r i1 p l ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "replies"  v ( r i1 p l ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "reply"  v ( r i1 p l ai1 ) ((pos "J3%" "K8%" ) (subcat "2A" "3A" )))
( "reply-paid"  j ( r i1 p l ai1 - p ei d ) ((pos "OA%" ) ))
( "replying"  v ( r i1 p l ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "repoint"  v ( r ii1 p oi1 n t ) ((pos "H0$" ) (subcat )))
( "repointed"  v ( r ii1 p oi1 n t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "repointing"  v ( r ii1 p oi1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "repoints"  v ( r ii1 p oi1 n t s ) ((pos "Ha$" ) (subcat )))
( "report"  v ( r i1 p oo1 t ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "6A" "6D" "9" "14" "15A" "15B" )))
( "reportable"  j ( r i1 p oo1 t @ b l ) ((pos "OA%" ) ))
( "reportage"  n ( r e2 p oo t aa1 zh ) ((pos "L@$" ) ))
( "reported"  v ( r i1 p oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "6D" "9" "14" "15A" "15B" )))
( "reportedly"  a ( r i1 p oo1 t i d l ii ) ((pos "Pu%" ) ))
( "reporter"  n ( r i1 p oo1 t @ r ) ((pos "K6%" ) ))
( "reporters"  n ( r i1 p oo1 t @ z ) ((pos "Kj%" ) ))
( "reporting"  v ( r i1 p oo1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "6D" "9" "14" "15A" "15B" )))
( "reports"  v ( r i1 p oo1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "6D" "9" "14" "15A" "15B" )))
( "repose"  v ( r i1 p ou1 z ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "reposed"  v ( r i1 p ou1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "reposeful"  j ( r i1 p ou1 z f @ l ) ((pos "OA$" ) ))
( "reposes"  v ( r i1 p ou1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "reposing"  v ( r i1 p ou1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "repositories"  n ( r i1 p o1 z i t r i z ) ((pos "Kj%" ) ))
( "repository"  n ( r i1 p o1 z i t r ii ) ((pos "K8%" ) ))
( "repot"  v ( r ii2 p o1 t ) ((pos "H4%" ) ))
( "repots"  v ( r ii2 p o1 t s ) ((pos "Ha%" ) ))
( "repotted"  v ( r ii2 p o1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "repotting"  v ( r ii2 p o1 t i ng ) ((pos "Hb%" ) ))
( "repp"  n ( r e1 p ) ((pos "L@$" ) ))
( "reprehend"  v ( r e2 p r i h e1 n d ) ((pos "H0%" ) (subcat )))
( "reprehended"  v ( r e2 p r i h e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reprehending"  v ( r e2 p r i h e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "reprehends"  v ( r e2 p r i h e1 n d z ) ((pos "Ha%" ) (subcat )))
( "reprehensible"  j ( r e2 p r i h e1 n s @ b l ) ((pos "OA%" ) ))
( "reprehensibly"  a ( r e2 p r i h e1 n s @ b l ii ) ((pos "Pu%" ) ))
( "represent"  v ( r e2 p r i z e1 n t ) ((pos "H0%" ) (subcat "6A" "9" "14" "16A" )))
( "represent"  v ( r ii2 p r i z e1 n t ) ((pos "H0%" ) ))
( "representation"  n ( r e2 p r i z e n t ei1 sh @ n ) ((pos "M6%" ) ))
( "representations"  n ( r e2 p r i z e n t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "representative"  n ( r e2 p r i z e1 n t @ t i v ) ((pos "K6%" "OA%" ) ))
( "representatives"  n ( r e2 p r i z e1 n t @ t i v z ) ((pos "Kj%" ) ))
( "represented"  v ( r e2 p r i z e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" "16A" )))
( "represented"  v ( r ii2 p r i z e1 n t i d ) ((pos "Hc%" "Hd%" ) ))
( "representing"  v ( r e2 p r i z e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" "16A" )))
( "representing"  v ( r ii2 p r i z e1 n t i ng ) ((pos "Hb$" ) ))
( "represents"  v ( r e2 p r i z e1 n t s ) ((pos "Ha%" ) (subcat "6A" "9" "14" "16A" )))
( "represents"  v ( r ii2 p r i z e1 n t s ) ((pos "Ha$" ) ))
( "repress"  v ( r i1 p r e1 s ) ((pos "H1%" ) (subcat )))
( "repressed"  v ( r i1 p r e1 s t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "represses"  v ( r i1 p r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "repressing"  v ( r i1 p r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "repression"  n ( r i1 p r e1 sh @ n ) ((pos "M6%" ) ))
( "repressions"  n ( r i1 p r e1 sh @ n z ) ((pos "Mj$" ) ))
( "repressive"  j ( r i1 p r e1 s i v ) ((pos "OA%" ) ))
( "reprieve"  v ( r i1 p r ii1 v ) ((pos "H2%" "K6%" ) (subcat )))
( "reprieved"  v ( r i1 p r ii1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reprieves"  v ( r i1 p r ii1 v z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "reprieving"  v ( r i1 p r ii1 v i ng ) ((pos "Hb%" ) (subcat )))
( "reprimand"  v ( r e1 p r i m aa n d ) ((pos "H0%" "K6%" ) (subcat )))
( "reprimanded"  v ( r e1 p r i m aa n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reprimanding"  v ( r e1 p r i m aa n d i ng ) ((pos "Hb%" ) (subcat )))
( "reprimands"  v ( r e1 p r i m aa n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "reprint"  n ( r ii1 p r i n t ) ((pos "K6%" ) ))
( "reprint"  v ( r ii2 p r i1 n t ) ((pos "H0%" ) (subcat )))
( "reprinted"  v ( r ii2 p r i1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reprinting"  v ( r ii2 p r i1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "reprints"  n ( r ii1 p r i n t s ) ((pos "Kj%" ) ))
( "reprints"  v ( r ii2 p r i1 n t s ) ((pos "Ha%" ) (subcat )))
( "reprisal"  n ( r i1 p r ai1 z l ) ((pos "M6%" ) ))
( "reprisals"  n ( r i1 p r ai1 z l z ) ((pos "Mj%" ) ))
( "reproach"  v ( r i1 p r ou1 ch ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "reproached"  v ( r i1 p r ou1 ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reproaches"  v ( r i1 p r ou1 ch i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "reproachful"  j ( r i1 p r ou1 ch f @ l ) ((pos "OA%" ) ))
( "reproachfully"  a ( r i1 p r ou1 ch f @ l ii ) ((pos "Pu%" ) ))
( "reproaching"  v ( r i1 p r ou1 ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reprobate"  v ( r e1 p r @ b ei t ) ((pos "H2$" "K6%" ) (subcat )))
( "reprobated"  v ( r e1 p r @ b ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "reprobates"  v ( r e1 p r @ b ei t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "reprobating"  v ( r e1 p r @ b ei t i ng ) ((pos "Hb$" ) (subcat )))
( "reprobation"  n ( r e2 p r @ b ei1 sh @ n ) ((pos "L@$" ) ))
( "reproduce"  v ( r ii2 p r @ d y uu1 s ) ((pos "J2%" ) (subcat "2A" )))
( "reproduced"  v ( r ii2 p r @ d y uu1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reproducer"  n ( r ii2 p r @ d y uu1 s @ r ) ((pos "K6%" ) ))
( "reproducers"  n ( r ii2 p r @ d y uu1 s @ z ) ((pos "Kj%" ) ))
( "reproduces"  v ( r ii2 p r @ d y uu1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "reproducible"  j ( r ii2 p r @ d y uu1 s @ b l ) ((pos "OA%" ) ))
( "reproducing"  v ( r ii2 p r @ d y uu1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reproduction"  n ( r ii2 p r @ d uh1 k sh @ n ) ((pos "M6%" ) ))
( "reproductions"  n ( r ii2 p r @ d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "reproductive"  j ( r ii2 p r @ d uh1 k t i v ) ((pos "OA%" ) ))
( "reproof"  v ( r ii2 p r uu1 f ) ((pos "H0%" ) (subcat )))
( "reproof"  n ( r i1 p r uu1 f ) ((pos "M6%" ) ))
( "reproofed"  v ( r ii2 p r uu1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reproofing"  v ( r ii2 p r uu1 f i ng ) ((pos "Hb%" ) (subcat )))
( "reproofs"  v ( r ii2 p r uu1 f s ) ((pos "Ha%" ) (subcat )))
( "reproofs"  n ( r i1 p r uu1 f s ) ((pos "Mj%" ) ))
( "reprove"  v ( r i1 p r uu1 v ) ((pos "H2%" ) (subcat "6A" )))
( "reproved"  v ( r i1 p r uu1 v d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "reproves"  v ( r i1 p r uu1 v z ) ((pos "Ha%" ) (subcat "6A" )))
( "reproving"  v ( r i1 p r uu1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reprovingly"  a ( r i1 p r uu1 v i ng l ii ) ((pos "Pu%" ) ))
( "reps"  n ( r e1 p s ) ((pos "L@$" "Mj%" ) ))
( "reptile"  n ( r e1 p t ai l ) ((pos "K6%" ) ))
( "reptiles"  n ( r e1 p t ai l z ) ((pos "Kj%" ) ))
( "reptilian"  n ( r e1 p t i1 l i@ n ) ((pos "K6$" "OA%" ) ))
( "reptilians"  n ( r e1 p t i1 l i@ n z ) ((pos "Kj$" ) ))
( "republic"  n ( r i1 p uh1 b l i k ) ((pos "K6%" ) ))
( "republican"  n ( r i1 p uh1 b l i k @ n ) ((pos "K6%" "OA%" ) ))
( "republicanism"  n ( r i1 p uh1 b l i k @ n i z @ m ) ((pos "L@%" ) ))
( "republicans"  n ( r i1 p uh1 b l i k @ n z ) ((pos "Kj%" ) ))
( "republics"  n ( r i1 p uh1 b l i k s ) ((pos "Kj%" ) ))
( "repudiate"  v ( r i1 p y uu1 d i ei t ) ((pos "H2%" ) (subcat )))
( "repudiated"  v ( r i1 p y uu1 d i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "repudiates"  v ( r i1 p y uu1 d i ei t s ) ((pos "Ha%" ) (subcat )))
( "repudiating"  v ( r i1 p y uu1 d i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "repudiation"  n ( r i1 p y uu2 d i ei1 sh @ n ) ((pos "K6%" ) ))
( "repudiations"  n ( r i1 p y uu2 d i ei1 sh @ n z ) ((pos "Kj%" ) ))
( "repugnance"  n ( r i1 p uh1 g n @ n s ) ((pos "L@%" ) ))
( "repugnant"  j ( r i1 p uh1 g n @ n t ) ((pos "OA%" ) ))
( "repulse"  v ( r i1 p uh1 l s ) ((pos "H2%" "K6%" ) (subcat )))
( "repulsed"  v ( r i1 p uh1 l s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "repulses"  v ( r i1 p uh1 l s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "repulsing"  v ( r i1 p uh1 l s i ng ) ((pos "Hb%" ) (subcat )))
( "repulsion"  n ( r i1 p uh1 l sh @ n ) ((pos "L@%" ) ))
( "repulsive"  j ( r i1 p uh1 l s i v ) ((pos "OA%" ) ))
( "repulsively"  a ( r i1 p uh1 l s i v l ii ) ((pos "Pu%" ) ))
( "reputable"  j ( r e1 p y u t @ b l ) ((pos "OA%" ) ))
( "reputably"  a ( r e1 p y u t @ b l ii ) ((pos "Pu%" ) ))
( "reputation"  n ( r e2 p y u t ei1 sh @ n ) ((pos "M6%" ) ))
( "reputations"  n ( r e2 p y u t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "repute"  v ( r i1 p y uu1 t ) ((pos "H2$" "L@%" ) (subcat )))
( "reputed"  v ( r i1 p y uu1 t i d ) ((pos "Hc$" "Hd%" "Oq%" ) (subcat )))
( "reputedly"  a ( r i1 p y uu1 t i d l ii ) ((pos "Pu%" ) ))
( "reputes"  v ( r i1 p y uu1 t s ) ((pos "Ha$" ) (subcat )))
( "reputing"  v ( r i1 p y uu1 t i ng ) ((pos "Hb$" ) (subcat )))
( "request"  v ( r i1 k w e1 s t ) ((pos "H0%" "M6%" ) (subcat "6A" "9" )))
( "requested"  v ( r i1 k w e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "requesting"  v ( r i1 k w e1 s t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "requests"  v ( r i1 k w e1 s t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "9" )))
( "requiem"  n ( r e1 k w i@ m ) ((pos "K6%" ) ))
( "requiems"  n ( r e1 k w i@ m z ) ((pos "Kj%" ) ))
( "require"  v ( r i1 k w ai1 @ r ) ((pos "H2%" ) (subcat "6A" "6D" "9" "14" )))
( "required"  v ( r i1 k w ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "9" "14" )))
( "requirement"  n ( r i1 k w ai1 @ m @ n t ) ((pos "K6%" ) ))
( "requirements"  n ( r i1 k w ai1 @ m @ n t s ) ((pos "Kj%" ) ))
( "requires"  v ( r i1 k w ai1 @ z ) ((pos "Ha%" ) (subcat "6A" "6D" "9" "14" )))
( "requiring"  v ( r i1 k w ai1 @ r i ng ) ((pos "Hb%" ) (subcat "6A" "6D" "9" "14" )))
( "requisite"  n ( r e1 k w i z i t ) ((pos "K6%" "OA%" ) ))
( "requisites"  n ( r e1 k w i z i t s ) ((pos "Kj%" ) ))
( "requisition"  v ( r e2 k w i z i1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "requisitioned"  v ( r e2 k w i z i1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "requisitioning"  v ( r e2 k w i z i1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "requisitions"  v ( r e2 k w i z i1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "requital"  n ( r i1 k w ai1 t l ) ((pos "L@%" ) ))
( "requite"  v ( r i1 k w ai1 t ) ((pos "H2$" ) (subcat "6A" )))
( "requited"  v ( r i1 k w ai1 t i d ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "requites"  v ( r i1 k w ai1 t s ) ((pos "Ha$" ) (subcat "6A" )))
( "requiting"  v ( r i1 k w ai1 t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "reran"  v ( r ii1 r a n ) ((pos "Hc%" ) ))
( "reread"  v ( r ii1 r e1 d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "reread"  v ( r ii1 r ii1 d ) ((pos "H5%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "rereading"  v ( r ii1 r ii1 d i ng ) ((pos "Hb%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "rereads"  v ( r ii1 r ii1 d z ) ((pos "Ha%" ) (subcat "2A" "2B" "2C" "6A" "12A" "13A" "15A" "15B" )))
( "reredos"  n ( r i@1 d o s ) ((pos "K7$" ) ))
( "reredoses"  n ( r i@1 d o s i z ) ((pos "Kj$" ) ))
( "rerun"  v ( r ii1 r uh n ) ((pos "H5%" "Hd%" "K6%" ) ))
( "rerunning"  v ( r ii1 r uh n i ng ) ((pos "Hb%" ) ))
( "reruns"  v ( r ii1 r uh n z ) ((pos "Ha%" "Kj%" ) ))
( "res"  n ( r e1 z ) ((pos "Y>$" ) ))
( "rescind"  v ( r i1 s i1 n d ) ((pos "H0%" ) (subcat )))
( "rescinded"  v ( r i1 s i1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rescinding"  v ( r i1 s i1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "rescinds"  v ( r i1 s i1 n d z ) ((pos "Ha%" ) (subcat )))
( "rescript"  n ( r ii1 s k r i p t ) ((pos "K6$" ) ))
( "rescripts"  n ( r ii1 s k r i p t s ) ((pos "Kj$" ) ))
( "rescue"  v ( r e1 s k y uu ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "rescued"  v ( r e1 s k y uu d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "rescuer"  n ( r e1 s k y uu @ r ) ((pos "K6%" ) ))
( "rescuers"  n ( r e1 s k y uu @ z ) ((pos "Kj%" ) ))
( "rescues"  v ( r e1 s k y uu z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "rescuing"  v ( r e1 s k y uu i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "research"  v ( r i1 s @@1 ch ) ((pos "I1%" "M7%" ) (subcat "2A" )))
( "researched"  v ( r i1 s @@1 ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "researcher"  n ( r i1 s @@1 ch @ r ) ((pos "K6%" ) ))
( "researchers"  n ( r i1 s @@1 ch @ z ) ((pos "Kj%" ) ))
( "researches"  v ( r i1 s @@1 ch i z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "researching"  v ( r i1 s @@1 ch i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "reseat"  v ( r ii2 s ii1 t ) ((pos "H0%" ) (subcat )))
( "reseated"  v ( r ii2 s ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reseating"  v ( r ii2 s ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "reseats"  v ( r ii2 s ii1 t s ) ((pos "Ha%" ) (subcat )))
( "reseed"  v ( r ii1 s ii1 d ) ((pos "H0$" ) (subcat "2A" )))
( "reseeded"  v ( r ii1 s ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "reseeding"  v ( r ii1 s ii1 d i ng ) ((pos "Hb$" ) (subcat "2A" )))
( "reseeds"  v ( r ii1 s ii1 d z ) ((pos "Ha$" ) (subcat "2A" )))
( "resell"  v ( r ii1 s e1 l ) ((pos "H5%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "reselling"  v ( r ii1 s e1 l i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "resells"  v ( r ii1 s e1 l z ) ((pos "Ha%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "resemblance"  n ( r i1 z e1 m b l @ n s ) ((pos "M6%" ) ))
( "resemblances"  n ( r i1 z e1 m b l @ n s i z ) ((pos "Mj%" ) ))
( "resemble"  v ( r i1 z e1 m b l ) ((pos "H2%" ) (subcat )))
( "resembled"  v ( r i1 z e1 m b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "resembles"  v ( r i1 z e1 m b l z ) ((pos "Ha%" ) (subcat )))
( "resembling"  v ( r i1 z e1 m b l i ng ) ((pos "Hb%" ) (subcat )))
( "resent"  v ( r i1 z e1 n t ) ((pos "H0%" ) (subcat "6A" "6C" )))
( "resented"  v ( r i1 z e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" )))
( "resentful"  j ( r i1 z e1 n t f @ l ) ((pos "OA%" ) ))
( "resentfully"  a ( r i1 z e1 n t f @ l ii ) ((pos "Pu%" ) ))
( "resenting"  v ( r i1 z e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" "6C" )))
( "resentment"  n ( r i1 z e1 n t m @ n t ) ((pos "L@%" ) ))
( "resents"  v ( r i1 z e1 n t s ) ((pos "Ha%" ) (subcat "6A" "6C" )))
( "reservation"  n ( r e2 z @ v ei1 sh @ n ) ((pos "M6%" ) ))
( "reservations"  n ( r e2 z @ v ei1 sh @ n z ) ((pos "Mj%" ) ))
( "reserve"  v ( r i1 z @@1 v ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "reserved"  v ( r i1 z @@1 v d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "reservedly"  a ( r i1 z @@1 v i d l ii ) ((pos "Pu%" ) ))
( "reserves"  v ( r i1 z @@1 v z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "reserving"  v ( r i1 z @@1 v i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "reservist"  n ( r i1 z @@1 v i s t ) ((pos "K6%" ) ))
( "reservists"  n ( r i1 z @@1 v i s t s ) ((pos "Kj%" ) ))
( "reservoir"  n ( r e1 z @ v w aa r ) ((pos "K6%" ) ))
( "reservoirs"  n ( r e1 z @ v w aa z ) ((pos "Kj%" ) ))
( "reset"  v ( r ii2 s e1 t ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "resets"  v ( r ii2 s e1 t s ) ((pos "Ha%" ) (subcat )))
( "resetting"  v ( r ii2 s e1 t i ng ) ((pos "Hb%" ) (subcat )))
( "resettle"  v ( r ii2 s e1 t l ) ((pos "J2%" ) (subcat "2A" )))
( "resettled"  v ( r ii2 s e1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "resettlement"  n ( r ii2 s e1 t l m @ n t ) ((pos "K6%" ) ))
( "resettlements"  n ( r ii2 s e1 t l m @ n t s ) ((pos "Kj%" ) ))
( "resettles"  v ( r ii2 s e1 t l z ) ((pos "Ja%" ) (subcat "2A" )))
( "resettling"  v ( r ii2 s e1 t l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reshape"  v ( r ii1 sh ei1 p ) ((pos "H2%" ) (subcat "2A" "2C" "6A" )))
( "reshaped"  v ( r ii1 sh ei1 p t ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" )))
( "reshapes"  v ( r ii1 sh ei1 p s ) ((pos "Ha%" ) (subcat "2A" "2C" "6A" )))
( "reshaping"  v ( r ii1 sh ei1 p i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" )))
( "reshuffle"  v ( r ii2 sh uh1 f l ) ((pos "H2%" "K6%" ) (subcat )))
( "reshuffled"  v ( r ii2 sh uh1 f l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reshuffles"  v ( r ii2 sh uh1 f l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "reshuffling"  v ( r ii2 sh uh1 f l i ng ) ((pos "Hb%" ) (subcat )))
( "reside"  v ( r i1 z ai1 d ) ((pos "I2%" ) (subcat "2C" )))
( "resided"  v ( r i1 z ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2C" )))
( "residence"  n ( r e1 z i d @ n s ) ((pos "M6%" ) ))
( "residences"  n ( r e1 z i d @ n s i z ) ((pos "Mj%" ) ))
( "residencies"  n ( r e1 z i d @ n s i z ) ((pos "Kj$" ) ))
( "residency"  n ( r e1 z i d @ n s ii ) ((pos "K8$" ) ))
( "resident"  n ( r e1 z i d @ n t ) ((pos "K6%" "OA%" ) ))
( "residential"  j ( r e2 z i d e1 n sh l ) ((pos "OA%" ) ))
( "residents"  n ( r e1 z i d @ n t s ) ((pos "Kj%" ) ))
( "resides"  v ( r i1 z ai1 d z ) ((pos "Ia%" ) (subcat "2C" )))
( "residing"  v ( r i1 z ai1 d i ng ) ((pos "Ib%" ) (subcat "2C" )))
( "residual"  j ( r i1 z i1 d y u@ l ) ((pos "OA%" ) ))
( "residuary"  j ( r i1 z i1 d y u@ r ii ) ((pos "OA%" ) ))
( "residue"  n ( r e1 z i d y uu ) ((pos "K6%" ) ))
( "residues"  n ( r e1 z i d y uu z ) ((pos "Kj%" ) ))
( "resign"  v ( r i1 z ai1 n ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "resignation"  n ( r e2 z i g n ei1 sh @ n ) ((pos "M6%" ) ))
( "resignations"  n ( r e2 z i g n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "resigned"  v ( r i1 z ai1 n d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" )))
( "resignedly"  a ( r i1 z ai1 n @ d l ii ) ((pos "Pu%" ) ))
( "resigning"  v ( r i1 z ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "resigns"  v ( r i1 z ai1 n z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "resilience"  n ( r i1 z i1 l i@ n s ) ((pos "L@%" ) ))
( "resiliency"  n ( r i1 z i1 l i@ n s ii ) ((pos "L@$" ) ))
( "resilient"  j ( r i1 z i1 l i@ n t ) ((pos "OA%" ) ))
( "resin"  n ( r e1 z i n ) ((pos "M6%" ) ))
( "resinated"  j ( r e1 z i n ei t i d ) ((pos "OA$" ) ))
( "resinous"  j ( r e1 z i n @ s ) ((pos "OA%" ) ))
( "resins"  n ( r e1 z i n z ) ((pos "Mj%" ) ))
( "resist"  v ( r i1 z i1 s t ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "resistance"  n ( r i1 z i1 s t @ n s ) ((pos "M6%" ) ))
( "resistances"  n ( r i1 z i1 s t @ n s i z ) ((pos "Mj%" ) ))
( "resistant"  j ( r i1 z i1 s t @ n t ) ((pos "OA%" ) ))
( "resisted"  v ( r i1 z i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "resister"  n ( r i1 z i1 s t @ r ) ((pos "K6$" ) ))
( "resisters"  n ( r i1 z i1 s t @ z ) ((pos "Kj$" ) ))
( "resisting"  v ( r i1 z i1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "resistive"  j ( r i1 z i1 s t i v ) ((pos "OA$" ) ))
( "resistivity"  n ( r e2 z i s t i1 v i t ii ) ((pos "L@$" ) ))
( "resistless"  j ( r i1 z i1 s t l @ s ) ((pos "OA$" ) ))
( "resistor"  n ( r i1 z i1 s t @ r ) ((pos "K6$" ) ))
( "resistors"  n ( r i1 z i1 s t @ z ) ((pos "Kj$" ) ))
( "resists"  v ( r i1 z i1 s t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "resold"  v ( r ii1 s ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "resole"  v ( r ii2 s ou1 l ) ((pos "H2%" ) (subcat )))
( "resoled"  v ( r ii2 s ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "resoles"  v ( r ii2 s ou1 l z ) ((pos "Ha%" ) (subcat )))
( "resoling"  v ( r ii2 s ou1 l i ng ) ((pos "Hb%" ) (subcat )))
( "resolute"  j ( r e1 z @ l uu t ) ((pos "OA%" ) ))
( "resolutely"  a ( r e1 z @ l uu t l ii ) ((pos "Pu%" ) ))
( "resoluteness"  n ( r e1 z @ l uu t n @ s ) ((pos "L@%" ) ))
( "resolution"  n ( r e2 z @ l uu1 sh @ n ) ((pos "M6%" ) ))
( "resolutions"  n ( r e2 z @ l uu1 sh @ n z ) ((pos "Mj%" ) ))
( "resolvable"  j ( r i1 z o1 l v @ b l ) ((pos "OA%" ) ))
( "resolve"  v ( r i1 z o1 l v ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "7A" "9" )))
( "resolved"  v ( r i1 z o1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "9" )))
( "resolves"  v ( r i1 z o1 l v z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "7A" "9" )))
( "resolving"  v ( r i1 z o1 l v i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "7A" "9" )))
( "resonance"  n ( r e1 z @ n @ n s ) ((pos "L@%" ) ))
( "resonant"  j ( r e1 z @ n @ n t ) ((pos "OA%" ) ))
( "resonate"  v ( r e1 z @ n ei t ) ((pos "J2%" ) ))
( "resonated"  v ( r e1 z @ n ei t i d ) ((pos "Jc%" "Jd%" ) ))
( "resonates"  v ( r e1 z @ n ei t s ) ((pos "Ja%" ) ))
( "resonating"  v ( r e1 z @ n ei t i ng ) ((pos "Jb%" ) ))
( "resonator"  n ( r e1 z @ n ei t @ r ) ((pos "K6%" ) ))
( "resonators"  n ( r e1 z @ n ei t @ z ) ((pos "Kj%" ) ))
( "resort"  v ( r i1 z oo1 t ) ((pos "I0%" "M6%" ) (subcat )))
( "resorted"  v ( r i1 z oo1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "resorting"  v ( r i1 z oo1 t i ng ) ((pos "Ib%" ) (subcat )))
( "resorts"  v ( r i1 z oo1 t s ) ((pos "Ia%" "Mj%" ) (subcat )))
( "resound"  v ( r i1 z au1 n d ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "resounded"  v ( r i1 z au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "resounding"  v ( r i1 z au1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "resoundingly"  a ( r i1 z au1 n d i ng l ii ) ((pos "Pu%" ) ))
( "resounds"  v ( r i1 z au1 n d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "resource"  n ( r i1 z oo1 s ) ((pos "M6%" ) ))
( "resourceful"  j ( r i1 z oo1 s f @ l ) ((pos "OA%" ) ))
( "resourcefully"  a ( r i1 z oo1 s f @ l ii ) ((pos "Pu%" ) ))
( "resourceless"  j ( r i1 z oo1 s l i s ) ((pos "OA%" ) ))
( "resources"  n ( r i1 z oo1 s i z ) ((pos "Mj%" ) ))
( "respect"  v ( r i1 s p e1 k t ) ((pos "H0%" "M6%" ) (subcat )))
( "respectabilities"  n ( r i1 s p e2 k t @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "respectability"  n ( r i1 s p e2 k t @ b i1 l i t ii ) ((pos "M8%" ) ))
( "respectable"  j ( r i1 s p e1 k t @ b l ) ((pos "OA%" ) ))
( "respectably"  a ( r i1 s p e1 k t @ b l ii ) ((pos "Pu%" ) ))
( "respected"  v ( r i1 s p e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "respecter"  n ( r i1 s p e1 k t @ r ) ((pos "K6%" ) ))
( "respecters"  n ( r i1 s p e1 k t @ z ) ((pos "Kj%" ) ))
( "respectful"  j ( r i1 s p e1 k t f @ l ) ((pos "OA%" ) ))
( "respectfully"  a ( r i1 s p e1 k t f @ l ii ) ((pos "Pu%" ) ))
( "respecting"  v ( r i1 s p e1 k t i ng ) ((pos "Hb%" "T-%" ) (subcat )))
( "respective"  j ( r i1 s p e1 k t i v ) ((pos "OA%" ) ))
( "respectively"  a ( r i1 s p e1 k t i v l ii ) ((pos "Pu%" ) ))
( "respects"  v ( r i1 s p e1 k t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "respiration"  n ( r e2 s p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "respirations"  n ( r e2 s p @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "respirator"  n ( r e1 s p @ r ei t @ r ) ((pos "K6%" ) ))
( "respirators"  n ( r e1 s p @ r ei t @ z ) ((pos "Kj%" ) ))
( "respiratory"  j ( r i1 s p i1 r @ t r ii ) ((pos "OA%" ) ))
( "respire"  v ( r i1 s p ai1 @ r ) ((pos "I2$" ) (subcat )))
( "respired"  v ( r i1 s p ai1 @ d ) ((pos "Ic$" "Id$" ) (subcat )))
( "respires"  v ( r i1 s p ai1 @ z ) ((pos "Ia$" ) (subcat )))
( "respiring"  v ( r i1 s p ai1 @ r i ng ) ((pos "Ib$" ) (subcat )))
( "respite"  v ( r e1 s p ai t ) ((pos "H2$" "M6%" ) (subcat )))
( "respited"  v ( r e1 s p ai t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "respites"  v ( r e1 s p ai t s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "respiting"  v ( r e1 s p ai t i ng ) ((pos "Hb$" ) (subcat )))
( "resplendence"  n ( r i1 s p l e1 n d @ n s ) ((pos "L@%" ) ))
( "resplendency"  n ( r i1 s p l e1 n d @ n s ii ) ((pos "L@$" ) ))
( "resplendent"  j ( r i1 s p l e1 n d @ n t ) ((pos "OA%" ) ))
( "resplendently"  a ( r i1 s p l e1 n d @ n t l ii ) ((pos "Pu%" ) ))
( "respond"  v ( r i1 s p o1 n d ) ((pos "I0%" ) (subcat "2A" "2C" "3A" )))
( "responded"  v ( r i1 s p o1 n d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "3A" )))
( "respondent"  n ( r i1 s p o1 n d @ n t ) ((pos "K6%" ) ))
( "respondents"  n ( r i1 s p o1 n d @ n t s ) ((pos "Kj%" ) ))
( "responding"  v ( r i1 s p o1 n d i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "3A" )))
( "responds"  v ( r i1 s p o1 n d z ) ((pos "Ia%" ) (subcat "2A" "2C" "3A" )))
( "response"  n ( r i1 s p o1 n s ) ((pos "M6%" ) ))
( "responses"  n ( r i1 s p o1 n s i z ) ((pos "Mj%" ) ))
( "responsibilities"  n ( r i1 s p o2 n s @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "responsibility"  n ( r i1 s p o2 n s @ b i1 l i t ii ) ((pos "M8%" ) ))
( "responsible"  j ( r i1 s p o1 n s @ b l ) ((pos "OA%" ) ))
( "responsibly"  a ( r i1 s p o1 n s @ b l ii ) ((pos "Pu%" ) ))
( "responsive"  j ( r i1 s p o1 n s i v ) ((pos "OA%" ) ))
( "responsiveness"  n ( r i1 s p o1 n s i v n i s ) ((pos "L@%" ) ))
( "rest"  v ( r e1 s t ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" )))
( "rest-cure"  n ( r e1 s t - k y u@ r ) ((pos "K6%" ) ))
( "rest-cures"  n ( r e1 s t - k y u@ z ) ((pos "Kj%" ) ))
( "rest-day"  n ( r e1 s t - d ei ) ((pos "K6%" ) ))
( "rest-days"  n ( r e1 s t - d ei z ) ((pos "Kj%" ) ))
( "rest-home"  n ( r e1 s t - h ou m ) ((pos "K6%" ) ))
( "rest-homes"  n ( r e1 s t - h ou m z ) ((pos "Kj%" ) ))
( "rest-house"  n ( r e1 s t - h au s ) ((pos "K6%" ) ))
( "rest-houses"  n ( r e1 s t - h au z i z ) ((pos "Kj%" ) ))
( "restart"  v ( r ii1 s t aa1 t ) ((pos "J0%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "restarted"  v ( r ii1 s t aa1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "restarting"  v ( r ii1 s t aa1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "restarts"  v ( r ii1 s t aa1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "restate"  v ( r ii2 s t ei1 t ) ((pos "H2%" ) (subcat )))
( "restated"  v ( r ii2 s t ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "restatement"  n ( r ii2 s t ei1 t m @ n t ) ((pos "K6%" ) ))
( "restatements"  n ( r ii2 s t ei1 t m @ n t s ) ((pos "Kj%" ) ))
( "restates"  v ( r ii2 s t ei1 t s ) ((pos "Ha%" ) (subcat )))
( "restating"  v ( r ii2 s t ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "restaurant"  n ( r e1 s t r o n t ) ((pos "K6%" ) ))
( "restauranteur"  n ( r e2 s t r o n t @@1 r ) ((pos "K6$" ) ))
( "restauranteurs"  n ( r e2 s t r o n t @@1 z ) ((pos "Kj$" ) ))
( "restaurants"  n ( r e1 s t r o n t s ) ((pos "Kj%" ) ))
( "restaurateur"  n ( r e2 s t @ r @ t @@1 r ) ((pos "K6%" ) ))
( "restaurateurs"  n ( r e2 s t @ r @ t @@1 z ) ((pos "Kj%" ) ))
( "rested"  v ( r e1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" )))
( "restful"  j ( r e1 s t f @ l ) ((pos "OA%" ) ))
( "restfully"  a ( r e1 s t f @ l ii ) ((pos "Pu%" ) ))
( "restfulness"  n ( r e1 s t f @ l n @ s ) ((pos "L@%" ) ))
( "resting"  v ( r e1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" )))
( "restitution"  n ( r e2 s t i t y uu1 sh @ n ) ((pos "L@%" ) ))
( "restive"  j ( r e1 s t i v ) ((pos "OA%" ) ))
( "restively"  a ( r e1 s t i v l ii ) ((pos "Pu%" ) ))
( "restiveness"  n ( r e1 s t i v n @ s ) ((pos "L@%" ) ))
( "restless"  j ( r e1 s t l @ s ) ((pos "OA%" ) ))
( "restlessly"  a ( r e1 s t l @ s l ii ) ((pos "Pu%" ) ))
( "restlessness"  n ( r e1 s t l @ s n @ s ) ((pos "L@%" ) ))
( "restock"  v ( r ii2 s t o1 k ) ((pos "H0%" ) (subcat )))
( "restocked"  v ( r ii2 s t o1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "restocking"  v ( r ii2 s t o1 k i ng ) ((pos "Hb%" ) (subcat )))
( "restocks"  v ( r ii2 s t o1 k s ) ((pos "Ha%" ) (subcat )))
( "restoration"  n ( r e2 s t @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "restorations"  n ( r e2 s t @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "restorative"  n ( r i1 s t oo1 r @ t i v ) ((pos "M6%" "OA%" ) ))
( "restoratives"  n ( r i1 s t oo1 r @ t i v z ) ((pos "Mj%" ) ))
( "restore"  v ( r i1 s t oo1 r ) ((pos "H2%" ) (subcat "6A" )))
( "restored"  v ( r i1 s t oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "restorer"  n ( r i1 s t oo1 r @ r ) ((pos "K6%" ) ))
( "restorers"  n ( r i1 s t oo1 r @ z ) ((pos "Kj%" ) ))
( "restores"  v ( r i1 s t oo1 z ) ((pos "Ha%" ) (subcat "6A" )))
( "restoring"  v ( r i1 s t oo1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "restrain"  v ( r i1 s t r ei1 n ) ((pos "H0%" ) (subcat "6A" )))
( "restrained"  v ( r i1 s t r ei1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "restraining"  v ( r i1 s t r ei1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "restrains"  v ( r i1 s t r ei1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "restraint"  n ( r i1 s t r ei1 n t ) ((pos "M6%" ) ))
( "restraints"  n ( r i1 s t r ei1 n t s ) ((pos "Mj%" ) ))
( "restrict"  v ( r i1 s t r i1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "restricted"  v ( r i1 s t r i1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "restricting"  v ( r i1 s t r i1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "restriction"  n ( r i1 s t r i1 k sh @ n ) ((pos "M6%" ) ))
( "restrictions"  n ( r i1 s t r i1 k sh @ n z ) ((pos "Mj%" ) ))
( "restrictive"  j ( r i1 s t r i1 k t i v ) ((pos "OA%" ) ))
( "restrictively"  a ( r i1 s t r i1 k t i v l ii ) ((pos "Pu%" ) ))
( "restrictiveness"  n ( r i1 s t r i1 k t i v n i s ) ((pos "L@%" ) ))
( "restricts"  v ( r i1 s t r i1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "restructure"  v ( r ii2 s t r uh1 k ch @ r ) ((pos "H2%" ) (subcat )))
( "restructured"  v ( r ii2 s t r uh1 k ch @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "restructures"  v ( r ii2 s t r uh1 k ch @ z ) ((pos "Ha%" ) (subcat )))
( "restructuring"  v ( r ii2 s t r uh1 k ch @ r i ng ) ((pos "Hb%" ) (subcat )))
( "rests"  v ( r e1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" )))
( "result"  v ( r i1 z uh1 l t ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "resultant"  j ( r i1 s uh1 l t @ n t ) ((pos "OA%" ) ))
( "resulted"  v ( r i1 z uh1 l t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "resulting"  v ( r i1 z uh1 l t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "results"  v ( r i1 z uh1 l t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "resume"  v ( r i1 z y uu1 m ) ((pos "H2%" ) (subcat "6A" )))
( "resumed"  v ( r i1 z y uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "resumes"  v ( r i1 z y uu1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "resuming"  v ( r i1 z y uu1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "resumption"  n ( r i1 z uh1 m p sh @ n ) ((pos "M6%" ) ))
( "resumptions"  n ( r i1 z uh1 m p sh @ n z ) ((pos "Mj$" ) ))
( "resurface"  v ( r ii2 s @@1 f i s ) ((pos "J2%" ) (subcat "2A" )))
( "resurfaced"  v ( r ii2 s @@1 f i s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "resurfaces"  v ( r ii2 s @@1 f i s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "resurfacing"  v ( r ii2 s @@1 f i s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "resurgence"  n ( r i1 s @@1 jh @ n s ) ((pos "K6%" ) ))
( "resurgences"  n ( r i1 s @@1 jh @ n s i z ) ((pos "Kj$" ) ))
( "resurgent"  j ( r i1 s @@1 jh @ n t ) ((pos "OA%" ) ))
( "resurrect"  v ( r e2 z @ r e1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "resurrected"  v ( r e2 z @ r e1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "resurrecting"  v ( r e2 z @ r e1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "resurrection"  n ( r e2 z @ r e1 k sh @ n ) ((pos "L@%" ) ))
( "resurrects"  v ( r e2 z @ r e1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "resuscitate"  v ( r i1 s uh1 s i t ei t ) ((pos "J2%" ) (subcat "2A" )))
( "resuscitated"  v ( r i1 s uh1 s i t ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "resuscitates"  v ( r i1 s uh1 s i t ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "resuscitating"  v ( r i1 s uh1 s i t ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "resuscitation"  n ( r i1 s uh2 s i t ei1 sh @ n ) ((pos "M6%" ) ))
( "resuscitations"  n ( r i1 s uh2 s i t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "ret"  n ( r i1 t ai1 @ d ) ((pos "Y~%" ) ))
( "ret"  v ( r e1 t ) ((pos "H4$" ) (subcat )))
( "retail"  v ( r ii1 t ei l ) ((pos "J0%" "K6%" "Pu%" ) (subcat "3A" )))
( "retailed"  v ( r ii1 t ei l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "retailer"  n ( r ii1 t ei l @ r ) ((pos "K6%" ) ))
( "retailers"  n ( r ii1 t ei l @ z ) ((pos "Kj%" ) ))
( "retailing"  v ( r ii1 t ei l i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "retails"  v ( r ii1 t ei l z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "retain"  v ( r i1 t ei1 n ) ((pos "H0%" ) (subcat )))
( "retained"  v ( r i1 t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "retainer"  n ( r i1 t ei1 n @ r ) ((pos "K6%" ) ))
( "retainers"  n ( r i1 t ei1 n @ z ) ((pos "Kj%" ) ))
( "retaining"  v ( r i1 t ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "retains"  v ( r i1 t ei1 n z ) ((pos "Ha%" ) (subcat )))
( "retake"  n ( r ii1 t ei k ) ((pos "K6%" ) ))
( "retake"  v ( r ii2 t ei1 k ) ((pos "H5%" ) (subcat )))
( "retaken"  v ( r ii2 t ei1 k @ n ) ((pos "Hd%" ) (subcat )))
( "retakes"  n ( r ii1 t ei k s ) ((pos "Kj%" ) ))
( "retakes"  v ( r ii2 t ei1 k s ) ((pos "Ha%" ) (subcat )))
( "retaking"  v ( r ii2 t ei1 k i ng ) ((pos "Hb%" ) (subcat )))
( "retaliate"  v ( r i1 t a1 l i ei t ) ((pos "I2%" ) (subcat "2A" )))
( "retaliated"  v ( r i1 t a1 l i ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "retaliates"  v ( r i1 t a1 l i ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "retaliating"  v ( r i1 t a1 l i ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "retaliation"  n ( r i1 t a2 l i ei1 sh @ n ) ((pos "L@%" ) ))
( "retaliative"  j ( r i1 t a1 l i@ t i v ) ((pos "OA$" ) ))
( "retaliatory"  j ( r i1 t a1 l i@ t r ii ) ((pos "OA%" ) ))
( "retard"  v ( r i1 t aa1 d ) ((pos "H0%" ) (subcat )))
( "retardation"  n ( r ii2 t aa d ei1 sh @ n ) ((pos "M6%" ) ))
( "retardations"  n ( r ii2 t aa d ei1 sh @ n z ) ((pos "Mj$" ) ))
( "retarded"  v ( r i1 t aa1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "retarding"  v ( r i1 t aa1 d i ng ) ((pos "Hb%" ) (subcat )))
( "retards"  v ( r i1 t aa1 d z ) ((pos "Ha%" ) (subcat )))
( "retch"  v ( r e1 ch ) ((pos "I1%" ) (subcat )))
( "retched"  v ( r e1 ch t ) ((pos "Ic%" "Id%" ) (subcat )))
( "retches"  v ( r e1 ch i z ) ((pos "Ia%" ) (subcat )))
( "retching"  v ( r e1 ch i ng ) ((pos "Ib%" ) (subcat )))
( "retd"  n ( r i1 t ai1 @ d ) ((pos "Y~%" ) ))
( "retell"  v ( r ii2 t e1 l ) ((pos "H5%" ) (subcat )))
( "retelling"  v ( r ii2 t e1 l i ng ) ((pos "Hb%" ) (subcat )))
( "retells"  v ( r ii2 t e1 l z ) ((pos "Ha%" ) (subcat )))
( "retention"  n ( r i1 t e1 n sh @ n ) ((pos "L@%" ) ))
( "retentive"  j ( r i1 t e1 n t i v ) ((pos "OA%" ) ))
( "retentively"  a ( r i1 t e1 n t i v l ii ) ((pos "Pu%" ) ))
( "retentiveness"  n ( r i1 t e1 n t i v n @ s ) ((pos "L@%" ) ))
( "rethink"  n ( r ii1 th i ng k ) ((pos "K6%" ) ))
( "rethink"  v ( r ii2 th i1 ng k ) ((pos "J5%" ) (subcat "2A" )))
( "rethinking"  v ( r ii2 th i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rethinks"  n ( r ii1 th i ng k s ) ((pos "Kj%" ) ))
( "rethinks"  v ( r ii2 th i1 ng k s ) ((pos "Ja%" ) (subcat "2A" )))
( "rethought"  v ( r ii2 th oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reticence"  n ( r e1 t i s n s ) ((pos "M6%" ) ))
( "reticences"  n ( r e1 t i s n s i z ) ((pos "Mj$" ) ))
( "reticent"  j ( r e1 t i s n t ) ((pos "OA%" ) ))
( "reticently"  a ( r e1 t i s n t l ii ) ((pos "Pu%" ) ))
( "reticulate"  j ( r i1 t i1 k y u l @ t ) ((pos "OA$" ) ))
( "reticulate"  v ( r i1 t i1 k y u l ei t ) ((pos "J2$" ) (subcat "2A" )))
( "reticulated"  v ( r i1 t i1 k y u l ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "reticulates"  v ( r i1 t i1 k y u l ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "reticulating"  v ( r i1 t i1 k y u l ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "reticulation"  n ( r i1 t i2 k y u l ei1 sh @ n ) ((pos "K6$" ) ))
( "reticulations"  n ( r i1 t i2 k y u l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "reticule"  n ( r e1 t i k y uu l ) ((pos "K6$" ) ))
( "reticules"  n ( r e1 t i k y uu l z ) ((pos "Kj$" ) ))
( "retina"  n ( r e1 t i n @ ) ((pos "K6%" ) ))
( "retinae"  n ( r e1 t i n ii ) ((pos "Kj$" ) ))
( "retinal"  j ( r e1 t i n @ l ) ((pos "OA$" ) ))
( "retinas"  n ( r e1 t i n @ z ) ((pos "Kj%" ) ))
( "retinue"  n ( r e1 t i n y uu ) ((pos "K6%" ) ))
( "retinues"  n ( r e1 t i n y uu z ) ((pos "Kj%" ) ))
( "retire"  v ( r i1 t ai1 @ r ) ((pos "J2%" "Ki%" ) (subcat "2A" "2C" "3A" )))
( "retired"  v ( r i1 t ai1 @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" )))
( "retirement"  n ( r i1 t ai1 @ m @ n t ) ((pos "M6%" ) ))
( "retirements"  n ( r i1 t ai1 @ m @ n t s ) ((pos "Mj%" ) ))
( "retires"  v ( r i1 t ai1 @ z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" )))
( "retiring"  v ( r i1 t ai1 @ r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "3A" )))
( "retold"  v ( r ii2 t ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "retook"  v ( r ii2 t u1 k ) ((pos "Hc%" ) (subcat )))
( "retool"  v ( r ii2 t uu1 l ) ((pos "H0$" ) (subcat )))
( "retooled"  v ( r ii2 t uu1 l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "retooling"  v ( r ii2 t uu1 l i ng ) ((pos "Hb$" ) (subcat )))
( "retools"  v ( r ii2 t uu1 l z ) ((pos "Ha$" ) (subcat )))
( "retort"  v ( r i1 t oo1 t ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "9" )))
( "retorted"  v ( r i1 t oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "9" )))
( "retorting"  v ( r i1 t oo1 t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "9" )))
( "retorts"  v ( r i1 t oo1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "9" )))
( "retouch"  v ( r ii2 t uh1 ch ) ((pos "H1%" ) (subcat )))
( "retouched"  v ( r ii2 t uh1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "retouches"  v ( r ii2 t uh1 ch i z ) ((pos "Ha%" ) (subcat )))
( "retouching"  v ( r ii2 t uh1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "retrace"  v ( r ii1 t r ei1 s ) ((pos "H2%" ) (subcat )))
( "retraced"  v ( r ii1 t r ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "retraces"  v ( r ii1 t r ei1 s i z ) ((pos "Ha%" ) (subcat )))
( "retracing"  v ( r ii1 t r ei1 s i ng ) ((pos "Hb%" ) (subcat )))
( "retract"  v ( r i1 t r a1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "retractable"  j ( r i1 t r a1 k t @ b l ) ((pos "OA%" ) ))
( "retracted"  v ( r i1 t r a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "retractile"  j ( r i1 t r a1 k t ai l ) ((pos "OA$" ) ))
( "retracting"  v ( r i1 t r a1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "retraction"  n ( r i1 t r a1 k sh @ n ) ((pos "M6%" ) ))
( "retractions"  n ( r i1 t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "retracts"  v ( r i1 t r a1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "retransmit"  v ( r ii2 t r a n z m i1 t ) ((pos "H4%" ) (subcat "6A" )))
( "retransmits"  v ( r ii2 t r a n z m i1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "retransmitted"  v ( r ii2 t r a n z m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "retransmitting"  v ( r ii2 t r a n z m i1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "retread"  n ( r ii1 t r e d ) ((pos "K6%" ) ))
( "retread"  v ( r ii2 t r e1 d ) ((pos "H0$" ) ))
( "retreaded"  v ( r ii2 t r e1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "retreading"  v ( r ii2 t r e1 d i ng ) ((pos "Hb$" ) ))
( "retreads"  n ( r ii1 t r e d z ) ((pos "Kj%" ) ))
( "retreads"  v ( r ii2 t r e1 d z ) ((pos "Ha$" ) ))
( "retreat"  v ( r i1 t r ii1 t ) ((pos "I0%" "M6%" ) (subcat "2A" "2C" )))
( "retreated"  v ( r i1 t r ii1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "retreating"  v ( r i1 t r ii1 t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "retreats"  v ( r i1 t r ii1 t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2C" )))
( "retrench"  v ( r i1 t r e1 n ch ) ((pos "J1%" ) (subcat "2A" )))
( "retrenched"  v ( r i1 t r e1 n ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "retrenches"  v ( r i1 t r e1 n ch i z ) ((pos "Ja%" ) (subcat "2A" )))
( "retrenching"  v ( r i1 t r e1 n ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "retrenchment"  n ( r i1 t r e1 n ch m @ n t ) ((pos "M6%" ) ))
( "retrenchments"  n ( r i1 t r e1 n ch m @ n t s ) ((pos "Mj$" ) ))
( "retrial"  n ( r ii2 t r ai1 @ l ) ((pos "K6%" ) ))
( "retrials"  n ( r ii2 t r ai1 @ l z ) ((pos "Kj%" ) ))
( "retribution"  n ( r e2 t r i b y uu1 sh @ n ) ((pos "L@%" ) ))
( "retributive"  j ( r i1 t r i1 b y u t i v ) ((pos "OA%" ) ))
( "retrievable"  j ( r i1 t r ii1 v @ b l ) ((pos "OA%" ) ))
( "retrieval"  n ( r i1 t r ii1 v @ l ) ((pos "L@%" ) ))
( "retrieve"  v ( r i1 t r ii1 v ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "retrieved"  v ( r i1 t r ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "retriever"  n ( r i1 t r ii1 v @ r ) ((pos "K6%" ) ))
( "retrievers"  n ( r i1 t r ii1 v @ z ) ((pos "Kj%" ) ))
( "retrieves"  v ( r i1 t r ii1 v z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "retrieving"  v ( r i1 t r ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "retroactive"  j ( r e2 t r ou a1 k t i v ) ((pos "OA%" ) ))
( "retroactively"  a ( r e2 t r ou a1 k t i v l ii ) ((pos "Pu%" ) ))
( "retrograde"  v ( r e1 t r @ g r ei d ) ((pos "I2$" "OA%" ) (subcat )))
( "retrograded"  v ( r e1 t r @ g r ei d i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "retrogrades"  v ( r e1 t r @ g r ei d z ) ((pos "Ia$" ) (subcat )))
( "retrograding"  v ( r e1 t r @ g r ei d i ng ) ((pos "Ib$" ) (subcat )))
( "retrogress"  v ( r e2 t r @ g r e1 s ) ((pos "I1$" ) (subcat )))
( "retrogressed"  v ( r e2 t r @ g r e1 s t ) ((pos "Ic$" "Id$" ) (subcat )))
( "retrogresses"  v ( r e2 t r @ g r e1 s i z ) ((pos "Ia$" ) (subcat )))
( "retrogressing"  v ( r e2 t r @ g r e1 s i ng ) ((pos "Ib$" ) (subcat )))
( "retrogression"  n ( r e2 t r @ g r e1 sh @ n ) ((pos "M6$" ) ))
( "retrogressions"  n ( r e2 t r @ g r e1 sh @ n z ) ((pos "Mj$" ) ))
( "retrogressive"  j ( r e2 t r @ g r e1 s i v ) ((pos "OA%" ) ))
( "retrorocket"  n ( r e1 t r ou r o k i t ) ((pos "K6%" ) ))
( "retrorockets"  n ( r e1 t r ou r o k i t s ) ((pos "Kj%" ) ))
( "retrospect"  n ( r e1 t r @ s p e k t ) ((pos "L@%" ) ))
( "retrospection"  n ( r e2 t r @ s p e1 k sh @ n ) ((pos "M6%" ) ))
( "retrospections"  n ( r e2 t r @ s p e1 k sh @ n z ) ((pos "Mj$" ) ))
( "retrospective"  j ( r e2 t r @ s p e1 k t i v ) ((pos "OA%" ) ))
( "retrospectively"  a ( r e2 t r @ s p e1 k t i v l ii ) ((pos "Pu%" ) ))
( "retrouss_e"  j ( r @1 t r uu1 s ei ) ((pos "OA%" ) ))
( "retroversion"  n ( r e2 t r ou v @@1 sh @ n ) ((pos "M6$" ) ))
( "retroversions"  n ( r e2 t r ou v @@1 sh @ n z ) ((pos "Mj$" ) ))
( "rets"  v ( r e1 t s ) ((pos "Ha$" ) (subcat )))
( "retsina"  n ( r e1 t s ii1 n @ ) ((pos "L@%" ) ))
( "retted"  v ( r e1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "retting"  v ( r e1 t i ng ) ((pos "Hb$" ) (subcat )))
( "return"  v ( r i1 t @@1 n ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" "13A" "15A" )))
( "returnable"  j ( r i1 t @@1 n @ b l ) ((pos "OA%" ) ))
( "returned"  v ( r i1 t @@1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" "13A" "15A" )))
( "returning"  v ( r i1 t @@1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" "13A" "15A" )))
( "returns"  v ( r i1 t @@1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "4A" "6A" "12A" "13A" "15A" )))
( "reunification"  n ( r ii1 y uu2 n i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "reunion"  n ( r ii2 y uu1 n i@ n ) ((pos "M6%" ) ))
( "reunions"  n ( r ii2 y uu1 n i@ n z ) ((pos "Mj%" ) ))
( "reunite"  v ( r ii2 y uu n ai1 t ) ((pos "J2%" ) (subcat "2A" )))
( "reunited"  v ( r ii2 y uu n ai1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reunites"  v ( r ii2 y uu n ai1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "reuniting"  v ( r ii2 y uu n ai1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rev"  v ( r e1 v ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "revaluation"  n ( r ii2 v a l y uu ei1 sh @ n ) ((pos "M6%" ) ))
( "revaluations"  n ( r ii2 v a l y uu ei1 sh @ n z ) ((pos "Mj%" ) ))
( "revalue"  v ( r ii1 v a1 l y uu ) ((pos "H2%" ) (subcat )))
( "revalued"  v ( r ii1 v a1 l y uu d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "revalues"  v ( r ii1 v a1 l y uu z ) ((pos "Ha%" ) (subcat )))
( "revaluing"  v ( r ii1 v a1 l y uu i ng ) ((pos "Hb%" ) (subcat )))
( "revamp"  v ( r ii2 v a1 m p ) ((pos "H0%" ) (subcat )))
( "revamped"  v ( r ii2 v a1 m p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "revamping"  v ( r ii2 v a1 m p i ng ) ((pos "Hb%" ) (subcat )))
( "revamps"  v ( r ii2 v a1 m p s ) ((pos "Ha%" ) (subcat )))
( "reveal"  v ( r i1 v ii1 l ) ((pos "H0%" ) (subcat "6A" "9" "14" )))
( "revealed"  v ( r i1 v ii1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" )))
( "revealing"  v ( r i1 v ii1 l i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" )))
( "reveals"  v ( r i1 v ii1 l z ) ((pos "Ha%" ) (subcat "6A" "9" "14" )))
( "reveille"  n ( r i1 v a1 l ii ) ((pos "K6%" ) ))
( "reveilles"  n ( r i1 v a1 l i z ) ((pos "Kj%" ) ))
( "revel"  v ( r e1 v @ l ) ((pos "I4%" "M6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "revelation"  n ( r e2 v @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "revelations"  n ( r e2 v @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "revelled"  v ( r e1 v @ l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" "3A" )))
( "reveller"  n ( r e1 v @ l @ r ) ((pos "K6%" ) ))
( "revellers"  n ( r e1 v @ l @ z ) ((pos "Kj%" ) ))
( "revelling"  v ( r e1 v @ l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" "3A" )))
( "revelries"  n ( r e1 v @ l r i z ) ((pos "Mj%" ) ))
( "revelry"  n ( r e1 v @ l r ii ) ((pos "M8%" ) ))
( "revels"  v ( r e1 v @ l z ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "revenge"  v ( r i1 v e1 n jh ) ((pos "H2%" "L@%" ) (subcat )))
( "revenged"  v ( r i1 v e1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "revengeful"  j ( r i1 v e1 n jh f @ l ) ((pos "OA%" ) ))
( "revengefully"  a ( r i1 v e1 n jh f @ l ii ) ((pos "Pu%" ) ))
( "revenges"  v ( r i1 v e1 n jh i z ) ((pos "Ha%" ) (subcat )))
( "revenging"  v ( r i1 v e1 n jh i ng ) ((pos "Hb%" ) (subcat )))
( "revenue"  n ( r e1 v @ n y uu ) ((pos "M6%" ) ))
( "revenues"  n ( r e1 v @ n y uu z ) ((pos "Mj%" ) ))
( "reverberant"  j ( r i1 v @@1 b @ r @ n t ) ((pos "OA$" ) ))
( "reverberate"  v ( r i1 v @@1 b @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "reverberated"  v ( r i1 v @@1 b @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reverberates"  v ( r i1 v @@1 b @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "reverberating"  v ( r i1 v @@1 b @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reverberation"  n ( r i1 v @@2 b @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "reverberations"  n ( r i1 v @@2 b @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "revere"  v ( r i1 v i@1 r ) ((pos "H2%" ) (subcat )))
( "revered"  v ( r i1 v i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reverence"  v ( r e1 v @ r @ n s ) ((pos "H2%" "L@%" ) (subcat )))
( "reverenced"  v ( r e1 v @ r @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reverences"  v ( r e1 v @ r @ n s i z ) ((pos "Ha%" ) (subcat )))
( "reverencing"  v ( r e1 v @ r @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "reverend"  n ( r e1 v @ r @ n d ) ((pos "K6%" "OA%" ) ))
( "reverends"  n ( r e1 v @ r @ n d z ) ((pos "Kj%" ) ))
( "reverent"  j ( r e1 v @ r @ n t ) ((pos "OA%" ) ))
( "reverential"  j ( r e2 v @ r e1 n sh l ) ((pos "OA%" ) ))
( "reverentially"  a ( r e2 v @ r e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "reverently"  a ( r e1 v @ r @ n t l ii ) ((pos "Pu%" ) ))
( "reveres"  v ( r i1 v i@1 z ) ((pos "Ha%" ) (subcat )))
( "reverie"  n ( r e1 v @ r ii ) ((pos "M6%" ) ))
( "reveries"  n ( r e1 v @ r i z ) ((pos "Mj%" ) ))
( "revering"  v ( r i1 v i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "revers"  n ( r i1 v i@1 r ) ((pos "Ki$" ) ))
( "revers"  n ( r i1 v i@1 z ) ((pos "Kj$" ) ))
( "reversal"  n ( r i1 v @@1 s l ) ((pos "M6%" ) ))
( "reversals"  n ( r i1 v @@1 s l z ) ((pos "Mj%" ) ))
( "reverse"  v ( r i1 v @@1 s ) ((pos "J2%" "M6%" "OA%" ) (subcat "2A" )))
( "reversed"  v ( r i1 v @@1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "reversely"  a ( r i1 v @@1 s l ii ) ((pos "Pu$" ) ))
( "reverses"  v ( r i1 v @@1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "reversibility"  n ( r i1 v @@2 s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "reversible"  j ( r i1 v @@1 s @ b l ) ((pos "OA%" ) ))
( "reversing"  v ( r i1 v @@1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "reversion"  n ( r i1 v @@1 sh @ n ) ((pos "M6%" ) ))
( "reversionary"  j ( r i1 v @@1 sh @ n @ r ii ) ((pos "OA$" ) ))
( "reversions"  n ( r i1 v @@1 sh @ n z ) ((pos "Mj$" ) ))
( "revert"  v ( r i1 v @@1 t ) ((pos "I0%" ) (subcat "2A" )))
( "reverted"  v ( r i1 v @@1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "revertible"  j ( r i1 v @@1 t @ b l ) ((pos "OA$" ) ))
( "reverting"  v ( r i1 v @@1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "reverts"  v ( r i1 v @@1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "revetment"  n ( r i1 v e1 t m @ n t ) ((pos "K6$" ) ))
( "revetments"  n ( r i1 v e1 t m @ n t s ) ((pos "Kj$" ) ))
( "review"  v ( r i1 v y uu1 ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "reviewed"  v ( r i1 v y uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "reviewer"  n ( r i1 v y uu1 @ r ) ((pos "K6%" ) ))
( "reviewers"  n ( r i1 v y uu1 @ z ) ((pos "Kj%" ) ))
( "reviewing"  v ( r i1 v y uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "reviews"  v ( r i1 v y uu1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "revile"  v ( r i1 v ai1 l ) ((pos "J2%" ) (subcat "3A" )))
( "reviled"  v ( r i1 v ai1 l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "reviles"  v ( r i1 v ai1 l z ) ((pos "Ja%" ) (subcat "3A" )))
( "reviling"  v ( r i1 v ai1 l i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "revise"  v ( r i1 v ai1 z ) ((pos "H2%" "K6%" ) (subcat )))
( "revised"  v ( r i1 v ai1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "reviser"  n ( r i1 v ai1 z @ r ) ((pos "K6%" ) ))
( "revisers"  n ( r i1 v ai1 z @ z ) ((pos "Kj%" ) ))
( "revises"  v ( r i1 v ai1 z i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "revising"  v ( r i1 v ai1 z i ng ) ((pos "Hb%" ) (subcat )))
( "revision"  n ( r i1 v i1 zh n ) ((pos "M6%" ) ))
( "revisionism"  n ( r i1 v i1 zh @ n i z @ m ) ((pos "L@%" ) ))
( "revisionist"  n ( r i1 v i1 zh @ n i s t ) ((pos "K6%" ) ))
( "revisionists"  n ( r i1 v i1 zh @ n i s t s ) ((pos "Kj%" ) ))
( "revisions"  n ( r i1 v i1 zh n z ) ((pos "Mj%" ) ))
( "revisit"  v ( r ii1 v i1 z i t ) ((pos "H0%" ) (subcat "2C" "3A" "6A" )))
( "revisited"  v ( r ii1 v i1 z i t i d ) ((pos "Hc%" "Hd%" ) (subcat "2C" "3A" "6A" )))
( "revisiting"  v ( r ii1 v i1 z i t i ng ) ((pos "Hb%" ) (subcat "2C" "3A" "6A" )))
( "revisits"  v ( r ii1 v i1 z i t s ) ((pos "Ha%" ) (subcat "2C" "3A" "6A" )))
( "revitalization"  n ( r ii2 v ai2 t @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "revitalizations"  n ( r ii2 v ai2 t @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "revitalize"  v ( r ii1 v ai1 t @ l ai z ) ((pos "H2%" ) ))
( "revitalized"  v ( r ii1 v ai1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "revitalizes"  v ( r ii1 v ai1 t @ l ai z i z ) ((pos "Ha%" ) ))
( "revitalizing"  v ( r ii1 v ai1 t @ l ai z i ng ) ((pos "Hb%" ) ))
( "revival"  n ( r i1 v ai1 v @ l ) ((pos "M6%" ) ))
( "revivalist"  n ( r i1 v ai1 v @ l i s t ) ((pos "K6%" ) ))
( "revivalists"  n ( r i1 v ai1 v @ l i s t s ) ((pos "Kj%" ) ))
( "revivals"  n ( r i1 v ai1 v @ l z ) ((pos "Mj%" ) ))
( "revive"  v ( r i1 v ai1 v ) ((pos "J2%" ) (subcat "2A" )))
( "revived"  v ( r i1 v ai1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "revives"  v ( r i1 v ai1 v z ) ((pos "Ja%" ) (subcat "2A" )))
( "revivified"  v ( r ii1 v i1 v i f ai d ) ((pos "Hc%" "Hd%" ) ))
( "revivifies"  v ( r ii1 v i1 v i f ai z ) ((pos "Ha%" ) ))
( "revivify"  v ( r ii1 v i1 v i f ai ) ((pos "H3%" ) ))
( "revivifying"  v ( r ii1 v i1 v i f ai i ng ) ((pos "Hb%" ) ))
( "reviving"  v ( r i1 v ai1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "revocable"  j ( r e1 v @ k @ b l ) ((pos "OA$" ) ))
( "revocation"  n ( r e2 v @ k ei1 sh @ n ) ((pos "M6%" ) ))
( "revocations"  n ( r e2 v @ k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "revoke"  v ( r i1 v ou1 k ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "revoked"  v ( r i1 v ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "revokes"  v ( r i1 v ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "revoking"  v ( r i1 v ou1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "revolt"  v ( r i1 v ou1 l t ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" )))
( "revolted"  v ( r i1 v ou1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "revolting"  v ( r i1 v ou1 l t i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" )))
( "revoltingly"  a ( r i1 v ou1 l t i ng l ii ) ((pos "Pu%" ) ))
( "revolts"  v ( r i1 v ou1 l t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" )))
( "revolution"  n ( r e2 v @ l uu1 sh @ n ) ((pos "M6%" ) ))
( "revolutionaries"  n ( r e2 v @ l uu1 sh @ n @ r i z ) ((pos "Kj%" ) ))
( "revolutionary"  n ( r e2 v @ l uu1 sh @ n r ii ) ((pos "K6%" "OA%" ) ))
( "revolutionize"  v ( r e2 v @ l uu1 sh @ n ai z ) ((pos "H2%" ) (subcat )))
( "revolutionized"  v ( r e2 v @ l uu1 sh @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "revolutionizes"  v ( r e2 v @ l uu1 sh @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "revolutionizing"  v ( r e2 v @ l uu1 sh @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "revolutions"  n ( r e2 v @ l uu1 sh @ n z ) ((pos "Mj%" ) ))
( "revolve"  v ( r i1 v o1 l v ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "revolved"  v ( r i1 v o1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "revolver"  n ( r i1 v o1 l v @ r ) ((pos "K6%" ) ))
( "revolvers"  n ( r i1 v o1 l v @ z ) ((pos "Kj%" ) ))
( "revolves"  v ( r i1 v o1 l v z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "revolving"  v ( r i1 v o1 l v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "revs"  v ( r e1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "revue"  n ( r i1 v y uu1 ) ((pos "M6%" ) ))
( "revues"  n ( r i1 v y uu1 z ) ((pos "Mj%" ) ))
( "revulsion"  n ( r i1 v uh1 l sh @ n ) ((pos "L@%" ) ))
( "revved"  v ( r e1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "revving"  v ( r e1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "reward"  v ( r i1 w oo1 d ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "rewarded"  v ( r i1 w oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "rewarding"  v ( r i1 w oo1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "rewards"  v ( r i1 w oo1 d z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "rewire"  v ( r ii2 w ai1 @ r ) ((pos "H2%" ) ))
( "rewired"  v ( r ii2 w ai1 @ d ) ((pos "Hc%" "Hd%" ) ))
( "rewires"  v ( r ii2 w ai1 @ z ) ((pos "Ha%" ) ))
( "rewiring"  v ( r ii2 w ai1 @ r i ng ) ((pos "Hb%" ) ))
( "reword"  v ( r ii2 w @@1 d ) ((pos "H0%" ) (subcat )))
( "reworded"  v ( r ii2 w @@1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rewording"  v ( r ii2 w @@1 d i ng ) ((pos "Hb%" ) (subcat )))
( "rewords"  v ( r ii2 w @@1 d z ) ((pos "Ha%" ) (subcat )))
( "rewrite"  n ( r ii1 r ai t ) ((pos "K6%" ) ))
( "rewrite"  v ( r ii2 r ai1 t ) ((pos "H5%" ) (subcat )))
( "rewrites"  n ( r ii1 r ai t s ) ((pos "Kj%" ) ))
( "rewrites"  v ( r ii2 r ai1 t s ) ((pos "Ha%" ) (subcat )))
( "rewriting"  v ( r ii2 r ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "rewritten"  v ( r ii2 r i1 t @ n ) ((pos "Hd%" ) (subcat )))
( "rewrote"  v ( r ii2 r ou1 t ) ((pos "Hc%" ) (subcat )))
( "rhapsodies"  n ( r a1 p s @ d i z ) ((pos "Kj%" ) ))
( "rhapsodize"  v ( r a1 p s @ d ai z ) ((pos "I2%" ) (subcat "2A" )))
( "rhapsodized"  v ( r a1 p s @ d ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "rhapsodizes"  v ( r a1 p s @ d ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "rhapsodizing"  v ( r a1 p s @ d ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "rhapsody"  n ( r a1 p s @ d ii ) ((pos "K8%" ) ))
( "rhea"  n ( r i@1 ) ((pos "K6$" ) ))
( "rheas"  n ( r i@1 z ) ((pos "Kj$" ) ))
( "rheostat"  n ( r ii1 @ s t a t ) ((pos "K6%" ) ))
( "rheostats"  n ( r ii1 @ s t a t s ) ((pos "Kj%" ) ))
( "rhesus"  n ( r ii1 s @ s ) ((pos "K7%" ) ))
( "rhesuses"  n ( r ii1 s @ s i z ) ((pos "Kj$" ) ))
( "rhetoric"  n ( r e1 t @ r i k ) ((pos "L@%" ) ))
( "rhetorical"  j ( r i1 t o1 r i k l ) ((pos "OA%" ) ))
( "rhetorically"  a ( r i1 t o1 r i k l ii ) ((pos "Pu%" ) ))
( "rhetorician"  n ( r e2 t @ r i1 sh @ n ) ((pos "K6%" ) ))
( "rhetoricians"  n ( r e2 t @ r i1 sh @ n z ) ((pos "Kj%" ) ))
( "rheum"  n ( r uu1 m ) ((pos "L@$" ) ))
( "rheumatic"  n ( r uu1 m a1 t i k ) ((pos "K6%" "OA%" ) ))
( "rheumatics"  n ( r uu1 m a1 t i k s ) ((pos "Kj%" ) ))
( "rheumatism"  n ( r uu1 m @ t i z @ m ) ((pos "L@%" ) ))
( "rheumatoid"  j ( r uu1 m @ t oi d ) ((pos "OA%" ) ))
( "rhinal"  j ( r ai1 n l ) ((pos "OA$" ) ))
( "rhino"  n ( r ai1 n ou ) ((pos "K6%" ) ))
( "rhinoceros"  n ( r ai1 n o1 s @ r @ s ) ((pos "K9%" ) ))
( "rhinoceroses"  n ( r ai1 n o1 s @ r @ s i z ) ((pos "Kj%" ) ))
( "rhinos"  n ( r ai1 n ou z ) ((pos "Kj%" ) ))
( "rhizome"  n ( r ai1 z ou m ) ((pos "K6$" ) ))
( "rhizomes"  n ( r ai1 z ou m z ) ((pos "Kj$" ) ))
( "rhododendron"  n ( r ou2 d @ d e1 n d r @ n ) ((pos "K6%" ) ))
( "rhododendrons"  n ( r ou2 d @ d e1 n d r @ n z ) ((pos "Kj%" ) ))
( "rhomb"  n ( r o1 m ) ((pos "K6$" ) ))
( "rhomboid"  n ( r o1 m b oi d ) ((pos "K6$" "OA$" ) ))
( "rhomboids"  n ( r o1 m b oi d z ) ((pos "Kj$" ) ))
( "rhombs"  n ( r o1 m z ) ((pos "Kj$" ) ))
( "rhombus"  n ( r o1 m b @ s ) ((pos "K7%" ) ))
( "rhombuses"  n ( r o1 m b @ s i z ) ((pos "Kj%" ) ))
( "rhubarb"  n ( r uu1 b aa b ) ((pos "L@%" ) ))
( "rhyme"  v ( r ai1 m ) ((pos "J2%" "M6%" ) (subcat "2A" "3A" )))
( "rhymed"  v ( r ai1 m d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" )))
( "rhymes"  v ( r ai1 m z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" )))
( "rhymester"  n ( r ai1 m s t @ r ) ((pos "K6$" ) ))
( "rhymesters"  n ( r ai1 m s t @ z ) ((pos "Kj$" ) ))
( "rhyming"  v ( r ai1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "rhythm"  n ( r i1 dh @ m ) ((pos "M6%" ) ))
( "rhythmic"  j ( r i1 dh m i k ) ((pos "OA%" ) ))
( "rhythmical"  j ( r i1 dh m i k l ) ((pos "OA%" ) ))
( "rhythmically"  a ( r i1 dh m i k l ii ) ((pos "Pu%" ) ))
( "rhythms"  n ( r i1 dh @ m z ) ((pos "Mj%" ) ))
( "rib"  v ( r i1 b ) ((pos "H4%" "K6%" ) (subcat )))
( "ribald"  n ( r i1 b @ l d ) ((pos "K6$" "OA%" ) ))
( "ribaldry"  n ( r i1 b @ l d r ii ) ((pos "L@%" ) ))
( "ribalds"  n ( r i1 b @ l d z ) ((pos "Kj$" ) ))
( "riband"  n ( r i1 b @ n d ) ((pos "K6%" ) ))
( "ribands"  n ( r i1 b @ n d z ) ((pos "Kj$" ) ))
( "ribbed"  v ( r i1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ribbing"  v ( r i1 b i ng ) ((pos "Hb%" ) (subcat )))
( "ribbon"  n ( r i1 b @ n ) ((pos "M6%" ) ))
( "ribbons"  n ( r i1 b @ n z ) ((pos "Mj%" ) ))
( "riboflavin"  n ( r ai2 b ou f l ei1 v i n ) ((pos "L@%" ) ))
( "ribs"  v ( r i1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "rice"  n ( r ai1 s ) ((pos "L@%" ) ))
( "rice-paper"  n ( r ai1 s - p ei p @ r ) ((pos "L@%" ) ))
( "rich"  j ( r i1 ch ) ((pos "OC%" ) ))
( "richer"  j ( r i1 ch @ r ) ((pos "Or%" ) ))
( "riches"  n ( r i1 ch i z ) ((pos "Kj%" ) ))
( "richest"  j ( r i1 ch i s t ) ((pos "Os%" ) ))
( "richly"  a ( r i1 ch l ii ) ((pos "Pu%" ) ))
( "richness"  n ( r i1 ch n @ s ) ((pos "L@%" ) ))
( "rick"  v ( r i1 k ) ((pos "H0%" "K6%" ) (subcat )))
( "ricked"  v ( r i1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rickets"  n ( r i1 k i t s ) ((pos "Lk%" ) ))
( "rickety"  j ( r i1 k @ t ii ) ((pos "OA%" ) ))
( "ricking"  v ( r i1 k i ng ) ((pos "Hb%" ) (subcat )))
( "ricks"  v ( r i1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "rickshaw"  n ( r i1 k sh oo ) ((pos "K6%" ) ))
( "rickshaws"  n ( r i1 k sh oo z ) ((pos "Kj%" ) ))
( "ricochet"  v ( r i1 k @ sh ei ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "ricocheted"  v ( r i1 k @ sh ei d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ricocheting"  v ( r i1 k @ sh ei i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "ricochets"  v ( r i1 k @ sh ei z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "ricochetted"  v ( r i1 k @ sh ei d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "ricochetting"  v ( r i1 k @ sh ei i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "rid"  v ( r i1 d ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "riddance"  n ( r i1 d n s ) ((pos "L@%" ) ))
( "ridden"  v ( r i1 d n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" )))
( "ridding"  v ( r i1 d i ng ) ((pos "Hb%" ) (subcat )))
( "riddle"  v ( r i1 d l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "riddled"  v ( r i1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "riddles"  v ( r i1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "riddling"  v ( r i1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ride"  v ( r ai1 d ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" )))
( "rider"  n ( r ai1 d @ r ) ((pos "K6%" ) ))
( "riderless"  j ( r ai1 d @ l @ s ) ((pos "OA%" ) ))
( "riders"  n ( r ai1 d @ z ) ((pos "Kj%" ) ))
( "rides"  v ( r ai1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" )))
( "ridge"  v ( r i1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "ridge-tile"  n ( r i1 jh - t ai l ) ((pos "K6%" ) ))
( "ridge-tiles"  n ( r i1 jh - t ai l z ) ((pos "Kj%" ) ))
( "ridged"  v ( r i1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ridgepole"  n ( r i1 jh p ou l ) ((pos "K6%" ) ))
( "ridgepoles"  n ( r i1 jh p ou l z ) ((pos "Kj%" ) ))
( "ridges"  v ( r i1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ridging"  v ( r i1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "ridicule"  v ( r i1 d i k y uu l ) ((pos "H2%" "K6%" ) ))
( "ridiculed"  v ( r i1 d i k y uu l d ) ((pos "Hc%" "Hd%" ) ))
( "ridicules"  v ( r i1 d i k y uu l z ) ((pos "Ha%" "Kj%" ) ))
( "ridiculing"  v ( r i1 d i k y uu l i ng ) ((pos "Hb%" ) ))
( "ridiculous"  j ( r i1 d i1 k y u l @ s ) ((pos "OA%" ) ))
( "ridiculously"  a ( r i1 d i1 k y u l @ s l ii ) ((pos "Pu%" ) ))
( "riding"  v ( r ai1 d i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" )))
( "riding-breeches"  n ( r ai1 d i ng - b r i ch i z ) ((pos "Kj%" ) ))
( "riding-habit"  n ( r ai1 d i ng - h a b i t ) ((pos "K6%" ) ))
( "riding-habits"  n ( r ai1 d i ng - h a b i t s ) ((pos "Kj$" ) ))
( "riding-lamp"  n ( r ai1 d i ng - l a m p ) ((pos "K6%" ) ))
( "riding-lamps"  n ( r ai1 d i ng - l a m p s ) ((pos "Kj%" ) ))
( "riding-light"  n ( r ai1 d i ng - l ai t ) ((pos "K6%" ) ))
( "riding-lights"  n ( r ai1 d i ng - l ai t s ) ((pos "Kj%" ) ))
( "riding-master"  n ( r ai1 d i ng - m aa s t @ r ) ((pos "K6%" ) ))
( "riding-masters"  n ( r ai1 d i ng - m aa s t @ z ) ((pos "Kj%" ) ))
( "riding-school"  n ( r ai1 d i ng - s k uu l ) ((pos "K6%" ) ))
( "riding-schools"  n ( r ai1 d i ng - s k uu l z ) ((pos "Kj%" ) ))
( "ridings"  n ( r ai1 d i ng z ) ((pos "Mj%" ) ))
( "rids"  v ( r i1 d z ) ((pos "Ha%" ) (subcat )))
( "rife"  j ( r ai1 f ) ((pos "Op%" ) ))
( "riff"  n ( r i1 f ) ((pos "K6$" ) ))
( "riff-raff"  n ( r i1 f - r a f ) ((pos "Mi%" ) ))
( "riffle"  v ( r i1 f l ) ((pos "J2$" ) (subcat "3A" )))
( "riffled"  v ( r i1 f l d ) ((pos "Jc$" "Jd$" ) (subcat "3A" )))
( "riffles"  v ( r i1 f l z ) ((pos "Ja$" ) (subcat "3A" )))
( "riffling"  v ( r i1 f l i ng ) ((pos "Jb$" ) (subcat "3A" )))
( "riffs"  n ( r i1 f s ) ((pos "Kj$" ) ))
( "rifle"  v ( r ai1 f @ l ) ((pos "H2%" "K6%" ) (subcat )))
( "rifle-range"  n ( r ai1 f l - r ei n jh ) ((pos "K6%" ) ))
( "rifle-ranges"  n ( r ai1 f l - r ei n jh i z ) ((pos "Kj%" ) ))
( "rifle-shot"  n ( r ai1 f l - sh o t ) ((pos "K6%" ) ))
( "rifle-shots"  n ( r ai1 f l - sh o t s ) ((pos "Kj%" ) ))
( "rifled"  v ( r ai1 f @ l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rifleman"  n ( r ai1 f l m @ n ) ((pos "Ki%" ) ))
( "riflemen"  n ( r ai1 f l m @ n ) ((pos "Kj%" ) ))
( "rifles"  v ( r ai1 f @ l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "rifling"  v ( r ai1 f l i ng ) ((pos "Hb%" ) (subcat )))
( "rift"  n ( r i1 f t ) ((pos "K6%" ) ))
( "rift-valley"  n ( r i1 f t - v a1 l ii ) ((pos "K6%" ) ))
( "rift-valleys"  n ( r i1 f t - v a1 l i z ) ((pos "Kj%" ) ))
( "rifts"  n ( r i1 f t s ) ((pos "Kj%" ) ))
( "rig"  v ( r i1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "rigged"  v ( r i1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "rigger"  n ( r i1 g @ r ) ((pos "K6$" ) ))
( "riggers"  n ( r i1 g @ z ) ((pos "Kj$" ) ))
( "rigging"  v ( r i1 g i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "14" )))
( "right"  v ( r ai1 t ) ((pos "H0%" "M6%" "OA*" "Pu*" ) (subcat )))
( "right-angled"  j ( r ai1 t - a ng g l d ) ((pos "OA%" ) ))
( "right-down"  j ( r ai1 t - d au n ) ((pos "OA$" "Pu$" ) ))
( "right-hand"  j ( r ai1 t - h a n d ) ((pos "OA%" ) ))
( "right-handed"  j ( r ai1 t - h a1 n d i d ) ((pos "OA%" ) ))
( "right-hander"  n ( r ai1 t - h a1 n d @ r ) ((pos "K6%" ) ))
( "right-handers"  n ( r ai1 t - h a1 n d @ z ) ((pos "Kj%" ) ))
( "right-minded"  j ( r ai1 t - m ai n d i d ) ((pos "OA%" ) ))
( "right-turn"  n ( r ai1 t - t @@1 n ) ((pos "K6%" ) ))
( "right-turns"  n ( r ai1 t - t @@1 n z ) ((pos "Kj%" ) ))
( "right-wing"  n ( r ai1 t - w i1 ng ) ((pos "K6%" ) ))
( "right-winger"  n ( r ai1 t - w i1 ng @ r ) ((pos "K6%" ) ))
( "right-wingers"  n ( r ai1 t - w i1 ng @ z ) ((pos "Kj%" ) ))
( "right-wings"  n ( r ai1 t - w i1 ng z ) ((pos "Kj%" ) ))
( "righted"  v ( r ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "righteous"  j ( r ai1 ch @ s ) ((pos "OA%" ) ))
( "righteously"  a ( r ai1 ch @ s l ii ) ((pos "Pu%" ) ))
( "righteousness"  n ( r ai1 ch @ s n @ s ) ((pos "L@%" ) ))
( "rightful"  j ( r ai1 t f @ l ) ((pos "OA%" ) ))
( "rightfully"  a ( r ai1 t f @ l ii ) ((pos "Pu%" ) ))
( "rightfulness"  n ( r ai1 t f l n @ s ) ((pos "L@$" ) ))
( "righting"  v ( r ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "rightist"  n ( r ai1 t i s t ) ((pos "K6$" "OA$" ) ))
( "rightists"  n ( r ai1 t i s t s ) ((pos "Kj$" ) ))
( "rightly"  a ( r ai1 t l ii ) ((pos "Pu%" ) ))
( "rightness"  n ( r ai1 t n @ s ) ((pos "L@%" ) ))
( "rights"  v ( r ai1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "rigid"  j ( r i1 jh i d ) ((pos "OA%" ) ))
( "rigidities"  n ( r i1 jh i1 d i t i z ) ((pos "Mj%" ) ))
( "rigidity"  n ( r i1 jh i1 d i t ii ) ((pos "M8%" ) ))
( "rigidly"  a ( r i1 jh i d l ii ) ((pos "Pu%" ) ))
( "rigmarole"  n ( r i1 g m @ r ou l ) ((pos "M6%" ) ))
( "rigmaroles"  n ( r i1 g m @ r ou l z ) ((pos "Mj%" ) ))
( "rigor mortis"  n ( r i2 g @ - m oo1 t i s ) ((pos "L@%" ) ))
( "rigorous"  j ( r i1 g @ r @ s ) ((pos "OA%" ) ))
( "rigorously"  a ( r i1 g @ r @ s l ii ) ((pos "Pu%" ) ))
( "rigour"  n ( r i1 g @ r ) ((pos "M6%" ) ))
( "rigours"  n ( r i1 g @ z ) ((pos "Mj%" ) ))
( "rigout"  n ( r i1 g au t ) ((pos "K6%" ) ))
( "rigouts"  n ( r i1 g au t s ) ((pos "Kj%" ) ))
( "rigs"  v ( r i1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "rile"  v ( r ai1 l ) ((pos "H2%" ) (subcat )))
( "riled"  v ( r ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "riles"  v ( r ai1 l z ) ((pos "Ha%" ) (subcat )))
( "riling"  v ( r ai1 l i ng ) ((pos "Hb%" ) (subcat )))
( "rill"  n ( r i1 l ) ((pos "K6$" ) ))
( "rills"  n ( r i1 l z ) ((pos "Kj$" ) ))
( "rim"  v ( r i1 m ) ((pos "H4%" "K6%" ) (subcat )))
( "rime"  v ( r ai1 m ) ((pos "H2$" "L@$" ) (subcat )))
( "rimed"  v ( r ai1 m d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rimes"  v ( r ai1 m z ) ((pos "Ha$" ) (subcat )))
( "riming"  v ( r ai1 m i ng ) ((pos "Hb$" ) (subcat )))
( "rimmed"  v ( r i1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rimming"  v ( r i1 m i ng ) ((pos "Hb%" ) (subcat )))
( "rims"  v ( r i1 m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "rind"  n ( r ai1 n d ) ((pos "M6%" ) ))
( "rinderpest"  n ( r i1 n d @ p e s t ) ((pos "L@$" ) ))
( "rinds"  n ( r ai1 n d z ) ((pos "Mj%" ) ))
( "ring"  v ( r i1 ng ) ((pos "J0%" "J5%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "ring-armour"  n ( r i1 ng - aa m @ r ) ((pos "L@$" ) ))
( "ring-finger"  n ( r i1 ng - f i ng g @ r ) ((pos "K6%" ) ))
( "ring-fingers"  n ( r i1 ng - f i ng g @ z ) ((pos "Kj%" ) ))
( "ring-mail"  n ( r i1 ng - m ei l ) ((pos "L@$" ) ))
( "ring-road"  n ( r i1 ng - r ou d ) ((pos "K6%" ) ))
( "ring-roads"  n ( r i1 ng - r ou d z ) ((pos "Kj%" ) ))
( "ringed"  v ( r i1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "ringer"  n ( r i1 ng @ r ) ((pos "K6%" ) ))
( "ringers"  n ( r i1 ng @ z ) ((pos "Kj%" ) ))
( "ringing"  v ( r i1 ng i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "ringleader"  n ( r i1 ng l ii d @ r ) ((pos "K6%" ) ))
( "ringleaders"  n ( r i1 ng l ii d @ z ) ((pos "Kj%" ) ))
( "ringlet"  n ( r i1 ng l i t ) ((pos "K6%" ) ))
( "ringlets"  n ( r i1 ng l i t s ) ((pos "Kj%" ) ))
( "ringmaster"  n ( r i1 ng m aa s t @ r ) ((pos "K6%" ) ))
( "ringmasters"  n ( r i1 ng m aa s t @ z ) ((pos "Kj%" ) ))
( "rings"  v ( r i1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "ringside"  n ( r i1 ng s ai d ) ((pos "K6%" ) ))
( "ringsides"  n ( r i1 ng s ai d z ) ((pos "Kj$" ) ))
( "ringworm"  n ( r i1 ng w @@ m ) ((pos "L@%" ) ))
( "rink"  n ( r i1 ng k ) ((pos "K6%" ) ))
( "rinks"  n ( r i1 ng k s ) ((pos "Kj%" ) ))
( "rinse"  v ( r i1 n s ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "rinsed"  v ( r i1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "rinses"  v ( r i1 n s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "rinsing"  v ( r i1 n s i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "riot"  v ( r ai1 @ t ) ((pos "I0%" "M6%" ) (subcat "2A" "2B" "2C" )))
( "rioted"  v ( r ai1 @ t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" )))
( "rioter"  n ( r ai1 @ t @ r ) ((pos "K6%" ) ))
( "rioters"  n ( r ai1 @ t @ z ) ((pos "Kj%" ) ))
( "rioting"  v ( r ai1 @ t i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" )))
( "riotous"  j ( r ai1 @ t @ s ) ((pos "OA%" ) ))
( "riotously"  a ( r ai1 @ t @ s l ii ) ((pos "Pu%" ) ))
( "riots"  v ( r ai1 @ t s ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2B" "2C" )))
( "rip"  v ( r i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rip-off"  n ( r i1 p - o f ) ((pos "K6%" ) ))
( "rip-offs"  n ( r i1 p - o f s ) ((pos "Kj%" ) ))
( "rip-roaring"  j ( r i1 p - r oo r i ng ) ((pos "OA%" ) ))
( "riparian"  j ( r ai1 p e@1 r i@ n ) ((pos "OA$" ) ))
( "ripcord"  n ( r i1 p k oo d ) ((pos "K6%" ) ))
( "ripcords"  n ( r i1 p k oo d z ) ((pos "Kj%" ) ))
( "ripe"  j ( r ai1 p ) ((pos "OB%" ) ))
( "ripely"  a ( r ai1 p l ii ) ((pos "Pu%" ) ))
( "ripen"  v ( r ai1 p @ n ) ((pos "J0%" ) (subcat "2A" )))
( "ripened"  v ( r ai1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ripeness"  n ( r ai1 p n @ s ) ((pos "L@%" ) ))
( "ripening"  v ( r ai1 p @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ripens"  v ( r ai1 p @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "riper"  j ( r ai1 p @ r ) ((pos "Or%" ) ))
( "ripest"  j ( r ai1 p i s t ) ((pos "Os%" ) ))
( "riposte"  v ( r i1 p o1 s t ) ((pos "I2$" "K6%" ) ))
( "riposted"  v ( r i1 p o1 s t i d ) ((pos "Ic$" "Id$" ) ))
( "ripostes"  v ( r i1 p o1 s t s ) ((pos "Ia$" "Kj$" ) ))
( "riposting"  v ( r i1 p o1 s t i ng ) ((pos "Ib$" ) ))
( "ripped"  v ( r i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "ripping"  v ( r i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "ripple"  v ( r i1 p l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "rippled"  v ( r i1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ripples"  v ( r i1 p l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "rippling"  v ( r i1 p l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rips"  v ( r i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "ripsaw"  n ( r i1 p s oo ) ((pos "K6%" ) ))
( "ripsaws"  n ( r i1 p s oo z ) ((pos "Kj%" ) ))
( "riptide"  n ( r i1 p t ai d ) ((pos "K6$" ) ))
( "riptides"  n ( r i1 p t ai d z ) ((pos "Kj$" ) ))
( "rise"  v ( r ai1 z ) ((pos "I5%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "risen"  v ( r i1 z n ) ((pos "Id%" ) (subcat "2A" "2B" "2C" "3A" )))
( "riser"  n ( r ai1 z @ r ) ((pos "K6%" ) ))
( "risers"  n ( r ai1 z @ z ) ((pos "Kj%" ) ))
( "rises"  v ( r ai1 z i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "risibility"  n ( r i2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "risible"  j ( r i1 z @ b l ) ((pos "OA%" ) ))
( "rising"  v ( r ai1 z i ng ) ((pos "Ib%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "risings"  n ( r ai1 z i ng z ) ((pos "Kj%" ) ))
( "risk"  v ( r i1 s k ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "risked"  v ( r i1 s k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "riskier"  j ( r i1 s k i@ r ) ((pos "Or%" ) ))
( "riskiest"  j ( r i1 s k i i s t ) ((pos "Os%" ) ))
( "riskily"  a ( r i1 s k i l ii ) ((pos "Pu%" ) ))
( "riskiness"  n ( r i1 s k i n @ s ) ((pos "L@%" ) ))
( "risking"  v ( r i1 s k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "risks"  v ( r i1 s k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "risky"  j ( r i1 s k ii ) ((pos "OD%" ) ))
( "risotto"  n ( r i1 z o1 t ou ) ((pos "M6%" ) ))
( "risottos"  n ( r i1 z o1 t ou z ) ((pos "Mj%" ) ))
( "risqu_e"  j ( r ii1 s k ei ) ((pos "OA%" ) ))
( "rissole"  n ( r i1 s ou l ) ((pos "K6%" ) ))
( "rissoles"  n ( r i1 s ou l z ) ((pos "Kj%" ) ))
( "rite"  n ( r ai1 t ) ((pos "K6%" ) ))
( "rites"  n ( r ai1 t s ) ((pos "Kj%" ) ))
( "ritual"  n ( r i1 ch u@ l ) ((pos "M6%" "OA%" ) ))
( "ritualism"  n ( r i1 ch u@ l i z @ m ) ((pos "L@$" ) ))
( "ritualist"  n ( r i1 ch u@ l i s t ) ((pos "K6%" ) ))
( "ritualistic"  j ( r i2 ch u@ l i1 s t i k ) ((pos "OA%" ) ))
( "ritualists"  n ( r i1 ch u@ l i s t s ) ((pos "Kj$" ) ))
( "rituals"  n ( r i1 ch u@ l z ) ((pos "Mj%" ) ))
( "ritzy"  j ( r i1 t s ii ) ((pos "OA%" ) ))
( "rival"  v ( r ai1 v @ l ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "rivalled"  v ( r ai1 v @ l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "rivalling"  v ( r ai1 v @ l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "rivalries"  n ( r ai1 v @ l r i z ) ((pos "Mj%" ) ))
( "rivalry"  n ( r ai1 v @ l r ii ) ((pos "M8%" ) ))
( "rivals"  v ( r ai1 v @ l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "rive"  v ( r ai1 v ) ((pos "J2$" ) (subcat "6A" "15A" )))
( "rived"  v ( r ai1 v d ) ((pos "Jc$" "Jd$" ) (subcat "6A" "15A" )))
( "riven"  v ( r i1 v n ) ((pos "Jd$" ) (subcat "6A" "15A" )))
( "river"  n ( r i1 v @ r ) ((pos "K6%" ) ))
( "river-basin"  n ( r i2 v @ - b ei1 s n ) ((pos "K6%" ) ))
( "river-basins"  n ( r i2 v @ - b ei1 s n z ) ((pos "Kj%" ) ))
( "river-bed"  n ( r i1 v @ - b e d ) ((pos "K6%" ) ))
( "river-beds"  n ( r i1 v @ - b e d z ) ((pos "Kj%" ) ))
( "rivers"  n ( r i1 v @ z ) ((pos "Kj%" ) ))
( "riverside"  n ( r i1 v @ s ai d ) ((pos "K6%" ) ))
( "riversides"  n ( r i1 v @ s ai d z ) ((pos "Kj%" ) ))
( "rives"  v ( r ai1 v z ) ((pos "Ja$" ) (subcat "6A" "15A" )))
( "rivet"  v ( r i1 v i t ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "riveted"  v ( r i1 v i t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "riveter"  n ( r i1 v i t @ r ) ((pos "K6%" ) ))
( "riveters"  n ( r i1 v i t @ z ) ((pos "Kj%" ) ))
( "riveting"  v ( r i1 v i t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "rivets"  v ( r i1 v i t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "riving"  v ( r ai1 v i ng ) ((pos "Jb$" ) (subcat "6A" "15A" )))
( "rivulet"  n ( r i1 v y u l i t ) ((pos "K6%" ) ))
( "rivulets"  n ( r i1 v y u l i t s ) ((pos "Kj%" ) ))
( "rly"  n ( r ei1 l w ei ) ((pos "Y>$" ) ))
( "roach"  n ( r ou1 ch ) ((pos "K9%" ) ))
( "roaches"  n ( r ou1 ch i z ) ((pos "Kj%" ) ))
( "road"  n ( r ou1 d ) ((pos "K6*" ) ))
( "road-book"  n ( r ou1 d - b u k ) ((pos "K6%" ) ))
( "road-books"  n ( r ou1 d - b u k s ) ((pos "Kj%" ) ))
( "road-hog"  n ( r ou1 d - h o g ) ((pos "K6%" ) ))
( "road-hogs"  n ( r ou1 d - h o g z ) ((pos "Kj%" ) ))
( "road-metal"  n ( r ou1 d - m e t l ) ((pos "L@$" ) ))
( "road-sense"  n ( r ou1 d - s e n s ) ((pos "L@%" ) ))
( "roadbed"  n ( r ou1 d b e d ) ((pos "K6%" ) ))
( "roadbeds"  n ( r ou1 d b e d z ) ((pos "Kj%" ) ))
( "roadblock"  n ( r ou1 d b l o k ) ((pos "K6%" ) ))
( "roadblocks"  n ( r ou1 d b l o k s ) ((pos "Kj%" ) ))
( "roadhouse"  n ( r ou1 d h au s ) ((pos "K6%" ) ))
( "roadhouses"  n ( r ou1 d h au z i z ) ((pos "Kj%" ) ))
( "roadless"  j ( r ou1 d l @ s ) ((pos "OA$" ) ))
( "roadman"  n ( r ou1 d m a n ) ((pos "Ki%" ) ))
( "roadmen"  n ( r ou1 d m e n ) ((pos "Kj%" ) ))
( "roadmender"  n ( r ou1 d m e n d @ r ) ((pos "K6%" ) ))
( "roadmenders"  n ( r ou1 d m e n d @ z ) ((pos "Kj%" ) ))
( "roads"  n ( r ou1 d z ) ((pos "Kj%" ) ))
( "roadside"  n ( r ou1 d s ai d ) ((pos "K6%" ) ))
( "roadsides"  n ( r ou1 d s ai d z ) ((pos "Kj%" ) ))
( "roadstead"  n ( r ou1 d s t e d ) ((pos "K6$" ) ))
( "roadsteads"  n ( r ou1 d s t e d z ) ((pos "Kj$" ) ))
( "roadster"  n ( r ou1 d s t @ r ) ((pos "K6$" ) ))
( "roadsters"  n ( r ou1 d s t @ z ) ((pos "Kj$" ) ))
( "roadway"  n ( r ou1 d w ei ) ((pos "K6%" ) ))
( "roadways"  n ( r ou1 d w ei z ) ((pos "Kj%" ) ))
( "roadworthy"  j ( r ou1 d w @@ dh ii ) ((pos "OA%" ) ))
( "roam"  v ( r ou1 m ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "roamed"  v ( r ou1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "roaming"  v ( r ou1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "roams"  v ( r ou1 m z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "roan"  n ( r ou1 n ) ((pos "M6$" "OA$" ) ))
( "roans"  n ( r ou1 n z ) ((pos "Mj$" ) ))
( "roar"  v ( r oo1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "roared"  v ( r oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "roaring"  v ( r oo1 r i ng ) ((pos "Jb%" "OA%" "Pu%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "roars"  v ( r oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "roast"  v ( r ou1 s t ) ((pos "J0%" "M6%" "Oq%" ) (subcat "2A" )))
( "roasted"  v ( r ou1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "roaster"  n ( r ou1 s t @ r ) ((pos "K6%" ) ))
( "roasters"  n ( r ou1 s t @ z ) ((pos "Kj%" ) ))
( "roasting"  v ( r ou1 s t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "roastings"  n ( r ou1 s t i ng z ) ((pos "Mj$" ) ))
( "roasts"  v ( r ou1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "rob"  v ( r o1 b ) ((pos "H4%" ) (subcat "6A" )))
( "robbed"  v ( r o1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "robber"  n ( r o1 b @ r ) ((pos "K6%" ) ))
( "robberies"  n ( r o1 b @ r i z ) ((pos "Mj%" ) ))
( "robbers"  n ( r o1 b @ z ) ((pos "Kj%" ) ))
( "robbery"  n ( r o1 b @ r ii ) ((pos "M8%" ) ))
( "robbing"  v ( r o1 b i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "robe"  v ( r ou1 b ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" )))
( "robed"  v ( r ou1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "robes"  v ( r ou1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "robin"  n ( r o1 b i n ) ((pos "K6%" ) ))
( "robing"  v ( r ou1 b i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "robins"  n ( r o1 b i n z ) ((pos "Kj%" ) ))
( "robot"  n ( r ou1 b o t ) ((pos "K6%" ) ))
( "robots"  n ( r ou1 b o t s ) ((pos "Kj%" ) ))
( "robs"  v ( r o1 b z ) ((pos "Ha%" ) (subcat "6A" )))
( "robust"  j ( r ou1 b uh1 s t ) ((pos "OA%" ) ))
( "robustly"  a ( r ou1 b uh1 s t l ii ) ((pos "Pu%" ) ))
( "robustness"  n ( r ou1 b uh1 s t n @ s ) ((pos "L@%" ) ))
( "roc"  n ( r o1 k ) ((pos "K6$" ) ))
( "rock"  v ( r o1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "rock-'n-roll"  n ( r o2 k - @ n - r ou1 l ) ((pos "L@%" ) ))
( "rock-bottom"  n ( r o1 k - b o1 t @ m ) ((pos "L@%" ) ))
( "rock-cake"  n ( r o1 k - k ei k ) ((pos "K6%" ) ))
( "rock-cakes"  n ( r o1 k - k ei k s ) ((pos "Kj%" ) ))
( "rock-climbing"  n ( r o1 k - k l ai m i ng ) ((pos "L@%" ) ))
( "rock-crystal"  n ( r o1 k - k r i s t l ) ((pos "K6%" ) ))
( "rock-crystals"  n ( r o1 k - k r i s t l z ) ((pos "Kj%" ) ))
( "rock-garden"  n ( r o1 k - g aa d n ) ((pos "K6%" ) ))
( "rock-gardens"  n ( r o1 k - g aa d n z ) ((pos "Kj%" ) ))
( "rock-plant"  n ( r o1 k - p l aa n t ) ((pos "K6%" ) ))
( "rock-plants"  n ( r o1 k - p l aa n t s ) ((pos "Kj%" ) ))
( "rock-salmon"  n ( r o1 k - s a1 m @ n ) ((pos "K9%" ) ))
( "rock-salt"  n ( r o1 k - s oo l t ) ((pos "L@%" ) ))
( "rocked"  v ( r o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "rocker"  n ( r o1 k @ r ) ((pos "K6%" ) ))
( "rockeries"  n ( r o1 k @ r i z ) ((pos "Kj%" ) ))
( "rockers"  n ( r o1 k @ z ) ((pos "Kj%" ) ))
( "rockery"  n ( r o1 k @ r ii ) ((pos "K8%" ) ))
( "rocket"  v ( r o1 k i t ) ((pos "I0%" "K6%" ) (subcat )))
( "rocket-base"  n ( r o1 k i t - b ei s ) ((pos "K6%" ) ))
( "rocket-bases"  n ( r o1 k i t - b ei s i z ) ((pos "Kj%" ) ))
( "rocket-range"  n ( r o1 k i t - r ei n jh ) ((pos "K6%" ) ))
( "rocket-ranges"  n ( r o1 k i t - r ei n jh i z ) ((pos "Kj%" ) ))
( "rocketed"  v ( r o1 k i t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "rocketing"  v ( r o1 k i t i ng ) ((pos "Ib%" ) (subcat )))
( "rocketry"  n ( r o1 k i t r ii ) ((pos "L@%" ) ))
( "rockets"  v ( r o1 k i t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "rockier"  j ( r o1 k i@ r ) ((pos "Or%" ) ))
( "rockiest"  j ( r o1 k i i s t ) ((pos "Os%" ) ))
( "rocking"  v ( r o1 k i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "rocking-chair"  n ( r o1 k i ng - ch e@ r ) ((pos "K6%" ) ))
( "rocking-chairs"  n ( r o1 k i ng - ch e@ z ) ((pos "Kj%" ) ))
( "rocking-horse"  n ( r o1 k i ng - h oo s ) ((pos "K6%" ) ))
( "rocking-horses"  n ( r o1 k i ng - h oo s i z ) ((pos "Kj%" ) ))
( "rocks"  v ( r o1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "rocky"  j ( r o1 k ii ) ((pos "OD%" ) ))
( "rococo"  j ( r @1 k ou1 k ou ) ((pos "OA%" ) ))
( "rocs"  n ( r o1 k s ) ((pos "Kj$" ) ))
( "rod"  n ( r o1 d ) ((pos "K6%" ) ))
( "rode"  v ( r ou1 d ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" )))
( "rodent"  n ( r ou1 d n t ) ((pos "K6%" ) ))
( "rodents"  n ( r ou1 d n t s ) ((pos "Kj%" ) ))
( "rodeo"  n ( r ou1 d ei1 ou ) ((pos "K6%" ) ))
( "rodeos"  n ( r ou1 d ei1 ou z ) ((pos "Kj%" ) ))
( "rodomontade"  n ( r o2 d @ m o n t ei1 d ) ((pos "L@$" ) ))
( "rods"  n ( r o1 d z ) ((pos "Kj%" ) ))
( "roe"  n ( r ou1 ) ((pos "M9%" ) ))
( "roebuck"  n ( r ou1 b uh k ) ((pos "K6%" ) ))
( "roebucks"  n ( r ou1 b uh k s ) ((pos "Kj%" ) ))
( "roes"  n ( r ou1 z ) ((pos "Kj%" ) ))
( "rogation"  n ( r ou1 g ei1 sh @ n ) ((pos "K6$" ) ))
( "rogations"  n ( r ou1 g ei1 sh @ n z ) ((pos "Kj$" ) ))
( "roger"  uh ( r o1 jh @ r ) ((pos "W-$" ) ))
( "rogue"  n ( r ou1 g ) ((pos "K6%" ) ))
( "rogue-elephant"  n ( r ou1 g - e1 l i f @ n t ) ((pos "K6%" ) ))
( "rogue-elephants"  n ( r ou1 g - e1 l i f @ n t s ) ((pos "Kj%" ) ))
( "rogueries"  n ( r ou1 g @ r i z ) ((pos "Mj%" ) ))
( "roguery"  n ( r ou1 g @ r ii ) ((pos "M8%" ) ))
( "rogues"  n ( r ou1 g z ) ((pos "Kj%" ) ))
( "roguish"  j ( r ou1 g i sh ) ((pos "OA%" ) ))
( "roguishly"  a ( r ou1 g i sh l ii ) ((pos "Pu%" ) ))
( "roguishness"  n ( r ou1 g i sh n @ s ) ((pos "L@%" ) ))
( "roisterer"  n ( r oi1 s t @ r @ r ) ((pos "K6%" ) ))
( "roisterers"  n ( r oi1 s t @ r @ z ) ((pos "Kj%" ) ))
( "role"  n ( r ou1 l ) ((pos "K6%" ) ))
( "roles"  n ( r ou1 l z ) ((pos "Kj%" ) ))
( "roll"  v ( r ou1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "12B" "13B" "14" "15A" "15B" )))
( "roll-call"  n ( r ou1 l - k oo l ) ((pos "K6%" ) ))
( "roll-calls"  n ( r ou1 l - k oo l z ) ((pos "Kj%" ) ))
( "roll-on"  n ( r ou1 l - o n ) ((pos "K6%" ) ))
( "roll-ons"  n ( r ou1 l - o n z ) ((pos "Kj%" ) ))
( "rolled"  v ( r ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "12B" "13B" "14" "15A" "15B" )))
( "roller"  n ( r ou1 l @ r ) ((pos "K6%" ) ))
( "roller-skate"  n ( r ou1 l @ - s k ei t ) ((pos "K6%" ) ))
( "roller-skates"  n ( r ou1 l @ - s k ei t s ) ((pos "Kj%" ) ))
( "rollers"  n ( r ou1 l @ z ) ((pos "Kj%" ) ))
( "rollicking"  j ( r o1 l i k i ng ) ((pos "OA%" ) ))
( "rolling"  v ( r ou1 l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "6A" "12B" "13B" "14" "15A" "15B" )))
( "rolling-mill"  n ( r ou1 l i ng - m i l ) ((pos "K6%" ) ))
( "rolling-mills"  n ( r ou1 l i ng - m i l z ) ((pos "Kj%" ) ))
( "rolling-pin"  n ( r ou1 l i ng - p i n ) ((pos "K6%" ) ))
( "rolling-pins"  n ( r ou1 l i ng - p i n z ) ((pos "Kj%" ) ))
( "rolling-stock"  n ( r ou1 l i ng - s t o k ) ((pos "L@%" ) ))
( "rolls"  v ( r ou1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "12B" "13B" "14" "15A" "15B" )))
( "roly-polies"  n ( r ou2 l i - p ou1 l i z ) ((pos "Kj$" ) ))
( "roly-poly"  n ( r ou2 l i - p ou1 l ii ) ((pos "K8%" ) ))
( "romance"  v ( r ou1 m a1 n s ) ((pos "I2%" "M6%" ) (subcat )))
( "romanced"  v ( r ou1 m a1 n s t ) ((pos "Ic%" "Id%" ) (subcat )))
( "romances"  v ( r ou1 m a1 n s i z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "romancing"  v ( r ou1 m a1 n s i ng ) ((pos "Ib%" ) (subcat )))
( "romantic"  n ( r @1 m a1 n t i k ) ((pos "K6%" "OA%" ) ))
( "romantically"  a ( r @1 m a1 n t i k l ii ) ((pos "Pu%" ) ))
( "romanticism"  n ( r @1 m a1 n t i s i z @ m ) ((pos "L@%" ) ))
( "romanticist"  n ( r @1 m a1 n t i s i s t ) ((pos "K6%" ) ))
( "romanticists"  n ( r @1 m a1 n t i s i s t s ) ((pos "Kj%" ) ))
( "romanticize"  v ( r ou1 m a1 n t i s ai z ) ((pos "J2%" ) (subcat "2A" )))
( "romanticized"  v ( r ou1 m a1 n t i s ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "romanticizes"  v ( r ou1 m a1 n t i s ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "romanticizing"  v ( r ou1 m a1 n t i s ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "romantics"  n ( r @1 m a1 n t i k s ) ((pos "Kj%" ) ))
( "romp"  v ( r o1 m p ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "romped"  v ( r o1 m p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "romper"  n ( r o1 m p @ r ) ((pos "K6%" ) ))
( "rompers"  n ( r o1 m p @ z ) ((pos "Kj%" ) ))
( "romping"  v ( r o1 m p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "romps"  v ( r o1 m p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "rondeau"  n ( r o1 n d ou ) ((pos "K6$" ) ))
( "rondeaus"  n ( r o1 n d ou z ) ((pos "Kj$" ) ))
( "rondel"  n ( r o1 n d l ) ((pos "K6$" ) ))
( "rondels"  n ( r o1 n d l z ) ((pos "Kj$" ) ))
( "rondo"  n ( r o1 n d ou ) ((pos "K6$" ) ))
( "rondos"  n ( r o1 n d ou z ) ((pos "Kj$" ) ))
( "rood"  n ( r uu1 d ) ((pos "K6$" ) ))
( "rood-tree"  n ( r uu1 d - t r ii ) ((pos "K6$" ) ))
( "rood-trees"  n ( r uu1 d - t r ii z ) ((pos "Kj$" ) ))
( "roods"  n ( r uu1 d z ) ((pos "Kj$" ) ))
( "roof"  v ( r uu1 f ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "roof-garden"  n ( r uu1 f - g aa d n ) ((pos "K6%" ) ))
( "roof-gardens"  n ( r uu1 f - g aa d n z ) ((pos "Kj%" ) ))
( "roof-tree"  n ( r uu1 f - t r ii ) ((pos "K6%" ) ))
( "roof-trees"  n ( r uu1 f - t r ii z ) ((pos "Kj%" ) ))
( "roofed"  v ( r uu1 f t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "roofing"  v ( r uu1 f i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" "15A" )))
( "roofless"  j ( r uu1 f l @ s ) ((pos "OA%" ) ))
( "roofs"  v ( r uu1 f s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "rook"  v ( r u1 k ) ((pos "H0$" "K6%" ) (subcat )))
( "rooked"  v ( r u1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rookeries"  n ( r u1 k @ r i z ) ((pos "Kj%" ) ))
( "rookery"  n ( r u1 k @ r ii ) ((pos "K8%" ) ))
( "rookie"  n ( r u1 k ii ) ((pos "K6%" ) ))
( "rookies"  n ( r u1 k i z ) ((pos "Kj%" ) ))
( "rooking"  v ( r u1 k i ng ) ((pos "Hb$" ) (subcat )))
( "rooks"  v ( r u1 k s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "room"  v ( r uu1 m ) ((pos "I0%" "M6*" ) (subcat )))
( "room-mate"  n ( r uu1 m - m ei t ) ((pos "K6%" ) ))
( "room-mates"  n ( r uu1 m - m ei t s ) ((pos "Kj%" ) ))
( "roomed"  v ( r uu1 m d ) ((pos "Ic%" "Id%" "Ot%" ) (subcat )))
( "roomer"  n ( r uu1 m @ r ) ((pos "K6$" ) ))
( "roomers"  n ( r uu1 m @ z ) ((pos "Kj$" ) ))
( "roomful"  n ( r uu1 m f u l ) ((pos "K6%" ) ))
( "roomfuls"  n ( r uu1 m f u l z ) ((pos "Kj%" ) ))
( "roomier"  j ( r uu1 m i@ r ) ((pos "Or%" ) ))
( "roomiest"  j ( r uu1 m i i s t ) ((pos "Os%" ) ))
( "roomily"  a ( r uu1 m i l ii ) ((pos "Pu%" ) ))
( "rooming"  v ( r uu1 m i ng ) ((pos "Ib%" ) (subcat )))
( "rooms"  v ( r uu1 m z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "roomy"  j ( r uu1 m ii ) ((pos "OD%" ) ))
( "roost"  v ( r uu1 s t ) ((pos "I0%" "K6%" ) (subcat )))
( "roosted"  v ( r uu1 s t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "rooster"  n ( r uu1 s t @ r ) ((pos "K6%" ) ))
( "roosters"  n ( r uu1 s t @ z ) ((pos "Kj%" ) ))
( "roosting"  v ( r uu1 s t i ng ) ((pos "Ib%" ) (subcat )))
( "roosts"  v ( r uu1 s t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "root"  v ( r uu1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "rooted"  v ( r uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "rooting"  v ( r uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "rootle"  v ( r uu1 t l ) ((pos "I2$" ) (subcat )))
( "rootled"  v ( r uu1 t l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "rootles"  v ( r uu1 t l z ) ((pos "Ia$" ) (subcat )))
( "rootless"  j ( r uu1 t l @ s ) ((pos "OA%" ) ))
( "rootling"  v ( r uu1 t l i ng ) ((pos "Ib$" ) (subcat )))
( "roots"  v ( r uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "rope"  v ( r ou1 p ) ((pos "H2%" "M6%" ) (subcat "15A" )))
( "rope-dancer"  n ( r ou1 p - d aa n s @ r ) ((pos "K6$" ) ))
( "rope-dancers"  n ( r ou1 p - d aa n s @ z ) ((pos "Kj$" ) ))
( "rope-ladder"  n ( r ou1 p - l a d @ r ) ((pos "K6%" ) ))
( "rope-ladders"  n ( r ou1 p - l a d @ z ) ((pos "Kj%" ) ))
( "rope-yard"  n ( r ou1 p - y aa d ) ((pos "K6$" ) ))
( "rope-yards"  n ( r ou1 p - y aa d z ) ((pos "Kj$" ) ))
( "rope-yarn"  n ( r ou1 p - y aa n ) ((pos "L@$" ) ))
( "roped"  v ( r ou1 p t ) ((pos "Hc%" "Hd%" ) (subcat "15A" )))
( "ropes"  v ( r ou1 p s ) ((pos "Ha%" "Mj%" ) (subcat "15A" )))
( "ropewalk"  n ( r ou1 p w oo k ) ((pos "K6$" ) ))
( "ropewalker"  n ( r ou1 p w oo k @ r ) ((pos "K6$" ) ))
( "ropewalkers"  n ( r ou1 p w oo k @ z ) ((pos "Kj$" ) ))
( "ropewalks"  n ( r ou1 p w oo k s ) ((pos "Kj$" ) ))
( "ropeway"  n ( r ou1 p w ei ) ((pos "K6$" ) ))
( "ropeways"  n ( r ou1 p w ei z ) ((pos "Kj$" ) ))
( "ropey"  j ( r ou1 p ii ) ((pos "OE%" ) ))
( "ropier"  j ( r ou1 p i@ r ) ((pos "Or%" ) ))
( "ropiest"  j ( r ou1 p i@ s t ) ((pos "Os%" ) ))
( "roping"  v ( r ou1 p i ng ) ((pos "Hb%" ) (subcat "15A" )))
( "rosaries"  n ( r ou1 z @ r i z ) ((pos "Kj%" ) ))
( "rosary"  n ( r ou1 z @ r ii ) ((pos "K8%" ) ))
( "rose"  v ( r ou1 z ) ((pos "Ic%" "M6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "rose-bed"  n ( r ou1 z - b e d ) ((pos "K6%" ) ))
( "rose-beds"  n ( r ou1 z - b e d z ) ((pos "Kj%" ) ))
( "rose-leaf"  n ( r ou1 z - l ii f ) ((pos "Ki%" ) ))
( "rose-leaves"  n ( r ou1 z - l ii v z ) ((pos "Kj%" ) ))
( "rose-red"  j ( r ou1 z - r e d ) ((pos "OA%" ) ))
( "rose-water"  n ( r ou1 z - w oo t @ r ) ((pos "L@%" ) ))
( "roseate"  j ( r ou1 z i@ t ) ((pos "OA%" ) ))
( "rosebud"  n ( r ou1 z b uh d ) ((pos "K6%" ) ))
( "rosebuds"  n ( r ou1 z b uh d z ) ((pos "Kj%" ) ))
( "rosemary"  n ( r ou1 z m @ r ii ) ((pos "L@%" ) ))
( "roses"  n ( r ou1 z i z ) ((pos "Mj%" ) ))
( "rosette"  n ( r ou1 z e1 t ) ((pos "K6%" ) ))
( "rosettes"  n ( r ou1 z e1 t s ) ((pos "Kj%" ) ))
( "rosewood"  n ( r ou1 z w u d ) ((pos "L@%" ) ))
( "rosier"  j ( r ou1 z i@ r ) ((pos "Or%" ) ))
( "rosiest"  j ( r ou1 z i i s t ) ((pos "Os%" ) ))
( "rosin"  v ( r o1 z i n ) ((pos "H4$" "L@$" ) (subcat )))
( "rosinned"  v ( r o1 z i n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rosinning"  v ( r o1 z i n i ng ) ((pos "Hb$" ) (subcat )))
( "rosins"  v ( r o1 z i n z ) ((pos "Ha$" ) (subcat )))
( "roster"  n ( r o1 s t @ r ) ((pos "K6%" ) ))
( "rosters"  n ( r o1 s t @ z ) ((pos "Kj%" ) ))
( "rostra"  n ( r o1 s t r @ ) ((pos "Kj$" ) ))
( "rostrum"  n ( r o1 s t r @ m ) ((pos "K6%" ) ))
( "rostrums"  n ( r o1 s t r @ m z ) ((pos "Kj$" ) ))
( "rosy"  j ( r ou1 z ii ) ((pos "OD%" ) ))
( "rot"  v ( r o1 t ) ((pos "J4%" "L@%" ) (subcat "2A" "2C" )))
( "rota"  n ( r ou1 t @ ) ((pos "K6%" ) ))
( "rotaries"  n ( r ou1 t @ r i z ) ((pos "Kj%" ) ))
( "rotary"  n ( r ou1 t @ r ii ) ((pos "K8%" "OA%" ) ))
( "rotas"  n ( r ou1 t @ z ) ((pos "Kj%" ) ))
( "rotate"  v ( r ou1 t ei1 t ) ((pos "J2%" ) (subcat "2A" )))
( "rotated"  v ( r ou1 t ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rotates"  v ( r ou1 t ei1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "rotating"  v ( r ou1 t ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rotation"  n ( r ou1 t ei1 sh @ n ) ((pos "M6%" ) ))
( "rotational"  j ( r ou1 t ei1 sh @ n l ) ((pos "OA%" ) ))
( "rotationally"  a ( r ou1 t ei1 sh @ n @ ii ) ((pos "Pu%" ) ))
( "rotations"  n ( r ou1 t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "rotatory"  j ( r ou1 t @ t @ r ii ) ((pos "OA$" ) ))
( "rote"  n ( r ou1 t ) ((pos "Ki%" ) ))
( "rotgut"  n ( r o1 t g uh t ) ((pos "L@%" ) ))
( "rotisserie"  n ( r ou1 t ii1 s @ r ii ) ((pos "K6%" ) ))
( "rotisseries"  n ( r ou1 t ii1 s @ r i z ) ((pos "Kj%" ) ))
( "rotogravure"  n ( r ou2 t ou g r @ v y u@1 r ) ((pos "M6$" ) ))
( "rotogravures"  n ( r ou2 t ou g r @ v y u@1 z ) ((pos "Mj$" ) ))
( "rotor"  n ( r ou1 t @ r ) ((pos "K6%" ) ))
( "rotors"  n ( r ou1 t @ z ) ((pos "Kj%" ) ))
( "rots"  v ( r o1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "rotted"  v ( r o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "rotten"  j ( r o1 t n ) ((pos "OA%" ) ))
( "rottenly"  a ( r o1 t n l ii ) ((pos "Pu%" ) ))
( "rottenness"  n ( r o1 t n - n @ s ) ((pos "L@%" ) ))
( "rotter"  n ( r o1 t @ r ) ((pos "K6%" ) ))
( "rotters"  n ( r o1 t @ z ) ((pos "Kj%" ) ))
( "rotting"  v ( r o1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "rotund"  j ( r ou1 t uh1 n d ) ((pos "OA%" ) ))
( "rotunda"  n ( r ou1 t uh1 n d @ ) ((pos "K6$" ) ))
( "rotundas"  n ( r ou1 t uh1 n d @ z ) ((pos "Kj$" ) ))
( "rotundity"  n ( r ou1 t uh1 n d i t ii ) ((pos "L@$" ) ))
( "rotundly"  a ( r ou1 t uh1 n d l ii ) ((pos "Pu$" ) ))
( "rou_e"  n ( r uu1 ei ) ((pos "K6$" ) ))
( "rou_es"  n ( r uu1 ei z ) ((pos "Kj$" ) ))
( "rouble"  n ( r uu1 b l ) ((pos "K6$" ) ))
( "roubles"  n ( r uu1 b l z ) ((pos "Kj$" ) ))
( "rouge"  v ( r uu1 zh ) ((pos "J2%" "L@%" ) (subcat "2A" )))
( "rouged"  v ( r uu1 zh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "rouges"  v ( r uu1 zh i z ) ((pos "Ja$" ) (subcat "2A" )))
( "rough"  v ( r uh1 f ) ((pos "H0%" "M6%" "OC%" "Pu%" ) (subcat "6A" )))
( "rough-and-tumble"  n ( r uh2 f - @ n - t uh1 m b l ) ((pos "K6%" "OA%" ) ))
( "rough-and-tumbles"  n ( r uh2 f - @ n - t uh1 m b l z ) ((pos "Kj$" ) ))
( "rough-dried"  v ( r uh1 f - d r ai d ) ((pos "Hc$" "Hd%" ) ))
( "rough-dries"  v ( r uh1 f - d r ai z ) ((pos "Ha$" ) ))
( "rough-dry"  v ( r uh1 f - d r ai ) ((pos "H3$" ) ))
( "rough-drying"  v ( r uh1 f - d r ai i ng ) ((pos "Hb$" ) ))
( "rough-hewn"  j ( r uh1 f - h y uu n ) ((pos "OA%" ) ))
( "rough-house"  v ( r uh1 f - h au s ) ((pos "J2$" ) (subcat "2A" )))
( "rough-housed"  v ( r uh1 f - h au s t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "rough-houses"  v ( r uh1 f - h au z i z ) ((pos "Ja$" ) (subcat "2A" )))
( "rough-housing"  v ( r uh1 f - h au s i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "rough-spoken"  j ( r uh1 f - s p ou1 k @ n ) ((pos "OA%" ) ))
( "roughage"  n ( r uh1 f i jh ) ((pos "L@%" ) ))
( "roughcast"  v ( r uh1 f k a s t ) ((pos "H5$" "Hc%" "Hd%" "L@%" ) ))
( "roughcasting"  v ( r uh1 f k a s t i ng ) ((pos "Hb$" ) ))
( "roughcasts"  v ( r uh1 f k a s t s ) ((pos "Ha$" ) ))
( "roughed"  v ( r uh1 f t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "roughen"  v ( r uh1 f n ) ((pos "J0%" ) ))
( "roughened"  v ( r uh1 f n d ) ((pos "Jc%" "Jd%" ) ))
( "roughening"  v ( r uh1 f n i ng ) ((pos "Jb%" ) ))
( "roughens"  v ( r uh1 f n z ) ((pos "Ja%" ) ))
( "rougher"  j ( r uh1 f @ r ) ((pos "Or%" ) ))
( "roughest"  j ( r uh1 f i s t ) ((pos "Os%" ) ))
( "roughing"  v ( r uh1 f i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "roughish"  j ( r uh1 f i sh ) ((pos "OA%" ) ))
( "roughly"  a ( r uh1 f l ii ) ((pos "Pu%" ) ))
( "roughneck"  n ( r uh1 f n e k ) ((pos "K6%" ) ))
( "roughnecks"  n ( r uh1 f n e k s ) ((pos "Kj%" ) ))
( "roughness"  n ( r uh1 f n @ s ) ((pos "L@%" ) ))
( "roughrider"  n ( r uh1 f r ai d @ r ) ((pos "K6$" ) ))
( "roughriders"  n ( r uh1 f r ai d @ z ) ((pos "Kj$" ) ))
( "roughs"  v ( r uh1 f s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "roughshod"  j ( r uh1 f sh o d ) ((pos "OA%" ) ))
( "rouging"  v ( r uu1 zh i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "roulette"  n ( r uu1 l e1 t ) ((pos "L@%" ) ))
( "round"  v ( r au n d ) ((pos "J0%" "M6%" "OC%" "P+%" "T-%" ) (subcat "2C" "3A" "6A" )))
( "round-arm"  j ( r au1 n d - aa m ) ((pos "OA$" "Pu$" ) ))
( "round-backed"  j ( r au1 n d - b a1 k t ) ((pos "OA%" ) ))
( "round-eyed"  j ( r au1 n d - ai d ) ((pos "OA%" ) ))
( "round-hand"  n ( r au1 n d - h a n d ) ((pos "L@$" ) ))
( "round-shot"  n ( r au1 n d - sh o t ) ((pos "K6$" ) ))
( "round-shots"  n ( r au1 n d - sh o t s ) ((pos "Kj$" ) ))
( "round-shouldered"  j ( r au1 n d - sh ou1 l d @ d ) ((pos "OA%" ) ))
( "round-the-clock"  j ( r au2 n d - dh @ - k l o1 k ) ((pos "Oq%" "Pu%" ) ))
( "roundabout"  n ( r au1 n d @ b au t ) ((pos "K6%" "Oq%" ) ))
( "roundabouts"  n ( r au1 n d @ b au t s ) ((pos "Kj%" ) ))
( "rounded"  v ( r au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "roundel"  n ( r au1 n d l ) ((pos "K6$" ) ))
( "roundelay"  n ( r au1 n d i l ei ) ((pos "K6$" ) ))
( "roundelays"  n ( r au1 n d i l ei z ) ((pos "Kj$" ) ))
( "roundels"  n ( r au1 n d l z ) ((pos "Kj$" ) ))
( "rounder"  j ( r au1 n d @ r ) ((pos "Or$" ) ))
( "rounders"  n ( r au1 n d @ z ) ((pos "Kj%" ) ))
( "roundest"  j ( r au1 n d i s t ) ((pos "Os%" ) ))
( "roundhouse"  n ( r au1 n d h au s ) ((pos "K6%" ) ))
( "roundhouses"  n ( r au1 n d h au z i z ) ((pos "Kj%" ) ))
( "rounding"  v ( r au1 n d i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "roundish"  j ( r au1 n d i sh ) ((pos "OA%" ) ))
( "roundly"  a ( r au1 n d l ii ) ((pos "Pu%" ) ))
( "roundness"  n ( r au1 n d n @ s ) ((pos "L@%" ) ))
( "rounds"  v ( r au1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "6A" )))
( "roundsman"  n ( r au1 n d z m @ n ) ((pos "Ki%" ) ))
( "roundsmen"  n ( r au1 n d z m @ n ) ((pos "Kj%" ) ))
( "roundup"  n ( r au1 n d uh p ) ((pos "K6%" ) ))
( "roundups"  n ( r au1 n d uh p s ) ((pos "Kj%" ) ))
( "rouse"  v ( r au1 z ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "roused"  v ( r au1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "rouses"  v ( r au1 z i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "rousing"  v ( r au1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "rout"  v ( r au1 t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "route"  v ( r uu1 t ) ((pos "H2%" "M6%" ) ))
( "routed"  v ( r au1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "routed"  v ( r uu1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "routemarch"  n ( r uu1 t m aa ch ) ((pos "K7%" ) ))
( "routemarches"  n ( r uu1 t m aa ch i z ) ((pos "Kj%" ) ))
( "routes"  v ( r uu1 t s ) ((pos "Ha%" "Mj%" ) ))
( "routine"  n ( r uu1 t ii1 n ) ((pos "M6%" "OA%" ) ))
( "routines"  n ( r uu1 t ii1 n z ) ((pos "Mj%" ) ))
( "routing"  v ( r au1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "routing"  v ( r uu1 t i ng ) ((pos "Hb%" ) ))
( "routs"  v ( r au1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "rove"  v ( r ou1 v ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "roved"  v ( r ou1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "rover"  n ( r ou1 v @ r ) ((pos "K6%" ) ))
( "rovers"  n ( r ou1 v @ z ) ((pos "Kj%" ) ))
( "roves"  v ( r ou1 v z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "roving"  v ( r ou1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "row"  v ( r ou1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "row"  v ( r au1 ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" )))
( "rowan"  n ( r ou1 @ n ) ((pos "K6%" ) ))
( "rowan-berries"  n ( r ou1 @ n - b e r i z ) ((pos "Kj%" ) ))
( "rowan-berry"  n ( r ou1 @ n - b e r ii ) ((pos "K8%" ) ))
( "rowan-tree"  n ( r ou1 @ n - t r ii ) ((pos "K6%" ) ))
( "rowan-trees"  n ( r ou1 @ n - t r ii z ) ((pos "Kj%" ) ))
( "rowans"  n ( r ou1 @ n z ) ((pos "Kj%" ) ))
( "rowboat"  n ( r ou1 b ou t ) ((pos "K6%" ) ))
( "rowboats"  n ( r ou1 b ou t s ) ((pos "Kj%" ) ))
( "rowdier"  j ( r au1 d i@ r ) ((pos "Or%" ) ))
( "rowdies"  n ( r au1 d i z ) ((pos "Kj%" ) ))
( "rowdiest"  j ( r au1 d i i s t ) ((pos "Os%" ) ))
( "rowdily"  a ( r au1 d i l ii ) ((pos "Pu%" ) ))
( "rowdiness"  n ( r au1 d i n @ s ) ((pos "L@%" ) ))
( "rowdy"  n ( r au1 d ii ) ((pos "K8%" "OD%" ) ))
( "rowdyism"  n ( r au1 d i i z @ m ) ((pos "L@$" ) ))
( "rowed"  v ( r ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "rowed"  v ( r au1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "rowel"  n ( r au1 @ l ) ((pos "K6$" ) ))
( "rowels"  n ( r au1 @ l z ) ((pos "Kj$" ) ))
( "rower"  n ( r ou1 @ r ) ((pos "K6%" ) ))
( "rowers"  n ( r ou1 @ z ) ((pos "Kj%" ) ))
( "rowing"  v ( r ou1 i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "rowing"  v ( r au1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "rowing-boat"  n ( r ou1 i ng - b ou t ) ((pos "K6%" ) ))
( "rowing-boats"  n ( r ou1 i ng - b ou t s ) ((pos "Kj%" ) ))
( "rowing-club"  n ( r ou1 i ng - k l uh b ) ((pos "K6%" ) ))
( "rowing-clubs"  n ( r ou1 i ng - k l uh b z ) ((pos "Kj%" ) ))
( "rowlock"  n ( r o1 l @ k ) ((pos "K6%" ) ))
( "rowlocks"  n ( r o1 l @ k s ) ((pos "Kj%" ) ))
( "rows"  v ( r ou1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "rows"  v ( r au1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" )))
( "royal"  j ( r oi1 @ l ) ((pos "OA%" ) ))
( "royalist"  n ( r oi1 @ l i s t ) ((pos "K6%" ) ))
( "royalists"  n ( r oi1 @ l i s t s ) ((pos "Kj%" ) ))
( "royally"  a ( r oi1 @ l ii ) ((pos "Pu%" ) ))
( "royals"  n ( r oi1 @ l z ) ((pos "Kj%" ) ))
( "royalties"  n ( r oi1 @ l t i z ) ((pos "Mj%" ) ))
( "royalty"  n ( r oi1 @ l t ii ) ((pos "M8%" ) ))
( "rpm"  n ( aa2 p i e1 m ) ((pos "Y]%" ) ))
( "rub"  v ( r uh1 b ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rub-a-dub"  n ( r uh2 b - @ - d uh1 b ) ((pos "L@$" ) ))
( "rub-down"  n ( r uh1 b - d au n ) ((pos "K6%" ) ))
( "rub-downs"  n ( r uh1 b - d au n z ) ((pos "Kj$" ) ))
( "rub-up"  n ( r uh1 b - uh p ) ((pos "K6$" ) ))
( "rub-ups"  n ( r uh1 b - uh p s ) ((pos "Kj$" ) ))
( "rubbed"  v ( r uh1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rubber"  v ( r uh1 b @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "rubber-stamp"  v ( r uh2 b @ - s t a1 m p ) ((pos "H0%" ) ))
( "rubber-stamped"  v ( r uh2 b @ - s t a1 m p t ) ((pos "Hc%" "Hd%" ) ))
( "rubber-stamping"  v ( r uh2 b @ - s t a1 m p i ng ) ((pos "Hb%" ) ))
( "rubber-stamps"  v ( r uh2 b @ - s t a1 m p s ) ((pos "Ha%" ) ))
( "rubbered"  v ( r uh1 b @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "rubbering"  v ( r uh1 b @ r i ng ) ((pos "Hb$" ) (subcat )))
( "rubberize"  v ( r uh1 b @ r ai z ) ((pos "H2%" ) (subcat )))
( "rubberized"  v ( r uh1 b @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rubberizes"  v ( r uh1 b @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "rubberizing"  v ( r uh1 b @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "rubberneck"  v ( r uh1 b @ n e k ) ((pos "I0$" "K6%" ) ))
( "rubbernecked"  v ( r uh1 b @ n e k t ) ((pos "Ic$" "Id$" ) ))
( "rubbernecking"  v ( r uh1 b @ n e k i ng ) ((pos "Ib$" ) ))
( "rubbernecks"  v ( r uh1 b @ n e k s ) ((pos "Ia$" "Kj$" ) ))
( "rubbers"  v ( r uh1 b @ z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "rubbery"  j ( r uh1 b @ r ii ) ((pos "OA%" ) ))
( "rubbing"  v ( r uh1 b i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "rubbings"  n ( r uh1 b i ng z ) ((pos "Mj%" ) ))
( "rubbish"  v ( r uh1 b i sh ) ((pos "H1$" "L@%" ) (subcat )))
( "rubbished"  v ( r uh1 b i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rubbishes"  v ( r uh1 b i sh i z ) ((pos "Ha%" ) (subcat )))
( "rubbishing"  v ( r uh1 b i sh i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "rubbishy"  j ( r uh1 b i sh ii ) ((pos "OA%" ) ))
( "rubble"  n ( r uh1 b l ) ((pos "L@%" ) ))
( "rubicund"  j ( r uu1 b i k @ n d ) ((pos "OA$" ) ))
( "rubies"  n ( r uu1 b i z ) ((pos "Kj%" ) ))
( "rubric"  n ( r uu1 b r i k ) ((pos "K6$" ) ))
( "rubrics"  n ( r uu1 b r i k s ) ((pos "Kj$" ) ))
( "rubs"  v ( r uh1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "ruby"  n ( r uu1 b ii ) ((pos "K8%" "OA%" ) ))
( "ruck"  v ( r uh1 k ) ((pos "J0$" "K6$" ) (subcat "2A" "2C" "6A" )))
( "rucked"  v ( r uh1 k t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" )))
( "rucking"  v ( r uh1 k i ng ) ((pos "Jb$" ) (subcat "2A" "2C" "6A" )))
( "rucks"  v ( r uh1 k s ) ((pos "Ja$" "Kj$" ) (subcat "2A" "2C" "6A" )))
( "rucksack"  n ( r uh1 k s a k ) ((pos "K6%" ) ))
( "rucksacks"  n ( r uh1 k s a k s ) ((pos "Kj%" ) ))
( "ruckus"  n ( r uh1 k @ s ) ((pos "K6%" ) ))
( "ruckuses"  n ( r uh1 k @ s i z ) ((pos "Kj$" ) ))
( "ructions"  n ( r uh1 k sh @ n z ) ((pos "Kj%" ) ))
( "rudder"  n ( r uh1 d @ r ) ((pos "K6%" ) ))
( "rudderless"  j ( r uh1 d @ l i s ) ((pos "OA%" ) ))
( "rudders"  n ( r uh1 d @ z ) ((pos "Kj%" ) ))
( "ruddier"  j ( r uh1 d i@ r ) ((pos "Or%" ) ))
( "ruddiest"  j ( r uh1 d i i s t ) ((pos "Os%" ) ))
( "ruddiness"  n ( r uh1 d i n i s ) ((pos "L@%" ) ))
( "ruddle"  v ( r uh1 d l ) ((pos "H2$" "L@$" ) ))
( "ruddled"  v ( r uh1 d l d ) ((pos "Hc$" "Hd$" ) ))
( "ruddles"  v ( r uh1 d l z ) ((pos "Ha$" ) ))
( "ruddling"  v ( r uh1 d l i ng ) ((pos "Hb$" ) ))
( "ruddy"  j ( r uh1 d ii ) ((pos "OD%" ) ))
( "rude"  j ( r uu1 d ) ((pos "OB%" ) ))
( "rudely"  a ( r uu1 d l ii ) ((pos "Pu%" ) ))
( "rudeness"  n ( r uu1 d n @ s ) ((pos "M7%" ) ))
( "rudenesses"  n ( r uu1 d n @ s i z ) ((pos "Mj%" ) ))
( "ruder"  j ( r uu1 d @ r ) ((pos "Or%" ) ))
( "rudest"  j ( r uu1 d i s t ) ((pos "Os%" ) ))
( "rudiment"  n ( r uu1 d i m @ n t ) ((pos "K6$" ) ))
( "rudimentary"  j ( r uu2 d i m e1 n t r ii ) ((pos "OA%" ) ))
( "rudiments"  n ( r uu1 d i m @ n t s ) ((pos "Kj%" ) ))
( "rue"  v ( r uu1 ) ((pos "H2%" "L@%" ) (subcat )))
( "rued"  v ( r uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rueful"  j ( r uu1 f @ l ) ((pos "OA%" ) ))
( "ruefully"  a ( r uu1 f @ l ii ) ((pos "Pu%" ) ))
( "rues"  v ( r uu1 z ) ((pos "Ha%" ) (subcat )))
( "ruff"  v ( r uh1 f ) ((pos "J0$" "K6%" ) ))
( "ruffed"  v ( r uh1 f t ) ((pos "Jc$" "Jd$" ) ))
( "ruffian"  n ( r uh1 f i@ n ) ((pos "K6%" ) ))
( "ruffianism"  n ( r uh1 f i@ n i z @ m ) ((pos "L@$" ) ))
( "ruffianly"  j ( r uh1 f i@ n l ii ) ((pos "OA%" ) ))
( "ruffians"  n ( r uh1 f i@ n z ) ((pos "Kj%" ) ))
( "ruffing"  v ( r uh1 f i ng ) ((pos "Jb$" ) ))
( "ruffle"  v ( r uh1 f l ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" )))
( "ruffled"  v ( r uh1 f l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "ruffles"  v ( r uh1 f l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "ruffling"  v ( r uh1 f l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "ruffs"  v ( r uh1 f s ) ((pos "Ja$" "Kj$" ) ))
( "rug"  n ( r uh1 g ) ((pos "K6%" ) ))
( "rugby"  n ( r uh1 g b ii ) ((pos "L@%" ) ))
( "rugged"  j ( r uh1 g i d ) ((pos "OA%" ) ))
( "ruggedly"  a ( r uh1 g i d l ii ) ((pos "Pu%" ) ))
( "ruggedness"  n ( r uh1 g i d n @ s ) ((pos "L@%" ) ))
( "rugger"  n ( r uh1 g @ r ) ((pos "L@%" ) ))
( "rugs"  n ( r uh1 g z ) ((pos "Kj%" ) ))
( "ruin"  v ( r uu1 i n ) ((pos "H0%" "M6%" ) (subcat )))
( "ruination"  n ( r uu2 i n ei1 sh @ n ) ((pos "L@%" ) ))
( "ruined"  v ( r uu1 i n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ruing"  v ( r uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "ruining"  v ( r uu1 i n i ng ) ((pos "Hb%" ) (subcat )))
( "ruinous"  j ( r uu1 i n @ s ) ((pos "OA%" ) ))
( "ruinously"  a ( r uu1 i n @ s l ii ) ((pos "Pu%" ) ))
( "ruins"  v ( r uu1 i n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "rule"  v ( r uu1 l ) ((pos "J2%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "9" "15A" "15B" )))
( "ruled"  v ( r uu1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "9" "15A" "15B" )))
( "ruler"  n ( r uu1 l @ r ) ((pos "K6%" ) ))
( "rulers"  n ( r uu1 l @ z ) ((pos "Kj%" ) ))
( "rules"  v ( r uu1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "9" "15A" "15B" )))
( "ruling"  v ( r uu1 l i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "9" "15A" "15B" )))
( "rulings"  n ( r uu1 l i ng z ) ((pos "Kj%" ) ))
( "rum"  n ( r uh1 m ) ((pos "L@%" ) ))
( "rum-runner"  n ( r uh1 m - r uh n @ r ) ((pos "K6%" ) ))
( "rum-runners"  n ( r uh1 m - r uh n @ z ) ((pos "Kj%" ) ))
( "rumba"  n ( r uh1 m b @ ) ((pos "K6%" ) ))
( "rumbas"  n ( r uh1 m b @ z ) ((pos "Kj%" ) ))
( "rumble"  v ( r uh1 m b l ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" )))
( "rumbled"  v ( r uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "rumbles"  v ( r uh1 m b l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "rumbling"  v ( r uh1 m b l i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" )))
( "rumblings"  n ( r uh1 m b l i ng z ) ((pos "Kj%" ) ))
( "rumbustious"  j ( r uh1 m b uh1 s t i@ s ) ((pos "OA%" ) ))
( "ruminant"  n ( r uu1 m i n @ n t ) ((pos "K6%" "OA%" ) ))
( "ruminants"  n ( r uu1 m i n @ n t s ) ((pos "Kj%" ) ))
( "ruminate"  v ( r uu1 m i n ei t ) ((pos "I2%" ) (subcat "2A" "2B" )))
( "ruminated"  v ( r uu1 m i n ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "ruminates"  v ( r uu1 m i n ei t s ) ((pos "Ia%" ) (subcat "2A" "2B" )))
( "ruminating"  v ( r uu1 m i n ei t i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "rumination"  n ( r uu2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "ruminative"  j ( r uu1 m i n @ t i v ) ((pos "OA%" ) ))
( "rummage"  v ( r uh1 m i jh ) ((pos "J2%" "L@%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "rummaged"  v ( r uh1 m i jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "rummages"  v ( r uh1 m i jh i z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "rummaging"  v ( r uh1 m i jh i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "rummer"  j ( r uh1 m @ r ) ((pos "Or$" ) ))
( "rummest"  j ( r uh1 m i s t ) ((pos "Os$" ) ))
( "rummy"  n ( r uh1 m ii ) ((pos "L@%" "OE%" ) ))
( "rumour"  v ( r uu1 m @ r ) ((pos "H0%" "M6%" ) ))
( "rumour-monger"  n ( r uu1 m @ - m uh ng g @ r ) ((pos "K6%" ) ))
( "rumour-mongers"  n ( r uu1 m @ - m uh ng g @ z ) ((pos "Kj%" ) ))
( "rumoured"  v ( r uu1 m @ d ) ((pos "Hc%" "Hd%" ) ))
( "rumouring"  v ( r uu1 m @ r i ng ) ((pos "Hb%" ) ))
( "rumours"  v ( r uu1 m @ z ) ((pos "Ha%" "Mj%" ) ))
( "rump"  n ( r uh1 m p ) ((pos "K6%" ) ))
( "rump-steak"  n ( r uh1 m p - s t ei1 k ) ((pos "M6%" ) ))
( "rump-steaks"  n ( r uh1 m p - s t ei1 k s ) ((pos "Mj%" ) ))
( "rumple"  v ( r uh1 m p l ) ((pos "H2%" ) (subcat )))
( "rumpled"  v ( r uh1 m p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rumples"  v ( r uh1 m p l z ) ((pos "Ha%" ) (subcat )))
( "rumpling"  v ( r uh1 m p l i ng ) ((pos "Hb%" ) (subcat )))
( "rumps"  n ( r uh1 m p s ) ((pos "Kj%" ) ))
( "rumpus"  n ( r uh1 m p @ s ) ((pos "Ki%" ) ))
( "run"  v ( r uh1 n ) ((pos "J5%" "Jd%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" "14" "15A" )))
( "run-of-the-mill"  j ( r uh1 n - @ v - dh @ - m i l ) ((pos "OA%" ) ))
( "run-off"  n ( r uh1 n - o f ) ((pos "K6$" ) ))
( "run-offs"  n ( r uh1 n - o f s ) ((pos "Kj$" ) ))
( "run-through"  n ( r uh1 n - th r uu ) ((pos "K6%" ) ))
( "run-throughs"  n ( r uh1 n - th r uu z ) ((pos "Kj$" ) ))
( "run-up"  n ( r uh1 n - uh p ) ((pos "K6%" ) ))
( "run-ups"  n ( r uh1 n - uh p s ) ((pos "Kj$" ) ))
( "runaway"  n ( r uh1 n @ w ei ) ((pos "K6%" "OA%" ) ))
( "runaways"  n ( r uh1 n @ w ei z ) ((pos "Kj%" ) ))
( "rundown"  n ( r uh1 n d au n ) ((pos "K6%" "OA%" ) ))
( "rundowns"  n ( r uh1 n d au n z ) ((pos "Kj$" ) ))
( "rune"  n ( r uu1 n ) ((pos "K6$" ) ))
( "runes"  n ( r uu1 n z ) ((pos "Kj$" ) ))
( "rung"  v ( r uh1 ng ) ((pos "Jd%" "K6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "6A" "15A" )))
( "rungs"  n ( r uh1 ng z ) ((pos "Kj%" ) ))
( "runic"  j ( r uu1 n i k ) ((pos "OA$" ) ))
( "runnel"  n ( r uh1 n l ) ((pos "K6$" ) ))
( "runnels"  n ( r uh1 n l z ) ((pos "Kj$" ) ))
( "runner"  n ( r uh1 n @ r ) ((pos "K6%" ) ))
( "runner-up"  n ( r uh2 n @ r - uh1 p ) ((pos "Ki%" ) ))
( "runners"  n ( r uh1 n @ z ) ((pos "Kj%" ) ))
( "runners-up"  n ( r uh2 n @ z - uh1 p ) ((pos "Kj%" ) ))
( "runnier"  j ( r uh1 n i@ r ) ((pos "Or%" ) ))
( "runniest"  j ( r uh1 n i i s t ) ((pos "Os%" ) ))
( "running"  v ( r uh1 n i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" "14" "15A" )))
( "running-board"  n ( r uh1 n i ng - b oo d ) ((pos "K6%" ) ))
( "running-boards"  n ( r uh1 n i ng - b oo d z ) ((pos "Kj%" ) ))
( "runny"  j ( r uh1 n ii ) ((pos "OD%" ) ))
( "runs"  v ( r uh1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" "14" "15A" )))
( "runt"  n ( r uh1 n t ) ((pos "K6%" ) ))
( "runts"  n ( r uh1 n t s ) ((pos "Kj%" ) ))
( "runway"  n ( r uh1 n w ei ) ((pos "K6%" ) ))
( "runways"  n ( r uh1 n w ei z ) ((pos "Kj%" ) ))
( "rupee"  n ( r uu1 p ii1 ) ((pos "K6%" ) ))
( "rupees"  n ( r uu1 p ii1 z ) ((pos "Kj%" ) ))
( "rupiah"  n ( r uu1 p ii1 @ ) ((pos "K6$" ) ))
( "rupiahs"  n ( r uu1 p ii1 @ z ) ((pos "Kj$" ) ))
( "rupture"  v ( r uh1 p ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "ruptured"  v ( r uh1 p ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ruptures"  v ( r uh1 p ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "rupturing"  v ( r uh1 p ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "rural"  j ( r u@1 r @ l ) ((pos "OA%" ) ))
( "ruse"  n ( r uu1 z ) ((pos "K6%" ) ))
( "ruses"  n ( r uu1 z i z ) ((pos "Kj%" ) ))
( "rush"  v ( r uh1 sh ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "rushed"  v ( r uh1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "rushes"  v ( r uh1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "rushier"  j ( r uh1 sh i@ r ) ((pos "Or$" ) ))
( "rushiest"  j ( r uh1 sh i i s t ) ((pos "Os$" ) ))
( "rushing"  v ( r uh1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "rushlight"  n ( r uh1 sh l ai t ) ((pos "K6$" ) ))
( "rushlights"  n ( r uh1 sh l ai t s ) ((pos "Kj$" ) ))
( "rushy"  j ( r uh1 sh ii ) ((pos "OD%" ) ))
( "rusk"  n ( r uh1 s k ) ((pos "K6%" ) ))
( "rusks"  n ( r uh1 s k s ) ((pos "Kj%" ) ))
( "russet"  n ( r uh1 s i t ) ((pos "M6%" "OA%" ) ))
( "russets"  n ( r uh1 s i t s ) ((pos "Mj$" ) ))
( "rust"  v ( r uh1 s t ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" )))
( "rusted"  v ( r uh1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "rustic"  n ( r uh1 s t i k ) ((pos "K6%" "OA%" ) ))
( "rusticate"  v ( r uh1 s t i k ei t ) ((pos "J2$" ) (subcat "2A" )))
( "rusticated"  v ( r uh1 s t i k ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "rusticates"  v ( r uh1 s t i k ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "rusticating"  v ( r uh1 s t i k ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "rusticity"  n ( r uh1 s t i1 s i t ii ) ((pos "L@$" ) ))
( "rustics"  n ( r uh1 s t i k s ) ((pos "Kj%" ) ))
( "rustier"  j ( r uh1 s t i@ r ) ((pos "Or%" ) ))
( "rustiest"  j ( r uh1 s t i i s t ) ((pos "Os%" ) ))
( "rustiness"  n ( r uh1 s t i n @ s ) ((pos "L@%" ) ))
( "rusting"  v ( r uh1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "rustle"  v ( r uh1 s l ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "rustled"  v ( r uh1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "rustler"  n ( r uh1 s l @ r ) ((pos "K6%" ) ))
( "rustlers"  n ( r uh1 s l @ z ) ((pos "Kj%" ) ))
( "rustles"  v ( r uh1 s l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "rustless"  j ( r uh1 s t l @ s ) ((pos "OA%" ) ))
( "rustling"  v ( r uh1 s l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "rustlings"  n ( r uh1 s l i ng z ) ((pos "Mj$" ) ))
( "rusts"  v ( r uh1 s t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "rusty"  j ( r uh1 s t ii ) ((pos "OD%" ) ))
( "rut"  v ( r uh1 t ) ((pos "H4%" "M6%" ) (subcat )))
( "ruthless"  j ( r uu1 th l i s ) ((pos "OA%" ) ))
( "ruthlessly"  a ( r uu1 th l i s l ii ) ((pos "Pu%" ) ))
( "ruthlessness"  n ( r uu1 th l i s n @ s ) ((pos "L@%" ) ))
( "ruts"  v ( r uh1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "rutted"  v ( r uh1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "rutting"  v ( r uh1 t i ng ) ((pos "Hb$" ) (subcat )))
( "rye"  n ( r ai1 ) ((pos "L@%" ) ))
( "rye-bread"  n ( r ai1 - b r e d ) ((pos "L@%" ) ))
( "s"  n ( e1 s ) ((pos "Ki$" ) ))
( "s's"  n ( e1 s i z ) ((pos "Kj$" ) ))
( "s_eance"  n ( s ei1 aa n s ) ((pos "K6%" ) ))
( "s_eances"  n ( s ei1 aa n s i z ) ((pos "Kj%" ) ))
( "sabbatarian"  n ( s a2 b @ t e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "sabbatarians"  n ( s a2 b @ t e@1 r i@ n z ) ((pos "Kj$" ) ))
( "sabbatical"  n ( s @1 b a1 t i k l ) ((pos "K6%" "OA%" ) ))
( "sabbaticals"  n ( s @1 b a1 t i k l z ) ((pos "Kj%" ) ))
( "sable"  n ( s ei1 b l ) ((pos "M6%" "OA%" ) ))
( "sables"  n ( s ei1 b l z ) ((pos "Mj%" ) ))
( "sabot"  n ( s a1 b ou ) ((pos "K6$" ) ))
( "sabotage"  v ( s a1 b @ t aa zh ) ((pos "H2%" "L@%" ) (subcat )))
( "sabotaged"  v ( s a1 b @ t aa zh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sabotages"  v ( s a1 b @ t aa zh i z ) ((pos "Ha%" ) (subcat )))
( "sabotaging"  v ( s a1 b @ t aa zh i ng ) ((pos "Hb%" ) (subcat )))
( "saboteur"  n ( s a2 b @ t @@1 r ) ((pos "K6%" ) ))
( "saboteurs"  n ( s a2 b @ t @@1 z ) ((pos "Kj%" ) ))
( "sabots"  n ( s a1 b ou z ) ((pos "Kj$" ) ))
( "sabre"  v ( s ei1 b @ r ) ((pos "H2$" "K6%" ) ))
( "sabre-rattling"  n ( s ei1 b @ - r a t l i ng ) ((pos "L@%" ) ))
( "sabre-toothed"  j ( s ei1 b @ - t uu th t ) ((pos "OA%" ) ))
( "sabred"  v ( s ei1 b @ d ) ((pos "Hc$" "Hd$" ) ))
( "sabres"  v ( s ei1 b @ z ) ((pos "Ha$" "Kj%" ) ))
( "sabring"  v ( s ei1 b @ r i ng ) ((pos "Hb$" ) ))
( "sac"  n ( s a1 k ) ((pos "K6$" ) ))
( "saccharin"  n ( s a1 k @ r i n ) ((pos "L@%" ) ))
( "saccharine"  j ( s a1 k @ r ii n ) ((pos "OA%" ) ))
( "sacerdotal"  j ( s a2 s @ d ou1 t l ) ((pos "OA$" ) ))
( "sacerdotalism"  n ( s a2 s @ d ou1 t @ l i z @ m ) ((pos "L@$" ) ))
( "sachet"  n ( s a1 sh ei ) ((pos "K6%" ) ))
( "sachets"  n ( s a1 sh ei z ) ((pos "Kj%" ) ))
( "sack"  v ( s a1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "sack-race"  n ( s a1 k - r ei s ) ((pos "K6%" ) ))
( "sack-races"  n ( s a1 k - r ei s i z ) ((pos "Kj%" ) ))
( "sackbut"  n ( s a1 k b uh t ) ((pos "K6$" ) ))
( "sackbuts"  n ( s a1 k b uh t s ) ((pos "Kj$" ) ))
( "sackcloth"  n ( s a1 k - k l o th ) ((pos "L@%" ) ))
( "sacked"  v ( s a1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sacking"  v ( s a1 k i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "sacks"  v ( s a1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sacrament"  n ( s a1 k r @ m @ n t ) ((pos "K6%" ) ))
( "sacramental"  j ( s a2 k r @ m e1 n t l ) ((pos "OA%" ) ))
( "sacraments"  n ( s a1 k r @ m @ n t s ) ((pos "Kj%" ) ))
( "sacred"  j ( s ei1 k r i d ) ((pos "OA%" ) ))
( "sacredly"  a ( s ei1 k r i d l ii ) ((pos "Pu%" ) ))
( "sacredness"  n ( s ei1 k r i d n @ s ) ((pos "L@%" ) ))
( "sacrifice"  v ( s a1 k r i f ai s ) ((pos "J2%" "M6%" ) (subcat "3A" "6A" "14" )))
( "sacrificed"  v ( s a1 k r i f ai s t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" )))
( "sacrifices"  v ( s a1 k r i f ai s i z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "14" )))
( "sacrificial"  j ( s a2 k r i f i1 sh l ) ((pos "OA%" ) ))
( "sacrificing"  v ( s a1 k r i f ai s i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" )))
( "sacrilege"  n ( s a1 k r i l i jh ) ((pos "L@%" ) ))
( "sacrilegious"  j ( s a2 k r i l i1 jh @ s ) ((pos "OA%" ) ))
( "sacristan"  n ( s a1 k r i s t @ n ) ((pos "K6$" ) ))
( "sacristans"  n ( s a1 k r i s t @ n z ) ((pos "Kj$" ) ))
( "sacristies"  n ( s a1 k r i s t i z ) ((pos "Kj%" ) ))
( "sacristy"  n ( s a1 k r i s t ii ) ((pos "K8%" ) ))
( "sacrosanct"  j ( s a1 k r ou s a ng k t ) ((pos "OA%" ) ))
( "sacs"  n ( s a1 k s ) ((pos "Kj$" ) ))
( "sad"  j ( s a1 d ) ((pos "OE%" ) ))
( "sadden"  v ( s a1 d n ) ((pos "J0%" ) (subcat "2A" )))
( "saddened"  v ( s a1 d n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "saddening"  v ( s a1 d n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "saddens"  v ( s a1 d n z ) ((pos "Ja%" ) (subcat "2A" )))
( "sadder"  j ( s a1 d @ r ) ((pos "Or%" ) ))
( "saddest"  j ( s a1 d i s t ) ((pos "Os%" ) ))
( "saddle"  v ( s a1 d l ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "saddle-sore"  j ( s a1 d l - s oo r ) ((pos "OA%" ) ))
( "saddlebag"  n ( s a1 d l b a g ) ((pos "K6%" ) ))
( "saddlebags"  n ( s a1 d l b a g z ) ((pos "Kj%" ) ))
( "saddled"  v ( s a1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "saddler"  n ( s a1 d l @ r ) ((pos "K6%" ) ))
( "saddleries"  n ( s a1 d l @ r i z ) ((pos "Mj%" ) ))
( "saddlers"  n ( s a1 d l @ z ) ((pos "Kj%" ) ))
( "saddlery"  n ( s a1 d l @ r ii ) ((pos "M8%" ) ))
( "saddles"  v ( s a1 d l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "saddling"  v ( s a1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sadhu"  n ( s aa1 d uu ) ((pos "K6$" ) ))
( "sadhus"  n ( s aa1 d uu z ) ((pos "Kj$" ) ))
( "sadism"  n ( s ei1 d i z @ m ) ((pos "L@%" ) ))
( "sadist"  n ( s ei1 d i s t ) ((pos "K6%" ) ))
( "sadistic"  j ( s @1 d i1 s t i k ) ((pos "OA%" ) ))
( "sadists"  n ( s ei1 d i s t s ) ((pos "Kj%" ) ))
( "sadly"  a ( s a1 d l ii ) ((pos "Pu%" ) ))
( "sadness"  n ( s a1 d n @ s ) ((pos "L@%" ) ))
( "sado-masochist"  n ( s ei2 d ou - m a1 s @ k i s t ) ((pos "K6%" ) ))
( "sado-masochists"  n ( s ei2 d ou - m a1 s @ k i s t s ) ((pos "Kj%" ) ))
( "sadomasochism"  n ( s ei2 d ou m a1 s @ k i z @ m ) ((pos "L@%" ) ))
( "sae"  n ( e2 s ei2 ii1 ) ((pos "Y>%" ) ))
( "safari"  n ( s @1 f aa1 r ii ) ((pos "M6%" ) ))
( "safaris"  n ( s @1 f aa1 r i z ) ((pos "Mj%" ) ))
( "safe"  n ( s ei1 f ) ((pos "K6%" "OB%" ) ))
( "safe-conduct"  n ( s ei1 f - k o1 n d uh k t ) ((pos "L@%" ) ))
( "safe-deposit"  n ( s ei1 f - d i p o z i t ) ((pos "K6%" "OA%" ) ))
( "safe-deposits"  n ( s ei1 f - d i p o z i t s ) ((pos "Kj%" ) ))
( "safecracker"  n ( s ei1 f k r a k @ r ) ((pos "K6%" ) ))
( "safecrackers"  n ( s ei1 f k r a k @ z ) ((pos "Kj%" ) ))
( "safeguard"  v ( s ei1 f g aa d ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "safeguarded"  v ( s ei1 f g aa d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "safeguarding"  v ( s ei1 f g aa d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "safeguards"  v ( s ei1 f g aa d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "safekeeping"  n ( s ei1 f k ii1 p i ng ) ((pos "L@%" ) ))
( "safely"  a ( s ei1 f l ii ) ((pos "Pu%" ) ))
( "safeness"  n ( s ei1 f n @ s ) ((pos "L@%" ) ))
( "safer"  j ( s ei1 f @ r ) ((pos "Or%" ) ))
( "safes"  n ( s ei1 f s ) ((pos "Kj%" ) ))
( "safest"  j ( s ei1 f i s t ) ((pos "Os%" ) ))
( "safety"  n ( s ei1 f t ii ) ((pos "L@%" ) ))
( "safety-belt"  n ( s ei1 f t i - b e l t ) ((pos "K6%" ) ))
( "safety-belts"  n ( s ei1 f t i - b e l t s ) ((pos "Kj%" ) ))
( "safety-bolt"  n ( s ei1 f t i - b ou l t ) ((pos "K6%" ) ))
( "safety-bolts"  n ( s ei1 f t i - b ou l t s ) ((pos "Kj%" ) ))
( "safety-catch"  n ( s ei1 f t i - k a ch ) ((pos "K7%" ) ))
( "safety-catches"  n ( s ei1 f t i - k a ch i z ) ((pos "Kj%" ) ))
( "safety-curtain"  n ( s ei1 f t i - k @@ t n ) ((pos "K6%" ) ))
( "safety-curtains"  n ( s ei1 f t i - k @@ t n z ) ((pos "Kj%" ) ))
( "safety-factor"  n ( s ei1 f t i - f a k t @ r ) ((pos "K6%" ) ))
( "safety-factors"  n ( s ei1 f t i - f a k t @ z ) ((pos "Kj%" ) ))
( "safety-lamp"  n ( s ei1 f t i - l a m p ) ((pos "K6%" ) ))
( "safety-lamps"  n ( s ei1 f t i - l a m p s ) ((pos "Kj%" ) ))
( "safety-lock"  n ( s ei1 f t i - l o k ) ((pos "K6%" ) ))
( "safety-locks"  n ( s ei1 f t i - l o k s ) ((pos "Kj%" ) ))
( "safety-match"  n ( s ei1 f t i - m a ch ) ((pos "K7%" ) ))
( "safety-matches"  n ( s ei1 f t i - m a ch i z ) ((pos "Kj%" ) ))
( "safety-pin"  n ( s ei1 f t i - p i n ) ((pos "K6%" ) ))
( "safety-pins"  n ( s ei1 f t i - p i n z ) ((pos "Kj%" ) ))
( "safety-razor"  n ( s ei1 f t i - r ei z @ r ) ((pos "K6%" ) ))
( "safety-razors"  n ( s ei1 f t i - r ei z @ z ) ((pos "Kj%" ) ))
( "safety-valve"  n ( s ei1 f t i - v a l v ) ((pos "K6%" ) ))
( "safety-valves"  n ( s ei1 f t i - v a l v z ) ((pos "Kj%" ) ))
( "saffron"  n ( s a1 f r @ n ) ((pos "M6%" ) ))
( "saffrons"  n ( s a1 f r @ n z ) ((pos "Mj$" ) ))
( "sag"  v ( s a1 g ) ((pos "I4%" "K6$" ) (subcat )))
( "saga"  n ( s aa1 g @ ) ((pos "K6%" ) ))
( "sagacious"  j ( s @1 g ei1 sh @ s ) ((pos "OA%" ) ))
( "sagaciously"  a ( s @1 g ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "sagacity"  n ( s @1 g a1 s i t ii ) ((pos "L@%" ) ))
( "sagas"  n ( s aa1 g @ z ) ((pos "Kj%" ) ))
( "sage"  n ( s ei1 jh ) ((pos "M6%" "OA%" ) ))
( "sage-green"  n ( s ei1 jh - g r ii1 n ) ((pos "L@%" "OA%" ) ))
( "sagely"  a ( s ei1 jh l ii ) ((pos "Pu%" ) ))
( "sages"  n ( s ei1 jh i z ) ((pos "Mj%" ) ))
( "sagged"  v ( s a1 g d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sagging"  v ( s a1 g i ng ) ((pos "Ib%" ) (subcat )))
( "sago"  n ( s ei1 g ou ) ((pos "L@%" ) ))
( "sags"  v ( s a1 g z ) ((pos "Ia%" "Kj$" ) (subcat )))
( "sahib"  n ( s aa1 b ) ((pos "K6$" ) ))
( "sahibs"  n ( s aa1 b z ) ((pos "Kj$" ) ))
( "said"  v ( s e1 d ) ((pos "Jc*" "Jd*" "Oq%" ) (subcat "6A" "9" "10" )))
( "sail"  v ( s ei1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "sailcloth"  n ( s ei1 l k l o th ) ((pos "L@%" ) ))
( "sailed"  v ( s ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" )))
( "sailing"  v ( s ei1 l i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "sailing-boat"  n ( s ei1 l i ng - b ou t ) ((pos "K6%" ) ))
( "sailing-boats"  n ( s ei1 l i ng - b ou t s ) ((pos "Kj%" ) ))
( "sailing-master"  n ( s ei1 l i ng - m aa s t @ r ) ((pos "K6%" ) ))
( "sailing-masters"  n ( s ei1 l i ng - m aa s t @ z ) ((pos "Kj%" ) ))
( "sailing-ship"  n ( s ei1 l i ng - sh i p ) ((pos "K6%" ) ))
( "sailing-ships"  n ( s ei1 l i ng - sh i p s ) ((pos "Kj%" ) ))
( "sailing-vessel"  n ( s ei1 l i ng - v e s l ) ((pos "K6%" ) ))
( "sailing-vessels"  n ( s ei1 l i ng - v e s l z ) ((pos "Kj%" ) ))
( "sailings"  n ( s ei1 l i ng z ) ((pos "Kj$" ) ))
( "sailor"  n ( s ei1 l @ r ) ((pos "K6%" ) ))
( "sailors"  n ( s ei1 l @ z ) ((pos "Kj%" ) ))
( "sails"  v ( s ei1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "saint"  n ( s ei1 n t ) ((pos "K6%" ) ))
( "saint's-day"  n ( s ei1 n t s - d ei ) ((pos "K6%" ) ))
( "saint's-days"  n ( s ei1 n t s - d ei z ) ((pos "Kj%" ) ))
( "sainted"  j ( s ei1 n t i d ) ((pos "OA%" ) ))
( "sainthood"  n ( s ei1 n t h u d ) ((pos "L@%" ) ))
( "saintlier"  j ( s ei1 n t l i@ r ) ((pos "Or%" ) ))
( "saintliest"  j ( s ei1 n t l i i s t ) ((pos "Os%" ) ))
( "saintlike"  j ( s ei1 n t l ai k ) ((pos "OA%" ) ))
( "saintliness"  n ( s ei1 n t l i n @ s ) ((pos "L@%" ) ))
( "saintly"  j ( s ei1 n t l ii ) ((pos "OD%" ) ))
( "saints"  n ( s ei1 n t s ) ((pos "Kj%" ) ))
( "saith"  v ( s e1 th ) ((pos "Je$" ) (subcat "6A" "9" "10" )))
( "sak_e"  n ( s aa1 k ii ) ((pos "L@$" ) ))
( "sake"  n ( s ei1 k ) ((pos "K6%" ) ))
( "sakes"  n ( s ei1 k s ) ((pos "Kj%" ) ))
( "sal volatile"  n ( s a2 l - v @ l a1 t @ l ii ) ((pos "L@%" ) ))
( "salaam"  v ( s @1 l aa1 m ) ((pos "I0$" "K6$" ) ))
( "salaamed"  v ( s @1 l aa1 m d ) ((pos "Ic$" "Id$" ) ))
( "salaaming"  v ( s @1 l aa1 m i ng ) ((pos "Ib$" ) ))
( "salaams"  v ( s @1 l aa1 m z ) ((pos "Ia$" "Kj$" ) ))
( "salable"  j ( s ei1 l @ b l ) ((pos "OA$" ) ))
( "salacious"  j ( s @1 l ei1 sh @ s ) ((pos "OA%" ) ))
( "salaciously"  a ( s @1 l ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "salaciousness"  n ( s @1 l ei1 sh @ s n @ s ) ((pos "L@%" ) ))
( "salacity"  n ( s @1 l a1 s i t ii ) ((pos "L@$" ) ))
( "salad"  n ( s a1 l @ d ) ((pos "M6%" ) ))
( "salad-days"  n ( s a1 l @ d - d ei z ) ((pos "Kj%" ) ))
( "salad-dressing"  n ( s a1 l @ d - d r e s i ng ) ((pos "M6%" ) ))
( "salad-dressings"  n ( s a1 l @ d - d r e s i ng z ) ((pos "Mj%" ) ))
( "salad-oil"  n ( s a1 l @ d - oi l ) ((pos "M6%" ) ))
( "salad-oils"  n ( s a1 l @ d - oi l z ) ((pos "Mj%" ) ))
( "salads"  n ( s a1 l @ d z ) ((pos "Mj%" ) ))
( "salamander"  n ( s a1 l @ m a n d @ r ) ((pos "K6$" ) ))
( "salamanders"  n ( s a1 l @ m a n d @ z ) ((pos "Kj$" ) ))
( "salami"  n ( s @1 l aa1 m ii ) ((pos "L@%" ) ))
( "salaried"  j ( s a1 l @ r i d ) ((pos "OA%" ) ))
( "salaries"  n ( s a1 l @ r i z ) ((pos "Kj%" ) ))
( "salary"  n ( s a1 l @ r ii ) ((pos "K8%" ) ))
( "sale"  n ( s ei1 l ) ((pos "M6%" ) ))
( "saleable"  j ( s ei1 l @ b l ) ((pos "OA%" ) ))
( "saleroom"  n ( s ei1 l r uu m ) ((pos "K6%" ) ))
( "salerooms"  n ( s ei1 l r uu m z ) ((pos "Kj%" ) ))
( "sales"  n ( s ei1 l z ) ((pos "Mj%" ) ))
( "salesman"  n ( s ei1 l z m @ n ) ((pos "Ki%" ) ))
( "salesmanship"  n ( s ei1 l z m @ n sh i p ) ((pos "L@%" ) ))
( "salesmen"  n ( s ei1 l z m @ n ) ((pos "Kj%" ) ))
( "saleswoman"  n ( s ei1 l z w u m @ n ) ((pos "Ki%" ) ))
( "saleswomen"  n ( s ei1 l z w i m i n ) ((pos "Kj%" ) ))
( "salience"  n ( s ei1 l i@ n s ) ((pos "L@%" ) ))
( "salient"  n ( s ei1 l i@ n t ) ((pos "K6$" "OA%" ) ))
( "salients"  n ( s ei1 l i@ n t s ) ((pos "Kj$" ) ))
( "saline"  n ( s ei1 l ai n ) ((pos "M6$" "OA%" ) ))
( "salines"  n ( s ei1 l ai n z ) ((pos "Mj$" ) ))
( "salinity"  n ( s @1 l i1 n i t ii ) ((pos "L@%" ) ))
( "saliva"  n ( s @1 l ai1 v @ ) ((pos "L@%" ) ))
( "salivary"  j ( s a1 l i v @ r ii ) ((pos "OA%" ) ))
( "salivate"  v ( s a1 l i v ei t ) ((pos "I2%" ) ))
( "salivated"  v ( s a1 l i v ei t i d ) ((pos "Ic%" "Id%" ) ))
( "salivates"  v ( s a1 l i v ei t s ) ((pos "Ia%" ) ))
( "salivating"  v ( s a1 l i v ei t i ng ) ((pos "Ib%" ) ))
( "sallied"  v ( s a1 l i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sallies"  v ( s a1 l i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "sallow"  v ( s a1 l ou ) ((pos "J0$" "OC%" ) ))
( "sallowed"  v ( s a1 l ou d ) ((pos "Jc$" "Jd$" ) ))
( "sallower"  j ( s a1 l ou @ r ) ((pos "Or$" ) ))
( "sallowest"  j ( s a1 l ou i s t ) ((pos "Os$" ) ))
( "sallowing"  v ( s a1 l ou i ng ) ((pos "Jb$" ) ))
( "sallows"  v ( s a1 l ou z ) ((pos "Ja$" ) ))
( "sally"  v ( s a1 l ii ) ((pos "I3%" "K8%" ) (subcat "2A" )))
( "sallying"  v ( s a1 l i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "salmon"  n ( s a1 m @ n ) ((pos "M9%" ) ))
( "salon"  n ( s a1 l o n ) ((pos "K6%" ) ))
( "salons"  n ( s a1 l o n z ) ((pos "Kj%" ) ))
( "saloon"  n ( s @1 l uu1 n ) ((pos "K6%" ) ))
( "saloons"  n ( s @1 l uu1 n z ) ((pos "Kj%" ) ))
( "salsify"  n ( s a1 l s i f ii ) ((pos "L@$" ) ))
( "salt"  v ( s oo1 l t ) ((pos "H0%" "M6%" "OA%" ) (subcat "6A" )))
( "salt-cellar"  n ( s oo1 l t - s e l @ r ) ((pos "K6%" ) ))
( "salt-cellars"  n ( s oo1 l t - s e l @ z ) ((pos "Kj%" ) ))
( "salt-lick"  n ( s oo1 l t - l i k ) ((pos "K6%" ) ))
( "salt-licks"  n ( s oo1 l t - l i k s ) ((pos "Kj%" ) ))
( "salted"  v ( s oo1 l t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "saltier"  j ( s oo1 l t i@ r ) ((pos "Or%" ) ))
( "saltiest"  j ( s oo1 l t i i s t ) ((pos "Os%" ) ))
( "saltiness"  n ( s oo1 l t i n @ s ) ((pos "L@%" ) ))
( "salting"  v ( s oo1 l t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "saltpan"  n ( s oo1 l t p a n ) ((pos "K6%" ) ))
( "saltpans"  n ( s oo1 l t p a n z ) ((pos "Kj%" ) ))
( "saltpetre"  n ( s oo1 l t p ii1 t @ r ) ((pos "L@%" ) ))
( "salts"  v ( s oo1 l t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "saltwater"  n ( s oo1 l t w oo t @ r ) ((pos "L@%" ) ))
( "saltworks"  n ( s oo1 l t w @@ k s ) ((pos "K9%" ) ))
( "salty"  j ( s oo1 l t ii ) ((pos "OD%" ) ))
( "salubrious"  j ( s @1 l uu1 b r i@ s ) ((pos "OA%" ) ))
( "salubrity"  n ( s @1 l uu1 b r i t ii ) ((pos "L@$" ) ))
( "salutary"  j ( s a1 l y u t r ii ) ((pos "OA%" ) ))
( "salutation"  n ( s a2 l y uu t ei1 sh @ n ) ((pos "M6%" ) ))
( "salutations"  n ( s a2 l y uu t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "salute"  v ( s @1 l uu1 t ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "saluted"  v ( s @1 l uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "salutes"  v ( s @1 l uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "saluting"  v ( s @1 l uu1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "salvage"  v ( s a1 l v i jh ) ((pos "H2%" "L@%" ) (subcat )))
( "salvaged"  v ( s a1 l v i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "salvages"  v ( s a1 l v i jh i z ) ((pos "Ha%" ) (subcat )))
( "salvaging"  v ( s a1 l v i jh i ng ) ((pos "Hb%" ) (subcat )))
( "salvation"  n ( s a1 l v ei1 sh @ n ) ((pos "L@%" ) ))
( "salve"  v ( s a1 l v ) ((pos "H2%" "M6%" ) (subcat )))
( "salved"  v ( s a1 l v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "salver"  n ( s a1 l v @ r ) ((pos "K6%" ) ))
( "salvers"  n ( s a1 l v @ z ) ((pos "Kj%" ) ))
( "salves"  v ( s a1 l v z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "salvia"  n ( s a1 l v i@ ) ((pos "K6$" ) ))
( "salvias"  n ( s a1 l v i@ z ) ((pos "Kj$" ) ))
( "salving"  v ( s a1 l v i ng ) ((pos "Hb%" ) (subcat )))
( "salvo"  n ( s a1 l v ou ) ((pos "K6%" ) ))
( "salvoes"  n ( s a1 l v ou z ) ((pos "Kj%" ) ))
( "salvos"  n ( s a1 l v ou z ) ((pos "Kj%" ) ))
( "samba"  n ( s a1 m b @ ) ((pos "K6%" ) ))
( "sambas"  n ( s a1 m b @ z ) ((pos "Kj%" ) ))
( "same"  j ( s ei m ) ((pos "OA*" "Pu*" "Qx*" ) ))
( "sameness"  n ( s ei1 m n @ s ) ((pos "L@%" ) ))
( "samovar"  n ( s a1 m @ v aa r ) ((pos "K6$" ) ))
( "samovars"  n ( s a1 m @ v aa z ) ((pos "Kj$" ) ))
( "sampan"  n ( s a1 m p a n ) ((pos "K6$" ) ))
( "sampans"  n ( s a1 m p a n z ) ((pos "Kj$" ) ))
( "sample"  v ( s aa1 m p l ) ((pos "H2%" "K6%" ) (subcat )))
( "sampled"  v ( s aa1 m p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sampler"  n ( s aa1 m p l @ r ) ((pos "K6%" ) ))
( "samplers"  n ( s aa1 m p l @ z ) ((pos "Kj%" ) ))
( "samples"  v ( s aa1 m p l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sampling"  v ( s aa1 m p l i ng ) ((pos "Hb%" ) (subcat )))
( "samurai"  n ( s a1 m u r ai ) ((pos "K9$" ) ))
( "sanatoria"  n ( s a2 n @ t oo1 r i@ ) ((pos "Kj%" ) ))
( "sanatorium"  n ( s a2 n @ t oo1 r i@ m ) ((pos "K6%" ) ))
( "sanatoriums"  n ( s a2 n @ t oo1 r i@ m z ) ((pos "Kj$" ) ))
( "sanctification"  n ( s a2 ng k t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "sanctifications"  n ( s a2 ng k t i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "sanctified"  v ( s a1 ng k t i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sanctifies"  v ( s a1 ng k t i f ai z ) ((pos "Ha%" ) (subcat )))
( "sanctify"  v ( s a1 ng k t i f ai ) ((pos "H3%" ) (subcat )))
( "sanctifying"  v ( s a1 ng k t i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "sanctimonious"  j ( s a2 ng k t i m ou1 n i@ s ) ((pos "OA%" ) ))
( "sanctimoniously"  a ( s a2 ng k t i m ou1 n i@ s l ii ) ((pos "Pu%" ) ))
( "sanction"  v ( s a1 ng k sh @ n ) ((pos "H0%" "M6%" ) (subcat )))
( "sanctioned"  v ( s a1 ng k sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sanctioning"  v ( s a1 ng k sh @ n i ng ) ((pos "Hb%" ) (subcat )))
( "sanctions"  v ( s a1 ng k sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sanctities"  n ( s a1 ng k t i t i z ) ((pos "Mj$" ) ))
( "sanctity"  n ( s a1 ng k t i t ii ) ((pos "M8%" ) ))
( "sanctuaries"  n ( s a1 ng k ch u@ r i z ) ((pos "Mj%" ) ))
( "sanctuary"  n ( s a1 ng k ch u@ r ii ) ((pos "M8%" ) ))
( "sanctum"  n ( s a1 ng k t @ m ) ((pos "K6%" ) ))
( "sanctums"  n ( s a1 ng k t @ m z ) ((pos "Kj%" ) ))
( "sand"  v ( s a1 n d ) ((pos "H0%" "M6%" ) (subcat )))
( "sand-bar"  n ( s a1 n d - b aa r ) ((pos "K6%" ) ))
( "sand-bars"  n ( s a1 n d - b aa z ) ((pos "Kj%" ) ))
( "sandal"  n ( s a1 n d l ) ((pos "K6%" ) ))
( "sandalled"  j ( s a1 n d l d ) ((pos "OA$" ) ))
( "sandals"  n ( s a1 n d l z ) ((pos "Kj%" ) ))
( "sandalwood"  n ( s a1 n d l w u d ) ((pos "L@%" ) ))
( "sandbag"  n ( s a1 n d b a g ) ((pos "K6%" ) ))
( "sandbags"  n ( s a1 n d b a g z ) ((pos "Kj%" ) ))
( "sandbank"  n ( s a1 n d b a ng k ) ((pos "K6%" ) ))
( "sandbanks"  n ( s a1 n d b a ng k s ) ((pos "Kj%" ) ))
( "sandblast"  v ( s a1 n d b l a s t ) ((pos "H0%" ) (subcat )))
( "sandblasted"  v ( s a1 n d b l a s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sandblasting"  v ( s a1 n d b l a s t i ng ) ((pos "Hb%" ) (subcat )))
( "sandblasts"  v ( s a1 n d b l a s t s ) ((pos "Ha%" ) (subcat )))
( "sandboy"  n ( s a1 n d b oi ) ((pos "K6%" ) ))
( "sandboys"  n ( s a1 n d b oi z ) ((pos "Kj%" ) ))
( "sanded"  v ( s a1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sandflies"  n ( s a1 n d f l ai z ) ((pos "Kj%" ) ))
( "sandfly"  n ( s a1 n d f l ai ) ((pos "K8%" ) ))
( "sandglass"  n ( s a1 n d g l aa s ) ((pos "K7$" ) ))
( "sandglasses"  n ( s a1 n d g l aa s i z ) ((pos "Kj$" ) ))
( "sandier"  j ( s a1 n d i@ r ) ((pos "Or%" ) ))
( "sandiest"  j ( s a1 n d i i s t ) ((pos "Os%" ) ))
( "sandiness"  n ( s a1 n d i n @ s ) ((pos "L@%" ) ))
( "sanding"  v ( s a1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "sandpaper"  v ( s a1 n d p ei p @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "sandpapered"  v ( s a1 n d p ei p @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sandpapering"  v ( s a1 n d p ei p @ r i ng ) ((pos "Hb%" ) (subcat )))
( "sandpapers"  v ( s a1 n d p ei p @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sandpiper"  n ( s a1 n d p ai p @ r ) ((pos "K6%" ) ))
( "sandpipers"  n ( s a1 n d p ai p @ z ) ((pos "Kj%" ) ))
( "sandpit"  n ( s a1 n d p i t ) ((pos "K6%" ) ))
( "sandpits"  n ( s a1 n d p i t s ) ((pos "Kj%" ) ))
( "sands"  v ( s a1 n d z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sandshoes"  n ( s a1 n d sh uu z ) ((pos "Kj%" ) ))
( "sandstone"  n ( s a1 n d s t ou n ) ((pos "L@%" ) ))
( "sandstorm"  n ( s a1 n d s t oo m ) ((pos "K6%" ) ))
( "sandstorms"  n ( s a1 n d s t oo m z ) ((pos "Kj%" ) ))
( "sandwich"  v ( s a1 n w i jh ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "sandwich-board"  n ( s a1 n w i jh - b oo d ) ((pos "K6%" ) ))
( "sandwich-boards"  n ( s a1 n w i jh - b oo d z ) ((pos "Kj%" ) ))
( "sandwiched"  v ( s a1 n w i jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "sandwiches"  v ( s a1 n w i jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "sandwiching"  v ( s a1 n w i jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sandwichman"  n ( s a1 n w i jh m a n ) ((pos "Ki%" ) ))
( "sandwichmen"  n ( s a1 n w i jh m e n ) ((pos "Kj%" ) ))
( "sandy"  j ( s a1 n d ii ) ((pos "OD%" ) ))
( "sane"  j ( s ei1 n ) ((pos "OB%" ) ))
( "sanely"  a ( s ei1 n l ii ) ((pos "Pu%" ) ))
( "saner"  j ( s ei1 n @ r ) ((pos "Or%" ) ))
( "sanest"  j ( s ei1 n i s t ) ((pos "Os%" ) ))
( "sang"  v ( s a1 ng ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" )))
( "sang froid"  n ( s o2 ng - f r w aa1 ) ((pos "L@%" ) ))
( "sanguinary"  j ( s a1 ng g w i n @ r ii ) ((pos "OA$" ) ))
( "sanguine"  j ( s a1 ng g w i n ) ((pos "OA%" ) ))
( "sanitary"  j ( s a1 n i t r ii ) ((pos "OA%" ) ))
( "sanitation"  n ( s a2 n i t ei1 sh @ n ) ((pos "L@%" ) ))
( "sanity"  n ( s a1 n i t ii ) ((pos "L@%" ) ))
( "sank"  v ( s a1 ng k ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sans"  in ( s a n z ) ((pos "T-$" ) ))
( "sap"  v ( s a1 p ) ((pos "J4%" "M6%" ) (subcat "2A" )))
( "saphead"  n ( s a1 p h e d ) ((pos "K6$" ) ))
( "sapheads"  n ( s a1 p h e d z ) ((pos "Kj$" ) ))
( "sapience"  n ( s ei1 p i@ n s ) ((pos "L@$" ) ))
( "sapient"  j ( s ei1 p i@ n t ) ((pos "OA$" ) ))
( "sapiently"  a ( s ei1 p i@ n t l ii ) ((pos "Pu$" ) ))
( "sapless"  j ( s a1 p l @ s ) ((pos "OA$" ) ))
( "sapling"  n ( s a1 p l i ng ) ((pos "K6%" ) ))
( "saplings"  n ( s a1 p l i ng z ) ((pos "Kj%" ) ))
( "sapped"  v ( s a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sapper"  n ( s a1 p @ r ) ((pos "K6%" ) ))
( "sappers"  n ( s a1 p @ z ) ((pos "Kj%" ) ))
( "sapphire"  n ( s a1 f ai @ r ) ((pos "M6%" ) ))
( "sapphires"  n ( s a1 f ai @ z ) ((pos "Mj%" ) ))
( "sappier"  j ( s a1 p i@ r ) ((pos "Or$" ) ))
( "sappiest"  j ( s a1 p i i s t ) ((pos "Os$" ) ))
( "sapping"  v ( s a1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sappy"  j ( s a1 p ii ) ((pos "OD%" ) ))
( "saps"  v ( s a1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "sapwood"  n ( s a1 p w u d ) ((pos "L@$" ) ))
( "saraband"  n ( s a1 r @ b a n d ) ((pos "K6$" ) ))
( "sarabands"  n ( s a1 r @ b a n d z ) ((pos "Kj$" ) ))
( "sarcasm"  n ( s aa1 k a z @ m ) ((pos "M6%" ) ))
( "sarcasms"  n ( s aa1 k a z @ m z ) ((pos "Mj$" ) ))
( "sarcastic"  j ( s aa1 k a1 s t i k ) ((pos "OA%" ) ))
( "sarcastically"  a ( s aa1 k a1 s t i k l ii ) ((pos "Pu%" ) ))
( "sarcophagi"  n ( s aa1 k o1 f @ g ai ) ((pos "Kj$" ) ))
( "sarcophagus"  n ( s aa1 k o1 f @ g @ s ) ((pos "Ki$" ) ))
( "sardine"  n ( s aa1 d ii1 n ) ((pos "M6%" ) ))
( "sardines"  n ( s aa1 d ii1 n z ) ((pos "Mj%" ) ))
( "sardonic"  j ( s aa1 d o1 n i k ) ((pos "OA%" ) ))
( "sardonically"  a ( s aa1 d o1 n i k l ii ) ((pos "Pu%" ) ))
( "sari"  n ( s aa1 r ii ) ((pos "K6%" ) ))
( "saris"  n ( s aa1 r i z ) ((pos "Kj%" ) ))
( "sarong"  n ( s @1 r o1 ng ) ((pos "K6$" ) ))
( "sarongs"  n ( s @1 r o1 ng z ) ((pos "Kj$" ) ))
( "sarsaparilla"  n ( s aa2 s p @ r i1 l @ ) ((pos "L@$" ) ))
( "sartorial"  j ( s aa1 t oo1 r i@ l ) ((pos "OA$" ) ))
( "sash"  n ( s a1 sh ) ((pos "K7%" ) ))
( "sash-cord"  n ( s a1 sh - k oo d ) ((pos "K6%" ) ))
( "sash-cords"  n ( s a1 sh - k oo d z ) ((pos "Kj%" ) ))
( "sash-line"  n ( s a1 sh - l ai n ) ((pos "K6$" ) ))
( "sash-lines"  n ( s a1 sh - l ai n z ) ((pos "Kj$" ) ))
( "sashes"  n ( s a1 sh i z ) ((pos "Kj%" ) ))
( "sat"  v ( s a1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "satchel"  n ( s a1 ch l ) ((pos "K6%" ) ))
( "satchels"  n ( s a1 ch l z ) ((pos "Kj%" ) ))
( "sate"  v ( s ei1 t ) ((pos "H2$" ) (subcat )))
( "sated"  v ( s ei1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "sateen"  n ( s @1 t ii1 n ) ((pos "L@$" ) ))
( "satellite"  n ( s a1 t @ l ai t ) ((pos "K6%" ) ))
( "satellites"  n ( s a1 t @ l ai t s ) ((pos "Kj%" ) ))
( "sates"  v ( s ei1 t s ) ((pos "Ha$" ) (subcat )))
( "satiable"  j ( s ei1 sh @ b l ) ((pos "OA%" ) ))
( "satiate"  v ( s ei1 sh i ei t ) ((pos "H2%" ) (subcat )))
( "satiated"  v ( s ei1 sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "satiates"  v ( s ei1 sh i ei t s ) ((pos "Ha%" ) (subcat )))
( "satiating"  v ( s ei1 sh i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "satiety"  n ( s @1 t ai1 @ t ii ) ((pos "L@%" ) ))
( "satin"  n ( s a1 t i n ) ((pos "L@%" "OA%" ) ))
( "sating"  v ( s ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "satinwood"  n ( s a1 t i n w u d ) ((pos "L@%" ) ))
( "satire"  n ( s a1 t ai @ r ) ((pos "M6%" ) ))
( "satires"  n ( s a1 t ai @ z ) ((pos "Mj%" ) ))
( "satirical"  j ( s @1 t i1 r i k l ) ((pos "OA%" ) ))
( "satirically"  a ( s @1 t i1 r i k l ii ) ((pos "Pu%" ) ))
( "satirist"  n ( s a1 t @ r i s t ) ((pos "K6%" ) ))
( "satirists"  n ( s a1 t @ r i s t s ) ((pos "Kj%" ) ))
( "satirize"  v ( s a1 t @ r ai z ) ((pos "H2%" ) (subcat )))
( "satirized"  v ( s a1 t @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "satirizes"  v ( s a1 t @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "satirizing"  v ( s a1 t @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "satisfaction"  n ( s a2 t i s f a1 k sh @ n ) ((pos "M6%" ) ))
( "satisfactions"  n ( s a2 t i s f a1 k sh @ n z ) ((pos "Mj%" ) ))
( "satisfactorily"  a ( s a2 t i s f a1 k t @ r i l ii ) ((pos "Pu%" ) ))
( "satisfactory"  j ( s a2 t i s f a1 k t @ r ii ) ((pos "OA%" ) ))
( "satisfied"  v ( s a1 t i s f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "11" )))
( "satisfies"  v ( s a1 t i s f ai z ) ((pos "Ja%" ) (subcat "2A" "6A" "11" )))
( "satisfy"  v ( s a1 t i s f ai ) ((pos "J3%" ) (subcat "2A" "6A" "11" )))
( "satisfying"  v ( s a1 t i s f ai i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "6A" "11" )))
( "satisfyingly"  a ( s a1 t i s f ai i ng l ii ) ((pos "Pu%" ) ))
( "satrap"  n ( s a1 t r a p ) ((pos "K6$" ) ))
( "satraps"  n ( s a1 t r a p s ) ((pos "Kj$" ) ))
( "satsuma"  n ( s a2 t s uu1 m @ ) ((pos "K6%" ) ))
( "satsumas"  n ( s a2 t s uu1 m @ z ) ((pos "Kj%" ) ))
( "saturate"  v ( s a1 ch @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "saturated"  v ( s a1 ch @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "saturates"  v ( s a1 ch @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "saturating"  v ( s a1 ch @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "saturation"  n ( s a2 ch @ r ei1 sh @ n ) ((pos "L@%" ) ))
( "saturnalia"  n ( s a2 t @ n ei1 l i@ ) ((pos "K6$" ) ))
( "saturnalias"  n ( s a2 t @ n ei1 l i@ z ) ((pos "Kj$" ) ))
( "saturnine"  j ( s a1 t @ n ai n ) ((pos "OA%" ) ))
( "satyr"  n ( s a1 t @ r ) ((pos "K6$" ) ))
( "satyric"  j ( s @1 t i1 r i k ) ((pos "OA$" ) ))
( "satyrs"  n ( s a1 t @ z ) ((pos "Kj$" ) ))
( "sauce"  v ( s oo1 s ) ((pos "H2$" "M6%" ) (subcat )))
( "sauce-boat"  n ( s oo1 s - b ou t ) ((pos "K6%" ) ))
( "sauce-boats"  n ( s oo1 s - b ou t s ) ((pos "Kj%" ) ))
( "sauced"  v ( s oo1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "saucepan"  n ( s oo1 s p @ n ) ((pos "K6%" ) ))
( "saucepans"  n ( s oo1 s p @ n z ) ((pos "Kj%" ) ))
( "saucer"  n ( s oo1 s @ r ) ((pos "K6%" ) ))
( "saucer-eyed"  j ( s oo1 s @ r - ai d ) ((pos "OA%" ) ))
( "saucers"  n ( s oo1 s @ z ) ((pos "Kj%" ) ))
( "sauces"  v ( s oo1 s i z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "saucier"  j ( s oo1 s i@ r ) ((pos "Or%" ) ))
( "sauciest"  j ( s oo1 s i i s t ) ((pos "Os%" ) ))
( "saucily"  a ( s oo1 s i l ii ) ((pos "Pu%" ) ))
( "sauciness"  n ( s oo1 s i n @ s ) ((pos "L@%" ) ))
( "saucing"  v ( s oo1 s i ng ) ((pos "Hb$" ) (subcat )))
( "saucy"  j ( s oo1 s ii ) ((pos "OD%" ) ))
( "sauerkraut"  n ( s au1 @ k r au t ) ((pos "L@%" ) ))
( "sauna"  n ( s au1 n @ ) ((pos "K6%" ) ))
( "saunas"  n ( s au1 n @ z ) ((pos "Kj%" ) ))
( "saunter"  v ( s oo1 n t @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "sauntered"  v ( s oo1 n t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "saunterer"  n ( s oo1 n t @ r @ r ) ((pos "K6$" ) ))
( "saunterers"  n ( s oo1 n t @ r @ z ) ((pos "Kj$" ) ))
( "sauntering"  v ( s oo1 n t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "saunters"  v ( s oo1 n t @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "saurian"  n ( s oo1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "saurians"  n ( s oo1 r i@ n z ) ((pos "Kj$" ) ))
( "sausage"  n ( s o1 s i jh ) ((pos "M6%" ) ))
( "sausage-dog"  n ( s o1 s i jh - d o g ) ((pos "K6%" ) ))
( "sausage-dogs"  n ( s o1 s i jh - d o g z ) ((pos "Kj%" ) ))
( "sausage-meat"  n ( s o1 s i jh - m ii t ) ((pos "L@%" ) ))
( "sausage-roll"  n ( s o2 s i jh - r ou1 l ) ((pos "K6%" ) ))
( "sausage-rolls"  n ( s o2 s i jh - r ou1 l z ) ((pos "Kj%" ) ))
( "sausages"  n ( s o1 s i jh i z ) ((pos "Mj%" ) ))
( "saut_e"  v ( s ou1 t ei ) ((pos "H0%" "OA%" ) ))
( "saut_eed"  v ( s ou1 t ei d ) ((pos "Hc%" "Hd%" ) ))
( "saut_eing"  v ( s ou1 t ei i ng ) ((pos "Hb%" ) ))
( "saut_es"  v ( s ou1 t ei z ) ((pos "Ha%" ) ))
( "savage"  v ( s a1 v i jh ) ((pos "H2%" "K6%" "OA%" ) (subcat )))
( "savaged"  v ( s a1 v i jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "savagely"  a ( s a1 v i jh l ii ) ((pos "Pu%" ) ))
( "savageness"  n ( s a1 v i jh n @ s ) ((pos "L@%" ) ))
( "savagery"  n ( s a1 v i jh r ii ) ((pos "L@%" ) ))
( "savages"  v ( s a1 v i jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "savaging"  v ( s a1 v i jh i ng ) ((pos "Hb%" ) (subcat )))
( "savanna"  n ( s @1 v a1 n @ ) ((pos "K6%" ) ))
( "savannah"  n ( s @1 v a1 n @ ) ((pos "K6%" ) ))
( "savannahs"  n ( s @1 v a1 n @ z ) ((pos "Kj$" ) ))
( "savannas"  n ( s @1 v a1 n @ z ) ((pos "Kj$" ) ))
( "savant"  n ( s a1 v @ n t ) ((pos "K6$" ) ))
( "savants"  n ( s a1 v @ n t s ) ((pos "Kj$" ) ))
( "save"  v ( s ei v ) ((pos "J2%" "K6%" "T-%" ) (subcat "2A" "2C" "3A" "6A" "6D" "12B" "12C" "13B" "14" )))
( "saved"  v ( s ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "6D" "12B" "12C" "13B" "14" )))
( "saveloy"  n ( s a1 v @ l oi ) ((pos "M6%" ) ))
( "saveloys"  n ( s a1 v @ l oi z ) ((pos "Mj%" ) ))
( "saver"  n ( s ei1 v @ r ) ((pos "K6%" ) ))
( "savers"  n ( s ei1 v @ z ) ((pos "Kj%" ) ))
( "saves"  v ( s ei1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "6D" "12B" "12C" "13B" "14" )))
( "saving"  v ( s ei1 v i ng ) ((pos "Jb%" "K6%" "OA%" "T-%" ) (subcat "2A" "2C" "3A" "6A" "6D" "12B" "12C" "13B" "14" )))
( "savings"  n ( s ei1 v i ng z ) ((pos "Kj%" ) ))
( "savings-bank"  n ( s ei1 v i ng z - b a ng k ) ((pos "K6%" ) ))
( "savings-banks"  n ( s ei1 v i ng z - b a ng k s ) ((pos "Kj%" ) ))
( "saviour"  n ( s ei1 v i@ r ) ((pos "K6%" ) ))
( "saviours"  n ( s ei1 v i@ z ) ((pos "Kj%" ) ))
( "savoir-faire"  n ( s a2 v w aa - f e@1 r ) ((pos "L@%" ) ))
( "savory"  n ( s ei1 v @ r ii ) ((pos "L@$" ) ))
( "savour"  v ( s ei1 v @ r ) ((pos "J0%" "M6%" ) (subcat "3A" )))
( "savoured"  v ( s ei1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "savouries"  n ( s ei1 v @ r i z ) ((pos "Kj%" ) ))
( "savouring"  v ( s ei1 v @ r i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "savours"  v ( s ei1 v @ z ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "savoury"  n ( s ei1 v @ r ii ) ((pos "K8%" "OA%" ) ))
( "savoy"  n ( s @1 v oi1 ) ((pos "M6$" ) ))
( "savoys"  n ( s @1 v oi1 z ) ((pos "Mj$" ) ))
( "savvy"  v ( s a1 v ii ) ((pos "I5%" "L@%" ) ))
( "saw"  v ( s oo1 ) ((pos "J0%" "Jc*" "K6%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "15B" "16B" "18A" "19A" "22" )))
( "saw-pit"  n ( s oo1 - p i t ) ((pos "K6$" ) ))
( "saw-pits"  n ( s oo1 - p i t s ) ((pos "Kj$" ) ))
( "sawdust"  n ( s oo1 d uh s t ) ((pos "L@%" ) ))
( "sawed"  v ( s oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "15B" "16B" "18A" "19A" "22" )))
( "sawhorse"  n ( s oo1 h oo s ) ((pos "K6$" ) ))
( "sawhorses"  n ( s oo1 h oo s i z ) ((pos "Kj$" ) ))
( "sawing"  v ( s oo1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "15B" "16B" "18A" "19A" "22" )))
( "sawmill"  n ( s oo1 m i l ) ((pos "K6%" ) ))
( "sawmills"  n ( s oo1 m i l z ) ((pos "Kj%" ) ))
( "sawn"  v ( s oo1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "saws"  v ( s oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "15B" "16B" "18A" "19A" "22" )))
( "sawyer"  n ( s oo1 y @ r ) ((pos "K6%" ) ))
( "sawyers"  n ( s oo1 y @ z ) ((pos "Kj%" ) ))
( "sax"  n ( s a1 k s ) ((pos "K7%" ) ))
( "saxes"  n ( s a1 k s i z ) ((pos "Kj%" ) ))
( "saxhorn"  n ( s a1 k s h oo n ) ((pos "K6$" ) ))
( "saxhorns"  n ( s a1 k s h oo n z ) ((pos "Kj$" ) ))
( "saxifrage"  n ( s a1 k s i f r i jh ) ((pos "L@%" ) ))
( "saxophone"  n ( s a1 k s @ f ou n ) ((pos "K6%" ) ))
( "saxophones"  n ( s a1 k s @ f ou n z ) ((pos "Kj%" ) ))
( "saxophonist"  n ( s a1 k s o1 f @ n i s t ) ((pos "K6%" ) ))
( "saxophonists"  n ( s a1 k s o1 f @ n i s t s ) ((pos "Kj%" ) ))
( "say"  v ( s ei1 ) ((pos "J5*" "Ki%" ) (subcat "6A" "9" "10" )))
( "saying"  v ( s ei1 i ng ) ((pos "Jb%" "K6%" ) (subcat "6A" "9" "10" )))
( "sayings"  n ( s ei1 i ng z ) ((pos "Kj%" ) ))
( "says"  v ( s e1 z ) ((pos "Ja*" ) (subcat "6A" "9" "10" )))
( "scab"  n ( s k a1 b ) ((pos "M6%" ) ))
( "scabbard"  n ( s k a1 b @ d ) ((pos "K6$" ) ))
( "scabbards"  n ( s k a1 b @ d z ) ((pos "Kj$" ) ))
( "scabbier"  j ( s k a1 b i@ r ) ((pos "Or$" ) ))
( "scabbiest"  j ( s k a1 b i i s t ) ((pos "Os$" ) ))
( "scabby"  j ( s k a1 b ii ) ((pos "OD%" ) ))
( "scabies"  n ( s k ei1 b ii z ) ((pos "L@%" ) ))
( "scabious"  n ( s k ei1 b i@ s ) ((pos "L@$" ) ))
( "scabrous"  j ( s k ei1 b r @ s ) ((pos "OA$" ) ))
( "scabs"  n ( s k a1 b z ) ((pos "Mj%" ) ))
( "scaffold"  n ( s k a1 f ou l d ) ((pos "K6%" ) ))
( "scaffolding"  n ( s k a1 f @ l d i ng ) ((pos "L@%" ) ))
( "scaffolds"  n ( s k a1 f ou l d z ) ((pos "Kj%" ) ))
( "scalawag"  n ( s k a1 l @ w a g ) ((pos "K6$" ) ))
( "scalawags"  n ( s k a1 l @ w a g z ) ((pos "Kj$" ) ))
( "scald"  v ( s k oo1 l d ) ((pos "H0%" "K6%" ) (subcat )))
( "scalded"  v ( s k oo1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scalding"  v ( s k oo1 l d i ng ) ((pos "Hb%" ) (subcat )))
( "scalds"  v ( s k oo1 l d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "scale"  v ( s k ei1 l ) ((pos "J2%" "M6%" ) (subcat "2B" "2C" "6A" "15A" )))
( "scaled"  v ( s k ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2B" "2C" "6A" "15A" )))
( "scales"  v ( s k ei1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2B" "2C" "6A" "15A" )))
( "scalier"  j ( s k ei1 l i@ r ) ((pos "Or%" ) ))
( "scaliest"  j ( s k ei1 l i i s t ) ((pos "Os%" ) ))
( "scaling"  v ( s k ei1 l i ng ) ((pos "Jb%" ) (subcat "2B" "2C" "6A" "15A" )))
( "scaling-ladder"  n ( s k ei1 l i ng - l a d @ r ) ((pos "K6%" ) ))
( "scaling-ladders"  n ( s k ei1 l i ng - l a d @ z ) ((pos "Kj%" ) ))
( "scallop"  v ( s k o1 l @ p ) ((pos "H0$" "K6%" ) (subcat )))
( "scallop-shell"  n ( s k o1 l @ p - sh e l ) ((pos "K6%" ) ))
( "scallop-shells"  n ( s k o1 l @ p - sh e l z ) ((pos "Kj%" ) ))
( "scalloped"  v ( s k o1 l @ p t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "scalloping"  v ( s k o1 l @ p i ng ) ((pos "Hb$" ) (subcat )))
( "scallops"  v ( s k o1 l @ p s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "scallywag"  n ( s k a1 l i w a g ) ((pos "K6%" ) ))
( "scallywags"  n ( s k a1 l i w a g z ) ((pos "Kj%" ) ))
( "scalp"  v ( s k a1 l p ) ((pos "H0%" "K6%" ) (subcat )))
( "scalped"  v ( s k a1 l p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scalpel"  n ( s k a1 l p @ l ) ((pos "K6%" ) ))
( "scalpels"  n ( s k a1 l p @ l z ) ((pos "Kj%" ) ))
( "scalping"  v ( s k a1 l p i ng ) ((pos "Hb%" ) (subcat )))
( "scalps"  v ( s k a1 l p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "scaly"  j ( s k ei1 l ii ) ((pos "OD%" ) ))
( "scamp"  v ( s k a1 m p ) ((pos "H0$" "K6%" ) (subcat )))
( "scamped"  v ( s k a1 m p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "scamper"  v ( s k a1 m p @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "scampered"  v ( s k a1 m p @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "scampering"  v ( s k a1 m p @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "scampers"  v ( s k a1 m p @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "scampi"  n ( s k a1 m p ii ) ((pos "Kj%" ) ))
( "scamping"  v ( s k a1 m p i ng ) ((pos "Hb$" ) (subcat )))
( "scamps"  v ( s k a1 m p s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "scan"  v ( s k a1 n ) ((pos "J4%" ) (subcat "2A" )))
( "scandal"  n ( s k a1 n d l ) ((pos "M6%" ) ))
( "scandalize"  v ( s k a1 n d @ l ai z ) ((pos "H2%" ) (subcat )))
( "scandalized"  v ( s k a1 n d @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scandalizes"  v ( s k a1 n d @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "scandalizing"  v ( s k a1 n d @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "scandalmonger"  n ( s k a1 n d l m uh2 ng g @ r ) ((pos "K6%" ) ))
( "scandalmongering"  n ( s k a1 n d l m uh2 ng g @ r i ng ) ((pos "L@%" ) ))
( "scandalmongers"  n ( s k a1 n d l m uh2 ng g @ z ) ((pos "Kj%" ) ))
( "scandalous"  j ( s k a1 n d @ l @ s ) ((pos "OA%" ) ))
( "scandalously"  a ( s k a1 n d @ l @ s l ii ) ((pos "Pu%" ) ))
( "scandals"  n ( s k a1 n d l z ) ((pos "Mj%" ) ))
( "scanned"  v ( s k a1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "scanner"  n ( s k a1 n @ r ) ((pos "K6%" ) ))
( "scanners"  n ( s k a1 n @ z ) ((pos "Kj%" ) ))
( "scanning"  v ( s k a1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scans"  v ( s k a1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "scansion"  n ( s k a1 n sh @ n ) ((pos "L@%" ) ))
( "scant"  v ( s k a1 n t ) ((pos "H0$" "OA%" ) (subcat )))
( "scanted"  v ( s k a1 n t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "scantier"  j ( s k a1 n t i@ r ) ((pos "Or%" ) ))
( "scantiest"  j ( s k a1 n t i i s t ) ((pos "Os%" ) ))
( "scantily"  a ( s k a1 n t i l ii ) ((pos "Pu%" ) ))
( "scantiness"  n ( s k a1 n t i n @ s ) ((pos "L@%" ) ))
( "scanting"  v ( s k a1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "scantling"  n ( s k a1 n t l i ng ) ((pos "K6$" ) ))
( "scantlings"  n ( s k a1 n t l i ng z ) ((pos "Kj$" ) ))
( "scants"  v ( s k a1 n t s ) ((pos "Ha$" ) (subcat )))
( "scanty"  j ( s k a1 n t ii ) ((pos "OD%" ) ))
( "scapegoat"  n ( s k ei1 p g ou t ) ((pos "K6%" ) ))
( "scapegoats"  n ( s k ei1 p g ou t s ) ((pos "Kj%" ) ))
( "scapegrace"  n ( s k ei1 p g r ei s ) ((pos "K6%" ) ))
( "scapegraces"  n ( s k ei1 p g r ei s i z ) ((pos "Kj$" ) ))
( "scapula"  n ( s k a1 p y u l @ ) ((pos "K6%" ) ))
( "scapulas"  n ( s k a1 p y u l @ z ) ((pos "Kj%" ) ))
( "scar"  v ( s k aa1 r ) ((pos "J4%" "K6%" ) (subcat "2C" )))
( "scarab"  n ( s k a1 r @ b ) ((pos "K6$" ) ))
( "scarabs"  n ( s k a1 r @ b z ) ((pos "Kj$" ) ))
( "scarce"  j ( s k e@1 s ) ((pos "OB%" ) ))
( "scarcely"  a ( s k e@1 s l ii ) ((pos "Pu%" ) ))
( "scarcer"  j ( s k e@1 s @ r ) ((pos "Or%" ) ))
( "scarcest"  j ( s k e@1 s i s t ) ((pos "Os%" ) ))
( "scarcities"  n ( s k e@1 s i t i z ) ((pos "Mj%" ) ))
( "scarcity"  n ( s k e@1 s i t ii ) ((pos "M8%" ) ))
( "scare"  v ( s k e@1 r ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" "15A" )))
( "scarecrow"  n ( s k e@1 k r ou ) ((pos "K6%" ) ))
( "scarecrows"  n ( s k e@1 k r ou z ) ((pos "Kj%" ) ))
( "scared"  v ( s k e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "scaremonger"  n ( s k e@1 m uh ng g @ r ) ((pos "K6%" ) ))
( "scaremongers"  n ( s k e@1 m uh ng g @ z ) ((pos "Kj%" ) ))
( "scares"  v ( s k e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "scarf"  n ( s k aa1 f ) ((pos "K6%" ) ))
( "scarf-pin"  n ( s k aa1 f - p i n ) ((pos "K6%" ) ))
( "scarf-pins"  n ( s k aa1 f - p i n z ) ((pos "Kj%" ) ))
( "scarfs"  n ( s k aa1 f s ) ((pos "Kj$" ) ))
( "scarier"  j ( s k e@1 r i@ r ) ((pos "Or%" ) ))
( "scariest"  j ( s k e@1 r i i s t ) ((pos "Os%" ) ))
( "scarified"  v ( s k a1 r i f ai d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "scarifies"  v ( s k a1 r i f ai z ) ((pos "Ha$" ) (subcat )))
( "scarify"  v ( s k a1 r i f ai ) ((pos "H3$" ) (subcat )))
( "scarifying"  v ( s k a1 r i f ai i ng ) ((pos "Hb$" ) (subcat )))
( "scaring"  v ( s k e@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "scarlet"  n ( s k aa1 l @ t ) ((pos "M6%" "OA%" ) ))
( "scarlets"  n ( s k aa1 l @ t s ) ((pos "Mj$" ) ))
( "scarp"  n ( s k aa1 p ) ((pos "K6$" ) ))
( "scarper"  v ( s k aa1 p @ r ) ((pos "I0%" ) (subcat )))
( "scarpered"  v ( s k aa1 p @ d ) ((pos "Ic%" ) (subcat )))
( "scarpering"  v ( s k aa1 p @ r i ng ) ((pos "Ib%" ) (subcat )))
( "scarpers"  v ( s k aa1 p @ z ) ((pos "Ia%" ) (subcat )))
( "scarps"  n ( s k aa1 p s ) ((pos "Kj$" ) ))
( "scarred"  v ( s k aa1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "scarring"  v ( s k aa1 r i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "scars"  v ( s k aa1 z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "scarves"  n ( s k aa1 v z ) ((pos "Kj%" ) ))
( "scary"  j ( s k e@1 r ii ) ((pos "OD%" ) ))
( "scat"  uh ( s k a1 t ) ((pos "W-$" ) ))
( "scathing"  j ( s k ei1 dh i ng ) ((pos "OA%" ) ))
( "scathingly"  a ( s k ei1 dh i ng l ii ) ((pos "Pu%" ) ))
( "scatter"  v ( s k a1 t @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scatterbrain"  n ( s k a1 t @ b r ei n ) ((pos "K6%" ) ))
( "scatterbrained"  j ( s k a1 t @ b r ei n d ) ((pos "OA%" ) ))
( "scatterbrains"  n ( s k a1 t @ b r ei n z ) ((pos "Kj%" ) ))
( "scattered"  v ( s k a1 t @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scattering"  v ( s k a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scatters"  v ( s k a1 t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scattier"  j ( s k a1 t i@ r ) ((pos "Or%" ) ))
( "scattiest"  j ( s k a1 t i i s t ) ((pos "Os%" ) ))
( "scatty"  j ( s k a1 t ii ) ((pos "OD%" ) ))
( "scavenge"  v ( s k a1 v i n jh ) ((pos "I2%" ) (subcat "2A" )))
( "scavenged"  v ( s k a1 v i n jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "scavenger"  n ( s k a1 v i n jh @ r ) ((pos "K6%" ) ))
( "scavengers"  n ( s k a1 v i n jh @ z ) ((pos "Kj%" ) ))
( "scavenges"  v ( s k a1 v i n jh i z ) ((pos "Ia%" ) (subcat "2A" )))
( "scavenging"  v ( s k a1 v i n jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "scenario"  n ( s i1 n aa1 r i@ @ ) ((pos "K6%" ) ))
( "scenarios"  n ( s i1 n aa1 r i@ u z ) ((pos "Kj%" ) ))
( "scenarist"  n ( s i1 n aa1 r i s t ) ((pos "K6$" ) ))
( "scenarists"  n ( s i1 n aa1 r i s t s ) ((pos "Kj$" ) ))
( "scene"  n ( s ii1 n ) ((pos "K6%" ) ))
( "scene-painter"  n ( s ii1 n - p ei n t @ r ) ((pos "K6%" ) ))
( "scene-painters"  n ( s ii1 n - p ei n t @ z ) ((pos "Kj%" ) ))
( "scene-shifter"  n ( s ii1 n - sh i f t @ r ) ((pos "K6%" ) ))
( "scene-shifters"  n ( s ii1 n - sh i f t @ z ) ((pos "Kj%" ) ))
( "scenery"  n ( s ii1 n @ r ii ) ((pos "L@%" ) ))
( "scenes"  n ( s ii1 n z ) ((pos "Kj%" ) ))
( "scenic"  j ( s ii1 n i k ) ((pos "OA%" ) ))
( "scenically"  a ( s ii1 n i k l ii ) ((pos "Pu%" ) ))
( "scent"  v ( s e1 n t ) ((pos "H0%" "M6%" ) (subcat )))
( "scented"  v ( s e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scenting"  v ( s e1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "scentless"  j ( s e1 n t l @ s ) ((pos "OA$" ) ))
( "scents"  v ( s e1 n t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sceptered"  j ( s e1 p t @ d ) ((pos "OA$" ) ))
( "sceptic"  n ( s k e1 p t i k ) ((pos "K6%" ) ))
( "sceptical"  j ( s k e1 p t i k l ) ((pos "OA%" ) ))
( "sceptically"  a ( s k e1 p t i k l ii ) ((pos "Pu%" ) ))
( "scepticism"  n ( s k e1 p t i s i z @ m ) ((pos "L@%" ) ))
( "sceptics"  n ( s k e1 p t i k s ) ((pos "Kj%" ) ))
( "sceptre"  n ( s e1 p t @ r ) ((pos "K6%" ) ))
( "sceptred"  j ( s e1 p t @ d ) ((pos "OA%" ) ))
( "sceptres"  n ( s e1 p t @ z ) ((pos "Kj%" ) ))
( "schedule"  v ( sh e1 d y uu l ) ((pos "H2%" "K6%" ) (subcat "6A" "7A" )))
( "scheduled"  v ( sh e1 d y uu l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "schedules"  v ( sh e1 d y uu l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "7A" )))
( "scheduling"  v ( sh e1 d y uu l i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "schema"  n ( s k ii1 m @ ) ((pos "K6$" ) ))
( "schemas"  n ( s k ii1 m @ z ) ((pos "Kj$" ) ))
( "schemata"  n ( s k ii1 m @ t @ ) ((pos "Kj$" ) ))
( "schematic"  j ( s k i1 m a1 t i k ) ((pos "OA%" ) ))
( "schematically"  a ( s k i1 m a1 t i k l ii ) ((pos "Pu%" ) ))
( "scheme"  v ( s k ii1 m ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "4A" )))
( "schemed"  v ( s k ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4A" )))
( "schemer"  n ( s k ii1 m @ r ) ((pos "K6%" ) ))
( "schemers"  n ( s k ii1 m @ z ) ((pos "Kj%" ) ))
( "schemes"  v ( s k ii1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "4A" )))
( "scheming"  v ( s k ii1 m i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" )))
( "scherzo"  n ( s k e@1 t s ou ) ((pos "K6$" ) ))
( "scherzos"  n ( s k e@1 t s ou z ) ((pos "Kj$" ) ))
( "schism"  n ( s k i1 z @ m ) ((pos "M6%" ) ))
( "schismatic"  j ( s k i1 z m a1 t i k ) ((pos "OA$" ) ))
( "schisms"  n ( s k i1 z @ m z ) ((pos "Mj$" ) ))
( "schist"  n ( sh i1 s t ) ((pos "K6$" ) ))
( "schists"  n ( sh i1 s t s ) ((pos "Kj$" ) ))
( "schizoid"  j ( s k i1 t s oi d ) ((pos "OA%" ) ))
( "schizophrenia"  n ( s k i2 t s ou f r ii1 n i@ ) ((pos "L@%" ) ))
( "schizophrenic"  n ( s k i2 t s ou f r e1 n i k ) ((pos "K6%" "OA%" ) ))
( "schizophrenics"  n ( s k i2 t s ou f r e1 n i k s ) ((pos "Kj%" ) ))
( "schmaltz"  n ( sh m oo1 l t s ) ((pos "L@%" ) ))
( "schmaltzier"  j ( sh m oo1 l t s i@ r ) ((pos "Or%" ) ))
( "schmaltziest"  j ( sh m oo1 l t s i i s t ) ((pos "Os%" ) ))
( "schmaltzy"  j ( sh m oo1 l t s ii ) ((pos "OD%" ) ))
( "schmalz"  n ( sh m oo1 l t s ) ((pos "L@$" ) ))
( "schmalzier"  j ( s m oo1 l t s i@ r ) ((pos "Or$" ) ))
( "schmalziest"  j ( s m oo1 l t s i i s t ) ((pos "Os$" ) ))
( "schmalzy"  j ( s m oo1 l t s ii ) ((pos "OD$" ) ))
( "schnapps"  n ( sh @ n a1 p s ) ((pos "L@%" ) ))
( "schnitzel"  n ( sh @ n i1 t s l ) ((pos "M6%" ) ))
( "schnitzels"  n ( sh @ n i1 t s l z ) ((pos "Mj$" ) ))
( "schnorkel"  n ( sh @ n oo1 k l ) ((pos "K6$" ) ))
( "schnorkels"  n ( sh @ n oo1 k l z ) ((pos "Kj$" ) ))
( "scholar"  n ( s k o1 l @ r ) ((pos "K6%" ) ))
( "scholarly"  j ( s k o1 l @ l ii ) ((pos "OA%" ) ))
( "scholars"  n ( s k o1 l @ z ) ((pos "Kj%" ) ))
( "scholarship"  n ( s k o1 l @ sh i p ) ((pos "M6%" ) ))
( "scholarships"  n ( s k o1 l @ sh i p s ) ((pos "Mj%" ) ))
( "scholastic"  j ( s k @1 l a1 s t i k ) ((pos "OA%" ) ))
( "scholasticism"  n ( s k @1 l a1 s t i s i z @ m ) ((pos "L@%" ) ))
( "school"  v ( s k uu1 l ) ((pos "H0%" "M6*" ) (subcat "6A" )))
( "school-board"  n ( s k uu1 l - b oo d ) ((pos "K6%" ) ))
( "school-boards"  n ( s k uu1 l - b oo d z ) ((pos "Kj%" ) ))
( "schoolbook"  n ( s k uu1 l b u k ) ((pos "K6%" ) ))
( "schoolbooks"  n ( s k uu1 l b u k s ) ((pos "Kj%" ) ))
( "schoolboy"  n ( s k uu1 l b oi ) ((pos "K6%" ) ))
( "schoolboys"  n ( s k uu1 l b oi z ) ((pos "Kj%" ) ))
( "schoolchild"  n ( s k uu1 l ch ai l d ) ((pos "Ki%" ) ))
( "schoolchildren"  n ( s k uu1 l ch i l d r @ n ) ((pos "Kj%" ) ))
( "schooldays"  n ( s k uu1 l d ei z ) ((pos "Kj%" ) ))
( "schooled"  v ( s k uu1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "schoolfellow"  n ( s k uu1 l f e l ou ) ((pos "K6%" ) ))
( "schoolfellows"  n ( s k uu1 l f e l ou z ) ((pos "Kj%" ) ))
( "schoolfriend"  n ( s k uu1 l f r e n d ) ((pos "K6%" ) ))
( "schoolfriends"  n ( s k uu1 l f r e n d z ) ((pos "Kj%" ) ))
( "schoolgirl"  n ( s k uu1 l g @@ l ) ((pos "K6%" ) ))
( "schoolgirls"  n ( s k uu1 l g @@ l z ) ((pos "Kj%" ) ))
( "schoolhouse"  n ( s k uu1 l h au s ) ((pos "K6%" ) ))
( "schoolhouses"  n ( s k uu1 l h au z i z ) ((pos "Kj%" ) ))
( "schooling"  v ( s k uu1 l i ng ) ((pos "Hb%" "L@%" ) (subcat "6A" )))
( "schoolman"  n ( s k uu1 l m a n ) ((pos "Ki$" ) ))
( "schoolmaster"  n ( s k uu1 l m aa s t @ r ) ((pos "K6%" ) ))
( "schoolmasters"  n ( s k uu1 l m aa s t @ z ) ((pos "Kj%" ) ))
( "schoolmate"  n ( s k uu1 l m ei t ) ((pos "K6%" ) ))
( "schoolmates"  n ( s k uu1 l m ei t s ) ((pos "Kj%" ) ))
( "schoolmen"  n ( s k uu1 l m e n ) ((pos "Kj$" ) ))
( "schoolmistress"  n ( s k uu1 l m i s t r i s ) ((pos "K7%" ) ))
( "schoolmistresses"  n ( s k uu1 l m i s t r i s i z ) ((pos "Kj%" ) ))
( "schoolroom"  n ( s k uu1 l r uu m ) ((pos "K6%" ) ))
( "schoolrooms"  n ( s k uu1 l r uu m z ) ((pos "Kj%" ) ))
( "schools"  v ( s k uu1 l z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "schoolteacher"  n ( s k uu1 l t ii ch @ r ) ((pos "K6%" ) ))
( "schoolteachers"  n ( s k uu1 l t ii ch @ z ) ((pos "Kj%" ) ))
( "schooltime"  n ( s k uu1 l t ai m ) ((pos "M6%" ) ))
( "schooltimes"  n ( s k uu1 l t ai m z ) ((pos "Mj%" ) ))
( "schooner"  n ( s k uu1 n @ r ) ((pos "K6%" ) ))
( "schooners"  n ( s k uu1 n @ z ) ((pos "Kj%" ) ))
( "schottische"  n ( sh o1 t ii1 sh ) ((pos "K6$" ) ))
( "schottisches"  n ( sh o1 t ii1 sh i z ) ((pos "Kj$" ) ))
( "schwa"  n ( sh w aa1 ) ((pos "K6$" ) ))
( "schwas"  n ( sh w aa1 z ) ((pos "Kj$" ) ))
( "sciatic"  j ( s ai1 a1 t i k ) ((pos "OA$" ) ))
( "sciatica"  n ( s ai1 a1 t i k @ ) ((pos "L@%" ) ))
( "science"  n ( s ai1 @ n s ) ((pos "M6%" ) ))
( "sciences"  n ( s ai1 @ n s i z ) ((pos "Mj%" ) ))
( "scientific"  j ( s ai2 @ n t i1 f i k ) ((pos "OA%" ) ))
( "scientifically"  a ( s ai2 @ n t i1 f i k l ii ) ((pos "Pu%" ) ))
( "scientist"  n ( s ai1 @ n t i s t ) ((pos "K6%" ) ))
( "scientists"  n ( s ai1 @ n t i s t s ) ((pos "Kj%" ) ))
( "scimitar"  n ( s i1 m i t @ r ) ((pos "K6$" ) ))
( "scimitars"  n ( s i1 m i t @ z ) ((pos "Kj$" ) ))
( "scintilla"  n ( s i1 n t i1 l @ ) ((pos "K6$" ) ))
( "scintillas"  n ( s i1 n t i1 l @ z ) ((pos "Kj$" ) ))
( "scintillate"  v ( s i1 n t i l ei t ) ((pos "I2%" ) (subcat )))
( "scintillated"  v ( s i1 n t i l ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "scintillates"  v ( s i1 n t i l ei t s ) ((pos "Ia%" ) (subcat )))
( "scintillating"  v ( s i1 n t i l ei t i ng ) ((pos "Ib%" ) (subcat )))
( "scintillation"  n ( s i2 n t i l ei1 sh @ n ) ((pos "K6%" ) ))
( "scintillations"  n ( s i2 n t i l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "scion"  n ( s ai1 @ n ) ((pos "K6$" ) ))
( "scions"  n ( s ai1 @ n z ) ((pos "Kj$" ) ))
( "scissors"  n ( s i1 z @ z ) ((pos "Kj%" ) ))
( "sclerosis"  n ( s k l @1 r ou1 s i s ) ((pos "L@%" ) ))
( "scoff"  v ( s k o1 f ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "scoffed"  v ( s k o1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "scoffer"  n ( s k o1 f @ r ) ((pos "K6%" ) ))
( "scoffers"  n ( s k o1 f @ z ) ((pos "Kj%" ) ))
( "scoffing"  v ( s k o1 f i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scoffingly"  a ( s k o1 f i ng l ii ) ((pos "Pu%" ) ))
( "scoffs"  v ( s k o1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "scold"  v ( s k ou1 l d ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "scolded"  v ( s k ou1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "scolding"  v ( s k ou1 l d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "6A" )))
( "scoldings"  n ( s k ou1 l d i ng z ) ((pos "Kj%" ) ))
( "scolds"  v ( s k ou1 l d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "scollop"  v ( s k o1 l @ p ) ((pos "H0$" "K6$" ) ))
( "scolloped"  v ( s k o1 l @ p t ) ((pos "Hc$" "Hd$" ) ))
( "scolloping"  v ( s k o1 l @ p i ng ) ((pos "Hb$" ) ))
( "scollops"  v ( s k o1 l @ p s ) ((pos "Ha$" "Kj$" ) ))
( "sconce"  n ( s k o1 n s ) ((pos "K6$" ) ))
( "sconces"  n ( s k o1 n s i z ) ((pos "Kj$" ) ))
( "scone"  n ( s k o1 n ) ((pos "K6%" ) ))
( "scones"  n ( s k o1 n z ) ((pos "Kj%" ) ))
( "scoop"  v ( s k uu1 p ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "scooped"  v ( s k uu1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "scoopful"  n ( s k uu1 p f u l ) ((pos "K6%" ) ))
( "scoopfuls"  n ( s k uu1 p f u l z ) ((pos "Kj%" ) ))
( "scooping"  v ( s k uu1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "scoops"  v ( s k uu1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "scoot"  v ( s k uu1 t ) ((pos "I0%" ) ))
( "scooted"  v ( s k uu1 t i d ) ((pos "Ic%" "Id%" ) ))
( "scooter"  n ( s k uu1 t @ r ) ((pos "K6%" ) ))
( "scooters"  n ( s k uu1 t @ z ) ((pos "Kj%" ) ))
( "scooting"  v ( s k uu1 t i ng ) ((pos "Ib%" ) ))
( "scoots"  v ( s k uu1 t s ) ((pos "Ia%" ) ))
( "scope"  n ( s k ou1 p ) ((pos "L@%" ) ))
( "scorbutic"  j ( s k oo1 b y uu1 t i k ) ((pos "OA$" ) ))
( "scorch"  v ( s k oo1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" )))
( "scorched"  v ( s k oo1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "scorcher"  n ( s k oo1 ch @ r ) ((pos "K6%" ) ))
( "scorchers"  n ( s k oo1 ch @ z ) ((pos "Kj%" ) ))
( "scorches"  v ( s k oo1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "scorching"  v ( s k oo1 ch i ng ) ((pos "Jb%" "OA%" "Pu%" ) (subcat "2A" "2C" )))
( "score"  v ( s k oo1 r ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "15A" )))
( "scoreboard"  n ( s k oo1 b oo d ) ((pos "K6%" ) ))
( "scoreboards"  n ( s k oo1 b oo d z ) ((pos "Kj%" ) ))
( "scorebook"  n ( s k oo1 b u k ) ((pos "K6%" ) ))
( "scorebooks"  n ( s k oo1 b u k s ) ((pos "Kj%" ) ))
( "scorecard"  n ( s k oo1 k aa d ) ((pos "K6%" ) ))
( "scorecards"  n ( s k oo1 k aa d z ) ((pos "Kj%" ) ))
( "scored"  v ( s k oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "15A" )))
( "scorer"  n ( s k oo1 r @ r ) ((pos "K6%" ) ))
( "scorers"  n ( s k oo1 r @ z ) ((pos "Kj%" ) ))
( "scores"  v ( s k oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "15A" )))
( "scoring"  v ( s k oo1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "15A" )))
( "scorn"  v ( s k oo1 n ) ((pos "H0%" "L@%" ) (subcat "6A" "6D" )))
( "scorned"  v ( s k oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" )))
( "scornful"  j ( s k oo1 n f @ l ) ((pos "OA%" ) ))
( "scornfully"  a ( s k oo1 n f @ l ii ) ((pos "Pu%" ) ))
( "scorning"  v ( s k oo1 n i ng ) ((pos "Hb%" ) (subcat "6A" "6D" )))
( "scorns"  v ( s k oo1 n z ) ((pos "Ha%" ) (subcat "6A" "6D" )))
( "scorpion"  n ( s k oo1 p i@ n ) ((pos "K6%" ) ))
( "scorpions"  n ( s k oo1 p i@ n z ) ((pos "Kj%" ) ))
( "scot"  n ( s k o1 t ) ((pos "K6$" ) ))
( "scot-free"  a ( s k o2 t - f r ii1 ) ((pos "Pu%" ) ))
( "scotch"  v ( s k o1 ch ) ((pos "H1%" "M7%" ) (subcat )))
( "scotched"  v ( s k o1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scotches"  v ( s k o1 ch i z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "scotching"  v ( s k o1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "scots"  n ( s k o1 t s ) ((pos "Kj$" ) ))
( "scoundrel"  n ( s k au1 n d r @ l ) ((pos "K6%" ) ))
( "scoundrelly"  j ( s k au1 n d r @ l ii ) ((pos "OA$" ) ))
( "scoundrels"  n ( s k au1 n d r @ l z ) ((pos "Kj%" ) ))
( "scour"  v ( s k au1 @ r ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "scoured"  v ( s k au1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "scourer"  n ( s k au1 @ r @ r ) ((pos "K6%" ) ))
( "scourers"  n ( s k au1 @ r @ z ) ((pos "Kj%" ) ))
( "scourge"  v ( s k @@1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "scourged"  v ( s k @@1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scourges"  v ( s k @@1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "scourging"  v ( s k @@1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "scouring"  v ( s k au1 @ r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "scours"  v ( s k au1 @ z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "scout"  v ( s k au1 t ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "scouted"  v ( s k au1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "scouting"  v ( s k au1 t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "scoutmaster"  n ( s k au1 t m aa s t @ r ) ((pos "K6%" ) ))
( "scoutmasters"  n ( s k au1 t m aa s t @ z ) ((pos "Kj%" ) ))
( "scouts"  v ( s k au1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "scow"  n ( s k au1 ) ((pos "K6$" ) ))
( "scowl"  v ( s k au1 l ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "scowled"  v ( s k au1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "scowling"  v ( s k au1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "scowls"  v ( s k au1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "scows"  n ( s k au1 z ) ((pos "Kj$" ) ))
( "scrabble"  v ( s k r a1 b l ) ((pos "I2%" "M6%" ) (subcat "2A" )))
( "scrabbled"  v ( s k r a1 b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "scrabbles"  v ( s k r a1 b l z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "scrabbling"  v ( s k r a1 b l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "scrag"  v ( s k r a1 g ) ((pos "H4$" "K6$" ) ))
( "scrag-end"  n ( s k r a1 g - e1 n d ) ((pos "K6%" ) ))
( "scrag-ends"  n ( s k r a1 g - e1 n d z ) ((pos "Kj%" ) ))
( "scragged"  v ( s k r a1 g d ) ((pos "Hc$" "Hd$" ) ))
( "scraggier"  j ( s k r a1 g i@ r ) ((pos "Or$" ) ))
( "scraggiest"  j ( s k r a1 g i i s t ) ((pos "Os$" ) ))
( "scragging"  v ( s k r a1 g i ng ) ((pos "Hb$" ) ))
( "scraggy"  j ( s k r a1 g ii ) ((pos "OD$" ) ))
( "scrags"  v ( s k r a1 g z ) ((pos "Ha$" "Kj$" ) ))
( "scram"  v ( s k r a1 m ) ((pos "I4%" ) ))
( "scramble"  v ( s k r a1 m b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "4A" )))
( "scrambled"  v ( s k r a1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" )))
( "scrambler"  n ( s k r a1 m b l @ r ) ((pos "K6%" ) ))
( "scramblers"  n ( s k r a1 m b l @ z ) ((pos "Kj%" ) ))
( "scrambles"  v ( s k r a1 m b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" )))
( "scrambling"  v ( s k r a1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" )))
( "scrammed"  v ( s k r a1 m d ) ((pos "Ic$" "Id$" ) ))
( "scramming"  v ( s k r a1 m i ng ) ((pos "Ib$" ) ))
( "scrams"  v ( s k r a1 m z ) ((pos "Ia$" ) ))
( "scrap"  v ( s k r a1 p ) ((pos "H4%" "M6%" ) (subcat )))
( "scrap-iron"  n ( s k r a1 p - ai1 @ n ) ((pos "L@%" ) ))
( "scrapbook"  n ( s k r a1 p b u k ) ((pos "K6%" ) ))
( "scrapbooks"  n ( s k r a1 p b u k s ) ((pos "Kj%" ) ))
( "scrape"  v ( s k r ei1 p ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" )))
( "scraped"  v ( s k r ei1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" )))
( "scraper"  n ( s k r ei1 p @ r ) ((pos "K6%" ) ))
( "scrapers"  n ( s k r ei1 p @ z ) ((pos "Kj%" ) ))
( "scrapes"  v ( s k r ei1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" )))
( "scrapheap"  n ( s k r a1 p h ii p ) ((pos "K6%" ) ))
( "scrapheaps"  n ( s k r a1 p h ii p s ) ((pos "Kj%" ) ))
( "scraping"  v ( s k r ei1 p i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" )))
( "scrapings"  n ( s k r ei1 p i ng z ) ((pos "Kj%" ) ))
( "scrapped"  v ( s k r a1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scrappier"  j ( s k r a1 p i@ r ) ((pos "Or%" ) ))
( "scrappiest"  j ( s k r a1 p i i s t ) ((pos "Os%" ) ))
( "scrappily"  a ( s k r a1 p i l ii ) ((pos "Pu%" ) ))
( "scrappiness"  n ( s k r a1 p i n @ s ) ((pos "L@$" ) ))
( "scrapping"  v ( s k r a1 p i ng ) ((pos "Hb%" ) (subcat )))
( "scrappy"  j ( s k r a1 p ii ) ((pos "OD%" ) ))
( "scraps"  v ( s k r a1 p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "scratch"  v ( s k r a1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" )))
( "scratch-pad"  n ( s k r a1 ch - p a d ) ((pos "K6%" ) ))
( "scratch-pads"  n ( s k r a1 ch - p a d z ) ((pos "Kj%" ) ))
( "scratch-race"  n ( s k r a1 ch - r ei s ) ((pos "K6%" ) ))
( "scratch-races"  n ( s k r a1 ch - r ei s i z ) ((pos "Kj%" ) ))
( "scratched"  v ( s k r a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "scratches"  v ( s k r a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "scratchier"  j ( s k r a1 ch i@ r ) ((pos "Or%" ) ))
( "scratchiest"  j ( s k r a1 ch i i s t ) ((pos "Os%" ) ))
( "scratching"  v ( s k r a1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "scratchy"  j ( s k r a1 ch ii ) ((pos "OD%" ) ))
( "scrawl"  v ( s k r oo1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "scrawled"  v ( s k r oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "scrawling"  v ( s k r oo1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "scrawls"  v ( s k r oo1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "scrawnier"  j ( s k r oo1 n i@ r ) ((pos "Or%" ) ))
( "scrawniest"  j ( s k r oo1 n i i s t ) ((pos "Os%" ) ))
( "scrawny"  j ( s k r oo1 n ii ) ((pos "OD%" ) ))
( "scream"  v ( s k r ii1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "screamed"  v ( s k r ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "screaming"  v ( s k r ii1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "screamingly"  a ( s k r ii1 m i ng l ii ) ((pos "Pu%" ) ))
( "screams"  v ( s k r ii1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "scree"  n ( s k r ii1 ) ((pos "M6$" ) ))
( "screech"  v ( s k r ii1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "15A" )))
( "screech-owl"  n ( s k r ii1 ch - au l ) ((pos "K6%" ) ))
( "screech-owls"  n ( s k r ii1 ch - au l z ) ((pos "Kj%" ) ))
( "screeched"  v ( s k r ii1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "screeches"  v ( s k r ii1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "screeching"  v ( s k r ii1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "screed"  n ( s k r ii1 d ) ((pos "K6$" ) ))
( "screeds"  n ( s k r ii1 d z ) ((pos "Kj$" ) ))
( "screen"  v ( s k r ii1 n ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "14" "15A" )))
( "screened"  v ( s k r ii1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "screening"  v ( s k r ii1 n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" "15A" )))
( "screens"  v ( s k r ii1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "14" "15A" )))
( "screes"  n ( s k r ii1 z ) ((pos "Mj$" ) ))
( "screw"  v ( s k r uu1 ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" "15A" )))
( "screw-topped"  j ( s k r uu1 - t o p t ) ((pos "OA%" ) ))
( "screwball"  n ( s k r uu1 b oo l ) ((pos "K6%" "OA%" ) ))
( "screwballs"  n ( s k r uu1 b oo l z ) ((pos "Kj%" ) ))
( "screwdriver"  n ( s k r uu1 d r ai v @ r ) ((pos "K6%" ) ))
( "screwdrivers"  n ( s k r uu1 d r ai v @ z ) ((pos "Kj%" ) ))
( "screwed"  v ( s k r uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "screwier"  j ( s k r uu1 i@ r ) ((pos "Or%" ) ))
( "screwiest"  j ( s k r uu1 i i s t ) ((pos "Os%" ) ))
( "screwing"  v ( s k r uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "screws"  v ( s k r uu1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "15A" )))
( "screwy"  j ( s k r uu1 ii ) ((pos "OD%" ) ))
( "scribble"  v ( s k r i1 b l ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "scribbled"  v ( s k r i1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "scribbler"  n ( s k r i1 b l @ r ) ((pos "K6%" ) ))
( "scribblers"  n ( s k r i1 b l @ z ) ((pos "Kj%" ) ))
( "scribbles"  v ( s k r i1 b l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "scribbling"  v ( s k r i1 b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scribbling-block"  n ( s k r i1 b l i ng - b l o k ) ((pos "K6%" ) ))
( "scribbling-blocks"  n ( s k r i1 b l i ng - b l o k s ) ((pos "Kj%" ) ))
( "scribe"  n ( s k r ai1 b ) ((pos "K6%" ) ))
( "scribes"  n ( s k r ai1 b z ) ((pos "Kj%" ) ))
( "scrimmage"  v ( s k r i1 m i jh ) ((pos "J2$" "K6%" ) ))
( "scrimmaged"  v ( s k r i1 m i jh d ) ((pos "Jc$" "Jd$" ) ))
( "scrimmages"  v ( s k r i1 m i jh i z ) ((pos "Ja$" "Kj$" ) ))
( "scrimmaging"  v ( s k r i1 m i jh i ng ) ((pos "Jb$" ) ))
( "scrimp"  v ( s k r i1 m p ) ((pos "J0%" ) ))
( "scrimped"  v ( s k r i1 m p t ) ((pos "Jc%" "Jd%" ) ))
( "scrimping"  v ( s k r i1 m p i ng ) ((pos "Jb%" ) ))
( "scrimps"  v ( s k r i1 m p s ) ((pos "Ja%" ) ))
( "scrimshank"  v ( s k r i1 m sh a ng k ) ((pos "I0$" ) ))
( "scrimshanked"  v ( s k r i1 m sh a ng k t ) ((pos "Ic$" "Id$" ) ))
( "scrimshanker"  n ( s k r i1 m sh a ng k @ r ) ((pos "K6$" ) ))
( "scrimshankers"  n ( s k r i1 m sh a ng k @ z ) ((pos "Kj$" ) ))
( "scrimshanking"  v ( s k r i1 m sh a ng k i ng ) ((pos "Ib$" ) ))
( "scrimshanks"  v ( s k r i1 m sh a ng k s ) ((pos "Ia$" ) ))
( "scrip"  n ( s k r i1 p ) ((pos "M6$" ) ))
( "scrips"  n ( s k r i1 p s ) ((pos "Mj$" ) ))
( "script"  n ( s k r i1 p t ) ((pos "M6%" ) ))
( "scripted"  j ( s k r i1 p t i d ) ((pos "OA%" ) ))
( "scripts"  n ( s k r i1 p t s ) ((pos "Mj%" ) ))
( "scriptural"  j ( s k r i1 p ch @ r @ l ) ((pos "OA%" ) ))
( "scripture"  n ( s k r i1 p ch @ r ) ((pos "K6%" ) ))
( "scriptures"  n ( s k r i1 p ch @ z ) ((pos "Kj%" ) ))
( "scriptwriter"  n ( s k r i1 p t r ai t @ r ) ((pos "K6%" ) ))
( "scriptwriters"  n ( s k r i1 p t r ai t @ z ) ((pos "Kj%" ) ))
( "scrivener"  n ( s k r i1 v @ n @ r ) ((pos "K6$" ) ))
( "scriveners"  n ( s k r i1 v @ n @ z ) ((pos "Kj$" ) ))
( "scrofula"  n ( s k r o1 f y u l @ ) ((pos "L@$" ) ))
( "scrofulous"  j ( s k r o1 f y u l @ s ) ((pos "OA$" ) ))
( "scroll"  n ( s k r ou1 l ) ((pos "K6%" ) ))
( "scrolls"  n ( s k r ou1 l z ) ((pos "Kj%" ) ))
( "scrotum"  n ( s k r ou1 t @ m ) ((pos "K6%" ) ))
( "scrotums"  n ( s k r ou1 t @ m z ) ((pos "Kj$" ) ))
( "scrounge"  v ( s k r au1 n jh ) ((pos "J2%" ) (subcat "2A" )))
( "scrounged"  v ( s k r au1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "scrounger"  n ( s k r au1 n jh @ r ) ((pos "K6%" ) ))
( "scroungers"  n ( s k r au1 n jh @ z ) ((pos "Kj%" ) ))
( "scrounges"  v ( s k r au1 n jh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "scrounging"  v ( s k r au1 n jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scrub"  v ( s k r uh1 b ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "6A" "15B" )))
( "scrubbed"  v ( s k r uh1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15B" )))
( "scrubbier"  j ( s k r uh1 b i@ r ) ((pos "Or$" ) ))
( "scrubbiest"  j ( s k r uh1 b i i s t ) ((pos "Os$" ) ))
( "scrubbing"  v ( s k r uh1 b i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15B" )))
( "scrubbing-brush"  n ( s k r uh1 b i ng - b r uh sh ) ((pos "K7%" ) ))
( "scrubbing-brushes"  n ( s k r uh1 b i ng - b r uh sh i z ) ((pos "Kj%" ) ))
( "scrubby"  j ( s k r uh1 b ii ) ((pos "OD%" ) ))
( "scrubs"  v ( s k r uh1 b z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15B" )))
( "scruff"  n ( s k r uh1 f ) ((pos "K6%" ) ))
( "scruffier"  j ( s k r uh1 f i@ r ) ((pos "Or%" ) ))
( "scruffiest"  j ( s k r uh1 f i i s t ) ((pos "Os%" ) ))
( "scruffs"  n ( s k r uh1 f s ) ((pos "Kj%" ) ))
( "scruffy"  j ( s k r uh1 f ii ) ((pos "OD%" ) ))
( "scrum"  n ( s k r uh1 m ) ((pos "K6%" ) ))
( "scrummage"  n ( s k r uh1 m i jh ) ((pos "K6%" ) ))
( "scrummages"  n ( s k r uh1 m i jh i z ) ((pos "Kj%" ) ))
( "scrumptious"  j ( s k r uh1 m p sh @ s ) ((pos "OA%" ) ))
( "scrums"  n ( s k r uh1 m z ) ((pos "Kj%" ) ))
( "scrunch"  v ( s k r uh1 n ch ) ((pos "H1$" "K7$" ) ))
( "scrunched"  v ( s k r uh1 n ch t ) ((pos "Hc$" "Hd$" ) ))
( "scrunches"  v ( s k r uh1 n ch i z ) ((pos "Ha$" "Kj$" ) ))
( "scrunching"  v ( s k r uh1 n ch i ng ) ((pos "Hb$" ) ))
( "scruple"  v ( s k r uu1 p l ) ((pos "I2%" "M6%" ) (subcat )))
( "scrupled"  v ( s k r uu1 p l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "scruples"  v ( s k r uu1 p l z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "scrupling"  v ( s k r uu1 p l i ng ) ((pos "Ib%" ) (subcat )))
( "scrupulous"  j ( s k r uu1 p y u l @ s ) ((pos "OA%" ) ))
( "scrupulously"  a ( s k r uu1 p y u l @ s l ii ) ((pos "Pu%" ) ))
( "scrutineer"  n ( s k r uu2 t i n i@1 r ) ((pos "K6%" ) ))
( "scrutineers"  n ( s k r uu2 t i n i@1 z ) ((pos "Kj%" ) ))
( "scrutinies"  n ( s k r uu1 t i n i z ) ((pos "Mj%" ) ))
( "scrutinize"  v ( s k r uu1 t i n ai z ) ((pos "H2%" ) (subcat )))
( "scrutinized"  v ( s k r uu1 t i n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "scrutinizes"  v ( s k r uu1 t i n ai z i z ) ((pos "Ha%" ) (subcat )))
( "scrutinizing"  v ( s k r uu1 t i n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "scrutiny"  n ( s k r uu1 t i n ii ) ((pos "M8%" ) ))
( "scud"  v ( s k uh1 d ) ((pos "I4$" "M6$" ) (subcat "2A" )))
( "scudded"  v ( s k uh1 d i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "scudding"  v ( s k uh1 d i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "scuds"  v ( s k uh1 d z ) ((pos "Ia$" "Mj$" ) (subcat "2A" )))
( "scuff"  v ( s k uh1 f ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scuffed"  v ( s k uh1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scuffing"  v ( s k uh1 f i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scuffle"  v ( s k uh1 f l ) ((pos "I2%" ) (subcat "2A" )))
( "scuffled"  v ( s k uh1 f l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "scuffles"  v ( s k uh1 f l z ) ((pos "Ia%" ) (subcat "2A" )))
( "scuffling"  v ( s k uh1 f l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "scuffs"  v ( s k uh1 f s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "scull"  v ( s k uh1 l ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "sculled"  v ( s k uh1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sculler"  n ( s k uh1 l @ r ) ((pos "K6%" ) ))
( "sculleries"  n ( s k uh1 l @ r i z ) ((pos "Kj%" ) ))
( "scullers"  n ( s k uh1 l @ z ) ((pos "Kj%" ) ))
( "scullery"  n ( s k uh1 l @ r ii ) ((pos "K8%" ) ))
( "sculling"  v ( s k uh1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scullion"  n ( s k uh1 l i@ n ) ((pos "K6$" ) ))
( "scullions"  n ( s k uh1 l i@ n z ) ((pos "Kj$" ) ))
( "sculls"  v ( s k uh1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "sculpt"  v ( s k uh1 l p t ) ((pos "J0%" ) ))
( "sculpted"  v ( s k uh1 l p t i d ) ((pos "Jc%" "Jd%" ) ))
( "sculpting"  v ( s k uh1 l p t i ng ) ((pos "Jb%" ) ))
( "sculptor"  n ( s k uh1 l p t @ r ) ((pos "K6%" ) ))
( "sculptors"  n ( s k uh1 l p t @ z ) ((pos "Kj%" ) ))
( "sculptress"  n ( s k uh1 l p t r i s ) ((pos "K7%" ) ))
( "sculptresses"  n ( s k uh1 l p t r i s i z ) ((pos "Kj%" ) ))
( "sculpts"  v ( s k uh1 l p t s ) ((pos "Ja%" ) ))
( "sculptural"  j ( s k uh1 l p ch @ r @ l ) ((pos "OA%" ) ))
( "sculpture"  v ( s k uh1 l p ch @ r ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "sculptured"  v ( s k uh1 l p ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sculptures"  v ( s k uh1 l p ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "sculpturing"  v ( s k uh1 l p ch @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "scum"  n ( s k uh1 m ) ((pos "L@%" ) ))
( "scummier"  j ( s k uh1 m i@ r ) ((pos "Or$" ) ))
( "scummiest"  j ( s k uh1 m i i s t ) ((pos "Os$" ) ))
( "scummy"  j ( s k uh1 m ii ) ((pos "OD%" ) ))
( "scupper"  v ( s k uh1 p @ r ) ((pos "H0$" "K6$" ) ))
( "scuppered"  v ( s k uh1 p @ d ) ((pos "Hc$" "Hd%" ) ))
( "scuppering"  v ( s k uh1 p @ r i ng ) ((pos "Hb$" ) ))
( "scuppers"  v ( s k uh1 p @ z ) ((pos "Ha$" "Kj$" ) ))
( "scurf"  n ( s k @@1 f ) ((pos "L@%" ) ))
( "scurfier"  j ( s k @@1 f i@ r ) ((pos "Or$" ) ))
( "scurfiest"  j ( s k @@1 f i i s t ) ((pos "Os$" ) ))
( "scurfy"  j ( s k @@1 f ii ) ((pos "OD%" ) ))
( "scurried"  v ( s k uh1 r i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "scurries"  v ( s k uh1 r i z ) ((pos "Ia%" "Mj%" ) (subcat "2A" "2C" )))
( "scurrility"  n ( s k @1 r i1 l i t ii ) ((pos "L@$" ) ))
( "scurrilous"  j ( s k uh1 r i l @ s ) ((pos "OA%" ) ))
( "scurrilously"  a ( s k uh1 r i l @ s l ii ) ((pos "Pu%" ) ))
( "scurry"  v ( s k uh1 r ii ) ((pos "I3%" "M8%" ) (subcat "2A" "2C" )))
( "scurrying"  v ( s k uh1 r i i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "scurvily"  a ( s k @@1 v i l ii ) ((pos "Pu%" ) ))
( "scurvy"  n ( s k @@1 v ii ) ((pos "L@%" "OA%" ) ))
( "scut"  n ( s k uh1 t ) ((pos "K6$" ) ))
( "scutcheon"  n ( s k uh1 ch @ n ) ((pos "K6$" ) ))
( "scutcheons"  n ( s k uh1 ch @ n z ) ((pos "Kj$" ) ))
( "scuts"  n ( s k uh1 t s ) ((pos "Kj$" ) ))
( "scuttle"  v ( s k uh1 t l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "scuttled"  v ( s k uh1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "scuttles"  v ( s k uh1 t l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "scuttling"  v ( s k uh1 t l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "scythe"  v ( s ai1 dh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "scythed"  v ( s ai1 dh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "scythes"  v ( s ai1 dh z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "scything"  v ( s ai1 dh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sea"  n ( s ii1 ) ((pos "M6%" ) ))
( "sea-anemone"  n ( s ii2 - @ n e1 m @ n ii ) ((pos "K6%" ) ))
( "sea-anemones"  n ( s ii2 - @ n e1 m @ n i z ) ((pos "Kj%" ) ))
( "sea-animal"  n ( s ii1 - a n i m l ) ((pos "K6%" ) ))
( "sea-animals"  n ( s ii1 - a n i m l z ) ((pos "Kj%" ) ))
( "sea-bathing"  n ( s ii1 - b ei dh i ng ) ((pos "L@%" ) ))
( "sea-boat"  n ( s ii1 - b ou t ) ((pos "K6%" ) ))
( "sea-boats"  n ( s ii1 - b ou t s ) ((pos "Kj%" ) ))
( "sea-bream"  n ( s ii1 - b r ii m ) ((pos "K9%" ) ))
( "sea-breeze"  n ( s ii1 - b r ii1 z ) ((pos "K6%" ) ))
( "sea-breezes"  n ( s ii1 - b r ii1 z i z ) ((pos "Kj%" ) ))
( "sea-coal"  n ( s ii1 - k ou l ) ((pos "L@$" ) ))
( "sea-cow"  n ( s ii1 - k au ) ((pos "K6$" ) ))
( "sea-cows"  n ( s ii1 - k au z ) ((pos "Kj$" ) ))
( "sea-dog"  n ( s ii1 - d o g ) ((pos "K6%" ) ))
( "sea-dogs"  n ( s ii1 - d o g z ) ((pos "Kj%" ) ))
( "sea-fish"  n ( s ii1 - f i sh ) ((pos "K9%" ) ))
( "sea-fishes"  n ( s ii1 - f i sh i z ) ((pos "Kj%" ) ))
( "sea-girt"  j ( s ii1 - g @@ t ) ((pos "OA$" ) ))
( "sea-god"  n ( s ii1 - g o d ) ((pos "K6%" ) ))
( "sea-gods"  n ( s ii1 - g o d z ) ((pos "Kj%" ) ))
( "sea-green"  n ( s ii1 - g r ii1 n ) ((pos "L@%" "OA%" ) ))
( "sea-horse"  n ( s ii1 - h oo s ) ((pos "K6%" ) ))
( "sea-horses"  n ( s ii1 - h oo s i z ) ((pos "Kj%" ) ))
( "sea-legs"  n ( s ii1 - l e g z ) ((pos "Kj%" ) ))
( "sea-level"  n ( s ii1 - l e v @ l ) ((pos "Ki%" ) ))
( "sea-lion"  n ( s ii1 - l ai @ n ) ((pos "K6%" ) ))
( "sea-lions"  n ( s ii1 - l ai @ n z ) ((pos "Kj%" ) ))
( "sea-power"  n ( s ii1 - p au @ r ) ((pos "L@%" ) ))
( "sea-rover"  n ( s ii1 - r ou v @ r ) ((pos "K6$" ) ))
( "sea-rovers"  n ( s ii1 - r ou v @ z ) ((pos "Kj$" ) ))
( "sea-snake"  n ( s ii1 - s n ei k ) ((pos "K6%" ) ))
( "sea-snakes"  n ( s ii1 - s n ei k s ) ((pos "Kj%" ) ))
( "sea-urchin"  n ( s ii1 - @@ ch i n ) ((pos "K6%" ) ))
( "sea-urchins"  n ( s ii1 - @@ ch i n z ) ((pos "Kj%" ) ))
( "sea-wall"  n ( s ii1 - w oo1 l ) ((pos "K6%" ) ))
( "sea-walls"  n ( s ii1 - w oo1 l z ) ((pos "Kj%" ) ))
( "sea-water"  n ( s ii1 - w oo t @ r ) ((pos "L@%" ) ))
( "seabed"  n ( s ii1 b e d ) ((pos "K6%" ) ))
( "seabeds"  n ( s ii1 b e d z ) ((pos "Kj$" ) ))
( "seabird"  n ( s ii1 b @@ d ) ((pos "K6%" ) ))
( "seabirds"  n ( s ii1 b @@ d z ) ((pos "Kj%" ) ))
( "seaboard"  n ( s ii1 b oo d ) ((pos "K6%" ) ))
( "seaboards"  n ( s ii1 b oo d z ) ((pos "Kj$" ) ))
( "seaborne"  j ( s ii1 b oo n ) ((pos "OA%" ) ))
( "seafarer"  n ( s ii1 f e@ r @ r ) ((pos "K6%" ) ))
( "seafarers"  n ( s ii1 f e@ r @ z ) ((pos "Kj%" ) ))
( "seafaring"  j ( s ii1 f e@ r i ng ) ((pos "OA%" ) ))
( "seafood"  n ( s ii1 f uu d ) ((pos "M6%" ) ))
( "seafoods"  n ( s ii1 f uu d z ) ((pos "Mj%" ) ))
( "seafront"  n ( s ii1 f r uh n t ) ((pos "K6%" ) ))
( "seafronts"  n ( s ii1 f r uh n t s ) ((pos "Kj$" ) ))
( "seagoing"  j ( s ii1 g ou i ng ) ((pos "OA%" ) ))
( "seagull"  n ( s ii1 g uh l ) ((pos "K6%" ) ))
( "seagulls"  n ( s ii1 g uh l z ) ((pos "Kj%" ) ))
( "seakale"  n ( s ii1 k ei l ) ((pos "L@$" ) ))
( "seal"  v ( s ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "15A" )))
( "seal-ring"  n ( s ii1 l - r i ng ) ((pos "K6$" ) ))
( "seal-rings"  n ( s ii1 l - r i ng z ) ((pos "Kj$" ) ))
( "sealed"  v ( s ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "sealer"  n ( s ii1 l @ r ) ((pos "K6$" ) ))
( "sealers"  n ( s ii1 l @ z ) ((pos "Kj$" ) ))
( "sealing"  v ( s ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "sealing-wax"  n ( s ii1 l i ng - w a k s ) ((pos "L@%" ) ))
( "seals"  v ( s ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "sealskin"  n ( s ii1 l s k i n ) ((pos "M6%" ) ))
( "sealskins"  n ( s ii1 l s k i n z ) ((pos "Mj%" ) ))
( "seam"  v ( s ii1 m ) ((pos "H0%" "K6%" ) ))
( "seaman"  n ( s ii1 m @ n ) ((pos "Ki%" ) ))
( "seamanlike"  j ( s ii1 m @ n l ai k ) ((pos "OA%" ) ))
( "seamanship"  n ( s ii1 m @ n sh i p ) ((pos "L@%" ) ))
( "seamed"  v ( s ii1 m d ) ((pos "Hc%" "Hd%" ) ))
( "seamen"  n ( s ii1 m @ n ) ((pos "Kj%" ) ))
( "seamier"  j ( s ii1 m i@ r ) ((pos "Or%" ) ))
( "seamiest"  j ( s ii1 m i i s t ) ((pos "Os%" ) ))
( "seaming"  v ( s ii1 m i ng ) ((pos "Hb$" ) ))
( "seamless"  j ( s ii1 m l @ s ) ((pos "OA%" ) ))
( "seams"  v ( s ii1 m z ) ((pos "Ha$" "Kj%" ) ))
( "seamstress"  n ( s ii1 m s t r i s ) ((pos "K7%" ) ))
( "seamstresses"  n ( s ii1 m s t r i s i z ) ((pos "Kj%" ) ))
( "seamy"  j ( s ii1 m ii ) ((pos "OD%" ) ))
( "seaplane"  n ( s ii1 p l ei n ) ((pos "K6%" ) ))
( "seaplanes"  n ( s ii1 p l ei n z ) ((pos "Kj%" ) ))
( "seaport"  n ( s ii1 p oo t ) ((pos "K6%" ) ))
( "seaports"  n ( s ii1 p oo t s ) ((pos "Kj%" ) ))
( "sear"  v ( s i@1 r ) ((pos "H0$" "OA$" ) (subcat )))
( "search"  v ( s @@1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "search-parties"  n ( s @@1 ch - p aa t i z ) ((pos "Kj%" ) ))
( "search-party"  n ( s @@1 ch - p aa t ii ) ((pos "K8%" ) ))
( "search-warrant"  n ( s @@1 ch - w o r @ n t ) ((pos "K6%" ) ))
( "search-warrants"  n ( s @@1 ch - w o r @ n t s ) ((pos "Kj%" ) ))
( "searched"  v ( s @@1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "searcher"  n ( s @@1 ch @ r ) ((pos "K6%" ) ))
( "searchers"  n ( s @@1 ch @ z ) ((pos "Kj%" ) ))
( "searches"  v ( s @@1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "searching"  v ( s @@1 ch i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "searchingly"  a ( s @@1 ch i ng l ii ) ((pos "Pu%" ) ))
( "searchlight"  n ( s @@1 ch l ai t ) ((pos "K6%" ) ))
( "searchlights"  n ( s @@1 ch l ai t s ) ((pos "Kj%" ) ))
( "seared"  v ( s i@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "searing"  v ( s i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "searing-iron"  n ( s e@1 r i ng - ai @ n ) ((pos "K6$" ) ))
( "searing-irons"  n ( s e@1 r i ng - ai @ n z ) ((pos "Kj$" ) ))
( "sears"  v ( s i@1 z ) ((pos "Ha$" ) (subcat )))
( "seas"  n ( s ii1 z ) ((pos "Mj%" ) ))
( "seascape"  n ( s ii1 s k ei p ) ((pos "K6%" ) ))
( "seascapes"  n ( s ii1 s k ei p s ) ((pos "Kj%" ) ))
( "seashell"  n ( s ii1 sh e l ) ((pos "K6%" ) ))
( "seashells"  n ( s ii1 sh e l z ) ((pos "Kj%" ) ))
( "seashore"  n ( s ii1 sh oo r ) ((pos "K6%" ) ))
( "seashores"  n ( s ii1 sh oo z ) ((pos "Kj%" ) ))
( "seasick"  j ( s ii1 s i k ) ((pos "OA%" ) ))
( "seasickness"  n ( s ii1 s i k n @ s ) ((pos "L@%" ) ))
( "seaside"  n ( s ii1 s ai d ) ((pos "K6%" ) ))
( "seasides"  n ( s ii1 s ai d z ) ((pos "Kj$" ) ))
( "season"  v ( s ii1 z n ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "season-ticket"  n ( s ii1 z n - t i k i t ) ((pos "K6%" ) ))
( "season-tickets"  n ( s ii1 z n - t i k i t s ) ((pos "Kj%" ) ))
( "seasonable"  j ( s ii1 z n @ b l ) ((pos "OA%" ) ))
( "seasonal"  j ( s ii1 z @ n l ) ((pos "OA%" ) ))
( "seasonally"  a ( s ii1 z @ n @ l ii ) ((pos "Pu%" ) ))
( "seasoned"  v ( s ii1 z n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "seasoning"  v ( s ii1 z n i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" )))
( "seasonings"  n ( s ii1 z n i ng z ) ((pos "Mj%" ) ))
( "seasons"  v ( s ii1 z n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "seat"  v ( s ii1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "seat-belt"  n ( s ii1 t - b e l t ) ((pos "K6%" ) ))
( "seat-belts"  n ( s ii1 t - b e l t s ) ((pos "Kj%" ) ))
( "seated"  v ( s ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "seating"  v ( s ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "seating-room"  n ( s ii1 t i ng - r uu m ) ((pos "L@$" ) ))
( "seats"  v ( s ii1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "seaward"  j ( s ii1 w @ d ) ((pos "OA%" ) ))
( "seawards"  a ( s ii1 w @ d z ) ((pos "Pu%" ) ))
( "seaway"  n ( s ii1 w ei ) ((pos "K6%" ) ))
( "seaways"  n ( s ii1 w ei z ) ((pos "Kj%" ) ))
( "seaweed"  n ( s ii1 w ii d ) ((pos "M6%" ) ))
( "seaweeds"  n ( s ii1 w ii d z ) ((pos "Mj$" ) ))
( "seaworthiness"  n ( s ii1 w @@ dh i n i s ) ((pos "L@%" ) ))
( "seaworthy"  j ( s ii1 w @@ dh ii ) ((pos "OA%" ) ))
( "sec"  n ( s e1 k ) ((pos "K6$" ) ))
( "secateurs"  n ( s e1 k @ t @@ z ) ((pos "Kj%" ) ))
( "secede"  v ( s i1 s ii1 d ) ((pos "I2%" ) (subcat "2A" )))
( "seceded"  v ( s i1 s ii1 d i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "secedes"  v ( s i1 s ii1 d z ) ((pos "Ia%" ) (subcat "2A" )))
( "seceding"  v ( s i1 s ii1 d i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "secession"  n ( s i1 s e1 sh @ n ) ((pos "M6%" ) ))
( "secessionist"  n ( s i1 s e1 sh @ n i s t ) ((pos "K6%" ) ))
( "secessionists"  n ( s i1 s e1 sh @ n i s t s ) ((pos "Kj%" ) ))
( "secessions"  n ( s i1 s e1 sh @ n z ) ((pos "Mj$" ) ))
( "seclude"  v ( s i1 k l uu1 d ) ((pos "H2$" ) (subcat "6A" "14" )))
( "secluded"  v ( s i1 k l uu1 d i d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat "6A" "14" )))
( "secludes"  v ( s i1 k l uu1 d z ) ((pos "Ha$" ) (subcat "6A" "14" )))
( "secluding"  v ( s i1 k l uu1 d i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "seclusion"  n ( s i1 k l uu1 zh n ) ((pos "L@%" ) ))
( "second"  v ( s e1 k @ n d ) ((pos "H0%" "K6*" "OA*" "Pu*" ) (subcat )))
( "second"  v ( s i1 k o1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "second-best"  n ( s e1 k @ n d - b e s t ) ((pos "Ki%" "OA%" "Pu%" ) ))
( "second-class"  n ( s e1 k @ n d - k l aa s ) ((pos "L@%" "OA%" "Pu%" ) ))
( "second-hand"  n ( s e1 k @ n d - h a n d ) ((pos "K6$" "OA%" ) ))
( "second-hands"  n ( s e1 k @ n d - h a n d z ) ((pos "Kj$" ) ))
( "second-rate"  j ( s e1 k @ n d - r ei t ) ((pos "OA%" ) ))
( "second-rater"  n ( s e2 k @ n d - r ei1 t @ r ) ((pos "K6%" ) ))
( "second-raters"  n ( s e2 k @ n d - r ei1 t @ z ) ((pos "Kj%" ) ))
( "second-sighted"  j ( s e2 k @ n d - s ai1 t i d ) ((pos "OA%" ) ))
( "secondarily"  a ( s e1 k @ n d r i l ii ) ((pos "Pu%" ) ))
( "secondary"  j ( s e1 k @ n d r ii ) ((pos "OA%" ) ))
( "seconded"  v ( s e1 k @ n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "seconded"  v ( s i1 k o1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "seconder"  n ( s e1 k @ n d @ r ) ((pos "K6%" ) ))
( "seconders"  n ( s e1 k @ n d @ z ) ((pos "Kj%" ) ))
( "seconding"  v ( s e1 k @ n d i ng ) ((pos "Hb%" ) (subcat )))
( "seconding"  v ( s i1 k o1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "secondly"  a ( s e1 k @ n d l ii ) ((pos "Pu%" ) ))
( "secondment"  n ( s i1 k o1 n d m @ n t ) ((pos "K6%" ) ))
( "secondments"  n ( s i1 k o1 n d m @ n t s ) ((pos "Kj%" ) ))
( "seconds"  v ( s e1 k @ n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "seconds"  v ( s i1 k o1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "secrecy"  n ( s ii1 k r @ s ii ) ((pos "L@%" ) ))
( "secret"  n ( s ii1 k r i t ) ((pos "M6%" "OA%" ) ))
( "secretarial"  j ( s e2 k r @ t e@1 r i@ l ) ((pos "OA%" ) ))
( "secretariat"  n ( s e2 k r @ t e@1 r i@ t ) ((pos "K6%" ) ))
( "secretariats"  n ( s e2 k r @ t e@1 r i@ t s ) ((pos "Kj%" ) ))
( "secretaries"  n ( s e1 k r @ t r i z ) ((pos "Kj%" ) ))
( "secretary"  n ( s e1 k r @ t r ii ) ((pos "K8%" ) ))
( "secrete"  v ( s i1 k r ii1 t ) ((pos "H2%" ) (subcat )))
( "secreted"  v ( s i1 k r ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "secretes"  v ( s i1 k r ii1 t s ) ((pos "Ha%" ) (subcat )))
( "secreting"  v ( s i1 k r ii1 t i ng ) ((pos "Hb%" ) (subcat )))
( "secretion"  n ( s i1 k r ii1 sh @ n ) ((pos "M6%" ) ))
( "secretions"  n ( s i1 k r ii1 sh @ n z ) ((pos "Mj%" ) ))
( "secretive"  j ( s ii1 k r @ t i v ) ((pos "OA%" ) ))
( "secretively"  a ( s ii1 k r @ t i v l ii ) ((pos "Pu%" ) ))
( "secretiveness"  n ( s ii1 k r @ t i v n @ s ) ((pos "L@%" ) ))
( "secretly"  a ( s ii1 k r i t l ii ) ((pos "Pu%" ) ))
( "secrets"  n ( s ii1 k r i t s ) ((pos "Mj%" ) ))
( "secs"  n ( s e1 k s ) ((pos "Kj$" ) ))
( "sect"  n ( s e1 k t ) ((pos "K6%" ) ))
( "sectarian"  n ( s e1 k t e@1 r i@ n ) ((pos "K6$" "OA%" ) ))
( "sectarianism"  n ( s e1 k t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "sectarians"  n ( s e1 k t e@1 r i@ n z ) ((pos "Kj$" ) ))
( "section"  n ( s e1 k sh @ n ) ((pos "K6%" ) ))
( "sectional"  j ( s e1 k sh @ n l ) ((pos "OA$" ) ))
( "sectionalism"  n ( s e1 k sh @ n @ l i z @ m ) ((pos "L@$" ) ))
( "sections"  n ( s e1 k sh @ n z ) ((pos "Kj%" ) ))
( "sector"  n ( s e1 k t @ r ) ((pos "K6%" ) ))
( "sectors"  n ( s e1 k t @ z ) ((pos "Kj%" ) ))
( "sects"  n ( s e1 k t s ) ((pos "Kj%" ) ))
( "secular"  j ( s e1 k y u l @ r ) ((pos "OA%" ) ))
( "secularism"  n ( s e1 k y u l @ r i z @ m ) ((pos "L@$" ) ))
( "secularist"  n ( s e1 k y u l @ r i s t ) ((pos "K6$" ) ))
( "secularists"  n ( s e1 k y u l @ r i s t s ) ((pos "Kj$" ) ))
( "secularize"  v ( s e1 k y @ l @ r ai z ) ((pos "H2$" ) (subcat )))
( "secularized"  v ( s e1 k y @ l @ r ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "secularizes"  v ( s e1 k y @ l @ r ai z i z ) ((pos "Ha$" ) (subcat )))
( "secularizing"  v ( s e1 k y @ l @ r ai z i ng ) ((pos "Hb$" ) (subcat )))
( "secure"  v ( s i1 k y u@1 r ) ((pos "H2%" "OA%" ) (subcat "6A" "12B" "13B" )))
( "secured"  v ( s i1 k y u@1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "12B" "13B" )))
( "securely"  a ( s i1 k y u@1 l ii ) ((pos "Pu%" ) ))
( "secures"  v ( s i1 k y u@1 z ) ((pos "Ha%" ) (subcat "6A" "12B" "13B" )))
( "securing"  v ( s i1 k y u@1 r i ng ) ((pos "Hb%" ) (subcat "6A" "12B" "13B" )))
( "securities"  n ( s i1 k y u@1 r i t i z ) ((pos "Mj%" ) ))
( "security"  n ( s i1 k y u@1 r i t ii ) ((pos "M8%" ) ))
( "sedan"  n ( s i1 d a1 n ) ((pos "K6$" ) ))
( "sedan-chair"  n ( s i1 d a1 n - ch e@ r ) ((pos "K6$" ) ))
( "sedan-chairs"  n ( s i1 d a1 n - ch e@ z ) ((pos "Kj$" ) ))
( "sedans"  n ( s i1 d a1 n z ) ((pos "Kj$" ) ))
( "sedate"  v ( s i1 d ei1 t ) ((pos "H2%" "OA%" ) (subcat )))
( "sedated"  v ( s i1 d ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sedately"  a ( s i1 d ei1 t l ii ) ((pos "Pu%" ) ))
( "sedateness"  n ( s i1 d ei1 t n @ s ) ((pos "L@%" ) ))
( "sedates"  v ( s i1 d ei1 t s ) ((pos "Ha%" ) (subcat )))
( "sedating"  v ( s i1 d ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "sedation"  n ( s i1 d ei1 sh @ n ) ((pos "L@%" ) ))
( "sedative"  n ( s e1 d @ t i v ) ((pos "M6%" "OA%" ) ))
( "sedatives"  n ( s e1 d @ t i v z ) ((pos "Mj%" ) ))
( "sedentary"  j ( s e1 d n t r ii ) ((pos "OA%" ) ))
( "sedge"  n ( s e1 jh ) ((pos "M6%" ) ))
( "sedges"  n ( s e1 jh i z ) ((pos "Mj%" ) ))
( "sedgier"  j ( s e1 jh i@ r ) ((pos "Or$" ) ))
( "sedgiest"  j ( s e1 jh i i s t ) ((pos "Os$" ) ))
( "sedgy"  j ( s e1 jh ii ) ((pos "OD$" ) ))
( "sediment"  n ( s e1 d i m @ n t ) ((pos "L@%" ) ))
( "sedimentary"  j ( s e2 d i m e1 n t r ii ) ((pos "OA%" ) ))
( "sedition"  n ( s i1 d i1 sh @ n ) ((pos "L@%" ) ))
( "seditious"  j ( s i1 d i1 sh @ s ) ((pos "OA%" ) ))
( "seduce"  v ( s i1 d y uu1 s ) ((pos "H2%" ) (subcat "6A" )))
( "seduced"  v ( s i1 d y uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "seducer"  n ( s i1 d y uu1 s @ r ) ((pos "K6%" ) ))
( "seducers"  n ( s i1 d y uu1 s @ z ) ((pos "Kj%" ) ))
( "seduces"  v ( s i1 d y uu1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "seducing"  v ( s i1 d y uu1 s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "seduction"  n ( s i1 d uh1 k sh @ n ) ((pos "M6%" ) ))
( "seductions"  n ( s i1 d uh1 k sh @ n z ) ((pos "Mj%" ) ))
( "seductive"  j ( s i1 d uh1 k t i v ) ((pos "OA%" ) ))
( "seductively"  a ( s i1 d uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "sedulous"  j ( s e1 d y u l @ s ) ((pos "OA$" ) ))
( "sedulously"  a ( s e1 d y u l @ s l ii ) ((pos "Pu$" ) ))
( "see"  v ( s ii1 ) ((pos "J5*" "K6%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "16B" "18A" "19A" "22" )))
( "see-through"  j ( s ii1 - th r uu ) ((pos "OA%" ) ))
( "seed"  v ( s ii1 d ) ((pos "J0$" "M9%" ) (subcat "2A" )))
( "seed-corn"  n ( s ii1 d - k oo n ) ((pos "L@%" ) ))
( "seed-pearls"  n ( s ii1 d - p @@ l z ) ((pos "Kj%" ) ))
( "seedbed"  n ( s ii1 d b e d ) ((pos "K6%" ) ))
( "seedbeds"  n ( s ii1 d b e d z ) ((pos "Kj%" ) ))
( "seedcake"  n ( s ii1 d k ei k ) ((pos "K6%" ) ))
( "seedcakes"  n ( s ii1 d k ei k s ) ((pos "Kj%" ) ))
( "seeded"  v ( s ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "seedier"  j ( s ii1 d i@ r ) ((pos "Or%" ) ))
( "seediest"  j ( s ii1 d i i s t ) ((pos "Os%" ) ))
( "seedily"  a ( s ii1 d i l ii ) ((pos "Pu%" ) ))
( "seediness"  n ( s ii1 d i n @ s ) ((pos "L@%" ) ))
( "seeding"  v ( s ii1 d i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "seedless"  j ( s ii1 d l @ s ) ((pos "OA%" ) ))
( "seedling"  n ( s ii1 d l i ng ) ((pos "K6%" ) ))
( "seedlings"  n ( s ii1 d l i ng z ) ((pos "Kj%" ) ))
( "seeds"  v ( s ii1 d z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "seedsman"  n ( s ii1 d z m @ n ) ((pos "Ki%" ) ))
( "seedsmen"  n ( s ii1 d z m @ n ) ((pos "Kj%" ) ))
( "seedtime"  n ( s ii1 d t ai m ) ((pos "K6%" ) ))
( "seedtimes"  n ( s ii1 d t ai m z ) ((pos "Kj$" ) ))
( "seedy"  j ( s ii1 d ii ) ((pos "OD%" ) ))
( "seeing"  v ( s ii1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "16B" "18A" "19A" "22" )))
( "seek"  v ( s ii1 k ) ((pos "H5%" ) (subcat "2A" "3A" "6A" "7A" )))
( "seeker"  n ( s ii1 k @ r ) ((pos "K6%" ) ))
( "seekers"  n ( s ii1 k @ z ) ((pos "Kj%" ) ))
( "seeking"  v ( s ii1 k i ng ) ((pos "Hb%" ) (subcat "2A" "3A" "6A" "7A" )))
( "seeks"  v ( s ii1 k s ) ((pos "Ha%" ) (subcat "2A" "3A" "6A" "7A" )))
( "seem"  v ( s ii1 m ) ((pos "I0*" ) (subcat "2A" "4D" )))
( "seemed"  v ( s ii1 m d ) ((pos "Ic*" "Id*" ) (subcat "2A" "4D" )))
( "seeming"  v ( s ii1 m i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" "4D" )))
( "seemingly"  a ( s ii1 m i ng l ii ) ((pos "Pu%" ) ))
( "seemlier"  j ( s ii1 m l i@ r ) ((pos "Or$" ) ))
( "seemliest"  j ( s ii1 m l i i s t ) ((pos "Os$" ) ))
( "seemliness"  n ( s ii1 m l i n @ s ) ((pos "L@%" ) ))
( "seemly"  j ( s ii1 m l ii ) ((pos "OD%" ) ))
( "seems"  v ( s ii1 m z ) ((pos "Ia%" ) (subcat "2A" "4D" )))
( "seen"  v ( s ii1 n ) ((pos "Jd*" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "16B" "18A" "19A" "22" )))
( "seep"  v ( s ii1 p ) ((pos "I0%" ) (subcat )))
( "seepage"  n ( s ii1 p i jh ) ((pos "L@%" ) ))
( "seeped"  v ( s ii1 p t ) ((pos "Ic%" "Id%" ) (subcat )))
( "seeping"  v ( s ii1 p i ng ) ((pos "Ib%" ) (subcat )))
( "seeps"  v ( s ii1 p s ) ((pos "Ia%" ) (subcat )))
( "seer"  n ( s i@1 r ) ((pos "K6%" ) ))
( "seers"  n ( s i@1 z ) ((pos "Kj%" ) ))
( "seersucker"  n ( s i@1 s uh k @ r ) ((pos "L@$" ) ))
( "sees"  v ( s ii1 z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2B" "2C" "4A" "6A" "8" "9" "10" "15A" "16B" "18A" "19A" "22" )))
( "seesaw"  v ( s ii1 s oo ) ((pos "I0%" "M6%" ) ))
( "seesawed"  v ( s ii1 s oo d ) ((pos "Ic%" "Id%" ) ))
( "seesawing"  v ( s ii1 s oo i ng ) ((pos "Ib%" ) ))
( "seesaws"  v ( s ii1 s oo z ) ((pos "Ia%" "Mj%" ) ))
( "seethe"  v ( s ii1 dh ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "seethed"  v ( s ii1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "seethes"  v ( s ii1 dh z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "seething"  v ( s ii1 dh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "segment"  n ( s e1 g m @ n t ) ((pos "K6%" ) ))
( "segment"  v ( s e1 g m e1 n t ) ((pos "J0%" ) ))
( "segmentation"  n ( s e2 g m @ n t ei1 sh @ n ) ((pos "M6$" ) ))
( "segmentations"  n ( s e2 g m @ n t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "segmented"  v ( s e1 g m e1 n t i d ) ((pos "Jc%" "Jd%" ) ))
( "segmenting"  v ( s e1 g m e1 n t i ng ) ((pos "Jb%" ) ))
( "segments"  n ( s e1 g m @ n t s ) ((pos "Kj%" ) ))
( "segments"  v ( s e1 g m e1 n t s ) ((pos "Ja%" ) ))
( "segregate"  v ( s e1 g r i g ei t ) ((pos "H2%" ) (subcat )))
( "segregated"  v ( s e1 g r i g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "segregates"  v ( s e1 g r i g ei t s ) ((pos "Ha%" ) (subcat )))
( "segregating"  v ( s e1 g r i g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "segregation"  n ( s e2 g r i g ei1 sh @ n ) ((pos "K6%" ) ))
( "segregations"  n ( s e2 g r i g ei1 sh @ n z ) ((pos "Kj%" ) ))
( "seignior"  n ( s ei1 n y @ r ) ((pos "K6$" ) ))
( "seigniors"  n ( s ei1 n y @ z ) ((pos "Kj$" ) ))
( "seine"  v ( s ei1 n ) ((pos "J2$" "K6$" ) ))
( "seined"  v ( s ei1 n d ) ((pos "Jc$" "Jd$" ) ))
( "seines"  v ( s ei1 n z ) ((pos "Ja$" "Kj$" ) ))
( "seining"  v ( s ei1 n i ng ) ((pos "Jb$" ) ))
( "seismic"  j ( s ai1 z m i k ) ((pos "OA%" ) ))
( "seismograph"  n ( s ai1 z m @ g r aa f ) ((pos "K6%" ) ))
( "seismographs"  n ( s ai1 z m @ g r aa f s ) ((pos "Kj%" ) ))
( "seismologist"  n ( s ai1 z m o1 l @ jh i s t ) ((pos "K6%" ) ))
( "seismologists"  n ( s ai1 z m o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "seismology"  n ( s ai1 z m o1 l @ jh ii ) ((pos "L@%" ) ))
( "seize"  v ( s ii1 z ) ((pos "J2%" ) (subcat "2A" "2C" "3A" "6A" )))
( "seized"  v ( s ii1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "seizes"  v ( s ii1 z i z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "seizing"  v ( s ii1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "seizure"  n ( s ii1 zh @ r ) ((pos "M6%" ) ))
( "seizures"  n ( s ii1 zh @ z ) ((pos "Mj%" ) ))
( "seldom"  a ( s e1 l d @ m ) ((pos "Pu%" ) ))
( "select"  v ( s i1 l e1 k t ) ((pos "H0%" "OA%" ) (subcat "6A" "15A" )))
( "selected"  v ( s i1 l e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "selecting"  v ( s i1 l e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "selection"  n ( s i1 l e1 k sh @ n ) ((pos "M6%" ) ))
( "selections"  n ( s i1 l e1 k sh @ n z ) ((pos "Mj%" ) ))
( "selective"  j ( s i1 l e1 k t i v ) ((pos "OA%" ) ))
( "selectively"  a ( s i1 l e1 k t i v l ii ) ((pos "Pu%" ) ))
( "selectivity"  n ( s i2 l e k t i1 v i t ii ) ((pos "L@%" ) ))
( "selector"  n ( s i1 l e1 k t @ r ) ((pos "K6%" ) ))
( "selectors"  n ( s i1 l e1 k t @ z ) ((pos "Kj%" ) ))
( "selects"  v ( s i1 l e1 k t s ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "selenium"  n ( s i1 l ii1 n i@ m ) ((pos "L@$" ) ))
( "self"  n ( s e1 l f ) ((pos "Mi%" ) ))
( "self-"  j ( s e l f - ) ((pos "U-%" ) ))
( "self-abasement"  n ( s e1 l f - @ b ei1 s m @ n t ) ((pos "L@%" ) ))
( "self-abnegation"  n ( s e1 l f - a2 b n i g ei1 sh @ n ) ((pos "L@$" ) ))
( "self-absorbed"  j ( s e2 l f - @ b z oo1 b d ) ((pos "OA%" ) ))
( "self-acting"  j ( s e1 l f - a1 k t i ng ) ((pos "OA$" ) ))
( "self-activating"  j ( s e1 l f - a1 k t i v ei t i ng ) ((pos "OA%" ) ))
( "self-addressed"  j ( s e2 l f - @ d r e1 s t ) ((pos "OA%" ) ))
( "self-appointed"  j ( s e2 l f - @ p oi1 n t i dh ) ((pos "OA%" ) ))
( "self-assertion"  n ( s e1 l f - @ s @@1 sh @ n ) ((pos "L@%" ) ))
( "self-assertive"  j ( s e1 l f - @ s @@1 t i v ) ((pos "OA%" ) ))
( "self-assurance"  n ( s e1 l f - @ sh u@1 r @ n s ) ((pos "L@%" ) ))
( "self-assured"  j ( s e2 l f - @ sh u@1 d ) ((pos "OA%" ) ))
( "self-centred"  j ( s e1 l f - s e1 n t @ d ) ((pos "OA%" ) ))
( "self-collected"  j ( s e2 l f - k @ l e1 k t i d ) ((pos "OA$" ) ))
( "self-coloured"  j ( s e1 l f - k uh1 l @ d ) ((pos "OA$" ) ))
( "self-command"  n ( s e1 l f - k @ m aa1 n d ) ((pos "L@%" ) ))
( "self-communion"  n ( s e1 l f - k @ m y uu1 n i@ n ) ((pos "L@$" ) ))
( "self-complacency"  n ( s e2 l f - k @ m p l ei1 s n s ii ) ((pos "L@$" ) ))
( "self-confessed"  j ( s e2 l f - k @ n f e1 s t ) ((pos "OA%" ) ))
( "self-confidence"  n ( s e1 l f - k o1 n f i d @ n s ) ((pos "L@%" ) ))
( "self-confident"  j ( s e1 l f - k o1 n f i d @ n t ) ((pos "OA%" ) ))
( "self-conscious"  j ( s e1 l f - k o1 n sh @ s ) ((pos "OA%" ) ))
( "self-consciousness"  n ( s e1 l f - k o1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "self-contained"  j ( s e2 l f - k @ n t ei1 n d ) ((pos "OA%" ) ))
( "self-control"  n ( s e1 l f - k @ n t r ou1 l ) ((pos "L@%" ) ))
( "self-defence"  n ( s e1 l f - d i f e1 n s ) ((pos "L@%" ) ))
( "self-denial"  n ( s e1 l f - d i n ai1 @ l ) ((pos "L@%" ) ))
( "self-denying"  j ( s e2 l f - d i n ai1 i ng ) ((pos "OA%" ) ))
( "self-determination"  n ( s e1 l f - d i t @@2 m i n ei1 sh @ n ) ((pos "L@%" ) ))
( "self-educated"  j ( s e1 l f - e1 d y @ k ei t i d ) ((pos "OA%" ) ))
( "self-effacing"  j ( s e2 l f - i f ei1 s i ng ) ((pos "OA%" ) ))
( "self-employed"  j ( s e2 l f - i m p l oi1 d ) ((pos "OA%" ) ))
( "self-esteem"  n ( s e1 l f - i s t ii1 m ) ((pos "L@%" ) ))
( "self-evident"  j ( s e1 l f - e1 v i d @ n t ) ((pos "OA%" ) ))
( "self-examination"  n ( s e2 l f - i g z a2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "self-examinations"  n ( s e2 l f - i g z a2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "self-explanatory"  j ( s e1 l f - i k s p l a1 n @ t r ii ) ((pos "OA%" ) ))
( "self-help"  n ( s e1 l f - h e1 l p ) ((pos "L@%" ) ))
( "self-importance"  n ( s e1 l f - i m p oo1 t n s ) ((pos "L@%" ) ))
( "self-important"  j ( s e1 l f - i m p oo1 t n t ) ((pos "OA%" ) ))
( "self-imposed"  j ( s e2 l f - i m p ou1 z d ) ((pos "OA%" ) ))
( "self-indulgence"  n ( s e1 l f - i n d uh1 l jh @ n s ) ((pos "L@%" ) ))
( "self-indulgent"  j ( s e2 l f - i n d uh1 l jh @ n t ) ((pos "OA%" ) ))
( "self-interest"  n ( s e1 l f - i1 n t r @ s t ) ((pos "L@%" ) ))
( "self-locking"  j ( s e1 l f - l o1 k i ng ) ((pos "OA%" ) ))
( "self-made"  j ( s e1 l f - m ei d ) ((pos "OA%" ) ))
( "self-opinionated"  j ( s e2 l f - @ p i1 n i@ n ei t i d ) ((pos "OA%" ) ))
( "self-pity"  n ( s e1 l f - p i1 t ii ) ((pos "L@%" ) ))
( "self-possessed"  j ( s e2 l f - p @ z e1 s t ) ((pos "OA%" ) ))
( "self-possession"  n ( s e1 l f - p @ z e1 sh @ n ) ((pos "L@%" ) ))
( "self-preservation"  n ( s e2 l f - p r e2 z @ v ei1 sh @ n ) ((pos "L@%" ) ))
( "self-raising"  j ( s e1 l f - r ei1 z i ng ) ((pos "OA%" ) ))
( "self-reliance"  n ( s e1 l f - r i l ai1 @ n s ) ((pos "L@%" ) ))
( "self-reliant"  j ( s e2 l f - r i l ai1 @ n t ) ((pos "OA%" ) ))
( "self-respect"  n ( s e1 l f - r i s p e1 k t ) ((pos "L@%" ) ))
( "self-respecting"  j ( s e2 l f - r i s p e1 k t i ng ) ((pos "OA%" ) ))
( "self-righteous"  j ( s e1 l f - r ai1 ch @ s ) ((pos "OA%" ) ))
( "self-rule"  n ( s e1 l f - r uu1 l ) ((pos "L@%" ) ))
( "self-sacrifice"  n ( s e1 l f - s a1 k r i f ai s ) ((pos "M6%" ) ))
( "self-sacrifices"  n ( s e1 l f - s a1 k r i f ai s i z ) ((pos "Mj$" ) ))
( "self-sacrificing"  j ( s e1 l f - s a1 k r i f ai s i ng ) ((pos "OA%" ) ))
( "self-same"  j ( s e1 l f - s ei m ) ((pos "OA%" ) ))
( "self-sealing"  j ( s e1 l f - s ii1 l i ng ) ((pos "OA%" ) ))
( "self-seeker"  n ( s e1 l f - s ii1 k @ r ) ((pos "K6$" ) ))
( "self-seekers"  n ( s e1 l f - s ii1 k @ z ) ((pos "Kj$" ) ))
( "self-seeking"  n ( s e1 l f - s ii1 k i ng ) ((pos "L@%" "OA%" ) ))
( "self-service"  n ( s e1 l f - s @@1 v i s ) ((pos "L@%" ) ))
( "self-sown"  j ( s e1 l f - s ou1 n ) ((pos "OA$" ) ))
( "self-starter"  n ( s e1 l f - s t aa1 t @ r ) ((pos "K6%" ) ))
( "self-starters"  n ( s e1 l f - s t aa1 t @ z ) ((pos "Kj%" ) ))
( "self-styled"  j ( s e1 l f - s t ai l d ) ((pos "OA%" ) ))
( "self-sufficiency"  n ( s e1 l f - s @ f i1 sh @ n s ii ) ((pos "L@%" ) ))
( "self-sufficient"  j ( s e1 l f - s @ f i1 sh @ n t ) ((pos "OA%" ) ))
( "self-sufficing"  j ( s e2 l f - s @ f ai1 s i ng ) ((pos "OA$" ) ))
( "self-supporting"  j ( s e2 l f - s @ p oo1 t i ng ) ((pos "OA%" ) ))
( "self-will"  n ( s e1 l f - w i1 l ) ((pos "L@%" ) ))
( "self-willed"  j ( s e1 l f - w i1 l d ) ((pos "OA%" ) ))
( "self-winding"  j ( s e1 l f - w ai1 n d i ng ) ((pos "OA%" ) ))
( "selfconsciously"  a ( s e1 l f k o1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "selfish"  j ( s e1 l f i sh ) ((pos "OA%" ) ))
( "selfishly"  a ( s e1 l f i sh l ii ) ((pos "Pu%" ) ))
( "selfishness"  n ( s e1 l f i sh n @ s ) ((pos "L@%" ) ))
( "sell"  v ( s e1 l ) ((pos "J5%" "Ki%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "seller"  n ( s e1 l @ r ) ((pos "K6%" ) ))
( "sellers"  n ( s e1 l @ z ) ((pos "Kj%" ) ))
( "selling"  v ( s e1 l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "sellout"  n ( s e1 l au t ) ((pos "K6%" ) ))
( "sellouts"  n ( s e1 l au t s ) ((pos "Kj$" ) ))
( "sells"  v ( s e1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "selvage"  n ( s e1 l v i jh ) ((pos "K6$" ) ))
( "selvages"  n ( s e1 l v i jh i z ) ((pos "Kj$" ) ))
( "selvedge"  n ( s e1 l v i jh ) ((pos "K6$" ) ))
( "selvedges"  n ( s e1 l v i jh i z ) ((pos "Kj$" ) ))
( "selves"  n ( s e1 l v z ) ((pos "Kj%" ) ))
( "semantic"  j ( s i1 m a1 n t i k ) ((pos "OA%" ) ))
( "semantics"  n ( s i1 m a1 n t i k s ) ((pos "Lk%" ) ))
( "semaphore"  v ( s e1 m @ f oo r ) ((pos "J2$" "L@%" ) (subcat "2A" )))
( "semaphored"  v ( s e1 m @ f oo d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "semaphores"  v ( s e1 m @ f oo z ) ((pos "Ja$" ) (subcat "2A" )))
( "semaphoring"  v ( s e1 m @ f oo r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "semblance"  n ( s e1 m b l @ n s ) ((pos "M6%" ) ))
( "semblances"  n ( s e1 m b l @ n s i z ) ((pos "Mj$" ) ))
( "semen"  n ( s ii1 m @ n ) ((pos "L@%" ) ))
( "semester"  n ( s i1 m e1 s t @ r ) ((pos "K6%" ) ))
( "semesters"  n ( s i1 m e1 s t @ z ) ((pos "Kj%" ) ))
( "semi-"  j ( s e1 m i - ) ((pos "U-%" ) ))
( "semibreve"  n ( s e1 m i b r ii v ) ((pos "K6$" ) ))
( "semibreves"  n ( s e1 m i b r ii v z ) ((pos "Kj$" ) ))
( "semicircle"  n ( s e1 m i s @@ k l ) ((pos "K6%" ) ))
( "semicircles"  n ( s e1 m i s @@ k l z ) ((pos "Kj%" ) ))
( "semicircular"  j ( s e2 m i s @@1 k y u l @ r ) ((pos "OA%" ) ))
( "semicolon"  n ( s e2 m i k ou1 l @ n ) ((pos "K6%" ) ))
( "semicolons"  n ( s e2 m i k ou1 l @ n z ) ((pos "Kj%" ) ))
( "semiconducting"  j ( s e2 m i k @ n d uh1 k t i ng ) ((pos "OA%" ) ))
( "semiconductor"  n ( s e2 m i k @ n d uh1 k t @ r ) ((pos "K6%" ) ))
( "semiconductors"  n ( s e2 m i k @ n d uh1 k t @ z ) ((pos "Kj%" ) ))
( "semiconscious"  j ( s e2 m i k o1 n sh @ s ) ((pos "OA%" ) ))
( "semidetached"  j ( s e2 m i d i t a1 ch t ) ((pos "OA%" ) ))
( "semifinal"  n ( s e2 m i f ai1 n l ) ((pos "K6%" ) ))
( "semifinalist"  n ( s e2 m i f ai1 n @ l i s t ) ((pos "K6%" ) ))
( "semifinalists"  n ( s e2 m i f ai1 n @ l i s t s ) ((pos "Kj%" ) ))
( "semifinals"  n ( s e2 m i f ai1 n l z ) ((pos "Kj%" ) ))
( "seminal"  j ( s e1 m i n l ) ((pos "OA%" ) ))
( "seminar"  n ( s e1 m i n aa r ) ((pos "K6%" ) ))
( "seminaries"  n ( s e1 m i n @ r i z ) ((pos "Kj$" ) ))
( "seminarist"  n ( s e1 m i n @ r i s t ) ((pos "K6$" ) ))
( "seminarists"  n ( s e1 m i n @ r i s t s ) ((pos "Kj$" ) ))
( "seminars"  n ( s e1 m i n aa z ) ((pos "Kj%" ) ))
( "seminary"  n ( s e1 m i n @ r ii ) ((pos "K8%" ) ))
( "semiofficial"  j ( s e2 m i@ f i1 sh l ) ((pos "OA%" ) ))
( "semiquaver"  n ( s e1 m i k w ei v @ r ) ((pos "K6%" ) ))
( "semiquavers"  n ( s e1 m i k w ei v @ z ) ((pos "Kj%" ) ))
( "semirigid"  j ( s e2 m i r i1 jh i d ) ((pos "OA$" ) ))
( "semitone"  n ( s e1 m i t ou n ) ((pos "K6%" ) ))
( "semitones"  n ( s e1 m i t ou n z ) ((pos "Kj%" ) ))
( "semitropical"  j ( s e2 m i t r o1 p i k l ) ((pos "OA$" ) ))
( "semivowel"  n ( s e1 m i v au @ l ) ((pos "K6$" ) ))
( "semivowels"  n ( s e1 m i v au @ l z ) ((pos "Kj$" ) ))
( "semolina"  n ( s e2 m @ l ii1 n @ ) ((pos "L@%" ) ))
( "sempstress"  n ( s e1 m p s t r i s ) ((pos "K7$" ) ))
( "sempstresses"  n ( s e1 m p s t r i s i z ) ((pos "Kj$" ) ))
( "senate"  n ( s e1 n i t ) ((pos "K6%" ) ))
( "senates"  n ( s e1 n i t s ) ((pos "Kj%" ) ))
( "senator"  n ( s e1 n @ t @ r ) ((pos "K6%" ) ))
( "senatorial"  j ( s e2 n @ t oo1 r i@ l ) ((pos "OA%" ) ))
( "senators"  n ( s e1 n @ t @ z ) ((pos "Kj%" ) ))
( "send"  v ( s e1 n d ) ((pos "J5%" ) (subcat "2C" "3A" "6A" "12A" "13A" "15A" "15B" "19B" )))
( "send-up"  n ( s e1 n d - uh p ) ((pos "K6%" ) ))
( "send-ups"  n ( s e1 n d - uh p s ) ((pos "Kj$" ) ))
( "sender"  n ( s e1 n d @ r ) ((pos "K6%" ) ))
( "senders"  n ( s e1 n d @ z ) ((pos "Kj%" ) ))
( "sending"  v ( s e1 n d i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "12A" "13A" "15A" "15B" "19B" )))
( "sendoff"  n ( s e1 n d o f ) ((pos "K6%" ) ))
( "sendoffs"  n ( s e1 n d o f s ) ((pos "Kj$" ) ))
( "sends"  v ( s e1 n z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "12A" "13A" "15A" "15B" "19B" )))
( "senescence"  n ( s i1 n e1 s n s ) ((pos "L@$" ) ))
( "senescent"  j ( s i1 n e1 s n t ) ((pos "OA$" ) ))
( "seneschal"  n ( s e1 n i sh l ) ((pos "K6$" ) ))
( "seneschals"  n ( s e1 n i sh l z ) ((pos "Kj$" ) ))
( "senile"  j ( s ii1 n ai l ) ((pos "OA%" ) ))
( "senility"  n ( s i1 n i1 l i t ii ) ((pos "L@%" ) ))
( "senior"  n ( s ii1 n i@ r ) ((pos "K6%" "OA%" ) ))
( "seniority"  n ( s ii2 n i o1 r i t ii ) ((pos "L@%" ) ))
( "seniors"  n ( s ii1 n i@ z ) ((pos "Kj%" ) ))
( "senna"  n ( s e1 n @ ) ((pos "L@$" ) ))
( "senora"  n ( s e1 n y oo1 r @ ) ((pos "K6$" ) ))
( "senoras"  n ( s e1 n y oo1 r @ z ) ((pos "Kj$" ) ))
( "senores"  n ( s e1 n y oo1 r ei z ) ((pos "Kj$" ) ))
( "senorita"  n ( s e2 n y oo r ii1 t @ ) ((pos "K6$" ) ))
( "senoritas"  n ( s e2 n y oo r ii1 t @ z ) ((pos "Kj$" ) ))
( "sensation"  n ( s e1 n s ei1 sh @ n ) ((pos "M6%" ) ))
( "sensational"  j ( s i1 n s ei1 sh @ n l ) ((pos "OA%" ) ))
( "sensationalism"  n ( s i1 n s ei1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "sensationalist"  n ( s i1 n s ei1 sh @ n @ l i s t ) ((pos "K6%" ) ))
( "sensationalists"  n ( s i1 n s ei1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "sensationally"  a ( s i1 n s ei1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "sensations"  n ( s e1 n s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "sense"  v ( s e1 n s ) ((pos "H2%" "M6*" ) (subcat "6A" )))
( "sense-organ"  n ( s e1 n s - oo g @ n ) ((pos "K6%" ) ))
( "sense-organs"  n ( s e1 n s - oo g @ n z ) ((pos "Kj%" ) ))
( "sensed"  v ( s e1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "senseless"  j ( s e1 n s l i s ) ((pos "OA%" ) ))
( "senselessly"  a ( s e1 n s l i s l ii ) ((pos "Pu%" ) ))
( "senselessness"  n ( s e1 n s l i s n @ s ) ((pos "L@%" ) ))
( "senses"  v ( s e1 n s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "sensibilities"  n ( s e2 n s @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "sensibility"  n ( s e2 n s @ b i1 l i t ii ) ((pos "M8%" ) ))
( "sensible"  j ( s e1 n s @ b l ) ((pos "OA%" ) ))
( "sensibly"  a ( s e1 n s @ b l ii ) ((pos "Pu%" ) ))
( "sensing"  v ( s e1 n s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sensitive"  j ( s e1 n s @ t i v ) ((pos "OA%" ) ))
( "sensitively"  a ( s e1 n s @ t i v l ii ) ((pos "Pu%" ) ))
( "sensitivities"  n ( s e2 n s @ t i1 v i t i z ) ((pos "Mj%" ) ))
( "sensitivity"  n ( s e2 n s @ t i1 v i t ii ) ((pos "M8%" ) ))
( "sensitization"  n ( s e2 n s i t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "sensitize"  v ( s e1 n s i t ai z ) ((pos "H2%" ) (subcat )))
( "sensitized"  v ( s e1 n s i t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sensitizes"  v ( s e1 n s i t ai z i z ) ((pos "Ha%" ) (subcat )))
( "sensitizing"  v ( s e1 n s i t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "sensory"  j ( s e1 n s @ r ii ) ((pos "OA%" ) ))
( "sensual"  j ( s e1 n sh u@ l ) ((pos "OA%" ) ))
( "sensualism"  n ( s e1 n sh u@ l i z @ m ) ((pos "L@$" ) ))
( "sensualist"  n ( s e1 n sh u@ l i s t ) ((pos "K6%" ) ))
( "sensualists"  n ( s e1 n sh u@ l i s t s ) ((pos "Kj$" ) ))
( "sensuality"  n ( s e2 n sh u a1 l i t ii ) ((pos "L@%" ) ))
( "sensuous"  j ( s e1 n sh u@ s ) ((pos "OA%" ) ))
( "sensuously"  a ( s e1 n sh u@ s l ii ) ((pos "Pu%" ) ))
( "sensuousness"  n ( s e1 n sh u@ s n @ s ) ((pos "L@%" ) ))
( "sent"  v ( s e1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "12A" "13A" "15A" "15B" "19B" )))
( "sentence"  v ( s e1 n t @ n s ) ((pos "H2%" "K6%" ) (subcat "6A" "14" )))
( "sentenced"  v ( s e1 n t @ n s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "sentences"  v ( s e1 n t @ n s i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "sentencing"  v ( s e1 n t @ n s i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "sententious"  j ( s e1 n t e1 n sh @ s ) ((pos "OA%" ) ))
( "sententiously"  a ( s e1 n t e1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "sentient"  j ( s e1 n sh @ n t ) ((pos "OA%" ) ))
( "sentiment"  n ( s e1 n t i m @ n t ) ((pos "M6%" ) ))
( "sentimental"  j ( s e2 n t i m e1 n t l ) ((pos "OA%" ) ))
( "sentimentalist"  n ( s e2 n t i m e1 n t @ l i s t ) ((pos "K6%" ) ))
( "sentimentalists"  n ( s e2 n t i m e1 n t @ l i s t s ) ((pos "Kj$" ) ))
( "sentimentality"  n ( s e2 n t i m e n t a1 l i t ii ) ((pos "L@%" ) ))
( "sentimentalize"  v ( s e2 n t i m e1 n t @ l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "sentimentalized"  v ( s e2 n t i m e1 n t @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sentimentalizes"  v ( s e2 n t i m e1 n t @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "sentimentalizing"  v ( s e2 n t i m e1 n t @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sentimentally"  a ( s e2 n t i m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "sentiments"  n ( s e1 n t i m @ n t s ) ((pos "Mj%" ) ))
( "sentinel"  n ( s e1 n t i n l ) ((pos "K6%" ) ))
( "sentinels"  n ( s e1 n t i n l z ) ((pos "Kj%" ) ))
( "sentries"  n ( s e1 n t r i z ) ((pos "Kj%" ) ))
( "sentry"  n ( s e1 n t r ii ) ((pos "K8%" ) ))
( "sentry-box"  n ( s e1 n t r i - b o k s ) ((pos "K7%" ) ))
( "sentry-boxes"  n ( s e1 n t r i - b o k s i z ) ((pos "Kj%" ) ))
( "sentry-go"  n ( s e1 n t r i - g ou ) ((pos "Ki$" ) ))
( "sepal"  n ( s e1 p l ) ((pos "K6$" ) ))
( "sepals"  n ( s e1 p l z ) ((pos "Kj$" ) ))
( "separability"  n ( s e2 p r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "separable"  j ( s e1 p @ r @ b l ) ((pos "OA%" ) ))
( "separably"  a ( s e1 p @ r @ b l ii ) ((pos "Pu%" ) ))
( "separate"  v ( s e1 p @ r ei t ) ((pos "J2%" ) (subcat "2A" "6A" "14" )))
( "separate"  n ( s e1 p r @ t ) ((pos "K6%" "OA%" ) ))
( "separated"  v ( s e1 p @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "separately"  a ( s e1 p r @ t l ii ) ((pos "Pu%" ) ))
( "separates"  v ( s e1 p @ r ei t s ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "separates"  n ( s e1 p r @ t s ) ((pos "Kj%" ) ))
( "separating"  v ( s e1 p @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "separation"  n ( s e2 p @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "separations"  n ( s e2 p @ r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "separatist"  n ( s e1 p @ r @ t i s t ) ((pos "K6%" ) ))
( "separatists"  n ( s e1 p @ r @ t i s t s ) ((pos "Kj%" ) ))
( "separator"  n ( s e1 p @ r ei t @ r ) ((pos "K6%" ) ))
( "separators"  n ( s e1 p @ r ei t @ z ) ((pos "Kj%" ) ))
( "sepia"  n ( s ii1 p i@ ) ((pos "L@%" ) ))
( "sepsis"  n ( s e1 p s i s ) ((pos "L@$" ) ))
( "septet"  n ( s e1 p t e1 t ) ((pos "K6%" ) ))
( "septets"  n ( s e1 p t e1 t s ) ((pos "Kj%" ) ))
( "septic"  j ( s e1 p t i k ) ((pos "OA%" ) ))
( "septicaemia"  n ( s e2 p t i s ii1 m i@ ) ((pos "L@%" ) ))
( "septicemia"  n ( s e2 p t i s ii1 m i@ ) ((pos "L@%" ) ))
( "septuagenarian"  n ( s e2 p t y u@ jh i n e@1 r i@ n ) ((pos "K6%" ) ))
( "septuagenarians"  n ( s e2 p t y u@ jh i n e@1 r i@ n z ) ((pos "Kj%" ) ))
( "sepulchral"  j ( s i1 p uh1 l k r @ l ) ((pos "OA%" ) ))
( "sepulchre"  n ( s e1 p l k @ r ) ((pos "K6%" ) ))
( "sepulchres"  n ( s e1 p l k @ z ) ((pos "Kj%" ) ))
( "sepulture"  n ( s e1 p l ch u@ r ) ((pos "L@$" ) ))
( "sequel"  n ( s ii1 k w @ l ) ((pos "K6%" ) ))
( "sequels"  n ( s ii1 k w @ l z ) ((pos "Kj%" ) ))
( "sequence"  n ( s ii1 k w @ n s ) ((pos "M6%" ) ))
( "sequences"  n ( s ii1 k w @ n s i z ) ((pos "Mj%" ) ))
( "sequent"  j ( s ii1 k w @ n t ) ((pos "OA$" ) ))
( "sequential"  j ( s i1 k w e1 n sh l ) ((pos "OA%" ) ))
( "sequentially"  a ( s i1 k w e1 n sh @ l ii ) ((pos "Pu%" ) ))
( "sequester"  v ( s i1 k w e1 s t @ r ) ((pos "H0$" ) (subcat )))
( "sequestered"  v ( s i1 k w e1 s t @ d ) ((pos "Hc$" "Hd$" "OA$" ) (subcat )))
( "sequestering"  v ( s i1 k w e1 s t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "sequesters"  v ( s i1 k w e1 s t @ z ) ((pos "Ha$" ) (subcat )))
( "sequestrate"  v ( s i1 k w e1 s t r ei t ) ((pos "H2$" ) (subcat )))
( "sequestrated"  v ( s i1 k w e1 s t r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "sequestrates"  v ( s i1 k w e1 s t r ei t s ) ((pos "Ha$" ) (subcat )))
( "sequestrating"  v ( s i1 k w e1 s t r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "sequestration"  n ( s ii2 k w e s t r ei1 sh @ n ) ((pos "M6$" ) ))
( "sequestrations"  n ( s ii2 k w e s t r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "sequin"  n ( s ii1 k w i n ) ((pos "K6%" ) ))
( "sequins"  n ( s ii1 k w i n z ) ((pos "Kj%" ) ))
( "sequoia"  n ( s i1 k w oi1 @ ) ((pos "K6$" ) ))
( "sequoias"  n ( s i1 k w oi1 @ z ) ((pos "Kj$" ) ))
( "seraglio"  n ( s e1 r aa1 l i@ @ ) ((pos "K6$" ) ))
( "seraglios"  n ( s e1 r aa1 l i@ u z ) ((pos "Kj$" ) ))
( "seraph"  n ( s e1 r @ f ) ((pos "K6$" ) ))
( "seraphic"  j ( s e1 r a1 f i k ) ((pos "OA$" ) ))
( "seraphim"  n ( s e1 r @ f i m ) ((pos "Kj$" ) ))
( "seraphs"  n ( s e1 r @ f s ) ((pos "Kj$" ) ))
( "sere"  j ( s i@1 r ) ((pos "OA$" ) ))
( "serenade"  v ( s e2 r @ n ei1 d ) ((pos "H2%" "K6%" ) (subcat )))
( "serenaded"  v ( s e2 r @ n ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "serenades"  v ( s e2 r @ n ei1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "serenading"  v ( s e2 r @ n ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "serendipity"  n ( s e2 r @ n d i1 p i t ii ) ((pos "L@%" ) ))
( "serene"  j ( s i1 r ii1 n ) ((pos "OA%" ) ))
( "serenely"  a ( s i1 r ii1 n l ii ) ((pos "Pu%" ) ))
( "serenity"  n ( s i1 r e1 n i t ii ) ((pos "L@%" ) ))
( "serf"  n ( s @@1 f ) ((pos "K6$" ) ))
( "serfdom"  n ( s @@1 f d @ m ) ((pos "L@$" ) ))
( "serfs"  n ( s @@1 f s ) ((pos "Kj$" ) ))
( "serge"  n ( s @@1 jh ) ((pos "L@$" ) ))
( "sergeant"  n ( s aa1 jh @ n t ) ((pos "K6%" ) ))
( "sergeant-major"  n ( s aa2 jh @ n t - m ei1 jh @ r ) ((pos "K6%" ) ))
( "sergeant-majors"  n ( s aa2 jh @ n t - m ei1 jh @ z ) ((pos "Kj%" ) ))
( "sergeants"  n ( s aa1 jh @ n t s ) ((pos "Kj%" ) ))
( "serial"  n ( s i@1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "serialize"  v ( s i@1 r i@ l ai z ) ((pos "H2%" ) (subcat )))
( "serialized"  v ( s i@1 r i@ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "serializes"  v ( s i@1 r i@ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "serializing"  v ( s i@1 r i@ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "serially"  a ( s i@1 r i@ l ii ) ((pos "Pu%" ) ))
( "serials"  n ( s i@1 r i@ l z ) ((pos "Kj%" ) ))
( "seriatim"  a ( s i@2 r i ei1 t i m ) ((pos "Pu$" ) ))
( "sericultural"  j ( s e2 r i k uh1 l ch @ r @ l ) ((pos "OA$" ) ))
( "sericulture"  n ( s e1 r i k uh l ch @ r ) ((pos "K6$" ) ))
( "sericultures"  n ( s e1 r i k uh l ch @ z ) ((pos "Kj$" ) ))
( "sericulturist"  n ( s e2 r i k uh1 l ch @ r i s t ) ((pos "K6$" ) ))
( "sericulturists"  n ( s e2 r i k uh1 l ch @ r i s t s ) ((pos "Kj$" ) ))
( "series"  n ( s i@1 r ii z ) ((pos "K9%" ) ))
( "seriocomic"  j ( s i@2 r i@ u k o1 m i k ) ((pos "OA$" ) ))
( "serious"  j ( s i@1 r i@ s ) ((pos "OA%" ) ))
( "seriously"  a ( s i@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "seriousness"  n ( s i@1 r i@ s n @ s ) ((pos "L@%" ) ))
( "serjeant"  n ( s aa1 jh @ n t ) ((pos "K6$" ) ))
( "serjeants"  n ( s aa1 jh @ n t s ) ((pos "Kj$" ) ))
( "sermon"  n ( s @@1 m @ n ) ((pos "K6%" ) ))
( "sermonize"  v ( s @@1 m @ n ai z ) ((pos "J2%" ) (subcat "2A" )))
( "sermonized"  v ( s @@1 m @ n ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sermonizes"  v ( s @@1 m @ n ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "sermonizing"  v ( s @@1 m @ n ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sermons"  n ( s @@1 m @ n z ) ((pos "Kj%" ) ))
( "serous"  j ( s i@1 r @ s ) ((pos "OA$" ) ))
( "serpent"  n ( s @@1 p @ n t ) ((pos "K6%" ) ))
( "serpentine"  j ( s @@1 p @ n t ai n ) ((pos "OA%" ) ))
( "serpents"  n ( s @@1 p @ n t s ) ((pos "Kj%" ) ))
( "serrated"  j ( s i1 r ei1 t i d ) ((pos "OA%" ) ))
( "serried"  j ( s e1 r i d ) ((pos "OA%" ) ))
( "serum"  n ( s i@1 r @ m ) ((pos "L@%" ) ))
( "servant"  n ( s @@1 v @ n t ) ((pos "K6%" ) ))
( "servants"  n ( s @@1 v @ n t s ) ((pos "Kj%" ) ))
( "serve"  v ( s @@1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "14" "15A" )))
( "served"  v ( s @@1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "14" "15A" )))
( "server"  n ( s @@1 v @ r ) ((pos "K6%" ) ))
( "servers"  n ( s @@1 v @ z ) ((pos "Kj%" ) ))
( "serves"  v ( s @@1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "14" "15A" )))
( "service"  v ( s @@1 v i s ) ((pos "H2%" "M6%" ) (subcat )))
( "serviceable"  j ( s @@1 v i s @ b l ) ((pos "OA%" ) ))
( "serviced"  v ( s @@1 v i s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "serviceman"  n ( s @@1 v i s m @ n ) ((pos "Ki%" ) ))
( "servicemen"  n ( s @@1 v i s m e n ) ((pos "Kj%" ) ))
( "services"  v ( s @@1 v i s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "servicing"  v ( s @@1 v i s i ng ) ((pos "Hb%" ) (subcat )))
( "serviette"  n ( s @@2 v i e1 t ) ((pos "K6%" ) ))
( "serviettes"  n ( s @@2 v i e1 t s ) ((pos "Kj%" ) ))
( "servile"  j ( s @@1 v ai l ) ((pos "OA%" ) ))
( "servilely"  a ( s @@1 v ai l ii ) ((pos "Pu$" ) ))
( "servility"  n ( s @@1 v i1 l i t ii ) ((pos "L@%" ) ))
( "serving"  v ( s @@1 v i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "14" "15A" )))
( "servings"  n ( s @@1 v i ng z ) ((pos "Kj%" ) ))
( "servitor"  n ( s @@1 v i t @ r ) ((pos "K6$" ) ))
( "servitors"  n ( s @@1 v i t @ z ) ((pos "Kj$" ) ))
( "servitude"  n ( s @@1 v i t y uu d ) ((pos "L@%" ) ))
( "servo-"  j ( s @@1 v ou - ) ((pos "U-%" ) ))
( "sesame"  n ( s e1 s @ m ii ) ((pos "L@%" ) ))
( "sesquipedalian"  j ( s e2 s k w i p i d ei1 l i@ n ) ((pos "OA$" ) ))
( "session"  n ( s e1 sh @ n ) ((pos "K6%" ) ))
( "sessions"  n ( s e1 sh @ n z ) ((pos "Kj%" ) ))
( "set"  v ( s e1 t ) ((pos "J5*" "Jc*" "Jd*" "M6*" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "16A" "17" "19B" )))
( "set-square"  n ( s e1 t - s k w e@ r ) ((pos "K6%" ) ))
( "set-squares"  n ( s e1 t - s k w e@ z ) ((pos "Kj%" ) ))
( "set-to"  n ( s e1 t - t uu1 ) ((pos "K6%" ) ))
( "set-tos"  n ( s e1 t - t uu1 z ) ((pos "Kj%" ) ))
( "set-up"  n ( s e1 t - uh p ) ((pos "K6%" ) ))
( "set-ups"  n ( s e1 t - uh p s ) ((pos "Kj$" ) ))
( "setback"  n ( s e1 t b a k ) ((pos "K6%" ) ))
( "setbacks"  n ( s e1 t b a k s ) ((pos "Kj%" ) ))
( "sets"  v ( s e1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "16A" "17" "19B" )))
( "sett"  n ( s e1 t ) ((pos "K6$" ) ))
( "settee"  n ( s e1 t ii1 ) ((pos "K6%" ) ))
( "settees"  n ( s e1 t ii1 z ) ((pos "Kj%" ) ))
( "setter"  n ( s e1 t @ r ) ((pos "K6%" ) ))
( "setters"  n ( s e1 t @ z ) ((pos "Kj%" ) ))
( "setting"  v ( s e1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "12C" "14" "15A" "16A" "17" "19B" )))
( "settings"  n ( s e1 t i ng z ) ((pos "Kj%" ) ))
( "settle"  v ( s e1 t l ) ((pos "J2%" "K6$" ) (subcat "2A" "2C" "3A" "6A" "7A" "8" "10" "14" "15A" )))
( "settled"  v ( s e1 t l d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "7A" "8" "10" "14" "15A" )))
( "settlement"  n ( s e1 t l m @ n t ) ((pos "M6%" ) ))
( "settlements"  n ( s e1 t l m @ n t s ) ((pos "Mj%" ) ))
( "settler"  n ( s e1 t l @ r ) ((pos "K6%" ) ))
( "settlers"  n ( s e1 t l @ z ) ((pos "Kj%" ) ))
( "settles"  v ( s e1 t l z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2C" "3A" "6A" "7A" "8" "10" "14" "15A" )))
( "settling"  v ( s e1 t l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "7A" "8" "10" "14" "15A" )))
( "setts"  n ( s e1 t s ) ((pos "Kj$" ) ))
( "seven"  n ( s e1 v n ) ((pos "K6%" "OA%" ) ))
( "sevenfold"  j ( s e1 v @ n f ou l d ) ((pos "OA%" "Pu%" ) ))
( "sevens"  n ( s e1 v n z ) ((pos "Kj%" ) ))
( "seventeen"  n ( s e2 v n t ii1 n ) ((pos "K6%" "OA%" ) ))
( "seventeens"  n ( s e2 v n t ii1 n z ) ((pos "Kj%" ) ))
( "seventeenth"  n ( s e2 v n t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "seventeenths"  n ( s e2 v n t ii1 n th s ) ((pos "Kj%" ) ))
( "seventh"  n ( s e1 v n th ) ((pos "K6%" "OA%" ) ))
( "seventhly"  a ( s e1 v n th l ii ) ((pos "Pu$" ) ))
( "sevenths"  n ( s e1 v n th s ) ((pos "Kj%" ) ))
( "seventies"  n ( s e1 v n t i z ) ((pos "Kj%" ) ))
( "seventieth"  n ( s e1 v n t i@ th ) ((pos "K6%" "OA%" ) ))
( "seventieths"  n ( s e1 v n t i@ th s ) ((pos "Kj%" ) ))
( "seventy"  n ( s e1 v n t ii ) ((pos "K8%" "OA%" ) ))
( "sever"  v ( s e1 v @ r ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "several"  j ( s e1 v r @ l ) ((pos "OA*" "Qx*" ) ))
( "severally"  a ( s e1 v r @ l ii ) ((pos "Pu%" ) ))
( "severance"  n ( s e1 v @ r @ n s ) ((pos "L@%" ) ))
( "severe"  j ( s i1 v i@1 r ) ((pos "OB%" ) ))
( "severed"  v ( s e1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "severely"  a ( s i1 v i@1 l ii ) ((pos "Pu%" ) ))
( "severer"  j ( s i1 v i@1 r @ r ) ((pos "Or%" ) ))
( "severest"  j ( s i1 v i@1 r i s t ) ((pos "Os%" ) ))
( "severing"  v ( s e1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "severities"  n ( s i1 v e1 r i t i z ) ((pos "Mj%" ) ))
( "severity"  n ( s i1 v e1 r i t ii ) ((pos "M8%" ) ))
( "severs"  v ( s e1 v @ z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "sew"  v ( s ou1 ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "6A" )))
( "sewage"  n ( s y uu1 i jh ) ((pos "L@%" ) ))
( "sewage-farm"  n ( s y uu1 i jh - f aa m ) ((pos "K6%" ) ))
( "sewage-farms"  n ( s y uu1 i jh - f aa m z ) ((pos "Kj%" ) ))
( "sewage-works"  n ( s y uu1 i jh - w @@ k s ) ((pos "K9%" ) ))
( "sewed"  v ( s ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" )))
( "sewer"  n ( s ou1 @ r ) ((pos "K6$" ) ))
( "sewer"  n ( s y uu1 @ r ) ((pos "K6%" ) ))
( "sewer-gas"  n ( s y uu1 @ - g a s ) ((pos "L@%" ) ))
( "sewer-rat"  n ( s y uu1 @ - r a t ) ((pos "K6%" ) ))
( "sewer-rats"  n ( s y uu1 @ - r a t s ) ((pos "Kj%" ) ))
( "sewerage"  n ( s y uu1 @ r i jh ) ((pos "M6%" ) ))
( "sewerages"  n ( s y uu1 @ r i jh i z ) ((pos "Mj$" ) ))
( "sewers"  n ( s ou1 @ z ) ((pos "Kj$" ) ))
( "sewers"  n ( s y uu1 @ z ) ((pos "Kj%" ) ))
( "sewing"  v ( s ou1 i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "6A" )))
( "sewing-machine"  n ( s ou1 i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "sewing-machines"  n ( s ou1 i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "sewn"  v ( s ou1 n ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" )))
( "sews"  v ( s ou1 z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" )))
( "sex"  v ( s e1 k s ) ((pos "H1%" "M7%" ) (subcat )))
( "sex-starved"  j ( s e1 k s - s t aa v d ) ((pos "OA%" ) ))
( "sexagenarian"  n ( s e2 k s @ jh i n e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "sexagenarians"  n ( s e2 k s @ jh i n e@1 r i@ n z ) ((pos "Kj%" ) ))
( "sexed"  v ( s e1 k s t ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "sexes"  v ( s e1 k s i z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "sexier"  j ( s e1 k s i@ r ) ((pos "Or%" ) ))
( "sexiest"  j ( s e1 k s i i s t ) ((pos "Os%" ) ))
( "sexing"  v ( s e1 k s i ng ) ((pos "Hb$" ) (subcat )))
( "sexism"  n ( s e1 k s i z @ m ) ((pos "L@%" ) ))
( "sexist"  n ( s e1 k s i s t ) ((pos "K6%" "OA%" ) ))
( "sexists"  n ( s e1 k s i s t s ) ((pos "Kj%" ) ))
( "sexless"  j ( s e1 k s l @ s ) ((pos "OA%" ) ))
( "sextant"  n ( s e1 k s t @ n t ) ((pos "K6$" ) ))
( "sextants"  n ( s e1 k s t @ n t s ) ((pos "Kj$" ) ))
( "sextet"  n ( s e1 k s t e1 t ) ((pos "K6%" ) ))
( "sextets"  n ( s e1 k s t e1 t s ) ((pos "Kj%" ) ))
( "sextette"  n ( s e1 k s t e1 t ) ((pos "K6$" ) ))
( "sextettes"  n ( s e1 k s t e1 t s ) ((pos "Kj$" ) ))
( "sexton"  n ( s e1 k s t @ n ) ((pos "K6%" ) ))
( "sextons"  n ( s e1 k s t @ n z ) ((pos "Kj%" ) ))
( "sexual"  j ( s e1 k sh u@ l ) ((pos "OA%" ) ))
( "sexuality"  n ( s e2 k sh u a1 l i t ii ) ((pos "L@%" ) ))
( "sexually"  a ( s e1 k sh u@ l ii ) ((pos "Pu%" ) ))
( "sexy"  j ( s e1 k s ii ) ((pos "OD%" ) ))
( "se~nor"  n ( s e1 n y oo1 r ) ((pos "Ki%" ) ))
( "sgd"  n ( s ai1 n d ) ((pos "Y~%" ) ))
( "shabbier"  j ( sh a1 b i@ r ) ((pos "Or%" ) ))
( "shabbiest"  j ( sh a1 b i i s t ) ((pos "Os%" ) ))
( "shabbily"  a ( sh a1 b i l ii ) ((pos "Pu%" ) ))
( "shabbiness"  n ( sh a1 b i n @ s ) ((pos "L@%" ) ))
( "shabby"  j ( sh a1 b ii ) ((pos "OD%" ) ))
( "shabby-genteel"  j ( sh a2 b i - jh e n t ii1 l ) ((pos "OA$" ) ))
( "shack"  v ( sh a1 k ) ((pos "I0%" "K6%" ) (subcat )))
( "shacked"  v ( sh a1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "shacking"  v ( sh a1 k i ng ) ((pos "Ib%" ) (subcat )))
( "shackle"  v ( sh a1 k l ) ((pos "H2%" "K6%" ) (subcat )))
( "shackled"  v ( sh a1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shackles"  v ( sh a1 k l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "shackling"  v ( sh a1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "shacks"  v ( sh a1 k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "shad"  n ( sh a1 d ) ((pos "K9$" ) ))
( "shaddock"  n ( sh a1 d @ k ) ((pos "K6$" ) ))
( "shaddocks"  n ( sh a1 d @ k s ) ((pos "Kj$" ) ))
( "shade"  v ( sh ei1 d ) ((pos "J2%" "M6%" ) (subcat "2C" "6A" )))
( "shade-tree"  n ( sh ei1 d - t r ii ) ((pos "K6$" ) ))
( "shade-trees"  n ( sh ei1 d - t r ii z ) ((pos "Kj$" ) ))
( "shaded"  v ( sh ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "shades"  v ( sh ei1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "shadier"  j ( sh ei1 d i@ r ) ((pos "Or%" ) ))
( "shadiest"  j ( sh ei1 d i i s t ) ((pos "Os%" ) ))
( "shading"  v ( sh ei1 d i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "6A" )))
( "shadings"  n ( sh ei1 d i ng z ) ((pos "Mj%" ) ))
( "shadow"  v ( sh a1 d ou ) ((pos "H0%" "M6%" ) (subcat )))
( "shadow-boxing"  n ( sh a1 d ou - b o k s i ng ) ((pos "L@%" ) ))
( "shadowed"  v ( sh a1 d ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shadowier"  j ( sh a1 d ou i@ r ) ((pos "Or$" ) ))
( "shadowiest"  j ( sh a1 d ou i i s t ) ((pos "Os$" ) ))
( "shadowing"  v ( sh a1 d ou i ng ) ((pos "Hb%" ) (subcat )))
( "shadows"  v ( sh a1 d ou z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "shadowy"  j ( sh a1 d ou ii ) ((pos "OD%" ) ))
( "shady"  j ( sh ei1 d ii ) ((pos "OD%" ) ))
( "shaft"  n ( sh aa1 f t ) ((pos "K6%" ) ))
( "shafts"  n ( sh aa1 f t s ) ((pos "Kj%" ) ))
( "shag"  v ( sh a1 g ) ((pos "J4$" "L@$" ) (subcat )))
( "shagged"  v ( sh a1 g d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "shaggier"  j ( sh a1 g i@ r ) ((pos "Or%" ) ))
( "shaggiest"  j ( sh a1 g i i s t ) ((pos "Os%" ) ))
( "shaggily"  a ( sh a1 g i l ii ) ((pos "Pu%" ) ))
( "shagginess"  n ( sh a1 g i n @ s ) ((pos "L@%" ) ))
( "shagging"  v ( sh a1 g i ng ) ((pos "Jb%" "L@%" ) (subcat )))
( "shaggy"  j ( sh a1 g ii ) ((pos "OD%" ) ))
( "shags"  v ( sh a1 g z ) ((pos "Ja%" ) (subcat )))
( "shah"  n ( sh aa1 ) ((pos "K6$" ) ))
( "shahs"  n ( sh aa1 z ) ((pos "Kj$" ) ))
( "shake"  v ( sh ei1 k ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "shake-up"  n ( sh ei1 k - uh p ) ((pos "K6%" ) ))
( "shake-ups"  n ( sh ei1 k - uh p s ) ((pos "Kj$" ) ))
( "shakedown"  n ( sh ei1 k d au n ) ((pos "K6%" ) ))
( "shakedowns"  n ( sh ei1 k d au n z ) ((pos "Kj$" ) ))
( "shaken"  v ( sh ei1 k @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "shakeout"  n ( sh ei1 k au t ) ((pos "K6%" ) ))
( "shakeouts"  n ( sh ei1 k au t s ) ((pos "Kj$" ) ))
( "shaker"  n ( sh ei1 k @ r ) ((pos "K6%" ) ))
( "shakers"  n ( sh ei1 k @ z ) ((pos "Kj%" ) ))
( "shakes"  v ( sh ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "shakier"  j ( sh ei1 k i@ r ) ((pos "Or%" ) ))
( "shakiest"  j ( sh ei1 k i i s t ) ((pos "Os%" ) ))
( "shakily"  a ( sh ei1 k i l ii ) ((pos "Pu%" ) ))
( "shakiness"  n ( sh ei1 k i n @ s ) ((pos "L@%" ) ))
( "shaking"  v ( sh ei1 k i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "shakings"  n ( sh ei1 k i ng z ) ((pos "Kj%" ) ))
( "shaky"  j ( sh ei1 k ii ) ((pos "OD%" ) ))
( "shale"  n ( sh ei1 l ) ((pos "L@%" ) ))
( "shale-oil"  n ( sh ei1 l - oi l ) ((pos "L@$" ) ))
( "shall"  v ( sh a l ) ((pos "G5*" ) ))
( "shallot"  n ( sh @1 l o1 t ) ((pos "K6$" ) ))
( "shallots"  n ( sh @1 l o1 t s ) ((pos "Kj$" ) ))
( "shallow"  v ( sh a1 l ou ) ((pos "I0$" "K6%" "OC%" ) ))
( "shallowed"  v ( sh a1 l ou d ) ((pos "Ic$" "Id$" ) ))
( "shallower"  j ( sh a1 l ou @ r ) ((pos "Or%" ) ))
( "shallowest"  j ( sh a1 l ou i s t ) ((pos "Os%" ) ))
( "shallowing"  v ( sh a1 l ou i ng ) ((pos "Ib$" ) ))
( "shallows"  v ( sh a1 l ou z ) ((pos "Ia$" "Kj%" ) ))
( "shalom"  uh ( sh a1 l o1 m ) ((pos "W-$" ) ))
( "shalt"  v ( sh a l t ) ((pos "Ge$" ) ))
( "sham"  v ( sh a1 m ) ((pos "J4%" "Mi%" "OA%" ) (subcat "2A" "2D" )))
( "shamble"  v ( sh a1 m b l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "shambled"  v ( sh a1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "shambles"  v ( sh a1 m b l z ) ((pos "Ia%" "Ki%" ) (subcat "2A" )))
( "shambling"  v ( sh a1 m b l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "shame"  v ( sh ei1 m ) ((pos "H2%" "L@%" ) (subcat "6A" )))
( "shame-making"  j ( sh ei1 m - m ei k i ng ) ((pos "OA$" ) ))
( "shamed"  v ( sh ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shamefaced"  j ( sh ei1 m f ei s t ) ((pos "OA%" ) ))
( "shamefacedly"  a ( sh ei1 m f ei s t l ii ) ((pos "Pu%" ) ))
( "shameful"  j ( sh ei1 m f @ l ) ((pos "OA%" ) ))
( "shamefully"  a ( sh ei1 m f @ l ii ) ((pos "Pu%" ) ))
( "shameless"  j ( sh ei1 m l @ s ) ((pos "OA%" ) ))
( "shamelessly"  a ( sh ei1 m l @ s l ii ) ((pos "Pu%" ) ))
( "shamelessness"  n ( sh ei1 m l @ s n @ s ) ((pos "L@%" ) ))
( "shames"  v ( sh ei1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "shaming"  v ( sh ei1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shammed"  v ( sh a1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2D" )))
( "shammies"  n ( sh a1 m i z ) ((pos "Kj$" ) ))
( "shamming"  v ( sh a1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2D" )))
( "shammy"  n ( sh a1 m ii ) ((pos "K8$" ) ))
( "shampoo"  v ( sh a1 m p uu1 ) ((pos "H0%" "M6%" ) (subcat )))
( "shampooed"  v ( sh a1 m p uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shampooing"  v ( sh a1 m p uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "shampoos"  v ( sh a1 m p uu1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "shamrock"  n ( sh a1 m r o k ) ((pos "K6%" ) ))
( "shamrocks"  n ( sh a1 m r o k s ) ((pos "Kj%" ) ))
( "shams"  v ( sh a1 m z ) ((pos "Ja$" ) (subcat "2A" "2D" )))
( "shan't"  v ( sh aa n t ) ((pos "Gg%" ) ))
( "shandies"  n ( sh a1 n d i z ) ((pos "Mj%" ) ))
( "shandy"  n ( sh a1 n d ii ) ((pos "M8%" ) ))
( "shanghai"  v ( sh a1 ng h ai1 ) ((pos "H0$" ) ))
( "shanghaied"  v ( sh a1 ng h ai1 d ) ((pos "Hc$" "Hd$" ) ))
( "shanghaiing"  v ( sh a1 ng h ai1 i ng ) ((pos "Hb$" ) ))
( "shanghais"  v ( sh a1 ng h ai1 z ) ((pos "Ha$" ) ))
( "shank"  n ( sh a1 ng k ) ((pos "K6%" ) ))
( "shanks"  n ( sh a1 ng k s ) ((pos "Kj%" ) ))
( "shanties"  n ( sh a1 n t i z ) ((pos "Kj$" ) ))
( "shantung"  n ( sh a1 n t uh1 ng ) ((pos "L@%" ) ))
( "shanty"  n ( sh a1 n t ii ) ((pos "K8%" ) ))
( "shantytown"  n ( sh a1 n t i t au n ) ((pos "K6%" ) ))
( "shantytowns"  n ( sh a1 n t i t au n z ) ((pos "Kj%" ) ))
( "shape"  v ( sh ei1 p ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "shaped"  v ( sh ei1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "shapeless"  j ( sh ei1 p l @ s ) ((pos "OA%" ) ))
( "shapelessly"  a ( sh ei1 p l @ s l ii ) ((pos "Pu%" ) ))
( "shapelessness"  n ( sh ei1 p l @ s n @ s ) ((pos "L@%" ) ))
( "shapelier"  j ( sh ei1 p l i@ r ) ((pos "Or%" ) ))
( "shapeliest"  j ( sh ei1 p l i i s t ) ((pos "Os%" ) ))
( "shapely"  j ( sh ei1 p l ii ) ((pos "OD%" ) ))
( "shapes"  v ( sh ei1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "shaping"  v ( sh ei1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "shard"  n ( sh aa1 d ) ((pos "K6$" ) ))
( "shards"  n ( sh aa1 d z ) ((pos "Kj$" ) ))
( "share"  v ( sh e@1 r ) ((pos "J2%" "M6%" ) (subcat "3A" "6A" "14" )))
( "share-out"  n ( sh e@1 r - au t ) ((pos "K6%" ) ))
( "share-outs"  n ( sh e@1 r - au t s ) ((pos "Kj%" ) ))
( "sharecropper"  n ( sh e@1 k r o p @ r ) ((pos "K6$" ) ))
( "sharecroppers"  n ( sh e@1 k r o p @ z ) ((pos "Kj$" ) ))
( "shared"  v ( sh e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" )))
( "shareholder"  n ( sh e@1 h ou l d @ r ) ((pos "K6%" ) ))
( "shareholders"  n ( sh e@1 h ou l d @ z ) ((pos "Kj%" ) ))
( "shareholding"  n ( sh e@1 h ou l d i ng ) ((pos "K6%" ) ))
( "shareholdings"  n ( sh e@1 h ou l d i ng z ) ((pos "Kj$" ) ))
( "shares"  v ( sh e@1 z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "14" )))
( "sharing"  v ( sh e@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" )))
( "shark"  n ( sh aa1 k ) ((pos "K6%" ) ))
( "sharks"  n ( sh aa1 k s ) ((pos "Kj%" ) ))
( "sharkskin"  n ( sh aa1 k s k i n ) ((pos "M6%" ) ))
( "sharkskins"  n ( sh aa1 k s k i n z ) ((pos "Mj$" ) ))
( "sharp"  n ( sh aa1 p ) ((pos "K6%" "OC%" "Pu%" ) ))
( "sharp-eyed"  j ( sh aa1 p - ai d ) ((pos "OA%" ) ))
( "sharp-set"  j ( sh aa1 p - s e t ) ((pos "OA$" ) ))
( "sharp-sighted"  j ( sh aa1 p - s ai1 t i d ) ((pos "OA%" ) ))
( "sharp-witted"  j ( sh aa1 p - w i1 t i d ) ((pos "OA%" ) ))
( "sharpen"  v ( sh aa1 p @ n ) ((pos "J0%" ) (subcat "2A" )))
( "sharpened"  v ( sh aa1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sharpener"  n ( sh aa1 p n @ r ) ((pos "K6%" ) ))
( "sharpeners"  n ( sh aa1 p n @ z ) ((pos "Kj%" ) ))
( "sharpening"  v ( sh aa1 p @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sharpens"  v ( sh aa1 p @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "sharper"  n ( sh aa1 p @ r ) ((pos "K6$" "Or%" ) ))
( "sharpers"  n ( sh aa1 p @ z ) ((pos "Kj$" ) ))
( "sharpest"  j ( sh aa1 p i s t ) ((pos "Os%" ) ))
( "sharply"  a ( sh aa1 p l ii ) ((pos "Pu%" ) ))
( "sharpness"  n ( sh aa1 p n @ s ) ((pos "L@%" ) ))
( "sharps"  n ( sh aa1 p s ) ((pos "Kj$" ) ))
( "sharpshooter"  n ( sh aa1 p sh uu t @ r ) ((pos "K6%" ) ))
( "sharpshooters"  n ( sh aa1 p sh uu t @ z ) ((pos "Kj%" ) ))
( "shat"  v ( sh a1 t ) ((pos "Ic%" ) (subcat "2A" )))
( "shatter"  v ( sh a1 t @ r ) ((pos "J0%" ) (subcat "2A" )))
( "shattered"  v ( sh a1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "shattering"  v ( sh a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "shatterproof"  j ( sh a1 t @ p r uu f ) ((pos "OA%" ) ))
( "shatters"  v ( sh a1 t @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "shave"  v ( sh ei1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" "15A" )))
( "shaved"  v ( sh ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "shaven"  v ( sh ei1 v n ) ((pos "Jd%" ) (subcat "2A" "6A" "15A" )))
( "shaver"  n ( sh ei1 v @ r ) ((pos "K6%" ) ))
( "shavers"  n ( sh ei1 v @ z ) ((pos "Kj%" ) ))
( "shaves"  v ( sh ei1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "shaving"  v ( sh ei1 v i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "shaving-brush"  n ( sh ei1 v i ng - b r uh sh ) ((pos "K7%" ) ))
( "shaving-brushes"  n ( sh ei1 v i ng - b r uh sh i z ) ((pos "Kj%" ) ))
( "shavings"  n ( sh ei1 v i ng z ) ((pos "Kj%" ) ))
( "shawl"  n ( sh oo1 l ) ((pos "K6%" ) ))
( "shawls"  n ( sh oo1 l z ) ((pos "Kj%" ) ))
( "she"  prp ( sh ii ) ((pos "Qx*" ) ))
( "she'd"  v ( sh ii d ) ((pos "Gf%" ) ))
( "she'll"  v ( sh ii l ) ((pos "Gf%" ) ))
( "she's"  v ( sh ii z ) ((pos "Gf%" ) ))
( "she-goat"  n ( sh ii1 - g ou t ) ((pos "K6%" ) ))
( "she-goats"  n ( sh ii1 - g ou t s ) ((pos "Kj%" ) ))
( "sheaf"  n ( sh ii1 f ) ((pos "Ki%" ) ))
( "shear"  v ( sh i@1 r ) ((pos "H0%" ) (subcat )))
( "sheared"  v ( sh i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shearing"  v ( sh i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "shears"  v ( sh i@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sheath"  n ( sh ii1 th ) ((pos "K6%" ) ))
( "sheath-knife"  n ( sh ii1 th - n ai f ) ((pos "Ki%" ) ))
( "sheath-knives"  n ( sh ii1 th - n ai v z ) ((pos "Kj%" ) ))
( "sheathe"  v ( sh ii1 dh ) ((pos "H2%" ) (subcat )))
( "sheathed"  v ( sh ii1 dh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sheathes"  v ( sh ii1 dh z ) ((pos "Ha%" ) (subcat )))
( "sheathing"  v ( sh ii1 dh i ng ) ((pos "Hb%" "K6$" ) (subcat )))
( "sheathings"  n ( sh ii1 dh i ng z ) ((pos "Kj$" ) ))
( "sheaths"  n ( sh ii1 dh z ) ((pos "Kj%" ) ))
( "sheaves"  n ( sh ii1 v z ) ((pos "Kj%" ) ))
( "shebang"  n ( sh i1 b a1 ng ) ((pos "Ki$" ) ))
( "shebeen"  n ( sh i1 b ii1 n ) ((pos "K6$" ) ))
( "shebeens"  n ( sh i1 b ii1 n z ) ((pos "Kj$" ) ))
( "shed"  v ( sh e1 d ) ((pos "H5%" "Hc%" "Hd%" "K6%" ) (subcat )))
( "shedding"  v ( sh e1 d i ng ) ((pos "Hb%" ) (subcat )))
( "sheds"  v ( sh e1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sheen"  n ( sh ii1 n ) ((pos "L@%" ) ))
( "sheep"  n ( sh ii1 p ) ((pos "K9%" ) ))
( "sheepdog"  n ( sh ii1 p d o g ) ((pos "K6%" ) ))
( "sheepdogs"  n ( sh ii1 p d o g z ) ((pos "Kj%" ) ))
( "sheepfold"  n ( sh ii1 p f ou l d ) ((pos "K6%" ) ))
( "sheepfolds"  n ( sh ii1 p f ou l d z ) ((pos "Kj%" ) ))
( "sheepish"  j ( sh ii1 p i sh ) ((pos "OA%" ) ))
( "sheepishly"  a ( sh ii1 p i sh l ii ) ((pos "Pu%" ) ))
( "sheepishness"  n ( sh ii1 p i sh n @ s ) ((pos "L@%" ) ))
( "sheeprun"  n ( sh ii1 p r uh n ) ((pos "K6%" ) ))
( "sheepruns"  n ( sh ii1 p r uh n z ) ((pos "Kj%" ) ))
( "sheepskin"  n ( sh ii1 p s k i n ) ((pos "K6%" ) ))
( "sheepskins"  n ( sh ii1 p s k i n z ) ((pos "Kj%" ) ))
( "sheer"  v ( sh i@1 r ) ((pos "I0%" "OC%" "Pu%" ) (subcat )))
( "sheered"  v ( sh i@1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sheerer"  j ( sh i@1 r @ r ) ((pos "Or$" ) ))
( "sheerest"  j ( sh i@1 r i s t ) ((pos "Os$" ) ))
( "sheering"  v ( sh i@1 r i ng ) ((pos "Ib%" ) (subcat )))
( "sheers"  v ( sh i@1 z ) ((pos "Ia%" ) (subcat )))
( "sheet"  n ( sh ii1 t ) ((pos "K6%" ) ))
( "sheet-anchor"  n ( sh ii1 t - a1 ng k @ r ) ((pos "K6%" ) ))
( "sheet-anchors"  n ( sh ii1 t - a1 ng k @ z ) ((pos "Kj%" ) ))
( "sheet-lightning"  n ( sh ii1 t - l ai1 t n i ng ) ((pos "L@%" ) ))
( "sheeting"  n ( sh ii1 t i ng ) ((pos "L@%" ) ))
( "sheets"  n ( sh ii1 t s ) ((pos "Kj%" ) ))
( "sheik"  n ( sh ei1 k ) ((pos "K6%" ) ))
( "sheikdom"  n ( sh ei1 k d @ m ) ((pos "K6%" ) ))
( "sheikdoms"  n ( sh ei1 k d @ m z ) ((pos "Kj%" ) ))
( "sheikh"  n ( sh ei1 k ) ((pos "K6%" ) ))
( "sheikhdom"  n ( sh ei1 k d @ m ) ((pos "K6$" ) ))
( "sheikhdoms"  n ( sh ei1 k d @ m z ) ((pos "Kj$" ) ))
( "sheikhs"  n ( sh ei1 k s ) ((pos "Kj%" ) ))
( "sheiks"  n ( sh ei1 k s ) ((pos "Kj%" ) ))
( "shekel"  n ( sh e1 k l ) ((pos "K6$" ) ))
( "shekels"  n ( sh e1 k l z ) ((pos "Kj$" ) ))
( "sheldrake"  n ( sh e1 l d r ei k ) ((pos "K6$" ) ))
( "sheldrakes"  n ( sh e1 l d r ei k s ) ((pos "Kj$" ) ))
( "shelf"  n ( sh e1 l f ) ((pos "Ki%" ) ))
( "shell"  v ( sh e1 l ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "shell-shock"  n ( sh e1 l - sh o k ) ((pos "L@%" ) ))
( "shellac"  v ( sh @1 l a1 k ) ((pos "H5$" "L@%" ) ))
( "shellacked"  v ( sh @1 l a1 k t ) ((pos "Hc$" "Hd$" ) ))
( "shellacking"  v ( sh @1 l a1 k i ng ) ((pos "Hb$" ) ))
( "shellacs"  v ( sh @1 l a1 k s ) ((pos "Ha$" ) ))
( "shelled"  v ( sh e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "shellfire"  n ( sh e1 l f ai @ r ) ((pos "L@%" ) ))
( "shellfish"  n ( sh e1 l f i sh ) ((pos "K9%" ) ))
( "shelling"  v ( sh e1 l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "shellproof"  j ( sh e1 l p r uu f ) ((pos "OA%" ) ))
( "shells"  v ( sh e1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "shelter"  v ( sh e1 l t @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "sheltered"  v ( sh e1 l t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "sheltering"  v ( sh e1 l t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "shelters"  v ( sh e1 l t @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "shelve"  v ( sh e1 l v ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "shelved"  v ( sh e1 l v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "shelves"  v ( sh e1 l v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "shelving"  v ( sh e1 l v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "shepherd"  v ( sh e1 p @ d ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "shepherded"  v ( sh e1 p @ d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shepherdess"  n ( sh e2 p @ d e1 s ) ((pos "K7%" ) ))
( "shepherdesses"  n ( sh e2 p @ d e1 s i z ) ((pos "Kj%" ) ))
( "shepherding"  v ( sh e1 p @ d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shepherds"  v ( sh e1 p @ d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "sherbet"  n ( sh @@1 b @ t ) ((pos "M6%" ) ))
( "sherbets"  n ( sh @@1 b @ t s ) ((pos "Mj$" ) ))
( "sheriff"  n ( sh e1 r i f ) ((pos "K6%" ) ))
( "sheriffs"  n ( sh e1 r i f s ) ((pos "Kj%" ) ))
( "sherries"  n ( sh e1 r i z ) ((pos "Mj%" ) ))
( "sherry"  n ( sh e1 r ii ) ((pos "M8%" ) ))
( "shew"  v ( sh ou1 ) ((pos "J0$" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shewed"  v ( sh ou1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shewing"  v ( sh ou1 i ng ) ((pos "Jb$" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shewn"  v ( sh ou1 n ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shews"  v ( sh ou1 z ) ((pos "Ja$" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shibboleth"  n ( sh i1 b @ l e th ) ((pos "K6$" ) ))
( "shibboleths"  n ( sh i1 b @ l e th s ) ((pos "Kj$" ) ))
( "shied"  v ( sh ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "shield"  v ( sh ii1 l d ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "shielded"  v ( sh ii1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shielding"  v ( sh ii1 l d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shields"  v ( sh ii1 l d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "shies"  v ( sh ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "shift"  v ( sh i1 f t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "shifted"  v ( sh i1 f t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "shiftier"  j ( sh i1 f t i@ r ) ((pos "Or%" ) ))
( "shiftiest"  j ( sh i1 f t i i s t ) ((pos "Os%" ) ))
( "shiftily"  a ( sh i1 f t i l ii ) ((pos "Pu%" ) ))
( "shiftiness"  n ( sh i1 f t i n @ s ) ((pos "L@%" ) ))
( "shifting"  v ( sh i1 f t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "shiftless"  j ( sh i1 f t l @ s ) ((pos "OA%" ) ))
( "shifts"  v ( sh i1 f t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "shifty"  j ( sh i1 f t ii ) ((pos "OD%" ) ))
( "shillelagh"  n ( sh i1 l ei1 l @ ) ((pos "K6$" ) ))
( "shillelaghs"  n ( sh i1 l ei1 l @ z ) ((pos "Kj$" ) ))
( "shilling"  n ( sh i1 l i ng ) ((pos "K6%" ) ))
( "shillings"  n ( sh i1 l i ng z ) ((pos "Kj%" ) ))
( "shillyshallied"  v ( sh i1 l i sh a l i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "shillyshallies"  v ( sh i1 l i sh a l i z ) ((pos "Ia%" ) (subcat )))
( "shillyshally"  v ( sh i1 l i sh a l ii ) ((pos "I3%" "L@%" ) (subcat )))
( "shillyshallying"  v ( sh i1 l i sh a l i i ng ) ((pos "Ib%" ) (subcat )))
( "shimmer"  v ( sh i1 m @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "shimmered"  v ( sh i1 m @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "shimmering"  v ( sh i1 m @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "shimmers"  v ( sh i1 m @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "shin"  v ( sh i1 n ) ((pos "I4%" "K6%" ) (subcat )))
( "shinbone"  n ( sh i1 n b ou n ) ((pos "K6%" ) ))
( "shinbones"  n ( sh i1 n b ou n z ) ((pos "Kj%" ) ))
( "shindies"  n ( sh i1 n d i z ) ((pos "Kj%" ) ))
( "shindig"  n ( sh i1 n d i g ) ((pos "K6%" ) ))
( "shindigs"  n ( sh i1 n d i g z ) ((pos "Kj%" ) ))
( "shindy"  n ( sh i1 n d ii ) ((pos "K8%" ) ))
( "shine"  v ( sh ai1 n ) ((pos "J5%" "L@%" ) (subcat "2A" "2C" )))
( "shines"  v ( sh ai1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "shingle"  v ( sh i1 ng g l ) ((pos "H2$" "M6%" ) (subcat )))
( "shingled"  v ( sh i1 ng g l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "shingles"  v ( sh i1 ng g l z ) ((pos "Ha$" "Lk%" "Mj%" ) (subcat )))
( "shinglier"  j ( sh i1 ng g l i@ r ) ((pos "Or$" ) ))
( "shingliest"  j ( sh i1 ng g l i i s t ) ((pos "Os$" ) ))
( "shingling"  v ( sh i1 ng g l i ng ) ((pos "Hb$" ) (subcat )))
( "shingly"  j ( sh i1 ng g l ii ) ((pos "OD%" ) ))
( "shinguard"  n ( sh i1 n g aa d ) ((pos "K6%" ) ))
( "shinguards"  n ( sh i1 n g aa d z ) ((pos "Kj%" ) ))
( "shinier"  j ( sh ai1 n i@ r ) ((pos "Or%" ) ))
( "shiniest"  j ( sh ai1 n i i s t ) ((pos "Os%" ) ))
( "shining"  v ( sh ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "shinned"  v ( sh i1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "shinning"  v ( sh i1 n i ng ) ((pos "Ib%" ) (subcat )))
( "shins"  v ( sh i1 n z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "shiny"  j ( sh ai1 n ii ) ((pos "OD%" ) ))
( "ship"  v ( sh i1 p ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "ship's-chandler"  n ( sh ii1 p s - ch aa1 n d l @ r ) ((pos "K6%" ) ))
( "ship's-chandlers"  n ( sh ii1 p s - ch aa1 n d l @ z ) ((pos "Kj%" ) ))
( "ship-breaker"  n ( sh i1 p - b r ei k @ r ) ((pos "K6%" ) ))
( "ship-breakers"  n ( sh i1 p - b r ei k @ z ) ((pos "Kj%" ) ))
( "ship-canal"  n ( sh i1 p - k @ n a l ) ((pos "K6%" ) ))
( "ship-canals"  n ( sh i1 p - k @ n a l z ) ((pos "Kj%" ) ))
( "shipboard"  j ( sh i1 p b oo d ) ((pos "Oq%" ) ))
( "shipbroker"  n ( sh i1 p b r ou k @ r ) ((pos "K6$" ) ))
( "shipbrokers"  n ( sh i1 p b r ou k @ z ) ((pos "Kj$" ) ))
( "shipbuilder"  n ( sh i1 p b i l d @ r ) ((pos "K6%" ) ))
( "shipbuilders"  n ( sh i1 p b i l d @ z ) ((pos "Kj%" ) ))
( "shipbuilding"  n ( sh i1 p b i l d i ng ) ((pos "L@%" ) ))
( "shipload"  n ( sh i1 p l ou d ) ((pos "K6%" ) ))
( "shiploads"  n ( sh i1 p l ou d z ) ((pos "Kj%" ) ))
( "shipmate"  n ( sh i1 p m ei t ) ((pos "K6%" ) ))
( "shipmates"  n ( sh i1 p m ei t s ) ((pos "Kj%" ) ))
( "shipment"  n ( sh i1 p m @ n t ) ((pos "M6%" ) ))
( "shipments"  n ( sh i1 p m @ n t s ) ((pos "Mj%" ) ))
( "shipowner"  n ( sh i1 p ou n @ r ) ((pos "K6%" ) ))
( "shipowners"  n ( sh i1 p ou n @ z ) ((pos "Kj%" ) ))
( "shipped"  v ( sh i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "shipper"  n ( sh i1 p @ r ) ((pos "K6%" ) ))
( "shippers"  n ( sh i1 p @ z ) ((pos "Kj%" ) ))
( "shipping"  v ( sh i1 p i ng ) ((pos "Jb%" "L@%" ) (subcat "2C" "6A" "15A" )))
( "shipping-agent"  n ( sh i1 p i ng - ei jh @ n t ) ((pos "K6%" ) ))
( "shipping-agents"  n ( sh i1 p i ng - ei jh @ n t s ) ((pos "Kj%" ) ))
( "shipping-office"  n ( sh i1 p i ng - o f i s ) ((pos "K6%" ) ))
( "shipping-offices"  n ( sh i1 p i ng - o f i s i z ) ((pos "Kj%" ) ))
( "ships"  v ( sh i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "shipshape"  j ( sh i1 p sh ei p ) ((pos "OA%" "Pu%" ) ))
( "shipway"  n ( sh i1 p w ei ) ((pos "K6%" ) ))
( "shipways"  n ( sh i1 p w ei z ) ((pos "Kj%" ) ))
( "shipwreck"  v ( sh i1 p r e k ) ((pos "I0$" "M6%" ) ))
( "shipwrecked"  v ( sh i1 p r e k t ) ((pos "Ic$" "Id%" ) ))
( "shipwrecking"  v ( sh i1 p r e k i ng ) ((pos "Ib$" ) ))
( "shipwrecks"  v ( sh i1 p r e k s ) ((pos "Ia$" "Mj%" ) ))
( "shipwright"  n ( sh i1 p r ai t ) ((pos "K6%" ) ))
( "shipwrights"  n ( sh i1 p r ai t s ) ((pos "Kj%" ) ))
( "shipyard"  n ( sh i1 p y aa d ) ((pos "K6%" ) ))
( "shipyards"  n ( sh i1 p y aa d z ) ((pos "Kj%" ) ))
( "shire"  n ( sh ai1 @ r ) ((pos "K6%" ) ))
( "shires"  n ( sh ai1 @ z ) ((pos "Kj%" ) ))
( "shirk"  v ( sh @@1 k ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "shirked"  v ( sh @@1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "shirker"  n ( sh @@1 k @ r ) ((pos "K6%" ) ))
( "shirkers"  n ( sh @@1 k @ z ) ((pos "Kj%" ) ))
( "shirking"  v ( sh @@1 k i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "shirks"  v ( sh @@1 k s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "shirt"  n ( sh @@1 t ) ((pos "K6%" ) ))
( "shirt-front"  n ( sh @@1 t - f r uh1 n t ) ((pos "K6%" ) ))
( "shirt-fronts"  n ( sh @@1 t - f r uh1 n t s ) ((pos "Kj%" ) ))
( "shirtier"  j ( sh @@1 t i@ r ) ((pos "Or$" ) ))
( "shirtiest"  j ( sh @@1 t i i s t ) ((pos "Os$" ) ))
( "shirting"  n ( sh @@1 t i ng ) ((pos "L@$" ) ))
( "shirts"  n ( sh @@1 t s ) ((pos "Kj%" ) ))
( "shirtsleeves"  n ( sh @@1 t s l ii v z ) ((pos "Kj%" ) ))
( "shirtwaist"  n ( sh @@1 t w ei1 s t ) ((pos "K6%" ) ))
( "shirtwaister"  n ( sh @@1 t w ei1 s t @ r ) ((pos "K6%" ) ))
( "shirtwaisters"  n ( sh @@1 t w ei1 s t @ z ) ((pos "Kj%" ) ))
( "shirtwaists"  n ( sh @@1 t w ei1 s t s ) ((pos "Kj%" ) ))
( "shirty"  j ( sh @@1 t ii ) ((pos "OD%" ) ))
( "shish kebab"  n ( sh i2 sh - k @ b a1 b ) ((pos "K6%" ) ))
( "shish kebabs"  n ( sh i2 sh - k @ b a1 b z ) ((pos "Kj%" ) ))
( "shit"  v ( sh i1 t ) ((pos "I4%" "L@%" "W-%" ) (subcat "2A" )))
( "shits"  v ( sh i1 t s ) ((pos "Ia%" ) (subcat "2A" )))
( "shitted"  v ( sh i1 t i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "shitting"  v ( sh i1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "shiver"  v ( sh i1 v @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "shivered"  v ( sh i1 v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "shivering"  v ( sh i1 v @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "shivers"  v ( sh i1 v @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "shivery"  j ( sh i1 v @ r ii ) ((pos "OA%" ) ))
( "shoal"  v ( sh ou1 l ) ((pos "I0$" "K6%" ) (subcat )))
( "shoaled"  v ( sh ou1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "shoaling"  v ( sh ou1 l i ng ) ((pos "Ib$" ) (subcat )))
( "shoals"  v ( sh ou1 l z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "shock"  v ( sh o1 k ) ((pos "H0%" "M6%" "Pu%" ) (subcat )))
( "shock-brigade"  n ( sh o1 k - b r i g ei d ) ((pos "K6$" ) ))
( "shock-brigades"  n ( sh o1 k - b r i g ei d z ) ((pos "Kj$" ) ))
( "shock-headed"  j ( sh o1 k - h e1 d i d ) ((pos "OA$" ) ))
( "shock-worker"  n ( sh o1 k - w @@ k @ r ) ((pos "K6$" ) ))
( "shock-workers"  n ( sh o1 k - w @@ k @ z ) ((pos "Kj$" ) ))
( "shockable"  j ( sh o1 k @ b l ) ((pos "OA%" ) ))
( "shocked"  v ( sh o1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shocker"  n ( sh o1 k @ r ) ((pos "K6%" ) ))
( "shockers"  n ( sh o1 k @ z ) ((pos "Kj%" ) ))
( "shocking"  v ( sh o1 k i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "shockingly"  a ( sh o1 k i ng l ii ) ((pos "Pu%" ) ))
( "shocks"  v ( sh o1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "shod"  v ( sh o1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shoddier"  j ( sh o1 d i@ r ) ((pos "Or%" ) ))
( "shoddiest"  j ( sh o1 d i i s t ) ((pos "Os%" ) ))
( "shoddiness"  n ( sh o1 d i n i s ) ((pos "L@%" ) ))
( "shoddy"  n ( sh o1 d ii ) ((pos "L@%" "OD%" ) ))
( "shoe"  v ( sh uu1 ) ((pos "H5%" "K6%" ) (subcat )))
( "shoe-leather"  n ( sh uu1 - l e dh @ r ) ((pos "M6%" ) ))
( "shoe-leathers"  n ( sh uu1 - l e dh @ z ) ((pos "Mj$" ) ))
( "shoeblack"  n ( sh uu1 b l a k ) ((pos "L@$" ) ))
( "shoehorn"  n ( sh uu1 h oo n ) ((pos "K6%" ) ))
( "shoehorns"  n ( sh uu1 h oo n z ) ((pos "Kj%" ) ))
( "shoeing"  v ( sh uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "shoelace"  n ( sh uu1 l ei s ) ((pos "K6%" ) ))
( "shoelaces"  n ( sh uu1 l ei s i z ) ((pos "Kj%" ) ))
( "shoemaker"  n ( sh uu1 m ei k @ r ) ((pos "K6%" ) ))
( "shoemakers"  n ( sh uu1 m ei k @ z ) ((pos "Kj%" ) ))
( "shoemaking"  n ( sh uu1 m ei k i ng ) ((pos "L@%" ) ))
( "shoes"  v ( sh uu1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "shoestring"  n ( sh uu1 s t r i ng ) ((pos "K6%" ) ))
( "shoestrings"  n ( sh uu1 s t r i ng z ) ((pos "Kj%" ) ))
( "shoetree"  n ( sh uu1 t r ii ) ((pos "K6$" ) ))
( "shoetrees"  n ( sh uu1 t r ii z ) ((pos "Kj$" ) ))
( "shogun"  n ( sh ou1 g uu n ) ((pos "K6$" ) ))
( "shoguns"  n ( sh ou1 g uu n z ) ((pos "Kj$" ) ))
( "shone"  v ( sh o1 n ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "shoo"  v ( sh uu1 ) ((pos "J0%" ) (subcat )))
( "shooed"  v ( sh uu1 d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "shooing"  v ( sh uu1 i ng ) ((pos "Jb%" ) (subcat )))
( "shook"  v ( sh u1 k ) ((pos "Jc%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "shoos"  v ( sh uu1 z ) ((pos "Ja%" ) (subcat )))
( "shoot"  v ( sh uu1 t ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "shooter"  n ( sh uu1 t @ r ) ((pos "K6%" ) ))
( "shooters"  n ( sh uu1 t @ z ) ((pos "Kj%" ) ))
( "shooting"  v ( sh uu1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "shooting-box"  n ( sh uu1 t i ng - b o k s ) ((pos "K7$" ) ))
( "shooting-boxes"  n ( sh uu1 t i ng - b o k s i z ) ((pos "Kj$" ) ))
( "shooting-brake"  n ( sh uu1 t i ng - b r ei k ) ((pos "K6%" ) ))
( "shooting-brakes"  n ( sh uu1 t i ng - b r ei k s ) ((pos "Kj%" ) ))
( "shooting-galleries"  n ( sh uu1 t i ng - g a l @ r i z ) ((pos "Kj%" ) ))
( "shooting-gallery"  n ( sh uu1 t i ng - g a l @ r ii ) ((pos "K8%" ) ))
( "shooting-range"  n ( sh uu1 t i ng - r ei n jh ) ((pos "K6%" ) ))
( "shooting-ranges"  n ( sh uu1 t i ng - r ei n jh i z ) ((pos "Kj%" ) ))
( "shooting-stick"  n ( sh uu1 t i ng - s t i k ) ((pos "K6%" ) ))
( "shooting-sticks"  n ( sh uu1 t i ng - s t i k s ) ((pos "Kj%" ) ))
( "shoots"  v ( sh uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "shop"  v ( sh o1 p ) ((pos "I4%" "M6%" ) (subcat "2A" )))
( "shop-assistant"  n ( sh o1 p - @ s i s t @ n t ) ((pos "K6%" ) ))
( "shop-assistants"  n ( sh o1 p - @ s i s t @ n t s ) ((pos "Kj%" ) ))
( "shop-bell"  n ( sh o1 p - b e l ) ((pos "K6%" ) ))
( "shop-bells"  n ( sh o1 p - b e l z ) ((pos "Kj%" ) ))
( "shop-boy"  n ( sh o1 p - b oi ) ((pos "K6$" ) ))
( "shop-boys"  n ( sh o1 p - b oi z ) ((pos "Kj$" ) ))
( "shop-front"  n ( sh o1 p - f r uh1 n t ) ((pos "K6%" ) ))
( "shop-fronts"  n ( sh o1 p - f r uh1 n t s ) ((pos "Kj%" ) ))
( "shop-girl"  n ( sh o1 p - g @@ l ) ((pos "K6$" ) ))
( "shop-girls"  n ( sh o1 p - g @@ l z ) ((pos "Kj$" ) ))
( "shop-steward"  n ( sh o1 p - s t y u@1 d ) ((pos "K6%" ) ))
( "shop-stewards"  n ( sh o1 p - s t y u@1 d z ) ((pos "Kj%" ) ))
( "shopkeeper"  n ( sh o1 p k ii p @ r ) ((pos "K6%" ) ))
( "shopkeepers"  n ( sh o1 p k ii p @ z ) ((pos "Kj%" ) ))
( "shoplift"  v ( sh o1 p l i f t ) ((pos "J0$" ) (subcat "2A" )))
( "shoplifted"  v ( sh o1 p l i f t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "shoplifter"  n ( sh o1 p l i f t @ r ) ((pos "K6%" ) ))
( "shoplifters"  n ( sh o1 p l i f t @ z ) ((pos "Kj%" ) ))
( "shoplifting"  v ( sh o1 p l i f t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "shoplifts"  v ( sh o1 p l i f t s ) ((pos "Ja$" ) (subcat "2A" )))
( "shopped"  v ( sh o1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "shopper"  n ( sh o1 p @ r ) ((pos "K6%" ) ))
( "shoppers"  n ( sh o1 p @ z ) ((pos "Kj%" ) ))
( "shopping"  v ( sh o1 p i ng ) ((pos "Ib%" "L@%" ) (subcat "2A" )))
( "shops"  v ( sh o1 p s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "shopsoiled"  j ( sh o1 p s oi l d ) ((pos "OA%" ) ))
( "shopwalker"  n ( sh o1 p w oo k @ r ) ((pos "K6$" ) ))
( "shopwalkers"  n ( sh o1 p w oo k @ z ) ((pos "Kj$" ) ))
( "shopwindow"  n ( sh o1 p w i1 n d ou ) ((pos "K6%" ) ))
( "shopwindows"  n ( sh o1 p w i1 n d ou z ) ((pos "Kj%" ) ))
( "shopworn"  j ( sh o1 p w oo n ) ((pos "OA%" ) ))
( "shore"  v ( sh oo1 r ) ((pos "H2%" "M6%" ) (subcat )))
( "shored"  v ( sh oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shores"  v ( sh oo1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "shoring"  v ( sh oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "shorn"  v ( sh oo1 n ) ((pos "Hd%" ) (subcat )))
( "short"  v ( sh oo1 t ) ((pos "J0%" "K6%" "OC*" "Pu*" ) ))
( "short-change"  v ( sh oo1 t - ch ei1 n jh ) ((pos "H2$" ) ))
( "short-changed"  v ( sh oo1 t - ch ei1 n jh d ) ((pos "Hc%" "Hd%" ) ))
( "short-changes"  v ( sh oo1 t - ch ei1 n jh i z ) ((pos "Ha$" ) ))
( "short-changing"  v ( sh oo1 t - ch ei1 n jh i ng ) ((pos "Hb$" ) ))
( "short-circuit"  v ( sh oo1 t - s @@1 k i t ) ((pos "J0%" "K6%" ) ))
( "short-circuited"  v ( sh oo1 t - s @@1 k i t i d ) ((pos "Jc%" "Jd%" ) ))
( "short-circuiting"  v ( sh oo1 t - s @@1 k i t i ng ) ((pos "Jb%" ) ))
( "short-circuits"  v ( sh oo1 t - s @@1 k i t s ) ((pos "Ja%" "Kj%" ) ))
( "short-dated"  j ( sh oo1 t - d ei1 t i d ) ((pos "OA$" ) ))
( "short-handed"  j ( sh oo1 t - h a1 n d i d ) ((pos "OA%" ) ))
( "short-lived"  j ( sh oo1 t - l i1 v d ) ((pos "OA%" ) ))
( "short-range"  j ( sh oo1 t - r ei n jh ) ((pos "OA%" ) ))
( "short-sighted"  j ( sh oo1 t - s ai1 t i d ) ((pos "OA%" ) ))
( "short-tempered"  j ( sh oo1 t - t e1 m p @ d ) ((pos "OA%" ) ))
( "short-term"  j ( sh oo1 t - t @@ m ) ((pos "Oq%" ) ))
( "short-winded"  j ( sh oo1 t - w i1 n d i d ) ((pos "OA%" ) ))
( "shortage"  n ( sh oo1 t i jh ) ((pos "M6%" ) ))
( "shortages"  n ( sh oo1 t i jh i z ) ((pos "Mj%" ) ))
( "shortbread"  n ( sh oo1 t b r e d ) ((pos "L@%" ) ))
( "shortcake"  n ( sh oo1 t k ei k ) ((pos "L@%" ) ))
( "shortcoming"  n ( sh oo1 t k uh m i ng ) ((pos "K6%" ) ))
( "shortcomings"  n ( sh oo1 t k uh m i ng z ) ((pos "Kj%" ) ))
( "shorted"  v ( sh oo1 t i d ) ((pos "Jc%" "Jd%" ) ))
( "shorten"  v ( sh oo1 t n ) ((pos "J0%" ) (subcat "2A" )))
( "shortened"  v ( sh oo1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "shortening"  v ( sh oo1 t n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "shortens"  v ( sh oo1 t n z ) ((pos "Ja%" ) (subcat "2A" )))
( "shorter"  j ( sh oo1 t @ r ) ((pos "Or%" ) ))
( "shortest"  j ( sh oo1 t i s t ) ((pos "Os%" ) ))
( "shortfall"  n ( sh oo1 t f oo l ) ((pos "K6%" ) ))
( "shortfalls"  n ( sh oo1 t f oo l z ) ((pos "Kj%" ) ))
( "shorthand"  n ( sh oo1 t h a n d ) ((pos "L@%" ) ))
( "shorthorn"  n ( sh oo1 t h oo n ) ((pos "K6$" ) ))
( "shorthorns"  n ( sh oo1 t h oo n z ) ((pos "Kj$" ) ))
( "shorting"  v ( sh oo1 t i ng ) ((pos "Jb%" ) ))
( "shortish"  j ( sh oo1 t i sh ) ((pos "OA%" ) ))
( "shortlist"  v ( sh oo1 t l i s t ) ((pos "H0%" "K6%" ) ))
( "shortlisted"  v ( sh oo1 t l i s t i d ) ((pos "Hc%" "Hd%" ) ))
( "shortlisting"  v ( sh oo1 t l i s t i ng ) ((pos "Hb%" ) ))
( "shortlists"  v ( sh oo1 t l i s t s ) ((pos "Ha%" "Kj%" ) ))
( "shortly"  a ( sh oo1 t l ii ) ((pos "Pu%" ) ))
( "shortness"  n ( sh oo1 t n @ s ) ((pos "L@%" ) ))
( "shorts"  v ( sh oo1 t s ) ((pos "Ja%" "Kj%" ) ))
( "shot"  v ( sh o1 t ) ((pos "Jc%" "Jd%" "M6%" ) (subcat "2A" "2C" "4A" "6A" "15A" )))
( "shot-put"  n ( sh o1 t - p u t ) ((pos "M6%" ) ))
( "shot-puts"  n ( sh o1 t - p u t s ) ((pos "Mj$" ) ))
( "shot-tower"  n ( sh o1 t - t au @ r ) ((pos "K6$" ) ))
( "shot-towers"  n ( sh o1 t - t au @ z ) ((pos "Kj$" ) ))
( "shotgun"  n ( sh o1 t g uh n ) ((pos "K6%" ) ))
( "shotguns"  n ( sh o1 t g uh n z ) ((pos "Kj%" ) ))
( "shots"  n ( sh o1 t s ) ((pos "Mj%" ) ))
( "should"  v ( sh u d ) ((pos "G5*" ) ))
( "shoulder"  v ( sh ou1 l d @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "shoulder-blade"  n ( sh ou1 l d @ - b l ei d ) ((pos "K6%" ) ))
( "shoulder-blades"  n ( sh ou1 l d @ - b l ei d z ) ((pos "Kj%" ) ))
( "shoulder-flash"  n ( sh ou1 l d @ - f l a sh ) ((pos "K7$" ) ))
( "shoulder-flashes"  n ( sh ou1 l d @ - f l a sh i z ) ((pos "Kj$" ) ))
( "shoulder-strap"  n ( sh ou1 l d @ - s t r a p ) ((pos "K6%" ) ))
( "shoulder-straps"  n ( sh ou1 l d @ - s t r a p s ) ((pos "Kj%" ) ))
( "shouldered"  v ( sh ou1 l d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shouldering"  v ( sh ou1 l d @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shoulders"  v ( sh ou1 l d @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "shouldn't"  v ( sh u1 d n t ) ((pos "Gg%" ) ))
( "shout"  v ( sh au1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4B" "6A" "15A" "15B" )))
( "shouted"  v ( sh au1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4B" "6A" "15A" "15B" )))
( "shouting"  v ( sh au1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "3A" "4B" "6A" "15A" "15B" )))
( "shouts"  v ( sh au1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "4B" "6A" "15A" "15B" )))
( "shove"  v ( sh uh1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shove-ha'penny"  n ( sh uh2 v - h ei1 p n ii ) ((pos "L@%" ) ))
( "shoved"  v ( sh uh1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shovel"  v ( sh uh1 v @ l ) ((pos "H4%" "K6%" ) (subcat "6A" "15A" )))
( "shovel-board"  n ( sh uh1 v l - b oo d ) ((pos "L@$" ) ))
( "shovelful"  n ( sh uh1 v @ l f u l ) ((pos "K6%" ) ))
( "shovelfuls"  n ( sh uh1 v @ l f u l z ) ((pos "Kj%" ) ))
( "shovelled"  v ( sh uh1 v @ l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "shovelling"  v ( sh uh1 v @ l i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "shovels"  v ( sh uh1 v @ l z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "shoves"  v ( sh uh1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shoving"  v ( sh uh1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "show"  v ( sh ou1 ) ((pos "J0*" "M6%" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "show-business"  n ( sh ou1 - b i z n @ s ) ((pos "L@%" ) ))
( "show-off"  n ( sh ou1 - o f ) ((pos "K6%" ) ))
( "show-offs"  n ( sh ou1 - o f s ) ((pos "Kj%" ) ))
( "show-window"  n ( sh ou1 - w i n d ou ) ((pos "K6$" ) ))
( "show-windows"  n ( sh ou1 - w i n d ou z ) ((pos "Kj$" ) ))
( "showbiz"  n ( sh ou1 b i z ) ((pos "L@%" ) ))
( "showboat"  n ( sh ou1 b ou t ) ((pos "K6%" ) ))
( "showboats"  n ( sh ou1 b ou t s ) ((pos "Kj%" ) ))
( "showcase"  n ( sh ou1 k ei s ) ((pos "K6%" ) ))
( "showcases"  n ( sh ou1 k ei s i z ) ((pos "Kj%" ) ))
( "showdown"  n ( sh ou1 d au n ) ((pos "K6%" ) ))
( "showdowns"  n ( sh ou1 d au n z ) ((pos "Kj$" ) ))
( "showed"  v ( sh ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "shower"  v ( sh au1 @ r ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "shower-bath"  n ( sh au1 @ - b aa th ) ((pos "K6%" ) ))
( "shower-baths"  n ( sh au1 @ - b aa dh z ) ((pos "Kj%" ) ))
( "showered"  v ( sh au1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "showerier"  j ( sh au1 @ r i@ r ) ((pos "Or$" ) ))
( "showeriest"  j ( sh au1 @ r i i s t ) ((pos "Os$" ) ))
( "showering"  v ( sh au1 @ r i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "showers"  v ( sh au1 @ z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "showery"  j ( sh au1 @ r ii ) ((pos "OD%" ) ))
( "showgirl"  n ( sh ou1 g @@ l ) ((pos "K6%" ) ))
( "showgirls"  n ( sh ou1 g @@ l z ) ((pos "Kj%" ) ))
( "showier"  j ( sh ou1 i@ r ) ((pos "Or$" ) ))
( "showiest"  j ( sh ou1 i i s t ) ((pos "Os$" ) ))
( "showily"  a ( sh ou1 i l ii ) ((pos "Pu%" ) ))
( "showiness"  n ( sh ou1 i n @ s ) ((pos "L@%" ) ))
( "showing"  v ( sh ou1 i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "showings"  n ( sh ou1 i ng z ) ((pos "Kj%" ) ))
( "showjumping"  n ( sh ou1 jh uh m p i ng ) ((pos "L@%" ) ))
( "showman"  n ( sh ou1 m @ n ) ((pos "Ki%" ) ))
( "showmanship"  n ( sh ou1 m @ n sh i p ) ((pos "L@%" ) ))
( "showmen"  n ( sh ou1 m @ n ) ((pos "Kj%" ) ))
( "shown"  v ( sh ou1 n ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "showplace"  n ( sh ou1 p l ei s ) ((pos "K6%" ) ))
( "showplaces"  n ( sh ou1 p l ei s i z ) ((pos "Kj%" ) ))
( "showroom"  n ( sh ou1 r uu m ) ((pos "K6%" ) ))
( "showrooms"  n ( sh ou1 r uu m z ) ((pos "Kj%" ) ))
( "shows"  v ( sh ou1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "9" "10" "12A" "13A" "14" "15A" "15B" "19B" "20" "21" "24A" )))
( "showy"  j ( sh ou1 ii ) ((pos "OD%" ) ))
( "shrank"  v ( sh r a1 ng k ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" )))
( "shrapnel"  n ( sh r a1 p n @ l ) ((pos "L@%" ) ))
( "shred"  v ( sh r e1 d ) ((pos "H4%" "K6%" ) (subcat )))
( "shredded"  v ( sh r e1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "shredding"  v ( sh r e1 d i ng ) ((pos "Hb%" ) (subcat )))
( "shreds"  v ( sh r e1 d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "shrew"  n ( sh r uu1 ) ((pos "K6%" ) ))
( "shrew-mice"  n ( sh r uu1 - m ai s ) ((pos "Kj$" ) ))
( "shrew-mouse"  n ( sh r uu1 - m au s ) ((pos "Ki$" ) ))
( "shrewd"  j ( sh r uu1 d ) ((pos "OC%" ) ))
( "shrewder"  j ( sh r uu1 d @ r ) ((pos "Or%" ) ))
( "shrewdest"  j ( sh r uu1 d i s t ) ((pos "Os%" ) ))
( "shrewdly"  a ( sh r uu1 d l ii ) ((pos "Pu%" ) ))
( "shrewdness"  n ( sh r uu1 d n @ s ) ((pos "L@%" ) ))
( "shrewish"  j ( sh r uu1 i sh ) ((pos "OA%" ) ))
( "shrewishly"  a ( sh r uu1 i sh l ii ) ((pos "Pu%" ) ))
( "shrewishness"  n ( sh r uu1 i sh n @ s ) ((pos "L@%" ) ))
( "shrews"  n ( sh r uu1 z ) ((pos "Kj%" ) ))
( "shriek"  v ( sh r ii1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "shrieked"  v ( sh r ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "shrieking"  v ( sh r ii1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "shrieks"  v ( sh r ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "shrift"  n ( sh r i1 f t ) ((pos "L@%" ) ))
( "shrike"  n ( sh r ai1 k ) ((pos "K6$" ) ))
( "shrikes"  n ( sh r ai1 k s ) ((pos "Kj$" ) ))
( "shrill"  v ( sh r i1 l ) ((pos "J0%" "OC%" ) ))
( "shrilled"  v ( sh r i1 l d ) ((pos "Jc%" "Jd%" ) ))
( "shriller"  j ( sh r i1 l @ r ) ((pos "Or%" ) ))
( "shrillest"  j ( sh r i1 l i s t ) ((pos "Os%" ) ))
( "shrilling"  v ( sh r i1 l i ng ) ((pos "Jb%" ) ))
( "shrillness"  n ( sh r i1 l n @ s ) ((pos "L@%" ) ))
( "shrills"  v ( sh r i1 l z ) ((pos "Ja%" ) ))
( "shrilly"  a ( sh r i1 l ii ) ((pos "Pu%" ) ))
( "shrimp"  v ( sh r i1 m p ) ((pos "I0$" "K6%" ) ))
( "shrimped"  v ( sh r i1 m p t ) ((pos "Ic$" "Id$" ) ))
( "shrimping"  v ( sh r i1 m p i ng ) ((pos "Ib%" ) ))
( "shrimps"  v ( sh r i1 m p s ) ((pos "Ia$" "Kj%" ) ))
( "shrine"  v ( sh r ai1 n ) ((pos "H2$" "K6%" ) ))
( "shrined"  v ( sh r ai1 n d ) ((pos "Hc$" "Hd$" ) ))
( "shrines"  v ( sh r ai1 n z ) ((pos "Ha$" "Kj%" ) ))
( "shrining"  v ( sh r ai1 n i ng ) ((pos "Hb$" ) ))
( "shrink"  v ( sh r i1 ng k ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "shrinkable"  j ( sh r i1 n k @ b l ) ((pos "OA$" ) ))
( "shrinkage"  n ( sh r i1 n k i jh ) ((pos "L@%" ) ))
( "shrinking"  v ( sh r i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "shrinks"  v ( sh r i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "shrive"  v ( sh r ai1 v ) ((pos "H2$" ) (subcat )))
( "shrived"  v ( sh r ai1 v d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "shrivel"  v ( sh r i1 v @ l ) ((pos "J4%" ) (subcat "2A" "2C" "6A" )))
( "shrivelled"  v ( sh r i1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "shrivelling"  v ( sh r i1 v @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "shrivels"  v ( sh r i1 v @ l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "shriven"  v ( sh r i1 v n ) ((pos "Hd$" ) (subcat )))
( "shrives"  v ( sh r ai1 v z ) ((pos "Ha$" ) (subcat )))
( "shriving"  v ( sh r ai1 v i ng ) ((pos "Hb$" ) (subcat )))
( "shroud"  v ( sh r au1 d ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "shrouded"  v ( sh r au1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shrouding"  v ( sh r au1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shrouds"  v ( sh r au1 d z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "shrove"  v ( sh r ou1 v ) ((pos "Hc$" ) (subcat )))
( "shrub"  n ( sh r uh1 b ) ((pos "K6%" ) ))
( "shrubberies"  n ( sh r uh1 b @ r i z ) ((pos "Kj%" ) ))
( "shrubbery"  n ( sh r uh1 b @ r ii ) ((pos "K8%" ) ))
( "shrubs"  n ( sh r uh1 b z ) ((pos "Kj%" ) ))
( "shrug"  v ( sh r uh1 g ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "shrugged"  v ( sh r uh1 g d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shrugging"  v ( sh r uh1 g i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shrugs"  v ( sh r uh1 g z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "shrunk"  v ( sh r uh1 ng k ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" )))
( "shrunken"  v ( sh r uh1 n k @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" )))
( "shuck"  v ( sh uh1 k ) ((pos "H0$" "K6$" ) (subcat )))
( "shucked"  v ( sh uh1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "shucking"  v ( sh uh1 k i ng ) ((pos "Hb$" ) (subcat )))
( "shucks"  v ( sh uh1 k s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "shudder"  v ( sh uh1 d @ r ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "shuddered"  v ( sh uh1 d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "shuddering"  v ( sh uh1 d @ r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "shudderingly"  a ( sh uh1 d @ r i ng l ii ) ((pos "Pu%" ) ))
( "shudders"  v ( sh uh1 d @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "shuffle"  v ( sh uh1 f l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shuffled"  v ( sh uh1 f l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shuffler"  n ( sh uh1 f l @ r ) ((pos "K6%" ) ))
( "shufflers"  n ( sh uh1 f l @ z ) ((pos "Kj%" ) ))
( "shuffles"  v ( sh uh1 f l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shuffling"  v ( sh uh1 f l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shun"  v ( sh uh1 n ) ((pos "H4%" ) (subcat "6A" )))
( "shunned"  v ( sh uh1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "shunning"  v ( sh uh1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "shuns"  v ( sh uh1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "shunt"  v ( sh uh1 n t ) ((pos "J0%" ) (subcat "2A" "6A" "15A" )))
( "shunted"  v ( sh uh1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "shunter"  n ( sh uh1 n t @ r ) ((pos "K6$" ) ))
( "shunters"  n ( sh uh1 n t @ z ) ((pos "Kj$" ) ))
( "shunting"  v ( sh uh1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "shunts"  v ( sh uh1 n t s ) ((pos "Ja%" ) (subcat "2A" "6A" "15A" )))
( "shush"  v ( sh u1 sh ) ((pos "J1$" ) ))
( "shushed"  v ( sh u1 sh t ) ((pos "Jc$" "Jd$" ) ))
( "shushes"  v ( sh u1 sh i z ) ((pos "Ja$" ) ))
( "shushing"  v ( sh u1 sh i ng ) ((pos "Jb$" ) ))
( "shut"  v ( sh uh1 t ) ((pos "J5%" "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shutdown"  n ( sh uh1 t d au n ) ((pos "K6%" ) ))
( "shutdowns"  n ( sh uh1 t d au n z ) ((pos "Kj$" ) ))
( "shuteye"  n ( sh uh1 t ai ) ((pos "L@%" ) ))
( "shuts"  v ( sh uh1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shutter"  v ( sh uh1 t @ r ) ((pos "H0$" "K6%" ) (subcat )))
( "shuttered"  v ( sh uh1 t @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "shuttering"  v ( sh uh1 t @ r i ng ) ((pos "Hb$" ) (subcat )))
( "shutters"  v ( sh uh1 t @ z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "shutting"  v ( sh uh1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "shuttle"  v ( sh uh1 t l ) ((pos "J2%" "K6%" ) (subcat "2C" "6A" )))
( "shuttlecock"  n ( sh uh1 t l k o k ) ((pos "K6%" ) ))
( "shuttlecocks"  n ( sh uh1 t l k o k s ) ((pos "Kj%" ) ))
( "shuttled"  v ( sh uh1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "shuttles"  v ( sh uh1 t l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "shuttling"  v ( sh uh1 t l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "shy"  v ( sh ai1 ) ((pos "J3%" "K8%" "OC%" ) (subcat "2A" "3A" "6A" )))
( "shyer"  j ( sh ai1 @ r ) ((pos "Or%" ) ))
( "shyest"  j ( sh ai1 i s t ) ((pos "Os%" ) ))
( "shying"  v ( sh ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "shyly"  a ( sh ai1 l ii ) ((pos "Pu%" ) ))
( "shyness"  n ( sh ai1 n @ s ) ((pos "L@%" ) ))
( "shyster"  n ( sh ai1 s t @ r ) ((pos "K6$" ) ))
( "shysters"  n ( sh ai1 s t @ z ) ((pos "Kj$" ) ))
( "sibilant"  n ( s i1 b i l @ n t ) ((pos "K6$" "OA%" ) ))
( "sibilants"  n ( s i1 b i l @ n t s ) ((pos "Kj$" ) ))
( "sibling"  n ( s i1 b l i ng ) ((pos "K6%" ) ))
( "siblings"  n ( s i1 b l i ng z ) ((pos "Kj%" ) ))
( "sibyl"  n ( s i1 b l ) ((pos "K6$" ) ))
( "sibylline"  j ( s i1 b @ l ai n ) ((pos "OA$" ) ))
( "sibyls"  n ( s i1 b l z ) ((pos "Kj$" ) ))
( "sic"  a ( s i1 k ) ((pos "Pu$" ) ))
( "sick"  v ( s i1 k ) ((pos "H0%" "OA%" ) ))
( "sick-benefit"  n ( s i1 k - b e n i f i t ) ((pos "M6%" ) ))
( "sick-benefits"  n ( s i1 k - b e n i f i t s ) ((pos "Mj%" ) ))
( "sick-berth"  n ( s i1 k - b @@ th ) ((pos "K6%" ) ))
( "sick-berths"  n ( s i1 k - b @@ th s ) ((pos "Kj%" ) ))
( "sick-headache"  n ( s i1 k - h e1 d ei k ) ((pos "K6%" ) ))
( "sick-headaches"  n ( s i1 k - h e1 d ei k s ) ((pos "Kj%" ) ))
( "sick-leave"  n ( s i1 k - l ii v ) ((pos "L@%" ) ))
( "sick-list"  n ( s i1 k - l i s t ) ((pos "K6%" ) ))
( "sick-lists"  n ( s i1 k - l i s t s ) ((pos "Kj%" ) ))
( "sick-parade"  n ( s i1 k - p @ r ei d ) ((pos "K6%" ) ))
( "sick-parades"  n ( s i1 k - p @ r ei d z ) ((pos "Kj%" ) ))
( "sick-pay"  n ( s i1 k - p ei ) ((pos "L@%" ) ))
( "sick-room"  n ( s i1 k - r uu m ) ((pos "K6%" ) ))
( "sick-rooms"  n ( s i1 k - r uu m z ) ((pos "Kj%" ) ))
( "sickbay"  n ( s i1 k b ei ) ((pos "K6%" ) ))
( "sickbays"  n ( s i1 k b ei z ) ((pos "Kj%" ) ))
( "sickbed"  n ( s i1 k b e d ) ((pos "K6%" ) ))
( "sickbeds"  n ( s i1 k b e d z ) ((pos "Kj%" ) ))
( "sicked"  v ( s i1 k t ) ((pos "Hc%" "Hd%" ) ))
( "sicken"  v ( s i1 k @ n ) ((pos "J0%" ) (subcat "2A" "3A" "4C" )))
( "sickened"  v ( s i1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "4C" )))
( "sickening"  v ( s i1 k @ n i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "4C" )))
( "sickeningly"  a ( s i1 k @ n i ng l ii ) ((pos "Pu%" ) ))
( "sickens"  v ( s i1 k @ n z ) ((pos "Ja%" ) (subcat "2A" "3A" "4C" )))
( "sicking"  v ( s i1 k i ng ) ((pos "Hb$" ) ))
( "sickish"  j ( s i1 k i sh ) ((pos "OA$" ) ))
( "sickle"  n ( s i1 k l ) ((pos "K6%" ) ))
( "sickles"  n ( s i1 k l z ) ((pos "Kj%" ) ))
( "sicklier"  j ( s i1 k l i@ r ) ((pos "Or%" ) ))
( "sickliest"  j ( s i1 k l i i s t ) ((pos "Os%" ) ))
( "sickly"  j ( s i1 k l ii ) ((pos "OD%" ) ))
( "sickness"  n ( s i1 k n @ s ) ((pos "M7%" ) ))
( "sicknesses"  n ( s i1 k n @ s i z ) ((pos "Mj%" ) ))
( "sicks"  v ( s i1 k s ) ((pos "Ha$" ) ))
( "side"  v ( s ai1 d ) ((pos "I2%" "M6*" ) (subcat )))
( "side-arms"  n ( s ai1 d - aa m z ) ((pos "Kj$" ) ))
( "side-chapel"  n ( s ai1 d - ch a p l ) ((pos "K6%" ) ))
( "side-chapels"  n ( s ai1 d - ch a p l z ) ((pos "Kj%" ) ))
( "side-dish"  n ( s ai1 d - d i sh ) ((pos "K7%" ) ))
( "side-dishes"  n ( s ai1 d - d i sh i z ) ((pos "Kj%" ) ))
( "side-drum"  n ( s ai1 d - d r uh m ) ((pos "K6%" ) ))
( "side-drums"  n ( s ai1 d - d r uh m z ) ((pos "Kj%" ) ))
( "side-face"  a ( s ai1 d - f ei s ) ((pos "Pu%" ) ))
( "side-glance"  n ( s ai1 d - g l aa1 n s ) ((pos "K6%" ) ))
( "side-glances"  n ( s ai1 d - g l aa1 n s i z ) ((pos "Kj%" ) ))
( "side-road"  n ( s ai1 d - r ou d ) ((pos "K6%" ) ))
( "side-roads"  n ( s ai1 d - r ou d z ) ((pos "Kj%" ) ))
( "side-saddle"  n ( s ai1 d - s a d l ) ((pos "K6%" "Pu%" ) ))
( "side-saddles"  n ( s ai1 d - s a d l z ) ((pos "Kj%" ) ))
( "side-slip"  v ( s ai1 d - s l ii p ) ((pos "I4$" "K6$" ) ))
( "side-slipped"  v ( s ai1 d - s l ii p t ) ((pos "Ic$" "Id$" ) ))
( "side-slipping"  v ( s ai1 d - s l ii p i ng ) ((pos "Ib$" ) ))
( "side-slips"  v ( s ai1 d - s l ii p s ) ((pos "Ia$" "Kj$" ) ))
( "side-splitting"  j ( s ai1 d - s p l i t i ng ) ((pos "OA%" ) ))
( "side-stroke"  n ( s ai1 d - s t r ou k ) ((pos "L@%" ) ))
( "side-view"  n ( s ai1 d - v y uu1 ) ((pos "K6%" ) ))
( "side-views"  n ( s ai1 d - v y uu1 z ) ((pos "Kj%" ) ))
( "side-whiskers"  n ( s ai1 d - w i s k @ z ) ((pos "Kj%" ) ))
( "sideboard"  n ( s ai1 d b oo d ) ((pos "K6%" ) ))
( "sideboards"  n ( s ai1 d b oo d z ) ((pos "Kj%" ) ))
( "sideburns"  n ( s ai1 d b @@ n z ) ((pos "Kj%" ) ))
( "sidecar"  n ( s ai1 d k aa r ) ((pos "K6%" ) ))
( "sidecars"  n ( s ai1 d k aa z ) ((pos "Kj%" ) ))
( "sided"  v ( s ai1 d i d ) ((pos "Ic%" "Id%" "Ot%" ) (subcat )))
( "sidelight"  n ( s ai1 d l ai t ) ((pos "K6%" ) ))
( "sidelights"  n ( s ai1 d l ai t s ) ((pos "Kj%" ) ))
( "sideline"  n ( s ai1 d l ai n ) ((pos "K6%" ) ))
( "sidelines"  n ( s ai1 d l ai n z ) ((pos "Kj%" ) ))
( "sidelong"  j ( s ai1 d l o ng ) ((pos "OA%" "Pu%" ) ))
( "sidereal"  j ( s ai1 d i@1 r i@ l ) ((pos "OA$" ) ))
( "sides"  v ( s ai1 d z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "sideshow"  n ( s ai1 d sh ou ) ((pos "K6%" ) ))
( "sideshows"  n ( s ai1 d sh ou z ) ((pos "Kj%" ) ))
( "sidesman"  n ( s ai1 d z m @ n ) ((pos "Ki%" ) ))
( "sidesmen"  n ( s ai1 d z m @ n ) ((pos "Kj%" ) ))
( "sidestep"  v ( s ai1 d s t e p ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "sidestepped"  v ( s ai1 d s t e p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sidestepping"  v ( s ai1 d s t e p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sidesteps"  v ( s ai1 d s t e p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "sidetrack"  v ( s ai1 d t r a k ) ((pos "H0$" "K6$" ) (subcat )))
( "sidetracked"  v ( s ai1 d t r a k t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "sidetracking"  v ( s ai1 d t r a k i ng ) ((pos "Hb$" ) (subcat )))
( "sidetracks"  v ( s ai1 d t r a k s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "sidewalk"  n ( s ai1 d w oo k ) ((pos "K6%" ) ))
( "sidewalks"  n ( s ai1 d w oo k s ) ((pos "Kj%" ) ))
( "sidewards"  a ( s ai1 d w @ d z ) ((pos "Pu%" ) ))
( "sideways"  a ( s ai1 d w ei z ) ((pos "Pu%" ) ))
( "siding"  v ( s ai1 d i ng ) ((pos "Ib%" "K6%" ) (subcat )))
( "sidings"  n ( s ai1 d i ng z ) ((pos "Kj%" ) ))
( "sidle"  v ( s ai1 d l ) ((pos "I2%" ) (subcat )))
( "sidled"  v ( s ai1 d l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sidles"  v ( s ai1 d l z ) ((pos "Ia%" ) (subcat )))
( "sidling"  v ( s ai1 d l i ng ) ((pos "Ib%" ) (subcat )))
( "siege"  n ( s ii1 jh ) ((pos "M6%" ) ))
( "sieges"  n ( s ii1 jh i z ) ((pos "Mj%" ) ))
( "sienna"  n ( s i1 e1 n @ ) ((pos "L@$" ) ))
( "sierra"  n ( s i1 e@1 r @ ) ((pos "K6$" ) ))
( "sierras"  n ( s i1 e@1 r @ z ) ((pos "Kj$" ) ))
( "siesta"  n ( s i1 e1 s t @ ) ((pos "K6%" ) ))
( "siestas"  n ( s i1 e1 s t @ z ) ((pos "Kj%" ) ))
( "sieve"  v ( s i1 v ) ((pos "H2%" "K6%" ) (subcat )))
( "sieved"  v ( s i1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sieves"  v ( s i1 v z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sieving"  v ( s i1 v i ng ) ((pos "Hb%" ) (subcat )))
( "sift"  v ( s i1 f t ) ((pos "J0%" ) (subcat "2C" "6A" "14" "15A" )))
( "sifted"  v ( s i1 f t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "sifter"  n ( s i1 f t @ r ) ((pos "K6$" ) ))
( "sifters"  n ( s i1 f t @ z ) ((pos "Kj$" ) ))
( "sifting"  v ( s i1 f t i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" "15A" )))
( "sifts"  v ( s i1 f t s ) ((pos "Ja%" ) (subcat "2C" "6A" "14" "15A" )))
( "sigh"  v ( s ai1 ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "sighed"  v ( s ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "sighing"  v ( s ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "sighs"  v ( s ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "sight"  v ( s ai1 t ) ((pos "H0%" "M6%" ) (subcat )))
( "sighted"  v ( s ai1 t i d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat )))
( "sighting"  v ( s ai1 t i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "sightings"  n ( s ai1 t i ng z ) ((pos "Kj%" ) ))
( "sightless"  j ( s ai1 t l i s ) ((pos "OA%" ) ))
( "sights"  v ( s ai1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sightseeing"  n ( s ai1 t s ii i ng ) ((pos "L@%" ) ))
( "sightseer"  n ( s ai1 t s ii @ r ) ((pos "K6%" ) ))
( "sightseers"  n ( s ai1 t s ii @ z ) ((pos "Kj%" ) ))
( "sign"  v ( s ai1 n ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sign-painter"  n ( s ai1 n - p ei n t @ r ) ((pos "K6%" ) ))
( "sign-painters"  n ( s ai1 n - p ei n t @ z ) ((pos "Kj%" ) ))
( "signal"  v ( s i1 g n @ l ) ((pos "J4%" "K6%" "Oq%" ) (subcat "2A" "2C" "6A" "9" )))
( "signal-box"  n ( s i1 g n @ l - b o k s ) ((pos "K7%" ) ))
( "signal-boxes"  n ( s i1 g n @ l - b o k s i z ) ((pos "Kj%" ) ))
( "signalize"  v ( s i1 g n @ l ai z ) ((pos "H2$" ) (subcat )))
( "signalized"  v ( s i1 g n @ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "signalizes"  v ( s i1 g n @ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "signalizing"  v ( s i1 g n @ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "signalled"  v ( s i1 g n @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" )))
( "signaller"  n ( s i1 g n @ l @ r ) ((pos "K6%" ) ))
( "signallers"  n ( s i1 g n @ l @ z ) ((pos "Kj%" ) ))
( "signalling"  v ( s i1 g n @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "9" )))
( "signally"  a ( s i1 g n @ l ii ) ((pos "Pu%" ) ))
( "signalman"  n ( s i1 g n @ l m @ n ) ((pos "Ki%" ) ))
( "signalmen"  n ( s i1 g n @ l m @ n ) ((pos "Kj%" ) ))
( "signals"  v ( s i1 g n @ l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "9" )))
( "signatories"  n ( s i1 g n @ t r i z ) ((pos "Kj%" ) ))
( "signatory"  n ( s i1 g n @ t r ii ) ((pos "K8%" ) ))
( "signature"  n ( s i1 g n @ ch @ r ) ((pos "K6%" ) ))
( "signatures"  n ( s i1 g n @ ch @ z ) ((pos "Kj%" ) ))
( "signed"  v ( s ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "signet"  n ( s i1 g n i t ) ((pos "K6%" ) ))
( "signet-ring"  n ( s i1 g n i t - r i ng ) ((pos "K6%" ) ))
( "signet-rings"  n ( s i1 g n i t - r i ng z ) ((pos "Kj%" ) ))
( "signets"  n ( s i1 g n i t s ) ((pos "Kj$" ) ))
( "significance"  n ( s i1 g n i1 f i k @ n s ) ((pos "L@%" ) ))
( "significant"  j ( s i1 g n i1 f i k @ n t ) ((pos "OA%" ) ))
( "significantly"  a ( s i1 g n i1 f i k @ n t l ii ) ((pos "Pu%" ) ))
( "signification"  n ( s i2 g n i f i k ei1 sh @ n ) ((pos "K6%" ) ))
( "significations"  n ( s i2 g n i f i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "significative"  j ( s i1 g n i1 f i k @ t i v ) ((pos "OA$" ) ))
( "signified"  v ( s i1 g n i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "9" )))
( "signifies"  v ( s i1 g n i f ai z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "9" )))
( "signify"  v ( s i1 g n i f ai ) ((pos "J3%" ) (subcat "2A" "2C" "6A" "9" )))
( "signifying"  v ( s i1 g n i f ai i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "9" )))
( "signing"  v ( s ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "signor"  n ( s ii1 n y oo r ) ((pos "K6$" ) ))
( "signora"  n ( s i1 n y oo1 r @ ) ((pos "K6$" ) ))
( "signoras"  n ( s i1 n y oo1 r @ z ) ((pos "Kj$" ) ))
( "signorina"  n ( s i2 n y oo r ii1 n @ ) ((pos "K6$" ) ))
( "signorinas"  n ( s i2 n y oo r ii1 n @ z ) ((pos "Kj$" ) ))
( "signors"  n ( s ii1 n y oo z ) ((pos "Kj$" ) ))
( "signpost"  v ( s ai1 n p ou s t ) ((pos "H0%" "K6%" ) ))
( "signposted"  v ( s ai1 n p ou s t i d ) ((pos "Hc%" "Hd%" ) ))
( "signposting"  v ( s ai1 n p ou s t i ng ) ((pos "Hb%" ) ))
( "signposts"  v ( s ai1 n p ou s t s ) ((pos "Ha%" "Kj%" ) ))
( "signs"  v ( s ai1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "silage"  n ( s ai1 l i jh ) ((pos "L@$" ) ))
( "silence"  v ( s ai1 l @ n s ) ((pos "H2%" "M6%" ) (subcat )))
( "silenced"  v ( s ai1 l @ n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "silencer"  n ( s ai1 l @ n s @ r ) ((pos "K6%" ) ))
( "silencers"  n ( s ai1 l @ n s @ z ) ((pos "Kj%" ) ))
( "silences"  v ( s ai1 l @ n s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "silencing"  v ( s ai1 l @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "silent"  j ( s ai1 l @ n t ) ((pos "OA%" ) ))
( "silently"  a ( s ai1 l @ n t l ii ) ((pos "Pu%" ) ))
( "silhouette"  v ( s i2 l uu e1 t ) ((pos "H2%" "K6%" ) ))
( "silhouetted"  v ( s i2 l uu e1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "silhouettes"  v ( s i2 l uu e1 t s ) ((pos "Ha%" "Kj%" ) ))
( "silhouetting"  v ( s i2 l uu e1 t i ng ) ((pos "Hb%" ) ))
( "silica"  n ( s i1 l i k @ ) ((pos "L@$" ) ))
( "silicate"  n ( s i1 l i k ei t ) ((pos "L@$" ) ))
( "silicon"  n ( s i1 l i k @ n ) ((pos "L@%" ) ))
( "silicone"  n ( s i1 l i k ou n ) ((pos "L@$" ) ))
( "silicosis"  n ( s i2 l i k ou1 s i s ) ((pos "L@$" ) ))
( "silk"  n ( s i1 l k ) ((pos "M6%" ) ))
( "silken"  j ( s i1 l k @ n ) ((pos "OA%" ) ))
( "silkier"  j ( s i1 l k i@ r ) ((pos "Or%" ) ))
( "silkiest"  j ( s i1 l k i i s t ) ((pos "Os%" ) ))
( "silkily"  a ( s i1 l k i l ii ) ((pos "Pu%" ) ))
( "silkiness"  n ( s i1 l k i n @ s ) ((pos "L@%" ) ))
( "silks"  n ( s i1 l k s ) ((pos "Mj%" ) ))
( "silkworm"  n ( s i1 l k w @@ m ) ((pos "K6%" ) ))
( "silkworms"  n ( s i1 l k w @@ m z ) ((pos "Kj%" ) ))
( "silky"  j ( s i1 l k ii ) ((pos "OD%" ) ))
( "sill"  n ( s i1 l ) ((pos "K6%" ) ))
( "sillabub"  n ( s i1 l @ b uh b ) ((pos "M6$" ) ))
( "sillabubs"  n ( s i1 l @ b uh b z ) ((pos "Mj$" ) ))
( "sillier"  j ( s i1 l i@ r ) ((pos "Or%" ) ))
( "sillies"  n ( s i1 l i z ) ((pos "Kj$" ) ))
( "silliest"  j ( s i1 l i i s t ) ((pos "Os%" ) ))
( "silliness"  n ( s i1 l i n @ s ) ((pos "L@%" ) ))
( "sills"  n ( s i1 l z ) ((pos "Kj%" ) ))
( "silly"  n ( s i1 l ii ) ((pos "K8$" "OD%" ) ))
( "silo"  n ( s ai1 l ou ) ((pos "K6%" ) ))
( "silos"  n ( s ai1 l ou z ) ((pos "Kj%" ) ))
( "silt"  v ( s i1 l t ) ((pos "J0%" "L@%" ) (subcat "2C" )))
( "silted"  v ( s i1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "silting"  v ( s i1 l t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "silts"  v ( s i1 l t s ) ((pos "Ja%" ) (subcat "2C" )))
( "silvan"  j ( s i1 l v @ n ) ((pos "OA$" ) ))
( "silver"  v ( s i1 l v @ r ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "silver-fish"  n ( s i1 l v @ - f i sh ) ((pos "K9%" ) ))
( "silver-fishes"  n ( s i1 l v @ - f i sh i z ) ((pos "Kj%" ) ))
( "silvered"  v ( s i1 l v @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "silverier"  j ( s i1 l v @ r i@ r ) ((pos "Or$" ) ))
( "silveriest"  j ( s i1 l v @ r i i s t ) ((pos "Os$" ) ))
( "silvering"  v ( s i1 l v @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "silvern"  j ( s i1 l v @ n ) ((pos "OA$" ) ))
( "silvers"  v ( s i1 l v @ z ) ((pos "Ja$" ) (subcat "2A" )))
( "silverside"  n ( s i1 l v @ s ai d ) ((pos "L@%" ) ))
( "silversmith"  n ( s i1 l v @ s m i th ) ((pos "K6%" ) ))
( "silversmiths"  n ( s i1 l v @ s m i th s ) ((pos "Kj%" ) ))
( "silvery"  j ( s i1 l v @ r ii ) ((pos "OD%" ) ))
( "simian"  n ( s i1 m i@ n ) ((pos "K6$" "OA%" ) ))
( "simians"  n ( s i1 m i@ n z ) ((pos "Kj$" ) ))
( "similar"  j ( s i1 m @ l @ r ) ((pos "OA%" ) ))
( "similarities"  n ( s i2 m @ l a1 r i t i z ) ((pos "Mj%" ) ))
( "similarity"  n ( s i2 m @ l a1 r i t ii ) ((pos "M8%" ) ))
( "similarly"  a ( s i1 m @ l @ l ii ) ((pos "Pu%" ) ))
( "simile"  n ( s i1 m i l ii ) ((pos "M6%" ) ))
( "similes"  n ( s i1 m i l i z ) ((pos "Mj%" ) ))
( "similitude"  n ( s i1 m i1 l i t y uu d ) ((pos "M6%" ) ))
( "similitudes"  n ( s i1 m i1 l i t y uu d z ) ((pos "Mj$" ) ))
( "simmer"  v ( s i1 m @ r ) ((pos "J0%" "Ki%" ) (subcat "2A" "2B" "2C" "6A" )))
( "simmered"  v ( s i1 m @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" )))
( "simmering"  v ( s i1 m @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" )))
( "simmers"  v ( s i1 m @ z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" )))
( "simony"  n ( s i1 m @ n ii ) ((pos "L@$" ) ))
( "simoom"  n ( s i1 m uu1 m ) ((pos "K6$" ) ))
( "simooms"  n ( s i1 m uu1 m z ) ((pos "Kj$" ) ))
( "simoon"  n ( s i1 m uu1 n ) ((pos "K6$" ) ))
( "simoons"  n ( s i1 m uu1 n z ) ((pos "Kj$" ) ))
( "simper"  v ( s i1 m p @ r ) ((pos "I0%" ) (subcat )))
( "simpered"  v ( s i1 m p @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "simpering"  v ( s i1 m p @ r i ng ) ((pos "Ib%" ) (subcat )))
( "simperingly"  a ( s i1 m p r i ng l ii ) ((pos "Pu%" ) ))
( "simpers"  v ( s i1 m p @ z ) ((pos "Ia%" ) (subcat )))
( "simple"  n ( s i1 m p l ) ((pos "K6%" "OB%" ) ))
( "simple-hearted"  j ( s i2 m p l - h aa1 t i d ) ((pos "OA%" ) ))
( "simple-minded"  j ( s i2 m p l - m ai1 n d i d ) ((pos "OA%" ) ))
( "simpler"  j ( s i1 m p l @ r ) ((pos "Or%" ) ))
( "simples"  n ( s i1 m p l z ) ((pos "Kj$" ) ))
( "simplest"  j ( s i1 m p l i s t ) ((pos "Os%" ) ))
( "simpleton"  n ( s i1 m p l t @ n ) ((pos "K6%" ) ))
( "simpletons"  n ( s i1 m p l t @ n z ) ((pos "Kj%" ) ))
( "simplicity"  n ( s i1 m p l i1 s i t ii ) ((pos "L@%" ) ))
( "simplification"  n ( s i2 m p l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "simplifications"  n ( s i2 m p l i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "simplified"  v ( s i1 m p l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "simplifies"  v ( s i1 m p l i f ai z ) ((pos "Ha%" ) (subcat )))
( "simplify"  v ( s i1 m p l i f ai ) ((pos "H3%" ) (subcat )))
( "simplifying"  v ( s i1 m p l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "simply"  a ( s i1 m p l ii ) ((pos "Pu%" ) ))
( "simulacra"  n ( s i2 m y u l ei1 k r @ ) ((pos "Kj$" ) ))
( "simulacrum"  n ( s i2 m y u l ei1 k r @ m ) ((pos "Ki$" ) ))
( "simulate"  v ( s i1 m y u l ei t ) ((pos "H2%" ) (subcat )))
( "simulated"  v ( s i1 m y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "simulates"  v ( s i1 m y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "simulating"  v ( s i1 m y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "simulation"  n ( s i2 m y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "simulator"  n ( s i1 m y u l ei t @ r ) ((pos "K6%" ) ))
( "simulators"  n ( s i1 m y u l ei t @ z ) ((pos "Kj%" ) ))
( "simultaneity"  n ( s i2 m l t @ n i@1 t ii ) ((pos "L@%" ) ))
( "simultaneous"  j ( s i2 m l t ei1 n i@ s ) ((pos "OA%" ) ))
( "simultaneously"  a ( s i2 m l t ei1 n i@ s l ii ) ((pos "Pu%" ) ))
( "simultaneousness"  n ( s i2 m l t ei1 n i@ s n @ s ) ((pos "L@$" ) ))
( "sin"  v ( s i1 n ) ((pos "I4%" "M6%" ) (subcat "2A" )))
( "since"  a ( s i n s ) ((pos "Pu*" "T-*" "V-*" ) ))
( "sincere"  j ( s i1 n s i@1 r ) ((pos "OA%" ) ))
( "sincerely"  a ( s i1 n s i@1 l ii ) ((pos "Pu%" ) ))
( "sincerity"  n ( s i1 n s e1 r i t ii ) ((pos "L@%" ) ))
( "sine"  n ( s ai1 n ) ((pos "K6$" ) ))
( "sine die"  a ( s ai2 n i - d ai1 ii ) ((pos "Pu$" ) ))
( "sine qua non"  n ( s i2 n ei - k w aa - n ou1 n ) ((pos "Ki%" ) ))
( "sinecure"  n ( s ai1 n i k y u@ r ) ((pos "K6%" ) ))
( "sinecures"  n ( s ai1 n i k y u@ z ) ((pos "Kj%" ) ))
( "sines"  n ( s ai1 n z ) ((pos "Kj$" ) ))
( "sinew"  n ( s i1 n y uu ) ((pos "K6%" ) ))
( "sinews"  n ( s i1 n y uu z ) ((pos "Kj%" ) ))
( "sinewy"  j ( s i1 n y uu ii ) ((pos "OA%" ) ))
( "sinful"  j ( s i1 n f @ l ) ((pos "OA%" ) ))
( "sinfulness"  n ( s i1 n f @ l n @ s ) ((pos "L@%" ) ))
( "sing"  v ( s i1 ng ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" )))
( "singable"  j ( s i1 ng @ b l ) ((pos "OA%" ) ))
( "singe"  v ( s i1 n jh ) ((pos "J5%" "K6%" ) (subcat "2A" )))
( "singed"  v ( s i1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "singeing"  v ( s i1 n jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "singer"  n ( s i1 ng @ r ) ((pos "K6%" ) ))
( "singers"  n ( s i1 ng @ z ) ((pos "Kj%" ) ))
( "singes"  v ( s i1 n jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "singing"  v ( s i1 ng i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" )))
( "single"  v ( s i1 ng g l ) ((pos "H2%" "K6%" "OA%" ) (subcat )))
( "single-breasted"  j ( s i2 ng g l - b r e1 s t i d ) ((pos "OA%" ) ))
( "single-handed"  j ( s i2 ng g l - h a1 n d i d ) ((pos "OA%" "Pu%" ) ))
( "single-minded"  j ( s i2 ng g l - m ai1 n d i d ) ((pos "OA%" ) ))
( "single-spacing"  n ( s i2 ng g l - s p ei1 s i ng ) ((pos "L@%" ) ))
( "singled"  v ( s i1 ng g l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "singleness"  n ( s i1 ng g l n @ s ) ((pos "L@%" ) ))
( "singles"  v ( s i1 ng g l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "singlestick"  n ( s i1 ng g l s t i k ) ((pos "M6$" ) ))
( "singlesticks"  n ( s i1 ng g l s t i k s ) ((pos "Mj$" ) ))
( "singlet"  n ( s i1 ng g l i t ) ((pos "K6%" ) ))
( "singleton"  n ( s i1 ng g l t @ n ) ((pos "K6%" ) ))
( "singletons"  n ( s i1 ng g l t @ n z ) ((pos "Kj%" ) ))
( "singlets"  n ( s i1 ng g l i t s ) ((pos "Kj%" ) ))
( "singling"  v ( s i1 ng g l i ng ) ((pos "Hb%" ) (subcat )))
( "singly"  a ( s i1 ng g l ii ) ((pos "Pu%" ) ))
( "sings"  v ( s i1 ng z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" )))
( "singsong"  n ( s i1 ng s o ng ) ((pos "K6%" ) ))
( "singsongs"  n ( s i1 ng s o ng z ) ((pos "Kj%" ) ))
( "singular"  n ( s i1 ng g y u l @ r ) ((pos "K6%" "OA%" ) ))
( "singularities"  n ( s i2 ng g y u l a1 r i t i z ) ((pos "Mj%" ) ))
( "singularity"  n ( s i2 ng g y u l a1 r i t ii ) ((pos "M8%" ) ))
( "singularize"  v ( s i1 ng g y u l @ r ai z ) ((pos "H2$" ) ))
( "singularized"  v ( s i1 ng g y u l @ r ai z d ) ((pos "Hc$" "Hd$" ) ))
( "singularizes"  v ( s i1 ng g y u l @ r ai z i z ) ((pos "Ha$" ) ))
( "singularizing"  v ( s i1 ng g y u l @ r ai z i ng ) ((pos "Hb$" ) ))
( "singularly"  a ( s i1 ng g y u l @ l ii ) ((pos "Pu%" ) ))
( "singulars"  n ( s i1 ng g y u l @ z ) ((pos "Kj%" ) ))
( "sinister"  j ( s i1 n i s t @ r ) ((pos "OA%" ) ))
( "sink"  v ( s i1 ng k ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sinkable"  j ( s i1 ng k @ b l ) ((pos "OA%" ) ))
( "sinker"  n ( s i1 ng k @ r ) ((pos "K6$" ) ))
( "sinkers"  n ( s i1 ng k @ z ) ((pos "Kj$" ) ))
( "sinking"  v ( s i1 ng k i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sinking-fund"  n ( s i1 ng k i ng - f uh n d ) ((pos "K6%" ) ))
( "sinking-funds"  n ( s i1 ng k i ng - f uh n d z ) ((pos "Kj%" ) ))
( "sinkings"  n ( s i1 ng k i ng z ) ((pos "Kj%" ) ))
( "sinks"  v ( s i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sinless"  j ( s i1 n l @ s ) ((pos "OA%" ) ))
( "sinlessness"  n ( s i1 n l @ s n @ s ) ((pos "L@$" ) ))
( "sinned"  v ( s i1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sinner"  n ( s i1 n @ r ) ((pos "K6%" ) ))
( "sinners"  n ( s i1 n @ z ) ((pos "Kj%" ) ))
( "sinning"  v ( s i1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sins"  v ( s i1 n z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "sinuosities"  n ( s i2 n y u o1 s i t i z ) ((pos "Mj$" ) ))
( "sinuosity"  n ( s i2 n y u o1 s i t ii ) ((pos "M8$" ) ))
( "sinuous"  j ( s i1 n y u@ s ) ((pos "OA%" ) ))
( "sinus"  n ( s ai1 n @ s ) ((pos "K7%" ) ))
( "sinuses"  n ( s ai1 n @ s i z ) ((pos "Kj%" ) ))
( "sinusitis"  n ( s ai2 n @ s ai1 t i s ) ((pos "L@%" ) ))
( "sip"  v ( s i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" )))
( "siphon"  v ( s ai1 f @ n ) ((pos "J0%" "K6%" ) (subcat )))
( "siphoned"  v ( s ai1 f @ n d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "siphoning"  v ( s ai1 f @ n i ng ) ((pos "Jb%" ) (subcat )))
( "siphons"  v ( s ai1 f @ n z ) ((pos "Ja%" "Kj%" ) (subcat )))
( "sipped"  v ( s i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "sipping"  v ( s i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "sips"  v ( s i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "sir"  n ( s @@1 r ) ((pos "K6%" ) ))
( "sirdar"  n ( s @@1 d aa r ) ((pos "K6$" ) ))
( "sirdars"  n ( s @@1 d aa z ) ((pos "Kj$" ) ))
( "sire"  v ( s ai1 @ r ) ((pos "H2%" "K6%" ) (subcat )))
( "sired"  v ( s ai1 @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "siren"  n ( s ai1 @ r @ n ) ((pos "K6%" ) ))
( "sirens"  n ( s ai1 @ r @ n z ) ((pos "Kj%" ) ))
( "sires"  v ( s ai1 @ z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "siring"  v ( s ai1 @ r i ng ) ((pos "Hb$" ) (subcat )))
( "sirloin"  n ( s @@1 l oi n ) ((pos "M6%" ) ))
( "sirloins"  n ( s @@1 l oi n z ) ((pos "Mj$" ) ))
( "sirocco"  n ( s i1 r o1 k ou ) ((pos "K6$" ) ))
( "siroccos"  n ( s i1 r o1 k ou z ) ((pos "Kj$" ) ))
( "sirrah"  n ( s i1 r @ ) ((pos "K6$" ) ))
( "sirrahs"  n ( s i1 r @ z ) ((pos "Kj$" ) ))
( "sirs"  n ( s @@1 z ) ((pos "Kj%" ) ))
( "sirup"  n ( s i1 r @ p ) ((pos "M6$" ) ))
( "sirups"  n ( s i1 r @ p s ) ((pos "Mj$" ) ))
( "sisal"  n ( s ai1 s l ) ((pos "L@$" ) ))
( "sissies"  n ( s i1 s i z ) ((pos "Kj%" ) ))
( "sissified"  j ( s i1 s i f ai d ) ((pos "OA$" ) ))
( "sissy"  n ( s i1 s ii ) ((pos "K8%" ) ))
( "sister"  n ( s i1 s t @ r ) ((pos "K6%" ) ))
( "sister-in-law"  n ( s i1 s t @ r - i n - l oo ) ((pos "Ki%" ) ))
( "sisterhood"  n ( s i1 s t @ h u d ) ((pos "L@%" ) ))
( "sisterly"  j ( s i1 s t @ l ii ) ((pos "OA%" ) ))
( "sisters"  n ( s i1 s t @ z ) ((pos "Kj%" ) ))
( "sisters-in-law"  n ( s i1 s t @ z - i n - l oo ) ((pos "Kj%" ) ))
( "sit"  v ( s i1 t ) ((pos "J5%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sit-in"  n ( s i1 t - i n ) ((pos "K6%" ) ))
( "sit-ins"  n ( s i1 t - i n z ) ((pos "Kj%" ) ))
( "sitar"  n ( s i1 t aa1 r ) ((pos "K6$" ) ))
( "sitars"  n ( s i1 t aa1 z ) ((pos "Kj$" ) ))
( "site"  v ( s ai1 t ) ((pos "I2%" "K6%" ) (subcat )))
( "sited"  v ( s ai1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sites"  v ( s ai1 t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "siting"  v ( s ai1 t i ng ) ((pos "Ib%" ) (subcat )))
( "sits"  v ( s i1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sitter"  n ( s i1 t @ r ) ((pos "K6%" ) ))
( "sitters"  n ( s i1 t @ z ) ((pos "Kj%" ) ))
( "sitting"  v ( s i1 t i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sitting-room"  n ( s i1 t i ng - r uu m ) ((pos "K6%" ) ))
( "sitting-rooms"  n ( s i1 t i ng - r uu m z ) ((pos "Kj%" ) ))
( "sittings"  n ( s i1 t i ng z ) ((pos "Kj%" ) ))
( "situated"  j ( s i1 ch u ei t i d ) ((pos "Op%" ) ))
( "situation"  n ( s i2 ch u ei1 sh @ n ) ((pos "K6%" ) ))
( "situations"  n ( s i2 ch u ei1 sh @ n z ) ((pos "Kj%" ) ))
( "six"  n ( s i1 k s ) ((pos "K7%" "OA*" ) ))
( "six-footer"  n ( s i1 k s - f u1 t @ r ) ((pos "K6%" ) ))
( "six-footers"  n ( s i1 k s - f u1 t @ z ) ((pos "Kj%" ) ))
( "six-shooter"  n ( s i1 k s - sh uu t @ r ) ((pos "K6%" ) ))
( "six-shooters"  n ( s i1 k s - sh uu t @ z ) ((pos "Kj%" ) ))
( "sixes"  n ( s i1 k s i z ) ((pos "Kj%" ) ))
( "sixfold"  j ( s i1 k s f ou l d ) ((pos "OA%" "Pu%" ) ))
( "sixpence"  n ( s i1 k s p @ n s ) ((pos "K6%" ) ))
( "sixpences"  n ( s i1 k s p @ n s i z ) ((pos "Kj%" ) ))
( "sixpenny"  j ( s i1 k s p @ n ii ) ((pos "OA%" ) ))
( "sixteen"  n ( s i1 k s t ii1 n ) ((pos "K6%" "OA%" ) ))
( "sixteens"  n ( s i1 k s t ii1 n z ) ((pos "Kj%" ) ))
( "sixteenth"  n ( s ii1 k s t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "sixteenths"  n ( s ii1 k s t ii1 n th s ) ((pos "Kj%" ) ))
( "sixth"  n ( s i1 k s th ) ((pos "K6%" "OA%" ) ))
( "sixth-former"  n ( s i1 k s th - f oo m @ r ) ((pos "K6%" ) ))
( "sixth-formers"  n ( s i1 k s th - f oo m @ z ) ((pos "Kj%" ) ))
( "sixthly"  a ( s i1 k s th l ii ) ((pos "Pu%" ) ))
( "sixths"  n ( s i1 k s th s ) ((pos "Kj%" ) ))
( "sixties"  n ( s i1 k s t i z ) ((pos "Kj%" ) ))
( "sixtieth"  n ( s i1 k s t i@ th ) ((pos "K6%" "OA%" ) ))
( "sixtieths"  n ( s i1 k s t i@ th s ) ((pos "Kj%" ) ))
( "sixty"  n ( s i1 k s t ii ) ((pos "K8%" "OA%" ) ))
( "sizable"  j ( s ai1 z @ b l ) ((pos "OA$" ) ))
( "size"  v ( s ai1 z ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "sizeable"  j ( s ai1 z @ b l ) ((pos "OA%" ) ))
( "sized"  v ( s ai1 z d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat "6A" )))
( "sizes"  v ( s ai1 z i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "sizing"  v ( s ai1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sizzle"  v ( s i1 z l ) ((pos "I2%" ) (subcat )))
( "sizzled"  v ( s i1 z l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sizzles"  v ( s i1 z l z ) ((pos "Ia%" ) (subcat )))
( "sizzling"  v ( s i1 z l i ng ) ((pos "Ib%" ) (subcat )))
( "skate"  v ( s k ei1 t ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "skateboard"  n ( s k ei1 t b oo d ) ((pos "K6%" ) ))
( "skateboarder"  n ( s k ei1 t b oo d @ r ) ((pos "K6%" ) ))
( "skateboarders"  n ( s k ei1 t b oo d @ z ) ((pos "Kj%" ) ))
( "skateboarding"  n ( s k ei1 t b oo d i ng ) ((pos "L@%" ) ))
( "skateboards"  n ( s k ei1 t b oo d z ) ((pos "Kj%" ) ))
( "skated"  v ( s k ei1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "skater"  n ( s k ei1 t @ r ) ((pos "K6%" ) ))
( "skaters"  n ( s k ei1 t @ z ) ((pos "Kj%" ) ))
( "skates"  v ( s k ei1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "skating"  v ( s k ei1 t i ng ) ((pos "Ib%" "L@%" ) (subcat "2A" )))
( "skating-rink"  n ( s k ei1 t i ng - r i ng k ) ((pos "K6%" ) ))
( "skating-rinks"  n ( s k ei1 t i ng - r i ng k s ) ((pos "Kj%" ) ))
( "skedaddle"  v ( s k i1 d a1 d l ) ((pos "I2$" ) (subcat )))
( "skedaddled"  v ( s k i1 d a1 d l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "skedaddles"  v ( s k i1 d a1 d l z ) ((pos "Ia$" ) (subcat )))
( "skedaddling"  v ( s k i1 d a1 d l i ng ) ((pos "Ib$" ) (subcat )))
( "skeet"  n ( s k ii1 t ) ((pos "K6$" ) ))
( "skeets"  n ( s k ii1 t s ) ((pos "Kj$" ) ))
( "skein"  n ( s k ei1 n ) ((pos "K6$" ) ))
( "skeins"  n ( s k ei1 n z ) ((pos "Kj$" ) ))
( "skeleton"  n ( s k e1 l i t n ) ((pos "K6%" ) ))
( "skeletons"  n ( s k e1 l i t n z ) ((pos "Kj%" ) ))
( "skep"  n ( s k e1 p ) ((pos "K6$" ) ))
( "skeps"  n ( s k e1 p s ) ((pos "Kj$" ) ))
( "skeptical"  j ( s k e1 p t i k l ) ((pos "OA$" ) ))
( "skeptically"  a ( s k e1 p t i k l ii ) ((pos "Pu$" ) ))
( "skepticism"  n ( s k e1 p t i s i z @ m ) ((pos "L@$" ) ))
( "sketch"  v ( s k e1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" )))
( "sketch-block"  n ( s k e1 ch - b l o k ) ((pos "K6%" ) ))
( "sketch-blocks"  n ( s k e1 ch - b l o k s ) ((pos "Kj%" ) ))
( "sketch-book"  n ( s k e1 ch - b u k ) ((pos "K6%" ) ))
( "sketch-books"  n ( s k e1 ch - b u k s ) ((pos "Kj%" ) ))
( "sketch-map"  n ( s k e1 ch - m a p ) ((pos "K6%" ) ))
( "sketch-maps"  n ( s k e1 ch - m a p s ) ((pos "Kj%" ) ))
( "sketched"  v ( s k e1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "sketcher"  n ( s k e1 ch @ r ) ((pos "K6%" ) ))
( "sketchers"  n ( s k e1 ch @ z ) ((pos "Kj%" ) ))
( "sketches"  v ( s k e1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "sketchier"  j ( s k e1 ch i@ r ) ((pos "Or%" ) ))
( "sketchiest"  j ( s k e1 ch i i s t ) ((pos "Os%" ) ))
( "sketchily"  a ( s k e1 ch i l ii ) ((pos "Pu%" ) ))
( "sketchiness"  n ( s k e1 ch i n @ s ) ((pos "L@%" ) ))
( "sketching"  v ( s k e1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "sketchy"  j ( s k e1 ch ii ) ((pos "OD%" ) ))
( "skew"  j ( s k y uu1 ) ((pos "OA%" ) ))
( "skew-eyed"  j ( s k y uu1 - ai d ) ((pos "OA$" ) ))
( "skewer"  v ( s k y u@1 r ) ((pos "H0%" "K6%" ) (subcat )))
( "skewered"  v ( s k y u@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "skewering"  v ( s k y u@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "skewers"  v ( s k y u@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ski"  v ( s k ii1 ) ((pos "I5%" "K6%" ) ))
( "ski'd"  v ( s k ii1 d ) ((pos "Ic%" "Id%" ) ))
( "ski-bob"  n ( s k ii1 - b o b ) ((pos "K6$" ) ))
( "ski-bobs"  n ( s k ii1 - b o b z ) ((pos "Kj$" ) ))
( "ski-jump"  n ( s k ii1 - jh uh m p ) ((pos "K6%" ) ))
( "ski-jumps"  n ( s k ii1 - jh uh m p s ) ((pos "Kj%" ) ))
( "ski-lift"  n ( s k ii1 - l i f t ) ((pos "K6%" ) ))
( "ski-lifts"  n ( s k ii1 - l i f t s ) ((pos "Kj%" ) ))
( "ski-plane"  n ( s k ii1 - p l ei n ) ((pos "K6%" ) ))
( "ski-planes"  n ( s k ii1 - p l ei n z ) ((pos "Kj%" ) ))
( "skid"  v ( s k i1 d ) ((pos "I4%" "K6%" ) (subcat )))
( "skidded"  v ( s k i1 d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "skidding"  v ( s k i1 d i ng ) ((pos "Ib%" ) (subcat )))
( "skidpan"  n ( s k i1 d p a n ) ((pos "K6%" ) ))
( "skidpans"  n ( s k i1 d p a n z ) ((pos "Kj%" ) ))
( "skids"  v ( s k i1 d z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "skied"  v ( s k ai1 d ) ((pos "Hc%" "Hd%" ) ))
( "skier"  n ( s k ii1 @ r ) ((pos "K6%" ) ))
( "skiers"  n ( s k ii1 @ z ) ((pos "Kj%" ) ))
( "skies"  v ( s k ai1 z ) ((pos "Ha%" "Kj%" ) ))
( "skiff"  n ( s k i1 f ) ((pos "K6$" ) ))
( "skiffle"  n ( s k i1 f l ) ((pos "L@%" ) ))
( "skiffle-group"  n ( s k i1 f l - g r uu p ) ((pos "K6%" ) ))
( "skiffle-groups"  n ( s k i1 f l - g r uu p s ) ((pos "Kj%" ) ))
( "skiffs"  n ( s k i1 f s ) ((pos "Kj$" ) ))
( "skiing"  v ( s k ii1 i ng ) ((pos "Ib%" ) ))
( "skilful"  j ( s k i1 l f @ l ) ((pos "OA%" ) ))
( "skilfully"  a ( s k i1 l f @ l ii ) ((pos "Pu%" ) ))
( "skill"  n ( s k i1 l ) ((pos "M6%" ) ))
( "skilled"  j ( s k i1 l d ) ((pos "OA%" ) ))
( "skillet"  n ( s k i1 l i t ) ((pos "K6$" ) ))
( "skillets"  n ( s k i1 l i t s ) ((pos "Kj$" ) ))
( "skills"  n ( s k i1 l z ) ((pos "Mj%" ) ))
( "skilly"  n ( s k i1 l ii ) ((pos "L@$" ) ))
( "skim"  v ( s k i1 m ) ((pos "J4%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "skimmed"  v ( s k i1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "skimmed-milk"  n ( s k i1 m d - m i1 l k ) ((pos "L@%" ) ))
( "skimmer"  n ( s k i1 m @ r ) ((pos "K6$" ) ))
( "skimmers"  n ( s k i1 m @ z ) ((pos "Kj$" ) ))
( "skimming"  v ( s k i1 m i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "skimp"  v ( s k i1 m p ) ((pos "J0%" ) (subcat "2A" )))
( "skimped"  v ( s k i1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "skimpier"  j ( s k i1 m p i@ r ) ((pos "Or%" ) ))
( "skimpiest"  j ( s k i1 m p i i s t ) ((pos "Os%" ) ))
( "skimpily"  a ( s k i1 m p i l ii ) ((pos "Pu%" ) ))
( "skimping"  v ( s k i1 m p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "skimps"  v ( s k i1 m p s ) ((pos "Ja%" ) (subcat "2A" )))
( "skimpy"  j ( s k i1 m p ii ) ((pos "OD%" ) ))
( "skims"  v ( s k i1 m z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "skin"  v ( s k i1 n ) ((pos "J4%" "M6%" ) (subcat "2C" "6A" )))
( "skin-deep"  j ( s k i1 n - d ii1 p ) ((pos "OA%" ) ))
( "skin-diving"  n ( s k i1 n - d ai v i ng ) ((pos "L@%" ) ))
( "skin-graft"  n ( s k i1 n - g r aa f t ) ((pos "K6%" ) ))
( "skin-grafts"  n ( s k i1 n - g r aa f t s ) ((pos "Kj%" ) ))
( "skin-tight"  j ( s k i1 n - t ai t ) ((pos "OA%" ) ))
( "skinflint"  n ( s k i1 n f l i n t ) ((pos "K6%" ) ))
( "skinflints"  n ( s k i1 n f l i n t s ) ((pos "Kj%" ) ))
( "skinhead"  n ( s k i1 n h e d ) ((pos "K6%" ) ))
( "skinheads"  n ( s k i1 n h e d z ) ((pos "Kj%" ) ))
( "skinned"  v ( s k i1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "skinnier"  j ( s k i1 n i@ r ) ((pos "Or%" ) ))
( "skinniest"  j ( s k i1 n i i s t ) ((pos "Os%" ) ))
( "skinning"  v ( s k i1 n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "skinny"  j ( s k i1 n ii ) ((pos "OD%" ) ))
( "skins"  v ( s k i1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "skint"  j ( s k i1 n t ) ((pos "OA%" ) ))
( "skip"  v ( s k i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "skipped"  v ( s k i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "skipper"  v ( s k i1 p @ r ) ((pos "H0%" "K6%" ) ))
( "skippered"  v ( s k i1 p @ d ) ((pos "Hc%" "Hd%" ) ))
( "skippering"  v ( s k i1 p @ r i ng ) ((pos "Hb%" ) ))
( "skippers"  v ( s k i1 p @ z ) ((pos "Ha%" "Kj%" ) ))
( "skipping"  v ( s k i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "skipping-rope"  n ( s k i1 p i ng - r ou p ) ((pos "K6%" ) ))
( "skipping-ropes"  n ( s k i1 p i ng - r ou p s ) ((pos "Kj%" ) ))
( "skips"  v ( s k i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "skirl"  n ( s k @@1 l ) ((pos "K6$" ) ))
( "skirls"  n ( s k @@1 l z ) ((pos "Kj$" ) ))
( "skirmish"  v ( s k @@1 m i sh ) ((pos "I1$" "K7%" ) (subcat "2A" )))
( "skirmished"  v ( s k @@1 m i sh t ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "skirmisher"  n ( s k @@1 m i sh @ r ) ((pos "K6$" ) ))
( "skirmishers"  n ( s k @@1 m i sh @ z ) ((pos "Kj$" ) ))
( "skirmishes"  v ( s k @@1 m i sh i z ) ((pos "Ia$" "Kj%" ) (subcat "2A" )))
( "skirmishing"  v ( s k @@1 m i sh i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "skirt"  v ( s k @@1 t ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "skirted"  v ( s k @@1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "skirting"  v ( s k @@1 t i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "skirting-board"  n ( s k @@1 t i ng - b oo d ) ((pos "K6%" ) ))
( "skirting-boards"  n ( s k @@1 t i ng - b oo d z ) ((pos "Kj%" ) ))
( "skirts"  v ( s k @@1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "skis"  v ( s k ii1 z ) ((pos "Ia%" "Kj%" ) ))
( "skit"  n ( s k i1 t ) ((pos "K6%" ) ))
( "skits"  n ( s k i1 t s ) ((pos "Kj%" ) ))
( "skitter"  v ( s k i1 t @ r ) ((pos "I0%" ) (subcat )))
( "skittered"  v ( s k i1 t @ d ) ((pos "Ic%" ) (subcat )))
( "skittering"  v ( s k i1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "skitters"  v ( s k i1 t @ z ) ((pos "Ia%" ) (subcat )))
( "skittish"  j ( s k i1 t i sh ) ((pos "OA%" ) ))
( "skittishly"  a ( s k i1 t i sh l ii ) ((pos "Pu%" ) ))
( "skittishness"  n ( s k i1 t i sh n @ s ) ((pos "L@%" ) ))
( "skittle"  v ( s k i1 t l ) ((pos "H2%" "K6%" ) (subcat )))
( "skittle-pin"  n ( s k i1 t l - p i n ) ((pos "K6$" ) ))
( "skittle-pins"  n ( s k i1 t l - p i n z ) ((pos "Kj$" ) ))
( "skittled"  v ( s k i1 t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "skittles"  v ( s k i1 t l z ) ((pos "Ha%" "Kj%" "Lk%" ) (subcat )))
( "skittling"  v ( s k i1 t l i ng ) ((pos "Hb%" ) (subcat )))
( "skivvies"  n ( s k i1 v i z ) ((pos "Kj$" ) ))
( "skivvy"  n ( s k i1 v ii ) ((pos "K8$" ) ))
( "skua"  n ( s k y uu1 @ ) ((pos "K6$" ) ))
( "skuas"  n ( s k y uu1 @ z ) ((pos "Kj$" ) ))
( "skulk"  v ( s k uh1 l k ) ((pos "I0%" ) (subcat "2A" )))
( "skulked"  v ( s k uh1 l k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "skulker"  n ( s k uh1 l k @ r ) ((pos "K6$" ) ))
( "skulkers"  n ( s k uh1 l k @ z ) ((pos "Kj$" ) ))
( "skulking"  v ( s k uh1 l k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "skulks"  v ( s k uh1 l k s ) ((pos "Ia%" ) (subcat "2A" )))
( "skull"  n ( s k uh1 l ) ((pos "K6%" ) ))
( "skullcap"  n ( s k uh1 l k a p ) ((pos "K6%" ) ))
( "skullcaps"  n ( s k uh1 l k a p s ) ((pos "Kj%" ) ))
( "skullduggery"  n ( s k uh1 l d uh1 g @ r ii ) ((pos "Ki%" ) ))
( "skulled"  j ( s k uh1 l d ) ((pos "Ot%" ) ))
( "skulls"  n ( s k uh1 l z ) ((pos "Kj%" ) ))
( "skunk"  n ( s k uh1 ng k ) ((pos "M6%" ) ))
( "skunks"  n ( s k uh1 ng k s ) ((pos "Mj%" ) ))
( "sky"  v ( s k ai1 ) ((pos "H3%" "K8%" ) ))
( "sky-blue"  n ( s k ai1 - b l uu1 ) ((pos "L@%" "OA%" ) ))
( "sky-high"  a ( s k ai1 - h ai1 ) ((pos "Pu%" ) ))
( "skying"  v ( s k ai1 i ng ) ((pos "Hb$" ) ))
( "skylark"  v ( s k ai1 l aa k ) ((pos "I0%" "K6%" ) ))
( "skylarked"  v ( s k ai1 l aa k t ) ((pos "Ic%" "Id%" ) ))
( "skylarking"  v ( s k ai1 l aa k i ng ) ((pos "Ib%" ) ))
( "skylarks"  v ( s k ai1 l aa k s ) ((pos "Ia%" "Kj%" ) ))
( "skylight"  n ( s k ai1 l ai t ) ((pos "K6%" ) ))
( "skylights"  n ( s k ai1 l ai t s ) ((pos "Kj%" ) ))
( "skyline"  n ( s k ai1 l ai n ) ((pos "K6%" ) ))
( "skylines"  n ( s k ai1 l ai n z ) ((pos "Kj$" ) ))
( "skyrocket"  v ( s k ai1 r o k i t ) ((pos "I0$" ) ))
( "skyrocketed"  v ( s k ai1 r o k i t i d ) ((pos "Ic$" "Id$" ) ))
( "skyrocketing"  v ( s k ai1 r o k i t i ng ) ((pos "Ib$" ) ))
( "skyrockets"  v ( s k ai1 r o k i t s ) ((pos "Ia$" ) ))
( "skyscraper"  n ( s k ai1 s k r ei p @ r ) ((pos "K6%" ) ))
( "skyscrapers"  n ( s k ai1 s k r ei p @ z ) ((pos "Kj%" ) ))
( "skyward"  j ( s k ai1 w @ d ) ((pos "OA%" "Pu%" ) ))
( "skywards"  j ( s k ai1 w @ d z ) ((pos "OA%" "Pu%" ) ))
( "skywriting"  n ( s k ai1 r ai t i ng ) ((pos "L@%" ) ))
( "slab"  n ( s l a1 b ) ((pos "K6%" ) ))
( "slabs"  n ( s l a1 b z ) ((pos "Kj%" ) ))
( "slack"  v ( s l a1 k ) ((pos "I0%" "M6%" "OC%" ) (subcat "2A" )))
( "slacked"  v ( s l a1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "slacken"  v ( s l a1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "slackened"  v ( s l a1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "slackening"  v ( s l a1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "slackens"  v ( s l a1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "slacker"  n ( s l a1 k @ r ) ((pos "K6%" "Or%" ) ))
( "slackers"  n ( s l a1 k @ z ) ((pos "Kj%" ) ))
( "slackest"  j ( s l a1 k i s t ) ((pos "Os%" ) ))
( "slacking"  v ( s l a1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "slackly"  a ( s l a1 k l ii ) ((pos "Pu%" ) ))
( "slackness"  n ( s l a1 k n @ s ) ((pos "L@%" ) ))
( "slacks"  v ( s l a1 k s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "slag"  n ( s l a1 g ) ((pos "L@%" ) ))
( "slag-heap"  n ( s l a1 g - h ii p ) ((pos "K6%" ) ))
( "slag-heaps"  n ( s l a1 g - h ii p s ) ((pos "Kj%" ) ))
( "slain"  v ( s l ei1 n ) ((pos "Hd$" ) (subcat )))
( "slake"  v ( s l ei1 k ) ((pos "H2%" ) (subcat )))
( "slaked"  v ( s l ei1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slakes"  v ( s l ei1 k s ) ((pos "Ha%" ) (subcat )))
( "slaking"  v ( s l ei1 k i ng ) ((pos "Hb%" ) (subcat )))
( "slalom"  n ( s l aa1 l @ m ) ((pos "K6%" ) ))
( "slaloms"  n ( s l aa1 l @ m z ) ((pos "Kj$" ) ))
( "slam"  v ( s l a1 m ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "slammed"  v ( s l a1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "slamming"  v ( s l a1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "slams"  v ( s l a1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "slander"  v ( s l aa1 n d @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "slandered"  v ( s l aa1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slanderer"  n ( s l aa1 n d @ r @ r ) ((pos "K6%" ) ))
( "slanderers"  n ( s l aa1 n d @ r @ z ) ((pos "Kj%" ) ))
( "slandering"  v ( s l aa1 n d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "slanderous"  j ( s l aa1 n d @ r @ s ) ((pos "OA%" ) ))
( "slanders"  v ( s l aa1 n d @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "slang"  v ( s l a1 ng ) ((pos "H0%" "L@%" ) (subcat )))
( "slanged"  v ( s l a1 ng d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slangier"  j ( s l a1 ng i@ r ) ((pos "Or$" ) ))
( "slangiest"  j ( s l a1 ng i i s t ) ((pos "Os$" ) ))
( "slangily"  a ( s l a1 ng i l ii ) ((pos "Pu$" ) ))
( "slanginess"  n ( s l a1 ng i n @ s ) ((pos "L@%" ) ))
( "slanging"  v ( s l a1 ng i ng ) ((pos "Hb%" ) (subcat )))
( "slangs"  v ( s l a1 ng z ) ((pos "Ha%" ) (subcat )))
( "slangy"  j ( s l a1 ng ii ) ((pos "OD%" ) ))
( "slant"  v ( s l aa1 n t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "slanted"  v ( s l aa1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "slanting"  v ( s l aa1 n t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "slantingly"  a ( s l aa1 n t i ng l ii ) ((pos "Pu%" ) ))
( "slants"  v ( s l aa1 n t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "slantwise"  a ( s l aa1 n t w ai z ) ((pos "Pu%" ) ))
( "slap"  v ( s l a1 p ) ((pos "H4%" "K6%" "Pu%" ) (subcat "6A" "15A" )))
( "slap-bang"  a ( s l a1 p - b a1 ng ) ((pos "Pu%" ) ))
( "slap-happy"  j ( s l a1 p - h a1 p ii ) ((pos "OA%" ) ))
( "slap-up"  j ( s l a1 p - uh p ) ((pos "OA%" ) ))
( "slapdash"  j ( s l a1 p d a sh ) ((pos "OA%" "Pu%" ) ))
( "slapped"  v ( s l a1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "slapping"  v ( s l a1 p i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "slaps"  v ( s l a1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "slapstick"  n ( s l a1 p s t i k ) ((pos "L@%" ) ))
( "slash"  v ( s l a1 sh ) ((pos "J1%" "K7%" ) (subcat "2C" )))
( "slashed"  v ( s l a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "slashes"  v ( s l a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "slashing"  v ( s l a1 sh i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "slat"  n ( s l a1 t ) ((pos "K6%" ) ))
( "slate"  v ( s l ei1 t ) ((pos "H2%" "M6%" ) (subcat )))
( "slate-club"  n ( s l ei1 t - k l uh b ) ((pos "K6$" ) ))
( "slate-clubs"  n ( s l ei1 t - k l uh b z ) ((pos "Kj$" ) ))
( "slate-pencil"  n ( s l ei1 t - p e n s l ) ((pos "K6$" ) ))
( "slate-pencils"  n ( s l ei1 t - p e n s l z ) ((pos "Kj$" ) ))
( "slated"  v ( s l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slates"  v ( s l ei1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "slatier"  j ( s l ei1 t i@ r ) ((pos "Or$" ) ))
( "slatiest"  j ( s l ei1 t i i s t ) ((pos "Os$" ) ))
( "slating"  v ( s l ei1 t i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "slatings"  n ( s l ei1 t i ng z ) ((pos "Kj$" ) ))
( "slats"  n ( s l a1 t s ) ((pos "Kj%" ) ))
( "slatted"  j ( s l a1 t i d ) ((pos "OA%" ) ))
( "slattern"  n ( s l a1 t @ n ) ((pos "K6%" ) ))
( "slatternliness"  n ( s l a1 t @ n l i n @ s ) ((pos "L@%" ) ))
( "slatternly"  j ( s l a1 t @ n l ii ) ((pos "OA%" ) ))
( "slatterns"  n ( s l a1 t @ n z ) ((pos "Kj%" ) ))
( "slaty"  j ( s l ei1 t ii ) ((pos "OD$" ) ))
( "slaughter"  v ( s l oo1 t @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "slaughtered"  v ( s l oo1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slaughterer"  n ( s l oo1 t @ r @ r ) ((pos "K6%" ) ))
( "slaughterers"  n ( s l oo1 t @ r @ z ) ((pos "Kj%" ) ))
( "slaughterhouse"  n ( s l oo1 t @ h au s ) ((pos "K6%" ) ))
( "slaughterhouses"  n ( s l oo1 t @ h au z i z ) ((pos "Kj%" ) ))
( "slaughtering"  v ( s l oo1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "slaughters"  v ( s l oo1 t @ z ) ((pos "Ha%" ) (subcat )))
( "slave"  v ( s l ei1 v ) ((pos "I2%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "slave-driver"  n ( s l ei1 v - d r ai v @ r ) ((pos "K6%" ) ))
( "slave-drivers"  n ( s l ei1 v - d r ai v @ z ) ((pos "Kj%" ) ))
( "slave-trade"  n ( s l ei1 v - t r ei d ) ((pos "L@%" ) ))
( "slave-traffic"  n ( s l ei1 v - t r a f i k ) ((pos "L@%" ) ))
( "slaved"  v ( s l ei1 v d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" )))
( "slaver"  v ( s l a1 v @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "slavered"  v ( s l a1 v @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "slavering"  v ( s l a1 v @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "slavers"  v ( s l a1 v @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "slavery"  n ( s l ei1 v @ r ii ) ((pos "L@%" ) ))
( "slaves"  v ( s l ei1 v z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "slavey"  n ( s l ei1 v ii ) ((pos "K6$" ) ))
( "slaveys"  n ( s l ei1 v i z ) ((pos "Kj$" ) ))
( "slaving"  v ( s l ei1 v i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" )))
( "slavish"  j ( s l ei1 v i sh ) ((pos "OA%" ) ))
( "slavishly"  a ( s l ei1 v i sh l ii ) ((pos "Pu%" ) ))
( "slaw"  n ( s l oo1 ) ((pos "L@$" ) ))
( "slay"  v ( s l ei1 ) ((pos "H5%" ) (subcat )))
( "slayer"  n ( s l ei1 @ r ) ((pos "K6%" ) ))
( "slayers"  n ( s l ei1 @ z ) ((pos "Kj%" ) ))
( "slaying"  v ( s l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "slays"  v ( s l ei1 z ) ((pos "Ha%" ) (subcat )))
( "sleazier"  j ( s l ii1 z i@ r ) ((pos "Or%" ) ))
( "sleaziest"  j ( s l ii1 z i i s t ) ((pos "Os%" ) ))
( "sleazy"  j ( s l ii1 z ii ) ((pos "OD%" ) ))
( "sled"  n ( s l e1 d ) ((pos "K6$" ) ))
( "sledge"  v ( s l e1 jh ) ((pos "J2%" "K6%" ) ))
( "sledged"  v ( s l e1 jh d ) ((pos "Jc%" "Jd%" ) ))
( "sledgehammer"  n ( s l e1 jh h a m @ r ) ((pos "K6%" ) ))
( "sledgehammers"  n ( s l e1 jh h a m @ z ) ((pos "Kj%" ) ))
( "sledges"  v ( s l e1 jh i z ) ((pos "Ja%" "Kj%" ) ))
( "sledging"  v ( s l e1 jh i ng ) ((pos "Jb%" ) ))
( "sleds"  n ( s l e1 d z ) ((pos "Kj$" ) ))
( "sleek"  v ( s l ii1 k ) ((pos "H0%" "OC%" ) (subcat )))
( "sleeked"  v ( s l ii1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sleeker"  j ( s l ii1 k @ r ) ((pos "Or%" ) ))
( "sleekest"  j ( s l ii1 k i s t ) ((pos "Os%" ) ))
( "sleeking"  v ( s l ii1 k i ng ) ((pos "Hb%" ) (subcat )))
( "sleekly"  a ( s l ii1 k l ii ) ((pos "Pu%" ) ))
( "sleekness"  n ( s l ii1 k n @ s ) ((pos "L@%" ) ))
( "sleeks"  v ( s l ii1 k s ) ((pos "Ha%" ) (subcat )))
( "sleep"  v ( s l ii1 p ) ((pos "J5%" "L@%" ) (subcat "2A" "2B" "2C" )))
( "sleeper"  n ( s l ii1 p @ r ) ((pos "K6%" ) ))
( "sleepers"  n ( s l ii1 p @ z ) ((pos "Kj%" ) ))
( "sleepier"  j ( s l ii1 p i@ r ) ((pos "Or%" ) ))
( "sleepiest"  j ( s l ii1 p i i s t ) ((pos "Os%" ) ))
( "sleepily"  a ( s l ii1 p i l ii ) ((pos "Pu%" ) ))
( "sleepiness"  n ( s l ii1 p i n @ s ) ((pos "L@%" ) ))
( "sleeping"  v ( s l ii1 p i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" )))
( "sleeping-bag"  n ( s l ii1 p i ng - b a g ) ((pos "K6%" ) ))
( "sleeping-bags"  n ( s l ii1 p i ng - b a g z ) ((pos "Kj%" ) ))
( "sleeping-car"  n ( s l ii1 p i ng - k aa r ) ((pos "K6%" ) ))
( "sleeping-cars"  n ( s l ii1 p i ng - k aa z ) ((pos "Kj%" ) ))
( "sleeping-draught"  n ( s l ii1 p i ng - d r aa f t ) ((pos "K6%" ) ))
( "sleeping-draughts"  n ( s l ii1 p i ng - d r aa f t s ) ((pos "Kj%" ) ))
( "sleeping-pill"  n ( s l ii1 p i ng - p i l ) ((pos "K6%" ) ))
( "sleeping-pills"  n ( s l ii1 p i ng - p i l z ) ((pos "Kj%" ) ))
( "sleeping-sickness"  n ( s l ii1 p i ng - s i k n @ s ) ((pos "L@%" ) ))
( "sleepless"  j ( s l ii1 p l @ s ) ((pos "OA%" ) ))
( "sleeplessly"  a ( s l ii1 p l @ s l ii ) ((pos "Pu%" ) ))
( "sleeplessness"  n ( s l ii1 p l @ s n @ s ) ((pos "L@%" ) ))
( "sleeps"  v ( s l ii1 p s ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" )))
( "sleepwalker"  n ( s l ii1 p w oo k @ r ) ((pos "K6%" ) ))
( "sleepwalkers"  n ( s l ii1 p w oo k @ z ) ((pos "Kj%" ) ))
( "sleepy"  j ( s l ii1 p ii ) ((pos "OD%" ) ))
( "sleepy-head"  n ( s l ii1 p i - h e d ) ((pos "K6%" ) ))
( "sleepy-heads"  n ( s l ii1 p i - h e d z ) ((pos "Kj%" ) ))
( "sleet"  v ( s l ii1 t ) ((pos "I0%" "L@%" ) (subcat )))
( "sleeted"  v ( s l ii1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sleetier"  j ( s l ii1 t i@ r ) ((pos "Or$" ) ))
( "sleetiest"  j ( s l ii1 t i i s t ) ((pos "Os$" ) ))
( "sleeting"  v ( s l ii1 t i ng ) ((pos "Ib%" ) (subcat )))
( "sleets"  v ( s l ii1 t s ) ((pos "Ia$" ) (subcat )))
( "sleety"  j ( s l ii1 t ii ) ((pos "OD$" ) ))
( "sleeve"  n ( s l ii1 v ) ((pos "K6%" ) ))
( "sleeved"  j ( s l ii1 v d ) ((pos "Ot%" ) ))
( "sleeveless"  j ( s l ii1 v l @ s ) ((pos "OA%" ) ))
( "sleeves"  n ( s l ii1 v z ) ((pos "Kj%" ) ))
( "sleigh"  v ( s l ei1 ) ((pos "J0$" "K6%" ) (subcat "2A" )))
( "sleigh-bell"  n ( s l ei1 - b e l ) ((pos "K6%" ) ))
( "sleigh-bells"  n ( s l ei1 - b e l z ) ((pos "Kj%" ) ))
( "sleighed"  v ( s l ei1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "sleighing"  v ( s l ei1 i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "sleighs"  v ( s l ei1 z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "sleight"  n ( s l ai1 t ) ((pos "K6%" ) ))
( "sleights"  n ( s l ai1 t s ) ((pos "Kj$" ) ))
( "slender"  j ( s l e1 n d @ r ) ((pos "OA%" ) ))
( "slenderize"  v ( s l e1 n d @ r ai z ) ((pos "J2$" ) (subcat )))
( "slenderized"  v ( s l e1 n d @ r ai z d ) ((pos "Jc$" "Jd$" ) (subcat )))
( "slenderizes"  v ( s l e1 n d @ r ai z i z ) ((pos "Ja$" ) (subcat )))
( "slenderizing"  v ( s l e1 n d @ r ai z i ng ) ((pos "Jb$" ) (subcat )))
( "slenderly"  a ( s l e1 n d @ l ii ) ((pos "Pu$" ) ))
( "slenderness"  n ( s l e1 n d @ n @ s ) ((pos "L@%" ) ))
( "slept"  v ( s l e1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "sleuth"  n ( s l uu1 th ) ((pos "K6%" ) ))
( "sleuth-hound"  n ( s l uu1 th - h au n d ) ((pos "K6%" ) ))
( "sleuth-hounds"  n ( s l uu1 th - h au n d z ) ((pos "Kj%" ) ))
( "sleuths"  n ( s l uu1 th s ) ((pos "Kj%" ) ))
( "slew"  v ( s l uu1 ) ((pos "Hc%" "J0$" ) (subcat "2C" "6A" )))
( "slewed"  v ( s l uu1 d ) ((pos "Jc$" "Jd$" ) (subcat "2C" "6A" )))
( "slewing"  v ( s l uu1 i ng ) ((pos "Jb$" ) (subcat "2C" "6A" )))
( "slews"  v ( s l uu1 z ) ((pos "Ja$" ) (subcat "2C" "6A" )))
( "slice"  v ( s l ai1 s ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "15A" "15B" )))
( "sliced"  v ( s l ai1 s t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "15A" "15B" )))
( "slices"  v ( s l ai1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "15A" "15B" )))
( "slicing"  v ( s l ai1 s i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "15A" "15B" )))
( "slick"  n ( s l i1 k ) ((pos "K6%" "OC%" "Pu%" ) ))
( "slicker"  n ( s l i1 k @ r ) ((pos "K6%" "Or%" ) ))
( "slickers"  n ( s l i1 k @ z ) ((pos "Kj%" ) ))
( "slickest"  j ( s l i1 k i s t ) ((pos "Os%" ) ))
( "slicks"  n ( s l i1 k s ) ((pos "Kj%" ) ))
( "slid"  v ( s l i1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "slide"  v ( s l ai1 d ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "slide-rule"  n ( s l ai1 d - r uu l ) ((pos "K6%" ) ))
( "slide-rules"  n ( s l ai1 d - r uu l z ) ((pos "Kj%" ) ))
( "slides"  v ( s l ai1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sliding"  v ( s l ai1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "slight"  v ( s l ai1 t ) ((pos "H0%" "K6%" "OC%" ) (subcat )))
( "slighted"  v ( s l ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "slighter"  j ( s l ai1 t @ r ) ((pos "Or%" ) ))
( "slightest"  j ( s l ai1 t i s t ) ((pos "Os%" ) ))
( "slighting"  v ( s l ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "slightingly"  a ( s l ai1 t i ng l ii ) ((pos "Pu%" ) ))
( "slightly"  a ( s l ai1 t l ii ) ((pos "Pu%" ) ))
( "slightness"  n ( s l ai1 t n @ s ) ((pos "L@%" ) ))
( "slights"  v ( s l ai1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "slim"  v ( s l i1 m ) ((pos "I4%" "OE%" ) ))
( "slime"  n ( s l ai1 m ) ((pos "L@%" ) ))
( "slimier"  j ( s l ai1 m i@ r ) ((pos "Or%" ) ))
( "slimiest"  j ( s l ai1 m i i s t ) ((pos "Os%" ) ))
( "sliminess"  n ( s l ai1 m i n i s ) ((pos "L@%" ) ))
( "slimly"  a ( s l i1 m l ii ) ((pos "Pu$" ) ))
( "slimmed"  v ( s l i1 m d ) ((pos "Ic%" "Id%" ) ))
( "slimmer"  j ( s l i1 m @ r ) ((pos "Or%" ) ))
( "slimmest"  j ( s l i1 m i s t ) ((pos "Os%" ) ))
( "slimming"  v ( s l i1 m i ng ) ((pos "Ib%" ) ))
( "slimness"  n ( s l i1 m n @ s ) ((pos "L@%" ) ))
( "slims"  v ( s l i1 m z ) ((pos "Ia%" ) ))
( "slimy"  j ( s l ai1 m ii ) ((pos "OD%" ) ))
( "sling"  v ( s l i1 ng ) ((pos "J5%" "K6%" ) (subcat "6A" "15A" )))
( "slinger"  n ( s l i1 ng @ r ) ((pos "K6$" ) ))
( "slingers"  n ( s l i1 ng @ z ) ((pos "Kj$" ) ))
( "slinging"  v ( s l i1 ng i ng ) ((pos "Jb%" ) (subcat "6A" "15A" )))
( "slings"  v ( s l i1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "6A" "15A" )))
( "slink"  v ( s l i1 ng k ) ((pos "I5%" ) (subcat )))
( "slinking"  v ( s l i1 ng k i ng ) ((pos "Ib%" ) (subcat )))
( "slinks"  v ( s l i1 ng k s ) ((pos "Ia%" ) (subcat )))
( "slip"  v ( s l i1 p ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slip-carriage"  n ( s l i1 p - k a r i jh ) ((pos "K6$" ) ))
( "slip-carriages"  n ( s l i1 p - k a r i jh i z ) ((pos "Kj$" ) ))
( "slip-coach"  n ( s l i1 p - k ou ch ) ((pos "K7$" ) ))
( "slip-coaches"  n ( s l i1 p - k ou ch i z ) ((pos "Kj$" ) ))
( "slip-road"  n ( s l i1 p - r ou d ) ((pos "K6%" ) ))
( "slip-roads"  n ( s l i1 p - r ou d z ) ((pos "Kj%" ) ))
( "slip-up"  n ( s l i1 p - uh p ) ((pos "K6%" ) ))
( "slip-ups"  n ( s l i1 p - uh p s ) ((pos "Kj%" ) ))
( "slipcover"  n ( s l i1 p k uh v @ r ) ((pos "K6%" ) ))
( "slipcovers"  n ( s l i1 p k uh v @ z ) ((pos "Kj%" ) ))
( "slipknot"  n ( s l i1 p n o t ) ((pos "K6%" ) ))
( "slipknots"  n ( s l i1 p n o t s ) ((pos "Kj%" ) ))
( "slipon"  n ( s l i1 p o n ) ((pos "K6%" ) ))
( "slipons"  n ( s l i1 p o n z ) ((pos "Kj$" ) ))
( "slipover"  n ( s l i1 p ou v @ r ) ((pos "K6%" ) ))
( "slipovers"  n ( s l i1 p ou v @ z ) ((pos "Kj%" ) ))
( "slipped"  v ( s l i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slipper"  n ( s l i1 p @ r ) ((pos "K6%" ) ))
( "slippered"  j ( s l i1 p @ d ) ((pos "OA%" ) ))
( "slipperier"  j ( s l i1 p @ r i@ r ) ((pos "Or%" ) ))
( "slipperiest"  j ( s l i1 p @ r i i s t ) ((pos "Os%" ) ))
( "slipperiness"  n ( s l i1 p @ r i n @ s ) ((pos "L@%" ) ))
( "slippers"  n ( s l i1 p @ z ) ((pos "Kj%" ) ))
( "slippery"  j ( s l i1 p @ r ii ) ((pos "OD%" ) ))
( "slipping"  v ( s l i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slippy"  j ( s l i1 p ii ) ((pos "OA%" ) ))
( "slips"  v ( s l i1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slipshod"  j ( s l i1 p sh o d ) ((pos "OA%" ) ))
( "slipstream"  n ( s l i1 p s t r ii m ) ((pos "K6%" ) ))
( "slipstreams"  n ( s l i1 p s t r ii m z ) ((pos "Kj$" ) ))
( "slipway"  n ( s l i1 p w ei ) ((pos "K6%" ) ))
( "slipways"  n ( s l i1 p w ei z ) ((pos "Kj%" ) ))
( "slit"  v ( s l i1 t ) ((pos "H5%" "Hc%" "Hd%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slither"  v ( s l i1 dh @ r ) ((pos "I0%" ) (subcat "2A" )))
( "slithered"  v ( s l i1 dh @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "slithering"  v ( s l i1 dh @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "slithers"  v ( s l i1 dh @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "slithery"  j ( s l i1 dh @ r ii ) ((pos "OA%" ) ))
( "slits"  v ( s l i1 t s ) ((pos "Ha%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "slitting"  v ( s l i1 t i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "sliver"  v ( s l i1 v @ r ) ((pos "J0$" "K6%" ) (subcat "2A" )))
( "slivered"  v ( s l i1 v @ d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "slivering"  v ( s l i1 v @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "slivers"  v ( s l i1 v @ z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "slob"  n ( s l o1 b ) ((pos "K6%" ) ))
( "slobber"  v ( s l o1 b @ r ) ((pos "J0%" "L@$" ) (subcat "2A" "3A" )))
( "slobbered"  v ( s l o1 b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "slobbering"  v ( s l o1 b @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "slobbers"  v ( s l o1 b @ z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "slobs"  n ( s l o1 b z ) ((pos "Kj%" ) ))
( "sloe"  n ( s l ou1 ) ((pos "K6$" ) ))
( "sloe-gin"  n ( s l ou1 - jh i n ) ((pos "L@$" ) ))
( "sloes"  n ( s l ou1 z ) ((pos "Kj$" ) ))
( "slog"  v ( s l o1 g ) ((pos "J4%" ) (subcat "2C" "3A" )))
( "slogan"  n ( s l ou1 g @ n ) ((pos "K6%" ) ))
( "slogans"  n ( s l ou1 g @ n z ) ((pos "Kj%" ) ))
( "slogged"  v ( s l o1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" )))
( "slogger"  n ( s l o1 g @ r ) ((pos "K6%" ) ))
( "sloggers"  n ( s l o1 g @ z ) ((pos "Kj%" ) ))
( "slogging"  v ( s l o1 g i ng ) ((pos "Jb%" ) (subcat "2C" "3A" )))
( "slogs"  v ( s l o1 g z ) ((pos "Ja%" ) (subcat "2C" "3A" )))
( "sloop"  n ( s l uu1 p ) ((pos "K6%" ) ))
( "sloops"  n ( s l uu1 p s ) ((pos "Kj%" ) ))
( "slop"  v ( s l o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "slop-basin"  n ( s l o1 p - b ei s n ) ((pos "K6%" ) ))
( "slop-basins"  n ( s l o1 p - b ei s n z ) ((pos "Kj%" ) ))
( "slop-pail"  n ( s l o1 p - p ei l ) ((pos "K6%" ) ))
( "slop-pails"  n ( s l o1 p - p ei l z ) ((pos "Kj%" ) ))
( "slop-shop"  n ( s l o1 p - sh o p ) ((pos "K6$" ) ))
( "slop-shops"  n ( s l o1 p - sh o p s ) ((pos "Kj$" ) ))
( "slope"  v ( s l ou1 p ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" )))
( "sloped"  v ( s l ou1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "slopes"  v ( s l ou1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "sloping"  v ( s l ou1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "slopingly"  a ( s l ou1 p i ng l ii ) ((pos "Pu$" ) ))
( "slopped"  v ( s l o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "sloppier"  j ( s l o1 p i@ r ) ((pos "Or%" ) ))
( "sloppiest"  j ( s l o1 p i i s t ) ((pos "Os%" ) ))
( "sloppily"  a ( s l o1 p i l ii ) ((pos "Pu%" ) ))
( "sloppiness"  n ( s l o1 p i n @ s ) ((pos "L@%" ) ))
( "slopping"  v ( s l o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "sloppy"  j ( s l o1 p ii ) ((pos "OD%" ) ))
( "slops"  v ( s l o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "slosh"  v ( s l o1 sh ) ((pos "J1%" ) (subcat "3A" "6A" "15A" )))
( "sloshed"  v ( s l o1 sh t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "3A" "6A" "15A" )))
( "sloshes"  v ( s l o1 sh i z ) ((pos "Ja%" ) (subcat "3A" "6A" "15A" )))
( "sloshing"  v ( s l o1 sh i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "15A" )))
( "slot"  v ( s l o1 t ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "slot-machine"  n ( s l o1 t - m @ sh ii n ) ((pos "K6%" ) ))
( "slot-machines"  n ( s l o1 t - m @ sh ii n z ) ((pos "Kj%" ) ))
( "sloth"  n ( s l ou1 th ) ((pos "M6%" ) ))
( "slothful"  j ( s l ou1 th f @ l ) ((pos "OA%" ) ))
( "sloths"  n ( s l ou1 th s ) ((pos "Mj$" ) ))
( "slots"  v ( s l o1 t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "slotted"  v ( s l o1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "slotting"  v ( s l o1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "slouch"  v ( s l au1 ch ) ((pos "I1%" "K7%" ) (subcat "2A" )))
( "slouch-hat"  n ( s l au1 ch - h a1 t ) ((pos "K6$" ) ))
( "slouch-hats"  n ( s l au1 ch - h a1 t s ) ((pos "Kj$" ) ))
( "slouched"  v ( s l au1 ch t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "slouches"  v ( s l au1 ch i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "slouching"  v ( s l au1 ch i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "slouchingly"  a ( s l au1 ch i ng l ii ) ((pos "Pu$" ) ))
( "slough"  v ( s l uh1 f ) ((pos "J0$" "K6$" ) (subcat "2A" "6A" )))
( "slough"  n ( s l au1 ) ((pos "K6$" ) ))
( "sloughed"  v ( s l uh1 f t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "6A" )))
( "sloughing"  v ( s l uh1 f i ng ) ((pos "Jb$" ) (subcat "2A" "6A" )))
( "sloughs"  v ( s l uh1 f s ) ((pos "Ja$" "Kj$" ) (subcat "2A" "6A" )))
( "sloughs"  n ( s l au1 z ) ((pos "Kj$" ) ))
( "sloven"  n ( s l uh1 v n ) ((pos "K6$" ) ))
( "slovenlier"  j ( s l uh1 v @ n l i@ r ) ((pos "Or%" ) ))
( "slovenliest"  j ( s l uh1 v @ n l i i s t ) ((pos "Os%" ) ))
( "slovenliness"  n ( s l uh1 v n l i n @ s ) ((pos "L@%" ) ))
( "slovenly"  j ( s l uh1 v @ n l ii ) ((pos "OD%" ) ))
( "slovens"  n ( s l uh1 v n z ) ((pos "Kj$" ) ))
( "slow"  v ( s l ou1 ) ((pos "J0%" "OC%" "Pu%" ) (subcat "2C" )))
( "slow-worm"  n ( s l ou1 - w @@ m ) ((pos "K6%" ) ))
( "slow-worms"  n ( s l ou1 - w @@ m z ) ((pos "Kj%" ) ))
( "slowcoach"  n ( s l ou1 k ou ch ) ((pos "K7%" ) ))
( "slowcoaches"  n ( s l ou1 k ou ch i z ) ((pos "Kj%" ) ))
( "slowdown"  n ( s l ou1 d au n ) ((pos "K6%" ) ))
( "slowdowns"  n ( s l ou1 d au n z ) ((pos "Kj$" ) ))
( "slowed"  v ( s l ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "slower"  j ( s l ou1 @ r ) ((pos "Or%" "Pu%" ) ))
( "slowest"  j ( s l ou1 i s t ) ((pos "Os%" "Pu%" ) ))
( "slowing"  v ( s l ou1 i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "slowly"  a ( s l ou1 l ii ) ((pos "Pu%" ) ))
( "slowness"  n ( s l ou1 n @ s ) ((pos "L@%" ) ))
( "slows"  v ( s l ou1 z ) ((pos "Ja%" ) (subcat "2C" )))
( "sludge"  n ( s l uh1 jh ) ((pos "L@%" ) ))
( "slug"  v ( s l uh1 g ) ((pos "J4%" "K6%" ) ))
( "sluggard"  n ( s l uh1 g @ d ) ((pos "K6%" ) ))
( "sluggards"  n ( s l uh1 g @ d z ) ((pos "Kj%" ) ))
( "slugged"  v ( s l uh1 g d ) ((pos "Jc%" "Jd%" ) ))
( "slugging"  v ( s l uh1 g i ng ) ((pos "Jb%" ) ))
( "sluggish"  j ( s l uh1 g i sh ) ((pos "OA%" ) ))
( "sluggishly"  a ( s l uh1 g i sh l ii ) ((pos "Pu%" ) ))
( "sluggishness"  n ( s l uh1 g i sh n @ s ) ((pos "L@%" ) ))
( "slugs"  v ( s l uh1 g z ) ((pos "Ja%" "Kj%" ) ))
( "sluice"  v ( s l uu1 s ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" )))
( "sluice-valve"  n ( s l uu1 s - v a l v ) ((pos "K6%" ) ))
( "sluice-valves"  n ( s l uu1 s - v a l v z ) ((pos "Kj%" ) ))
( "sluiced"  v ( s l uu1 s t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "sluicegate"  n ( s l uu1 s g ei t ) ((pos "K6%" ) ))
( "sluicegates"  n ( s l uu1 s g ei t s ) ((pos "Kj%" ) ))
( "sluices"  v ( s l uu1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "sluicing"  v ( s l uu1 s i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "slum"  v ( s l uh1 m ) ((pos "I4%" "K6%" ) ))
( "slumber"  v ( s l uh1 m b @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "slumbered"  v ( s l uh1 m b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "slumberer"  n ( s l uh1 m b @ r @ r ) ((pos "K6%" ) ))
( "slumberers"  n ( s l uh1 m b @ r @ z ) ((pos "Kj%" ) ))
( "slumbering"  v ( s l uh1 m b @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "slumberous"  j ( s l uh1 m b @ r @ s ) ((pos "OA$" ) ))
( "slumbers"  v ( s l uh1 m b @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "slummed"  v ( s l uh1 m d ) ((pos "Ic%" "Id%" ) ))
( "slummier"  j ( s l uh1 m i@ r ) ((pos "Or%" ) ))
( "slummiest"  j ( s l uh1 m i i s t ) ((pos "Os%" ) ))
( "slumming"  v ( s l uh1 m i ng ) ((pos "Ib%" ) ))
( "slummy"  j ( s l uh1 m ii ) ((pos "OD%" ) ))
( "slump"  v ( s l uh1 m p ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "slumped"  v ( s l uh1 m p t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "slumping"  v ( s l uh1 m p i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "slumps"  v ( s l uh1 m p s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "slums"  v ( s l uh1 m z ) ((pos "Ia%" "Kj%" ) ))
( "slung"  v ( s l uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15A" )))
( "slunk"  v ( s l uh1 ng k ) ((pos "Ic%" "Id%" ) (subcat )))
( "slur"  v ( s l @@1 r ) ((pos "J4%" "M6%" ) (subcat "3A" )))
( "slurred"  v ( s l @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "slurring"  v ( s l @@1 r i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "slurry"  n ( s l uh1 r ii ) ((pos "L@$" ) ))
( "slurs"  v ( s l @@1 z ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "slush"  n ( s l uh1 sh ) ((pos "L@%" ) ))
( "slushier"  j ( s l uh1 sh i@ r ) ((pos "Or%" ) ))
( "slushiest"  j ( s l uh1 sh i i s t ) ((pos "Os%" ) ))
( "slushy"  j ( s l uh1 sh ii ) ((pos "OD%" ) ))
( "slut"  n ( s l uh1 t ) ((pos "K6%" ) ))
( "sluts"  n ( s l uh1 t s ) ((pos "Kj%" ) ))
( "sluttish"  j ( s l uh1 t i sh ) ((pos "OA%" ) ))
( "sly"  j ( s l ai1 ) ((pos "OC%" ) ))
( "slyer"  j ( s l ai1 @ r ) ((pos "Or%" ) ))
( "slyest"  j ( s l ai1 i s t ) ((pos "Os%" ) ))
( "slyly"  a ( s l ai1 l ii ) ((pos "Pu%" ) ))
( "slyness"  n ( s l ai1 n @ s ) ((pos "L@%" ) ))
( "smack"  v ( s m a1 k ) ((pos "J0%" "K6%" "Pu%" ) (subcat "3A" )))
( "smacked"  v ( s m a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "smacker"  n ( s m a1 k @ r ) ((pos "K6%" ) ))
( "smackers"  n ( s m a1 k @ z ) ((pos "Kj%" ) ))
( "smacking"  v ( s m a1 k i ng ) ((pos "Jb%" "M6%" ) (subcat "3A" )))
( "smackings"  n ( s m a1 k i ng z ) ((pos "Mj%" ) ))
( "smacks"  v ( s m a1 k s ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "small"  n ( s m oo1 l ) ((pos "K6%" "OC*" "Pu*" ) ))
( "small-arms"  n ( s m oo1 l - aa m z ) ((pos "Kj%" ) ))
( "small-minded"  j ( s m oo1 l - m ai1 n d i d ) ((pos "OA%" ) ))
( "smaller"  j ( s m oo1 l @ r ) ((pos "Or%" ) ))
( "smallest"  j ( s m oo1 l i s t ) ((pos "Os%" ) ))
( "smallholder"  n ( s m oo1 l h ou l d @ r ) ((pos "K6%" ) ))
( "smallholders"  n ( s m oo1 l h ou l d @ z ) ((pos "Kj%" ) ))
( "smallholding"  n ( s m oo1 l h ou l d i ng ) ((pos "K6%" ) ))
( "smallholdings"  n ( s m oo1 l h ou l d i ng z ) ((pos "Kj%" ) ))
( "smallness"  n ( s m oo1 l n @ s ) ((pos "L@%" ) ))
( "smallpox"  n ( s m oo1 l p o k s ) ((pos "L@%" ) ))
( "smalls"  n ( s m oo1 l z ) ((pos "Kj%" ) ))
( "smalltime"  j ( s m oo1 l t ai m ) ((pos "OA%" ) ))
( "smarmy"  j ( s m aa1 m ii ) ((pos "OA%" ) ))
( "smart"  v ( s m aa1 t ) ((pos "I0%" "L@%" "OC%" ) (subcat "2A" "2C" )))
( "smarted"  v ( s m aa1 t i d ) ((pos "Hd%" "Ic%" ) (subcat "2A" "2C" )))
( "smarten"  v ( s m aa1 t n ) ((pos "J0%" ) (subcat "2C" "6A" )))
( "smartened"  v ( s m aa1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "smartening"  v ( s m aa1 t n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "smartens"  v ( s m aa1 t n z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "smarter"  j ( s m aa1 t @ r ) ((pos "Or%" ) ))
( "smartest"  j ( s m aa1 t i s t ) ((pos "Os%" ) ))
( "smarting"  v ( s m aa1 t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "smartly"  a ( s m aa1 t l ii ) ((pos "Pu%" ) ))
( "smartness"  n ( s m aa1 t n @ s ) ((pos "L@%" ) ))
( "smarts"  v ( s m aa1 t s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "smash"  v ( s m a1 sh ) ((pos "J1%" "K7%" "Pu%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "smash-up"  n ( s m a1 sh - uh p ) ((pos "K6%" ) ))
( "smash-ups"  n ( s m a1 sh - uh p s ) ((pos "Kj$" ) ))
( "smashed"  v ( s m a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "smasher"  n ( s m a1 sh @ r ) ((pos "K6%" ) ))
( "smashers"  n ( s m a1 sh @ z ) ((pos "Kj%" ) ))
( "smashes"  v ( s m a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "smashing"  v ( s m a1 sh i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "smattering"  n ( s m a1 t @ r i ng ) ((pos "K6%" ) ))
( "smatterings"  n ( s m a1 t @ r i ng z ) ((pos "Kj$" ) ))
( "smear"  v ( s m i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "smear-word"  n ( s m i@1 - w @@ d ) ((pos "K6$" ) ))
( "smear-words"  n ( s m i@1 - w @@ d z ) ((pos "Kj$" ) ))
( "smeared"  v ( s m i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "smearing"  v ( s m i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "smears"  v ( s m i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "smell"  v ( s m e1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelled"  v ( s m e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smellier"  j ( s m e1 l i@ r ) ((pos "Or%" ) ))
( "smelliest"  j ( s m e1 l i i s t ) ((pos "Os%" ) ))
( "smelling"  v ( s m e1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelling-bottle"  n ( s m e1 l i ng - b o t l ) ((pos "K6$" ) ))
( "smelling-bottles"  n ( s m e1 l i ng - b o t l z ) ((pos "Kj$" ) ))
( "smelling-salts"  n ( s m e1 l i ng - s o l t s ) ((pos "Kj%" ) ))
( "smells"  v ( s m e1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelly"  j ( s m e1 l ii ) ((pos "OD%" ) ))
( "smelt"  v ( s m e1 l t ) ((pos "H0$" "Jc$" "Jd$" "K6$" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelted"  v ( s m e1 l t i d ) ((pos "Hc$" "Hd$" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelting"  v ( s m e1 l t i ng ) ((pos "Hb$" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smelts"  v ( s m e1 l t s ) ((pos "Ha$" "Kj$" ) (subcat "2A" "2C" "2D" "3A" "6A" "15B" )))
( "smilax"  n ( s m ai1 l a k s ) ((pos "L@$" ) ))
( "smile"  v ( s m ai1 l ) ((pos "J2%" "K6%" ) (subcat "2A" "2B" "3A" "4B" )))
( "smiled"  v ( s m ai1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "3A" "4B" )))
( "smiles"  v ( s m ai1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "3A" "4B" )))
( "smiling"  v ( s m ai1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "3A" "4B" )))
( "smilingly"  a ( s m ai1 l i ng l ii ) ((pos "Pu%" ) ))
( "smirch"  v ( s m @@1 ch ) ((pos "H1$" "K7$" ) (subcat )))
( "smirched"  v ( s m @@1 ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "smirches"  v ( s m @@1 ch i z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "smirching"  v ( s m @@1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "smirk"  v ( s m @@1 k ) ((pos "I0%" "K6%" ) (subcat )))
( "smirked"  v ( s m @@1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "smirking"  v ( s m @@1 k i ng ) ((pos "Ib%" ) (subcat )))
( "smirks"  v ( s m @@1 k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "smite"  v ( s m ai1 t ) ((pos "J5$" ) (subcat "2A" "2C" )))
( "smites"  v ( s m ai1 t s ) ((pos "Ja$" ) (subcat "2A" "2C" )))
( "smith"  n ( s m i1 th ) ((pos "K6%" ) ))
( "smithereens"  n ( s m i2 dh @ r ii1 n z ) ((pos "Kj%" ) ))
( "smithies"  n ( s m i1 dh i z ) ((pos "Kj%" ) ))
( "smiths"  n ( s m i1 th s ) ((pos "Kj%" ) ))
( "smithy"  n ( s m i1 dh ii ) ((pos "K8%" ) ))
( "smiting"  v ( s m ai1 t i ng ) ((pos "Jb$" ) (subcat "2A" "2C" )))
( "smitten"  v ( s m i1 t n ) ((pos "Jd%" ) (subcat "2A" "2C" )))
( "smock"  n ( s m o1 k ) ((pos "K6%" ) ))
( "smocking"  n ( s m o1 k i ng ) ((pos "L@%" ) ))
( "smocks"  n ( s m o1 k s ) ((pos "Kj%" ) ))
( "smog"  n ( s m o1 g ) ((pos "M6%" ) ))
( "smogs"  n ( s m o1 g z ) ((pos "Mj$" ) ))
( "smoke"  v ( s m ou1 k ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" "15B" )))
( "smoke-bomb"  n ( s m ou1 k - b o m ) ((pos "K6%" ) ))
( "smoke-bombs"  n ( s m ou1 k - b o m z ) ((pos "Kj%" ) ))
( "smoke-cured"  j ( s m ou1 k - k y uu @ d ) ((pos "OA%" ) ))
( "smoke-dried"  j ( s m ou1 k - d r ai d ) ((pos "OA%" ) ))
( "smoke-screen"  n ( s m ou1 k - s k r ii n ) ((pos "K6%" ) ))
( "smoke-screens"  n ( s m ou1 k - s k r ii n z ) ((pos "Kj%" ) ))
( "smoked"  v ( s m ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15B" )))
( "smokeless"  j ( s m ou1 k l @ s ) ((pos "OA%" ) ))
( "smoker"  n ( s m ou1 k @ r ) ((pos "K6%" ) ))
( "smokers"  n ( s m ou1 k @ z ) ((pos "Kj%" ) ))
( "smokes"  v ( s m ou1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15B" )))
( "smokestack"  n ( s m ou1 k s t a k ) ((pos "K6%" ) ))
( "smokestacks"  n ( s m ou1 k s t a k s ) ((pos "Kj%" ) ))
( "smokier"  j ( s m ou1 k i@ r ) ((pos "Or%" ) ))
( "smokiest"  j ( s m ou1 k i i s t ) ((pos "Os%" ) ))
( "smoking"  v ( s m ou1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "15B" )))
( "smoking-car"  n ( s m ou1 k i ng - k aa r ) ((pos "K6%" ) ))
( "smoking-carriage"  n ( s m ou1 k i ng - k a r i jh ) ((pos "K6%" ) ))
( "smoking-carriages"  n ( s m ou1 k i ng - k a r i jh i z ) ((pos "Kj%" ) ))
( "smoking-cars"  n ( s m ou1 k i ng - k aa z ) ((pos "Kj%" ) ))
( "smoking-compartment"  n ( s m ou1 k i ng - k @ m p aa2 t m @ n t ) ((pos "K6%" ) ))
( "smoking-compartments"  n ( s m ou1 k i ng - k @ m p aa2 t m @ n t s ) ((pos "Kj%" ) ))
( "smoking-mixture"  n ( s m ou1 k i ng - m i k s ch @ r ) ((pos "L@%" ) ))
( "smoking-room"  n ( s m ou1 k i ng - r uu m ) ((pos "K6%" ) ))
( "smoking-rooms"  n ( s m ou1 k i ng - r uu m z ) ((pos "Kj%" ) ))
( "smoky"  j ( s m ou1 k ii ) ((pos "OD%" ) ))
( "smooth"  v ( s m uu1 dh ) ((pos "J0%" "Ki%" "OC%" ) (subcat "2C" "6A" )))
( "smooth-bore"  j ( s m uu1 dh - b oo r ) ((pos "OA%" ) ))
( "smooth-faced"  j ( s m uu1 dh - f ei s t ) ((pos "OA%" ) ))
( "smooth-spoken"  j ( s m uu1 dh - s p ou1 k @ n ) ((pos "OA%" ) ))
( "smooth-tongued"  j ( s m uu1 dh - t uh ng d ) ((pos "OA%" ) ))
( "smoothed"  v ( s m uu1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "smoother"  j ( s m uu1 dh @ r ) ((pos "Or%" ) ))
( "smoothest"  j ( s m uu1 dh i s t ) ((pos "Os%" ) ))
( "smoothing"  v ( s m uu1 dh i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "smoothing-iron"  n ( s m uu1 dh i ng - ai @ n ) ((pos "K6%" ) ))
( "smoothing-irons"  n ( s m uu1 dh i ng - ai @ n z ) ((pos "Kj%" ) ))
( "smoothing-plane"  n ( s m uu1 dh i ng - p l ei n ) ((pos "K6$" ) ))
( "smoothing-planes"  n ( s m uu1 dh i ng - p l ei n z ) ((pos "Kj$" ) ))
( "smoothly"  a ( s m uu1 dh l ii ) ((pos "Pu%" ) ))
( "smoothness"  n ( s m uu1 dh n @ s ) ((pos "L@%" ) ))
( "smooths"  v ( s m uu1 dh z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "smorgasbord"  n ( s m oo1 g @ s b oo d ) ((pos "L@$" ) ))
( "smote"  v ( s m ou1 t ) ((pos "Jc$" ) (subcat "2A" "2C" )))
( "smother"  v ( s m uh1 dh @ r ) ((pos "H0%" "Ki%" ) (subcat "6A" "14" )))
( "smothered"  v ( s m uh1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "smothering"  v ( s m uh1 dh @ r i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "smothers"  v ( s m uh1 dh @ z ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "smoulder"  v ( s m ou1 l d @ r ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "smouldered"  v ( s m ou1 l d @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "smouldering"  v ( s m ou1 l d @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "smoulders"  v ( s m ou1 l d @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "smudge"  v ( s m uh1 jh ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "smudged"  v ( s m uh1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "smudges"  v ( s m uh1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "smudging"  v ( s m uh1 jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "smudgy"  j ( s m uh1 jh ii ) ((pos "OA%" ) ))
( "smug"  j ( s m uh1 g ) ((pos "OE%" ) ))
( "smugger"  j ( s m uh1 g @ r ) ((pos "Or%" ) ))
( "smuggest"  j ( s m uh1 g i s t ) ((pos "Os%" ) ))
( "smuggle"  v ( s m uh1 g l ) ((pos "H2%" ) (subcat "6A" "14" "15A" )))
( "smuggled"  v ( s m uh1 g l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "15A" )))
( "smuggler"  n ( s m uh1 g l @ r ) ((pos "K6%" ) ))
( "smugglers"  n ( s m uh1 g l @ z ) ((pos "Kj%" ) ))
( "smuggles"  v ( s m uh1 g l z ) ((pos "Ha%" ) (subcat "6A" "14" "15A" )))
( "smuggling"  v ( s m uh1 g l i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15A" )))
( "smugly"  a ( s m uh1 g l ii ) ((pos "Pu%" ) ))
( "smugness"  n ( s m uh1 g n @ s ) ((pos "L@%" ) ))
( "smut"  v ( s m uh1 t ) ((pos "H4$" "M6%" ) (subcat )))
( "smuts"  v ( s m uh1 t s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "smutted"  v ( s m uh1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "smuttier"  j ( s m uh1 t i@ r ) ((pos "Or%" ) ))
( "smuttiest"  j ( s m uh1 t i i s t ) ((pos "Os%" ) ))
( "smuttily"  a ( s m uh1 t i l ii ) ((pos "Pu%" ) ))
( "smuttiness"  n ( s m uh1 t i n @ s ) ((pos "L@%" ) ))
( "smutting"  v ( s m uh1 t i ng ) ((pos "Hb$" ) (subcat )))
( "smutty"  j ( s m uh1 t ii ) ((pos "OD%" ) ))
( "snack"  n ( s n a1 k ) ((pos "K6%" ) ))
( "snack-bar"  n ( s n a1 k - b aa r ) ((pos "K6%" ) ))
( "snack-bars"  n ( s n a1 k - b aa z ) ((pos "Kj%" ) ))
( "snack-counter"  n ( s n a1 k - k au n t @ r ) ((pos "K6%" ) ))
( "snack-counters"  n ( s n a1 k - k au n t @ z ) ((pos "Kj%" ) ))
( "snacks"  n ( s n a1 k s ) ((pos "Kj%" ) ))
( "snaffle"  v ( s n a1 f l ) ((pos "H2%" "K6$" ) (subcat )))
( "snaffle-bit"  n ( s n a1 f l - b i t ) ((pos "K6$" ) ))
( "snaffle-bits"  n ( s n a1 f l - b i t s ) ((pos "Kj$" ) ))
( "snaffled"  v ( s n a1 f l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "snaffles"  v ( s n a1 f l z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "snaffling"  v ( s n a1 f l i ng ) ((pos "Hb%" ) (subcat )))
( "snag"  v ( s n a1 g ) ((pos "H4$" "K6%" ) ))
( "snagged"  v ( s n a1 g d ) ((pos "Hc$" "Hd$" ) ))
( "snagging"  v ( s n a1 g i ng ) ((pos "Hb$" ) ))
( "snags"  v ( s n a1 g z ) ((pos "Ha$" "Kj%" ) ))
( "snail"  n ( s n ei1 l ) ((pos "K6%" ) ))
( "snails"  n ( s n ei1 l z ) ((pos "Kj%" ) ))
( "snake"  v ( s n ei1 k ) ((pos "I2%" "K6%" ) (subcat )))
( "snake-charmer"  n ( s n ei1 k - ch aa m @ r ) ((pos "K6%" ) ))
( "snake-charmers"  n ( s n ei1 k - ch aa m @ z ) ((pos "Kj%" ) ))
( "snaked"  v ( s n ei1 k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "snakes"  v ( s n ei1 k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "snakier"  j ( s n ei1 k i@ r ) ((pos "Or$" ) ))
( "snakiest"  j ( s n ei1 k i i s t ) ((pos "Os$" ) ))
( "snaking"  v ( s n ei1 k i ng ) ((pos "Ib%" ) (subcat )))
( "snaky"  j ( s n ei1 k ii ) ((pos "OD%" ) ))
( "snap"  v ( s n a1 p ) ((pos "J4%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "snap-fastener"  n ( s n a1 p - f aa s n @ r ) ((pos "K6%" ) ))
( "snap-fasteners"  n ( s n a1 p - f aa s n @ z ) ((pos "Kj%" ) ))
( "snapdragon"  n ( s n a1 p d r a g @ n ) ((pos "K6%" ) ))
( "snapdragons"  n ( s n a1 p d r a g @ n z ) ((pos "Kj%" ) ))
( "snapped"  v ( s n a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "snappier"  j ( s n a1 p i@ r ) ((pos "Or%" ) ))
( "snappiest"  j ( s n a1 p i i s t ) ((pos "Os%" ) ))
( "snapping"  v ( s n a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "snappish"  j ( s n a1 p i sh ) ((pos "OA$" ) ))
( "snappishly"  a ( s n a1 p i sh l ii ) ((pos "Pu$" ) ))
( "snappishness"  n ( s n a1 p i sh n @ s ) ((pos "L@%" ) ))
( "snappy"  j ( s n a1 p ii ) ((pos "OD%" ) ))
( "snaps"  v ( s n a1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" "15B" )))
( "snapshot"  n ( s n a1 p sh o t ) ((pos "K6%" ) ))
( "snapshots"  n ( s n a1 p sh o t s ) ((pos "Kj%" ) ))
( "snare"  v ( s n e@1 r ) ((pos "H2%" "K6%" ) (subcat )))
( "snare-drum"  n ( s n e@1 - d r uh m ) ((pos "K6%" ) ))
( "snare-drums"  n ( s n e@1 - d r uh m z ) ((pos "Kj%" ) ))
( "snared"  v ( s n e@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "snares"  v ( s n e@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "snaring"  v ( s n e@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "snarl"  v ( s n aa1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "snarl-up"  n ( s n aa1 l - uh p ) ((pos "K6%" ) ))
( "snarl-ups"  n ( s n aa1 l - uh p s ) ((pos "Kj%" ) ))
( "snarled"  v ( s n aa1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "snarling"  v ( s n aa1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "snarls"  v ( s n aa1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "snatch"  v ( s n a1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "6A" "15A" )))
( "snatched"  v ( s n a1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "snatcher"  n ( s n a1 ch @ r ) ((pos "K6%" ) ))
( "snatchers"  n ( s n a1 ch @ z ) ((pos "Kj%" ) ))
( "snatches"  v ( s n a1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "15A" )))
( "snatching"  v ( s n a1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "snazzy"  j ( s n a1 z ii ) ((pos "OA%" ) ))
( "sneak"  v ( s n ii1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" )))
( "sneak-thief"  n ( s n ii1 k - th ii f ) ((pos "Ki%" ) ))
( "sneak-thieves"  n ( s n ii1 k - th ii v z ) ((pos "Kj%" ) ))
( "sneaked"  v ( s n ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "sneakers"  n ( s n ii1 k @ z ) ((pos "Kj%" ) ))
( "sneakier"  j ( s n ii1 k i@ r ) ((pos "Or%" ) ))
( "sneakiest"  j ( s n ii1 k i i s t ) ((pos "Os%" ) ))
( "sneaking"  v ( s n ii1 k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" )))
( "sneakingly"  a ( s n ii1 k i ng l ii ) ((pos "Pu$" ) ))
( "sneaks"  v ( s n ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "sneaky"  j ( s n ii1 k ii ) ((pos "OD%" ) ))
( "sneer"  v ( s n i@1 r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "sneered"  v ( s n i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sneering"  v ( s n i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sneeringly"  a ( s n i@1 r i ng l ii ) ((pos "Pu%" ) ))
( "sneers"  v ( s n i@1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "sneeze"  v ( s n ii1 z ) ((pos "I2%" "K6%" ) (subcat )))
( "sneezed"  v ( s n ii1 z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sneezes"  v ( s n ii1 z i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "sneezing"  v ( s n ii1 z i ng ) ((pos "Ib%" ) (subcat )))
( "snick"  v ( s n i1 k ) ((pos "J0$" "K6$" ) ))
( "snicked"  v ( s n i1 k t ) ((pos "Jc$" "Jd$" ) ))
( "snicker"  v ( s n i1 k @ r ) ((pos "I0$" "K6$" ) ))
( "snickered"  v ( s n i1 k @ d ) ((pos "Ic$" "Id$" ) ))
( "snickering"  v ( s n i1 k @ r i ng ) ((pos "Ib$" ) ))
( "snickers"  v ( s n i1 k @ z ) ((pos "Ia$" "Kj$" ) ))
( "snicking"  v ( s n i1 k i ng ) ((pos "Jb$" ) ))
( "snicks"  v ( s n i1 k s ) ((pos "Ja$" "Kj$" ) ))
( "snide"  j ( s n ai1 d ) ((pos "OA%" ) ))
( "sniff"  v ( s n i1 f ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "sniffed"  v ( s n i1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "sniffier"  j ( s n i1 f i@ r ) ((pos "Or$" ) ))
( "sniffiest"  j ( s n i1 f i i s t ) ((pos "Os$" ) ))
( "sniffing"  v ( s n i1 f i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "sniffle"  v ( s n i1 f l ) ((pos "I2%" ) ))
( "sniffled"  v ( s n i1 f l d ) ((pos "Ic%" "Id%" ) ))
( "sniffles"  v ( s n i1 f l z ) ((pos "Ia%" ) ))
( "sniffling"  v ( s n i1 f l i ng ) ((pos "Ib%" ) ))
( "sniffs"  v ( s n i1 f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "sniffy"  j ( s n i1 f ii ) ((pos "OD%" ) ))
( "snifter"  n ( s n i1 f t @ r ) ((pos "K6%" ) ))
( "snifters"  n ( s n i1 f t @ z ) ((pos "Kj%" ) ))
( "snigger"  v ( s n i1 g @ r ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "sniggered"  v ( s n i1 g @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sniggering"  v ( s n i1 g @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sniggers"  v ( s n i1 g @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "snip"  v ( s n i1 p ) ((pos "J4%" "K6%" ) (subcat "3A" "6A" )))
( "snipe"  v ( s n ai1 p ) ((pos "J2%" "K9%" ) (subcat "2A" "3A" )))
( "sniped"  v ( s n ai1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "sniper"  n ( s n ai1 p @ r ) ((pos "K6%" ) ))
( "snipers"  n ( s n ai1 p @ z ) ((pos "Kj%" ) ))
( "snipes"  v ( s n ai1 p s ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "sniping"  v ( s n ai1 p i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "snipped"  v ( s n i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "snippet"  n ( s n i1 p i t ) ((pos "K6%" ) ))
( "snippets"  n ( s n i1 p i t s ) ((pos "Kj%" ) ))
( "snipping"  v ( s n i1 p i ng ) ((pos "Jb%" "K6%" ) (subcat "3A" "6A" )))
( "snippings"  n ( s n i1 p i ng z ) ((pos "Kj%" ) ))
( "snips"  v ( s n i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "snitch"  v ( s n i1 ch ) ((pos "J1$" ) (subcat "2A" "3A" )))
( "snitched"  v ( s n i1 ch t ) ((pos "Jc$" "Jd$" ) (subcat "2A" "3A" )))
( "snitches"  v ( s n i1 ch i z ) ((pos "Ja$" ) (subcat "2A" "3A" )))
( "snitching"  v ( s n i1 ch i ng ) ((pos "Jb$" ) (subcat "2A" "3A" )))
( "snivel"  v ( s n i1 v @ l ) ((pos "I4%" ) (subcat )))
( "snivelled"  v ( s n i1 v @ l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sniveller"  n ( s n i1 v l @ r ) ((pos "K6%" ) ))
( "snivellers"  n ( s n i1 v l @ z ) ((pos "Kj%" ) ))
( "snivelling"  v ( s n i1 v @ l i ng ) ((pos "Ib%" ) (subcat )))
( "snivels"  v ( s n i1 v @ l z ) ((pos "Ia%" ) (subcat )))
( "snob"  n ( s n o1 b ) ((pos "K6%" ) ))
( "snobbery"  n ( s n o1 b @ r ii ) ((pos "L@%" ) ))
( "snobbish"  j ( s n o1 b i sh ) ((pos "OA%" ) ))
( "snobbishly"  a ( s n o1 b i sh l ii ) ((pos "Pu%" ) ))
( "snobbishness"  n ( s n o1 b i sh n @ s ) ((pos "L@%" ) ))
( "snobs"  n ( s n o1 b z ) ((pos "Kj%" ) ))
( "snog"  v ( s n o1 g ) ((pos "I4%" ) (subcat )))
( "snogged"  v ( s n o1 g d ) ((pos "Ic%" ) (subcat )))
( "snogging"  v ( s n o1 g i ng ) ((pos "Ib%" "L@%" ) (subcat )))
( "snogs"  v ( s n o1 g z ) ((pos "Ia$" ) (subcat )))
( "snood"  n ( s n uu1 d ) ((pos "K6$" ) ))
( "snoods"  n ( s n uu1 d z ) ((pos "Kj$" ) ))
( "snook"  n ( s n uu1 k ) ((pos "K6%" ) ))
( "snooker"  n ( s n uu1 k @ r ) ((pos "M6%" ) ))
( "snookers"  n ( s n uu1 k @ z ) ((pos "Mj%" ) ))
( "snooks"  n ( s n uu1 k s ) ((pos "Kj$" ) ))
( "snoop"  v ( s n uu1 p ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "snooped"  v ( s n uu1 p t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "snooper"  n ( s n uu1 p @ r ) ((pos "K6%" ) ))
( "snoopers"  n ( s n uu1 p @ z ) ((pos "Kj%" ) ))
( "snooping"  v ( s n uu1 p i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "snoops"  v ( s n uu1 p s ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "snootier"  j ( s n uu1 t i@ r ) ((pos "Or%" ) ))
( "snootiest"  j ( s n uu1 t i i s t ) ((pos "Os%" ) ))
( "snootily"  a ( s n uu1 t i l ii ) ((pos "Pu%" ) ))
( "snooty"  j ( s n uu1 t ii ) ((pos "OD%" ) ))
( "snooze"  v ( s n uu1 z ) ((pos "I2%" "K6%" ) (subcat )))
( "snoozed"  v ( s n uu1 z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "snoozes"  v ( s n uu1 z i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "snoozing"  v ( s n uu1 z i ng ) ((pos "Ib%" ) (subcat )))
( "snore"  v ( s n oo1 r ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "snored"  v ( s n oo1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "snorer"  n ( s n oo1 r @ r ) ((pos "K6%" ) ))
( "snorers"  n ( s n oo1 r @ z ) ((pos "Kj%" ) ))
( "snores"  v ( s n oo1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "snoring"  v ( s n oo1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "snorkel"  n ( s n oo1 k l ) ((pos "K6%" ) ))
( "snorkels"  n ( s n oo1 k l z ) ((pos "Kj%" ) ))
( "snort"  v ( s n oo1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "snorted"  v ( s n oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "snorter"  n ( s n oo1 t @ r ) ((pos "K6$" ) ))
( "snorters"  n ( s n oo1 t @ z ) ((pos "Kj$" ) ))
( "snortier"  j ( s n oo1 t i@ r ) ((pos "Or$" ) ))
( "snortiest"  j ( s n oo1 t i i s t ) ((pos "Os$" ) ))
( "snorting"  v ( s n oo1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "snorts"  v ( s n oo1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "snorty"  j ( s n oo1 t ii ) ((pos "OD$" ) ))
( "snot"  n ( s n o1 t ) ((pos "L@%" ) ))
( "snot-nosed"  j ( s n o1 t - n ou z d ) ((pos "OA%" ) ))
( "snottier"  j ( s n o1 t i@ r ) ((pos "Or$" ) ))
( "snottiest"  j ( s n o1 t i i s t ) ((pos "Os$" ) ))
( "snotty"  j ( s n o1 t ii ) ((pos "OD%" ) ))
( "snout"  n ( s n au1 t ) ((pos "K6%" ) ))
( "snouts"  n ( s n au1 t s ) ((pos "Kj%" ) ))
( "snow"  v ( s n ou1 ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" )))
( "snow-clad"  j ( s n ou1 - k l a d ) ((pos "OA%" ) ))
( "snow-covered"  j ( s n ou1 - k uh v @ d ) ((pos "OA%" ) ))
( "snow-line"  n ( s n ou1 - l ai n ) ((pos "K6%" ) ))
( "snow-lines"  n ( s n ou1 - l ai n z ) ((pos "Kj%" ) ))
( "snow-white"  j ( s n ou1 - w ai t ) ((pos "OA%" ) ))
( "snowball"  v ( s n ou1 b aa l ) ((pos "J0%" "K6%" ) ))
( "snowballed"  v ( s n ou1 b aa l d ) ((pos "Jc%" "Jd%" ) ))
( "snowballing"  v ( s n ou1 b aa l i ng ) ((pos "Jb%" ) ))
( "snowballs"  v ( s n ou1 b aa l z ) ((pos "Ja%" "Kj%" ) ))
( "snowberries"  n ( s n ou1 b e r i z ) ((pos "Kj$" ) ))
( "snowberry"  n ( s n ou1 b e r ii ) ((pos "K8$" ) ))
( "snowblind"  j ( s n ou1 b l ai n d ) ((pos "OA%" ) ))
( "snowblindness"  n ( s n ou1 b l ai n - n @ s ) ((pos "L@%" ) ))
( "snowbound"  j ( s n ou1 b au n d ) ((pos "OA%" ) ))
( "snowcapped"  j ( s n ou1 k a p t ) ((pos "OA%" ) ))
( "snowdrift"  n ( s n ou1 d r i f t ) ((pos "K6%" ) ))
( "snowdrifts"  n ( s n ou1 d r i f t s ) ((pos "Kj%" ) ))
( "snowdrop"  n ( s n ou1 d r o p ) ((pos "K6%" ) ))
( "snowdrops"  n ( s n ou1 d r o p s ) ((pos "Kj%" ) ))
( "snowed"  v ( s n ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" )))
( "snowfall"  n ( s n ou1 f oo l ) ((pos "K6%" ) ))
( "snowfalls"  n ( s n ou1 f oo l z ) ((pos "Kj%" ) ))
( "snowfield"  n ( s n ou1 f ii l d ) ((pos "K6%" ) ))
( "snowfields"  n ( s n ou1 f ii l d z ) ((pos "Kj%" ) ))
( "snowflake"  n ( s n ou1 f l ei k ) ((pos "K6%" ) ))
( "snowflakes"  n ( s n ou1 f l ei k s ) ((pos "Kj%" ) ))
( "snowier"  j ( s n ou1 i@ r ) ((pos "Or$" ) ))
( "snowiest"  j ( s n ou1 i i s t ) ((pos "Os$" ) ))
( "snowing"  v ( s n ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" )))
( "snowman"  n ( s n ou1 m a n ) ((pos "Ki%" ) ))
( "snowmen"  n ( s n ou1 m e n ) ((pos "Kj%" ) ))
( "snowplough"  n ( s n ou1 p l au ) ((pos "K6%" ) ))
( "snowploughs"  n ( s n ou1 p l au z ) ((pos "Kj%" ) ))
( "snows"  v ( s n ou1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" )))
( "snowshoes"  n ( s n ou1 sh uu z ) ((pos "Kj%" ) ))
( "snowstorm"  n ( s n ou1 s t oo m ) ((pos "K6%" ) ))
( "snowstorms"  n ( s n ou1 s t oo m z ) ((pos "Kj%" ) ))
( "snowy"  j ( s n ou1 ii ) ((pos "OD%" ) ))
( "snub"  v ( s n uh1 b ) ((pos "H4%" "K6%" "OA%" ) (subcat )))
( "snub-nosed"  j ( s n uh1 b - n ou1 z d ) ((pos "OA%" ) ))
( "snubbed"  v ( s n uh1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "snubbing"  v ( s n uh1 b i ng ) ((pos "Hb%" ) (subcat )))
( "snubs"  v ( s n uh1 b z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "snuff"  v ( s n uh1 f ) ((pos "J0%" "M6%" ) (subcat "2C" "6A" )))
( "snuff-colour"  n ( s n uh1 f - k uh l @ r ) ((pos "L@$" "OA$" ) ))
( "snuff-coloured"  j ( s n uh1 f - k uh l @ d ) ((pos "OA$" ) ))
( "snuffbox"  n ( s n uh1 f b o k s ) ((pos "K7%" ) ))
( "snuffboxes"  n ( s n uh1 f b o k s i z ) ((pos "Kj%" ) ))
( "snuffed"  v ( s n uh1 f t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "snuffers"  n ( s n uh1 f @ z ) ((pos "Kj$" ) ))
( "snuffing"  v ( s n uh1 f i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "snuffle"  v ( s n uh1 f l ) ((pos "I2%" "K6%" ) (subcat )))
( "snuffled"  v ( s n uh1 f l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "snuffles"  v ( s n uh1 f l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "snuffling"  v ( s n uh1 f l i ng ) ((pos "Ib%" ) (subcat )))
( "snuffs"  v ( s n uh1 f s ) ((pos "Ja%" "Mj$" ) (subcat "2C" "6A" )))
( "snug"  n ( s n uh1 g ) ((pos "K6%" "OE%" ) ))
( "snugger"  j ( s n uh1 g @ r ) ((pos "Or%" ) ))
( "snuggeries"  n ( s n uh1 g @ r i z ) ((pos "Kj$" ) ))
( "snuggery"  n ( s n uh1 g @ r ii ) ((pos "K8$" ) ))
( "snuggest"  j ( s n uh1 g i s t ) ((pos "Os%" ) ))
( "snuggle"  v ( s n uh1 g l ) ((pos "J2%" ) (subcat "2C" )))
( "snuggled"  v ( s n uh1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "snuggles"  v ( s n uh1 g l z ) ((pos "Ja%" ) (subcat "2C" )))
( "snuggling"  v ( s n uh1 g l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "snugly"  a ( s n uh1 g l ii ) ((pos "Pu%" ) ))
( "snugness"  n ( s n uh1 g n @ s ) ((pos "L@$" ) ))
( "snugs"  n ( s n uh1 g z ) ((pos "Kj$" ) ))
( "so"  n ( s ou ) ((pos "Ki%" "Pu*" "V-*" ) ))
( "so-and-so"  n ( s ou1 - @ n - s ou ) ((pos "K6%" ) ))
( "so-and-sos"  n ( s ou1 - @ n - s ou z ) ((pos "Kj%" ) ))
( "so-called"  j ( s ou1 - k oo1 l d ) ((pos "OA%" ) ))
( "so-so"  j ( s ou2 - s ou1 ) ((pos "Op%" "Pu%" ) ))
( "soak"  v ( s ou1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "soaked"  v ( s ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "soaker"  n ( s ou1 k @ r ) ((pos "K6$" ) ))
( "soakers"  n ( s ou1 k @ z ) ((pos "Kj$" ) ))
( "soaking"  v ( s ou1 k i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "soaks"  v ( s ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "14" "15A" )))
( "soap"  v ( s ou1 p ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "soap-bubble"  n ( s ou1 p - b uh b l ) ((pos "K6%" ) ))
( "soap-bubbles"  n ( s ou1 p - b uh b l z ) ((pos "Kj%" ) ))
( "soap-opera"  n ( s ou1 p - o p r @ ) ((pos "M6%" ) ))
( "soap-operas"  n ( s ou1 p - o p r @ z ) ((pos "Mj%" ) ))
( "soapbox"  n ( s ou1 p b o k s ) ((pos "K7%" ) ))
( "soapboxes"  n ( s ou1 p b o k s i z ) ((pos "Kj%" ) ))
( "soaped"  v ( s ou1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "soapier"  j ( s ou1 p i@ r ) ((pos "Or$" ) ))
( "soapiest"  j ( s ou1 p i i s t ) ((pos "Os$" ) ))
( "soaping"  v ( s ou1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "soaps"  v ( s ou1 p s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "soapsuds"  n ( s ou1 p s uh d z ) ((pos "Kj%" ) ))
( "soapy"  j ( s ou1 p ii ) ((pos "OD%" ) ))
( "soar"  v ( s oo1 r ) ((pos "I0%" ) (subcat "2A" )))
( "soared"  v ( s oo1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "soaring"  v ( s oo1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "soars"  v ( s oo1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "sob"  v ( s o1 b ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "15A" )))
( "sob-stuff"  n ( s o1 b - s t uh f ) ((pos "L@%" ) ))
( "sobbed"  v ( s o1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "15A" )))
( "sobbing"  v ( s o1 b i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "15A" )))
( "sobbingly"  a ( s o1 b i ng l ii ) ((pos "Pu%" ) ))
( "sober"  v ( s ou1 b @ r ) ((pos "J0%" "OA%" ) (subcat "2C" "6A" )))
( "sober-sides"  n ( s ou1 b @ - s ai d z ) ((pos "Ki%" ) ))
( "sobered"  v ( s ou1 b @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "sobering"  v ( s ou1 b @ r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "soberly"  a ( s ou1 b @ l ii ) ((pos "Pu%" ) ))
( "sobers"  v ( s ou1 b @ z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "sobriety"  n ( s @1 b r ai1 @ t ii ) ((pos "L@%" ) ))
( "sobriquet"  n ( s ou1 b r i k ei ) ((pos "K6$" ) ))
( "sobriquets"  n ( s ou1 b r i k ei z ) ((pos "Kj$" ) ))
( "sobs"  v ( s o1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "15A" )))
( "soccer"  n ( s o1 k @ r ) ((pos "L@%" ) ))
( "sociability"  n ( s ou2 sh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "sociable"  j ( s ou1 sh @ b l ) ((pos "OA%" ) ))
( "sociably"  a ( s ou1 sh @ b l ii ) ((pos "Pu%" ) ))
( "social"  n ( s ou1 sh l ) ((pos "K6%" "OA%" ) ))
( "social-work"  n ( s ou1 sh l - w @@ k ) ((pos "L@%" ) ))
( "socialism"  n ( s ou1 sh @ l i z @ m ) ((pos "L@%" ) ))
( "socialist"  n ( s ou1 sh @ l i s t ) ((pos "K6%" "OA%" ) ))
( "socialists"  n ( s ou1 sh @ l i s t s ) ((pos "Kj%" ) ))
( "socialite"  n ( s ou1 sh @ l ai t ) ((pos "K6%" ) ))
( "socialites"  n ( s ou1 sh @ l ai t s ) ((pos "Kj%" ) ))
( "socialization"  n ( s ou2 sh @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "socializations"  n ( s ou2 sh @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "socialize"  v ( s ou1 sh @ l ai z ) ((pos "H2%" ) (subcat )))
( "socialized"  v ( s ou1 sh @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "socializes"  v ( s ou1 sh @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "socializing"  v ( s ou1 sh @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "socially"  a ( s ou1 sh @ l ii ) ((pos "Pu%" ) ))
( "socials"  n ( s ou1 sh l z ) ((pos "Kj%" ) ))
( "societies"  n ( s @1 s ai1 @ t i z ) ((pos "Mj%" ) ))
( "society"  n ( s @1 s ai1 @ t ii ) ((pos "M8%" ) ))
( "socio-"  j ( s ou2 s i@ u - ) ((pos "U-%" ) ))
( "sociological"  j ( s ou2 s i@ l o1 jh i k l ) ((pos "OA%" ) ))
( "sociologically"  a ( s ou2 s i@ l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "sociologist"  n ( s ou2 sh i o1 l @ jh i s t ) ((pos "K6%" ) ))
( "sociologists"  n ( s ou2 sh i o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "sociology"  n ( s ou2 s i o1 l @ jh ii ) ((pos "L@%" ) ))
( "sock"  v ( s o1 k ) ((pos "H0%" "K6%" "Pu%" ) (subcat "6A" )))
( "socked"  v ( s o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "socket"  n ( s o1 k i t ) ((pos "K6%" ) ))
( "sockets"  n ( s o1 k i t s ) ((pos "Kj%" ) ))
( "socking"  v ( s o1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "socks"  v ( s o1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "sod"  v ( s o1 d ) ((pos "I4$" "M6%" ) ))
( "soda"  n ( s ou1 d @ ) ((pos "L@%" ) ))
( "soda-biscuit"  n ( s ou1 d @ - b i s k i t ) ((pos "K6$" ) ))
( "soda-biscuits"  n ( s ou1 d @ - b i s k i t s ) ((pos "Kj$" ) ))
( "soda-cracker"  n ( s ou1 d @ - k r a k @ r ) ((pos "K6$" ) ))
( "soda-crackers"  n ( s ou1 d @ - k r a k @ z ) ((pos "Kj$" ) ))
( "soda-fountain"  n ( s ou1 d @ - f au n t i n ) ((pos "K6%" ) ))
( "soda-fountains"  n ( s ou1 d @ - f au n t i n z ) ((pos "Kj%" ) ))
( "soda-water"  n ( s ou1 d @ - w oo t @ r ) ((pos "M6%" ) ))
( "soda-waters"  n ( s ou1 d @ - w oo t @ z ) ((pos "Mj$" ) ))
( "sodded"  v ( s o1 d i d ) ((pos "Ic$" "Id$" ) ))
( "sodden"  j ( s o1 d n ) ((pos "OA%" ) ))
( "sodding"  v ( s o1 d i ng ) ((pos "Ib$" "Oq%" ) ))
( "sodium"  n ( s ou1 d i@ m ) ((pos "L@%" ) ))
( "sodomite"  n ( s o1 d @ m ai t ) ((pos "K6$" ) ))
( "sodomites"  n ( s o1 d @ m ai t s ) ((pos "Kj$" ) ))
( "sodomy"  n ( s o1 d @ m ii ) ((pos "L@%" ) ))
( "sods"  v ( s o1 d z ) ((pos "Ia$" "Mj%" ) ))
( "sofa"  n ( s ou1 f @ ) ((pos "K6%" ) ))
( "sofas"  n ( s ou1 f @ z ) ((pos "Kj%" ) ))
( "soft"  j ( s o1 f t ) ((pos "OC%" ) ))
( "soft-boiled"  j ( s o1 f t - b oi l d ) ((pos "OA%" ) ))
( "soft-footed"  j ( s o1 f t - f u1 t i d ) ((pos "OA%" ) ))
( "soft-headed"  j ( s o1 f t - h e1 d i d ) ((pos "OA%" ) ))
( "soft-hearted"  j ( s o1 f t - h aa1 t i d ) ((pos "OA%" ) ))
( "soft-pedal"  v ( s o1 f t - p e1 d @ l ) ((pos "J4%" ) ))
( "soft-pedalled"  v ( s o1 f t - p e1 d @ l d ) ((pos "Jc%" "Jd%" ) ))
( "soft-pedalling"  v ( s o1 f t - p e1 d @ l i ng ) ((pos "Jb%" ) ))
( "soft-pedals"  v ( s o1 f t - p e1 d @ l z ) ((pos "Ja%" ) ))
( "soft-soap"  v ( s o1 f t - s ou1 p ) ((pos "H0%" ) ))
( "soft-soaped"  v ( s o1 f t - s ou1 p t ) ((pos "Hc%" "Hd%" ) ))
( "soft-soaping"  v ( s o1 f t - s ou1 p i ng ) ((pos "Hb%" ) ))
( "soft-soaps"  v ( s o1 f t - s ou1 p s ) ((pos "Ha%" ) ))
( "soft-solder"  v ( s o1 f t - s ou1 l d @ r ) ((pos "H0$" "L@$" ) ))
( "soft-soldered"  v ( s o1 f t - s ou1 l d @ d ) ((pos "Hc$" "Hd$" ) ))
( "soft-soldering"  v ( s o1 f t - s ou1 l d @ r i ng ) ((pos "Hb$" ) ))
( "soft-solders"  v ( s o1 f t - s ou1 l d @ z ) ((pos "Ha$" ) ))
( "soft-spoken"  j ( s o1 f t - s p ou1 k @ n ) ((pos "OA%" ) ))
( "soft-witted"  j ( s o1 f t - w i1 t i d ) ((pos "OA$" ) ))
( "soften"  v ( s o1 f n ) ((pos "J0%" ) (subcat "2A" )))
( "softened"  v ( s o1 f n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "softener"  n ( s o1 f n @ r ) ((pos "K6%" ) ))
( "softeners"  n ( s o1 f n @ z ) ((pos "Kj%" ) ))
( "softening"  v ( s o1 f n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "softens"  v ( s o1 f n z ) ((pos "Ja%" ) (subcat "2A" )))
( "softer"  j ( s o1 f t @ r ) ((pos "Or%" ) ))
( "softest"  j ( s o1 f t i s t ) ((pos "Os%" ) ))
( "softie"  n ( s o1 f t ii ) ((pos "K6%" ) ))
( "softies"  n ( s o1 f t i z ) ((pos "Kj%" ) ))
( "softish"  j ( s o1 f t i sh ) ((pos "OA%" ) ))
( "softland"  v ( s o1 f t l a1 n d ) ((pos "I0$" ) ))
( "softlanded"  v ( s o1 f t l a1 n d i d ) ((pos "Ic$" "Id$" ) ))
( "softlanding"  v ( s o1 f t l a1 n d i ng ) ((pos "Ib$" ) ))
( "softlands"  v ( s o1 f t l a1 n d z ) ((pos "Ia$" ) ))
( "softly"  a ( s o1 f t l ii ) ((pos "Pu%" ) ))
( "softness"  n ( s o1 f t n @ s ) ((pos "L@%" ) ))
( "software"  n ( s o1 f t w e@ r ) ((pos "L@%" ) ))
( "softwood"  n ( s o1 f t w u d ) ((pos "M6%" ) ))
( "softwoods"  n ( s o1 f t w u d z ) ((pos "Mj%" ) ))
( "softy"  n ( s o1 f t ii ) ((pos "K8%" ) ))
( "soggier"  j ( s o1 g i@ r ) ((pos "Or%" ) ))
( "soggiest"  j ( s o1 g i i s t ) ((pos "Os%" ) ))
( "sogginess"  n ( s o1 g i n @ s ) ((pos "L@%" ) ))
( "soggy"  j ( s o1 g ii ) ((pos "OD%" ) ))
( "soh"  n ( s ou1 ) ((pos "Ki$" ) ))
( "soign_e"  j ( s w aa1 n y ei ) ((pos "OA$" ) ))
( "soignee"  j ( s w aa1 n y ei ) ((pos "OA$" ) ))
( "soil"  v ( s oi1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" )))
( "soil-pipe"  n ( s oi1 l - p ai p ) ((pos "K6%" ) ))
( "soil-pipes"  n ( s oi1 l - p ai p s ) ((pos "Kj%" ) ))
( "soiled"  v ( s oi1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "soiling"  v ( s oi1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "soils"  v ( s oi1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" )))
( "soir_ee"  n ( s w aa1 r ei ) ((pos "K6%" ) ))
( "soir_ees"  n ( s w aa1 r ei z ) ((pos "Kj%" ) ))
( "sojourn"  v ( s o1 jh @ n ) ((pos "I0$" "K6%" ) (subcat )))
( "sojourned"  v ( s o1 jh @ n d ) ((pos "Ic$" "Id$" ) (subcat )))
( "sojourner"  n ( s o1 jh @ n @ r ) ((pos "K6$" ) ))
( "sojourners"  n ( s o1 jh @ n @ z ) ((pos "Kj$" ) ))
( "sojourning"  v ( s o1 jh @ n i ng ) ((pos "Ib$" ) (subcat )))
( "sojourns"  v ( s o1 jh @ n z ) ((pos "Ia$" "Kj$" ) (subcat )))
( "sol-fa"  n ( s o2 l - f aa1 ) ((pos "Ki$" ) ))
( "solace"  v ( s o1 l i s ) ((pos "H2$" "M6%" ) (subcat "6A" )))
( "solaced"  v ( s o1 l i s t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "solaces"  v ( s o1 l i s i z ) ((pos "Ha$" "Mj$" ) (subcat "6A" )))
( "solacing"  v ( s o1 l i s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "solar"  j ( s ou1 l @ r ) ((pos "OA%" ) ))
( "solar plexus"  n ( s ou2 l @ - p l e1 k s @ s ) ((pos "Ki%" ) ))
( "solaria"  n ( s ou1 l e@1 r i@ ) ((pos "Kj$" ) ))
( "solarium"  n ( s ou1 l e@1 r i@ m ) ((pos "Ki$" ) ))
( "sold"  v ( s ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12A" "13A" )))
( "solder"  v ( s o1 l d @ r ) ((pos "H0%" "L@%" ) (subcat "6A" "15A" )))
( "soldered"  v ( s o1 l d @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "soldering"  v ( s o1 l d @ r i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "soldering-iron"  n ( s o1 l d @ r i ng - ai @ n ) ((pos "K6%" ) ))
( "soldering-irons"  n ( s o1 l d @ r i ng - ai @ n z ) ((pos "Kj%" ) ))
( "solders"  v ( s o1 l d @ z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "soldier"  v ( s ou1 l jh @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "soldiered"  v ( s ou1 l jh @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "soldiering"  v ( s ou1 l jh @ r i ng ) ((pos "Ib%" ) (subcat )))
( "soldiers"  v ( s ou1 l jh @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "soldiery"  n ( s ou1 l jh @ r ii ) ((pos "Ki%" ) ))
( "sole"  v ( s ou1 l ) ((pos "H2%" "K6%" "OA%" ) (subcat )))
( "solecism"  n ( s o1 l i s i z @ m ) ((pos "K6$" ) ))
( "solecisms"  n ( s o1 l i s i z @ m z ) ((pos "Kj$" ) ))
( "soled"  v ( s ou1 l d ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat )))
( "solely"  a ( s ou1 l ii ) ((pos "Pu%" ) ))
( "solemn"  j ( s o1 l @ m ) ((pos "OA%" ) ))
( "solemnities"  n ( s @1 l e1 m n i t i z ) ((pos "Mj%" ) ))
( "solemnity"  n ( s @1 l e1 m n i t ii ) ((pos "M8%" ) ))
( "solemnization"  n ( s o2 l @ m n ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "solemnize"  v ( s o1 l @ m n ai z ) ((pos "H2$" ) (subcat )))
( "solemnized"  v ( s o1 l @ m n ai z d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "solemnizes"  v ( s o1 l @ m n ai z i z ) ((pos "Ha$" ) (subcat )))
( "solemnizing"  v ( s o1 l @ m n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "solemnly"  a ( s o1 l @ m l ii ) ((pos "Pu%" ) ))
( "solemnness"  n ( s o1 l @ m n @ s ) ((pos "L@%" ) ))
( "soles"  v ( s ou1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "solicit"  v ( s @1 l i1 s i t ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "solicitation"  n ( s @1 l i2 s i t ei1 sh @ n ) ((pos "M6%" ) ))
( "solicitations"  n ( s @1 l i2 s i t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "solicited"  v ( s @1 l i1 s i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "soliciting"  v ( s @1 l i1 s i t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "solicitor"  n ( s @1 l i1 s i t @ r ) ((pos "K6%" ) ))
( "solicitors"  n ( s @1 l i1 s i t @ z ) ((pos "Kj%" ) ))
( "solicitous"  j ( s @1 l i1 s i t @ s ) ((pos "OA%" ) ))
( "solicitously"  a ( s @1 l i1 s i t @ s l ii ) ((pos "Pu%" ) ))
( "solicits"  v ( s @1 l i1 s i t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "solicitude"  n ( s @1 l i1 s i t y uu d ) ((pos "L@%" ) ))
( "solid"  n ( s o1 l i d ) ((pos "K6%" "OA%" ) ))
( "solid-state"  j ( s o2 l i d - s t ei1 t ) ((pos "OA%" ) ))
( "solidarity"  n ( s o2 l i d a1 r i t ii ) ((pos "L@%" ) ))
( "solidification"  n ( s @1 l i2 d i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "solidifications"  n ( s @1 l i2 d i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "solidified"  v ( s @1 l i1 d i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "solidifies"  v ( s @1 l i1 d i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "solidify"  v ( s @1 l i1 d i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "solidifying"  v ( s @1 l i1 d i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "solidity"  n ( s @1 l i1 d i t ii ) ((pos "L@%" ) ))
( "solidly"  a ( s o1 l i d l ii ) ((pos "Pu%" ) ))
( "solidness"  n ( s o1 l i d n @ s ) ((pos "L@%" ) ))
( "solids"  n ( s o1 l i d z ) ((pos "Kj%" ) ))
( "soliloquies"  n ( s @1 l i1 l @ k w i z ) ((pos "Mj%" ) ))
( "soliloquize"  v ( s @1 l i1 l @ k w ai z ) ((pos "I2$" ) (subcat )))
( "soliloquized"  v ( s @1 l i1 l @ k w ai z d ) ((pos "Ic$" "Id$" ) (subcat )))
( "soliloquizes"  v ( s @1 l i1 l @ k w ai z i z ) ((pos "Ia$" ) (subcat )))
( "soliloquizing"  v ( s @1 l i1 l @ k w ai z i ng ) ((pos "Ib$" ) (subcat )))
( "soliloquy"  n ( s @1 l i1 l @ k w ii ) ((pos "M8%" ) ))
( "soling"  v ( s ou1 l i ng ) ((pos "Hb%" ) (subcat )))
( "solipsism"  n ( s o1 l i p s i z @ m ) ((pos "L@$" ) ))
( "solitaire"  n ( s o2 l i t e@1 r ) ((pos "M6%" ) ))
( "solitaires"  n ( s o2 l i t e@1 z ) ((pos "Mj$" ) ))
( "solitarily"  a ( s o1 l i t r @ l ii ) ((pos "Pu%" ) ))
( "solitary"  j ( s o1 l i t r ii ) ((pos "OA%" ) ))
( "solitude"  n ( s o1 l i t y uu d ) ((pos "M6%" ) ))
( "solitudes"  n ( s o1 l i t y uu d z ) ((pos "Mj$" ) ))
( "solo"  n ( s ou1 l ou ) ((pos "M6%" ) ))
( "soloist"  n ( s ou1 l ou i s t ) ((pos "K6%" ) ))
( "soloists"  n ( s ou1 l ou i s t s ) ((pos "Kj%" ) ))
( "solos"  n ( s ou1 l ou z ) ((pos "Mj%" ) ))
( "solstice"  n ( s o1 l s t i s ) ((pos "K6%" ) ))
( "solstices"  n ( s o1 l s t i s i z ) ((pos "Kj$" ) ))
( "solubility"  n ( s o2 l y u b i1 l i t ii ) ((pos "L@%" ) ))
( "soluble"  j ( s o1 l y u b l ) ((pos "OA%" ) ))
( "solution"  n ( s @1 l uu1 sh @ n ) ((pos "M6%" ) ))
( "solutions"  n ( s @1 l uu1 sh @ n z ) ((pos "Mj%" ) ))
( "solvable"  j ( s o1 l v @ b l ) ((pos "OA%" ) ))
( "solve"  v ( s o1 l v ) ((pos "H2%" ) (subcat )))
( "solved"  v ( s o1 l v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "solvency"  n ( s o1 l v @ n s ii ) ((pos "L@%" ) ))
( "solvent"  n ( s o1 l v @ n t ) ((pos "K6%" "OA%" ) ))
( "solvents"  n ( s o1 l v @ n t s ) ((pos "Kj%" ) ))
( "solves"  v ( s o1 l v z ) ((pos "Ha%" ) (subcat )))
( "solving"  v ( s o1 l v i ng ) ((pos "Hb%" ) (subcat )))
( "somatic"  j ( s ou1 m a1 t i k ) ((pos "OA$" ) ))
( "sombre"  j ( s o1 m b @ r ) ((pos "OA%" ) ))
( "sombrely"  a ( s o1 m b @ l ii ) ((pos "Pu%" ) ))
( "sombreness"  n ( s o1 m b @ n @ s ) ((pos "L@%" ) ))
( "sombrero"  n ( s o1 m b r e@1 r ou ) ((pos "K6%" ) ))
( "sombreros"  n ( s o1 m b r e@1 r ou z ) ((pos "Kj%" ) ))
( "some"  j ( s uh m ) ((pos "OA*" "Pu*" "Qx*" ) ))
( "somebody"  prp ( s uh1 m b @ d ii ) ((pos "Qx%" ) ))
( "someday"  a ( s uh1 m d ei ) ((pos "Pu%" ) ))
( "somehow"  a ( s uh1 m h au ) ((pos "Pu%" ) ))
( "someone"  n ( s uh1 m w uh n ) ((pos "Ki%" "Qx%" ) ))
( "someplace"  a ( s uh1 m p l ei s ) ((pos "Pu$" ) ))
( "somersault"  v ( s uh1 m @ s oo l t ) ((pos "I0%" "K6%" ) (subcat )))
( "somersaulted"  v ( s uh1 m @ s oo l t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "somersaulting"  v ( s uh1 m @ s oo l t i ng ) ((pos "Ib%" ) (subcat )))
( "somersaults"  v ( s uh1 m @ s oo l t s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "something"  a ( s uh1 m th i ng ) ((pos "Pu*" "Qx*" ) ))
( "sometime"  a ( s uh1 m t ai m ) ((pos "Pu%" ) ))
( "sometimes"  a ( s uh1 m t ai m z ) ((pos "Pu*" ) ))
( "someway"  a ( s uh1 m w ei ) ((pos "Pu$" ) ))
( "somewhat"  a ( s uh1 m w o t ) ((pos "Pu%" ) ))
( "somewhere"  a ( s uh1 m w e@ r ) ((pos "Pu%" ) ))
( "somnambulism"  n ( s o1 m n a1 m b y u l i z @ m ) ((pos "L@%" ) ))
( "somnambulist"  n ( s o1 m n a1 m b y u l i s t ) ((pos "K6%" ) ))
( "somnambulists"  n ( s o1 m n a1 m b y u l i s t s ) ((pos "Kj%" ) ))
( "somnolence"  n ( s o1 m n @ l @ n s ) ((pos "L@%" ) ))
( "somnolent"  j ( s o1 m n @ l @ n t ) ((pos "OA%" ) ))
( "somnolently"  a ( s o1 m n @ l @ n t l ii ) ((pos "Pu%" ) ))
( "son"  n ( s uh1 n ) ((pos "K6%" ) ))
( "son-in-law"  n ( s uh1 n - i n - l oo ) ((pos "Ki%" ) ))
( "sonar"  n ( s ou1 n aa r ) ((pos "K6$" ) ))
( "sonars"  n ( s ou1 n aa z ) ((pos "Kj$" ) ))
( "sonata"  n ( s @1 n aa1 t @ ) ((pos "K6%" ) ))
( "sonatas"  n ( s @1 n aa1 t @ z ) ((pos "Kj%" ) ))
( "song"  n ( s o1 ng ) ((pos "M6%" ) ))
( "songbird"  n ( s o1 ng b @@ d ) ((pos "K6%" ) ))
( "songbirds"  n ( s o1 ng b @@ d z ) ((pos "Kj%" ) ))
( "songbook"  n ( s o1 ng b u k ) ((pos "K6%" ) ))
( "songbooks"  n ( s o1 ng b u k s ) ((pos "Kj%" ) ))
( "songs"  n ( s o1 ng z ) ((pos "Mj%" ) ))
( "songster"  n ( s o1 ng s t @ r ) ((pos "K6%" ) ))
( "songsters"  n ( s o1 ng s t @ z ) ((pos "Kj%" ) ))
( "songstress"  n ( s o1 ng s t r i s ) ((pos "K7$" ) ))
( "songstresses"  n ( s o1 ng s t r i s i z ) ((pos "Kj$" ) ))
( "sonic"  j ( s o1 n i k ) ((pos "OA%" ) ))
( "sonnet"  n ( s o1 n i t ) ((pos "K6%" ) ))
( "sonneteer"  n ( s o2 n i t i@1 r ) ((pos "K6$" ) ))
( "sonneteers"  n ( s o2 n i t i@1 z ) ((pos "Kj$" ) ))
( "sonnets"  n ( s o1 n i t s ) ((pos "Kj%" ) ))
( "sonnies"  n ( s uh1 n i z ) ((pos "Kj$" ) ))
( "sonny"  n ( s uh1 n ii ) ((pos "K8%" ) ))
( "sonorities"  n ( s @1 n o1 r i t i z ) ((pos "Mj$" ) ))
( "sonority"  n ( s @1 n o1 r i t ii ) ((pos "M8$" ) ))
( "sonorous"  j ( s @1 n oo1 r @ s ) ((pos "OA%" ) ))
( "sonorously"  a ( s @1 n oo1 r @ s l ii ) ((pos "Pu%" ) ))
( "sons"  n ( s uh1 n z ) ((pos "Kj%" ) ))
( "sons-in-law"  n ( s uh1 n z - i n - l oo ) ((pos "Kj%" ) ))
( "sonsy"  j ( s o1 n s ii ) ((pos "OA$" ) ))
( "soon"  a ( s uu1 n ) ((pos "Pu*" ) ))
( "sooner"  a ( s uu1 n @ r ) ((pos "Pu%" ) ))
( "soonest"  a ( s uu1 n i s t ) ((pos "Pu%" ) ))
( "soot"  v ( s u1 t ) ((pos "H0$" "L@%" ) ))
( "sooted"  v ( s u1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "sooth"  n ( s uu1 th ) ((pos "K6$" ) ))
( "soothe"  v ( s uu1 dh ) ((pos "H2%" ) (subcat )))
( "soothed"  v ( s uu1 dh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "soothes"  v ( s uu1 dh z ) ((pos "Ha%" ) (subcat )))
( "soothing"  v ( s uu1 dh i ng ) ((pos "Hb%" ) (subcat )))
( "soothingly"  a ( s uu1 dh i ng l ii ) ((pos "Pu%" ) ))
( "sooths"  n ( s uu1 th s ) ((pos "Kj$" ) ))
( "soothsayer"  n ( s uu1 th s ei @ r ) ((pos "K6%" ) ))
( "soothsayers"  n ( s uu1 th s ei @ z ) ((pos "Kj%" ) ))
( "sootier"  j ( s u1 t i@ r ) ((pos "Or%" ) ))
( "sootiest"  j ( s u1 t i i s t ) ((pos "Os%" ) ))
( "sooting"  v ( s u1 t i ng ) ((pos "Hb$" ) ))
( "soots"  v ( s u1 t s ) ((pos "Ha$" ) ))
( "sooty"  j ( s u1 t ii ) ((pos "OD%" ) ))
( "sop"  v ( s o1 p ) ((pos "H4$" "K6%" ) (subcat "6A" )))
( "sophism"  n ( s o1 f i z @ m ) ((pos "M6$" ) ))
( "sophisms"  n ( s o1 f i z @ m z ) ((pos "Mj$" ) ))
( "sophist"  n ( s o1 f i s t ) ((pos "K6$" ) ))
( "sophisticated"  j ( s @1 f i1 s t i k ei t i d ) ((pos "OA%" ) ))
( "sophistication"  n ( s @1 f i2 s t i k ei1 sh @ n ) ((pos "L@%" ) ))
( "sophistries"  n ( s o1 f i s t r i z ) ((pos "Mj$" ) ))
( "sophistry"  n ( s o1 f i s t r ii ) ((pos "M8%" ) ))
( "sophists"  n ( s o1 f i s t s ) ((pos "Kj$" ) ))
( "sophomore"  n ( s o1 f @ m oo r ) ((pos "K6$" ) ))
( "sophomores"  n ( s o1 f @ m oo z ) ((pos "Kj$" ) ))
( "soporific"  n ( s o2 p @ r i1 f i k ) ((pos "K6$" "OA%" ) ))
( "soporifics"  n ( s o2 p @ r i1 f i k s ) ((pos "Kj$" ) ))
( "sopped"  v ( s o1 p t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "soppier"  j ( s o1 p i@ r ) ((pos "Or%" ) ))
( "soppiest"  j ( s o1 p i i s t ) ((pos "Os%" ) ))
( "sopping"  v ( s o1 p i ng ) ((pos "Hb$" "OA%" "Pu%" ) (subcat "6A" )))
( "soppy"  j ( s o1 p ii ) ((pos "OD%" ) ))
( "soprano"  n ( s @1 p r aa1 n ou ) ((pos "K6%" ) ))
( "sopranos"  n ( s @1 p r aa1 n ou z ) ((pos "Kj%" ) ))
( "sops"  v ( s o1 p s ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "sorbet"  n ( s oo1 b @ t ) ((pos "K6%" ) ))
( "sorbets"  n ( s oo1 b @ t s ) ((pos "Kj%" ) ))
( "sorcerer"  n ( s oo1 s @ r @ r ) ((pos "K6%" ) ))
( "sorcerers"  n ( s oo1 s @ r @ z ) ((pos "Kj%" ) ))
( "sorceress"  n ( s oo1 s @ r i s ) ((pos "K7%" ) ))
( "sorceresses"  n ( s oo1 s @ r i s i z ) ((pos "Kj$" ) ))
( "sorceries"  n ( s oo1 s @ r i z ) ((pos "Mj$" ) ))
( "sorcery"  n ( s oo1 s @ r ii ) ((pos "M8%" ) ))
( "sordid"  j ( s oo1 d ii d ) ((pos "OA%" ) ))
( "sordidly"  a ( s oo1 d ii d l ii ) ((pos "Pu%" ) ))
( "sordidness"  n ( s oo1 d ii d n @ s ) ((pos "L@%" ) ))
( "sore"  n ( s oo1 r ) ((pos "K6%" "OA%" ) ))
( "sorely"  a ( s oo1 l ii ) ((pos "Pu%" ) ))
( "soreness"  n ( s oo1 n @ s ) ((pos "L@%" ) ))
( "sores"  n ( s oo1 z ) ((pos "Kj%" ) ))
( "sorghum"  n ( s oo1 g @ m ) ((pos "L@$" ) ))
( "sororities"  n ( s @1 r o1 r i t i z ) ((pos "Kj$" ) ))
( "sorority"  n ( s @1 r o1 r i t ii ) ((pos "K8$" ) ))
( "sorrel"  n ( s o1 r @ l ) ((pos "M6$" "OA$" ) ))
( "sorrels"  n ( s o1 r @ l z ) ((pos "Mj$" ) ))
( "sorrier"  j ( s o1 r i@ r ) ((pos "Or%" ) ))
( "sorriest"  j ( s o1 r i i s t ) ((pos "Os%" ) ))
( "sorrow"  v ( s o1 r ou ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "sorrowed"  v ( s o1 r ou d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sorrowful"  j ( s o1 r ou f @ l ) ((pos "OA%" ) ))
( "sorrowfully"  a ( s o1 r ou f @ l ii ) ((pos "Pu%" ) ))
( "sorrowing"  v ( s o1 r ou i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sorrows"  v ( s o1 r ou z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "sorry"  j ( s o1 r ii ) ((pos "OD%" ) ))
( "sort"  v ( s oo1 t ) ((pos "J0%" "K6%" ) (subcat "6A" )))
( "sorted"  v ( s oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "sorter"  n ( s oo1 t @ r ) ((pos "K6%" ) ))
( "sorters"  n ( s oo1 t @ z ) ((pos "Kj%" ) ))
( "sortie"  n ( s oo1 t ii ) ((pos "K6%" ) ))
( "sorties"  n ( s oo1 t ii z ) ((pos "Kj%" ) ))
( "sorting"  v ( s oo1 t i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "sorts"  v ( s oo1 t s ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "sot"  n ( s o1 t ) ((pos "K6%" ) ))
( "sots"  n ( s o1 t s ) ((pos "Kj%" ) ))
( "sottish"  j ( s o1 t i sh ) ((pos "OA$" ) ))
( "sottishly"  a ( s o1 t i sh l ii ) ((pos "Pu$" ) ))
( "sottishness"  n ( s o1 t i sh n @ s ) ((pos "L@$" ) ))
( "sotto voce"  a ( s o2 t ou - v ou1 ch ii ) ((pos "Pu$" ) ))
( "sou"  n ( s uu1 ) ((pos "K6$" ) ))
( "sou'-east"  n ( s au2 - ii1 s t ) ((pos "L@$" "Pu$" ) ))
( "sou'-sou'-east"  n ( s au2 - s au - ii1 s t ) ((pos "L@$" "Pu$" ) ))
( "sou'-sou'-west"  n ( s au2 - s au - w e1 s t ) ((pos "L@$" "Pu$" ) ))
( "sou'-west"  n ( s au2 - w e1 s t ) ((pos "L@$" "Pu$" ) ))
( "sou'-wester"  n ( s au2 - w e1 s t @ r ) ((pos "K6%" ) ))
( "sou'-westers"  n ( s au2 - w e1 s t @ z ) ((pos "Kj%" ) ))
( "soubrette"  n ( s uu1 b r e1 t ) ((pos "K6$" ) ))
( "soubrettes"  n ( s uu1 b r e1 t s ) ((pos "Kj$" ) ))
( "soubriquet"  n ( s uu1 b r i k ei ) ((pos "K6$" ) ))
( "soubriquets"  n ( s uu1 b r i k ei z ) ((pos "Kj$" ) ))
( "souffl_e"  n ( s uu1 f l ei ) ((pos "K6%" ) ))
( "souffl_es"  n ( s uu1 f l ei z ) ((pos "Kj%" ) ))
( "sough"  v ( s uh1 f ) ((pos "I0$" ) (subcat )))
( "soughed"  v ( s uh1 f t ) ((pos "Ic$" "Id$" ) (subcat )))
( "soughing"  v ( s uh1 f i ng ) ((pos "Ib$" ) (subcat )))
( "soughs"  v ( s uh1 f s ) ((pos "Ia$" ) (subcat )))
( "sought"  v ( s oo1 t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "3A" "6A" "7A" )))
( "soul"  n ( s ou1 l ) ((pos "K6%" ) ))
( "soul-destroying"  j ( s ou1 l - d i s t r oi i ng ) ((pos "OA%" ) ))
( "soul-stirring"  j ( s ou1 l - s t @@ r i ng ) ((pos "OA%" ) ))
( "soulful"  j ( s ou1 l f @ l ) ((pos "OA%" ) ))
( "soulfully"  a ( s ou1 l f @ l ii ) ((pos "Pu%" ) ))
( "soulless"  j ( s ou1 l - l @ s ) ((pos "OA%" ) ))
( "soullessly"  a ( s ou1 l - l @ s l ii ) ((pos "Pu%" ) ))
( "souls"  n ( s ou1 l z ) ((pos "Kj%" ) ))
( "sound"  v ( s au1 n d ) ((pos "J0%" "M6%" "OA%" "Pu%" ) (subcat "2A" "2C" "2D" "6A" "15A" )))
( "sound-film"  n ( s au1 n d - f i l m ) ((pos "K6$" ) ))
( "sound-films"  n ( s au1 n d - f i l m z ) ((pos "Kj$" ) ))
( "sound-recording"  n ( s au1 n d - r i k oo2 d i ng ) ((pos "M6%" ) ))
( "sound-recordings"  n ( s au1 n d - r i k oo2 d i ng z ) ((pos "Mj%" ) ))
( "sound-wave"  n ( s au1 n d - w ei v ) ((pos "K6%" ) ))
( "sound-waves"  n ( s au1 n d - w ei v z ) ((pos "Kj%" ) ))
( "soundbox"  n ( s au1 n d b o k s ) ((pos "K7$" ) ))
( "soundboxes"  n ( s au1 n d b o k s i z ) ((pos "Kj$" ) ))
( "sounded"  v ( s au1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "2D" "6A" "15A" )))
( "sounding"  v ( s au1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "6A" "15A" )))
( "sounding-board"  n ( s au1 n d i ng - b oo d ) ((pos "K6%" ) ))
( "sounding-boards"  n ( s au1 n d i ng - b oo d z ) ((pos "Kj%" ) ))
( "soundings"  n ( s au1 n d i ng z ) ((pos "Kj%" ) ))
( "soundless"  j ( s au1 n d l @ s ) ((pos "OA%" ) ))
( "soundlessly"  a ( s au1 n d l @ s l ii ) ((pos "Pu%" ) ))
( "soundly"  a ( s au1 n d l ii ) ((pos "Pu%" ) ))
( "soundness"  n ( s au1 n d n @ s ) ((pos "L@%" ) ))
( "soundproof"  v ( s au1 n d p r uu f ) ((pos "H0%" "OA%" ) (subcat )))
( "soundproofed"  v ( s au1 n d p r uu f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "soundproofing"  v ( s au1 n d p r uu f i ng ) ((pos "Hb%" ) (subcat )))
( "soundproofs"  v ( s au1 n d p r uu f s ) ((pos "Ha%" ) (subcat )))
( "sounds"  v ( s au1 n d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "2D" "6A" "15A" )))
( "soundtrack"  n ( s au1 n d t r a k ) ((pos "K6%" ) ))
( "soundtracks"  n ( s au1 n d t r a k s ) ((pos "Kj%" ) ))
( "soup"  v ( s uu1 p ) ((pos "H0%" "M6%" ) (subcat )))
( "soup-kitchen"  n ( s uu1 p - k i ch i n ) ((pos "K6%" ) ))
( "soup-kitchens"  n ( s uu1 p - k i ch i n z ) ((pos "Kj%" ) ))
( "soup<con"  n ( s uu1 p s o n ) ((pos "K6%" ) ))
( "soup<cons"  n ( s uu1 p s o n z ) ((pos "Kj$" ) ))
( "souped"  v ( s uu1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "souping"  v ( s uu1 p i ng ) ((pos "Hb$" ) (subcat )))
( "soups"  v ( s uu1 p s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "sour"  v ( s au1 @ r ) ((pos "J0%" "OA%" ) (subcat "2A" )))
( "source"  n ( s oo1 s ) ((pos "K6%" ) ))
( "sources"  n ( s oo1 s i z ) ((pos "Kj%" ) ))
( "soured"  v ( s au1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "souring"  v ( s au1 @ r i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "sourly"  a ( s au1 @ l ii ) ((pos "Pu%" ) ))
( "sourness"  n ( s au1 @ n @ s ) ((pos "L@%" ) ))
( "sours"  v ( s au1 @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "sous"  n ( s uu1 z ) ((pos "Kj$" ) ))
( "souse"  v ( s au1 s ) ((pos "H2$" ) (subcat )))
( "soused"  v ( s au1 s t ) ((pos "Hc$" "Hd$" "OA$" ) (subcat )))
( "souses"  v ( s au1 s i z ) ((pos "Ha$" ) (subcat )))
( "sousing"  v ( s au1 s i ng ) ((pos "Hb$" ) (subcat )))
( "soutane"  n ( s uu1 t aa1 n ) ((pos "K6$" ) ))
( "soutanes"  n ( s uu1 t aa1 n z ) ((pos "Kj$" ) ))
( "south"  n ( s au1 th ) ((pos "L@%" "Pu%" ) ))
( "south-southeast"  n ( s au2 th - s au th ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "south-southwest"  n ( s au2 th - s au th w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "southeast"  n ( s au1 th ii1 s t ) ((pos "L@%" "Pu%" ) ))
( "southeaster"  n ( s au1 th ii1 s t @ r ) ((pos "K6%" ) ))
( "southeasterly"  j ( s au1 th ii1 s t @ l ii ) ((pos "OA%" ) ))
( "southeastern"  j ( s au1 th ii1 s t @ n ) ((pos "OA%" ) ))
( "southeasters"  n ( s au1 th ii1 s t @ z ) ((pos "Kj%" ) ))
( "southerly"  j ( s uh1 dh @ l ii ) ((pos "OA%" "Pu%" ) ))
( "southern"  j ( s uh1 dh @ n ) ((pos "OA%" ) ))
( "southerner"  n ( s uh1 dh @ n @ r ) ((pos "K6%" ) ))
( "southerners"  n ( s uh1 dh @ n @ z ) ((pos "Kj%" ) ))
( "southernmost"  j ( s uh1 dh @ n m ou s t ) ((pos "OA%" ) ))
( "southpaw"  n ( s au1 th p oo ) ((pos "K6%" ) ))
( "southpaws"  n ( s au1 th p oo z ) ((pos "Kj%" ) ))
( "southward"  a ( s au1 th w @ d ) ((pos "Pu%" ) ))
( "southwards"  a ( s au1 th w @ d z ) ((pos "Pu%" ) ))
( "southwest"  n ( s au1 th w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "southwester"  n ( s au1 th w e1 s t @ r ) ((pos "K6%" ) ))
( "southwesterly"  j ( s au1 th w e1 s t @ l ii ) ((pos "OA%" ) ))
( "southwestern"  j ( s au1 th w e1 s t @ n ) ((pos "OA%" ) ))
( "southwesters"  n ( s au1 th w e1 s t @ z ) ((pos "Kj%" ) ))
( "souvenir"  n ( s uu2 v @ n i@1 r ) ((pos "K6%" ) ))
( "souvenirs"  n ( s uu2 v @ n i@1 z ) ((pos "Kj%" ) ))
( "sovereign"  n ( s o1 v r i n ) ((pos "K6%" "OA%" ) ))
( "sovereigns"  n ( s o1 v r i n z ) ((pos "Kj%" ) ))
( "sovereignty"  n ( s o1 v r @ n t ii ) ((pos "L@%" ) ))
( "soviet"  n ( s ou1 v i@ t ) ((pos "K6%" ) ))
( "sovietize"  v ( s o1 v i@ t ai z ) ((pos "H2$" ) ))
( "sovietized"  v ( s o1 v i@ t ai z d ) ((pos "Hc$" "Hd$" ) ))
( "sovietizes"  v ( s o1 v i@ t ai z i z ) ((pos "Ha$" ) ))
( "sovietizing"  v ( s o1 v i@ t ai z i ng ) ((pos "Hb$" ) ))
( "soviets"  n ( s ou1 v i@ t s ) ((pos "Kj%" ) ))
( "sow"  v ( s ou1 ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "sow"  n ( s au1 ) ((pos "K6%" ) ))
( "sowed"  v ( s ou1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "sower"  n ( s ou1 @ r ) ((pos "K6%" ) ))
( "sowers"  n ( s ou1 @ z ) ((pos "Kj%" ) ))
( "sowing"  v ( s ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "sown"  v ( s ou1 n ) ((pos "Jd%" ) (subcat "2A" "6A" )))
( "sows"  v ( s ou1 z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "sows"  n ( s au1 z ) ((pos "Kj%" ) ))
( "sox"  n ( s o1 k s ) ((pos "Kj$" ) ))
( "soy"  n ( s oi1 ) ((pos "L@%" ) ))
( "soya"  n ( s oi1 @ ) ((pos "L@%" ) ))
( "sozzled"  j ( s o1 z l d ) ((pos "OA%" ) ))
( "spa"  n ( s p aa1 ) ((pos "K6%" ) ))
( "space"  v ( s p ei1 s ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "space-bar"  n ( s p ei1 s - b aa r ) ((pos "K6%" ) ))
( "space-bars"  n ( s p ei1 s - b aa z ) ((pos "Kj%" ) ))
( "space-capsule"  n ( s p ei1 s - k a p s y uu l ) ((pos "K6%" ) ))
( "space-capsules"  n ( s p ei1 s - k a p s y uu l z ) ((pos "Kj%" ) ))
( "space-heater"  n ( s p ei1 s - h ii t @ r ) ((pos "K6%" ) ))
( "space-heaters"  n ( s p ei1 s - h ii t @ z ) ((pos "Kj%" ) ))
( "space-helmet"  n ( s p ei1 s - h e l m i t ) ((pos "K6%" ) ))
( "space-helmets"  n ( s p ei1 s - h e l m i t s ) ((pos "Kj%" ) ))
( "space-rocket"  n ( s p ei1 s - r o k i t ) ((pos "K6%" ) ))
( "space-rockets"  n ( s p ei1 s - r o k i t s ) ((pos "Kj%" ) ))
( "space-time"  n ( s p ei1 s - t ai m ) ((pos "L@%" ) ))
( "space-vehicle"  n ( s p ei1 s - v ii i k l ) ((pos "K6%" ) ))
( "space-vehicles"  n ( s p ei1 s - v ii i k l z ) ((pos "Kj%" ) ))
( "spacecraft"  n ( s p ei1 s k r aa f t ) ((pos "K9%" ) ))
( "spaced"  v ( s p ei1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "spaces"  v ( s p ei1 s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "spaceship"  n ( s p ei1 s - sh i p ) ((pos "K6%" ) ))
( "spaceships"  n ( s p ei1 s - sh i p s ) ((pos "Kj%" ) ))
( "spacesuit"  n ( s p ei1 s - s uu t ) ((pos "K6%" ) ))
( "spacesuits"  n ( s p ei1 s - s uu t s ) ((pos "Kj%" ) ))
( "spacing"  v ( s p ei1 s i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "spacings"  n ( s p ei1 s i ng ) ((pos "Mj%" ) ))
( "spacious"  j ( s p ei1 sh @ s ) ((pos "OA%" ) ))
( "spaciously"  a ( s p ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "spaciousness"  n ( s p ei1 sh @ s n @ s ) ((pos "L@%" ) ))
( "spade"  v ( s p ei1 d ) ((pos "H2$" "K6%" ) (subcat "6A" )))
( "spaded"  v ( s p ei1 d i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "spadeful"  n ( s p ei1 d f u l ) ((pos "K6%" ) ))
( "spadefuls"  n ( s p ei1 d f u l z ) ((pos "Kj%" ) ))
( "spades"  v ( s p ei1 d z ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "spadework"  n ( s p ei1 d w @@ k ) ((pos "L@%" ) ))
( "spading"  v ( s p ei1 d i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "spaghetti"  n ( s p @1 g e1 t ii ) ((pos "L@%" ) ))
( "spake"  v ( s p ei1 k ) ((pos "Jc$" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "spam"  n ( s p a1 m ) ((pos "L@%" ) ))
( "span"  v ( s p a1 n ) ((pos "H4%" "Jc%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "spangle"  v ( s p a1 ng g l ) ((pos "H2%" "K6%" ) ))
( "spangled"  v ( s p a1 ng g l d ) ((pos "Hc%" "Hd%" ) ))
( "spangles"  v ( s p a1 ng g l z ) ((pos "Ha%" "Kj%" ) ))
( "spangling"  v ( s p a1 ng g l i ng ) ((pos "Hb%" ) ))
( "spaniel"  n ( s p a1 n i@ l ) ((pos "K6%" ) ))
( "spaniels"  n ( s p a1 n i@ l z ) ((pos "Kj%" ) ))
( "spank"  v ( s p a1 ng k ) ((pos "J0%" ) (subcat "2C" )))
( "spanked"  v ( s p a1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "spanking"  v ( s p a1 ng k i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2C" )))
( "spankings"  n ( s p a1 ng k i ng z ) ((pos "Kj%" ) ))
( "spanks"  v ( s p a1 ng k s ) ((pos "Ja%" ) (subcat "2C" )))
( "spanned"  v ( s p a1 n d ) ((pos "Hc%" "Hd%" ) (subcat "2A" "2C" "6A" "14" )))
( "spanner"  n ( s p a1 n @ r ) ((pos "K6%" ) ))
( "spanners"  n ( s p a1 n @ z ) ((pos "Kj%" ) ))
( "spanning"  v ( s p a1 n i ng ) ((pos "Hb%" ) (subcat "2A" "2C" "6A" "14" )))
( "spans"  v ( s p a1 n z ) ((pos "Ha%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "spar"  v ( s p aa1 r ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "spare"  v ( s p e@1 r ) ((pos "J2%" "K6%" "OA%" ) (subcat "6A" "12A" "12B" "13A" )))
( "spare-rib"  n ( s p e@1 - r i b ) ((pos "K6%" ) ))
( "spare-ribs"  n ( s p e@1 - r i b z ) ((pos "Kj%" ) ))
( "spared"  v ( s p e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "6A" "12A" "12B" "13A" )))
( "sparely"  a ( s p e@1 l ii ) ((pos "Pu%" ) ))
( "spareness"  n ( s p e@1 n @ s ) ((pos "L@$" ) ))
( "spares"  v ( s p e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "6A" "12A" "12B" "13A" )))
( "sparing"  v ( s p e@1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "6A" "12A" "12B" "13A" )))
( "sparingly"  a ( s p e@1 r i ng l ii ) ((pos "Pu%" ) ))
( "spark"  v ( s p aa1 k ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "spark-plug"  n ( s p aa1 k - p l uh g ) ((pos "K6%" ) ))
( "spark-plugs"  n ( s p aa1 k - p l uh g z ) ((pos "Kj%" ) ))
( "sparked"  v ( s p aa1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sparking"  v ( s p aa1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sparking-plug"  n ( s p aa1 k i ng - p l uh g ) ((pos "K6%" ) ))
( "sparking-plugs"  n ( s p aa1 k i ng - p l uh g z ) ((pos "Kj%" ) ))
( "sparkle"  v ( s p aa1 k l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "sparkled"  v ( s p aa1 k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sparkler"  n ( s p aa1 k l @ r ) ((pos "K6%" ) ))
( "sparklers"  n ( s p aa1 k l @ z ) ((pos "Kj%" ) ))
( "sparkles"  v ( s p aa1 k l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "sparkling"  v ( s p aa1 k l i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "sparks"  v ( s p aa1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "sparred"  v ( s p aa1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sparring"  v ( s p aa1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sparring-match"  n ( s p aa1 r i ng - m a ch ) ((pos "K7%" ) ))
( "sparring-matches"  n ( s p aa1 r i ng - m a ch i z ) ((pos "Kj%" ) ))
( "sparring-partner"  n ( s p aa1 r i ng - p aa t n @ r ) ((pos "K6%" ) ))
( "sparring-partners"  n ( s p aa1 r i ng - p aa t n @ z ) ((pos "Kj%" ) ))
( "sparrow"  n ( s p a1 r ou ) ((pos "K6%" ) ))
( "sparrows"  n ( s p a1 r ou z ) ((pos "Kj%" ) ))
( "spars"  v ( s p aa1 z ) ((pos "Ia$" "Kj$" ) (subcat "2A" )))
( "sparse"  j ( s p aa1 s ) ((pos "OB%" ) ))
( "sparsely"  a ( s p aa1 s l ii ) ((pos "Pu%" ) ))
( "sparseness"  n ( s p aa1 s n @ s ) ((pos "L@%" ) ))
( "sparser"  j ( s p aa1 s @ r ) ((pos "Or%" ) ))
( "sparsest"  j ( s p aa1 s i s t ) ((pos "Os%" ) ))
( "sparsity"  n ( s p aa1 s i t ii ) ((pos "L@%" ) ))
( "spas"  n ( s p aa1 z ) ((pos "Kj%" ) ))
( "spasm"  n ( s p a1 z @ m ) ((pos "K6%" ) ))
( "spasmodic"  j ( s p a1 z m o1 d i k ) ((pos "OA%" ) ))
( "spasmodically"  a ( s p a1 z m o1 d i k l ii ) ((pos "Pu%" ) ))
( "spasms"  n ( s p a1 z @ m z ) ((pos "Kj%" ) ))
( "spastic"  n ( s p a1 s t i k ) ((pos "K6%" "OA%" ) ))
( "spastics"  n ( s p a1 s t i k s ) ((pos "Kj%" ) ))
( "spat"  v ( s p a1 t ) ((pos "J4$" "Jc%" "Jd%" "K6$" ) (subcat "2A" "2C" "3A" "6A" )))
( "spatchcock"  v ( s p a1 ch k o k ) ((pos "H0$" "K6$" ) (subcat "6A" )))
( "spatchcocked"  v ( s p a1 ch k o k t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "spatchcocking"  v ( s p a1 ch k o k i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "spatchcocks"  v ( s p a1 ch k o k s ) ((pos "Ha$" "Kj$" ) (subcat "6A" )))
( "spate"  n ( s p ei1 t ) ((pos "K6%" ) ))
( "spates"  n ( s p ei1 t s ) ((pos "Kj%" ) ))
( "spatial"  j ( s p ei1 sh l ) ((pos "OA%" ) ))
( "spatially"  a ( s p ei1 sh @ l ii ) ((pos "Pu%" ) ))
( "spats"  v ( s p a1 t s ) ((pos "Ja$" "Kj$" ) (subcat "2A" "2C" "3A" "6A" )))
( "spatted"  v ( s p a1 t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" "3A" "6A" )))
( "spatter"  v ( s p a1 t @ r ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "spattered"  v ( s p a1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "spattering"  v ( s p a1 t @ r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "spatters"  v ( s p a1 t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "spatting"  v ( s p a1 t i ng ) ((pos "Jb$" ) (subcat "2A" "2C" "3A" "6A" )))
( "spatula"  n ( s p a1 t y u l @ ) ((pos "K6%" ) ))
( "spatulas"  n ( s p a1 t y u l @ z ) ((pos "Kj%" ) ))
( "spavin"  n ( s p a1 v i n ) ((pos "L@$" ) ))
( "spavined"  j ( s p a1 v i n d ) ((pos "OA$" ) ))
( "spawn"  v ( s p oo1 n ) ((pos "J0%" "L@%" ) (subcat "2A" )))
( "spawned"  v ( s p oo1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "spawning"  v ( s p oo1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "spawns"  v ( s p oo1 n z ) ((pos "Ja%" ) (subcat "2A" )))
( "spay"  v ( s p ei1 ) ((pos "H0$" ) (subcat )))
( "spayed"  v ( s p ei1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "spaying"  v ( s p ei1 i ng ) ((pos "Hb$" ) (subcat )))
( "spays"  v ( s p ei1 z ) ((pos "Ha$" ) (subcat )))
( "speak"  v ( s p ii1 k ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "speaker"  n ( s p ii1 k @ r ) ((pos "K6%" ) ))
( "speakers"  n ( s p ii1 k @ z ) ((pos "Kj%" ) ))
( "speakership"  n ( s p ii1 k @ sh i p ) ((pos "K6$" ) ))
( "speakerships"  n ( s p ii1 k @ sh i p s ) ((pos "Kj$" ) ))
( "speaking"  v ( s p ii1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "speaking-trumpet"  n ( s p ii1 k i ng - t r uh m p i t ) ((pos "K6$" ) ))
( "speaking-trumpets"  n ( s p ii1 k i ng - t r uh m p i t s ) ((pos "Kj$" ) ))
( "speaking-tube"  n ( s p ii1 k i ng - t y uu b ) ((pos "K6%" ) ))
( "speaking-tubes"  n ( s p ii1 k i ng - t y uu b z ) ((pos "Kj%" ) ))
( "speaks"  v ( s p ii1 k s ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "spear"  v ( s p i@1 r ) ((pos "H0%" "K6%" ) (subcat )))
( "speared"  v ( s p i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spearhead"  n ( s p i@1 h e d ) ((pos "K6%" ) ))
( "spearheads"  n ( s p i@1 h e d z ) ((pos "Kj%" ) ))
( "spearing"  v ( s p i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "spearmint"  n ( s p i@1 m i n t ) ((pos "L@%" ) ))
( "spears"  v ( s p i@1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "spec"  n ( s p e1 k ) ((pos "K6$" ) ))
( "special"  n ( s p e1 sh l ) ((pos "K6%" "OA*" ) ))
( "specialism"  n ( s p e1 sh @ l i z m ) ((pos "K6%" ) ))
( "specialisms"  n ( s p e1 sh @ l i z @ m z ) ((pos "Kj%" ) ))
( "specialist"  n ( s p e1 sh @ l i s t ) ((pos "K6%" ) ))
( "specialists"  n ( s p e1 sh @ l i s t s ) ((pos "Kj%" ) ))
( "specialities"  n ( s p e2 sh i a1 l i t i z ) ((pos "Kj%" ) ))
( "speciality"  n ( s p e2 sh i a1 l i t ii ) ((pos "K8%" ) ))
( "specialization"  n ( s p e2 sh @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "specializations"  n ( s p e2 sh @ l ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "specialize"  v ( s p e1 sh @ l ai z ) ((pos "J2%" ) (subcat "2A" )))
( "specialized"  v ( s p e1 sh @ l ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "specializes"  v ( s p e1 sh @ l ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "specializing"  v ( s p e1 sh @ l ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "specially"  a ( s p e1 sh @ l ii ) ((pos "Pu%" ) ))
( "specials"  n ( s p e1 sh l z ) ((pos "Kj%" ) ))
( "specialties"  n ( s p e1 sh @ l t i z ) ((pos "Kj%" ) ))
( "specialty"  n ( s p e1 sh @ l t ii ) ((pos "K8%" ) ))
( "specie"  n ( s p ii1 sh ii ) ((pos "L@$" ) ))
( "species"  n ( s p ii1 sh ii z ) ((pos "K9%" ) ))
( "specifiable"  j ( s p e2 s i f ai1 @ b l ) ((pos "OA%" ) ))
( "specific"  n ( s p @1 s i1 f i k ) ((pos "K6%" "OA%" ) ))
( "specifically"  a ( s p @1 s i1 f i k l ii ) ((pos "Pu%" ) ))
( "specification"  n ( s p e2 s i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "specifications"  n ( s p e2 s i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "specificity"  n ( s p e2 s i f i1 s i t ii ) ((pos "L@%" ) ))
( "specifics"  n ( s p @1 s i1 f i k s ) ((pos "Kj%" ) ))
( "specified"  v ( s p e1 s i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "specifies"  v ( s p e1 s i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "specify"  v ( s p e1 s i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "specifying"  v ( s p e1 s i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "specimen"  n ( s p e1 s i m i n ) ((pos "K6%" ) ))
( "specimens"  n ( s p e1 s i m i n z ) ((pos "Kj%" ) ))
( "specious"  j ( s p ii1 sh @ s ) ((pos "OA$" ) ))
( "speciously"  a ( s p ii1 sh @ s l ii ) ((pos "Pu$" ) ))
( "speciousness"  n ( s p ii1 sh @ s n @ s ) ((pos "L@$" ) ))
( "speck"  n ( s p e1 k ) ((pos "K6%" ) ))
( "specked"  j ( s p e1 k t ) ((pos "OA%" ) ))
( "speckle"  n ( s p e1 k l ) ((pos "K6%" ) ))
( "speckled"  j ( s p e1 k l d ) ((pos "OA%" ) ))
( "speckles"  n ( s p e1 k l z ) ((pos "Kj%" ) ))
( "speckless"  j ( s p e1 k l @ s ) ((pos "OA$" ) ))
( "specks"  n ( s p e1 k s ) ((pos "Kj%" ) ))
( "specs"  n ( s p e1 k s ) ((pos "Kj%" ) ))
( "spectacle"  n ( s p e1 k t @ k l ) ((pos "K6%" ) ))
( "spectacled"  j ( s p e1 k t @ k l d ) ((pos "OA%" ) ))
( "spectacles"  n ( s p e1 k t @ k l z ) ((pos "Kj%" ) ))
( "spectacular"  n ( s p e1 k t a1 k y u l @ r ) ((pos "K6%" "OA%" ) ))
( "spectacularly"  a ( s p e1 k t a1 k y u l @ l ii ) ((pos "Pu%" ) ))
( "spectaculars"  n ( s p e1 k t a1 k y u l @ z ) ((pos "Kj%" ) ))
( "spectator"  n ( s p e1 k t ei1 t @ r ) ((pos "K6%" ) ))
( "spectators"  n ( s p e1 k t ei1 t @ z ) ((pos "Kj%" ) ))
( "spectra"  n ( s p e1 k t r @ ) ((pos "Kj$" ) ))
( "spectral"  j ( s p e1 k t r @ l ) ((pos "OA%" ) ))
( "spectre"  n ( s p e1 k t @ r ) ((pos "K6%" ) ))
( "spectres"  n ( s p e1 k t @ z ) ((pos "Kj%" ) ))
( "spectroscope"  n ( s p e1 k t r @ s k ou p ) ((pos "K6$" ) ))
( "spectroscopes"  n ( s p e1 k t r @ s k ou p s ) ((pos "Kj$" ) ))
( "spectroscopic"  j ( s p e2 k t r @ s k o1 p i k ) ((pos "OA$" ) ))
( "spectrum"  n ( s p e1 k t r @ m ) ((pos "Ki%" ) ))
( "speculate"  v ( s p e1 k y u l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "speculated"  v ( s p e1 k y u l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "speculates"  v ( s p e1 k y u l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "speculating"  v ( s p e1 k y u l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "speculation"  n ( s p e2 k y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "speculations"  n ( s p e2 k y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "speculative"  j ( s p e1 k y u l @ t i v ) ((pos "OA%" ) ))
( "speculatively"  a ( s p e1 k y u l @ t i v l ii ) ((pos "Pu%" ) ))
( "speculator"  n ( s p e1 k y u l ei t @ r ) ((pos "K6%" ) ))
( "speculators"  n ( s p e1 k y u l ei t @ z ) ((pos "Kj%" ) ))
( "sped"  v ( s p e1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "speech"  n ( s p ii1 ch ) ((pos "M7%" ) ))
( "speech-day"  n ( s p ii1 ch - d ei ) ((pos "K6%" ) ))
( "speech-days"  n ( s p ii1 ch - d ei z ) ((pos "Kj%" ) ))
( "speeches"  n ( s p ii1 ch i z ) ((pos "Mj%" ) ))
( "speechified"  v ( s p ii1 ch i f ai d ) ((pos "Ic$" "Id$" ) ))
( "speechifies"  v ( s p ii1 ch i f ai z ) ((pos "Ia$" ) ))
( "speechify"  v ( s p ii1 ch i f ai ) ((pos "I3$" ) ))
( "speechifying"  v ( s p ii1 ch i f ai i ng ) ((pos "Ib$" ) ))
( "speechless"  j ( s p ii1 ch l @ s ) ((pos "OA%" ) ))
( "speechlessly"  a ( s p ii1 ch l @ s l ii ) ((pos "Pu%" ) ))
( "speed"  v ( s p ii1 d ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "14" )))
( "speed-cop"  n ( s p ii1 d - k o p ) ((pos "K6%" ) ))
( "speed-cops"  n ( s p ii1 d - k o p s ) ((pos "Kj%" ) ))
( "speed-indicator"  n ( s p ii1 d - i n d i k ei t @ r ) ((pos "K6%" ) ))
( "speed-indicators"  n ( s p ii1 d - i n d i k ei t @ z ) ((pos "Kj%" ) ))
( "speed-limit"  n ( s p ii1 d - l i m i t ) ((pos "K6%" ) ))
( "speed-limits"  n ( s p ii1 d - l i m i t s ) ((pos "Kj%" ) ))
( "speed-up"  n ( s p ii1 d - uh p ) ((pos "K6%" ) ))
( "speed-ups"  n ( s p ii1 d - uh p s ) ((pos "Kj$" ) ))
( "speedboat"  n ( s p ii1 d b ou t ) ((pos "K6%" ) ))
( "speedboats"  n ( s p ii1 d b ou t s ) ((pos "Kj%" ) ))
( "speeded"  v ( s p ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "speedier"  j ( s p ii1 d i@ r ) ((pos "Or%" ) ))
( "speediest"  j ( s p ii1 d i i s t ) ((pos "Os%" ) ))
( "speedily"  a ( s p ii1 d i l ii ) ((pos "Pu%" ) ))
( "speeding"  v ( s p ii1 d i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "14" )))
( "speedometer"  n ( s p ii1 d o1 m i t @ r ) ((pos "K6%" ) ))
( "speedometers"  n ( s p ii1 d o1 m i t @ z ) ((pos "Kj%" ) ))
( "speeds"  v ( s p ii1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" )))
( "speedway"  n ( s p ii1 d w ei ) ((pos "M6%" ) ))
( "speedways"  n ( s p ii1 d w ei z ) ((pos "Mj%" ) ))
( "speedwell"  n ( s p ii1 d w e l ) ((pos "M6$" ) ))
( "speedwells"  n ( s p ii1 d w e l z ) ((pos "Mj$" ) ))
( "speedy"  j ( s p ii1 d ii ) ((pos "OD%" ) ))
( "spelaeologist"  n ( s p ii2 l i o1 l @ jh i s t ) ((pos "K6$" ) ))
( "spelaeologists"  n ( s p ii2 l i o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "spelaeology"  n ( s p ii2 l i o1 l @ jh ii ) ((pos "L@$" ) ))
( "speleologist"  n ( s p ii2 l i o1 l @ jh i s t ) ((pos "K6$" ) ))
( "speleologists"  n ( s p ii2 l i o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "speleology"  n ( s p ii2 l i o1 l @ jh ii ) ((pos "L@$" ) ))
( "spell"  v ( s p e1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "spellbinder"  n ( s p e1 l b ai n d @ r ) ((pos "K6%" ) ))
( "spellbinders"  n ( s p e1 l b ai n d @ z ) ((pos "Kj%" ) ))
( "spellbound"  j ( s p e1 l b au n d ) ((pos "OA%" ) ))
( "spelled"  v ( s p e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "speller"  n ( s p e1 l @ r ) ((pos "K6%" ) ))
( "spellers"  n ( s p e1 l @ z ) ((pos "Kj%" ) ))
( "spelling"  v ( s p e1 l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" )))
( "spellings"  n ( s p e1 l i ng z ) ((pos "Mj%" ) ))
( "spells"  v ( s p e1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "spelt"  v ( s p e1 l t ) ((pos "Jc%" "Jd%" "L@$" ) (subcat "2A" "6A" )))
( "spend"  v ( s p e1 n d ) ((pos "J5%" ) (subcat "2A" "6A" "14" )))
( "spender"  n ( s p e1 n d @ r ) ((pos "K6%" ) ))
( "spenders"  n ( s p e1 n d @ z ) ((pos "Kj%" ) ))
( "spending"  v ( s p e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "spends"  v ( s p e1 n d z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "spendthrift"  n ( s p e1 n d th r i f t ) ((pos "K6%" ) ))
( "spendthrifts"  n ( s p e1 n d th r i f t s ) ((pos "Kj%" ) ))
( "spent"  v ( s p e1 n t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "6A" "14" )))
( "sperm"  n ( s p @@1 m ) ((pos "M6%" ) ))
( "sperm-whale"  n ( s p @@1 m - w ei l ) ((pos "K6%" ) ))
( "sperm-whales"  n ( s p @@1 m - w ei l z ) ((pos "Kj%" ) ))
( "spermaceti"  n ( s p @@2 m @ s e1 t ii ) ((pos "L@$" ) ))
( "spermatozoa"  n ( s p @@2 m @ t @ z ou1 @ ) ((pos "Kj$" ) ))
( "spermatozoon"  n ( s p @@2 m @ t @ z ou1 @ n ) ((pos "Ki$" ) ))
( "spermicide"  n ( s p @@1 m i s ai d ) ((pos "M6%" ) ))
( "spermicides"  n ( s p @@1 m i s ai d z ) ((pos "Mj%" ) ))
( "sperms"  n ( s p @@1 m z ) ((pos "Mj%" ) ))
( "spew"  v ( s p y uu1 ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "spewed"  v ( s p y uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "spewing"  v ( s p y uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "spews"  v ( s p y uu1 z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "sphagnum"  n ( s f a1 g n @ m ) ((pos "M6$" ) ))
( "sphagnums"  n ( s f a1 g n @ m z ) ((pos "Mj$" ) ))
( "sphere"  n ( s f i@1 r ) ((pos "K6%" ) ))
( "spheres"  n ( s f i@1 z ) ((pos "Kj%" ) ))
( "spherical"  j ( s f e1 r i k l ) ((pos "OA%" ) ))
( "spheroid"  n ( s f i@1 r oi d ) ((pos "K6$" ) ))
( "spheroids"  n ( s f i@1 r oi d z ) ((pos "Kj$" ) ))
( "sphinx"  n ( s f i1 ng k s ) ((pos "K7%" ) ))
( "sphinxes"  n ( s f i1 ng k s i z ) ((pos "Kj%" ) ))
( "spice"  v ( s p ai1 s ) ((pos "H2%" "M6%" ) (subcat )))
( "spiced"  v ( s p ai1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spices"  v ( s p ai1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "spicier"  j ( s p ai1 s i@ r ) ((pos "Or%" ) ))
( "spiciest"  j ( s p ai1 s i i s t ) ((pos "Os%" ) ))
( "spicily"  a ( s p ai1 s i l ii ) ((pos "Pu%" ) ))
( "spiciness"  n ( s p ai1 s i n @ s ) ((pos "L@%" ) ))
( "spicing"  v ( s p ai1 s i ng ) ((pos "Hb%" ) (subcat )))
( "spick"  j ( s p i1 k ) ((pos "OA%" ) ))
( "spicy"  j ( s p ai1 s ii ) ((pos "OD%" ) ))
( "spider"  n ( s p ai1 d @ r ) ((pos "K6%" ) ))
( "spiders"  n ( s p ai1 d @ z ) ((pos "Kj%" ) ))
( "spidery"  j ( s p ai1 d @ r ii ) ((pos "OA%" ) ))
( "spied"  v ( s p ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "15B" )))
( "spiel"  v ( sh p ii1 l ) ((pos "J0$" "K6%" ) ))
( "spieled"  v ( sh p ii1 l d ) ((pos "Jc$" "Jd$" ) ))
( "spieling"  v ( sh p ii1 l i ng ) ((pos "Jb$" ) ))
( "spiels"  v ( sh p ii1 l z ) ((pos "Ja$" "Kj$" ) ))
( "spies"  v ( s p ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "15B" )))
( "spigot"  n ( s p i1 g @ t ) ((pos "K6$" ) ))
( "spigots"  n ( s p i1 g @ t s ) ((pos "Kj$" ) ))
( "spike"  v ( s p ai1 k ) ((pos "H2%" "K6%" ) (subcat )))
( "spiked"  v ( s p ai1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spikenard"  n ( s p ai1 k n aa d ) ((pos "L@$" ) ))
( "spikes"  v ( s p ai1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "spikier"  j ( s p ai1 k i@ r ) ((pos "Or%" ) ))
( "spikiest"  j ( s p ai1 k i i s t ) ((pos "Os%" ) ))
( "spiking"  v ( s p ai1 k i ng ) ((pos "Hb%" ) (subcat )))
( "spiky"  j ( s p ai1 k ii ) ((pos "OD%" ) ))
( "spill"  v ( s p i1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "spillage"  n ( s p i1 l i jh ) ((pos "K6%" ) ))
( "spillages"  n ( s p i1 l i jh i z ) ((pos "Kj%" ) ))
( "spilled"  v ( s p i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "spilling"  v ( s p i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "spillover"  n ( s p i1 l ou v @ r ) ((pos "K6%" ) ))
( "spillovers"  n ( s p i1 l ou v @ z ) ((pos "Kj$" ) ))
( "spills"  v ( s p i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "spillway"  n ( s p i1 l w ei ) ((pos "K6$" ) ))
( "spillways"  n ( s p i1 l w ei z ) ((pos "Kj$" ) ))
( "spilt"  v ( s p i1 l t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "spin"  v ( s p i1 n ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "6A" "14" )))
( "spin-dried"  v ( s p i1 n - d r ai d ) ((pos "Hc%" "Hd%" ) ))
( "spin-drier"  n ( s p i2 n - d r ai1 @ r ) ((pos "K6%" ) ))
( "spin-driers"  n ( s p i2 n - d r ai1 @ z ) ((pos "Kj%" ) ))
( "spin-dries"  v ( s p i1 n - d r ai z ) ((pos "Ha%" ) ))
( "spin-dry"  v ( s p i1 n - d r ai ) ((pos "H3%" ) ))
( "spin-drying"  v ( s p i1 n - d r ai i ng ) ((pos "Hb%" ) ))
( "spin-off"  n ( s p i1 n - o f ) ((pos "K6%" ) ))
( "spin-offs"  n ( s p i1 n - o f s ) ((pos "Kj%" ) ))
( "spinach"  n ( s p i1 n i jh ) ((pos "L@%" ) ))
( "spinal"  j ( s p ai1 n l ) ((pos "OA%" ) ))
( "spindle"  n ( s p i1 n d l ) ((pos "K6%" ) ))
( "spindle-berries"  n ( s p i1 n d l - b e r i z ) ((pos "Kj$" ) ))
( "spindle-berry"  n ( s p i1 n d l - b e r ii ) ((pos "K8$" ) ))
( "spindle-legged"  j ( s p i1 n d l - l e g d ) ((pos "OA$" ) ))
( "spindle-shanked"  j ( s p i1 n d l - sh a n k t ) ((pos "OA$" ) ))
( "spindle-shanks"  n ( s p i1 n d l - sh a n k s ) ((pos "Ki$" ) ))
( "spindle-tree"  n ( s p i1 n d l - t r ii ) ((pos "K6$" ) ))
( "spindle-trees"  n ( s p i1 n d l - t r ii z ) ((pos "Kj$" ) ))
( "spindles"  n ( s p i1 n d l z ) ((pos "Kj%" ) ))
( "spindlier"  j ( s p i1 n d l i@ r ) ((pos "Or%" ) ))
( "spindliest"  j ( s p i1 n d l i i s t ) ((pos "Os%" ) ))
( "spindly"  j ( s p i1 n d l ii ) ((pos "OD%" ) ))
( "spindrift"  n ( s p i1 n d r i f t ) ((pos "L@$" ) ))
( "spine"  n ( s p ai1 n ) ((pos "K6%" ) ))
( "spineless"  j ( s p ai1 n l @ s ) ((pos "OA%" ) ))
( "spines"  n ( s p ai1 n z ) ((pos "Kj%" ) ))
( "spinet"  n ( s p i1 n e1 t ) ((pos "K6$" ) ))
( "spinets"  n ( s p i1 n e1 t s ) ((pos "Kj$" ) ))
( "spinier"  j ( s p ai1 n i@ r ) ((pos "Or$" ) ))
( "spiniest"  j ( s p ai1 n i i s t ) ((pos "Os$" ) ))
( "spinnaker"  n ( s p i1 n @ k @ r ) ((pos "K6$" ) ))
( "spinnakers"  n ( s p i1 n @ k @ z ) ((pos "Kj$" ) ))
( "spinney"  n ( s p i1 n ii ) ((pos "K6$" ) ))
( "spinneys"  n ( s p i1 n i z ) ((pos "Kj$" ) ))
( "spinning"  v ( s p i1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "spinning-wheel"  n ( s p i1 n i ng - w ii l ) ((pos "K6%" ) ))
( "spinning-wheels"  n ( s p i1 n i ng - w ii l z ) ((pos "Kj%" ) ))
( "spins"  v ( s p i1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" )))
( "spinster"  n ( s p i1 n s t @ r ) ((pos "K6%" ) ))
( "spinsterhood"  n ( s p i1 n s t @ h u d ) ((pos "L@%" ) ))
( "spinsters"  n ( s p i1 n s t @ z ) ((pos "Kj%" ) ))
( "spiny"  j ( s p ai1 n ii ) ((pos "OD%" ) ))
( "spiral"  v ( s p ai1 @ r @ l ) ((pos "I4%" "K6%" "OA%" ) (subcat "2A" )))
( "spiralled"  v ( s p ai1 @ r @ l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "spiralling"  v ( s p ai1 @ r @ l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "spirally"  a ( s p ai1 @ r @ l ii ) ((pos "Pu%" ) ))
( "spirals"  v ( s p ai1 @ r @ l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "spire"  n ( s p ai1 @ r ) ((pos "K6%" ) ))
( "spires"  n ( s p ai1 @ z ) ((pos "Kj%" ) ))
( "spirit"  v ( s p i1 r i t ) ((pos "H0%" "M6%" ) (subcat )))
( "spirit-lamp"  n ( s p i1 r i t - l a m p ) ((pos "K6%" ) ))
( "spirit-lamps"  n ( s p i1 r i t - l a m p s ) ((pos "Kj%" ) ))
( "spirit-level"  n ( s p i1 r i t - l e v @ l ) ((pos "K6%" ) ))
( "spirit-levels"  n ( s p i1 r i t - l e v @ l z ) ((pos "Kj%" ) ))
( "spirit-rapper"  n ( s p i1 r i t - r a p @ r ) ((pos "K6$" ) ))
( "spirit-rappers"  n ( s p i1 r i t - r a p @ z ) ((pos "Kj$" ) ))
( "spirit-stove"  n ( s p i1 r i t - s t ou v ) ((pos "K6%" ) ))
( "spirit-stoves"  n ( s p i1 r i t - s t ou v z ) ((pos "Kj%" ) ))
( "spirited"  v ( s p i1 r i t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "spiriting"  v ( s p i1 r i t i ng ) ((pos "Hb%" ) (subcat )))
( "spiritless"  j ( s p i1 r i t l @ s ) ((pos "OA%" ) ))
( "spirits"  v ( s p i1 r i t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "spiritual"  n ( s p i1 r i ch u@ l ) ((pos "K6%" "OA%" ) ))
( "spiritualism"  n ( s p i1 r i ch u@ l i z @ m ) ((pos "L@%" ) ))
( "spiritualist"  n ( s p i1 r i ch u@ l i s t ) ((pos "K6%" ) ))
( "spiritualistic"  j ( s p i2 r i ch u@ l i1 s t i k ) ((pos "OA$" ) ))
( "spiritualists"  n ( s p i1 r i ch u@ l i s t s ) ((pos "Kj%" ) ))
( "spirituality"  n ( s p i2 r i ch u a1 l i t ii ) ((pos "L@%" ) ))
( "spiritualization"  n ( s p i2 r i ch u@ l ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "spiritualize"  v ( s p i1 r i ch u@ l ai z ) ((pos "H2$" ) (subcat )))
( "spiritualized"  v ( s p i1 r i ch u@ l ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "spiritualizes"  v ( s p i1 r i ch u@ l ai z i z ) ((pos "Ha$" ) (subcat )))
( "spiritualizing"  v ( s p i1 r i ch u@ l ai z i ng ) ((pos "Hb$" ) (subcat )))
( "spiritually"  a ( s p i1 r i ch u@ l ii ) ((pos "Pu%" ) ))
( "spirituals"  n ( s p i1 r i ch u@ l z ) ((pos "Kj%" ) ))
( "spirituous"  j ( s p i1 r i t y u@ s ) ((pos "OA$" ) ))
( "spirt"  v ( s p @@1 t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "spirted"  v ( s p @@1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "spirting"  v ( s p @@1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "spirts"  v ( s p @@1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "spit"  v ( s p i1 t ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "spite"  v ( s p ai1 t ) ((pos "H2%" "L@%" ) (subcat )))
( "spited"  v ( s p ai1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spiteful"  j ( s p ai1 t f @ l ) ((pos "OA%" ) ))
( "spitefully"  a ( s p ai1 t f @ l ii ) ((pos "Pu%" ) ))
( "spitefulness"  n ( s p ai1 t f @ l n @ s ) ((pos "L@%" ) ))
( "spites"  v ( s p ai1 t s ) ((pos "Ha%" ) (subcat )))
( "spitfire"  n ( s p i1 t f ai @ r ) ((pos "K6%" ) ))
( "spitfires"  n ( s p i1 t f ai @ z ) ((pos "Kj%" ) ))
( "spiting"  v ( s p ai1 t i ng ) ((pos "Hb$" ) (subcat )))
( "spits"  v ( s p i1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "spitting"  v ( s p i1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "3A" "6A" )))
( "spittle"  n ( s p i1 t l ) ((pos "L@%" ) ))
( "spittoon"  n ( s p i1 t uu1 n ) ((pos "K6%" ) ))
( "spittoons"  n ( s p i1 t uu1 n z ) ((pos "Kj%" ) ))
( "spiv"  n ( s p i1 v ) ((pos "K6%" ) ))
( "spivs"  n ( s p i1 v z ) ((pos "Kj%" ) ))
( "splash"  v ( s p l a1 sh ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "14" )))
( "splashdown"  n ( s p l a1 sh d au n ) ((pos "K6%" ) ))
( "splashdowns"  n ( s p l a1 sh d au n z ) ((pos "Kj$" ) ))
( "splashed"  v ( s p l a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "splashes"  v ( s p l a1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "splashing"  v ( s p l a1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "splay"  v ( s p l ei1 ) ((pos "J0%" "K6$" "OA%" ) ))
( "splayed"  v ( s p l ei1 d ) ((pos "Jc%" "Jd%" ) ))
( "splayfeet"  n ( s p l ei1 f ii t ) ((pos "Kj%" ) ))
( "splayfoot"  n ( s p l ei1 f u t ) ((pos "Ki%" ) ))
( "splayfooted"  j ( s p l ei1 f u t i d ) ((pos "OA%" ) ))
( "splaying"  v ( s p l ei1 i ng ) ((pos "Jb%" ) ))
( "splays"  v ( s p l ei1 z ) ((pos "Ja%" "Kj$" ) ))
( "spleen"  n ( s p l ii1 n ) ((pos "M6%" ) ))
( "spleens"  n ( s p l ii1 n z ) ((pos "Mj%" ) ))
( "splendid"  j ( s p l e1 n d i d ) ((pos "OA%" ) ))
( "splendidly"  a ( s p l e1 n d i d l ii ) ((pos "Pu%" ) ))
( "splendiferous"  j ( s p l e1 n d i1 f @ r @ s ) ((pos "OA$" ) ))
( "splendour"  n ( s p l e1 n d @ r ) ((pos "M6%" ) ))
( "splendours"  n ( s p l e1 n d @ z ) ((pos "Mj%" ) ))
( "splenetic"  j ( s p l i1 n e1 t i k ) ((pos "OA$" ) ))
( "splice"  v ( s p l ai1 s ) ((pos "H2%" "K6%" ) (subcat )))
( "spliced"  v ( s p l ai1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "splicer"  n ( s p l ai1 s @ r ) ((pos "K6$" ) ))
( "splicers"  n ( s p l ai1 s @ z ) ((pos "Kj$" ) ))
( "splices"  v ( s p l ai1 s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "splicing"  v ( s p l ai1 s i ng ) ((pos "Hb%" ) (subcat )))
( "splint"  n ( s p l i1 n t ) ((pos "K6%" ) ))
( "splinter"  v ( s p l i1 n t @ r ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "splinter-proof"  j ( s p l i1 n t @ - p r uu f ) ((pos "OA$" ) ))
( "splintered"  v ( s p l i1 n t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "splintering"  v ( s p l i1 n t @ r i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "splinters"  v ( s p l i1 n t @ z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "splintery"  j ( s p l i1 n t @ r ii ) ((pos "OA%" ) ))
( "splints"  n ( s p l i1 n t s ) ((pos "Kj%" ) ))
( "split"  v ( s p l i1 t ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "14" "15A" "15B" )))
( "splits"  v ( s p l i1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "14" "15A" "15B" )))
( "splitting"  v ( s p l i1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "2D" "3A" "6A" "14" "15A" "15B" )))
( "splodge"  n ( s p l o1 jh ) ((pos "K6%" ) ))
( "splodges"  n ( s p l o1 jh i z ) ((pos "Kj%" ) ))
( "splosh"  v ( s p l o1 sh ) ((pos "H1%" ) ))
( "sploshed"  v ( s p l o1 sh t ) ((pos "Hc%" "Hd%" ) ))
( "sploshes"  v ( s p l o1 sh i z ) ((pos "Ha%" ) ))
( "sploshing"  v ( s p l o1 sh i ng ) ((pos "Hb%" ) ))
( "splotch"  n ( s p l o1 ch ) ((pos "K7%" ) ))
( "splotches"  n ( s p l o1 ch i z ) ((pos "Kj%" ) ))
( "splurge"  v ( s p l @@1 jh ) ((pos "I2%" "K6%" ) ))
( "splurged"  v ( s p l @@1 jh d ) ((pos "Ic%" "Id%" ) ))
( "splurges"  v ( s p l @@1 jh i z ) ((pos "Ia%" "Kj%" ) ))
( "splurging"  v ( s p l @@1 jh i ng ) ((pos "Ib%" ) ))
( "splutter"  v ( s p l uh1 t @ r ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "spluttered"  v ( s p l uh1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "spluttering"  v ( s p l uh1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "splutters"  v ( s p l uh1 t @ z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "spoil"  v ( s p oi1 l ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "spoiled"  v ( s p oi1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "spoiling"  v ( s p oi1 l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "spoils"  v ( s p oi1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "spoilsport"  n ( s p oi1 l s p oo t ) ((pos "K6%" ) ))
( "spoilsports"  n ( s p oi1 l s p oo t s ) ((pos "Kj%" ) ))
( "spoilt"  v ( s p oi1 l t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "spoke"  v ( s p ou1 k ) ((pos "Jc%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "spoken"  v ( s p ou1 k @ n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" )))
( "spokes"  n ( s p ou1 k s ) ((pos "Kj%" ) ))
( "spokesman"  n ( s p ou1 k s m @ n ) ((pos "Ki%" ) ))
( "spokesmen"  n ( s p ou1 k s m @ n ) ((pos "Kj%" ) ))
( "spoliation"  n ( s p ou2 l i ei1 sh @ n ) ((pos "L@$" ) ))
( "spondaic"  j ( s p o1 n d ei1 i k ) ((pos "OA$" ) ))
( "spondee"  n ( s p o1 n d ii ) ((pos "K6$" ) ))
( "spondees"  n ( s p o1 n d ii z ) ((pos "Kj$" ) ))
( "sponge"  v ( s p uh1 n jh ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "14" )))
( "sponge-cake"  n ( s p uh1 n jh - k ei k ) ((pos "M6%" ) ))
( "sponge-cakes"  n ( s p uh1 n jh - k ei k s ) ((pos "Mj%" ) ))
( "sponged"  v ( s p uh1 n jh d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" )))
( "sponger"  n ( s p uh1 n jh @ r ) ((pos "K6%" ) ))
( "spongers"  n ( s p uh1 n jh @ z ) ((pos "Kj%" ) ))
( "sponges"  v ( s p uh1 n jh i z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" )))
( "spongier"  j ( s p uh1 n jh i@ r ) ((pos "Or%" ) ))
( "spongiest"  j ( s p uh1 n jh i i s t ) ((pos "Os%" ) ))
( "sponginess"  n ( s p uh1 n jh i n @ s ) ((pos "L@%" ) ))
( "sponging"  v ( s p uh1 n jh i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" )))
( "spongy"  j ( s p uh1 n jh ii ) ((pos "OD%" ) ))
( "sponsor"  v ( s p o1 n s @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "sponsored"  v ( s p o1 n s @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sponsoring"  v ( s p o1 n s @ r i ng ) ((pos "Hb%" ) (subcat )))
( "sponsors"  v ( s p o1 n s @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sponsorship"  n ( s p o1 n s @ sh i p ) ((pos "M6%" ) ))
( "sponsorships"  n ( s p o1 n s @ sh i p s ) ((pos "Mj%" ) ))
( "spontaneity"  n ( s p o2 n t @ n i@1 t ii ) ((pos "L@%" ) ))
( "spontaneous"  j ( s p o1 n t ei1 n i@ s ) ((pos "OA%" ) ))
( "spontaneously"  a ( s p o1 n t ei1 n i@ s l ii ) ((pos "Pu%" ) ))
( "spontaneousness"  n ( s p o1 n t ei1 n i@ s n @ s ) ((pos "L@$" ) ))
( "spoof"  v ( s p uu1 f ) ((pos "H0$" "K6%" ) ))
( "spoofed"  v ( s p uu1 f t ) ((pos "Hc$" "Hd$" ) ))
( "spoofing"  v ( s p uu1 f i ng ) ((pos "Hb$" ) ))
( "spoofs"  v ( s p uu1 f s ) ((pos "Ha$" "Kj%" ) ))
( "spook"  n ( s p uu1 k ) ((pos "K6%" ) ))
( "spookier"  j ( s p uu1 k i@ r ) ((pos "Or%" ) ))
( "spookiest"  j ( s p uu1 k i i s t ) ((pos "Os%" ) ))
( "spooks"  n ( s p uu1 k s ) ((pos "Kj%" ) ))
( "spooky"  j ( s p uu1 k ii ) ((pos "OD%" ) ))
( "spool"  n ( s p uu1 l ) ((pos "K6%" ) ))
( "spools"  n ( s p uu1 l z ) ((pos "Kj%" ) ))
( "spoon"  v ( s p uu1 n ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "spooned"  v ( s p uu1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "spoonerism"  n ( s p uu1 n @ r i z @ m ) ((pos "K6%" ) ))
( "spoonerisms"  n ( s p uu1 n @ r i z @ m z ) ((pos "Kj%" ) ))
( "spoonfed"  v ( s p uu1 n f e d ) ((pos "Hc%" "Hd%" ) ))
( "spoonfeed"  v ( s p uu1 n f ii d ) ((pos "H5%" ) ))
( "spoonfeeding"  v ( s p uu1 n f ii d i ng ) ((pos "Hb%" "L@%" ) ))
( "spoonfeeds"  v ( s p uu1 n f ii d z ) ((pos "Ha%" ) ))
( "spoonful"  n ( s p uu1 n f u l ) ((pos "K6%" ) ))
( "spoonfuls"  n ( s p uu1 n f u l z ) ((pos "Kj%" ) ))
( "spooning"  v ( s p uu1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "spoons"  v ( s p uu1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "spoor"  n ( s p u@1 r ) ((pos "K6%" ) ))
( "spoors"  n ( s p u@1 z ) ((pos "Kj%" ) ))
( "sporadic"  j ( s p @1 r a1 d i k ) ((pos "OA%" ) ))
( "sporadically"  a ( s p @1 r a1 d i k l ii ) ((pos "Pu%" ) ))
( "spore"  n ( s p oo1 r ) ((pos "K6%" ) ))
( "spores"  n ( s p oo1 z ) ((pos "Kj%" ) ))
( "sporran"  n ( s p o1 r @ n ) ((pos "K6%" ) ))
( "sporrans"  n ( s p o1 r @ n z ) ((pos "Kj%" ) ))
( "sport"  v ( s p oo1 t ) ((pos "J0%" "M6%" ) (subcat "2C" )))
( "sported"  v ( s p oo1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "sporting"  v ( s p oo1 t i ng ) ((pos "Jb%" "OA%" ) (subcat "2C" )))
( "sportingly"  a ( s p oo1 t i ng l ii ) ((pos "Pu%" ) ))
( "sportive"  j ( s p oo1 t i v ) ((pos "OA%" ) ))
( "sportively"  a ( s p oo1 t i v l ii ) ((pos "Pu%" ) ))
( "sportiveness"  n ( s p oo1 t i v n @ s ) ((pos "L@%" ) ))
( "sports"  v ( s p oo1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "sports-car"  n ( s p oo1 t s - k aa r ) ((pos "K6%" ) ))
( "sports-cars"  n ( s p oo1 t s - k aa z ) ((pos "Kj%" ) ))
( "sports-coat"  n ( s p oo1 t s - k ou t ) ((pos "K6%" ) ))
( "sports-coats"  n ( s p oo1 t s - k ou t s ) ((pos "Kj%" ) ))
( "sports-editor"  n ( s p oo1 t s - e d i t @ r ) ((pos "K6%" ) ))
( "sports-editors"  n ( s p oo1 t s - e d i t @ z ) ((pos "Kj%" ) ))
( "sports-jacket"  n ( s p oo1 t s - jh a k i t ) ((pos "K6%" ) ))
( "sports-jackets"  n ( s p oo1 t s - jh a k i t s ) ((pos "Kj%" ) ))
( "sportsman"  n ( s p oo1 t s m @ n ) ((pos "Ki%" ) ))
( "sportsmanlike"  j ( s p oo1 t s m @ n l ai k ) ((pos "OA%" ) ))
( "sportsmanship"  n ( s p oo1 t s m @ n sh i p ) ((pos "L@%" ) ))
( "sportsmen"  n ( s p oo1 t s m @ n ) ((pos "Kj%" ) ))
( "spot"  v ( s p o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "spotless"  j ( s p o1 t l @ s ) ((pos "OA%" ) ))
( "spotlessly"  a ( s p o1 t l @ s l ii ) ((pos "Pu%" ) ))
( "spotlight"  v ( s p o1 t l ai t ) ((pos "H0%" "K6%" ) (subcat )))
( "spotlighted"  v ( s p o1 t l ai t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spotlighting"  v ( s p o1 t l ai t i ng ) ((pos "Hb%" ) (subcat )))
( "spotlights"  v ( s p o1 t l ai t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "spots"  v ( s p o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "spotted"  v ( s p o1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "spotter"  n ( s p o1 t @ r ) ((pos "K6%" ) ))
( "spotters"  n ( s p o1 t @ z ) ((pos "Kj%" ) ))
( "spottier"  j ( s p o1 t i@ r ) ((pos "Or%" ) ))
( "spottiest"  j ( s p o1 t i i s t ) ((pos "Os%" ) ))
( "spotting"  v ( s p o1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "spotty"  j ( s p o1 t ii ) ((pos "OD%" ) ))
( "spouse"  n ( s p au1 z ) ((pos "K6%" ) ))
( "spouses"  n ( s p au1 z i z ) ((pos "Kj%" ) ))
( "spout"  v ( s p au1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "spouted"  v ( s p au1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "spouting"  v ( s p au1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "spouts"  v ( s p au1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "sprain"  v ( s p r ei1 n ) ((pos "H0%" "K6%" ) (subcat )))
( "sprained"  v ( s p r ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spraining"  v ( s p r ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "sprains"  v ( s p r ei1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "sprang"  v ( s p r a1 ng ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" )))
( "sprat"  n ( s p r a1 t ) ((pos "K6%" ) ))
( "sprats"  n ( s p r a1 t s ) ((pos "Kj%" ) ))
( "sprawl"  v ( s p r oo1 l ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "sprawled"  v ( s p r oo1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sprawling"  v ( s p r oo1 l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sprawls"  v ( s p r oo1 l z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "spray"  v ( s p r ei1 ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "spray-gun"  n ( s p r ei1 - g uh n ) ((pos "K6%" ) ))
( "spray-guns"  n ( s p r ei1 - g uh n z ) ((pos "Kj%" ) ))
( "sprayed"  v ( s p r ei1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "sprayer"  n ( s p r ei1 @ r ) ((pos "K6%" ) ))
( "sprayers"  n ( s p r ei1 @ z ) ((pos "Kj%" ) ))
( "spraying"  v ( s p r ei1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "sprays"  v ( s p r ei1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "spread"  v ( s p r e1 d ) ((pos "J5%" "Jc%" "Jd%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "spread-over"  n ( s p r e1 d - ou v @ r ) ((pos "L@$" ) ))
( "spreadeagle"  v ( s p r e1 d ii1 g l ) ((pos "H2%" "K6$" ) ))
( "spreadeagled"  v ( s p r e1 d ii1 g l d ) ((pos "Hc%" "Hd%" ) ))
( "spreadeagles"  v ( s p r e1 d ii1 g l z ) ((pos "Ha$" "Kj$" ) ))
( "spreadeagling"  v ( s p r e1 d ii1 g l i ng ) ((pos "Hb$" ) ))
( "spreader"  n ( s p r e1 d @ r ) ((pos "K6%" ) ))
( "spreaders"  n ( s p r e1 d @ z ) ((pos "Kj%" ) ))
( "spreading"  v ( s p r e1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "spreads"  v ( s p r e1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "spree"  n ( s p r ii1 ) ((pos "K6%" ) ))
( "sprees"  n ( s p r ii1 z ) ((pos "Kj$" ) ))
( "sprig"  n ( s p r i1 g ) ((pos "K6%" ) ))
( "sprigged"  j ( s p r i1 g d ) ((pos "OA%" ) ))
( "sprightlier"  j ( s p r ai1 t l i@ r ) ((pos "Or%" ) ))
( "sprightliest"  j ( s p r ai1 t l i i s t ) ((pos "Os%" ) ))
( "sprightliness"  n ( s p r ai1 t l i n @ s ) ((pos "L@%" ) ))
( "sprightly"  j ( s p r ai1 t l ii ) ((pos "OD%" ) ))
( "sprigs"  n ( s p r i1 g z ) ((pos "Kj%" ) ))
( "spring"  v ( s p r i1 ng ) ((pos "J5%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "spring-balance"  n ( s p r i1 ng - b a1 l @ n s ) ((pos "K6%" ) ))
( "spring-balances"  n ( s p r i1 ng - b a1 l @ n s i z ) ((pos "Kj%" ) ))
( "spring-clean"  v ( s p r i1 ng - k l ii1 n ) ((pos "H0%" "K6%" ) ))
( "spring-cleaned"  v ( s p r i1 ng - k l ii1 n d ) ((pos "Hc%" "Hd%" ) ))
( "spring-cleaning"  v ( s p r i1 ng - k l ii1 n i ng ) ((pos "Hb%" "L@%" ) ))
( "spring-cleans"  v ( s p r i1 ng - k l ii1 n z ) ((pos "Ha%" "Kj%" ) ))
( "spring-gun"  n ( s p r i1 ng - g uh n ) ((pos "K6$" ) ))
( "spring-guns"  n ( s p r i1 ng - g uh n z ) ((pos "Kj$" ) ))
( "spring-mattress"  n ( s p r i1 ng - m a1 t r i s ) ((pos "K7%" ) ))
( "spring-mattresses"  n ( s p r i1 ng - m a1 t r i s i z ) ((pos "Kj%" ) ))
( "springboard"  n ( s p r i1 ng b oo d ) ((pos "K6%" ) ))
( "springboards"  n ( s p r i1 ng b oo d z ) ((pos "Kj%" ) ))
( "springbok"  n ( s p r i1 ng b o k ) ((pos "K6%" ) ))
( "springboks"  n ( s p r i1 ng b o k s ) ((pos "Kj%" ) ))
( "springier"  j ( s p r i1 ng i@ r ) ((pos "Or%" ) ))
( "springiest"  j ( s p r i1 ng i i s t ) ((pos "Os%" ) ))
( "springing"  v ( s p r i1 ng i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "springless"  j ( s p r i1 ng l @ s ) ((pos "OA$" ) ))
( "springlike"  j ( s p r i1 ng l ai k ) ((pos "OA%" ) ))
( "springs"  v ( s p r i1 ng z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "springtide"  n ( s p r i1 ng t ai d ) ((pos "K6%" ) ))
( "springtides"  n ( s p r i1 ng t ai d z ) ((pos "Kj%" ) ))
( "springtime"  n ( s p r i1 ng t ai m ) ((pos "M6%" ) ))
( "springtimes"  n ( s p r i1 ng t ai m z ) ((pos "Mj%" ) ))
( "springy"  j ( s p r i1 ng ii ) ((pos "OD%" ) ))
( "sprinkle"  v ( s p r i1 ng k l ) ((pos "H2%" ) (subcat "6A" )))
( "sprinkled"  v ( s p r i1 ng k l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "sprinkler"  n ( s p r i1 ng k l @ r ) ((pos "K6%" ) ))
( "sprinklers"  n ( s p r i1 ng k l @ z ) ((pos "Kj%" ) ))
( "sprinkles"  v ( s p r i1 ng k l z ) ((pos "Ha%" ) (subcat "6A" )))
( "sprinkling"  v ( s p r i1 ng k l i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" )))
( "sprinklings"  n ( s p r i1 ng k l i ng z ) ((pos "Kj%" ) ))
( "sprint"  v ( s p r i1 n t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "sprinted"  v ( s p r i1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sprinter"  n ( s p r i1 n t @ r ) ((pos "K6%" ) ))
( "sprinters"  n ( s p r i1 n t @ z ) ((pos "Kj%" ) ))
( "sprinting"  v ( s p r i1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sprints"  v ( s p r i1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "sprit"  n ( s p r i1 t ) ((pos "K6$" ) ))
( "sprite"  n ( s p r ai1 t ) ((pos "K6%" ) ))
( "sprites"  n ( s p r ai1 t s ) ((pos "Kj%" ) ))
( "sprits"  n ( s p r i1 t s ) ((pos "Kj$" ) ))
( "spritsail"  n ( s p r i1 t s ei l ) ((pos "K6$" ) ))
( "spritsails"  n ( s p r i1 t s ei l z ) ((pos "Kj$" ) ))
( "sprocket"  n ( s p r o1 k i t ) ((pos "K6%" ) ))
( "sprocket-wheel"  n ( s p r o1 k i t - w ii l ) ((pos "K6%" ) ))
( "sprocket-wheels"  n ( s p r o1 k i t - w ii l z ) ((pos "Kj%" ) ))
( "sprockets"  n ( s p r o1 k i t s ) ((pos "Kj%" ) ))
( "sprout"  v ( s p r au1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "sprouted"  v ( s p r au1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "sprouting"  v ( s p r au1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "sprouts"  v ( s p r au1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "spruce"  v ( s p r uu1 s ) ((pos "J2%" "M6%" "OA%" ) (subcat "2C" "6A" )))
( "spruced"  v ( s p r uu1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "sprucely"  a ( s p r uu1 s l ii ) ((pos "Pu%" ) ))
( "spruceness"  n ( s p r uu1 s n @ s ) ((pos "L@%" ) ))
( "spruces"  v ( s p r uu1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "sprucing"  v ( s p r uu1 s i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "sprung"  v ( s p r uh1 ng ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "spry"  j ( s p r ai1 ) ((pos "OC%" ) ))
( "spryer"  j ( s p r ai1 @ r ) ((pos "Or$" ) ))
( "spryest"  j ( s p r ai1 i s t ) ((pos "Os$" ) ))
( "spud"  n ( s p uh1 d ) ((pos "K6%" ) ))
( "spuds"  n ( s p uh1 d z ) ((pos "Kj%" ) ))
( "spue"  v ( s p y uu1 ) ((pos "J2$" ) ))
( "spued"  v ( s p y uu1 d ) ((pos "Jc$" "Jd$" ) ))
( "spues"  v ( s p y uu1 z ) ((pos "Ja$" ) ))
( "spuing"  v ( s p y uu1 i ng ) ((pos "Jb$" ) ))
( "spume"  n ( s p y uu1 m ) ((pos "L@$" ) ))
( "spun"  v ( s p uh1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "spunk"  n ( s p uh1 ng k ) ((pos "L@%" ) ))
( "spunkier"  j ( s p uh1 ng k i@ r ) ((pos "Or$" ) ))
( "spunkiest"  j ( s p uh1 ng k i i s t ) ((pos "Os$" ) ))
( "spunky"  j ( s p uh1 ng k ii ) ((pos "OD%" ) ))
( "spur"  v ( s p @@1 r ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" )))
( "spurious"  j ( s p y u@1 r i@ s ) ((pos "OA%" ) ))
( "spuriously"  a ( s p y u@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "spuriousness"  n ( s p y u@1 r i@ s n @ s ) ((pos "L@%" ) ))
( "spurn"  v ( s p @@1 n ) ((pos "H0%" ) (subcat )))
( "spurned"  v ( s p @@1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "spurning"  v ( s p @@1 n i ng ) ((pos "Hb%" ) (subcat )))
( "spurns"  v ( s p @@1 n z ) ((pos "Ha%" ) (subcat )))
( "spurred"  v ( s p @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "spurring"  v ( s p @@1 r i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "spurs"  v ( s p @@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "spurt"  v ( s p @@1 t ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "spurted"  v ( s p @@1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "spurting"  v ( s p @@1 t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "spurts"  v ( s p @@1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "sputnik"  n ( s p u1 t n i k ) ((pos "K6%" ) ))
( "sputniks"  n ( s p u1 t n i k s ) ((pos "Kj$" ) ))
( "sputter"  v ( s p uh1 t @ r ) ((pos "J0%" ) (subcat "2A" )))
( "sputtered"  v ( s p uh1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sputtering"  v ( s p uh1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sputters"  v ( s p uh1 t @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "sputum"  n ( s p y uu1 t @ m ) ((pos "L@$" ) ))
( "spy"  v ( s p ai1 ) ((pos "J3%" "K8%" ) (subcat "2A" "3A" "6A" "15B" )))
( "spy-hole"  n ( s p ai1 - h ou l ) ((pos "K6%" ) ))
( "spy-holes"  n ( s p ai1 - h ou l z ) ((pos "Kj%" ) ))
( "spyglass"  n ( s p ai1 g l aa s ) ((pos "K7%" ) ))
( "spyglasses"  n ( s p ai1 g l aa s i z ) ((pos "Kj%" ) ))
( "spying"  v ( s p ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "15B" )))
( "squab"  n ( s k w o1 b ) ((pos "K6$" ) ))
( "squabble"  v ( s k w o1 b l ) ((pos "I2%" "K6%" ) (subcat )))
( "squabbled"  v ( s k w o1 b l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "squabbles"  v ( s k w o1 b l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "squabbling"  v ( s k w o1 b l i ng ) ((pos "Ib%" ) (subcat )))
( "squabs"  n ( s k w o1 b z ) ((pos "Kj$" ) ))
( "squad"  n ( s k w o1 d ) ((pos "K6%" ) ))
( "squadron"  n ( s k w o1 d r @ n ) ((pos "K6%" ) ))
( "squadrons"  n ( s k w o1 d r @ n z ) ((pos "Kj%" ) ))
( "squads"  n ( s k w o1 d z ) ((pos "Kj%" ) ))
( "squalid"  j ( s k w o1 l i d ) ((pos "OA%" ) ))
( "squalidly"  a ( s k w o1 l i d l ii ) ((pos "Pu%" ) ))
( "squall"  v ( s k w oo1 l ) ((pos "I0$" "K6%" ) (subcat )))
( "squalled"  v ( s k w oo1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "squalling"  v ( s k w oo1 l i ng ) ((pos "Ib$" ) (subcat )))
( "squalls"  v ( s k w oo1 l z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "squally"  j ( s k w oo1 l ii ) ((pos "OA%" ) ))
( "squalor"  n ( s k w o1 l @ r ) ((pos "L@%" ) ))
( "squander"  v ( s k w o1 n d @ r ) ((pos "H0%" ) (subcat )))
( "squandered"  v ( s k w o1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "squandering"  v ( s k w o1 n d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "squandermania"  n ( s k w o2 n d @ m ei1 n i@ ) ((pos "L@$" ) ))
( "squanders"  v ( s k w o1 n d @ z ) ((pos "Ha%" ) (subcat )))
( "square"  v ( s k w e@1 r ) ((pos "J2%" "K6%" "OA%" "Pu%" ) (subcat "2C" "3A" "6A" "14" )))
( "square-bashing"  n ( s k w e@1 - b a sh i ng ) ((pos "L@%" ) ))
( "square-built"  j ( s k w e@1 - b i l t ) ((pos "OA%" ) ))
( "square-rigged"  j ( s k w e@1 - r i1 g d ) ((pos "OA%" ) ))
( "square-shouldered"  j ( s k w e@1 - sh ou1 l d @ d ) ((pos "OA%" ) ))
( "square-toed"  j ( s k w e@1 - t ou d ) ((pos "OA%" ) ))
( "square-toes"  n ( s k w e@1 - t ou z ) ((pos "Ki%" ) ))
( "squared"  v ( s k w e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" )))
( "squarely"  a ( s k w e@1 l ii ) ((pos "Pu%" ) ))
( "squareness"  n ( s k w e@1 n @ s ) ((pos "L@%" ) ))
( "squares"  v ( s k w e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" )))
( "squaring"  v ( s k w e@1 r i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" )))
( "squash"  v ( s k w o1 sh ) ((pos "J1%" "M9%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "squash-rackets"  n ( s k w o1 sh - r a k i t s ) ((pos "Kj%" ) ))
( "squashed"  v ( s k w o1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "squashes"  v ( s k w o1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "squashier"  j ( s k w o1 sh i@ r ) ((pos "Or$" ) ))
( "squashiest"  j ( s k w o1 sh i i s t ) ((pos "Os$" ) ))
( "squashing"  v ( s k w o1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "squashy"  j ( s k w o1 sh ii ) ((pos "OD%" ) ))
( "squat"  v ( s k w o1 t ) ((pos "I4%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "squats"  v ( s k w o1 t s ) ((pos "Ia%" ) (subcat "2A" "2C" "6A" )))
( "squatted"  v ( s k w o1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" "6A" )))
( "squatter"  n ( s k w o1 t @ r ) ((pos "K6%" ) ))
( "squatters"  n ( s k w o1 t @ z ) ((pos "Kj%" ) ))
( "squatting"  v ( s k w o1 t i ng ) ((pos "Ib%" ) (subcat "2A" "2C" "6A" )))
( "squaw"  n ( s k w oo1 ) ((pos "K6%" ) ))
( "squawk"  v ( s k w oo1 k ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "squawked"  v ( s k w oo1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "squawker"  n ( s k w oo1 k @ r ) ((pos "K6$" ) ))
( "squawkers"  n ( s k w oo1 k @ z ) ((pos "Kj$" ) ))
( "squawking"  v ( s k w oo1 k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "squawks"  v ( s k w oo1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "squaws"  n ( s k w oo1 z ) ((pos "Kj%" ) ))
( "squeak"  v ( s k w ii1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "squeaked"  v ( s k w ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "squeaker"  n ( s k w ii1 k @ r ) ((pos "K6%" ) ))
( "squeakers"  n ( s k w ii1 k @ z ) ((pos "Kj%" ) ))
( "squeakier"  j ( s k w ii1 k i@ r ) ((pos "Or$" ) ))
( "squeakiest"  j ( s k w ii1 k i i s t ) ((pos "Os$" ) ))
( "squeaking"  v ( s k w ii1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "squeaks"  v ( s k w ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "squeaky"  j ( s k w ii1 k ii ) ((pos "OD%" ) ))
( "squeal"  v ( s k w ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "squealed"  v ( s k w ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "squealer"  n ( s k w ii1 l @ r ) ((pos "K6%" ) ))
( "squealers"  n ( s k w ii1 l @ z ) ((pos "Kj%" ) ))
( "squealing"  v ( s k w ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "squeals"  v ( s k w ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "squeamish"  j ( s k w ii1 m i sh ) ((pos "OA%" ) ))
( "squeamishly"  a ( s k w ii1 m i sh l ii ) ((pos "Pu%" ) ))
( "squeamishness"  n ( s k w ii1 m i sh n @ s ) ((pos "L@%" ) ))
( "squeegee"  v ( s k w ii2 jh ii1 ) ((pos "H5$" "K6%" ) (subcat )))
( "squeegeed"  v ( s k w ii2 jh ii1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "squeegeeing"  v ( s k w ii2 jh ii1 i ng ) ((pos "Hb$" ) (subcat )))
( "squeegees"  v ( s k w ii2 jh ii1 z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "squeeze"  v ( s k w ii1 z ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "squeezed"  v ( s k w ii1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "squeezer"  n ( s k w ii1 z @ r ) ((pos "K6%" ) ))
( "squeezers"  n ( s k w ii1 z @ z ) ((pos "Kj%" ) ))
( "squeezes"  v ( s k w ii1 z i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "squeezing"  v ( s k w ii1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" "15B" )))
( "squelch"  v ( s k w e1 l ch ) ((pos "J1%" "K7%" ) (subcat "2A" )))
( "squelched"  v ( s k w e1 l ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "squelches"  v ( s k w e1 l ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "squelching"  v ( s k w e1 l ch i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "squib"  n ( s k w i1 b ) ((pos "K6%" ) ))
( "squibs"  n ( s k w i1 b z ) ((pos "Kj%" ) ))
( "squid"  n ( s k w i1 d ) ((pos "K6%" ) ))
( "squids"  n ( s k w i1 d z ) ((pos "Kj%" ) ))
( "squiffy"  j ( s k w i1 f ii ) ((pos "OA$" ) ))
( "squiggle"  n ( s k w i1 g l ) ((pos "K6%" ) ))
( "squiggles"  n ( s k w i1 g l z ) ((pos "Kj%" ) ))
( "squigglier"  j ( s k w i1 g l i@ r ) ((pos "Or$" ) ))
( "squiggliest"  j ( s k w i1 g l i i s t ) ((pos "Os$" ) ))
( "squiggly"  j ( s k w i1 g l ii ) ((pos "OD%" ) ))
( "squint"  v ( s k w i1 n t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "squint-eyed"  j ( s k w i1 n t - ai d ) ((pos "OA%" ) ))
( "squinted"  v ( s k w i1 n t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "squinting"  v ( s k w i1 n t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "squints"  v ( s k w i1 n t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "squire"  v ( s k w ai1 @ r ) ((pos "H2$" "K6%" ) (subcat "6A" )))
( "squirearchies"  n ( s k w ai1 r aa k i z ) ((pos "Kj$" ) ))
( "squirearchy"  n ( s k w ai1 r aa k ii ) ((pos "K8%" ) ))
( "squired"  v ( s k w ai1 @ d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "squires"  v ( s k w ai1 @ z ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "squiring"  v ( s k w ai1 @ r i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "squirm"  v ( s k w @@1 m ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "squirmed"  v ( s k w @@1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "squirming"  v ( s k w @@1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "squirms"  v ( s k w @@1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "squirrel"  n ( s k w i1 r @ l ) ((pos "K6%" ) ))
( "squirrels"  n ( s k w i1 r @ l z ) ((pos "Kj%" ) ))
( "squirt"  v ( s k w @@1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "squirted"  v ( s k w @@1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "squirting"  v ( s k w @@1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "squirts"  v ( s k w @@1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "stab"  v ( s t a1 b ) ((pos "J4%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "stabbed"  v ( s t a1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "stabber"  n ( s t a1 b @ r ) ((pos "K6$" ) ))
( "stabbers"  n ( s t a1 b @ z ) ((pos "Kj$" ) ))
( "stabbing"  v ( s t a1 b i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "stability"  n ( s t @1 b i1 l i t ii ) ((pos "L@%" ) ))
( "stabilization"  n ( s t ei2 b @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "stabilizations"  n ( s t ei2 b @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "stabilize"  v ( s t ei1 b @ l ai z ) ((pos "H2%" ) (subcat )))
( "stabilized"  v ( s t ei1 b @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stabilizer"  n ( s t ei1 b @ l ai z @ r ) ((pos "K6%" ) ))
( "stabilizers"  n ( s t ei1 b @ l ai z @ z ) ((pos "Kj%" ) ))
( "stabilizes"  v ( s t ei1 b @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "stabilizing"  v ( s t ei1 b @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "stable"  v ( s t ei1 b l ) ((pos "H2%" "K6%" "OA%" ) ))
( "stable-companion"  n ( s t ei1 b l - k @ m p a2 n i@ n ) ((pos "K6%" ) ))
( "stable-companions"  n ( s t ei1 b l - k @ m p a2 n i@ n z ) ((pos "Kj%" ) ))
( "stableboy"  n ( s t ei1 b l b oi ) ((pos "K6%" ) ))
( "stableboys"  n ( s t ei1 b l b oi z ) ((pos "Kj%" ) ))
( "stabled"  v ( s t ei1 b l d ) ((pos "Hc%" "Hd%" ) ))
( "stableman"  n ( s t ei1 b l m a n ) ((pos "Ki%" ) ))
( "stablemate"  n ( s t ei1 b l m ei t ) ((pos "K6%" ) ))
( "stablemates"  n ( s t ei1 b l m ei t s ) ((pos "Kj%" ) ))
( "stablemen"  n ( s t ei1 b l m e n ) ((pos "Kj%" ) ))
( "stables"  v ( s t ei1 b l z ) ((pos "Ha%" "Kj%" ) ))
( "stabling"  v ( s t ei1 b l i ng ) ((pos "Hb%" "L@%" ) ))
( "stabs"  v ( s t a1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "staccato"  j ( s t @1 k aa1 t ou ) ((pos "OA%" "Pu%" ) ))
( "stack"  v ( s t a1 k ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "stacked"  v ( s t a1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "stacking"  v ( s t a1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stacks"  v ( s t a1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "stadium"  n ( s t ei1 d i@ m ) ((pos "K6%" ) ))
( "stadiums"  n ( s t ei1 d i@ m z ) ((pos "Kj%" ) ))
( "staff"  v ( s t aa1 f ) ((pos "H0%" "K6%" ) (subcat )))
( "staff-office"  n ( s t aa1 f - o f i s ) ((pos "K6$" ) ))
( "staff-offices"  n ( s t aa1 f - o f i s i z ) ((pos "Kj$" ) ))
( "staffed"  v ( s t aa1 f t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "staffing"  v ( s t aa1 f i ng ) ((pos "Hb%" ) (subcat )))
( "staffs"  v ( s t aa1 f s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "stag"  n ( s t a1 g ) ((pos "K6%" ) ))
( "stag-parties"  n ( s t a1 g - p aa t i z ) ((pos "Kj%" ) ))
( "stag-party"  n ( s t a1 g - p aa t ii ) ((pos "K8%" ) ))
( "stage"  v ( s t ei1 jh ) ((pos "J2%" "K6%" ) (subcat "2C" )))
( "stage-struck"  j ( s t ei1 jh - s t r uh k ) ((pos "OA%" ) ))
( "stage-whisper"  n ( s t ei1 jh - w i s p @ r ) ((pos "K6%" ) ))
( "stage-whispers"  n ( s t ei1 jh - w i s p @ z ) ((pos "Kj%" ) ))
( "stagecoach"  n ( s t ei1 jh k ou ch ) ((pos "K7%" ) ))
( "stagecoaches"  n ( s t ei1 jh k ou ch i z ) ((pos "Kj%" ) ))
( "stagecraft"  n ( s t ei1 jh k r aa f t ) ((pos "M6%" ) ))
( "stagecrafts"  n ( s t ei1 jh k r aa f t s ) ((pos "Mj$" ) ))
( "staged"  v ( s t ei1 jh d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "stager"  n ( s t ei1 jh @ r ) ((pos "K6%" ) ))
( "stagers"  n ( s t ei1 jh @ z ) ((pos "Kj%" ) ))
( "stages"  v ( s t ei1 jh i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "stagflation"  n ( s t a2 g f l ei1 sh @ n ) ((pos "L@$" ) ))
( "stagger"  v ( s t a1 g @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "staggered"  v ( s t a1 g @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "staggerer"  n ( s t a1 g @ r @ r ) ((pos "K6$" ) ))
( "staggerers"  n ( s t a1 g @ r @ z ) ((pos "Kj$" ) ))
( "staggering"  v ( s t a1 g @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "staggeringly"  a ( s t a1 g @ r i ng l ii ) ((pos "Pu%" ) ))
( "staggers"  v ( s t a1 g @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "stagily"  a ( s t ei1 jh i l ii ) ((pos "Pu%" ) ))
( "staginess"  n ( s t ei1 jh i n @ s ) ((pos "L@%" ) ))
( "staging"  v ( s t ei1 jh i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" )))
( "stagings"  n ( s t ei1 jh i ng z ) ((pos "Mj%" ) ))
( "stagnancy"  n ( s t a1 g n @ n s ii ) ((pos "L@$" ) ))
( "stagnant"  j ( s t a1 g n @ n t ) ((pos "OA%" ) ))
( "stagnate"  v ( s t a1 g n ei1 t ) ((pos "I2%" ) (subcat )))
( "stagnated"  v ( s t a1 g n ei1 t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "stagnates"  v ( s t a1 g n ei1 t s ) ((pos "Ia%" ) (subcat )))
( "stagnating"  v ( s t a1 g n ei1 t i ng ) ((pos "Ib%" ) (subcat )))
( "stagnation"  n ( s t a1 g n ei1 sh @ n ) ((pos "L@%" ) ))
( "stags"  n ( s t a1 g z ) ((pos "Kj%" ) ))
( "stagy"  j ( s t ei1 jh ii ) ((pos "OA%" ) ))
( "staid"  j ( s t ei1 d ) ((pos "OA%" ) ))
( "staidly"  a ( s t ei1 d l ii ) ((pos "Pu%" ) ))
( "staidness"  n ( s t ei1 d n @ s ) ((pos "L@%" ) ))
( "stain"  v ( s t ei1 n ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "stained"  v ( s t ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "staining"  v ( s t ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "stainless"  j ( s t ei1 n l @ s ) ((pos "OA%" ) ))
( "stains"  v ( s t ei1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "stair"  n ( s t e@1 r ) ((pos "K6%" ) ))
( "stair-carpet"  n ( s t e@1 - k aa p i t ) ((pos "K6%" ) ))
( "stair-carpets"  n ( s t e@1 - k aa p i t s ) ((pos "Kj%" ) ))
( "stair-rod"  n ( s t e@1 - r o d ) ((pos "K6%" ) ))
( "stair-rods"  n ( s t e@1 - r o d z ) ((pos "Kj%" ) ))
( "staircase"  n ( s t e@1 k ei s ) ((pos "K6%" ) ))
( "staircases"  n ( s t e@1 k ei s i z ) ((pos "Kj%" ) ))
( "stairs"  n ( s t e@1 z ) ((pos "Kj%" ) ))
( "stairway"  n ( s t e@1 w ei ) ((pos "K6%" ) ))
( "stairways"  n ( s t e@1 w ei z ) ((pos "Kj%" ) ))
( "stake"  v ( s t ei1 k ) ((pos "H2%" "K6%" ) (subcat "6A" "14" )))
( "stake-holder"  n ( s t ei1 k - h ou l d @ r ) ((pos "K6%" ) ))
( "stake-holders"  n ( s t ei1 k - h ou l d @ z ) ((pos "Kj%" ) ))
( "staked"  v ( s t ei1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "stakes"  v ( s t ei1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "staking"  v ( s t ei1 k i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "stalactite"  n ( s t a1 l @ k t ai t ) ((pos "K6%" ) ))
( "stalactites"  n ( s t a1 l @ k t ai t s ) ((pos "Kj%" ) ))
( "stalagmite"  n ( s t a1 l @ g m ai t ) ((pos "K6%" ) ))
( "stalagmites"  n ( s t a1 l @ g m ai t s ) ((pos "Kj%" ) ))
( "stale"  v ( s t ei1 l ) ((pos "I2$" "OB%" ) (subcat )))
( "staled"  v ( s t ei1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "stalemate"  v ( s t ei1 l m ei t ) ((pos "H2$" "M6%" ) (subcat )))
( "stalemated"  v ( s t ei1 l m ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "stalemates"  v ( s t ei1 l m ei t s ) ((pos "Ha$" "Mj$" ) (subcat )))
( "stalemating"  v ( s t ei1 l m ei t i ng ) ((pos "Hb$" ) (subcat )))
( "staleness"  n ( s t ei1 l n @ s ) ((pos "L@%" ) ))
( "staler"  j ( s t ei1 l @ r ) ((pos "Or%" ) ))
( "stales"  v ( s t ei1 l z ) ((pos "Ia$" ) (subcat )))
( "stalest"  j ( s t ei1 l i s t ) ((pos "Os%" ) ))
( "staling"  v ( s t ei1 l i ng ) ((pos "Ib$" ) (subcat )))
( "stalk"  v ( s t oo1 k ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "stalked"  v ( s t oo1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "stalker"  n ( s t oo1 k @ r ) ((pos "K6%" ) ))
( "stalkers"  n ( s t oo1 k @ z ) ((pos "Kj%" ) ))
( "stalking"  v ( s t oo1 k i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "stalking-horse"  n ( s t oo1 k i ng - h oo s ) ((pos "K6%" ) ))
( "stalking-horses"  n ( s t oo1 k i ng - h oo s i z ) ((pos "Kj%" ) ))
( "stalks"  v ( s t oo1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "stall"  v ( s t oo1 l ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "stall-fed"  j ( s t oo1 l - f e d ) ((pos "OA%" ) ))
( "stalled"  v ( s t oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "stalling"  v ( s t oo1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "stallion"  n ( s t a1 l i@ n ) ((pos "K6%" ) ))
( "stallions"  n ( s t a1 l i@ n z ) ((pos "Kj%" ) ))
( "stalls"  v ( s t oo1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "stalwart"  n ( s t oo1 l w @ t ) ((pos "K6%" "OA%" ) ))
( "stalwarts"  n ( s t oo1 l w @ t s ) ((pos "Kj%" ) ))
( "stamen"  n ( s t ei1 m @ n ) ((pos "K6$" ) ))
( "stamens"  n ( s t ei1 m @ n z ) ((pos "Kj$" ) ))
( "stamina"  n ( s t a1 m i n @ ) ((pos "L@%" ) ))
( "stammer"  v ( s t a1 m @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "stammered"  v ( s t a1 m @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "stammerer"  n ( s t a1 m @ r @ r ) ((pos "K6%" ) ))
( "stammerers"  n ( s t a1 m @ r @ z ) ((pos "Kj%" ) ))
( "stammering"  v ( s t a1 m @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "stammeringly"  a ( s t a1 m @ r i ng l ii ) ((pos "Pu%" ) ))
( "stammers"  v ( s t a1 m @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "stamp"  v ( s t a1 m p ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" "16B" )))
( "stamp-album"  n ( s t a1 m p - a l b @ m ) ((pos "K6%" ) ))
( "stamp-albums"  n ( s t a1 m p - a l b @ m z ) ((pos "Kj%" ) ))
( "stamp-collector"  n ( s t a1 m p - k @ l e2 k t @ r ) ((pos "K6%" ) ))
( "stamp-collectors"  n ( s t a1 m p - k @ l e2 k t @ z ) ((pos "Kj%" ) ))
( "stamp-dealer"  n ( s t a1 m p - d ii l @ r ) ((pos "K6%" ) ))
( "stamp-dealers"  n ( s t a1 m p - d ii l @ z ) ((pos "Kj%" ) ))
( "stamp-duties"  n ( s t a1 m p - d y uu t i z ) ((pos "Mj$" ) ))
( "stamp-duty"  n ( s t a1 m p - d y uu t ii ) ((pos "M8%" ) ))
( "stamped"  v ( s t a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" "16B" )))
( "stampede"  v ( s t a1 m p ii1 d ) ((pos "J2%" "K6%" ) (subcat "2A" "6A" )))
( "stampeded"  v ( s t a1 m p ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "stampedes"  v ( s t a1 m p ii1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "stampeding"  v ( s t a1 m p ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "stamping"  v ( s t a1 m p i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" "16B" )))
( "stamping-ground"  n ( s t a1 m p i ng - g r au n d ) ((pos "K6%" ) ))
( "stamping-grounds"  n ( s t a1 m p i ng - g r au n d z ) ((pos "Kj%" ) ))
( "stamps"  v ( s t a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" "15A" "15B" "16B" )))
( "stance"  n ( s t a1 n s ) ((pos "K6%" ) ))
( "stances"  n ( s t a1 n s i z ) ((pos "Kj$" ) ))
( "stanchion"  n ( s t a1 n ch @ n ) ((pos "K6$" ) ))
( "stanchions"  n ( s t a1 n ch @ n z ) ((pos "Kj$" ) ))
( "stand"  v ( s t a1 n d ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6C" "12A" "15A" )))
( "stand-in"  n ( s t a1 n d - i n ) ((pos "K6%" ) ))
( "stand-ins"  n ( s t a1 n d - i n z ) ((pos "Kj%" ) ))
( "stand-to"  n ( s t a1 n d - t uu1 ) ((pos "Ki$" ) ))
( "stand-up"  j ( s t a1 n d - uh p ) ((pos "OA%" ) ))
( "standard"  n ( s t a1 n d @ d ) ((pos "K6%" "OA%" ) ))
( "standard-bearer"  n ( s t a1 n d @ d - b e@ r @ r ) ((pos "K6%" ) ))
( "standard-bearers"  n ( s t a1 n d @ d - b e@ r @ z ) ((pos "Kj%" ) ))
( "standardization"  n ( s t a2 n d @ d ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "standardizations"  n ( s t a2 n d @ d ai z ei1 sh @ n z ) ((pos "Mj%" ) ))
( "standardize"  v ( s t a1 n d @ d ai z ) ((pos "H2%" ) (subcat )))
( "standardized"  v ( s t a1 n d @ d ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "standardizes"  v ( s t a1 n d @ d ai z i z ) ((pos "Ha%" ) (subcat )))
( "standardizing"  v ( s t a1 n d @ d ai z i ng ) ((pos "Hb%" ) (subcat )))
( "standards"  n ( s t a1 n d @ d z ) ((pos "Kj%" ) ))
( "standby"  n ( s t a1 n b ai ) ((pos "M6%" ) ))
( "standbys"  n ( s t a1 n b ai z ) ((pos "Mj%" ) ))
( "standing"  v ( s t a1 n d i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6C" "12A" "15A" )))
( "standoffish"  j ( s t a1 n d o1 f i sh ) ((pos "OA%" ) ))
( "standoffishly"  a ( s t a1 n d o1 f i sh l ii ) ((pos "Pu%" ) ))
( "standoffishness"  n ( s t a1 n d o1 f i sh n @ s ) ((pos "L@%" ) ))
( "standpipe"  n ( s t a1 n d p ai p ) ((pos "K6%" ) ))
( "standpipes"  n ( s t a1 n d p ai p s ) ((pos "Kj%" ) ))
( "standpoint"  n ( s t a1 n d p oi n t ) ((pos "K6%" ) ))
( "standpoints"  n ( s t a1 n d p oi n t s ) ((pos "Kj%" ) ))
( "stands"  v ( s t a1 n d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6C" "12A" "15A" )))
( "standstill"  n ( s t a1 n s t i l ) ((pos "Ki%" ) ))
( "stank"  v ( s t a1 ng k ) ((pos "Jc%" ) (subcat "2A" "3A" )))
( "stanza"  n ( s t a1 n z @ ) ((pos "K6%" ) ))
( "stanzas"  n ( s t a1 n z @ z ) ((pos "Kj%" ) ))
( "staple"  v ( s t ei1 p l ) ((pos "H2%" "M6%" ) (subcat )))
( "stapled"  v ( s t ei1 p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stapler"  n ( s t ei1 p l @ r ) ((pos "K6%" ) ))
( "staplers"  n ( s t ei1 p l @ z ) ((pos "Kj%" ) ))
( "staples"  v ( s t ei1 p l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "stapling"  v ( s t ei1 p l i ng ) ((pos "Hb%" ) (subcat )))
( "stapling-machine"  n ( s t ei1 p l i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "stapling-machines"  n ( s t ei1 p l i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "star"  v ( s t aa1 r ) ((pos "J4%" "K6%" ) (subcat "3A" "6A" )))
( "starboard"  v ( s t aa1 b @ d ) ((pos "H0$" "K6%" ) ))
( "starboarded"  v ( s t aa1 b @ d i d ) ((pos "Hc$" "Hd$" ) ))
( "starboarding"  v ( s t aa1 b @ d i ng ) ((pos "Hb$" ) ))
( "starboards"  v ( s t aa1 b @ d z ) ((pos "Ha$" "Kj$" ) ))
( "starch"  v ( s t aa1 ch ) ((pos "H1$" "L@%" ) (subcat )))
( "starched"  v ( s t aa1 ch t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "starches"  v ( s t aa1 ch i z ) ((pos "Ha$" ) (subcat )))
( "starchier"  j ( s t aa1 ch i@ r ) ((pos "Or$" ) ))
( "starchiest"  j ( s t aa1 ch i i s t ) ((pos "Os$" ) ))
( "starching"  v ( s t aa1 ch i ng ) ((pos "Hb$" ) (subcat )))
( "starchy"  j ( s t aa1 ch ii ) ((pos "OD%" ) ))
( "stardom"  n ( s t aa1 d @ m ) ((pos "L@%" ) ))
( "stardust"  n ( s t aa1 d uh s t ) ((pos "L@%" ) ))
( "stare"  v ( s t e@1 r ) ((pos "J2%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "stared"  v ( s t e@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" )))
( "stares"  v ( s t e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "starfish"  n ( s t aa1 f i sh ) ((pos "K9%" ) ))
( "starfishes"  n ( s t aa1 f i sh i z ) ((pos "Kj%" ) ))
( "stargazer"  n ( s t aa1 g ei z @ r ) ((pos "K6%" ) ))
( "stargazers"  n ( s t aa1 g ei z @ z ) ((pos "Kj%" ) ))
( "staring"  v ( s t e@1 r i ng ) ((pos "Jb%" "OA%" "Pu%" ) (subcat "2A" "2B" "2C" "3A" )))
( "stark"  j ( s t aa1 k ) ((pos "OA%" "Pu%" ) ))
( "starkers"  j ( s t aa1 k @ z ) ((pos "Op%" ) ))
( "starkly"  a ( s t aa1 k l ii ) ((pos "Pu%" ) ))
( "starless"  j ( s t aa1 l @ s ) ((pos "OA%" ) ))
( "starlet"  n ( s t aa1 l @ t ) ((pos "K6%" ) ))
( "starlets"  n ( s t aa1 l @ t s ) ((pos "Kj%" ) ))
( "starlight"  n ( s t aa1 l ai t ) ((pos "L@%" ) ))
( "starling"  n ( s t aa1 l i ng ) ((pos "K6%" ) ))
( "starlings"  n ( s t aa1 l i ng z ) ((pos "Kj%" ) ))
( "starlit"  j ( s t aa1 l i t ) ((pos "OA%" ) ))
( "starred"  v ( s t aa1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "starrier"  j ( s t aa1 r i@ r ) ((pos "Or%" ) ))
( "starriest"  j ( s t aa1 r i i s t ) ((pos "Os%" ) ))
( "starring"  v ( s t aa1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "starry"  j ( s t aa1 r ii ) ((pos "OD%" ) ))
( "starry-eyed"  j ( s t aa2 r i - ai1 d ) ((pos "OA%" ) ))
( "stars"  v ( s t aa1 z ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "start"  v ( s t aa1 t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "started"  v ( s t aa1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "starter"  n ( s t aa1 t @ r ) ((pos "K6%" ) ))
( "starters"  n ( s t aa1 t @ z ) ((pos "Kj%" ) ))
( "starting"  v ( s t aa1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "starting-gate"  n ( s t aa1 t i ng - g ei t ) ((pos "K6%" ) ))
( "starting-gates"  n ( s t aa1 t i ng - g ei t s ) ((pos "Kj%" ) ))
( "starting-point"  n ( s t aa1 t i ng - p oi n t ) ((pos "K6%" ) ))
( "starting-points"  n ( s t aa1 t i ng - p oi n t s ) ((pos "Kj%" ) ))
( "starting-post"  n ( s t aa1 t i ng - p ou s t ) ((pos "K6%" ) ))
( "starting-posts"  n ( s t aa1 t i ng - p ou s t s ) ((pos "Kj%" ) ))
( "starting-prices"  n ( s t aa1 t i ng - p r ai s i z ) ((pos "Kj%" ) ))
( "startle"  v ( s t aa1 t l ) ((pos "H2%" ) (subcat "6A" )))
( "startled"  v ( s t aa1 t l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "startles"  v ( s t aa1 t l z ) ((pos "Ha%" ) (subcat "6A" )))
( "startling"  v ( s t aa1 t l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "startlingly"  a ( s t aa1 t l i ng l ii ) ((pos "Pu%" ) ))
( "starts"  v ( s t aa1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "6D" "7A" "15A" )))
( "starvation"  n ( s t aa1 v ei1 sh @ n ) ((pos "L@%" ) ))
( "starve"  v ( s t aa1 v ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "starved"  v ( s t aa1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "starveling"  n ( s t aa1 v l i ng ) ((pos "K6$" ) ))
( "starvelings"  n ( s t aa1 v l i ng z ) ((pos "Kj$" ) ))
( "starves"  v ( s t aa1 v z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "starving"  v ( s t aa1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "stash"  v ( s t a1 sh ) ((pos "H1%" ) (subcat )))
( "stashed"  v ( s t a1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stashes"  v ( s t a1 sh i z ) ((pos "Ha%" ) (subcat )))
( "stashing"  v ( s t a1 sh i ng ) ((pos "Hb%" ) (subcat )))
( "state"  v ( s t ei1 t ) ((pos "H2%" "M6*" ) (subcat "6A" )))
( "statecraft"  n ( s t ei1 t k r aa f t ) ((pos "L@%" ) ))
( "stated"  v ( s t ei1 t i d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "stateless"  j ( s t ei1 t l @ s ) ((pos "OA%" ) ))
( "statelier"  j ( s t ei1 t l i@ r ) ((pos "Or%" ) ))
( "stateliest"  j ( s t ei1 t l i i s t ) ((pos "Os%" ) ))
( "stateliness"  n ( s t ei1 t l i n @ s ) ((pos "L@%" ) ))
( "stately"  j ( s t ei1 t l ii ) ((pos "OD%" ) ))
( "statement"  n ( s t ei1 t m @ n t ) ((pos "M6%" ) ))
( "statements"  n ( s t ei1 t m @ n t s ) ((pos "Mj%" ) ))
( "states"  v ( s t ei1 t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "statesman"  n ( s t ei1 t s m @ n ) ((pos "Ki%" ) ))
( "statesmanlike"  j ( s t ei1 t s m @ n l ai k ) ((pos "OA%" ) ))
( "statesmanship"  n ( s t ei1 t s m @ n sh i p ) ((pos "L@%" ) ))
( "statesmen"  n ( s t ei1 t s m @ n ) ((pos "Kj%" ) ))
( "static"  j ( s t a1 t i k ) ((pos "OA%" ) ))
( "statically"  a ( s t a1 t i k l ii ) ((pos "Pu%" ) ))
( "statics"  n ( s t a1 t i k s ) ((pos "Lk$" ) ))
( "stating"  v ( s t ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "station"  v ( s t ei1 sh @ n ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "station-waggon"  n ( s t ei1 sh @ n - w a g @ n ) ((pos "K6%" ) ))
( "station-waggons"  n ( s t ei1 sh @ n - w a g @ n z ) ((pos "Kj%" ) ))
( "stationary"  j ( s t ei1 sh @ n r ii ) ((pos "OA%" ) ))
( "stationed"  v ( s t ei1 sh @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "stationer"  n ( s t ei1 sh @ n @ r ) ((pos "K6%" ) ))
( "stationers"  n ( s t ei1 sh @ n @ z ) ((pos "Kj%" ) ))
( "stationery"  n ( s t ei1 sh @ n r ii ) ((pos "L@%" ) ))
( "stationing"  v ( s t ei1 sh @ n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stationmaster"  n ( s t ei1 sh @ n m aa s t @ r ) ((pos "K6%" ) ))
( "stationmasters"  n ( s t ei1 sh @ n m aa s t @ z ) ((pos "Kj%" ) ))
( "stations"  v ( s t ei1 sh @ n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "statistic"  n ( s t @1 t i1 s t i k ) ((pos "K6%" ) ))
( "statistical"  j ( s t @1 t i1 s t i k l ) ((pos "OA%" ) ))
( "statistically"  a ( s t @1 t i1 s t i k l ii ) ((pos "Pu%" ) ))
( "statistician"  n ( s t a2 t i s t i1 sh @ n ) ((pos "K6%" ) ))
( "statisticians"  n ( s t a2 t i s t i1 sh @ n z ) ((pos "Kj%" ) ))
( "statistics"  n ( s t @1 t i1 s t i k s ) ((pos "Kj%" "Lk%" ) ))
( "statuary"  n ( s t a1 ch u@ r ii ) ((pos "L@%" "OA$" ) ))
( "statue"  n ( s t a1 ch uu ) ((pos "K6%" ) ))
( "statues"  n ( s t a1 ch uu z ) ((pos "Kj%" ) ))
( "statuesque"  j ( s t a2 ch u e1 s k ) ((pos "OA%" ) ))
( "statuette"  n ( s t a2 ch u e1 t ) ((pos "K6%" ) ))
( "statuettes"  n ( s t a2 ch u e1 t s ) ((pos "Kj%" ) ))
( "stature"  n ( s t a1 ch @ r ) ((pos "L@%" ) ))
( "status"  n ( s t ei1 t @ s ) ((pos "L@%" ) ))
( "status quo"  n ( s t ei2 t @ s - k w ou1 ) ((pos "Ki%" ) ))
( "statute"  n ( s t a1 ch uu t ) ((pos "K6%" ) ))
( "statute-book"  n ( s t a1 ch uu t - b u k ) ((pos "K6%" ) ))
( "statute-books"  n ( s t a1 ch uu t - b u k s ) ((pos "Kj$" ) ))
( "statutes"  n ( s t a1 ch uu t s ) ((pos "Kj%" ) ))
( "statutorily"  a ( s t a1 ch u t r i l ii ) ((pos "Pu%" ) ))
( "statutory"  j ( s t a1 ch u t r ii ) ((pos "OA%" ) ))
( "staunch"  v ( s t oo1 n ch ) ((pos "H1%" "OA%" ) (subcat )))
( "staunched"  v ( s t oo1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "staunches"  v ( s t oo1 n ch i z ) ((pos "Ha%" ) (subcat )))
( "staunching"  v ( s t oo1 n ch i ng ) ((pos "Hb%" ) (subcat )))
( "staunchly"  a ( s t oo1 n ch l ii ) ((pos "Pu%" ) ))
( "staunchness"  n ( s t oo1 n ch n @ s ) ((pos "L@%" ) ))
( "stave"  v ( s t ei1 v ) ((pos "J2%" "K6$" ) (subcat "3A" )))
( "staved"  v ( s t ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "staves"  v ( s t ei1 v z ) ((pos "Ja%" "Kj$" ) (subcat "3A" )))
( "staving"  v ( s t ei1 v i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "stay"  v ( s t ei1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "4A" "6A" )))
( "stay-at-home"  n ( s t ei1 - @ t - h ou m ) ((pos "K6$" ) ))
( "stay-at-homes"  n ( s t ei1 - @ t - h ou m z ) ((pos "Kj$" ) ))
( "stayed"  v ( s t ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "4A" "6A" )))
( "stayer"  n ( s t ei1 @ r ) ((pos "K6%" ) ))
( "stayers"  n ( s t ei1 @ z ) ((pos "Kj%" ) ))
( "staying"  v ( s t ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "4A" "6A" )))
( "stays"  v ( s t ei1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "4A" "6A" )))
( "stead"  n ( s t e1 d ) ((pos "L@%" ) ))
( "steadfast"  j ( s t e1 d f aa s t ) ((pos "OA%" ) ))
( "steadfastly"  a ( s t e1 d f aa s t l ii ) ((pos "Pu%" ) ))
( "steadfastness"  n ( s t e1 d f aa s t n @ s ) ((pos "L@%" ) ))
( "steadied"  v ( s t e1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "steadier"  j ( s t e1 d i@ r ) ((pos "Or%" ) ))
( "steadies"  v ( s t e1 d i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "steadiest"  j ( s t e1 d i i s t ) ((pos "Os%" ) ))
( "steadily"  a ( s t e1 d i l ii ) ((pos "Pu%" ) ))
( "steadiness"  n ( s t e1 d i n @ s ) ((pos "L@%" ) ))
( "steady"  v ( s t e1 d ii ) ((pos "J3%" "K8%" "OD%" "Pu%" ) (subcat "2A" )))
( "steadying"  v ( s t e1 d i i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "steak"  n ( s t ei1 k ) ((pos "M6%" ) ))
( "steaks"  n ( s t ei1 k s ) ((pos "Mj%" ) ))
( "steal"  v ( s t ii1 l ) ((pos "J5%" ) (subcat "2A" "2C" "6A" "14" )))
( "stealing"  v ( s t ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" )))
( "steals"  v ( s t ii1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "14" )))
( "stealth"  n ( s t e1 l th ) ((pos "L@%" ) ))
( "stealthier"  j ( s t e1 l th i@ r ) ((pos "Or%" ) ))
( "stealthiest"  j ( s t e1 l th i i s t ) ((pos "Os%" ) ))
( "stealthily"  a ( s t e1 l th i l ii ) ((pos "Pu%" ) ))
( "stealthy"  j ( s t e1 l th ii ) ((pos "OD%" ) ))
( "steam"  v ( s t ii1 m ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "steam-boiler"  n ( s t ii1 m - b oi l @ r ) ((pos "K6%" ) ))
( "steam-boilers"  n ( s t ii1 m - b oi l @ z ) ((pos "Kj%" ) ))
( "steam-coal"  n ( s t ii1 m - k ou l ) ((pos "L@%" ) ))
( "steam-engine"  n ( s t ii1 m - e n jh i n ) ((pos "K6%" ) ))
( "steam-engines"  n ( s t ii1 m - e n jh i n z ) ((pos "Kj%" ) ))
( "steam-heat"  v ( s t ii1 m - h ii1 t ) ((pos "H0$" "L@$" ) ))
( "steam-heated"  v ( s t ii1 m - h ii1 t i d ) ((pos "Hc$" "Hd$" ) ))
( "steam-heating"  v ( s t ii1 m - h ii1 t i ng ) ((pos "Hb$" ) ))
( "steam-heats"  v ( s t ii1 m - h ii1 t s ) ((pos "Ha$" ) ))
( "steamboat"  n ( s t ii1 m b ou t ) ((pos "K6%" ) ))
( "steamboats"  n ( s t ii1 m b ou t s ) ((pos "Kj%" ) ))
( "steamed"  v ( s t ii1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "steamer"  n ( s t ii1 m @ r ) ((pos "K6%" ) ))
( "steamers"  n ( s t ii1 m @ z ) ((pos "Kj%" ) ))
( "steamier"  j ( s t ii1 m i@ r ) ((pos "Or%" ) ))
( "steamiest"  j ( s t ii1 m i i s t ) ((pos "Os%" ) ))
( "steaming"  v ( s t ii1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "steamroller"  v ( s t ii1 m r ou l @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "steamrollered"  v ( s t ii1 m r ou l @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "steamrollering"  v ( s t ii1 m r ou l @ r i ng ) ((pos "Hb$" ) (subcat )))
( "steamrollers"  v ( s t ii1 m r ou l @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "steams"  v ( s t ii1 m z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "steamship"  n ( s t ii1 m sh i p ) ((pos "K6%" ) ))
( "steamships"  n ( s t ii1 m sh i p s ) ((pos "Kj%" ) ))
( "steamy"  j ( s t ii1 m ii ) ((pos "OD%" ) ))
( "steed"  n ( s t ii1 d ) ((pos "K6$" ) ))
( "steeds"  n ( s t ii1 d z ) ((pos "Kj$" ) ))
( "steel"  v ( s t ii1 l ) ((pos "H0%" "L@%" ) (subcat "6A" "15A" )))
( "steel-clad"  j ( s t ii1 l - k l a d ) ((pos "OA%" ) ))
( "steel-plated"  j ( s t ii1 l - p l ei1 t i d ) ((pos "OA%" ) ))
( "steeled"  v ( s t ii1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "steelier"  j ( s t ii1 l i@ r ) ((pos "Or%" ) ))
( "steeliest"  j ( s t ii1 l i i s t ) ((pos "Os%" ) ))
( "steeling"  v ( s t ii1 l i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "steels"  v ( s t ii1 l z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "steelworks"  n ( s t ii1 l w @@ k s ) ((pos "K9%" ) ))
( "steely"  j ( s t ii1 l ii ) ((pos "OD%" ) ))
( "steelyard"  n ( s t ii1 l y aa d ) ((pos "K6%" ) ))
( "steelyards"  n ( s t ii1 l y aa d z ) ((pos "Kj%" ) ))
( "steenbok"  n ( s t ii1 n b o k ) ((pos "K6$" ) ))
( "steenboks"  n ( s t ii1 n b o k s ) ((pos "Kj$" ) ))
( "steep"  v ( s t ii1 p ) ((pos "J0%" "OC%" ) (subcat "2A" "6A" )))
( "steeped"  v ( s t ii1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "steepen"  v ( s t ii1 p @ n ) ((pos "J0%" ) (subcat "2A" )))
( "steepened"  v ( s t ii1 p @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "steepening"  v ( s t ii1 p @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "steepens"  v ( s t ii1 p @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "steeper"  j ( s t ii1 p @ r ) ((pos "Or%" ) ))
( "steepest"  j ( s t ii1 p i s t ) ((pos "Os%" ) ))
( "steeping"  v ( s t ii1 p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "steepish"  j ( s t ii1 p i sh ) ((pos "OA%" ) ))
( "steeple"  n ( s t ii1 p l ) ((pos "K6%" ) ))
( "steeplechase"  n ( s t ii1 p l ch ei s ) ((pos "K6%" ) ))
( "steeplechaser"  n ( s t ii1 p l ch ei s @ r ) ((pos "K6%" ) ))
( "steeplechasers"  n ( s t ii1 p l ch ei s @ z ) ((pos "Kj%" ) ))
( "steeplechases"  n ( s t ii1 p l ch ei s i z ) ((pos "Kj%" ) ))
( "steeplejack"  n ( s t ii1 p l jh a k ) ((pos "K6%" ) ))
( "steeplejacks"  n ( s t ii1 p l jh a k s ) ((pos "Kj%" ) ))
( "steeples"  n ( s t ii1 p l z ) ((pos "Kj%" ) ))
( "steeply"  a ( s t ii1 p l ii ) ((pos "Pu%" ) ))
( "steepness"  n ( s t ii1 p n @ s ) ((pos "L@%" ) ))
( "steeps"  v ( s t ii1 p s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "steer"  v ( s t i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "steerage"  n ( s t i@1 r i jh ) ((pos "M6%" ) ))
( "steerages"  n ( s t i@1 r i jh i z ) ((pos "Mj$" ) ))
( "steerageway"  n ( s t i@1 r i jh w ei ) ((pos "L@$" ) ))
( "steered"  v ( s t i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "steering"  v ( s t i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "steering-gear"  n ( s t i@1 r i ng - g i@ r ) ((pos "L@%" ) ))
( "steering-wheel"  n ( s t i@1 r i ng - w ii l ) ((pos "K6%" ) ))
( "steering-wheels"  n ( s t i@1 r i ng - w ii l z ) ((pos "Kj%" ) ))
( "steers"  v ( s t i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "steersman"  n ( s t i@1 z m @ n ) ((pos "Ki%" ) ))
( "steersmen"  n ( s t i@1 z m @ n ) ((pos "Kj%" ) ))
( "stelae"  n ( s t ii1 l ii ) ((pos "Kj$" ) ))
( "stele"  n ( s t ii1 l ii ) ((pos "Ki$" ) ))
( "stellar"  j ( s t e1 l @ r ) ((pos "OA%" ) ))
( "stem"  v ( s t e1 m ) ((pos "J4%" "K6%" ) (subcat "3A" )))
( "stemmed"  v ( s t e1 m d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "3A" )))
( "stemming"  v ( s t e1 m i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "stems"  v ( s t e1 m z ) ((pos "Ja%" "Kj%" ) (subcat "3A" )))
( "stench"  n ( s t e1 n ch ) ((pos "K7%" ) ))
( "stenches"  n ( s t e1 n ch i z ) ((pos "Kj$" ) ))
( "stencil"  v ( s t e1 n s l ) ((pos "H4%" "K6%" ) (subcat )))
( "stencilled"  v ( s t e1 n s l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stencilling"  v ( s t e1 n s @ l i ng ) ((pos "Hb%" ) (subcat )))
( "stencils"  v ( s t e1 n s l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "stenographer"  n ( s t @1 n o1 g r @ f @ r ) ((pos "K6%" ) ))
( "stenographers"  n ( s t @1 n o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "stenography"  n ( s t @1 n o1 g r @ f ii ) ((pos "L@%" ) ))
( "stentorian"  j ( s t e1 n t oo1 r i@ n ) ((pos "OA%" ) ))
( "step"  v ( s t e1 p ) ((pos "J4%" "K6%" ) (subcat "2C" )))
( "step-"  j ( s t e p - ) ((pos "U-%" ) ))
( "step-ins"  n ( s t e1 p - i n z ) ((pos "Kj$" ) ))
( "stepbrother"  n ( s t e1 p b r uh dh @ r ) ((pos "K6%" ) ))
( "stepbrothers"  n ( s t e1 p b r uh dh @ z ) ((pos "Kj%" ) ))
( "stepchild"  n ( s t e1 p ch ai l d ) ((pos "Ki%" ) ))
( "stepchildren"  n ( s t e1 p ch i l d r @ n ) ((pos "Kj%" ) ))
( "stepdaughter"  n ( s t e1 p d oo t @ r ) ((pos "K6%" ) ))
( "stepdaughters"  n ( s t e1 p d oo t @ z ) ((pos "Kj%" ) ))
( "stepfather"  n ( s t e1 p f aa dh @ r ) ((pos "K6%" ) ))
( "stepfathers"  n ( s t e1 p f aa dh @ z ) ((pos "Kj%" ) ))
( "stepladder"  n ( s t e1 p l a d @ r ) ((pos "K6%" ) ))
( "stepladders"  n ( s t e1 p l a d @ z ) ((pos "Kj%" ) ))
( "stepmother"  n ( s t e1 p m uh dh @ r ) ((pos "K6%" ) ))
( "stepmothers"  n ( s t e1 p m uh dh @ z ) ((pos "Kj%" ) ))
( "stepparent"  n ( s t e1 p - p e@ r @ n t ) ((pos "K6$" ) ))
( "stepparents"  n ( s t e1 p - p e@ r @ n t s ) ((pos "Kj$" ) ))
( "steppe"  n ( s t e1 p ) ((pos "K6$" ) ))
( "stepped"  v ( s t e1 p t ) ((pos "Jc$" "Jd%" ) (subcat "2C" )))
( "steppes"  n ( s t e1 p s ) ((pos "Kj$" ) ))
( "stepping"  v ( s t e1 p i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "stepping-stone"  n ( s t e1 p i ng - s t ou n ) ((pos "K6%" ) ))
( "stepping-stones"  n ( s t e1 p i ng - s t ou n z ) ((pos "Kj%" ) ))
( "steps"  v ( s t e1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "stepsister"  n ( s t e1 p s i s t @ r ) ((pos "K6%" ) ))
( "stepsisters"  n ( s t e1 p s i s t @ z ) ((pos "Kj%" ) ))
( "stepson"  n ( s t e1 p s uh n ) ((pos "K6%" ) ))
( "stepsons"  n ( s t e1 p s uh n z ) ((pos "Kj%" ) ))
( "stereo"  n ( s t e1 r i@ @ ) ((pos "K6%" ) ))
( "stereophonic"  j ( s t e2 r i@ f o1 n i k ) ((pos "OA%" ) ))
( "stereos"  n ( s t e1 r i@ u z ) ((pos "Kj%" ) ))
( "stereoscope"  n ( s t e1 r i@ s k ou p ) ((pos "K6$" ) ))
( "stereoscopes"  n ( s t e1 r i@ s k ou p s ) ((pos "Kj$" ) ))
( "stereoscopic"  j ( s t e2 r i@ s k o1 p i k ) ((pos "OA%" ) ))
( "stereotype"  v ( s t e1 r i@ t ai p ) ((pos "H2$" "M6%" ) (subcat )))
( "stereotyped"  v ( s t e1 r i@ t ai p t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "stereotypes"  v ( s t e1 r i@ t ai p s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "stereotyping"  v ( s t e1 r i@ t ai p i ng ) ((pos "Hb$" ) (subcat )))
( "sterile"  j ( s t e1 r ai l ) ((pos "OA%" ) ))
( "sterility"  n ( s t @1 r i1 l i t ii ) ((pos "L@%" ) ))
( "sterilization"  n ( s t e2 r @ l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "sterilize"  v ( s t e1 r @ l ai z ) ((pos "H2%" ) (subcat )))
( "sterilized"  v ( s t e1 r @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sterilizes"  v ( s t e1 r @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "sterilizing"  v ( s t e1 r @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "sterling"  n ( s t @@1 l i ng ) ((pos "L@%" "OA%" ) ))
( "stern"  n ( s t @@1 n ) ((pos "K6%" "OC%" ) ))
( "sterner"  j ( s t @@1 n @ r ) ((pos "Or%" ) ))
( "sternest"  j ( s t @@1 n i s t ) ((pos "Os%" ) ))
( "sternly"  a ( s t @@1 n l ii ) ((pos "Pu%" ) ))
( "sternness"  n ( s t @@1 n - n @ s ) ((pos "L@%" ) ))
( "sterns"  n ( s t @@1 n z ) ((pos "Kj$" ) ))
( "sternum"  n ( s t @@1 n @ m ) ((pos "K6%" ) ))
( "sternums"  n ( s t @@1 n @ m z ) ((pos "Kj$" ) ))
( "sternwheeler"  n ( s t @@1 n w ii1 l @ r ) ((pos "K6$" ) ))
( "sternwheelers"  n ( s t @@1 n w ii1 l @ z ) ((pos "Kj$" ) ))
( "stertorous"  j ( s t @@1 t @ r @ s ) ((pos "OA$" ) ))
( "stertorously"  a ( s t @@1 t @ r @ s l ii ) ((pos "Pu$" ) ))
( "stet"  nil ( s t e1 t ) ((pos "Z-$" ) ))
( "stethoscope"  n ( s t e1 th @ s k ou p ) ((pos "K6%" ) ))
( "stethoscopes"  n ( s t e1 th @ s k ou p s ) ((pos "Kj%" ) ))
( "stetson"  n ( s t e1 t s n ) ((pos "K6%" ) ))
( "stetsons"  n ( s t e1 t s n z ) ((pos "Kj%" ) ))
( "stevedore"  n ( s t ii1 v @ d oo r ) ((pos "K6%" ) ))
( "stevedores"  n ( s t ii1 v @ d oo z ) ((pos "Kj%" ) ))
( "stew"  v ( s t y uu1 ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "steward"  n ( s t y u@1 d ) ((pos "K6%" ) ))
( "stewardess"  n ( s t y u@2 d e1 s ) ((pos "K7%" ) ))
( "stewardesses"  n ( s t y u@2 d e1 s i z ) ((pos "Kj%" ) ))
( "stewards"  n ( s t y u@1 d z ) ((pos "Kj%" ) ))
( "stewardship"  n ( s t y uu1 @ d sh i p ) ((pos "L@%" ) ))
( "stewed"  v ( s t y uu1 d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "stewing"  v ( s t y uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "stews"  v ( s t y uu1 z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "stick"  v ( s t i1 k ) ((pos "H0%" "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "stick-in-the-mud"  n ( s t i1 k - i n - dh @ - m uh d ) ((pos "Ki%" "Oq%" ) ))
( "stick-on"  j ( s t i1 k - o n ) ((pos "Oq%" ) ))
( "stick-up"  n ( s t i1 k - uh p ) ((pos "K6%" ) ))
( "stick-ups"  n ( s t i1 k - uh p s ) ((pos "Kj$" ) ))
( "sticked"  v ( s t i1 k t ) ((pos "Hc$" "Hd$" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "sticker"  n ( s t i1 k @ r ) ((pos "K6%" ) ))
( "stickers"  n ( s t i1 k @ z ) ((pos "Kj%" ) ))
( "stickier"  j ( s t i1 k i@ r ) ((pos "Or%" ) ))
( "stickiest"  j ( s t i1 k i i s t ) ((pos "Os%" ) ))
( "stickily"  a ( s t i1 k i l ii ) ((pos "Pu%" ) ))
( "stickiness"  n ( s t i1 k i n @ s ) ((pos "L@%" ) ))
( "sticking"  v ( s t i1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "sticking-plaster"  n ( s t i1 k i ng - p l aa s t @ r ) ((pos "M6%" ) ))
( "sticking-plasters"  n ( s t i1 k i ng - p l aa s t @ z ) ((pos "Mj%" ) ))
( "stickler"  n ( s t i1 k l @ r ) ((pos "K6%" ) ))
( "sticklers"  n ( s t i1 k l @ z ) ((pos "Kj%" ) ))
( "sticks"  v ( s t i1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "sticky"  j ( s t i1 k ii ) ((pos "OD%" ) ))
( "sties"  n ( s t ai1 z ) ((pos "Kj%" ) ))
( "stiff"  n ( s t i1 f ) ((pos "K6%" "OC%" "Pu%" ) ))
( "stiff-necked"  j ( s t i1 f - n e1 k t ) ((pos "OA%" ) ))
( "stiffen"  v ( s t i1 f n ) ((pos "J0%" ) (subcat "2A" )))
( "stiffened"  v ( s t i1 f n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "stiffener"  n ( s t i1 f n @ r ) ((pos "K6%" ) ))
( "stiffeners"  n ( s t i1 f n @ z ) ((pos "Kj%" ) ))
( "stiffening"  v ( s t i1 f n i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "stiffenings"  n ( s t i1 f n i ng z ) ((pos "Mj%" ) ))
( "stiffens"  v ( s t i1 f n z ) ((pos "Ja%" ) (subcat "2A" )))
( "stiffer"  j ( s t i1 f @ r ) ((pos "Or%" ) ))
( "stiffest"  j ( s t i1 f i s t ) ((pos "Os%" ) ))
( "stiffly"  a ( s t i1 f l ii ) ((pos "Pu%" ) ))
( "stiffness"  n ( s t i1 f n @ s ) ((pos "L@%" ) ))
( "stiffs"  n ( s t i1 f s ) ((pos "Kj$" ) ))
( "stifle"  v ( s t ai1 f @ l ) ((pos "J2%" ) (subcat "2A" )))
( "stifled"  v ( s t ai1 f @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "stifles"  v ( s t ai1 f @ l z ) ((pos "Ja%" ) (subcat "2A" )))
( "stifling"  v ( s t ai1 f l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "stigma"  n ( s t i1 g m @ ) ((pos "K6%" ) ))
( "stigmas"  n ( s t i1 g m @ z ) ((pos "Kj$" ) ))
( "stigmatize"  v ( s t i1 g m @ t ai z ) ((pos "H2%" ) (subcat )))
( "stigmatized"  v ( s t i1 g m @ t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stigmatizes"  v ( s t i1 g m @ t ai z i z ) ((pos "Ha%" ) (subcat )))
( "stigmatizing"  v ( s t i1 g m @ t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "stile"  n ( s t ai1 l ) ((pos "K6%" ) ))
( "stiles"  n ( s t ai1 l z ) ((pos "Kj%" ) ))
( "stiletto"  n ( s t i1 l e1 t ou ) ((pos "K6%" ) ))
( "stilettoes"  n ( s t i1 l e1 t ou z ) ((pos "Kj%" ) ))
( "stilettos"  n ( s t i1 l e1 t ou z ) ((pos "Kj%" ) ))
( "still"  v ( s t i1 l ) ((pos "H0%" "K6%" "OC*" "Pu*" ) (subcat )))
( "still-life"  n ( s t i1 l - l ai1 f ) ((pos "M6%" ) ))
( "still-lifes"  n ( s t i1 l - l ai1 f s ) ((pos "Mj%" ) ))
( "still-room"  n ( s t i1 l - r uu m ) ((pos "K6$" ) ))
( "still-rooms"  n ( s t i1 l - r uu m z ) ((pos "Kj$" ) ))
( "stillbirth"  n ( s t i1 l b @@ th ) ((pos "K6%" ) ))
( "stillbirths"  n ( s t i1 l b @@ th s ) ((pos "Kj%" ) ))
( "stillborn"  j ( s t i1 l b oo n ) ((pos "OA%" ) ))
( "stilled"  v ( s t i1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stiller"  j ( s t i1 l @ r ) ((pos "Or$" ) ))
( "stillest"  j ( s t i1 l i s t ) ((pos "Os$" ) ))
( "stilling"  v ( s t i1 l i ng ) ((pos "Hb%" ) (subcat )))
( "stillness"  n ( s t i1 l n @ s ) ((pos "L@%" ) ))
( "stills"  v ( s t i1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "stilly"  j ( s t i1 l ii ) ((pos "OA$" ) ))
( "stilt"  n ( s t i1 l t ) ((pos "K6%" ) ))
( "stilted"  j ( s t i1 l t i d ) ((pos "OA%" ) ))
( "stiltedly"  a ( s t i1 l t i d l ii ) ((pos "Pu%" ) ))
( "stilts"  n ( s t i1 l t s ) ((pos "Kj%" ) ))
( "stimulant"  n ( s t i1 m y u l @ n t ) ((pos "K6%" ) ))
( "stimulants"  n ( s t i1 m y u l @ n t s ) ((pos "Kj%" ) ))
( "stimulate"  v ( s t i1 m y u l ei t ) ((pos "H2%" ) (subcat "6A" "14" )))
( "stimulated"  v ( s t i1 m y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "stimulates"  v ( s t i1 m y u l ei t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "stimulating"  v ( s t i1 m y u l ei t i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" "14" )))
( "stimulation"  n ( s t i2 m y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "stimulations"  n ( s t i2 m y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "stimuli"  n ( s t i1 m y u l ai ) ((pos "Kj%" ) ))
( "stimulus"  n ( s t i1 m y u l @ s ) ((pos "Ki%" ) ))
( "sting"  v ( s t i1 ng ) ((pos "J5%" "M6%" ) (subcat "2A" "6A" "14" )))
( "stinger"  n ( s t i1 ng @ r ) ((pos "K6%" ) ))
( "stingers"  n ( s t i1 ng @ z ) ((pos "Kj%" ) ))
( "stingier"  j ( s t i1 n jh i@ r ) ((pos "Or%" ) ))
( "stingiest"  j ( s t i1 n jh i i s t ) ((pos "Os%" ) ))
( "stingily"  a ( s t i1 n jh i l ii ) ((pos "Pu%" ) ))
( "stinginess"  n ( s t i1 ng jh i n @ s ) ((pos "L@%" ) ))
( "stinging"  v ( s t i1 ng i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "stingless"  j ( s t i1 ng l @ s ) ((pos "OA$" ) ))
( "stingray"  n ( s t i1 ng r ei ) ((pos "K6%" ) ))
( "stingrays"  n ( s t i1 ng r ei z ) ((pos "Kj%" ) ))
( "stings"  v ( s t i1 ng z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" "14" )))
( "stingy"  j ( s t i1 n jh ii ) ((pos "OD%" ) ))
( "stink"  v ( s t i1 ng k ) ((pos "J5%" "K6%" ) (subcat "2A" "3A" )))
( "stinker"  n ( s t i1 ng k @ r ) ((pos "K6%" ) ))
( "stinkers"  n ( s t i1 ng k @ z ) ((pos "Kj%" ) ))
( "stinking"  v ( s t i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "stinks"  v ( s t i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "stint"  v ( s t i1 n t ) ((pos "J0$" "K6%" ) (subcat "6A" )))
( "stinted"  v ( s t i1 n t i d ) ((pos "Jc$" "Jd$" ) (subcat "6A" )))
( "stinting"  v ( s t i1 n t i ng ) ((pos "Jb$" ) (subcat "6A" )))
( "stints"  v ( s t i1 n t s ) ((pos "Ja$" "Kj%" ) (subcat "6A" )))
( "stipend"  n ( s t ai1 p e n d ) ((pos "K6%" ) ))
( "stipendiaries"  n ( s t ai1 p e1 n d i@ r i z ) ((pos "Kj$" ) ))
( "stipendiary"  n ( s t ai1 p e1 n d i@ r ii ) ((pos "K8$" "OA$" ) ))
( "stipends"  n ( s t ai1 p e n d z ) ((pos "Kj$" ) ))
( "stipple"  v ( s t i1 p l ) ((pos "H2$" ) (subcat )))
( "stippled"  v ( s t i1 p l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "stipples"  v ( s t i1 p l z ) ((pos "Ha$" ) (subcat )))
( "stippling"  v ( s t i1 p l i ng ) ((pos "Hb$" ) (subcat )))
( "stipulate"  v ( s t i1 p y u l ei t ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "stipulated"  v ( s t i1 p y u l ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "stipulates"  v ( s t i1 p y u l ei t s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "stipulating"  v ( s t i1 p y u l ei t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "stipulation"  n ( s t i2 p y u l ei1 sh @ n ) ((pos "K6%" ) ))
( "stipulations"  n ( s t i2 p y u l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "stir"  v ( s t @@1 r ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "14" )))
( "stirred"  v ( s t @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "stirring"  v ( s t @@1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "6A" "14" )))
( "stirringly"  a ( s t @@1 r i ng l ii ) ((pos "Pu%" ) ))
( "stirrup"  n ( s t i1 r @ p ) ((pos "K6%" ) ))
( "stirrup-cup"  n ( s t i1 r @ p - k uh p ) ((pos "K6%" ) ))
( "stirrup-cups"  n ( s t i1 r @ p - k uh p s ) ((pos "Kj$" ) ))
( "stirrups"  n ( s t i1 r @ p s ) ((pos "Kj%" ) ))
( "stirs"  v ( s t @@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "stitch"  v ( s t i1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "15A" )))
( "stitched"  v ( s t i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "stitches"  v ( s t i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "stitching"  v ( s t i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "stoat"  n ( s t ou1 t ) ((pos "K6%" ) ))
( "stoats"  n ( s t ou1 t s ) ((pos "Kj%" ) ))
( "stochastic"  j ( s t @1 k a1 s t i k ) ((pos "OA$" ) ))
( "stochastically"  a ( s t @1 k a1 s t i k l ii ) ((pos "Pu$" ) ))
( "stock"  v ( s t o1 k ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "stock-cube"  n ( s t o1 k - k y uu b ) ((pos "K6%" ) ))
( "stock-cubes"  n ( s t o1 k - k y uu b z ) ((pos "Kj%" ) ))
( "stock-farmer"  n ( s t o1 k - f aa m @ r ) ((pos "K6%" ) ))
( "stock-farmers"  n ( s t o1 k - f aa m @ z ) ((pos "Kj%" ) ))
( "stock-in-trade"  n ( s t o2 k - i n - t r ei1 d ) ((pos "M6%" ) ))
( "stock-in-trades"  n ( s t o2 k - i n - t r ei1 d z ) ((pos "Mj$" ) ))
( "stock-list"  n ( s t o1 k - l i s t ) ((pos "K6%" ) ))
( "stock-lists"  n ( s t o1 k - l i s t s ) ((pos "Kj%" ) ))
( "stock-still"  a ( s t o2 k - s t i1 l ) ((pos "Pu%" ) ))
( "stockade"  v ( s t o1 k ei1 d ) ((pos "H2$" "K6%" ) ))
( "stockaded"  v ( s t o1 k ei1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "stockades"  v ( s t o1 k ei1 d z ) ((pos "Ha$" "Kj%" ) ))
( "stockading"  v ( s t o1 k ei1 d i ng ) ((pos "Hb$" ) ))
( "stockbreeder"  n ( s t o1 k b r ii d @ r ) ((pos "K6%" ) ))
( "stockbreeders"  n ( s t o1 k b r ii d @ z ) ((pos "Kj%" ) ))
( "stockbroker"  n ( s t o1 k b r ou k @ r ) ((pos "K6%" ) ))
( "stockbrokers"  n ( s t o1 k b r ou k @ z ) ((pos "Kj%" ) ))
( "stockcar"  n ( s t o1 k - k aa r ) ((pos "K6%" ) ))
( "stockcars"  n ( s t o1 k - k aa z ) ((pos "Kj%" ) ))
( "stocked"  v ( s t o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "stockfish"  n ( s t o1 k f i sh ) ((pos "K9%" ) ))
( "stockholder"  n ( s t o1 k h ou l d @ r ) ((pos "K6%" ) ))
( "stockholders"  n ( s t o1 k h ou l d @ z ) ((pos "Kj%" ) ))
( "stockholding"  n ( s t o1 k h ou l d i ng ) ((pos "K6%" ) ))
( "stockholdings"  n ( s t o1 k h ou l d i ng z ) ((pos "Kj$" ) ))
( "stockier"  j ( s t o1 k i@ r ) ((pos "Or%" ) ))
( "stockiest"  j ( s t o1 k i i s t ) ((pos "Os%" ) ))
( "stockily"  a ( s t o1 k i l ii ) ((pos "Pu%" ) ))
( "stockinette"  n ( s t o2 k i n e1 t ) ((pos "L@%" ) ))
( "stocking"  v ( s t o1 k i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" )))
( "stockinged"  j ( s t o1 k i ng d ) ((pos "OA%" ) ))
( "stockings"  n ( s t o1 k i ng z ) ((pos "Kj%" ) ))
( "stockist"  n ( s t o1 k i s t ) ((pos "K6%" ) ))
( "stockists"  n ( s t o1 k i s t s ) ((pos "Kj%" ) ))
( "stockjobber"  n ( s t o1 k jh o b @ r ) ((pos "K6$" ) ))
( "stockjobbers"  n ( s t o1 k jh o b @ z ) ((pos "Kj$" ) ))
( "stockpile"  n ( s t o1 k p ai l ) ((pos "K6%" ) ))
( "stockpiles"  n ( s t o1 k p ai l z ) ((pos "Kj%" ) ))
( "stockpiling"  n ( s t o1 k p ai l i ng ) ((pos "L@%" ) ))
( "stockpot"  n ( s t o1 k p o t ) ((pos "K6%" ) ))
( "stockpots"  n ( s t o1 k p o t s ) ((pos "Kj%" ) ))
( "stockroom"  n ( s t o1 k r uu m ) ((pos "K6%" ) ))
( "stockrooms"  n ( s t o1 k r uu m z ) ((pos "Kj%" ) ))
( "stocks"  v ( s t o1 k s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "stocktaking"  n ( s t o1 k t ei k i ng ) ((pos "M6%" ) ))
( "stocktakings"  n ( s t o1 k t ei k i ng z ) ((pos "Mj$" ) ))
( "stocky"  j ( s t o1 k ii ) ((pos "OD%" ) ))
( "stockyard"  n ( s t o1 k y aa d ) ((pos "K6%" ) ))
( "stockyards"  n ( s t o1 k y aa d z ) ((pos "Kj%" ) ))
( "stodge"  n ( s t o1 jh ) ((pos "L@%" ) ))
( "stodgier"  j ( s t o1 jh i@ r ) ((pos "Or%" ) ))
( "stodgiest"  j ( s t o1 jh i i s t ) ((pos "Os%" ) ))
( "stodginess"  n ( s t o1 jh i n i s ) ((pos "L@%" ) ))
( "stodgy"  j ( s t o1 jh ii ) ((pos "OD%" ) ))
( "stoep"  n ( s t u1 p ) ((pos "K6$" ) ))
( "stoeps"  n ( s t u1 p s ) ((pos "Kj$" ) ))
( "stoic"  n ( s t ou1 i k ) ((pos "K6%" ) ))
( "stoical"  j ( s t ou1 i k l ) ((pos "OA%" ) ))
( "stoically"  a ( s t ou1 i k l ii ) ((pos "Pu%" ) ))
( "stoicism"  n ( s t ou1 i s i z @ m ) ((pos "L@%" ) ))
( "stoics"  n ( s t ou1 i k s ) ((pos "Kj%" ) ))
( "stoke"  v ( s t ou1 k ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "stoked"  v ( s t ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "stokehold"  n ( s t ou1 k h ou l d ) ((pos "K6$" ) ))
( "stokeholds"  n ( s t ou1 k h ou l d z ) ((pos "Kj$" ) ))
( "stokehole"  n ( s t ou1 k h ou l ) ((pos "K6$" ) ))
( "stokeholes"  n ( s t ou1 k h ou l z ) ((pos "Kj$" ) ))
( "stoker"  n ( s t ou1 k @ r ) ((pos "K6%" ) ))
( "stokers"  n ( s t ou1 k @ z ) ((pos "Kj%" ) ))
( "stokes"  v ( s t ou1 k s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "stoking"  v ( s t ou1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "stole"  v ( s t ou1 l ) ((pos "Jc%" "K6$" ) (subcat "2A" "2C" "6A" "14" )))
( "stolen"  v ( s t ou1 l @ n ) ((pos "Jd%" "OA%" ) (subcat "2A" "2C" "6A" "14" )))
( "stoles"  n ( s t ou1 l z ) ((pos "Kj$" ) ))
( "stolid"  j ( s t o1 l i d ) ((pos "OA%" ) ))
( "stolidity"  n ( s t @1 l i1 d i t ii ) ((pos "L@%" ) ))
( "stolidly"  a ( s t o1 l i d l ii ) ((pos "Pu%" ) ))
( "stolidness"  n ( s t o1 l i d n @ s ) ((pos "L@%" ) ))
( "stomach"  v ( s t uh1 m @ k ) ((pos "H0%" "M6%" ) (subcat )))
( "stomach-ache"  n ( s t uh1 m @ k - ei k ) ((pos "K6%" ) ))
( "stomach-aches"  n ( s t uh1 m @ k - ei k s ) ((pos "Kj%" ) ))
( "stomach-pump"  n ( s t uh1 m @ k - p uh m p ) ((pos "K6%" ) ))
( "stomach-pumps"  n ( s t uh1 m @ k - p uh m p s ) ((pos "Kj%" ) ))
( "stomached"  v ( s t uh1 m @ k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stomaching"  v ( s t uh1 m @ k i ng ) ((pos "Hb$" ) (subcat )))
( "stomachs"  v ( s t uh1 m @ k s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "stomp"  v ( s t o1 m p ) ((pos "I0%" "K6%" ) (subcat )))
( "stomped"  v ( s t o1 m p t ) ((pos "Ic%" "Id%" ) (subcat )))
( "stomping"  v ( s t o1 m p i ng ) ((pos "Ib%" ) (subcat )))
( "stomps"  v ( s t o1 m p s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "stone"  v ( s t ou1 n ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "stone-blind"  j ( s t ou1 n - b l ai1 n d ) ((pos "OA%" ) ))
( "stone-cold"  j ( s t ou1 n - k ou1 l d ) ((pos "OA%" ) ))
( "stone-dead"  j ( s t ou1 n - d e1 d ) ((pos "OA%" ) ))
( "stone-deaf"  j ( s t ou1 n - d e1 f ) ((pos "OA%" ) ))
( "stone-fruit"  n ( s t ou1 n - f r uu t ) ((pos "M6$" ) ))
( "stone-fruits"  n ( s t ou1 n - f r uu t s ) ((pos "Mj$" ) ))
( "stone-pit"  n ( s t ou1 n - p i t ) ((pos "K6$" ) ))
( "stone-pits"  n ( s t ou1 n - p i t s ) ((pos "Kj$" ) ))
( "stone-sober"  j ( s t ou1 n - s ou b @ r ) ((pos "OA%" ) ))
( "stonebreaker"  n ( s t ou1 n b r ei k @ r ) ((pos "K6$" ) ))
( "stonebreakers"  n ( s t ou1 n b r ei k @ z ) ((pos "Kj$" ) ))
( "stoned"  v ( s t ou1 n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" )))
( "stoneless"  j ( s t ou1 n l @ s ) ((pos "OA%" ) ))
( "stonemason"  n ( s t ou1 n m ei s n ) ((pos "K6%" ) ))
( "stonemasons"  n ( s t ou1 n m ei s n z ) ((pos "Kj%" ) ))
( "stones"  v ( s t ou1 n z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "stonewall"  v ( s t ou1 n w aa1 l ) ((pos "H0%" ) ))
( "stonewalled"  v ( s t ou1 n w aa1 l d ) ((pos "Hc%" "Hd%" ) ))
( "stonewaller"  n ( s t ou1 n w aa1 l @ r ) ((pos "K6%" ) ))
( "stonewallers"  n ( s t ou1 n w aa1 l @ z ) ((pos "Kj%" ) ))
( "stonewalling"  v ( s t ou1 n w aa1 l i ng ) ((pos "Hb%" "L@%" ) ))
( "stonewalls"  v ( s t ou1 n w aa1 l z ) ((pos "Ha%" ) ))
( "stoneware"  n ( s t ou1 n w e@ r ) ((pos "L@%" ) ))
( "stonework"  n ( s t ou1 n w @@ k ) ((pos "L@%" ) ))
( "stonier"  j ( s t ou1 n i@ r ) ((pos "Or$" ) ))
( "stoniest"  j ( s t ou1 n i i s t ) ((pos "Os%" ) ))
( "stonily"  a ( s t ou1 n i l ii ) ((pos "Pu%" ) ))
( "stoning"  v ( s t ou1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stony"  j ( s t ou1 n ii ) ((pos "OD%" ) ))
( "stony-broke"  j ( s t ou2 n i - b r ou1 k ) ((pos "OA%" ) ))
( "stood"  v ( s t u1 d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6C" "12A" "15A" )))
( "stooge"  v ( s t uu1 jh ) ((pos "I2$" "K6%" ) ))
( "stooged"  v ( s t uu1 jh d ) ((pos "Ic$" "Id$" ) ))
( "stooges"  v ( s t uu1 jh i z ) ((pos "Ia$" "Kj%" ) ))
( "stooging"  v ( s t uu1 jh i ng ) ((pos "Ib$" ) ))
( "stool"  n ( s t uu1 l ) ((pos "K6%" ) ))
( "stools"  n ( s t uu1 l z ) ((pos "Kj%" ) ))
( "stoop"  v ( s t uu1 p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "4A" )))
( "stooped"  v ( s t uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "4A" )))
( "stooping"  v ( s t uu1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "4A" )))
( "stoops"  v ( s t uu1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" )))
( "stop"  v ( s t o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6B" "6C" "14" "15A" )))
( "stopcock"  n ( s t o1 p k o k ) ((pos "K6%" ) ))
( "stopcocks"  n ( s t o1 p k o k s ) ((pos "Kj%" ) ))
( "stopgap"  n ( s t o1 p g a p ) ((pos "K6%" ) ))
( "stopgaps"  n ( s t o1 p g a p s ) ((pos "Kj%" ) ))
( "stopover"  n ( s t o1 p ou v @ r ) ((pos "K6%" ) ))
( "stopovers"  n ( s t o1 p ou v @ z ) ((pos "Kj%" ) ))
( "stoppage"  n ( s t o1 p i jh ) ((pos "K6%" ) ))
( "stoppages"  n ( s t o1 p i jh i z ) ((pos "Kj%" ) ))
( "stopped"  v ( s t o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6B" "6C" "14" "15A" )))
( "stopper"  n ( s t o1 p @ r ) ((pos "K6%" ) ))
( "stoppers"  n ( s t o1 p @ z ) ((pos "Kj%" ) ))
( "stopping"  v ( s t o1 p i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6B" "6C" "14" "15A" )))
( "stoppings"  n ( s t o1 p i ng z ) ((pos "Mj%" ) ))
( "stops"  v ( s t o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "6B" "6C" "14" "15A" )))
( "stopwatch"  n ( s t o1 p w o ch ) ((pos "K7%" ) ))
( "stopwatches"  n ( s t o1 p w o ch i z ) ((pos "Kj%" ) ))
( "storage"  n ( s t oo1 r i jh ) ((pos "L@%" ) ))
( "store"  v ( s t oo1 r ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "stored"  v ( s t oo1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "storehouse"  n ( s t oo1 h au s ) ((pos "K6%" ) ))
( "storehouses"  n ( s t oo1 h au z i z ) ((pos "Kj%" ) ))
( "storeroom"  n ( s t oo1 r uu m ) ((pos "K6%" ) ))
( "storerooms"  n ( s t oo1 r uu m z ) ((pos "Kj%" ) ))
( "stores"  v ( s t oo1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "storey"  n ( s t oo1 r ii ) ((pos "K6%" ) ))
( "storeyed"  j ( s t oo1 r i d ) ((pos "Ot%" ) ))
( "storeys"  n ( s t oo1 r i z ) ((pos "Kj%" ) ))
( "storied"  j ( s t oo1 r i d ) ((pos "OA$" ) ))
( "stories"  n ( s t oo1 r i z ) ((pos "Kj%" ) ))
( "storing"  v ( s t oo1 r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stork"  n ( s t oo1 k ) ((pos "K6%" ) ))
( "storks"  n ( s t oo1 k s ) ((pos "Kj%" ) ))
( "storm"  v ( s t oo1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "storm-beaten"  j ( s t oo1 m - b ii t n ) ((pos "OA$" ) ))
( "storm-bound"  j ( s t oo1 m - b au n d ) ((pos "OA%" ) ))
( "storm-centre"  n ( s t oo1 m - s e n t @ r ) ((pos "K6%" ) ))
( "storm-centres"  n ( s t oo1 m - s e n t @ z ) ((pos "Kj$" ) ))
( "storm-cloud"  n ( s t oo1 m - k l au d ) ((pos "K6%" ) ))
( "storm-clouds"  n ( s t oo1 m - k l au d z ) ((pos "Kj%" ) ))
( "storm-cone"  n ( s t oo1 m - k ou n ) ((pos "K6%" ) ))
( "storm-cones"  n ( s t oo1 m - k ou n z ) ((pos "Kj%" ) ))
( "storm-lantern"  n ( s t oo1 m - l a n t @ n ) ((pos "K6%" ) ))
( "storm-lanterns"  n ( s t oo1 m - l a n t @ n z ) ((pos "Kj%" ) ))
( "storm-signal"  n ( s t oo1 m - s i g n @ l ) ((pos "K6%" ) ))
( "storm-signals"  n ( s t oo1 m - s i g n @ l z ) ((pos "Kj%" ) ))
( "storm-tossed"  j ( s t oo1 m - t o s t ) ((pos "OA%" ) ))
( "storm-trooper"  n ( s t oo1 m - t r uu p @ r ) ((pos "K6%" ) ))
( "storm-troopers"  n ( s t oo1 m - t r uu p @ z ) ((pos "Kj%" ) ))
( "storm-troops"  n ( s t oo1 m - t r uu p s ) ((pos "Kj%" ) ))
( "stormed"  v ( s t oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "stormier"  j ( s t oo1 m i@ r ) ((pos "Or%" ) ))
( "stormiest"  j ( s t oo1 m i i s t ) ((pos "Os%" ) ))
( "stormily"  a ( s t oo1 m i l ii ) ((pos "Pu%" ) ))
( "storming"  v ( s t oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "stormproof"  j ( s t oo1 m p r uu f ) ((pos "OA%" ) ))
( "storms"  v ( s t oo1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "stormy"  j ( s t oo1 m ii ) ((pos "OD%" ) ))
( "story"  n ( s t oo1 r ii ) ((pos "K8*" ) ))
( "storyteller"  n ( s t oo1 r i t e l @ r ) ((pos "K6%" ) ))
( "storytellers"  n ( s t oo1 r i t e l @ z ) ((pos "Kj%" ) ))
( "stoup"  n ( s t uu1 p ) ((pos "K6$" ) ))
( "stoups"  n ( s t uu1 p s ) ((pos "Kj$" ) ))
( "stout"  n ( s t au1 t ) ((pos "L@%" "OC%" ) ))
( "stouter"  j ( s t au1 t @ r ) ((pos "Or%" ) ))
( "stoutest"  j ( s t au1 t i s t ) ((pos "Os%" ) ))
( "stouthearted"  j ( s t au1 t h aa1 t i d ) ((pos "OA%" ) ))
( "stoutly"  a ( s t au1 t l ii ) ((pos "Pu%" ) ))
( "stoutness"  n ( s t au1 t n @ s ) ((pos "L@%" ) ))
( "stove"  v ( s t ou1 v ) ((pos "Jc%" "Jd%" "K6%" ) (subcat "3A" )))
( "stovepipe"  n ( s t ou1 v p ai p ) ((pos "K6%" ) ))
( "stovepipes"  n ( s t ou1 v p ai p s ) ((pos "Kj%" ) ))
( "stoves"  n ( s t ou1 v z ) ((pos "Kj%" ) ))
( "stow"  v ( s t ou1 ) ((pos "H0%" ) (subcat "6A" "15A" )))
( "stowaway"  n ( s t ou1 @ w ei ) ((pos "K6%" ) ))
( "stowaways"  n ( s t ou1 @ w ei z ) ((pos "Kj%" ) ))
( "stowed"  v ( s t ou1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "stowing"  v ( s t ou1 i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "stows"  v ( s t ou1 z ) ((pos "Ha%" ) (subcat "6A" "15A" )))
( "straddle"  v ( s t r a1 d l ) ((pos "J2%" ) (subcat "2A" )))
( "straddled"  v ( s t r a1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "straddles"  v ( s t r a1 d l z ) ((pos "Ja%" ) (subcat "2A" )))
( "straddling"  v ( s t r a1 d l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "strafe"  v ( s t r aa1 f ) ((pos "H2$" ) (subcat )))
( "strafed"  v ( s t r aa1 f t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "strafes"  v ( s t r aa1 f s ) ((pos "Ha$" ) (subcat )))
( "strafing"  v ( s t r aa1 f i ng ) ((pos "Hb$" ) (subcat )))
( "straggle"  v ( s t r a1 g l ) ((pos "I2%" ) (subcat "2A" )))
( "straggled"  v ( s t r a1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "straggler"  n ( s t r a1 g l @ r ) ((pos "K6%" ) ))
( "stragglers"  n ( s t r a1 g l @ z ) ((pos "Kj%" ) ))
( "straggles"  v ( s t r a1 g l z ) ((pos "Ia%" ) (subcat "2A" )))
( "stragglier"  j ( s t r a1 g l i@ r ) ((pos "Or$" ) ))
( "straggliest"  j ( s t r a1 g l i i s t ) ((pos "Os$" ) ))
( "straggling"  v ( s t r a1 g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "straggly"  j ( s t r a1 g l ii ) ((pos "OD%" ) ))
( "straight"  n ( s t r ei1 t ) ((pos "K6%" "OA%" "Pu%" ) ))
( "straighten"  v ( s t r ei1 t n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "straightened"  v ( s t r ei1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "straightening"  v ( s t r ei1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "straightens"  v ( s t r ei1 t n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "straightforward"  j ( s t r ei2 t f oo1 w @ d ) ((pos "OA%" ) ))
( "straightforwardly"  a ( s t r ei2 t f oo1 w @ d l ii ) ((pos "Pu%" ) ))
( "straightness"  n ( s t r ei1 t n @ s ) ((pos "L@%" ) ))
( "straights"  n ( s t r ei1 t s ) ((pos "Kj$" ) ))
( "straightway"  a ( s t r ei2 t w ei1 ) ((pos "Pu%" ) ))
( "strain"  v ( s t r ei1 n ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "6A" "15A" "15B" )))
( "strained"  v ( s t r ei1 n d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" "15A" "15B" )))
( "strainer"  n ( s t r ei1 n @ r ) ((pos "K6%" ) ))
( "strainers"  n ( s t r ei1 n @ z ) ((pos "Kj%" ) ))
( "straining"  v ( s t r ei1 n i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "15A" "15B" )))
( "strains"  v ( s t r ei1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "15A" "15B" )))
( "strait"  n ( s t r ei1 t ) ((pos "K6$" "OA$" ) ))
( "strait-laced"  j ( s t r ei1 t - l ei1 s t ) ((pos "OA%" ) ))
( "straiten"  v ( s t r ei1 t n ) ((pos "H0%" ) ))
( "straitened"  v ( s t r ei1 t n d ) ((pos "Hc%" "Hd%" ) ))
( "straitening"  v ( s t r ei1 t n i ng ) ((pos "Hb%" ) ))
( "straitens"  v ( s t r ei1 t n z ) ((pos "Ha%" ) ))
( "straitjacket"  n ( s t r ei1 t jh a k i t ) ((pos "K6%" ) ))
( "straitjackets"  n ( s t r ei1 t jh a k i t s ) ((pos "Kj%" ) ))
( "straits"  n ( s t r ei1 t s ) ((pos "Kj%" ) ))
( "strand"  v ( s t r a1 n d ) ((pos "J0$" "K6%" ) (subcat "2A" )))
( "stranded"  v ( s t r a1 n d i d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "stranding"  v ( s t r a1 n d i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "strands"  v ( s t r a1 n d z ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "strange"  j ( s t r ei1 n jh ) ((pos "OB%" ) ))
( "strangely"  a ( s t r ei1 n jh l ii ) ((pos "Pu%" ) ))
( "strangeness"  n ( s t r ei1 n jh n @ s ) ((pos "L@%" ) ))
( "stranger"  n ( s t r ei1 n jh @ r ) ((pos "K6%" "Or%" ) ))
( "strangers"  n ( s t r ei1 n jh @ z ) ((pos "Kj%" ) ))
( "strangest"  j ( s t r ei1 n jh i s t ) ((pos "Os%" ) ))
( "strangle"  v ( s t r a1 ng g l ) ((pos "H2%" ) (subcat )))
( "strangled"  v ( s t r a1 ng g l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stranglehold"  n ( s t r a1 ng g l h ou l d ) ((pos "K6%" ) ))
( "strangleholds"  n ( s t r a1 ng g l h ou l d z ) ((pos "Kj$" ) ))
( "strangles"  v ( s t r a1 ng g l z ) ((pos "Ha%" ) (subcat )))
( "strangling"  v ( s t r a1 ng g l i ng ) ((pos "Hb%" ) (subcat )))
( "strangulation"  n ( s t r a2 ng g y u l ei1 sh @ n ) ((pos "L@%" ) ))
( "strap"  v ( s t r a1 p ) ((pos "H4%" "M6%" ) (subcat "6A" )))
( "straphanger"  n ( s t r a1 p h a ng @ r ) ((pos "K6%" ) ))
( "straphangers"  n ( s t r a1 p h a ng @ z ) ((pos "Kj%" ) ))
( "strapped"  v ( s t r a1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "strapping"  v ( s t r a1 p i ng ) ((pos "Hb%" "Ki%" "OA%" ) (subcat "6A" )))
( "straps"  v ( s t r a1 p s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "strata"  n ( s t r aa1 t @ ) ((pos "Kj%" ) ))
( "stratagem"  n ( s t r a1 t @ jh @ m ) ((pos "M6%" ) ))
( "stratagems"  n ( s t r a1 t @ jh @ m z ) ((pos "Mj%" ) ))
( "strategic"  j ( s t r @1 t ii1 jh i k ) ((pos "OA%" ) ))
( "strategical"  j ( s t r @1 t ii1 jh i k l ) ((pos "OA$" ) ))
( "strategically"  a ( s t r @1 t ii1 jh i k l ii ) ((pos "Pu%" ) ))
( "strategics"  n ( s t r @1 t ii1 jh i k s ) ((pos "Lk$" ) ))
( "strategies"  n ( s t r a1 t @ jh i z ) ((pos "Mj%" ) ))
( "strategist"  n ( s t r a1 t i jh i s t ) ((pos "K6%" ) ))
( "strategists"  n ( s t r a1 t i jh i s t s ) ((pos "Kj%" ) ))
( "strategy"  n ( s t r a1 t @ jh ii ) ((pos "M8%" ) ))
( "stratification"  n ( s t r a2 t i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "stratifications"  n ( s t r a2 t i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "stratified"  v ( s t r a1 t i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "stratifies"  v ( s t r a1 t i f ai z ) ((pos "Ja%" ) (subcat "2A" )))
( "stratify"  v ( s t r a1 t i f ai ) ((pos "J3%" ) (subcat "2A" )))
( "stratifying"  v ( s t r a1 t i f ai i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "stratosphere"  n ( s t r a1 t @ s f i@ r ) ((pos "K6%" ) ))
( "stratospheres"  n ( s t r a1 t @ s f i@ z ) ((pos "Kj$" ) ))
( "stratum"  n ( s t r aa1 t @ m ) ((pos "Ki$" ) ))
( "straw"  v ( s t r oo1 ) ((pos "H0$" "M6%" ) ))
( "straw-coloured"  j ( s t r oo1 - k uh l @ d ) ((pos "OA%" ) ))
( "strawberries"  n ( s t r oo1 b r i z ) ((pos "Kj%" ) ))
( "strawberry"  n ( s t r oo1 b r ii ) ((pos "K8%" ) ))
( "strawboard"  n ( s t r oo1 b oo d ) ((pos "L@$" ) ))
( "strawed"  v ( s t r oo1 d ) ((pos "Hc$" "Hd$" ) ))
( "strawing"  v ( s t r oo1 i ng ) ((pos "Hb$" ) ))
( "straws"  v ( s t r oo1 z ) ((pos "Ha$" "Mj%" ) ))
( "stray"  v ( s t r ei1 ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "strayed"  v ( s t r ei1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "straying"  v ( s t r ei1 i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "strays"  v ( s t r ei1 z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "streak"  v ( s t r ii1 k ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "streaked"  v ( s t r ii1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "streakier"  j ( s t r ii1 k i@ r ) ((pos "Or$" ) ))
( "streakiest"  j ( s t r ii1 k i i s t ) ((pos "Os$" ) ))
( "streaking"  v ( s t r ii1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "streaks"  v ( s t r ii1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "streaky"  j ( s t r ii1 k ii ) ((pos "OD%" ) ))
( "stream"  v ( s t r ii1 m ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "streamed"  v ( s t r ii1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "streamer"  n ( s t r ii1 m @ r ) ((pos "K6%" ) ))
( "streamers"  n ( s t r ii1 m @ z ) ((pos "Kj%" ) ))
( "streaming"  v ( s t r ii1 m i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "streamlet"  n ( s t r ii1 m l i t ) ((pos "K6$" ) ))
( "streamlets"  n ( s t r ii1 m l i t s ) ((pos "Kj$" ) ))
( "streamline"  v ( s t r ii1 m l ai n ) ((pos "H2%" ) ))
( "streamlined"  v ( s t r ii1 m l ai n d ) ((pos "Hc%" "Hd%" "OA%" ) ))
( "streamlines"  v ( s t r ii1 m l ai n z ) ((pos "Ha%" ) ))
( "streamlining"  v ( s t r ii1 m l ai n i ng ) ((pos "Hb%" ) ))
( "streams"  v ( s t r ii1 m z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "street"  n ( s t r ii1 t ) ((pos "K6%" ) ))
( "street-girl"  n ( s t r ii1 t - g @@ l ) ((pos "K6%" ) ))
( "street-girls"  n ( s t r ii1 t - g @@ l z ) ((pos "Kj%" ) ))
( "street-urchin"  n ( s t r ii1 t - @@ ch i n ) ((pos "K6%" ) ))
( "street-urchins"  n ( s t r ii1 t - @@ ch i n z ) ((pos "Kj%" ) ))
( "streetcar"  n ( s t r ii1 t k aa r ) ((pos "K6%" ) ))
( "streetcars"  n ( s t r ii1 t k aa z ) ((pos "Kj%" ) ))
( "streets"  n ( s t r ii1 t s ) ((pos "Kj%" ) ))
( "streetwalker"  n ( s t r ii1 t w oo k @ r ) ((pos "K6%" ) ))
( "streetwalkers"  n ( s t r ii1 t w oo k @ z ) ((pos "Kj%" ) ))
( "strength"  n ( s t r e1 ng th ) ((pos "M6%" ) ))
( "strengthen"  v ( s t r e1 ng th n ) ((pos "J0%" ) (subcat "2A" )))
( "strengthened"  v ( s t r e1 ng th n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "strengthening"  v ( s t r e1 ng th n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "strengthens"  v ( s t r e1 ng th n z ) ((pos "Ja%" ) (subcat "2A" )))
( "strengths"  n ( s t r e1 ng th s ) ((pos "Mj%" ) ))
( "strenuous"  j ( s t r e1 n y u@ s ) ((pos "OA%" ) ))
( "strenuously"  a ( s t r e1 n y u@ s l ii ) ((pos "Pu%" ) ))
( "strenuousness"  n ( s t r e1 n y u@ s n @ s ) ((pos "L@%" ) ))
( "streptococci"  n ( s t r e2 p t @ k o1 k ai ) ((pos "Kj$" ) ))
( "streptococcus"  n ( s t r e2 p t @ k o1 k @ s ) ((pos "Ki$" ) ))
( "streptomycin"  n ( s t r e2 p t @ m ai1 s i n ) ((pos "L@$" ) ))
( "stress"  v ( s t r e1 s ) ((pos "H1%" "M7%" ) (subcat )))
( "stress-mark"  n ( s t r e1 s - m aa k ) ((pos "K6%" ) ))
( "stress-marks"  n ( s t r e1 s - m aa k s ) ((pos "Kj%" ) ))
( "stressed"  v ( s t r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stresses"  v ( s t r e1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "stressing"  v ( s t r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "stretch"  v ( s t r e1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "2B" "2C" "6A" "15A" "15B" "16A" )))
( "stretched"  v ( s t r e1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "15A" "15B" "16A" )))
( "stretcher"  n ( s t r e1 ch @ r ) ((pos "K6%" ) ))
( "stretcher-bearer"  n ( s t r e1 ch @ - b e@ r @ r ) ((pos "K6%" ) ))
( "stretcher-bearers"  n ( s t r e1 ch @ - b e@ r @ z ) ((pos "Kj%" ) ))
( "stretcher-parties"  n ( s t r e1 ch @ - p aa t i z ) ((pos "Kj%" ) ))
( "stretcher-party"  n ( s t r e1 ch @ - p aa t ii ) ((pos "K8%" ) ))
( "stretchers"  n ( s t r e1 ch @ z ) ((pos "Kj%" ) ))
( "stretches"  v ( s t r e1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "6A" "15A" "15B" "16A" )))
( "stretching"  v ( s t r e1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" "15A" "15B" "16A" )))
( "strew"  v ( s t r uu1 ) ((pos "H0$" ) (subcat "6A" )))
( "strewed"  v ( s t r uu1 d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "strewing"  v ( s t r uu1 i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "strewn"  v ( s t r uu1 n ) ((pos "Hd%" ) (subcat "6A" )))
( "strews"  v ( s t r uu1 z ) ((pos "Ha$" ) (subcat "6A" )))
( "strewth"  uh ( s t r uu1 th ) ((pos "W-$" ) ))
( "striated"  j ( s t r ai1 ei1 t i d ) ((pos "OA$" ) ))
( "stricken"  j ( s t r i1 k @ n ) ((pos "Op%" ) ))
( "strict"  j ( s t r i1 k t ) ((pos "OC%" ) ))
( "stricter"  j ( s t r i1 k t @ r ) ((pos "Or%" ) ))
( "strictest"  j ( s t r i1 k t i s t ) ((pos "Os%" ) ))
( "strictly"  a ( s t r i1 k l ii ) ((pos "Pu%" ) ))
( "strictness"  n ( s t r i1 k t n @ s ) ((pos "L@%" ) ))
( "stricture"  n ( s t r i1 k ch @ r ) ((pos "K6%" ) ))
( "strictures"  n ( s t r i1 k ch @ z ) ((pos "Kj%" ) ))
( "stridden"  v ( s t r i1 d n ) ((pos "Jd$" ) (subcat "2C" )))
( "stride"  v ( s t r ai1 d ) ((pos "J5%" "K6%" ) (subcat "2C" )))
( "strident"  j ( s t r ai1 d n t ) ((pos "OA%" ) ))
( "stridently"  a ( s t r ai1 d n t l ii ) ((pos "Pu%" ) ))
( "strides"  v ( s t r ai1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "striding"  v ( s t r ai1 d i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "stridulate"  v ( s t r i1 d y u l ei t ) ((pos "I2$" ) (subcat )))
( "stridulated"  v ( s t r i1 d y u l ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "stridulates"  v ( s t r i1 d y u l ei t s ) ((pos "Ia$" ) (subcat )))
( "stridulating"  v ( s t r i1 d y u l ei t i ng ) ((pos "Ib$" ) (subcat )))
( "stridulation"  n ( s t r i2 d y u l ei1 sh @ n ) ((pos "M6$" ) ))
( "stridulations"  n ( s t r i2 d y u l ei1 sh @ n z ) ((pos "Mj$" ) ))
( "strife"  n ( s t r ai1 f ) ((pos "L@%" ) ))
( "strike"  v ( s t r ai1 k ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12C" "14" "15B" "16A" )))
( "strike-leader"  n ( s t r ai1 k - l ii d @ r ) ((pos "K6%" ) ))
( "strike-leaders"  n ( s t r ai1 k - l ii d @ z ) ((pos "Kj%" ) ))
( "strike-pay"  n ( s t r ai1 k - p ei ) ((pos "L@%" ) ))
( "strikebound"  j ( s t r ai1 k b au n d ) ((pos "OA%" ) ))
( "strikebreaker"  n ( s t r ai1 k b r ei k @ r ) ((pos "K6%" ) ))
( "strikebreakers"  n ( s t r ai1 k b r ei k @ z ) ((pos "Kj%" ) ))
( "striker"  n ( s t r ai1 k @ r ) ((pos "K6%" ) ))
( "strikers"  n ( s t r ai1 k @ z ) ((pos "Kj%" ) ))
( "strikes"  v ( s t r ai1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12C" "14" "15B" "16A" )))
( "striking"  v ( s t r ai1 k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12C" "14" "15B" "16A" )))
( "strikingly"  a ( s t r ai1 k i ng l ii ) ((pos "Pu%" ) ))
( "string"  v ( s t r i1 ng ) ((pos "J5%" "M6%" ) (subcat "2C" "6A" "15A" )))
( "stringencies"  n ( s t r i1 n jh @ n s i z ) ((pos "Mj%" ) ))
( "stringency"  n ( s t r i1 n jh @ n s ii ) ((pos "M8%" ) ))
( "stringent"  j ( s t r i1 n jh @ n t ) ((pos "OA%" ) ))
( "stringently"  a ( s t r i1 n jh @ n t l ii ) ((pos "Pu%" ) ))
( "stringier"  j ( s t r i1 ng i@ r ) ((pos "Or%" ) ))
( "stringiest"  j ( s t r i1 ng i i s t ) ((pos "Os%" ) ))
( "stringing"  v ( s t r i1 ng i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "strings"  v ( s t r i1 ng z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" "15A" )))
( "stringy"  j ( s t r i1 ng ii ) ((pos "OD%" ) ))
( "strip"  v ( s t r i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "strip-lighting"  n ( s t r i1 p - l ai th i ng ) ((pos "L@%" ) ))
( "strip-poker"  n ( s t r i1 p - p ou1 k @ r ) ((pos "L@%" ) ))
( "strip-show"  n ( s t r i1 p - sh ou ) ((pos "K6%" ) ))
( "strip-shows"  n ( s t r i1 p - sh ou z ) ((pos "Kj%" ) ))
( "stripe"  n ( s t r ai1 p ) ((pos "K6%" ) ))
( "striped"  j ( s t r ai1 p t ) ((pos "OA%" ) ))
( "stripes"  n ( s t r ai1 p s ) ((pos "Kj%" ) ))
( "stripier"  j ( s t r ai1 p i@ r ) ((pos "Or$" ) ))
( "stripiest"  j ( s t r ai1 p i i s t ) ((pos "Os$" ) ))
( "stripling"  n ( s t r i1 p l i ng ) ((pos "K6%" ) ))
( "striplings"  n ( s t r i1 p l i ng z ) ((pos "Kj%" ) ))
( "stripped"  v ( s t r i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "stripper"  n ( s t r i1 p @ r ) ((pos "K6%" ) ))
( "strippers"  n ( s t r i1 p @ z ) ((pos "Kj%" ) ))
( "stripping"  v ( s t r i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "strips"  v ( s t r i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15B" )))
( "striptease"  n ( s t r i1 p t ii1 z ) ((pos "L@%" ) ))
( "stripy"  j ( s t r ai1 p ii ) ((pos "OD%" ) ))
( "strive"  v ( s t r ai1 v ) ((pos "I5%" ) (subcat "2A" "3A" )))
( "striven"  v ( s t r i1 v n ) ((pos "Id$" ) (subcat "2A" "3A" )))
( "striver"  n ( s t r ai1 v @ r ) ((pos "K6$" ) ))
( "strivers"  n ( s t r ai1 v @ z ) ((pos "Kj$" ) ))
( "strives"  v ( s t r ai1 v z ) ((pos "Ia%" ) (subcat "2A" "3A" )))
( "striving"  v ( s t r ai1 v i ng ) ((pos "Ib%" ) (subcat "2A" "3A" )))
( "stroboscope"  n ( s t r ou1 b @ s k ou p ) ((pos "K6%" ) ))
( "stroboscopes"  n ( s t r ou1 b @ s k ou p s ) ((pos "Kj$" ) ))
( "strode"  v ( s t r ou1 d ) ((pos "Jc%" ) (subcat "2C" )))
( "stroke"  v ( s t r ou1 k ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "stroked"  v ( s t r ou1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "strokes"  v ( s t r ou1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "stroking"  v ( s t r ou1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stroll"  v ( s t r ou1 l ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "strolled"  v ( s t r ou1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "stroller"  n ( s t r ou1 l @ r ) ((pos "K6%" ) ))
( "strollers"  n ( s t r ou1 l @ z ) ((pos "Kj$" ) ))
( "strolling"  v ( s t r ou1 l i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "strolls"  v ( s t r ou1 l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "strong"  j ( s t r o1 ng ) ((pos "OC%" ) ))
( "strong-arm"  j ( s t r o1 ng - aa m ) ((pos "OA%" ) ))
( "strong-boned"  j ( s t r o1 ng - b ou1 n d ) ((pos "OA%" ) ))
( "strong-minded"  j ( s t r o1 ng - m ai1 n d i d ) ((pos "OA%" ) ))
( "strongbox"  n ( s t r o1 ng b o k s ) ((pos "K7%" ) ))
( "strongboxes"  n ( s t r o1 ng b o k s i z ) ((pos "Kj%" ) ))
( "stronger"  j ( s t r o1 ng g @ r ) ((pos "Or%" ) ))
( "strongest"  j ( s t r o1 ng g i s t ) ((pos "Os%" ) ))
( "stronghold"  n ( s t r o1 ng h ou l d ) ((pos "K6%" ) ))
( "strongholds"  n ( s t r o1 ng h ou l d z ) ((pos "Kj%" ) ))
( "strongly"  a ( s t r o1 ng l ii ) ((pos "Pu%" ) ))
( "strongroom"  n ( s t r o1 ng r uu m ) ((pos "K6%" ) ))
( "strongrooms"  n ( s t r o1 ng r uu m z ) ((pos "Kj%" ) ))
( "strontium"  n ( s t r o1 n t i@ m ) ((pos "L@%" ) ))
( "strop"  v ( s t r o1 p ) ((pos "H4$" "K6$" ) (subcat )))
( "strophe"  n ( s t r ou1 f ii ) ((pos "K6$" ) ))
( "strophes"  n ( s t r ou1 f i z ) ((pos "Kj$" ) ))
( "stropped"  v ( s t r o1 p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "stropping"  v ( s t r o1 p i ng ) ((pos "Hb$" ) (subcat )))
( "stroppy"  j ( s t r o1 p ii ) ((pos "OA%" ) ))
( "strops"  v ( s t r o1 p s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "strove"  v ( s t r ou1 v ) ((pos "Ic%" ) (subcat "2A" "3A" )))
( "struck"  v ( s t r uh1 k ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2A" "2C" "2D" "3A" "6A" "12C" "14" "15B" "16A" )))
( "structural"  j ( s t r uh1 k ch @ r @ l ) ((pos "OA%" ) ))
( "structurally"  a ( s t r uh1 k ch @ r @ l ii ) ((pos "Pu%" ) ))
( "structure"  n ( s t r uh1 k ch @ r ) ((pos "M6%" ) ))
( "structured"  j ( s t r uh1 k ch @ d ) ((pos "OA%" ) ))
( "structures"  n ( s t r uh1 k ch @ z ) ((pos "Mj%" ) ))
( "strudel"  n ( s t r uu1 d l ) ((pos "M6$" ) ))
( "strudels"  n ( s t r uu1 d l z ) ((pos "Mj$" ) ))
( "struggle"  v ( s t r uh1 g l ) ((pos "I2%" "K6%" ) (subcat "2A" "2B" "3A" )))
( "struggled"  v ( s t r uh1 g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "3A" )))
( "struggles"  v ( s t r uh1 g l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "3A" )))
( "struggling"  v ( s t r uh1 g l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "3A" )))
( "strum"  v ( s t r uh1 m ) ((pos "J4%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "strummed"  v ( s t r uh1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" )))
( "strumming"  v ( s t r uh1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" )))
( "strumpet"  n ( s t r uh1 m p i t ) ((pos "K6$" ) ))
( "strumpets"  n ( s t r uh1 m p i t s ) ((pos "Kj$" ) ))
( "strums"  v ( s t r uh1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "strung"  v ( s t r uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "strut"  v ( s t r uh1 t ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "struth"  uh ( s t r uu1 th ) ((pos "W-$" ) ))
( "struts"  v ( s t r uh1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "strutted"  v ( s t r uh1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "strutting"  v ( s t r uh1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "strychnine"  n ( s t r i1 k n ii n ) ((pos "L@%" ) ))
( "stub"  v ( s t uh1 b ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "stubbed"  v ( s t uh1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "stubbier"  j ( s t uh1 b i@ r ) ((pos "Or$" ) ))
( "stubbiest"  j ( s t uh1 b i i s t ) ((pos "Os$" ) ))
( "stubbing"  v ( s t uh1 b i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "stubble"  n ( s t uh1 b l ) ((pos "L@%" ) ))
( "stubblier"  j ( s t uh1 b l i@ r ) ((pos "Or$" ) ))
( "stubbliest"  j ( s t uh1 b l i i s t ) ((pos "Os$" ) ))
( "stubbly"  j ( s t uh1 b l ii ) ((pos "OD%" ) ))
( "stubborn"  j ( s t uh1 b @ n ) ((pos "OA%" ) ))
( "stubbornly"  a ( s t uh1 b @ n l ii ) ((pos "Pu%" ) ))
( "stubbornness"  n ( s t uh1 b @ n - n @ s ) ((pos "L@%" ) ))
( "stubby"  j ( s t uh1 b ii ) ((pos "OD%" ) ))
( "stubs"  v ( s t uh1 b z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "stucco"  v ( s t uh1 k ou ) ((pos "H0$" "M6%" ) ))
( "stuccoed"  v ( s t uh1 k ou d ) ((pos "Hc$" "Hd%" ) ))
( "stuccoes"  n ( s t uh1 k ou z ) ((pos "Kj$" ) ))
( "stuccoing"  v ( s t uh1 k ou i ng ) ((pos "Hb$" ) ))
( "stuccos"  v ( s t uh1 k ou z ) ((pos "Ha$" "Mj$" ) ))
( "stuck"  v ( s t uh1 k ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "stuck-up"  j ( s t uh2 k - uh1 p ) ((pos "OA%" ) ))
( "stud"  v ( s t uh1 d ) ((pos "H4$" "K6%" ) ))
( "stud-farm"  n ( s t uh1 d - f aa m ) ((pos "K6%" ) ))
( "stud-farms"  n ( s t uh1 d - f aa m z ) ((pos "Kj%" ) ))
( "stud-mare"  n ( s t uh1 d - m e@ r ) ((pos "K6%" ) ))
( "stud-mares"  n ( s t uh1 d - m e@ z ) ((pos "Kj%" ) ))
( "studbook"  n ( s t uh1 d b u k ) ((pos "K6$" ) ))
( "studbooks"  n ( s t uh1 d b u k s ) ((pos "Kj$" ) ))
( "studded"  v ( s t uh1 d i d ) ((pos "Hc$" "Hd%" ) ))
( "studding"  v ( s t uh1 d i ng ) ((pos "Hb$" ) ))
( "student"  n ( s t y uu1 d n t ) ((pos "K6%" ) ))
( "students"  n ( s t y uu1 d n t s ) ((pos "Kj%" ) ))
( "studied"  v ( s t uh1 d i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "4A" "6A" "8" )))
( "studies"  v ( s t uh1 d i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "4A" "6A" "8" )))
( "studio"  n ( s t y uu1 d i@ @ ) ((pos "K6%" ) ))
( "studios"  n ( s t y uu1 d i@ u z ) ((pos "Kj%" ) ))
( "studious"  j ( s t y uu1 d i@ s ) ((pos "OA%" ) ))
( "studiously"  a ( s t y uu1 d i@ s l ii ) ((pos "Pu%" ) ))
( "studiousness"  n ( s t y uu1 d i@ s n @ s ) ((pos "L@%" ) ))
( "studs"  v ( s t uh1 d z ) ((pos "Ha$" "Kj%" ) ))
( "study"  v ( s t uh1 d ii ) ((pos "J3%" "M8%" ) (subcat "2A" "2B" "4A" "6A" "8" )))
( "studying"  v ( s t uh1 d i i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "4A" "6A" "8" )))
( "stuff"  v ( s t uh1 f ) ((pos "H0%" "M6%" ) (subcat "2A" "6A" "14" )))
( "stuffed"  v ( s t uh1 f t ) ((pos "Hc%" "Hd%" ) (subcat "2A" "6A" "14" )))
( "stuffier"  j ( s t uh1 f i@ r ) ((pos "Or%" ) ))
( "stuffiest"  j ( s t uh1 f i i s t ) ((pos "Os%" ) ))
( "stuffily"  a ( s t uh1 f i l ii ) ((pos "Pu%" ) ))
( "stuffiness"  n ( s t uh1 f i n @ s ) ((pos "L@%" ) ))
( "stuffing"  v ( s t uh1 f i ng ) ((pos "Hb%" "L@%" ) (subcat "2A" "6A" "14" )))
( "stuffs"  v ( s t uh1 f s ) ((pos "Ha%" "Mj$" ) (subcat "2A" "6A" "14" )))
( "stuffy"  j ( s t uh1 f ii ) ((pos "OD%" ) ))
( "stultification"  n ( s t uh2 l t i f i k ei1 sh @ n ) ((pos "M6$" ) ))
( "stultifications"  n ( s t uh2 l t i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "stultified"  v ( s t uh1 l t i f ai d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "stultifies"  v ( s t uh1 l t i f ai z ) ((pos "Ha$" ) (subcat )))
( "stultify"  v ( s t uh1 l t i f ai ) ((pos "H3$" ) (subcat )))
( "stultifying"  v ( s t uh1 l t i f ai i ng ) ((pos "Hb$" ) (subcat )))
( "stumble"  v ( s t uh1 m b l ) ((pos "I2%" "K6%" ) (subcat "2A" "2C" )))
( "stumbled"  v ( s t uh1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "stumbles"  v ( s t uh1 m b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "stumbling"  v ( s t uh1 m b l i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "stumbling-block"  n ( s t uh1 m b l i ng - b l o k ) ((pos "K6%" ) ))
( "stumbling-blocks"  n ( s t uh1 m b l i ng - b l o k s ) ((pos "Kj%" ) ))
( "stump"  v ( s t uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "stumped"  v ( s t uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "stumper"  n ( s t uh1 m p @ r ) ((pos "K6%" ) ))
( "stumpers"  n ( s t uh1 m p @ z ) ((pos "Kj%" ) ))
( "stumpier"  j ( s t uh1 m p i@ r ) ((pos "Or$" ) ))
( "stumpiest"  j ( s t uh1 m p i i s t ) ((pos "Os$" ) ))
( "stumping"  v ( s t uh1 m p i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "stumps"  v ( s t uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "stumpy"  j ( s t uh1 m p ii ) ((pos "OD%" ) ))
( "stun"  v ( s t uh1 n ) ((pos "H4%" ) (subcat )))
( "stung"  v ( s t uh1 g ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "stunk"  v ( s t uh1 ng k ) ((pos "Jc$" "Jd$" ) (subcat "2A" "3A" )))
( "stunned"  v ( s t uh1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stunner"  n ( s t uh1 n @ r ) ((pos "K6%" ) ))
( "stunners"  n ( s t uh1 n @ z ) ((pos "Kj%" ) ))
( "stunning"  v ( s t uh1 n i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "stunningly"  a ( s t uh1 n i ng l ii ) ((pos "Pu%" ) ))
( "stuns"  v ( s t uh1 n z ) ((pos "Ha%" ) (subcat )))
( "stunt"  v ( s t uh1 n t ) ((pos "H0%" "K6%" ) (subcat )))
( "stunted"  v ( s t uh1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stunting"  v ( s t uh1 n t i ng ) ((pos "Hb$" ) (subcat )))
( "stunts"  v ( s t uh1 n t s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "stupefaction"  n ( s t y uu2 p i f a1 k sh @ n ) ((pos "L@%" ) ))
( "stupefied"  v ( s t y uu1 p i f ai d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "stupefies"  v ( s t y uu1 p i f ai z ) ((pos "Ha$" ) (subcat )))
( "stupefy"  v ( s t y uu1 p i f ai ) ((pos "H3$" ) (subcat )))
( "stupefying"  v ( s t y uu1 p i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "stupendous"  j ( s t y uu1 p e1 n d @ s ) ((pos "OA%" ) ))
( "stupendously"  a ( s t y uu1 p e1 n d @ s l ii ) ((pos "Pu%" ) ))
( "stupid"  n ( s t y uu1 p i d ) ((pos "Ki%" "OA%" ) ))
( "stupidities"  n ( s t y uu1 p i1 d i t i z ) ((pos "Mj%" ) ))
( "stupidity"  n ( s t y uu1 p i1 d i t ii ) ((pos "M8%" ) ))
( "stupidly"  a ( s t y uu1 p i d l ii ) ((pos "Pu%" ) ))
( "stupified"  v ( s t y uu1 p i f ai d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "stupifies"  v ( s t y uu1 p i f ai z ) ((pos "Ha$" ) (subcat )))
( "stupify"  v ( s t y uu1 p i f ai ) ((pos "H3$" ) (subcat )))
( "stupifying"  v ( s t y uu1 p i f ai i ng ) ((pos "Hb$" "OA%" ) (subcat )))
( "stupor"  n ( s t y uu1 p @ r ) ((pos "M6%" ) ))
( "stupors"  n ( s t y uu1 p @ z ) ((pos "Mj$" ) ))
( "sturdier"  j ( s t @@1 d i@ r ) ((pos "Or%" ) ))
( "sturdiest"  j ( s t @@1 d i i s t ) ((pos "Os%" ) ))
( "sturdily"  a ( s t @@1 d i l ii ) ((pos "Pu%" ) ))
( "sturdiness"  n ( s t @@1 d i n @ s ) ((pos "L@%" ) ))
( "sturdy"  j ( s t @@1 d ii ) ((pos "OD%" ) ))
( "sturgeon"  n ( s t @@1 jh @ n ) ((pos "M6$" ) ))
( "sturgeons"  n ( s t @@1 jh @ n z ) ((pos "Mj$" ) ))
( "stutter"  v ( s t uh1 t @ r ) ((pos "J0%" "K6%" ) ))
( "stuttered"  v ( s t uh1 t @ d ) ((pos "Jc%" "Jd%" ) ))
( "stutterer"  n ( s t uh1 t @ r @ r ) ((pos "K6%" ) ))
( "stutterers"  n ( s t uh1 t @ r @ z ) ((pos "Kj%" ) ))
( "stuttering"  v ( s t uh1 t @ r i ng ) ((pos "Jb%" ) ))
( "stutteringly"  a ( s t uh1 t @ r i ng l ii ) ((pos "Pu%" ) ))
( "stutters"  v ( s t uh1 t @ z ) ((pos "Ja%" "Kj%" ) ))
( "sty"  n ( s t ai1 ) ((pos "K8%" ) ))
( "stye"  n ( s t ai1 ) ((pos "K6%" ) ))
( "styes"  n ( s t ai1 z ) ((pos "Kj%" ) ))
( "style"  v ( s t ai1 l ) ((pos "H2%" "M6%" ) (subcat )))
( "styled"  v ( s t ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "styles"  v ( s t ai1 l z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "styling"  v ( s t ai1 l i ng ) ((pos "Hb%" ) (subcat )))
( "stylish"  j ( s t ai1 l i sh ) ((pos "OA%" ) ))
( "stylishly"  a ( s t ai1 l i sh l ii ) ((pos "Pu%" ) ))
( "stylishness"  n ( s t ai1 l i sh n @ s ) ((pos "L@%" ) ))
( "stylist"  n ( s t ai1 l i s t ) ((pos "K6%" ) ))
( "stylistic"  j ( s t ai1 l i1 s t i k ) ((pos "OA%" ) ))
( "stylistically"  a ( s t ai1 l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "stylists"  n ( s t ai1 l i s t s ) ((pos "Kj%" ) ))
( "stylization"  n ( s t ai2 @ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "stylizations"  n ( s t ai2 @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "stylize"  v ( s t ai1 @ l ai z ) ((pos "H2%" ) ))
( "stylized"  v ( s t ai1 @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "stylizes"  v ( s t ai1 @ l ai z i z ) ((pos "Ha$" ) ))
( "stylizing"  v ( s t ai1 @ l ai z i ng ) ((pos "Hb$" ) ))
( "stylus"  n ( s t ai1 l @ s ) ((pos "K7%" ) ))
( "styluses"  n ( s t ai1 l @ s i z ) ((pos "Kj%" ) ))
( "stymie"  v ( s t ai1 m ii ) ((pos "H5%" "K6$" ) (subcat )))
( "stymied"  v ( s t ai1 m i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "stymies"  v ( s t ai1 m i z ) ((pos "Ha%" "Kj$" ) (subcat )))
( "stymying"  v ( s t ai1 m i i ng ) ((pos "Hb$" ) (subcat )))
( "styptic"  n ( s t i1 p t i k ) ((pos "K6$" "OA%" ) ))
( "styptics"  n ( s t i1 p t i k s ) ((pos "Kj$" ) ))
( "suasion"  n ( s w ei1 zh n ) ((pos "L@$" ) ))
( "suave"  j ( s w aa1 v ) ((pos "OA%" ) ))
( "suavely"  a ( s w aa1 v l ii ) ((pos "Pu%" ) ))
( "suavity"  n ( s w aa1 v i t ii ) ((pos "L@%" ) ))
( "sub"  v ( s uh1 b ) ((pos "J4%" "K6%" ) (subcat "2A" "3A" )))
( "sub judice"  j ( s uh2 b - jh uu1 d i s ii ) ((pos "OA%" ) ))
( "sub rosa"  a ( s uh2 b - r ou1 z @ ) ((pos "Pu$" ) ))
( "sub-"  j ( s uh b - ) ((pos "U-%" ) ))
( "subaltern"  n ( s uh1 b l t @ n ) ((pos "K6%" ) ))
( "subalterns"  n ( s uh1 b l t @ n z ) ((pos "Kj%" ) ))
( "subatomic"  j ( s uh2 b @ t o1 m i k ) ((pos "OA%" ) ))
( "subbed"  v ( s uh1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "subbing"  v ( s uh1 b i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "subcommittee"  n ( s uh1 b k @ m i t ii ) ((pos "K6%" ) ))
( "subcommittees"  n ( s uh1 b k @ m i t i z ) ((pos "Kj%" ) ))
( "subconscious"  n ( s uh2 b k o1 n sh @ s ) ((pos "Ki%" "OA%" ) ))
( "subconsciously"  a ( s uh2 b k o1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "subconsciousness"  n ( s uh2 b k o1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "subcontinent"  n ( s uh2 b k o1 n t i n @ n t ) ((pos "K6%" ) ))
( "subcontinents"  n ( s uh2 b k o1 n t i n @ n t s ) ((pos "Kj%" ) ))
( "subcontract"  n ( s uh2 b k o1 n t r a k t ) ((pos "K6$" ) ))
( "subcontract"  v ( s uh2 b k @ n t r a1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "subcontracted"  v ( s uh2 b k @ n t r a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "subcontracting"  v ( s uh2 b k @ n t r a1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "subcontractor"  n ( s uh2 b k @ n t r a1 k t @ r ) ((pos "K6%" ) ))
( "subcontractors"  n ( s uh2 b k @ n t r a1 k t @ z ) ((pos "Kj%" ) ))
( "subcontracts"  n ( s uh2 b k o1 n t r a k t s ) ((pos "Kj$" ) ))
( "subcontracts"  v ( s uh2 b k @ n t r a1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "subcutaneous"  j ( s uh2 b k y uu t ei1 n i@ s ) ((pos "OA$" ) ))
( "subdivide"  v ( s uh2 b d i v ai1 d ) ((pos "J2%" ) (subcat "2A" )))
( "subdivided"  v ( s uh2 b d i v ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "subdivides"  v ( s uh2 b d i v ai1 d z ) ((pos "Ja%" ) (subcat "2A" )))
( "subdividing"  v ( s uh2 b d i v ai1 d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "subdivision"  n ( s uh2 b d i v i1 zh n ) ((pos "M6%" ) ))
( "subdivisions"  n ( s uh2 b d i v i1 zh n z ) ((pos "Mj%" ) ))
( "subdue"  v ( s @1 b d y uu1 ) ((pos "H2%" ) (subcat )))
( "subdued"  v ( s @1 b d y uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "subdues"  v ( s @1 b d y uu1 z ) ((pos "Ha%" ) (subcat )))
( "subduing"  v ( s @1 b d y uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "subedit"  v ( s uh1 b e1 d i t ) ((pos "H0%" ) ))
( "subedited"  v ( s uh1 b e1 d i t i d ) ((pos "Hc%" "Hd%" ) ))
( "subediting"  v ( s uh1 b e1 d i t i ng ) ((pos "Hb%" ) ))
( "subeditor"  n ( s uh1 b e1 d i t @ r ) ((pos "K6%" ) ))
( "subeditors"  n ( s uh1 b e1 d i t @ z ) ((pos "Kj%" ) ))
( "subedits"  v ( s uh1 b e1 d i t s ) ((pos "Ha%" ) ))
( "subfusc"  j ( s uh1 b f uh s k ) ((pos "OA$" ) ))
( "subgroup"  n ( s uh1 b g r uu p ) ((pos "K6%" ) ))
( "subgroups"  n ( s uh1 b g r uu p s ) ((pos "Kj%" ) ))
( "subheading"  n ( s uh1 b h e d i ng ) ((pos "K6%" ) ))
( "subheadings"  n ( s uh1 b h e d i ng z ) ((pos "Kj%" ) ))
( "subhuman"  j ( s uh2 b h y uu1 m @ n ) ((pos "OA%" ) ))
( "subject"  n ( s uh1 b jh i k t ) ((pos "K6%" "OA%" ) ))
( "subject"  v ( s @1 b jh e1 k t ) ((pos "H0%" ) (subcat )))
( "subjected"  v ( s @1 b jh e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "subjecting"  v ( s @1 b jh e1 k t i ng ) ((pos "Hb%" ) (subcat )))
( "subjection"  n ( s @1 b jh e1 k sh @ n ) ((pos "L@%" ) ))
( "subjective"  j ( s @1 b jh e1 k t i v ) ((pos "OA%" ) ))
( "subjectively"  a ( s @1 b jh e1 k t i v l ii ) ((pos "Pu%" ) ))
( "subjectivity"  n ( s uh2 b jh e k t i1 v i t ii ) ((pos "L@%" ) ))
( "subjects"  n ( s uh1 b jh i k t s ) ((pos "Kj%" ) ))
( "subjects"  v ( s @1 b jh e1 k t s ) ((pos "Ha%" ) (subcat )))
( "subjoin"  v ( s uh1 b jh oi1 n ) ((pos "H0$" ) (subcat )))
( "subjoined"  v ( s uh1 b jh oi1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "subjoining"  v ( s uh1 b jh oi1 n i ng ) ((pos "Hb$" ) (subcat )))
( "subjoins"  v ( s uh1 b jh oi1 n z ) ((pos "Ha$" ) (subcat )))
( "subjugate"  v ( s uh1 b jh u g ei t ) ((pos "H2%" ) (subcat )))
( "subjugated"  v ( s uh1 b jh u g ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "subjugates"  v ( s uh1 b jh u g ei t s ) ((pos "Ha%" ) (subcat )))
( "subjugating"  v ( s uh1 b jh u g ei t i ng ) ((pos "Hb%" ) (subcat )))
( "subjugation"  n ( s uh2 b jh u g ei1 sh @ n ) ((pos "M6%" ) ))
( "subjugations"  n ( s uh2 b jh u g ei1 sh @ n z ) ((pos "Mj%" ) ))
( "subjunctive"  n ( s @1 b jh uh1 ng k t i v ) ((pos "M6%" "OA%" ) ))
( "subjunctives"  n ( s @1 b jh uh1 ng k t i v z ) ((pos "Mj%" ) ))
( "sublease"  v ( s uh2 b l ii1 s ) ((pos "J2$" "K6$" ) (subcat "2A" )))
( "subleased"  v ( s uh2 b l ii1 s t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "subleases"  v ( s uh2 b l ii1 s i z ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "subleasing"  v ( s uh2 b l ii1 s i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "sublet"  v ( s uh2 b l e1 t ) ((pos "J5%" "Jc%" "Jd%" ) (subcat "2A" )))
( "sublets"  v ( s uh2 b l e1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "subletting"  v ( s uh2 b l e1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "sublieutenant"  n ( s uh2 b l @ t e1 n @ n t ) ((pos "K6%" ) ))
( "sublieutenants"  n ( s uh2 b l @ t e1 n @ n t s ) ((pos "Kj%" ) ))
( "sublimate"  v ( s uh1 b l i m ei t ) ((pos "H2%" "K6$" "OA$" ) (subcat )))
( "sublimated"  v ( s uh1 b l i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sublimates"  v ( s uh1 b l i m ei t s ) ((pos "Ha%" "Kj$" ) (subcat )))
( "sublimating"  v ( s uh1 b l i m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "sublimation"  n ( s uh2 b l i m ei1 sh @ n ) ((pos "M6%" ) ))
( "sublimations"  n ( s uh2 b l i m ei1 sh @ n z ) ((pos "Mj$" ) ))
( "sublime"  n ( s @1 b l ai1 m ) ((pos "Ki%" "OA%" ) ))
( "sublimely"  a ( s @1 b l ai1 m l ii ) ((pos "Pu%" ) ))
( "subliminal"  j ( s uh2 b l i1 m i n l ) ((pos "OA%" ) ))
( "sublimity"  n ( s @1 b l i1 m i t ii ) ((pos "L@$" ) ))
( "submarine"  n ( s uh2 b m @ r ii1 n ) ((pos "K6%" "OA$" ) ))
( "submariner"  n ( s uh1 b m a1 r i n @ r ) ((pos "K6%" ) ))
( "submariners"  n ( s uh1 b m a1 r i n @ z ) ((pos "Kj%" ) ))
( "submarines"  n ( s uh2 b m @ r ii1 n z ) ((pos "Kj%" ) ))
( "submerge"  v ( s @1 b m @@1 jh ) ((pos "J2%" ) (subcat "2A" )))
( "submerged"  v ( s @1 b m @@1 jh d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "submergence"  n ( s @1 b m @@1 jh @ n s ) ((pos "L@$" ) ))
( "submerges"  v ( s @1 b m @@1 jh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "submerging"  v ( s @1 b m @@1 jh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "submersible"  j ( s @1 b m @@1 s @ b l ) ((pos "OA%" ) ))
( "submersion"  n ( s @1 b m @@1 sh @ n ) ((pos "L@%" ) ))
( "submission"  n ( s @1 b m i1 sh @ n ) ((pos "M6%" ) ))
( "submissions"  n ( s @1 b m i1 sh @ n z ) ((pos "Mj%" ) ))
( "submissive"  j ( s @1 b m i1 s i v ) ((pos "OA%" ) ))
( "submissively"  a ( s @1 b m i1 s i v l ii ) ((pos "Pu%" ) ))
( "submissiveness"  n ( s @1 b m i1 s i v n @ s ) ((pos "L@%" ) ))
( "submit"  v ( s @1 b m i1 t ) ((pos "J4%" ) (subcat "3A" "6A" "9" )))
( "submits"  v ( s @1 b m i1 t s ) ((pos "Ja%" ) (subcat "3A" "6A" "9" )))
( "submitted"  v ( s @1 b m i1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" )))
( "submitting"  v ( s @1 b m i1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "9" )))
( "subnormal"  n ( s uh2 b n oo1 m l ) ((pos "K6$" "OA%" ) ))
( "subnormals"  n ( s uh2 b n oo1 m l z ) ((pos "Kj$" ) ))
( "suborbital"  j ( s uh2 b oo1 b i t l ) ((pos "OA$" ) ))
( "subordinate"  n ( s @1 b oo1 d i n @ t ) ((pos "K6%" "OA%" ) ))
( "subordinate"  v ( s @1 b oo1 d i n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "subordinated"  v ( s @1 b oo1 d i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "subordinates"  n ( s @1 b oo1 d i n @ t s ) ((pos "Kj%" ) ))
( "subordinates"  v ( s @1 b oo1 d i n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "subordinating"  v ( s @1 b oo1 d i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "subordination"  n ( s @1 b oo2 d i n ei1 sh @ n ) ((pos "M6%" ) ))
( "subordinations"  n ( s @1 b oo2 d i n ei1 sh @ n z ) ((pos "Mj$" ) ))
( "subordinative"  j ( s @1 b oo1 d i n @ t i v ) ((pos "OA$" ) ))
( "suborn"  v ( s @1 b oo1 n ) ((pos "H0$" ) (subcat )))
( "subornation"  n ( s uh2 b oo n ei1 sh @ n ) ((pos "L@$" ) ))
( "suborned"  v ( s @1 b oo1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "suborning"  v ( s @1 b oo1 n i ng ) ((pos "Hb$" ) (subcat )))
( "suborns"  v ( s @1 b oo1 n z ) ((pos "Ha$" ) (subcat )))
( "subpoena"  v ( s @1 p ii1 n @ ) ((pos "H0%" "K6%" ) (subcat )))
( "subpoenaed"  v ( s @1 p ii1 n @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "subpoenaing"  v ( s @1 p ii1 n @ r i ng ) ((pos "Hb$" ) (subcat )))
( "subpoenas"  v ( s @1 p ii1 n @ z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "subs"  v ( s uh1 b z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" )))
( "subscribe"  v ( s @1 b s k r ai1 b ) ((pos "J2%" ) (subcat "2A" "3A" "6A" )))
( "subscribed"  v ( s @1 b s k r ai1 b d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "subscriber"  n ( s @1 b s k r ai1 b @ r ) ((pos "K6%" ) ))
( "subscribers"  n ( s @1 b s k r ai1 b @ z ) ((pos "Kj%" ) ))
( "subscribes"  v ( s @1 b s k r ai1 b z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "subscribing"  v ( s @1 b s k r ai1 b i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "subscript"  n ( s uh1 b s k r i p t ) ((pos "K6%" ) ))
( "subscription"  n ( s @1 b s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "subscriptions"  n ( s @1 b s k r i1 p sh @ n z ) ((pos "Mj%" ) ))
( "subscripts"  n ( s uh1 b s k r i p t s ) ((pos "Kj%" ) ))
( "subsection"  n ( s uh1 b s e k sh @ n ) ((pos "K6%" ) ))
( "subsections"  n ( s uh1 b s e k sh @ n z ) ((pos "Kj%" ) ))
( "subsequent"  j ( s uh1 b s i k w @ n t ) ((pos "OA%" ) ))
( "subsequently"  a ( s uh1 b s i k w @ n t l ii ) ((pos "Pu%" ) ))
( "subserve"  v ( s @1 b s @@1 v ) ((pos "H2$" ) (subcat )))
( "subserved"  v ( s @1 b s @@1 v d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "subserves"  v ( s @1 b s @@1 v z ) ((pos "Ha$" ) (subcat )))
( "subservience"  n ( s @1 b s @@1 v i@ n s ) ((pos "L@%" ) ))
( "subservient"  j ( s @1 b s @@1 v i@ n t ) ((pos "OA%" ) ))
( "subserviently"  a ( s @1 b s @@1 v i@ n t l ii ) ((pos "Pu%" ) ))
( "subserving"  v ( s @1 b s @@1 v i ng ) ((pos "Hb$" ) (subcat )))
( "subside"  v ( s @1 b s ai1 d ) ((pos "I2%" ) (subcat )))
( "subsided"  v ( s @1 b s ai1 d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "subsidence"  n ( s @1 b s ai1 d n s ) ((pos "M6%" ) ))
( "subsidences"  n ( s @1 b s ai1 d n s i z ) ((pos "Mj$" ) ))
( "subsides"  v ( s @1 b s ai1 d z ) ((pos "Ia%" ) (subcat )))
( "subsidiaries"  n ( s @1 b s i1 d i@ r i z ) ((pos "Kj%" ) ))
( "subsidiary"  n ( s @1 b s i1 d i@ r ii ) ((pos "K8%" "OA%" ) ))
( "subsidies"  n ( s uh1 b s @ d i z ) ((pos "Kj%" ) ))
( "subsiding"  v ( s @1 b s ai1 d i ng ) ((pos "Ib%" ) (subcat )))
( "subsidization"  n ( s uh2 b s i d ai z ei1 sh @ n ) ((pos "L@$" ) ))
( "subsidize"  v ( s uh1 b s i d ai z ) ((pos "H2%" ) (subcat )))
( "subsidized"  v ( s uh1 b s i d ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "subsidizes"  v ( s uh1 b s i d ai z i z ) ((pos "Ha%" ) (subcat )))
( "subsidizing"  v ( s uh1 b s i d ai z i ng ) ((pos "Hb%" ) (subcat )))
( "subsidy"  n ( s uh1 b s @ d ii ) ((pos "K8%" ) ))
( "subsist"  v ( s @1 b s i1 s t ) ((pos "I0%" ) (subcat "2A" )))
( "subsisted"  v ( s @1 b s i1 s t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "subsistence"  n ( s @1 b s i1 s t @ n s ) ((pos "L@%" ) ))
( "subsisting"  v ( s @1 b s i1 s t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "subsists"  v ( s @1 b s i1 s t s ) ((pos "Ia%" ) (subcat "2A" )))
( "subsoil"  n ( s uh1 b s oi l ) ((pos "L@%" ) ))
( "subsonic"  j ( s uh2 b s o1 n i k ) ((pos "OA%" ) ))
( "substance"  n ( s uh1 b s t @ n s ) ((pos "M6%" ) ))
( "substances"  n ( s uh1 b s t @ n s i z ) ((pos "Mj%" ) ))
( "substandard"  j ( s uh2 b s t a1 n d @ d ) ((pos "OA%" ) ))
( "substantial"  j ( s @1 b s t a1 n sh l ) ((pos "OA%" ) ))
( "substantially"  a ( s @1 b s t a1 n sh @ l ii ) ((pos "Pu%" ) ))
( "substantiate"  v ( s @1 b s t a1 n sh i ei t ) ((pos "H2%" ) (subcat )))
( "substantiated"  v ( s @1 b s t a1 n sh i ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "substantiates"  v ( s @1 b s t a1 n sh i ei t s ) ((pos "Ha%" ) (subcat )))
( "substantiating"  v ( s @1 b s t a1 n sh i ei t i ng ) ((pos "Hb%" ) (subcat )))
( "substantiation"  n ( s @1 b s t a2 n sh i ei1 sh @ n ) ((pos "M6%" ) ))
( "substantiations"  n ( s @1 b s t a2 n sh i ei1 sh @ n z ) ((pos "Mj$" ) ))
( "substantival"  j ( s uh2 b s t @ n t ai1 v @ l ) ((pos "OA$" ) ))
( "substantive"  n ( s uh1 b s t @ n t i v ) ((pos "K6$" "OA$" ) ))
( "substantives"  n ( s uh1 b s t @ n t i v z ) ((pos "Kj$" ) ))
( "substation"  n ( s uh1 b s t ei sh @ n ) ((pos "K6%" ) ))
( "substations"  n ( s uh1 b s t ei sh @ n z ) ((pos "Kj%" ) ))
( "substitute"  v ( s uh1 b s t i t y uu t ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" )))
( "substituted"  v ( s uh1 b s t i t y uu t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "substitutes"  v ( s uh1 b s t i t y uu t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" )))
( "substituting"  v ( s uh1 b s t i t y uu t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "substitution"  n ( s uh2 b s t i t y uu1 sh @ n ) ((pos "M6%" ) ))
( "substitutions"  n ( s uh2 b s t i t y uu1 sh @ n z ) ((pos "Mj%" ) ))
( "substrata"  n ( s uh2 b s t r aa1 t @ ) ((pos "Kj$" ) ))
( "substrate"  n ( s uh1 b s t r ei t ) ((pos "K6$" ) ))
( "substrates"  n ( s uh1 b s t r ei t s ) ((pos "Kj$" ) ))
( "substratum"  n ( s uh2 b s t r aa1 t @ m ) ((pos "Ki$" ) ))
( "substructure"  n ( s uh1 b s t r uh k ch @ r ) ((pos "K6%" ) ))
( "substructures"  n ( s uh1 b s t r uh k ch @ z ) ((pos "Kj%" ) ))
( "subsume"  v ( s @1 b s y uu1 m ) ((pos "H2%" ) (subcat "6A" )))
( "subsumed"  v ( s @1 b s y uu1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "subsumes"  v ( s @1 b s y uu1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "subsuming"  v ( s @1 b s y uu1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "subtend"  v ( s @1 b t e1 n d ) ((pos "H0$" ) ))
( "subtended"  v ( s @1 b t e1 n d i d ) ((pos "Hc$" "Hd$" ) ))
( "subtending"  v ( s @1 b t e1 n d i ng ) ((pos "Hb$" ) ))
( "subtends"  v ( s @1 b t e1 n d z ) ((pos "Ha$" ) ))
( "subterfuge"  n ( s uh1 b t @ f y uu jh ) ((pos "M6%" ) ))
( "subterfuges"  n ( s uh1 b t @ f y uu jh i z ) ((pos "Mj%" ) ))
( "subterranean"  j ( s uh2 b t @ r ei1 n i@ n ) ((pos "OA%" ) ))
( "subtitle"  n ( s uh1 b t ai t l ) ((pos "K6%" ) ))
( "subtitles"  n ( s uh1 b t ai t l z ) ((pos "Kj%" ) ))
( "subtle"  j ( s uh1 t l ) ((pos "OB%" ) ))
( "subtler"  j ( s uh1 t l @ r ) ((pos "Or%" ) ))
( "subtlest"  j ( s uh1 t l i s t ) ((pos "Os%" ) ))
( "subtleties"  n ( s uh1 t l t i z ) ((pos "Mj%" ) ))
( "subtlety"  n ( s uh1 t l t ii ) ((pos "M8%" ) ))
( "subtly"  a ( s uh1 t l ii ) ((pos "Pu%" ) ))
( "subtopia"  n ( s uh2 b t ou1 p i@ ) ((pos "L@$" ) ))
( "subtract"  v ( s @1 b t r a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "subtracted"  v ( s @1 b t r a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "subtracting"  v ( s @1 b t r a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "subtraction"  n ( s @1 b t r a1 k sh @ n ) ((pos "M6%" ) ))
( "subtractions"  n ( s @1 b t r a1 k sh @ n z ) ((pos "Mj%" ) ))
( "subtracts"  v ( s @1 b t r a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "subtropical"  j ( s uh2 b t r o1 p i k l ) ((pos "OA%" ) ))
( "suburb"  n ( s uh1 b @@ b ) ((pos "K6%" ) ))
( "suburban"  j ( s @1 b @@1 b @ n ) ((pos "OA%" ) ))
( "suburbia"  n ( s @1 b @@1 b i@ ) ((pos "L@%" ) ))
( "suburbs"  n ( s uh1 b @@ b z ) ((pos "Kj%" ) ))
( "subvention"  n ( s @1 b v e1 n sh @ n ) ((pos "K6%" ) ))
( "subventions"  n ( s @1 b v e1 n sh @ n z ) ((pos "Kj%" ) ))
( "subversion"  n ( s @1 b v @@1 sh @ n ) ((pos "L@%" ) ))
( "subversive"  n ( s @1 b v @@1 s i v ) ((pos "K6%" "OA%" ) ))
( "subversives"  n ( s @1 b v @@1 s i v z ) ((pos "Kj%" ) ))
( "subvert"  v ( s uh1 b v @@1 t ) ((pos "H0%" ) (subcat )))
( "subverted"  v ( s uh1 b v @@1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "subverting"  v ( s uh1 b v @@1 t i ng ) ((pos "Hb%" ) (subcat )))
( "subverts"  v ( s uh1 b v @@1 t s ) ((pos "Ha%" ) (subcat )))
( "subway"  n ( s uh1 b w ei ) ((pos "K6%" ) ))
( "subways"  n ( s uh1 b w ei z ) ((pos "Kj%" ) ))
( "succeed"  v ( s @1 k s ii1 d ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "succeeded"  v ( s @1 k s ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "succeeding"  v ( s @1 k s ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "succeeds"  v ( s @1 k s ii1 d z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "success"  n ( s @1 k s e1 s ) ((pos "M7%" ) ))
( "successes"  n ( s @1 k s e1 s i z ) ((pos "Mj%" ) ))
( "successful"  j ( s @1 k s e1 s f @ l ) ((pos "OA%" ) ))
( "successfully"  a ( s @1 k s e1 s f @ l ii ) ((pos "Pu%" ) ))
( "succession"  n ( s @1 k s e1 sh @ n ) ((pos "M6%" ) ))
( "successions"  n ( s @1 k s e1 sh @ n z ) ((pos "Mj$" ) ))
( "successive"  j ( s @1 k s e1 s i v ) ((pos "OA%" ) ))
( "successively"  a ( s @1 k s e1 s i v l ii ) ((pos "Pu%" ) ))
( "successor"  n ( s @1 k s e1 s @ r ) ((pos "K6%" ) ))
( "successors"  n ( s @1 k s e1 s @ z ) ((pos "Kj%" ) ))
( "succinct"  j ( s @1 k s i1 ng k t ) ((pos "OA%" ) ))
( "succinctly"  a ( s @1 k s i1 ng k t l ii ) ((pos "Pu%" ) ))
( "succinctness"  n ( s @1 k s i1 ng k t n @ s ) ((pos "L@%" ) ))
( "succour"  v ( s uh1 k @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "succoured"  v ( s uh1 k @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "succouring"  v ( s uh1 k @ r i ng ) ((pos "Hb%" ) (subcat )))
( "succours"  v ( s uh1 k @ z ) ((pos "Ha%" ) (subcat )))
( "succubus"  n ( s uh1 k y u b @ s ) ((pos "K7$" ) ))
( "succubuses"  n ( s uh1 k y u b @ s i z ) ((pos "Kj$" ) ))
( "succulence"  n ( s uh1 k y u l @ n s ) ((pos "L@%" ) ))
( "succulent"  n ( s uh1 k y u l @ n t ) ((pos "K6%" "OA%" ) ))
( "succulents"  n ( s uh1 k y u l @ n t s ) ((pos "Kj%" ) ))
( "succumb"  v ( s @1 k uh1 m ) ((pos "I0%" ) (subcat "2A" )))
( "succumbed"  v ( s @1 k uh1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "succumbing"  v ( s @1 k uh1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "succumbs"  v ( s @1 k uh1 m z ) ((pos "Ia%" ) (subcat "2A" )))
( "such"  j ( s uh ch ) ((pos "OA*" "Qx*" ) ))
( "suchlike"  j ( s uh1 ch l ai k ) ((pos "OA%" ) ))
( "suck"  v ( s uh1 k ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" "15A" "15B" )))
( "sucked"  v ( s uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" "15B" )))
( "sucker"  n ( s uh1 k @ r ) ((pos "K6%" ) ))
( "suckers"  n ( s uh1 k @ z ) ((pos "Kj%" ) ))
( "sucking"  v ( s uh1 k i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" "15B" )))
( "sucking-pig"  n ( s uh1 k i ng - p i g ) ((pos "M6%" ) ))
( "sucking-pigs"  n ( s uh1 k i ng - p i g z ) ((pos "Mj%" ) ))
( "suckle"  v ( s uh1 k l ) ((pos "H2%" ) (subcat )))
( "suckled"  v ( s uh1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "suckles"  v ( s uh1 k l z ) ((pos "Ha%" ) (subcat )))
( "suckling"  v ( s uh1 k l i ng ) ((pos "Hb%" "K6$" ) (subcat )))
( "sucklings"  n ( s uh1 k l i ng z ) ((pos "Kj$" ) ))
( "sucks"  v ( s uh1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" "15B" )))
( "suction"  n ( s uh1 k sh @ n ) ((pos "L@%" ) ))
( "sudden"  n ( s uh1 d n ) ((pos "Ki%" "OA%" ) ))
( "suddenly"  a ( s uh1 d n l ii ) ((pos "Pu%" ) ))
( "suddenness"  n ( s uh1 d n - n @ s ) ((pos "L@%" ) ))
( "suds"  n ( s uh1 d z ) ((pos "Kj%" ) ))
( "sue"  v ( s y uu1 ) ((pos "J2%" ) (subcat "3A" "6A" )))
( "sued"  v ( s y uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "suede"  n ( s w ei1 d ) ((pos "L@%" ) ))
( "sues"  v ( s y uu1 z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "suet"  n ( s uu1 i t ) ((pos "L@%" ) ))
( "suety"  j ( s uu1 i t ii ) ((pos "OA$" ) ))
( "suffer"  v ( s uh1 f @ r ) ((pos "J0%" ) (subcat "2A" "3A" "6A" )))
( "sufferable"  j ( s uh1 f @ r @ b l ) ((pos "OA%" ) ))
( "sufferance"  n ( s uh1 f @ r @ n s ) ((pos "L@%" ) ))
( "suffered"  v ( s uh1 f @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "sufferer"  n ( s uh1 f @ r @ r ) ((pos "K6%" ) ))
( "sufferers"  n ( s uh1 f @ r @ z ) ((pos "Kj%" ) ))
( "suffering"  v ( s uh1 f @ r i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "3A" "6A" )))
( "sufferings"  n ( s uh1 f @ r i ng z ) ((pos "Mj%" ) ))
( "suffers"  v ( s uh1 f @ z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "suffice"  v ( s @1 f ai1 s ) ((pos "J2%" ) (subcat "2A" "3A" )))
( "sufficed"  v ( s @1 f ai1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" )))
( "suffices"  v ( s @1 f ai1 s i z ) ((pos "Ja%" ) (subcat "2A" "3A" )))
( "sufficiency"  n ( s @1 f i1 sh @ n s ii ) ((pos "L@%" ) ))
( "sufficient"  j ( s @1 f i1 sh @ n t ) ((pos "OA%" ) ))
( "sufficiently"  a ( s @1 f i1 sh @ n t l ii ) ((pos "Pu%" ) ))
( "sufficing"  v ( s @1 f ai1 s i ng ) ((pos "Jb%" ) (subcat "2A" "3A" )))
( "suffix"  n ( s uh1 f i k s ) ((pos "K7%" ) ))
( "suffixes"  n ( s uh1 f i k s i z ) ((pos "Kj%" ) ))
( "suffocate"  v ( s uh1 f @ k ei t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "suffocated"  v ( s uh1 f @ k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "suffocates"  v ( s uh1 f @ k ei t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "suffocating"  v ( s uh1 f @ k ei t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "suffocation"  n ( s uh2 f @ k ei1 sh @ n ) ((pos "L@%" ) ))
( "suffragan"  n ( s uh1 f r @ g @ n ) ((pos "K6$" ) ))
( "suffragans"  n ( s uh1 f r @ g @ n z ) ((pos "Kj$" ) ))
( "suffrage"  n ( s uh1 f r i jh ) ((pos "M6%" ) ))
( "suffrages"  n ( s uh1 f r i jh i z ) ((pos "Mj$" ) ))
( "suffragette"  n ( s uh2 f r @ jh e1 t ) ((pos "K6%" ) ))
( "suffragettes"  n ( s uh2 f r @ jh e1 t s ) ((pos "Kj%" ) ))
( "suffuse"  v ( s @1 f y uu1 z ) ((pos "H2%" ) (subcat )))
( "suffused"  v ( s @1 f y uu1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "suffuses"  v ( s @1 f y uu1 z i z ) ((pos "Ha%" ) (subcat )))
( "suffusing"  v ( s @1 f y uu1 z i ng ) ((pos "Hb%" ) (subcat )))
( "suffusion"  n ( s @1 f y uu1 zh n ) ((pos "L@%" ) ))
( "sugar"  v ( sh u1 g @ r ) ((pos "H0%" "M6%" ) (subcat )))
( "sugar-beet"  n ( sh u1 g @ - b ii t ) ((pos "M9%" ) ))
( "sugar-candies"  n ( sh u2 g @ - k a1 n d i z ) ((pos "Mj$" ) ))
( "sugar-candy"  n ( sh u2 g @ - k a1 n d ii ) ((pos "M8%" ) ))
( "sugar-cane"  n ( sh u1 g @ - k ei n ) ((pos "L@%" ) ))
( "sugar-coated"  j ( sh u1 g @ - k ou t i d ) ((pos "OA%" ) ))
( "sugar-daddies"  n ( sh u1 g @ - d a d i z ) ((pos "Kj%" ) ))
( "sugar-daddy"  n ( sh u1 g @ - d a d ii ) ((pos "K8%" ) ))
( "sugar-loaf"  n ( sh u1 g @ - l ou f ) ((pos "L@%" ) ))
( "sugar-refineries"  n ( sh u1 g @ - r i f ai n @ r i z ) ((pos "Kj%" ) ))
( "sugar-refinery"  n ( sh u1 g @ - r i f ai n @ r ii ) ((pos "K8%" ) ))
( "sugar-tongs"  n ( sh u1 g @ - t o ng z ) ((pos "Kj%" ) ))
( "sugared"  v ( sh u1 g @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sugarier"  j ( sh u1 g @ r i@ r ) ((pos "Or%" ) ))
( "sugariest"  j ( sh u1 g @ r i i s t ) ((pos "Os%" ) ))
( "sugaring"  v ( sh u1 g @ r i ng ) ((pos "Hb%" ) (subcat )))
( "sugarlump"  n ( sh u1 g @ l uh m p ) ((pos "K6%" ) ))
( "sugarlumps"  n ( sh u1 g @ l uh m p s ) ((pos "Kj%" ) ))
( "sugars"  v ( sh u1 g @ z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sugary"  j ( sh u1 g @ r ii ) ((pos "OD%" ) ))
( "suggest"  v ( s @1 jh e1 s t ) ((pos "H0%" ) (subcat "6A" "6C" "9" "10" )))
( "suggested"  v ( s @1 jh e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6C" "9" "10" )))
( "suggestibility"  n ( s @1 jh e2 s t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "suggestible"  j ( s @1 jh e1 s t @ b l ) ((pos "OA%" ) ))
( "suggesting"  v ( s @1 jh e1 s t i ng ) ((pos "Hb%" ) (subcat "6A" "6C" "9" "10" )))
( "suggestion"  n ( s @1 jh e1 s ch @ n ) ((pos "M6%" ) ))
( "suggestions"  n ( s @1 jh e1 s ch @ n z ) ((pos "Mj%" ) ))
( "suggestive"  j ( s @1 jh e1 s t i v ) ((pos "OA%" ) ))
( "suggestively"  a ( s @1 jh e1 s t i v l ii ) ((pos "Pu%" ) ))
( "suggests"  v ( s @1 jh e1 s t s ) ((pos "Ha%" ) (subcat "6A" "6C" "9" "10" )))
( "suicidal"  j ( s y uu2 i s ai1 d l ) ((pos "OA%" ) ))
( "suicide"  n ( s y uu1 i s ai d ) ((pos "M6%" ) ))
( "suicides"  n ( s y uu1 i s ai d z ) ((pos "Mj%" ) ))
( "suing"  v ( s y uu1 i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "suit"  v ( s uu1 t ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "suitability"  n ( s uu2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "suitable"  j ( s uu1 t @ b l ) ((pos "OA%" ) ))
( "suitableness"  n ( s uu1 t @ b l n @ s ) ((pos "L@%" ) ))
( "suitably"  a ( s uu1 t @ b l ii ) ((pos "Pu%" ) ))
( "suitcase"  n ( s uu1 t k ei s ) ((pos "K6%" ) ))
( "suitcases"  n ( s uu1 t k ei s i z ) ((pos "Kj%" ) ))
( "suite"  n ( s w ii1 t ) ((pos "K6%" ) ))
( "suited"  v ( s uu1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "suites"  n ( s w ii1 t s ) ((pos "Kj%" ) ))
( "suiting"  v ( s uu1 t i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "6A" )))
( "suitor"  n ( s uu1 t @ r ) ((pos "K6%" ) ))
( "suitors"  n ( s uu1 t @ z ) ((pos "Kj%" ) ))
( "suits"  v ( s uu1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "sulk"  v ( s uh1 l k ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "sulked"  v ( s uh1 l k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sulkier"  j ( s uh1 l k i@ r ) ((pos "Or%" ) ))
( "sulkies"  n ( s uh1 l k i z ) ((pos "Kj$" ) ))
( "sulkiest"  j ( s uh1 l k i i s t ) ((pos "Os%" ) ))
( "sulkily"  a ( s uh1 l k i l ii ) ((pos "Pu%" ) ))
( "sulkiness"  n ( s uh1 l k i n @ s ) ((pos "L@%" ) ))
( "sulking"  v ( s uh1 l k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sulks"  v ( s uh1 l k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "sulky"  n ( s uh1 l k ii ) ((pos "K8$" "OD%" ) ))
( "sullen"  j ( s uh1 l @ n ) ((pos "OA%" ) ))
( "sullenly"  a ( s uh1 l @ n l ii ) ((pos "Pu%" ) ))
( "sullenness"  n ( s uh1 l @ n - n @ s ) ((pos "L@%" ) ))
( "sullied"  v ( s uh1 l i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sullies"  v ( s uh1 l i z ) ((pos "Ha%" ) (subcat )))
( "sully"  v ( s uh1 l ii ) ((pos "H3%" ) (subcat )))
( "sullying"  v ( s uh1 l i i ng ) ((pos "Hb%" ) (subcat )))
( "sulpha"  n ( s uh1 l f @ ) ((pos "L@$" ) ))
( "sulphate"  n ( s uh1 l f ei t ) ((pos "M6%" ) ))
( "sulphates"  n ( s uh1 l f ei t s ) ((pos "Mj%" ) ))
( "sulphide"  n ( s uh1 l f ai d ) ((pos "M6%" ) ))
( "sulphides"  n ( s uh1 l f ai d z ) ((pos "Mj%" ) ))
( "sulphonamides"  n ( s uh1 l f o1 n @ m ai d z ) ((pos "Kj$" ) ))
( "sulphur"  n ( s uh1 l f @ r ) ((pos "L@%" ) ))
( "sulphuretted"  j ( s uh1 l f y u r e t i d ) ((pos "OA$" ) ))
( "sulphuric"  j ( s uh1 l f y u1 r i k ) ((pos "OA%" ) ))
( "sulphurous"  j ( s uh1 l f @ r @ s ) ((pos "OA%" ) ))
( "sultan"  n ( s uh1 l t @ n ) ((pos "K6%" ) ))
( "sultana"  n ( s uh1 l t aa1 n @ ) ((pos "K6%" ) ))
( "sultanas"  n ( s uh1 l t aa1 n @ z ) ((pos "Kj%" ) ))
( "sultanate"  n ( s uh1 l t @ n ei t ) ((pos "K6%" ) ))
( "sultanates"  n ( s uh1 l t @ n ei t s ) ((pos "Kj%" ) ))
( "sultans"  n ( s uh1 l t @ n z ) ((pos "Kj%" ) ))
( "sultrier"  j ( s uh1 l t r i@ r ) ((pos "Or%" ) ))
( "sultriest"  j ( s uh1 l t r i i s t ) ((pos "Os%" ) ))
( "sultrily"  a ( s uh1 l t r i l ii ) ((pos "Pu%" ) ))
( "sultriness"  n ( s uh1 l t r i n @ s ) ((pos "L@%" ) ))
( "sultry"  j ( s uh1 l t r ii ) ((pos "OD%" ) ))
( "sum"  v ( s uh1 m ) ((pos "J4%" "K6%" ) (subcat "2C" )))
( "sumac"  n ( sh uu1 m a k ) ((pos "L@$" ) ))
( "sumach"  n ( sh uu1 m a k ) ((pos "L@$" ) ))
( "summaries"  n ( s uh1 m @ r i z ) ((pos "Kj%" ) ))
( "summarily"  a ( s uh1 m @ r @ l ii ) ((pos "Pu%" ) ))
( "summarize"  v ( s uh1 m @ r ai z ) ((pos "H2%" ) (subcat )))
( "summarized"  v ( s uh1 m @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "summarizes"  v ( s uh1 m @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "summarizing"  v ( s uh1 m @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "summary"  n ( s uh1 m @ r ii ) ((pos "K8%" "OA%" ) ))
( "summat"  prp ( s uh1 m @ t ) ((pos "Qx$" ) ))
( "summation"  n ( s @1 m ei1 sh @ n ) ((pos "K6%" ) ))
( "summations"  n ( s @1 m ei1 sh @ n z ) ((pos "Kj$" ) ))
( "summed"  v ( s uh1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "summer"  v ( s uh1 m @ r ) ((pos "I0$" "M6%" ) (subcat )))
( "summered"  v ( s uh1 m @ d ) ((pos "Ic$" "Id$" ) (subcat )))
( "summerhouse"  n ( s uh1 m @ h au s ) ((pos "K6%" ) ))
( "summerhouses"  n ( s uh1 m @ h au z i z ) ((pos "Kj%" ) ))
( "summerier"  j ( s uh1 m @ r i@ r ) ((pos "Or$" ) ))
( "summeriest"  j ( s uh1 m @ r i i s t ) ((pos "Os$" ) ))
( "summering"  v ( s uh1 m @ r i ng ) ((pos "Ib$" ) (subcat )))
( "summers"  v ( s uh1 m @ z ) ((pos "Ia$" "Mj%" ) (subcat )))
( "summertime"  n ( s uh1 m @ t ai m ) ((pos "M6%" ) ))
( "summertimes"  n ( s uh1 m @ t ai m z ) ((pos "Mj$" ) ))
( "summery"  j ( s uh1 m @ r ii ) ((pos "OD%" ) ))
( "summing"  v ( s uh1 m i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "summing-up"  n ( s uh2 m i ng - uh1 p ) ((pos "K6%" ) ))
( "summing-ups"  n ( s uh2 m i ng - uh1 p s ) ((pos "Kj$" ) ))
( "summit"  n ( s uh1 m i t ) ((pos "K6%" ) ))
( "summits"  n ( s uh1 m i t s ) ((pos "Kj%" ) ))
( "summon"  v ( s uh1 m @ n ) ((pos "H0%" ) (subcat "6A" "14" "15B" )))
( "summoned"  v ( s uh1 m @ n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" "15B" )))
( "summoning"  v ( s uh1 m @ n i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15B" )))
( "summons"  v ( s uh1 m @ n z ) ((pos "H1%" "Ha%" "K7%" ) (subcat "6A" "14" "15B" )))
( "summonsed"  v ( s uh1 m @ n z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "summonses"  v ( s uh1 m @ n z i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "summonsing"  v ( s uh1 m @ n z i ng ) ((pos "Hb%" ) (subcat )))
( "sump"  n ( s uh1 m p ) ((pos "K6%" ) ))
( "sumps"  n ( s uh1 m p s ) ((pos "Kj%" ) ))
( "sumpter"  n ( s uh1 m p t @ r ) ((pos "K6$" ) ))
( "sumpters"  n ( s uh1 m p t @ z ) ((pos "Kj$" ) ))
( "sumptuary"  j ( s uh1 m p t y u@ r ii ) ((pos "OA$" ) ))
( "sumptuous"  j ( s uh1 m p ch u@ s ) ((pos "OA%" ) ))
( "sumptuously"  a ( s uh1 m p ch u@ s l ii ) ((pos "Pu%" ) ))
( "sumptuousness"  n ( s uh1 m p ch u@ s n @ s ) ((pos "L@%" ) ))
( "sums"  v ( s uh1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "sun"  v ( s uh1 n ) ((pos "H4%" "M6%" ) (subcat )))
( "sun-drenched"  j ( s uh1 n - d r e n ch t ) ((pos "OA%" ) ))
( "sun-dried"  j ( s uh1 n - d r ai d ) ((pos "OA%" ) ))
( "sun-god"  n ( s uh1 n - g o d ) ((pos "K6$" ) ))
( "sun-gods"  n ( s uh1 n - g o d z ) ((pos "Kj$" ) ))
( "sun-helmet"  n ( s uh1 n - h e l m i t ) ((pos "K6$" ) ))
( "sun-helmets"  n ( s uh1 n - h e l m i t s ) ((pos "Kj$" ) ))
( "sun-lounge"  n ( s uh1 n - l au n jh ) ((pos "K6%" ) ))
( "sun-lounges"  n ( s uh1 n - l au n jh i z ) ((pos "Kj%" ) ))
( "sun-parlour"  n ( s uh1 n - p aa l @ r ) ((pos "K6$" ) ))
( "sun-parlours"  n ( s uh1 n - p aa l @ z ) ((pos "Kj$" ) ))
( "sun-porch"  n ( s uh1 n - p oo ch ) ((pos "K7%" ) ))
( "sun-porches"  n ( s uh1 n - p oo ch i z ) ((pos "Kj$" ) ))
( "sun-up"  n ( s uh1 n - uh p ) ((pos "L@%" ) ))
( "sun-visor"  n ( s uh1 n - v ai z @ r ) ((pos "K6%" ) ))
( "sun-visors"  n ( s uh1 n - v ai z @ z ) ((pos "Kj%" ) ))
( "sun-worship"  n ( s uh1 n - w @@ sh i p ) ((pos "L@%" ) ))
( "sunbaked"  j ( s uh1 n b ei k t ) ((pos "OA%" ) ))
( "sunbathe"  v ( s uh1 n b ei dh ) ((pos "I2%" "Ki%" ) ))
( "sunbathed"  v ( s uh1 n b ei dh d ) ((pos "Ic%" "Id%" ) ))
( "sunbathes"  v ( s uh1 n b ei dh z ) ((pos "Ia%" ) ))
( "sunbathing"  v ( s uh1 n b ei dh i ng ) ((pos "Ib%" ) ))
( "sunbeam"  n ( s uh1 n b ii m ) ((pos "K6%" ) ))
( "sunbeams"  n ( s uh1 n b ii m z ) ((pos "Kj%" ) ))
( "sunblind"  n ( s uh1 n b l ai n d ) ((pos "K6$" ) ))
( "sunblinds"  n ( s uh1 n b l ai n d z ) ((pos "Kj$" ) ))
( "sunbonnet"  n ( s uh1 n b o n i t ) ((pos "K6%" ) ))
( "sunbonnets"  n ( s uh1 n b o n i t s ) ((pos "Kj%" ) ))
( "sunburn"  n ( s uh1 n b @@ n ) ((pos "M6%" ) ))
( "sunburned"  j ( s uh1 n b @@ n d ) ((pos "OA%" ) ))
( "sunburns"  n ( s uh1 n b @@ n z ) ((pos "Mj%" ) ))
( "sunburnt"  j ( s uh1 n b @@ n t ) ((pos "OA%" ) ))
( "sunburst"  n ( s uh1 n b @@ s t ) ((pos "K6%" ) ))
( "sunbursts"  n ( s uh1 n b @@ s t s ) ((pos "Kj%" ) ))
( "sundae"  n ( s uh1 n d ei ) ((pos "K6$" ) ))
( "sundaes"  n ( s uh1 n d ei z ) ((pos "Kj$" ) ))
( "sunder"  v ( s uh1 n d @ r ) ((pos "H0%" "L@%" ) (subcat )))
( "sundered"  v ( s uh1 n d @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sundering"  v ( s uh1 n d @ r i ng ) ((pos "Hb%" ) (subcat )))
( "sunders"  v ( s uh1 n d @ z ) ((pos "Ha%" ) (subcat )))
( "sundial"  n ( s uh1 n d ai @ l ) ((pos "K6%" ) ))
( "sundials"  n ( s uh1 n d ai @ l z ) ((pos "Kj%" ) ))
( "sundown"  n ( s uh1 n d au n ) ((pos "L@%" ) ))
( "sundowner"  n ( s uh1 n d au n @ r ) ((pos "K6$" ) ))
( "sundowners"  n ( s uh1 n d au n @ z ) ((pos "Kj$" ) ))
( "sundries"  n ( s uh1 n d r i z ) ((pos "Kj%" ) ))
( "sundry"  j ( s uh1 n d r ii ) ((pos "OA%" ) ))
( "sunfish"  n ( s uh1 n f i sh ) ((pos "K9$" ) ))
( "sunfishes"  n ( s uh1 n f i sh i z ) ((pos "Kj$" ) ))
( "sunflower"  n ( s uh1 n f l au @ r ) ((pos "K6%" ) ))
( "sunflowers"  n ( s uh1 n f l au @ z ) ((pos "Kj%" ) ))
( "sung"  v ( s uh1 ng ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "12B" "13B" )))
( "sunglasses"  n ( s uh1 n g l a s i z ) ((pos "Kj%" ) ))
( "sunhat"  n ( s uh1 n h a t ) ((pos "K6%" ) ))
( "sunhats"  n ( s uh1 n h a t s ) ((pos "Kj%" ) ))
( "sunk"  v ( s uh1 ng k ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sunken"  v ( s uh1 ng k @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "sunlamp"  n ( s uh1 n l a m p ) ((pos "K6%" ) ))
( "sunlamps"  n ( s uh1 n l a m p s ) ((pos "Kj%" ) ))
( "sunless"  j ( s uh1 n l @ s ) ((pos "OA%" ) ))
( "sunlight"  n ( s uh1 n l ai t ) ((pos "L@%" ) ))
( "sunlit"  j ( s uh1 n l i t ) ((pos "OA%" ) ))
( "sunned"  v ( s uh1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sunnier"  j ( s uh1 n i@ r ) ((pos "Or%" ) ))
( "sunniest"  j ( s uh1 n i i s t ) ((pos "Os%" ) ))
( "sunnily"  a ( s uh1 n i l ii ) ((pos "Pu%" ) ))
( "sunning"  v ( s uh1 n i ng ) ((pos "Hb%" ) (subcat )))
( "sunny"  j ( s uh1 n ii ) ((pos "OD%" ) ))
( "sunray"  j ( s uh1 n r ei ) ((pos "OA%" ) ))
( "sunrise"  n ( s uh1 n r ai z ) ((pos "M6%" ) ))
( "sunrises"  n ( s uh1 n r ai z i z ) ((pos "Mj%" ) ))
( "sunroof"  n ( s uh1 n r uu f ) ((pos "K6%" ) ))
( "sunroofs"  n ( s uh1 n r uu f s ) ((pos "Kj%" ) ))
( "suns"  v ( s uh1 n z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "sunset"  n ( s uh1 n s e t ) ((pos "M6%" ) ))
( "sunsets"  n ( s uh1 n s e t s ) ((pos "Mj%" ) ))
( "sunshade"  n ( s uh1 n sh ei d ) ((pos "K6%" ) ))
( "sunshades"  n ( s uh1 n sh ei d z ) ((pos "Kj%" ) ))
( "sunshine"  n ( s uh1 n sh ai n ) ((pos "L@%" ) ))
( "sunshine-roof"  n ( s uh2 n sh ai n - r uu1 f ) ((pos "K6%" ) ))
( "sunshine-roofs"  n ( s uh2 n sh ai n - r uu1 f s ) ((pos "Kj%" ) ))
( "sunspot"  n ( s uh1 n s p o t ) ((pos "K6%" ) ))
( "sunspots"  n ( s uh1 n s p o t s ) ((pos "Kj%" ) ))
( "sunstroke"  n ( s uh1 n s t r ou k ) ((pos "L@%" ) ))
( "suntan"  n ( s uh1 n t a n ) ((pos "M6%" ) ))
( "suntans"  n ( s uh1 n t a n z ) ((pos "Mj$" ) ))
( "suntrap"  n ( s uh1 n t r a p ) ((pos "K6%" ) ))
( "suntraps"  n ( s uh1 n t r a p s ) ((pos "Kj%" ) ))
( "sup"  v ( s uh1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "super"  n ( s uu1 p @ r ) ((pos "K6%" "OA%" ) ))
( "superabundance"  n ( s uu2 p @ r @ b uh1 n d @ n s ) ((pos "Ki%" ) ))
( "superabundant"  j ( s uu2 p @ r @ b uh1 n d @ n t ) ((pos "OA%" ) ))
( "superannuate"  v ( s uu2 p @ r a1 n y u ei t ) ((pos "H2$" ) (subcat )))
( "superannuated"  v ( s uu2 p @ r a1 n y u ei t i d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "superannuates"  v ( s uu2 p @ r a1 n y u ei t s ) ((pos "Ha$" ) (subcat )))
( "superannuating"  v ( s uu2 p @ r a1 n y u ei t i ng ) ((pos "Hb$" ) (subcat )))
( "superannuation"  n ( s uu2 p @ r a2 n y u ei1 sh @ n ) ((pos "M6%" ) ))
( "superannuations"  n ( s uu2 p @ r a2 n y u ei1 sh @ n z ) ((pos "Mj$" ) ))
( "superb"  j ( s uu1 p @@1 b ) ((pos "OA%" ) ))
( "superbly"  a ( s uu1 p @@1 b l ii ) ((pos "Pu%" ) ))
( "supercargo"  n ( s uu1 p @ k aa g ou ) ((pos "K7$" ) ))
( "supercargoes"  n ( s uu1 p @ k aa g ou z ) ((pos "Kj$" ) ))
( "supercharged"  j ( s uu1 p @ ch aa jh d ) ((pos "OA%" ) ))
( "supercharger"  n ( s uu1 p @ ch aa jh @ r ) ((pos "K6%" ) ))
( "superchargers"  n ( s uu1 p @ ch aa jh @ z ) ((pos "Kj%" ) ))
( "supercilious"  j ( s uu2 p @ s i1 l i@ s ) ((pos "OA%" ) ))
( "superciliously"  a ( s uu2 p @ s i1 l i@ s l ii ) ((pos "Pu%" ) ))
( "superciliousness"  n ( s uu2 p @ s i1 l i@ s n @ s ) ((pos "L@%" ) ))
( "superego"  n ( s uu1 p @ r e g ou ) ((pos "L@%" ) ))
( "supererogation"  n ( s uu2 p @ r e2 r @ g ei1 sh @ n ) ((pos "L@$" ) ))
( "superfatted"  j ( s uu2 p @ f a1 t i d ) ((pos "OA$" ) ))
( "superficial"  j ( s uu2 p @ f i1 sh l ) ((pos "OA%" ) ))
( "superficialities"  n ( s uu2 p @ f i2 sh i a1 l i t i z ) ((pos "Mj%" ) ))
( "superficiality"  n ( s uu2 p @ f i2 sh i a1 l i t ii ) ((pos "M8%" ) ))
( "superficially"  a ( s uu2 p @ f i1 sh @ l ii ) ((pos "Pu%" ) ))
( "superficies"  n ( s uu2 p @ f i1 sh ii z ) ((pos "K9$" ) ))
( "superfine"  j ( s uu1 p @ f ai n ) ((pos "OA%" ) ))
( "superfluities"  n ( s uu2 p @ f l u1 i t i z ) ((pos "Mj$" ) ))
( "superfluity"  n ( s uu2 p @ f l u1 i t ii ) ((pos "M8%" ) ))
( "superfluous"  j ( s uu1 p @@1 f l u@ s ) ((pos "OA%" ) ))
( "superfluously"  a ( s uu1 p @@1 f l u@ s l ii ) ((pos "Pu%" ) ))
( "superhuman"  j ( s uu2 p @ h y uu1 m @ n ) ((pos "OA%" ) ))
( "superimpose"  v ( s uu2 p @ r i m p ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "superimposed"  v ( s uu2 p @ r i m p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "superimposes"  v ( s uu2 p @ r i m p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "superimposing"  v ( s uu2 p @ r i m p ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "superintend"  v ( s uu2 p @ r i n t e1 n d ) ((pos "J0%" ) (subcat "2A" )))
( "superintended"  v ( s uu2 p @ r i n t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "superintendence"  n ( s uu2 p @ r i n t e1 n d @ n s ) ((pos "L@%" ) ))
( "superintendent"  n ( s uu2 p @ r i n t e1 n d @ n t ) ((pos "K6%" ) ))
( "superintendents"  n ( s uu2 p @ r i n t e1 n d @ n t s ) ((pos "Kj%" ) ))
( "superintending"  v ( s uu2 p @ r i n t e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "superintends"  v ( s uu2 p @ r i n t e1 n d z ) ((pos "Ja%" ) (subcat "2A" )))
( "superior"  n ( s uu1 p i@1 r i@ r ) ((pos "K6%" "OA%" ) ))
( "superiority"  n ( s uu1 p i@2 r i o1 r i t ii ) ((pos "L@%" ) ))
( "superiors"  n ( s uu1 p i@1 r i@ z ) ((pos "Kj%" ) ))
( "superlative"  n ( s uu1 p @@1 l @ t i v ) ((pos "K6%" "OA%" ) ))
( "superlatives"  n ( s uu1 p @@1 l @ t i v z ) ((pos "Kj%" ) ))
( "superman"  n ( s uu1 p @ m a n ) ((pos "Ki%" ) ))
( "supermarket"  n ( s uu1 p @ m aa k i t ) ((pos "K6%" ) ))
( "supermarkets"  n ( s uu1 p @ m aa k i t s ) ((pos "Kj%" ) ))
( "supermen"  n ( s uu1 p @ m e n ) ((pos "Kj%" ) ))
( "supernal"  j ( s uu1 p @@1 n l ) ((pos "OA$" ) ))
( "supernatural"  j ( s uu2 p @ n a1 ch r @ l ) ((pos "OA%" ) ))
( "supernaturally"  a ( s uu2 p @ n a1 ch r @ l ii ) ((pos "Pu%" ) ))
( "supernormal"  j ( s uu2 p @ n oo1 m l ) ((pos "OA%" ) ))
( "supernumeraries"  n ( s uu2 p @ n y uu1 m @ r @ r i z ) ((pos "Kj%" ) ))
( "supernumerary"  n ( s uu2 p @ n y uu1 m @ r @ r ii ) ((pos "K8%" ) ))
( "supers"  n ( s uu1 p @ z ) ((pos "Kj$" ) ))
( "superscription"  n ( s uu2 p @ s k r i1 p sh @ n ) ((pos "K6$" ) ))
( "superscriptions"  n ( s uu2 p @ s k r i1 p sh @ n z ) ((pos "Kj$" ) ))
( "supersede"  v ( s uu2 p @ s ii1 d ) ((pos "H2%" ) (subcat )))
( "superseded"  v ( s uu2 p @ s ii1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "supersedes"  v ( s uu2 p @ s ii1 d z ) ((pos "Ha%" ) (subcat )))
( "superseding"  v ( s uu2 p @ s ii1 d i ng ) ((pos "Hb%" ) (subcat )))
( "supersession"  n ( s uu2 p @ s e1 sh @ n ) ((pos "L@$" ) ))
( "supersonic"  j ( s uu2 p @ s o1 n i k ) ((pos "OA%" ) ))
( "superstition"  n ( s uu2 p @ s t i1 sh @ n ) ((pos "M6%" ) ))
( "superstitions"  n ( s uu2 p @ s t i1 sh @ n z ) ((pos "Mj%" ) ))
( "superstitious"  j ( s uu2 p @ s t i1 sh @ s ) ((pos "OA%" ) ))
( "superstitiously"  a ( s uu2 p @ s t i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "superstructure"  n ( s uu1 p @ s t r uh k ch @ r ) ((pos "K6%" ) ))
( "superstructures"  n ( s uu1 p @ s t r uh k ch @ z ) ((pos "Kj%" ) ))
( "supertax"  n ( s uu1 p @ t a k s ) ((pos "M7%" ) ))
( "supertaxes"  n ( s uu1 p @ t a k s i z ) ((pos "Mj$" ) ))
( "supervene"  v ( s uu2 p @ v ii1 n ) ((pos "I2%" ) (subcat )))
( "supervened"  v ( s uu2 p @ v ii1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "supervenes"  v ( s uu2 p @ v ii1 n z ) ((pos "Ia%" ) (subcat )))
( "supervening"  v ( s uu2 p @ v ii1 n i ng ) ((pos "Ib%" ) (subcat )))
( "supervise"  v ( s uu1 p @ v ai z ) ((pos "J2%" ) (subcat "2A" )))
( "supervised"  v ( s uu1 p @ v ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "supervises"  v ( s uu1 p @ v ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "supervising"  v ( s uu1 p @ v ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "supervision"  n ( s uu2 p @ v i1 zh n ) ((pos "M6%" ) ))
( "supervisions"  n ( s uu2 p @ v i1 zh n z ) ((pos "Mj%" ) ))
( "supervisor"  n ( s uu1 p @ v ai z @ r ) ((pos "K6%" ) ))
( "supervisors"  n ( s uu1 p @ v ai z @ z ) ((pos "Kj%" ) ))
( "supervisory"  j ( s uu2 p @ v ai1 z @ r ii ) ((pos "OA%" ) ))
( "supine"  j ( s uu1 p ai n ) ((pos "OA$" ) ))
( "supinely"  a ( s uu1 p ai n l ii ) ((pos "Pu$" ) ))
( "supped"  v ( s uh1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "supper"  n ( s uh1 p @ r ) ((pos "M6%" ) ))
( "supperless"  j ( s uh1 p @ l @ s ) ((pos "OA%" ) ))
( "suppers"  n ( s uh1 p @ z ) ((pos "Mj%" ) ))
( "supping"  v ( s uh1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "supplant"  v ( s @1 p l aa1 n t ) ((pos "H0%" ) (subcat )))
( "supplanted"  v ( s @1 p l aa1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "supplanter"  n ( s @1 p l aa1 n t @ r ) ((pos "K6$" ) ))
( "supplanters"  n ( s @1 p l aa1 n t @ z ) ((pos "Kj$" ) ))
( "supplanting"  v ( s @1 p l aa1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "supplants"  v ( s @1 p l aa1 n t s ) ((pos "Ha%" ) (subcat )))
( "supple"  j ( s uh1 p l ) ((pos "OB%" ) ))
( "supplement"  n ( s uh1 p l i m @ n t ) ((pos "K6%" ) ))
( "supplement"  v ( s uh1 p l i m e n t ) ((pos "H0%" ) (subcat "6A" )))
( "supplementary"  j ( s uh2 p l i m e1 n t r ii ) ((pos "OA%" ) ))
( "supplemented"  v ( s uh1 p l i m e n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "supplementing"  v ( s uh1 p l i m e n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "supplements"  n ( s uh1 p l i m @ n t s ) ((pos "Kj%" ) ))
( "supplements"  v ( s uh1 p l i m e n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "suppleness"  n ( s uh1 p l n @ s ) ((pos "L@%" ) ))
( "suppler"  j ( s uh1 p l @ r ) ((pos "Or$" ) ))
( "supplest"  j ( s uh1 p l i s t ) ((pos "Os$" ) ))
( "suppliant"  n ( s uh1 p l i@ n t ) ((pos "K6$" "OA$" ) ))
( "suppliants"  n ( s uh1 p l i@ n t s ) ((pos "Kj$" ) ))
( "supplicant"  n ( s uh1 p l i k @ n t ) ((pos "K6%" ) ))
( "supplicants"  n ( s uh1 p l i k @ n t s ) ((pos "Kj%" ) ))
( "supplicate"  v ( s uh1 p l i k ei t ) ((pos "J2%" ) (subcat "2C" "6A" "14" )))
( "supplicated"  v ( s uh1 p l i k ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" )))
( "supplicates"  v ( s uh1 p l i k ei t s ) ((pos "Ja%" ) (subcat "2C" "6A" "14" )))
( "supplicating"  v ( s uh1 p l i k ei t i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" )))
( "supplication"  n ( s uh2 p l i k ei1 sh @ n ) ((pos "M6%" ) ))
( "supplications"  n ( s uh2 p l i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "supplied"  v ( s @1 p l ai1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "supplier"  n ( s @1 p l ai1 @ r ) ((pos "K6%" ) ))
( "suppliers"  n ( s @1 p l ai1 @ z ) ((pos "Kj%" ) ))
( "supplies"  v ( s @1 p l ai1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "supply"  v ( s @1 p l ai1 ) ((pos "H3%" "M8%" ) (subcat "6A" )))
( "supplying"  v ( s @1 p l ai1 i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "support"  v ( s @1 p oo1 t ) ((pos "H0%" "M6%" ) (subcat )))
( "supportable"  j ( s @1 p oo1 t @ b l ) ((pos "OA%" ) ))
( "supported"  v ( s @1 p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "supporter"  n ( s @1 p oo1 t @ r ) ((pos "K6%" ) ))
( "supporters"  n ( s @1 p oo1 t @ z ) ((pos "Kj%" ) ))
( "supporting"  v ( s @1 p oo1 t i ng ) ((pos "Hb%" ) (subcat )))
( "supports"  v ( s @1 p oo1 t s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "suppose"  v ( s @1 p ou1 z ) ((pos "H2%" ) (subcat "6A" "9" )))
( "supposed"  v ( s @1 p ou1 z d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "9" )))
( "supposedly"  a ( s @1 p ou1 z @ d l ii ) ((pos "Pu%" ) ))
( "supposes"  v ( s @1 p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" "9" )))
( "supposing"  v ( s @1 p ou1 z i ng ) ((pos "Hb%" "V-%" ) (subcat "6A" "9" )))
( "supposition"  n ( s uh2 p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "suppositions"  n ( s uh2 p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "suppositories"  n ( s @1 p o1 z i t r i z ) ((pos "Kj%" ) ))
( "suppository"  n ( s @1 p o1 z i t r ii ) ((pos "K8%" ) ))
( "suppress"  v ( s @1 p r e1 s ) ((pos "H1%" ) (subcat )))
( "suppressed"  v ( s @1 p r e1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "suppresses"  v ( s @1 p r e1 s i z ) ((pos "Ha%" ) (subcat )))
( "suppressing"  v ( s @1 p r e1 s i ng ) ((pos "Hb%" ) (subcat )))
( "suppression"  n ( s @1 p r e1 sh @ n ) ((pos "K6%" ) ))
( "suppressions"  n ( s @1 p r e1 sh @ n z ) ((pos "Kj$" ) ))
( "suppressive"  j ( s @1 p r e1 s i v ) ((pos "OA$" ) ))
( "suppressor"  n ( s @1 p r e1 s @ r ) ((pos "K6%" ) ))
( "suppressors"  n ( s @1 p r e1 s @ z ) ((pos "Kj%" ) ))
( "suppurate"  v ( s uh1 p y u r ei t ) ((pos "I2%" ) (subcat )))
( "suppurated"  v ( s uh1 p y u r ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "suppurates"  v ( s uh1 p y u r ei t s ) ((pos "Ia%" ) (subcat )))
( "suppurating"  v ( s uh1 p y u r ei t i ng ) ((pos "Ib%" ) (subcat )))
( "suppuration"  n ( s uh2 p y u r ei1 sh @ n ) ((pos "M6%" ) ))
( "suppurations"  n ( s uh2 p y u r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "supra"  a ( s uu1 p r @ ) ((pos "Pu$" ) ))
( "supranational"  j ( s uu2 p r @ n a1 sh @ n l ) ((pos "OA%" ) ))
( "supremacy"  n ( s u1 p r e1 m @ s ii ) ((pos "L@%" ) ))
( "supreme"  j ( s uu1 p r ii1 m ) ((pos "OA%" ) ))
( "supremely"  a ( s uu1 p r ii1 m l ii ) ((pos "Pu%" ) ))
( "sups"  v ( s uh1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "surcharge"  v ( s @@1 ch aa jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "surcharged"  v ( s @@1 ch aa jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "surcharges"  v ( s @@1 ch aa jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "surcharging"  v ( s @@1 ch aa jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "surd"  n ( s @@1 d ) ((pos "K6$" ) ))
( "surds"  n ( s @@1 d z ) ((pos "Kj$" ) ))
( "sure"  j ( sh u@1 r ) ((pos "OB*" "Pu*" ) ))
( "sure-footed"  j ( sh u@1 - f u1 t i d ) ((pos "OA%" ) ))
( "surely"  a ( sh u@1 l ii ) ((pos "Pu%" ) ))
( "sureness"  n ( sh u@1 n @ s ) ((pos "L@%" ) ))
( "surer"  j ( sh u@1 r @ r ) ((pos "Or%" ) ))
( "surest"  j ( sh u@1 r i s t ) ((pos "Os%" ) ))
( "sureties"  n ( sh u@1 r @ t i z ) ((pos "Mj$" ) ))
( "surety"  n ( sh u@1 r @ t ii ) ((pos "M8$" ) ))
( "surf"  n ( s @@1 f ) ((pos "L@%" ) ))
( "surface"  v ( s @@1 f i s ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "surface-to-air"  j ( s @@2 f i s - t @ - e@1 r ) ((pos "OA%" ) ))
( "surfaced"  v ( s @@1 f i s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "surfaces"  v ( s @@1 f i s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "surfacing"  v ( s @@1 f i s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "surfboard"  n ( s @@1 f b oo d ) ((pos "K6%" ) ))
( "surfboards"  n ( s @@1 f b oo d z ) ((pos "Kj%" ) ))
( "surfboat"  n ( s @@1 f b ou t ) ((pos "K6$" ) ))
( "surfboats"  n ( s @@1 f b ou t s ) ((pos "Kj$" ) ))
( "surfeit"  v ( s @@1 f i t ) ((pos "H0$" "K6%" ) (subcat "6A" )))
( "surfeited"  v ( s @@1 f i t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "surfeiting"  v ( s @@1 f i t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "surfeits"  v ( s @@1 f i t s ) ((pos "Ha$" "Kj$" ) (subcat "6A" )))
( "surfing"  n ( s @@1 f i ng ) ((pos "L@%" ) ))
( "surfriding"  n ( s @@1 f r ai d i ng ) ((pos "L@%" ) ))
( "surge"  v ( s @@1 jh ) ((pos "I2%" "K6%" ) (subcat )))
( "surged"  v ( s @@1 jh d ) ((pos "Ic%" "Id%" ) (subcat )))
( "surgeon"  n ( s @@1 jh @ n ) ((pos "K6%" ) ))
( "surgeons"  n ( s @@1 jh @ n z ) ((pos "Kj%" ) ))
( "surgeries"  n ( s @@1 jh @ r i z ) ((pos "Mj%" ) ))
( "surgery"  n ( s @@1 jh @ r ii ) ((pos "M8%" ) ))
( "surges"  v ( s @@1 jh i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "surgical"  j ( s @@1 jh i k l ) ((pos "OA%" ) ))
( "surgically"  a ( s @@1 jh i k l ii ) ((pos "Pu%" ) ))
( "surging"  v ( s @@1 jh i ng ) ((pos "Ib%" ) (subcat )))
( "surlier"  j ( s @@1 l i@ r ) ((pos "Or%" ) ))
( "surliest"  j ( s @@1 l i i s t ) ((pos "Os%" ) ))
( "surlily"  a ( s @@1 l i l ii ) ((pos "Pu$" ) ))
( "surliness"  n ( s @@1 l i n @ s ) ((pos "L@%" ) ))
( "surly"  j ( s @@1 l ii ) ((pos "OD%" ) ))
( "surmise"  n ( s @@1 m ai z ) ((pos "K6%" ) ))
( "surmise"  v ( s @1 m ai1 z ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "surmised"  v ( s @1 m ai1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "surmises"  n ( s @@1 m ai z i z ) ((pos "Kj$" ) ))
( "surmises"  v ( s @1 m ai1 z i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "surmising"  v ( s @1 m ai1 z i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "surmount"  v ( s @1 m au1 n t ) ((pos "H0%" ) (subcat )))
( "surmountable"  j ( s @1 m au1 n t @ b l ) ((pos "OA%" ) ))
( "surmounted"  v ( s @1 m au1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "surmounting"  v ( s @1 m au1 n t i ng ) ((pos "Hb%" ) (subcat )))
( "surmounts"  v ( s @1 m au1 n t s ) ((pos "Ha%" ) (subcat )))
( "surname"  n ( s @@1 n ei m ) ((pos "K6%" ) ))
( "surnames"  n ( s @@1 n ei m z ) ((pos "Kj%" ) ))
( "surpass"  v ( s @1 p aa1 s ) ((pos "H1%" ) (subcat "6A" )))
( "surpassed"  v ( s @1 p aa1 s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "surpasses"  v ( s @1 p aa1 s i z ) ((pos "Ha%" ) (subcat "6A" )))
( "surpassing"  v ( s @1 p aa1 s i ng ) ((pos "Hb%" "OA%" ) (subcat "6A" )))
( "surpassingly"  a ( s @1 p aa1 s i ng l ii ) ((pos "Pu%" ) ))
( "surplice"  n ( s @@1 p l i s ) ((pos "K6%" ) ))
( "surpliced"  j ( s @@1 p l i s t ) ((pos "OA$" ) ))
( "surplices"  n ( s @@1 p l i s i z ) ((pos "Kj%" ) ))
( "surplus"  n ( s @@1 p l @ s ) ((pos "K7%" ) ))
( "surpluses"  n ( s @@1 p l @ s i z ) ((pos "Kj%" ) ))
( "surprise"  v ( s @1 p r ai1 z ) ((pos "H2%" "M6%" ) (subcat )))
( "surprised"  v ( s @1 p r ai1 z d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "surprisedly"  a ( s @1 p r ai1 z @ d l ii ) ((pos "Pu$" ) ))
( "surprises"  v ( s @1 p r ai1 z i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "surprising"  v ( s @1 p r ai1 z i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "surprisingly"  a ( s @1 p r ai1 z i ng l ii ) ((pos "Pu%" ) ))
( "surrealism"  n ( s @1 r i@1 l i z @ m ) ((pos "L@%" ) ))
( "surrealist"  n ( s @1 r i@1 l i s t ) ((pos "K6%" ) ))
( "surrealistic"  j ( s @1 r i@ l i1 s t i k ) ((pos "OA%" ) ))
( "surrealists"  n ( s @1 r i@1 l i s t s ) ((pos "Kj%" ) ))
( "surrender"  v ( s @1 r e1 n d @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "surrendered"  v ( s @1 r e1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "surrendering"  v ( s @1 r e1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "surrenders"  v ( s @1 r e1 n d @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "surreptitious"  j ( s uh2 r @ p t i1 sh @ s ) ((pos "OA%" ) ))
( "surreptitiously"  a ( s uh2 r @ p t i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "surrogate"  n ( s uh1 r @ g ei t ) ((pos "K6%" "OA%" ) ))
( "surrogates"  n ( s uh1 r @ g ei t s ) ((pos "Kj$" ) ))
( "surround"  v ( s @1 r au1 n d ) ((pos "H0%" "K6%" ) (subcat )))
( "surrounded"  v ( s @1 r au1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "surrounding"  v ( s @1 r au1 n d i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "surroundings"  n ( s @1 r au1 n d i ng z ) ((pos "Kj%" ) ))
( "surrounds"  v ( s @1 r au1 n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "surtax"  v ( s @@1 t a k s ) ((pos "H1$" "M7%" ) ))
( "surtaxed"  v ( s @@1 t a k s t ) ((pos "Hc$" "Hd$" ) ))
( "surtaxes"  v ( s @@1 t a k s i z ) ((pos "Ha$" "Mj$" ) ))
( "surtaxing"  v ( s @@1 t a k s i ng ) ((pos "Hb$" ) ))
( "surveillance"  n ( s @@1 v ei1 l @ n s ) ((pos "L@%" ) ))
( "survey"  n ( s @@1 v ei ) ((pos "K6%" ) ))
( "survey"  v ( s @1 v ei1 ) ((pos "H0%" ) (subcat )))
( "surveyed"  v ( s @1 v ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "surveying"  v ( s @1 v ei1 i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "surveyor"  n ( s @1 v ei1 @ r ) ((pos "K6%" ) ))
( "surveyors"  n ( s @1 v ei1 @ z ) ((pos "Kj%" ) ))
( "surveys"  n ( s @@1 v ei z ) ((pos "Kj%" ) ))
( "surveys"  v ( s @1 v ei1 z ) ((pos "Ha%" ) (subcat )))
( "survival"  n ( s @1 v ai1 v @ l ) ((pos "M6%" ) ))
( "survivals"  n ( s @1 v ai1 v @ l z ) ((pos "Mj$" ) ))
( "survive"  v ( s @1 v ai1 v ) ((pos "J2%" ) (subcat "2A" )))
( "survived"  v ( s @1 v ai1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "survives"  v ( s @1 v ai1 v z ) ((pos "Ja%" ) (subcat "2A" )))
( "surviving"  v ( s @1 v ai1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "survivor"  n ( s @1 v ai1 v @ r ) ((pos "K6%" ) ))
( "survivors"  n ( s @1 v ai1 v @ z ) ((pos "Kj%" ) ))
( "susceptibilities"  n ( s @1 s e2 p t @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "susceptibility"  n ( s @1 s e2 p t @ b i1 l i t ii ) ((pos "M8%" ) ))
( "susceptible"  j ( s @1 s e1 p t @ b l ) ((pos "OA%" ) ))
( "suspect"  n ( s uh1 s p e k t ) ((pos "K6%" "Op%" ) ))
( "suspect"  v ( s @1 s p e1 k t ) ((pos "H0%" ) (subcat "6A" "9" "14" )))
( "suspected"  v ( s @1 s p e1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" "14" )))
( "suspecting"  v ( s @1 s p e1 k t i ng ) ((pos "Hb%" ) (subcat "6A" "9" "14" )))
( "suspects"  n ( s uh1 s p e k t s ) ((pos "Kj%" ) ))
( "suspects"  v ( s @1 s p e1 k t s ) ((pos "Ha%" ) (subcat "6A" "9" "14" )))
( "suspend"  v ( s @1 s p e1 n d ) ((pos "H0%" ) (subcat "6A" )))
( "suspended"  v ( s @1 s p e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "suspender"  n ( s @1 s p e1 n d @ r ) ((pos "K6%" ) ))
( "suspenders"  n ( s @1 s p e1 n d @ z ) ((pos "Kj%" ) ))
( "suspending"  v ( s @1 s p e1 n d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "suspends"  v ( s @1 s p e1 n d z ) ((pos "Ha%" ) (subcat "6A" )))
( "suspense"  n ( s @1 s p e1 n s ) ((pos "L@%" ) ))
( "suspension"  n ( s @1 s p e1 n sh @ n ) ((pos "L@%" ) ))
( "suspicion"  n ( s @1 s p i1 sh @ n ) ((pos "M6%" ) ))
( "suspicions"  n ( s @1 s p i1 sh @ n z ) ((pos "Mj%" ) ))
( "suspicious"  j ( s @1 s p i1 sh @ s ) ((pos "OA%" ) ))
( "suspiciously"  a ( s @1 s p i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "suss"  v ( s uh1 s ) ((pos "H1$" ) (subcat )))
( "sussed"  v ( s uh1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "susses"  v ( s uh1 s i z ) ((pos "Ha$" ) (subcat )))
( "sussing"  v ( s uh1 s i ng ) ((pos "Hb$" ) (subcat )))
( "sustain"  v ( s @1 s t ei1 n ) ((pos "H0%" ) (subcat )))
( "sustained"  v ( s @1 s t ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "sustaining"  v ( s @1 s t ei1 n i ng ) ((pos "Hb%" ) (subcat )))
( "sustains"  v ( s @1 s t ei1 n z ) ((pos "Ha%" ) (subcat )))
( "sustenance"  n ( s uh1 s t i n @ n s ) ((pos "L@%" ) ))
( "suttee"  n ( s uh1 t ii ) ((pos "M6$" ) ))
( "suttees"  n ( s uh1 t ii z ) ((pos "Mj$" ) ))
( "suture"  n ( s uu1 ch @ r ) ((pos "K6$" ) ))
( "sutures"  n ( s uu1 ch @ z ) ((pos "Kj$" ) ))
( "suzerain"  n ( s uu1 z @ r ei n ) ((pos "K6$" ) ))
( "suzerains"  n ( s uu1 z @ r ei n z ) ((pos "Kj$" ) ))
( "suzerainties"  n ( s uu1 z @ r @ n t i z ) ((pos "Kj$" ) ))
( "suzerainty"  n ( s uu1 z @ r @ n t ii ) ((pos "K8$" ) ))
( "svelte"  j ( s v e1 l t ) ((pos "OA$" ) ))
( "swab"  v ( s w o1 b ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "swabbed"  v ( s w o1 b d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "swabbing"  v ( s w o1 b i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "swabs"  v ( s w o1 b z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "swaddle"  v ( s w o1 d l ) ((pos "H2$" ) (subcat )))
( "swaddled"  v ( s w o1 d l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "swaddles"  v ( s w o1 d l z ) ((pos "Ha$" ) (subcat )))
( "swaddling"  v ( s w o1 d l i ng ) ((pos "Hb$" ) (subcat )))
( "swaddling-clothes"  n ( s w o1 d l i ng - k l ou dh z ) ((pos "Kj%" ) ))
( "swag"  n ( s w a1 g ) ((pos "L@%" ) ))
( "swagger"  v ( s w a1 g @ r ) ((pos "I0%" "K6%" "OA%" ) (subcat "2A" )))
( "swaggered"  v ( s w a1 g @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "swaggerer"  n ( s w a1 g @ r @ r ) ((pos "K6$" ) ))
( "swaggerers"  n ( s w a1 g @ r @ z ) ((pos "Kj$" ) ))
( "swaggering"  v ( s w a1 g @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "swaggers"  v ( s w a1 g @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "swain"  n ( s w ei1 n ) ((pos "K6$" ) ))
( "swains"  n ( s w ei1 n z ) ((pos "Kj$" ) ))
( "swallow"  v ( s w o1 l ou ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "swallow-tailed"  j ( s w o1 l ou - t ei l d ) ((pos "OA%" ) ))
( "swallowed"  v ( s w o1 l ou d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "swallowing"  v ( s w o1 l ou i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "swallows"  v ( s w o1 l ou z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "swam"  v ( s w a1 m ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "3A" )))
( "swami"  n ( s w aa1 m ii ) ((pos "K6$" ) ))
( "swamis"  n ( s w aa1 m i z ) ((pos "Kj$" ) ))
( "swamp"  v ( s w o1 m p ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "swamped"  v ( s w o1 m p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "swampier"  j ( s w o1 m p i@ r ) ((pos "Or$" ) ))
( "swampiest"  j ( s w o1 m p i i s t ) ((pos "Os$" ) ))
( "swamping"  v ( s w o1 m p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "swamps"  v ( s w o1 m p s ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "swampy"  j ( s w o1 m p ii ) ((pos "OD%" ) ))
( "swan"  v ( s w o1 n ) ((pos "I4%" "K6%" ) (subcat )))
( "swan's-down"  n ( s w o1 n z - d au n ) ((pos "L@%" ) ))
( "swan-song"  n ( s w o1 n - s o ng ) ((pos "K6%" ) ))
( "swan-songs"  n ( s w o1 n - s o ng z ) ((pos "Kj%" ) ))
( "swank"  v ( s w a1 ng k ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "swanked"  v ( s w a1 ng k t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "swankier"  j ( s w a1 ng k i@ r ) ((pos "Or%" ) ))
( "swankiest"  j ( s w a1 ng k i i s t ) ((pos "Os%" ) ))
( "swanking"  v ( s w a1 ng k i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "swanks"  v ( s w a1 ng k s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "swanky"  j ( s w a1 ng k ii ) ((pos "OD%" ) ))
( "swanned"  v ( s w o1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "swanning"  v ( s w o1 n i ng ) ((pos "Ib%" ) (subcat )))
( "swans"  v ( s w o1 n z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "swap"  v ( s w o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" )))
( "swapped"  v ( s w o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "swapping"  v ( s w o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "swaps"  v ( s w o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "sward"  n ( s w oo1 d ) ((pos "L@$" ) ))
( "swarm"  v ( s w oo1 m ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "swarmed"  v ( s w oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "swarming"  v ( s w oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "swarms"  v ( s w oo1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "swarthy"  j ( s w oo1 dh ii ) ((pos "OA%" ) ))
( "swashbuckler"  n ( s w o1 sh b uh k l @ r ) ((pos "K6%" ) ))
( "swashbucklers"  n ( s w o1 sh b uh k l @ z ) ((pos "Kj%" ) ))
( "swashbuckling"  n ( s w o1 sh b uh k l i ng ) ((pos "L@%" "OA%" ) ))
( "swastika"  n ( s w o1 s t i k @ ) ((pos "K6%" ) ))
( "swastikas"  n ( s w o1 s t i k @ z ) ((pos "Kj%" ) ))
( "swat"  v ( s w o1 t ) ((pos "H4%" "K6%" ) ))
( "swath"  n ( s w oo1 th ) ((pos "K6$" ) ))
( "swathe"  v ( s w ei1 dh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "swathed"  v ( s w ei1 dh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "swathes"  v ( s w ei1 dh z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "swathing"  v ( s w ei1 dh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "swaths"  n ( s w oo1 th s ) ((pos "Kj$" ) ))
( "swats"  v ( s w o1 t s ) ((pos "Ha%" "Kj%" ) ))
( "swatted"  v ( s w o1 t i d ) ((pos "Hc%" "Hd%" ) ))
( "swatting"  v ( s w o1 t i ng ) ((pos "Hb%" ) ))
( "sway"  v ( s w ei1 ) ((pos "J0%" "L@%" ) (subcat "2A" "2C" "6A" )))
( "swayed"  v ( s w ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "swaying"  v ( s w ei1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "sways"  v ( s w ei1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "swear"  v ( s w e@1 r ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "swearer"  n ( s w e@1 r @ r ) ((pos "K6$" ) ))
( "swearers"  n ( s w e@1 r @ z ) ((pos "Kj$" ) ))
( "swearing"  v ( s w e@1 r i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "swears"  v ( s w e@1 z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "swearword"  n ( s w e@1 w @@ d ) ((pos "K6%" ) ))
( "swearwords"  n ( s w e@1 w @@ d z ) ((pos "Kj%" ) ))
( "sweat"  v ( s w e1 t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" )))
( "sweatband"  n ( s w e1 t b a n d ) ((pos "K6%" ) ))
( "sweatbands"  n ( s w e1 t b a n d z ) ((pos "Kj%" ) ))
( "sweated"  v ( s w e1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "sweater"  n ( s w e1 t @ r ) ((pos "K6%" ) ))
( "sweaters"  n ( s w e1 t @ z ) ((pos "Kj%" ) ))
( "sweatier"  j ( s w e1 t i@ r ) ((pos "Or$" ) ))
( "sweatiest"  j ( s w e1 t i i s t ) ((pos "Os$" ) ))
( "sweating"  v ( s w e1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "sweats"  v ( s w e1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "sweatshop"  n ( s w e1 ch o p ) ((pos "K6%" ) ))
( "sweatshops"  n ( s w e1 ch o p s ) ((pos "Kj%" ) ))
( "sweaty"  j ( s w e1 t ii ) ((pos "OD%" ) ))
( "swede"  n ( s w ii1 d ) ((pos "K6%" ) ))
( "swedes"  n ( s w ii1 d z ) ((pos "Kj%" ) ))
( "sweep"  v ( s w ii1 p ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "6A" "12A" "15A" "15B" )))
( "sweeper"  n ( s w ii1 p @ r ) ((pos "K6%" ) ))
( "sweepers"  n ( s w ii1 p @ z ) ((pos "Kj%" ) ))
( "sweeping"  v ( s w ii1 p i ng ) ((pos "Jb%" "M6%" "OA%" ) (subcat "2A" "2C" "6A" "12A" "15A" "15B" )))
( "sweepingly"  a ( s w ii1 p i ng l ii ) ((pos "Pu%" ) ))
( "sweepings"  n ( s w ii1 p i ng z ) ((pos "Mj%" ) ))
( "sweeps"  v ( s w ii1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "12A" "15A" "15B" )))
( "sweet"  n ( s w ii1 t ) ((pos "M6%" "OC%" ) ))
( "sweet-scented"  j ( s w ii1 t - s e1 n t i d ) ((pos "OA%" ) ))
( "sweetbread"  n ( s w ii1 t b r e d ) ((pos "K6$" ) ))
( "sweetbreads"  n ( s w ii1 t b r e d z ) ((pos "Kj%" ) ))
( "sweetbriar"  n ( s w ii1 t b r ai @ r ) ((pos "L@$" ) ))
( "sweetbrier"  n ( s w ii1 t b r ai @ r ) ((pos "L@$" ) ))
( "sweeten"  v ( s w ii1 t n ) ((pos "J0%" ) (subcat "2A" )))
( "sweetened"  v ( s w ii1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "sweetening"  v ( s w ii1 t n i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" )))
( "sweetenings"  n ( s w ii1 t n i ng z ) ((pos "Mj%" ) ))
( "sweetens"  v ( s w ii1 t n z ) ((pos "Ja%" ) (subcat "2A" )))
( "sweeter"  j ( s w ii1 t @ r ) ((pos "Or%" ) ))
( "sweetest"  j ( s w ii1 t i s t ) ((pos "Os%" ) ))
( "sweetheart"  n ( s w ii1 t h aa t ) ((pos "K6%" ) ))
( "sweethearts"  n ( s w ii1 t h aa t s ) ((pos "Kj%" ) ))
( "sweetie"  n ( s w ii1 t ii ) ((pos "K6%" ) ))
( "sweeties"  n ( s w ii1 t i z ) ((pos "Kj%" ) ))
( "sweetish"  j ( s w ii1 t i sh ) ((pos "OA%" ) ))
( "sweetly"  a ( s w ii1 t l ii ) ((pos "Pu%" ) ))
( "sweetmeat"  n ( s w ii1 t m ii t ) ((pos "K6%" ) ))
( "sweetmeats"  n ( s w ii1 t m ii t s ) ((pos "Kj%" ) ))
( "sweetness"  n ( s w ii1 t n @ s ) ((pos "L@%" ) ))
( "sweets"  n ( s w ii1 t s ) ((pos "Mj%" ) ))
( "swell"  v ( s w e1 l ) ((pos "J0%" "K6%" "OA%" ) (subcat "2A" "2C" "6A" "14" )))
( "swelled"  v ( s w e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "swelling"  v ( s w e1 l i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2C" "6A" "14" )))
( "swellings"  n ( s w e1 l i ng z ) ((pos "Mj%" ) ))
( "swells"  v ( s w e1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" )))
( "swelter"  v ( s w e1 l t @ r ) ((pos "I0%" ) (subcat )))
( "sweltered"  v ( s w e1 l t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "sweltering"  v ( s w e1 l t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "swelters"  v ( s w e1 l t @ z ) ((pos "Ia%" ) (subcat )))
( "swept"  v ( s w e1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12A" "15A" "15B" )))
( "sweptback"  j ( s w e1 p t b a k ) ((pos "OA%" ) ))
( "swerve"  v ( s w @@1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "4A" )))
( "swerved"  v ( s w @@1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" )))
( "swerves"  v ( s w @@1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4A" )))
( "swerving"  v ( s w @@1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" )))
( "swift"  n ( s w i1 f t ) ((pos "K6%" "OC%" ) ))
( "swifter"  j ( s w i1 f t @ r ) ((pos "Or%" ) ))
( "swiftest"  j ( s w i1 f t i s t ) ((pos "Os%" ) ))
( "swiftly"  a ( s w i1 f t l ii ) ((pos "Pu%" ) ))
( "swiftness"  n ( s w i1 f t n @ s ) ((pos "L@%" ) ))
( "swifts"  n ( s w i1 f t s ) ((pos "Kj%" ) ))
( "swig"  v ( s w i1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "swigged"  v ( s w i1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "swigging"  v ( s w i1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "swigs"  v ( s w i1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "swill"  v ( s w i1 l ) ((pos "J0%" "M6%" ) (subcat "6A" )))
( "swilled"  v ( s w i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "swilling"  v ( s w i1 l i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "swills"  v ( s w i1 l z ) ((pos "Ja%" "Mj%" ) (subcat "6A" )))
( "swim"  v ( s w i1 m ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "swimmer"  n ( s w i1 m @ r ) ((pos "K6%" ) ))
( "swimmers"  n ( s w i1 m @ z ) ((pos "Kj%" ) ))
( "swimming"  v ( s w i1 m i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "3A" )))
( "swimming-bath"  n ( s w i1 m i ng - b aa th ) ((pos "K6%" ) ))
( "swimming-baths"  n ( s w i1 m i ng - b aa dh z ) ((pos "Kj%" ) ))
( "swimming-costume"  n ( s w i1 m i ng - k o s t y uu m ) ((pos "K6%" ) ))
( "swimming-costumes"  n ( s w i1 m i ng - k o s t y uu m z ) ((pos "Kj%" ) ))
( "swimming-pool"  n ( s w i1 m i ng - p uu l ) ((pos "K6%" ) ))
( "swimming-pools"  n ( s w i1 m i ng - p uu l z ) ((pos "Kj%" ) ))
( "swimming-trunks"  n ( s w i1 m i ng - t r uh n k s ) ((pos "Kj%" ) ))
( "swimmingly"  a ( s w i1 m i ng l ii ) ((pos "Pu%" ) ))
( "swims"  v ( s w i1 m z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "swimsuit"  n ( s w i1 m s uu t ) ((pos "K6%" ) ))
( "swimsuits"  n ( s w i1 m s uu t s ) ((pos "Kj%" ) ))
( "swindle"  v ( s w i1 n d l ) ((pos "J2%" "K6%" ) (subcat "6A" )))
( "swindled"  v ( s w i1 n d l d ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "swindler"  n ( s w i1 n d l @ r ) ((pos "K6%" ) ))
( "swindlers"  n ( s w i1 n d l @ z ) ((pos "Kj%" ) ))
( "swindles"  v ( s w i1 n d l z ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "swindling"  v ( s w i1 n d l i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "swine"  n ( s w ai1 n ) ((pos "K9%" ) ))
( "swineherd"  n ( s w ai1 n h @@ d ) ((pos "K6%" ) ))
( "swineherds"  n ( s w ai1 n h @@ d z ) ((pos "Kj%" ) ))
( "swing"  v ( s w i1 ng ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "6C" "15A" "15B" )))
( "swinge"  v ( s w i1 n jh ) ((pos "H2$" ) ))
( "swinged"  v ( s w i1 n jh d ) ((pos "Hc$" "Hd$" ) ))
( "swingeing"  j ( s w i1 n jh i ng ) ((pos "OA%" ) ))
( "swinges"  v ( s w i1 n jh i z ) ((pos "Ha$" ) ))
( "swinging"  v ( s w i1 ng i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "6A" "6C" "15A" "15B" )))
( "swinging"  v ( s w i1 n jh i ng ) ((pos "Hb$" ) ))
( "swings"  v ( s w i1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "6C" "15A" "15B" )))
( "swinish"  j ( s w ai1 n i sh ) ((pos "OA%" ) ))
( "swipe"  v ( s w ai1 p ) ((pos "H2%" "K6%" ) (subcat "3A" "6A" )))
( "swiped"  v ( s w ai1 p t ) ((pos "Hc%" "Hd%" ) (subcat "3A" "6A" )))
( "swipes"  v ( s w ai1 p s ) ((pos "Ha%" "Kj%" ) (subcat "3A" "6A" )))
( "swiping"  v ( s w ai1 p i ng ) ((pos "Hb%" ) (subcat "3A" "6A" )))
( "swirl"  v ( s w @@1 l ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "swirled"  v ( s w @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "swirling"  v ( s w @@1 l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "swirls"  v ( s w @@1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "swish"  v ( s w i1 sh ) ((pos "J1%" "K7%" "OA%" ) (subcat "2A" "6A" )))
( "swished"  v ( s w i1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "swishes"  v ( s w i1 sh i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "swishing"  v ( s w i1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "switch"  v ( s w i1 ch ) ((pos "J1%" "K7%" ) (subcat "6A" "15A" )))
( "switchboard"  n ( s w i1 ch b oo d ) ((pos "K6%" ) ))
( "switchboards"  n ( s w i1 ch b oo d z ) ((pos "Kj%" ) ))
( "switched"  v ( s w i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15A" )))
( "switches"  v ( s w i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "6A" "15A" )))
( "switching"  v ( s w i1 ch i ng ) ((pos "Jb%" ) (subcat "6A" "15A" )))
( "switchman"  n ( s w i1 ch m @ n ) ((pos "Ki$" ) ))
( "switchmen"  n ( s w i1 ch m @ n ) ((pos "Kj$" ) ))
( "swivel"  v ( s w i1 v @ l ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "swivelled"  v ( s w i1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "swivelling"  v ( s w i1 v @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "swivels"  v ( s w i1 v @ l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "swiz"  n ( s w i1 z ) ((pos "Ki%" ) ))
( "swizzle"  n ( s w i1 z l ) ((pos "K6%" ) ))
( "swizzle-stick"  n ( s w i1 z l - s t i k ) ((pos "K6%" ) ))
( "swizzle-sticks"  n ( s w i1 z l - s t i k s ) ((pos "Kj%" ) ))
( "swizzles"  n ( s w i1 z l z ) ((pos "Kj$" ) ))
( "swob"  v ( s w o1 b ) ((pos "H4$" "K6$" ) ))
( "swobbed"  v ( s w o1 b d ) ((pos "Hc$" "Hd$" ) ))
( "swobbing"  v ( s w o1 b i ng ) ((pos "Hb$" ) ))
( "swobs"  v ( s w o1 b z ) ((pos "Ha$" "Kj$" ) ))
( "swollen"  v ( s w ou1 l @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "14" )))
( "swollen-headed"  j ( s w ou2 l @ n - h e1 d i d ) ((pos "OA%" ) ))
( "swoon"  v ( s w uu1 n ) ((pos "I0%" "K6%" ) (subcat )))
( "swooned"  v ( s w uu1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "swooning"  v ( s w uu1 n i ng ) ((pos "Ib%" ) (subcat )))
( "swoons"  v ( s w uu1 n z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "swoop"  v ( s w uu1 p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "swooped"  v ( s w uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "swooping"  v ( s w uu1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "swoops"  v ( s w uu1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "swop"  v ( s w o1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "6A" )))
( "swopped"  v ( s w o1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "swopping"  v ( s w o1 p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "swops"  v ( s w o1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "sword"  n ( s oo1 d ) ((pos "K6%" ) ))
( "sword-cane"  n ( s oo1 d - k ei n ) ((pos "K6$" ) ))
( "sword-canes"  n ( s oo1 d - k ei n z ) ((pos "Kj$" ) ))
( "sword-cut"  n ( s oo1 d - k uh t ) ((pos "K6%" ) ))
( "sword-cuts"  n ( s oo1 d - k uh t s ) ((pos "Kj%" ) ))
( "sword-dance"  n ( s oo1 d - d aa n s ) ((pos "K6%" ) ))
( "sword-dances"  n ( s oo1 d - d aa n s i z ) ((pos "Kj%" ) ))
( "swordfish"  n ( s oo1 d f i sh ) ((pos "K9%" ) ))
( "swordfishes"  n ( s oo1 d f i sh i z ) ((pos "Kj%" ) ))
( "swordplay"  n ( s oo1 d p l ei ) ((pos "L@%" ) ))
( "swords"  n ( s oo1 d z ) ((pos "Kj%" ) ))
( "swordsman"  n ( s oo1 d z m @ n ) ((pos "Ki%" ) ))
( "swordsmanship"  n ( s oo1 d z m @ n sh i p ) ((pos "L@%" ) ))
( "swordsmen"  n ( s oo1 d z m @ n ) ((pos "Kj%" ) ))
( "swordstick"  n ( s oo1 d s t i k ) ((pos "K6%" ) ))
( "swordsticks"  n ( s oo1 d s t i k s ) ((pos "Kj%" ) ))
( "swore"  v ( s w oo1 r ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "sworn"  v ( s w oo1 n ) ((pos "Jd%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "swot"  v ( s w o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "swots"  v ( s w o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "swotted"  v ( s w o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "swotting"  v ( s w o1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "swum"  v ( s w uh1 m ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "3A" )))
( "swung"  v ( s w uh1 ng ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "6C" "15A" "15B" )))
( "sybarite"  n ( s i1 b @ r ai t ) ((pos "K6$" ) ))
( "sybarites"  n ( s i1 b @ r ai t s ) ((pos "Kj$" ) ))
( "sybaritic"  j ( s i2 b @ r i1 t i k ) ((pos "OA$" ) ))
( "sycamore"  n ( s i1 k @ m oo r ) ((pos "M6%" ) ))
( "sycamores"  n ( s i1 k @ m oo z ) ((pos "Mj%" ) ))
( "sycophancy"  n ( s i1 k @ f @ n s ii ) ((pos "L@$" ) ))
( "sycophant"  n ( s i1 k @ f a n t ) ((pos "K6%" ) ))
( "sycophantic"  j ( s i2 k @ f a1 n t i k ) ((pos "OA%" ) ))
( "sycophants"  n ( s i1 k @ f a n t s ) ((pos "Kj%" ) ))
( "syllabaries"  n ( s i1 l @ b @ r i z ) ((pos "Kj$" ) ))
( "syllabary"  n ( s i1 l @ b @ r ii ) ((pos "K8$" ) ))
( "syllabi"  n ( s i1 l @ b ai ) ((pos "Kj$" ) ))
( "syllabic"  j ( s i1 l a1 b i k ) ((pos "OA%" ) ))
( "syllabicate"  v ( s i1 l a1 b i k ei t ) ((pos "H2$" ) ))
( "syllabicated"  v ( s i1 l a1 b i k ei t i d ) ((pos "Hc$" "Hd$" ) ))
( "syllabicates"  v ( s i1 l a1 b i k ei t s ) ((pos "Ha$" ) ))
( "syllabicating"  v ( s i1 l a1 b i k ei t i ng ) ((pos "Hb$" ) ))
( "syllabication"  n ( s i1 l a2 b i k ei1 sh @ n ) ((pos "L@$" ) ))
( "syllabification"  n ( s i1 l a2 b i f i k ei1 sh @ n ) ((pos "L@$" ) ))
( "syllabified"  v ( s i1 l a1 b i f ai d ) ((pos "Hc$" "Hd$" ) ))
( "syllabifies"  v ( s i1 l a1 b i f ai z ) ((pos "Ha$" ) ))
( "syllabify"  v ( s i1 l a1 b i f ai ) ((pos "H3$" ) ))
( "syllabifying"  v ( s i1 l a1 b i f ai i ng ) ((pos "Hb$" ) ))
( "syllabize"  v ( s i1 l @ b ai z ) ((pos "H2$" ) ))
( "syllabized"  v ( s i1 l @ b ai z d ) ((pos "Hc$" "Hd$" ) ))
( "syllabizes"  v ( s i1 l @ b ai z i z ) ((pos "Ha$" ) ))
( "syllabizing"  v ( s i1 l @ b ai z i ng ) ((pos "Hb$" ) ))
( "syllable"  n ( s i1 l @ b l ) ((pos "K6%" ) ))
( "syllabled"  j ( s i1 l @ b l d ) ((pos "Ot%" ) ))
( "syllables"  n ( s i1 l @ b l z ) ((pos "Kj%" ) ))
( "syllabus"  n ( s i1 l @ b @ s ) ((pos "K7%" ) ))
( "syllabuses"  n ( s i1 l @ b @ s i z ) ((pos "Kj%" ) ))
( "syllogism"  n ( s i1 l @ jh i z @ m ) ((pos "K6%" ) ))
( "syllogisms"  n ( s i1 l @ jh i z @ m z ) ((pos "Kj%" ) ))
( "syllogistic"  j ( s i2 l @ jh i1 s t i k ) ((pos "OA%" ) ))
( "sylph"  n ( s i1 l f ) ((pos "K6$" ) ))
( "sylph-like"  j ( s i1 l f - l ai k ) ((pos "OA%" ) ))
( "sylphs"  n ( s i1 l f s ) ((pos "Kj$" ) ))
( "sylvan"  j ( s i1 l v @ n ) ((pos "OA$" ) ))
( "symbiosis"  n ( s i2 m b i ou1 s i s ) ((pos "L@$" ) ))
( "symbol"  n ( s i1 m b l ) ((pos "K6%" ) ))
( "symbolic"  j ( s i1 m b o1 l i k ) ((pos "OA%" ) ))
( "symbolical"  j ( s i1 m b o1 l i k l ) ((pos "OA$" ) ))
( "symbolically"  a ( s i1 m b o1 l i k l ii ) ((pos "Pu%" ) ))
( "symbolism"  n ( s i1 m b @ l i z @ m ) ((pos "M6%" ) ))
( "symbolisms"  n ( s i1 m b @ l i z @ m z ) ((pos "Mj%" ) ))
( "symbolization"  n ( s i2 m b @ l ai z ei1 sh @ n ) ((pos "M6$" ) ))
( "symbolizations"  n ( s i2 m b @ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "symbolize"  v ( s ii1 m b @ l ai z ) ((pos "H2%" ) (subcat )))
( "symbolized"  v ( s ii1 m b @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "symbolizes"  v ( s ii1 m b @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "symbolizing"  v ( s ii1 m b @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "symbols"  n ( s i1 m b l z ) ((pos "Kj%" ) ))
( "symmetric"  j ( s i1 m e1 t r i k ) ((pos "OA$" ) ))
( "symmetrical"  j ( s i1 m e1 t r i k l ) ((pos "OA%" ) ))
( "symmetrically"  a ( s i1 m e1 t r i k l ii ) ((pos "Pu%" ) ))
( "symmetry"  n ( s i1 m @ t r ii ) ((pos "L@%" ) ))
( "sympathetic"  j ( s i2 m p @ th e1 t i k ) ((pos "OA%" ) ))
( "sympathetically"  a ( s i2 m p @ th e1 t i k l ii ) ((pos "Pu%" ) ))
( "sympathies"  n ( s i1 m p @ th i z ) ((pos "Mj%" ) ))
( "sympathize"  v ( s i1 m p @ th ai z ) ((pos "I2%" ) (subcat "2A" )))
( "sympathized"  v ( s i1 m p @ th ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "sympathizer"  n ( s i1 m p @ th ai z @ r ) ((pos "K6%" ) ))
( "sympathizers"  n ( s i1 m p @ th ai z @ z ) ((pos "Kj%" ) ))
( "sympathizes"  v ( s i1 m p @ th ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "sympathizing"  v ( s i1 m p @ th ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "sympathy"  n ( s i1 m p @ th ii ) ((pos "M8%" ) ))
( "symphonic"  j ( s i1 m f o1 n i k ) ((pos "OA%" ) ))
( "symphonies"  n ( s i1 m f @ n i z ) ((pos "Kj%" ) ))
( "symphony"  n ( s i1 m f @ n ii ) ((pos "K8%" ) ))
( "symposia"  n ( s i1 m p ou1 z i@ ) ((pos "Kj%" ) ))
( "symposium"  n ( s i1 m p ou1 z i@ m ) ((pos "K6%" ) ))
( "symposiums"  n ( s i1 m p ou1 z i@ m z ) ((pos "Kj%" ) ))
( "symptom"  n ( s i1 m p t @ m ) ((pos "K6%" ) ))
( "symptomatic"  j ( s i2 m p t @ m a1 t i k ) ((pos "OA%" ) ))
( "symptomatically"  a ( s i2 m p t @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "symptomless"  j ( s i1 m p t @ m l i s ) ((pos "OA%" ) ))
( "symptoms"  n ( s i1 m p t @ m z ) ((pos "Kj%" ) ))
( "synagogue"  n ( s i1 n @ g o g ) ((pos "K6%" ) ))
( "synagogues"  n ( s i1 n @ g o g z ) ((pos "Kj%" ) ))
( "synchroflash"  n ( s i1 ng k r ou f l a sh ) ((pos "Ki$" ) ))
( "synchromesh"  n ( s i2 ng k r ou m e1 sh ) ((pos "L@%" ) ))
( "synchronization"  n ( s i2 ng k r @ n ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "synchronizations"  n ( s i2 ng k r @ n ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "synchronize"  v ( s i1 ng k r @ n ai z ) ((pos "J2%" ) (subcat "2A" )))
( "synchronized"  v ( s i1 ng k r @ n ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "synchronizes"  v ( s i1 ng k r @ n ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "synchronizing"  v ( s i1 ng k r @ n ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "synchronous"  j ( s i1 ng k r @ n @ s ) ((pos "OA$" ) ))
( "synchronously"  a ( s i1 ng k r @ n @ s l ii ) ((pos "Pu$" ) ))
( "synchrony"  n ( s i1 ng k r @ n ii ) ((pos "L@$" ) ))
( "synchrotron"  n ( s i1 ng k r ou t r o n ) ((pos "K6$" ) ))
( "synchrotrons"  n ( s i1 ng k r ou t r o n z ) ((pos "Kj$" ) ))
( "syncopate"  v ( s i1 ng k @ p ei t ) ((pos "H2%" ) (subcat )))
( "syncopated"  v ( s i1 ng k @ p ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "syncopates"  v ( s i1 ng k @ p ei t s ) ((pos "Ha%" ) (subcat )))
( "syncopating"  v ( s i1 ng k @ p ei t i ng ) ((pos "Hb%" ) (subcat )))
( "syncopation"  n ( s i2 ng k @ p ei1 sh @ n ) ((pos "M6%" ) ))
( "syncopations"  n ( s i2 ng k @ p ei1 sh @ n z ) ((pos "Mj%" ) ))
( "syncope"  n ( s i1 ng k @ p ii ) ((pos "L@$" ) ))
( "syndic"  n ( s i1 n d i k ) ((pos "K6$" ) ))
( "syndicalism"  n ( s i1 n d i k @ l i z @ m ) ((pos "L@%" ) ))
( "syndicalist"  n ( s i1 n d i k @ l i s t ) ((pos "K6%" ) ))
( "syndicalists"  n ( s i1 n d i k @ l i s t s ) ((pos "Kj%" ) ))
( "syndicate"  n ( s i1 n d i k @ t ) ((pos "K6%" ) ))
( "syndicate"  v ( s i1 n d i k ei t ) ((pos "H2%" ) (subcat )))
( "syndicated"  v ( s i1 n d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "syndicates"  n ( s i1 n d i k @ t s ) ((pos "Kj%" ) ))
( "syndicates"  v ( s i1 n d i k ei t s ) ((pos "Ha%" ) (subcat )))
( "syndicating"  v ( s i1 n d i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "syndication"  n ( s i2 n d i k ei1 sh @ n ) ((pos "K6$" ) ))
( "syndications"  n ( s i2 n d i k ei1 sh @ n z ) ((pos "Kj$" ) ))
( "syndics"  n ( s i1 n d i k s ) ((pos "Kj$" ) ))
( "syndrome"  n ( s i1 n d r ou m ) ((pos "K6%" ) ))
( "syndromes"  n ( s i1 n d r ou m z ) ((pos "Kj%" ) ))
( "synod"  n ( s i1 n @ d ) ((pos "K6%" ) ))
( "synods"  n ( s i1 n @ d z ) ((pos "Kj%" ) ))
( "synonym"  n ( s i1 n @ n i m ) ((pos "K6%" ) ))
( "synonymous"  j ( s i1 n o1 n i m @ s ) ((pos "OA%" ) ))
( "synonyms"  n ( s i1 n @ n i m z ) ((pos "Kj%" ) ))
( "synopses"  n ( s i1 n o1 p s ii z ) ((pos "Kj%" ) ))
( "synopsis"  n ( s i1 n o1 p s i s ) ((pos "Ki%" ) ))
( "synoptic"  j ( s i1 n o1 p t i k ) ((pos "OA$" ) ))
( "synoptically"  a ( s i1 n o1 p t i k l ii ) ((pos "Pu$" ) ))
( "syntactic"  j ( s i1 n t a1 k t i k ) ((pos "OA%" ) ))
( "syntactically"  a ( s i1 n t a1 k t i k l ii ) ((pos "Pu%" ) ))
( "syntax"  n ( s i1 n t a k s ) ((pos "L@%" ) ))
( "syntheses"  n ( s i1 n th @ s ii z ) ((pos "Kj$" ) ))
( "synthesis"  n ( s i1 n th @ s i s ) ((pos "Mi%" ) ))
( "synthesize"  v ( s i1 n th @ s ai z ) ((pos "H2%" ) (subcat )))
( "synthesized"  v ( s i1 n th @ s ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "synthesizes"  v ( s i1 n th @ s ai z i z ) ((pos "Ha%" ) (subcat )))
( "synthesizing"  v ( s i1 n th @ s ai z i ng ) ((pos "Hb%" ) (subcat )))
( "synthetic"  n ( s i1 n th e1 t i k ) ((pos "K6$" "OA%" ) ))
( "synthetically"  a ( s i1 n th e1 t i k l ii ) ((pos "Pu%" ) ))
( "synthetics"  n ( s i1 n th e1 t i k s ) ((pos "Kj%" ) ))
( "syphilis"  n ( s i1 f i l i s ) ((pos "L@%" ) ))
( "syphilitic"  n ( s i2 f i l i1 t i k ) ((pos "K6%" "OA%" ) ))
( "syphilitics"  n ( s i2 f i l i1 t i k s ) ((pos "Kj%" ) ))
( "syphon"  v ( s ai1 f @ n ) ((pos "J0%" "K6%" ) (subcat )))
( "syphoned"  v ( s ai1 f @ n d ) ((pos "Jc%" "Jd%" ) (subcat )))
( "syphoning"  v ( s ai1 f @ n i ng ) ((pos "Jb%" ) (subcat )))
( "syphons"  v ( s ai1 f @ n z ) ((pos "Ja%" "Kj%" ) (subcat )))
( "syringa"  n ( s i1 r i1 ng g @ ) ((pos "M6$" ) ))
( "syringas"  n ( s i1 r i1 ng g @ z ) ((pos "Mj$" ) ))
( "syringe"  v ( s i1 r i1 n jh ) ((pos "H2%" "K6%" ) (subcat "6A" )))
( "syringed"  v ( s i1 r i1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "syringes"  v ( s i1 r i1 n jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "syringing"  v ( s i1 r i1 n jh i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "syrup"  n ( s i1 r @ p ) ((pos "M6%" ) ))
( "syrups"  n ( s i1 r @ p s ) ((pos "Mj%" ) ))
( "syrupy"  j ( s i1 r @ p ii ) ((pos "OA%" ) ))
( "system"  n ( s i1 s t @ m ) ((pos "M6*" ) ))
( "systematic"  j ( s i2 s t @ m a1 t i k ) ((pos "OA%" ) ))
( "systematically"  a ( s i2 s t @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "systematization"  n ( s i2 s t @ m @ t ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "systematize"  v ( s i1 s t @ m @ t ai z ) ((pos "H2%" ) (subcat )))
( "systematized"  v ( s i1 s t @ m @ t ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "systematizes"  v ( s i1 s t @ m @ t ai z i z ) ((pos "Ha%" ) (subcat )))
( "systematizing"  v ( s i1 s t @ m @ t ai z i ng ) ((pos "Hb%" ) (subcat )))
( "systems"  n ( s i1 s t @ m z ) ((pos "Mj%" ) ))
( "t"  n ( t ii1 ) ((pos "Ki$" ) ))
( "t's"  n ( t ii1 z ) ((pos "Kj$" ) ))
( "t^ete-`a-t^ete"  n ( t ei2 t - aa - t ei1 t ) ((pos "K6%" "Pu%" ) ))
( "t^ete-`a-t^etes"  n ( t ei2 t - aa - t ei1 t s ) ((pos "Kj$" ) ))
( "ta"  uh ( t aa1 ) ((pos "W-$" ) ))
( "ta ta"  uh ( t @2 - t aa1 ) ((pos "W-%" ) ))
( "tab"  n ( t a1 b ) ((pos "K6%" ) ))
( "tabard"  n ( t a1 b @ d ) ((pos "K6$" ) ))
( "tabards"  n ( t a1 b @ d z ) ((pos "Kj$" ) ))
( "tabbies"  n ( t a1 b i z ) ((pos "Kj$" ) ))
( "tabby"  n ( t a1 b ii ) ((pos "K8%" "Oq%" ) ))
( "tabby-cat"  n ( t a1 b i - k a t ) ((pos "K6%" ) ))
( "tabby-cats"  n ( t a1 b i - k a t s ) ((pos "Kj%" ) ))
( "tabernacle"  n ( t a1 b @ n a k l ) ((pos "K6%" ) ))
( "tabernacles"  n ( t a1 b @ n a k l z ) ((pos "Kj%" ) ))
( "table"  v ( t ei1 b l ) ((pos "H2%" "K6*" ) (subcat )))
( "table d'h^ote"  j ( t aa2 b l - d ou1 t ) ((pos "OA%" "Pu%" ) ))
( "table-knife"  n ( t ei1 b l - n ai f ) ((pos "Ki%" ) ))
( "table-knives"  n ( t ei1 b l - n ai v z ) ((pos "Kj%" ) ))
( "table-lifting"  n ( t ei1 b l - l i f t i ng ) ((pos "L@$" ) ))
( "table-linen"  n ( t ei1 b l - l i n i n ) ((pos "L@%" ) ))
( "table-rapping"  n ( t ei1 b l - r a p i ng ) ((pos "L@$" ) ))
( "table-talk"  n ( t ei1 b l - t oo k ) ((pos "L@%" ) ))
( "table-turning"  n ( t ei1 b l - t @@ n i ng ) ((pos "L@%" ) ))
( "tableau"  n ( t a1 b l ou ) ((pos "Ki%" ) ))
( "tableau vivant"  n ( t a2 b l ou - v ii1 v aa n ) ((pos "Ki$" ) ))
( "tableaux"  n ( t a1 b l ou z ) ((pos "Kj$" ) ))
( "tableaux vivants"  n ( t a2 b l ou - v ii1 v aa n ) ((pos "Kj$" ) ))
( "tablecloth"  n ( t ei1 b l k l o th ) ((pos "K6%" ) ))
( "tablecloths"  n ( t ei1 b l k l o th s ) ((pos "Kj%" ) ))
( "tabled"  v ( t ei1 b l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tablemat"  n ( t ei1 b l m a t ) ((pos "K6%" ) ))
( "tablemats"  n ( t ei1 b l m a t s ) ((pos "Kj%" ) ))
( "tables"  v ( t ei1 b l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "tablespoon"  n ( t ei1 b l s p uu n ) ((pos "K6%" ) ))
( "tablespoonful"  n ( t ei1 b l s p uu n f u l ) ((pos "K6%" ) ))
( "tablespoonfuls"  n ( t ei1 b l s p uu n f u l z ) ((pos "Kj%" ) ))
( "tablespoons"  n ( t ei1 b l s p uu n z ) ((pos "Kj%" ) ))
( "tablet"  n ( t a1 b l i t ) ((pos "K6%" ) ))
( "tablets"  n ( t a1 b l i t s ) ((pos "Kj%" ) ))
( "tableware"  n ( t ei1 b l w e@ r ) ((pos "L@%" ) ))
( "tabling"  v ( t ei1 b l i ng ) ((pos "Hb%" ) (subcat )))
( "tabloid"  n ( t a1 b l oi d ) ((pos "K6%" ) ))
( "tabloids"  n ( t a1 b l oi d z ) ((pos "Kj%" ) ))
( "taboo"  v ( t @1 b uu1 ) ((pos "H0$" "M6%" "OA%" ) (subcat )))
( "tabooed"  v ( t @1 b uu1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "tabooing"  v ( t @1 b uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "taboos"  v ( t @1 b uu1 z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "tabor"  n ( t ei1 b @ r ) ((pos "K6$" ) ))
( "tabors"  n ( t ei1 b @ z ) ((pos "Kj$" ) ))
( "tabs"  n ( t a1 b z ) ((pos "Kj%" ) ))
( "tabular"  j ( t a1 b y u l @ r ) ((pos "OA%" ) ))
( "tabulate"  v ( t a1 b y u l ei t ) ((pos "H2%" ) (subcat )))
( "tabulated"  v ( t a1 b y u l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tabulates"  v ( t a1 b y u l ei t s ) ((pos "Ha%" ) (subcat )))
( "tabulating"  v ( t a1 b y u l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "tabulation"  n ( t a2 b y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "tabulations"  n ( t a2 b y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "tabulator"  n ( t a1 b y u l ei t @ r ) ((pos "K6%" ) ))
( "tabulators"  n ( t a1 b y u l ei t @ z ) ((pos "Kj%" ) ))
( "tachograph"  n ( t a1 k ou g r aa f ) ((pos "K6%" ) ))
( "tachographs"  n ( t a1 k ou g r aa f s ) ((pos "Kj%" ) ))
( "tacit"  j ( t a1 s i t ) ((pos "OA%" ) ))
( "tacitly"  a ( t a1 s i t l ii ) ((pos "Pu%" ) ))
( "taciturn"  j ( t a1 s i t @@ n ) ((pos "OA%" ) ))
( "taciturnity"  n ( t a2 s i t @@1 n i t ii ) ((pos "L@%" ) ))
( "taciturnly"  a ( t a1 s i t @@ n l ii ) ((pos "Pu%" ) ))
( "tack"  v ( t a1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "tacked"  v ( t a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "tackier"  j ( t a1 k i@ r ) ((pos "Or%" ) ))
( "tackiest"  j ( t a1 k i i s t ) ((pos "Os%" ) ))
( "tacking"  v ( t a1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "tackle"  v ( t a1 k l ) ((pos "J2%" "M6%" ) (subcat "2A" "6A" )))
( "tackled"  v ( t a1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "tackles"  v ( t a1 k l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "tackling"  v ( t a1 k l i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "tacks"  v ( t a1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "tacky"  j ( t a1 k ii ) ((pos "OD%" ) ))
( "tact"  n ( t a1 k t ) ((pos "L@%" ) ))
( "tactful"  j ( t a1 k t f u l ) ((pos "OA%" ) ))
( "tactfully"  a ( t a1 k t f @ l ii ) ((pos "Pu%" ) ))
( "tactic"  n ( t a1 k t i k ) ((pos "K6%" ) ))
( "tactical"  j ( t a1 k t i k l ) ((pos "OA%" ) ))
( "tactically"  a ( t a1 k t i k l ii ) ((pos "Pu%" ) ))
( "tactician"  n ( t a1 k t i1 sh @ n ) ((pos "K6%" ) ))
( "tacticians"  n ( t a1 k t i1 sh @ n z ) ((pos "Kj%" ) ))
( "tactics"  n ( t a1 k t i k s ) ((pos "Kj%" ) ))
( "tactile"  j ( t a1 k t ai l ) ((pos "OA%" ) ))
( "tactless"  j ( t a1 k t l @ s ) ((pos "OA%" ) ))
( "tactlessly"  a ( t a1 k t l @ s l ii ) ((pos "Pu%" ) ))
( "tactlessness"  n ( t a1 k t l @ s n @ s ) ((pos "L@%" ) ))
( "tactual"  j ( t a1 k ch u@ l ) ((pos "OA$" ) ))
( "tadpole"  n ( t a1 d p ou l ) ((pos "K6%" ) ))
( "tadpoles"  n ( t a1 d p ou l z ) ((pos "Kj%" ) ))
( "taffeta"  n ( t a1 f i t @ ) ((pos "L@$" ) ))
( "taffies"  n ( t a1 f i z ) ((pos "Kj$" ) ))
( "taffrail"  n ( t a1 f r ei l ) ((pos "K6$" ) ))
( "taffrails"  n ( t a1 f r ei l z ) ((pos "Kj$" ) ))
( "taffy"  n ( t a1 f ii ) ((pos "K8$" ) ))
( "tag"  v ( t a1 g ) ((pos "J4%" "M6%" ) (subcat "2C" "6A" "14" "15A" )))
( "tagged"  v ( t a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "14" "15A" )))
( "tagging"  v ( t a1 g i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "14" "15A" )))
( "tags"  v ( t a1 g z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" "14" "15A" )))
( "tail"  v ( t ei1 l ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" )))
( "tail-coat"  n ( t ei1 l - k ou t ) ((pos "K6%" ) ))
( "tail-coats"  n ( t ei1 l - k ou t s ) ((pos "Kj%" ) ))
( "tail-end"  n ( t ei1 l - e1 n d ) ((pos "K6%" ) ))
( "tail-ends"  n ( t ei1 l - e1 n d z ) ((pos "Kj%" ) ))
( "tail-light"  n ( t ei1 l - l ai t ) ((pos "K6%" ) ))
( "tail-lights"  n ( t ei1 l - l ai t s ) ((pos "Kj%" ) ))
( "tailboard"  n ( t ei1 l b oo d ) ((pos "K6%" ) ))
( "tailboards"  n ( t ei1 l b oo d z ) ((pos "Kj%" ) ))
( "tailed"  v ( t ei1 l d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2C" "3A" )))
( "tailgate"  n ( t ei1 l g ei t ) ((pos "K6%" ) ))
( "tailgates"  n ( t ei1 l g ei t s ) ((pos "Kj%" ) ))
( "tailing"  v ( t ei1 l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" )))
( "tailless"  j ( t ei1 l - l @ s ) ((pos "OA%" ) ))
( "tailor"  v ( t ei1 l @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "tailor-made"  j ( t ei1 l @ - m ei d ) ((pos "OA%" ) ))
( "tailored"  v ( t ei1 l @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tailoring"  v ( t ei1 l @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tailors"  v ( t ei1 l @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "tailpiece"  n ( t ei1 l p ii s ) ((pos "K6%" ) ))
( "tailpieces"  n ( t ei1 l p ii s i z ) ((pos "Kj%" ) ))
( "tailplane"  n ( t ei1 l p l ei n ) ((pos "K6%" ) ))
( "tailplanes"  n ( t ei1 l p l ei n z ) ((pos "Kj%" ) ))
( "tails"  v ( t ei1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" )))
( "tailspin"  n ( t ei1 l s p i n ) ((pos "K6%" ) ))
( "tailspins"  n ( t ei1 l s p i n z ) ((pos "Kj$" ) ))
( "taint"  v ( t ei1 n t ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "tainted"  v ( t ei1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tainting"  v ( t ei1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "taintless"  j ( t ei1 n t l @ s ) ((pos "OA$" ) ))
( "taints"  v ( t ei1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "take"  v ( t ei1 k ) ((pos "J5*" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "6B" "12A" "13A" "14" "15A" "15B" "16B" "19B" )))
( "take-home"  j ( t ei1 k - h ou m ) ((pos "OA%" ) ))
( "take-off"  n ( t ei1 k - o f ) ((pos "K6%" ) ))
( "take-offs"  n ( t ei1 k - o f s ) ((pos "Kj%" ) ))
( "take-up"  n ( t ei1 k - uh p ) ((pos "L@%" ) ))
( "takeaway"  j ( t ei1 k @ w ei ) ((pos "Oq%" ) ))
( "taken"  v ( t ei1 k @ n ) ((pos "Jd*" ) (subcat "2A" "2B" "2C" "3A" "6A" "6B" "12A" "13A" "14" "15A" "15B" "16B" "19B" )))
( "takeover"  n ( t ei1 k ou v @ r ) ((pos "K6%" ) ))
( "takeovers"  n ( t ei1 k ou v @ z ) ((pos "Kj%" ) ))
( "taker"  n ( t ei1 k @ r ) ((pos "K6%" ) ))
( "takers"  n ( t ei1 k @ z ) ((pos "Kj%" ) ))
( "takes"  v ( t ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "6B" "12A" "13A" "14" "15A" "15B" "16B" "19B" )))
( "taking"  v ( t ei1 k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "6B" "12A" "13A" "14" "15A" "15B" "16B" "19B" )))
( "takings"  n ( t ei1 k i ng z ) ((pos "Kj%" ) ))
( "talc"  n ( t a1 l k ) ((pos "L@%" ) ))
( "talcum"  n ( t a1 l k @ m ) ((pos "L@%" ) ))
( "tale"  n ( t ei1 l ) ((pos "K6%" ) ))
( "tale-bearer"  n ( t ei1 l - b e@ r @ r ) ((pos "K6%" ) ))
( "tale-bearers"  n ( t ei1 l - b e@ r @ z ) ((pos "Kj%" ) ))
( "tale-teller"  n ( t ei1 l - t e l @ r ) ((pos "K6%" ) ))
( "tale-tellers"  n ( t ei1 l - t e l @ z ) ((pos "Kj%" ) ))
( "talent"  n ( t a1 l @ n t ) ((pos "M6%" ) ))
( "talented"  j ( t a1 l @ n t i d ) ((pos "OA%" ) ))
( "talents"  n ( t a1 l @ n t s ) ((pos "Mj%" ) ))
( "tales"  n ( t ei1 l z ) ((pos "Kj%" ) ))
( "talisman"  n ( t a1 l i z m @ n ) ((pos "K6%" ) ))
( "talismans"  n ( t a1 l i z m @ n z ) ((pos "Kj$" ) ))
( "talk"  v ( t oo1 k ) ((pos "J0*" "M6*" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15B" )))
( "talkative"  j ( t oo1 k @ t i v ) ((pos "OA%" ) ))
( "talked"  v ( t oo1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15B" )))
( "talker"  n ( t oo1 k @ r ) ((pos "K6%" ) ))
( "talkers"  n ( t oo1 k @ z ) ((pos "Kj%" ) ))
( "talkie"  n ( t oo1 k ii ) ((pos "K6%" ) ))
( "talkies"  n ( t oo1 k i z ) ((pos "Kj%" ) ))
( "talking"  v ( t oo1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15B" )))
( "talking-point"  n ( t oo1 k i ng - p oi n t ) ((pos "K6%" ) ))
( "talking-points"  n ( t oo1 k i ng - p oi n t s ) ((pos "Kj%" ) ))
( "talking-to"  n ( t oo1 k i ng - t uu ) ((pos "K6%" ) ))
( "talking-tos"  n ( t oo1 k i ng - t uu z ) ((pos "Kj$" ) ))
( "talks"  v ( t oo1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15B" )))
( "tall"  j ( t oo1 l ) ((pos "OC%" ) ))
( "tallboy"  n ( t oo1 l b oi ) ((pos "K6%" ) ))
( "tallboys"  n ( t oo1 l b oi z ) ((pos "Kj%" ) ))
( "taller"  j ( t oo1 l @ r ) ((pos "Or%" ) ))
( "tallest"  j ( t oo1 l i s t ) ((pos "Os%" ) ))
( "tallied"  v ( t a1 l i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "tallies"  v ( t a1 l i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "tallish"  j ( t oo1 l i sh ) ((pos "OA$" ) ))
( "tallow"  n ( t a1 l ou ) ((pos "L@%" ) ))
( "tally"  v ( t a1 l ii ) ((pos "I3%" "K8%" ) (subcat "2A" )))
( "tally-clerk"  n ( t a1 l i - k l aa k ) ((pos "K6%" ) ))
( "tally-clerks"  n ( t a1 l i - k l aa k s ) ((pos "Kj%" ) ))
( "tally-ho"  uh ( t a2 l i - h ou1 ) ((pos "W-$" ) ))
( "tallying"  v ( t a1 l i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "tallyman"  n ( t a1 l i m a n ) ((pos "Ki%" ) ))
( "tallymen"  n ( t a1 l i m e n ) ((pos "Kj%" ) ))
( "talon"  n ( t a1 l @ n ) ((pos "K6%" ) ))
( "talons"  n ( t a1 l @ n z ) ((pos "Kj%" ) ))
( "talus"  n ( t ei1 l @ s ) ((pos "K7$" ) ))
( "taluses"  n ( t ei1 l @ s i z ) ((pos "Kj$" ) ))
( "tam-o'-shanter"  n ( t a2 m - @ - sh a1 n t @ r ) ((pos "K6%" ) ))
( "tam-o'-shanters"  n ( t a2 m - @ - sh a1 n t @ z ) ((pos "Kj%" ) ))
( "tamable"  j ( t ei1 m @ b l ) ((pos "OA%" ) ))
( "tamale"  n ( t @1 m aa1 l ii ) ((pos "M6$" ) ))
( "tamales"  n ( t @1 m aa1 l i z ) ((pos "Mj$" ) ))
( "tamarind"  n ( t a1 m @ r i n d ) ((pos "M6$" ) ))
( "tamarinds"  n ( t a1 m @ r i n d z ) ((pos "Mj$" ) ))
( "tamarisk"  n ( t a1 m @ r i s k ) ((pos "K6$" ) ))
( "tamarisks"  n ( t a1 m @ r i s k s ) ((pos "Kj$" ) ))
( "tambour"  n ( t a1 m b u@ r ) ((pos "K6$" ) ))
( "tambourine"  n ( t a2 m b @ r ii1 n ) ((pos "K6%" ) ))
( "tambourines"  n ( t a2 m b @ r ii1 n z ) ((pos "Kj%" ) ))
( "tambours"  n ( t a1 m b u@ z ) ((pos "Kj$" ) ))
( "tame"  v ( t ei1 m ) ((pos "H2%" "OB%" ) (subcat )))
( "tamed"  v ( t ei1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tamely"  a ( t ei1 m l ii ) ((pos "Pu%" ) ))
( "tameness"  n ( t ei1 m n @ s ) ((pos "L@%" ) ))
( "tamer"  n ( t ei1 m @ r ) ((pos "K6%" "Or%" ) ))
( "tamers"  n ( t ei1 m @ z ) ((pos "Kj%" ) ))
( "tames"  v ( t ei1 m z ) ((pos "Ha%" ) (subcat )))
( "tamest"  j ( t ei1 m i s t ) ((pos "Os%" ) ))
( "taming"  v ( t ei1 m i ng ) ((pos "Hb%" ) (subcat )))
( "tammies"  n ( t a1 m i z ) ((pos "Kj$" ) ))
( "tammy"  n ( t a1 m ii ) ((pos "K8$" ) ))
( "tamp"  v ( t a1 m p ) ((pos "H0%" ) (subcat )))
( "tamped"  v ( t a1 m p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tamper"  v ( t a1 m p @ r ) ((pos "I0%" ) (subcat )))
( "tampered"  v ( t a1 m p @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "tampering"  v ( t a1 m p @ r i ng ) ((pos "Ib%" ) (subcat )))
( "tampers"  v ( t a1 m p @ z ) ((pos "Ia%" ) (subcat )))
( "tamping"  v ( t a1 m p i ng ) ((pos "Hb%" ) (subcat )))
( "tamps"  v ( t a1 m p s ) ((pos "Ha%" ) (subcat )))
( "tan"  v ( t a1 n ) ((pos "J4%" "K6%" "OA%" ) (subcat "2A" )))
( "tandem"  n ( t a1 n d @ m ) ((pos "K6%" "Pu%" ) ))
( "tandems"  n ( t a1 n d @ m z ) ((pos "Kj%" ) ))
( "tang"  n ( t a1 ng ) ((pos "K6%" ) ))
( "tangent"  n ( t a1 n jh @ n t ) ((pos "K6%" ) ))
( "tangents"  n ( t a1 n jh @ n t s ) ((pos "Kj%" ) ))
( "tangerine"  n ( t a2 n jh @ r ii1 n ) ((pos "M6%" "OA%" ) ))
( "tangerines"  n ( t a2 n jh @ r ii1 n z ) ((pos "Mj%" ) ))
( "tangibility"  n ( t a2 n jh @ b i1 l i t ii ) ((pos "L@%" ) ))
( "tangible"  j ( t a1 n jh @ b l ) ((pos "OA%" ) ))
( "tangibly"  a ( t a1 n jh @ b l ii ) ((pos "Pu%" ) ))
( "tangier"  j ( t a1 ng i@ r ) ((pos "Or%" ) ))
( "tangiest"  j ( t a1 ng i i s t ) ((pos "Os%" ) ))
( "tangle"  v ( t a1 ng g l ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "6A" )))
( "tangled"  v ( t a1 ng g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" )))
( "tangles"  v ( t a1 ng g l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" )))
( "tangling"  v ( t a1 ng g l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" )))
( "tango"  n ( t a1 ng g ou ) ((pos "K6%" ) ))
( "tangos"  n ( t a1 ng g ou z ) ((pos "Kj%" ) ))
( "tangs"  n ( t a1 ng z ) ((pos "Kj%" ) ))
( "tangy"  j ( t a1 ng ii ) ((pos "OD%" ) ))
( "tank"  v ( t a1 ng k ) ((pos "I0$" "K6%" ) ))
( "tank-car"  n ( t a1 ng k - k aa r ) ((pos "K6$" ) ))
( "tank-cars"  n ( t a1 ng k - k aa z ) ((pos "Kj$" ) ))
( "tankard"  n ( t a1 ng k @ d ) ((pos "K6%" ) ))
( "tankards"  n ( t a1 ng k @ d z ) ((pos "Kj%" ) ))
( "tanked"  v ( t a1 ng k t ) ((pos "Ic$" "Id$" ) ))
( "tanker"  n ( t a1 ng k @ r ) ((pos "K6%" ) ))
( "tankers"  n ( t a1 ng k @ z ) ((pos "Kj%" ) ))
( "tanking"  v ( t a1 ng k i ng ) ((pos "Ib$" ) ))
( "tanks"  v ( t a1 ng k s ) ((pos "Ia$" "Kj%" ) ))
( "tanned"  v ( t a1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tanner"  n ( t a1 n @ r ) ((pos "K6%" ) ))
( "tanneries"  n ( t a1 n @ r i z ) ((pos "Kj%" ) ))
( "tanners"  n ( t a1 n @ z ) ((pos "Kj%" ) ))
( "tannery"  n ( t a1 n @ r ii ) ((pos "K8%" ) ))
( "tannic"  j ( t a1 n i k ) ((pos "OA%" ) ))
( "tannin"  n ( t a1 n i n ) ((pos "L@%" ) ))
( "tanning"  v ( t a1 n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tannoy"  n ( t a1 n oi ) ((pos "K6%" ) ))
( "tannoys"  n ( t a1 n oi z ) ((pos "Kj$" ) ))
( "tans"  v ( t a1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "tansies"  n ( t a1 n z i z ) ((pos "Kj$" ) ))
( "tansy"  n ( t a1 n z ii ) ((pos "K8$" ) ))
( "tantalize"  v ( t a1 n t @ l ai z ) ((pos "H2%" ) (subcat )))
( "tantalized"  v ( t a1 n t @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tantalizes"  v ( t a1 n t @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "tantalizing"  v ( t a1 n t @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "tantamount"  j ( t a1 n t @ m au n t ) ((pos "OA%" ) ))
( "tantrum"  n ( t a1 n t r @ m ) ((pos "K6%" ) ))
( "tantrums"  n ( t a1 n t r @ m z ) ((pos "Kj%" ) ))
( "tap"  v ( t a1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "tap-dancing"  n ( t a1 p - d aa n s i ng ) ((pos "L@%" ) ))
( "tape"  v ( t ei1 p ) ((pos "H2%" "M6%" ) (subcat )))
( "tape-measure"  n ( t ei1 p - m e zh @ r ) ((pos "K6%" ) ))
( "tape-measures"  n ( t ei1 p - m e zh @ z ) ((pos "Kj%" ) ))
( "tape-recorder"  n ( t ei1 p - r i k oo2 d @ r ) ((pos "K6%" ) ))
( "tape-recorders"  n ( t ei1 p - r i k oo2 d @ z ) ((pos "Kj%" ) ))
( "taped"  v ( t ei1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "taper"  v ( t ei1 p @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "tapered"  v ( t ei1 p @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "tapering"  v ( t ei1 p @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "tapers"  v ( t ei1 p @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "tapes"  v ( t ei1 p s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "tapestried"  j ( t a1 p i s t r i d ) ((pos "OA%" ) ))
( "tapestries"  n ( t a1 p i s t r i z ) ((pos "Mj%" ) ))
( "tapestry"  n ( t a1 p i s t r ii ) ((pos "M8%" ) ))
( "tapeworm"  n ( t ei1 p w @@ m ) ((pos "K6%" ) ))
( "tapeworms"  n ( t ei1 p w @@ m z ) ((pos "Kj%" ) ))
( "taping"  v ( t ei1 p i ng ) ((pos "Hb%" ) (subcat )))
( "tapioca"  n ( t a2 p i ou1 k @ ) ((pos "L@%" ) ))
( "tapir"  n ( t ei1 p @ r ) ((pos "K6$" ) ))
( "tapirs"  n ( t ei1 p @ z ) ((pos "Kj$" ) ))
( "tapped"  v ( t a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "tapping"  v ( t a1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "taproom"  n ( t a1 p r uu m ) ((pos "K6%" ) ))
( "taprooms"  n ( t a1 p r uu m z ) ((pos "Kj%" ) ))
( "taproot"  n ( t a1 p r uu t ) ((pos "K6%" ) ))
( "taproots"  n ( t a1 p r uu t s ) ((pos "Kj%" ) ))
( "taps"  v ( t a1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "14" "15A" )))
( "tapster"  n ( t a1 p s t @ r ) ((pos "K6%" ) ))
( "tapsters"  n ( t a1 p s t @ z ) ((pos "Kj%" ) ))
( "tar"  v ( t aa1 r ) ((pos "H4%" "M6%" ) (subcat )))
( "tar-macadam"  n ( t aa2 - m @ k a1 d @ m ) ((pos "L@%" ) ))
( "taradiddle"  n ( t a1 r @ d i d l ) ((pos "K6$" ) ))
( "taradiddles"  n ( t a1 r @ d i d l z ) ((pos "Kj$" ) ))
( "tarantella"  n ( t a2 r @ n t e1 l @ ) ((pos "K6$" ) ))
( "tarantellas"  n ( t a2 r @ n t e1 l @ z ) ((pos "Kj$" ) ))
( "tarantelle"  n ( t a2 r @ n t e1 l ) ((pos "K6$" ) ))
( "tarantelles"  n ( t a2 r @ n t e1 l z ) ((pos "Kj$" ) ))
( "tarantula"  n ( t @1 r a1 n t y u l @ ) ((pos "K6$" ) ))
( "tarantulas"  n ( t @1 r a1 n t y u l @ z ) ((pos "Kj$" ) ))
( "tarboosh"  n ( t aa1 b uu1 sh ) ((pos "K7$" ) ))
( "tarbooshes"  n ( t aa1 b uu1 sh i z ) ((pos "Kj$" ) ))
( "tardier"  j ( t aa1 d i@ r ) ((pos "Or$" ) ))
( "tardiest"  j ( t aa1 d i i s t ) ((pos "Os$" ) ))
( "tardily"  a ( t aa1 d i l ii ) ((pos "Pu%" ) ))
( "tardiness"  n ( t aa1 d i n @ s ) ((pos "L@%" ) ))
( "tardy"  j ( t aa1 d ii ) ((pos "OD%" ) ))
( "tare"  n ( t e@1 r ) ((pos "K6$" ) ))
( "tares"  n ( t e@1 z ) ((pos "Kj$" ) ))
( "target"  n ( t aa1 g i t ) ((pos "K6%" ) ))
( "targets"  n ( t aa1 g i t s ) ((pos "Kj%" ) ))
( "tariff"  n ( t a1 r i f ) ((pos "K6%" ) ))
( "tariffs"  n ( t a1 r i f s ) ((pos "Kj%" ) ))
( "tarmac"  v ( t aa1 m a k ) ((pos "H5%" "L@%" ) ))
( "tarmacked"  v ( t aa1 m a k t ) ((pos "Hc%" "Hd%" ) ))
( "tarmacking"  v ( t aa1 m a k i ng ) ((pos "Hb%" ) ))
( "tarmacs"  v ( t aa1 m a k s ) ((pos "Ha$" ) ))
( "tarn"  n ( t aa1 n ) ((pos "K6$" ) ))
( "tarnish"  v ( t aa1 n i sh ) ((pos "J1%" "L@%" ) (subcat "2A" )))
( "tarnished"  v ( t aa1 n i sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tarnishes"  v ( t aa1 n i sh i z ) ((pos "Ja%" ) (subcat "2A" )))
( "tarnishing"  v ( t aa1 n i sh i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tarns"  n ( t aa1 n z ) ((pos "Kj$" ) ))
( "taro"  n ( t aa1 r ou ) ((pos "M6$" ) ))
( "taros"  n ( t aa1 r ou z ) ((pos "Mj$" ) ))
( "tarpaulin"  n ( t aa1 p oo1 l i n ) ((pos "M6%" ) ))
( "tarpaulins"  n ( t aa1 p oo1 l i n z ) ((pos "Mj%" ) ))
( "tarpon"  n ( t aa1 p o n ) ((pos "K6$" ) ))
( "tarpons"  n ( t aa1 p o n z ) ((pos "Kj$" ) ))
( "tarradiddle"  n ( t a1 r @ d i d l ) ((pos "K6$" ) ))
( "tarradiddles"  n ( t a1 r @ d i d l z ) ((pos "Kj$" ) ))
( "tarragon"  n ( t a1 r @ g @ n ) ((pos "L@%" ) ))
( "tarred"  v ( t aa1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tarried"  v ( t a1 r i d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "tarries"  v ( t a1 r i z ) ((pos "Ia%" ) (subcat "2A" "2B" )))
( "tarring"  v ( t aa1 r i ng ) ((pos "Hb%" ) (subcat )))
( "tarry"  v ( t a1 r ii ) ((pos "I3%" ) (subcat "2A" "2B" )))
( "tarry"  j ( t aa1 r ii ) ((pos "OA%" ) ))
( "tarrying"  v ( t a1 r i i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "tars"  v ( t aa1 z ) ((pos "Ha$" "Mj$" ) (subcat )))
( "tarsal"  n ( t aa1 s l ) ((pos "K6$" "OA$" ) ))
( "tarsals"  n ( t aa1 s l z ) ((pos "Kj$" ) ))
( "tarsi"  n ( t aa1 s ai ) ((pos "Kj$" ) ))
( "tarsus"  n ( t aa1 s @ s ) ((pos "Ki$" ) ))
( "tart"  v ( t aa1 t ) ((pos "H0%" "K6%" "OA%" ) (subcat )))
( "tartan"  n ( t aa1 t n ) ((pos "M6%" ) ))
( "tartans"  n ( t aa1 t n z ) ((pos "Mj%" ) ))
( "tartar"  n ( t aa1 t @ r ) ((pos "M6$" ) ))
( "tartaric"  j ( t aa1 t a1 r i k ) ((pos "OA%" ) ))
( "tartars"  n ( t aa1 t @ z ) ((pos "Mj$" ) ))
( "tarted"  v ( t aa1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tarting"  v ( t aa1 t i ng ) ((pos "Hb%" ) (subcat )))
( "tartly"  a ( t aa1 t l ii ) ((pos "Pu%" ) ))
( "tartness"  n ( t aa1 t n @ s ) ((pos "L@%" ) ))
( "tarts"  v ( t aa1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "task"  v ( t aa1 s k ) ((pos "H0$" "K6%" ) (subcat )))
( "task-force"  n ( t aa1 s k - f oo s ) ((pos "K6%" ) ))
( "task-forces"  n ( t aa1 s k - f oo s i z ) ((pos "Kj%" ) ))
( "tasked"  v ( t aa1 s k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "tasking"  v ( t aa1 s k i ng ) ((pos "Hb$" ) (subcat )))
( "taskmaster"  n ( t aa1 s k m aa s t @ r ) ((pos "K6%" ) ))
( "taskmasters"  n ( t aa1 s k m aa s t @ z ) ((pos "Kj%" ) ))
( "tasks"  v ( t aa1 s k s ) ((pos "Ha$" "Kj%" ) (subcat )))
( "tassel"  n ( t a1 s l ) ((pos "K6%" ) ))
( "tasseled"  j ( t a1 s l d ) ((pos "OA$" ) ))
( "tasselled"  j ( t a1 s l d ) ((pos "OA%" ) ))
( "tassels"  n ( t a1 s l z ) ((pos "Kj%" ) ))
( "taste"  v ( t ei1 s t ) ((pos "J2%" "M6%" ) (subcat "2A" "2D" "3A" )))
( "tasted"  v ( t ei1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2D" "3A" )))
( "tasteful"  j ( t ei1 s t f @ l ) ((pos "OA%" ) ))
( "tastefully"  a ( t ei1 s t f @ l ii ) ((pos "Pu%" ) ))
( "tasteless"  j ( t ei1 s t l @ s ) ((pos "OA%" ) ))
( "tastelessly"  a ( t ei1 s t l @ s l ii ) ((pos "Pu%" ) ))
( "taster"  n ( t ei1 s t @ r ) ((pos "K6%" ) ))
( "tasters"  n ( t ei1 s t @ z ) ((pos "Kj%" ) ))
( "tastes"  v ( t ei1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2D" "3A" )))
( "tastier"  j ( t ei1 s t i@ r ) ((pos "Or%" ) ))
( "tastiest"  j ( t ei1 s t i i s t ) ((pos "Os%" ) ))
( "tastily"  a ( t ei1 s t i l ii ) ((pos "Pu%" ) ))
( "tasting"  v ( t ei1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2D" "3A" )))
( "tasty"  j ( t ei1 s t ii ) ((pos "OD%" ) ))
( "tat"  v ( t a1 t ) ((pos "J4$" "L@$" ) ))
( "tats"  v ( t a1 t s ) ((pos "Ja$" ) ))
( "tatted"  v ( t a1 t i d ) ((pos "Jc$" "Jd$" ) ))
( "tatter"  n ( t a1 t @ r ) ((pos "K6$" ) ))
( "tatterdemalion"  n ( t a2 t @ d @ m ei1 l i@ n ) ((pos "K6$" ) ))
( "tatterdemalions"  n ( t a2 t @ d @ m ei1 l i@ n z ) ((pos "Kj$" ) ))
( "tattered"  j ( t a1 t @ d ) ((pos "OA%" ) ))
( "tatters"  n ( t a1 t @ z ) ((pos "Kj%" ) ))
( "tattier"  j ( t a1 t i@ r ) ((pos "Or%" ) ))
( "tattiest"  j ( t a1 t i i s t ) ((pos "Os%" ) ))
( "tattily"  a ( t a1 t i l ii ) ((pos "Pu%" ) ))
( "tatting"  v ( t a1 t i ng ) ((pos "Jb$" "L@$" ) ))
( "tattle"  v ( t a1 t l ) ((pos "J2$" "L@$" ) (subcat )))
( "tattled"  v ( t a1 t l d ) ((pos "Jc$" "Jd$" ) (subcat )))
( "tattler"  n ( t a1 t l @ r ) ((pos "K6$" ) ))
( "tattlers"  n ( t a1 t l @ z ) ((pos "Kj$" ) ))
( "tattles"  v ( t a1 t l z ) ((pos "Ja$" ) (subcat )))
( "tattling"  v ( t a1 t l i ng ) ((pos "Jb$" ) (subcat )))
( "tattoo"  v ( t @1 t uu1 ) ((pos "H0%" "K6%" ) (subcat )))
( "tattooed"  v ( t @1 t uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tattooing"  v ( t @1 t uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "tattoos"  v ( t @1 t uu1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "tatty"  j ( t a1 t ii ) ((pos "OD%" ) ))
( "taught"  v ( t oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "11" "12A" "13A" "17" "20" )))
( "taunt"  v ( t oo1 n t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "taunted"  v ( t oo1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "taunting"  v ( t oo1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tauntingly"  a ( t oo1 n t i ng l ii ) ((pos "Pu%" ) ))
( "taunts"  v ( t oo1 n t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "taut"  j ( t oo1 t ) ((pos "OC%" ) ))
( "tauter"  j ( t oo1 t @ r ) ((pos "Or$" ) ))
( "tautest"  j ( t oo1 t i s t ) ((pos "Os$" ) ))
( "tautly"  a ( t oo1 t l ii ) ((pos "Pu%" ) ))
( "tautness"  n ( t oo1 t n @ s ) ((pos "L@%" ) ))
( "tautological"  j ( t oo2 t @ l o1 jh i k l ) ((pos "OA%" ) ))
( "tautologies"  n ( t oo1 t o1 l @ jh i z ) ((pos "Mj%" ) ))
( "tautology"  n ( t oo1 t o1 l @ jh ii ) ((pos "M8%" ) ))
( "tavern"  n ( t a1 v @ n ) ((pos "K6%" ) ))
( "taverns"  n ( t a1 v @ n z ) ((pos "Kj%" ) ))
( "tawdrier"  j ( t oo1 d r i@ r ) ((pos "Or$" ) ))
( "tawdriest"  j ( t oo1 d r i i s t ) ((pos "Os$" ) ))
( "tawdrily"  a ( t oo1 d r i l ii ) ((pos "Pu%" ) ))
( "tawdriness"  n ( t oo1 d r i n @ s ) ((pos "L@%" ) ))
( "tawdry"  j ( t oo1 d r ii ) ((pos "OD%" ) ))
( "tawny"  j ( t oo1 n ii ) ((pos "OA%" ) ))
( "tawse"  n ( t oo1 z ) ((pos "K6$" ) ))
( "tawses"  n ( t oo1 z i z ) ((pos "Kj$" ) ))
( "tax"  v ( t a1 k s ) ((pos "H1%" "M7%" ) (subcat "6A" )))
( "tax-collector"  n ( t a1 k s - k @ l e2 k t @ r ) ((pos "K6%" ) ))
( "tax-collectors"  n ( t a1 k s - k @ l e2 k t @ z ) ((pos "Kj%" ) ))
( "tax-free"  j ( t a1 k s - f r ii ) ((pos "OA%" ) ))
( "taxability"  n ( t a2 k s @ b i1 l i t ii ) ((pos "L@%" ) ))
( "taxable"  j ( t a1 k s @ b l ) ((pos "OA%" ) ))
( "taxation"  n ( t a1 k s ei1 sh @ n ) ((pos "L@%" ) ))
( "taxed"  v ( t a1 k s t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "taxes"  v ( t a1 k s i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "taxi"  v ( t a1 k s ii ) ((pos "J0%" "K6%" ) (subcat "2C" )))
( "taxicab"  n ( t a1 k s i k a b ) ((pos "K6%" ) ))
( "taxicabs"  n ( t a1 k s i k a b z ) ((pos "Kj%" ) ))
( "taxidermist"  n ( t a1 k s i1 d @ m i s t ) ((pos "K6%" ) ))
( "taxidermists"  n ( t a1 k s i1 d @ m i s t s ) ((pos "Kj%" ) ))
( "taxidermy"  n ( t a1 k s i d @@ m ii ) ((pos "L@%" ) ))
( "taxied"  v ( t a1 k s i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "taxiing"  v ( t a1 k s i i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "taximeter"  n ( t a1 k s i m ii t @ r ) ((pos "K6$" ) ))
( "taximeters"  n ( t a1 k s i m ii t @ z ) ((pos "Kj$" ) ))
( "taxing"  v ( t a1 k s i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "taxis"  v ( t a1 k s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "taxonomies"  n ( t a1 k s o1 n @ m i z ) ((pos "Mj$" ) ))
( "taxonomy"  n ( t a1 k s o1 n @ m ii ) ((pos "M8%" ) ))
( "taxpayer"  n ( t a1 k s p ei @ r ) ((pos "K6%" ) ))
( "taxpayers"  n ( t a1 k s p ei @ z ) ((pos "Kj%" ) ))
( "tea"  n ( t ii1 ) ((pos "M6%" ) ))
( "tea-bag"  n ( t ii1 - b a g ) ((pos "K6%" ) ))
( "tea-bags"  n ( t ii1 - b a g z ) ((pos "Kj%" ) ))
( "tea-break"  n ( t ii1 - b r ei k ) ((pos "K6%" ) ))
( "tea-breaks"  n ( t ii1 - b r ei k s ) ((pos "Kj%" ) ))
( "tea-caddies"  n ( t ii1 - k a d i z ) ((pos "Kj%" ) ))
( "tea-caddy"  n ( t ii1 - k a d ii ) ((pos "K8%" ) ))
( "tea-chest"  n ( t ii1 - ch e s t ) ((pos "K6%" ) ))
( "tea-chests"  n ( t ii1 - ch e s t s ) ((pos "Kj%" ) ))
( "tea-cloth"  n ( t ii1 - k l o th ) ((pos "K6%" ) ))
( "tea-cloths"  n ( t ii1 - k l o th s ) ((pos "Kj%" ) ))
( "tea-cosies"  n ( t ii1 - k ou z i z ) ((pos "Kj%" ) ))
( "tea-cosy"  n ( t ii1 - k ou z ii ) ((pos "K8%" ) ))
( "tea-garden"  n ( t ii1 - g aa d n ) ((pos "K6%" ) ))
( "tea-gardens"  n ( t ii1 - g aa d n z ) ((pos "Kj%" ) ))
( "tea-kettle"  n ( t ii1 - k e t l ) ((pos "K6%" ) ))
( "tea-kettles"  n ( t ii1 - k e t l z ) ((pos "Kj%" ) ))
( "tea-leaf"  n ( t ii1 - l ii f ) ((pos "Ki%" ) ))
( "tea-leaves"  n ( t ii1 - l ii v z ) ((pos "Kj%" ) ))
( "tea-parties"  n ( t ii1 - p aa t i z ) ((pos "Kj%" ) ))
( "tea-party"  n ( t ii1 - p aa t ii ) ((pos "K8%" ) ))
( "tea-service"  n ( t ii1 - s @@ v i s ) ((pos "K6%" ) ))
( "tea-services"  n ( t ii1 - s @@ v i s i z ) ((pos "Kj%" ) ))
( "tea-set"  n ( t ii1 - s e t ) ((pos "K6%" ) ))
( "tea-sets"  n ( t ii1 - s e t s ) ((pos "Kj%" ) ))
( "tea-strainer"  n ( t ii1 - s t r ei n @ r ) ((pos "K6%" ) ))
( "tea-strainers"  n ( t ii1 - s t r ei n @ z ) ((pos "Kj%" ) ))
( "tea-table"  n ( t ii1 - t ei b l ) ((pos "K6%" ) ))
( "tea-tables"  n ( t ii1 - t ei b l z ) ((pos "Kj%" ) ))
( "tea-things"  n ( t ii1 - th i ng z ) ((pos "Kj%" ) ))
( "tea-time"  n ( t ii1 - t ai m ) ((pos "L@%" ) ))
( "tea-towel"  n ( t ii1 - t au @ l ) ((pos "K6%" ) ))
( "tea-towels"  n ( t ii1 - t au @ l z ) ((pos "Kj%" ) ))
( "tea-tray"  n ( t ii1 - t r ei ) ((pos "K6%" ) ))
( "tea-trays"  n ( t ii1 - t r ei z ) ((pos "Kj%" ) ))
( "tea-trolley"  n ( t ii1 - t r o l ii ) ((pos "K6%" ) ))
( "tea-trolleys"  n ( t ii1 - t r o l i z ) ((pos "Kj%" ) ))
( "tea-urn"  n ( t ii1 - @@ n ) ((pos "K6%" ) ))
( "tea-urns"  n ( t ii1 - @@ n z ) ((pos "Kj%" ) ))
( "tea-wagon"  n ( t ii1 - w a g @ n ) ((pos "K6%" ) ))
( "tea-wagons"  n ( t ii1 - w a g @ n z ) ((pos "Kj%" ) ))
( "teacake"  n ( t ii1 k ei k ) ((pos "K6%" ) ))
( "teacakes"  n ( t ii1 k ei k s ) ((pos "Kj%" ) ))
( "teach"  v ( t ii1 ch ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "6A" "11" "12A" "13A" "17" "20" )))
( "teach-in"  n ( t ii1 ch - i n ) ((pos "K6$" ) ))
( "teach-ins"  n ( t ii1 ch - i n z ) ((pos "Kj$" ) ))
( "teachable"  j ( t ii1 ch @ b l ) ((pos "OA%" ) ))
( "teacher"  n ( t ii1 ch @ r ) ((pos "K6%" ) ))
( "teachers"  n ( t ii1 ch @ z ) ((pos "Kj%" ) ))
( "teaches"  v ( t ii1 ch i z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "6A" "11" "12A" "13A" "17" "20" )))
( "teaching"  v ( t ii1 ch i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "6A" "11" "12A" "13A" "17" "20" )))
( "teachings"  n ( t ii1 ch i ng z ) ((pos "Mj%" ) ))
( "teacup"  n ( t ii1 k uh p ) ((pos "K6%" ) ))
( "teacups"  n ( t ii1 k uh p s ) ((pos "Kj%" ) ))
( "teahouse"  n ( t ii1 h au s ) ((pos "K6%" ) ))
( "teahouses"  n ( t ii1 h au z i z ) ((pos "Kj%" ) ))
( "teak"  n ( t ii1 k ) ((pos "L@%" ) ))
( "teal"  n ( t ii1 l ) ((pos "K9$" ) ))
( "team"  v ( t ii1 m ) ((pos "I0%" "K6%" ) (subcat )))
( "teamed"  v ( t ii1 m d ) ((pos "Ic%" "Id%" ) (subcat )))
( "teaming"  v ( t ii1 m i ng ) ((pos "Ib%" ) (subcat )))
( "teams"  v ( t ii1 m z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "teamster"  n ( t ii1 m s t @ r ) ((pos "K6$" ) ))
( "teamsters"  n ( t ii1 m s t @ z ) ((pos "Kj$" ) ))
( "teamwork"  n ( t ii1 m w @@ k ) ((pos "L@%" ) ))
( "teapot"  n ( t ii1 p o t ) ((pos "K6%" ) ))
( "teapots"  n ( t ii1 p o t s ) ((pos "Kj%" ) ))
( "tear"  n ( t i@1 r ) ((pos "K6%" ) ))
( "tear"  v ( t e@1 r ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "tear-drop"  n ( t ii1 @ - d r o p ) ((pos "K6%" ) ))
( "tear-drops"  n ( t ii1 @ - d r o p s ) ((pos "Kj%" ) ))
( "tear-gas"  n ( t ii1 @ - g a s ) ((pos "L@%" ) ))
( "tearaway"  n ( t e@1 r @ w ei ) ((pos "K6%" "OA%" ) ))
( "tearaways"  n ( t e@1 r @ w ei z ) ((pos "Kj%" ) ))
( "tearful"  j ( t i@1 f @ l ) ((pos "OA%" ) ))
( "tearfully"  a ( t i@1 f @ l ii ) ((pos "Pu%" ) ))
( "tearing"  v ( t e@1 r i ng ) ((pos "Jb%" "L@%" "Oq%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "tearless"  j ( t i@1 l @ s ) ((pos "OA$" ) ))
( "tearoom"  n ( t ii1 r uu m ) ((pos "K6%" ) ))
( "tearooms"  n ( t ii1 r uu m z ) ((pos "Kj%" ) ))
( "tears"  n ( t i@1 z ) ((pos "Kj%" ) ))
( "tears"  v ( t e@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "teas"  n ( t ii1 z ) ((pos "Mj%" ) ))
( "tease"  v ( t ii1 z ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "teased"  v ( t ii1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "teasel"  n ( t ii1 z l ) ((pos "K6%" ) ))
( "teasels"  n ( t ii1 z l z ) ((pos "Kj%" ) ))
( "teaser"  n ( t ii1 z @ r ) ((pos "K6%" ) ))
( "teasers"  n ( t ii1 z @ z ) ((pos "Kj%" ) ))
( "teases"  v ( t ii1 z i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "teashop"  n ( t ii1 sh o p ) ((pos "K6%" ) ))
( "teashops"  n ( t ii1 sh o p s ) ((pos "Kj%" ) ))
( "teasing"  v ( t ii1 z i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "teasingly"  a ( t ii1 z i ng l ii ) ((pos "Pu%" ) ))
( "teaspoon"  n ( t ii1 s p uu n ) ((pos "K6%" ) ))
( "teaspoonful"  n ( t ii1 s p uu n f u l ) ((pos "K6%" ) ))
( "teaspoonfuls"  n ( t ii1 s p uu n f u l z ) ((pos "Kj%" ) ))
( "teaspoons"  n ( t ii1 s p uu n z ) ((pos "Kj%" ) ))
( "teat"  n ( t ii1 t ) ((pos "K6%" ) ))
( "teats"  n ( t ii1 t s ) ((pos "Kj%" ) ))
( "teazel"  n ( t ii1 z l ) ((pos "K6$" ) ))
( "teazels"  n ( t ii1 z l z ) ((pos "Kj$" ) ))
( "teazle"  n ( t ii1 z l ) ((pos "K6$" ) ))
( "teazles"  n ( t ii1 z l z ) ((pos "Kj$" ) ))
( "tec"  n ( t e1 k ) ((pos "K6$" ) ))
( "tech"  n ( t e1 k ) ((pos "K6$" ) ))
( "technical"  j ( t e1 k n i k l ) ((pos "OA%" ) ))
( "technicalities"  n ( t e2 k n i k a1 l i t i z ) ((pos "Mj%" ) ))
( "technicality"  n ( t e2 k n i k a1 l i t ii ) ((pos "M8%" ) ))
( "technically"  a ( t e1 k n i k l ii ) ((pos "Pu%" ) ))
( "technician"  n ( t e1 k n i1 sh @ n ) ((pos "K6%" ) ))
( "technicians"  n ( t e1 k n i1 sh @ n z ) ((pos "Kj%" ) ))
( "technique"  n ( t e1 k n ii1 k ) ((pos "M6%" ) ))
( "techniques"  n ( t e1 k n ii1 k s ) ((pos "Mj%" ) ))
( "technocracies"  n ( t e1 k n o1 k r @ s i z ) ((pos "Mj$" ) ))
( "technocracy"  n ( t e1 k n o1 k r @ s ii ) ((pos "M8$" ) ))
( "technocrat"  n ( t e1 k n @ k r a t ) ((pos "K6%" ) ))
( "technocrats"  n ( t e1 k n @ k r a t s ) ((pos "Kj%" ) ))
( "technological"  j ( t e2 k n @ l o1 jh i k l ) ((pos "OA%" ) ))
( "technologies"  n ( t e1 k n o1 l @ jh i z ) ((pos "Mj%" ) ))
( "technologist"  n ( t e1 k n o1 l @ jh i s t ) ((pos "K6%" ) ))
( "technologists"  n ( t e1 k n o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "technology"  n ( t e1 k n o1 l @ jh ii ) ((pos "M8%" ) ))
( "techs"  n ( t e1 k s ) ((pos "Kj$" ) ))
( "techy"  j ( t e1 ch ii ) ((pos "OA%" ) ))
( "tecs"  n ( t e1 k s ) ((pos "Kj$" ) ))
( "teddies"  n ( t e1 d i z ) ((pos "Kj%" ) ))
( "teddy"  n ( t e1 d ii ) ((pos "K8%" ) ))
( "tedious"  j ( t ii1 d i@ s ) ((pos "OA%" ) ))
( "tediously"  a ( t ii1 d i@ s l ii ) ((pos "Pu%" ) ))
( "tediousness"  n ( t ii1 d i@ s n @ s ) ((pos "L@%" ) ))
( "tedium"  n ( t ii1 d i@ m ) ((pos "L@%" ) ))
( "tee"  v ( t ii1 ) ((pos "J5$" "K6$" ) (subcat "2A" "2C" )))
( "tee-shirt"  n ( t ii1 - sh @@ t ) ((pos "K6%" ) ))
( "tee-shirts"  n ( t ii1 - sh @@ t s ) ((pos "Kj%" ) ))
( "teed"  v ( t ii1 d ) ((pos "Jc$" "Jd$" ) (subcat "2A" "2C" )))
( "teeing"  v ( t ii1 i ng ) ((pos "Jb$" ) (subcat "2A" "2C" )))
( "teem"  v ( t ii1 m ) ((pos "I0%" ) (subcat "2A" "2C" )))
( "teemed"  v ( t ii1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "teeming"  v ( t ii1 m i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "teems"  v ( t ii1 m z ) ((pos "Ia%" ) (subcat "2A" "2C" )))
( "teenage"  j ( t ii1 n ei jh ) ((pos "OA%" ) ))
( "teenager"  n ( t ii1 n ei jh @ r ) ((pos "K6%" ) ))
( "teenagers"  n ( t ii1 n ei jh @ z ) ((pos "Kj%" ) ))
( "teenier"  j ( t ii1 n i@ r ) ((pos "Or%" ) ))
( "teeniest"  j ( t ii1 n i i s t ) ((pos "Os%" ) ))
( "teens"  n ( t ii1 n z ) ((pos "Kj%" ) ))
( "teeny"  j ( t ii1 n ii ) ((pos "OD%" ) ))
( "tees"  v ( t ii1 z ) ((pos "Ja$" "Kj$" ) (subcat "2A" "2C" )))
( "teeter"  v ( t ii1 t @ r ) ((pos "I0%" ) (subcat )))
( "teetered"  v ( t ii1 t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "teetering"  v ( t ii1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "teeters"  v ( t ii1 t @ z ) ((pos "Ia%" ) (subcat )))
( "teeth"  n ( t ii1 th ) ((pos "Kj%" ) ))
( "teethe"  v ( t ii1 dh ) ((pos "I2$" ) (subcat )))
( "teethed"  v ( t ii1 dh d ) ((pos "Ic$" "Id$" ) (subcat )))
( "teethes"  v ( t ii1 dh z ) ((pos "Ia$" ) (subcat )))
( "teething"  v ( t ii1 dh i ng ) ((pos "Ib%" ) (subcat )))
( "teetotal"  j ( t ii1 t ou1 t l ) ((pos "OA%" ) ))
( "teetotaller"  n ( t ii1 t ou1 t l @ r ) ((pos "K6%" ) ))
( "teetotallers"  n ( t ii1 t ou1 t l @ z ) ((pos "Kj%" ) ))
( "teetotum"  n ( t ii1 t ou1 t @ m ) ((pos "K6$" ) ))
( "teetotums"  n ( t ii1 t ou1 t @ m z ) ((pos "Kj$" ) ))
( "teg"  n ( t e1 g ) ((pos "K6$" ) ))
( "tegs"  n ( t e1 g z ) ((pos "Kj$" ) ))
( "tegument"  n ( t e1 g y u m @ n t ) ((pos "K6$" ) ))
( "teguments"  n ( t e1 g y u m @ n t s ) ((pos "Kj$" ) ))
( "tel"  n ( t e1 l ) ((pos "Y~%" ) ))
( "telecast"  v ( t e1 l i k aa s t ) ((pos "H5$" "Hc$" "Hd$" "K6$" ) ))
( "telecasting"  v ( t e1 l i k aa s t i ng ) ((pos "Hb$" ) ))
( "telecasts"  v ( t e1 l i k aa s t s ) ((pos "Ha$" "Kj$" ) ))
( "telecommunication"  n ( t e2 l i k @ m y uu2 n i k ei1 sh @ n ) ((pos "M6$" ) ))
( "telecommunications"  n ( t e2 l i k @ m y uu2 n i k ei1 sh @ n z ) ((pos "Kj%" ) ))
( "telegram"  n ( t e1 l i g r a m ) ((pos "K6%" ) ))
( "telegrams"  n ( t e1 l i g r a m z ) ((pos "Kj%" ) ))
( "telegraph"  v ( t e1 l i g r aa f ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "11" "12A" )))
( "telegraph-line"  n ( t e1 l i g r aa f - l ai n ) ((pos "K6%" ) ))
( "telegraph-lines"  n ( t e1 l i g r aa f - l ai n z ) ((pos "Kj%" ) ))
( "telegraph-pole"  n ( t e1 l i g r aa f - p ou l ) ((pos "K6%" ) ))
( "telegraph-poles"  n ( t e1 l i g r aa f - p ou l z ) ((pos "Kj%" ) ))
( "telegraph-post"  n ( t e1 l i g r aa f - p ou s t ) ((pos "K6%" ) ))
( "telegraph-posts"  n ( t e1 l i g r aa f - p ou s t s ) ((pos "Kj%" ) ))
( "telegraph-wire"  n ( t e1 l i g r aa f - w ai @ r ) ((pos "K6%" ) ))
( "telegraph-wires"  n ( t e1 l i g r aa f - w ai @ z ) ((pos "Kj%" ) ))
( "telegraphed"  v ( t e1 l i g r aa f t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "11" "12A" )))
( "telegrapher"  n ( t i1 l e1 g r @ f @ r ) ((pos "K6%" ) ))
( "telegraphers"  n ( t i1 l e1 g r @ f @ z ) ((pos "Kj%" ) ))
( "telegraphese"  n ( t e2 l i g r @ f ii1 z ) ((pos "L@$" ) ))
( "telegraphic"  j ( t e2 l i g r a1 f i k ) ((pos "OA%" ) ))
( "telegraphically"  a ( t e2 l i g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "telegraphing"  v ( t e1 l i g r aa f i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "11" "12A" )))
( "telegraphist"  n ( t i1 l e1 g r @ f i s t ) ((pos "K6%" ) ))
( "telegraphists"  n ( t i1 l e1 g r @ f i s t s ) ((pos "Kj%" ) ))
( "telegraphs"  v ( t e1 l i g r aa f s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "11" "12A" )))
( "telegraphy"  n ( t i1 l e1 g r @ f ii ) ((pos "L@%" ) ))
( "telemetry"  n ( t i1 l e1 m @ t r ii ) ((pos "L@$" ) ))
( "teleological"  j ( t e2 l i@ l o1 jh i k l ) ((pos "OA$" ) ))
( "teleologies"  n ( t e2 l i o1 l @ jh i z ) ((pos "Mj$" ) ))
( "teleologist"  n ( t e1 l i o1 l @ jh i s t ) ((pos "K6$" ) ))
( "teleologists"  n ( t e1 l i o1 l @ jh i s t s ) ((pos "Kj$" ) ))
( "teleology"  n ( t e2 l i o1 l @ jh ii ) ((pos "M8$" ) ))
( "telepathic"  j ( t e2 l i p a1 th i k ) ((pos "OA%" ) ))
( "telepathist"  n ( t i1 l e1 p @ th i s t ) ((pos "K6$" ) ))
( "telepathists"  n ( t i1 l e1 p @ th i s t s ) ((pos "Kj$" ) ))
( "telepathy"  n ( t i1 l e1 p @ th ii ) ((pos "L@%" ) ))
( "telephone"  v ( t e1 l i f ou n ) ((pos "J2%" "M6%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "telephoned"  v ( t e1 l i f ou n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "telephones"  v ( t e1 l i f ou n z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "telephonic"  j ( t e2 l i f o1 n i k ) ((pos "OA%" ) ))
( "telephoning"  v ( t e1 l i f ou n i ng ) ((pos "Jb%" ) (subcat "2A" "4A" "6A" "11" "12A" )))
( "telephonist"  n ( t i1 l e1 f @ n i s t ) ((pos "K6%" ) ))
( "telephonists"  n ( t i1 l e1 f @ n i s t s ) ((pos "Kj%" ) ))
( "telephony"  n ( t i1 l e1 f @ n ii ) ((pos "L@%" ) ))
( "telephoto"  n ( t e2 l i f ou1 t ou ) ((pos "L@%" ) ))
( "telephotograph"  n ( t e2 l i f ou1 t @ g r aa f ) ((pos "K6$" ) ))
( "telephotographs"  n ( t e2 l i f ou1 t @ g r aa f s ) ((pos "Kj$" ) ))
( "telephotography"  n ( t e2 l i f @ t o1 g r @ f ii ) ((pos "L@$" ) ))
( "teleprinter"  n ( t e1 l i p r i n t @ r ) ((pos "K6%" ) ))
( "teleprinters"  n ( t e1 l i p r i n t @ z ) ((pos "Kj%" ) ))
( "teleprompter"  n ( t e1 l i p r o m p t @ r ) ((pos "K6%" ) ))
( "teleprompters"  n ( t e1 l i p r o m p t @ z ) ((pos "Kj%" ) ))
( "telescope"  v ( t e1 l i s k ou p ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "telescoped"  v ( t e1 l i s k ou p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "telescopes"  v ( t e1 l i s k ou p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "telescopic"  j ( t e2 l i s k o1 p i k ) ((pos "OA%" ) ))
( "telescopically"  a ( t e2 l i s k o1 p i k l ii ) ((pos "Pu%" ) ))
( "telescoping"  v ( t e1 l i s k ou p i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "teletypewriter"  n ( t e2 l i t ai1 p r ai t @ r ) ((pos "K6$" ) ))
( "teletypewriters"  n ( t e2 l i t ai1 p r ai t @ z ) ((pos "Kj$" ) ))
( "televise"  v ( t e1 l i v ai z ) ((pos "H2%" ) (subcat )))
( "televised"  v ( t e1 l i v ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "televises"  v ( t e1 l i v ai z i z ) ((pos "Ha%" ) (subcat )))
( "televising"  v ( t e1 l i v ai z i ng ) ((pos "Hb%" ) (subcat )))
( "television"  n ( t e1 l i v i zh n ) ((pos "M6%" ) ))
( "televisions"  n ( t e1 l i v i zh n z ) ((pos "Mj%" ) ))
( "telex"  n ( t e1 l e k s ) ((pos "K7%" ) ))
( "telexes"  n ( t e1 l e k s i z ) ((pos "Kj%" ) ))
( "telfer"  n ( t e1 l f @ r ) ((pos "K6$" ) ))
( "telfers"  n ( t e1 l f @ z ) ((pos "Kj$" ) ))
( "tell"  v ( t e1 l ) ((pos "J5*" ) (subcat "2A" "3A" "6A" "8" "10" "11" "12A" "13A" "14" "15A" "15B" "17" "20" )))
( "teller"  n ( t e1 l @ r ) ((pos "K6%" ) ))
( "tellers"  n ( t e1 l @ z ) ((pos "Kj%" ) ))
( "tellies"  n ( t e1 l i z ) ((pos "Kj%" ) ))
( "telling"  v ( t e1 l i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "8" "10" "11" "12A" "13A" "14" "15A" "15B" "17" "20" )))
( "tellingly"  a ( t e1 l i ng l ii ) ((pos "Pu%" ) ))
( "tells"  v ( t e1 l z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "8" "10" "11" "12A" "13A" "14" "15A" "15B" "17" "20" )))
( "telltale"  n ( t e1 l t ei l ) ((pos "K6%" ) ))
( "telltales"  n ( t e1 l t ei l z ) ((pos "Kj%" ) ))
( "telly"  n ( t e1 l ii ) ((pos "K8%" ) ))
( "telpher"  n ( t e1 l f @ r ) ((pos "K6$" ) ))
( "telphers"  n ( t e1 l f @ z ) ((pos "Kj$" ) ))
( "temerity"  n ( t i1 m e1 r i t ii ) ((pos "L@%" ) ))
( "temp"  n ( t e1 m p ) ((pos "K6%" ) ))
( "temper"  v ( t e1 m p @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "6A" )))
( "tempera"  n ( t e1 m p @ r @ ) ((pos "L@%" ) ))
( "temperament"  n ( t e1 m p r @ m @ n t ) ((pos "M6%" ) ))
( "temperamental"  j ( t e2 m p r @ m e1 n t l ) ((pos "OA%" ) ))
( "temperamentally"  a ( t e2 m p r @ m e1 n t @ l ii ) ((pos "Pu%" ) ))
( "temperaments"  n ( t e1 m p r @ m @ n t s ) ((pos "Mj%" ) ))
( "temperance"  n ( t e1 m p @ r @ n s ) ((pos "L@%" ) ))
( "temperate"  j ( t e1 m p @ r @ t ) ((pos "OA%" ) ))
( "temperately"  a ( t e1 m p @ r @ t l ii ) ((pos "Pu%" ) ))
( "temperateness"  n ( t e1 m p @ r @ t n @ s ) ((pos "L@$" ) ))
( "temperature"  n ( t e1 m p r @ ch @ r ) ((pos "M6%" ) ))
( "temperatures"  n ( t e1 m p r @ ch @ z ) ((pos "Mj%" ) ))
( "tempered"  v ( t e1 m p @ d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2A" "6A" )))
( "tempering"  v ( t e1 m p @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "tempers"  v ( t e1 m p @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "tempest"  n ( t e1 m p i s t ) ((pos "K6%" ) ))
( "tempest-swept"  j ( t e1 m p i s t - s w e p t ) ((pos "OA$" ) ))
( "tempest-tossed"  j ( t e1 m p i s t - o s t ) ((pos "OA$" ) ))
( "tempests"  n ( t e1 m p i s t s ) ((pos "Kj%" ) ))
( "tempestuous"  j ( t e1 m p e1 s ch u@ s ) ((pos "OA%" ) ))
( "tempi"  n ( t e1 m p ii ) ((pos "Kj$" ) ))
( "template"  n ( t e1 m p l i t ) ((pos "K6%" ) ))
( "templates"  n ( t e1 m p l i t s ) ((pos "Kj%" ) ))
( "temple"  n ( t e1 m p l ) ((pos "K6%" ) ))
( "temples"  n ( t e1 m p l z ) ((pos "Kj%" ) ))
( "templet"  n ( t e1 m p l i t ) ((pos "K6$" ) ))
( "templets"  n ( t e1 m p l i t s ) ((pos "Kj$" ) ))
( "tempo"  n ( t e1 m p ou ) ((pos "K6%" ) ))
( "temporal"  j ( t e1 m p @ r @ l ) ((pos "OA%" ) ))
( "temporality"  n ( t e2 m p @ r a1 l i t ii ) ((pos "L@$" ) ))
( "temporalty"  n ( t e1 m p @ r @ l t ii ) ((pos "L@$" ) ))
( "temporarily"  a ( t e1 m p r @ r @ l ii ) ((pos "Pu%" ) ))
( "temporariness"  n ( t e1 m p r @ r i n @ s ) ((pos "L@%" ) ))
( "temporary"  j ( t e1 m p r @ r ii ) ((pos "OA%" ) ))
( "temporize"  v ( t e1 m p @ r ai z ) ((pos "I2%" ) (subcat )))
( "temporized"  v ( t e1 m p @ r ai z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "temporizes"  v ( t e1 m p @ r ai z i z ) ((pos "Ia%" ) (subcat )))
( "temporizing"  v ( t e1 m p @ r ai z i ng ) ((pos "Ib%" ) (subcat )))
( "tempos"  n ( t e1 m p ou z ) ((pos "Kj$" ) ))
( "temps"  n ( t e1 m p s ) ((pos "Kj%" ) ))
( "tempt"  v ( t e1 m p t ) ((pos "H0%" ) (subcat "6A" "14" )))
( "temptation"  n ( t e1 m p t ei1 sh @ n ) ((pos "M6%" ) ))
( "temptations"  n ( t e1 m p t ei1 sh @ n z ) ((pos "Mj%" ) ))
( "tempted"  v ( t e1 m p t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "tempter"  n ( t e1 m p t @ r ) ((pos "K6%" ) ))
( "tempters"  n ( t e1 m p t @ z ) ((pos "Kj%" ) ))
( "tempting"  v ( t e1 m p t i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "temptingly"  a ( t e1 m t i ng l ii ) ((pos "Pu%" ) ))
( "temptress"  n ( t e1 m p t r i s ) ((pos "K7%" ) ))
( "temptresses"  n ( t e1 m p t r i s i z ) ((pos "Kj%" ) ))
( "tempts"  v ( t e1 m p t s ) ((pos "Ha%" ) (subcat "6A" "14" )))
( "ten"  n ( t e1 n ) ((pos "K6%" "OA%" ) ))
( "tenability"  n ( t e2 n @ b i1 l i t ii ) ((pos "L@%" ) ))
( "tenable"  j ( t e1 n @ b l ) ((pos "OA%" ) ))
( "tenacious"  j ( t i1 n ei1 sh @ s ) ((pos "OA%" ) ))
( "tenaciously"  a ( t i1 n ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "tenaciousness"  n ( t i1 n ei1 sh @ s n @ s ) ((pos "L@%" ) ))
( "tenacity"  n ( t i1 n a1 s i t ii ) ((pos "L@%" ) ))
( "tenancies"  n ( t e1 n @ n s i z ) ((pos "Mj%" ) ))
( "tenancy"  n ( t e1 n @ n s ii ) ((pos "M8%" ) ))
( "tenant"  v ( t e1 n @ n t ) ((pos "H0$" "K6%" ) ))
( "tenanted"  v ( t e1 n @ n t i d ) ((pos "Hc$" "Hd%" ) ))
( "tenanting"  v ( t e1 n @ n t i ng ) ((pos "Hb$" ) ))
( "tenantries"  n ( t e1 n @ n t r i z ) ((pos "Kj$" ) ))
( "tenantry"  n ( t e1 n @ n t r ii ) ((pos "K8$" ) ))
( "tenants"  v ( t e1 n @ n t s ) ((pos "Ha$" "Kj%" ) ))
( "tench"  n ( t e1 n ch ) ((pos "K9$" ) ))
( "tend"  v ( t e1 n d ) ((pos "J0%" ) (subcat "2C" "4A" )))
( "tended"  v ( t e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "4A" )))
( "tendencies"  n ( t e1 n d @ n s i z ) ((pos "Mj%" ) ))
( "tendency"  n ( t e1 n d @ n s ii ) ((pos "M8%" ) ))
( "tendentious"  j ( t e1 n d e1 n sh @ s ) ((pos "OA%" ) ))
( "tendentiously"  a ( t e1 n d e1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "tendentiousness"  n ( t e1 n d e1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "tender"  v ( t e1 n d @ r ) ((pos "J0%" "K6%" "OC%" ) (subcat "2A" "3A" "6A" "12A" )))
( "tendered"  v ( t e1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "12A" )))
( "tenderer"  j ( t e1 n d @ r @ r ) ((pos "Or%" ) ))
( "tenderest"  j ( t e1 n d @ r i s t ) ((pos "Os%" ) ))
( "tenderfoot"  n ( t e1 n d @ f u t ) ((pos "K6%" ) ))
( "tenderfoots"  n ( t e1 n d @ f u t s ) ((pos "Kj$" ) ))
( "tenderhearted"  j ( t e2 n d @ h aa1 t i d ) ((pos "OA%" ) ))
( "tendering"  v ( t e1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "12A" )))
( "tenderloin"  n ( t e1 n d @ l oi n ) ((pos "L@$" ) ))
( "tenderly"  a ( t e1 n d @ l ii ) ((pos "Pu%" ) ))
( "tenderness"  n ( t e1 n d @ n @ s ) ((pos "L@%" ) ))
( "tenders"  v ( t e1 n d @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "12A" )))
( "tending"  v ( t e1 n d i ng ) ((pos "Jb%" ) (subcat "2C" "4A" )))
( "tendon"  n ( t e1 n d @ n ) ((pos "K6%" ) ))
( "tendons"  n ( t e1 n d @ n z ) ((pos "Kj%" ) ))
( "tendril"  n ( t e1 n d r @ l ) ((pos "K6%" ) ))
( "tendrils"  n ( t e1 n d r @ l z ) ((pos "Kj%" ) ))
( "tends"  v ( t e1 n d z ) ((pos "Ja%" ) (subcat "2C" "4A" )))
( "tenement"  n ( t e1 n @ m @ n t ) ((pos "K6%" ) ))
( "tenement-house"  n ( t e1 n @ m @ n t - h au s ) ((pos "K6%" ) ))
( "tenement-houses"  n ( t e1 n @ m @ n t - h au z i z ) ((pos "Kj%" ) ))
( "tenements"  n ( t e1 n @ m @ n t s ) ((pos "Kj%" ) ))
( "tenet"  n ( t e1 n i t ) ((pos "K6%" ) ))
( "tenets"  n ( t e1 n i t s ) ((pos "Kj%" ) ))
( "tenfold"  a ( t e1 n f ou l d ) ((pos "Pu%" ) ))
( "tenner"  n ( t e1 n @ r ) ((pos "K6%" ) ))
( "tenners"  n ( t e1 n @ z ) ((pos "Kj%" ) ))
( "tennis"  n ( t e1 n i s ) ((pos "L@%" ) ))
( "tennis-court"  n ( t e1 n i s - k oo t ) ((pos "K6%" ) ))
( "tennis-courts"  n ( t e1 n i s - k oo t s ) ((pos "Kj%" ) ))
( "tennis-elbow"  n ( t e2 n i s - e1 l b ou ) ((pos "L@%" ) ))
( "tenon"  n ( t e1 n @ n ) ((pos "K6$" ) ))
( "tenons"  n ( t e1 n @ n z ) ((pos "Kj$" ) ))
( "tenor"  n ( t e1 n @ r ) ((pos "K6%" ) ))
( "tenors"  n ( t e1 n @ z ) ((pos "Kj%" ) ))
( "tenpence"  n ( t e1 n p @ n s ) ((pos "K6%" ) ))
( "tenpences"  n ( t e1 n p @ n s i z ) ((pos "Kj%" ) ))
( "tenpin"  n ( t e1 n p i n ) ((pos "K6%" ) ))
( "tenpins"  n ( t e1 n p i n z ) ((pos "Kj$" ) ))
( "tens"  n ( t e1 n z ) ((pos "Kj%" ) ))
( "tense"  v ( t e1 n s ) ((pos "J2%" "K6%" "OB%" ) (subcat "2A" )))
( "tensed"  v ( t e1 n s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tensely"  a ( t e1 n s l ii ) ((pos "Pu%" ) ))
( "tenseness"  n ( t e1 n s n @ s ) ((pos "L@%" ) ))
( "tenser"  j ( t e1 n s @ r ) ((pos "Or%" ) ))
( "tenses"  v ( t e1 n s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "tensest"  j ( t e1 n s i s t ) ((pos "Os%" ) ))
( "tensile"  j ( t e1 n s ai l ) ((pos "OA$" ) ))
( "tensing"  v ( t e1 n s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tension"  n ( t e1 n sh @ n ) ((pos "M6%" ) ))
( "tensions"  n ( t e1 n sh @ n z ) ((pos "Mj%" ) ))
( "tensity"  n ( t e1 n s i t ii ) ((pos "L@$" ) ))
( "tent"  n ( t e1 n t ) ((pos "K6%" ) ))
( "tent-peg"  n ( t e1 n t - p e g ) ((pos "K6%" ) ))
( "tent-pegs"  n ( t e1 n t - p e g z ) ((pos "Kj%" ) ))
( "tentacle"  n ( t e1 n t @ k l ) ((pos "K6%" ) ))
( "tentacles"  n ( t e1 n t @ k l z ) ((pos "Kj%" ) ))
( "tentative"  j ( t e1 n t @ t i v ) ((pos "OA%" ) ))
( "tentatively"  a ( t e1 n t @ t i v l ii ) ((pos "Pu%" ) ))
( "tenterhooks"  n ( t e1 n t @ h u k s ) ((pos "Kj%" ) ))
( "tenth"  n ( t e1 n th ) ((pos "K6%" "OA%" ) ))
( "tenthly"  a ( t e1 n th l ii ) ((pos "Pu%" ) ))
( "tenths"  n ( t e1 n th s ) ((pos "Kj%" ) ))
( "tents"  n ( t e1 n t s ) ((pos "Kj%" ) ))
( "tenuity"  n ( t i1 n y uu1 i t ii ) ((pos "L@$" ) ))
( "tenuous"  j ( t e1 n y u@ s ) ((pos "OA%" ) ))
( "tenuously"  a ( t e1 n y u@ s l ii ) ((pos "Pu%" ) ))
( "tenure"  n ( t e1 n y u@ r ) ((pos "M6%" ) ))
( "tenures"  n ( t e1 n y u@ z ) ((pos "Mj%" ) ))
( "tepee"  n ( t ii1 p ii ) ((pos "K6$" ) ))
( "tepees"  n ( t ii1 p ii z ) ((pos "Kj$" ) ))
( "tepid"  j ( t e1 p i d ) ((pos "OA%" ) ))
( "tepidity"  n ( t e1 p i1 d i t ii ) ((pos "L@$" ) ))
( "tepidly"  a ( t e1 p i d l ii ) ((pos "Pu%" ) ))
( "tepidness"  n ( t e1 p i d n @ s ) ((pos "L@%" ) ))
( "tercentenaries"  n ( t @@2 s e n t ii1 n @ r i z ) ((pos "Kj%" ) ))
( "tercentenary"  n ( t @@2 s e n t ii1 n @ r ii ) ((pos "K8%" ) ))
( "tercentennial"  n ( t @@2 s e n t e1 n i@ l ) ((pos "K6%" ) ))
( "tercentennials"  n ( t @@2 s e n t e1 n i@ l z ) ((pos "Kj%" ) ))
( "tergiversate"  v ( t @@1 jh i v @ s ei t ) ((pos "I2$" ) ))
( "tergiversated"  v ( t @@1 jh i v @ s ei t i d ) ((pos "Ic$" "Id$" ) ))
( "tergiversates"  v ( t @@1 jh i v @ s ei t s ) ((pos "Ia$" ) ))
( "tergiversating"  v ( t @@1 jh i v @ s ei t i ng ) ((pos "Ib$" ) ))
( "tergiversation"  n ( t @@2 jh i v @ s ei1 sh @ n ) ((pos "L@$" ) ))
( "term"  v ( t @@1 m ) ((pos "H0%" "K6%" ) (subcat )))
( "termagant"  n ( t @@1 m @ g @ n t ) ((pos "K6$" ) ))
( "termagants"  n ( t @@1 m @ g @ n t s ) ((pos "Kj$" ) ))
( "termed"  v ( t @@1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "terminable"  j ( t @@1 m i n @ b l ) ((pos "OA$" ) ))
( "terminal"  n ( t @@1 m i n l ) ((pos "K6%" "OA%" ) ))
( "terminally"  a ( t @@1 m i n @ l ii ) ((pos "Pu%" ) ))
( "terminals"  n ( t @@1 m i n l z ) ((pos "Kj%" ) ))
( "terminate"  v ( t @@1 m i n ei t ) ((pos "J2%" ) (subcat "2A" "6A" )))
( "terminated"  v ( t @@1 m i n ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "terminates"  v ( t @@1 m i n ei t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "terminating"  v ( t @@1 m i n ei t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "termination"  n ( t @@2 m i n ei1 sh @ n ) ((pos "M6%" ) ))
( "terminations"  n ( t @@2 m i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "terming"  v ( t @@1 m i ng ) ((pos "Hb%" ) (subcat )))
( "terminological"  j ( t @@2 m i n @ l o1 jh i k l ) ((pos "OA$" ) ))
( "terminologies"  n ( t @@2 m i n o1 l @ jh i z ) ((pos "Mj$" ) ))
( "terminology"  n ( t @@2 m i n o1 l @ jh ii ) ((pos "M8%" ) ))
( "terminus"  n ( t @@1 m i n @ s ) ((pos "K7%" ) ))
( "terminuses"  n ( t @@1 m i n @ s i z ) ((pos "Kj%" ) ))
( "termite"  n ( t @@1 m ai t ) ((pos "K6%" ) ))
( "termites"  n ( t @@1 m ai t s ) ((pos "Kj%" ) ))
( "terms"  v ( t @@1 m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "tern"  n ( t @@1 n ) ((pos "K6%" ) ))
( "terns"  n ( t @@1 n z ) ((pos "Kj%" ) ))
( "terra firma"  n ( t e2 r @ - f @@1 m @ ) ((pos "L@%" ) ))
( "terra incognita"  n ( t e2 r @ - ii n k o1 g n i t @ ) ((pos "L@$" ) ))
( "terra-cotta"  n ( t e2 r @ - k o1 t @ ) ((pos "L@%" ) ))
( "terrace"  v ( t e1 r @ s ) ((pos "H2%" "K6%" ) (subcat )))
( "terraced"  v ( t e1 r @ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "terraces"  v ( t e1 r @ s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "terracing"  v ( t e1 r @ s i ng ) ((pos "Hb%" ) (subcat )))
( "terrain"  n ( t e1 r ei1 n ) ((pos "L@%" ) ))
( "terrapin"  n ( t e1 r @ p i n ) ((pos "K6%" ) ))
( "terrapins"  n ( t e1 r @ p i n z ) ((pos "Kj%" ) ))
( "terrestrial"  j ( t i1 r e1 s t r i@ l ) ((pos "OA%" ) ))
( "terrible"  j ( t e1 r @ b l ) ((pos "OA%" ) ))
( "terribly"  a ( t e1 r @ b l ii ) ((pos "Pu%" ) ))
( "terrier"  n ( t e1 r i@ r ) ((pos "K6%" ) ))
( "terriers"  n ( t e1 r i@ z ) ((pos "Kj%" ) ))
( "terrific"  j ( t @1 r i1 f i k ) ((pos "OA%" ) ))
( "terrifically"  a ( t @1 r i1 f i k l ii ) ((pos "Pu%" ) ))
( "terrified"  v ( t e1 r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "terrifies"  v ( t e1 r i f ai z ) ((pos "Ha%" ) (subcat "6A" )))
( "terrify"  v ( t e1 r i f ai ) ((pos "H3%" ) (subcat "6A" )))
( "terrifying"  v ( t e1 r i f ai i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "territorial"  n ( t e2 r i t oo1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "territorials"  n ( t e2 r i t oo1 r i@ l z ) ((pos "Kj%" ) ))
( "territories"  n ( t e1 r i t r i z ) ((pos "Mj%" ) ))
( "territory"  n ( t e1 r i t r ii ) ((pos "M8%" ) ))
( "terror"  n ( t e1 r @ r ) ((pos "M6%" ) ))
( "terror-stricken"  j ( t e1 r @ - s t r i k @ n ) ((pos "OA%" ) ))
( "terror-struck"  j ( t e1 r @ - s t r uh k ) ((pos "OA%" ) ))
( "terrorism"  n ( t e1 r @ r i z @ m ) ((pos "L@%" ) ))
( "terrorist"  n ( t e1 r @ r i s t ) ((pos "K6%" ) ))
( "terrorists"  n ( t e1 r @ r i s t s ) ((pos "Kj%" ) ))
( "terrorize"  v ( t e1 r @ r ai z ) ((pos "H2%" ) (subcat )))
( "terrorized"  v ( t e1 r @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "terrorizes"  v ( t e1 r @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "terrorizing"  v ( t e1 r @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "terrors"  n ( t e1 r @ z ) ((pos "Mj%" ) ))
( "terse"  j ( t @@1 s ) ((pos "OB%" ) ))
( "tersely"  a ( t @@1 s l ii ) ((pos "Pu%" ) ))
( "terseness"  n ( t @@1 s n @ s ) ((pos "L@%" ) ))
( "terser"  j ( t @@1 s @ r ) ((pos "Or$" ) ))
( "tersest"  j ( t @@1 s i s t ) ((pos "Os$" ) ))
( "tertian"  j ( t @@1 sh @ n ) ((pos "OA$" ) ))
( "tertiary"  j ( t @@1 sh @ r ii ) ((pos "OA$" ) ))
( "terylene"  n ( t e1 r @ l ii n ) ((pos "L@%" ) ))
( "tessellated"  j ( t e1 s @ l ei t i d ) ((pos "OA%" ) ))
( "test"  v ( t e1 s t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "test-drive"  v ( t e1 s t - d r ai v ) ((pos "H5$" "K6%" ) ))
( "test-driven"  v ( t e1 s t - d r i v n ) ((pos "Hd$" ) ))
( "test-drives"  v ( t e1 s t - d r ai v z ) ((pos "Ha$" "Kj$" ) ))
( "test-driving"  v ( t e1 s t - d r ai v i ng ) ((pos "Hb$" ) ))
( "test-drove"  v ( t e1 s t - d r ou v ) ((pos "Hc$" ) ))
( "test-tube"  n ( t e1 s t - t y uu b ) ((pos "K6%" ) ))
( "test-tubes"  n ( t e1 s t - t y uu b z ) ((pos "Kj%" ) ))
( "testament"  n ( t e1 s t @ m @ n t ) ((pos "K6%" ) ))
( "testamentary"  j ( t e2 s t @ m e1 n t r ii ) ((pos "OA$" ) ))
( "testaments"  n ( t e1 s t @ m @ n t s ) ((pos "Kj%" ) ))
( "testate"  n ( t e1 s t ei t ) ((pos "K6$" "OA$" ) ))
( "testates"  n ( t e1 s t ei t s ) ((pos "Kj$" ) ))
( "testator"  n ( t e1 s t ei1 t @ r ) ((pos "K6$" ) ))
( "testators"  n ( t e1 s t ei1 t @ z ) ((pos "Kj$" ) ))
( "testatrix"  n ( t e1 s t ei1 t r i k s ) ((pos "K7$" ) ))
( "testatrixes"  n ( t e1 s t ei1 t r i k s i z ) ((pos "Kj$" ) ))
( "testbed"  n ( t e1 s t b e d ) ((pos "K6%" ) ))
( "testbeds"  n ( t e1 s t b e d z ) ((pos "Kj$" ) ))
( "tested"  v ( t e1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "testes"  n ( t e1 s t ii z ) ((pos "Kj%" ) ))
( "testicle"  n ( t e1 s t i k l ) ((pos "K6%" ) ))
( "testicles"  n ( t e1 s t i k l z ) ((pos "Kj%" ) ))
( "testier"  j ( t e1 s t i@ r ) ((pos "Or$" ) ))
( "testiest"  j ( t e1 s t i i s t ) ((pos "Os$" ) ))
( "testified"  v ( t e1 s t i f ai d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "testifies"  v ( t e1 s t i f ai z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "testify"  v ( t e1 s t i f ai ) ((pos "J3%" ) (subcat "2A" "3A" "6A" )))
( "testifying"  v ( t e1 s t i f ai i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "testily"  a ( t e1 s t i l ii ) ((pos "Pu%" ) ))
( "testimonial"  n ( t e2 s t i m ou1 n i@ l ) ((pos "K6%" ) ))
( "testimonials"  n ( t e2 s t i m ou1 n i@ l z ) ((pos "Kj%" ) ))
( "testimonies"  n ( t e1 s t i m @ n i z ) ((pos "Mj%" ) ))
( "testimony"  n ( t e1 s t i m @ n ii ) ((pos "M8%" ) ))
( "testiness"  n ( t e1 s t i n @ s ) ((pos "L@%" ) ))
( "testing"  v ( t e1 s t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "testis"  n ( t e1 s t i s ) ((pos "Ki$" ) ))
( "tests"  v ( t e1 s t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "testy"  j ( t e1 s t ii ) ((pos "OD%" ) ))
( "tetanus"  n ( t e1 t @ n @ s ) ((pos "L@%" ) ))
( "tetchier"  j ( t e1 ch i@ r ) ((pos "Or%" ) ))
( "tetchiest"  j ( t e1 ch i i s t ) ((pos "Os%" ) ))
( "tetchily"  a ( t e1 ch i l ii ) ((pos "Pu%" ) ))
( "tetchiness"  n ( t e1 ch i n @ s ) ((pos "L@%" ) ))
( "tetchy"  j ( t e1 ch ii ) ((pos "OD%" ) ))
( "tether"  v ( t e1 dh @ r ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "tethered"  v ( t e1 dh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tethering"  v ( t e1 dh @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tethers"  v ( t e1 dh @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "text"  n ( t e1 k s t ) ((pos "M6%" ) ))
( "textbook"  n ( t e1 k s b u k ) ((pos "K6%" ) ))
( "textbooks"  n ( t e1 k s b u k s ) ((pos "Kj%" ) ))
( "textile"  n ( t e1 k s t ai l ) ((pos "K6%" "Oq%" ) ))
( "textiles"  n ( t e1 k s t ai l z ) ((pos "Kj%" ) ))
( "texts"  n ( t e1 k s t s ) ((pos "Mj%" ) ))
( "textual"  j ( t e1 k s ch u@ l ) ((pos "OA%" ) ))
( "texture"  n ( t e1 k s ch @ r ) ((pos "M6%" ) ))
( "textured"  j ( t e1 k s ch @ d ) ((pos "OA%" ) ))
( "textures"  n ( t e1 k s ch @ z ) ((pos "Mj%" ) ))
( "thalidomide"  n ( th @1 l i1 d @ m ai d ) ((pos "L@%" ) ))
( "than"  cc ( dh a n ) ((pos "V-*" ) ))
( "thane"  n ( th ei1 n ) ((pos "K6$" ) ))
( "thanes"  n ( th ei1 n z ) ((pos "Kj$" ) ))
( "thank"  v ( th a1 ng k ) ((pos "H0%" ) (subcat "6A" "11" "14" )))
( "thank-offering"  n ( th a1 ng k - o1 f @ r i ng ) ((pos "K6%" ) ))
( "thank-offerings"  n ( th a1 ng k - o1 f @ r i ng z ) ((pos "Kj%" ) ))
( "thanked"  v ( th a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "14" )))
( "thankful"  j ( th a1 ng k f @ l ) ((pos "OA%" ) ))
( "thankfully"  a ( th a1 ng k f @ l ii ) ((pos "Pu%" ) ))
( "thankfulness"  n ( th a1 ng k f @ l n @ s ) ((pos "L@%" ) ))
( "thanking"  v ( th a1 ng k i ng ) ((pos "Hb%" ) (subcat "6A" "11" "14" )))
( "thankless"  j ( th a1 ng k l @ s ) ((pos "OA%" ) ))
( "thanks"  v ( th a1 ng k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "11" "14" )))
( "thanksgiving"  n ( th a1 n k s g i1 v i ng ) ((pos "M6%" ) ))
( "thanksgivings"  n ( th a1 n k s g i1 v i ng z ) ((pos "Mj%" ) ))
( "that"  j ( dh a t ) ((pos "OA*" "Pu%" "Qx%" "Qz*" "V-*" ) ))
( "that'd"  v ( dh a1 t @ d ) ((pos "Gf%" ) ))
( "that'll"  v ( dh a1 t l ) ((pos "Gf%" ) ))
( "that's"  v ( dh a t s ) ((pos "Gf%" ) ))
( "thatch"  v ( th a1 ch ) ((pos "H1%" "L@%" ) (subcat )))
( "thatched"  v ( th a1 ch t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "thatcher"  n ( th a1 ch @ r ) ((pos "K6%" ) ))
( "thatchers"  n ( th a1 ch @ z ) ((pos "Kj%" ) ))
( "thatches"  v ( th a1 ch i z ) ((pos "Ha%" ) (subcat )))
( "thatching"  v ( th a1 ch i ng ) ((pos "Hb%" ) (subcat )))
( "thaw"  v ( th oo1 ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "thawed"  v ( th oo1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "thawing"  v ( th oo1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "thaws"  v ( th oo1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "the"  a ( dh @ ) ((pos "Pu$" "R-*" ) ))
( "theatre"  n ( th i@1 t @ r ) ((pos "M6%" ) ))
( "theatregoer"  n ( th i@1 t @ g ou @ r ) ((pos "K6%" ) ))
( "theatregoers"  n ( th i@1 t @ g ou @ z ) ((pos "Kj%" ) ))
( "theatres"  n ( th i@1 t @ z ) ((pos "Mj%" ) ))
( "theatrical"  n ( th i1 a1 t r i k l ) ((pos "K6%" "OA%" ) ))
( "theatrically"  a ( th i1 a1 t r i k l ii ) ((pos "Pu%" ) ))
( "theatricals"  n ( th i1 a1 t r i k l z ) ((pos "Kj%" ) ))
( "thee"  prp ( dh ii ) ((pos "Qx$" ) ))
( "theft"  n ( th e1 f t ) ((pos "M6%" ) ))
( "thefts"  n ( th e1 f t s ) ((pos "Mj%" ) ))
( "their"  j ( dh e@1 r ) ((pos "OA*" ) ))
( "theirs"  prp ( dh e@ z ) ((pos "Qx%" ) ))
( "theism"  n ( th ii1 i z @ m ) ((pos "L@$" ) ))
( "theist"  n ( th ii1 i s t ) ((pos "K6$" ) ))
( "theistic"  j ( th ii1 i1 s t i k ) ((pos "OA$" ) ))
( "theistical"  j ( th ii1 i1 s t i k @ l ) ((pos "OA$" ) ))
( "theists"  n ( th ii1 i s t s ) ((pos "Kj$" ) ))
( "them"  prp ( dh e m ) ((pos "Qx*" ) ))
( "thematic"  j ( th i1 m a1 t i k ) ((pos "OA%" ) ))
( "theme"  n ( th ii1 m ) ((pos "K6%" ) ))
( "themes"  n ( th ii1 m z ) ((pos "Kj%" ) ))
( "themselves"  prp ( dh @1 m s e1 l v z ) ((pos "Qx%" ) ))
( "then"  a ( dh e1 n ) ((pos "Pu*" ) ))
( "thence"  a ( dh e1 n s ) ((pos "Pu%" ) ))
( "thenceforth"  a ( dh e2 n s f oo1 th ) ((pos "Pu%" ) ))
( "thenceforward"  a ( dh e2 n s f oo1 w @ d ) ((pos "Pu%" ) ))
( "theocracies"  n ( th i1 o1 k r @ s i z ) ((pos "Mj$" ) ))
( "theocracy"  n ( th i1 o1 k r @ s ii ) ((pos "M8$" ) ))
( "theocratic"  j ( th i@2 k r a1 t i k ) ((pos "OA$" ) ))
( "theodolite"  n ( th i1 o1 d @ l ai t ) ((pos "K6$" ) ))
( "theodolites"  n ( th i1 o1 d @ l ai t s ) ((pos "Kj$" ) ))
( "theologian"  n ( th i@2 l ou1 jh @ n ) ((pos "K6%" ) ))
( "theologians"  n ( th i@2 l ou1 jh @ n z ) ((pos "Kj%" ) ))
( "theological"  j ( th i@2 l o1 jh i k l ) ((pos "OA%" ) ))
( "theologically"  a ( th i@2 l o1 jh i k l ii ) ((pos "Pu%" ) ))
( "theologies"  n ( th i1 o1 l @ jh i z ) ((pos "Mj%" ) ))
( "theology"  n ( th i1 o1 l @ jh ii ) ((pos "M8%" ) ))
( "theorem"  n ( th i@1 r @ m ) ((pos "K6%" ) ))
( "theorems"  n ( th i@1 r @ m z ) ((pos "Kj%" ) ))
( "theoretic"  j ( th i@1 r e1 t i k ) ((pos "OA%" ) ))
( "theoretical"  j ( th i@1 r e1 t i k l ) ((pos "OA%" ) ))
( "theoretically"  a ( th i@1 r e1 t i k l ii ) ((pos "Pu%" ) ))
( "theoretician"  n ( th i@2 r @ t i1 sh @ n ) ((pos "K6%" ) ))
( "theoreticians"  n ( th i@2 r @ t i1 sh @ n z ) ((pos "Kj%" ) ))
( "theories"  n ( th i@1 r i z ) ((pos "Mj%" ) ))
( "theorist"  n ( th i@1 r i s t ) ((pos "K6%" ) ))
( "theorists"  n ( th i@1 r i s t s ) ((pos "Kj%" ) ))
( "theorize"  v ( th i@1 r ai z ) ((pos "I2%" ) (subcat "2A" )))
( "theorized"  v ( th i@1 r ai z d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "theorizes"  v ( th i@1 r ai z i z ) ((pos "Ia%" ) (subcat "2A" )))
( "theorizing"  v ( th i@1 r ai z i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "theory"  n ( th i@1 r ii ) ((pos "M8%" ) ))
( "theosophical"  j ( th ii2 @ s o1 f i k l ) ((pos "OA$" ) ))
( "theosophist"  n ( th i1 o1 s @ f i s t ) ((pos "K6$" ) ))
( "theosophists"  n ( th i1 o1 s @ f i s t s ) ((pos "Kj$" ) ))
( "theosophy"  n ( th ii1 o1 s @ f ii ) ((pos "L@$" ) ))
( "therapeutic"  j ( th e2 r @ p y uu1 t i k ) ((pos "OA%" ) ))
( "therapeutical"  j ( th e2 r @ p y uu1 t i k l ) ((pos "OA$" ) ))
( "therapeutics"  n ( th e2 r @ p y uu1 t i k s ) ((pos "Lk$" ) ))
( "therapies"  n ( th e1 r @ p i z ) ((pos "Mj%" ) ))
( "therapist"  n ( th e1 r @ p i s t ) ((pos "K6%" ) ))
( "therapists"  n ( th e1 r @ p i s t s ) ((pos "Kj%" ) ))
( "therapy"  n ( th e1 r @ p ii ) ((pos "M8%" ) ))
( "there"  a ( dh e@1 r ) ((pos "Pu*" "W-%" ) ))
( "there's"  v ( dh e@ z ) ((pos "Gh%" ) ))
( "thereabout"  a ( dh e@1 r @ b au t ) ((pos "Pu$" ) ))
( "thereabouts"  a ( dh e@1 r @ b au t s ) ((pos "Pu%" ) ))
( "thereafter"  a ( dh e@1 r aa1 f t @ r ) ((pos "Pu%" ) ))
( "thereby"  a ( dh e@1 b ai1 ) ((pos "Pu%" ) ))
( "therefore"  a ( dh e@1 f oo r ) ((pos "Pu%" ) ))
( "therefrom"  a ( dh e@1 f r o1 m ) ((pos "Pu%" ) ))
( "therein"  a ( dh e@1 r i1 n ) ((pos "Pu%" ) ))
( "thereinafter"  a ( dh e@2 r i n aa1 f t @ r ) ((pos "Pu$" ) ))
( "thereof"  a ( dh e@1 r o1 v ) ((pos "Pu%" ) ))
( "thereon"  a ( dh e@1 r o1 n ) ((pos "Pu%" ) ))
( "thereto"  a ( dh e@1 t uu1 ) ((pos "Pu%" ) ))
( "thereunder"  a ( dh e@1 r uh1 n d @ r ) ((pos "Pu%" ) ))
( "thereupon"  a ( dh e@2 r @ p o1 n ) ((pos "Pu%" ) ))
( "therewith"  a ( dh e@1 w i1 dh ) ((pos "Pu%" ) ))
( "therewithal"  a ( dh e@1 w i dh oo l ) ((pos "Pu$" ) ))
( "therm"  n ( th @@1 m ) ((pos "K6%" ) ))
( "thermal"  n ( th @@1 m l ) ((pos "K6%" "OA%" ) ))
( "thermally"  a ( th @@1 m @ l ii ) ((pos "Pu%" ) ))
( "thermals"  n ( th @@1 m l z ) ((pos "Kj%" ) ))
( "thermionic"  j ( th @@2 m i o1 n i k ) ((pos "OA$" ) ))
( "thermo-"  j ( th @@2 m ou - ) ((pos "U-%" ) ))
( "thermodynamics"  n ( th @@2 m ou d ai n a1 m i k s ) ((pos "Lk%" ) ))
( "thermometer"  n ( th @1 m o1 m i t @ r ) ((pos "K6%" ) ))
( "thermometers"  n ( th @1 m o1 m i t @ z ) ((pos "Kj%" ) ))
( "thermonuclear"  j ( th @@2 m ou n y uu1 k l i@ r ) ((pos "OA%" ) ))
( "thermoplastic"  n ( th @@2 m ou p l a1 s t i k ) ((pos "K6$" "OA$" ) ))
( "thermoplastics"  n ( th @@2 m ou p l a1 s t i k s ) ((pos "Kj$" ) ))
( "thermos"  n ( th @@1 m @ s ) ((pos "K7%" ) ))
( "thermoses"  n ( th @@1 m @ s i z ) ((pos "Kj%" ) ))
( "thermosetting"  j ( th @@1 m ou s e t i ng ) ((pos "OA$" ) ))
( "thermostat"  n ( th @@1 m @ s t a t ) ((pos "K6%" ) ))
( "thermostatic"  j ( th @@2 m @ s t a1 t i k ) ((pos "OA%" ) ))
( "thermostatically"  a ( th @@2 m @ s t a1 t i k l ii ) ((pos "Pu%" ) ))
( "thermostats"  n ( th @@1 m @ s t a t s ) ((pos "Kj%" ) ))
( "therms"  n ( th @@1 m z ) ((pos "Kj%" ) ))
( "thesaurus"  n ( th i1 s oo1 r @ s ) ((pos "K7%" ) ))
( "thesauruses"  n ( th i1 s oo1 r @ s i z ) ((pos "Kj%" ) ))
( "these"  j ( dh ii z ) ((pos "OA*" "Qx*" ) ))
( "theses"  n ( th ii1 s ii z ) ((pos "Kj%" ) ))
( "thesis"  n ( th ii1 s i s ) ((pos "Ki%" ) ))
( "thews"  n ( th y uu1 z ) ((pos "Kj$" ) ))
( "they"  prp ( dh ei ) ((pos "Qx*" ) ))
( "they'd"  v ( dh ei d ) ((pos "Gf%" ) ))
( "they'll"  v ( dh ei l ) ((pos "Gf%" ) ))
( "they're"  v ( dh e@ r ) ((pos "Gf%" ) ))
( "they've"  v ( dh ei v ) ((pos "Gf%" ) ))
( "thick"  n ( th i1 k ) ((pos "L@%" "OC%" "Pu%" ) ))
( "thick-headed"  j ( th i1 k - h e1 d i d ) ((pos "OA%" ) ))
( "thick-set"  j ( th i1 k - s e t ) ((pos "OA%" ) ))
( "thick-skinned"  j ( th i1 k - s k i n d ) ((pos "OA%" ) ))
( "thicken"  v ( th i1 k @ n ) ((pos "J0%" ) ))
( "thickened"  v ( th i1 k @ n d ) ((pos "Jc%" "Jd%" ) ))
( "thickening"  v ( th i1 k @ n i ng ) ((pos "Jb%" "L@%" ) ))
( "thickens"  v ( th i1 k @ n z ) ((pos "Ja%" ) ))
( "thicker"  j ( th i1 k @ r ) ((pos "Or%" ) ))
( "thickest"  j ( th i1 k i s t ) ((pos "Os%" ) ))
( "thicket"  n ( th i1 k i t ) ((pos "K6%" ) ))
( "thickets"  n ( th i1 k i t s ) ((pos "Kj$" ) ))
( "thickly"  a ( th i1 k l ii ) ((pos "Pu%" ) ))
( "thickness"  n ( th i1 k n @ s ) ((pos "M7%" ) ))
( "thicknesses"  n ( th i1 k n @ s i z ) ((pos "Mj%" ) ))
( "thief"  n ( th ii1 f ) ((pos "Ki%" ) ))
( "thieve"  v ( th ii1 v ) ((pos "J2%" ) (subcat "2A" )))
( "thieved"  v ( th ii1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "thievery"  n ( th ii1 v @ r ii ) ((pos "L@$" ) ))
( "thieves"  v ( th ii1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "thieving"  v ( th ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "thievish"  j ( th ii1 v i sh ) ((pos "OA$" ) ))
( "thievishly"  a ( th ii1 v i sh l ii ) ((pos "Pu$" ) ))
( "thigh"  n ( th ai1 ) ((pos "K6%" ) ))
( "thighbone"  n ( th ai1 b ou n ) ((pos "K6%" ) ))
( "thighbones"  n ( th ai1 b ou n z ) ((pos "Kj%" ) ))
( "thighs"  n ( th ai1 z ) ((pos "Kj%" ) ))
( "thimble"  n ( th i1 m b l ) ((pos "K6%" ) ))
( "thimbleful"  n ( th i1 m b l f u l ) ((pos "K6%" ) ))
( "thimblefuls"  n ( th i1 m b l f u l z ) ((pos "Kj%" ) ))
( "thimbles"  n ( th i1 m b l z ) ((pos "Kj%" ) ))
( "thin"  v ( th i1 n ) ((pos "J4%" "OE%" "Pu%" ) (subcat "2A" "2C" "6A" )))
( "thin-skinned"  j ( th i1 n - s k i n d ) ((pos "OA%" ) ))
( "thine"  j ( dh ai n ) ((pos "OA$" "Qx$" ) ))
( "thing"  n ( th i1 ng ) ((pos "K6*" ) ))
( "thingmabob"  n ( th i1 ng m @ b o b ) ((pos "K6$" ) ))
( "thingmabobs"  n ( th i1 ng m @ b o b z ) ((pos "Kj$" ) ))
( "thingmajig"  n ( th i1 ng m @ jh i g ) ((pos "K6$" ) ))
( "thingmajigs"  n ( th i1 ng m @ jh i g z ) ((pos "Kj$" ) ))
( "things"  n ( th i1 ng z ) ((pos "Kj*" ) ))
( "thingumabob"  n ( th i1 ng @ m @ b o b ) ((pos "K6$" ) ))
( "thingumabobs"  n ( th i1 ng @ m @ b o b z ) ((pos "Kj$" ) ))
( "thingumajig"  n ( th i1 ng @ m @ jh i g ) ((pos "K6$" ) ))
( "thingumajigs"  n ( th i1 ng @ m @ jh i g z ) ((pos "Kj$" ) ))
( "thingummies"  n ( th i1 ng @ m i z ) ((pos "Kj$" ) ))
( "thingummy"  n ( th i1 ng @ m ii ) ((pos "K8$" ) ))
( "think"  v ( th i1 ng k ) ((pos "J5*" "K6%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "8" "9" "10" "15B" "22" )))
( "think-tank"  n ( th i1 ng k - t a ng k ) ((pos "K6%" ) ))
( "think-tanks"  n ( th i1 ng k - t a ng k s ) ((pos "Kj%" ) ))
( "thinkable"  j ( th i1 ng k @ b l ) ((pos "OA$" ) ))
( "thinker"  n ( th i1 ng k @ r ) ((pos "K6%" ) ))
( "thinkers"  n ( th i1 ng k @ z ) ((pos "Kj%" ) ))
( "thinking"  v ( th i1 ng k i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "8" "9" "10" "15B" "22" )))
( "thinks"  v ( th i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "8" "9" "10" "15B" "22" )))
( "thinly"  a ( th i1 n l ii ) ((pos "Pu%" ) ))
( "thinned"  v ( th i1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "thinner"  j ( th i1 n @ r ) ((pos "Or%" ) ))
( "thinness"  n ( th i1 n - n @ s ) ((pos "L@%" ) ))
( "thinnest"  j ( th i1 n i s t ) ((pos "Os%" ) ))
( "thinning"  v ( th i1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "thins"  v ( th i1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "third"  n ( th @@1 d ) ((pos "K6%" "OA%" ) ))
( "third-rate"  j ( th @@1 d - r ei t ) ((pos "OA%" ) ))
( "third-rater"  n ( th @@1 d - r ei1 t @ r ) ((pos "K6%" ) ))
( "third-raters"  n ( th @@1 d - r ei1 t @ z ) ((pos "Kj%" ) ))
( "thirdly"  a ( th @@1 d l ii ) ((pos "Pu%" ) ))
( "thirds"  n ( th @@1 d z ) ((pos "Kj%" ) ))
( "thirst"  v ( th @@1 s t ) ((pos "H0%" "L@%" ) (subcat "2A" )))
( "thirsted"  v ( th @@1 s t i d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "thirstier"  j ( th @@1 s t i@ r ) ((pos "Or%" ) ))
( "thirstiest"  j ( th @@1 s t i i s t ) ((pos "Os%" ) ))
( "thirstily"  a ( th @@1 s t i l ii ) ((pos "Pu%" ) ))
( "thirsting"  v ( th @@1 s t i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "thirsts"  v ( th @@1 s t s ) ((pos "Ha%" ) (subcat "2A" )))
( "thirsty"  j ( th @@1 s t ii ) ((pos "OD%" ) ))
( "thirteen"  n ( th @@2 t ii1 n ) ((pos "K6%" "OA%" ) ))
( "thirteens"  n ( th @@2 t ii1 n z ) ((pos "Kj%" ) ))
( "thirteenth"  n ( th @@2 t ii1 n th ) ((pos "K6%" "OA%" ) ))
( "thirteenths"  n ( th @@2 t ii1 n th s ) ((pos "Kj%" ) ))
( "thirties"  n ( th @@1 t i z ) ((pos "Kj%" ) ))
( "thirtieth"  n ( th @@1 t i@ th ) ((pos "K6%" "OA%" ) ))
( "thirtieths"  n ( th @@1 t i@ th s ) ((pos "Kj%" ) ))
( "thirty"  n ( th @@1 t ii ) ((pos "K8%" "OA%" ) ))
( "this"  j ( dh i s ) ((pos "OA*" "Pu%" "Qx*" ) ))
( "thistle"  n ( th i1 s l ) ((pos "K6%" ) ))
( "thistledown"  n ( th i1 s l d au n ) ((pos "L@%" ) ))
( "thistles"  n ( th i1 s l z ) ((pos "Kj%" ) ))
( "thither"  a ( dh i1 dh @ r ) ((pos "Pu%" ) ))
( "tho'"  a ( dh ou ) ((pos "Pu%" "V-%" ) ))
( "thole"  n ( th ou1 l ) ((pos "K6$" ) ))
( "tholepin"  n ( th ou1 l p i n ) ((pos "K6$" ) ))
( "tholepins"  n ( th ou1 l p i n z ) ((pos "Kj$" ) ))
( "tholes"  n ( th ou1 l z ) ((pos "Kj$" ) ))
( "thong"  n ( th o1 ng ) ((pos "K6%" ) ))
( "thongs"  n ( th o1 ng z ) ((pos "Kj%" ) ))
( "thorax"  n ( th oo1 r a k s ) ((pos "K7%" ) ))
( "thoraxes"  n ( th oo1 r a k s i z ) ((pos "Kj$" ) ))
( "thorn"  n ( th oo1 n ) ((pos "M6%" ) ))
( "thornier"  j ( th oo1 n i@ r ) ((pos "Or%" ) ))
( "thorniest"  j ( th oo1 n i i s t ) ((pos "Os%" ) ))
( "thorns"  n ( th oo1 n z ) ((pos "Mj%" ) ))
( "thorny"  j ( th oo1 n ii ) ((pos "OD%" ) ))
( "thorough"  j ( th uh1 r @ ) ((pos "OA%" ) ))
( "thoroughbred"  n ( th uh1 r @ b r e d ) ((pos "K6%" "OA%" ) ))
( "thoroughbreds"  n ( th uh1 r @ b r e d z ) ((pos "Kj%" ) ))
( "thoroughfare"  n ( th uh1 r @ f e@ r ) ((pos "K6%" ) ))
( "thoroughfares"  n ( th uh1 r @ f e@ z ) ((pos "Kj%" ) ))
( "thoroughgoing"  j ( th uh1 r @ g ou i ng ) ((pos "OA%" ) ))
( "thoroughly"  a ( th uh1 r @ l ii ) ((pos "Pu%" ) ))
( "thoroughness"  n ( th uh1 r @ n @ s ) ((pos "L@%" ) ))
( "those"  j ( dh ou z ) ((pos "OA*" "Qx*" ) ))
( "thou"  prp ( dh au ) ((pos "Qx$" ) ))
( "though"  a ( dh ou ) ((pos "Pu%" "V-*" ) ))
( "thought"  v ( th oo1 t ) ((pos "Jc*" "Jd*" "M6*" ) (subcat "2A" "2B" "2C" "3A" "6A" "7A" "8" "9" "10" "15B" "22" )))
( "thought-reader"  n ( th oo1 t - r ii d @ r ) ((pos "K6%" ) ))
( "thought-readers"  n ( th oo1 t - r ii d @ z ) ((pos "Kj%" ) ))
( "thoughtful"  j ( th oo1 t f @ l ) ((pos "OA%" ) ))
( "thoughtfully"  a ( th oo1 t f @ l ii ) ((pos "Pu%" ) ))
( "thoughtfulness"  n ( th oo1 t f @ l n @ s ) ((pos "L@%" ) ))
( "thoughtless"  j ( th oo1 t l @ s ) ((pos "OA%" ) ))
( "thoughtlessly"  a ( th oo1 t l @ s l ii ) ((pos "Pu%" ) ))
( "thoughtlessness"  n ( th oo1 t l @ s n @ s ) ((pos "L@%" ) ))
( "thoughts"  n ( th oo1 t s ) ((pos "Mj%" ) ))
( "thousand"  n ( th au1 z n d ) ((pos "K6%" "OA%" ) ))
( "thousandfold"  j ( th au1 z n d f ou l d ) ((pos "OA%" "Pu%" ) ))
( "thousands"  n ( th au1 z n d z ) ((pos "Kj%" ) ))
( "thousandth"  n ( th au1 z n th ) ((pos "K6%" "OA%" ) ))
( "thousandths"  n ( th au1 z n th s ) ((pos "Kj%" ) ))
( "thraldom"  n ( th r aa1 l d @ m ) ((pos "L@$" ) ))
( "thrall"  n ( th r oo1 l ) ((pos "M6%" ) ))
( "thralls"  n ( th r oo1 l z ) ((pos "Mj$" ) ))
( "thrash"  v ( th r a1 sh ) ((pos "J1%" ) (subcat "2C" "6A" "15A" )))
( "thrashed"  v ( th r a1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "thrashes"  v ( th r a1 sh i z ) ((pos "Ja%" ) (subcat "2C" "6A" "15A" )))
( "thrashing"  v ( th r a1 sh i ng ) ((pos "Jb%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "thrashings"  n ( th r a1 sh i ng z ) ((pos "Kj%" ) ))
( "thread"  v ( th r e1 d ) ((pos "H0%" "M6%" ) (subcat "6A" )))
( "threadbare"  j ( th r e1 d b e@ r ) ((pos "OA%" ) ))
( "threaded"  v ( th r e1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "threading"  v ( th r e1 d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "threadlike"  j ( th r e1 d l ai k ) ((pos "OA%" ) ))
( "threads"  v ( th r e1 d z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "threat"  n ( th r e1 t ) ((pos "K6%" ) ))
( "threaten"  v ( th r e1 t n ) ((pos "J0%" ) (subcat "2A" "6A" "14" )))
( "threatened"  v ( th r e1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" )))
( "threatening"  v ( th r e1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "threateningly"  a ( th r e1 t n i ng l ii ) ((pos "Pu%" ) ))
( "threatens"  v ( th r e1 t n z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "threats"  n ( th r e1 t s ) ((pos "Kj%" ) ))
( "three"  n ( th r ii1 ) ((pos "K6%" "OA*" ) ))
( "three-D"  n ( th r ii1 - d ii1 ) ((pos "L@$" "OA$" ) ))
( "three-cornered"  j ( th r ii1 - k oo1 n @ d ) ((pos "OA%" ) ))
( "three-decker"  n ( th r ii1 - d e1 k @ r ) ((pos "K6$" ) ))
( "three-deckers"  n ( th r ii1 - d e1 k @ z ) ((pos "Kj$" ) ))
( "three-dimensional"  j ( th r ii2 - d i m e1 n sh @ n @ l ) ((pos "OA%" ) ))
( "three-figure"  j ( th r ii1 - f i g @ r ) ((pos "OA$" ) ))
( "three-funnelled"  j ( th r ii1 - f uh1 n @ l d ) ((pos "OA$" ) ))
( "three-lane"  j ( th r ii1 - l ei n ) ((pos "OA%" ) ))
( "three-legged"  j ( th r ii1 - l e g i d ) ((pos "OA%" ) ))
( "three-piece"  j ( th r ii1 - p ii s ) ((pos "OA%" ) ))
( "three-ply"  j ( th r ii1 - p l ai ) ((pos "OA%" ) ))
( "three-quarter"  n ( th r ii1 - k w oo1 t @ r ) ((pos "K6%" "OA%" ) ))
( "three-quarters"  n ( th r ii1 - k w oo1 t @ z ) ((pos "Kj%" ) ))
( "three-score"  n ( th r ii1 - s k oo r ) ((pos "L@%" "OA%" ) ))
( "three-storey"  j ( th r ii1 - s t oo r ii ) ((pos "OA%" ) ))
( "three-storeyed"  j ( th r ii1 - s t oo r i d ) ((pos "OA%" ) ))
( "three-wheeled"  j ( th r ii1 - w ii l d ) ((pos "OA%" ) ))
( "threefold"  j ( th r ii1 f ou l d ) ((pos "OA%" "Pu%" ) ))
( "threepence"  n ( th r e1 p @ n s ) ((pos "K6%" ) ))
( "threepences"  n ( th r e1 p @ n s i z ) ((pos "Kj%" ) ))
( "threepenny"  j ( th r e1 p @ n ii ) ((pos "OA%" ) ))
( "threes"  n ( th r ii1 z ) ((pos "Kj%" ) ))
( "threesome"  n ( th r ii1 s @ m ) ((pos "K6%" ) ))
( "threesomes"  n ( th r ii1 s @ m z ) ((pos "Kj%" ) ))
( "threnodies"  n ( th r e1 n @ d i z ) ((pos "Kj$" ) ))
( "threnody"  n ( th r e1 n @ d ii ) ((pos "K8$" ) ))
( "thresh"  v ( th r e1 sh ) ((pos "J1%" ) (subcat "2A" "6A" )))
( "threshed"  v ( th r e1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "thresher"  n ( th r e1 sh @ r ) ((pos "K6%" ) ))
( "threshers"  n ( th r e1 sh @ z ) ((pos "Kj%" ) ))
( "threshes"  v ( th r e1 sh i z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "threshing"  v ( th r e1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "threshing-floor"  n ( th r e1 sh i ng - f l oo r ) ((pos "K6%" ) ))
( "threshing-floors"  n ( th r e1 sh i ng - f l oo z ) ((pos "Kj$" ) ))
( "threshing-machine"  n ( th r e1 sh i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "threshing-machines"  n ( th r e1 sh i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "threshold"  n ( th r e1 sh h ou l d ) ((pos "K6%" ) ))
( "thresholds"  n ( th r e1 sh h ou l d z ) ((pos "Kj%" ) ))
( "threw"  v ( th r uu1 ) ((pos "Jc%" ) (subcat "2A" "6A" "12A" "13A" "15A" "15B" )))
( "thrice"  a ( th r ai1 s ) ((pos "Pu$" ) ))
( "thrift"  n ( th r i1 f t ) ((pos "L@%" ) ))
( "thriftier"  j ( th r i1 f t i@ r ) ((pos "Or$" ) ))
( "thriftiest"  j ( th r i1 f t i i s t ) ((pos "Os$" ) ))
( "thriftily"  a ( th r i1 f t i l ii ) ((pos "Pu$" ) ))
( "thriftless"  j ( th r i1 f t l @ s ) ((pos "OA$" ) ))
( "thriftlessly"  a ( th r i1 f t l @ s l ii ) ((pos "Pu$" ) ))
( "thriftlessness"  n ( th r i1 f t l @ s n @ s ) ((pos "L@$" ) ))
( "thrifty"  j ( th r i1 f t ii ) ((pos "OD%" ) ))
( "thrill"  v ( th r i1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "thrilled"  v ( th r i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "thriller"  n ( th r i1 l @ r ) ((pos "K6%" ) ))
( "thrillers"  n ( th r i1 l @ z ) ((pos "Kj%" ) ))
( "thrilling"  v ( th r i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "thrills"  v ( th r i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "thrive"  v ( th r ai1 v ) ((pos "I2%" ) (subcat "2A" )))
( "thrived"  v ( th r ai1 v d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "thriven"  v ( th r i1 v @ n ) ((pos "Id$" ) (subcat "2A" )))
( "thrives"  v ( th r ai1 v z ) ((pos "Ia%" ) (subcat "2A" )))
( "thriving"  v ( th r ai1 v i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "thro'"  in ( th r uu ) ((pos "T-%" ) ))
( "throat"  n ( th r ou1 t ) ((pos "K6%" ) ))
( "throated"  j ( th r ou1 t i d ) ((pos "Ot%" ) ))
( "throatier"  j ( th r ou1 t i@ r ) ((pos "Or%" ) ))
( "throatiest"  j ( th r ou1 t i i s t ) ((pos "Os%" ) ))
( "throats"  n ( th r ou1 t s ) ((pos "Kj%" ) ))
( "throaty"  j ( th r ou1 t ii ) ((pos "OD%" ) ))
( "throb"  v ( th r o1 b ) ((pos "I4%" "K6%" ) (subcat "2A" )))
( "throbbed"  v ( th r o1 b d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "throbbing"  v ( th r o1 b i ng ) ((pos "Ib%" "OA%" ) (subcat "2A" )))
( "throbs"  v ( th r o1 b z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "throe"  n ( th r ou1 ) ((pos "K6%" ) ))
( "throes"  n ( th r ou1 z ) ((pos "Kj%" ) ))
( "thrombosis"  n ( th r o1 m b ou1 s i s ) ((pos "L@%" ) ))
( "throne"  n ( th r ou1 n ) ((pos "K6%" ) ))
( "thrones"  n ( th r ou1 n z ) ((pos "Kj%" ) ))
( "throng"  v ( th r o1 ng ) ((pos "J0%" "K6%" ) (subcat "2C" "4A" )))
( "thronged"  v ( th r o1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "4A" )))
( "thronging"  v ( th r o1 ng i ng ) ((pos "Jb%" ) (subcat "2C" "4A" )))
( "throngs"  v ( th r o1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "4A" )))
( "throstle"  n ( th r o1 s l ) ((pos "K6%" ) ))
( "throstles"  n ( th r o1 s l z ) ((pos "Kj$" ) ))
( "throttle"  v ( th r o1 t l ) ((pos "J2%" "M6%" ) (subcat "2C" "6A" )))
( "throttle-valve"  n ( th r o1 t l - v a l v ) ((pos "K6%" ) ))
( "throttle-valves"  n ( th r o1 t l - v a l v z ) ((pos "Kj%" ) ))
( "throttled"  v ( th r o1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "throttles"  v ( th r o1 t l z ) ((pos "Ja%" "Mj%" ) (subcat "2C" "6A" )))
( "throttling"  v ( th r o1 t l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "through"  a ( th r uu ) ((pos "Pu*" "T-*" ) ))
( "throughout"  a ( th r uu1 au1 t ) ((pos "Pu%" "T-%" ) ))
( "throughput"  n ( th r uu1 p u t ) ((pos "M6%" ) ))
( "throughputs"  n ( th r uu1 p u t s ) ((pos "Mj$" ) ))
( "throughway"  n ( th r uu1 w ei ) ((pos "K6$" ) ))
( "throughways"  n ( th r uu1 w ei z ) ((pos "Kj$" ) ))
( "throve"  v ( th r ou1 v ) ((pos "Ic$" ) (subcat "2A" )))
( "throw"  v ( th r ou1 ) ((pos "J5%" "K6%" ) (subcat "2A" "6A" "12A" "13A" "15A" "15B" )))
( "throw-in"  n ( th r ou1 - i n ) ((pos "K6%" ) ))
( "throw-ins"  n ( th r ou1 - i n z ) ((pos "Kj%" ) ))
( "throwaway"  n ( th r ou1 @ w ei ) ((pos "K6%" ) ))
( "throwaways"  n ( th r ou1 @ w ei z ) ((pos "Kj$" ) ))
( "throwback"  n ( th r ou1 b a k ) ((pos "K6%" ) ))
( "throwbacks"  n ( th r ou1 b a k s ) ((pos "Kj%" ) ))
( "throwing"  v ( th r ou1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "12A" "13A" "15A" "15B" )))
( "thrown"  v ( th r ou1 n ) ((pos "Jd%" ) (subcat "2A" "6A" "12A" "13A" "15A" "15B" )))
( "throws"  v ( th r ou1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "12A" "13A" "15A" "15B" )))
( "thru"  in ( th r uu ) ((pos "T-$" ) ))
( "thrum"  v ( th r uh1 m ) ((pos "J4$" ) (subcat "3A" )))
( "thrummed"  v ( th r uh1 m d ) ((pos "Jc$" "Jd$" ) (subcat "3A" )))
( "thrumming"  v ( th r uh1 m i ng ) ((pos "Jb$" ) (subcat "3A" )))
( "thrums"  v ( th r uh1 m z ) ((pos "Ja$" ) (subcat "3A" )))
( "thrush"  n ( th r uh1 sh ) ((pos "K7%" ) ))
( "thrushes"  n ( th r uh1 sh i z ) ((pos "Kj%" ) ))
( "thrust"  v ( th r uh1 s t ) ((pos "J5%" "Jc%" "Jd%" "M6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "thruster"  n ( th r uh1 s t @ r ) ((pos "K6%" ) ))
( "thrusters"  n ( th r uh1 s t @ z ) ((pos "Kj$" ) ))
( "thrusting"  v ( th r uh1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "thrusts"  v ( th r uh1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "thud"  v ( th uh1 d ) ((pos "I4%" "K6%" ) (subcat )))
( "thudded"  v ( th uh1 d i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "thudding"  v ( th uh1 d i ng ) ((pos "Ib%" ) (subcat )))
( "thuds"  v ( th uh1 d z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "thug"  n ( th uh1 g ) ((pos "K6%" ) ))
( "thuggery"  n ( th uh1 g @ r ii ) ((pos "L@%" ) ))
( "thugs"  n ( th uh1 g z ) ((pos "Kj%" ) ))
( "thumb"  v ( th uh1 m ) ((pos "H0%" "K6%" ) (subcat )))
( "thumbed"  v ( th uh1 m d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "thumbing"  v ( th uh1 m i ng ) ((pos "Hb%" ) (subcat )))
( "thumbnut"  n ( th uh1 m n uh t ) ((pos "K6$" ) ))
( "thumbnuts"  n ( th uh1 m n uh t s ) ((pos "Kj$" ) ))
( "thumbs"  v ( th uh1 m z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "thumbscrew"  n ( th uh1 m s k r uu ) ((pos "K6%" ) ))
( "thumbscrews"  n ( th uh1 m s k r uu z ) ((pos "Kj%" ) ))
( "thumbstall"  n ( th uh1 m s t oo l ) ((pos "K6$" ) ))
( "thumbstalls"  n ( th uh1 m s t oo l z ) ((pos "Kj$" ) ))
( "thumbtack"  n ( th uh1 m t a k ) ((pos "K6%" ) ))
( "thumbtacks"  n ( th uh1 m t a k s ) ((pos "Kj%" ) ))
( "thump"  v ( th uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "thumped"  v ( th uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "thumping"  v ( th uh1 m p i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "thumps"  v ( th uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "thunder"  v ( th uh1 n d @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "3A" )))
( "thunderbolt"  n ( th uh1 n d @ b ou l t ) ((pos "K6%" ) ))
( "thunderbolts"  n ( th uh1 n d @ b ou l t s ) ((pos "Kj%" ) ))
( "thunderclap"  n ( th uh1 n d @ k l a p ) ((pos "K6%" ) ))
( "thunderclaps"  n ( th uh1 n d @ k l a p s ) ((pos "Kj%" ) ))
( "thundered"  v ( th uh1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "thundering"  v ( th uh1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "thunderous"  j ( th uh1 n d @ r @ s ) ((pos "OA%" ) ))
( "thunders"  v ( th uh1 n d @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" )))
( "thunderstorm"  n ( th uh1 n d @ s t oo m ) ((pos "K6%" ) ))
( "thunderstorms"  n ( th uh1 n d @ s t oo m z ) ((pos "Kj%" ) ))
( "thunderstruck"  j ( th uh1 n d @ s t r uh k ) ((pos "OA%" ) ))
( "thundery"  j ( th uh1 n d @ r ii ) ((pos "OA%" ) ))
( "thurible"  n ( th y u@1 r @ b l ) ((pos "K6$" ) ))
( "thuribles"  n ( th y u@1 r @ b l z ) ((pos "Kj$" ) ))
( "thus"  a ( dh uh1 s ) ((pos "Pu%" ) ))
( "thwack"  v ( th w a1 k ) ((pos "H0$" "K6%" ) ))
( "thwacked"  v ( th w a1 k t ) ((pos "Hc$" "Hd$" ) ))
( "thwacking"  v ( th w a1 k i ng ) ((pos "Hb$" ) ))
( "thwacks"  v ( th w a1 k s ) ((pos "Ha$" "Kj$" ) ))
( "thwart"  v ( th w oo1 t ) ((pos "H0%" "K6%" ) (subcat )))
( "thwarted"  v ( th w oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "thwarting"  v ( th w oo1 t i ng ) ((pos "Hb%" ) (subcat )))
( "thwarts"  v ( th w oo1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "thy"  j ( dh ai1 ) ((pos "OA$" ) ))
( "thyme"  n ( t ai1 m ) ((pos "L@%" ) ))
( "thyroid"  n ( th ai1 r oi d ) ((pos "K6%" ) ))
( "thyroids"  n ( th ai1 r oi d z ) ((pos "Kj$" ) ))
( "thyself"  prp ( dh ai1 s e1 l f ) ((pos "Qx$" ) ))
( "ti"  n ( t ii1 ) ((pos "Ki$" ) ))
( "tiara"  n ( t i1 aa1 r @ ) ((pos "K6%" ) ))
( "tiaras"  n ( t i1 aa1 r @ z ) ((pos "Kj%" ) ))
( "tibia"  n ( t i1 b i@ ) ((pos "Ki%" ) ))
( "tibiae"  n ( t i1 b i ii ) ((pos "Kj$" ) ))
( "tic"  n ( t i1 k ) ((pos "K6%" ) ))
( "tick"  v ( t i1 k ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "10" )))
( "tick-tock"  n ( t i1 k - t o k ) ((pos "K6%" ) ))
( "tick-tocks"  n ( t i1 k - t o k s ) ((pos "Kj$" ) ))
( "ticked"  v ( t i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "10" )))
( "ticker"  n ( t i1 k @ r ) ((pos "K6%" ) ))
( "ticker-tape"  n ( t i1 k @ - t ei p ) ((pos "L@%" ) ))
( "tickers"  n ( t i1 k @ z ) ((pos "Kj%" ) ))
( "ticket"  v ( t i1 k i t ) ((pos "H0%" "K6%" ) (subcat )))
( "ticket-collector"  n ( t i1 k i t - k @ l e2 k t @ r ) ((pos "K6%" ) ))
( "ticket-collectors"  n ( t i1 k i t - k @ l e2 k t @ z ) ((pos "Kj%" ) ))
( "ticketed"  v ( t i1 k i t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ticketing"  v ( t i1 k i t i ng ) ((pos "Hb%" ) (subcat )))
( "tickets"  v ( t i1 k i t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "ticking"  v ( t i1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "10" )))
( "tickle"  v ( t i1 k l ) ((pos "J2%" ) (subcat "2A" )))
( "tickled"  v ( t i1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tickler"  n ( t i1 k l @ r ) ((pos "K6%" ) ))
( "ticklers"  n ( t i1 k l @ z ) ((pos "Kj%" ) ))
( "tickles"  v ( t i1 k l z ) ((pos "Ja%" ) (subcat "2A" )))
( "tickling"  v ( t i1 k l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ticklish"  j ( t i1 k l i sh ) ((pos "OA%" ) ))
( "ticks"  v ( t i1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "10" )))
( "tics"  n ( t i1 k s ) ((pos "Kj%" ) ))
( "tidal"  j ( t ai1 d l ) ((pos "OA%" ) ))
( "tidbit"  n ( t i1 d b i t ) ((pos "K6$" ) ))
( "tidbits"  n ( t i1 d b i t s ) ((pos "Kj$" ) ))
( "tiddler"  n ( t i1 d l @ r ) ((pos "K6%" ) ))
( "tiddlers"  n ( t i1 d l @ z ) ((pos "Kj%" ) ))
( "tiddley"  j ( t i1 d l ii ) ((pos "OA%" ) ))
( "tiddlywinks"  n ( t i1 d l i w i ng k s ) ((pos "Lk%" ) ))
( "tide"  v ( t ai1 d ) ((pos "H2%" "M6%" ) (subcat "14" )))
( "tided"  v ( t ai1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "tidemark"  n ( t ai1 d m aa k ) ((pos "K6%" ) ))
( "tidemarks"  n ( t ai1 d m aa k s ) ((pos "Kj%" ) ))
( "tides"  v ( t ai1 d z ) ((pos "Ha$" "Mj%" ) (subcat "14" )))
( "tideway"  n ( t ai1 d w ei ) ((pos "K6%" ) ))
( "tideways"  n ( t ai1 d w ei z ) ((pos "Kj%" ) ))
( "tidied"  v ( t ai1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "tidier"  j ( t ai1 d i@ r ) ((pos "Or%" ) ))
( "tidies"  v ( t ai1 d i z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2C" "6A" )))
( "tidiest"  j ( t ai1 d i i s t ) ((pos "Os%" ) ))
( "tidily"  a ( t ai1 d i l ii ) ((pos "Pu%" ) ))
( "tidiness"  n ( t ai1 d i n @ s ) ((pos "L@%" ) ))
( "tiding"  v ( t ai1 d i ng ) ((pos "Hb$" ) (subcat "14" )))
( "tidings"  n ( t ai1 d i ng z ) ((pos "Kj%" ) ))
( "tidy"  v ( t ai1 d ii ) ((pos "J3%" "K8$" "OD%" ) (subcat "2A" "2C" "6A" )))
( "tidying"  v ( t ai1 d i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "tie"  v ( t ai1 ) ((pos "J5%" "K6%" ) (subcat "2A" "3A" "6A" "15A" )))
( "tie-on"  j ( t ai1 - o n ) ((pos "Oq%" ) ))
( "tie-up"  n ( t ai1 - uh p ) ((pos "K6%" ) ))
( "tie-ups"  n ( t ai1 - uh p s ) ((pos "Kj$" ) ))
( "tied"  v ( t ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "15A" )))
( "tier"  n ( t i@1 r ) ((pos "K6%" ) ))
( "tiers"  n ( t i@1 z ) ((pos "Kj%" ) ))
( "ties"  v ( t ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "15A" )))
( "tiff"  n ( t i1 f ) ((pos "K6%" ) ))
( "tiffs"  n ( t i1 f s ) ((pos "Kj%" ) ))
( "tiger"  n ( t ai1 g @ r ) ((pos "K6%" ) ))
( "tiger-lilies"  n ( t ai1 g @ - l i l i z ) ((pos "Kj%" ) ))
( "tiger-lily"  n ( t ai1 g @ - l i l ii ) ((pos "K8%" ) ))
( "tigerish"  j ( t ai1 g @ r i sh ) ((pos "OA%" ) ))
( "tigers"  n ( t ai1 g @ z ) ((pos "Kj%" ) ))
( "tight"  j ( t ai1 t ) ((pos "OC%" "Pu%" ) ))
( "tight-laced"  j ( t ai1 t - l ei1 s t ) ((pos "OA%" ) ))
( "tight-lipped"  j ( t ai1 t - l i1 p t ) ((pos "OA%" ) ))
( "tight-wad"  n ( t ai1 t - w o d ) ((pos "K6$" ) ))
( "tight-wads"  n ( t ai1 t - w o d z ) ((pos "Kj$" ) ))
( "tighten"  v ( t ai1 t n ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "tightened"  v ( t ai1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "tightening"  v ( t ai1 t n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "tightens"  v ( t ai1 t n z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "tighter"  j ( t ai1 t @ r ) ((pos "Or%" ) ))
( "tightest"  j ( t ai1 t i s t ) ((pos "Os%" ) ))
( "tightly"  a ( t ai1 t l ii ) ((pos "Pu%" ) ))
( "tightness"  n ( t ai1 t n @ s ) ((pos "L@%" ) ))
( "tightrope"  n ( t ai1 t r ou p ) ((pos "K6%" ) ))
( "tightropes"  n ( t ai1 t r ou p s ) ((pos "Kj%" ) ))
( "tights"  n ( t ai1 t s ) ((pos "Kj%" ) ))
( "tigress"  n ( t ai1 g r i s ) ((pos "K7%" ) ))
( "tigresses"  n ( t ai1 g r i s i z ) ((pos "Kj%" ) ))
( "tike"  n ( t ai1 k ) ((pos "K6$" ) ))
( "tikes"  n ( t ai1 k s ) ((pos "Kj$" ) ))
( "tilde"  n ( t i1 l d @ ) ((pos "K6$" ) ))
( "tilde"  n ( t i1 l d ) ((pos "K6$" ) ))
( "tildes"  n ( t i1 l d @ z ) ((pos "Kj$" ) ))
( "tildes"  n ( t i1 l d z ) ((pos "Kj$" ) ))
( "tile"  v ( t ai1 l ) ((pos "H2%" "K6%" ) (subcat )))
( "tiled"  v ( t ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tiles"  v ( t ai1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "tiling"  v ( t ai1 l i ng ) ((pos "Hb%" ) (subcat )))
( "till"  v ( t i l ) ((pos "H0%" "K6%" "T-%" "V-%" ) (subcat )))
( "tillage"  n ( t i1 l i jh ) ((pos "L@$" ) ))
( "tilled"  v ( t i1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tiller"  n ( t i1 l @ r ) ((pos "K6%" ) ))
( "tillers"  n ( t i1 l @ z ) ((pos "Kj%" ) ))
( "tilling"  v ( t i1 l i ng ) ((pos "Hb%" ) (subcat )))
( "tills"  v ( t i1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "tilt"  v ( t i1 l t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tilted"  v ( t i1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tilth"  n ( t i1 l th ) ((pos "K6$" ) ))
( "tilths"  n ( t i1 l th s ) ((pos "Kj$" ) ))
( "tilting"  v ( t i1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tilts"  v ( t i1 l t s ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tiltyard"  n ( t i1 l t y aa d ) ((pos "K6$" ) ))
( "tiltyards"  n ( t i1 l t y aa d z ) ((pos "Kj$" ) ))
( "timber"  n ( t i1 m b @ r ) ((pos "M6%" ) ))
( "timbered"  j ( t i1 m b @ d ) ((pos "OA%" ) ))
( "timbers"  n ( t i1 m b @ z ) ((pos "Mj%" ) ))
( "timbre"  n ( t a1 m b r @ ) ((pos "K6%" ) ))
( "timbrel"  n ( t i1 m b r @ l ) ((pos "K6$" ) ))
( "timbrels"  n ( t i1 m b r @ l z ) ((pos "Kj$" ) ))
( "timbres"  n ( t a1 m b r @ z ) ((pos "Kj$" ) ))
( "time"  v ( t ai1 m ) ((pos "H2%" "M6*" ) (subcat "6A" )))
( "time-ball"  n ( t ai1 m - b oo l ) ((pos "K6$" ) ))
( "time-balls"  n ( t ai1 m - b oo l z ) ((pos "Kj$" ) ))
( "time-bomb"  n ( t ai1 m - b o m ) ((pos "K6%" ) ))
( "time-bombs"  n ( t ai1 m - b o m z ) ((pos "Kj%" ) ))
( "time-expired"  j ( t ai2 m - i k s p ai1 @ d ) ((pos "OA$" ) ))
( "time-exposure"  n ( t ai1 m - i k s p ou2 zh @ r ) ((pos "K6%" ) ))
( "time-exposures"  n ( t ai1 m - i k s p ou2 zh @ z ) ((pos "Kj%" ) ))
( "time-fuse"  n ( t ai1 m - f y uu z ) ((pos "K6%" ) ))
( "time-fuses"  n ( t ai1 m - f y uu z i z ) ((pos "Kj%" ) ))
( "time-honoured"  j ( t ai1 m - o n @ d ) ((pos "OA%" ) ))
( "time-lag"  n ( t ai1 m - l a g ) ((pos "K6%" ) ))
( "time-lags"  n ( t ai1 m - l a g z ) ((pos "Kj%" ) ))
( "time-limit"  n ( t ai1 m - l i m i t ) ((pos "K6%" ) ))
( "time-limits"  n ( t ai1 m - l i m i t s ) ((pos "Kj%" ) ))
( "time-sheet"  n ( t ai1 m - sh ii t ) ((pos "K6%" ) ))
( "time-sheets"  n ( t ai1 m - sh ii t s ) ((pos "Kj%" ) ))
( "time-signal"  n ( t ai1 m - s i g n @ l ) ((pos "K6%" ) ))
( "time-signals"  n ( t ai1 m - s i g n @ l z ) ((pos "Kj%" ) ))
( "time-switch"  n ( t ai1 m - s w i ch ) ((pos "K7%" ) ))
( "time-switches"  n ( t ai1 m - s w i ch i z ) ((pos "Kj%" ) ))
( "timecard"  n ( t ai1 m k aa d ) ((pos "K6$" ) ))
( "timecards"  n ( t ai1 m k aa d z ) ((pos "Kj$" ) ))
( "timed"  v ( t ai1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "timekeeper"  n ( t ai1 m k ii p @ r ) ((pos "K6%" ) ))
( "timekeepers"  n ( t ai1 m k ii p @ z ) ((pos "Kj%" ) ))
( "timekeeping"  n ( t ai1 m k ii p i ng ) ((pos "L@%" ) ))
( "timeless"  j ( t ai1 m l @ s ) ((pos "OA%" ) ))
( "timelier"  j ( t ai1 m l i@ r ) ((pos "Or$" ) ))
( "timeliest"  j ( t ai1 m l i i s t ) ((pos "Os$" ) ))
( "timeliness"  n ( t ai1 m l i n @ s ) ((pos "L@%" ) ))
( "timely"  j ( t ai1 m l ii ) ((pos "OD%" ) ))
( "timepiece"  n ( t ai1 m p ii s ) ((pos "K6%" ) ))
( "timepieces"  n ( t ai1 m p ii s i z ) ((pos "Kj%" ) ))
( "times"  v ( t ai1 m z ) ((pos "Ha%" "Mj*" ) (subcat "6A" )))
( "timesaving"  j ( t ai1 m s ei v i ng ) ((pos "OA%" ) ))
( "timeserver"  n ( t ai1 m s @@ v @ r ) ((pos "K6%" ) ))
( "timeservers"  n ( t ai1 m s @@ v @ z ) ((pos "Kj%" ) ))
( "timeserving"  j ( t ai1 m s @@ v i ng ) ((pos "OA%" ) ))
( "timetable"  n ( t ai1 m t ei b l ) ((pos "K6%" ) ))
( "timetables"  n ( t ai1 m t ei b l z ) ((pos "Kj%" ) ))
( "timework"  n ( t ai1 m w @@ k ) ((pos "L@$" ) ))
( "timid"  j ( t i1 m i d ) ((pos "OA%" ) ))
( "timidity"  n ( t i1 m i1 d i t ii ) ((pos "L@%" ) ))
( "timidly"  a ( t i1 m i d l ii ) ((pos "Pu%" ) ))
( "timidness"  n ( t i1 m i d n @ s ) ((pos "L@%" ) ))
( "timing"  v ( t ai1 m i ng ) ((pos "Hb%" "M6%" ) (subcat "6A" )))
( "timings"  n ( t ai1 m i ng z ) ((pos "Mj%" ) ))
( "timorous"  j ( t i1 m @ r @ s ) ((pos "OA%" ) ))
( "timorously"  a ( t i1 m @ r @ s l ii ) ((pos "Pu%" ) ))
( "timothy"  n ( t i1 m @ th ii ) ((pos "L@$" ) ))
( "timpani"  n ( t i1 m p @ n ii ) ((pos "Kj%" ) ))
( "timpanist"  n ( t i1 m p @ n i s t ) ((pos "K6%" ) ))
( "timpanists"  n ( t i1 m p @ n i s t s ) ((pos "Kj%" ) ))
( "tin"  v ( t i1 n ) ((pos "H4%" "M6%" ) (subcat )))
( "tin pan alley"  n ( t i2 n - p a n - a1 l ii ) ((pos "K6%" ) ))
( "tin pan alleys"  n ( t i2 n - p a n - a1 l i z ) ((pos "Kj$" ) ))
( "tin-opener"  n ( t i1 n - ou p @ n @ r ) ((pos "K6%" ) ))
( "tin-openers"  n ( t i1 n - ou p @ n @ z ) ((pos "Kj%" ) ))
( "tin-plate"  n ( t i1 n - p l ei t ) ((pos "L@$" ) ))
( "tincture"  v ( t i1 ng k ch @ r ) ((pos "H2$" "K6%" ) ))
( "tinctured"  v ( t i1 ng k ch @ d ) ((pos "Hc$" "Hd$" ) ))
( "tinctures"  v ( t i1 ng k ch @ z ) ((pos "Ha$" "Kj$" ) ))
( "tincturing"  v ( t i1 ng k ch @ r i ng ) ((pos "Hb$" ) ))
( "tinder"  n ( t i1 n d @ r ) ((pos "L@%" ) ))
( "tinderbox"  n ( t i1 n d @ b o k s ) ((pos "K7%" ) ))
( "tinderboxes"  n ( t i1 n d @ b o k s i z ) ((pos "Kj%" ) ))
( "tine"  n ( t ai1 n ) ((pos "K6$" ) ))
( "tined"  j ( t ai1 n d ) ((pos "Ot$" ) ))
( "tines"  n ( t ai1 n z ) ((pos "Kj$" ) ))
( "tinfoil"  n ( t i1 n f oi l ) ((pos "L@%" ) ))
( "ting"  v ( t i1 ng ) ((pos "J0$" "K6%" ) ))
( "tinge"  v ( t i1 n jh ) ((pos "H2$" "K6%" ) (subcat "6A" )))
( "tinged"  v ( t i1 ng d ) ((pos "Jc$" "Jd$" ) ))
( "tinged"  v ( t i1 n jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tinges"  v ( t i1 n jh i z ) ((pos "Ha$" "Kj$" ) (subcat "6A" )))
( "tinging"  v ( t i1 ng i ng ) ((pos "Jb$" ) ))
( "tinging"  v ( t i1 n jh i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "tingle"  v ( t i1 ng g l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "tingled"  v ( t i1 ng g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "tingles"  v ( t i1 ng g l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "tingling"  v ( t i1 ng g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "tings"  v ( t i1 ng z ) ((pos "Ja$" "Kj$" ) ))
( "tinier"  j ( t ai1 n i@ r ) ((pos "Or%" ) ))
( "tiniest"  j ( t ai1 n i i s t ) ((pos "Os%" ) ))
( "tinker"  v ( t i1 ng k @ r ) ((pos "I0%" "K6%" ) (subcat "2A" "2C" )))
( "tinkered"  v ( t i1 ng k @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "tinkering"  v ( t i1 ng k @ r i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "tinkers"  v ( t i1 ng k @ z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "tinkle"  v ( t i1 ng k l ) ((pos "J2%" "Ki%" ) (subcat "2A" "2C" )))
( "tinkled"  v ( t i1 ng k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "tinkles"  v ( t i1 ng k l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "tinkling"  v ( t i1 ng k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "tinned"  v ( t i1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tinnier"  j ( t i1 n i@ r ) ((pos "Or%" ) ))
( "tinniest"  j ( t i1 n i i s t ) ((pos "Os%" ) ))
( "tinning"  v ( t i1 n i ng ) ((pos "Hb$" ) (subcat )))
( "tinny"  j ( t i1 n ii ) ((pos "OD%" ) ))
( "tins"  v ( t i1 n z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "tinsel"  v ( t i1 n s l ) ((pos "H4$" "L@%" ) ))
( "tinselled"  v ( t i1 n s l d ) ((pos "Hc$" "Hd$" ) ))
( "tinselling"  v ( t i1 n s @ l i ng ) ((pos "Hb$" ) ))
( "tinselly"  j ( t i1 n s @ l ii ) ((pos "OA%" ) ))
( "tinsels"  v ( t i1 n s l z ) ((pos "Ha$" ) ))
( "tinsmith"  n ( t i1 n s m i th ) ((pos "K6%" ) ))
( "tinsmiths"  n ( t i1 n s m i th s ) ((pos "Kj%" ) ))
( "tint"  v ( t i1 n t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "tintack"  n ( t i1 n t a k ) ((pos "K6%" ) ))
( "tintacks"  n ( t i1 n t a k s ) ((pos "Kj%" ) ))
( "tinted"  v ( t i1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tinting"  v ( t i1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tintinnabulation"  n ( t i2 n t i n a2 b y u l ei1 sh @ n ) ((pos "L@$" ) ))
( "tints"  v ( t i1 n t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "tiny"  j ( t ai1 n ii ) ((pos "OD%" ) ))
( "tip"  v ( t i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "6A" "12C" "15A" )))
( "tip-and-run"  j ( t i2 p - @ n - r uh1 n ) ((pos "OA$" ) ))
( "tip-off"  n ( t i1 p - o f ) ((pos "K6%" ) ))
( "tip-offs"  n ( t i1 p - o f s ) ((pos "Kj%" ) ))
( "tip-top"  j ( t i1 p - t o p ) ((pos "OA%" "Pu$" ) ))
( "tip-up"  j ( t i1 p - uh p ) ((pos "OA%" ) ))
( "tipped"  v ( t i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "12C" "15A" )))
( "tippet"  n ( t i1 p i t ) ((pos "K6$" ) ))
( "tippets"  n ( t i1 p i t s ) ((pos "Kj$" ) ))
( "tipping"  v ( t i1 p i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "12C" "15A" )))
( "tipple"  v ( t i1 p l ) ((pos "J2%" "L@%" ) (subcat "2A" )))
( "tippled"  v ( t i1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tippler"  n ( t i1 p l @ r ) ((pos "K6%" ) ))
( "tipplers"  n ( t i1 p l @ z ) ((pos "Kj%" ) ))
( "tipples"  v ( t i1 p l z ) ((pos "Ja%" ) (subcat "2A" )))
( "tippling"  v ( t i1 p l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tips"  v ( t i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "12C" "15A" )))
( "tipstaff"  n ( t i1 p s t aa f ) ((pos "K6$" ) ))
( "tipstaffs"  n ( t i1 p s t aa f s ) ((pos "Kj$" ) ))
( "tipster"  n ( t i1 p s t @ r ) ((pos "K6$" ) ))
( "tipsters"  n ( t i1 p s t @ z ) ((pos "Kj$" ) ))
( "tipsy"  j ( t i1 p s ii ) ((pos "OA%" ) ))
( "tiptoe"  v ( t i1 p t ou ) ((pos "I5%" "Pu%" ) (subcat "2A" )))
( "tiptoed"  v ( t i1 p t ou d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "tiptoeing"  v ( t i1 p t ou i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "tiptoes"  v ( t i1 p t ou z ) ((pos "Ia%" ) (subcat "2A" )))
( "tirade"  n ( t ai1 r ei1 d ) ((pos "K6%" ) ))
( "tirades"  n ( t ai1 r ei1 d z ) ((pos "Kj%" ) ))
( "tire"  v ( t ai1 @ r ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "tired"  v ( t ai1 @ d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "6A" )))
( "tiredness"  n ( t ai1 @ d n @ s ) ((pos "L@%" ) ))
( "tireless"  j ( t ai1 @ l @ s ) ((pos "OA%" ) ))
( "tirelessly"  a ( t ai1 @ l @ s l ii ) ((pos "Pu%" ) ))
( "tires"  v ( t ai1 @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "tiresome"  j ( t ai1 @ s @ m ) ((pos "OA%" ) ))
( "tiresomely"  a ( t ai1 @ s @ m l ii ) ((pos "Pu%" ) ))
( "tiring"  v ( t ai1 @ r i ng ) ((pos "Jb%" "Pu%" ) (subcat "2A" "3A" "6A" )))
( "tiro"  n ( t ai1 @ r ou ) ((pos "K6$" ) ))
( "tiros"  n ( t ai1 @ r ou z ) ((pos "Kj$" ) ))
( "tissue"  n ( t i1 sh uu ) ((pos "M6%" ) ))
( "tissues"  n ( t i1 sh uu z ) ((pos "Mj%" ) ))
( "tit"  n ( t i1 t ) ((pos "K6%" ) ))
( "titan"  n ( t ai1 t n ) ((pos "K6$" ) ))
( "titanic"  j ( t ai1 t a1 n i k ) ((pos "OA$" ) ))
( "titans"  n ( t ai1 t n z ) ((pos "Kj$" ) ))
( "titbit"  n ( t i1 t b i t ) ((pos "K6%" ) ))
( "titbits"  n ( t i1 t b i t s ) ((pos "Kj%" ) ))
( "tithe"  n ( t ai1 dh ) ((pos "K6$" ) ))
( "tithe-barn"  n ( t ai1 dh - b aa n ) ((pos "K6$" ) ))
( "tithe-barns"  n ( t ai1 dh - b aa n z ) ((pos "Kj$" ) ))
( "tithes"  n ( t ai1 dh z ) ((pos "Kj$" ) ))
( "titillate"  v ( t i1 t i l ei t ) ((pos "H2%" ) (subcat )))
( "titillated"  v ( t i1 t i l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "titillates"  v ( t i1 t i l ei t s ) ((pos "Ha%" ) (subcat )))
( "titillating"  v ( t i1 t i l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "titillation"  n ( t i2 t i l ei1 sh @ n ) ((pos "K6%" ) ))
( "titillations"  n ( t i2 t i l ei1 sh @ n z ) ((pos "Kj%" ) ))
( "titivate"  v ( t i1 t i v ei t ) ((pos "J2%" ) (subcat "2A" )))
( "titivated"  v ( t i1 t i v ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "titivates"  v ( t i1 t i v ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "titivating"  v ( t i1 t i v ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "titlark"  n ( t i1 t l aa k ) ((pos "K6$" ) ))
( "titlarks"  n ( t i1 t l aa k s ) ((pos "Kj$" ) ))
( "title"  n ( t ai1 t l ) ((pos "M6%" ) ))
( "title-deed"  n ( t ai1 t l - d ii d ) ((pos "K6%" ) ))
( "title-deeds"  n ( t ai1 t l - d ii d z ) ((pos "Kj%" ) ))
( "title-page"  n ( t ai1 t l - p ei jh ) ((pos "K6%" ) ))
( "title-pages"  n ( t ai1 t l - p ei jh i z ) ((pos "Kj%" ) ))
( "title-role"  n ( t ai1 t l - r ou l ) ((pos "K6%" ) ))
( "title-roles"  n ( t ai1 t l - r ou l z ) ((pos "Kj%" ) ))
( "titled"  j ( t ai1 t l d ) ((pos "OA%" ) ))
( "titles"  n ( t ai1 t l z ) ((pos "Mj%" ) ))
( "titmice"  n ( t i1 t m ai s ) ((pos "Kj$" ) ))
( "titmouse"  n ( t i1 t m au s ) ((pos "Ki$" ) ))
( "tits"  n ( t i1 t s ) ((pos "Kj%" ) ))
( "titter"  v ( t i1 t @ r ) ((pos "I0%" ) (subcat )))
( "tittered"  v ( t i1 t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "tittering"  v ( t i1 t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "titters"  v ( t i1 t @ z ) ((pos "Ia%" ) (subcat )))
( "tittivate"  v ( t i1 t i v ei t ) ((pos "J2%" ) (subcat "2A" )))
( "tittivated"  v ( t i1 t i v ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tittivates"  v ( t i1 t i v ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "tittivating"  v ( t i1 t i v ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tittle"  n ( t i1 t l ) ((pos "Ki%" ) ))
( "tittle-tattle"  v ( t i1 t l - t a t l ) ((pos "I2%" "L@%" ) ))
( "tittle-tattled"  v ( t i1 t l - t a t l d ) ((pos "Ic%" "Id%" ) ))
( "tittle-tattles"  v ( t i1 t l - t a t l z ) ((pos "Ia%" ) ))
( "tittle-tattling"  v ( t i1 t l - t a t l i ng ) ((pos "Ib%" ) ))
( "titular"  j ( t i1 t y u l @ r ) ((pos "OA%" ) ))
( "tizzies"  n ( t i1 z i z ) ((pos "Kj$" ) ))
( "tizzy"  n ( t i1 z ii ) ((pos "K8%" ) ))
( "to"  in ( t @ ) ((pos "T-*" "X-*" ) ))
( "to"  a ( t uu1 ) ((pos "Pu%" ) ))
( "to-do"  n ( t @1 - d uu1 ) ((pos "K6%" ) ))
( "to-dos"  n ( t @1 - d uu1 z ) ((pos "Kj$" ) ))
( "toad"  n ( t ou1 d ) ((pos "K6%" ) ))
( "toad-in-the-hole"  n ( t ou2 d - i n - dh @ - h ou1 l ) ((pos "Mi%" ) ))
( "toadied"  v ( t ou1 d i d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "toadies"  v ( t ou1 d i z ) ((pos "Ia$" "Kj$" ) (subcat "2A" )))
( "toads"  n ( t ou1 d z ) ((pos "Kj%" ) ))
( "toadstool"  n ( t ou1 d s t uu l ) ((pos "K6%" ) ))
( "toadstools"  n ( t ou1 d s t uu l z ) ((pos "Kj%" ) ))
( "toady"  v ( t ou1 d ii ) ((pos "I3$" "K8$" ) (subcat "2A" )))
( "toadying"  v ( t ou1 d i i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "toast"  v ( t ou1 s t ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "toasted"  v ( t ou1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "toaster"  n ( t ou1 s t @ r ) ((pos "K6%" ) ))
( "toasters"  n ( t ou1 s t @ z ) ((pos "Kj%" ) ))
( "toasting"  v ( t ou1 s t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "toasting-fork"  n ( t ou1 s t i ng - f oo k ) ((pos "K6%" ) ))
( "toasting-forks"  n ( t ou1 s t i ng - f oo k s ) ((pos "Kj%" ) ))
( "toastmaster"  n ( t ou1 s t m aa s t @ r ) ((pos "K6%" ) ))
( "toastmasters"  n ( t ou1 s t m aa s t @ z ) ((pos "Kj%" ) ))
( "toastrack"  n ( t ou1 s t r a k ) ((pos "K6%" ) ))
( "toastracks"  n ( t ou1 s t r a k s ) ((pos "Kj%" ) ))
( "toasts"  v ( t ou1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "tobacco"  n ( t @1 b a1 k ou ) ((pos "M6%" ) ))
( "tobacconist"  n ( t @1 b a1 k @ n i s t ) ((pos "K6%" ) ))
( "tobacconists"  n ( t @1 b a1 k @ n i s t s ) ((pos "Kj%" ) ))
( "tobaccos"  n ( t @1 b a1 k ou z ) ((pos "Mj%" ) ))
( "toboggan"  v ( t @1 b o1 g @ n ) ((pos "I0$" "K6%" ) (subcat "2A" )))
( "tobogganed"  v ( t @1 b o1 g @ n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "tobogganing"  v ( t @1 b o1 g @ n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "toboggans"  v ( t @1 b o1 g @ n z ) ((pos "Ia$" "Kj%" ) (subcat "2A" )))
( "toby-jug"  n ( t ou1 b i - jh uh g ) ((pos "K6%" ) ))
( "toby-jugs"  n ( t ou1 b i - jh uh g z ) ((pos "Kj%" ) ))
( "toccata"  n ( t @1 k aa1 t @ ) ((pos "K6$" ) ))
( "toccatas"  n ( t @1 k aa1 t @ z ) ((pos "Kj$" ) ))
( "tocsin"  n ( t o1 k s i n ) ((pos "K6$" ) ))
( "tocsins"  n ( t o1 k s i n z ) ((pos "Kj$" ) ))
( "today"  n ( t @1 d ei1 ) ((pos "L@*" "Pu*" ) ))
( "toddies"  n ( t o1 d i z ) ((pos "Mj$" ) ))
( "toddle"  v ( t o1 d l ) ((pos "I2%" ) (subcat "2A" )))
( "toddled"  v ( t o1 d l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "toddler"  n ( t o1 d l @ r ) ((pos "K6%" ) ))
( "toddlers"  n ( t o1 d l @ z ) ((pos "Kj%" ) ))
( "toddles"  v ( t o1 d l z ) ((pos "Ia%" ) (subcat "2A" )))
( "toddling"  v ( t o1 d l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "toddy"  n ( t o1 d ii ) ((pos "M8%" ) ))
( "toe"  v ( t ou1 ) ((pos "H5%" "K6%" ) (subcat )))
( "toecap"  n ( t ou1 k a p ) ((pos "K6%" ) ))
( "toecaps"  n ( t ou1 k a p s ) ((pos "Kj%" ) ))
( "toed"  v ( t ou1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "toehold"  n ( t ou1 h ou l d ) ((pos "K6%" ) ))
( "toeholds"  n ( t ou1 h ou l d z ) ((pos "Kj%" ) ))
( "toeing"  v ( t ou1 i ng ) ((pos "Hb%" ) (subcat )))
( "toenail"  n ( t ou1 n ei l ) ((pos "K6%" ) ))
( "toenails"  n ( t ou1 n ei l z ) ((pos "Kj%" ) ))
( "toes"  v ( t ou1 z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "toff"  n ( t o1 f ) ((pos "K6%" ) ))
( "toffee"  n ( t o1 f ii ) ((pos "M6%" ) ))
( "toffees"  n ( t o1 f i z ) ((pos "Mj%" ) ))
( "toffs"  n ( t o1 f s ) ((pos "Kj%" ) ))
( "tog"  v ( t o1 g ) ((pos "H4%" ) (subcat )))
( "toga"  n ( t ou1 g @ ) ((pos "K6%" ) ))
( "togas"  n ( t ou1 g @ z ) ((pos "Kj%" ) ))
( "together"  a ( t @1 g e1 dh @ r ) ((pos "Pu*" ) ))
( "togetherness"  n ( t @1 g e1 dh @ n @ s ) ((pos "L@%" ) ))
( "togged"  v ( t o1 g d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "togging"  v ( t o1 g i ng ) ((pos "Hb%" ) (subcat )))
( "toggle"  n ( t o1 g l ) ((pos "K6%" ) ))
( "toggles"  n ( t o1 g l z ) ((pos "Kj%" ) ))
( "togs"  v ( t o1 g z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "toil"  v ( t oi1 l ) ((pos "I0%" "M6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "toiled"  v ( t oi1 l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" "3A" )))
( "toiler"  n ( t oi1 l @ r ) ((pos "K6%" ) ))
( "toilers"  n ( t oi1 l @ z ) ((pos "Kj%" ) ))
( "toilet"  n ( t oi1 l i t ) ((pos "K6%" ) ))
( "toilet-paper"  n ( t oi1 l i t - p ei p @ r ) ((pos "K6%" ) ))
( "toilet-papers"  n ( t oi1 l i t - p ei p @ z ) ((pos "Kj$" ) ))
( "toilet-powder"  n ( t oi1 l i t - p au d @ r ) ((pos "L@$" ) ))
( "toilet-roll"  n ( t oi1 l i t - r ou l ) ((pos "K6%" ) ))
( "toilet-rolls"  n ( t oi1 l i t - r ou l z ) ((pos "Kj%" ) ))
( "toilet-table"  n ( t oi1 l i t - t ei b l ) ((pos "K6$" ) ))
( "toilet-tables"  n ( t oi1 l i t - t ei b l z ) ((pos "Kj$" ) ))
( "toilets"  n ( t oi1 l i t s ) ((pos "Kj%" ) ))
( "toiling"  v ( t oi1 l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" "3A" )))
( "toils"  v ( t oi1 l z ) ((pos "Ia%" "Mj$" ) (subcat "2A" "2B" "2C" "3A" )))
( "toilsome"  j ( t oi1 l s @ m ) ((pos "OA$" ) ))
( "token"  n ( t ou1 k @ n ) ((pos "K6%" ) ))
( "tokens"  n ( t ou1 k @ n z ) ((pos "Kj%" ) ))
( "told"  v ( t ou1 l d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "3A" "6A" "8" "10" "11" "12A" "13A" "14" "15A" "15B" "17" "20" )))
( "tolerable"  j ( t o1 l @ r @ b l ) ((pos "OA%" ) ))
( "tolerably"  a ( t o1 l @ r @ b l ii ) ((pos "Pu%" ) ))
( "tolerance"  n ( t o1 l @ r @ n s ) ((pos "M6%" ) ))
( "tolerances"  n ( t o1 l @ r @ n s i z ) ((pos "Mj%" ) ))
( "tolerant"  j ( t o1 l @ r @ n t ) ((pos "OA%" ) ))
( "tolerantly"  a ( t o1 l @ r @ n t l ii ) ((pos "Pu%" ) ))
( "tolerate"  v ( t o1 l @ r ei t ) ((pos "H2%" ) (subcat "6A" )))
( "tolerated"  v ( t o1 l @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tolerates"  v ( t o1 l @ r ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "tolerating"  v ( t o1 l @ r ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "toleration"  n ( t o2 l @ r ei1 sh @ n ) ((pos "L@$" ) ))
( "toll"  v ( t ou1 l ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "tollbar"  n ( t ou1 l b aa r ) ((pos "K6$" ) ))
( "tollbars"  n ( t ou1 l b aa z ) ((pos "Kj$" ) ))
( "tollbooth"  n ( t ou1 l b uu dh ) ((pos "K6%" ) ))
( "tollbooths"  n ( t ou1 l b uu dh z ) ((pos "Kj%" ) ))
( "tolled"  v ( t ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "tollgate"  n ( t ou1 l g ei t ) ((pos "K6%" ) ))
( "tollgates"  n ( t ou1 l g ei t s ) ((pos "Kj%" ) ))
( "tollhouse"  n ( t ou1 l h au s ) ((pos "K6$" ) ))
( "tollhouses"  n ( t ou1 l h au z i z ) ((pos "Kj$" ) ))
( "tolling"  v ( t ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tolls"  v ( t ou1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "tomahawk"  v ( t o1 m @ h oo k ) ((pos "H0$" "K6%" ) ))
( "tomahawked"  v ( t o1 m @ h oo k t ) ((pos "Hc$" "Hd$" ) ))
( "tomahawking"  v ( t o1 m @ h oo k i ng ) ((pos "Hb$" ) ))
( "tomahawks"  v ( t o1 m @ h oo k s ) ((pos "Ha$" "Kj%" ) ))
( "tomato"  n ( t @1 m aa1 t ou ) ((pos "K7%" ) ))
( "tomatoes"  n ( t @1 m aa1 t ou z ) ((pos "Kj%" ) ))
( "tomb"  n ( t uu1 m ) ((pos "K6%" ) ))
( "tombola"  n ( t o1 m b ou1 l @ ) ((pos "K6%" ) ))
( "tombolas"  n ( t o1 m b ou1 l @ z ) ((pos "Kj$" ) ))
( "tomboy"  n ( t o1 m b oi ) ((pos "K6%" ) ))
( "tomboys"  n ( t o1 m b oi z ) ((pos "Kj%" ) ))
( "tombs"  n ( t uu1 m z ) ((pos "Kj%" ) ))
( "tombstone"  n ( t uu1 m s t ou n ) ((pos "K6%" ) ))
( "tombstones"  n ( t uu1 m s t ou n z ) ((pos "Kj%" ) ))
( "tomcat"  n ( t o1 m k a t ) ((pos "K6%" ) ))
( "tomcats"  n ( t o1 m k a t s ) ((pos "Kj%" ) ))
( "tome"  n ( t ou1 m ) ((pos "K6%" ) ))
( "tomes"  n ( t ou1 m z ) ((pos "Kj%" ) ))
( "tomfool"  n ( t o2 m f uu1 l ) ((pos "K6$" ) ))
( "tomfooleries"  n ( t o1 m f uu1 l @ r i z ) ((pos "Mj%" ) ))
( "tomfoolery"  n ( t o1 m f uu1 l @ r ii ) ((pos "M8%" ) ))
( "tomfools"  n ( t o2 m f uu1 l z ) ((pos "Kj$" ) ))
( "tommy-gun"  n ( t o1 m i - g uh n ) ((pos "K6%" ) ))
( "tommy-guns"  n ( t o1 m i - g uh n z ) ((pos "Kj%" ) ))
( "tommy-rot"  n ( t o1 m i - r o t ) ((pos "L@%" ) ))
( "tomorrow"  n ( t @1 m o1 r ou ) ((pos "M6%" "Pu%" ) ))
( "tomorrows"  n ( t @1 m o1 r ou z ) ((pos "Mj%" ) ))
( "tomtit"  n ( t o1 m t i t ) ((pos "K6%" ) ))
( "tomtits"  n ( t o1 m t i t s ) ((pos "Kj%" ) ))
( "tomtom"  n ( t o1 m t o m ) ((pos "K6%" ) ))
( "tomtoms"  n ( t o1 m t o m z ) ((pos "Kj%" ) ))
( "ton"  v ( t uh1 n ) ((pos "I4$" "K6%" ) (subcat )))
( "tonal"  j ( t ou1 n l ) ((pos "OA%" ) ))
( "tonalities"  n ( t ou1 n a1 l i t i z ) ((pos "Mj%" ) ))
( "tonality"  n ( t ou1 n a1 l i t ii ) ((pos "M8%" ) ))
( "tone"  v ( t ou1 n ) ((pos "J2%" "M6%" ) (subcat "2C" "6A" )))
( "tone-deaf"  j ( t ou1 n - d e1 f ) ((pos "OA%" ) ))
( "tone-poem"  n ( t ou1 n - p ou i m ) ((pos "K6%" ) ))
( "tone-poems"  n ( t ou1 n - p ou i m z ) ((pos "Kj%" ) ))
( "toned"  v ( t ou1 n d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2C" "6A" )))
( "toneless"  j ( t ou1 n l @ s ) ((pos "OA%" ) ))
( "tonelessly"  a ( t ou1 n l @ s l ii ) ((pos "Pu%" ) ))
( "tones"  v ( t ou1 n z ) ((pos "Ja$" "Mj%" ) (subcat "2C" "6A" )))
( "tongs"  n ( t o1 ng z ) ((pos "Kj%" ) ))
( "tongue"  n ( t uh1 ng ) ((pos "M6%" ) ))
( "tongue-in-cheek"  j ( t uh2 ng - i n - ch ii1 k ) ((pos "OA%" "Pu%" ) ))
( "tongue-tied"  j ( t uh1 n - t ai d ) ((pos "OA%" ) ))
( "tongue-twister"  n ( t uh1 n - t w i s t @ r ) ((pos "K6%" ) ))
( "tongue-twisters"  n ( t uh1 n - t w i s t @ z ) ((pos "Kj%" ) ))
( "tongued"  j ( t uh1 ng d ) ((pos "Ot%" ) ))
( "tongues"  n ( t uh1 ng z ) ((pos "Mj%" ) ))
( "tonic"  n ( t o1 n i k ) ((pos "K6%" "OA%" ) ))
( "tonic sol-fa"  n ( t o2 n i k - s o1 l - f aa ) ((pos "L@%" ) ))
( "tonics"  n ( t o1 n i k s ) ((pos "Kj$" ) ))
( "tonight"  n ( t @1 n ai1 t ) ((pos "L@%" "Pu%" ) ))
( "toning"  v ( t ou1 n i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "tonnage"  n ( t uh1 n i jh ) ((pos "K6%" ) ))
( "tonnages"  n ( t uh1 n i jh i z ) ((pos "Kj$" ) ))
( "tonne"  n ( t uh1 n ) ((pos "K6%" ) ))
( "tonned"  v ( t uh1 n d ) ((pos "Ic$" "Id$" ) (subcat )))
( "tonnes"  n ( t uh1 n z ) ((pos "Kj%" ) ))
( "tonning"  v ( t uh1 n i ng ) ((pos "Ib$" ) (subcat )))
( "tons"  v ( t uh1 n z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "tonsil"  n ( t o1 n s i l ) ((pos "K6%" ) ))
( "tonsillitis"  n ( t o2 n s i l ai1 t i s ) ((pos "L@%" ) ))
( "tonsils"  n ( t o1 n s i l z ) ((pos "Kj%" ) ))
( "tonsorial"  j ( t o1 n s oo1 r i@ l ) ((pos "OA$" ) ))
( "tonsure"  v ( t o1 n sh @ r ) ((pos "H2$" "K6$" ) ))
( "tonsured"  v ( t o1 n sh @ d ) ((pos "Hc$" "Hd$" ) ))
( "tonsures"  v ( t o1 n sh @ z ) ((pos "Ha$" "Kj$" ) ))
( "tonsuring"  v ( t o1 n sh @ r i ng ) ((pos "Hb$" ) ))
( "tontine"  n ( t o1 n t ii n ) ((pos "K6$" ) ))
( "tontines"  n ( t o1 n t ii n z ) ((pos "Kj$" ) ))
( "too"  a ( t uu1 ) ((pos "Pu*" ) ))
( "took"  v ( t u1 k ) ((pos "Jc*" ) (subcat "2A" "2B" "2C" "3A" "6A" "6B" "12A" "13A" "14" "15A" "15B" "16B" "19B" )))
( "tool"  v ( t uu1 l ) ((pos "H0$" "K6%" ) (subcat "2C" )))
( "tooled"  v ( t uu1 l d ) ((pos "Hc$" "Hd$" ) (subcat "2C" )))
( "tooling"  v ( t uu1 l i ng ) ((pos "Hb$" ) (subcat "2C" )))
( "tools"  v ( t uu1 l z ) ((pos "Ha$" "Kj%" ) (subcat "2C" )))
( "toot"  v ( t uu1 t ) ((pos "J0$" "K6$" ) (subcat "2A" )))
( "tooted"  v ( t uu1 t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "tooth"  n ( t uu1 th ) ((pos "Ki%" ) ))
( "toothache"  n ( t uu1 th ei k ) ((pos "M@%" ) ))
( "toothbrush"  n ( t uu1 th b r uh sh ) ((pos "K7%" ) ))
( "toothbrushes"  n ( t uu1 th b r uh sh i z ) ((pos "Kj%" ) ))
( "toothed"  j ( t uu1 th t ) ((pos "Ot%" ) ))
( "toothless"  j ( t uu1 th l @ s ) ((pos "OA%" ) ))
( "toothpaste"  n ( t uu1 th p ei s t ) ((pos "M6%" ) ))
( "toothpastes"  n ( t uu1 th p ei s t s ) ((pos "Mj%" ) ))
( "toothpick"  n ( t uu1 th p i k ) ((pos "K6%" ) ))
( "toothpicks"  n ( t uu1 th p i k s ) ((pos "Kj%" ) ))
( "toothpowder"  n ( t uu1 th p au d @ r ) ((pos "L@%" ) ))
( "toothsome"  j ( t uu1 th s @ m ) ((pos "OA$" ) ))
( "tooting"  v ( t uu1 t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "tootle"  v ( t uu1 t l ) ((pos "I2%" "K6%" ) ))
( "tootled"  v ( t uu1 t l d ) ((pos "Ic%" "Id%" ) ))
( "tootles"  v ( t uu1 t l z ) ((pos "Ia%" "Kj%" ) ))
( "tootling"  v ( t uu1 t l i ng ) ((pos "Ib%" ) ))
( "toots"  v ( t uu1 t s ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "top"  v ( t o1 p ) ((pos "H4%" "K6*" ) (subcat )))
( "top-boot"  n ( t o1 p - b uu t ) ((pos "K6$" ) ))
( "top-boots"  n ( t o1 p - b uu t s ) ((pos "Kj$" ) ))
( "top-dress"  v ( t o1 p - d r e s ) ((pos "H1$" ) ))
( "top-dressed"  v ( t o1 p - d r e s t ) ((pos "Hc$" "Hd$" ) ))
( "top-dresses"  v ( t o1 p - d r e s i z ) ((pos "Ha$" ) ))
( "top-dressing"  v ( t o1 p - d r e s i ng ) ((pos "Hb$" "M6%" ) ))
( "top-dressings"  n ( t o1 p - d r e s i ng z ) ((pos "Mj$" ) ))
( "top-flight"  j ( t o1 p - f l ai t ) ((pos "Oq%" ) ))
( "top-heavy"  j ( t o1 p - h e1 v ii ) ((pos "OA%" ) ))
( "top-hole"  j ( t o1 p - h ou1 l ) ((pos "OA%" ) ))
( "top-ranking"  j ( t o1 p - r a1 n k i ng ) ((pos "OA%" ) ))
( "topaz"  n ( t ou1 p a z ) ((pos "M7%" ) ))
( "topazes"  n ( t ou1 p a z i z ) ((pos "Mj$" ) ))
( "topcoat"  n ( t o1 p k ou t ) ((pos "K6%" ) ))
( "topcoats"  n ( t o1 p k ou t s ) ((pos "Kj%" ) ))
( "tope"  v ( t ou1 p ) ((pos "J2$" ) (subcat "2A" )))
( "toped"  v ( t ou1 p t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "toper"  n ( t ou1 p @ r ) ((pos "K6$" ) ))
( "topers"  n ( t ou1 p @ z ) ((pos "Kj$" ) ))
( "topes"  v ( t ou1 p s ) ((pos "Ja$" ) (subcat "2A" )))
( "topgallant"  n ( t o1 p g a l @ n t ) ((pos "K6$" "OA$" ) ))
( "topgallants"  n ( t o1 p g a l @ n t s ) ((pos "Kj$" ) ))
( "topi"  n ( t ou1 p ii ) ((pos "K6$" ) ))
( "topiary"  n ( t ou1 p i@ r ii ) ((pos "L@$" ) ))
( "topic"  n ( t o1 p i k ) ((pos "K6%" ) ))
( "topical"  j ( t o1 p i k l ) ((pos "OA%" ) ))
( "topically"  a ( t o1 p i k l ii ) ((pos "Pu%" ) ))
( "topics"  n ( t o1 p i k s ) ((pos "Kj%" ) ))
( "toping"  v ( t ou1 p i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "topis"  n ( t ou1 p i z ) ((pos "Kj$" ) ))
( "topknot"  n ( t o1 p n o t ) ((pos "K6%" ) ))
( "topknots"  n ( t o1 p n o t s ) ((pos "Kj%" ) ))
( "topless"  j ( t o1 p l @ s ) ((pos "OA%" ) ))
( "topmast"  n ( t o1 p m aa s t ) ((pos "K6$" ) ))
( "topmasts"  n ( t o1 p m aa s t s ) ((pos "Kj$" ) ))
( "topmost"  j ( t o1 p m ou s t ) ((pos "OA%" ) ))
( "topnotch"  j ( t o1 p n o ch ) ((pos "Oq%" ) ))
( "topographical"  j ( t o2 p @ g r a1 f i k l ) ((pos "OA%" ) ))
( "topographically"  a ( t o2 p @ g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "topography"  n ( t @1 p o1 g r @ f ii ) ((pos "L@%" ) ))
( "topped"  v ( t o1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "topper"  n ( t o1 p @ r ) ((pos "K6%" ) ))
( "toppers"  n ( t o1 p @ z ) ((pos "Kj%" ) ))
( "topping"  v ( t o1 p i ng ) ((pos "Hb%" "OA%" ) (subcat )))
( "toppingly"  a ( t o1 p i ng l ii ) ((pos "Pu$" ) ))
( "topple"  v ( t o1 p l ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "toppled"  v ( t o1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "topples"  v ( t o1 p l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "toppling"  v ( t o1 p l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "tops"  v ( t o1 p s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "topsail"  n ( t o1 p s ei l ) ((pos "K6%" ) ))
( "topsails"  n ( t o1 p s ei l z ) ((pos "Kj%" ) ))
( "topsy-turvy"  j ( t o2 p s i - t @@1 v ii ) ((pos "OA%" "Pu%" ) ))
( "topsy-turvydom"  n ( t o2 p s i - t @@1 v i d @ m ) ((pos "K6$" ) ))
( "topsy-turvydoms"  n ( t o2 p s i - t @@1 v i d @ m z ) ((pos "Kj$" ) ))
( "toque"  n ( t ou1 k ) ((pos "K6$" ) ))
( "toques"  n ( t ou1 k s ) ((pos "Kj$" ) ))
( "tor"  n ( t oo1 r ) ((pos "K6$" ) ))
( "torch"  n ( t oo1 ch ) ((pos "K7%" ) ))
( "torch-race"  n ( t oo1 ch - r ei s ) ((pos "K6$" ) ))
( "torch-races"  n ( t oo1 ch - r ei s i z ) ((pos "Kj$" ) ))
( "torch-singer"  n ( t oo1 ch - s i ng @ r ) ((pos "K6$" ) ))
( "torch-singers"  n ( t oo1 ch - s i ng @ z ) ((pos "Kj$" ) ))
( "torches"  n ( t oo1 ch i z ) ((pos "Kj%" ) ))
( "torchlight"  n ( t oo1 ch l ai t ) ((pos "L@%" ) ))
( "tore"  v ( t oo1 r ) ((pos "Jc%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "toreador"  n ( t o1 r i@ d oo r ) ((pos "K6%" ) ))
( "toreadors"  n ( t o1 r i@ d oo z ) ((pos "Kj%" ) ))
( "torment"  n ( t oo1 m e n t ) ((pos "M6%" ) ))
( "torment"  v ( t oo1 m e1 n t ) ((pos "H0%" ) (subcat "6A" )))
( "tormented"  v ( t oo1 m e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "tormenting"  v ( t oo1 m e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tormentor"  n ( t oo1 m e1 n t @ r ) ((pos "K6%" ) ))
( "tormentors"  n ( t oo1 m e1 n t @ z ) ((pos "Kj%" ) ))
( "torments"  n ( t oo1 m e n t s ) ((pos "Mj%" ) ))
( "torments"  v ( t oo1 m e1 n t s ) ((pos "Ha%" ) (subcat "6A" )))
( "torn"  v ( t oo1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" "15B" )))
( "tornado"  n ( t oo1 n ei1 d ou ) ((pos "K7%" ) ))
( "tornadoes"  n ( t oo1 n ei1 d ou z ) ((pos "Kj%" ) ))
( "torpedo"  v ( t oo1 p ii1 d ou ) ((pos "H0%" "K7%" ) (subcat )))
( "torpedo-boat"  n ( t oo1 p ii1 d ou - b ou t ) ((pos "K6%" ) ))
( "torpedo-boats"  n ( t oo1 p ii1 d ou - b ou t s ) ((pos "Kj%" ) ))
( "torpedo-tube"  n ( t oo1 p ii1 d ou - t y uu b ) ((pos "K6$" ) ))
( "torpedo-tubes"  n ( t oo1 p ii1 d ou - t y uu b z ) ((pos "Kj$" ) ))
( "torpedoed"  v ( t oo1 p ii1 d ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "torpedoes"  n ( t oo1 p ii1 d ou z ) ((pos "Kj%" ) ))
( "torpedoing"  v ( t oo1 p ii1 d ou i ng ) ((pos "Hb%" ) (subcat )))
( "torpedos"  v ( t oo1 p ii1 d ou z ) ((pos "Ha%" ) (subcat )))
( "torpid"  j ( t oo1 p i d ) ((pos "OA$" ) ))
( "torpidity"  n ( t oo1 p i1 d i t ii ) ((pos "L@$" ) ))
( "torpidly"  a ( t oo1 p i d l ii ) ((pos "Pu$" ) ))
( "torpidness"  n ( t oo1 p i d n @ s ) ((pos "L@$" ) ))
( "torpor"  n ( t oo1 p @ r ) ((pos "M6$" ) ))
( "torpors"  n ( t oo1 p @ z ) ((pos "Mj$" ) ))
( "torque"  n ( t oo1 k ) ((pos "M6$" ) ))
( "torques"  n ( t oo1 k s ) ((pos "Mj$" ) ))
( "torrent"  n ( t o1 r @ n t ) ((pos "K6%" ) ))
( "torrential"  j ( t @1 r e1 n sh l ) ((pos "OA%" ) ))
( "torrents"  n ( t o1 r @ n t s ) ((pos "Kj%" ) ))
( "torrid"  j ( t o1 r i d ) ((pos "OA%" ) ))
( "torridity"  n ( t o1 r i d i t ii ) ((pos "L@$" ) ))
( "tors"  n ( t oo1 z ) ((pos "Kj$" ) ))
( "torsion"  n ( t oo1 sh @ n ) ((pos "L@$" ) ))
( "torso"  n ( t oo1 s ou ) ((pos "K6%" ) ))
( "torsos"  n ( t oo1 s ou z ) ((pos "Kj%" ) ))
( "tort"  n ( t oo1 t ) ((pos "K6$" ) ))
( "tortilla"  n ( t oo1 t ii1 y @ ) ((pos "K6$" ) ))
( "tortillas"  n ( t oo1 t ii1 y @ z ) ((pos "Kj$" ) ))
( "tortoise"  n ( t oo1 t @ s ) ((pos "K6%" ) ))
( "tortoises"  n ( t oo1 t @ s i z ) ((pos "Kj%" ) ))
( "tortoiseshell"  n ( t oo1 t @ s - sh e l ) ((pos "L@%" ) ))
( "torts"  n ( t oo1 t s ) ((pos "Kj$" ) ))
( "tortuous"  j ( t oo1 ch u@ s ) ((pos "OA%" ) ))
( "tortuously"  a ( t oo1 ch u@ s l ii ) ((pos "Pu%" ) ))
( "torture"  v ( t oo1 ch @ r ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "tortured"  v ( t oo1 ch @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "torturer"  n ( t oo1 ch @ r @ r ) ((pos "K6%" ) ))
( "torturers"  n ( t oo1 ch @ r @ z ) ((pos "Kj%" ) ))
( "tortures"  v ( t oo1 ch @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "torturing"  v ( t oo1 ch @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "tosh"  n ( t o1 sh ) ((pos "K7$" ) ))
( "toshes"  n ( t o1 sh i z ) ((pos "Kj$" ) ))
( "toss"  v ( t o1 s ) ((pos "J1%" "K7%" ) (subcat "2C" "6A" "12A" "13A" "15A" )))
( "toss-up"  n ( t o1 s - uh p ) ((pos "K6%" ) ))
( "toss-ups"  n ( t o1 s - uh p s ) ((pos "Kj$" ) ))
( "tossed"  v ( t o1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "12A" "13A" "15A" )))
( "tosses"  v ( t o1 s i z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "12A" "13A" "15A" )))
( "tossing"  v ( t o1 s i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "12A" "13A" "15A" )))
( "tot"  v ( t o1 t ) ((pos "J4%" "K6%" ) (subcat "2C" )))
( "total"  v ( t ou1 t l ) ((pos "J4%" "K6%" "OA%" ) (subcat "2C" )))
( "totalitarian"  j ( t ou2 t a l i t e@1 r i@ n ) ((pos "OA%" ) ))
( "totalitarianism"  n ( t ou2 t a l i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "totality"  n ( t ou1 t a1 l i t ii ) ((pos "L@%" ) ))
( "totalizator"  n ( t ou1 t @ l ai z ei t @ r ) ((pos "K6$" ) ))
( "totalizators"  n ( t ou1 t @ l ai z ei t @ z ) ((pos "Kj$" ) ))
( "totalled"  v ( t ou1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "totalling"  v ( t ou1 t @ l i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "totally"  a ( t ou1 t @ l ii ) ((pos "Pu%" ) ))
( "totals"  v ( t ou1 t l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" )))
( "tote"  v ( t ou1 t ) ((pos "H2$" "K6$" ) (subcat )))
( "toted"  v ( t ou1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "totem"  n ( t ou1 t @ m ) ((pos "K6%" ) ))
( "totem-pole"  n ( t ou1 t @ m - p ou l ) ((pos "K6%" ) ))
( "totem-poles"  n ( t ou1 t @ m - p ou l z ) ((pos "Kj%" ) ))
( "totems"  n ( t ou1 t @ m z ) ((pos "Kj%" ) ))
( "totes"  v ( t ou1 t s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "toting"  v ( t ou1 t i ng ) ((pos "Hb$" ) (subcat )))
( "tots"  v ( t o1 t s ) ((pos "Ja$" "Kj%" ) (subcat "2C" )))
( "totted"  v ( t o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "totter"  v ( t o1 t @ r ) ((pos "I0%" ) (subcat "2A" )))
( "tottered"  v ( t o1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "tottering"  v ( t o1 t @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "totters"  v ( t o1 t @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "tottery"  j ( t o1 t @ r ii ) ((pos "OA%" ) ))
( "totting"  v ( t o1 t i ng ) ((pos "Jb$" ) (subcat "2C" )))
( "toucan"  n ( t uu1 k @ n ) ((pos "K6$" ) ))
( "toucans"  n ( t uu1 k @ n z ) ((pos "Kj$" ) ))
( "touch"  v ( t uh1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "touch-and-go"  j ( t uh2 ch - @ n - g ou1 ) ((pos "OA%" ) ))
( "touch-type"  v ( t uh1 ch - t ai p ) ((pos "I2%" ) (subcat )))
( "touch-typed"  v ( t uh1 ch - t ai p t ) ((pos "Ic$" "Id$" ) (subcat )))
( "touch-types"  v ( t uh1 ch - t ai p s ) ((pos "Ia%" ) (subcat )))
( "touch-typing"  v ( t uh1 ch - t ai p i ng ) ((pos "Ib%" ) (subcat )))
( "touchable"  j ( t uh1 ch @ b l ) ((pos "OA%" ) ))
( "touchdown"  n ( t uh1 ch d au n ) ((pos "K6%" ) ))
( "touchdowns"  n ( t uh1 ch d au n z ) ((pos "Kj%" ) ))
( "touched"  v ( t uh1 ch t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "touches"  v ( t uh1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "touchier"  j ( t uh1 ch i@ r ) ((pos "Or$" ) ))
( "touchiest"  j ( t uh1 ch i i s t ) ((pos "Os$" ) ))
( "touchily"  a ( t uh1 ch i l ii ) ((pos "Pu%" ) ))
( "touchiness"  n ( t uh1 ch i n @ s ) ((pos "L@%" ) ))
( "touching"  v ( t uh1 ch i ng ) ((pos "Jb%" "OA%" "T-%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "touchingly"  a ( t uh1 ch i ng l ii ) ((pos "Pu%" ) ))
( "touchline"  n ( t uh1 ch l ai n ) ((pos "K6%" ) ))
( "touchlines"  n ( t uh1 ch l ai n z ) ((pos "Kj%" ) ))
( "touchstone"  n ( t uh1 ch - s t ou n ) ((pos "K6%" ) ))
( "touchstones"  n ( t uh1 ch - s t ou n z ) ((pos "Kj$" ) ))
( "touchy"  j ( t uh1 ch ii ) ((pos "OD%" ) ))
( "tough"  n ( t uh1 f ) ((pos "K6%" "OC%" ) ))
( "toughen"  v ( t uh1 f n ) ((pos "J0%" ) ))
( "toughened"  v ( t uh1 f n d ) ((pos "Jc%" "Jd%" ) ))
( "toughening"  v ( t uh1 f n i ng ) ((pos "Jb%" ) ))
( "toughens"  v ( t uh1 f n z ) ((pos "Ja%" ) ))
( "tougher"  j ( t uh1 f @ r ) ((pos "Or%" ) ))
( "toughest"  j ( t uh1 f i s t ) ((pos "Os%" ) ))
( "toughie"  n ( t uh1 f ii ) ((pos "K6$" ) ))
( "toughies"  n ( t uh1 f i z ) ((pos "Kj$" ) ))
( "toughly"  a ( t uh1 f l ii ) ((pos "Pu%" ) ))
( "toughness"  n ( t uh1 f n @ s ) ((pos "L@%" ) ))
( "toughs"  n ( t uh1 f s ) ((pos "Kj$" ) ))
( "toupee"  n ( t uu1 p ei ) ((pos "K6$" ) ))
( "toupees"  n ( t uu1 p ei z ) ((pos "Kj$" ) ))
( "tour"  v ( t u@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "tour de force"  n ( t u@2 - d @ - f oo1 s ) ((pos "Ki%" ) ))
( "toured"  v ( t u@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "touring"  v ( t u@1 r i ng ) ((pos "Jb%" "L@%" "OA%" ) (subcat "2A" "2C" )))
( "tourism"  n ( t u@1 r i z @ m ) ((pos "L@%" ) ))
( "tourist"  n ( t u@1 r i s t ) ((pos "K6%" ) ))
( "tourists"  n ( t u@1 r i s t s ) ((pos "Kj%" ) ))
( "tournament"  n ( t oo1 n @ m @ n t ) ((pos "K6%" ) ))
( "tournaments"  n ( t oo1 n @ m @ n t s ) ((pos "Kj%" ) ))
( "tourney"  n ( t @@1 n ii ) ((pos "K6$" ) ))
( "tourneys"  n ( t @@1 n i z ) ((pos "Kj$" ) ))
( "tourniquet"  n ( t u@1 n i k ei ) ((pos "K6%" ) ))
( "tourniquets"  n ( t u@1 n i k ei z ) ((pos "Kj%" ) ))
( "tours"  v ( t u@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "tours de force"  n ( t u@2 - d @ - f oo1 s ) ((pos "Kj$" ) ))
( "tousle"  v ( t au1 z l ) ((pos "H2$" ) (subcat )))
( "tousled"  v ( t au1 z l d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "tousles"  v ( t au1 z l z ) ((pos "Ha$" ) (subcat )))
( "tousling"  v ( t au1 z l i ng ) ((pos "Hb$" ) (subcat )))
( "tout"  v ( t au1 t ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "tout ensemble"  n ( t uu2 t - o n s o1 m b l ) ((pos "Ki$" "Pu$" ) ))
( "touted"  v ( t au1 t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "touting"  v ( t au1 t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "touts"  v ( t au1 t s ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "tow"  v ( t ou1 ) ((pos "H0%" "M6%" ) (subcat "6A" "15A" )))
( "toward"  in ( t @1 w oo1 d ) ((pos "T-%" ) ))
( "towards"  in ( t @1 w oo1 d z ) ((pos "T-%" ) ))
( "towed"  v ( t ou1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "towel"  v ( t au1 @ l ) ((pos "H4%" "K6%" ) ))
( "towel-horse"  n ( t au1 @ l - h oo s ) ((pos "K6$" ) ))
( "towel-horses"  n ( t au1 @ l - h oo s i z ) ((pos "Kj$" ) ))
( "towel-rack"  n ( t au1 @ l - r a k ) ((pos "K6%" ) ))
( "towel-racks"  n ( t au1 @ l - r a k s ) ((pos "Kj%" ) ))
( "towel-rail"  n ( t au1 @ l - r ei l ) ((pos "K6%" ) ))
( "towel-rails"  n ( t au1 @ l - r ei l z ) ((pos "Kj%" ) ))
( "towelled"  v ( t au1 @ l d ) ((pos "Hc%" "Hd%" ) ))
( "towelling"  v ( t au1 @ l i ng ) ((pos "Hb%" "L@%" ) ))
( "towels"  v ( t au1 @ l z ) ((pos "Ha%" "Kj%" ) ))
( "tower"  v ( t au1 @ r ) ((pos "I0%" "K6%" ) (subcat )))
( "tower-block"  n ( t au1 @ - b l o k ) ((pos "K6%" ) ))
( "tower-blocks"  n ( t au1 @ - b l o k s ) ((pos "Kj%" ) ))
( "towered"  v ( t au1 @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "towering"  v ( t au1 @ r i ng ) ((pos "Ib%" "OA%" ) (subcat )))
( "towers"  v ( t au1 @ z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "towing"  v ( t ou1 i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "towing-line"  n ( t ou1 i ng - l ai n ) ((pos "K6$" ) ))
( "towing-lines"  n ( t ou1 i ng - l ai n z ) ((pos "Kj$" ) ))
( "towing-path"  n ( t ou1 i ng - p aa th ) ((pos "K6$" ) ))
( "towing-paths"  n ( t ou1 i ng - p aa dh z ) ((pos "Kj$" ) ))
( "towing-rope"  n ( t ou1 i n - r ou p ) ((pos "K6$" ) ))
( "towing-ropes"  n ( t ou1 i n - r ou p s ) ((pos "Kj$" ) ))
( "towline"  n ( t ou1 l ai n ) ((pos "K6%" ) ))
( "towlines"  n ( t ou1 l ai n z ) ((pos "Kj%" ) ))
( "town"  n ( t au1 n ) ((pos "K6*" ) ))
( "town-crier"  n ( t au1 n - k r ai1 @ r ) ((pos "K6%" ) ))
( "town-criers"  n ( t au1 n - k r ai1 @ z ) ((pos "Kj%" ) ))
( "town-gas"  n ( t au1 n - g a s ) ((pos "L@$" ) ))
( "townee"  n ( t au1 n ii ) ((pos "K6$" ) ))
( "townees"  n ( t au1 n i z ) ((pos "Kj$" ) ))
( "towns"  n ( t au1 n z ) ((pos "Kj%" ) ))
( "townsfolk"  n ( t au1 n z f ou k ) ((pos "Kj%" ) ))
( "township"  n ( t au1 n sh i p ) ((pos "K6%" ) ))
( "townships"  n ( t au1 n sh i p s ) ((pos "Kj%" ) ))
( "townsman"  n ( t au1 n z m @ n ) ((pos "Ki%" ) ))
( "townsmen"  n ( t au1 n z m @ n ) ((pos "Kj%" ) ))
( "townspeople"  n ( t au1 n z p ii p l ) ((pos "Kj%" ) ))
( "towpath"  n ( t ou1 p aa th ) ((pos "K6%" ) ))
( "towpaths"  n ( t ou1 p aa dh z ) ((pos "Kj%" ) ))
( "towrope"  n ( t ou1 r ou p ) ((pos "K6%" ) ))
( "towropes"  n ( t ou1 r ou p s ) ((pos "Kj%" ) ))
( "tows"  v ( t ou1 z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "toxaemia"  n ( t o1 k s ii1 m i@ ) ((pos "L@$" ) ))
( "toxic"  j ( t o1 k s i k ) ((pos "OA%" ) ))
( "toxicity"  n ( t o1 k s i1 s i t ii ) ((pos "L@%" ) ))
( "toxicologist"  n ( t o2 k s i k o1 l @ jh i s t ) ((pos "K6%" ) ))
( "toxicologists"  n ( t o2 k s i k o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "toxicology"  n ( t o2 k s i k o1 l @ jh ii ) ((pos "L@%" ) ))
( "toxin"  n ( t o1 k s i n ) ((pos "K6%" ) ))
( "toxins"  n ( t o1 k s i n z ) ((pos "Kj%" ) ))
( "toy"  v ( t oi1 ) ((pos "I0%" "K6%" ) (subcat )))
( "toyed"  v ( t oi1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "toying"  v ( t oi1 i ng ) ((pos "Ib%" ) (subcat )))
( "toys"  v ( t oi1 z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "toyshop"  n ( t oi1 sh o p ) ((pos "K6%" ) ))
( "toyshops"  n ( t oi1 sh o p s ) ((pos "Kj%" ) ))
( "trace"  v ( t r ei1 s ) ((pos "J2%" "M6%" ) (subcat "6A" "15A" )))
( "traceable"  j ( t r ei1 s @ b l ) ((pos "OA%" ) ))
( "traced"  v ( t r ei1 s t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15A" )))
( "tracer"  n ( t r ei1 s @ r ) ((pos "K6%" ) ))
( "traceries"  n ( t r ei1 s @ r i z ) ((pos "Mj$" ) ))
( "tracers"  n ( t r ei1 s @ z ) ((pos "Kj%" ) ))
( "tracery"  n ( t r ei1 s @ r ii ) ((pos "M8%" ) ))
( "traces"  v ( t r ei1 s i z ) ((pos "Ja%" "Mj%" ) (subcat "6A" "15A" )))
( "trachea"  n ( t r @1 k i@1 ) ((pos "Ki%" ) ))
( "tracheae"  n ( t r @1 k i1 ii ) ((pos "Kj$" ) ))
( "trachoma"  n ( t r @1 k ou1 m @ ) ((pos "L@$" ) ))
( "tracing"  v ( t r ei1 s i ng ) ((pos "Jb%" "K6%" ) (subcat "6A" "15A" )))
( "tracing-paper"  n ( t r ei1 s i ng - p ei p @ r ) ((pos "L@%" ) ))
( "tracings"  n ( t r ei1 s i ng z ) ((pos "Kj%" ) ))
( "track"  v ( t r a1 k ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "tracked"  v ( t r a1 k t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat "6A" "15A" )))
( "tracker"  n ( t r a1 k @ r ) ((pos "K6%" ) ))
( "trackers"  n ( t r a1 k @ z ) ((pos "Kj%" ) ))
( "tracking"  v ( t r a1 k i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "trackless"  j ( t r a1 k l @ s ) ((pos "OA%" ) ))
( "tracks"  v ( t r a1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "tract"  n ( t r a1 k t ) ((pos "K6%" ) ))
( "tractabilities"  n ( t r a2 k t @ b i1 l i t i z ) ((pos "Mj$" ) ))
( "tractability"  n ( t r a2 k t @ b i1 l i t ii ) ((pos "M8%" ) ))
( "tractable"  j ( t r a1 k t @ b l ) ((pos "OA%" ) ))
( "traction"  n ( t r a1 k sh @ n ) ((pos "L@%" ) ))
( "traction-engine"  n ( t r a1 k sh @ n - e n jh i n ) ((pos "K6%" ) ))
( "traction-engines"  n ( t r a1 k sh @ n - e n jh i n z ) ((pos "Kj%" ) ))
( "tractor"  n ( t r a1 k t @ r ) ((pos "K6%" ) ))
( "tractors"  n ( t r a1 k t @ z ) ((pos "Kj%" ) ))
( "tracts"  n ( t r a1 k t s ) ((pos "Kj%" ) ))
( "trad"  n ( t r a1 d ) ((pos "L@$" ) ))
( "trade"  v ( t r ei1 d ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "3A" "14" )))
( "trade-in"  n ( t r ei1 d - i n ) ((pos "K6%" ) ))
( "trade-ins"  n ( t r ei1 d - i n z ) ((pos "Kj%" ) ))
( "trade-union"  n ( t r ei1 d - y uu1 n i@ n ) ((pos "K6%" ) ))
( "trade-unionism"  n ( t r ei1 d - y uu1 n i@ n i z @ m ) ((pos "L@%" ) ))
( "trade-unionist"  n ( t r ei1 d - y uu1 n i@ n i s t ) ((pos "K6%" ) ))
( "trade-unionists"  n ( t r ei1 d - y uu1 n i@ n i s t s ) ((pos "Kj%" ) ))
( "trade-unions"  n ( t r ei1 d - y uu1 n i@ n z ) ((pos "Kj%" ) ))
( "trade-wind"  n ( t r ei1 d - w i n d ) ((pos "K6%" ) ))
( "trade-winds"  n ( t r ei1 d - w i n d z ) ((pos "Kj%" ) ))
( "traded"  v ( t r ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "14" )))
( "trademark"  n ( t r ei1 d m aa k ) ((pos "K6%" ) ))
( "trademarks"  n ( t r ei1 d m aa k s ) ((pos "Kj%" ) ))
( "trader"  n ( t r ei1 d @ r ) ((pos "K6%" ) ))
( "traders"  n ( t r ei1 d @ z ) ((pos "Kj%" ) ))
( "trades"  v ( t r ei1 d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "3A" "14" )))
( "trades-union"  n ( t r ei1 d z - y uu1 n i@ n ) ((pos "K6%" ) ))
( "trades-unions"  n ( t r ei1 d z - y uu1 n i@ n z ) ((pos "Kj%" ) ))
( "tradesfolk"  n ( t r ei1 d z f ou k ) ((pos "Kj%" ) ))
( "tradesman"  n ( t r ei1 d z m @ n ) ((pos "Ki%" ) ))
( "tradesmen"  n ( t r ei1 d z m @ n ) ((pos "Kj%" ) ))
( "tradespeople"  n ( t r ei1 d z p ii p l ) ((pos "Kj%" ) ))
( "trading"  v ( t r ei1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "14" )))
( "tradition"  n ( t r @1 d i1 sh @ n ) ((pos "M6%" ) ))
( "traditional"  j ( t r @1 d i1 sh @ n l ) ((pos "OA%" ) ))
( "traditionalism"  n ( t r @1 d i1 sh @ n @ l i z @ m ) ((pos "L@%" ) ))
( "traditionalist"  n ( t r @1 d i1 sh @ n @ l i s t ) ((pos "K6%" ) ))
( "traditionalists"  n ( t r @1 d i1 sh @ n @ l i s t s ) ((pos "Kj%" ) ))
( "traditionally"  a ( t r @1 d i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "traditions"  n ( t r @1 d i1 sh @ n z ) ((pos "Mj%" ) ))
( "traduce"  v ( t r @1 d y uu1 s ) ((pos "H2$" ) (subcat )))
( "traduced"  v ( t r @1 d y uu1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "traducer"  n ( t r @1 d y uu1 s @ r ) ((pos "K6$" ) ))
( "traducers"  n ( t r @1 d y uu1 s @ z ) ((pos "Kj$" ) ))
( "traduces"  v ( t r @1 d y uu1 s i z ) ((pos "Ha$" ) (subcat )))
( "traducing"  v ( t r @1 d y uu1 s i ng ) ((pos "Hb$" ) (subcat )))
( "traffic"  v ( t r a1 f i k ) ((pos "I5%" "L@%" ) (subcat )))
( "trafficator"  n ( t r a1 f i k ei t @ r ) ((pos "K6$" ) ))
( "trafficators"  n ( t r a1 f i k ei t @ z ) ((pos "Kj$" ) ))
( "trafficked"  v ( t r a1 f i k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "trafficker"  n ( t r a1 f i k @ r ) ((pos "K6%" ) ))
( "traffickers"  n ( t r a1 f i k @ z ) ((pos "Kj%" ) ))
( "trafficking"  v ( t r a1 f i k i ng ) ((pos "Ib%" ) (subcat )))
( "traffics"  v ( t r a1 f i k s ) ((pos "Ia%" ) (subcat )))
( "tragedian"  n ( t r @1 jh ii1 d i@ n ) ((pos "K6%" ) ))
( "tragedians"  n ( t r @1 jh ii1 d i@ n z ) ((pos "Kj%" ) ))
( "tragedienne"  n ( t r @1 jh ii2 d i e1 n ) ((pos "K6$" ) ))
( "tragediennes"  n ( t r @1 jh ii2 d i e1 n z ) ((pos "Kj$" ) ))
( "tragedies"  n ( t r a1 jh @ d i z ) ((pos "Mj%" ) ))
( "tragedy"  n ( t r a1 jh @ d ii ) ((pos "M8%" ) ))
( "tragic"  j ( t r a1 jh i k ) ((pos "OA%" ) ))
( "tragically"  a ( t r a1 jh i k l ii ) ((pos "Pu%" ) ))
( "tragicomedies"  n ( t r a2 jh i k o1 m @ d i z ) ((pos "Kj%" ) ))
( "tragicomedy"  n ( t r a2 jh i k o1 m @ d ii ) ((pos "K8%" ) ))
( "tragicomic"  j ( t r a2 jh i k o1 m i k ) ((pos "OA%" ) ))
( "trail"  v ( t r ei1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "trailed"  v ( t r ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "trailer"  n ( t r ei1 l @ r ) ((pos "K6%" ) ))
( "trailers"  n ( t r ei1 l @ z ) ((pos "Kj%" ) ))
( "trailing"  v ( t r ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "trails"  v ( t r ei1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "train"  v ( t r ei1 n ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "trainbearer"  n ( t r ei1 n b e@ r @ r ) ((pos "K6%" ) ))
( "trainbearers"  n ( t r ei1 n b e@ r @ z ) ((pos "Kj%" ) ))
( "trained"  v ( t r ei1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "trainee"  n ( t r ei1 n ii1 ) ((pos "K6%" ) ))
( "trainees"  n ( t r ei1 n ii1 z ) ((pos "Kj%" ) ))
( "trainer"  n ( t r ei1 n @ r ) ((pos "K6%" ) ))
( "trainers"  n ( t r ei1 n @ z ) ((pos "Kj%" ) ))
( "training"  v ( t r ei1 n i ng ) ((pos "Jb%" "L@%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "training-college"  n ( t r ei1 n i ng - k o l i jh ) ((pos "K6%" ) ))
( "training-colleges"  n ( t r ei1 n i ng - k o l i jh i z ) ((pos "Kj%" ) ))
( "training-ship"  n ( t r ei1 n i ng - sh i p ) ((pos "K6%" ) ))
( "training-ships"  n ( t r ei1 n i ng - sh i p s ) ((pos "Kj%" ) ))
( "trainload"  n ( t r ei1 n l ou d ) ((pos "K6%" ) ))
( "trainloads"  n ( t r ei1 n l ou d z ) ((pos "Kj%" ) ))
( "trainman"  n ( t r ei1 n m a n ) ((pos "Ki$" ) ))
( "trainmen"  n ( t r ei1 n m e n ) ((pos "Kj$" ) ))
( "trains"  v ( t r ei1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "14" "15A" )))
( "traipse"  v ( t r ei1 p s ) ((pos "I2%" ) (subcat "2A" "2B" )))
( "traipsed"  v ( t r ei1 p s t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "traipses"  v ( t r ei1 p s i z ) ((pos "Ia%" ) (subcat "2A" "2B" )))
( "traipsing"  v ( t r ei1 p s i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "trait"  n ( t r ei1 t ) ((pos "K6%" ) ))
( "traitor"  n ( t r ei1 t @ r ) ((pos "K6%" ) ))
( "traitorous"  j ( t r ei1 t @ r @ s ) ((pos "OA%" ) ))
( "traitorously"  a ( t r ei1 t @ r @ s l ii ) ((pos "Pu%" ) ))
( "traitors"  n ( t r ei1 t @ z ) ((pos "Kj%" ) ))
( "traitress"  n ( t r ei1 t r i s ) ((pos "K7$" ) ))
( "traitresses"  n ( t r ei1 t r i s i z ) ((pos "Kj$" ) ))
( "traits"  n ( t r ei1 t s ) ((pos "Kj%" ) ))
( "trajectories"  n ( t r @1 jh e1 k t @ r i z ) ((pos "Kj%" ) ))
( "trajectory"  n ( t r @1 jh e1 k t @ r ii ) ((pos "K8%" ) ))
( "tram"  n ( t r a1 m ) ((pos "K6%" ) ))
( "tram-car"  n ( t r a1 m - k aa r ) ((pos "K6%" ) ))
( "tram-cars"  n ( t r a1 m - k aa z ) ((pos "Kj%" ) ))
( "tramline"  n ( t r a1 m l ai n ) ((pos "K6%" ) ))
( "tramlines"  n ( t r a1 m l ai n z ) ((pos "Kj%" ) ))
( "trammel"  v ( t r a1 m @ l ) ((pos "H4$" ) (subcat )))
( "trammelled"  v ( t r a1 m @ l d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "trammelling"  v ( t r a1 m @ l i ng ) ((pos "Hb$" ) (subcat )))
( "trammels"  v ( t r a1 m @ l z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "tramp"  v ( t r a1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "tramp-steamer"  n ( t r a1 m p - s t ii m @ r ) ((pos "K6%" ) ))
( "tramp-steamers"  n ( t r a1 m p - s t ii m @ z ) ((pos "Kj%" ) ))
( "tramped"  v ( t r a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "tramping"  v ( t r a1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "trample"  v ( t r a1 m p l ) ((pos "J2%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "trampled"  v ( t r a1 m p l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "tramples"  v ( t r a1 m p l z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "trampling"  v ( t r a1 m p l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" )))
( "trampoline"  n ( t r a1 m p @ l ii n ) ((pos "K6%" ) ))
( "trampolines"  n ( t r a1 m p @ l ii n z ) ((pos "Kj%" ) ))
( "tramps"  v ( t r a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "trams"  n ( t r a1 m z ) ((pos "Kj%" ) ))
( "tramway"  n ( t r a1 m w ei ) ((pos "K6%" ) ))
( "tramways"  n ( t r a1 m w ei z ) ((pos "Kj%" ) ))
( "trance"  n ( t r aa1 n s ) ((pos "K6%" ) ))
( "trances"  n ( t r aa1 n s i z ) ((pos "Kj%" ) ))
( "tranquil"  j ( t r a1 ng k w i l ) ((pos "OA%" ) ))
( "tranquility"  n ( t r a1 n k w i1 l i t ii ) ((pos "L@%" ) ))
( "tranquilize"  v ( t r a1 n k w i l ai z ) ((pos "H2%" ) (subcat )))
( "tranquilized"  v ( t r a1 n k w i l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tranquilizes"  v ( t r a1 n k w i l ai z i z ) ((pos "Ha%" ) (subcat )))
( "tranquilizing"  v ( t r a1 n k w i l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "tranquillity"  n ( t r a1 n k w i1 l i t ii ) ((pos "L@%" ) ))
( "tranquillize"  v ( t r a1 n k w i l ai z ) ((pos "H2%" ) (subcat )))
( "tranquillized"  v ( t r a1 n k w i l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tranquillizer"  n ( t r a1 n k w i l ai z @ r ) ((pos "K6%" ) ))
( "tranquillizers"  n ( t r a1 n k w i l ai z @ z ) ((pos "Kj%" ) ))
( "tranquillizes"  v ( t r a1 n k w i l ai z i z ) ((pos "Ha%" ) (subcat )))
( "tranquillizing"  v ( t r a1 n k w i l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "tranquilly"  a ( t r a1 ng k w i l ii ) ((pos "Pu%" ) ))
( "trans"  n ( t r a1 n z ) ((pos "Y~$" ) ))
( "trans-"  j ( t r a n z - ) ((pos "U-%" ) ))
( "transact"  v ( t r a1 n z a1 k t ) ((pos "H0%" ) (subcat "6A" )))
( "transacted"  v ( t r a1 n z a1 k t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "transacting"  v ( t r a1 n z a1 k t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "transaction"  n ( t r a1 n z a1 k sh @ n ) ((pos "M6%" ) ))
( "transactions"  n ( t r a1 n z a1 k sh @ n z ) ((pos "Mj%" ) ))
( "transacts"  v ( t r a1 n z a1 k t s ) ((pos "Ha%" ) (subcat "6A" )))
( "transalpine"  n ( t r a1 n z a1 l p ai n ) ((pos "K6$" "OA%" ) ))
( "transalpines"  n ( t r a1 n z a1 l p ai n z ) ((pos "Kj$" ) ))
( "transatlantic"  j ( t r a2 n z @ t l a1 n t i k ) ((pos "OA%" ) ))
( "transcend"  v ( t r a1 n s e1 n d ) ((pos "H0%" ) (subcat )))
( "transcended"  v ( t r a1 n s e1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "transcendence"  n ( t r a1 n s e1 n d @ n s ) ((pos "L@%" ) ))
( "transcendency"  n ( t r a1 n s e1 n d @ n s ii ) ((pos "L@$" ) ))
( "transcendent"  j ( t r a1 n s e1 n d @ n t ) ((pos "OA%" ) ))
( "transcendental"  j ( t r a2 n s e n d e1 n t l ) ((pos "OA%" ) ))
( "transcendentalism"  n ( t r a2 n s e n d e1 n t @ l i z @ m ) ((pos "L@$" ) ))
( "transcendentalist"  n ( t r a2 n s e n d e1 n t @ l i s t ) ((pos "K6$" ) ))
( "transcendentalists"  n ( t r a2 n s e n d e1 n t @ l i s t s ) ((pos "Kj$" ) ))
( "transcendentally"  a ( t r a2 n s @ n d e1 n t @ l ii ) ((pos "Pu%" ) ))
( "transcending"  v ( t r a1 n s e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "transcends"  v ( t r a1 n s e1 n d z ) ((pos "Ha%" ) (subcat )))
( "transcontinental"  j ( t r a2 n z k o n t i n e1 n t l ) ((pos "OA%" ) ))
( "transcribe"  v ( t r a1 n s k r ai1 b ) ((pos "H2%" ) (subcat )))
( "transcribed"  v ( t r a1 n s k r ai1 b d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "transcribes"  v ( t r a1 n s k r ai1 b z ) ((pos "Ha%" ) (subcat )))
( "transcribing"  v ( t r a1 n s k r ai1 b i ng ) ((pos "Hb%" ) (subcat )))
( "transcript"  n ( t r a1 n s k r i p t ) ((pos "K6%" ) ))
( "transcription"  n ( t r a1 n s k r i1 p sh @ n ) ((pos "M6%" ) ))
( "transcriptions"  n ( t r a1 n s k r i1 p sh @ n z ) ((pos "Mj%" ) ))
( "transcripts"  n ( t r a1 n s k r i p t s ) ((pos "Kj%" ) ))
( "transept"  n ( t r a1 n s e p t ) ((pos "K6%" ) ))
( "transepts"  n ( t r a1 n s e p t s ) ((pos "Kj%" ) ))
( "transfer"  n ( t r a1 n s f @@ r ) ((pos "M6%" ) ))
( "transfer"  v ( t r a1 n s f @@1 r ) ((pos "J4%" ) (subcat "3A" "6A" )))
( "transferability"  n ( t r a2 n s f @@2 r @ b i1 l i t ii ) ((pos "L@%" ) ))
( "transferable"  j ( t r a1 n s f @@1 r @ b l ) ((pos "OA%" ) ))
( "transference"  n ( t r a1 n s f @ r @ n s ) ((pos "M6%" ) ))
( "transferences"  n ( t r a1 n s f @ r @ n s i z ) ((pos "Mj$" ) ))
( "transferred"  v ( t r a1 n s f @@1 d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "transferring"  v ( t r a1 n s f @@1 r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "transfers"  n ( t r a1 n s f @@ z ) ((pos "Mj%" ) ))
( "transfers"  v ( t r a1 n s f @@1 z ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "transfiguration"  n ( t r a2 n s f i g @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "transfigurations"  n ( t r a2 n s f i g @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "transfigure"  v ( t r a1 n s f i1 g @ r ) ((pos "H2$" ) (subcat )))
( "transfigured"  v ( t r a1 n s f i1 g @ d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "transfigures"  v ( t r a1 n s f i1 g @ z ) ((pos "Ha$" ) (subcat )))
( "transfiguring"  v ( t r a1 n s f i1 g @ r i ng ) ((pos "Hb$" ) (subcat )))
( "transfix"  v ( t r a1 n s f i1 k s ) ((pos "H1%" ) (subcat )))
( "transfixed"  v ( t r a1 n s f i1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "transfixes"  v ( t r a1 n s f i1 k s i z ) ((pos "Ha%" ) (subcat )))
( "transfixing"  v ( t r a1 n s f i1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "transform"  v ( t r a1 n s f oo1 m ) ((pos "H0%" ) (subcat "6A" )))
( "transformable"  j ( t r a1 n s f oo1 m @ b l ) ((pos "OA%" ) ))
( "transformation"  n ( t r a2 n s f @ m ei1 sh @ n ) ((pos "M6%" ) ))
( "transformations"  n ( t r a2 n s f @ m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "transformed"  v ( t r a1 n s f oo1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "transformer"  n ( t r a1 n s f oo1 m @ r ) ((pos "K6%" ) ))
( "transformers"  n ( t r a1 n s f oo1 m @ z ) ((pos "Kj%" ) ))
( "transforming"  v ( t r a1 n s f oo1 m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "transforms"  v ( t r a1 n s f oo1 m z ) ((pos "Ha%" ) (subcat "6A" )))
( "transfuse"  v ( t r a1 n s f y uu1 z ) ((pos "H2$" ) (subcat )))
( "transfused"  v ( t r a1 n s f y uu1 z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "transfuses"  v ( t r a1 n s f y uu1 z i z ) ((pos "Ha$" ) (subcat )))
( "transfusing"  v ( t r a1 n s f y uu1 z i ng ) ((pos "Hb$" ) (subcat )))
( "transfusion"  n ( t r a1 n s f y uu1 zh n ) ((pos "M6%" ) ))
( "transfusions"  n ( t r a1 n s f y uu1 zh n z ) ((pos "Mj%" ) ))
( "transgress"  v ( t r a1 n z g r e1 s ) ((pos "J1%" ) (subcat "2A" )))
( "transgressed"  v ( t r a1 n z g r e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "transgresses"  v ( t r a1 n z g r e1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "transgressing"  v ( t r a1 n z g r e1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "transgression"  n ( t r a1 n z g r e1 sh @ n ) ((pos "M6%" ) ))
( "transgressions"  n ( t r a1 n z g r e1 sh @ n z ) ((pos "Mj%" ) ))
( "transgressor"  n ( t r a1 n z g r e1 s @ r ) ((pos "K6%" ) ))
( "transgressors"  n ( t r a1 n z g r e1 s @ z ) ((pos "Kj%" ) ))
( "transience"  n ( t r a1 n z i@ n s ) ((pos "L@%" ) ))
( "transiency"  n ( t r a1 n z i@ n s ii ) ((pos "L@$" ) ))
( "transient"  n ( t r a1 n z i@ n t ) ((pos "K6$" "OA%" ) ))
( "transiently"  a ( t r a1 n z i@ n t l ii ) ((pos "Pu%" ) ))
( "transients"  n ( t r a1 n z i@ n t s ) ((pos "Kj$" ) ))
( "transistor"  n ( t r a1 n z i1 s t @ r ) ((pos "K6%" ) ))
( "transistorized"  j ( t r a1 n z i1 s t @ r ai z d ) ((pos "OA%" ) ))
( "transistors"  n ( t r a1 n z i1 s t @ z ) ((pos "Kj%" ) ))
( "transit"  n ( t r a1 n s i t ) ((pos "L@%" ) ))
( "transition"  n ( t r a1 n z i1 sh @ n ) ((pos "M6%" ) ))
( "transitional"  j ( t r a1 n s i1 sh @ n l ) ((pos "OA%" ) ))
( "transitionally"  a ( t r a1 n s i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "transitions"  n ( t r a1 n z i1 sh @ n z ) ((pos "Mj%" ) ))
( "transitive"  j ( t r a1 n s @ t i v ) ((pos "OA%" ) ))
( "transitively"  a ( t r a1 n s @ t i v l ii ) ((pos "Pu%" ) ))
( "transitory"  j ( t r a1 n s i t r ii ) ((pos "OA%" ) ))
( "translatable"  j ( t r a1 n s l ei1 t @ b l ) ((pos "OA%" ) ))
( "translate"  v ( t r a1 n z l ei1 t ) ((pos "H2%" ) (subcat "6A" )))
( "translated"  v ( t r a1 n z l ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "translates"  v ( t r a1 n z l ei1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "translating"  v ( t r a1 n z l ei1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "translation"  n ( t r a1 n s l ei1 sh @ n ) ((pos "M6%" ) ))
( "translations"  n ( t r a1 n s l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "translator"  n ( t r a1 n z l ei1 t @ r ) ((pos "K6%" ) ))
( "translators"  n ( t r a1 n z l ei1 t @ z ) ((pos "Kj%" ) ))
( "transliterate"  v ( t r a1 n z l i1 t @ r ei t ) ((pos "H2$" ) (subcat "6A" )))
( "transliterated"  v ( t r a1 n z l i1 t @ r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "transliterates"  v ( t r a1 n z l i1 t @ r ei t s ) ((pos "Ha$" ) (subcat "6A" )))
( "transliterating"  v ( t r a1 n z l i1 t @ r ei t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "transliteration"  n ( t r a2 n z l i t @ r ei1 sh @ n ) ((pos "M6$" ) ))
( "transliterations"  n ( t r a2 n z l i t @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "translucence"  n ( t r a1 n z l uu1 s n s ) ((pos "L@%" ) ))
( "translucency"  n ( t r a1 n z l uu1 s n s ii ) ((pos "L@$" ) ))
( "translucent"  j ( t r a1 n z l uu1 s n t ) ((pos "OA%" ) ))
( "transmigration"  n ( t r a2 n z m ai g r ei1 sh @ n ) ((pos "L@%" ) ))
( "transmission"  n ( t r a1 n z m i1 sh @ n ) ((pos "M6%" ) ))
( "transmissions"  n ( t r a1 n z m i1 sh @ n z ) ((pos "Mj%" ) ))
( "transmit"  v ( t r a1 n z m i1 t ) ((pos "H4%" ) (subcat "6A" )))
( "transmits"  v ( t r a1 n z m i1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "transmitted"  v ( t r a1 n z m i1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "transmitter"  n ( t r a1 n z m i1 t @ r ) ((pos "K6%" ) ))
( "transmitters"  n ( t r a1 n z m i1 t @ z ) ((pos "Kj%" ) ))
( "transmitting"  v ( t r a1 n z m i1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "transmogrification"  n ( t r a2 n z m o g r i f i k ei1 sh @ n ) ((pos "M6$" ) ))
( "transmogrifications"  n ( t r a2 n z m o g r i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "transmogrified"  v ( t r a1 n z m o1 g r i f ai d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "transmogrifies"  v ( t r a1 n z m o1 g r i f ai z ) ((pos "Ha$" ) (subcat )))
( "transmogrify"  v ( t r a1 n z m o1 g r i f ai ) ((pos "H3$" ) (subcat )))
( "transmogrifying"  v ( t r a1 n z m o1 g r i f ai i ng ) ((pos "Hb$" ) (subcat )))
( "transmutable"  j ( t r a1 n z m y uu1 t @ b l ) ((pos "OA$" ) ))
( "transmutation"  n ( t r a2 n z m y uu t ei1 sh @ n ) ((pos "M6$" ) ))
( "transmutations"  n ( t r a2 n z m y uu t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "transmute"  v ( t r a1 n z m y uu1 t ) ((pos "H2$" ) (subcat "6A" )))
( "transmuted"  v ( t r a1 n z m y uu1 t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "transmutes"  v ( t r a1 n z m y uu1 t s ) ((pos "Ha$" ) (subcat "6A" )))
( "transmuting"  v ( t r a1 n z m y uu1 t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "transoceanic"  j ( t r a2 n z ou2 sh i a1 n i k ) ((pos "OA$" ) ))
( "transom"  n ( t r a1 n s @ m ) ((pos "K6$" ) ))
( "transom-window"  n ( t r a2 n s @ m - w i1 n d ou ) ((pos "K6$" ) ))
( "transom-windows"  n ( t r a2 n s @ m - w i1 n d ou z ) ((pos "Kj$" ) ))
( "transoms"  n ( t r a1 n s @ m z ) ((pos "Kj$" ) ))
( "transparence"  n ( t r a1 n s p a1 r @ n s ) ((pos "L@%" ) ))
( "transparencies"  n ( t r a1 n s p a1 r @ n s i z ) ((pos "Mj%" ) ))
( "transparency"  n ( t r a1 n s p a1 r @ n s ii ) ((pos "M8%" ) ))
( "transparent"  j ( t r a1 n s p a1 r @ n t ) ((pos "OA%" ) ))
( "transparently"  a ( t r a1 n s p a1 r @ n t l ii ) ((pos "Pu%" ) ))
( "transpiration"  n ( t r a2 n s p i r ei1 sh @ n ) ((pos "L@$" ) ))
( "transpire"  v ( t r a1 n s p ai1 @ r ) ((pos "J2%" ) (subcat "2A" )))
( "transpired"  v ( t r a1 n s p ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "transpires"  v ( t r a1 n s p ai1 @ z ) ((pos "Ja%" ) (subcat "2A" )))
( "transpiring"  v ( t r a1 n s p ai1 @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "transplant"  n ( t r a1 n s p l aa n t ) ((pos "K6%" ) ))
( "transplant"  v ( t r a1 n s p l aa1 n t ) ((pos "J0%" ) (subcat "2A" )))
( "transplantation"  n ( t r a2 n s p l aa n t ei1 sh @ n ) ((pos "M6%" ) ))
( "transplantations"  n ( t r a2 n s p l aa n t ei1 sh @ n z ) ((pos "Mj$" ) ))
( "transplanted"  v ( t r a1 n s p l aa1 n t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "transplanting"  v ( t r a1 n s p l aa1 n t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "transplants"  n ( t r a1 n s p l aa n t s ) ((pos "Kj%" ) ))
( "transplants"  v ( t r a1 n s p l aa1 n t s ) ((pos "Ja%" ) (subcat "2A" )))
( "transpolar"  j ( t r a2 n z p ou1 l @ r ) ((pos "OA$" ) ))
( "transport"  n ( t r a1 n s p oo t ) ((pos "M6%" ) ))
( "transport"  v ( t r a1 n s p oo1 t ) ((pos "H0%" ) (subcat "6A" )))
( "transportable"  j ( t r a1 n s p oo1 t @ b l ) ((pos "OA%" ) ))
( "transportation"  n ( t r a2 n s p oo t ei1 sh @ n ) ((pos "L@%" ) ))
( "transported"  v ( t r a1 n s p oo1 t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "transporter"  n ( t r a1 n s p oo1 t @ r ) ((pos "K6%" ) ))
( "transporters"  n ( t r a1 n s p oo1 t @ z ) ((pos "Kj%" ) ))
( "transporting"  v ( t r a1 n s p oo1 t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "transports"  n ( t r a1 n s p oo t s ) ((pos "Mj%" ) ))
( "transports"  v ( t r a1 n s p oo1 t s ) ((pos "Ha%" ) (subcat "6A" )))
( "transpose"  v ( t r a1 n s p ou1 z ) ((pos "H2%" ) (subcat "6A" )))
( "transposed"  v ( t r a1 n s p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "transposes"  v ( t r a1 n s p ou1 z i z ) ((pos "Ha%" ) (subcat "6A" )))
( "transposing"  v ( t r a1 n s p ou1 z i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "transposition"  n ( t r a2 n s p @ z i1 sh @ n ) ((pos "M6%" ) ))
( "transpositions"  n ( t r a2 n s p @ z i1 sh @ n z ) ((pos "Mj%" ) ))
( "transsexual"  n ( t r a1 n z s e1 k sh u@ l ) ((pos "K6$" ) ))
( "transsexuals"  n ( t r a1 n z s e1 k sh u@ l z ) ((pos "Kj$" ) ))
( "transship"  v ( t r a1 n sh i1 p ) ((pos "H4$" ) (subcat )))
( "transshipment"  n ( t r a1 n sh i1 p m @ n t ) ((pos "K6$" ) ))
( "transshipments"  n ( t r a1 n sh i1 p m @ n t s ) ((pos "Kj$" ) ))
( "transshipped"  v ( t r a1 n sh i1 p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "transshipping"  v ( t r a1 n sh i1 p i ng ) ((pos "Hb$" ) (subcat )))
( "transships"  v ( t r a1 n sh i1 p s ) ((pos "Ha$" ) (subcat )))
( "transubstantiation"  n ( t r a2 n s @ b s t a2 n sh i ei1 sh @ n ) ((pos "L@%" ) ))
( "transverse"  j ( t r a1 n z v @@ s ) ((pos "OA%" ) ))
( "transversely"  a ( t r a1 n z v @@ s l ii ) ((pos "Pu%" ) ))
( "transvestism"  n ( t r a1 n z v e1 s t i z @ m ) ((pos "L@%" ) ))
( "transvestite"  n ( t r a1 n z v e1 s t ai t ) ((pos "K6%" ) ))
( "transvestites"  n ( t r a1 n z v e1 s t ai t s ) ((pos "Kj%" ) ))
( "trap"  v ( t r a1 p ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "trap-door"  n ( t r a1 p - d oo r ) ((pos "K6%" ) ))
( "trap-doors"  n ( t r a1 p - d oo z ) ((pos "Kj%" ) ))
( "trap-shooting"  n ( t r a1 p - sh uu t i ng ) ((pos "L@$" ) ))
( "trapeze"  n ( t r @1 p ii1 z ) ((pos "K6%" ) ))
( "trapezes"  n ( t r @1 p ii1 z i z ) ((pos "Kj%" ) ))
( "trapezium"  n ( t r @1 p ii1 z i@ m ) ((pos "K6%" ) ))
( "trapeziums"  n ( t r @1 p ii1 z i@ m z ) ((pos "Kj%" ) ))
( "trapezoid"  n ( t r a1 p i z oi d ) ((pos "K6$" ) ))
( "trapezoids"  n ( t r a1 p i z oi d z ) ((pos "Kj$" ) ))
( "trapped"  v ( t r a1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "trapper"  n ( t r a1 p @ r ) ((pos "K6%" ) ))
( "trappers"  n ( t r a1 p @ z ) ((pos "Kj%" ) ))
( "trapping"  v ( t r a1 p i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "trappings"  n ( t r a1 p i ng z ) ((pos "Kj%" ) ))
( "traps"  v ( t r a1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "trash"  n ( t r a1 sh ) ((pos "L@%" ) ))
( "trashier"  j ( t r a1 sh i@ r ) ((pos "Or$" ) ))
( "trashiest"  j ( t r a1 sh i i s t ) ((pos "Os$" ) ))
( "trashy"  j ( t r a1 sh ii ) ((pos "OD%" ) ))
( "trauma"  n ( t r oo1 m @ ) ((pos "K6%" ) ))
( "traumas"  n ( t r oo1 m @ z ) ((pos "Kj%" ) ))
( "traumatic"  j ( t r oo1 m a1 t i k ) ((pos "OA%" ) ))
( "travail"  n ( t r a1 v ei l ) ((pos "M6%" ) ))
( "travails"  n ( t r a1 v ei l z ) ((pos "Mj%" ) ))
( "travel"  v ( t r a1 v @ l ) ((pos "J4%" "M6%" ) (subcat "2A" "2B" "2C" "3A" )))
( "travel-soiled"  j ( t r a1 v l - s oi l d ) ((pos "OA%" ) ))
( "travel-stained"  j ( t r a1 v l - s t ei n d ) ((pos "OA%" ) ))
( "travel-worn"  j ( t r a1 v l - w oo n ) ((pos "OA%" ) ))
( "traveled"  j ( t r a1 v @ l d ) ((pos "OA$" ) ))
( "travelled"  v ( t r a1 v @ l d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "2C" "3A" )))
( "traveller"  n ( t r a1 v l @ r ) ((pos "K6%" ) ))
( "travellers"  n ( t r a1 v l @ z ) ((pos "Kj%" ) ))
( "travelling"  v ( t r a1 v @ l i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "3A" )))
( "travelogue"  n ( t r a1 v @ l o g ) ((pos "K6%" ) ))
( "travelogues"  n ( t r a1 v @ l o g z ) ((pos "Kj%" ) ))
( "travels"  v ( t r a1 v @ l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" )))
( "traverse"  v ( t r a1 v @@ s ) ((pos "H2%" "K6$" ) (subcat )))
( "traversed"  v ( t r a1 v @@ s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "traverses"  v ( t r a1 v @@ s i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "traversing"  v ( t r a1 v @@ s i ng ) ((pos "Hb%" ) (subcat )))
( "travestied"  v ( t r a1 v @ s t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "travesties"  v ( t r a1 v @ s t i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "travesty"  v ( t r a1 v @ s t ii ) ((pos "H3%" "K8%" ) (subcat )))
( "travestying"  v ( t r a1 v @ s t i i ng ) ((pos "Hb$" ) (subcat )))
( "trawl"  v ( t r oo1 l ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "trawl-net"  n ( t r oo1 l - n e t ) ((pos "K6%" ) ))
( "trawl-nets"  n ( t r oo1 l - n e t s ) ((pos "Kj%" ) ))
( "trawled"  v ( t r oo1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "trawler"  n ( t r oo1 l @ r ) ((pos "K6%" ) ))
( "trawlers"  n ( t r oo1 l @ z ) ((pos "Kj%" ) ))
( "trawling"  v ( t r oo1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "trawls"  v ( t r oo1 l z ) ((pos "Ja%" "Kj$" ) (subcat "2A" )))
( "tray"  n ( t r ei1 ) ((pos "K6%" ) ))
( "tray-cloth"  n ( t r ei1 - k l o th ) ((pos "K6%" ) ))
( "tray-cloths"  n ( t r ei1 - k l o th s ) ((pos "Kj%" ) ))
( "trays"  n ( t r ei1 z ) ((pos "Kj%" ) ))
( "treacheries"  n ( t r e1 ch @ r i z ) ((pos "Mj$" ) ))
( "treacherous"  j ( t r e1 ch @ r @ s ) ((pos "OA%" ) ))
( "treacherously"  a ( t r e1 ch @ r @ s l ii ) ((pos "Pu%" ) ))
( "treachery"  n ( t r e1 ch @ r ii ) ((pos "M8%" ) ))
( "treacle"  n ( t r ii1 k l ) ((pos "L@%" ) ))
( "treacly"  j ( t r ii1 k l ii ) ((pos "OA%" ) ))
( "tread"  v ( t r e1 d ) ((pos "J5%" "K6%" ) (subcat "2C" "3A" "6A" "15A" )))
( "treading"  v ( t r e1 d i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "treadle"  v ( t r e1 d l ) ((pos "I2%" "K6%" ) (subcat )))
( "treadled"  v ( t r e1 d l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "treadles"  v ( t r e1 d l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "treadling"  v ( t r e1 d l i ng ) ((pos "Ib%" ) (subcat )))
( "treadmill"  n ( t r e1 d m i l ) ((pos "K6%" ) ))
( "treadmills"  n ( t r e1 d m i l z ) ((pos "Kj%" ) ))
( "treads"  v ( t r e1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" "15A" )))
( "treas"  n ( t r e1 zh @ r @ r ) ((pos "Y>%" ) ))
( "treason"  n ( t r ii1 z n ) ((pos "L@%" ) ))
( "treasonable"  j ( t r ii1 z @ n @ b l ) ((pos "OA%" ) ))
( "treasonably"  a ( t r ii1 z @ n @ b l ii ) ((pos "Pu%" ) ))
( "treasonous"  j ( t r ii1 z @ n @ s ) ((pos "OA$" ) ))
( "treasure"  v ( t r e1 zh @ r ) ((pos "H2%" "M6%" ) (subcat "6A" )))
( "treasure-house"  n ( t r e1 zh @ - h au s ) ((pos "K6%" ) ))
( "treasure-houses"  n ( t r e1 zh @ - h au z i z ) ((pos "Kj$" ) ))
( "treasure-trove"  n ( t r e1 zh @ - t r ou v ) ((pos "L@%" ) ))
( "treasured"  v ( t r e1 zh @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "treasurer"  n ( t r e1 zh @ r @ r ) ((pos "K6%" ) ))
( "treasurers"  n ( t r e1 zh @ r @ z ) ((pos "Kj%" ) ))
( "treasures"  v ( t r e1 zh @ z ) ((pos "Ha%" "Mj%" ) (subcat "6A" )))
( "treasuries"  n ( t r e1 zh @ r i z ) ((pos "Kj%" ) ))
( "treasuring"  v ( t r e1 zh @ r i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "treasury"  n ( t r e1 zh @ r ii ) ((pos "K8%" ) ))
( "treat"  v ( t r ii1 t ) ((pos "J0%" "K6%" ) (subcat "3A" "6A" "14" "15A" )))
( "treated"  v ( t r ii1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "14" "15A" )))
( "treaties"  n ( t r ii1 t i z ) ((pos "Mj%" ) ))
( "treating"  v ( t r ii1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "14" "15A" )))
( "treatise"  n ( t r ii1 t i s ) ((pos "K6%" ) ))
( "treatises"  n ( t r ii1 t i s i z ) ((pos "Kj%" ) ))
( "treatment"  n ( t r ii1 t m @ n t ) ((pos "M6%" ) ))
( "treatments"  n ( t r ii1 t m @ n t s ) ((pos "Mj%" ) ))
( "treats"  v ( t r ii1 t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "14" "15A" )))
( "treaty"  n ( t r ii1 t ii ) ((pos "M8%" ) ))
( "treble"  v ( t r e1 b l ) ((pos "J2%" "K6%" "OA%" ) (subcat "2A" )))
( "trebled"  v ( t r e1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "trebles"  v ( t r e1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "trebling"  v ( t r e1 b l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tree"  v ( t r ii1 ) ((pos "H5$" "K6%" ) (subcat )))
( "tree-fern"  n ( t r ii1 - f @@ n ) ((pos "K6%" ) ))
( "tree-ferns"  n ( t r ii1 - f @@ n z ) ((pos "Kj%" ) ))
( "treed"  v ( t r ii1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "treeing"  v ( t r ii1 i ng ) ((pos "Hb$" ) (subcat )))
( "treeless"  j ( t r ii1 l @ s ) ((pos "OA%" ) ))
( "trees"  v ( t r ii1 z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "trefoil"  n ( t r e1 f oi l ) ((pos "K6$" ) ))
( "trefoils"  n ( t r e1 f oi l z ) ((pos "Kj$" ) ))
( "trek"  v ( t r e1 k ) ((pos "I4%" "K6%" ) (subcat "2A" "2B" )))
( "trekked"  v ( t r e1 k t ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "trekking"  v ( t r e1 k i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "treks"  v ( t r e1 k s ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" )))
( "trellis"  v ( t r e1 l i s ) ((pos "H1$" "K7%" ) (subcat )))
( "trellised"  v ( t r e1 l i s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "trellises"  v ( t r e1 l i s i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "trellising"  v ( t r e1 l i s i ng ) ((pos "Hb$" ) (subcat )))
( "tremble"  v ( t r e1 m b l ) ((pos "I2%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "trembled"  v ( t r e1 m b l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" "2C" )))
( "trembles"  v ( t r e1 m b l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "trembling"  v ( t r e1 m b l i ng ) ((pos "Ib%" ) (subcat "2A" "2B" "2C" )))
( "tremendous"  j ( t r i1 m e1 n d @ s ) ((pos "OA%" ) ))
( "tremendously"  a ( t r i1 m e1 n d @ s l ii ) ((pos "Pu%" ) ))
( "tremolo"  n ( t r e1 m @ l ou ) ((pos "K6%" ) ))
( "tremolos"  n ( t r e1 m @ l ou z ) ((pos "Kj%" ) ))
( "tremor"  n ( t r e1 m @ r ) ((pos "K6%" ) ))
( "tremors"  n ( t r e1 m @ z ) ((pos "Kj%" ) ))
( "tremulous"  j ( t r e1 m y u l @ s ) ((pos "OA$" ) ))
( "tremulously"  a ( t r e1 m y u l @ s l ii ) ((pos "Pu$" ) ))
( "trench"  v ( t r e1 n ch ) ((pos "H1$" "K7%" ) (subcat )))
( "trenchancy"  n ( t r e1 n ch @ n s ii ) ((pos "L@$" ) ))
( "trenchant"  j ( t r e1 n ch @ n t ) ((pos "OA%" ) ))
( "trenchantly"  a ( t r e1 n ch @ n t l ii ) ((pos "Pu%" ) ))
( "trenched"  v ( t r e1 n ch t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "trencher"  n ( t r e1 n ch @ r ) ((pos "K6$" ) ))
( "trencherman"  n ( t r e1 n ch @ m @ n ) ((pos "Ki%" ) ))
( "trenchermen"  n ( t r e1 n ch @ m @ n ) ((pos "Kj%" ) ))
( "trenchers"  n ( t r e1 n ch @ z ) ((pos "Kj$" ) ))
( "trenches"  v ( t r e1 n ch i z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "trenching"  v ( t r e1 n ch i ng ) ((pos "Hb$" ) (subcat )))
( "trend"  v ( t r e1 n d ) ((pos "I0$" "K6%" ) (subcat )))
( "trend-setter"  n ( t r e1 n d - s e t @ r ) ((pos "K6%" ) ))
( "trend-setters"  n ( t r e1 n d - s e t @ z ) ((pos "Kj%" ) ))
( "trend-setting"  n ( t r e1 n d - s e t i ng ) ((pos "L@%" ) ))
( "trended"  v ( t r e1 n d i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "trendier"  j ( t r e1 n d i@ r ) ((pos "Or%" ) ))
( "trendiest"  j ( t r e1 n d i i s t ) ((pos "Os%" ) ))
( "trending"  v ( t r e1 n d i ng ) ((pos "Ib$" ) (subcat )))
( "trends"  v ( t r e1 n d z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "trendy"  j ( t r e1 n d ii ) ((pos "OD%" ) ))
( "trepan"  v ( t r i1 p a1 n ) ((pos "H4$" ) ))
( "trepanned"  v ( t r i1 p a1 n d ) ((pos "Hc$" "Hd$" ) ))
( "trepanning"  v ( t r i1 p a1 n i ng ) ((pos "Hb$" ) ))
( "trepans"  v ( t r i1 p a1 n z ) ((pos "Ha$" ) ))
( "trephine"  v ( t r i1 f ii1 n ) ((pos "H2$" "K6$" ) (subcat )))
( "trephined"  v ( t r i1 f ii1 n d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "trephines"  v ( t r i1 f ii1 n z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "trephining"  v ( t r i1 f ii1 n i ng ) ((pos "Hb$" ) (subcat )))
( "trepidation"  n ( t r e2 p i d ei1 sh @ n ) ((pos "L@%" ) ))
( "trespass"  v ( t r e1 s p @ s ) ((pos "I1%" "M7%" ) (subcat "2A" )))
( "trespassed"  v ( t r e1 s p @ s t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "trespasser"  n ( t r e1 s p @ s @ r ) ((pos "K6%" ) ))
( "trespassers"  n ( t r e1 s p @ s @ z ) ((pos "Kj%" ) ))
( "trespasses"  v ( t r e1 s p @ s i z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "trespassing"  v ( t r e1 s p @ s i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "tress"  n ( t r e1 s ) ((pos "K7%" ) ))
( "tresses"  n ( t r e1 s i z ) ((pos "Kj%" ) ))
( "trestle"  n ( t r e1 s l ) ((pos "K6%" ) ))
( "trestle-bridge"  n ( t r e1 s l - b r i jh ) ((pos "K6$" ) ))
( "trestle-bridges"  n ( t r e1 s l - b r i jh i z ) ((pos "Kj$" ) ))
( "trestle-table"  n ( t r e1 s l - t ei b l ) ((pos "K6%" ) ))
( "trestle-tables"  n ( t r e1 s l - t ei b l z ) ((pos "Kj%" ) ))
( "trestles"  n ( t r e1 s l z ) ((pos "Kj%" ) ))
( "trews"  n ( t r uu1 z ) ((pos "Kj$" ) ))
( "tri-"  j ( t r ai - ) ((pos "U-%" ) ))
( "triad"  n ( t r ai1 a d ) ((pos "K6%" ) ))
( "triads"  n ( t r ai1 a d z ) ((pos "Kj%" ) ))
( "trial"  n ( t r ai1 @ l ) ((pos "M6%" ) ))
( "trials"  n ( t r ai1 @ l z ) ((pos "Mj%" ) ))
( "triangle"  n ( t r ai1 a ng g l ) ((pos "K6%" ) ))
( "triangles"  n ( t r ai1 a ng g l z ) ((pos "Kj%" ) ))
( "triangular"  j ( t r ai1 a1 ng g y u l @ r ) ((pos "OA%" ) ))
( "tribal"  j ( t r ai1 b l ) ((pos "OA%" ) ))
( "tribalism"  n ( t r ai1 b @ l i z @ m ) ((pos "L@%" ) ))
( "tribe"  n ( t r ai1 b ) ((pos "K6%" ) ))
( "tribes"  n ( t r ai1 b z ) ((pos "Kj%" ) ))
( "tribesman"  n ( t r ai1 b z m @ n ) ((pos "Ki%" ) ))
( "tribesmen"  n ( t r ai1 b z m @ n ) ((pos "Kj%" ) ))
( "tribulation"  n ( t r i2 b y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "tribulations"  n ( t r i2 b y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "tribunal"  n ( t r ai1 b y uu1 n l ) ((pos "K6%" ) ))
( "tribunals"  n ( t r ai1 b y uu1 n l z ) ((pos "Kj%" ) ))
( "tribune"  n ( t r i1 b y uu n ) ((pos "K6%" ) ))
( "tribunes"  n ( t r i1 b y uu n z ) ((pos "Kj%" ) ))
( "tributaries"  n ( t r i1 b y u t @ r i z ) ((pos "Kj%" ) ))
( "tributary"  n ( t r i1 b y u t @ r ii ) ((pos "K8%" "OA%" ) ))
( "tribute"  n ( t r i1 b y uu t ) ((pos "M6%" ) ))
( "tributes"  n ( t r i1 b y uu t s ) ((pos "Mj%" ) ))
( "trice"  v ( t r ai1 s ) ((pos "H2$" "K6%" ) (subcat )))
( "triced"  v ( t r ai1 s t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "trices"  v ( t r ai1 s i z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "tricing"  v ( t r ai1 s i ng ) ((pos "Hb$" ) (subcat )))
( "trick"  v ( t r i1 k ) ((pos "H0%" "K6%" ) (subcat "6A" "14" )))
( "tricked"  v ( t r i1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "14" )))
( "trickery"  n ( t r i1 k @ r ii ) ((pos "L@%" ) ))
( "trickier"  j ( t r i1 k i@ r ) ((pos "Or%" ) ))
( "trickiest"  j ( t r i1 k i i s t ) ((pos "Os%" ) ))
( "trickiness"  n ( t r i1 k i n i s ) ((pos "L@%" ) ))
( "tricking"  v ( t r i1 k i ng ) ((pos "Hb%" ) (subcat "6A" "14" )))
( "trickle"  v ( t r i1 k l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "trickled"  v ( t r i1 k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "trickles"  v ( t r i1 k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "trickling"  v ( t r i1 k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "tricks"  v ( t r i1 k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "14" )))
( "trickster"  n ( t r i1 k s t @ r ) ((pos "K6%" ) ))
( "tricksters"  n ( t r i1 k s t @ z ) ((pos "Kj%" ) ))
( "tricksy"  j ( t r i1 k s ii ) ((pos "OA%" ) ))
( "tricky"  j ( t r i1 k ii ) ((pos "OD%" ) ))
( "tricolour"  n ( t r i1 k @ l @ r ) ((pos "K6$" ) ))
( "tricolours"  n ( t r i1 k @ l @ z ) ((pos "Kj$" ) ))
( "tricycle"  n ( t r ai1 s i k l ) ((pos "K6%" ) ))
( "tricycles"  n ( t r ai1 s i k l z ) ((pos "Kj%" ) ))
( "trident"  n ( t r ai1 d n t ) ((pos "K6%" ) ))
( "tridents"  n ( t r ai1 d n t s ) ((pos "Kj$" ) ))
( "tried"  v ( t r ai1 d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "3A" "6A" "6B" "6C" "7A" "10" "15A" )))
( "triennial"  n ( t r ai1 e1 n i@ l ) ((pos "K6$" "OA$" ) ))
( "triennials"  n ( t r ai1 e1 n i@ l z ) ((pos "Kj$" ) ))
( "trier"  n ( t r ai1 @ r ) ((pos "K6$" ) ))
( "triers"  n ( t r ai1 @ z ) ((pos "Kj$" ) ))
( "tries"  v ( t r ai1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "3A" "6A" "6B" "6C" "7A" "10" "15A" )))
( "trifle"  v ( t r ai1 f @ l ) ((pos "J2%" "M6%" ) (subcat "3A" )))
( "trifled"  v ( t r ai1 f @ l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "trifler"  n ( t r ai1 f l @ r ) ((pos "K6$" ) ))
( "triflers"  n ( t r ai1 f l @ z ) ((pos "Kj$" ) ))
( "trifles"  v ( t r ai1 f @ l z ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "trifling"  v ( t r ai1 f l i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" )))
( "trigger"  v ( t r i1 g @ r ) ((pos "H0%" "K6%" ) (subcat )))
( "trigger-happy"  j ( t r i1 g @ - h a p ii ) ((pos "OA%" ) ))
( "triggered"  v ( t r i1 g @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "triggering"  v ( t r i1 g @ r i ng ) ((pos "Hb$" ) (subcat )))
( "triggers"  v ( t r i1 g @ z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "trigonometry"  n ( t r i2 g @ n o1 m @ t r ii ) ((pos "L@%" ) ))
( "trilateral"  j ( t r ai2 l a1 t @ r @ l ) ((pos "OA$" ) ))
( "trilbies"  n ( t r i1 l b i z ) ((pos "Kj$" ) ))
( "trilby"  n ( t r i1 l b ii ) ((pos "K8%" ) ))
( "trill"  v ( t r i1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "trilled"  v ( t r i1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "trilling"  v ( t r i1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "trillion"  n ( t r i1 l i@ n ) ((pos "K6%" "OA%" ) ))
( "trillions"  n ( t r i1 l i@ n z ) ((pos "Kj%" ) ))
( "trillionth"  n ( t r i1 l i@ n th ) ((pos "K6%" "OA%" ) ))
( "trillionths"  n ( t r i1 l i@ n th s ) ((pos "Kj%" ) ))
( "trills"  v ( t r i1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "trilogies"  n ( t r i1 l @ jh i z ) ((pos "Kj%" ) ))
( "trilogy"  n ( t r i1 l @ jh ii ) ((pos "K8%" ) ))
( "trim"  v ( t r i1 m ) ((pos "J4%" "L@%" "OE%" ) (subcat "2A" "6A" "14" "15A" )))
( "trimaran"  n ( t r ai1 m @ r a n ) ((pos "K6%" ) ))
( "trimarans"  n ( t r ai1 m @ r a n z ) ((pos "Kj%" ) ))
( "trimly"  a ( t r i1 m l ii ) ((pos "Pu%" ) ))
( "trimmed"  v ( t r i1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "14" "15A" )))
( "trimmer"  n ( t r i1 m @ r ) ((pos "K6$" "Or%" ) ))
( "trimmers"  n ( t r i1 m @ z ) ((pos "Kj$" ) ))
( "trimmest"  j ( t r i1 m i s t ) ((pos "Os%" ) ))
( "trimming"  v ( t r i1 m i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "6A" "14" "15A" )))
( "trimmings"  n ( t r i1 m i ng z ) ((pos "Mj%" ) ))
( "trims"  v ( t r i1 m z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" "15A" )))
( "trinities"  n ( t r i1 n i t i z ) ((pos "Kj$" ) ))
( "trinitrotoluene"  n ( t r ai2 n ai2 t r ou t o1 l y u ii n ) ((pos "L@$" ) ))
( "trinity"  n ( t r i1 n i t ii ) ((pos "K8%" ) ))
( "trinket"  n ( t r i1 ng k i t ) ((pos "K6%" ) ))
( "trinkets"  n ( t r i1 ng k i t s ) ((pos "Kj%" ) ))
( "trio"  n ( t r ii1 ou ) ((pos "K6%" ) ))
( "trios"  n ( t r ii1 ou z ) ((pos "Kj%" ) ))
( "trip"  v ( t r i1 p ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "tripartite"  j ( t r ai2 p aa1 t ai t ) ((pos "OA%" ) ))
( "tripe"  n ( t r ai1 p ) ((pos "L@%" ) ))
( "triple"  v ( t r i1 p l ) ((pos "J2%" "OA%" ) (subcat "2A" )))
( "tripled"  v ( t r i1 p l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "triples"  v ( t r i1 p l z ) ((pos "Ja%" ) (subcat "2A" )))
( "triplet"  n ( t r i1 p l @ t ) ((pos "K6%" ) ))
( "triplets"  n ( t r i1 p l @ t s ) ((pos "Kj%" ) ))
( "triplex"  j ( t r i1 p l e k s ) ((pos "OA$" ) ))
( "triplicate"  n ( t r i1 p l i k @ t ) ((pos "K6$" "OA%" ) ))
( "triplicate"  v ( t r i1 p l i k ei t ) ((pos "H2$" ) (subcat )))
( "triplicated"  v ( t r i1 p l i k ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "triplicates"  n ( t r i1 p l i k @ t s ) ((pos "Kj$" ) ))
( "triplicates"  v ( t r i1 p l i k ei t s ) ((pos "Ha$" ) (subcat )))
( "triplicating"  v ( t r i1 p l i k ei t i ng ) ((pos "Hb$" ) (subcat )))
( "tripling"  v ( t r i1 p l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "tripod"  n ( t r ai1 p o d ) ((pos "K6%" ) ))
( "tripods"  n ( t r ai1 p o d z ) ((pos "Kj%" ) ))
( "tripos"  n ( t r ai1 p o s ) ((pos "K7$" ) ))
( "triposes"  n ( t r ai1 p o s i z ) ((pos "Kj$" ) ))
( "tripped"  v ( t r i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "tripper"  n ( t r i1 p @ r ) ((pos "K6%" ) ))
( "trippers"  n ( t r i1 p @ z ) ((pos "Kj%" ) ))
( "tripping"  v ( t r i1 p i ng ) ((pos "Jb%" "OA$" ) (subcat "2A" "2C" "3A" )))
( "trippingly"  a ( t r i1 p i ng l ii ) ((pos "Pu$" ) ))
( "trips"  v ( t r i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "triptych"  n ( t r i1 p t i k ) ((pos "K6$" ) ))
( "triptychs"  n ( t r i1 p t i k s ) ((pos "Kj$" ) ))
( "trireme"  n ( t r ai1 r ii m ) ((pos "K6$" ) ))
( "triremes"  n ( t r ai1 r ii m z ) ((pos "Kj$" ) ))
( "trisect"  v ( t r ai1 s e1 k t ) ((pos "H0$" ) (subcat )))
( "trisected"  v ( t r ai1 s e1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "trisecting"  v ( t r ai1 s e1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "trisects"  v ( t r ai1 s e1 k t s ) ((pos "Ha$" ) (subcat )))
( "trite"  j ( t r ai1 t ) ((pos "OA%" ) ))
( "tritely"  a ( t r ai1 t l ii ) ((pos "Pu%" ) ))
( "triteness"  n ( t r ai1 t n @ s ) ((pos "L@%" ) ))
( "triumph"  v ( t r ai1 @ m f ) ((pos "I0%" "M6%" ) (subcat "2A" )))
( "triumphal"  j ( t r ai1 uh1 m f @ l ) ((pos "OA%" ) ))
( "triumphant"  j ( t r ai1 uh1 m f @ n t ) ((pos "OA%" ) ))
( "triumphantly"  a ( t r ai1 uh1 m f @ n t l ii ) ((pos "Pu%" ) ))
( "triumphed"  v ( t r ai1 @ m f t ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "triumphing"  v ( t r ai1 @ m f i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "triumphs"  v ( t r ai1 @ m f s ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "triumvir"  n ( t r ai1 uh1 m v @ r ) ((pos "K6$" ) ))
( "triumvirate"  n ( t r ai1 uh1 m v i r @ t ) ((pos "K6%" ) ))
( "triumvirates"  n ( t r ai1 uh1 m v i r @ t s ) ((pos "Kj$" ) ))
( "triumvirs"  n ( t r ai1 uh1 m v @ z ) ((pos "Kj$" ) ))
( "triune"  j ( t r ai1 y uu n ) ((pos "OA$" ) ))
( "trivet"  n ( t r i1 v i t ) ((pos "K6$" ) ))
( "trivets"  n ( t r i1 v i t s ) ((pos "Kj$" ) ))
( "trivia"  n ( t r i1 v i@ ) ((pos "Kj%" ) ))
( "trivial"  j ( t r i1 v i@ l ) ((pos "OA%" ) ))
( "trivialities"  n ( t r i2 v i a1 l i t i z ) ((pos "Mj%" ) ))
( "triviality"  n ( t r i2 v i a1 l i t ii ) ((pos "M8%" ) ))
( "trivialize"  v ( t r i1 v i@ l ai z ) ((pos "H2%" ) (subcat )))
( "trivialized"  v ( t r i1 v i@ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "trivializes"  v ( t r i1 v i@ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "trivializing"  v ( t r i1 v i@ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "trivially"  a ( t r i1 v i@ l ii ) ((pos "Pu%" ) ))
( "trochaic"  j ( t r ou1 k ei1 i k ) ((pos "OA$" ) ))
( "trochee"  n ( t r ou1 k ii ) ((pos "K6$" ) ))
( "trochees"  n ( t r ou1 k ii z ) ((pos "Kj$" ) ))
( "trod"  v ( t r o1 d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "trodden"  v ( t r o1 d n ) ((pos "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "troglodyte"  n ( t r o1 g l @ d ai t ) ((pos "K6$" ) ))
( "troglodytes"  n ( t r o1 g l @ d ai t s ) ((pos "Kj$" ) ))
( "troika"  n ( t r oi1 k @ ) ((pos "K6$" ) ))
( "troikas"  n ( t r oi1 k @ z ) ((pos "Kj$" ) ))
( "troll"  v ( t r ou1 l ) ((pos "J0$" "K6$" ) (subcat "2A" )))
( "trolled"  v ( t r ou1 l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "trolley"  n ( t r o1 l ii ) ((pos "K6%" ) ))
( "trolley-car"  n ( t r o1 l i - k aa r ) ((pos "K6%" ) ))
( "trolley-cars"  n ( t r o1 l i - k aa z ) ((pos "Kj%" ) ))
( "trolleys"  n ( t r o1 l i z ) ((pos "Kj%" ) ))
( "trolling"  v ( t r ou1 l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "trollop"  n ( t r o1 l @ p ) ((pos "K6%" ) ))
( "trollops"  n ( t r o1 l @ p s ) ((pos "Kj%" ) ))
( "trolls"  v ( t r ou1 l z ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "trombone"  n ( t r o1 m b ou1 n ) ((pos "K6%" ) ))
( "trombones"  n ( t r o1 m b ou1 n z ) ((pos "Kj%" ) ))
( "trombonist"  n ( t r o1 m b ou1 n i s t ) ((pos "K6%" ) ))
( "trombonists"  n ( t r o1 m b ou1 n i s t s ) ((pos "Kj%" ) ))
( "troop"  v ( t r uu1 p ) ((pos "J0%" "K6%" ) (subcat )))
( "troop-carrier"  n ( t r uu1 p - k a r i@ r ) ((pos "K6%" ) ))
( "troop-carriers"  n ( t r uu1 p - k a r i@ z ) ((pos "Kj%" ) ))
( "trooped"  v ( t r uu1 p t ) ((pos "Jc%" "Jd%" ) (subcat )))
( "trooper"  n ( t r uu1 p @ r ) ((pos "K6%" ) ))
( "troopers"  n ( t r uu1 p @ z ) ((pos "Kj%" ) ))
( "trooping"  v ( t r uu1 p i ng ) ((pos "Jb%" ) (subcat )))
( "troops"  v ( t r uu1 p s ) ((pos "Ja%" "Kj%" ) (subcat )))
( "troopship"  n ( t r uu1 p sh i p ) ((pos "K6%" ) ))
( "troopships"  n ( t r uu1 p sh i p s ) ((pos "Kj%" ) ))
( "trope"  n ( t r ou1 p ) ((pos "K6$" ) ))
( "tropes"  n ( t r ou1 p s ) ((pos "Kj$" ) ))
( "trophies"  n ( t r ou1 f i z ) ((pos "Kj%" ) ))
( "trophy"  n ( t r ou1 f ii ) ((pos "K8%" ) ))
( "tropic"  n ( t r o1 p i k ) ((pos "K6%" ) ))
( "tropical"  j ( t r o1 p i k l ) ((pos "OA%" ) ))
( "tropically"  a ( t r o1 p i k l ii ) ((pos "Pu%" ) ))
( "tropics"  n ( t r o1 p i k s ) ((pos "Kj%" ) ))
( "trot"  v ( t r o1 t ) ((pos "J4%" "K6%" ) (subcat "2A" "2B" "2C" "15A" )))
( "troth"  n ( t r ou1 th ) ((pos "L@$" ) ))
( "trots"  v ( t r o1 t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "15A" )))
( "trotted"  v ( t r o1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "15A" )))
( "trotter"  n ( t r o1 t @ r ) ((pos "K6%" ) ))
( "trotters"  n ( t r o1 t @ z ) ((pos "Kj%" ) ))
( "trotting"  v ( t r o1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "15A" )))
( "troubadour"  n ( t r uu1 b @ d oo r ) ((pos "K6%" ) ))
( "troubadours"  n ( t r uu1 b @ d oo z ) ((pos "Kj%" ) ))
( "trouble"  v ( t r uh1 b l ) ((pos "J2%" "M6%" ) (subcat "2A" "2C" "4A" "6A" "14" )))
( "troubled"  v ( t r uh1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" "6A" "14" )))
( "troublemaker"  n ( t r uh1 b l m ei k @ r ) ((pos "K6%" ) ))
( "troublemakers"  n ( t r uh1 b l m ei k @ z ) ((pos "Kj%" ) ))
( "troubles"  v ( t r uh1 b l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "4A" "6A" "14" )))
( "troubleshooter"  n ( t r uh1 b l sh uu t @ r ) ((pos "K6%" ) ))
( "troubleshooters"  n ( t r uh1 b l sh uu t @ z ) ((pos "Kj%" ) ))
( "troublesome"  j ( t r uh1 b l s @ m ) ((pos "OA%" ) ))
( "troubling"  v ( t r uh1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" "6A" "14" )))
( "troublous"  j ( t r uh1 b l @ s ) ((pos "OA$" ) ))
( "trough"  n ( t r o1 f ) ((pos "K6%" ) ))
( "troughs"  n ( t r o1 f s ) ((pos "Kj%" ) ))
( "trounce"  v ( t r au1 n s ) ((pos "H2%" ) (subcat )))
( "trounced"  v ( t r au1 n s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "trounces"  v ( t r au1 n s i z ) ((pos "Ha%" ) (subcat )))
( "trouncing"  v ( t r au1 n s i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "trouncings"  n ( t r au1 n s i ng z ) ((pos "Kj$" ) ))
( "troupe"  n ( t r uu1 p ) ((pos "K6%" ) ))
( "trouper"  n ( t r uu1 p @ r ) ((pos "K6%" ) ))
( "troupers"  n ( t r uu1 p @ z ) ((pos "Kj%" ) ))
( "troupes"  n ( t r uu1 p s ) ((pos "Kj%" ) ))
( "trouser"  n ( t r au1 z @ r ) ((pos "K6%" ) ))
( "trousers"  n ( t r au1 z @ z ) ((pos "Kj%" ) ))
( "trousseau"  n ( t r uu1 s ou ) ((pos "K6%" ) ))
( "trousseaus"  n ( t r uu1 s ou z ) ((pos "Kj$" ) ))
( "trousseaux"  n ( t r uu1 s ou z ) ((pos "Kj$" ) ))
( "trout"  n ( t r au1 t ) ((pos "M9%" ) ))
( "trove"  n ( t r ou1 v ) ((pos "K6%" ) ))
( "troves"  n ( t r ou1 v z ) ((pos "Kj$" ) ))
( "trowel"  n ( t r au1 @ l ) ((pos "K6%" ) ))
( "trowels"  n ( t r au1 @ l z ) ((pos "Kj%" ) ))
( "troy"  n ( t r oi1 ) ((pos "L@$" ) ))
( "truancies"  n ( t r uu1 @ n s i z ) ((pos "Mj$" ) ))
( "truancy"  n ( t r uu1 @ n s ii ) ((pos "M8%" ) ))
( "truant"  n ( t r uu1 @ n t ) ((pos "K6%" ) ))
( "truants"  n ( t r uu1 @ n t s ) ((pos "Kj%" ) ))
( "truce"  n ( t r uu1 s ) ((pos "K6%" ) ))
( "truces"  n ( t r uu1 s i z ) ((pos "Kj$" ) ))
( "truck"  n ( t r uh1 k ) ((pos "M6%" ) ))
( "truckle"  v ( t r uh1 k l ) ((pos "I2$" "K6$" ) (subcat )))
( "truckle-bed"  n ( t r uh1 k l - b e d ) ((pos "K6%" ) ))
( "truckle-beds"  n ( t r uh1 k l - b e d z ) ((pos "Kj%" ) ))
( "truckled"  v ( t r uh1 k l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "truckles"  v ( t r uh1 k l z ) ((pos "Ia$" "Kj$" ) (subcat )))
( "truckling"  v ( t r uh1 k l i ng ) ((pos "Ib$" ) (subcat )))
( "trucks"  n ( t r uh1 k s ) ((pos "Mj%" ) ))
( "truculence"  n ( t r uh1 k y u l @ n s ) ((pos "L@%" ) ))
( "truculency"  n ( t r uh1 k y u l @ n s ii ) ((pos "L@$" ) ))
( "truculent"  j ( t r uh1 k y u l @ n t ) ((pos "OA%" ) ))
( "truculently"  a ( t r uh1 k y u l @ n t l ii ) ((pos "Pu%" ) ))
( "trudge"  v ( t r uh1 jh ) ((pos "I2%" "K6%" ) (subcat "2A" "2B" )))
( "trudged"  v ( t r uh1 jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2B" )))
( "trudges"  v ( t r uh1 jh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2B" )))
( "trudging"  v ( t r uh1 jh i ng ) ((pos "Ib%" ) (subcat "2A" "2B" )))
( "true"  v ( t r uu1 ) ((pos "H2$" "K6$" "OB*" "Pu*" ) (subcat )))
( "true-blue"  n ( t r uu1 - b l uu1 ) ((pos "K6%" "OA%" ) ))
( "true-blues"  n ( t r uu1 - b l uu1 z ) ((pos "Kj%" ) ))
( "true-hearted"  j ( t r uu1 - h aa1 t i d ) ((pos "OA$" ) ))
( "trued"  v ( t r uu1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "truelove"  n ( t r uu1 l uh1 v ) ((pos "K6%" ) ))
( "trueloves"  n ( t r uu1 l uh1 v z ) ((pos "Kj$" ) ))
( "truer"  j ( t r uu1 @ r ) ((pos "Or%" ) ))
( "trues"  v ( t r uu1 z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "truest"  j ( t r uu1 i s t ) ((pos "Os%" ) ))
( "truffle"  n ( t r uh1 f l ) ((pos "K6%" ) ))
( "truffles"  n ( t r uh1 f l z ) ((pos "Kj%" ) ))
( "truing"  v ( t r uu1 i ng ) ((pos "Hb$" ) (subcat )))
( "truism"  n ( t r uu1 i z @ m ) ((pos "K6%" ) ))
( "truisms"  n ( t r uu1 i z @ m z ) ((pos "Kj%" ) ))
( "truly"  a ( t r uu1 l ii ) ((pos "Pu%" ) ))
( "trump"  v ( t r uh1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "trumped"  v ( t r uh1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "trumpery"  j ( t r uh1 m p @ r ii ) ((pos "OA%" ) ))
( "trumpet"  v ( t r uh1 m p i t ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "trumpeted"  v ( t r uh1 m p i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "trumpeter"  n ( t r uh1 m p i t @ r ) ((pos "K6%" ) ))
( "trumpeters"  n ( t r uh1 m p i t @ z ) ((pos "Kj%" ) ))
( "trumpeting"  v ( t r uh1 m p i t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "trumpets"  v ( t r uh1 m p i t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "trumping"  v ( t r uh1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "trumps"  v ( t r uh1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "truncate"  v ( t r uh1 ng k ei1 t ) ((pos "H2%" ) (subcat )))
( "truncated"  v ( t r uh1 ng k ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "truncates"  v ( t r uh1 ng k ei1 t s ) ((pos "Ha%" ) (subcat )))
( "truncating"  v ( t r uh1 ng k ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "truncheon"  n ( t r uh1 n ch @ n ) ((pos "K6%" ) ))
( "truncheons"  n ( t r uh1 n ch @ n z ) ((pos "Kj%" ) ))
( "trundle"  v ( t r uh1 n d l ) ((pos "J2%" ) (subcat "2C" "6A" "15A" )))
( "trundled"  v ( t r uh1 n d l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "trundles"  v ( t r uh1 n d l z ) ((pos "Ja%" ) (subcat "2C" "6A" "15A" )))
( "trundling"  v ( t r uh1 n d l i ng ) ((pos "Jb%" ) (subcat "2C" "6A" "15A" )))
( "trunk"  n ( t r uh1 ng k ) ((pos "K6%" ) ))
( "trunk-call"  n ( t r uh1 ng k - k oo l ) ((pos "K6%" ) ))
( "trunk-calls"  n ( t r uh1 ng k - k oo l z ) ((pos "Kj%" ) ))
( "trunk-line"  n ( t r uh1 ng k - l ai n ) ((pos "K6$" ) ))
( "trunk-lines"  n ( t r uh1 ng k - l ai n z ) ((pos "Kj$" ) ))
( "trunk-road"  n ( t r uh1 ng k - r ou d ) ((pos "K6%" ) ))
( "trunk-roads"  n ( t r uh1 ng k - r ou d z ) ((pos "Kj%" ) ))
( "trunking"  n ( t r uh1 ng k i ng ) ((pos "L@%" ) ))
( "trunks"  n ( t r uh1 ng k s ) ((pos "Kj%" ) ))
( "truss"  v ( t r uh1 s ) ((pos "H1$" "K7%" ) (subcat "6A" )))
( "trussed"  v ( t r uh1 s t ) ((pos "Hc$" "Hd%" ) (subcat "6A" )))
( "trusses"  v ( t r uh1 s i z ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "trussing"  v ( t r uh1 s i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "trust"  v ( t r uh1 s t ) ((pos "J0%" "M6%" ) (subcat "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "trusted"  v ( t r uh1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "trustee"  n ( t r uh1 s t ii1 ) ((pos "K6%" ) ))
( "trustees"  n ( t r uh1 s t ii1 z ) ((pos "Kj%" ) ))
( "trusteeship"  n ( t r uh1 s t ii1 sh i p ) ((pos "K6$" ) ))
( "trusteeships"  n ( t r uh1 s t ii1 sh i p s ) ((pos "Kj$" ) ))
( "trustful"  j ( t r uh1 s t f @ l ) ((pos "OA%" ) ))
( "trustfully"  a ( t r uh1 s t f @ l ii ) ((pos "Pu%" ) ))
( "trustier"  j ( t r uh1 s t i@ r ) ((pos "Or$" ) ))
( "trustiest"  j ( t r uh1 s t i i s t ) ((pos "Os$" ) ))
( "trusting"  v ( t r uh1 s t i ng ) ((pos "Jb%" "OA%" ) (subcat "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "trustingly"  a ( t r uh1 s t i ng l ii ) ((pos "Pu%" ) ))
( "trusts"  v ( t r uh1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "7A" "9" "14" "15A" "15B" )))
( "trustworthiness"  n ( t r uh1 s t w @@ dh i n @ s ) ((pos "L@%" ) ))
( "trustworthy"  j ( t r uh1 s t w @@ dh ii ) ((pos "OA%" ) ))
( "trusty"  j ( t r uh1 s t ii ) ((pos "OD%" ) ))
( "truth"  n ( t r uu1 th ) ((pos "M6%" ) ))
( "truthful"  j ( t r uu1 th f @ l ) ((pos "OA%" ) ))
( "truthfully"  a ( t r uu1 th f @ l ii ) ((pos "Pu%" ) ))
( "truthfulness"  n ( t r uu1 th f @ l n @ s ) ((pos "L@%" ) ))
( "truths"  n ( t r uu1 dh z ) ((pos "Mj%" ) ))
( "try"  v ( t r ai1 ) ((pos "J3%" "K8%" ) (subcat "2A" "2B" "3A" "6A" "6B" "6C" "7A" "10" "15A" )))
( "try-on"  n ( t r ai1 - o n ) ((pos "K6%" ) ))
( "try-ons"  n ( t r ai1 - o n z ) ((pos "Kj$" ) ))
( "try-out"  n ( t r ai1 - au t ) ((pos "K6%" ) ))
( "try-outs"  n ( t r ai1 - au t s ) ((pos "Kj$" ) ))
( "trying"  v ( t r ai1 i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "3A" "6A" "6B" "6C" "7A" "10" "15A" )))
( "tryst"  n ( t r i1 s t ) ((pos "K6$" ) ))
( "trysts"  n ( t r i1 s t s ) ((pos "Kj$" ) ))
( "tsetse"  n ( t s e1 t s ii ) ((pos "K6$" ) ))
( "tsetse-flies"  n ( t s e1 t s i - f l ai z ) ((pos "Kj%" ) ))
( "tsetse-fly"  n ( t s e1 t s i - f l ai ) ((pos "K8%" ) ))
( "tsetses"  n ( t s e1 t s i z ) ((pos "Kj$" ) ))
( "tu quoque"  uh ( t y uu2 - k w ou1 k w ii ) ((pos "W-$" ) ))
( "tub"  n ( t uh1 b ) ((pos "K6%" ) ))
( "tub-thumper"  n ( t uh1 b - th uh m p @ r ) ((pos "K6%" ) ))
( "tub-thumpers"  n ( t uh1 b - th uh m p @ z ) ((pos "Kj%" ) ))
( "tuba"  n ( t y uu1 b @ ) ((pos "K6%" ) ))
( "tubas"  n ( t y uu1 b @ z ) ((pos "Kj%" ) ))
( "tubbier"  j ( t uh1 b i@ r ) ((pos "Or$" ) ))
( "tubbiest"  j ( t uh1 b i i s t ) ((pos "Os$" ) ))
( "tubby"  j ( t uh1 b ii ) ((pos "OD%" ) ))
( "tube"  n ( t y uu1 b ) ((pos "M6%" ) ))
( "tube-well"  n ( t y uu1 b - w e l ) ((pos "K6$" ) ))
( "tube-wells"  n ( t y uu1 b - w e l z ) ((pos "Kj$" ) ))
( "tubeless"  j ( t y uu1 b l @ s ) ((pos "OA%" ) ))
( "tuber"  n ( t y uu1 b @ r ) ((pos "K6$" ) ))
( "tubercular"  j ( t y uu1 b @@1 k y u l @ r ) ((pos "OA%" ) ))
( "tuberculosis"  n ( t y uu1 b @@2 k y u l ou1 s i s ) ((pos "L@%" ) ))
( "tuberculous"  j ( t y uu1 b @@1 k y u l @ s ) ((pos "OA%" ) ))
( "tubers"  n ( t y uu1 b @ z ) ((pos "Kj$" ) ))
( "tubes"  n ( t y uu1 b z ) ((pos "Mj%" ) ))
( "tubful"  n ( t uh1 b f u l ) ((pos "K6%" ) ))
( "tubfuls"  n ( t uh1 b f u l z ) ((pos "Kj%" ) ))
( "tubing"  n ( t y uu1 b i ng ) ((pos "L@%" ) ))
( "tubs"  n ( t uh1 b z ) ((pos "Kj%" ) ))
( "tubular"  j ( t y uu1 b y u l @ r ) ((pos "OA%" ) ))
( "tuck"  v ( t uh1 k ) ((pos "J0%" "M6%" ) (subcat "2C" "3A" "15A" )))
( "tuck-in"  n ( t uh1 k - i n ) ((pos "K6%" ) ))
( "tuck-ins"  n ( t uh1 k - i n z ) ((pos "Kj$" ) ))
( "tuck-shop"  n ( t uh1 k - sh o p ) ((pos "K6%" ) ))
( "tuck-shops"  n ( t uh1 k - sh o p s ) ((pos "Kj%" ) ))
( "tucked"  v ( t uh1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "15A" )))
( "tucker"  n ( t uh1 k @ r ) ((pos "K6$" ) ))
( "tuckers"  n ( t uh1 k @ z ) ((pos "Kj$" ) ))
( "tucking"  v ( t uh1 k i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "15A" )))
( "tucks"  v ( t uh1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2C" "3A" "15A" )))
( "tuft"  n ( t uh1 f t ) ((pos "K6%" ) ))
( "tufted"  j ( t uh1 f t i d ) ((pos "OA%" ) ))
( "tufts"  n ( t uh1 f t s ) ((pos "Kj%" ) ))
( "tug"  v ( t uh1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tugboat"  n ( t uh1 g b ou t ) ((pos "K6%" ) ))
( "tugboats"  n ( t uh1 g b ou t s ) ((pos "Kj%" ) ))
( "tugged"  v ( t uh1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tugging"  v ( t uh1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tugs"  v ( t uh1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tuition"  n ( t y uu1 i1 sh @ n ) ((pos "L@%" ) ))
( "tulip"  n ( t y uu1 l i p ) ((pos "K6%" ) ))
( "tulips"  n ( t y uu1 l i p s ) ((pos "Kj%" ) ))
( "tulle"  n ( t y uu1 l ) ((pos "L@$" ) ))
( "tumble"  v ( t uh1 m b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tumble-down"  j ( t uh1 m b l - d au n ) ((pos "Oq%" ) ))
( "tumbled"  v ( t uh1 m b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tumbler"  n ( t uh1 m b l @ r ) ((pos "K6%" ) ))
( "tumblers"  n ( t uh1 m b l @ z ) ((pos "Kj%" ) ))
( "tumbles"  v ( t uh1 m b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tumbleweed"  n ( t uh1 m b l w ii d ) ((pos "L@$" ) ))
( "tumbling"  v ( t uh1 m b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15A" )))
( "tumbrel"  n ( t uh1 m b r @ l ) ((pos "K6$" ) ))
( "tumbrels"  n ( t uh1 m b r @ l z ) ((pos "Kj$" ) ))
( "tumbril"  n ( t uh1 m b r @ l ) ((pos "K6$" ) ))
( "tumbrils"  n ( t uh1 m b r @ l z ) ((pos "Kj$" ) ))
( "tumescence"  n ( t y uu1 m e1 s n s ) ((pos "M6%" ) ))
( "tumescences"  n ( t y uu1 m e1 s n s i z ) ((pos "Mj$" ) ))
( "tumescent"  j ( t y uu1 m e1 s n t ) ((pos "OA%" ) ))
( "tumid"  j ( t y uu1 m i d ) ((pos "OA$" ) ))
( "tumidity"  n ( t y uu1 m i1 d i t ii ) ((pos "L@$" ) ))
( "tummies"  n ( t uh1 m i z ) ((pos "Kj%" ) ))
( "tummy"  n ( t uh1 m ii ) ((pos "K8%" ) ))
( "tumour"  n ( t y uu1 m @ r ) ((pos "K6%" ) ))
( "tumours"  n ( t y uu1 m @ z ) ((pos "Kj%" ) ))
( "tumuli"  n ( t y uu1 m y u l ai ) ((pos "Kj$" ) ))
( "tumult"  n ( t y uu1 m uh l t ) ((pos "M6%" ) ))
( "tumults"  n ( t y uu1 m uh l t s ) ((pos "Mj%" ) ))
( "tumultuous"  j ( t y uu1 m uh1 l ch u@ s ) ((pos "OA%" ) ))
( "tumultuously"  a ( t y uu1 m uh1 l ch u@ s l ii ) ((pos "Pu%" ) ))
( "tumulus"  n ( t y uu1 m y u l @ s ) ((pos "Ki$" ) ))
( "tun"  n ( t uh1 n ) ((pos "K6$" ) ))
( "tuna"  n ( t y uu1 n @ ) ((pos "M9%" ) ))
( "tunas"  n ( t y uu1 n @ z ) ((pos "Kj$" ) ))
( "tundra"  n ( t uh1 n d r @ ) ((pos "M6%" ) ))
( "tundras"  n ( t uh1 n d r @ z ) ((pos "Mj$" ) ))
( "tune"  v ( t y uu1 n ) ((pos "J2%" "M6%" ) (subcat "2C" )))
( "tuned"  v ( t y uu1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2C" )))
( "tuneful"  j ( t y uu1 n f @ l ) ((pos "OA%" ) ))
( "tunefully"  a ( t y uu1 n f @ l ii ) ((pos "Pu%" ) ))
( "tunefulness"  n ( t y uu1 n f @ l n @ s ) ((pos "L@%" ) ))
( "tuner"  n ( t y uu1 n @ r ) ((pos "K6%" ) ))
( "tuners"  n ( t y uu1 n @ z ) ((pos "Kj%" ) ))
( "tunes"  v ( t y uu1 n z ) ((pos "Ja%" "Mj%" ) (subcat "2C" )))
( "tung-oil"  n ( t uh1 ng - oi l ) ((pos "L@$" ) ))
( "tungsten"  n ( t uh1 ng s t @ n ) ((pos "L@%" ) ))
( "tunic"  n ( t y uu1 n i k ) ((pos "K6%" ) ))
( "tunics"  n ( t y uu1 n i k s ) ((pos "Kj%" ) ))
( "tuning"  v ( t y uu1 n i ng ) ((pos "Jb%" ) (subcat "2C" )))
( "tuning-fork"  n ( t y uu1 n i ng - f oo k ) ((pos "K6%" ) ))
( "tuning-forks"  n ( t y uu1 n i ng - f oo k s ) ((pos "Kj%" ) ))
( "tunnel"  v ( t uh1 n l ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "tunnelled"  v ( t uh1 n l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "tunnelling"  v ( t uh1 n @ l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "tunnels"  v ( t uh1 n l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "tunnies"  n ( t uh1 n i z ) ((pos "Kj$" ) ))
( "tunny"  n ( t uh1 n ii ) ((pos "K9$" ) ))
( "tuns"  n ( t uh1 n z ) ((pos "Kj$" ) ))
( "tup"  n ( t uh1 p ) ((pos "K6$" ) ))
( "tuppence"  n ( t uh1 p @ n s ) ((pos "K6%" ) ))
( "tuppences"  n ( t uh1 p @ n s i z ) ((pos "Kj%" ) ))
( "tuppenny"  j ( t uh1 p @ n ii ) ((pos "OA%" ) ))
( "tups"  n ( t uh1 p s ) ((pos "Kj$" ) ))
( "turban"  n ( t @@1 b @ n ) ((pos "K6%" ) ))
( "turbaned"  j ( t @@1 b @ n d ) ((pos "OA%" ) ))
( "turbans"  n ( t @@1 b @ n z ) ((pos "Kj%" ) ))
( "turbid"  j ( t @@1 b i d ) ((pos "OA$" ) ))
( "turbidity"  n ( t @@1 b i1 d i t ii ) ((pos "L@$" ) ))
( "turbidness"  n ( t @@1 b i d n @ s ) ((pos "L@$" ) ))
( "turbine"  n ( t @@1 b ai n ) ((pos "K6%" ) ))
( "turbines"  n ( t @@1 b ai n z ) ((pos "Kj%" ) ))
( "turbojet"  n ( t @@2 b ou jh e1 t ) ((pos "K6%" ) ))
( "turbojets"  n ( t @@2 b ou jh e1 t s ) ((pos "Kj%" ) ))
( "turboprop"  n ( t @@2 b ou p r o1 p ) ((pos "K6%" ) ))
( "turboprops"  n ( t @@2 b ou p r o1 p s ) ((pos "Kj%" ) ))
( "turbot"  n ( t @@1 b @ t ) ((pos "M9%" ) ))
( "turbulence"  n ( t @@1 b y u l @ n s ) ((pos "M6%" ) ))
( "turbulences"  n ( t @@1 b y u l @ n s i z ) ((pos "Mj$" ) ))
( "turbulent"  j ( t @@1 b y u l @ n t ) ((pos "OA%" ) ))
( "turbulently"  a ( t @@1 b y u l @ n t l ii ) ((pos "Pu%" ) ))
( "turd"  n ( t @@1 d ) ((pos "K6%" ) ))
( "turds"  n ( t @@1 d z ) ((pos "Kj%" ) ))
( "tureen"  n ( t y u1 r ii1 n ) ((pos "K6%" ) ))
( "tureens"  n ( t y u1 r ii1 n z ) ((pos "Kj%" ) ))
( "turf"  v ( t @@1 f ) ((pos "H0$" "M6%" ) (subcat "6A" )))
( "turfed"  v ( t @@1 f t ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "turfing"  v ( t @@1 f i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "turfs"  v ( t @@1 f s ) ((pos "Ha$" "Mj$" ) (subcat "6A" )))
( "turgid"  j ( t @@1 jh i d ) ((pos "OA%" ) ))
( "turgidity"  n ( t @@1 jh i1 d i t ii ) ((pos "L@$" ) ))
( "turgidly"  a ( t @@1 jh i d l ii ) ((pos "Pu%" ) ))
( "turkey"  n ( t @@1 k ii ) ((pos "M6%" ) ))
( "turkeys"  n ( t @@1 k i z ) ((pos "Mj%" ) ))
( "turmeric"  n ( t @@1 m @ r i k ) ((pos "L@%" ) ))
( "turmoil"  n ( t @@1 m oi l ) ((pos "M6%" ) ))
( "turmoils"  n ( t @@1 m oi l z ) ((pos "Mj$" ) ))
( "turn"  v ( t @@1 n ) ((pos "J0*" "K6%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" )))
( "turn-off"  n ( t @@1 n - o f ) ((pos "K6%" ) ))
( "turn-offs"  n ( t @@1 n - o f s ) ((pos "Kj$" ) ))
( "turn-on"  n ( t @@1 n - o n ) ((pos "K6%" ) ))
( "turn-ons"  n ( t @@1 n - o n z ) ((pos "Kj$" ) ))
( "turn-out"  n ( t @@1 n - au t ) ((pos "K6%" ) ))
( "turn-outs"  n ( t @@1 n - au t s ) ((pos "Kj$" ) ))
( "turn-round"  n ( t @@1 n - r au n d ) ((pos "K6%" ) ))
( "turn-rounds"  n ( t @@1 n - r au n d z ) ((pos "Kj$" ) ))
( "turn-up"  n ( t @@1 n - uh p ) ((pos "K6%" ) ))
( "turn-ups"  n ( t @@1 n - uh p s ) ((pos "Kj%" ) ))
( "turncoat"  n ( t @@1 n k ou t ) ((pos "K6%" ) ))
( "turncoats"  n ( t @@1 n k ou t s ) ((pos "Kj%" ) ))
( "turncock"  n ( t @@1 n k o k ) ((pos "K6$" ) ))
( "turncocks"  n ( t @@1 n k o k s ) ((pos "Kj$" ) ))
( "turned"  v ( t @@1 n d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" )))
( "turner"  n ( t @@1 n @ r ) ((pos "K6%" ) ))
( "turners"  n ( t @@1 n @ z ) ((pos "Kj%" ) ))
( "turning"  v ( t @@1 n i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" )))
( "turning-point"  n ( t @@1 n i ng - p oi n t ) ((pos "K6%" ) ))
( "turning-points"  n ( t @@1 n i ng - p oi n t s ) ((pos "Kj%" ) ))
( "turnings"  n ( t @@1 n i ng z ) ((pos "Kj%" ) ))
( "turnip"  n ( t @@1 n i p ) ((pos "K6%" ) ))
( "turnips"  n ( t @@1 n i p s ) ((pos "Kj%" ) ))
( "turnkey"  n ( t @@1 n k ii ) ((pos "K6%" ) ))
( "turnkeys"  n ( t @@1 n k ii z ) ((pos "Kj%" ) ))
( "turnover"  n ( t @@1 n ou v @ r ) ((pos "K6%" ) ))
( "turnovers"  n ( t @@1 n ou v @ z ) ((pos "Kj%" ) ))
( "turnpike"  n ( t @@1 n p ai k ) ((pos "K6%" ) ))
( "turnpikes"  n ( t @@1 n p ai k s ) ((pos "Kj%" ) ))
( "turns"  v ( t @@1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "4A" "6A" "14" "15A" )))
( "turnspit"  n ( t @@1 n s p i t ) ((pos "K6$" ) ))
( "turnspits"  n ( t @@1 n s p i t s ) ((pos "Kj$" ) ))
( "turnstile"  n ( t @@1 n s t ai l ) ((pos "K6%" ) ))
( "turnstiles"  n ( t @@1 n s t ai l z ) ((pos "Kj%" ) ))
( "turntable"  n ( t @@1 n t ei b l ) ((pos "K6%" ) ))
( "turntables"  n ( t @@1 n t ei b l z ) ((pos "Kj%" ) ))
( "turpentine"  n ( t @@1 p @ n t ai n ) ((pos "L@%" ) ))
( "turpitude"  n ( t @@1 p i t y uu d ) ((pos "L@%" ) ))
( "turps"  n ( t @@1 p s ) ((pos "L@%" ) ))
( "turquoise"  n ( t @@1 k w oi z ) ((pos "M6%" ) ))
( "turquoises"  n ( t @@1 k w oi z i z ) ((pos "Mj$" ) ))
( "turret"  n ( t uh1 r i t ) ((pos "K6%" ) ))
( "turrets"  n ( t uh1 r i t s ) ((pos "Kj%" ) ))
( "turtle"  n ( t @@1 t l ) ((pos "K6%" ) ))
( "turtledove"  n ( t @@1 t l d uh v ) ((pos "K6%" ) ))
( "turtledoves"  n ( t @@1 t l d uh v z ) ((pos "Kj%" ) ))
( "turtleneck"  j ( t @@1 t l n e k ) ((pos "OA%" ) ))
( "turtlenecked"  j ( t @@1 t l n e k t ) ((pos "OA%" ) ))
( "turtles"  n ( t @@1 t l z ) ((pos "Kj%" ) ))
( "turves"  n ( t @@1 v z ) ((pos "Kj$" ) ))
( "tusk"  n ( t uh1 s k ) ((pos "K6%" ) ))
( "tusks"  n ( t uh1 s k s ) ((pos "Kj%" ) ))
( "tussle"  v ( t uh1 s l ) ((pos "I2%" "K6%" ) ))
( "tussled"  v ( t uh1 s l d ) ((pos "Ic%" "Id%" ) ))
( "tussles"  v ( t uh1 s l z ) ((pos "Ia%" "Kj%" ) ))
( "tussling"  v ( t uh1 s l i ng ) ((pos "Ib%" ) ))
( "tussock"  n ( t uh1 s @ k ) ((pos "K6%" ) ))
( "tussocks"  n ( t uh1 s @ k s ) ((pos "Kj%" ) ))
( "tut"  v ( t uh1 t ) ((pos "H4$" "W-$" ) (subcat )))
( "tut-tut"  uh ( t uh2 t - t uh1 t ) ((pos "W-%" ) ))
( "tutelage"  n ( t y uu1 t i l i jh ) ((pos "L@%" ) ))
( "tutelary"  j ( t y uu1 t i l @ r ii ) ((pos "OA$" ) ))
( "tutor"  v ( t y uu1 t @ r ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "tutored"  v ( t y uu1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "tutorial"  n ( t y uu1 t oo1 r i@ l ) ((pos "K6%" "OA%" ) ))
( "tutorially"  a ( t y uu1 t oo1 r i@ l ii ) ((pos "Pu%" ) ))
( "tutorials"  n ( t y uu1 t oo1 r i@ l z ) ((pos "Kj%" ) ))
( "tutoring"  v ( t y uu1 t @ r i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "tutors"  v ( t y uu1 t @ z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "tutorship"  n ( t y uu1 t @ sh i p ) ((pos "K6%" ) ))
( "tutorships"  n ( t y uu1 t @ sh i p s ) ((pos "Kj$" ) ))
( "tuts"  v ( t uh1 t s ) ((pos "Ha$" ) (subcat )))
( "tutted"  v ( t uh1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "tutti-frutti"  n ( t uu2 t i - f r uu1 t ii ) ((pos "M6%" ) ))
( "tutti-fruttis"  n ( t uu2 t i - f r uu1 t i z ) ((pos "Mj$" ) ))
( "tutting"  v ( t uh1 t i ng ) ((pos "Hb$" ) (subcat )))
( "tutu"  n ( t uu1 t uu ) ((pos "K6$" ) ))
( "tutus"  n ( t uu1 t uu z ) ((pos "Kj$" ) ))
( "tuxedo"  n ( t uh1 k s ii1 d ou ) ((pos "K6%" ) ))
( "tuxedos"  n ( t uh1 k s ii1 d ou z ) ((pos "Kj%" ) ))
( "twaddle"  v ( t w o1 d l ) ((pos "I2$" "L@%" ) (subcat )))
( "twaddled"  v ( t w o1 d l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "twaddles"  v ( t w o1 d l z ) ((pos "Ia$" ) (subcat )))
( "twaddling"  v ( t w o1 d l i ng ) ((pos "Ib$" ) (subcat )))
( "twain"  n ( t w ei1 n ) ((pos "K6%" ) ))
( "twains"  n ( t w ei1 n z ) ((pos "Kj$" ) ))
( "twang"  v ( t w a1 ng ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "twanged"  v ( t w a1 ng d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "twanging"  v ( t w a1 ng i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "twangs"  v ( t w a1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "tweak"  v ( t w ii1 k ) ((pos "H0%" "K6%" ) (subcat )))
( "tweaked"  v ( t w ii1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "tweaking"  v ( t w ii1 k i ng ) ((pos "Hb%" ) (subcat )))
( "tweaks"  v ( t w ii1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "twee"  j ( t w ii1 ) ((pos "OA%" ) ))
( "tweed"  n ( t w ii1 d ) ((pos "M6%" ) ))
( "tweeds"  n ( t w ii1 d z ) ((pos "Mj%" ) ))
( "tweet"  v ( t w ii1 t ) ((pos "I0$" "K6$" ) ))
( "tweeted"  v ( t w ii1 t i d ) ((pos "Ic$" "Id$" ) ))
( "tweeter"  n ( t w ii1 t @ r ) ((pos "K6$" ) ))
( "tweeters"  n ( t w ii1 t @ z ) ((pos "Kj$" ) ))
( "tweeting"  v ( t w ii1 t i ng ) ((pos "Ib$" ) ))
( "tweets"  v ( t w ii1 t s ) ((pos "Ia$" "Kj$" ) ))
( "tweezers"  n ( t w ii1 z @ z ) ((pos "Kj%" ) ))
( "twelfth"  n ( t w e1 l f th ) ((pos "K6%" "OA%" ) ))
( "twelfths"  n ( t w e1 l f th s ) ((pos "Kj%" ) ))
( "twelve"  n ( t w e1 l v ) ((pos "K6%" "OA%" ) ))
( "twelvemonth"  n ( t w e1 l v m uh n th ) ((pos "K6%" ) ))
( "twelvemonths"  n ( t w e1 l v m uh n th s ) ((pos "Kj$" ) ))
( "twelves"  n ( t w e1 l v z ) ((pos "Kj%" ) ))
( "twenties"  n ( t w e1 n t i z ) ((pos "Kj%" ) ))
( "twentieth"  n ( t w e1 n t i@ th ) ((pos "K6%" "OA%" ) ))
( "twentieths"  n ( t w e1 n t i@ th s ) ((pos "Kj%" ) ))
( "twenty"  n ( t w e1 n t ii ) ((pos "K8%" "OA%" ) ))
( "twerp"  n ( t w @@1 p ) ((pos "K6%" ) ))
( "twerps"  n ( t w @@1 p s ) ((pos "Kj%" ) ))
( "twice"  a ( t w ai1 s ) ((pos "Pu%" ) ))
( "twiddle"  v ( t w i1 d l ) ((pos "J2%" "K6$" ) (subcat "3A" )))
( "twiddled"  v ( t w i1 d l d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "twiddles"  v ( t w i1 d l z ) ((pos "Ja%" "Kj$" ) (subcat "3A" )))
( "twiddling"  v ( t w i1 d l i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "twiddly"  j ( t w i1 d l ii ) ((pos "OA%" ) ))
( "twig"  v ( t w i1 g ) ((pos "J4%" "K6%" ) (subcat "2A" )))
( "twigged"  v ( t w i1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "twiggier"  j ( t w i1 g i@ r ) ((pos "Or$" ) ))
( "twiggiest"  j ( t w i1 g i i s t ) ((pos "Os$" ) ))
( "twigging"  v ( t w i1 g i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "twiggy"  j ( t w i1 g ii ) ((pos "OD%" ) ))
( "twigs"  v ( t w i1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "twilight"  n ( t w ai1 l ai t ) ((pos "L@%" ) ))
( "twilit"  j ( t w ai1 l i t ) ((pos "OA$" ) ))
( "twill"  n ( t w i1 l ) ((pos "L@$" ) ))
( "twilled"  j ( t w i1 l d ) ((pos "OA$" ) ))
( "twin"  v ( t w i1 n ) ((pos "H4%" "K6%" ) (subcat "6A" )))
( "twine"  v ( t w ai1 n ) ((pos "J2%" "L@%" ) (subcat "2A" "2C" "15A" )))
( "twined"  v ( t w ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "15A" )))
( "twines"  v ( t w ai1 n z ) ((pos "Ja%" ) (subcat "2A" "2C" "15A" )))
( "twinge"  n ( t w i1 n jh ) ((pos "K6%" ) ))
( "twinges"  n ( t w i1 n jh i z ) ((pos "Kj%" ) ))
( "twining"  v ( t w ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "15A" )))
( "twinkle"  v ( t w i1 ng k l ) ((pos "I2%" "L@%" ) (subcat "2A" )))
( "twinkled"  v ( t w i1 ng k l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "twinkles"  v ( t w i1 ng k l z ) ((pos "Ia%" ) (subcat "2A" )))
( "twinkling"  v ( t w i1 ng k l i ng ) ((pos "Ib%" "Ki%" ) (subcat "2A" )))
( "twinned"  v ( t w i1 n d ) ((pos "Hc%" "Hd%" "Oq%" ) (subcat "6A" )))
( "twinning"  v ( t w i1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "twins"  v ( t w i1 n z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "twirl"  v ( t w @@1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15B" )))
( "twirled"  v ( t w @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15B" )))
( "twirling"  v ( t w @@1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15B" )))
( "twirls"  v ( t w @@1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15B" )))
( "twist"  v ( t w i1 s t ) ((pos "J0%" "M6%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "twisted"  v ( t w i1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "twister"  n ( t w i1 s t @ r ) ((pos "K6%" ) ))
( "twisters"  n ( t w i1 s t @ z ) ((pos "Kj%" ) ))
( "twistier"  j ( t w i1 s t i@ r ) ((pos "Or$" ) ))
( "twistiest"  j ( t w i1 s t i i s t ) ((pos "Os$" ) ))
( "twisting"  v ( t w i1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "twists"  v ( t w i1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "twisty"  j ( t w i1 s t ii ) ((pos "OD%" ) ))
( "twit"  v ( t w i1 t ) ((pos "H4$" "K6%" ) (subcat "6A" )))
( "twitch"  v ( t w i1 ch ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" "6A" "15A" )))
( "twitched"  v ( t w i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "twitches"  v ( t w i1 ch i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "twitching"  v ( t w i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "twits"  v ( t w i1 t s ) ((pos "Ha$" "Kj%" ) (subcat "6A" )))
( "twitted"  v ( t w i1 t i d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "twitter"  v ( t w i1 t @ r ) ((pos "I0%" "K6%" ) (subcat "15A" )))
( "twittered"  v ( t w i1 t @ d ) ((pos "Ic%" "Id%" ) (subcat "15A" )))
( "twittering"  v ( t w i1 t @ r i ng ) ((pos "Ib%" ) (subcat "15A" )))
( "twitters"  v ( t w i1 t @ z ) ((pos "Ia%" "Kj%" ) (subcat "15A" )))
( "twitting"  v ( t w i1 t i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "two"  n ( t uu1 ) ((pos "K6%" "OA*" ) ))
( "two-a-penny"  j ( t uu2 - @ - p e1 n ii ) ((pos "OA%" ) ))
( "two-edged"  j ( t uu1 - e1 jh d ) ((pos "OA%" ) ))
( "two-faced"  j ( t uu1 - f ei1 s t ) ((pos "OA%" ) ))
( "two-funnelled"  j ( t uu1 - f uh1 n @ l d ) ((pos "OA%" ) ))
( "two-handed"  j ( t uu1 - h a1 n d i d ) ((pos "OA%" ) ))
( "two-piece"  n ( t uu1 - p ii s ) ((pos "Ki%" ) ))
( "two-ply"  j ( t uu1 - p l ai ) ((pos "OA%" ) ))
( "two-seater"  n ( t uu1 - s ii1 t @ r ) ((pos "K6%" ) ))
( "two-seaters"  n ( t uu1 - s ii1 t @ z ) ((pos "Kj%" ) ))
( "two-step"  n ( t uu1 - s t e p ) ((pos "K6%" ) ))
( "two-steps"  n ( t uu1 - s t e p s ) ((pos "Kj%" ) ))
( "two-timing"  j ( t uu1 - t ai m i ng ) ((pos "OA%" ) ))
( "two-way"  j ( t uu1 - w ei ) ((pos "Oq%" ) ))
( "twofold"  j ( t uu1 f ou l d ) ((pos "OA%" "Pu%" ) ))
( "twopence"  n ( t uh1 p @ n s ) ((pos "K6%" ) ))
( "twopences"  n ( t uh1 p @ n s i z ) ((pos "Kj%" ) ))
( "twopenny"  j ( t uh1 p @ n ii ) ((pos "OA%" ) ))
( "twopenny-halfpenny"  j ( t uh2 p n i - h ei1 p n ii ) ((pos "OA%" ) ))
( "twos"  n ( t uu1 z ) ((pos "Kj%" ) ))
( "tycoon"  n ( t ai1 k uu1 n ) ((pos "K6%" ) ))
( "tycoons"  n ( t ai1 k uu1 n z ) ((pos "Kj%" ) ))
( "tying"  v ( t ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "15A" )))
( "tyke"  n ( t ai1 k ) ((pos "K6$" ) ))
( "tykes"  n ( t ai1 k s ) ((pos "Kj$" ) ))
( "tympana"  n ( t i1 m p @ n @ ) ((pos "Kj$" ) ))
( "tympanum"  n ( t i1 m p @ n @ m ) ((pos "K6$" ) ))
( "tympanums"  n ( t i1 m p @ n @ m z ) ((pos "Kj$" ) ))
( "type"  v ( t ai1 p ) ((pos "J2%" "M6%" ) (subcat "2A" )))
( "typecast"  v ( t ai1 p k aa s t ) ((pos "H5$" "Hc$" "Hd%" ) (subcat )))
( "typecasting"  v ( t ai1 p k aa s t i ng ) ((pos "Hb$" ) (subcat )))
( "typecasts"  v ( t ai1 p k aa s t s ) ((pos "Ha$" ) (subcat )))
( "typed"  v ( t ai1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "typeface"  n ( t ai1 p f ei s ) ((pos "K6%" ) ))
( "typefaces"  n ( t ai1 p f ei s i z ) ((pos "Kj%" ) ))
( "types"  v ( t ai1 p s ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "typescript"  n ( t ai1 p s k r i p t ) ((pos "K6%" ) ))
( "typescripts"  n ( t ai1 p s k r i p t s ) ((pos "Kj%" ) ))
( "typesetter"  n ( t ai1 p s e t @ r ) ((pos "K6%" ) ))
( "typesetters"  n ( t ai1 p s e t @ z ) ((pos "Kj%" ) ))
( "typewriter"  n ( t ai1 p r ai t @ r ) ((pos "K6%" ) ))
( "typewriters"  n ( t ai1 p r ai t @ z ) ((pos "Kj%" ) ))
( "typewritten"  j ( t ai1 p r i t n ) ((pos "OA%" ) ))
( "typhoid"  n ( t ai1 f oi d ) ((pos "L@%" ) ))
( "typhoon"  n ( t ai1 f uu1 n ) ((pos "K6%" ) ))
( "typhoons"  n ( t ai1 f uu1 n z ) ((pos "Kj%" ) ))
( "typhus"  n ( t ai1 f @ s ) ((pos "L@%" ) ))
( "typical"  j ( t i1 p i k l ) ((pos "OA%" ) ))
( "typically"  a ( t i1 p i k l ii ) ((pos "Pu%" ) ))
( "typified"  v ( t i1 p i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "typifies"  v ( t i1 p i f ai z ) ((pos "Ha%" ) (subcat )))
( "typify"  v ( t i1 p i f ai ) ((pos "H3%" ) (subcat )))
( "typifying"  v ( t i1 p i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "typing"  v ( t ai1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "typist"  n ( t ai1 p i s t ) ((pos "K6%" ) ))
( "typists"  n ( t ai1 p i s t s ) ((pos "Kj%" ) ))
( "typographer"  n ( t ai1 p o1 g r @ f @ r ) ((pos "K6%" ) ))
( "typographers"  n ( t ai1 p o1 g r @ f @ z ) ((pos "Kj%" ) ))
( "typographic"  j ( t ai2 p @ g r a1 f i k ) ((pos "OA%" ) ))
( "typographically"  a ( t ai2 p @ g r a1 f i k l ii ) ((pos "Pu%" ) ))
( "typography"  n ( t ai1 p o1 g r @ f ii ) ((pos "L@%" ) ))
( "tyrannical"  j ( t i1 r a1 n i k l ) ((pos "OA%" ) ))
( "tyrannies"  n ( t i1 r @ n i z ) ((pos "Mj%" ) ))
( "tyrannize"  v ( t i1 r @ n ai z ) ((pos "J2%" ) (subcat "3A" )))
( "tyrannized"  v ( t i1 r @ n ai z d ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "tyrannizes"  v ( t i1 r @ n ai z i z ) ((pos "Ja%" ) (subcat "3A" )))
( "tyrannizing"  v ( t i1 r @ n ai z i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "tyrannous"  j ( t i1 r @ n @ s ) ((pos "OA%" ) ))
( "tyranny"  n ( t i1 r @ n ii ) ((pos "M8%" ) ))
( "tyrant"  n ( t ai1 @ r @ n t ) ((pos "K6%" ) ))
( "tyrants"  n ( t ai1 @ r @ n t s ) ((pos "Kj%" ) ))
( "tyre"  n ( t ai1 @ r ) ((pos "K6%" ) ))
( "tyres"  n ( t ai1 @ z ) ((pos "Kj%" ) ))
( "tyro"  n ( t ai1 @ r ou ) ((pos "K6$" ) ))
( "tyros"  n ( t ai1 @ r ou z ) ((pos "Kj$" ) ))
( "tzar"  n ( z aa1 r ) ((pos "K6%" ) ))
( "tzarina"  n ( z aa1 r ii1 n @ ) ((pos "K6%" ) ))
( "tzarinas"  n ( z aa1 r ii1 n @ z ) ((pos "Kj$" ) ))
( "tzars"  n ( z aa1 z ) ((pos "Kj$" ) ))
( "u"  n ( y uu1 ) ((pos "Ki$" ) ))
( "u's"  n ( y uu1 z ) ((pos "Kj$" ) ))
( "ubiquitous"  j ( y uu1 b i1 k w i t @ s ) ((pos "OA%" ) ))
( "ubiquity"  n ( y uu1 b i1 k w i t ii ) ((pos "L@$" ) ))
( "udder"  n ( uh1 d @ r ) ((pos "K6%" ) ))
( "udders"  n ( uh1 d @ z ) ((pos "Kj%" ) ))
( "ugh"  uh ( @@1 ) ((pos "W-$" ) ))
( "uglier"  j ( uh1 g l i@ r ) ((pos "Or%" ) ))
( "ugliest"  j ( uh1 g l i i s t ) ((pos "Os%" ) ))
( "uglified"  v ( uh1 g l i f ai d ) ((pos "Hc$" "Hd$" ) ))
( "uglifies"  v ( uh1 g l i f ai z ) ((pos "Ha$" ) ))
( "uglify"  v ( uh1 g l i f ai ) ((pos "H3$" ) ))
( "uglifying"  v ( uh1 g l i f ai i ng ) ((pos "Hb$" ) ))
( "ugliness"  n ( uh1 g l i n @ s ) ((pos "L@%" ) ))
( "ugly"  j ( uh1 g l ii ) ((pos "OD%" ) ))
( "ukase"  n ( y uu1 k ei1 s ) ((pos "K6$" ) ))
( "ukases"  n ( y uu1 k ei1 s i z ) ((pos "Kj$" ) ))
( "ukulele"  n ( y uu2 k @ l ei1 l ii ) ((pos "K6%" ) ))
( "ukuleles"  n ( y uu2 k @ l ei1 l i z ) ((pos "Kj%" ) ))
( "ulcer"  n ( uh1 l s @ r ) ((pos "K6%" ) ))
( "ulcerate"  v ( uh1 l s @ r ei t ) ((pos "J2%" ) (subcat "2A" )))
( "ulcerated"  v ( uh1 l s @ r ei t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "ulcerates"  v ( uh1 l s @ r ei t s ) ((pos "Ja%" ) (subcat "2A" )))
( "ulcerating"  v ( uh1 l s @ r ei t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "ulceration"  n ( uh2 l s @ r ei1 sh @ n ) ((pos "K6%" ) ))
( "ulcerations"  n ( uh2 l s @ r ei1 sh @ n z ) ((pos "Kj%" ) ))
( "ulcerous"  j ( uh1 l s @ r @ s ) ((pos "OA%" ) ))
( "ulcers"  n ( uh1 l s @ z ) ((pos "Kj%" ) ))
( "ulna"  n ( uh1 l n @ ) ((pos "Ki%" ) ))
( "ulnae"  n ( uh1 l n ii ) ((pos "Kj$" ) ))
( "ulster"  n ( uh1 l s t @ r ) ((pos "K6$" ) ))
( "ulsters"  n ( uh1 l s t @ z ) ((pos "Kj$" ) ))
( "ulterior"  j ( uh1 l t i@1 r i@ r ) ((pos "OA%" ) ))
( "ultimata"  n ( uh2 l t i m ei1 t @ ) ((pos "Kj$" ) ))
( "ultimate"  j ( uh1 l t i m @ t ) ((pos "OA%" ) ))
( "ultimately"  a ( uh1 l t i m @ t l ii ) ((pos "Pu%" ) ))
( "ultimatum"  n ( uh2 l t i m ei1 t @ m ) ((pos "K6%" ) ))
( "ultimatums"  n ( uh2 l t i m ei1 t @ m z ) ((pos "Kj%" ) ))
( "ultimo"  j ( uh1 l t i m ou ) ((pos "OA$" ) ))
( "ultra vires"  j ( uh2 l t r @ - v ai1 @ r ii z ) ((pos "OA$" "Pu$" ) ))
( "ultra-"  j ( uh2 l t r @ - ) ((pos "U-%" ) ))
( "ultramarine"  n ( uh2 l t r @ m @ r ii1 n ) ((pos "K6$" "OA%" ) ))
( "ultramarines"  n ( uh2 l t r @ m @ r ii1 n z ) ((pos "Kj$" ) ))
( "ultramontane"  j ( uh2 l t r @ m o n t ei1 n ) ((pos "OA$" ) ))
( "ultrasonic"  j ( uh2 l t r @ s o1 n i k ) ((pos "OA%" ) ))
( "ultraviolet"  j ( uh2 l t r @ v ai1 @ l @ t ) ((pos "OA%" ) ))
( "ululate"  v ( y uu1 l y u l ei t ) ((pos "I2$" ) (subcat )))
( "ululated"  v ( y uu1 l y u l ei t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "ululates"  v ( y uu1 l y u l ei t s ) ((pos "Ia$" ) (subcat )))
( "ululating"  v ( y uu1 l y u l ei t i ng ) ((pos "Ib$" ) (subcat )))
( "ululation"  n ( y uu2 l y u l ei1 sh @ n ) ((pos "K6$" ) ))
( "ululations"  n ( y uu2 l y u l ei1 sh @ n z ) ((pos "Kj$" ) ))
( "umber"  n ( uh1 m b @ r ) ((pos "K6$" "OA$" ) ))
( "umbers"  n ( uh1 m b @ z ) ((pos "Kj$" ) ))
( "umbilical"  j ( uh1 m b i1 l i k l ) ((pos "OA%" ) ))
( "umbrage"  n ( uh1 m b r i jh ) ((pos "L@%" ) ))
( "umbrella"  n ( uh1 m b r e1 l @ ) ((pos "K6%" ) ))
( "umbrellas"  n ( uh1 m b r e1 l @ z ) ((pos "Kj%" ) ))
( "umlaut"  n ( u1 m l au t ) ((pos "K6$" ) ))
( "umlauts"  n ( u1 m l au t s ) ((pos "Kj$" ) ))
( "umpire"  v ( uh1 m p ai @ r ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "umpired"  v ( uh1 m p ai @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "umpires"  v ( uh1 m p ai @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "umpiring"  v ( uh1 m p ai @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "umpteen"  j ( uh1 m p t ii n ) ((pos "OA%" ) ))
( "umpteenth"  j ( uh1 m p t ii n th ) ((pos "OA%" ) ))
( "un-"  j ( uh n - ) ((pos "U-%" ) ))
( "un-come-at-able"  j ( uh2 n - k uh m - a1 t - @ b l ) ((pos "OA$" ) ))
( "un-get-at-able"  j ( uh2 n - g e t - a1 t - @ b l ) ((pos "OA$" ) ))
( "unabashed"  j ( uh2 n @ b a1 sh t ) ((pos "OA%" ) ))
( "unabated"  j ( uh2 n @ b ei1 t i d ) ((pos "OA%" ) ))
( "unable"  j ( uh1 n ei1 b l ) ((pos "Op%" ) ))
( "unabridged"  j ( uh2 n @ b r i1 jh d ) ((pos "OA%" ) ))
( "unacceptable"  j ( uh2 n @ k s e1 p t @ b l ) ((pos "OA%" ) ))
( "unacceptably"  a ( uh2 n @ k s e1 p t @ b l ii ) ((pos "Pu%" ) ))
( "unaccommodating"  j ( uh2 n @ k o1 m @ d ei t i ng ) ((pos "OA%" ) ))
( "unaccompanied"  j ( uh2 n @ k uh1 m p @ n i d ) ((pos "OA%" ) ))
( "unaccountable"  j ( uh2 n @ k au1 n t @ b l ) ((pos "OA%" ) ))
( "unaccountably"  a ( uh2 n @ k au1 n t @ b l ii ) ((pos "Pu%" ) ))
( "unaccounted"  j ( uh2 n @ k au1 n t i d ) ((pos "OA%" ) ))
( "unaccustomed"  j ( uh2 n @ k uh1 s t @ m d ) ((pos "OA%" ) ))
( "unacknowledged"  j ( uh2 n @ k n o1 l i jh d ) ((pos "OA%" ) ))
( "unacquainted"  j ( uh2 n @ k w ei1 n t i d ) ((pos "OA%" ) ))
( "unadorned"  j ( uh2 n @ d oo1 n d ) ((pos "OA%" ) ))
( "unadulterated"  j ( uh2 n @ d uh1 l t @ r ei t i d ) ((pos "OA%" ) ))
( "unadventurous"  j ( uh2 n @ d v e1 n ch @ r @ s ) ((pos "OA%" ) ))
( "unadvised"  j ( uh2 n @ d v ai1 z d ) ((pos "OA$" ) ))
( "unadvisedly"  a ( uh2 n @ d v ai1 z @ d l ii ) ((pos "Pu%" ) ))
( "unaffected"  j ( uh2 n @ f e1 k t i d ) ((pos "OA%" ) ))
( "unafraid"  j ( uh2 n @ f r ei1 d ) ((pos "Op%" ) ))
( "unaided"  j ( uh1 n ei1 d i d ) ((pos "OA%" ) ))
( "unalienable"  j ( uh1 n ei1 l i@ n @ b l ) ((pos "OA$" ) ))
( "unaligned"  j ( uh2 n @ l ai1 n d ) ((pos "OA%" ) ))
( "unalloyed"  j ( uh2 n @ l oi1 d ) ((pos "OA%" ) ))
( "unalterable"  j ( uh1 n o1 l t @ r @ b l ) ((pos "OA%" ) ))
( "unalterably"  a ( uh1 n oo1 l t @ r @ b l ii ) ((pos "Pu%" ) ))
( "unaltered"  j ( uh1 n oo1 l t @ d ) ((pos "OA%" ) ))
( "unambiguous"  j ( uh2 n a m b i1 g y u@ s ) ((pos "OA%" ) ))
( "unambiguously"  a ( uh2 n a m b i1 g y u@ s l ii ) ((pos "Pu%" ) ))
( "unamended"  j ( uh2 n @ m e1 n d i d ) ((pos "OA%" ) ))
( "unanimity"  n ( y uu2 n @ n i1 m i t ii ) ((pos "L@%" ) ))
( "unanimous"  j ( y uu1 n a1 n i m @ s ) ((pos "OA%" ) ))
( "unanimously"  a ( y uu1 n a1 n i m @ s l ii ) ((pos "Pu%" ) ))
( "unannounced"  j ( uh2 n @ n au1 n s t ) ((pos "OA%" ) ))
( "unanswerable"  j ( uh2 n aa1 n s @ r @ b l ) ((pos "OA%" ) ))
( "unanswered"  j ( uh2 n aa1 n s @ d ) ((pos "OA%" ) ))
( "unanticipated"  j ( uh2 n a n t i1 s i p ei t i d ) ((pos "OA%" ) ))
( "unappealing"  j ( uh2 n @ p ii1 l i ng ) ((pos "OA%" ) ))
( "unappetizing"  j ( uh1 n a1 p i t ai z i ng ) ((pos "OA%" ) ))
( "unappreciated"  j ( uh2 n @ p r ii1 sh i ei t i d ) ((pos "OA%" ) ))
( "unappreciative"  j ( uh2 n @ p r ii1 sh @ t i v ) ((pos "OA%" ) ))
( "unapproachable"  j ( uh2 n @ p r ou1 ch @ b l ) ((pos "OA%" ) ))
( "unarguable"  j ( uh1 n aa1 g y u@ b l ) ((pos "OA%" ) ))
( "unarguably"  a ( uh1 n aa1 g y u@ b l ii ) ((pos "Pu%" ) ))
( "unarmed"  j ( uh2 n aa1 m d ) ((pos "OA%" ) ))
( "unarticulated"  j ( uh2 n aa t i1 k y u l ei t i d ) ((pos "OA%" ) ))
( "unashamed"  j ( uh2 n @ sh ei1 m d ) ((pos "Op%" ) ))
( "unashamedly"  a ( uh2 n @ sh ei1 m i d l ii ) ((pos "Pu%" ) ))
( "unasked"  j ( uh2 n aa1 s k t ) ((pos "OA%" ) ))
( "unassailable"  j ( uh2 n @ s ei1 l @ b l ) ((pos "OA%" ) ))
( "unassailably"  a ( uh2 n @ s ei1 l @ b l ii ) ((pos "Pu%" ) ))
( "unassisted"  j ( uh2 n @ s i1 s t i d ) ((pos "OA%" ) ))
( "unassuming"  j ( uh2 n @ s y uu1 m i ng ) ((pos "OA%" ) ))
( "unassumingly"  a ( uh2 n @ s y uu1 m i ng l ii ) ((pos "Pu%" ) ))
( "unattached"  j ( uh2 n @ t a1 ch t ) ((pos "OA%" ) ))
( "unattainable"  j ( uh2 n @ t ai1 n @ b l ) ((pos "OA%" ) ))
( "unattainably"  a ( uh2 n @ t ei1 n @ b l ii ) ((pos "Pu%" ) ))
( "unattended"  j ( uh2 n @ t e1 n d i d ) ((pos "OA%" ) ))
( "unattractive"  j ( uh2 n @ t r a1 k t i v ) ((pos "OA%" ) ))
( "unattractively"  a ( uh2 n @ t r a1 k t i v l ii ) ((pos "Pu%" ) ))
( "unauthorized"  j ( uh1 n oo1 th @ r ai z d ) ((pos "OA%" ) ))
( "unavailable"  j ( uh2 n @ v ei1 l @ b l ) ((pos "OA%" ) ))
( "unavailing"  j ( uh2 n @ v ei1 l i ng ) ((pos "OA%" ) ))
( "unavoidable"  j ( uh2 n @ v oi1 d @ b l ) ((pos "OA%" ) ))
( "unavoidably"  a ( uh2 n @ v oi1 d @ b l ii ) ((pos "Pu%" ) ))
( "unawakened"  j ( uh2 n @ w ei1 k @ n d ) ((pos "OA%" ) ))
( "unaware"  j ( uh2 n @ w e@1 r ) ((pos "Op%" ) ))
( "unawares"  a ( uh2 n @ w e@1 z ) ((pos "Pu%" ) ))
( "unbacked"  j ( uh2 n b a1 k t ) ((pos "OA$" ) ))
( "unbalance"  v ( uh1 n b a1 l @ n s ) ((pos "H2%" ) (subcat )))
( "unbalanced"  j ( uh2 n b a1 l @ n s t ) ((pos "OA%" ) ))
( "unbalanced"  v ( uh1 n b a1 l @ n s t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "unbalances"  v ( uh1 n b a1 l @ n s i z ) ((pos "Ha%" ) (subcat )))
( "unbalancing"  v ( uh1 n b a1 l @ n s i ng ) ((pos "Hb%" ) (subcat )))
( "unbar"  v ( uh2 n b aa1 r ) ((pos "H4$" ) ))
( "unbarred"  v ( uh2 n b aa1 d ) ((pos "Hc$" "Hd%" ) ))
( "unbarring"  v ( uh2 n b aa1 r i ng ) ((pos "Hb$" ) ))
( "unbars"  v ( uh2 n b aa1 z ) ((pos "Ha$" ) ))
( "unbearable"  j ( uh1 n b e@1 r @ b l ) ((pos "OA%" ) ))
( "unbearably"  a ( uh1 n b e@1 r @ b l ii ) ((pos "Pu%" ) ))
( "unbeatable"  j ( uh1 n b ii1 t @ b l ) ((pos "OA%" ) ))
( "unbeaten"  j ( uh2 n b ii1 t n ) ((pos "OA%" ) ))
( "unbecoming"  j ( uh2 n b i k uh1 m i ng ) ((pos "OA%" ) ))
( "unbecomingly"  a ( uh2 n b i k uh1 m i ng l ii ) ((pos "Pu%" ) ))
( "unbeknown"  j ( uh2 n b i n ou1 n ) ((pos "OA%" "Pu%" ) ))
( "unbeknownst"  j ( uh2 n b i n ou1 n s t ) ((pos "OA%" "Pu%" ) ))
( "unbelief"  n ( uh2 n b i l ii1 f ) ((pos "L@%" ) ))
( "unbelievable"  j ( uh2 n b i l ii1 v @ b l ) ((pos "OA%" ) ))
( "unbelievably"  a ( uh2 n b i l ii1 v @ b l ii ) ((pos "Pu%" ) ))
( "unbeliever"  n ( uh2 n b i l ii1 v @ r ) ((pos "K6%" ) ))
( "unbelievers"  n ( uh2 n b i l ii1 v @ z ) ((pos "Kj%" ) ))
( "unbelieving"  j ( uh2 n b i l ii1 v i ng ) ((pos "OA%" ) ))
( "unbelievingly"  a ( uh2 n b i l ii1 v i ng l ii ) ((pos "Pu%" ) ))
( "unbeloved"  j ( uh2 n b i l uh1 v d ) ((pos "OA%" ) ))
( "unbend"  v ( uh2 n b e1 n d ) ((pos "J0%" ) (subcat "2A" )))
( "unbended"  v ( uh2 n b e1 n d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unbending"  v ( uh2 n b e1 n d i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" )))
( "unbends"  v ( uh2 n b e1 n d z ) ((pos "Ja%" ) (subcat "2A" )))
( "unbent"  v ( uh2 n b e1 n t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unbiased"  j ( uh2 n b ai1 @ s t ) ((pos "OA%" ) ))
( "unbiassed"  j ( uh2 n b ai1 @ s t ) ((pos "OA%" ) ))
( "unbidden"  j ( uh2 n b i1 d n ) ((pos "OA%" ) ))
( "unbind"  v ( uh2 n b ai1 n d ) ((pos "H5%" ) (subcat )))
( "unbinding"  v ( uh2 n b ai1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "unbinds"  v ( uh2 n b ai1 n d z ) ((pos "Ha%" ) (subcat )))
( "unblock"  v ( uh1 n b l o1 k ) ((pos "H0%" ) (subcat "6A" )))
( "unblocked"  v ( uh1 n b l o1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "unblocking"  v ( uh1 n b l o1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "unblocks"  v ( uh1 n b l o1 k s ) ((pos "Ha%" ) (subcat "6A" )))
( "unblushing"  j ( uh2 n b l uh1 sh i ng ) ((pos "OA%" ) ))
( "unblushingly"  a ( uh2 n b l uh1 sh i ng l ii ) ((pos "Pu%" ) ))
( "unblushingly"  a ( uh1 n b l uh1 sh i ng l ii ) ((pos "Pu%" ) ))
( "unbolt"  v ( uh1 n b ou1 l t ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "unbolted"  v ( uh1 n b ou1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "unbolting"  v ( uh1 n b ou1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "unbolts"  v ( uh1 n b ou1 l t s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "unborn"  j ( uh2 n b oo1 n ) ((pos "OA%" ) ))
( "unbosom"  v ( uh1 n b u1 z @ m ) ((pos "H0$" ) (subcat "6A" )))
( "unbosomed"  v ( uh1 n b u1 z @ m d ) ((pos "Hc$" "Hd$" ) (subcat "6A" )))
( "unbosoming"  v ( uh1 n b u1 z @ m i ng ) ((pos "Hb$" ) (subcat "6A" )))
( "unbosoms"  v ( uh1 n b u1 z @ m z ) ((pos "Ha$" ) (subcat "6A" )))
( "unbound"  v ( uh2 n b au1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unbounded"  j ( uh1 n b au1 n d i d ) ((pos "OA%" ) ))
( "unbowed"  j ( uh2 n b au1 d ) ((pos "OA%" ) ))
( "unbreakable"  j ( uh1 n b r ei1 k @ b l ) ((pos "OA%" ) ))
( "unbridled"  j ( uh2 n b r ai1 d l d ) ((pos "OA%" ) ))
( "unbroken"  j ( uh2 n b r ou1 k @ n ) ((pos "OA%" ) ))
( "unbuckle"  v ( uh2 n b uh1 k l ) ((pos "H2%" ) (subcat )))
( "unbuckled"  v ( uh2 n b uh1 k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unbuckles"  v ( uh2 n b uh1 k l z ) ((pos "Ha%" ) (subcat )))
( "unbuckling"  v ( uh2 n b uh1 k l i ng ) ((pos "Hb%" ) (subcat )))
( "unburden"  v ( uh2 n b @@1 d n ) ((pos "H0%" ) (subcat "6A" )))
( "unburdened"  v ( uh2 n b @@1 d n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "unburdening"  v ( uh2 n b @@1 d n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "unburdens"  v ( uh2 n b @@1 d n z ) ((pos "Ha%" ) (subcat "6A" )))
( "unburied"  j ( uh1 n b e1 r i d ) ((pos "OA%" ) ))
( "unbutton"  v ( uh1 n b uh1 t n ) ((pos "H0%" ) (subcat )))
( "unbuttoned"  j ( uh2 n b uh1 t n d ) ((pos "OA%" ) ))
( "unbuttoned"  v ( uh1 n b uh1 t n d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "unbuttoning"  v ( uh1 n b uh1 t n i ng ) ((pos "Hb%" ) (subcat )))
( "unbuttons"  v ( uh1 n b uh1 t n z ) ((pos "Ha%" ) (subcat )))
( "uncalled-for"  j ( uh1 n k oo1 l d - f oo r ) ((pos "OA%" ) ))
( "uncannily"  a ( uh1 n k a1 n i l ii ) ((pos "Pu%" ) ))
( "uncanny"  j ( uh1 n k a1 n ii ) ((pos "OA%" ) ))
( "uncared-for"  j ( uh2 n k e@1 d - f oo r ) ((pos "OA%" ) ))
( "uncarpeted"  j ( uh1 n k aa1 p i t i d ) ((pos "OA%" ) ))
( "unceasing"  j ( uh1 n s ii1 s i ng ) ((pos "OA%" ) ))
( "unceasingly"  a ( uh1 n s ii1 s i ng l ii ) ((pos "Pu%" ) ))
( "uncensored"  j ( uh1 n s e1 n s @ d ) ((pos "OA%" ) ))
( "unceremonious"  j ( uh2 n s e2 r i m ou1 n i@ s ) ((pos "OA%" ) ))
( "unceremoniously"  a ( uh2 n s e2 r i m ou1 n i@ s l ii ) ((pos "Pu%" ) ))
( "unceremoniousness"  n ( uh2 n s e2 r i m ou1 n i@ s n @ s ) ((pos "L@$" ) ))
( "uncertain"  j ( uh1 n s @@1 t n ) ((pos "OA%" ) ))
( "uncertainly"  a ( uh1 n s @@1 t n l ii ) ((pos "Pu%" ) ))
( "uncertainties"  n ( uh1 n s @@1 t n t i z ) ((pos "Mj%" ) ))
( "uncertainty"  n ( uh1 n s @@1 t n t ii ) ((pos "M8%" ) ))
( "unchain"  v ( uh1 n ch ei1 n ) ((pos "H0$" ) (subcat "6A" "15A" )))
( "unchained"  v ( uh1 n ch ei1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "unchaining"  v ( uh1 n ch ei1 n i ng ) ((pos "Hb$" ) (subcat "6A" "15A" )))
( "unchains"  v ( uh1 n ch ei1 n z ) ((pos "Ha$" ) (subcat "6A" "15A" )))
( "unchallengeable"  j ( uh1 n ch a1 l @ n jh @ b l ) ((pos "OA%" ) ))
( "unchallenged"  j ( uh1 n ch a1 l @ n jh d ) ((pos "OA%" ) ))
( "unchanged"  j ( uh1 n ch ei1 n jh d ) ((pos "OA%" ) ))
( "unchanging"  j ( uh1 n ch ei1 n jh i ng ) ((pos "OA%" ) ))
( "uncharacteristic"  j ( uh2 n k a r @ k t @ r i1 s t i k ) ((pos "OA%" ) ))
( "uncharacteristically"  a ( uh2 n k a r @ k t @ r i1 s t i k l ii ) ((pos "Pu%" ) ))
( "uncharitable"  j ( uh2 n ch a1 r i t @ b l ) ((pos "OA%" ) ))
( "uncharted"  j ( uh2 n ch aa1 t i d ) ((pos "OA%" ) ))
( "unchecked"  j ( uh2 n ch e1 k t ) ((pos "OA%" ) ))
( "unchivalrously"  a ( uh1 n sh i1 v @ l r @ s l ii ) ((pos "Pu%" ) ))
( "unchristian"  j ( uh2 n k r i1 s ch @ n ) ((pos "OA%" ) ))
( "uncivil"  j ( uh2 n s i1 v @ l ) ((pos "OA%" ) ))
( "uncivilized"  j ( uh1 n s i1 v i l ai z d ) ((pos "OA%" ) ))
( "unclaimed"  j ( uh2 n k l ei1 m d ) ((pos "OA%" ) ))
( "unclassified"  j ( uh1 n k l a1 s i f ai d ) ((pos "OA%" ) ))
( "uncle"  n ( uh1 ng k l ) ((pos "K6%" ) ))
( "unclean"  j ( uh2 n k l ii1 n ) ((pos "OA%" ) ))
( "unclear"  j ( uh1 n k l i@1 r ) ((pos "OA%" ) ))
( "uncles"  n ( uh1 ng k l z ) ((pos "Kj%" ) ))
( "unclouded"  j ( uh2 n k l au1 d i d ) ((pos "OA%" ) ))
( "uncluttered"  j ( uh1 n k l uh1 t @ d ) ((pos "OA%" ) ))
( "unco"  j ( uh1 ng k ou ) ((pos "OA$" "Pu$" ) ))
( "unco-operative"  j ( uh2 n k ou - o1 p @ r @ t i v ) ((pos "OA%" ) ))
( "unco-ordinated"  j ( uh2 n k ou - oo1 d i n ei t i d ) ((pos "OA%" ) ))
( "uncoloured"  j ( uh2 n k uh1 l @ d ) ((pos "OA%" ) ))
( "uncomfortable"  j ( uh1 n k uh1 m f t @ b l ) ((pos "OA%" ) ))
( "uncomfortably"  a ( uh1 n k uh1 m f @ t @ b l ii ) ((pos "Pu%" ) ))
( "uncommercialized"  j ( uh2 n k @ m @@1 sh @ l ai z d ) ((pos "OA%" ) ))
( "uncommitted"  j ( uh2 n k @ m i1 t i d ) ((pos "OA%" ) ))
( "uncommon"  j ( uh1 n k o1 m @ n ) ((pos "OA%" ) ))
( "uncommonly"  a ( uh1 n k o1 m @ n l ii ) ((pos "Pu%" ) ))
( "uncommunicative"  j ( uh2 n k @ m y uu1 n i k @ t i v ) ((pos "OA%" ) ))
( "uncompetitive"  j ( uh2 n k @ m p e1 t @ t i v ) ((pos "OA%" ) ))
( "uncomplaining"  j ( uh2 n k @ m p l ei1 n i ng ) ((pos "OA%" ) ))
( "uncomplainingly"  a ( uh2 n k @ m p l ei1 n i ng l ii ) ((pos "Pu%" ) ))
( "uncompleted"  j ( uh2 n k @ m p l ii1 t i d ) ((pos "OA%" ) ))
( "uncomplicated"  j ( uh1 n k o1 m p l i k ei t i d ) ((pos "OA%" ) ))
( "uncomplimentary"  j ( uh2 n k o m p l i m e1 n t r ii ) ((pos "OA%" ) ))
( "uncomprehending"  j ( uh2 n k o m p r i h e1 n d i ng ) ((pos "OA%" ) ))
( "uncompromising"  j ( uh1 n k o1 m p r @ m ai z i ng ) ((pos "OA%" ) ))
( "unconcealed"  j ( uh2 n k @ n s ii1 l d ) ((pos "OA%" ) ))
( "unconcern"  n ( uh2 n k @ n s @@1 n ) ((pos "L@%" ) ))
( "unconcerned"  j ( uh2 n k @ n s @@1 n d ) ((pos "OA%" ) ))
( "unconcernedly"  a ( uh2 n k @ n s @@1 n @ d l ii ) ((pos "Pu%" ) ))
( "unconditional"  j ( uh2 n k @ n d i1 sh @ n l ) ((pos "OA%" ) ))
( "unconditionally"  a ( uh2 n k @ n d i1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "unconditioned"  j ( uh2 n k @ n d i1 sh @ n d ) ((pos "OA%" ) ))
( "unconfined"  j ( uh2 n k @ n f ai1 n d ) ((pos "OA%" ) ))
( "unconfirmed"  j ( uh2 n k @ n f @@1 m d ) ((pos "OA%" ) ))
( "unconformable"  j ( uh2 n k @ n f oo1 m @ b l ) ((pos "OA%" ) ))
( "uncongenial"  j ( uh2 n k @ n jh ii1 n i@ l ) ((pos "OA%" ) ))
( "unconnected"  j ( uh2 n k @ n e1 k t i d ) ((pos "OA%" ) ))
( "unconquered"  j ( uh1 n k o1 ng k @ d ) ((pos "OA%" ) ))
( "unconscionable"  j ( uh1 n k o1 n sh @ n @ b l ) ((pos "OA%" ) ))
( "unconscious"  n ( uh1 n k o1 n sh @ s ) ((pos "K7%" "OA%" ) ))
( "unconsciouses"  n ( uh1 n k o1 n sh @ s i z ) ((pos "Kj$" ) ))
( "unconsciously"  a ( uh1 n k o1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "unconsciousness"  n ( uh1 n k o1 n sh @ s n @ s ) ((pos "L@%" ) ))
( "unconsidered"  j ( uh1 n k @ n s i d @ d ) ((pos "OA%" ) ))
( "unconstitutional"  j ( uh2 n k o n s t i t y uu1 sh @ n l ) ((pos "OA%" ) ))
( "unconstitutionally"  a ( uh2 n k o n s t i t y uu1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "unconstrained"  j ( uh2 n k @ n s t r ei1 n d ) ((pos "OA%" ) ))
( "unconstructive"  j ( uh2 n k @ n s t r uh1 k t i v ) ((pos "OA%" ) ))
( "unconsummated"  j ( uh1 n k o1 n s @ m ei t i d ) ((pos "OA%" ) ))
( "uncontaminated"  j ( uh2 n k @ n t a1 m i n ei t i d ) ((pos "OA%" ) ))
( "uncontrollable"  j ( uh2 n k @ n t r ou1 l @ b l ) ((pos "OA%" ) ))
( "uncontrollably"  a ( uh2 n k @ n t r ou1 l @ b l ii ) ((pos "Pu%" ) ))
( "uncontrolled"  j ( uh2 n k @ n t r ou1 l d ) ((pos "OA%" ) ))
( "uncontroversial"  j ( uh2 n k o n t r @ v @@1 sh l ) ((pos "OA%" ) ))
( "unconventional"  j ( uh2 n k @ n v e1 n sh @ n l ) ((pos "OA%" ) ))
( "unconventionality"  n ( uh2 n k @ n v e2 n sh @ n a1 l i t ii ) ((pos "L@%" ) ))
( "unconverted"  j ( uh2 n k @ n v @@1 t i d ) ((pos "OA%" ) ))
( "unconvinced"  j ( uh2 n k @ n v i1 n s t ) ((pos "OA%" ) ))
( "unconvincing"  j ( uh2 n k @ n v i1 n s i ng ) ((pos "OA%" ) ))
( "uncooked"  j ( uh1 n k u1 k t ) ((pos "OA%" ) ))
( "uncork"  v ( uh2 n k oo1 k ) ((pos "H0%" ) (subcat )))
( "uncorked"  v ( uh2 n k oo1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uncorking"  v ( uh2 n k oo1 k i ng ) ((pos "Hb%" ) (subcat )))
( "uncorks"  v ( uh2 n k oo1 k s ) ((pos "Ha%" ) (subcat )))
( "uncorrected"  j ( uh2 n k @ r e1 k t i d ) ((pos "OA%" ) ))
( "uncorrelated"  j ( uh1 n k o1 r @ l ei t i d ) ((pos "OA%" ) ))
( "uncorroborated"  j ( uh2 n k @ r o1 b @ r ei t i d ) ((pos "OA%" ) ))
( "uncouple"  v ( uh2 n k uh1 p l ) ((pos "H2%" ) (subcat )))
( "uncoupled"  v ( uh2 n k uh1 p l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uncouples"  v ( uh2 n k uh1 p l z ) ((pos "Ha%" ) (subcat )))
( "uncoupling"  v ( uh2 n k uh1 p l i ng ) ((pos "Hb%" ) (subcat )))
( "uncouth"  j ( uh1 n k uu1 th ) ((pos "OA%" ) ))
( "uncouthly"  a ( uh1 n k uu1 th l ii ) ((pos "Pu%" ) ))
( "uncouthness"  n ( uh1 n k uu1 th n @ s ) ((pos "L@%" ) ))
( "uncover"  v ( uh1 n k uh1 v @ r ) ((pos "H0%" ) (subcat "2A" )))
( "uncovered"  v ( uh1 n k uh1 v @ d ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "uncovering"  v ( uh1 n k uh1 v @ r i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "uncovers"  v ( uh1 n k uh1 v @ z ) ((pos "Ha%" ) (subcat "2A" )))
( "uncritical"  j ( uh1 n k r i1 t i k l ) ((pos "OA%" ) ))
( "uncritically"  a ( uh1 n k r i1 t i k l ii ) ((pos "Pu%" ) ))
( "uncross"  v ( uh1 n k r o1 s ) ((pos "H1%" ) (subcat )))
( "uncrossed"  j ( uh2 n k r o1 s t ) ((pos "OA%" ) ))
( "uncrossed"  v ( uh1 n k r o1 s t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "uncrosses"  v ( uh1 n k r o1 s i z ) ((pos "Ha%" ) (subcat )))
( "uncrossing"  v ( uh1 n k r o1 s i ng ) ((pos "Hb%" ) (subcat )))
( "uncrowned"  j ( uh2 n k r au1 n d ) ((pos "OA%" ) ))
( "unction"  n ( uh1 ng k sh @ n ) ((pos "L@%" ) ))
( "unctuous"  j ( uh1 ng k ch u@ s ) ((pos "OA%" ) ))
( "unctuously"  a ( uh1 ng k ch u@ s l ii ) ((pos "Pu%" ) ))
( "uncultivated"  j ( uh1 n k uh1 l t i v ei t i d ) ((pos "OA%" ) ))
( "uncultured"  j ( uh1 n k uh1 l ch @ d ) ((pos "OA%" ) ))
( "uncurl"  v ( uh1 n k @@1 l ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "15A" )))
( "uncurled"  v ( uh1 n k @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "uncurling"  v ( uh1 n k @@1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "uncurls"  v ( uh1 n k @@1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "uncut"  j ( uh2 n k uh1 t ) ((pos "OA%" ) ))
( "undamaged"  j ( uh1 n d a1 m i jh d ) ((pos "OA%" ) ))
( "undated"  j ( uh2 n d ei1 t i d ) ((pos "OA%" ) ))
( "undaunted"  j ( uh2 n d oo1 n t i d ) ((pos "OA%" ) ))
( "undeceive"  v ( uh2 n d i s ii1 v ) ((pos "H2$" ) (subcat )))
( "undeceived"  v ( uh2 n d i s ii1 v d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "undeceives"  v ( uh2 n d i s ii1 v z ) ((pos "Ha$" ) (subcat )))
( "undeceiving"  v ( uh2 n d i s ii1 v i ng ) ((pos "Hb$" ) (subcat )))
( "undecided"  j ( uh2 n d i s ai1 d i d ) ((pos "OA%" ) ))
( "undeclared"  j ( uh2 n d i k l e@1 d ) ((pos "OA%" ) ))
( "undefeated"  j ( uh2 n d i f ii1 t i d ) ((pos "OA%" ) ))
( "undefended"  j ( uh2 n d i f e1 n d i d ) ((pos "OA%" ) ))
( "undeferential"  j ( uh2 n d e f @ r e1 n sh l ) ((pos "OA%" ) ))
( "undefinable"  j ( uh2 n d i f ai1 n @ b l ) ((pos "OA%" ) ))
( "undefined"  j ( uh2 n d i f ai1 n d ) ((pos "OA%" ) ))
( "undemanding"  j ( uh2 n d i m aa1 n d i ng ) ((pos "OA%" ) ))
( "undemocratic"  j ( uh2 n d e m @ k r a1 t i k ) ((pos "OA%" ) ))
( "undemocratically"  a ( uh2 n d e m @ k r a1 t i k l ii ) ((pos "Pu%" ) ))
( "undemonstrative"  j ( uh2 n d i m o1 n s t r @ t i v ) ((pos "OA%" ) ))
( "undeniable"  j ( uh2 n d i n ai1 @ b l ) ((pos "OA%" ) ))
( "undeniably"  a ( uh2 n d i n ai1 @ b l ii ) ((pos "Pu%" ) ))
( "undenominational"  j ( uh2 n d i n o2 m i n ei1 sh @ n l ) ((pos "OA%" ) ))
( "under"  a ( uh1 n d @ r ) ((pos "Pu*" "T-*" ) ))
( "under-"  j ( uh1 n d @ r - ) ((pos "U-%" ) ))
( "under-the-counter"  j ( uh2 n d @ - dh @ - k au1 n t @ r ) ((pos "OA%" ) ))
( "underact"  v ( uh2 n d @ r a1 k t ) ((pos "J0%" ) (subcat "2A" )))
( "underacted"  v ( uh2 n d @ r a1 k t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "underacting"  v ( uh2 n d @ r a1 k t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "underacts"  v ( uh2 n d @ r a1 k t s ) ((pos "Ja%" ) (subcat "2A" )))
( "underarm"  j ( uh1 n d @ r aa m ) ((pos "OA%" "Pu%" ) ))
( "underbellies"  n ( uh1 n d @ b e l i z ) ((pos "Kj%" ) ))
( "underbelly"  n ( uh1 n d @ b e l ii ) ((pos "K8%" ) ))
( "underbid"  v ( uh2 n d @ b i1 d ) ((pos "H5%" "Hc%" "Hd%" ) ))
( "underbidding"  v ( uh2 n d @ b i1 d i ng ) ((pos "Hb%" ) ))
( "underbids"  v ( uh2 n d @ b i1 d z ) ((pos "Ha%" ) ))
( "underbred"  j ( uh2 n d @ b r e1 d ) ((pos "OA$" ) ))
( "underbrush"  n ( uh1 n d @ b r uh sh ) ((pos "L@$" ) ))
( "undercarriage"  n ( uh1 n d @ k a r i jh ) ((pos "K6%" ) ))
( "undercarriages"  n ( uh1 n d @ k a r i jh i z ) ((pos "Kj%" ) ))
( "undercharge"  n ( uh1 n d @ ch aa jh ) ((pos "K6$" ) ))
( "undercharge"  v ( uh2 n d @ ch aa1 jh ) ((pos "H2%" ) (subcat )))
( "undercharged"  v ( uh2 n d @ ch aa1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "undercharges"  n ( uh1 n d @ ch aa jh i z ) ((pos "Kj$" ) ))
( "undercharges"  v ( uh2 n d @ ch aa1 jh i z ) ((pos "Ha%" ) (subcat )))
( "undercharging"  v ( uh2 n d @ ch aa1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "underclothes"  n ( uh1 n d @ k l ou dh z ) ((pos "Kj%" ) ))
( "underclothing"  n ( uh1 n d @ k l ou dh i ng ) ((pos "L@%" ) ))
( "undercoat"  n ( uh1 n d @ k ou t ) ((pos "K6%" ) ))
( "undercoats"  n ( uh1 n d @ k ou t s ) ((pos "Kj%" ) ))
( "undercover"  j ( uh2 n d @ k uh1 v @ r ) ((pos "OA%" ) ))
( "undercurrent"  n ( uh1 n d @ k uh r @ n t ) ((pos "K6%" ) ))
( "undercurrents"  n ( uh1 n d @ k uh r @ n t s ) ((pos "Kj%" ) ))
( "undercut"  n ( uh1 n d @ k uh t ) ((pos "L@%" ) ))
( "undercut"  v ( uh2 n d @ k uh1 t ) ((pos "H5%" "Hc%" "Hd%" ) (subcat )))
( "undercuts"  v ( uh2 n d @ k uh1 t s ) ((pos "Ha%" ) (subcat )))
( "undercutting"  v ( uh2 n d @ k uh1 t i ng ) ((pos "Hb%" ) (subcat )))
( "underdeveloped"  j ( uh2 n d @ d i v e1 l @ p t ) ((pos "OA%" ) ))
( "underdevelopment"  n ( uh2 n d @ d i v e1 l @ p m @ n t ) ((pos "L@%" ) ))
( "underdog"  n ( uh1 n d @ d o g ) ((pos "K6%" ) ))
( "underdogs"  n ( uh1 n d @ d o g z ) ((pos "Kj%" ) ))
( "underdone"  j ( uh2 n d @ d uh1 n ) ((pos "OA%" ) ))
( "underemployed"  j ( uh2 n d @ r i m p l oi1 d ) ((pos "OA%" ) ))
( "underestimate"  n ( uh2 n d @ r e1 s t i m @ t ) ((pos "K6%" ) ))
( "underestimate"  v ( uh2 n d @ r e1 s t i m ei t ) ((pos "H2%" ) (subcat )))
( "underestimated"  v ( uh2 n d @ r e1 s t i m ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underestimates"  n ( uh2 n d @ r e1 s t i m @ t s ) ((pos "Kj%" ) ))
( "underestimates"  v ( uh2 n d @ r e1 s t i m ei t s ) ((pos "Ha%" ) (subcat )))
( "underestimating"  v ( uh2 n d @ r e1 s t i m ei t i ng ) ((pos "Hb%" ) (subcat )))
( "underestimation"  n ( uh2 n d @ r e2 s t i m ei1 sh @ n ) ((pos "M6%" ) ))
( "underestimations"  n ( uh2 n d @ r e2 s t i m ei1 sh @ n z ) ((pos "Mj%" ) ))
( "underexpose"  v ( uh2 n d @ r i k s p ou1 z ) ((pos "H2%" ) (subcat )))
( "underexposed"  v ( uh2 n d @ r i k s p ou1 z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underexposes"  v ( uh2 n d @ r i k s p ou1 z i z ) ((pos "Ha%" ) (subcat )))
( "underexposing"  v ( uh2 n d @ r i k s p ou1 z i ng ) ((pos "Hb%" ) (subcat )))
( "underexposure"  n ( uh2 n d @ r i k s p ou1 zh @ r ) ((pos "K6%" ) ))
( "underexposures"  n ( uh2 n d @ r i k s p ou1 zh @ z ) ((pos "Kj%" ) ))
( "underfed"  j ( uh2 n d @ f e1 d ) ((pos "OA%" ) ))
( "underfelt"  n ( uh1 n d @ f e l t ) ((pos "L@%" ) ))
( "underfloor"  j ( uh2 n d @ f l oo1 r ) ((pos "OA%" ) ))
( "underfoot"  a ( uh2 n d @ f u1 t ) ((pos "Pu%" ) ))
( "undergarment"  n ( uh1 n d @ g aa m @ n t ) ((pos "K6%" ) ))
( "undergarments"  n ( uh1 n d @ g aa m @ n t s ) ((pos "Kj%" ) ))
( "undergo"  v ( uh2 n d @ g ou1 ) ((pos "H5%" ) (subcat )))
( "undergoes"  v ( uh2 n d @ g ou1 z ) ((pos "Ha%" ) (subcat )))
( "undergoing"  v ( uh2 n d @ g ou1 i ng ) ((pos "Hb%" ) (subcat )))
( "undergone"  v ( uh2 n d @ g o1 n ) ((pos "Hd%" ) (subcat )))
( "undergraduate"  n ( uh2 n d @ g r a1 jh u@ t ) ((pos "K6%" ) ))
( "undergraduates"  n ( uh2 n d @ g r a1 jh u@ t s ) ((pos "Kj%" ) ))
( "underground"  n ( uh1 n d @ g r au n d ) ((pos "K6%" "Oq%" "Pu%" ) ))
( "undergrounds"  n ( uh1 n d @ g r au n d z ) ((pos "Kj$" ) ))
( "undergrowth"  n ( uh1 n d @ g r ou th ) ((pos "L@%" ) ))
( "underhand"  j ( uh1 n d @ h a n d ) ((pos "OA%" "Pu%" ) ))
( "underhanded"  j ( uh1 n d @ h a n d i d ) ((pos "OA%" ) ))
( "underhung"  j ( uh2 n d @ h uh1 ng ) ((pos "OA$" ) ))
( "underlain"  v ( uh2 n d @ l ei1 n ) ((pos "Hd%" ) (subcat )))
( "underlay"  n ( uh1 n d @ l ei ) ((pos "L@%" ) ))
( "underlay"  v ( uh2 n d @ l ei1 ) ((pos "Hc%" ) (subcat )))
( "underlie"  v ( uh2 n d @ l ai1 ) ((pos "H5%" ) (subcat )))
( "underlies"  v ( uh2 n d @ l ai1 z ) ((pos "Ha%" ) (subcat )))
( "underline"  n ( uh1 n d @ l ai n ) ((pos "K6%" ) ))
( "underline"  v ( uh2 n d @ l ai1 n ) ((pos "H2%" ) (subcat )))
( "underlined"  v ( uh2 n d @ l ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underlines"  n ( uh1 n d @ l ai n z ) ((pos "Kj%" ) ))
( "underlines"  v ( uh2 n d @ l ai1 n z ) ((pos "Ha%" ) (subcat )))
( "underling"  n ( uh1 n d @ l i ng ) ((pos "K6%" ) ))
( "underlings"  n ( uh1 n d @ l i ng z ) ((pos "Kj%" ) ))
( "underlining"  v ( uh2 n d @ l ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "underlying"  v ( uh2 n d @ l ai1 i ng ) ((pos "Hb%" ) (subcat )))
( "undermanned"  j ( uh2 n d @ m a1 n d ) ((pos "OA%" ) ))
( "undermentioned"  j ( uh2 n d @ m e1 n sh @ n d ) ((pos "OA%" ) ))
( "undermine"  v ( uh2 n d @ m ai1 n ) ((pos "H2%" ) (subcat )))
( "undermined"  v ( uh2 n d @ m ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "undermines"  v ( uh2 n d @ m ai1 n z ) ((pos "Ha%" ) (subcat )))
( "undermining"  v ( uh2 n d @ m ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "underneath"  a ( uh2 n d @ n ii1 th ) ((pos "Pu%" "T-%" ) ))
( "undernourished"  j ( uh2 n d @ n uh1 r i sh t ) ((pos "OA%" ) ))
( "undernourishment"  n ( uh2 n d @ n uh1 r i sh m @ n t ) ((pos "L@%" ) ))
( "underpaid"  v ( uh2 n d @ p ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underpants"  n ( uh1 n d @ p a n t s ) ((pos "Kj%" ) ))
( "underpass"  n ( uh1 n d @ p aa s ) ((pos "K7%" ) ))
( "underpasses"  n ( uh1 n d @ p aa s i z ) ((pos "Kj%" ) ))
( "underpay"  v ( uh2 n d @ p ei1 ) ((pos "H5%" ) (subcat )))
( "underpaying"  v ( uh2 n d @ p ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "underpayment"  n ( uh2 n d @ p ei1 m @ n t ) ((pos "M6%" ) ))
( "underpayments"  n ( uh2 n d @ p ei1 m @ n t s ) ((pos "Mj%" ) ))
( "underpays"  v ( uh2 n d @ p ei1 z ) ((pos "Ha%" ) (subcat )))
( "underpin"  v ( uh2 n d @ p i1 n ) ((pos "H4%" ) ))
( "underpinned"  v ( uh2 n d @ p i1 n d ) ((pos "Hc%" "Hd%" ) ))
( "underpinning"  v ( uh2 n d @ p i1 n i ng ) ((pos "Hb%" ) ))
( "underpins"  v ( uh2 n d @ p i1 n z ) ((pos "Ha%" ) ))
( "underpopulated"  j ( uh2 n d @ p o1 p y u l ei t i d ) ((pos "OA%" ) ))
( "underprivileged"  j ( uh2 n d @ p r i1 v @ l i jh d ) ((pos "OA%" ) ))
( "underproduction"  n ( uh2 n d @ p r @ d uh1 k sh @ n ) ((pos "L@%" ) ))
( "underquote"  v ( uh2 n d @ k w ou1 t ) ((pos "H2$" ) (subcat )))
( "underquoted"  v ( uh2 n d @ k w ou1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "underquotes"  v ( uh2 n d @ k w ou1 t s ) ((pos "Ha$" ) (subcat )))
( "underquoting"  v ( uh2 n d @ k w ou1 t i ng ) ((pos "Hb$" ) (subcat )))
( "underrate"  v ( uh2 n d @ r ei1 t ) ((pos "H2%" ) (subcat )))
( "underrated"  v ( uh2 n d @ r ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underrates"  v ( uh2 n d @ r ei1 t s ) ((pos "Ha%" ) (subcat )))
( "underrating"  v ( uh2 n d @ r ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "underscore"  v ( uh2 n d @ s k oo1 r ) ((pos "H2%" ) (subcat )))
( "underscored"  v ( uh2 n d @ s k oo1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underscores"  v ( uh2 n d @ s k oo1 z ) ((pos "Ha%" ) (subcat )))
( "underscoring"  v ( uh2 n d @ s k oo1 r i ng ) ((pos "Hb%" ) (subcat )))
( "undersea"  j ( uh1 n d @ s ii ) ((pos "OA%" ) ))
( "underseal"  n ( uh1 n d @ s ii l ) ((pos "L@%" ) ))
( "undersealed"  j ( uh1 n d @ s ii l d ) ((pos "OA%" ) ))
( "undersecretaries"  n ( uh2 n d @ s e1 k r @ t r i z ) ((pos "Kj%" ) ))
( "undersecretary"  n ( uh2 n d @ s e1 k r @ t r ii ) ((pos "K8%" ) ))
( "undersell"  v ( uh2 n d @ s e1 l ) ((pos "H5%" ) (subcat )))
( "underselling"  v ( uh2 n d @ s e1 l i ng ) ((pos "Hb%" ) (subcat )))
( "undersells"  v ( uh2 n d @ s e1 l z ) ((pos "Ha%" ) (subcat )))
( "undersexed"  j ( uh2 n d @ s e1 k s t ) ((pos "OA%" ) ))
( "undershoot"  v ( uh2 n d @ sh uu1 t ) ((pos "H5%" ) ))
( "undershooting"  v ( uh2 n d @ sh uu1 t i ng ) ((pos "Hb%" ) ))
( "undershoots"  v ( uh2 n d @ sh uu1 t s ) ((pos "Ha%" ) ))
( "undershot"  v ( uh2 n d @ sh o1 t ) ((pos "Hc%" "Hd%" ) ))
( "underside"  n ( uh1 n d @ s ai d ) ((pos "K6%" ) ))
( "undersides"  n ( uh1 n d @ s ai d z ) ((pos "Kj%" ) ))
( "undersign"  v ( uh2 n d @ s ai1 n ) ((pos "H0%" ) (subcat )))
( "undersigned"  v ( uh2 n d @ s ai1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "undersigning"  v ( uh2 n d @ s ai1 n i ng ) ((pos "Hb%" ) (subcat )))
( "undersigns"  v ( uh2 n d @ s ai1 n z ) ((pos "Ha%" ) (subcat )))
( "undersized"  j ( uh2 n d @ s ai1 z d ) ((pos "OA%" ) ))
( "underskirt"  n ( uh1 n d @ s k @@ t ) ((pos "K6%" ) ))
( "underskirts"  n ( uh1 n d @ s k @@ t s ) ((pos "Kj%" ) ))
( "underslung"  j ( uh2 n d @ s l uh1 ng ) ((pos "OA$" ) ))
( "undersold"  v ( uh2 n d @ s ou1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "underspend"  v ( uh2 n d @ s p e1 n d ) ((pos "J5%" ) (subcat "2A" "6A" "14" )))
( "underspending"  v ( uh2 n d @ s p e1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "14" )))
( "underspends"  v ( uh2 n d @ s p e1 n d z ) ((pos "Ja%" ) (subcat "2A" "6A" "14" )))
( "underspent"  v ( uh2 n d @ s p e1 n t ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "6A" "14" )))
( "understaffed"  j ( uh2 n d @ s t aa1 f t ) ((pos "OA%" ) ))
( "understand"  v ( uh2 n d @ s t a1 n d ) ((pos "J5%" ) (subcat "2A" "6A" "6C" "8" "9" "10" "17" )))
( "understandable"  j ( uh2 n d @ s t a1 n d @ b l ) ((pos "OA%" ) ))
( "understandably"  a ( uh2 n d @ s t a1 n d @ b l ii ) ((pos "Pu%" ) ))
( "understanding"  v ( uh2 n d @ s t a1 n d i ng ) ((pos "Jb%" "M6%" "OA%" ) (subcat "2A" "6A" "6C" "8" "9" "10" "17" )))
( "understandings"  n ( uh2 n d @ s t a1 n d i ng z ) ((pos "Mj$" ) ))
( "understands"  v ( uh2 n d @ s t a1 n d z ) ((pos "Ja%" ) (subcat "2A" "6A" "6C" "8" "9" "10" "17" )))
( "understate"  v ( uh2 n d @ s t ei1 t ) ((pos "H2%" ) (subcat )))
( "understated"  v ( uh2 n d @ s t ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "understatement"  n ( uh1 n d @ s t ei t m @ n t ) ((pos "M6%" ) ))
( "understatements"  n ( uh1 n d @ s t ei t m @ n t s ) ((pos "Mj%" ) ))
( "understates"  v ( uh2 n d @ s t ei1 t s ) ((pos "Ha%" ) (subcat )))
( "understating"  v ( uh2 n d @ s t ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "understock"  v ( uh2 n d @ s t o1 k ) ((pos "H0%" ) (subcat )))
( "understocked"  v ( uh2 n d @ s t o1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "understocking"  v ( uh2 n d @ s t o1 k i ng ) ((pos "Hb%" ) (subcat )))
( "understocks"  v ( uh2 n d @ s t o1 k s ) ((pos "Ha%" ) (subcat )))
( "understood"  v ( uh2 n d @ s t u1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "6C" "8" "9" "10" "17" )))
( "understudied"  v ( uh1 n d @ s t uh d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "understudies"  v ( uh1 n d @ s t uh d i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "understudy"  v ( uh1 n d @ s t uh d ii ) ((pos "H3%" "K8%" ) (subcat )))
( "understudying"  v ( uh1 n d @ s t uh d i i ng ) ((pos "Hb%" ) (subcat )))
( "undersurface"  n ( uh1 n d @ s @@ f i s ) ((pos "K6%" ) ))
( "undersurfaces"  n ( uh1 n d @ s @@ f i s i z ) ((pos "Kj%" ) ))
( "undertake"  v ( uh2 n d @ t ei1 k ) ((pos "H5%" ) (subcat "6A" "7A" )))
( "undertaken"  v ( uh2 n d @ t ei1 k @ n ) ((pos "Hc%" ) (subcat "6A" "7A" )))
( "undertaker"  n ( uh1 n d @ t ei k @ r ) ((pos "K6%" ) ))
( "undertakers"  n ( uh1 n d @ t ei k @ z ) ((pos "Kj%" ) ))
( "undertakes"  v ( uh2 n d @ t ei1 k s ) ((pos "Ha%" ) (subcat "6A" "7A" )))
( "undertaking"  n ( uh1 n d @ t ei k i ng ) ((pos "M6%" ) ))
( "undertaking"  v ( uh2 n d @ t ei1 k i ng ) ((pos "Hb%" "K6%" ) (subcat "6A" "7A" )))
( "undertakings"  n ( uh1 n d @ t ei k i ng z ) ((pos "Mj$" ) ))
( "undertakings"  n ( uh2 n d @ t ei1 k i ng z ) ((pos "Kj%" ) ))
( "undertone"  n ( uh1 n d @ t ou n ) ((pos "K6%" ) ))
( "undertones"  n ( uh1 n d @ t ou n z ) ((pos "Kj%" ) ))
( "undertook"  v ( uh2 n d @ t u1 k ) ((pos "Hc%" ) (subcat "6A" "7A" )))
( "undertow"  n ( uh1 n d @ t ou ) ((pos "K6%" ) ))
( "undertows"  n ( uh1 n d @ t ou z ) ((pos "Kj$" ) ))
( "undervaluation"  n ( uh2 n d @ v a2 l y u ei1 sh @ n ) ((pos "M6%" ) ))
( "undervaluations"  n ( uh2 n d @ v a2 l y u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "undervalue"  v ( uh2 n d @ v a1 l y uu ) ((pos "H2%" ) (subcat )))
( "undervalued"  v ( uh2 n d @ v a1 l y uu d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "undervalues"  v ( uh2 n d @ v a1 l y uu z ) ((pos "Ha%" ) (subcat )))
( "undervaluing"  v ( uh2 n d @ v a1 l y uu i ng ) ((pos "Hb%" ) (subcat )))
( "underwater"  j ( uh1 n d @ w oo t @ r ) ((pos "OA%" ) ))
( "underwear"  n ( uh1 n d @ w e@ r ) ((pos "L@%" ) ))
( "underweight"  j ( uh2 n d @ w ei1 t ) ((pos "OA%" ) ))
( "underwent"  v ( uh2 n d @ w e1 n t ) ((pos "Hc%" ) (subcat )))
( "underworld"  n ( uh1 n d @ w @@ l d ) ((pos "K6%" ) ))
( "underworlds"  n ( uh1 n d @ w @@ l d z ) ((pos "Kj$" ) ))
( "underwrite"  v ( uh2 n d @ r ai1 t ) ((pos "H5%" ) (subcat )))
( "underwriter"  n ( uh2 n d @ r ai1 t @ r ) ((pos "K6%" ) ))
( "underwriters"  n ( uh2 n d @ r ai1 t @ z ) ((pos "Kj%" ) ))
( "underwrites"  v ( uh2 n d @ r ai1 t s ) ((pos "Ha%" ) (subcat )))
( "underwriting"  v ( uh2 n d @ r ai1 t i ng ) ((pos "Hb%" ) (subcat )))
( "underwritten"  v ( uh2 n d @ r i1 t n ) ((pos "Hd%" ) (subcat )))
( "underwrote"  v ( uh2 n d @ r ou1 t ) ((pos "Hc%" ) (subcat )))
( "undeserved"  j ( uh2 n d i z @@1 v d ) ((pos "OA%" ) ))
( "undeserving"  j ( uh2 n d i z @@1 v i ng ) ((pos "OA%" ) ))
( "undesigned"  j ( uh2 n d i z ai1 n d ) ((pos "OA%" ) ))
( "undesirable"  n ( uh2 n d i z ai1 @ r @ b l ) ((pos "K6%" "OA%" ) ))
( "undesirables"  n ( uh2 n d i z ai1 @ r @ b l z ) ((pos "Kj%" ) ))
( "undetected"  j ( uh2 n d i t e1 k t i d ) ((pos "OA%" ) ))
( "undetermined"  j ( uh2 n d i t @@1 m i n d ) ((pos "OA%" ) ))
( "undeterred"  j ( uh2 n d i t @@1 d ) ((pos "OA%" ) ))
( "undeveloped"  j ( uh2 n d i v e1 l @ p t ) ((pos "OA%" ) ))
( "undiagnosed"  j ( uh1 n d ai1 @ g n ou z d ) ((pos "OA%" ) ))
( "undid"  v ( uh1 n d i1 d ) ((pos "Hc%" ) (subcat )))
( "undies"  n ( uh1 n d i z ) ((pos "Kj%" ) ))
( "undifferentiated"  j ( uh2 n d i f @ r e1 n sh i ei t i d ) ((pos "OA%" ) ))
( "undigested"  j ( uh2 n d i jh e1 s t i d ) ((pos "OA%" ) ))
( "undignified"  j ( uh1 n d i1 g n i f ai d ) ((pos "OA%" ) ))
( "undiluted"  j ( uh2 n d ai l y uu1 t i d ) ((pos "OA%" ) ))
( "undiminished"  j ( uh2 n d i m i1 n i sh t ) ((pos "OA%" ) ))
( "undimmed"  j ( uh1 n d i1 m d ) ((pos "OA%" ) ))
( "undiplomatic"  j ( uh2 n d i p l @ m a1 t i k ) ((pos "OA%" ) ))
( "undiplomatically"  a ( uh2 n d i p l @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "undischarged"  j ( uh2 n d i s ch aa1 jh d ) ((pos "OA%" ) ))
( "undisciplined"  j ( uh1 n d i1 s i p l i n d ) ((pos "OA%" ) ))
( "undisclosed"  j ( uh2 n d i s k l ou1 z d ) ((pos "OA%" ) ))
( "undiscovered"  j ( uh2 n d i s k uh1 v @ d ) ((pos "OA%" ) ))
( "undiscriminating"  j ( uh2 n d i s k r i1 m i n ei t i ng ) ((pos "OA%" ) ))
( "undisguised"  j ( uh2 n d i s g ai1 z d ) ((pos "OA%" ) ))
( "undismayed"  j ( uh2 n d i s m ei1 d ) ((pos "OA%" ) ))
( "undisputed"  j ( uh2 n d i s p y uu1 t i d ) ((pos "OA%" ) ))
( "undisputedly"  a ( uh2 n d i s p y uu1 t i d l ii ) ((pos "Pu%" ) ))
( "undissolved"  j ( uh2 n d i z o1 l v d ) ((pos "OA%" ) ))
( "undistinguishable"  j ( uh2 n d i s t i1 ng g w i sh @ b l ) ((pos "OA%" ) ))
( "undistinguished"  j ( uh2 n d i s t i1 ng g w i sh t ) ((pos "OA%" ) ))
( "undistributed"  j ( uh2 n d i s t r i1 b y uu t i d ) ((pos "OA%" ) ))
( "undisturbed"  j ( uh2 n d i s t @@1 b d ) ((pos "OA%" ) ))
( "undivided"  j ( uh2 n d i v ai1 d i d ) ((pos "OA%" ) ))
( "undo"  v ( uh1 n d uu1 ) ((pos "H5%" ) (subcat )))
( "undock"  v ( uh1 n d o1 k ) ((pos "J0$" ) (subcat "2A" )))
( "undocked"  v ( uh1 n d o1 k t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "undocking"  v ( uh1 n d o1 k i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "undocks"  v ( uh1 n d o1 k s ) ((pos "Ja$" ) (subcat "2A" )))
( "undocumented"  j ( uh1 n d o1 k y u m e n t i d ) ((pos "OA%" ) ))
( "undoes"  v ( uh1 n d uh1 z ) ((pos "Ha%" ) (subcat )))
( "undogmatic"  j ( uh2 n d o g m a1 t i k ) ((pos "OA%" ) ))
( "undoing"  v ( uh1 n d uu1 i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "undoings"  n ( uh1 n d uu1 i ng z ) ((pos "Kj$" ) ))
( "undomesticated"  j ( uh2 n d @ m e1 s t i k ei t i d ) ((pos "OA%" ) ))
( "undone"  v ( uh1 n d uh1 n ) ((pos "Hd%" "Op%" ) (subcat )))
( "undoubted"  j ( uh1 n d au1 t i d ) ((pos "OA%" ) ))
( "undoubtedly"  a ( uh1 n d au1 t i d l ii ) ((pos "Pu%" ) ))
( "undramatic"  j ( uh2 n d r @ m a1 t i k ) ((pos "OA%" ) ))
( "undramatically"  a ( uh2 n d r @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "undreamed"  j ( uh1 n d r ii1 m d ) ((pos "OA%" ) ))
( "undreamed-of"  j ( uh1 n d r ii1 m d - @ v ) ((pos "OA%" ) ))
( "undreamt"  j ( uh1 n d r e1 m t ) ((pos "OA%" ) ))
( "undress"  v ( uh1 n d r e1 s ) ((pos "J1%" "L@%" ) (subcat "2A" )))
( "undressed"  v ( uh1 n d r e1 s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "undresses"  v ( uh1 n d r e1 s i z ) ((pos "Ja%" ) (subcat "2A" )))
( "undressing"  v ( uh1 n d r e1 s i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "undrinkable"  j ( uh1 n d r i1 ng k @ b l ) ((pos "OA%" ) ))
( "undue"  j ( uh2 n d y uu1 ) ((pos "OA%" ) ))
( "undulate"  v ( uh1 n d y u l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "undulated"  v ( uh1 n d y u l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "undulates"  v ( uh1 n d y u l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "undulating"  v ( uh1 n d y u l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "undulation"  n ( uh2 n d y u l ei1 sh @ n ) ((pos "M6%" ) ))
( "undulations"  n ( uh2 n d y u l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "unduly"  a ( uh2 n d y uu1 l ii ) ((pos "Pu%" ) ))
( "undying"  j ( uh2 n d ai1 i ng ) ((pos "OA%" ) ))
( "unearned"  j ( uh2 n @@1 n d ) ((pos "OA%" ) ))
( "unearth"  v ( uh1 n @@1 th ) ((pos "H0%" ) (subcat )))
( "unearthed"  v ( uh1 n @@1 th t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unearthing"  v ( uh1 n @@1 th i ng ) ((pos "Hb$" ) (subcat )))
( "unearthly"  j ( uh1 n @@1 th l ii ) ((pos "OA%" ) ))
( "unearths"  v ( uh1 n @@1 th s ) ((pos "Ha$" ) (subcat )))
( "unease"  n ( uh1 n ii1 z ) ((pos "L@%" ) ))
( "uneasily"  a ( uh1 n ii1 z i l ii ) ((pos "Pu%" ) ))
( "uneasiness"  n ( uh1 n ii1 z i n @ s ) ((pos "L@%" ) ))
( "uneasy"  j ( uh1 n ii1 z ii ) ((pos "OA%" ) ))
( "uneatable"  j ( uh1 n ii1 t @ b l ) ((pos "OA%" ) ))
( "uneaten"  j ( uh1 n ii1 t n ) ((pos "OA%" ) ))
( "uneconomic"  j ( uh2 n ii k @ n o1 m i k ) ((pos "OA%" ) ))
( "uneconomical"  j ( uh2 n ii k @ n o1 m i k l ) ((pos "OA%" ) ))
( "unedifying"  j ( uh1 n e1 d i f ai i ng ) ((pos "OA%" ) ))
( "unedited"  j ( uh1 n e1 d i t i d ) ((pos "OA%" ) ))
( "uneducated"  j ( uh1 n e1 jh u k ei t i d ) ((pos "OA%" ) ))
( "uneffective"  j ( uh2 n i f e1 k t i v ) ((pos "OA%" ) ))
( "unelaborated"  j ( uh2 n i l a1 b @ r ei t i d ) ((pos "OA%" ) ))
( "unemotional"  j ( uh2 n i m ou1 sh @ n l ) ((pos "OA%" ) ))
( "unemotionally"  a ( uh2 n i m ou1 sh @ n @ l ii ) ((pos "Pu%" ) ))
( "unemployable"  j ( uh2 n i m p l oi1 @ b l ) ((pos "OA%" ) ))
( "unemployed"  j ( uh2 n i m p l oi1 d ) ((pos "OA%" ) ))
( "unemployment"  n ( uh2 n i m p l oi1 m @ n t ) ((pos "L@%" ) ))
( "unending"  j ( uh1 n e1 n d i ng ) ((pos "OA%" ) ))
( "unendingly"  a ( uh1 n e1 n d i ng l ii ) ((pos "Pu%" ) ))
( "unendurable"  j ( uh2 n i n d y u@1 r @ b l ) ((pos "OA%" ) ))
( "unenlightened"  j ( uh2 n i n l ai1 t n d ) ((pos "OA%" ) ))
( "unenterprising"  j ( uh1 n e1 n t @ p r ai z i ng ) ((pos "OA%" ) ))
( "unenthusiastic"  j ( uh2 n i n th y uu2 z i a1 s t i k ) ((pos "OA%" ) ))
( "unenthusiastically"  a ( uh2 n i n th y uu2 z i a1 s t i k l ii ) ((pos "Pu%" ) ))
( "unenviable"  j ( uh1 n e1 n v i@ b l ) ((pos "OA%" ) ))
( "unequal"  j ( uh2 n ii1 k w @ l ) ((pos "OA%" ) ))
( "unequalled"  j ( uh2 n ii1 k w @ l d ) ((pos "OA%" ) ))
( "unequally"  a ( uh2 n ii1 k w @ l ii ) ((pos "Pu%" ) ))
( "unequipped"  j ( uh2 n i k w i1 p t ) ((pos "OA%" ) ))
( "unequivocal"  j ( uh2 n i k w i1 v @ k l ) ((pos "OA%" ) ))
( "unequivocally"  a ( uh2 n i k w i1 v @ k l ii ) ((pos "Pu%" ) ))
( "unerring"  j ( uh1 n @@1 r i ng ) ((pos "OA%" ) ))
( "unerringly"  a ( uh1 n @@1 r i ng l ii ) ((pos "Pu%" ) ))
( "unethical"  j ( uh1 n e1 th i k l ) ((pos "OA%" ) ))
( "unethically"  a ( uh1 n e1 th i k l ii ) ((pos "Pu%" ) ))
( "uneven"  j ( uh1 n ii1 v n ) ((pos "OA%" ) ))
( "unevenly"  a ( uh1 n ii1 v n l ii ) ((pos "Pu%" ) ))
( "uneventful"  j ( uh2 n i v e1 n t f @ l ) ((pos "OA%" ) ))
( "uneventfully"  a ( uh2 n i v e1 n t f @ l ii ) ((pos "Pu%" ) ))
( "unexampled"  j ( uh2 n i g z aa1 m p l d ) ((pos "OA%" ) ))
( "unexceeded"  j ( uh2 n i k s ii1 d i d ) ((pos "OA%" ) ))
( "unexceptionable"  j ( uh2 n i k s e1 p sh @ n @ b l ) ((pos "OA%" ) ))
( "unexceptional"  j ( uh2 n i k s e1 p sh @ n l ) ((pos "OA%" ) ))
( "unexciting"  j ( uh2 n i k s ai1 t i ng ) ((pos "OA%" ) ))
( "unexpected"  j ( uh2 n i k s p e1 k t i d ) ((pos "OA%" ) ))
( "unexpectedly"  a ( uh2 n i k s p e1 k t i d l ii ) ((pos "Pu%" ) ))
( "unexpectedness"  n ( uh2 n i k s p e1 k t i d n i s ) ((pos "L@%" ) ))
( "unexpired"  j ( uh2 n i k s p ai1 @ d ) ((pos "OA%" ) ))
( "unexplained"  j ( uh2 n i k s p l ei1 n d ) ((pos "OA%" ) ))
( "unexplored"  j ( uh2 n i k s p l oo1 d ) ((pos "OA%" ) ))
( "unexposed"  j ( uh2 n i k s p ou1 z d ) ((pos "OA%" ) ))
( "unexpressed"  j ( uh2 n i k s p r e1 s t ) ((pos "OA%" ) ))
( "unexpurgated"  j ( uh1 n e1 k s p @ g ei t i d ) ((pos "OA%" ) ))
( "unfailing"  j ( uh1 n f ei1 l i ng ) ((pos "OA%" ) ))
( "unfailingly"  a ( uh1 n f ei1 l i ng l ii ) ((pos "Pu%" ) ))
( "unfair"  j ( uh2 n f e@1 r ) ((pos "OA%" ) ))
( "unfairly"  a ( uh2 n f e@1 l ii ) ((pos "Pu%" ) ))
( "unfairness"  n ( uh2 n f e@1 n @ s ) ((pos "L@%" ) ))
( "unfaithful"  j ( uh2 n f ei1 th f @ l ) ((pos "OA%" ) ))
( "unfaithfully"  a ( uh2 n f ei1 th f @ l ii ) ((pos "Pu%" ) ))
( "unfaithfulness"  n ( uh2 n f ei1 th f @ l n @ s ) ((pos "L@%" ) ))
( "unfaltering"  j ( uh1 n f oo1 l t @ r i ng ) ((pos "OA%" ) ))
( "unfalteringly"  a ( uh1 n f oo1 l t @ r i ng l ii ) ((pos "Pu%" ) ))
( "unfamiliar"  j ( uh2 n f @ m i1 l i@ r ) ((pos "OA%" ) ))
( "unfamiliarity"  n ( uh2 n f @ m i2 l i a1 r i t ii ) ((pos "L@%" ) ))
( "unfashionable"  j ( uh1 n f a1 sh @ n @ b l ) ((pos "OA%" ) ))
( "unfashionably"  a ( uh1 n f a1 sh @ n @ b l ii ) ((pos "Pu%" ) ))
( "unfasten"  v ( uh1 n f aa1 s n ) ((pos "J0%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "unfastened"  v ( uh1 n f aa1 s n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "unfastening"  v ( uh1 n f aa1 s n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "unfastens"  v ( uh1 n f aa1 s n z ) ((pos "Ja%" ) (subcat "2A" "2C" "3A" "6A" "14" "15A" )))
( "unfathomable"  j ( uh1 n f a1 dh @ m @ b l ) ((pos "OA%" ) ))
( "unfathomed"  j ( uh1 n f a1 dh @ m d ) ((pos "OA%" ) ))
( "unfavourable"  j ( uh1 n f ei1 v @ r @ b l ) ((pos "OA%" ) ))
( "unfavourably"  a ( uh1 n f ei1 v @ r @ b l ii ) ((pos "Pu%" ) ))
( "unfed"  j ( uh1 n f e1 d ) ((pos "OA%" ) ))
( "unfeeling"  j ( uh1 n f ii1 l i ng ) ((pos "OA%" ) ))
( "unfeelingly"  a ( uh1 n f ii1 l i ng l ii ) ((pos "Pu%" ) ))
( "unfeigned"  j ( uh1 n f ei1 n d ) ((pos "OA%" ) ))
( "unfeignedly"  a ( uh1 n f ei1 n @ d l ii ) ((pos "Pu%" ) ))
( "unfermented"  j ( uh2 n f @ m e1 n t i d ) ((pos "OA%" ) ))
( "unfertilized"  j ( uh1 n f @@1 t @ l ai z d ) ((pos "OA%" ) ))
( "unfettered"  j ( uh1 n f e1 t @ d ) ((pos "OA%" ) ))
( "unfinished"  j ( uh1 n f i1 n i sh t ) ((pos "OA%" ) ))
( "unfit"  v ( uh2 n f i1 t ) ((pos "H4$" "OA%" ) (subcat )))
( "unfits"  v ( uh2 n f i1 t s ) ((pos "Ha$" ) (subcat )))
( "unfitted"  v ( uh2 n f i1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "unfitting"  v ( uh2 n f i1 t i ng ) ((pos "Hb$" ) (subcat )))
( "unflagging"  j ( uh2 n f l a1 g i ng ) ((pos "OA%" ) ))
( "unflappable"  j ( uh2 n f l a1 p @ b l ) ((pos "OA%" ) ))
( "unfledged"  j ( uh2 n f l e1 jh d ) ((pos "OA%" ) ))
( "unflinching"  j ( uh1 n f l i1 n ch i ng ) ((pos "OA%" ) ))
( "unflurried"  j ( uh1 n f l uh1 r i d ) ((pos "OA%" ) ))
( "unfold"  v ( uh1 n f ou1 l d ) ((pos "J0%" ) (subcat "2A" "2C" )))
( "unfolded"  v ( uh1 n f ou1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "unfolding"  v ( uh1 n f ou1 l d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "unfolds"  v ( uh1 n f ou1 l d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "unforeseeable"  j ( uh2 n f oo s ii1 @ b l ) ((pos "OA%" ) ))
( "unforeseen"  j ( uh2 n f oo s ii1 n ) ((pos "OA%" ) ))
( "unforgettable"  j ( uh2 n f @ g e1 t @ b l ) ((pos "OA%" ) ))
( "unforgettably"  a ( uh2 n f @ g e1 t @ b l ii ) ((pos "Pu%" ) ))
( "unforgivable"  j ( uh2 n f @ g i1 v @ b l ) ((pos "OA%" ) ))
( "unforgivably"  a ( uh2 n f @ g i1 v @ b l ii ) ((pos "Pu%" ) ))
( "unforgiving"  j ( uh2 n f @ g i1 v i ng ) ((pos "OA%" ) ))
( "unformed"  j ( uh1 n f oo1 m d ) ((pos "OA%" ) ))
( "unforthcoming"  j ( uh2 n f oo th k uh1 m i ng ) ((pos "OA%" ) ))
( "unfortunate"  n ( uh1 n f oo1 ch u n @ t ) ((pos "K6$" "OA%" ) ))
( "unfortunately"  a ( uh1 n f oo1 ch u n @ t l ii ) ((pos "Pu%" ) ))
( "unfortunates"  n ( uh1 n f oo1 ch @ n @ t s ) ((pos "Kj$" ) ))
( "unfounded"  j ( uh2 n f au1 n d i d ) ((pos "OA%" ) ))
( "unframed"  j ( uh1 n f r ei1 m d ) ((pos "OA%" ) ))
( "unfrequented"  j ( uh2 n f r i k w e1 n t i d ) ((pos "OA%" ) ))
( "unfriendliness"  n ( uh1 n f r e1 n d l i n @ s ) ((pos "L@%" ) ))
( "unfriendly"  j ( uh2 n f r e1 n d l ii ) ((pos "OA%" ) ))
( "unfrock"  v ( uh2 n f r o1 k ) ((pos "H0%" ) (subcat )))
( "unfrocked"  v ( uh2 n f r o1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unfrocking"  v ( uh2 n f r o1 k i ng ) ((pos "Hb$" ) (subcat )))
( "unfrocks"  v ( uh2 n f r o1 k s ) ((pos "Ha$" ) (subcat )))
( "unfruitful"  j ( uh1 n f r uu1 t f @ l ) ((pos "OA%" ) ))
( "unfulfilled"  j ( uh2 n f u l f i1 l d ) ((pos "OA%" ) ))
( "unfurl"  v ( uh1 n f @@1 l ) ((pos "J0%" ) (subcat "2A" )))
( "unfurled"  v ( uh1 n f @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unfurling"  v ( uh1 n f @@1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unfurls"  v ( uh1 n f @@1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "unfurnished"  j ( uh2 n f @@1 n i sh t ) ((pos "OA%" ) ))
( "ungainly"  j ( uh1 n g ei1 n l ii ) ((pos "OA%" ) ))
( "ungenerous"  j ( uh1 n jh e1 n @ r @ s ) ((pos "OA%" ) ))
( "ungentle"  j ( uh1 n jh e1 n t l ) ((pos "OA%" ) ))
( "ungentlemanly"  j ( uh1 n jh e1 n t l m @ n l ii ) ((pos "OA%" ) ))
( "unglazed"  j ( uh1 n g l ei1 z d ) ((pos "OA%" ) ))
( "ungodly"  j ( uh1 n g o1 d l ii ) ((pos "OA%" ) ))
( "ungovernable"  j ( uh1 n g uh1 v @ n @ b l ) ((pos "OA%" ) ))
( "ungraceful"  j ( uh1 n g r ei1 s f @ l ) ((pos "OA%" ) ))
( "ungracious"  j ( uh1 n g r ei1 sh @ s ) ((pos "OA%" ) ))
( "ungraciously"  a ( uh1 n g r ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "ungrammatical"  j ( uh2 n g r @ m a1 t i k l ) ((pos "OA%" ) ))
( "ungrammatically"  a ( uh2 n g r @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "ungrateful"  j ( uh1 n g r ei1 t f @ l ) ((pos "OA%" ) ))
( "ungratefully"  a ( uh1 n g r ei1 t f @ l ii ) ((pos "Pu%" ) ))
( "ungratefulness"  n ( uh1 n g r ei1 t f @ l n @ s ) ((pos "L@%" ) ))
( "ungrudging"  j ( uh1 n g r uh1 jh i ng ) ((pos "OA%" ) ))
( "ungrudgingly"  a ( uh1 n g r uh1 jh i ng l ii ) ((pos "Pu%" ) ))
( "unguarded"  j ( uh2 n g aa1 d i d ) ((pos "OA%" ) ))
( "unguent"  n ( uh1 ng g w @ n t ) ((pos "M6$" ) ))
( "unguents"  n ( uh1 ng g w @ n t s ) ((pos "Mj$" ) ))
( "unhallowed"  j ( uh1 n h a1 l ou d ) ((pos "OA%" ) ))
( "unhampered"  j ( uh1 n h a1 m p @ d ) ((pos "OA%" ) ))
( "unhand"  v ( uh1 n h a1 n d ) ((pos "H0$" ) (subcat )))
( "unhanded"  v ( uh1 n h a1 n d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "unhanding"  v ( uh1 n h a1 n d i ng ) ((pos "Hb$" ) (subcat )))
( "unhands"  v ( uh1 n h a1 n d z ) ((pos "Ha$" ) (subcat )))
( "unhappier"  j ( uh1 n h a1 p i@ r ) ((pos "Or%" ) ))
( "unhappiest"  j ( uh1 n h a1 p i i s t ) ((pos "Os%" ) ))
( "unhappily"  a ( uh1 n h a1 p i l ii ) ((pos "Pu%" ) ))
( "unhappiness"  n ( uh1 n h a1 p i n @ s ) ((pos "L@%" ) ))
( "unhappy"  j ( uh1 n h a1 p ii ) ((pos "OD%" ) ))
( "unharmed"  j ( uh1 n h aa1 m d ) ((pos "OA%" ) ))
( "unhealed"  j ( uh1 n h ii1 l d ) ((pos "OA%" ) ))
( "unhealthy"  j ( uh1 n h e1 l th ii ) ((pos "OA%" ) ))
( "unheard"  j ( uh1 n h @@1 d ) ((pos "OA%" ) ))
( "unheard-of"  j ( uh1 n h @@1 d - o v ) ((pos "OA%" ) ))
( "unhearing"  j ( uh1 n h i@1 r i ng ) ((pos "OA%" ) ))
( "unheated"  j ( uh1 n h ii1 t i d ) ((pos "OA%" ) ))
( "unheeded"  j ( uh1 n h ii1 d i d ) ((pos "OA%" ) ))
( "unhelpful"  j ( uh1 n h e1 l p f @ l ) ((pos "OA%" ) ))
( "unhelpfully"  a ( uh1 n h e1 l p f @ l ii ) ((pos "Pu%" ) ))
( "unheralded"  j ( uh1 n h e1 r @ l d i d ) ((pos "OA%" ) ))
( "unhesitating"  j ( uh1 n h e1 z i t ei t i ng ) ((pos "OA%" ) ))
( "unhesitatingly"  a ( uh1 n h e1 s i t ei t i ng l ii ) ((pos "Pu%" ) ))
( "unhindered"  j ( uh1 n h i1 n d @ d ) ((pos "OA%" ) ))
( "unhinge"  v ( uh1 n h i1 n jh ) ((pos "H2$" ) (subcat )))
( "unhinged"  v ( uh1 n h i1 n jh d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "unhinges"  v ( uh1 n h i1 n jh i z ) ((pos "Ha$" ) (subcat )))
( "unhinging"  v ( uh1 n h i1 n jh i ng ) ((pos "Hb$" ) (subcat )))
( "unhitch"  v ( uh1 n h i1 ch ) ((pos "J1%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unhitched"  v ( uh1 n h i1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unhitches"  v ( uh1 n h i1 ch i z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unhitching"  v ( uh1 n h i1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unholy"  j ( uh1 n h ou1 l ii ) ((pos "OA%" ) ))
( "unhook"  v ( uh1 n h u1 k ) ((pos "H0%" ) (subcat )))
( "unhooked"  v ( uh1 n h u1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unhooking"  v ( uh1 n h u1 k i ng ) ((pos "Hb%" ) (subcat )))
( "unhooks"  v ( uh1 n h u1 k s ) ((pos "Ha%" ) (subcat )))
( "unhoped-for"  j ( uh1 n h ou1 p t - f oo r ) ((pos "OA%" ) ))
( "unhorse"  v ( uh2 n h oo1 s ) ((pos "H2$" ) (subcat )))
( "unhorsed"  v ( uh2 n h oo1 s t ) ((pos "Hc$" "Hd%" ) (subcat )))
( "unhorses"  v ( uh2 n h oo1 s i z ) ((pos "Ha$" ) (subcat )))
( "unhorsing"  v ( uh2 n h oo1 s i ng ) ((pos "Hb$" ) (subcat )))
( "unhurried"  j ( uh1 n h uh1 r i d ) ((pos "OA%" ) ))
( "unhurriedly"  a ( uh1 n h uh1 r i d l ii ) ((pos "Pu%" ) ))
( "unhurt"  j ( uh1 n h @@1 t ) ((pos "OA%" ) ))
( "unhygienic"  j ( uh2 n h ai jh ii1 n i k ) ((pos "OA%" ) ))
( "unhygienically"  a ( uh2 n h ai jh ii1 n i k l ii ) ((pos "Pu%" ) ))
( "unicorn"  n ( y uu1 n i k oo n ) ((pos "K6%" ) ))
( "unicorns"  n ( y uu1 n i k oo n z ) ((pos "Kj%" ) ))
( "unidentifiable"  j ( uh2 n ai d e2 n t i f ai1 @ b l ) ((pos "OA%" ) ))
( "unidentified"  j ( uh2 n ai d e1 n t i f ai d ) ((pos "OA%" ) ))
( "unification"  n ( y uu2 n i f i k ei1 sh @ n ) ((pos "L@%" ) ))
( "unified"  v ( y uu1 n i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unifies"  v ( y uu1 n i f ai z ) ((pos "Ha%" ) (subcat )))
( "uniform"  n ( y uu1 n i f oo m ) ((pos "M6%" "OA%" ) ))
( "uniformed"  j ( y uu1 n i f oo m d ) ((pos "OA%" ) ))
( "uniformity"  n ( y uu2 n i f oo1 m i t ii ) ((pos "L@%" ) ))
( "uniformly"  a ( y uu1 n i f oo m l ii ) ((pos "Pu%" ) ))
( "uniforms"  n ( y uu1 n i f oo m z ) ((pos "Mj%" ) ))
( "unify"  v ( y uu1 n i f ai ) ((pos "H3%" ) (subcat )))
( "unifying"  v ( y uu1 n i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "unilateral"  j ( y uu2 n i l a1 t r @ l ) ((pos "OA%" ) ))
( "unilateralism"  n ( y uu2 n i l a1 t r @ l i z m ) ((pos "L@%" ) ))
( "unilateralist"  n ( y uu2 n i l a1 t r @ l i s t ) ((pos "K6%" "OA%" ) ))
( "unilateralists"  n ( y uu2 n i l a1 t r @ l i s t s ) ((pos "Kj%" ) ))
( "unilaterally"  a ( y uu2 n i l a1 t r @ l ii ) ((pos "Pu%" ) ))
( "unimaginable"  j ( uh2 n i m a1 jh i n @ b l ) ((pos "OA%" ) ))
( "unimaginably"  a ( uh2 n i m a1 jh i n @ b l ii ) ((pos "Pu%" ) ))
( "unimaginative"  j ( uh2 n i m a1 jh i n @ t i v ) ((pos "OA%" ) ))
( "unimaginatively"  a ( uh2 n i m a1 jh i n @ t i v l ii ) ((pos "Pu%" ) ))
( "unimpaired"  j ( uh2 n i m p e@1 d ) ((pos "OA%" ) ))
( "unimpeachable"  j ( uh2 n i m p ii1 ch @ b l ) ((pos "OA%" ) ))
( "unimpeded"  j ( uh2 n i m p ii1 d i d ) ((pos "OA%" ) ))
( "unimportant"  j ( uh2 n i m p oo1 t n t ) ((pos "OA%" ) ))
( "unimpressed"  j ( uh2 n i m p r e1 s t ) ((pos "OA%" ) ))
( "unimpressive"  j ( uh2 n i m p r e1 s i v ) ((pos "OA%" ) ))
( "unimpressively"  a ( uh2 n i m p r e1 s i v l ii ) ((pos "Pu%" ) ))
( "uninfluenced"  j ( uh1 n i1 n f l u@ n s t ) ((pos "OA%" ) ))
( "uninformative"  j ( uh2 n i n f oo1 m @ t i v ) ((pos "OA%" ) ))
( "uninformed"  j ( uh2 n i n f oo1 m d ) ((pos "OA%" ) ))
( "uninhabitable"  j ( uh2 n i n h a1 b i t @ b l ) ((pos "OA%" ) ))
( "uninhabited"  j ( uh2 n i n h a1 b i t i d ) ((pos "OA%" ) ))
( "uninhibited"  j ( uh2 n i n h i1 b i t i d ) ((pos "OA%" ) ))
( "uninitiated"  j ( uh2 n i n i1 sh i ei t i d ) ((pos "OA%" ) ))
( "uninjured"  j ( uh1 n i1 n jh @ d ) ((pos "OA%" ) ))
( "uninspired"  j ( uh2 n i n s p ai1 @ d ) ((pos "OA%" ) ))
( "uninspiring"  j ( uh2 n i n s p ai1 @ r i ng ) ((pos "OA%" ) ))
( "uninsured"  j ( uh2 n i n sh u@1 d ) ((pos "OA%" ) ))
( "unintelligent"  j ( uh2 n i n t e1 l i jh @ n t ) ((pos "OA%" ) ))
( "unintelligently"  a ( uh2 n i n t e1 l i jh @ n t l ii ) ((pos "Pu%" ) ))
( "unintelligible"  j ( uh2 n i n t e1 l i jh @ b l ) ((pos "OA%" ) ))
( "unintelligibly"  a ( uh2 n i n t e1 l i jh @ b l ii ) ((pos "Pu%" ) ))
( "unintended"  j ( uh2 n i n t e1 n d i d ) ((pos "OA%" ) ))
( "unintentional"  j ( uh2 n i n t e1 n sh @ n @ l ) ((pos "OA%" ) ))
( "unintentionally"  a ( uh2 n i n t e1 n sh @ n @ l ii ) ((pos "Pu%" ) ))
( "uninterested"  j ( uh1 n i1 n t r @ s t i d ) ((pos "OA%" ) ))
( "uninteresting"  j ( uh1 n i1 n t r @ s t i ng ) ((pos "OA%" ) ))
( "uninterestingly"  a ( uh1 n i1 n t r @ s t i ng l ii ) ((pos "Pu%" ) ))
( "uninterrupted"  j ( uh2 n i n t @ r uh1 p t i d ) ((pos "OA%" ) ))
( "uninterruptedly"  a ( uh2 n i n t @ r uh1 p t i d l ii ) ((pos "Pu%" ) ))
( "uninvited"  j ( uh2 n i n v ai1 t i d ) ((pos "OA%" ) ))
( "uninviting"  j ( uh2 n i n v ai1 t i ng ) ((pos "OA%" ) ))
( "union"  n ( y uu1 n i@ n ) ((pos "M6%" ) ))
( "unionist"  n ( y uu1 n i@ n i s t ) ((pos "K6%" ) ))
( "unionists"  n ( y uu1 n i@ n i s t s ) ((pos "Kj%" ) ))
( "unions"  n ( y uu1 n i@ n z ) ((pos "Mj%" ) ))
( "unique"  j ( y uu1 n ii1 k ) ((pos "OA%" ) ))
( "uniquely"  a ( y uu1 n ii1 k l ii ) ((pos "Pu%" ) ))
( "uniqueness"  n ( y uu1 n ii1 k n @ s ) ((pos "L@%" ) ))
( "unisex"  j ( y uu1 n i s e k s ) ((pos "OA%" ) ))
( "unison"  n ( y uu1 n i s n ) ((pos "L@%" ) ))
( "unit"  n ( y uu1 n i t ) ((pos "K6%" ) ))
( "unitary"  j ( y uu1 n i t r ii ) ((pos "OA%" ) ))
( "unite"  v ( y uu1 n ai1 t ) ((pos "J2%" ) (subcat "2A" "3A" "4A" )))
( "united"  v ( y uu1 n ai1 t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "3A" "4A" )))
( "unitedly"  a ( y uu1 n ai1 t i d l ii ) ((pos "Pu$" ) ))
( "unites"  v ( y uu1 n ai1 t s ) ((pos "Ja%" ) (subcat "2A" "3A" "4A" )))
( "unities"  n ( y uu1 n i t i z ) ((pos "Mj%" ) ))
( "uniting"  v ( y uu1 n ai1 t i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "4A" )))
( "units"  n ( y uu1 n i t s ) ((pos "Kj%" ) ))
( "unity"  n ( y uu1 n i t ii ) ((pos "M8%" ) ))
( "universal"  j ( y uu2 n i v @@1 s l ) ((pos "OA%" ) ))
( "universality"  n ( y uu2 n i v @@ s a1 l i t ii ) ((pos "L@%" ) ))
( "universally"  a ( y uu2 n i v @@1 s @ l ii ) ((pos "Pu%" ) ))
( "universe"  n ( y uu1 n i v @@ s ) ((pos "K6%" ) ))
( "universes"  n ( y uu1 n i v @@ s i z ) ((pos "Kj%" ) ))
( "universities"  n ( y uu2 n i v @@1 s i t i z ) ((pos "Kj%" ) ))
( "university"  n ( y uu2 n i v @@1 s i t ii ) ((pos "K8%" ) ))
( "unjust"  j ( uh1 n jh uh1 s t ) ((pos "OA%" ) ))
( "unjustifiable"  j ( uh2 n jh uh s t i f ai1 @ b l ) ((pos "OA%" ) ))
( "unjustifiably"  a ( uh2 n jh uh s t i f ai1 @ b l ii ) ((pos "Pu%" ) ))
( "unjustified"  j ( uh1 n jh uh1 s t i f ai d ) ((pos "OA%" ) ))
( "unjustly"  a ( uh1 n jh uh1 s t l ii ) ((pos "Pu%" ) ))
( "unkempt"  j ( uh2 n k e1 m p t ) ((pos "OA%" ) ))
( "unkind"  j ( uh1 n k ai1 n d ) ((pos "OA%" ) ))
( "unkindly"  a ( uh1 n k ai1 n d l ii ) ((pos "Pu%" ) ))
( "unkissed"  j ( uh1 n k i1 s t ) ((pos "OA%" ) ))
( "unknowable"  j ( uh1 n n ou1 @ b l ) ((pos "OA%" ) ))
( "unknowing"  j ( uh1 n n ou1 i ng ) ((pos "OA%" ) ))
( "unknowingly"  a ( uh1 n n ou1 i ng l ii ) ((pos "Pu%" ) ))
( "unknown"  n ( uh2 n n ou1 n ) ((pos "K6%" "OA%" ) ))
( "unknowns"  n ( uh1 n n ou1 n z ) ((pos "Kj$" ) ))
( "unlabelled"  j ( uh1 n l ei1 b l d ) ((pos "OA%" ) ))
( "unladylike"  j ( uh1 n l ei1 d i l ai k ) ((pos "OA%" ) ))
( "unlamented"  j ( uh2 n l @ m e1 n t i d ) ((pos "OA%" ) ))
( "unlatched"  j ( uh1 n l a1 ch t ) ((pos "OA%" ) ))
( "unlawful"  j ( uh1 n l oo1 f @ l ) ((pos "OA%" ) ))
( "unlawfully"  a ( uh1 n l oo1 f @ l ii ) ((pos "Pu%" ) ))
( "unlearn"  v ( uh2 n l @@1 n ) ((pos "H0%" ) (subcat )))
( "unlearned"  v ( uh2 n l @@1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unlearning"  v ( uh2 n l @@1 n i ng ) ((pos "Hb%" ) (subcat )))
( "unlearns"  v ( uh2 n l @@1 n z ) ((pos "Ha%" ) (subcat )))
( "unlearnt"  v ( uh2 n l @@1 n t ) ((pos "Hd%" ) (subcat )))
( "unleash"  v ( uh1 n l ii1 sh ) ((pos "H1%" ) (subcat )))
( "unleashed"  v ( uh1 n l ii1 sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unleashes"  v ( uh1 n l ii1 sh i z ) ((pos "Ha%" ) (subcat )))
( "unleashing"  v ( uh1 n l ii1 sh i ng ) ((pos "Hb%" ) (subcat )))
( "unleavened"  j ( uh2 n l e1 v n d ) ((pos "OA%" ) ))
( "unless"  cc ( @1 n l e1 s ) ((pos "V-%" ) ))
( "unlettered"  j ( uh2 n l e1 t @ d ) ((pos "OA%" ) ))
( "unlicensed"  j ( uh1 n l ai1 s n s t ) ((pos "OA%" ) ))
( "unlighted"  j ( uh1 n l ai1 t i d ) ((pos "OA%" ) ))
( "unlikable"  j ( uh1 n l ai1 k @ b l ) ((pos "OA%" ) ))
( "unlike"  j ( uh2 n l ai1 k ) ((pos "Op%" "T-%" ) ))
( "unlikely"  j ( uh1 n l ai1 k l ii ) ((pos "OA%" ) ))
( "unlimited"  j ( uh1 n l i1 m i t i d ) ((pos "OA%" ) ))
( "unlined"  j ( uh1 n l ai1 n d ) ((pos "OA%" ) ))
( "unlisted"  j ( uh1 n l i1 s t i d ) ((pos "OA%" ) ))
( "unlit"  j ( uh1 n l i1 t ) ((pos "OA%" ) ))
( "unliterary"  j ( uh1 n l i1 t @ r @ r ii ) ((pos "OA%" ) ))
( "unload"  v ( uh1 n l ou1 d ) ((pos "J0%" ) (subcat "2A" "6A" )))
( "unloaded"  v ( uh1 n l ou1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "unloading"  v ( uh1 n l ou1 d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "unloads"  v ( uh1 n l ou1 d z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "unlocated"  j ( uh2 n l ou k ei1 t i d ) ((pos "OA%" ) ))
( "unlock"  v ( uh1 n l o1 k ) ((pos "J0%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unlocked"  v ( uh1 n l o1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unlocking"  v ( uh1 n l o1 k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unlocks"  v ( uh1 n l o1 k s ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" "15A" )))
( "unlooked-for"  j ( uh1 n l u1 k t - f oo r ) ((pos "OA%" ) ))
( "unloose"  v ( uh1 n l uu1 s ) ((pos "H2%" ) (subcat )))
( "unloosed"  v ( uh1 n l uu1 s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unlooses"  v ( uh1 n l uu1 s i z ) ((pos "Ha%" ) (subcat )))
( "unloosing"  v ( uh1 n l uu1 s i ng ) ((pos "Hb%" ) (subcat )))
( "unlovable"  j ( uh1 n l uh1 v @ b l ) ((pos "OA%" ) ))
( "unloved"  j ( uh1 ng l uh1 v d ) ((pos "OA%" ) ))
( "unlovely"  j ( uh1 n l uh1 v l ii ) ((pos "OA%" ) ))
( "unluckily"  a ( uh1 n l uh1 k i l ii ) ((pos "Pu%" ) ))
( "unlucky"  j ( uh1 n l uh1 k ii ) ((pos "OA%" ) ))
( "unmade"  j ( uh1 n m ei1 d ) ((pos "OA%" ) ))
( "unman"  v ( uh2 n m a1 n ) ((pos "H4$" ) (subcat )))
( "unmanageable"  j ( uh1 n m a1 n i jh @ b l ) ((pos "OA%" ) ))
( "unmanageably"  a ( uh1 n m a1 n i jh @ b l ii ) ((pos "Pu%" ) ))
( "unmanly"  j ( uh1 n m a1 n l ii ) ((pos "OA%" ) ))
( "unmanned"  v ( uh2 n m a1 n d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "unmannered"  j ( uh1 n m a1 n @ d ) ((pos "OA$" ) ))
( "unmannerly"  j ( uh1 n m a1 n @ l ii ) ((pos "OA%" ) ))
( "unmanning"  v ( uh2 n m a1 n i ng ) ((pos "Hb$" ) (subcat )))
( "unmans"  v ( uh2 n m a1 n z ) ((pos "Ha$" ) (subcat )))
( "unmarked"  j ( uh1 n m aa1 k t ) ((pos "OA%" ) ))
( "unmarried"  j ( uh1 n m a1 r i d ) ((pos "OA%" ) ))
( "unmask"  v ( uh1 n m aa1 s k ) ((pos "J0%" ) (subcat "2A" )))
( "unmasked"  v ( uh1 n m aa1 s k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unmasking"  v ( uh1 n m aa1 s k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unmasks"  v ( uh1 n m aa1 s k s ) ((pos "Ja%" ) (subcat "2A" )))
( "unmatchable"  j ( uh1 n m a1 ch @ b l ) ((pos "OA%" ) ))
( "unmatched"  j ( uh1 n m a1 ch t ) ((pos "OA%" ) ))
( "unmechanized"  j ( uh1 n m e1 k @ n ai z d ) ((pos "OA%" ) ))
( "unmemorable"  j ( uh1 n m e1 m @ r @ b l ) ((pos "OA%" ) ))
( "unmemorably"  a ( uh1 n m e1 m @ r @ b l ii ) ((pos "Pu%" ) ))
( "unmentionable"  j ( uh1 n m e1 n sh @ n @ b l ) ((pos "OA%" ) ))
( "unmerciful"  j ( uh1 n m @@1 s i f @ l ) ((pos "OA%" ) ))
( "unmercifully"  a ( uh1 n m @@1 s i f @ l ii ) ((pos "Pu%" ) ))
( "unmerited"  j ( uh1 n m e1 r i t i d ) ((pos "OA%" ) ))
( "unmindful"  j ( uh1 n m ai1 n d f @ l ) ((pos "OA%" ) ))
( "unmingled"  j ( uh1 n m i1 ng g l d ) ((pos "OA%" ) ))
( "unmistakable"  j ( uh2 n m i s t ei1 k @ b l ) ((pos "OA%" ) ))
( "unmistakably"  a ( uh2 n m i s t ei1 k @ b l ii ) ((pos "Pu%" ) ))
( "unmitigated"  j ( uh1 n m i1 t i g ei t i d ) ((pos "OA%" ) ))
( "unmixed"  j ( uh1 n m i1 k s t ) ((pos "OA%" ) ))
( "unmodernized"  j ( uh1 n m o1 d @ n ai z d ) ((pos "OA%" ) ))
( "unmodified"  j ( uh1 n m o1 d i f ai d ) ((pos "OA%" ) ))
( "unmolested"  j ( uh2 n m @ l e1 s t i d ) ((pos "OA%" ) ))
( "unmourned"  j ( uh1 n m oo1 n d ) ((pos "OA%" ) ))
( "unmovable"  j ( uh1 n m uu1 v @ b l ) ((pos "OA%" ) ))
( "unmoved"  j ( uh1 n m uu1 v d ) ((pos "OA%" ) ))
( "unmoving"  j ( uh1 n m uu1 v i ng ) ((pos "OA%" ) ))
( "unmusical"  j ( uh1 n m y uu1 z i k l ) ((pos "OA%" ) ))
( "unmusically"  a ( uh1 n m y uu1 z i k l ii ) ((pos "Pu%" ) ))
( "unnamed"  j ( uh1 n n ei1 m d ) ((pos "OA%" ) ))
( "unnatural"  j ( uh1 n n a1 ch r @ l ) ((pos "OA%" ) ))
( "unnaturally"  a ( uh1 n n a1 ch r @ l ii ) ((pos "Pu%" ) ))
( "unnecessarily"  a ( uh2 n n e1 s @ s @ r @ l ii ) ((pos "Pu%" ) ))
( "unnecessary"  j ( uh1 n n e1 s @ s r ii ) ((pos "OA%" ) ))
( "unnerve"  v ( uh2 n n @@1 v ) ((pos "H2%" ) (subcat )))
( "unnerved"  v ( uh2 n n @@1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unnerves"  v ( uh2 n n @@1 v z ) ((pos "Ha%" ) (subcat )))
( "unnerving"  v ( uh2 n n @@1 v i ng ) ((pos "Hb%" ) (subcat )))
( "unnoticeable"  j ( uh1 n n ou1 t i s @ b l ) ((pos "OA%" ) ))
( "unnoticed"  j ( uh2 n n ou1 t i s t ) ((pos "OA%" ) ))
( "unnumbered"  j ( uh2 n n uh1 m b @ d ) ((pos "OA%" ) ))
( "unobjectionable"  j ( uh2 n @ b jh e1 k sh @ n @ b l ) ((pos "OA%" ) ))
( "unobservant"  j ( uh2 n @ b z @@1 v @ n t ) ((pos "OA%" ) ))
( "unobserved"  j ( uh2 n @ b z @@1 v d ) ((pos "OA%" ) ))
( "unobtainable"  j ( uh2 n @ b t ei1 n @ b l ) ((pos "OA%" ) ))
( "unobtrusive"  j ( uh2 n @ b t r uu1 s i v ) ((pos "OA%" ) ))
( "unobtrusively"  a ( uh2 n @ b t r uu1 s i v l ii ) ((pos "Pu%" ) ))
( "unobvious"  j ( uh1 n o1 b v i@ s ) ((pos "OA%" ) ))
( "unoccupied"  j ( uh1 n o1 k y u p ai d ) ((pos "OA%" ) ))
( "unofficial"  j ( uh2 n @ f i1 sh l ) ((pos "OA%" ) ))
( "unofficially"  a ( uh2 n @ f i1 sh @ l ii ) ((pos "Pu%" ) ))
( "unopen"  j ( uh1 n ou1 p @ n ) ((pos "OA%" ) ))
( "unopened"  j ( uh1 n ou1 p @ n d ) ((pos "OA%" ) ))
( "unopposed"  j ( uh2 n @ p ou1 z d ) ((pos "OA%" ) ))
( "unoriginal"  j ( uh2 n @ r i1 jh @ n l ) ((pos "OA%" ) ))
( "unorthodox"  j ( uh2 n oo1 th @ d o k s ) ((pos "OA%" ) ))
( "unpack"  v ( uh2 n p a1 k ) ((pos "J0%" ) (subcat "2A" )))
( "unpacked"  v ( uh2 n p a1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unpacking"  v ( uh2 n p a1 k i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unpacks"  v ( uh2 n p a1 k s ) ((pos "Ja%" ) (subcat "2A" )))
( "unpaid"  j ( uh1 n p ei1 d ) ((pos "OA%" ) ))
( "unpainted"  j ( uh1 n p ei1 n t i d ) ((pos "OA%" ) ))
( "unpalatable"  j ( uh1 n p a1 l @ t @ b l ) ((pos "OA%" ) ))
( "unpalatably"  a ( uh1 n p a1 l @ t @ b l ii ) ((pos "Pu%" ) ))
( "unparalleled"  j ( uh1 n p a1 r @ l e l d ) ((pos "OA%" ) ))
( "unpardonable"  j ( uh1 n p aa1 d n @ b l ) ((pos "OA%" ) ))
( "unpardonably"  a ( uh1 n p aa1 d n @ b l ii ) ((pos "Pu%" ) ))
( "unparliamentary"  j ( uh2 n p aa2 l @ m e1 n t r ii ) ((pos "OA%" ) ))
( "unpatriotic"  j ( uh2 n p a t r i o1 t i k ) ((pos "OA%" ) ))
( "unpatriotically"  a ( uh2 n p a t r i o1 t i k l ii ) ((pos "Pu%" ) ))
( "unpaved"  j ( uh1 n p ei1 v d ) ((pos "OA%" ) ))
( "unpersuaded"  j ( uh2 n p @ s w ei1 d i d ) ((pos "OA%" ) ))
( "unperturbed"  j ( uh2 n p @ t @@1 b d ) ((pos "OA%" ) ))
( "unpick"  v ( uh1 n p i1 k ) ((pos "J0%" ) (subcat "3A" "6A" )))
( "unpicked"  v ( uh1 n p i1 k t ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" )))
( "unpicking"  v ( uh1 n p i1 k i ng ) ((pos "Jb%" ) (subcat "3A" "6A" )))
( "unpicks"  v ( uh1 n p i1 k s ) ((pos "Ja%" ) (subcat "3A" "6A" )))
( "unplaced"  j ( uh2 n p l ei1 s t ) ((pos "OA%" ) ))
( "unplanned"  j ( uh1 n p l a1 n d ) ((pos "OA%" ) ))
( "unplayable"  j ( uh1 n p l ei1 @ b l ) ((pos "OA%" ) ))
( "unpleasant"  j ( uh1 n p l e1 z n t ) ((pos "OA%" ) ))
( "unpleasantly"  a ( uh1 n p l e1 z n t l ii ) ((pos "Pu%" ) ))
( "unpleasantness"  n ( uh1 n p l e1 z n t n @ s ) ((pos "M7%" ) ))
( "unpleasantnesses"  n ( uh1 n p l e1 z n t n @ s i z ) ((pos "Mj%" ) ))
( "unplug"  v ( uh1 n p l uh1 g ) ((pos "J4%" ) (subcat "2C" "6A" )))
( "unplugged"  v ( uh1 n p l uh1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" )))
( "unplugging"  v ( uh1 n p l uh1 g i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "unplugs"  v ( uh1 n p l uh1 g z ) ((pos "Ja%" ) (subcat "2C" "6A" )))
( "unplumbed"  j ( uh1 n p l uh1 m d ) ((pos "OA%" ) ))
( "unpolluted"  j ( uh2 n p @ l uu1 t i d ) ((pos "OA%" ) ))
( "unpompous"  j ( uh1 n p o1 m p @ s ) ((pos "OA%" ) ))
( "unpopular"  j ( uh1 n p o1 p y u l @ r ) ((pos "OA%" ) ))
( "unpopularity"  n ( uh2 n p o p y u l a1 r i t ii ) ((pos "L@%" ) ))
( "unpractised"  j ( uh1 n p r a1 k t i s t ) ((pos "OA%" ) ))
( "unprecedented"  j ( uh1 n p r e1 s i d e n t i d ) ((pos "OA%" ) ))
( "unprecedentedly"  a ( uh1 n p r e1 s i d e n t i d l ii ) ((pos "Pu%" ) ))
( "unpredictability"  n ( uh2 n p r i d i2 k t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "unpredictable"  j ( uh2 n p r i d i1 k t @ b l ) ((pos "OA%" ) ))
( "unpredictably"  a ( uh2 n p r i d i1 k t @ b l ii ) ((pos "Pu%" ) ))
( "unpredicted"  j ( uh2 n p r i d i1 k t i d ) ((pos "OA%" ) ))
( "unprejudiced"  j ( uh1 n p r e1 jh u d i s t ) ((pos "OA%" ) ))
( "unpremeditated"  j ( uh2 n p r ii m e1 d i t ei t i d ) ((pos "OA%" ) ))
( "unprepared"  j ( uh2 n p r i p e@1 d ) ((pos "OA%" ) ))
( "unprepossessing"  j ( uh2 n p r ii p @ z e1 s i ng ) ((pos "OA$" ) ))
( "unpretentious"  j ( uh2 n p r i t e1 n sh @ s ) ((pos "OA%" ) ))
( "unprincipled"  j ( uh1 n p r i1 n s @ p l d ) ((pos "OA%" ) ))
( "unprintable"  j ( uh1 n p r i1 n t @ b l ) ((pos "OA%" ) ))
( "unproductive"  j ( uh2 n p r @ d uh1 k t i v ) ((pos "OA%" ) ))
( "unproductively"  a ( uh2 n p r @ d uh1 k t i v l ii ) ((pos "Pu%" ) ))
( "unprofessional"  j ( uh2 n p r @ f e1 sh @ n l ) ((pos "OA%" ) ))
( "unprofitable"  j ( uh1 n p r o1 f i t @ b l ) ((pos "OA%" ) ))
( "unprofitably"  a ( uh1 n p r o1 f i t @ b l ii ) ((pos "Pu%" ) ))
( "unpromising"  j ( uh1 n p r o1 m i s i ng ) ((pos "OA%" ) ))
( "unprompted"  j ( uh2 n p r o1 m p t i d ) ((pos "OA%" ) ))
( "unpronounceable"  j ( uh2 n p r @ n au1 n s @ b l ) ((pos "OA%" ) ))
( "unpropitious"  j ( uh2 n p r @ p i1 sh @ s ) ((pos "OA%" ) ))
( "unprotected"  j ( uh2 n p r @ t e1 k t i d ) ((pos "OA%" ) ))
( "unproved"  j ( uh1 n p r uu1 v d ) ((pos "OA%" ) ))
( "unproven"  j ( uh1 n p r uu1 v n ) ((pos "Op%" ) ))
( "unprovided"  j ( uh2 n p r @ v ai1 d i d ) ((pos "OA%" ) ))
( "unprovoked"  j ( uh2 n p r @ v ou1 k t ) ((pos "OA%" ) ))
( "unpublished"  j ( uh1 n p uh1 b l i sh t ) ((pos "OA%" ) ))
( "unpunished"  j ( uh1 n p uh1 n i sh t ) ((pos "OA%" ) ))
( "unputdownable"  j ( uh2 n p u t d au1 n @ b l ) ((pos "OA$" ) ))
( "unqualified"  j ( uh1 n k w o1 l i f ai d ) ((pos "OA%" ) ))
( "unquestionable"  j ( uh1 n k w e1 s ch @ n @ b l ) ((pos "OA%" ) ))
( "unquestionably"  a ( uh1 n k w e1 s ch @ n @ b l ii ) ((pos "Pu%" ) ))
( "unquestioned"  j ( uh1 n k w e1 s ch @ n d ) ((pos "OA%" ) ))
( "unquestioning"  j ( uh1 n k w e1 s ch @ n i ng ) ((pos "OA%" ) ))
( "unquiet"  j ( uh1 n k w ai1 @ t ) ((pos "OA%" ) ))
( "unquotable"  j ( uh1 n k w ou1 t @ b l ) ((pos "OA%" ) ))
( "unquote"  a ( uh1 n k w ou1 t ) ((pos "Pu%" ) ))
( "unravel"  v ( uh1 n r a1 v @ l ) ((pos "J4%" ) (subcat "2A" )))
( "unravelled"  v ( uh1 n r a1 v @ l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unravelling"  v ( uh1 n r a1 v @ l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unravels"  v ( uh1 n r a1 v @ l z ) ((pos "Ja%" ) (subcat "2A" )))
( "unreached"  j ( uh1 n r ii1 ch t ) ((pos "OA%" ) ))
( "unread"  j ( uh1 n r e1 d ) ((pos "OA%" ) ))
( "unreadable"  j ( uh1 n r ii1 d @ b l ) ((pos "OA%" ) ))
( "unready"  j ( uh1 n r e1 d ii ) ((pos "OA%" ) ))
( "unreal"  j ( uh1 n r i@1 l ) ((pos "OA%" ) ))
( "unrealistic"  j ( uh2 n r i@ l i1 s t i k ) ((pos "OA%" ) ))
( "unrealistically"  a ( uh2 n r i@ l i1 s t i k l ii ) ((pos "Pu%" ) ))
( "unreality"  n ( uh2 n r i a1 l i t ii ) ((pos "L@%" ) ))
( "unrealized"  j ( uh1 n r i@1 l ai z d ) ((pos "OA%" ) ))
( "unreasonable"  j ( uh1 n r ii1 z n @ b l ) ((pos "OA%" ) ))
( "unreasonably"  a ( uh1 n r ii1 z n @ b l ii ) ((pos "Pu%" ) ))
( "unreasoning"  j ( uh1 n r ii1 z @ n i ng ) ((pos "OA%" ) ))
( "unreceptive"  j ( uh2 n r i s e1 p t i v ) ((pos "OA%" ) ))
( "unreciprocated"  j ( uh2 n r i s i1 p r @ k ei t i d ) ((pos "OA%" ) ))
( "unrecognizable"  j ( uh1 n r e1 k @ g n ai z @ b l ) ((pos "OA%" ) ))
( "unrecognizably"  a ( uh2 n r e k @ g n ai1 z @ b l ii ) ((pos "Pu%" ) ))
( "unrecognized"  j ( uh1 n r e1 k @ g n ai z d ) ((pos "OA%" ) ))
( "unreconciled"  j ( uh1 n r e1 k @ n s ai l d ) ((pos "OA%" ) ))
( "unrecorded"  j ( uh2 n r i k oo1 d i d ) ((pos "OA%" ) ))
( "unredeemable"  j ( uh2 n r i d ii1 m @ b l ) ((pos "OA%" ) ))
( "unrefined"  j ( uh2 n r i f ai1 n d ) ((pos "OA%" ) ))
( "unreflective"  j ( uh2 n r i f l e1 k t i v ) ((pos "OA%" ) ))
( "unregenerate"  j ( uh2 n r i jh e1 n @ r @ t ) ((pos "OA%" ) ))
( "unregistered"  j ( uh1 n r e1 jh i s t @ d ) ((pos "OA%" ) ))
( "unrehearsed"  j ( uh2 n r i h @@1 s t ) ((pos "OA%" ) ))
( "unrelated"  j ( uh2 n r i l ei1 t i d ) ((pos "OA%" ) ))
( "unrelaxed"  j ( uh2 n r i l a1 k s t ) ((pos "OA%" ) ))
( "unrelenting"  j ( uh2 n r i l e1 n t i ng ) ((pos "OA%" ) ))
( "unreliable"  j ( uh2 n r i l ai1 @ b l ) ((pos "OA%" ) ))
( "unreliably"  a ( uh2 n r i l ai1 @ b l ii ) ((pos "Pu%" ) ))
( "unrelieved"  j ( uh2 n r i l ii1 v d ) ((pos "OA%" ) ))
( "unremarkable"  j ( uh2 n r i m aa1 k @ b l ) ((pos "OA%" ) ))
( "unremarkably"  a ( uh2 n r i m aa1 k @ b l ii ) ((pos "Pu%" ) ))
( "unremitting"  j ( uh2 n r i m i1 t i ng ) ((pos "OA%" ) ))
( "unrepeatable"  j ( uh2 n r i p ii1 t @ b l ) ((pos "OA%" ) ))
( "unrepentant"  j ( uh2 n r i p e1 n t @ n t ) ((pos "OA%" ) ))
( "unrepresentative"  j ( uh2 n r e p r i z e1 n t @ t i v ) ((pos "OA%" ) ))
( "unrequested"  j ( uh2 n r i k w e1 s t i d ) ((pos "OA%" ) ))
( "unrequited"  j ( uh2 n r i k w ai1 t i d ) ((pos "OA%" ) ))
( "unreserved"  j ( uh2 n r i z @@1 v d ) ((pos "OA%" ) ))
( "unreservedly"  a ( uh2 n r i z @@1 v i d l ii ) ((pos "Pu%" ) ))
( "unresisting"  j ( uh2 n r i z i1 s t i ng ) ((pos "OA%" ) ))
( "unresolved"  j ( uh2 n r i z o1 l v d ) ((pos "OA%" ) ))
( "unrest"  n ( uh1 n r e1 s t ) ((pos "L@%" ) ))
( "unrestrained"  j ( uh2 n r i s t r ei1 n d ) ((pos "OA%" ) ))
( "unrestricted"  j ( uh2 n r i s t r i1 k t i d ) ((pos "OA%" ) ))
( "unrevised"  j ( uh2 n r i v ai1 z d ) ((pos "OA%" ) ))
( "unrewarded"  j ( uh2 n r i w oo1 d i d ) ((pos "OA%" ) ))
( "unrewarding"  j ( uh2 n r i w oo1 d i ng ) ((pos "OA%" ) ))
( "unrhythmical"  j ( uh1 n r i1 dh m i k l ) ((pos "OA%" ) ))
( "unrighteous"  j ( uh1 n r ai1 ch @ s ) ((pos "OA%" ) ))
( "unrighteously"  a ( uh1 n r ai1 ch @ s l ii ) ((pos "Pu%" ) ))
( "unripe"  j ( uh1 n r ai1 p ) ((pos "OA%" ) ))
( "unripened"  j ( uh1 n r ai1 p @ n d ) ((pos "OA%" ) ))
( "unrivalled"  j ( uh1 n r ai1 v @ l d ) ((pos "OA%" ) ))
( "unroll"  v ( uh1 n r ou1 l ) ((pos "J0%" ) (subcat "2A" )))
( "unrolled"  v ( uh1 n r ou1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unrolling"  v ( uh1 n r ou1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unrolls"  v ( uh1 n r ou1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "unromantic"  j ( uh2 n r @ m a1 n t i k ) ((pos "OA%" ) ))
( "unromantically"  a ( uh2 n r @ m a1 n t i k l ii ) ((pos "Pu%" ) ))
( "unruffled"  j ( uh1 n r uh1 f l d ) ((pos "OA%" ) ))
( "unrulier"  j ( uh1 n r uu1 l i@ r ) ((pos "Or$" ) ))
( "unruliest"  j ( uh1 n r uu1 l i i s t ) ((pos "Os%" ) ))
( "unruly"  j ( uh1 n r uu1 l ii ) ((pos "OD%" ) ))
( "unsaddle"  v ( uh1 n s a1 d l ) ((pos "H2%" ) (subcat )))
( "unsaddled"  v ( uh1 n s a1 d l d ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "unsaddles"  v ( uh1 n s a1 d l z ) ((pos "Ha%" ) (subcat )))
( "unsaddling"  v ( uh1 n s a1 d l i ng ) ((pos "Hb%" ) (subcat )))
( "unsafe"  j ( uh1 n s ei1 f ) ((pos "OA%" ) ))
( "unsaid"  v ( uh2 n s e1 d ) ((pos "Hc$" "Hd%" "OA%" ) (subcat )))
( "unsaleable"  j ( uh1 n s ei1 l @ b l ) ((pos "OA%" ) ))
( "unsalted"  j ( uh1 n s oo1 l t i d ) ((pos "OA%" ) ))
( "unsatisfactorily"  a ( uh2 n s a t i s f a1 k t @ r i l ii ) ((pos "Pu%" ) ))
( "unsatisfactory"  j ( uh2 n s a t i s f a1 k t @ r ii ) ((pos "OA%" ) ))
( "unsatisfied"  j ( uh1 n s a1 t i s f ai d ) ((pos "OA%" ) ))
( "unsatisfying"  j ( uh1 n s a1 t i s f ai i ng ) ((pos "OA%" ) ))
( "unsaturated"  j ( uh1 n s a1 ch @ r ei t i d ) ((pos "OA%" ) ))
( "unsavoury"  j ( uh1 n s ei1 v @ r ii ) ((pos "OA%" ) ))
( "unsay"  v ( uh2 n s ei1 ) ((pos "H5$" ) (subcat )))
( "unsaying"  v ( uh2 n s ei1 i ng ) ((pos "Hb$" ) (subcat )))
( "unsays"  v ( uh2 n s ei1 z ) ((pos "Ha$" ) (subcat )))
( "unscathed"  j ( uh1 n s k ei1 dh d ) ((pos "OA%" ) ))
( "unscheduled"  j ( uh1 n sh e1 d y uu l d ) ((pos "OA%" ) ))
( "unscientific"  j ( uh2 n s ai @ n t i1 f i k ) ((pos "OA%" ) ))
( "unscientifically"  a ( uh2 n s ai @ n t i1 f i k l ii ) ((pos "Pu%" ) ))
( "unscramble"  v ( uh2 n s k r a1 m b l ) ((pos "H2%" ) ))
( "unscrambled"  v ( uh2 n s k r a1 m b l d ) ((pos "Hc%" "Hd%" ) ))
( "unscrambles"  v ( uh2 n s k r a1 m b l z ) ((pos "Ha%" ) ))
( "unscrambling"  v ( uh2 n s k r a1 m b l i ng ) ((pos "Hb%" ) ))
( "unscrew"  v ( uh1 n s k r uu1 ) ((pos "J0%" ) (subcat "2A" "6A" "15A" )))
( "unscrewed"  v ( uh1 n s k r uu1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "unscrewing"  v ( uh1 n s k r uu1 i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "unscrews"  v ( uh1 n s k r uu1 z ) ((pos "Ja%" ) (subcat "2A" "6A" "15A" )))
( "unscripted"  j ( uh1 n s k r i1 p t i d ) ((pos "OA%" ) ))
( "unscrupulous"  j ( uh1 n s k r uu1 p y u l @ s ) ((pos "OA%" ) ))
( "unscrupulously"  a ( uh1 n s k r uu1 p y u l @ s l ii ) ((pos "Pu%" ) ))
( "unsealed"  j ( uh1 n s ii1 l d ) ((pos "OA%" ) ))
( "unseasonable"  j ( uh1 n s ii1 z n @ b l ) ((pos "OA%" ) ))
( "unseasonably"  a ( uh1 n s ii1 z @ n @ b l ii ) ((pos "Pu%" ) ))
( "unseasoned"  j ( uh2 n s ii1 z n d ) ((pos "OA%" ) ))
( "unseat"  v ( uh2 n s ii1 t ) ((pos "H0$" ) (subcat )))
( "unseated"  v ( uh2 n s ii1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unseating"  v ( uh2 n s ii1 t i ng ) ((pos "Hb$" ) (subcat )))
( "unseats"  v ( uh2 n s ii1 t s ) ((pos "Ha$" ) (subcat )))
( "unseaworthy"  j ( uh1 n s ii1 w @@ dh ii ) ((pos "OA%" ) ))
( "unsecured"  j ( uh1 n s i k y u@1 d ) ((pos "OA%" ) ))
( "unseeded"  j ( uh1 n s ii1 d i d ) ((pos "OA%" ) ))
( "unseeing"  j ( uh1 n s ii1 i ng ) ((pos "OA%" ) ))
( "unseemly"  j ( uh1 n s ii1 m l ii ) ((pos "OA%" ) ))
( "unseen"  n ( uh1 n s ii1 n ) ((pos "K6%" "OA%" ) ))
( "unseens"  n ( uh1 n s ii1 n z ) ((pos "Kj%" ) ))
( "unselected"  j ( uh1 n s i l e1 k t i d ) ((pos "OA%" ) ))
( "unselective"  j ( uh1 n s i l e1 k t i v ) ((pos "OA%" ) ))
( "unselfconscious"  j ( uh2 n s e l f k o1 n sh @ s ) ((pos "OA%" ) ))
( "unselfconsciously"  a ( uh2 n s e l f k o1 n sh @ s l ii ) ((pos "Pu%" ) ))
( "unselfish"  j ( uh1 n s e1 l f i sh ) ((pos "OA%" ) ))
( "unselfishly"  a ( uh1 n s e1 l f i sh l ii ) ((pos "Pu%" ) ))
( "unselfishness"  n ( uh1 n s e1 l f i sh n @ s ) ((pos "L@%" ) ))
( "unsettle"  v ( uh2 n s e1 t l ) ((pos "H2%" ) (subcat )))
( "unsettled"  v ( uh2 n s e1 t l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unsettles"  v ( uh2 n s e1 t l z ) ((pos "Ha%" ) (subcat )))
( "unsettling"  v ( uh2 n s e1 t l i ng ) ((pos "Hb%" ) (subcat )))
( "unsex"  v ( uh2 n s e1 k s ) ((pos "H1$" ) (subcat )))
( "unsexed"  v ( uh2 n s e1 k s t ) ((pos "Hc$" "Hd$" "OA$" ) (subcat )))
( "unsexes"  v ( uh2 n s e1 k s i z ) ((pos "Ha$" ) (subcat )))
( "unsexing"  v ( uh2 n s e1 k s i ng ) ((pos "Hb$" ) (subcat )))
( "unshakable"  j ( uh1 n sh ei1 k @ b l ) ((pos "OA%" ) ))
( "unshakably"  a ( uh1 n sh ei1 k @ b l ii ) ((pos "Pu%" ) ))
( "unshaved"  j ( uh1 n sh ei1 v d ) ((pos "OA%" ) ))
( "unshaven"  j ( uh1 n sh ei1 v n ) ((pos "OA%" ) ))
( "unsheathe"  v ( uh1 n sh ii1 dh ) ((pos "H2%" ) (subcat )))
( "unsheathed"  v ( uh1 n sh ii1 dh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "unsheathes"  v ( uh1 n sh ii1 dh z ) ((pos "Ha%" ) (subcat )))
( "unsheathing"  v ( uh1 n sh ii1 dh i ng ) ((pos "Hb%" ) (subcat )))
( "unshielded"  j ( uh1 n sh ii1 l d i d ) ((pos "OA%" ) ))
( "unshrinkable"  j ( uh1 n sh r i1 n k @ b l ) ((pos "OA%" ) ))
( "unshrinking"  j ( uh1 n sh r i1 ng k i ng ) ((pos "OA%" ) ))
( "unshuttered"  j ( uh1 n sh uh1 t @ d ) ((pos "OA%" ) ))
( "unsighted"  j ( uh1 n s ai1 t i d ) ((pos "OA%" ) ))
( "unsightliness"  n ( uh1 n s ai1 t l i n @ s ) ((pos "L@%" ) ))
( "unsightly"  j ( uh1 n s ai1 t l ii ) ((pos "OA%" ) ))
( "unsigned"  j ( uh1 n s ai1 n d ) ((pos "OA%" ) ))
( "unsilenced"  j ( uh1 n s ai1 l @ n s t ) ((pos "OA%" ) ))
( "unsinkable"  j ( uh1 n s i1 ng k @ b l ) ((pos "OA%" ) ))
( "unskilled"  j ( uh2 n s k i1 l d ) ((pos "OA%" ) ))
( "unsmiling"  j ( uh1 n s m ai1 l i ng ) ((pos "OA%" ) ))
( "unsociable"  j ( uh1 n s ou1 sh @ b l ) ((pos "OA%" ) ))
( "unsociably"  a ( uh1 n s ou1 sh @ b l ii ) ((pos "Pu%" ) ))
( "unsocial"  j ( uh1 n s ou1 sh l ) ((pos "OA%" ) ))
( "unsold"  j ( uh1 n s ou1 l d ) ((pos "OA%" ) ))
( "unsolicited"  j ( uh2 n s @ l i1 s i t i d ) ((pos "OA%" ) ))
( "unsolved"  j ( uh1 n s o1 l v d ) ((pos "OA%" ) ))
( "unsophisticated"  j ( uh2 n s @ f i1 s t i k ei t i d ) ((pos "OA%" ) ))
( "unsound"  j ( uh2 n s au1 n d ) ((pos "OA%" ) ))
( "unsparing"  j ( uh1 n s p e@1 r i ng ) ((pos "OA%" ) ))
( "unsparingly"  a ( uh1 n s p e@1 r i ng l ii ) ((pos "Pu%" ) ))
( "unspeakable"  j ( uh1 n s p ii1 k @ b l ) ((pos "OA%" ) ))
( "unspeakably"  a ( uh1 n s p ii1 k @ b l ii ) ((pos "Pu%" ) ))
( "unspecialized"  j ( uh1 n s p e1 sh @ l ai z d ) ((pos "OA%" ) ))
( "unspecific"  j ( uh2 n s p @ s i1 f i k ) ((pos "OA%" ) ))
( "unspecifically"  a ( uh2 n s p @ s i1 f i k l ii ) ((pos "Pu%" ) ))
( "unspecified"  j ( uh1 n s p e1 s i f ai d ) ((pos "OA%" ) ))
( "unspectacular"  j ( uh2 n s p e k t a1 k y u l @ r ) ((pos "OA%" ) ))
( "unspoiled"  j ( uh1 n s p oi1 l d ) ((pos "OA%" ) ))
( "unspoilt"  j ( uh1 n s p oi1 l t ) ((pos "OA%" ) ))
( "unspoken"  j ( uh1 n s p ou1 k @ n ) ((pos "OA%" ) ))
( "unsporting"  j ( uh1 n s p oo1 t i ng ) ((pos "OA%" ) ))
( "unsportingly"  a ( uh1 n s p oo1 t i ng l ii ) ((pos "Pu%" ) ))
( "unsportsmanlike"  j ( uh1 n s p oo1 t s m @ n l ai k ) ((pos "OA%" ) ))
( "unspotted"  j ( uh1 n s p o1 t i d ) ((pos "OA%" ) ))
( "unstable"  j ( uh1 n s t ei1 b l ) ((pos "OA%" ) ))
( "unstartling"  j ( uh1 n s t aa1 t l i ng ) ((pos "OA%" ) ))
( "unstated"  j ( uh1 n s t ei1 t i d ) ((pos "OA%" ) ))
( "unstatesmanlike"  j ( uh1 n s t ei1 t s m @ n l ai k ) ((pos "OA%" ) ))
( "unsteadily"  a ( uh1 n s t e1 d i l ii ) ((pos "Pu%" ) ))
( "unsteady"  j ( uh1 n s t e1 d ii ) ((pos "Op%" ) ))
( "unstoppable"  j ( uh1 n s t o1 p @ b l ) ((pos "OA%" ) ))
( "unstrained"  j ( uh1 n s t r ei1 n d ) ((pos "OA%" ) ))
( "unstressed"  j ( uh1 n s t r e1 s t ) ((pos "OA%" ) ))
( "unstructured"  j ( uh1 n s t r uh1 k ch @ d ) ((pos "OA%" ) ))
( "unstrung"  j ( uh2 n s t r uh1 ng ) ((pos "OA$" ) ))
( "unstuck"  j ( uh2 n s t uh1 k ) ((pos "OA%" ) ))
( "unstudied"  j ( uh2 n s t uh1 d i d ) ((pos "OA%" ) ))
( "unsubtle"  j ( uh1 n s uh1 t l ) ((pos "OA%" ) ))
( "unsuccessful"  j ( uh2 n s @ k s e1 s f @ l ) ((pos "OA%" ) ))
( "unsuccessfully"  a ( uh2 n s @ k s e1 s f @ l ii ) ((pos "Pu%" ) ))
( "unsuitability"  n ( uh1 n s uu2 t @ b i1 l i t ii ) ((pos "L@%" ) ))
( "unsuitable"  j ( uh1 n s uu1 t @ b l ) ((pos "OA%" ) ))
( "unsuitably"  a ( uh1 n s uu1 t @ b l ii ) ((pos "Pu%" ) ))
( "unsuited"  j ( uh1 n s uu1 t i d ) ((pos "OA%" ) ))
( "unsullied"  j ( uh1 n s uh1 l i d ) ((pos "OA%" ) ))
( "unsung"  j ( uh2 n s uh1 ng ) ((pos "OA%" ) ))
( "unsupervised"  j ( uh1 n s uu1 p @ v ai z d ) ((pos "OA%" ) ))
( "unsupported"  j ( uh2 n s @ p oo1 t i d ) ((pos "OA%" ) ))
( "unsure"  j ( uh1 n sh u@1 r ) ((pos "OA%" ) ))
( "unsurpassed"  j ( uh2 n s @ p aa1 s t ) ((pos "OA%" ) ))
( "unsuspected"  j ( uh2 n s @ s p e1 k t i d ) ((pos "OA%" ) ))
( "unsuspecting"  j ( uh2 n s @ s p e1 k t i ng ) ((pos "OA%" ) ))
( "unsuspectingly"  a ( uh2 n s @ s p e1 k t i ng l ii ) ((pos "Pu%" ) ))
( "unsuspicious"  j ( uh2 n s @ s p i1 sh @ s ) ((pos "OA%" ) ))
( "unswayed"  j ( uh1 n s w ei1 d ) ((pos "OA%" ) ))
( "unsweetened"  j ( uh1 n s w ii1 t n d ) ((pos "OA%" ) ))
( "unswept"  j ( uh1 n s w e1 p t ) ((pos "OA%" ) ))
( "unswerving"  j ( uh1 n s w @@1 v i ng ) ((pos "OA%" ) ))
( "unswervingly"  a ( uh1 n s w @@1 v i ng l ii ) ((pos "Pu%" ) ))
( "unsyllabic"  j ( uh2 n s i l a1 b i k ) ((pos "OA$" ) ))
( "unsympathetic"  j ( uh2 n s i m p @ th e1 t i k ) ((pos "OA%" ) ))
( "unsympathetically"  a ( uh2 n s i m p @ th e1 t i k l ii ) ((pos "Pu%" ) ))
( "unsystematic"  j ( uh2 n s i s t @ m a1 t i k ) ((pos "OA%" ) ))
( "unsystematically"  a ( uh2 n s i s t @ m a1 t i k l ii ) ((pos "Pu%" ) ))
( "untainted"  j ( uh1 n t ei1 n t i d ) ((pos "OA%" ) ))
( "untamed"  j ( uh1 n t ei1 m d ) ((pos "OA%" ) ))
( "untapped"  j ( uh1 n t a1 p t ) ((pos "OA%" ) ))
( "untarnished"  j ( uh1 n t aa1 n i sh t ) ((pos "OA%" ) ))
( "untaxed"  j ( uh1 n t a1 k s t ) ((pos "OA%" ) ))
( "untenable"  j ( uh1 n t e1 n @ b l ) ((pos "OA%" ) ))
( "untenanted"  j ( uh1 n t e1 n @ n t i d ) ((pos "OA%" ) ))
( "untended"  j ( uh1 n t e1 n d i d ) ((pos "OA%" ) ))
( "untested"  j ( uh1 n t e1 s t i d ) ((pos "OA%" ) ))
( "untethered"  j ( uh1 n t e1 dh @ d ) ((pos "OA%" ) ))
( "unthinkable"  j ( uh1 n th i1 ng k @ b l ) ((pos "OA%" ) ))
( "unthinking"  j ( uh1 n th i1 ng k i ng ) ((pos "OA%" ) ))
( "unthinkingly"  a ( uh1 n th i1 ng k i ng l ii ) ((pos "Pu%" ) ))
( "unthought-of"  j ( uh1 n th oo1 t - o v ) ((pos "OA%" ) ))
( "untidier"  j ( uh1 n t ai1 d i@ r ) ((pos "Or%" ) ))
( "untidiest"  j ( uh1 n t ai1 d i i s t ) ((pos "Os%" ) ))
( "untidily"  a ( uh1 n t ai1 d i l ii ) ((pos "Pu%" ) ))
( "untidiness"  n ( uh1 n t ai1 d i n @ s ) ((pos "L@%" ) ))
( "untidy"  j ( uh1 n t ai1 d ii ) ((pos "OD%" ) ))
( "untie"  v ( uh1 n t ai1 ) ((pos "J5%" ) (subcat "2A" "3A" "6A" "15A" )))
( "untied"  v ( uh1 n t ai1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "15A" )))
( "unties"  v ( uh1 n t ai1 z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" "15A" )))
( "until"  in ( uh1 n t i1 l ) ((pos "T-*" "V-*" ) ))
( "untimely"  j ( uh1 n t ai1 m l ii ) ((pos "OA%" ) ))
( "untiring"  j ( uh1 n t ai1 @ r i ng ) ((pos "OA%" ) ))
( "untitled"  j ( uh1 n t ai1 t l d ) ((pos "OA%" ) ))
( "unto"  in ( uh1 n t uu ) ((pos "T-%" ) ))
( "untold"  j ( uh2 n t ou1 l d ) ((pos "OA%" ) ))
( "untouchable"  n ( uh1 n t uh1 ch @ b l ) ((pos "K6%" "OA%" ) ))
( "untouchables"  n ( uh1 n t uh1 ch @ b l z ) ((pos "Kj%" ) ))
( "untouched"  j ( uh1 n t uh1 ch t ) ((pos "OA%" ) ))
( "untoward"  j ( uh2 n t @ w oo1 d ) ((pos "OA%" ) ))
( "untrained"  j ( uh1 n t r ei1 n d ) ((pos "OA%" ) ))
( "untrammelled"  j ( uh1 n t r a1 m @ l d ) ((pos "OA%" ) ))
( "untranslatable"  j ( uh2 n t r a n s l ei1 t @ b l ) ((pos "OA%" ) ))
( "untreated"  j ( uh1 n t r ii1 t i d ) ((pos "OA%" ) ))
( "untried"  j ( uh1 n t r ai1 d ) ((pos "OA%" ) ))
( "untroubled"  j ( uh1 n t r uh1 b l d ) ((pos "OA%" ) ))
( "untrue"  j ( uh1 n t r uu1 ) ((pos "OA%" ) ))
( "untruly"  a ( uh1 n t r uu1 l ii ) ((pos "Pu%" ) ))
( "untrustworthy"  j ( uh1 n t r uh1 s t w @@ dh ii ) ((pos "OA%" ) ))
( "untruth"  n ( uh1 n t r uu1 th ) ((pos "M6%" ) ))
( "untruthful"  j ( uh1 n t r uu1 th f @ l ) ((pos "OA%" ) ))
( "untruthfully"  a ( uh1 n t r uu1 th f @ l ii ) ((pos "Pu%" ) ))
( "untruths"  n ( uh1 n t r uu1 dh z ) ((pos "Mj%" ) ))
( "untucked"  j ( uh1 n t uh1 k t ) ((pos "OA%" ) ))
( "unturned"  j ( uh1 n t @@1 n d ) ((pos "OA%" ) ))
( "untutored"  j ( uh1 n t y uu1 t @ d ) ((pos "OA%" ) ))
( "untwisted"  j ( uh1 n t w i1 s t i d ) ((pos "OA%" ) ))
( "untying"  v ( uh1 n t ai1 i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "15A" )))
( "untypical"  j ( uh1 n t i1 p i k l ) ((pos "OA%" ) ))
( "untypically"  a ( uh1 n t i1 p i k l ii ) ((pos "Pu%" ) ))
( "unused"  j ( uh1 n y uu1 s t ) ((pos "OA%" ) ))
( "unused"  j ( uh1 n y uu1 z d ) ((pos "OA%" ) ))
( "unusual"  j ( uh1 n y uu1 zh u@ l ) ((pos "OA%" ) ))
( "unusually"  a ( uh1 n y uu1 zh u@ l ii ) ((pos "Pu%" ) ))
( "unutterable"  j ( uh1 n uh1 t @ r @ b l ) ((pos "OA%" ) ))
( "unutterably"  a ( uh1 n uh1 t @ r @ b l ii ) ((pos "Pu%" ) ))
( "unvaried"  j ( uh1 n v e@1 r i d ) ((pos "OA%" ) ))
( "unvarnished"  j ( uh1 n v aa1 n i sh t ) ((pos "OA%" ) ))
( "unvarying"  j ( uh1 n v e@1 r i i ng ) ((pos "OA%" ) ))
( "unveil"  v ( uh2 n v ei1 l ) ((pos "J0%" ) (subcat "2A" )))
( "unveiled"  v ( uh2 n v ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "unveiling"  v ( uh2 n v ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "unveils"  v ( uh2 n v ei1 l z ) ((pos "Ja%" ) (subcat "2A" )))
( "unverified"  j ( uh1 n v e1 r i f ai d ) ((pos "OA%" ) ))
( "unversed"  j ( uh1 n v @@1 s t ) ((pos "OA%" ) ))
( "unvigilant"  j ( uh1 n v i1 jh i l @ n t ) ((pos "OA%" ) ))
( "unvoiced"  j ( uh2 n v oi1 s t ) ((pos "OA%" ) ))
( "unwanted"  j ( uh1 n w o1 n t i d ) ((pos "OA%" ) ))
( "unwarily"  a ( uh1 n w e@1 r i l ii ) ((pos "Pu%" ) ))
( "unwarrantably"  a ( uh1 n w o1 r @ n t @ b l ii ) ((pos "Pu%" ) ))
( "unwarranted"  j ( uh1 n w o1 r @ n t i d ) ((pos "OA%" ) ))
( "unwary"  j ( uh1 n w e@1 r ii ) ((pos "OA%" ) ))
( "unwashed"  j ( uh1 n w o1 sh t ) ((pos "OA%" ) ))
( "unwavering"  j ( uh1 n w ei1 v @ r i ng ) ((pos "OA%" ) ))
( "unwaveringly"  a ( uh1 n w ei1 v @ r i ng l ii ) ((pos "Pu%" ) ))
( "unweaned"  j ( uh1 n w ii1 n d ) ((pos "OA%" ) ))
( "unwearied"  j ( uh1 n w i@1 r i d ) ((pos "OA%" ) ))
( "unwed"  j ( uh1 n w e1 d ) ((pos "OA%" ) ))
( "unwelcome"  j ( uh1 n w e1 l k @ m ) ((pos "OA%" ) ))
( "unwell"  j ( uh1 n w e1 l ) ((pos "Op%" ) ))
( "unwholesome"  j ( uh1 n h ou1 l s @ m ) ((pos "OA%" ) ))
( "unwieldiness"  n ( uh1 n w ii1 l d i n @ s ) ((pos "L@%" ) ))
( "unwieldy"  j ( uh1 n w ii1 l d ii ) ((pos "OA%" ) ))
( "unwilling"  j ( uh1 n w i1 l i ng ) ((pos "OA%" ) ))
( "unwillingly"  a ( uh1 n w i1 l i ng l ii ) ((pos "Pu%" ) ))
( "unwillingness"  n ( uh1 n w i1 l i ng n i s ) ((pos "L@%" ) ))
( "unwind"  v ( uh2 n w ai1 n d ) ((pos "J5%" ) (subcat "2A" "6A" )))
( "unwinding"  v ( uh2 n w ai1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "unwinds"  v ( uh2 n w ai1 n d z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "unwise"  j ( uh1 n w ai1 z ) ((pos "OA%" ) ))
( "unwisely"  a ( uh1 n w ai1 z l ii ) ((pos "Pu%" ) ))
( "unwitnessed"  j ( uh1 n w i1 t n @ s t ) ((pos "OA%" ) ))
( "unwitting"  j ( uh1 n w i1 t i ng ) ((pos "OA%" ) ))
( "unwittingly"  a ( uh1 n w i1 t i ng l ii ) ((pos "Pu%" ) ))
( "unwonted"  j ( uh1 n w ou1 n t i d ) ((pos "Oq%" ) ))
( "unwontedly"  a ( uh1 n w ou1 n t i d l ii ) ((pos "Pu%" ) ))
( "unworkable"  j ( uh1 n w @@1 k @ b l ) ((pos "OA%" ) ))
( "unworkmanlike"  j ( uh1 n w @@1 k m @ n l ai k ) ((pos "OA%" ) ))
( "unworldly"  j ( uh1 n w @@1 l d l ii ) ((pos "OA%" ) ))
( "unworn"  j ( uh1 n w oo1 n ) ((pos "OA%" ) ))
( "unworried"  j ( uh1 n w uh1 r i d ) ((pos "OA%" ) ))
( "unworthily"  a ( uh1 n w @@1 dh i l ii ) ((pos "Pu%" ) ))
( "unworthiness"  n ( uh1 n w @@1 dh i n @ s ) ((pos "L@%" ) ))
( "unworthy"  j ( uh1 n w @@1 dh ii ) ((pos "OA%" ) ))
( "unwound"  v ( uh2 n w au1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "unwrap"  v ( uh1 n r a1 p ) ((pos "J4%" ) (subcat "6A" "14" "15A" )))
( "unwrapped"  v ( uh1 n r a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "14" "15A" )))
( "unwrapping"  v ( uh1 n r a1 p i ng ) ((pos "Jb%" ) (subcat "6A" "14" "15A" )))
( "unwraps"  v ( uh1 n r a1 p s ) ((pos "Ja%" ) (subcat "6A" "14" "15A" )))
( "unwritten"  j ( uh2 n r i1 t n ) ((pos "OA%" ) ))
( "unyielding"  j ( uh1 n y ii1 l d i ng ) ((pos "OA%" ) ))
( "unzip"  v ( uh2 n z i1 p ) ((pos "H4%" ) ))
( "unzipped"  v ( uh2 n z i1 p t ) ((pos "Hc%" "Hd%" ) ))
( "unzipping"  v ( uh2 n z i1 p i ng ) ((pos "Hb%" ) ))
( "unzips"  v ( uh2 n z i1 p s ) ((pos "Ha%" ) ))
( "up"  v ( uh p ) ((pos "J4%" "P+*" "T-*" ) (subcat "2A" )))
( "up-"  j ( uh p - ) ((pos "U-%" ) ))
( "up-and-coming"  j ( uh2 p - @ n - k uh1 m i ng ) ((pos "OA%" ) ))
( "up-beat"  n ( uh1 p - b ii t ) ((pos "K6%" ) ))
( "up-beats"  n ( uh1 p - b ii t s ) ((pos "Kj%" ) ))
( "up-market"  j ( uh1 p - m aa k i t ) ((pos "OA%" ) ))
( "up-to-date"  j ( uh2 p - t @ - d ei1 t ) ((pos "OA%" ) ))
( "up-to-the-minute"  j ( uh2 p - t @ - dh @ - m i1 n i t ) ((pos "Oq%" ) ))
( "upbraid"  v ( uh2 p b r ei1 d ) ((pos "H0%" ) (subcat "6A" )))
( "upbraided"  v ( uh2 p b r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "upbraiding"  v ( uh2 p b r ei1 d i ng ) ((pos "Hb%" "K6$" ) (subcat "6A" )))
( "upbraidings"  n ( uh2 p b r ei1 d i ng z ) ((pos "Kj%" ) ))
( "upbraids"  v ( uh2 p b r ei1 d z ) ((pos "Ha%" ) (subcat "6A" )))
( "upbringing"  n ( uh1 p b r i ng i ng ) ((pos "L@%" ) ))
( "upcountry"  j ( uh2 p k uh1 n t r ii ) ((pos "OA%" "Pu%" ) ))
( "update"  v ( uh2 p d ei1 t ) ((pos "H2%" ) (subcat )))
( "updated"  v ( uh2 p d ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "updates"  v ( uh2 p d ei1 t s ) ((pos "Ha%" ) (subcat )))
( "updating"  v ( uh2 p d ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "upgrade"  n ( uh1 p g r ei d ) ((pos "K6%" ) ))
( "upgrade"  v ( uh2 p g r ei1 d ) ((pos "H2%" ) (subcat )))
( "upgraded"  v ( uh2 p g r ei1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "upgrades"  n ( uh1 p g r ei d z ) ((pos "Kj%" ) ))
( "upgrades"  v ( uh2 p g r ei1 d z ) ((pos "Ha%" ) (subcat )))
( "upgrading"  v ( uh2 p g r ei1 d i ng ) ((pos "Hb%" ) (subcat )))
( "upheaval"  n ( uh2 p h ii1 v @ l ) ((pos "K6%" ) ))
( "upheavals"  n ( uh2 p h ii1 v @ l z ) ((pos "Kj%" ) ))
( "upheld"  v ( uh2 p h e1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uphill"  j ( uh2 p h i1 l ) ((pos "OA%" "Pu%" ) ))
( "uphold"  v ( uh2 p h ou1 l d ) ((pos "H5%" ) (subcat )))
( "upholding"  v ( uh2 p h ou1 l d i ng ) ((pos "Hb%" ) (subcat )))
( "upholds"  v ( uh2 p h ou1 l d z ) ((pos "Ha%" ) (subcat )))
( "upholster"  v ( uh2 p h ou1 l s t @ r ) ((pos "H0%" ) (subcat )))
( "upholstered"  v ( uh2 p h ou1 l s t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "upholsterer"  n ( uh2 p h ou1 l s t @ r @ r ) ((pos "K6%" ) ))
( "upholsterers"  n ( uh2 p h ou1 l s t @ r @ z ) ((pos "Kj%" ) ))
( "upholstering"  v ( uh2 p h ou1 l s t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "upholsters"  v ( uh2 p h ou1 l s t @ z ) ((pos "Ha%" ) (subcat )))
( "upholstery"  n ( uh2 p h ou1 l s t @ r ii ) ((pos "L@%" ) ))
( "upkeep"  n ( uh1 p k ii p ) ((pos "L@%" ) ))
( "upland"  n ( uh1 p l @ n d ) ((pos "K6%" ) ))
( "uplands"  n ( uh1 p l @ n d z ) ((pos "Kj%" ) ))
( "uplift"  n ( uh1 p l i f t ) ((pos "L@%" ) ))
( "uplift"  v ( uh2 p l i1 f t ) ((pos "H0%" ) (subcat )))
( "uplifted"  v ( uh2 p l i1 f t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uplifting"  v ( uh2 p l i1 f t i ng ) ((pos "Hb%" ) (subcat )))
( "uplifts"  v ( uh2 p l i1 f t s ) ((pos "Ha%" ) (subcat )))
( "upmost"  j ( uh1 p m ou s t ) ((pos "OA%" ) ))
( "upon"  in ( @1 p o1 n ) ((pos "T-*" ) ))
( "upped"  v ( uh1 p t ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "upper"  n ( uh1 p @ r ) ((pos "K6%" "OA%" ) ))
( "upper-cuts"  n ( uh1 p @ - k uh t s ) ((pos "Kj%" ) ))
( "uppercut"  n ( uh1 p @ k uh t ) ((pos "K6%" ) ))
( "uppermost"  j ( uh1 p @ m ou s t ) ((pos "OA%" "Pu%" ) ))
( "uppers"  n ( uh1 p @ z ) ((pos "Kj%" ) ))
( "upping"  v ( uh1 p i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "uppish"  j ( uh1 p i sh ) ((pos "OA$" ) ))
( "uppishly"  a ( uh1 p i sh l ii ) ((pos "Pu$" ) ))
( "uppishness"  n ( uh1 p i sh n @ s ) ((pos "L@$" ) ))
( "uppity"  j ( uh1 p i t ii ) ((pos "OA%" ) ))
( "upright"  n ( uh1 p r ai t ) ((pos "K6%" "OA%" ) ))
( "uprightly"  a ( uh1 p r ai t l ii ) ((pos "Pu%" ) ))
( "uprightness"  n ( uh1 p r ai t n @ s ) ((pos "L@%" ) ))
( "uprights"  n ( uh1 p r ai t s ) ((pos "Kj%" ) ))
( "uprising"  n ( uh1 p r ai z i ng ) ((pos "K6%" ) ))
( "uprisings"  n ( uh1 p r ai z i ng z ) ((pos "Kj%" ) ))
( "uproar"  n ( uh1 p r oo r ) ((pos "M6%" ) ))
( "uproarious"  j ( uh1 p r oo1 r i@ s ) ((pos "OA%" ) ))
( "uproariously"  a ( uh1 p r oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "uproars"  n ( uh1 p r oo z ) ((pos "Mj%" ) ))
( "uproot"  v ( uh2 p r uu1 t ) ((pos "H0%" ) (subcat )))
( "uprooted"  v ( uh2 p r uu1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uprooting"  v ( uh2 p r uu1 t i ng ) ((pos "Hb%" ) (subcat )))
( "uproots"  v ( uh2 p r uu1 t s ) ((pos "Ha%" ) (subcat )))
( "ups"  v ( uh1 p s ) ((pos "Ja$" "Kj$" ) (subcat "2A" )))
( "upset"  n ( uh1 p s e t ) ((pos "K6%" ) ))
( "upset"  v ( uh2 p s e1 t ) ((pos "J5%" "Jc%" "Jd%" ) (subcat "2A" )))
( "upsets"  n ( uh1 p s e t s ) ((pos "Kj%" ) ))
( "upsets"  v ( uh2 p s e1 t s ) ((pos "Ja%" ) (subcat "2A" )))
( "upsetting"  v ( uh2 p s e1 t i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "upshot"  n ( uh1 p sh o t ) ((pos "K6%" ) ))
( "upshots"  n ( uh1 p sh o t s ) ((pos "Kj$" ) ))
( "upside-down"  a ( uh2 p s ai d - d au1 n ) ((pos "Pu%" ) ))
( "upstage"  v ( uh2 p s t ei1 jh ) ((pos "H2%" "OA%" "Pu%" ) (subcat )))
( "upstaged"  v ( uh2 p s t ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "upstages"  v ( uh2 p s t ei1 jh i z ) ((pos "Ha%" ) (subcat )))
( "upstaging"  v ( uh2 p s t ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "upstairs"  j ( uh2 p s t e@1 z ) ((pos "Oq%" "Pu%" ) ))
( "upstanding"  j ( uh2 p s t a1 n d i ng ) ((pos "OA%" ) ))
( "upstart"  n ( uh1 p s t aa t ) ((pos "K6%" "OA%" ) ))
( "upstarts"  n ( uh1 p s t aa t s ) ((pos "Kj%" ) ))
( "upstream"  a ( uh2 p s t r ii1 m ) ((pos "Pu%" ) ))
( "upsurge"  n ( uh1 p s @@ jh ) ((pos "K6%" ) ))
( "upsurges"  n ( uh1 p s @@ jh i z ) ((pos "Kj%" ) ))
( "uptake"  n ( uh1 p t ei k ) ((pos "M6%" ) ))
( "uptakes"  n ( uh1 p t ei k s ) ((pos "Mj$" ) ))
( "uptight"  j ( uh2 p t ai1 t ) ((pos "OA%" ) ))
( "uptown"  j ( uh2 p t au1 n ) ((pos "OA%" "Pu%" ) ))
( "upturn"  n ( uh1 p t @@ n ) ((pos "K6%" ) ))
( "upturned"  j ( uh1 p t @@ n d ) ((pos "OA%" ) ))
( "upturns"  n ( uh1 p t @@ n z ) ((pos "Kj%" ) ))
( "upward"  j ( uh1 p w @ d ) ((pos "OA%" "Pu%" ) ))
( "upwards"  a ( uh1 p w @ d z ) ((pos "Pu%" ) ))
( "uranium"  n ( y u1 r ei1 n i@ m ) ((pos "L@%" ) ))
( "urban"  j ( @@1 b @ n ) ((pos "OA%" ) ))
( "urbane"  j ( @@1 b ei1 n ) ((pos "OA%" ) ))
( "urbanely"  a ( @@1 b ei1 n l ii ) ((pos "Pu%" ) ))
( "urbanity"  n ( @@1 b a1 n i t ii ) ((pos "L@%" ) ))
( "urbanization"  n ( @@2 b @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "urbanize"  v ( @@1 b @ n ai z ) ((pos "H2%" ) (subcat )))
( "urbanized"  v ( @@1 b @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "urbanizes"  v ( @@1 b @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "urbanizing"  v ( @@1 b @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "urchin"  n ( @@1 ch i n ) ((pos "K6%" ) ))
( "urchins"  n ( @@1 ch i n z ) ((pos "Kj%" ) ))
( "urge"  v ( @@1 jh ) ((pos "H2%" "M6%" ) (subcat "6A" "6D" "9" "14" "15B" "17" )))
( "urged"  v ( @@1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "6D" "9" "14" "15B" "17" )))
( "urgency"  n ( @@1 jh @ n s ii ) ((pos "L@%" ) ))
( "urgent"  j ( @@1 jh @ n t ) ((pos "OA%" ) ))
( "urgently"  a ( @@1 jh @ n t l ii ) ((pos "Pu%" ) ))
( "urges"  v ( @@1 jh i z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "6D" "9" "14" "15B" "17" )))
( "urging"  v ( @@1 jh i ng ) ((pos "Hb%" "K6$" ) (subcat "6A" "6D" "9" "14" "15B" "17" )))
( "urgings"  n ( @@1 jh i ng z ) ((pos "Kj%" ) ))
( "uric"  j ( y u@1 r i k ) ((pos "OA$" ) ))
( "urinal"  n ( y u1 r ai1 n @ l ) ((pos "K6%" ) ))
( "urinals"  n ( y u1 r ai1 n @ l z ) ((pos "Kj%" ) ))
( "urinary"  j ( y u@1 r i n r ii ) ((pos "OA%" ) ))
( "urinate"  v ( y u@1 r i n ei t ) ((pos "I2%" ) ))
( "urinated"  v ( y u@1 r i n ei t i d ) ((pos "Ic%" "Id%" ) ))
( "urinates"  v ( y u@1 r i n ei t s ) ((pos "Ia%" ) ))
( "urinating"  v ( y u@1 r i n ei t i ng ) ((pos "Ib%" ) ))
( "urine"  n ( y u@1 r i n ) ((pos "L@%" ) ))
( "urn"  n ( @@1 n ) ((pos "K6%" ) ))
( "urns"  n ( @@1 n z ) ((pos "Kj%" ) ))
( "us"  prp ( uh z ) ((pos "Qx*" ) ))
( "usable"  j ( y uu1 z @ b l ) ((pos "OA%" ) ))
( "usage"  n ( y uu1 s i jh ) ((pos "M6%" ) ))
( "usages"  n ( y uu1 s i jh i z ) ((pos "Mj%" ) ))
( "use"  n ( y uu1 s ) ((pos "M6%" ) ))
( "use"  v ( y uu1 z ) ((pos "H2*" ) (subcat "6A" "14" "15A" "15B" )))
( "used"  v ( y uu s t ) ((pos "Gc*" "OA%" ) ))
( "used"  v ( y uu1 z d ) ((pos "Hc*" "Hd*" "OA%" ) (subcat "6A" "14" "15A" "15B" )))
( "useful"  j ( y uu1 s f @ l ) ((pos "OA%" ) ))
( "usefully"  a ( y uu1 s f @ l ii ) ((pos "Pu%" ) ))
( "usefulness"  n ( y uu1 s f @ l n @ s ) ((pos "L@%" ) ))
( "useless"  j ( y uu1 s l @ s ) ((pos "OA%" ) ))
( "uselessly"  a ( y uu1 s l @ s l ii ) ((pos "Pu%" ) ))
( "uselessness"  n ( y uu1 s l @ s n @ s ) ((pos "L@%" ) ))
( "user"  n ( y uu1 z @ r ) ((pos "K6%" ) ))
( "users"  n ( y uu1 z @ z ) ((pos "Kj%" ) ))
( "uses"  n ( y uu1 s i z ) ((pos "Mj%" ) ))
( "uses"  v ( y uu1 z i z ) ((pos "Ha%" ) (subcat "6A" "14" "15A" "15B" )))
( "usher"  v ( uh1 sh @ r ) ((pos "H0%" "K6%" ) (subcat "14" )))
( "ushered"  v ( uh1 sh @ d ) ((pos "Hc%" "Hd%" ) (subcat "14" )))
( "usherette"  n ( uh2 sh @ r e1 t ) ((pos "K6%" ) ))
( "usherettes"  n ( uh2 sh @ r e1 t s ) ((pos "Kj%" ) ))
( "ushering"  v ( uh1 sh @ r i ng ) ((pos "Hb%" ) (subcat "14" )))
( "ushers"  v ( uh1 sh @ z ) ((pos "Ha%" "Kj%" ) (subcat "14" )))
( "using"  v ( y uu1 z i ng ) ((pos "Hb%" ) (subcat "6A" "14" "15A" "15B" )))
( "usual"  j ( y uu1 zh u@ l ) ((pos "OA%" ) ))
( "usually"  a ( y uu1 zh @ l ii ) ((pos "Pu*" ) ))
( "usurer"  n ( y uu1 zh @ r @ r ) ((pos "K6$" ) ))
( "usurers"  n ( y uu1 zh @ r @ z ) ((pos "Kj$" ) ))
( "usurious"  j ( y uu1 z y u@1 r i@ s ) ((pos "OA$" ) ))
( "usurp"  v ( y uu1 z @@1 p ) ((pos "H0%" ) (subcat )))
( "usurpation"  n ( y uu2 z @@ p ei1 sh @ n ) ((pos "M6%" ) ))
( "usurpations"  n ( y uu2 z @@ p ei1 sh @ n z ) ((pos "Mj$" ) ))
( "usurped"  v ( y uu1 z @@1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "usurper"  n ( y uu1 z @@1 p @ r ) ((pos "K6%" ) ))
( "usurpers"  n ( y uu1 z @@1 p @ z ) ((pos "Kj%" ) ))
( "usurping"  v ( y uu1 z @@1 p i ng ) ((pos "Hb%" ) (subcat )))
( "usurps"  v ( y uu1 z @@1 p s ) ((pos "Ha%" ) (subcat )))
( "usury"  n ( y uu1 zh @ r ii ) ((pos "L@$" ) ))
( "utensil"  n ( y uu1 t e1 n s i l ) ((pos "K6%" ) ))
( "utensils"  n ( y uu1 t e1 n s i l z ) ((pos "Kj%" ) ))
( "uterine"  j ( y uu1 t @ r ai n ) ((pos "OA%" ) ))
( "uterus"  n ( y uu1 t @ r @ s ) ((pos "K7%" ) ))
( "uteruses"  n ( y uu1 t @ r @ s i z ) ((pos "Kj$" ) ))
( "utilitarian"  n ( y uu1 t i2 l i t e@1 r i@ n ) ((pos "K6%" "OA%" ) ))
( "utilitarianism"  n ( y uu1 t i2 l i t e@1 r i@ n i z @ m ) ((pos "L@%" ) ))
( "utilitarians"  n ( y uu1 t i2 l i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "utilities"  n ( y uu1 t i1 l i t i z ) ((pos "Mj%" ) ))
( "utility"  n ( y uu1 t i1 l i t ii ) ((pos "M8%" ) ))
( "utilizable"  j ( y uu2 t i l ai1 z @ b l ) ((pos "OA%" ) ))
( "utilization"  n ( y uu2 t i l ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "utilize"  v ( y uu1 t i l ai z ) ((pos "H2%" ) (subcat )))
( "utilized"  v ( y uu1 t i l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "utilizes"  v ( y uu1 t i l ai z i z ) ((pos "Ha%" ) (subcat )))
( "utilizing"  v ( y uu1 t i l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "utmost"  n ( uh1 t m ou s t ) ((pos "K6%" "OA%" ) ))
( "utmosts"  n ( uh1 t m ou s t s ) ((pos "Kj$" ) ))
( "utter"  v ( uh1 t @ r ) ((pos "H0%" "OA%" ) (subcat )))
( "utterance"  n ( uh1 t @ r @ n s ) ((pos "M6%" ) ))
( "utterances"  n ( uh1 t @ r @ n s i z ) ((pos "Mj%" ) ))
( "uttered"  v ( uh1 t @ d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "uttering"  v ( uh1 t @ r i ng ) ((pos "Hb%" ) (subcat )))
( "utterly"  a ( uh1 t @ l ii ) ((pos "Pu%" ) ))
( "uttermost"  n ( uh1 t @ m ou s t ) ((pos "K6%" "OA%" ) ))
( "uttermosts"  n ( uh1 t @ m ou s t s ) ((pos "Kj$" ) ))
( "utters"  v ( uh1 t @ z ) ((pos "Ha%" ) (subcat )))
( "uvula"  n ( y uu1 v y u l @ ) ((pos "K6$" ) ))
( "uvular"  j ( y uu1 v y u l @ r ) ((pos "OA$" ) ))
( "uvulas"  n ( y uu1 v y u l @ z ) ((pos "Kj$" ) ))
( "uxorious"  j ( uh2 k s oo1 r i@ s ) ((pos "OA%" ) ))
( "uxoriously"  a ( uh2 k s oo1 r i@ s l ii ) ((pos "Pu$" ) ))
( "uxoriousness"  n ( uh2 k s oo1 r i@ s n @ s ) ((pos "L@$" ) ))
( "v"  n ( v ii1 ) ((pos "Ki$" ) ))
( "v's"  n ( v ii1 z ) ((pos "Kj$" ) ))
( "vac"  n ( v a1 k ) ((pos "K6$" ) ))
( "vacancies"  n ( v ei1 k @ n s i z ) ((pos "Mj%" ) ))
( "vacancy"  n ( v ei1 k @ n s ii ) ((pos "M8%" ) ))
( "vacant"  j ( v ei1 k @ n t ) ((pos "OA%" ) ))
( "vacantly"  a ( v ei1 k @ n t l ii ) ((pos "Pu%" ) ))
( "vacate"  v ( v @1 k ei1 t ) ((pos "H2%" ) (subcat )))
( "vacated"  v ( v @1 k ei1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vacates"  v ( v @1 k ei1 t s ) ((pos "Ha%" ) (subcat )))
( "vacating"  v ( v @1 k ei1 t i ng ) ((pos "Hb%" ) (subcat )))
( "vacation"  v ( v @1 k ei1 sh @ n ) ((pos "I0$" "M6%" ) (subcat )))
( "vacationed"  v ( v @1 k ei1 sh @ n d ) ((pos "Ic$" "Id$" ) (subcat )))
( "vacationing"  v ( v @1 k ei1 sh @ n i ng ) ((pos "Ib$" ) (subcat )))
( "vacationist"  n ( v @1 k ei1 sh @ n i s t ) ((pos "K6$" ) ))
( "vacationists"  n ( v @1 k ei1 sh @ n i s t s ) ((pos "Kj$" ) ))
( "vacations"  v ( v @1 k ei1 sh @ n z ) ((pos "Ia$" "Mj%" ) (subcat )))
( "vaccinate"  v ( v a1 k s i n ei t ) ((pos "H2%" ) (subcat "6A" )))
( "vaccinated"  v ( v a1 k s i n ei t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "vaccinates"  v ( v a1 k s i n ei t s ) ((pos "Ha%" ) (subcat "6A" )))
( "vaccinating"  v ( v a1 k s i n ei t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "vaccination"  n ( v a2 k s i n ei1 sh @ n ) ((pos "M6%" ) ))
( "vaccinations"  n ( v a2 k s i n ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vaccine"  n ( v a1 k s ii n ) ((pos "M6%" ) ))
( "vaccines"  n ( v a1 k s ii n z ) ((pos "Mj%" ) ))
( "vacillate"  v ( v a1 s i l ei t ) ((pos "I2%" ) (subcat "2A" )))
( "vacillated"  v ( v a1 s i l ei t i d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "vacillates"  v ( v a1 s i l ei t s ) ((pos "Ia%" ) (subcat "2A" )))
( "vacillating"  v ( v a1 s i l ei t i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "vacillation"  n ( v a2 s i l ei1 sh @ n ) ((pos "M6%" ) ))
( "vacillations"  n ( v a2 s i l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vacs"  n ( v a1 k s ) ((pos "Kj$" ) ))
( "vacua"  n ( v a1 k y u@ ) ((pos "Kj$" ) ))
( "vacuities"  n ( v @1 k y uu1 i t i z ) ((pos "Mj$" ) ))
( "vacuity"  n ( v @1 k y uu1 i t ii ) ((pos "M8%" ) ))
( "vacuous"  j ( v a1 k y u@ s ) ((pos "OA%" ) ))
( "vacuously"  a ( v a1 k y u@ s l ii ) ((pos "Pu%" ) ))
( "vacuum"  n ( v a1 k y u@ m ) ((pos "K6%" ) ))
( "vacuums"  n ( v a1 k y u@ m z ) ((pos "Kj%" ) ))
( "vade-mecum"  n ( v ei2 d i - m ii1 k @ m ) ((pos "K6$" ) ))
( "vade-mecums"  n ( v ei2 d i - m ii1 k @ m z ) ((pos "Kj$" ) ))
( "vagabond"  n ( v a1 g @ b o n d ) ((pos "K6%" "OA%" ) ))
( "vagabonds"  n ( v a1 g @ b o n d z ) ((pos "Kj%" ) ))
( "vagaries"  n ( v ei1 g @ r i z ) ((pos "Kj%" ) ))
( "vagary"  n ( v ei1 g @ r ii ) ((pos "K8%" ) ))
( "vagina"  n ( v @1 jh ai1 n @ ) ((pos "K6%" ) ))
( "vaginal"  j ( v @1 jh ai1 n l ) ((pos "OA%" ) ))
( "vaginas"  n ( v @1 jh ai1 n @ z ) ((pos "Kj%" ) ))
( "vagrancy"  n ( v ei1 g r @ n s ii ) ((pos "L@%" ) ))
( "vagrant"  n ( v ei1 g r @ n t ) ((pos "K6%" "OA%" ) ))
( "vagrants"  n ( v ei1 g r @ n t s ) ((pos "Kj%" ) ))
( "vague"  j ( v ei1 g ) ((pos "OB%" ) ))
( "vaguely"  a ( v ei1 g l ii ) ((pos "Pu%" ) ))
( "vagueness"  n ( v ei1 g n @ s ) ((pos "L@%" ) ))
( "vaguer"  j ( v ei1 g @ r ) ((pos "Or%" ) ))
( "vaguest"  j ( v ei1 g i s t ) ((pos "Os%" ) ))
( "vain"  j ( v ei1 n ) ((pos "OC%" ) ))
( "vainer"  j ( v ei1 n @ r ) ((pos "Or%" ) ))
( "vainest"  j ( v ei1 n i s t ) ((pos "Os%" ) ))
( "vainglorious"  j ( v ei1 n g l oo1 r i@ s ) ((pos "OA%" ) ))
( "vainglory"  n ( v ei1 n g l oo1 r ii ) ((pos "L@%" ) ))
( "vainly"  a ( v ei1 n l ii ) ((pos "Pu%" ) ))
( "valance"  n ( v a1 l @ n s ) ((pos "K6$" ) ))
( "valances"  n ( v a1 l @ n s i z ) ((pos "Kj$" ) ))
( "vale"  n ( v ei1 l ) ((pos "K6%" ) ))
( "valediction"  n ( v a2 l i d i1 k sh @ n ) ((pos "K6%" ) ))
( "valedictions"  n ( v a2 l i d i1 k sh @ n z ) ((pos "Kj%" ) ))
( "valedictory"  j ( v a2 l i d i1 k t @ r ii ) ((pos "OA%" ) ))
( "valence"  n ( v a1 l @ n s ) ((pos "K6$" ) ))
( "valence"  n ( v ei1 l @ n s ) ((pos "M6$" ) ))
( "valences"  n ( v a1 l @ n s i z ) ((pos "Kj$" ) ))
( "valences"  n ( v ei1 l @ n s i z ) ((pos "Mj$" ) ))
( "valencies"  n ( v ei1 l @ n s i z ) ((pos "Kj$" ) ))
( "valency"  n ( v ei1 l @ n s ii ) ((pos "K8$" ) ))
( "valentine"  n ( v a1 l @ n t ai n ) ((pos "K6%" ) ))
( "valentines"  n ( v a1 l @ n t ai n z ) ((pos "Kj%" ) ))
( "valerian"  n ( v @1 l i@1 r i@ n ) ((pos "L@$" ) ))
( "vales"  n ( v ei1 l z ) ((pos "Kj%" ) ))
( "valet"  v ( v a1 l ei ) ((pos "H0$" "K6%" ) (subcat )))
( "valeted"  v ( v a1 l ei d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "valeting"  v ( v a1 l ei i ng ) ((pos "Hb$" ) (subcat )))
( "valets"  v ( v a1 l ei z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "valetudinarian"  n ( v a2 l i t y uu d i n e@1 r i@ n ) ((pos "K6$" "OA$" ) ))
( "valetudinarians"  n ( v a2 l i t y uu d i n e@1 r i@ n z ) ((pos "Kj$" ) ))
( "valiant"  j ( v a1 l i@ n t ) ((pos "OA%" ) ))
( "valiantly"  a ( v a1 l i@ n t l ii ) ((pos "Pu%" ) ))
( "valid"  j ( v a1 l i d ) ((pos "OA%" ) ))
( "validate"  v ( v a1 l i d ei t ) ((pos "H2%" ) (subcat )))
( "validated"  v ( v a1 l i d ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "validates"  v ( v a1 l i d ei t s ) ((pos "Ha%" ) (subcat )))
( "validating"  v ( v a1 l i d ei t i ng ) ((pos "Hb%" ) (subcat )))
( "validity"  n ( v @1 l i1 d i t ii ) ((pos "L@%" ) ))
( "validly"  a ( v a1 l i d l ii ) ((pos "Pu%" ) ))
( "valise"  n ( v @1 l ii1 z ) ((pos "K6%" ) ))
( "valises"  n ( v @1 l ii1 z i z ) ((pos "Kj$" ) ))
( "valley"  n ( v a1 l ii ) ((pos "K6%" ) ))
( "valleys"  n ( v a1 l i z ) ((pos "Kj%" ) ))
( "valorous"  j ( v a1 l @ r @ s ) ((pos "OA%" ) ))
( "valour"  n ( v a1 l @ r ) ((pos "L@%" ) ))
( "valuable"  n ( v a1 l y u@ b l ) ((pos "K6%" "OA%" ) ))
( "valuables"  n ( v a1 l y u@ b l z ) ((pos "Kj%" ) ))
( "valuation"  n ( v a2 l y u ei1 sh @ n ) ((pos "M6%" ) ))
( "valuations"  n ( v a2 l y u ei1 sh @ n z ) ((pos "Mj%" ) ))
( "value"  v ( v a1 l y uu ) ((pos "H2%" "M6%" ) (subcat "6A" "15A" )))
( "value-added"  j ( v a2 l y uu - a1 d i d ) ((pos "Oq%" ) ))
( "valued"  v ( v a1 l y uu d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "valueless"  j ( v a1 l y uu l @ s ) ((pos "OA%" ) ))
( "valuer"  n ( v a1 l y uu @ r ) ((pos "K6%" ) ))
( "valuers"  n ( v a1 l y uu @ z ) ((pos "Kj%" ) ))
( "values"  v ( v a1 l y uu z ) ((pos "Ha%" "Mj%" ) (subcat "6A" "15A" )))
( "valuing"  v ( v a1 l y uu i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "valve"  n ( v a1 l v ) ((pos "K6%" ) ))
( "valves"  n ( v a1 l v z ) ((pos "Kj%" ) ))
( "valvular"  j ( v a1 l v y u l @ r ) ((pos "OA$" ) ))
( "vamoose"  v ( v a2 m uu1 s ) ((pos "I2$" ) ))
( "vamoosed"  v ( v a2 m uu1 s t ) ((pos "Ic$" "Id$" ) ))
( "vamooses"  v ( v a2 m uu1 s i z ) ((pos "Ia$" ) ))
( "vamoosing"  v ( v a2 m uu1 s i ng ) ((pos "Ib$" ) ))
( "vamp"  v ( v a1 m p ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" )))
( "vamped"  v ( v a1 m p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "vamping"  v ( v a1 m p i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "vampire"  n ( v a1 m p ai @ r ) ((pos "K6%" ) ))
( "vampires"  n ( v a1 m p ai @ z ) ((pos "Kj%" ) ))
( "vamps"  v ( v a1 m p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "van"  n ( v a1 n ) ((pos "K6%" ) ))
( "vandal"  n ( v a1 n d @ l ) ((pos "K6%" ) ))
( "vandalism"  n ( v a1 n d @ l i z @ m ) ((pos "L@%" ) ))
( "vandals"  n ( v a1 n d @ l z ) ((pos "Kj%" ) ))
( "vane"  n ( v ei1 n ) ((pos "K6$" ) ))
( "vanes"  n ( v ei1 n z ) ((pos "Kj$" ) ))
( "vanguard"  n ( v a1 n g aa d ) ((pos "K6%" ) ))
( "vanguards"  n ( v a1 n g aa d z ) ((pos "Kj$" ) ))
( "vanilla"  n ( v @1 n i1 l @ ) ((pos "M6%" ) ))
( "vanillas"  n ( v @1 n i1 l @ z ) ((pos "Mj$" ) ))
( "vanish"  v ( v a1 n i sh ) ((pos "I1%" ) (subcat )))
( "vanished"  v ( v a1 n i sh t ) ((pos "Ic%" "Id%" ) (subcat )))
( "vanishes"  v ( v a1 n i sh i z ) ((pos "Ia%" ) (subcat )))
( "vanishing"  v ( v a1 n i sh i ng ) ((pos "Ib%" ) (subcat )))
( "vanities"  n ( v a1 n i t i z ) ((pos "Mj%" ) ))
( "vanity"  n ( v a1 n i t ii ) ((pos "M8%" ) ))
( "vanquish"  v ( v a1 ng k w i sh ) ((pos "H1%" ) (subcat )))
( "vanquished"  v ( v a1 ng k w i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vanquishes"  v ( v a1 ng k w i sh i z ) ((pos "Ha%" ) (subcat )))
( "vanquishing"  v ( v a1 ng k w i sh i ng ) ((pos "Hb%" ) (subcat )))
( "vans"  n ( v a1 n z ) ((pos "Kj%" ) ))
( "vantage"  n ( v aa1 n t i jh ) ((pos "M6%" ) ))
( "vantage-point"  n ( v aa1 n t i jh - p oi n t ) ((pos "K6%" ) ))
( "vantage-points"  n ( v aa1 n t i jh - p oi n t s ) ((pos "Kj%" ) ))
( "vantages"  n ( v aa1 n t i jh i z ) ((pos "Mj$" ) ))
( "vapid"  j ( v a1 p i d ) ((pos "OA%" ) ))
( "vapidity"  n ( v @1 p i1 d i t ii ) ((pos "L@$" ) ))
( "vapidly"  a ( v a1 p i d l ii ) ((pos "Pu%" ) ))
( "vapidness"  n ( v a1 p i d n @ s ) ((pos "L@$" ) ))
( "vaporization"  n ( v ei2 p @ r ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "vaporizations"  n ( v ei2 p @ r ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "vaporize"  v ( v ei1 p @ r ai z ) ((pos "J2%" ) (subcat "2A" )))
( "vaporized"  v ( v ei1 p @ r ai z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "vaporizes"  v ( v ei1 p @ r ai z i z ) ((pos "Ja%" ) (subcat "2A" )))
( "vaporizing"  v ( v ei1 p @ r ai z i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "vaporous"  j ( v ei1 p @ r @ s ) ((pos "OA%" ) ))
( "vapour"  n ( v ei1 p @ r ) ((pos "M6%" ) ))
( "vapour-bath"  n ( v ei1 p @ - b aa th ) ((pos "K6$" ) ))
( "vapour-baths"  n ( v ei1 p @ - b aa dh z ) ((pos "Kj$" ) ))
( "vapours"  n ( v ei1 p @ z ) ((pos "Mj%" ) ))
( "variability"  n ( v e@2 r i@ b i1 l i t ii ) ((pos "L@%" ) ))
( "variable"  n ( v e@1 r i@ b l ) ((pos "K6%" "OA%" ) ))
( "variableness"  n ( v e@1 r i@ b l n @ s ) ((pos "L@%" ) ))
( "variables"  n ( v e@1 r i@ b l z ) ((pos "Kj%" ) ))
( "variably"  a ( v e@1 r i@ b l ii ) ((pos "Pu%" ) ))
( "variance"  n ( v e@1 r i@ n s ) ((pos "M6%" ) ))
( "variances"  n ( v e@1 r i@ n s i z ) ((pos "Mj$" ) ))
( "variant"  n ( v e@1 r i@ n t ) ((pos "K6%" "OA%" ) ))
( "variants"  n ( v e@1 r i@ n t s ) ((pos "Kj%" ) ))
( "variation"  n ( v e@2 r i ei1 sh @ n ) ((pos "M6%" ) ))
( "variations"  n ( v e@2 r i ei1 sh @ n z ) ((pos "Mj%" ) ))
( "varicoloured"  j ( v e@1 r i k uh l @ d ) ((pos "OA$" ) ))
( "varicose"  j ( v a1 r i k ou s ) ((pos "OA%" ) ))
( "varied"  v ( v e@1 r i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" )))
( "variegated"  j ( v e@1 r i g ei t i d ) ((pos "OA%" ) ))
( "variegation"  n ( v e@2 r i g ei1 sh @ n ) ((pos "M6%" ) ))
( "variegations"  n ( v e@2 r i g ei1 sh @ n z ) ((pos "Mj$" ) ))
( "varies"  v ( v e@1 r i z ) ((pos "Ja%" ) (subcat "2A" )))
( "varieties"  n ( v @1 r ai1 @ t i z ) ((pos "Mj%" ) ))
( "variety"  n ( v @1 r ai1 @ t ii ) ((pos "M8%" ) ))
( "variform"  j ( v e@2 r i f oo1 m ) ((pos "OA$" ) ))
( "variorum"  j ( v e@1 r i oo r @ m ) ((pos "OA$" ) ))
( "various"  j ( v e@1 r i@ s ) ((pos "OA%" ) ))
( "variously"  a ( v e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "varlet"  n ( v aa1 l i t ) ((pos "K6$" ) ))
( "varlets"  n ( v aa1 l i t s ) ((pos "Kj$" ) ))
( "varnish"  v ( v aa1 n i sh ) ((pos "H1%" "M7%" ) (subcat )))
( "varnished"  v ( v aa1 n i sh t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "varnishes"  v ( v aa1 n i sh i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "varnishing"  v ( v aa1 n i sh i ng ) ((pos "Hb%" ) (subcat )))
( "varsities"  n ( v aa1 s i t i z ) ((pos "Kj$" ) ))
( "varsity"  n ( v aa1 s i t ii ) ((pos "K8%" ) ))
( "vary"  v ( v e@1 r ii ) ((pos "J3%" ) (subcat "2A" )))
( "varying"  v ( v e@1 r i i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "vascular"  j ( v a1 s k y u l @ r ) ((pos "OA%" ) ))
( "vase"  n ( v aa1 z ) ((pos "K6%" ) ))
( "vasectomies"  n ( v @1 s e1 k t @ m i z ) ((pos "Kj%" ) ))
( "vasectomy"  n ( v @1 s e1 k t @ m ii ) ((pos "K8%" ) ))
( "vaseline"  n ( v a1 s @ l ii n ) ((pos "L@%" ) ))
( "vases"  n ( v aa1 z i z ) ((pos "Kj%" ) ))
( "vassal"  n ( v a1 s @ l ) ((pos "K6$" ) ))
( "vassalage"  n ( v a1 s @ l i jh ) ((pos "L@$" ) ))
( "vassals"  n ( v a1 s @ l z ) ((pos "Kj$" ) ))
( "vast"  j ( v aa1 s t ) ((pos "OA%" ) ))
( "vastly"  a ( v aa1 s t l ii ) ((pos "Pu%" ) ))
( "vastness"  n ( v aa1 s t n @ s ) ((pos "L@%" ) ))
( "vat"  n ( v a1 t ) ((pos "K6%" ) ))
( "vats"  n ( v a1 t s ) ((pos "Kj%" ) ))
( "vaudeville"  n ( v oo1 d @ v i l ) ((pos "L@%" ) ))
( "vault"  v ( v oo1 l t ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "vaulted"  v ( v oo1 l t i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "2C" )))
( "vaulter"  n ( v oo1 l t @ r ) ((pos "K6%" ) ))
( "vaulters"  n ( v oo1 l t @ z ) ((pos "Kj%" ) ))
( "vaulting"  v ( v oo1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "vaulting-horse"  n ( v oo1 l t i ng - h oo s ) ((pos "K6%" ) ))
( "vaulting-horses"  n ( v oo1 l t i ng - h oo s i z ) ((pos "Kj%" ) ))
( "vaults"  v ( v oo1 l t s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "vaunt"  v ( v oo1 n t ) ((pos "J0$" "K6$" ) ))
( "vaunted"  v ( v oo1 n t i d ) ((pos "Jc$" "Jd%" ) ))
( "vaunter"  n ( v oo1 n t @ r ) ((pos "K6$" ) ))
( "vaunters"  n ( v oo1 n t @ z ) ((pos "Kj$" ) ))
( "vaunting"  v ( v oo1 n t i ng ) ((pos "Jb$" ) ))
( "vauntingly"  a ( v oo1 n t i ng l ii ) ((pos "Pu$" ) ))
( "vaunts"  v ( v oo1 n t s ) ((pos "Ja$" "Kj$" ) ))
( "veal"  n ( v ii1 l ) ((pos "L@%" ) ))
( "vector"  n ( v e1 k t @ r ) ((pos "K6$" ) ))
( "vectors"  n ( v e1 k t @ z ) ((pos "Kj$" ) ))
( "veer"  v ( v i@1 r ) ((pos "I0%" ) (subcat "2A" )))
( "veered"  v ( v i@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "veering"  v ( v i@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "veers"  v ( v i@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "vegetable"  n ( v e1 jh i t @ b l ) ((pos "K6%" "OA%" ) ))
( "vegetables"  n ( v e1 jh i t @ b l z ) ((pos "Kj%" ) ))
( "vegetarian"  n ( v e2 jh i t e@1 r i@ n ) ((pos "K6%" ) ))
( "vegetarians"  n ( v e2 jh i t e@1 r i@ n z ) ((pos "Kj%" ) ))
( "vegetate"  v ( v e1 jh i t ei t ) ((pos "I2%" ) (subcat )))
( "vegetated"  v ( v e1 jh i t ei t i d ) ((pos "Ic%" "Id%" ) (subcat )))
( "vegetates"  v ( v e1 jh i t ei t s ) ((pos "Ia%" ) (subcat )))
( "vegetating"  v ( v e1 jh i t ei t i ng ) ((pos "Ib%" ) (subcat )))
( "vegetation"  n ( v e2 jh i t ei1 sh @ n ) ((pos "L@%" ) ))
( "vehemence"  n ( v ii1 @ m @ n s ) ((pos "L@%" ) ))
( "vehement"  j ( v ii1 @ m @ n t ) ((pos "OA%" ) ))
( "vehemently"  a ( v ii1 @ m @ n t l ii ) ((pos "Pu%" ) ))
( "vehicle"  n ( v ii1 i k l ) ((pos "K6%" ) ))
( "vehicles"  n ( v ii1 i k l z ) ((pos "Kj%" ) ))
( "vehicular"  j ( v i1 i1 k y u l @ r ) ((pos "OA%" ) ))
( "veil"  v ( v ei1 l ) ((pos "H0%" "K6%" ) (subcat )))
( "veiled"  v ( v ei1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "veiling"  v ( v ei1 l i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "veils"  v ( v ei1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "vein"  n ( v ei1 n ) ((pos "K6%" ) ))
( "veined"  j ( v ei1 n d ) ((pos "OA%" ) ))
( "veins"  n ( v ei1 n z ) ((pos "Kj%" ) ))
( "veld"  n ( v e1 l t ) ((pos "L@$" ) ))
( "vellum"  n ( v e1 l @ m ) ((pos "L@%" ) ))
( "velocipede"  n ( v @1 l o1 s i p ii d ) ((pos "K6$" ) ))
( "velocipedes"  n ( v @1 l o1 s i p ii d z ) ((pos "Kj$" ) ))
( "velocities"  n ( v @1 l o1 s i t i z ) ((pos "Mj%" ) ))
( "velocity"  n ( v @1 l o1 s i t ii ) ((pos "M8%" ) ))
( "velour"  n ( v @1 l u@1 r ) ((pos "L@%" ) ))
( "velvet"  n ( v e1 l v i t ) ((pos "L@%" ) ))
( "velveteen"  n ( v e2 l v i t ii1 n ) ((pos "L@%" ) ))
( "velvety"  j ( v e1 l v i t ii ) ((pos "OA%" ) ))
( "venal"  j ( v ii1 n @ l ) ((pos "OA%" ) ))
( "venality"  n ( v i1 n a1 l i t ii ) ((pos "L@$" ) ))
( "venally"  a ( v ii1 n @ l ii ) ((pos "Pu%" ) ))
( "vend"  v ( v e1 n d ) ((pos "H0$" ) (subcat )))
( "vended"  v ( v e1 n d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "vendee"  n ( v e1 n d ii1 ) ((pos "K6$" ) ))
( "vendees"  n ( v e1 n d ii1 z ) ((pos "Kj$" ) ))
( "vender"  n ( v e1 n d @ r ) ((pos "K6$" ) ))
( "venders"  n ( v e1 n d @ z ) ((pos "Kj$" ) ))
( "vendetta"  n ( v e1 n d e1 t @ ) ((pos "K6%" ) ))
( "vendettas"  n ( v e1 n d e1 t @ z ) ((pos "Kj%" ) ))
( "vending"  v ( v e1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "vendor"  n ( v e1 n d @ r ) ((pos "K6%" ) ))
( "vendors"  n ( v e1 n d @ z ) ((pos "Kj%" ) ))
( "vends"  v ( v e1 n d z ) ((pos "Ha$" ) (subcat )))
( "veneer"  v ( v @1 n i@1 r ) ((pos "H0%" "M6%" ) (subcat )))
( "veneered"  v ( v @1 n i@1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "veneering"  v ( v @1 n i@1 r i ng ) ((pos "Hb%" ) (subcat )))
( "veneers"  v ( v @1 n i@1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "venerable"  j ( v e1 n @ r @ b l ) ((pos "OA%" ) ))
( "venerate"  v ( v e1 n @ r ei t ) ((pos "H2%" ) (subcat )))
( "venerated"  v ( v e1 n @ r ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "venerates"  v ( v e1 n @ r ei t s ) ((pos "Ha%" ) (subcat )))
( "venerating"  v ( v e1 n @ r ei t i ng ) ((pos "Hb%" ) (subcat )))
( "veneration"  n ( v e2 n @ r ei1 sh @ n ) ((pos "M6%" ) ))
( "venerations"  n ( v e2 n @ r ei1 sh @ n z ) ((pos "Mj$" ) ))
( "venereal"  j ( v @1 n i@1 r i@ l ) ((pos "OA%" ) ))
( "vengeance"  n ( v e1 n jh @ n s ) ((pos "L@%" ) ))
( "vengeful"  j ( v e1 n jh f @ l ) ((pos "OA%" ) ))
( "venial"  j ( v ii1 n i@ l ) ((pos "OA%" ) ))
( "venison"  n ( v e1 n i s n ) ((pos "L@%" ) ))
( "venom"  n ( v e1 n @ m ) ((pos "L@%" ) ))
( "venomed"  j ( v e1 n @ m d ) ((pos "OA%" ) ))
( "venomous"  j ( v e1 n @ m @ s ) ((pos "OA%" ) ))
( "venomously"  a ( v e1 n @ m @ s l ii ) ((pos "Pu%" ) ))
( "venous"  j ( v ii1 n @ s ) ((pos "OA$" ) ))
( "vent"  v ( v e1 n t ) ((pos "H0%" "K6%" ) (subcat "6A" )))
( "vent-hole"  n ( v e1 n t - h ou l ) ((pos "K6%" ) ))
( "vent-holes"  n ( v e1 n t - h ou l z ) ((pos "Kj%" ) ))
( "vented"  v ( v e1 n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "ventilate"  v ( v e1 n t i l ei t ) ((pos "H2%" ) (subcat )))
( "ventilated"  v ( v e1 n t i l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "ventilates"  v ( v e1 n t i l ei t s ) ((pos "Ha%" ) (subcat )))
( "ventilating"  v ( v e1 n t i l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "ventilation"  n ( v e2 n t i l ei1 sh @ n ) ((pos "L@%" ) ))
( "ventilator"  n ( v e1 n t i l ei t @ r ) ((pos "K6%" ) ))
( "ventilators"  n ( v e1 n t i l ei t @ z ) ((pos "Kj%" ) ))
( "venting"  v ( v e1 n t i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "ventral"  j ( v e1 n t r @ l ) ((pos "OA$" ) ))
( "ventricle"  n ( v e1 n t r i k l ) ((pos "K6%" ) ))
( "ventricles"  n ( v e1 n t r i k l z ) ((pos "Kj%" ) ))
( "ventricular"  j ( v e1 n t r i1 k y u l @ ) ((pos "OA$" ) ))
( "ventriloquism"  n ( v e1 n t r i1 l @ k w i z @ m ) ((pos "L@%" ) ))
( "ventriloquist"  n ( v e1 n t r i1 l @ k w i s t ) ((pos "K6%" ) ))
( "ventriloquists"  n ( v e1 n t r i1 l @ k w i s t s ) ((pos "Kj%" ) ))
( "vents"  v ( v e1 n t s ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "venture"  v ( v e1 n ch @ r ) ((pos "J2%" "M6%" ) (subcat "3A" "6A" "7A" "15A" )))
( "ventured"  v ( v e1 n ch @ d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "7A" "15A" )))
( "ventures"  v ( v e1 n ch @ z ) ((pos "Ja%" "Mj%" ) (subcat "3A" "6A" "7A" "15A" )))
( "venturesome"  j ( v e1 n ch @ s @ m ) ((pos "OA%" ) ))
( "venturing"  v ( v e1 n ch @ r i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "7A" "15A" )))
( "venturous"  j ( v e1 n ch @ r @ s ) ((pos "OA%" ) ))
( "venue"  n ( v e1 n y uu ) ((pos "K6%" ) ))
( "venues"  n ( v e1 n y uu z ) ((pos "Kj%" ) ))
( "veracious"  j ( v @1 r ei1 sh @ s ) ((pos "OA$" ) ))
( "veraciously"  a ( v @1 r ei1 sh @ s l ii ) ((pos "Pu$" ) ))
( "veracity"  n ( v @1 r a1 s i t ii ) ((pos "L@%" ) ))
( "veranda"  n ( v @1 r a1 n d @ ) ((pos "K6%" ) ))
( "verandah"  n ( v @1 r a1 n d @ ) ((pos "K6%" ) ))
( "verandahs"  n ( v @1 r a1 n d @ z ) ((pos "Kj%" ) ))
( "verandas"  n ( v @1 r a1 n d @ z ) ((pos "Kj%" ) ))
( "verb"  n ( v @@1 b ) ((pos "K6%" ) ))
( "verbal"  j ( v @@1 b l ) ((pos "OA%" ) ))
( "verbalize"  v ( v @@1 b @ l ai z ) ((pos "H2%" ) ))
( "verbalized"  v ( v @@1 b @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "verbalizes"  v ( v @@1 b @ l ai z i z ) ((pos "Ha%" ) ))
( "verbalizing"  v ( v @@1 b @ l ai z i ng ) ((pos "Hb%" ) ))
( "verbally"  a ( v @@1 b @ l ii ) ((pos "Pu%" ) ))
( "verbatim"  j ( v @@1 b ei1 t i m ) ((pos "OA%" "Pu%" ) ))
( "verbena"  n ( v @@1 b ii1 n @ ) ((pos "K6$" ) ))
( "verbenas"  n ( v @@1 b ii1 n @ z ) ((pos "Kj$" ) ))
( "verbiage"  n ( v @@1 b i i jh ) ((pos "L@%" ) ))
( "verbose"  j ( v @@1 b ou1 s ) ((pos "OA%" ) ))
( "verbosely"  a ( v @@1 b ou1 s l ii ) ((pos "Pu%" ) ))
( "verboseness"  n ( v @@1 b ou1 s n @ s ) ((pos "L@$" ) ))
( "verbosity"  n ( v @@1 b o1 s i t ii ) ((pos "L@%" ) ))
( "verbs"  n ( v @@1 b z ) ((pos "Kj%" ) ))
( "verdancy"  n ( v @@1 d n s ii ) ((pos "L@$" ) ))
( "verdant"  j ( v @@1 d n t ) ((pos "OA%" ) ))
( "verdict"  n ( v @@1 d i k t ) ((pos "K6%" ) ))
( "verdicts"  n ( v @@1 d i k t s ) ((pos "Kj%" ) ))
( "verdigris"  n ( v @@1 d i g r i s ) ((pos "L@%" ) ))
( "verdure"  n ( v @@1 d y @ r ) ((pos "L@%" ) ))
( "verge"  v ( v @@1 jh ) ((pos "I2%" "K6%" ) (subcat )))
( "verged"  v ( v @@1 jh d ) ((pos "Ic%" "Id%" ) (subcat )))
( "verger"  n ( v @@1 jh @ r ) ((pos "K6%" ) ))
( "vergers"  n ( v @@1 jh @ z ) ((pos "Kj%" ) ))
( "verges"  v ( v @@1 jh i z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "verging"  v ( v @@1 jh i ng ) ((pos "Ib%" ) (subcat )))
( "verifiable"  j ( v e1 r i f ai @ b l ) ((pos "OA%" ) ))
( "verification"  n ( v e2 r i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "verifications"  n ( v e2 r i f i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "verified"  v ( v e1 r i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "verifies"  v ( v e1 r i f ai z ) ((pos "Ha%" ) (subcat )))
( "verify"  v ( v e1 r i f ai ) ((pos "H3%" ) (subcat )))
( "verifying"  v ( v e1 r i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "verily"  a ( v e1 r @ l ii ) ((pos "Pu%" ) ))
( "verisimilitude"  n ( v e2 r i s i m i1 l i t y uu d ) ((pos "M6%" ) ))
( "verisimilitudes"  n ( v e2 r i s i m i1 l i t y uu d z ) ((pos "Mj%" ) ))
( "veritable"  j ( v e1 r i t @ b l ) ((pos "OA%" ) ))
( "verities"  n ( v e1 r i t i z ) ((pos "Mj$" ) ))
( "verity"  n ( v e1 r i t ii ) ((pos "M8$" ) ))
( "vermicelli"  n ( v @@2 m i s e1 l ii ) ((pos "L@$" ) ))
( "vermiform"  j ( v @@1 m i f oo m ) ((pos "OA$" ) ))
( "vermilion"  n ( v @1 m i1 l i@ n ) ((pos "M6%" "OA%" ) ))
( "vermilions"  n ( v @1 m i1 l i@ n z ) ((pos "Mj$" ) ))
( "vermin"  n ( v @@1 m i n ) ((pos "L@%" ) ))
( "verminous"  j ( v @@1 m i n @ s ) ((pos "OA$" ) ))
( "vermouth"  n ( v @@1 m @ th ) ((pos "M6%" ) ))
( "vermouths"  n ( v @@1 m @ th s ) ((pos "Mj$" ) ))
( "vernacular"  n ( v @1 n a1 k y u l @ r ) ((pos "K6%" "OA%" ) ))
( "vernaculars"  n ( v @1 n a1 k y u l @ z ) ((pos "Kj$" ) ))
( "vernal"  j ( v @@1 n @ l ) ((pos "OA%" ) ))
( "veronica"  n ( v @1 r o1 n i k @ ) ((pos "K6$" ) ))
( "veronicas"  n ( v @1 r o1 n i k @ z ) ((pos "Kj$" ) ))
( "verruca"  n ( v @1 r uu1 k @ ) ((pos "K6$" ) ))
( "verrucas"  n ( v @1 r uu1 k @ z ) ((pos "Kj$" ) ))
( "versatile"  j ( v @@1 s @ t ai l ) ((pos "OA%" ) ))
( "versatility"  n ( v @@2 s @ t i1 l i t ii ) ((pos "L@%" ) ))
( "verse"  n ( v @@1 s ) ((pos "M6%" ) ))
( "versed"  j ( v @@1 s t ) ((pos "OA%" ) ))
( "verses"  n ( v @@1 s i z ) ((pos "Mj%" ) ))
( "versification"  n ( v @@2 s i f i k ei1 sh @ n ) ((pos "L@$" ) ))
( "versified"  v ( v @@1 s i f ai d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "versifier"  n ( v @@1 s i f ai @ r ) ((pos "K6$" ) ))
( "versifiers"  n ( v @@1 s i f ai @ z ) ((pos "Kj$" ) ))
( "versifies"  v ( v @@1 s i f ai z ) ((pos "Ja$" ) (subcat "2A" )))
( "versify"  v ( v @@1 s i f ai ) ((pos "J3$" ) (subcat "2A" )))
( "versifying"  v ( v @@1 s i f ai i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "version"  n ( v @@1 sh @ n ) ((pos "K6%" ) ))
( "versions"  n ( v @@1 sh @ n z ) ((pos "Kj%" ) ))
( "verso"  n ( v @@1 s ou ) ((pos "K6$" ) ))
( "versos"  n ( v @@1 s ou z ) ((pos "Kj$" ) ))
( "versus"  in ( v @@1 s @ s ) ((pos "T-%" ) ))
( "vertebra"  n ( v @@1 t i b r @ ) ((pos "Ki%" ) ))
( "vertebrae"  n ( v @@1 t i b r ii ) ((pos "Kj%" ) ))
( "vertebrate"  n ( v @@1 t i b r ei t ) ((pos "K6%" "OA%" ) ))
( "vertebrates"  n ( v @@1 t i b r ei t s ) ((pos "Kj%" ) ))
( "vertex"  n ( v @@1 t e k s ) ((pos "Ki%" ) ))
( "vertical"  n ( v @@1 t i k l ) ((pos "K6$" "OA%" ) ))
( "vertically"  a ( v @@1 t i k l ii ) ((pos "Pu%" ) ))
( "verticals"  n ( v @@1 t i k l z ) ((pos "Kj$" ) ))
( "vertices"  n ( v @@1 t i s ii z ) ((pos "Kj$" ) ))
( "vertiginous"  j ( v @@1 t i1 jh i n @ s ) ((pos "OA$" ) ))
( "vertigo"  n ( v @@1 t i g ou ) ((pos "L@%" ) ))
( "verve"  n ( v @@1 v ) ((pos "L@%" ) ))
( "very"  j ( v e1 r ii ) ((pos "Oq%" "Pu*" ) ))
( "vesicle"  n ( v e1 s i k l ) ((pos "K6$" ) ))
( "vesicles"  n ( v e1 s i k l z ) ((pos "Kj$" ) ))
( "vesicular"  j ( v @1 s i1 k y u l @ r ) ((pos "OA$" ) ))
( "vespers"  n ( v e1 s p @ z ) ((pos "Kj$" ) ))
( "vessel"  n ( v e1 s @ l ) ((pos "K6%" ) ))
( "vessels"  n ( v e1 s @ l z ) ((pos "Kj%" ) ))
( "vest"  v ( v e1 s t ) ((pos "J0$" "K6%" ) (subcat "3A" "6A" )))
( "vestal"  n ( v e1 s t @ l ) ((pos "K6$" "OA$" ) ))
( "vestals"  n ( v e1 s t @ l z ) ((pos "Kj$" ) ))
( "vested"  v ( v e1 s t i d ) ((pos "Jc$" "Jd%" ) (subcat "3A" "6A" )))
( "vestibule"  n ( v e1 s t i b y uu l ) ((pos "K6%" ) ))
( "vestibules"  n ( v e1 s t i b y uu l z ) ((pos "Kj%" ) ))
( "vestige"  n ( v e1 s t i jh ) ((pos "M6%" ) ))
( "vestiges"  n ( v e1 s t i jh i z ) ((pos "Mj%" ) ))
( "vestigial"  j ( v e1 s t i1 jh i@ l ) ((pos "OA%" ) ))
( "vesting"  v ( v e1 s t i ng ) ((pos "Jb$" ) (subcat "3A" "6A" )))
( "vestment"  n ( v e1 s t m @ n t ) ((pos "K6%" ) ))
( "vestments"  n ( v e1 s t m @ n t s ) ((pos "Kj%" ) ))
( "vestries"  n ( v e1 s t r i z ) ((pos "Kj%" ) ))
( "vestry"  n ( v e1 s t r ii ) ((pos "K8%" ) ))
( "vestryman"  n ( v e1 s t r i m @ n ) ((pos "Ki%" ) ))
( "vestrymen"  n ( v e1 s t r i m @ n ) ((pos "Kj%" ) ))
( "vests"  v ( v e1 s t s ) ((pos "Ja$" "Kj%" ) (subcat "3A" "6A" )))
( "vesture"  v ( v e1 s ch @ r ) ((pos "H2$" "K6$" ) ))
( "vestured"  v ( v e1 s ch @ d ) ((pos "Hc$" "Hd$" ) ))
( "vestures"  v ( v e1 s ch @ z ) ((pos "Ha$" "Kj$" ) ))
( "vesturing"  v ( v e1 s ch @ r i ng ) ((pos "Hb$" ) ))
( "vet"  v ( v e1 t ) ((pos "H4%" "K6%" ) (subcat )))
( "vetch"  n ( v e1 ch ) ((pos "M7$" ) ))
( "vetches"  n ( v e1 ch i z ) ((pos "Mj$" ) ))
( "veteran"  n ( v e1 t @ r @ n ) ((pos "K6%" ) ))
( "veterans"  n ( v e1 t @ r @ n z ) ((pos "Kj%" ) ))
( "veterinary"  j ( v e1 t r i n r ii ) ((pos "OA%" ) ))
( "veto"  v ( v ii1 t ou ) ((pos "H0%" "K7%" ) (subcat )))
( "vetoed"  v ( v ii1 t ou d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vetoes"  n ( v ii1 t ou z ) ((pos "Kj%" ) ))
( "vetoing"  v ( v ii1 t ou i ng ) ((pos "Hb%" ) (subcat )))
( "vetos"  v ( v ii1 t ou z ) ((pos "Ha%" ) (subcat )))
( "vets"  v ( v e1 t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "vetted"  v ( v e1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vetting"  v ( v e1 t i ng ) ((pos "Hb%" ) (subcat )))
( "vex"  v ( v e1 k s ) ((pos "H1%" ) (subcat )))
( "vexation"  n ( v e1 k s ei1 sh @ n ) ((pos "M6%" ) ))
( "vexations"  n ( v e1 k s ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vexatious"  j ( v e1 k s ei1 sh @ s ) ((pos "OA%" ) ))
( "vexed"  v ( v e1 k s t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vexes"  v ( v e1 k s i z ) ((pos "Ha%" ) (subcat )))
( "vexing"  v ( v e1 k s i ng ) ((pos "Hb%" ) (subcat )))
( "via"  in ( v ai1 @ ) ((pos "T-%" ) ))
( "via media"  n ( v ai2 @ - m ii1 d i@ ) ((pos "Ki$" ) ))
( "viability"  n ( v ai2 @ b i1 l i t ii ) ((pos "L@%" ) ))
( "viable"  j ( v ai1 @ b l ) ((pos "OA%" ) ))
( "viaduct"  n ( v ai1 @ d uh k t ) ((pos "K6%" ) ))
( "viaducts"  n ( v ai1 @ d uh k t s ) ((pos "Kj%" ) ))
( "vial"  n ( v ai1 @ l ) ((pos "K6$" ) ))
( "vials"  n ( v ai1 @ l z ) ((pos "Kj$" ) ))
( "vibes"  n ( v ai1 b z ) ((pos "Kj%" ) ))
( "vibrant"  j ( v ai1 b r @ n t ) ((pos "OA%" ) ))
( "vibraphone"  n ( v ai1 b r @ f ou n ) ((pos "K6%" ) ))
( "vibraphones"  n ( v ai1 b r @ f ou n z ) ((pos "Kj%" ) ))
( "vibrate"  v ( v ai1 b r ei1 t ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "vibrated"  v ( v ai1 b r ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "vibrates"  v ( v ai1 b r ei1 t s ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "vibrating"  v ( v ai1 b r ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "vibration"  n ( v ai1 b r ei1 sh @ n ) ((pos "M6%" ) ))
( "vibrations"  n ( v ai1 b r ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vibrato"  n ( v i1 b r aa1 t ou ) ((pos "M6%" ) ))
( "vibrator"  n ( v ai1 b r ei1 t @ r ) ((pos "K6%" ) ))
( "vibrators"  n ( v ai1 b r ei1 t @ z ) ((pos "Kj%" ) ))
( "vibratos"  n ( v i1 b r aa1 t ou z ) ((pos "Mj$" ) ))
( "vicar"  n ( v i1 k @ r ) ((pos "K6%" ) ))
( "vicarage"  n ( v i1 k @ r i jh ) ((pos "K6%" ) ))
( "vicarages"  n ( v i1 k @ r i jh i z ) ((pos "Kj%" ) ))
( "vicarious"  j ( v i1 k e@1 r i@ s ) ((pos "OA%" ) ))
( "vicariously"  a ( v i1 k e@1 r i@ s l ii ) ((pos "Pu%" ) ))
( "vicars"  n ( v i1 k @ z ) ((pos "Kj%" ) ))
( "vice"  n ( v ai s ) ((pos "M6%" "T-%" ) ))
( "vice versa"  a ( v ai2 s i - v @@1 s @ ) ((pos "Pu%" ) ))
( "vice-"  j ( v ai s - ) ((pos "U-%" ) ))
( "viceregal"  j ( v ai1 s r ii1 g @ l ) ((pos "OA$" ) ))
( "vicereine"  n ( v ai1 s r ei1 n ) ((pos "K6$" ) ))
( "vicereines"  n ( v ai1 s r ei1 n z ) ((pos "Kj$" ) ))
( "viceroy"  n ( v ai1 s r oi ) ((pos "K6%" ) ))
( "viceroys"  n ( v ai1 s r oi z ) ((pos "Kj%" ) ))
( "vices"  n ( v ai1 s i z ) ((pos "Mj%" ) ))
( "vicinities"  n ( v i1 s i1 n i t i z ) ((pos "Mj$" ) ))
( "vicinity"  n ( v i1 s i1 n i t ii ) ((pos "M8%" ) ))
( "vicious"  j ( v i1 sh @ s ) ((pos "OA%" ) ))
( "viciously"  a ( v i1 sh @ s l ii ) ((pos "Pu%" ) ))
( "viciousness"  n ( v i1 sh @ s n @ s ) ((pos "L@%" ) ))
( "vicissitude"  n ( v i1 s i1 s i t y uu d ) ((pos "K6%" ) ))
( "vicissitudes"  n ( v i1 s i1 s i t y uu d z ) ((pos "Kj%" ) ))
( "victim"  n ( v i1 k t i m ) ((pos "K6%" ) ))
( "victimization"  n ( v i2 k t i m ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "victimizations"  n ( v i2 k t i m ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "victimize"  v ( v i1 k t i m ai z ) ((pos "H2%" ) (subcat )))
( "victimized"  v ( v i1 k t i m ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "victimizes"  v ( v i1 k t i m ai z i z ) ((pos "Ha%" ) (subcat )))
( "victimizing"  v ( v i1 k t i m ai z i ng ) ((pos "Hb%" ) (subcat )))
( "victims"  n ( v i1 k t i m z ) ((pos "Kj%" ) ))
( "victor"  n ( v i1 k t @ r ) ((pos "K6%" ) ))
( "victoria"  n ( v i1 k t oo1 r i@ ) ((pos "K6$" ) ))
( "victorias"  n ( v i1 k t oo1 r i@ z ) ((pos "Kj$" ) ))
( "victories"  n ( v i1 k t @ r i z ) ((pos "Mj%" ) ))
( "victorious"  j ( v i1 k t oo1 r i@ s ) ((pos "OA%" ) ))
( "victoriously"  a ( v i1 k t oo1 r i@ s l ii ) ((pos "Pu%" ) ))
( "victors"  n ( v i1 k t @ z ) ((pos "Kj%" ) ))
( "victory"  n ( v i1 k t @ r ii ) ((pos "M8%" ) ))
( "victual"  v ( v i1 t l ) ((pos "J4$" "K6$" ) (subcat "2A" )))
( "victualled"  v ( v i1 t l d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "victualler"  n ( v i1 t l @ r ) ((pos "K6$" ) ))
( "victuallers"  n ( v i1 t l @ z ) ((pos "Kj$" ) ))
( "victualling"  v ( v i1 t @ l i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "victuals"  v ( v i1 t l z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "vicu~na"  n ( v i1 k y uu1 n @ ) ((pos "K6$" ) ))
( "vicu~nas"  n ( v i1 k y uu1 n @ z ) ((pos "Kj$" ) ))
( "vide"  nil ( v ai1 d ii ) ((pos "Z-$" ) ))
( "videlicet"  a ( v i1 d ii1 l i s e t ) ((pos "Pu$" ) ))
( "video"  n ( v i1 d i@ @ ) ((pos "K6%" ) ))
( "videos"  n ( v i1 d i@ u z ) ((pos "Kj%" ) ))
( "videotape"  v ( v i1 d i@ u t ei p ) ((pos "H2%" "L@%" ) (subcat )))
( "videotaped"  v ( v i1 d i@ u t ei p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "videotapes"  v ( v i1 d i@ u t ei p s ) ((pos "Ha%" ) (subcat )))
( "videotaping"  v ( v i1 d i@ u t ei p i ng ) ((pos "Hb%" ) (subcat )))
( "vie"  v ( v ai1 ) ((pos "I5%" ) (subcat )))
( "vied"  v ( v ai1 d ) ((pos "Ic%" "Id%" ) (subcat )))
( "vies"  v ( v ai1 z ) ((pos "Ia%" ) (subcat )))
( "view"  v ( v y uu1 ) ((pos "H0%" "M6%" ) (subcat )))
( "viewed"  v ( v y uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "viewer"  n ( v y uu1 @ r ) ((pos "K6%" ) ))
( "viewers"  n ( v y uu1 @ z ) ((pos "Kj%" ) ))
( "viewfinder"  n ( v y uu1 f ai n d @ r ) ((pos "K6%" ) ))
( "viewfinders"  n ( v y uu1 f ai n d @ z ) ((pos "Kj%" ) ))
( "viewing"  v ( v y uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "viewless"  j ( v y uu1 l @ s ) ((pos "OA$" ) ))
( "viewpoint"  n ( v y uu1 p oi n t ) ((pos "K6%" ) ))
( "viewpoints"  n ( v y uu1 p oi n t s ) ((pos "Kj%" ) ))
( "views"  v ( v y uu1 z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "vigil"  n ( v i1 jh i l ) ((pos "M6%" ) ))
( "vigilance"  n ( v i1 jh i l @ n s ) ((pos "L@%" ) ))
( "vigilant"  j ( v i1 jh i l @ n t ) ((pos "OA%" ) ))
( "vigilante"  n ( v i2 jh i l a1 n t ii ) ((pos "K6%" ) ))
( "vigilantes"  n ( v i2 jh i l a1 n t i z ) ((pos "Kj%" ) ))
( "vigilantly"  a ( v i1 jh i l @ n t l ii ) ((pos "Pu%" ) ))
( "vigils"  n ( v i1 jh i l z ) ((pos "Mj%" ) ))
( "vignette"  n ( v ii1 n y e1 t ) ((pos "K6%" ) ))
( "vignettes"  n ( v ii1 n y e1 t s ) ((pos "Kj%" ) ))
( "vigorous"  j ( v i1 g @ r @ s ) ((pos "OA%" ) ))
( "vigorously"  a ( v i1 g @ r @ s l ii ) ((pos "Pu%" ) ))
( "vigour"  n ( v i1 g @ r ) ((pos "L@%" ) ))
( "vile"  j ( v ai1 l ) ((pos "OB%" ) ))
( "vilely"  a ( v ai1 l - l ii ) ((pos "Pu%" ) ))
( "vileness"  n ( v ai1 l n @ s ) ((pos "L@%" ) ))
( "viler"  j ( v ai1 l @ r ) ((pos "Or%" ) ))
( "vilest"  j ( v ai1 l i s t ) ((pos "Os%" ) ))
( "vilification"  n ( v i2 l i f i k ei1 sh @ n ) ((pos "M6%" ) ))
( "vilifications"  n ( v i2 l i f i k ei1 sh @ n z ) ((pos "Mj$" ) ))
( "vilified"  v ( v i1 l i f ai d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vilifies"  v ( v i1 l i f ai z ) ((pos "Ha%" ) (subcat )))
( "vilify"  v ( v i1 l i f ai ) ((pos "H3%" ) (subcat )))
( "vilifying"  v ( v i1 l i f ai i ng ) ((pos "Hb%" ) (subcat )))
( "villa"  n ( v i1 l @ ) ((pos "K6%" ) ))
( "village"  n ( v i1 l i jh ) ((pos "K6%" ) ))
( "villager"  n ( v i1 l i jh @ r ) ((pos "K6%" ) ))
( "villagers"  n ( v i1 l i jh @ z ) ((pos "Kj%" ) ))
( "villages"  n ( v i1 l i jh i z ) ((pos "Kj%" ) ))
( "villain"  n ( v i1 l @ n ) ((pos "K6%" ) ))
( "villainess"  n ( v i1 l @ n e s ) ((pos "K6%" ) ))
( "villainesses"  n ( v i1 l @ n e s i z ) ((pos "Kj$" ) ))
( "villainies"  n ( v i1 l @ n i z ) ((pos "Mj%" ) ))
( "villainous"  j ( v i1 l @ n @ s ) ((pos "OA%" ) ))
( "villains"  n ( v i1 l @ n z ) ((pos "Kj%" ) ))
( "villainy"  n ( v i1 l @ n ii ) ((pos "M8%" ) ))
( "villas"  n ( v i1 l @ z ) ((pos "Kj%" ) ))
( "villein"  n ( v i1 l ei n ) ((pos "K6$" ) ))
( "villeinage"  n ( v i1 l i n i jh ) ((pos "L@$" ) ))
( "villeins"  n ( v i1 l ei n z ) ((pos "Kj$" ) ))
( "vim"  n ( v i1 m ) ((pos "L@%" ) ))
( "vinaigrette"  n ( v i2 n i g r e1 t ) ((pos "L@%" ) ))
( "vindicate"  v ( v i1 n d i k ei t ) ((pos "H2%" ) (subcat )))
( "vindicated"  v ( v i1 n d i k ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vindicates"  v ( v i1 n d i k ei t s ) ((pos "Ha%" ) (subcat )))
( "vindicating"  v ( v i1 n d i k ei t i ng ) ((pos "Hb%" ) (subcat )))
( "vindication"  n ( v i2 n d i k ei1 sh @ n ) ((pos "M6%" ) ))
( "vindications"  n ( v i2 n d i k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vindictive"  j ( v i1 n d i1 k t i v ) ((pos "OA%" ) ))
( "vindictively"  a ( v i1 n d i1 k t i v l ii ) ((pos "Pu%" ) ))
( "vindictiveness"  n ( v i1 n d i1 k t i v n @ s ) ((pos "L@%" ) ))
( "vine"  n ( v ai1 n ) ((pos "K6%" ) ))
( "vinegar"  n ( v i1 n i g @ r ) ((pos "M6%" ) ))
( "vinegars"  n ( v i1 n i g @ z ) ((pos "Mj$" ) ))
( "vinegary"  j ( v i1 n i g @ r ii ) ((pos "OA%" ) ))
( "vineries"  n ( v ai1 n @ r i z ) ((pos "Kj$" ) ))
( "vinery"  n ( v ai1 n @ r ii ) ((pos "K8$" ) ))
( "vines"  n ( v ai1 n z ) ((pos "Kj%" ) ))
( "vineyard"  n ( v i1 n y @ d ) ((pos "K6%" ) ))
( "vineyards"  n ( v i1 n y @ d z ) ((pos "Kj%" ) ))
( "vino"  n ( v ii1 n ou ) ((pos "M7$" ) ))
( "vinoes"  n ( v ii1 n ou z ) ((pos "Mj$" ) ))
( "vinous"  j ( v ai1 n @ s ) ((pos "OA$" ) ))
( "vintage"  n ( v i1 n t i jh ) ((pos "M6%" ) ))
( "vintages"  n ( v i1 n t i jh i z ) ((pos "Mj%" ) ))
( "vintner"  n ( v i1 n t n @ r ) ((pos "K6%" ) ))
( "vintners"  n ( v i1 n t n @ z ) ((pos "Kj%" ) ))
( "vinyl"  n ( v ai1 n i l ) ((pos "M6%" ) ))
( "vinyls"  n ( v ai1 n i l z ) ((pos "Mj$" ) ))
( "viol"  n ( v ai1 @ l ) ((pos "K6$" ) ))
( "viola"  n ( v ai1 @ l @ ) ((pos "K6$" ) ))
( "viola"  n ( v i1 ou1 l @ ) ((pos "K6%" ) ))
( "violas"  n ( v ai1 @ l @ z ) ((pos "Kj$" ) ))
( "violas"  n ( v i1 ou1 l @ z ) ((pos "Kj%" ) ))
( "violate"  v ( v ai1 @ l ei t ) ((pos "H2%" ) (subcat )))
( "violated"  v ( v ai1 @ l ei t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "violates"  v ( v ai1 @ l ei t s ) ((pos "Ha%" ) (subcat )))
( "violating"  v ( v ai1 @ l ei t i ng ) ((pos "Hb%" ) (subcat )))
( "violation"  n ( v ai2 @ l ei1 sh @ n ) ((pos "M6%" ) ))
( "violations"  n ( v ai2 @ l ei1 sh @ n z ) ((pos "Mj%" ) ))
( "violence"  n ( v ai1 @ l @ n s ) ((pos "L@%" ) ))
( "violent"  j ( v ai1 @ l @ n t ) ((pos "OA%" ) ))
( "violently"  a ( v ai1 @ l @ n t l ii ) ((pos "Pu%" ) ))
( "violet"  n ( v ai1 @ l @ t ) ((pos "M6%" ) ))
( "violets"  n ( v ai1 @ l @ t s ) ((pos "Mj%" ) ))
( "violin"  n ( v ai2 @ l i1 n ) ((pos "K6%" ) ))
( "violinist"  n ( v ai2 @ l i1 n i s t ) ((pos "K6%" ) ))
( "violinists"  n ( v ai2 @ l i1 n i s t s ) ((pos "Kj%" ) ))
( "violins"  n ( v ai2 @ l i1 n z ) ((pos "Kj%" ) ))
( "viols"  n ( v ai1 @ l z ) ((pos "Kj$" ) ))
( "viper"  n ( v ai1 p @ r ) ((pos "K6%" ) ))
( "vipers"  n ( v ai1 p @ z ) ((pos "Kj%" ) ))
( "virago"  n ( v i1 r aa1 g ou ) ((pos "K6$" ) ))
( "viragoes"  n ( v i1 r aa1 g ou z ) ((pos "Kj$" ) ))
( "viragos"  n ( v i1 r aa1 g ou z ) ((pos "Kj$" ) ))
( "virgin"  n ( v @@1 jh i n ) ((pos "K6%" "OA%" ) ))
( "virginal"  n ( v @@1 jh i n l ) ((pos "K6%" "OA%" ) ))
( "virginals"  n ( v @@1 jh i n l z ) ((pos "Kj%" ) ))
( "virginity"  n ( v @1 jh i1 n i t ii ) ((pos "L@%" ) ))
( "virgins"  n ( v @@1 jh i n z ) ((pos "Kj%" ) ))
( "virgule"  n ( v @@1 g y uu l ) ((pos "K6$" ) ))
( "virgules"  n ( v @@1 g y uu l z ) ((pos "Kj$" ) ))
( "virile"  j ( v i1 r ai l ) ((pos "OA%" ) ))
( "virility"  n ( v i1 r i1 l i t ii ) ((pos "L@%" ) ))
( "virology"  n ( v ai1 @ r o1 l @ jh ii ) ((pos "L@%" ) ))
( "virtu"  n ( v @@1 t uu1 ) ((pos "L@$" ) ))
( "virtual"  j ( v @@1 ch u@ l ) ((pos "OA%" ) ))
( "virtually"  a ( v @@1 ch u@ l ii ) ((pos "Pu%" ) ))
( "virtue"  n ( v @@1 ch uu ) ((pos "M6%" ) ))
( "virtues"  n ( v @@1 ch uu z ) ((pos "Mj%" ) ))
( "virtuosi"  n ( v @@2 ch u ou1 z ii ) ((pos "Kj%" ) ))
( "virtuosity"  n ( v @@2 ch u o1 s i t ii ) ((pos "L@%" ) ))
( "virtuoso"  n ( v @@2 ch u ou1 z ou ) ((pos "K6%" ) ))
( "virtuosos"  n ( v @@2 ch u ou1 z ou z ) ((pos "Kj%" ) ))
( "virtuous"  j ( v @@1 ch u@ s ) ((pos "OA%" ) ))
( "virtuously"  a ( v @@1 ch u@ s l ii ) ((pos "Pu%" ) ))
( "virulence"  n ( v i1 r u l @ n s ) ((pos "L@%" ) ))
( "virulent"  j ( v i1 r u l @ n t ) ((pos "OA%" ) ))
( "virulently"  a ( v i1 r u l @ n t l ii ) ((pos "Pu%" ) ))
( "virus"  n ( v ai1 @ r @ s ) ((pos "K7%" ) ))
( "viruses"  n ( v ai1 @ r @ s i z ) ((pos "Kj%" ) ))
( "vis-`a-vis"  a ( v ii2 z - aa - v ii1 ) ((pos "Pu%" "T-%" ) ))
( "visa"  v ( v ii1 z @ ) ((pos "H0$" "K6%" ) (subcat )))
( "visaed"  v ( v ii1 z @ d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "visage"  n ( v i1 z i jh ) ((pos "K6%" ) ))
( "visaged"  j ( v i1 z i jh d ) ((pos "Ot%" ) ))
( "visages"  n ( v i1 z i jh i z ) ((pos "Kj%" ) ))
( "visaing"  v ( v ii1 z @ r i ng ) ((pos "Hb$" ) (subcat )))
( "visas"  v ( v ii1 z @ z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "viscera"  n ( v i1 s @ r @ ) ((pos "Kj$" ) ))
( "visceral"  j ( v i1 s @ r @ l ) ((pos "OA%" ) ))
( "viscid"  j ( v i1 s i d ) ((pos "OA$" ) ))
( "viscosity"  n ( v i1 s k o1 s i t ii ) ((pos "L@%" ) ))
( "viscount"  n ( v ai1 k au n t ) ((pos "K6%" ) ))
( "viscountcies"  n ( v ai1 k au n t s i z ) ((pos "Kj$" ) ))
( "viscountcy"  n ( v ai1 k au n t s ii ) ((pos "K8$" ) ))
( "viscountess"  n ( v ai1 k au n t i s ) ((pos "K7%" ) ))
( "viscountesses"  n ( v ai1 k au n t i s i z ) ((pos "Kj$" ) ))
( "viscounts"  n ( v ai1 k au n t s ) ((pos "Kj%" ) ))
( "viscous"  j ( v i1 s k @ s ) ((pos "OA%" ) ))
( "visibility"  n ( v i2 z @ b i1 l i t ii ) ((pos "L@%" ) ))
( "visible"  j ( v i1 z @ b l ) ((pos "OA%" ) ))
( "visibly"  a ( v i1 z @ b l ii ) ((pos "Pu%" ) ))
( "vision"  n ( v i1 zh n ) ((pos "M6%" ) ))
( "visionaries"  n ( v i1 zh @ n r i z ) ((pos "Kj%" ) ))
( "visionary"  n ( v i1 zh @ n r ii ) ((pos "K8%" "OA%" ) ))
( "visions"  n ( v i1 zh n z ) ((pos "Mj%" ) ))
( "visit"  v ( v i1 z i t ) ((pos "J0%" "K6%" ) (subcat "2C" "3A" "6A" )))
( "visitant"  n ( v i1 z i t @ n t ) ((pos "K6$" ) ))
( "visitants"  n ( v i1 z i t @ n t s ) ((pos "Kj$" ) ))
( "visitation"  n ( v i2 z i t ei1 sh @ n ) ((pos "K6%" ) ))
( "visitations"  n ( v i2 z i t ei1 sh @ n z ) ((pos "Kj%" ) ))
( "visited"  v ( v i1 z i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" )))
( "visiting"  v ( v i1 z i t i ng ) ((pos "Jb%" "L@%" ) (subcat "2C" "3A" "6A" )))
( "visitor"  n ( v i1 z i t @ r ) ((pos "K6%" ) ))
( "visitors"  n ( v i1 z i t @ z ) ((pos "Kj%" ) ))
( "visits"  v ( v i1 z i t s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "3A" "6A" )))
( "visor"  n ( v ai1 z @ r ) ((pos "K6%" ) ))
( "visors"  n ( v ai1 z @ z ) ((pos "Kj%" ) ))
( "vista"  n ( v i1 s t @ ) ((pos "K6%" ) ))
( "vistas"  n ( v i1 s t @ z ) ((pos "Kj%" ) ))
( "visual"  j ( v i1 zh u@ l ) ((pos "OA%" ) ))
( "visualization"  n ( v i2 zh u@ l ai z ei1 sh @ n ) ((pos "M6%" ) ))
( "visualizations"  n ( v i2 zh u@ l ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "visualize"  v ( v i1 zh u@ l ai z ) ((pos "H2%" ) (subcat )))
( "visualized"  v ( v i1 zh u@ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "visualizes"  v ( v i1 zh u@ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "visualizing"  v ( v i1 zh u@ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "visually"  a ( v i1 zh u@ l ii ) ((pos "Pu%" ) ))
( "vital"  j ( v ai1 t l ) ((pos "OA%" ) ))
( "vitalism"  n ( v ai1 t @ l i z @ m ) ((pos "L@$" ) ))
( "vitalist"  n ( v ai1 t @ l i s t ) ((pos "K6$" ) ))
( "vitalists"  n ( v ai1 t @ l i s t s ) ((pos "Kj$" ) ))
( "vitality"  n ( v ai1 t a1 l i t ii ) ((pos "L@%" ) ))
( "vitalize"  v ( v ai1 t @ l ai z ) ((pos "H2%" ) (subcat )))
( "vitalized"  v ( v ai1 t @ l ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vitalizes"  v ( v ai1 t @ l ai z i z ) ((pos "Ha%" ) (subcat )))
( "vitalizing"  v ( v ai1 t @ l ai z i ng ) ((pos "Hb%" ) (subcat )))
( "vitally"  a ( v ai1 t @ l ii ) ((pos "Pu%" ) ))
( "vitals"  n ( v ai1 t l z ) ((pos "Kj$" ) ))
( "vitamin"  n ( v i1 t @ m i n ) ((pos "K6%" ) ))
( "vitamins"  n ( v i1 t @ m i n z ) ((pos "Kj%" ) ))
( "vitiate"  v ( v i1 sh i ei t ) ((pos "H2$" ) (subcat )))
( "vitiated"  v ( v i1 sh i ei t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "vitiates"  v ( v i1 sh i ei t s ) ((pos "Ha$" ) (subcat )))
( "vitiating"  v ( v i1 sh i ei t i ng ) ((pos "Hb$" ) (subcat )))
( "vitreous"  j ( v i1 t r i@ s ) ((pos "OA%" ) ))
( "vitrified"  v ( v i1 t r i f ai d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "vitrifies"  v ( v i1 t r i f ai z ) ((pos "Ja$" ) (subcat "2A" )))
( "vitrify"  v ( v i1 t r i f ai ) ((pos "J3$" ) (subcat "2A" )))
( "vitrifying"  v ( v i1 t r i f ai i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "vitriol"  n ( v i1 t r i@ l ) ((pos "L@%" ) ))
( "vitriolic"  j ( v i2 t r i o1 l i k ) ((pos "OA%" ) ))
( "vituperate"  v ( v i1 t y uu1 p @ r ei t ) ((pos "H2$" ) (subcat )))
( "vituperated"  v ( v i1 t y uu1 p @ r ei t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "vituperates"  v ( v i1 t y uu1 p @ r ei t s ) ((pos "Ha$" ) (subcat )))
( "vituperating"  v ( v i1 t y uu1 p @ r ei t i ng ) ((pos "Hb$" ) (subcat )))
( "vituperation"  n ( v i1 t y uu2 p @ r ei1 sh @ n ) ((pos "L@$" ) ))
( "vituperative"  j ( v i1 t y uu1 p @ r @ t i v ) ((pos "OA%" ) ))
( "viva"  n ( v ai1 v @ ) ((pos "K6$" ) ))
( "viva voce"  n ( v ai2 v @ - v ou1 s ii ) ((pos "K6$" "OA$" "Pu$" ) ))
( "viva voces"  n ( v ai2 v @ - v ou1 s i z ) ((pos "Kj$" ) ))
( "vivace"  a ( v i1 v aa1 ch ei ) ((pos "Pu$" ) ))
( "vivacious"  j ( v i1 v ei1 sh @ s ) ((pos "OA%" ) ))
( "vivaciously"  a ( v i1 v ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "vivacity"  n ( v i1 v a1 s i t ii ) ((pos "L@%" ) ))
( "vivas"  n ( v ai1 v @ z ) ((pos "Kj$" ) ))
( "vivid"  j ( v i1 v i d ) ((pos "OA%" ) ))
( "vividly"  a ( v i1 v i d l ii ) ((pos "Pu%" ) ))
( "vividness"  n ( v i1 v i d n @ s ) ((pos "L@%" ) ))
( "viviparous"  j ( v i1 v i1 p @ r @ s ) ((pos "OA$" ) ))
( "vivisect"  v ( v i2 v i s e1 k t ) ((pos "H0$" ) (subcat )))
( "vivisected"  v ( v i2 v i s e1 k t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "vivisecting"  v ( v i2 v i s e1 k t i ng ) ((pos "Hb$" ) (subcat )))
( "vivisection"  n ( v i2 v i s e1 k sh @ n ) ((pos "M6%" ) ))
( "vivisectionist"  n ( v i2 v i s e1 k sh @ n i s t ) ((pos "K6$" ) ))
( "vivisectionists"  n ( v i2 v i s e1 k sh @ n i s t s ) ((pos "Kj$" ) ))
( "vivisections"  n ( v i2 v i s e1 k sh @ n z ) ((pos "Mj$" ) ))
( "vivisects"  v ( v i2 v i s e1 k t s ) ((pos "Ha$" ) (subcat )))
( "vixen"  n ( v i1 k s n ) ((pos "K6%" ) ))
( "vixenish"  j ( v i1 k s @ n i sh ) ((pos "OA$" ) ))
( "vixens"  n ( v i1 k s n z ) ((pos "Kj%" ) ))
( "viz"  n ( v i1 z ) ((pos "Y~%" ) ))
( "vizier"  n ( v i1 z i@1 r ) ((pos "K6$" ) ))
( "viziers"  n ( v i1 z i@1 z ) ((pos "Kj$" ) ))
( "vocabularies"  n ( v @1 k a1 b y u l @ r i z ) ((pos "Mj%" ) ))
( "vocabulary"  n ( v @1 k a1 b y u l @ r ii ) ((pos "M8%" ) ))
( "vocal"  j ( v ou1 k l ) ((pos "OA%" ) ))
( "vocalist"  n ( v ou1 k @ l i s t ) ((pos "K6%" ) ))
( "vocalists"  n ( v ou1 k @ l i s t s ) ((pos "Kj%" ) ))
( "vocalize"  v ( v ou1 k @ l ai z ) ((pos "H2%" ) ))
( "vocalized"  v ( v ou1 k @ l ai z d ) ((pos "Hc%" "Hd%" ) ))
( "vocalizes"  v ( v ou1 k @ l ai z i z ) ((pos "Ha%" ) ))
( "vocalizing"  v ( v ou1 k @ l ai z i ng ) ((pos "Hb%" ) ))
( "vocally"  a ( v ou1 k @ l ii ) ((pos "Pu%" ) ))
( "vocation"  n ( v ou1 k ei1 sh @ n ) ((pos "M6%" ) ))
( "vocational"  j ( v ou1 k ei1 sh @ n @ l ) ((pos "OA%" ) ))
( "vocations"  n ( v ou1 k ei1 sh @ n z ) ((pos "Mj%" ) ))
( "vocative"  n ( v o1 k @ t i v ) ((pos "K6$" "OA$" ) ))
( "vocatives"  n ( v o1 k @ t i v z ) ((pos "Kj$" ) ))
( "vociferate"  v ( v @1 s i1 f @ r ei t ) ((pos "J2$" ) (subcat "2A" )))
( "vociferated"  v ( v @1 s i1 f @ r ei t i d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "vociferates"  v ( v @1 s i1 f @ r ei t s ) ((pos "Ja$" ) (subcat "2A" )))
( "vociferating"  v ( v @1 s i1 f @ r ei t i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "vociferation"  n ( v @1 s i2 f @ r ei1 sh @ n ) ((pos "L@$" ) ))
( "vociferous"  j ( v @1 s i1 f @ r @ s ) ((pos "OA%" ) ))
( "vodka"  n ( v o1 d k @ ) ((pos "M6%" ) ))
( "vodkas"  n ( v o1 d k @ z ) ((pos "Mj%" ) ))
( "vogue"  n ( v ou1 g ) ((pos "K6%" ) ))
( "vogues"  n ( v ou1 g z ) ((pos "Kj$" ) ))
( "voice"  v ( v oi1 s ) ((pos "H2%" "M6*" ) (subcat )))
( "voiced"  v ( v oi1 s t ) ((pos "Hc%" "Hd%" "Ot%" ) (subcat )))
( "voiceless"  j ( v oi1 s l @ s ) ((pos "OA%" ) ))
( "voices"  v ( v oi1 s i z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "voicing"  v ( v oi1 s i ng ) ((pos "Hb%" ) (subcat )))
( "void"  v ( v oi1 d ) ((pos "H0$" "K6%" "OA%" ) (subcat )))
( "voided"  v ( v oi1 d i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "voiding"  v ( v oi1 d i ng ) ((pos "Hb$" ) (subcat )))
( "voids"  v ( v oi1 d z ) ((pos "Ha$" "Kj$" ) (subcat )))
( "voile"  n ( v oi1 l ) ((pos "L@$" ) ))
( "vol"  n ( v o1 l ) ((pos "Y>%" ) ))
( "volatile"  j ( v o1 l @ t ai l ) ((pos "OA%" ) ))
( "volatilities"  n ( v o2 l @ t i1 l i t i z ) ((pos "Mj$" ) ))
( "volatility"  n ( v o2 l @ t i1 l i t ii ) ((pos "M8%" ) ))
( "volcanic"  j ( v o1 l k a1 n i k ) ((pos "OA%" ) ))
( "volcano"  n ( v o1 l k ei1 n ou ) ((pos "K6%" ) ))
( "volcanoes"  n ( v o1 l k ei1 n ou z ) ((pos "Kj%" ) ))
( "volcanos"  n ( v o1 l k ei1 n ou z ) ((pos "Kj%" ) ))
( "vole"  n ( v ou1 l ) ((pos "K6%" ) ))
( "voles"  n ( v ou1 l z ) ((pos "Kj%" ) ))
( "volition"  n ( v @1 l i1 sh @ n ) ((pos "L@%" ) ))
( "volitional"  j ( v @1 l i1 sh @ n @ l ) ((pos "OA$" ) ))
( "volley"  v ( v o1 l ii ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" )))
( "volleyball"  n ( v o1 l i b oo l ) ((pos "M6%" ) ))
( "volleyballs"  n ( v o1 l i b oo l z ) ((pos "Mj$" ) ))
( "volleyed"  v ( v o1 l i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "volleying"  v ( v o1 l i i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "volleys"  v ( v o1 l i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "vols"  n ( v o1 l z ) ((pos "Y)%" ) ))
( "volt"  n ( v ou1 l t ) ((pos "K6%" ) ))
( "voltage"  n ( v ou1 l t i jh ) ((pos "M6%" ) ))
( "voltages"  n ( v ou1 l t i jh i z ) ((pos "Mj%" ) ))
( "volte-face"  n ( v o2 l t - f aa1 s ) ((pos "Ki$" ) ))
( "volts"  n ( v ou1 l t s ) ((pos "Kj%" ) ))
( "volubility"  n ( v o2 l y u b i1 l i t ii ) ((pos "L@%" ) ))
( "voluble"  j ( v o1 l y u b l ) ((pos "OA%" ) ))
( "volubly"  a ( v o1 l y u b l ii ) ((pos "Pu%" ) ))
( "volume"  n ( v o1 l y uu m ) ((pos "M6%" ) ))
( "volumes"  n ( v o1 l y uu m z ) ((pos "Mj%" ) ))
( "voluminous"  j ( v @1 l y uu1 m i n @ s ) ((pos "OA%" ) ))
( "voluntaries"  n ( v o1 l @ n t r i z ) ((pos "Kj$" ) ))
( "voluntarily"  a ( v o1 l @ n t r @ l ii ) ((pos "Pu%" ) ))
( "voluntary"  n ( v o1 l @ n t r ii ) ((pos "K8$" "OA%" ) ))
( "volunteer"  v ( v o2 l @ n t i@1 r ) ((pos "J0%" "K6%" ) (subcat "2A" "3A" "6A" )))
( "volunteered"  v ( v o2 l @ n t i@1 d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "volunteering"  v ( v o2 l @ n t i@1 r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "volunteers"  v ( v o2 l @ n t i@1 z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" )))
( "voluptuaries"  n ( v @1 l uh1 p ch u@ r i z ) ((pos "Kj$" ) ))
( "voluptuary"  n ( v @1 l uh1 p ch u@ r ii ) ((pos "K8$" ) ))
( "voluptuous"  j ( v @1 l uh1 p ch u@ s ) ((pos "OA%" ) ))
( "voluptuously"  a ( v @1 l uh1 p ch u@ s l ii ) ((pos "Pu%" ) ))
( "voluptuousness"  n ( v @1 l uh1 p ch u@ s n @ s ) ((pos "L@%" ) ))
( "volute"  n ( v @1 l y uu1 t ) ((pos "K6$" ) ))
( "voluted"  j ( v @1 l y uu1 t i d ) ((pos "OA$" ) ))
( "volutes"  n ( v @1 l y uu1 t s ) ((pos "Kj$" ) ))
( "vomit"  v ( v o1 m i t ) ((pos "J0%" "L@%" ) (subcat "2A" "6A" )))
( "vomited"  v ( v o1 m i t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "vomiting"  v ( v o1 m i t i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "vomits"  v ( v o1 m i t s ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "voodoo"  n ( v uu1 d uu ) ((pos "L@%" ) ))
( "voodooism"  n ( v uu1 d uu i z @ m ) ((pos "L@$" ) ))
( "voracious"  j ( v @1 r ei1 sh @ s ) ((pos "OA%" ) ))
( "voraciously"  a ( v @1 r ei1 sh @ s l ii ) ((pos "Pu%" ) ))
( "voracity"  n ( v @1 r a1 s i t ii ) ((pos "L@$" ) ))
( "vortex"  n ( v oo1 t e k s ) ((pos "K7%" ) ))
( "vortexes"  n ( v oo1 t e k s i z ) ((pos "Kj%" ) ))
( "vortices"  n ( v oo1 t i s ii z ) ((pos "Kj%" ) ))
( "votaries"  n ( v ou1 t @ r i z ) ((pos "Kj$" ) ))
( "votary"  n ( v ou1 t @ r ii ) ((pos "K8$" ) ))
( "vote"  v ( v ou1 t ) ((pos "J2%" "K6%" ) (subcat "3A" "6A" "9" "12B" "13B" "15B" )))
( "voted"  v ( v ou1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "3A" "6A" "9" "12B" "13B" "15B" )))
( "voteless"  j ( v ou1 t l @ s ) ((pos "OA$" ) ))
( "voter"  n ( v ou1 t @ r ) ((pos "K6%" ) ))
( "voters"  n ( v ou1 t @ z ) ((pos "Kj%" ) ))
( "votes"  v ( v ou1 t s ) ((pos "Ja%" "Kj%" ) (subcat "3A" "6A" "9" "12B" "13B" "15B" )))
( "voting"  v ( v ou1 t i ng ) ((pos "Jb%" ) (subcat "3A" "6A" "9" "12B" "13B" "15B" )))
( "votive"  j ( v ou1 t i v ) ((pos "OA$" ) ))
( "vouch"  v ( v au1 ch ) ((pos "I1%" ) ))
( "vouched"  v ( v au1 ch t ) ((pos "Ic%" "Id%" ) ))
( "voucher"  n ( v au1 ch @ r ) ((pos "K6%" ) ))
( "vouchers"  n ( v au1 ch @ z ) ((pos "Kj%" ) ))
( "vouches"  v ( v au1 ch i z ) ((pos "Ia%" ) ))
( "vouching"  v ( v au1 ch i ng ) ((pos "Ib%" ) ))
( "vouchsafe"  v ( v au1 ch s ei1 f ) ((pos "H2%" ) (subcat "6A" "7A" )))
( "vouchsafed"  v ( v au1 ch s ei1 f t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "vouchsafes"  v ( v au1 ch s ei1 f s ) ((pos "Ha%" ) (subcat "6A" "7A" )))
( "vouchsafing"  v ( v au1 ch s ei1 f i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "vow"  v ( v au1 ) ((pos "H0%" "K6%" ) (subcat "6A" "7A" )))
( "vowed"  v ( v au1 d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "7A" )))
( "vowel"  n ( v au1 @ l ) ((pos "K6%" ) ))
( "vowels"  n ( v au1 @ l z ) ((pos "Kj%" ) ))
( "vowing"  v ( v au1 i ng ) ((pos "Hb%" ) (subcat "6A" "7A" )))
( "vows"  v ( v au1 z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "7A" )))
( "vox"  n ( v o1 k s ) ((pos "Ki$" ) ))
( "vox populi"  n ( v o2 k s - p o1 p y u l ai ) ((pos "Ki$" ) ))
( "voyage"  v ( v oi1 i jh ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "voyaged"  v ( v oi1 i jh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "voyager"  n ( v oi1 @ jh @ r ) ((pos "K6%" ) ))
( "voyagers"  n ( v oi1 @ jh @ z ) ((pos "Kj%" ) ))
( "voyages"  v ( v oi1 i jh i z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "voyaging"  v ( v oi1 i jh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "voyeur"  n ( v w aa1 y @@1 r ) ((pos "K6%" ) ))
( "voyeurism"  n ( v w aa1 y @@1 r i z m ) ((pos "L@%" ) ))
( "voyeuristic"  j ( v w aa2 y @ r i1 s t i k ) ((pos "OA%" ) ))
( "voyeuristically"  a ( v w aa2 y @ r i1 s t i k l ii ) ((pos "Pu%" ) ))
( "voyeurs"  n ( v w aa1 y @@1 z ) ((pos "Kj%" ) ))
( "vs"  n ( v @@1 s @ s ) ((pos "Y~%" ) ))
( "vulcanite"  n ( v uh1 l k @ n ai t ) ((pos "L@$" ) ))
( "vulcanization"  n ( v uh2 l k @ n ai z ei1 sh @ n ) ((pos "M6$" ) ))
( "vulcanizations"  n ( v uh2 l k @ n ai z ei1 sh @ n z ) ((pos "Mj$" ) ))
( "vulcanize"  v ( v uh1 l k @ n ai z ) ((pos "H2$" ) (subcat )))
( "vulcanized"  v ( v uh1 l k @ n ai z d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "vulcanizes"  v ( v uh1 l k @ n ai z i z ) ((pos "Ha$" ) (subcat )))
( "vulcanizing"  v ( v uh1 l k @ n ai z i ng ) ((pos "Hb$" ) (subcat )))
( "vulgar"  j ( v uh1 l g @ r ) ((pos "OA%" ) ))
( "vulgarian"  n ( v uh1 l g e@1 r i@ n ) ((pos "K6$" ) ))
( "vulgarians"  n ( v uh1 l g e@1 r i@ n z ) ((pos "Kj$" ) ))
( "vulgarism"  n ( v uh1 l g @ r i z @ m ) ((pos "M6$" ) ))
( "vulgarisms"  n ( v uh1 l g @ r i z @ m z ) ((pos "Mj$" ) ))
( "vulgarities"  n ( v uh1 l g a1 r i t i z ) ((pos "Mj%" ) ))
( "vulgarity"  n ( v uh1 l g a1 r i t ii ) ((pos "M8%" ) ))
( "vulgarization"  n ( v uh2 l g @ r ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "vulgarize"  v ( v uh1 l g @ r ai z ) ((pos "H2%" ) (subcat )))
( "vulgarized"  v ( v uh1 l g @ r ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "vulgarizes"  v ( v uh1 l g @ r ai z i z ) ((pos "Ha%" ) (subcat )))
( "vulgarizing"  v ( v uh1 l g @ r ai z i ng ) ((pos "Hb%" ) (subcat )))
( "vulgarly"  a ( v uh1 l g @ l ii ) ((pos "Pu%" ) ))
( "vulnerabilities"  n ( v uh2 l n @ r @ b i1 l i t i z ) ((pos "Mj%" ) ))
( "vulnerability"  n ( v uh2 l n @ r @ b i1 l i t ii ) ((pos "M8%" ) ))
( "vulnerable"  j ( v uh1 l n @ r @ b l ) ((pos "OA%" ) ))
( "vulpine"  j ( v uh1 l p ai n ) ((pos "OA$" ) ))
( "vulture"  n ( v uh1 l ch @ r ) ((pos "K6%" ) ))
( "vultures"  n ( v uh1 l ch @ z ) ((pos "Kj%" ) ))
( "vulva"  n ( v uh1 l v @ ) ((pos "K6$" ) ))
( "vulvas"  n ( v uh1 l v @ z ) ((pos "Kj$" ) ))
( "vying"  v ( v ai1 i ng ) ((pos "Ib%" ) (subcat )))
( "w"  n ( d uh1 b l y uu ) ((pos "Ki$" ) ))
( "w's"  n ( d uh1 b l y uu z ) ((pos "Kj$" ) ))
( "wad"  v ( w o1 d ) ((pos "H4$" "K6%" ) ))
( "wadded"  v ( w o1 d i d ) ((pos "Hc$" "Hd$" ) ))
( "wadding"  v ( w o1 d i ng ) ((pos "Hb$" "L@%" ) ))
( "waddle"  v ( w o1 d l ) ((pos "I2%" "Ki%" ) (subcat "2A" )))
( "waddled"  v ( w o1 d l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "waddles"  v ( w o1 d l z ) ((pos "Ia%" ) (subcat "2A" )))
( "waddling"  v ( w o1 d l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "wade"  v ( w ei1 d ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "waded"  v ( w ei1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "wader"  n ( w ei1 d @ r ) ((pos "K6%" ) ))
( "waders"  n ( w ei1 d @ z ) ((pos "Kj%" ) ))
( "wades"  v ( w ei1 d z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "wadi"  n ( w o1 d ii ) ((pos "K6$" ) ))
( "wading"  v ( w ei1 d i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "wadis"  n ( w o1 d i z ) ((pos "Kj$" ) ))
( "wads"  v ( w o1 d z ) ((pos "Ha$" "Kj%" ) ))
( "wafer"  n ( w ei1 f @ r ) ((pos "K6%" ) ))
( "wafers"  n ( w ei1 f @ z ) ((pos "Kj%" ) ))
( "waffle"  v ( w o1 f l ) ((pos "I2%" "M6%" ) (subcat "2A" )))
( "waffled"  v ( w o1 f l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "waffles"  v ( w o1 f l z ) ((pos "Ia%" "Mj%" ) (subcat "2A" )))
( "waffling"  v ( w o1 f l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "waft"  v ( w o1 f t ) ((pos "H0%" "K6%" ) (subcat )))
( "wafted"  v ( w o1 f t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wafting"  v ( w o1 f t i ng ) ((pos "Hb%" ) (subcat )))
( "wafts"  v ( w o1 f t s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "wag"  v ( w a1 g ) ((pos "J4%" "K6%" ) (subcat "2A" "2C" )))
( "wage"  v ( w ei1 jh ) ((pos "H2%" "K6%" ) (subcat )))
( "wage-claim"  n ( w ei1 jh - k l ei m ) ((pos "K6%" ) ))
( "wage-claims"  n ( w ei1 jh - k l ei m z ) ((pos "Kj%" ) ))
( "wage-earner"  n ( w ei1 jh - @@ n @ r ) ((pos "K6%" ) ))
( "wage-earners"  n ( w ei1 jh - @@ n @ z ) ((pos "Kj%" ) ))
( "wage-freeze"  n ( w ei1 jh - f r ii z ) ((pos "K6%" ) ))
( "wage-freezes"  n ( w ei1 jh - f r ii z i z ) ((pos "Kj%" ) ))
( "waged"  v ( w ei1 jh d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wager"  v ( w ei1 jh @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "6A" "11" "12C" )))
( "wagered"  v ( w ei1 jh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "11" "12C" )))
( "wagering"  v ( w ei1 jh @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "11" "12C" )))
( "wagers"  v ( w ei1 jh @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "11" "12C" )))
( "wages"  v ( w ei1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "wagged"  v ( w a1 g d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "waggeries"  n ( w a1 g @ r i z ) ((pos "Mj$" ) ))
( "waggery"  n ( w a1 g @ r ii ) ((pos "M8$" ) ))
( "wagging"  v ( w a1 g i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "waggish"  j ( w a1 g i sh ) ((pos "OA$" ) ))
( "waggishly"  a ( w a1 g i sh l ii ) ((pos "Pu$" ) ))
( "waggishness"  n ( w a1 g i sh n @ s ) ((pos "L@$" ) ))
( "waggle"  v ( w a1 g l ) ((pos "J2%" ) ))
( "waggled"  v ( w a1 g l d ) ((pos "Jc%" "Jd%" ) ))
( "waggles"  v ( w a1 g l z ) ((pos "Ja%" ) ))
( "waggling"  v ( w a1 g l i ng ) ((pos "Jb%" ) ))
( "waggon"  n ( w a1 g @ n ) ((pos "K6%" ) ))
( "waggoner"  n ( w a1 g @ n @ r ) ((pos "K6%" ) ))
( "waggoners"  n ( w a1 g @ n @ z ) ((pos "Kj%" ) ))
( "waggons"  n ( w a1 g @ n z ) ((pos "Kj%" ) ))
( "waging"  v ( w ei1 jh i ng ) ((pos "Hb%" ) (subcat )))
( "wagon-lit"  n ( v a2 g o n - l ii1 ) ((pos "Ki$" ) ))
( "wagons-lit"  n ( v a2 g o n - l ii1 ) ((pos "Kj$" ) ))
( "wags"  v ( w a1 g z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "wagtail"  n ( w a1 g t ei l ) ((pos "K6%" ) ))
( "wagtails"  n ( w a1 g t ei l z ) ((pos "Kj%" ) ))
( "waif"  n ( w ei1 f ) ((pos "K6%" ) ))
( "waifs"  n ( w ei1 f s ) ((pos "Kj%" ) ))
( "wail"  v ( w ei1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "wailed"  v ( w ei1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "wailing"  v ( w ei1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "wails"  v ( w ei1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "wain"  n ( w ei1 n ) ((pos "K6$" ) ))
( "wains"  n ( w ei1 n z ) ((pos "Kj$" ) ))
( "wainscot"  n ( w ei1 n s k @ t ) ((pos "K6%" ) ))
( "wainscoted"  j ( w ei1 n s k @ t i d ) ((pos "OA$" ) ))
( "wainscots"  n ( w ei1 n s k @ t s ) ((pos "Kj$" ) ))
( "waist"  n ( w ei1 s t ) ((pos "K6%" ) ))
( "waist-deep"  j ( w ei1 s t - d ii1 p ) ((pos "OA%" "Pu%" ) ))
( "waist-high"  j ( w ei1 s t - h ai1 ) ((pos "OA%" "Pu%" ) ))
( "waistband"  n ( w ei1 s t b a n d ) ((pos "K6%" ) ))
( "waistbands"  n ( w ei1 s t b a n d z ) ((pos "Kj%" ) ))
( "waistcoat"  n ( w ei1 s k ou t ) ((pos "K6%" ) ))
( "waistcoats"  n ( w ei1 s k ou t s ) ((pos "Kj%" ) ))
( "waistline"  n ( w ei1 s t l ai n ) ((pos "K6%" ) ))
( "waistlines"  n ( w ei1 s t l ai n z ) ((pos "Kj%" ) ))
( "waists"  n ( w ei1 s t s ) ((pos "Kj%" ) ))
( "wait"  v ( w ei1 t ) ((pos "J0%" "M6%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" )))
( "waited"  v ( w ei1 t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" )))
( "waiter"  n ( w ei1 t @ r ) ((pos "K6%" ) ))
( "waiters"  n ( w ei1 t @ z ) ((pos "Kj%" ) ))
( "waiting"  v ( w ei1 t i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" )))
( "waiting-list"  n ( w ei1 t i ng - l i s t ) ((pos "K6%" ) ))
( "waiting-lists"  n ( w ei1 t i ng - l i s t s ) ((pos "Kj%" ) ))
( "waiting-room"  n ( w ei1 t i ng - r uu m ) ((pos "K6%" ) ))
( "waiting-rooms"  n ( w ei1 t i ng - r uu m z ) ((pos "Kj%" ) ))
( "waitress"  n ( w ei1 t r i s ) ((pos "K7%" ) ))
( "waitresses"  n ( w ei1 t r i s i z ) ((pos "Kj%" ) ))
( "waits"  v ( w ei1 t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" )))
( "waive"  v ( w ei1 v ) ((pos "H2%" ) (subcat )))
( "waived"  v ( w ei1 v d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "waiver"  n ( w ei1 v @ r ) ((pos "K6%" ) ))
( "waivers"  n ( w ei1 v @ z ) ((pos "Kj%" ) ))
( "waives"  v ( w ei1 v z ) ((pos "Ha%" ) (subcat )))
( "waiving"  v ( w ei1 v i ng ) ((pos "Hb%" ) (subcat )))
( "wake"  v ( w ei1 k ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "4B" "6A" )))
( "waked"  v ( w ei1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4B" "6A" )))
( "wakeful"  j ( w ei1 k f @ l ) ((pos "OA%" ) ))
( "wakefulness"  n ( w ei1 k f @ l n @ s ) ((pos "L@%" ) ))
( "waken"  v ( w ei1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "wakened"  v ( w ei1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "wakening"  v ( w ei1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "wakens"  v ( w ei1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "wakes"  v ( w ei1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4B" "6A" )))
( "waking"  v ( w ei1 k i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2C" "4B" "6A" )))
( "wale"  n ( w ei1 l ) ((pos "K6$" ) ))
( "wales"  n ( w ei1 l z ) ((pos "Kj$" ) ))
( "walk"  v ( w oo1 k ) ((pos "J0%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "walkabout"  n ( w oo1 k @ b au t ) ((pos "K6%" ) ))
( "walkabouts"  n ( w oo1 k @ b au t s ) ((pos "Kj$" ) ))
( "walkaway"  n ( w oo1 k @ w ei ) ((pos "K6$" ) ))
( "walkaways"  n ( w oo1 k @ w ei z ) ((pos "Kj$" ) ))
( "walked"  v ( w oo1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "walker"  n ( w oo1 k @ r ) ((pos "K6%" ) ))
( "walkers"  n ( w oo1 k @ z ) ((pos "Kj%" ) ))
( "walkie-talkie"  n ( w oo2 k i - t oo1 k ii ) ((pos "K6%" ) ))
( "walkie-talkies"  n ( w oo2 k i - t oo1 k i z ) ((pos "Kj%" ) ))
( "walking"  v ( w oo1 k i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "walkout"  n ( w oo1 k au t ) ((pos "K6%" ) ))
( "walkouts"  n ( w oo1 k au t s ) ((pos "Kj%" ) ))
( "walkover"  n ( w oo1 k ou v @ r ) ((pos "K6%" ) ))
( "walkovers"  n ( w oo1 k ou v @ z ) ((pos "Kj%" ) ))
( "walks"  v ( w oo1 k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" "6A" "15A" )))
( "wall"  v ( w oo1 l ) ((pos "H0%" "K6%" ) (subcat )))
( "wall-eyed"  j ( w oo2 l - ai1 d ) ((pos "OA$" ) ))
( "wall-painting"  n ( w oo1 l - p ei n t i ng ) ((pos "K6%" ) ))
( "wall-paintings"  n ( w oo1 l - p ei n t i ng z ) ((pos "Kj%" ) ))
( "wallabies"  n ( w o1 l @ b i z ) ((pos "Kj%" ) ))
( "wallaby"  n ( w o1 l @ b ii ) ((pos "K8%" ) ))
( "wallah"  n ( w o1 l @ ) ((pos "K6$" ) ))
( "wallahs"  n ( w o1 l @ z ) ((pos "Kj$" ) ))
( "walled"  v ( w oo1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wallet"  n ( w o1 l i t ) ((pos "K6%" ) ))
( "wallets"  n ( w o1 l i t s ) ((pos "Kj%" ) ))
( "wallflower"  n ( w oo1 l f l au @ r ) ((pos "K6%" ) ))
( "wallflowers"  n ( w oo1 l f l au @ z ) ((pos "Kj%" ) ))
( "walling"  v ( w oo1 l i ng ) ((pos "Hb%" ) (subcat )))
( "wallop"  v ( w o1 l @ p ) ((pos "H0%" "K6%" ) ))
( "walloped"  v ( w o1 l @ p t ) ((pos "Hc%" "Hd%" ) ))
( "walloping"  v ( w o1 l @ p i ng ) ((pos "Hb%" "OA%" ) ))
( "wallops"  v ( w o1 l @ p s ) ((pos "Ha%" "Kj%" ) ))
( "wallow"  v ( w o1 l ou ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "wallowed"  v ( w o1 l ou d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "wallowing"  v ( w o1 l ou i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "wallows"  v ( w o1 l ou z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "wallpaper"  n ( w oo1 l p ei p @ r ) ((pos "L@%" ) ))
( "walls"  v ( w oo1 l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "walnut"  n ( w oo1 l n uh t ) ((pos "M6%" ) ))
( "walnuts"  n ( w oo1 l n uh t s ) ((pos "Mj%" ) ))
( "walrus"  n ( w oo1 l r @ s ) ((pos "K7%" ) ))
( "walruses"  n ( w oo1 l r @ s i z ) ((pos "Kj%" ) ))
( "waltz"  v ( w oo1 l s ) ((pos "J1%" "K7%" ) (subcat "2A" "2C" )))
( "waltzed"  v ( w oo1 l s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "waltzes"  v ( w oo1 l s i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "waltzing"  v ( w oo1 l s i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "wampum"  n ( w o1 m p @ m ) ((pos "L@$" ) ))
( "wan"  j ( w o1 n ) ((pos "OE%" ) ))
( "wand"  n ( w o1 n d ) ((pos "K6%" ) ))
( "wander"  v ( w o1 n d @ r ) ((pos "J0%" ) (subcat "2A" "2B" "2C" )))
( "wandered"  v ( w o1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "wanderer"  n ( w o1 n d @ r @ r ) ((pos "K6%" ) ))
( "wanderers"  n ( w o1 n d @ r @ z ) ((pos "Kj%" ) ))
( "wandering"  v ( w o1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "wanderings"  n ( w o1 n d @ r i ng z ) ((pos "Kj%" ) ))
( "wanderlust"  n ( w o1 n d @ l uh s t ) ((pos "L@%" ) ))
( "wanders"  v ( w o1 n d @ z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" )))
( "wands"  n ( w o1 n d z ) ((pos "Kj%" ) ))
( "wane"  v ( w ei1 n ) ((pos "I2%" "K6%" ) (subcat )))
( "waned"  v ( w ei1 n d ) ((pos "Ic%" "Id%" ) (subcat )))
( "wanes"  v ( w ei1 n z ) ((pos "Ia%" "Kj$" ) (subcat )))
( "wangle"  v ( w a1 ng g l ) ((pos "H2%" "K6%" ) (subcat )))
( "wangled"  v ( w a1 ng g l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wangles"  v ( w a1 ng g l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "wangling"  v ( w a1 ng g l i ng ) ((pos "Hb%" ) (subcat )))
( "waning"  v ( w ei1 n i ng ) ((pos "Ib%" ) (subcat )))
( "wank"  v ( w a1 ng k ) ((pos "I0%" "K6%" ) (subcat )))
( "wanked"  v ( w a1 ng k t ) ((pos "Ic%" "Id%" ) (subcat )))
( "wanking"  v ( w a1 ng k i ng ) ((pos "Ib%" ) (subcat )))
( "wanks"  v ( w a1 ng k s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "wanly"  a ( w o1 n l ii ) ((pos "Pu%" ) ))
( "wanna"  v ( w o1 n @ ) ((pos "Gh$" ) ))
( "wanner"  j ( w o1 n @ r ) ((pos "Or$" ) ))
( "wanness"  n ( w o1 n - n @ s ) ((pos "L@$" ) ))
( "wannest"  j ( w o1 n i s t ) ((pos "Os$" ) ))
( "want"  v ( w o1 n t ) ((pos "J0*" "M6%" ) (subcat "2A" "3A" "6A" "6E" "7A" "17" "19B" )))
( "want-ad"  n ( w o1 n t - a d ) ((pos "K6$" ) ))
( "want-ads"  n ( w o1 n t - a d z ) ((pos "Kj$" ) ))
( "wanted"  v ( w o1 n t i d ) ((pos "Jc*" "Jd*" ) (subcat "2A" "3A" "6A" "6E" "7A" "17" "19B" )))
( "wanting"  v ( w o1 n t i ng ) ((pos "Jb%" "T-%" ) (subcat "2A" "3A" "6A" "6E" "7A" "17" "19B" )))
( "wanton"  v ( w o1 n t @ n ) ((pos "I0$" "K6%" "OA%" ) (subcat "2A" )))
( "wantoned"  v ( w o1 n t @ n d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "wantoning"  v ( w o1 n t @ n i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "wantonly"  a ( w o1 n t @ n l ii ) ((pos "Pu%" ) ))
( "wantonness"  n ( w o1 n t @ n - n @ s ) ((pos "L@%" ) ))
( "wantons"  v ( w o1 n t @ n z ) ((pos "Ia$" "Kj%" ) (subcat "2A" )))
( "wants"  v ( w o1 n t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "6E" "7A" "17" "19B" )))
( "war"  v ( w oo1 r ) ((pos "I4%" "M6*" ) ))
( "war-babies"  n ( w oo1 - b ei b i z ) ((pos "Kj%" ) ))
( "war-baby"  n ( w oo1 - b ei b ii ) ((pos "K8%" ) ))
( "war-bride"  n ( w oo1 - b r ai d ) ((pos "K6%" ) ))
( "war-brides"  n ( w oo1 - b r ai d z ) ((pos "Kj%" ) ))
( "war-cloud"  n ( w oo1 - k l au d ) ((pos "K6%" ) ))
( "war-clouds"  n ( w oo1 - k l au d z ) ((pos "Kj%" ) ))
( "war-cries"  n ( w oo1 - k r ai z ) ((pos "Kj%" ) ))
( "war-cry"  n ( w oo1 - k r ai ) ((pos "K8%" ) ))
( "war-dance"  n ( w oo1 - d aa n s ) ((pos "K6%" ) ))
( "war-dances"  n ( w oo1 - d aa n s i z ) ((pos "Kj%" ) ))
( "war-god"  n ( w oo1 - g o d ) ((pos "K6%" ) ))
( "war-gods"  n ( w oo1 - g o d z ) ((pos "Kj%" ) ))
( "war-torn"  j ( w oo1 - t oo n ) ((pos "OA%" ) ))
( "war-widow"  n ( w oo1 - w i d ou ) ((pos "K6%" ) ))
( "war-widows"  n ( w oo1 - w i d ou z ) ((pos "Kj%" ) ))
( "warble"  v ( w oo1 b l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" )))
( "warbled"  v ( w oo1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "warbler"  n ( w oo1 b l @ r ) ((pos "K6%" ) ))
( "warblers"  n ( w oo1 b l @ z ) ((pos "Kj%" ) ))
( "warbles"  v ( w oo1 b l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" )))
( "warbling"  v ( w oo1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "ward"  v ( w oo1 d ) ((pos "H0%" "M6%" ) (subcat )))
( "warded"  v ( w oo1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "warden"  n ( w oo1 d n ) ((pos "K6%" ) ))
( "wardens"  n ( w oo1 d n z ) ((pos "Kj%" ) ))
( "warder"  n ( w oo1 d @ r ) ((pos "K6%" ) ))
( "warders"  n ( w oo1 d @ z ) ((pos "Kj%" ) ))
( "warding"  v ( w oo1 d i ng ) ((pos "Hb%" ) (subcat )))
( "wardress"  n ( w oo1 d r i s ) ((pos "K7$" ) ))
( "wardresses"  n ( w oo1 d r i s i z ) ((pos "Kj$" ) ))
( "wardrobe"  n ( w oo1 d r ou b ) ((pos "K6%" ) ))
( "wardrobes"  n ( w oo1 d r ou b z ) ((pos "Kj%" ) ))
( "wardroom"  n ( w oo1 d r u m ) ((pos "K6%" ) ))
( "wardrooms"  n ( w oo1 d r u m z ) ((pos "Kj%" ) ))
( "wards"  v ( w oo1 d z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "ware"  v ( w e@1 r ) ((pos "H2$" "K6%" ) (subcat )))
( "wared"  v ( w e@1 d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "warehouse"  n ( w e@1 h au s ) ((pos "K6%" ) ))
( "warehouses"  n ( w e@1 h au z i z ) ((pos "Kj%" ) ))
( "warehousing"  n ( w e@1 h au z i ng ) ((pos "L@%" ) ))
( "wares"  v ( w e@1 z ) ((pos "Ha$" "Kj%" ) (subcat )))
( "warfare"  n ( w oo1 f e@ r ) ((pos "L@%" ) ))
( "warhead"  n ( w oo1 h e d ) ((pos "K6%" ) ))
( "warheads"  n ( w oo1 h e d z ) ((pos "Kj%" ) ))
( "warhorse"  n ( w oo1 h oo s ) ((pos "K6%" ) ))
( "warhorses"  n ( w oo1 h oo s i z ) ((pos "Kj%" ) ))
( "warier"  j ( w e@1 r i@ r ) ((pos "Or%" ) ))
( "wariest"  j ( w e@1 r i i s t ) ((pos "Os%" ) ))
( "warily"  a ( w e@1 r i l ii ) ((pos "Pu%" ) ))
( "wariness"  n ( w e@1 r i n @ s ) ((pos "L@%" ) ))
( "waring"  v ( w e@1 r i ng ) ((pos "Hb$" ) (subcat )))
( "warlike"  j ( w oo1 l ai k ) ((pos "OA%" ) ))
( "warlord"  n ( w oo1 l oo d ) ((pos "K6%" ) ))
( "warlords"  n ( w oo1 l oo d z ) ((pos "Kj%" ) ))
( "warm"  v ( w oo1 m ) ((pos "J0%" "OC%" ) (subcat "2A" "2C" "6A" )))
( "warm-blooded"  j ( w oo1 m - b l uh1 d i d ) ((pos "OA%" ) ))
( "warm-hearted"  j ( w oo1 m - h aa1 t i d ) ((pos "OA%" ) ))
( "warmed"  v ( w oo1 m d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "warmer"  n ( w oo1 m @ r ) ((pos "K6%" "Or%" ) ))
( "warmers"  n ( w oo1 m @ z ) ((pos "Kj%" ) ))
( "warmest"  j ( w oo1 m i s t ) ((pos "Os%" ) ))
( "warming"  v ( w oo1 m i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "warming-pan"  n ( w oo1 m i ng - p a n ) ((pos "K6%" ) ))
( "warming-pans"  n ( w oo1 m i ng - p a n z ) ((pos "Kj%" ) ))
( "warmly"  a ( w oo1 m l ii ) ((pos "Pu%" ) ))
( "warmonger"  n ( w oo1 m uh ng g @ r ) ((pos "K6%" ) ))
( "warmongers"  n ( w oo1 m uh ng g @ z ) ((pos "Kj%" ) ))
( "warms"  v ( w oo1 m z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "warmth"  n ( w oo1 m th ) ((pos "L@%" ) ))
( "warn"  v ( w oo1 n ) ((pos "H0%" ) (subcat "6A" "11" "14" "15B" )))
( "warned"  v ( w oo1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "11" "14" "15B" )))
( "warning"  v ( w oo1 n i ng ) ((pos "Hb%" "M6%" "OA%" ) (subcat "6A" "11" "14" "15B" )))
( "warnings"  n ( w oo1 n i ng z ) ((pos "Mj%" ) ))
( "warns"  v ( w oo1 n z ) ((pos "Ha%" ) (subcat "6A" "11" "14" "15B" )))
( "warp"  v ( w oo1 p ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "warpaint"  n ( w oo1 p ei n t ) ((pos "L@%" ) ))
( "warpath"  n ( w oo1 p aa th ) ((pos "K6%" ) ))
( "warpaths"  n ( w oo1 p aa dh z ) ((pos "Kj%" ) ))
( "warped"  v ( w oo1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "warping"  v ( w oo1 p i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "warps"  v ( w oo1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "warrant"  v ( w o1 r @ n t ) ((pos "H0%" "M6%" ) (subcat "6A" "9" )))
( "warranted"  v ( w o1 r @ n t i d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "9" )))
( "warrantee"  n ( w o2 r @ n t ii1 ) ((pos "K6$" ) ))
( "warrantees"  n ( w o2 r @ n t ii1 z ) ((pos "Kj$" ) ))
( "warranties"  n ( w o1 r @ n t i z ) ((pos "Kj%" ) ))
( "warranting"  v ( w o1 r @ n t i ng ) ((pos "Hb%" ) (subcat "6A" "9" )))
( "warrantor"  n ( w o1 r @ n t oo r ) ((pos "K6$" ) ))
( "warrantors"  n ( w o1 r @ n t oo z ) ((pos "Kj$" ) ))
( "warrants"  v ( w o1 r @ n t s ) ((pos "Ha%" "Mj%" ) (subcat "6A" "9" )))
( "warranty"  n ( w o1 r @ n t ii ) ((pos "K8%" ) ))
( "warred"  v ( w oo1 d ) ((pos "Ic%" "Id%" ) ))
( "warren"  n ( w o1 r @ n ) ((pos "K6%" ) ))
( "warrens"  n ( w o1 r @ n z ) ((pos "Kj%" ) ))
( "warring"  v ( w oo1 r i ng ) ((pos "Ib%" ) ))
( "warrior"  n ( w o1 r i@ r ) ((pos "K6%" ) ))
( "warriors"  n ( w o1 r i@ z ) ((pos "Kj%" ) ))
( "wars"  v ( w oo1 z ) ((pos "Ia%" "Mj%" ) ))
( "warship"  n ( w oo1 sh i p ) ((pos "K6%" ) ))
( "warships"  n ( w oo1 sh i p s ) ((pos "Kj%" ) ))
( "wart"  n ( w oo1 t ) ((pos "K6%" ) ))
( "warthog"  n ( w oo1 t h o g ) ((pos "K6%" ) ))
( "warthogs"  n ( w oo1 t h o g z ) ((pos "Kj%" ) ))
( "wartime"  n ( w oo1 t ai m ) ((pos "L@%" ) ))
( "warts"  n ( w oo1 t s ) ((pos "Kj%" ) ))
( "wary"  j ( w e@1 r ii ) ((pos "OD%" ) ))
( "was"  v ( w o z ) ((pos "Gc*" "Ic%" ) (subcat )))
( "wash"  v ( w o1 sh ) ((pos "J1%" "M7%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "wash-"  j ( w o sh - ) ((pos "U-%" ) ))
( "wash-drawing"  n ( w o1 sh - d r oo i ng ) ((pos "K6$" ) ))
( "wash-drawings"  n ( w o1 sh - d r oo i ng z ) ((pos "Kj$" ) ))
( "wash-hand-basin"  n ( w o1 sh - h a n d - b ei s n ) ((pos "K6$" ) ))
( "wash-hand-basins"  n ( w o1 sh - h a n d - b ei s n z ) ((pos "Kj$" ) ))
( "wash-hand-stand"  n ( w o1 sh - h a n d - s t a n d ) ((pos "K6$" ) ))
( "wash-hand-stands"  n ( w o1 sh - h a n d - s t a n d z ) ((pos "Kj$" ) ))
( "wash-house"  n ( w o1 sh - h au s ) ((pos "K6%" ) ))
( "wash-houses"  n ( w o1 sh - h au z i z ) ((pos "Kj%" ) ))
( "wash-leather"  n ( w o1 sh - l e dh @ r ) ((pos "M6%" ) ))
( "wash-leathers"  n ( w o1 sh - l e dh @ z ) ((pos "Mj%" ) ))
( "washable"  j ( w o1 sh @ b l ) ((pos "OA%" ) ))
( "washbasin"  n ( w o1 sh b ei s n ) ((pos "K6%" ) ))
( "washbasins"  n ( w o1 sh b ei s n z ) ((pos "Kj%" ) ))
( "washboard"  n ( w o1 sh b oo d ) ((pos "K6%" ) ))
( "washboards"  n ( w o1 sh b oo d z ) ((pos "Kj%" ) ))
( "washbowl"  n ( w o1 sh b ou l ) ((pos "K6%" ) ))
( "washbowls"  n ( w o1 sh b ou l z ) ((pos "Kj%" ) ))
( "washcloth"  n ( w o1 sh k l o th ) ((pos "K6%" ) ))
( "washcloths"  n ( w o1 sh k l o th s ) ((pos "Kj%" ) ))
( "washday"  n ( w o1 sh d ei ) ((pos "K6%" ) ))
( "washdays"  n ( w o1 sh d ei z ) ((pos "Kj%" ) ))
( "washed"  v ( w o1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "washer"  n ( w o1 sh @ r ) ((pos "K6%" ) ))
( "washers"  n ( w o1 sh @ z ) ((pos "Kj%" ) ))
( "washerwoman"  n ( w o1 sh @ w u m @ n ) ((pos "Ki%" ) ))
( "washerwomen"  n ( w o1 sh @ w i m i n ) ((pos "Kj%" ) ))
( "washes"  v ( w o1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "washing"  v ( w o1 sh i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" "2C" "6A" "15A" "15B" )))
( "washing-day"  n ( w o1 sh i ng - d ei ) ((pos "K6%" ) ))
( "washing-days"  n ( w o1 sh i ng - d ei z ) ((pos "Kj%" ) ))
( "washing-machine"  n ( w o1 sh i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "washing-machines"  n ( w o1 sh i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "washing-up"  n ( w o2 sh i ng - uh1 p ) ((pos "M6%" ) ))
( "washing-ups"  n ( w o2 sh i ng - uh1 p s ) ((pos "Mj$" ) ))
( "washout"  n ( w o1 sh au t ) ((pos "K6%" ) ))
( "washouts"  n ( w o1 sh au t s ) ((pos "Kj$" ) ))
( "washroom"  n ( w o1 sh r uu m ) ((pos "K6%" ) ))
( "washrooms"  n ( w o1 sh r uu m z ) ((pos "Kj%" ) ))
( "washstand"  n ( w o1 sh - s t a n d ) ((pos "K6%" ) ))
( "washstands"  n ( w o1 sh - s t a n d z ) ((pos "Kj%" ) ))
( "washtub"  n ( w o1 sh t uh b ) ((pos "K6%" ) ))
( "washtubs"  n ( w o1 sh t uh b z ) ((pos "Kj%" ) ))
( "washy"  j ( w o1 sh ii ) ((pos "OA$" ) ))
( "wasn't"  v ( w o1 z n t ) ((pos "Gg%" ) ))
( "wasp"  n ( w o1 s p ) ((pos "K6%" ) ))
( "wasp-waisted"  j ( w o1 s p - w ei1 s t i d ) ((pos "OA%" ) ))
( "waspish"  j ( w o1 s p i sh ) ((pos "OA%" ) ))
( "wasps"  n ( w o1 s p s ) ((pos "Kj%" ) ))
( "wassail"  n ( w o1 s ei l ) ((pos "K6$" ) ))
( "wassails"  n ( w o1 s ei l z ) ((pos "Kj$" ) ))
( "wastage"  n ( w ei1 s t i jh ) ((pos "L@%" ) ))
( "waste"  v ( w ei1 s t ) ((pos "J2%" "M6%" "OA%" ) (subcat "2A" "2C" "6A" )))
( "waste-paper-basket"  n ( w ei1 s t - p ei1 p @ - b a s k i t ) ((pos "K6%" ) ))
( "waste-paper-baskets"  n ( w ei1 s t - p ei1 p @ - b a s k i t s ) ((pos "Kj%" ) ))
( "waste-pipe"  n ( w ei1 s t - p ai p ) ((pos "K6%" ) ))
( "waste-pipes"  n ( w ei1 s t - p ai p s ) ((pos "Kj%" ) ))
( "wastebasket"  n ( w ei1 s t b aa s k i t ) ((pos "K6%" ) ))
( "wastebaskets"  n ( w ei1 s t b aa s k i t s ) ((pos "Kj%" ) ))
( "wastebin"  n ( w ei1 s t b i n ) ((pos "K6%" ) ))
( "wastebins"  n ( w ei1 s t b i n z ) ((pos "Kj%" ) ))
( "wasted"  v ( w ei1 s t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "wasteful"  j ( w ei1 s t f @ l ) ((pos "OA%" ) ))
( "wastefully"  a ( w ei1 s t f @ l ii ) ((pos "Pu%" ) ))
( "wasteland"  n ( w ei1 s t l a n d ) ((pos "K6%" ) ))
( "wastelands"  n ( w ei1 s t l a n d z ) ((pos "Kj$" ) ))
( "waster"  n ( w ei1 s t @ r ) ((pos "K6$" ) ))
( "wasters"  n ( w ei1 s t @ z ) ((pos "Kj$" ) ))
( "wastes"  v ( w ei1 s t s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2C" "6A" )))
( "wasting"  v ( w ei1 s t i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "wastrel"  n ( w ei1 s t r @ l ) ((pos "K6$" ) ))
( "wastrels"  n ( w ei1 s t r @ l z ) ((pos "Kj$" ) ))
( "watch"  v ( w o1 ch ) ((pos "J1%" "M7%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "8" "10" "15A" "18A" )))
( "watch-chain"  n ( w o1 ch - ch ei n ) ((pos "K6%" ) ))
( "watch-chains"  n ( w o1 ch - ch ei n z ) ((pos "Kj%" ) ))
( "watch-glass"  n ( w o1 ch - g l aa s ) ((pos "K7%" ) ))
( "watch-glasses"  n ( w o1 ch - g l aa s i z ) ((pos "Kj$" ) ))
( "watch-guard"  n ( w o1 ch - g aa d ) ((pos "K6$" ) ))
( "watch-guards"  n ( w o1 ch - g aa d z ) ((pos "Kj$" ) ))
( "watch-key"  n ( w o1 ch - k ii ) ((pos "K6$" ) ))
( "watch-keys"  n ( w o1 ch - k ii z ) ((pos "Kj$" ) ))
( "watchdog"  n ( w o1 ch d o g ) ((pos "K6%" ) ))
( "watchdogs"  n ( w o1 ch d o g z ) ((pos "Kj%" ) ))
( "watched"  v ( w o1 ch t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "8" "10" "15A" "18A" )))
( "watcher"  n ( w o1 ch @ r ) ((pos "K6%" ) ))
( "watchers"  n ( w o1 ch @ z ) ((pos "Kj%" ) ))
( "watches"  v ( w o1 ch i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "8" "10" "15A" "18A" )))
( "watchful"  j ( w o1 ch f @ l ) ((pos "OA%" ) ))
( "watchfully"  a ( w o1 ch f @ l ii ) ((pos "Pu%" ) ))
( "watchfulness"  n ( w o1 ch f @ l n @ s ) ((pos "L@%" ) ))
( "watching"  v ( w o1 ch i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "3A" "4A" "6A" "8" "10" "15A" "18A" )))
( "watchmaker"  n ( w o1 ch m ei k @ r ) ((pos "K6%" ) ))
( "watchmakers"  n ( w o1 ch m ei k @ z ) ((pos "Kj%" ) ))
( "watchman"  n ( w o1 ch m @ n ) ((pos "Ki%" ) ))
( "watchmen"  n ( w o1 ch m @ n ) ((pos "Kj%" ) ))
( "watchtower"  n ( w o1 ch t au @ r ) ((pos "K6%" ) ))
( "watchtowers"  n ( w o1 ch t au @ z ) ((pos "Kj%" ) ))
( "watchword"  n ( w o1 ch w @@ d ) ((pos "K6%" ) ))
( "watchwords"  n ( w o1 ch w @@ d z ) ((pos "Kj%" ) ))
( "water"  v ( w oo1 t @ r ) ((pos "J0%" "M6*" ) (subcat "2A" "6A" )))
( "water-biscuit"  n ( w oo1 t @ - b i s k i t ) ((pos "K6%" ) ))
( "water-biscuits"  n ( w oo1 t @ - b i s k i t s ) ((pos "Kj%" ) ))
( "water-blister"  n ( w oo1 t @ - b l i s t @ r ) ((pos "K6$" ) ))
( "water-blisters"  n ( w oo1 t @ - b l i s t @ z ) ((pos "Kj$" ) ))
( "water-bottle"  n ( w oo1 t @ - b o t l ) ((pos "K6%" ) ))
( "water-bottles"  n ( w oo1 t @ - b o t l z ) ((pos "Kj%" ) ))
( "water-buffalo"  n ( w oo1 t @ - b uh f @ l ou ) ((pos "K6%" ) ))
( "water-buffalos"  n ( w oo1 t @ - b uh f @ l ou z ) ((pos "Kj$" ) ))
( "water-butt"  n ( w oo1 t @ - b uh t ) ((pos "K6%" ) ))
( "water-butts"  n ( w oo1 t @ - b uh t s ) ((pos "Kj%" ) ))
( "water-cart"  n ( w oo1 t @ - k aa t ) ((pos "K6%" ) ))
( "water-carts"  n ( w oo1 t @ - k aa t s ) ((pos "Kj%" ) ))
( "water-closet"  n ( w oo1 t @ - k l o z i t ) ((pos "K6%" ) ))
( "water-closets"  n ( w oo1 t @ - k l o z i t s ) ((pos "Kj%" ) ))
( "water-finder"  n ( w oo1 t @ - f ai n d @ r ) ((pos "K6$" ) ))
( "water-finders"  n ( w oo1 t @ - f ai n d @ z ) ((pos "Kj$" ) ))
( "water-glass"  n ( w oo1 t @ - g l aa s ) ((pos "L@$" ) ))
( "water-hole"  n ( w oo1 t @ - h ou l ) ((pos "K6%" ) ))
( "water-holes"  n ( w oo1 t @ - h ou l z ) ((pos "Kj%" ) ))
( "water-ice"  n ( w oo1 t @ r - ai s ) ((pos "M6%" ) ))
( "water-ices"  n ( w oo1 t @ r - ai s i z ) ((pos "Mj%" ) ))
( "water-jacket"  n ( w oo1 t @ - jh a k i t ) ((pos "K6$" ) ))
( "water-jackets"  n ( w oo1 t @ - jh a k i t s ) ((pos "Kj$" ) ))
( "water-level"  n ( w oo1 t @ - l e v @ l ) ((pos "K6%" ) ))
( "water-levels"  n ( w oo1 t @ - l e v @ l z ) ((pos "Kj%" ) ))
( "water-lilies"  n ( w oo1 t @ - l i l i z ) ((pos "Kj%" ) ))
( "water-lily"  n ( w oo1 t @ - l i l ii ) ((pos "K8%" ) ))
( "water-line"  n ( w oo1 t @ - l ai n ) ((pos "K6%" ) ))
( "water-lines"  n ( w oo1 t @ - l ai n z ) ((pos "Kj$" ) ))
( "water-main"  n ( w oo1 t @ - m ei n ) ((pos "K6%" ) ))
( "water-mains"  n ( w oo1 t @ - m ei n z ) ((pos "Kj%" ) ))
( "water-nymph"  n ( w oo1 t @ - n i m f ) ((pos "K6%" ) ))
( "water-nymphs"  n ( w oo1 t @ - n i m f s ) ((pos "Kj%" ) ))
( "water-polo"  n ( w oo1 t @ - p ou l ou ) ((pos "L@%" ) ))
( "water-power"  n ( w oo1 t @ - p au @ r ) ((pos "L@%" ) ))
( "water-rat"  n ( w oo1 t @ - r a t ) ((pos "K6%" ) ))
( "water-rate"  n ( w oo1 t @ - r ei t ) ((pos "K6%" ) ))
( "water-rates"  n ( w oo1 t @ - r ei t s ) ((pos "Kj%" ) ))
( "water-rats"  n ( w oo1 t @ - r a t s ) ((pos "Kj%" ) ))
( "water-skiing"  n ( w oo1 t @ - s k ii i ng ) ((pos "L@%" ) ))
( "water-skin"  n ( w oo1 t @ - s k i n ) ((pos "K6$" ) ))
( "water-skins"  n ( w oo1 t @ - s k i n z ) ((pos "Kj$" ) ))
( "water-softener"  n ( w oo1 t @ - s o f n @ r ) ((pos "M6%" ) ))
( "water-softeners"  n ( w oo1 t @ - s o f n @ z ) ((pos "Mj%" ) ))
( "water-spaniel"  n ( w oo1 t @ - s p a n i@ l ) ((pos "K6$" ) ))
( "water-spaniels"  n ( w oo1 t @ - s p a n i@ l z ) ((pos "Kj$" ) ))
( "water-supplies"  n ( w oo1 t @ - s @ p l ai z ) ((pos "Kj%" ) ))
( "water-supply"  n ( w oo1 t @ - s @ p l ai ) ((pos "K8%" ) ))
( "water-tower"  n ( w oo1 t @ - t au @ r ) ((pos "K6%" ) ))
( "water-towers"  n ( w oo1 t @ - t au @ z ) ((pos "Kj%" ) ))
( "water-vole"  n ( w oo1 t @ - v ou l ) ((pos "K6%" ) ))
( "water-voles"  n ( w oo1 t @ - v ou l z ) ((pos "Kj%" ) ))
( "water-waggon"  n ( w oo1 t @ - w a g @ n ) ((pos "K6%" ) ))
( "water-waggons"  n ( w oo1 t @ - w a g @ n z ) ((pos "Kj%" ) ))
( "water-wagon"  n ( w oo1 t @ - w a g @ n ) ((pos "K6$" ) ))
( "water-wagons"  n ( w oo1 t @ - w a g @ n z ) ((pos "Kj$" ) ))
( "water-wheel"  n ( w oo1 t @ - w ii l ) ((pos "K6%" ) ))
( "water-wheels"  n ( w oo1 t @ - w ii l z ) ((pos "Kj%" ) ))
( "water-wings"  n ( w oo1 t @ - w i ng z ) ((pos "Kj%" ) ))
( "water-worn"  j ( w oo1 t @ - w oo n ) ((pos "OA%" ) ))
( "waterbird"  n ( w oo1 t @ b @@ d ) ((pos "K6%" ) ))
( "waterbirds"  n ( w oo1 t @ b @@ d z ) ((pos "Kj%" ) ))
( "waterborne"  j ( w oo1 t @ b oo n ) ((pos "OA%" ) ))
( "watercannon"  n ( w oo1 t @ k a n @ n ) ((pos "K6%" ) ))
( "watercannons"  n ( w oo1 t @ k a n @ n z ) ((pos "Kj$" ) ))
( "waterchute"  n ( w oo1 t @ sh uu t ) ((pos "K6%" ) ))
( "waterchutes"  n ( w oo1 t @ sh uu t s ) ((pos "Kj%" ) ))
( "watercolour"  n ( w oo1 t @ k uh l @ r ) ((pos "M6%" ) ))
( "watercolourist"  n ( w oo1 t @ k uh l @ r i s t ) ((pos "K6%" ) ))
( "watercolourists"  n ( w oo1 t @ k uh l @ r i s t s ) ((pos "Kj%" ) ))
( "watercolours"  n ( w oo1 t @ k uh l @ z ) ((pos "Mj%" ) ))
( "watercourse"  n ( w oo1 t @ k oo s ) ((pos "K6%" ) ))
( "watercourses"  n ( w oo1 t @ k oo s i z ) ((pos "Kj%" ) ))
( "watercress"  n ( w oo1 t @ k r e s ) ((pos "L@%" ) ))
( "watered"  v ( w oo1 t @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "waterfall"  n ( w oo1 t @ f oo l ) ((pos "K6%" ) ))
( "waterfalls"  n ( w oo1 t @ f oo l z ) ((pos "Kj%" ) ))
( "waterfowl"  n ( w oo1 t @ f au l ) ((pos "K6%" ) ))
( "waterfowls"  n ( w oo1 t @ f au l z ) ((pos "Kj$" ) ))
( "waterfront"  n ( w oo1 t @ f r uh n t ) ((pos "K6%" ) ))
( "waterfronts"  n ( w oo1 t @ f r uh n t s ) ((pos "Kj$" ) ))
( "waterhen"  n ( w oo1 t @ h e n ) ((pos "K6%" ) ))
( "waterhens"  n ( w oo1 t @ h e n z ) ((pos "Kj%" ) ))
( "waterhyacinth"  n ( w oo1 t @ h ai @ s i n th ) ((pos "K6$" ) ))
( "waterhyacinths"  n ( w oo1 t @ h ai @ s i n th s ) ((pos "Kj$" ) ))
( "waterier"  j ( w oo1 t @ r i@ r ) ((pos "Or$" ) ))
( "wateriest"  j ( w oo1 t @ r i i s t ) ((pos "Os$" ) ))
( "watering"  v ( w oo1 t @ r i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "watering-can"  n ( w oo1 t @ r i ng - k a n ) ((pos "K6%" ) ))
( "watering-cans"  n ( w oo1 t @ r i ng - k a n z ) ((pos "Kj%" ) ))
( "watering-cart"  n ( w oo1 t @ r i ng - k aa t ) ((pos "K6%" ) ))
( "watering-carts"  n ( w oo1 t @ r i ng - k aa t s ) ((pos "Kj%" ) ))
( "waterless"  j ( w oo1 t @ l i s ) ((pos "OA%" ) ))
( "waterlogged"  j ( w oo1 t @ l o g d ) ((pos "OA%" ) ))
( "waterman"  n ( w oo1 t @ m @ n ) ((pos "Ki%" ) ))
( "watermark"  n ( w oo1 t @ m aa k ) ((pos "K6%" ) ))
( "watermarks"  n ( w oo1 t @ m aa k s ) ((pos "Kj%" ) ))
( "watermelon"  n ( w oo1 t @ m e l @ n ) ((pos "K6%" ) ))
( "watermelons"  n ( w oo1 t @ m e l @ n z ) ((pos "Kj%" ) ))
( "watermen"  n ( w oo1 t @ m @ n ) ((pos "Kj%" ) ))
( "watermill"  n ( w oo1 t @ m i l ) ((pos "K6%" ) ))
( "watermills"  n ( w oo1 t @ m i l z ) ((pos "Kj%" ) ))
( "waterproof"  v ( w oo1 t @ p r uu f ) ((pos "H0%" "K6%" "OA%" ) ))
( "waterproofed"  v ( w oo1 t @ p r uu f t ) ((pos "Hc%" "Hd%" ) ))
( "waterproofing"  v ( w oo1 t @ p r uu f i ng ) ((pos "Hb%" ) ))
( "waterproofs"  v ( w oo1 t @ p r uu f s ) ((pos "Ha%" "Kj%" ) ))
( "waters"  v ( w oo1 t @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "6A" )))
( "watershed"  n ( w oo1 t @ sh e d ) ((pos "K6%" ) ))
( "watersheds"  n ( w oo1 t @ sh e d z ) ((pos "Kj%" ) ))
( "waterside"  n ( w oo1 t @ s ai d ) ((pos "K6%" ) ))
( "watersides"  n ( w oo1 t @ s ai d z ) ((pos "Kj$" ) ))
( "waterspout"  n ( w oo1 t @ s p au t ) ((pos "K6%" ) ))
( "waterspouts"  n ( w oo1 t @ s p au t s ) ((pos "Kj%" ) ))
( "watertable"  n ( w oo1 t @ t ei b l ) ((pos "K6%" ) ))
( "watertables"  n ( w oo1 t @ t ei b l z ) ((pos "Kj$" ) ))
( "watertight"  j ( w oo1 t @ t ai t ) ((pos "OA%" ) ))
( "waterway"  n ( w oo1 t @ w ei ) ((pos "K6%" ) ))
( "waterways"  n ( w oo1 t @ w ei z ) ((pos "Kj%" ) ))
( "waterworks"  n ( w oo1 t @ w @@ k s ) ((pos "K9%" ) ))
( "watery"  j ( w oo1 t @ r ii ) ((pos "OD%" ) ))
( "watt"  n ( w o1 t ) ((pos "K6%" ) ))
( "wattage"  n ( w o1 t i jh ) ((pos "M6%" ) ))
( "wattages"  n ( w o1 t i jh i z ) ((pos "Mj$" ) ))
( "wattle"  n ( w o1 t l ) ((pos "M6%" ) ))
( "wattles"  n ( w o1 t l z ) ((pos "Mj$" ) ))
( "watts"  n ( w o1 t s ) ((pos "Kj%" ) ))
( "wave"  v ( w ei1 v ) ((pos "J2%" "K6%" ) (subcat "2A" "3A" "6A" "12A" "13A" "15A" "15B" )))
( "waved"  v ( w ei1 v d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "12A" "13A" "15A" "15B" )))
( "wavelength"  n ( w ei1 v l e ng th ) ((pos "K6%" ) ))
( "wavelengths"  n ( w ei1 v l e ng th s ) ((pos "Kj%" ) ))
( "waver"  v ( w ei1 v @ r ) ((pos "I0%" ) (subcat "2A" )))
( "wavered"  v ( w ei1 v @ d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "waverer"  n ( w ei1 v @ r @ r ) ((pos "K6$" ) ))
( "waverers"  n ( w ei1 v @ r @ z ) ((pos "Kj$" ) ))
( "wavering"  v ( w ei1 v @ r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "wavers"  v ( w ei1 v @ z ) ((pos "Ia%" ) (subcat "2A" )))
( "waves"  v ( w ei1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "3A" "6A" "12A" "13A" "15A" "15B" )))
( "wavier"  j ( w ei1 v i@ r ) ((pos "Or$" ) ))
( "waviest"  j ( w ei1 v i i s t ) ((pos "Os$" ) ))
( "waving"  v ( w ei1 v i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "12A" "13A" "15A" "15B" )))
( "wavy"  j ( w ei1 v ii ) ((pos "OD%" ) ))
( "wax"  v ( w a1 k s ) ((pos "J1%" "M7%" ) (subcat "2A" "2D" )))
( "wax-chandler"  n ( w a1 k s - ch aa n d l @ r ) ((pos "K6$" ) ))
( "wax-chandlers"  n ( w a1 k s - ch aa n d l @ z ) ((pos "Kj$" ) ))
( "wax-paper"  n ( w a1 k s - p ei p @ r ) ((pos "K6%" ) ))
( "wax-papers"  n ( w a1 k s - p ei p @ z ) ((pos "Kj$" ) ))
( "waxed"  v ( w a1 k s t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2D" )))
( "waxen"  j ( w a1 k s @ n ) ((pos "OA%" ) ))
( "waxes"  v ( w a1 k s i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2D" )))
( "waxier"  j ( w a1 k s i@ r ) ((pos "Or$" ) ))
( "waxiest"  j ( w a1 k s i i s t ) ((pos "Os$" ) ))
( "waxing"  v ( w a1 k s i ng ) ((pos "Jb%" ) (subcat "2A" "2D" )))
( "waxwork"  n ( w a1 k s w @@ k ) ((pos "K6%" ) ))
( "waxworks"  n ( w a1 k s w @@ k s ) ((pos "Kj%" ) ))
( "waxy"  j ( w a1 k s ii ) ((pos "OD%" ) ))
( "way"  n ( w ei1 ) ((pos "M6*" "Pu*" ) ))
( "way-out"  j ( w ei1 - au t ) ((pos "OA%" ) ))
( "waybill"  n ( w ei1 b i l ) ((pos "K6%" ) ))
( "waybills"  n ( w ei1 b i l z ) ((pos "Kj$" ) ))
( "wayfarer"  n ( w ei1 f e@ r @ r ) ((pos "K6%" ) ))
( "wayfarers"  n ( w ei1 f e@ r @ z ) ((pos "Kj%" ) ))
( "wayfaring"  j ( w ei1 f e@ r i ng ) ((pos "OA%" ) ))
( "waylaid"  v ( w ei2 l ei1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "waylay"  v ( w ei2 l ei1 ) ((pos "H5%" ) (subcat )))
( "waylaying"  v ( w ei2 l ei1 i ng ) ((pos "Hb%" ) (subcat )))
( "waylays"  v ( w ei2 l ei1 z ) ((pos "Ha%" ) (subcat )))
( "ways"  n ( w ei1 z ) ((pos "Mj%" ) ))
( "wayside"  n ( w ei1 s ai d ) ((pos "K6%" ) ))
( "waysides"  n ( w ei1 s ai d z ) ((pos "Kj%" ) ))
( "wayward"  j ( w ei1 w @ d ) ((pos "OA%" ) ))
( "we"  prp ( w ii ) ((pos "Qx*" ) ))
( "we'd"  v ( w ii d ) ((pos "Gf%" ) ))
( "we'll"  v ( w ii l ) ((pos "Gf%" ) ))
( "we're"  v ( w i@ r ) ((pos "Gf%" ) ))
( "we've"  v ( w ii v ) ((pos "Gf%" ) ))
( "weak"  j ( w ii1 k ) ((pos "OC%" ) ))
( "weak-kneed"  j ( w ii1 k - n ii1 d ) ((pos "OA%" ) ))
( "weaken"  v ( w ii1 k @ n ) ((pos "J0%" ) (subcat "2A" )))
( "weakened"  v ( w ii1 k @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "weakening"  v ( w ii1 k @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "weakens"  v ( w ii1 k @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "weaker"  j ( w ii1 k @ r ) ((pos "Or%" ) ))
( "weakest"  j ( w ii1 k i s t ) ((pos "Os%" ) ))
( "weaklier"  j ( w ii1 k l i@ r ) ((pos "Or$" ) ))
( "weakliest"  j ( w ii1 k l i i s t ) ((pos "Os$" ) ))
( "weakling"  n ( w ii1 k l i ng ) ((pos "K6%" ) ))
( "weaklings"  n ( w ii1 k l i ng z ) ((pos "Kj%" ) ))
( "weakly"  j ( w ii1 k l ii ) ((pos "OD%" "Pu%" ) ))
( "weakness"  n ( w ii1 k n @ s ) ((pos "M7%" ) ))
( "weaknesses"  n ( w ii1 k n @ s i z ) ((pos "Mj%" ) ))
( "weal"  n ( w ii1 l ) ((pos "M6$" ) ))
( "weald"  n ( w ii1 l d ) ((pos "K6$" ) ))
( "wealds"  n ( w ii1 l d z ) ((pos "Kj$" ) ))
( "weals"  n ( w ii1 l z ) ((pos "Mj$" ) ))
( "wealth"  n ( w e1 l th ) ((pos "L@%" ) ))
( "wealthier"  j ( w e1 l th i@ r ) ((pos "Or%" ) ))
( "wealthiest"  j ( w e1 l th i i s t ) ((pos "Os%" ) ))
( "wealthily"  a ( w e1 l th i l ii ) ((pos "Pu%" ) ))
( "wealthy"  j ( w e1 l th ii ) ((pos "OD%" ) ))
( "wean"  v ( w ii1 n ) ((pos "H0%" ) (subcat "6A" )))
( "weaned"  v ( w ii1 n d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "weaning"  v ( w ii1 n i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "weans"  v ( w ii1 n z ) ((pos "Ha%" ) (subcat "6A" )))
( "weapon"  n ( w e1 p @ n ) ((pos "K6%" ) ))
( "weaponless"  j ( w e1 p @ n l @ s ) ((pos "OA$" ) ))
( "weapons"  n ( w e1 p @ n z ) ((pos "Kj%" ) ))
( "wear"  v ( w e@1 r ) ((pos "J5%" "L@%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15A" "15B" )))
( "wearable"  j ( w e@1 r @ b l ) ((pos "OA%" ) ))
( "wearer"  n ( w e@1 r @ r ) ((pos "K6%" ) ))
( "wearers"  n ( w e@1 r @ z ) ((pos "Kj%" ) ))
( "wearied"  v ( w i@1 r i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" )))
( "wearier"  j ( w i@1 r i@ r ) ((pos "Or%" ) ))
( "wearies"  v ( w i@1 r i z ) ((pos "Ja%" ) (subcat "2A" "3A" "6A" )))
( "weariest"  j ( w i@1 r i i s t ) ((pos "Os%" ) ))
( "wearily"  a ( w i@1 r i l ii ) ((pos "Pu%" ) ))
( "weariness"  n ( w i@1 r i n @ s ) ((pos "L@%" ) ))
( "wearing"  v ( w e@1 r i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15A" "15B" )))
( "wearisome"  j ( w i@1 r i s @ m ) ((pos "OA%" ) ))
( "wears"  v ( w e@1 z ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15A" "15B" )))
( "weary"  v ( w i@1 r ii ) ((pos "J3%" "OD%" ) (subcat "2A" "3A" "6A" )))
( "wearying"  v ( w i@1 r i i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" )))
( "weasel"  n ( w ii1 z l ) ((pos "K6%" ) ))
( "weasels"  n ( w ii1 z l z ) ((pos "Kj%" ) ))
( "weather"  v ( w e1 dh @ r ) ((pos "J0%" "M6%" ) (subcat "2A" )))
( "weather-beaten"  j ( w e1 dh @ - b ii t n ) ((pos "OA%" ) ))
( "weather-bound"  j ( w e1 dh @ - b au n d ) ((pos "OA%" ) ))
( "weather-bureau"  n ( w e1 dh @ - b y u@ r ou ) ((pos "K6%" ) ))
( "weather-bureaus"  n ( w e1 dh @ - b y u@ r ou z ) ((pos "Kj$" ) ))
( "weather-chart"  n ( w e1 dh @ - ch aa t ) ((pos "K6%" ) ))
( "weather-charts"  n ( w e1 dh @ - ch aa t s ) ((pos "Kj%" ) ))
( "weather-glass"  n ( w e1 dh @ - g l aa s ) ((pos "K7$" ) ))
( "weather-glasses"  n ( w e1 dh @ - g l aa s i z ) ((pos "Kj$" ) ))
( "weather-map"  n ( w e1 dh @ - m a p ) ((pos "K6%" ) ))
( "weather-maps"  n ( w e1 dh @ - m a p s ) ((pos "Kj%" ) ))
( "weather-ship"  n ( w e1 dh @ - sh i p ) ((pos "K6$" ) ))
( "weather-ships"  n ( w e1 dh @ - sh i p s ) ((pos "Kj$" ) ))
( "weather-station"  n ( w e1 dh @ - s t ei sh @ n ) ((pos "K6%" ) ))
( "weather-stations"  n ( w e1 dh @ - s t ei sh @ n z ) ((pos "Kj%" ) ))
( "weather-vane"  n ( w e1 dh @ - v ei n ) ((pos "K6%" ) ))
( "weather-vanes"  n ( w e1 dh @ - v ei n z ) ((pos "Kj%" ) ))
( "weatherboarding"  n ( w e1 dh @ b oo d i ng ) ((pos "L@%" ) ))
( "weatherboards"  n ( w e1 dh @ b oo d z ) ((pos "Kj%" ) ))
( "weathercock"  n ( w e1 dh @ k o k ) ((pos "K6%" ) ))
( "weathercocks"  n ( w e1 dh @ k o k s ) ((pos "Kj%" ) ))
( "weathered"  v ( w e1 dh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "weathering"  v ( w e1 dh @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "weatherman"  n ( w e1 dh @ m a n ) ((pos "Ki%" ) ))
( "weathermen"  n ( w e1 dh @ m e n ) ((pos "Kj%" ) ))
( "weatherproof"  j ( w e1 dh @ p r uu f ) ((pos "OA%" ) ))
( "weathers"  v ( w e1 dh @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" )))
( "weave"  v ( w ii1 v ) ((pos "J5%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "weaver"  n ( w ii1 v @ r ) ((pos "K6%" ) ))
( "weaverbird"  n ( w ii1 v @ b @@ d ) ((pos "K6%" ) ))
( "weaverbirds"  n ( w ii1 v @ b @@ d z ) ((pos "Kj%" ) ))
( "weavers"  n ( w ii1 v @ z ) ((pos "Kj%" ) ))
( "weaves"  v ( w ii1 v z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "weaving"  v ( w ii1 v i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "web"  n ( w e1 b ) ((pos "K6%" ) ))
( "web-footed"  j ( w e1 b - f u1 t i d ) ((pos "OA%" ) ))
( "web-toed"  j ( w e1 b - t ou d ) ((pos "OA%" ) ))
( "webbed"  j ( w e1 b d ) ((pos "OA%" ) ))
( "webbing"  n ( w e1 b i ng ) ((pos "L@%" ) ))
( "webs"  n ( w e1 b z ) ((pos "Kj%" ) ))
( "wed"  v ( w e1 d ) ((pos "J4%" "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "wedded"  v ( w e1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "wedding"  v ( w e1 d i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "6A" )))
( "wedding-cake"  n ( w e1 d i ng - k ei k ) ((pos "M6%" ) ))
( "wedding-cakes"  n ( w e1 d i ng - k ei k s ) ((pos "Mj%" ) ))
( "wedding-ring"  n ( w e1 d i ng - r i ng ) ((pos "K6%" ) ))
( "wedding-rings"  n ( w e1 d i ng - r i ng z ) ((pos "Kj%" ) ))
( "weddings"  n ( w e1 d i ng z ) ((pos "Kj%" ) ))
( "wedge"  v ( w e1 jh ) ((pos "H2%" "K6%" ) (subcat "6A" "15A" )))
( "wedged"  v ( w e1 jh d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "wedges"  v ( w e1 jh i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "wedging"  v ( w e1 jh i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "wedlock"  n ( w e1 d l o k ) ((pos "L@%" ) ))
( "weds"  v ( w e1 d z ) ((pos "Ja%" ) (subcat "2A" "6A" )))
( "wee"  v ( w ii1 ) ((pos "I5%" "M6%" "OA%" ) ))
( "wee-wee"  n ( w ii1 - w ii ) ((pos "M6%" ) ))
( "wee-wees"  n ( w ii1 - w ii z ) ((pos "Mj%" ) ))
( "weed"  v ( w ii1 d ) ((pos "Ic%" "Id%" "J0%" "K6%" ) (subcat "2A" "6A" )))
( "weeded"  v ( w ii1 d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" )))
( "weedier"  j ( w ii1 d i@ r ) ((pos "Or%" ) ))
( "weediest"  j ( w ii1 d i i s t ) ((pos "Os%" ) ))
( "weeding"  v ( w ii1 d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" )))
( "weedkiller"  n ( w ii1 d k i l @ r ) ((pos "M6%" ) ))
( "weedkillers"  n ( w ii1 d k i l @ z ) ((pos "Mj%" ) ))
( "weeds"  v ( w ii1 d z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" )))
( "weedy"  j ( w ii1 d ii ) ((pos "OD%" ) ))
( "weeing"  v ( w ii1 i ng ) ((pos "Ib%" ) ))
( "week"  n ( w ii1 k ) ((pos "K6*" ) ))
( "weekday"  n ( w ii1 k d ei ) ((pos "K6%" ) ))
( "weekdays"  n ( w ii1 k d ei z ) ((pos "Kj%" ) ))
( "weekend"  v ( w ii1 k e1 n d ) ((pos "I0$" "K6%" ) ))
( "weekended"  v ( w ii1 k e1 n d i d ) ((pos "Ic$" "Id$" ) ))
( "weekender"  n ( w ii1 k e1 n d @ r ) ((pos "K6$" ) ))
( "weekenders"  n ( w ii1 k e1 n d @ z ) ((pos "Kj$" ) ))
( "weekending"  v ( w ii1 k e1 n d i ng ) ((pos "Ib$" ) ))
( "weekends"  v ( w ii1 k e1 n d z ) ((pos "Ia$" "Kj%" ) ))
( "weeklies"  n ( w ii1 k l i z ) ((pos "Kj%" ) ))
( "weekly"  n ( w ii1 k l ii ) ((pos "K8%" "OA%" "Pu%" ) ))
( "weeks"  n ( w ii1 k s ) ((pos "Kj%" ) ))
( "weenier"  j ( w ii1 n i@ r ) ((pos "Or%" ) ))
( "weeniest"  j ( w ii1 n i i s t ) ((pos "Os%" ) ))
( "weeny"  j ( w ii1 n ii ) ((pos "OD%" ) ))
( "weep"  v ( w ii1 p ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "3A" "4B" )))
( "weeping"  v ( w ii1 p i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "4B" )))
( "weeps"  v ( w ii1 p s ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "3A" "4B" )))
( "wees"  v ( w ii1 z ) ((pos "Ia%" "Mj$" ) ))
( "weevil"  n ( w ii1 v i l ) ((pos "K6%" ) ))
( "weevils"  n ( w ii1 v i l z ) ((pos "Kj%" ) ))
( "weft"  n ( w e1 f t ) ((pos "K6%" ) ))
( "wefts"  n ( w e1 f t s ) ((pos "Kj$" ) ))
( "weigh"  v ( w ei1 ) ((pos "J0%" ) (subcat "2B" "2C" "3A" "6A" "14" )))
( "weighbridge"  n ( w ei1 b r i jh ) ((pos "K6%" ) ))
( "weighbridges"  n ( w ei1 b r i jh i z ) ((pos "Kj%" ) ))
( "weighed"  v ( w ei1 d ) ((pos "Jc%" "Jd%" ) (subcat "2B" "2C" "3A" "6A" "14" )))
( "weighing"  v ( w ei1 i ng ) ((pos "Jb%" ) (subcat "2B" "2C" "3A" "6A" "14" )))
( "weighing-machine"  n ( w ei1 i ng - m @ sh ii n ) ((pos "K6%" ) ))
( "weighing-machines"  n ( w ei1 i ng - m @ sh ii n z ) ((pos "Kj%" ) ))
( "weighs"  v ( w ei1 z ) ((pos "Ja%" ) (subcat "2B" "2C" "3A" "6A" "14" )))
( "weight"  v ( w ei1 t ) ((pos "H0$" "M6%" ) (subcat )))
( "weighted"  v ( w ei1 t i d ) ((pos "Hc$" "Hd%" ) (subcat )))
( "weightier"  j ( w ei1 t i@ r ) ((pos "Or%" ) ))
( "weightiest"  j ( w ei1 t i i s t ) ((pos "Os%" ) ))
( "weightily"  a ( w ei1 t i l ii ) ((pos "Pu%" ) ))
( "weightiness"  n ( w ei1 t i n @ s ) ((pos "L@$" ) ))
( "weighting"  v ( w ei1 t i ng ) ((pos "Hb$" ) (subcat )))
( "weightless"  j ( w ei1 t l @ s ) ((pos "OA%" ) ))
( "weightlessness"  n ( w ei1 t l @ s n @ s ) ((pos "L@%" ) ))
( "weightlifting"  n ( w ei1 t l i f t i ng ) ((pos "L@%" ) ))
( "weights"  v ( w ei1 t s ) ((pos "Ha$" "Mj%" ) (subcat )))
( "weighty"  j ( w ei1 t ii ) ((pos "OD%" ) ))
( "weir"  n ( w i@1 r ) ((pos "K6%" ) ))
( "weird"  j ( w i@1 d ) ((pos "OC%" ) ))
( "weirder"  j ( w i@1 d @ r ) ((pos "Or%" ) ))
( "weirdest"  j ( w i@1 d i s t ) ((pos "Os%" ) ))
( "weirdie"  n ( w i@1 d ii ) ((pos "K6$" ) ))
( "weirdies"  n ( w i@1 d i z ) ((pos "Kj$" ) ))
( "weirdly"  a ( w i@1 d l ii ) ((pos "Pu%" ) ))
( "weirdness"  n ( w i@1 d n @ s ) ((pos "L@%" ) ))
( "weirs"  n ( w i@1 z ) ((pos "Kj%" ) ))
( "welcome"  v ( w e1 l k @ m ) ((pos "H2%" "K6%" "OA%" ) (subcat "6A" )))
( "welcomed"  v ( w e1 l k @ m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "welcomes"  v ( w e1 l k @ m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "welcoming"  v ( w e1 l k @ m i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "weld"  v ( w e1 l d ) ((pos "J0%" "K6$" ) (subcat "2A" "6A" "15A" )))
( "welded"  v ( w e1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "15A" )))
( "welder"  n ( w e1 l d @ r ) ((pos "K6%" ) ))
( "welders"  n ( w e1 l d @ z ) ((pos "Kj%" ) ))
( "welding"  v ( w e1 l d i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "15A" )))
( "welds"  v ( w e1 l d z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "6A" "15A" )))
( "welfare"  n ( w e1 l f e@ r ) ((pos "L@%" ) ))
( "welkin"  n ( w e1 l k i n ) ((pos "Ki$" ) ))
( "well"  v ( w e1 l ) ((pos "I0%" "K6%" "Op%" "Pu*" "W-%" ) (subcat )))
( "well-"  j ( w e l - ) ((pos "U-%" ) ))
( "well-adjusted"  j ( w e2 l - @ jh uh1 s t i d ) ((pos "OA%" ) ))
( "well-advised"  j ( w e2 l - @ d v ai1 z d ) ((pos "OA%" ) ))
( "well-appointed"  j ( w e2 l - @ p oi1 n t i dh ) ((pos "OA%" ) ))
( "well-balanced"  j ( w e1 l - b a1 l @ n s t ) ((pos "OA%" ) ))
( "well-behaved"  j ( w e2 l - b i h ei1 v d ) ((pos "OA%" ) ))
( "well-being"  n ( w e1 l - b ii1 i ng ) ((pos "L@%" ) ))
( "well-born"  j ( w e1 l - b oo n ) ((pos "OA%" ) ))
( "well-bred"  j ( w e1 l - b r e d ) ((pos "OA%" ) ))
( "well-conducted"  j ( w e2 l - k @ n d uh1 k t i d ) ((pos "OA%" ) ))
( "well-connected"  j ( w e2 l - k @ n e1 k t i d ) ((pos "OA%" ) ))
( "well-disposed"  j ( w e2 l - d i s p ou1 z d ) ((pos "OA%" ) ))
( "well-doer"  n ( w e1 l - d uu @ r ) ((pos "K6%" ) ))
( "well-doers"  n ( w e1 l - d uu @ z ) ((pos "Kj%" ) ))
( "well-doing"  n ( w e1 l - d uu i ng ) ((pos "L@%" ) ))
( "well-favoured"  j ( w e1 l - f ei1 v @ d ) ((pos "OA%" ) ))
( "well-found"  j ( w e1 l - f au n d ) ((pos "OA$" ) ))
( "well-founded"  j ( w e1 l - f au1 n d i d ) ((pos "OA%" ) ))
( "well-groomed"  j ( w e1 l - g r uu1 m d ) ((pos "OA%" ) ))
( "well-grounded"  j ( w e1 l - g r au1 n d i d ) ((pos "OA%" ) ))
( "well-heeled"  j ( w e1 l - h i@1 l d ) ((pos "OA%" ) ))
( "well-informed"  j ( w e2 l - i n f oo1 m d ) ((pos "OA%" ) ))
( "well-intentioned"  j ( w e2 l - i n t e1 n sh @ n d ) ((pos "OA%" ) ))
( "well-knit"  j ( w e1 l - n i t ) ((pos "OA%" ) ))
( "well-known"  j ( w e1 l - n ou n ) ((pos "OA%" ) ))
( "well-lined"  j ( w e1 l - l ai1 n d ) ((pos "OA%" ) ))
( "well-marked"  j ( w e1 l - m aa k t ) ((pos "OA%" ) ))
( "well-meaning"  j ( w e1 l - m ii n i ng ) ((pos "OA%" ) ))
( "well-meant"  j ( w e1 l - m e n t ) ((pos "OA%" ) ))
( "well-nigh"  a ( w e1 l - n ai ) ((pos "Pu%" ) ))
( "well-read"  j ( w e1 l - r e d ) ((pos "OA%" ) ))
( "well-rounded"  j ( w e1 l - r au1 n d i d ) ((pos "OA%" ) ))
( "well-set"  j ( w e1 l - s e t ) ((pos "OA%" ) ))
( "well-shaven"  j ( w e1 l - sh ei v n ) ((pos "OA%" ) ))
( "well-spoken"  j ( w e1 l - s p ou k @ n ) ((pos "OA%" ) ))
( "well-timed"  j ( w e1 l - t ai1 m d ) ((pos "OA%" ) ))
( "well-to-do"  j ( w e2 l - t @ - d uu1 ) ((pos "OA%" ) ))
( "well-tried"  j ( w e1 l - t r ai1 d ) ((pos "OA%" ) ))
( "well-turned"  j ( w e1 l - t @@1 n d ) ((pos "OA%" ) ))
( "well-water"  n ( w e1 l - w oo t @ r ) ((pos "L@%" ) ))
( "well-wisher"  n ( w e1 l - w i sh @ r ) ((pos "K6%" ) ))
( "well-wishers"  n ( w e1 l - w i sh @ z ) ((pos "Kj%" ) ))
( "well-worn"  j ( w e1 l - w oo n ) ((pos "OA%" ) ))
( "welled"  v ( w e1 l d ) ((pos "Ic%" "Id%" ) (subcat )))
( "wellhead"  n ( w e1 l h e d ) ((pos "K6%" ) ))
( "wellheads"  n ( w e1 l h e d z ) ((pos "Kj$" ) ))
( "welling"  v ( w e1 l i ng ) ((pos "Ib%" ) (subcat )))
( "wellington"  n ( w e1 l i ng t @ n ) ((pos "K6%" ) ))
( "wellingtons"  n ( w e1 l i ng t @ n z ) ((pos "Kj%" ) ))
( "wells"  v ( w e1 l z ) ((pos "Ia%" "Kj%" ) (subcat )))
( "welsh"  v ( w e1 l sh ) ((pos "I1$" ) (subcat )))
( "welshed"  v ( w e1 l sh t ) ((pos "Ic$" "Id$" ) (subcat )))
( "welsher"  n ( w e1 l sh @ r ) ((pos "K6$" ) ))
( "welshers"  n ( w e1 l sh @ z ) ((pos "Kj$" ) ))
( "welshes"  v ( w e1 l sh i z ) ((pos "Ia$" ) (subcat )))
( "welshing"  v ( w e1 l sh i ng ) ((pos "Ib$" ) (subcat )))
( "welt"  n ( w e1 l t ) ((pos "K6$" ) ))
( "welter"  v ( w e1 l t @ r ) ((pos "I0$" "Ki$" "OA$" ) (subcat )))
( "weltered"  v ( w e1 l t @ d ) ((pos "Ic$" "Id$" ) (subcat )))
( "weltering"  v ( w e1 l t @ r i ng ) ((pos "Ib$" ) (subcat )))
( "welters"  v ( w e1 l t @ z ) ((pos "Ia$" ) (subcat )))
( "welterweight"  n ( w e1 l t @ w ei t ) ((pos "K6%" ) ))
( "welterweights"  n ( w e1 l t @ w ei t s ) ((pos "Kj%" ) ))
( "welts"  n ( w e1 l t s ) ((pos "Kj$" ) ))
( "wen"  n ( w e1 n ) ((pos "K6$" ) ))
( "wench"  v ( w e1 n ch ) ((pos "I1$" "K7%" ) (subcat )))
( "wenched"  v ( w e1 n ch t ) ((pos "Ic$" "Id$" ) (subcat )))
( "wenches"  v ( w e1 n ch i z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "wenching"  v ( w e1 n ch i ng ) ((pos "Ib$" ) (subcat )))
( "wend"  v ( w e1 n d ) ((pos "H0%" ) ))
( "wended"  v ( w e1 n d i d ) ((pos "Hc%" "Hd%" ) ))
( "wending"  v ( w e1 n d i ng ) ((pos "Hb%" ) ))
( "wends"  v ( w e1 n d z ) ((pos "Ha%" ) ))
( "wens"  n ( w e1 n z ) ((pos "Kj$" ) ))
( "went"  v ( w e1 n t ) ((pos "Ic*" ) (subcat "2A" "2B" "2C" "2D" "2E" "3A" "4A" "6A" )))
( "wept"  v ( w e1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "3A" "4B" )))
( "were"  v ( w @@ r ) ((pos "Gc*" "Ic%" ) (subcat )))
( "weren't"  v ( w @@ n t ) ((pos "Gg%" ) ))
( "werewolf"  n ( w i@1 w u l f ) ((pos "Ki%" ) ))
( "werewolves"  n ( w i@1 w u l v z ) ((pos "Kj%" ) ))
( "wert"  v ( w @@ t ) ((pos "Ge$" ) ))
( "west"  n ( w e1 s t ) ((pos "L@%" "Pu%" ) ))
( "west-country"  j ( w e1 s t - k uh n t r ii ) ((pos "OA%" ) ))
( "west-end"  j ( w e1 s t - e n d ) ((pos "OA%" ) ))
( "westerly"  j ( w e1 s t @ l ii ) ((pos "Oq%" "Pu%" ) ))
( "western"  n ( w e1 s t @ n ) ((pos "K6%" "OA%" ) ))
( "westerner"  n ( w e1 s t @ n @ r ) ((pos "K6%" ) ))
( "westerners"  n ( w e1 s t @ n @ z ) ((pos "Kj%" ) ))
( "westernization"  n ( w e2 s t @ n ai z ei1 sh @ n ) ((pos "L@%" ) ))
( "westernize"  v ( w e1 s t @ n ai z ) ((pos "H2%" ) (subcat )))
( "westernized"  v ( w e1 s t @ n ai z d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "westernizes"  v ( w e1 s t @ n ai z i z ) ((pos "Ha%" ) (subcat )))
( "westernizing"  v ( w e1 s t @ n ai z i ng ) ((pos "Hb%" ) (subcat )))
( "westernmost"  j ( w e1 s t @ n m ou s t ) ((pos "OA%" ) ))
( "westerns"  n ( w e1 s t @ n z ) ((pos "Kj%" ) ))
( "westward"  j ( w e1 s t w @ d ) ((pos "OA%" "Pu%" ) ))
( "westwards"  a ( w e1 s t w @ d z ) ((pos "Pu%" ) ))
( "wet"  v ( w e1 t ) ((pos "H4%" "Hc%" "Hd%" "L@%" "OE%" ) (subcat )))
( "wet-nurse"  n ( w e1 t - n @@ s ) ((pos "K6%" ) ))
( "wet-nurses"  n ( w e1 t - n @@ s i z ) ((pos "Kj%" ) ))
( "wether"  n ( w e1 dh @ r ) ((pos "K6$" ) ))
( "wethers"  n ( w e1 dh @ z ) ((pos "Kj$" ) ))
( "wets"  v ( w e1 t s ) ((pos "Ha%" ) (subcat )))
( "wetted"  v ( w e1 t i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wetter"  j ( w e1 t @ r ) ((pos "Or%" ) ))
( "wettest"  j ( w e1 t i s t ) ((pos "Os%" ) ))
( "wetting"  v ( w e1 t i ng ) ((pos "Hb%" "K6%" ) (subcat )))
( "wettings"  n ( w e1 t i ng z ) ((pos "Kj%" ) ))
( "whack"  v ( w a1 k ) ((pos "H0%" "K6%" ) (subcat )))
( "whacked"  v ( w a1 k t ) ((pos "Hc%" "Hd%" "OA%" ) (subcat )))
( "whacker"  n ( w a1 k @ r ) ((pos "K6$" ) ))
( "whackers"  n ( w a1 k @ z ) ((pos "Kj$" ) ))
( "whacking"  v ( w a1 k i ng ) ((pos "Hb%" "K6%" "OA%" "Pu%" ) (subcat )))
( "whackings"  n ( w a1 k i ng z ) ((pos "Kj%" ) ))
( "whacks"  v ( w a1 k s ) ((pos "Ha%" "Kj%" ) (subcat )))
( "whale"  v ( w ei1 l ) ((pos "I2$" "K6%" ) (subcat )))
( "whalebone"  n ( w ei1 l b ou n ) ((pos "L@%" ) ))
( "whaled"  v ( w ei1 l d ) ((pos "Ic$" "Id$" ) (subcat )))
( "whaler"  n ( w ei1 l @ r ) ((pos "K6%" ) ))
( "whalers"  n ( w ei1 l @ z ) ((pos "Kj%" ) ))
( "whales"  v ( w ei1 l z ) ((pos "Ia$" "Kj$" ) (subcat )))
( "whaling"  v ( w ei1 l i ng ) ((pos "Ib%" ) (subcat )))
( "whaling-gun"  n ( w ei1 l i ng - g uh n ) ((pos "K6%" ) ))
( "whaling-guns"  n ( w ei1 l i ng - g uh n z ) ((pos "Kj%" ) ))
( "whang"  v ( w a1 ng ) ((pos "H0$" "K6$" "Pu$" ) ))
( "whanged"  v ( w a1 ng d ) ((pos "Hc$" "Hd$" ) ))
( "whanging"  v ( w a1 ng i ng ) ((pos "Hb$" ) ))
( "whangs"  v ( w a1 ng z ) ((pos "Ha$" "Kj$" ) ))
( "wharf"  n ( w oo1 f ) ((pos "K6%" ) ))
( "wharfage"  n ( w oo1 f i jh ) ((pos "L@%" ) ))
( "wharfs"  n ( w oo1 f s ) ((pos "Kj%" ) ))
( "wharves"  n ( w oo1 v z ) ((pos "Kj%" ) ))
( "what"  j ( w o t ) ((pos "OA*" "Qy*" "Qz*" ) ))
( "what'll"  v ( w o1 t l ) ((pos "Gf%" ) ))
( "what's"  v ( w o t s ) ((pos "Gf%" ) ))
( "what-for"  n ( w o1 t - f oo1 r ) ((pos "L@%" ) ))
( "whate'er"  j ( w o1 t e@1 r ) ((pos "OA%" ) ))
( "whatever"  j ( w o1 t e1 v @ r ) ((pos "OA%" "Qx%" ) ))
( "whatnot"  n ( w o1 t n o t ) ((pos "L@%" ) ))
( "whatsoe'er"  j ( w o2 t s ou e@1 r ) ((pos "OA%" ) ))
( "whatsoever"  j ( w o2 t s ou e1 v @ r ) ((pos "OA%" ) ))
( "wheat"  n ( w ii1 t ) ((pos "L@%" ) ))
( "wheaten"  j ( w ii1 t n ) ((pos "OA%" ) ))
( "wheedle"  v ( w ii1 d l ) ((pos "H2%" ) (subcat "6A" )))
( "wheedled"  v ( w ii1 d l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "wheedles"  v ( w ii1 d l z ) ((pos "Ha%" ) (subcat "6A" )))
( "wheedling"  v ( w ii1 d l i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "wheel"  v ( w ii1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" "15A" )))
( "wheelbarrow"  n ( w ii1 l b a r ou ) ((pos "K6%" ) ))
( "wheelbarrows"  n ( w ii1 l b a r ou z ) ((pos "Kj%" ) ))
( "wheelbase"  n ( w ii1 l b ei s ) ((pos "K6%" ) ))
( "wheelbases"  n ( w ii1 l b ei s i z ) ((pos "Kj%" ) ))
( "wheelchair"  n ( w ii1 l ch e@ r ) ((pos "K6%" ) ))
( "wheelchairs"  n ( w ii1 l ch e@ z ) ((pos "Kj%" ) ))
( "wheeled"  v ( w ii1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "wheelhouse"  n ( w ii1 l h au s ) ((pos "Ki%" ) ))
( "wheelhouses"  n ( w ii1 l h au z i z ) ((pos "Kj%" ) ))
( "wheeling"  v ( w ii1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15A" )))
( "wheels"  v ( w ii1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15A" )))
( "wheelwright"  n ( w ii1 l r ai t ) ((pos "K6%" ) ))
( "wheelwrights"  n ( w ii1 l r ai t s ) ((pos "Kj%" ) ))
( "wheeze"  v ( w ii1 z ) ((pos "J2%" "K6%" ) (subcat "2A" "2B" "2C" )))
( "wheezed"  v ( w ii1 z d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" )))
( "wheezes"  v ( w ii1 z i z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2B" "2C" )))
( "wheezier"  j ( w ii1 z i@ r ) ((pos "Or$" ) ))
( "wheeziest"  j ( w ii1 z i i s t ) ((pos "Os$" ) ))
( "wheezily"  a ( w ii1 z i l ii ) ((pos "Pu%" ) ))
( "wheeziness"  n ( w ii1 z i n i s ) ((pos "L@%" ) ))
( "wheezing"  v ( w ii1 z i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" )))
( "wheezy"  j ( w ii1 z ii ) ((pos "OD%" ) ))
( "whelk"  n ( w e1 l k ) ((pos "K6%" ) ))
( "whelks"  n ( w e1 l k s ) ((pos "Kj%" ) ))
( "whelp"  v ( w e1 l p ) ((pos "I0$" "K6$" ) ))
( "whelped"  v ( w e1 l p t ) ((pos "Ic$" "Id$" ) ))
( "whelping"  v ( w e1 l p i ng ) ((pos "Ib$" ) ))
( "whelps"  v ( w e1 l p s ) ((pos "Ia$" "Kj$" ) ))
( "when"  a ( w e n ) ((pos "Pv*" "Pw*" "V-*" ) ))
( "when's"  v ( w e n z ) ((pos "Gh%" ) ))
( "whence"  a ( w e1 n s ) ((pos "Pu%" ) ))
( "whencesoever"  j ( w e2 n s ou e1 v @ r ) ((pos "OA$" "V-$" ) ))
( "whenever"  a ( w e1 n e1 v @ r ) ((pos "Pu%" "V-%" ) ))
( "where"  a ( w e@1 r ) ((pos "Pv*" "Pw*" ) ))
( "where's"  v ( w e@ z ) ((pos "Gh%" ) ))
( "whereabouts"  n ( w e@1 r @ b au t s ) ((pos "Kj%" "Pu%" ) ))
( "whereas"  cc ( w e@2 r a1 z ) ((pos "V-%" ) ))
( "whereat"  a ( w e@2 r a1 t ) ((pos "Pu$" ) ))
( "whereby"  a ( w e@2 b ai1 ) ((pos "Pu%" ) ))
( "wherefore"  a ( w e@1 f oo r ) ((pos "Pu%" ) ))
( "wherefores"  n ( w e@1 f oo z ) ((pos "Kj$" ) ))
( "wherein"  a ( w e@2 r i1 n ) ((pos "Pu$" ) ))
( "whereof"  a ( w e@2 r o1 v ) ((pos "Pu$" ) ))
( "whereon"  a ( w e@2 r o1 n ) ((pos "Pu$" ) ))
( "wheresoever"  a ( w e@2 s ou e1 v @ r ) ((pos "Pu%" ) ))
( "whereto"  a ( w e@2 t uu1 ) ((pos "Pu$" ) ))
( "whereunto"  a ( w e@2 r uh1 n t uu ) ((pos "Pu$" ) ))
( "whereupon"  a ( w e@2 r uh p o1 n ) ((pos "Pu%" ) ))
( "wherever"  a ( w e@2 r e1 v @ r ) ((pos "Pu%" ) ))
( "wherewith"  a ( w e@2 w i1 dh ) ((pos "Pu$" ) ))
( "wherewithal"  n ( w e@1 w i dh oo l ) ((pos "L@%" "Pu%" ) ))
( "wherries"  n ( w e1 r i z ) ((pos "Kj$" ) ))
( "wherry"  n ( w e1 r ii ) ((pos "K8$" ) ))
( "whet"  v ( w e1 t ) ((pos "H4$" ) (subcat )))
( "whether"  cc ( w e1 dh @ r ) ((pos "V-*" ) ))
( "whets"  v ( w e1 t s ) ((pos "Ha$" ) (subcat )))
( "whetstone"  n ( w e1 t s t ou n ) ((pos "K6%" ) ))
( "whetstones"  n ( w e1 t s t ou n z ) ((pos "Kj%" ) ))
( "whetted"  v ( w e1 t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "whetting"  v ( w e1 t i ng ) ((pos "Hb$" ) (subcat )))
( "whew"  uh ( f y uu1 ) ((pos "W-$" ) ))
( "whey"  n ( w ei1 ) ((pos "L@$" ) ))
( "which"  j ( w i ch ) ((pos "OA*" "Qy*" "Qz*" ) ))
( "whichever"  j ( w i1 ch e1 v @ r ) ((pos "OA%" "Qx%" ) ))
( "whichsoever"  j ( w i2 ch - s ou e1 v @ r ) ((pos "OA$" "Qx$" ) ))
( "whiff"  n ( w i1 f ) ((pos "K6%" ) ))
( "whiffs"  n ( w i1 f s ) ((pos "Kj%" ) ))
( "while"  v ( w ai l ) ((pos "H2%" "Ki%" "V-*" ) (subcat )))
( "whiled"  v ( w ai1 l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "whiles"  v ( w ai1 l z ) ((pos "Ha%" ) (subcat )))
( "whiling"  v ( w ai1 l i ng ) ((pos "Hb%" ) (subcat )))
( "whilst"  cc ( w ai l s t ) ((pos "V-%" ) ))
( "whim"  n ( w i1 m ) ((pos "K6%" ) ))
( "whimper"  v ( w i1 m p @ r ) ((pos "J0%" "K6%" ) (subcat "2A" )))
( "whimpered"  v ( w i1 m p @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "whimpering"  v ( w i1 m p @ r i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "whimpers"  v ( w i1 m p @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "whims"  n ( w i1 m z ) ((pos "Kj%" ) ))
( "whimsey"  n ( w i1 m z ii ) ((pos "M6$" ) ))
( "whimseys"  n ( w i1 m z i z ) ((pos "Mj$" ) ))
( "whimsical"  j ( w i1 m z i k l ) ((pos "OA%" ) ))
( "whimsicalities"  n ( w i2 m z i k a1 l @ t i z ) ((pos "Mj%" ) ))
( "whimsicality"  n ( w i2 m z i k a1 l @ t ii ) ((pos "M8%" ) ))
( "whimsically"  a ( w i1 m z i k l ii ) ((pos "Pu%" ) ))
( "whimsies"  n ( w i1 m z i z ) ((pos "Mj%" ) ))
( "whimsy"  n ( w i1 m z ii ) ((pos "M8%" ) ))
( "whin"  n ( w i1 n ) ((pos "L@$" ) ))
( "whine"  v ( w ai1 n ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "4A" "6A" )))
( "whined"  v ( w ai1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "4A" "6A" )))
( "whiner"  n ( w ai1 n @ r ) ((pos "K6$" ) ))
( "whiners"  n ( w ai1 n @ z ) ((pos "Kj$" ) ))
( "whines"  v ( w ai1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "4A" "6A" )))
( "whining"  v ( w ai1 n i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "4A" "6A" )))
( "whinnied"  v ( w i1 n i d ) ((pos "Ic%" "Id%" ) ))
( "whinnies"  v ( w i1 n i z ) ((pos "Ia%" "Kj%" ) ))
( "whinny"  v ( w i1 n ii ) ((pos "I3%" "K8%" ) ))
( "whinnying"  v ( w i1 n i i ng ) ((pos "Ib%" ) ))
( "whip"  v ( w i1 p ) ((pos "J4%" "K6%" ) (subcat "2C" "6A" "15A" )))
( "whip-round"  n ( w i1 p - r au n d ) ((pos "K6%" ) ))
( "whip-rounds"  n ( w i1 p - r au n d z ) ((pos "Kj$" ) ))
( "whipcord"  n ( w i1 p k oo d ) ((pos "L@%" ) ))
( "whipped"  v ( w i1 p t ) ((pos "Jc%" "Jd%" ) (subcat "2C" "6A" "15A" )))
( "whipper-in"  n ( w i2 p @ r - i1 n ) ((pos "Ki$" ) ))
( "whippers-in"  n ( w i2 p @ z - i1 n ) ((pos "Kj$" ) ))
( "whippersnapper"  n ( w i1 p @ s n a p @ r ) ((pos "K6%" ) ))
( "whippersnappers"  n ( w i1 p @ s n a p @ z ) ((pos "Kj%" ) ))
( "whippet"  n ( w i1 p i t ) ((pos "K6%" ) ))
( "whippets"  n ( w i1 p i t s ) ((pos "Kj%" ) ))
( "whippier"  j ( w i1 p i@ r ) ((pos "Or$" ) ))
( "whippiest"  j ( w i1 p i i s t ) ((pos "Os$" ) ))
( "whipping"  v ( w i1 p i ng ) ((pos "Jb%" "M6%" ) (subcat "2C" "6A" "15A" )))
( "whipping-boy"  n ( w i1 p i ng - b oi ) ((pos "K6%" ) ))
( "whipping-boys"  n ( w i1 p i ng - b oi z ) ((pos "Kj%" ) ))
( "whipping-post"  n ( w i1 p i ng - p ou s t ) ((pos "K6%" ) ))
( "whipping-posts"  n ( w i1 p i ng - p ou s t s ) ((pos "Kj%" ) ))
( "whipping-top"  n ( w i1 p i ng - t o p ) ((pos "K6%" ) ))
( "whipping-tops"  n ( w i1 p i ng - t o p s ) ((pos "Kj%" ) ))
( "whippings"  n ( w i1 p i ng z ) ((pos "Mj%" ) ))
( "whippoorwill"  n ( w i1 p @ w i l ) ((pos "K6$" ) ))
( "whippoorwills"  n ( w i1 p @ w i l z ) ((pos "Kj$" ) ))
( "whippy"  j ( w i1 p ii ) ((pos "OD%" ) ))
( "whips"  v ( w i1 p s ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" "15A" )))
( "whir"  v ( w @@1 r ) ((pos "I4$" "Ki$" ) (subcat "2A" )))
( "whirl"  v ( w @@1 l ) ((pos "J0%" "Ki%" ) (subcat "2A" "2C" "15A" )))
( "whirled"  v ( w @@1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "15A" )))
( "whirligig"  n ( w @@1 l i g i g ) ((pos "K6%" ) ))
( "whirligigs"  n ( w @@1 l i g i g z ) ((pos "Kj%" ) ))
( "whirling"  v ( w @@1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "15A" )))
( "whirlpool"  n ( w @@1 l p uu l ) ((pos "K6%" ) ))
( "whirlpools"  n ( w @@1 l p uu l z ) ((pos "Kj%" ) ))
( "whirls"  v ( w @@1 l z ) ((pos "Ja%" ) (subcat "2A" "2C" "15A" )))
( "whirlwind"  n ( w @@1 l w i n d ) ((pos "K6%" ) ))
( "whirlwinds"  n ( w @@1 l w i n d z ) ((pos "Kj%" ) ))
( "whirr"  v ( w @@1 r ) ((pos "I0%" "Ki%" ) (subcat "2A" )))
( "whirred"  v ( w @@1 d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "whirring"  v ( w @@1 r i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "whirrs"  v ( w @@1 z ) ((pos "Ia%" ) (subcat "2A" )))
( "whirs"  v ( w @@1 z ) ((pos "Ia$" ) (subcat "2A" )))
( "whisk"  v ( w i1 s k ) ((pos "J0%" "K6%" ) (subcat "6A" )))
( "whisked"  v ( w i1 s k t ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "whisker"  n ( w i1 s k @ r ) ((pos "K6%" ) ))
( "whiskered"  j ( w i1 s k @ d ) ((pos "OA%" ) ))
( "whiskers"  n ( w i1 s k @ z ) ((pos "Kj%" ) ))
( "whiskey"  n ( w i1 s k ii ) ((pos "M6%" ) ))
( "whiskeys"  n ( w i1 s k i z ) ((pos "Mj%" ) ))
( "whiskies"  n ( w i1 s k i z ) ((pos "Mj%" ) ))
( "whisking"  v ( w i1 s k i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "whisks"  v ( w i1 s k s ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "whisky"  n ( w i1 s k ii ) ((pos "M8%" ) ))
( "whisper"  v ( w i1 s p @ r ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "whispered"  v ( w i1 s p @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "whisperer"  n ( w i1 s p @ r @ r ) ((pos "K6%" ) ))
( "whisperers"  n ( w i1 s p @ r @ z ) ((pos "Kj%" ) ))
( "whispering"  v ( w i1 s p @ r i ng ) ((pos "Jb%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "whispering-galleries"  n ( w i2 s p @ r i ng - g a1 l @ r i z ) ((pos "Kj$" ) ))
( "whispering-gallery"  n ( w i2 s p @ r i ng - g a1 l @ r ii ) ((pos "K8$" ) ))
( "whisperings"  n ( w i1 s p @ r i ng z ) ((pos "Kj%" ) ))
( "whispers"  v ( w i1 s p @ z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "14" )))
( "whist"  n ( w i1 s t ) ((pos "L@%" ) ))
( "whist-drive"  n ( w i1 s t - d r ai v ) ((pos "K6%" ) ))
( "whist-drives"  n ( w i1 s t - d r ai v z ) ((pos "Kj%" ) ))
( "whistle"  v ( w i1 s l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" "15B" )))
( "whistle-stop"  n ( w i1 s l - s t o p ) ((pos "K6%" ) ))
( "whistle-stops"  n ( w i1 s l - s t o p s ) ((pos "Kj$" ) ))
( "whistled"  v ( w i1 s l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" "15B" )))
( "whistles"  v ( w i1 s l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" "15B" )))
( "whistling"  v ( w i1 s l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" "15B" )))
( "whit"  n ( w i1 t ) ((pos "Ki$" ) ))
( "white"  n ( w ai1 t ) ((pos "M6%" "OB*" ) ))
( "white-collar"  j ( w ai2 t - k o1 l @ r ) ((pos "OA%" ) ))
( "white-hot"  j ( w ai2 t - h o1 t ) ((pos "OA%" ) ))
( "white-lipped"  j ( w ai2 t - l i1 p t ) ((pos "OA$" ) ))
( "white-livered"  j ( w ai2 t - l i1 v @ d ) ((pos "OA$" ) ))
( "whitebait"  n ( w ai1 t b ei t ) ((pos "L@%" ) ))
( "whitecaps"  n ( w ai1 t k a p s ) ((pos "Kj$" ) ))
( "whiten"  v ( w ai1 t n ) ((pos "J0%" ) (subcat "2A" )))
( "whitened"  v ( w ai1 t n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "whiteness"  n ( w ai1 t n i s ) ((pos "L@%" ) ))
( "whitening"  v ( w ai1 t n i ng ) ((pos "Jb%" "L@%" ) (subcat "2A" )))
( "whitens"  v ( w ai1 t n z ) ((pos "Ja%" ) (subcat "2A" )))
( "whiter"  j ( w ai1 t @ r ) ((pos "Or%" ) ))
( "whites"  n ( w ai1 t s ) ((pos "Mj%" ) ))
( "whitest"  j ( w ai1 t i s t ) ((pos "Os%" ) ))
( "whitethorn"  n ( w ai1 t th oo n ) ((pos "M6$" ) ))
( "whitethorns"  n ( w ai1 t th oo n z ) ((pos "Mj$" ) ))
( "whitewash"  v ( w ai1 t w o sh ) ((pos "H1%" "L@%" ) ))
( "whitewashed"  v ( w ai1 t w o sh t ) ((pos "Hc%" "Hd%" ) ))
( "whitewashes"  v ( w ai1 t w o sh i z ) ((pos "Ha%" ) ))
( "whitewashing"  v ( w ai1 t w o sh i ng ) ((pos "Hb%" ) ))
( "whither"  a ( w i1 dh @ r ) ((pos "Pu%" ) ))
( "whithersoever"  a ( w i2 dh @ s ou e1 v @ r ) ((pos "Pu$" ) ))
( "whiting"  n ( w ai1 t i ng ) ((pos "M9%" ) ))
( "whitish"  j ( w ai1 t i sh ) ((pos "OA%" ) ))
( "whitlow"  n ( w i1 t l ou ) ((pos "K6$" ) ))
( "whitlows"  n ( w i1 t l ou z ) ((pos "Kj$" ) ))
( "whittle"  v ( w i1 t l ) ((pos "J2%" ) (subcat "2C" "3A" "6A" "15A" )))
( "whittled"  v ( w i1 t l d ) ((pos "Jc%" "Jd%" ) (subcat "2C" "3A" "6A" "15A" )))
( "whittles"  v ( w i1 t l z ) ((pos "Ja%" ) (subcat "2C" "3A" "6A" "15A" )))
( "whittling"  v ( w i1 t l i ng ) ((pos "Jb%" ) (subcat "2C" "3A" "6A" "15A" )))
( "whiz"  v ( w i1 z ) ((pos "I5%" "L@%" ) (subcat )))
( "whizz-kid"  n ( w i1 z - k i d ) ((pos "K6%" ) ))
( "whizz-kids"  n ( w i1 z - k i d z ) ((pos "Kj%" ) ))
( "whizzed"  v ( w i1 z d ) ((pos "Ic%" "Id%" ) (subcat )))
( "whizzes"  v ( w i1 z i z ) ((pos "Ia%" ) (subcat )))
( "whizzing"  v ( w i1 z i ng ) ((pos "Ib%" ) (subcat )))
( "who"  prp ( h uu ) ((pos "Qy*" "Qz*" ) ))
( "who'd"  v ( h uu d ) ((pos "Gf%" ) ))
( "who'll"  v ( h uu l ) ((pos "Gf%" ) ))
( "who're"  v ( h uu1 @ r ) ((pos "Gf%" ) ))
( "who's"  v ( h uu z ) ((pos "Gf%" ) ))
( "whoa"  uh ( w ou1 ) ((pos "W-$" ) ))
( "whodunit"  n ( h uu2 d uh1 n i t ) ((pos "K6%" ) ))
( "whodunits"  n ( h uu2 d uh1 n i t s ) ((pos "Kj%" ) ))
( "whoever"  prp ( h uu1 e1 v @ r ) ((pos "Qx%" ) ))
( "whole"  n ( h ou1 l ) ((pos "K6*" "OA*" ) ))
( "whole-wheat"  n ( h ou1 l - w ii t ) ((pos "L@%" ) ))
( "wholehearted"  j ( h ou2 l h aa1 t i d ) ((pos "OA%" ) ))
( "wholeheartedly"  a ( h ou2 l h aa1 t i d l ii ) ((pos "Pu%" ) ))
( "wholeheartedness"  n ( h ou2 l h aa1 t i d n i s ) ((pos "L@%" ) ))
( "wholemeal"  n ( h ou1 l m ii l ) ((pos "L@%" ) ))
( "wholes"  n ( h ou1 l z ) ((pos "Kj$" ) ))
( "wholesale"  n ( h ou1 l s ei l ) ((pos "L@%" "OA%" "Pu%" ) ))
( "wholesaler"  n ( h ou1 l s ei l @ r ) ((pos "K6%" ) ))
( "wholesalers"  n ( h ou1 l s ei l @ z ) ((pos "Kj%" ) ))
( "wholesome"  j ( h ou1 l s @ m ) ((pos "OA%" ) ))
( "wholesomely"  a ( h ou1 l s @ m l ii ) ((pos "Pu%" ) ))
( "wholly"  a ( h ou1 l - l ii ) ((pos "Pu%" ) ))
( "whom"  prp ( h uu m ) ((pos "Qy%" "Qz%" ) ))
( "whoop"  v ( h uu1 p ) ((pos "J0$" "K6$" ) ))
( "whooped"  v ( h uu1 p t ) ((pos "Jc$" "Jd$" ) ))
( "whoopee"  n ( w u1 p ii ) ((pos "K6$" ) ))
( "whoopees"  n ( w u1 p i z ) ((pos "Kj$" ) ))
( "whooping"  v ( h uu1 p i ng ) ((pos "Jb%" ) ))
( "whooping-cough"  n ( h uu1 p i ng - k o f ) ((pos "L@%" ) ))
( "whoops"  v ( h uu1 p s ) ((pos "Ja$" "Kj$" ) ))
( "whop"  v ( w o1 p ) ((pos "H4$" ) (subcat )))
( "whopped"  v ( w o1 p t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "whopper"  n ( w o1 p @ r ) ((pos "K6%" ) ))
( "whoppers"  n ( w o1 p @ z ) ((pos "Kj%" ) ))
( "whopping"  v ( w o1 p i ng ) ((pos "Hb$" "OA%" "Pu%" ) (subcat )))
( "whops"  v ( w o1 p s ) ((pos "Ha$" ) (subcat )))
( "whore"  n ( h oo1 r ) ((pos "K6%" ) ))
( "whoremonger"  n ( h oo1 m uh ng g @ r ) ((pos "K6%" ) ))
( "whoremongers"  n ( h oo1 m uh ng g @ z ) ((pos "Kj%" ) ))
( "whores"  n ( h oo1 z ) ((pos "Kj%" ) ))
( "whorl"  n ( w @@1 l ) ((pos "K6$" ) ))
( "whorled"  j ( w @@1 l d ) ((pos "OA$" ) ))
( "whorls"  n ( w @@1 l z ) ((pos "Kj$" ) ))
( "whose"  prp ( h uu z ) ((pos "Qx%" ) ))
( "whoso"  prp ( h uu1 s ou ) ((pos "Qx$" ) ))
( "whosoever"  prp ( h uu2 s ou e1 v @ r ) ((pos "Qx%" ) ))
( "why"  n ( w ai1 ) ((pos "K6%" "Pv*" "Pw*" "W-%" ) ))
( "whys"  n ( w ai1 z ) ((pos "Kj%" ) ))
( "wich-"  j ( w i ch - ) ((pos "U-$" ) ))
( "wick"  n ( w i1 k ) ((pos "M6%" ) ))
( "wicked"  j ( w i1 k i d ) ((pos "OA%" ) ))
( "wickedly"  a ( w i1 k i d l ii ) ((pos "Pu%" ) ))
( "wickedness"  n ( w i1 k i d n i s ) ((pos "L@%" ) ))
( "wicker"  n ( w i1 k @ r ) ((pos "L@%" ) ))
( "wickerwork"  n ( w i1 k @ w @@ k ) ((pos "L@%" ) ))
( "wicket"  n ( w i1 k i t ) ((pos "K6%" ) ))
( "wicket-door"  n ( w i2 k i t - d oo1 r ) ((pos "K6$" ) ))
( "wicket-doors"  n ( w i2 k i t - d oo1 z ) ((pos "Kj$" ) ))
( "wicket-gate"  n ( w i2 k i t - g ei1 t ) ((pos "K6%" ) ))
( "wicket-gates"  n ( w i2 k i t - g ei1 t s ) ((pos "Kj%" ) ))
( "wicket-keeper"  n ( w i1 k i t - k ii p @ r ) ((pos "K6%" ) ))
( "wicket-keepers"  n ( w i1 k i t - k ii p @ z ) ((pos "Kj%" ) ))
( "wickets"  n ( w i1 k i t s ) ((pos "Kj%" ) ))
( "wicks"  n ( w i1 k s ) ((pos "Mj%" ) ))
( "wide"  j ( w ai1 d ) ((pos "OB%" "Pu%" ) ))
( "wide-awake"  j ( w ai2 d - @ w ei1 k ) ((pos "OA%" ) ))
( "widely"  a ( w ai1 d l ii ) ((pos "Pu%" ) ))
( "widen"  v ( w ai1 d n ) ((pos "J0%" ) (subcat "2A" )))
( "widened"  v ( w ai1 d n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "widening"  v ( w ai1 d n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "widens"  v ( w ai1 d n z ) ((pos "Ja%" ) (subcat "2A" )))
( "wider"  j ( w ai1 d @ r ) ((pos "Or%" ) ))
( "widespread"  j ( w ai1 d s p r e d ) ((pos "OA%" ) ))
( "widest"  j ( w ai1 d i s t ) ((pos "Os%" ) ))
( "widgeon"  n ( w i1 jh @ n ) ((pos "K6$" ) ))
( "widgeons"  n ( w i1 jh @ n z ) ((pos "Kj$" ) ))
( "widow"  n ( w i1 d ou ) ((pos "K6%" ) ))
( "widowed"  j ( w i1 d ou d ) ((pos "OA%" ) ))
( "widower"  n ( w i1 d ou @ r ) ((pos "K6%" ) ))
( "widowers"  n ( w i1 d ou @ z ) ((pos "Kj%" ) ))
( "widowhood"  n ( w i1 d ou h u d ) ((pos "L@%" ) ))
( "widows"  n ( w i1 d ou z ) ((pos "Kj%" ) ))
( "width"  n ( w i1 t th ) ((pos "M6%" ) ))
( "widths"  n ( w i1 t th s ) ((pos "Mj%" ) ))
( "wield"  v ( w ii1 l d ) ((pos "H0%" ) (subcat )))
( "wielded"  v ( w ii1 l d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wielding"  v ( w ii1 l d i ng ) ((pos "Hb%" ) (subcat )))
( "wields"  v ( w ii1 l d z ) ((pos "Ha%" ) (subcat )))
( "wife"  n ( w ai1 f ) ((pos "Ki*" ) ))
( "wifelier"  j ( w ai1 f l i@ r ) ((pos "Or$" ) ))
( "wifeliest"  j ( w ai1 f l i i s t ) ((pos "Os$" ) ))
( "wifelike"  j ( w ai1 f l ai k ) ((pos "OA%" ) ))
( "wifely"  j ( w ai1 f l ii ) ((pos "OD%" ) ))
( "wig"  n ( w i1 g ) ((pos "K6%" ) ))
( "wigged"  j ( w i1 g d ) ((pos "OA%" ) ))
( "wigging"  n ( w i1 g i ng ) ((pos "K6%" ) ))
( "wiggings"  n ( w i1 g i ng z ) ((pos "Kj$" ) ))
( "wiggle"  v ( w i1 g l ) ((pos "J2%" "K6%" ) (subcat "2A" )))
( "wiggled"  v ( w i1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "wiggles"  v ( w i1 g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" )))
( "wiggling"  v ( w i1 g l i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "wight"  n ( w ai1 t ) ((pos "K6$" ) ))
( "wights"  n ( w ai1 t s ) ((pos "Kj$" ) ))
( "wigs"  n ( w i1 g z ) ((pos "Kj%" ) ))
( "wigwam"  n ( w i1 g w a m ) ((pos "K6%" ) ))
( "wigwams"  n ( w i1 g w a m z ) ((pos "Kj%" ) ))
( "wild"  n ( w ai1 l d ) ((pos "Kj%" "OC%" "Pu%" ) ))
( "wildcat"  j ( w ai1 l d k a t ) ((pos "Oq%" ) ))
( "wildebeest"  n ( w i1 l d i b ii s t ) ((pos "K6$" ) ))
( "wildebeests"  n ( w i1 l d i b ii s t s ) ((pos "Kj$" ) ))
( "wilder"  j ( w ai1 l d @ r ) ((pos "Or%" ) ))
( "wilderness"  n ( w i1 l d @ n @ s ) ((pos "K7%" ) ))
( "wildernesses"  n ( w i1 l d @ n @ s i z ) ((pos "Kj%" ) ))
( "wildest"  j ( w ai1 l d i s t ) ((pos "Os%" ) ))
( "wildfire"  n ( w ai1 l d f ai @ r ) ((pos "L@%" ) ))
( "wildfowl"  n ( w ai1 l d f au l ) ((pos "Kj%" ) ))
( "wildly"  a ( w ai1 l d l ii ) ((pos "Pu%" ) ))
( "wildness"  n ( w ai1 l d n i s ) ((pos "L@%" ) ))
( "wile"  n ( w ai1 l ) ((pos "K6$" ) ))
( "wiles"  n ( w ai1 l z ) ((pos "Kj%" ) ))
( "wilful"  j ( w i1 l f @ l ) ((pos "OA%" ) ))
( "wilfully"  a ( w i1 l f @ l ii ) ((pos "Pu%" ) ))
( "wilfulness"  n ( w i1 l f @ l n i s ) ((pos "L@%" ) ))
( "wilier"  j ( w ai1 l i@ r ) ((pos "Or%" ) ))
( "wiliest"  j ( w ai1 l i i s t ) ((pos "Os%" ) ))
( "will"  v ( w i l ) ((pos "G5*" "J0%" "M6%" ) (subcat "2A" "5" "6A" "9" "12A" "13A" "14" "15A" )))
( "will-o'-the-wisp"  n ( w ii2 l - @ - dh @ - w i1 s p ) ((pos "K6%" ) ))
( "will-o'-the-wisps"  n ( w ii2 l - @ - dh @ - w i1 s p s ) ((pos "Kj$" ) ))
( "willed"  v ( w i1 l d ) ((pos "Jc%" "Jd%" "Ot%" ) (subcat "2A" "5" "6A" "9" "12A" "13A" "14" "15A" )))
( "willful"  j ( w i1 l f @ l ) ((pos "OA%" ) ))
( "willies"  n ( w i1 l i z ) ((pos "Kj%" ) ))
( "willing"  v ( w i1 l i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "5" "6A" "9" "12A" "13A" "14" "15A" )))
( "willingly"  a ( w i1 l i ng l ii ) ((pos "Pu%" ) ))
( "willingness"  n ( w i1 l i ng n i s ) ((pos "L@%" ) ))
( "willow"  n ( w i1 l ou ) ((pos "M6%" ) ))
( "willow-pattern"  n ( w i1 l ou - p a t @ n ) ((pos "L@%" ) ))
( "willow-tree"  n ( w i1 l ou - t r ii ) ((pos "K6%" ) ))
( "willow-trees"  n ( w i1 l ou - t r ii z ) ((pos "Kj%" ) ))
( "willows"  n ( w i1 l ou z ) ((pos "Mj%" ) ))
( "willowy"  j ( w i1 l ou ii ) ((pos "OA%" ) ))
( "willpower"  n ( w i1 l p au @ r ) ((pos "L@%" ) ))
( "wills"  v ( w i1 l z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "5" "6A" "9" "12A" "13A" "14" "15A" )))
( "willy-nilly"  a ( w i2 l i - n i1 l ii ) ((pos "Pu%" ) ))
( "wilt"  v ( w i1 l t ) ((pos "He$" "J0%" ) (subcat "2A" "5" "6A" )))
( "wilted"  v ( w i1 l t i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "5" "6A" )))
( "wilting"  v ( w i1 l t i ng ) ((pos "Jb%" ) (subcat "2A" "5" "6A" )))
( "wilts"  v ( w i1 l t s ) ((pos "Ja%" ) (subcat "2A" "5" "6A" )))
( "wily"  j ( w ai1 l ii ) ((pos "OD%" ) ))
( "wimple"  n ( w i1 m p l ) ((pos "K6$" ) ))
( "wimples"  n ( w i1 m p l z ) ((pos "Kj$" ) ))
( "win"  v ( w i1 n ) ((pos "J5%" "K6%" ) (subcat "2A" "6A" "12B" "13B" "15A" "15B" )))
( "wince"  v ( w i1 n s ) ((pos "H2%" "K6%" ) (subcat "2A" )))
( "winced"  v ( w i1 n s t ) ((pos "Hc%" "Hd%" ) (subcat "2A" )))
( "winces"  v ( w i1 n s i z ) ((pos "Ha%" "Kj%" ) (subcat "2A" )))
( "winceyette"  n ( w i2 n s i e1 t ) ((pos "L@%" ) ))
( "winch"  v ( w i1 n ch ) ((pos "H1%" "K7%" ) (subcat "6A" )))
( "winched"  v ( w i1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "winches"  v ( w i1 n ch i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" )))
( "winching"  v ( w i1 n ch i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "wincing"  v ( w i1 n s i ng ) ((pos "Hb%" ) (subcat "2A" )))
( "wind"  v ( w i1 n d ) ((pos "H0%" "M6%" ) (subcat )))
( "wind"  v ( w ai1 n d ) ((pos "J5%" "K6%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "wind-gauge"  n ( w i1 n d - g ei jh ) ((pos "K6%" ) ))
( "wind-gauges"  n ( w i1 n d - g ei jh i z ) ((pos "Kj%" ) ))
( "wind-tunnel"  n ( w i1 n d - t uh n @ l ) ((pos "K6%" ) ))
( "wind-tunnels"  n ( w i1 n d - t uh n @ l z ) ((pos "Kj%" ) ))
( "windbag"  n ( w i1 n d b a g ) ((pos "K6%" ) ))
( "windbags"  n ( w i1 n d b a g z ) ((pos "Kj%" ) ))
( "windblown"  j ( w i1 n d b l ou n ) ((pos "OA%" ) ))
( "windbreak"  n ( w i1 n d b r ei k ) ((pos "K6%" ) ))
( "windbreaker"  n ( w i1 n d b r ei k @ r ) ((pos "K6%" ) ))
( "windbreakers"  n ( w i1 n d b r ei k @ z ) ((pos "Kj%" ) ))
( "windbreaks"  n ( w i1 n d b r ei k s ) ((pos "Kj%" ) ))
( "windcheater"  n ( w i1 n d ch ii t @ r ) ((pos "K6%" ) ))
( "windcheaters"  n ( w i1 n d ch ii t @ z ) ((pos "Kj%" ) ))
( "winded"  v ( w i1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "windfall"  n ( w i1 n d f oo l ) ((pos "K6%" ) ))
( "windfalls"  n ( w i1 n d f oo l z ) ((pos "Kj%" ) ))
( "windflower"  n ( w i1 n d f l au @ r ) ((pos "K6%" ) ))
( "windflowers"  n ( w i1 n d f l au @ z ) ((pos "Kj%" ) ))
( "windier"  j ( w i1 n d i@ r ) ((pos "Or%" ) ))
( "windiest"  j ( w i1 n d i i s t ) ((pos "Os%" ) ))
( "windily"  a ( w i1 n d i l ii ) ((pos "Pu%" ) ))
( "windiness"  n ( w i1 n d i n i s ) ((pos "L@$" ) ))
( "winding"  v ( w i1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "winding"  v ( w ai1 n d i ng ) ((pos "Jb%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "winding-sheet"  n ( w ai1 n d i ng - sh ii t ) ((pos "K6%" ) ))
( "winding-sheets"  n ( w ai1 n d i ng - sh ii t s ) ((pos "Kj%" ) ))
( "windjammer"  n ( w i1 n jh a m @ r ) ((pos "K6%" ) ))
( "windjammers"  n ( w i1 n jh a m @ z ) ((pos "Kj%" ) ))
( "windlass"  n ( w i1 n d l @ s ) ((pos "K7$" ) ))
( "windlasses"  n ( w i1 n d l @ s i z ) ((pos "Kj$" ) ))
( "windless"  j ( w i1 n d l @ s ) ((pos "OA%" ) ))
( "windmill"  n ( w i1 n d m i l ) ((pos "K6%" ) ))
( "windmills"  n ( w i1 n d m i l z ) ((pos "Kj%" ) ))
( "window"  n ( w i1 n d ou ) ((pos "K6%" ) ))
( "window-box"  n ( w i1 n d ou - b o k s ) ((pos "K7%" ) ))
( "window-boxes"  n ( w i1 n d ou - b o k s i z ) ((pos "Kj%" ) ))
( "window-dressing"  n ( w i1 n d ou - d r e2 s i ng ) ((pos "L@%" ) ))
( "windowpane"  n ( w i1 n d ou p ei n ) ((pos "K6%" ) ))
( "windowpanes"  n ( w i1 n d ou p ei n z ) ((pos "Kj%" ) ))
( "windows"  n ( w i1 n d ou z ) ((pos "Kj%" ) ))
( "windowsill"  n ( w i1 n d ou s i l ) ((pos "K6%" ) ))
( "windowsills"  n ( w i1 n d ou s i l z ) ((pos "Kj%" ) ))
( "windpipe"  n ( w i1 n d p ai p ) ((pos "K6%" ) ))
( "windpipes"  n ( w i1 n d p ai p s ) ((pos "Kj%" ) ))
( "winds"  v ( w i1 n d z ) ((pos "Ha$" "Mj%" ) (subcat )))
( "winds"  v ( w ai1 n d z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "windscreen"  n ( w i1 n d s k r ii n ) ((pos "K6%" ) ))
( "windscreen-wiper"  n ( w i1 n d s k r ii n - w ai2 p @ r ) ((pos "K6%" ) ))
( "windscreen-wipers"  n ( w i1 n d s k r ii n - w ai2 p @ z ) ((pos "Kj%" ) ))
( "windscreens"  n ( w i1 n d s k r ii n z ) ((pos "Kj%" ) ))
( "windshield"  n ( w i1 n d sh ii l d ) ((pos "K6%" ) ))
( "windshields"  n ( w i1 n d sh ii l d z ) ((pos "Kj%" ) ))
( "windsock"  n ( w i1 n d s o k ) ((pos "K6%" ) ))
( "windsocks"  n ( w i1 n d s o k s ) ((pos "Kj%" ) ))
( "windswept"  j ( w i1 n d s w e p t ) ((pos "OA%" ) ))
( "windward"  n ( w i1 n d w @ d ) ((pos "L@%" "OA%" ) ))
( "windy"  j ( w i1 n d ii ) ((pos "OD%" ) ))
( "wine"  v ( w ai1 n ) ((pos "H2%" "M6%" ) ))
( "wined"  v ( w ai1 n d ) ((pos "Hc%" "Hd%" ) ))
( "wineglass"  n ( w ai1 n - g l aa s ) ((pos "K7%" ) ))
( "wineglasses"  n ( w ai1 n - g l aa s i z ) ((pos "Kj%" ) ))
( "winepress"  n ( w ai1 n p r e s ) ((pos "K7%" ) ))
( "winepresses"  n ( w ai1 n p r e s i z ) ((pos "Kj%" ) ))
( "wines"  v ( w ai1 n z ) ((pos "Ha%" "Mj%" ) ))
( "wineskin"  n ( w ai1 n s k i n ) ((pos "K6%" ) ))
( "wineskins"  n ( w ai1 n s k i n z ) ((pos "Kj%" ) ))
( "wing"  v ( w i1 ng ) ((pos "J0%" "K6%" ) (subcat "2C" "6A" )))
( "wing-commander"  n ( w i1 ng - k @ m aa2 n d @ r ) ((pos "K6%" ) ))
( "wing-commanders"  n ( w i1 ng - k @ m aa2 n d @ z ) ((pos "Kj%" ) ))
( "wing-nut"  n ( w i1 ng - n uh t ) ((pos "K6%" ) ))
( "wing-nuts"  n ( w i1 ng - n uh t s ) ((pos "Kj%" ) ))
( "wing-screw"  n ( w i1 ng - s k r uu ) ((pos "K6%" ) ))
( "wing-screws"  n ( w i1 ng - s k r uu z ) ((pos "Kj%" ) ))
( "winged"  v ( w i1 ng d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2C" "6A" )))
( "winger"  n ( w i1 ng @ r ) ((pos "K6%" ) ))
( "wingers"  n ( w i1 ng @ z ) ((pos "Kj%" ) ))
( "winging"  v ( w i1 ng i ng ) ((pos "Jb%" ) (subcat "2C" "6A" )))
( "wingless"  j ( w i1 ng l @ s ) ((pos "OA%" ) ))
( "wings"  v ( w i1 ng z ) ((pos "Ja%" "Kj%" ) (subcat "2C" "6A" )))
( "wingspan"  n ( w i1 ng s p a n ) ((pos "K6%" ) ))
( "wingspans"  n ( w i1 ng s p a n z ) ((pos "Kj%" ) ))
( "wingspread"  n ( w i1 ng s p r e d ) ((pos "K6%" ) ))
( "wingspreads"  n ( w i1 ng s p r e d z ) ((pos "Kj%" ) ))
( "wining"  v ( w ai1 n i ng ) ((pos "Hb%" ) ))
( "wink"  v ( w i1 ng k ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "3A" )))
( "winked"  v ( w i1 ng k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" )))
( "winking"  v ( w i1 ng k i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" )))
( "winkle"  v ( w i1 ng k l ) ((pos "H2%" "K6%" ) (subcat )))
( "winkled"  v ( w i1 ng k l d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "winkles"  v ( w i1 ng k l z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "winkling"  v ( w i1 ng k l i ng ) ((pos "Hb%" ) (subcat )))
( "winks"  v ( w i1 ng k s ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" )))
( "winner"  n ( w i1 n @ r ) ((pos "K6%" ) ))
( "winners"  n ( w i1 n @ z ) ((pos "Kj%" ) ))
( "winning"  v ( w i1 n i ng ) ((pos "Jb%" ) (subcat "2A" "6A" "12B" "13B" "15A" "15B" )))
( "winning-post"  n ( w i1 n i ng - p ou s t ) ((pos "K6%" ) ))
( "winning-posts"  n ( w i1 n i ng - p ou s t s ) ((pos "Kj%" ) ))
( "winnings"  n ( w i1 n i ng z ) ((pos "Kj%" ) ))
( "winnow"  v ( w i1 n ou ) ((pos "H0$" ) (subcat "6A" "14" "15A" )))
( "winnowed"  v ( w i1 n ou d ) ((pos "Hc$" "Hd$" ) (subcat "6A" "14" "15A" )))
( "winnowing"  v ( w i1 n ou i ng ) ((pos "Hb$" ) (subcat "6A" "14" "15A" )))
( "winnows"  v ( w i1 n ou z ) ((pos "Ha$" ) (subcat "6A" "14" "15A" )))
( "wins"  v ( w i1 n z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "6A" "12B" "13B" "15A" "15B" )))
( "winsome"  j ( w i1 n s @ m ) ((pos "OA$" ) ))
( "winsomely"  a ( w i1 n s @ m l ii ) ((pos "Pu$" ) ))
( "winsomeness"  n ( w i1 n s @ m n i s ) ((pos "L@$" ) ))
( "winter"  v ( w i1 n t @ r ) ((pos "I0%" "M6%" ) (subcat )))
( "wintered"  v ( w i1 n t @ d ) ((pos "Ic%" "Id%" ) (subcat )))
( "winterier"  j ( w i1 n t r i@ r ) ((pos "Or%" ) ))
( "winteriest"  j ( w i1 n t r i i s t ) ((pos "Os%" ) ))
( "wintering"  v ( w i1 n t @ r i ng ) ((pos "Ib%" ) (subcat )))
( "winters"  v ( w i1 n t @ z ) ((pos "Ia%" "Mj%" ) (subcat )))
( "wintery"  j ( w i1 n t r ii ) ((pos "OD%" ) ))
( "wintrier"  j ( w i1 n t r i@ r ) ((pos "Or$" ) ))
( "wintriest"  j ( w i1 n t r i i s t ) ((pos "Os$" ) ))
( "wintry"  j ( w i1 n t r ii ) ((pos "OD$" ) ))
( "wipe"  v ( w ai1 p ) ((pos "J2%" "K6%" ) (subcat "6A" "15A" "15B" )))
( "wiped"  v ( w ai1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "15A" "15B" )))
( "wiper"  n ( w ai1 p @ r ) ((pos "K6%" ) ))
( "wipers"  n ( w ai1 p @ z ) ((pos "Kj%" ) ))
( "wipes"  v ( w ai1 p s ) ((pos "Ja%" "Kj%" ) (subcat "6A" "15A" "15B" )))
( "wiping"  v ( w ai1 p i ng ) ((pos "Jb%" ) (subcat "6A" "15A" "15B" )))
( "wire"  v ( w ai1 @ r ) ((pos "J2%" "M6%" ) (subcat "6A" "11" "12A" "13A" "15A" "15B" )))
( "wire-cutters"  n ( w ai1 @ - k uh2 t @ z ) ((pos "Kj%" ) ))
( "wire-haired"  j ( w ai2 @ - h e@1 d ) ((pos "OA%" ) ))
( "wired"  v ( w ai1 @ d ) ((pos "Jc%" "Jd%" ) (subcat "6A" "11" "12A" "13A" "15A" "15B" )))
( "wireless"  n ( w ai1 @ l i s ) ((pos "M7%" "OA%" ) ))
( "wirelesses"  n ( w ai1 @ l i s i z ) ((pos "Mj%" ) ))
( "wirepuller"  n ( w ai1 @ p u2 l @ r ) ((pos "K6%" ) ))
( "wirepullers"  n ( w ai1 @ p u2 l @ z ) ((pos "Kj%" ) ))
( "wires"  v ( w ai1 @ z ) ((pos "Ja%" "Mj%" ) (subcat "6A" "11" "12A" "13A" "15A" "15B" )))
( "wireworm"  n ( w ai1 @ w @@ m ) ((pos "K6%" ) ))
( "wireworms"  n ( w ai1 @ w @@ m z ) ((pos "Kj%" ) ))
( "wirier"  j ( w ai1 @ r i@ r ) ((pos "Or$" ) ))
( "wiriest"  j ( w ai1 @ r i i s t ) ((pos "Os$" ) ))
( "wiring"  v ( w ai1 @ r i ng ) ((pos "Jb%" "L@%" ) (subcat "6A" "11" "12A" "13A" "15A" "15B" )))
( "wiry"  j ( w ai1 @ r ii ) ((pos "OD%" ) ))
( "wisdom"  n ( w i1 z d @ m ) ((pos "L@%" ) ))
( "wisdom-teeth"  n ( w i1 z d @ m - t ii th ) ((pos "Kj%" ) ))
( "wisdom-tooth"  n ( w i1 z d @ m - t uu th ) ((pos "Ki%" ) ))
( "wise"  n ( w ai1 z ) ((pos "Ki%" "OB%" ) ))
( "wiseacre"  n ( w ai1 z ei k @ r ) ((pos "K6%" ) ))
( "wiseacres"  n ( w ai1 z ei k @ z ) ((pos "Kj%" ) ))
( "wisecrack"  v ( w ai1 z k r a k ) ((pos "I0%" "K6%" ) ))
( "wisecracked"  v ( w ai1 z k r a k t ) ((pos "Ic%" "Id%" ) ))
( "wisecracking"  v ( w ai1 z k r a k i ng ) ((pos "Ib%" ) ))
( "wisecracks"  v ( w ai1 z k r a k s ) ((pos "Ia%" "Kj%" ) ))
( "wisely"  a ( w ai1 z l ii ) ((pos "Pu%" ) ))
( "wiser"  j ( w ai1 z @ r ) ((pos "Or%" ) ))
( "wisest"  j ( w ai1 z i s t ) ((pos "Os%" ) ))
( "wish"  v ( w i1 sh ) ((pos "J1%" "M7%" ) (subcat "2A" "3A" "6A" "7A" "9" "12A" "13A" "15A" "17" )))
( "wishbone"  n ( w i1 sh b ou n ) ((pos "K6%" ) ))
( "wishbones"  n ( w i1 sh b ou n z ) ((pos "Kj%" ) ))
( "wished"  v ( w i1 sh t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "7A" "9" "12A" "13A" "15A" "17" )))
( "wishes"  v ( w i1 sh i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "7A" "9" "12A" "13A" "15A" "17" )))
( "wishful"  j ( w i1 sh f @ l ) ((pos "OA%" ) ))
( "wishfully"  a ( w i1 sh f @ l ii ) ((pos "Pu%" ) ))
( "wishing"  v ( w i1 sh i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "6A" "7A" "9" "12A" "13A" "15A" "17" )))
( "wishing-cap"  n ( w i1 sh i ng - k a p ) ((pos "K6$" ) ))
( "wishing-caps"  n ( w i1 sh i ng - k a p s ) ((pos "Kj$" ) ))
( "wishy-washy"  j ( w i1 sh i - w o sh ii ) ((pos "OA%" ) ))
( "wisp"  n ( w i1 s p ) ((pos "K6%" ) ))
( "wispier"  j ( w i1 s p i@ r ) ((pos "Or%" ) ))
( "wispiest"  j ( w i1 s p i i s t ) ((pos "Os%" ) ))
( "wisps"  n ( w i1 s p s ) ((pos "Kj%" ) ))
( "wispy"  j ( w i1 s p ii ) ((pos "OD%" ) ))
( "wisteria"  n ( w i1 s t i@1 r i@ ) ((pos "M6%" ) ))
( "wisterias"  n ( w i1 s t i@1 r i@ z ) ((pos "Mj$" ) ))
( "wistful"  j ( w i1 s t f @ l ) ((pos "OA%" ) ))
( "wistfully"  a ( w i1 s t f @ l ii ) ((pos "Pu%" ) ))
( "wit"  n ( w i1 t ) ((pos "M6%" ) ))
( "witch"  n ( w i1 ch ) ((pos "K7%" ) ))
( "witch-"  j ( w i ch - ) ((pos "U-$" ) ))
( "witch-doctor"  n ( w i1 ch - d o k t @ r ) ((pos "K6%" ) ))
( "witch-doctors"  n ( w i1 ch - d o k t @ z ) ((pos "Kj%" ) ))
( "witch-elm"  n ( w i1 ch - e l m ) ((pos "K6$" ) ))
( "witch-elms"  n ( w i1 ch - e l m z ) ((pos "Kj$" ) ))
( "witch-hazel"  n ( w i1 ch - h ei z l ) ((pos "M6%" ) ))
( "witch-hazels"  n ( w i1 ch - h ei z l z ) ((pos "Mj$" ) ))
( "witch-hunt"  n ( w i1 ch - h uh n t ) ((pos "K6%" ) ))
( "witch-hunts"  n ( w i1 ch - h uh n t s ) ((pos "Kj%" ) ))
( "witchcraft"  n ( w i1 ch k r aa f t ) ((pos "L@%" ) ))
( "witchery"  n ( w i1 ch @ r ii ) ((pos "L@%" ) ))
( "witches"  n ( w i1 ch i z ) ((pos "Kj%" ) ))
( "witching"  j ( w i1 ch i ng ) ((pos "OA%" ) ))
( "with"  in ( w i dh ) ((pos "T-*" ) ))
( "withal"  a ( w i1 dh oo1 l ) ((pos "Pu%" ) ))
( "withdraw"  v ( w i1 dh d r oo1 ) ((pos "J5%" ) (subcat "2A" "2C" "6A" )))
( "withdrawal"  n ( w i1 dh d r oo1 @ l ) ((pos "M6%" ) ))
( "withdrawals"  n ( w i1 dh d r oo1 @ l z ) ((pos "Mj%" ) ))
( "withdrawing"  v ( w i1 dh d r oo1 i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "withdrawn"  v ( w i1 dh d r oo1 n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" )))
( "withdraws"  v ( w i1 dh d r oo1 z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "withdrew"  v ( w i1 dh d r uu1 ) ((pos "Jc%" ) (subcat "2A" "2C" "6A" )))
( "withe"  n ( w i1 th ) ((pos "K6$" ) ))
( "wither"  v ( w i1 dh @ r ) ((pos "J0%" ) (subcat "2A" "2C" "6A" )))
( "withered"  v ( w i1 dh @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "withering"  v ( w i1 dh @ r i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "witheringly"  a ( w i1 dh @ r i ng l ii ) ((pos "Pu%" ) ))
( "withers"  v ( w i1 dh @ z ) ((pos "Ja%" "Kj$" ) (subcat "2A" "2C" "6A" )))
( "withes"  n ( w i1 th s ) ((pos "Kj$" ) ))
( "withheld"  v ( w i1 dh h e1 l d ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "withhold"  v ( w i1 dh h ou1 l d ) ((pos "H5%" ) (subcat "6A" )))
( "withholding"  v ( w i1 dh h ou1 l d i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "withholds"  v ( w i1 dh h ou1 l d z ) ((pos "Ha%" ) (subcat "6A" )))
( "withies"  n ( w i1 dh i z ) ((pos "Kj$" ) ))
( "within"  a ( w i1 dh ii1 n ) ((pos "Pu*" "T-*" ) ))
( "without"  a ( w i1 dh au1 t ) ((pos "Pu*" "T-*" ) ))
( "withstand"  v ( w i1 dh s t a1 n d ) ((pos "H5%" ) (subcat )))
( "withstanding"  v ( w i1 dh s t a1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "withstands"  v ( w i1 dh s t a1 n d z ) ((pos "Ha%" ) (subcat )))
( "withstood"  v ( w i1 dh s t u1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "withy"  n ( w i1 dh ii ) ((pos "K8$" ) ))
( "witless"  j ( w i1 t l i s ) ((pos "OA%" ) ))
( "witness"  v ( w i1 t n @ s ) ((pos "J1%" "M7%" ) (subcat "3A" )))
( "witness-box"  n ( w i1 t n i s - b o k s ) ((pos "K7%" ) ))
( "witness-boxes"  n ( w i1 t n i s - b o k s i z ) ((pos "Kj$" ) ))
( "witness-stand"  n ( w i1 t n i s - s t a n d ) ((pos "K6%" ) ))
( "witness-stands"  n ( w i1 t n i s - s t a n d z ) ((pos "Kj$" ) ))
( "witnessed"  v ( w i1 t n @ s t ) ((pos "Jc%" "Jd%" ) (subcat "3A" )))
( "witnesses"  v ( w i1 t n @ s i z ) ((pos "Ja%" "Mj%" ) (subcat "3A" )))
( "witnessing"  v ( w i1 t n @ s i ng ) ((pos "Jb%" ) (subcat "3A" )))
( "wits"  n ( w i1 t s ) ((pos "Mj%" ) ))
( "witticism"  n ( w i1 t i s i z @ m ) ((pos "K6%" ) ))
( "witticisms"  n ( w i1 t i s i z @ m z ) ((pos "Kj%" ) ))
( "wittier"  j ( w i1 t i@ r ) ((pos "Or%" ) ))
( "wittiest"  j ( w i1 t i i s t ) ((pos "Os%" ) ))
( "wittily"  a ( w i1 t i l ii ) ((pos "Pu%" ) ))
( "wittingly"  a ( w i1 t i ng l ii ) ((pos "Pu%" ) ))
( "witty"  j ( w i1 t ii ) ((pos "OD%" ) ))
( "wive"  v ( w ai1 v ) ((pos "J2$" ) (subcat "2A" )))
( "wived"  v ( w ai1 v d ) ((pos "Jc$" "Jd$" ) (subcat "2A" )))
( "wives"  v ( w ai1 v z ) ((pos "Ja$" "Kj%" ) (subcat "2A" )))
( "wiving"  v ( w ai1 v i ng ) ((pos "Jb$" ) (subcat "2A" )))
( "wizard"  n ( w i1 z @ d ) ((pos "K6%" "OA%" ) ))
( "wizardry"  n ( w i1 z @ d r ii ) ((pos "L@%" ) ))
( "wizards"  n ( w i1 z @ d z ) ((pos "Kj%" ) ))
( "wizened"  j ( w i1 z n d ) ((pos "OA%" ) ))
( "wk"  n ( w ii1 k ) ((pos "Y]%" ) ))
( "wo"  uh ( w ou1 ) ((pos "W-$" ) ))
( "woad"  n ( w ou1 d ) ((pos "L@$" ) ))
( "wobble"  v ( w o1 b l ) ((pos "J2%" ) (subcat "2A" "2C" )))
( "wobbled"  v ( w o1 b l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" )))
( "wobbler"  n ( w o1 b l @ r ) ((pos "K6$" ) ))
( "wobblers"  n ( w o1 b l @ z ) ((pos "Kj$" ) ))
( "wobbles"  v ( w o1 b l z ) ((pos "Ja%" ) (subcat "2A" "2C" )))
( "wobblier"  j ( w o1 b l i@ r ) ((pos "Or$" ) ))
( "wobbliest"  j ( w o1 b l i i s t ) ((pos "Os$" ) ))
( "wobbling"  v ( w o1 b l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" )))
( "wobbly"  j ( w o1 b l ii ) ((pos "OD%" ) ))
( "woe"  n ( w ou1 ) ((pos "M6%" ) ))
( "woebegone"  j ( w ou1 b i g o n ) ((pos "OA%" ) ))
( "woeful"  j ( w ou1 f @ l ) ((pos "OA%" ) ))
( "woefully"  a ( w ou1 f @ l ii ) ((pos "Pu%" ) ))
( "woes"  n ( w ou1 z ) ((pos "Kj%" ) ))
( "woke"  v ( w ou1 k ) ((pos "Jc%" ) (subcat "2A" "2C" "4B" "6A" )))
( "woken"  v ( w ou1 k @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "4B" "6A" )))
( "wold"  n ( w ou1 l d ) ((pos "M6$" ) ))
( "wolds"  n ( w ou1 l d z ) ((pos "Mj$" ) ))
( "wolf"  v ( w u1 l f ) ((pos "H0%" "Ki%" ) (subcat "6A" )))
( "wolf's-bane"  n ( w u1 l f s - b ei n ) ((pos "L@%" ) ))
( "wolf-cub"  n ( w u1 l f - k uh b ) ((pos "K6%" ) ))
( "wolf-cubs"  n ( w u1 l f - k uh b z ) ((pos "Kj%" ) ))
( "wolfed"  v ( w u1 l f t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "wolfhound"  n ( w u1 l f h au n d ) ((pos "K6%" ) ))
( "wolfhounds"  n ( w u1 l f h au n d z ) ((pos "Kj%" ) ))
( "wolfing"  v ( w u1 l f i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "wolfish"  j ( w u1 l f i sh ) ((pos "OA%" ) ))
( "wolfram"  n ( w u1 l f r @ m ) ((pos "L@$" ) ))
( "wolfs"  v ( w u1 l f s ) ((pos "Ha%" ) (subcat "6A" )))
( "wolves"  n ( w u1 l v z ) ((pos "Kj%" ) ))
( "woman"  n ( w u1 m @ n ) ((pos "Ki*" ) ))
( "womanhood"  n ( w u1 m @ n h u d ) ((pos "L@%" ) ))
( "womanish"  j ( w u1 m @ n i sh ) ((pos "OA%" ) ))
( "womanize"  v ( w u1 m @ n ai z ) ((pos "I2%" ) ))
( "womanized"  v ( w u1 m @ n ai z d ) ((pos "Ic%" "Id%" ) ))
( "womanizer"  n ( w u1 m @ n ai z @ r ) ((pos "K6%" ) ))
( "womanizers"  n ( w u1 m @ n ai z @ z ) ((pos "Kj%" ) ))
( "womanizes"  v ( w u1 m @ n ai z i z ) ((pos "Ia%" ) ))
( "womanizing"  v ( w u1 m @ n ai z i ng ) ((pos "Ib%" ) ))
( "womankind"  n ( w u1 m @ n k ai n d ) ((pos "L@%" ) ))
( "womanlier"  j ( w u1 m @ n l i@ r ) ((pos "Or$" ) ))
( "womanliest"  j ( w u1 m @ n l i i s t ) ((pos "Os$" ) ))
( "womanlike"  j ( w u1 m @ n l ai k ) ((pos "OA$" ) ))
( "womanly"  j ( w u1 m @ n l ii ) ((pos "OD%" ) ))
( "womb"  n ( w uu1 m ) ((pos "K6%" ) ))
( "wombat"  n ( w o1 m b a t ) ((pos "K6$" ) ))
( "wombats"  n ( w o1 m b a t s ) ((pos "Kj$" ) ))
( "wombs"  n ( w uu1 m z ) ((pos "Kj%" ) ))
( "women"  n ( w i1 m i n ) ((pos "Kj*" ) ))
( "womenfolk"  n ( w i1 m i n f ou k ) ((pos "Kj%" ) ))
( "won"  v ( w uh1 n ) ((pos "Jc%" "Jd%" ) (subcat "2A" "6A" "12B" "13B" "15A" "15B" )))
( "won't"  v ( w ou n t ) ((pos "Gg%" ) ))
( "wonder"  v ( w uh1 n d @ r ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "3B" "4B" "8" )))
( "wonder-struck"  j ( w uh1 n d @ - s t r uh k ) ((pos "OA%" ) ))
( "wondered"  v ( w uh1 n d @ d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "3B" "4B" "8" )))
( "wonderful"  j ( w uh1 n d @ f @ l ) ((pos "OA%" ) ))
( "wonderfully"  a ( w uh1 n d @ f @ l ii ) ((pos "Pu%" ) ))
( "wondering"  v ( w uh1 n d @ r i ng ) ((pos "Jb%" ) (subcat "2A" "3A" "3B" "4B" "8" )))
( "wonderingly"  a ( w uh1 n d r i ng l ii ) ((pos "Pu%" ) ))
( "wonderland"  n ( w uh1 n d @ l a n d ) ((pos "K6%" ) ))
( "wonderlands"  n ( w uh1 n d @ l a n d z ) ((pos "Kj$" ) ))
( "wonderment"  n ( w uh1 n d @ m @ n t ) ((pos "L@%" ) ))
( "wonders"  v ( w uh1 n d @ z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "3B" "4B" "8" )))
( "wondrous"  j ( w uh1 n d r @ s ) ((pos "OA%" "Pu%" ) ))
( "wonky"  j ( w o1 ng k ii ) ((pos "OA%" ) ))
( "wont"  n ( w ou1 n t ) ((pos "L@%" "Op%" ) ))
( "wonted"  j ( w ou1 n t i d ) ((pos "Oq%" ) ))
( "woo"  v ( w uu1 ) ((pos "H0%" ) (subcat )))
( "wood"  n ( w u1 d ) ((pos "M6%" ) ))
( "wood-block"  n ( w u1 d - b l o k ) ((pos "K6%" ) ))
( "wood-blocks"  n ( w u1 d - b l o k s ) ((pos "Kj%" ) ))
( "wood-pulp"  n ( w u1 d - p uh l p ) ((pos "L@%" ) ))
( "woodbine"  n ( w u1 d b ai n ) ((pos "M6%" ) ))
( "woodbines"  n ( w u1 d b ai n z ) ((pos "Mj%" ) ))
( "woodcock"  n ( w u1 d k o k ) ((pos "K6%" ) ))
( "woodcocks"  n ( w u1 d k o k s ) ((pos "Kj%" ) ))
( "woodcraft"  n ( w u1 d k r aa f t ) ((pos "M6%" ) ))
( "woodcrafts"  n ( w u1 d k r aa f t s ) ((pos "Mj$" ) ))
( "woodcut"  n ( w u1 d k uh t ) ((pos "K6%" ) ))
( "woodcuts"  n ( w u1 d k uh t s ) ((pos "Kj%" ) ))
( "woodcutter"  n ( w u1 d k uh t @ r ) ((pos "K6%" ) ))
( "woodcutters"  n ( w u1 d k uh t @ z ) ((pos "Kj%" ) ))
( "wooded"  j ( w u1 d i d ) ((pos "OA%" ) ))
( "wooden"  j ( w u1 d n ) ((pos "OA%" ) ))
( "woodenheaded"  j ( w u2 d n h e1 d i d ) ((pos "OA%" ) ))
( "woodier"  j ( w u1 d i@ r ) ((pos "Or%" ) ))
( "woodiest"  j ( w u1 d i i s t ) ((pos "Os%" ) ))
( "woodland"  n ( w u1 d l @ n d ) ((pos "M6%" ) ))
( "woodlands"  n ( w u1 d l @ n d z ) ((pos "Mj%" ) ))
( "woodlice"  n ( w u1 d l ai s ) ((pos "Kj%" ) ))
( "woodlouse"  n ( w u1 d l au s ) ((pos "Ki%" ) ))
( "woodman"  n ( w u1 d m @ n ) ((pos "Ki%" ) ))
( "woodmen"  n ( w u1 d m e n ) ((pos "Kj%" ) ))
( "woodpecker"  n ( w u1 d p e k @ r ) ((pos "K6%" ) ))
( "woodpeckers"  n ( w u1 d p e k @ z ) ((pos "Kj%" ) ))
( "woodpile"  n ( w u1 d p ai l ) ((pos "K6%" ) ))
( "woodpiles"  n ( w u1 d p ai l z ) ((pos "Kj%" ) ))
( "woods"  n ( w u1 d z ) ((pos "Mj%" ) ))
( "woodshed"  n ( w u1 d sh e d ) ((pos "K6%" ) ))
( "woodsheds"  n ( w u1 d sh e d z ) ((pos "Kj%" ) ))
( "woodsman"  n ( w u1 d z m @ n ) ((pos "Ki%" ) ))
( "woodsmen"  n ( w u1 d z m e n ) ((pos "Kj%" ) ))
( "woodwind"  n ( w u1 d w i n d ) ((pos "L@%" ) ))
( "woodwork"  n ( w u1 d w @@ k ) ((pos "L@%" ) ))
( "woodworm"  n ( w u1 d w @@ m ) ((pos "L@%" ) ))
( "woody"  j ( w u1 d ii ) ((pos "OD%" ) ))
( "wooed"  v ( w uu1 d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wooer"  n ( w uu1 @ r ) ((pos "K6%" ) ))
( "wooers"  n ( w uu1 @ z ) ((pos "Kj%" ) ))
( "woof"  n ( w uu1 f ) ((pos "K6$" ) ))
( "woofer"  n ( w u1 f @ r ) ((pos "K6$" ) ))
( "woofers"  n ( w u1 f @ z ) ((pos "Kj$" ) ))
( "woofs"  n ( w uu1 f s ) ((pos "Kj$" ) ))
( "wooing"  v ( w uu1 i ng ) ((pos "Hb%" ) (subcat )))
( "wool"  n ( w u1 l ) ((pos "M6%" ) ))
( "woolen"  j ( w u1 l @ n ) ((pos "OA%" ) ))
( "woolens"  n ( w u1 l @ n z ) ((pos "Kj%" ) ))
( "woolgathering"  n ( w u1 l g a dh @ r i ng ) ((pos "L@%" "OA%" ) ))
( "woolier"  j ( w u1 l i@ r ) ((pos "Or$" ) ))
( "woolies"  n ( w u1 l i z ) ((pos "Kj%" ) ))
( "wooliest"  j ( w u1 l i i s t ) ((pos "Os$" ) ))
( "woollen"  j ( w u1 l @ n ) ((pos "OA%" ) ))
( "woollens"  n ( w u1 l @ n z ) ((pos "Kj%" ) ))
( "woollier"  j ( w u1 l i@ r ) ((pos "Or%" ) ))
( "woolliest"  j ( w u1 l i i s t ) ((pos "Os%" ) ))
( "woolly"  j ( w u1 l ii ) ((pos "OD%" ) ))
( "wools"  n ( w u1 l z ) ((pos "Mj%" ) ))
( "wooly"  n ( w u1 l ii ) ((pos "K8$" "OD$" ) ))
( "woos"  v ( w uu1 z ) ((pos "Ha%" ) (subcat )))
( "word"  v ( w @@1 d ) ((pos "H0%" "K6*" ) (subcat )))
( "word-division"  n ( w @@1 d - d i v i zh @ n ) ((pos "L@%" ) ))
( "word-painter"  n ( w @@1 d - p ei n t @ r ) ((pos "K6$" ) ))
( "word-painters"  n ( w @@1 d - p ei n t @ z ) ((pos "Kj$" ) ))
( "word-perfect"  j ( w @@2 d - p @@1 f e k t ) ((pos "OA%" ) ))
( "word-picture"  n ( w @@1 d - p i k ch @ r ) ((pos "K6%" ) ))
( "word-pictures"  n ( w @@1 d - p i k ch @ z ) ((pos "Kj%" ) ))
( "word-splitting"  n ( w @@1 d - s p l i t i ng ) ((pos "L@%" ) ))
( "wordbook"  n ( w @@1 d b u k ) ((pos "K6%" ) ))
( "wordbooks"  n ( w @@1 d b u k s ) ((pos "Kj%" ) ))
( "worded"  v ( w @@1 d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wordier"  j ( w @@1 d i@ r ) ((pos "Or%" ) ))
( "wordiest"  j ( w @@1 d i i s t ) ((pos "Os%" ) ))
( "wordily"  a ( w @@1 d i l ii ) ((pos "Pu%" ) ))
( "wordiness"  n ( w @@1 d i n i s ) ((pos "L@%" ) ))
( "wording"  v ( w @@1 d i ng ) ((pos "Hb%" "Ki%" ) (subcat )))
( "wordless"  j ( w @@1 d l @ s ) ((pos "OA%" ) ))
( "words"  v ( w @@1 d z ) ((pos "Ha%" "Kj*" ) (subcat )))
( "wordy"  j ( w @@1 d ii ) ((pos "OD%" ) ))
( "wore"  v ( w oo1 r ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15A" "15B" )))
( "work"  v ( w @@1 k ) ((pos "J0*" "M6%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" "14" "15A" "15B" )))
( "work-in"  n ( w @@1 k - i n ) ((pos "K6%" ) ))
( "work-ins"  n ( w @@1 k - i n z ) ((pos "Kj$" ) ))
( "work-out"  n ( w @@1 k - au t ) ((pos "K6%" ) ))
( "work-outs"  n ( w @@1 k - au t s ) ((pos "Kj$" ) ))
( "work-studies"  n ( w @@1 k - s t uh d i z ) ((pos "Kj%" ) ))
( "work-study"  n ( w @@1 k - s t uh d ii ) ((pos "K8%" ) ))
( "workable"  j ( w @@1 k @ b l ) ((pos "OA%" ) ))
( "workaday"  j ( w @@1 k @ d ei ) ((pos "OA%" ) ))
( "workbag"  n ( w @@1 k b a g ) ((pos "K6%" ) ))
( "workbags"  n ( w @@1 k b a g z ) ((pos "Kj%" ) ))
( "workbasket"  n ( w @@1 k b aa s k i t ) ((pos "K6%" ) ))
( "workbaskets"  n ( w @@1 k b aa s k i t s ) ((pos "Kj%" ) ))
( "workbench"  n ( w @@1 k b e n ch ) ((pos "K7%" ) ))
( "workbenches"  n ( w @@1 k b e n ch i z ) ((pos "Kj%" ) ))
( "workbook"  n ( w @@1 k b u k ) ((pos "K6%" ) ))
( "workbooks"  n ( w @@1 k b u k s ) ((pos "Kj%" ) ))
( "workbox"  n ( w @@1 k b o k s ) ((pos "K7%" ) ))
( "workboxes"  n ( w @@1 k b o k s i z ) ((pos "Kj%" ) ))
( "workday"  n ( w @@1 k d ei ) ((pos "K6%" ) ))
( "workdays"  n ( w @@1 k d ei z ) ((pos "Kj%" ) ))
( "worked"  v ( w @@1 k t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" "14" "15A" "15B" )))
( "worker"  n ( w @@1 k @ r ) ((pos "K6%" ) ))
( "workers"  n ( w @@1 k @ z ) ((pos "Kj%" ) ))
( "workhouse"  n ( w @@1 k h au s ) ((pos "Ki%" ) ))
( "workhouses"  n ( w @@1 k h au z i z ) ((pos "Kj%" ) ))
( "working"  v ( w @@1 k i ng ) ((pos "Jb%" "K6%" "OA%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" "14" "15A" "15B" )))
( "working-class"  j ( w @@1 k i ng - k l aa s ) ((pos "OA%" ) ))
( "working-out"  n ( w @@2 k i ng - au1 t ) ((pos "L@%" ) ))
( "workings"  n ( w @@1 k i ng z ) ((pos "Kj%" ) ))
( "workman"  n ( w @@1 k m @ n ) ((pos "Ki%" ) ))
( "workmanlike"  j ( w @@1 k m @ n l ai k ) ((pos "OA%" ) ))
( "workmanship"  n ( w @@1 k m @ n sh i p ) ((pos "L@%" ) ))
( "workmate"  n ( w @@1 k m ei t ) ((pos "K6%" ) ))
( "workmates"  n ( w @@1 k m ei t s ) ((pos "Kj%" ) ))
( "workmen"  n ( w @@1 k m e n ) ((pos "Kj%" ) ))
( "workroom"  n ( w @@1 k r uu m ) ((pos "K6%" ) ))
( "workrooms"  n ( w @@1 k r uu m z ) ((pos "Kj%" ) ))
( "works"  v ( w @@1 k s ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" "14" "15A" "15B" )))
( "workshop"  n ( w @@1 k sh o p ) ((pos "K6%" ) ))
( "workshops"  n ( w @@1 k sh o p s ) ((pos "Kj%" ) ))
( "workshy"  j ( w @@1 k sh ai ) ((pos "OA%" ) ))
( "worktable"  n ( w @@1 k t ei b l ) ((pos "K6%" ) ))
( "worktables"  n ( w @@1 k t ei b l z ) ((pos "Kj%" ) ))
( "world"  n ( w @@1 l d ) ((pos "K6*" ) ))
( "world-weary"  j ( w @@1 l d - w i@ r ii ) ((pos "OA%" ) ))
( "worldlier"  j ( w @@1 l d l i@ r ) ((pos "Or%" ) ))
( "worldliest"  j ( w @@1 l d l i i s t ) ((pos "Os%" ) ))
( "worldliness"  n ( w @@1 l d l i n i s ) ((pos "L@%" ) ))
( "worldly"  j ( w @@1 l d l ii ) ((pos "OD%" ) ))
( "worlds"  n ( w @@1 l d z ) ((pos "Kj%" ) ))
( "worldwide"  j ( w @@1 l d w ai d ) ((pos "OA%" ) ))
( "worm"  v ( w @@1 m ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "worm-eaten"  j ( w @@1 m - ii t n ) ((pos "OA%" ) ))
( "worm-gear"  n ( w @@1 m - g i@ r ) ((pos "K6%" ) ))
( "worm-gears"  n ( w @@1 m - g i@ z ) ((pos "Kj%" ) ))
( "wormcast"  n ( w @@1 m k aa s t ) ((pos "K6%" ) ))
( "wormcasts"  n ( w @@1 m k aa s t s ) ((pos "Kj%" ) ))
( "wormed"  v ( w @@1 m d ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "wormhole"  n ( w @@1 m h ou l ) ((pos "K6%" ) ))
( "wormholes"  n ( w @@1 m h ou l z ) ((pos "Kj%" ) ))
( "wormier"  j ( w @@1 m i@ r ) ((pos "Or$" ) ))
( "wormiest"  j ( w @@1 m i i s t ) ((pos "Os$" ) ))
( "worming"  v ( w @@1 m i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "worms"  v ( w @@1 m z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "wormwood"  n ( w @@1 m w u d ) ((pos "L@%" ) ))
( "wormy"  j ( w @@1 m ii ) ((pos "OD%" ) ))
( "worn"  v ( w oo1 n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "2D" "6A" "15A" "15B" )))
( "worried"  v ( w uh1 r i d ) ((pos "Jc%" "Jd%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "worries"  v ( w uh1 r i z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "worrisome"  j ( w uh1 r i s @ m ) ((pos "OA%" ) ))
( "worry"  v ( w uh1 r ii ) ((pos "J3%" "M8%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "worrying"  v ( w uh1 r i i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "2B" "2C" "3A" "6A" "14" "15A" "15B" "17" )))
( "worryingly"  a ( w uh1 r i i ng l ii ) ((pos "Pu%" ) ))
( "worse"  n ( w @@1 s ) ((pos "L@%" "Or%" "Pu%" ) ))
( "worsen"  v ( w @@1 s @ n ) ((pos "J0%" ) (subcat "2A" )))
( "worsened"  v ( w @@1 s @ n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" )))
( "worsening"  v ( w @@1 s @ n i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "worsens"  v ( w @@1 s @ n z ) ((pos "Ja%" ) (subcat "2A" )))
( "worship"  v ( w @@1 sh i p ) ((pos "J4%" "L@%" ) (subcat "2A" "2B" )))
( "worshipful"  j ( w @@1 sh i p f @ l ) ((pos "OA%" ) ))
( "worshipped"  v ( w @@1 sh i p t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" )))
( "worshipper"  n ( w @@1 sh i p @ r ) ((pos "K6%" ) ))
( "worshippers"  n ( w @@1 sh i p @ z ) ((pos "Kj%" ) ))
( "worshipping"  v ( w @@1 sh i p i ng ) ((pos "Jb%" ) (subcat "2A" "2B" )))
( "worships"  v ( w @@1 sh i p s ) ((pos "Ja%" ) (subcat "2A" "2B" )))
( "worst"  v ( w @@1 s t ) ((pos "H0$" "L@%" "Os%" "Pu%" ) (subcat )))
( "worsted"  v ( w @@1 s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "worsted"  n ( w u1 s t i d ) ((pos "L@$" ) ))
( "worsting"  v ( w @@1 s t i ng ) ((pos "Hb$" ) (subcat )))
( "worsts"  v ( w @@1 s t s ) ((pos "Ha$" ) (subcat )))
( "worth"  n ( w @@1 th ) ((pos "L@%" "Op%" ) ))
( "worthier"  j ( w @@1 dh i@ r ) ((pos "Or%" ) ))
( "worthies"  n ( w @@1 dh i z ) ((pos "Kj%" ) ))
( "worthiest"  j ( w @@1 dh i i s t ) ((pos "Os%" ) ))
( "worthily"  a ( w @@1 dh i l ii ) ((pos "Pu%" ) ))
( "worthiness"  n ( w @@1 dh i n @ s ) ((pos "L@%" ) ))
( "worthless"  j ( w @@1 th l @ s ) ((pos "OA%" ) ))
( "worthlessly"  a ( w @@1 th l @ s l ii ) ((pos "Pu%" ) ))
( "worthlessness"  n ( w @@1 th l @ s n @ s ) ((pos "L@%" ) ))
( "worthwhile"  j ( w @@1 th w ai l ) ((pos "OA%" ) ))
( "worthy"  n ( w @@1 dh ii ) ((pos "K8%" "OD%" ) ))
( "wot"  v ( w o1 t ) ((pos "He$" ) ))
( "wotcher"  uh ( w o1 ch @ r ) ((pos "W-$" ) ))
( "would"  v ( w u d ) ((pos "Gc*" "Hc%" ) (subcat "5" "6A" )))
( "would-be"  j ( w u1 d - b ii ) ((pos "Oq%" ) ))
( "wouldn't"  v ( w u1 d n t ) ((pos "Gg%" ) ))
( "wouldst"  v ( w u d s t ) ((pos "Ge$" ) ))
( "wound"  v ( w au1 n d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "6A" "14" "15A" )))
( "wound"  v ( w uu1 n d ) ((pos "H0%" "K6%" ) (subcat )))
( "wounded"  v ( w uu1 n d i d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wounding"  v ( w uu1 n d i ng ) ((pos "Hb%" ) (subcat )))
( "wounds"  v ( w uu1 n d z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "wove"  v ( w ou1 v ) ((pos "Jc%" ) (subcat "2A" "2C" "6A" "15A" )))
( "woven"  v ( w ou1 v @ n ) ((pos "Jd%" ) (subcat "2A" "2C" "6A" "15A" )))
( "wow"  n ( w au1 ) ((pos "M6$" "W-$" ) ))
( "wows"  n ( w au1 z ) ((pos "Mj$" ) ))
( "wpb"  n ( d uh2 b l y uu p ii2 b ii1 ) ((pos "Y>%" ) ))
( "wpm"  n ( d uh2 b l y uu p ii2 e1 m ) ((pos "Y]%" ) ))
( "wrack"  n ( r a1 k ) ((pos "L@%" ) ))
( "wraith"  n ( r ei1 th ) ((pos "K6%" ) ))
( "wraiths"  n ( r ei1 th s ) ((pos "Kj%" ) ))
( "wrangle"  v ( r a1 ng g l ) ((pos "I2%" "K6%" ) (subcat "2A" )))
( "wrangled"  v ( r a1 ng g l d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "wrangles"  v ( r a1 ng g l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "wrangling"  v ( r a1 ng g l i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "wrap"  v ( r a1 p ) ((pos "J4%" "K6%" ) (subcat "6A" "14" "15A" )))
( "wrapped"  v ( r a1 p t ) ((pos "Jc%" "Jd%" ) (subcat "6A" "14" "15A" )))
( "wrapper"  n ( r a1 p @ r ) ((pos "K6%" ) ))
( "wrappers"  n ( r a1 p @ z ) ((pos "Kj%" ) ))
( "wrapping"  v ( r a1 p i ng ) ((pos "Jb%" "M6%" ) (subcat "6A" "14" "15A" )))
( "wrappings"  n ( r a1 p i ng z ) ((pos "Mj%" ) ))
( "wraps"  v ( r a1 p s ) ((pos "Ja%" "Kj%" ) (subcat "6A" "14" "15A" )))
( "wrath"  n ( r o1 th ) ((pos "L@%" ) ))
( "wrathful"  j ( r o1 th f @ l ) ((pos "OA%" ) ))
( "wrathfully"  a ( r o1 th f @ l ii ) ((pos "Pu%" ) ))
( "wreak"  v ( r ii1 k ) ((pos "H0%" ) (subcat "6A" )))
( "wreaked"  v ( r ii1 k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" )))
( "wreaking"  v ( r ii1 k i ng ) ((pos "Hb%" ) (subcat "6A" )))
( "wreaks"  v ( r ii1 k s ) ((pos "Ha%" ) (subcat "6A" )))
( "wreath"  n ( r ii1 th ) ((pos "K6%" ) ))
( "wreathe"  v ( r ii1 dh ) ((pos "J2%" ) (subcat "2A" "2C" "6A" )))
( "wreathed"  v ( r ii1 dh d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "wreathes"  v ( r ii1 dh z ) ((pos "Ja%" ) (subcat "2A" "2C" "6A" )))
( "wreathing"  v ( r ii1 dh i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "wreaths"  n ( r ii1 dh z ) ((pos "Kj%" ) ))
( "wreck"  v ( r e1 k ) ((pos "H0%" "M6%" ) (subcat )))
( "wreckage"  n ( r e1 k i jh ) ((pos "L@%" ) ))
( "wrecked"  v ( r e1 k t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wrecker"  n ( r e1 k @ r ) ((pos "K6%" ) ))
( "wreckers"  n ( r e1 k @ z ) ((pos "Kj%" ) ))
( "wrecking"  v ( r e1 k i ng ) ((pos "Hb%" ) (subcat )))
( "wrecks"  v ( r e1 k s ) ((pos "Ha%" "Mj%" ) (subcat )))
( "wren"  n ( r e1 n ) ((pos "K6%" ) ))
( "wrench"  v ( r e1 n ch ) ((pos "H1%" "K7%" ) (subcat "6A" "15A" )))
( "wrenched"  v ( r e1 n ch t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "wrenches"  v ( r e1 n ch i z ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "wrenching"  v ( r e1 n ch i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "wrens"  n ( r e1 n z ) ((pos "Kj%" ) ))
( "wrest"  v ( r e1 s t ) ((pos "H0$" ) (subcat )))
( "wrested"  v ( r e1 s t i d ) ((pos "Hc$" "Hd$" ) (subcat )))
( "wresting"  v ( r e1 s t i ng ) ((pos "Hb$" ) (subcat )))
( "wrestle"  v ( r e1 s l ) ((pos "I2%" "K6%" ) (subcat "2A" "2C" )))
( "wrestled"  v ( r e1 s l d ) ((pos "Ic%" "Id%" ) (subcat "2A" "2C" )))
( "wrestler"  n ( r e1 s l @ r ) ((pos "K6%" ) ))
( "wrestlers"  n ( r e1 s l @ z ) ((pos "Kj%" ) ))
( "wrestles"  v ( r e1 s l z ) ((pos "Ia%" "Kj%" ) (subcat "2A" "2C" )))
( "wrestling"  v ( r e1 s l i ng ) ((pos "Ib%" ) (subcat "2A" "2C" )))
( "wrests"  v ( r e1 s t s ) ((pos "Ha$" ) (subcat )))
( "wretch"  n ( r e1 ch ) ((pos "K7%" ) ))
( "wretched"  j ( r e1 ch i d ) ((pos "OA%" ) ))
( "wretchedly"  a ( r e1 ch i d l ii ) ((pos "Pu%" ) ))
( "wretchedness"  n ( r e1 ch i d n @ s ) ((pos "L@%" ) ))
( "wretches"  n ( r e1 ch i z ) ((pos "Kj%" ) ))
( "wrick"  v ( r i1 k ) ((pos "H0$" "K6$" ) (subcat )))
( "wricked"  v ( r i1 k t ) ((pos "Hc$" "Hd$" ) (subcat )))
( "wricking"  v ( r i1 k i ng ) ((pos "Hb$" ) (subcat )))
( "wricks"  v ( r i1 k s ) ((pos "Ha$" "Kj$" ) (subcat )))
( "wrier"  j ( r ai1 @ r ) ((pos "Or$" ) ))
( "wriest"  j ( r ai1 i s t ) ((pos "Os$" ) ))
( "wriggle"  v ( r i1 g l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "wriggled"  v ( r i1 g l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "wriggler"  n ( r i1 g l @ r ) ((pos "K6%" ) ))
( "wrigglers"  n ( r i1 g l @ z ) ((pos "Kj%" ) ))
( "wriggles"  v ( r i1 g l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "wriggling"  v ( r i1 g l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "3A" "6A" "15B" )))
( "wright"  n ( r ai1 t ) ((pos "K6$" ) ))
( "wrights"  n ( r ai1 t s ) ((pos "Kj$" ) ))
( "wring"  v ( r i1 ng ) ((pos "H5$" "K6$" ) (subcat "6A" "14" )))
( "wringer"  n ( r i1 ng @ r ) ((pos "K6$" ) ))
( "wringers"  n ( r i1 ng @ z ) ((pos "Kj$" ) ))
( "wringing"  v ( r i1 ng i ng ) ((pos "Hb$" ) (subcat "6A" "14" )))
( "wrings"  v ( r i1 ng z ) ((pos "Ha$" "Kj$" ) (subcat "6A" "14" )))
( "wrinkle"  v ( r i1 ng k l ) ((pos "J2%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "wrinkled"  v ( r i1 ng k l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "wrinkles"  v ( r i1 ng k l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "wrinklier"  j ( r i1 ng k l i@ r ) ((pos "Or$" ) ))
( "wrinkliest"  j ( r i1 ng k l i i s t ) ((pos "Os$" ) ))
( "wrinkling"  v ( r i1 ng k l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "wrinkly"  j ( r i1 ng k l ii ) ((pos "OD%" ) ))
( "wrist"  n ( r i1 s t ) ((pos "K6%" ) ))
( "wristband"  n ( r i1 s t b a n d ) ((pos "K6%" ) ))
( "wristbands"  n ( r i1 s t b a n d z ) ((pos "Kj%" ) ))
( "wristlet"  n ( r i1 s t l i t ) ((pos "K6$" ) ))
( "wristlets"  n ( r i1 s t l i t s ) ((pos "Kj$" ) ))
( "wrists"  n ( r i1 s t s ) ((pos "Kj%" ) ))
( "wristwatch"  n ( r i1 s t w o ch ) ((pos "K7%" ) ))
( "wristwatches"  n ( r i1 s t w o ch i z ) ((pos "Kj%" ) ))
( "writ"  n ( r i1 t ) ((pos "K6%" ) ))
( "writ large"  j ( r i1 t - l aa1 jh ) ((pos "OA%" ) ))
( "write"  v ( r ai1 t ) ((pos "J5%" ) (subcat "2A" "2B" "2C" "4A" "6A" "12A" "13A" )))
( "write-off"  n ( r ai1 t - o f ) ((pos "K6%" ) ))
( "write-offs"  n ( r ai1 t - o f s ) ((pos "Kj%" ) ))
( "write-up"  n ( r ai1 t - uh p ) ((pos "K6%" ) ))
( "write-ups"  n ( r ai1 t - uh p s ) ((pos "Kj%" ) ))
( "writer"  n ( r ai1 t @ r ) ((pos "K6%" ) ))
( "writers"  n ( r ai1 t @ z ) ((pos "Kj%" ) ))
( "writes"  v ( r ai1 t s ) ((pos "Ja%" ) (subcat "2A" "2B" "2C" "4A" "6A" "12A" "13A" )))
( "writhe"  v ( r ai1 dh ) ((pos "I2%" ) (subcat "2A" )))
( "writhed"  v ( r ai1 dh d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "writhes"  v ( r ai1 dh z ) ((pos "Ia%" ) (subcat "2A" )))
( "writhing"  v ( r ai1 dh i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "writing"  v ( r ai1 t i ng ) ((pos "Jb%" "M6%" ) (subcat "2A" "2B" "2C" "4A" "6A" "12A" "13A" )))
( "writing-desk"  n ( r ai1 t i ng - d e s k ) ((pos "K6%" ) ))
( "writing-desks"  n ( r ai1 t i ng - d e s k s ) ((pos "Kj%" ) ))
( "writing-ink"  n ( r ai1 t i ng - i ng k ) ((pos "K6%" ) ))
( "writing-inks"  n ( r ai1 t i ng - i ng k s ) ((pos "Kj%" ) ))
( "writing-paper"  n ( r ai1 t i ng - p ei p @ r ) ((pos "L@%" ) ))
( "writings"  n ( r ai1 t i ng z ) ((pos "Mj%" ) ))
( "writs"  n ( r i1 t s ) ((pos "Kj$" ) ))
( "written"  v ( r i1 t n ) ((pos "Jd%" ) (subcat "2A" "2B" "2C" "4A" "6A" "12A" "13A" )))
( "wrong"  v ( r o1 ng ) ((pos "H0%" "M6%" "OA%" "Pu%" ) (subcat )))
( "wrong-headed"  j ( r o1 n - h e1 d i d ) ((pos "OA%" ) ))
( "wrong-headedly"  a ( r o1 n - h e1 d i d l ii ) ((pos "Pu%" ) ))
( "wrongdoer"  n ( r o1 ng d uu @ r ) ((pos "K6%" ) ))
( "wrongdoers"  n ( r o1 ng d uu @ z ) ((pos "Kj%" ) ))
( "wrongdoing"  n ( r o1 ng d uu i ng ) ((pos "M6%" ) ))
( "wrongdoings"  n ( r o1 ng d uu i ng z ) ((pos "Mj%" ) ))
( "wronged"  v ( r o1 ng d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "wrongful"  j ( r o1 ng f @ l ) ((pos "OA%" ) ))
( "wrongfully"  a ( r o1 ng f @ l ii ) ((pos "Pu%" ) ))
( "wronging"  v ( r o1 ng i ng ) ((pos "Hb%" ) (subcat )))
( "wrongly"  a ( r o1 ng l ii ) ((pos "Pu%" ) ))
( "wrongs"  v ( r o1 ng z ) ((pos "Ha%" "Mj%" ) (subcat )))
( "wrote"  v ( r ou1 t ) ((pos "Jc%" ) (subcat "2A" "2B" "2C" "4A" "6A" "12A" "13A" )))
( "wroth"  j ( r ou1 th ) ((pos "OA$" ) ))
( "wrought"  v ( r oo1 t ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2B" "2C" "2D" "3A" "4A" "6A" "14" "15A" "15B" )))
( "wrung"  v ( r uh1 ng ) ((pos "Hc$" "Hd$" ) (subcat "6A" "14" )))
( "wry"  j ( r ai1 ) ((pos "OD%" ) ))
( "wryly"  a ( r ai1 l ii ) ((pos "Pu%" ) ))
( "wt"  n ( w ei1 t ) ((pos "Y}%" ) ))
( "wych-"  j ( w i ch - ) ((pos "U-$" ) ))
( "x"  n ( e1 k s ) ((pos "Ki$" ) ))
( "x's"  n ( e1 k s i z ) ((pos "Kj$" ) ))
( "xenophobia"  n ( z e2 n @ f ou1 b i@ ) ((pos "L@%" ) ))
( "xylophone"  n ( z ai1 l @ f ou n ) ((pos "K6%" ) ))
( "xylophones"  n ( z ai1 l @ f ou n z ) ((pos "Kj%" ) ))
( "y"  n ( w ai1 ) ((pos "Ki$" ) ))
( "y's"  n ( w ai1 z ) ((pos "Kj$" ) ))
( "yacht"  v ( y o1 t ) ((pos "I0$" "K6%" ) (subcat )))
( "yacht-club"  n ( y o1 t - k l uh b ) ((pos "K6%" ) ))
( "yacht-clubs"  n ( y o1 t - k l uh b z ) ((pos "Kj%" ) ))
( "yachted"  v ( y o1 t i d ) ((pos "Ic$" "Id$" ) (subcat )))
( "yachting"  v ( y o1 t i ng ) ((pos "Ib$" "L@%" ) (subcat )))
( "yachts"  v ( y o1 t s ) ((pos "Ia$" "Kj%" ) (subcat )))
( "yachtsman"  n ( y o1 t s m @ n ) ((pos "Ki%" ) ))
( "yachtsmen"  n ( y o1 t s m @ n ) ((pos "Kj%" ) ))
( "yah"  uh ( y aa1 ) ((pos "W-$" ) ))
( "yahoo"  n ( y aa1 h uu1 ) ((pos "K6$" ) ))
( "yahoos"  n ( y aa1 h uu1 z ) ((pos "Kj$" ) ))
( "yak"  n ( y a1 k ) ((pos "K6$" ) ))
( "yaks"  n ( y a1 k s ) ((pos "Kj$" ) ))
( "yam"  n ( y a1 m ) ((pos "K6$" ) ))
( "yammer"  v ( y a1 m @ r ) ((pos "I0$" ) ))
( "yammered"  v ( y a1 m @ d ) ((pos "Ic$" "Id$" ) ))
( "yammering"  v ( y a1 m @ r i ng ) ((pos "Ib$" ) ))
( "yammers"  v ( y a1 m @ z ) ((pos "Ia$" ) ))
( "yams"  n ( y a1 m z ) ((pos "Kj$" ) ))
( "yank"  v ( y a1 ng k ) ((pos "H0%" "K6%" ) (subcat "6A" "15A" )))
( "yanked"  v ( y a1 ng k t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15A" )))
( "yanking"  v ( y a1 ng k i ng ) ((pos "Hb%" ) (subcat "6A" "15A" )))
( "yanks"  v ( y a1 ng k s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15A" )))
( "yap"  v ( y a1 p ) ((pos "I4%" "K6%" ) (subcat )))
( "yapped"  v ( y a1 p t ) ((pos "Ic%" "Id%" ) (subcat )))
( "yapping"  v ( y a1 p i ng ) ((pos "Ib%" ) (subcat )))
( "yaps"  v ( y a1 p s ) ((pos "Ia%" "Kj%" ) (subcat )))
( "yard"  n ( y aa1 d ) ((pos "K6%" ) ))
( "yard-measure"  n ( y aa1 d - m e zh @ r ) ((pos "K6$" ) ))
( "yard-measures"  n ( y aa1 d - m e zh @ z ) ((pos "Kj$" ) ))
( "yardarm"  n ( y aa1 d aa m ) ((pos "K6%" ) ))
( "yardarms"  n ( y aa1 d aa m z ) ((pos "Kj%" ) ))
( "yards"  n ( y aa1 d z ) ((pos "Kj%" ) ))
( "yardstick"  n ( y aa1 d s t i k ) ((pos "K6%" ) ))
( "yardsticks"  n ( y aa1 d s t i k s ) ((pos "Kj%" ) ))
( "yarn"  v ( y aa1 n ) ((pos "I0$" "M6%" ) (subcat "2A" )))
( "yarned"  v ( y aa1 n d ) ((pos "Ic$" "Id$" ) (subcat "2A" )))
( "yarning"  v ( y aa1 n i ng ) ((pos "Ib$" ) (subcat "2A" )))
( "yarns"  v ( y aa1 n z ) ((pos "Ia$" "Mj%" ) (subcat "2A" )))
( "yarrow"  n ( y a1 r ou ) ((pos "L@$" ) ))
( "yashmak"  n ( y a1 sh m a k ) ((pos "K6$" ) ))
( "yashmaks"  n ( y a1 sh m a k s ) ((pos "Kj$" ) ))
( "yaw"  v ( y oo1 ) ((pos "I0$" "K6$" ) ))
( "yawed"  v ( y oo1 d ) ((pos "Ic$" "Id$" ) ))
( "yawing"  v ( y oo1 i ng ) ((pos "Ib$" ) ))
( "yawl"  n ( y oo1 l ) ((pos "K6$" ) ))
( "yawls"  n ( y oo1 l z ) ((pos "Kj$" ) ))
( "yawn"  v ( y oo1 n ) ((pos "I0%" "K6%" ) (subcat "2A" )))
( "yawned"  v ( y oo1 n d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "yawning"  v ( y oo1 n i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "yawns"  v ( y oo1 n z ) ((pos "Ia%" "Kj%" ) (subcat "2A" )))
( "yaws"  v ( y oo1 z ) ((pos "Ia$" "Kj$" ) ))
( "ye"  prp ( y ii ) ((pos "Qx$" "R-$" ) ))
( "yea"  n ( y ei1 ) ((pos "K6$" "Pu$" "W-$" ) ))
( "yeah"  a ( y e@1 ) ((pos "Pu$" ) ))
( "year"  n ( y @@1 r ) ((pos "K6*" ) ))
( "yearbook"  n ( y @@1 b u k ) ((pos "K6%" ) ))
( "yearbooks"  n ( y @@1 b u k s ) ((pos "Kj%" ) ))
( "yearling"  n ( y @@1 l i ng ) ((pos "K6%" ) ))
( "yearlings"  n ( y @@1 l i ng z ) ((pos "Kj%" ) ))
( "yearlong"  j ( y @@1 l o ng ) ((pos "OA%" ) ))
( "yearly"  j ( y @@1 l ii ) ((pos "OA%" "Pu%" ) ))
( "yearn"  v ( y @@1 n ) ((pos "I0%" ) (subcat "3A" )))
( "yearned"  v ( y @@1 n d ) ((pos "Ic%" "Id%" ) (subcat "3A" )))
( "yearning"  v ( y @@1 n i ng ) ((pos "Ib%" "K6%" ) (subcat "3A" )))
( "yearningly"  a ( y @@1 n i ng l ii ) ((pos "Pu%" ) ))
( "yearnings"  n ( y @@1 n i ng z ) ((pos "Kj%" ) ))
( "yearns"  v ( y @@1 n z ) ((pos "Ia%" ) (subcat "3A" )))
( "years"  n ( y @@1 z ) ((pos "Kj*" ) ))
( "yeas"  n ( y ei1 z ) ((pos "Kj$" ) ))
( "yeast"  n ( y ii1 s t ) ((pos "L@%" ) ))
( "yeastier"  j ( y ii1 s t i@ r ) ((pos "Or$" ) ))
( "yeastiest"  j ( y ii1 s t i i s t ) ((pos "Os$" ) ))
( "yeasty"  j ( y ii1 s t ii ) ((pos "OD%" ) ))
( "yell"  v ( y e1 l ) ((pos "J0%" "K6%" ) (subcat "2A" "2C" "6A" )))
( "yelled"  v ( y e1 l d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "2C" "6A" )))
( "yelling"  v ( y e1 l i ng ) ((pos "Jb%" ) (subcat "2A" "2C" "6A" )))
( "yellow"  v ( y e1 l ou ) ((pos "J0$" "M6%" "OC%" ) (subcat "2A" )))
( "yellow-bellied"  j ( y e2 l ou - b e1 l i d ) ((pos "OA%" ) ))
( "yellow-flag"  n ( y e1 l ou - f l a g ) ((pos "K6$" ) ))
( "yellow-flags"  n ( y e1 l ou - f l a g z ) ((pos "Kj$" ) ))
( "yellowed"  v ( y e1 l ou d ) ((pos "Jc$" "Jd%" ) (subcat "2A" )))
( "yellower"  j ( y e1 l ou @ r ) ((pos "Or%" ) ))
( "yellowest"  j ( y e1 l ou i s t ) ((pos "Os%" ) ))
( "yellowing"  v ( y e1 l ou i ng ) ((pos "Jb%" ) (subcat "2A" )))
( "yellowish"  j ( y e1 l ou i sh ) ((pos "OA%" ) ))
( "yellowness"  n ( y e1 l ou n @ s ) ((pos "L@%" ) ))
( "yellows"  v ( y e1 l ou z ) ((pos "Ja$" "Mj%" ) (subcat "2A" )))
( "yells"  v ( y e1 l z ) ((pos "Ja%" "Kj%" ) (subcat "2A" "2C" "6A" )))
( "yelp"  v ( y e1 l p ) ((pos "I0%" ) (subcat )))
( "yelped"  v ( y e1 l p t ) ((pos "Ic%" "Id%" ) (subcat )))
( "yelping"  v ( y e1 l p i ng ) ((pos "Ib%" ) (subcat )))
( "yelps"  v ( y e1 l p s ) ((pos "Ia%" ) (subcat )))
( "yen"  v ( y e1 n ) ((pos "I4$" "K9%" ) ))
( "yenned"  v ( y e1 n d ) ((pos "Ic$" "Id$" ) ))
( "yenning"  v ( y e1 n i ng ) ((pos "Ib$" ) ))
( "yens"  v ( y e1 n z ) ((pos "Ia$" ) ))
( "yeoman"  n ( y ou1 m @ n ) ((pos "Ki%" ) ))
( "yeomanries"  n ( y ou1 m @ n r i z ) ((pos "Kj$" ) ))
( "yeomanry"  n ( y ou1 m @ n r ii ) ((pos "K8%" ) ))
( "yeomen"  n ( y ou1 m @ n ) ((pos "Kj%" ) ))
( "yes"  n ( y e1 s ) ((pos "K7%" "W-*" ) ))
( "yeses"  n ( y e1 s i z ) ((pos "Kj$" ) ))
( "yester-"  j ( y e1 s t @ r - ) ((pos "U-%" ) ))
( "yesterday"  n ( y e1 s t @ d ii ) ((pos "K6%" "Pu%" ) ))
( "yesterdays"  n ( y e1 s t @ d i z ) ((pos "Kj%" ) ))
( "yet"  a ( y e t ) ((pos "Pu*" "V-*" ) ))
( "yeti"  n ( y e1 t ii ) ((pos "K6$" ) ))
( "yetis"  n ( y e1 t i z ) ((pos "Kj$" ) ))
( "yew"  n ( y uu1 ) ((pos "M6%" ) ))
( "yew-tree"  n ( y uu1 - t r ii ) ((pos "K6%" ) ))
( "yew-trees"  n ( y uu1 - t r ii z ) ((pos "Kj%" ) ))
( "yews"  n ( y uu1 z ) ((pos "Mj%" ) ))
( "yield"  v ( y ii1 l d ) ((pos "J0%" "M6%" ) (subcat "2A" "3A" "6A" "15A" )))
( "yielded"  v ( y ii1 l d i d ) ((pos "Jc%" "Jd%" ) (subcat "2A" "3A" "6A" "15A" )))
( "yielding"  v ( y ii1 l d i ng ) ((pos "Jb%" "OA%" ) (subcat "2A" "3A" "6A" "15A" )))
( "yieldingly"  a ( y i@1 l d i ng l ii ) ((pos "Pu%" ) ))
( "yields"  v ( y ii1 l d z ) ((pos "Ja%" "Mj%" ) (subcat "2A" "3A" "6A" "15A" )))
( "yippee"  uh ( y i1 p ii ) ((pos "W-$" ) ))
( "yo-heave-ho"  uh ( y ou2 - h ii1 v - h ou ) ((pos "W-$" ) ))
( "yo-yo"  n ( y ou1 - y ou ) ((pos "K6%" ) ))
( "yo-yos"  n ( y ou1 - y ou z ) ((pos "Kj%" ) ))
( "yob"  n ( y o1 b ) ((pos "K6%" ) ))
( "yobo"  n ( y o1 b ou ) ((pos "K6$" ) ))
( "yobos"  n ( y o1 b ou z ) ((pos "Kj$" ) ))
( "yobs"  n ( y o1 b z ) ((pos "Kj%" ) ))
( "yodel"  v ( y ou1 d l ) ((pos "J4%" "K6%" ) ))
( "yodelled"  v ( y ou1 d l d ) ((pos "Jc%" "Jd%" ) ))
( "yodeller"  n ( y ou1 d l @ r ) ((pos "K6%" ) ))
( "yodellers"  n ( y ou1 d l @ z ) ((pos "Kj%" ) ))
( "yodelling"  v ( y ou1 d @ l i ng ) ((pos "Jb%" ) ))
( "yodels"  v ( y ou1 d l z ) ((pos "Ja%" "Kj%" ) ))
( "yoga"  n ( y ou1 g @ ) ((pos "L@%" ) ))
( "yoghourt"  n ( y o1 g @ t ) ((pos "M6%" ) ))
( "yoghourts"  n ( y o1 g @ t s ) ((pos "Mj%" ) ))
( "yoghurt"  n ( y o1 g @ t ) ((pos "M6%" ) ))
( "yoghurts"  n ( y o1 g @ t s ) ((pos "Mj%" ) ))
( "yogi"  n ( y ou1 g ii ) ((pos "K6%" ) ))
( "yogis"  n ( y ou1 g i z ) ((pos "Kj%" ) ))
( "yogurt"  n ( y o1 g @ t ) ((pos "M6%" ) ))
( "yogurts"  n ( y o1 g @ t s ) ((pos "Mj%" ) ))
( "yoke"  v ( y ou1 k ) ((pos "J2%" "K6%" ) (subcat "6A" )))
( "yoked"  v ( y ou1 k t ) ((pos "Jc%" "Jd%" ) (subcat "6A" )))
( "yokel"  n ( y ou1 k l ) ((pos "K6%" ) ))
( "yokels"  n ( y ou1 k l z ) ((pos "Kj%" ) ))
( "yokes"  v ( y ou1 k s ) ((pos "Ja%" "Kj%" ) (subcat "6A" )))
( "yoking"  v ( y ou1 k i ng ) ((pos "Jb%" ) (subcat "6A" )))
( "yolk"  n ( y ou1 k ) ((pos "M6%" ) ))
( "yolks"  n ( y ou1 k s ) ((pos "Mj%" ) ))
( "yon"  j ( y o1 n ) ((pos "OA$" "Pu$" ) ))
( "yonder"  j ( y o1 n d @ r ) ((pos "OA%" "Pu%" ) ))
( "yore"  n ( y oo1 r ) ((pos "L@$" ) ))
( "you"  prp ( y uu ) ((pos "Qx*" ) ))
( "you'd"  v ( y uu d ) ((pos "Gf%" ) ))
( "you'll"  v ( y uu l ) ((pos "Gf%" ) ))
( "you're"  v ( y u@ r ) ((pos "Gf%" ) ))
( "you've"  v ( y uu v ) ((pos "Gf%" ) ))
( "young"  n ( y uh1 ng ) ((pos "L@%" "OC*" ) ))
( "younger"  j ( y uh1 ng g @ r ) ((pos "Or%" ) ))
( "youngest"  j ( y uh1 ng g i s t ) ((pos "Os%" ) ))
( "youngish"  j ( y uh1 ng i sh ) ((pos "OA%" ) ))
( "youngster"  n ( y uh1 ng s t @ r ) ((pos "K6%" ) ))
( "youngsters"  n ( y uh1 ng s t @ z ) ((pos "Kj%" ) ))
( "your"  j ( y oo1 r ) ((pos "OA*" ) ))
( "yours"  j ( y oo z ) ((pos "Op%" "Qx%" ) ))
( "yourself"  prp ( y oo1 s e1 l f ) ((pos "Qx%" ) ))
( "yourselves"  prp ( y oo1 s e1 l v z ) ((pos "Qx%" ) ))
( "youth"  n ( y uu1 th ) ((pos "M6%" ) ))
( "youthful"  j ( y uu1 th f @ l ) ((pos "OA%" ) ))
( "youthfully"  a ( y uu1 th f @ l ii ) ((pos "Pu%" ) ))
( "youthfulness"  n ( y uu1 th f @ l n @ s ) ((pos "L@%" ) ))
( "youths"  n ( y uu1 dh z ) ((pos "Mj%" ) ))
( "yowl"  v ( y au1 l ) ((pos "I0$" ) ))
( "yowled"  v ( y au1 l d ) ((pos "Ic$" "Id$" ) ))
( "yowling"  v ( y au1 l i ng ) ((pos "Ib$" ) ))
( "yowls"  v ( y au1 l z ) ((pos "Ia$" ) ))
( "yr"  n ( y i@1 r ) ((pos "Y>%" ) ))
( "yule"  n ( y uu1 l ) ((pos "L@%" ) ))
( "yule-log"  n ( y uu1 l - l o g ) ((pos "K6%" ) ))
( "yule-logs"  n ( y uu1 l - l o g z ) ((pos "Kj%" ) ))
( "yuletide"  n ( y uu1 l t ai d ) ((pos "M6%" ) ))
( "yuletides"  n ( y uu1 l t ai d z ) ((pos "Mj$" ) ))
( "z"  n ( z e1 d ) ((pos "Ki$" ) ))
( "z's"  n ( z e1 d z ) ((pos "Kj$" ) ))
( "zanier"  j ( z ei1 n i@ r ) ((pos "Or%" ) ))
( "zanies"  n ( z ei1 n i z ) ((pos "Kj$" ) ))
( "zaniest"  j ( z ei1 n i i s t ) ((pos "Os%" ) ))
( "zany"  n ( z ei1 n ii ) ((pos "K8$" "OD%" ) ))
( "zap"  v ( z a1 p ) ((pos "H4%" ) (subcat )))
( "zapped"  v ( z a1 p t ) ((pos "Hc%" "Hd%" ) (subcat )))
( "zapping"  v ( z a1 p i ng ) ((pos "Hb%" ) (subcat )))
( "zaps"  v ( z a1 p s ) ((pos "Ha%" ) (subcat )))
( "zeal"  n ( z ii1 l ) ((pos "L@%" ) ))
( "zealot"  n ( z e1 l @ t ) ((pos "K6%" ) ))
( "zealotry"  n ( z e1 l @ t r ii ) ((pos "L@$" ) ))
( "zealots"  n ( z e1 l @ t s ) ((pos "Kj%" ) ))
( "zealous"  j ( z e1 l @ s ) ((pos "OA%" ) ))
( "zealously"  a ( z e1 l @ s l ii ) ((pos "Pu%" ) ))
( "zebra"  n ( z e1 b r @ ) ((pos "K6%" ) ))
( "zebras"  n ( z e1 b r @ z ) ((pos "Kj%" ) ))
( "zebu"  n ( z ii1 b y uu ) ((pos "K6$" ) ))
( "zebus"  n ( z ii1 b y uu z ) ((pos "Kj$" ) ))
( "zee"  n ( z ii1 ) ((pos "K6$" ) ))
( "zees"  n ( z ii1 z ) ((pos "Kj$" ) ))
( "zenith"  n ( z e1 n i th ) ((pos "K6%" ) ))
( "zenithal"  j ( z e1 n i th @ l ) ((pos "OA$" ) ))
( "zeniths"  n ( z e1 n i th s ) ((pos "Kj$" ) ))
( "zephyr"  n ( z e1 f @ r ) ((pos "K6%" ) ))
( "zephyrs"  n ( z e1 f @ z ) ((pos "Kj%" ) ))
( "zeppelin"  n ( z e1 p @ l i n ) ((pos "K6%" ) ))
( "zeppelins"  n ( z e1 p @ l i n z ) ((pos "Kj%" ) ))
( "zero"  v ( z i@1 r ou ) ((pos "I0$" "K6%" ) (subcat )))
( "zeroed"  v ( z i@1 r ou d ) ((pos "Ic$" "Id$" ) (subcat )))
( "zeroing"  v ( z i@1 r ou i ng ) ((pos "Ib$" ) (subcat )))
( "zeros"  v ( z i@1 r ou z ) ((pos "Ia$" "Kj%" ) (subcat )))
( "zest"  n ( z e1 s t ) ((pos "L@%" ) ))
( "zestful"  j ( z e1 s t f @ l ) ((pos "OA%" ) ))
( "zestfully"  a ( z e1 s t f @ l ii ) ((pos "Pu%" ) ))
( "zigzag"  v ( z i1 g z a g ) ((pos "I4%" "K6%" "Pu%" ) ))
( "zigzagged"  v ( z i1 g z a g d ) ((pos "Ic%" "Id%" ) ))
( "zigzagging"  v ( z i1 g z a g i ng ) ((pos "Ib%" ) ))
( "zigzags"  v ( z i1 g z a g z ) ((pos "Ia%" "Kj%" ) ))
( "zinc"  n ( z i1 ng k ) ((pos "L@%" ) ))
( "zing"  n ( z i1 ng ) ((pos "L@%" ) ))
( "zinnia"  n ( z i1 n i@ ) ((pos "K6$" ) ))
( "zinnias"  n ( z i1 n i@ z ) ((pos "Kj$" ) ))
( "zip"  v ( z i1 p ) ((pos "H4%" "K6%" ) (subcat "6A" "15B" )))
( "zip code"  n ( z i1 p - k ou d ) ((pos "K6%" ) ))
( "zip codes"  n ( z i1 p - k ou d z ) ((pos "Kj%" ) ))
( "zip-fastener"  n ( z i1 p - f a s n @ r ) ((pos "K6%" ) ))
( "zip-fasteners"  n ( z i1 p - f a s n @ z ) ((pos "Kj%" ) ))
( "zipped"  v ( z i1 p t ) ((pos "Hc%" "Hd%" ) (subcat "6A" "15B" )))
( "zipper"  n ( z i1 p @ r ) ((pos "K6%" ) ))
( "zippers"  n ( z i1 p @ z ) ((pos "Kj%" ) ))
( "zipping"  v ( z i1 p i ng ) ((pos "Hb%" ) (subcat "6A" "15B" )))
( "zips"  v ( z i1 p s ) ((pos "Ha%" "Kj%" ) (subcat "6A" "15B" )))
( "zither"  n ( z i1 dh @ r ) ((pos "K6%" ) ))
( "zithers"  n ( z i1 dh @ z ) ((pos "Kj%" ) ))
( "zloty"  n ( z l o1 t ii ) ((pos "K6$" ) ))
( "zlotys"  n ( z l o1 t i z ) ((pos "Kj$" ) ))
( "zodiac"  n ( z ou1 d i a k ) ((pos "K6%" ) ))
( "zodiacs"  n ( z ou1 d i a k s ) ((pos "Kj$" ) ))
( "zombie"  n ( z o1 m b ii ) ((pos "K6%" ) ))
( "zombies"  n ( z o1 m b i z ) ((pos "Kj%" ) ))
( "zonal"  j ( z ou1 n l ) ((pos "OA%" ) ))
( "zone"  v ( z ou1 n ) ((pos "H2%" "K6%" ) (subcat )))
( "zoned"  v ( z ou1 n d ) ((pos "Hc%" "Hd%" ) (subcat )))
( "zones"  v ( z ou1 n z ) ((pos "Ha%" "Kj%" ) (subcat )))
( "zoning"  v ( z ou1 n i ng ) ((pos "Hb%" "L@%" ) (subcat )))
( "zoo"  n ( z uu1 ) ((pos "K6%" ) ))
( "zoological"  j ( z uu2 @ l o1 jh i k l ) ((pos "OA%" ) ))
( "zoologist"  n ( z uu1 o1 l @ jh i s t ) ((pos "K6%" ) ))
( "zoologists"  n ( z uu1 o1 l @ jh i s t s ) ((pos "Kj%" ) ))
( "zoology"  n ( z uu1 o1 l @ jh ii ) ((pos "L@%" ) ))
( "zoom"  v ( z uu1 m ) ((pos "I0%" "L@%" ) (subcat "2A" )))
( "zoomed"  v ( z uu1 m d ) ((pos "Ic%" "Id%" ) (subcat "2A" )))
( "zooming"  v ( z uu1 m i ng ) ((pos "Ib%" ) (subcat "2A" )))
( "zooms"  v ( z uu1 m z ) ((pos "Ia%" ) (subcat "2A" )))
( "zoophyte"  n ( z ou1 @ f ai t ) ((pos "K6$" ) ))
( "zoophytes"  n ( z ou1 @ f ai t s ) ((pos "Kj$" ) ))
( "zoos"  n ( z uu1 z ) ((pos "Kj%" ) ))
( "zoot suit"  n ( z uu1 t - s uu t ) ((pos "K6$" ) ))
( "zoot suits"  n ( z uu1 t - s uu t s ) ((pos "Kj$" ) ))
( "zucchini"  n ( z u1 k ii1 n ii ) ((pos "M9%" ) ))
( "zucchinis"  n ( z u1 k ii1 n i z ) ((pos "Kj$" ) ))
